#include "fitz.h"
#include "mupdf.h"

/* Adobe-CNS1-0 */

static const pdf_range pdf_cmap_Adobe_CNS1_0_ranges[] =
{
	{ 0x0000, 0x3712, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x3712, PDF_CMAP_RANGE, 14080 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_0_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_0 =
{
	-1, "Adobe-CNS1-0", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x37ff },
	},
	56, 56, (pdf_range*) pdf_cmap_Adobe_CNS1_0_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_0_table,
};

/* Adobe-CNS1-1 */

static const pdf_range pdf_cmap_Adobe_CNS1_1_ranges[] =
{
	{ 0x0000, 0x43ff, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_1_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_1 =
{
	-1, "Adobe-CNS1-1", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x43ff },
	},
	68, 68, (pdf_range*) pdf_cmap_Adobe_CNS1_1_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_1_table,
};

/* Adobe-CNS1-2 */

static const pdf_range pdf_cmap_Adobe_CNS1_2_ranges[] =
{
	{ 0x0000, 0x44c0, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44c0, PDF_CMAP_RANGE, 17408 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_2_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_2 =
{
	-1, "Adobe-CNS1-2", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x44ff },
	},
	69, 69, (pdf_range*) pdf_cmap_Adobe_CNS1_2_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_2_table,
};

/* Adobe-CNS1-3 */

static const pdf_range pdf_cmap_Adobe_CNS1_3_ranges[] =
{
	{ 0x0000, 0x499d, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x499d, PDF_CMAP_RANGE, 18688 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_3_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_3 =
{
	-1, "Adobe-CNS1-3", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x49ff },
	},
	74, 74, (pdf_range*) pdf_cmap_Adobe_CNS1_3_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_3_table,
};

/* Adobe-CNS1-4 */

static const pdf_range pdf_cmap_Adobe_CNS1_4_ranges[] =
{
	{ 0x0000, 0x4a14, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4a14, PDF_CMAP_RANGE, 18944 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_4_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_4 =
{
	-1, "Adobe-CNS1-4", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x4aff },
	},
	75, 75, (pdf_range*) pdf_cmap_Adobe_CNS1_4_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_4_table,
};

/* Adobe-CNS1-5 */

static const pdf_range pdf_cmap_Adobe_CNS1_5_ranges[] =
{
	{ 0x0000, 0x4a8f, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4a8f, PDF_CMAP_RANGE, 18944 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_5_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_5 =
{
	-1, "Adobe-CNS1-5", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x4aff },
	},
	75, 75, (pdf_range*) pdf_cmap_Adobe_CNS1_5_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_5_table,
};

/* Adobe-CNS1-6 */

static const pdf_range pdf_cmap_Adobe_CNS1_6_ranges[] =
{
	{ 0x0000, 0x4ad3, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4ad3, PDF_CMAP_RANGE, 18944 },
};

static const unsigned short pdf_cmap_Adobe_CNS1_6_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_CNS1_6 =
{
	-1, "Adobe-CNS1-6", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x4aff },
	},
	75, 75, (pdf_range*) pdf_cmap_Adobe_CNS1_6_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_CNS1_6_table,
};

/* B5-H */

static const pdf_range pdf_cmap_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 13648 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 0 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 2 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 4 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 13629 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
};

static const unsigned short pdf_cmap_B5_H_table[7] =
{
	248,247,5551,5550,13585,13629,13641,
};

pdf_cmap pdf_cmap_B5_H =
{
	-1, "B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa140, 0xfefe },
	},
	244, 244, (pdf_range*) pdf_cmap_B5_H_ranges,
	7, 7, (unsigned short*) pdf_cmap_B5_H_table,
};

/* B5-V */

static const pdf_range pdf_cmap_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_B5_V =
{
	-1, "B5-V", "B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_B5_V_table,
};

/* B5pc-H */

static const pdf_range pdf_cmap_B5pc_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x0080, 0x0080, PDF_CMAP_SINGLE, 61 },
	{ 0x00fd, 0x00ff, PDF_CMAP_RANGE, 96 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 0 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa3c0, 0xa3e0, PDF_CMAP_RANGE, 562 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 2 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 4 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 13629 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
};

static const unsigned short pdf_cmap_B5pc_H_table[7] =
{
	248,247,5551,5550,13585,13629,13641,
};

pdf_cmap pdf_cmap_B5pc_H =
{
	-1, "B5pc-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa140, 0xfcfe },
		{ 1, 0x00fd, 0x00ff },
	},
	247, 247, (pdf_range*) pdf_cmap_B5pc_H_ranges,
	7, 7, (unsigned short*) pdf_cmap_B5pc_H_table,
};

/* B5pc-V */

static const pdf_range pdf_cmap_B5pc_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_B5pc_V_table[1] = { 0 };

pdf_cmap pdf_cmap_B5pc_V =
{
	-1, "B5pc-V", "B5pc-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_B5pc_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_B5pc_V_table,
};

/* CNS-EUC-H */

static const pdf_range pdf_cmap_CNS_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 13648 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 99 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 5996 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 99 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 193 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 6090 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 193 },
	{ 0xa3a1, 0xa3ce, PDF_CMAP_RANGE, 287 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 6184 },
	{ 0xa3a1, 0xa3ce, PDF_CMAP_RANGE, 287 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 333 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 6278 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 333 },
	{ 0xa5a1, 0xa5ec, PDF_CMAP_RANGE, 427 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 6372 },
	{ 0xa5a1, 0xa5ec, PDF_CMAP_RANGE, 427 },
	{ 0xa5ee, 0xa5f0, PDF_CMAP_RANGE, 503 },
	{ 0xa5ee, 0xa5f0, PDF_CMAP_RANGE, 503 },
	{ 0xa6a1, 0xa6be, PDF_CMAP_RANGE, 506 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 6466 },
	{ 0xa6a1, 0xa6be, PDF_CMAP_RANGE, 506 },
	{ 0xa7a1, 0xa7a1, PDF_CMAP_SINGLE, 595 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 6560 },
	{ 0xa7a1, 0xa7a1, PDF_CMAP_SINGLE, 595 },
	{ 0xa7a2, 0xa7a4, PDF_CMAP_RANGE, 536 },
	{ 0xa7a2, 0xa7a4, PDF_CMAP_RANGE, 536 },
	{ 0xa7a5, 0xa7a5, PDF_CMAP_SINGLE, 596 },
	{ 0xa7a5, 0xa7a6, PDF_CMAP_TABLE, 0 },
	{ 0xa7a6, 0xa7a7, PDF_CMAP_TABLE, 2 },
	{ 0xa7a7, 0xa7a8, PDF_CMAP_TABLE, 4 },
	{ 0xa7a8, 0xa7a8, PDF_CMAP_SINGLE, 540 },
	{ 0xa7a9, 0xa7ac, PDF_CMAP_RANGE, 603 },
	{ 0xa7a9, 0xa7ac, PDF_CMAP_RANGE, 603 },
	{ 0xa7ad, 0xa7af, PDF_CMAP_RANGE, 541 },
	{ 0xa7ad, 0xa7af, PDF_CMAP_RANGE, 541 },
	{ 0xa7b0, 0xa7b0, PDF_CMAP_SINGLE, 607 },
	{ 0xa7b0, 0xa7b1, PDF_CMAP_TABLE, 6 },
	{ 0xa7b1, 0xa7b2, PDF_CMAP_TABLE, 8 },
	{ 0xa7b2, 0xa7b3, PDF_CMAP_TABLE, 10 },
	{ 0xa7b3, 0xa7b4, PDF_CMAP_TABLE, 12 },
	{ 0xa7b4, 0xa7b5, PDF_CMAP_TABLE, 14 },
	{ 0xa7b5, 0xa7b6, PDF_CMAP_TABLE, 16 },
	{ 0xa7b6, 0xa7b7, PDF_CMAP_TABLE, 18 },
	{ 0xa7b7, 0xa7b7, PDF_CMAP_SINGLE, 545 },
	{ 0xa7b8, 0xa7b9, PDF_CMAP_RANGE, 612 },
	{ 0xa7b8, 0xa7b9, PDF_CMAP_RANGE, 612 },
	{ 0xa7ba, 0xa7ba, PDF_CMAP_SINGLE, 546 },
	{ 0xa7ba, 0xa7bb, PDF_CMAP_TABLE, 20 },
	{ 0xa7bb, 0xa7bc, PDF_CMAP_TABLE, 22 },
	{ 0xa7bc, 0xa7bd, PDF_CMAP_TABLE, 24 },
	{ 0xa7bd, 0xa7be, PDF_CMAP_TABLE, 26 },
	{ 0xa7be, 0xa7bf, PDF_CMAP_TABLE, 28 },
	{ 0xa7bf, 0xa7bf, PDF_CMAP_SINGLE, 6005 },
	{ 0xa7c0, 0xa7c1, PDF_CMAP_RANGE, 634 },
	{ 0xa7c0, 0xa7c1, PDF_CMAP_RANGE, 634 },
	{ 0xa7c2, 0xa7c2, PDF_CMAP_SINGLE, 548 },
	{ 0xa7c2, 0xa7c2, PDF_CMAP_SINGLE, 548 },
	{ 0xa7c3, 0xa7c6, PDF_CMAP_RANGE, 636 },
	{ 0xa7c3, 0xa7c6, PDF_CMAP_RANGE, 636 },
	{ 0xa7c7, 0xa7c7, PDF_CMAP_SINGLE, 549 },
	{ 0xa7c7, 0xa7c7, PDF_CMAP_SINGLE, 549 },
	{ 0xa7c8, 0xa7cb, PDF_CMAP_RANGE, 642 },
	{ 0xa7c8, 0xa7cb, PDF_CMAP_RANGE, 642 },
	{ 0xa7cc, 0xa7cc, PDF_CMAP_SINGLE, 6006 },
	{ 0xa7cc, 0xa7cd, PDF_CMAP_TABLE, 30 },
	{ 0xa7cd, 0xa7ce, PDF_CMAP_TABLE, 32 },
	{ 0xa7ce, 0xa7ce, PDF_CMAP_SINGLE, 550 },
	{ 0xa7cf, 0xa7d0, PDF_CMAP_RANGE, 648 },
	{ 0xa7cf, 0xa7d0, PDF_CMAP_RANGE, 648 },
	{ 0xa7d1, 0xa7d2, PDF_CMAP_RANGE, 652 },
	{ 0xa7d1, 0xa7d2, PDF_CMAP_RANGE, 652 },
	{ 0xa7d3, 0xa7d5, PDF_CMAP_RANGE, 551 },
	{ 0xa7d3, 0xa7d5, PDF_CMAP_RANGE, 551 },
	{ 0xa7d6, 0xa7d8, PDF_CMAP_RANGE, 654 },
	{ 0xa7d6, 0xa7d8, PDF_CMAP_RANGE, 654 },
	{ 0xa7d9, 0xa7da, PDF_CMAP_RANGE, 554 },
	{ 0xa7d9, 0xa7da, PDF_CMAP_RANGE, 554 },
	{ 0xa7db, 0xa7db, PDF_CMAP_SINGLE, 6007 },
	{ 0xa7db, 0xa7db, PDF_CMAP_SINGLE, 6007 },
	{ 0xa7dc, 0xa7df, PDF_CMAP_RANGE, 720 },
	{ 0xa7dc, 0xa7df, PDF_CMAP_RANGE, 720 },
	{ 0xa7e0, 0xa7e0, PDF_CMAP_SINGLE, 725 },
	{ 0xa7e0, 0xa7e1, PDF_CMAP_TABLE, 34 },
	{ 0xa7e1, 0xa7e1, PDF_CMAP_SINGLE, 556 },
	{ 0xa7e2, 0xa7e5, PDF_CMAP_RANGE, 726 },
	{ 0xa7e2, 0xa7e5, PDF_CMAP_RANGE, 726 },
	{ 0xa7e6, 0xa7e6, PDF_CMAP_SINGLE, 557 },
	{ 0xa7e6, 0xa7e6, PDF_CMAP_SINGLE, 557 },
	{ 0xa7e7, 0xa7ed, PDF_CMAP_RANGE, 730 },
	{ 0xa7e7, 0xa7ed, PDF_CMAP_RANGE, 730 },
	{ 0xa7ee, 0xa7ee, PDF_CMAP_SINGLE, 6026 },
	{ 0xa7ee, 0xa7ee, PDF_CMAP_SINGLE, 6026 },
	{ 0xa7ef, 0xa7f2, PDF_CMAP_RANGE, 737 },
	{ 0xa7ef, 0xa7f2, PDF_CMAP_RANGE, 737 },
	{ 0xa7f3, 0xa7f3, PDF_CMAP_SINGLE, 6028 },
	{ 0xa7f3, 0xa7f3, PDF_CMAP_SINGLE, 6028 },
	{ 0xa7f4, 0xa7f8, PDF_CMAP_RANGE, 741 },
	{ 0xa7f4, 0xa7f8, PDF_CMAP_RANGE, 741 },
	{ 0xa7f9, 0xa7f9, PDF_CMAP_SINGLE, 6029 },
	{ 0xa7f9, 0xa7f9, PDF_CMAP_SINGLE, 6029 },
	{ 0xa7fa, 0xa7fd, PDF_CMAP_RANGE, 746 },
	{ 0xa7fa, 0xa7fd, PDF_CMAP_RANGE, 746 },
	{ 0xa7fe, 0xa7fe, PDF_CMAP_SINGLE, 854 },
	{ 0xa7fe, 0xa7fe, PDF_CMAP_SINGLE, 854 },
	{ 0xa8a1, 0xa8a6, PDF_CMAP_RANGE, 855 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 6654 },
	{ 0xa8a1, 0xa8a6, PDF_CMAP_RANGE, 855 },
	{ 0xa8a7, 0xa8a7, PDF_CMAP_SINGLE, 862 },
	{ 0xa8a7, 0xa8a8, PDF_CMAP_TABLE, 36 },
	{ 0xa8a8, 0xa8a8, PDF_CMAP_SINGLE, 866 },
	{ 0xa8a9, 0xa8aa, PDF_CMAP_RANGE, 558 },
	{ 0xa8a9, 0xa8aa, PDF_CMAP_RANGE, 558 },
	{ 0xa8ab, 0xa8b2, PDF_CMAP_RANGE, 867 },
	{ 0xa8ab, 0xa8b2, PDF_CMAP_RANGE, 867 },
	{ 0xa8b3, 0xa8b3, PDF_CMAP_SINGLE, 6066 },
	{ 0xa8b3, 0xa8b3, PDF_CMAP_SINGLE, 6066 },
	{ 0xa8b4, 0xa8b6, PDF_CMAP_RANGE, 875 },
	{ 0xa8b4, 0xa8b6, PDF_CMAP_RANGE, 875 },
	{ 0xa8b7, 0xa8ba, PDF_CMAP_RANGE, 1014 },
	{ 0xa8b7, 0xa8ba, PDF_CMAP_RANGE, 1014 },
	{ 0xa8bb, 0xa8bb, PDF_CMAP_SINGLE, 6162 },
	{ 0xa8bb, 0xa8bb, PDF_CMAP_SINGLE, 6162 },
	{ 0xa8bc, 0xa8be, PDF_CMAP_RANGE, 1018 },
	{ 0xa8bc, 0xa8be, PDF_CMAP_RANGE, 1018 },
	{ 0xa8bf, 0xa8c3, PDF_CMAP_RANGE, 1022 },
	{ 0xa8bf, 0xa8c3, PDF_CMAP_RANGE, 1022 },
	{ 0xa8c4, 0xa8cc, PDF_CMAP_RANGE, 1029 },
	{ 0xa8c4, 0xa8cc, PDF_CMAP_RANGE, 1029 },
	{ 0xa8cd, 0xa8cd, PDF_CMAP_SINGLE, 6163 },
	{ 0xa8cd, 0xa8ce, PDF_CMAP_TABLE, 38 },
	{ 0xa8ce, 0xa8ce, PDF_CMAP_SINGLE, 6168 },
	{ 0xa8cf, 0xa8d2, PDF_CMAP_RANGE, 1039 },
	{ 0xa8cf, 0xa8d2, PDF_CMAP_RANGE, 1039 },
	{ 0xa8d3, 0xa8d3, PDF_CMAP_SINGLE, 6169 },
	{ 0xa8d3, 0xa8d3, PDF_CMAP_SINGLE, 6169 },
	{ 0xa8d4, 0xa8d9, PDF_CMAP_RANGE, 1288 },
	{ 0xa8d4, 0xa8d9, PDF_CMAP_RANGE, 1288 },
	{ 0xa8da, 0xa8da, PDF_CMAP_SINGLE, 6375 },
	{ 0xa8da, 0xa8da, PDF_CMAP_SINGLE, 6375 },
	{ 0xa8db, 0xa8e2, PDF_CMAP_RANGE, 1294 },
	{ 0xa8db, 0xa8e2, PDF_CMAP_RANGE, 1294 },
	{ 0xa8e3, 0xa8e3, PDF_CMAP_SINGLE, 560 },
	{ 0xa8e3, 0xa8e4, PDF_CMAP_TABLE, 40 },
	{ 0xa8e4, 0xa8e4, PDF_CMAP_SINGLE, 1307 },
	{ 0xa8e5, 0xa8e7, PDF_CMAP_RANGE, 1312 },
	{ 0xa8e5, 0xa8e7, PDF_CMAP_RANGE, 1312 },
	{ 0xa8e8, 0xa8eb, PDF_CMAP_RANGE, 1686 },
	{ 0xa8e8, 0xa8eb, PDF_CMAP_RANGE, 1686 },
	{ 0xa8ec, 0xa8ec, PDF_CMAP_SINGLE, 561 },
	{ 0xa8ec, 0xa8ec, PDF_CMAP_SINGLE, 561 },
	{ 0xa8ed, 0xa8f0, PDF_CMAP_RANGE, 1695 },
	{ 0xa8ed, 0xa8f0, PDF_CMAP_RANGE, 1695 },
	{ 0xa8f1, 0xa8fb, PDF_CMAP_RANGE, 2086 },
	{ 0xa8f1, 0xa8fb, PDF_CMAP_RANGE, 2086 },
	{ 0xa8fc, 0xa8fe, PDF_CMAP_RANGE, 2549 },
	{ 0xa8fc, 0xa8fe, PDF_CMAP_RANGE, 2549 },
	{ 0xa9a1, 0xa9a1, PDF_CMAP_SINGLE, 7731 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 6748 },
	{ 0xa9a1, 0xa9a2, PDF_CMAP_TABLE, 42 },
	{ 0xa9a2, 0xa9a3, PDF_CMAP_TABLE, 44 },
	{ 0xa9a3, 0xa9a3, PDF_CMAP_SINGLE, 7732 },
	{ 0xa9a4, 0xa9a5, PDF_CMAP_RANGE, 2553 },
	{ 0xa9a4, 0xa9a5, PDF_CMAP_RANGE, 2553 },
	{ 0xa9a6, 0xa9ab, PDF_CMAP_RANGE, 3041 },
	{ 0xa9a6, 0xa9ab, PDF_CMAP_RANGE, 3041 },
	{ 0xa9ac, 0xa9ae, PDF_CMAP_RANGE, 3515 },
	{ 0xa9ac, 0xa9ae, PDF_CMAP_RANGE, 3515 },
	{ 0xa9af, 0xa9af, PDF_CMAP_SINGLE, 9056 },
	{ 0xa9af, 0xa9b0, PDF_CMAP_TABLE, 46 },
	{ 0xa9b0, 0xa9b0, PDF_CMAP_SINGLE, 9746 },
	{ 0xa9b1, 0xa9b3, PDF_CMAP_RANGE, 3963 },
	{ 0xa9b1, 0xa9b3, PDF_CMAP_RANGE, 3963 },
	{ 0xa9b4, 0xa9b5, PDF_CMAP_RANGE, 4352 },
	{ 0xa9b4, 0xa9b5, PDF_CMAP_RANGE, 4352 },
	{ 0xa9b6, 0xa9b6, PDF_CMAP_SINGLE, 4745 },
	{ 0xa9b6, 0xa9b6, PDF_CMAP_SINGLE, 4745 },
	{ 0xa9b7, 0xa9b8, PDF_CMAP_RANGE, 5042 },
	{ 0xa9b7, 0xa9b8, PDF_CMAP_RANGE, 5042 },
	{ 0xa9b9, 0xa9b9, PDF_CMAP_SINGLE, 12045 },
	{ 0xa9b9, 0xa9b9, PDF_CMAP_SINGLE, 12045 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 6842 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 6936 },
	{ 0xaca1, 0xacfe, PDF_CMAP_RANGE, 7030 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 7124 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 7218 },
	{ 0xafa1, 0xaffe, PDF_CMAP_RANGE, 7312 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 7406 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 7500 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 7594 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 7688 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 7782 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 7876 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 7970 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 8064 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 8158 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 8252 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 8346 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 8440 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 8534 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 8628 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 8722 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 8816 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 8910 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 9004 },
	{ 0xc2a1, 0xc2c1, PDF_CMAP_RANGE, 562 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 9098 },
	{ 0xc2a1, 0xc2c1, PDF_CMAP_RANGE, 562 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 9192 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 595 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 9286 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 595 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 689 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 9380 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 689 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 783 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 9474 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 783 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 877 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 9568 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 877 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 971 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 9662 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 971 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 1065 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 9756 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 1065 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 1159 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 9850 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 1159 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 1253 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 9944 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 1253 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 1347 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 10038 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 1347 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 1441 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 10132 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 1441 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 1535 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 10226 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 1535 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 1629 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 10320 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 1629 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 1723 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 10414 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 1723 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 1817 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 10508 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 1817 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 1911 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 10602 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 1911 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 2005 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 10696 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 2005 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 2099 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 10790 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 2099 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 2193 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 10884 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 2193 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 2287 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 10978 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 2287 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 2381 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 11072 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 2381 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 2475 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 11166 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 2475 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 2569 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 11260 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 2569 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 2663 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 11354 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 2663 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 2757 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 11448 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 2757 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 2851 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 11542 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 2851 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 2945 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 11636 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 2945 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 3039 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 11730 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 3039 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 3133 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 11824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 3133 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 3227 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 11918 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 3227 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 3321 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 12012 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 3321 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 3415 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 12106 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 3415 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 3509 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 12200 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 3509 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 3603 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 12294 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 3603 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 3697 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 12388 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 3697 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 3791 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 12482 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 3791 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 3885 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 12576 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 3885 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 3979 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 12670 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 3979 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 4073 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 12764 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 4073 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 4167 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 12858 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 4167 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 4261 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 12952 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 4261 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 4355 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 13046 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 4355 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 4449 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 13140 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 4449 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 4543 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 13234 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 4543 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 4637 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 13328 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 4637 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 4731 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 13422 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 4731 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 4825 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 13516 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 4825 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf2a1, 0xf2c4, PDF_CMAP_RANGE, 13610 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
	{ 0xf2a1, 0xf2c4, PDF_CMAP_RANGE, 13610 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
};

static const unsigned short pdf_cmap_CNS_EUC_H_table[48] =
{
	596,539,539,602,602,540,607,5998,
	5998,608,608,610,610,544,544,611,
	611,5999,5999,545,546,6000,6000,547,
	547,614,614,633,633,6005,6006,646,
	646,550,725,556,862,866,6163,6168,
	560,1307,7731,2552,2552,7732,9056,9746,
};

pdf_cmap pdf_cmap_CNS_EUC_H =
{
	-1, "CNS-EUC-H", "", nil, 0,
	5, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 4, 0xa1a1, 0xfefe },
		{ 4, 0xa1a1, 0xfefe },
		{ 4, 0xa1a1, 0xfefe },
		{ 2, 0xa1a1, 0xfefe },
	},
	395, 395, (pdf_range*) pdf_cmap_CNS_EUC_H_ranges,
	48, 48, (unsigned short*) pdf_cmap_CNS_EUC_H_table,
};

/* CNS-EUC-V */

static const pdf_range pdf_cmap_CNS_EUC_V_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 13648 },
	{ 0xa1a1, 0xa1ab, PDF_CMAP_RANGE, 99 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 5996 },
	{ 0xa1a1, 0xa1ab, PDF_CMAP_RANGE, 99 },
	{ 0xa1ac, 0xa1ac, PDF_CMAP_SINGLE, 13646 },
	{ 0xa1ac, 0xa1ac, PDF_CMAP_SINGLE, 13646 },
	{ 0xa1ad, 0xa1ba, PDF_CMAP_RANGE, 111 },
	{ 0xa1ad, 0xa1ba, PDF_CMAP_RANGE, 111 },
	{ 0xa1bb, 0xa1bb, PDF_CMAP_SINGLE, 124 },
	{ 0xa1bb, 0xa1bc, PDF_CMAP_TABLE, 0 },
	{ 0xa1bc, 0xa1bd, PDF_CMAP_TABLE, 2 },
	{ 0xa1bd, 0xa1bd, PDF_CMAP_SINGLE, 126 },
	{ 0xa1be, 0xa1bf, PDF_CMAP_RANGE, 130 },
	{ 0xa1be, 0xa1bf, PDF_CMAP_RANGE, 130 },
	{ 0xa1c0, 0xa1c1, PDF_CMAP_RANGE, 130 },
	{ 0xa1c0, 0xa1c1, PDF_CMAP_RANGE, 130 },
	{ 0xa1c2, 0xa1c3, PDF_CMAP_RANGE, 134 },
	{ 0xa1c2, 0xa1c3, PDF_CMAP_RANGE, 134 },
	{ 0xa1c4, 0xa1c5, PDF_CMAP_RANGE, 134 },
	{ 0xa1c4, 0xa1c5, PDF_CMAP_RANGE, 134 },
	{ 0xa1c6, 0xa1c7, PDF_CMAP_RANGE, 138 },
	{ 0xa1c6, 0xa1c7, PDF_CMAP_RANGE, 138 },
	{ 0xa1c8, 0xa1c9, PDF_CMAP_RANGE, 138 },
	{ 0xa1c8, 0xa1c9, PDF_CMAP_RANGE, 138 },
	{ 0xa1ca, 0xa1cb, PDF_CMAP_RANGE, 142 },
	{ 0xa1ca, 0xa1cb, PDF_CMAP_RANGE, 142 },
	{ 0xa1cc, 0xa1cd, PDF_CMAP_RANGE, 142 },
	{ 0xa1cc, 0xa1cd, PDF_CMAP_RANGE, 142 },
	{ 0xa1ce, 0xa1cf, PDF_CMAP_RANGE, 146 },
	{ 0xa1ce, 0xa1cf, PDF_CMAP_RANGE, 146 },
	{ 0xa1d0, 0xa1d1, PDF_CMAP_RANGE, 146 },
	{ 0xa1d0, 0xa1d1, PDF_CMAP_RANGE, 146 },
	{ 0xa1d2, 0xa1d3, PDF_CMAP_RANGE, 150 },
	{ 0xa1d2, 0xa1d3, PDF_CMAP_RANGE, 150 },
	{ 0xa1d4, 0xa1d5, PDF_CMAP_RANGE, 150 },
	{ 0xa1d4, 0xa1d5, PDF_CMAP_RANGE, 150 },
	{ 0xa1d6, 0xa1d7, PDF_CMAP_RANGE, 154 },
	{ 0xa1d6, 0xa1d7, PDF_CMAP_RANGE, 154 },
	{ 0xa1d8, 0xa1d9, PDF_CMAP_RANGE, 154 },
	{ 0xa1d8, 0xa1d9, PDF_CMAP_RANGE, 154 },
	{ 0xa1da, 0xa1db, PDF_CMAP_RANGE, 158 },
	{ 0xa1da, 0xa1db, PDF_CMAP_RANGE, 158 },
	{ 0xa1dc, 0xa1fe, PDF_CMAP_RANGE, 158 },
	{ 0xa1dc, 0xa1fe, PDF_CMAP_RANGE, 158 },
	{ 0xa2a1, 0xa2c3, PDF_CMAP_RANGE, 193 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 6090 },
	{ 0xa2a1, 0xa2c3, PDF_CMAP_RANGE, 193 },
	{ 0xa2c4, 0xa2c4, PDF_CMAP_SINGLE, 13647 },
	{ 0xa2c4, 0xa2c4, PDF_CMAP_SINGLE, 13647 },
	{ 0xa2c5, 0xa2fe, PDF_CMAP_RANGE, 229 },
	{ 0xa2c5, 0xa2fe, PDF_CMAP_RANGE, 229 },
	{ 0xa3a1, 0xa3ce, PDF_CMAP_RANGE, 287 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 6184 },
	{ 0xa3a1, 0xa3ce, PDF_CMAP_RANGE, 287 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 333 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 6278 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 333 },
	{ 0xa5a1, 0xa5ec, PDF_CMAP_RANGE, 427 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 6372 },
	{ 0xa5a1, 0xa5ec, PDF_CMAP_RANGE, 427 },
	{ 0xa5ee, 0xa5f0, PDF_CMAP_RANGE, 503 },
	{ 0xa5ee, 0xa5f0, PDF_CMAP_RANGE, 503 },
	{ 0xa6a1, 0xa6be, PDF_CMAP_RANGE, 506 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 6466 },
	{ 0xa6a1, 0xa6be, PDF_CMAP_RANGE, 506 },
	{ 0xa7a1, 0xa7a1, PDF_CMAP_SINGLE, 595 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 6560 },
	{ 0xa7a1, 0xa7a1, PDF_CMAP_SINGLE, 595 },
	{ 0xa7a2, 0xa7a4, PDF_CMAP_RANGE, 536 },
	{ 0xa7a2, 0xa7a4, PDF_CMAP_RANGE, 536 },
	{ 0xa7a5, 0xa7a5, PDF_CMAP_SINGLE, 596 },
	{ 0xa7a5, 0xa7a6, PDF_CMAP_TABLE, 4 },
	{ 0xa7a6, 0xa7a7, PDF_CMAP_TABLE, 6 },
	{ 0xa7a7, 0xa7a8, PDF_CMAP_TABLE, 8 },
	{ 0xa7a8, 0xa7a8, PDF_CMAP_SINGLE, 540 },
	{ 0xa7a9, 0xa7ac, PDF_CMAP_RANGE, 603 },
	{ 0xa7a9, 0xa7ac, PDF_CMAP_RANGE, 603 },
	{ 0xa7ad, 0xa7af, PDF_CMAP_RANGE, 541 },
	{ 0xa7ad, 0xa7af, PDF_CMAP_RANGE, 541 },
	{ 0xa7b0, 0xa7b0, PDF_CMAP_SINGLE, 607 },
	{ 0xa7b0, 0xa7b1, PDF_CMAP_TABLE, 10 },
	{ 0xa7b1, 0xa7b2, PDF_CMAP_TABLE, 12 },
	{ 0xa7b2, 0xa7b3, PDF_CMAP_TABLE, 14 },
	{ 0xa7b3, 0xa7b4, PDF_CMAP_TABLE, 16 },
	{ 0xa7b4, 0xa7b5, PDF_CMAP_TABLE, 18 },
	{ 0xa7b5, 0xa7b6, PDF_CMAP_TABLE, 20 },
	{ 0xa7b6, 0xa7b7, PDF_CMAP_TABLE, 22 },
	{ 0xa7b7, 0xa7b7, PDF_CMAP_SINGLE, 545 },
	{ 0xa7b8, 0xa7b9, PDF_CMAP_RANGE, 612 },
	{ 0xa7b8, 0xa7b9, PDF_CMAP_RANGE, 612 },
	{ 0xa7ba, 0xa7ba, PDF_CMAP_SINGLE, 546 },
	{ 0xa7ba, 0xa7bb, PDF_CMAP_TABLE, 24 },
	{ 0xa7bb, 0xa7bc, PDF_CMAP_TABLE, 26 },
	{ 0xa7bc, 0xa7bd, PDF_CMAP_TABLE, 28 },
	{ 0xa7bd, 0xa7be, PDF_CMAP_TABLE, 30 },
	{ 0xa7be, 0xa7bf, PDF_CMAP_TABLE, 32 },
	{ 0xa7bf, 0xa7bf, PDF_CMAP_SINGLE, 6005 },
	{ 0xa7c0, 0xa7c1, PDF_CMAP_RANGE, 634 },
	{ 0xa7c0, 0xa7c1, PDF_CMAP_RANGE, 634 },
	{ 0xa7c2, 0xa7c2, PDF_CMAP_SINGLE, 548 },
	{ 0xa7c2, 0xa7c2, PDF_CMAP_SINGLE, 548 },
	{ 0xa7c3, 0xa7c6, PDF_CMAP_RANGE, 636 },
	{ 0xa7c3, 0xa7c6, PDF_CMAP_RANGE, 636 },
	{ 0xa7c7, 0xa7c7, PDF_CMAP_SINGLE, 549 },
	{ 0xa7c7, 0xa7c7, PDF_CMAP_SINGLE, 549 },
	{ 0xa7c8, 0xa7cb, PDF_CMAP_RANGE, 642 },
	{ 0xa7c8, 0xa7cb, PDF_CMAP_RANGE, 642 },
	{ 0xa7cc, 0xa7cc, PDF_CMAP_SINGLE, 6006 },
	{ 0xa7cc, 0xa7cd, PDF_CMAP_TABLE, 34 },
	{ 0xa7cd, 0xa7ce, PDF_CMAP_TABLE, 36 },
	{ 0xa7ce, 0xa7ce, PDF_CMAP_SINGLE, 550 },
	{ 0xa7cf, 0xa7d0, PDF_CMAP_RANGE, 648 },
	{ 0xa7cf, 0xa7d0, PDF_CMAP_RANGE, 648 },
	{ 0xa7d1, 0xa7d2, PDF_CMAP_RANGE, 652 },
	{ 0xa7d1, 0xa7d2, PDF_CMAP_RANGE, 652 },
	{ 0xa7d3, 0xa7d5, PDF_CMAP_RANGE, 551 },
	{ 0xa7d3, 0xa7d5, PDF_CMAP_RANGE, 551 },
	{ 0xa7d6, 0xa7d8, PDF_CMAP_RANGE, 654 },
	{ 0xa7d6, 0xa7d8, PDF_CMAP_RANGE, 654 },
	{ 0xa7d9, 0xa7da, PDF_CMAP_RANGE, 554 },
	{ 0xa7d9, 0xa7da, PDF_CMAP_RANGE, 554 },
	{ 0xa7db, 0xa7db, PDF_CMAP_SINGLE, 6007 },
	{ 0xa7db, 0xa7db, PDF_CMAP_SINGLE, 6007 },
	{ 0xa7dc, 0xa7df, PDF_CMAP_RANGE, 720 },
	{ 0xa7dc, 0xa7df, PDF_CMAP_RANGE, 720 },
	{ 0xa7e0, 0xa7e0, PDF_CMAP_SINGLE, 725 },
	{ 0xa7e0, 0xa7e1, PDF_CMAP_TABLE, 38 },
	{ 0xa7e1, 0xa7e1, PDF_CMAP_SINGLE, 556 },
	{ 0xa7e2, 0xa7e5, PDF_CMAP_RANGE, 726 },
	{ 0xa7e2, 0xa7e5, PDF_CMAP_RANGE, 726 },
	{ 0xa7e6, 0xa7e6, PDF_CMAP_SINGLE, 557 },
	{ 0xa7e6, 0xa7e6, PDF_CMAP_SINGLE, 557 },
	{ 0xa7e7, 0xa7ed, PDF_CMAP_RANGE, 730 },
	{ 0xa7e7, 0xa7ed, PDF_CMAP_RANGE, 730 },
	{ 0xa7ee, 0xa7ee, PDF_CMAP_SINGLE, 6026 },
	{ 0xa7ee, 0xa7ee, PDF_CMAP_SINGLE, 6026 },
	{ 0xa7ef, 0xa7f2, PDF_CMAP_RANGE, 737 },
	{ 0xa7ef, 0xa7f2, PDF_CMAP_RANGE, 737 },
	{ 0xa7f3, 0xa7f3, PDF_CMAP_SINGLE, 6028 },
	{ 0xa7f3, 0xa7f3, PDF_CMAP_SINGLE, 6028 },
	{ 0xa7f4, 0xa7f8, PDF_CMAP_RANGE, 741 },
	{ 0xa7f4, 0xa7f8, PDF_CMAP_RANGE, 741 },
	{ 0xa7f9, 0xa7f9, PDF_CMAP_SINGLE, 6029 },
	{ 0xa7f9, 0xa7f9, PDF_CMAP_SINGLE, 6029 },
	{ 0xa7fa, 0xa7fd, PDF_CMAP_RANGE, 746 },
	{ 0xa7fa, 0xa7fd, PDF_CMAP_RANGE, 746 },
	{ 0xa7fe, 0xa7fe, PDF_CMAP_SINGLE, 854 },
	{ 0xa7fe, 0xa7fe, PDF_CMAP_SINGLE, 854 },
	{ 0xa8a1, 0xa8a6, PDF_CMAP_RANGE, 855 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 6654 },
	{ 0xa8a1, 0xa8a6, PDF_CMAP_RANGE, 855 },
	{ 0xa8a7, 0xa8a7, PDF_CMAP_SINGLE, 862 },
	{ 0xa8a7, 0xa8a8, PDF_CMAP_TABLE, 40 },
	{ 0xa8a8, 0xa8a8, PDF_CMAP_SINGLE, 866 },
	{ 0xa8a9, 0xa8aa, PDF_CMAP_RANGE, 558 },
	{ 0xa8a9, 0xa8aa, PDF_CMAP_RANGE, 558 },
	{ 0xa8ab, 0xa8b2, PDF_CMAP_RANGE, 867 },
	{ 0xa8ab, 0xa8b2, PDF_CMAP_RANGE, 867 },
	{ 0xa8b3, 0xa8b3, PDF_CMAP_SINGLE, 6066 },
	{ 0xa8b3, 0xa8b3, PDF_CMAP_SINGLE, 6066 },
	{ 0xa8b4, 0xa8b6, PDF_CMAP_RANGE, 875 },
	{ 0xa8b4, 0xa8b6, PDF_CMAP_RANGE, 875 },
	{ 0xa8b7, 0xa8ba, PDF_CMAP_RANGE, 1014 },
	{ 0xa8b7, 0xa8ba, PDF_CMAP_RANGE, 1014 },
	{ 0xa8bb, 0xa8bb, PDF_CMAP_SINGLE, 6162 },
	{ 0xa8bb, 0xa8bb, PDF_CMAP_SINGLE, 6162 },
	{ 0xa8bc, 0xa8be, PDF_CMAP_RANGE, 1018 },
	{ 0xa8bc, 0xa8be, PDF_CMAP_RANGE, 1018 },
	{ 0xa8bf, 0xa8c3, PDF_CMAP_RANGE, 1022 },
	{ 0xa8bf, 0xa8c3, PDF_CMAP_RANGE, 1022 },
	{ 0xa8c4, 0xa8cc, PDF_CMAP_RANGE, 1029 },
	{ 0xa8c4, 0xa8cc, PDF_CMAP_RANGE, 1029 },
	{ 0xa8cd, 0xa8cd, PDF_CMAP_SINGLE, 6163 },
	{ 0xa8cd, 0xa8ce, PDF_CMAP_TABLE, 42 },
	{ 0xa8ce, 0xa8ce, PDF_CMAP_SINGLE, 6168 },
	{ 0xa8cf, 0xa8d2, PDF_CMAP_RANGE, 1039 },
	{ 0xa8cf, 0xa8d2, PDF_CMAP_RANGE, 1039 },
	{ 0xa8d3, 0xa8d3, PDF_CMAP_SINGLE, 6169 },
	{ 0xa8d3, 0xa8d3, PDF_CMAP_SINGLE, 6169 },
	{ 0xa8d4, 0xa8d9, PDF_CMAP_RANGE, 1288 },
	{ 0xa8d4, 0xa8d9, PDF_CMAP_RANGE, 1288 },
	{ 0xa8da, 0xa8da, PDF_CMAP_SINGLE, 6375 },
	{ 0xa8da, 0xa8da, PDF_CMAP_SINGLE, 6375 },
	{ 0xa8db, 0xa8e2, PDF_CMAP_RANGE, 1294 },
	{ 0xa8db, 0xa8e2, PDF_CMAP_RANGE, 1294 },
	{ 0xa8e3, 0xa8e3, PDF_CMAP_SINGLE, 560 },
	{ 0xa8e3, 0xa8e4, PDF_CMAP_TABLE, 44 },
	{ 0xa8e4, 0xa8e4, PDF_CMAP_SINGLE, 1307 },
	{ 0xa8e5, 0xa8e7, PDF_CMAP_RANGE, 1312 },
	{ 0xa8e5, 0xa8e7, PDF_CMAP_RANGE, 1312 },
	{ 0xa8e8, 0xa8eb, PDF_CMAP_RANGE, 1686 },
	{ 0xa8e8, 0xa8eb, PDF_CMAP_RANGE, 1686 },
	{ 0xa8ec, 0xa8ec, PDF_CMAP_SINGLE, 561 },
	{ 0xa8ec, 0xa8ec, PDF_CMAP_SINGLE, 561 },
	{ 0xa8ed, 0xa8f0, PDF_CMAP_RANGE, 1695 },
	{ 0xa8ed, 0xa8f0, PDF_CMAP_RANGE, 1695 },
	{ 0xa8f1, 0xa8fb, PDF_CMAP_RANGE, 2086 },
	{ 0xa8f1, 0xa8fb, PDF_CMAP_RANGE, 2086 },
	{ 0xa8fc, 0xa8fe, PDF_CMAP_RANGE, 2549 },
	{ 0xa8fc, 0xa8fe, PDF_CMAP_RANGE, 2549 },
	{ 0xa9a1, 0xa9a1, PDF_CMAP_SINGLE, 7731 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 6748 },
	{ 0xa9a1, 0xa9a2, PDF_CMAP_TABLE, 46 },
	{ 0xa9a2, 0xa9a3, PDF_CMAP_TABLE, 48 },
	{ 0xa9a3, 0xa9a3, PDF_CMAP_SINGLE, 7732 },
	{ 0xa9a4, 0xa9a5, PDF_CMAP_RANGE, 2553 },
	{ 0xa9a4, 0xa9a5, PDF_CMAP_RANGE, 2553 },
	{ 0xa9a6, 0xa9ab, PDF_CMAP_RANGE, 3041 },
	{ 0xa9a6, 0xa9ab, PDF_CMAP_RANGE, 3041 },
	{ 0xa9ac, 0xa9ae, PDF_CMAP_RANGE, 3515 },
	{ 0xa9ac, 0xa9ae, PDF_CMAP_RANGE, 3515 },
	{ 0xa9af, 0xa9af, PDF_CMAP_SINGLE, 9056 },
	{ 0xa9af, 0xa9b0, PDF_CMAP_TABLE, 50 },
	{ 0xa9b0, 0xa9b0, PDF_CMAP_SINGLE, 9746 },
	{ 0xa9b1, 0xa9b3, PDF_CMAP_RANGE, 3963 },
	{ 0xa9b1, 0xa9b3, PDF_CMAP_RANGE, 3963 },
	{ 0xa9b4, 0xa9b5, PDF_CMAP_RANGE, 4352 },
	{ 0xa9b4, 0xa9b5, PDF_CMAP_RANGE, 4352 },
	{ 0xa9b6, 0xa9b6, PDF_CMAP_SINGLE, 4745 },
	{ 0xa9b6, 0xa9b6, PDF_CMAP_SINGLE, 4745 },
	{ 0xa9b7, 0xa9b8, PDF_CMAP_RANGE, 5042 },
	{ 0xa9b7, 0xa9b8, PDF_CMAP_RANGE, 5042 },
	{ 0xa9b9, 0xa9b9, PDF_CMAP_SINGLE, 12045 },
	{ 0xa9b9, 0xa9b9, PDF_CMAP_SINGLE, 12045 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 6842 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 6936 },
	{ 0xaca1, 0xacfe, PDF_CMAP_RANGE, 7030 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 7124 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 7218 },
	{ 0xafa1, 0xaffe, PDF_CMAP_RANGE, 7312 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 7406 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 7500 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 7594 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 7688 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 7782 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 7876 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 7970 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 8064 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 8158 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 8252 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 8346 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 8440 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 8534 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 8628 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 8722 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 8816 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 8910 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 9004 },
	{ 0xc2a1, 0xc2c1, PDF_CMAP_RANGE, 562 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 9098 },
	{ 0xc2a1, 0xc2c1, PDF_CMAP_RANGE, 562 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 9192 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 595 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 9286 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 595 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 689 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 9380 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 689 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 783 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 9474 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 783 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 877 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 9568 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 877 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 971 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 9662 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 971 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 1065 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 9756 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 1065 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 1159 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 9850 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 1159 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 1253 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 9944 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 1253 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 1347 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 10038 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 1347 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 1441 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 10132 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 1441 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 1535 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 10226 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 1535 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 1629 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 10320 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 1629 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 1723 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 10414 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 1723 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 1817 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 10508 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 1817 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 1911 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 10602 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 1911 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 2005 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 10696 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 2005 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 2099 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 10790 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 2099 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 2193 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 10884 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 2193 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 2287 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 10978 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 2287 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 2381 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 11072 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 2381 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 2475 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 11166 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 2475 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 2569 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 11260 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 2569 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 2663 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 11354 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 2663 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 2757 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 11448 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 2757 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 2851 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 11542 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 2851 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 2945 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 11636 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 2945 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 3039 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 11730 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 3039 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 3133 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 11824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 3133 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 3227 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 11918 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 3227 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 3321 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 12012 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 3321 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 3415 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 12106 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 3415 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 3509 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 12200 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 3509 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 3603 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 12294 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 3603 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 3697 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 12388 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 3697 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 3791 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 12482 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 3791 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 3885 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 12576 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 3885 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 3979 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 12670 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 3979 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 4073 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 12764 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 4073 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 4167 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 12858 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 4167 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 4261 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 12952 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 4261 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 4355 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 13046 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 4355 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 4449 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 13140 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 4449 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 4543 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 13234 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 4543 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 4637 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 13328 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 4637 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 4731 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 13422 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 4731 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 4825 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 13516 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 4825 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf2a1, 0xf2c4, PDF_CMAP_RANGE, 13610 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 4825 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf2a1, 0xf2c4, PDF_CMAP_RANGE, 13610 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 4919 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 5013 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 5107 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 5201 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 5295 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 5389 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 5483 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 5577 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 5671 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 5765 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfca1, 0xfcfe, PDF_CMAP_RANGE, 5859 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
	{ 0xfda1, 0xfdcb, PDF_CMAP_RANGE, 5953 },
};

static const unsigned short pdf_cmap_CNS_EUC_V_table[52] =
{
	124,126,126,126,596,539,539,602,
	602,540,607,5998,5998,608,608,610,
	610,544,544,611,611,5999,5999,545,
	546,6000,6000,547,547,614,614,633,
	633,6005,6006,646,646,550,725,556,
	862,866,6163,6168,560,1307,7731,2552,
	2552,7732,9056,9746,
};

pdf_cmap pdf_cmap_CNS_EUC_V =
{
	-1, "CNS-EUC-V", "", nil, 1,
	5, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 4, 0xa1a1, 0xfefe },
		{ 4, 0xa1a1, 0xfefe },
		{ 4, 0xa1a1, 0xfefe },
		{ 2, 0xa1a1, 0xfefe },
	},
	441, 441, (pdf_range*) pdf_cmap_CNS_EUC_V_ranges,
	52, 52, (unsigned short*) pdf_cmap_CNS_EUC_V_table,
};

/* CNS1-H */

static const pdf_range pdf_cmap_CNS1_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 99 },
	{ 0x2221, 0x227e, PDF_CMAP_RANGE, 193 },
	{ 0x2321, 0x234e, PDF_CMAP_RANGE, 287 },
	{ 0x2421, 0x247e, PDF_CMAP_RANGE, 333 },
	{ 0x2521, 0x256c, PDF_CMAP_RANGE, 427 },
	{ 0x256e, 0x2570, PDF_CMAP_RANGE, 503 },
	{ 0x2621, 0x263e, PDF_CMAP_RANGE, 506 },
	{ 0x2721, 0x2721, PDF_CMAP_SINGLE, 595 },
	{ 0x2722, 0x2724, PDF_CMAP_RANGE, 536 },
	{ 0x2725, 0x2728, PDF_CMAP_TABLE, 0 },
	{ 0x2729, 0x272c, PDF_CMAP_RANGE, 603 },
	{ 0x272d, 0x272f, PDF_CMAP_RANGE, 541 },
	{ 0x2730, 0x2737, PDF_CMAP_TABLE, 4 },
	{ 0x2738, 0x2739, PDF_CMAP_RANGE, 612 },
	{ 0x273a, 0x273f, PDF_CMAP_TABLE, 12 },
	{ 0x2740, 0x2741, PDF_CMAP_RANGE, 634 },
	{ 0x2742, 0x2742, PDF_CMAP_SINGLE, 548 },
	{ 0x2743, 0x2746, PDF_CMAP_RANGE, 636 },
	{ 0x2747, 0x2747, PDF_CMAP_SINGLE, 549 },
	{ 0x2748, 0x274b, PDF_CMAP_RANGE, 642 },
	{ 0x274c, 0x274e, PDF_CMAP_TABLE, 18 },
	{ 0x274f, 0x2750, PDF_CMAP_RANGE, 648 },
	{ 0x2751, 0x2752, PDF_CMAP_RANGE, 652 },
	{ 0x2753, 0x2755, PDF_CMAP_RANGE, 551 },
	{ 0x2756, 0x2758, PDF_CMAP_RANGE, 654 },
	{ 0x2759, 0x275a, PDF_CMAP_RANGE, 554 },
	{ 0x275b, 0x275b, PDF_CMAP_SINGLE, 6007 },
	{ 0x275c, 0x275f, PDF_CMAP_RANGE, 720 },
	{ 0x2760, 0x2761, PDF_CMAP_TABLE, 21 },
	{ 0x2762, 0x2765, PDF_CMAP_RANGE, 726 },
	{ 0x2766, 0x2766, PDF_CMAP_SINGLE, 557 },
	{ 0x2767, 0x276d, PDF_CMAP_RANGE, 730 },
	{ 0x276e, 0x276e, PDF_CMAP_SINGLE, 6026 },
	{ 0x276f, 0x2772, PDF_CMAP_RANGE, 737 },
	{ 0x2773, 0x2773, PDF_CMAP_SINGLE, 6028 },
	{ 0x2774, 0x2778, PDF_CMAP_RANGE, 741 },
	{ 0x2779, 0x2779, PDF_CMAP_SINGLE, 6029 },
	{ 0x277a, 0x277d, PDF_CMAP_RANGE, 746 },
	{ 0x277e, 0x277e, PDF_CMAP_SINGLE, 854 },
	{ 0x2821, 0x2826, PDF_CMAP_RANGE, 855 },
	{ 0x2827, 0x2828, PDF_CMAP_TABLE, 23 },
	{ 0x2829, 0x282a, PDF_CMAP_RANGE, 558 },
	{ 0x282b, 0x2832, PDF_CMAP_RANGE, 867 },
	{ 0x2833, 0x2833, PDF_CMAP_SINGLE, 6066 },
	{ 0x2834, 0x2836, PDF_CMAP_RANGE, 875 },
	{ 0x2837, 0x283a, PDF_CMAP_RANGE, 1014 },
	{ 0x283b, 0x283b, PDF_CMAP_SINGLE, 6162 },
	{ 0x283c, 0x283e, PDF_CMAP_RANGE, 1018 },
	{ 0x283f, 0x2843, PDF_CMAP_RANGE, 1022 },
	{ 0x2844, 0x284c, PDF_CMAP_RANGE, 1029 },
	{ 0x284d, 0x284e, PDF_CMAP_TABLE, 25 },
	{ 0x284f, 0x2852, PDF_CMAP_RANGE, 1039 },
	{ 0x2853, 0x2853, PDF_CMAP_SINGLE, 6169 },
	{ 0x2854, 0x2859, PDF_CMAP_RANGE, 1288 },
	{ 0x285a, 0x285a, PDF_CMAP_SINGLE, 6375 },
	{ 0x285b, 0x2862, PDF_CMAP_RANGE, 1294 },
	{ 0x2863, 0x2864, PDF_CMAP_TABLE, 27 },
	{ 0x2865, 0x2867, PDF_CMAP_RANGE, 1312 },
	{ 0x2868, 0x286b, PDF_CMAP_RANGE, 1686 },
	{ 0x286c, 0x286c, PDF_CMAP_SINGLE, 561 },
	{ 0x286d, 0x2870, PDF_CMAP_RANGE, 1695 },
	{ 0x2871, 0x287b, PDF_CMAP_RANGE, 2086 },
	{ 0x287c, 0x287e, PDF_CMAP_RANGE, 2549 },
	{ 0x2921, 0x2923, PDF_CMAP_TABLE, 29 },
	{ 0x2924, 0x2925, PDF_CMAP_RANGE, 2553 },
	{ 0x2926, 0x292b, PDF_CMAP_RANGE, 3041 },
	{ 0x292c, 0x292e, PDF_CMAP_RANGE, 3515 },
	{ 0x292f, 0x2930, PDF_CMAP_TABLE, 32 },
	{ 0x2931, 0x2933, PDF_CMAP_RANGE, 3963 },
	{ 0x2934, 0x2935, PDF_CMAP_RANGE, 4352 },
	{ 0x2936, 0x2936, PDF_CMAP_SINGLE, 4745 },
	{ 0x2937, 0x2938, PDF_CMAP_RANGE, 5042 },
	{ 0x2939, 0x2939, PDF_CMAP_SINGLE, 12045 },
	{ 0x4221, 0x4241, PDF_CMAP_RANGE, 562 },
	{ 0x4421, 0x447e, PDF_CMAP_RANGE, 595 },
	{ 0x4521, 0x457e, PDF_CMAP_RANGE, 689 },
	{ 0x4621, 0x467e, PDF_CMAP_RANGE, 783 },
	{ 0x4721, 0x477e, PDF_CMAP_RANGE, 877 },
	{ 0x4821, 0x487e, PDF_CMAP_RANGE, 971 },
	{ 0x4921, 0x497e, PDF_CMAP_RANGE, 1065 },
	{ 0x4a21, 0x4a7e, PDF_CMAP_RANGE, 1159 },
	{ 0x4b21, 0x4b7e, PDF_CMAP_RANGE, 1253 },
	{ 0x4c21, 0x4c7e, PDF_CMAP_RANGE, 1347 },
	{ 0x4d21, 0x4d7e, PDF_CMAP_RANGE, 1441 },
	{ 0x4e21, 0x4e7e, PDF_CMAP_RANGE, 1535 },
	{ 0x4f21, 0x4f7e, PDF_CMAP_RANGE, 1629 },
	{ 0x5021, 0x507e, PDF_CMAP_RANGE, 1723 },
	{ 0x5121, 0x517e, PDF_CMAP_RANGE, 1817 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 1911 },
	{ 0x5321, 0x537e, PDF_CMAP_RANGE, 2005 },
	{ 0x5421, 0x547e, PDF_CMAP_RANGE, 2099 },
	{ 0x5521, 0x557e, PDF_CMAP_RANGE, 2193 },
	{ 0x5621, 0x567e, PDF_CMAP_RANGE, 2287 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 2381 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 2475 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 2569 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 2663 },
	{ 0x5b21, 0x5b7e, PDF_CMAP_RANGE, 2757 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 2851 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 2945 },
	{ 0x5e21, 0x5e7e, PDF_CMAP_RANGE, 3039 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 3133 },
	{ 0x6021, 0x607e, PDF_CMAP_RANGE, 3227 },
	{ 0x6121, 0x617e, PDF_CMAP_RANGE, 3321 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 3415 },
	{ 0x6321, 0x637e, PDF_CMAP_RANGE, 3509 },
	{ 0x6421, 0x647e, PDF_CMAP_RANGE, 3603 },
	{ 0x6521, 0x657e, PDF_CMAP_RANGE, 3697 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 3791 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 3885 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 3979 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 4073 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 4167 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 4261 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 4355 },
	{ 0x6d21, 0x6d7e, PDF_CMAP_RANGE, 4449 },
	{ 0x6e21, 0x6e7e, PDF_CMAP_RANGE, 4543 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 4637 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 4731 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 4825 },
	{ 0x7221, 0x727e, PDF_CMAP_RANGE, 4919 },
	{ 0x7321, 0x737e, PDF_CMAP_RANGE, 5013 },
	{ 0x7421, 0x747e, PDF_CMAP_RANGE, 5107 },
	{ 0x7521, 0x757e, PDF_CMAP_RANGE, 5201 },
	{ 0x7621, 0x767e, PDF_CMAP_RANGE, 5295 },
	{ 0x7721, 0x777e, PDF_CMAP_RANGE, 5389 },
	{ 0x7821, 0x787e, PDF_CMAP_RANGE, 5483 },
	{ 0x7921, 0x797e, PDF_CMAP_RANGE, 5577 },
	{ 0x7a21, 0x7a7e, PDF_CMAP_RANGE, 5671 },
	{ 0x7b21, 0x7b7e, PDF_CMAP_RANGE, 5765 },
	{ 0x7c21, 0x7c7e, PDF_CMAP_RANGE, 5859 },
	{ 0x7d21, 0x7d4b, PDF_CMAP_RANGE, 5953 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 3791 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 3885 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 3979 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 4073 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 4167 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 4261 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 4355 },
	{ 0x6d21, 0x6d7e, PDF_CMAP_RANGE, 4449 },
	{ 0x6e21, 0x6e7e, PDF_CMAP_RANGE, 4543 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 4637 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 4731 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 4825 },
	{ 0x7221, 0x727e, PDF_CMAP_RANGE, 4919 },
	{ 0x7321, 0x737e, PDF_CMAP_RANGE, 5013 },
	{ 0x7421, 0x747e, PDF_CMAP_RANGE, 5107 },
	{ 0x7521, 0x757e, PDF_CMAP_RANGE, 5201 },
	{ 0x7621, 0x767e, PDF_CMAP_RANGE, 5295 },
	{ 0x7721, 0x777e, PDF_CMAP_RANGE, 5389 },
	{ 0x7821, 0x787e, PDF_CMAP_RANGE, 5483 },
	{ 0x7921, 0x797e, PDF_CMAP_RANGE, 5577 },
	{ 0x7a21, 0x7a7e, PDF_CMAP_RANGE, 5671 },
	{ 0x7b21, 0x7b7e, PDF_CMAP_RANGE, 5765 },
	{ 0x7c21, 0x7c7e, PDF_CMAP_RANGE, 5859 },
	{ 0x7d21, 0x7d4b, PDF_CMAP_RANGE, 5953 },
};

static const unsigned short pdf_cmap_CNS1_H_table[34] =
{
	596,539,602,540,607,5998,608,610,
	544,611,5999,545,546,6000,547,614,
	633,6005,6006,646,550,725,556,862,
	866,6163,6168,560,1307,7731,2552,7732,
	9056,9746,
};

pdf_cmap pdf_cmap_CNS1_H =
{
	-1, "CNS1-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	156, 156, (pdf_range*) pdf_cmap_CNS1_H_ranges,
	34, 34, (unsigned short*) pdf_cmap_CNS1_H_table,
};

/* CNS1-V */

static const pdf_range pdf_cmap_CNS1_V_ranges[] =
{
	{ 0x212c, 0x212c, PDF_CMAP_SINGLE, 13646 },
	{ 0x213b, 0x213b, PDF_CMAP_SINGLE, 124 },
	{ 0x213d, 0x213d, PDF_CMAP_SINGLE, 126 },
	{ 0x213e, 0x213f, PDF_CMAP_RANGE, 130 },
	{ 0x2142, 0x2143, PDF_CMAP_RANGE, 134 },
	{ 0x2146, 0x2147, PDF_CMAP_RANGE, 138 },
	{ 0x214a, 0x214b, PDF_CMAP_RANGE, 142 },
	{ 0x214e, 0x214f, PDF_CMAP_RANGE, 146 },
	{ 0x2152, 0x2153, PDF_CMAP_RANGE, 150 },
	{ 0x2156, 0x2157, PDF_CMAP_RANGE, 154 },
	{ 0x215a, 0x215b, PDF_CMAP_RANGE, 158 },
	{ 0x2244, 0x2244, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_CNS1_V_table[1] = { 0 };

pdf_cmap pdf_cmap_CNS1_V =
{
	-1, "CNS1-V", "CNS1-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_CNS1_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_CNS1_V_table,
};

/* CNS2-H */

static const pdf_range pdf_cmap_CNS2_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 5996 },
	{ 0x2221, 0x227e, PDF_CMAP_RANGE, 6090 },
	{ 0x2321, 0x237e, PDF_CMAP_RANGE, 6184 },
	{ 0x2421, 0x247e, PDF_CMAP_RANGE, 6278 },
	{ 0x2521, 0x257e, PDF_CMAP_RANGE, 6372 },
	{ 0x2621, 0x267e, PDF_CMAP_RANGE, 6466 },
	{ 0x2721, 0x277e, PDF_CMAP_RANGE, 6560 },
	{ 0x2821, 0x287e, PDF_CMAP_RANGE, 6654 },
	{ 0x2921, 0x297e, PDF_CMAP_RANGE, 6748 },
	{ 0x2a21, 0x2a7e, PDF_CMAP_RANGE, 6842 },
	{ 0x2b21, 0x2b7e, PDF_CMAP_RANGE, 6936 },
	{ 0x2c21, 0x2c7e, PDF_CMAP_RANGE, 7030 },
	{ 0x2d21, 0x2d7e, PDF_CMAP_RANGE, 7124 },
	{ 0x2e21, 0x2e7e, PDF_CMAP_RANGE, 7218 },
	{ 0x2f21, 0x2f7e, PDF_CMAP_RANGE, 7312 },
	{ 0x3021, 0x307e, PDF_CMAP_RANGE, 7406 },
	{ 0x3121, 0x317e, PDF_CMAP_RANGE, 7500 },
	{ 0x3221, 0x327e, PDF_CMAP_RANGE, 7594 },
	{ 0x3321, 0x337e, PDF_CMAP_RANGE, 7688 },
	{ 0x3421, 0x347e, PDF_CMAP_RANGE, 7782 },
	{ 0x3521, 0x357e, PDF_CMAP_RANGE, 7876 },
	{ 0x3621, 0x367e, PDF_CMAP_RANGE, 7970 },
	{ 0x3721, 0x377e, PDF_CMAP_RANGE, 8064 },
	{ 0x3821, 0x387e, PDF_CMAP_RANGE, 8158 },
	{ 0x3921, 0x397e, PDF_CMAP_RANGE, 8252 },
	{ 0x3a21, 0x3a7e, PDF_CMAP_RANGE, 8346 },
	{ 0x3b21, 0x3b7e, PDF_CMAP_RANGE, 8440 },
	{ 0x3c21, 0x3c7e, PDF_CMAP_RANGE, 8534 },
	{ 0x3d21, 0x3d7e, PDF_CMAP_RANGE, 8628 },
	{ 0x3e21, 0x3e7e, PDF_CMAP_RANGE, 8722 },
	{ 0x3f21, 0x3f7e, PDF_CMAP_RANGE, 8816 },
	{ 0x4021, 0x407e, PDF_CMAP_RANGE, 8910 },
	{ 0x4121, 0x417e, PDF_CMAP_RANGE, 9004 },
	{ 0x4221, 0x427e, PDF_CMAP_RANGE, 9098 },
	{ 0x4321, 0x437e, PDF_CMAP_RANGE, 9192 },
	{ 0x4421, 0x447e, PDF_CMAP_RANGE, 9286 },
	{ 0x4521, 0x457e, PDF_CMAP_RANGE, 9380 },
	{ 0x4621, 0x467e, PDF_CMAP_RANGE, 9474 },
	{ 0x4721, 0x477e, PDF_CMAP_RANGE, 9568 },
	{ 0x4821, 0x487e, PDF_CMAP_RANGE, 9662 },
	{ 0x4921, 0x497e, PDF_CMAP_RANGE, 9756 },
	{ 0x4a21, 0x4a7e, PDF_CMAP_RANGE, 9850 },
	{ 0x4b21, 0x4b7e, PDF_CMAP_RANGE, 9944 },
	{ 0x4c21, 0x4c7e, PDF_CMAP_RANGE, 10038 },
	{ 0x4d21, 0x4d7e, PDF_CMAP_RANGE, 10132 },
	{ 0x4e21, 0x4e7e, PDF_CMAP_RANGE, 10226 },
	{ 0x4f21, 0x4f7e, PDF_CMAP_RANGE, 10320 },
	{ 0x5021, 0x507e, PDF_CMAP_RANGE, 10414 },
	{ 0x5121, 0x517e, PDF_CMAP_RANGE, 10508 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 10602 },
	{ 0x5321, 0x537e, PDF_CMAP_RANGE, 10696 },
	{ 0x5421, 0x547e, PDF_CMAP_RANGE, 10790 },
	{ 0x5521, 0x557e, PDF_CMAP_RANGE, 10884 },
	{ 0x5621, 0x567e, PDF_CMAP_RANGE, 10978 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 11072 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 11166 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 11260 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 11354 },
	{ 0x5b21, 0x5b7e, PDF_CMAP_RANGE, 11448 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 11542 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 11636 },
	{ 0x5e21, 0x5e7e, PDF_CMAP_RANGE, 11730 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 11824 },
	{ 0x6021, 0x607e, PDF_CMAP_RANGE, 11918 },
	{ 0x6121, 0x617e, PDF_CMAP_RANGE, 12012 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 12106 },
	{ 0x6321, 0x637e, PDF_CMAP_RANGE, 12200 },
	{ 0x6421, 0x647e, PDF_CMAP_RANGE, 12294 },
	{ 0x6521, 0x657e, PDF_CMAP_RANGE, 12388 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 12482 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 12576 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 12670 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 12764 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 12858 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 12952 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 13046 },
	{ 0x6d21, 0x6d7e, PDF_CMAP_RANGE, 13140 },
	{ 0x6e21, 0x6e7e, PDF_CMAP_RANGE, 13234 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 13328 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 13422 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 13516 },
	{ 0x7221, 0x7244, PDF_CMAP_RANGE, 13610 },
};

static const unsigned short pdf_cmap_CNS2_H_table[1] = { 0 };

pdf_cmap pdf_cmap_CNS2_H =
{
	-1, "CNS2-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	82, 82, (pdf_range*) pdf_cmap_CNS2_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_CNS2_H_table,
};

/* CNS2-V */

static const pdf_range pdf_cmap_CNS2_V_ranges[] =
{
	/* dummy entry for non-c99 compilers */
	{ 0x0, 0x0, PDF_CMAP_RANGE, 0 }
};

static const unsigned short pdf_cmap_CNS2_V_table[1] = { 0 };

pdf_cmap pdf_cmap_CNS2_V =
{
	-1, "CNS2-V", "CNS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	0, 0, (pdf_range*) pdf_cmap_CNS2_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_CNS2_V_table,
};

/* ETen-B5-H */

static const pdf_range pdf_cmap_ETen_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 13648 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 0 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 2 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc6a1, 0xc6be, PDF_CMAP_RANGE, 506 },
	{ 0xc6bf, 0xc6d7, PDF_CMAP_RANGE, 537 },
	{ 0xc6d8, 0xc6de, PDF_CMAP_RANGE, 13747 },
	{ 0xc6df, 0xc6df, PDF_CMAP_SINGLE, 6036 },
	{ 0xc6e0, 0xc6fe, PDF_CMAP_RANGE, 13754 },
	{ 0xc740, 0xc77e, PDF_CMAP_RANGE, 13785 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 13848 },
	{ 0xc840, 0xc87e, PDF_CMAP_RANGE, 13942 },
	{ 0xc8a1, 0xc8d3, PDF_CMAP_RANGE, 14005 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 4 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9d6, 0xf9fe, PDF_CMAP_RANGE, 14056 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9d6, 0xf9fe, PDF_CMAP_RANGE, 14056 },
};

static const unsigned short pdf_cmap_ETen_B5_H_table[7] =
{
	248,247,5551,5550,13585,13629,13641,
};

pdf_cmap pdf_cmap_ETen_B5_H =
{
	-1, "ETen-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa140, 0xfefe },
	},
	254, 254, (pdf_range*) pdf_cmap_ETen_B5_H_ranges,
	7, 7, (unsigned short*) pdf_cmap_ETen_B5_H_table,
};

/* ETen-B5-V */

static const pdf_range pdf_cmap_ETen_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
	{ 0xc6e4, 0xc6e5, PDF_CMAP_RANGE, 14097 },
};

static const unsigned short pdf_cmap_ETen_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_ETen_B5_V =
{
	-1, "ETen-B5-V", "ETen-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	13, 13, (pdf_range*) pdf_cmap_ETen_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_ETen_B5_V_table,
};

/* ETenms-B5-H */

static const pdf_range pdf_cmap_ETenms_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
};

static const unsigned short pdf_cmap_ETenms_B5_H_table[1] = { 0 };

pdf_cmap pdf_cmap_ETenms_B5_H =
{
	-1, "ETenms-B5-H", "ETen-B5-H", nil, 0,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	1, 1, (pdf_range*) pdf_cmap_ETenms_B5_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_ETenms_B5_H_table,
};

/* ETenms-B5-V */

static const pdf_range pdf_cmap_ETenms_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14c, PDF_CMAP_TABLE, 0 },
	{ 0xa156, 0xa156, PDF_CMAP_SINGLE, 312 },
	{ 0xa158, 0xa158, PDF_CMAP_SINGLE, 122 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa17d, 0xa17e, PDF_CMAP_RANGE, 130 },
	{ 0xa1a1, 0xa1a2, PDF_CMAP_RANGE, 134 },
	{ 0xa1a3, 0xa1a4, PDF_CMAP_RANGE, 138 },
	{ 0xc6e4, 0xc6e5, PDF_CMAP_RANGE, 14097 },
	{ 0xc6e4, 0xc6e5, PDF_CMAP_RANGE, 14097 },
};

static const unsigned short pdf_cmap_ETenms_B5_V_table[2] =
{
	13646,109,
};

pdf_cmap pdf_cmap_ETenms_B5_V =
{
	-1, "ETenms-B5-V", "ETenms-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	18, 18, (pdf_range*) pdf_cmap_ETenms_B5_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_ETenms_B5_V_table,
};

/* ETHK-B5-H */

static const pdf_range pdf_cmap_ETHK_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8740, 0x8765, PDF_CMAP_RANGE, 18965 },
	{ 0x8767, 0x8779, PDF_CMAP_RANGE, 19003 },
	{ 0x877a, 0x877e, PDF_CMAP_RANGE, 19088 },
	{ 0x87a1, 0x87df, PDF_CMAP_RANGE, 19093 },
	{ 0x8840, 0x8855, PDF_CMAP_RANGE, 17609 },
	{ 0x8856, 0x887e, PDF_CMAP_RANGE, 18785 },
	{ 0x88a1, 0x88a8, PDF_CMAP_RANGE, 18826 },
	{ 0x88a9, 0x88aa, PDF_CMAP_RANGE, 18844 },
	{ 0x8940, 0x8941, PDF_CMAP_RANGE, 17716 },
	{ 0x8943, 0x8943, PDF_CMAP_SINGLE, 17718 },
	{ 0x8946, 0x8949, PDF_CMAP_RANGE, 17719 },
	{ 0x894c, 0x894d, PDF_CMAP_TABLE, 0 },
	{ 0x894e, 0x8950, PDF_CMAP_RANGE, 17724 },
	{ 0x8951, 0x8951, PDF_CMAP_SINGLE, 17306 },
	{ 0x8952, 0x897e, PDF_CMAP_RANGE, 17727 },
	{ 0x89a1, 0x89a5, PDF_CMAP_RANGE, 17772 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 17314 },
	{ 0x89ab, 0x89ae, PDF_CMAP_TABLE, 2 },
	{ 0x89b0, 0x89b2, PDF_CMAP_RANGE, 17779 },
	{ 0x89b5, 0x89bf, PDF_CMAP_RANGE, 17782 },
	{ 0x89c1, 0x89c3, PDF_CMAP_RANGE, 17793 },
	{ 0x89c5, 0x89ce, PDF_CMAP_RANGE, 17796 },
	{ 0x89cf, 0x89cf, PDF_CMAP_SINGLE, 17340 },
	{ 0x89d0, 0x89d8, PDF_CMAP_RANGE, 17806 },
	{ 0x89d9, 0x89dd, PDF_CMAP_TABLE, 6 },
	{ 0x89de, 0x89e0, PDF_CMAP_RANGE, 17817 },
	{ 0x89e1, 0x89e3, PDF_CMAP_TABLE, 11 },
	{ 0x89e4, 0x89e9, PDF_CMAP_RANGE, 17821 },
	{ 0x89ea, 0x89eb, PDF_CMAP_RANGE, 17317 },
	{ 0x89ec, 0x89f9, PDF_CMAP_RANGE, 17827 },
	{ 0x89fa, 0x89fa, PDF_CMAP_SINGLE, 17321 },
	{ 0x89fb, 0x89fe, PDF_CMAP_RANGE, 17841 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 14 },
	{ 0x8a43, 0x8a4c, PDF_CMAP_RANGE, 17163 },
	{ 0x8a4d, 0x8a4d, PDF_CMAP_SINGLE, 17846 },
	{ 0x8a4e, 0x8a59, PDF_CMAP_RANGE, 17174 },
	{ 0x8a5a, 0x8a5a, PDF_CMAP_SINGLE, 17847 },
	{ 0x8a5b, 0x8a5d, PDF_CMAP_RANGE, 17187 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 17848 },
	{ 0x8a5f, 0x8a62, PDF_CMAP_RANGE, 17191 },
	{ 0x8a64, 0x8a70, PDF_CMAP_RANGE, 17196 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 17849 },
	{ 0x8a72, 0x8a74, PDF_CMAP_RANGE, 17210 },
	{ 0x8a76, 0x8a77, PDF_CMAP_TABLE, 16 },
	{ 0x8a78, 0x8a79, PDF_CMAP_RANGE, 17216 },
	{ 0x8a7a, 0x8a7e, PDF_CMAP_TABLE, 18 },
	{ 0x8aa1, 0x8aa7, PDF_CMAP_RANGE, 17223 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 17854 },
	{ 0x8aa9, 0x8aaa, PDF_CMAP_RANGE, 17231 },
	{ 0x8aac, 0x8ab0, PDF_CMAP_RANGE, 17234 },
	{ 0x8ab2, 0x8ab5, PDF_CMAP_RANGE, 17240 },
	{ 0x8ab6, 0x8ab9, PDF_CMAP_TABLE, 23 },
	{ 0x8abb, 0x8ac7, PDF_CMAP_RANGE, 17249 },
	{ 0x8ac9, 0x8acb, PDF_CMAP_RANGE, 17263 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 17857 },
	{ 0x8ace, 0x8ad5, PDF_CMAP_RANGE, 17268 },
	{ 0x8ad6, 0x8ad7, PDF_CMAP_RANGE, 17858 },
	{ 0x8ad8, 0x8adc, PDF_CMAP_RANGE, 17278 },
	{ 0x8adf, 0x8ae5, PDF_CMAP_RANGE, 17285 },
	{ 0x8ae6, 0x8ae7, PDF_CMAP_TABLE, 27 },
	{ 0x8ae8, 0x8af4, PDF_CMAP_RANGE, 17861 },
	{ 0x8af6, 0x8afe, PDF_CMAP_RANGE, 17874 },
	{ 0x8b40, 0x8b40, PDF_CMAP_SINGLE, 17883 },
	{ 0x8b41, 0x8b42, PDF_CMAP_RANGE, 17292 },
	{ 0x8b43, 0x8b44, PDF_CMAP_RANGE, 17884 },
	{ 0x8b45, 0x8b4c, PDF_CMAP_TABLE, 29 },
	{ 0x8b4d, 0x8b50, PDF_CMAP_RANGE, 17298 },
	{ 0x8b51, 0x8b53, PDF_CMAP_RANGE, 17890 },
	{ 0x8b55, 0x8b57, PDF_CMAP_RANGE, 17893 },
	{ 0x8b58, 0x8b5b, PDF_CMAP_TABLE, 37 },
	{ 0x8b5c, 0x8b60, PDF_CMAP_RANGE, 17897 },
	{ 0x8b61, 0x8b61, PDF_CMAP_SINGLE, 17319 },
	{ 0x8b62, 0x8b67, PDF_CMAP_RANGE, 17902 },
	{ 0x8b68, 0x8b68, PDF_CMAP_SINGLE, 17324 },
	{ 0x8b69, 0x8b7e, PDF_CMAP_RANGE, 17908 },
	{ 0x8ba1, 0x8bbf, PDF_CMAP_RANGE, 17930 },
	{ 0x8bc0, 0x8bdc, PDF_CMAP_RANGE, 17631 },
	{ 0x8bde, 0x8bfd, PDF_CMAP_RANGE, 17660 },
	{ 0x8c40, 0x8c61, PDF_CMAP_RANGE, 18849 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 19022 },
	{ 0x8c63, 0x8c7e, PDF_CMAP_RANGE, 18883 },
	{ 0x8ca1, 0x8ca5, PDF_CMAP_RANGE, 18911 },
	{ 0x8ca7, 0x8cc5, PDF_CMAP_RANGE, 18916 },
	{ 0x8cc9, 0x8ccc, PDF_CMAP_RANGE, 18947 },
	{ 0x8cce, 0x8cda, PDF_CMAP_RANGE, 18951 },
	{ 0x8cdb, 0x8cdc, PDF_CMAP_TABLE, 41 },
	{ 0x8cdd, 0x8ce4, PDF_CMAP_RANGE, 19024 },
	{ 0x8ce6, 0x8cfe, PDF_CMAP_RANGE, 19032 },
	{ 0x8d40, 0x8d40, PDF_CMAP_SINGLE, 19057 },
	{ 0x8d42, 0x8d5f, PDF_CMAP_RANGE, 19058 },
	{ 0x8d60, 0x8d61, PDF_CMAP_RANGE, 17961 },
	{ 0x8d62, 0x8d62, PDF_CMAP_SINGLE, 17338 },
	{ 0x8d63, 0x8d67, PDF_CMAP_RANGE, 17963 },
	{ 0x8d68, 0x8d6a, PDF_CMAP_TABLE, 43 },
	{ 0x8d6b, 0x8d6d, PDF_CMAP_RANGE, 17968 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 17342 },
	{ 0x8d6f, 0x8d75, PDF_CMAP_RANGE, 17971 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 17343 },
	{ 0x8d77, 0x8d79, PDF_CMAP_RANGE, 17978 },
	{ 0x8d7a, 0x8d7c, PDF_CMAP_TABLE, 46 },
	{ 0x8d7d, 0x8d7e, PDF_CMAP_RANGE, 17982 },
	{ 0x8da1, 0x8da4, PDF_CMAP_RANGE, 17984 },
	{ 0x8da5, 0x8da5, PDF_CMAP_SINGLE, 17346 },
	{ 0x8da6, 0x8da7, PDF_CMAP_RANGE, 17988 },
	{ 0x8da8, 0x8da9, PDF_CMAP_TABLE, 49 },
	{ 0x8daa, 0x8db5, PDF_CMAP_RANGE, 17990 },
	{ 0x8db6, 0x8db6, PDF_CMAP_SINGLE, 17351 },
	{ 0x8db7, 0x8dc2, PDF_CMAP_RANGE, 18002 },
	{ 0x8dc3, 0x8dc3, PDF_CMAP_SINGLE, 17352 },
	{ 0x8dc4, 0x8df9, PDF_CMAP_RANGE, 18014 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 17401 },
	{ 0x8dfb, 0x8dfe, PDF_CMAP_RANGE, 18068 },
	{ 0x8e40, 0x8e44, PDF_CMAP_RANGE, 14123 },
	{ 0x8e45, 0x8e45, PDF_CMAP_SINGLE, 18072 },
	{ 0x8e46, 0x8e68, PDF_CMAP_RANGE, 14128 },
	{ 0x8e69, 0x8e6a, PDF_CMAP_TABLE, 51 },
	{ 0x8e6b, 0x8e6c, PDF_CMAP_RANGE, 18073 },
	{ 0x8e6d, 0x8e6e, PDF_CMAP_RANGE, 14166 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 11752 },
	{ 0x8e70, 0x8e75, PDF_CMAP_RANGE, 14169 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 18075 },
	{ 0x8e77, 0x8e7a, PDF_CMAP_RANGE, 14175 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 18076 },
	{ 0x8e7c, 0x8e7d, PDF_CMAP_RANGE, 14180 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 289 },
	{ 0x8ea1, 0x8ea5, PDF_CMAP_RANGE, 14182 },
	{ 0x8ea6, 0x8ea6, PDF_CMAP_SINGLE, 18077 },
	{ 0x8ea7, 0x8eaa, PDF_CMAP_RANGE, 14187 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 4203 },
	{ 0x8eac, 0x8eb3, PDF_CMAP_RANGE, 14192 },
	{ 0x8eb4, 0x8eb4, PDF_CMAP_SINGLE, 4902 },
	{ 0x8eb5, 0x8eb7, PDF_CMAP_RANGE, 14201 },
	{ 0x8eb8, 0x8eb8, PDF_CMAP_SINGLE, 18078 },
	{ 0x8eb9, 0x8ec8, PDF_CMAP_RANGE, 14205 },
	{ 0x8ec9, 0x8ec9, PDF_CMAP_SINGLE, 18079 },
	{ 0x8eca, 0x8ecc, PDF_CMAP_RANGE, 14221 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 1643 },
	{ 0x8ece, 0x8ecf, PDF_CMAP_RANGE, 14225 },
	{ 0x8ed0, 0x8ed0, PDF_CMAP_SINGLE, 4910 },
	{ 0x8ed1, 0x8ee4, PDF_CMAP_RANGE, 14228 },
	{ 0x8ee5, 0x8ee5, PDF_CMAP_SINGLE, 18080 },
	{ 0x8ee6, 0x8eee, PDF_CMAP_RANGE, 14248 },
	{ 0x8eef, 0x8eef, PDF_CMAP_SINGLE, 18081 },
	{ 0x8ef0, 0x8ef5, PDF_CMAP_RANGE, 14257 },
	{ 0x8ef6, 0x8ef6, PDF_CMAP_SINGLE, 18082 },
	{ 0x8ef7, 0x8efe, PDF_CMAP_RANGE, 14264 },
	{ 0x8f40, 0x8f56, PDF_CMAP_RANGE, 14272 },
	{ 0x8f57, 0x8f59, PDF_CMAP_TABLE, 53 },
	{ 0x8f5a, 0x8f5e, PDF_CMAP_RANGE, 14297 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 18084 },
	{ 0x8f60, 0x8f66, PDF_CMAP_RANGE, 14302 },
	{ 0x8f67, 0x8f69, PDF_CMAP_TABLE, 56 },
	{ 0x8f6a, 0x8f6d, PDF_CMAP_RANGE, 14311 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 8877 },
	{ 0x8f6f, 0x8f78, PDF_CMAP_RANGE, 14316 },
	{ 0x8f79, 0x8f79, PDF_CMAP_SINGLE, 18086 },
	{ 0x8f7a, 0x8f7e, PDF_CMAP_RANGE, 14327 },
	{ 0x8fa1, 0x8faf, PDF_CMAP_RANGE, 14332 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 18087 },
	{ 0x8fb1, 0x8fc4, PDF_CMAP_RANGE, 14348 },
	{ 0x8fc5, 0x8fc7, PDF_CMAP_TABLE, 59 },
	{ 0x8fc8, 0x8fc9, PDF_CMAP_RANGE, 14369 },
	{ 0x8fca, 0x8fcc, PDF_CMAP_TABLE, 62 },
	{ 0x8fcd, 0x8fd9, PDF_CMAP_RANGE, 14374 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 18091 },
	{ 0x8fdb, 0x8fe2, PDF_CMAP_RANGE, 14387 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 18092 },
	{ 0x8fe4, 0x8ffb, PDF_CMAP_RANGE, 14396 },
	{ 0x8ffc, 0x8ffe, PDF_CMAP_TABLE, 65 },
	{ 0x9040, 0x9054, PDF_CMAP_RANGE, 14422 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 18094 },
	{ 0x9056, 0x905b, PDF_CMAP_RANGE, 14444 },
	{ 0x905c, 0x905e, PDF_CMAP_RANGE, 18095 },
	{ 0x905f, 0x906c, PDF_CMAP_RANGE, 14451 },
	{ 0x906d, 0x906f, PDF_CMAP_TABLE, 68 },
	{ 0x9070, 0x9079, PDF_CMAP_RANGE, 14467 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 14057 },
	{ 0x907b, 0x907e, PDF_CMAP_RANGE, 14477 },
	{ 0x90a1, 0x90a5, PDF_CMAP_RANGE, 14481 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 18099 },
	{ 0x90a7, 0x90b7, PDF_CMAP_RANGE, 14486 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 18100 },
	{ 0x90b9, 0x90db, PDF_CMAP_RANGE, 14503 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 5009 },
	{ 0x90dd, 0x90f0, PDF_CMAP_RANGE, 14539 },
	{ 0x90f1, 0x90f1, PDF_CMAP_SINGLE, 5796 },
	{ 0x90f2, 0x90fe, PDF_CMAP_RANGE, 14560 },
	{ 0x9140, 0x9164, PDF_CMAP_RANGE, 14573 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 18101 },
	{ 0x9166, 0x916d, PDF_CMAP_RANGE, 14610 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 18102 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 14618 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 18103 },
	{ 0x91a1, 0x91a2, PDF_CMAP_TABLE, 71 },
	{ 0x91a3, 0x91be, PDF_CMAP_RANGE, 14634 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 12402 },
	{ 0x91c0, 0x91c7, PDF_CMAP_RANGE, 14663 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 18105 },
	{ 0x91c9, 0x91fe, PDF_CMAP_RANGE, 14672 },
	{ 0x9240, 0x9243, PDF_CMAP_RANGE, 14726 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 14728 },
	{ 0x9245, 0x9263, PDF_CMAP_RANGE, 14731 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 18106 },
	{ 0x9265, 0x926c, PDF_CMAP_RANGE, 14763 },
	{ 0x926d, 0x926d, PDF_CMAP_SINGLE, 18107 },
	{ 0x926e, 0x927e, PDF_CMAP_RANGE, 14772 },
	{ 0x92a1, 0x92ae, PDF_CMAP_RANGE, 14789 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 281 },
	{ 0x92b1, 0x92b2, PDF_CMAP_TABLE, 73 },
	{ 0x92b3, 0x92c7, PDF_CMAP_RANGE, 14803 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 16300 },
	{ 0x92c9, 0x92d0, PDF_CMAP_RANGE, 14825 },
	{ 0x92d1, 0x92d1, PDF_CMAP_SINGLE, 10620 },
	{ 0x92d2, 0x92e4, PDF_CMAP_RANGE, 14834 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 18108 },
	{ 0x92e6, 0x92f1, PDF_CMAP_RANGE, 14853 },
	{ 0x92f2, 0x92f2, PDF_CMAP_SINGLE, 18109 },
	{ 0x92f3, 0x92fe, PDF_CMAP_RANGE, 14865 },
	{ 0x9340, 0x9367, PDF_CMAP_RANGE, 14877 },
	{ 0x9368, 0x9368, PDF_CMAP_SINGLE, 18110 },
	{ 0x9369, 0x937e, PDF_CMAP_RANGE, 14917 },
	{ 0x93a1, 0x93a9, PDF_CMAP_RANGE, 14939 },
	{ 0x93aa, 0x93aa, PDF_CMAP_SINGLE, 18111 },
	{ 0x93ab, 0x93c1, PDF_CMAP_RANGE, 14948 },
	{ 0x93c2, 0x93c2, PDF_CMAP_SINGLE, 18112 },
	{ 0x93c3, 0x93e4, PDF_CMAP_RANGE, 14971 },
	{ 0x93e5, 0x93e5, PDF_CMAP_SINGLE, 18113 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 15005 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 18114 },
	{ 0x93e9, 0x93ea, PDF_CMAP_RANGE, 15008 },
	{ 0x93eb, 0x93eb, PDF_CMAP_SINGLE, 18115 },
	{ 0x93ec, 0x93fe, PDF_CMAP_RANGE, 15010 },
	{ 0x9440, 0x9445, PDF_CMAP_RANGE, 15029 },
	{ 0x9446, 0x9447, PDF_CMAP_TABLE, 75 },
	{ 0x9448, 0x9478, PDF_CMAP_RANGE, 15036 },
	{ 0x9479, 0x9479, PDF_CMAP_SINGLE, 18117 },
	{ 0x947a, 0x947e, PDF_CMAP_RANGE, 15086 },
	{ 0x94a1, 0x94c9, PDF_CMAP_RANGE, 15091 },
	{ 0x94ca, 0x94cb, PDF_CMAP_TABLE, 77 },
	{ 0x94cc, 0x94fe, PDF_CMAP_RANGE, 15134 },
	{ 0x9540, 0x954c, PDF_CMAP_RANGE, 15185 },
	{ 0x954d, 0x954d, PDF_CMAP_SINGLE, 18119 },
	{ 0x954e, 0x9559, PDF_CMAP_RANGE, 15198 },
	{ 0x955a, 0x955a, PDF_CMAP_SINGLE, 18120 },
	{ 0x955b, 0x955e, PDF_CMAP_RANGE, 15210 },
	{ 0x955f, 0x955f, PDF_CMAP_SINGLE, 18121 },
	{ 0x9560, 0x957e, PDF_CMAP_RANGE, 15215 },
	{ 0x95a1, 0x95c5, PDF_CMAP_RANGE, 15246 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 18122 },
	{ 0x95c7, 0x95d8, PDF_CMAP_RANGE, 15283 },
	{ 0x95d9, 0x95d9, PDF_CMAP_SINGLE, 6171 },
	{ 0x95da, 0x95fe, PDF_CMAP_RANGE, 15302 },
	{ 0x9640, 0x9643, PDF_CMAP_RANGE, 15339 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 15919 },
	{ 0x9645, 0x9650, PDF_CMAP_RANGE, 15344 },
	{ 0x9651, 0x9651, PDF_CMAP_SINGLE, 18123 },
	{ 0x9652, 0x9669, PDF_CMAP_RANGE, 15357 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 18124 },
	{ 0x966b, 0x967e, PDF_CMAP_RANGE, 15382 },
	{ 0x96a1, 0x96d3, PDF_CMAP_RANGE, 15402 },
	{ 0x96d4, 0x96d4, PDF_CMAP_SINGLE, 18125 },
	{ 0x96d5, 0x96ec, PDF_CMAP_RANGE, 15453 },
	{ 0x96ed, 0x96ed, PDF_CMAP_SINGLE, 15478 },
	{ 0x96ee, 0x96fb, PDF_CMAP_RANGE, 15478 },
	{ 0x96fc, 0x96fc, PDF_CMAP_SINGLE, 11044 },
	{ 0x96fd, 0x96fe, PDF_CMAP_RANGE, 15493 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 15495 },
	{ 0x97a1, 0x97fe, PDF_CMAP_RANGE, 15558 },
	{ 0x9840, 0x9843, PDF_CMAP_RANGE, 15652 },
	{ 0x9844, 0x9845, PDF_CMAP_RANGE, 18126 },
	{ 0x9846, 0x986e, PDF_CMAP_RANGE, 15658 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 18128 },
	{ 0x9870, 0x9874, PDF_CMAP_RANGE, 15700 },
	{ 0x9875, 0x9876, PDF_CMAP_RANGE, 18129 },
	{ 0x9877, 0x9877, PDF_CMAP_SINGLE, 15705 },
	{ 0x9878, 0x9879, PDF_CMAP_RANGE, 18131 },
	{ 0x987a, 0x987a, PDF_CMAP_SINGLE, 15706 },
	{ 0x987b, 0x987e, PDF_CMAP_RANGE, 18133 },
	{ 0x98a1, 0x98a2, PDF_CMAP_RANGE, 18137 },
	{ 0x98a3, 0x98a3, PDF_CMAP_SINGLE, 15707 },
	{ 0x98a4, 0x98ae, PDF_CMAP_RANGE, 18139 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 15708 },
	{ 0x98b0, 0x98b3, PDF_CMAP_RANGE, 18150 },
	{ 0x98b4, 0x98bc, PDF_CMAP_TABLE, 79 },
	{ 0x98bd, 0x98be, PDF_CMAP_RANGE, 15711 },
	{ 0x98bf, 0x98c1, PDF_CMAP_RANGE, 18158 },
	{ 0x98c2, 0x98c5, PDF_CMAP_TABLE, 88 },
	{ 0x98c6, 0x98c7, PDF_CMAP_RANGE, 15715 },
	{ 0x98c8, 0x98d1, PDF_CMAP_RANGE, 18163 },
	{ 0x98d2, 0x98d2, PDF_CMAP_SINGLE, 17357 },
	{ 0x98d3, 0x98d7, PDF_CMAP_RANGE, 18173 },
	{ 0x98d8, 0x98d9, PDF_CMAP_RANGE, 17358 },
	{ 0x98da, 0x98db, PDF_CMAP_TABLE, 92 },
	{ 0x98dc, 0x98de, PDF_CMAP_RANGE, 18179 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 17364 },
	{ 0x98e0, 0x98e2, PDF_CMAP_RANGE, 18182 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 15717 },
	{ 0x98e4, 0x98e6, PDF_CMAP_RANGE, 18185 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 15718 },
	{ 0x98e8, 0x98ec, PDF_CMAP_RANGE, 18188 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 15719 },
	{ 0x98ee, 0x98ef, PDF_CMAP_RANGE, 18193 },
	{ 0x98f0, 0x98f3, PDF_CMAP_TABLE, 94 },
	{ 0x98f4, 0x98f5, PDF_CMAP_RANGE, 17365 },
	{ 0x98f6, 0x98fb, PDF_CMAP_RANGE, 18197 },
	{ 0x98fc, 0x98fe, PDF_CMAP_TABLE, 98 },
	{ 0x9940, 0x9941, PDF_CMAP_RANGE, 18204 },
	{ 0x9942, 0x9947, PDF_CMAP_TABLE, 101 },
	{ 0x9948, 0x994e, PDF_CMAP_RANGE, 18208 },
	{ 0x994f, 0x994f, PDF_CMAP_SINGLE, 15725 },
	{ 0x9950, 0x9953, PDF_CMAP_RANGE, 18215 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 17369 },
	{ 0x9955, 0x995b, PDF_CMAP_RANGE, 18219 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 17370 },
	{ 0x995d, 0x9963, PDF_CMAP_RANGE, 18226 },
	{ 0x9964, 0x9964, PDF_CMAP_SINGLE, 17372 },
	{ 0x9965, 0x9969, PDF_CMAP_RANGE, 18233 },
	{ 0x996a, 0x996a, PDF_CMAP_SINGLE, 15726 },
	{ 0x996b, 0x996d, PDF_CMAP_RANGE, 18238 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 15727 },
	{ 0x996f, 0x9974, PDF_CMAP_RANGE, 18241 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 15728 },
	{ 0x9976, 0x9977, PDF_CMAP_RANGE, 18247 },
	{ 0x9978, 0x9978, PDF_CMAP_SINGLE, 15729 },
	{ 0x9979, 0x997e, PDF_CMAP_RANGE, 18249 },
	{ 0x99a1, 0x99a6, PDF_CMAP_TABLE, 107 },
	{ 0x99a7, 0x99ad, PDF_CMAP_RANGE, 18258 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 15731 },
	{ 0x99af, 0x99b1, PDF_CMAP_RANGE, 18265 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 17374 },
	{ 0x99b3, 0x99b5, PDF_CMAP_RANGE, 18268 },
	{ 0x99b6, 0x99b6, PDF_CMAP_SINGLE, 15732 },
	{ 0x99b7, 0x99b9, PDF_CMAP_RANGE, 18271 },
	{ 0x99ba, 0x99ba, PDF_CMAP_SINGLE, 15733 },
	{ 0x99bb, 0x99c9, PDF_CMAP_RANGE, 18274 },
	{ 0x99ca, 0x99ca, PDF_CMAP_SINGLE, 17376 },
	{ 0x99cb, 0x99cc, PDF_CMAP_RANGE, 18289 },
	{ 0x99cd, 0x99cd, PDF_CMAP_SINGLE, 17378 },
	{ 0x99ce, 0x99d2, PDF_CMAP_RANGE, 18291 },
	{ 0x99d3, 0x99d3, PDF_CMAP_SINGLE, 17379 },
	{ 0x99d4, 0x99d5, PDF_CMAP_RANGE, 18296 },
	{ 0x99d6, 0x99d6, PDF_CMAP_SINGLE, 17381 },
	{ 0x99d7, 0x99de, PDF_CMAP_RANGE, 18298 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 17375 },
	{ 0x99e0, 0x99e1, PDF_CMAP_RANGE, 18306 },
	{ 0x99e2, 0x99e8, PDF_CMAP_TABLE, 113 },
	{ 0x99e9, 0x99ee, PDF_CMAP_RANGE, 18311 },
	{ 0x99ef, 0x99ef, PDF_CMAP_SINGLE, 17405 },
	{ 0x99f0, 0x99f3, PDF_CMAP_RANGE, 18317 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 15735 },
	{ 0x99f5, 0x99fe, PDF_CMAP_RANGE, 18321 },
	{ 0x9a40, 0x9a49, PDF_CMAP_RANGE, 18331 },
	{ 0x9a4a, 0x9a4c, PDF_CMAP_TABLE, 120 },
	{ 0x9a4d, 0x9a58, PDF_CMAP_RANGE, 18342 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 15738 },
	{ 0x9a5a, 0x9a5e, PDF_CMAP_RANGE, 18354 },
	{ 0x9a5f, 0x9a61, PDF_CMAP_TABLE, 123 },
	{ 0x9a62, 0x9a65, PDF_CMAP_RANGE, 18360 },
	{ 0x9a66, 0x9a6b, PDF_CMAP_TABLE, 126 },
	{ 0x9a6c, 0x9a72, PDF_CMAP_RANGE, 18366 },
	{ 0x9a73, 0x9a75, PDF_CMAP_TABLE, 132 },
	{ 0x9a76, 0x9a7d, PDF_CMAP_RANGE, 18374 },
	{ 0x9a7e, 0x9a7e, PDF_CMAP_SINGLE, 15742 },
	{ 0x9aa1, 0x9aa2, PDF_CMAP_RANGE, 18382 },
	{ 0x9aa3, 0x9aa5, PDF_CMAP_TABLE, 135 },
	{ 0x9aa6, 0x9aa8, PDF_CMAP_RANGE, 18385 },
	{ 0x9aa9, 0x9aaa, PDF_CMAP_TABLE, 138 },
	{ 0x9aab, 0x9ab1, PDF_CMAP_RANGE, 18388 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 15743 },
	{ 0x9ab3, 0x9ab6, PDF_CMAP_RANGE, 18395 },
	{ 0x9ab7, 0x9abd, PDF_CMAP_TABLE, 140 },
	{ 0x9abe, 0x9ac6, PDF_CMAP_RANGE, 18402 },
	{ 0x9ac7, 0x9ac7, PDF_CMAP_SINGLE, 15747 },
	{ 0x9ac8, 0x9acf, PDF_CMAP_RANGE, 18411 },
	{ 0x9ad0, 0x9ad2, PDF_CMAP_TABLE, 147 },
	{ 0x9ad3, 0x9ad8, PDF_CMAP_RANGE, 18420 },
	{ 0x9ad9, 0x9adb, PDF_CMAP_RANGE, 15750 },
	{ 0x9adc, 0x9ae1, PDF_CMAP_RANGE, 18426 },
	{ 0x9ae2, 0x9ae4, PDF_CMAP_TABLE, 150 },
	{ 0x9ae5, 0x9ae7, PDF_CMAP_RANGE, 18433 },
	{ 0x9ae8, 0x9ae9, PDF_CMAP_TABLE, 153 },
	{ 0x9aea, 0x9aed, PDF_CMAP_RANGE, 18436 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 17330 },
	{ 0x9aef, 0x9af1, PDF_CMAP_RANGE, 18440 },
	{ 0x9af2, 0x9af2, PDF_CMAP_SINGLE, 15756 },
	{ 0x9af3, 0x9af5, PDF_CMAP_RANGE, 18443 },
	{ 0x9af6, 0x9af6, PDF_CMAP_SINGLE, 15757 },
	{ 0x9af7, 0x9afa, PDF_CMAP_RANGE, 18446 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 15758 },
	{ 0x9afc, 0x9afe, PDF_CMAP_RANGE, 18450 },
	{ 0x9b40, 0x9b45, PDF_CMAP_RANGE, 18453 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 15759 },
	{ 0x9b47, 0x9b49, PDF_CMAP_RANGE, 18459 },
	{ 0x9b4a, 0x9b4a, PDF_CMAP_SINGLE, 15760 },
	{ 0x9b4b, 0x9b53, PDF_CMAP_RANGE, 18462 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 15762 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 18471 },
	{ 0x9b58, 0x9b5d, PDF_CMAP_TABLE, 155 },
	{ 0x9b5e, 0x9b5f, PDF_CMAP_RANGE, 15766 },
	{ 0x9b60, 0x9b60, PDF_CMAP_SINGLE, 18477 },
	{ 0x9b62, 0x9b6f, PDF_CMAP_RANGE, 18478 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 15768 },
	{ 0x9b74, 0x9b75, PDF_CMAP_RANGE, 18492 },
	{ 0x9b76, 0x9b78, PDF_CMAP_TABLE, 161 },
	{ 0x9b79, 0x9b7a, PDF_CMAP_RANGE, 18494 },
	{ 0x9b7b, 0x9b7e, PDF_CMAP_TABLE, 164 },
	{ 0x9ba1, 0x9ba2, PDF_CMAP_TABLE, 168 },
	{ 0x9ba3, 0x9ba4, PDF_CMAP_RANGE, 15779 },
	{ 0x9ba5, 0x9ba6, PDF_CMAP_RANGE, 18498 },
	{ 0x9ba7, 0x9baa, PDF_CMAP_RANGE, 15781 },
	{ 0x9bab, 0x9bac, PDF_CMAP_TABLE, 170 },
	{ 0x9bad, 0x9bae, PDF_CMAP_RANGE, 18501 },
	{ 0x9baf, 0x9baf, PDF_CMAP_SINGLE, 15786 },
	{ 0x9bb0, 0x9bb1, PDF_CMAP_RANGE, 18503 },
	{ 0x9bb2, 0x9bb9, PDF_CMAP_RANGE, 15787 },
	{ 0x9bba, 0x9bbd, PDF_CMAP_RANGE, 18505 },
	{ 0x9bbe, 0x9bbf, PDF_CMAP_TABLE, 172 },
	{ 0x9bc0, 0x9bc5, PDF_CMAP_RANGE, 15796 },
	{ 0x9bc6, 0x9bc6, PDF_CMAP_SINGLE, 10969 },
	{ 0x9bc7, 0x9bc9, PDF_CMAP_RANGE, 18510 },
	{ 0x9bca, 0x9bcf, PDF_CMAP_TABLE, 174 },
	{ 0x9bd0, 0x9bd1, PDF_CMAP_RANGE, 15805 },
	{ 0x9bd2, 0x9bd5, PDF_CMAP_TABLE, 180 },
	{ 0x9bd6, 0x9bd7, PDF_CMAP_RANGE, 18518 },
	{ 0x9bd8, 0x9bda, PDF_CMAP_RANGE, 15809 },
	{ 0x9bdb, 0x9bdc, PDF_CMAP_RANGE, 18520 },
	{ 0x9bdd, 0x9be3, PDF_CMAP_TABLE, 184 },
	{ 0x9be4, 0x9be6, PDF_CMAP_RANGE, 18524 },
	{ 0x9be7, 0x9be8, PDF_CMAP_TABLE, 191 },
	{ 0x9be9, 0x9beb, PDF_CMAP_RANGE, 15817 },
	{ 0x9bec, 0x9bed, PDF_CMAP_TABLE, 193 },
	{ 0x9bee, 0x9bef, PDF_CMAP_RANGE, 15821 },
	{ 0x9bf0, 0x9bf2, PDF_CMAP_RANGE, 18529 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 15823 },
	{ 0x9bf4, 0x9bf5, PDF_CMAP_RANGE, 18532 },
	{ 0x9bf6, 0x9bf7, PDF_CMAP_TABLE, 195 },
	{ 0x9bf8, 0x9bf9, PDF_CMAP_RANGE, 15825 },
	{ 0x9bfa, 0x9bfa, PDF_CMAP_SINGLE, 18535 },
	{ 0x9bfb, 0x9bfc, PDF_CMAP_RANGE, 15827 },
	{ 0x9bfd, 0x9bfe, PDF_CMAP_RANGE, 18536 },
	{ 0x9c40, 0x9c41, PDF_CMAP_RANGE, 15829 },
	{ 0x9c42, 0x9c43, PDF_CMAP_TABLE, 197 },
	{ 0x9c44, 0x9c46, PDF_CMAP_RANGE, 15832 },
	{ 0x9c47, 0x9c4a, PDF_CMAP_TABLE, 199 },
	{ 0x9c4b, 0x9c4c, PDF_CMAP_RANGE, 18541 },
	{ 0x9c4d, 0x9c52, PDF_CMAP_RANGE, 15837 },
	{ 0x9c53, 0x9c56, PDF_CMAP_TABLE, 203 },
	{ 0x9c57, 0x9c5b, PDF_CMAP_RANGE, 15845 },
	{ 0x9c5c, 0x9c5d, PDF_CMAP_TABLE, 207 },
	{ 0x9c5e, 0x9c5f, PDF_CMAP_RANGE, 18546 },
	{ 0x9c60, 0x9c63, PDF_CMAP_TABLE, 209 },
	{ 0x9c64, 0x9c66, PDF_CMAP_RANGE, 15853 },
	{ 0x9c67, 0x9c6e, PDF_CMAP_TABLE, 213 },
	{ 0x9c6f, 0x9c72, PDF_CMAP_RANGE, 15859 },
	{ 0x9c73, 0x9c74, PDF_CMAP_RANGE, 18554 },
	{ 0x9c75, 0x9c76, PDF_CMAP_RANGE, 15863 },
	{ 0x9c77, 0x9c7a, PDF_CMAP_TABLE, 221 },
	{ 0x9c7b, 0x9c7c, PDF_CMAP_RANGE, 15867 },
	{ 0x9c7d, 0x9c7e, PDF_CMAP_TABLE, 225 },
	{ 0x9ca1, 0x9ca2, PDF_CMAP_RANGE, 15870 },
	{ 0x9ca3, 0x9ca4, PDF_CMAP_RANGE, 18559 },
	{ 0x9ca5, 0x9ca5, PDF_CMAP_SINGLE, 15872 },
	{ 0x9ca6, 0x9ca7, PDF_CMAP_RANGE, 18561 },
	{ 0x9ca8, 0x9ca9, PDF_CMAP_RANGE, 15873 },
	{ 0x9caa, 0x9cac, PDF_CMAP_TABLE, 227 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 15876 },
	{ 0x9caf, 0x9cb0, PDF_CMAP_RANGE, 18565 },
	{ 0x9cb1, 0x9cba, PDF_CMAP_RANGE, 15878 },
	{ 0x9cbb, 0x9cbd, PDF_CMAP_TABLE, 230 },
	{ 0x9cbe, 0x9cc2, PDF_CMAP_RANGE, 15890 },
	{ 0x9cc3, 0x9cc5, PDF_CMAP_RANGE, 18568 },
	{ 0x9cc6, 0x9ccd, PDF_CMAP_RANGE, 15895 },
	{ 0x9cce, 0x9cd0, PDF_CMAP_TABLE, 233 },
	{ 0x9cd1, 0x9cd3, PDF_CMAP_RANGE, 15905 },
	{ 0x9cd4, 0x9cd7, PDF_CMAP_RANGE, 18572 },
	{ 0x9cd8, 0x9cda, PDF_CMAP_RANGE, 15908 },
	{ 0x9cdb, 0x9cdb, PDF_CMAP_SINGLE, 18576 },
	{ 0x9cdc, 0x9ce5, PDF_CMAP_RANGE, 15911 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 18577 },
	{ 0x9ce7, 0x9ce9, PDF_CMAP_RANGE, 15921 },
	{ 0x9cea, 0x9cea, PDF_CMAP_SINGLE, 18578 },
	{ 0x9ceb, 0x9cec, PDF_CMAP_RANGE, 15924 },
	{ 0x9ced, 0x9ced, PDF_CMAP_SINGLE, 18579 },
	{ 0x9cee, 0x9cf9, PDF_CMAP_RANGE, 15926 },
	{ 0x9cfa, 0x9cfc, PDF_CMAP_RANGE, 18580 },
	{ 0x9cfd, 0x9cfe, PDF_CMAP_TABLE, 236 },
	{ 0x9d40, 0x9d40, PDF_CMAP_SINGLE, 17384 },
	{ 0x9d41, 0x9d45, PDF_CMAP_RANGE, 18584 },
	{ 0x9d46, 0x9d46, PDF_CMAP_SINGLE, 15939 },
	{ 0x9d47, 0x9d48, PDF_CMAP_RANGE, 18589 },
	{ 0x9d49, 0x9d49, PDF_CMAP_SINGLE, 15940 },
	{ 0x9d4a, 0x9d4b, PDF_CMAP_RANGE, 18591 },
	{ 0x9d4c, 0x9d4d, PDF_CMAP_RANGE, 15942 },
	{ 0x9d4e, 0x9d51, PDF_CMAP_TABLE, 238 },
	{ 0x9d52, 0x9d54, PDF_CMAP_RANGE, 18595 },
	{ 0x9d55, 0x9d57, PDF_CMAP_TABLE, 242 },
	{ 0x9d58, 0x9d59, PDF_CMAP_RANGE, 18599 },
	{ 0x9d5a, 0x9d5a, PDF_CMAP_SINGLE, 3340 },
	{ 0x9d5b, 0x9d60, PDF_CMAP_RANGE, 18601 },
	{ 0x9d61, 0x9d64, PDF_CMAP_TABLE, 245 },
	{ 0x9d65, 0x9d77, PDF_CMAP_RANGE, 18608 },
	{ 0x9d78, 0x9d79, PDF_CMAP_TABLE, 249 },
	{ 0x9d7a, 0x9d7d, PDF_CMAP_RANGE, 18627 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15951 },
	{ 0x9da1, 0x9da4, PDF_CMAP_RANGE, 18631 },
	{ 0x9da5, 0x9da8, PDF_CMAP_RANGE, 15952 },
	{ 0x9da9, 0x9dab, PDF_CMAP_TABLE, 251 },
	{ 0x9dac, 0x9dad, PDF_CMAP_RANGE, 15957 },
	{ 0x9dae, 0x9daf, PDF_CMAP_RANGE, 18637 },
	{ 0x9db0, 0x9db0, PDF_CMAP_SINGLE, 15960 },
	{ 0x9db1, 0x9db2, PDF_CMAP_RANGE, 18639 },
	{ 0x9db3, 0x9db7, PDF_CMAP_TABLE, 254 },
	{ 0x9db8, 0x9dbb, PDF_CMAP_RANGE, 18643 },
	{ 0x9dbc, 0x9dbd, PDF_CMAP_RANGE, 15964 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 18647 },
	{ 0x9dbf, 0x9dc0, PDF_CMAP_RANGE, 15966 },
	{ 0x9dc1, 0x9dc2, PDF_CMAP_RANGE, 18648 },
	{ 0x9dc3, 0x9dc4, PDF_CMAP_TABLE, 259 },
	{ 0x9dc5, 0x9dc6, PDF_CMAP_RANGE, 18650 },
	{ 0x9dc7, 0x9dc8, PDF_CMAP_RANGE, 15970 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_TABLE, 261 },
	{ 0x9dcb, 0x9dcc, PDF_CMAP_RANGE, 18653 },
	{ 0x9dcd, 0x9dd1, PDF_CMAP_RANGE, 15973 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 18655 },
	{ 0x9dd3, 0x9dd5, PDF_CMAP_RANGE, 15978 },
	{ 0x9dd6, 0x9dd9, PDF_CMAP_RANGE, 18656 },
	{ 0x9dda, 0x9dfb, PDF_CMAP_RANGE, 15981 },
	{ 0x9dfc, 0x9dfc, PDF_CMAP_SINGLE, 18660 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_RANGE, 16015 },
	{ 0x9e40, 0x9e42, PDF_CMAP_RANGE, 16017 },
	{ 0x9e43, 0x9e43, PDF_CMAP_SINGLE, 18661 },
	{ 0x9e44, 0x9e5e, PDF_CMAP_RANGE, 16021 },
	{ 0x9e5f, 0x9e5f, PDF_CMAP_SINGLE, 18662 },
	{ 0x9e60, 0x9e62, PDF_CMAP_RANGE, 16049 },
	{ 0x9e63, 0x9e63, PDF_CMAP_SINGLE, 18663 },
	{ 0x9e64, 0x9e65, PDF_CMAP_RANGE, 16052 },
	{ 0x9e66, 0x9e67, PDF_CMAP_RANGE, 18664 },
	{ 0x9e68, 0x9e6a, PDF_CMAP_TABLE, 263 },
	{ 0x9e6b, 0x9e70, PDF_CMAP_RANGE, 18667 },
	{ 0x9e71, 0x9e73, PDF_CMAP_TABLE, 266 },
	{ 0x9e74, 0x9e76, PDF_CMAP_RANGE, 18674 },
	{ 0x9e77, 0x9e78, PDF_CMAP_RANGE, 16058 },
	{ 0x9e79, 0x9e7e, PDF_CMAP_TABLE, 269 },
	{ 0x9ea1, 0x9ea2, PDF_CMAP_RANGE, 16063 },
	{ 0x9ea3, 0x9ea3, PDF_CMAP_SINGLE, 18680 },
	{ 0x9ea4, 0x9ea6, PDF_CMAP_RANGE, 16065 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 18681 },
	{ 0x9ea9, 0x9eab, PDF_CMAP_TABLE, 275 },
	{ 0x9eac, 0x9ead, PDF_CMAP_RANGE, 16070 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 18684 },
	{ 0x9eaf, 0x9eb1, PDF_CMAP_RANGE, 16072 },
	{ 0x9eb2, 0x9eb3, PDF_CMAP_RANGE, 18685 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 278 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 16076 },
	{ 0x9eb8, 0x9eb9, PDF_CMAP_TABLE, 280 },
	{ 0x9eba, 0x9ebb, PDF_CMAP_RANGE, 18689 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 16079 },
	{ 0x9ebd, 0x9ebe, PDF_CMAP_RANGE, 18691 },
	{ 0x9ebf, 0x9ec0, PDF_CMAP_RANGE, 16080 },
	{ 0x9ec1, 0x9ec3, PDF_CMAP_RANGE, 18693 },
	{ 0x9ec4, 0x9ec5, PDF_CMAP_RANGE, 16082 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 18696 },
	{ 0x9ec7, 0x9eca, PDF_CMAP_RANGE, 16084 },
	{ 0x9ecb, 0x9ecc, PDF_CMAP_RANGE, 18697 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 16089 },
	{ 0x9ece, 0x9ecf, PDF_CMAP_RANGE, 18699 },
	{ 0x9ed0, 0x9ed1, PDF_CMAP_RANGE, 16090 },
	{ 0x9ed2, 0x9ed3, PDF_CMAP_TABLE, 282 },
	{ 0x9ed4, 0x9ed5, PDF_CMAP_RANGE, 18702 },
	{ 0x9ed6, 0x9ed7, PDF_CMAP_RANGE, 16093 },
	{ 0x9ed8, 0x9ed9, PDF_CMAP_RANGE, 18704 },
	{ 0x9eda, 0x9eee, PDF_CMAP_RANGE, 16095 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 9398 },
	{ 0x9ef0, 0x9ef1, PDF_CMAP_RANGE, 16117 },
	{ 0x9ef2, 0x9ef2, PDF_CMAP_SINGLE, 18706 },
	{ 0x9ef3, 0x9ef5, PDF_CMAP_RANGE, 16119 },
	{ 0x9ef6, 0x9ef8, PDF_CMAP_RANGE, 18707 },
	{ 0x9ef9, 0x9efa, PDF_CMAP_RANGE, 16122 },
	{ 0x9efb, 0x9efe, PDF_CMAP_TABLE, 284 },
	{ 0x9f40, 0x9f42, PDF_CMAP_RANGE, 16127 },
	{ 0x9f43, 0x9f43, PDF_CMAP_SINGLE, 18711 },
	{ 0x9f44, 0x9f47, PDF_CMAP_RANGE, 16130 },
	{ 0x9f48, 0x9f48, PDF_CMAP_SINGLE, 18712 },
	{ 0x9f49, 0x9f4a, PDF_CMAP_RANGE, 16134 },
	{ 0x9f4b, 0x9f4c, PDF_CMAP_RANGE, 18713 },
	{ 0x9f4d, 0x9f5f, PDF_CMAP_RANGE, 16136 },
	{ 0x9f60, 0x9f60, PDF_CMAP_SINGLE, 13585 },
	{ 0x9f61, 0x9f65, PDF_CMAP_RANGE, 16156 },
	{ 0x9f66, 0x9f66, PDF_CMAP_SINGLE, 14661 },
	{ 0x9f67, 0x9f68, PDF_CMAP_RANGE, 18715 },
	{ 0x9f69, 0x9f6f, PDF_CMAP_RANGE, 16163 },
	{ 0x9f70, 0x9f70, PDF_CMAP_SINGLE, 18717 },
	{ 0x9f71, 0x9f7e, PDF_CMAP_RANGE, 16170 },
	{ 0x9fa1, 0x9fb4, PDF_CMAP_RANGE, 16184 },
	{ 0x9fb5, 0x9fb5, PDF_CMAP_SINGLE, 18718 },
	{ 0x9fb6, 0x9fba, PDF_CMAP_RANGE, 16204 },
	{ 0x9fbb, 0x9fbb, PDF_CMAP_SINGLE, 18719 },
	{ 0x9fbc, 0x9fbe, PDF_CMAP_RANGE, 16209 },
	{ 0x9fbf, 0x9fc1, PDF_CMAP_TABLE, 288 },
	{ 0x9fc2, 0x9fca, PDF_CMAP_RANGE, 16213 },
	{ 0x9fcb, 0x9fcc, PDF_CMAP_TABLE, 291 },
	{ 0x9fcd, 0x9fd3, PDF_CMAP_RANGE, 16224 },
	{ 0x9fd4, 0x9fd4, PDF_CMAP_SINGLE, 18723 },
	{ 0x9fd5, 0x9fd7, PDF_CMAP_RANGE, 16232 },
	{ 0x9fd8, 0x9fd8, PDF_CMAP_SINGLE, 14910 },
	{ 0x9fd9, 0x9fe3, PDF_CMAP_RANGE, 16236 },
	{ 0x9fe4, 0x9fe4, PDF_CMAP_SINGLE, 18724 },
	{ 0x9fe5, 0x9ff8, PDF_CMAP_RANGE, 16247 },
	{ 0x9ff9, 0x9ff9, PDF_CMAP_SINGLE, 18725 },
	{ 0x9ffa, 0x9ffe, PDF_CMAP_RANGE, 16267 },
	{ 0xa040, 0xa040, PDF_CMAP_SINGLE, 18726 },
	{ 0xa041, 0xa046, PDF_CMAP_RANGE, 16272 },
	{ 0xa047, 0xa047, PDF_CMAP_SINGLE, 18727 },
	{ 0xa048, 0xa054, PDF_CMAP_RANGE, 16278 },
	{ 0xa055, 0xa055, PDF_CMAP_SINGLE, 18728 },
	{ 0xa056, 0xa062, PDF_CMAP_RANGE, 16291 },
	{ 0xa063, 0xa063, PDF_CMAP_SINGLE, 14353 },
	{ 0xa064, 0xa06c, PDF_CMAP_RANGE, 16305 },
	{ 0xa06d, 0xa06d, PDF_CMAP_SINGLE, 18729 },
	{ 0xa06e, 0xa076, PDF_CMAP_RANGE, 16314 },
	{ 0xa077, 0xa077, PDF_CMAP_SINGLE, 1522 },
	{ 0xa078, 0xa07a, PDF_CMAP_RANGE, 16324 },
	{ 0xa07b, 0xa07b, PDF_CMAP_SINGLE, 18730 },
	{ 0xa07c, 0xa07e, PDF_CMAP_RANGE, 16327 },
	{ 0xa0a1, 0xa0a2, PDF_CMAP_TABLE, 293 },
	{ 0xa0a3, 0xa0a6, PDF_CMAP_RANGE, 16331 },
	{ 0xa0a7, 0xa0a7, PDF_CMAP_SINGLE, 18732 },
	{ 0xa0a8, 0xa0c4, PDF_CMAP_RANGE, 16335 },
	{ 0xa0c5, 0xa0c5, PDF_CMAP_SINGLE, 18733 },
	{ 0xa0c6, 0xa0cf, PDF_CMAP_RANGE, 16364 },
	{ 0xa0d0, 0xa0d0, PDF_CMAP_SINGLE, 18734 },
	{ 0xa0d1, 0xa0d4, PDF_CMAP_RANGE, 16374 },
	{ 0xa0d5, 0xa0d5, PDF_CMAP_SINGLE, 15086 },
	{ 0xa0d6, 0xa0de, PDF_CMAP_RANGE, 16379 },
	{ 0xa0df, 0xa0df, PDF_CMAP_SINGLE, 9341 },
	{ 0xa0e0, 0xa0e2, PDF_CMAP_RANGE, 16389 },
	{ 0xa0e3, 0xa0e4, PDF_CMAP_TABLE, 295 },
	{ 0xa0e5, 0xa0e6, PDF_CMAP_RANGE, 16393 },
	{ 0xa0e7, 0xa0ed, PDF_CMAP_RANGE, 18736 },
	{ 0xa0ee, 0xa0ee, PDF_CMAP_SINGLE, 17332 },
	{ 0xa0ef, 0xa0f1, PDF_CMAP_RANGE, 18743 },
	{ 0xa0f2, 0xa0f2, PDF_CMAP_SINGLE, 17336 },
	{ 0xa0f3, 0xa0fe, PDF_CMAP_RANGE, 18746 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 297 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 299 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc6a1, 0xc6be, PDF_CMAP_RANGE, 506 },
	{ 0xc6bf, 0xc6d7, PDF_CMAP_RANGE, 537 },
	{ 0xc6d8, 0xc6de, PDF_CMAP_RANGE, 13747 },
	{ 0xc6df, 0xc6df, PDF_CMAP_SINGLE, 6036 },
	{ 0xc6e0, 0xc6fe, PDF_CMAP_RANGE, 13754 },
	{ 0xc740, 0xc77e, PDF_CMAP_RANGE, 13785 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 13848 },
	{ 0xc840, 0xc87e, PDF_CMAP_RANGE, 13942 },
	{ 0xc8a1, 0xc8d3, PDF_CMAP_RANGE, 14005 },
	{ 0xc8d4, 0xc8d6, PDF_CMAP_RANGE, 17606 },
	{ 0xc8d7, 0xc8df, PDF_CMAP_RANGE, 17692 },
	{ 0xc8e0, 0xc8e0, PDF_CMAP_SINGLE, 18846 },
	{ 0xc8e1, 0xc8e8, PDF_CMAP_RANGE, 17701 },
	{ 0xc8e9, 0xc8e9, PDF_CMAP_SINGLE, 18847 },
	{ 0xc8ea, 0xc8f0, PDF_CMAP_RANGE, 17709 },
	{ 0xc8f1, 0xc8f1, PDF_CMAP_SINGLE, 18848 },
	{ 0xc8f5, 0xc8fe, PDF_CMAP_RANGE, 18834 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 301 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9d6, 0xf9fe, PDF_CMAP_RANGE, 14056 },
	{ 0xfa40, 0xfa5e, PDF_CMAP_RANGE, 16395 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 2106 },
	{ 0xfa60, 0xfa65, PDF_CMAP_RANGE, 16427 },
	{ 0xfa66, 0xfa66, PDF_CMAP_SINGLE, 2557 },
	{ 0xfa67, 0xfa7e, PDF_CMAP_RANGE, 16434 },
	{ 0xfaa1, 0xfaa8, PDF_CMAP_RANGE, 16458 },
	{ 0xfaa9, 0xfaaa, PDF_CMAP_RANGE, 18758 },
	{ 0xfaab, 0xfabc, PDF_CMAP_RANGE, 16468 },
	{ 0xfabd, 0xfabd, PDF_CMAP_SINGLE, 781 },
	{ 0xfabe, 0xfac4, PDF_CMAP_RANGE, 16487 },
	{ 0xfac5, 0xfac5, PDF_CMAP_SINGLE, 363 },
	{ 0xfac6, 0xfad4, PDF_CMAP_RANGE, 16495 },
	{ 0xfad5, 0xfad5, PDF_CMAP_SINGLE, 2144 },
	{ 0xfad6, 0xfafe, PDF_CMAP_RANGE, 16511 },
	{ 0xfb40, 0xfb47, PDF_CMAP_RANGE, 16552 },
	{ 0xfb48, 0xfb48, PDF_CMAP_SINGLE, 16002 },
	{ 0xfb49, 0xfb52, PDF_CMAP_RANGE, 16561 },
	{ 0xfb53, 0xfb53, PDF_CMAP_SINGLE, 18760 },
	{ 0xfb54, 0xfb6d, PDF_CMAP_RANGE, 16572 },
	{ 0xfb6e, 0xfb6e, PDF_CMAP_SINGLE, 18761 },
	{ 0xfb6f, 0xfb7e, PDF_CMAP_RANGE, 16599 },
	{ 0xfba1, 0xfba2, PDF_CMAP_RANGE, 16615 },
	{ 0xfba3, 0xfba3, PDF_CMAP_SINGLE, 18762 },
	{ 0xfba4, 0xfbb7, PDF_CMAP_RANGE, 16618 },
	{ 0xfbb8, 0xfbb8, PDF_CMAP_SINGLE, 3107 },
	{ 0xfbb9, 0xfbbe, PDF_CMAP_RANGE, 16639 },
	{ 0xfbbf, 0xfbbf, PDF_CMAP_SINGLE, 18763 },
	{ 0xfbc0, 0xfbcc, PDF_CMAP_RANGE, 16645 },
	{ 0xfbcd, 0xfbcd, PDF_CMAP_SINGLE, 18764 },
	{ 0xfbce, 0xfbf2, PDF_CMAP_RANGE, 16658 },
	{ 0xfbf3, 0xfbf3, PDF_CMAP_SINGLE, 6116 },
	{ 0xfbf4, 0xfbf8, PDF_CMAP_RANGE, 16696 },
	{ 0xfbf9, 0xfbf9, PDF_CMAP_SINGLE, 16014 },
	{ 0xfbfa, 0xfbfe, PDF_CMAP_RANGE, 16702 },
	{ 0xfc40, 0xfc49, PDF_CMAP_RANGE, 16707 },
	{ 0xfc4a, 0xfc4a, PDF_CMAP_SINGLE, 18765 },
	{ 0xfc4b, 0xfc4e, PDF_CMAP_RANGE, 16717 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 8495 },
	{ 0xfc50, 0xfc51, PDF_CMAP_RANGE, 16721 },
	{ 0xfc52, 0xfc52, PDF_CMAP_SINGLE, 18766 },
	{ 0xfc53, 0xfc62, PDF_CMAP_RANGE, 16723 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 18767 },
	{ 0xfc64, 0xfc6b, PDF_CMAP_RANGE, 16739 },
	{ 0xfc6c, 0xfc6d, PDF_CMAP_TABLE, 304 },
	{ 0xfc6e, 0xfc74, PDF_CMAP_RANGE, 16749 },
	{ 0xfc75, 0xfc75, PDF_CMAP_SINGLE, 18769 },
	{ 0xfc76, 0xfc7e, PDF_CMAP_RANGE, 16756 },
	{ 0xfca1, 0xfcb8, PDF_CMAP_RANGE, 16765 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 4447 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 16789 },
	{ 0xfcbc, 0xfcbd, PDF_CMAP_RANGE, 18770 },
	{ 0xfcbe, 0xfccb, PDF_CMAP_RANGE, 16792 },
	{ 0xfccc, 0xfccc, PDF_CMAP_SINGLE, 18772 },
	{ 0xfccd, 0xfce1, PDF_CMAP_RANGE, 16807 },
	{ 0xfce2, 0xfce3, PDF_CMAP_TABLE, 306 },
	{ 0xfce4, 0xfced, PDF_CMAP_RANGE, 16829 },
	{ 0xfcee, 0xfcee, PDF_CMAP_SINGLE, 18774 },
	{ 0xfcef, 0xfcf0, PDF_CMAP_RANGE, 16839 },
	{ 0xfcf1, 0xfcf1, PDF_CMAP_SINGLE, 1219 },
	{ 0xfcf2, 0xfcfe, PDF_CMAP_RANGE, 16842 },
	{ 0xfd40, 0xfd48, PDF_CMAP_RANGE, 16855 },
	{ 0xfd49, 0xfd49, PDF_CMAP_SINGLE, 18775 },
	{ 0xfd4a, 0xfd69, PDF_CMAP_RANGE, 16864 },
	{ 0xfd6a, 0xfd6a, PDF_CMAP_SINGLE, 18776 },
	{ 0xfd6b, 0xfd7e, PDF_CMAP_RANGE, 16897 },
	{ 0xfda1, 0xfdb6, PDF_CMAP_RANGE, 16917 },
	{ 0xfdb7, 0xfdb8, PDF_CMAP_TABLE, 308 },
	{ 0xfdb9, 0xfdba, PDF_CMAP_RANGE, 16941 },
	{ 0xfdbb, 0xfdbb, PDF_CMAP_SINGLE, 3237 },
	{ 0xfdbc, 0xfde2, PDF_CMAP_RANGE, 16944 },
	{ 0xfde3, 0xfde3, PDF_CMAP_SINGLE, 18777 },
	{ 0xfde4, 0xfdf0, PDF_CMAP_RANGE, 16984 },
	{ 0xfdf1, 0xfdf2, PDF_CMAP_TABLE, 310 },
	{ 0xfdf3, 0xfdfe, PDF_CMAP_RANGE, 16998 },
	{ 0xfe40, 0xfe51, PDF_CMAP_RANGE, 17010 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 15728 },
	{ 0xfe53, 0xfe6c, PDF_CMAP_RANGE, 17029 },
	{ 0xfe6d, 0xfe6f, PDF_CMAP_TABLE, 312 },
	{ 0xfe70, 0xfe77, PDF_CMAP_RANGE, 17057 },
	{ 0xfe78, 0xfe78, PDF_CMAP_SINGLE, 18780 },
	{ 0xfe79, 0xfe7e, PDF_CMAP_RANGE, 17065 },
	{ 0xfea1, 0xfea9, PDF_CMAP_RANGE, 17071 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 288 },
	{ 0xfeab, 0xfedc, PDF_CMAP_RANGE, 17080 },
	{ 0xfedd, 0xfedd, PDF_CMAP_SINGLE, 7080 },
	{ 0xfede, 0xfedf, PDF_CMAP_RANGE, 18781 },
	{ 0xfee0, 0xfeec, PDF_CMAP_RANGE, 17131 },
	{ 0xfeed, 0xfeee, PDF_CMAP_RANGE, 18783 },
	{ 0xfeef, 0xfefe, PDF_CMAP_RANGE, 17144 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c4, PDF_CMAP_SINGLE, 13585 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 13629 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9d6, 0xf9fe, PDF_CMAP_RANGE, 14056 },
	{ 0xfa40, 0xfa5e, PDF_CMAP_RANGE, 16395 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 2106 },
	{ 0xfa60, 0xfa65, PDF_CMAP_RANGE, 16427 },
	{ 0xfa66, 0xfa66, PDF_CMAP_SINGLE, 2557 },
	{ 0xfa67, 0xfa7e, PDF_CMAP_RANGE, 16434 },
	{ 0xfaa1, 0xfaa8, PDF_CMAP_RANGE, 16458 },
	{ 0xfaa9, 0xfaaa, PDF_CMAP_RANGE, 18758 },
	{ 0xfaab, 0xfabc, PDF_CMAP_RANGE, 16468 },
	{ 0xfabd, 0xfabd, PDF_CMAP_SINGLE, 781 },
	{ 0xfabe, 0xfac4, PDF_CMAP_RANGE, 16487 },
	{ 0xfac5, 0xfac5, PDF_CMAP_SINGLE, 363 },
	{ 0xfac6, 0xfad4, PDF_CMAP_RANGE, 16495 },
	{ 0xfad5, 0xfad5, PDF_CMAP_SINGLE, 2144 },
	{ 0xfad6, 0xfafe, PDF_CMAP_RANGE, 16511 },
	{ 0xfb40, 0xfb47, PDF_CMAP_RANGE, 16552 },
	{ 0xfb48, 0xfb48, PDF_CMAP_SINGLE, 16002 },
	{ 0xfb49, 0xfb52, PDF_CMAP_RANGE, 16561 },
	{ 0xfb53, 0xfb53, PDF_CMAP_SINGLE, 18760 },
	{ 0xfb54, 0xfb6d, PDF_CMAP_RANGE, 16572 },
	{ 0xfb6e, 0xfb6e, PDF_CMAP_SINGLE, 18761 },
	{ 0xfb6f, 0xfb7e, PDF_CMAP_RANGE, 16599 },
	{ 0xfba1, 0xfba2, PDF_CMAP_RANGE, 16615 },
	{ 0xfba3, 0xfba3, PDF_CMAP_SINGLE, 18762 },
	{ 0xfba4, 0xfbb7, PDF_CMAP_RANGE, 16618 },
	{ 0xfbb8, 0xfbb8, PDF_CMAP_SINGLE, 3107 },
	{ 0xfbb9, 0xfbbe, PDF_CMAP_RANGE, 16639 },
	{ 0xfbbf, 0xfbbf, PDF_CMAP_SINGLE, 18763 },
	{ 0xfbc0, 0xfbcc, PDF_CMAP_RANGE, 16645 },
	{ 0xfbcd, 0xfbcd, PDF_CMAP_SINGLE, 18764 },
	{ 0xfbce, 0xfbf2, PDF_CMAP_RANGE, 16658 },
	{ 0xfbf3, 0xfbf3, PDF_CMAP_SINGLE, 6116 },
	{ 0xfbf4, 0xfbf8, PDF_CMAP_RANGE, 16696 },
	{ 0xfbf9, 0xfbf9, PDF_CMAP_SINGLE, 16014 },
	{ 0xfbfa, 0xfbfe, PDF_CMAP_RANGE, 16702 },
	{ 0xfc40, 0xfc49, PDF_CMAP_RANGE, 16707 },
	{ 0xfc4a, 0xfc4a, PDF_CMAP_SINGLE, 18765 },
	{ 0xfc4b, 0xfc4e, PDF_CMAP_RANGE, 16717 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 8495 },
	{ 0xfc50, 0xfc51, PDF_CMAP_RANGE, 16721 },
	{ 0xfc52, 0xfc52, PDF_CMAP_SINGLE, 18766 },
	{ 0xfc53, 0xfc62, PDF_CMAP_RANGE, 16723 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 18767 },
	{ 0xfc64, 0xfc6b, PDF_CMAP_RANGE, 16739 },
	{ 0xfc6c, 0xfc6c, PDF_CMAP_SINGLE, 16385 },
	{ 0xfc6d, 0xfc6d, PDF_CMAP_SINGLE, 18768 },
	{ 0xfc6e, 0xfc74, PDF_CMAP_RANGE, 16749 },
	{ 0xfc75, 0xfc75, PDF_CMAP_SINGLE, 18769 },
	{ 0xfc76, 0xfc7e, PDF_CMAP_RANGE, 16756 },
	{ 0xfca1, 0xfcb8, PDF_CMAP_RANGE, 16765 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 4447 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 16789 },
	{ 0xfcbc, 0xfcbd, PDF_CMAP_RANGE, 18770 },
	{ 0xfcbe, 0xfccb, PDF_CMAP_RANGE, 16792 },
	{ 0xfccc, 0xfccc, PDF_CMAP_SINGLE, 18772 },
	{ 0xfccd, 0xfce1, PDF_CMAP_RANGE, 16807 },
	{ 0xfce2, 0xfce2, PDF_CMAP_SINGLE, 3193 },
	{ 0xfce3, 0xfce3, PDF_CMAP_SINGLE, 18773 },
	{ 0xfce4, 0xfced, PDF_CMAP_RANGE, 16829 },
	{ 0xfcee, 0xfcee, PDF_CMAP_SINGLE, 18774 },
	{ 0xfcef, 0xfcf0, PDF_CMAP_RANGE, 16839 },
	{ 0xfcf1, 0xfcf1, PDF_CMAP_SINGLE, 1219 },
	{ 0xfcf2, 0xfcfe, PDF_CMAP_RANGE, 16842 },
	{ 0xfd40, 0xfd48, PDF_CMAP_RANGE, 16855 },
	{ 0xfd49, 0xfd49, PDF_CMAP_SINGLE, 18775 },
	{ 0xfd4a, 0xfd69, PDF_CMAP_RANGE, 16864 },
	{ 0xfd6a, 0xfd6a, PDF_CMAP_SINGLE, 18776 },
	{ 0xfd6b, 0xfd7e, PDF_CMAP_RANGE, 16897 },
	{ 0xfda1, 0xfdb6, PDF_CMAP_RANGE, 16917 },
	{ 0xfdb7, 0xfdb7, PDF_CMAP_SINGLE, 6333 },
	{ 0xfdb8, 0xfdb8, PDF_CMAP_SINGLE, 3261 },
	{ 0xfdb9, 0xfdba, PDF_CMAP_RANGE, 16941 },
	{ 0xfdbb, 0xfdbb, PDF_CMAP_SINGLE, 3237 },
	{ 0xfdbc, 0xfde2, PDF_CMAP_RANGE, 16944 },
	{ 0xfde3, 0xfde3, PDF_CMAP_SINGLE, 18777 },
	{ 0xfde4, 0xfdf0, PDF_CMAP_RANGE, 16984 },
	{ 0xfdf1, 0xfdf1, PDF_CMAP_SINGLE, 3278 },
	{ 0xfdf2, 0xfdf2, PDF_CMAP_SINGLE, 18778 },
	{ 0xfdf3, 0xfdfe, PDF_CMAP_RANGE, 16998 },
	{ 0xfe40, 0xfe51, PDF_CMAP_RANGE, 17010 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 15728 },
	{ 0xfe53, 0xfe6c, PDF_CMAP_RANGE, 17029 },
	{ 0xfe6d, 0xfe6d, PDF_CMAP_SINGLE, 18779 },
	{ 0xfe6e, 0xfe6e, PDF_CMAP_SINGLE, 17055 },
	{ 0xfe6f, 0xfe6f, PDF_CMAP_SINGLE, 3716 },
	{ 0xfe70, 0xfe77, PDF_CMAP_RANGE, 17057 },
	{ 0xfe78, 0xfe78, PDF_CMAP_SINGLE, 18780 },
	{ 0xfe79, 0xfe7e, PDF_CMAP_RANGE, 17065 },
	{ 0xfea1, 0xfea9, PDF_CMAP_RANGE, 17071 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 288 },
	{ 0xfeab, 0xfedc, PDF_CMAP_RANGE, 17080 },
	{ 0xfedd, 0xfedd, PDF_CMAP_SINGLE, 7080 },
	{ 0xfede, 0xfedf, PDF_CMAP_RANGE, 18781 },
	{ 0xfee0, 0xfeec, PDF_CMAP_RANGE, 17131 },
	{ 0xfeed, 0xfeee, PDF_CMAP_RANGE, 18783 },
	{ 0xfeef, 0xfefe, PDF_CMAP_RANGE, 17144 },
};

static const unsigned short pdf_cmap_ETHK_B5_H_table[315] =
{
	17723,17347,17388,17777,17387,17778,17308,17815,
	17310,17816,17311,17313,17820,17315,17845,17161,
	17214,17850,17851,17219,17852,17221,17853,17855,
	17245,17856,17247,17860,17371,17294,17886,17295,
	17887,17296,17888,17297,17889,17303,17896,17304,
	17348,19023,18964,17339,17312,17341,17344,17981,
	17345,17337,17325,4181,14164,3381,14296,18083,
	18085,14309,10178,18088,14368,18089,18090,4940,
	16377,18093,14420,2510,3015,14466,18098,14633,
	18104,284,283,18116,7430,10657,18118,17354,
	18154,15709,18155,17356,15710,18156,17402,18157,
	15713,18161,15714,18162,18178,17361,15720,18195,
	15721,18196,15722,18203,17367,17404,15723,18206,
	15724,18207,17368,18255,15730,18256,17349,18257,
	17350,15734,18308,17323,18309,17383,18310,17385,
	15736,18341,15737,17327,18359,15739,17389,18364,
	15740,17390,18365,17407,15741,18373,17393,17395,
	18384,17394,17400,17396,15744,18399,15745,18400,
	15746,18401,17335,15748,18419,15749,15753,18432,
	15754,15755,17328,15763,18474,15764,18475,15765,
	18476,12112,15773,5925,13037,15776,18496,15777,
	15778,18497,18500,15785,15795,18509,15803,18513,
	15804,18514,17360,18515,18516,15807,18517,15808,
	15812,7188,15813,18522,15814,18523,15815,15816,
	18527,16890,18528,4841,18534,11438,18538,18539,
	15835,18540,15836,6756,18543,15844,18544,18545,
	15850,15851,18548,5124,18549,18550,8996,18551,
	15857,13418,18552,15858,18553,8849,18556,15866,
	18557,18558,15869,18563,15875,18564,18567,6023,
	2399,18571,15903,7833,15938,18583,18593,15944,
	18594,15945,15946,18598,9665,17353,15948,18607,
	15949,17397,15950,18635,15956,18636,15961,18641,
	15962,18642,15963,15968,1510,18652,15972,16054,
	18666,16055,16056,18673,16057,18677,16060,18678,
	16061,18679,16062,1832,16069,18683,16075,18687,
	18688,16078,18701,16092,18710,16124,6150,16126,
	18720,16212,18721,3970,18722,16330,18731,18735,
	15049,248,247,5551,5550,13585,13629,13641,
	16385,18768,3193,18773,6333,3261,3278,18778,
	18779,17055,3716,
};

pdf_cmap pdf_cmap_ETHK_B5_H =
{
	-1, "ETHK-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8740, 0xfefe },
	},
	1209, 1209, (pdf_range*) pdf_cmap_ETHK_B5_H_ranges,
	315, 315, (unsigned short*) pdf_cmap_ETHK_B5_H_table,
};

/* ETHK-B5-V */

static const pdf_range pdf_cmap_ETHK_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
	{ 0xc6e4, 0xc6e5, PDF_CMAP_RANGE, 14097 },
};

static const unsigned short pdf_cmap_ETHK_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_ETHK_B5_V =
{
	-1, "ETHK-B5-V", "ETHK-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	13, 13, (pdf_range*) pdf_cmap_ETHK_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_ETHK_B5_V_table,
};

/* HKdla-B5-H */

static const pdf_range pdf_cmap_HKdla_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 0 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 2 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 4 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xfa41, 0xfa7e, PDF_CMAP_TABLE, 7 },
	{ 0xfaa1, 0xfab2, PDF_CMAP_TABLE, 69 },
	{ 0xfab3, 0xfab4, PDF_CMAP_RANGE, 16001 },
	{ 0xfab5, 0xfafe, PDF_CMAP_TABLE, 87 },
	{ 0xfb40, 0xfb75, PDF_CMAP_TABLE, 161 },
	{ 0xfb76, 0xfb77, PDF_CMAP_RANGE, 16228 },
	{ 0xfb78, 0xfb7e, PDF_CMAP_TABLE, 215 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_TABLE, 222 },
	{ 0xfc40, 0xfc7e, PDF_CMAP_TABLE, 316 },
	{ 0xfca1, 0xfcb5, PDF_CMAP_TABLE, 379 },
	{ 0xfcb6, 0xfcb7, PDF_CMAP_RANGE, 16862 },
	{ 0xfcb8, 0xfcd3, PDF_CMAP_TABLE, 400 },
	{ 0xfcd4, 0xfcd5, PDF_CMAP_RANGE, 16960 },
	{ 0xfcd6, 0xfcfe, PDF_CMAP_TABLE, 428 },
	{ 0xfd40, 0xfd7e, PDF_CMAP_TABLE, 469 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_TABLE, 532 },
	{ 0xfe40, 0xfe67, PDF_CMAP_TABLE, 626 },
	{ 0xfe68, 0xfe69, PDF_CMAP_RANGE, 15723 },
	{ 0xfe6a, 0xfe7e, PDF_CMAP_TABLE, 666 },
	{ 0xfea1, 0xfeb4, PDF_CMAP_TABLE, 687 },
	{ 0xfeb5, 0xfeb6, PDF_CMAP_RANGE, 14566 },
	{ 0xfeb7, 0xfefe, PDF_CMAP_TABLE, 707 },
	{ 0xfa53, 0xfa53, PDF_CMAP_SINGLE, 16523 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 15974 },
	{ 0xfa55, 0xfa55, PDF_CMAP_SINGLE, 15973 },
	{ 0xfa56, 0xfa56, PDF_CMAP_SINGLE, 15759 },
	{ 0xfa57, 0xfa57, PDF_CMAP_SINGLE, 16798 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 14950 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 16737 },
	{ 0xfa5a, 0xfa5a, PDF_CMAP_SINGLE, 16010 },
	{ 0xfa5b, 0xfa5b, PDF_CMAP_SINGLE, 15758 },
	{ 0xfa5c, 0xfa5c, PDF_CMAP_SINGLE, 16013 },
	{ 0xfa5d, 0xfa5d, PDF_CMAP_SINGLE, 15804 },
	{ 0xfa5e, 0xfa5e, PDF_CMAP_SINGLE, 16021 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 14755 },
	{ 0xfa60, 0xfa60, PDF_CMAP_SINGLE, 16884 },
	{ 0xfa61, 0xfa61, PDF_CMAP_SINGLE, 16017 },
	{ 0xfa62, 0xfa62, PDF_CMAP_SINGLE, 15302 },
	{ 0xfa63, 0xfa63, PDF_CMAP_SINGLE, 15811 },
	{ 0xfa64, 0xfa64, PDF_CMAP_SINGLE, 16046 },
	{ 0xfa65, 0xfa65, PDF_CMAP_SINGLE, 14150 },
	{ 0xfa66, 0xfa66, PDF_CMAP_SINGLE, 14449 },
	{ 0xfa67, 0xfa67, PDF_CMAP_SINGLE, 14848 },
	{ 0xfa68, 0xfa68, PDF_CMAP_SINGLE, 17036 },
	{ 0xfa69, 0xfa69, PDF_CMAP_SINGLE, 16987 },
	{ 0xfa6a, 0xfa6a, PDF_CMAP_SINGLE, 16045 },
	{ 0xfa6b, 0xfa6b, PDF_CMAP_SINGLE, 16675 },
	{ 0xfa6c, 0xfa6c, PDF_CMAP_SINGLE, 16038 },
	{ 0xfa6d, 0xfa6d, PDF_CMAP_SINGLE, 16032 },
	{ 0xfa6e, 0xfa6e, PDF_CMAP_SINGLE, 15150 },
	{ 0xfa6f, 0xfa6f, PDF_CMAP_SINGLE, 15805 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 14436 },
	{ 0xfa71, 0xfa71, PDF_CMAP_SINGLE, 15485 },
	{ 0xfa72, 0xfa72, PDF_CMAP_SINGLE, 16041 },
	{ 0xfa73, 0xfa73, PDF_CMAP_SINGLE, 16909 },
	{ 0xfa74, 0xfa74, PDF_CMAP_SINGLE, 16216 },
	{ 0xfa75, 0xfa75, PDF_CMAP_SINGLE, 16024 },
	{ 0xfa76, 0xfa76, PDF_CMAP_SINGLE, 15728 },
	{ 0xfa77, 0xfa77, PDF_CMAP_SINGLE, 16040 },
	{ 0xfa78, 0xfa78, PDF_CMAP_SINGLE, 16020 },
	{ 0xfa79, 0xfa79, PDF_CMAP_SINGLE, 16029 },
	{ 0xfa7a, 0xfa7a, PDF_CMAP_SINGLE, 15018 },
	{ 0xfa7b, 0xfa7b, PDF_CMAP_SINGLE, 16037 },
	{ 0xfa7c, 0xfa7c, PDF_CMAP_SINGLE, 15564 },
	{ 0xfa7d, 0xfa7d, PDF_CMAP_SINGLE, 16536 },
	{ 0xfa7e, 0xfa7e, PDF_CMAP_SINGLE, 16031 },
	{ 0xfaa1, 0xfaa1, PDF_CMAP_SINGLE, 16885 },
	{ 0xfaa2, 0xfaa2, PDF_CMAP_SINGLE, 16011 },
	{ 0xfaa3, 0xfaa3, PDF_CMAP_SINGLE, 15218 },
	{ 0xfaa4, 0xfaa4, PDF_CMAP_SINGLE, 14304 },
	{ 0xfaa5, 0xfaa5, PDF_CMAP_SINGLE, 15071 },
	{ 0xfaa6, 0xfaa6, PDF_CMAP_SINGLE, 17120 },
	{ 0xfaa7, 0xfaa7, PDF_CMAP_SINGLE, 16023 },
	{ 0xfaa8, 0xfaa8, PDF_CMAP_SINGLE, 16786 },
	{ 0xfaa9, 0xfaa9, PDF_CMAP_SINGLE, 14472 },
	{ 0xfaaa, 0xfaaa, PDF_CMAP_SINGLE, 17096 },
	{ 0xfaab, 0xfaab, PDF_CMAP_SINGLE, 16016 },
	{ 0xfaac, 0xfaac, PDF_CMAP_SINGLE, 14447 },
	{ 0xfaad, 0xfaad, PDF_CMAP_SINGLE, 16028 },
	{ 0xfaae, 0xfaae, PDF_CMAP_SINGLE, 16708 },
	{ 0xfaaf, 0xfaaf, PDF_CMAP_SINGLE, 16710 },
	{ 0xfab0, 0xfab0, PDF_CMAP_SINGLE, 16030 },
	{ 0xfab1, 0xfab1, PDF_CMAP_SINGLE, 16009 },
	{ 0xfab2, 0xfab2, PDF_CMAP_SINGLE, 16531 },
	{ 0xfab3, 0xfab4, PDF_CMAP_RANGE, 16001 },
	{ 0xfab5, 0xfab5, PDF_CMAP_SINGLE, 15374 },
	{ 0xfab6, 0xfab6, PDF_CMAP_SINGLE, 16005 },
	{ 0xfab7, 0xfab7, PDF_CMAP_SINGLE, 14556 },
	{ 0xfab8, 0xfab8, PDF_CMAP_SINGLE, 16489 },
	{ 0xfab9, 0xfab9, PDF_CMAP_SINGLE, 14296 },
	{ 0xfaba, 0xfaba, PDF_CMAP_SINGLE, 16025 },
	{ 0xfabb, 0xfabb, PDF_CMAP_SINGLE, 16003 },
	{ 0xfabc, 0xfabc, PDF_CMAP_SINGLE, 16008 },
	{ 0xfabd, 0xfabd, PDF_CMAP_SINGLE, 16000 },
	{ 0xfabe, 0xfabe, PDF_CMAP_SINGLE, 16049 },
	{ 0xfabf, 0xfabf, PDF_CMAP_SINGLE, 16746 },
	{ 0xfac0, 0xfac0, PDF_CMAP_SINGLE, 14187 },
	{ 0xfac1, 0xfac1, PDF_CMAP_SINGLE, 15975 },
	{ 0xfac2, 0xfac2, PDF_CMAP_SINGLE, 15992 },
	{ 0xfac3, 0xfac3, PDF_CMAP_SINGLE, 16994 },
	{ 0xfac4, 0xfac4, PDF_CMAP_SINGLE, 14325 },
	{ 0xfac5, 0xfac5, PDF_CMAP_SINGLE, 14324 },
	{ 0xfac6, 0xfac6, PDF_CMAP_SINGLE, 15173 },
	{ 0xfac7, 0xfac7, PDF_CMAP_SINGLE, 15961 },
	{ 0xfac8, 0xfac8, PDF_CMAP_SINGLE, 10940 },
	{ 0xfac9, 0xfac9, PDF_CMAP_SINGLE, 15807 },
	{ 0xfaca, 0xfaca, PDF_CMAP_SINGLE, 16573 },
	{ 0xfacb, 0xfacb, PDF_CMAP_SINGLE, 15998 },
	{ 0xfacc, 0xfacc, PDF_CMAP_SINGLE, 14381 },
	{ 0xfacd, 0xfacd, PDF_CMAP_SINGLE, 16048 },
	{ 0xface, 0xface, PDF_CMAP_SINGLE, 16043 },
	{ 0xfacf, 0xfacf, PDF_CMAP_SINGLE, 16050 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 16929 },
	{ 0xfad1, 0xfad1, PDF_CMAP_SINGLE, 16234 },
	{ 0xfad2, 0xfad2, PDF_CMAP_SINGLE, 14363 },
	{ 0xfad3, 0xfad3, PDF_CMAP_SINGLE, 16026 },
	{ 0xfad4, 0xfad4, PDF_CMAP_SINGLE, 16931 },
	{ 0xfad5, 0xfad5, PDF_CMAP_SINGLE, 16014 },
	{ 0xfad6, 0xfad6, PDF_CMAP_SINGLE, 15357 },
	{ 0xfad7, 0xfad7, PDF_CMAP_SINGLE, 16474 },
	{ 0xfad8, 0xfad8, PDF_CMAP_SINGLE, 16047 },
	{ 0xfad9, 0xfad9, PDF_CMAP_SINGLE, 15560 },
	{ 0xfada, 0xfada, PDF_CMAP_SINGLE, 14664 },
	{ 0xfadb, 0xfadb, PDF_CMAP_SINGLE, 17085 },
	{ 0xfadc, 0xfadc, PDF_CMAP_SINGLE, 16588 },
	{ 0xfadd, 0xfadd, PDF_CMAP_SINGLE, 15964 },
	{ 0xfade, 0xfade, PDF_CMAP_SINGLE, 16044 },
	{ 0xfadf, 0xfadf, PDF_CMAP_SINGLE, 14618 },
	{ 0xfae0, 0xfae0, PDF_CMAP_SINGLE, 16042 },
	{ 0xfae1, 0xfae1, PDF_CMAP_SINGLE, 17305 },
	{ 0xfae2, 0xfae2, PDF_CMAP_SINGLE, 14490 },
	{ 0xfae3, 0xfae3, PDF_CMAP_SINGLE, 16012 },
	{ 0xfae4, 0xfae4, PDF_CMAP_SINGLE, 14495 },
	{ 0xfae5, 0xfae5, PDF_CMAP_SINGLE, 16033 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 16775 },
	{ 0xfae7, 0xfae7, PDF_CMAP_SINGLE, 16015 },
	{ 0xfae8, 0xfae8, PDF_CMAP_SINGLE, 16323 },
	{ 0xfae9, 0xfae9, PDF_CMAP_SINGLE, 16034 },
	{ 0xfaea, 0xfaea, PDF_CMAP_SINGLE, 16027 },
	{ 0xfaeb, 0xfaeb, PDF_CMAP_SINGLE, 15997 },
	{ 0xfaec, 0xfaec, PDF_CMAP_SINGLE, 16465 },
	{ 0xfaed, 0xfaed, PDF_CMAP_SINGLE, 16316 },
	{ 0xfaee, 0xfaee, PDF_CMAP_SINGLE, 16341 },
	{ 0xfaef, 0xfaef, PDF_CMAP_SINGLE, 15124 },
	{ 0xfaf0, 0xfaf0, PDF_CMAP_SINGLE, 16930 },
	{ 0xfaf1, 0xfaf1, PDF_CMAP_SINGLE, 14235 },
	{ 0xfaf2, 0xfaf2, PDF_CMAP_SINGLE, 16507 },
	{ 0xfaf3, 0xfaf3, PDF_CMAP_SINGLE, 14216 },
	{ 0xfaf4, 0xfaf4, PDF_CMAP_SINGLE, 15284 },
	{ 0xfaf5, 0xfaf5, PDF_CMAP_SINGLE, 16848 },
	{ 0xfaf6, 0xfaf6, PDF_CMAP_SINGLE, 16996 },
	{ 0xfaf7, 0xfaf7, PDF_CMAP_SINGLE, 14430 },
	{ 0xfaf8, 0xfaf8, PDF_CMAP_SINGLE, 14188 },
	{ 0xfaf9, 0xfaf9, PDF_CMAP_SINGLE, 15806 },
	{ 0xfafa, 0xfafa, PDF_CMAP_SINGLE, 14355 },
	{ 0xfafb, 0xfafb, PDF_CMAP_SINGLE, 16383 },
	{ 0xfafc, 0xfafc, PDF_CMAP_SINGLE, 15715 },
	{ 0xfafd, 0xfafd, PDF_CMAP_SINGLE, 14214 },
	{ 0xfafe, 0xfafe, PDF_CMAP_SINGLE, 15178 },
	{ 0xfb40, 0xfb40, PDF_CMAP_SINGLE, 14138 },
	{ 0xfb41, 0xfb41, PDF_CMAP_SINGLE, 17018 },
	{ 0xfb42, 0xfb42, PDF_CMAP_SINGLE, 15482 },
	{ 0xfb43, 0xfb43, PDF_CMAP_SINGLE, 16901 },
	{ 0xfb44, 0xfb44, PDF_CMAP_SINGLE, 15398 },
	{ 0xfb45, 0xfb45, PDF_CMAP_SINGLE, 14511 },
	{ 0xfb46, 0xfb46, PDF_CMAP_SINGLE, 14646 },
	{ 0xfb47, 0xfb47, PDF_CMAP_SINGLE, 14248 },
	{ 0xfb48, 0xfb48, PDF_CMAP_SINGLE, 15220 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 16237 },
	{ 0xfb4a, 0xfb4a, PDF_CMAP_SINGLE, 16022 },
	{ 0xfb4b, 0xfb4b, PDF_CMAP_SINGLE, 14268 },
	{ 0xfb4c, 0xfb4c, PDF_CMAP_SINGLE, 15995 },
	{ 0xfb4d, 0xfb4d, PDF_CMAP_SINGLE, 16362 },
	{ 0xfb4e, 0xfb4e, PDF_CMAP_SINGLE, 16842 },
	{ 0xfb4f, 0xfb4f, PDF_CMAP_SINGLE, 14807 },
	{ 0xfb50, 0xfb50, PDF_CMAP_SINGLE, 16593 },
	{ 0xfb51, 0xfb51, PDF_CMAP_SINGLE, 15217 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 16335 },
	{ 0xfb53, 0xfb53, PDF_CMAP_SINGLE, 1986 },
	{ 0xfb54, 0xfb54, PDF_CMAP_SINGLE, 17150 },
	{ 0xfb55, 0xfb55, PDF_CMAP_SINGLE, 15565 },
	{ 0xfb56, 0xfb56, PDF_CMAP_SINGLE, 15984 },
	{ 0xfb57, 0xfb57, PDF_CMAP_SINGLE, 15986 },
	{ 0xfb58, 0xfb58, PDF_CMAP_SINGLE, 14156 },
	{ 0xfb59, 0xfb59, PDF_CMAP_SINGLE, 14185 },
	{ 0xfb5a, 0xfb5a, PDF_CMAP_SINGLE, 15183 },
	{ 0xfb5b, 0xfb5b, PDF_CMAP_SINGLE, 14239 },
	{ 0xfb5c, 0xfb5c, PDF_CMAP_SINGLE, 11845 },
	{ 0xfb5d, 0xfb5d, PDF_CMAP_SINGLE, 14349 },
	{ 0xfb5e, 0xfb5e, PDF_CMAP_SINGLE, 16290 },
	{ 0xfb5f, 0xfb5f, PDF_CMAP_SINGLE, 14364 },
	{ 0xfb60, 0xfb60, PDF_CMAP_SINGLE, 16252 },
	{ 0xfb61, 0xfb61, PDF_CMAP_SINGLE, 16217 },
	{ 0xfb62, 0xfb62, PDF_CMAP_SINGLE, 16286 },
	{ 0xfb63, 0xfb63, PDF_CMAP_SINGLE, 15763 },
	{ 0xfb64, 0xfb64, PDF_CMAP_SINGLE, 14357 },
	{ 0xfb65, 0xfb65, PDF_CMAP_SINGLE, 14475 },
	{ 0xfb66, 0xfb66, PDF_CMAP_SINGLE, 15456 },
	{ 0xfb67, 0xfb67, PDF_CMAP_SINGLE, 14576 },
	{ 0xfb68, 0xfb68, PDF_CMAP_SINGLE, 14250 },
	{ 0xfb69, 0xfb69, PDF_CMAP_SINGLE, 16180 },
	{ 0xfb6a, 0xfb6a, PDF_CMAP_SINGLE, 15378 },
	{ 0xfb6b, 0xfb6b, PDF_CMAP_SINGLE, 14592 },
	{ 0xfb6c, 0xfb6c, PDF_CMAP_SINGLE, 16298 },
	{ 0xfb6d, 0xfb6d, PDF_CMAP_SINGLE, 14603 },
	{ 0xfb6e, 0xfb6e, PDF_CMAP_SINGLE, 14633 },
	{ 0xfb6f, 0xfb6f, PDF_CMAP_SINGLE, 16167 },
	{ 0xfb70, 0xfb70, PDF_CMAP_SINGLE, 16272 },
	{ 0xfb71, 0xfb71, PDF_CMAP_SINGLE, 16215 },
	{ 0xfb72, 0xfb72, PDF_CMAP_SINGLE, 16219 },
	{ 0xfb73, 0xfb73, PDF_CMAP_SINGLE, 16226 },
	{ 0xfb74, 0xfb74, PDF_CMAP_SINGLE, 15734 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 14792 },
	{ 0xfb76, 0xfb77, PDF_CMAP_RANGE, 16228 },
	{ 0xfb78, 0xfb78, PDF_CMAP_SINGLE, 14653 },
	{ 0xfb79, 0xfb79, PDF_CMAP_SINGLE, 15462 },
	{ 0xfb7a, 0xfb7a, PDF_CMAP_SINGLE, 14800 },
	{ 0xfb7b, 0xfb7b, PDF_CMAP_SINGLE, 16418 },
	{ 0xfb7c, 0xfb7c, PDF_CMAP_SINGLE, 16172 },
	{ 0xfb7d, 0xfb7d, PDF_CMAP_SINGLE, 16168 },
	{ 0xfb7e, 0xfb7e, PDF_CMAP_SINGLE, 14796 },
	{ 0xfba1, 0xfba1, PDF_CMAP_SINGLE, 16166 },
	{ 0xfba2, 0xfba2, PDF_CMAP_SINGLE, 14797 },
	{ 0xfba3, 0xfba3, PDF_CMAP_SINGLE, 16191 },
	{ 0xfba4, 0xfba4, PDF_CMAP_SINGLE, 14789 },
	{ 0xfba5, 0xfba5, PDF_CMAP_SINGLE, 16334 },
	{ 0xfba6, 0xfba6, PDF_CMAP_SINGLE, 16436 },
	{ 0xfba7, 0xfba7, PDF_CMAP_SINGLE, 16222 },
	{ 0xfba8, 0xfba8, PDF_CMAP_SINGLE, 16434 },
	{ 0xfba9, 0xfba9, PDF_CMAP_SINGLE, 16468 },
	{ 0xfbaa, 0xfbaa, PDF_CMAP_SINGLE, 16760 },
	{ 0xfbab, 0xfbab, PDF_CMAP_SINGLE, 16224 },
	{ 0xfbac, 0xfbac, PDF_CMAP_SINGLE, 16169 },
	{ 0xfbad, 0xfbad, PDF_CMAP_SINGLE, 16477 },
	{ 0xfbae, 0xfbae, PDF_CMAP_SINGLE, 16195 },
	{ 0xfbaf, 0xfbaf, PDF_CMAP_SINGLE, 16232 },
	{ 0xfbb0, 0xfbb0, PDF_CMAP_SINGLE, 16480 },
	{ 0xfbb1, 0xfbb1, PDF_CMAP_SINGLE, 16170 },
	{ 0xfbb2, 0xfbb2, PDF_CMAP_SINGLE, 16483 },
	{ 0xfbb3, 0xfbb3, PDF_CMAP_SINGLE, 14823 },
	{ 0xfbb4, 0xfbb4, PDF_CMAP_SINGLE, 14518 },
	{ 0xfbb5, 0xfbb5, PDF_CMAP_SINGLE, 16528 },
	{ 0xfbb6, 0xfbb6, PDF_CMAP_SINGLE, 16456 },
	{ 0xfbb7, 0xfbb7, PDF_CMAP_SINGLE, 16402 },
	{ 0xfbb8, 0xfbb8, PDF_CMAP_SINGLE, 15740 },
	{ 0xfbb9, 0xfbb9, PDF_CMAP_SINGLE, 16459 },
	{ 0xfbba, 0xfbba, PDF_CMAP_SINGLE, 16461 },
	{ 0xfbbb, 0xfbbb, PDF_CMAP_SINGLE, 16463 },
	{ 0xfbbc, 0xfbbc, PDF_CMAP_SINGLE, 16975 },
	{ 0xfbbd, 0xfbbd, PDF_CMAP_SINGLE, 15737 },
	{ 0xfbbe, 0xfbbe, PDF_CMAP_SINGLE, 16192 },
	{ 0xfbbf, 0xfbbf, PDF_CMAP_SINGLE, 16138 },
	{ 0xfbc0, 0xfbc0, PDF_CMAP_SINGLE, 16336 },
	{ 0xfbc1, 0xfbc1, PDF_CMAP_SINGLE, 16182 },
	{ 0xfbc2, 0xfbc2, PDF_CMAP_SINGLE, 16493 },
	{ 0xfbc3, 0xfbc3, PDF_CMAP_SINGLE, 16517 },
	{ 0xfbc4, 0xfbc4, PDF_CMAP_SINGLE, 16516 },
	{ 0xfbc5, 0xfbc5, PDF_CMAP_SINGLE, 16702 },
	{ 0xfbc6, 0xfbc6, PDF_CMAP_SINGLE, 16699 },
	{ 0xfbc7, 0xfbc7, PDF_CMAP_SINGLE, 16326 },
	{ 0xfbc8, 0xfbc8, PDF_CMAP_SINGLE, 16518 },
	{ 0xfbc9, 0xfbc9, PDF_CMAP_SINGLE, 14674 },
	{ 0xfbca, 0xfbca, PDF_CMAP_SINGLE, 16545 },
	{ 0xfbcb, 0xfbcb, PDF_CMAP_SINGLE, 16178 },
	{ 0xfbcc, 0xfbcc, PDF_CMAP_SINGLE, 16899 },
	{ 0xfbcd, 0xfbcd, PDF_CMAP_SINGLE, 16230 },
	{ 0xfbce, 0xfbce, PDF_CMAP_SINGLE, 15747 },
	{ 0xfbcf, 0xfbcf, PDF_CMAP_SINGLE, 15989 },
	{ 0xfbd0, 0xfbd0, PDF_CMAP_SINGLE, 16546 },
	{ 0xfbd1, 0xfbd1, PDF_CMAP_SINGLE, 1392 },
	{ 0xfbd2, 0xfbd2, PDF_CMAP_SINGLE, 16185 },
	{ 0xfbd3, 0xfbd3, PDF_CMAP_SINGLE, 2161 },
	{ 0xfbd4, 0xfbd4, PDF_CMAP_SINGLE, 16164 },
	{ 0xfbd5, 0xfbd5, PDF_CMAP_SINGLE, 16558 },
	{ 0xfbd6, 0xfbd6, PDF_CMAP_SINGLE, 16196 },
	{ 0xfbd7, 0xfbd7, PDF_CMAP_SINGLE, 14581 },
	{ 0xfbd8, 0xfbd8, PDF_CMAP_SINGLE, 16329 },
	{ 0xfbd9, 0xfbd9, PDF_CMAP_SINGLE, 16333 },
	{ 0xfbda, 0xfbda, PDF_CMAP_SINGLE, 16309 },
	{ 0xfbdb, 0xfbdb, PDF_CMAP_SINGLE, 15996 },
	{ 0xfbdc, 0xfbdc, PDF_CMAP_SINGLE, 16338 },
	{ 0xfbdd, 0xfbdd, PDF_CMAP_SINGLE, 16205 },
	{ 0xfbde, 0xfbde, PDF_CMAP_SINGLE, 16572 },
	{ 0xfbdf, 0xfbdf, PDF_CMAP_SINGLE, 16567 },
	{ 0xfbe0, 0xfbe0, PDF_CMAP_SINGLE, 14424 },
	{ 0xfbe1, 0xfbe1, PDF_CMAP_SINGLE, 17327 },
	{ 0xfbe2, 0xfbe2, PDF_CMAP_SINGLE, 16236 },
	{ 0xfbe3, 0xfbe3, PDF_CMAP_SINGLE, 15749 },
	{ 0xfbe4, 0xfbe4, PDF_CMAP_SINGLE, 17302 },
	{ 0xfbe5, 0xfbe5, PDF_CMAP_SINGLE, 14758 },
	{ 0xfbe6, 0xfbe6, PDF_CMAP_SINGLE, 16253 },
	{ 0xfbe7, 0xfbe7, PDF_CMAP_SINGLE, 16319 },
	{ 0xfbe8, 0xfbe8, PDF_CMAP_SINGLE, 16584 },
	{ 0xfbe9, 0xfbe9, PDF_CMAP_SINGLE, 16223 },
	{ 0xfbea, 0xfbea, PDF_CMAP_SINGLE, 16171 },
	{ 0xfbeb, 0xfbeb, PDF_CMAP_SINGLE, 15730 },
	{ 0xfbec, 0xfbec, PDF_CMAP_SINGLE, 16591 },
	{ 0xfbed, 0xfbed, PDF_CMAP_SINGLE, 14853 },
	{ 0xfbee, 0xfbee, PDF_CMAP_SINGLE, 15746 },
	{ 0xfbef, 0xfbef, PDF_CMAP_SINGLE, 16604 },
	{ 0xfbf0, 0xfbf0, PDF_CMAP_SINGLE, 15717 },
	{ 0xfbf1, 0xfbf1, PDF_CMAP_SINGLE, 16596 },
	{ 0xfbf2, 0xfbf2, PDF_CMAP_SINGLE, 16602 },
	{ 0xfbf3, 0xfbf3, PDF_CMAP_SINGLE, 16173 },
	{ 0xfbf4, 0xfbf4, PDF_CMAP_SINGLE, 1429 },
	{ 0xfbf5, 0xfbf5, PDF_CMAP_SINGLE, 15745 },
	{ 0xfbf6, 0xfbf6, PDF_CMAP_SINGLE, 16625 },
	{ 0xfbf7, 0xfbf7, PDF_CMAP_SINGLE, 1781 },
	{ 0xfbf8, 0xfbf8, PDF_CMAP_SINGLE, 16632 },
	{ 0xfbf9, 0xfbf9, PDF_CMAP_SINGLE, 16635 },
	{ 0xfbfa, 0xfbfa, PDF_CMAP_SINGLE, 16193 },
	{ 0xfbfb, 0xfbfb, PDF_CMAP_SINGLE, 16664 },
	{ 0xfbfc, 0xfbfc, PDF_CMAP_SINGLE, 15753 },
	{ 0xfbfd, 0xfbfd, PDF_CMAP_SINGLE, 16220 },
	{ 0xfbfe, 0xfbfe, PDF_CMAP_SINGLE, 15719 },
	{ 0xfc40, 0xfc40, PDF_CMAP_SINGLE, 16142 },
	{ 0xfc41, 0xfc41, PDF_CMAP_SINGLE, 16147 },
	{ 0xfc42, 0xfc42, PDF_CMAP_SINGLE, 16674 },
	{ 0xfc43, 0xfc43, PDF_CMAP_SINGLE, 16330 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 16681 },
	{ 0xfc45, 0xfc45, PDF_CMAP_SINGLE, 16140 },
	{ 0xfc46, 0xfc46, PDF_CMAP_SINGLE, 16139 },
	{ 0xfc47, 0xfc47, PDF_CMAP_SINGLE, 16225 },
	{ 0xfc48, 0xfc48, PDF_CMAP_SINGLE, 15754 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 16174 },
	{ 0xfc4a, 0xfc4a, PDF_CMAP_SINGLE, 6513 },
	{ 0xfc4b, 0xfc4b, PDF_CMAP_SINGLE, 16693 },
	{ 0xfc4c, 0xfc4c, PDF_CMAP_SINGLE, 14891 },
	{ 0xfc4d, 0xfc4d, PDF_CMAP_SINGLE, 16235 },
	{ 0xfc4e, 0xfc4e, PDF_CMAP_SINGLE, 15269 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 16452 },
	{ 0xfc50, 0xfc50, PDF_CMAP_SINGLE, 16981 },
	{ 0xfc51, 0xfc51, PDF_CMAP_SINGLE, 14135 },
	{ 0xfc52, 0xfc52, PDF_CMAP_SINGLE, 16165 },
	{ 0xfc53, 0xfc53, PDF_CMAP_SINGLE, 14137 },
	{ 0xfc54, 0xfc54, PDF_CMAP_SINGLE, 14896 },
	{ 0xfc55, 0xfc55, PDF_CMAP_SINGLE, 16707 },
	{ 0xfc56, 0xfc56, PDF_CMAP_SINGLE, 16580 },
	{ 0xfc57, 0xfc57, PDF_CMAP_SINGLE, 15716 },
	{ 0xfc58, 0xfc58, PDF_CMAP_SINGLE, 16318 },
	{ 0xfc59, 0xfc59, PDF_CMAP_SINGLE, 16294 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 16428 },
	{ 0xfc5b, 0xfc5b, PDF_CMAP_SINGLE, 16727 },
	{ 0xfc5c, 0xfc5c, PDF_CMAP_SINGLE, 16287 },
	{ 0xfc5d, 0xfc5d, PDF_CMAP_SINGLE, 11072 },
	{ 0xfc5e, 0xfc5e, PDF_CMAP_SINGLE, 16295 },
	{ 0xfc5f, 0xfc5f, PDF_CMAP_SINGLE, 16389 },
	{ 0xfc60, 0xfc60, PDF_CMAP_SINGLE, 16385 },
	{ 0xfc61, 0xfc61, PDF_CMAP_SINGLE, 7323 },
	{ 0xfc62, 0xfc62, PDF_CMAP_SINGLE, 15748 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 2709 },
	{ 0xfc64, 0xfc64, PDF_CMAP_SINGLE, 16748 },
	{ 0xfc65, 0xfc65, PDF_CMAP_SINGLE, 16393 },
	{ 0xfc66, 0xfc66, PDF_CMAP_SINGLE, 15733 },
	{ 0xfc67, 0xfc67, PDF_CMAP_SINGLE, 15068 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 16296 },
	{ 0xfc69, 0xfc69, PDF_CMAP_SINGLE, 16183 },
	{ 0xfc6a, 0xfc6a, PDF_CMAP_SINGLE, 16756 },
	{ 0xfc6b, 0xfc6b, PDF_CMAP_SINGLE, 16390 },
	{ 0xfc6c, 0xfc6c, PDF_CMAP_SINGLE, 16386 },
	{ 0xfc6d, 0xfc6d, PDF_CMAP_SINGLE, 16314 },
	{ 0xfc6e, 0xfc6e, PDF_CMAP_SINGLE, 16320 },
	{ 0xfc6f, 0xfc6f, PDF_CMAP_SINGLE, 4784 },
	{ 0xfc70, 0xfc70, PDF_CMAP_SINGLE, 16317 },
	{ 0xfc71, 0xfc71, PDF_CMAP_SINGLE, 16755 },
	{ 0xfc72, 0xfc72, PDF_CMAP_SINGLE, 16774 },
	{ 0xfc73, 0xfc73, PDF_CMAP_SINGLE, 14932 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 16325 },
	{ 0xfc75, 0xfc75, PDF_CMAP_SINGLE, 16985 },
	{ 0xfc76, 0xfc76, PDF_CMAP_SINGLE, 16322 },
	{ 0xfc77, 0xfc77, PDF_CMAP_SINGLE, 16149 },
	{ 0xfc78, 0xfc78, PDF_CMAP_SINGLE, 16208 },
	{ 0xfc79, 0xfc79, PDF_CMAP_SINGLE, 16262 },
	{ 0xfc7a, 0xfc7a, PDF_CMAP_SINGLE, 16321 },
	{ 0xfc7b, 0xfc7b, PDF_CMAP_SINGLE, 16780 },
	{ 0xfc7c, 0xfc7c, PDF_CMAP_SINGLE, 16327 },
	{ 0xfc7d, 0xfc7d, PDF_CMAP_SINGLE, 16150 },
	{ 0xfc7e, 0xfc7e, PDF_CMAP_SINGLE, 16782 },
	{ 0xfca1, 0xfca1, PDF_CMAP_SINGLE, 16203 },
	{ 0xfca2, 0xfca2, PDF_CMAP_SINGLE, 16797 },
	{ 0xfca3, 0xfca3, PDF_CMAP_SINGLE, 4447 },
	{ 0xfca4, 0xfca4, PDF_CMAP_SINGLE, 16324 },
	{ 0xfca5, 0xfca5, PDF_CMAP_SINGLE, 13188 },
	{ 0xfca6, 0xfca6, PDF_CMAP_SINGLE, 15769 },
	{ 0xfca7, 0xfca7, PDF_CMAP_SINGLE, 16825 },
	{ 0xfca8, 0xfca8, PDF_CMAP_SINGLE, 14985 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 16823 },
	{ 0xfcaa, 0xfcaa, PDF_CMAP_SINGLE, 15755 },
	{ 0xfcab, 0xfcab, PDF_CMAP_SINGLE, 15718 },
	{ 0xfcac, 0xfcac, PDF_CMAP_SINGLE, 994 },
	{ 0xfcad, 0xfcad, PDF_CMAP_SINGLE, 15722 },
	{ 0xfcae, 0xfcae, PDF_CMAP_SINGLE, 15738 },
	{ 0xfcaf, 0xfcaf, PDF_CMAP_SINGLE, 16867 },
	{ 0xfcb0, 0xfcb0, PDF_CMAP_SINGLE, 1561 },
	{ 0xfcb1, 0xfcb1, PDF_CMAP_SINGLE, 14742 },
	{ 0xfcb2, 0xfcb2, PDF_CMAP_SINGLE, 16328 },
	{ 0xfcb3, 0xfcb3, PDF_CMAP_SINGLE, 15006 },
	{ 0xfcb4, 0xfcb4, PDF_CMAP_SINGLE, 16184 },
	{ 0xfcb5, 0xfcb5, PDF_CMAP_SINGLE, 15744 },
	{ 0xfcb6, 0xfcb7, PDF_CMAP_RANGE, 16862 },
	{ 0xfcb8, 0xfcb8, PDF_CMAP_SINGLE, 17131 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 15750 },
	{ 0xfcba, 0xfcba, PDF_CMAP_SINGLE, 16859 },
	{ 0xfcbb, 0xfcbb, PDF_CMAP_SINGLE, 16190 },
	{ 0xfcbc, 0xfcbc, PDF_CMAP_SINGLE, 16470 },
	{ 0xfcbd, 0xfcbd, PDF_CMAP_SINGLE, 16857 },
	{ 0xfcbe, 0xfcbe, PDF_CMAP_SINGLE, 16374 },
	{ 0xfcbf, 0xfcbf, PDF_CMAP_SINGLE, 16221 },
	{ 0xfcc0, 0xfcc0, PDF_CMAP_SINGLE, 15732 },
	{ 0xfcc1, 0xfcc1, PDF_CMAP_SINGLE, 16872 },
	{ 0xfcc2, 0xfcc2, PDF_CMAP_SINGLE, 16839 },
	{ 0xfcc3, 0xfcc3, PDF_CMAP_SINGLE, 15761 },
	{ 0xfcc4, 0xfcc4, PDF_CMAP_SINGLE, 15179 },
	{ 0xfcc5, 0xfcc5, PDF_CMAP_SINGLE, 15682 },
	{ 0xfcc6, 0xfcc6, PDF_CMAP_SINGLE, 15021 },
	{ 0xfcc7, 0xfcc7, PDF_CMAP_SINGLE, 15013 },
	{ 0xfcc8, 0xfcc8, PDF_CMAP_SINGLE, 16882 },
	{ 0xfcc9, 0xfcc9, PDF_CMAP_SINGLE, 16204 },
	{ 0xfcca, 0xfcca, PDF_CMAP_SINGLE, 16888 },
	{ 0xfccb, 0xfccb, PDF_CMAP_SINGLE, 16238 },
	{ 0xfccc, 0xfccc, PDF_CMAP_SINGLE, 16249 },
	{ 0xfccd, 0xfccd, PDF_CMAP_SINGLE, 16269 },
	{ 0xfcce, 0xfcce, PDF_CMAP_SINGLE, 16387 },
	{ 0xfccf, 0xfccf, PDF_CMAP_SINGLE, 16273 },
	{ 0xfcd0, 0xfcd0, PDF_CMAP_SINGLE, 15043 },
	{ 0xfcd1, 0xfcd1, PDF_CMAP_SINGLE, 2330 },
	{ 0xfcd2, 0xfcd2, PDF_CMAP_SINGLE, 16948 },
	{ 0xfcd3, 0xfcd3, PDF_CMAP_SINGLE, 16379 },
	{ 0xfcd4, 0xfcd5, PDF_CMAP_RANGE, 16960 },
	{ 0xfcd6, 0xfcd6, PDF_CMAP_SINGLE, 16381 },
	{ 0xfcd7, 0xfcd7, PDF_CMAP_SINGLE, 16248 },
	{ 0xfcd8, 0xfcd8, PDF_CMAP_SINGLE, 15477 },
	{ 0xfcd9, 0xfcd9, PDF_CMAP_SINGLE, 15475 },
	{ 0xfcda, 0xfcda, PDF_CMAP_SINGLE, 14501 },
	{ 0xfcdb, 0xfcdb, PDF_CMAP_SINGLE, 17401 },
	{ 0xfcdc, 0xfcdc, PDF_CMAP_SINGLE, 15614 },
	{ 0xfcdd, 0xfcdd, PDF_CMAP_SINGLE, 16983 },
	{ 0xfcde, 0xfcde, PDF_CMAP_SINGLE, 16947 },
	{ 0xfcdf, 0xfcdf, PDF_CMAP_SINGLE, 15721 },
	{ 0xfce0, 0xfce0, PDF_CMAP_SINGLE, 15486 },
	{ 0xfce1, 0xfce1, PDF_CMAP_SINGLE, 15727 },
	{ 0xfce2, 0xfce2, PDF_CMAP_SINGLE, 16384 },
	{ 0xfce3, 0xfce3, PDF_CMAP_SINGLE, 15808 },
	{ 0xfce4, 0xfce4, PDF_CMAP_SINGLE, 16388 },
	{ 0xfce5, 0xfce5, PDF_CMAP_SINGLE, 3698 },
	{ 0xfce6, 0xfce6, PDF_CMAP_SINGLE, 14648 },
	{ 0xfce7, 0xfce7, PDF_CMAP_SINGLE, 16292 },
	{ 0xfce8, 0xfce8, PDF_CMAP_SINGLE, 10586 },
	{ 0xfce9, 0xfce9, PDF_CMAP_SINGLE, 17011 },
	{ 0xfcea, 0xfcea, PDF_CMAP_SINGLE, 15049 },
	{ 0xfceb, 0xfceb, PDF_CMAP_SINGLE, 17021 },
	{ 0xfcec, 0xfcec, PDF_CMAP_SINGLE, 15050 },
	{ 0xfced, 0xfced, PDF_CMAP_SINGLE, 16306 },
	{ 0xfcee, 0xfcee, PDF_CMAP_SINGLE, 16310 },
	{ 0xfcef, 0xfcef, PDF_CMAP_SINGLE, 16279 },
	{ 0xfcf0, 0xfcf0, PDF_CMAP_SINGLE, 16282 },
	{ 0xfcf1, 0xfcf1, PDF_CMAP_SINGLE, 17035 },
	{ 0xfcf2, 0xfcf2, PDF_CMAP_SINGLE, 17402 },
	{ 0xfcf3, 0xfcf3, PDF_CMAP_SINGLE, 17034 },
	{ 0xfcf4, 0xfcf4, PDF_CMAP_SINGLE, 16209 },
	{ 0xfcf5, 0xfcf5, PDF_CMAP_SINGLE, 16280 },
	{ 0xfcf6, 0xfcf6, PDF_CMAP_SINGLE, 16803 },
	{ 0xfcf7, 0xfcf7, PDF_CMAP_SINGLE, 15708 },
	{ 0xfcf8, 0xfcf8, PDF_CMAP_SINGLE, 16227 },
	{ 0xfcf9, 0xfcf9, PDF_CMAP_SINGLE, 17040 },
	{ 0xfcfa, 0xfcfa, PDF_CMAP_SINGLE, 8084 },
	{ 0xfcfb, 0xfcfb, PDF_CMAP_SINGLE, 17094 },
	{ 0xfcfc, 0xfcfc, PDF_CMAP_SINGLE, 17058 },
	{ 0xfcfd, 0xfcfd, PDF_CMAP_SINGLE, 16378 },
	{ 0xfcfe, 0xfcfe, PDF_CMAP_SINGLE, 16285 },
	{ 0xfd40, 0xfd40, PDF_CMAP_SINGLE, 16382 },
	{ 0xfd41, 0xfd41, PDF_CMAP_SINGLE, 16283 },
	{ 0xfd42, 0xfd42, PDF_CMAP_SINGLE, 16351 },
	{ 0xfd43, 0xfd43, PDF_CMAP_SINGLE, 16358 },
	{ 0xfd44, 0xfd44, PDF_CMAP_SINGLE, 16350 },
	{ 0xfd45, 0xfd45, PDF_CMAP_SINGLE, 17079 },
	{ 0xfd46, 0xfd46, PDF_CMAP_SINGLE, 16372 },
	{ 0xfd47, 0xfd47, PDF_CMAP_SINGLE, 17080 },
	{ 0xfd48, 0xfd48, PDF_CMAP_SINGLE, 16363 },
	{ 0xfd49, 0xfd49, PDF_CMAP_SINGLE, 17087 },
	{ 0xfd4a, 0xfd4a, PDF_CMAP_SINGLE, 15729 },
	{ 0xfd4b, 0xfd4b, PDF_CMAP_SINGLE, 16207 },
	{ 0xfd4c, 0xfd4c, PDF_CMAP_SINGLE, 17093 },
	{ 0xfd4d, 0xfd4d, PDF_CMAP_SINGLE, 17109 },
	{ 0xfd4e, 0xfd4e, PDF_CMAP_SINGLE, 17104 },
	{ 0xfd4f, 0xfd4f, PDF_CMAP_SINGLE, 17112 },
	{ 0xfd50, 0xfd50, PDF_CMAP_SINGLE, 16213 },
	{ 0xfd51, 0xfd51, PDF_CMAP_SINGLE, 16360 },
	{ 0xfd52, 0xfd52, PDF_CMAP_SINGLE, 16371 },
	{ 0xfd53, 0xfd53, PDF_CMAP_SINGLE, 17119 },
	{ 0xfd54, 0xfd54, PDF_CMAP_SINGLE, 16339 },
	{ 0xfd55, 0xfd55, PDF_CMAP_SINGLE, 15987 },
	{ 0xfd56, 0xfd56, PDF_CMAP_SINGLE, 16277 },
	{ 0xfd57, 0xfd57, PDF_CMAP_SINGLE, 17134 },
	{ 0xfd58, 0xfd58, PDF_CMAP_SINGLE, 17140 },
	{ 0xfd59, 0xfd59, PDF_CMAP_SINGLE, 16268 },
	{ 0xfd5a, 0xfd5a, PDF_CMAP_SINGLE, 16335 },
	{ 0xfd5b, 0xfd5b, PDF_CMAP_SINGLE, 17132 },
	{ 0xfd5c, 0xfd5c, PDF_CMAP_SINGLE, 16340 },
	{ 0xfd5d, 0xfd5d, PDF_CMAP_SINGLE, 17136 },
	{ 0xfd5e, 0xfd5e, PDF_CMAP_SINGLE, 1986 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 2389 },
	{ 0xfd60, 0xfd60, PDF_CMAP_SINGLE, 17150 },
	{ 0xfd61, 0xfd61, PDF_CMAP_SINGLE, 17153 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 15565 },
	{ 0xfd63, 0xfd63, PDF_CMAP_SINGLE, 17159 },
	{ 0xfd64, 0xfd64, PDF_CMAP_SINGLE, 17403 },
	{ 0xfd65, 0xfd65, PDF_CMAP_SINGLE, 16346 },
	{ 0xfd66, 0xfd66, PDF_CMAP_SINGLE, 16267 },
	{ 0xfd67, 0xfd67, PDF_CMAP_SINGLE, 14131 },
	{ 0xfd68, 0xfd68, PDF_CMAP_SINGLE, 16266 },
	{ 0xfd69, 0xfd69, PDF_CMAP_SINGLE, 14145 },
	{ 0xfd6a, 0xfd6a, PDF_CMAP_SINGLE, 14144 },
	{ 0xfd6b, 0xfd6b, PDF_CMAP_SINGLE, 14146 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 16275 },
	{ 0xfd6d, 0xfd6d, PDF_CMAP_SINGLE, 16271 },
	{ 0xfd6e, 0xfd6e, PDF_CMAP_SINGLE, 15984 },
	{ 0xfd6f, 0xfd6f, PDF_CMAP_SINGLE, 17097 },
	{ 0xfd70, 0xfd70, PDF_CMAP_SINGLE, 15986 },
	{ 0xfd71, 0xfd71, PDF_CMAP_SINGLE, 16297 },
	{ 0xfd72, 0xfd72, PDF_CMAP_SINGLE, 17101 },
	{ 0xfd73, 0xfd73, PDF_CMAP_SINGLE, 17362 },
	{ 0xfd74, 0xfd74, PDF_CMAP_SINGLE, 16270 },
	{ 0xfd75, 0xfd75, PDF_CMAP_SINGLE, 16247 },
	{ 0xfd76, 0xfd76, PDF_CMAP_SINGLE, 15726 },
	{ 0xfd77, 0xfd77, PDF_CMAP_SINGLE, 3338 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 16145 },
	{ 0xfd79, 0xfd79, PDF_CMAP_SINGLE, 14162 },
	{ 0xfd7a, 0xfd7a, PDF_CMAP_SINGLE, 16148 },
	{ 0xfd7b, 0xfd7b, PDF_CMAP_SINGLE, 16152 },
	{ 0xfd7c, 0xfd7c, PDF_CMAP_SINGLE, 16211 },
	{ 0xfd7d, 0xfd7d, PDF_CMAP_SINGLE, 16159 },
	{ 0xfd7e, 0xfd7e, PDF_CMAP_SINGLE, 16156 },
	{ 0xfda1, 0xfda1, PDF_CMAP_SINGLE, 16312 },
	{ 0xfda2, 0xfda2, PDF_CMAP_SINGLE, 13213 },
	{ 0xfda3, 0xfda3, PDF_CMAP_SINGLE, 15167 },
	{ 0xfda4, 0xfda4, PDF_CMAP_SINGLE, 16210 },
	{ 0xfda5, 0xfda5, PDF_CMAP_SINGLE, 16154 },
	{ 0xfda6, 0xfda6, PDF_CMAP_SINGLE, 16157 },
	{ 0xfda7, 0xfda7, PDF_CMAP_SINGLE, 14172 },
	{ 0xfda8, 0xfda8, PDF_CMAP_SINGLE, 14469 },
	{ 0xfda9, 0xfda9, PDF_CMAP_SINGLE, 14177 },
	{ 0xfdaa, 0xfdaa, PDF_CMAP_SINGLE, 16343 },
	{ 0xfdab, 0xfdab, PDF_CMAP_SINGLE, 16342 },
	{ 0xfdac, 0xfdac, PDF_CMAP_SINGLE, 16345 },
	{ 0xfdad, 0xfdad, PDF_CMAP_SINGLE, 16344 },
	{ 0xfdae, 0xfdae, PDF_CMAP_SINGLE, 16349 },
	{ 0xfdaf, 0xfdaf, PDF_CMAP_SINGLE, 16019 },
	{ 0xfdb0, 0xfdb0, PDF_CMAP_SINGLE, 14185 },
	{ 0xfdb1, 0xfdb1, PDF_CMAP_SINGLE, 14190 },
	{ 0xfdb2, 0xfdb2, PDF_CMAP_SINGLE, 15180 },
	{ 0xfdb3, 0xfdb3, PDF_CMAP_SINGLE, 14196 },
	{ 0xfdb4, 0xfdb4, PDF_CMAP_SINGLE, 14207 },
	{ 0xfdb5, 0xfdb5, PDF_CMAP_SINGLE, 14203 },
	{ 0xfdb6, 0xfdb6, PDF_CMAP_SINGLE, 14210 },
	{ 0xfdb7, 0xfdb7, PDF_CMAP_SINGLE, 15183 },
	{ 0xfdb8, 0xfdb8, PDF_CMAP_SINGLE, 14226 },
	{ 0xfdb9, 0xfdb9, PDF_CMAP_SINGLE, 16355 },
	{ 0xfdba, 0xfdba, PDF_CMAP_SINGLE, 16348 },
	{ 0xfdbb, 0xfdbb, PDF_CMAP_SINGLE, 16347 },
	{ 0xfdbc, 0xfdbc, PDF_CMAP_SINGLE, 16357 },
	{ 0xfdbd, 0xfdbd, PDF_CMAP_SINGLE, 15221 },
	{ 0xfdbe, 0xfdbe, PDF_CMAP_SINGLE, 14242 },
	{ 0xfdbf, 0xfdbf, PDF_CMAP_SINGLE, 16354 },
	{ 0xfdc0, 0xfdc0, PDF_CMAP_SINGLE, 16359 },
	{ 0xfdc1, 0xfdc1, PDF_CMAP_SINGLE, 15981 },
	{ 0xfdc2, 0xfdc2, PDF_CMAP_SINGLE, 3370 },
	{ 0xfdc3, 0xfdc3, PDF_CMAP_SINGLE, 15993 },
	{ 0xfdc4, 0xfdc4, PDF_CMAP_SINGLE, 15990 },
	{ 0xfdc5, 0xfdc5, PDF_CMAP_SINGLE, 14239 },
	{ 0xfdc6, 0xfdc6, PDF_CMAP_SINGLE, 16366 },
	{ 0xfdc7, 0xfdc7, PDF_CMAP_SINGLE, 17108 },
	{ 0xfdc8, 0xfdc8, PDF_CMAP_SINGLE, 15764 },
	{ 0xfdc9, 0xfdc9, PDF_CMAP_SINGLE, 15713 },
	{ 0xfdca, 0xfdca, PDF_CMAP_SINGLE, 16353 },
	{ 0xfdcb, 0xfdcb, PDF_CMAP_SINGLE, 14174 },
	{ 0xfdcc, 0xfdcc, PDF_CMAP_SINGLE, 15226 },
	{ 0xfdcd, 0xfdcd, PDF_CMAP_SINGLE, 14347 },
	{ 0xfdce, 0xfdce, PDF_CMAP_SINGLE, 16364 },
	{ 0xfdcf, 0xfdcf, PDF_CMAP_SINGLE, 16367 },
	{ 0xfdd0, 0xfdd0, PDF_CMAP_SINGLE, 11845 },
	{ 0xfdd1, 0xfdd1, PDF_CMAP_SINGLE, 16380 },
	{ 0xfdd2, 0xfdd2, PDF_CMAP_SINGLE, 16365 },
	{ 0xfdd3, 0xfdd3, PDF_CMAP_SINGLE, 14349 },
	{ 0xfdd4, 0xfdd4, PDF_CMAP_SINGLE, 16356 },
	{ 0xfdd5, 0xfdd5, PDF_CMAP_SINGLE, 16274 },
	{ 0xfdd6, 0xfdd6, PDF_CMAP_SINGLE, 16302 },
	{ 0xfdd7, 0xfdd7, PDF_CMAP_SINGLE, 14353 },
	{ 0xfdd8, 0xfdd8, PDF_CMAP_SINGLE, 16250 },
	{ 0xfdd9, 0xfdd9, PDF_CMAP_SINGLE, 15243 },
	{ 0xfdda, 0xfdda, PDF_CMAP_SINGLE, 16218 },
	{ 0xfddb, 0xfddb, PDF_CMAP_SINGLE, 14358 },
	{ 0xfddc, 0xfddc, PDF_CMAP_SINGLE, 16290 },
	{ 0xfddd, 0xfddd, PDF_CMAP_SINGLE, 15246 },
	{ 0xfdde, 0xfdde, PDF_CMAP_SINGLE, 14364 },
	{ 0xfddf, 0xfddf, PDF_CMAP_SINGLE, 8276 },
	{ 0xfde0, 0xfde0, PDF_CMAP_SINGLE, 14367 },
	{ 0xfde1, 0xfde1, PDF_CMAP_SINGLE, 16288 },
	{ 0xfde2, 0xfde2, PDF_CMAP_SINGLE, 17404 },
	{ 0xfde3, 0xfde3, PDF_CMAP_SINGLE, 15894 },
	{ 0xfde4, 0xfde4, PDF_CMAP_SINGLE, 15252 },
	{ 0xfde5, 0xfde5, PDF_CMAP_SINGLE, 16252 },
	{ 0xfde6, 0xfde6, PDF_CMAP_SINGLE, 16391 },
	{ 0xfde7, 0xfde7, PDF_CMAP_SINGLE, 16251 },
	{ 0xfde8, 0xfde8, PDF_CMAP_SINGLE, 15731 },
	{ 0xfde9, 0xfde9, PDF_CMAP_SINGLE, 15991 },
	{ 0xfdea, 0xfdea, PDF_CMAP_SINGLE, 15762 },
	{ 0xfdeb, 0xfdeb, PDF_CMAP_SINGLE, 16289 },
	{ 0xfdec, 0xfdec, PDF_CMAP_SINGLE, 14273 },
	{ 0xfded, 0xfded, PDF_CMAP_SINGLE, 14291 },
	{ 0xfdee, 0xfdee, PDF_CMAP_SINGLE, 15994 },
	{ 0xfdef, 0xfdef, PDF_CMAP_SINGLE, 16217 },
	{ 0xfdf0, 0xfdf0, PDF_CMAP_SINGLE, 15445 },
	{ 0xfdf1, 0xfdf1, PDF_CMAP_SINGLE, 16376 },
	{ 0xfdf2, 0xfdf2, PDF_CMAP_SINGLE, 16373 },
	{ 0xfdf3, 0xfdf3, PDF_CMAP_SINGLE, 15436 },
	{ 0xfdf4, 0xfdf4, PDF_CMAP_SINGLE, 15756 },
	{ 0xfdf5, 0xfdf5, PDF_CMAP_SINGLE, 16291 },
	{ 0xfdf6, 0xfdf6, PDF_CMAP_SINGLE, 14272 },
	{ 0xfdf7, 0xfdf7, PDF_CMAP_SINGLE, 16212 },
	{ 0xfdf8, 0xfdf8, PDF_CMAP_SINGLE, 14336 },
	{ 0xfdf9, 0xfdf9, PDF_CMAP_SINGLE, 15231 },
	{ 0xfdfa, 0xfdfa, PDF_CMAP_SINGLE, 16286 },
	{ 0xfdfb, 0xfdfb, PDF_CMAP_SINGLE, 14370 },
	{ 0xfdfc, 0xfdfc, PDF_CMAP_SINGLE, 16361 },
	{ 0xfdfd, 0xfdfd, PDF_CMAP_SINGLE, 14371 },
	{ 0xfdfe, 0xfdfe, PDF_CMAP_SINGLE, 16352 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 15763 },
	{ 0xfe41, 0xfe41, PDF_CMAP_SINGLE, 16377 },
	{ 0xfe42, 0xfe42, PDF_CMAP_SINGLE, 14375 },
	{ 0xfe43, 0xfe43, PDF_CMAP_SINGLE, 16293 },
	{ 0xfe44, 0xfe44, PDF_CMAP_SINGLE, 16368 },
	{ 0xfe45, 0xfe45, PDF_CMAP_SINGLE, 14390 },
	{ 0xfe46, 0xfe46, PDF_CMAP_SINGLE, 16281 },
	{ 0xfe47, 0xfe47, PDF_CMAP_SINGLE, 16370 },
	{ 0xfe48, 0xfe48, PDF_CMAP_SINGLE, 15757 },
	{ 0xfe49, 0xfe49, PDF_CMAP_SINGLE, 17382 },
	{ 0xfe4a, 0xfe4a, PDF_CMAP_SINGLE, 13602 },
	{ 0xfe4b, 0xfe4b, PDF_CMAP_SINGLE, 16369 },
	{ 0xfe4c, 0xfe4c, PDF_CMAP_SINGLE, 16284 },
	{ 0xfe4d, 0xfe4d, PDF_CMAP_SINGLE, 16305 },
	{ 0xfe4e, 0xfe4e, PDF_CMAP_SINGLE, 16278 },
	{ 0xfe4f, 0xfe4f, PDF_CMAP_SINGLE, 15983 },
	{ 0xfe50, 0xfe50, PDF_CMAP_SINGLE, 16233 },
	{ 0xfe51, 0xfe51, PDF_CMAP_SINGLE, 16375 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 16258 },
	{ 0xfe53, 0xfe53, PDF_CMAP_SINGLE, 16264 },
	{ 0xfe54, 0xfe54, PDF_CMAP_SINGLE, 16261 },
	{ 0xfe55, 0xfe55, PDF_CMAP_SINGLE, 16259 },
	{ 0xfe56, 0xfe56, PDF_CMAP_SINGLE, 16255 },
	{ 0xfe57, 0xfe57, PDF_CMAP_SINGLE, 16265 },
	{ 0xfe58, 0xfe58, PDF_CMAP_SINGLE, 15285 },
	{ 0xfe59, 0xfe59, PDF_CMAP_SINGLE, 14434 },
	{ 0xfe5a, 0xfe5a, PDF_CMAP_SINGLE, 16263 },
	{ 0xfe5b, 0xfe5b, PDF_CMAP_SINGLE, 16177 },
	{ 0xfe5c, 0xfe5c, PDF_CMAP_SINGLE, 14357 },
	{ 0xfe5d, 0xfe5d, PDF_CMAP_SINGLE, 14462 },
	{ 0xfe5e, 0xfe5e, PDF_CMAP_SINGLE, 16201 },
	{ 0xfe5f, 0xfe5f, PDF_CMAP_SINGLE, 16200 },
	{ 0xfe60, 0xfe60, PDF_CMAP_SINGLE, 16199 },
	{ 0xfe61, 0xfe61, PDF_CMAP_SINGLE, 16197 },
	{ 0xfe62, 0xfe62, PDF_CMAP_SINGLE, 17387 },
	{ 0xfe63, 0xfe63, PDF_CMAP_SINGLE, 14466 },
	{ 0xfe64, 0xfe64, PDF_CMAP_SINGLE, 16239 },
	{ 0xfe65, 0xfe65, PDF_CMAP_SINGLE, 16206 },
	{ 0xfe66, 0xfe66, PDF_CMAP_SINGLE, 15304 },
	{ 0xfe67, 0xfe67, PDF_CMAP_SINGLE, 17405 },
	{ 0xfe68, 0xfe69, PDF_CMAP_RANGE, 15723 },
	{ 0xfe6a, 0xfe6a, PDF_CMAP_SINGLE, 17406 },
	{ 0xfe6b, 0xfe6b, PDF_CMAP_SINGLE, 16331 },
	{ 0xfe6c, 0xfe6c, PDF_CMAP_SINGLE, 15826 },
	{ 0xfe6d, 0xfe6d, PDF_CMAP_SINGLE, 14457 },
	{ 0xfe6e, 0xfe6e, PDF_CMAP_SINGLE, 16144 },
	{ 0xfe6f, 0xfe6f, PDF_CMAP_SINGLE, 14382 },
	{ 0xfe70, 0xfe70, PDF_CMAP_SINGLE, 14475 },
	{ 0xfe71, 0xfe71, PDF_CMAP_SINGLE, 15456 },
	{ 0xfe72, 0xfe72, PDF_CMAP_SINGLE, 10911 },
	{ 0xfe73, 0xfe73, PDF_CMAP_SINGLE, 15457 },
	{ 0xfe74, 0xfe74, PDF_CMAP_SINGLE, 14483 },
	{ 0xfe75, 0xfe75, PDF_CMAP_SINGLE, 15988 },
	{ 0xfe76, 0xfe76, PDF_CMAP_SINGLE, 16300 },
	{ 0xfe77, 0xfe77, PDF_CMAP_SINGLE, 16257 },
	{ 0xfe78, 0xfe78, PDF_CMAP_SINGLE, 15309 },
	{ 0xfe79, 0xfe79, PDF_CMAP_SINGLE, 15316 },
	{ 0xfe7a, 0xfe7a, PDF_CMAP_SINGLE, 15315 },
	{ 0xfe7b, 0xfe7b, PDF_CMAP_SINGLE, 16332 },
	{ 0xfe7c, 0xfe7c, PDF_CMAP_SINGLE, 16337 },
	{ 0xfe7d, 0xfe7d, PDF_CMAP_SINGLE, 16189 },
	{ 0xfe7e, 0xfe7e, PDF_CMAP_SINGLE, 16202 },
	{ 0xfea1, 0xfea1, PDF_CMAP_SINGLE, 9042 },
	{ 0xfea2, 0xfea2, PDF_CMAP_SINGLE, 14509 },
	{ 0xfea3, 0xfea3, PDF_CMAP_SINGLE, 16198 },
	{ 0xfea4, 0xfea4, PDF_CMAP_SINGLE, 14517 },
	{ 0xfea5, 0xfea5, PDF_CMAP_SINGLE, 16141 },
	{ 0xfea6, 0xfea6, PDF_CMAP_SINGLE, 16307 },
	{ 0xfea7, 0xfea7, PDF_CMAP_SINGLE, 14528 },
	{ 0xfea8, 0xfea8, PDF_CMAP_SINGLE, 15919 },
	{ 0xfea9, 0xfea9, PDF_CMAP_SINGLE, 14543 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 14549 },
	{ 0xfeab, 0xfeab, PDF_CMAP_SINGLE, 16179 },
	{ 0xfeac, 0xfeac, PDF_CMAP_SINGLE, 15765 },
	{ 0xfead, 0xfead, PDF_CMAP_SINGLE, 16181 },
	{ 0xfeae, 0xfeae, PDF_CMAP_SINGLE, 14551 },
	{ 0xfeaf, 0xfeaf, PDF_CMAP_SINGLE, 16175 },
	{ 0xfeb0, 0xfeb0, PDF_CMAP_SINGLE, 14560 },
	{ 0xfeb1, 0xfeb1, PDF_CMAP_SINGLE, 15362 },
	{ 0xfeb2, 0xfeb2, PDF_CMAP_SINGLE, 16242 },
	{ 0xfeb3, 0xfeb3, PDF_CMAP_SINGLE, 16240 },
	{ 0xfeb4, 0xfeb4, PDF_CMAP_SINGLE, 15766 },
	{ 0xfeb5, 0xfeb6, PDF_CMAP_RANGE, 14566 },
	{ 0xfeb7, 0xfeb7, PDF_CMAP_SINGLE, 16437 },
	{ 0xfeb8, 0xfeb8, PDF_CMAP_SINGLE, 16243 },
	{ 0xfeb9, 0xfeb9, PDF_CMAP_SINGLE, 14576 },
	{ 0xfeba, 0xfeba, PDF_CMAP_SINGLE, 15371 },
	{ 0xfebb, 0xfebb, PDF_CMAP_SINGLE, 14583 },
	{ 0xfebc, 0xfebc, PDF_CMAP_SINGLE, 14250 },
	{ 0xfebd, 0xfebd, PDF_CMAP_SINGLE, 17407 },
	{ 0xfebe, 0xfebe, PDF_CMAP_SINGLE, 14586 },
	{ 0xfebf, 0xfebf, PDF_CMAP_SINGLE, 15377 },
	{ 0xfec0, 0xfec0, PDF_CMAP_SINGLE, 16180 },
	{ 0xfec1, 0xfec1, PDF_CMAP_SINGLE, 16313 },
	{ 0xfec2, 0xfec2, PDF_CMAP_SINGLE, 15378 },
	{ 0xfec3, 0xfec3, PDF_CMAP_SINGLE, 16394 },
	{ 0xfec4, 0xfec4, PDF_CMAP_SINGLE, 15735 },
	{ 0xfec5, 0xfec5, PDF_CMAP_SINGLE, 16194 },
	{ 0xfec6, 0xfec6, PDF_CMAP_SINGLE, 14592 },
	{ 0xfec7, 0xfec7, PDF_CMAP_SINGLE, 14594 },
	{ 0xfec8, 0xfec8, PDF_CMAP_SINGLE, 16244 },
	{ 0xfec9, 0xfec9, PDF_CMAP_SINGLE, 15767 },
	{ 0xfeca, 0xfeca, PDF_CMAP_SINGLE, 16298 },
	{ 0xfecb, 0xfecb, PDF_CMAP_SINGLE, 16153 },
	{ 0xfecc, 0xfecc, PDF_CMAP_SINGLE, 16155 },
	{ 0xfecd, 0xfecd, PDF_CMAP_SINGLE, 14606 },
	{ 0xfece, 0xfece, PDF_CMAP_SINGLE, 14601 },
	{ 0xfecf, 0xfecf, PDF_CMAP_SINGLE, 14603 },
	{ 0xfed0, 0xfed0, PDF_CMAP_SINGLE, 16146 },
	{ 0xfed1, 0xfed1, PDF_CMAP_SINGLE, 16151 },
	{ 0xfed2, 0xfed2, PDF_CMAP_SINGLE, 16311 },
	{ 0xfed3, 0xfed3, PDF_CMAP_SINGLE, 16160 },
	{ 0xfed4, 0xfed4, PDF_CMAP_SINGLE, 16315 },
	{ 0xfed5, 0xfed5, PDF_CMAP_SINGLE, 16188 },
	{ 0xfed6, 0xfed6, PDF_CMAP_SINGLE, 16301 },
	{ 0xfed7, 0xfed7, PDF_CMAP_SINGLE, 14617 },
	{ 0xfed8, 0xfed8, PDF_CMAP_SINGLE, 16214 },
	{ 0xfed9, 0xfed9, PDF_CMAP_SINGLE, 16241 },
	{ 0xfeda, 0xfeda, PDF_CMAP_SINGLE, 16308 },
	{ 0xfedb, 0xfedb, PDF_CMAP_SINGLE, 14620 },
	{ 0xfedc, 0xfedc, PDF_CMAP_SINGLE, 16158 },
	{ 0xfedd, 0xfedd, PDF_CMAP_SINGLE, 16186 },
	{ 0xfede, 0xfede, PDF_CMAP_SINGLE, 14619 },
	{ 0xfedf, 0xfedf, PDF_CMAP_SINGLE, 15751 },
	{ 0xfee0, 0xfee0, PDF_CMAP_SINGLE, 15418 },
	{ 0xfee1, 0xfee1, PDF_CMAP_SINGLE, 16162 },
	{ 0xfee2, 0xfee2, PDF_CMAP_SINGLE, 15985 },
	{ 0xfee3, 0xfee3, PDF_CMAP_SINGLE, 15417 },
	{ 0xfee4, 0xfee4, PDF_CMAP_SINGLE, 15742 },
	{ 0xfee5, 0xfee5, PDF_CMAP_SINGLE, 16676 },
	{ 0xfee6, 0xfee6, PDF_CMAP_SINGLE, 16187 },
	{ 0xfee7, 0xfee7, PDF_CMAP_SINGLE, 15982 },
	{ 0xfee8, 0xfee8, PDF_CMAP_SINGLE, 16299 },
	{ 0xfee9, 0xfee9, PDF_CMAP_SINGLE, 14629 },
	{ 0xfeea, 0xfeea, PDF_CMAP_SINGLE, 16176 },
	{ 0xfeeb, 0xfeeb, PDF_CMAP_SINGLE, 14633 },
	{ 0xfeec, 0xfeec, PDF_CMAP_SINGLE, 16260 },
	{ 0xfeed, 0xfeed, PDF_CMAP_SINGLE, 16143 },
	{ 0xfeee, 0xfeee, PDF_CMAP_SINGLE, 16246 },
	{ 0xfeef, 0xfeef, PDF_CMAP_SINGLE, 16276 },
	{ 0xfef0, 0xfef0, PDF_CMAP_SINGLE, 14642 },
	{ 0xfef1, 0xfef1, PDF_CMAP_SINGLE, 16167 },
	{ 0xfef2, 0xfef2, PDF_CMAP_SINGLE, 14643 },
	{ 0xfef3, 0xfef3, PDF_CMAP_SINGLE, 16245 },
	{ 0xfef4, 0xfef4, PDF_CMAP_SINGLE, 14647 },
	{ 0xfef5, 0xfef5, PDF_CMAP_SINGLE, 15736 },
	{ 0xfef6, 0xfef6, PDF_CMAP_SINGLE, 16161 },
	{ 0xfef7, 0xfef7, PDF_CMAP_SINGLE, 16272 },
	{ 0xfef8, 0xfef8, PDF_CMAP_SINGLE, 14656 },
	{ 0xfef9, 0xfef9, PDF_CMAP_SINGLE, 15743 },
	{ 0xfefa, 0xfefa, PDF_CMAP_SINGLE, 14666 },
	{ 0xfefb, 0xfefb, PDF_CMAP_SINGLE, 16256 },
	{ 0xfefc, 0xfefc, PDF_CMAP_SINGLE, 16163 },
	{ 0xfefd, 0xfefd, PDF_CMAP_SINGLE, 14672 },
	{ 0xfefe, 0xfefe, PDF_CMAP_SINGLE, 16254 },
};

static const unsigned short pdf_cmap_HKdla_B5_H_table[779] =
{
	248,247,5551,5550,13585,13629,13641,16713,
	16035,16004,16007,15877,16534,14788,15725,15999,
	15479,16006,14460,16039,16910,16036,16778,16479,
	16662,16523,15974,15973,15759,16798,14950,16737,
	16010,15758,16013,15804,16021,14755,16884,16017,
	15302,15811,16046,14150,14449,14848,17036,16987,
	16045,16675,16038,16032,15150,15805,14436,15485,
	16041,16909,16216,16024,15728,16040,16020,16029,
	15018,16037,15564,16536,16031,16885,16011,15218,
	14304,15071,17120,16023,16786,14472,17096,16016,
	14447,16028,16708,16710,16030,16009,16531,15374,
	16005,14556,16489,14296,16025,16003,16008,16000,
	16049,16746,14187,15975,15992,16994,14325,14324,
	15173,15961,10940,15807,16573,15998,14381,16048,
	16043,16050,16929,16234,14363,16026,16931,16014,
	15357,16474,16047,15560,14664,17085,16588,15964,
	16044,14618,16042,17305,14490,16012,14495,16033,
	16775,16015,16323,16034,16027,15997,16465,16316,
	16341,15124,16930,14235,16507,14216,15284,16848,
	16996,14430,14188,15806,14355,16383,15715,14214,
	15178,14138,17018,15482,16901,15398,14511,14646,
	14248,15220,16237,16022,14268,15995,16362,16842,
	14807,16593,15217,16335,1986,17150,15565,15984,
	15986,14156,14185,15183,14239,11845,14349,16290,
	14364,16252,16217,16286,15763,14357,14475,15456,
	14576,14250,16180,15378,14592,16298,14603,14633,
	16167,16272,16215,16219,16226,15734,14792,14653,
	15462,14800,16418,16172,16168,14796,16166,14797,
	16191,14789,16334,16436,16222,16434,16468,16760,
	16224,16169,16477,16195,16232,16480,16170,16483,
	14823,14518,16528,16456,16402,15740,16459,16461,
	16463,16975,15737,16192,16138,16336,16182,16493,
	16517,16516,16702,16699,16326,16518,14674,16545,
	16178,16899,16230,15747,15989,16546,1392,16185,
	2161,16164,16558,16196,14581,16329,16333,16309,
	15996,16338,16205,16572,16567,14424,17327,16236,
	15749,17302,14758,16253,16319,16584,16223,16171,
	15730,16591,14853,15746,16604,15717,16596,16602,
	16173,1429,15745,16625,1781,16632,16635,16193,
	16664,15753,16220,15719,16142,16147,16674,16330,
	16681,16140,16139,16225,15754,16174,6513,16693,
	14891,16235,15269,16452,16981,14135,16165,14137,
	14896,16707,16580,15716,16318,16294,16428,16727,
	16287,11072,16295,16389,16385,7323,15748,2709,
	16748,16393,15733,15068,16296,16183,16756,16390,
	16386,16314,16320,4784,16317,16755,16774,14932,
	16325,16985,16322,16149,16208,16262,16321,16780,
	16327,16150,16782,16203,16797,4447,16324,13188,
	15769,16825,14985,16823,15755,15718,994,15722,
	15738,16867,1561,14742,16328,15006,16184,15744,
	17131,15750,16859,16190,16470,16857,16374,16221,
	15732,16872,16839,15761,15179,15682,15021,15013,
	16882,16204,16888,16238,16249,16269,16387,16273,
	15043,2330,16948,16379,16381,16248,15477,15475,
	14501,17401,15614,16983,16947,15721,15486,15727,
	16384,15808,16388,3698,14648,16292,10586,17011,
	15049,17021,15050,16306,16310,16279,16282,17035,
	17402,17034,16209,16280,16803,15708,16227,17040,
	8084,17094,17058,16378,16285,16382,16283,16351,
	16358,16350,17079,16372,17080,16363,17087,15729,
	16207,17093,17109,17104,17112,16213,16360,16371,
	17119,16339,15987,16277,17134,17140,16268,16335,
	17132,16340,17136,1986,2389,17150,17153,15565,
	17159,17403,16346,16267,14131,16266,14145,14144,
	14146,16275,16271,15984,17097,15986,16297,17101,
	17362,16270,16247,15726,3338,16145,14162,16148,
	16152,16211,16159,16156,16312,13213,15167,16210,
	16154,16157,14172,14469,14177,16343,16342,16345,
	16344,16349,16019,14185,14190,15180,14196,14207,
	14203,14210,15183,14226,16355,16348,16347,16357,
	15221,14242,16354,16359,15981,3370,15993,15990,
	14239,16366,17108,15764,15713,16353,14174,15226,
	14347,16364,16367,11845,16380,16365,14349,16356,
	16274,16302,14353,16250,15243,16218,14358,16290,
	15246,14364,8276,14367,16288,17404,15894,15252,
	16252,16391,16251,15731,15991,15762,16289,14273,
	14291,15994,16217,15445,16376,16373,15436,15756,
	16291,14272,16212,14336,15231,16286,14370,16361,
	14371,16352,15763,16377,14375,16293,16368,14390,
	16281,16370,15757,17382,13602,16369,16284,16305,
	16278,15983,16233,16375,16258,16264,16261,16259,
	16255,16265,15285,14434,16263,16177,14357,14462,
	16201,16200,16199,16197,17387,14466,16239,16206,
	15304,17405,17406,16331,15826,14457,16144,14382,
	14475,15456,10911,15457,14483,15988,16300,16257,
	15309,15316,15315,16332,16337,16189,16202,9042,
	14509,16198,14517,16141,16307,14528,15919,14543,
	14549,16179,15765,16181,14551,16175,14560,15362,
	16242,16240,15766,16437,16243,14576,15371,14583,
	14250,17407,14586,15377,16180,16313,15378,16394,
	15735,16194,14592,14594,16244,15767,16298,16153,
	16155,14606,14601,14603,16146,16151,16311,16160,
	16315,16188,16301,14617,16214,16241,16308,14620,
	16158,16186,14619,15751,15418,16162,15985,15417,
	15742,16676,16187,15982,16299,14629,16176,14633,
	16260,16143,16246,16276,14642,16167,14643,16245,
	14647,15736,16161,16272,14656,15743,14666,16256,
	16163,14672,16254,
};

pdf_cmap pdf_cmap_HKdla_B5_H =
{
	-1, "HKdla-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa140, 0xfefe },
	},
	1022, 1022, (pdf_range*) pdf_cmap_HKdla_B5_H_ranges,
	779, 779, (unsigned short*) pdf_cmap_HKdla_B5_H_table,
};

/* HKdla-B5-V */

static const pdf_range pdf_cmap_HKdla_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_HKdla_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_HKdla_B5_V =
{
	-1, "HKdla-B5-V", "HKdla-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_HKdla_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_HKdla_B5_V_table,
};

/* HKdlb-B5-H */

static const pdf_range pdf_cmap_HKdlb_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_TABLE, 0 },
	{ 0x8ea1, 0x8ec4, PDF_CMAP_TABLE, 63 },
	{ 0x8ec5, 0x8ec6, PDF_CMAP_RANGE, 15220 },
	{ 0x8ec7, 0x8efe, PDF_CMAP_TABLE, 99 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_TABLE, 155 },
	{ 0x8fa1, 0x8ffe, PDF_CMAP_TABLE, 218 },
	{ 0x9040, 0x907e, PDF_CMAP_TABLE, 312 },
	{ 0x90a1, 0x90fe, PDF_CMAP_TABLE, 375 },
	{ 0x9140, 0x917e, PDF_CMAP_TABLE, 469 },
	{ 0x91a1, 0x91be, PDF_CMAP_TABLE, 532 },
	{ 0x91bf, 0x91c0, PDF_CMAP_RANGE, 16862 },
	{ 0x91c1, 0x91d5, PDF_CMAP_TABLE, 562 },
	{ 0x91d7, 0x91fe, PDF_CMAP_TABLE, 583 },
	{ 0x9240, 0x9265, PDF_CMAP_TABLE, 623 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 661 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 663 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 665 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0x8fc3, 0x8fc3, PDF_CMAP_SINGLE, 16208 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 16393 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 16303 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 16262 },
	{ 0x8fc7, 0x8fc7, PDF_CMAP_SINGLE, 16531 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 15021 },
	{ 0x8fc9, 0x8fc9, PDF_CMAP_SINGLE, 16184 },
	{ 0x8fca, 0x8fca, PDF_CMAP_SINGLE, 15733 },
	{ 0x8fcb, 0x8fcb, PDF_CMAP_SINGLE, 16185 },
	{ 0x8fcc, 0x8fcc, PDF_CMAP_SINGLE, 17101 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 16248 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 16285 },
	{ 0x8fcf, 0x8fcf, PDF_CMAP_SINGLE, 16528 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 15744 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 16174 },
	{ 0x8fd2, 0x8fd2, PDF_CMAP_SINGLE, 14518 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 16825 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 15754 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 16164 },
	{ 0x8fd6, 0x8fd6, PDF_CMAP_SINGLE, 15006 },
	{ 0x8fd7, 0x8fd7, PDF_CMAP_SINGLE, 15068 },
	{ 0x8fd8, 0x8fd8, PDF_CMAP_SINGLE, 15717 },
	{ 0x8fd9, 0x8fd9, PDF_CMAP_SINGLE, 16517 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 16209 },
	{ 0x8fdb, 0x8fdb, PDF_CMAP_SINGLE, 17402 },
	{ 0x8fdc, 0x8fdc, PDF_CMAP_SINGLE, 16227 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 16760 },
	{ 0x8fde, 0x8fde, PDF_CMAP_SINGLE, 16823 },
	{ 0x8fdf, 0x8fdf, PDF_CMAP_SINGLE, 15986 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 16857 },
	{ 0x8fe1, 0x8fe1, PDF_CMAP_SINGLE, 16782 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 15734 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 17108 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 17058 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 16983 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 16140 },
	{ 0x8fe7, 0x8fe7, PDF_CMAP_SINGLE, 14742 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 16981 },
	{ 0x8fe9, 0x8fe9, PDF_CMAP_SINGLE, 16567 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 16282 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 16330 },
	{ 0x8fec, 0x8fec, PDF_CMAP_SINGLE, 16222 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 15749 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 16378 },
	{ 0x8fef, 0x8fef, PDF_CMAP_SINGLE, 16310 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 15043 },
	{ 0x8ff1, 0x8ff1, PDF_CMAP_SINGLE, 16328 },
	{ 0x8ff2, 0x8ff2, PDF_CMAP_SINGLE, 16203 },
	{ 0x8ff3, 0x8ff3, PDF_CMAP_SINGLE, 16625 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 16436 },
	{ 0x8ff5, 0x8ff5, PDF_CMAP_SINGLE, 16219 },
	{ 0x8ff6, 0x8ff6, PDF_CMAP_SINGLE, 17097 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 15477 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 16249 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 17094 },
	{ 0x8ffa, 0x8ffa, PDF_CMAP_SINGLE, 14543 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 16884 },
	{ 0x8ffc, 0x8ffc, PDF_CMAP_SINGLE, 16193 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 14603 },
	{ 0x8ffe, 0x8ffe, PDF_CMAP_SINGLE, 14560 },
	{ 0x9040, 0x9040, PDF_CMAP_SINGLE, 17021 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 17136 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 16167 },
	{ 0x9043, 0x9043, PDF_CMAP_SINGLE, 14370 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 14296 },
	{ 0x9045, 0x9045, PDF_CMAP_SINGLE, 14242 },
	{ 0x9046, 0x9046, PDF_CMAP_SINGLE, 16243 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 17080 },
	{ 0x9048, 0x9048, PDF_CMAP_SINGLE, 16153 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 17079 },
	{ 0x904a, 0x904a, PDF_CMAP_SINGLE, 15755 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 16681 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 16596 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 14581 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 16707 },
	{ 0x904f, 0x904f, PDF_CMAP_SINGLE, 16418 },
	{ 0x9050, 0x9050, PDF_CMAP_SINGLE, 16848 },
	{ 0x9051, 0x9051, PDF_CMAP_SINGLE, 16885 },
	{ 0x9052, 0x9052, PDF_CMAP_SINGLE, 16195 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 16584 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 16534 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 16477 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 16756 },
	{ 0x9057, 0x9057, PDF_CMAP_SINGLE, 16702 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 16699 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 14788 },
	{ 0x905a, 0x905a, PDF_CMAP_SINGLE, 15753 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 16901 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 14059 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 14789 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 15748 },
	{ 0x905f, 0x905f, PDF_CMAP_SINGLE, 16468 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 6463 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 1561 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 2161 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 4447 },
	{ 0x9064, 0x9064, PDF_CMAP_SINGLE, 16211 },
	{ 0x9065, 0x9065, PDF_CMAP_SINGLE, 16159 },
	{ 0x9066, 0x9066, PDF_CMAP_SINGLE, 13213 },
	{ 0x9067, 0x9067, PDF_CMAP_SINGLE, 16312 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 15167 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 16210 },
	{ 0x906a, 0x906a, PDF_CMAP_SINGLE, 14177 },
	{ 0x906b, 0x906b, PDF_CMAP_SINGLE, 14172 },
	{ 0x906c, 0x906c, PDF_CMAP_SINGLE, 16345 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 16343 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 16342 },
	{ 0x906f, 0x906f, PDF_CMAP_SINGLE, 16019 },
	{ 0x9070, 0x9070, PDF_CMAP_SINGLE, 14196 },
	{ 0x9071, 0x9071, PDF_CMAP_SINGLE, 14203 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 15183 },
	{ 0x9073, 0x9073, PDF_CMAP_SINGLE, 16391 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 16354 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 15993 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 15990 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 16359 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 3370 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 16366 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 16353 },
	{ 0x907b, 0x907b, PDF_CMAP_SINGLE, 15991 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 16291 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 16212 },
	{ 0x907e, 0x907e, PDF_CMAP_SINGLE, 14268 },
	{ 0x90a1, 0x90a1, PDF_CMAP_SINGLE, 15994 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 16289 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 14273 },
	{ 0x90a4, 0x90a4, PDF_CMAP_SINGLE, 16373 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 16286 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 15231 },
	{ 0x90a7, 0x90a7, PDF_CMAP_SINGLE, 14347 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 16367 },
	{ 0x90a9, 0x90a9, PDF_CMAP_SINGLE, 16380 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 14349 },
	{ 0x90ab, 0x90ab, PDF_CMAP_SINGLE, 16365 },
	{ 0x90ac, 0x90ac, PDF_CMAP_SINGLE, 16356 },
	{ 0x90ad, 0x90ad, PDF_CMAP_SINGLE, 14353 },
	{ 0x90ae, 0x90ae, PDF_CMAP_SINGLE, 16250 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 16290 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 15246 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 14364 },
	{ 0x90b2, 0x90b2, PDF_CMAP_SINGLE, 16288 },
	{ 0x90b3, 0x90b3, PDF_CMAP_SINGLE, 16352 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 15763 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 14375 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 16368 },
	{ 0x90b7, 0x90b7, PDF_CMAP_SINGLE, 16293 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 14390 },
	{ 0x90b9, 0x90b9, PDF_CMAP_SINGLE, 15757 },
	{ 0x90ba, 0x90ba, PDF_CMAP_SINGLE, 16233 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 16258 },
	{ 0x90bc, 0x90bc, PDF_CMAP_SINGLE, 16261 },
	{ 0x90bd, 0x90bd, PDF_CMAP_SINGLE, 16259 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 16265 },
	{ 0x90bf, 0x90bf, PDF_CMAP_SINGLE, 16255 },
	{ 0x90c0, 0x90c0, PDF_CMAP_SINGLE, 15285 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 16263 },
	{ 0x90c2, 0x90c2, PDF_CMAP_SINGLE, 16177 },
	{ 0x90c3, 0x90c3, PDF_CMAP_SINGLE, 15764 },
	{ 0x90c4, 0x90c4, PDF_CMAP_SINGLE, 560 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 17406 },
	{ 0x90c6, 0x90c6, PDF_CMAP_SINGLE, 16144 },
	{ 0x90c7, 0x90c7, PDF_CMAP_SINGLE, 14462 },
	{ 0x90c8, 0x90c8, PDF_CMAP_SINGLE, 17387 },
	{ 0x90c9, 0x90c9, PDF_CMAP_SINGLE, 16239 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 16027 },
	{ 0x90cb, 0x90cb, PDF_CMAP_SINGLE, 16206 },
	{ 0x90cc, 0x90cc, PDF_CMAP_SINGLE, 15304 },
	{ 0x90cd, 0x90cd, PDF_CMAP_SINGLE, 7721 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 16200 },
	{ 0x90cf, 0x90cf, PDF_CMAP_SINGLE, 14382 },
	{ 0x90d0, 0x90d0, PDF_CMAP_SINGLE, 15456 },
	{ 0x90d1, 0x90d1, PDF_CMAP_SINGLE, 10911 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 14483 },
	{ 0x90d3, 0x90d3, PDF_CMAP_SINGLE, 17399 },
	{ 0x90d4, 0x90d4, PDF_CMAP_SINGLE, 15988 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 15309 },
	{ 0x90d6, 0x90d6, PDF_CMAP_SINGLE, 15049 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 16332 },
	{ 0x90d8, 0x90d8, PDF_CMAP_SINGLE, 16337 },
	{ 0x90d9, 0x90d9, PDF_CMAP_SINGLE, 16189 },
	{ 0x90da, 0x90da, PDF_CMAP_SINGLE, 16202 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 14509 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 9042 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 16198 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 14514 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 16307 },
	{ 0x90e0, 0x90e0, PDF_CMAP_SINGLE, 16141 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 14528 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 15919 },
	{ 0x90e3, 0x90e3, PDF_CMAP_SINGLE, 14549 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 16181 },
	{ 0x90e5, 0x90e5, PDF_CMAP_SINGLE, 14551 },
	{ 0x90e6, 0x90e6, PDF_CMAP_SINGLE, 16175 },
	{ 0x90e7, 0x90e7, PDF_CMAP_SINGLE, 16179 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 15765 },
	{ 0x90e9, 0x90e9, PDF_CMAP_SINGLE, 15362 },
	{ 0x90ea, 0x90ea, PDF_CMAP_SINGLE, 16242 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 15766 },
	{ 0x90ec, 0x90ec, PDF_CMAP_SINGLE, 14567 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 16437 },
	{ 0x90ee, 0x90ee, PDF_CMAP_SINGLE, 15371 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 14576 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 14583 },
	{ 0x90f1, 0x90f1, PDF_CMAP_SINGLE, 16313 },
	{ 0x90f2, 0x90f2, PDF_CMAP_SINGLE, 14592 },
	{ 0x90f3, 0x90f3, PDF_CMAP_SINGLE, 16244 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 14601 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 16146 },
	{ 0x90f6, 0x90f6, PDF_CMAP_SINGLE, 16151 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 16160 },
	{ 0x90f8, 0x90f8, PDF_CMAP_SINGLE, 16315 },
	{ 0x90f9, 0x90f9, PDF_CMAP_SINGLE, 16188 },
	{ 0x90fa, 0x90fa, PDF_CMAP_SINGLE, 16308 },
	{ 0x90fb, 0x90fb, PDF_CMAP_SINGLE, 16301 },
	{ 0x90fc, 0x90fc, PDF_CMAP_SINGLE, 14618 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 14620 },
	{ 0x90fe, 0x90fe, PDF_CMAP_SINGLE, 14619 },
	{ 0x9140, 0x9140, PDF_CMAP_SINGLE, 15417 },
	{ 0x9141, 0x9141, PDF_CMAP_SINGLE, 15985 },
	{ 0x9142, 0x9142, PDF_CMAP_SINGLE, 15742 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 16187 },
	{ 0x9144, 0x9144, PDF_CMAP_SINGLE, 16176 },
	{ 0x9145, 0x9145, PDF_CMAP_SINGLE, 16260 },
	{ 0x9146, 0x9146, PDF_CMAP_SINGLE, 17400 },
	{ 0x9147, 0x9147, PDF_CMAP_SINGLE, 16143 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 16246 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 14643 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 16245 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 14656 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 16161 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 16272 },
	{ 0x914e, 0x914e, PDF_CMAP_SINGLE, 14666 },
	{ 0x914f, 0x914f, PDF_CMAP_SINGLE, 16256 },
	{ 0x9150, 0x9150, PDF_CMAP_SINGLE, 16163 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 14672 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 15767 },
	{ 0x9153, 0x9153, PDF_CMAP_SINGLE, 11845 },
	{ 0x9154, 0x9154, PDF_CMAP_SINGLE, 1392 },
	{ 0x9155, 0x9155, PDF_CMAP_SINGLE, 16045 },
	{ 0x9156, 0x9156, PDF_CMAP_SINGLE, 13602 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 3043 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 1429 },
	{ 0x9159, 0x9159, PDF_CMAP_SINGLE, 552 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 2709 },
	{ 0x915b, 0x915b, PDF_CMAP_SINGLE, 4784 },
	{ 0x915c, 0x915c, PDF_CMAP_SINGLE, 2330 },
	{ 0x915d, 0x915d, PDF_CMAP_SINGLE, 17404 },
	{ 0x915e, 0x915e, PDF_CMAP_SINGLE, 16274 },
	{ 0x915f, 0x915f, PDF_CMAP_SINGLE, 16302 },
	{ 0x9160, 0x9160, PDF_CMAP_SINGLE, 1986 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 2389 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 14213 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 15982 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 15725 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 16226 },
	{ 0x9166, 0x9166, PDF_CMAP_SINGLE, 14792 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 16229 },
	{ 0x9168, 0x9168, PDF_CMAP_SINGLE, 14653 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 16166 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 16493 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 16336 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 16182 },
	{ 0x916d, 0x916d, PDF_CMAP_SINGLE, 16518 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 14674 },
	{ 0x916f, 0x916f, PDF_CMAP_SINGLE, 16338 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 14424 },
	{ 0x9171, 0x9171, PDF_CMAP_SINGLE, 16379 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 16231 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 16001 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 16329 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 16309 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 15996 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 15989 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 15479 },
	{ 0x9179, 0x9179, PDF_CMAP_SINGLE, 15150 },
	{ 0x917a, 0x917a, PDF_CMAP_SINGLE, 16205 },
	{ 0x917b, 0x917b, PDF_CMAP_SINGLE, 17327 },
	{ 0x917c, 0x917c, PDF_CMAP_SINGLE, 15975 },
	{ 0x917d, 0x917d, PDF_CMAP_SINGLE, 16573 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 16236 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 14758 },
	{ 0x91a2, 0x91a2, PDF_CMAP_SINGLE, 14664 },
	{ 0x91a3, 0x91a3, PDF_CMAP_SINGLE, 16319 },
	{ 0x91a4, 0x91a4, PDF_CMAP_SINGLE, 16223 },
	{ 0x91a5, 0x91a5, PDF_CMAP_SINGLE, 14853 },
	{ 0x91a6, 0x91a6, PDF_CMAP_SINGLE, 1781 },
	{ 0x91a7, 0x91a7, PDF_CMAP_SINGLE, 16632 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 16813 },
	{ 0x91a9, 0x91a9, PDF_CMAP_SINGLE, 16142 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 16674 },
	{ 0x91ab, 0x91ab, PDF_CMAP_SINGLE, 16147 },
	{ 0x91ac, 0x91ac, PDF_CMAP_SINGLE, 16139 },
	{ 0x91ad, 0x91ad, PDF_CMAP_SINGLE, 16693 },
	{ 0x91ae, 0x91ae, PDF_CMAP_SINGLE, 14891 },
	{ 0x91af, 0x91af, PDF_CMAP_SINGLE, 16696 },
	{ 0x91b0, 0x91b0, PDF_CMAP_SINGLE, 550 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 16713 },
	{ 0x91b2, 0x91b2, PDF_CMAP_SINGLE, 15269 },
	{ 0x91b3, 0x91b3, PDF_CMAP_SINGLE, 16385 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 16317 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 16296 },
	{ 0x91b6, 0x91b6, PDF_CMAP_SINGLE, 16774 },
	{ 0x91b7, 0x91b7, PDF_CMAP_SINGLE, 14932 },
	{ 0x91b8, 0x91b8, PDF_CMAP_SINGLE, 16322 },
	{ 0x91b9, 0x91b9, PDF_CMAP_SINGLE, 16786 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 16775 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 13188 },
	{ 0x91bc, 0x91bc, PDF_CMAP_SINGLE, 16324 },
	{ 0x91bd, 0x91bd, PDF_CMAP_SINGLE, 15769 },
	{ 0x91be, 0x91be, PDF_CMAP_SINGLE, 15018 },
	{ 0x91bf, 0x91c0, PDF_CMAP_RANGE, 16862 },
	{ 0x91c1, 0x91c1, PDF_CMAP_SINGLE, 16374 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 16221 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 16839 },
	{ 0x91c4, 0x91c4, PDF_CMAP_SINGLE, 15761 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 15732 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 16204 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 16882 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 15013 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 15179 },
	{ 0x91ca, 0x91ca, PDF_CMAP_SINGLE, 15750 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 16888 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 16960 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 16381 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 16295 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 16985 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 3698 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 16994 },
	{ 0x91d2, 0x91d2, PDF_CMAP_SINGLE, 15486 },
	{ 0x91d3, 0x91d3, PDF_CMAP_SINGLE, 14648 },
	{ 0x91d4, 0x91d4, PDF_CMAP_SINGLE, 16947 },
	{ 0x91d5, 0x91d5, PDF_CMAP_SINGLE, 10586 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 16279 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 16306 },
	{ 0x91d9, 0x91d9, PDF_CMAP_SINGLE, 17034 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 16280 },
	{ 0x91db, 0x91db, PDF_CMAP_SINGLE, 8084 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 16192 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 16207 },
	{ 0x91de, 0x91de, PDF_CMAP_SINGLE, 16271 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 15727 },
	{ 0x91e0, 0x91e0, PDF_CMAP_SINGLE, 17119 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 16213 },
	{ 0x91e2, 0x91e2, PDF_CMAP_SINGLE, 16339 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 17134 },
	{ 0x91e4, 0x91e4, PDF_CMAP_SINGLE, 16335 },
	{ 0x91e5, 0x91e5, PDF_CMAP_SINGLE, 16340 },
	{ 0x91e6, 0x91e6, PDF_CMAP_SINGLE, 15997 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 17159 },
	{ 0x91e8, 0x91e8, PDF_CMAP_SINGLE, 17403 },
	{ 0x91e9, 0x91e9, PDF_CMAP_SINGLE, 16346 },
	{ 0x91ea, 0x91ea, PDF_CMAP_SINGLE, 3338 },
	{ 0x91eb, 0x91eb, PDF_CMAP_SINGLE, 16247 },
	{ 0x91ec, 0x91ec, PDF_CMAP_SINGLE, 15807 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 16428 },
	{ 0x91ee, 0x91ee, PDF_CMAP_SINGLE, 16292 },
	{ 0x91ef, 0x91ef, PDF_CMAP_SINGLE, 15727 },
	{ 0x91f0, 0x91f0, PDF_CMAP_SINGLE, 15565 },
	{ 0x91f1, 0x91f1, PDF_CMAP_SINGLE, 16228 },
	{ 0x91f2, 0x91f2, PDF_CMAP_SINGLE, 16267 },
	{ 0x91f3, 0x91f3, PDF_CMAP_SINGLE, 14447 },
	{ 0x91f4, 0x91f4, PDF_CMAP_SINGLE, 6513 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 7323 },
	{ 0x91f6, 0x91f6, PDF_CMAP_SINGLE, 1861 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 7541 },
	{ 0x91f8, 0x91f8, PDF_CMAP_SINGLE, 14472 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 14146 },
	{ 0x91fa, 0x91fa, PDF_CMAP_SINGLE, 11072 },
	{ 0x91fb, 0x91fb, PDF_CMAP_SINGLE, 14057 },
	{ 0x91fc, 0x91fc, PDF_CMAP_SINGLE, 14304 },
	{ 0x91fd, 0x91fd, PDF_CMAP_SINGLE, 15894 },
	{ 0x91fe, 0x91fe, PDF_CMAP_SINGLE, 14460 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 14250 },
	{ 0x9241, 0x9241, PDF_CMAP_SINGLE, 16270 },
	{ 0x9242, 0x9242, PDF_CMAP_SINGLE, 16604 },
	{ 0x9243, 0x9243, PDF_CMAP_SINGLE, 15482 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 16294 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 16479 },
	{ 0x9246, 0x9246, PDF_CMAP_SINGLE, 15398 },
	{ 0x9247, 0x9247, PDF_CMAP_SINGLE, 14430 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 16165 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 16456 },
	{ 0x924a, 0x924a, PDF_CMAP_SINGLE, 15745 },
	{ 0x924b, 0x924b, PDF_CMAP_SINGLE, 14606 },
	{ 0x924c, 0x924c, PDF_CMAP_SINGLE, 16516 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 16664 },
	{ 0x924e, 0x924e, PDF_CMAP_SINGLE, 16727 },
	{ 0x924f, 0x924f, PDF_CMAP_SINGLE, 14594 },
	{ 0x9250, 0x9250, PDF_CMAP_SINGLE, 14495 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 16377 },
	{ 0x9252, 0x9252, PDF_CMAP_SINGLE, 14371 },
	{ 0x9253, 0x9253, PDF_CMAP_SINGLE, 14449 },
	{ 0x9254, 0x9254, PDF_CMAP_SINGLE, 14058 },
	{ 0x9255, 0x9255, PDF_CMAP_SINGLE, 14188 },
	{ 0x9256, 0x9256, PDF_CMAP_SINGLE, 14144 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 14324 },
	{ 0x9258, 0x9258, PDF_CMAP_SINGLE, 17120 },
	{ 0x9259, 0x9259, PDF_CMAP_SINGLE, 16987 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 14061 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 15124 },
	{ 0x925c, 0x925c, PDF_CMAP_SINGLE, 17093 },
	{ 0x925d, 0x925d, PDF_CMAP_SINGLE, 16523 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 14501 },
	{ 0x925f, 0x925f, PDF_CMAP_SINGLE, 16746 },
	{ 0x9260, 0x9260, PDF_CMAP_SINGLE, 16797 },
	{ 0x9261, 0x9261, PDF_CMAP_SINGLE, 16708 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 16975 },
	{ 0x9263, 0x9263, PDF_CMAP_SINGLE, 16225 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 16344 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 1564 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f6, PDF_CMAP_SINGLE, 248 },
	{ 0xa1f7, 0xa1f7, PDF_CMAP_SINGLE, 247 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3b9, PDF_CMAP_SINGLE, 5551 },
	{ 0xc3ba, 0xc3ba, PDF_CMAP_SINGLE, 5550 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c4, PDF_CMAP_SINGLE, 13585 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 13629 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
};

static const unsigned short pdf_cmap_HKdlb_B5_H_table[668] =
{
	15735,15724,16217,14617,15984,14162,14511,14214,
	16155,15708,16281,16360,14475,14556,16298,17096,
	14145,15743,17132,16275,14490,14457,14138,17104,
	15983,16152,14056,14137,16025,15180,15826,16297,
	16300,16331,14131,16287,16602,16169,15462,16930,
	16558,15738,16269,14800,16996,15746,16899,16859,
	16350,16372,16388,16230,16283,15718,16459,16351,
	16635,16402,16358,17131,16676,15740,15721,16382,
	15730,16326,15614,17401,15682,16149,16334,17035,
	16591,17398,16389,16173,16737,16172,15050,15722,
	16483,16325,14823,16170,16480,17011,16929,16546,
	16232,16545,16215,16240,16311,14150,16299,16370,
	16268,16264,14291,17140,15436,15226,14469,14135,
	16347,16157,16364,16357,14174,16375,14355,16355,
	16154,14566,16348,14207,15751,15315,15736,15316,
	16278,16277,15756,14647,16156,14642,15723,14336,
	16376,15418,16194,17405,16305,14226,16276,16158,
	16394,14466,16284,15713,16162,16361,15762,14207,
	15252,14190,17153,16186,14435,16369,17150,15356,
	16197,14239,14187,16148,16199,14434,14185,15378,
	16201,14367,14633,16214,16180,16349,14586,16251,
	15377,16252,17407,15987,16241,14629,15445,14517,
	16218,14358,15981,14210,16145,15243,14272,14357,
	14395,15731,15726,16266,16190,16948,16867,16327,
	16371,16387,16434,16238,16803,16390,16171,16253,
	15475,16150,16386,16470,16314,16235,16273,16318,
	16474,15071,15937,15719,16224,16872,16320,15747,
	16755,16220,16178,16191,15715,16710,16333,15737,
	16168,16321,16572,15729,16183,17112,16363,16961,
	14896,14797,17087,16910,16138,14796,16452,16536,
	16780,15716,16196,16463,17109,17040,16384,14985,
	16748,16580,16461,15808,16208,16393,16303,16262,
	16531,15021,16184,15733,16185,17101,16248,16285,
	16528,15744,16174,14518,16825,15754,16164,15006,
	15068,15717,16517,16209,17402,16227,16760,16823,
	15986,16857,16782,15734,17108,17058,16983,16140,
	14742,16981,16567,16282,16330,16222,15749,16378,
	16310,15043,16328,16203,16625,16436,16219,17097,
	15477,16249,17094,14543,16884,16193,14603,14560,
	17021,17136,16167,14370,14296,14242,16243,17080,
	16153,17079,15755,16681,16596,14581,16707,16418,
	16848,16885,16195,16584,16534,16477,16756,16702,
	16699,14788,15753,16901,14059,14789,15748,16468,
	6463,1561,2161,4447,16211,16159,13213,16312,
	15167,16210,14177,14172,16345,16343,16342,16019,
	14196,14203,15183,16391,16354,15993,15990,16359,
	3370,16366,16353,15991,16291,16212,14268,15994,
	16289,14273,16373,16286,15231,14347,16367,16380,
	14349,16365,16356,14353,16250,16290,15246,14364,
	16288,16352,15763,14375,16368,16293,14390,15757,
	16233,16258,16261,16259,16265,16255,15285,16263,
	16177,15764,560,17406,16144,14462,17387,16239,
	16027,16206,15304,7721,16200,14382,15456,10911,
	14483,17399,15988,15309,15049,16332,16337,16189,
	16202,14509,9042,16198,14514,16307,16141,14528,
	15919,14549,16181,14551,16175,16179,15765,15362,
	16242,15766,14567,16437,15371,14576,14583,16313,
	14592,16244,14601,16146,16151,16160,16315,16188,
	16308,16301,14618,14620,14619,15417,15985,15742,
	16187,16176,16260,17400,16143,16246,14643,16245,
	14656,16161,16272,14666,16256,16163,14672,15767,
	11845,1392,16045,13602,3043,1429,552,2709,
	4784,2330,17404,16274,16302,1986,2389,14213,
	15982,15725,16226,14792,16229,14653,16166,16493,
	16336,16182,16518,14674,16338,14424,16379,16231,
	16001,16329,16309,15996,15989,15479,15150,16205,
	17327,15975,16573,16236,14758,14664,16319,16223,
	14853,1781,16632,16813,16142,16674,16147,16139,
	16693,14891,16696,550,16713,15269,16385,16317,
	16296,16774,14932,16322,16786,16775,13188,16324,
	15769,15018,16374,16221,16839,15761,15732,16204,
	16882,15013,15179,15750,16888,16960,16381,16295,
	16985,3698,16994,15486,14648,16947,10586,16279,
	16306,17034,16280,8084,16192,16207,16271,15727,
	17119,16213,16339,17134,16335,16340,15997,17159,
	17403,16346,3338,16247,15807,16428,16292,15727,
	15565,16228,16267,14447,6513,7323,1861,7541,
	14472,14146,11072,14057,14304,15894,14460,14250,
	16270,16604,15482,16294,16479,15398,14430,16165,
	16456,15745,14606,16516,16664,16727,14594,14495,
	16377,14371,14449,14058,14188,14144,14324,17120,
	16987,14061,15124,17093,16523,14501,16746,16797,
	16708,16975,16225,16344,1564,248,247,5551,
	5550,13585,13629,13641,
};

pdf_cmap pdf_cmap_HKdlb_B5_H =
{
	-1, "HKdlb-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8e40, 0xfefe },
	},
	907, 907, (pdf_range*) pdf_cmap_HKdlb_B5_H_ranges,
	668, 668, (unsigned short*) pdf_cmap_HKdlb_B5_H_table,
};

/* HKdlb-B5-V */

static const pdf_range pdf_cmap_HKdlb_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_HKdlb_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_HKdlb_B5_V =
{
	-1, "HKdlb-B5-V", "HKdlb-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_HKdlb_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_HKdlb_B5_V_table,
};

/* HKgccs-B5-H */

static const pdf_range pdf_cmap_HKgccs_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8a40, 0x8a7e, PDF_CMAP_RANGE, 17160 },
	{ 0x8aa1, 0x8ae5, PDF_CMAP_RANGE, 17223 },
	{ 0x8b41, 0x8b42, PDF_CMAP_RANGE, 17292 },
	{ 0x8b45, 0x8b45, PDF_CMAP_SINGLE, 17294 },
	{ 0x8b47, 0x8b47, PDF_CMAP_SINGLE, 17295 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 17296 },
	{ 0x8b4b, 0x8b4b, PDF_CMAP_SINGLE, 17297 },
	{ 0x8b4d, 0x8b50, PDF_CMAP_RANGE, 17298 },
	{ 0x8b54, 0x8b54, PDF_CMAP_SINGLE, 17302 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 17303 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 17304 },
	{ 0x8e40, 0x8e44, PDF_CMAP_RANGE, 14123 },
	{ 0x8e46, 0x8e6a, PDF_CMAP_RANGE, 14128 },
	{ 0x8e6c, 0x8e75, PDF_CMAP_RANGE, 14165 },
	{ 0x8e77, 0x8e7d, PDF_CMAP_RANGE, 14175 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 289 },
	{ 0x8ea1, 0x8ea5, PDF_CMAP_RANGE, 14182 },
	{ 0x8ea7, 0x8ec8, PDF_CMAP_RANGE, 14187 },
	{ 0x8eca, 0x8ee4, PDF_CMAP_RANGE, 14221 },
	{ 0x8ee6, 0x8eee, PDF_CMAP_RANGE, 14248 },
	{ 0x8ef0, 0x8efe, PDF_CMAP_RANGE, 14257 },
	{ 0x8f40, 0x8f58, PDF_CMAP_RANGE, 14272 },
	{ 0x8f5a, 0x8f5e, PDF_CMAP_RANGE, 14297 },
	{ 0x8f60, 0x8f66, PDF_CMAP_RANGE, 14302 },
	{ 0x8f68, 0x8f7e, PDF_CMAP_RANGE, 14309 },
	{ 0x8fa1, 0x8fc4, PDF_CMAP_RANGE, 14332 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 14368 },
	{ 0x8fc8, 0x8fd9, PDF_CMAP_RANGE, 14369 },
	{ 0x8fdb, 0x8ffb, PDF_CMAP_RANGE, 14387 },
	{ 0x8ffd, 0x8ffe, PDF_CMAP_RANGE, 14420 },
	{ 0x9040, 0x905b, PDF_CMAP_RANGE, 14422 },
	{ 0x905e, 0x906e, PDF_CMAP_RANGE, 14450 },
	{ 0x9070, 0x9079, PDF_CMAP_RANGE, 14467 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 14057 },
	{ 0x907b, 0x907e, PDF_CMAP_RANGE, 14477 },
	{ 0x90a1, 0x90a5, PDF_CMAP_RANGE, 14481 },
	{ 0x90a7, 0x90b7, PDF_CMAP_RANGE, 14486 },
	{ 0x90b9, 0x90fe, PDF_CMAP_RANGE, 14503 },
	{ 0x9140, 0x9164, PDF_CMAP_RANGE, 14573 },
	{ 0x9166, 0x916d, PDF_CMAP_RANGE, 14610 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 14618 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 14633 },
	{ 0x91a3, 0x91fe, PDF_CMAP_RANGE, 14634 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 14726 },
	{ 0x92a1, 0x92ae, PDF_CMAP_RANGE, 14789 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 281 },
	{ 0x92b1, 0x92b2, PDF_CMAP_TABLE, 0 },
	{ 0x92b3, 0x92e4, PDF_CMAP_RANGE, 14803 },
	{ 0x92e6, 0x92f1, PDF_CMAP_RANGE, 14853 },
	{ 0x92f3, 0x92fe, PDF_CMAP_RANGE, 14865 },
	{ 0x9340, 0x9367, PDF_CMAP_RANGE, 14877 },
	{ 0x9369, 0x937e, PDF_CMAP_RANGE, 14917 },
	{ 0x93a1, 0x93a9, PDF_CMAP_RANGE, 14939 },
	{ 0x93ab, 0x93c1, PDF_CMAP_RANGE, 14948 },
	{ 0x93c3, 0x93e4, PDF_CMAP_RANGE, 14971 },
	{ 0x93e6, 0x93ea, PDF_CMAP_RANGE, 15005 },
	{ 0x93ec, 0x93fe, PDF_CMAP_RANGE, 15010 },
	{ 0x9440, 0x9445, PDF_CMAP_RANGE, 15029 },
	{ 0x9447, 0x947e, PDF_CMAP_RANGE, 15035 },
	{ 0x94a1, 0x94fe, PDF_CMAP_RANGE, 15091 },
	{ 0x9540, 0x954c, PDF_CMAP_RANGE, 15185 },
	{ 0x954e, 0x9559, PDF_CMAP_RANGE, 15198 },
	{ 0x955b, 0x957e, PDF_CMAP_RANGE, 15210 },
	{ 0x95a1, 0x95c5, PDF_CMAP_RANGE, 15246 },
	{ 0x95c7, 0x95fe, PDF_CMAP_RANGE, 15283 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 15339 },
	{ 0x96a1, 0x96d3, PDF_CMAP_RANGE, 15402 },
	{ 0x96d5, 0x96fe, PDF_CMAP_RANGE, 15453 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 15495 },
	{ 0x97a1, 0x97fe, PDF_CMAP_RANGE, 15558 },
	{ 0x9840, 0x9874, PDF_CMAP_RANGE, 15652 },
	{ 0x9877, 0x9877, PDF_CMAP_SINGLE, 15705 },
	{ 0x987a, 0x987a, PDF_CMAP_SINGLE, 15706 },
	{ 0x98a3, 0x98a3, PDF_CMAP_SINGLE, 15707 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 15708 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 15709 },
	{ 0x98b9, 0x98b9, PDF_CMAP_SINGLE, 15710 },
	{ 0x98bd, 0x98be, PDF_CMAP_RANGE, 15711 },
	{ 0x98c2, 0x98c2, PDF_CMAP_SINGLE, 15713 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 15714 },
	{ 0x98c6, 0x98c7, PDF_CMAP_RANGE, 15715 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 15717 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 15718 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 15719 },
	{ 0x98f0, 0x98f0, PDF_CMAP_SINGLE, 15720 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 15721 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 15722 },
	{ 0x9943, 0x9943, PDF_CMAP_SINGLE, 15723 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 15724 },
	{ 0x994f, 0x994f, PDF_CMAP_SINGLE, 15725 },
	{ 0x996a, 0x996a, PDF_CMAP_SINGLE, 15726 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 15727 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 15728 },
	{ 0x9978, 0x9978, PDF_CMAP_SINGLE, 15729 },
	{ 0x99a2, 0x99a2, PDF_CMAP_SINGLE, 15730 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 15731 },
	{ 0x99b6, 0x99b6, PDF_CMAP_SINGLE, 15732 },
	{ 0x99ba, 0x99ba, PDF_CMAP_SINGLE, 15733 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 15734 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 15735 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 15736 },
	{ 0x9a4c, 0x9a4c, PDF_CMAP_SINGLE, 15737 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 15738 },
	{ 0x9a61, 0x9a61, PDF_CMAP_SINGLE, 15739 },
	{ 0x9a68, 0x9a68, PDF_CMAP_SINGLE, 15740 },
	{ 0x9a73, 0x9a73, PDF_CMAP_SINGLE, 15741 },
	{ 0x9a7e, 0x9a7e, PDF_CMAP_SINGLE, 15742 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 15743 },
	{ 0x9ab7, 0x9ab7, PDF_CMAP_SINGLE, 15744 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 15745 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 15746 },
	{ 0x9ac7, 0x9ac7, PDF_CMAP_SINGLE, 15747 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 15748 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 15749 },
	{ 0x9ad9, 0x9adb, PDF_CMAP_RANGE, 15750 },
	{ 0x9ae2, 0x9ae2, PDF_CMAP_SINGLE, 15753 },
	{ 0x9ae4, 0x9ae4, PDF_CMAP_SINGLE, 15754 },
	{ 0x9ae8, 0x9ae8, PDF_CMAP_SINGLE, 15755 },
	{ 0x9af2, 0x9af2, PDF_CMAP_SINGLE, 15756 },
	{ 0x9af6, 0x9af6, PDF_CMAP_SINGLE, 15757 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 15758 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 15759 },
	{ 0x9b4a, 0x9b4a, PDF_CMAP_SINGLE, 15760 },
	{ 0x9b4c, 0x9b4c, PDF_CMAP_SINGLE, 15761 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 15762 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 15763 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 15764 },
	{ 0x9b5c, 0x9b5c, PDF_CMAP_SINGLE, 15765 },
	{ 0x9b5e, 0x9b5f, PDF_CMAP_RANGE, 15766 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 15768 },
	{ 0x9b76, 0x9b78, PDF_CMAP_RANGE, 15772 },
	{ 0x9b7b, 0x9b7c, PDF_CMAP_RANGE, 15775 },
	{ 0x9b7e, 0x9b7e, PDF_CMAP_SINGLE, 15777 },
	{ 0x9ba1, 0x9ba1, PDF_CMAP_SINGLE, 15778 },
	{ 0x9ba3, 0x9ba4, PDF_CMAP_RANGE, 15779 },
	{ 0x9ba7, 0x9baa, PDF_CMAP_RANGE, 15781 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 15785 },
	{ 0x9baf, 0x9baf, PDF_CMAP_SINGLE, 15786 },
	{ 0x9bb2, 0x9bb9, PDF_CMAP_RANGE, 15787 },
	{ 0x9bbe, 0x9bbe, PDF_CMAP_SINGLE, 15795 },
	{ 0x9bc0, 0x9bc6, PDF_CMAP_RANGE, 15796 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 15803 },
	{ 0x9bcc, 0x9bcc, PDF_CMAP_SINGLE, 15804 },
	{ 0x9bd0, 0x9bd1, PDF_CMAP_RANGE, 15805 },
	{ 0x9bd3, 0x9bd3, PDF_CMAP_SINGLE, 15807 },
	{ 0x9bd5, 0x9bd5, PDF_CMAP_SINGLE, 15808 },
	{ 0x9bd8, 0x9bda, PDF_CMAP_RANGE, 15809 },
	{ 0x9bdd, 0x9bdf, PDF_CMAP_TABLE, 2 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 15814 },
	{ 0x9be3, 0x9be3, PDF_CMAP_SINGLE, 15815 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 15816 },
	{ 0x9be9, 0x9bec, PDF_CMAP_RANGE, 15817 },
	{ 0x9bee, 0x9bef, PDF_CMAP_RANGE, 15821 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 15823 },
	{ 0x9bf6, 0x9bf6, PDF_CMAP_SINGLE, 15824 },
	{ 0x9bf8, 0x9bf9, PDF_CMAP_RANGE, 15825 },
	{ 0x9bfb, 0x9bfc, PDF_CMAP_RANGE, 15827 },
	{ 0x9c40, 0x9c42, PDF_CMAP_RANGE, 15829 },
	{ 0x9c44, 0x9c46, PDF_CMAP_RANGE, 15832 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 15835 },
	{ 0x9c4a, 0x9c4a, PDF_CMAP_SINGLE, 15836 },
	{ 0x9c4d, 0x9c53, PDF_CMAP_RANGE, 15837 },
	{ 0x9c55, 0x9c55, PDF_CMAP_SINGLE, 15844 },
	{ 0x9c57, 0x9c5b, PDF_CMAP_RANGE, 15845 },
	{ 0x9c5d, 0x9c5d, PDF_CMAP_SINGLE, 15850 },
	{ 0x9c60, 0x9c60, PDF_CMAP_SINGLE, 15851 },
	{ 0x9c62, 0x9c62, PDF_CMAP_SINGLE, 15852 },
	{ 0x9c64, 0x9c66, PDF_CMAP_RANGE, 15853 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 15856 },
	{ 0x9c6a, 0x9c6b, PDF_CMAP_TABLE, 5 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 15858 },
	{ 0x9c6f, 0x9c72, PDF_CMAP_RANGE, 15859 },
	{ 0x9c75, 0x9c77, PDF_CMAP_RANGE, 15863 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 15866 },
	{ 0x9c7b, 0x9c7c, PDF_CMAP_RANGE, 15867 },
	{ 0x9c7e, 0x9c7e, PDF_CMAP_SINGLE, 15869 },
	{ 0x9ca1, 0x9ca2, PDF_CMAP_RANGE, 15870 },
	{ 0x9ca5, 0x9ca5, PDF_CMAP_SINGLE, 15872 },
	{ 0x9ca8, 0x9ca9, PDF_CMAP_RANGE, 15873 },
	{ 0x9cab, 0x9cab, PDF_CMAP_SINGLE, 15875 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 15876 },
	{ 0x9cb1, 0x9cba, PDF_CMAP_RANGE, 15878 },
	{ 0x9cbc, 0x9cc2, PDF_CMAP_RANGE, 15888 },
	{ 0x9cc6, 0x9ccd, PDF_CMAP_RANGE, 15895 },
	{ 0x9ccf, 0x9cd3, PDF_CMAP_RANGE, 15903 },
	{ 0x9cd8, 0x9cda, PDF_CMAP_RANGE, 15908 },
	{ 0x9cdc, 0x9ce5, PDF_CMAP_RANGE, 15911 },
	{ 0x9ce7, 0x9ce9, PDF_CMAP_RANGE, 15921 },
	{ 0x9ceb, 0x9cec, PDF_CMAP_RANGE, 15924 },
	{ 0x9cee, 0x9cf9, PDF_CMAP_RANGE, 15926 },
	{ 0x9cfd, 0x9cfd, PDF_CMAP_SINGLE, 15938 },
	{ 0x9d46, 0x9d46, PDF_CMAP_SINGLE, 15939 },
	{ 0x9d49, 0x9d4a, PDF_CMAP_RANGE, 15940 },
	{ 0x9d4c, 0x9d4d, PDF_CMAP_RANGE, 15942 },
	{ 0x9d4f, 0x9d4f, PDF_CMAP_SINGLE, 15944 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 15945 },
	{ 0x9d55, 0x9d55, PDF_CMAP_SINGLE, 15946 },
	{ 0x9d57, 0x9d57, PDF_CMAP_SINGLE, 9665 },
	{ 0x9d5a, 0x9d5a, PDF_CMAP_SINGLE, 15947 },
	{ 0x9d62, 0x9d62, PDF_CMAP_SINGLE, 15948 },
	{ 0x9d64, 0x9d64, PDF_CMAP_SINGLE, 15949 },
	{ 0x9d79, 0x9d79, PDF_CMAP_SINGLE, 15950 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15951 },
	{ 0x9da5, 0x9da8, PDF_CMAP_RANGE, 15952 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 15956 },
	{ 0x9dac, 0x9dae, PDF_CMAP_RANGE, 15957 },
	{ 0x9db0, 0x9db0, PDF_CMAP_SINGLE, 15960 },
	{ 0x9db3, 0x9db3, PDF_CMAP_SINGLE, 15961 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 15962 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 15963 },
	{ 0x9dbc, 0x9dbd, PDF_CMAP_RANGE, 15964 },
	{ 0x9dbf, 0x9dc0, PDF_CMAP_RANGE, 15966 },
	{ 0x9dc3, 0x9dc4, PDF_CMAP_RANGE, 15968 },
	{ 0x9dc7, 0x9dc8, PDF_CMAP_RANGE, 15970 },
	{ 0x9dca, 0x9dca, PDF_CMAP_SINGLE, 15972 },
	{ 0x9dcd, 0x9dd1, PDF_CMAP_RANGE, 15973 },
	{ 0x9dd3, 0x9dd5, PDF_CMAP_RANGE, 15978 },
	{ 0x9dda, 0x9dfb, PDF_CMAP_RANGE, 15981 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_RANGE, 16015 },
	{ 0x9e40, 0x9e62, PDF_CMAP_RANGE, 16017 },
	{ 0x9e64, 0x9e65, PDF_CMAP_RANGE, 16052 },
	{ 0x9e68, 0x9e68, PDF_CMAP_SINGLE, 16054 },
	{ 0x9e6a, 0x9e6a, PDF_CMAP_SINGLE, 16055 },
	{ 0x9e71, 0x9e71, PDF_CMAP_SINGLE, 16056 },
	{ 0x9e73, 0x9e73, PDF_CMAP_SINGLE, 16057 },
	{ 0x9e77, 0x9e78, PDF_CMAP_RANGE, 16058 },
	{ 0x9e7a, 0x9e7a, PDF_CMAP_SINGLE, 16060 },
	{ 0x9e7c, 0x9e7c, PDF_CMAP_SINGLE, 16061 },
	{ 0x9e7e, 0x9e7e, PDF_CMAP_SINGLE, 16062 },
	{ 0x9ea1, 0x9ea2, PDF_CMAP_RANGE, 16063 },
	{ 0x9ea4, 0x9ea6, PDF_CMAP_RANGE, 16065 },
	{ 0x9ea9, 0x9eaa, PDF_CMAP_RANGE, 16068 },
	{ 0x9eac, 0x9ead, PDF_CMAP_RANGE, 16070 },
	{ 0x9eaf, 0x9eb1, PDF_CMAP_RANGE, 16072 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 16075 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 16076 },
	{ 0x9eb9, 0x9eb9, PDF_CMAP_SINGLE, 16078 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 16079 },
	{ 0x9ebf, 0x9ec0, PDF_CMAP_RANGE, 16080 },
	{ 0x9ec4, 0x9ec5, PDF_CMAP_RANGE, 16082 },
	{ 0x9ec7, 0x9eca, PDF_CMAP_RANGE, 16084 },
	{ 0x9ecc, 0x9ecd, PDF_CMAP_RANGE, 16088 },
	{ 0x9ed0, 0x9ed1, PDF_CMAP_RANGE, 16090 },
	{ 0x9ed3, 0x9ed3, PDF_CMAP_SINGLE, 16092 },
	{ 0x9ed6, 0x9ed7, PDF_CMAP_RANGE, 16093 },
	{ 0x9eda, 0x9ef1, PDF_CMAP_RANGE, 16095 },
	{ 0x9ef3, 0x9ef5, PDF_CMAP_RANGE, 16119 },
	{ 0x9ef9, 0x9efa, PDF_CMAP_RANGE, 16122 },
	{ 0x9efc, 0x9efe, PDF_CMAP_RANGE, 16124 },
	{ 0x9f40, 0x9f42, PDF_CMAP_RANGE, 16127 },
	{ 0x9f44, 0x9f47, PDF_CMAP_RANGE, 16130 },
	{ 0x9f49, 0x9f4a, PDF_CMAP_RANGE, 16134 },
	{ 0x9f4d, 0x9f67, PDF_CMAP_RANGE, 16136 },
	{ 0x9f69, 0x9f6f, PDF_CMAP_RANGE, 16163 },
	{ 0x9f71, 0x9f7e, PDF_CMAP_RANGE, 16170 },
	{ 0x9fa1, 0x9fb4, PDF_CMAP_RANGE, 16184 },
	{ 0x9fb6, 0x9fba, PDF_CMAP_RANGE, 16204 },
	{ 0x9fbc, 0x9fbe, PDF_CMAP_RANGE, 16209 },
	{ 0x9fc0, 0x9fc0, PDF_CMAP_SINGLE, 16212 },
	{ 0x9fc2, 0x9fe3, PDF_CMAP_RANGE, 16213 },
	{ 0x9fe5, 0x9ff8, PDF_CMAP_RANGE, 16247 },
	{ 0x9ffa, 0x9ffe, PDF_CMAP_RANGE, 16267 },
	{ 0xa041, 0xa046, PDF_CMAP_RANGE, 16272 },
	{ 0xa048, 0xa054, PDF_CMAP_RANGE, 16278 },
	{ 0xa056, 0xa06c, PDF_CMAP_RANGE, 16291 },
	{ 0xa06e, 0xa07a, PDF_CMAP_RANGE, 16314 },
	{ 0xa07c, 0xa07e, PDF_CMAP_RANGE, 16327 },
	{ 0xa0a1, 0xa0a1, PDF_CMAP_SINGLE, 16330 },
	{ 0xa0a3, 0xa0a6, PDF_CMAP_RANGE, 16331 },
	{ 0xa0a8, 0xa0c4, PDF_CMAP_RANGE, 16335 },
	{ 0xa0c6, 0xa0cf, PDF_CMAP_RANGE, 16364 },
	{ 0xa0d1, 0xa0e2, PDF_CMAP_RANGE, 16374 },
	{ 0xa0e4, 0xa0e6, PDF_CMAP_RANGE, 16392 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 7 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 9 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 11 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xfa40, 0xfa7e, PDF_CMAP_RANGE, 16395 },
	{ 0xfaa1, 0xfafe, PDF_CMAP_RANGE, 16458 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 16552 },
	{ 0xfba1, 0xfbbe, PDF_CMAP_RANGE, 16615 },
	{ 0xfbc0, 0xfbcc, PDF_CMAP_RANGE, 16645 },
	{ 0xfbce, 0xfbfe, PDF_CMAP_RANGE, 16658 },
	{ 0xfc40, 0xfc49, PDF_CMAP_RANGE, 16707 },
	{ 0xfc4b, 0xfc4e, PDF_CMAP_RANGE, 16717 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 8495 },
	{ 0xfc50, 0xfc51, PDF_CMAP_RANGE, 16721 },
	{ 0xfc53, 0xfc62, PDF_CMAP_RANGE, 16723 },
	{ 0xfc64, 0xfc74, PDF_CMAP_RANGE, 16739 },
	{ 0xfc76, 0xfc7e, PDF_CMAP_RANGE, 16756 },
	{ 0xfca1, 0xfcb8, PDF_CMAP_RANGE, 16765 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 4447 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 16789 },
	{ 0xfcbd, 0xfce2, PDF_CMAP_RANGE, 16791 },
	{ 0xfce4, 0xfced, PDF_CMAP_RANGE, 16829 },
	{ 0xfcef, 0xfcfe, PDF_CMAP_RANGE, 16839 },
	{ 0xfd40, 0xfd48, PDF_CMAP_RANGE, 16855 },
	{ 0xfd4a, 0xfd7e, PDF_CMAP_RANGE, 16864 },
	{ 0xfda1, 0xfdf1, PDF_CMAP_RANGE, 16917 },
	{ 0xfdf3, 0xfdfe, PDF_CMAP_RANGE, 16998 },
	{ 0xfe40, 0xfe6c, PDF_CMAP_RANGE, 17010 },
	{ 0xfe6e, 0xfe77, PDF_CMAP_RANGE, 17055 },
	{ 0xfe79, 0xfe7e, PDF_CMAP_RANGE, 17065 },
	{ 0xfea1, 0xfea9, PDF_CMAP_RANGE, 17071 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 288 },
	{ 0xfeab, 0xfedd, PDF_CMAP_RANGE, 17080 },
	{ 0xfee0, 0xfeec, PDF_CMAP_RANGE, 17131 },
	{ 0xfeef, 0xfefe, PDF_CMAP_RANGE, 17144 },
	{ 0xfe40, 0xfe6c, PDF_CMAP_RANGE, 17010 },
	{ 0xfe6e, 0xfe77, PDF_CMAP_RANGE, 17055 },
	{ 0xfe79, 0xfe7e, PDF_CMAP_RANGE, 17065 },
	{ 0xfea1, 0xfea9, PDF_CMAP_RANGE, 17071 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 288 },
	{ 0xfeab, 0xfedd, PDF_CMAP_RANGE, 17080 },
	{ 0xfee0, 0xfeec, PDF_CMAP_RANGE, 17131 },
	{ 0xfeef, 0xfefe, PDF_CMAP_RANGE, 17144 },
};

static const unsigned short pdf_cmap_HKgccs_B5_H_table[14] =
{
	284,283,15812,7188,15813,15857,13418,248,
	247,5551,5550,13585,13629,13641,
};

pdf_cmap pdf_cmap_HKgccs_B5_H =
{
	-1, "HKgccs-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8a40, 0xfefe },
	},
	552, 552, (pdf_range*) pdf_cmap_HKgccs_B5_H_ranges,
	14, 14, (unsigned short*) pdf_cmap_HKgccs_B5_H_table,
};

/* HKgccs-B5-V */

static const pdf_range pdf_cmap_HKgccs_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_HKgccs_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_HKgccs_B5_V =
{
	-1, "HKgccs-B5-V", "HKgccs-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_HKgccs_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_HKgccs_B5_V_table,
};

/* HKm314-B5-H */

static const pdf_range pdf_cmap_HKm314_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 0 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 2 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc740, 0xc74e, PDF_CMAP_TABLE, 4 },
	{ 0xc74f, 0xc750, PDF_CMAP_RANGE, 16456 },
	{ 0xc751, 0xc771, PDF_CMAP_TABLE, 19 },
	{ 0xc772, 0xc773, PDF_CMAP_RANGE, 16040 },
	{ 0xc774, 0xc77e, PDF_CMAP_TABLE, 52 },
	{ 0xc7a1, 0xc7bd, PDF_CMAP_TABLE, 63 },
	{ 0xc7be, 0xc7bf, PDF_CMAP_RANGE, 17329 },
	{ 0xc7c0, 0xc7d6, PDF_CMAP_TABLE, 92 },
	{ 0xc7d7, 0xc7d8, PDF_CMAP_RANGE, 17340 },
	{ 0xc7d9, 0xc7f4, PDF_CMAP_TABLE, 115 },
	{ 0xc7f5, 0xc7f6, PDF_CMAP_RANGE, 17346 },
	{ 0xc7f7, 0xc7fe, PDF_CMAP_TABLE, 143 },
	{ 0xc840, 0xc865, PDF_CMAP_TABLE, 151 },
	{ 0xc866, 0xc867, PDF_CMAP_RANGE, 17355 },
	{ 0xc868, 0xc868, PDF_CMAP_SINGLE, 16306 },
	{ 0xc869, 0xc86b, PDF_CMAP_RANGE, 17034 },
	{ 0xc86c, 0xc87e, PDF_CMAP_TABLE, 189 },
	{ 0xc8a1, 0xc8bc, PDF_CMAP_TABLE, 208 },
	{ 0xc8bd, 0xc8be, PDF_CMAP_RANGE, 14363 },
	{ 0xc8bf, 0xc8c0, PDF_CMAP_RANGE, 14366 },
	{ 0xc8c1, 0xc8c9, PDF_CMAP_TABLE, 236 },
	{ 0xc8ca, 0xc8cb, PDF_CMAP_RANGE, 17383 },
	{ 0xc8cc, 0xc8cf, PDF_CMAP_TABLE, 245 },
	{ 0xc8d0, 0xc8d1, PDF_CMAP_RANGE, 14435 },
	{ 0xc8d2, 0xc8e5, PDF_CMAP_TABLE, 249 },
	{ 0xc8e6, 0xc8e7, PDF_CMAP_RANGE, 15377 },
	{ 0xc8e8, 0xc8ee, PDF_CMAP_TABLE, 269 },
	{ 0xc8ef, 0xc8f0, PDF_CMAP_RANGE, 14618 },
	{ 0xc8f1, 0xc8f5, PDF_CMAP_TABLE, 276 },
	{ 0xc8f6, 0xc8f7, PDF_CMAP_RANGE, 17394 },
	{ 0xc8f8, 0xc8fa, PDF_CMAP_TABLE, 281 },
	{ 0xc8fb, 0xc8fc, PDF_CMAP_RANGE, 14646 },
	{ 0xc8fd, 0xc8fe, PDF_CMAP_TABLE, 284 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 286 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xc859, 0xc859, PDF_CMAP_SINGLE, 15808 },
	{ 0xc85a, 0xc85a, PDF_CMAP_SINGLE, 14826 },
	{ 0xc85b, 0xc85b, PDF_CMAP_SINGLE, 16384 },
	{ 0xc85c, 0xc85c, PDF_CMAP_SINGLE, 16996 },
	{ 0xc85d, 0xc85d, PDF_CMAP_SINGLE, 16994 },
	{ 0xc85e, 0xc85e, PDF_CMAP_SINGLE, 16383 },
	{ 0xc85f, 0xc85f, PDF_CMAP_SINGLE, 16026 },
	{ 0xc860, 0xc860, PDF_CMAP_SINGLE, 17011 },
	{ 0xc861, 0xc861, PDF_CMAP_SINGLE, 14697 },
	{ 0xc862, 0xc862, PDF_CMAP_SINGLE, 17018 },
	{ 0xc863, 0xc863, PDF_CMAP_SINGLE, 17354 },
	{ 0xc864, 0xc864, PDF_CMAP_SINGLE, 16282 },
	{ 0xc865, 0xc865, PDF_CMAP_SINGLE, 16053 },
	{ 0xc866, 0xc867, PDF_CMAP_RANGE, 17355 },
	{ 0xc868, 0xc868, PDF_CMAP_SINGLE, 16306 },
	{ 0xc869, 0xc86b, PDF_CMAP_RANGE, 17034 },
	{ 0xc86c, 0xc86c, PDF_CMAP_SINGLE, 15071 },
	{ 0xc86d, 0xc86d, PDF_CMAP_SINGLE, 17096 },
	{ 0xc86e, 0xc86e, PDF_CMAP_SINGLE, 16045 },
	{ 0xc86f, 0xc86f, PDF_CMAP_SINGLE, 17358 },
	{ 0xc870, 0xc870, PDF_CMAP_SINGLE, 16271 },
	{ 0xc871, 0xc871, PDF_CMAP_SINGLE, 17359 },
	{ 0xc872, 0xc872, PDF_CMAP_SINGLE, 15986 },
	{ 0xc873, 0xc873, PDF_CMAP_SINGLE, 16042 },
	{ 0xc874, 0xc874, PDF_CMAP_SINGLE, 17363 },
	{ 0xc875, 0xc875, PDF_CMAP_SINGLE, 17120 },
	{ 0xc876, 0xc876, PDF_CMAP_SINGLE, 14381 },
	{ 0xc877, 0xc877, PDF_CMAP_SINGLE, 16341 },
	{ 0xc878, 0xc878, PDF_CMAP_SINGLE, 15124 },
	{ 0xc879, 0xc879, PDF_CMAP_SINGLE, 14056 },
	{ 0xc87a, 0xc87a, PDF_CMAP_SINGLE, 17136 },
	{ 0xc87b, 0xc87b, PDF_CMAP_SINGLE, 15997 },
	{ 0xc87c, 0xc87c, PDF_CMAP_SINGLE, 17150 },
	{ 0xc87d, 0xc87d, PDF_CMAP_SINGLE, 17157 },
	{ 0xc87e, 0xc87e, PDF_CMAP_SINGLE, 14138 },
	{ 0xc8a1, 0xc8a1, PDF_CMAP_SINGLE, 14150 },
	{ 0xc8a2, 0xc8a2, PDF_CMAP_SINGLE, 14162 },
	{ 0xc8a3, 0xc8a3, PDF_CMAP_SINGLE, 14061 },
	{ 0xc8a4, 0xc8a4, PDF_CMAP_SINGLE, 15178 },
	{ 0xc8a5, 0xc8a5, PDF_CMAP_SINGLE, 15173 },
	{ 0xc8a6, 0xc8a6, PDF_CMAP_SINGLE, 17370 },
	{ 0xc8a7, 0xc8a7, PDF_CMAP_SINGLE, 16094 },
	{ 0xc8a8, 0xc8a8, PDF_CMAP_SINGLE, 14188 },
	{ 0xc8a9, 0xc8a9, PDF_CMAP_SINGLE, 14187 },
	{ 0xc8aa, 0xc8aa, PDF_CMAP_SINGLE, 14210 },
	{ 0xc8ab, 0xc8ab, PDF_CMAP_SINGLE, 15977 },
	{ 0xc8ac, 0xc8ac, PDF_CMAP_SINGLE, 15183 },
	{ 0xc8ad, 0xc8ad, PDF_CMAP_SINGLE, 14214 },
	{ 0xc8ae, 0xc8ae, PDF_CMAP_SINGLE, 17371 },
	{ 0xc8af, 0xc8af, PDF_CMAP_SINGLE, 14216 },
	{ 0xc8b0, 0xc8b0, PDF_CMAP_SINGLE, 17372 },
	{ 0xc8b1, 0xc8b1, PDF_CMAP_SINGLE, 16077 },
	{ 0xc8b2, 0xc8b2, PDF_CMAP_SINGLE, 14231 },
	{ 0xc8b3, 0xc8b3, PDF_CMAP_SINGLE, 16909 },
	{ 0xc8b4, 0xc8b4, PDF_CMAP_SINGLE, 14296 },
	{ 0xc8b5, 0xc8b5, PDF_CMAP_SINGLE, 15217 },
	{ 0xc8b6, 0xc8b6, PDF_CMAP_SINGLE, 14556 },
	{ 0xc8b7, 0xc8b7, PDF_CMAP_SINGLE, 14304 },
	{ 0xc8b8, 0xc8b8, PDF_CMAP_SINGLE, 14324 },
	{ 0xc8b9, 0xc8b9, PDF_CMAP_SINGLE, 14355 },
	{ 0xc8ba, 0xc8ba, PDF_CMAP_SINGLE, 16048 },
	{ 0xc8bb, 0xc8bb, PDF_CMAP_SINGLE, 14349 },
	{ 0xc8bc, 0xc8bc, PDF_CMAP_SINGLE, 16047 },
	{ 0xc8bd, 0xc8be, PDF_CMAP_RANGE, 14363 },
	{ 0xc8bf, 0xc8c0, PDF_CMAP_RANGE, 14366 },
	{ 0xc8c1, 0xc8c1, PDF_CMAP_SINGLE, 16049 },
	{ 0xc8c2, 0xc8c2, PDF_CMAP_SINGLE, 16361 },
	{ 0xc8c3, 0xc8c3, PDF_CMAP_SINGLE, 17380 },
	{ 0xc8c4, 0xc8c4, PDF_CMAP_SINGLE, 16377 },
	{ 0xc8c5, 0xc8c5, PDF_CMAP_SINGLE, 15961 },
	{ 0xc8c6, 0xc8c6, PDF_CMAP_SINGLE, 16025 },
	{ 0xc8c7, 0xc8c7, PDF_CMAP_SINGLE, 15966 },
	{ 0xc8c8, 0xc8c8, PDF_CMAP_SINGLE, 15284 },
	{ 0xc8c9, 0xc8c9, PDF_CMAP_SINGLE, 16043 },
	{ 0xc8ca, 0xc8cb, PDF_CMAP_RANGE, 17383 },
	{ 0xc8cc, 0xc8cc, PDF_CMAP_SINGLE, 14430 },
	{ 0xc8cd, 0xc8cd, PDF_CMAP_SINGLE, 17385 },
	{ 0xc8ce, 0xc8ce, PDF_CMAP_SINGLE, 15806 },
	{ 0xc8cf, 0xc8cf, PDF_CMAP_SINGLE, 16005 },
	{ 0xc8d0, 0xc8d1, PDF_CMAP_RANGE, 14435 },
	{ 0xc8d2, 0xc8d2, PDF_CMAP_SINGLE, 15724 },
	{ 0xc8d3, 0xc8d3, PDF_CMAP_SINGLE, 14449 },
	{ 0xc8d4, 0xc8d4, PDF_CMAP_SINGLE, 14460 },
	{ 0xc8d5, 0xc8d5, PDF_CMAP_SINGLE, 14466 },
	{ 0xc8d6, 0xc8d6, PDF_CMAP_SINGLE, 16027 },
	{ 0xc8d7, 0xc8d7, PDF_CMAP_SINGLE, 14472 },
	{ 0xc8d8, 0xc8d8, PDF_CMAP_SINGLE, 15374 },
	{ 0xc8d9, 0xc8d9, PDF_CMAP_SINGLE, 14382 },
	{ 0xc8da, 0xc8da, PDF_CMAP_SINGLE, 14490 },
	{ 0xc8db, 0xc8db, PDF_CMAP_SINGLE, 14475 },
	{ 0xc8dc, 0xc8dc, PDF_CMAP_SINGLE, 15364 },
	{ 0xc8dd, 0xc8dd, PDF_CMAP_SINGLE, 14057 },
	{ 0xc8de, 0xc8de, PDF_CMAP_SINGLE, 17085 },
	{ 0xc8df, 0xc8df, PDF_CMAP_SINGLE, 15309 },
	{ 0xc8e0, 0xc8e0, PDF_CMAP_SINGLE, 14495 },
	{ 0xc8e1, 0xc8e1, PDF_CMAP_SINGLE, 14235 },
	{ 0xc8e2, 0xc8e2, PDF_CMAP_SINGLE, 14511 },
	{ 0xc8e3, 0xc8e3, PDF_CMAP_SINGLE, 16023 },
	{ 0xc8e4, 0xc8e4, PDF_CMAP_SINGLE, 14576 },
	{ 0xc8e5, 0xc8e5, PDF_CMAP_SINGLE, 15357 },
	{ 0xc8e6, 0xc8e7, PDF_CMAP_RANGE, 15377 },
	{ 0xc8e8, 0xc8e8, PDF_CMAP_SINGLE, 17391 },
	{ 0xc8e9, 0xc8e9, PDF_CMAP_SINGLE, 14213 },
	{ 0xc8ea, 0xc8ea, PDF_CMAP_SINGLE, 14594 },
	{ 0xc8eb, 0xc8eb, PDF_CMAP_SINGLE, 16298 },
	{ 0xc8ec, 0xc8ec, PDF_CMAP_SINGLE, 16153 },
	{ 0xc8ed, 0xc8ed, PDF_CMAP_SINGLE, 14617 },
	{ 0xc8ee, 0xc8ee, PDF_CMAP_SINGLE, 17393 },
	{ 0xc8ef, 0xc8f0, PDF_CMAP_RANGE, 14618 },
	{ 0xc8f1, 0xc8f1, PDF_CMAP_SINGLE, 15974 },
	{ 0xc8f2, 0xc8f2, PDF_CMAP_SINGLE, 16162 },
	{ 0xc8f3, 0xc8f3, PDF_CMAP_SINGLE, 15742 },
	{ 0xc8f4, 0xc8f4, PDF_CMAP_SINGLE, 15417 },
	{ 0xc8f5, 0xc8f5, PDF_CMAP_SINGLE, 16033 },
	{ 0xc8f6, 0xc8f7, PDF_CMAP_RANGE, 17394 },
	{ 0xc8f8, 0xc8f8, PDF_CMAP_SINGLE, 16176 },
	{ 0xc8f9, 0xc8f9, PDF_CMAP_SINGLE, 14633 },
	{ 0xc8fa, 0xc8fa, PDF_CMAP_SINGLE, 16316 },
	{ 0xc8fb, 0xc8fc, PDF_CMAP_RANGE, 14646 },
	{ 0xc8fd, 0xc8fd, PDF_CMAP_SINGLE, 16272 },
	{ 0xc8fe, 0xc8fe, PDF_CMAP_SINGLE, 2746 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c4, PDF_CMAP_SINGLE, 13585 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 13629 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
};

static const unsigned short pdf_cmap_HKm314_B5_H_table[289] =
{
	248,247,5551,5550,15725,14788,17305,15877,
	14058,14792,14653,17307,16418,17309,16436,17311,
	16035,16038,16465,16459,16463,17314,16474,16036,
	15805,15807,16476,16479,17316,16507,16514,16237,
	16523,17318,16531,16534,17319,16536,15999,15479,
	16039,16010,16024,16012,16545,14848,14755,16013,
	17320,16216,15804,15564,15150,15302,16011,16031,
	17196,17321,16136,16037,16009,15811,16002,16001,
	16020,16003,16008,17323,16017,17324,15996,16000,
	16015,15975,15992,15976,17325,14290,16205,17326,
	15998,16572,15971,16573,17327,16567,16055,16109,
	16234,16021,14664,15560,16588,16006,15482,15485,
	17332,16593,15730,15398,14059,16575,16640,15493,
	17333,14877,14325,14062,15967,16662,16671,16664,
	16675,16679,16681,16014,14447,16707,16710,16708,
	16713,15269,16727,16737,14060,16746,16386,16774,
	16314,16208,16022,16778,16321,15759,17345,16032,
	16150,16029,16786,14950,16028,16016,16797,14248,
	16798,16775,16034,16007,16004,16046,15220,15221,
	15218,16050,16044,16842,16884,16848,15018,16885,
	16888,16891,16901,15758,16910,15715,16930,16929,
	16931,16489,16431,14501,15605,15721,16987,15048,
	15808,14826,16384,16996,16994,16383,16026,17011,
	14697,17018,17354,16282,16053,15071,17096,16045,
	17358,16271,17359,15986,16042,17363,17120,14381,
	16341,15124,14056,17136,15997,17150,17157,14138,
	14150,14162,14061,15178,15173,17370,16094,14188,
	14187,14210,15977,15183,14214,17371,14216,17372,
	16077,14231,16909,14296,15217,14556,14304,14324,
	14355,16048,14349,16047,16049,16361,17380,16377,
	15961,16025,15966,15284,16043,14430,17385,15806,
	16005,15724,14449,14460,14466,16027,14472,15374,
	14382,14490,14475,15364,14057,17085,15309,14495,
	14235,14511,16023,14576,15357,17391,14213,14594,
	16298,16153,14617,17393,15974,16162,15742,15417,
	16033,16176,14633,16316,16272,2746,13585,13629,
	13641,
};

pdf_cmap pdf_cmap_HKm314_B5_H =
{
	-1, "HKm314-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa140, 0xfefe },
	},
	542, 542, (pdf_range*) pdf_cmap_HKm314_B5_H_ranges,
	289, 289, (unsigned short*) pdf_cmap_HKm314_B5_H_table,
};

/* HKm314-B5-V */

static const pdf_range pdf_cmap_HKm314_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_HKm314_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_HKm314_B5_V =
{
	-1, "HKm314-B5-V", "HKm314-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_HKm314_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_HKm314_B5_V_table,
};

/* HKm471-B5-H */

static const pdf_range pdf_cmap_HKm471_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 0 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 2 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 4 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xfa40, 0xfa4b, PDF_CMAP_TABLE, 7 },
	{ 0xfa4c, 0xfa4d, PDF_CMAP_RANGE, 14796 },
	{ 0xfa4e, 0xfa51, PDF_CMAP_TABLE, 19 },
	{ 0xfa52, 0xfa53, PDF_CMAP_RANGE, 17310 },
	{ 0xfa54, 0xfa57, PDF_CMAP_TABLE, 23 },
	{ 0xfa58, 0xfa59, PDF_CMAP_RANGE, 16456 },
	{ 0xfa5a, 0xfa5e, PDF_CMAP_TABLE, 27 },
	{ 0xfa5f, 0xfa60, PDF_CMAP_RANGE, 17313 },
	{ 0xfa61, 0xfa66, PDF_CMAP_TABLE, 32 },
	{ 0xfa67, 0xfa68, PDF_CMAP_RANGE, 16479 },
	{ 0xfa69, 0xfa6a, PDF_CMAP_TABLE, 38 },
	{ 0xfa6b, 0xfa6c, PDF_CMAP_RANGE, 16506 },
	{ 0xfa6d, 0xfa7e, PDF_CMAP_TABLE, 40 },
	{ 0xfaa1, 0xfaab, PDF_CMAP_TABLE, 58 },
	{ 0xfaac, 0xfaad, PDF_CMAP_RANGE, 16040 },
	{ 0xfaae, 0xfadb, PDF_CMAP_TABLE, 69 },
	{ 0xfadc, 0xfade, PDF_CMAP_RANGE, 17329 },
	{ 0xfadf, 0xfafa, PDF_CMAP_TABLE, 115 },
	{ 0xfafb, 0xfafc, PDF_CMAP_RANGE, 17335 },
	{ 0xfafd, 0xfafe, PDF_CMAP_TABLE, 143 },
	{ 0xfb40, 0xfb6f, PDF_CMAP_TABLE, 145 },
	{ 0xfb70, 0xfb71, PDF_CMAP_RANGE, 17346 },
	{ 0xfb72, 0xfb7b, PDF_CMAP_TABLE, 193 },
	{ 0xfb7c, 0xfb7d, PDF_CMAP_RANGE, 15220 },
	{ 0xfb7e, 0xfb7e, PDF_CMAP_SINGLE, 14242 },
	{ 0xfba1, 0xfba3, PDF_CMAP_TABLE, 203 },
	{ 0xfba4, 0xfba5, PDF_CMAP_RANGE, 17348 },
	{ 0xfba6, 0xfbd3, PDF_CMAP_TABLE, 206 },
	{ 0xfbd4, 0xfbd5, PDF_CMAP_RANGE, 17353 },
	{ 0xfbd6, 0xfbd7, PDF_CMAP_TABLE, 252 },
	{ 0xfbd8, 0xfbd9, PDF_CMAP_RANGE, 17355 },
	{ 0xfbda, 0xfbda, PDF_CMAP_SINGLE, 16306 },
	{ 0xfbdb, 0xfbdc, PDF_CMAP_RANGE, 17034 },
	{ 0xfbdd, 0xfbed, PDF_CMAP_TABLE, 254 },
	{ 0xfbee, 0xfbf0, PDF_CMAP_RANGE, 17362 },
	{ 0xfbf1, 0xfbf8, PDF_CMAP_TABLE, 271 },
	{ 0xfbf9, 0xfbfa, PDF_CMAP_RANGE, 17365 },
	{ 0xfbfb, 0xfbfe, PDF_CMAP_TABLE, 279 },
	{ 0xfc40, 0xfc73, PDF_CMAP_TABLE, 283 },
	{ 0xfc74, 0xfc75, PDF_CMAP_RANGE, 17376 },
	{ 0xfc76, 0xfc7e, PDF_CMAP_TABLE, 335 },
	{ 0xfca1, 0xfca1, PDF_CMAP_SINGLE, 14364 },
	{ 0xfca2, 0xfca3, PDF_CMAP_RANGE, 14366 },
	{ 0xfca4, 0xfcb3, PDF_CMAP_TABLE, 344 },
	{ 0xfcb4, 0xfcb5, PDF_CMAP_RANGE, 17383 },
	{ 0xfcb6, 0xfcb9, PDF_CMAP_TABLE, 360 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 14435 },
	{ 0xfcbc, 0xfcd9, PDF_CMAP_TABLE, 364 },
	{ 0xfcda, 0xfcdb, PDF_CMAP_RANGE, 17389 },
	{ 0xfcdc, 0xfcdc, PDF_CMAP_SINGLE, 15357 },
	{ 0xfcdd, 0xfcde, PDF_CMAP_RANGE, 15377 },
	{ 0xfcdf, 0xfcf3, PDF_CMAP_TABLE, 394 },
	{ 0xfcf4, 0xfcf5, PDF_CMAP_RANGE, 17394 },
	{ 0xfcf6, 0xfcfe, PDF_CMAP_TABLE, 415 },
	{ 0xfa78, 0xfa78, PDF_CMAP_SINGLE, 15999 },
	{ 0xfa79, 0xfa79, PDF_CMAP_SINGLE, 15479 },
	{ 0xfa7a, 0xfa7a, PDF_CMAP_SINGLE, 16039 },
	{ 0xfa7b, 0xfa7b, PDF_CMAP_SINGLE, 16899 },
	{ 0xfa7c, 0xfa7c, PDF_CMAP_SINGLE, 16081 },
	{ 0xfa7d, 0xfa7d, PDF_CMAP_SINGLE, 16010 },
	{ 0xfa7e, 0xfa7e, PDF_CMAP_SINGLE, 16024 },
	{ 0xfaa1, 0xfaa1, PDF_CMAP_SINGLE, 16012 },
	{ 0xfaa2, 0xfaa2, PDF_CMAP_SINGLE, 16545 },
	{ 0xfaa3, 0xfaa3, PDF_CMAP_SINGLE, 15747 },
	{ 0xfaa4, 0xfaa4, PDF_CMAP_SINGLE, 14848 },
	{ 0xfaa5, 0xfaa5, PDF_CMAP_SINGLE, 14755 },
	{ 0xfaa6, 0xfaa6, PDF_CMAP_SINGLE, 16013 },
	{ 0xfaa7, 0xfaa7, PDF_CMAP_SINGLE, 17320 },
	{ 0xfaa8, 0xfaa8, PDF_CMAP_SINGLE, 16216 },
	{ 0xfaa9, 0xfaa9, PDF_CMAP_SINGLE, 15804 },
	{ 0xfaaa, 0xfaaa, PDF_CMAP_SINGLE, 16546 },
	{ 0xfaab, 0xfaab, PDF_CMAP_SINGLE, 15564 },
	{ 0xfaac, 0xfaad, PDF_CMAP_RANGE, 16040 },
	{ 0xfaae, 0xfaae, PDF_CMAP_SINGLE, 15150 },
	{ 0xfaaf, 0xfaaf, PDF_CMAP_SINGLE, 15302 },
	{ 0xfab0, 0xfab0, PDF_CMAP_SINGLE, 16011 },
	{ 0xfab1, 0xfab1, PDF_CMAP_SINGLE, 16031 },
	{ 0xfab2, 0xfab2, PDF_CMAP_SINGLE, 17196 },
	{ 0xfab3, 0xfab3, PDF_CMAP_SINGLE, 17321 },
	{ 0xfab4, 0xfab4, PDF_CMAP_SINGLE, 16136 },
	{ 0xfab5, 0xfab5, PDF_CMAP_SINGLE, 16037 },
	{ 0xfab6, 0xfab6, PDF_CMAP_SINGLE, 16009 },
	{ 0xfab7, 0xfab7, PDF_CMAP_SINGLE, 15811 },
	{ 0xfab8, 0xfab8, PDF_CMAP_SINGLE, 16002 },
	{ 0xfab9, 0xfab9, PDF_CMAP_SINGLE, 16001 },
	{ 0xfaba, 0xfaba, PDF_CMAP_SINGLE, 16020 },
	{ 0xfabb, 0xfabb, PDF_CMAP_SINGLE, 17322 },
	{ 0xfabc, 0xfabc, PDF_CMAP_SINGLE, 16003 },
	{ 0xfabd, 0xfabd, PDF_CMAP_SINGLE, 16008 },
	{ 0xfabe, 0xfabe, PDF_CMAP_SINGLE, 17323 },
	{ 0xfabf, 0xfabf, PDF_CMAP_SINGLE, 16017 },
	{ 0xfac0, 0xfac0, PDF_CMAP_SINGLE, 17324 },
	{ 0xfac1, 0xfac1, PDF_CMAP_SINGLE, 15996 },
	{ 0xfac2, 0xfac2, PDF_CMAP_SINGLE, 16000 },
	{ 0xfac3, 0xfac3, PDF_CMAP_SINGLE, 16015 },
	{ 0xfac4, 0xfac4, PDF_CMAP_SINGLE, 17189 },
	{ 0xfac5, 0xfac5, PDF_CMAP_SINGLE, 15975 },
	{ 0xfac6, 0xfac6, PDF_CMAP_SINGLE, 15992 },
	{ 0xfac7, 0xfac7, PDF_CMAP_SINGLE, 15976 },
	{ 0xfac8, 0xfac8, PDF_CMAP_SINGLE, 17325 },
	{ 0xfac9, 0xfac9, PDF_CMAP_SINGLE, 14290 },
	{ 0xfaca, 0xfaca, PDF_CMAP_SINGLE, 16205 },
	{ 0xfacb, 0xfacb, PDF_CMAP_SINGLE, 17326 },
	{ 0xfacc, 0xfacc, PDF_CMAP_SINGLE, 15998 },
	{ 0xfacd, 0xfacd, PDF_CMAP_SINGLE, 16572 },
	{ 0xface, 0xface, PDF_CMAP_SINGLE, 15971 },
	{ 0xfacf, 0xfacf, PDF_CMAP_SINGLE, 16573 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 17327 },
	{ 0xfad1, 0xfad1, PDF_CMAP_SINGLE, 16236 },
	{ 0xfad2, 0xfad2, PDF_CMAP_SINGLE, 16121 },
	{ 0xfad3, 0xfad3, PDF_CMAP_SINGLE, 16567 },
	{ 0xfad4, 0xfad4, PDF_CMAP_SINGLE, 16055 },
	{ 0xfad5, 0xfad5, PDF_CMAP_SINGLE, 16109 },
	{ 0xfad6, 0xfad6, PDF_CMAP_SINGLE, 16234 },
	{ 0xfad7, 0xfad7, PDF_CMAP_SINGLE, 14758 },
	{ 0xfad8, 0xfad8, PDF_CMAP_SINGLE, 17328 },
	{ 0xfad9, 0xfad9, PDF_CMAP_SINGLE, 16021 },
	{ 0xfada, 0xfada, PDF_CMAP_SINGLE, 14664 },
	{ 0xfadb, 0xfadb, PDF_CMAP_SINGLE, 15560 },
	{ 0xfadc, 0xfade, PDF_CMAP_RANGE, 17329 },
	{ 0xfadf, 0xfadf, PDF_CMAP_SINGLE, 16588 },
	{ 0xfae0, 0xfae0, PDF_CMAP_SINGLE, 16006 },
	{ 0xfae1, 0xfae1, PDF_CMAP_SINGLE, 16584 },
	{ 0xfae2, 0xfae2, PDF_CMAP_SINGLE, 15482 },
	{ 0xfae3, 0xfae3, PDF_CMAP_SINGLE, 15485 },
	{ 0xfae4, 0xfae4, PDF_CMAP_SINGLE, 17332 },
	{ 0xfae5, 0xfae5, PDF_CMAP_SINGLE, 16593 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 15730 },
	{ 0xfae7, 0xfae7, PDF_CMAP_SINGLE, 16596 },
	{ 0xfae8, 0xfae8, PDF_CMAP_SINGLE, 15398 },
	{ 0xfae9, 0xfae9, PDF_CMAP_SINGLE, 14059 },
	{ 0xfaea, 0xfaea, PDF_CMAP_SINGLE, 16602 },
	{ 0xfaeb, 0xfaeb, PDF_CMAP_SINGLE, 16601 },
	{ 0xfaec, 0xfaec, PDF_CMAP_SINGLE, 16575 },
	{ 0xfaed, 0xfaed, PDF_CMAP_SINGLE, 15745 },
	{ 0xfaee, 0xfaee, PDF_CMAP_SINGLE, 16625 },
	{ 0xfaef, 0xfaef, PDF_CMAP_SINGLE, 16640 },
	{ 0xfaf0, 0xfaf0, PDF_CMAP_SINGLE, 15493 },
	{ 0xfaf1, 0xfaf1, PDF_CMAP_SINGLE, 17333 },
	{ 0xfaf2, 0xfaf2, PDF_CMAP_SINGLE, 14877 },
	{ 0xfaf3, 0xfaf3, PDF_CMAP_SINGLE, 14325 },
	{ 0xfaf4, 0xfaf4, PDF_CMAP_SINGLE, 14062 },
	{ 0xfaf5, 0xfaf5, PDF_CMAP_SINGLE, 15967 },
	{ 0xfaf6, 0xfaf6, PDF_CMAP_SINGLE, 16662 },
	{ 0xfaf7, 0xfaf7, PDF_CMAP_SINGLE, 17083 },
	{ 0xfaf8, 0xfaf8, PDF_CMAP_SINGLE, 16671 },
	{ 0xfaf9, 0xfaf9, PDF_CMAP_SINGLE, 17334 },
	{ 0xfafa, 0xfafa, PDF_CMAP_SINGLE, 16664 },
	{ 0xfafb, 0xfafc, PDF_CMAP_RANGE, 17335 },
	{ 0xfafd, 0xfafd, PDF_CMAP_SINGLE, 16675 },
	{ 0xfafe, 0xfafe, PDF_CMAP_SINGLE, 16674 },
	{ 0xfb40, 0xfb40, PDF_CMAP_SINGLE, 16679 },
	{ 0xfb41, 0xfb41, PDF_CMAP_SINGLE, 17337 },
	{ 0xfb42, 0xfb42, PDF_CMAP_SINGLE, 16681 },
	{ 0xfb43, 0xfb43, PDF_CMAP_SINGLE, 17338 },
	{ 0xfb44, 0xfb44, PDF_CMAP_SINGLE, 15829 },
	{ 0xfb45, 0xfb45, PDF_CMAP_SINGLE, 16111 },
	{ 0xfb46, 0xfb46, PDF_CMAP_SINGLE, 15754 },
	{ 0xfb47, 0xfb47, PDF_CMAP_SINGLE, 17339 },
	{ 0xfb48, 0xfb48, PDF_CMAP_SINGLE, 14891 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 17340 },
	{ 0xfb4a, 0xfb4a, PDF_CMAP_SINGLE, 16696 },
	{ 0xfb4b, 0xfb4b, PDF_CMAP_SINGLE, 17341 },
	{ 0xfb4c, 0xfb4c, PDF_CMAP_SINGLE, 16014 },
	{ 0xfb4d, 0xfb4d, PDF_CMAP_SINGLE, 14896 },
	{ 0xfb4e, 0xfb4e, PDF_CMAP_SINGLE, 14447 },
	{ 0xfb4f, 0xfb4f, PDF_CMAP_SINGLE, 16707 },
	{ 0xfb50, 0xfb50, PDF_CMAP_SINGLE, 16710 },
	{ 0xfb51, 0xfb51, PDF_CMAP_SINGLE, 16708 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 16713 },
	{ 0xfb53, 0xfb53, PDF_CMAP_SINGLE, 15269 },
	{ 0xfb54, 0xfb54, PDF_CMAP_SINGLE, 16727 },
	{ 0xfb55, 0xfb55, PDF_CMAP_SINGLE, 17342 },
	{ 0xfb56, 0xfb56, PDF_CMAP_SINGLE, 16737 },
	{ 0xfb57, 0xfb57, PDF_CMAP_SINGLE, 14060 },
	{ 0xfb58, 0xfb58, PDF_CMAP_SINGLE, 16746 },
	{ 0xfb59, 0xfb59, PDF_CMAP_SINGLE, 15748 },
	{ 0xfb5a, 0xfb5a, PDF_CMAP_SINGLE, 16386 },
	{ 0xfb5b, 0xfb5b, PDF_CMAP_SINGLE, 16774 },
	{ 0xfb5c, 0xfb5c, PDF_CMAP_SINGLE, 16314 },
	{ 0xfb5d, 0xfb5d, PDF_CMAP_SINGLE, 17343 },
	{ 0xfb5e, 0xfb5e, PDF_CMAP_SINGLE, 16325 },
	{ 0xfb5f, 0xfb5f, PDF_CMAP_SINGLE, 16208 },
	{ 0xfb60, 0xfb60, PDF_CMAP_SINGLE, 16022 },
	{ 0xfb61, 0xfb61, PDF_CMAP_SINGLE, 17344 },
	{ 0xfb62, 0xfb62, PDF_CMAP_SINGLE, 16778 },
	{ 0xfb63, 0xfb63, PDF_CMAP_SINGLE, 16321 },
	{ 0xfb64, 0xfb64, PDF_CMAP_SINGLE, 15759 },
	{ 0xfb65, 0xfb65, PDF_CMAP_SINGLE, 17345 },
	{ 0xfb66, 0xfb66, PDF_CMAP_SINGLE, 16032 },
	{ 0xfb67, 0xfb67, PDF_CMAP_SINGLE, 16150 },
	{ 0xfb68, 0xfb68, PDF_CMAP_SINGLE, 16029 },
	{ 0xfb69, 0xfb69, PDF_CMAP_SINGLE, 16782 },
	{ 0xfb6a, 0xfb6a, PDF_CMAP_SINGLE, 16786 },
	{ 0xfb6b, 0xfb6b, PDF_CMAP_SINGLE, 14950 },
	{ 0xfb6c, 0xfb6c, PDF_CMAP_SINGLE, 16028 },
	{ 0xfb6d, 0xfb6d, PDF_CMAP_SINGLE, 17295 },
	{ 0xfb6e, 0xfb6e, PDF_CMAP_SINGLE, 16016 },
	{ 0xfb6f, 0xfb6f, PDF_CMAP_SINGLE, 16797 },
	{ 0xfb70, 0xfb71, PDF_CMAP_RANGE, 17346 },
	{ 0xfb72, 0xfb72, PDF_CMAP_SINGLE, 17238 },
	{ 0xfb73, 0xfb73, PDF_CMAP_SINGLE, 14248 },
	{ 0xfb74, 0xfb74, PDF_CMAP_SINGLE, 16798 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 16793 },
	{ 0xfb76, 0xfb76, PDF_CMAP_SINGLE, 16775 },
	{ 0xfb77, 0xfb77, PDF_CMAP_SINGLE, 16034 },
	{ 0xfb78, 0xfb78, PDF_CMAP_SINGLE, 16799 },
	{ 0xfb79, 0xfb79, PDF_CMAP_SINGLE, 16007 },
	{ 0xfb7a, 0xfb7a, PDF_CMAP_SINGLE, 16004 },
	{ 0xfb7b, 0xfb7b, PDF_CMAP_SINGLE, 16046 },
	{ 0xfb7c, 0xfb7d, PDF_CMAP_RANGE, 15220 },
	{ 0xfb7e, 0xfb7e, PDF_CMAP_SINGLE, 14242 },
	{ 0xfba1, 0xfba1, PDF_CMAP_SINGLE, 15218 },
	{ 0xfba2, 0xfba2, PDF_CMAP_SINGLE, 15993 },
	{ 0xfba3, 0xfba3, PDF_CMAP_SINGLE, 15981 },
	{ 0xfba4, 0xfba5, PDF_CMAP_RANGE, 17348 },
	{ 0xfba6, 0xfba6, PDF_CMAP_SINGLE, 16050 },
	{ 0xfba7, 0xfba7, PDF_CMAP_SINGLE, 17350 },
	{ 0xfba8, 0xfba8, PDF_CMAP_SINGLE, 16044 },
	{ 0xfba9, 0xfba9, PDF_CMAP_SINGLE, 16842 },
	{ 0xfbaa, 0xfbaa, PDF_CMAP_SINGLE, 16884 },
	{ 0xfbab, 0xfbab, PDF_CMAP_SINGLE, 17351 },
	{ 0xfbac, 0xfbac, PDF_CMAP_SINGLE, 16848 },
	{ 0xfbad, 0xfbad, PDF_CMAP_SINGLE, 15018 },
	{ 0xfbae, 0xfbae, PDF_CMAP_SINGLE, 16862 },
	{ 0xfbaf, 0xfbaf, PDF_CMAP_SINGLE, 16859 },
	{ 0xfbb0, 0xfbb0, PDF_CMAP_SINGLE, 16872 },
	{ 0xfbb1, 0xfbb1, PDF_CMAP_SINGLE, 17352 },
	{ 0xfbb2, 0xfbb2, PDF_CMAP_SINGLE, 16374 },
	{ 0xfbb3, 0xfbb3, PDF_CMAP_SINGLE, 16857 },
	{ 0xfbb4, 0xfbb4, PDF_CMAP_SINGLE, 15682 },
	{ 0xfbb5, 0xfbb5, PDF_CMAP_SINGLE, 16885 },
	{ 0xfbb6, 0xfbb6, PDF_CMAP_SINGLE, 16888 },
	{ 0xfbb7, 0xfbb7, PDF_CMAP_SINGLE, 16891 },
	{ 0xfbb8, 0xfbb8, PDF_CMAP_SINGLE, 16901 },
	{ 0xfbb9, 0xfbb9, PDF_CMAP_SINGLE, 15758 },
	{ 0xfbba, 0xfbba, PDF_CMAP_SINGLE, 16910 },
	{ 0xfbbb, 0xfbbb, PDF_CMAP_SINGLE, 15715 },
	{ 0xfbbc, 0xfbbc, PDF_CMAP_SINGLE, 16930 },
	{ 0xfbbd, 0xfbbd, PDF_CMAP_SINGLE, 16929 },
	{ 0xfbbe, 0xfbbe, PDF_CMAP_SINGLE, 16931 },
	{ 0xfbbf, 0xfbbf, PDF_CMAP_SINGLE, 16489 },
	{ 0xfbc0, 0xfbc0, PDF_CMAP_SINGLE, 16431 },
	{ 0xfbc1, 0xfbc1, PDF_CMAP_SINGLE, 16962 },
	{ 0xfbc2, 0xfbc2, PDF_CMAP_SINGLE, 14501 },
	{ 0xfbc3, 0xfbc3, PDF_CMAP_SINGLE, 16966 },
	{ 0xfbc4, 0xfbc4, PDF_CMAP_SINGLE, 15605 },
	{ 0xfbc5, 0xfbc5, PDF_CMAP_SINGLE, 15721 },
	{ 0xfbc6, 0xfbc6, PDF_CMAP_SINGLE, 15614 },
	{ 0xfbc7, 0xfbc7, PDF_CMAP_SINGLE, 16987 },
	{ 0xfbc8, 0xfbc8, PDF_CMAP_SINGLE, 15048 },
	{ 0xfbc9, 0xfbc9, PDF_CMAP_SINGLE, 15808 },
	{ 0xfbca, 0xfbca, PDF_CMAP_SINGLE, 14826 },
	{ 0xfbcb, 0xfbcb, PDF_CMAP_SINGLE, 16384 },
	{ 0xfbcc, 0xfbcc, PDF_CMAP_SINGLE, 16996 },
	{ 0xfbcd, 0xfbcd, PDF_CMAP_SINGLE, 16994 },
	{ 0xfbce, 0xfbce, PDF_CMAP_SINGLE, 16383 },
	{ 0xfbcf, 0xfbcf, PDF_CMAP_SINGLE, 16026 },
	{ 0xfbd0, 0xfbd0, PDF_CMAP_SINGLE, 17011 },
	{ 0xfbd1, 0xfbd1, PDF_CMAP_SINGLE, 14697 },
	{ 0xfbd2, 0xfbd2, PDF_CMAP_SINGLE, 17018 },
	{ 0xfbd3, 0xfbd3, PDF_CMAP_SINGLE, 17027 },
	{ 0xfbd4, 0xfbd5, PDF_CMAP_RANGE, 17353 },
	{ 0xfbd6, 0xfbd6, PDF_CMAP_SINGLE, 16282 },
	{ 0xfbd7, 0xfbd7, PDF_CMAP_SINGLE, 16053 },
	{ 0xfbd8, 0xfbd9, PDF_CMAP_RANGE, 17355 },
	{ 0xfbda, 0xfbda, PDF_CMAP_SINGLE, 16306 },
	{ 0xfbdb, 0xfbdc, PDF_CMAP_RANGE, 17034 },
	{ 0xfbdd, 0xfbdd, PDF_CMAP_SINGLE, 15127 },
	{ 0xfbde, 0xfbde, PDF_CMAP_SINGLE, 17036 },
	{ 0xfbdf, 0xfbdf, PDF_CMAP_SINGLE, 17040 },
	{ 0xfbe0, 0xfbe0, PDF_CMAP_SINGLE, 15071 },
	{ 0xfbe1, 0xfbe1, PDF_CMAP_SINGLE, 17357 },
	{ 0xfbe2, 0xfbe2, PDF_CMAP_SINGLE, 17093 },
	{ 0xfbe3, 0xfbe3, PDF_CMAP_SINGLE, 17096 },
	{ 0xfbe4, 0xfbe4, PDF_CMAP_SINGLE, 16045 },
	{ 0xfbe5, 0xfbe5, PDF_CMAP_SINGLE, 17358 },
	{ 0xfbe6, 0xfbe6, PDF_CMAP_SINGLE, 16271 },
	{ 0xfbe7, 0xfbe7, PDF_CMAP_SINGLE, 17359 },
	{ 0xfbe8, 0xfbe8, PDF_CMAP_SINGLE, 15986 },
	{ 0xfbe9, 0xfbe9, PDF_CMAP_SINGLE, 15984 },
	{ 0xfbea, 0xfbea, PDF_CMAP_SINGLE, 17360 },
	{ 0xfbeb, 0xfbeb, PDF_CMAP_SINGLE, 16042 },
	{ 0xfbec, 0xfbec, PDF_CMAP_SINGLE, 17361 },
	{ 0xfbed, 0xfbed, PDF_CMAP_SINGLE, 16297 },
	{ 0xfbee, 0xfbf0, PDF_CMAP_RANGE, 17362 },
	{ 0xfbf1, 0xfbf1, PDF_CMAP_SINGLE, 17112 },
	{ 0xfbf2, 0xfbf2, PDF_CMAP_SINGLE, 16371 },
	{ 0xfbf3, 0xfbf3, PDF_CMAP_SINGLE, 17120 },
	{ 0xfbf4, 0xfbf4, PDF_CMAP_SINGLE, 15104 },
	{ 0xfbf5, 0xfbf5, PDF_CMAP_SINGLE, 17175 },
	{ 0xfbf6, 0xfbf6, PDF_CMAP_SINGLE, 14381 },
	{ 0xfbf7, 0xfbf7, PDF_CMAP_SINGLE, 15987 },
	{ 0xfbf8, 0xfbf8, PDF_CMAP_SINGLE, 16341 },
	{ 0xfbf9, 0xfbfa, PDF_CMAP_RANGE, 17365 },
	{ 0xfbfb, 0xfbfb, PDF_CMAP_SINGLE, 15124 },
	{ 0xfbfc, 0xfbfc, PDF_CMAP_SINGLE, 14056 },
	{ 0xfbfd, 0xfbfd, PDF_CMAP_SINGLE, 17140 },
	{ 0xfbfe, 0xfbfe, PDF_CMAP_SINGLE, 15118 },
	{ 0xfc40, 0xfc40, PDF_CMAP_SINGLE, 17132 },
	{ 0xfc41, 0xfc41, PDF_CMAP_SINGLE, 17136 },
	{ 0xfc42, 0xfc42, PDF_CMAP_SINGLE, 15997 },
	{ 0xfc43, 0xfc43, PDF_CMAP_SINGLE, 17367 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 17150 },
	{ 0xfc45, 0xfc45, PDF_CMAP_SINGLE, 17157 },
	{ 0xfc46, 0xfc46, PDF_CMAP_SINGLE, 16346 },
	{ 0xfc47, 0xfc47, PDF_CMAP_SINGLE, 15857 },
	{ 0xfc48, 0xfc48, PDF_CMAP_SINGLE, 14131 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 15741 },
	{ 0xfc4a, 0xfc4a, PDF_CMAP_SINGLE, 17368 },
	{ 0xfc4b, 0xfc4b, PDF_CMAP_SINGLE, 14138 },
	{ 0xfc4c, 0xfc4c, PDF_CMAP_SINGLE, 14144 },
	{ 0xfc4d, 0xfc4d, PDF_CMAP_SINGLE, 14146 },
	{ 0xfc4e, 0xfc4e, PDF_CMAP_SINGLE, 14150 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 14162 },
	{ 0xfc50, 0xfc50, PDF_CMAP_SINGLE, 16159 },
	{ 0xfc51, 0xfc51, PDF_CMAP_SINGLE, 17369 },
	{ 0xfc52, 0xfc52, PDF_CMAP_SINGLE, 14061 },
	{ 0xfc53, 0xfc53, PDF_CMAP_SINGLE, 15178 },
	{ 0xfc54, 0xfc54, PDF_CMAP_SINGLE, 14174 },
	{ 0xfc55, 0xfc55, PDF_CMAP_SINGLE, 15173 },
	{ 0xfc56, 0xfc56, PDF_CMAP_SINGLE, 17370 },
	{ 0xfc57, 0xfc57, PDF_CMAP_SINGLE, 16094 },
	{ 0xfc58, 0xfc58, PDF_CMAP_SINGLE, 14185 },
	{ 0xfc59, 0xfc59, PDF_CMAP_SINGLE, 14188 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 14187 },
	{ 0xfc5b, 0xfc5b, PDF_CMAP_SINGLE, 14190 },
	{ 0xfc5c, 0xfc5c, PDF_CMAP_SINGLE, 14207 },
	{ 0xfc5d, 0xfc5d, PDF_CMAP_SINGLE, 14203 },
	{ 0xfc5e, 0xfc5e, PDF_CMAP_SINGLE, 14210 },
	{ 0xfc5f, 0xfc5f, PDF_CMAP_SINGLE, 15977 },
	{ 0xfc60, 0xfc60, PDF_CMAP_SINGLE, 15183 },
	{ 0xfc61, 0xfc61, PDF_CMAP_SINGLE, 14214 },
	{ 0xfc62, 0xfc62, PDF_CMAP_SINGLE, 17371 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 14216 },
	{ 0xfc64, 0xfc64, PDF_CMAP_SINGLE, 17372 },
	{ 0xfc65, 0xfc65, PDF_CMAP_SINGLE, 16077 },
	{ 0xfc66, 0xfc66, PDF_CMAP_SINGLE, 14231 },
	{ 0xfc67, 0xfc67, PDF_CMAP_SINGLE, 16909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 17373 },
	{ 0xfc69, 0xfc69, PDF_CMAP_SINGLE, 14268 },
	{ 0xfc6a, 0xfc6a, PDF_CMAP_SINGLE, 17374 },
	{ 0xfc6b, 0xfc6b, PDF_CMAP_SINGLE, 14296 },
	{ 0xfc6c, 0xfc6c, PDF_CMAP_SINGLE, 15217 },
	{ 0xfc6d, 0xfc6d, PDF_CMAP_SINGLE, 15994 },
	{ 0xfc6e, 0xfc6e, PDF_CMAP_SINGLE, 17375 },
	{ 0xfc6f, 0xfc6f, PDF_CMAP_SINGLE, 14556 },
	{ 0xfc70, 0xfc70, PDF_CMAP_SINGLE, 14304 },
	{ 0xfc71, 0xfc71, PDF_CMAP_SINGLE, 14318 },
	{ 0xfc72, 0xfc72, PDF_CMAP_SINGLE, 14324 },
	{ 0xfc73, 0xfc73, PDF_CMAP_SINGLE, 14355 },
	{ 0xfc74, 0xfc75, PDF_CMAP_RANGE, 17376 },
	{ 0xfc76, 0xfc76, PDF_CMAP_SINGLE, 16048 },
	{ 0xfc77, 0xfc77, PDF_CMAP_SINGLE, 17378 },
	{ 0xfc78, 0xfc78, PDF_CMAP_SINGLE, 16365 },
	{ 0xfc79, 0xfc79, PDF_CMAP_SINGLE, 14349 },
	{ 0xfc7a, 0xfc7a, PDF_CMAP_SINGLE, 16047 },
	{ 0xfc7b, 0xfc7b, PDF_CMAP_SINGLE, 16274 },
	{ 0xfc7c, 0xfc7c, PDF_CMAP_SINGLE, 14358 },
	{ 0xfc7d, 0xfc7d, PDF_CMAP_SINGLE, 14363 },
	{ 0xfc7e, 0xfc7e, PDF_CMAP_SINGLE, 16290 },
	{ 0xfca1, 0xfca1, PDF_CMAP_SINGLE, 14364 },
	{ 0xfca2, 0xfca3, PDF_CMAP_RANGE, 14366 },
	{ 0xfca4, 0xfca4, PDF_CMAP_SINGLE, 16049 },
	{ 0xfca5, 0xfca5, PDF_CMAP_SINGLE, 4938 },
	{ 0xfca6, 0xfca6, PDF_CMAP_SINGLE, 17379 },
	{ 0xfca7, 0xfca7, PDF_CMAP_SINGLE, 16361 },
	{ 0xfca8, 0xfca8, PDF_CMAP_SINGLE, 17380 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 16377 },
	{ 0xfcaa, 0xfcaa, PDF_CMAP_SINGLE, 15763 },
	{ 0xfcab, 0xfcab, PDF_CMAP_SINGLE, 17381 },
	{ 0xfcac, 0xfcac, PDF_CMAP_SINGLE, 15961 },
	{ 0xfcad, 0xfcad, PDF_CMAP_SINGLE, 17382 },
	{ 0xfcae, 0xfcae, PDF_CMAP_SINGLE, 14391 },
	{ 0xfcaf, 0xfcaf, PDF_CMAP_SINGLE, 16281 },
	{ 0xfcb0, 0xfcb0, PDF_CMAP_SINGLE, 16025 },
	{ 0xfcb1, 0xfcb1, PDF_CMAP_SINGLE, 15966 },
	{ 0xfcb2, 0xfcb2, PDF_CMAP_SINGLE, 15284 },
	{ 0xfcb3, 0xfcb3, PDF_CMAP_SINGLE, 16043 },
	{ 0xfcb4, 0xfcb5, PDF_CMAP_RANGE, 17383 },
	{ 0xfcb6, 0xfcb6, PDF_CMAP_SINGLE, 14430 },
	{ 0xfcb7, 0xfcb7, PDF_CMAP_SINGLE, 17385 },
	{ 0xfcb8, 0xfcb8, PDF_CMAP_SINGLE, 15806 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 16005 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 14435 },
	{ 0xfcbc, 0xfcbc, PDF_CMAP_SINGLE, 15724 },
	{ 0xfcbd, 0xfcbd, PDF_CMAP_SINGLE, 14449 },
	{ 0xfcbe, 0xfcbe, PDF_CMAP_SINGLE, 16096 },
	{ 0xfcbf, 0xfcbf, PDF_CMAP_SINGLE, 17386 },
	{ 0xfcc0, 0xfcc0, PDF_CMAP_SINGLE, 14460 },
	{ 0xfcc1, 0xfcc1, PDF_CMAP_SINGLE, 17387 },
	{ 0xfcc2, 0xfcc2, PDF_CMAP_SINGLE, 16056 },
	{ 0xfcc3, 0xfcc3, PDF_CMAP_SINGLE, 14466 },
	{ 0xfcc4, 0xfcc4, PDF_CMAP_SINGLE, 16027 },
	{ 0xfcc5, 0xfcc5, PDF_CMAP_SINGLE, 17388 },
	{ 0xfcc6, 0xfcc6, PDF_CMAP_SINGLE, 14472 },
	{ 0xfcc7, 0xfcc7, PDF_CMAP_SINGLE, 15374 },
	{ 0xfcc8, 0xfcc8, PDF_CMAP_SINGLE, 14382 },
	{ 0xfcc9, 0xfcc9, PDF_CMAP_SINGLE, 14490 },
	{ 0xfcca, 0xfcca, PDF_CMAP_SINGLE, 14475 },
	{ 0xfccb, 0xfccb, PDF_CMAP_SINGLE, 15364 },
	{ 0xfccc, 0xfccc, PDF_CMAP_SINGLE, 14057 },
	{ 0xfccd, 0xfccd, PDF_CMAP_SINGLE, 17085 },
	{ 0xfcce, 0xfcce, PDF_CMAP_SINGLE, 15309 },
	{ 0xfccf, 0xfccf, PDF_CMAP_SINGLE, 14495 },
	{ 0xfcd0, 0xfcd0, PDF_CMAP_SINGLE, 15838 },
	{ 0xfcd1, 0xfcd1, PDF_CMAP_SINGLE, 14235 },
	{ 0xfcd2, 0xfcd2, PDF_CMAP_SINGLE, 15315 },
	{ 0xfcd3, 0xfcd3, PDF_CMAP_SINGLE, 14511 },
	{ 0xfcd4, 0xfcd4, PDF_CMAP_SINGLE, 16023 },
	{ 0xfcd5, 0xfcd5, PDF_CMAP_SINGLE, 14550 },
	{ 0xfcd6, 0xfcd6, PDF_CMAP_SINGLE, 16181 },
	{ 0xfcd7, 0xfcd7, PDF_CMAP_SINGLE, 14560 },
	{ 0xfcd8, 0xfcd8, PDF_CMAP_SINGLE, 16437 },
	{ 0xfcd9, 0xfcd9, PDF_CMAP_SINGLE, 14576 },
	{ 0xfcda, 0xfcdb, PDF_CMAP_RANGE, 17389 },
	{ 0xfcdc, 0xfcdc, PDF_CMAP_SINGLE, 15357 },
	{ 0xfcdd, 0xfcde, PDF_CMAP_RANGE, 15377 },
	{ 0xfcdf, 0xfcdf, PDF_CMAP_SINGLE, 17391 },
	{ 0xfce0, 0xfce0, PDF_CMAP_SINGLE, 14213 },
	{ 0xfce1, 0xfce1, PDF_CMAP_SINGLE, 14594 },
	{ 0xfce2, 0xfce2, PDF_CMAP_SINGLE, 15836 },
	{ 0xfce3, 0xfce3, PDF_CMAP_SINGLE, 17392 },
	{ 0xfce4, 0xfce4, PDF_CMAP_SINGLE, 16298 },
	{ 0xfce5, 0xfce5, PDF_CMAP_SINGLE, 14603 },
	{ 0xfce6, 0xfce6, PDF_CMAP_SINGLE, 16153 },
	{ 0xfce7, 0xfce7, PDF_CMAP_SINGLE, 14606 },
	{ 0xfce8, 0xfce8, PDF_CMAP_SINGLE, 14617 },
	{ 0xfce9, 0xfce9, PDF_CMAP_SINGLE, 17393 },
	{ 0xfcea, 0xfcea, PDF_CMAP_SINGLE, 15420 },
	{ 0xfceb, 0xfceb, PDF_CMAP_SINGLE, 14618 },
	{ 0xfcec, 0xfcec, PDF_CMAP_SINGLE, 15418 },
	{ 0xfced, 0xfced, PDF_CMAP_SINGLE, 14619 },
	{ 0xfcee, 0xfcee, PDF_CMAP_SINGLE, 15974 },
	{ 0xfcef, 0xfcef, PDF_CMAP_SINGLE, 16162 },
	{ 0xfcf0, 0xfcf0, PDF_CMAP_SINGLE, 15742 },
	{ 0xfcf1, 0xfcf1, PDF_CMAP_SINGLE, 15985 },
	{ 0xfcf2, 0xfcf2, PDF_CMAP_SINGLE, 15417 },
	{ 0xfcf3, 0xfcf3, PDF_CMAP_SINGLE, 16033 },
	{ 0xfcf4, 0xfcf5, PDF_CMAP_RANGE, 17394 },
	{ 0xfcf6, 0xfcf6, PDF_CMAP_SINGLE, 16176 },
	{ 0xfcf7, 0xfcf7, PDF_CMAP_SINGLE, 14633 },
	{ 0xfcf8, 0xfcf8, PDF_CMAP_SINGLE, 16316 },
	{ 0xfcf9, 0xfcf9, PDF_CMAP_SINGLE, 14646 },
	{ 0xfcfa, 0xfcfa, PDF_CMAP_SINGLE, 17396 },
	{ 0xfcfb, 0xfcfb, PDF_CMAP_SINGLE, 14647 },
	{ 0xfcfc, 0xfcfc, PDF_CMAP_SINGLE, 16272 },
	{ 0xfcfd, 0xfcfd, PDF_CMAP_SINGLE, 17397 },
	{ 0xfcfe, 0xfcfe, PDF_CMAP_SINGLE, 2746 },
};

static const unsigned short pdf_cmap_HKm471_B5_H_table[424] =
{
	248,247,5551,5550,13585,13629,13641,15725,
	14788,17305,16207,15877,17306,15246,14058,14792,
	14653,17307,16418,17308,14789,17309,16436,16035,
	16038,17312,16465,16459,16463,16468,16760,16138,
	16474,16036,15805,17315,15807,16476,16483,17316,
	16509,16514,16237,17317,16523,17318,16185,16531,
	16534,17319,16536,15999,15479,16039,16899,16081,
	16010,16024,16012,16545,15747,14848,14755,16013,
	17320,16216,15804,16546,15564,15150,15302,16011,
	16031,17196,17321,16136,16037,16009,15811,16002,
	16001,16020,17322,16003,16008,17323,16017,17324,
	15996,16000,16015,17189,15975,15992,15976,17325,
	14290,16205,17326,15998,16572,15971,16573,17327,
	16236,16121,16567,16055,16109,16234,14758,17328,
	16021,14664,15560,16588,16006,16584,15482,15485,
	17332,16593,15730,16596,15398,14059,16602,16601,
	16575,15745,16625,16640,15493,17333,14877,14325,
	14062,15967,16662,17083,16671,17334,16664,16675,
	16674,16679,17337,16681,17338,15829,16111,15754,
	17339,14891,17340,16696,17341,16014,14896,14447,
	16707,16710,16708,16713,15269,16727,17342,16737,
	14060,16746,15748,16386,16774,16314,17343,16325,
	16208,16022,17344,16778,16321,15759,17345,16032,
	16150,16029,16782,16786,14950,16028,17295,16016,
	16797,17238,14248,16798,16793,16775,16034,16799,
	16007,16004,16046,15218,15993,15981,16050,17350,
	16044,16842,16884,17351,16848,15018,16862,16859,
	16872,17352,16374,16857,15682,16885,16888,16891,
	16901,15758,16910,15715,16930,16929,16931,16489,
	16431,16962,14501,16966,15605,15721,15614,16987,
	15048,15808,14826,16384,16996,16994,16383,16026,
	17011,14697,17018,17027,16282,16053,15127,17036,
	17040,15071,17357,17093,17096,16045,17358,16271,
	17359,15986,15984,17360,16042,17361,16297,17112,
	16371,17120,15104,17175,14381,15987,16341,15124,
	14056,17140,15118,17132,17136,15997,17367,17150,
	17157,16346,15857,14131,15741,17368,14138,14144,
	14146,14150,14162,16159,17369,14061,15178,14174,
	15173,17370,16094,14185,14188,14187,14190,14207,
	14203,14210,15977,15183,14214,17371,14216,17372,
	16077,14231,16909,17373,14268,17374,14296,15217,
	15994,17375,14556,14304,14318,14324,14355,16048,
	17378,16365,14349,16047,16274,14358,14363,16290,
	16049,4938,17379,16361,17380,16377,15763,17381,
	15961,17382,14391,16281,16025,15966,15284,16043,
	14430,17385,15806,16005,15724,14449,16096,17386,
	14460,17387,16056,14466,16027,17388,14472,15374,
	14382,14490,14475,15364,14057,17085,15309,14495,
	15838,14235,15315,14511,16023,14550,16181,14560,
	16437,14576,17391,14213,14594,15836,17392,16298,
	14603,16153,14606,14617,17393,15420,14618,15418,
	14619,15974,16162,15742,15985,15417,16033,16176,
	14633,16316,14646,17396,14647,16272,17397,2746,
};

pdf_cmap pdf_cmap_HKm471_B5_H =
{
	-1, "HKm471-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa140, 0xfefe },
	},
	689, 689, (pdf_range*) pdf_cmap_HKm471_B5_H_ranges,
	424, 424, (unsigned short*) pdf_cmap_HKm471_B5_H_table,
};

/* HKm471-B5-V */

static const pdf_range pdf_cmap_HKm471_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
};

static const unsigned short pdf_cmap_HKm471_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_HKm471_B5_V =
{
	-1, "HKm471-B5-V", "HKm471-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	12, 12, (pdf_range*) pdf_cmap_HKm471_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_HKm471_B5_V_table,
};

/* HKscs-B5-H */

static const pdf_range pdf_cmap_HKscs_B5_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8740, 0x8765, PDF_CMAP_RANGE, 18965 },
	{ 0x8767, 0x8779, PDF_CMAP_RANGE, 19003 },
	{ 0x877a, 0x877e, PDF_CMAP_RANGE, 19088 },
	{ 0x87a1, 0x87df, PDF_CMAP_RANGE, 19093 },
	{ 0x8840, 0x8855, PDF_CMAP_RANGE, 17609 },
	{ 0x8856, 0x887e, PDF_CMAP_RANGE, 18785 },
	{ 0x88a1, 0x88a8, PDF_CMAP_RANGE, 18826 },
	{ 0x88a9, 0x88aa, PDF_CMAP_RANGE, 18844 },
	{ 0x8940, 0x8941, PDF_CMAP_RANGE, 17716 },
	{ 0x8943, 0x8943, PDF_CMAP_SINGLE, 17718 },
	{ 0x8946, 0x8949, PDF_CMAP_RANGE, 17719 },
	{ 0x894c, 0x894d, PDF_CMAP_TABLE, 0 },
	{ 0x894e, 0x8950, PDF_CMAP_RANGE, 17724 },
	{ 0x8951, 0x8951, PDF_CMAP_SINGLE, 17306 },
	{ 0x8952, 0x897e, PDF_CMAP_RANGE, 17727 },
	{ 0x89a1, 0x89a5, PDF_CMAP_RANGE, 17772 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 17314 },
	{ 0x89ab, 0x89ae, PDF_CMAP_TABLE, 2 },
	{ 0x89b0, 0x89b2, PDF_CMAP_RANGE, 17779 },
	{ 0x89b5, 0x89bf, PDF_CMAP_RANGE, 17782 },
	{ 0x89c1, 0x89c3, PDF_CMAP_RANGE, 17793 },
	{ 0x89c5, 0x89ce, PDF_CMAP_RANGE, 17796 },
	{ 0x89cf, 0x89cf, PDF_CMAP_SINGLE, 17340 },
	{ 0x89d0, 0x89d8, PDF_CMAP_RANGE, 17806 },
	{ 0x89d9, 0x89dd, PDF_CMAP_TABLE, 6 },
	{ 0x89de, 0x89e0, PDF_CMAP_RANGE, 17817 },
	{ 0x89e1, 0x89e3, PDF_CMAP_TABLE, 11 },
	{ 0x89e4, 0x89e9, PDF_CMAP_RANGE, 17821 },
	{ 0x89ea, 0x89eb, PDF_CMAP_RANGE, 17317 },
	{ 0x89ec, 0x89f9, PDF_CMAP_RANGE, 17827 },
	{ 0x89fa, 0x89fa, PDF_CMAP_SINGLE, 17321 },
	{ 0x89fb, 0x89fe, PDF_CMAP_RANGE, 17841 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 14 },
	{ 0x8a43, 0x8a4c, PDF_CMAP_RANGE, 17163 },
	{ 0x8a4d, 0x8a4d, PDF_CMAP_SINGLE, 17846 },
	{ 0x8a4e, 0x8a59, PDF_CMAP_RANGE, 17174 },
	{ 0x8a5a, 0x8a5a, PDF_CMAP_SINGLE, 17847 },
	{ 0x8a5b, 0x8a5d, PDF_CMAP_RANGE, 17187 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 17848 },
	{ 0x8a5f, 0x8a62, PDF_CMAP_RANGE, 17191 },
	{ 0x8a64, 0x8a70, PDF_CMAP_RANGE, 17196 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 17849 },
	{ 0x8a72, 0x8a74, PDF_CMAP_RANGE, 17210 },
	{ 0x8a76, 0x8a77, PDF_CMAP_TABLE, 16 },
	{ 0x8a78, 0x8a79, PDF_CMAP_RANGE, 17216 },
	{ 0x8a7a, 0x8a7e, PDF_CMAP_TABLE, 18 },
	{ 0x8aa1, 0x8aa7, PDF_CMAP_RANGE, 17223 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 17854 },
	{ 0x8aa9, 0x8aaa, PDF_CMAP_RANGE, 17231 },
	{ 0x8aac, 0x8ab0, PDF_CMAP_RANGE, 17234 },
	{ 0x8ab2, 0x8ab5, PDF_CMAP_RANGE, 17240 },
	{ 0x8ab6, 0x8ab9, PDF_CMAP_TABLE, 23 },
	{ 0x8abb, 0x8ac7, PDF_CMAP_RANGE, 17249 },
	{ 0x8ac9, 0x8acb, PDF_CMAP_RANGE, 17263 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 17857 },
	{ 0x8ace, 0x8ad5, PDF_CMAP_RANGE, 17268 },
	{ 0x8ad6, 0x8ad7, PDF_CMAP_RANGE, 17858 },
	{ 0x8ad8, 0x8adc, PDF_CMAP_RANGE, 17278 },
	{ 0x8adf, 0x8ae5, PDF_CMAP_RANGE, 17285 },
	{ 0x8ae6, 0x8ae7, PDF_CMAP_TABLE, 27 },
	{ 0x8ae8, 0x8af4, PDF_CMAP_RANGE, 17861 },
	{ 0x8af6, 0x8afe, PDF_CMAP_RANGE, 17874 },
	{ 0x8b40, 0x8b40, PDF_CMAP_SINGLE, 17883 },
	{ 0x8b41, 0x8b42, PDF_CMAP_RANGE, 17292 },
	{ 0x8b43, 0x8b44, PDF_CMAP_RANGE, 17884 },
	{ 0x8b45, 0x8b4c, PDF_CMAP_TABLE, 29 },
	{ 0x8b4d, 0x8b50, PDF_CMAP_RANGE, 17298 },
	{ 0x8b51, 0x8b53, PDF_CMAP_RANGE, 17890 },
	{ 0x8b55, 0x8b57, PDF_CMAP_RANGE, 17893 },
	{ 0x8b58, 0x8b5b, PDF_CMAP_TABLE, 37 },
	{ 0x8b5c, 0x8b60, PDF_CMAP_RANGE, 17897 },
	{ 0x8b61, 0x8b61, PDF_CMAP_SINGLE, 17319 },
	{ 0x8b62, 0x8b67, PDF_CMAP_RANGE, 17902 },
	{ 0x8b68, 0x8b68, PDF_CMAP_SINGLE, 17324 },
	{ 0x8b69, 0x8b7e, PDF_CMAP_RANGE, 17908 },
	{ 0x8ba1, 0x8bbf, PDF_CMAP_RANGE, 17930 },
	{ 0x8bc0, 0x8bdc, PDF_CMAP_RANGE, 17631 },
	{ 0x8bde, 0x8bfd, PDF_CMAP_RANGE, 17660 },
	{ 0x8c40, 0x8c61, PDF_CMAP_RANGE, 18849 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 19022 },
	{ 0x8c63, 0x8c7e, PDF_CMAP_RANGE, 18883 },
	{ 0x8ca1, 0x8ca5, PDF_CMAP_RANGE, 18911 },
	{ 0x8ca7, 0x8cc5, PDF_CMAP_RANGE, 18916 },
	{ 0x8cc9, 0x8ccc, PDF_CMAP_RANGE, 18947 },
	{ 0x8cce, 0x8cda, PDF_CMAP_RANGE, 18951 },
	{ 0x8cdb, 0x8cdc, PDF_CMAP_TABLE, 41 },
	{ 0x8cdd, 0x8ce4, PDF_CMAP_RANGE, 19024 },
	{ 0x8ce6, 0x8cfe, PDF_CMAP_RANGE, 19032 },
	{ 0x8d40, 0x8d40, PDF_CMAP_SINGLE, 19057 },
	{ 0x8d42, 0x8d5f, PDF_CMAP_RANGE, 19058 },
	{ 0x8d60, 0x8d61, PDF_CMAP_RANGE, 17961 },
	{ 0x8d62, 0x8d62, PDF_CMAP_SINGLE, 17338 },
	{ 0x8d63, 0x8d67, PDF_CMAP_RANGE, 17963 },
	{ 0x8d68, 0x8d6a, PDF_CMAP_TABLE, 43 },
	{ 0x8d6b, 0x8d6d, PDF_CMAP_RANGE, 17968 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 17342 },
	{ 0x8d6f, 0x8d75, PDF_CMAP_RANGE, 17971 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 17343 },
	{ 0x8d77, 0x8d79, PDF_CMAP_RANGE, 17978 },
	{ 0x8d7a, 0x8d7c, PDF_CMAP_TABLE, 46 },
	{ 0x8d7d, 0x8d7e, PDF_CMAP_RANGE, 17982 },
	{ 0x8da1, 0x8da4, PDF_CMAP_RANGE, 17984 },
	{ 0x8da5, 0x8da5, PDF_CMAP_SINGLE, 17346 },
	{ 0x8da6, 0x8da7, PDF_CMAP_RANGE, 17988 },
	{ 0x8da8, 0x8da9, PDF_CMAP_TABLE, 49 },
	{ 0x8daa, 0x8db5, PDF_CMAP_RANGE, 17990 },
	{ 0x8db6, 0x8db6, PDF_CMAP_SINGLE, 17351 },
	{ 0x8db7, 0x8dc2, PDF_CMAP_RANGE, 18002 },
	{ 0x8dc3, 0x8dc3, PDF_CMAP_SINGLE, 17352 },
	{ 0x8dc4, 0x8df9, PDF_CMAP_RANGE, 18014 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 17401 },
	{ 0x8dfb, 0x8dfe, PDF_CMAP_RANGE, 18068 },
	{ 0x8e40, 0x8e44, PDF_CMAP_RANGE, 14123 },
	{ 0x8e45, 0x8e45, PDF_CMAP_SINGLE, 18072 },
	{ 0x8e46, 0x8e68, PDF_CMAP_RANGE, 14128 },
	{ 0x8e69, 0x8e6a, PDF_CMAP_TABLE, 51 },
	{ 0x8e6b, 0x8e6c, PDF_CMAP_RANGE, 18073 },
	{ 0x8e6d, 0x8e6e, PDF_CMAP_RANGE, 14166 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 11752 },
	{ 0x8e70, 0x8e75, PDF_CMAP_RANGE, 14169 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 18075 },
	{ 0x8e77, 0x8e7a, PDF_CMAP_RANGE, 14175 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 18076 },
	{ 0x8e7c, 0x8e7d, PDF_CMAP_RANGE, 14180 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 289 },
	{ 0x8ea1, 0x8ea5, PDF_CMAP_RANGE, 14182 },
	{ 0x8ea6, 0x8ea6, PDF_CMAP_SINGLE, 18077 },
	{ 0x8ea7, 0x8eaa, PDF_CMAP_RANGE, 14187 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 4203 },
	{ 0x8eac, 0x8eb3, PDF_CMAP_RANGE, 14192 },
	{ 0x8eb4, 0x8eb4, PDF_CMAP_SINGLE, 4902 },
	{ 0x8eb5, 0x8eb7, PDF_CMAP_RANGE, 14201 },
	{ 0x8eb8, 0x8eb8, PDF_CMAP_SINGLE, 18078 },
	{ 0x8eb9, 0x8ec8, PDF_CMAP_RANGE, 14205 },
	{ 0x8ec9, 0x8ec9, PDF_CMAP_SINGLE, 18079 },
	{ 0x8eca, 0x8ecc, PDF_CMAP_RANGE, 14221 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 1643 },
	{ 0x8ece, 0x8ecf, PDF_CMAP_RANGE, 14225 },
	{ 0x8ed0, 0x8ed0, PDF_CMAP_SINGLE, 4910 },
	{ 0x8ed1, 0x8ee4, PDF_CMAP_RANGE, 14228 },
	{ 0x8ee5, 0x8ee5, PDF_CMAP_SINGLE, 18080 },
	{ 0x8ee6, 0x8eee, PDF_CMAP_RANGE, 14248 },
	{ 0x8eef, 0x8eef, PDF_CMAP_SINGLE, 18081 },
	{ 0x8ef0, 0x8ef5, PDF_CMAP_RANGE, 14257 },
	{ 0x8ef6, 0x8ef6, PDF_CMAP_SINGLE, 18082 },
	{ 0x8ef7, 0x8efe, PDF_CMAP_RANGE, 14264 },
	{ 0x8f40, 0x8f56, PDF_CMAP_RANGE, 14272 },
	{ 0x8f57, 0x8f59, PDF_CMAP_TABLE, 53 },
	{ 0x8f5a, 0x8f5e, PDF_CMAP_RANGE, 14297 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 18084 },
	{ 0x8f60, 0x8f66, PDF_CMAP_RANGE, 14302 },
	{ 0x8f67, 0x8f69, PDF_CMAP_TABLE, 56 },
	{ 0x8f6a, 0x8f6d, PDF_CMAP_RANGE, 14311 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 8877 },
	{ 0x8f6f, 0x8f78, PDF_CMAP_RANGE, 14316 },
	{ 0x8f79, 0x8f79, PDF_CMAP_SINGLE, 18086 },
	{ 0x8f7a, 0x8f7e, PDF_CMAP_RANGE, 14327 },
	{ 0x8fa1, 0x8faf, PDF_CMAP_RANGE, 14332 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 18087 },
	{ 0x8fb1, 0x8fc4, PDF_CMAP_RANGE, 14348 },
	{ 0x8fc5, 0x8fc7, PDF_CMAP_TABLE, 59 },
	{ 0x8fc8, 0x8fc9, PDF_CMAP_RANGE, 14369 },
	{ 0x8fca, 0x8fcc, PDF_CMAP_TABLE, 62 },
	{ 0x8fcd, 0x8fd9, PDF_CMAP_RANGE, 14374 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 18091 },
	{ 0x8fdb, 0x8fe2, PDF_CMAP_RANGE, 14387 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 18092 },
	{ 0x8fe4, 0x8ffb, PDF_CMAP_RANGE, 14396 },
	{ 0x8ffc, 0x8ffe, PDF_CMAP_TABLE, 65 },
	{ 0x9040, 0x9054, PDF_CMAP_RANGE, 14422 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 18094 },
	{ 0x9056, 0x905b, PDF_CMAP_RANGE, 14444 },
	{ 0x905c, 0x905e, PDF_CMAP_RANGE, 18095 },
	{ 0x905f, 0x906c, PDF_CMAP_RANGE, 14451 },
	{ 0x906d, 0x906f, PDF_CMAP_TABLE, 68 },
	{ 0x9070, 0x9079, PDF_CMAP_RANGE, 14467 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 14057 },
	{ 0x907b, 0x907e, PDF_CMAP_RANGE, 14477 },
	{ 0x90a1, 0x90a5, PDF_CMAP_RANGE, 14481 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 18099 },
	{ 0x90a7, 0x90b7, PDF_CMAP_RANGE, 14486 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 18100 },
	{ 0x90b9, 0x90db, PDF_CMAP_RANGE, 14503 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 5009 },
	{ 0x90dd, 0x90f0, PDF_CMAP_RANGE, 14539 },
	{ 0x90f1, 0x90f1, PDF_CMAP_SINGLE, 5796 },
	{ 0x90f2, 0x90fe, PDF_CMAP_RANGE, 14560 },
	{ 0x9140, 0x9164, PDF_CMAP_RANGE, 14573 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 18101 },
	{ 0x9166, 0x916d, PDF_CMAP_RANGE, 14610 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 18102 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 14618 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 18103 },
	{ 0x91a1, 0x91a2, PDF_CMAP_TABLE, 71 },
	{ 0x91a3, 0x91be, PDF_CMAP_RANGE, 14634 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 12402 },
	{ 0x91c0, 0x91c7, PDF_CMAP_RANGE, 14663 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 18105 },
	{ 0x91c9, 0x91fe, PDF_CMAP_RANGE, 14672 },
	{ 0x9240, 0x9243, PDF_CMAP_RANGE, 14726 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 14728 },
	{ 0x9245, 0x9263, PDF_CMAP_RANGE, 14731 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 18106 },
	{ 0x9265, 0x926c, PDF_CMAP_RANGE, 14763 },
	{ 0x926d, 0x926d, PDF_CMAP_SINGLE, 18107 },
	{ 0x926e, 0x927e, PDF_CMAP_RANGE, 14772 },
	{ 0x92a1, 0x92ae, PDF_CMAP_RANGE, 14789 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 281 },
	{ 0x92b1, 0x92b2, PDF_CMAP_TABLE, 73 },
	{ 0x92b3, 0x92c7, PDF_CMAP_RANGE, 14803 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 16300 },
	{ 0x92c9, 0x92d0, PDF_CMAP_RANGE, 14825 },
	{ 0x92d1, 0x92d1, PDF_CMAP_SINGLE, 10620 },
	{ 0x92d2, 0x92e4, PDF_CMAP_RANGE, 14834 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 18108 },
	{ 0x92e6, 0x92f1, PDF_CMAP_RANGE, 14853 },
	{ 0x92f2, 0x92f2, PDF_CMAP_SINGLE, 18109 },
	{ 0x92f3, 0x92fe, PDF_CMAP_RANGE, 14865 },
	{ 0x9340, 0x9367, PDF_CMAP_RANGE, 14877 },
	{ 0x9368, 0x9368, PDF_CMAP_SINGLE, 18110 },
	{ 0x9369, 0x937e, PDF_CMAP_RANGE, 14917 },
	{ 0x93a1, 0x93a9, PDF_CMAP_RANGE, 14939 },
	{ 0x93aa, 0x93aa, PDF_CMAP_SINGLE, 18111 },
	{ 0x93ab, 0x93c1, PDF_CMAP_RANGE, 14948 },
	{ 0x93c2, 0x93c2, PDF_CMAP_SINGLE, 18112 },
	{ 0x93c3, 0x93e4, PDF_CMAP_RANGE, 14971 },
	{ 0x93e5, 0x93e5, PDF_CMAP_SINGLE, 18113 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 15005 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 18114 },
	{ 0x93e9, 0x93ea, PDF_CMAP_RANGE, 15008 },
	{ 0x93eb, 0x93eb, PDF_CMAP_SINGLE, 18115 },
	{ 0x93ec, 0x93fe, PDF_CMAP_RANGE, 15010 },
	{ 0x9440, 0x9445, PDF_CMAP_RANGE, 15029 },
	{ 0x9446, 0x9447, PDF_CMAP_TABLE, 75 },
	{ 0x9448, 0x9478, PDF_CMAP_RANGE, 15036 },
	{ 0x9479, 0x9479, PDF_CMAP_SINGLE, 18117 },
	{ 0x947a, 0x947e, PDF_CMAP_RANGE, 15086 },
	{ 0x94a1, 0x94c9, PDF_CMAP_RANGE, 15091 },
	{ 0x94ca, 0x94cb, PDF_CMAP_TABLE, 77 },
	{ 0x94cc, 0x94fe, PDF_CMAP_RANGE, 15134 },
	{ 0x9540, 0x954c, PDF_CMAP_RANGE, 15185 },
	{ 0x954d, 0x954d, PDF_CMAP_SINGLE, 18119 },
	{ 0x954e, 0x9559, PDF_CMAP_RANGE, 15198 },
	{ 0x955a, 0x955a, PDF_CMAP_SINGLE, 18120 },
	{ 0x955b, 0x955e, PDF_CMAP_RANGE, 15210 },
	{ 0x955f, 0x955f, PDF_CMAP_SINGLE, 18121 },
	{ 0x9560, 0x957e, PDF_CMAP_RANGE, 15215 },
	{ 0x95a1, 0x95c5, PDF_CMAP_RANGE, 15246 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 18122 },
	{ 0x95c7, 0x95d8, PDF_CMAP_RANGE, 15283 },
	{ 0x95d9, 0x95d9, PDF_CMAP_SINGLE, 6171 },
	{ 0x95da, 0x95fe, PDF_CMAP_RANGE, 15302 },
	{ 0x9640, 0x9643, PDF_CMAP_RANGE, 15339 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 15919 },
	{ 0x9645, 0x9650, PDF_CMAP_RANGE, 15344 },
	{ 0x9651, 0x9651, PDF_CMAP_SINGLE, 18123 },
	{ 0x9652, 0x9669, PDF_CMAP_RANGE, 15357 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 18124 },
	{ 0x966b, 0x967e, PDF_CMAP_RANGE, 15382 },
	{ 0x96a1, 0x96d3, PDF_CMAP_RANGE, 15402 },
	{ 0x96d4, 0x96d4, PDF_CMAP_SINGLE, 18125 },
	{ 0x96d5, 0x96ec, PDF_CMAP_RANGE, 15453 },
	{ 0x96ed, 0x96ed, PDF_CMAP_SINGLE, 15478 },
	{ 0x96ee, 0x96fb, PDF_CMAP_RANGE, 15478 },
	{ 0x96fc, 0x96fc, PDF_CMAP_SINGLE, 11044 },
	{ 0x96fd, 0x96fe, PDF_CMAP_RANGE, 15493 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 15495 },
	{ 0x97a1, 0x97fe, PDF_CMAP_RANGE, 15558 },
	{ 0x9840, 0x9843, PDF_CMAP_RANGE, 15652 },
	{ 0x9844, 0x9845, PDF_CMAP_RANGE, 18126 },
	{ 0x9846, 0x986e, PDF_CMAP_RANGE, 15658 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 18128 },
	{ 0x9870, 0x9874, PDF_CMAP_RANGE, 15700 },
	{ 0x9875, 0x9876, PDF_CMAP_RANGE, 18129 },
	{ 0x9877, 0x9877, PDF_CMAP_SINGLE, 15705 },
	{ 0x9878, 0x9879, PDF_CMAP_RANGE, 18131 },
	{ 0x987a, 0x987a, PDF_CMAP_SINGLE, 15706 },
	{ 0x987b, 0x987e, PDF_CMAP_RANGE, 18133 },
	{ 0x98a1, 0x98a2, PDF_CMAP_RANGE, 18137 },
	{ 0x98a3, 0x98a3, PDF_CMAP_SINGLE, 15707 },
	{ 0x98a4, 0x98ae, PDF_CMAP_RANGE, 18139 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 15708 },
	{ 0x98b0, 0x98b3, PDF_CMAP_RANGE, 18150 },
	{ 0x98b4, 0x98bc, PDF_CMAP_TABLE, 79 },
	{ 0x98bd, 0x98be, PDF_CMAP_RANGE, 15711 },
	{ 0x98bf, 0x98c1, PDF_CMAP_RANGE, 18158 },
	{ 0x98c2, 0x98c5, PDF_CMAP_TABLE, 88 },
	{ 0x98c6, 0x98c7, PDF_CMAP_RANGE, 15715 },
	{ 0x98c8, 0x98d1, PDF_CMAP_RANGE, 18163 },
	{ 0x98d2, 0x98d2, PDF_CMAP_SINGLE, 17357 },
	{ 0x98d3, 0x98d7, PDF_CMAP_RANGE, 18173 },
	{ 0x98d8, 0x98d9, PDF_CMAP_RANGE, 17358 },
	{ 0x98da, 0x98db, PDF_CMAP_TABLE, 92 },
	{ 0x98dc, 0x98de, PDF_CMAP_RANGE, 18179 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 17364 },
	{ 0x98e0, 0x98e2, PDF_CMAP_RANGE, 18182 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 15717 },
	{ 0x98e4, 0x98e6, PDF_CMAP_RANGE, 18185 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 15718 },
	{ 0x98e8, 0x98ec, PDF_CMAP_RANGE, 18188 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 15719 },
	{ 0x98ee, 0x98ef, PDF_CMAP_RANGE, 18193 },
	{ 0x98f0, 0x98f3, PDF_CMAP_TABLE, 94 },
	{ 0x98f4, 0x98f5, PDF_CMAP_RANGE, 17365 },
	{ 0x98f6, 0x98fb, PDF_CMAP_RANGE, 18197 },
	{ 0x98fc, 0x98fe, PDF_CMAP_TABLE, 98 },
	{ 0x9940, 0x9941, PDF_CMAP_RANGE, 18204 },
	{ 0x9942, 0x9947, PDF_CMAP_TABLE, 101 },
	{ 0x9948, 0x994e, PDF_CMAP_RANGE, 18208 },
	{ 0x994f, 0x994f, PDF_CMAP_SINGLE, 15725 },
	{ 0x9950, 0x9953, PDF_CMAP_RANGE, 18215 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 17369 },
	{ 0x9955, 0x995b, PDF_CMAP_RANGE, 18219 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 17370 },
	{ 0x995d, 0x9963, PDF_CMAP_RANGE, 18226 },
	{ 0x9964, 0x9964, PDF_CMAP_SINGLE, 17372 },
	{ 0x9965, 0x9969, PDF_CMAP_RANGE, 18233 },
	{ 0x996a, 0x996a, PDF_CMAP_SINGLE, 15726 },
	{ 0x996b, 0x996d, PDF_CMAP_RANGE, 18238 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 15727 },
	{ 0x996f, 0x9974, PDF_CMAP_RANGE, 18241 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 15728 },
	{ 0x9976, 0x9977, PDF_CMAP_RANGE, 18247 },
	{ 0x9978, 0x9978, PDF_CMAP_SINGLE, 15729 },
	{ 0x9979, 0x997e, PDF_CMAP_RANGE, 18249 },
	{ 0x99a1, 0x99a6, PDF_CMAP_TABLE, 107 },
	{ 0x99a7, 0x99ad, PDF_CMAP_RANGE, 18258 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 15731 },
	{ 0x99af, 0x99b1, PDF_CMAP_RANGE, 18265 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 17374 },
	{ 0x99b3, 0x99b5, PDF_CMAP_RANGE, 18268 },
	{ 0x99b6, 0x99b6, PDF_CMAP_SINGLE, 15732 },
	{ 0x99b7, 0x99b9, PDF_CMAP_RANGE, 18271 },
	{ 0x99ba, 0x99ba, PDF_CMAP_SINGLE, 15733 },
	{ 0x99bb, 0x99c9, PDF_CMAP_RANGE, 18274 },
	{ 0x99ca, 0x99ca, PDF_CMAP_SINGLE, 17376 },
	{ 0x99cb, 0x99cc, PDF_CMAP_RANGE, 18289 },
	{ 0x99cd, 0x99cd, PDF_CMAP_SINGLE, 17378 },
	{ 0x99ce, 0x99d2, PDF_CMAP_RANGE, 18291 },
	{ 0x99d3, 0x99d3, PDF_CMAP_SINGLE, 17379 },
	{ 0x99d4, 0x99d5, PDF_CMAP_RANGE, 18296 },
	{ 0x99d6, 0x99d6, PDF_CMAP_SINGLE, 17381 },
	{ 0x99d7, 0x99de, PDF_CMAP_RANGE, 18298 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 17375 },
	{ 0x99e0, 0x99e1, PDF_CMAP_RANGE, 18306 },
	{ 0x99e2, 0x99e8, PDF_CMAP_TABLE, 113 },
	{ 0x99e9, 0x99ee, PDF_CMAP_RANGE, 18311 },
	{ 0x99ef, 0x99ef, PDF_CMAP_SINGLE, 17405 },
	{ 0x99f0, 0x99f3, PDF_CMAP_RANGE, 18317 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 15735 },
	{ 0x99f5, 0x99fe, PDF_CMAP_RANGE, 18321 },
	{ 0x9a40, 0x9a49, PDF_CMAP_RANGE, 18331 },
	{ 0x9a4a, 0x9a4c, PDF_CMAP_TABLE, 120 },
	{ 0x9a4d, 0x9a58, PDF_CMAP_RANGE, 18342 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 15738 },
	{ 0x9a5a, 0x9a5e, PDF_CMAP_RANGE, 18354 },
	{ 0x9a5f, 0x9a61, PDF_CMAP_TABLE, 123 },
	{ 0x9a62, 0x9a65, PDF_CMAP_RANGE, 18360 },
	{ 0x9a66, 0x9a6b, PDF_CMAP_TABLE, 126 },
	{ 0x9a6c, 0x9a72, PDF_CMAP_RANGE, 18366 },
	{ 0x9a73, 0x9a75, PDF_CMAP_TABLE, 132 },
	{ 0x9a76, 0x9a7d, PDF_CMAP_RANGE, 18374 },
	{ 0x9a7e, 0x9a7e, PDF_CMAP_SINGLE, 15742 },
	{ 0x9aa1, 0x9aa2, PDF_CMAP_RANGE, 18382 },
	{ 0x9aa3, 0x9aa5, PDF_CMAP_TABLE, 135 },
	{ 0x9aa6, 0x9aa8, PDF_CMAP_RANGE, 18385 },
	{ 0x9aa9, 0x9aaa, PDF_CMAP_TABLE, 138 },
	{ 0x9aab, 0x9ab1, PDF_CMAP_RANGE, 18388 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 15743 },
	{ 0x9ab3, 0x9ab6, PDF_CMAP_RANGE, 18395 },
	{ 0x9ab7, 0x9abd, PDF_CMAP_TABLE, 140 },
	{ 0x9abe, 0x9ac6, PDF_CMAP_RANGE, 18402 },
	{ 0x9ac7, 0x9ac7, PDF_CMAP_SINGLE, 15747 },
	{ 0x9ac8, 0x9acf, PDF_CMAP_RANGE, 18411 },
	{ 0x9ad0, 0x9ad2, PDF_CMAP_TABLE, 147 },
	{ 0x9ad3, 0x9ad8, PDF_CMAP_RANGE, 18420 },
	{ 0x9ad9, 0x9adb, PDF_CMAP_RANGE, 15750 },
	{ 0x9adc, 0x9ae1, PDF_CMAP_RANGE, 18426 },
	{ 0x9ae2, 0x9ae4, PDF_CMAP_TABLE, 150 },
	{ 0x9ae5, 0x9ae7, PDF_CMAP_RANGE, 18433 },
	{ 0x9ae8, 0x9ae9, PDF_CMAP_TABLE, 153 },
	{ 0x9aea, 0x9aed, PDF_CMAP_RANGE, 18436 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 17330 },
	{ 0x9aef, 0x9af1, PDF_CMAP_RANGE, 18440 },
	{ 0x9af2, 0x9af2, PDF_CMAP_SINGLE, 15756 },
	{ 0x9af3, 0x9af5, PDF_CMAP_RANGE, 18443 },
	{ 0x9af6, 0x9af6, PDF_CMAP_SINGLE, 15757 },
	{ 0x9af7, 0x9afa, PDF_CMAP_RANGE, 18446 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 15758 },
	{ 0x9afc, 0x9afe, PDF_CMAP_RANGE, 18450 },
	{ 0x9b40, 0x9b45, PDF_CMAP_RANGE, 18453 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 15759 },
	{ 0x9b47, 0x9b49, PDF_CMAP_RANGE, 18459 },
	{ 0x9b4a, 0x9b4a, PDF_CMAP_SINGLE, 15760 },
	{ 0x9b4b, 0x9b53, PDF_CMAP_RANGE, 18462 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 15762 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 18471 },
	{ 0x9b58, 0x9b5d, PDF_CMAP_TABLE, 155 },
	{ 0x9b5e, 0x9b5f, PDF_CMAP_RANGE, 15766 },
	{ 0x9b60, 0x9b60, PDF_CMAP_SINGLE, 18477 },
	{ 0x9b62, 0x9b6f, PDF_CMAP_RANGE, 18478 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 15768 },
	{ 0x9b74, 0x9b75, PDF_CMAP_RANGE, 18492 },
	{ 0x9b76, 0x9b78, PDF_CMAP_TABLE, 161 },
	{ 0x9b79, 0x9b7a, PDF_CMAP_RANGE, 18494 },
	{ 0x9b7b, 0x9b7e, PDF_CMAP_TABLE, 164 },
	{ 0x9ba1, 0x9ba2, PDF_CMAP_TABLE, 168 },
	{ 0x9ba3, 0x9ba4, PDF_CMAP_RANGE, 15779 },
	{ 0x9ba5, 0x9ba6, PDF_CMAP_RANGE, 18498 },
	{ 0x9ba7, 0x9baa, PDF_CMAP_RANGE, 15781 },
	{ 0x9bab, 0x9bac, PDF_CMAP_TABLE, 170 },
	{ 0x9bad, 0x9bae, PDF_CMAP_RANGE, 18501 },
	{ 0x9baf, 0x9baf, PDF_CMAP_SINGLE, 15786 },
	{ 0x9bb0, 0x9bb1, PDF_CMAP_RANGE, 18503 },
	{ 0x9bb2, 0x9bb9, PDF_CMAP_RANGE, 15787 },
	{ 0x9bba, 0x9bbd, PDF_CMAP_RANGE, 18505 },
	{ 0x9bbe, 0x9bbf, PDF_CMAP_TABLE, 172 },
	{ 0x9bc0, 0x9bc5, PDF_CMAP_RANGE, 15796 },
	{ 0x9bc6, 0x9bc6, PDF_CMAP_SINGLE, 10969 },
	{ 0x9bc7, 0x9bc9, PDF_CMAP_RANGE, 18510 },
	{ 0x9bca, 0x9bcf, PDF_CMAP_TABLE, 174 },
	{ 0x9bd0, 0x9bd1, PDF_CMAP_RANGE, 15805 },
	{ 0x9bd2, 0x9bd5, PDF_CMAP_TABLE, 180 },
	{ 0x9bd6, 0x9bd7, PDF_CMAP_RANGE, 18518 },
	{ 0x9bd8, 0x9bda, PDF_CMAP_RANGE, 15809 },
	{ 0x9bdb, 0x9bdc, PDF_CMAP_RANGE, 18520 },
	{ 0x9bdd, 0x9be3, PDF_CMAP_TABLE, 184 },
	{ 0x9be4, 0x9be6, PDF_CMAP_RANGE, 18524 },
	{ 0x9be7, 0x9be8, PDF_CMAP_TABLE, 191 },
	{ 0x9be9, 0x9beb, PDF_CMAP_RANGE, 15817 },
	{ 0x9bec, 0x9bed, PDF_CMAP_TABLE, 193 },
	{ 0x9bee, 0x9bef, PDF_CMAP_RANGE, 15821 },
	{ 0x9bf0, 0x9bf2, PDF_CMAP_RANGE, 18529 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 15823 },
	{ 0x9bf4, 0x9bf5, PDF_CMAP_RANGE, 18532 },
	{ 0x9bf6, 0x9bf7, PDF_CMAP_TABLE, 195 },
	{ 0x9bf8, 0x9bf9, PDF_CMAP_RANGE, 15825 },
	{ 0x9bfa, 0x9bfa, PDF_CMAP_SINGLE, 18535 },
	{ 0x9bfb, 0x9bfc, PDF_CMAP_RANGE, 15827 },
	{ 0x9bfd, 0x9bfe, PDF_CMAP_RANGE, 18536 },
	{ 0x9c40, 0x9c41, PDF_CMAP_RANGE, 15829 },
	{ 0x9c42, 0x9c43, PDF_CMAP_TABLE, 197 },
	{ 0x9c44, 0x9c46, PDF_CMAP_RANGE, 15832 },
	{ 0x9c47, 0x9c4a, PDF_CMAP_TABLE, 199 },
	{ 0x9c4b, 0x9c4c, PDF_CMAP_RANGE, 18541 },
	{ 0x9c4d, 0x9c52, PDF_CMAP_RANGE, 15837 },
	{ 0x9c53, 0x9c56, PDF_CMAP_TABLE, 203 },
	{ 0x9c57, 0x9c5b, PDF_CMAP_RANGE, 15845 },
	{ 0x9c5c, 0x9c5d, PDF_CMAP_TABLE, 207 },
	{ 0x9c5e, 0x9c5f, PDF_CMAP_RANGE, 18546 },
	{ 0x9c60, 0x9c63, PDF_CMAP_TABLE, 209 },
	{ 0x9c64, 0x9c66, PDF_CMAP_RANGE, 15853 },
	{ 0x9c67, 0x9c6e, PDF_CMAP_TABLE, 213 },
	{ 0x9c6f, 0x9c72, PDF_CMAP_RANGE, 15859 },
	{ 0x9c73, 0x9c74, PDF_CMAP_RANGE, 18554 },
	{ 0x9c75, 0x9c76, PDF_CMAP_RANGE, 15863 },
	{ 0x9c77, 0x9c7a, PDF_CMAP_TABLE, 221 },
	{ 0x9c7b, 0x9c7c, PDF_CMAP_RANGE, 15867 },
	{ 0x9c7d, 0x9c7e, PDF_CMAP_TABLE, 225 },
	{ 0x9ca1, 0x9ca2, PDF_CMAP_RANGE, 15870 },
	{ 0x9ca3, 0x9ca4, PDF_CMAP_RANGE, 18559 },
	{ 0x9ca5, 0x9ca5, PDF_CMAP_SINGLE, 15872 },
	{ 0x9ca6, 0x9ca7, PDF_CMAP_RANGE, 18561 },
	{ 0x9ca8, 0x9ca9, PDF_CMAP_RANGE, 15873 },
	{ 0x9caa, 0x9cac, PDF_CMAP_TABLE, 227 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 15876 },
	{ 0x9caf, 0x9cb0, PDF_CMAP_RANGE, 18565 },
	{ 0x9cb1, 0x9cba, PDF_CMAP_RANGE, 15878 },
	{ 0x9cbb, 0x9cbd, PDF_CMAP_TABLE, 230 },
	{ 0x9cbe, 0x9cc2, PDF_CMAP_RANGE, 15890 },
	{ 0x9cc3, 0x9cc5, PDF_CMAP_RANGE, 18568 },
	{ 0x9cc6, 0x9ccd, PDF_CMAP_RANGE, 15895 },
	{ 0x9cce, 0x9cd0, PDF_CMAP_TABLE, 233 },
	{ 0x9cd1, 0x9cd3, PDF_CMAP_RANGE, 15905 },
	{ 0x9cd4, 0x9cd7, PDF_CMAP_RANGE, 18572 },
	{ 0x9cd8, 0x9cda, PDF_CMAP_RANGE, 15908 },
	{ 0x9cdb, 0x9cdb, PDF_CMAP_SINGLE, 18576 },
	{ 0x9cdc, 0x9ce5, PDF_CMAP_RANGE, 15911 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 18577 },
	{ 0x9ce7, 0x9ce9, PDF_CMAP_RANGE, 15921 },
	{ 0x9cea, 0x9cea, PDF_CMAP_SINGLE, 18578 },
	{ 0x9ceb, 0x9cec, PDF_CMAP_RANGE, 15924 },
	{ 0x9ced, 0x9ced, PDF_CMAP_SINGLE, 18579 },
	{ 0x9cee, 0x9cf9, PDF_CMAP_RANGE, 15926 },
	{ 0x9cfa, 0x9cfc, PDF_CMAP_RANGE, 18580 },
	{ 0x9cfd, 0x9cfe, PDF_CMAP_TABLE, 236 },
	{ 0x9d40, 0x9d40, PDF_CMAP_SINGLE, 17384 },
	{ 0x9d41, 0x9d45, PDF_CMAP_RANGE, 18584 },
	{ 0x9d46, 0x9d46, PDF_CMAP_SINGLE, 15939 },
	{ 0x9d47, 0x9d48, PDF_CMAP_RANGE, 18589 },
	{ 0x9d49, 0x9d49, PDF_CMAP_SINGLE, 15940 },
	{ 0x9d4a, 0x9d4b, PDF_CMAP_RANGE, 18591 },
	{ 0x9d4c, 0x9d4d, PDF_CMAP_RANGE, 15942 },
	{ 0x9d4e, 0x9d51, PDF_CMAP_TABLE, 238 },
	{ 0x9d52, 0x9d54, PDF_CMAP_RANGE, 18595 },
	{ 0x9d55, 0x9d57, PDF_CMAP_TABLE, 242 },
	{ 0x9d58, 0x9d59, PDF_CMAP_RANGE, 18599 },
	{ 0x9d5a, 0x9d5a, PDF_CMAP_SINGLE, 3340 },
	{ 0x9d5b, 0x9d60, PDF_CMAP_RANGE, 18601 },
	{ 0x9d61, 0x9d64, PDF_CMAP_TABLE, 245 },
	{ 0x9d65, 0x9d77, PDF_CMAP_RANGE, 18608 },
	{ 0x9d78, 0x9d79, PDF_CMAP_TABLE, 249 },
	{ 0x9d7a, 0x9d7d, PDF_CMAP_RANGE, 18627 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15951 },
	{ 0x9da1, 0x9da4, PDF_CMAP_RANGE, 18631 },
	{ 0x9da5, 0x9da8, PDF_CMAP_RANGE, 15952 },
	{ 0x9da9, 0x9dab, PDF_CMAP_TABLE, 251 },
	{ 0x9dac, 0x9dad, PDF_CMAP_RANGE, 15957 },
	{ 0x9dae, 0x9daf, PDF_CMAP_RANGE, 18637 },
	{ 0x9db0, 0x9db0, PDF_CMAP_SINGLE, 15960 },
	{ 0x9db1, 0x9db2, PDF_CMAP_RANGE, 18639 },
	{ 0x9db3, 0x9db7, PDF_CMAP_TABLE, 254 },
	{ 0x9db8, 0x9dbb, PDF_CMAP_RANGE, 18643 },
	{ 0x9dbc, 0x9dbd, PDF_CMAP_RANGE, 15964 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 18647 },
	{ 0x9dbf, 0x9dc0, PDF_CMAP_RANGE, 15966 },
	{ 0x9dc1, 0x9dc2, PDF_CMAP_RANGE, 18648 },
	{ 0x9dc3, 0x9dc4, PDF_CMAP_TABLE, 259 },
	{ 0x9dc5, 0x9dc6, PDF_CMAP_RANGE, 18650 },
	{ 0x9dc7, 0x9dc8, PDF_CMAP_RANGE, 15970 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_TABLE, 261 },
	{ 0x9dcb, 0x9dcc, PDF_CMAP_RANGE, 18653 },
	{ 0x9dcd, 0x9dd1, PDF_CMAP_RANGE, 15973 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 18655 },
	{ 0x9dd3, 0x9dd5, PDF_CMAP_RANGE, 15978 },
	{ 0x9dd6, 0x9dd9, PDF_CMAP_RANGE, 18656 },
	{ 0x9dda, 0x9dfb, PDF_CMAP_RANGE, 15981 },
	{ 0x9dfc, 0x9dfc, PDF_CMAP_SINGLE, 18660 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_RANGE, 16015 },
	{ 0x9e40, 0x9e42, PDF_CMAP_RANGE, 16017 },
	{ 0x9e43, 0x9e43, PDF_CMAP_SINGLE, 18661 },
	{ 0x9e44, 0x9e5e, PDF_CMAP_RANGE, 16021 },
	{ 0x9e5f, 0x9e5f, PDF_CMAP_SINGLE, 18662 },
	{ 0x9e60, 0x9e62, PDF_CMAP_RANGE, 16049 },
	{ 0x9e63, 0x9e63, PDF_CMAP_SINGLE, 18663 },
	{ 0x9e64, 0x9e65, PDF_CMAP_RANGE, 16052 },
	{ 0x9e66, 0x9e67, PDF_CMAP_RANGE, 18664 },
	{ 0x9e68, 0x9e6a, PDF_CMAP_TABLE, 263 },
	{ 0x9e6b, 0x9e70, PDF_CMAP_RANGE, 18667 },
	{ 0x9e71, 0x9e73, PDF_CMAP_TABLE, 266 },
	{ 0x9e74, 0x9e76, PDF_CMAP_RANGE, 18674 },
	{ 0x9e77, 0x9e78, PDF_CMAP_RANGE, 16058 },
	{ 0x9e79, 0x9e7e, PDF_CMAP_TABLE, 269 },
	{ 0x9ea1, 0x9ea2, PDF_CMAP_RANGE, 16063 },
	{ 0x9ea3, 0x9ea3, PDF_CMAP_SINGLE, 18680 },
	{ 0x9ea4, 0x9ea6, PDF_CMAP_RANGE, 16065 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 18681 },
	{ 0x9ea9, 0x9eab, PDF_CMAP_TABLE, 275 },
	{ 0x9eac, 0x9ead, PDF_CMAP_RANGE, 16070 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 18684 },
	{ 0x9eaf, 0x9eb1, PDF_CMAP_RANGE, 16072 },
	{ 0x9eb2, 0x9eb3, PDF_CMAP_RANGE, 18685 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 278 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 16076 },
	{ 0x9eb8, 0x9eb9, PDF_CMAP_TABLE, 280 },
	{ 0x9eba, 0x9ebb, PDF_CMAP_RANGE, 18689 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 16079 },
	{ 0x9ebd, 0x9ebe, PDF_CMAP_RANGE, 18691 },
	{ 0x9ebf, 0x9ec0, PDF_CMAP_RANGE, 16080 },
	{ 0x9ec1, 0x9ec3, PDF_CMAP_RANGE, 18693 },
	{ 0x9ec4, 0x9ec5, PDF_CMAP_RANGE, 16082 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 18696 },
	{ 0x9ec7, 0x9eca, PDF_CMAP_RANGE, 16084 },
	{ 0x9ecb, 0x9ecc, PDF_CMAP_RANGE, 18697 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 16089 },
	{ 0x9ece, 0x9ecf, PDF_CMAP_RANGE, 18699 },
	{ 0x9ed0, 0x9ed1, PDF_CMAP_RANGE, 16090 },
	{ 0x9ed2, 0x9ed3, PDF_CMAP_TABLE, 282 },
	{ 0x9ed4, 0x9ed5, PDF_CMAP_RANGE, 18702 },
	{ 0x9ed6, 0x9ed7, PDF_CMAP_RANGE, 16093 },
	{ 0x9ed8, 0x9ed9, PDF_CMAP_RANGE, 18704 },
	{ 0x9eda, 0x9eee, PDF_CMAP_RANGE, 16095 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 9398 },
	{ 0x9ef0, 0x9ef1, PDF_CMAP_RANGE, 16117 },
	{ 0x9ef2, 0x9ef2, PDF_CMAP_SINGLE, 18706 },
	{ 0x9ef3, 0x9ef5, PDF_CMAP_RANGE, 16119 },
	{ 0x9ef6, 0x9ef8, PDF_CMAP_RANGE, 18707 },
	{ 0x9ef9, 0x9efa, PDF_CMAP_RANGE, 16122 },
	{ 0x9efb, 0x9efe, PDF_CMAP_TABLE, 284 },
	{ 0x9f40, 0x9f42, PDF_CMAP_RANGE, 16127 },
	{ 0x9f43, 0x9f43, PDF_CMAP_SINGLE, 18711 },
	{ 0x9f44, 0x9f47, PDF_CMAP_RANGE, 16130 },
	{ 0x9f48, 0x9f48, PDF_CMAP_SINGLE, 18712 },
	{ 0x9f49, 0x9f4a, PDF_CMAP_RANGE, 16134 },
	{ 0x9f4b, 0x9f4c, PDF_CMAP_RANGE, 18713 },
	{ 0x9f4d, 0x9f5f, PDF_CMAP_RANGE, 16136 },
	{ 0x9f60, 0x9f60, PDF_CMAP_SINGLE, 13585 },
	{ 0x9f61, 0x9f65, PDF_CMAP_RANGE, 16156 },
	{ 0x9f66, 0x9f66, PDF_CMAP_SINGLE, 14661 },
	{ 0x9f67, 0x9f68, PDF_CMAP_RANGE, 18715 },
	{ 0x9f69, 0x9f6f, PDF_CMAP_RANGE, 16163 },
	{ 0x9f70, 0x9f70, PDF_CMAP_SINGLE, 18717 },
	{ 0x9f71, 0x9f7e, PDF_CMAP_RANGE, 16170 },
	{ 0x9fa1, 0x9fb4, PDF_CMAP_RANGE, 16184 },
	{ 0x9fb5, 0x9fb5, PDF_CMAP_SINGLE, 18718 },
	{ 0x9fb6, 0x9fba, PDF_CMAP_RANGE, 16204 },
	{ 0x9fbb, 0x9fbb, PDF_CMAP_SINGLE, 18719 },
	{ 0x9fbc, 0x9fbe, PDF_CMAP_RANGE, 16209 },
	{ 0x9fbf, 0x9fc1, PDF_CMAP_TABLE, 288 },
	{ 0x9fc2, 0x9fca, PDF_CMAP_RANGE, 16213 },
	{ 0x9fcb, 0x9fcc, PDF_CMAP_TABLE, 291 },
	{ 0x9fcd, 0x9fd3, PDF_CMAP_RANGE, 16224 },
	{ 0x9fd4, 0x9fd4, PDF_CMAP_SINGLE, 18723 },
	{ 0x9fd5, 0x9fd7, PDF_CMAP_RANGE, 16232 },
	{ 0x9fd8, 0x9fd8, PDF_CMAP_SINGLE, 14910 },
	{ 0x9fd9, 0x9fe3, PDF_CMAP_RANGE, 16236 },
	{ 0x9fe4, 0x9fe4, PDF_CMAP_SINGLE, 18724 },
	{ 0x9fe5, 0x9ff8, PDF_CMAP_RANGE, 16247 },
	{ 0x9ff9, 0x9ff9, PDF_CMAP_SINGLE, 18725 },
	{ 0x9ffa, 0x9ffe, PDF_CMAP_RANGE, 16267 },
	{ 0xa040, 0xa040, PDF_CMAP_SINGLE, 18726 },
	{ 0xa041, 0xa046, PDF_CMAP_RANGE, 16272 },
	{ 0xa047, 0xa047, PDF_CMAP_SINGLE, 18727 },
	{ 0xa048, 0xa054, PDF_CMAP_RANGE, 16278 },
	{ 0xa055, 0xa055, PDF_CMAP_SINGLE, 18728 },
	{ 0xa056, 0xa062, PDF_CMAP_RANGE, 16291 },
	{ 0xa063, 0xa063, PDF_CMAP_SINGLE, 14353 },
	{ 0xa064, 0xa06c, PDF_CMAP_RANGE, 16305 },
	{ 0xa06d, 0xa06d, PDF_CMAP_SINGLE, 18729 },
	{ 0xa06e, 0xa076, PDF_CMAP_RANGE, 16314 },
	{ 0xa077, 0xa077, PDF_CMAP_SINGLE, 1522 },
	{ 0xa078, 0xa07a, PDF_CMAP_RANGE, 16324 },
	{ 0xa07b, 0xa07b, PDF_CMAP_SINGLE, 18730 },
	{ 0xa07c, 0xa07e, PDF_CMAP_RANGE, 16327 },
	{ 0xa0a1, 0xa0a2, PDF_CMAP_TABLE, 293 },
	{ 0xa0a3, 0xa0a6, PDF_CMAP_RANGE, 16331 },
	{ 0xa0a7, 0xa0a7, PDF_CMAP_SINGLE, 18732 },
	{ 0xa0a8, 0xa0c4, PDF_CMAP_RANGE, 16335 },
	{ 0xa0c5, 0xa0c5, PDF_CMAP_SINGLE, 18733 },
	{ 0xa0c6, 0xa0cf, PDF_CMAP_RANGE, 16364 },
	{ 0xa0d0, 0xa0d0, PDF_CMAP_SINGLE, 18734 },
	{ 0xa0d1, 0xa0d4, PDF_CMAP_RANGE, 16374 },
	{ 0xa0d5, 0xa0d5, PDF_CMAP_SINGLE, 15086 },
	{ 0xa0d6, 0xa0de, PDF_CMAP_RANGE, 16379 },
	{ 0xa0df, 0xa0df, PDF_CMAP_SINGLE, 9341 },
	{ 0xa0e0, 0xa0e2, PDF_CMAP_RANGE, 16389 },
	{ 0xa0e3, 0xa0e4, PDF_CMAP_TABLE, 295 },
	{ 0xa0e5, 0xa0e6, PDF_CMAP_RANGE, 16393 },
	{ 0xa0e7, 0xa0ed, PDF_CMAP_RANGE, 18736 },
	{ 0xa0ee, 0xa0ee, PDF_CMAP_SINGLE, 17332 },
	{ 0xa0ef, 0xa0f1, PDF_CMAP_RANGE, 18743 },
	{ 0xa0f2, 0xa0f2, PDF_CMAP_SINGLE, 17336 },
	{ 0xa0f3, 0xa0fe, PDF_CMAP_RANGE, 18746 },
	{ 0xa140, 0xa158, PDF_CMAP_RANGE, 99 },
	{ 0xa159, 0xa15c, PDF_CMAP_RANGE, 13743 },
	{ 0xa15d, 0xa17e, PDF_CMAP_RANGE, 128 },
	{ 0xa1a1, 0xa1f5, PDF_CMAP_RANGE, 162 },
	{ 0xa1f6, 0xa1f7, PDF_CMAP_TABLE, 297 },
	{ 0xa1f8, 0xa1fe, PDF_CMAP_RANGE, 249 },
	{ 0xa240, 0xa27e, PDF_CMAP_RANGE, 256 },
	{ 0xa2a1, 0xa2fe, PDF_CMAP_RANGE, 319 },
	{ 0xa340, 0xa37e, PDF_CMAP_RANGE, 413 },
	{ 0xa3a1, 0xa3bb, PDF_CMAP_RANGE, 476 },
	{ 0xa3bd, 0xa3bf, PDF_CMAP_RANGE, 503 },
	{ 0xa440, 0xa47e, PDF_CMAP_RANGE, 595 },
	{ 0xa4a1, 0xa4fe, PDF_CMAP_RANGE, 658 },
	{ 0xa540, 0xa57e, PDF_CMAP_RANGE, 752 },
	{ 0xa5a1, 0xa5fe, PDF_CMAP_RANGE, 815 },
	{ 0xa640, 0xa67e, PDF_CMAP_RANGE, 909 },
	{ 0xa6a1, 0xa6fe, PDF_CMAP_RANGE, 972 },
	{ 0xa740, 0xa77e, PDF_CMAP_RANGE, 1066 },
	{ 0xa7a1, 0xa7fe, PDF_CMAP_RANGE, 1129 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 1223 },
	{ 0xa8a1, 0xa8fe, PDF_CMAP_RANGE, 1286 },
	{ 0xa940, 0xa97e, PDF_CMAP_RANGE, 1380 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 1443 },
	{ 0xaa40, 0xaa7e, PDF_CMAP_RANGE, 1537 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 1600 },
	{ 0xab40, 0xab7e, PDF_CMAP_RANGE, 1694 },
	{ 0xaba1, 0xabfe, PDF_CMAP_RANGE, 1757 },
	{ 0xac40, 0xac7e, PDF_CMAP_RANGE, 1851 },
	{ 0xaca1, 0xacfd, PDF_CMAP_RANGE, 1914 },
	{ 0xacfe, 0xacfe, PDF_CMAP_SINGLE, 2431 },
	{ 0xad40, 0xad7e, PDF_CMAP_RANGE, 2007 },
	{ 0xada1, 0xadfe, PDF_CMAP_RANGE, 2070 },
	{ 0xae40, 0xae7e, PDF_CMAP_RANGE, 2164 },
	{ 0xaea1, 0xaefe, PDF_CMAP_RANGE, 2227 },
	{ 0xaf40, 0xaf7e, PDF_CMAP_RANGE, 2321 },
	{ 0xafa1, 0xafcf, PDF_CMAP_RANGE, 2384 },
	{ 0xafd0, 0xaffe, PDF_CMAP_RANGE, 2432 },
	{ 0xb040, 0xb07e, PDF_CMAP_RANGE, 2479 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 2542 },
	{ 0xb140, 0xb17e, PDF_CMAP_RANGE, 2636 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 2699 },
	{ 0xb240, 0xb27e, PDF_CMAP_RANGE, 2793 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 2856 },
	{ 0xb340, 0xb37e, PDF_CMAP_RANGE, 2950 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 3013 },
	{ 0xb440, 0xb47e, PDF_CMAP_RANGE, 3107 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 3170 },
	{ 0xb540, 0xb57e, PDF_CMAP_RANGE, 3264 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 3327 },
	{ 0xb640, 0xb67e, PDF_CMAP_RANGE, 3421 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 3484 },
	{ 0xb740, 0xb77e, PDF_CMAP_RANGE, 3578 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 3641 },
	{ 0xb840, 0xb87e, PDF_CMAP_RANGE, 3735 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 3798 },
	{ 0xb940, 0xb97e, PDF_CMAP_RANGE, 3892 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 3955 },
	{ 0xba40, 0xba7e, PDF_CMAP_RANGE, 4049 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 4112 },
	{ 0xbb40, 0xbb7e, PDF_CMAP_RANGE, 4206 },
	{ 0xbba1, 0xbbc7, PDF_CMAP_RANGE, 4269 },
	{ 0xbbc8, 0xbbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xbc40, 0xbc7e, PDF_CMAP_RANGE, 4364 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 4427 },
	{ 0xbd40, 0xbd7e, PDF_CMAP_RANGE, 4521 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 4584 },
	{ 0xbe40, 0xbe51, PDF_CMAP_RANGE, 4678 },
	{ 0xbe52, 0xbe52, PDF_CMAP_SINGLE, 4308 },
	{ 0xbe53, 0xbe7e, PDF_CMAP_RANGE, 4696 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 4740 },
	{ 0xbf40, 0xbf7e, PDF_CMAP_RANGE, 4834 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 4897 },
	{ 0xc040, 0xc07e, PDF_CMAP_RANGE, 4991 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 5054 },
	{ 0xc140, 0xc17e, PDF_CMAP_RANGE, 5148 },
	{ 0xc1a1, 0xc1aa, PDF_CMAP_RANGE, 5211 },
	{ 0xc1ab, 0xc1fe, PDF_CMAP_RANGE, 5222 },
	{ 0xc240, 0xc27e, PDF_CMAP_RANGE, 5306 },
	{ 0xc2a1, 0xc2ca, PDF_CMAP_RANGE, 5369 },
	{ 0xc2cb, 0xc2cb, PDF_CMAP_SINGLE, 5221 },
	{ 0xc2cc, 0xc2fe, PDF_CMAP_RANGE, 5411 },
	{ 0xc340, 0xc360, PDF_CMAP_RANGE, 5462 },
	{ 0xc361, 0xc37e, PDF_CMAP_RANGE, 5496 },
	{ 0xc3a1, 0xc3b8, PDF_CMAP_RANGE, 5526 },
	{ 0xc3b9, 0xc3ba, PDF_CMAP_TABLE, 299 },
	{ 0xc3bb, 0xc3fe, PDF_CMAP_RANGE, 5552 },
	{ 0xc440, 0xc455, PDF_CMAP_RANGE, 5620 },
	{ 0xc456, 0xc456, PDF_CMAP_SINGLE, 5495 },
	{ 0xc457, 0xc47e, PDF_CMAP_RANGE, 5642 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 5682 },
	{ 0xc540, 0xc57e, PDF_CMAP_RANGE, 5776 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 5839 },
	{ 0xc640, 0xc67e, PDF_CMAP_RANGE, 5933 },
	{ 0xc6a1, 0xc6be, PDF_CMAP_RANGE, 506 },
	{ 0xc6bf, 0xc6ce, PDF_CMAP_RANGE, 537 },
	{ 0xc6d0, 0xc6d2, PDF_CMAP_RANGE, 554 },
	{ 0xc6d4, 0xc6d4, PDF_CMAP_SINGLE, 558 },
	{ 0xc6d6, 0xc6d6, PDF_CMAP_SINGLE, 560 },
	{ 0xc6d8, 0xc6dd, PDF_CMAP_RANGE, 13747 },
	{ 0xc6e0, 0xc6fe, PDF_CMAP_RANGE, 13754 },
	{ 0xc740, 0xc77e, PDF_CMAP_RANGE, 13785 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 13848 },
	{ 0xc840, 0xc87e, PDF_CMAP_RANGE, 13942 },
	{ 0xc8a1, 0xc8a4, PDF_CMAP_RANGE, 14005 },
	{ 0xc8cd, 0xc8d3, PDF_CMAP_RANGE, 14049 },
	{ 0xc8d4, 0xc8d6, PDF_CMAP_RANGE, 17606 },
	{ 0xc8d7, 0xc8df, PDF_CMAP_RANGE, 17692 },
	{ 0xc8e0, 0xc8e0, PDF_CMAP_SINGLE, 18846 },
	{ 0xc8e1, 0xc8e8, PDF_CMAP_RANGE, 17701 },
	{ 0xc8e9, 0xc8e9, PDF_CMAP_SINGLE, 18847 },
	{ 0xc8ea, 0xc8f0, PDF_CMAP_RANGE, 17709 },
	{ 0xc8f1, 0xc8f1, PDF_CMAP_SINGLE, 18848 },
	{ 0xc8f5, 0xc8fe, PDF_CMAP_RANGE, 18834 },
	{ 0xc940, 0xc949, PDF_CMAP_RANGE, 5996 },
	{ 0xc94a, 0xc94a, PDF_CMAP_SINGLE, 628 },
	{ 0xc94b, 0xc96b, PDF_CMAP_RANGE, 6006 },
	{ 0xc96c, 0xc97e, PDF_CMAP_RANGE, 6040 },
	{ 0xc9a1, 0xc9bd, PDF_CMAP_RANGE, 6059 },
	{ 0xc9be, 0xc9be, PDF_CMAP_SINGLE, 6039 },
	{ 0xc9bf, 0xc9ec, PDF_CMAP_RANGE, 6088 },
	{ 0xc9ed, 0xc9fe, PDF_CMAP_RANGE, 6135 },
	{ 0xca40, 0xca7e, PDF_CMAP_RANGE, 6153 },
	{ 0xcaa1, 0xcaf6, PDF_CMAP_RANGE, 6216 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 6134 },
	{ 0xcaf8, 0xcafe, PDF_CMAP_RANGE, 6302 },
	{ 0xcb40, 0xcb7e, PDF_CMAP_RANGE, 6309 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 6372 },
	{ 0xcc40, 0xcc7e, PDF_CMAP_RANGE, 6466 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 6529 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 6623 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 6686 },
	{ 0xce40, 0xce7e, PDF_CMAP_RANGE, 6780 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 6843 },
	{ 0xcf40, 0xcf7e, PDF_CMAP_RANGE, 6937 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 7000 },
	{ 0xd040, 0xd07e, PDF_CMAP_RANGE, 7094 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 7157 },
	{ 0xd140, 0xd17e, PDF_CMAP_RANGE, 7251 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c6, PDF_CMAP_TABLE, 301 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9d6, 0xf9fe, PDF_CMAP_RANGE, 14056 },
	{ 0xfa40, 0xfa5e, PDF_CMAP_RANGE, 16395 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 2106 },
	{ 0xfa60, 0xfa65, PDF_CMAP_RANGE, 16427 },
	{ 0xfa66, 0xfa66, PDF_CMAP_SINGLE, 2557 },
	{ 0xfa67, 0xfa7e, PDF_CMAP_RANGE, 16434 },
	{ 0xfaa1, 0xfaa8, PDF_CMAP_RANGE, 16458 },
	{ 0xfaa9, 0xfaaa, PDF_CMAP_RANGE, 18758 },
	{ 0xfaab, 0xfabc, PDF_CMAP_RANGE, 16468 },
	{ 0xfabd, 0xfabd, PDF_CMAP_SINGLE, 781 },
	{ 0xfabe, 0xfac4, PDF_CMAP_RANGE, 16487 },
	{ 0xfac5, 0xfac5, PDF_CMAP_SINGLE, 363 },
	{ 0xfac6, 0xfad4, PDF_CMAP_RANGE, 16495 },
	{ 0xfad5, 0xfad5, PDF_CMAP_SINGLE, 2144 },
	{ 0xfad6, 0xfafe, PDF_CMAP_RANGE, 16511 },
	{ 0xfb40, 0xfb47, PDF_CMAP_RANGE, 16552 },
	{ 0xfb48, 0xfb48, PDF_CMAP_SINGLE, 16002 },
	{ 0xfb49, 0xfb52, PDF_CMAP_RANGE, 16561 },
	{ 0xfb53, 0xfb53, PDF_CMAP_SINGLE, 18760 },
	{ 0xfb54, 0xfb6d, PDF_CMAP_RANGE, 16572 },
	{ 0xfb6e, 0xfb6e, PDF_CMAP_SINGLE, 18761 },
	{ 0xfb6f, 0xfb7e, PDF_CMAP_RANGE, 16599 },
	{ 0xfba1, 0xfba2, PDF_CMAP_RANGE, 16615 },
	{ 0xfba3, 0xfba3, PDF_CMAP_SINGLE, 18762 },
	{ 0xfba4, 0xfbb7, PDF_CMAP_RANGE, 16618 },
	{ 0xfbb8, 0xfbb8, PDF_CMAP_SINGLE, 3107 },
	{ 0xfbb9, 0xfbbe, PDF_CMAP_RANGE, 16639 },
	{ 0xfbbf, 0xfbbf, PDF_CMAP_SINGLE, 18763 },
	{ 0xfbc0, 0xfbcc, PDF_CMAP_RANGE, 16645 },
	{ 0xfbcd, 0xfbcd, PDF_CMAP_SINGLE, 18764 },
	{ 0xfbce, 0xfbf2, PDF_CMAP_RANGE, 16658 },
	{ 0xfbf3, 0xfbf3, PDF_CMAP_SINGLE, 6116 },
	{ 0xfbf4, 0xfbf8, PDF_CMAP_RANGE, 16696 },
	{ 0xfbf9, 0xfbf9, PDF_CMAP_SINGLE, 16014 },
	{ 0xfbfa, 0xfbfe, PDF_CMAP_RANGE, 16702 },
	{ 0xfc40, 0xfc49, PDF_CMAP_RANGE, 16707 },
	{ 0xfc4a, 0xfc4a, PDF_CMAP_SINGLE, 18765 },
	{ 0xfc4b, 0xfc4e, PDF_CMAP_RANGE, 16717 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 8495 },
	{ 0xfc50, 0xfc51, PDF_CMAP_RANGE, 16721 },
	{ 0xfc52, 0xfc52, PDF_CMAP_SINGLE, 18766 },
	{ 0xfc53, 0xfc62, PDF_CMAP_RANGE, 16723 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 18767 },
	{ 0xfc64, 0xfc6b, PDF_CMAP_RANGE, 16739 },
	{ 0xfc6c, 0xfc6d, PDF_CMAP_TABLE, 304 },
	{ 0xfc6e, 0xfc74, PDF_CMAP_RANGE, 16749 },
	{ 0xfc75, 0xfc75, PDF_CMAP_SINGLE, 18769 },
	{ 0xfc76, 0xfc7e, PDF_CMAP_RANGE, 16756 },
	{ 0xfca1, 0xfcb8, PDF_CMAP_RANGE, 16765 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 4447 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 16789 },
	{ 0xfcbc, 0xfcbd, PDF_CMAP_RANGE, 18770 },
	{ 0xfcbe, 0xfccb, PDF_CMAP_RANGE, 16792 },
	{ 0xfccc, 0xfccc, PDF_CMAP_SINGLE, 18772 },
	{ 0xfccd, 0xfce1, PDF_CMAP_RANGE, 16807 },
	{ 0xfce2, 0xfce3, PDF_CMAP_TABLE, 306 },
	{ 0xfce4, 0xfced, PDF_CMAP_RANGE, 16829 },
	{ 0xfcee, 0xfcee, PDF_CMAP_SINGLE, 18774 },
	{ 0xfcef, 0xfcf0, PDF_CMAP_RANGE, 16839 },
	{ 0xfcf1, 0xfcf1, PDF_CMAP_SINGLE, 1219 },
	{ 0xfcf2, 0xfcfe, PDF_CMAP_RANGE, 16842 },
	{ 0xfd40, 0xfd48, PDF_CMAP_RANGE, 16855 },
	{ 0xfd49, 0xfd49, PDF_CMAP_SINGLE, 18775 },
	{ 0xfd4a, 0xfd69, PDF_CMAP_RANGE, 16864 },
	{ 0xfd6a, 0xfd6a, PDF_CMAP_SINGLE, 18776 },
	{ 0xfd6b, 0xfd7e, PDF_CMAP_RANGE, 16897 },
	{ 0xfda1, 0xfdb6, PDF_CMAP_RANGE, 16917 },
	{ 0xfdb7, 0xfdb8, PDF_CMAP_TABLE, 308 },
	{ 0xfdb9, 0xfdba, PDF_CMAP_RANGE, 16941 },
	{ 0xfdbb, 0xfdbb, PDF_CMAP_SINGLE, 3237 },
	{ 0xfdbc, 0xfde2, PDF_CMAP_RANGE, 16944 },
	{ 0xfde3, 0xfde3, PDF_CMAP_SINGLE, 18777 },
	{ 0xfde4, 0xfdf0, PDF_CMAP_RANGE, 16984 },
	{ 0xfdf1, 0xfdf2, PDF_CMAP_TABLE, 310 },
	{ 0xfdf3, 0xfdfe, PDF_CMAP_RANGE, 16998 },
	{ 0xfe40, 0xfe51, PDF_CMAP_RANGE, 17010 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 15728 },
	{ 0xfe53, 0xfe6c, PDF_CMAP_RANGE, 17029 },
	{ 0xfe6d, 0xfe6f, PDF_CMAP_TABLE, 312 },
	{ 0xfe70, 0xfe77, PDF_CMAP_RANGE, 17057 },
	{ 0xfe78, 0xfe78, PDF_CMAP_SINGLE, 18780 },
	{ 0xfe79, 0xfe7e, PDF_CMAP_RANGE, 17065 },
	{ 0xfea1, 0xfea9, PDF_CMAP_RANGE, 17071 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 288 },
	{ 0xfeab, 0xfedc, PDF_CMAP_RANGE, 17080 },
	{ 0xfedd, 0xfedd, PDF_CMAP_SINGLE, 7080 },
	{ 0xfede, 0xfedf, PDF_CMAP_RANGE, 18781 },
	{ 0xfee0, 0xfeec, PDF_CMAP_RANGE, 17131 },
	{ 0xfeed, 0xfeee, PDF_CMAP_RANGE, 18783 },
	{ 0xfeef, 0xfefe, PDF_CMAP_RANGE, 17144 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 7314 },
	{ 0xd240, 0xd27e, PDF_CMAP_RANGE, 7408 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 7471 },
	{ 0xd340, 0xd37e, PDF_CMAP_RANGE, 7565 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 7628 },
	{ 0xd440, 0xd47e, PDF_CMAP_RANGE, 7722 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 7785 },
	{ 0xd540, 0xd57e, PDF_CMAP_RANGE, 7879 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 7942 },
	{ 0xd640, 0xd67e, PDF_CMAP_RANGE, 8036 },
	{ 0xd6a1, 0xd6cb, PDF_CMAP_RANGE, 8099 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 8788 },
	{ 0xd6cd, 0xd6fe, PDF_CMAP_RANGE, 8143 },
	{ 0xd740, 0xd779, PDF_CMAP_RANGE, 8193 },
	{ 0xd77a, 0xd77a, PDF_CMAP_SINGLE, 8889 },
	{ 0xd77b, 0xd77e, PDF_CMAP_RANGE, 8251 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 8255 },
	{ 0xd840, 0xd87e, PDF_CMAP_RANGE, 8349 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 8412 },
	{ 0xd940, 0xd97e, PDF_CMAP_RANGE, 8506 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 8569 },
	{ 0xda40, 0xda7e, PDF_CMAP_RANGE, 8663 },
	{ 0xdaa1, 0xdade, PDF_CMAP_RANGE, 8726 },
	{ 0xdadf, 0xdadf, PDF_CMAP_SINGLE, 8142 },
	{ 0xdae0, 0xdafe, PDF_CMAP_RANGE, 8789 },
	{ 0xdb40, 0xdb7e, PDF_CMAP_RANGE, 8820 },
	{ 0xdba1, 0xdba6, PDF_CMAP_RANGE, 8883 },
	{ 0xdba7, 0xdbfe, PDF_CMAP_RANGE, 8890 },
	{ 0xdc40, 0xdc7e, PDF_CMAP_RANGE, 8978 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 9041 },
	{ 0xdd40, 0xdd7e, PDF_CMAP_RANGE, 9135 },
	{ 0xdda1, 0xddfb, PDF_CMAP_RANGE, 9198 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 9089 },
	{ 0xddfd, 0xddfe, PDF_CMAP_RANGE, 9289 },
	{ 0xde40, 0xde7e, PDF_CMAP_RANGE, 9291 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 9354 },
	{ 0xdf40, 0xdf7e, PDF_CMAP_RANGE, 9448 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 9511 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 9605 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 9668 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 9762 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 9825 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 9919 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 9982 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 10076 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 10139 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 10233 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 10296 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 10390 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 10453 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 10547 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 10610 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 10704 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 10767 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 10861 },
	{ 0xe8a1, 0xe8a2, PDF_CMAP_RANGE, 10924 },
	{ 0xe8a3, 0xe8fe, PDF_CMAP_RANGE, 10927 },
	{ 0xe940, 0xe975, PDF_CMAP_RANGE, 11019 },
	{ 0xe976, 0xe97e, PDF_CMAP_RANGE, 11074 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 11083 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 11177 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 11240 },
	{ 0xeb40, 0xeb5a, PDF_CMAP_RANGE, 11334 },
	{ 0xeb5b, 0xeb7e, PDF_CMAP_RANGE, 11362 },
	{ 0xeba1, 0xebf0, PDF_CMAP_RANGE, 11398 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 10926 },
	{ 0xebf2, 0xebfe, PDF_CMAP_RANGE, 11478 },
	{ 0xec40, 0xec7e, PDF_CMAP_RANGE, 11491 },
	{ 0xeca1, 0xecdd, PDF_CMAP_RANGE, 11554 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 11073 },
	{ 0xecdf, 0xecfe, PDF_CMAP_RANGE, 11615 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 11647 },
	{ 0xeda1, 0xeda9, PDF_CMAP_RANGE, 11710 },
	{ 0xedaa, 0xedfe, PDF_CMAP_RANGE, 11720 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 11805 },
	{ 0xeea1, 0xeeea, PDF_CMAP_RANGE, 11868 },
	{ 0xeeeb, 0xeeeb, PDF_CMAP_SINGLE, 12308 },
	{ 0xeeec, 0xeefe, PDF_CMAP_RANGE, 11942 },
	{ 0xef40, 0xef7e, PDF_CMAP_RANGE, 11961 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 12024 },
	{ 0xf040, 0xf055, PDF_CMAP_RANGE, 12118 },
	{ 0xf056, 0xf056, PDF_CMAP_SINGLE, 11719 },
	{ 0xf057, 0xf07e, PDF_CMAP_RANGE, 12140 },
	{ 0xf0a1, 0xf0ca, PDF_CMAP_RANGE, 12180 },
	{ 0xf0cb, 0xf0cb, PDF_CMAP_SINGLE, 11361 },
	{ 0xf0cc, 0xf0fe, PDF_CMAP_RANGE, 12222 },
	{ 0xf140, 0xf162, PDF_CMAP_RANGE, 12273 },
	{ 0xf163, 0xf16a, PDF_CMAP_RANGE, 12309 },
	{ 0xf16b, 0xf16b, PDF_CMAP_SINGLE, 12640 },
	{ 0xf16c, 0xf17e, PDF_CMAP_RANGE, 12317 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 12336 },
	{ 0xf240, 0xf267, PDF_CMAP_RANGE, 12430 },
	{ 0xf268, 0xf268, PDF_CMAP_SINGLE, 12783 },
	{ 0xf269, 0xf27e, PDF_CMAP_RANGE, 12470 },
	{ 0xf2a1, 0xf2c2, PDF_CMAP_RANGE, 12492 },
	{ 0xf2c3, 0xf2fe, PDF_CMAP_RANGE, 12527 },
	{ 0xf340, 0xf374, PDF_CMAP_RANGE, 12587 },
	{ 0xf375, 0xf37e, PDF_CMAP_RANGE, 12641 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 12651 },
	{ 0xf440, 0xf465, PDF_CMAP_RANGE, 12745 },
	{ 0xf466, 0xf47e, PDF_CMAP_RANGE, 12784 },
	{ 0xf4a1, 0xf4b4, PDF_CMAP_RANGE, 12809 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 12526 },
	{ 0xf4b6, 0xf4fc, PDF_CMAP_RANGE, 12829 },
	{ 0xf4fd, 0xf4fe, PDF_CMAP_RANGE, 12901 },
	{ 0xf540, 0xf57e, PDF_CMAP_RANGE, 12903 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 12966 },
	{ 0xf640, 0xf662, PDF_CMAP_RANGE, 13060 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 12900 },
	{ 0xf664, 0xf67e, PDF_CMAP_RANGE, 13095 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 13122 },
	{ 0xf740, 0xf77e, PDF_CMAP_RANGE, 13216 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 13279 },
	{ 0xf840, 0xf87e, PDF_CMAP_RANGE, 13373 },
	{ 0xf8a1, 0xf8fe, PDF_CMAP_RANGE, 13436 },
	{ 0xf940, 0xf976, PDF_CMAP_RANGE, 13530 },
	{ 0xf977, 0xf97e, PDF_CMAP_RANGE, 13586 },
	{ 0xf9a1, 0xf9c3, PDF_CMAP_RANGE, 13594 },
	{ 0xf9c4, 0xf9c4, PDF_CMAP_SINGLE, 13585 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 13629 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 13641 },
	{ 0xf9c7, 0xf9d1, PDF_CMAP_RANGE, 13630 },
	{ 0xf9d2, 0xf9d5, PDF_CMAP_RANGE, 13642 },
	{ 0xf9d6, 0xf9fe, PDF_CMAP_RANGE, 14056 },
	{ 0xfa40, 0xfa5e, PDF_CMAP_RANGE, 16395 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 2106 },
	{ 0xfa60, 0xfa65, PDF_CMAP_RANGE, 16427 },
	{ 0xfa66, 0xfa66, PDF_CMAP_SINGLE, 2557 },
	{ 0xfa67, 0xfa7e, PDF_CMAP_RANGE, 16434 },
	{ 0xfaa1, 0xfaa8, PDF_CMAP_RANGE, 16458 },
	{ 0xfaa9, 0xfaaa, PDF_CMAP_RANGE, 18758 },
	{ 0xfaab, 0xfabc, PDF_CMAP_RANGE, 16468 },
	{ 0xfabd, 0xfabd, PDF_CMAP_SINGLE, 781 },
	{ 0xfabe, 0xfac4, PDF_CMAP_RANGE, 16487 },
	{ 0xfac5, 0xfac5, PDF_CMAP_SINGLE, 363 },
	{ 0xfac6, 0xfad4, PDF_CMAP_RANGE, 16495 },
	{ 0xfad5, 0xfad5, PDF_CMAP_SINGLE, 2144 },
	{ 0xfad6, 0xfafe, PDF_CMAP_RANGE, 16511 },
	{ 0xfb40, 0xfb47, PDF_CMAP_RANGE, 16552 },
	{ 0xfb48, 0xfb48, PDF_CMAP_SINGLE, 16002 },
	{ 0xfb49, 0xfb52, PDF_CMAP_RANGE, 16561 },
	{ 0xfb53, 0xfb53, PDF_CMAP_SINGLE, 18760 },
	{ 0xfb54, 0xfb6d, PDF_CMAP_RANGE, 16572 },
	{ 0xfb6e, 0xfb6e, PDF_CMAP_SINGLE, 18761 },
	{ 0xfb6f, 0xfb7e, PDF_CMAP_RANGE, 16599 },
	{ 0xfba1, 0xfba2, PDF_CMAP_RANGE, 16615 },
	{ 0xfba3, 0xfba3, PDF_CMAP_SINGLE, 18762 },
	{ 0xfba4, 0xfbb7, PDF_CMAP_RANGE, 16618 },
	{ 0xfbb8, 0xfbb8, PDF_CMAP_SINGLE, 3107 },
	{ 0xfbb9, 0xfbbe, PDF_CMAP_RANGE, 16639 },
	{ 0xfbbf, 0xfbbf, PDF_CMAP_SINGLE, 18763 },
	{ 0xfbc0, 0xfbcc, PDF_CMAP_RANGE, 16645 },
	{ 0xfbcd, 0xfbcd, PDF_CMAP_SINGLE, 18764 },
	{ 0xfbce, 0xfbf2, PDF_CMAP_RANGE, 16658 },
	{ 0xfbf3, 0xfbf3, PDF_CMAP_SINGLE, 6116 },
	{ 0xfbf4, 0xfbf8, PDF_CMAP_RANGE, 16696 },
	{ 0xfbf9, 0xfbf9, PDF_CMAP_SINGLE, 16014 },
	{ 0xfbfa, 0xfbfe, PDF_CMAP_RANGE, 16702 },
	{ 0xfc40, 0xfc49, PDF_CMAP_RANGE, 16707 },
	{ 0xfc4a, 0xfc4a, PDF_CMAP_SINGLE, 18765 },
	{ 0xfc4b, 0xfc4e, PDF_CMAP_RANGE, 16717 },
	{ 0xfc4f, 0xfc4f, PDF_CMAP_SINGLE, 8495 },
	{ 0xfc50, 0xfc51, PDF_CMAP_RANGE, 16721 },
	{ 0xfc52, 0xfc52, PDF_CMAP_SINGLE, 18766 },
	{ 0xfc53, 0xfc62, PDF_CMAP_RANGE, 16723 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 18767 },
	{ 0xfc64, 0xfc6b, PDF_CMAP_RANGE, 16739 },
	{ 0xfc6c, 0xfc6c, PDF_CMAP_SINGLE, 16385 },
	{ 0xfc6d, 0xfc6d, PDF_CMAP_SINGLE, 18768 },
	{ 0xfc6e, 0xfc74, PDF_CMAP_RANGE, 16749 },
	{ 0xfc75, 0xfc75, PDF_CMAP_SINGLE, 18769 },
	{ 0xfc76, 0xfc7e, PDF_CMAP_RANGE, 16756 },
	{ 0xfca1, 0xfcb8, PDF_CMAP_RANGE, 16765 },
	{ 0xfcb9, 0xfcb9, PDF_CMAP_SINGLE, 4447 },
	{ 0xfcba, 0xfcbb, PDF_CMAP_RANGE, 16789 },
	{ 0xfcbc, 0xfcbd, PDF_CMAP_RANGE, 18770 },
	{ 0xfcbe, 0xfccb, PDF_CMAP_RANGE, 16792 },
	{ 0xfccc, 0xfccc, PDF_CMAP_SINGLE, 18772 },
	{ 0xfccd, 0xfce1, PDF_CMAP_RANGE, 16807 },
	{ 0xfce2, 0xfce2, PDF_CMAP_SINGLE, 3193 },
	{ 0xfce3, 0xfce3, PDF_CMAP_SINGLE, 18773 },
	{ 0xfce4, 0xfced, PDF_CMAP_RANGE, 16829 },
	{ 0xfcee, 0xfcee, PDF_CMAP_SINGLE, 18774 },
	{ 0xfcef, 0xfcf0, PDF_CMAP_RANGE, 16839 },
	{ 0xfcf1, 0xfcf1, PDF_CMAP_SINGLE, 1219 },
	{ 0xfcf2, 0xfcfe, PDF_CMAP_RANGE, 16842 },
	{ 0xfd40, 0xfd48, PDF_CMAP_RANGE, 16855 },
	{ 0xfd49, 0xfd49, PDF_CMAP_SINGLE, 18775 },
	{ 0xfd4a, 0xfd69, PDF_CMAP_RANGE, 16864 },
	{ 0xfd6a, 0xfd6a, PDF_CMAP_SINGLE, 18776 },
	{ 0xfd6b, 0xfd7e, PDF_CMAP_RANGE, 16897 },
	{ 0xfda1, 0xfdb6, PDF_CMAP_RANGE, 16917 },
	{ 0xfdb7, 0xfdb7, PDF_CMAP_SINGLE, 6333 },
	{ 0xfdb8, 0xfdb8, PDF_CMAP_SINGLE, 3261 },
	{ 0xfdb9, 0xfdba, PDF_CMAP_RANGE, 16941 },
	{ 0xfdbb, 0xfdbb, PDF_CMAP_SINGLE, 3237 },
	{ 0xfdbc, 0xfde2, PDF_CMAP_RANGE, 16944 },
	{ 0xfde3, 0xfde3, PDF_CMAP_SINGLE, 18777 },
	{ 0xfde4, 0xfdf0, PDF_CMAP_RANGE, 16984 },
	{ 0xfdf1, 0xfdf1, PDF_CMAP_SINGLE, 3278 },
	{ 0xfdf2, 0xfdf2, PDF_CMAP_SINGLE, 18778 },
	{ 0xfdf3, 0xfdfe, PDF_CMAP_RANGE, 16998 },
	{ 0xfe40, 0xfe51, PDF_CMAP_RANGE, 17010 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 15728 },
	{ 0xfe53, 0xfe6c, PDF_CMAP_RANGE, 17029 },
	{ 0xfe6d, 0xfe6d, PDF_CMAP_SINGLE, 18779 },
	{ 0xfe6e, 0xfe6e, PDF_CMAP_SINGLE, 17055 },
	{ 0xfe6f, 0xfe6f, PDF_CMAP_SINGLE, 3716 },
	{ 0xfe70, 0xfe77, PDF_CMAP_RANGE, 17057 },
	{ 0xfe78, 0xfe78, PDF_CMAP_SINGLE, 18780 },
	{ 0xfe79, 0xfe7e, PDF_CMAP_RANGE, 17065 },
	{ 0xfea1, 0xfea9, PDF_CMAP_RANGE, 17071 },
	{ 0xfeaa, 0xfeaa, PDF_CMAP_SINGLE, 288 },
	{ 0xfeab, 0xfedc, PDF_CMAP_RANGE, 17080 },
	{ 0xfedd, 0xfedd, PDF_CMAP_SINGLE, 7080 },
	{ 0xfede, 0xfedf, PDF_CMAP_RANGE, 18781 },
	{ 0xfee0, 0xfeec, PDF_CMAP_RANGE, 17131 },
	{ 0xfeed, 0xfeee, PDF_CMAP_RANGE, 18783 },
	{ 0xfeef, 0xfefe, PDF_CMAP_RANGE, 17144 },
};

static const unsigned short pdf_cmap_HKscs_B5_H_table[315] =
{
	17723,17347,17388,17777,17387,17778,17308,17815,
	17310,17816,17311,17313,17820,17315,17845,17161,
	17214,17850,17851,17219,17852,17221,17853,17855,
	17245,17856,17247,17860,17371,17294,17886,17295,
	17887,17296,17888,17297,17889,17303,17896,17304,
	17348,19023,18964,17339,17312,17341,17344,17981,
	17345,17337,17325,4181,14164,3381,14296,18083,
	18085,14309,10178,18088,14368,18089,18090,4940,
	16377,18093,14420,2510,3015,14466,18098,14633,
	18104,284,283,18116,7430,10657,18118,17354,
	18154,15709,18155,17356,15710,18156,17402,18157,
	15713,18161,15714,18162,18178,17361,15720,18195,
	15721,18196,15722,18203,17367,17404,15723,18206,
	15724,18207,17368,18255,15730,18256,17349,18257,
	17350,15734,18308,17323,18309,17383,18310,17385,
	15736,18341,15737,17327,18359,15739,17389,18364,
	15740,17390,18365,17407,15741,18373,17393,17395,
	18384,17394,17400,17396,15744,18399,15745,18400,
	15746,18401,17335,15748,18419,15749,15753,18432,
	15754,15755,17328,15763,18474,15764,18475,15765,
	18476,12112,15773,5925,13037,15776,18496,15777,
	15778,18497,18500,15785,15795,18509,15803,18513,
	15804,18514,17360,18515,18516,15807,18517,15808,
	15812,7188,15813,18522,15814,18523,15815,15816,
	18527,16890,18528,4841,18534,11438,18538,18539,
	15835,18540,15836,6756,18543,15844,18544,18545,
	15850,15851,18548,5124,18549,18550,8996,18551,
	15857,13418,18552,15858,18553,8849,18556,15866,
	18557,18558,15869,18563,15875,18564,18567,6023,
	2399,18571,15903,7833,15938,18583,18593,15944,
	18594,15945,15946,18598,9665,17353,15948,18607,
	15949,17397,15950,18635,15956,18636,15961,18641,
	15962,18642,15963,15968,1510,18652,15972,16054,
	18666,16055,16056,18673,16057,18677,16060,18678,
	16061,18679,16062,1832,16069,18683,16075,18687,
	18688,16078,18701,16092,18710,16124,6150,16126,
	18720,16212,18721,3970,18722,16330,18731,18735,
	15049,248,247,5551,5550,13585,13629,13641,
	16385,18768,3193,18773,6333,3261,3278,18778,
	18779,17055,3716,
};

pdf_cmap pdf_cmap_HKscs_B5_H =
{
	-1, "HKscs-B5-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8740, 0xfefe },
	},
	1212, 1212, (pdf_range*) pdf_cmap_HKscs_B5_H_ranges,
	315, 315, (unsigned short*) pdf_cmap_HKscs_B5_H_table,
};

/* HKscs-B5-V */

static const pdf_range pdf_cmap_HKscs_B5_V_ranges[] =
{
	{ 0xa14b, 0xa14b, PDF_CMAP_SINGLE, 13646 },
	{ 0xa15a, 0xa15a, PDF_CMAP_SINGLE, 13743 },
	{ 0xa15c, 0xa15c, PDF_CMAP_SINGLE, 13745 },
	{ 0xa15d, 0xa15e, PDF_CMAP_RANGE, 130 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 134 },
	{ 0xa165, 0xa166, PDF_CMAP_RANGE, 138 },
	{ 0xa169, 0xa16a, PDF_CMAP_RANGE, 142 },
	{ 0xa16d, 0xa16e, PDF_CMAP_RANGE, 146 },
	{ 0xa171, 0xa172, PDF_CMAP_RANGE, 150 },
	{ 0xa175, 0xa176, PDF_CMAP_RANGE, 154 },
	{ 0xa179, 0xa17a, PDF_CMAP_RANGE, 158 },
	{ 0xa1e3, 0xa1e3, PDF_CMAP_SINGLE, 13647 },
	{ 0xc6e4, 0xc6e5, PDF_CMAP_RANGE, 14097 },
};

static const unsigned short pdf_cmap_HKscs_B5_V_table[1] = { 0 };

pdf_cmap pdf_cmap_HKscs_B5_V =
{
	-1, "HKscs-B5-V", "HKscs-B5-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	13, 13, (pdf_range*) pdf_cmap_HKscs_B5_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_HKscs_B5_V_table,
};

/* UniCNS-UCS2-H */

static const pdf_range pdf_cmap_UniCNS_UCS2_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a2, 0x00a3, PDF_CMAP_RANGE, 262 },
	{ 0x00a5, 0x00a5, PDF_CMAP_SINGLE, 260 },
	{ 0x00a7, 0x00a8, PDF_CMAP_TABLE, 0 },
	{ 0x00ac, 0x00ac, PDF_CMAP_SINGLE, 14049 },
	{ 0x00b0, 0x00b1, PDF_CMAP_TABLE, 2 },
	{ 0x00b7, 0x00b7, PDF_CMAP_SINGLE, 115 },
	{ 0x00c0, 0x00c1, PDF_CMAP_TABLE, 4 },
	{ 0x00c8, 0x00ca, PDF_CMAP_TABLE, 6 },
	{ 0x00d2, 0x00d3, PDF_CMAP_TABLE, 9 },
	{ 0x00d7, 0x00d7, PDF_CMAP_SINGLE, 210 },
	{ 0x00e0, 0x00e1, PDF_CMAP_TABLE, 11 },
	{ 0x00e8, 0x00ea, PDF_CMAP_TABLE, 13 },
	{ 0x00ec, 0x00ed, PDF_CMAP_TABLE, 16 },
	{ 0x00f2, 0x00f3, PDF_CMAP_TABLE, 18 },
	{ 0x00f7, 0x00fa, PDF_CMAP_TABLE, 20 },
	{ 0x00fc, 0x00fc, PDF_CMAP_SINGLE, 18827 },
	{ 0x0100, 0x0101, PDF_CMAP_TABLE, 24 },
	{ 0x0112, 0x0113, PDF_CMAP_TABLE, 26 },
	{ 0x011a, 0x011b, PDF_CMAP_TABLE, 28 },
	{ 0x012b, 0x012b, PDF_CMAP_SINGLE, 18811 },
	{ 0x014b, 0x014d, PDF_CMAP_TABLE, 30 },
	{ 0x0153, 0x0153, PDF_CMAP_SINGLE, 18839 },
	{ 0x016b, 0x016b, PDF_CMAP_SINGLE, 18819 },
	{ 0x01cd, 0x01ce, PDF_CMAP_TABLE, 33 },
	{ 0x01d0, 0x01d2, PDF_CMAP_TABLE, 35 },
	{ 0x01d4, 0x01d4, PDF_CMAP_SINGLE, 18821 },
	{ 0x01d6, 0x01d6, PDF_CMAP_SINGLE, 18823 },
	{ 0x01d8, 0x01d8, PDF_CMAP_SINGLE, 18824 },
	{ 0x01da, 0x01da, PDF_CMAP_SINGLE, 18825 },
	{ 0x01dc, 0x01dc, PDF_CMAP_SINGLE, 18826 },
	{ 0x0250, 0x0251, PDF_CMAP_TABLE, 38 },
	{ 0x0254, 0x0254, PDF_CMAP_SINGLE, 18837 },
	{ 0x025b, 0x025b, PDF_CMAP_SINGLE, 18836 },
	{ 0x0261, 0x0261, PDF_CMAP_SINGLE, 18833 },
	{ 0x026a, 0x026a, PDF_CMAP_SINGLE, 18843 },
	{ 0x0275, 0x0275, PDF_CMAP_SINGLE, 18838 },
	{ 0x0283, 0x0283, PDF_CMAP_SINGLE, 18834 },
	{ 0x028a, 0x028a, PDF_CMAP_SINGLE, 18842 },
	{ 0x02c6, 0x02c7, PDF_CMAP_TABLE, 40 },
	{ 0x02ca, 0x02cb, PDF_CMAP_TABLE, 42 },
	{ 0x02d9, 0x02d9, PDF_CMAP_SINGLE, 502 },
	{ 0x0308, 0x0308, PDF_CMAP_SINGLE, 13747 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 417 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 434 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 441 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 458 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 13936 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 13930 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 13937 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 13970 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 13969 },
	{ 0x1ebe, 0x1ec1, PDF_CMAP_TABLE, 44 },
	{ 0x2013, 0x2014, PDF_CMAP_TABLE, 48 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 166 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 168 },
	{ 0x2022, 0x2022, PDF_CMAP_SINGLE, 104 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 50 },
	{ 0x2032, 0x2032, PDF_CMAP_SINGLE, 173 },
	{ 0x2035, 0x2035, PDF_CMAP_SINGLE, 172 },
	{ 0x203b, 0x203b, PDF_CMAP_SINGLE, 177 },
	{ 0x203e, 0x203e, PDF_CMAP_SINGLE, 195 },
	{ 0x20ac, 0x20ac, PDF_CMAP_SINGLE, 17601 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 266 },
	{ 0x2105, 0x2105, PDF_CMAP_SINGLE, 194 },
	{ 0x2109, 0x2109, PDF_CMAP_SINGLE, 267 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 14054 },
	{ 0x2121, 0x2121, PDF_CMAP_SINGLE, 14055 },
	{ 0x2160, 0x2169, PDF_CMAP_RANGE, 343 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 526 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 52 },
	{ 0x2196, 0x2197, PDF_CMAP_RANGE, 249 },
	{ 0x2198, 0x2199, PDF_CMAP_TABLE, 56 },
	{ 0x21b8, 0x21b9, PDF_CMAP_RANGE, 13997 },
	{ 0x21e7, 0x21e7, PDF_CMAP_SINGLE, 13996 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 213 },
	{ 0x221e, 0x2220, PDF_CMAP_TABLE, 58 },
	{ 0x2223, 0x2223, PDF_CMAP_SINGLE, 254 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 253 },
	{ 0x2229, 0x222a, PDF_CMAP_RANGE, 229 },
	{ 0x222b, 0x222b, PDF_CMAP_SINGLE, 237 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 238 },
	{ 0x2234, 0x2235, PDF_CMAP_TABLE, 61 },
	{ 0x223c, 0x223c, PDF_CMAP_SINGLE, 228 },
	{ 0x2252, 0x2252, PDF_CMAP_SINGLE, 221 },
	{ 0x2260, 0x2261, PDF_CMAP_TABLE, 63 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 217 },
	{ 0x22a5, 0x22a5, PDF_CMAP_SINGLE, 231 },
	{ 0x22bf, 0x22bf, PDF_CMAP_SINGLE, 234 },
	{ 0x2400, 0x241f, PDF_CMAP_RANGE, 562 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 594 },
	{ 0x2460, 0x2469, PDF_CMAP_RANGE, 506 },
	{ 0x2474, 0x247d, PDF_CMAP_RANGE, 516 },
	{ 0x2500, 0x2500, PDF_CMAP_SINGLE, 311 },
	{ 0x2502, 0x2502, PDF_CMAP_SINGLE, 312 },
	{ 0x250c, 0x250c, PDF_CMAP_SINGLE, 314 },
	{ 0x2510, 0x2510, PDF_CMAP_SINGLE, 315 },
	{ 0x2514, 0x2514, PDF_CMAP_SINGLE, 316 },
	{ 0x2518, 0x2518, PDF_CMAP_SINGLE, 317 },
	{ 0x251c, 0x251c, PDF_CMAP_SINGLE, 309 },
	{ 0x2524, 0x2524, PDF_CMAP_SINGLE, 308 },
	{ 0x252c, 0x252c, PDF_CMAP_SINGLE, 307 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 306 },
	{ 0x253c, 0x253c, PDF_CMAP_SINGLE, 305 },
	{ 0x2550, 0x256c, PDF_CMAP_TABLE, 65 },
	{ 0x256d, 0x256e, PDF_CMAP_RANGE, 318 },
	{ 0x256f, 0x2570, PDF_CMAP_TABLE, 94 },
	{ 0x2571, 0x2573, PDF_CMAP_RANGE, 330 },
	{ 0x2581, 0x2588, PDF_CMAP_RANGE, 290 },
	{ 0x2589, 0x258f, PDF_CMAP_TABLE, 96 },
	{ 0x2593, 0x2595, PDF_CMAP_TABLE, 103 },
	{ 0x25a0, 0x25a1, PDF_CMAP_TABLE, 106 },
	{ 0x25b2, 0x25b3, PDF_CMAP_TABLE, 108 },
	{ 0x25bc, 0x25bd, PDF_CMAP_TABLE, 110 },
	{ 0x25c6, 0x25c7, PDF_CMAP_TABLE, 112 },
	{ 0x25cb, 0x25cb, PDF_CMAP_SINGLE, 180 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 114 },
	{ 0x25e2, 0x25e3, PDF_CMAP_RANGE, 326 },
	{ 0x25e4, 0x25e5, PDF_CMAP_TABLE, 116 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 118 },
	{ 0x2609, 0x2609, PDF_CMAP_SINGLE, 244 },
	{ 0x2640, 0x2642, PDF_CMAP_TABLE, 120 },
	{ 0x273d, 0x273d, PDF_CMAP_SINGLE, 13760 },
	{ 0x2e80, 0x2e80, PDF_CMAP_SINGLE, 17608 },
	{ 0x2e84, 0x2e84, PDF_CMAP_SINGLE, 17692 },
	{ 0x2e86, 0x2e88, PDF_CMAP_RANGE, 17693 },
	{ 0x2e8a, 0x2e8a, PDF_CMAP_SINGLE, 17696 },
	{ 0x2e8c, 0x2e8d, PDF_CMAP_RANGE, 17697 },
	{ 0x2e95, 0x2e95, PDF_CMAP_SINGLE, 17699 },
	{ 0x2e9c, 0x2e9d, PDF_CMAP_TABLE, 123 },
	{ 0x2ea5, 0x2ea5, PDF_CMAP_SINGLE, 17701 },
	{ 0x2ea7, 0x2ea7, PDF_CMAP_SINGLE, 17702 },
	{ 0x2eaa, 0x2eaa, PDF_CMAP_SINGLE, 17703 },
	{ 0x2eac, 0x2eac, PDF_CMAP_SINGLE, 17704 },
	{ 0x2eae, 0x2eae, PDF_CMAP_SINGLE, 17705 },
	{ 0x2eb6, 0x2eb6, PDF_CMAP_SINGLE, 17706 },
	{ 0x2ebc, 0x2ebc, PDF_CMAP_SINGLE, 17707 },
	{ 0x2ebe, 0x2ebe, PDF_CMAP_SINGLE, 17708 },
	{ 0x2ec6, 0x2ec6, PDF_CMAP_SINGLE, 1289 },
	{ 0x2eca, 0x2eca, PDF_CMAP_SINGLE, 17709 },
	{ 0x2ecc, 0x2ecd, PDF_CMAP_RANGE, 17710 },
	{ 0x2ecf, 0x2ecf, PDF_CMAP_SINGLE, 17712 },
	{ 0x2ed6, 0x2ed7, PDF_CMAP_RANGE, 17713 },
	{ 0x2ede, 0x2ede, PDF_CMAP_SINGLE, 17715 },
	{ 0x2ee3, 0x2ee3, PDF_CMAP_SINGLE, 2550 },
	{ 0x2f33, 0x2f33, PDF_CMAP_SINGLE, 551 },
	{ 0x3000, 0x3000, PDF_CMAP_SINGLE, 99 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 101 },
	{ 0x3003, 0x3003, PDF_CMAP_SINGLE, 179 },
	{ 0x3005, 0x3007, PDF_CMAP_RANGE, 13754 },
	{ 0x3008, 0x3009, PDF_CMAP_RANGE, 148 },
	{ 0x300a, 0x300b, PDF_CMAP_RANGE, 144 },
	{ 0x300c, 0x300d, PDF_CMAP_RANGE, 152 },
	{ 0x300e, 0x300f, PDF_CMAP_RANGE, 156 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 140 },
	{ 0x3012, 0x3012, PDF_CMAP_SINGLE, 261 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 136 },
	{ 0x301d, 0x301e, PDF_CMAP_RANGE, 170 },
	{ 0x3021, 0x3029, PDF_CMAP_RANGE, 353 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 13761 },
	{ 0x309b, 0x309c, PDF_CMAP_RANGE, 17606 },
	{ 0x309d, 0x309e, PDF_CMAP_RANGE, 13751 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 13844 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 13757 },
	{ 0x30fd, 0x30fe, PDF_CMAP_RANGE, 13749 },
	{ 0x3105, 0x3129, PDF_CMAP_RANGE, 465 },
	{ 0x3231, 0x3231, PDF_CMAP_SINGLE, 14053 },
	{ 0x32a3, 0x32a3, PDF_CMAP_SINGLE, 193 },
	{ 0x338e, 0x338f, PDF_CMAP_RANGE, 277 },
	{ 0x339c, 0x339e, PDF_CMAP_RANGE, 272 },
	{ 0x33a1, 0x33a1, PDF_CMAP_SINGLE, 276 },
	{ 0x33c4, 0x33c4, PDF_CMAP_SINGLE, 279 },
	{ 0x33ce, 0x33ce, PDF_CMAP_SINGLE, 275 },
	{ 0x33d1, 0x33d2, PDF_CMAP_TABLE, 125 },
	{ 0x33d5, 0x33d5, PDF_CMAP_SINGLE, 271 },
	{ 0x3435, 0x3435, PDF_CMAP_SINGLE, 14781 },
	{ 0x3440, 0x3440, PDF_CMAP_SINGLE, 15463 },
	{ 0x344c, 0x344c, PDF_CMAP_SINGLE, 17811 },
	{ 0x3464, 0x3464, PDF_CMAP_SINGLE, 14981 },
	{ 0x3473, 0x3473, PDF_CMAP_SINGLE, 15813 },
	{ 0x347a, 0x347a, PDF_CMAP_SINGLE, 16435 },
	{ 0x347d, 0x347e, PDF_CMAP_TABLE, 127 },
	{ 0x3493, 0x3493, PDF_CMAP_SINGLE, 17310 },
	{ 0x3496, 0x3496, PDF_CMAP_SINGLE, 14300 },
	{ 0x34a5, 0x34a5, PDF_CMAP_SINGLE, 17816 },
	{ 0x34af, 0x34af, PDF_CMAP_SINGLE, 15487 },
	{ 0x34bc, 0x34bc, PDF_CMAP_SINGLE, 17280 },
	{ 0x34c1, 0x34c1, PDF_CMAP_SINGLE, 17659 },
	{ 0x34c8, 0x34c8, PDF_CMAP_SINGLE, 15616 },
	{ 0x34df, 0x34df, PDF_CMAP_SINGLE, 16036 },
	{ 0x34e4, 0x34e4, PDF_CMAP_SINGLE, 15956 },
	{ 0x34fb, 0x34fb, PDF_CMAP_SINGLE, 15818 },
	{ 0x3506, 0x3506, PDF_CMAP_SINGLE, 17206 },
	{ 0x353e, 0x353e, PDF_CMAP_SINGLE, 17639 },
	{ 0x3551, 0x3551, PDF_CMAP_SINGLE, 17825 },
	{ 0x3553, 0x3553, PDF_CMAP_SINGLE, 17317 },
	{ 0x3561, 0x3561, PDF_CMAP_SINGLE, 16600 },
	{ 0x356d, 0x356d, PDF_CMAP_SINGLE, 17828 },
	{ 0x3570, 0x3570, PDF_CMAP_SINGLE, 15151 },
	{ 0x3572, 0x3572, PDF_CMAP_SINGLE, 17829 },
	{ 0x3577, 0x3578, PDF_CMAP_TABLE, 129 },
	{ 0x3584, 0x3584, PDF_CMAP_SINGLE, 14843 },
	{ 0x3597, 0x3598, PDF_CMAP_TABLE, 131 },
	{ 0x35a1, 0x35a1, PDF_CMAP_SINGLE, 16610 },
	{ 0x35a5, 0x35a5, PDF_CMAP_SINGLE, 17841 },
	{ 0x35ad, 0x35ad, PDF_CMAP_SINGLE, 16124 },
	{ 0x35bf, 0x35bf, PDF_CMAP_SINGLE, 17842 },
	{ 0x35c1, 0x35c1, PDF_CMAP_SINGLE, 17792 },
	{ 0x35c5, 0x35c5, PDF_CMAP_SINGLE, 17844 },
	{ 0x35c7, 0x35c7, PDF_CMAP_SINGLE, 17823 },
	{ 0x35ca, 0x35ca, PDF_CMAP_SINGLE, 15939 },
	{ 0x35ce, 0x35ce, PDF_CMAP_SINGLE, 16001 },
	{ 0x35d2, 0x35d2, PDF_CMAP_SINGLE, 16329 },
	{ 0x35d6, 0x35d6, PDF_CMAP_SINGLE, 16309 },
	{ 0x35db, 0x35db, PDF_CMAP_SINGLE, 18189 },
	{ 0x35dd, 0x35dd, PDF_CMAP_SINGLE, 17324 },
	{ 0x35f1, 0x35f3, PDF_CMAP_TABLE, 133 },
	{ 0x35fb, 0x35fb, PDF_CMAP_SINGLE, 17864 },
	{ 0x35fe, 0x35fe, PDF_CMAP_SINGLE, 16234 },
	{ 0x3609, 0x3609, PDF_CMAP_SINGLE, 17909 },
	{ 0x3618, 0x3618, PDF_CMAP_SINGLE, 18545 },
	{ 0x361a, 0x361a, PDF_CMAP_SINGLE, 17946 },
	{ 0x3623, 0x3623, PDF_CMAP_SINGLE, 16582 },
	{ 0x362d, 0x362d, PDF_CMAP_SINGLE, 16006 },
	{ 0x3635, 0x3635, PDF_CMAP_SINGLE, 18734 },
	{ 0x3639, 0x3639, PDF_CMAP_SINGLE, 16741 },
	{ 0x363e, 0x363e, PDF_CMAP_SINGLE, 14856 },
	{ 0x3647, 0x3649, PDF_CMAP_TABLE, 136 },
	{ 0x364e, 0x364e, PDF_CMAP_SINGLE, 18072 },
	{ 0x365f, 0x365f, PDF_CMAP_SINGLE, 15859 },
	{ 0x367a, 0x367a, PDF_CMAP_SINGLE, 16099 },
	{ 0x3681, 0x3681, PDF_CMAP_SINGLE, 17830 },
	{ 0x369a, 0x369a, PDF_CMAP_SINGLE, 15473 },
	{ 0x36a5, 0x36a5, PDF_CMAP_SINGLE, 18690 },
	{ 0x36aa, 0x36aa, PDF_CMAP_SINGLE, 15152 },
	{ 0x36ac, 0x36ac, PDF_CMAP_SINGLE, 18688 },
	{ 0x36b0, 0x36b1, PDF_CMAP_TABLE, 139 },
	{ 0x36b5, 0x36b5, PDF_CMAP_SINGLE, 15298 },
	{ 0x36b9, 0x36b9, PDF_CMAP_SINGLE, 18567 },
	{ 0x36bc, 0x36bc, PDF_CMAP_SINGLE, 15615 },
	{ 0x36c1, 0x36c1, PDF_CMAP_SINGLE, 14277 },
	{ 0x36c3, 0x36c5, PDF_CMAP_TABLE, 141 },
	{ 0x36c7, 0x36c8, PDF_CMAP_TABLE, 144 },
	{ 0x36d3, 0x36d4, PDF_CMAP_TABLE, 146 },
	{ 0x36d6, 0x36d6, PDF_CMAP_SINGLE, 15628 },
	{ 0x36dd, 0x36dd, PDF_CMAP_SINGLE, 14902 },
	{ 0x36e1, 0x36e2, PDF_CMAP_TABLE, 148 },
	{ 0x36e5, 0x36e6, PDF_CMAP_TABLE, 150 },
	{ 0x36f5, 0x36f5, PDF_CMAP_SINGLE, 14872 },
	{ 0x3701, 0x3701, PDF_CMAP_SINGLE, 14900 },
	{ 0x3703, 0x3703, PDF_CMAP_SINGLE, 17935 },
	{ 0x3708, 0x3708, PDF_CMAP_SINGLE, 16639 },
	{ 0x370a, 0x370a, PDF_CMAP_SINGLE, 15573 },
	{ 0x370d, 0x370d, PDF_CMAP_SINGLE, 16952 },
	{ 0x371c, 0x371c, PDF_CMAP_SINGLE, 15870 },
	{ 0x3722, 0x3723, PDF_CMAP_TABLE, 152 },
	{ 0x3725, 0x3725, PDF_CMAP_SINGLE, 14409 },
	{ 0x372c, 0x372d, PDF_CMAP_TABLE, 154 },
	{ 0x3730, 0x3730, PDF_CMAP_SINGLE, 18780 },
	{ 0x3732, 0x3733, PDF_CMAP_TABLE, 156 },
	{ 0x373a, 0x373a, PDF_CMAP_SINGLE, 15958 },
	{ 0x3740, 0x3740, PDF_CMAP_SINGLE, 16898 },
	{ 0x3743, 0x3743, PDF_CMAP_SINGLE, 16438 },
	{ 0x3762, 0x3762, PDF_CMAP_SINGLE, 15798 },
	{ 0x376f, 0x376f, PDF_CMAP_SINGLE, 18379 },
	{ 0x3797, 0x3797, PDF_CMAP_SINGLE, 17901 },
	{ 0x37a0, 0x37a0, PDF_CMAP_SINGLE, 14888 },
	{ 0x37b9, 0x37b9, PDF_CMAP_SINGLE, 17335 },
	{ 0x37be, 0x37be, PDF_CMAP_SINGLE, 14654 },
	{ 0x37f2, 0x37f2, PDF_CMAP_SINGLE, 15265 },
	{ 0x37f8, 0x37f8, PDF_CMAP_SINGLE, 17106 },
	{ 0x37fb, 0x37fb, PDF_CMAP_SINGLE, 16117 },
	{ 0x380f, 0x380f, PDF_CMAP_SINGLE, 17964 },
	{ 0x3819, 0x3819, PDF_CMAP_SINGLE, 14767 },
	{ 0x3820, 0x3820, PDF_CMAP_SINGLE, 17967 },
	{ 0x382d, 0x382d, PDF_CMAP_SINGLE, 16686 },
	{ 0x3836, 0x3836, PDF_CMAP_SINGLE, 16691 },
	{ 0x3838, 0x3838, PDF_CMAP_SINGLE, 17339 },
	{ 0x3863, 0x3863, PDF_CMAP_SINGLE, 18115 },
	{ 0x38a0, 0x38a0, PDF_CMAP_SINGLE, 16709 },
	{ 0x38c3, 0x38c3, PDF_CMAP_SINGLE, 14610 },
	{ 0x38cc, 0x38cc, PDF_CMAP_SINGLE, 16502 },
	{ 0x38d1, 0x38d1, PDF_CMAP_SINGLE, 14997 },
	{ 0x38fa, 0x38fa, PDF_CMAP_SINGLE, 17643 },
	{ 0x3908, 0x3908, PDF_CMAP_SINGLE, 17970 },
	{ 0x3914, 0x3914, PDF_CMAP_SINGLE, 17342 },
	{ 0x3927, 0x3927, PDF_CMAP_SINGLE, 15409 },
	{ 0x3932, 0x3932, PDF_CMAP_SINGLE, 16770 },
	{ 0x393f, 0x393f, PDF_CMAP_SINGLE, 17971 },
	{ 0x394d, 0x394d, PDF_CMAP_SINGLE, 17972 },
	{ 0x3963, 0x3963, PDF_CMAP_SINGLE, 16739 },
	{ 0x3980, 0x3980, PDF_CMAP_SINGLE, 14452 },
	{ 0x3989, 0x398a, PDF_CMAP_TABLE, 158 },
	{ 0x3992, 0x3992, PDF_CMAP_SINGLE, 17270 },
	{ 0x3999, 0x3999, PDF_CMAP_SINGLE, 14778 },
	{ 0x399b, 0x399b, PDF_CMAP_SINGLE, 15795 },
	{ 0x39a1, 0x39a1, PDF_CMAP_SINGLE, 15897 },
	{ 0x39a4, 0x39a4, PDF_CMAP_SINGLE, 15887 },
	{ 0x39b8, 0x39b8, PDF_CMAP_SINGLE, 17979 },
	{ 0x39dc, 0x39dc, PDF_CMAP_SINGLE, 16078 },
	{ 0x39e2, 0x39e2, PDF_CMAP_SINGLE, 18120 },
	{ 0x39e5, 0x39e5, PDF_CMAP_SINGLE, 14651 },
	{ 0x39ec, 0x39ec, PDF_CMAP_SINGLE, 17168 },
	{ 0x39f8, 0x39f8, PDF_CMAP_SINGLE, 17982 },
	{ 0x39fb, 0x39fb, PDF_CMAP_SINGLE, 17221 },
	{ 0x39fe, 0x39fe, PDF_CMAP_SINGLE, 17256 },
	{ 0x3a01, 0x3a01, PDF_CMAP_SINGLE, 16864 },
	{ 0x3a03, 0x3a03, PDF_CMAP_SINGLE, 17984 },
	{ 0x3a06, 0x3a06, PDF_CMAP_SINGLE, 17271 },
	{ 0x3a17, 0x3a18, PDF_CMAP_TABLE, 160 },
	{ 0x3a29, 0x3a2a, PDF_CMAP_TABLE, 162 },
	{ 0x3a34, 0x3a34, PDF_CMAP_SINGLE, 17177 },
	{ 0x3a4b, 0x3a4b, PDF_CMAP_SINGLE, 17988 },
	{ 0x3a52, 0x3a52, PDF_CMAP_SINGLE, 16083 },
	{ 0x3a57, 0x3a57, PDF_CMAP_SINGLE, 16798 },
	{ 0x3a5c, 0x3a5c, PDF_CMAP_SINGLE, 16324 },
	{ 0x3a5e, 0x3a5e, PDF_CMAP_SINGLE, 15111 },
	{ 0x3a66, 0x3a67, PDF_CMAP_TABLE, 164 },
	{ 0x3a97, 0x3a97, PDF_CMAP_SINGLE, 17991 },
	{ 0x3aab, 0x3aab, PDF_CMAP_SINGLE, 16529 },
	{ 0x3abd, 0x3abd, PDF_CMAP_SINGLE, 17993 },
	{ 0x3ade, 0x3ade, PDF_CMAP_SINGLE, 16716 },
	{ 0x3ae0, 0x3ae0, PDF_CMAP_SINGLE, 14970 },
	{ 0x3af0, 0x3af0, PDF_CMAP_SINGLE, 18098 },
	{ 0x3af2, 0x3af2, PDF_CMAP_SINGLE, 17996 },
	{ 0x3afb, 0x3afb, PDF_CMAP_SINGLE, 15090 },
	{ 0x3b0e, 0x3b0e, PDF_CMAP_SINGLE, 14568 },
	{ 0x3b19, 0x3b19, PDF_CMAP_SINGLE, 18117 },
	{ 0x3b22, 0x3b22, PDF_CMAP_SINGLE, 17998 },
	{ 0x3b2b, 0x3b2b, PDF_CMAP_SINGLE, 18774 },
	{ 0x3b39, 0x3b39, PDF_CMAP_SINGLE, 18251 },
	{ 0x3b42, 0x3b42, PDF_CMAP_SINGLE, 18000 },
	{ 0x3b58, 0x3b58, PDF_CMAP_SINGLE, 18002 },
	{ 0x3b60, 0x3b60, PDF_CMAP_SINGLE, 14650 },
	{ 0x3b71, 0x3b72, PDF_CMAP_TABLE, 166 },
	{ 0x3b7b, 0x3b7c, PDF_CMAP_TABLE, 168 },
	{ 0x3b80, 0x3b80, PDF_CMAP_SINGLE, 16866 },
	{ 0x3b96, 0x3b96, PDF_CMAP_SINGLE, 15004 },
	{ 0x3b99, 0x3b99, PDF_CMAP_SINGLE, 15000 },
	{ 0x3ba1, 0x3ba1, PDF_CMAP_SINGLE, 16873 },
	{ 0x3bbc, 0x3bbc, PDF_CMAP_SINGLE, 17352 },
	{ 0x3bbe, 0x3bbe, PDF_CMAP_SINGLE, 15793 },
	{ 0x3bc2, 0x3bc2, PDF_CMAP_SINGLE, 16692 },
	{ 0x3bc4, 0x3bc4, PDF_CMAP_SINGLE, 15008 },
	{ 0x3bd7, 0x3bd7, PDF_CMAP_SINGLE, 15020 },
	{ 0x3bdd, 0x3bdd, PDF_CMAP_SINGLE, 18015 },
	{ 0x3bec, 0x3bec, PDF_CMAP_SINGLE, 18020 },
	{ 0x3bf2, 0x3bf4, PDF_CMAP_TABLE, 170 },
	{ 0x3c0d, 0x3c0d, PDF_CMAP_SINGLE, 16887 },
	{ 0x3c11, 0x3c11, PDF_CMAP_SINGLE, 15936 },
	{ 0x3c15, 0x3c15, PDF_CMAP_SINGLE, 14744 },
	{ 0x3c54, 0x3c54, PDF_CMAP_SINGLE, 15872 },
	{ 0x3ccb, 0x3ccb, PDF_CMAP_SINGLE, 18032 },
	{ 0x3ccd, 0x3ccd, PDF_CMAP_SINGLE, 15589 },
	{ 0x3cd1, 0x3cd1, PDF_CMAP_SINGLE, 16387 },
	{ 0x3cd6, 0x3cd6, PDF_CMAP_SINGLE, 15607 },
	{ 0x3cdc, 0x3cdc, PDF_CMAP_SINGLE, 16462 },
	{ 0x3ceb, 0x3ceb, PDF_CMAP_SINGLE, 16919 },
	{ 0x3cef, 0x3cef, PDF_CMAP_SINGLE, 18037 },
	{ 0x3d13, 0x3d13, PDF_CMAP_SINGLE, 14195 },
	{ 0x3d1d, 0x3d1d, PDF_CMAP_SINGLE, 14652 },
	{ 0x3d32, 0x3d32, PDF_CMAP_SINGLE, 18775 },
	{ 0x3d3b, 0x3d3b, PDF_CMAP_SINGLE, 16965 },
	{ 0x3d46, 0x3d46, PDF_CMAP_SINGLE, 18053 },
	{ 0x3d4c, 0x3d4c, PDF_CMAP_SINGLE, 15595 },
	{ 0x3d4e, 0x3d4e, PDF_CMAP_SINGLE, 16962 },
	{ 0x3d51, 0x3d51, PDF_CMAP_SINGLE, 14570 },
	{ 0x3d5f, 0x3d5f, PDF_CMAP_SINGLE, 16729 },
	{ 0x3d62, 0x3d62, PDF_CMAP_SINGLE, 15454 },
	{ 0x3d69, 0x3d6a, PDF_CMAP_TABLE, 173 },
	{ 0x3d6f, 0x3d6f, PDF_CMAP_SINGLE, 15612 },
	{ 0x3d75, 0x3d75, PDF_CMAP_SINGLE, 18058 },
	{ 0x3d7d, 0x3d7d, PDF_CMAP_SINGLE, 15407 },
	{ 0x3d85, 0x3d85, PDF_CMAP_SINGLE, 18763 },
	{ 0x3d8a, 0x3d8a, PDF_CMAP_SINGLE, 18061 },
	{ 0x3d8f, 0x3d8f, PDF_CMAP_SINGLE, 15037 },
	{ 0x3d91, 0x3d91, PDF_CMAP_SINGLE, 18063 },
	{ 0x3da5, 0x3da5, PDF_CMAP_SINGLE, 15702 },
	{ 0x3dad, 0x3dad, PDF_CMAP_SINGLE, 18073 },
	{ 0x3db4, 0x3db4, PDF_CMAP_SINGLE, 16550 },
	{ 0x3dbf, 0x3dbf, PDF_CMAP_SINGLE, 14288 },
	{ 0x3dc6, 0x3dc7, PDF_CMAP_TABLE, 175 },
	{ 0x3dcc, 0x3dcd, PDF_CMAP_TABLE, 177 },
	{ 0x3dd3, 0x3dd3, PDF_CMAP_SINGLE, 14305 },
	{ 0x3ddb, 0x3ddb, PDF_CMAP_SINGLE, 16383 },
	{ 0x3de7, 0x3de8, PDF_CMAP_TABLE, 179 },
	{ 0x3deb, 0x3deb, PDF_CMAP_SINGLE, 15962 },
	{ 0x3df3, 0x3df3, PDF_CMAP_SINGLE, 18132 },
	{ 0x3df7, 0x3df7, PDF_CMAP_SINGLE, 18603 },
	{ 0x3dfc, 0x3dfd, PDF_CMAP_TABLE, 181 },
	{ 0x3e06, 0x3e06, PDF_CMAP_SINGLE, 18717 },
	{ 0x3e40, 0x3e40, PDF_CMAP_SINGLE, 16745 },
	{ 0x3e43, 0x3e43, PDF_CMAP_SINGLE, 17261 },
	{ 0x3e48, 0x3e48, PDF_CMAP_SINGLE, 17813 },
	{ 0x3e55, 0x3e55, PDF_CMAP_SINGLE, 17023 },
	{ 0x3e74, 0x3e74, PDF_CMAP_SINGLE, 16098 },
	{ 0x3ea8, 0x3eaa, PDF_CMAP_TABLE, 183 },
	{ 0x3ead, 0x3ead, PDF_CMAP_SINGLE, 15261 },
	{ 0x3eb1, 0x3eb1, PDF_CMAP_SINGLE, 15064 },
	{ 0x3eb8, 0x3eb8, PDF_CMAP_SINGLE, 14923 },
	{ 0x3ebf, 0x3ebf, PDF_CMAP_SINGLE, 15115 },
	{ 0x3ec2, 0x3ec2, PDF_CMAP_SINGLE, 15320 },
	{ 0x3ec7, 0x3ec7, PDF_CMAP_SINGLE, 14709 },
	{ 0x3eca, 0x3eca, PDF_CMAP_SINGLE, 18161 },
	{ 0x3ecc, 0x3ecc, PDF_CMAP_SINGLE, 15330 },
	{ 0x3ed0, 0x3ed1, PDF_CMAP_TABLE, 186 },
	{ 0x3ed6, 0x3ed7, PDF_CMAP_TABLE, 188 },
	{ 0x3eda, 0x3eda, PDF_CMAP_SINGLE, 15618 },
	{ 0x3ede, 0x3ede, PDF_CMAP_SINGLE, 14834 },
	{ 0x3ee1, 0x3ee2, PDF_CMAP_TABLE, 190 },
	{ 0x3ee7, 0x3ee7, PDF_CMAP_SINGLE, 15324 },
	{ 0x3ee9, 0x3ee9, PDF_CMAP_SINGLE, 15524 },
	{ 0x3eeb, 0x3eeb, PDF_CMAP_SINGLE, 14698 },
	{ 0x3ef0, 0x3ef0, PDF_CMAP_SINGLE, 18167 },
	{ 0x3ef3, 0x3ef4, PDF_CMAP_TABLE, 192 },
	{ 0x3efa, 0x3efa, PDF_CMAP_SINGLE, 18169 },
	{ 0x3efc, 0x3efc, PDF_CMAP_SINGLE, 15336 },
	{ 0x3eff, 0x3f00, PDF_CMAP_TABLE, 194 },
	{ 0x3f04, 0x3f04, PDF_CMAP_SINGLE, 17091 },
	{ 0x3f06, 0x3f06, PDF_CMAP_SINGLE, 15063 },
	{ 0x3f0e, 0x3f0e, PDF_CMAP_SINGLE, 18171 },
	{ 0x3f53, 0x3f53, PDF_CMAP_SINGLE, 18172 },
	{ 0x3f58, 0x3f59, PDF_CMAP_TABLE, 196 },
	{ 0x3f63, 0x3f63, PDF_CMAP_SINGLE, 15078 },
	{ 0x3f7c, 0x3f7c, PDF_CMAP_SINGLE, 18176 },
	{ 0x3f93, 0x3f93, PDF_CMAP_SINGLE, 17869 },
	{ 0x3fc0, 0x3fc0, PDF_CMAP_SINGLE, 17359 },
	{ 0x3fd7, 0x3fd7, PDF_CMAP_SINGLE, 17361 },
	{ 0x3fdc, 0x3fdc, PDF_CMAP_SINGLE, 18180 },
	{ 0x3fe5, 0x3fe5, PDF_CMAP_SINGLE, 18143 },
	{ 0x3fed, 0x3fed, PDF_CMAP_SINGLE, 17205 },
	{ 0x3ff9, 0x3ffa, PDF_CMAP_TABLE, 198 },
	{ 0x4004, 0x4004, PDF_CMAP_SINGLE, 16654 },
	{ 0x401d, 0x401d, PDF_CMAP_SINGLE, 18185 },
	{ 0x4039, 0x4039, PDF_CMAP_SINGLE, 18187 },
	{ 0x4045, 0x4045, PDF_CMAP_SINGLE, 18188 },
	{ 0x4053, 0x4053, PDF_CMAP_SINGLE, 17846 },
	{ 0x4057, 0x4057, PDF_CMAP_SINGLE, 14749 },
	{ 0x4062, 0x4062, PDF_CMAP_SINGLE, 15307 },
	{ 0x4065, 0x4065, PDF_CMAP_SINGLE, 16339 },
	{ 0x406a, 0x406a, PDF_CMAP_SINGLE, 18191 },
	{ 0x406f, 0x406f, PDF_CMAP_SINGLE, 18192 },
	{ 0x40a8, 0x40a8, PDF_CMAP_SINGLE, 17365 },
	{ 0x40bb, 0x40bb, PDF_CMAP_SINGLE, 17856 },
	{ 0x40bf, 0x40bf, PDF_CMAP_SINGLE, 16108 },
	{ 0x40c8, 0x40c8, PDF_CMAP_SINGLE, 15118 },
	{ 0x40d8, 0x40d8, PDF_CMAP_SINGLE, 16811 },
	{ 0x40df, 0x40df, PDF_CMAP_SINGLE, 15895 },
	{ 0x40fa, 0x40fa, PDF_CMAP_SINGLE, 16062 },
	{ 0x4103, 0x4104, PDF_CMAP_TABLE, 200 },
	{ 0x4109, 0x4109, PDF_CMAP_SINGLE, 18204 },
	{ 0x410e, 0x410e, PDF_CMAP_SINGLE, 15131 },
	{ 0x4132, 0x4132, PDF_CMAP_SINGLE, 15141 },
	{ 0x4167, 0x4167, PDF_CMAP_SINGLE, 18207 },
	{ 0x416c, 0x416c, PDF_CMAP_SINGLE, 14510 },
	{ 0x416e, 0x416e, PDF_CMAP_SINGLE, 15139 },
	{ 0x417f, 0x417f, PDF_CMAP_SINGLE, 15234 },
	{ 0x4190, 0x4190, PDF_CMAP_SINGLE, 18112 },
	{ 0x41b2, 0x41b2, PDF_CMAP_SINGLE, 18208 },
	{ 0x41c4, 0x41c4, PDF_CMAP_SINGLE, 18211 },
	{ 0x41ca, 0x41ca, PDF_CMAP_SINGLE, 14143 },
	{ 0x41cf, 0x41cf, PDF_CMAP_SINGLE, 18214 },
	{ 0x41db, 0x41db, PDF_CMAP_SINGLE, 14271 },
	{ 0x41ef, 0x41ef, PDF_CMAP_SINGLE, 14147 },
	{ 0x41f9, 0x41f9, PDF_CMAP_SINGLE, 15166 },
	{ 0x4211, 0x4211, PDF_CMAP_SINGLE, 15169 },
	{ 0x4240, 0x4240, PDF_CMAP_SINGLE, 14321 },
	{ 0x4260, 0x4260, PDF_CMAP_SINGLE, 18219 },
	{ 0x426a, 0x426a, PDF_CMAP_SINGLE, 15189 },
	{ 0x427a, 0x427a, PDF_CMAP_SINGLE, 18220 },
	{ 0x428c, 0x428c, PDF_CMAP_SINGLE, 18223 },
	{ 0x4294, 0x4294, PDF_CMAP_SINGLE, 18225 },
	{ 0x42b5, 0x42b5, PDF_CMAP_SINGLE, 16400 },
	{ 0x42b9, 0x42b9, PDF_CMAP_SINGLE, 14502 },
	{ 0x42bc, 0x42bc, PDF_CMAP_SINGLE, 15498 },
	{ 0x42f4, 0x42f4, PDF_CMAP_SINGLE, 15289 },
	{ 0x42fb, 0x42fc, PDF_CMAP_TABLE, 202 },
	{ 0x432b, 0x432b, PDF_CMAP_SINGLE, 14205 },
	{ 0x436e, 0x436e, PDF_CMAP_SINGLE, 18122 },
	{ 0x4397, 0x4397, PDF_CMAP_SINGLE, 18235 },
	{ 0x43ba, 0x43ba, PDF_CMAP_SINGLE, 17247 },
	{ 0x43c1, 0x43c1, PDF_CMAP_SINGLE, 18069 },
	{ 0x43d9, 0x43d9, PDF_CMAP_SINGLE, 17214 },
	{ 0x43df, 0x43df, PDF_CMAP_SINGLE, 15945 },
	{ 0x43ed, 0x43ed, PDF_CMAP_SINGLE, 18245 },
	{ 0x43f2, 0x43f2, PDF_CMAP_SINGLE, 15944 },
	{ 0x4401, 0x4402, PDF_CMAP_TABLE, 204 },
	{ 0x4413, 0x4413, PDF_CMAP_SINGLE, 18255 },
	{ 0x4425, 0x4425, PDF_CMAP_SINGLE, 18257 },
	{ 0x442d, 0x442d, PDF_CMAP_SINGLE, 18258 },
	{ 0x447a, 0x447a, PDF_CMAP_SINGLE, 14255 },
	{ 0x448f, 0x448f, PDF_CMAP_SINGLE, 18264 },
	{ 0x449f, 0x44a0, PDF_CMAP_TABLE, 206 },
	{ 0x44a2, 0x44a2, PDF_CMAP_SINGLE, 16505 },
	{ 0x44b0, 0x44b0, PDF_CMAP_SINGLE, 18268 },
	{ 0x44b7, 0x44b7, PDF_CMAP_SINGLE, 16289 },
	{ 0x44c0, 0x44c0, PDF_CMAP_SINGLE, 15367 },
	{ 0x44c5, 0x44c5, PDF_CMAP_SINGLE, 16912 },
	{ 0x44ce, 0x44ce, PDF_CMAP_SINGLE, 15651 },
	{ 0x44dd, 0x44dd, PDF_CMAP_SINGLE, 14813 },
	{ 0x44df, 0x44df, PDF_CMAP_SINGLE, 15650 },
	{ 0x44e4, 0x44e4, PDF_CMAP_SINGLE, 14306 },
	{ 0x44e9, 0x44ec, PDF_CMAP_TABLE, 208 },
	{ 0x44f4, 0x44f4, PDF_CMAP_SINGLE, 14738 },
	{ 0x4503, 0x4504, PDF_CMAP_TABLE, 212 },
	{ 0x4509, 0x4509, PDF_CMAP_SINGLE, 15952 },
	{ 0x450b, 0x450b, PDF_CMAP_SINGLE, 14292 },
	{ 0x4516, 0x4516, PDF_CMAP_SINGLE, 14329 },
	{ 0x451d, 0x451d, PDF_CMAP_SINGLE, 14183 },
	{ 0x4527, 0x4527, PDF_CMAP_SINGLE, 14327 },
	{ 0x452e, 0x452e, PDF_CMAP_SINGLE, 15571 },
	{ 0x4533, 0x4533, PDF_CMAP_SINGLE, 15441 },
	{ 0x453b, 0x453b, PDF_CMAP_SINGLE, 18282 },
	{ 0x453d, 0x453d, PDF_CMAP_SINGLE, 14532 },
	{ 0x453f, 0x453f, PDF_CMAP_SINGLE, 15890 },
	{ 0x4543, 0x4543, PDF_CMAP_SINGLE, 14323 },
	{ 0x4551, 0x4552, PDF_CMAP_TABLE, 214 },
	{ 0x4555, 0x4555, PDF_CMAP_SINGLE, 16958 },
	{ 0x455c, 0x455c, PDF_CMAP_SINGLE, 14219 },
	{ 0x4562, 0x4562, PDF_CMAP_SINGLE, 18752 },
	{ 0x456a, 0x456a, PDF_CMAP_SINGLE, 14340 },
	{ 0x4577, 0x4577, PDF_CMAP_SINGLE, 18286 },
	{ 0x4585, 0x4585, PDF_CMAP_SINGLE, 14533 },
	{ 0x45e9, 0x45e9, PDF_CMAP_SINGLE, 16100 },
	{ 0x4606, 0x4606, PDF_CMAP_SINGLE, 18291 },
	{ 0x460f, 0x460f, PDF_CMAP_SINGLE, 14357 },
	{ 0x4615, 0x4615, PDF_CMAP_SINGLE, 14403 },
	{ 0x4617, 0x4617, PDF_CMAP_SINGLE, 18292 },
	{ 0x465b, 0x465b, PDF_CMAP_SINGLE, 14365 },
	{ 0x467a, 0x467a, PDF_CMAP_SINGLE, 14825 },
	{ 0x4680, 0x4680, PDF_CMAP_SINGLE, 15617 },
	{ 0x46cf, 0x46d0, PDF_CMAP_TABLE, 216 },
	{ 0x46f5, 0x46f5, PDF_CMAP_SINGLE, 15263 },
	{ 0x4713, 0x4713, PDF_CMAP_SINGLE, 14387 },
	{ 0x4718, 0x4718, PDF_CMAP_SINGLE, 15815 },
	{ 0x474e, 0x474e, PDF_CMAP_SINGLE, 16060 },
	{ 0x477c, 0x477c, PDF_CMAP_SINGLE, 15821 },
	{ 0x4798, 0x4798, PDF_CMAP_SINGLE, 18305 },
	{ 0x47a6, 0x47a6, PDF_CMAP_SINGLE, 16547 },
	{ 0x47b6, 0x47b6, PDF_CMAP_SINGLE, 16106 },
	{ 0x47d5, 0x47d5, PDF_CMAP_SINGLE, 17178 },
	{ 0x47ed, 0x47ed, PDF_CMAP_SINGLE, 18307 },
	{ 0x47f4, 0x47f4, PDF_CMAP_SINGLE, 17199 },
	{ 0x4800, 0x4800, PDF_CMAP_SINGLE, 17950 },
	{ 0x480b, 0x480b, PDF_CMAP_SINGLE, 17234 },
	{ 0x4837, 0x4837, PDF_CMAP_SINGLE, 18311 },
	{ 0x485d, 0x485d, PDF_CMAP_SINGLE, 16655 },
	{ 0x4871, 0x4871, PDF_CMAP_SINGLE, 15619 },
	{ 0x489b, 0x489b, PDF_CMAP_SINGLE, 15293 },
	{ 0x48ad, 0x48ae, PDF_CMAP_TABLE, 218 },
	{ 0x48d0, 0x48d0, PDF_CMAP_SINGLE, 15783 },
	{ 0x48dd, 0x48dd, PDF_CMAP_SINGLE, 16672 },
	{ 0x48ed, 0x48ed, PDF_CMAP_SINGLE, 17032 },
	{ 0x48f3, 0x48f3, PDF_CMAP_SINGLE, 16065 },
	{ 0x48fa, 0x48fa, PDF_CMAP_SINGLE, 15940 },
	{ 0x4906, 0x4906, PDF_CMAP_SINGLE, 15303 },
	{ 0x4911, 0x4911, PDF_CMAP_SINGLE, 17796 },
	{ 0x491e, 0x491e, PDF_CMAP_SINGLE, 18324 },
	{ 0x4925, 0x4925, PDF_CMAP_SINGLE, 15375 },
	{ 0x492a, 0x492a, PDF_CMAP_SINGLE, 18094 },
	{ 0x492d, 0x492d, PDF_CMAP_SINGLE, 18125 },
	{ 0x4935, 0x4935, PDF_CMAP_SINGLE, 15555 },
	{ 0x493c, 0x493c, PDF_CMAP_SINGLE, 15352 },
	{ 0x493e, 0x493e, PDF_CMAP_SINGLE, 15622 },
	{ 0x4945, 0x4945, PDF_CMAP_SINGLE, 18339 },
	{ 0x4951, 0x4951, PDF_CMAP_SINGLE, 18340 },
	{ 0x4953, 0x4953, PDF_CMAP_SINGLE, 17069 },
	{ 0x4965, 0x4965, PDF_CMAP_SINGLE, 14489 },
	{ 0x496a, 0x496a, PDF_CMAP_SINGLE, 18345 },
	{ 0x4972, 0x4972, PDF_CMAP_SINGLE, 14884 },
	{ 0x4989, 0x4989, PDF_CMAP_SINGLE, 14235 },
	{ 0x49a1, 0x49a1, PDF_CMAP_SINGLE, 14519 },
	{ 0x49a7, 0x49a7, PDF_CMAP_SINGLE, 18350 },
	{ 0x49df, 0x49df, PDF_CMAP_SINGLE, 14506 },
	{ 0x49e5, 0x49e5, PDF_CMAP_SINGLE, 18353 },
	{ 0x49e7, 0x49e7, PDF_CMAP_SINGLE, 17953 },
	{ 0x4a0f, 0x4a0f, PDF_CMAP_SINGLE, 14531 },
	{ 0x4a1d, 0x4a1d, PDF_CMAP_SINGLE, 15340 },
	{ 0x4a24, 0x4a24, PDF_CMAP_SINGLE, 18354 },
	{ 0x4a35, 0x4a35, PDF_CMAP_SINGLE, 18356 },
	{ 0x4a96, 0x4a96, PDF_CMAP_SINGLE, 15591 },
	{ 0x4ab4, 0x4ab4, PDF_CMAP_SINGLE, 17249 },
	{ 0x4ab8, 0x4ab8, PDF_CMAP_SINGLE, 15784 },
	{ 0x4ad1, 0x4ad1, PDF_CMAP_SINGLE, 14563 },
	{ 0x4ae4, 0x4ae4, PDF_CMAP_SINGLE, 18359 },
	{ 0x4aff, 0x4aff, PDF_CMAP_SINGLE, 14578 },
	{ 0x4b19, 0x4b19, PDF_CMAP_SINGLE, 18361 },
	{ 0x4b2c, 0x4b2c, PDF_CMAP_SINGLE, 17951 },
	{ 0x4b37, 0x4b37, PDF_CMAP_SINGLE, 16809 },
	{ 0x4b6f, 0x4b70, PDF_CMAP_TABLE, 220 },
	{ 0x4b72, 0x4b72, PDF_CMAP_SINGLE, 14588 },
	{ 0x4b7b, 0x4b7b, PDF_CMAP_SINGLE, 15501 },
	{ 0x4b7e, 0x4b7e, PDF_CMAP_SINGLE, 16394 },
	{ 0x4b8e, 0x4b8e, PDF_CMAP_SINGLE, 14839 },
	{ 0x4b90, 0x4b90, PDF_CMAP_SINGLE, 15392 },
	{ 0x4b93, 0x4b93, PDF_CMAP_SINGLE, 14988 },
	{ 0x4b96, 0x4b97, PDF_CMAP_TABLE, 222 },
	{ 0x4b9d, 0x4b9d, PDF_CMAP_SINGLE, 18370 },
	{ 0x4bbd, 0x4bbe, PDF_CMAP_TABLE, 224 },
	{ 0x4bc0, 0x4bc0, PDF_CMAP_SINGLE, 15836 },
	{ 0x4c04, 0x4c04, PDF_CMAP_SINGLE, 16315 },
	{ 0x4c07, 0x4c07, PDF_CMAP_SINGLE, 16311 },
	{ 0x4c0e, 0x4c0e, PDF_CMAP_SINGLE, 14604 },
	{ 0x4c3b, 0x4c3b, PDF_CMAP_SINGLE, 16188 },
	{ 0x4c3e, 0x4c3e, PDF_CMAP_SINGLE, 17787 },
	{ 0x4c5b, 0x4c5b, PDF_CMAP_SINGLE, 16089 },
	{ 0x4c6d, 0x4c6d, PDF_CMAP_SINGLE, 18377 },
	{ 0x4c7d, 0x4c7d, PDF_CMAP_SINGLE, 15974 },
	{ 0x4ca4, 0x4ca4, PDF_CMAP_SINGLE, 18622 },
	{ 0x4cae, 0x4cae, PDF_CMAP_SINGLE, 15426 },
	{ 0x4cb0, 0x4cb0, PDF_CMAP_SINGLE, 15429 },
	{ 0x4cb7, 0x4cb7, PDF_CMAP_SINGLE, 15905 },
	{ 0x4ccd, 0x4ccd, PDF_CMAP_SINGLE, 17784 },
	{ 0x4ce1, 0x4ce1, PDF_CMAP_SINGLE, 16115 },
	{ 0x4ced, 0x4ced, PDF_CMAP_SINGLE, 16555 },
	{ 0x4d09, 0x4d09, PDF_CMAP_SINGLE, 16086 },
	{ 0x4d10, 0x4d10, PDF_CMAP_SINGLE, 16663 },
	{ 0x4d34, 0x4d34, PDF_CMAP_SINGLE, 14645 },
	{ 0x4d91, 0x4d91, PDF_CMAP_SINGLE, 17397 },
	{ 0x4d9c, 0x4d9c, PDF_CMAP_SINGLE, 18628 },
	{ 0x4e00, 0x4e01, PDF_CMAP_TABLE, 226 },
	{ 0x4e03, 0x4e04, PDF_CMAP_TABLE, 228 },
	{ 0x4e07, 0x4e11, PDF_CMAP_TABLE, 230 },
	{ 0x4e14, 0x4e16, PDF_CMAP_TABLE, 241 },
	{ 0x4e18, 0x4e1a, PDF_CMAP_TABLE, 244 },
	{ 0x4e1c, 0x4e1c, PDF_CMAP_SINGLE, 18656 },
	{ 0x4e1e, 0x4e1f, PDF_CMAP_RANGE, 878 },
	{ 0x4e21, 0x4e21, PDF_CMAP_SINGLE, 15725 },
	{ 0x4e24, 0x4e24, PDF_CMAP_SINGLE, 17805 },
	{ 0x4e26, 0x4e26, PDF_CMAP_SINGLE, 1320 },
	{ 0x4e28, 0x4e28, PDF_CMAP_SINGLE, 536 },
	{ 0x4e2a, 0x4e2e, PDF_CMAP_TABLE, 247 },
	{ 0x4e30, 0x4e33, PDF_CMAP_TABLE, 252 },
	{ 0x4e36, 0x4e39, PDF_CMAP_TABLE, 256 },
	{ 0x4e3b, 0x4e3d, PDF_CMAP_TABLE, 260 },
	{ 0x4e3f, 0x4e3f, PDF_CMAP_SINGLE, 538 },
	{ 0x4e41, 0x4e43, PDF_CMAP_TABLE, 263 },
	{ 0x4e45, 0x4e45, PDF_CMAP_SINGLE, 622 },
	{ 0x4e47, 0x4e49, PDF_CMAP_TABLE, 266 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 664 },
	{ 0x4e4d, 0x4e4f, PDF_CMAP_TABLE, 269 },
	{ 0x4e52, 0x4e53, PDF_CMAP_RANGE, 880 },
	{ 0x4e56, 0x4e56, PDF_CMAP_SINGLE, 1321 },
	{ 0x4e58, 0x4e5f, PDF_CMAP_TABLE, 272 },
	{ 0x4e69, 0x4e6a, PDF_CMAP_TABLE, 280 },
	{ 0x4e73, 0x4e73, PDF_CMAP_SINGLE, 1322 },
	{ 0x4e78, 0x4e78, PDF_CMAP_SINGLE, 15758 },
	{ 0x4e7e, 0x4e82, PDF_CMAP_TABLE, 282 },
	{ 0x4e83, 0x4e84, PDF_CMAP_RANGE, 9057 },
	{ 0x4e85, 0x4e89, PDF_CMAP_TABLE, 287 },
	{ 0x4e8b, 0x4e8e, PDF_CMAP_TABLE, 292 },
	{ 0x4e91, 0x4e95, PDF_CMAP_TABLE, 296 },
	{ 0x4e98, 0x4e9b, PDF_CMAP_TABLE, 301 },
	{ 0x4e9e, 0x4ea2, PDF_CMAP_TABLE, 305 },
	{ 0x4ea4, 0x4ea6, PDF_CMAP_TABLE, 310 },
	{ 0x4ea8, 0x4ea8, PDF_CMAP_SINGLE, 1046 },
	{ 0x4eab, 0x4eac, PDF_CMAP_RANGE, 1326 },
	{ 0x4ead, 0x4eae, PDF_CMAP_RANGE, 1700 },
	{ 0x4eb3, 0x4eb3, PDF_CMAP_SINGLE, 2098 },
	{ 0x4eb6, 0x4eb7, PDF_CMAP_TABLE, 313 },
	{ 0x4eb9, 0x4ebc, PDF_CMAP_TABLE, 315 },
	{ 0x4ebf, 0x4ec4, PDF_CMAP_TABLE, 319 },
	{ 0x4ec6, 0x4ec7, PDF_CMAP_RANGE, 675 },
	{ 0x4ec8, 0x4ec9, PDF_CMAP_TABLE, 325 },
	{ 0x4eca, 0x4ecb, PDF_CMAP_RANGE, 678 },
	{ 0x4ecd, 0x4ece, PDF_CMAP_TABLE, 327 },
	{ 0x4ed4, 0x4ed7, PDF_CMAP_RANGE, 762 },
	{ 0x4ed8, 0x4eda, PDF_CMAP_TABLE, 329 },
	{ 0x4edc, 0x4edf, PDF_CMAP_TABLE, 332 },
	{ 0x4ee1, 0x4ee1, PDF_CMAP_SINGLE, 6035 },
	{ 0x4ee3, 0x4ee4, PDF_CMAP_RANGE, 766 },
	{ 0x4ee5, 0x4ee5, PDF_CMAP_SINGLE, 760 },
	{ 0x4ee8, 0x4eeb, PDF_CMAP_TABLE, 336 },
	{ 0x4eee, 0x4eee, PDF_CMAP_SINGLE, 15734 },
	{ 0x4ef0, 0x4ef8, PDF_CMAP_TABLE, 340 },
	{ 0x4efb, 0x4efb, PDF_CMAP_SINGLE, 898 },
	{ 0x4efd, 0x4efd, PDF_CMAP_SINGLE, 901 },
	{ 0x4eff, 0x4f05, PDF_CMAP_TABLE, 349 },
	{ 0x4f08, 0x4f0b, PDF_CMAP_TABLE, 356 },
	{ 0x4f0d, 0x4f0f, PDF_CMAP_TABLE, 360 },
	{ 0x4f10, 0x4f11, PDF_CMAP_RANGE, 893 },
	{ 0x4f12, 0x4f15, PDF_CMAP_TABLE, 363 },
	{ 0x4f18, 0x4f1a, PDF_CMAP_TABLE, 367 },
	{ 0x4f1d, 0x4f1d, PDF_CMAP_SINGLE, 6078 },
	{ 0x4f22, 0x4f22, PDF_CMAP_SINGLE, 6081 },
	{ 0x4f28, 0x4f29, PDF_CMAP_TABLE, 370 },
	{ 0x4f2c, 0x4f2d, PDF_CMAP_TABLE, 372 },
	{ 0x4f2f, 0x4f30, PDF_CMAP_TABLE, 374 },
	{ 0x4f32, 0x4f34, PDF_CMAP_TABLE, 376 },
	{ 0x4f36, 0x4f3f, PDF_CMAP_TABLE, 379 },
	{ 0x4f41, 0x4f43, PDF_CMAP_TABLE, 389 },
	{ 0x4f45, 0x4f49, PDF_CMAP_TABLE, 392 },
	{ 0x4f4b, 0x4f4f, PDF_CMAP_TABLE, 397 },
	{ 0x4f50, 0x4f51, PDF_CMAP_RANGE, 1056 },
	{ 0x4f52, 0x4f64, PDF_CMAP_TABLE, 402 },
	{ 0x4f67, 0x4f67, PDF_CMAP_SINGLE, 6186 },
	{ 0x4f69, 0x4f6c, PDF_CMAP_TABLE, 421 },
	{ 0x4f6e, 0x4f70, PDF_CMAP_TABLE, 425 },
	{ 0x4f72, 0x4f8b, PDF_CMAP_TABLE, 428 },
	{ 0x4f8d, 0x4f8d, PDF_CMAP_SINGLE, 1330 },
	{ 0x4f8f, 0x4f92, PDF_CMAP_TABLE, 454 },
	{ 0x4f94, 0x4f98, PDF_CMAP_TABLE, 458 },
	{ 0x4f9a, 0x4f9e, PDF_CMAP_TABLE, 463 },
	{ 0x4fa2, 0x4fa2, PDF_CMAP_SINGLE, 14791 },
	{ 0x4fa8, 0x4fa8, PDF_CMAP_SINGLE, 17726 },
	{ 0x4fab, 0x4fab, PDF_CMAP_SINGLE, 16418 },
	{ 0x4fae, 0x4fb0, PDF_CMAP_TABLE, 468 },
	{ 0x4fb2, 0x4fb3, PDF_CMAP_TABLE, 471 },
	{ 0x4fb5, 0x4fb7, PDF_CMAP_TABLE, 473 },
	{ 0x4fb9, 0x4fbb, PDF_CMAP_TABLE, 476 },
	{ 0x4fbd, 0x4fbd, PDF_CMAP_SINGLE, 17812 },
	{ 0x4fbf, 0x4fc5, PDF_CMAP_TABLE, 479 },
	{ 0x4fc7, 0x4fd1, PDF_CMAP_TABLE, 486 },
	{ 0x4fd3, 0x4fd4, PDF_CMAP_TABLE, 497 },
	{ 0x4fd6, 0x4fe1, PDF_CMAP_TABLE, 499 },
	{ 0x4fe4, 0x4fe5, PDF_CMAP_TABLE, 511 },
	{ 0x4fec, 0x4fec, PDF_CMAP_SINGLE, 6764 },
	{ 0x4fee, 0x4fef, PDF_CMAP_TABLE, 513 },
	{ 0x4ff1, 0x4ff5, PDF_CMAP_TABLE, 515 },
	{ 0x4ff6, 0x4ff7, PDF_CMAP_RANGE, 7208 },
	{ 0x4ff8, 0x4ffa, PDF_CMAP_TABLE, 520 },
	{ 0x4ffd, 0x4ffe, PDF_CMAP_TABLE, 523 },
	{ 0x5000, 0x5000, PDF_CMAP_SINGLE, 2115 },
	{ 0x5003, 0x5003, PDF_CMAP_SINGLE, 17814 },
	{ 0x5005, 0x5009, PDF_CMAP_TABLE, 525 },
	{ 0x500b, 0x500b, PDF_CMAP_SINGLE, 2120 },
	{ 0x500c, 0x500d, PDF_CMAP_RANGE, 2099 },
	{ 0x500e, 0x500f, PDF_CMAP_TABLE, 530 },
	{ 0x5011, 0x501c, PDF_CMAP_TABLE, 532 },
	{ 0x501e, 0x5023, PDF_CMAP_TABLE, 544 },
	{ 0x5025, 0x5031, PDF_CMAP_TABLE, 550 },
	{ 0x5033, 0x5035, PDF_CMAP_TABLE, 563 },
	{ 0x5037, 0x5037, PDF_CMAP_SINGLE, 7206 },
	{ 0x503c, 0x503c, PDF_CMAP_SINGLE, 2109 },
	{ 0x5040, 0x5041, PDF_CMAP_TABLE, 566 },
	{ 0x5043, 0x5043, PDF_CMAP_SINGLE, 2560 },
	{ 0x5045, 0x504f, PDF_CMAP_TABLE, 568 },
	{ 0x5051, 0x5051, PDF_CMAP_SINGLE, 7761 },
	{ 0x5053, 0x5053, PDF_CMAP_SINGLE, 7739 },
	{ 0x5055, 0x5058, PDF_CMAP_TABLE, 579 },
	{ 0x505a, 0x5062, PDF_CMAP_TABLE, 583 },
	{ 0x5063, 0x5064, PDF_CMAP_RANGE, 7754 },
	{ 0x5065, 0x5066, PDF_CMAP_TABLE, 592 },
	{ 0x5068, 0x5070, PDF_CMAP_TABLE, 594 },
	{ 0x5072, 0x5077, PDF_CMAP_TABLE, 603 },
	{ 0x507a, 0x507a, PDF_CMAP_SINGLE, 2556 },
	{ 0x507d, 0x507d, PDF_CMAP_SINGLE, 2557 },
	{ 0x5080, 0x5083, PDF_CMAP_TABLE, 609 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 3049 },
	{ 0x5087, 0x5088, PDF_CMAP_TABLE, 613 },
	{ 0x508b, 0x508e, PDF_CMAP_TABLE, 615 },
	{ 0x5090, 0x5092, PDF_CMAP_TABLE, 619 },
	{ 0x5094, 0x5096, PDF_CMAP_TABLE, 622 },
	{ 0x5098, 0x509e, PDF_CMAP_TABLE, 625 },
	{ 0x50a2, 0x50a3, PDF_CMAP_TABLE, 632 },
	{ 0x50a6, 0x50a6, PDF_CMAP_SINGLE, 16191 },
	{ 0x50ac, 0x50b1, PDF_CMAP_TABLE, 634 },
	{ 0x50b2, 0x50b3, PDF_CMAP_RANGE, 3521 },
	{ 0x50b4, 0x50b8, PDF_CMAP_TABLE, 640 },
	{ 0x50ba, 0x50bb, PDF_CMAP_TABLE, 645 },
	{ 0x50bd, 0x50bf, PDF_CMAP_TABLE, 647 },
	{ 0x50c1, 0x50c2, PDF_CMAP_TABLE, 650 },
	{ 0x50c4, 0x50cb, PDF_CMAP_TABLE, 652 },
	{ 0x50cd, 0x50d1, PDF_CMAP_TABLE, 660 },
	{ 0x50d3, 0x50d7, PDF_CMAP_TABLE, 665 },
	{ 0x50d9, 0x50db, PDF_CMAP_TABLE, 670 },
	{ 0x50dd, 0x50e1, PDF_CMAP_TABLE, 673 },
	{ 0x50e3, 0x50ea, PDF_CMAP_TABLE, 678 },
	{ 0x50ec, 0x50f1, PDF_CMAP_TABLE, 686 },
	{ 0x50f3, 0x50f6, PDF_CMAP_TABLE, 692 },
	{ 0x50f8, 0x50f9, PDF_CMAP_TABLE, 696 },
	{ 0x50fb, 0x5105, PDF_CMAP_TABLE, 698 },
	{ 0x5106, 0x5107, PDF_CMAP_RANGE, 10369 },
	{ 0x5108, 0x5109, PDF_CMAP_RANGE, 4360 },
	{ 0x510a, 0x510a, PDF_CMAP_SINGLE, 10376 },
	{ 0x510b, 0x510c, PDF_CMAP_RANGE, 10373 },
	{ 0x510d, 0x510e, PDF_CMAP_TABLE, 709 },
	{ 0x5110, 0x5115, PDF_CMAP_TABLE, 711 },
	{ 0x5117, 0x5118, PDF_CMAP_TABLE, 717 },
	{ 0x511a, 0x511a, PDF_CMAP_SINGLE, 11011 },
	{ 0x511c, 0x511c, PDF_CMAP_SINGLE, 11008 },
	{ 0x511f, 0x5122, PDF_CMAP_TABLE, 719 },
	{ 0x5124, 0x5126, PDF_CMAP_TABLE, 723 },
	{ 0x5129, 0x512b, PDF_CMAP_TABLE, 726 },
	{ 0x512d, 0x512e, PDF_CMAP_RANGE, 12047 },
	{ 0x5130, 0x5133, PDF_CMAP_TABLE, 729 },
	{ 0x5134, 0x5135, PDF_CMAP_RANGE, 12410 },
	{ 0x5137, 0x5138, PDF_CMAP_RANGE, 5736 },
	{ 0x5139, 0x513d, PDF_CMAP_TABLE, 733 },
	{ 0x513f, 0x5141, PDF_CMAP_TABLE, 738 },
	{ 0x5143, 0x5149, PDF_CMAP_TABLE, 741 },
	{ 0x514b, 0x514d, PDF_CMAP_TABLE, 748 },
	{ 0x5152, 0x5152, PDF_CMAP_SINGLE, 1349 },
	{ 0x5154, 0x5157, PDF_CMAP_TABLE, 751 },
	{ 0x5159, 0x5163, PDF_CMAP_TABLE, 755 },
	{ 0x5165, 0x5165, PDF_CMAP_SINGLE, 605 },
	{ 0x5167, 0x516c, PDF_CMAP_TABLE, 766 },
	{ 0x516d, 0x516e, PDF_CMAP_RANGE, 684 },
	{ 0x5171, 0x5171, PDF_CMAP_SINGLE, 909 },
	{ 0x5174, 0x5179, PDF_CMAP_TABLE, 772 },
	{ 0x517c, 0x517c, PDF_CMAP_SINGLE, 2130 },
	{ 0x5180, 0x5180, PDF_CMAP_SINGLE, 4751 },
	{ 0x5182, 0x5182, PDF_CMAP_SINGLE, 541 },
	{ 0x5186, 0x5188, PDF_CMAP_TABLE, 778 },
	{ 0x5189, 0x518a, PDF_CMAP_RANGE, 772 },
	{ 0x518d, 0x518d, PDF_CMAP_SINGLE, 910 },
	{ 0x518f, 0x518f, PDF_CMAP_SINGLE, 6195 },
	{ 0x5191, 0x5198, PDF_CMAP_TABLE, 781 },
	{ 0x519a, 0x519a, PDF_CMAP_SINGLE, 16035 },
	{ 0x519c, 0x519c, PDF_CMAP_SINGLE, 17728 },
	{ 0x519e, 0x519e, PDF_CMAP_SINGLE, 6416 },
	{ 0x51a0, 0x51a0, PDF_CMAP_SINGLE, 1727 },
	{ 0x51a2, 0x51a2, PDF_CMAP_SINGLE, 2133 },
	{ 0x51a4, 0x51a5, PDF_CMAP_RANGE, 2131 },
	{ 0x51a7, 0x51a8, PDF_CMAP_TABLE, 789 },
	{ 0x51aa, 0x51ac, PDF_CMAP_TABLE, 791 },
	{ 0x51b0, 0x51b5, PDF_CMAP_TABLE, 794 },
	{ 0x51b6, 0x51b7, PDF_CMAP_RANGE, 1079 },
	{ 0x51b8, 0x51b9, PDF_CMAP_TABLE, 800 },
	{ 0x51bc, 0x51be, PDF_CMAP_TABLE, 802 },
	{ 0x51c3, 0x51c3, PDF_CMAP_SINGLE, 14811 },
	{ 0x51c4, 0x51c5, PDF_CMAP_RANGE, 7222 },
	{ 0x51c6, 0x51d4, PDF_CMAP_TABLE, 805 },
	{ 0x51d7, 0x51d8, PDF_CMAP_TABLE, 820 },
	{ 0x51db, 0x51e2, PDF_CMAP_TABLE, 822 },
	{ 0x51e4, 0x51e4, PDF_CMAP_SINGLE, 17729 },
	{ 0x51ed, 0x51ed, PDF_CMAP_SINGLE, 16468 },
	{ 0x51f0, 0x51f1, PDF_CMAP_TABLE, 830 },
	{ 0x51f3, 0x51f6, PDF_CMAP_TABLE, 832 },
	{ 0x51f8, 0x51f8, PDF_CMAP_SINGLE, 777 },
	{ 0x51f9, 0x51fa, PDF_CMAP_RANGE, 775 },
	{ 0x51fc, 0x51fe, PDF_CMAP_TABLE, 836 },
	{ 0x5200, 0x5201, PDF_CMAP_RANGE, 608 },
	{ 0x5202, 0x5203, PDF_CMAP_TABLE, 839 },
	{ 0x5205, 0x5205, PDF_CMAP_SINGLE, 17313 },
	{ 0x5206, 0x5208, PDF_CMAP_RANGE, 689 },
	{ 0x5209, 0x520c, PDF_CMAP_TABLE, 841 },
	{ 0x520e, 0x520e, PDF_CMAP_SINGLE, 915 },
	{ 0x5210, 0x5210, PDF_CMAP_SINGLE, 6088 },
	{ 0x5211, 0x5212, PDF_CMAP_RANGE, 913 },
	{ 0x5213, 0x5213, PDF_CMAP_SINGLE, 6087 },
	{ 0x5216, 0x5217, PDF_CMAP_TABLE, 845 },
	{ 0x521c, 0x5221, PDF_CMAP_TABLE, 847 },
	{ 0x5224, 0x5228, PDF_CMAP_TABLE, 853 },
	{ 0x5229, 0x522a, PDF_CMAP_RANGE, 1083 },
	{ 0x522e, 0x522e, PDF_CMAP_SINGLE, 1362 },
	{ 0x5230, 0x5231, PDF_CMAP_TABLE, 858 },
	{ 0x5232, 0x5233, PDF_CMAP_RANGE, 6420 },
	{ 0x5234, 0x5238, PDF_CMAP_TABLE, 860 },
	{ 0x523a, 0x523c, PDF_CMAP_TABLE, 865 },
	{ 0x5241, 0x5241, PDF_CMAP_SINGLE, 1364 },
	{ 0x5243, 0x5244, PDF_CMAP_TABLE, 868 },
	{ 0x5246, 0x5247, PDF_CMAP_TABLE, 870 },
	{ 0x5249, 0x524e, PDF_CMAP_TABLE, 872 },
	{ 0x5252, 0x5252, PDF_CMAP_SINGLE, 7228 },
	{ 0x5254, 0x5257, PDF_CMAP_TABLE, 878 },
	{ 0x5259, 0x525d, PDF_CMAP_TABLE, 882 },
	{ 0x525e, 0x525f, PDF_CMAP_RANGE, 7229 },
	{ 0x5260, 0x5262, PDF_CMAP_TABLE, 887 },
	{ 0x5268, 0x526f, PDF_CMAP_TABLE, 890 },
	{ 0x5272, 0x5273, PDF_CMAP_TABLE, 898 },
	{ 0x5274, 0x5275, PDF_CMAP_RANGE, 3059 },
	{ 0x5277, 0x527a, PDF_CMAP_TABLE, 900 },
	{ 0x527b, 0x527c, PDF_CMAP_RANGE, 9080 },
	{ 0x527d, 0x527d, PDF_CMAP_SINGLE, 3532 },
	{ 0x527f, 0x527f, PDF_CMAP_SINGLE, 3530 },
	{ 0x5280, 0x5281, PDF_CMAP_RANGE, 9763 },
	{ 0x5282, 0x5284, PDF_CMAP_TABLE, 904 },
	{ 0x5287, 0x5289, PDF_CMAP_RANGE, 4364 },
	{ 0x528a, 0x528a, PDF_CMAP_SINGLE, 4368 },
	{ 0x528b, 0x528c, PDF_CMAP_RANGE, 10377 },
	{ 0x528d, 0x528d, PDF_CMAP_SINGLE, 4367 },
	{ 0x528f, 0x5291, PDF_CMAP_TABLE, 907 },
	{ 0x5293, 0x5294, PDF_CMAP_TABLE, 910 },
	{ 0x5296, 0x529b, PDF_CMAP_TABLE, 912 },
	{ 0x529f, 0x52a1, PDF_CMAP_TABLE, 918 },
	{ 0x52a3, 0x52a4, PDF_CMAP_TABLE, 921 },
	{ 0x52a6, 0x52a6, PDF_CMAP_SINGLE, 6089 },
	{ 0x52a8, 0x52a8, PDF_CMAP_SINGLE, 17731 },
	{ 0x52a9, 0x52aa, PDF_CMAP_RANGE, 1087 },
	{ 0x52ab, 0x52ac, PDF_CMAP_TABLE, 923 },
	{ 0x52ad, 0x52ae, PDF_CMAP_RANGE, 6200 },
	{ 0x52b5, 0x52b5, PDF_CMAP_SINGLE, 16476 },
	{ 0x52b9, 0x52b9, PDF_CMAP_SINGLE, 16479 },
	{ 0x52bb, 0x52bc, PDF_CMAP_TABLE, 925 },
	{ 0x52be, 0x52be, PDF_CMAP_SINGLE, 1365 },
	{ 0x52c0, 0x52c3, PDF_CMAP_TABLE, 927 },
	{ 0x52c5, 0x52c5, PDF_CMAP_SINGLE, 16480 },
	{ 0x52c7, 0x52c7, PDF_CMAP_SINGLE, 1735 },
	{ 0x52c9, 0x52c9, PDF_CMAP_SINGLE, 1736 },
	{ 0x52cc, 0x52cd, PDF_CMAP_TABLE, 931 },
	{ 0x52d0, 0x52d3, PDF_CMAP_TABLE, 933 },
	{ 0x52d5, 0x52d9, PDF_CMAP_TABLE, 937 },
	{ 0x52db, 0x52db, PDF_CMAP_SINGLE, 3064 },
	{ 0x52dd, 0x52e1, PDF_CMAP_TABLE, 942 },
	{ 0x52e2, 0x52e3, PDF_CMAP_RANGE, 3536 },
	{ 0x52e4, 0x52e4, PDF_CMAP_SINGLE, 3535 },
	{ 0x52e6, 0x52e6, PDF_CMAP_SINGLE, 3534 },
	{ 0x52e9, 0x52e9, PDF_CMAP_SINGLE, 9765 },
	{ 0x52eb, 0x52eb, PDF_CMAP_SINGLE, 9766 },
	{ 0x52ef, 0x52f1, PDF_CMAP_TABLE, 947 },
	{ 0x52f3, 0x52f5, PDF_CMAP_TABLE, 950 },
	{ 0x52f7, 0x52fc, PDF_CMAP_TABLE, 953 },
	{ 0x52fe, 0x52ff, PDF_CMAP_RANGE, 693 },
	{ 0x5301, 0x5301, PDF_CMAP_SINGLE, 15737 },
	{ 0x5305, 0x5306, PDF_CMAP_RANGE, 781 },
	{ 0x5308, 0x5309, PDF_CMAP_TABLE, 959 },
	{ 0x530a, 0x530b, PDF_CMAP_RANGE, 6425 },
	{ 0x530d, 0x5312, PDF_CMAP_TABLE, 961 },
	{ 0x5315, 0x5317, PDF_CMAP_TABLE, 967 },
	{ 0x5319, 0x531a, PDF_CMAP_TABLE, 970 },
	{ 0x531c, 0x531d, PDF_CMAP_TABLE, 972 },
	{ 0x531f, 0x5323, PDF_CMAP_TABLE, 974 },
	{ 0x5327, 0x5327, PDF_CMAP_SINGLE, 17822 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 2143 },
	{ 0x532c, 0x532d, PDF_CMAP_TABLE, 979 },
	{ 0x532f, 0x5334, PDF_CMAP_TABLE, 981 },
	{ 0x5337, 0x5339, PDF_CMAP_TABLE, 987 },
	{ 0x533b, 0x533e, PDF_CMAP_TABLE, 990 },
	{ 0x533f, 0x5340, PDF_CMAP_RANGE, 2587 },
	{ 0x5341, 0x5345, PDF_CMAP_TABLE, 994 },
	{ 0x5347, 0x534a, PDF_CMAP_TABLE, 999 },
	{ 0x534c, 0x534e, PDF_CMAP_TABLE, 1003 },
	{ 0x5351, 0x5354, PDF_CMAP_TABLE, 1006 },
	{ 0x5357, 0x5357, PDF_CMAP_SINGLE, 1740 },
	{ 0x535a, 0x535a, PDF_CMAP_SINGLE, 3065 },
	{ 0x535c, 0x5361, PDF_CMAP_TABLE, 1010 },
	{ 0x5363, 0x5364, PDF_CMAP_TABLE, 1016 },
	{ 0x5366, 0x5367, PDF_CMAP_TABLE, 1018 },
	{ 0x5369, 0x5369, PDF_CMAP_SINGLE, 546 },
	{ 0x536c, 0x536f, PDF_CMAP_TABLE, 1020 },
	{ 0x5370, 0x5371, PDF_CMAP_RANGE, 921 },
	{ 0x5372, 0x5375, PDF_CMAP_TABLE, 1024 },
	{ 0x5377, 0x5379, PDF_CMAP_RANGE, 1372 },
	{ 0x537b, 0x537f, PDF_CMAP_TABLE, 1028 },
	{ 0x5382, 0x5382, PDF_CMAP_SINGLE, 6000 },
	{ 0x5384, 0x5384, PDF_CMAP_SINGLE, 701 },
	{ 0x538a, 0x538a, PDF_CMAP_SINGLE, 6093 },
	{ 0x538e, 0x538f, PDF_CMAP_RANGE, 6205 },
	{ 0x5392, 0x5394, PDF_CMAP_TABLE, 1033 },
	{ 0x5396, 0x539a, PDF_CMAP_TABLE, 1036 },
	{ 0x539c, 0x53a0, PDF_CMAP_TABLE, 1041 },
	{ 0x53a2, 0x53a2, PDF_CMAP_SINGLE, 15915 },
	{ 0x53a4, 0x53ae, PDF_CMAP_TABLE, 1046 },
	{ 0x53b0, 0x53b0, PDF_CMAP_SINGLE, 17826 },
	{ 0x53b2, 0x53b2, PDF_CMAP_SINGLE, 4370 },
	{ 0x53b4, 0x53b4, PDF_CMAP_SINGLE, 12414 },
	{ 0x53b6, 0x53b6, PDF_CMAP_SINGLE, 547 },
	{ 0x53b9, 0x53b9, PDF_CMAP_SINGLE, 6019 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 792 },
	{ 0x53c1, 0x53c3, PDF_CMAP_TABLE, 1057 },
	{ 0x53c5, 0x53c5, PDF_CMAP_SINGLE, 16524 },
	{ 0x53c8, 0x53cd, PDF_CMAP_TABLE, 1060 },
	{ 0x53d0, 0x53d2, PDF_CMAP_TABLE, 1066 },
	{ 0x53d4, 0x53d4, PDF_CMAP_SINGLE, 1376 },
	{ 0x53d6, 0x53db, PDF_CMAP_TABLE, 1069 },
	{ 0x53df, 0x53e6, PDF_CMAP_TABLE, 1075 },
	{ 0x53e8, 0x53f3, PDF_CMAP_TABLE, 1083 },
	{ 0x53f5, 0x53f8, PDF_CMAP_TABLE, 1095 },
	{ 0x53fb, 0x53fc, PDF_CMAP_TABLE, 1099 },
	{ 0x53fe, 0x53fe, PDF_CMAP_SINGLE, 16577 },
	{ 0x5401, 0x5401, PDF_CMAP_SINGLE, 928 },
	{ 0x5403, 0x5404, PDF_CMAP_TABLE, 1101 },
	{ 0x5406, 0x5414, PDF_CMAP_TABLE, 1103 },
	{ 0x5416, 0x5416, PDF_CMAP_SINGLE, 15999 },
	{ 0x5418, 0x5421, PDF_CMAP_TABLE, 1118 },
	{ 0x5423, 0x5433, PDF_CMAP_TABLE, 1128 },
	{ 0x5435, 0x5436, PDF_CMAP_RANGE, 1112 },
	{ 0x5437, 0x5439, PDF_CMAP_TABLE, 1145 },
	{ 0x543b, 0x543e, PDF_CMAP_TABLE, 1148 },
	{ 0x5440, 0x5443, PDF_CMAP_TABLE, 1152 },
	{ 0x5445, 0x5448, PDF_CMAP_TABLE, 1156 },
	{ 0x544a, 0x544b, PDF_CMAP_TABLE, 1160 },
	{ 0x544d, 0x544f, PDF_CMAP_TABLE, 1162 },
	{ 0x5454, 0x5454, PDF_CMAP_SINGLE, 6210 },
	{ 0x5460, 0x546d, PDF_CMAP_TABLE, 1165 },
	{ 0x546f, 0x5478, PDF_CMAP_TABLE, 1179 },
	{ 0x547a, 0x5482, PDF_CMAP_TABLE, 1189 },
	{ 0x5484, 0x5488, PDF_CMAP_TABLE, 1198 },
	{ 0x548b, 0x5498, PDF_CMAP_TABLE, 1203 },
	{ 0x549a, 0x549a, PDF_CMAP_SINGLE, 1394 },
	{ 0x549c, 0x549c, PDF_CMAP_SINGLE, 16230 },
	{ 0x549e, 0x549e, PDF_CMAP_SINGLE, 18413 },
	{ 0x54a0, 0x54b4, PDF_CMAP_TABLE, 1217 },
	{ 0x54b6, 0x54c4, PDF_CMAP_TABLE, 1238 },
	{ 0x54c5, 0x54c6, PDF_CMAP_RANGE, 6786 },
	{ 0x54c7, 0x54c9, PDF_CMAP_TABLE, 1253 },
	{ 0x54cb, 0x54d0, PDF_CMAP_TABLE, 1256 },
	{ 0x54d6, 0x54d6, PDF_CMAP_SINGLE, 6784 },
	{ 0x54da, 0x54da, PDF_CMAP_SINGLE, 18723 },
	{ 0x54de, 0x54de, PDF_CMAP_SINGLE, 6794 },
	{ 0x54e0, 0x54eb, PDF_CMAP_TABLE, 1262 },
	{ 0x54ed, 0x54ef, PDF_CMAP_TABLE, 1274 },
	{ 0x54f1, 0x54f3, PDF_CMAP_TABLE, 1277 },
	{ 0x54f7, 0x54f8, PDF_CMAP_RANGE, 7253 },
	{ 0x54fa, 0x54fd, PDF_CMAP_TABLE, 1280 },
	{ 0x54ff, 0x54ff, PDF_CMAP_SINGLE, 7244 },
	{ 0x5501, 0x5514, PDF_CMAP_TABLE, 1284 },
	{ 0x5517, 0x5518, PDF_CMAP_TABLE, 1304 },
	{ 0x551a, 0x551a, PDF_CMAP_SINGLE, 7243 },
	{ 0x551e, 0x551e, PDF_CMAP_SINGLE, 16040 },
	{ 0x5523, 0x5523, PDF_CMAP_SINGLE, 17838 },
	{ 0x5525, 0x5528, PDF_CMAP_TABLE, 1306 },
	{ 0x552a, 0x5539, PDF_CMAP_TABLE, 1310 },
	{ 0x553b, 0x553c, PDF_CMAP_TABLE, 1326 },
	{ 0x553e, 0x5541, PDF_CMAP_TABLE, 1328 },
	{ 0x5543, 0x554b, PDF_CMAP_TABLE, 1332 },
	{ 0x554d, 0x5553, PDF_CMAP_TABLE, 1341 },
	{ 0x5555, 0x5557, PDF_CMAP_TABLE, 1348 },
	{ 0x555c, 0x555f, PDF_CMAP_TABLE, 1351 },
	{ 0x5561, 0x5566, PDF_CMAP_TABLE, 1355 },
	{ 0x5569, 0x556b, PDF_CMAP_TABLE, 1361 },
	{ 0x5571, 0x5573, PDF_CMAP_TABLE, 1364 },
	{ 0x5575, 0x5576, PDF_CMAP_RANGE, 7771 },
	{ 0x5577, 0x5577, PDF_CMAP_SINGLE, 8391 },
	{ 0x5579, 0x5579, PDF_CMAP_SINGLE, 17245 },
	{ 0x557b, 0x5584, PDF_CMAP_TABLE, 1367 },
	{ 0x5586, 0x5595, PDF_CMAP_TABLE, 1377 },
	{ 0x5598, 0x559a, PDF_CMAP_TABLE, 1393 },
	{ 0x559c, 0x559d, PDF_CMAP_TABLE, 1396 },
	{ 0x559f, 0x559f, PDF_CMAP_SINGLE, 3083 },
	{ 0x55a1, 0x55ae, PDF_CMAP_TABLE, 1398 },
	{ 0x55b0, 0x55b5, PDF_CMAP_TABLE, 1412 },
	{ 0x55b9, 0x55bc, PDF_CMAP_TABLE, 1418 },
	{ 0x55bf, 0x55c4, PDF_CMAP_TABLE, 1422 },
	{ 0x55c5, 0x55c6, PDF_CMAP_RANGE, 3552 },
	{ 0x55c7, 0x55df, PDF_CMAP_TABLE, 1428 },
	{ 0x55e1, 0x55e2, PDF_CMAP_TABLE, 1453 },
	{ 0x55e3, 0x55e4, PDF_CMAP_RANGE, 3547 },
	{ 0x55e5, 0x55ea, PDF_CMAP_TABLE, 1455 },
	{ 0x55ec, 0x55ec, PDF_CMAP_SINGLE, 14290 },
	{ 0x55ef, 0x55f2, PDF_CMAP_TABLE, 1461 },
	{ 0x55f5, 0x55f7, PDF_CMAP_TABLE, 1465 },
	{ 0x55f9, 0x5602, PDF_CMAP_TABLE, 1468 },
	{ 0x5604, 0x5606, PDF_CMAP_TABLE, 1478 },
	{ 0x5608, 0x5609, PDF_CMAP_TABLE, 1481 },
	{ 0x560c, 0x560c, PDF_CMAP_SINGLE, 9771 },
	{ 0x560d, 0x560e, PDF_CMAP_RANGE, 3992 },
	{ 0x560f, 0x5617, PDF_CMAP_TABLE, 1483 },
	{ 0x561b, 0x5623, PDF_CMAP_TABLE, 1492 },
	{ 0x5625, 0x5625, PDF_CMAP_SINGLE, 15992 },
	{ 0x5627, 0x5627, PDF_CMAP_SINGLE, 9769 },
	{ 0x5629, 0x562a, PDF_CMAP_TABLE, 1501 },
	{ 0x562c, 0x562e, PDF_CMAP_TABLE, 1503 },
	{ 0x562f, 0x5630, PDF_CMAP_RANGE, 4383 },
	{ 0x5632, 0x563b, PDF_CMAP_TABLE, 1506 },
	{ 0x563d, 0x5643, PDF_CMAP_TABLE, 1516 },
	{ 0x5645, 0x5646, PDF_CMAP_TABLE, 1523 },
	{ 0x5648, 0x564a, PDF_CMAP_TABLE, 1525 },
	{ 0x564c, 0x5650, PDF_CMAP_TABLE, 1528 },
	{ 0x5652, 0x5654, PDF_CMAP_TABLE, 1533 },
	{ 0x5657, 0x565a, PDF_CMAP_TABLE, 1536 },
	{ 0x565d, 0x565e, PDF_CMAP_TABLE, 1540 },
	{ 0x5660, 0x5666, PDF_CMAP_TABLE, 1542 },
	{ 0x5668, 0x5674, PDF_CMAP_TABLE, 1549 },
	{ 0x5676, 0x567c, PDF_CMAP_TABLE, 1562 },
	{ 0x567e, 0x5687, PDF_CMAP_TABLE, 1569 },
	{ 0x5689, 0x568b, PDF_CMAP_TABLE, 1579 },
	{ 0x568c, 0x568d, PDF_CMAP_RANGE, 11586 },
	{ 0x568e, 0x5690, PDF_CMAP_TABLE, 1582 },
	{ 0x5692, 0x5693, PDF_CMAP_TABLE, 1585 },
	{ 0x5695, 0x5695, PDF_CMAP_SINGLE, 5322 },
	{ 0x5697, 0x569a, PDF_CMAP_TABLE, 1587 },
	{ 0x569c, 0x569f, PDF_CMAP_TABLE, 1591 },
	{ 0x56a1, 0x56a1, PDF_CMAP_SINGLE, 17259 },
	{ 0x56a4, 0x56a5, PDF_CMAP_TABLE, 1595 },
	{ 0x56a6, 0x56a7, PDF_CMAP_RANGE, 12417 },
	{ 0x56a8, 0x56a8, PDF_CMAP_SINGLE, 5495 },
	{ 0x56aa, 0x56af, PDF_CMAP_TABLE, 1597 },
	{ 0x56b1, 0x56b7, PDF_CMAP_TABLE, 1603 },
	{ 0x56b9, 0x56b9, PDF_CMAP_SINGLE, 18543 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 5645 },
	{ 0x56bd, 0x56be, PDF_CMAP_RANGE, 13002 },
	{ 0x56bf, 0x56c3, PDF_CMAP_TABLE, 1610 },
	{ 0x56c5, 0x56c6, PDF_CMAP_TABLE, 1615 },
	{ 0x56c8, 0x56cd, PDF_CMAP_TABLE, 1617 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 5919 },
	{ 0x56d3, 0x56d4, PDF_CMAP_TABLE, 1623 },
	{ 0x56d6, 0x56d7, PDF_CMAP_TABLE, 1625 },
	{ 0x56da, 0x56db, PDF_CMAP_TABLE, 1627 },
	{ 0x56dd, 0x56e2, PDF_CMAP_TABLE, 1629 },
	{ 0x56e4, 0x56e5, PDF_CMAP_TABLE, 1635 },
	{ 0x56e7, 0x56e7, PDF_CMAP_SINGLE, 6223 },
	{ 0x56ea, 0x56eb, PDF_CMAP_TABLE, 1637 },
	{ 0x56ed, 0x56f1, PDF_CMAP_TABLE, 1639 },
	{ 0x56f7, 0x56f7, PDF_CMAP_SINGLE, 6451 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 1644 },
	{ 0x56fd, 0x56fd, PDF_CMAP_SINGLE, 15716 },
	{ 0x56ff, 0x5700, PDF_CMAP_TABLE, 1646 },
	{ 0x5701, 0x5702, PDF_CMAP_RANGE, 7259 },
	{ 0x5703, 0x5704, PDF_CMAP_RANGE, 2169 },
	{ 0x5707, 0x570d, PDF_CMAP_TABLE, 1648 },
	{ 0x5712, 0x5713, PDF_CMAP_RANGE, 3556 },
	{ 0x5714, 0x5716, PDF_CMAP_TABLE, 1655 },
	{ 0x5718, 0x5718, PDF_CMAP_SINGLE, 4000 },
	{ 0x571a, 0x5720, PDF_CMAP_TABLE, 1658 },
	{ 0x5722, 0x5723, PDF_CMAP_RANGE, 6042 },
	{ 0x5728, 0x572a, PDF_CMAP_TABLE, 1665 },
	{ 0x572c, 0x5730, PDF_CMAP_TABLE, 1668 },
	{ 0x5732, 0x5734, PDF_CMAP_TABLE, 1673 },
	{ 0x573b, 0x573b, PDF_CMAP_SINGLE, 1134 },
	{ 0x573e, 0x5743, PDF_CMAP_TABLE, 1676 },
	{ 0x5745, 0x5747, PDF_CMAP_TABLE, 1682 },
	{ 0x5749, 0x5752, PDF_CMAP_TABLE, 1685 },
	{ 0x5754, 0x5754, PDF_CMAP_SINGLE, 18309 },
	{ 0x5757, 0x5757, PDF_CMAP_SINGLE, 18406 },
	{ 0x575b, 0x575b, PDF_CMAP_SINGLE, 14722 },
	{ 0x575f, 0x575f, PDF_CMAP_SINGLE, 16319 },
	{ 0x5761, 0x5762, PDF_CMAP_TABLE, 1695 },
	{ 0x5764, 0x5764, PDF_CMAP_SINGLE, 1407 },
	{ 0x5766, 0x576b, PDF_CMAP_TABLE, 1697 },
	{ 0x576d, 0x576d, PDF_CMAP_SINGLE, 6455 },
	{ 0x576f, 0x5772, PDF_CMAP_TABLE, 1703 },
	{ 0x5773, 0x5774, PDF_CMAP_RANGE, 6463 },
	{ 0x5775, 0x5777, PDF_CMAP_TABLE, 1707 },
	{ 0x577a, 0x5780, PDF_CMAP_TABLE, 1710 },
	{ 0x5782, 0x5783, PDF_CMAP_TABLE, 1717 },
	{ 0x5788, 0x5788, PDF_CMAP_SINGLE, 18507 },
	{ 0x578a, 0x578d, PDF_CMAP_TABLE, 1719 },
	{ 0x578f, 0x5790, PDF_CMAP_TABLE, 1723 },
	{ 0x5793, 0x5795, PDF_CMAP_TABLE, 1725 },
	{ 0x5797, 0x579d, PDF_CMAP_TABLE, 1728 },
	{ 0x579e, 0x579f, PDF_CMAP_RANGE, 6797 },
	{ 0x57a0, 0x57a5, PDF_CMAP_TABLE, 1735 },
	{ 0x57a7, 0x57a7, PDF_CMAP_SINGLE, 18708 },
	{ 0x57aa, 0x57aa, PDF_CMAP_SINGLE, 18693 },
	{ 0x57ae, 0x57ae, PDF_CMAP_SINGLE, 1773 },
	{ 0x57b4, 0x57b6, PDF_CMAP_TABLE, 1741 },
	{ 0x57b8, 0x57bf, PDF_CMAP_TABLE, 1744 },
	{ 0x57c1, 0x57c4, PDF_CMAP_TABLE, 1752 },
	{ 0x57c6, 0x57c8, PDF_CMAP_TABLE, 1756 },
	{ 0x57cb, 0x57cc, PDF_CMAP_TABLE, 1759 },
	{ 0x57ce, 0x57d0, PDF_CMAP_TABLE, 1761 },
	{ 0x57d2, 0x57d2, PDF_CMAP_SINGLE, 7264 },
	{ 0x57d4, 0x57d5, PDF_CMAP_TABLE, 1764 },
	{ 0x57d7, 0x57d7, PDF_CMAP_SINGLE, 15485 },
	{ 0x57dc, 0x57e7, PDF_CMAP_TABLE, 1766 },
	{ 0x57e9, 0x57e9, PDF_CMAP_SINGLE, 7825 },
	{ 0x57ec, 0x57fe, PDF_CMAP_TABLE, 1778 },
	{ 0x5800, 0x580e, PDF_CMAP_TABLE, 1797 },
	{ 0x5810, 0x5810, PDF_CMAP_SINGLE, 7820 },
	{ 0x5812, 0x5812, PDF_CMAP_SINGLE, 15626 },
	{ 0x5814, 0x5814, PDF_CMAP_SINGLE, 7798 },
	{ 0x5819, 0x5819, PDF_CMAP_SINGLE, 8412 },
	{ 0x581b, 0x581e, PDF_CMAP_TABLE, 1812 },
	{ 0x5820, 0x582a, PDF_CMAP_TABLE, 1816 },
	{ 0x582c, 0x582f, PDF_CMAP_TABLE, 1827 },
	{ 0x5830, 0x5831, PDF_CMAP_RANGE, 3099 },
	{ 0x5832, 0x583b, PDF_CMAP_TABLE, 1831 },
	{ 0x583d, 0x583d, PDF_CMAP_SINGLE, 9119 },
	{ 0x583f, 0x5840, PDF_CMAP_TABLE, 1841 },
	{ 0x5844, 0x5844, PDF_CMAP_SINGLE, 18363 },
	{ 0x5847, 0x584f, PDF_CMAP_TABLE, 1843 },
	{ 0x5851, 0x5855, PDF_CMAP_TABLE, 1852 },
	{ 0x5857, 0x585f, PDF_CMAP_TABLE, 1857 },
	{ 0x5862, 0x5865, PDF_CMAP_TABLE, 1866 },
	{ 0x5868, 0x5869, PDF_CMAP_TABLE, 1870 },
	{ 0x586b, 0x586d, PDF_CMAP_TABLE, 1872 },
	{ 0x586f, 0x586f, PDF_CMAP_SINGLE, 9112 },
	{ 0x5871, 0x5876, PDF_CMAP_TABLE, 1875 },
	{ 0x5879, 0x5883, PDF_CMAP_TABLE, 1881 },
	{ 0x5885, 0x588b, PDF_CMAP_TABLE, 1892 },
	{ 0x588e, 0x5891, PDF_CMAP_TABLE, 1899 },
	{ 0x5893, 0x5894, PDF_CMAP_TABLE, 1903 },
	{ 0x5898, 0x589a, PDF_CMAP_TABLE, 1905 },
	{ 0x589c, 0x58a1, PDF_CMAP_TABLE, 1908 },
	{ 0x58a3, 0x58a3, PDF_CMAP_SINGLE, 10404 },
	{ 0x58a5, 0x58ac, PDF_CMAP_TABLE, 1914 },
	{ 0x58ae, 0x58b1, PDF_CMAP_TABLE, 1922 },
	{ 0x58b3, 0x58b3, PDF_CMAP_SINGLE, 4388 },
	{ 0x58b5, 0x58b6, PDF_CMAP_TABLE, 1926 },
	{ 0x58ba, 0x58bf, PDF_CMAP_TABLE, 1928 },
	{ 0x58c1, 0x58c2, PDF_CMAP_TABLE, 1934 },
	{ 0x58c5, 0x58c9, PDF_CMAP_TABLE, 1936 },
	{ 0x58cb, 0x58cb, PDF_CMAP_SINGLE, 14857 },
	{ 0x58ce, 0x58cf, PDF_CMAP_TABLE, 1941 },
	{ 0x58d1, 0x58d6, PDF_CMAP_TABLE, 1943 },
	{ 0x58d8, 0x58dd, PDF_CMAP_TABLE, 1949 },
	{ 0x58de, 0x58df, PDF_CMAP_RANGE, 5496 },
	{ 0x58e0, 0x58e0, PDF_CMAP_SINGLE, 16601 },
	{ 0x58e2, 0x58e4, PDF_CMAP_TABLE, 1955 },
	{ 0x58e7, 0x58e9, PDF_CMAP_TABLE, 1958 },
	{ 0x58eb, 0x58ec, PDF_CMAP_TABLE, 1961 },
	{ 0x58ef, 0x58f0, PDF_CMAP_TABLE, 1963 },
	{ 0x58f2, 0x58f4, PDF_CMAP_TABLE, 1965 },
	{ 0x58f9, 0x58fa, PDF_CMAP_RANGE, 3104 },
	{ 0x58fb, 0x58ff, PDF_CMAP_TABLE, 1968 },
	{ 0x5902, 0x5907, PDF_CMAP_TABLE, 1973 },
	{ 0x590a, 0x590a, PDF_CMAP_SINGLE, 548 },
	{ 0x590c, 0x590f, PDF_CMAP_TABLE, 1979 },
	{ 0x5911, 0x5912, PDF_CMAP_TABLE, 1983 },
	{ 0x5914, 0x5917, PDF_CMAP_TABLE, 1985 },
	{ 0x5919, 0x591a, PDF_CMAP_RANGE, 948 },
	{ 0x591c, 0x591c, PDF_CMAP_SINGLE, 1409 },
	{ 0x591f, 0x5920, PDF_CMAP_TABLE, 1989 },
	{ 0x5922, 0x5922, PDF_CMAP_SINGLE, 4012 },
	{ 0x5924, 0x5925, PDF_CMAP_TABLE, 1991 },
	{ 0x5927, 0x5927, PDF_CMAP_SINGLE, 637 },
	{ 0x5929, 0x592f, PDF_CMAP_TABLE, 1993 },
	{ 0x5931, 0x5932, PDF_CMAP_TABLE, 2000 },
	{ 0x5934, 0x5934, PDF_CMAP_SINGLE, 17741 },
	{ 0x5937, 0x5938, PDF_CMAP_RANGE, 950 },
	{ 0x593c, 0x593c, PDF_CMAP_SINGLE, 6100 },
	{ 0x593e, 0x593e, PDF_CMAP_SINGLE, 1136 },
	{ 0x5940, 0x5940, PDF_CMAP_SINGLE, 6232 },
	{ 0x5944, 0x5945, PDF_CMAP_TABLE, 2002 },
	{ 0x5947, 0x5948, PDF_CMAP_RANGE, 1411 },
	{ 0x5949, 0x594a, PDF_CMAP_TABLE, 2004 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 2006 },
	{ 0x5953, 0x5955, PDF_CMAP_TABLE, 2010 },
	{ 0x5957, 0x5958, PDF_CMAP_RANGE, 2177 },
	{ 0x595a, 0x595a, PDF_CMAP_SINGLE, 2179 },
	{ 0x595c, 0x595c, PDF_CMAP_SINGLE, 7829 },
	{ 0x5960, 0x5962, PDF_CMAP_TABLE, 2013 },
	{ 0x5965, 0x5965, PDF_CMAP_SINGLE, 18514 },
	{ 0x5967, 0x5967, PDF_CMAP_SINGLE, 3571 },
	{ 0x5969, 0x596b, PDF_CMAP_TABLE, 2016 },
	{ 0x596d, 0x596e, PDF_CMAP_TABLE, 2019 },
	{ 0x5970, 0x5979, PDF_CMAP_TABLE, 2021 },
	{ 0x597b, 0x5985, PDF_CMAP_TABLE, 2031 },
	{ 0x5989, 0x598a, PDF_CMAP_TABLE, 2042 },
	{ 0x598d, 0x5990, PDF_CMAP_TABLE, 2044 },
	{ 0x5992, 0x5994, PDF_CMAP_TABLE, 2048 },
	{ 0x5996, 0x599a, PDF_CMAP_TABLE, 2051 },
	{ 0x599d, 0x59a8, PDF_CMAP_TABLE, 2056 },
	{ 0x59ac, 0x59ac, PDF_CMAP_SINGLE, 15745 },
	{ 0x59ae, 0x59be, PDF_CMAP_TABLE, 2068 },
	{ 0x59c0, 0x59c1, PDF_CMAP_TABLE, 2085 },
	{ 0x59c3, 0x59d4, PDF_CMAP_TABLE, 2087 },
	{ 0x59d6, 0x59d6, PDF_CMAP_SINGLE, 6480 },
	{ 0x59d8, 0x59de, PDF_CMAP_TABLE, 2105 },
	{ 0x59e0, 0x59e1, PDF_CMAP_TABLE, 2112 },
	{ 0x59e3, 0x59e6, PDF_CMAP_TABLE, 2114 },
	{ 0x59e8, 0x5a03, PDF_CMAP_TABLE, 2118 },
	{ 0x5a09, 0x5a0d, PDF_CMAP_TABLE, 2146 },
	{ 0x5a0f, 0x5a0f, PDF_CMAP_SINGLE, 7283 },
	{ 0x5a11, 0x5a13, PDF_CMAP_TABLE, 2151 },
	{ 0x5a15, 0x5a19, PDF_CMAP_TABLE, 2154 },
	{ 0x5a1b, 0x5a1c, PDF_CMAP_TABLE, 2159 },
	{ 0x5a1e, 0x5a21, PDF_CMAP_TABLE, 2161 },
	{ 0x5a23, 0x5a25, PDF_CMAP_TABLE, 2165 },
	{ 0x5a27, 0x5a27, PDF_CMAP_SINGLE, 15841 },
	{ 0x5a29, 0x5a2c, PDF_CMAP_TABLE, 2168 },
	{ 0x5a2d, 0x5a2e, PDF_CMAP_RANGE, 7280 },
	{ 0x5a33, 0x5a33, PDF_CMAP_SINGLE, 7287 },
	{ 0x5a35, 0x5a39, PDF_CMAP_TABLE, 2172 },
	{ 0x5a3c, 0x5a3e, PDF_CMAP_TABLE, 2177 },
	{ 0x5a40, 0x5a4a, PDF_CMAP_TABLE, 2180 },
	{ 0x5a4c, 0x5a4d, PDF_CMAP_TABLE, 2191 },
	{ 0x5a50, 0x5a6e, PDF_CMAP_TABLE, 2193 },
	{ 0x5a70, 0x5a71, PDF_CMAP_TABLE, 2224 },
	{ 0x5a77, 0x5a7f, PDF_CMAP_TABLE, 2226 },
	{ 0x5a81, 0x5a84, PDF_CMAP_TABLE, 2235 },
	{ 0x5a86, 0x5a86, PDF_CMAP_SINGLE, 15233 },
	{ 0x5a88, 0x5a88, PDF_CMAP_SINGLE, 16995 },
	{ 0x5a8a, 0x5a8c, PDF_CMAP_TABLE, 2239 },
	{ 0x5a8e, 0x5a97, PDF_CMAP_TABLE, 2242 },
	{ 0x5a99, 0x5aa2, PDF_CMAP_TABLE, 2252 },
	{ 0x5aa5, 0x5aa7, PDF_CMAP_TABLE, 2262 },
	{ 0x5aa9, 0x5aa9, PDF_CMAP_SINGLE, 8451 },
	{ 0x5aab, 0x5aac, PDF_CMAP_TABLE, 2265 },
	{ 0x5aae, 0x5ac4, PDF_CMAP_TABLE, 2267 },
	{ 0x5ac6, 0x5acf, PDF_CMAP_TABLE, 2290 },
	{ 0x5ad3, 0x5ad3, PDF_CMAP_SINGLE, 16642 },
	{ 0x5ad5, 0x5ae6, PDF_CMAP_TABLE, 2300 },
	{ 0x5ae8, 0x5aee, PDF_CMAP_TABLE, 2318 },
	{ 0x5af0, 0x5af0, PDF_CMAP_SINGLE, 16103 },
	{ 0x5af2, 0x5afb, PDF_CMAP_TABLE, 2325 },
	{ 0x5afd, 0x5aff, PDF_CMAP_TABLE, 2335 },
	{ 0x5b01, 0x5b03, PDF_CMAP_TABLE, 2338 },
	{ 0x5b05, 0x5b05, PDF_CMAP_SINGLE, 10421 },
	{ 0x5b07, 0x5b09, PDF_CMAP_TABLE, 2341 },
	{ 0x5b0b, 0x5b0d, PDF_CMAP_TABLE, 2344 },
	{ 0x5b0f, 0x5b11, PDF_CMAP_TABLE, 2347 },
	{ 0x5b13, 0x5b14, PDF_CMAP_TABLE, 2350 },
	{ 0x5b16, 0x5b17, PDF_CMAP_TABLE, 2352 },
	{ 0x5b19, 0x5b1b, PDF_CMAP_TABLE, 2354 },
	{ 0x5b1d, 0x5b21, PDF_CMAP_TABLE, 2357 },
	{ 0x5b23, 0x5b28, PDF_CMAP_TABLE, 2362 },
	{ 0x5b2a, 0x5b30, PDF_CMAP_TABLE, 2368 },
	{ 0x5b32, 0x5b32, PDF_CMAP_SINGLE, 11601 },
	{ 0x5b34, 0x5b34, PDF_CMAP_SINGLE, 4777 },
	{ 0x5b38, 0x5b38, PDF_CMAP_SINGLE, 5326 },
	{ 0x5b3c, 0x5b3c, PDF_CMAP_SINGLE, 12056 },
	{ 0x5b3d, 0x5b3f, PDF_CMAP_RANGE, 12425 },
	{ 0x5b40, 0x5b41, PDF_CMAP_TABLE, 2375 },
	{ 0x5b43, 0x5b48, PDF_CMAP_TABLE, 2377 },
	{ 0x5b4a, 0x5b4a, PDF_CMAP_SINGLE, 14541 },
	{ 0x5b4b, 0x5b4c, PDF_CMAP_RANGE, 13180 },
	{ 0x5b4d, 0x5b4f, PDF_CMAP_TABLE, 2383 },
	{ 0x5b50, 0x5b51, PDF_CMAP_RANGE, 639 },
	{ 0x5b53, 0x5b56, PDF_CMAP_TABLE, 2386 },
	{ 0x5b57, 0x5b58, PDF_CMAP_RANGE, 959 },
	{ 0x5b5a, 0x5b5b, PDF_CMAP_RANGE, 1151 },
	{ 0x5b5c, 0x5b5d, PDF_CMAP_TABLE, 2390 },
	{ 0x5b5f, 0x5b5f, PDF_CMAP_SINGLE, 1431 },
	{ 0x5b62, 0x5b66, PDF_CMAP_TABLE, 2392 },
	{ 0x5b68, 0x5b69, PDF_CMAP_TABLE, 2397 },
	{ 0x5b6b, 0x5b6e, PDF_CMAP_TABLE, 2399 },
	{ 0x5b70, 0x5b78, PDF_CMAP_TABLE, 2403 },
	{ 0x5b7a, 0x5b7d, PDF_CMAP_TABLE, 2412 },
	{ 0x5b7f, 0x5b85, PDF_CMAP_TABLE, 2416 },
	{ 0x5b87, 0x5b88, PDF_CMAP_RANGE, 961 },
	{ 0x5b89, 0x5b89, PDF_CMAP_SINGLE, 964 },
	{ 0x5b8b, 0x5b8c, PDF_CMAP_TABLE, 2423 },
	{ 0x5b8e, 0x5b90, PDF_CMAP_TABLE, 2425 },
	{ 0x5b92, 0x5b93, PDF_CMAP_TABLE, 2428 },
	{ 0x5b95, 0x5b95, PDF_CMAP_SINGLE, 6490 },
	{ 0x5b97, 0x5b9d, PDF_CMAP_TABLE, 2430 },
	{ 0x5b9e, 0x5b9f, PDF_CMAP_RANGE, 17743 },
	{ 0x5ba2, 0x5ba8, PDF_CMAP_TABLE, 2437 },
	{ 0x5baa, 0x5baa, PDF_CMAP_SINGLE, 16763 },
	{ 0x5bac, 0x5bae, PDF_CMAP_TABLE, 2444 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 2195 },
	{ 0x5bb3, 0x5bb6, PDF_CMAP_TABLE, 2447 },
	{ 0x5bb8, 0x5bb9, PDF_CMAP_TABLE, 2451 },
	{ 0x5bbf, 0x5bc7, PDF_CMAP_TABLE, 2453 },
	{ 0x5bca, 0x5bce, PDF_CMAP_TABLE, 2462 },
	{ 0x5bd0, 0x5bd7, PDF_CMAP_TABLE, 2467 },
	{ 0x5bd8, 0x5bd9, PDF_CMAP_RANGE, 9144 },
	{ 0x5bde, 0x5be4, PDF_CMAP_TABLE, 2475 },
	{ 0x5be5, 0x5be6, PDF_CMAP_RANGE, 4027 },
	{ 0x5be7, 0x5bec, PDF_CMAP_TABLE, 2482 },
	{ 0x5bee, 0x5bf0, PDF_CMAP_TABLE, 2488 },
	{ 0x5bf1, 0x5bf2, PDF_CMAP_RANGE, 11609 },
	{ 0x5bf3, 0x5bf3, PDF_CMAP_SINGLE, 16661 },
	{ 0x5bf5, 0x5bf6, PDF_CMAP_TABLE, 2491 },
	{ 0x5bf8, 0x5bf8, PDF_CMAP_SINGLE, 642 },
	{ 0x5bfa, 0x5bfa, PDF_CMAP_SINGLE, 965 },
	{ 0x5bff, 0x5bff, PDF_CMAP_SINGLE, 16605 },
	{ 0x5c01, 0x5c01, PDF_CMAP_SINGLE, 1798 },
	{ 0x5c03, 0x5c05, PDF_CMAP_TABLE, 2493 },
	{ 0x5c07, 0x5c09, PDF_CMAP_TABLE, 2496 },
	{ 0x5c0a, 0x5c0b, PDF_CMAP_RANGE, 3119 },
	{ 0x5c0c, 0x5c16, PDF_CMAP_TABLE, 2499 },
	{ 0x5c1a, 0x5c1a, PDF_CMAP_SINGLE, 1440 },
	{ 0x5c1c, 0x5c1c, PDF_CMAP_SINGLE, 17898 },
	{ 0x5c1e, 0x5c20, PDF_CMAP_TABLE, 2510 },
	{ 0x5c22, 0x5c25, PDF_CMAP_TABLE, 2513 },
	{ 0x5c28, 0x5c28, PDF_CMAP_SINGLE, 6245 },
	{ 0x5c2a, 0x5c2a, PDF_CMAP_SINGLE, 6246 },
	{ 0x5c2c, 0x5c2c, PDF_CMAP_SINGLE, 1156 },
	{ 0x5c30, 0x5c31, PDF_CMAP_TABLE, 2517 },
	{ 0x5c33, 0x5c33, PDF_CMAP_SINGLE, 9147 },
	{ 0x5c37, 0x5c3c, PDF_CMAP_TABLE, 2519 },
	{ 0x5c3e, 0x5c3f, PDF_CMAP_TABLE, 2525 },
	{ 0x5c40, 0x5c41, PDF_CMAP_RANGE, 1157 },
	{ 0x5c44, 0x5c44, PDF_CMAP_SINGLE, 6491 },
	{ 0x5c45, 0x5c46, PDF_CMAP_RANGE, 1442 },
	{ 0x5c47, 0x5c4d, PDF_CMAP_TABLE, 2527 },
	{ 0x5c4e, 0x5c4f, PDF_CMAP_RANGE, 1799 },
	{ 0x5c50, 0x5c51, PDF_CMAP_TABLE, 2534 },
	{ 0x5c53, 0x5c56, PDF_CMAP_TABLE, 2536 },
	{ 0x5c58, 0x5c59, PDF_CMAP_TABLE, 2540 },
	{ 0x5c5c, 0x5c5d, PDF_CMAP_RANGE, 2652 },
	{ 0x5c5e, 0x5c5e, PDF_CMAP_SINGLE, 15719 },
	{ 0x5c60, 0x5c60, PDF_CMAP_SINGLE, 2651 },
	{ 0x5c62, 0x5c63, PDF_CMAP_TABLE, 2542 },
	{ 0x5c64, 0x5c65, PDF_CMAP_RANGE, 4404 },
	{ 0x5c67, 0x5c68, PDF_CMAP_TABLE, 2544 },
	{ 0x5c69, 0x5c6a, PDF_CMAP_RANGE, 12057 },
	{ 0x5c6c, 0x5c6f, PDF_CMAP_TABLE, 2546 },
	{ 0x5c71, 0x5c71, PDF_CMAP_SINGLE, 646 },
	{ 0x5c73, 0x5c74, PDF_CMAP_TABLE, 2550 },
	{ 0x5c79, 0x5c79, PDF_CMAP_SINGLE, 967 },
	{ 0x5c7a, 0x5c7b, PDF_CMAP_RANGE, 6112 },
	{ 0x5c7c, 0x5c7c, PDF_CMAP_SINGLE, 6111 },
	{ 0x5c7e, 0x5c7e, PDF_CMAP_SINGLE, 6114 },
	{ 0x5c85, 0x5c86, PDF_CMAP_TABLE, 2552 },
	{ 0x5c88, 0x5c8d, PDF_CMAP_TABLE, 2554 },
	{ 0x5c8f, 0x5c8f, PDF_CMAP_SINGLE, 6248 },
	{ 0x5c90, 0x5c91, PDF_CMAP_RANGE, 1161 },
	{ 0x5c92, 0x5c95, PDF_CMAP_TABLE, 2560 },
	{ 0x5c99, 0x5c9a, PDF_CMAP_TABLE, 2564 },
	{ 0x5c9c, 0x5cb1, PDF_CMAP_TABLE, 2566 },
	{ 0x5cb3, 0x5cb3, PDF_CMAP_SINGLE, 1450 },
	{ 0x5cb5, 0x5cb8, PDF_CMAP_TABLE, 2588 },
	{ 0x5cba, 0x5cba, PDF_CMAP_SINGLE, 16683 },
	{ 0x5cc1, 0x5cc2, PDF_CMAP_TABLE, 2592 },
	{ 0x5cc6, 0x5ccc, PDF_CMAP_TABLE, 2594 },
	{ 0x5cce, 0x5cd2, PDF_CMAP_TABLE, 2601 },
	{ 0x5cd3, 0x5cd4, PDF_CMAP_RANGE, 6849 },
	{ 0x5cd6, 0x5cdb, PDF_CMAP_TABLE, 2606 },
	{ 0x5cde, 0x5cdf, PDF_CMAP_TABLE, 2612 },
	{ 0x5ce5, 0x5ce5, PDF_CMAP_SINGLE, 17975 },
	{ 0x5ce8, 0x5cea, PDF_CMAP_TABLE, 2614 },
	{ 0x5cec, 0x5cf1, PDF_CMAP_TABLE, 2617 },
	{ 0x5cf4, 0x5cf4, PDF_CMAP_SINGLE, 2215 },
	{ 0x5cf6, 0x5cf9, PDF_CMAP_TABLE, 2623 },
	{ 0x5cfb, 0x5cfb, PDF_CMAP_SINGLE, 2209 },
	{ 0x5cfd, 0x5cfd, PDF_CMAP_SINGLE, 2208 },
	{ 0x5cff, 0x5d01, PDF_CMAP_TABLE, 2627 },
	{ 0x5d06, 0x5d07, PDF_CMAP_TABLE, 2630 },
	{ 0x5d0b, 0x5d12, PDF_CMAP_TABLE, 2632 },
	{ 0x5d14, 0x5d1b, PDF_CMAP_TABLE, 2640 },
	{ 0x5d1d, 0x5d20, PDF_CMAP_TABLE, 2648 },
	{ 0x5d22, 0x5d29, PDF_CMAP_TABLE, 2652 },
	{ 0x5d2c, 0x5d2c, PDF_CMAP_SINGLE, 15863 },
	{ 0x5d2e, 0x5d3a, PDF_CMAP_TABLE, 2660 },
	{ 0x5d3c, 0x5d43, PDF_CMAP_TABLE, 2673 },
	{ 0x5d45, 0x5d4c, PDF_CMAP_TABLE, 2681 },
	{ 0x5d4e, 0x5d4e, PDF_CMAP_SINGLE, 8476 },
	{ 0x5d50, 0x5d52, PDF_CMAP_TABLE, 2689 },
	{ 0x5d55, 0x5d57, PDF_CMAP_TABLE, 2692 },
	{ 0x5d59, 0x5d59, PDF_CMAP_SINGLE, 8483 },
	{ 0x5d5b, 0x5d5b, PDF_CMAP_SINGLE, 15829 },
	{ 0x5d5e, 0x5d5e, PDF_CMAP_SINGLE, 9154 },
	{ 0x5d62, 0x5d63, PDF_CMAP_TABLE, 2695 },
	{ 0x5d65, 0x5d65, PDF_CMAP_SINGLE, 9151 },
	{ 0x5d67, 0x5d69, PDF_CMAP_TABLE, 2697 },
	{ 0x5d6b, 0x5d6c, PDF_CMAP_TABLE, 2700 },
	{ 0x5d6f, 0x5d72, PDF_CMAP_TABLE, 2702 },
	{ 0x5d74, 0x5d74, PDF_CMAP_SINGLE, 16111 },
	{ 0x5d77, 0x5d77, PDF_CMAP_SINGLE, 9833 },
	{ 0x5d79, 0x5d7a, PDF_CMAP_TABLE, 2706 },
	{ 0x5d7c, 0x5d82, PDF_CMAP_TABLE, 2708 },
	{ 0x5d84, 0x5d8b, PDF_CMAP_TABLE, 2715 },
	{ 0x5d8d, 0x5d8e, PDF_CMAP_TABLE, 2723 },
	{ 0x5d92, 0x5d95, PDF_CMAP_TABLE, 2725 },
	{ 0x5d97, 0x5d97, PDF_CMAP_SINGLE, 10425 },
	{ 0x5d99, 0x5d9a, PDF_CMAP_TABLE, 2729 },
	{ 0x5d9c, 0x5da2, PDF_CMAP_TABLE, 2731 },
	{ 0x5da4, 0x5da4, PDF_CMAP_SINGLE, 17965 },
	{ 0x5da7, 0x5db2, PDF_CMAP_TABLE, 2738 },
	{ 0x5db4, 0x5dba, PDF_CMAP_TABLE, 2750 },
	{ 0x5dbc, 0x5dbd, PDF_CMAP_TABLE, 2757 },
	{ 0x5dc0, 0x5dc3, PDF_CMAP_TABLE, 2759 },
	{ 0x5dc6, 0x5dc7, PDF_CMAP_RANGE, 12749 },
	{ 0x5dc9, 0x5dc9, PDF_CMAP_SINGLE, 5651 },
	{ 0x5dcb, 0x5dcb, PDF_CMAP_SINGLE, 13006 },
	{ 0x5dcd, 0x5dcd, PDF_CMAP_SINGLE, 5743 },
	{ 0x5dcf, 0x5dcf, PDF_CMAP_SINGLE, 13007 },
	{ 0x5dd1, 0x5dd2, PDF_CMAP_TABLE, 2763 },
	{ 0x5dd4, 0x5dd8, PDF_CMAP_TABLE, 2765 },
	{ 0x5ddb, 0x5ddb, PDF_CMAP_SINGLE, 550 },
	{ 0x5ddd, 0x5de2, PDF_CMAP_TABLE, 2770 },
	{ 0x5de5, 0x5de8, PDF_CMAP_TABLE, 2776 },
	{ 0x5deb, 0x5deb, PDF_CMAP_SINGLE, 1165 },
	{ 0x5dee, 0x5dee, PDF_CMAP_SINGLE, 2216 },
	{ 0x5df0, 0x5df0, PDF_CMAP_SINGLE, 9158 },
	{ 0x5df1, 0x5df3, PDF_CMAP_RANGE, 649 },
	{ 0x5df4, 0x5df5, PDF_CMAP_TABLE, 2780 },
	{ 0x5df7, 0x5df7, PDF_CMAP_SINGLE, 1805 },
	{ 0x5df9, 0x5df9, PDF_CMAP_SINGLE, 6857 },
	{ 0x5dfd, 0x5dff, PDF_CMAP_TABLE, 2782 },
	{ 0x5e02, 0x5e03, PDF_CMAP_RANGE, 825 },
	{ 0x5e04, 0x5e04, PDF_CMAP_SINGLE, 6052 },
	{ 0x5e06, 0x5e06, PDF_CMAP_SINGLE, 969 },
	{ 0x5e09, 0x5e0c, PDF_CMAP_TABLE, 2785 },
	{ 0x5e0e, 0x5e0e, PDF_CMAP_SINGLE, 6259 },
	{ 0x5e11, 0x5e12, PDF_CMAP_TABLE, 2789 },
	{ 0x5e14, 0x5e1b, PDF_CMAP_TABLE, 2791 },
	{ 0x5e1d, 0x5e1d, PDF_CMAP_SINGLE, 1806 },
	{ 0x5e1f, 0x5e20, PDF_CMAP_TABLE, 2799 },
	{ 0x5e21, 0x5e23, PDF_CMAP_RANGE, 6858 },
	{ 0x5e24, 0x5e25, PDF_CMAP_TABLE, 2801 },
	{ 0x5e28, 0x5e29, PDF_CMAP_TABLE, 2803 },
	{ 0x5e2b, 0x5e2b, PDF_CMAP_SINGLE, 2218 },
	{ 0x5e2d, 0x5e2e, PDF_CMAP_TABLE, 2805 },
	{ 0x5e33, 0x5e34, PDF_CMAP_TABLE, 2807 },
	{ 0x5e36, 0x5e38, PDF_CMAP_TABLE, 2809 },
	{ 0x5e3d, 0x5e3e, PDF_CMAP_TABLE, 2812 },
	{ 0x5e40, 0x5e45, PDF_CMAP_TABLE, 2814 },
	{ 0x5e48, 0x5e48, PDF_CMAP_SINGLE, 14892 },
	{ 0x5e4a, 0x5e4f, PDF_CMAP_TABLE, 2820 },
	{ 0x5e53, 0x5e55, PDF_CMAP_TABLE, 2826 },
	{ 0x5e57, 0x5e57, PDF_CMAP_SINGLE, 4040 },
	{ 0x5e58, 0x5e59, PDF_CMAP_RANGE, 9842 },
	{ 0x5e5b, 0x5e63, PDF_CMAP_TABLE, 2829 },
	{ 0x5e66, 0x5e66, PDF_CMAP_SINGLE, 11064 },
	{ 0x5e67, 0x5e68, PDF_CMAP_RANGE, 11062 },
	{ 0x5e69, 0x5e6c, PDF_CMAP_TABLE, 2838 },
	{ 0x5e6d, 0x5e6e, PDF_CMAP_RANGE, 12060 },
	{ 0x5e6f, 0x5e70, PDF_CMAP_TABLE, 2842 },
	{ 0x5e72, 0x5e76, PDF_CMAP_TABLE, 2844 },
	{ 0x5e78, 0x5e80, PDF_CMAP_TABLE, 2849 },
	{ 0x5e82, 0x5e84, PDF_CMAP_TABLE, 2858 },
	{ 0x5e86, 0x5e8d, PDF_CMAP_TABLE, 2861 },
	{ 0x5e8f, 0x5e8f, PDF_CMAP_SINGLE, 1167 },
	{ 0x5e95, 0x5e96, PDF_CMAP_RANGE, 1461 },
	{ 0x5e97, 0x5e97, PDF_CMAP_SINGLE, 1459 },
	{ 0x5e9a, 0x5e9c, PDF_CMAP_TABLE, 2869 },
	{ 0x5ea0, 0x5ea0, PDF_CMAP_SINGLE, 1810 },
	{ 0x5ea2, 0x5ea8, PDF_CMAP_TABLE, 2872 },
	{ 0x5eaa, 0x5eae, PDF_CMAP_TABLE, 2879 },
	{ 0x5eb0, 0x5eb1, PDF_CMAP_TABLE, 2884 },
	{ 0x5eb2, 0x5eb3, PDF_CMAP_RANGE, 7889 },
	{ 0x5eb4, 0x5eb6, PDF_CMAP_TABLE, 2886 },
	{ 0x5eb7, 0x5eb8, PDF_CMAP_RANGE, 2672 },
	{ 0x5eb9, 0x5eb9, PDF_CMAP_SINGLE, 7888 },
	{ 0x5ebd, 0x5ebe, PDF_CMAP_TABLE, 2889 },
	{ 0x5ec1, 0x5ec2, PDF_CMAP_RANGE, 3133 },
	{ 0x5ec4, 0x5ece, PDF_CMAP_TABLE, 2891 },
	{ 0x5ed0, 0x5ee3, PDF_CMAP_TABLE, 2902 },
	{ 0x5ee5, 0x5ee9, PDF_CMAP_TABLE, 2922 },
	{ 0x5eec, 0x5eec, PDF_CMAP_SINGLE, 5501 },
	{ 0x5eee, 0x5eef, PDF_CMAP_RANGE, 12751 },
	{ 0x5ef1, 0x5ef4, PDF_CMAP_TABLE, 2927 },
	{ 0x5ef6, 0x5efc, PDF_CMAP_TABLE, 2931 },
	{ 0x5efe, 0x5eff, PDF_CMAP_TABLE, 2938 },
	{ 0x5f01, 0x5f02, PDF_CMAP_TABLE, 2940 },
	{ 0x5f04, 0x5f05, PDF_CMAP_TABLE, 2942 },
	{ 0x5f07, 0x5f08, PDF_CMAP_TABLE, 2944 },
	{ 0x5f0a, 0x5f0f, PDF_CMAP_TABLE, 2946 },
	{ 0x5f12, 0x5f13, PDF_CMAP_TABLE, 2952 },
	{ 0x5f14, 0x5f15, PDF_CMAP_RANGE, 718 },
	{ 0x5f17, 0x5f18, PDF_CMAP_TABLE, 2954 },
	{ 0x5f1a, 0x5f1b, PDF_CMAP_TABLE, 2956 },
	{ 0x5f1d, 0x5f1d, PDF_CMAP_SINGLE, 6266 },
	{ 0x5f1f, 0x5f1f, PDF_CMAP_SINGLE, 1172 },
	{ 0x5f22, 0x5f24, PDF_CMAP_RANGE, 6515 },
	{ 0x5f25, 0x5f25, PDF_CMAP_SINGLE, 17968 },
	{ 0x5f26, 0x5f27, PDF_CMAP_RANGE, 1464 },
	{ 0x5f28, 0x5f29, PDF_CMAP_TABLE, 2958 },
	{ 0x5f2d, 0x5f2e, PDF_CMAP_TABLE, 2960 },
	{ 0x5f30, 0x5f31, PDF_CMAP_TABLE, 2962 },
	{ 0x5f33, 0x5f33, PDF_CMAP_SINGLE, 7308 },
	{ 0x5f35, 0x5f38, PDF_CMAP_TABLE, 2964 },
	{ 0x5f3a, 0x5f3a, PDF_CMAP_SINGLE, 16718 },
	{ 0x5f3c, 0x5f3c, PDF_CMAP_SINGLE, 3136 },
	{ 0x5f40, 0x5f40, PDF_CMAP_SINGLE, 9169 },
	{ 0x5f43, 0x5f44, PDF_CMAP_TABLE, 2968 },
	{ 0x5f46, 0x5f46, PDF_CMAP_SINGLE, 4045 },
	{ 0x5f48, 0x5f51, PDF_CMAP_TABLE, 2970 },
	{ 0x5f54, 0x5f54, PDF_CMAP_SINGLE, 6518 },
	{ 0x5f56, 0x5f59, PDF_CMAP_TABLE, 2980 },
	{ 0x5f5c, 0x5f5d, PDF_CMAP_TABLE, 2984 },
	{ 0x5f61, 0x5f65, PDF_CMAP_TABLE, 2986 },
	{ 0x5f67, 0x5f67, PDF_CMAP_SINGLE, 7310 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 2991 },
	{ 0x5f6f, 0x5f74, PDF_CMAP_TABLE, 2996 },
	{ 0x5f76, 0x5f79, PDF_CMAP_TABLE, 3002 },
	{ 0x5f7b, 0x5f7f, PDF_CMAP_TABLE, 3006 },
	{ 0x5f80, 0x5f81, PDF_CMAP_RANGE, 1467 },
	{ 0x5f82, 0x5f83, PDF_CMAP_TABLE, 3011 },
	{ 0x5f85, 0x5f89, PDF_CMAP_TABLE, 3013 },
	{ 0x5f8a, 0x5f8b, PDF_CMAP_RANGE, 1818 },
	{ 0x5f8c, 0x5f8c, PDF_CMAP_SINGLE, 1821 },
	{ 0x5f90, 0x5f92, PDF_CMAP_TABLE, 3018 },
	{ 0x5f96, 0x5f99, PDF_CMAP_TABLE, 3021 },
	{ 0x5f9b, 0x5f9c, PDF_CMAP_TABLE, 3025 },
	{ 0x5f9e, 0x5fa1, PDF_CMAP_TABLE, 3027 },
	{ 0x5fa4, 0x5fa8, PDF_CMAP_TABLE, 3031 },
	{ 0x5fa9, 0x5faa, PDF_CMAP_RANGE, 3138 },
	{ 0x5fab, 0x5faf, PDF_CMAP_TABLE, 3036 },
	{ 0x5fb1, 0x5fb2, PDF_CMAP_TABLE, 3041 },
	{ 0x5fb5, 0x5fb7, PDF_CMAP_TABLE, 3043 },
	{ 0x5fb9, 0x5fbf, PDF_CMAP_TABLE, 3046 },
	{ 0x5fc0, 0x5fc1, PDF_CMAP_RANGE, 12753 },
	{ 0x5fc3, 0x5fc5, PDF_CMAP_TABLE, 3053 },
	{ 0x5fc9, 0x5fc9, PDF_CMAP_SINGLE, 6055 },
	{ 0x5fcc, 0x5fcd, PDF_CMAP_TABLE, 3056 },
	{ 0x5fcf, 0x5fd2, PDF_CMAP_TABLE, 3058 },
	{ 0x5fd4, 0x5fd9, PDF_CMAP_TABLE, 3062 },
	{ 0x5fdb, 0x5fdb, PDF_CMAP_SINGLE, 14922 },
	{ 0x5fdd, 0x5fe1, PDF_CMAP_TABLE, 3068 },
	{ 0x5fe3, 0x5fe5, PDF_CMAP_TABLE, 3073 },
	{ 0x5fe8, 0x5fe8, PDF_CMAP_SINGLE, 6273 },
	{ 0x5fea, 0x5feb, PDF_CMAP_TABLE, 3076 },
	{ 0x5fed, 0x5fef, PDF_CMAP_TABLE, 3078 },
	{ 0x5ff1, 0x5ff1, PDF_CMAP_SINGLE, 1181 },
	{ 0x5ff3, 0x5ff5, PDF_CMAP_TABLE, 3081 },
	{ 0x5ff7, 0x5ff8, PDF_CMAP_TABLE, 3084 },
	{ 0x5ffa, 0x5ffb, PDF_CMAP_TABLE, 3086 },
	{ 0x5ffd, 0x5ffd, PDF_CMAP_SINGLE, 1473 },
	{ 0x5fff, 0x6000, PDF_CMAP_TABLE, 3088 },
	{ 0x6009, 0x6017, PDF_CMAP_TABLE, 3090 },
	{ 0x6019, 0x601e, PDF_CMAP_TABLE, 3105 },
	{ 0x6020, 0x602f, PDF_CMAP_TABLE, 3111 },
	{ 0x6031, 0x6035, PDF_CMAP_TABLE, 3127 },
	{ 0x6037, 0x6037, PDF_CMAP_SINGLE, 6873 },
	{ 0x6039, 0x6039, PDF_CMAP_SINGLE, 6874 },
	{ 0x603b, 0x603b, PDF_CMAP_SINGLE, 17747 },
	{ 0x6040, 0x6047, PDF_CMAP_TABLE, 3132 },
	{ 0x6049, 0x604a, PDF_CMAP_TABLE, 3140 },
	{ 0x604c, 0x604d, PDF_CMAP_TABLE, 3142 },
	{ 0x6050, 0x6050, PDF_CMAP_SINGLE, 2229 },
	{ 0x6052, 0x6055, PDF_CMAP_TABLE, 3144 },
	{ 0x6058, 0x605b, PDF_CMAP_TABLE, 3148 },
	{ 0x605d, 0x605f, PDF_CMAP_TABLE, 3152 },
	{ 0x6062, 0x6070, PDF_CMAP_TABLE, 3155 },
	{ 0x6072, 0x6072, PDF_CMAP_SINGLE, 6876 },
	{ 0x6075, 0x6075, PDF_CMAP_SINGLE, 14934 },
	{ 0x6077, 0x6077, PDF_CMAP_SINGLE, 16389 },
	{ 0x607e, 0x6081, PDF_CMAP_TABLE, 3170 },
	{ 0x6083, 0x608a, PDF_CMAP_TABLE, 3174 },
	{ 0x608c, 0x608e, PDF_CMAP_TABLE, 3182 },
	{ 0x6090, 0x6090, PDF_CMAP_SINGLE, 7897 },
	{ 0x6092, 0x6092, PDF_CMAP_SINGLE, 7318 },
	{ 0x6094, 0x6097, PDF_CMAP_TABLE, 3185 },
	{ 0x609a, 0x60a0, PDF_CMAP_TABLE, 3189 },
	{ 0x60a2, 0x60a4, PDF_CMAP_TABLE, 3196 },
	{ 0x60a7, 0x60a8, PDF_CMAP_TABLE, 3199 },
	{ 0x60b0, 0x60c1, PDF_CMAP_TABLE, 3201 },
	{ 0x60c3, 0x60cf, PDF_CMAP_TABLE, 3219 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 3141 },
	{ 0x60d3, 0x60d4, PDF_CMAP_RANGE, 7902 },
	{ 0x60d5, 0x60d5, PDF_CMAP_SINGLE, 2705 },
	{ 0x60d7, 0x60e4, PDF_CMAP_TABLE, 3232 },
	{ 0x60e6, 0x60e9, PDF_CMAP_TABLE, 3246 },
	{ 0x60f0, 0x6101, PDF_CMAP_TABLE, 3250 },
	{ 0x6103, 0x6110, PDF_CMAP_TABLE, 3268 },
	{ 0x6112, 0x6116, PDF_CMAP_TABLE, 3282 },
	{ 0x6118, 0x611d, PDF_CMAP_TABLE, 3287 },
	{ 0x611f, 0x611f, PDF_CMAP_SINGLE, 3594 },
	{ 0x6123, 0x6123, PDF_CMAP_SINGLE, 3147 },
	{ 0x6127, 0x6129, PDF_CMAP_TABLE, 3293 },
	{ 0x612b, 0x612c, PDF_CMAP_TABLE, 3296 },
	{ 0x612e, 0x6130, PDF_CMAP_TABLE, 3298 },
	{ 0x6132, 0x6132, PDF_CMAP_SINGLE, 9178 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 3605 },
	{ 0x6136, 0x6137, PDF_CMAP_TABLE, 3301 },
	{ 0x613b, 0x613b, PDF_CMAP_SINGLE, 9874 },
	{ 0x613d, 0x6142, PDF_CMAP_TABLE, 3303 },
	{ 0x6144, 0x6148, PDF_CMAP_TABLE, 3309 },
	{ 0x6149, 0x614a, PDF_CMAP_RANGE, 9173 },
	{ 0x614b, 0x6150, PDF_CMAP_TABLE, 3314 },
	{ 0x6152, 0x6153, PDF_CMAP_RANGE, 9864 },
	{ 0x6154, 0x6156, PDF_CMAP_TABLE, 3320 },
	{ 0x6158, 0x6161, PDF_CMAP_TABLE, 3323 },
	{ 0x6162, 0x6163, PDF_CMAP_RANGE, 4052 },
	{ 0x6164, 0x6168, PDF_CMAP_TABLE, 3333 },
	{ 0x616a, 0x616c, PDF_CMAP_TABLE, 3338 },
	{ 0x616e, 0x6177, PDF_CMAP_TABLE, 3341 },
	{ 0x6179, 0x617a, PDF_CMAP_TABLE, 3351 },
	{ 0x617c, 0x617e, PDF_CMAP_TABLE, 3353 },
	{ 0x6180, 0x6183, PDF_CMAP_TABLE, 3356 },
	{ 0x6187, 0x6187, PDF_CMAP_SINGLE, 16762 },
	{ 0x6189, 0x618e, PDF_CMAP_TABLE, 3360 },
	{ 0x6190, 0x6196, PDF_CMAP_TABLE, 3366 },
	{ 0x6198, 0x619d, PDF_CMAP_TABLE, 3373 },
	{ 0x619f, 0x619f, PDF_CMAP_SINGLE, 10457 },
	{ 0x61a1, 0x61a2, PDF_CMAP_TABLE, 3379 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 4437 },
	{ 0x61a7, 0x61ba, PDF_CMAP_TABLE, 3381 },
	{ 0x61bc, 0x61bc, PDF_CMAP_SINGLE, 11617 },
	{ 0x61be, 0x61c3, PDF_CMAP_TABLE, 3401 },
	{ 0x61c5, 0x61cd, PDF_CMAP_TABLE, 3407 },
	{ 0x61cf, 0x61d0, PDF_CMAP_TABLE, 3416 },
	{ 0x61d3, 0x61d3, PDF_CMAP_SINGLE, 16766 },
	{ 0x61d6, 0x61d6, PDF_CMAP_SINGLE, 12070 },
	{ 0x61d8, 0x61d8, PDF_CMAP_SINGLE, 12062 },
	{ 0x61da, 0x61da, PDF_CMAP_SINGLE, 14513 },
	{ 0x61de, 0x61e0, PDF_CMAP_TABLE, 3418 },
	{ 0x61e2, 0x61eb, PDF_CMAP_TABLE, 3421 },
	{ 0x61ed, 0x61ee, PDF_CMAP_RANGE, 12064 },
	{ 0x61f0, 0x61f2, PDF_CMAP_TABLE, 3431 },
	{ 0x61f5, 0x6201, PDF_CMAP_TABLE, 3434 },
	{ 0x6203, 0x6204, PDF_CMAP_RANGE, 13336 },
	{ 0x6207, 0x620a, PDF_CMAP_TABLE, 3447 },
	{ 0x620c, 0x620d, PDF_CMAP_RANGE, 977 },
	{ 0x620e, 0x620e, PDF_CMAP_SINGLE, 976 },
	{ 0x6210, 0x6212, PDF_CMAP_TABLE, 3451 },
	{ 0x6214, 0x6216, PDF_CMAP_TABLE, 3454 },
	{ 0x6219, 0x6219, PDF_CMAP_SINGLE, 7328 },
	{ 0x621a, 0x621b, PDF_CMAP_RANGE, 2711 },
	{ 0x621f, 0x6220, PDF_CMAP_TABLE, 3457 },
	{ 0x6221, 0x6222, PDF_CMAP_RANGE, 3610 },
	{ 0x6223, 0x6225, PDF_CMAP_TABLE, 3459 },
	{ 0x6227, 0x6227, PDF_CMAP_SINGLE, 9879 },
	{ 0x6229, 0x622e, PDF_CMAP_TABLE, 3462 },
	{ 0x6230, 0x6230, PDF_CMAP_SINGLE, 4791 },
	{ 0x6232, 0x6234, PDF_CMAP_TABLE, 3468 },
	{ 0x6236, 0x6237, PDF_CMAP_TABLE, 3471 },
	{ 0x6239, 0x623a, PDF_CMAP_TABLE, 3473 },
	{ 0x623d, 0x6241, PDF_CMAP_TABLE, 3475 },
	{ 0x6242, 0x6243, PDF_CMAP_RANGE, 6892 },
	{ 0x6246, 0x624e, PDF_CMAP_TABLE, 3480 },
	{ 0x6250, 0x6252, PDF_CMAP_TABLE, 3489 },
	{ 0x6253, 0x6254, PDF_CMAP_RANGE, 834 },
	{ 0x6258, 0x625c, PDF_CMAP_TABLE, 3492 },
	{ 0x625e, 0x625e, PDF_CMAP_SINGLE, 6125 },
	{ 0x6260, 0x6266, PDF_CMAP_TABLE, 3497 },
	{ 0x6268, 0x6268, PDF_CMAP_SINGLE, 16149 },
	{ 0x626d, 0x6274, PDF_CMAP_TABLE, 3504 },
	{ 0x6276, 0x6277, PDF_CMAP_TABLE, 3512 },
	{ 0x6279, 0x628a, PDF_CMAP_TABLE, 3514 },
	{ 0x628c, 0x628c, PDF_CMAP_SINGLE, 6287 },
	{ 0x628e, 0x628f, PDF_CMAP_RANGE, 6288 },
	{ 0x6290, 0x6298, PDF_CMAP_TABLE, 3532 },
	{ 0x629d, 0x629d, PDF_CMAP_SINGLE, 16022 },
	{ 0x62a4, 0x62a4, PDF_CMAP_SINGLE, 14953 },
	{ 0x62a6, 0x62a6, PDF_CMAP_SINGLE, 16321 },
	{ 0x62a8, 0x62b1, PDF_CMAP_TABLE, 3541 },
	{ 0x62b3, 0x62b6, PDF_CMAP_TABLE, 3551 },
	{ 0x62b8, 0x62b9, PDF_CMAP_TABLE, 3555 },
	{ 0x62bb, 0x62bf, PDF_CMAP_TABLE, 3557 },
	{ 0x62c2, 0x62d2, PDF_CMAP_TABLE, 3562 },
	{ 0x62d3, 0x62d4, PDF_CMAP_RANGE, 1503 },
	{ 0x62d5, 0x62d5, PDF_CMAP_SINGLE, 16780 },
	{ 0x62d6, 0x62d7, PDF_CMAP_RANGE, 1518 },
	{ 0x62d8, 0x62dc, PDF_CMAP_TABLE, 3579 },
	{ 0x62df, 0x62df, PDF_CMAP_SINGLE, 16412 },
	{ 0x62e5, 0x62e5, PDF_CMAP_SINGLE, 17981 },
	{ 0x62eb, 0x6303, PDF_CMAP_TABLE, 3584 },
	{ 0x6307, 0x6309, PDF_CMAP_TABLE, 3609 },
	{ 0x630b, 0x6311, PDF_CMAP_TABLE, 3612 },
	{ 0x6313, 0x6314, PDF_CMAP_RANGE, 6907 },
	{ 0x6315, 0x6316, PDF_CMAP_TABLE, 3619 },
	{ 0x6318, 0x6318, PDF_CMAP_SINGLE, 17345 },
	{ 0x6328, 0x6329, PDF_CMAP_TABLE, 3621 },
	{ 0x632a, 0x632b, PDF_CMAP_RANGE, 2257 },
	{ 0x632c, 0x632f, PDF_CMAP_TABLE, 3623 },
	{ 0x6331, 0x633e, PDF_CMAP_TABLE, 3627 },
	{ 0x6340, 0x6343, PDF_CMAP_TABLE, 3641 },
	{ 0x6344, 0x6345, PDF_CMAP_RANGE, 7334 },
	{ 0x6346, 0x6351, PDF_CMAP_TABLE, 3645 },
	{ 0x6354, 0x635a, PDF_CMAP_TABLE, 3657 },
	{ 0x6364, 0x6365, PDF_CMAP_TABLE, 3664 },
	{ 0x6367, 0x6369, PDF_CMAP_TABLE, 3666 },
	{ 0x636b, 0x6372, PDF_CMAP_TABLE, 3669 },
	{ 0x6375, 0x637d, PDF_CMAP_TABLE, 3677 },
	{ 0x637f, 0x6385, PDF_CMAP_TABLE, 3686 },
	{ 0x6387, 0x6392, PDF_CMAP_TABLE, 3693 },
	{ 0x6394, 0x6394, PDF_CMAP_SINGLE, 8524 },
	{ 0x6396, 0x6399, PDF_CMAP_TABLE, 3705 },
	{ 0x639b, 0x63a5, PDF_CMAP_TABLE, 3709 },
	{ 0x63a7, 0x63b1, PDF_CMAP_TABLE, 3720 },
	{ 0x63b9, 0x63b9, PDF_CMAP_SINGLE, 16029 },
	{ 0x63bd, 0x63be, PDF_CMAP_TABLE, 3731 },
	{ 0x63c0, 0x63d3, PDF_CMAP_TABLE, 3733 },
	{ 0x63d5, 0x63eb, PDF_CMAP_TABLE, 3753 },
	{ 0x63ed, 0x63ee, PDF_CMAP_RANGE, 3175 },
	{ 0x63ef, 0x63f6, PDF_CMAP_TABLE, 3776 },
	{ 0x63f8, 0x63f9, PDF_CMAP_TABLE, 3784 },
	{ 0x63fb, 0x63fc, PDF_CMAP_TABLE, 3786 },
	{ 0x63fe, 0x63fe, PDF_CMAP_SINGLE, 16030 },
	{ 0x6406, 0x6407, PDF_CMAP_TABLE, 3788 },
	{ 0x6409, 0x6410, PDF_CMAP_TABLE, 3790 },
	{ 0x6412, 0x6415, PDF_CMAP_TABLE, 3798 },
	{ 0x6416, 0x6417, PDF_CMAP_RANGE, 3624 },
	{ 0x6418, 0x6418, PDF_CMAP_SINGLE, 9202 },
	{ 0x641a, 0x641c, PDF_CMAP_TABLE, 3802 },
	{ 0x641e, 0x6421, PDF_CMAP_TABLE, 3805 },
	{ 0x6422, 0x6423, PDF_CMAP_RANGE, 9205 },
	{ 0x6424, 0x6428, PDF_CMAP_TABLE, 3809 },
	{ 0x642a, 0x6430, PDF_CMAP_TABLE, 3814 },
	{ 0x6432, 0x643b, PDF_CMAP_TABLE, 3821 },
	{ 0x643d, 0x6441, PDF_CMAP_TABLE, 3831 },
	{ 0x6443, 0x6443, PDF_CMAP_SINGLE, 9199 },
	{ 0x644b, 0x644b, PDF_CMAP_SINGLE, 9897 },
	{ 0x644d, 0x644e, PDF_CMAP_TABLE, 3836 },
	{ 0x6450, 0x6454, PDF_CMAP_TABLE, 3838 },
	{ 0x6458, 0x6461, PDF_CMAP_TABLE, 3843 },
	{ 0x6465, 0x6469, PDF_CMAP_TABLE, 3853 },
	{ 0x646b, 0x6471, PDF_CMAP_TABLE, 3858 },
	{ 0x6472, 0x6473, PDF_CMAP_RANGE, 9887 },
	{ 0x6474, 0x647d, PDF_CMAP_TABLE, 3865 },
	{ 0x647f, 0x647f, PDF_CMAP_SINGLE, 9901 },
	{ 0x6482, 0x6482, PDF_CMAP_SINGLE, 9894 },
	{ 0x6485, 0x6485, PDF_CMAP_SINGLE, 10469 },
	{ 0x6487, 0x648d, PDF_CMAP_TABLE, 3875 },
	{ 0x648f, 0x6493, PDF_CMAP_TABLE, 3882 },
	{ 0x6495, 0x649a, PDF_CMAP_TABLE, 3887 },
	{ 0x649c, 0x64a0, PDF_CMAP_TABLE, 3893 },
	{ 0x64a2, 0x64a6, PDF_CMAP_TABLE, 3898 },
	{ 0x64a9, 0x64a9, PDF_CMAP_SINGLE, 4452 },
	{ 0x64ab, 0x64b4, PDF_CMAP_TABLE, 3903 },
	{ 0x64b6, 0x64b6, PDF_CMAP_SINGLE, 14948 },
	{ 0x64bb, 0x64bc, PDF_CMAP_RANGE, 4795 },
	{ 0x64bd, 0x64c5, PDF_CMAP_TABLE, 3913 },
	{ 0x64c7, 0x64c7, PDF_CMAP_SINGLE, 4799 },
	{ 0x64c9, 0x64cb, PDF_CMAP_TABLE, 3922 },
	{ 0x64cd, 0x64d0, PDF_CMAP_TABLE, 3925 },
	{ 0x64d2, 0x64d4, PDF_CMAP_TABLE, 3929 },
	{ 0x64d6, 0x64db, PDF_CMAP_TABLE, 3932 },
	{ 0x64dd, 0x64dd, PDF_CMAP_SINGLE, 17182 },
	{ 0x64e0, 0x64ed, PDF_CMAP_TABLE, 3938 },
	{ 0x64ef, 0x64f4, PDF_CMAP_TABLE, 3952 },
	{ 0x64f7, 0x64f8, PDF_CMAP_TABLE, 3958 },
	{ 0x64fa, 0x64fb, PDF_CMAP_RANGE, 5334 },
	{ 0x64fc, 0x6501, PDF_CMAP_TABLE, 3960 },
	{ 0x6503, 0x6504, PDF_CMAP_TABLE, 3966 },
	{ 0x6506, 0x6507, PDF_CMAP_TABLE, 3968 },
	{ 0x6509, 0x650a, PDF_CMAP_TABLE, 3970 },
	{ 0x650c, 0x6511, PDF_CMAP_TABLE, 3972 },
	{ 0x6513, 0x6519, PDF_CMAP_TABLE, 3978 },
	{ 0x651b, 0x6526, PDF_CMAP_TABLE, 3985 },
	{ 0x6529, 0x6530, PDF_CMAP_TABLE, 3997 },
	{ 0x6532, 0x6539, PDF_CMAP_TABLE, 4005 },
	{ 0x653b, 0x653b, PDF_CMAP_SINGLE, 1208 },
	{ 0x653d, 0x653f, PDF_CMAP_TABLE, 4013 },
	{ 0x6541, 0x6541, PDF_CMAP_SINGLE, 6913 },
	{ 0x6543, 0x6543, PDF_CMAP_SINGLE, 6914 },
	{ 0x6545, 0x6546, PDF_CMAP_TABLE, 4016 },
	{ 0x6548, 0x6549, PDF_CMAP_RANGE, 2262 },
	{ 0x654a, 0x654a, PDF_CMAP_SINGLE, 7358 },
	{ 0x654d, 0x654d, PDF_CMAP_SINGLE, 16799 },
	{ 0x654f, 0x654f, PDF_CMAP_SINGLE, 2749 },
	{ 0x6551, 0x6551, PDF_CMAP_SINGLE, 2745 },
	{ 0x6553, 0x6559, PDF_CMAP_TABLE, 4018 },
	{ 0x655c, 0x655f, PDF_CMAP_TABLE, 4025 },
	{ 0x6562, 0x6563, PDF_CMAP_RANGE, 3186 },
	{ 0x6564, 0x6568, PDF_CMAP_TABLE, 4029 },
	{ 0x656a, 0x656d, PDF_CMAP_TABLE, 4034 },
	{ 0x656f, 0x656f, PDF_CMAP_SINGLE, 9225 },
	{ 0x6572, 0x6576, PDF_CMAP_TABLE, 4038 },
	{ 0x6577, 0x6578, PDF_CMAP_RANGE, 4463 },
	{ 0x6579, 0x657c, PDF_CMAP_TABLE, 4043 },
	{ 0x657f, 0x6581, PDF_CMAP_TABLE, 4047 },
	{ 0x6582, 0x6583, PDF_CMAP_RANGE, 5089 },
	{ 0x6584, 0x6589, PDF_CMAP_TABLE, 4050 },
	{ 0x658c, 0x658c, PDF_CMAP_SINGLE, 8563 },
	{ 0x6590, 0x6592, PDF_CMAP_TABLE, 4056 },
	{ 0x6594, 0x6597, PDF_CMAP_TABLE, 4059 },
	{ 0x6599, 0x6599, PDF_CMAP_SINGLE, 2264 },
	{ 0x659b, 0x659c, PDF_CMAP_TABLE, 4063 },
	{ 0x659d, 0x659e, PDF_CMAP_RANGE, 8564 },
	{ 0x659f, 0x65a2, PDF_CMAP_TABLE, 4065 },
	{ 0x65a4, 0x65a5, PDF_CMAP_TABLE, 4069 },
	{ 0x65a7, 0x65a8, PDF_CMAP_TABLE, 4071 },
	{ 0x65aa, 0x65ac, PDF_CMAP_TABLE, 4073 },
	{ 0x65ae, 0x65b0, PDF_CMAP_TABLE, 4076 },
	{ 0x65b2, 0x65b3, PDF_CMAP_RANGE, 10485 },
	{ 0x65b5, 0x65b9, PDF_CMAP_TABLE, 4079 },
	{ 0x65bb, 0x65bf, PDF_CMAP_TABLE, 4084 },
	{ 0x65c1, 0x65c2, PDF_CMAP_TABLE, 4089 },
	{ 0x65c3, 0x65c4, PDF_CMAP_RANGE, 7361 },
	{ 0x65c5, 0x65c6, PDF_CMAP_TABLE, 4091 },
	{ 0x65cb, 0x65cc, PDF_CMAP_RANGE, 2757 },
	{ 0x65cd, 0x65d4, PDF_CMAP_TABLE, 4093 },
	{ 0x65d6, 0x65d7, PDF_CMAP_TABLE, 4101 },
	{ 0x65da, 0x65db, PDF_CMAP_TABLE, 4103 },
	{ 0x65dd, 0x65e3, PDF_CMAP_TABLE, 4105 },
	{ 0x65e5, 0x65e6, PDF_CMAP_TABLE, 4112 },
	{ 0x65e8, 0x65e9, PDF_CMAP_TABLE, 4114 },
	{ 0x65ec, 0x65ed, PDF_CMAP_RANGE, 986 },
	{ 0x65ee, 0x65f5, PDF_CMAP_TABLE, 4116 },
	{ 0x65fa, 0x65fd, PDF_CMAP_TABLE, 4124 },
	{ 0x65ff, 0x6600, PDF_CMAP_TABLE, 4128 },
	{ 0x6602, 0x6615, PDF_CMAP_TABLE, 4130 },
	{ 0x6618, 0x6618, PDF_CMAP_SINGLE, 16817 },
	{ 0x661c, 0x6628, PDF_CMAP_TABLE, 4150 },
	{ 0x662b, 0x662b, PDF_CMAP_SINGLE, 6925 },
	{ 0x662d, 0x6636, PDF_CMAP_TABLE, 4163 },
	{ 0x6639, 0x663a, PDF_CMAP_TABLE, 4173 },
	{ 0x6641, 0x6645, PDF_CMAP_TABLE, 4175 },
	{ 0x6647, 0x664c, PDF_CMAP_TABLE, 4180 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 2269 },
	{ 0x6651, 0x6653, PDF_CMAP_TABLE, 4186 },
	{ 0x6657, 0x6657, PDF_CMAP_SINGLE, 16539 },
	{ 0x6659, 0x665f, PDF_CMAP_TABLE, 4189 },
	{ 0x6661, 0x6668, PDF_CMAP_TABLE, 4196 },
	{ 0x666a, 0x666c, PDF_CMAP_TABLE, 4204 },
	{ 0x666e, 0x6674, PDF_CMAP_TABLE, 4207 },
	{ 0x6676, 0x667e, PDF_CMAP_TABLE, 4214 },
	{ 0x6680, 0x6680, PDF_CMAP_SINGLE, 8572 },
	{ 0x6684, 0x6686, PDF_CMAP_TABLE, 4223 },
	{ 0x6687, 0x6688, PDF_CMAP_RANGE, 3632 },
	{ 0x6689, 0x668e, PDF_CMAP_TABLE, 4226 },
	{ 0x6690, 0x6692, PDF_CMAP_TABLE, 4232 },
	{ 0x6694, 0x669a, PDF_CMAP_TABLE, 4235 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 4077 },
	{ 0x669f, 0x66a2, PDF_CMAP_TABLE, 4242 },
	{ 0x66a4, 0x66a4, PDF_CMAP_SINGLE, 16570 },
	{ 0x66a8, 0x66ab, PDF_CMAP_TABLE, 4246 },
	{ 0x66ad, 0x66bb, PDF_CMAP_TABLE, 4250 },
	{ 0x66bd, 0x66c0, PDF_CMAP_TABLE, 4265 },
	{ 0x66c4, 0x66c4, PDF_CMAP_SINGLE, 4810 },
	{ 0x66c6, 0x66c9, PDF_CMAP_TABLE, 4269 },
	{ 0x66ca, 0x66cb, PDF_CMAP_RANGE, 11104 },
	{ 0x66cc, 0x66cf, PDF_CMAP_TABLE, 4273 },
	{ 0x66d2, 0x66d2, PDF_CMAP_SINGLE, 11634 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 5092 },
	{ 0x66d8, 0x66d9, PDF_CMAP_TABLE, 4277 },
	{ 0x66da, 0x66db, PDF_CMAP_RANGE, 12081 },
	{ 0x66dc, 0x66de, PDF_CMAP_TABLE, 4279 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 5508 },
	{ 0x66e3, 0x66e4, PDF_CMAP_RANGE, 12762 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 5657 },
	{ 0x66e8, 0x66e9, PDF_CMAP_TABLE, 4282 },
	{ 0x66eb, 0x66ec, PDF_CMAP_TABLE, 4284 },
	{ 0x66ed, 0x66ee, PDF_CMAP_RANGE, 13445 },
	{ 0x66f0, 0x66f1, PDF_CMAP_TABLE, 4286 },
	{ 0x66f2, 0x66f3, PDF_CMAP_RANGE, 988 },
	{ 0x66f4, 0x66f4, PDF_CMAP_SINGLE, 1211 },
	{ 0x66f6, 0x66f9, PDF_CMAP_TABLE, 4288 },
	{ 0x66fc, 0x66fc, PDF_CMAP_SINGLE, 2591 },
	{ 0x66fe, 0x66ff, PDF_CMAP_RANGE, 3200 },
	{ 0x6700, 0x6705, PDF_CMAP_TABLE, 4292 },
	{ 0x6708, 0x670e, PDF_CMAP_TABLE, 4298 },
	{ 0x670f, 0x6710, PDF_CMAP_RANGE, 6931 },
	{ 0x6712, 0x6713, PDF_CMAP_RANGE, 7368 },
	{ 0x6714, 0x6715, PDF_CMAP_RANGE, 2276 },
	{ 0x6716, 0x6718, PDF_CMAP_TABLE, 4305 },
	{ 0x671b, 0x671b, PDF_CMAP_SINGLE, 2768 },
	{ 0x671d, 0x6723, PDF_CMAP_TABLE, 4308 },
	{ 0x6725, 0x6728, PDF_CMAP_TABLE, 4315 },
	{ 0x672a, 0x672b, PDF_CMAP_RANGE, 842 },
	{ 0x672c, 0x672e, PDF_CMAP_TABLE, 4319 },
	{ 0x6731, 0x6731, PDF_CMAP_SINGLE, 993 },
	{ 0x6733, 0x6736, PDF_CMAP_TABLE, 4322 },
	{ 0x6738, 0x673f, PDF_CMAP_TABLE, 4326 },
	{ 0x6744, 0x6749, PDF_CMAP_TABLE, 4334 },
	{ 0x674b, 0x674d, PDF_CMAP_TABLE, 4340 },
	{ 0x674e, 0x6751, PDF_CMAP_RANGE, 1213 },
	{ 0x6753, 0x6753, PDF_CMAP_SINGLE, 1223 },
	{ 0x6755, 0x6757, PDF_CMAP_TABLE, 4343 },
	{ 0x6759, 0x675a, PDF_CMAP_TABLE, 4346 },
	{ 0x675c, 0x6762, PDF_CMAP_TABLE, 4348 },
	{ 0x6767, 0x6767, PDF_CMAP_SINGLE, 16842 },
	{ 0x676a, 0x676a, PDF_CMAP_SINGLE, 1560 },
	{ 0x676c, 0x676d, PDF_CMAP_TABLE, 4355 },
	{ 0x676f, 0x6770, PDF_CMAP_RANGE, 1550 },
	{ 0x6771, 0x6777, PDF_CMAP_TABLE, 4357 },
	{ 0x6778, 0x6779, PDF_CMAP_RANGE, 6598 },
	{ 0x677a, 0x677f, PDF_CMAP_TABLE, 4364 },
	{ 0x6781, 0x6781, PDF_CMAP_SINGLE, 6597 },
	{ 0x6783, 0x6787, PDF_CMAP_TABLE, 4370 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 1553 },
	{ 0x678b, 0x6795, PDF_CMAP_TABLE, 4375 },
	{ 0x6797, 0x679a, PDF_CMAP_TABLE, 4386 },
	{ 0x679c, 0x679d, PDF_CMAP_TABLE, 4390 },
	{ 0x679f, 0x67a0, PDF_CMAP_TABLE, 4392 },
	{ 0x67a4, 0x67a4, PDF_CMAP_SINGLE, 18001 },
	{ 0x67ac, 0x67ac, PDF_CMAP_SINGLE, 16867 },
	{ 0x67ae, 0x67bb, PDF_CMAP_TABLE, 4394 },
	{ 0x67bf, 0x67c6, PDF_CMAP_TABLE, 4408 },
	{ 0x67c8, 0x67c8, PDF_CMAP_SINGLE, 6935 },
	{ 0x67c9, 0x67ca, PDF_CMAP_RANGE, 6968 },
	{ 0x67cb, 0x67d4, PDF_CMAP_TABLE, 4416 },
	{ 0x67d6, 0x67df, PDF_CMAP_TABLE, 4426 },
	{ 0x67e2, 0x67e7, PDF_CMAP_TABLE, 4436 },
	{ 0x67e9, 0x67fa, PDF_CMAP_TABLE, 4442 },
	{ 0x67fc, 0x67fc, PDF_CMAP_SINGLE, 6960 },
	{ 0x67fe, 0x6804, PDF_CMAP_TABLE, 4460 },
	{ 0x680d, 0x680d, PDF_CMAP_SINGLE, 17025 },
	{ 0x6810, 0x6810, PDF_CMAP_SINGLE, 14748 },
	{ 0x6812, 0x6814, PDF_CMAP_TABLE, 4467 },
	{ 0x6816, 0x6818, PDF_CMAP_TABLE, 4470 },
	{ 0x681a, 0x6822, PDF_CMAP_TABLE, 4473 },
	{ 0x6825, 0x6826, PDF_CMAP_TABLE, 4482 },
	{ 0x6828, 0x682b, PDF_CMAP_TABLE, 4484 },
	{ 0x682d, 0x682f, PDF_CMAP_TABLE, 4488 },
	{ 0x6831, 0x6831, PDF_CMAP_SINGLE, 7379 },
	{ 0x6832, 0x6833, PDF_CMAP_RANGE, 7373 },
	{ 0x6834, 0x683e, PDF_CMAP_TABLE, 4491 },
	{ 0x6840, 0x6851, PDF_CMAP_TABLE, 4502 },
	{ 0x6853, 0x6856, PDF_CMAP_TABLE, 4520 },
	{ 0x685d, 0x685d, PDF_CMAP_SINGLE, 16854 },
	{ 0x6865, 0x6865, PDF_CMAP_SINGLE, 17751 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 7977 },
	{ 0x686d, 0x686e, PDF_CMAP_RANGE, 7961 },
	{ 0x686f, 0x686f, PDF_CMAP_SINGLE, 7966 },
	{ 0x6871, 0x6872, PDF_CMAP_TABLE, 4524 },
	{ 0x6874, 0x6879, PDF_CMAP_TABLE, 4526 },
	{ 0x687b, 0x688c, PDF_CMAP_TABLE, 4532 },
	{ 0x688f, 0x6894, PDF_CMAP_TABLE, 4550 },
	{ 0x6896, 0x6898, PDF_CMAP_TABLE, 4556 },
	{ 0x689b, 0x689d, PDF_CMAP_TABLE, 4559 },
	{ 0x689f, 0x68a4, PDF_CMAP_TABLE, 4562 },
	{ 0x68a6, 0x68b6, PDF_CMAP_TABLE, 4568 },
	{ 0x68b9, 0x68b9, PDF_CMAP_SINGLE, 15636 },
	{ 0x68bd, 0x68bd, PDF_CMAP_SINGLE, 17020 },
	{ 0x68c3, 0x68ce, PDF_CMAP_TABLE, 4585 },
	{ 0x68d0, 0x68d8, PDF_CMAP_TABLE, 4597 },
	{ 0x68da, 0x68da, PDF_CMAP_SINGLE, 3224 },
	{ 0x68dc, 0x68dc, PDF_CMAP_SINGLE, 8581 },
	{ 0x68dd, 0x68de, PDF_CMAP_RANGE, 8615 },
	{ 0x68df, 0x68e1, PDF_CMAP_TABLE, 4606 },
	{ 0x68e3, 0x68e4, PDF_CMAP_TABLE, 4609 },
	{ 0x68e6, 0x68ec, PDF_CMAP_TABLE, 4611 },
	{ 0x68ee, 0x68fd, PDF_CMAP_TABLE, 4618 },
	{ 0x6900, 0x6915, PDF_CMAP_TABLE, 4634 },
	{ 0x6917, 0x6918, PDF_CMAP_TABLE, 4656 },
	{ 0x691a, 0x691b, PDF_CMAP_TABLE, 4658 },
	{ 0x6925, 0x6925, PDF_CMAP_SINGLE, 8624 },
	{ 0x692a, 0x692a, PDF_CMAP_SINGLE, 8582 },
	{ 0x692c, 0x692c, PDF_CMAP_SINGLE, 16184 },
	{ 0x692f, 0x6930, PDF_CMAP_TABLE, 4660 },
	{ 0x6932, 0x6939, PDF_CMAP_TABLE, 4662 },
	{ 0x693b, 0x6946, PDF_CMAP_TABLE, 4670 },
	{ 0x6948, 0x6949, PDF_CMAP_RANGE, 9252 },
	{ 0x694a, 0x694c, PDF_CMAP_TABLE, 4682 },
	{ 0x694e, 0x694f, PDF_CMAP_TABLE, 4685 },
	{ 0x6951, 0x697b, PDF_CMAP_TABLE, 4687 },
	{ 0x6980, 0x6980, PDF_CMAP_SINGLE, 16190 },
	{ 0x6982, 0x6983, PDF_CMAP_TABLE, 4730 },
	{ 0x6985, 0x6986, PDF_CMAP_TABLE, 4732 },
	{ 0x698a, 0x698a, PDF_CMAP_SINGLE, 15732 },
	{ 0x698d, 0x698e, PDF_CMAP_TABLE, 4734 },
	{ 0x6990, 0x6991, PDF_CMAP_TABLE, 4736 },
	{ 0x6993, 0x699c, PDF_CMAP_TABLE, 4738 },
	{ 0x699e, 0x69b7, PDF_CMAP_TABLE, 4748 },
	{ 0x69b9, 0x69b9, PDF_CMAP_SINGLE, 9938 },
	{ 0x69bb, 0x69c4, PDF_CMAP_TABLE, 4774 },
	{ 0x69c6, 0x69c6, PDF_CMAP_SINGLE, 9953 },
	{ 0x69c9, 0x69d1, PDF_CMAP_TABLE, 4784 },
	{ 0x69d3, 0x69d6, PDF_CMAP_TABLE, 4793 },
	{ 0x69d9, 0x69d9, PDF_CMAP_SINGLE, 9947 },
	{ 0x69e1, 0x69e2, PDF_CMAP_TABLE, 4797 },
	{ 0x69e4, 0x69e9, PDF_CMAP_TABLE, 4799 },
	{ 0x69eb, 0x69ee, PDF_CMAP_TABLE, 4805 },
	{ 0x69f1, 0x69f4, PDF_CMAP_TABLE, 4809 },
	{ 0x69f6, 0x6a0d, PDF_CMAP_TABLE, 4813 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 10526 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 4483 },
	{ 0x6a13, 0x6a21, PDF_CMAP_TABLE, 4837 },
	{ 0x6a23, 0x6a23, PDF_CMAP_SINGLE, 4469 },
	{ 0x6a25, 0x6a28, PDF_CMAP_TABLE, 4852 },
	{ 0x6a2b, 0x6a2d, PDF_CMAP_TABLE, 4856 },
	{ 0x6a32, 0x6a35, PDF_CMAP_TABLE, 4859 },
	{ 0x6a38, 0x6a41, PDF_CMAP_TABLE, 4863 },
	{ 0x6a44, 0x6a49, PDF_CMAP_TABLE, 4873 },
	{ 0x6a4b, 0x6a4c, PDF_CMAP_TABLE, 4879 },
	{ 0x6a4d, 0x6a4e, PDF_CMAP_RANGE, 11141 },
	{ 0x6a4f, 0x6a5b, PDF_CMAP_TABLE, 4881 },
	{ 0x6a5d, 0x6a62, PDF_CMAP_TABLE, 4894 },
	{ 0x6a64, 0x6a6b, PDF_CMAP_TABLE, 4900 },
	{ 0x6a6d, 0x6a6d, PDF_CMAP_SINGLE, 11120 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 11134 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 18018 },
	{ 0x6a74, 0x6a74, PDF_CMAP_SINGLE, 18017 },
	{ 0x6a76, 0x6a76, PDF_CMAP_SINGLE, 11121 },
	{ 0x6a7a, 0x6a7a, PDF_CMAP_SINGLE, 15021 },
	{ 0x6a7e, 0x6a85, PDF_CMAP_TABLE, 4908 },
	{ 0x6a87, 0x6a87, PDF_CMAP_SINGLE, 11644 },
	{ 0x6a89, 0x6a8a, PDF_CMAP_TABLE, 4916 },
	{ 0x6a8c, 0x6a97, PDF_CMAP_TABLE, 4918 },
	{ 0x6a99, 0x6aa8, PDF_CMAP_TABLE, 4930 },
	{ 0x6aab, 0x6aad, PDF_CMAP_TABLE, 4946 },
	{ 0x6aae, 0x6aaf, PDF_CMAP_RANGE, 5346 },
	{ 0x6ab1, 0x6ab5, PDF_CMAP_TABLE, 4949 },
	{ 0x6ab6, 0x6ab7, PDF_CMAP_RANGE, 12090 },
	{ 0x6ab8, 0x6abb, PDF_CMAP_TABLE, 4954 },
	{ 0x6abd, 0x6abe, PDF_CMAP_TABLE, 4958 },
	{ 0x6ac2, 0x6ac3, PDF_CMAP_TABLE, 4960 },
	{ 0x6ac5, 0x6acd, PDF_CMAP_TABLE, 4962 },
	{ 0x6acf, 0x6ad1, PDF_CMAP_TABLE, 4971 },
	{ 0x6ad3, 0x6ad4, PDF_CMAP_TABLE, 4974 },
	{ 0x6ad8, 0x6ae1, PDF_CMAP_TABLE, 4976 },
	{ 0x6ae5, 0x6ae5, PDF_CMAP_SINGLE, 5510 },
	{ 0x6ae7, 0x6ae8, PDF_CMAP_TABLE, 4986 },
	{ 0x6aea, 0x6aec, PDF_CMAP_TABLE, 4988 },
	{ 0x6aee, 0x6aef, PDF_CMAP_RANGE, 12770 },
	{ 0x6af0, 0x6af1, PDF_CMAP_TABLE, 4991 },
	{ 0x6af3, 0x6af3, PDF_CMAP_SINGLE, 12764 },
	{ 0x6af6, 0x6af6, PDF_CMAP_SINGLE, 18026 },
	{ 0x6af8, 0x6afc, PDF_CMAP_TABLE, 4993 },
	{ 0x6b00, 0x6b00, PDF_CMAP_SINGLE, 13015 },
	{ 0x6b02, 0x6b05, PDF_CMAP_TABLE, 4998 },
	{ 0x6b08, 0x6b09, PDF_CMAP_RANGE, 13190 },
	{ 0x6b0a, 0x6b0b, PDF_CMAP_TABLE, 5002 },
	{ 0x6b0f, 0x6b10, PDF_CMAP_TABLE, 5004 },
	{ 0x6b11, 0x6b12, PDF_CMAP_RANGE, 13342 },
	{ 0x6b13, 0x6b13, PDF_CMAP_SINGLE, 13447 },
	{ 0x6b16, 0x6b17, PDF_CMAP_TABLE, 5006 },
	{ 0x6b18, 0x6b19, PDF_CMAP_RANGE, 13516 },
	{ 0x6b1a, 0x6b1a, PDF_CMAP_SINGLE, 13519 },
	{ 0x6b1d, 0x6b1e, PDF_CMAP_TABLE, 5008 },
	{ 0x6b20, 0x6b21, PDF_CMAP_TABLE, 5010 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 1562 },
	{ 0x6b25, 0x6b25, PDF_CMAP_SINGLE, 6601 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 6973 },
	{ 0x6b2c, 0x6b2d, PDF_CMAP_TABLE, 5012 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 7399 },
	{ 0x6b31, 0x6b36, PDF_CMAP_TABLE, 5014 },
	{ 0x6b37, 0x6b38, PDF_CMAP_RANGE, 7997 },
	{ 0x6b39, 0x6b3f, PDF_CMAP_TABLE, 5020 },
	{ 0x6b41, 0x6b43, PDF_CMAP_TABLE, 5027 },
	{ 0x6b45, 0x6b4e, PDF_CMAP_TABLE, 5030 },
	{ 0x6b50, 0x6b52, PDF_CMAP_TABLE, 5040 },
	{ 0x6b54, 0x6b57, PDF_CMAP_TABLE, 5043 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 4828 },
	{ 0x6b5b, 0x6b5c, PDF_CMAP_TABLE, 5047 },
	{ 0x6b5e, 0x6b65, PDF_CMAP_TABLE, 5049 },
	{ 0x6b66, 0x6b67, PDF_CMAP_RANGE, 1563 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 1897 },
	{ 0x6b6d, 0x6b6d, PDF_CMAP_SINGLE, 7403 },
	{ 0x6b6f, 0x6b6f, PDF_CMAP_SINGLE, 17685 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 3659 },
	{ 0x6b74, 0x6b74, PDF_CMAP_SINGLE, 16892 },
	{ 0x6b76, 0x6b7b, PDF_CMAP_TABLE, 5057 },
	{ 0x6b7e, 0x6b84, PDF_CMAP_TABLE, 5063 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 1899 },
	{ 0x6b88, 0x6b8a, PDF_CMAP_TABLE, 5070 },
	{ 0x6b8c, 0x6b8c, PDF_CMAP_SINGLE, 8003 },
	{ 0x6b8d, 0x6b8e, PDF_CMAP_RANGE, 8001 },
	{ 0x6b8f, 0x6b8f, PDF_CMAP_SINGLE, 8000 },
	{ 0x6b91, 0x6b91, PDF_CMAP_SINGLE, 7999 },
	{ 0x6b94, 0x6b99, PDF_CMAP_TABLE, 5073 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 9288 },
	{ 0x6b9e, 0x6ba0, PDF_CMAP_RANGE, 9957 },
	{ 0x6ba2, 0x6ba7, PDF_CMAP_TABLE, 5079 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 11148 },
	{ 0x6bad, 0x6bb0, PDF_CMAP_TABLE, 5085 },
	{ 0x6bb2, 0x6bb3, PDF_CMAP_TABLE, 5089 },
	{ 0x6bb5, 0x6bb7, PDF_CMAP_TABLE, 5091 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 2792 },
	{ 0x6bbc, 0x6bbd, PDF_CMAP_TABLE, 5094 },
	{ 0x6bbf, 0x6bc1, PDF_CMAP_TABLE, 5096 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 9960 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_RANGE, 4487 },
	{ 0x6bc7, 0x6bcd, PDF_CMAP_TABLE, 5099 },
	{ 0x6bcf, 0x6bd0, PDF_CMAP_TABLE, 5106 },
	{ 0x6bd2, 0x6bd4, PDF_CMAP_TABLE, 5108 },
	{ 0x6bd6, 0x6bd8, PDF_CMAP_TABLE, 5111 },
	{ 0x6bda, 0x6bdc, PDF_CMAP_TABLE, 5114 },
	{ 0x6bde, 0x6bde, PDF_CMAP_SINGLE, 6604 },
	{ 0x6be0, 0x6be4, PDF_CMAP_TABLE, 5117 },
	{ 0x6be6, 0x6be8, PDF_CMAP_TABLE, 5122 },
	{ 0x6bea, 0x6bea, PDF_CMAP_SINGLE, 18029 },
	{ 0x6beb, 0x6bec, PDF_CMAP_RANGE, 2793 },
	{ 0x6bef, 0x6bf0, PDF_CMAP_TABLE, 5125 },
	{ 0x6bf2, 0x6bf3, PDF_CMAP_RANGE, 8636 },
	{ 0x6bf7, 0x6bf8, PDF_CMAP_RANGE, 9292 },
	{ 0x6bf9, 0x6bfa, PDF_CMAP_TABLE, 5127 },
	{ 0x6bfb, 0x6bfc, PDF_CMAP_RANGE, 9289 },
	{ 0x6bfd, 0x6c06, PDF_CMAP_TABLE, 5129 },
	{ 0x6c08, 0x6c09, PDF_CMAP_TABLE, 5139 },
	{ 0x6c0b, 0x6c0d, PDF_CMAP_TABLE, 5141 },
	{ 0x6c0f, 0x6c11, PDF_CMAP_TABLE, 5144 },
	{ 0x6c13, 0x6c16, PDF_CMAP_TABLE, 5147 },
	{ 0x6c18, 0x6c18, PDF_CMAP_SINGLE, 6145 },
	{ 0x6c19, 0x6c1a, PDF_CMAP_RANGE, 6318 },
	{ 0x6c1b, 0x6c1d, PDF_CMAP_TABLE, 5151 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 1903 },
	{ 0x6c20, 0x6c21, PDF_CMAP_RANGE, 6980 },
	{ 0x6c23, 0x6c26, PDF_CMAP_TABLE, 5154 },
	{ 0x6c27, 0x6c28, PDF_CMAP_RANGE, 2307 },
	{ 0x6c2a, 0x6c2c, PDF_CMAP_TABLE, 5158 },
	{ 0x6c2e, 0x6c2f, PDF_CMAP_RANGE, 3234 },
	{ 0x6c30, 0x6c31, PDF_CMAP_TABLE, 5161 },
	{ 0x6c33, 0x6c3b, PDF_CMAP_TABLE, 5163 },
	{ 0x6c3d, 0x6c43, PDF_CMAP_TABLE, 5172 },
	{ 0x6c46, 0x6c46, PDF_CMAP_SINGLE, 6146 },
	{ 0x6c49, 0x6c4a, PDF_CMAP_TABLE, 5179 },
	{ 0x6c4b, 0x6c4c, PDF_CMAP_RANGE, 6152 },
	{ 0x6c4d, 0x6c4e, PDF_CMAP_RANGE, 1008 },
	{ 0x6c4f, 0x6c50, PDF_CMAP_TABLE, 5181 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 6147 },
	{ 0x6c54, 0x6c55, PDF_CMAP_TABLE, 5183 },
	{ 0x6c57, 0x6c5e, PDF_CMAP_TABLE, 5185 },
	{ 0x6c5f, 0x6c60, PDF_CMAP_RANGE, 1002 },
	{ 0x6c61, 0x6c61, PDF_CMAP_SINGLE, 1006 },
	{ 0x6c65, 0x6c6b, PDF_CMAP_TABLE, 5193 },
	{ 0x6c6d, 0x6c76, PDF_CMAP_TABLE, 5200 },
	{ 0x6c78, 0x6c7b, PDF_CMAP_TABLE, 5210 },
	{ 0x6c7d, 0x6c87, PDF_CMAP_TABLE, 5214 },
	{ 0x6c88, 0x6c89, PDF_CMAP_RANGE, 1231 },
	{ 0x6c8a, 0x6c90, PDF_CMAP_TABLE, 5225 },
	{ 0x6c92, 0x6c96, PDF_CMAP_TABLE, 5232 },
	{ 0x6c98, 0x6c9d, PDF_CMAP_TABLE, 5237 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 14473 },
	{ 0x6ca2, 0x6ca2, PDF_CMAP_SINGLE, 15715 },
	{ 0x6caa, 0x6cb4, PDF_CMAP_TABLE, 5243 },
	{ 0x6cb6, 0x6cbc, PDF_CMAP_TABLE, 5254 },
	{ 0x6cbd, 0x6cbe, PDF_CMAP_RANGE, 1575 },
	{ 0x6cbf, 0x6cc7, PDF_CMAP_TABLE, 5261 },
	{ 0x6cc9, 0x6cd7, PDF_CMAP_TABLE, 5270 },
	{ 0x6cd9, 0x6ce3, PDF_CMAP_TABLE, 5285 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 1573 },
	{ 0x6ce7, 0x6cf3, PDF_CMAP_TABLE, 5296 },
	{ 0x6cf5, 0x6cf5, PDF_CMAP_SINGLE, 1919 },
	{ 0x6cf9, 0x6cf9, PDF_CMAP_SINGLE, 6635 },
	{ 0x6d00, 0x6d12, PDF_CMAP_TABLE, 5309 },
	{ 0x6d16, 0x6d1b, PDF_CMAP_TABLE, 5328 },
	{ 0x6d1d, 0x6d20, PDF_CMAP_TABLE, 5334 },
	{ 0x6d22, 0x6d22, PDF_CMAP_SINGLE, 7010 },
	{ 0x6d24, 0x6d3c, PDF_CMAP_TABLE, 5338 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_RANGE, 1915 },
	{ 0x6d3f, 0x6d42, PDF_CMAP_TABLE, 5363 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 17752 },
	{ 0x6d57, 0x6d5c, PDF_CMAP_TABLE, 5367 },
	{ 0x6d5e, 0x6d62, PDF_CMAP_TABLE, 5373 },
	{ 0x6d63, 0x6d64, PDF_CMAP_RANGE, 7414 },
	{ 0x6d65, 0x6d6a, PDF_CMAP_TABLE, 5378 },
	{ 0x6d6c, 0x6d72, PDF_CMAP_TABLE, 5384 },
	{ 0x6d74, 0x6d98, PDF_CMAP_TABLE, 5391 },
	{ 0x6da4, 0x6da5, PDF_CMAP_TABLE, 5428 },
	{ 0x6daa, 0x6dac, PDF_CMAP_TABLE, 5430 },
	{ 0x6dae, 0x6daf, PDF_CMAP_TABLE, 5433 },
	{ 0x6db1, 0x6db5, PDF_CMAP_TABLE, 5435 },
	{ 0x6db7, 0x6dc0, PDF_CMAP_TABLE, 5440 },
	{ 0x6dc2, 0x6dc2, PDF_CMAP_SINGLE, 8033 },
	{ 0x6dc4, 0x6dcd, PDF_CMAP_TABLE, 5450 },
	{ 0x6dcf, 0x6ddb, PDF_CMAP_TABLE, 5460 },
	{ 0x6ddc, 0x6ddd, PDF_CMAP_RANGE, 8023 },
	{ 0x6dde, 0x6de6, PDF_CMAP_TABLE, 5473 },
	{ 0x6de8, 0x6df7, PDF_CMAP_TABLE, 5482 },
	{ 0x6df9, 0x6dfd, PDF_CMAP_TABLE, 5498 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 8016 },
	{ 0x6e02, 0x6e05, PDF_CMAP_TABLE, 5503 },
	{ 0x6e0a, 0x6e0a, PDF_CMAP_SINGLE, 15041 },
	{ 0x6e0f, 0x6e0f, PDF_CMAP_SINGLE, 15600 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 18040 },
	{ 0x6e18, 0x6e1d, PDF_CMAP_TABLE, 5507 },
	{ 0x6e1f, 0x6e34, PDF_CMAP_TABLE, 5513 },
	{ 0x6e35, 0x6e36, PDF_CMAP_RANGE, 8684 },
	{ 0x6e38, 0x6e3b, PDF_CMAP_TABLE, 5535 },
	{ 0x6e3c, 0x6e3d, PDF_CMAP_RANGE, 8645 },
	{ 0x6e3e, 0x6e41, PDF_CMAP_TABLE, 5539 },
	{ 0x6e43, 0x6e45, PDF_CMAP_TABLE, 5543 },
	{ 0x6e46, 0x6e47, PDF_CMAP_RANGE, 8640 },
	{ 0x6e49, 0x6e4b, PDF_CMAP_TABLE, 5546 },
	{ 0x6e4d, 0x6e56, PDF_CMAP_TABLE, 5549 },
	{ 0x6e58, 0x6e69, PDF_CMAP_TABLE, 5559 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 8670 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 5577 },
	{ 0x6e71, 0x6e74, PDF_CMAP_TABLE, 5579 },
	{ 0x6e76, 0x6e79, PDF_CMAP_TABLE, 5583 },
	{ 0x6e7c, 0x6e7c, PDF_CMAP_SINGLE, 16931 },
	{ 0x6e86, 0x6e86, PDF_CMAP_SINGLE, 18043 },
	{ 0x6e88, 0x6e89, PDF_CMAP_TABLE, 5587 },
	{ 0x6e8b, 0x6e8b, PDF_CMAP_SINGLE, 15295 },
	{ 0x6e8d, 0x6e90, PDF_CMAP_TABLE, 5589 },
	{ 0x6e92, 0x6e92, PDF_CMAP_SINGLE, 9322 },
	{ 0x6e93, 0x6e94, PDF_CMAP_RANGE, 9300 },
	{ 0x6e96, 0x6ea7, PDF_CMAP_TABLE, 5593 },
	{ 0x6eaa, 0x6eab, PDF_CMAP_TABLE, 5611 },
	{ 0x6eae, 0x6ed6, PDF_CMAP_TABLE, 5613 },
	{ 0x6ed8, 0x6edd, PDF_CMAP_TABLE, 5654 },
	{ 0x6ee2, 0x6ee2, PDF_CMAP_SINGLE, 18046 },
	{ 0x6ee8, 0x6ee9, PDF_CMAP_RANGE, 18049 },
	{ 0x6eeb, 0x6eef, PDF_CMAP_TABLE, 5660 },
	{ 0x6ef1, 0x6ef2, PDF_CMAP_TABLE, 5665 },
	{ 0x6ef4, 0x6f0f, PDF_CMAP_TABLE, 5667 },
	{ 0x6f12, 0x6f18, PDF_CMAP_TABLE, 5695 },
	{ 0x6f19, 0x6f1a, PDF_CMAP_RANGE, 9974 },
	{ 0x6f1c, 0x6f1c, PDF_CMAP_SINGLE, 9999 },
	{ 0x6f1e, 0x6f27, PDF_CMAP_TABLE, 5702 },
	{ 0x6f29, 0x6f41, PDF_CMAP_TABLE, 5712 },
	{ 0x6f43, 0x6f44, PDF_CMAP_TABLE, 5737 },
	{ 0x6f4e, 0x6f58, PDF_CMAP_TABLE, 5739 },
	{ 0x6f5a, 0x6f64, PDF_CMAP_TABLE, 5750 },
	{ 0x6f66, 0x6f67, PDF_CMAP_TABLE, 5761 },
	{ 0x6f69, 0x6f70, PDF_CMAP_TABLE, 5763 },
	{ 0x6f72, 0x6f74, PDF_CMAP_TABLE, 5771 },
	{ 0x6f76, 0x6f82, PDF_CMAP_TABLE, 5774 },
	{ 0x6f84, 0x6f8e, PDF_CMAP_TABLE, 5787 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 10571 },
	{ 0x6f92, 0x6f97, PDF_CMAP_TABLE, 5798 },
	{ 0x6f9d, 0x6fb6, PDF_CMAP_TABLE, 5804 },
	{ 0x6fb8, 0x6fc4, PDF_CMAP_TABLE, 5830 },
	{ 0x6fc6, 0x6fcf, PDF_CMAP_TABLE, 5843 },
	{ 0x6fd3, 0x6fd5, PDF_CMAP_TABLE, 5853 },
	{ 0x6fd8, 0x6fde, PDF_CMAP_TABLE, 5856 },
	{ 0x6fdf, 0x6fe0, PDF_CMAP_RANGE, 5110 },
	{ 0x6fe1, 0x6fe4, PDF_CMAP_TABLE, 5863 },
	{ 0x6fe6, 0x6fe9, PDF_CMAP_TABLE, 5867 },
	{ 0x6feb, 0x6ff2, PDF_CMAP_TABLE, 5871 },
	{ 0x6ff4, 0x6ff4, PDF_CMAP_SINGLE, 11663 },
	{ 0x6ff6, 0x6ff8, PDF_CMAP_TABLE, 5879 },
	{ 0x6ffa, 0x6ffc, PDF_CMAP_TABLE, 5882 },
	{ 0x6ffe, 0x7001, PDF_CMAP_TABLE, 5885 },
	{ 0x7003, 0x7007, PDF_CMAP_TABLE, 5889 },
	{ 0x7009, 0x700b, PDF_CMAP_TABLE, 5894 },
	{ 0x700c, 0x700d, PDF_CMAP_RANGE, 12099 },
	{ 0x700e, 0x700f, PDF_CMAP_TABLE, 5897 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5356 },
	{ 0x7014, 0x7020, PDF_CMAP_TABLE, 5899 },
	{ 0x7021, 0x7023, PDF_CMAP_RANGE, 12463 },
	{ 0x7024, 0x7024, PDF_CMAP_SINGLE, 12468 },
	{ 0x7026, 0x702c, PDF_CMAP_TABLE, 5912 },
	{ 0x702f, 0x7035, PDF_CMAP_TABLE, 5919 },
	{ 0x7037, 0x703c, PDF_CMAP_TABLE, 5926 },
	{ 0x703e, 0x7042, PDF_CMAP_TABLE, 5932 },
	{ 0x7043, 0x7044, PDF_CMAP_RANGE, 13016 },
	{ 0x7045, 0x7046, PDF_CMAP_RANGE, 13021 },
	{ 0x7048, 0x7049, PDF_CMAP_RANGE, 13019 },
	{ 0x704a, 0x704c, PDF_CMAP_TABLE, 5937 },
	{ 0x7050, 0x7052, PDF_CMAP_TABLE, 5940 },
	{ 0x7054, 0x7054, PDF_CMAP_SINGLE, 18065 },
	{ 0x7055, 0x7057, PDF_CMAP_RANGE, 13193 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 5833 },
	{ 0x705a, 0x7068, PDF_CMAP_TABLE, 5943 },
	{ 0x7069, 0x706a, PDF_CMAP_RANGE, 13639 },
	{ 0x706b, 0x706c, PDF_CMAP_TABLE, 5958 },
	{ 0x706f, 0x7071, PDF_CMAP_TABLE, 5960 },
	{ 0x7074, 0x7076, PDF_CMAP_TABLE, 5963 },
	{ 0x7078, 0x707a, PDF_CMAP_TABLE, 5966 },
	{ 0x707c, 0x707d, PDF_CMAP_RANGE, 1255 },
	{ 0x707e, 0x707f, PDF_CMAP_TABLE, 5969 },
	{ 0x7081, 0x7086, PDF_CMAP_TABLE, 5971 },
	{ 0x7089, 0x708b, PDF_CMAP_TABLE, 5977 },
	{ 0x708e, 0x708f, PDF_CMAP_TABLE, 5980 },
	{ 0x7091, 0x7096, PDF_CMAP_TABLE, 5982 },
	{ 0x7098, 0x709a, PDF_CMAP_TABLE, 5988 },
	{ 0x709f, 0x70a1, PDF_CMAP_TABLE, 5991 },
	{ 0x70a3, 0x70a7, PDF_CMAP_TABLE, 5994 },
	{ 0x70a9, 0x70a9, PDF_CMAP_SINGLE, 7021 },
	{ 0x70ab, 0x70b1, PDF_CMAP_TABLE, 5999 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 1930 },
	{ 0x70b4, 0x70b5, PDF_CMAP_RANGE, 7019 },
	{ 0x70b7, 0x70be, PDF_CMAP_TABLE, 6006 },
	{ 0x70c0, 0x70c0, PDF_CMAP_SINGLE, 18227 },
	{ 0x70c4, 0x70c4, PDF_CMAP_SINGLE, 15274 },
	{ 0x70c5, 0x70c7, PDF_CMAP_RANGE, 7463 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 2337 },
	{ 0x70ca, 0x70d4, PDF_CMAP_TABLE, 6014 },
	{ 0x70d5, 0x70d6, PDF_CMAP_RANGE, 16985 },
	{ 0x70d7, 0x70da, PDF_CMAP_TABLE, 6025 },
	{ 0x70dc, 0x70e2, PDF_CMAP_TABLE, 6029 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2335 },
	{ 0x70ef, 0x70f1, PDF_CMAP_TABLE, 6036 },
	{ 0x70f3, 0x7100, PDF_CMAP_TABLE, 6039 },
	{ 0x7102, 0x7102, PDF_CMAP_SINGLE, 8062 },
	{ 0x7104, 0x7106, PDF_CMAP_TABLE, 6053 },
	{ 0x7109, 0x710a, PDF_CMAP_RANGE, 2835 },
	{ 0x710b, 0x710e, PDF_CMAP_TABLE, 6056 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 8053 },
	{ 0x7113, 0x7113, PDF_CMAP_SINGLE, 8057 },
	{ 0x7117, 0x7117, PDF_CMAP_SINGLE, 8047 },
	{ 0x7119, 0x711a, PDF_CMAP_RANGE, 3272 },
	{ 0x711b, 0x7123, PDF_CMAP_TABLE, 6060 },
	{ 0x7125, 0x7126, PDF_CMAP_TABLE, 6069 },
	{ 0x7128, 0x7129, PDF_CMAP_TABLE, 6071 },
	{ 0x712b, 0x712c, PDF_CMAP_TABLE, 6073 },
	{ 0x712e, 0x7136, PDF_CMAP_TABLE, 6075 },
	{ 0x713a, 0x713b, PDF_CMAP_TABLE, 6084 },
	{ 0x713e, 0x713e, PDF_CMAP_SINGLE, 14126 },
	{ 0x7140, 0x7147, PDF_CMAP_TABLE, 6086 },
	{ 0x7149, 0x7154, PDF_CMAP_TABLE, 6094 },
	{ 0x7156, 0x7156, PDF_CMAP_SINGLE, 3700 },
	{ 0x7158, 0x715a, PDF_CMAP_TABLE, 6106 },
	{ 0x715c, 0x716c, PDF_CMAP_TABLE, 6109 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 3278 },
	{ 0x7170, 0x7173, PDF_CMAP_TABLE, 6126 },
	{ 0x7175, 0x7178, PDF_CMAP_TABLE, 6130 },
	{ 0x717a, 0x717e, PDF_CMAP_TABLE, 6134 },
	{ 0x7180, 0x7182, PDF_CMAP_TABLE, 6139 },
	{ 0x7184, 0x718a, PDF_CMAP_TABLE, 6142 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 14826 },
	{ 0x718e, 0x7190, PDF_CMAP_TABLE, 6149 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 4134 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 4129 },
	{ 0x7196, 0x71a5, PDF_CMAP_TABLE, 6152 },
	{ 0x71a7, 0x71aa, PDF_CMAP_TABLE, 6168 },
	{ 0x71ac, 0x71ad, PDF_CMAP_TABLE, 6172 },
	{ 0x71af, 0x71b5, PDF_CMAP_TABLE, 6174 },
	{ 0x71b7, 0x71ba, PDF_CMAP_TABLE, 6181 },
	{ 0x71bc, 0x71bf, PDF_CMAP_TABLE, 6185 },
	{ 0x71c0, 0x71c1, PDF_CMAP_RANGE, 11189 },
	{ 0x71c2, 0x71c2, PDF_CMAP_SINGLE, 11185 },
	{ 0x71c3, 0x71c4, PDF_CMAP_RANGE, 4855 },
	{ 0x71c5, 0x71cb, PDF_CMAP_TABLE, 6189 },
	{ 0x71ce, 0x71d2, PDF_CMAP_TABLE, 6196 },
	{ 0x71d4, 0x71d6, PDF_CMAP_TABLE, 6201 },
	{ 0x71d8, 0x71d9, PDF_CMAP_TABLE, 6204 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 11200 },
	{ 0x71dc, 0x71dd, PDF_CMAP_TABLE, 6206 },
	{ 0x71df, 0x71e2, PDF_CMAP_TABLE, 6208 },
	{ 0x71e4, 0x71e8, PDF_CMAP_TABLE, 6212 },
	{ 0x71eb, 0x71ee, PDF_CMAP_TABLE, 6217 },
	{ 0x71f0, 0x71f2, PDF_CMAP_TABLE, 6221 },
	{ 0x71f4, 0x71f6, PDF_CMAP_TABLE, 6224 },
	{ 0x71f8, 0x71f9, PDF_CMAP_TABLE, 6227 },
	{ 0x71fb, 0x71fc, PDF_CMAP_RANGE, 5358 },
	{ 0x71fd, 0x71ff, PDF_CMAP_TABLE, 6229 },
	{ 0x7201, 0x7203, PDF_CMAP_TABLE, 6232 },
	{ 0x7205, 0x7207, PDF_CMAP_TABLE, 6235 },
	{ 0x7209, 0x720a, PDF_CMAP_TABLE, 6238 },
	{ 0x720c, 0x7210, PDF_CMAP_TABLE, 6240 },
	{ 0x7213, 0x7214, PDF_CMAP_RANGE, 12788 },
	{ 0x7215, 0x7217, PDF_CMAP_TABLE, 6245 },
	{ 0x7219, 0x721b, PDF_CMAP_TABLE, 6248 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 13023 },
	{ 0x721e, 0x721f, PDF_CMAP_RANGE, 13197 },
	{ 0x7222, 0x7224, PDF_CMAP_TABLE, 6251 },
	{ 0x7226, 0x722b, PDF_CMAP_TABLE, 6254 },
	{ 0x722c, 0x722d, PDF_CMAP_RANGE, 1605 },
	{ 0x722e, 0x722e, PDF_CMAP_SINGLE, 17017 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 1937 },
	{ 0x7235, 0x7236, PDF_CMAP_TABLE, 6260 },
	{ 0x7238, 0x723b, PDF_CMAP_TABLE, 6262 },
	{ 0x723d, 0x7242, PDF_CMAP_TABLE, 6266 },
	{ 0x7244, 0x7244, PDF_CMAP_SINGLE, 10018 },
	{ 0x7246, 0x7248, PDF_CMAP_TABLE, 6272 },
	{ 0x7249, 0x724a, PDF_CMAP_RANGE, 7023 },
	{ 0x724b, 0x724c, PDF_CMAP_TABLE, 6275 },
	{ 0x724f, 0x7250, PDF_CMAP_TABLE, 6277 },
	{ 0x7252, 0x7253, PDF_CMAP_TABLE, 6279 },
	{ 0x7255, 0x7263, PDF_CMAP_TABLE, 6281 },
	{ 0x7266, 0x7267, PDF_CMAP_TABLE, 6296 },
	{ 0x7269, 0x726a, PDF_CMAP_TABLE, 6298 },
	{ 0x726c, 0x726c, PDF_CMAP_SINGLE, 7025 },
	{ 0x726e, 0x7270, PDF_CMAP_TABLE, 6300 },
	{ 0x7272, 0x7274, PDF_CMAP_TABLE, 6303 },
	{ 0x7276, 0x7279, PDF_CMAP_TABLE, 6306 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 8065 },
	{ 0x727d, 0x7282, PDF_CMAP_TABLE, 6310 },
	{ 0x7284, 0x7287, PDF_CMAP_TABLE, 6316 },
	{ 0x7288, 0x7289, PDF_CMAP_RANGE, 8703 },
	{ 0x728b, 0x7298, PDF_CMAP_TABLE, 6320 },
	{ 0x729a, 0x729b, PDF_CMAP_TABLE, 6334 },
	{ 0x729d, 0x729e, PDF_CMAP_RANGE, 11202 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 18234 },
	{ 0x72a1, 0x72a4, PDF_CMAP_TABLE, 6336 },
	{ 0x72a5, 0x72a6, PDF_CMAP_RANGE, 12475 },
	{ 0x72a7, 0x72aa, PDF_CMAP_TABLE, 6340 },
	{ 0x72ac, 0x72b0, PDF_CMAP_TABLE, 6344 },
	{ 0x72b2, 0x72b2, PDF_CMAP_SINGLE, 16310 },
	{ 0x72b4, 0x72b5, PDF_CMAP_RANGE, 6156 },
	{ 0x72ba, 0x72ba, PDF_CMAP_SINGLE, 6347 },
	{ 0x72bd, 0x72bd, PDF_CMAP_SINGLE, 6343 },
	{ 0x72bf, 0x72c6, PDF_CMAP_TABLE, 6349 },
	{ 0x72c9, 0x72ce, PDF_CMAP_TABLE, 6357 },
	{ 0x72d0, 0x72d2, PDF_CMAP_TABLE, 6363 },
	{ 0x72d4, 0x72d4, PDF_CMAP_SINGLE, 6657 },
	{ 0x72d6, 0x72da, PDF_CMAP_TABLE, 6366 },
	{ 0x72dc, 0x72dc, PDF_CMAP_SINGLE, 6655 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 7033 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 1942 },
	{ 0x72e2, 0x72e4, PDF_CMAP_TABLE, 6371 },
	{ 0x72e6, 0x72e6, PDF_CMAP_SINGLE, 7035 },
	{ 0x72e8, 0x72eb, PDF_CMAP_TABLE, 6374 },
	{ 0x72f3, 0x72f4, PDF_CMAP_TABLE, 6378 },
	{ 0x72f6, 0x7302, PDF_CMAP_TABLE, 6380 },
	{ 0x7304, 0x7304, PDF_CMAP_SINGLE, 16053 },
	{ 0x7307, 0x7308, PDF_CMAP_TABLE, 6393 },
	{ 0x730a, 0x730c, PDF_CMAP_TABLE, 6395 },
	{ 0x730f, 0x7313, PDF_CMAP_TABLE, 6398 },
	{ 0x7316, 0x7319, PDF_CMAP_TABLE, 6403 },
	{ 0x731b, 0x731e, PDF_CMAP_TABLE, 6407 },
	{ 0x7322, 0x7323, PDF_CMAP_TABLE, 6411 },
	{ 0x7325, 0x732e, PDF_CMAP_TABLE, 6413 },
	{ 0x7330, 0x733c, PDF_CMAP_TABLE, 6423 },
	{ 0x733e, 0x7345, PDF_CMAP_TABLE, 6436 },
	{ 0x7348, 0x734a, PDF_CMAP_TABLE, 6444 },
	{ 0x734c, 0x7352, PDF_CMAP_TABLE, 6447 },
	{ 0x7357, 0x735b, PDF_CMAP_TABLE, 6454 },
	{ 0x735d, 0x735d, PDF_CMAP_SINGLE, 10605 },
	{ 0x735e, 0x7360, PDF_CMAP_RANGE, 10602 },
	{ 0x7361, 0x7362, PDF_CMAP_TABLE, 6459 },
	{ 0x7365, 0x7365, PDF_CMAP_SINGLE, 11208 },
	{ 0x7366, 0x7367, PDF_CMAP_RANGE, 11205 },
	{ 0x7368, 0x736c, PDF_CMAP_TABLE, 6461 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 11683 },
	{ 0x7370, 0x7378, PDF_CMAP_TABLE, 6466 },
	{ 0x737a, 0x738b, PDF_CMAP_TABLE, 6475 },
	{ 0x738e, 0x738f, PDF_CMAP_TABLE, 6493 },
	{ 0x7392, 0x7392, PDF_CMAP_SINGLE, 6353 },
	{ 0x7393, 0x7394, PDF_CMAP_RANGE, 6351 },
	{ 0x7395, 0x7398, PDF_CMAP_TABLE, 6495 },
	{ 0x739c, 0x73a2, PDF_CMAP_TABLE, 6499 },
	{ 0x73a4, 0x73ad, PDF_CMAP_TABLE, 6506 },
	{ 0x73b2, 0x73b9, PDF_CMAP_TABLE, 6516 },
	{ 0x73bb, 0x73bc, PDF_CMAP_TABLE, 6524 },
	{ 0x73be, 0x73c0, PDF_CMAP_TABLE, 6526 },
	{ 0x73c2, 0x73c3, PDF_CMAP_TABLE, 6529 },
	{ 0x73c5, 0x73d0, PDF_CMAP_TABLE, 6531 },
	{ 0x73d2, 0x73d6, PDF_CMAP_TABLE, 6543 },
	{ 0x73d7, 0x73d8, PDF_CMAP_RANGE, 7495 },
	{ 0x73d9, 0x73de, PDF_CMAP_TABLE, 6548 },
	{ 0x73e0, 0x73eb, PDF_CMAP_TABLE, 6554 },
	{ 0x73ed, 0x73ef, PDF_CMAP_TABLE, 6566 },
	{ 0x73f3, 0x7401, PDF_CMAP_TABLE, 6569 },
	{ 0x7403, 0x740d, PDF_CMAP_TABLE, 6584 },
	{ 0x7411, 0x7412, PDF_CMAP_TABLE, 6595 },
	{ 0x7414, 0x7417, PDF_CMAP_TABLE, 6597 },
	{ 0x7419, 0x741d, PDF_CMAP_TABLE, 6601 },
	{ 0x741e, 0x741f, PDF_CMAP_RANGE, 18164 },
	{ 0x7420, 0x7426, PDF_CMAP_TABLE, 6606 },
	{ 0x7428, 0x7434, PDF_CMAP_TABLE, 6613 },
	{ 0x7435, 0x7436, PDF_CMAP_RANGE, 3292 },
	{ 0x7437, 0x743a, PDF_CMAP_TABLE, 6626 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 15071 },
	{ 0x743f, 0x7449, PDF_CMAP_TABLE, 6630 },
	{ 0x744a, 0x744b, PDF_CMAP_RANGE, 9371 },
	{ 0x744d, 0x744e, PDF_CMAP_TABLE, 6641 },
	{ 0x744f, 0x7450, PDF_CMAP_RANGE, 9377 },
	{ 0x7451, 0x7457, PDF_CMAP_TABLE, 6643 },
	{ 0x7459, 0x745a, PDF_CMAP_TABLE, 6650 },
	{ 0x745b, 0x745c, PDF_CMAP_RANGE, 3715 },
	{ 0x745d, 0x7460, PDF_CMAP_TABLE, 6652 },
	{ 0x7462, 0x7465, PDF_CMAP_TABLE, 6656 },
	{ 0x7467, 0x7476, PDF_CMAP_TABLE, 6660 },
	{ 0x7479, 0x747a, PDF_CMAP_TABLE, 6676 },
	{ 0x747c, 0x7483, PDF_CMAP_TABLE, 6678 },
	{ 0x7485, 0x7488, PDF_CMAP_TABLE, 6686 },
	{ 0x7489, 0x748a, PDF_CMAP_RANGE, 10612 },
	{ 0x748b, 0x748d, PDF_CMAP_TABLE, 6690 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 11688 },
	{ 0x7492, 0x7492, PDF_CMAP_SINGLE, 11215 },
	{ 0x7494, 0x7495, PDF_CMAP_TABLE, 6693 },
	{ 0x7497, 0x749c, PDF_CMAP_TABLE, 6695 },
	{ 0x749e, 0x74a1, PDF_CMAP_TABLE, 6701 },
	{ 0x74a3, 0x74ab, PDF_CMAP_TABLE, 6705 },
	{ 0x74ad, 0x74ad, PDF_CMAP_SINGLE, 11690 },
	{ 0x74af, 0x74b2, PDF_CMAP_TABLE, 6714 },
	{ 0x74b4, 0x74b8, PDF_CMAP_TABLE, 6718 },
	{ 0x74ba, 0x74bb, PDF_CMAP_TABLE, 6723 },
	{ 0x74bd, 0x74c3, PDF_CMAP_TABLE, 6725 },
	{ 0x74c5, 0x74c5, PDF_CMAP_SINGLE, 12481 },
	{ 0x74c8, 0x74c8, PDF_CMAP_SINGLE, 17077 },
	{ 0x74ca, 0x74cc, PDF_CMAP_TABLE, 6732 },
	{ 0x74cf, 0x74d0, PDF_CMAP_TABLE, 6735 },
	{ 0x74d3, 0x74dd, PDF_CMAP_TABLE, 6737 },
	{ 0x74de, 0x74df, PDF_CMAP_RANGE, 7498 },
	{ 0x74e0, 0x74e9, PDF_CMAP_TABLE, 6748 },
	{ 0x74ec, 0x74ec, PDF_CMAP_SINGLE, 7054 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 7055 },
	{ 0x74f0, 0x74f2, PDF_CMAP_TABLE, 6758 },
	{ 0x74f4, 0x74f5, PDF_CMAP_RANGE, 7500 },
	{ 0x74f6, 0x74f7, PDF_CMAP_RANGE, 2855 },
	{ 0x74f8, 0x74f8, PDF_CMAP_SINGLE, 16372 },
	{ 0x74fb, 0x74fb, PDF_CMAP_SINGLE, 8736 },
	{ 0x74fd, 0x7500, PDF_CMAP_TABLE, 6761 },
	{ 0x7502, 0x7503, PDF_CMAP_RANGE, 10035 },
	{ 0x7504, 0x7505, PDF_CMAP_TABLE, 6765 },
	{ 0x7507, 0x7508, PDF_CMAP_TABLE, 6767 },
	{ 0x750b, 0x750b, PDF_CMAP_SINGLE, 11218 },
	{ 0x750c, 0x750d, PDF_CMAP_RANGE, 4864 },
	{ 0x750e, 0x750f, PDF_CMAP_TABLE, 6769 },
	{ 0x7510, 0x7512, PDF_CMAP_RANGE, 11694 },
	{ 0x7513, 0x751a, PDF_CMAP_TABLE, 6771 },
	{ 0x751c, 0x751f, PDF_CMAP_TABLE, 6779 },
	{ 0x7521, 0x7522, PDF_CMAP_TABLE, 6783 },
	{ 0x7525, 0x7526, PDF_CMAP_RANGE, 3299 },
	{ 0x7528, 0x7529, PDF_CMAP_RANGE, 860 },
	{ 0x752a, 0x752f, PDF_CMAP_TABLE, 6785 },
	{ 0x7530, 0x7533, PDF_CMAP_RANGE, 862 },
	{ 0x7534, 0x7535, PDF_CMAP_TABLE, 6791 },
	{ 0x7537, 0x7538, PDF_CMAP_RANGE, 1266 },
	{ 0x7539, 0x753b, PDF_CMAP_TABLE, 6793 },
	{ 0x753d, 0x753e, PDF_CMAP_TABLE, 6796 },
	{ 0x753f, 0x7540, PDF_CMAP_RANGE, 6671 },
	{ 0x7542, 0x7542, PDF_CMAP_SINGLE, 17357 },
	{ 0x7546, 0x7546, PDF_CMAP_SINGLE, 16192 },
	{ 0x7547, 0x7548, PDF_CMAP_RANGE, 7057 },
	{ 0x754a, 0x754f, PDF_CMAP_TABLE, 6798 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 15614 },
	{ 0x7553, 0x7555, PDF_CMAP_TABLE, 6804 },
	{ 0x7559, 0x755d, PDF_CMAP_TABLE, 6807 },
	{ 0x755f, 0x7560, PDF_CMAP_TABLE, 6812 },
	{ 0x7562, 0x7564, PDF_CMAP_TABLE, 6814 },
	{ 0x7565, 0x7566, PDF_CMAP_RANGE, 2859 },
	{ 0x7567, 0x7567, PDF_CMAP_SINGLE, 17088 },
	{ 0x756a, 0x7570, PDF_CMAP_TABLE, 6817 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 18174 },
	{ 0x7576, 0x757a, PDF_CMAP_TABLE, 6824 },
	{ 0x757d, 0x7580, PDF_CMAP_TABLE, 6829 },
	{ 0x7583, 0x7584, PDF_CMAP_TABLE, 6833 },
	{ 0x7586, 0x7587, PDF_CMAP_TABLE, 6835 },
	{ 0x758a, 0x7592, PDF_CMAP_TABLE, 6837 },
	{ 0x7594, 0x7595, PDF_CMAP_RANGE, 6356 },
	{ 0x7598, 0x7598, PDF_CMAP_SINGLE, 6675 },
	{ 0x7599, 0x759a, PDF_CMAP_RANGE, 1623 },
	{ 0x759d, 0x759e, PDF_CMAP_TABLE, 6846 },
	{ 0x75a2, 0x75a3, PDF_CMAP_RANGE, 1960 },
	{ 0x75a4, 0x75a5, PDF_CMAP_RANGE, 1958 },
	{ 0x75a7, 0x75a7, PDF_CMAP_SINGLE, 7059 },
	{ 0x75aa, 0x75ab, PDF_CMAP_TABLE, 6848 },
	{ 0x75b0, 0x75b1, PDF_CMAP_TABLE, 6850 },
	{ 0x75b2, 0x75b3, PDF_CMAP_RANGE, 2361 },
	{ 0x75b4, 0x75b6, PDF_CMAP_TABLE, 6852 },
	{ 0x75b8, 0x75c5, PDF_CMAP_TABLE, 6855 },
	{ 0x75c7, 0x75c8, PDF_CMAP_TABLE, 6869 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 2867 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 8099 },
	{ 0x75cd, 0x75d2, PDF_CMAP_TABLE, 6871 },
	{ 0x75d4, 0x75d5, PDF_CMAP_RANGE, 2864 },
	{ 0x75d7, 0x75e4, PDF_CMAP_TABLE, 6877 },
	{ 0x75e6, 0x75e7, PDF_CMAP_TABLE, 6891 },
	{ 0x75ed, 0x75ed, PDF_CMAP_SINGLE, 9403 },
	{ 0x75ef, 0x7603, PDF_CMAP_TABLE, 6893 },
	{ 0x7607, 0x760d, PDF_CMAP_TABLE, 6914 },
	{ 0x760f, 0x7611, PDF_CMAP_TABLE, 6921 },
	{ 0x7613, 0x7616, PDF_CMAP_TABLE, 6924 },
	{ 0x7619, 0x7620, PDF_CMAP_TABLE, 6928 },
	{ 0x7621, 0x7622, PDF_CMAP_RANGE, 4529 },
	{ 0x7623, 0x7629, PDF_CMAP_TABLE, 6936 },
	{ 0x762c, 0x762d, PDF_CMAP_TABLE, 6943 },
	{ 0x762f, 0x7635, PDF_CMAP_TABLE, 6945 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 4867 },
	{ 0x763a, 0x763d, PDF_CMAP_TABLE, 6952 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 16102 },
	{ 0x7642, 0x7643, PDF_CMAP_TABLE, 6956 },
	{ 0x7646, 0x7647, PDF_CMAP_TABLE, 6958 },
	{ 0x7648, 0x7649, PDF_CMAP_RANGE, 11700 },
	{ 0x764c, 0x7654, PDF_CMAP_TABLE, 6960 },
	{ 0x7656, 0x765a, PDF_CMAP_TABLE, 6969 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 12128 },
	{ 0x765f, 0x7662, PDF_CMAP_TABLE, 6974 },
	{ 0x7664, 0x7667, PDF_CMAP_TABLE, 6978 },
	{ 0x7669, 0x766a, PDF_CMAP_TABLE, 6982 },
	{ 0x766c, 0x7670, PDF_CMAP_TABLE, 6984 },
	{ 0x7671, 0x7672, PDF_CMAP_RANGE, 5923 },
	{ 0x7673, 0x7676, PDF_CMAP_TABLE, 6989 },
	{ 0x7678, 0x767a, PDF_CMAP_TABLE, 6993 },
	{ 0x767b, 0x767c, PDF_CMAP_RANGE, 3310 },
	{ 0x767d, 0x7682, PDF_CMAP_TABLE, 6996 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 1625 },
	{ 0x7686, 0x7688, PDF_CMAP_RANGE, 1963 },
	{ 0x7689, 0x768c, PDF_CMAP_TABLE, 7002 },
	{ 0x768e, 0x7690, PDF_CMAP_TABLE, 7006 },
	{ 0x7692, 0x7693, PDF_CMAP_TABLE, 7009 },
	{ 0x7695, 0x7696, PDF_CMAP_TABLE, 7011 },
	{ 0x7699, 0x769b, PDF_CMAP_TABLE, 7013 },
	{ 0x769c, 0x769e, PDF_CMAP_RANGE, 10633 },
	{ 0x76a1, 0x76a1, PDF_CMAP_SINGLE, 17109 },
	{ 0x76a4, 0x76a6, PDF_CMAP_TABLE, 7016 },
	{ 0x76aa, 0x76ab, PDF_CMAP_TABLE, 7019 },
	{ 0x76ad, 0x76b0, PDF_CMAP_TABLE, 7021 },
	{ 0x76b4, 0x76b5, PDF_CMAP_TABLE, 7025 },
	{ 0x76b7, 0x76b8, PDF_CMAP_TABLE, 7027 },
	{ 0x76ba, 0x76bb, PDF_CMAP_TABLE, 7029 },
	{ 0x76bd, 0x76bf, PDF_CMAP_TABLE, 7031 },
	{ 0x76c2, 0x76c6, PDF_CMAP_TABLE, 7034 },
	{ 0x76c8, 0x76ca, PDF_CMAP_TABLE, 7039 },
	{ 0x76cc, 0x76cc, PDF_CMAP_SINGLE, 17112 },
	{ 0x76cd, 0x76ce, PDF_CMAP_RANGE, 2371 },
	{ 0x76d2, 0x76d4, PDF_CMAP_TABLE, 7042 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 16213 },
	{ 0x76da, 0x76dd, PDF_CMAP_TABLE, 7045 },
	{ 0x76de, 0x76df, PDF_CMAP_RANGE, 3728 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 4152 },
	{ 0x76e3, 0x76e7, PDF_CMAP_TABLE, 7049 },
	{ 0x76e9, 0x76ea, PDF_CMAP_TABLE, 7054 },
	{ 0x76ec, 0x76f5, PDF_CMAP_TABLE, 7056 },
	{ 0x76f7, 0x76fc, PDF_CMAP_TABLE, 7066 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 1975 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 1970 },
	{ 0x7703, 0x7705, PDF_CMAP_RANGE, 7064 },
	{ 0x7707, 0x770c, PDF_CMAP_TABLE, 7072 },
	{ 0x770e, 0x770f, PDF_CMAP_RANGE, 15106 },
	{ 0x7710, 0x7713, PDF_CMAP_TABLE, 7078 },
	{ 0x7715, 0x7715, PDF_CMAP_SINGLE, 7522 },
	{ 0x7719, 0x771a, PDF_CMAP_RANGE, 7523 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 7516 },
	{ 0x771d, 0x771e, PDF_CMAP_TABLE, 7082 },
	{ 0x771f, 0x7720, PDF_CMAP_RANGE, 2374 },
	{ 0x7722, 0x7729, PDF_CMAP_TABLE, 7084 },
	{ 0x772b, 0x772b, PDF_CMAP_SINGLE, 16371 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 8108 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 8107 },
	{ 0x7731, 0x7732, PDF_CMAP_RANGE, 8109 },
	{ 0x7733, 0x773e, PDF_CMAP_TABLE, 7092 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 17120 },
	{ 0x7743, 0x7745, PDF_CMAP_TABLE, 7104 },
	{ 0x7746, 0x7747, PDF_CMAP_RANGE, 8751 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 8756 },
	{ 0x774b, 0x774c, PDF_CMAP_RANGE, 8758 },
	{ 0x774d, 0x774f, PDF_CMAP_TABLE, 7107 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 9412 },
	{ 0x7754, 0x7756, PDF_CMAP_TABLE, 7110 },
	{ 0x7758, 0x775c, PDF_CMAP_TABLE, 7113 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 3733 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 9410 },
	{ 0x7761, 0x7763, PDF_CMAP_TABLE, 7118 },
	{ 0x7765, 0x776f, PDF_CMAP_TABLE, 7121 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14699 },
	{ 0x7777, 0x7785, PDF_CMAP_TABLE, 7132 },
	{ 0x7787, 0x7789, PDF_CMAP_TABLE, 7147 },
	{ 0x778b, 0x778f, PDF_CMAP_TABLE, 7150 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 4537 },
	{ 0x7793, 0x7793, PDF_CMAP_SINGLE, 14381 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 11238 },
	{ 0x7797, 0x779d, PDF_CMAP_TABLE, 7155 },
	{ 0x779e, 0x779f, PDF_CMAP_RANGE, 4872 },
	{ 0x77a0, 0x77a1, PDF_CMAP_TABLE, 7162 },
	{ 0x77a2, 0x77a3, PDF_CMAP_RANGE, 11236 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 4874 },
	{ 0x77a7, 0x77a8, PDF_CMAP_TABLE, 7164 },
	{ 0x77aa, 0x77ad, PDF_CMAP_TABLE, 7166 },
	{ 0x77af, 0x77b7, PDF_CMAP_TABLE, 7170 },
	{ 0x77b9, 0x77ba, PDF_CMAP_TABLE, 7179 },
	{ 0x77bb, 0x77bc, PDF_CMAP_RANGE, 5372 },
	{ 0x77bd, 0x77bf, PDF_CMAP_TABLE, 7181 },
	{ 0x77c2, 0x77c5, PDF_CMAP_TABLE, 7184 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5534 },
	{ 0x77c9, 0x77ca, PDF_CMAP_RANGE, 12486 },
	{ 0x77cb, 0x77cd, PDF_CMAP_TABLE, 7188 },
	{ 0x77ce, 0x77cf, PDF_CMAP_RANGE, 12799 },
	{ 0x77d0, 0x77d0, PDF_CMAP_SINGLE, 13029 },
	{ 0x77d3, 0x77d5, PDF_CMAP_TABLE, 7191 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5925 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 13523 },
	{ 0x77da, 0x77dc, PDF_CMAP_TABLE, 7194 },
	{ 0x77de, 0x77de, PDF_CMAP_SINGLE, 8760 },
	{ 0x77e0, 0x77e0, PDF_CMAP_SINGLE, 9420 },
	{ 0x77e2, 0x77e3, PDF_CMAP_TABLE, 7197 },
	{ 0x77e5, 0x77e6, PDF_CMAP_TABLE, 7199 },
	{ 0x77e7, 0x77e8, PDF_CMAP_RANGE, 7071 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 2377 },
	{ 0x77ec, 0x77f4, PDF_CMAP_TABLE, 7201 },
	{ 0x77f7, 0x77fe, PDF_CMAP_TABLE, 7210 },
	{ 0x7802, 0x7803, PDF_CMAP_TABLE, 7218 },
	{ 0x7805, 0x7806, PDF_CMAP_TABLE, 7220 },
	{ 0x7808, 0x7809, PDF_CMAP_TABLE, 7222 },
	{ 0x780c, 0x780d, PDF_CMAP_RANGE, 1981 },
	{ 0x780e, 0x7810, PDF_CMAP_TABLE, 7224 },
	{ 0x7811, 0x7812, PDF_CMAP_RANGE, 7074 },
	{ 0x7813, 0x7814, PDF_CMAP_TABLE, 7227 },
	{ 0x7818, 0x7818, PDF_CMAP_SINGLE, 18195 },
	{ 0x781c, 0x7823, PDF_CMAP_TABLE, 7229 },
	{ 0x7825, 0x7835, PDF_CMAP_TABLE, 7237 },
	{ 0x7837, 0x7839, PDF_CMAP_TABLE, 7254 },
	{ 0x783c, 0x783d, PDF_CMAP_TABLE, 7257 },
	{ 0x7842, 0x7845, PDF_CMAP_TABLE, 7259 },
	{ 0x7847, 0x784e, PDF_CMAP_TABLE, 7263 },
	{ 0x7850, 0x7854, PDF_CMAP_TABLE, 7271 },
	{ 0x785c, 0x785e, PDF_CMAP_TABLE, 7276 },
	{ 0x7860, 0x7860, PDF_CMAP_SINGLE, 8762 },
	{ 0x7862, 0x7862, PDF_CMAP_SINGLE, 8774 },
	{ 0x7864, 0x7865, PDF_CMAP_RANGE, 8763 },
	{ 0x7866, 0x7866, PDF_CMAP_SINGLE, 18199 },
	{ 0x7868, 0x7871, PDF_CMAP_TABLE, 7279 },
	{ 0x7879, 0x787c, PDF_CMAP_TABLE, 7289 },
	{ 0x787e, 0x7881, PDF_CMAP_TABLE, 7293 },
	{ 0x7883, 0x7884, PDF_CMAP_TABLE, 7297 },
	{ 0x7885, 0x7886, PDF_CMAP_RANGE, 9427 },
	{ 0x7887, 0x7889, PDF_CMAP_TABLE, 7299 },
	{ 0x788c, 0x788f, PDF_CMAP_TABLE, 7302 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3750 },
	{ 0x7893, 0x7896, PDF_CMAP_TABLE, 7306 },
	{ 0x7897, 0x7898, PDF_CMAP_RANGE, 3745 },
	{ 0x7899, 0x789a, PDF_CMAP_TABLE, 7310 },
	{ 0x789e, 0x78a5, PDF_CMAP_TABLE, 7312 },
	{ 0x78a7, 0x78ad, PDF_CMAP_TABLE, 7320 },
	{ 0x78af, 0x78b4, PDF_CMAP_TABLE, 7327 },
	{ 0x78b6, 0x78b6, PDF_CMAP_SINGLE, 15468 },
	{ 0x78b8, 0x78bc, PDF_CMAP_TABLE, 7333 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 4543 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 4158 },
	{ 0x78c3, 0x78c4, PDF_CMAP_RANGE, 10649 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 4540 },
	{ 0x78c7, 0x78d5, PDF_CMAP_TABLE, 7338 },
	{ 0x78d7, 0x78d8, PDF_CMAP_TABLE, 7353 },
	{ 0x78da, 0x78db, PDF_CMAP_TABLE, 7355 },
	{ 0x78dd, 0x78de, PDF_CMAP_TABLE, 7357 },
	{ 0x78df, 0x78e0, PDF_CMAP_RANGE, 11251 },
	{ 0x78e1, 0x78e2, PDF_CMAP_RANGE, 11248 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 11246 },
	{ 0x78e5, 0x78e5, PDF_CMAP_SINGLE, 11243 },
	{ 0x78e7, 0x78ea, PDF_CMAP_TABLE, 7359 },
	{ 0x78ec, 0x78f5, PDF_CMAP_TABLE, 7363 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 5151 },
	{ 0x78f9, 0x78fa, PDF_CMAP_TABLE, 7373 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 11717 },
	{ 0x78fd, 0x78ff, PDF_CMAP_TABLE, 7375 },
	{ 0x7901, 0x7902, PDF_CMAP_TABLE, 7378 },
	{ 0x7904, 0x7906, PDF_CMAP_TABLE, 7380 },
	{ 0x7909, 0x7909, PDF_CMAP_SINGLE, 12143 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 12140 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 5374 },
	{ 0x7910, 0x7912, PDF_CMAP_TABLE, 7383 },
	{ 0x7913, 0x7914, PDF_CMAP_RANGE, 12141 },
	{ 0x7917, 0x7917, PDF_CMAP_SINGLE, 12494 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5535 },
	{ 0x791b, 0x791e, PDF_CMAP_TABLE, 7386 },
	{ 0x7921, 0x7921, PDF_CMAP_SINGLE, 12492 },
	{ 0x7923, 0x7926, PDF_CMAP_TABLE, 7390 },
	{ 0x7927, 0x7928, PDF_CMAP_RANGE, 12805 },
	{ 0x7929, 0x792f, PDF_CMAP_TABLE, 7394 },
	{ 0x7931, 0x7936, PDF_CMAP_TABLE, 7401 },
	{ 0x7938, 0x793f, PDF_CMAP_TABLE, 7407 },
	{ 0x7940, 0x7941, PDF_CMAP_RANGE, 1632 },
	{ 0x7942, 0x7942, PDF_CMAP_SINGLE, 6687 },
	{ 0x7944, 0x794c, PDF_CMAP_TABLE, 7415 },
	{ 0x794f, 0x7965, PDF_CMAP_TABLE, 7424 },
	{ 0x7967, 0x7968, PDF_CMAP_TABLE, 7447 },
	{ 0x7969, 0x796a, PDF_CMAP_RANGE, 8128 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 8131 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2884 },
	{ 0x7970, 0x7974, PDF_CMAP_TABLE, 7449 },
	{ 0x7979, 0x797a, PDF_CMAP_TABLE, 7454 },
	{ 0x797c, 0x7983, PDF_CMAP_TABLE, 7456 },
	{ 0x7986, 0x7988, PDF_CMAP_TABLE, 7464 },
	{ 0x798a, 0x798b, PDF_CMAP_RANGE, 10070 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 4166 },
	{ 0x798e, 0x798f, PDF_CMAP_RANGE, 4164 },
	{ 0x7990, 0x799d, PDF_CMAP_TABLE, 7467 },
	{ 0x799f, 0x79a2, PDF_CMAP_TABLE, 7481 },
	{ 0x79a4, 0x79ae, PDF_CMAP_TABLE, 7485 },
	{ 0x79b0, 0x79b4, PDF_CMAP_TABLE, 7496 },
	{ 0x79b6, 0x79b8, PDF_CMAP_TABLE, 7501 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 1987 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 8133 },
	{ 0x79bd, 0x79c1, PDF_CMAP_TABLE, 7504 },
	{ 0x79c4, 0x79c6, PDF_CMAP_TABLE, 7509 },
	{ 0x79c8, 0x79c9, PDF_CMAP_TABLE, 7512 },
	{ 0x79cb, 0x79cf, PDF_CMAP_TABLE, 7514 },
	{ 0x79d1, 0x79d2, PDF_CMAP_RANGE, 1989 },
	{ 0x79d4, 0x79d6, PDF_CMAP_TABLE, 7519 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2404 },
	{ 0x79dc, 0x79e0, PDF_CMAP_TABLE, 7522 },
	{ 0x79e2, 0x79e4, PDF_CMAP_TABLE, 7527 },
	{ 0x79e6, 0x79e7, PDF_CMAP_TABLE, 7530 },
	{ 0x79e9, 0x79ea, PDF_CMAP_TABLE, 7532 },
	{ 0x79eb, 0x79ec, PDF_CMAP_RANGE, 7547 },
	{ 0x79ed, 0x79ee, PDF_CMAP_TABLE, 7534 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 15143 },
	{ 0x79f4, 0x79f4, PDF_CMAP_SINGLE, 15138 },
	{ 0x79f6, 0x79f7, PDF_CMAP_RANGE, 8136 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 8135 },
	{ 0x79fa, 0x79fb, PDF_CMAP_TABLE, 7536 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 3325 },
	{ 0x7a02, 0x7a06, PDF_CMAP_TABLE, 7538 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 3322 },
	{ 0x7a0a, 0x7a0d, PDF_CMAP_TABLE, 7543 },
	{ 0x7a10, 0x7a15, PDF_CMAP_TABLE, 7547 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 9444 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 9441 },
	{ 0x7a1a, 0x7a1c, PDF_CMAP_TABLE, 7553 },
	{ 0x7a1e, 0x7a20, PDF_CMAP_TABLE, 7556 },
	{ 0x7a22, 0x7a22, PDF_CMAP_SINGLE, 9446 },
	{ 0x7a26, 0x7a26, PDF_CMAP_SINGLE, 10085 },
	{ 0x7a28, 0x7a28, PDF_CMAP_SINGLE, 10084 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 10080 },
	{ 0x7a2d, 0x7a31, PDF_CMAP_TABLE, 7559 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 4551 },
	{ 0x7a39, 0x7a40, PDF_CMAP_TABLE, 7564 },
	{ 0x7a43, 0x7a4c, PDF_CMAP_TABLE, 7572 },
	{ 0x7a4d, 0x7a4e, PDF_CMAP_RANGE, 4880 },
	{ 0x7a54, 0x7a54, PDF_CMAP_SINGLE, 11731 },
	{ 0x7a56, 0x7a58, PDF_CMAP_TABLE, 7582 },
	{ 0x7a5a, 0x7a5c, PDF_CMAP_TABLE, 7585 },
	{ 0x7a5f, 0x7a60, PDF_CMAP_TABLE, 7588 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5376 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 14134 },
	{ 0x7a67, 0x7a68, PDF_CMAP_RANGE, 12497 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5538 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 5537 },
	{ 0x7a6c, 0x7a6d, PDF_CMAP_RANGE, 12811 },
	{ 0x7a6e, 0x7a6e, PDF_CMAP_SINGLE, 12810 },
	{ 0x7a70, 0x7a71, PDF_CMAP_RANGE, 13209 },
	{ 0x7a74, 0x7a76, PDF_CMAP_TABLE, 7590 },
	{ 0x7a78, 0x7a7b, PDF_CMAP_TABLE, 7593 },
	{ 0x7a7d, 0x7a81, PDF_CMAP_TABLE, 7597 },
	{ 0x7a83, 0x7a8a, PDF_CMAP_TABLE, 7602 },
	{ 0x7a8b, 0x7a8c, PDF_CMAP_RANGE, 7559 },
	{ 0x7a8f, 0x7a92, PDF_CMAP_TABLE, 7610 },
	{ 0x7a94, 0x7a99, PDF_CMAP_TABLE, 7614 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 9452 },
	{ 0x7a9f, 0x7aa0, PDF_CMAP_RANGE, 3764 },
	{ 0x7aa2, 0x7aa3, PDF_CMAP_TABLE, 7620 },
	{ 0x7aa8, 0x7aaa, PDF_CMAP_TABLE, 7622 },
	{ 0x7aab, 0x7aac, PDF_CMAP_RANGE, 10087 },
	{ 0x7aae, 0x7ab8, PDF_CMAP_TABLE, 7625 },
	{ 0x7aba, 0x7abc, PDF_CMAP_TABLE, 7636 },
	{ 0x7abe, 0x7abf, PDF_CMAP_TABLE, 7639 },
	{ 0x7ac0, 0x7ac1, PDF_CMAP_RANGE, 11734 },
	{ 0x7ac2, 0x7ac3, PDF_CMAP_RANGE, 14139 },
	{ 0x7ac4, 0x7ac5, PDF_CMAP_RANGE, 5379 },
	{ 0x7ac7, 0x7acb, PDF_CMAP_TABLE, 7641 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 18212 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 7095 },
	{ 0x7ad3, 0x7ad3, PDF_CMAP_SINGLE, 16266 },
	{ 0x7ad8, 0x7add, PDF_CMAP_TABLE, 7646 },
	{ 0x7adf, 0x7ae0, PDF_CMAP_TABLE, 7652 },
	{ 0x7ae2, 0x7ae7, PDF_CMAP_TABLE, 7654 },
	{ 0x7ae9, 0x7aeb, PDF_CMAP_TABLE, 7660 },
	{ 0x7aed, 0x7aef, PDF_CMAP_TABLE, 7663 },
	{ 0x7af6, 0x7af7, PDF_CMAP_TABLE, 7666 },
	{ 0x7af9, 0x7afb, PDF_CMAP_TABLE, 7668 },
	{ 0x7afd, 0x7aff, PDF_CMAP_TABLE, 7671 },
	{ 0x7b00, 0x7b01, PDF_CMAP_RANGE, 7096 },
	{ 0x7b04, 0x7b06, PDF_CMAP_TABLE, 7674 },
	{ 0x7b08, 0x7b0c, PDF_CMAP_TABLE, 7677 },
	{ 0x7b0e, 0x7b14, PDF_CMAP_TABLE, 7682 },
	{ 0x7b18, 0x7b1b, PDF_CMAP_TABLE, 7689 },
	{ 0x7b1d, 0x7b20, PDF_CMAP_TABLE, 7693 },
	{ 0x7b22, 0x7b35, PDF_CMAP_TABLE, 7697 },
	{ 0x7b38, 0x7b39, PDF_CMAP_TABLE, 7717 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 8142 },
	{ 0x7b40, 0x7b40, PDF_CMAP_SINGLE, 8793 },
	{ 0x7b42, 0x7b52, PDF_CMAP_TABLE, 7719 },
	{ 0x7b54, 0x7b56, PDF_CMAP_TABLE, 7736 },
	{ 0x7b58, 0x7b58, PDF_CMAP_SINGLE, 8794 },
	{ 0x7b60, 0x7b67, PDF_CMAP_TABLE, 7739 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 9458 },
	{ 0x7b6c, 0x7b78, PDF_CMAP_TABLE, 7747 },
	{ 0x7b7b, 0x7b7b, PDF_CMAP_SINGLE, 18217 },
	{ 0x7b82, 0x7b82, PDF_CMAP_SINGLE, 10105 },
	{ 0x7b84, 0x7b85, PDF_CMAP_TABLE, 7760 },
	{ 0x7b87, 0x7b88, PDF_CMAP_TABLE, 7762 },
	{ 0x7b8a, 0x7b92, PDF_CMAP_TABLE, 7764 },
	{ 0x7b94, 0x7b99, PDF_CMAP_TABLE, 7773 },
	{ 0x7b9b, 0x7b9d, PDF_CMAP_TABLE, 7779 },
	{ 0x7ba0, 0x7ba4, PDF_CMAP_TABLE, 7782 },
	{ 0x7bac, 0x7bad, PDF_CMAP_TABLE, 7787 },
	{ 0x7baf, 0x7baf, PDF_CMAP_SINGLE, 10668 },
	{ 0x7bb1, 0x7bb2, PDF_CMAP_TABLE, 7789 },
	{ 0x7bb4, 0x7bb5, PDF_CMAP_TABLE, 7791 },
	{ 0x7bb7, 0x7bb9, PDF_CMAP_TABLE, 7793 },
	{ 0x7bbe, 0x7bbe, PDF_CMAP_SINGLE, 10665 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 7796 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 4557 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_RANGE, 4559 },
	{ 0x7bc9, 0x7bcc, PDF_CMAP_TABLE, 7798 },
	{ 0x7bce, 0x7bd0, PDF_CMAP_TABLE, 7802 },
	{ 0x7bd4, 0x7bd5, PDF_CMAP_TABLE, 7805 },
	{ 0x7bd8, 0x7beb, PDF_CMAP_TABLE, 7807 },
	{ 0x7bf0, 0x7bf1, PDF_CMAP_RANGE, 11753 },
	{ 0x7bf2, 0x7bf4, PDF_CMAP_TABLE, 7827 },
	{ 0x7bf7, 0x7c03, PDF_CMAP_TABLE, 7830 },
	{ 0x7c05, 0x7c07, PDF_CMAP_TABLE, 7843 },
	{ 0x7c09, 0x7c12, PDF_CMAP_TABLE, 7846 },
	{ 0x7c15, 0x7c15, PDF_CMAP_SINGLE, 16481 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 12152 },
	{ 0x7c1b, 0x7c23, PDF_CMAP_TABLE, 7856 },
	{ 0x7c25, 0x7c2d, PDF_CMAP_TABLE, 7865 },
	{ 0x7c30, 0x7c30, PDF_CMAP_SINGLE, 12161 },
	{ 0x7c33, 0x7c33, PDF_CMAP_SINGLE, 12499 },
	{ 0x7c35, 0x7c35, PDF_CMAP_SINGLE, 14169 },
	{ 0x7c37, 0x7c39, PDF_CMAP_TABLE, 7874 },
	{ 0x7c3b, 0x7c3d, PDF_CMAP_TABLE, 7877 },
	{ 0x7c3e, 0x7c3f, PDF_CMAP_RANGE, 5539 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 5544 },
	{ 0x7c42, 0x7c45, PDF_CMAP_TABLE, 7880 },
	{ 0x7c47, 0x7c4a, PDF_CMAP_TABLE, 7884 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 7888 },
	{ 0x7c50, 0x7c51, PDF_CMAP_TABLE, 7890 },
	{ 0x7c53, 0x7c54, PDF_CMAP_TABLE, 7892 },
	{ 0x7c56, 0x7c57, PDF_CMAP_TABLE, 7894 },
	{ 0x7c59, 0x7c5d, PDF_CMAP_TABLE, 7896 },
	{ 0x7c5f, 0x7c60, PDF_CMAP_TABLE, 7901 },
	{ 0x7c63, 0x7c67, PDF_CMAP_TABLE, 7903 },
	{ 0x7c69, 0x7c70, PDF_CMAP_TABLE, 7908 },
	{ 0x7c72, 0x7c75, PDF_CMAP_TABLE, 7916 },
	{ 0x7c78, 0x7c79, PDF_CMAP_RANGE, 7099 },
	{ 0x7c7a, 0x7c7e, PDF_CMAP_TABLE, 7920 },
	{ 0x7c7f, 0x7c81, PDF_CMAP_RANGE, 7101 },
	{ 0x7c83, 0x7c86, PDF_CMAP_TABLE, 7925 },
	{ 0x7c88, 0x7c8a, PDF_CMAP_TABLE, 7929 },
	{ 0x7c8c, 0x7c8e, PDF_CMAP_TABLE, 7932 },
	{ 0x7c91, 0x7c92, PDF_CMAP_TABLE, 7935 },
	{ 0x7c94, 0x7c98, PDF_CMAP_TABLE, 7937 },
	{ 0x7c9c, 0x7c9c, PDF_CMAP_SINGLE, 18222 },
	{ 0x7c9e, 0x7c9f, PDF_CMAP_TABLE, 7942 },
	{ 0x7ca1, 0x7ca3, PDF_CMAP_TABLE, 7944 },
	{ 0x7ca5, 0x7ca8, PDF_CMAP_TABLE, 7947 },
	{ 0x7cac, 0x7cac, PDF_CMAP_SINGLE, 14469 },
	{ 0x7cae, 0x7caf, PDF_CMAP_TABLE, 7951 },
	{ 0x7cb1, 0x7cb5, PDF_CMAP_TABLE, 7953 },
	{ 0x7cb8, 0x7cbc, PDF_CMAP_TABLE, 7958 },
	{ 0x7cbd, 0x7cbe, PDF_CMAP_RANGE, 4185 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 10107 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 16343 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 10672 },
	{ 0x7cc7, 0x7cce, PDF_CMAP_TABLE, 7963 },
	{ 0x7cd0, 0x7cd1, PDF_CMAP_RANGE, 11281 },
	{ 0x7cd2, 0x7cd4, PDF_CMAP_TABLE, 7971 },
	{ 0x7cd5, 0x7cd6, PDF_CMAP_RANGE, 4894 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 11280 },
	{ 0x7cd9, 0x7cda, PDF_CMAP_TABLE, 7974 },
	{ 0x7cdc, 0x7ce0, PDF_CMAP_TABLE, 7976 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 5169 },
	{ 0x7ce6, 0x7ce8, PDF_CMAP_TABLE, 7981 },
	{ 0x7cea, 0x7cea, PDF_CMAP_SINGLE, 12505 },
	{ 0x7cec, 0x7cee, PDF_CMAP_TABLE, 7984 },
	{ 0x7cef, 0x7cf0, PDF_CMAP_RANGE, 5677 },
	{ 0x7cf1, 0x7cf9, PDF_CMAP_TABLE, 7987 },
	{ 0x7cfb, 0x7cfe, PDF_CMAP_TABLE, 7996 },
	{ 0x7d00, 0x7d22, PDF_CMAP_TABLE, 8000 },
	{ 0x7d25, 0x7d25, PDF_CMAP_SINGLE, 16094 },
	{ 0x7d28, 0x7d29, PDF_CMAP_TABLE, 8035 },
	{ 0x7d2b, 0x7d2c, PDF_CMAP_TABLE, 8037 },
	{ 0x7d2e, 0x7d33, PDF_CMAP_TABLE, 8039 },
	{ 0x7d35, 0x7d36, PDF_CMAP_TABLE, 8045 },
	{ 0x7d38, 0x7d3d, PDF_CMAP_TABLE, 8047 },
	{ 0x7d3e, 0x7d3f, PDF_CMAP_RANGE, 8174 },
	{ 0x7d40, 0x7d47, PDF_CMAP_TABLE, 8053 },
	{ 0x7d4a, 0x7d4a, PDF_CMAP_SINGLE, 8176 },
	{ 0x7d4d, 0x7d56, PDF_CMAP_TABLE, 8061 },
	{ 0x7d58, 0x7d58, PDF_CMAP_SINGLE, 8800 },
	{ 0x7d5a, 0x7d5f, PDF_CMAP_TABLE, 8071 },
	{ 0x7d61, 0x7d63, PDF_CMAP_TABLE, 8077 },
	{ 0x7d66, 0x7d6b, PDF_CMAP_TABLE, 8080 },
	{ 0x7d6d, 0x7d73, PDF_CMAP_TABLE, 8086 },
	{ 0x7d79, 0x7d7d, PDF_CMAP_TABLE, 8093 },
	{ 0x7d7f, 0x7d81, PDF_CMAP_TABLE, 8098 },
	{ 0x7d83, 0x7d86, PDF_CMAP_TABLE, 8101 },
	{ 0x7d88, 0x7d89, PDF_CMAP_TABLE, 8105 },
	{ 0x7d8c, 0x7d8f, PDF_CMAP_TABLE, 8107 },
	{ 0x7d91, 0x7d94, PDF_CMAP_TABLE, 8111 },
	{ 0x7d96, 0x7d97, PDF_CMAP_TABLE, 8115 },
	{ 0x7d9c, 0x7da4, PDF_CMAP_TABLE, 8117 },
	{ 0x7da6, 0x7db5, PDF_CMAP_TABLE, 8126 },
	{ 0x7db7, 0x7dbc, PDF_CMAP_TABLE, 8142 },
	{ 0x7dbd, 0x7dbe, PDF_CMAP_RANGE, 4190 },
	{ 0x7dbf, 0x7dc2, PDF_CMAP_TABLE, 8148 },
	{ 0x7dc4, 0x7dc7, PDF_CMAP_TABLE, 8152 },
	{ 0x7dc9, 0x7dca, PDF_CMAP_TABLE, 8156 },
	{ 0x7dcb, 0x7dcc, PDF_CMAP_RANGE, 10122 },
	{ 0x7dcd, 0x7dd0, PDF_CMAP_TABLE, 8158 },
	{ 0x7dd2, 0x7dd4, PDF_CMAP_TABLE, 8162 },
	{ 0x7dd6, 0x7de1, PDF_CMAP_TABLE, 8165 },
	{ 0x7de3, 0x7dea, PDF_CMAP_TABLE, 8177 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 4570 },
	{ 0x7dee, 0x7df7, PDF_CMAP_TABLE, 8185 },
	{ 0x7df9, 0x7dfb, PDF_CMAP_TABLE, 8195 },
	{ 0x7dfd, 0x7dfe, PDF_CMAP_TABLE, 8198 },
	{ 0x7e03, 0x7e03, PDF_CMAP_SINGLE, 10682 },
	{ 0x7e07, 0x7e17, PDF_CMAP_TABLE, 8200 },
	{ 0x7e1a, 0x7e1e, PDF_CMAP_TABLE, 8217 },
	{ 0x7e1f, 0x7e20, PDF_CMAP_RANGE, 11287 },
	{ 0x7e21, 0x7e25, PDF_CMAP_TABLE, 8222 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 14207 },
	{ 0x7e29, 0x7e2b, PDF_CMAP_TABLE, 8227 },
	{ 0x7e2d, 0x7e49, PDF_CMAP_TABLE, 8230 },
	{ 0x7e4c, 0x7e4c, PDF_CMAP_SINGLE, 11769 },
	{ 0x7e50, 0x7e53, PDF_CMAP_TABLE, 8259 },
	{ 0x7e54, 0x7e55, PDF_CMAP_RANGE, 5388 },
	{ 0x7e56, 0x7e5a, PDF_CMAP_TABLE, 8263 },
	{ 0x7e5c, 0x7e5c, PDF_CMAP_SINGLE, 12162 },
	{ 0x7e5e, 0x7e63, PDF_CMAP_TABLE, 8268 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 18130 },
	{ 0x7e67, 0x7e68, PDF_CMAP_TABLE, 8274 },
	{ 0x7e69, 0x7e6a, PDF_CMAP_RANGE, 5548 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 5545 },
	{ 0x7e6d, 0x7e70, PDF_CMAP_TABLE, 8276 },
	{ 0x7e72, 0x7e82, PDF_CMAP_TABLE, 8280 },
	{ 0x7e86, 0x7e86, PDF_CMAP_SINGLE, 13040 },
	{ 0x7e87, 0x7e88, PDF_CMAP_RANGE, 13037 },
	{ 0x7e8a, 0x7e8f, PDF_CMAP_TABLE, 8297 },
	{ 0x7e91, 0x7e9c, PDF_CMAP_TABLE, 8303 },
	{ 0x7e9f, 0x7e9f, PDF_CMAP_SINGLE, 18619 },
	{ 0x7ea4, 0x7ea4, PDF_CMAP_SINGLE, 17755 },
	{ 0x7eac, 0x7eac, PDF_CMAP_SINGLE, 17756 },
	{ 0x7eba, 0x7eba, PDF_CMAP_SINGLE, 17757 },
	{ 0x7ec7, 0x7ec7, PDF_CMAP_SINGLE, 17758 },
	{ 0x7ecf, 0x7ecf, PDF_CMAP_SINGLE, 17759 },
	{ 0x7edf, 0x7edf, PDF_CMAP_SINGLE, 17760 },
	{ 0x7f06, 0x7f06, PDF_CMAP_SINGLE, 17761 },
	{ 0x7f36, 0x7f3a, PDF_CMAP_TABLE, 8315 },
	{ 0x7f3d, 0x7f3d, PDF_CMAP_SINGLE, 2913 },
	{ 0x7f3e, 0x7f3f, PDF_CMAP_RANGE, 8817 },
	{ 0x7f40, 0x7f41, PDF_CMAP_RANGE, 14208 },
	{ 0x7f43, 0x7f45, PDF_CMAP_TABLE, 8320 },
	{ 0x7f47, 0x7f55, PDF_CMAP_TABLE, 8323 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 7107 },
	{ 0x7f5b, 0x7f61, PDF_CMAP_TABLE, 8338 },
	{ 0x7f63, 0x7f63, PDF_CMAP_SINGLE, 8179 },
	{ 0x7f65, 0x7f66, PDF_CMAP_RANGE, 8819 },
	{ 0x7f67, 0x7f68, PDF_CMAP_RANGE, 9489 },
	{ 0x7f69, 0x7f6a, PDF_CMAP_RANGE, 3781 },
	{ 0x7f6b, 0x7f6e, PDF_CMAP_TABLE, 8345 },
	{ 0x7f70, 0x7f73, PDF_CMAP_TABLE, 8349 },
	{ 0x7f75, 0x7f7a, PDF_CMAP_TABLE, 8353 },
	{ 0x7f7b, 0x7f7c, PDF_CMAP_RANGE, 11303 },
	{ 0x7f7d, 0x7f7f, PDF_CMAP_TABLE, 8359 },
	{ 0x7f83, 0x7f83, PDF_CMAP_SINGLE, 12518 },
	{ 0x7f85, 0x7f8f, PDF_CMAP_TABLE, 8362 },
	{ 0x7f91, 0x7f97, PDF_CMAP_TABLE, 8373 },
	{ 0x7f9a, 0x7f9b, PDF_CMAP_TABLE, 8380 },
	{ 0x7f9c, 0x7f9d, PDF_CMAP_RANGE, 8181 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 2914 },
	{ 0x7fa0, 0x7fa1, PDF_CMAP_RANGE, 8822 },
	{ 0x7fa2, 0x7fa9, PDF_CMAP_TABLE, 8382 },
	{ 0x7fac, 0x7fc3, PDF_CMAP_TABLE, 8390 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 2428 },
	{ 0x7fc7, 0x7fc7, PDF_CMAP_SINGLE, 8189 },
	{ 0x7fc9, 0x7fc9, PDF_CMAP_SINGLE, 8191 },
	{ 0x7fca, 0x7fcb, PDF_CMAP_RANGE, 8184 },
	{ 0x7fcc, 0x7fcf, PDF_CMAP_TABLE, 8414 },
	{ 0x7fd0, 0x7fd1, PDF_CMAP_RANGE, 8187 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2918 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_RANGE, 3356 },
	{ 0x7fd7, 0x7fd7, PDF_CMAP_SINGLE, 8824 },
	{ 0x7fdb, 0x7fdc, PDF_CMAP_RANGE, 9495 },
	{ 0x7fdd, 0x7fdf, PDF_CMAP_TABLE, 8418 },
	{ 0x7fe0, 0x7fe1, PDF_CMAP_RANGE, 4207 },
	{ 0x7fe2, 0x7fe3, PDF_CMAP_RANGE, 10135 },
	{ 0x7fe5, 0x7fef, PDF_CMAP_TABLE, 8421 },
	{ 0x7ff0, 0x7ff1, PDF_CMAP_RANGE, 4907 },
	{ 0x7ff2, 0x7ff5, PDF_CMAP_TABLE, 8432 },
	{ 0x7ff7, 0x7ff8, PDF_CMAP_RANGE, 12175 },
	{ 0x7ff9, 0x7ffc, PDF_CMAP_TABLE, 8436 },
	{ 0x7ffd, 0x7ffe, PDF_CMAP_RANGE, 12521 },
	{ 0x7fff, 0x8008, PDF_CMAP_TABLE, 8440 },
	{ 0x800b, 0x800d, PDF_CMAP_TABLE, 8450 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 7112 },
	{ 0x8010, 0x8012, PDF_CMAP_TABLE, 8453 },
	{ 0x8014, 0x8019, PDF_CMAP_TABLE, 8456 },
	{ 0x801b, 0x8021, PDF_CMAP_TABLE, 8462 },
	{ 0x8024, 0x8026, PDF_CMAP_TABLE, 8469 },
	{ 0x8028, 0x802a, PDF_CMAP_TABLE, 8472 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 11781 },
	{ 0x802e, 0x8031, PDF_CMAP_TABLE, 8475 },
	{ 0x8033, 0x8037, PDF_CMAP_TABLE, 8479 },
	{ 0x8039, 0x8039, PDF_CMAP_SINGLE, 7605 },
	{ 0x803b, 0x803c, PDF_CMAP_RANGE, 14231 },
	{ 0x803d, 0x803f, PDF_CMAP_TABLE, 8484 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 8196 },
	{ 0x8046, 0x8048, PDF_CMAP_TABLE, 8487 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 2920 },
	{ 0x804f, 0x8050, PDF_CMAP_RANGE, 8826 },
	{ 0x8051, 0x8052, PDF_CMAP_TABLE, 8490 },
	{ 0x8054, 0x8054, PDF_CMAP_SINGLE, 18240 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 3787 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 3788 },
	{ 0x805a, 0x805e, PDF_CMAP_TABLE, 8492 },
	{ 0x8061, 0x8064, PDF_CMAP_TABLE, 8497 },
	{ 0x8066, 0x8067, PDF_CMAP_TABLE, 8501 },
	{ 0x806c, 0x806c, PDF_CMAP_SINGLE, 11310 },
	{ 0x806f, 0x8070, PDF_CMAP_TABLE, 8503 },
	{ 0x8071, 0x8072, PDF_CMAP_RANGE, 5193 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 5197 },
	{ 0x8075, 0x8079, PDF_CMAP_TABLE, 8505 },
	{ 0x807d, 0x8080, PDF_CMAP_TABLE, 8510 },
	{ 0x8082, 0x8082, PDF_CMAP_SINGLE, 7404 },
	{ 0x8084, 0x8087, PDF_CMAP_TABLE, 8514 },
	{ 0x8089, 0x808a, PDF_CMAP_TABLE, 8518 },
	{ 0x808b, 0x808c, PDF_CMAP_RANGE, 1027 },
	{ 0x808f, 0x8090, PDF_CMAP_TABLE, 8520 },
	{ 0x8092, 0x8093, PDF_CMAP_TABLE, 8522 },
	{ 0x8095, 0x8096, PDF_CMAP_TABLE, 8524 },
	{ 0x8098, 0x809d, PDF_CMAP_TABLE, 8526 },
	{ 0x809f, 0x809f, PDF_CMAP_SINGLE, 17782 },
	{ 0x80a1, 0x80a3, PDF_CMAP_TABLE, 8532 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 1645 },
	{ 0x80a7, 0x80a7, PDF_CMAP_SINGLE, 14240 },
	{ 0x80a9, 0x80ab, PDF_CMAP_TABLE, 8535 },
	{ 0x80ad, 0x80af, PDF_CMAP_TABLE, 8538 },
	{ 0x80b1, 0x80b2, PDF_CMAP_TABLE, 8541 },
	{ 0x80b4, 0x80b8, PDF_CMAP_TABLE, 8543 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 1644 },
	{ 0x80bc, 0x80bd, PDF_CMAP_TABLE, 8548 },
	{ 0x80c2, 0x80c2, PDF_CMAP_SINGLE, 7121 },
	{ 0x80c3, 0x80c4, PDF_CMAP_RANGE, 2014 },
	{ 0x80c5, 0x80ca, PDF_CMAP_TABLE, 8550 },
	{ 0x80cc, 0x80d1, PDF_CMAP_TABLE, 8556 },
	{ 0x80d4, 0x80de, PDF_CMAP_TABLE, 8562 },
	{ 0x80e0, 0x80e1, PDF_CMAP_TABLE, 8573 },
	{ 0x80e3, 0x80e6, PDF_CMAP_TABLE, 8575 },
	{ 0x80e9, 0x80e9, PDF_CMAP_SINGLE, 18244 },
	{ 0x80ec, 0x80ed, PDF_CMAP_TABLE, 8579 },
	{ 0x80ef, 0x80f6, PDF_CMAP_TABLE, 8581 },
	{ 0x80f8, 0x80fe, PDF_CMAP_TABLE, 8589 },
	{ 0x8100, 0x8103, PDF_CMAP_TABLE, 8596 },
	{ 0x8105, 0x810a, PDF_CMAP_TABLE, 8600 },
	{ 0x810c, 0x810c, PDF_CMAP_SINGLE, 18246 },
	{ 0x810e, 0x810e, PDF_CMAP_SINGLE, 18247 },
	{ 0x8112, 0x8112, PDF_CMAP_SINGLE, 18248 },
	{ 0x8114, 0x811b, PDF_CMAP_TABLE, 8606 },
	{ 0x811d, 0x811f, PDF_CMAP_TABLE, 8614 },
	{ 0x8121, 0x8125, PDF_CMAP_TABLE, 8617 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 8208 },
	{ 0x8129, 0x812d, PDF_CMAP_TABLE, 8622 },
	{ 0x812f, 0x8130, PDF_CMAP_TABLE, 8627 },
	{ 0x8132, 0x8132, PDF_CMAP_SINGLE, 15993 },
	{ 0x8134, 0x8134, PDF_CMAP_SINGLE, 17900 },
	{ 0x8137, 0x8137, PDF_CMAP_SINGLE, 15218 },
	{ 0x8139, 0x813a, PDF_CMAP_TABLE, 8629 },
	{ 0x813d, 0x813e, PDF_CMAP_TABLE, 8631 },
	{ 0x8142, 0x8144, PDF_CMAP_TABLE, 8633 },
	{ 0x8146, 0x8148, PDF_CMAP_TABLE, 8636 },
	{ 0x814a, 0x8156, PDF_CMAP_TABLE, 8639 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 18253 },
	{ 0x815b, 0x815c, PDF_CMAP_TABLE, 8652 },
	{ 0x815e, 0x815e, PDF_CMAP_SINGLE, 9507 },
	{ 0x8160, 0x8162, PDF_CMAP_TABLE, 8654 },
	{ 0x8164, 0x8167, PDF_CMAP_TABLE, 8657 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 9502 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 3797 },
	{ 0x816d, 0x8174, PDF_CMAP_TABLE, 8661 },
	{ 0x8176, 0x8178, PDF_CMAP_TABLE, 8669 },
	{ 0x8179, 0x817a, PDF_CMAP_RANGE, 3798 },
	{ 0x817c, 0x817c, PDF_CMAP_SINGLE, 18256 },
	{ 0x817f, 0x8180, PDF_CMAP_TABLE, 8672 },
	{ 0x8182, 0x8184, PDF_CMAP_TABLE, 8674 },
	{ 0x8186, 0x818d, PDF_CMAP_TABLE, 8677 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 4215 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 17350 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 10705 },
	{ 0x8197, 0x819a, PDF_CMAP_TABLE, 8685 },
	{ 0x819b, 0x819d, PDF_CMAP_RANGE, 4586 },
	{ 0x819e, 0x81a0, PDF_CMAP_TABLE, 8689 },
	{ 0x81a2, 0x81a3, PDF_CMAP_TABLE, 8692 },
	{ 0x81a5, 0x81ac, PDF_CMAP_TABLE, 8694 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 11313 },
	{ 0x81b0, 0x81b7, PDF_CMAP_TABLE, 8702 },
	{ 0x81b9, 0x81ca, PDF_CMAP_TABLE, 8710 },
	{ 0x81cc, 0x81cd, PDF_CMAP_TABLE, 8728 },
	{ 0x81cf, 0x81d0, PDF_CMAP_TABLE, 8730 },
	{ 0x81d1, 0x81d2, PDF_CMAP_RANGE, 12178 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 12525 },
	{ 0x81d7, 0x81db, PDF_CMAP_TABLE, 8732 },
	{ 0x81dd, 0x81df, PDF_CMAP_TABLE, 8737 },
	{ 0x81e0, 0x81e1, PDF_CMAP_RANGE, 13533 },
	{ 0x81e2, 0x81ea, PDF_CMAP_TABLE, 8740 },
	{ 0x81ec, 0x81ef, PDF_CMAP_TABLE, 8749 },
	{ 0x81f2, 0x81f4, PDF_CMAP_TABLE, 8753 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 15713 },
	{ 0x81f7, 0x81f9, PDF_CMAP_RANGE, 8840 },
	{ 0x81fa, 0x81fc, PDF_CMAP_TABLE, 8756 },
	{ 0x81fe, 0x8202, PDF_CMAP_TABLE, 8759 },
	{ 0x8204, 0x8205, PDF_CMAP_TABLE, 8764 },
	{ 0x8207, 0x820d, PDF_CMAP_TABLE, 8766 },
	{ 0x8210, 0x8212, PDF_CMAP_TABLE, 8773 },
	{ 0x8214, 0x8216, PDF_CMAP_TABLE, 8776 },
	{ 0x8218, 0x8218, PDF_CMAP_SINGLE, 14250 },
	{ 0x821a, 0x8222, PDF_CMAP_TABLE, 8779 },
	{ 0x8225, 0x8226, PDF_CMAP_TABLE, 8788 },
	{ 0x8228, 0x8229, PDF_CMAP_TABLE, 8790 },
	{ 0x822a, 0x822b, PDF_CMAP_RANGE, 2456 },
	{ 0x822c, 0x822d, PDF_CMAP_TABLE, 8792 },
	{ 0x822f, 0x822f, PDF_CMAP_SINGLE, 7614 },
	{ 0x8232, 0x823a, PDF_CMAP_TABLE, 8794 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 8844 },
	{ 0x823e, 0x8240, PDF_CMAP_TABLE, 8803 },
	{ 0x8242, 0x8242, PDF_CMAP_SINGLE, 9517 },
	{ 0x8244, 0x8245, PDF_CMAP_TABLE, 8806 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3802 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 9514 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 4225 },
	{ 0x824e, 0x8254, PDF_CMAP_TABLE, 8808 },
	{ 0x8255, 0x8257, PDF_CMAP_RANGE, 11324 },
	{ 0x8258, 0x8259, PDF_CMAP_RANGE, 4916 },
	{ 0x825a, 0x825c, PDF_CMAP_TABLE, 8815 },
	{ 0x825e, 0x825f, PDF_CMAP_TABLE, 8818 },
	{ 0x8261, 0x8266, PDF_CMAP_TABLE, 8820 },
	{ 0x8268, 0x8269, PDF_CMAP_RANGE, 12829 },
	{ 0x826b, 0x826f, PDF_CMAP_TABLE, 8826 },
	{ 0x8271, 0x8272, PDF_CMAP_TABLE, 8831 },
	{ 0x8274, 0x8280, PDF_CMAP_TABLE, 8833 },
	{ 0x8283, 0x8284, PDF_CMAP_RANGE, 6373 },
	{ 0x8285, 0x8285, PDF_CMAP_SINGLE, 6368 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 16690 },
	{ 0x828a, 0x828b, PDF_CMAP_TABLE, 8846 },
	{ 0x828d, 0x8294, PDF_CMAP_TABLE, 8848 },
	{ 0x8298, 0x829b, PDF_CMAP_TABLE, 8856 },
	{ 0x829d, 0x82b1, PDF_CMAP_TABLE, 8860 },
	{ 0x82b3, 0x82be, PDF_CMAP_TABLE, 8881 },
	{ 0x82c0, 0x82c0, PDF_CMAP_SINGLE, 6704 },
	{ 0x82c2, 0x82c4, PDF_CMAP_TABLE, 8893 },
	{ 0x82ca, 0x82ca, PDF_CMAP_SINGLE, 18266 },
	{ 0x82cf, 0x82d9, PDF_CMAP_TABLE, 8896 },
	{ 0x82db, 0x82dc, PDF_CMAP_TABLE, 8907 },
	{ 0x82de, 0x82e8, PDF_CMAP_TABLE, 8909 },
	{ 0x82ea, 0x82f7, PDF_CMAP_TABLE, 8920 },
	{ 0x82f9, 0x8309, PDF_CMAP_TABLE, 8934 },
	{ 0x830b, 0x830d, PDF_CMAP_TABLE, 8951 },
	{ 0x8316, 0x831e, PDF_CMAP_TABLE, 8954 },
	{ 0x8320, 0x8320, PDF_CMAP_SINGLE, 7637 },
	{ 0x8322, 0x8322, PDF_CMAP_SINGLE, 7627 },
	{ 0x8324, 0x832d, PDF_CMAP_TABLE, 8963 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 7639 },
	{ 0x8331, 0x833d, PDF_CMAP_TABLE, 8973 },
	{ 0x833f, 0x8345, PDF_CMAP_TABLE, 8986 },
	{ 0x8347, 0x8354, PDF_CMAP_TABLE, 8993 },
	{ 0x8356, 0x8357, PDF_CMAP_TABLE, 9007 },
	{ 0x8362, 0x8363, PDF_CMAP_TABLE, 9009 },
	{ 0x8366, 0x8366, PDF_CMAP_SINGLE, 15902 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 17765 },
	{ 0x8373, 0x8378, PDF_CMAP_TABLE, 9011 },
	{ 0x837a, 0x837a, PDF_CMAP_SINGLE, 8222 },
	{ 0x837b, 0x837c, PDF_CMAP_RANGE, 2948 },
	{ 0x837d, 0x837f, PDF_CMAP_TABLE, 9017 },
	{ 0x8381, 0x8381, PDF_CMAP_SINGLE, 8227 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 8234 },
	{ 0x8385, 0x839e, PDF_CMAP_TABLE, 9020 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 2946 },
	{ 0x83a2, 0x83ac, PDF_CMAP_TABLE, 9046 },
	{ 0x83ae, 0x83b0, PDF_CMAP_TABLE, 9057 },
	{ 0x83b9, 0x83b9, PDF_CMAP_SINGLE, 17063 },
	{ 0x83bd, 0x83cf, PDF_CMAP_TABLE, 9060 },
	{ 0x83d1, 0x83d1, PDF_CMAP_SINGLE, 8886 },
	{ 0x83d3, 0x83d9, PDF_CMAP_TABLE, 9079 },
	{ 0x83db, 0x83e5, PDF_CMAP_TABLE, 9086 },
	{ 0x83e7, 0x83f6, PDF_CMAP_TABLE, 9097 },
	{ 0x83f8, 0x83ff, PDF_CMAP_TABLE, 9113 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 8865 },
	{ 0x8403, 0x8407, PDF_CMAP_TABLE, 9121 },
	{ 0x8409, 0x8414, PDF_CMAP_TABLE, 9126 },
	{ 0x8416, 0x8416, PDF_CMAP_SINGLE, 16439 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 15994 },
	{ 0x841b, 0x841c, PDF_CMAP_TABLE, 9138 },
	{ 0x8420, 0x8421, PDF_CMAP_TABLE, 9140 },
	{ 0x8423, 0x8424, PDF_CMAP_TABLE, 9142 },
	{ 0x8426, 0x8426, PDF_CMAP_SINGLE, 18136 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 9549 },
	{ 0x842b, 0x8440, PDF_CMAP_TABLE, 9144 },
	{ 0x8442, 0x844e, PDF_CMAP_TABLE, 9166 },
	{ 0x8450, 0x8469, PDF_CMAP_TABLE, 9179 },
	{ 0x846b, 0x847a, PDF_CMAP_TABLE, 9205 },
	{ 0x847d, 0x8480, PDF_CMAP_TABLE, 9221 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 3803 },
	{ 0x8486, 0x8486, PDF_CMAP_SINGLE, 9529 },
	{ 0x8488, 0x8488, PDF_CMAP_SINGLE, 18274 },
	{ 0x848d, 0x8494, PDF_CMAP_TABLE, 9225 },
	{ 0x8496, 0x84a4, PDF_CMAP_TABLE, 9233 },
	{ 0x84a7, 0x84b2, PDF_CMAP_TABLE, 9248 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 10161 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 10181 },
	{ 0x84b8, 0x84c2, PDF_CMAP_TABLE, 9260 },
	{ 0x84c4, 0x84c7, PDF_CMAP_TABLE, 9271 },
	{ 0x84c9, 0x84d4, PDF_CMAP_TABLE, 9275 },
	{ 0x84d6, 0x84d7, PDF_CMAP_TABLE, 9287 },
	{ 0x84da, 0x84db, PDF_CMAP_TABLE, 9289 },
	{ 0x84de, 0x84de, PDF_CMAP_SINGLE, 14314 },
	{ 0x84e1, 0x84e2, PDF_CMAP_TABLE, 9291 },
	{ 0x84e4, 0x84e5, PDF_CMAP_TABLE, 9293 },
	{ 0x84e7, 0x84e8, PDF_CMAP_RANGE, 10755 },
	{ 0x84e9, 0x84ec, PDF_CMAP_TABLE, 9295 },
	{ 0x84ee, 0x84f4, PDF_CMAP_TABLE, 9299 },
	{ 0x84f6, 0x8500, PDF_CMAP_TABLE, 9306 },
	{ 0x8502, 0x8503, PDF_CMAP_TABLE, 9317 },
	{ 0x8505, 0x851a, PDF_CMAP_TABLE, 9319 },
	{ 0x851c, 0x8521, PDF_CMAP_TABLE, 9341 },
	{ 0x8523, 0x852b, PDF_CMAP_TABLE, 9347 },
	{ 0x852c, 0x852d, PDF_CMAP_RANGE, 4596 },
	{ 0x852e, 0x8531, PDF_CMAP_TABLE, 9356 },
	{ 0x8533, 0x8534, PDF_CMAP_TABLE, 9360 },
	{ 0x8538, 0x8538, PDF_CMAP_SINGLE, 18280 },
	{ 0x853b, 0x853b, PDF_CMAP_SINGLE, 10717 },
	{ 0x853d, 0x853e, PDF_CMAP_TABLE, 9362 },
	{ 0x8540, 0x854e, PDF_CMAP_TABLE, 9364 },
	{ 0x8551, 0x855b, PDF_CMAP_TABLE, 9379 },
	{ 0x855d, 0x855e, PDF_CMAP_TABLE, 9390 },
	{ 0x8560, 0x8567, PDF_CMAP_TABLE, 9392 },
	{ 0x8568, 0x8569, PDF_CMAP_RANGE, 4921 },
	{ 0x856a, 0x856e, PDF_CMAP_TABLE, 9400 },
	{ 0x856f, 0x8570, PDF_CMAP_RANGE, 18283 },
	{ 0x8571, 0x8571, PDF_CMAP_SINGLE, 11345 },
	{ 0x8573, 0x8573, PDF_CMAP_SINGLE, 14328 },
	{ 0x8575, 0x857c, PDF_CMAP_TABLE, 9405 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 5212 },
	{ 0x8580, 0x8591, PDF_CMAP_TABLE, 9413 },
	{ 0x8594, 0x8596, PDF_CMAP_TABLE, 9431 },
	{ 0x8598, 0x85a4, PDF_CMAP_TABLE, 9434 },
	{ 0x85a6, 0x85aa, PDF_CMAP_TABLE, 9447 },
	{ 0x85af, 0x85b1, PDF_CMAP_TABLE, 9452 },
	{ 0x85b3, 0x85ba, PDF_CMAP_TABLE, 9455 },
	{ 0x85bd, 0x85c9, PDF_CMAP_TABLE, 9463 },
	{ 0x85cb, 0x85cb, PDF_CMAP_SINGLE, 12194 },
	{ 0x85cd, 0x85d2, PDF_CMAP_TABLE, 9476 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 5559 },
	{ 0x85d7, 0x85da, PDF_CMAP_TABLE, 9482 },
	{ 0x85dc, 0x85e3, PDF_CMAP_TABLE, 9486 },
	{ 0x85e4, 0x85e5, PDF_CMAP_RANGE, 5560 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 12547 },
	{ 0x85e8, 0x85f2, PDF_CMAP_TABLE, 9494 },
	{ 0x85f6, 0x8602, PDF_CMAP_TABLE, 9505 },
	{ 0x8604, 0x8607, PDF_CMAP_TABLE, 9518 },
	{ 0x8609, 0x860d, PDF_CMAP_TABLE, 9522 },
	{ 0x8610, 0x8611, PDF_CMAP_TABLE, 9527 },
	{ 0x8614, 0x8614, PDF_CMAP_SINGLE, 18110 },
	{ 0x8616, 0x861c, PDF_CMAP_TABLE, 9529 },
	{ 0x861e, 0x862a, PDF_CMAP_TABLE, 9536 },
	{ 0x862c, 0x862f, PDF_CMAP_TABLE, 9549 },
	{ 0x8631, 0x8633, PDF_CMAP_TABLE, 9553 },
	{ 0x8634, 0x8635, PDF_CMAP_RANGE, 13223 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 13228 },
	{ 0x8638, 0x863c, PDF_CMAP_TABLE, 9556 },
	{ 0x863e, 0x8640, PDF_CMAP_TABLE, 9561 },
	{ 0x8642, 0x8643, PDF_CMAP_TABLE, 9564 },
	{ 0x8645, 0x8645, PDF_CMAP_SINGLE, 15206 },
	{ 0x8646, 0x8648, PDF_CMAP_RANGE, 13535 },
	{ 0x864b, 0x864e, PDF_CMAP_TABLE, 9566 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 2047 },
	{ 0x8652, 0x8656, PDF_CMAP_TABLE, 9570 },
	{ 0x8659, 0x8659, PDF_CMAP_SINGLE, 8251 },
	{ 0x865b, 0x865c, PDF_CMAP_TABLE, 9575 },
	{ 0x865e, 0x865f, PDF_CMAP_TABLE, 9577 },
	{ 0x8661, 0x8665, PDF_CMAP_TABLE, 9579 },
	{ 0x8667, 0x866c, PDF_CMAP_TABLE, 9584 },
	{ 0x866d, 0x866e, PDF_CMAP_RANGE, 6726 },
	{ 0x866f, 0x8674, PDF_CMAP_TABLE, 9590 },
	{ 0x8677, 0x8677, PDF_CMAP_SINGLE, 7162 },
	{ 0x8679, 0x867c, PDF_CMAP_TABLE, 9596 },
	{ 0x867e, 0x867e, PDF_CMAP_SINGLE, 14346 },
	{ 0x8685, 0x8687, PDF_CMAP_TABLE, 9600 },
	{ 0x868a, 0x868e, PDF_CMAP_TABLE, 9603 },
	{ 0x8690, 0x869a, PDF_CMAP_TABLE, 9608 },
	{ 0x869c, 0x869e, PDF_CMAP_TABLE, 9619 },
	{ 0x86a0, 0x86a5, PDF_CMAP_TABLE, 9622 },
	{ 0x86a7, 0x86aa, PDF_CMAP_TABLE, 9628 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 16134 },
	{ 0x86af, 0x86ba, PDF_CMAP_TABLE, 9632 },
	{ 0x86bb, 0x86bc, PDF_CMAP_RANGE, 8266 },
	{ 0x86bd, 0x86be, PDF_CMAP_RANGE, 8269 },
	{ 0x86bf, 0x86c9, PDF_CMAP_TABLE, 9644 },
	{ 0x86cb, 0x86cc, PDF_CMAP_TABLE, 9655 },
	{ 0x86d0, 0x86d1, PDF_CMAP_TABLE, 9657 },
	{ 0x86d3, 0x86d4, PDF_CMAP_TABLE, 9659 },
	{ 0x86d6, 0x86df, PDF_CMAP_TABLE, 9661 },
	{ 0x86e2, 0x86e4, PDF_CMAP_TABLE, 9671 },
	{ 0x86e6, 0x86e6, PDF_CMAP_SINGLE, 8902 },
	{ 0x86e8, 0x86ed, PDF_CMAP_TABLE, 9674 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 18289 },
	{ 0x86f5, 0x86fb, PDF_CMAP_TABLE, 9680 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 3828 },
	{ 0x8700, 0x8701, PDF_CMAP_TABLE, 9687 },
	{ 0x8702, 0x8703, PDF_CMAP_RANGE, 3830 },
	{ 0x8704, 0x870e, PDF_CMAP_TABLE, 9689 },
	{ 0x8711, 0x8713, PDF_CMAP_TABLE, 9700 },
	{ 0x8718, 0x871c, PDF_CMAP_TABLE, 9703 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 10197 },
	{ 0x8720, 0x8725, PDF_CMAP_TABLE, 9708 },
	{ 0x8726, 0x8727, PDF_CMAP_RANGE, 10216 },
	{ 0x8728, 0x872a, PDF_CMAP_TABLE, 9714 },
	{ 0x872c, 0x872e, PDF_CMAP_TABLE, 9717 },
	{ 0x8730, 0x8735, PDF_CMAP_TABLE, 9720 },
	{ 0x8737, 0x8738, PDF_CMAP_TABLE, 9726 },
	{ 0x873a, 0x873c, PDF_CMAP_TABLE, 9728 },
	{ 0x873e, 0x8743, PDF_CMAP_TABLE, 9731 },
	{ 0x8746, 0x8746, PDF_CMAP_SINGLE, 10205 },
	{ 0x874c, 0x8762, PDF_CMAP_TABLE, 9737 },
	{ 0x8763, 0x8764, PDF_CMAP_RANGE, 10768 },
	{ 0x8765, 0x8771, PDF_CMAP_TABLE, 9760 },
	{ 0x8773, 0x877b, PDF_CMAP_TABLE, 9773 },
	{ 0x877d, 0x877d, PDF_CMAP_SINGLE, 16453 },
	{ 0x8781, 0x8789, PDF_CMAP_TABLE, 9782 },
	{ 0x878b, 0x878d, PDF_CMAP_TABLE, 9791 },
	{ 0x878f, 0x878f, PDF_CMAP_SINGLE, 11363 },
	{ 0x8790, 0x8791, PDF_CMAP_RANGE, 11375 },
	{ 0x8792, 0x8794, PDF_CMAP_TABLE, 9794 },
	{ 0x8796, 0x8798, PDF_CMAP_TABLE, 9797 },
	{ 0x879a, 0x879f, PDF_CMAP_TABLE, 9800 },
	{ 0x87a2, 0x87a5, PDF_CMAP_TABLE, 9806 },
	{ 0x87a9, 0x87c6, PDF_CMAP_TABLE, 9810 },
	{ 0x87c8, 0x87cc, PDF_CMAP_TABLE, 9840 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 16365 },
	{ 0x87d1, 0x87d4, PDF_CMAP_TABLE, 9845 },
	{ 0x87d6, 0x87e8, PDF_CMAP_TABLE, 9849 },
	{ 0x87ea, 0x87ef, PDF_CMAP_TABLE, 9868 },
	{ 0x87f2, 0x87f5, PDF_CMAP_TABLE, 9874 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 12554 },
	{ 0x87f9, 0x87fc, PDF_CMAP_TABLE, 9878 },
	{ 0x87fe, 0x8806, PDF_CMAP_TABLE, 9882 },
	{ 0x8808, 0x880d, PDF_CMAP_TABLE, 9891 },
	{ 0x880f, 0x880f, PDF_CMAP_SINGLE, 14353 },
	{ 0x8810, 0x8811, PDF_CMAP_RANGE, 12844 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 12847 },
	{ 0x8814, 0x8815, PDF_CMAP_RANGE, 5695 },
	{ 0x8816, 0x8819, PDF_CMAP_TABLE, 9897 },
	{ 0x881b, 0x881d, PDF_CMAP_TABLE, 9901 },
	{ 0x881f, 0x8833, PDF_CMAP_TABLE, 9904 },
	{ 0x8835, 0x8839, PDF_CMAP_TABLE, 9925 },
	{ 0x883b, 0x8842, PDF_CMAP_TABLE, 9930 },
	{ 0x8843, 0x8844, PDF_CMAP_RANGE, 7674 },
	{ 0x8845, 0x8846, PDF_CMAP_RANGE, 14358 },
	{ 0x8848, 0x8848, PDF_CMAP_SINGLE, 8915 },
	{ 0x884a, 0x884f, PDF_CMAP_TABLE, 9938 },
	{ 0x8852, 0x8853, PDF_CMAP_TABLE, 9944 },
	{ 0x8855, 0x8857, PDF_CMAP_TABLE, 9946 },
	{ 0x8859, 0x885b, PDF_CMAP_TABLE, 9949 },
	{ 0x885d, 0x885e, PDF_CMAP_TABLE, 9952 },
	{ 0x8860, 0x8865, PDF_CMAP_TABLE, 9954 },
	{ 0x8867, 0x886b, PDF_CMAP_TABLE, 9960 },
	{ 0x886d, 0x8872, PDF_CMAP_TABLE, 9965 },
	{ 0x8874, 0x8874, PDF_CMAP_SINGLE, 7686 },
	{ 0x8875, 0x8876, PDF_CMAP_RANGE, 7677 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2489 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 2493 },
	{ 0x887c, 0x8880, PDF_CMAP_TABLE, 9971 },
	{ 0x8881, 0x8882, PDF_CMAP_RANGE, 2490 },
	{ 0x8883, 0x8884, PDF_CMAP_TABLE, 9976 },
	{ 0x8887, 0x8889, PDF_CMAP_TABLE, 9978 },
	{ 0x888b, 0x888e, PDF_CMAP_TABLE, 9981 },
	{ 0x8891, 0x8893, PDF_CMAP_TABLE, 9985 },
	{ 0x8895, 0x889c, PDF_CMAP_TABLE, 9988 },
	{ 0x889e, 0x88a2, PDF_CMAP_TABLE, 9996 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 8286 },
	{ 0x88a7, 0x88a8, PDF_CMAP_TABLE, 10001 },
	{ 0x88aa, 0x88ac, PDF_CMAP_TABLE, 10003 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 18293 },
	{ 0x88b1, 0x88b2, PDF_CMAP_TABLE, 10006 },
	{ 0x88b4, 0x88b5, PDF_CMAP_RANGE, 14366 },
	{ 0x88b6, 0x88ba, PDF_CMAP_TABLE, 10008 },
	{ 0x88bc, 0x88c0, PDF_CMAP_TABLE, 10013 },
	{ 0x88c1, 0x88c2, PDF_CMAP_RANGE, 3410 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 14368 },
	{ 0x88c7, 0x88c7, PDF_CMAP_SINGLE, 16049 },
	{ 0x88c9, 0x88cc, PDF_CMAP_TABLE, 10018 },
	{ 0x88cd, 0x88ce, PDF_CMAP_RANGE, 9593 },
	{ 0x88cf, 0x88d0, PDF_CMAP_TABLE, 10022 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 3844 },
	{ 0x88d4, 0x88df, PDF_CMAP_TABLE, 10024 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3841 },
	{ 0x88e6, 0x88e8, PDF_CMAP_TABLE, 10036 },
	{ 0x88eb, 0x88ec, PDF_CMAP_TABLE, 10039 },
	{ 0x88ee, 0x88f0, PDF_CMAP_TABLE, 10041 },
	{ 0x88f1, 0x88f2, PDF_CMAP_RANGE, 10225 },
	{ 0x88f3, 0x8902, PDF_CMAP_TABLE, 10044 },
	{ 0x8905, 0x8907, PDF_CMAP_TABLE, 10060 },
	{ 0x8909, 0x890c, PDF_CMAP_TABLE, 10063 },
	{ 0x890e, 0x890e, PDF_CMAP_SINGLE, 10810 },
	{ 0x8910, 0x8911, PDF_CMAP_TABLE, 10067 },
	{ 0x8912, 0x8913, PDF_CMAP_RANGE, 4622 },
	{ 0x8914, 0x8916, PDF_CMAP_TABLE, 10069 },
	{ 0x8917, 0x8919, PDF_CMAP_RANGE, 10804 },
	{ 0x891a, 0x891a, PDF_CMAP_SINGLE, 4259 },
	{ 0x891e, 0x891f, PDF_CMAP_TABLE, 10072 },
	{ 0x8921, 0x8927, PDF_CMAP_TABLE, 10074 },
	{ 0x8929, 0x892c, PDF_CMAP_TABLE, 10081 },
	{ 0x892d, 0x892e, PDF_CMAP_RANGE, 11386 },
	{ 0x892f, 0x8933, PDF_CMAP_TABLE, 10085 },
	{ 0x8935, 0x8938, PDF_CMAP_TABLE, 10090 },
	{ 0x893b, 0x893e, PDF_CMAP_TABLE, 10094 },
	{ 0x8941, 0x8944, PDF_CMAP_TABLE, 10098 },
	{ 0x8946, 0x8947, PDF_CMAP_TABLE, 10102 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 12235 },
	{ 0x894b, 0x894d, PDF_CMAP_TABLE, 10104 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 12230 },
	{ 0x8950, 0x8951, PDF_CMAP_RANGE, 12233 },
	{ 0x8952, 0x8954, PDF_CMAP_TABLE, 10107 },
	{ 0x8956, 0x8959, PDF_CMAP_TABLE, 10110 },
	{ 0x895a, 0x895b, PDF_CMAP_RANGE, 12566 },
	{ 0x895c, 0x8966, PDF_CMAP_TABLE, 10114 },
	{ 0x8969, 0x896f, PDF_CMAP_TABLE, 10125 },
	{ 0x8971, 0x8974, PDF_CMAP_TABLE, 10132 },
	{ 0x8976, 0x8977, PDF_CMAP_TABLE, 10136 },
	{ 0x8979, 0x897a, PDF_CMAP_RANGE, 13538 },
	{ 0x897b, 0x897c, PDF_CMAP_TABLE, 10138 },
	{ 0x897e, 0x8983, PDF_CMAP_TABLE, 10140 },
	{ 0x8985, 0x898b, PDF_CMAP_TABLE, 10146 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 2973 },
	{ 0x8991, 0x8991, PDF_CMAP_SINGLE, 18296 },
	{ 0x8993, 0x8998, PDF_CMAP_TABLE, 10153 },
	{ 0x899b, 0x899f, PDF_CMAP_TABLE, 10159 },
	{ 0x89a1, 0x89a7, PDF_CMAP_TABLE, 10164 },
	{ 0x89a9, 0x89aa, PDF_CMAP_TABLE, 10171 },
	{ 0x89ac, 0x89af, PDF_CMAP_TABLE, 10173 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 5416 },
	{ 0x89b6, 0x89b7, PDF_CMAP_TABLE, 10177 },
	{ 0x89b9, 0x89ba, PDF_CMAP_TABLE, 10179 },
	{ 0x89bc, 0x89c1, PDF_CMAP_TABLE, 10181 },
	{ 0x89c6, 0x89c6, PDF_CMAP_SINGLE, 17766 },
	{ 0x89d2, 0x89d6, PDF_CMAP_TABLE, 10187 },
	{ 0x89d9, 0x89d9, PDF_CMAP_SINGLE, 8293 },
	{ 0x89da, 0x89db, PDF_CMAP_RANGE, 8935 },
	{ 0x89dc, 0x89dd, PDF_CMAP_TABLE, 10192 },
	{ 0x89df, 0x89e9, PDF_CMAP_TABLE, 10194 },
	{ 0x89eb, 0x89ed, PDF_CMAP_TABLE, 10205 },
	{ 0x89f0, 0x89f1, PDF_CMAP_TABLE, 10208 },
	{ 0x89f2, 0x89f3, PDF_CMAP_RANGE, 11865 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 5417 },
	{ 0x89f6, 0x89f8, PDF_CMAP_TABLE, 10210 },
	{ 0x89fa, 0x89fc, PDF_CMAP_TABLE, 10213 },
	{ 0x89fe, 0x8a00, PDF_CMAP_TABLE, 10216 },
	{ 0x8a02, 0x8a03, PDF_CMAP_RANGE, 2056 },
	{ 0x8a04, 0x8a04, PDF_CMAP_SINGLE, 7172 },
	{ 0x8a07, 0x8a08, PDF_CMAP_TABLE, 10219 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2499 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a0e, 0x8a13, PDF_CMAP_TABLE, 10221 },
	{ 0x8a15, 0x8a18, PDF_CMAP_TABLE, 10227 },
	{ 0x8a1b, 0x8a1f, PDF_CMAP_TABLE, 10231 },
	{ 0x8a22, 0x8a23, PDF_CMAP_TABLE, 10236 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 2977 },
	{ 0x8a27, 0x8a27, PDF_CMAP_SINGLE, 8296 },
	{ 0x8a29, 0x8a2d, PDF_CMAP_TABLE, 10238 },
	{ 0x8a30, 0x8a31, PDF_CMAP_TABLE, 10243 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 3425 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 3427 },
	{ 0x8a38, 0x8a41, PDF_CMAP_TABLE, 10245 },
	{ 0x8a44, 0x8a45, PDF_CMAP_RANGE, 8944 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 3424 },
	{ 0x8a48, 0x8a4a, PDF_CMAP_TABLE, 10255 },
	{ 0x8a4c, 0x8a52, PDF_CMAP_TABLE, 10258 },
	{ 0x8a54, 0x8a56, PDF_CMAP_TABLE, 10265 },
	{ 0x8a57, 0x8a58, PDF_CMAP_RANGE, 8942 },
	{ 0x8a59, 0x8a59, PDF_CMAP_SINGLE, 8940 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 3422 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 3418 },
	{ 0x8a60, 0x8a63, PDF_CMAP_TABLE, 10268 },
	{ 0x8a66, 0x8a69, PDF_CMAP_TABLE, 10272 },
	{ 0x8a6b, 0x8a6e, PDF_CMAP_TABLE, 10276 },
	{ 0x8a70, 0x8a71, PDF_CMAP_TABLE, 10280 },
	{ 0x8a72, 0x8a73, PDF_CMAP_RANGE, 3848 },
	{ 0x8a74, 0x8a77, PDF_CMAP_TABLE, 10282 },
	{ 0x8a79, 0x8a7c, PDF_CMAP_TABLE, 10286 },
	{ 0x8a7e, 0x8a7f, PDF_CMAP_TABLE, 10290 },
	{ 0x8a81, 0x8a87, PDF_CMAP_TABLE, 10292 },
	{ 0x8a8b, 0x8a8d, PDF_CMAP_TABLE, 10299 },
	{ 0x8a8f, 0x8a96, PDF_CMAP_TABLE, 10302 },
	{ 0x8a98, 0x8a9a, PDF_CMAP_TABLE, 10310 },
	{ 0x8a9c, 0x8a9c, PDF_CMAP_SINGLE, 14380 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 4263 },
	{ 0x8aa0, 0x8aa1, PDF_CMAP_TABLE, 10313 },
	{ 0x8aa3, 0x8aab, PDF_CMAP_TABLE, 10315 },
	{ 0x8aaf, 0x8ab0, PDF_CMAP_TABLE, 10324 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 4633 },
	{ 0x8ab4, 0x8ab4, PDF_CMAP_SINGLE, 18300 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 4640 },
	{ 0x8ab8, 0x8ac0, PDF_CMAP_TABLE, 10326 },
	{ 0x8ac2, 0x8ac9, PDF_CMAP_TABLE, 10335 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 4631 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 4639 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 10818 },
	{ 0x8ad1, 0x8ad3, PDF_CMAP_TABLE, 10343 },
	{ 0x8ad4, 0x8ad5, PDF_CMAP_RANGE, 10823 },
	{ 0x8ad6, 0x8ae2, PDF_CMAP_TABLE, 10346 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 11404 },
	{ 0x8ae6, 0x8ae8, PDF_CMAP_TABLE, 10359 },
	{ 0x8aea, 0x8aeb, PDF_CMAP_TABLE, 10362 },
	{ 0x8aed, 0x8af3, PDF_CMAP_TABLE, 10364 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 11400 },
	{ 0x8af6, 0x8af8, PDF_CMAP_TABLE, 10371 },
	{ 0x8afa, 0x8afc, PDF_CMAP_TABLE, 10374 },
	{ 0x8afe, 0x8b00, PDF_CMAP_TABLE, 10377 },
	{ 0x8b01, 0x8b02, PDF_CMAP_RANGE, 4950 },
	{ 0x8b04, 0x8b08, PDF_CMAP_TABLE, 10380 },
	{ 0x8b0a, 0x8b20, PDF_CMAP_TABLE, 10385 },
	{ 0x8b22, 0x8b28, PDF_CMAP_TABLE, 10408 },
	{ 0x8b2a, 0x8b31, PDF_CMAP_TABLE, 10415 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 12239 },
	{ 0x8b35, 0x8b37, PDF_CMAP_TABLE, 10423 },
	{ 0x8b39, 0x8b43, PDF_CMAP_TABLE, 10426 },
	{ 0x8b45, 0x8b4b, PDF_CMAP_TABLE, 10437 },
	{ 0x8b4c, 0x8b4d, PDF_CMAP_RANGE, 14395 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_RANGE, 5577 },
	{ 0x8b50, 0x8b5a, PDF_CMAP_TABLE, 10444 },
	{ 0x8b5c, 0x8b60, PDF_CMAP_TABLE, 10455 },
	{ 0x8b62, 0x8b63, PDF_CMAP_TABLE, 10460 },
	{ 0x8b65, 0x8b6d, PDF_CMAP_TABLE, 10462 },
	{ 0x8b6f, 0x8b70, PDF_CMAP_TABLE, 10471 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 5775 },
	{ 0x8b77, 0x8b79, PDF_CMAP_TABLE, 10473 },
	{ 0x8b7a, 0x8b7b, PDF_CMAP_RANGE, 13076 },
	{ 0x8b7d, 0x8b82, PDF_CMAP_TABLE, 10476 },
	{ 0x8b84, 0x8b86, PDF_CMAP_TABLE, 10482 },
	{ 0x8b88, 0x8b88, PDF_CMAP_SINGLE, 13376 },
	{ 0x8b8a, 0x8b8c, PDF_CMAP_TABLE, 10485 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 13374 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 14399 },
	{ 0x8b92, 0x8b93, PDF_CMAP_TABLE, 10488 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 13468 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 5933 },
	{ 0x8b98, 0x8b99, PDF_CMAP_RANGE, 13543 },
	{ 0x8b9a, 0x8b9c, PDF_CMAP_TABLE, 10490 },
	{ 0x8b9e, 0x8ba0, PDF_CMAP_TABLE, 10493 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 17767 },
	{ 0x8be2, 0x8be2, PDF_CMAP_SINGLE, 17768 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1291 },
	{ 0x8c39, 0x8c39, PDF_CMAP_SINGLE, 8299 },
	{ 0x8c3b, 0x8c3f, PDF_CMAP_TABLE, 10496 },
	{ 0x8c41, 0x8c43, PDF_CMAP_TABLE, 10501 },
	{ 0x8c45, 0x8c51, PDF_CMAP_TABLE, 10504 },
	{ 0x8c54, 0x8c57, PDF_CMAP_TABLE, 10517 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 2984 },
	{ 0x8c5c, 0x8c5d, PDF_CMAP_RANGE, 8301 },
	{ 0x8c5f, 0x8c5f, PDF_CMAP_SINGLE, 8952 },
	{ 0x8c61, 0x8c62, PDF_CMAP_TABLE, 10521 },
	{ 0x8c64, 0x8c66, PDF_CMAP_TABLE, 10523 },
	{ 0x8c68, 0x8c69, PDF_CMAP_RANGE, 10250 },
	{ 0x8c6a, 0x8c6d, PDF_CMAP_TABLE, 10526 },
	{ 0x8c6f, 0x8c73, PDF_CMAP_TABLE, 10530 },
	{ 0x8c75, 0x8c7b, PDF_CMAP_TABLE, 10535 },
	{ 0x8c7d, 0x8c7d, PDF_CMAP_SINGLE, 8303 },
	{ 0x8c80, 0x8c82, PDF_CMAP_TABLE, 10542 },
	{ 0x8c84, 0x8c85, PDF_CMAP_RANGE, 9630 },
	{ 0x8c86, 0x8c86, PDF_CMAP_SINGLE, 9629 },
	{ 0x8c89, 0x8c8a, PDF_CMAP_TABLE, 10545 },
	{ 0x8c8c, 0x8c8d, PDF_CMAP_TABLE, 10547 },
	{ 0x8c8f, 0x8c90, PDF_CMAP_TABLE, 10549 },
	{ 0x8c91, 0x8c92, PDF_CMAP_RANGE, 11414 },
	{ 0x8c93, 0x8c95, PDF_CMAP_TABLE, 10551 },
	{ 0x8c97, 0x8ca0, PDF_CMAP_TABLE, 10554 },
	{ 0x8ca1, 0x8ca2, PDF_CMAP_RANGE, 2508 },
	{ 0x8ca3, 0x8ca5, PDF_CMAP_TABLE, 10564 },
	{ 0x8ca7, 0x8cad, PDF_CMAP_TABLE, 10567 },
	{ 0x8caf, 0x8cb0, PDF_CMAP_TABLE, 10574 },
	{ 0x8cb2, 0x8cc5, PDF_CMAP_TABLE, 10576 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_RANGE, 3871 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 3870 },
	{ 0x8ccc, 0x8ccd, PDF_CMAP_TABLE, 10596 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 10253 },
	{ 0x8cd1, 0x8cd2, PDF_CMAP_RANGE, 4280 },
	{ 0x8cd3, 0x8cd7, PDF_CMAP_TABLE, 10598 },
	{ 0x8cd9, 0x8ce1, PDF_CMAP_TABLE, 10603 },
	{ 0x8ce2, 0x8ce3, PDF_CMAP_RANGE, 4652 },
	{ 0x8ce4, 0x8ceb, PDF_CMAP_TABLE, 10612 },
	{ 0x8cec, 0x8ced, PDF_CMAP_RANGE, 4650 },
	{ 0x8cee, 0x8cee, PDF_CMAP_SINGLE, 11418 },
	{ 0x8cf0, 0x8cf5, PDF_CMAP_TABLE, 10620 },
	{ 0x8cf7, 0x8cfe, PDF_CMAP_TABLE, 10626 },
	{ 0x8d00, 0x8d00, PDF_CMAP_SINGLE, 12264 },
	{ 0x8d02, 0x8d05, PDF_CMAP_TABLE, 10634 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 12595 },
	{ 0x8d08, 0x8d0d, PDF_CMAP_TABLE, 10638 },
	{ 0x8d0f, 0x8d15, PDF_CMAP_TABLE, 10644 },
	{ 0x8d16, 0x8d17, PDF_CMAP_RANGE, 5849 },
	{ 0x8d18, 0x8d19, PDF_CMAP_TABLE, 10651 },
	{ 0x8d1b, 0x8d1d, PDF_CMAP_TABLE, 10653 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 1295 },
	{ 0x8d66, 0x8d67, PDF_CMAP_TABLE, 10656 },
	{ 0x8d68, 0x8d69, PDF_CMAP_RANGE, 9633 },
	{ 0x8d6b, 0x8d70, PDF_CMAP_TABLE, 10658 },
	{ 0x8d72, 0x8d74, PDF_CMAP_TABLE, 10664 },
	{ 0x8d76, 0x8d7b, PDF_CMAP_TABLE, 10667 },
	{ 0x8d7d, 0x8d7d, PDF_CMAP_SINGLE, 8305 },
	{ 0x8d80, 0x8d82, PDF_CMAP_TABLE, 10673 },
	{ 0x8d84, 0x8d85, PDF_CMAP_TABLE, 10676 },
	{ 0x8d89, 0x8d8a, PDF_CMAP_TABLE, 10678 },
	{ 0x8d8c, 0x8d8d, PDF_CMAP_TABLE, 10680 },
	{ 0x8d8e, 0x8d8f, PDF_CMAP_RANGE, 9637 },
	{ 0x8d90, 0x8d92, PDF_CMAP_TABLE, 10682 },
	{ 0x8d93, 0x8d94, PDF_CMAP_RANGE, 9640 },
	{ 0x8d95, 0x8d96, PDF_CMAP_TABLE, 10685 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 4283 },
	{ 0x8d9b, 0x8d9c, PDF_CMAP_TABLE, 10687 },
	{ 0x8d9f, 0x8da1, PDF_CMAP_TABLE, 10689 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 4659 },
	{ 0x8da5, 0x8daf, PDF_CMAP_TABLE, 10692 },
	{ 0x8db2, 0x8db7, PDF_CMAP_TABLE, 10703 },
	{ 0x8db9, 0x8dba, PDF_CMAP_TABLE, 10709 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 8308 },
	{ 0x8dbe, 0x8dc3, PDF_CMAP_TABLE, 10711 },
	{ 0x8dc5, 0x8dc8, PDF_CMAP_TABLE, 10717 },
	{ 0x8dcb, 0x8dd1, PDF_CMAP_TABLE, 10721 },
	{ 0x8dd3, 0x8ddd, PDF_CMAP_TABLE, 10728 },
	{ 0x8ddf, 0x8de4, PDF_CMAP_TABLE, 10739 },
	{ 0x8de6, 0x8dec, PDF_CMAP_TABLE, 10745 },
	{ 0x8dee, 0x8df4, PDF_CMAP_TABLE, 10752 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 3883 },
	{ 0x8dfc, 0x8e07, PDF_CMAP_TABLE, 10759 },
	{ 0x8e09, 0x8e0a, PDF_CMAP_TABLE, 10771 },
	{ 0x8e0d, 0x8e25, PDF_CMAP_TABLE, 10773 },
	{ 0x8e26, 0x8e27, PDF_CMAP_RANGE, 10859 },
	{ 0x8e28, 0x8e2b, PDF_CMAP_TABLE, 10798 },
	{ 0x8e2d, 0x8e2e, PDF_CMAP_TABLE, 10802 },
	{ 0x8e30, 0x8e31, PDF_CMAP_TABLE, 10804 },
	{ 0x8e33, 0x8e36, PDF_CMAP_TABLE, 10806 },
	{ 0x8e38, 0x8e3a, PDF_CMAP_TABLE, 10810 },
	{ 0x8e3c, 0x8e3d, PDF_CMAP_RANGE, 11432 },
	{ 0x8e3e, 0x8e42, PDF_CMAP_TABLE, 10813 },
	{ 0x8e44, 0x8e50, PDF_CMAP_TABLE, 10818 },
	{ 0x8e53, 0x8e57, PDF_CMAP_TABLE, 10831 },
	{ 0x8e59, 0x8e6a, PDF_CMAP_TABLE, 10836 },
	{ 0x8e6c, 0x8e6d, PDF_CMAP_TABLE, 10854 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 12606 },
	{ 0x8e71, 0x8e78, PDF_CMAP_TABLE, 10856 },
	{ 0x8e7a, 0x8e7c, PDF_CMAP_TABLE, 10864 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 17188 },
	{ 0x8e80, 0x8e82, PDF_CMAP_TABLE, 10867 },
	{ 0x8e84, 0x8e8e, PDF_CMAP_TABLE, 10870 },
	{ 0x8e90, 0x8e95, PDF_CMAP_TABLE, 10881 },
	{ 0x8e96, 0x8e97, PDF_CMAP_RANGE, 13250 },
	{ 0x8e98, 0x8e98, PDF_CMAP_SINGLE, 13379 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 13247 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 13473 },
	{ 0x8e9e, 0x8ea0, PDF_CMAP_RANGE, 13470 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 5961 },
	{ 0x8ea3, 0x8ead, PDF_CMAP_TABLE, 10887 },
	{ 0x8eb0, 0x8eb0, PDF_CMAP_SINGLE, 17385 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 3887 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14431 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 4669 },
	{ 0x8ebc, 0x8ebd, PDF_CMAP_TABLE, 10898 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 5430 },
	{ 0x8ec2, 0x8ec3, PDF_CMAP_TABLE, 10900 },
	{ 0x8ec9, 0x8ecf, PDF_CMAP_TABLE, 10902 },
	{ 0x8ed1, 0x8ed4, PDF_CMAP_TABLE, 10909 },
	{ 0x8ed7, 0x8ed8, PDF_CMAP_TABLE, 10913 },
	{ 0x8eda, 0x8edf, PDF_CMAP_TABLE, 10915 },
	{ 0x8ee0, 0x8ee1, PDF_CMAP_RANGE, 8318 },
	{ 0x8ee2, 0x8ee2, PDF_CMAP_SINGLE, 18313 },
	{ 0x8ee4, 0x8ee6, PDF_CMAP_TABLE, 10921 },
	{ 0x8ee7, 0x8ee8, PDF_CMAP_RANGE, 8983 },
	{ 0x8ee9, 0x8ee9, PDF_CMAP_SINGLE, 8990 },
	{ 0x8eeb, 0x8eef, PDF_CMAP_TABLE, 10924 },
	{ 0x8ef1, 0x8ef2, PDF_CMAP_TABLE, 10929 },
	{ 0x8ef4, 0x8efc, PDF_CMAP_TABLE, 10931 },
	{ 0x8efe, 0x8f03, PDF_CMAP_TABLE, 10940 },
	{ 0x8f05, 0x8f06, PDF_CMAP_TABLE, 10946 },
	{ 0x8f07, 0x8f08, PDF_CMAP_RANGE, 9662 },
	{ 0x8f09, 0x8f0b, PDF_CMAP_TABLE, 10948 },
	{ 0x8f0d, 0x8f0e, PDF_CMAP_TABLE, 10951 },
	{ 0x8f10, 0x8f11, PDF_CMAP_RANGE, 10269 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 10953 },
	{ 0x8f16, 0x8f17, PDF_CMAP_RANGE, 10874 },
	{ 0x8f18, 0x8f20, PDF_CMAP_TABLE, 10957 },
	{ 0x8f23, 0x8f26, PDF_CMAP_TABLE, 10966 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 10970 },
	{ 0x8f2c, 0x8f30, PDF_CMAP_TABLE, 10972 },
	{ 0x8f32, 0x8f39, PDF_CMAP_TABLE, 10977 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 4967 },
	{ 0x8f3e, 0x8f45, PDF_CMAP_TABLE, 10985 },
	{ 0x8f46, 0x8f48, PDF_CMAP_RANGE, 12279 },
	{ 0x8f49, 0x8f4b, PDF_CMAP_TABLE, 10993 },
	{ 0x8f4d, 0x8f4e, PDF_CMAP_TABLE, 10996 },
	{ 0x8f4f, 0x8f50, PDF_CMAP_RANGE, 12611 },
	{ 0x8f51, 0x8f55, PDF_CMAP_TABLE, 10998 },
	{ 0x8f56, 0x8f57, PDF_CMAP_RANGE, 12866 },
	{ 0x8f58, 0x8f64, PDF_CMAP_TABLE, 11003 },
	{ 0x8f66, 0x8f67, PDF_CMAP_RANGE, 17769 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 17771 },
	{ 0x8f93, 0x8f93, PDF_CMAP_SINGLE, 18226 },
	{ 0x8f9b, 0x8f9c, PDF_CMAP_TABLE, 11016 },
	{ 0x8f9f, 0x8fa0, PDF_CMAP_TABLE, 11018 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 4290 },
	{ 0x8fa5, 0x8fa6, PDF_CMAP_TABLE, 11020 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 4971 },
	{ 0x8fad, 0x8fba, PDF_CMAP_TABLE, 11022 },
	{ 0x8fbb, 0x8fbc, PDF_CMAP_RANGE, 15723 },
	{ 0x8fbe, 0x8fbf, PDF_CMAP_TABLE, 11036 },
	{ 0x8fc1, 0x8fc2, PDF_CMAP_TABLE, 11038 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 11040 },
	{ 0x8fc9, 0x8fce, PDF_CMAP_TABLE, 11043 },
	{ 0x8fd0, 0x8fd7, PDF_CMAP_TABLE, 11049 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 16331 },
	{ 0x8fe0, 0x8fe6, PDF_CMAP_TABLE, 11057 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 2073 },
	{ 0x8fea, 0x8feb, PDF_CMAP_TABLE, 11064 },
	{ 0x8fed, 0x8fee, PDF_CMAP_TABLE, 11066 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 2065 },
	{ 0x8ff4, 0x9006, PDF_CMAP_TABLE, 11068 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 16096 },
	{ 0x900b, 0x900d, PDF_CMAP_TABLE, 11087 },
	{ 0x900f, 0x9012, PDF_CMAP_TABLE, 11090 },
	{ 0x9014, 0x9017, PDF_CMAP_TABLE, 11094 },
	{ 0x9019, 0x9024, PDF_CMAP_TABLE, 11098 },
	{ 0x902d, 0x902f, PDF_CMAP_TABLE, 11110 },
	{ 0x9031, 0x9038, PDF_CMAP_TABLE, 11113 },
	{ 0x903c, 0x903f, PDF_CMAP_TABLE, 11121 },
	{ 0x9041, 0x9042, PDF_CMAP_TABLE, 11125 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 9668 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 3902 },
	{ 0x9049, 0x9056, PDF_CMAP_TABLE, 11127 },
	{ 0x9058, 0x9059, PDF_CMAP_TABLE, 11141 },
	{ 0x905b, 0x905e, PDF_CMAP_TABLE, 11143 },
	{ 0x9060, 0x9064, PDF_CMAP_TABLE, 11147 },
	{ 0x9067, 0x9069, PDF_CMAP_TABLE, 11152 },
	{ 0x906b, 0x9070, PDF_CMAP_TABLE, 11155 },
	{ 0x9072, 0x9080, PDF_CMAP_TABLE, 11161 },
	{ 0x9081, 0x9082, PDF_CMAP_RANGE, 5269 },
	{ 0x9083, 0x9088, PDF_CMAP_TABLE, 11176 },
	{ 0x908a, 0x908b, PDF_CMAP_RANGE, 5593 },
	{ 0x908d, 0x908d, PDF_CMAP_SINGLE, 12871 },
	{ 0x908f, 0x9091, PDF_CMAP_TABLE, 11182 },
	{ 0x9094, 0x9095, PDF_CMAP_TABLE, 11185 },
	{ 0x9097, 0x9098, PDF_CMAP_RANGE, 6171 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 6170 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 6173 },
	{ 0x909e, 0x90a3, PDF_CMAP_TABLE, 11187 },
	{ 0x90a5, 0x90a8, PDF_CMAP_TABLE, 11193 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 1309 },
	{ 0x90ae, 0x90b6, PDF_CMAP_TABLE, 11197 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 1682 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 15300 },
	{ 0x90bd, 0x90bf, PDF_CMAP_TABLE, 11206 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 2076 },
	{ 0x90c3, 0x90c5, PDF_CMAP_TABLE, 11209 },
	{ 0x90c7, 0x90c8, PDF_CMAP_TABLE, 11212 },
	{ 0x90ca, 0x90cb, PDF_CMAP_TABLE, 11214 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 2075 },
	{ 0x90d4, 0x90d8, PDF_CMAP_TABLE, 11216 },
	{ 0x90d9, 0x90da, PDF_CMAP_RANGE, 7711 },
	{ 0x90db, 0x90dd, PDF_CMAP_TABLE, 11221 },
	{ 0x90df, 0x90e5, PDF_CMAP_TABLE, 11224 },
	{ 0x90e8, 0x90ea, PDF_CMAP_TABLE, 11231 },
	{ 0x90eb, 0x90ec, PDF_CMAP_RANGE, 8333 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 3014 },
	{ 0x90ef, 0x90f1, PDF_CMAP_TABLE, 11234 },
	{ 0x90f2, 0x90f3, PDF_CMAP_RANGE, 8330 },
	{ 0x90f4, 0x90f5, PDF_CMAP_TABLE, 11237 },
	{ 0x90f9, 0x9109, PDF_CMAP_TABLE, 11239 },
	{ 0x910b, 0x910b, PDF_CMAP_SINGLE, 9677 },
	{ 0x910d, 0x9112, PDF_CMAP_TABLE, 11256 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 9676 },
	{ 0x9116, 0x9124, PDF_CMAP_TABLE, 11262 },
	{ 0x9126, 0x9128, PDF_CMAP_TABLE, 11277 },
	{ 0x9129, 0x912a, PDF_CMAP_RANGE, 10883 },
	{ 0x912b, 0x9134, PDF_CMAP_TABLE, 11280 },
	{ 0x9135, 0x9136, PDF_CMAP_RANGE, 11451 },
	{ 0x9138, 0x9139, PDF_CMAP_TABLE, 11290 },
	{ 0x913a, 0x913b, PDF_CMAP_RANGE, 12284 },
	{ 0x913e, 0x9141, PDF_CMAP_TABLE, 11292 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 12872 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 13087 },
	{ 0x9146, 0x914d, PDF_CMAP_TABLE, 11296 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7722 },
	{ 0x9150, 0x9153, PDF_CMAP_TABLE, 11304 },
	{ 0x9155, 0x915a, PDF_CMAP_TABLE, 11308 },
	{ 0x915c, 0x915c, PDF_CMAP_SINGLE, 16199 },
	{ 0x915e, 0x9165, PDF_CMAP_TABLE, 11314 },
	{ 0x9167, 0x916a, PDF_CMAP_TABLE, 11322 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 3911 },
	{ 0x916e, 0x916f, PDF_CMAP_RANGE, 9679 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 16056 },
	{ 0x9172, 0x917a, PDF_CMAP_TABLE, 11326 },
	{ 0x917c, 0x917c, PDF_CMAP_SINGLE, 16239 },
	{ 0x9180, 0x9180, PDF_CMAP_SINGLE, 10894 },
	{ 0x9181, 0x9182, PDF_CMAP_RANGE, 10891 },
	{ 0x9183, 0x9184, PDF_CMAP_TABLE, 11335 },
	{ 0x9185, 0x9186, PDF_CMAP_RANGE, 10888 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 4688 },
	{ 0x9189, 0x918f, PDF_CMAP_TABLE, 11337 },
	{ 0x9190, 0x9191, PDF_CMAP_RANGE, 11454 },
	{ 0x9192, 0x9193, PDF_CMAP_TABLE, 11344 },
	{ 0x9199, 0x91a3, PDF_CMAP_TABLE, 11346 },
	{ 0x91a5, 0x91a5, PDF_CMAP_SINGLE, 12288 },
	{ 0x91a7, 0x91aa, PDF_CMAP_TABLE, 11357 },
	{ 0x91ab, 0x91ac, PDF_CMAP_RANGE, 5436 },
	{ 0x91ad, 0x91b1, PDF_CMAP_TABLE, 11361 },
	{ 0x91b2, 0x91b3, PDF_CMAP_RANGE, 12876 },
	{ 0x91b4, 0x91b7, PDF_CMAP_TABLE, 11366 },
	{ 0x91b9, 0x91be, PDF_CMAP_TABLE, 11370 },
	{ 0x91c0, 0x91c7, PDF_CMAP_TABLE, 11376 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 3914 },
	{ 0x91cb, 0x91d1, PDF_CMAP_TABLE, 11384 },
	{ 0x91d3, 0x91da, PDF_CMAP_TABLE, 11391 },
	{ 0x91dc, 0x91dd, PDF_CMAP_TABLE, 11399 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 17722 },
	{ 0x91e2, 0x91e9, PDF_CMAP_TABLE, 11401 },
	{ 0x91ea, 0x91eb, PDF_CMAP_RANGE, 8348 },
	{ 0x91ec, 0x91ee, PDF_CMAP_TABLE, 11409 },
	{ 0x91f1, 0x91f1, PDF_CMAP_SINGLE, 8343 },
	{ 0x91f3, 0x91f5, PDF_CMAP_TABLE, 11412 },
	{ 0x91f7, 0x91fa, PDF_CMAP_TABLE, 11415 },
	{ 0x91fd, 0x920a, PDF_CMAP_TABLE, 11419 },
	{ 0x920c, 0x9213, PDF_CMAP_TABLE, 11433 },
	{ 0x9214, 0x9215, PDF_CMAP_RANGE, 3471 },
	{ 0x9216, 0x9217, PDF_CMAP_TABLE, 11441 },
	{ 0x9219, 0x921a, PDF_CMAP_TABLE, 11443 },
	{ 0x921c, 0x921c, PDF_CMAP_SINGLE, 9028 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 3475 },
	{ 0x9223, 0x9228, PDF_CMAP_TABLE, 11445 },
	{ 0x922a, 0x922b, PDF_CMAP_TABLE, 11451 },
	{ 0x922d, 0x922e, PDF_CMAP_TABLE, 11453 },
	{ 0x9230, 0x923a, PDF_CMAP_TABLE, 11455 },
	{ 0x923c, 0x9241, PDF_CMAP_TABLE, 11466 },
	{ 0x9244, 0x9246, PDF_CMAP_TABLE, 11472 },
	{ 0x9248, 0x9258, PDF_CMAP_TABLE, 11475 },
	{ 0x925a, 0x925b, PDF_CMAP_TABLE, 11492 },
	{ 0x925d, 0x9267, PDF_CMAP_TABLE, 11494 },
	{ 0x926b, 0x9270, PDF_CMAP_TABLE, 11505 },
	{ 0x9272, 0x9272, PDF_CMAP_SINGLE, 9706 },
	{ 0x9276, 0x928f, PDF_CMAP_TABLE, 11511 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 4318 },
	{ 0x9293, 0x929d, PDF_CMAP_TABLE, 11537 },
	{ 0x92a0, 0x92ac, PDF_CMAP_TABLE, 11548 },
	{ 0x92ae, 0x92ae, PDF_CMAP_SINGLE, 18329 },
	{ 0x92b1, 0x92b7, PDF_CMAP_TABLE, 11561 },
	{ 0x92b9, 0x92bc, PDF_CMAP_TABLE, 11568 },
	{ 0x92be, 0x92c2, PDF_CMAP_TABLE, 11572 },
	{ 0x92c3, 0x92c4, PDF_CMAP_RANGE, 10896 },
	{ 0x92c5, 0x92d5, PDF_CMAP_TABLE, 11577 },
	{ 0x92d7, 0x92d9, PDF_CMAP_TABLE, 11594 },
	{ 0x92db, 0x92db, PDF_CMAP_SINGLE, 16396 },
	{ 0x92dd, 0x92e1, PDF_CMAP_TABLE, 11597 },
	{ 0x92e3, 0x92ec, PDF_CMAP_TABLE, 11602 },
	{ 0x92ee, 0x92f2, PDF_CMAP_TABLE, 11612 },
	{ 0x92f3, 0x92f4, PDF_CMAP_RANGE, 18333 },
	{ 0x92f6, 0x9304, PDF_CMAP_TABLE, 11617 },
	{ 0x9306, 0x9309, PDF_CMAP_TABLE, 11632 },
	{ 0x930b, 0x9310, PDF_CMAP_TABLE, 11636 },
	{ 0x9312, 0x9316, PDF_CMAP_TABLE, 11642 },
	{ 0x9318, 0x931b, PDF_CMAP_TABLE, 11647 },
	{ 0x931d, 0x9331, PDF_CMAP_TABLE, 11651 },
	{ 0x9333, 0x9336, PDF_CMAP_TABLE, 11672 },
	{ 0x9338, 0x9339, PDF_CMAP_TABLE, 11676 },
	{ 0x933c, 0x933c, PDF_CMAP_SINGLE, 11466 },
	{ 0x9340, 0x934f, PDF_CMAP_TABLE, 11678 },
	{ 0x9350, 0x9351, PDF_CMAP_RANGE, 11923 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 11933 },
	{ 0x9354, 0x935c, PDF_CMAP_TABLE, 11694 },
	{ 0x935e, 0x936e, PDF_CMAP_TABLE, 11703 },
	{ 0x9370, 0x9371, PDF_CMAP_TABLE, 11720 },
	{ 0x9373, 0x937e, PDF_CMAP_TABLE, 11722 },
	{ 0x9380, 0x938a, PDF_CMAP_TABLE, 11734 },
	{ 0x938c, 0x9392, PDF_CMAP_TABLE, 11745 },
	{ 0x9394, 0x93aa, PDF_CMAP_TABLE, 11752 },
	{ 0x93ac, 0x93b5, PDF_CMAP_TABLE, 11775 },
	{ 0x93b7, 0x93b8, PDF_CMAP_TABLE, 11785 },
	{ 0x93ba, 0x93bb, PDF_CMAP_TABLE, 11787 },
	{ 0x93bd, 0x93bd, PDF_CMAP_SINGLE, 14887 },
	{ 0x93bf, 0x93c0, PDF_CMAP_TABLE, 11789 },
	{ 0x93c2, 0x93c4, PDF_CMAP_TABLE, 11791 },
	{ 0x93c6, 0x93c8, PDF_CMAP_TABLE, 11794 },
	{ 0x93ca, 0x93db, PDF_CMAP_TABLE, 11797 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 5602 },
	{ 0x93de, 0x93e4, PDF_CMAP_TABLE, 11815 },
	{ 0x93e6, 0x93e8, PDF_CMAP_TABLE, 11822 },
	{ 0x93ec, 0x93ec, PDF_CMAP_SINGLE, 12626 },
	{ 0x93ee, 0x93ee, PDF_CMAP_SINGLE, 12633 },
	{ 0x93f0, 0x93f1, PDF_CMAP_TABLE, 11825 },
	{ 0x93f3, 0x93f3, PDF_CMAP_SINGLE, 14690 },
	{ 0x93f5, 0x9401, PDF_CMAP_TABLE, 11827 },
	{ 0x9403, 0x9404, PDF_CMAP_TABLE, 11840 },
	{ 0x9406, 0x9419, PDF_CMAP_TABLE, 11842 },
	{ 0x941b, 0x941b, PDF_CMAP_SINGLE, 16712 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 18342 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 12881 },
	{ 0x9424, 0x942e, PDF_CMAP_TABLE, 11862 },
	{ 0x9430, 0x9433, PDF_CMAP_TABLE, 11873 },
	{ 0x9435, 0x9440, PDF_CMAP_TABLE, 11877 },
	{ 0x9442, 0x9447, PDF_CMAP_TABLE, 11889 },
	{ 0x9448, 0x9449, PDF_CMAP_RANGE, 13262 },
	{ 0x944a, 0x944b, PDF_CMAP_RANGE, 13257 },
	{ 0x944c, 0x944d, PDF_CMAP_TABLE, 11895 },
	{ 0x944f, 0x9450, PDF_CMAP_TABLE, 11897 },
	{ 0x9451, 0x9452, PDF_CMAP_RANGE, 5856 },
	{ 0x9454, 0x9455, PDF_CMAP_TABLE, 11899 },
	{ 0x9457, 0x9458, PDF_CMAP_TABLE, 11901 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 14495 },
	{ 0x945d, 0x945e, PDF_CMAP_TABLE, 11903 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 5902 },
	{ 0x9462, 0x9465, PDF_CMAP_TABLE, 11905 },
	{ 0x9467, 0x9467, PDF_CMAP_SINGLE, 15032 },
	{ 0x9468, 0x9469, PDF_CMAP_RANGE, 13478 },
	{ 0x946a, 0x9479, PDF_CMAP_TABLE, 11909 },
	{ 0x947b, 0x947c, PDF_CMAP_TABLE, 11925 },
	{ 0x947d, 0x947e, PDF_CMAP_RANGE, 5981 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 5988 },
	{ 0x9480, 0x9481, PDF_CMAP_RANGE, 13625 },
	{ 0x9482, 0x9483, PDF_CMAP_TABLE, 11927 },
	{ 0x9485, 0x9485, PDF_CMAP_SINGLE, 17671 },
	{ 0x949f, 0x949f, PDF_CMAP_SINGLE, 18565 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 17691 },
	{ 0x94c1, 0x94c1, PDF_CMAP_SINGLE, 18401 },
	{ 0x94c3, 0x94c3, PDF_CMAP_SINGLE, 18399 },
	{ 0x94dc, 0x94dc, PDF_CMAP_SINGLE, 18391 },
	{ 0x94f6, 0x94f6, PDF_CMAP_SINGLE, 18386 },
	{ 0x952d, 0x952d, PDF_CMAP_SINGLE, 18348 },
	{ 0x9547, 0x9547, PDF_CMAP_SINGLE, 18623 },
	{ 0x9577, 0x9578, PDF_CMAP_TABLE, 11929 },
	{ 0x957a, 0x957d, PDF_CMAP_TABLE, 11931 },
	{ 0x957f, 0x9580, PDF_CMAP_TABLE, 11935 },
	{ 0x9582, 0x9583, PDF_CMAP_TABLE, 11937 },
	{ 0x9585, 0x9586, PDF_CMAP_TABLE, 11939 },
	{ 0x9588, 0x9589, PDF_CMAP_TABLE, 11941 },
	{ 0x958b, 0x9594, PDF_CMAP_TABLE, 11943 },
	{ 0x9596, 0x9599, PDF_CMAP_TABLE, 11953 },
	{ 0x959b, 0x959c, PDF_CMAP_TABLE, 11957 },
	{ 0x959e, 0x95a7, PDF_CMAP_TABLE, 11959 },
	{ 0x95a8, 0x95a9, PDF_CMAP_RANGE, 4320 },
	{ 0x95aa, 0x95ae, PDF_CMAP_TABLE, 11969 },
	{ 0x95b0, 0x95b1, PDF_CMAP_TABLE, 11974 },
	{ 0x95b5, 0x95b7, PDF_CMAP_TABLE, 11976 },
	{ 0x95b9, 0x95ba, PDF_CMAP_RANGE, 11500 },
	{ 0x95bb, 0x95c0, PDF_CMAP_TABLE, 11979 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 11947 },
	{ 0x95c5, 0x95c9, PDF_CMAP_TABLE, 11985 },
	{ 0x95ca, 0x95cc, PDF_CMAP_RANGE, 5290 },
	{ 0x95cd, 0x95cd, PDF_CMAP_SINGLE, 11498 },
	{ 0x95d0, 0x95d1, PDF_CMAP_TABLE, 11990 },
	{ 0x95d2, 0x95d3, PDF_CMAP_RANGE, 12317 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 11992 },
	{ 0x95da, 0x95db, PDF_CMAP_RANGE, 12643 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 5611 },
	{ 0x95de, 0x95e5, PDF_CMAP_TABLE, 11995 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 17674 },
	{ 0x95f4, 0x95f4, PDF_CMAP_SINGLE, 18351 },
	{ 0x961c, 0x961e, PDF_CMAP_TABLE, 12003 },
	{ 0x9620, 0x9624, PDF_CMAP_TABLE, 12006 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 6385 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 1318 },
	{ 0x962c, 0x9633, PDF_CMAP_TABLE, 12011 },
	{ 0x9638, 0x963d, PDF_CMAP_TABLE, 12019 },
	{ 0x963f, 0x9645, PDF_CMAP_TABLE, 12025 },
	{ 0x964a, 0x964a, PDF_CMAP_SINGLE, 7194 },
	{ 0x964b, 0x964d, PDF_CMAP_RANGE, 2083 },
	{ 0x964e, 0x9651, PDF_CMAP_TABLE, 12032 },
	{ 0x9653, 0x9654, PDF_CMAP_TABLE, 12036 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 15847 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 2545 },
	{ 0x965b, 0x965f, PDF_CMAP_TABLE, 12038 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 2541 },
	{ 0x9662, 0x9663, PDF_CMAP_RANGE, 2539 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 2544 },
	{ 0x9669, 0x966d, PDF_CMAP_TABLE, 12043 },
	{ 0x966f, 0x9675, PDF_CMAP_TABLE, 12048 },
	{ 0x9676, 0x9677, PDF_CMAP_RANGE, 3031 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 3028 },
	{ 0x967b, 0x967e, PDF_CMAP_TABLE, 12055 },
	{ 0x9680, 0x9681, PDF_CMAP_TABLE, 12059 },
	{ 0x9683, 0x9684, PDF_CMAP_TABLE, 12061 },
	{ 0x9685, 0x9686, PDF_CMAP_RANGE, 3491 },
	{ 0x9687, 0x9687, PDF_CMAP_SINGLE, 9038 },
	{ 0x9688, 0x9689, PDF_CMAP_RANGE, 9040 },
	{ 0x968a, 0x968b, PDF_CMAP_TABLE, 12063 },
	{ 0x968d, 0x968f, PDF_CMAP_TABLE, 12065 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 9718 },
	{ 0x9692, 0x9693, PDF_CMAP_RANGE, 9716 },
	{ 0x9694, 0x9695, PDF_CMAP_RANGE, 3934 },
	{ 0x9696, 0x9699, PDF_CMAP_TABLE, 12068 },
	{ 0x969b, 0x969c, PDF_CMAP_TABLE, 12072 },
	{ 0x969e, 0x969e, PDF_CMAP_SINGLE, 10322 },
	{ 0x96a1, 0x96a5, PDF_CMAP_TABLE, 12074 },
	{ 0x96a7, 0x96a8, PDF_CMAP_RANGE, 4998 },
	{ 0x96a9, 0x96aa, PDF_CMAP_TABLE, 12079 },
	{ 0x96ac, 0x96ac, PDF_CMAP_SINGLE, 11952 },
	{ 0x96ae, 0x96ae, PDF_CMAP_SINGLE, 11950 },
	{ 0x96b0, 0x96b1, PDF_CMAP_TABLE, 12081 },
	{ 0x96b3, 0x96b4, PDF_CMAP_TABLE, 12083 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 561 },
	{ 0x96b8, 0x96b9, PDF_CMAP_TABLE, 12085 },
	{ 0x96bb, 0x96bd, PDF_CMAP_TABLE, 12087 },
	{ 0x96bf, 0x96c5, PDF_CMAP_TABLE, 12090 },
	{ 0x96c6, 0x96c7, PDF_CMAP_RANGE, 3499 },
	{ 0x96c8, 0x96c8, PDF_CMAP_SINGLE, 9045 },
	{ 0x96c9, 0x96ca, PDF_CMAP_RANGE, 3938 },
	{ 0x96cb, 0x96ce, PDF_CMAP_TABLE, 12097 },
	{ 0x96d2, 0x96df, PDF_CMAP_TABLE, 12101 },
	{ 0x96e1, 0x96e3, PDF_CMAP_TABLE, 12115 },
	{ 0x96e5, 0x96e5, PDF_CMAP_SINGLE, 13480 },
	{ 0x96e8, 0x96ea, PDF_CMAP_TABLE, 12118 },
	{ 0x96ef, 0x96f2, PDF_CMAP_TABLE, 12121 },
	{ 0x96f4, 0x96fb, PDF_CMAP_TABLE, 12125 },
	{ 0x96fd, 0x96fd, PDF_CMAP_SINGLE, 9722 },
	{ 0x96ff, 0x9700, PDF_CMAP_TABLE, 12133 },
	{ 0x9702, 0x9705, PDF_CMAP_TABLE, 12135 },
	{ 0x9706, 0x9707, PDF_CMAP_RANGE, 4707 },
	{ 0x9708, 0x9709, PDF_CMAP_TABLE, 12139 },
	{ 0x970b, 0x970b, PDF_CMAP_SINGLE, 11508 },
	{ 0x970d, 0x9713, PDF_CMAP_TABLE, 12141 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 5004 },
	{ 0x9718, 0x9719, PDF_CMAP_TABLE, 12148 },
	{ 0x971b, 0x972a, PDF_CMAP_TABLE, 12150 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 12647 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 12911 },
	{ 0x9730, 0x9732, PDF_CMAP_TABLE, 12166 },
	{ 0x9735, 0x9736, PDF_CMAP_TABLE, 12169 },
	{ 0x9738, 0x9739, PDF_CMAP_RANGE, 5793 },
	{ 0x973a, 0x973a, PDF_CMAP_SINGLE, 13106 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 5858 },
	{ 0x973f, 0x9744, PDF_CMAP_TABLE, 12171 },
	{ 0x9746, 0x9749, PDF_CMAP_TABLE, 12177 },
	{ 0x974b, 0x974b, PDF_CMAP_SINGLE, 13606 },
	{ 0x9751, 0x9752, PDF_CMAP_TABLE, 12181 },
	{ 0x9756, 0x9758, PDF_CMAP_TABLE, 12183 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 10940 },
	{ 0x975b, 0x975c, PDF_CMAP_RANGE, 5008 },
	{ 0x975d, 0x9762, PDF_CMAP_TABLE, 12186 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 5010 },
	{ 0x9768, 0x976a, PDF_CMAP_TABLE, 12192 },
	{ 0x976c, 0x976e, PDF_CMAP_TABLE, 12195 },
	{ 0x9770, 0x9774, PDF_CMAP_TABLE, 12198 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 3946 },
	{ 0x9777, 0x9778, PDF_CMAP_RANGE, 9726 },
	{ 0x977a, 0x9785, PDF_CMAP_TABLE, 12203 },
	{ 0x9787, 0x978b, PDF_CMAP_TABLE, 12215 },
	{ 0x978d, 0x978f, PDF_CMAP_TABLE, 12220 },
	{ 0x9794, 0x9794, PDF_CMAP_SINGLE, 11513 },
	{ 0x9797, 0x97a6, PDF_CMAP_TABLE, 12223 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 12332 },
	{ 0x97aa, 0x97ae, PDF_CMAP_TABLE, 12239 },
	{ 0x97b1, 0x97b4, PDF_CMAP_TABLE, 12244 },
	{ 0x97b6, 0x97bb, PDF_CMAP_TABLE, 12248 },
	{ 0x97bd, 0x97c3, PDF_CMAP_TABLE, 12254 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 13388 },
	{ 0x97c6, 0x97c9, PDF_CMAP_TABLE, 12261 },
	{ 0x97cb, 0x97d0, PDF_CMAP_TABLE, 12265 },
	{ 0x97d2, 0x97d9, PDF_CMAP_TABLE, 12271 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 5617 },
	{ 0x97dd, 0x97df, PDF_CMAP_RANGE, 12654 },
	{ 0x97e0, 0x97e1, PDF_CMAP_TABLE, 12279 },
	{ 0x97e3, 0x97e3, PDF_CMAP_SINGLE, 13266 },
	{ 0x97e5, 0x97e6, PDF_CMAP_TABLE, 12281 },
	{ 0x97ed, 0x97ee, PDF_CMAP_TABLE, 12283 },
	{ 0x97f0, 0x97f3, PDF_CMAP_TABLE, 12285 },
	{ 0x97f5, 0x97f6, PDF_CMAP_TABLE, 12289 },
	{ 0x97f8, 0x97fb, PDF_CMAP_TABLE, 12291 },
	{ 0x97fd, 0x97fe, PDF_CMAP_RANGE, 12915 },
	{ 0x97ff, 0x9801, PDF_CMAP_TABLE, 12295 },
	{ 0x9802, 0x9803, PDF_CMAP_RANGE, 3039 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 8363 },
	{ 0x9805, 0x9806, PDF_CMAP_RANGE, 3504 },
	{ 0x9807, 0x9808, PDF_CMAP_TABLE, 12298 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 3950 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 3952 },
	{ 0x980d, 0x980e, PDF_CMAP_RANGE, 9730 },
	{ 0x980f, 0x980f, PDF_CMAP_SINGLE, 9729 },
	{ 0x9810, 0x9811, PDF_CMAP_RANGE, 3947 },
	{ 0x9812, 0x9816, PDF_CMAP_TABLE, 12300 },
	{ 0x9817, 0x9818, PDF_CMAP_RANGE, 4334 },
	{ 0x981b, 0x9821, PDF_CMAP_TABLE, 12305 },
	{ 0x9823, 0x9824, PDF_CMAP_TABLE, 12312 },
	{ 0x9826, 0x9829, PDF_CMAP_TABLE, 12314 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 4715 },
	{ 0x982d, 0x9830, PDF_CMAP_TABLE, 12318 },
	{ 0x9832, 0x9835, PDF_CMAP_TABLE, 12322 },
	{ 0x9837, 0x9839, PDF_CMAP_TABLE, 12326 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 5014 },
	{ 0x9841, 0x9841, PDF_CMAP_SINGLE, 11966 },
	{ 0x9843, 0x984f, PDF_CMAP_TABLE, 12329 },
	{ 0x9850, 0x9852, PDF_CMAP_RANGE, 12343 },
	{ 0x9853, 0x9853, PDF_CMAP_SINGLE, 5468 },
	{ 0x9857, 0x9859, PDF_CMAP_TABLE, 12342 },
	{ 0x985b, 0x9860, PDF_CMAP_TABLE, 12345 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 12918 },
	{ 0x9864, 0x9867, PDF_CMAP_TABLE, 12351 },
	{ 0x9869, 0x986c, PDF_CMAP_TABLE, 12355 },
	{ 0x986f, 0x9872, PDF_CMAP_TABLE, 12359 },
	{ 0x9873, 0x9874, PDF_CMAP_RANGE, 13607 },
	{ 0x9875, 0x9875, PDF_CMAP_SINGLE, 17677 },
	{ 0x98a8, 0x98a9, PDF_CMAP_TABLE, 12363 },
	{ 0x98ac, 0x98ac, PDF_CMAP_SINGLE, 9732 },
	{ 0x98ad, 0x98ae, PDF_CMAP_RANGE, 10339 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 4336 },
	{ 0x98b1, 0x98b4, PDF_CMAP_TABLE, 12365 },
	{ 0x98b6, 0x98c0, PDF_CMAP_TABLE, 12369 },
	{ 0x98c1, 0x98c2, PDF_CMAP_RANGE, 12921 },
	{ 0x98c3, 0x98c4, PDF_CMAP_TABLE, 12380 },
	{ 0x98c6, 0x98cc, PDF_CMAP_TABLE, 12382 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 17678 },
	{ 0x98db, 0x98dc, PDF_CMAP_TABLE, 12389 },
	{ 0x98de, 0x98e3, PDF_CMAP_TABLE, 12391 },
	{ 0x98e5, 0x98e7, PDF_CMAP_TABLE, 12397 },
	{ 0x98e9, 0x98ed, PDF_CMAP_TABLE, 12400 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 3509 },
	{ 0x98f1, 0x98f2, PDF_CMAP_TABLE, 12405 },
	{ 0x98f4, 0x98f6, PDF_CMAP_TABLE, 12407 },
	{ 0x98f9, 0x98fa, PDF_CMAP_TABLE, 12410 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 3953 },
	{ 0x98fd, 0x98fe, PDF_CMAP_RANGE, 3955 },
	{ 0x9900, 0x9900, PDF_CMAP_SINGLE, 10342 },
	{ 0x9902, 0x9903, PDF_CMAP_TABLE, 12412 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 4339 },
	{ 0x9907, 0x990a, PDF_CMAP_TABLE, 12414 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 4340 },
	{ 0x990e, 0x990e, PDF_CMAP_SINGLE, 17389 },
	{ 0x9910, 0x9915, PDF_CMAP_TABLE, 12418 },
	{ 0x9916, 0x9917, PDF_CMAP_RANGE, 10959 },
	{ 0x9918, 0x991c, PDF_CMAP_TABLE, 12424 },
	{ 0x991e, 0x991f, PDF_CMAP_TABLE, 12429 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 5023 },
	{ 0x9924, 0x9925, PDF_CMAP_TABLE, 12431 },
	{ 0x9927, 0x992a, PDF_CMAP_TABLE, 12433 },
	{ 0x992b, 0x992c, PDF_CMAP_RANGE, 11973 },
	{ 0x992d, 0x9933, PDF_CMAP_TABLE, 12437 },
	{ 0x9935, 0x9935, PDF_CMAP_SINGLE, 5304 },
	{ 0x9937, 0x993d, PDF_CMAP_TABLE, 12444 },
	{ 0x993e, 0x993f, PDF_CMAP_RANGE, 5470 },
	{ 0x9940, 0x9943, PDF_CMAP_TABLE, 12451 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 5623 },
	{ 0x9947, 0x994e, PDF_CMAP_TABLE, 12455 },
	{ 0x9950, 0x9959, PDF_CMAP_TABLE, 12463 },
	{ 0x995b, 0x995f, PDF_CMAP_TABLE, 12473 },
	{ 0x9961, 0x9963, PDF_CMAP_TABLE, 12478 },
	{ 0x9996, 0x9999, PDF_CMAP_TABLE, 12481 },
	{ 0x999b, 0x999e, PDF_CMAP_TABLE, 12485 },
	{ 0x99a1, 0x99a1, PDF_CMAP_SINGLE, 11984 },
	{ 0x99a3, 0x99a5, PDF_CMAP_TABLE, 12489 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 12668 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5722 },
	{ 0x99aa, 0x99b5, PDF_CMAP_TABLE, 12492 },
	{ 0x99b8, 0x99bd, PDF_CMAP_TABLE, 12504 },
	{ 0x99c1, 0x99c5, PDF_CMAP_TABLE, 12510 },
	{ 0x99c7, 0x99c7, PDF_CMAP_SINGLE, 10352 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 10968 },
	{ 0x99cb, 0x99d2, PDF_CMAP_TABLE, 12515 },
	{ 0x99d3, 0x99d4, PDF_CMAP_RANGE, 10965 },
	{ 0x99d5, 0x99dd, PDF_CMAP_TABLE, 12523 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 4724 },
	{ 0x99e1, 0x99e7, PDF_CMAP_TABLE, 12532 },
	{ 0x99e9, 0x99ea, PDF_CMAP_TABLE, 12539 },
	{ 0x99ec, 0x99ee, PDF_CMAP_TABLE, 12541 },
	{ 0x99f0, 0x99f1, PDF_CMAP_TABLE, 12544 },
	{ 0x99f4, 0x99fc, PDF_CMAP_TABLE, 12546 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 11993 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 5306 },
	{ 0x9a01, 0x9a07, PDF_CMAP_TABLE, 12555 },
	{ 0x9a09, 0x9a11, PDF_CMAP_TABLE, 12562 },
	{ 0x9a14, 0x9a16, PDF_CMAP_TABLE, 12571 },
	{ 0x9a19, 0x9a27, PDF_CMAP_TABLE, 12574 },
	{ 0x9a29, 0x9a32, PDF_CMAP_TABLE, 12589 },
	{ 0x9a34, 0x9a46, PDF_CMAP_TABLE, 12599 },
	{ 0x9a48, 0x9a4a, PDF_CMAP_TABLE, 12618 },
	{ 0x9a4c, 0x9a50, PDF_CMAP_TABLE, 12621 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 13280 },
	{ 0x9a53, 0x9a54, PDF_CMAP_RANGE, 13273 },
	{ 0x9a55, 0x9a59, PDF_CMAP_TABLE, 12626 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_RANGE, 5907 },
	{ 0x9a5c, 0x9a5c, PDF_CMAP_SINGLE, 15384 },
	{ 0x9a5e, 0x9a60, PDF_CMAP_TABLE, 12631 },
	{ 0x9a62, 0x9a65, PDF_CMAP_TABLE, 12634 },
	{ 0x9a66, 0x9a67, PDF_CMAP_RANGE, 13613 },
	{ 0x9a68, 0x9a6c, PDF_CMAP_TABLE, 12638 },
	{ 0x9a8f, 0x9a8f, PDF_CMAP_SINGLE, 17798 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2550 },
	{ 0x9aab, 0x9aab, PDF_CMAP_SINGLE, 9740 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 9739 },
	{ 0x9aaf, 0x9ab0, PDF_CMAP_RANGE, 4343 },
	{ 0x9ab1, 0x9ab4, PDF_CMAP_TABLE, 12643 },
	{ 0x9ab6, 0x9ac2, PDF_CMAP_TABLE, 12647 },
	{ 0x9ac6, 0x9ac7, PDF_CMAP_TABLE, 12660 },
	{ 0x9aca, 0x9aca, PDF_CMAP_SINGLE, 12940 },
	{ 0x9acd, 0x9acd, PDF_CMAP_SINGLE, 13123 },
	{ 0x9acf, 0x9ad2, PDF_CMAP_TABLE, 12662 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 5910 },
	{ 0x9ad5, 0x9ad8, PDF_CMAP_TABLE, 12666 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 12361 },
	{ 0x9adf, 0x9ae3, PDF_CMAP_TABLE, 12670 },
	{ 0x9ae6, 0x9ae7, PDF_CMAP_TABLE, 12675 },
	{ 0x9aeb, 0x9aed, PDF_CMAP_TABLE, 12677 },
	{ 0x9aee, 0x9aef, PDF_CMAP_RANGE, 4731 },
	{ 0x9af1, 0x9af4, PDF_CMAP_TABLE, 12680 },
	{ 0x9af6, 0x9af7, PDF_CMAP_TABLE, 12684 },
	{ 0x9af9, 0x9aff, PDF_CMAP_TABLE, 12686 },
	{ 0x9b01, 0x9b03, PDF_CMAP_TABLE, 12693 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 12363 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 5478 },
	{ 0x9b08, 0x9b12, PDF_CMAP_TABLE, 12696 },
	{ 0x9b14, 0x9b16, PDF_CMAP_TABLE, 12707 },
	{ 0x9b17, 0x9b18, PDF_CMAP_RANGE, 13125 },
	{ 0x9b19, 0x9b1a, PDF_CMAP_TABLE, 12710 },
	{ 0x9b1e, 0x9b20, PDF_CMAP_RANGE, 13393 },
	{ 0x9b22, 0x9b25, PDF_CMAP_TABLE, 12712 },
	{ 0x9b27, 0x9b2b, PDF_CMAP_TABLE, 12716 },
	{ 0x9b2d, 0x9b2f, PDF_CMAP_TABLE, 12721 },
	{ 0x9b31, 0x9b35, PDF_CMAP_TABLE, 12724 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 12688 },
	{ 0x9b39, 0x9b3c, PDF_CMAP_TABLE, 12729 },
	{ 0x9b3e, 0x9b3f, PDF_CMAP_RANGE, 10356 },
	{ 0x9b40, 0x9b40, PDF_CMAP_SINGLE, 14613 },
	{ 0x9b41, 0x9b42, PDF_CMAP_RANGE, 4346 },
	{ 0x9b43, 0x9b46, PDF_CMAP_TABLE, 12733 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 12002 },
	{ 0x9b4a, 0x9b52, PDF_CMAP_TABLE, 12737 },
	{ 0x9b54, 0x9b56, PDF_CMAP_TABLE, 12746 },
	{ 0x9b58, 0x9b5b, PDF_CMAP_TABLE, 12749 },
	{ 0x9b5f, 0x9b5f, PDF_CMAP_SINGLE, 10360 },
	{ 0x9b60, 0x9b61, PDF_CMAP_RANGE, 10358 },
	{ 0x9b64, 0x9b64, PDF_CMAP_SINGLE, 10990 },
	{ 0x9b66, 0x9b69, PDF_CMAP_TABLE, 12753 },
	{ 0x9b6c, 0x9b6c, PDF_CMAP_SINGLE, 10991 },
	{ 0x9b6f, 0x9b71, PDF_CMAP_TABLE, 12757 },
	{ 0x9b74, 0x9b77, PDF_CMAP_TABLE, 12760 },
	{ 0x9b7a, 0x9b83, PDF_CMAP_TABLE, 12764 },
	{ 0x9b85, 0x9b88, PDF_CMAP_TABLE, 12774 },
	{ 0x9b8b, 0x9b8b, PDF_CMAP_SINGLE, 16110 },
	{ 0x9b8d, 0x9b93, PDF_CMAP_TABLE, 12778 },
	{ 0x9b95, 0x9b95, PDF_CMAP_SINGLE, 11553 },
	{ 0x9b97, 0x9b97, PDF_CMAP_SINGLE, 16241 },
	{ 0x9b9a, 0x9b9b, PDF_CMAP_TABLE, 12785 },
	{ 0x9b9d, 0x9ba2, PDF_CMAP_TABLE, 12787 },
	{ 0x9ba4, 0x9ba6, PDF_CMAP_TABLE, 12793 },
	{ 0x9ba8, 0x9ba8, PDF_CMAP_SINGLE, 12004 },
	{ 0x9baa, 0x9bab, PDF_CMAP_TABLE, 12796 },
	{ 0x9bad, 0x9bb0, PDF_CMAP_TABLE, 12798 },
	{ 0x9bb5, 0x9bb6, PDF_CMAP_TABLE, 12802 },
	{ 0x9bb8, 0x9bb9, PDF_CMAP_TABLE, 12804 },
	{ 0x9bbd, 0x9bbd, PDF_CMAP_SINGLE, 12381 },
	{ 0x9bbf, 0x9bc1, PDF_CMAP_TABLE, 12806 },
	{ 0x9bc3, 0x9bc4, PDF_CMAP_TABLE, 12809 },
	{ 0x9bc6, 0x9bca, PDF_CMAP_TABLE, 12811 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 15420 },
	{ 0x9bd3, 0x9bd7, PDF_CMAP_TABLE, 12816 },
	{ 0x9bd9, 0x9bde, PDF_CMAP_TABLE, 12821 },
	{ 0x9be0, 0x9be2, PDF_CMAP_TABLE, 12827 },
	{ 0x9be4, 0x9be9, PDF_CMAP_TABLE, 12830 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 12689 },
	{ 0x9bec, 0x9bed, PDF_CMAP_TABLE, 12836 },
	{ 0x9bf0, 0x9bf1, PDF_CMAP_TABLE, 12838 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 18376 },
	{ 0x9bf7, 0x9bf8, PDF_CMAP_TABLE, 12840 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5484 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 14619 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 14618 },
	{ 0x9c05, 0x9c0e, PDF_CMAP_TABLE, 12842 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 15418 },
	{ 0x9c12, 0x9c15, PDF_CMAP_TABLE, 12852 },
	{ 0x9c17, 0x9c17, PDF_CMAP_SINGLE, 12955 },
	{ 0x9c1b, 0x9c1d, PDF_CMAP_TABLE, 12856 },
	{ 0x9c1f, 0x9c21, PDF_CMAP_TABLE, 12859 },
	{ 0x9c23, 0x9c26, PDF_CMAP_TABLE, 12862 },
	{ 0x9c28, 0x9c29, PDF_CMAP_RANGE, 13135 },
	{ 0x9c2b, 0x9c2f, PDF_CMAP_TABLE, 12866 },
	{ 0x9c31, 0x9c37, PDF_CMAP_TABLE, 12871 },
	{ 0x9c39, 0x9c41, PDF_CMAP_TABLE, 12878 },
	{ 0x9c44, 0x9c46, PDF_CMAP_TABLE, 12887 },
	{ 0x9c48, 0x9c50, PDF_CMAP_TABLE, 12890 },
	{ 0x9c52, 0x9c59, PDF_CMAP_TABLE, 12899 },
	{ 0x9c5d, 0x9c60, PDF_CMAP_TABLE, 12907 },
	{ 0x9c62, 0x9c63, PDF_CMAP_TABLE, 12911 },
	{ 0x9c66, 0x9c68, PDF_CMAP_TABLE, 12913 },
	{ 0x9c6d, 0x9c6e, PDF_CMAP_TABLE, 12916 },
	{ 0x9c71, 0x9c75, PDF_CMAP_TABLE, 12918 },
	{ 0x9c77, 0x9c78, PDF_CMAP_RANGE, 5984 },
	{ 0x9c79, 0x9c7c, PDF_CMAP_TABLE, 12923 },
	{ 0x9ce5, 0x9ce7, PDF_CMAP_TABLE, 12927 },
	{ 0x9ce9, 0x9cea, PDF_CMAP_TABLE, 12930 },
	{ 0x9ced, 0x9ced, PDF_CMAP_SINGLE, 9743 },
	{ 0x9cf1, 0x9cf2, PDF_CMAP_RANGE, 10361 },
	{ 0x9cf3, 0x9cf7, PDF_CMAP_TABLE, 12932 },
	{ 0x9cf9, 0x9cfd, PDF_CMAP_TABLE, 12937 },
	{ 0x9cff, 0x9d00, PDF_CMAP_TABLE, 12942 },
	{ 0x9d02, 0x9d09, PDF_CMAP_TABLE, 12944 },
	{ 0x9d0c, 0x9d0c, PDF_CMAP_SINGLE, 15430 },
	{ 0x9d10, 0x9d10, PDF_CMAP_SINGLE, 11565 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 5038 },
	{ 0x9d14, 0x9d19, PDF_CMAP_TABLE, 12952 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 5039 },
	{ 0x9d1d, 0x9d23, PDF_CMAP_TABLE, 12958 },
	{ 0x9d25, 0x9d26, PDF_CMAP_TABLE, 12965 },
	{ 0x9d28, 0x9d29, PDF_CMAP_TABLE, 12967 },
	{ 0x9d2d, 0x9d2d, PDF_CMAP_SINGLE, 12032 },
	{ 0x9d2e, 0x9d2f, PDF_CMAP_RANGE, 12019 },
	{ 0x9d30, 0x9d31, PDF_CMAP_TABLE, 12969 },
	{ 0x9d33, 0x9d34, PDF_CMAP_TABLE, 12971 },
	{ 0x9d36, 0x9d39, PDF_CMAP_TABLE, 12973 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 5311 },
	{ 0x9d3d, 0x9d41, PDF_CMAP_TABLE, 12977 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 12025 },
	{ 0x9d44, 0x9d45, PDF_CMAP_TABLE, 12982 },
	{ 0x9d49, 0x9d4c, PDF_CMAP_TABLE, 12984 },
	{ 0x9d4e, 0x9d54, PDF_CMAP_TABLE, 12988 },
	{ 0x9d56, 0x9d61, PDF_CMAP_TABLE, 12995 },
	{ 0x9d67, 0x9d73, PDF_CMAP_TABLE, 13007 },
	{ 0x9d74, 0x9d75, PDF_CMAP_RANGE, 12719 },
	{ 0x9d77, 0x9d79, PDF_CMAP_TABLE, 13020 },
	{ 0x9d7b, 0x9d8a, PDF_CMAP_TABLE, 13023 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 12715 },
	{ 0x9d90, 0x9d90, PDF_CMAP_SINGLE, 12964 },
	{ 0x9d92, 0x9d94, PDF_CMAP_TABLE, 13039 },
	{ 0x9d96, 0x9da5, PDF_CMAP_TABLE, 13042 },
	{ 0x9da6, 0x9da7, PDF_CMAP_RANGE, 12980 },
	{ 0x9da8, 0x9dad, PDF_CMAP_TABLE, 13058 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 5809 },
	{ 0x9db1, 0x9db8, PDF_CMAP_TABLE, 13064 },
	{ 0x9db9, 0x9dba, PDF_CMAP_RANGE, 13152 },
	{ 0x9dbb, 0x9dc5, PDF_CMAP_TABLE, 13072 },
	{ 0x9dc7, 0x9ddf, PDF_CMAP_TABLE, 13083 },
	{ 0x9de1, 0x9de6, PDF_CMAP_TABLE, 13108 },
	{ 0x9de8, 0x9de9, PDF_CMAP_TABLE, 13114 },
	{ 0x9deb, 0x9df0, PDF_CMAP_TABLE, 13116 },
	{ 0x9df2, 0x9df8, PDF_CMAP_TABLE, 13122 },
	{ 0x9df9, 0x9dfa, PDF_CMAP_RANGE, 5947 },
	{ 0x9dfb, 0x9dff, PDF_CMAP_TABLE, 13129 },
	{ 0x9e00, 0x9e01, PDF_CMAP_RANGE, 13501 },
	{ 0x9e02, 0x9e07, PDF_CMAP_TABLE, 13134 },
	{ 0x9e09, 0x9e15, PDF_CMAP_TABLE, 13140 },
	{ 0x9e17, 0x9e1f, PDF_CMAP_TABLE, 13153 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 3043 },
	{ 0x9e79, 0x9e7b, PDF_CMAP_TABLE, 13162 },
	{ 0x9e7c, 0x9e7d, PDF_CMAP_RANGE, 5949 },
	{ 0x9e7f, 0x9e85, PDF_CMAP_TABLE, 13165 },
	{ 0x9e86, 0x9e87, PDF_CMAP_RANGE, 11569 },
	{ 0x9e88, 0x9e8e, PDF_CMAP_TABLE, 13172 },
	{ 0x9e90, 0x9ea2, PDF_CMAP_TABLE, 13179 },
	{ 0x9ea4, 0x9eb1, PDF_CMAP_TABLE, 13198 },
	{ 0x9eb4, 0x9eb7, PDF_CMAP_TABLE, 13212 },
	{ 0x9ebb, 0x9ec4, PDF_CMAP_TABLE, 13216 },
	{ 0x9ec6, 0x9ec8, PDF_CMAP_TABLE, 13226 },
	{ 0x9ecc, 0x9ed1, PDF_CMAP_TABLE, 13229 },
	{ 0x9ed3, 0x9ed4, PDF_CMAP_TABLE, 13235 },
	{ 0x9ed5, 0x9ed6, PDF_CMAP_RANGE, 11573 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 5040 },
	{ 0x9eda, 0x9edb, PDF_CMAP_TABLE, 13237 },
	{ 0x9edc, 0x9edd, PDF_CMAP_RANGE, 5316 },
	{ 0x9ede, 0x9ee0, PDF_CMAP_TABLE, 13239 },
	{ 0x9ee2, 0x9ee2, PDF_CMAP_SINGLE, 18392 },
	{ 0x9ee4, 0x9ee8, PDF_CMAP_TABLE, 13242 },
	{ 0x9eeb, 0x9eeb, PDF_CMAP_SINGLE, 13164 },
	{ 0x9eed, 0x9ef1, PDF_CMAP_TABLE, 13247 },
	{ 0x9ef2, 0x9ef3, PDF_CMAP_RANGE, 13427 },
	{ 0x9ef4, 0x9f02, PDF_CMAP_TABLE, 13252 },
	{ 0x9f06, 0x9f0a, PDF_CMAP_TABLE, 13267 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3963 },
	{ 0x9f0f, 0x9f10, PDF_CMAP_RANGE, 11006 },
	{ 0x9f12, 0x9f13, PDF_CMAP_TABLE, 13272 },
	{ 0x9f15, 0x9f1c, PDF_CMAP_TABLE, 13274 },
	{ 0x9f1e, 0x9f1e, PDF_CMAP_SINGLE, 13508 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 3965 },
	{ 0x9f22, 0x9f39, PDF_CMAP_TABLE, 13282 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 4352 },
	{ 0x9f3d, 0x9f3e, PDF_CMAP_TABLE, 13306 },
	{ 0x9f40, 0x9f41, PDF_CMAP_RANGE, 12737 },
	{ 0x9f42, 0x9f50, PDF_CMAP_TABLE, 13308 },
	{ 0x9f52, 0x9f55, PDF_CMAP_TABLE, 13323 },
	{ 0x9f56, 0x9f58, PDF_CMAP_RANGE, 12740 },
	{ 0x9f59, 0x9f65, PDF_CMAP_TABLE, 13327 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 5817 },
	{ 0x9f69, 0x9f6c, PDF_CMAP_TABLE, 13340 },
	{ 0x9f6e, 0x9f6f, PDF_CMAP_RANGE, 13438 },
	{ 0x9f70, 0x9f72, PDF_CMAP_TABLE, 13344 },
	{ 0x9f74, 0x9f76, PDF_CMAP_RANGE, 13510 },
	{ 0x9f77, 0x9f7b, PDF_CMAP_TABLE, 13347 },
	{ 0x9f7e, 0x9f7f, PDF_CMAP_TABLE, 13352 },
	{ 0x9f8d, 0x9f8e, PDF_CMAP_TABLE, 13354 },
	{ 0x9f90, 0x9f92, PDF_CMAP_TABLE, 13356 },
	{ 0x9f94, 0x9f95, PDF_CMAP_TABLE, 13359 },
	{ 0x9f98, 0x9f99, PDF_CMAP_TABLE, 13361 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 5043 },
	{ 0x9f9f, 0x9fa0, PDF_CMAP_TABLE, 13363 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 13325 },
	{ 0x9fa4, 0x9fa5, PDF_CMAP_TABLE, 13365 },
	{ 0xe000, 0xe000, PDF_CMAP_SINGLE, 16395 },
	{ 0xe002, 0xe003, PDF_CMAP_RANGE, 16397 },
	{ 0xe009, 0xe00a, PDF_CMAP_RANGE, 16404 },
	{ 0xe00c, 0xe00c, PDF_CMAP_SINGLE, 16407 },
	{ 0xe00e, 0xe00e, PDF_CMAP_SINGLE, 16409 },
	{ 0xe012, 0xe016, PDF_CMAP_RANGE, 16413 },
	{ 0xe018, 0xe018, PDF_CMAP_SINGLE, 16419 },
	{ 0xe01a, 0xe01e, PDF_CMAP_RANGE, 16421 },
	{ 0xe020, 0xe020, PDF_CMAP_SINGLE, 16427 },
	{ 0xe022, 0xe023, PDF_CMAP_RANGE, 16429 },
	{ 0xe025, 0xe025, PDF_CMAP_SINGLE, 16432 },
	{ 0xe02d, 0xe02e, PDF_CMAP_RANGE, 16440 },
	{ 0xe030, 0xe030, PDF_CMAP_SINGLE, 16443 },
	{ 0xe032, 0xe038, PDF_CMAP_RANGE, 16445 },
	{ 0xe03b, 0xe03c, PDF_CMAP_RANGE, 16454 },
	{ 0xe045, 0xe045, PDF_CMAP_SINGLE, 16464 },
	{ 0xe04c, 0xe04c, PDF_CMAP_SINGLE, 16471 },
	{ 0xe05a, 0xe05a, PDF_CMAP_SINGLE, 16485 },
	{ 0xe05c, 0xe05c, PDF_CMAP_SINGLE, 16487 },
	{ 0xe05f, 0xe061, PDF_CMAP_RANGE, 16490 },
	{ 0xe064, 0xe064, PDF_CMAP_SINGLE, 16495 },
	{ 0xe066, 0xe066, PDF_CMAP_SINGLE, 16497 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 16503 },
	{ 0xe071, 0xe071, PDF_CMAP_SINGLE, 16508 },
	{ 0xe074, 0xe075, PDF_CMAP_RANGE, 16511 },
	{ 0xe078, 0xe078, PDF_CMAP_SINGLE, 16515 },
	{ 0xe07d, 0xe07d, PDF_CMAP_SINGLE, 16520 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 16530 },
	{ 0xe089, 0xe08a, PDF_CMAP_RANGE, 16532 },
	{ 0xe08c, 0xe08c, PDF_CMAP_SINGLE, 16535 },
	{ 0xe093, 0xe094, PDF_CMAP_RANGE, 16542 },
	{ 0xe099, 0xe09a, PDF_CMAP_RANGE, 16548 },
	{ 0xe09c, 0xe09e, PDF_CMAP_RANGE, 16551 },
	{ 0xe0a6, 0xe0a7, PDF_CMAP_RANGE, 16561 },
	{ 0xe0a9, 0xe0a9, PDF_CMAP_SINGLE, 16564 },
	{ 0xe0ab, 0xe0ab, PDF_CMAP_SINGLE, 16566 },
	{ 0xe0ae, 0xe0ae, PDF_CMAP_SINGLE, 16569 },
	{ 0xe0ba, 0xe0ba, PDF_CMAP_SINGLE, 16581 },
	{ 0xe0bc, 0xe0bc, PDF_CMAP_SINGLE, 16583 },
	{ 0xe0be, 0xe0be, PDF_CMAP_SINGLE, 16585 },
	{ 0xe0c3, 0xe0c3, PDF_CMAP_SINGLE, 16590 },
	{ 0xe0c5, 0xe0c5, PDF_CMAP_SINGLE, 16592 },
	{ 0xe0c7, 0xe0c8, PDF_CMAP_RANGE, 16594 },
	{ 0xe0ca, 0xe0ca, PDF_CMAP_SINGLE, 16597 },
	{ 0xe0d0, 0xe0d0, PDF_CMAP_SINGLE, 16603 },
	{ 0xe0d4, 0xe0d5, PDF_CMAP_RANGE, 16607 },
	{ 0xe0dc, 0xe0dc, PDF_CMAP_SINGLE, 16615 },
	{ 0xe0df, 0xe0df, PDF_CMAP_SINGLE, 16618 },
	{ 0xe0e2, 0xe0e5, PDF_CMAP_RANGE, 16621 },
	{ 0xe0e7, 0xe0e9, PDF_CMAP_RANGE, 16626 },
	{ 0xe0eb, 0xe0ec, PDF_CMAP_RANGE, 16630 },
	{ 0xe0ee, 0xe0ee, PDF_CMAP_SINGLE, 16633 },
	{ 0xe0f2, 0xe0f2, PDF_CMAP_SINGLE, 16637 },
	{ 0xe0f8, 0xe0f9, PDF_CMAP_RANGE, 16643 },
	{ 0xe0fb, 0xe0fb, PDF_CMAP_SINGLE, 16645 },
	{ 0xe0fd, 0xe0fd, PDF_CMAP_SINGLE, 16647 },
	{ 0xe103, 0xe103, PDF_CMAP_SINGLE, 16653 },
	{ 0xe106, 0xe106, PDF_CMAP_SINGLE, 16656 },
	{ 0xe109, 0xe10b, PDF_CMAP_RANGE, 16658 },
	{ 0xe110, 0xe110, PDF_CMAP_SINGLE, 16665 },
	{ 0xe114, 0xe115, PDF_CMAP_RANGE, 16669 },
	{ 0xe11c, 0xe11d, PDF_CMAP_RANGE, 16677 },
	{ 0xe121, 0xe121, PDF_CMAP_SINGLE, 16682 },
	{ 0xe126, 0xe128, PDF_CMAP_RANGE, 16687 },
	{ 0xe130, 0xe131, PDF_CMAP_RANGE, 16697 },
	{ 0xe136, 0xe136, PDF_CMAP_SINGLE, 16703 },
	{ 0xe139, 0xe139, PDF_CMAP_SINGLE, 16706 },
	{ 0xe141, 0xe142, PDF_CMAP_RANGE, 16714 },
	{ 0xe145, 0xe145, PDF_CMAP_SINGLE, 16717 },
	{ 0xe147, 0xe148, PDF_CMAP_RANGE, 16719 },
	{ 0xe14a, 0xe14a, PDF_CMAP_SINGLE, 16721 },
	{ 0xe14d, 0xe14d, PDF_CMAP_SINGLE, 16723 },
	{ 0xe150, 0xe150, PDF_CMAP_SINGLE, 16726 },
	{ 0xe154, 0xe157, PDF_CMAP_RANGE, 16730 },
	{ 0xe159, 0xe15a, PDF_CMAP_RANGE, 16735 },
	{ 0xe15d, 0xe15d, PDF_CMAP_SINGLE, 18767 },
	{ 0xe162, 0xe162, PDF_CMAP_SINGLE, 16743 },
	{ 0xe168, 0xe16a, PDF_CMAP_RANGE, 16749 },
	{ 0xe171, 0xe172, PDF_CMAP_RANGE, 16757 },
	{ 0xe175, 0xe175, PDF_CMAP_SINGLE, 16761 },
	{ 0xe178, 0xe179, PDF_CMAP_RANGE, 16764 },
	{ 0xe17b, 0xe17c, PDF_CMAP_RANGE, 16767 },
	{ 0xe17f, 0xe180, PDF_CMAP_RANGE, 16771 },
	{ 0xe185, 0xe185, PDF_CMAP_SINGLE, 16777 },
	{ 0xe187, 0xe187, PDF_CMAP_SINGLE, 16779 },
	{ 0xe189, 0xe189, PDF_CMAP_SINGLE, 16781 },
	{ 0xe18b, 0xe18b, PDF_CMAP_SINGLE, 16783 },
	{ 0xe18f, 0xe190, PDF_CMAP_RANGE, 16787 },
	{ 0xe193, 0xe193, PDF_CMAP_SINGLE, 16790 },
	{ 0xe195, 0xe196, PDF_CMAP_TABLE, 13367 },
	{ 0xe19f, 0xe1a0, PDF_CMAP_RANGE, 16801 },
	{ 0xe1a4, 0xe1a4, PDF_CMAP_SINGLE, 18772 },
	{ 0xe1a6, 0xe1a6, PDF_CMAP_SINGLE, 16808 },
	{ 0xe1aa, 0xe1aa, PDF_CMAP_SINGLE, 16812 },
	{ 0xe1ae, 0xe1ae, PDF_CMAP_SINGLE, 16816 },
	{ 0xe1b0, 0xe1b1, PDF_CMAP_RANGE, 16818 },
	{ 0xe1b3, 0xe1b4, PDF_CMAP_RANGE, 16821 },
	{ 0xe1b6, 0xe1b6, PDF_CMAP_SINGLE, 16824 },
	{ 0xe1b8, 0xe1b8, PDF_CMAP_SINGLE, 16826 },
	{ 0xe1bb, 0xe1bb, PDF_CMAP_SINGLE, 18773 },
	{ 0xe1bc, 0xe1bf, PDF_CMAP_RANGE, 16829 },
	{ 0xe1c1, 0xe1c1, PDF_CMAP_SINGLE, 16834 },
	{ 0xe1c4, 0xe1c5, PDF_CMAP_RANGE, 16837 },
	{ 0xe1c8, 0xe1c8, PDF_CMAP_SINGLE, 16840 },
	{ 0xe1cc, 0xe1cd, PDF_CMAP_RANGE, 16844 },
	{ 0xe1d4, 0xe1d5, PDF_CMAP_RANGE, 16852 },
	{ 0xe1d7, 0xe1d7, PDF_CMAP_SINGLE, 16855 },
	{ 0xe1da, 0xe1da, PDF_CMAP_SINGLE, 16858 },
	{ 0xe1dd, 0xe1dd, PDF_CMAP_SINGLE, 16861 },
	{ 0xe1e2, 0xe1e2, PDF_CMAP_SINGLE, 16865 },
	{ 0xe1e6, 0xe1e6, PDF_CMAP_SINGLE, 16869 },
	{ 0xe1eb, 0xe1eb, PDF_CMAP_SINGLE, 16874 },
	{ 0xe1ef, 0xe1f0, PDF_CMAP_RANGE, 16878 },
	{ 0xe1f2, 0xe1f2, PDF_CMAP_SINGLE, 16881 },
	{ 0xe1f7, 0xe1f7, PDF_CMAP_SINGLE, 16886 },
	{ 0xe1fa, 0xe1fa, PDF_CMAP_SINGLE, 16889 },
	{ 0xe1fe, 0xe1fe, PDF_CMAP_SINGLE, 16893 },
	{ 0xe200, 0xe202, PDF_CMAP_TABLE, 13369 },
	{ 0xe205, 0xe205, PDF_CMAP_SINGLE, 16900 },
	{ 0xe207, 0xe207, PDF_CMAP_SINGLE, 16902 },
	{ 0xe20a, 0xe20c, PDF_CMAP_RANGE, 16905 },
	{ 0xe210, 0xe210, PDF_CMAP_SINGLE, 16911 },
	{ 0xe212, 0xe216, PDF_CMAP_RANGE, 16913 },
	{ 0xe219, 0xe219, PDF_CMAP_SINGLE, 16920 },
	{ 0xe21b, 0xe21c, PDF_CMAP_RANGE, 16922 },
	{ 0xe21f, 0xe221, PDF_CMAP_RANGE, 16926 },
	{ 0xe226, 0xe228, PDF_CMAP_RANGE, 16933 },
	{ 0xe22a, 0xe22a, PDF_CMAP_SINGLE, 16937 },
	{ 0xe22f, 0xe22f, PDF_CMAP_SINGLE, 16942 },
	{ 0xe231, 0xe231, PDF_CMAP_SINGLE, 16944 },
	{ 0xe233, 0xe233, PDF_CMAP_SINGLE, 16946 },
	{ 0xe237, 0xe237, PDF_CMAP_SINGLE, 16950 },
	{ 0xe23b, 0xe23b, PDF_CMAP_SINGLE, 16954 },
	{ 0xe23d, 0xe23e, PDF_CMAP_RANGE, 16956 },
	{ 0xe245, 0xe245, PDF_CMAP_SINGLE, 16964 },
	{ 0xe248, 0xe248, PDF_CMAP_SINGLE, 16967 },
	{ 0xe24a, 0xe24f, PDF_CMAP_RANGE, 16969 },
	{ 0xe251, 0xe254, PDF_CMAP_RANGE, 16976 },
	{ 0xe261, 0xe261, PDF_CMAP_SINGLE, 16992 },
	{ 0xe268, 0xe268, PDF_CMAP_SINGLE, 16998 },
	{ 0xe26b, 0xe26b, PDF_CMAP_SINGLE, 17001 },
	{ 0xe26d, 0xe26f, PDF_CMAP_RANGE, 17003 },
	{ 0xe271, 0xe271, PDF_CMAP_SINGLE, 17007 },
	{ 0xe27a, 0xe27a, PDF_CMAP_SINGLE, 17016 },
	{ 0xe27d, 0xe27d, PDF_CMAP_SINGLE, 17019 },
	{ 0xe282, 0xe282, PDF_CMAP_SINGLE, 17024 },
	{ 0xe288, 0xe289, PDF_CMAP_RANGE, 17030 },
	{ 0xe28b, 0xe28b, PDF_CMAP_SINGLE, 17033 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 17038 },
	{ 0xe294, 0xe296, PDF_CMAP_RANGE, 17042 },
	{ 0xe299, 0xe29b, PDF_CMAP_RANGE, 17047 },
	{ 0xe29f, 0xe29f, PDF_CMAP_SINGLE, 17053 },
	{ 0xe2a1, 0xe2a1, PDF_CMAP_SINGLE, 18779 },
	{ 0xe2a4, 0xe2a4, PDF_CMAP_SINGLE, 17057 },
	{ 0xe2a6, 0xe2a6, PDF_CMAP_SINGLE, 17059 },
	{ 0xe2b2, 0xe2b5, PDF_CMAP_RANGE, 17070 },
	{ 0xe2b7, 0xe2b8, PDF_CMAP_RANGE, 17075 },
	{ 0xe2ba, 0xe2ba, PDF_CMAP_SINGLE, 17078 },
	{ 0xe2be, 0xe2bf, PDF_CMAP_RANGE, 17081 },
	{ 0xe2c1, 0xe2c1, PDF_CMAP_SINGLE, 17084 },
	{ 0xe2c7, 0xe2c7, PDF_CMAP_SINGLE, 17090 },
	{ 0xe2c9, 0xe2c9, PDF_CMAP_SINGLE, 17092 },
	{ 0xe2d4, 0xe2d4, PDF_CMAP_SINGLE, 17103 },
	{ 0xe2d8, 0xe2d9, PDF_CMAP_RANGE, 17107 },
	{ 0xe2de, 0xe2de, PDF_CMAP_SINGLE, 17113 },
	{ 0xe2e0, 0xe2e2, PDF_CMAP_RANGE, 17115 },
	{ 0xe2e7, 0xe2e7, PDF_CMAP_SINGLE, 17122 },
	{ 0xe2e9, 0xe2e9, PDF_CMAP_SINGLE, 17124 },
	{ 0xe2eb, 0xe2ed, PDF_CMAP_RANGE, 17126 },
	{ 0xe2f0, 0xe2f1, PDF_CMAP_RANGE, 18781 },
	{ 0xe2f8, 0xe2f8, PDF_CMAP_SINGLE, 17137 },
	{ 0xe2fc, 0xe2fc, PDF_CMAP_SINGLE, 17141 },
	{ 0xe2ff, 0xe300, PDF_CMAP_RANGE, 18783 },
	{ 0xe301, 0xe301, PDF_CMAP_SINGLE, 17144 },
	{ 0xe306, 0xe306, PDF_CMAP_SINGLE, 17149 },
	{ 0xe308, 0xe308, PDF_CMAP_SINGLE, 17151 },
	{ 0xe30c, 0xe30c, PDF_CMAP_SINGLE, 17155 },
	{ 0xe30f, 0xe30f, PDF_CMAP_SINGLE, 17158 },
	{ 0xe311, 0xe311, PDF_CMAP_SINGLE, 14123 },
	{ 0xe313, 0xe313, PDF_CMAP_SINGLE, 14125 },
	{ 0xe315, 0xe315, PDF_CMAP_SINGLE, 14127 },
	{ 0xe318, 0xe318, PDF_CMAP_SINGLE, 14129 },
	{ 0xe31b, 0xe31c, PDF_CMAP_RANGE, 14132 },
	{ 0xe31f, 0xe31f, PDF_CMAP_SINGLE, 14136 },
	{ 0xe325, 0xe325, PDF_CMAP_SINGLE, 14142 },
	{ 0xe32c, 0xe32c, PDF_CMAP_SINGLE, 14149 },
	{ 0xe330, 0xe331, PDF_CMAP_RANGE, 14153 },
	{ 0xe335, 0xe336, PDF_CMAP_RANGE, 14158 },
	{ 0xe33b, 0xe33b, PDF_CMAP_SINGLE, 14164 },
	{ 0xe33d, 0xe33d, PDF_CMAP_SINGLE, 18074 },
	{ 0xe342, 0xe342, PDF_CMAP_SINGLE, 14170 },
	{ 0xe345, 0xe345, PDF_CMAP_SINGLE, 14173 },
	{ 0xe348, 0xe348, PDF_CMAP_SINGLE, 14175 },
	{ 0xe352, 0xe352, PDF_CMAP_SINGLE, 14184 },
	{ 0xe354, 0xe354, PDF_CMAP_SINGLE, 14186 },
	{ 0xe35b, 0xe35c, PDF_CMAP_RANGE, 14192 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 14197 },
	{ 0xe362, 0xe362, PDF_CMAP_SINGLE, 14199 },
	{ 0xe364, 0xe365, PDF_CMAP_RANGE, 14201 },
	{ 0xe36f, 0xe36f, PDF_CMAP_SINGLE, 14212 },
	{ 0xe372, 0xe372, PDF_CMAP_SINGLE, 14215 },
	{ 0xe374, 0xe375, PDF_CMAP_RANGE, 14217 },
	{ 0xe377, 0xe377, PDF_CMAP_SINGLE, 14220 },
	{ 0xe379, 0xe379, PDF_CMAP_SINGLE, 14221 },
	{ 0xe37b, 0xe37b, PDF_CMAP_SINGLE, 14223 },
	{ 0xe381, 0xe382, PDF_CMAP_RANGE, 14229 },
	{ 0xe386, 0xe386, PDF_CMAP_SINGLE, 14234 },
	{ 0xe388, 0xe38a, PDF_CMAP_RANGE, 14236 },
	{ 0xe38d, 0xe38d, PDF_CMAP_SINGLE, 14241 },
	{ 0xe391, 0xe391, PDF_CMAP_SINGLE, 14245 },
	{ 0xe393, 0xe394, PDF_CMAP_TABLE, 13372 },
	{ 0xe396, 0xe396, PDF_CMAP_SINGLE, 14249 },
	{ 0xe398, 0xe398, PDF_CMAP_SINGLE, 14251 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 14253 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 18081 },
	{ 0xe39f, 0xe3a0, PDF_CMAP_RANGE, 14257 },
	{ 0xe3a2, 0xe3a2, PDF_CMAP_SINGLE, 14260 },
	{ 0xe3a5, 0xe3a5, PDF_CMAP_SINGLE, 18082 },
	{ 0xe3a6, 0xe3a9, PDF_CMAP_RANGE, 14264 },
	{ 0xe3ac, 0xe3ac, PDF_CMAP_SINGLE, 14270 },
	{ 0xe3b1, 0xe3b2, PDF_CMAP_RANGE, 14275 },
	{ 0xe3b4, 0xe3b5, PDF_CMAP_RANGE, 14278 },
	{ 0xe3b9, 0xe3bd, PDF_CMAP_RANGE, 14283 },
	{ 0xe3c3, 0xe3c3, PDF_CMAP_SINGLE, 14293 },
	{ 0xe3c8, 0xe3c8, PDF_CMAP_SINGLE, 14297 },
	{ 0xe3cc, 0xe3cd, PDF_CMAP_TABLE, 13374 },
	{ 0xe3cf, 0xe3cf, PDF_CMAP_SINGLE, 14303 },
	{ 0xe3d5, 0xe3d5, PDF_CMAP_SINGLE, 18085 },
	{ 0xe3d8, 0xe3da, PDF_CMAP_RANGE, 14311 },
	{ 0xe3e1, 0xe3e1, PDF_CMAP_SINGLE, 14320 },
	{ 0xe3e3, 0xe3e3, PDF_CMAP_SINGLE, 14322 },
	{ 0xe3e7, 0xe3e7, PDF_CMAP_SINGLE, 18086 },
	{ 0xe3ed, 0xe3ee, PDF_CMAP_RANGE, 14332 },
	{ 0xe3f0, 0xe3f0, PDF_CMAP_SINGLE, 14335 },
	{ 0xe3f2, 0xe3f4, PDF_CMAP_RANGE, 14337 },
	{ 0xe3f8, 0xe3f8, PDF_CMAP_SINGLE, 14343 },
	{ 0xe3fa, 0xe3fa, PDF_CMAP_SINGLE, 14345 },
	{ 0xe3fd, 0xe3fd, PDF_CMAP_SINGLE, 14348 },
	{ 0xe3ff, 0xe3ff, PDF_CMAP_SINGLE, 14350 },
	{ 0xe409, 0xe40b, PDF_CMAP_RANGE, 14360 },
	{ 0xe414, 0xe414, PDF_CMAP_SINGLE, 14369 },
	{ 0xe41b, 0xe41b, PDF_CMAP_SINGLE, 14376 },
	{ 0xe41d, 0xe41e, PDF_CMAP_RANGE, 14378 },
	{ 0xe423, 0xe423, PDF_CMAP_SINGLE, 14384 },
	{ 0xe425, 0xe426, PDF_CMAP_TABLE, 13376 },
	{ 0xe428, 0xe428, PDF_CMAP_SINGLE, 14388 },
	{ 0xe42c, 0xe42d, PDF_CMAP_RANGE, 14392 },
	{ 0xe432, 0xe432, PDF_CMAP_SINGLE, 14398 },
	{ 0xe436, 0xe436, PDF_CMAP_SINGLE, 14402 },
	{ 0xe43a, 0xe43c, PDF_CMAP_RANGE, 14406 },
	{ 0xe43e, 0xe43e, PDF_CMAP_SINGLE, 14410 },
	{ 0xe440, 0xe441, PDF_CMAP_RANGE, 14412 },
	{ 0xe444, 0xe444, PDF_CMAP_SINGLE, 14416 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 18093 },
	{ 0xe44c, 0xe44e, PDF_CMAP_RANGE, 14423 },
	{ 0xe450, 0xe451, PDF_CMAP_RANGE, 14427 },
	{ 0xe459, 0xe45a, PDF_CMAP_RANGE, 14436 },
	{ 0xe45d, 0xe45e, PDF_CMAP_RANGE, 14440 },
	{ 0xe461, 0xe463, PDF_CMAP_RANGE, 14444 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 14448 },
	{ 0xe467, 0xe469, PDF_CMAP_RANGE, 18095 },
	{ 0xe46a, 0xe46a, PDF_CMAP_SINGLE, 14451 },
	{ 0xe46c, 0xe46c, PDF_CMAP_SINGLE, 14453 },
	{ 0xe46e, 0xe46f, PDF_CMAP_RANGE, 14455 },
	{ 0xe471, 0xe472, PDF_CMAP_RANGE, 14458 },
	{ 0xe474, 0xe474, PDF_CMAP_SINGLE, 14461 },
	{ 0xe476, 0xe476, PDF_CMAP_SINGLE, 14463 },
	{ 0xe47e, 0xe47f, PDF_CMAP_RANGE, 14470 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 14476 },
	{ 0xe486, 0xe48a, PDF_CMAP_RANGE, 14477 },
	{ 0xe48d, 0xe48e, PDF_CMAP_RANGE, 14484 },
	{ 0xe491, 0xe492, PDF_CMAP_RANGE, 14487 },
	{ 0xe495, 0xe497, PDF_CMAP_RANGE, 14491 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 18100 },
	{ 0xe4a2, 0xe4a4, PDF_CMAP_RANGE, 14503 },
	{ 0xe4ab, 0xe4ab, PDF_CMAP_SINGLE, 14512 },
	{ 0xe4af, 0xe4af, PDF_CMAP_SINGLE, 14516 },
	{ 0xe4b3, 0xe4ba, PDF_CMAP_RANGE, 14520 },
	{ 0xe4c1, 0xe4c1, PDF_CMAP_SINGLE, 14534 },
	{ 0xe4c6, 0xe4c6, PDF_CMAP_SINGLE, 14539 },
	{ 0xe4c9, 0xe4c9, PDF_CMAP_SINGLE, 14542 },
	{ 0xe4cd, 0xe4ce, PDF_CMAP_RANGE, 14546 },
	{ 0xe4d6, 0xe4d6, PDF_CMAP_SINGLE, 14555 },
	{ 0xe4d9, 0xe4d9, PDF_CMAP_SINGLE, 14558 },
	{ 0xe4dc, 0xe4dd, PDF_CMAP_RANGE, 14561 },
	{ 0xe4e4, 0xe4e4, PDF_CMAP_SINGLE, 14569 },
	{ 0xe4e6, 0xe4e8, PDF_CMAP_RANGE, 14571 },
	{ 0xe4ee, 0xe4ef, PDF_CMAP_RANGE, 14579 },
	{ 0xe4f6, 0xe4f6, PDF_CMAP_SINGLE, 14587 },
	{ 0xe4f8, 0xe4f9, PDF_CMAP_RANGE, 14589 },
	{ 0xe4ff, 0xe4ff, PDF_CMAP_SINGLE, 14596 },
	{ 0xe501, 0xe502, PDF_CMAP_RANGE, 14598 },
	{ 0xe505, 0xe505, PDF_CMAP_SINGLE, 14602 },
	{ 0xe50a, 0xe50a, PDF_CMAP_SINGLE, 14607 },
	{ 0xe50d, 0xe50d, PDF_CMAP_SINGLE, 18101 },
	{ 0xe50f, 0xe50f, PDF_CMAP_SINGLE, 14611 },
	{ 0xe512, 0xe512, PDF_CMAP_SINGLE, 14614 },
	{ 0xe514, 0xe514, PDF_CMAP_SINGLE, 14616 },
	{ 0xe516, 0xe516, PDF_CMAP_SINGLE, 18102 },
	{ 0xe51a, 0xe51a, PDF_CMAP_SINGLE, 14621 },
	{ 0xe51c, 0xe521, PDF_CMAP_RANGE, 14623 },
	{ 0xe524, 0xe524, PDF_CMAP_SINGLE, 14631 },
	{ 0xe528, 0xe528, PDF_CMAP_SINGLE, 18104 },
	{ 0xe529, 0xe52c, PDF_CMAP_RANGE, 14634 },
	{ 0xe52e, 0xe52f, PDF_CMAP_RANGE, 14639 },
	{ 0xe537, 0xe537, PDF_CMAP_SINGLE, 14648 },
	{ 0xe53e, 0xe53e, PDF_CMAP_SINGLE, 14655 },
	{ 0xe543, 0xe543, PDF_CMAP_SINGLE, 14660 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 14668 },
	{ 0xe554, 0xe555, PDF_CMAP_RANGE, 14677 },
	{ 0xe557, 0xe559, PDF_CMAP_RANGE, 14680 },
	{ 0xe55b, 0xe55c, PDF_CMAP_RANGE, 14684 },
	{ 0xe55e, 0xe55f, PDF_CMAP_RANGE, 14687 },
	{ 0xe562, 0xe562, PDF_CMAP_SINGLE, 14691 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 14695 },
	{ 0xe56d, 0xe56e, PDF_CMAP_RANGE, 14702 },
	{ 0xe570, 0xe570, PDF_CMAP_SINGLE, 14705 },
	{ 0xe572, 0xe573, PDF_CMAP_RANGE, 14707 },
	{ 0xe575, 0xe577, PDF_CMAP_RANGE, 14710 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 14714 },
	{ 0xe57c, 0xe57e, PDF_CMAP_RANGE, 14717 },
	{ 0xe580, 0xe580, PDF_CMAP_SINGLE, 14721 },
	{ 0xe582, 0xe586, PDF_CMAP_RANGE, 14723 },
	{ 0xe588, 0xe588, PDF_CMAP_SINGLE, 14729 },
	{ 0xe58b, 0xe590, PDF_CMAP_RANGE, 14732 },
	{ 0xe592, 0xe594, PDF_CMAP_RANGE, 14739 },
	{ 0xe59a, 0xe59a, PDF_CMAP_SINGLE, 14747 },
	{ 0xe59d, 0xe5a1, PDF_CMAP_RANGE, 14750 },
	{ 0xe5a4, 0xe5a4, PDF_CMAP_SINGLE, 14757 },
	{ 0xe5a6, 0xe5a8, PDF_CMAP_RANGE, 14759 },
	{ 0xe5aa, 0xe5aa, PDF_CMAP_SINGLE, 14763 },
	{ 0xe5af, 0xe5af, PDF_CMAP_SINGLE, 14768 },
	{ 0xe5b1, 0xe5b2, PDF_CMAP_TABLE, 13378 },
	{ 0xe5b5, 0xe5b5, PDF_CMAP_SINGLE, 14774 },
	{ 0xe5b7, 0xe5b8, PDF_CMAP_RANGE, 14776 },
	{ 0xe5ba, 0xe5bb, PDF_CMAP_RANGE, 14779 },
	{ 0xe5be, 0xe5c0, PDF_CMAP_RANGE, 14783 },
	{ 0xe5c2, 0xe5c2, PDF_CMAP_SINGLE, 14787 },
	{ 0xe5c8, 0xe5c9, PDF_CMAP_RANGE, 14793 },
	{ 0xe5d7, 0xe5d9, PDF_CMAP_RANGE, 14804 },
	{ 0xe5db, 0xe5dc, PDF_CMAP_RANGE, 14808 },
	{ 0xe5df, 0xe5df, PDF_CMAP_SINGLE, 14812 },
	{ 0xe5e1, 0xe5e3, PDF_CMAP_RANGE, 14814 },
	{ 0xe5e5, 0xe5e6, PDF_CMAP_RANGE, 14818 },
	{ 0xe5e9, 0xe5e9, PDF_CMAP_SINGLE, 14822 },
	{ 0xe5ee, 0xe5f1, PDF_CMAP_RANGE, 14827 },
	{ 0xe5f3, 0xe5f3, PDF_CMAP_SINGLE, 14832 },
	{ 0xe5fb, 0xe5fc, PDF_CMAP_RANGE, 14840 },
	{ 0xe600, 0xe600, PDF_CMAP_SINGLE, 14845 },
	{ 0xe602, 0xe602, PDF_CMAP_SINGLE, 14847 },
	{ 0xe604, 0xe604, PDF_CMAP_SINGLE, 14849 },
	{ 0xe60f, 0xe612, PDF_CMAP_RANGE, 14859 },
	{ 0xe619, 0xe61c, PDF_CMAP_RANGE, 14868 },
	{ 0xe621, 0xe621, PDF_CMAP_SINGLE, 14876 },
	{ 0xe623, 0xe624, PDF_CMAP_RANGE, 14878 },
	{ 0xe627, 0xe627, PDF_CMAP_SINGLE, 14882 },
	{ 0xe62b, 0xe62b, PDF_CMAP_SINGLE, 14886 },
	{ 0xe632, 0xe634, PDF_CMAP_RANGE, 14893 },
	{ 0xe637, 0xe638, PDF_CMAP_RANGE, 14898 },
	{ 0xe63a, 0xe63a, PDF_CMAP_SINGLE, 14901 },
	{ 0xe63c, 0xe63c, PDF_CMAP_SINGLE, 14903 },
	{ 0xe63f, 0xe642, PDF_CMAP_RANGE, 14906 },
	{ 0xe645, 0xe646, PDF_CMAP_RANGE, 14912 },
	{ 0xe648, 0xe648, PDF_CMAP_SINGLE, 14915 },
	{ 0xe64b, 0xe64b, PDF_CMAP_SINGLE, 14917 },
	{ 0xe64e, 0xe64f, PDF_CMAP_RANGE, 14920 },
	{ 0xe652, 0xe655, PDF_CMAP_RANGE, 14924 },
	{ 0xe657, 0xe659, PDF_CMAP_RANGE, 14929 },
	{ 0xe65d, 0xe660, PDF_CMAP_RANGE, 14935 },
	{ 0xe662, 0xe663, PDF_CMAP_RANGE, 14940 },
	{ 0xe665, 0xe668, PDF_CMAP_RANGE, 14943 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 18111 },
	{ 0xe66e, 0xe66f, PDF_CMAP_RANGE, 14951 },
	{ 0xe671, 0xe671, PDF_CMAP_SINGLE, 14954 },
	{ 0xe676, 0xe679, PDF_CMAP_RANGE, 14959 },
	{ 0xe67b, 0xe67d, PDF_CMAP_RANGE, 14964 },
	{ 0xe67f, 0xe680, PDF_CMAP_RANGE, 14968 },
	{ 0xe683, 0xe688, PDF_CMAP_RANGE, 14971 },
	{ 0xe68a, 0xe68c, PDF_CMAP_RANGE, 14978 },
	{ 0xe68e, 0xe690, PDF_CMAP_RANGE, 14982 },
	{ 0xe692, 0xe692, PDF_CMAP_SINGLE, 14986 },
	{ 0xe696, 0xe697, PDF_CMAP_RANGE, 14990 },
	{ 0xe699, 0xe69a, PDF_CMAP_RANGE, 14993 },
	{ 0xe69c, 0xe69c, PDF_CMAP_SINGLE, 14996 },
	{ 0xe69e, 0xe69f, PDF_CMAP_RANGE, 14998 },
	{ 0xe6a1, 0xe6a2, PDF_CMAP_RANGE, 15001 },
	{ 0xe6a6, 0xe6a6, PDF_CMAP_SINGLE, 15005 },
	{ 0xe6ac, 0xe6ad, PDF_CMAP_RANGE, 15010 },
	{ 0xe6b2, 0xe6b2, PDF_CMAP_SINGLE, 15016 },
	{ 0xe6b9, 0xe6ba, PDF_CMAP_RANGE, 15023 },
	{ 0xe6bd, 0xe6be, PDF_CMAP_RANGE, 15027 },
	{ 0xe6c0, 0xe6c1, PDF_CMAP_RANGE, 15030 },
	{ 0xe6c5, 0xe6c5, PDF_CMAP_SINGLE, 18116 },
	{ 0xe6c7, 0xe6c7, PDF_CMAP_SINGLE, 15036 },
	{ 0xe6ca, 0xe6ca, PDF_CMAP_SINGLE, 15039 },
	{ 0xe6d1, 0xe6d2, PDF_CMAP_RANGE, 15046 },
	{ 0xe6d7, 0xe6d8, PDF_CMAP_RANGE, 15052 },
	{ 0xe6da, 0xe6db, PDF_CMAP_RANGE, 15055 },
	{ 0xe6dd, 0xe6e1, PDF_CMAP_RANGE, 15058 },
	{ 0xe6e4, 0xe6e6, PDF_CMAP_RANGE, 15065 },
	{ 0xe6ee, 0xe6ee, PDF_CMAP_SINGLE, 15075 },
	{ 0xe6f2, 0xe6f3, PDF_CMAP_RANGE, 15079 },
	{ 0xe6f7, 0xe6f7, PDF_CMAP_SINGLE, 15084 },
	{ 0xe6fa, 0xe6fc, PDF_CMAP_RANGE, 15087 },
	{ 0xe6ff, 0xe6ff, PDF_CMAP_SINGLE, 15092 },
	{ 0xe701, 0xe702, PDF_CMAP_RANGE, 15094 },
	{ 0xe706, 0xe707, PDF_CMAP_RANGE, 15099 },
	{ 0xe709, 0xe70a, PDF_CMAP_RANGE, 15102 },
	{ 0xe70c, 0xe70c, PDF_CMAP_SINGLE, 15105 },
	{ 0xe710, 0xe711, PDF_CMAP_RANGE, 15109 },
	{ 0xe713, 0xe713, PDF_CMAP_SINGLE, 15112 },
	{ 0xe717, 0xe718, PDF_CMAP_RANGE, 15116 },
	{ 0xe71a, 0xe71b, PDF_CMAP_RANGE, 15119 },
	{ 0xe71d, 0xe71d, PDF_CMAP_SINGLE, 15122 },
	{ 0xe724, 0xe725, PDF_CMAP_RANGE, 15129 },
	{ 0xe72b, 0xe72b, PDF_CMAP_SINGLE, 15136 },
	{ 0xe72f, 0xe72f, PDF_CMAP_SINGLE, 15140 },
	{ 0xe733, 0xe737, PDF_CMAP_RANGE, 15144 },
	{ 0xe73c, 0xe73d, PDF_CMAP_RANGE, 15153 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 15157 },
	{ 0xe744, 0xe744, PDF_CMAP_SINGLE, 15161 },
	{ 0xe747, 0xe747, PDF_CMAP_SINGLE, 15164 },
	{ 0xe74d, 0xe74f, PDF_CMAP_RANGE, 15170 },
	{ 0xe751, 0xe752, PDF_CMAP_RANGE, 15174 },
	{ 0xe75b, 0xe75d, PDF_CMAP_RANGE, 15184 },
	{ 0xe75f, 0xe75f, PDF_CMAP_SINGLE, 15188 },
	{ 0xe761, 0xe762, PDF_CMAP_RANGE, 15190 },
	{ 0xe764, 0xe765, PDF_CMAP_RANGE, 15193 },
	{ 0xe767, 0xe768, PDF_CMAP_RANGE, 15196 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 18119 },
	{ 0xe76c, 0xe76e, PDF_CMAP_RANGE, 15200 },
	{ 0xe770, 0xe770, PDF_CMAP_SINGLE, 15204 },
	{ 0xe775, 0xe775, PDF_CMAP_SINGLE, 15209 },
	{ 0xe777, 0xe778, PDF_CMAP_RANGE, 15210 },
	{ 0xe77a, 0xe77a, PDF_CMAP_SINGLE, 15213 },
	{ 0xe77c, 0xe77d, PDF_CMAP_RANGE, 15215 },
	{ 0xe784, 0xe784, PDF_CMAP_SINGLE, 15223 },
	{ 0xe786, 0xe786, PDF_CMAP_SINGLE, 15225 },
	{ 0xe789, 0xe789, PDF_CMAP_SINGLE, 15228 },
	{ 0xe78b, 0xe78b, PDF_CMAP_SINGLE, 15230 },
	{ 0xe78d, 0xe78d, PDF_CMAP_SINGLE, 15232 },
	{ 0xe790, 0xe790, PDF_CMAP_SINGLE, 15235 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 15237 },
	{ 0xe794, 0xe794, PDF_CMAP_SINGLE, 15239 },
	{ 0xe796, 0xe797, PDF_CMAP_RANGE, 15241 },
	{ 0xe799, 0xe799, PDF_CMAP_SINGLE, 15244 },
	{ 0xe7a2, 0xe7a2, PDF_CMAP_SINGLE, 15253 },
	{ 0xe7a4, 0xe7a9, PDF_CMAP_RANGE, 15255 },
	{ 0xe7b1, 0xe7b1, PDF_CMAP_SINGLE, 15268 },
	{ 0xe7b4, 0xe7b4, PDF_CMAP_SINGLE, 15271 },
	{ 0xe7b8, 0xe7b8, PDF_CMAP_SINGLE, 15275 },
	{ 0xe7bc, 0xe7bc, PDF_CMAP_SINGLE, 15279 },
	{ 0xe7c4, 0xe7c4, PDF_CMAP_SINGLE, 15286 },
	{ 0xe7ca, 0xe7ca, PDF_CMAP_SINGLE, 15292 },
	{ 0xe7ce, 0xe7ce, PDF_CMAP_SINGLE, 15296 },
	{ 0xe7d1, 0xe7d1, PDF_CMAP_SINGLE, 15299 },
	{ 0xe7d8, 0xe7d8, PDF_CMAP_SINGLE, 15306 },
	{ 0xe7da, 0xe7da, PDF_CMAP_SINGLE, 15308 },
	{ 0xe7dc, 0xe7dd, PDF_CMAP_RANGE, 15310 },
	{ 0xe7e4, 0xe7e4, PDF_CMAP_SINGLE, 15318 },
	{ 0xe7e7, 0xe7e9, PDF_CMAP_RANGE, 15321 },
	{ 0xe7eb, 0xe7eb, PDF_CMAP_SINGLE, 15325 },
	{ 0xe7ed, 0xe7ef, PDF_CMAP_RANGE, 15327 },
	{ 0xe7f1, 0xe7f3, PDF_CMAP_RANGE, 15331 },
	{ 0xe7f8, 0xe7f8, PDF_CMAP_SINGLE, 15338 },
	{ 0xe7fb, 0xe7fc, PDF_CMAP_RANGE, 15341 },
	{ 0xe7ff, 0xe802, PDF_CMAP_RANGE, 15345 },
	{ 0xe808, 0xe808, PDF_CMAP_SINGLE, 15354 },
	{ 0xe80c, 0xe80c, PDF_CMAP_SINGLE, 15358 },
	{ 0xe80e, 0xe80f, PDF_CMAP_RANGE, 15360 },
	{ 0xe811, 0xe811, PDF_CMAP_SINGLE, 15363 },
	{ 0xe813, 0xe813, PDF_CMAP_SINGLE, 15365 },
	{ 0xe816, 0xe818, PDF_CMAP_RANGE, 15368 },
	{ 0xe81a, 0xe81a, PDF_CMAP_SINGLE, 15372 },
	{ 0xe821, 0xe821, PDF_CMAP_SINGLE, 15379 },
	{ 0xe827, 0xe828, PDF_CMAP_RANGE, 15385 },
	{ 0xe82d, 0xe82d, PDF_CMAP_SINGLE, 15391 },
	{ 0xe82f, 0xe82f, PDF_CMAP_SINGLE, 15393 },
	{ 0xe835, 0xe835, PDF_CMAP_SINGLE, 15399 },
	{ 0xe837, 0xe838, PDF_CMAP_RANGE, 15401 },
	{ 0xe83a, 0xe83b, PDF_CMAP_RANGE, 15404 },
	{ 0xe841, 0xe842, PDF_CMAP_RANGE, 15411 },
	{ 0xe84b, 0xe84b, PDF_CMAP_SINGLE, 15421 },
	{ 0xe84d, 0xe84e, PDF_CMAP_RANGE, 15423 },
	{ 0xe851, 0xe851, PDF_CMAP_SINGLE, 15427 },
	{ 0xe855, 0xe858, PDF_CMAP_RANGE, 15431 },
	{ 0xe85b, 0xe85d, PDF_CMAP_RANGE, 15437 },
	{ 0xe860, 0xe862, PDF_CMAP_RANGE, 15442 },
	{ 0xe865, 0xe867, PDF_CMAP_RANGE, 15447 },
	{ 0xe869, 0xe86a, PDF_CMAP_RANGE, 15451 },
	{ 0xe86c, 0xe86c, PDF_CMAP_SINGLE, 15453 },
	{ 0xe871, 0xe871, PDF_CMAP_SINGLE, 15458 },
	{ 0xe878, 0xe878, PDF_CMAP_SINGLE, 15465 },
	{ 0xe87d, 0xe87f, PDF_CMAP_RANGE, 15470 },
	{ 0xe887, 0xe887, PDF_CMAP_SINGLE, 15480 },
	{ 0xe88f, 0xe88f, PDF_CMAP_SINGLE, 15488 },
	{ 0xe891, 0xe892, PDF_CMAP_RANGE, 15490 },
	{ 0xe894, 0xe895, PDF_CMAP_RANGE, 15493 },
	{ 0xe89a, 0xe89a, PDF_CMAP_SINGLE, 15499 },
	{ 0xe89d, 0xe89d, PDF_CMAP_SINGLE, 15502 },
	{ 0xe8a0, 0xe8a1, PDF_CMAP_RANGE, 15505 },
	{ 0xe8a3, 0xe8a7, PDF_CMAP_RANGE, 15508 },
	{ 0xe8a9, 0xe8aa, PDF_CMAP_RANGE, 15514 },
	{ 0xe8ac, 0xe8b0, PDF_CMAP_RANGE, 15517 },
	{ 0xe8b5, 0xe8b6, PDF_CMAP_RANGE, 15526 },
	{ 0xe8b8, 0xe8b8, PDF_CMAP_SINGLE, 15529 },
	{ 0xe8bd, 0xe8c3, PDF_CMAP_RANGE, 15534 },
	{ 0xe8c5, 0xe8c5, PDF_CMAP_SINGLE, 15542 },
	{ 0xe8c7, 0xe8c7, PDF_CMAP_SINGLE, 15544 },
	{ 0xe8cb, 0xe8cb, PDF_CMAP_SINGLE, 15548 },
	{ 0xe8cd, 0xe8ce, PDF_CMAP_RANGE, 15550 },
	{ 0xe8d0, 0xe8d1, PDF_CMAP_RANGE, 15553 },
	{ 0xe8d3, 0xe8d3, PDF_CMAP_SINGLE, 15556 },
	{ 0xe8d5, 0xe8d6, PDF_CMAP_RANGE, 15558 },
	{ 0xe8d8, 0xe8da, PDF_CMAP_RANGE, 15561 },
	{ 0xe8dd, 0xe8dd, PDF_CMAP_SINGLE, 15566 },
	{ 0xe8df, 0xe8e1, PDF_CMAP_RANGE, 15568 },
	{ 0xe8e5, 0xe8e6, PDF_CMAP_RANGE, 15574 },
	{ 0xe8e8, 0xe8ea, PDF_CMAP_RANGE, 15577 },
	{ 0xe8ed, 0xe8ed, PDF_CMAP_SINGLE, 15582 },
	{ 0xe8f1, 0xe8f2, PDF_CMAP_RANGE, 15586 },
	{ 0xe8f5, 0xe8f5, PDF_CMAP_SINGLE, 15590 },
	{ 0xe8fb, 0xe8fb, PDF_CMAP_SINGLE, 15596 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 15599 },
	{ 0xe900, 0xe900, PDF_CMAP_SINGLE, 15601 },
	{ 0xe903, 0xe903, PDF_CMAP_SINGLE, 15604 },
	{ 0xe907, 0xe908, PDF_CMAP_RANGE, 15608 },
	{ 0xe90c, 0xe90c, PDF_CMAP_SINGLE, 15613 },
	{ 0xe917, 0xe918, PDF_CMAP_RANGE, 15624 },
	{ 0xe91c, 0xe91c, PDF_CMAP_SINGLE, 15629 },
	{ 0xe91e, 0xe922, PDF_CMAP_RANGE, 15631 },
	{ 0xe925, 0xe92c, PDF_CMAP_RANGE, 15638 },
	{ 0xe92e, 0xe92f, PDF_CMAP_RANGE, 15647 },
	{ 0xe933, 0xe936, PDF_CMAP_RANGE, 15652 },
	{ 0xe938, 0xe938, PDF_CMAP_SINGLE, 18127 },
	{ 0xe93a, 0xe93a, PDF_CMAP_SINGLE, 15659 },
	{ 0xe93c, 0xe93e, PDF_CMAP_RANGE, 15661 },
	{ 0xe941, 0xe941, PDF_CMAP_SINGLE, 15666 },
	{ 0xe944, 0xe945, PDF_CMAP_RANGE, 15669 },
	{ 0xe948, 0xe948, PDF_CMAP_SINGLE, 15673 },
	{ 0xe94d, 0xe94d, PDF_CMAP_SINGLE, 15678 },
	{ 0xe950, 0xe950, PDF_CMAP_SINGLE, 15681 },
	{ 0xe952, 0xe955, PDF_CMAP_RANGE, 15683 },
	{ 0xe957, 0xe958, PDF_CMAP_RANGE, 15688 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 15691 },
	{ 0xe95c, 0xe95e, PDF_CMAP_RANGE, 15693 },
	{ 0xe960, 0xe961, PDF_CMAP_RANGE, 15697 },
	{ 0xe964, 0xe964, PDF_CMAP_SINGLE, 15701 },
	{ 0xe967, 0xe968, PDF_CMAP_TABLE, 13380 },
	{ 0xe96e, 0xe96f, PDF_CMAP_RANGE, 18133 },
	{ 0xe973, 0xe975, PDF_CMAP_TABLE, 13382 },
	{ 0xe977, 0xe978, PDF_CMAP_RANGE, 18141 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 18144 },
	{ 0xe97d, 0xe97d, PDF_CMAP_SINGLE, 18147 },
	{ 0xe981, 0xe982, PDF_CMAP_RANGE, 18150 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 15709 },
	{ 0xe98a, 0xe98a, PDF_CMAP_SINGLE, 15710 },
	{ 0xe98e, 0xe98e, PDF_CMAP_SINGLE, 15711 },
	{ 0xe991, 0xe991, PDF_CMAP_SINGLE, 18159 },
	{ 0xe9b0, 0xe9b0, PDF_CMAP_SINGLE, 17364 },
	{ 0xe9cb, 0xe9cb, PDF_CMAP_SINGLE, 18201 },
	{ 0xe9ef, 0xe9ef, PDF_CMAP_SINGLE, 18228 },
	{ 0xea43, 0xea43, PDF_CMAP_SINGLE, 18297 },
	{ 0xeaa9, 0xeaa9, PDF_CMAP_SINGLE, 18380 },
	{ 0xeabf, 0xeac1, PDF_CMAP_RANGE, 18396 },
	{ 0xeac5, 0xeac5, PDF_CMAP_SINGLE, 18400 },
	{ 0xeac9, 0xeacc, PDF_CMAP_RANGE, 18402 },
	{ 0xeacf, 0xead1, PDF_CMAP_RANGE, 18408 },
	{ 0xead3, 0xead4, PDF_CMAP_RANGE, 18411 },
	{ 0xead6, 0xeada, PDF_CMAP_RANGE, 18414 },
	{ 0xeadc, 0xeadc, PDF_CMAP_SINGLE, 18419 },
	{ 0xeade, 0xeae3, PDF_CMAP_RANGE, 18420 },
	{ 0xeae7, 0xeaec, PDF_CMAP_RANGE, 18426 },
	{ 0xeaee, 0xeaee, PDF_CMAP_SINGLE, 18432 },
	{ 0xeaf0, 0xeaf2, PDF_CMAP_RANGE, 18433 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 17328 },
	{ 0xeaf5, 0xeaf6, PDF_CMAP_RANGE, 18436 },
	{ 0xeaf8, 0xeaf9, PDF_CMAP_TABLE, 13385 },
	{ 0xeafa, 0xeafb, PDF_CMAP_RANGE, 18440 },
	{ 0xeafe, 0xeb00, PDF_CMAP_RANGE, 18443 },
	{ 0xeb02, 0xeb05, PDF_CMAP_RANGE, 18446 },
	{ 0xeb08, 0xeb0f, PDF_CMAP_RANGE, 18451 },
	{ 0xeb11, 0xeb12, PDF_CMAP_RANGE, 18459 },
	{ 0xeb15, 0xeb15, PDF_CMAP_SINGLE, 18462 },
	{ 0xeb18, 0xeb1d, PDF_CMAP_RANGE, 18465 },
	{ 0xeb1f, 0xeb21, PDF_CMAP_RANGE, 18471 },
	{ 0xeb23, 0xeb23, PDF_CMAP_SINGLE, 18474 },
	{ 0xeb25, 0xeb25, PDF_CMAP_SINGLE, 18475 },
	{ 0xeb27, 0xeb27, PDF_CMAP_SINGLE, 18476 },
	{ 0xeb2a, 0xeb2a, PDF_CMAP_SINGLE, 18477 },
	{ 0xeb2c, 0xeb39, PDF_CMAP_RANGE, 18478 },
	{ 0xeb3d, 0xeb3d, PDF_CMAP_SINGLE, 15771 },
	{ 0xeb3e, 0xeb3f, PDF_CMAP_RANGE, 18492 },
	{ 0xeb41, 0xeb41, PDF_CMAP_SINGLE, 15773 },
	{ 0xeb43, 0xeb44, PDF_CMAP_RANGE, 18494 },
	{ 0xeb46, 0xeb46, PDF_CMAP_SINGLE, 15776 },
	{ 0xeb4a, 0xeb4a, PDF_CMAP_SINGLE, 18497 },
	{ 0xeb4b, 0xeb4c, PDF_CMAP_RANGE, 15779 },
	{ 0xeb4d, 0xeb4e, PDF_CMAP_RANGE, 18498 },
	{ 0xeb4f, 0xeb50, PDF_CMAP_RANGE, 15781 },
	{ 0xeb53, 0xeb54, PDF_CMAP_TABLE, 13387 },
	{ 0xeb55, 0xeb56, PDF_CMAP_RANGE, 18501 },
	{ 0xeb57, 0xeb57, PDF_CMAP_SINGLE, 15786 },
	{ 0xeb58, 0xeb59, PDF_CMAP_RANGE, 18503 },
	{ 0xeb5a, 0xeb5b, PDF_CMAP_RANGE, 15787 },
	{ 0xeb5d, 0xeb5d, PDF_CMAP_SINGLE, 15790 },
	{ 0xeb5f, 0xeb5f, PDF_CMAP_SINGLE, 15792 },
	{ 0xeb61, 0xeb61, PDF_CMAP_SINGLE, 15794 },
	{ 0xeb62, 0xeb63, PDF_CMAP_RANGE, 18505 },
	{ 0xeb65, 0xeb65, PDF_CMAP_SINGLE, 18508 },
	{ 0xeb67, 0xeb67, PDF_CMAP_SINGLE, 18509 },
	{ 0xeb68, 0xeb69, PDF_CMAP_RANGE, 15796 },
	{ 0xeb6b, 0xeb6b, PDF_CMAP_SINGLE, 15799 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 15801 },
	{ 0xeb6f, 0xeb71, PDF_CMAP_RANGE, 18510 },
	{ 0xeb73, 0xeb74, PDF_CMAP_TABLE, 13389 },
	{ 0xeb76, 0xeb78, PDF_CMAP_TABLE, 13391 },
	{ 0xeb7a, 0xeb7a, PDF_CMAP_SINGLE, 18516 },
	{ 0xeb7e, 0xeb7f, PDF_CMAP_RANGE, 18518 },
	{ 0xeb80, 0xeb81, PDF_CMAP_RANGE, 15809 },
	{ 0xeb83, 0xeb84, PDF_CMAP_RANGE, 18520 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 15812 },
	{ 0xeb88, 0xeb8a, PDF_CMAP_TABLE, 13394 },
	{ 0xeb8c, 0xeb8e, PDF_CMAP_RANGE, 18524 },
	{ 0xeb8f, 0xeb90, PDF_CMAP_TABLE, 13397 },
	{ 0xeb93, 0xeb93, PDF_CMAP_SINGLE, 15819 },
	{ 0xeb95, 0xeb95, PDF_CMAP_SINGLE, 18528 },
	{ 0xeb97, 0xeb97, PDF_CMAP_SINGLE, 15822 },
	{ 0xeb98, 0xeb9a, PDF_CMAP_RANGE, 18529 },
	{ 0xeb9b, 0xeb9b, PDF_CMAP_SINGLE, 15823 },
	{ 0xeb9f, 0xeba0, PDF_CMAP_TABLE, 13399 },
	{ 0xeba2, 0xeba3, PDF_CMAP_TABLE, 13401 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 18536 },
	{ 0xeba8, 0xeba8, PDF_CMAP_SINGLE, 15830 },
	{ 0xebaa, 0xebab, PDF_CMAP_TABLE, 13403 },
	{ 0xebad, 0xebb0, PDF_CMAP_TABLE, 13405 },
	{ 0xebb2, 0xebb3, PDF_CMAP_RANGE, 18541 },
	{ 0xebb4, 0xebb4, PDF_CMAP_SINGLE, 15837 },
	{ 0xebb6, 0xebb6, PDF_CMAP_SINGLE, 15839 },
	{ 0xebb9, 0xebb9, PDF_CMAP_SINGLE, 15842 },
	{ 0xebbc, 0xebbd, PDF_CMAP_TABLE, 13409 },
	{ 0xebbf, 0xebbf, PDF_CMAP_SINGLE, 15846 },
	{ 0xebc2, 0xebc2, PDF_CMAP_SINGLE, 15849 },
	{ 0xebc5, 0xebc6, PDF_CMAP_RANGE, 18546 },
	{ 0xebc7, 0xebc8, PDF_CMAP_TABLE, 13411 },
	{ 0xebca, 0xebca, PDF_CMAP_SINGLE, 18549 },
	{ 0xebcc, 0xebcd, PDF_CMAP_RANGE, 15854 },
	{ 0xebce, 0xebce, PDF_CMAP_SINGLE, 18550 },
	{ 0xebd0, 0xebd0, PDF_CMAP_SINGLE, 18551 },
	{ 0xebd3, 0xebd3, PDF_CMAP_SINGLE, 18552 },
	{ 0xebd5, 0xebd5, PDF_CMAP_SINGLE, 18553 },
	{ 0xebd7, 0xebd8, PDF_CMAP_RANGE, 15860 },
	{ 0xebda, 0xebdb, PDF_CMAP_RANGE, 18554 },
	{ 0xebdd, 0xebdd, PDF_CMAP_SINGLE, 15864 },
	{ 0xebdf, 0xebdf, PDF_CMAP_SINGLE, 18556 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 18557 },
	{ 0xebe4, 0xebe4, PDF_CMAP_SINGLE, 18558 },
	{ 0xebe7, 0xebe7, PDF_CMAP_SINGLE, 15871 },
	{ 0xebe8, 0xebe9, PDF_CMAP_RANGE, 18559 },
	{ 0xebeb, 0xebec, PDF_CMAP_RANGE, 18561 },
	{ 0xebee, 0xebef, PDF_CMAP_TABLE, 13413 },
	{ 0xebf1, 0xebf2, PDF_CMAP_TABLE, 13415 },
	{ 0xebf5, 0xebf6, PDF_CMAP_TABLE, 13417 },
	{ 0xebf9, 0xebfb, PDF_CMAP_RANGE, 15881 },
	{ 0xebfd, 0xebfe, PDF_CMAP_RANGE, 15885 },
	{ 0xec05, 0xec06, PDF_CMAP_RANGE, 15892 },
	{ 0xec08, 0xec09, PDF_CMAP_RANGE, 18568 },
	{ 0xec0c, 0xec0c, PDF_CMAP_SINGLE, 15896 },
	{ 0xec0e, 0xec10, PDF_CMAP_RANGE, 15898 },
	{ 0xec13, 0xec14, PDF_CMAP_TABLE, 13419 },
	{ 0xec17, 0xec18, PDF_CMAP_RANGE, 15906 },
	{ 0xec19, 0xec1c, PDF_CMAP_RANGE, 18572 },
	{ 0xec1d, 0xec1d, PDF_CMAP_SINGLE, 15908 },
	{ 0xec20, 0xec20, PDF_CMAP_SINGLE, 18576 },
	{ 0xec22, 0xec22, PDF_CMAP_SINGLE, 15912 },
	{ 0xec24, 0xec24, PDF_CMAP_SINGLE, 15914 },
	{ 0xec26, 0xec26, PDF_CMAP_SINGLE, 15916 },
	{ 0xec2a, 0xec2b, PDF_CMAP_TABLE, 13421 },
	{ 0xec2c, 0xec2e, PDF_CMAP_RANGE, 15921 },
	{ 0xec2f, 0xec2f, PDF_CMAP_SINGLE, 18578 },
	{ 0xec31, 0xec32, PDF_CMAP_TABLE, 13423 },
	{ 0xec35, 0xec35, PDF_CMAP_SINGLE, 15928 },
	{ 0xec37, 0xec37, PDF_CMAP_SINGLE, 15930 },
	{ 0xec3b, 0xec3c, PDF_CMAP_RANGE, 15934 },
	{ 0xec3f, 0xec41, PDF_CMAP_RANGE, 18580 },
	{ 0xec42, 0xec44, PDF_CMAP_TABLE, 13425 },
	{ 0xec45, 0xec49, PDF_CMAP_RANGE, 18584 },
	{ 0xec4b, 0xec4c, PDF_CMAP_RANGE, 18589 },
	{ 0xec4f, 0xec4f, PDF_CMAP_SINGLE, 18592 },
	{ 0xec52, 0xec52, PDF_CMAP_SINGLE, 18593 },
	{ 0xec54, 0xec54, PDF_CMAP_SINGLE, 18594 },
	{ 0xec56, 0xec58, PDF_CMAP_RANGE, 18595 },
	{ 0xec5a, 0xec5a, PDF_CMAP_SINGLE, 18598 },
	{ 0xec5c, 0xec5d, PDF_CMAP_RANGE, 18599 },
	{ 0xec5f, 0xec60, PDF_CMAP_RANGE, 18601 },
	{ 0xec62, 0xec63, PDF_CMAP_RANGE, 18604 },
	{ 0xec67, 0xec68, PDF_CMAP_TABLE, 13428 },
	{ 0xec69, 0xec73, PDF_CMAP_RANGE, 18608 },
	{ 0xec75, 0xec76, PDF_CMAP_RANGE, 18620 },
	{ 0xec79, 0xec79, PDF_CMAP_SINGLE, 18624 },
	{ 0xec7b, 0xec7b, PDF_CMAP_SINGLE, 18626 },
	{ 0xec7e, 0xec7e, PDF_CMAP_SINGLE, 18627 },
	{ 0xec80, 0xec80, PDF_CMAP_SINGLE, 18629 },
	{ 0xec84, 0xec84, PDF_CMAP_SINGLE, 18632 },
	{ 0xec86, 0xec86, PDF_CMAP_SINGLE, 18634 },
	{ 0xec8d, 0xec8e, PDF_CMAP_TABLE, 13430 },
	{ 0xec91, 0xec92, PDF_CMAP_TABLE, 13432 },
	{ 0xec93, 0xec94, PDF_CMAP_RANGE, 18639 },
	{ 0xec95, 0xec96, PDF_CMAP_TABLE, 13434 },
	{ 0xec98, 0xec99, PDF_CMAP_TABLE, 13436 },
	{ 0xec9a, 0xec9b, PDF_CMAP_RANGE, 18643 },
	{ 0xec9d, 0xec9e, PDF_CMAP_TABLE, 13438 },
	{ 0xeca0, 0xeca0, PDF_CMAP_SINGLE, 18647 },
	{ 0xeca3, 0xeca4, PDF_CMAP_RANGE, 18648 },
	{ 0xeca7, 0xeca8, PDF_CMAP_RANGE, 18650 },
	{ 0xecab, 0xecab, PDF_CMAP_SINGLE, 18652 },
	{ 0xecad, 0xecad, PDF_CMAP_SINGLE, 18653 },
	{ 0xecaf, 0xecaf, PDF_CMAP_SINGLE, 15973 },
	{ 0xecb4, 0xecb4, PDF_CMAP_SINGLE, 18655 },
	{ 0xecb6, 0xecb6, PDF_CMAP_SINGLE, 15979 },
	{ 0xecb9, 0xecbb, PDF_CMAP_RANGE, 18657 },
	{ 0xeccf, 0xeccf, PDF_CMAP_SINGLE, 16000 },
	{ 0xecd4, 0xecd4, PDF_CMAP_SINGLE, 16005 },
	{ 0xecdb, 0xecdc, PDF_CMAP_RANGE, 16012 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 18660 },
	{ 0xecdf, 0xece1, PDF_CMAP_RANGE, 16015 },
	{ 0xece5, 0xece5, PDF_CMAP_SINGLE, 16021 },
	{ 0xecf0, 0xecf0, PDF_CMAP_SINGLE, 16032 },
	{ 0xecf2, 0xecf2, PDF_CMAP_SINGLE, 16034 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 16037 },
	{ 0xecfc, 0xecfc, PDF_CMAP_SINGLE, 16044 },
	{ 0xed04, 0xed04, PDF_CMAP_SINGLE, 18663 },
	{ 0xed07, 0xed07, PDF_CMAP_SINGLE, 18664 },
	{ 0xed0a, 0xed0a, PDF_CMAP_SINGLE, 18666 },
	{ 0xed0c, 0xed11, PDF_CMAP_RANGE, 18667 },
	{ 0xed13, 0xed13, PDF_CMAP_SINGLE, 18673 },
	{ 0xed15, 0xed17, PDF_CMAP_RANGE, 18674 },
	{ 0xed18, 0xed18, PDF_CMAP_SINGLE, 16058 },
	{ 0xed1a, 0xed1a, PDF_CMAP_SINGLE, 18677 },
	{ 0xed1c, 0xed1e, PDF_CMAP_TABLE, 13440 },
	{ 0xed22, 0xed22, PDF_CMAP_SINGLE, 18680 },
	{ 0xed24, 0xed25, PDF_CMAP_RANGE, 16066 },
	{ 0xed26, 0xed27, PDF_CMAP_RANGE, 18681 },
	{ 0xed29, 0xed2a, PDF_CMAP_TABLE, 13443 },
	{ 0xed2c, 0xed2d, PDF_CMAP_TABLE, 13445 },
	{ 0xed3b, 0xed3b, PDF_CMAP_SINGLE, 16079 },
	{ 0xed3d, 0xed3d, PDF_CMAP_SINGLE, 18692 },
	{ 0xed41, 0xed42, PDF_CMAP_RANGE, 18694 },
	{ 0xed45, 0xed45, PDF_CMAP_SINGLE, 18696 },
	{ 0xed47, 0xed47, PDF_CMAP_SINGLE, 16085 },
	{ 0xed4a, 0xed4b, PDF_CMAP_RANGE, 18697 },
	{ 0xed4d, 0xed4e, PDF_CMAP_RANGE, 18699 },
	{ 0xed4f, 0xed4f, PDF_CMAP_SINGLE, 16090 },
	{ 0xed51, 0xed52, PDF_CMAP_TABLE, 13447 },
	{ 0xed53, 0xed54, PDF_CMAP_RANGE, 18702 },
	{ 0xed57, 0xed58, PDF_CMAP_RANGE, 18704 },
	{ 0xed5f, 0xed5f, PDF_CMAP_SINGLE, 16101 },
	{ 0xed62, 0xed62, PDF_CMAP_SINGLE, 16104 },
	{ 0xed6b, 0xed6b, PDF_CMAP_SINGLE, 16113 },
	{ 0xed71, 0xed72, PDF_CMAP_TABLE, 13449 },
	{ 0xed75, 0xed75, PDF_CMAP_SINGLE, 18707 },
	{ 0xed77, 0xed78, PDF_CMAP_TABLE, 13451 },
	{ 0xed7a, 0xed7a, PDF_CMAP_SINGLE, 18710 },
	{ 0xed81, 0xed81, PDF_CMAP_SINGLE, 18711 },
	{ 0xed84, 0xed85, PDF_CMAP_RANGE, 16132 },
	{ 0xed86, 0xed86, PDF_CMAP_SINGLE, 18712 },
	{ 0xed88, 0xed88, PDF_CMAP_SINGLE, 16135 },
	{ 0xed89, 0xed8a, PDF_CMAP_RANGE, 18713 },
	{ 0xed8b, 0xed8b, PDF_CMAP_SINGLE, 16136 },
	{ 0xed8e, 0xed8e, PDF_CMAP_SINGLE, 16139 },
	{ 0xed91, 0xed91, PDF_CMAP_SINGLE, 16142 },
	{ 0xeda6, 0xeda6, PDF_CMAP_SINGLE, 18716 },
	{ 0xedc6, 0xedc6, PDF_CMAP_SINGLE, 16193 },
	{ 0xedcf, 0xedcf, PDF_CMAP_SINGLE, 16202 },
	{ 0xedd1, 0xedd1, PDF_CMAP_SINGLE, 18718 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 18719 },
	{ 0xedda, 0xeddb, PDF_CMAP_TABLE, 13453 },
	{ 0xeddd, 0xeddd, PDF_CMAP_SINGLE, 18721 },
	{ 0xede1, 0xede1, PDF_CMAP_SINGLE, 16216 },
	{ 0xeded, 0xeded, PDF_CMAP_SINGLE, 16228 },
	{ 0xee08, 0xee08, PDF_CMAP_SINGLE, 16254 },
	{ 0xee11, 0xee11, PDF_CMAP_SINGLE, 16263 },
	{ 0xee15, 0xee15, PDF_CMAP_SINGLE, 18725 },
	{ 0xee1b, 0xee1b, PDF_CMAP_SINGLE, 18726 },
	{ 0xee20, 0xee20, PDF_CMAP_SINGLE, 16276 },
	{ 0xee22, 0xee22, PDF_CMAP_SINGLE, 18727 },
	{ 0xee2a, 0xee2a, PDF_CMAP_SINGLE, 16285 },
	{ 0xee30, 0xee31, PDF_CMAP_TABLE, 13455 },
	{ 0xee34, 0xee34, PDF_CMAP_SINGLE, 16294 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 18729 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 18730 },
	{ 0xee5b, 0xee5b, PDF_CMAP_SINGLE, 18731 },
	{ 0xee5d, 0xee5d, PDF_CMAP_SINGLE, 16332 },
	{ 0xee60, 0xee60, PDF_CMAP_SINGLE, 18732 },
	{ 0xee64, 0xee64, PDF_CMAP_SINGLE, 16338 },
	{ 0xee6a, 0xee6a, PDF_CMAP_SINGLE, 16344 },
	{ 0xee76, 0xee76, PDF_CMAP_SINGLE, 16356 },
	{ 0xee7c, 0xee7c, PDF_CMAP_SINGLE, 16362 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 18733 },
	{ 0xee91, 0xee91, PDF_CMAP_SINGLE, 16381 },
	{ 0xee9c, 0xee9c, PDF_CMAP_SINGLE, 18735 },
	{ 0xeea1, 0xeea6, PDF_CMAP_RANGE, 18737 },
	{ 0xeea7, 0xeea7, PDF_CMAP_SINGLE, 17332 },
	{ 0xeea8, 0xeeaa, PDF_CMAP_RANGE, 18743 },
	{ 0xeeac, 0xeeb1, PDF_CMAP_RANGE, 18746 },
	{ 0xeeb4, 0xeeb4, PDF_CMAP_SINGLE, 18754 },
	{ 0xeeb7, 0xeeb7, PDF_CMAP_SINGLE, 18757 },
	{ 0xf303, 0xf318, PDF_CMAP_RANGE, 17609 },
	{ 0xf325, 0xf325, PDF_CMAP_SINGLE, 18797 },
	{ 0xf327, 0xf327, PDF_CMAP_SINGLE, 18799 },
	{ 0xf344, 0xf344, PDF_CMAP_SINGLE, 18828 },
	{ 0xf346, 0xf346, PDF_CMAP_SINGLE, 18830 },
	{ 0xf34a, 0xf34b, PDF_CMAP_RANGE, 18844 },
	{ 0xf3a0, 0xf3a1, PDF_CMAP_RANGE, 17716 },
	{ 0xf3ac, 0xf3ac, PDF_CMAP_SINGLE, 17723 },
	{ 0xf3f0, 0xf3f0, PDF_CMAP_SINGLE, 17781 },
	{ 0xf3f9, 0xf3fa, PDF_CMAP_RANGE, 17788 },
	{ 0xf3fc, 0xf3fc, PDF_CMAP_SINGLE, 17791 },
	{ 0xf408, 0xf40b, PDF_CMAP_RANGE, 17801 },
	{ 0xf43d, 0xf43d, PDF_CMAP_SINGLE, 17845 },
	{ 0xf440, 0xf442, PDF_CMAP_RANGE, 17163 },
	{ 0xf444, 0xf444, PDF_CMAP_SINGLE, 17167 },
	{ 0xf446, 0xf446, PDF_CMAP_SINGLE, 17169 },
	{ 0xf448, 0xf449, PDF_CMAP_RANGE, 17171 },
	{ 0xf44b, 0xf44b, PDF_CMAP_SINGLE, 17174 },
	{ 0xf44d, 0xf44d, PDF_CMAP_SINGLE, 17176 },
	{ 0xf450, 0xf452, PDF_CMAP_RANGE, 17179 },
	{ 0xf454, 0xf456, PDF_CMAP_RANGE, 17183 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 17847 },
	{ 0xf45a, 0xf45b, PDF_CMAP_TABLE, 13457 },
	{ 0xf45c, 0xf45f, PDF_CMAP_RANGE, 17191 },
	{ 0xf461, 0xf463, PDF_CMAP_RANGE, 17196 },
	{ 0xf465, 0xf466, PDF_CMAP_RANGE, 17200 },
	{ 0xf469, 0xf469, PDF_CMAP_SINGLE, 17204 },
	{ 0xf46c, 0xf46d, PDF_CMAP_RANGE, 17207 },
	{ 0xf46e, 0xf46f, PDF_CMAP_TABLE, 13459 },
	{ 0xf474, 0xf474, PDF_CMAP_SINGLE, 17850 },
	{ 0xf477, 0xf479, PDF_CMAP_TABLE, 13461 },
	{ 0xf47b, 0xf47b, PDF_CMAP_SINGLE, 17853 },
	{ 0xf47c, 0xf47e, PDF_CMAP_RANGE, 17223 },
	{ 0xf480, 0xf481, PDF_CMAP_RANGE, 17227 },
	{ 0xf483, 0xf483, PDF_CMAP_SINGLE, 17854 },
	{ 0xf484, 0xf485, PDF_CMAP_RANGE, 17231 },
	{ 0xf488, 0xf488, PDF_CMAP_SINGLE, 17235 },
	{ 0xf48b, 0xf48b, PDF_CMAP_SINGLE, 17238 },
	{ 0xf48d, 0xf490, PDF_CMAP_RANGE, 17240 },
	{ 0xf491, 0xf491, PDF_CMAP_SINGLE, 17855 },
	{ 0xf497, 0xf498, PDF_CMAP_RANGE, 17250 },
	{ 0xf49b, 0xf49b, PDF_CMAP_SINGLE, 17254 },
	{ 0xf49e, 0xf49e, PDF_CMAP_SINGLE, 17257 },
	{ 0xf4a4, 0xf4a6, PDF_CMAP_RANGE, 17263 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 17857 },
	{ 0xf4a9, 0xf4aa, PDF_CMAP_RANGE, 17268 },
	{ 0xf4ad, 0xf4ad, PDF_CMAP_SINGLE, 17272 },
	{ 0xf4af, 0xf4b0, PDF_CMAP_RANGE, 17274 },
	{ 0xf4b2, 0xf4b2, PDF_CMAP_SINGLE, 17859 },
	{ 0xf4b3, 0xf4b4, PDF_CMAP_RANGE, 17278 },
	{ 0xf4b7, 0xf4b7, PDF_CMAP_SINGLE, 17282 },
	{ 0xf4ba, 0xf4c0, PDF_CMAP_RANGE, 17285 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 17860 },
	{ 0xf4c3, 0xf4c5, PDF_CMAP_RANGE, 17861 },
	{ 0xf4c7, 0xf4ca, PDF_CMAP_RANGE, 17865 },
	{ 0xf4cc, 0xf4cf, PDF_CMAP_RANGE, 17870 },
	{ 0xf4d1, 0xf4d5, PDF_CMAP_RANGE, 17874 },
	{ 0xf4d7, 0xf4d7, PDF_CMAP_SINGLE, 17880 },
	{ 0xf4d9, 0xf4da, PDF_CMAP_RANGE, 17882 },
	{ 0xf4db, 0xf4dc, PDF_CMAP_RANGE, 17292 },
	{ 0xf4dd, 0xf4de, PDF_CMAP_RANGE, 17884 },
	{ 0xf4df, 0xf4e0, PDF_CMAP_TABLE, 13464 },
	{ 0xf4e2, 0xf4e6, PDF_CMAP_TABLE, 13466 },
	{ 0xf4e7, 0xf4e9, PDF_CMAP_RANGE, 17298 },
	{ 0xf4ef, 0xf4ef, PDF_CMAP_SINGLE, 17893 },
	{ 0xf4f2, 0xf4f3, PDF_CMAP_TABLE, 13471 },
	{ 0xf4f8, 0xf4f8, PDF_CMAP_SINGLE, 17899 },
	{ 0xf4fc, 0xf4fc, PDF_CMAP_SINGLE, 17902 },
	{ 0xf4fe, 0xf501, PDF_CMAP_RANGE, 17904 },
	{ 0xf503, 0xf503, PDF_CMAP_SINGLE, 17908 },
	{ 0xf505, 0xf505, PDF_CMAP_SINGLE, 17910 },
	{ 0xf507, 0xf511, PDF_CMAP_RANGE, 17912 },
	{ 0xf513, 0xf516, PDF_CMAP_RANGE, 17924 },
	{ 0xf518, 0xf51d, PDF_CMAP_RANGE, 17929 },
	{ 0xf51f, 0xf526, PDF_CMAP_RANGE, 17936 },
	{ 0xf52a, 0xf52a, PDF_CMAP_SINGLE, 17947 },
	{ 0xf533, 0xf534, PDF_CMAP_RANGE, 17956 },
	{ 0xf53a, 0xf53b, PDF_CMAP_RANGE, 17633 },
	{ 0xf53d, 0xf53d, PDF_CMAP_SINGLE, 17636 },
	{ 0xf54e, 0xf54e, PDF_CMAP_SINGLE, 17653 },
	{ 0xf553, 0xf553, PDF_CMAP_SINGLE, 17658 },
	{ 0xf556, 0xf556, PDF_CMAP_SINGLE, 17660 },
	{ 0xf559, 0xf55a, PDF_CMAP_RANGE, 17663 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 17668 },
	{ 0xf565, 0xf565, PDF_CMAP_SINGLE, 17675 },
	{ 0xf56b, 0xf56b, PDF_CMAP_SINGLE, 17681 },
	{ 0xf572, 0xf572, PDF_CMAP_SINGLE, 17688 },
	{ 0xf694, 0xf694, PDF_CMAP_SINGLE, 18044 },
	{ 0xf69e, 0xf69e, PDF_CMAP_SINGLE, 18054 },
	{ 0xf6ad, 0xf6ad, PDF_CMAP_SINGLE, 18068 },
	{ 0xf6b0, 0xf6b0, PDF_CMAP_SINGLE, 18071 },
	{ 0xf7e5, 0xf7e6, PDF_CMAP_RANGE, 13999 },
	{ 0xf7e8, 0xf7e8, PDF_CMAP_SINGLE, 14002 },
	{ 0xf7ea, 0xf7eb, PDF_CMAP_RANGE, 14004 },
	{ 0xf7ed, 0xf7ee, PDF_CMAP_RANGE, 14007 },
	{ 0xfa0c, 0xfa0d, PDF_CMAP_TABLE, 13473 },
	{ 0xfe30, 0xfe31, PDF_CMAP_TABLE, 13475 },
	{ 0xfe33, 0xfe34, PDF_CMAP_TABLE, 13477 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 130 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 134 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 138 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 142 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 146 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 150 },
	{ 0xfe41, 0xfe42, PDF_CMAP_RANGE, 154 },
	{ 0xfe43, 0xfe44, PDF_CMAP_RANGE, 158 },
	{ 0xfe49, 0xfe4a, PDF_CMAP_RANGE, 199 },
	{ 0xfe4b, 0xfe4c, PDF_CMAP_RANGE, 203 },
	{ 0xfe4d, 0xfe4e, PDF_CMAP_RANGE, 201 },
	{ 0xfe4f, 0xfe50, PDF_CMAP_TABLE, 13479 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 114 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 116 },
	{ 0xfe59, 0xfe5e, PDF_CMAP_RANGE, 160 },
	{ 0xfe5f, 0xfe61, PDF_CMAP_RANGE, 205 },
	{ 0xfe62, 0xfe66, PDF_CMAP_RANGE, 223 },
	{ 0xfe69, 0xfe6b, PDF_CMAP_RANGE, 268 },
	{ 0xff01, 0xff07, PDF_CMAP_TABLE, 13481 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 128 },
	{ 0xff0a, 0xff0f, PDF_CMAP_TABLE, 13488 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 333 },
	{ 0xff1a, 0xff20, PDF_CMAP_TABLE, 13494 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 365 },
	{ 0xff3b, 0xff3f, PDF_CMAP_TABLE, 13501 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 391 },
	{ 0xff5b, 0xff5d, PDF_CMAP_TABLE, 13506 },
	{ 0xff64, 0xff64, PDF_CMAP_SINGLE, 113 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 14049 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 14050 },
	{ 0x677a, 0x677a, PDF_CMAP_SINGLE, 6591 },
	{ 0x677b, 0x677b, PDF_CMAP_SINGLE, 6584 },
	{ 0x677c, 0x677c, PDF_CMAP_SINGLE, 1559 },
	{ 0x677d, 0x677d, PDF_CMAP_SINGLE, 6596 },
	{ 0x677e, 0x677e, PDF_CMAP_SINGLE, 1554 },
	{ 0x677f, 0x677f, PDF_CMAP_SINGLE, 1552 },
	{ 0x6781, 0x6781, PDF_CMAP_SINGLE, 6597 },
	{ 0x6783, 0x6783, PDF_CMAP_SINGLE, 6595 },
	{ 0x6784, 0x6784, PDF_CMAP_SINGLE, 6587 },
	{ 0x6785, 0x6785, PDF_CMAP_SINGLE, 6579 },
	{ 0x6786, 0x6786, PDF_CMAP_SINGLE, 6586 },
	{ 0x6787, 0x6787, PDF_CMAP_SINGLE, 1547 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 1553 },
	{ 0x678b, 0x678b, PDF_CMAP_SINGLE, 1541 },
	{ 0x678c, 0x678c, PDF_CMAP_SINGLE, 6590 },
	{ 0x678d, 0x678d, PDF_CMAP_SINGLE, 6589 },
	{ 0x678e, 0x678e, PDF_CMAP_SINGLE, 6581 },
	{ 0x678f, 0x678f, PDF_CMAP_SINGLE, 18113 },
	{ 0x6790, 0x6790, PDF_CMAP_SINGLE, 1555 },
	{ 0x6791, 0x6791, PDF_CMAP_SINGLE, 6593 },
	{ 0x6792, 0x6792, PDF_CMAP_SINGLE, 6582 },
	{ 0x6793, 0x6793, PDF_CMAP_SINGLE, 1558 },
	{ 0x6794, 0x6794, PDF_CMAP_SINGLE, 6600 },
	{ 0x6795, 0x6795, PDF_CMAP_SINGLE, 1542 },
	{ 0x6797, 0x6797, PDF_CMAP_SINGLE, 1549 },
	{ 0x6798, 0x6798, PDF_CMAP_SINGLE, 6585 },
	{ 0x6799, 0x6799, PDF_CMAP_SINGLE, 6594 },
	{ 0x679a, 0x679a, PDF_CMAP_SINGLE, 1557 },
	{ 0x679c, 0x679c, PDF_CMAP_SINGLE, 1544 },
	{ 0x679d, 0x679d, PDF_CMAP_SINGLE, 1548 },
	{ 0x679f, 0x679f, PDF_CMAP_SINGLE, 6592 },
	{ 0x67a0, 0x67a0, PDF_CMAP_SINGLE, 15738 },
	{ 0x67a4, 0x67a4, PDF_CMAP_SINGLE, 18001 },
	{ 0x67ac, 0x67ac, PDF_CMAP_SINGLE, 16867 },
	{ 0x67ae, 0x67ae, PDF_CMAP_SINGLE, 6964 },
	{ 0x67af, 0x67af, PDF_CMAP_SINGLE, 1879 },
	{ 0x67b0, 0x67b0, PDF_CMAP_SINGLE, 1892 },
	{ 0x67b1, 0x67b1, PDF_CMAP_SINGLE, 16884 },
	{ 0x67b2, 0x67b2, PDF_CMAP_SINGLE, 6959 },
	{ 0x67b3, 0x67b3, PDF_CMAP_SINGLE, 6949 },
	{ 0x67b4, 0x67b4, PDF_CMAP_SINGLE, 1885 },
	{ 0x67b5, 0x67b5, PDF_CMAP_SINGLE, 6947 },
	{ 0x67b6, 0x67b6, PDF_CMAP_SINGLE, 1878 },
	{ 0x67b7, 0x67b7, PDF_CMAP_SINGLE, 6942 },
	{ 0x67b8, 0x67b8, PDF_CMAP_SINGLE, 1888 },
	{ 0x67b9, 0x67b9, PDF_CMAP_SINGLE, 6955 },
	{ 0x67ba, 0x67ba, PDF_CMAP_SINGLE, 6936 },
	{ 0x67bb, 0x67bb, PDF_CMAP_SINGLE, 6938 },
	{ 0x67bf, 0x67bf, PDF_CMAP_SINGLE, 14330 },
	{ 0x67c0, 0x67c0, PDF_CMAP_SINGLE, 6941 },
	{ 0x67c1, 0x67c1, PDF_CMAP_SINGLE, 6933 },
	{ 0x67c2, 0x67c2, PDF_CMAP_SINGLE, 6954 },
	{ 0x67c3, 0x67c3, PDF_CMAP_SINGLE, 6970 },
	{ 0x67c4, 0x67c4, PDF_CMAP_SINGLE, 1883 },
	{ 0x67c5, 0x67c5, PDF_CMAP_SINGLE, 6943 },
	{ 0x67c6, 0x67c6, PDF_CMAP_SINGLE, 6961 },
	{ 0x67c8, 0x67c8, PDF_CMAP_SINGLE, 6935 },
	{ 0x67c9, 0x67ca, PDF_CMAP_RANGE, 6968 },
	{ 0x67cb, 0x67cb, PDF_CMAP_SINGLE, 6972 },
	{ 0x67cc, 0x67cc, PDF_CMAP_SINGLE, 6963 },
	{ 0x67cd, 0x67cd, PDF_CMAP_SINGLE, 6948 },
	{ 0x67ce, 0x67ce, PDF_CMAP_SINGLE, 6956 },
	{ 0x67cf, 0x67cf, PDF_CMAP_SINGLE, 1889 },
	{ 0x67d0, 0x67d0, PDF_CMAP_SINGLE, 1876 },
	{ 0x67d1, 0x67d1, PDF_CMAP_SINGLE, 1884 },
	{ 0x67d2, 0x67d2, PDF_CMAP_SINGLE, 1896 },
	{ 0x67d3, 0x67d3, PDF_CMAP_SINGLE, 1873 },
	{ 0x67d4, 0x67d4, PDF_CMAP_SINGLE, 1875 },
	{ 0x67d6, 0x67d6, PDF_CMAP_SINGLE, 15192 },
	{ 0x67d7, 0x67d7, PDF_CMAP_SINGLE, 16846 },
	{ 0x67d8, 0x67d8, PDF_CMAP_SINGLE, 6940 },
	{ 0x67d9, 0x67d9, PDF_CMAP_SINGLE, 1893 },
	{ 0x67da, 0x67da, PDF_CMAP_SINGLE, 1886 },
	{ 0x67db, 0x67db, PDF_CMAP_SINGLE, 6966 },
	{ 0x67dc, 0x67dc, PDF_CMAP_SINGLE, 6937 },
	{ 0x67dd, 0x67dd, PDF_CMAP_SINGLE, 1895 },
	{ 0x67de, 0x67de, PDF_CMAP_SINGLE, 1890 },
	{ 0x67df, 0x67df, PDF_CMAP_SINGLE, 6946 },
	{ 0x67e2, 0x67e2, PDF_CMAP_SINGLE, 1894 },
	{ 0x67e3, 0x67e3, PDF_CMAP_SINGLE, 6953 },
	{ 0x67e4, 0x67e4, PDF_CMAP_SINGLE, 6945 },
	{ 0x67e5, 0x67e5, PDF_CMAP_SINGLE, 1887 },
	{ 0x67e6, 0x67e6, PDF_CMAP_SINGLE, 6965 },
	{ 0x67e7, 0x67e7, PDF_CMAP_SINGLE, 6957 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 1881 },
	{ 0x67ea, 0x67ea, PDF_CMAP_SINGLE, 6971 },
	{ 0x67eb, 0x67eb, PDF_CMAP_SINGLE, 6944 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 1877 },
	{ 0x67ed, 0x67ed, PDF_CMAP_SINGLE, 6962 },
	{ 0x67ee, 0x67ee, PDF_CMAP_SINGLE, 6952 },
	{ 0x67ef, 0x67ef, PDF_CMAP_SINGLE, 1882 },
	{ 0x67f0, 0x67f0, PDF_CMAP_SINGLE, 6958 },
	{ 0x67f1, 0x67f1, PDF_CMAP_SINGLE, 1874 },
	{ 0x67f2, 0x67f2, PDF_CMAP_SINGLE, 6934 },
	{ 0x67f3, 0x67f3, PDF_CMAP_SINGLE, 1891 },
	{ 0x67f4, 0x67f4, PDF_CMAP_SINGLE, 2293 },
	{ 0x67f5, 0x67f5, PDF_CMAP_SINGLE, 1880 },
	{ 0x67f6, 0x67f6, PDF_CMAP_SINGLE, 6951 },
	{ 0x67f7, 0x67f7, PDF_CMAP_SINGLE, 6950 },
	{ 0x67f8, 0x67f8, PDF_CMAP_SINGLE, 6939 },
	{ 0x67f9, 0x67f9, PDF_CMAP_SINGLE, 14742 },
	{ 0x67fa, 0x67fa, PDF_CMAP_SINGLE, 6967 },
	{ 0x67fc, 0x67fc, PDF_CMAP_SINGLE, 6960 },
	{ 0x67fe, 0x67fe, PDF_CMAP_SINGLE, 17749 },
	{ 0x67ff, 0x67ff, PDF_CMAP_SINGLE, 1872 },
	{ 0x6800, 0x6800, PDF_CMAP_SINGLE, 17351 },
	{ 0x6801, 0x6801, PDF_CMAP_SINGLE, 16851 },
	{ 0x6802, 0x6802, PDF_CMAP_SINGLE, 16328 },
	{ 0x6803, 0x6803, PDF_CMAP_SINGLE, 15722 },
	{ 0x6804, 0x6804, PDF_CMAP_SINGLE, 17750 },
	{ 0x680d, 0x680d, PDF_CMAP_SINGLE, 17025 },
	{ 0x6810, 0x6810, PDF_CMAP_SINGLE, 14748 },
	{ 0x6812, 0x6812, PDF_CMAP_SINGLE, 7389 },
	{ 0x6813, 0x6813, PDF_CMAP_SINGLE, 2300 },
	{ 0x6814, 0x6814, PDF_CMAP_SINGLE, 7390 },
	{ 0x6816, 0x6816, PDF_CMAP_SINGLE, 7378 },
	{ 0x6817, 0x6817, PDF_CMAP_SINGLE, 2289 },
	{ 0x6818, 0x6818, PDF_CMAP_SINGLE, 2301 },
	{ 0x681a, 0x681a, PDF_CMAP_SINGLE, 7371 },
	{ 0x681b, 0x681b, PDF_CMAP_SINGLE, 16498 },
	{ 0x681c, 0x681c, PDF_CMAP_SINGLE, 7380 },
	{ 0x681d, 0x681d, PDF_CMAP_SINGLE, 7388 },
	{ 0x681e, 0x681e, PDF_CMAP_SINGLE, 18114 },
	{ 0x681f, 0x681f, PDF_CMAP_SINGLE, 7370 },
	{ 0x6820, 0x6820, PDF_CMAP_SINGLE, 7397 },
	{ 0x6821, 0x6821, PDF_CMAP_SINGLE, 2279 },
	{ 0x6822, 0x6822, PDF_CMAP_SINGLE, 16848 },
	{ 0x6825, 0x6825, PDF_CMAP_SINGLE, 7396 },
	{ 0x6826, 0x6826, PDF_CMAP_SINGLE, 7391 },
	{ 0x6828, 0x6828, PDF_CMAP_SINGLE, 7392 },
	{ 0x6829, 0x6829, PDF_CMAP_SINGLE, 2287 },
	{ 0x682a, 0x682a, PDF_CMAP_SINGLE, 2298 },
	{ 0x682b, 0x682b, PDF_CMAP_SINGLE, 7382 },
	{ 0x682d, 0x682d, PDF_CMAP_SINGLE, 7383 },
	{ 0x682e, 0x682e, PDF_CMAP_SINGLE, 7393 },
	{ 0x682f, 0x682f, PDF_CMAP_SINGLE, 7384 },
	{ 0x6831, 0x6831, PDF_CMAP_SINGLE, 7379 },
	{ 0x6832, 0x6833, PDF_CMAP_RANGE, 7373 },
	{ 0x6834, 0x6834, PDF_CMAP_SINGLE, 7387 },
	{ 0x6835, 0x6835, PDF_CMAP_SINGLE, 7381 },
	{ 0x6836, 0x6836, PDF_CMAP_SINGLE, 15918 },
	{ 0x6837, 0x6837, PDF_CMAP_SINGLE, 16925 },
	{ 0x6838, 0x6838, PDF_CMAP_SINGLE, 2280 },
	{ 0x6839, 0x6839, PDF_CMAP_SINGLE, 2284 },
	{ 0x683a, 0x683a, PDF_CMAP_SINGLE, 7395 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 7375 },
	{ 0x683c, 0x683c, PDF_CMAP_SINGLE, 2296 },
	{ 0x683d, 0x683d, PDF_CMAP_SINGLE, 2292 },
	{ 0x683e, 0x683e, PDF_CMAP_SINGLE, 16711 },
	{ 0x6840, 0x6840, PDF_CMAP_SINGLE, 2295 },
	{ 0x6841, 0x6841, PDF_CMAP_SINGLE, 2302 },
	{ 0x6842, 0x6842, PDF_CMAP_SINGLE, 2285 },
	{ 0x6843, 0x6843, PDF_CMAP_SINGLE, 2297 },
	{ 0x6844, 0x6844, PDF_CMAP_SINGLE, 7386 },
	{ 0x6845, 0x6845, PDF_CMAP_SINGLE, 2299 },
	{ 0x6846, 0x6846, PDF_CMAP_SINGLE, 2282 },
	{ 0x6847, 0x6847, PDF_CMAP_SINGLE, 15339 },
	{ 0x6848, 0x6848, PDF_CMAP_SINGLE, 2281 },
	{ 0x6849, 0x6849, PDF_CMAP_SINGLE, 7372 },
	{ 0x684a, 0x684a, PDF_CMAP_SINGLE, 18003 },
	{ 0x684b, 0x684b, PDF_CMAP_SINGLE, 7376 },
	{ 0x684c, 0x684c, PDF_CMAP_SINGLE, 2290 },
	{ 0x684d, 0x684d, PDF_CMAP_SINGLE, 7394 },
	{ 0x684e, 0x684e, PDF_CMAP_SINGLE, 7385 },
	{ 0x684f, 0x684f, PDF_CMAP_SINGLE, 7377 },
	{ 0x6850, 0x6850, PDF_CMAP_SINGLE, 2294 },
	{ 0x6851, 0x6851, PDF_CMAP_SINGLE, 2291 },
	{ 0x6853, 0x6853, PDF_CMAP_SINGLE, 2283 },
	{ 0x6854, 0x6854, PDF_CMAP_SINGLE, 2286 },
	{ 0x6855, 0x6855, PDF_CMAP_SINGLE, 15760 },
	{ 0x6856, 0x6856, PDF_CMAP_SINGLE, 14356 },
	{ 0x685d, 0x685d, PDF_CMAP_SINGLE, 16854 },
	{ 0x6865, 0x6865, PDF_CMAP_SINGLE, 17751 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 7977 },
	{ 0x686d, 0x686e, PDF_CMAP_RANGE, 7961 },
	{ 0x686f, 0x686f, PDF_CMAP_SINGLE, 7966 },
	{ 0x6871, 0x6871, PDF_CMAP_SINGLE, 7981 },
	{ 0x6872, 0x6872, PDF_CMAP_SINGLE, 7978 },
	{ 0x6874, 0x6874, PDF_CMAP_SINGLE, 7971 },
	{ 0x6875, 0x6875, PDF_CMAP_SINGLE, 7970 },
	{ 0x6876, 0x6876, PDF_CMAP_SINGLE, 2775 },
	{ 0x6877, 0x6877, PDF_CMAP_SINGLE, 7974 },
	{ 0x6878, 0x6878, PDF_CMAP_SINGLE, 7989 },
	{ 0x6879, 0x6879, PDF_CMAP_SINGLE, 7957 },
	{ 0x687b, 0x687b, PDF_CMAP_SINGLE, 7990 },
	{ 0x687c, 0x687c, PDF_CMAP_SINGLE, 7976 },
	{ 0x687d, 0x687d, PDF_CMAP_SINGLE, 7994 },
	{ 0x687e, 0x687e, PDF_CMAP_SINGLE, 7982 },
	{ 0x687f, 0x687f, PDF_CMAP_SINGLE, 2774 },
	{ 0x6880, 0x6880, PDF_CMAP_SINGLE, 7980 },
	{ 0x6881, 0x6881, PDF_CMAP_SINGLE, 2769 },
	{ 0x6882, 0x6882, PDF_CMAP_SINGLE, 2790 },
	{ 0x6883, 0x6883, PDF_CMAP_SINGLE, 2780 },
	{ 0x6884, 0x6884, PDF_CMAP_SINGLE, 18004 },
	{ 0x6885, 0x6885, PDF_CMAP_SINGLE, 2784 },
	{ 0x6886, 0x6886, PDF_CMAP_SINGLE, 2783 },
	{ 0x6887, 0x6887, PDF_CMAP_SINGLE, 7958 },
	{ 0x6888, 0x6888, PDF_CMAP_SINGLE, 18103 },
	{ 0x6889, 0x6889, PDF_CMAP_SINGLE, 7987 },
	{ 0x688a, 0x688a, PDF_CMAP_SINGLE, 7993 },
	{ 0x688b, 0x688b, PDF_CMAP_SINGLE, 7985 },
	{ 0x688c, 0x688c, PDF_CMAP_SINGLE, 7992 },
	{ 0x688f, 0x688f, PDF_CMAP_SINGLE, 7973 },
	{ 0x6890, 0x6890, PDF_CMAP_SINGLE, 7959 },
	{ 0x6891, 0x6891, PDF_CMAP_SINGLE, 7991 },
	{ 0x6892, 0x6892, PDF_CMAP_SINGLE, 7975 },
	{ 0x6893, 0x6893, PDF_CMAP_SINGLE, 2772 },
	{ 0x6894, 0x6894, PDF_CMAP_SINGLE, 2785 },
	{ 0x6896, 0x6896, PDF_CMAP_SINGLE, 7984 },
	{ 0x6897, 0x6897, PDF_CMAP_SINGLE, 2778 },
	{ 0x6898, 0x6898, PDF_CMAP_SINGLE, 15018 },
	{ 0x689b, 0x689b, PDF_CMAP_SINGLE, 7983 },
	{ 0x689c, 0x689c, PDF_CMAP_SINGLE, 7960 },
	{ 0x689d, 0x689d, PDF_CMAP_SINGLE, 2786 },
	{ 0x689f, 0x689f, PDF_CMAP_SINGLE, 2788 },
	{ 0x68a0, 0x68a0, PDF_CMAP_SINGLE, 7986 },
	{ 0x68a1, 0x68a1, PDF_CMAP_SINGLE, 2789 },
	{ 0x68a2, 0x68a2, PDF_CMAP_SINGLE, 2771 },
	{ 0x68a3, 0x68a3, PDF_CMAP_SINGLE, 7967 },
	{ 0x68a4, 0x68a4, PDF_CMAP_SINGLE, 7988 },
	{ 0x68a6, 0x68a6, PDF_CMAP_SINGLE, 16612 },
	{ 0x68a7, 0x68a7, PDF_CMAP_SINGLE, 2777 },
	{ 0x68a8, 0x68a8, PDF_CMAP_SINGLE, 2787 },
	{ 0x68a9, 0x68a9, PDF_CMAP_SINGLE, 7969 },
	{ 0x68aa, 0x68aa, PDF_CMAP_SINGLE, 7979 },
	{ 0x68ab, 0x68ab, PDF_CMAP_SINGLE, 7964 },
	{ 0x68ac, 0x68ac, PDF_CMAP_SINGLE, 7968 },
	{ 0x68ad, 0x68ad, PDF_CMAP_SINGLE, 2782 },
	{ 0x68ae, 0x68ae, PDF_CMAP_SINGLE, 7963 },
	{ 0x68af, 0x68af, PDF_CMAP_SINGLE, 2770 },
	{ 0x68b0, 0x68b0, PDF_CMAP_SINGLE, 2779 },
	{ 0x68b1, 0x68b1, PDF_CMAP_SINGLE, 2776 },
	{ 0x68b2, 0x68b2, PDF_CMAP_SINGLE, 7972 },
	{ 0x68b3, 0x68b3, PDF_CMAP_SINGLE, 2288 },
	{ 0x68b4, 0x68b4, PDF_CMAP_SINGLE, 8600 },
	{ 0x68b5, 0x68b5, PDF_CMAP_SINGLE, 2773 },
	{ 0x68b6, 0x68b6, PDF_CMAP_SINGLE, 15006 },
	{ 0x68b9, 0x68b9, PDF_CMAP_SINGLE, 15636 },
	{ 0x68bd, 0x68bd, PDF_CMAP_SINGLE, 17020 },
	{ 0x68c3, 0x68c3, PDF_CMAP_SINGLE, 16860 },
	{ 0x68c4, 0x68c4, PDF_CMAP_SINGLE, 2781 },
	{ 0x68c5, 0x68c5, PDF_CMAP_SINGLE, 15406 },
	{ 0x68c6, 0x68c6, PDF_CMAP_SINGLE, 8603 },
	{ 0x68c7, 0x68c7, PDF_CMAP_SINGLE, 8625 },
	{ 0x68c8, 0x68c8, PDF_CMAP_SINGLE, 8614 },
	{ 0x68c9, 0x68c9, PDF_CMAP_SINGLE, 3223 },
	{ 0x68ca, 0x68ca, PDF_CMAP_SINGLE, 17131 },
	{ 0x68cb, 0x68cb, PDF_CMAP_SINGLE, 3218 },
	{ 0x68cc, 0x68cc, PDF_CMAP_SINGLE, 8597 },
	{ 0x68cd, 0x68cd, PDF_CMAP_SINGLE, 3219 },
	{ 0x68ce, 0x68ce, PDF_CMAP_SINGLE, 8613 },
	{ 0x68d0, 0x68d0, PDF_CMAP_SINGLE, 8606 },
	{ 0x68d1, 0x68d1, PDF_CMAP_SINGLE, 8619 },
	{ 0x68d2, 0x68d2, PDF_CMAP_SINGLE, 3215 },
	{ 0x68d3, 0x68d3, PDF_CMAP_SINGLE, 8579 },
	{ 0x68d4, 0x68d4, PDF_CMAP_SINGLE, 8621 },
	{ 0x68d5, 0x68d5, PDF_CMAP_SINGLE, 3205 },
	{ 0x68d6, 0x68d6, PDF_CMAP_SINGLE, 8587 },
	{ 0x68d7, 0x68d7, PDF_CMAP_SINGLE, 3208 },
	{ 0x68d8, 0x68d8, PDF_CMAP_SINGLE, 3207 },
	{ 0x68da, 0x68da, PDF_CMAP_SINGLE, 3224 },
	{ 0x68dc, 0x68dc, PDF_CMAP_SINGLE, 8581 },
	{ 0x68dd, 0x68de, PDF_CMAP_RANGE, 8615 },
	{ 0x68df, 0x68df, PDF_CMAP_SINGLE, 3210 },
	{ 0x68e0, 0x68e0, PDF_CMAP_SINGLE, 3206 },
	{ 0x68e1, 0x68e1, PDF_CMAP_SINGLE, 8595 },
	{ 0x68e3, 0x68e3, PDF_CMAP_SINGLE, 3217 },
	{ 0x68e4, 0x68e4, PDF_CMAP_SINGLE, 8590 },
	{ 0x68e6, 0x68e6, PDF_CMAP_SINGLE, 8617 },
	{ 0x68e7, 0x68e7, PDF_CMAP_SINGLE, 3213 },
	{ 0x68e8, 0x68e8, PDF_CMAP_SINGLE, 8609 },
	{ 0x68e9, 0x68e9, PDF_CMAP_SINGLE, 8622 },
	{ 0x68ea, 0x68ea, PDF_CMAP_SINGLE, 8584 },
	{ 0x68eb, 0x68eb, PDF_CMAP_SINGLE, 8589 },
	{ 0x68ec, 0x68ec, PDF_CMAP_SINGLE, 8583 },
	{ 0x68ee, 0x68ee, PDF_CMAP_SINGLE, 3212 },
	{ 0x68ef, 0x68ef, PDF_CMAP_SINGLE, 8602 },
	{ 0x68f0, 0x68f0, PDF_CMAP_SINGLE, 9259 },
	{ 0x68f1, 0x68f1, PDF_CMAP_SINGLE, 8585 },
	{ 0x68f2, 0x68f2, PDF_CMAP_SINGLE, 3216 },
	{ 0x68f3, 0x68f3, PDF_CMAP_SINGLE, 8594 },
	{ 0x68f4, 0x68f4, PDF_CMAP_SINGLE, 8618 },
	{ 0x68f5, 0x68f5, PDF_CMAP_SINGLE, 3211 },
	{ 0x68f6, 0x68f6, PDF_CMAP_SINGLE, 8591 },
	{ 0x68f7, 0x68f7, PDF_CMAP_SINGLE, 8588 },
	{ 0x68f8, 0x68f8, PDF_CMAP_SINGLE, 8605 },
	{ 0x68f9, 0x68f9, PDF_CMAP_SINGLE, 3214 },
	{ 0x68fa, 0x68fa, PDF_CMAP_SINGLE, 3204 },
	{ 0x68fb, 0x68fb, PDF_CMAP_SINGLE, 3226 },
	{ 0x68fc, 0x68fc, PDF_CMAP_SINGLE, 8608 },
	{ 0x68fd, 0x68fd, PDF_CMAP_SINGLE, 8607 },
	{ 0x6900, 0x6900, PDF_CMAP_SINGLE, 16863 },
	{ 0x6901, 0x6901, PDF_CMAP_SINGLE, 16862 },
	{ 0x6902, 0x6902, PDF_CMAP_SINGLE, 17147 },
	{ 0x6903, 0x6903, PDF_CMAP_SINGLE, 17052 },
	{ 0x6904, 0x6904, PDF_CMAP_SINGLE, 8580 },
	{ 0x6905, 0x6905, PDF_CMAP_SINGLE, 3209 },
	{ 0x6906, 0x6906, PDF_CMAP_SINGLE, 8620 },
	{ 0x6907, 0x6907, PDF_CMAP_SINGLE, 8596 },
	{ 0x6908, 0x6908, PDF_CMAP_SINGLE, 8598 },
	{ 0x6909, 0x6909, PDF_CMAP_SINGLE, 18008 },
	{ 0x690a, 0x690a, PDF_CMAP_SINGLE, 8611 },
	{ 0x690b, 0x690b, PDF_CMAP_SINGLE, 8610 },
	{ 0x690c, 0x690c, PDF_CMAP_SINGLE, 8578 },
	{ 0x690d, 0x690d, PDF_CMAP_SINGLE, 3220 },
	{ 0x690e, 0x690e, PDF_CMAP_SINGLE, 3222 },
	{ 0x690f, 0x690f, PDF_CMAP_SINGLE, 8586 },
	{ 0x6910, 0x6910, PDF_CMAP_SINGLE, 8593 },
	{ 0x6911, 0x6911, PDF_CMAP_SINGLE, 8601 },
	{ 0x6912, 0x6912, PDF_CMAP_SINGLE, 3221 },
	{ 0x6913, 0x6913, PDF_CMAP_SINGLE, 8592 },
	{ 0x6914, 0x6914, PDF_CMAP_SINGLE, 8604 },
	{ 0x6915, 0x6915, PDF_CMAP_SINGLE, 8623 },
	{ 0x6917, 0x6917, PDF_CMAP_SINGLE, 8612 },
	{ 0x6918, 0x6918, PDF_CMAP_SINGLE, 15778 },
	{ 0x691a, 0x691a, PDF_CMAP_SINGLE, 15744 },
	{ 0x691b, 0x691b, PDF_CMAP_SINGLE, 14269 },
	{ 0x6925, 0x6925, PDF_CMAP_SINGLE, 8624 },
	{ 0x692a, 0x692a, PDF_CMAP_SINGLE, 8582 },
	{ 0x692c, 0x692c, PDF_CMAP_SINGLE, 16184 },
	{ 0x692f, 0x692f, PDF_CMAP_SINGLE, 9279 },
	{ 0x6930, 0x6930, PDF_CMAP_SINGLE, 3646 },
	{ 0x6932, 0x6932, PDF_CMAP_SINGLE, 9277 },
	{ 0x6933, 0x6933, PDF_CMAP_SINGLE, 9256 },
	{ 0x6934, 0x6934, PDF_CMAP_SINGLE, 9261 },
	{ 0x6935, 0x6935, PDF_CMAP_SINGLE, 9254 },
	{ 0x6936, 0x6936, PDF_CMAP_SINGLE, 16871 },
	{ 0x6937, 0x6937, PDF_CMAP_SINGLE, 9273 },
	{ 0x6938, 0x6938, PDF_CMAP_SINGLE, 9240 },
	{ 0x6939, 0x6939, PDF_CMAP_SINGLE, 9247 },
	{ 0x693b, 0x693b, PDF_CMAP_SINGLE, 9271 },
	{ 0x693c, 0x693c, PDF_CMAP_SINGLE, 9281 },
	{ 0x693d, 0x693d, PDF_CMAP_SINGLE, 9257 },
	{ 0x693e, 0x693e, PDF_CMAP_SINGLE, 16470 },
	{ 0x693f, 0x693f, PDF_CMAP_SINGLE, 9244 },
	{ 0x6940, 0x6940, PDF_CMAP_SINGLE, 9263 },
	{ 0x6941, 0x6941, PDF_CMAP_SINGLE, 9268 },
	{ 0x6942, 0x6942, PDF_CMAP_SINGLE, 9248 },
	{ 0x6943, 0x6943, PDF_CMAP_SINGLE, 18009 },
	{ 0x6944, 0x6944, PDF_CMAP_SINGLE, 9265 },
	{ 0x6945, 0x6945, PDF_CMAP_SINGLE, 9245 },
	{ 0x6946, 0x6946, PDF_CMAP_SINGLE, 17775 },
	{ 0x6948, 0x6949, PDF_CMAP_RANGE, 9252 },
	{ 0x694a, 0x694a, PDF_CMAP_SINGLE, 3648 },
	{ 0x694b, 0x694b, PDF_CMAP_SINGLE, 9272 },
	{ 0x694c, 0x694c, PDF_CMAP_SINGLE, 9270 },
	{ 0x694e, 0x694e, PDF_CMAP_SINGLE, 9241 },
	{ 0x694f, 0x694f, PDF_CMAP_SINGLE, 9275 },
	{ 0x6951, 0x6951, PDF_CMAP_SINGLE, 9276 },
	{ 0x6952, 0x6952, PDF_CMAP_SINGLE, 9278 },
	{ 0x6953, 0x6953, PDF_CMAP_SINGLE, 3652 },
	{ 0x6954, 0x6954, PDF_CMAP_SINGLE, 3644 },
	{ 0x6955, 0x6955, PDF_CMAP_SINGLE, 15750 },
	{ 0x6956, 0x6956, PDF_CMAP_SINGLE, 7965 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 9249 },
	{ 0x6958, 0x6958, PDF_CMAP_SINGLE, 9267 },
	{ 0x6959, 0x6959, PDF_CMAP_SINGLE, 9250 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 3641 },
	{ 0x695b, 0x695b, PDF_CMAP_SINGLE, 3657 },
	{ 0x695c, 0x695c, PDF_CMAP_SINGLE, 9274 },
	{ 0x695d, 0x695d, PDF_CMAP_SINGLE, 3655 },
	{ 0x695e, 0x695e, PDF_CMAP_SINGLE, 3651 },
	{ 0x695f, 0x695f, PDF_CMAP_SINGLE, 9239 },
	{ 0x6960, 0x6960, PDF_CMAP_SINGLE, 3643 },
	{ 0x6961, 0x6961, PDF_CMAP_SINGLE, 16868 },
	{ 0x6962, 0x6962, PDF_CMAP_SINGLE, 9242 },
	{ 0x6963, 0x6963, PDF_CMAP_SINGLE, 3656 },
	{ 0x6964, 0x6964, PDF_CMAP_SINGLE, 18011 },
	{ 0x6965, 0x6965, PDF_CMAP_SINGLE, 9258 },
	{ 0x6966, 0x6966, PDF_CMAP_SINGLE, 9238 },
	{ 0x6967, 0x6967, PDF_CMAP_SINGLE, 15637 },
	{ 0x6968, 0x6968, PDF_CMAP_SINGLE, 3649 },
	{ 0x6969, 0x6969, PDF_CMAP_SINGLE, 9262 },
	{ 0x696a, 0x696a, PDF_CMAP_SINGLE, 9246 },
	{ 0x696b, 0x696b, PDF_CMAP_SINGLE, 3650 },
	{ 0x696c, 0x696c, PDF_CMAP_SINGLE, 9255 },
	{ 0x696d, 0x696d, PDF_CMAP_SINGLE, 3640 },
	{ 0x696e, 0x696e, PDF_CMAP_SINGLE, 3225 },
	{ 0x696f, 0x696f, PDF_CMAP_SINGLE, 9264 },
	{ 0x6970, 0x6970, PDF_CMAP_SINGLE, 8599 },
	{ 0x6971, 0x6971, PDF_CMAP_SINGLE, 9243 },
	{ 0x6972, 0x6972, PDF_CMAP_SINGLE, 14957 },
	{ 0x6973, 0x6973, PDF_CMAP_SINGLE, 16859 },
	{ 0x6974, 0x6974, PDF_CMAP_SINGLE, 9269 },
	{ 0x6975, 0x6975, PDF_CMAP_SINGLE, 3645 },
	{ 0x6976, 0x6976, PDF_CMAP_SINGLE, 9266 },
	{ 0x6977, 0x6977, PDF_CMAP_SINGLE, 3642 },
	{ 0x6978, 0x6978, PDF_CMAP_SINGLE, 9260 },
	{ 0x6979, 0x6979, PDF_CMAP_SINGLE, 3653 },
	{ 0x697a, 0x697a, PDF_CMAP_SINGLE, 9251 },
	{ 0x697b, 0x697b, PDF_CMAP_SINGLE, 9280 },
	{ 0x6980, 0x6980, PDF_CMAP_SINGLE, 16190 },
	{ 0x6982, 0x6982, PDF_CMAP_SINGLE, 3647 },
	{ 0x6983, 0x6983, PDF_CMAP_SINGLE, 9391 },
	{ 0x6985, 0x6985, PDF_CMAP_SINGLE, 18013 },
	{ 0x6986, 0x6986, PDF_CMAP_SINGLE, 3654 },
	{ 0x698a, 0x698a, PDF_CMAP_SINGLE, 15732 },
	{ 0x698d, 0x698d, PDF_CMAP_SINGLE, 9929 },
	{ 0x698e, 0x698e, PDF_CMAP_SINGLE, 9927 },
	{ 0x6990, 0x6990, PDF_CMAP_SINGLE, 9949 },
	{ 0x6991, 0x6991, PDF_CMAP_SINGLE, 9925 },
	{ 0x6993, 0x6993, PDF_CMAP_SINGLE, 9943 },
	{ 0x6994, 0x6994, PDF_CMAP_SINGLE, 3639 },
	{ 0x6995, 0x6995, PDF_CMAP_SINGLE, 4080 },
	{ 0x6996, 0x6996, PDF_CMAP_SINGLE, 9921 },
	{ 0x6997, 0x6997, PDF_CMAP_SINGLE, 9948 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 16872 },
	{ 0x6999, 0x6999, PDF_CMAP_SINGLE, 9926 },
	{ 0x699a, 0x699a, PDF_CMAP_SINGLE, 9940 },
	{ 0x699b, 0x699b, PDF_CMAP_SINGLE, 4085 },
	{ 0x699c, 0x699c, PDF_CMAP_SINGLE, 4078 },
	{ 0x699e, 0x699e, PDF_CMAP_SINGLE, 9946 },
	{ 0x699f, 0x699f, PDF_CMAP_SINGLE, 18012 },
	{ 0x69a0, 0x69a0, PDF_CMAP_SINGLE, 9919 },
	{ 0x69a1, 0x69a1, PDF_CMAP_SINGLE, 9945 },
	{ 0x69a2, 0x69a2, PDF_CMAP_SINGLE, 14128 },
	{ 0x69a3, 0x69a3, PDF_CMAP_SINGLE, 4096 },
	{ 0x69a4, 0x69a4, PDF_CMAP_SINGLE, 9936 },
	{ 0x69a5, 0x69a5, PDF_CMAP_SINGLE, 9952 },
	{ 0x69a6, 0x69a6, PDF_CMAP_SINGLE, 4094 },
	{ 0x69a7, 0x69a7, PDF_CMAP_SINGLE, 9928 },
	{ 0x69a8, 0x69a8, PDF_CMAP_SINGLE, 4079 },
	{ 0x69a9, 0x69a9, PDF_CMAP_SINGLE, 9930 },
	{ 0x69aa, 0x69aa, PDF_CMAP_SINGLE, 9944 },
	{ 0x69ab, 0x69ab, PDF_CMAP_SINGLE, 4088 },
	{ 0x69ac, 0x69ac, PDF_CMAP_SINGLE, 9923 },
	{ 0x69ad, 0x69ad, PDF_CMAP_SINGLE, 4092 },
	{ 0x69ae, 0x69ae, PDF_CMAP_SINGLE, 4082 },
	{ 0x69af, 0x69af, PDF_CMAP_SINGLE, 9932 },
	{ 0x69b0, 0x69b0, PDF_CMAP_SINGLE, 9922 },
	{ 0x69b1, 0x69b1, PDF_CMAP_SINGLE, 9916 },
	{ 0x69b2, 0x69b2, PDF_CMAP_SINGLE, 16374 },
	{ 0x69b3, 0x69b3, PDF_CMAP_SINGLE, 9942 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 4089 },
	{ 0x69b5, 0x69b5, PDF_CMAP_SINGLE, 9951 },
	{ 0x69b6, 0x69b6, PDF_CMAP_SINGLE, 9917 },
	{ 0x69b7, 0x69b7, PDF_CMAP_SINGLE, 4086 },
	{ 0x69b9, 0x69b9, PDF_CMAP_SINGLE, 9938 },
	{ 0x69bb, 0x69bb, PDF_CMAP_SINGLE, 4087 },
	{ 0x69bc, 0x69bc, PDF_CMAP_SINGLE, 9924 },
	{ 0x69bd, 0x69bd, PDF_CMAP_SINGLE, 9935 },
	{ 0x69be, 0x69be, PDF_CMAP_SINGLE, 9931 },
	{ 0x69bf, 0x69bf, PDF_CMAP_SINGLE, 9933 },
	{ 0x69c0, 0x69c0, PDF_CMAP_SINGLE, 16221 },
	{ 0x69c1, 0x69c1, PDF_CMAP_SINGLE, 4081 },
	{ 0x69c2, 0x69c2, PDF_CMAP_SINGLE, 9950 },
	{ 0x69c3, 0x69c3, PDF_CMAP_SINGLE, 4095 },
	{ 0x69c4, 0x69c4, PDF_CMAP_SINGLE, 9934 },
	{ 0x69c6, 0x69c6, PDF_CMAP_SINGLE, 9953 },
	{ 0x69c9, 0x69c9, PDF_CMAP_SINGLE, 9918 },
	{ 0x69ca, 0x69ca, PDF_CMAP_SINGLE, 9939 },
	{ 0x69cb, 0x69cb, PDF_CMAP_SINGLE, 4084 },
	{ 0x69cc, 0x69cc, PDF_CMAP_SINGLE, 4093 },
	{ 0x69cd, 0x69cd, PDF_CMAP_SINGLE, 4091 },
	{ 0x69ce, 0x69ce, PDF_CMAP_SINGLE, 9920 },
	{ 0x69cf, 0x69cf, PDF_CMAP_SINGLE, 9941 },
	{ 0x69d0, 0x69d0, PDF_CMAP_SINGLE, 4090 },
	{ 0x69d1, 0x69d1, PDF_CMAP_SINGLE, 14831 },
	{ 0x69d3, 0x69d3, PDF_CMAP_SINGLE, 4083 },
	{ 0x69d4, 0x69d4, PDF_CMAP_SINGLE, 9937 },
	{ 0x69d5, 0x69d5, PDF_CMAP_SINGLE, 15932 },
	{ 0x69d6, 0x69d6, PDF_CMAP_SINGLE, 18014 },
	{ 0x69d9, 0x69d9, PDF_CMAP_SINGLE, 9947 },
	{ 0x69e1, 0x69e1, PDF_CMAP_SINGLE, 16856 },
	{ 0x69e2, 0x69e2, PDF_CMAP_SINGLE, 10505 },
	{ 0x69e4, 0x69e4, PDF_CMAP_SINGLE, 10501 },
	{ 0x69e5, 0x69e5, PDF_CMAP_SINGLE, 10497 },
	{ 0x69e6, 0x69e6, PDF_CMAP_SINGLE, 10517 },
	{ 0x69e7, 0x69e7, PDF_CMAP_SINGLE, 10514 },
	{ 0x69e8, 0x69e8, PDF_CMAP_SINGLE, 4471 },
	{ 0x69e9, 0x69e9, PDF_CMAP_SINGLE, 15761 },
	{ 0x69eb, 0x69eb, PDF_CMAP_SINGLE, 10521 },
	{ 0x69ec, 0x69ec, PDF_CMAP_SINGLE, 10504 },
	{ 0x69ed, 0x69ed, PDF_CMAP_SINGLE, 4482 },
	{ 0x69ee, 0x69ee, PDF_CMAP_SINGLE, 10511 },
	{ 0x69f1, 0x69f1, PDF_CMAP_SINGLE, 10500 },
	{ 0x69f2, 0x69f2, PDF_CMAP_SINGLE, 10510 },
	{ 0x69f3, 0x69f3, PDF_CMAP_SINGLE, 4479 },
	{ 0x69f4, 0x69f4, PDF_CMAP_SINGLE, 10530 },
	{ 0x69f6, 0x69f6, PDF_CMAP_SINGLE, 10527 },
	{ 0x69f7, 0x69f7, PDF_CMAP_SINGLE, 10513 },
	{ 0x69f8, 0x69f8, PDF_CMAP_SINGLE, 10498 },
	{ 0x69f9, 0x69f9, PDF_CMAP_SINGLE, 16877 },
	{ 0x69fa, 0x69fa, PDF_CMAP_SINGLE, 16839 },
	{ 0x69fb, 0x69fb, PDF_CMAP_SINGLE, 10518 },
	{ 0x69fc, 0x69fc, PDF_CMAP_SINGLE, 10520 },
	{ 0x69fd, 0x69fd, PDF_CMAP_SINGLE, 4475 },
	{ 0x69fe, 0x69fe, PDF_CMAP_SINGLE, 10508 },
	{ 0x69ff, 0x69ff, PDF_CMAP_SINGLE, 10503 },
	{ 0x6a00, 0x6a00, PDF_CMAP_SINGLE, 10494 },
	{ 0x6a01, 0x6a01, PDF_CMAP_SINGLE, 4472 },
	{ 0x6a02, 0x6a02, PDF_CMAP_SINGLE, 4480 },
	{ 0x6a03, 0x6a03, PDF_CMAP_SINGLE, 15913 },
	{ 0x6a04, 0x6a04, PDF_CMAP_SINGLE, 10523 },
	{ 0x6a05, 0x6a05, PDF_CMAP_SINGLE, 4481 },
	{ 0x6a06, 0x6a06, PDF_CMAP_SINGLE, 10495 },
	{ 0x6a07, 0x6a07, PDF_CMAP_SINGLE, 10529 },
	{ 0x6a08, 0x6a08, PDF_CMAP_SINGLE, 10516 },
	{ 0x6a09, 0x6a09, PDF_CMAP_SINGLE, 10522 },
	{ 0x6a0a, 0x6a0a, PDF_CMAP_SINGLE, 4478 },
	{ 0x6a0b, 0x6a0b, PDF_CMAP_SINGLE, 16857 },
	{ 0x6a0c, 0x6a0c, PDF_CMAP_SINGLE, 15696 },
	{ 0x6a0d, 0x6a0d, PDF_CMAP_SINGLE, 10519 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 10526 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 4483 },
	{ 0x6a13, 0x6a13, PDF_CMAP_SINGLE, 4477 },
	{ 0x6a14, 0x6a14, PDF_CMAP_SINGLE, 10512 },
	{ 0x6a15, 0x6a15, PDF_CMAP_SINGLE, 10499 },
	{ 0x6a16, 0x6a16, PDF_CMAP_SINGLE, 10531 },
	{ 0x6a17, 0x6a17, PDF_CMAP_SINGLE, 10496 },
	{ 0x6a18, 0x6a18, PDF_CMAP_SINGLE, 10524 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 4474 },
	{ 0x6a1a, 0x6a1a, PDF_CMAP_SINGLE, 15646 },
	{ 0x6a1b, 0x6a1b, PDF_CMAP_SINGLE, 10506 },
	{ 0x6a1c, 0x6a1c, PDF_CMAP_SINGLE, 15208 },
	{ 0x6a1d, 0x6a1d, PDF_CMAP_SINGLE, 10507 },
	{ 0x6a1e, 0x6a1e, PDF_CMAP_SINGLE, 4473 },
	{ 0x6a1f, 0x6a1f, PDF_CMAP_SINGLE, 4470 },
	{ 0x6a20, 0x6a20, PDF_CMAP_SINGLE, 10502 },
	{ 0x6a21, 0x6a21, PDF_CMAP_SINGLE, 4476 },
	{ 0x6a23, 0x6a23, PDF_CMAP_SINGLE, 4469 },
	{ 0x6a25, 0x6a25, PDF_CMAP_SINGLE, 10525 },
	{ 0x6a26, 0x6a26, PDF_CMAP_SINGLE, 10528 },
	{ 0x6a27, 0x6a27, PDF_CMAP_SINGLE, 10509 },
	{ 0x6a28, 0x6a28, PDF_CMAP_SINGLE, 11124 },
	{ 0x6a2b, 0x6a2b, PDF_CMAP_SINGLE, 15682 },
	{ 0x6a2c, 0x6a2c, PDF_CMAP_SINGLE, 17067 },
	{ 0x6a2d, 0x6a2d, PDF_CMAP_SINGLE, 15135 },
	{ 0x6a32, 0x6a32, PDF_CMAP_SINGLE, 11116 },
	{ 0x6a33, 0x6a33, PDF_CMAP_SINGLE, 15012 },
	{ 0x6a34, 0x6a34, PDF_CMAP_SINGLE, 11112 },
	{ 0x6a35, 0x6a35, PDF_CMAP_SINGLE, 4825 },
	{ 0x6a38, 0x6a38, PDF_CMAP_SINGLE, 4814 },
	{ 0x6a39, 0x6a39, PDF_CMAP_SINGLE, 4819 },
	{ 0x6a3a, 0x6a3a, PDF_CMAP_SINGLE, 4815 },
	{ 0x6a3b, 0x6a3b, PDF_CMAP_SINGLE, 11126 },
	{ 0x6a3c, 0x6a3c, PDF_CMAP_SINGLE, 11137 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 4813 },
	{ 0x6a3e, 0x6a3e, PDF_CMAP_SINGLE, 11118 },
	{ 0x6a3f, 0x6a3f, PDF_CMAP_SINGLE, 11127 },
	{ 0x6a40, 0x6a40, PDF_CMAP_SINGLE, 10515 },
	{ 0x6a41, 0x6a41, PDF_CMAP_SINGLE, 11128 },
	{ 0x6a44, 0x6a44, PDF_CMAP_SINGLE, 4820 },
	{ 0x6a45, 0x6a45, PDF_CMAP_SINGLE, 16880 },
	{ 0x6a46, 0x6a46, PDF_CMAP_SINGLE, 11143 },
	{ 0x6a47, 0x6a47, PDF_CMAP_SINGLE, 4824 },
	{ 0x6a48, 0x6a48, PDF_CMAP_SINGLE, 4827 },
	{ 0x6a49, 0x6a49, PDF_CMAP_SINGLE, 11114 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 4823 },
	{ 0x6a4c, 0x6a4c, PDF_CMAP_SINGLE, 15019 },
	{ 0x6a4d, 0x6a4e, PDF_CMAP_RANGE, 11141 },
	{ 0x6a4f, 0x6a4f, PDF_CMAP_SINGLE, 11132 },
	{ 0x6a50, 0x6a50, PDF_CMAP_SINGLE, 11131 },
	{ 0x6a51, 0x6a51, PDF_CMAP_SINGLE, 11123 },
	{ 0x6a52, 0x6a52, PDF_CMAP_SINGLE, 15013 },
	{ 0x6a53, 0x6a53, PDF_CMAP_SINGLE, 14252 },
	{ 0x6a54, 0x6a54, PDF_CMAP_SINGLE, 11133 },
	{ 0x6a55, 0x6a55, PDF_CMAP_SINGLE, 11140 },
	{ 0x6a56, 0x6a56, PDF_CMAP_SINGLE, 11139 },
	{ 0x6a57, 0x6a57, PDF_CMAP_SINGLE, 14298 },
	{ 0x6a58, 0x6a58, PDF_CMAP_SINGLE, 4818 },
	{ 0x6a59, 0x6a59, PDF_CMAP_SINGLE, 4816 },
	{ 0x6a5a, 0x6a5a, PDF_CMAP_SINGLE, 11125 },
	{ 0x6a5b, 0x6a5b, PDF_CMAP_SINGLE, 11122 },
	{ 0x6a5d, 0x6a5d, PDF_CMAP_SINGLE, 11119 },
	{ 0x6a5e, 0x6a5e, PDF_CMAP_SINGLE, 11138 },
	{ 0x6a5f, 0x6a5f, PDF_CMAP_SINGLE, 4826 },
	{ 0x6a60, 0x6a60, PDF_CMAP_SINGLE, 11136 },
	{ 0x6a61, 0x6a61, PDF_CMAP_SINGLE, 4822 },
	{ 0x6a62, 0x6a62, PDF_CMAP_SINGLE, 4821 },
	{ 0x6a64, 0x6a64, PDF_CMAP_SINGLE, 11130 },
	{ 0x6a65, 0x6a65, PDF_CMAP_SINGLE, 18016 },
	{ 0x6a66, 0x6a66, PDF_CMAP_SINGLE, 11113 },
	{ 0x6a67, 0x6a67, PDF_CMAP_SINGLE, 11115 },
	{ 0x6a68, 0x6a68, PDF_CMAP_SINGLE, 11117 },
	{ 0x6a69, 0x6a69, PDF_CMAP_SINGLE, 11135 },
	{ 0x6a6a, 0x6a6a, PDF_CMAP_SINGLE, 11129 },
	{ 0x6a6b, 0x6a6b, PDF_CMAP_SINGLE, 4817 },
	{ 0x6a6d, 0x6a6d, PDF_CMAP_SINGLE, 11120 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 11134 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 18018 },
	{ 0x6a74, 0x6a74, PDF_CMAP_SINGLE, 18017 },
	{ 0x6a76, 0x6a76, PDF_CMAP_SINGLE, 11121 },
	{ 0x6a7a, 0x6a7a, PDF_CMAP_SINGLE, 15021 },
	{ 0x6a7e, 0x6a7e, PDF_CMAP_SINGLE, 5100 },
	{ 0x6a7f, 0x6a7f, PDF_CMAP_SINGLE, 11652 },
	{ 0x6a80, 0x6a80, PDF_CMAP_SINGLE, 5093 },
	{ 0x6a81, 0x6a81, PDF_CMAP_SINGLE, 11637 },
	{ 0x6a82, 0x6a82, PDF_CMAP_SINGLE, 18019 },
	{ 0x6a83, 0x6a83, PDF_CMAP_SINGLE, 11648 },
	{ 0x6a84, 0x6a84, PDF_CMAP_SINGLE, 5095 },
	{ 0x6a85, 0x6a85, PDF_CMAP_SINGLE, 11655 },
	{ 0x6a87, 0x6a87, PDF_CMAP_SINGLE, 11644 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 11639 },
	{ 0x6a8a, 0x6a8a, PDF_CMAP_SINGLE, 16694 },
	{ 0x6a8c, 0x6a8c, PDF_CMAP_SINGLE, 11656 },
	{ 0x6a8d, 0x6a8d, PDF_CMAP_SINGLE, 11635 },
	{ 0x6a8e, 0x6a8e, PDF_CMAP_SINGLE, 11646 },
	{ 0x6a8f, 0x6a8f, PDF_CMAP_SINGLE, 15387 },
	{ 0x6a90, 0x6a90, PDF_CMAP_SINGLE, 5102 },
	{ 0x6a91, 0x6a91, PDF_CMAP_SINGLE, 11651 },
	{ 0x6a92, 0x6a92, PDF_CMAP_SINGLE, 11657 },
	{ 0x6a93, 0x6a93, PDF_CMAP_SINGLE, 11645 },
	{ 0x6a94, 0x6a94, PDF_CMAP_SINGLE, 5094 },
	{ 0x6a95, 0x6a95, PDF_CMAP_SINGLE, 11647 },
	{ 0x6a96, 0x6a96, PDF_CMAP_SINGLE, 11636 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 5101 },
	{ 0x6a99, 0x6a99, PDF_CMAP_SINGLE, 18021 },
	{ 0x6a9a, 0x6a9a, PDF_CMAP_SINGLE, 11654 },
	{ 0x6a9b, 0x6a9b, PDF_CMAP_SINGLE, 11641 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 5097 },
	{ 0x6a9d, 0x6a9d, PDF_CMAP_SINGLE, 16882 },
	{ 0x6a9e, 0x6a9e, PDF_CMAP_SINGLE, 11643 },
	{ 0x6a9f, 0x6a9f, PDF_CMAP_SINGLE, 11640 },
	{ 0x6aa0, 0x6aa0, PDF_CMAP_SINGLE, 5103 },
	{ 0x6aa1, 0x6aa1, PDF_CMAP_SINGLE, 11642 },
	{ 0x6aa2, 0x6aa2, PDF_CMAP_SINGLE, 5096 },
	{ 0x6aa3, 0x6aa3, PDF_CMAP_SINGLE, 5099 },
	{ 0x6aa4, 0x6aa4, PDF_CMAP_SINGLE, 11650 },
	{ 0x6aa5, 0x6aa5, PDF_CMAP_SINGLE, 11638 },
	{ 0x6aa6, 0x6aa6, PDF_CMAP_SINGLE, 11653 },
	{ 0x6aa7, 0x6aa7, PDF_CMAP_SINGLE, 14307 },
	{ 0x6aa8, 0x6aa8, PDF_CMAP_SINGLE, 11649 },
	{ 0x6aab, 0x6aab, PDF_CMAP_SINGLE, 18023 },
	{ 0x6aac, 0x6aac, PDF_CMAP_SINGLE, 5341 },
	{ 0x6aad, 0x6aad, PDF_CMAP_SINGLE, 12094 },
	{ 0x6aae, 0x6aaf, PDF_CMAP_RANGE, 5346 },
	{ 0x6ab1, 0x6ab1, PDF_CMAP_SINGLE, 15955 },
	{ 0x6ab2, 0x6ab2, PDF_CMAP_SINGLE, 15179 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 5340 },
	{ 0x6ab4, 0x6ab4, PDF_CMAP_SINGLE, 12093 },
	{ 0x6ab5, 0x6ab5, PDF_CMAP_SINGLE, 18024 },
	{ 0x6ab6, 0x6ab7, PDF_CMAP_RANGE, 12090 },
	{ 0x6ab8, 0x6ab8, PDF_CMAP_SINGLE, 5344 },
	{ 0x6ab9, 0x6ab9, PDF_CMAP_SINGLE, 12085 },
	{ 0x6aba, 0x6aba, PDF_CMAP_SINGLE, 12089 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 5343 },
	{ 0x6abd, 0x6abd, PDF_CMAP_SINGLE, 12086 },
	{ 0x6abe, 0x6abe, PDF_CMAP_SINGLE, 15009 },
	{ 0x6ac2, 0x6ac2, PDF_CMAP_SINGLE, 5345 },
	{ 0x6ac3, 0x6ac3, PDF_CMAP_SINGLE, 5342 },
	{ 0x6ac5, 0x6ac5, PDF_CMAP_SINGLE, 12084 },
	{ 0x6ac6, 0x6ac6, PDF_CMAP_SINGLE, 12088 },
	{ 0x6ac7, 0x6ac7, PDF_CMAP_SINGLE, 12092 },
	{ 0x6ac8, 0x6ac8, PDF_CMAP_SINGLE, 16885 },
	{ 0x6ac9, 0x6ac9, PDF_CMAP_SINGLE, 15014 },
	{ 0x6aca, 0x6aca, PDF_CMAP_SINGLE, 16204 },
	{ 0x6acb, 0x6acb, PDF_CMAP_SINGLE, 12447 },
	{ 0x6acc, 0x6acc, PDF_CMAP_SINGLE, 12444 },
	{ 0x6acd, 0x6acd, PDF_CMAP_SINGLE, 12453 },
	{ 0x6acf, 0x6acf, PDF_CMAP_SINGLE, 12452 },
	{ 0x6ad0, 0x6ad0, PDF_CMAP_SINGLE, 12450 },
	{ 0x6ad1, 0x6ad1, PDF_CMAP_SINGLE, 12445 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 5513 },
	{ 0x6ad4, 0x6ad4, PDF_CMAP_SINGLE, 18025 },
	{ 0x6ad8, 0x6ad8, PDF_CMAP_SINGLE, 15530 },
	{ 0x6ad9, 0x6ad9, PDF_CMAP_SINGLE, 12446 },
	{ 0x6ada, 0x6ada, PDF_CMAP_SINGLE, 5512 },
	{ 0x6adb, 0x6adb, PDF_CMAP_SINGLE, 5098 },
	{ 0x6adc, 0x6adc, PDF_CMAP_SINGLE, 12449 },
	{ 0x6add, 0x6add, PDF_CMAP_SINGLE, 5511 },
	{ 0x6ade, 0x6ade, PDF_CMAP_SINGLE, 12454 },
	{ 0x6adf, 0x6adf, PDF_CMAP_SINGLE, 12448 },
	{ 0x6ae0, 0x6ae0, PDF_CMAP_SINGLE, 12443 },
	{ 0x6ae1, 0x6ae1, PDF_CMAP_SINGLE, 12087 },
	{ 0x6ae5, 0x6ae5, PDF_CMAP_SINGLE, 5510 },
	{ 0x6ae7, 0x6ae7, PDF_CMAP_SINGLE, 12442 },
	{ 0x6ae8, 0x6ae8, PDF_CMAP_SINGLE, 12767 },
	{ 0x6aea, 0x6aea, PDF_CMAP_SINGLE, 12766 },
	{ 0x6aeb, 0x6aeb, PDF_CMAP_SINGLE, 12451 },
	{ 0x6aec, 0x6aec, PDF_CMAP_SINGLE, 5659 },
	{ 0x6aee, 0x6aef, PDF_CMAP_RANGE, 12770 },
	{ 0x6af0, 0x6af0, PDF_CMAP_SINGLE, 12765 },
	{ 0x6af1, 0x6af1, PDF_CMAP_SINGLE, 12769 },
	{ 0x6af3, 0x6af3, PDF_CMAP_SINGLE, 12764 },
	{ 0x6af6, 0x6af6, PDF_CMAP_SINGLE, 18026 },
	{ 0x6af8, 0x6af8, PDF_CMAP_SINGLE, 13014 },
	{ 0x6af9, 0x6af9, PDF_CMAP_SINGLE, 12768 },
	{ 0x6afa, 0x6afa, PDF_CMAP_SINGLE, 5752 },
	{ 0x6afb, 0x6afb, PDF_CMAP_SINGLE, 5750 },
	{ 0x6afc, 0x6afc, PDF_CMAP_SINGLE, 13012 },
	{ 0x6b00, 0x6b00, PDF_CMAP_SINGLE, 13015 },
	{ 0x6b02, 0x6b02, PDF_CMAP_SINGLE, 13011 },
	{ 0x6b03, 0x6b03, PDF_CMAP_SINGLE, 13013 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 5751 },
	{ 0x6b05, 0x6b05, PDF_CMAP_SINGLE, 15015 },
	{ 0x6b08, 0x6b09, PDF_CMAP_RANGE, 13190 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 5830 },
	{ 0x6b0b, 0x6b0b, PDF_CMAP_SINGLE, 13189 },
	{ 0x6b0f, 0x6b0f, PDF_CMAP_SINGLE, 13344 },
	{ 0x6b10, 0x6b10, PDF_CMAP_SINGLE, 5885 },
	{ 0x6b11, 0x6b12, PDF_CMAP_RANGE, 13342 },
	{ 0x6b13, 0x6b13, PDF_CMAP_SINGLE, 13447 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 5955 },
	{ 0x6b17, 0x6b17, PDF_CMAP_SINGLE, 13518 },
	{ 0x6b18, 0x6b19, PDF_CMAP_RANGE, 13516 },
	{ 0x6b1a, 0x6b1a, PDF_CMAP_SINGLE, 13519 },
	{ 0x6b1d, 0x6b1d, PDF_CMAP_SINGLE, 16888 },
	{ 0x6b1e, 0x6b1e, PDF_CMAP_SINGLE, 13620 },
	{ 0x6b20, 0x6b20, PDF_CMAP_SINGLE, 734 },
	{ 0x6b21, 0x6b21, PDF_CMAP_SINGLE, 995 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 1562 },
	{ 0x6b25, 0x6b25, PDF_CMAP_SINGLE, 6601 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 6973 },
	{ 0x6b2c, 0x6b2c, PDF_CMAP_SINGLE, 7398 },
	{ 0x6b2d, 0x6b2d, PDF_CMAP_SINGLE, 7400 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 7399 },
	{ 0x6b31, 0x6b31, PDF_CMAP_SINGLE, 7401 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 2791 },
	{ 0x6b33, 0x6b33, PDF_CMAP_SINGLE, 7996 },
	{ 0x6b34, 0x6b34, PDF_CMAP_SINGLE, 7402 },
	{ 0x6b35, 0x6b35, PDF_CMAP_SINGLE, 16891 },
	{ 0x6b36, 0x6b36, PDF_CMAP_SINGLE, 7995 },
	{ 0x6b37, 0x6b38, PDF_CMAP_RANGE, 7997 },
	{ 0x6b39, 0x6b39, PDF_CMAP_SINGLE, 8626 },
	{ 0x6b3a, 0x6b3a, PDF_CMAP_SINGLE, 3228 },
	{ 0x6b3b, 0x6b3b, PDF_CMAP_SINGLE, 8627 },
	{ 0x6b3c, 0x6b3c, PDF_CMAP_SINGLE, 8629 },
	{ 0x6b3d, 0x6b3d, PDF_CMAP_SINGLE, 3229 },
	{ 0x6b3e, 0x6b3e, PDF_CMAP_SINGLE, 3227 },
	{ 0x6b3f, 0x6b3f, PDF_CMAP_SINGLE, 8628 },
	{ 0x6b41, 0x6b41, PDF_CMAP_SINGLE, 9287 },
	{ 0x6b42, 0x6b42, PDF_CMAP_SINGLE, 9285 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 9284 },
	{ 0x6b45, 0x6b45, PDF_CMAP_SINGLE, 9283 },
	{ 0x6b46, 0x6b46, PDF_CMAP_SINGLE, 9282 },
	{ 0x6b47, 0x6b47, PDF_CMAP_SINGLE, 3658 },
	{ 0x6b48, 0x6b48, PDF_CMAP_SINGLE, 9286 },
	{ 0x6b49, 0x6b49, PDF_CMAP_SINGLE, 4097 },
	{ 0x6b4a, 0x6b4a, PDF_CMAP_SINGLE, 9954 },
	{ 0x6b4b, 0x6b4b, PDF_CMAP_SINGLE, 9956 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 4098 },
	{ 0x6b4d, 0x6b4d, PDF_CMAP_SINGLE, 9955 },
	{ 0x6b4e, 0x6b4e, PDF_CMAP_SINGLE, 4485 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 4484 },
	{ 0x6b51, 0x6b51, PDF_CMAP_SINGLE, 10532 },
	{ 0x6b52, 0x6b52, PDF_CMAP_SINGLE, 16238 },
	{ 0x6b54, 0x6b54, PDF_CMAP_SINGLE, 11145 },
	{ 0x6b55, 0x6b55, PDF_CMAP_SINGLE, 11144 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 11146 },
	{ 0x6b57, 0x6b57, PDF_CMAP_SINGLE, 15022 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 4828 },
	{ 0x6b5b, 0x6b5b, PDF_CMAP_SINGLE, 11658 },
	{ 0x6b5c, 0x6b5c, PDF_CMAP_SINGLE, 5104 },
	{ 0x6b5e, 0x6b5e, PDF_CMAP_SINGLE, 12095 },
	{ 0x6b5f, 0x6b5f, PDF_CMAP_SINGLE, 5348 },
	{ 0x6b60, 0x6b60, PDF_CMAP_SINGLE, 12455 },
	{ 0x6b61, 0x6b61, PDF_CMAP_SINGLE, 5831 },
	{ 0x6b62, 0x6b62, PDF_CMAP_SINGLE, 735 },
	{ 0x6b63, 0x6b63, PDF_CMAP_SINGLE, 845 },
	{ 0x6b64, 0x6b64, PDF_CMAP_SINGLE, 996 },
	{ 0x6b65, 0x6b65, PDF_CMAP_SINGLE, 1225 },
	{ 0x6b66, 0x6b67, PDF_CMAP_RANGE, 1563 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 1897 },
	{ 0x6b6d, 0x6b6d, PDF_CMAP_SINGLE, 7403 },
	{ 0x6b6f, 0x6b6f, PDF_CMAP_SINGLE, 17685 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 3659 },
	{ 0x6b74, 0x6b74, PDF_CMAP_SINGLE, 16892 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 10658 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 4829 },
	{ 0x6b78, 0x6b78, PDF_CMAP_SINGLE, 5349 },
	{ 0x6b79, 0x6b79, PDF_CMAP_SINGLE, 736 },
	{ 0x6b7a, 0x6b7a, PDF_CMAP_SINGLE, 17646 },
	{ 0x6b7b, 0x6b7b, PDF_CMAP_SINGLE, 997 },
	{ 0x6b7e, 0x6b7e, PDF_CMAP_SINGLE, 6603 },
	{ 0x6b7f, 0x6b7f, PDF_CMAP_SINGLE, 1565 },
	{ 0x6b80, 0x6b80, PDF_CMAP_SINGLE, 6602 },
	{ 0x6b81, 0x6b81, PDF_CMAP_SINGLE, 18027 },
	{ 0x6b82, 0x6b82, PDF_CMAP_SINGLE, 6974 },
	{ 0x6b83, 0x6b83, PDF_CMAP_SINGLE, 1898 },
	{ 0x6b84, 0x6b84, PDF_CMAP_SINGLE, 6975 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 1899 },
	{ 0x6b88, 0x6b88, PDF_CMAP_SINGLE, 7405 },
	{ 0x6b89, 0x6b89, PDF_CMAP_SINGLE, 2304 },
	{ 0x6b8a, 0x6b8a, PDF_CMAP_SINGLE, 2303 },
	{ 0x6b8c, 0x6b8c, PDF_CMAP_SINGLE, 8003 },
	{ 0x6b8d, 0x6b8e, PDF_CMAP_RANGE, 8001 },
	{ 0x6b8f, 0x6b8f, PDF_CMAP_SINGLE, 8000 },
	{ 0x6b91, 0x6b91, PDF_CMAP_SINGLE, 7999 },
	{ 0x6b94, 0x6b94, PDF_CMAP_SINGLE, 8630 },
	{ 0x6b95, 0x6b95, PDF_CMAP_SINGLE, 8633 },
	{ 0x6b96, 0x6b96, PDF_CMAP_SINGLE, 3231 },
	{ 0x6b97, 0x6b97, PDF_CMAP_SINGLE, 8631 },
	{ 0x6b98, 0x6b98, PDF_CMAP_SINGLE, 3230 },
	{ 0x6b99, 0x6b99, PDF_CMAP_SINGLE, 8632 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 9288 },
	{ 0x6b9e, 0x6ba0, PDF_CMAP_RANGE, 9957 },
	{ 0x6ba2, 0x6ba2, PDF_CMAP_SINGLE, 10535 },
	{ 0x6ba3, 0x6ba3, PDF_CMAP_SINGLE, 10534 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 4486 },
	{ 0x6ba5, 0x6ba5, PDF_CMAP_SINGLE, 10533 },
	{ 0x6ba6, 0x6ba6, PDF_CMAP_SINGLE, 10536 },
	{ 0x6ba7, 0x6ba7, PDF_CMAP_SINGLE, 11147 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 11148 },
	{ 0x6bad, 0x6bad, PDF_CMAP_SINGLE, 11659 },
	{ 0x6bae, 0x6bae, PDF_CMAP_SINGLE, 5105 },
	{ 0x6baf, 0x6baf, PDF_CMAP_SINGLE, 5350 },
	{ 0x6bb0, 0x6bb0, PDF_CMAP_SINGLE, 12456 },
	{ 0x6bb2, 0x6bb2, PDF_CMAP_SINGLE, 5753 },
	{ 0x6bb3, 0x6bb3, PDF_CMAP_SINGLE, 6026 },
	{ 0x6bb5, 0x6bb5, PDF_CMAP_SINGLE, 1900 },
	{ 0x6bb6, 0x6bb6, PDF_CMAP_SINGLE, 6976 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 2305 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 2792 },
	{ 0x6bbc, 0x6bbc, PDF_CMAP_SINGLE, 3232 },
	{ 0x6bbd, 0x6bbd, PDF_CMAP_SINGLE, 8634 },
	{ 0x6bbf, 0x6bbf, PDF_CMAP_SINGLE, 3661 },
	{ 0x6bc0, 0x6bc0, PDF_CMAP_SINGLE, 3660 },
	{ 0x6bc1, 0x6bc1, PDF_CMAP_SINGLE, 18028 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 9960 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_RANGE, 4487 },
	{ 0x6bc7, 0x6bc7, PDF_CMAP_SINGLE, 11151 },
	{ 0x6bc8, 0x6bc8, PDF_CMAP_SINGLE, 11150 },
	{ 0x6bc9, 0x6bc9, PDF_CMAP_SINGLE, 12096 },
	{ 0x6bca, 0x6bca, PDF_CMAP_SINGLE, 13345 },
	{ 0x6bcb, 0x6bcb, PDF_CMAP_SINGLE, 737 },
	{ 0x6bcc, 0x6bcc, PDF_CMAP_SINGLE, 6027 },
	{ 0x6bcd, 0x6bcd, PDF_CMAP_SINGLE, 846 },
	{ 0x6bcf, 0x6bcf, PDF_CMAP_SINGLE, 1226 },
	{ 0x6bd0, 0x6bd0, PDF_CMAP_SINGLE, 6317 },
	{ 0x6bd2, 0x6bd2, PDF_CMAP_SINGLE, 1901 },
	{ 0x6bd3, 0x6bd3, PDF_CMAP_SINGLE, 3662 },
	{ 0x6bd4, 0x6bd4, PDF_CMAP_SINGLE, 738 },
	{ 0x6bd6, 0x6bd6, PDF_CMAP_SINGLE, 6977 },
	{ 0x6bd7, 0x6bd7, PDF_CMAP_SINGLE, 1902 },
	{ 0x6bd8, 0x6bd8, PDF_CMAP_SINGLE, 6978 },
	{ 0x6bda, 0x6bda, PDF_CMAP_SINGLE, 5106 },
	{ 0x6bdb, 0x6bdb, PDF_CMAP_SINGLE, 739 },
	{ 0x6bdc, 0x6bdc, PDF_CMAP_SINGLE, 16903 },
	{ 0x6bde, 0x6bde, PDF_CMAP_SINGLE, 6604 },
	{ 0x6be0, 0x6be0, PDF_CMAP_SINGLE, 6979 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 16901 },
	{ 0x6be2, 0x6be2, PDF_CMAP_SINGLE, 7410 },
	{ 0x6be3, 0x6be3, PDF_CMAP_SINGLE, 7409 },
	{ 0x6be4, 0x6be4, PDF_CMAP_SINGLE, 7407 },
	{ 0x6be6, 0x6be6, PDF_CMAP_SINGLE, 7406 },
	{ 0x6be7, 0x6be7, PDF_CMAP_SINGLE, 7411 },
	{ 0x6be8, 0x6be8, PDF_CMAP_SINGLE, 7408 },
	{ 0x6bea, 0x6bea, PDF_CMAP_SINGLE, 18029 },
	{ 0x6beb, 0x6bec, PDF_CMAP_RANGE, 2793 },
	{ 0x6bef, 0x6bef, PDF_CMAP_SINGLE, 3233 },
	{ 0x6bf0, 0x6bf0, PDF_CMAP_SINGLE, 8635 },
	{ 0x6bf2, 0x6bf3, PDF_CMAP_RANGE, 8636 },
	{ 0x6bf7, 0x6bf8, PDF_CMAP_RANGE, 9292 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 9291 },
	{ 0x6bfa, 0x6bfa, PDF_CMAP_SINGLE, 16249 },
	{ 0x6bfb, 0x6bfc, PDF_CMAP_RANGE, 9289 },
	{ 0x6bfd, 0x6bfd, PDF_CMAP_SINGLE, 3663 },
	{ 0x6bfe, 0x6bfe, PDF_CMAP_SINGLE, 9962 },
	{ 0x6bff, 0x6bff, PDF_CMAP_SINGLE, 10539 },
	{ 0x6c00, 0x6c00, PDF_CMAP_SINGLE, 10538 },
	{ 0x6c01, 0x6c01, PDF_CMAP_SINGLE, 10537 },
	{ 0x6c02, 0x6c02, PDF_CMAP_SINGLE, 10540 },
	{ 0x6c03, 0x6c03, PDF_CMAP_SINGLE, 11153 },
	{ 0x6c04, 0x6c04, PDF_CMAP_SINGLE, 11152 },
	{ 0x6c05, 0x6c05, PDF_CMAP_SINGLE, 4830 },
	{ 0x6c06, 0x6c06, PDF_CMAP_SINGLE, 11154 },
	{ 0x6c08, 0x6c08, PDF_CMAP_SINGLE, 5107 },
	{ 0x6c09, 0x6c09, PDF_CMAP_SINGLE, 11660 },
	{ 0x6c0b, 0x6c0b, PDF_CMAP_SINGLE, 12097 },
	{ 0x6c0c, 0x6c0c, PDF_CMAP_SINGLE, 12457 },
	{ 0x6c0d, 0x6c0d, PDF_CMAP_SINGLE, 13192 },
	{ 0x6c0f, 0x6c0f, PDF_CMAP_SINGLE, 740 },
	{ 0x6c10, 0x6c10, PDF_CMAP_SINGLE, 848 },
	{ 0x6c11, 0x6c11, PDF_CMAP_SINGLE, 847 },
	{ 0x6c13, 0x6c13, PDF_CMAP_SINGLE, 1566 },
	{ 0x6c14, 0x6c14, PDF_CMAP_SINGLE, 6028 },
	{ 0x6c15, 0x6c15, PDF_CMAP_SINGLE, 6058 },
	{ 0x6c16, 0x6c16, PDF_CMAP_SINGLE, 998 },
	{ 0x6c18, 0x6c18, PDF_CMAP_SINGLE, 6145 },
	{ 0x6c19, 0x6c1a, PDF_CMAP_RANGE, 6318 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 1567 },
	{ 0x6c1c, 0x6c1c, PDF_CMAP_SINGLE, 14507 },
	{ 0x6c1d, 0x6c1d, PDF_CMAP_SINGLE, 6605 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 1903 },
	{ 0x6c20, 0x6c21, PDF_CMAP_RANGE, 6980 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 2306 },
	{ 0x6c24, 0x6c24, PDF_CMAP_SINGLE, 2310 },
	{ 0x6c25, 0x6c25, PDF_CMAP_SINGLE, 7412 },
	{ 0x6c26, 0x6c26, PDF_CMAP_SINGLE, 2309 },
	{ 0x6c27, 0x6c28, PDF_CMAP_RANGE, 2307 },
	{ 0x6c2a, 0x6c2a, PDF_CMAP_SINGLE, 8004 },
	{ 0x6c2b, 0x6c2b, PDF_CMAP_SINGLE, 2795 },
	{ 0x6c2c, 0x6c2c, PDF_CMAP_SINGLE, 3236 },
	{ 0x6c2e, 0x6c2f, PDF_CMAP_RANGE, 3234 },
	{ 0x6c30, 0x6c30, PDF_CMAP_SINGLE, 8638 },
	{ 0x6c31, 0x6c31, PDF_CMAP_SINGLE, 16227 },
	{ 0x6c33, 0x6c33, PDF_CMAP_SINGLE, 4099 },
	{ 0x6c34, 0x6c34, PDF_CMAP_SINGLE, 741 },
	{ 0x6c35, 0x6c35, PDF_CMAP_SINGLE, 17647 },
	{ 0x6c36, 0x6c36, PDF_CMAP_SINGLE, 6059 },
	{ 0x6c37, 0x6c37, PDF_CMAP_SINGLE, 16904 },
	{ 0x6c38, 0x6c38, PDF_CMAP_SINGLE, 849 },
	{ 0x6c39, 0x6c39, PDF_CMAP_SINGLE, 15877 },
	{ 0x6c3a, 0x6c3a, PDF_CMAP_SINGLE, 17648 },
	{ 0x6c3b, 0x6c3b, PDF_CMAP_SINGLE, 6062 },
	{ 0x6c3d, 0x6c3d, PDF_CMAP_SINGLE, 17281 },
	{ 0x6c3e, 0x6c3e, PDF_CMAP_SINGLE, 852 },
	{ 0x6c3f, 0x6c3f, PDF_CMAP_SINGLE, 6061 },
	{ 0x6c40, 0x6c40, PDF_CMAP_SINGLE, 851 },
	{ 0x6c41, 0x6c41, PDF_CMAP_SINGLE, 850 },
	{ 0x6c42, 0x6c42, PDF_CMAP_SINGLE, 1227 },
	{ 0x6c43, 0x6c43, PDF_CMAP_SINGLE, 6060 },
	{ 0x6c46, 0x6c46, PDF_CMAP_SINGLE, 6146 },
	{ 0x6c49, 0x6c49, PDF_CMAP_SINGLE, 16525 },
	{ 0x6c4a, 0x6c4a, PDF_CMAP_SINGLE, 6150 },
	{ 0x6c4b, 0x6c4c, PDF_CMAP_RANGE, 6152 },
	{ 0x6c4d, 0x6c4e, PDF_CMAP_RANGE, 1008 },
	{ 0x6c4f, 0x6c4f, PDF_CMAP_SINGLE, 6149 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 1004 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 6147 },
	{ 0x6c54, 0x6c54, PDF_CMAP_SINGLE, 6151 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 1005 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 1000 },
	{ 0x6c58, 0x6c58, PDF_CMAP_SINGLE, 15290 },
	{ 0x6c59, 0x6c59, PDF_CMAP_SINGLE, 1001 },
	{ 0x6c5a, 0x6c5a, PDF_CMAP_SINGLE, 16908 },
	{ 0x6c5b, 0x6c5b, PDF_CMAP_SINGLE, 1007 },
	{ 0x6c5c, 0x6c5c, PDF_CMAP_SINGLE, 6148 },
	{ 0x6c5d, 0x6c5d, PDF_CMAP_SINGLE, 999 },
	{ 0x6c5e, 0x6c5e, PDF_CMAP_SINGLE, 1228 },
	{ 0x6c5f, 0x6c60, PDF_CMAP_RANGE, 1002 },
	{ 0x6c61, 0x6c61, PDF_CMAP_SINGLE, 1006 },
	{ 0x6c65, 0x6c65, PDF_CMAP_SINGLE, 6336 },
	{ 0x6c66, 0x6c66, PDF_CMAP_SINGLE, 6334 },
	{ 0x6c67, 0x6c67, PDF_CMAP_SINGLE, 6321 },
	{ 0x6c68, 0x6c68, PDF_CMAP_SINGLE, 1240 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 6328 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 1235 },
	{ 0x6c6b, 0x6c6b, PDF_CMAP_SINGLE, 6322 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 6330 },
	{ 0x6c6e, 0x6c6e, PDF_CMAP_SINGLE, 16586 },
	{ 0x6c6f, 0x6c6f, PDF_CMAP_SINGLE, 6327 },
	{ 0x6c70, 0x6c70, PDF_CMAP_SINGLE, 1238 },
	{ 0x6c71, 0x6c71, PDF_CMAP_SINGLE, 6326 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 1245 },
	{ 0x6c73, 0x6c73, PDF_CMAP_SINGLE, 6335 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 1247 },
	{ 0x6c75, 0x6c75, PDF_CMAP_SINGLE, 18030 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 1249 },
	{ 0x6c78, 0x6c78, PDF_CMAP_SINGLE, 6320 },
	{ 0x6c79, 0x6c79, PDF_CMAP_SINGLE, 16910 },
	{ 0x6c7a, 0x6c7a, PDF_CMAP_SINGLE, 1236 },
	{ 0x6c7b, 0x6c7b, PDF_CMAP_SINGLE, 6337 },
	{ 0x6c7d, 0x6c7d, PDF_CMAP_SINGLE, 1243 },
	{ 0x6c7e, 0x6c7e, PDF_CMAP_SINGLE, 1246 },
	{ 0x6c7f, 0x6c7f, PDF_CMAP_SINGLE, 14246 },
	{ 0x6c80, 0x6c80, PDF_CMAP_SINGLE, 6628 },
	{ 0x6c81, 0x6c81, PDF_CMAP_SINGLE, 1230 },
	{ 0x6c82, 0x6c82, PDF_CMAP_SINGLE, 1253 },
	{ 0x6c83, 0x6c83, PDF_CMAP_SINGLE, 1244 },
	{ 0x6c84, 0x6c84, PDF_CMAP_SINGLE, 6323 },
	{ 0x6c85, 0x6c85, PDF_CMAP_SINGLE, 1233 },
	{ 0x6c86, 0x6c86, PDF_CMAP_SINGLE, 1248 },
	{ 0x6c87, 0x6c87, PDF_CMAP_SINGLE, 6331 },
	{ 0x6c88, 0x6c89, PDF_CMAP_RANGE, 1231 },
	{ 0x6c8a, 0x6c8a, PDF_CMAP_SINGLE, 6626 },
	{ 0x6c8b, 0x6c8b, PDF_CMAP_SINGLE, 6324 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 1239 },
	{ 0x6c8d, 0x6c8d, PDF_CMAP_SINGLE, 1250 },
	{ 0x6c8e, 0x6c8e, PDF_CMAP_SINGLE, 6338 },
	{ 0x6c8f, 0x6c8f, PDF_CMAP_SINGLE, 6325 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 1237 },
	{ 0x6c92, 0x6c92, PDF_CMAP_SINGLE, 1242 },
	{ 0x6c93, 0x6c93, PDF_CMAP_SINGLE, 6606 },
	{ 0x6c94, 0x6c94, PDF_CMAP_SINGLE, 1251 },
	{ 0x6c95, 0x6c95, PDF_CMAP_SINGLE, 6332 },
	{ 0x6c96, 0x6c96, PDF_CMAP_SINGLE, 1241 },
	{ 0x6c98, 0x6c98, PDF_CMAP_SINGLE, 1252 },
	{ 0x6c99, 0x6c99, PDF_CMAP_SINGLE, 1229 },
	{ 0x6c9a, 0x6c9a, PDF_CMAP_SINGLE, 6329 },
	{ 0x6c9b, 0x6c9b, PDF_CMAP_SINGLE, 1234 },
	{ 0x6c9c, 0x6c9c, PDF_CMAP_SINGLE, 6333 },
	{ 0x6c9d, 0x6c9d, PDF_CMAP_SINGLE, 6627 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 14473 },
	{ 0x6ca2, 0x6ca2, PDF_CMAP_SINGLE, 15715 },
	{ 0x6caa, 0x6caa, PDF_CMAP_SINGLE, 18031 },
	{ 0x6cab, 0x6cab, PDF_CMAP_SINGLE, 1579 },
	{ 0x6cac, 0x6cac, PDF_CMAP_SINGLE, 1595 },
	{ 0x6cad, 0x6cad, PDF_CMAP_SINGLE, 6613 },
	{ 0x6cae, 0x6cae, PDF_CMAP_SINGLE, 1586 },
	{ 0x6caf, 0x6caf, PDF_CMAP_SINGLE, 15606 },
	{ 0x6cb0, 0x6cb0, PDF_CMAP_SINGLE, 6634 },
	{ 0x6cb1, 0x6cb1, PDF_CMAP_SINGLE, 1571 },
	{ 0x6cb2, 0x6cb2, PDF_CMAP_SINGLE, 16076 },
	{ 0x6cb3, 0x6cb3, PDF_CMAP_SINGLE, 1574 },
	{ 0x6cb4, 0x6cb4, PDF_CMAP_SINGLE, 6625 },
	{ 0x6cb6, 0x6cb6, PDF_CMAP_SINGLE, 6611 },
	{ 0x6cb7, 0x6cb7, PDF_CMAP_SINGLE, 6615 },
	{ 0x6cb8, 0x6cb8, PDF_CMAP_SINGLE, 1582 },
	{ 0x6cb9, 0x6cb9, PDF_CMAP_SINGLE, 1584 },
	{ 0x6cba, 0x6cba, PDF_CMAP_SINGLE, 6618 },
	{ 0x6cbb, 0x6cbb, PDF_CMAP_SINGLE, 1591 },
	{ 0x6cbc, 0x6cbc, PDF_CMAP_SINGLE, 1577 },
	{ 0x6cbd, 0x6cbe, PDF_CMAP_RANGE, 1575 },
	{ 0x6cbf, 0x6cbf, PDF_CMAP_SINGLE, 1590 },
	{ 0x6cc0, 0x6cc0, PDF_CMAP_SINGLE, 6630 },
	{ 0x6cc1, 0x6cc1, PDF_CMAP_SINGLE, 1585 },
	{ 0x6cc2, 0x6cc2, PDF_CMAP_SINGLE, 6617 },
	{ 0x6cc3, 0x6cc3, PDF_CMAP_SINGLE, 6619 },
	{ 0x6cc4, 0x6cc4, PDF_CMAP_SINGLE, 1583 },
	{ 0x6cc5, 0x6cc5, PDF_CMAP_SINGLE, 1588 },
	{ 0x6cc6, 0x6cc6, PDF_CMAP_SINGLE, 6620 },
	{ 0x6cc7, 0x6cc7, PDF_CMAP_SINGLE, 6633 },
	{ 0x6cc9, 0x6cc9, PDF_CMAP_SINGLE, 1904 },
	{ 0x6cca, 0x6cca, PDF_CMAP_SINGLE, 1594 },
	{ 0x6ccb, 0x6ccb, PDF_CMAP_SINGLE, 16496 },
	{ 0x6ccc, 0x6ccc, PDF_CMAP_SINGLE, 1572 },
	{ 0x6ccd, 0x6ccd, PDF_CMAP_SINGLE, 6632 },
	{ 0x6cce, 0x6cce, PDF_CMAP_SINGLE, 16273 },
	{ 0x6ccf, 0x6ccf, PDF_CMAP_SINGLE, 6636 },
	{ 0x6cd0, 0x6cd0, PDF_CMAP_SINGLE, 6616 },
	{ 0x6cd1, 0x6cd1, PDF_CMAP_SINGLE, 6638 },
	{ 0x6cd2, 0x6cd2, PDF_CMAP_SINGLE, 6623 },
	{ 0x6cd3, 0x6cd3, PDF_CMAP_SINGLE, 1581 },
	{ 0x6cd4, 0x6cd4, PDF_CMAP_SINGLE, 6612 },
	{ 0x6cd5, 0x6cd5, PDF_CMAP_SINGLE, 1580 },
	{ 0x6cd6, 0x6cd6, PDF_CMAP_SINGLE, 1598 },
	{ 0x6cd7, 0x6cd7, PDF_CMAP_SINGLE, 1587 },
	{ 0x6cd9, 0x6cd9, PDF_CMAP_SINGLE, 6610 },
	{ 0x6cda, 0x6cda, PDF_CMAP_SINGLE, 6990 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 1593 },
	{ 0x6cdc, 0x6cdc, PDF_CMAP_SINGLE, 1597 },
	{ 0x6cdd, 0x6cdd, PDF_CMAP_SINGLE, 6624 },
	{ 0x6cde, 0x6cde, PDF_CMAP_SINGLE, 6629 },
	{ 0x6cdf, 0x6cdf, PDF_CMAP_SINGLE, 15703 },
	{ 0x6ce0, 0x6ce0, PDF_CMAP_SINGLE, 1599 },
	{ 0x6ce1, 0x6ce1, PDF_CMAP_SINGLE, 1592 },
	{ 0x6ce2, 0x6ce2, PDF_CMAP_SINGLE, 1578 },
	{ 0x6ce3, 0x6ce3, PDF_CMAP_SINGLE, 1568 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 1573 },
	{ 0x6ce7, 0x6ce7, PDF_CMAP_SINGLE, 6614 },
	{ 0x6ce8, 0x6ce8, PDF_CMAP_SINGLE, 1569 },
	{ 0x6ce9, 0x6ce9, PDF_CMAP_SINGLE, 6637 },
	{ 0x6cea, 0x6cea, PDF_CMAP_SINGLE, 16269 },
	{ 0x6ceb, 0x6ceb, PDF_CMAP_SINGLE, 6608 },
	{ 0x6cec, 0x6cec, PDF_CMAP_SINGLE, 6607 },
	{ 0x6ced, 0x6ced, PDF_CMAP_SINGLE, 6621 },
	{ 0x6cee, 0x6cee, PDF_CMAP_SINGLE, 6609 },
	{ 0x6cef, 0x6cef, PDF_CMAP_SINGLE, 1596 },
	{ 0x6cf0, 0x6cf0, PDF_CMAP_SINGLE, 2311 },
	{ 0x6cf1, 0x6cf1, PDF_CMAP_SINGLE, 1589 },
	{ 0x6cf2, 0x6cf2, PDF_CMAP_SINGLE, 6622 },
	{ 0x6cf3, 0x6cf3, PDF_CMAP_SINGLE, 1570 },
	{ 0x6cf5, 0x6cf5, PDF_CMAP_SINGLE, 1919 },
	{ 0x6cf9, 0x6cf9, PDF_CMAP_SINGLE, 6635 },
	{ 0x6d00, 0x6d00, PDF_CMAP_SINGLE, 6997 },
	{ 0x6d01, 0x6d01, PDF_CMAP_SINGLE, 7000 },
	{ 0x6d02, 0x6d02, PDF_CMAP_SINGLE, 18033 },
	{ 0x6d03, 0x6d03, PDF_CMAP_SINGLE, 7003 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 6992 },
	{ 0x6d05, 0x6d05, PDF_CMAP_SINGLE, 14873 },
	{ 0x6d06, 0x6d06, PDF_CMAP_SINGLE, 18034 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 7006 },
	{ 0x6d08, 0x6d08, PDF_CMAP_SINGLE, 7009 },
	{ 0x6d09, 0x6d09, PDF_CMAP_SINGLE, 7011 },
	{ 0x6d0a, 0x6d0a, PDF_CMAP_SINGLE, 6989 },
	{ 0x6d0b, 0x6d0b, PDF_CMAP_SINGLE, 1905 },
	{ 0x6d0c, 0x6d0c, PDF_CMAP_SINGLE, 1910 },
	{ 0x6d0d, 0x6d0d, PDF_CMAP_SINGLE, 7417 },
	{ 0x6d0e, 0x6d0e, PDF_CMAP_SINGLE, 1926 },
	{ 0x6d0f, 0x6d0f, PDF_CMAP_SINGLE, 7004 },
	{ 0x6d10, 0x6d10, PDF_CMAP_SINGLE, 7012 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 6996 },
	{ 0x6d12, 0x6d12, PDF_CMAP_SINGLE, 6988 },
	{ 0x6d16, 0x6d16, PDF_CMAP_SINGLE, 7444 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 1913 },
	{ 0x6d18, 0x6d18, PDF_CMAP_SINGLE, 7001 },
	{ 0x6d19, 0x6d19, PDF_CMAP_SINGLE, 6993 },
	{ 0x6d1a, 0x6d1a, PDF_CMAP_SINGLE, 6995 },
	{ 0x6d1b, 0x6d1b, PDF_CMAP_SINGLE, 1918 },
	{ 0x6d1d, 0x6d1d, PDF_CMAP_SINGLE, 6998 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 1912 },
	{ 0x6d1f, 0x6d1f, PDF_CMAP_SINGLE, 6985 },
	{ 0x6d20, 0x6d20, PDF_CMAP_SINGLE, 7007 },
	{ 0x6d22, 0x6d22, PDF_CMAP_SINGLE, 7010 },
	{ 0x6d24, 0x6d24, PDF_CMAP_SINGLE, 15043 },
	{ 0x6d25, 0x6d25, PDF_CMAP_SINGLE, 1909 },
	{ 0x6d26, 0x6d26, PDF_CMAP_SINGLE, 18035 },
	{ 0x6d27, 0x6d27, PDF_CMAP_SINGLE, 1921 },
	{ 0x6d28, 0x6d28, PDF_CMAP_SINGLE, 6982 },
	{ 0x6d29, 0x6d29, PDF_CMAP_SINGLE, 1923 },
	{ 0x6d2a, 0x6d2a, PDF_CMAP_SINGLE, 1907 },
	{ 0x6d2b, 0x6d2b, PDF_CMAP_SINGLE, 1927 },
	{ 0x6d2c, 0x6d2c, PDF_CMAP_SINGLE, 7008 },
	{ 0x6d2d, 0x6d2d, PDF_CMAP_SINGLE, 6984 },
	{ 0x6d2e, 0x6d2e, PDF_CMAP_SINGLE, 1924 },
	{ 0x6d2f, 0x6d2f, PDF_CMAP_SINGLE, 7438 },
	{ 0x6d30, 0x6d30, PDF_CMAP_SINGLE, 6631 },
	{ 0x6d31, 0x6d31, PDF_CMAP_SINGLE, 1911 },
	{ 0x6d32, 0x6d32, PDF_CMAP_SINGLE, 1906 },
	{ 0x6d33, 0x6d33, PDF_CMAP_SINGLE, 6991 },
	{ 0x6d34, 0x6d34, PDF_CMAP_SINGLE, 6983 },
	{ 0x6d35, 0x6d35, PDF_CMAP_SINGLE, 1925 },
	{ 0x6d36, 0x6d36, PDF_CMAP_SINGLE, 1917 },
	{ 0x6d37, 0x6d37, PDF_CMAP_SINGLE, 7002 },
	{ 0x6d38, 0x6d38, PDF_CMAP_SINGLE, 1922 },
	{ 0x6d39, 0x6d39, PDF_CMAP_SINGLE, 1920 },
	{ 0x6d3a, 0x6d3a, PDF_CMAP_SINGLE, 6994 },
	{ 0x6d3b, 0x6d3b, PDF_CMAP_SINGLE, 1914 },
	{ 0x6d3c, 0x6d3c, PDF_CMAP_SINGLE, 6986 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_RANGE, 1915 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 6987 },
	{ 0x6d40, 0x6d40, PDF_CMAP_SINGLE, 7005 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 1908 },
	{ 0x6d42, 0x6d42, PDF_CMAP_SINGLE, 6999 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 17752 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 15603 },
	{ 0x6d58, 0x6d58, PDF_CMAP_SINGLE, 7420 },
	{ 0x6d59, 0x6d59, PDF_CMAP_SINGLE, 2319 },
	{ 0x6d5a, 0x6d5a, PDF_CMAP_SINGLE, 2324 },
	{ 0x6d5b, 0x6d5b, PDF_CMAP_SINGLE, 16540 },
	{ 0x6d5c, 0x6d5c, PDF_CMAP_SINGLE, 16930 },
	{ 0x6d5e, 0x6d5e, PDF_CMAP_SINGLE, 7429 },
	{ 0x6d5f, 0x6d5f, PDF_CMAP_SINGLE, 7435 },
	{ 0x6d60, 0x6d60, PDF_CMAP_SINGLE, 7431 },
	{ 0x6d61, 0x6d61, PDF_CMAP_SINGLE, 7418 },
	{ 0x6d62, 0x6d62, PDF_CMAP_SINGLE, 7421 },
	{ 0x6d63, 0x6d64, PDF_CMAP_RANGE, 7414 },
	{ 0x6d65, 0x6d65, PDF_CMAP_SINGLE, 2331 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 2316 },
	{ 0x6d67, 0x6d67, PDF_CMAP_SINGLE, 7430 },
	{ 0x6d68, 0x6d68, PDF_CMAP_SINGLE, 7439 },
	{ 0x6d69, 0x6d69, PDF_CMAP_SINGLE, 2326 },
	{ 0x6d6a, 0x6d6a, PDF_CMAP_SINGLE, 2312 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 2321 },
	{ 0x6d6d, 0x6d6d, PDF_CMAP_SINGLE, 7422 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 2323 },
	{ 0x6d6f, 0x6d6f, PDF_CMAP_SINGLE, 7423 },
	{ 0x6d70, 0x6d70, PDF_CMAP_SINGLE, 7433 },
	{ 0x6d71, 0x6d71, PDF_CMAP_SINGLE, 16629 },
	{ 0x6d72, 0x6d72, PDF_CMAP_SINGLE, 14807 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 2325 },
	{ 0x6d75, 0x6d75, PDF_CMAP_SINGLE, 7448 },
	{ 0x6d76, 0x6d76, PDF_CMAP_SINGLE, 7416 },
	{ 0x6d77, 0x6d77, PDF_CMAP_SINGLE, 2318 },
	{ 0x6d78, 0x6d78, PDF_CMAP_SINGLE, 2317 },
	{ 0x6d79, 0x6d79, PDF_CMAP_SINGLE, 2329 },
	{ 0x6d7a, 0x6d7a, PDF_CMAP_SINGLE, 7413 },
	{ 0x6d7b, 0x6d7b, PDF_CMAP_SINGLE, 7446 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 7434 },
	{ 0x6d7d, 0x6d7d, PDF_CMAP_SINGLE, 7447 },
	{ 0x6d7e, 0x6d7e, PDF_CMAP_SINGLE, 7441 },
	{ 0x6d7f, 0x6d7f, PDF_CMAP_SINGLE, 7427 },
	{ 0x6d80, 0x6d80, PDF_CMAP_SINGLE, 7442 },
	{ 0x6d81, 0x6d81, PDF_CMAP_SINGLE, 18036 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 7436 },
	{ 0x6d83, 0x6d83, PDF_CMAP_SINGLE, 7445 },
	{ 0x6d84, 0x6d84, PDF_CMAP_SINGLE, 7443 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 2330 },
	{ 0x6d86, 0x6d86, PDF_CMAP_SINGLE, 7428 },
	{ 0x6d87, 0x6d87, PDF_CMAP_SINGLE, 2315 },
	{ 0x6d88, 0x6d88, PDF_CMAP_SINGLE, 2314 },
	{ 0x6d89, 0x6d89, PDF_CMAP_SINGLE, 2322 },
	{ 0x6d8a, 0x6d8a, PDF_CMAP_SINGLE, 2328 },
	{ 0x6d8b, 0x6d8b, PDF_CMAP_SINGLE, 7440 },
	{ 0x6d8c, 0x6d8c, PDF_CMAP_SINGLE, 2327 },
	{ 0x6d8d, 0x6d8d, PDF_CMAP_SINGLE, 7425 },
	{ 0x6d8e, 0x6d8e, PDF_CMAP_SINGLE, 2796 },
	{ 0x6d8f, 0x6d8f, PDF_CMAP_SINGLE, 15848 },
	{ 0x6d90, 0x6d90, PDF_CMAP_SINGLE, 7449 },
	{ 0x6d91, 0x6d91, PDF_CMAP_SINGLE, 7424 },
	{ 0x6d92, 0x6d92, PDF_CMAP_SINGLE, 7419 },
	{ 0x6d93, 0x6d93, PDF_CMAP_SINGLE, 2320 },
	{ 0x6d94, 0x6d94, PDF_CMAP_SINGLE, 2332 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 2313 },
	{ 0x6d96, 0x6d96, PDF_CMAP_SINGLE, 16929 },
	{ 0x6d97, 0x6d97, PDF_CMAP_SINGLE, 7432 },
	{ 0x6d98, 0x6d98, PDF_CMAP_SINGLE, 7437 },
	{ 0x6da4, 0x6da4, PDF_CMAP_SINGLE, 18038 },
	{ 0x6da5, 0x6da5, PDF_CMAP_SINGLE, 15033 },
	{ 0x6daa, 0x6daa, PDF_CMAP_SINGLE, 2830 },
	{ 0x6dab, 0x6dab, PDF_CMAP_SINGLE, 8006 },
	{ 0x6dac, 0x6dac, PDF_CMAP_SINGLE, 8010 },
	{ 0x6dae, 0x6dae, PDF_CMAP_SINGLE, 2811 },
	{ 0x6daf, 0x6daf, PDF_CMAP_SINGLE, 2809 },
	{ 0x6db1, 0x6db1, PDF_CMAP_SINGLE, 18039 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 2800 },
	{ 0x6db3, 0x6db3, PDF_CMAP_SINGLE, 8008 },
	{ 0x6db4, 0x6db4, PDF_CMAP_SINGLE, 8007 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 2820 },
	{ 0x6db7, 0x6db7, PDF_CMAP_SINGLE, 8013 },
	{ 0x6db8, 0x6db8, PDF_CMAP_SINGLE, 2814 },
	{ 0x6db9, 0x6db9, PDF_CMAP_SINGLE, 16379 },
	{ 0x6dba, 0x6dba, PDF_CMAP_SINGLE, 8031 },
	{ 0x6dbb, 0x6dbb, PDF_CMAP_SINGLE, 8043 },
	{ 0x6dbc, 0x6dbc, PDF_CMAP_SINGLE, 2797 },
	{ 0x6dbd, 0x6dbd, PDF_CMAP_SINGLE, 8028 },
	{ 0x6dbe, 0x6dbe, PDF_CMAP_SINGLE, 8021 },
	{ 0x6dbf, 0x6dbf, PDF_CMAP_SINGLE, 2832 },
	{ 0x6dc0, 0x6dc0, PDF_CMAP_SINGLE, 8005 },
	{ 0x6dc2, 0x6dc2, PDF_CMAP_SINGLE, 8033 },
	{ 0x6dc4, 0x6dc4, PDF_CMAP_SINGLE, 2829 },
	{ 0x6dc5, 0x6dc5, PDF_CMAP_SINGLE, 2817 },
	{ 0x6dc6, 0x6dc6, PDF_CMAP_SINGLE, 2828 },
	{ 0x6dc7, 0x6dc7, PDF_CMAP_SINGLE, 2807 },
	{ 0x6dc8, 0x6dc8, PDF_CMAP_SINGLE, 8017 },
	{ 0x6dc9, 0x6dc9, PDF_CMAP_SINGLE, 8035 },
	{ 0x6dca, 0x6dca, PDF_CMAP_SINGLE, 8027 },
	{ 0x6dcb, 0x6dcb, PDF_CMAP_SINGLE, 2808 },
	{ 0x6dcc, 0x6dcc, PDF_CMAP_SINGLE, 2802 },
	{ 0x6dcd, 0x6dcd, PDF_CMAP_SINGLE, 8041 },
	{ 0x6dcf, 0x6dcf, PDF_CMAP_SINGLE, 8034 },
	{ 0x6dd0, 0x6dd0, PDF_CMAP_SINGLE, 8036 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 2810 },
	{ 0x6dd2, 0x6dd2, PDF_CMAP_SINGLE, 2818 },
	{ 0x6dd3, 0x6dd3, PDF_CMAP_SINGLE, 8038 },
	{ 0x6dd4, 0x6dd4, PDF_CMAP_SINGLE, 8015 },
	{ 0x6dd5, 0x6dd5, PDF_CMAP_SINGLE, 8032 },
	{ 0x6dd6, 0x6dd6, PDF_CMAP_SINGLE, 8020 },
	{ 0x6dd7, 0x6dd7, PDF_CMAP_SINGLE, 8040 },
	{ 0x6dd8, 0x6dd8, PDF_CMAP_SINGLE, 2823 },
	{ 0x6dd9, 0x6dd9, PDF_CMAP_SINGLE, 2799 },
	{ 0x6dda, 0x6dda, PDF_CMAP_SINGLE, 2821 },
	{ 0x6ddb, 0x6ddb, PDF_CMAP_SINGLE, 8025 },
	{ 0x6ddc, 0x6ddd, PDF_CMAP_RANGE, 8023 },
	{ 0x6dde, 0x6dde, PDF_CMAP_SINGLE, 2812 },
	{ 0x6ddf, 0x6ddf, PDF_CMAP_SINGLE, 8019 },
	{ 0x6de0, 0x6de0, PDF_CMAP_SINGLE, 8018 },
	{ 0x6de1, 0x6de1, PDF_CMAP_SINGLE, 2801 },
	{ 0x6de2, 0x6de2, PDF_CMAP_SINGLE, 8012 },
	{ 0x6de3, 0x6de3, PDF_CMAP_SINGLE, 8042 },
	{ 0x6de4, 0x6de4, PDF_CMAP_SINGLE, 2803 },
	{ 0x6de5, 0x6de5, PDF_CMAP_SINGLE, 8022 },
	{ 0x6de6, 0x6de6, PDF_CMAP_SINGLE, 2833 },
	{ 0x6de8, 0x6de8, PDF_CMAP_SINGLE, 2827 },
	{ 0x6de9, 0x6de9, PDF_CMAP_SINGLE, 8011 },
	{ 0x6dea, 0x6dea, PDF_CMAP_SINGLE, 2824 },
	{ 0x6deb, 0x6deb, PDF_CMAP_SINGLE, 2822 },
	{ 0x6dec, 0x6dec, PDF_CMAP_SINGLE, 2831 },
	{ 0x6ded, 0x6ded, PDF_CMAP_SINGLE, 8029 },
	{ 0x6dee, 0x6dee, PDF_CMAP_SINGLE, 2826 },
	{ 0x6def, 0x6def, PDF_CMAP_SINGLE, 7426 },
	{ 0x6df0, 0x6df0, PDF_CMAP_SINGLE, 8030 },
	{ 0x6df1, 0x6df1, PDF_CMAP_SINGLE, 2825 },
	{ 0x6df2, 0x6df2, PDF_CMAP_SINGLE, 8037 },
	{ 0x6df3, 0x6df3, PDF_CMAP_SINGLE, 2798 },
	{ 0x6df4, 0x6df4, PDF_CMAP_SINGLE, 8026 },
	{ 0x6df5, 0x6df5, PDF_CMAP_SINGLE, 2816 },
	{ 0x6df6, 0x6df6, PDF_CMAP_SINGLE, 8014 },
	{ 0x6df7, 0x6df7, PDF_CMAP_SINGLE, 2815 },
	{ 0x6df9, 0x6df9, PDF_CMAP_SINGLE, 2813 },
	{ 0x6dfa, 0x6dfa, PDF_CMAP_SINGLE, 2805 },
	{ 0x6dfb, 0x6dfb, PDF_CMAP_SINGLE, 2804 },
	{ 0x6dfc, 0x6dfc, PDF_CMAP_SINGLE, 8639 },
	{ 0x6dfd, 0x6dfd, PDF_CMAP_SINGLE, 8039 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 8016 },
	{ 0x6e02, 0x6e02, PDF_CMAP_SINGLE, 15610 },
	{ 0x6e03, 0x6e03, PDF_CMAP_SINGLE, 8660 },
	{ 0x6e04, 0x6e04, PDF_CMAP_SINGLE, 15038 },
	{ 0x6e05, 0x6e05, PDF_CMAP_SINGLE, 2806 },
	{ 0x6e0a, 0x6e0a, PDF_CMAP_SINGLE, 15041 },
	{ 0x6e0f, 0x6e0f, PDF_CMAP_SINGLE, 15600 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 18040 },
	{ 0x6e18, 0x6e18, PDF_CMAP_SINGLE, 18041 },
	{ 0x6e19, 0x6e19, PDF_CMAP_SINGLE, 3265 },
	{ 0x6e1a, 0x6e1a, PDF_CMAP_SINGLE, 2819 },
	{ 0x6e1b, 0x6e1b, PDF_CMAP_SINGLE, 3247 },
	{ 0x6e1c, 0x6e1c, PDF_CMAP_SINGLE, 8654 },
	{ 0x6e1d, 0x6e1d, PDF_CMAP_SINGLE, 3261 },
	{ 0x6e1f, 0x6e1f, PDF_CMAP_SINGLE, 8642 },
	{ 0x6e20, 0x6e20, PDF_CMAP_SINGLE, 3244 },
	{ 0x6e21, 0x6e21, PDF_CMAP_SINGLE, 3240 },
	{ 0x6e22, 0x6e22, PDF_CMAP_SINGLE, 8672 },
	{ 0x6e23, 0x6e23, PDF_CMAP_SINGLE, 3246 },
	{ 0x6e24, 0x6e24, PDF_CMAP_SINGLE, 3250 },
	{ 0x6e25, 0x6e25, PDF_CMAP_SINGLE, 3245 },
	{ 0x6e26, 0x6e26, PDF_CMAP_SINGLE, 3254 },
	{ 0x6e27, 0x6e27, PDF_CMAP_SINGLE, 8676 },
	{ 0x6e28, 0x6e28, PDF_CMAP_SINGLE, 8667 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 18042 },
	{ 0x6e2a, 0x6e2a, PDF_CMAP_SINGLE, 17154 },
	{ 0x6e2b, 0x6e2b, PDF_CMAP_SINGLE, 8649 },
	{ 0x6e2c, 0x6e2c, PDF_CMAP_SINGLE, 3259 },
	{ 0x6e2d, 0x6e2d, PDF_CMAP_SINGLE, 3253 },
	{ 0x6e2e, 0x6e2e, PDF_CMAP_SINGLE, 8661 },
	{ 0x6e2f, 0x6e2f, PDF_CMAP_SINGLE, 3237 },
	{ 0x6e30, 0x6e30, PDF_CMAP_SINGLE, 8673 },
	{ 0x6e31, 0x6e31, PDF_CMAP_SINGLE, 8666 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 3241 },
	{ 0x6e33, 0x6e33, PDF_CMAP_SINGLE, 8655 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 3256 },
	{ 0x6e35, 0x6e36, PDF_CMAP_RANGE, 8684 },
	{ 0x6e38, 0x6e38, PDF_CMAP_SINGLE, 3238 },
	{ 0x6e39, 0x6e39, PDF_CMAP_SINGLE, 8671 },
	{ 0x6e3a, 0x6e3a, PDF_CMAP_SINGLE, 3258 },
	{ 0x6e3b, 0x6e3b, PDF_CMAP_SINGLE, 8659 },
	{ 0x6e3c, 0x6e3d, PDF_CMAP_RANGE, 8645 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 3262 },
	{ 0x6e3f, 0x6e3f, PDF_CMAP_SINGLE, 8650 },
	{ 0x6e40, 0x6e40, PDF_CMAP_SINGLE, 8657 },
	{ 0x6e41, 0x6e41, PDF_CMAP_SINGLE, 8651 },
	{ 0x6e43, 0x6e43, PDF_CMAP_SINGLE, 3260 },
	{ 0x6e44, 0x6e44, PDF_CMAP_SINGLE, 3268 },
	{ 0x6e45, 0x6e45, PDF_CMAP_SINGLE, 8647 },
	{ 0x6e46, 0x6e47, PDF_CMAP_RANGE, 8640 },
	{ 0x6e49, 0x6e49, PDF_CMAP_SINGLE, 8643 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 3243 },
	{ 0x6e4b, 0x6e4b, PDF_CMAP_SINGLE, 8656 },
	{ 0x6e4d, 0x6e4d, PDF_CMAP_SINGLE, 3257 },
	{ 0x6e4e, 0x6e4e, PDF_CMAP_SINGLE, 3266 },
	{ 0x6e4f, 0x6e4f, PDF_CMAP_SINGLE, 16948 },
	{ 0x6e50, 0x6e50, PDF_CMAP_SINGLE, 16849 },
	{ 0x6e51, 0x6e51, PDF_CMAP_SINGLE, 8658 },
	{ 0x6e52, 0x6e52, PDF_CMAP_SINGLE, 8682 },
	{ 0x6e53, 0x6e53, PDF_CMAP_SINGLE, 8674 },
	{ 0x6e54, 0x6e54, PDF_CMAP_SINGLE, 3239 },
	{ 0x6e55, 0x6e55, PDF_CMAP_SINGLE, 8680 },
	{ 0x6e56, 0x6e56, PDF_CMAP_SINGLE, 3251 },
	{ 0x6e58, 0x6e58, PDF_CMAP_SINGLE, 3249 },
	{ 0x6e59, 0x6e59, PDF_CMAP_SINGLE, 16614 },
	{ 0x6e5a, 0x6e5a, PDF_CMAP_SINGLE, 8686 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 3248 },
	{ 0x6e5c, 0x6e5c, PDF_CMAP_SINGLE, 8664 },
	{ 0x6e5d, 0x6e5d, PDF_CMAP_SINGLE, 8652 },
	{ 0x6e5e, 0x6e5e, PDF_CMAP_SINGLE, 8662 },
	{ 0x6e5f, 0x6e5f, PDF_CMAP_SINGLE, 3271 },
	{ 0x6e60, 0x6e60, PDF_CMAP_SINGLE, 8668 },
	{ 0x6e61, 0x6e61, PDF_CMAP_SINGLE, 8665 },
	{ 0x6e62, 0x6e62, PDF_CMAP_SINGLE, 8648 },
	{ 0x6e63, 0x6e63, PDF_CMAP_SINGLE, 3267 },
	{ 0x6e64, 0x6e64, PDF_CMAP_SINGLE, 8678 },
	{ 0x6e65, 0x6e65, PDF_CMAP_SINGLE, 8675 },
	{ 0x6e66, 0x6e66, PDF_CMAP_SINGLE, 8683 },
	{ 0x6e67, 0x6e67, PDF_CMAP_SINGLE, 3242 },
	{ 0x6e68, 0x6e68, PDF_CMAP_SINGLE, 8663 },
	{ 0x6e69, 0x6e69, PDF_CMAP_SINGLE, 3270 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 8670 },
	{ 0x6e6e, 0x6e6e, PDF_CMAP_SINGLE, 3252 },
	{ 0x6e6f, 0x6e6f, PDF_CMAP_SINGLE, 3255 },
	{ 0x6e71, 0x6e71, PDF_CMAP_SINGLE, 8669 },
	{ 0x6e72, 0x6e72, PDF_CMAP_SINGLE, 3269 },
	{ 0x6e73, 0x6e73, PDF_CMAP_SINGLE, 8653 },
	{ 0x6e74, 0x6e74, PDF_CMAP_SINGLE, 8009 },
	{ 0x6e76, 0x6e76, PDF_CMAP_SINGLE, 14803 },
	{ 0x6e77, 0x6e77, PDF_CMAP_SINGLE, 8679 },
	{ 0x6e78, 0x6e78, PDF_CMAP_SINGLE, 8677 },
	{ 0x6e79, 0x6e79, PDF_CMAP_SINGLE, 8681 },
	{ 0x6e7c, 0x6e7c, PDF_CMAP_SINGLE, 16931 },
	{ 0x6e86, 0x6e86, PDF_CMAP_SINGLE, 18043 },
	{ 0x6e88, 0x6e88, PDF_CMAP_SINGLE, 8644 },
	{ 0x6e89, 0x6e89, PDF_CMAP_SINGLE, 3264 },
	{ 0x6e8b, 0x6e8b, PDF_CMAP_SINGLE, 15295 },
	{ 0x6e8d, 0x6e8d, PDF_CMAP_SINGLE, 9324 },
	{ 0x6e8e, 0x6e8e, PDF_CMAP_SINGLE, 9323 },
	{ 0x6e8f, 0x6e8f, PDF_CMAP_SINGLE, 9297 },
	{ 0x6e90, 0x6e90, PDF_CMAP_SINGLE, 3669 },
	{ 0x6e92, 0x6e92, PDF_CMAP_SINGLE, 9322 },
	{ 0x6e93, 0x6e94, PDF_CMAP_RANGE, 9300 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 3679 },
	{ 0x6e97, 0x6e97, PDF_CMAP_SINGLE, 9331 },
	{ 0x6e98, 0x6e98, PDF_CMAP_SINGLE, 3674 },
	{ 0x6e99, 0x6e99, PDF_CMAP_SINGLE, 9321 },
	{ 0x6e9a, 0x6e9a, PDF_CMAP_SINGLE, 17793 },
	{ 0x6e9b, 0x6e9b, PDF_CMAP_SINGLE, 9294 },
	{ 0x6e9c, 0x6e9c, PDF_CMAP_SINGLE, 3680 },
	{ 0x6e9d, 0x6e9d, PDF_CMAP_SINGLE, 3670 },
	{ 0x6e9e, 0x6e9e, PDF_CMAP_SINGLE, 9309 },
	{ 0x6e9f, 0x6e9f, PDF_CMAP_SINGLE, 9299 },
	{ 0x6ea0, 0x6ea0, PDF_CMAP_SINGLE, 9302 },
	{ 0x6ea1, 0x6ea1, PDF_CMAP_SINGLE, 9326 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 3664 },
	{ 0x6ea3, 0x6ea3, PDF_CMAP_SINGLE, 9333 },
	{ 0x6ea4, 0x6ea4, PDF_CMAP_SINGLE, 9325 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 3673 },
	{ 0x6ea6, 0x6ea6, PDF_CMAP_SINGLE, 9314 },
	{ 0x6ea7, 0x6ea7, PDF_CMAP_SINGLE, 3684 },
	{ 0x6eaa, 0x6eaa, PDF_CMAP_SINGLE, 3683 },
	{ 0x6eab, 0x6eab, PDF_CMAP_SINGLE, 3677 },
	{ 0x6eae, 0x6eae, PDF_CMAP_SINGLE, 9332 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 3665 },
	{ 0x6eb0, 0x6eb0, PDF_CMAP_SINGLE, 9312 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 9303 },
	{ 0x6eb2, 0x6eb2, PDF_CMAP_SINGLE, 9316 },
	{ 0x6eb3, 0x6eb3, PDF_CMAP_SINGLE, 9328 },
	{ 0x6eb4, 0x6eb4, PDF_CMAP_SINGLE, 3685 },
	{ 0x6eb5, 0x6eb5, PDF_CMAP_SINGLE, 16894 },
	{ 0x6eb6, 0x6eb6, PDF_CMAP_SINGLE, 3667 },
	{ 0x6eb7, 0x6eb7, PDF_CMAP_SINGLE, 9311 },
	{ 0x6eb8, 0x6eb8, PDF_CMAP_SINGLE, 15884 },
	{ 0x6eb9, 0x6eb9, PDF_CMAP_SINGLE, 9304 },
	{ 0x6eba, 0x6eba, PDF_CMAP_SINGLE, 3676 },
	{ 0x6ebb, 0x6ebb, PDF_CMAP_SINGLE, 18045 },
	{ 0x6ebc, 0x6ebc, PDF_CMAP_SINGLE, 3675 },
	{ 0x6ebd, 0x6ebd, PDF_CMAP_SINGLE, 9307 },
	{ 0x6ebe, 0x6ebe, PDF_CMAP_SINGLE, 9317 },
	{ 0x6ebf, 0x6ebf, PDF_CMAP_SINGLE, 9327 },
	{ 0x6ec0, 0x6ec0, PDF_CMAP_SINGLE, 9298 },
	{ 0x6ec1, 0x6ec1, PDF_CMAP_SINGLE, 9308 },
	{ 0x6ec2, 0x6ec2, PDF_CMAP_SINGLE, 3668 },
	{ 0x6ec3, 0x6ec3, PDF_CMAP_SINGLE, 9318 },
	{ 0x6ec4, 0x6ec4, PDF_CMAP_SINGLE, 3681 },
	{ 0x6ec5, 0x6ec5, PDF_CMAP_SINGLE, 3672 },
	{ 0x6ec6, 0x6ec6, PDF_CMAP_SINGLE, 9305 },
	{ 0x6ec7, 0x6ec7, PDF_CMAP_SINGLE, 3671 },
	{ 0x6ec8, 0x6ec8, PDF_CMAP_SINGLE, 9296 },
	{ 0x6ec9, 0x6ec9, PDF_CMAP_SINGLE, 9310 },
	{ 0x6eca, 0x6eca, PDF_CMAP_SINGLE, 9330 },
	{ 0x6ecb, 0x6ecb, PDF_CMAP_SINGLE, 3263 },
	{ 0x6ecc, 0x6ecc, PDF_CMAP_SINGLE, 4127 },
	{ 0x6ecd, 0x6ecd, PDF_CMAP_SINGLE, 9313 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 9963 },
	{ 0x6ecf, 0x6ecf, PDF_CMAP_SINGLE, 9315 },
	{ 0x6ed0, 0x6ed0, PDF_CMAP_SINGLE, 9329 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 3678 },
	{ 0x6ed2, 0x6ed2, PDF_CMAP_SINGLE, 9306 },
	{ 0x6ed3, 0x6ed3, PDF_CMAP_SINGLE, 3666 },
	{ 0x6ed4, 0x6ed4, PDF_CMAP_SINGLE, 3682 },
	{ 0x6ed5, 0x6ed5, PDF_CMAP_SINGLE, 4506 },
	{ 0x6ed6, 0x6ed6, PDF_CMAP_SINGLE, 9295 },
	{ 0x6ed8, 0x6ed8, PDF_CMAP_SINGLE, 9320 },
	{ 0x6ed9, 0x6ed9, PDF_CMAP_SINGLE, 16489 },
	{ 0x6eda, 0x6eda, PDF_CMAP_SINGLE, 18047 },
	{ 0x6edb, 0x6edb, PDF_CMAP_SINGLE, 16431 },
	{ 0x6edc, 0x6edc, PDF_CMAP_SINGLE, 9319 },
	{ 0x6edd, 0x6edd, PDF_CMAP_SINGLE, 17720 },
	{ 0x6ee2, 0x6ee2, PDF_CMAP_SINGLE, 18046 },
	{ 0x6ee8, 0x6ee9, PDF_CMAP_RANGE, 18049 },
	{ 0x6eeb, 0x6eeb, PDF_CMAP_SINGLE, 9991 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 4124 },
	{ 0x6eed, 0x6eed, PDF_CMAP_SINGLE, 9980 },
	{ 0x6eee, 0x6eee, PDF_CMAP_SINGLE, 9985 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 4113 },
	{ 0x6ef1, 0x6ef1, PDF_CMAP_SINGLE, 9965 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 4126 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 4104 },
	{ 0x6ef5, 0x6ef5, PDF_CMAP_SINGLE, 9964 },
	{ 0x6ef6, 0x6ef6, PDF_CMAP_SINGLE, 9997 },
	{ 0x6ef7, 0x6ef7, PDF_CMAP_SINGLE, 4128 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 9968 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 9984 },
	{ 0x6efa, 0x6efa, PDF_CMAP_SINGLE, 16955 },
	{ 0x6efb, 0x6efb, PDF_CMAP_SINGLE, 9970 },
	{ 0x6efc, 0x6efc, PDF_CMAP_SINGLE, 10000 },
	{ 0x6efd, 0x6efd, PDF_CMAP_SINGLE, 9996 },
	{ 0x6efe, 0x6efe, PDF_CMAP_SINGLE, 4102 },
	{ 0x6eff, 0x6eff, PDF_CMAP_SINGLE, 4112 },
	{ 0x6f00, 0x6f00, PDF_CMAP_SINGLE, 10566 },
	{ 0x6f01, 0x6f01, PDF_CMAP_SINGLE, 4125 },
	{ 0x6f02, 0x6f02, PDF_CMAP_SINGLE, 4110 },
	{ 0x6f03, 0x6f03, PDF_CMAP_SINGLE, 9966 },
	{ 0x6f04, 0x6f04, PDF_CMAP_SINGLE, 16932 },
	{ 0x6f05, 0x6f05, PDF_CMAP_SINGLE, 9995 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 4114 },
	{ 0x6f07, 0x6f07, PDF_CMAP_SINGLE, 9992 },
	{ 0x6f08, 0x6f08, PDF_CMAP_SINGLE, 10005 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 9972 },
	{ 0x6f0a, 0x6f0a, PDF_CMAP_SINGLE, 9981 },
	{ 0x6f0b, 0x6f0b, PDF_CMAP_SINGLE, 15034 },
	{ 0x6f0c, 0x6f0c, PDF_CMAP_SINGLE, 15611 },
	{ 0x6f0d, 0x6f0d, PDF_CMAP_SINGLE, 10003 },
	{ 0x6f0e, 0x6f0e, PDF_CMAP_SINGLE, 9993 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 4109 },
	{ 0x6f12, 0x6f12, PDF_CMAP_SINGLE, 9979 },
	{ 0x6f13, 0x6f13, PDF_CMAP_SINGLE, 4103 },
	{ 0x6f14, 0x6f14, PDF_CMAP_SINGLE, 4101 },
	{ 0x6f15, 0x6f15, PDF_CMAP_SINGLE, 4119 },
	{ 0x6f16, 0x6f16, PDF_CMAP_SINGLE, 16800 },
	{ 0x6f17, 0x6f17, PDF_CMAP_SINGLE, 18126 },
	{ 0x6f18, 0x6f18, PDF_CMAP_SINGLE, 9977 },
	{ 0x6f19, 0x6f1a, PDF_CMAP_RANGE, 9974 },
	{ 0x6f1c, 0x6f1c, PDF_CMAP_SINGLE, 9999 },
	{ 0x6f1e, 0x6f1e, PDF_CMAP_SINGLE, 10004 },
	{ 0x6f1f, 0x6f1f, PDF_CMAP_SINGLE, 10002 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 4107 },
	{ 0x6f21, 0x6f21, PDF_CMAP_SINGLE, 10006 },
	{ 0x6f22, 0x6f22, PDF_CMAP_SINGLE, 4111 },
	{ 0x6f23, 0x6f23, PDF_CMAP_SINGLE, 4118 },
	{ 0x6f24, 0x6f24, PDF_CMAP_SINGLE, 18051 },
	{ 0x6f25, 0x6f25, PDF_CMAP_SINGLE, 9967 },
	{ 0x6f26, 0x6f26, PDF_CMAP_SINGLE, 10542 },
	{ 0x6f27, 0x6f27, PDF_CMAP_SINGLE, 9976 },
	{ 0x6f29, 0x6f29, PDF_CMAP_SINGLE, 4105 },
	{ 0x6f2a, 0x6f2a, PDF_CMAP_SINGLE, 4123 },
	{ 0x6f2b, 0x6f2b, PDF_CMAP_SINGLE, 4120 },
	{ 0x6f2c, 0x6f2c, PDF_CMAP_SINGLE, 4108 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 9986 },
	{ 0x6f2e, 0x6f2e, PDF_CMAP_SINGLE, 9971 },
	{ 0x6f2f, 0x6f2f, PDF_CMAP_SINGLE, 4121 },
	{ 0x6f30, 0x6f30, PDF_CMAP_SINGLE, 9988 },
	{ 0x6f31, 0x6f31, PDF_CMAP_SINGLE, 4115 },
	{ 0x6f32, 0x6f32, PDF_CMAP_SINGLE, 4117 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 4100 },
	{ 0x6f34, 0x6f34, PDF_CMAP_SINGLE, 18052 },
	{ 0x6f35, 0x6f35, PDF_CMAP_SINGLE, 9990 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 9982 },
	{ 0x6f37, 0x6f37, PDF_CMAP_SINGLE, 9969 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 4116 },
	{ 0x6f39, 0x6f39, PDF_CMAP_SINGLE, 9998 },
	{ 0x6f3a, 0x6f3a, PDF_CMAP_SINGLE, 10001 },
	{ 0x6f3b, 0x6f3b, PDF_CMAP_SINGLE, 9978 },
	{ 0x6f3c, 0x6f3c, PDF_CMAP_SINGLE, 9989 },
	{ 0x6f3d, 0x6f3d, PDF_CMAP_SINGLE, 17026 },
	{ 0x6f3e, 0x6f3e, PDF_CMAP_SINGLE, 4106 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 4489 },
	{ 0x6f40, 0x6f40, PDF_CMAP_SINGLE, 9987 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 10541 },
	{ 0x6f43, 0x6f43, PDF_CMAP_SINGLE, 9994 },
	{ 0x6f44, 0x6f44, PDF_CMAP_SINGLE, 16960 },
	{ 0x6f4e, 0x6f4e, PDF_CMAP_SINGLE, 9973 },
	{ 0x6f4f, 0x6f4f, PDF_CMAP_SINGLE, 10551 },
	{ 0x6f50, 0x6f50, PDF_CMAP_SINGLE, 10561 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 4492 },
	{ 0x6f52, 0x6f52, PDF_CMAP_SINGLE, 10560 },
	{ 0x6f53, 0x6f53, PDF_CMAP_SINGLE, 10572 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 4494 },
	{ 0x6f55, 0x6f55, PDF_CMAP_SINGLE, 10558 },
	{ 0x6f56, 0x6f56, PDF_CMAP_SINGLE, 15954 },
	{ 0x6f57, 0x6f57, PDF_CMAP_SINGLE, 10562 },
	{ 0x6f58, 0x6f58, PDF_CMAP_SINGLE, 4505 },
	{ 0x6f5a, 0x6f5a, PDF_CMAP_SINGLE, 10553 },
	{ 0x6f5b, 0x6f5b, PDF_CMAP_SINGLE, 4497 },
	{ 0x6f5c, 0x6f5c, PDF_CMAP_SINGLE, 16961 },
	{ 0x6f5d, 0x6f5d, PDF_CMAP_SINGLE, 10565 },
	{ 0x6f5e, 0x6f5e, PDF_CMAP_SINGLE, 11162 },
	{ 0x6f5f, 0x6f5f, PDF_CMAP_SINGLE, 4509 },
	{ 0x6f60, 0x6f60, PDF_CMAP_SINGLE, 4508 },
	{ 0x6f61, 0x6f61, PDF_CMAP_SINGLE, 10567 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 10550 },
	{ 0x6f63, 0x6f63, PDF_CMAP_SINGLE, 10577 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 4503 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 4493 },
	{ 0x6f67, 0x6f67, PDF_CMAP_SINGLE, 10570 },
	{ 0x6f69, 0x6f69, PDF_CMAP_SINGLE, 10574 },
	{ 0x6f6a, 0x6f6a, PDF_CMAP_SINGLE, 10579 },
	{ 0x6f6b, 0x6f6b, PDF_CMAP_SINGLE, 10568 },
	{ 0x6f6c, 0x6f6c, PDF_CMAP_SINGLE, 10556 },
	{ 0x6f6d, 0x6f6d, PDF_CMAP_SINGLE, 4496 },
	{ 0x6f6e, 0x6f6e, PDF_CMAP_SINGLE, 4499 },
	{ 0x6f6f, 0x6f6f, PDF_CMAP_SINGLE, 4507 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 4502 },
	{ 0x6f72, 0x6f72, PDF_CMAP_SINGLE, 10559 },
	{ 0x6f73, 0x6f73, PDF_CMAP_SINGLE, 9983 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 16963 },
	{ 0x6f76, 0x6f76, PDF_CMAP_SINGLE, 10555 },
	{ 0x6f77, 0x6f77, PDF_CMAP_SINGLE, 10578 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 4498 },
	{ 0x6f79, 0x6f79, PDF_CMAP_SINGLE, 15294 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 4501 },
	{ 0x6f7b, 0x6f7b, PDF_CMAP_SINGLE, 10580 },
	{ 0x6f7c, 0x6f7c, PDF_CMAP_SINGLE, 4490 },
	{ 0x6f7d, 0x6f7d, PDF_CMAP_SINGLE, 10569 },
	{ 0x6f7e, 0x6f7e, PDF_CMAP_SINGLE, 10543 },
	{ 0x6f7f, 0x6f7f, PDF_CMAP_SINGLE, 10575 },
	{ 0x6f80, 0x6f80, PDF_CMAP_SINGLE, 5116 },
	{ 0x6f81, 0x6f81, PDF_CMAP_SINGLE, 18055 },
	{ 0x6f82, 0x6f82, PDF_CMAP_SINGLE, 10557 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 4491 },
	{ 0x6f85, 0x6f85, PDF_CMAP_SINGLE, 10552 },
	{ 0x6f86, 0x6f86, PDF_CMAP_SINGLE, 4495 },
	{ 0x6f87, 0x6f87, PDF_CMAP_SINGLE, 10544 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 4122 },
	{ 0x6f89, 0x6f89, PDF_CMAP_SINGLE, 10548 },
	{ 0x6f8a, 0x6f8a, PDF_CMAP_SINGLE, 16789 },
	{ 0x6f8b, 0x6f8b, PDF_CMAP_SINGLE, 10573 },
	{ 0x6f8c, 0x6f8c, PDF_CMAP_SINGLE, 10549 },
	{ 0x6f8d, 0x6f8d, PDF_CMAP_SINGLE, 10547 },
	{ 0x6f8e, 0x6f8e, PDF_CMAP_SINGLE, 4500 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 10571 },
	{ 0x6f92, 0x6f92, PDF_CMAP_SINGLE, 10546 },
	{ 0x6f93, 0x6f93, PDF_CMAP_SINGLE, 10564 },
	{ 0x6f94, 0x6f94, PDF_CMAP_SINGLE, 10563 },
	{ 0x6f95, 0x6f95, PDF_CMAP_SINGLE, 10576 },
	{ 0x6f96, 0x6f96, PDF_CMAP_SINGLE, 10554 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 4504 },
	{ 0x6f9d, 0x6f9d, PDF_CMAP_SINGLE, 16953 },
	{ 0x6f9e, 0x6f9e, PDF_CMAP_SINGLE, 11165 },
	{ 0x6f9f, 0x6f9f, PDF_CMAP_SINGLE, 16966 },
	{ 0x6fa0, 0x6fa0, PDF_CMAP_SINGLE, 4843 },
	{ 0x6fa1, 0x6fa1, PDF_CMAP_SINGLE, 4833 },
	{ 0x6fa2, 0x6fa2, PDF_CMAP_SINGLE, 11177 },
	{ 0x6fa3, 0x6fa3, PDF_CMAP_SINGLE, 11157 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 4835 },
	{ 0x6fa5, 0x6fa5, PDF_CMAP_SINGLE, 11169 },
	{ 0x6fa6, 0x6fa6, PDF_CMAP_SINGLE, 4842 },
	{ 0x6fa7, 0x6fa7, PDF_CMAP_SINGLE, 4837 },
	{ 0x6fa8, 0x6fa8, PDF_CMAP_SINGLE, 11167 },
	{ 0x6fa9, 0x6fa9, PDF_CMAP_SINGLE, 11662 },
	{ 0x6faa, 0x6faa, PDF_CMAP_SINGLE, 11173 },
	{ 0x6fab, 0x6fab, PDF_CMAP_SINGLE, 11179 },
	{ 0x6fac, 0x6fac, PDF_CMAP_SINGLE, 11172 },
	{ 0x6fad, 0x6fad, PDF_CMAP_SINGLE, 11155 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 11170 },
	{ 0x6faf, 0x6faf, PDF_CMAP_SINGLE, 11181 },
	{ 0x6fb0, 0x6fb0, PDF_CMAP_SINGLE, 11183 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 4832 },
	{ 0x6fb2, 0x6fb2, PDF_CMAP_SINGLE, 11182 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 4838 },
	{ 0x6fb4, 0x6fb4, PDF_CMAP_SINGLE, 4844 },
	{ 0x6fb5, 0x6fb5, PDF_CMAP_SINGLE, 15911 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 4841 },
	{ 0x6fb8, 0x6fb8, PDF_CMAP_SINGLE, 11176 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 4840 },
	{ 0x6fba, 0x6fba, PDF_CMAP_SINGLE, 11171 },
	{ 0x6fbb, 0x6fbb, PDF_CMAP_SINGLE, 15291 },
	{ 0x6fbc, 0x6fbc, PDF_CMAP_SINGLE, 11159 },
	{ 0x6fbd, 0x6fbd, PDF_CMAP_SINGLE, 11164 },
	{ 0x6fbe, 0x6fbe, PDF_CMAP_SINGLE, 18056 },
	{ 0x6fbf, 0x6fbf, PDF_CMAP_SINGLE, 11175 },
	{ 0x6fc0, 0x6fc0, PDF_CMAP_SINGLE, 4839 },
	{ 0x6fc1, 0x6fc1, PDF_CMAP_SINGLE, 4836 },
	{ 0x6fc2, 0x6fc2, PDF_CMAP_SINGLE, 4831 },
	{ 0x6fc3, 0x6fc3, PDF_CMAP_SINGLE, 4834 },
	{ 0x6fc4, 0x6fc4, PDF_CMAP_SINGLE, 11163 },
	{ 0x6fc6, 0x6fc6, PDF_CMAP_SINGLE, 10545 },
	{ 0x6fc7, 0x6fc7, PDF_CMAP_SINGLE, 11158 },
	{ 0x6fc8, 0x6fc8, PDF_CMAP_SINGLE, 11161 },
	{ 0x6fc9, 0x6fc9, PDF_CMAP_SINGLE, 11178 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 11166 },
	{ 0x6fcb, 0x6fcb, PDF_CMAP_SINGLE, 11156 },
	{ 0x6fcc, 0x6fcc, PDF_CMAP_SINGLE, 11661 },
	{ 0x6fcd, 0x6fcd, PDF_CMAP_SINGLE, 11180 },
	{ 0x6fce, 0x6fce, PDF_CMAP_SINGLE, 11160 },
	{ 0x6fcf, 0x6fcf, PDF_CMAP_SINGLE, 11174 },
	{ 0x6fd3, 0x6fd3, PDF_CMAP_SINGLE, 16968 },
	{ 0x6fd4, 0x6fd4, PDF_CMAP_SINGLE, 11664 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 5120 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 5108 },
	{ 0x6fd9, 0x6fd9, PDF_CMAP_SINGLE, 15476 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 15475 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 5112 },
	{ 0x6fdc, 0x6fdc, PDF_CMAP_SINGLE, 11666 },
	{ 0x6fdd, 0x6fdd, PDF_CMAP_SINGLE, 11672 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 11670 },
	{ 0x6fdf, 0x6fe0, PDF_CMAP_RANGE, 5110 },
	{ 0x6fe1, 0x6fe1, PDF_CMAP_SINGLE, 5118 },
	{ 0x6fe2, 0x6fe2, PDF_CMAP_SINGLE, 11673 },
	{ 0x6fe3, 0x6fe3, PDF_CMAP_SINGLE, 11665 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 5113 },
	{ 0x6fe6, 0x6fe6, PDF_CMAP_SINGLE, 11669 },
	{ 0x6fe7, 0x6fe7, PDF_CMAP_SINGLE, 11668 },
	{ 0x6fe8, 0x6fe8, PDF_CMAP_SINGLE, 11674 },
	{ 0x6fe9, 0x6fe9, PDF_CMAP_SINGLE, 5119 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 5114 },
	{ 0x6fec, 0x6fec, PDF_CMAP_SINGLE, 5117 },
	{ 0x6fed, 0x6fed, PDF_CMAP_SINGLE, 11667 },
	{ 0x6fee, 0x6fee, PDF_CMAP_SINGLE, 5121 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 5115 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 5122 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 5109 },
	{ 0x6ff2, 0x6ff2, PDF_CMAP_SINGLE, 11671 },
	{ 0x6ff4, 0x6ff4, PDF_CMAP_SINGLE, 11663 },
	{ 0x6ff6, 0x6ff6, PDF_CMAP_SINGLE, 14501 },
	{ 0x6ff7, 0x6ff7, PDF_CMAP_SINGLE, 12110 },
	{ 0x6ff8, 0x6ff8, PDF_CMAP_SINGLE, 16951 },
	{ 0x6ffa, 0x6ffa, PDF_CMAP_SINGLE, 5355 },
	{ 0x6ffb, 0x6ffb, PDF_CMAP_SINGLE, 12107 },
	{ 0x6ffc, 0x6ffc, PDF_CMAP_SINGLE, 12109 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 5353 },
	{ 0x6fff, 0x6fff, PDF_CMAP_SINGLE, 12105 },
	{ 0x7000, 0x7000, PDF_CMAP_SINGLE, 12106 },
	{ 0x7001, 0x7001, PDF_CMAP_SINGLE, 12101 },
	{ 0x7003, 0x7003, PDF_CMAP_SINGLE, 18146 },
	{ 0x7004, 0x7004, PDF_CMAP_SINGLE, 11168 },
	{ 0x7005, 0x7005, PDF_CMAP_SINGLE, 12102 },
	{ 0x7006, 0x7006, PDF_CMAP_SINGLE, 5354 },
	{ 0x7007, 0x7007, PDF_CMAP_SINGLE, 12098 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 5351 },
	{ 0x700a, 0x700a, PDF_CMAP_SINGLE, 12111 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 5352 },
	{ 0x700c, 0x700d, PDF_CMAP_RANGE, 12099 },
	{ 0x700e, 0x700e, PDF_CMAP_SINGLE, 12104 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 5357 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5356 },
	{ 0x7014, 0x7014, PDF_CMAP_SINGLE, 12103 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 5519 },
	{ 0x7016, 0x7016, PDF_CMAP_SINGLE, 12461 },
	{ 0x7017, 0x7017, PDF_CMAP_SINGLE, 12467 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 5520 },
	{ 0x7019, 0x7019, PDF_CMAP_SINGLE, 12458 },
	{ 0x701a, 0x701a, PDF_CMAP_SINGLE, 5517 },
	{ 0x701b, 0x701b, PDF_CMAP_SINGLE, 5514 },
	{ 0x701c, 0x701c, PDF_CMAP_SINGLE, 12469 },
	{ 0x701d, 0x701d, PDF_CMAP_SINGLE, 5518 },
	{ 0x701e, 0x701e, PDF_CMAP_SINGLE, 15478 },
	{ 0x701f, 0x701f, PDF_CMAP_SINGLE, 5515 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 12460 },
	{ 0x7021, 0x7023, PDF_CMAP_RANGE, 12463 },
	{ 0x7024, 0x7024, PDF_CMAP_SINGLE, 12468 },
	{ 0x7026, 0x7026, PDF_CMAP_SINGLE, 12108 },
	{ 0x7027, 0x7027, PDF_CMAP_SINGLE, 12459 },
	{ 0x7028, 0x7028, PDF_CMAP_SINGLE, 5516 },
	{ 0x7029, 0x7029, PDF_CMAP_SINGLE, 12466 },
	{ 0x702a, 0x702a, PDF_CMAP_SINGLE, 12783 },
	{ 0x702b, 0x702b, PDF_CMAP_SINGLE, 12462 },
	{ 0x702c, 0x702c, PDF_CMAP_SINGLE, 18062 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 12774 },
	{ 0x7030, 0x7030, PDF_CMAP_SINGLE, 5661 },
	{ 0x7031, 0x7031, PDF_CMAP_SINGLE, 12777 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 5662 },
	{ 0x7033, 0x7033, PDF_CMAP_SINGLE, 12786 },
	{ 0x7034, 0x7034, PDF_CMAP_SINGLE, 12776 },
	{ 0x7035, 0x7035, PDF_CMAP_SINGLE, 12773 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 12775 },
	{ 0x7038, 0x7038, PDF_CMAP_SINGLE, 12779 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 12782 },
	{ 0x703a, 0x703a, PDF_CMAP_SINGLE, 12781 },
	{ 0x703b, 0x703b, PDF_CMAP_SINGLE, 12785 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 12772 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 5660 },
	{ 0x703f, 0x703f, PDF_CMAP_SINGLE, 12780 },
	{ 0x7040, 0x7040, PDF_CMAP_SINGLE, 12784 },
	{ 0x7041, 0x7041, PDF_CMAP_SINGLE, 12787 },
	{ 0x7042, 0x7042, PDF_CMAP_SINGLE, 12778 },
	{ 0x7043, 0x7044, PDF_CMAP_RANGE, 13016 },
	{ 0x7045, 0x7046, PDF_CMAP_RANGE, 13021 },
	{ 0x7048, 0x7049, PDF_CMAP_RANGE, 13019 },
	{ 0x704a, 0x704a, PDF_CMAP_SINGLE, 13018 },
	{ 0x704b, 0x704b, PDF_CMAP_SINGLE, 16980 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 5754 },
	{ 0x7050, 0x7050, PDF_CMAP_SINGLE, 18064 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 5832 },
	{ 0x7052, 0x7052, PDF_CMAP_SINGLE, 13196 },
	{ 0x7054, 0x7054, PDF_CMAP_SINGLE, 18065 },
	{ 0x7055, 0x7057, PDF_CMAP_RANGE, 13193 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 5833 },
	{ 0x705a, 0x705a, PDF_CMAP_SINGLE, 13347 },
	{ 0x705b, 0x705b, PDF_CMAP_SINGLE, 13346 },
	{ 0x705c, 0x705c, PDF_CMAP_SINGLE, 14418 },
	{ 0x705d, 0x705d, PDF_CMAP_SINGLE, 13450 },
	{ 0x705e, 0x705e, PDF_CMAP_SINGLE, 5922 },
	{ 0x705f, 0x705f, PDF_CMAP_SINGLE, 13448 },
	{ 0x7060, 0x7060, PDF_CMAP_SINGLE, 13451 },
	{ 0x7061, 0x7061, PDF_CMAP_SINGLE, 13449 },
	{ 0x7062, 0x7062, PDF_CMAP_SINGLE, 13520 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 5956 },
	{ 0x7064, 0x7064, PDF_CMAP_SINGLE, 5970 },
	{ 0x7065, 0x7065, PDF_CMAP_SINGLE, 13596 },
	{ 0x7066, 0x7066, PDF_CMAP_SINGLE, 13574 },
	{ 0x7067, 0x7067, PDF_CMAP_SINGLE, 15605 },
	{ 0x7068, 0x7068, PDF_CMAP_SINGLE, 13595 },
	{ 0x7069, 0x706a, PDF_CMAP_RANGE, 13639 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 742 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 17649 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 18066 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 1010 },
	{ 0x7071, 0x7071, PDF_CMAP_SINGLE, 6154 },
	{ 0x7074, 0x7074, PDF_CMAP_SINGLE, 6339 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 14535 },
	{ 0x7076, 0x7076, PDF_CMAP_SINGLE, 1254 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 1257 },
	{ 0x7079, 0x7079, PDF_CMAP_SINGLE, 16537 },
	{ 0x707a, 0x707a, PDF_CMAP_SINGLE, 6340 },
	{ 0x707c, 0x707d, PDF_CMAP_RANGE, 1255 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 16981 },
	{ 0x707f, 0x707f, PDF_CMAP_SINGLE, 18067 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 18777 },
	{ 0x7082, 0x7082, PDF_CMAP_SINGLE, 6647 },
	{ 0x7083, 0x7083, PDF_CMAP_SINGLE, 6649 },
	{ 0x7084, 0x7084, PDF_CMAP_SINGLE, 6644 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 6641 },
	{ 0x7086, 0x7086, PDF_CMAP_SINGLE, 6643 },
	{ 0x7089, 0x7089, PDF_CMAP_SINGLE, 17401 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 1603 },
	{ 0x708b, 0x708b, PDF_CMAP_SINGLE, 16587 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 1601 },
	{ 0x708f, 0x708f, PDF_CMAP_SINGLE, 18099 },
	{ 0x7091, 0x7091, PDF_CMAP_SINGLE, 6645 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 1602 },
	{ 0x7093, 0x7093, PDF_CMAP_SINGLE, 6642 },
	{ 0x7094, 0x7094, PDF_CMAP_SINGLE, 6639 },
	{ 0x7095, 0x7095, PDF_CMAP_SINGLE, 1600 },
	{ 0x7096, 0x7096, PDF_CMAP_SINGLE, 6646 },
	{ 0x7098, 0x7098, PDF_CMAP_SINGLE, 6640 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 1604 },
	{ 0x709a, 0x709a, PDF_CMAP_SINGLE, 6648 },
	{ 0x709f, 0x709f, PDF_CMAP_SINGLE, 7014 },
	{ 0x70a0, 0x70a0, PDF_CMAP_SINGLE, 18242 },
	{ 0x70a1, 0x70a1, PDF_CMAP_SINGLE, 7018 },
	{ 0x70a3, 0x70a3, PDF_CMAP_SINGLE, 14706 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 1936 },
	{ 0x70a5, 0x70a5, PDF_CMAP_SINGLE, 15543 },
	{ 0x70a6, 0x70a6, PDF_CMAP_SINGLE, 15699 },
	{ 0x70a7, 0x70a7, PDF_CMAP_SINGLE, 16982 },
	{ 0x70a9, 0x70a9, PDF_CMAP_SINGLE, 7021 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 1928 },
	{ 0x70ac, 0x70ac, PDF_CMAP_SINGLE, 1931 },
	{ 0x70ad, 0x70ad, PDF_CMAP_SINGLE, 1933 },
	{ 0x70ae, 0x70ae, PDF_CMAP_SINGLE, 1935 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 1932 },
	{ 0x70b0, 0x70b0, PDF_CMAP_SINGLE, 7017 },
	{ 0x70b1, 0x70b1, PDF_CMAP_SINGLE, 7016 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 1930 },
	{ 0x70b4, 0x70b5, PDF_CMAP_RANGE, 7019 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 7013 },
	{ 0x70b8, 0x70b8, PDF_CMAP_SINGLE, 1934 },
	{ 0x70b9, 0x70b9, PDF_CMAP_SINGLE, 15721 },
	{ 0x70ba, 0x70ba, PDF_CMAP_SINGLE, 1929 },
	{ 0x70bb, 0x70bb, PDF_CMAP_SINGLE, 18450 },
	{ 0x70bc, 0x70bc, PDF_CMAP_SINGLE, 17753 },
	{ 0x70bd, 0x70bd, PDF_CMAP_SINGLE, 17008 },
	{ 0x70be, 0x70be, PDF_CMAP_SINGLE, 7015 },
	{ 0x70c0, 0x70c0, PDF_CMAP_SINGLE, 18227 },
	{ 0x70c4, 0x70c4, PDF_CMAP_SINGLE, 15274 },
	{ 0x70c5, 0x70c7, PDF_CMAP_RANGE, 7463 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 2337 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 2333 },
	{ 0x70cb, 0x70cb, PDF_CMAP_SINGLE, 7454 },
	{ 0x70cc, 0x70cc, PDF_CMAP_SINGLE, 16984 },
	{ 0x70cd, 0x70cd, PDF_CMAP_SINGLE, 7462 },
	{ 0x70ce, 0x70ce, PDF_CMAP_SINGLE, 7467 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 2338 },
	{ 0x70d0, 0x70d0, PDF_CMAP_SINGLE, 14701 },
	{ 0x70d1, 0x70d1, PDF_CMAP_SINGLE, 7452 },
	{ 0x70d2, 0x70d2, PDF_CMAP_SINGLE, 7458 },
	{ 0x70d3, 0x70d3, PDF_CMAP_SINGLE, 7451 },
	{ 0x70d4, 0x70d4, PDF_CMAP_SINGLE, 7461 },
	{ 0x70d5, 0x70d6, PDF_CMAP_RANGE, 16985 },
	{ 0x70d7, 0x70d7, PDF_CMAP_SINGLE, 7457 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 2334 },
	{ 0x70d9, 0x70d9, PDF_CMAP_SINGLE, 2336 },
	{ 0x70da, 0x70da, PDF_CMAP_SINGLE, 7466 },
	{ 0x70dc, 0x70dc, PDF_CMAP_SINGLE, 7450 },
	{ 0x70dd, 0x70dd, PDF_CMAP_SINGLE, 7453 },
	{ 0x70de, 0x70de, PDF_CMAP_SINGLE, 7459 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 16987 },
	{ 0x70e0, 0x70e0, PDF_CMAP_SINGLE, 7460 },
	{ 0x70e1, 0x70e1, PDF_CMAP_SINGLE, 7468 },
	{ 0x70e2, 0x70e2, PDF_CMAP_SINGLE, 7456 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2335 },
	{ 0x70ef, 0x70ef, PDF_CMAP_SINGLE, 2838 },
	{ 0x70f0, 0x70f0, PDF_CMAP_SINGLE, 8050 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 15048 },
	{ 0x70f3, 0x70f3, PDF_CMAP_SINGLE, 8052 },
	{ 0x70f4, 0x70f4, PDF_CMAP_SINGLE, 8048 },
	{ 0x70f5, 0x70f5, PDF_CMAP_SINGLE, 18272 },
	{ 0x70f6, 0x70f6, PDF_CMAP_SINGLE, 8060 },
	{ 0x70f7, 0x70f7, PDF_CMAP_SINGLE, 8046 },
	{ 0x70f8, 0x70f8, PDF_CMAP_SINGLE, 8059 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 2834 },
	{ 0x70fa, 0x70fa, PDF_CMAP_SINGLE, 8044 },
	{ 0x70fb, 0x70fb, PDF_CMAP_SINGLE, 8690 },
	{ 0x70fc, 0x70fc, PDF_CMAP_SINGLE, 8054 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 2837 },
	{ 0x70fe, 0x70fe, PDF_CMAP_SINGLE, 15630 },
	{ 0x70ff, 0x70ff, PDF_CMAP_SINGLE, 8055 },
	{ 0x7100, 0x7100, PDF_CMAP_SINGLE, 8058 },
	{ 0x7102, 0x7102, PDF_CMAP_SINGLE, 8062 },
	{ 0x7104, 0x7104, PDF_CMAP_SINGLE, 8051 },
	{ 0x7105, 0x7105, PDF_CMAP_SINGLE, 14883 },
	{ 0x7106, 0x7106, PDF_CMAP_SINGLE, 8056 },
	{ 0x7109, 0x710a, PDF_CMAP_RANGE, 2835 },
	{ 0x710b, 0x710b, PDF_CMAP_SINGLE, 8061 },
	{ 0x710c, 0x710c, PDF_CMAP_SINGLE, 8049 },
	{ 0x710d, 0x710d, PDF_CMAP_SINGLE, 8045 },
	{ 0x710e, 0x710e, PDF_CMAP_SINGLE, 8063 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 8053 },
	{ 0x7113, 0x7113, PDF_CMAP_SINGLE, 8057 },
	{ 0x7117, 0x7117, PDF_CMAP_SINGLE, 8047 },
	{ 0x7119, 0x711a, PDF_CMAP_RANGE, 3272 },
	{ 0x711b, 0x711b, PDF_CMAP_SINGLE, 8700 },
	{ 0x711c, 0x711c, PDF_CMAP_SINGLE, 3279 },
	{ 0x711d, 0x711d, PDF_CMAP_SINGLE, 15687 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 8688 },
	{ 0x711f, 0x711f, PDF_CMAP_SINGLE, 8697 },
	{ 0x7120, 0x7120, PDF_CMAP_SINGLE, 8687 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 3276 },
	{ 0x7122, 0x7122, PDF_CMAP_SINGLE, 8695 },
	{ 0x7123, 0x7123, PDF_CMAP_SINGLE, 8693 },
	{ 0x7125, 0x7125, PDF_CMAP_SINGLE, 8694 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 3274 },
	{ 0x7128, 0x7128, PDF_CMAP_SINGLE, 8698 },
	{ 0x7129, 0x7129, PDF_CMAP_SINGLE, 15273 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 16993 },
	{ 0x712c, 0x712c, PDF_CMAP_SINGLE, 17002 },
	{ 0x712e, 0x712e, PDF_CMAP_SINGLE, 8691 },
	{ 0x712f, 0x712f, PDF_CMAP_SINGLE, 8689 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 3275 },
	{ 0x7131, 0x7131, PDF_CMAP_SINGLE, 8692 },
	{ 0x7132, 0x7132, PDF_CMAP_SINGLE, 8696 },
	{ 0x7133, 0x7133, PDF_CMAP_SINGLE, 15700 },
	{ 0x7134, 0x7134, PDF_CMAP_SINGLE, 15486 },
	{ 0x7135, 0x7135, PDF_CMAP_SINGLE, 14189 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 3277 },
	{ 0x713a, 0x713a, PDF_CMAP_SINGLE, 8699 },
	{ 0x713b, 0x713b, PDF_CMAP_SINGLE, 15045 },
	{ 0x713e, 0x713e, PDF_CMAP_SINGLE, 14126 },
	{ 0x7140, 0x7140, PDF_CMAP_SINGLE, 17304 },
	{ 0x7141, 0x7141, PDF_CMAP_SINGLE, 9339 },
	{ 0x7142, 0x7142, PDF_CMAP_SINGLE, 9346 },
	{ 0x7143, 0x7143, PDF_CMAP_SINGLE, 9348 },
	{ 0x7144, 0x7144, PDF_CMAP_SINGLE, 9354 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 16994 },
	{ 0x7146, 0x7146, PDF_CMAP_SINGLE, 3698 },
	{ 0x7147, 0x7147, PDF_CMAP_SINGLE, 9334 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 3690 },
	{ 0x714a, 0x714a, PDF_CMAP_SINGLE, 16996 },
	{ 0x714b, 0x714b, PDF_CMAP_SINGLE, 9349 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 3695 },
	{ 0x714d, 0x714d, PDF_CMAP_SINGLE, 9355 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 3686 },
	{ 0x714f, 0x714f, PDF_CMAP_SINGLE, 16999 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 9352 },
	{ 0x7151, 0x7151, PDF_CMAP_SINGLE, 18533 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 9336 },
	{ 0x7153, 0x7153, PDF_CMAP_SINGLE, 9353 },
	{ 0x7154, 0x7154, PDF_CMAP_SINGLE, 9335 },
	{ 0x7156, 0x7156, PDF_CMAP_SINGLE, 3700 },
	{ 0x7158, 0x7158, PDF_CMAP_SINGLE, 9347 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 3687 },
	{ 0x715a, 0x715a, PDF_CMAP_SINGLE, 9356 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 3692 },
	{ 0x715d, 0x715d, PDF_CMAP_SINGLE, 9340 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 3697 },
	{ 0x715f, 0x715f, PDF_CMAP_SINGLE, 9351 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 9338 },
	{ 0x7161, 0x7161, PDF_CMAP_SINGLE, 9345 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 9341 },
	{ 0x7163, 0x7163, PDF_CMAP_SINGLE, 9337 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 3689 },
	{ 0x7165, 0x7165, PDF_CMAP_SINGLE, 3696 },
	{ 0x7166, 0x7166, PDF_CMAP_SINGLE, 3694 },
	{ 0x7167, 0x7167, PDF_CMAP_SINGLE, 3691 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 3699 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 3688 },
	{ 0x716a, 0x716a, PDF_CMAP_SINGLE, 9344 },
	{ 0x716b, 0x716b, PDF_CMAP_SINGLE, 15557 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 3693 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 3278 },
	{ 0x7170, 0x7170, PDF_CMAP_SINGLE, 9350 },
	{ 0x7171, 0x7171, PDF_CMAP_SINGLE, 15808 },
	{ 0x7172, 0x7172, PDF_CMAP_SINGLE, 9342 },
	{ 0x7173, 0x7173, PDF_CMAP_SINGLE, 18407 },
	{ 0x7175, 0x7175, PDF_CMAP_SINGLE, 15597 },
	{ 0x7176, 0x7176, PDF_CMAP_SINGLE, 16384 },
	{ 0x7177, 0x7177, PDF_CMAP_SINGLE, 14679 },
	{ 0x7178, 0x7178, PDF_CMAP_SINGLE, 9343 },
	{ 0x717a, 0x717a, PDF_CMAP_SINGLE, 18724 },
	{ 0x717b, 0x717b, PDF_CMAP_SINGLE, 10014 },
	{ 0x717c, 0x717c, PDF_CMAP_SINGLE, 14389 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 4131 },
	{ 0x717e, 0x717e, PDF_CMAP_SINGLE, 14704 },
	{ 0x7180, 0x7180, PDF_CMAP_SINGLE, 10010 },
	{ 0x7181, 0x7181, PDF_CMAP_SINGLE, 10016 },
	{ 0x7182, 0x7182, PDF_CMAP_SINGLE, 10012 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 4133 },
	{ 0x7185, 0x7185, PDF_CMAP_SINGLE, 10011 },
	{ 0x7186, 0x7186, PDF_CMAP_SINGLE, 10015 },
	{ 0x7187, 0x7187, PDF_CMAP_SINGLE, 10007 },
	{ 0x7188, 0x7188, PDF_CMAP_SINGLE, 18182 },
	{ 0x7189, 0x7189, PDF_CMAP_SINGLE, 10009 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 4132 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 14826 },
	{ 0x718e, 0x718e, PDF_CMAP_SINGLE, 15675 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 10013 },
	{ 0x7190, 0x7190, PDF_CMAP_SINGLE, 10008 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 4134 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 4129 },
	{ 0x7196, 0x7196, PDF_CMAP_SINGLE, 16991 },
	{ 0x7197, 0x7197, PDF_CMAP_SINGLE, 10017 },
	{ 0x7198, 0x7198, PDF_CMAP_SINGLE, 18461 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 4130 },
	{ 0x719a, 0x719a, PDF_CMAP_SINGLE, 10586 },
	{ 0x719b, 0x719b, PDF_CMAP_SINGLE, 10583 },
	{ 0x719c, 0x719c, PDF_CMAP_SINGLE, 10595 },
	{ 0x719d, 0x719d, PDF_CMAP_SINGLE, 10589 },
	{ 0x719e, 0x719e, PDF_CMAP_SINGLE, 10591 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 4510 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 10585 },
	{ 0x71a1, 0x71a1, PDF_CMAP_SINGLE, 10593 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 18625 },
	{ 0x71a3, 0x71a3, PDF_CMAP_SINGLE, 16072 },
	{ 0x71a4, 0x71a4, PDF_CMAP_SINGLE, 10592 },
	{ 0x71a5, 0x71a5, PDF_CMAP_SINGLE, 10590 },
	{ 0x71a7, 0x71a7, PDF_CMAP_SINGLE, 10596 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 4513 },
	{ 0x71a9, 0x71a9, PDF_CMAP_SINGLE, 10587 },
	{ 0x71aa, 0x71aa, PDF_CMAP_SINGLE, 10594 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 4511 },
	{ 0x71ad, 0x71ad, PDF_CMAP_SINGLE, 15901 },
	{ 0x71af, 0x71af, PDF_CMAP_SINGLE, 10582 },
	{ 0x71b0, 0x71b0, PDF_CMAP_SINGLE, 10584 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 4512 },
	{ 0x71b2, 0x71b2, PDF_CMAP_SINGLE, 10581 },
	{ 0x71b3, 0x71b3, PDF_CMAP_SINGLE, 10597 },
	{ 0x71b4, 0x71b4, PDF_CMAP_SINGLE, 16990 },
	{ 0x71b5, 0x71b5, PDF_CMAP_SINGLE, 10588 },
	{ 0x71b7, 0x71b7, PDF_CMAP_SINGLE, 18059 },
	{ 0x71b8, 0x71b8, PDF_CMAP_SINGLE, 11187 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 4851 },
	{ 0x71ba, 0x71ba, PDF_CMAP_SINGLE, 17006 },
	{ 0x71bc, 0x71bc, PDF_CMAP_SINGLE, 11198 },
	{ 0x71bd, 0x71bd, PDF_CMAP_SINGLE, 11196 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 4845 },
	{ 0x71bf, 0x71bf, PDF_CMAP_SINGLE, 11186 },
	{ 0x71c0, 0x71c1, PDF_CMAP_RANGE, 11189 },
	{ 0x71c2, 0x71c2, PDF_CMAP_SINGLE, 11185 },
	{ 0x71c3, 0x71c4, PDF_CMAP_RANGE, 4855 },
	{ 0x71c5, 0x71c5, PDF_CMAP_SINGLE, 11184 },
	{ 0x71c6, 0x71c6, PDF_CMAP_SINGLE, 11199 },
	{ 0x71c7, 0x71c7, PDF_CMAP_SINGLE, 11194 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 4849 },
	{ 0x71c9, 0x71c9, PDF_CMAP_SINGLE, 4846 },
	{ 0x71ca, 0x71ca, PDF_CMAP_SINGLE, 11193 },
	{ 0x71cb, 0x71cb, PDF_CMAP_SINGLE, 11191 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 4852 },
	{ 0x71cf, 0x71cf, PDF_CMAP_SINGLE, 11195 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 4847 },
	{ 0x71d1, 0x71d1, PDF_CMAP_SINGLE, 14141 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 4848 },
	{ 0x71d4, 0x71d4, PDF_CMAP_SINGLE, 11192 },
	{ 0x71d5, 0x71d5, PDF_CMAP_SINGLE, 4850 },
	{ 0x71d6, 0x71d6, PDF_CMAP_SINGLE, 11188 },
	{ 0x71d8, 0x71d8, PDF_CMAP_SINGLE, 11197 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 4853 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 11200 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 4854 },
	{ 0x71dd, 0x71dd, PDF_CMAP_SINGLE, 16504 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 5124 },
	{ 0x71e0, 0x71e0, PDF_CMAP_SINGLE, 5131 },
	{ 0x71e1, 0x71e1, PDF_CMAP_SINGLE, 11675 },
	{ 0x71e2, 0x71e2, PDF_CMAP_SINGLE, 11681 },
	{ 0x71e4, 0x71e4, PDF_CMAP_SINGLE, 11679 },
	{ 0x71e5, 0x71e5, PDF_CMAP_SINGLE, 5127 },
	{ 0x71e6, 0x71e6, PDF_CMAP_SINGLE, 5126 },
	{ 0x71e7, 0x71e7, PDF_CMAP_SINGLE, 5123 },
	{ 0x71e8, 0x71e8, PDF_CMAP_SINGLE, 11677 },
	{ 0x71eb, 0x71eb, PDF_CMAP_SINGLE, 15489 },
	{ 0x71ec, 0x71ec, PDF_CMAP_SINGLE, 5129 },
	{ 0x71ed, 0x71ed, PDF_CMAP_SINGLE, 5128 },
	{ 0x71ee, 0x71ee, PDF_CMAP_SINGLE, 5125 },
	{ 0x71f0, 0x71f0, PDF_CMAP_SINGLE, 11680 },
	{ 0x71f1, 0x71f1, PDF_CMAP_SINGLE, 11676 },
	{ 0x71f2, 0x71f2, PDF_CMAP_SINGLE, 11678 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 5130 },
	{ 0x71f5, 0x71f5, PDF_CMAP_SINGLE, 18158 },
	{ 0x71f6, 0x71f6, PDF_CMAP_SINGLE, 16026 },
	{ 0x71f8, 0x71f8, PDF_CMAP_SINGLE, 5361 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 12114 },
	{ 0x71fb, 0x71fc, PDF_CMAP_RANGE, 5358 },
	{ 0x71fd, 0x71fd, PDF_CMAP_SINGLE, 12116 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 5360 },
	{ 0x71ff, 0x71ff, PDF_CMAP_SINGLE, 12113 },
	{ 0x7201, 0x7201, PDF_CMAP_SINGLE, 12112 },
	{ 0x7202, 0x7202, PDF_CMAP_SINGLE, 12473 },
	{ 0x7203, 0x7203, PDF_CMAP_SINGLE, 12115 },
	{ 0x7205, 0x7205, PDF_CMAP_SINGLE, 12474 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 5521 },
	{ 0x7207, 0x7207, PDF_CMAP_SINGLE, 12472 },
	{ 0x7209, 0x7209, PDF_CMAP_SINGLE, 15803 },
	{ 0x720a, 0x720a, PDF_CMAP_SINGLE, 12471 },
	{ 0x720c, 0x720c, PDF_CMAP_SINGLE, 12470 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 5522 },
	{ 0x720e, 0x720e, PDF_CMAP_SINGLE, 17009 },
	{ 0x720f, 0x720f, PDF_CMAP_SINGLE, 14844 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 5663 },
	{ 0x7213, 0x7214, PDF_CMAP_RANGE, 12788 },
	{ 0x7215, 0x7215, PDF_CMAP_SINGLE, 17011 },
	{ 0x7216, 0x7216, PDF_CMAP_SINGLE, 14675 },
	{ 0x7217, 0x7217, PDF_CMAP_SINGLE, 14697 },
	{ 0x7219, 0x7219, PDF_CMAP_SINGLE, 13025 },
	{ 0x721a, 0x721a, PDF_CMAP_SINGLE, 13024 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 5755 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 13023 },
	{ 0x721e, 0x721f, PDF_CMAP_RANGE, 13197 },
	{ 0x7222, 0x7222, PDF_CMAP_SINGLE, 13348 },
	{ 0x7223, 0x7223, PDF_CMAP_SINGLE, 13452 },
	{ 0x7224, 0x7224, PDF_CMAP_SINGLE, 17014 },
	{ 0x7226, 0x7226, PDF_CMAP_SINGLE, 13521 },
	{ 0x7227, 0x7227, PDF_CMAP_SINGLE, 13621 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 5990 },
	{ 0x7229, 0x7229, PDF_CMAP_SINGLE, 13641 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 743 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 17650 },
	{ 0x722c, 0x722d, PDF_CMAP_RANGE, 1605 },
	{ 0x722e, 0x722e, PDF_CMAP_SINGLE, 17017 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 1937 },
	{ 0x7235, 0x7235, PDF_CMAP_SINGLE, 5132 },
	{ 0x7236, 0x7236, PDF_CMAP_SINGLE, 744 },
	{ 0x7238, 0x7238, PDF_CMAP_SINGLE, 1607 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 2339 },
	{ 0x723a, 0x723a, PDF_CMAP_SINGLE, 3701 },
	{ 0x723b, 0x723b, PDF_CMAP_SINGLE, 745 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 2839 },
	{ 0x723e, 0x723e, PDF_CMAP_SINGLE, 4135 },
	{ 0x723f, 0x723f, PDF_CMAP_SINGLE, 6029 },
	{ 0x7240, 0x7240, PDF_CMAP_SINGLE, 17018 },
	{ 0x7241, 0x7241, PDF_CMAP_SINGLE, 7022 },
	{ 0x7242, 0x7242, PDF_CMAP_SINGLE, 7469 },
	{ 0x7244, 0x7244, PDF_CMAP_SINGLE, 10018 },
	{ 0x7246, 0x7246, PDF_CMAP_SINGLE, 5133 },
	{ 0x7247, 0x7247, PDF_CMAP_SINGLE, 746 },
	{ 0x7248, 0x7248, PDF_CMAP_SINGLE, 1608 },
	{ 0x7249, 0x724a, PDF_CMAP_RANGE, 7023 },
	{ 0x724b, 0x724b, PDF_CMAP_SINGLE, 8701 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 3280 },
	{ 0x724f, 0x724f, PDF_CMAP_SINGLE, 9357 },
	{ 0x7250, 0x7250, PDF_CMAP_SINGLE, 15049 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 3702 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 10019 },
	{ 0x7255, 0x7255, PDF_CMAP_SINGLE, 17021 },
	{ 0x7256, 0x7256, PDF_CMAP_SINGLE, 4514 },
	{ 0x7257, 0x7257, PDF_CMAP_SINGLE, 17022 },
	{ 0x7258, 0x7258, PDF_CMAP_SINGLE, 5523 },
	{ 0x7259, 0x7259, PDF_CMAP_SINGLE, 747 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 8702 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 748 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 18010 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 1012 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 6155 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 1011 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 1260 },
	{ 0x7261, 0x7261, PDF_CMAP_SINGLE, 1259 },
	{ 0x7262, 0x7262, PDF_CMAP_SINGLE, 1258 },
	{ 0x7263, 0x7263, PDF_CMAP_SINGLE, 6341 },
	{ 0x7266, 0x7266, PDF_CMAP_SINGLE, 16059 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 1609 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 1610 },
	{ 0x726a, 0x726a, PDF_CMAP_SINGLE, 6650 },
	{ 0x726c, 0x726c, PDF_CMAP_SINGLE, 7025 },
	{ 0x726e, 0x726e, PDF_CMAP_SINGLE, 7028 },
	{ 0x726f, 0x726f, PDF_CMAP_SINGLE, 1939 },
	{ 0x7270, 0x7270, PDF_CMAP_SINGLE, 7026 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 1938 },
	{ 0x7273, 0x7273, PDF_CMAP_SINGLE, 7027 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1940 },
	{ 0x7276, 0x7276, PDF_CMAP_SINGLE, 7472 },
	{ 0x7277, 0x7277, PDF_CMAP_SINGLE, 7471 },
	{ 0x7278, 0x7278, PDF_CMAP_SINGLE, 7470 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 2340 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 8065 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 2840 },
	{ 0x727e, 0x727e, PDF_CMAP_SINGLE, 8064 },
	{ 0x727f, 0x727f, PDF_CMAP_SINGLE, 8067 },
	{ 0x7280, 0x7280, PDF_CMAP_SINGLE, 3282 },
	{ 0x7281, 0x7281, PDF_CMAP_SINGLE, 2841 },
	{ 0x7282, 0x7282, PDF_CMAP_SINGLE, 17027 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 3281 },
	{ 0x7285, 0x7285, PDF_CMAP_SINGLE, 8706 },
	{ 0x7286, 0x7286, PDF_CMAP_SINGLE, 8705 },
	{ 0x7287, 0x7287, PDF_CMAP_SINGLE, 15050 },
	{ 0x7288, 0x7289, PDF_CMAP_RANGE, 8703 },
	{ 0x728b, 0x728b, PDF_CMAP_SINGLE, 8707 },
	{ 0x728c, 0x728c, PDF_CMAP_SINGLE, 9359 },
	{ 0x728d, 0x728d, PDF_CMAP_SINGLE, 9358 },
	{ 0x728e, 0x728e, PDF_CMAP_SINGLE, 9362 },
	{ 0x728f, 0x728f, PDF_CMAP_SINGLE, 17353 },
	{ 0x7290, 0x7290, PDF_CMAP_SINGLE, 9361 },
	{ 0x7291, 0x7291, PDF_CMAP_SINGLE, 9360 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 4136 },
	{ 0x7293, 0x7293, PDF_CMAP_SINGLE, 10022 },
	{ 0x7294, 0x7294, PDF_CMAP_SINGLE, 15051 },
	{ 0x7295, 0x7295, PDF_CMAP_SINGLE, 10021 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 4137 },
	{ 0x7297, 0x7297, PDF_CMAP_SINGLE, 10020 },
	{ 0x7298, 0x7298, PDF_CMAP_SINGLE, 10598 },
	{ 0x729a, 0x729a, PDF_CMAP_SINGLE, 10599 },
	{ 0x729b, 0x729b, PDF_CMAP_SINGLE, 4515 },
	{ 0x729d, 0x729e, PDF_CMAP_RANGE, 11202 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 18234 },
	{ 0x72a1, 0x72a1, PDF_CMAP_SINGLE, 12479 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 5524 },
	{ 0x72a3, 0x72a3, PDF_CMAP_SINGLE, 12478 },
	{ 0x72a4, 0x72a4, PDF_CMAP_SINGLE, 12477 },
	{ 0x72a5, 0x72a6, PDF_CMAP_RANGE, 12475 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 5756 },
	{ 0x72a8, 0x72a8, PDF_CMAP_SINGLE, 12790 },
	{ 0x72a9, 0x72a9, PDF_CMAP_SINGLE, 13199 },
	{ 0x72aa, 0x72aa, PDF_CMAP_SINGLE, 13522 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 749 },
	{ 0x72ad, 0x72ad, PDF_CMAP_SINGLE, 17652 },
	{ 0x72ae, 0x72ae, PDF_CMAP_SINGLE, 6063 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 853 },
	{ 0x72b0, 0x72b0, PDF_CMAP_SINGLE, 6064 },
	{ 0x72b2, 0x72b2, PDF_CMAP_SINGLE, 16310 },
	{ 0x72b4, 0x72b5, PDF_CMAP_RANGE, 6156 },
	{ 0x72ba, 0x72ba, PDF_CMAP_SINGLE, 6347 },
	{ 0x72bd, 0x72bd, PDF_CMAP_SINGLE, 6343 },
	{ 0x72bf, 0x72bf, PDF_CMAP_SINGLE, 6342 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 1611 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 6346 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 1262 },
	{ 0x72c3, 0x72c3, PDF_CMAP_SINGLE, 6344 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 1261 },
	{ 0x72c5, 0x72c5, PDF_CMAP_SINGLE, 6348 },
	{ 0x72c6, 0x72c6, PDF_CMAP_SINGLE, 6345 },
	{ 0x72c9, 0x72c9, PDF_CMAP_SINGLE, 6654 },
	{ 0x72ca, 0x72ca, PDF_CMAP_SINGLE, 7029 },
	{ 0x72cb, 0x72cb, PDF_CMAP_SINGLE, 6652 },
	{ 0x72cc, 0x72cc, PDF_CMAP_SINGLE, 6659 },
	{ 0x72cd, 0x72cd, PDF_CMAP_SINGLE, 17354 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 1612 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1615 },
	{ 0x72d1, 0x72d1, PDF_CMAP_SINGLE, 6660 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 6656 },
	{ 0x72d4, 0x72d4, PDF_CMAP_SINGLE, 6657 },
	{ 0x72d6, 0x72d6, PDF_CMAP_SINGLE, 6651 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 1614 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 6653 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 1613 },
	{ 0x72da, 0x72da, PDF_CMAP_SINGLE, 6658 },
	{ 0x72dc, 0x72dc, PDF_CMAP_SINGLE, 6655 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 7033 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 1942 },
	{ 0x72e2, 0x72e2, PDF_CMAP_SINGLE, 16279 },
	{ 0x72e3, 0x72e3, PDF_CMAP_SINGLE, 7036 },
	{ 0x72e4, 0x72e4, PDF_CMAP_SINGLE, 7030 },
	{ 0x72e6, 0x72e6, PDF_CMAP_SINGLE, 7035 },
	{ 0x72e8, 0x72e8, PDF_CMAP_SINGLE, 7031 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 1941 },
	{ 0x72ea, 0x72ea, PDF_CMAP_SINGLE, 7034 },
	{ 0x72eb, 0x72eb, PDF_CMAP_SINGLE, 7032 },
	{ 0x72f3, 0x72f3, PDF_CMAP_SINGLE, 7478 },
	{ 0x72f4, 0x72f4, PDF_CMAP_SINGLE, 7475 },
	{ 0x72f6, 0x72f6, PDF_CMAP_SINGLE, 7477 },
	{ 0x72f7, 0x72f7, PDF_CMAP_SINGLE, 2345 },
	{ 0x72f8, 0x72f8, PDF_CMAP_SINGLE, 2344 },
	{ 0x72f9, 0x72f9, PDF_CMAP_SINGLE, 2342 },
	{ 0x72fa, 0x72fa, PDF_CMAP_SINGLE, 7474 },
	{ 0x72fb, 0x72fb, PDF_CMAP_SINGLE, 7479 },
	{ 0x72fc, 0x72fc, PDF_CMAP_SINGLE, 2341 },
	{ 0x72fd, 0x72fd, PDF_CMAP_SINGLE, 2343 },
	{ 0x72fe, 0x72fe, PDF_CMAP_SINGLE, 7476 },
	{ 0x72ff, 0x72ff, PDF_CMAP_SINGLE, 8075 },
	{ 0x7300, 0x7300, PDF_CMAP_SINGLE, 7473 },
	{ 0x7301, 0x7301, PDF_CMAP_SINGLE, 7480 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 16282 },
	{ 0x7304, 0x7304, PDF_CMAP_SINGLE, 16053 },
	{ 0x7307, 0x7307, PDF_CMAP_SINGLE, 8070 },
	{ 0x7308, 0x7308, PDF_CMAP_SINGLE, 8074 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 8073 },
	{ 0x730b, 0x730b, PDF_CMAP_SINGLE, 8709 },
	{ 0x730c, 0x730c, PDF_CMAP_SINGLE, 8720 },
	{ 0x730f, 0x730f, PDF_CMAP_SINGLE, 8076 },
	{ 0x7310, 0x7310, PDF_CMAP_SINGLE, 18154 },
	{ 0x7311, 0x7311, PDF_CMAP_SINGLE, 8071 },
	{ 0x7312, 0x7312, PDF_CMAP_SINGLE, 8708 },
	{ 0x7313, 0x7313, PDF_CMAP_SINGLE, 2845 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 2844 },
	{ 0x7317, 0x7317, PDF_CMAP_SINGLE, 8069 },
	{ 0x7318, 0x7318, PDF_CMAP_SINGLE, 8072 },
	{ 0x7319, 0x7319, PDF_CMAP_SINGLE, 2846 },
	{ 0x731b, 0x731b, PDF_CMAP_SINGLE, 2843 },
	{ 0x731c, 0x731c, PDF_CMAP_SINGLE, 2842 },
	{ 0x731d, 0x731d, PDF_CMAP_SINGLE, 8068 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 8077 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 8711 },
	{ 0x7323, 0x7323, PDF_CMAP_SINGLE, 8718 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 3284 },
	{ 0x7326, 0x7326, PDF_CMAP_SINGLE, 8717 },
	{ 0x7327, 0x7327, PDF_CMAP_SINGLE, 8714 },
	{ 0x7328, 0x7328, PDF_CMAP_SINGLE, 17034 },
	{ 0x7329, 0x7329, PDF_CMAP_SINGLE, 3286 },
	{ 0x732a, 0x732a, PDF_CMAP_SINGLE, 15728 },
	{ 0x732b, 0x732b, PDF_CMAP_SINGLE, 17029 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 16306 },
	{ 0x732d, 0x732d, PDF_CMAP_SINGLE, 8716 },
	{ 0x732e, 0x732e, PDF_CMAP_SINGLE, 17035 },
	{ 0x7330, 0x7330, PDF_CMAP_SINGLE, 8710 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 8712 },
	{ 0x7332, 0x7332, PDF_CMAP_SINGLE, 8715 },
	{ 0x7333, 0x7333, PDF_CMAP_SINGLE, 8713 },
	{ 0x7334, 0x7334, PDF_CMAP_SINGLE, 3285 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 8719 },
	{ 0x7336, 0x7336, PDF_CMAP_SINGLE, 3283 },
	{ 0x7337, 0x7337, PDF_CMAP_SINGLE, 3703 },
	{ 0x7338, 0x7338, PDF_CMAP_SINGLE, 18155 },
	{ 0x7339, 0x7339, PDF_CMAP_SINGLE, 17356 },
	{ 0x733a, 0x733a, PDF_CMAP_SINGLE, 9366 },
	{ 0x733b, 0x733b, PDF_CMAP_SINGLE, 9365 },
	{ 0x733c, 0x733c, PDF_CMAP_SINGLE, 9363 },
	{ 0x733e, 0x733e, PDF_CMAP_SINGLE, 3706 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 3705 },
	{ 0x7340, 0x7340, PDF_CMAP_SINGLE, 9367 },
	{ 0x7341, 0x7341, PDF_CMAP_SINGLE, 18156 },
	{ 0x7342, 0x7342, PDF_CMAP_SINGLE, 9364 },
	{ 0x7343, 0x7343, PDF_CMAP_SINGLE, 10023 },
	{ 0x7344, 0x7344, PDF_CMAP_SINGLE, 4138 },
	{ 0x7345, 0x7345, PDF_CMAP_SINGLE, 3704 },
	{ 0x7348, 0x7348, PDF_CMAP_SINGLE, 17402 },
	{ 0x7349, 0x7349, PDF_CMAP_SINGLE, 9369 },
	{ 0x734a, 0x734a, PDF_CMAP_SINGLE, 9368 },
	{ 0x734c, 0x734c, PDF_CMAP_SINGLE, 10026 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 10024 },
	{ 0x734e, 0x734e, PDF_CMAP_SINGLE, 4516 },
	{ 0x734f, 0x734f, PDF_CMAP_SINGLE, 16209 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 4139 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 10025 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 10601 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 4517 },
	{ 0x7358, 0x7358, PDF_CMAP_SINGLE, 10600 },
	{ 0x7359, 0x7359, PDF_CMAP_SINGLE, 10609 },
	{ 0x735a, 0x735a, PDF_CMAP_SINGLE, 10608 },
	{ 0x735b, 0x735b, PDF_CMAP_SINGLE, 10606 },
	{ 0x735d, 0x735d, PDF_CMAP_SINGLE, 10605 },
	{ 0x735e, 0x7360, PDF_CMAP_RANGE, 10602 },
	{ 0x7361, 0x7361, PDF_CMAP_SINGLE, 10607 },
	{ 0x7362, 0x7362, PDF_CMAP_SINGLE, 10610 },
	{ 0x7365, 0x7365, PDF_CMAP_SINGLE, 11208 },
	{ 0x7366, 0x7367, PDF_CMAP_RANGE, 11205 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 4857 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 11204 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 11210 },
	{ 0x736b, 0x736b, PDF_CMAP_SINGLE, 11209 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 11207 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 11683 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 5134 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 16280 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 5135 },
	{ 0x7373, 0x7373, PDF_CMAP_SINGLE, 11682 },
	{ 0x7374, 0x7374, PDF_CMAP_SINGLE, 16084 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 5363 },
	{ 0x7376, 0x7376, PDF_CMAP_SINGLE, 12117 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 5362 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 5525 },
	{ 0x737a, 0x737a, PDF_CMAP_SINGLE, 5526 },
	{ 0x737b, 0x737b, PDF_CMAP_SINGLE, 5664 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 12792 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 12791 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 13026 },
	{ 0x737f, 0x737f, PDF_CMAP_SINGLE, 13200 },
	{ 0x7380, 0x7380, PDF_CMAP_SINGLE, 5834 },
	{ 0x7381, 0x7381, PDF_CMAP_SINGLE, 13350 },
	{ 0x7382, 0x7382, PDF_CMAP_SINGLE, 13349 },
	{ 0x7383, 0x7383, PDF_CMAP_SINGLE, 13351 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 854 },
	{ 0x7385, 0x7385, PDF_CMAP_SINGLE, 7037 },
	{ 0x7386, 0x7386, PDF_CMAP_SINGLE, 2346 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 2847 },
	{ 0x7388, 0x7388, PDF_CMAP_SINGLE, 8078 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 855 },
	{ 0x738a, 0x738a, PDF_CMAP_SINGLE, 6065 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 750 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6158 },
	{ 0x738f, 0x738f, PDF_CMAP_SINGLE, 18149 },
	{ 0x7392, 0x7392, PDF_CMAP_SINGLE, 6353 },
	{ 0x7393, 0x7394, PDF_CMAP_RANGE, 6351 },
	{ 0x7395, 0x7395, PDF_CMAP_SINGLE, 6349 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 1263 },
	{ 0x7397, 0x7397, PDF_CMAP_SINGLE, 6350 },
	{ 0x7398, 0x7398, PDF_CMAP_SINGLE, 14838 },
	{ 0x739c, 0x739c, PDF_CMAP_SINGLE, 16522 },
	{ 0x739d, 0x739d, PDF_CMAP_SINGLE, 6668 },
	{ 0x739e, 0x739e, PDF_CMAP_SINGLE, 14683 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 1618 },
	{ 0x73a0, 0x73a0, PDF_CMAP_SINGLE, 6666 },
	{ 0x73a1, 0x73a1, PDF_CMAP_SINGLE, 6662 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 6665 },
	{ 0x73a4, 0x73a4, PDF_CMAP_SINGLE, 6661 },
	{ 0x73a5, 0x73a5, PDF_CMAP_SINGLE, 1620 },
	{ 0x73a6, 0x73a6, PDF_CMAP_SINGLE, 6664 },
	{ 0x73a7, 0x73a7, PDF_CMAP_SINGLE, 16519 },
	{ 0x73a8, 0x73a8, PDF_CMAP_SINGLE, 1617 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 1616 },
	{ 0x73aa, 0x73aa, PDF_CMAP_SINGLE, 17037 },
	{ 0x73ab, 0x73ab, PDF_CMAP_SINGLE, 1619 },
	{ 0x73ac, 0x73ac, PDF_CMAP_SINGLE, 6667 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 6663 },
	{ 0x73b2, 0x73b2, PDF_CMAP_SINGLE, 1947 },
	{ 0x73b3, 0x73b3, PDF_CMAP_SINGLE, 1950 },
	{ 0x73b4, 0x73b4, PDF_CMAP_SINGLE, 7045 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 7044 },
	{ 0x73b6, 0x73b6, PDF_CMAP_SINGLE, 7043 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 1944 },
	{ 0x73b8, 0x73b8, PDF_CMAP_SINGLE, 7052 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 7042 },
	{ 0x73bb, 0x73bb, PDF_CMAP_SINGLE, 1946 },
	{ 0x73bc, 0x73bc, PDF_CMAP_SINGLE, 7485 },
	{ 0x73be, 0x73be, PDF_CMAP_SINGLE, 7049 },
	{ 0x73bf, 0x73bf, PDF_CMAP_SINGLE, 7047 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 1949 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 7039 },
	{ 0x73c3, 0x73c3, PDF_CMAP_SINGLE, 7050 },
	{ 0x73c5, 0x73c5, PDF_CMAP_SINGLE, 7041 },
	{ 0x73c6, 0x73c6, PDF_CMAP_SINGLE, 7051 },
	{ 0x73c7, 0x73c7, PDF_CMAP_SINGLE, 7048 },
	{ 0x73c8, 0x73c8, PDF_CMAP_SINGLE, 7040 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 17040 },
	{ 0x73ca, 0x73ca, PDF_CMAP_SINGLE, 1945 },
	{ 0x73cb, 0x73cb, PDF_CMAP_SINGLE, 7053 },
	{ 0x73cc, 0x73cc, PDF_CMAP_SINGLE, 7038 },
	{ 0x73cd, 0x73cd, PDF_CMAP_SINGLE, 1948 },
	{ 0x73ce, 0x73ce, PDF_CMAP_SINGLE, 16667 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 17036 },
	{ 0x73d0, 0x73d0, PDF_CMAP_SINGLE, 15127 },
	{ 0x73d2, 0x73d2, PDF_CMAP_SINGLE, 7490 },
	{ 0x73d3, 0x73d3, PDF_CMAP_SINGLE, 7481 },
	{ 0x73d4, 0x73d4, PDF_CMAP_SINGLE, 7492 },
	{ 0x73d5, 0x73d5, PDF_CMAP_SINGLE, 16499 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 7484 },
	{ 0x73d7, 0x73d8, PDF_CMAP_RANGE, 7495 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 7482 },
	{ 0x73da, 0x73da, PDF_CMAP_SINGLE, 7494 },
	{ 0x73db, 0x73db, PDF_CMAP_SINGLE, 7491 },
	{ 0x73dc, 0x73dc, PDF_CMAP_SINGLE, 7489 },
	{ 0x73dd, 0x73dd, PDF_CMAP_SINGLE, 7493 },
	{ 0x73de, 0x73de, PDF_CMAP_SINGLE, 2352 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 2350 },
	{ 0x73e1, 0x73e1, PDF_CMAP_SINGLE, 18160 },
	{ 0x73e2, 0x73e2, PDF_CMAP_SINGLE, 15523 },
	{ 0x73e3, 0x73e3, PDF_CMAP_SINGLE, 7487 },
	{ 0x73e4, 0x73e4, PDF_CMAP_SINGLE, 17061 },
	{ 0x73e5, 0x73e5, PDF_CMAP_SINGLE, 7483 },
	{ 0x73e6, 0x73e6, PDF_CMAP_SINGLE, 15546 },
	{ 0x73e7, 0x73e7, PDF_CMAP_SINGLE, 7486 },
	{ 0x73e8, 0x73e8, PDF_CMAP_SINGLE, 7497 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 7488 },
	{ 0x73ea, 0x73ea, PDF_CMAP_SINGLE, 2351 },
	{ 0x73eb, 0x73eb, PDF_CMAP_SINGLE, 7046 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 2347 },
	{ 0x73ee, 0x73ee, PDF_CMAP_SINGLE, 2349 },
	{ 0x73ef, 0x73ef, PDF_CMAP_SINGLE, 16401 },
	{ 0x73f3, 0x73f3, PDF_CMAP_SINGLE, 15531 },
	{ 0x73f4, 0x73f4, PDF_CMAP_SINGLE, 8092 },
	{ 0x73f5, 0x73f5, PDF_CMAP_SINGLE, 8081 },
	{ 0x73f6, 0x73f6, PDF_CMAP_SINGLE, 8079 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 17050 },
	{ 0x73f8, 0x73f8, PDF_CMAP_SINGLE, 8080 },
	{ 0x73f9, 0x73f9, PDF_CMAP_SINGLE, 15353 },
	{ 0x73fa, 0x73fa, PDF_CMAP_SINGLE, 8087 },
	{ 0x73fb, 0x73fb, PDF_CMAP_SINGLE, 15532 },
	{ 0x73fc, 0x73fc, PDF_CMAP_SINGLE, 8088 },
	{ 0x73fd, 0x73fd, PDF_CMAP_SINGLE, 8084 },
	{ 0x73fe, 0x73fe, PDF_CMAP_SINGLE, 2852 },
	{ 0x73ff, 0x73ff, PDF_CMAP_SINGLE, 8089 },
	{ 0x7400, 0x7400, PDF_CMAP_SINGLE, 8086 },
	{ 0x7401, 0x7401, PDF_CMAP_SINGLE, 8083 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 2850 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 8082 },
	{ 0x7405, 0x7405, PDF_CMAP_SINGLE, 2848 },
	{ 0x7406, 0x7406, PDF_CMAP_SINGLE, 2851 },
	{ 0x7407, 0x7407, PDF_CMAP_SINGLE, 8085 },
	{ 0x7408, 0x7408, PDF_CMAP_SINGLE, 8093 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 2348 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 2849 },
	{ 0x740b, 0x740b, PDF_CMAP_SINGLE, 8091 },
	{ 0x740c, 0x740c, PDF_CMAP_SINGLE, 8090 },
	{ 0x740d, 0x740d, PDF_CMAP_SINGLE, 2853 },
	{ 0x7411, 0x7411, PDF_CMAP_SINGLE, 17772 },
	{ 0x7412, 0x7412, PDF_CMAP_SINGLE, 15334 },
	{ 0x7414, 0x7414, PDF_CMAP_SINGLE, 14836 },
	{ 0x7415, 0x7415, PDF_CMAP_SINGLE, 17051 },
	{ 0x7416, 0x7416, PDF_CMAP_SINGLE, 8725 },
	{ 0x7417, 0x7417, PDF_CMAP_SINGLE, 16557 },
	{ 0x7419, 0x7419, PDF_CMAP_SINGLE, 18163 },
	{ 0x741a, 0x741a, PDF_CMAP_SINGLE, 8726 },
	{ 0x741b, 0x741b, PDF_CMAP_SINGLE, 3296 },
	{ 0x741c, 0x741c, PDF_CMAP_SINGLE, 14557 },
	{ 0x741d, 0x741d, PDF_CMAP_SINGLE, 8732 },
	{ 0x741e, 0x741f, PDF_CMAP_RANGE, 18164 },
	{ 0x7420, 0x7420, PDF_CMAP_SINGLE, 8734 },
	{ 0x7421, 0x7421, PDF_CMAP_SINGLE, 8727 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 3290 },
	{ 0x7423, 0x7423, PDF_CMAP_SINGLE, 8731 },
	{ 0x7424, 0x7424, PDF_CMAP_SINGLE, 8730 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 3291 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 3297 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 3298 },
	{ 0x7429, 0x7429, PDF_CMAP_SINGLE, 8733 },
	{ 0x742a, 0x742a, PDF_CMAP_SINGLE, 3288 },
	{ 0x742b, 0x742b, PDF_CMAP_SINGLE, 8724 },
	{ 0x742c, 0x742c, PDF_CMAP_SINGLE, 8722 },
	{ 0x742d, 0x742d, PDF_CMAP_SINGLE, 8728 },
	{ 0x742e, 0x742e, PDF_CMAP_SINGLE, 8721 },
	{ 0x742f, 0x742f, PDF_CMAP_SINGLE, 3295 },
	{ 0x7430, 0x7430, PDF_CMAP_SINGLE, 8723 },
	{ 0x7431, 0x7431, PDF_CMAP_SINGLE, 8729 },
	{ 0x7432, 0x7432, PDF_CMAP_SINGLE, 8735 },
	{ 0x7433, 0x7433, PDF_CMAP_SINGLE, 3289 },
	{ 0x7434, 0x7434, PDF_CMAP_SINGLE, 3294 },
	{ 0x7435, 0x7436, PDF_CMAP_RANGE, 3292 },
	{ 0x7437, 0x7437, PDF_CMAP_SINGLE, 15073 },
	{ 0x7438, 0x7438, PDF_CMAP_SINGLE, 15113 },
	{ 0x7439, 0x7439, PDF_CMAP_SINGLE, 17054 },
	{ 0x743a, 0x743a, PDF_CMAP_SINGLE, 3287 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 15071 },
	{ 0x743f, 0x743f, PDF_CMAP_SINGLE, 3713 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 9376 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 3712 },
	{ 0x7442, 0x7442, PDF_CMAP_SINGLE, 9380 },
	{ 0x7443, 0x7443, PDF_CMAP_SINGLE, 14874 },
	{ 0x7444, 0x7444, PDF_CMAP_SINGLE, 9370 },
	{ 0x7445, 0x7445, PDF_CMAP_SINGLE, 15319 },
	{ 0x7446, 0x7446, PDF_CMAP_SINGLE, 9381 },
	{ 0x7447, 0x7447, PDF_CMAP_SINGLE, 17060 },
	{ 0x7448, 0x7448, PDF_CMAP_SINGLE, 15541 },
	{ 0x7449, 0x7449, PDF_CMAP_SINGLE, 17041 },
	{ 0x744a, 0x744b, PDF_CMAP_RANGE, 9371 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 9382 },
	{ 0x744e, 0x744e, PDF_CMAP_SINGLE, 9379 },
	{ 0x744f, 0x7450, PDF_CMAP_RANGE, 9377 },
	{ 0x7451, 0x7451, PDF_CMAP_SINGLE, 9374 },
	{ 0x7452, 0x7452, PDF_CMAP_SINGLE, 9373 },
	{ 0x7453, 0x7453, PDF_CMAP_SINGLE, 16945 },
	{ 0x7454, 0x7454, PDF_CMAP_SINGLE, 9383 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 3709 },
	{ 0x7456, 0x7456, PDF_CMAP_SINGLE, 14837 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 9375 },
	{ 0x7459, 0x7459, PDF_CMAP_SINGLE, 3714 },
	{ 0x745a, 0x745a, PDF_CMAP_SINGLE, 3708 },
	{ 0x745b, 0x745c, PDF_CMAP_RANGE, 3715 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 17094 },
	{ 0x745e, 0x745e, PDF_CMAP_SINGLE, 3711 },
	{ 0x745f, 0x745f, PDF_CMAP_SINGLE, 3710 },
	{ 0x7460, 0x7460, PDF_CMAP_SINGLE, 17058 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 10027 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 4141 },
	{ 0x7464, 0x7464, PDF_CMAP_SINGLE, 4140 },
	{ 0x7465, 0x7465, PDF_CMAP_SINGLE, 16641 },
	{ 0x7467, 0x7467, PDF_CMAP_SINGLE, 10032 },
	{ 0x7468, 0x7468, PDF_CMAP_SINGLE, 15086 },
	{ 0x7469, 0x7469, PDF_CMAP_SINGLE, 4518 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 4142 },
	{ 0x746b, 0x746b, PDF_CMAP_SINGLE, 15335 },
	{ 0x746c, 0x746c, PDF_CMAP_SINGLE, 17064 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 4144 },
	{ 0x746e, 0x746e, PDF_CMAP_SINGLE, 10033 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 3707 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 4143 },
	{ 0x7471, 0x7471, PDF_CMAP_SINGLE, 10029 },
	{ 0x7472, 0x7472, PDF_CMAP_SINGLE, 10031 },
	{ 0x7473, 0x7473, PDF_CMAP_SINGLE, 10028 },
	{ 0x7474, 0x7474, PDF_CMAP_SINGLE, 17065 },
	{ 0x7475, 0x7475, PDF_CMAP_SINGLE, 10030 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 17062 },
	{ 0x7479, 0x7479, PDF_CMAP_SINGLE, 10620 },
	{ 0x747a, 0x747a, PDF_CMAP_SINGLE, 15057 },
	{ 0x747c, 0x747c, PDF_CMAP_SINGLE, 10619 },
	{ 0x747d, 0x747d, PDF_CMAP_SINGLE, 10616 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 4521 },
	{ 0x747f, 0x747f, PDF_CMAP_SINGLE, 11211 },
	{ 0x7480, 0x7480, PDF_CMAP_SINGLE, 4522 },
	{ 0x7481, 0x7481, PDF_CMAP_SINGLE, 10615 },
	{ 0x7482, 0x7482, PDF_CMAP_SINGLE, 17068 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 4520 },
	{ 0x7485, 0x7485, PDF_CMAP_SINGLE, 10617 },
	{ 0x7486, 0x7486, PDF_CMAP_SINGLE, 10614 },
	{ 0x7487, 0x7487, PDF_CMAP_SINGLE, 10611 },
	{ 0x7488, 0x7488, PDF_CMAP_SINGLE, 10618 },
	{ 0x7489, 0x748a, PDF_CMAP_RANGE, 10612 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 4519 },
	{ 0x748c, 0x748c, PDF_CMAP_SINGLE, 14881 },
	{ 0x748d, 0x748d, PDF_CMAP_SINGLE, 18077 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 11688 },
	{ 0x7492, 0x7492, PDF_CMAP_SINGLE, 11215 },
	{ 0x7494, 0x7494, PDF_CMAP_SINGLE, 11214 },
	{ 0x7495, 0x7495, PDF_CMAP_SINGLE, 11216 },
	{ 0x7497, 0x7497, PDF_CMAP_SINGLE, 11685 },
	{ 0x7498, 0x7498, PDF_CMAP_SINGLE, 4860 },
	{ 0x7499, 0x7499, PDF_CMAP_SINGLE, 14835 },
	{ 0x749a, 0x749a, PDF_CMAP_SINGLE, 11212 },
	{ 0x749b, 0x749b, PDF_CMAP_SINGLE, 15114 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 4858 },
	{ 0x749e, 0x749e, PDF_CMAP_SINGLE, 4862 },
	{ 0x749f, 0x749f, PDF_CMAP_SINGLE, 4861 },
	{ 0x74a0, 0x74a0, PDF_CMAP_SINGLE, 11213 },
	{ 0x74a1, 0x74a1, PDF_CMAP_SINGLE, 11217 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 4859 },
	{ 0x74a4, 0x74a4, PDF_CMAP_SINGLE, 16752 },
	{ 0x74a5, 0x74a5, PDF_CMAP_SINGLE, 11692 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 5138 },
	{ 0x74a7, 0x74a7, PDF_CMAP_SINGLE, 5364 },
	{ 0x74a8, 0x74a8, PDF_CMAP_SINGLE, 5139 },
	{ 0x74a9, 0x74a9, PDF_CMAP_SINGLE, 5136 },
	{ 0x74aa, 0x74aa, PDF_CMAP_SINGLE, 11689 },
	{ 0x74ab, 0x74ab, PDF_CMAP_SINGLE, 11687 },
	{ 0x74ad, 0x74ad, PDF_CMAP_SINGLE, 11690 },
	{ 0x74af, 0x74af, PDF_CMAP_SINGLE, 11693 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 5137 },
	{ 0x74b1, 0x74b1, PDF_CMAP_SINGLE, 11691 },
	{ 0x74b2, 0x74b2, PDF_CMAP_SINGLE, 11686 },
	{ 0x74b4, 0x74b4, PDF_CMAP_SINGLE, 15525 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 12120 },
	{ 0x74b6, 0x74b6, PDF_CMAP_SINGLE, 12123 },
	{ 0x74b7, 0x74b7, PDF_CMAP_SINGLE, 12482 },
	{ 0x74b8, 0x74b8, PDF_CMAP_SINGLE, 12118 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 12793 },
	{ 0x74bb, 0x74bb, PDF_CMAP_SINGLE, 12124 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 5527 },
	{ 0x74be, 0x74be, PDF_CMAP_SINGLE, 12122 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 5365 },
	{ 0x74c0, 0x74c0, PDF_CMAP_SINGLE, 12119 },
	{ 0x74c1, 0x74c1, PDF_CMAP_SINGLE, 12121 },
	{ 0x74c2, 0x74c2, PDF_CMAP_SINGLE, 12125 },
	{ 0x74c3, 0x74c3, PDF_CMAP_SINGLE, 12483 },
	{ 0x74c5, 0x74c5, PDF_CMAP_SINGLE, 12481 },
	{ 0x74c8, 0x74c8, PDF_CMAP_SINGLE, 17077 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 5528 },
	{ 0x74cb, 0x74cb, PDF_CMAP_SINGLE, 12480 },
	{ 0x74cc, 0x74cc, PDF_CMAP_SINGLE, 15070 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 5665 },
	{ 0x74d0, 0x74d0, PDF_CMAP_SINGLE, 15003 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 18170 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 5758 },
	{ 0x74d5, 0x74d5, PDF_CMAP_SINGLE, 13202 },
	{ 0x74d6, 0x74d6, PDF_CMAP_SINGLE, 5757 },
	{ 0x74d7, 0x74d7, PDF_CMAP_SINGLE, 13204 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 13201 },
	{ 0x74d9, 0x74d9, PDF_CMAP_SINGLE, 13203 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 5886 },
	{ 0x74db, 0x74db, PDF_CMAP_SINGLE, 13453 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 856 },
	{ 0x74dd, 0x74dd, PDF_CMAP_SINGLE, 6669 },
	{ 0x74de, 0x74df, PDF_CMAP_RANGE, 7498 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 2854 },
	{ 0x74e1, 0x74e1, PDF_CMAP_SINGLE, 9384 },
	{ 0x74e2, 0x74e2, PDF_CMAP_SINGLE, 4863 },
	{ 0x74e3, 0x74e3, PDF_CMAP_SINGLE, 5529 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 5835 },
	{ 0x74e5, 0x74e5, PDF_CMAP_SINGLE, 13454 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 857 },
	{ 0x74e7, 0x74e7, PDF_CMAP_SINGLE, 16382 },
	{ 0x74e8, 0x74e8, PDF_CMAP_SINGLE, 6670 },
	{ 0x74e9, 0x74e9, PDF_CMAP_SINGLE, 288 },
	{ 0x74ec, 0x74ec, PDF_CMAP_SINGLE, 7054 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 7055 },
	{ 0x74f0, 0x74f0, PDF_CMAP_SINGLE, 16358 },
	{ 0x74f1, 0x74f1, PDF_CMAP_SINGLE, 16283 },
	{ 0x74f2, 0x74f2, PDF_CMAP_SINGLE, 16351 },
	{ 0x74f4, 0x74f5, PDF_CMAP_RANGE, 7500 },
	{ 0x74f6, 0x74f7, PDF_CMAP_RANGE, 2855 },
	{ 0x74f8, 0x74f8, PDF_CMAP_SINGLE, 16372 },
	{ 0x74fb, 0x74fb, PDF_CMAP_SINGLE, 8736 },
	{ 0x74fd, 0x74fd, PDF_CMAP_SINGLE, 9387 },
	{ 0x74fe, 0x74fe, PDF_CMAP_SINGLE, 9386 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 9385 },
	{ 0x7500, 0x7500, PDF_CMAP_SINGLE, 10034 },
	{ 0x7502, 0x7503, PDF_CMAP_RANGE, 10035 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 4145 },
	{ 0x7505, 0x7505, PDF_CMAP_SINGLE, 16350 },
	{ 0x7507, 0x7507, PDF_CMAP_SINGLE, 10622 },
	{ 0x7508, 0x7508, PDF_CMAP_SINGLE, 10621 },
	{ 0x750b, 0x750b, PDF_CMAP_SINGLE, 11218 },
	{ 0x750c, 0x750d, PDF_CMAP_RANGE, 4864 },
	{ 0x750e, 0x750e, PDF_CMAP_SINGLE, 17079 },
	{ 0x750f, 0x750f, PDF_CMAP_SINGLE, 11697 },
	{ 0x7510, 0x7512, PDF_CMAP_RANGE, 11694 },
	{ 0x7513, 0x7513, PDF_CMAP_SINGLE, 12127 },
	{ 0x7514, 0x7514, PDF_CMAP_SINGLE, 12126 },
	{ 0x7515, 0x7515, PDF_CMAP_SINGLE, 5366 },
	{ 0x7516, 0x7516, PDF_CMAP_SINGLE, 12484 },
	{ 0x7517, 0x7517, PDF_CMAP_SINGLE, 13027 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 858 },
	{ 0x7519, 0x7519, PDF_CMAP_SINGLE, 17795 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 1951 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 2857 },
	{ 0x751d, 0x751d, PDF_CMAP_SINGLE, 9388 },
	{ 0x751e, 0x751e, PDF_CMAP_SINGLE, 17080 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 859 },
	{ 0x7521, 0x7521, PDF_CMAP_SINGLE, 7502 },
	{ 0x7522, 0x7522, PDF_CMAP_SINGLE, 2858 },
	{ 0x7525, 0x7526, PDF_CMAP_RANGE, 3299 },
	{ 0x7528, 0x7529, PDF_CMAP_RANGE, 860 },
	{ 0x752a, 0x752a, PDF_CMAP_SINGLE, 6159 },
	{ 0x752b, 0x752b, PDF_CMAP_SINGLE, 1265 },
	{ 0x752c, 0x752c, PDF_CMAP_SINGLE, 1264 },
	{ 0x752d, 0x752d, PDF_CMAP_SINGLE, 1952 },
	{ 0x752e, 0x752e, PDF_CMAP_SINGLE, 7056 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 8737 },
	{ 0x7530, 0x7533, PDF_CMAP_RANGE, 862 },
	{ 0x7534, 0x7534, PDF_CMAP_SINGLE, 16007 },
	{ 0x7535, 0x7535, PDF_CMAP_SINGLE, 17754 },
	{ 0x7537, 0x7538, PDF_CMAP_RANGE, 1266 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 6355 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 6354 },
	{ 0x753b, 0x753b, PDF_CMAP_SINGLE, 15867 },
	{ 0x753d, 0x753d, PDF_CMAP_SINGLE, 1621 },
	{ 0x753e, 0x753e, PDF_CMAP_SINGLE, 6673 },
	{ 0x753f, 0x7540, PDF_CMAP_RANGE, 6671 },
	{ 0x7542, 0x7542, PDF_CMAP_SINGLE, 17357 },
	{ 0x7546, 0x7546, PDF_CMAP_SINGLE, 16192 },
	{ 0x7547, 0x7548, PDF_CMAP_RANGE, 7057 },
	{ 0x754a, 0x754a, PDF_CMAP_SINGLE, 17087 },
	{ 0x754b, 0x754b, PDF_CMAP_SINGLE, 1956 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 1954 },
	{ 0x754d, 0x754d, PDF_CMAP_SINGLE, 17086 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 1955 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 1953 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 15614 },
	{ 0x7553, 0x7553, PDF_CMAP_SINGLE, 16363 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 2353 },
	{ 0x7555, 0x7555, PDF_CMAP_SINGLE, 15082 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 2357 },
	{ 0x755a, 0x755a, PDF_CMAP_SINGLE, 2356 },
	{ 0x755b, 0x755b, PDF_CMAP_SINGLE, 7503 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 2355 },
	{ 0x755d, 0x755d, PDF_CMAP_SINGLE, 2354 },
	{ 0x755f, 0x755f, PDF_CMAP_SINGLE, 7504 },
	{ 0x7560, 0x7560, PDF_CMAP_SINGLE, 15729 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 2861 },
	{ 0x7563, 0x7563, PDF_CMAP_SINGLE, 8095 },
	{ 0x7564, 0x7564, PDF_CMAP_SINGLE, 8094 },
	{ 0x7565, 0x7566, PDF_CMAP_RANGE, 2859 },
	{ 0x7567, 0x7567, PDF_CMAP_SINGLE, 17088 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 3302 },
	{ 0x756b, 0x756b, PDF_CMAP_SINGLE, 3301 },
	{ 0x756c, 0x756c, PDF_CMAP_SINGLE, 8739 },
	{ 0x756d, 0x756d, PDF_CMAP_SINGLE, 18173 },
	{ 0x756e, 0x756e, PDF_CMAP_SINGLE, 17089 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 8738 },
	{ 0x7570, 0x7570, PDF_CMAP_SINGLE, 2862 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 18174 },
	{ 0x7576, 0x7576, PDF_CMAP_SINGLE, 3717 },
	{ 0x7577, 0x7577, PDF_CMAP_SINGLE, 9390 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 3718 },
	{ 0x7579, 0x7579, PDF_CMAP_SINGLE, 9389 },
	{ 0x757a, 0x757a, PDF_CMAP_SINGLE, 16207 },
	{ 0x757d, 0x757d, PDF_CMAP_SINGLE, 10037 },
	{ 0x757e, 0x757e, PDF_CMAP_SINGLE, 10623 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 4523 },
	{ 0x7580, 0x7580, PDF_CMAP_SINGLE, 11219 },
	{ 0x7583, 0x7583, PDF_CMAP_SINGLE, 15077 },
	{ 0x7584, 0x7584, PDF_CMAP_SINGLE, 11698 },
	{ 0x7586, 0x7586, PDF_CMAP_SINGLE, 5531 },
	{ 0x7587, 0x7587, PDF_CMAP_SINGLE, 5530 },
	{ 0x758a, 0x758a, PDF_CMAP_SINGLE, 5836 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 866 },
	{ 0x758c, 0x758c, PDF_CMAP_SINGLE, 6674 },
	{ 0x758d, 0x758d, PDF_CMAP_SINGLE, 18175 },
	{ 0x758e, 0x758e, PDF_CMAP_SINGLE, 17093 },
	{ 0x758f, 0x758f, PDF_CMAP_SINGLE, 2863 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 10038 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 4146 },
	{ 0x7592, 0x7592, PDF_CMAP_SINGLE, 558 },
	{ 0x7594, 0x7595, PDF_CMAP_RANGE, 6356 },
	{ 0x7598, 0x7598, PDF_CMAP_SINGLE, 6675 },
	{ 0x7599, 0x759a, PDF_CMAP_RANGE, 1623 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 1622 },
	{ 0x759e, 0x759e, PDF_CMAP_SINGLE, 17095 },
	{ 0x75a2, 0x75a3, PDF_CMAP_RANGE, 1960 },
	{ 0x75a4, 0x75a5, PDF_CMAP_RANGE, 1958 },
	{ 0x75a7, 0x75a7, PDF_CMAP_SINGLE, 7059 },
	{ 0x75aa, 0x75aa, PDF_CMAP_SINGLE, 7060 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 1957 },
	{ 0x75b0, 0x75b0, PDF_CMAP_SINGLE, 7505 },
	{ 0x75b1, 0x75b1, PDF_CMAP_SINGLE, 16045 },
	{ 0x75b2, 0x75b3, PDF_CMAP_RANGE, 2361 },
	{ 0x75b4, 0x75b4, PDF_CMAP_SINGLE, 17096 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 2866 },
	{ 0x75b6, 0x75b6, PDF_CMAP_SINGLE, 7511 },
	{ 0x75b8, 0x75b8, PDF_CMAP_SINGLE, 2367 },
	{ 0x75b9, 0x75b9, PDF_CMAP_SINGLE, 2365 },
	{ 0x75ba, 0x75ba, PDF_CMAP_SINGLE, 7512 },
	{ 0x75bb, 0x75bb, PDF_CMAP_SINGLE, 7507 },
	{ 0x75bc, 0x75bc, PDF_CMAP_SINGLE, 2364 },
	{ 0x75bd, 0x75bd, PDF_CMAP_SINGLE, 2363 },
	{ 0x75be, 0x75be, PDF_CMAP_SINGLE, 2358 },
	{ 0x75bf, 0x75bf, PDF_CMAP_SINGLE, 7510 },
	{ 0x75c0, 0x75c0, PDF_CMAP_SINGLE, 7509 },
	{ 0x75c1, 0x75c1, PDF_CMAP_SINGLE, 7506 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 2366 },
	{ 0x75c3, 0x75c3, PDF_CMAP_SINGLE, 16275 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 7508 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 2359 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 2360 },
	{ 0x75c8, 0x75c8, PDF_CMAP_SINGLE, 18177 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 2867 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 8099 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 2868 },
	{ 0x75ce, 0x75ce, PDF_CMAP_SINGLE, 8096 },
	{ 0x75cf, 0x75cf, PDF_CMAP_SINGLE, 8098 },
	{ 0x75d0, 0x75d0, PDF_CMAP_SINGLE, 8102 },
	{ 0x75d1, 0x75d1, PDF_CMAP_SINGLE, 8101 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 8097 },
	{ 0x75d4, 0x75d5, PDF_CMAP_RANGE, 2864 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 8747 },
	{ 0x75d8, 0x75d8, PDF_CMAP_SINGLE, 3307 },
	{ 0x75d9, 0x75d9, PDF_CMAP_SINGLE, 3306 },
	{ 0x75da, 0x75da, PDF_CMAP_SINGLE, 8741 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 3304 },
	{ 0x75dc, 0x75dc, PDF_CMAP_SINGLE, 17358 },
	{ 0x75dd, 0x75dd, PDF_CMAP_SINGLE, 8744 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 3308 },
	{ 0x75df, 0x75df, PDF_CMAP_SINGLE, 8745 },
	{ 0x75e0, 0x75e0, PDF_CMAP_SINGLE, 3309 },
	{ 0x75e1, 0x75e1, PDF_CMAP_SINGLE, 8742 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 3303 },
	{ 0x75e3, 0x75e3, PDF_CMAP_SINGLE, 3305 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 8746 },
	{ 0x75e6, 0x75e6, PDF_CMAP_SINGLE, 8743 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 8740 },
	{ 0x75ed, 0x75ed, PDF_CMAP_SINGLE, 9403 },
	{ 0x75ef, 0x75ef, PDF_CMAP_SINGLE, 9392 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 3720 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 3723 },
	{ 0x75f2, 0x75f2, PDF_CMAP_SINGLE, 3722 },
	{ 0x75f3, 0x75f3, PDF_CMAP_SINGLE, 3727 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 3726 },
	{ 0x75f5, 0x75f5, PDF_CMAP_SINGLE, 9404 },
	{ 0x75f6, 0x75f6, PDF_CMAP_SINGLE, 9402 },
	{ 0x75f7, 0x75f7, PDF_CMAP_SINGLE, 9395 },
	{ 0x75f8, 0x75f8, PDF_CMAP_SINGLE, 9399 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 9398 },
	{ 0x75fa, 0x75fa, PDF_CMAP_SINGLE, 3724 },
	{ 0x75fb, 0x75fb, PDF_CMAP_SINGLE, 9401 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 9397 },
	{ 0x75fd, 0x75fd, PDF_CMAP_SINGLE, 9405 },
	{ 0x75fe, 0x75fe, PDF_CMAP_SINGLE, 9396 },
	{ 0x75ff, 0x75ff, PDF_CMAP_SINGLE, 3725 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 3719 },
	{ 0x7601, 0x7601, PDF_CMAP_SINGLE, 3721 },
	{ 0x7602, 0x7602, PDF_CMAP_SINGLE, 17097 },
	{ 0x7603, 0x7603, PDF_CMAP_SINGLE, 9394 },
	{ 0x7607, 0x7607, PDF_CMAP_SINGLE, 16271 },
	{ 0x7608, 0x7608, PDF_CMAP_SINGLE, 10040 },
	{ 0x7609, 0x7609, PDF_CMAP_SINGLE, 4150 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 10044 },
	{ 0x760b, 0x760b, PDF_CMAP_SINGLE, 4149 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 10041 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 4148 },
	{ 0x760f, 0x760f, PDF_CMAP_SINGLE, 9393 },
	{ 0x7610, 0x7610, PDF_CMAP_SINGLE, 9400 },
	{ 0x7611, 0x7611, PDF_CMAP_SINGLE, 10043 },
	{ 0x7613, 0x7613, PDF_CMAP_SINGLE, 4151 },
	{ 0x7614, 0x7614, PDF_CMAP_SINGLE, 10045 },
	{ 0x7615, 0x7615, PDF_CMAP_SINGLE, 10042 },
	{ 0x7616, 0x7616, PDF_CMAP_SINGLE, 10039 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 10626 },
	{ 0x761a, 0x761a, PDF_CMAP_SINGLE, 10630 },
	{ 0x761b, 0x761b, PDF_CMAP_SINGLE, 10632 },
	{ 0x761c, 0x761c, PDF_CMAP_SINGLE, 10628 },
	{ 0x761d, 0x761d, PDF_CMAP_SINGLE, 10627 },
	{ 0x761e, 0x761e, PDF_CMAP_SINGLE, 10625 },
	{ 0x761f, 0x761f, PDF_CMAP_SINGLE, 4526 },
	{ 0x7620, 0x7620, PDF_CMAP_SINGLE, 4524 },
	{ 0x7621, 0x7622, PDF_CMAP_RANGE, 4529 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 10629 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 4527 },
	{ 0x7625, 0x7625, PDF_CMAP_SINGLE, 10624 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 4528 },
	{ 0x7627, 0x7627, PDF_CMAP_SINGLE, 4147 },
	{ 0x7628, 0x7628, PDF_CMAP_SINGLE, 10631 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 4525 },
	{ 0x762c, 0x762c, PDF_CMAP_SINGLE, 17098 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 11221 },
	{ 0x762f, 0x762f, PDF_CMAP_SINGLE, 11220 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 11228 },
	{ 0x7631, 0x7631, PDF_CMAP_SINGLE, 11222 },
	{ 0x7632, 0x7632, PDF_CMAP_SINGLE, 11227 },
	{ 0x7633, 0x7633, PDF_CMAP_SINGLE, 11224 },
	{ 0x7634, 0x7634, PDF_CMAP_SINGLE, 4866 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 11226 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 4867 },
	{ 0x763a, 0x763a, PDF_CMAP_SINGLE, 4868 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 15984 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 11225 },
	{ 0x763d, 0x763d, PDF_CMAP_SINGLE, 11223 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 16102 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 5141 },
	{ 0x7643, 0x7643, PDF_CMAP_SINGLE, 11699 },
	{ 0x7646, 0x7646, PDF_CMAP_SINGLE, 5140 },
	{ 0x7647, 0x7647, PDF_CMAP_SINGLE, 11702 },
	{ 0x7648, 0x7649, PDF_CMAP_RANGE, 11700 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 5142 },
	{ 0x764d, 0x764d, PDF_CMAP_SINGLE, 18178 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 15986 },
	{ 0x764f, 0x764f, PDF_CMAP_SINGLE, 17100 },
	{ 0x7650, 0x7650, PDF_CMAP_SINGLE, 12131 },
	{ 0x7651, 0x7651, PDF_CMAP_SINGLE, 17099 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 5369 },
	{ 0x7653, 0x7653, PDF_CMAP_SINGLE, 12132 },
	{ 0x7654, 0x7654, PDF_CMAP_SINGLE, 16112 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 5367 },
	{ 0x7657, 0x7657, PDF_CMAP_SINGLE, 12133 },
	{ 0x7658, 0x7658, PDF_CMAP_SINGLE, 5368 },
	{ 0x7659, 0x7659, PDF_CMAP_SINGLE, 12130 },
	{ 0x765a, 0x765a, PDF_CMAP_SINGLE, 12134 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 12128 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 5532 },
	{ 0x7660, 0x7660, PDF_CMAP_SINGLE, 12485 },
	{ 0x7661, 0x7661, PDF_CMAP_SINGLE, 5533 },
	{ 0x7662, 0x7662, PDF_CMAP_SINGLE, 5666 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 12129 },
	{ 0x7665, 0x7665, PDF_CMAP_SINGLE, 5667 },
	{ 0x7666, 0x7666, PDF_CMAP_SINGLE, 16042 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 16297 },
	{ 0x7669, 0x7669, PDF_CMAP_SINGLE, 5759 },
	{ 0x766a, 0x766a, PDF_CMAP_SINGLE, 13028 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 5838 },
	{ 0x766d, 0x766d, PDF_CMAP_SINGLE, 13205 },
	{ 0x766e, 0x766e, PDF_CMAP_SINGLE, 5837 },
	{ 0x766f, 0x766f, PDF_CMAP_SINGLE, 17101 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 13352 },
	{ 0x7671, 0x7672, PDF_CMAP_RANGE, 5923 },
	{ 0x7673, 0x7673, PDF_CMAP_SINGLE, 15083 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 18179 },
	{ 0x7675, 0x7675, PDF_CMAP_SINGLE, 13635 },
	{ 0x7676, 0x7676, PDF_CMAP_SINGLE, 559 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 1962 },
	{ 0x7679, 0x7679, PDF_CMAP_SINGLE, 7061 },
	{ 0x767a, 0x767a, PDF_CMAP_SINGLE, 18181 },
	{ 0x767b, 0x767c, PDF_CMAP_RANGE, 3310 },
	{ 0x767d, 0x767d, PDF_CMAP_SINGLE, 867 },
	{ 0x767e, 0x767e, PDF_CMAP_SINGLE, 1013 },
	{ 0x767f, 0x767f, PDF_CMAP_SINGLE, 6160 },
	{ 0x7680, 0x7680, PDF_CMAP_SINGLE, 17363 },
	{ 0x7681, 0x7681, PDF_CMAP_SINGLE, 6358 },
	{ 0x7682, 0x7682, PDF_CMAP_SINGLE, 1268 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 1625 },
	{ 0x7686, 0x7688, PDF_CMAP_RANGE, 1963 },
	{ 0x7689, 0x7689, PDF_CMAP_SINGLE, 8104 },
	{ 0x768a, 0x768a, PDF_CMAP_SINGLE, 7513 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 2368 },
	{ 0x768c, 0x768c, PDF_CMAP_SINGLE, 17364 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 2869 },
	{ 0x768f, 0x768f, PDF_CMAP_SINGLE, 8103 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 17104 },
	{ 0x7692, 0x7692, PDF_CMAP_SINGLE, 8749 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 3313 },
	{ 0x7695, 0x7695, PDF_CMAP_SINGLE, 8748 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 3312 },
	{ 0x7699, 0x7699, PDF_CMAP_SINGLE, 9406 },
	{ 0x769a, 0x769a, PDF_CMAP_SINGLE, 4531 },
	{ 0x769b, 0x769b, PDF_CMAP_SINGLE, 10636 },
	{ 0x769c, 0x769e, PDF_CMAP_RANGE, 10633 },
	{ 0x76a1, 0x76a1, PDF_CMAP_SINGLE, 17109 },
	{ 0x76a4, 0x76a4, PDF_CMAP_SINGLE, 11703 },
	{ 0x76a5, 0x76a5, PDF_CMAP_SINGLE, 17110 },
	{ 0x76a6, 0x76a6, PDF_CMAP_SINGLE, 12135 },
	{ 0x76aa, 0x76aa, PDF_CMAP_SINGLE, 12795 },
	{ 0x76ab, 0x76ab, PDF_CMAP_SINGLE, 12794 },
	{ 0x76ad, 0x76ad, PDF_CMAP_SINGLE, 13206 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 868 },
	{ 0x76af, 0x76af, PDF_CMAP_SINGLE, 6676 },
	{ 0x76b0, 0x76b0, PDF_CMAP_SINGLE, 2369 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 3314 },
	{ 0x76b5, 0x76b5, PDF_CMAP_SINGLE, 9407 },
	{ 0x76b7, 0x76b7, PDF_CMAP_SINGLE, 17111 },
	{ 0x76b8, 0x76b8, PDF_CMAP_SINGLE, 10046 },
	{ 0x76ba, 0x76ba, PDF_CMAP_SINGLE, 4532 },
	{ 0x76bb, 0x76bb, PDF_CMAP_SINGLE, 11229 },
	{ 0x76bd, 0x76bd, PDF_CMAP_SINGLE, 12136 },
	{ 0x76be, 0x76be, PDF_CMAP_SINGLE, 12796 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 869 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 1626 },
	{ 0x76c3, 0x76c3, PDF_CMAP_SINGLE, 1968 },
	{ 0x76c4, 0x76c4, PDF_CMAP_SINGLE, 7062 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 1969 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 1967 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 1966 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 7514 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 2370 },
	{ 0x76cc, 0x76cc, PDF_CMAP_SINGLE, 17112 },
	{ 0x76cd, 0x76ce, PDF_CMAP_RANGE, 2371 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 2871 },
	{ 0x76d3, 0x76d3, PDF_CMAP_SINGLE, 8105 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 2870 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 16213 },
	{ 0x76da, 0x76da, PDF_CMAP_SINGLE, 8750 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 2872 },
	{ 0x76dc, 0x76dc, PDF_CMAP_SINGLE, 3315 },
	{ 0x76dd, 0x76dd, PDF_CMAP_SINGLE, 9408 },
	{ 0x76de, 0x76df, PDF_CMAP_RANGE, 3728 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 4152 },
	{ 0x76e3, 0x76e3, PDF_CMAP_SINGLE, 4153 },
	{ 0x76e4, 0x76e4, PDF_CMAP_SINGLE, 4533 },
	{ 0x76e5, 0x76e5, PDF_CMAP_SINGLE, 4870 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 11230 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 4869 },
	{ 0x76e9, 0x76e9, PDF_CMAP_SINGLE, 11704 },
	{ 0x76ea, 0x76ea, PDF_CMAP_SINGLE, 5143 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 12137 },
	{ 0x76ed, 0x76ed, PDF_CMAP_SINGLE, 12797 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 870 },
	{ 0x76ef, 0x76ef, PDF_CMAP_SINGLE, 1269 },
	{ 0x76f0, 0x76f0, PDF_CMAP_SINGLE, 6679 },
	{ 0x76f1, 0x76f1, PDF_CMAP_SINGLE, 6678 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 1627 },
	{ 0x76f3, 0x76f3, PDF_CMAP_SINGLE, 6677 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 1628 },
	{ 0x76f5, 0x76f5, PDF_CMAP_SINGLE, 6680 },
	{ 0x76f7, 0x76f7, PDF_CMAP_SINGLE, 7068 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 1972 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 1971 },
	{ 0x76fa, 0x76fa, PDF_CMAP_SINGLE, 7070 },
	{ 0x76fb, 0x76fb, PDF_CMAP_SINGLE, 7069 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 1976 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 1975 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 1970 },
	{ 0x7703, 0x7705, PDF_CMAP_RANGE, 7064 },
	{ 0x7707, 0x7707, PDF_CMAP_SINGLE, 1977 },
	{ 0x7708, 0x7708, PDF_CMAP_SINGLE, 7063 },
	{ 0x7709, 0x7709, PDF_CMAP_SINGLE, 1973 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 7067 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 1974 },
	{ 0x770c, 0x770c, PDF_CMAP_SINGLE, 15714 },
	{ 0x770e, 0x770f, PDF_CMAP_RANGE, 15106 },
	{ 0x7710, 0x7710, PDF_CMAP_SINGLE, 7517 },
	{ 0x7711, 0x7711, PDF_CMAP_SINGLE, 7521 },
	{ 0x7712, 0x7712, PDF_CMAP_SINGLE, 7519 },
	{ 0x7713, 0x7713, PDF_CMAP_SINGLE, 7518 },
	{ 0x7715, 0x7715, PDF_CMAP_SINGLE, 7522 },
	{ 0x7719, 0x771a, PDF_CMAP_RANGE, 7523 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 7516 },
	{ 0x771d, 0x771d, PDF_CMAP_SINGLE, 7515 },
	{ 0x771e, 0x771e, PDF_CMAP_SINGLE, 17118 },
	{ 0x771f, 0x7720, PDF_CMAP_RANGE, 2374 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 7525 },
	{ 0x7723, 0x7723, PDF_CMAP_SINGLE, 7520 },
	{ 0x7724, 0x7724, PDF_CMAP_SINGLE, 16360 },
	{ 0x7725, 0x7725, PDF_CMAP_SINGLE, 8114 },
	{ 0x7726, 0x7726, PDF_CMAP_SINGLE, 17119 },
	{ 0x7727, 0x7727, PDF_CMAP_SINGLE, 7526 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 2376 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 2373 },
	{ 0x772b, 0x772b, PDF_CMAP_SINGLE, 16371 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 8108 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 8107 },
	{ 0x7731, 0x7732, PDF_CMAP_RANGE, 8109 },
	{ 0x7733, 0x7733, PDF_CMAP_SINGLE, 8112 },
	{ 0x7734, 0x7734, PDF_CMAP_SINGLE, 8111 },
	{ 0x7735, 0x7735, PDF_CMAP_SINGLE, 8116 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 2876 },
	{ 0x7737, 0x7737, PDF_CMAP_SINGLE, 2873 },
	{ 0x7738, 0x7738, PDF_CMAP_SINGLE, 2877 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 8106 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 2878 },
	{ 0x773b, 0x773b, PDF_CMAP_SINGLE, 8115 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 2875 },
	{ 0x773d, 0x773d, PDF_CMAP_SINGLE, 8113 },
	{ 0x773e, 0x773e, PDF_CMAP_SINGLE, 2874 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 17120 },
	{ 0x7743, 0x7743, PDF_CMAP_SINGLE, 18186 },
	{ 0x7744, 0x7744, PDF_CMAP_SINGLE, 8753 },
	{ 0x7745, 0x7745, PDF_CMAP_SINGLE, 8755 },
	{ 0x7746, 0x7747, PDF_CMAP_RANGE, 8751 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 8756 },
	{ 0x774b, 0x774c, PDF_CMAP_RANGE, 8758 },
	{ 0x774d, 0x774d, PDF_CMAP_SINGLE, 8754 },
	{ 0x774e, 0x774e, PDF_CMAP_SINGLE, 8757 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 3316 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 9412 },
	{ 0x7754, 0x7754, PDF_CMAP_SINGLE, 9417 },
	{ 0x7755, 0x7755, PDF_CMAP_SINGLE, 9409 },
	{ 0x7756, 0x7756, PDF_CMAP_SINGLE, 9413 },
	{ 0x7758, 0x7758, PDF_CMAP_SINGLE, 17123 },
	{ 0x7759, 0x7759, PDF_CMAP_SINGLE, 9418 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 9414 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 3730 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 3738 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 3733 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 9410 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 4157 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 3741 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 3734 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 3739 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 3732 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 9416 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 3740 },
	{ 0x7769, 0x7769, PDF_CMAP_SINGLE, 9415 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 3736 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 3731 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 3737 },
	{ 0x776d, 0x776d, PDF_CMAP_SINGLE, 9419 },
	{ 0x776e, 0x776e, PDF_CMAP_SINGLE, 10051 },
	{ 0x776f, 0x776f, PDF_CMAP_SINGLE, 10053 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14699 },
	{ 0x7777, 0x7777, PDF_CMAP_SINGLE, 18089 },
	{ 0x7778, 0x7778, PDF_CMAP_SINGLE, 15104 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 3735 },
	{ 0x777a, 0x777a, PDF_CMAP_SINGLE, 17175 },
	{ 0x777b, 0x777b, PDF_CMAP_SINGLE, 15108 },
	{ 0x777c, 0x777c, PDF_CMAP_SINGLE, 10048 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 4155 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 10054 },
	{ 0x777f, 0x777f, PDF_CMAP_SINGLE, 4156 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 10052 },
	{ 0x7781, 0x7781, PDF_CMAP_SINGLE, 10047 },
	{ 0x7782, 0x7782, PDF_CMAP_SINGLE, 10050 },
	{ 0x7783, 0x7783, PDF_CMAP_SINGLE, 10055 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 4154 },
	{ 0x7785, 0x7785, PDF_CMAP_SINGLE, 10049 },
	{ 0x7787, 0x7787, PDF_CMAP_SINGLE, 4535 },
	{ 0x7788, 0x7788, PDF_CMAP_SINGLE, 10640 },
	{ 0x7789, 0x7789, PDF_CMAP_SINGLE, 10639 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 4538 },
	{ 0x778c, 0x778c, PDF_CMAP_SINGLE, 4536 },
	{ 0x778d, 0x778d, PDF_CMAP_SINGLE, 10637 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 4534 },
	{ 0x778f, 0x778f, PDF_CMAP_SINGLE, 10638 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 4537 },
	{ 0x7793, 0x7793, PDF_CMAP_SINGLE, 14381 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 11238 },
	{ 0x7797, 0x7797, PDF_CMAP_SINGLE, 11240 },
	{ 0x7798, 0x7798, PDF_CMAP_SINGLE, 18190 },
	{ 0x7799, 0x7799, PDF_CMAP_SINGLE, 11239 },
	{ 0x779a, 0x779a, PDF_CMAP_SINGLE, 11231 },
	{ 0x779b, 0x779b, PDF_CMAP_SINGLE, 11235 },
	{ 0x779c, 0x779c, PDF_CMAP_SINGLE, 11234 },
	{ 0x779d, 0x779d, PDF_CMAP_SINGLE, 11232 },
	{ 0x779e, 0x779f, PDF_CMAP_RANGE, 4872 },
	{ 0x77a0, 0x77a0, PDF_CMAP_SINGLE, 4871 },
	{ 0x77a1, 0x77a1, PDF_CMAP_SINGLE, 11233 },
	{ 0x77a2, 0x77a3, PDF_CMAP_RANGE, 11236 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 4874 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 5148 },
	{ 0x77a8, 0x77a8, PDF_CMAP_SINGLE, 11712 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 5145 },
	{ 0x77ab, 0x77ab, PDF_CMAP_SINGLE, 11706 },
	{ 0x77ac, 0x77ac, PDF_CMAP_SINGLE, 5147 },
	{ 0x77ad, 0x77ad, PDF_CMAP_SINGLE, 5149 },
	{ 0x77af, 0x77af, PDF_CMAP_SINGLE, 17125 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 5146 },
	{ 0x77b1, 0x77b1, PDF_CMAP_SINGLE, 11711 },
	{ 0x77b2, 0x77b2, PDF_CMAP_SINGLE, 11707 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 5144 },
	{ 0x77b4, 0x77b4, PDF_CMAP_SINGLE, 11710 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 11705 },
	{ 0x77b6, 0x77b6, PDF_CMAP_SINGLE, 11709 },
	{ 0x77b7, 0x77b7, PDF_CMAP_SINGLE, 11708 },
	{ 0x77b9, 0x77b9, PDF_CMAP_SINGLE, 15987 },
	{ 0x77ba, 0x77ba, PDF_CMAP_SINGLE, 12139 },
	{ 0x77bb, 0x77bc, PDF_CMAP_RANGE, 5372 },
	{ 0x77bd, 0x77bd, PDF_CMAP_SINGLE, 5370 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 18193 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 5371 },
	{ 0x77c2, 0x77c2, PDF_CMAP_SINGLE, 12138 },
	{ 0x77c3, 0x77c3, PDF_CMAP_SINGLE, 15281 },
	{ 0x77c4, 0x77c4, PDF_CMAP_SINGLE, 12488 },
	{ 0x77c5, 0x77c5, PDF_CMAP_SINGLE, 16833 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5534 },
	{ 0x77c9, 0x77ca, PDF_CMAP_RANGE, 12486 },
	{ 0x77cb, 0x77cb, PDF_CMAP_SINGLE, 18194 },
	{ 0x77cc, 0x77cc, PDF_CMAP_SINGLE, 12798 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 12801 },
	{ 0x77ce, 0x77cf, PDF_CMAP_RANGE, 12799 },
	{ 0x77d0, 0x77d0, PDF_CMAP_SINGLE, 13029 },
	{ 0x77d3, 0x77d3, PDF_CMAP_SINGLE, 5760 },
	{ 0x77d4, 0x77d4, PDF_CMAP_SINGLE, 13353 },
	{ 0x77d5, 0x77d5, PDF_CMAP_SINGLE, 13455 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5925 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 13523 },
	{ 0x77da, 0x77da, PDF_CMAP_SINGLE, 5971 },
	{ 0x77db, 0x77db, PDF_CMAP_SINGLE, 871 },
	{ 0x77dc, 0x77dc, PDF_CMAP_SINGLE, 1978 },
	{ 0x77de, 0x77de, PDF_CMAP_SINGLE, 8760 },
	{ 0x77e0, 0x77e0, PDF_CMAP_SINGLE, 9420 },
	{ 0x77e2, 0x77e2, PDF_CMAP_SINGLE, 872 },
	{ 0x77e3, 0x77e3, PDF_CMAP_SINGLE, 1270 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 1629 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 16513 },
	{ 0x77e7, 0x77e8, PDF_CMAP_RANGE, 7071 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 2377 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 8761 },
	{ 0x77ed, 0x77ed, PDF_CMAP_SINGLE, 3317 },
	{ 0x77ee, 0x77ee, PDF_CMAP_SINGLE, 3742 },
	{ 0x77ef, 0x77ef, PDF_CMAP_SINGLE, 5150 },
	{ 0x77f0, 0x77f0, PDF_CMAP_SINGLE, 11713 },
	{ 0x77f1, 0x77f1, PDF_CMAP_SINGLE, 12489 },
	{ 0x77f2, 0x77f2, PDF_CMAP_SINGLE, 12802 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 873 },
	{ 0x77f4, 0x77f4, PDF_CMAP_SINGLE, 17129 },
	{ 0x77f7, 0x77f7, PDF_CMAP_SINGLE, 6686 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 6681 },
	{ 0x77f9, 0x77f9, PDF_CMAP_SINGLE, 6683 },
	{ 0x77fa, 0x77fa, PDF_CMAP_SINGLE, 6685 },
	{ 0x77fb, 0x77fb, PDF_CMAP_SINGLE, 6684 },
	{ 0x77fc, 0x77fc, PDF_CMAP_SINGLE, 6682 },
	{ 0x77fd, 0x77fd, PDF_CMAP_SINGLE, 1630 },
	{ 0x77fe, 0x77fe, PDF_CMAP_SINGLE, 16341 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 1979 },
	{ 0x7803, 0x7803, PDF_CMAP_SINGLE, 7081 },
	{ 0x7805, 0x7805, PDF_CMAP_SINGLE, 7076 },
	{ 0x7806, 0x7806, PDF_CMAP_SINGLE, 7073 },
	{ 0x7808, 0x7808, PDF_CMAP_SINGLE, 15942 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 7080 },
	{ 0x780c, 0x780d, PDF_CMAP_RANGE, 1981 },
	{ 0x780e, 0x780e, PDF_CMAP_SINGLE, 7079 },
	{ 0x780f, 0x780f, PDF_CMAP_SINGLE, 7078 },
	{ 0x7810, 0x7810, PDF_CMAP_SINGLE, 7077 },
	{ 0x7811, 0x7812, PDF_CMAP_RANGE, 7074 },
	{ 0x7813, 0x7813, PDF_CMAP_SINGLE, 7082 },
	{ 0x7814, 0x7814, PDF_CMAP_SINGLE, 1980 },
	{ 0x7818, 0x7818, PDF_CMAP_SINGLE, 18195 },
	{ 0x781c, 0x781c, PDF_CMAP_SINGLE, 18196 },
	{ 0x781d, 0x781d, PDF_CMAP_SINGLE, 2381 },
	{ 0x781e, 0x781e, PDF_CMAP_SINGLE, 15123 },
	{ 0x781f, 0x781f, PDF_CMAP_SINGLE, 2387 },
	{ 0x7820, 0x7820, PDF_CMAP_SINGLE, 2386 },
	{ 0x7821, 0x7821, PDF_CMAP_SINGLE, 7535 },
	{ 0x7822, 0x7822, PDF_CMAP_SINGLE, 7529 },
	{ 0x7823, 0x7823, PDF_CMAP_SINGLE, 7527 },
	{ 0x7825, 0x7825, PDF_CMAP_SINGLE, 2384 },
	{ 0x7826, 0x7826, PDF_CMAP_SINGLE, 8123 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 2379 },
	{ 0x7828, 0x7828, PDF_CMAP_SINGLE, 7532 },
	{ 0x7829, 0x7829, PDF_CMAP_SINGLE, 7536 },
	{ 0x782a, 0x782a, PDF_CMAP_SINGLE, 7538 },
	{ 0x782b, 0x782b, PDF_CMAP_SINGLE, 7534 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 7528 },
	{ 0x782d, 0x782d, PDF_CMAP_SINGLE, 2385 },
	{ 0x782e, 0x782e, PDF_CMAP_SINGLE, 7533 },
	{ 0x782f, 0x782f, PDF_CMAP_SINGLE, 7531 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 2378 },
	{ 0x7831, 0x7831, PDF_CMAP_SINGLE, 7539 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 2388 },
	{ 0x7833, 0x7833, PDF_CMAP_SINGLE, 7537 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 2382 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 7530 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 2383 },
	{ 0x7838, 0x7838, PDF_CMAP_SINGLE, 2380 },
	{ 0x7839, 0x7839, PDF_CMAP_SINGLE, 17366 },
	{ 0x783c, 0x783c, PDF_CMAP_SINGLE, 16410 },
	{ 0x783d, 0x783d, PDF_CMAP_SINGLE, 15466 },
	{ 0x7842, 0x7842, PDF_CMAP_SINGLE, 15044 },
	{ 0x7843, 0x7843, PDF_CMAP_SINGLE, 2880 },
	{ 0x7844, 0x7844, PDF_CMAP_SINGLE, 15403 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 8124 },
	{ 0x7847, 0x7847, PDF_CMAP_SINGLE, 18197 },
	{ 0x7848, 0x7848, PDF_CMAP_SINGLE, 8117 },
	{ 0x7849, 0x7849, PDF_CMAP_SINGLE, 8119 },
	{ 0x784a, 0x784a, PDF_CMAP_SINGLE, 8121 },
	{ 0x784b, 0x784b, PDF_CMAP_SINGLE, 15469 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 8122 },
	{ 0x784d, 0x784d, PDF_CMAP_SINGLE, 8120 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 2881 },
	{ 0x7850, 0x7850, PDF_CMAP_SINGLE, 8125 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 18198 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 8118 },
	{ 0x7853, 0x7853, PDF_CMAP_SINGLE, 16277 },
	{ 0x7854, 0x7854, PDF_CMAP_SINGLE, 15467 },
	{ 0x785c, 0x785c, PDF_CMAP_SINGLE, 8765 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3318 },
	{ 0x785e, 0x785e, PDF_CMAP_SINGLE, 8773 },
	{ 0x7860, 0x7860, PDF_CMAP_SINGLE, 8762 },
	{ 0x7862, 0x7862, PDF_CMAP_SINGLE, 8774 },
	{ 0x7864, 0x7865, PDF_CMAP_RANGE, 8763 },
	{ 0x7866, 0x7866, PDF_CMAP_SINGLE, 18199 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 8772 },
	{ 0x7869, 0x7869, PDF_CMAP_SINGLE, 8771 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 8768 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 2879 },
	{ 0x786c, 0x786c, PDF_CMAP_SINGLE, 3319 },
	{ 0x786d, 0x786d, PDF_CMAP_SINGLE, 8766 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 8769 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 3320 },
	{ 0x7870, 0x7870, PDF_CMAP_SINGLE, 8770 },
	{ 0x7871, 0x7871, PDF_CMAP_SINGLE, 8767 },
	{ 0x7879, 0x7879, PDF_CMAP_SINGLE, 9431 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 16105 },
	{ 0x787b, 0x787b, PDF_CMAP_SINGLE, 9435 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 3749 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 10061 },
	{ 0x787f, 0x787f, PDF_CMAP_SINGLE, 3752 },
	{ 0x7880, 0x7880, PDF_CMAP_SINGLE, 9433 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 14056 },
	{ 0x7883, 0x7883, PDF_CMAP_SINGLE, 9430 },
	{ 0x7884, 0x7884, PDF_CMAP_SINGLE, 9425 },
	{ 0x7885, 0x7886, PDF_CMAP_RANGE, 9427 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 9421 },
	{ 0x7888, 0x7888, PDF_CMAP_SINGLE, 15125 },
	{ 0x7889, 0x7889, PDF_CMAP_SINGLE, 3748 },
	{ 0x788c, 0x788c, PDF_CMAP_SINGLE, 3747 },
	{ 0x788d, 0x788d, PDF_CMAP_SINGLE, 15124 },
	{ 0x788e, 0x788e, PDF_CMAP_SINGLE, 3743 },
	{ 0x788f, 0x788f, PDF_CMAP_SINGLE, 9424 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3750 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 3751 },
	{ 0x7894, 0x7894, PDF_CMAP_SINGLE, 9423 },
	{ 0x7895, 0x7895, PDF_CMAP_SINGLE, 9426 },
	{ 0x7896, 0x7896, PDF_CMAP_SINGLE, 9434 },
	{ 0x7897, 0x7898, PDF_CMAP_RANGE, 3745 },
	{ 0x7899, 0x7899, PDF_CMAP_SINGLE, 9432 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 9422 },
	{ 0x789e, 0x789e, PDF_CMAP_SINGLE, 10063 },
	{ 0x789f, 0x789f, PDF_CMAP_SINGLE, 4159 },
	{ 0x78a0, 0x78a0, PDF_CMAP_SINGLE, 10065 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 9429 },
	{ 0x78a2, 0x78a2, PDF_CMAP_SINGLE, 10067 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 4163 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 10068 },
	{ 0x78a5, 0x78a5, PDF_CMAP_SINGLE, 10064 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 4160 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 10060 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 4162 },
	{ 0x78aa, 0x78aa, PDF_CMAP_SINGLE, 10057 },
	{ 0x78ab, 0x78ab, PDF_CMAP_SINGLE, 10062 },
	{ 0x78ac, 0x78ac, PDF_CMAP_SINGLE, 10066 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 10059 },
	{ 0x78af, 0x78af, PDF_CMAP_SINGLE, 17132 },
	{ 0x78b0, 0x78b0, PDF_CMAP_SINGLE, 3744 },
	{ 0x78b1, 0x78b1, PDF_CMAP_SINGLE, 17140 },
	{ 0x78b2, 0x78b2, PDF_CMAP_SINGLE, 10056 },
	{ 0x78b3, 0x78b3, PDF_CMAP_SINGLE, 4161 },
	{ 0x78b4, 0x78b4, PDF_CMAP_SINGLE, 10058 },
	{ 0x78b6, 0x78b6, PDF_CMAP_SINGLE, 15468 },
	{ 0x78b8, 0x78b8, PDF_CMAP_SINGLE, 17777 },
	{ 0x78b9, 0x78b9, PDF_CMAP_SINGLE, 15459 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 4541 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 10642 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 4545 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 4543 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 4158 },
	{ 0x78c3, 0x78c4, PDF_CMAP_RANGE, 10649 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 4540 },
	{ 0x78c7, 0x78c7, PDF_CMAP_SINGLE, 17133 },
	{ 0x78c8, 0x78c8, PDF_CMAP_SINGLE, 10648 },
	{ 0x78c9, 0x78c9, PDF_CMAP_SINGLE, 10651 },
	{ 0x78ca, 0x78ca, PDF_CMAP_SINGLE, 4542 },
	{ 0x78cb, 0x78cb, PDF_CMAP_SINGLE, 4539 },
	{ 0x78cc, 0x78cc, PDF_CMAP_SINGLE, 10644 },
	{ 0x78cd, 0x78cd, PDF_CMAP_SINGLE, 10641 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 10646 },
	{ 0x78cf, 0x78cf, PDF_CMAP_SINGLE, 10643 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 4546 },
	{ 0x78d1, 0x78d1, PDF_CMAP_SINGLE, 10645 },
	{ 0x78d2, 0x78d2, PDF_CMAP_SINGLE, 15126 },
	{ 0x78d3, 0x78d3, PDF_CMAP_SINGLE, 17134 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 10647 },
	{ 0x78d5, 0x78d5, PDF_CMAP_SINGLE, 4544 },
	{ 0x78d7, 0x78d7, PDF_CMAP_SINGLE, 17138 },
	{ 0x78d8, 0x78d8, PDF_CMAP_SINGLE, 16268 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 4876 },
	{ 0x78db, 0x78db, PDF_CMAP_SINGLE, 11247 },
	{ 0x78dd, 0x78dd, PDF_CMAP_SINGLE, 11241 },
	{ 0x78de, 0x78de, PDF_CMAP_SINGLE, 11245 },
	{ 0x78df, 0x78e0, PDF_CMAP_RANGE, 11251 },
	{ 0x78e1, 0x78e2, PDF_CMAP_RANGE, 11248 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 11246 },
	{ 0x78e5, 0x78e5, PDF_CMAP_SINGLE, 11243 },
	{ 0x78e7, 0x78e7, PDF_CMAP_SINGLE, 4878 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 4875 },
	{ 0x78e9, 0x78e9, PDF_CMAP_SINGLE, 11242 },
	{ 0x78ea, 0x78ea, PDF_CMAP_SINGLE, 11244 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 4877 },
	{ 0x78ed, 0x78ed, PDF_CMAP_SINGLE, 11250 },
	{ 0x78ee, 0x78ee, PDF_CMAP_SINGLE, 14977 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 5154 },
	{ 0x78f0, 0x78f0, PDF_CMAP_SINGLE, 15162 },
	{ 0x78f1, 0x78f1, PDF_CMAP_SINGLE, 16574 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 11720 },
	{ 0x78f3, 0x78f3, PDF_CMAP_SINGLE, 11714 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 5153 },
	{ 0x78f5, 0x78f5, PDF_CMAP_SINGLE, 14515 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 5151 },
	{ 0x78f9, 0x78f9, PDF_CMAP_SINGLE, 11722 },
	{ 0x78fa, 0x78fa, PDF_CMAP_SINGLE, 5152 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 11717 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 11715 },
	{ 0x78fe, 0x78fe, PDF_CMAP_SINGLE, 11723 },
	{ 0x78ff, 0x78ff, PDF_CMAP_SINGLE, 11719 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 5155 },
	{ 0x7902, 0x7902, PDF_CMAP_SINGLE, 11716 },
	{ 0x7904, 0x7904, PDF_CMAP_SINGLE, 11724 },
	{ 0x7905, 0x7905, PDF_CMAP_SINGLE, 11721 },
	{ 0x7906, 0x7906, PDF_CMAP_SINGLE, 16335 },
	{ 0x7909, 0x7909, PDF_CMAP_SINGLE, 12143 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 12140 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 5374 },
	{ 0x7910, 0x7910, PDF_CMAP_SINGLE, 12144 },
	{ 0x7911, 0x7911, PDF_CMAP_SINGLE, 12146 },
	{ 0x7912, 0x7912, PDF_CMAP_SINGLE, 12145 },
	{ 0x7913, 0x7914, PDF_CMAP_RANGE, 12141 },
	{ 0x7917, 0x7917, PDF_CMAP_SINGLE, 12494 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5535 },
	{ 0x791b, 0x791b, PDF_CMAP_SINGLE, 12491 },
	{ 0x791c, 0x791c, PDF_CMAP_SINGLE, 12493 },
	{ 0x791d, 0x791d, PDF_CMAP_SINGLE, 12490 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 12495 },
	{ 0x7921, 0x7921, PDF_CMAP_SINGLE, 12492 },
	{ 0x7923, 0x7923, PDF_CMAP_SINGLE, 12804 },
	{ 0x7924, 0x7924, PDF_CMAP_SINGLE, 12807 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 12803 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 5668 },
	{ 0x7927, 0x7928, PDF_CMAP_RANGE, 12805 },
	{ 0x7929, 0x7929, PDF_CMAP_SINGLE, 12808 },
	{ 0x792a, 0x792a, PDF_CMAP_SINGLE, 5669 },
	{ 0x792b, 0x792b, PDF_CMAP_SINGLE, 5671 },
	{ 0x792c, 0x792c, PDF_CMAP_SINGLE, 5670 },
	{ 0x792d, 0x792d, PDF_CMAP_SINGLE, 13030 },
	{ 0x792e, 0x792e, PDF_CMAP_SINGLE, 17136 },
	{ 0x792f, 0x792f, PDF_CMAP_SINGLE, 13032 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 13031 },
	{ 0x7932, 0x7932, PDF_CMAP_SINGLE, 18203 },
	{ 0x7933, 0x7933, PDF_CMAP_SINGLE, 18202 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 17139 },
	{ 0x7935, 0x7935, PDF_CMAP_SINGLE, 13207 },
	{ 0x7936, 0x7936, PDF_CMAP_SINGLE, 14211 },
	{ 0x7938, 0x7938, PDF_CMAP_SINGLE, 13456 },
	{ 0x7939, 0x7939, PDF_CMAP_SINGLE, 13525 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 874 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 17655 },
	{ 0x793c, 0x793c, PDF_CMAP_SINGLE, 17152 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 6359 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 1631 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 6688 },
	{ 0x7940, 0x7941, PDF_CMAP_RANGE, 1632 },
	{ 0x7942, 0x7942, PDF_CMAP_SINGLE, 6687 },
	{ 0x7944, 0x7944, PDF_CMAP_SINGLE, 7087 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 7086 },
	{ 0x7946, 0x7946, PDF_CMAP_SINGLE, 1983 },
	{ 0x7947, 0x7947, PDF_CMAP_SINGLE, 1986 },
	{ 0x7948, 0x7948, PDF_CMAP_SINGLE, 1985 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 1984 },
	{ 0x794a, 0x794a, PDF_CMAP_SINGLE, 7083 },
	{ 0x794b, 0x794b, PDF_CMAP_SINGLE, 7085 },
	{ 0x794c, 0x794c, PDF_CMAP_SINGLE, 7084 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 7542 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 2390 },
	{ 0x7951, 0x7951, PDF_CMAP_SINGLE, 7546 },
	{ 0x7952, 0x7952, PDF_CMAP_SINGLE, 7545 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 7544 },
	{ 0x7954, 0x7954, PDF_CMAP_SINGLE, 7540 },
	{ 0x7955, 0x7955, PDF_CMAP_SINGLE, 2389 },
	{ 0x7956, 0x7956, PDF_CMAP_SINGLE, 2393 },
	{ 0x7957, 0x7957, PDF_CMAP_SINGLE, 2396 },
	{ 0x7958, 0x7958, PDF_CMAP_SINGLE, 14309 },
	{ 0x7959, 0x7959, PDF_CMAP_SINGLE, 15128 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 2397 },
	{ 0x795b, 0x795b, PDF_CMAP_SINGLE, 7541 },
	{ 0x795c, 0x795c, PDF_CMAP_SINGLE, 7543 },
	{ 0x795d, 0x795d, PDF_CMAP_SINGLE, 2395 },
	{ 0x795e, 0x795e, PDF_CMAP_SINGLE, 2394 },
	{ 0x795f, 0x795f, PDF_CMAP_SINGLE, 2392 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 2391 },
	{ 0x7961, 0x7961, PDF_CMAP_SINGLE, 8132 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 15997 },
	{ 0x7963, 0x7963, PDF_CMAP_SINGLE, 8130 },
	{ 0x7964, 0x7964, PDF_CMAP_SINGLE, 8126 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 2882 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 8127 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 2883 },
	{ 0x7969, 0x796a, PDF_CMAP_RANGE, 8128 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 8131 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2884 },
	{ 0x7970, 0x7970, PDF_CMAP_SINGLE, 8778 },
	{ 0x7971, 0x7971, PDF_CMAP_SINGLE, 16744 },
	{ 0x7972, 0x7972, PDF_CMAP_SINGLE, 8777 },
	{ 0x7973, 0x7973, PDF_CMAP_SINGLE, 8776 },
	{ 0x7974, 0x7974, PDF_CMAP_SINGLE, 8775 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 9439 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 3753 },
	{ 0x797c, 0x797c, PDF_CMAP_SINGLE, 9436 },
	{ 0x797d, 0x797d, PDF_CMAP_SINGLE, 9438 },
	{ 0x797e, 0x797e, PDF_CMAP_SINGLE, 15910 },
	{ 0x797f, 0x797f, PDF_CMAP_SINGLE, 3754 },
	{ 0x7980, 0x7980, PDF_CMAP_SINGLE, 17148 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 3755 },
	{ 0x7982, 0x7982, PDF_CMAP_SINGLE, 9437 },
	{ 0x7983, 0x7983, PDF_CMAP_SINGLE, 15862 },
	{ 0x7986, 0x7986, PDF_CMAP_SINGLE, 17145 },
	{ 0x7987, 0x7987, PDF_CMAP_SINGLE, 17800 },
	{ 0x7988, 0x7988, PDF_CMAP_SINGLE, 10077 },
	{ 0x798a, 0x798b, PDF_CMAP_RANGE, 10070 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 4166 },
	{ 0x798e, 0x798f, PDF_CMAP_RANGE, 4164 },
	{ 0x7990, 0x7990, PDF_CMAP_SINGLE, 10079 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 18205 },
	{ 0x7992, 0x7992, PDF_CMAP_SINGLE, 10078 },
	{ 0x7993, 0x7993, PDF_CMAP_SINGLE, 10075 },
	{ 0x7994, 0x7994, PDF_CMAP_SINGLE, 10074 },
	{ 0x7995, 0x7995, PDF_CMAP_SINGLE, 10073 },
	{ 0x7996, 0x7996, PDF_CMAP_SINGLE, 10072 },
	{ 0x7997, 0x7997, PDF_CMAP_SINGLE, 10076 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 10069 },
	{ 0x7999, 0x7999, PDF_CMAP_SINGLE, 17404 },
	{ 0x799a, 0x799a, PDF_CMAP_SINGLE, 10652 },
	{ 0x799b, 0x799b, PDF_CMAP_SINGLE, 10657 },
	{ 0x799c, 0x799c, PDF_CMAP_SINGLE, 10655 },
	{ 0x799d, 0x799d, PDF_CMAP_SINGLE, 17150 },
	{ 0x799f, 0x799f, PDF_CMAP_SINGLE, 14686 },
	{ 0x79a0, 0x79a0, PDF_CMAP_SINGLE, 10654 },
	{ 0x79a1, 0x79a1, PDF_CMAP_SINGLE, 10653 },
	{ 0x79a2, 0x79a2, PDF_CMAP_SINGLE, 10656 },
	{ 0x79a4, 0x79a4, PDF_CMAP_SINGLE, 11253 },
	{ 0x79a5, 0x79a5, PDF_CMAP_SINGLE, 15134 },
	{ 0x79a6, 0x79a6, PDF_CMAP_SINGLE, 4879 },
	{ 0x79a7, 0x79a7, PDF_CMAP_SINGLE, 5156 },
	{ 0x79a8, 0x79a8, PDF_CMAP_SINGLE, 11726 },
	{ 0x79a9, 0x79a9, PDF_CMAP_SINGLE, 17153 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5157 },
	{ 0x79ab, 0x79ab, PDF_CMAP_SINGLE, 11725 },
	{ 0x79ac, 0x79ac, PDF_CMAP_SINGLE, 12148 },
	{ 0x79ad, 0x79ad, PDF_CMAP_SINGLE, 12147 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 5375 },
	{ 0x79b0, 0x79b0, PDF_CMAP_SINGLE, 12496 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 5536 },
	{ 0x79b2, 0x79b2, PDF_CMAP_SINGLE, 12809 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5839 },
	{ 0x79b4, 0x79b4, PDF_CMAP_SINGLE, 13208 },
	{ 0x79b6, 0x79b6, PDF_CMAP_SINGLE, 13458 },
	{ 0x79b7, 0x79b7, PDF_CMAP_SINGLE, 13457 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 6066 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 1987 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 8133 },
	{ 0x79bd, 0x79bd, PDF_CMAP_SINGLE, 3757 },
	{ 0x79be, 0x79be, PDF_CMAP_SINGLE, 875 },
	{ 0x79bf, 0x79bf, PDF_CMAP_SINGLE, 1273 },
	{ 0x79c0, 0x79c0, PDF_CMAP_SINGLE, 1272 },
	{ 0x79c1, 0x79c1, PDF_CMAP_SINGLE, 1271 },
	{ 0x79c4, 0x79c4, PDF_CMAP_SINGLE, 15565 },
	{ 0x79c5, 0x79c5, PDF_CMAP_SINGLE, 6689 },
	{ 0x79c6, 0x79c6, PDF_CMAP_SINGLE, 17157 },
	{ 0x79c8, 0x79c8, PDF_CMAP_SINGLE, 1635 },
	{ 0x79c9, 0x79c9, PDF_CMAP_SINGLE, 1634 },
	{ 0x79cb, 0x79cb, PDF_CMAP_SINGLE, 1991 },
	{ 0x79cc, 0x79cc, PDF_CMAP_SINGLE, 16947 },
	{ 0x79cd, 0x79cd, PDF_CMAP_SINGLE, 7089 },
	{ 0x79ce, 0x79ce, PDF_CMAP_SINGLE, 7092 },
	{ 0x79cf, 0x79cf, PDF_CMAP_SINGLE, 7090 },
	{ 0x79d1, 0x79d2, PDF_CMAP_RANGE, 1989 },
	{ 0x79d4, 0x79d4, PDF_CMAP_SINGLE, 17159 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 7088 },
	{ 0x79d6, 0x79d6, PDF_CMAP_SINGLE, 7091 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2404 },
	{ 0x79dc, 0x79dc, PDF_CMAP_SINGLE, 7553 },
	{ 0x79dd, 0x79dd, PDF_CMAP_SINGLE, 7555 },
	{ 0x79de, 0x79de, PDF_CMAP_SINGLE, 7554 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 2401 },
	{ 0x79e0, 0x79e0, PDF_CMAP_SINGLE, 7549 },
	{ 0x79e2, 0x79e2, PDF_CMAP_SINGLE, 15450 },
	{ 0x79e3, 0x79e3, PDF_CMAP_SINGLE, 2399 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 2398 },
	{ 0x79e6, 0x79e6, PDF_CMAP_SINGLE, 2402 },
	{ 0x79e7, 0x79e7, PDF_CMAP_SINGLE, 2400 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 2403 },
	{ 0x79ea, 0x79ea, PDF_CMAP_SINGLE, 7552 },
	{ 0x79eb, 0x79ec, PDF_CMAP_RANGE, 7547 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 7551 },
	{ 0x79ee, 0x79ee, PDF_CMAP_SINGLE, 7550 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 15143 },
	{ 0x79f4, 0x79f4, PDF_CMAP_SINGLE, 15138 },
	{ 0x79f6, 0x79f7, PDF_CMAP_RANGE, 8136 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 8135 },
	{ 0x79fa, 0x79fa, PDF_CMAP_SINGLE, 8134 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 2885 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 3325 },
	{ 0x7a02, 0x7a02, PDF_CMAP_SINGLE, 8779 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 8781 },
	{ 0x7a04, 0x7a04, PDF_CMAP_SINGLE, 8783 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 3324 },
	{ 0x7a06, 0x7a06, PDF_CMAP_SINGLE, 18206 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 3322 },
	{ 0x7a0a, 0x7a0a, PDF_CMAP_SINGLE, 8780 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 3323 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 8782 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 3321 },
	{ 0x7a10, 0x7a10, PDF_CMAP_SINGLE, 9449 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 9440 },
	{ 0x7a12, 0x7a12, PDF_CMAP_SINGLE, 9443 },
	{ 0x7a13, 0x7a13, PDF_CMAP_SINGLE, 9447 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 3761 },
	{ 0x7a15, 0x7a15, PDF_CMAP_SINGLE, 9445 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 9444 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 9441 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 3759 },
	{ 0x7a1b, 0x7a1b, PDF_CMAP_SINGLE, 9448 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 3758 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 3763 },
	{ 0x7a1f, 0x7a1f, PDF_CMAP_SINGLE, 3762 },
	{ 0x7a20, 0x7a20, PDF_CMAP_SINGLE, 3760 },
	{ 0x7a22, 0x7a22, PDF_CMAP_SINGLE, 9446 },
	{ 0x7a26, 0x7a26, PDF_CMAP_SINGLE, 10085 },
	{ 0x7a28, 0x7a28, PDF_CMAP_SINGLE, 10084 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 10080 },
	{ 0x7a2d, 0x7a2d, PDF_CMAP_SINGLE, 16346 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 4167 },
	{ 0x7a2f, 0x7a2f, PDF_CMAP_SINGLE, 10083 },
	{ 0x7a30, 0x7a30, PDF_CMAP_SINGLE, 10082 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 4168 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 4551 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 10659 },
	{ 0x7a3a, 0x7a3a, PDF_CMAP_SINGLE, 15137 },
	{ 0x7a3b, 0x7a3b, PDF_CMAP_SINGLE, 4552 },
	{ 0x7a3c, 0x7a3c, PDF_CMAP_SINGLE, 4548 },
	{ 0x7a3d, 0x7a3d, PDF_CMAP_SINGLE, 4550 },
	{ 0x7a3e, 0x7a3e, PDF_CMAP_SINGLE, 16267 },
	{ 0x7a3f, 0x7a3f, PDF_CMAP_SINGLE, 4547 },
	{ 0x7a40, 0x7a40, PDF_CMAP_SINGLE, 4549 },
	{ 0x7a43, 0x7a43, PDF_CMAP_SINGLE, 14700 },
	{ 0x7a44, 0x7a44, PDF_CMAP_SINGLE, 11254 },
	{ 0x7a45, 0x7a45, PDF_CMAP_SINGLE, 15857 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 4882 },
	{ 0x7a47, 0x7a47, PDF_CMAP_SINGLE, 11256 },
	{ 0x7a48, 0x7a48, PDF_CMAP_SINGLE, 11255 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 14131 },
	{ 0x7a4a, 0x7a4a, PDF_CMAP_SINGLE, 10081 },
	{ 0x7a4b, 0x7a4b, PDF_CMAP_SINGLE, 4884 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 4883 },
	{ 0x7a4d, 0x7a4e, PDF_CMAP_RANGE, 4880 },
	{ 0x7a54, 0x7a54, PDF_CMAP_SINGLE, 11731 },
	{ 0x7a56, 0x7a56, PDF_CMAP_SINGLE, 11729 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5158 },
	{ 0x7a58, 0x7a58, PDF_CMAP_SINGLE, 11730 },
	{ 0x7a5a, 0x7a5a, PDF_CMAP_SINGLE, 11732 },
	{ 0x7a5b, 0x7a5b, PDF_CMAP_SINGLE, 11728 },
	{ 0x7a5c, 0x7a5c, PDF_CMAP_SINGLE, 11727 },
	{ 0x7a5f, 0x7a5f, PDF_CMAP_SINGLE, 12149 },
	{ 0x7a60, 0x7a60, PDF_CMAP_SINGLE, 5378 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5376 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 14134 },
	{ 0x7a67, 0x7a68, PDF_CMAP_RANGE, 12497 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5538 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 5537 },
	{ 0x7a6c, 0x7a6d, PDF_CMAP_RANGE, 12811 },
	{ 0x7a6e, 0x7a6e, PDF_CMAP_SINGLE, 12810 },
	{ 0x7a70, 0x7a71, PDF_CMAP_RANGE, 13209 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 876 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 6161 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 1274 },
	{ 0x7a78, 0x7a78, PDF_CMAP_SINGLE, 6690 },
	{ 0x7a79, 0x7a79, PDF_CMAP_SINGLE, 1637 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 1636 },
	{ 0x7a7b, 0x7a7b, PDF_CMAP_SINGLE, 6691 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 14135 },
	{ 0x7a7e, 0x7a7e, PDF_CMAP_SINGLE, 7094 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 1992 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 7093 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 1993 },
	{ 0x7a83, 0x7a83, PDF_CMAP_SINGLE, 15741 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 2405 },
	{ 0x7a85, 0x7a85, PDF_CMAP_SINGLE, 7558 },
	{ 0x7a86, 0x7a86, PDF_CMAP_SINGLE, 7556 },
	{ 0x7a87, 0x7a87, PDF_CMAP_SINGLE, 7562 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 2406 },
	{ 0x7a89, 0x7a89, PDF_CMAP_SINGLE, 7557 },
	{ 0x7a8a, 0x7a8a, PDF_CMAP_SINGLE, 7561 },
	{ 0x7a8b, 0x7a8c, PDF_CMAP_RANGE, 7559 },
	{ 0x7a8f, 0x7a8f, PDF_CMAP_SINGLE, 8138 },
	{ 0x7a90, 0x7a90, PDF_CMAP_SINGLE, 8140 },
	{ 0x7a91, 0x7a91, PDF_CMAP_SINGLE, 17368 },
	{ 0x7a92, 0x7a92, PDF_CMAP_SINGLE, 2886 },
	{ 0x7a94, 0x7a94, PDF_CMAP_SINGLE, 8139 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 2887 },
	{ 0x7a96, 0x7a96, PDF_CMAP_SINGLE, 3328 },
	{ 0x7a97, 0x7a97, PDF_CMAP_SINGLE, 3327 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 3326 },
	{ 0x7a99, 0x7a99, PDF_CMAP_SINGLE, 8784 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 9452 },
	{ 0x7a9f, 0x7aa0, PDF_CMAP_RANGE, 3764 },
	{ 0x7aa2, 0x7aa2, PDF_CMAP_SINGLE, 9451 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 9450 },
	{ 0x7aa8, 0x7aa8, PDF_CMAP_SINGLE, 10086 },
	{ 0x7aa9, 0x7aa9, PDF_CMAP_SINGLE, 4170 },
	{ 0x7aaa, 0x7aaa, PDF_CMAP_SINGLE, 4169 },
	{ 0x7aab, 0x7aac, PDF_CMAP_RANGE, 10087 },
	{ 0x7aae, 0x7aae, PDF_CMAP_SINGLE, 4554 },
	{ 0x7aaf, 0x7aaf, PDF_CMAP_SINGLE, 4553 },
	{ 0x7ab0, 0x7ab0, PDF_CMAP_SINGLE, 14138 },
	{ 0x7ab1, 0x7ab1, PDF_CMAP_SINGLE, 11260 },
	{ 0x7ab2, 0x7ab2, PDF_CMAP_SINGLE, 10660 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 10662 },
	{ 0x7ab4, 0x7ab4, PDF_CMAP_SINGLE, 10661 },
	{ 0x7ab5, 0x7ab5, PDF_CMAP_SINGLE, 11259 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 11257 },
	{ 0x7ab7, 0x7ab7, PDF_CMAP_SINGLE, 11261 },
	{ 0x7ab8, 0x7ab8, PDF_CMAP_SINGLE, 11258 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 4885 },
	{ 0x7abb, 0x7abb, PDF_CMAP_SINGLE, 14137 },
	{ 0x7abc, 0x7abc, PDF_CMAP_SINGLE, 18209 },
	{ 0x7abe, 0x7abe, PDF_CMAP_SINGLE, 11733 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 5159 },
	{ 0x7ac0, 0x7ac1, PDF_CMAP_RANGE, 11734 },
	{ 0x7ac2, 0x7ac3, PDF_CMAP_RANGE, 14139 },
	{ 0x7ac4, 0x7ac5, PDF_CMAP_RANGE, 5379 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 5672 },
	{ 0x7ac8, 0x7ac8, PDF_CMAP_SINGLE, 15743 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 17776 },
	{ 0x7aca, 0x7aca, PDF_CMAP_SINGLE, 5887 },
	{ 0x7acb, 0x7acb, PDF_CMAP_SINGLE, 877 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 18212 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 7095 },
	{ 0x7ad3, 0x7ad3, PDF_CMAP_SINGLE, 16266 },
	{ 0x7ad8, 0x7ad8, PDF_CMAP_SINGLE, 7563 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 2407 },
	{ 0x7ada, 0x7ada, PDF_CMAP_SINGLE, 14144 },
	{ 0x7adb, 0x7adb, PDF_CMAP_SINGLE, 18213 },
	{ 0x7adc, 0x7adc, PDF_CMAP_SINGLE, 14673 },
	{ 0x7add, 0x7add, PDF_CMAP_SINGLE, 14145 },
	{ 0x7adf, 0x7adf, PDF_CMAP_SINGLE, 3038 },
	{ 0x7ae0, 0x7ae0, PDF_CMAP_SINGLE, 3037 },
	{ 0x7ae2, 0x7ae2, PDF_CMAP_SINGLE, 15155 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 3330 },
	{ 0x7ae4, 0x7ae4, PDF_CMAP_SINGLE, 8786 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 3329 },
	{ 0x7ae6, 0x7ae6, PDF_CMAP_SINGLE, 8785 },
	{ 0x7ae7, 0x7ae7, PDF_CMAP_SINGLE, 14429 },
	{ 0x7ae9, 0x7ae9, PDF_CMAP_SINGLE, 14385 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 14146 },
	{ 0x7aeb, 0x7aeb, PDF_CMAP_SINGLE, 9453 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 4171 },
	{ 0x7aee, 0x7aee, PDF_CMAP_SINGLE, 10089 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 4172 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 5673 },
	{ 0x7af7, 0x7af7, PDF_CMAP_SINGLE, 12813 },
	{ 0x7af9, 0x7af9, PDF_CMAP_SINGLE, 1014 },
	{ 0x7afa, 0x7afa, PDF_CMAP_SINGLE, 1638 },
	{ 0x7afb, 0x7afb, PDF_CMAP_SINGLE, 6692 },
	{ 0x7afd, 0x7afd, PDF_CMAP_SINGLE, 1995 },
	{ 0x7afe, 0x7afe, PDF_CMAP_SINGLE, 15165 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1994 },
	{ 0x7b00, 0x7b01, PDF_CMAP_RANGE, 7096 },
	{ 0x7b04, 0x7b04, PDF_CMAP_SINGLE, 7565 },
	{ 0x7b05, 0x7b05, PDF_CMAP_SINGLE, 7567 },
	{ 0x7b06, 0x7b06, PDF_CMAP_SINGLE, 2408 },
	{ 0x7b08, 0x7b08, PDF_CMAP_SINGLE, 7569 },
	{ 0x7b09, 0x7b09, PDF_CMAP_SINGLE, 7572 },
	{ 0x7b0a, 0x7b0a, PDF_CMAP_SINGLE, 7570 },
	{ 0x7b0b, 0x7b0b, PDF_CMAP_SINGLE, 14150 },
	{ 0x7b0c, 0x7b0c, PDF_CMAP_SINGLE, 15203 },
	{ 0x7b0e, 0x7b0e, PDF_CMAP_SINGLE, 7571 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 7568 },
	{ 0x7b10, 0x7b10, PDF_CMAP_SINGLE, 7564 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 2409 },
	{ 0x7b12, 0x7b12, PDF_CMAP_SINGLE, 7573 },
	{ 0x7b13, 0x7b13, PDF_CMAP_SINGLE, 7566 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 16247 },
	{ 0x7b18, 0x7b18, PDF_CMAP_SINGLE, 8149 },
	{ 0x7b19, 0x7b19, PDF_CMAP_SINGLE, 2893 },
	{ 0x7b1a, 0x7b1a, PDF_CMAP_SINGLE, 8158 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 2890 },
	{ 0x7b1d, 0x7b1d, PDF_CMAP_SINGLE, 8151 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 2894 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 16145 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 2888 },
	{ 0x7b22, 0x7b22, PDF_CMAP_SINGLE, 8146 },
	{ 0x7b23, 0x7b23, PDF_CMAP_SINGLE, 8159 },
	{ 0x7b24, 0x7b24, PDF_CMAP_SINGLE, 8147 },
	{ 0x7b25, 0x7b25, PDF_CMAP_SINGLE, 8144 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 2892 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 15199 },
	{ 0x7b28, 0x7b28, PDF_CMAP_SINGLE, 2889 },
	{ 0x7b29, 0x7b29, PDF_CMAP_SINGLE, 14152 },
	{ 0x7b2a, 0x7b2a, PDF_CMAP_SINGLE, 8150 },
	{ 0x7b2b, 0x7b2b, PDF_CMAP_SINGLE, 8153 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 2891 },
	{ 0x7b2d, 0x7b2d, PDF_CMAP_SINGLE, 8154 },
	{ 0x7b2e, 0x7b2e, PDF_CMAP_SINGLE, 2895 },
	{ 0x7b2f, 0x7b2f, PDF_CMAP_SINGLE, 8155 },
	{ 0x7b30, 0x7b30, PDF_CMAP_SINGLE, 8145 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 8152 },
	{ 0x7b32, 0x7b32, PDF_CMAP_SINGLE, 8156 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 8148 },
	{ 0x7b34, 0x7b34, PDF_CMAP_SINGLE, 8143 },
	{ 0x7b35, 0x7b35, PDF_CMAP_SINGLE, 8141 },
	{ 0x7b38, 0x7b38, PDF_CMAP_SINGLE, 8157 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 15726 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 8142 },
	{ 0x7b40, 0x7b40, PDF_CMAP_SINGLE, 8793 },
	{ 0x7b42, 0x7b42, PDF_CMAP_SINGLE, 15853 },
	{ 0x7b43, 0x7b43, PDF_CMAP_SINGLE, 15909 },
	{ 0x7b44, 0x7b44, PDF_CMAP_SINGLE, 8789 },
	{ 0x7b45, 0x7b45, PDF_CMAP_SINGLE, 8795 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 3333 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 8788 },
	{ 0x7b48, 0x7b48, PDF_CMAP_SINGLE, 8790 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 3331 },
	{ 0x7b4a, 0x7b4a, PDF_CMAP_SINGLE, 8787 },
	{ 0x7b4b, 0x7b4b, PDF_CMAP_SINGLE, 3338 },
	{ 0x7b4c, 0x7b4c, PDF_CMAP_SINGLE, 8791 },
	{ 0x7b4d, 0x7b4d, PDF_CMAP_SINGLE, 3337 },
	{ 0x7b4e, 0x7b4e, PDF_CMAP_SINGLE, 8792 },
	{ 0x7b4f, 0x7b4f, PDF_CMAP_SINGLE, 3339 },
	{ 0x7b50, 0x7b50, PDF_CMAP_SINGLE, 3334 },
	{ 0x7b51, 0x7b51, PDF_CMAP_SINGLE, 3340 },
	{ 0x7b52, 0x7b52, PDF_CMAP_SINGLE, 3335 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 3336 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 14151 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 3332 },
	{ 0x7b58, 0x7b58, PDF_CMAP_SINGLE, 8794 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 3768 },
	{ 0x7b61, 0x7b61, PDF_CMAP_SINGLE, 9464 },
	{ 0x7b62, 0x7b62, PDF_CMAP_SINGLE, 18215 },
	{ 0x7b63, 0x7b63, PDF_CMAP_SINGLE, 9467 },
	{ 0x7b64, 0x7b64, PDF_CMAP_SINGLE, 9455 },
	{ 0x7b65, 0x7b65, PDF_CMAP_SINGLE, 9460 },
	{ 0x7b66, 0x7b66, PDF_CMAP_SINGLE, 9454 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 3770 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 9458 },
	{ 0x7b6c, 0x7b6c, PDF_CMAP_SINGLE, 18216 },
	{ 0x7b6d, 0x7b6d, PDF_CMAP_SINGLE, 9456 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 3769 },
	{ 0x7b6f, 0x7b6f, PDF_CMAP_SINGLE, 14156 },
	{ 0x7b70, 0x7b70, PDF_CMAP_SINGLE, 9463 },
	{ 0x7b71, 0x7b71, PDF_CMAP_SINGLE, 9462 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 9459 },
	{ 0x7b73, 0x7b73, PDF_CMAP_SINGLE, 9461 },
	{ 0x7b74, 0x7b74, PDF_CMAP_SINGLE, 9457 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 4176 },
	{ 0x7b76, 0x7b76, PDF_CMAP_SINGLE, 9466 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 3766 },
	{ 0x7b78, 0x7b78, PDF_CMAP_SINGLE, 9465 },
	{ 0x7b7b, 0x7b7b, PDF_CMAP_SINGLE, 18217 },
	{ 0x7b82, 0x7b82, PDF_CMAP_SINGLE, 10105 },
	{ 0x7b84, 0x7b84, PDF_CMAP_SINGLE, 4183 },
	{ 0x7b85, 0x7b85, PDF_CMAP_SINGLE, 10100 },
	{ 0x7b87, 0x7b87, PDF_CMAP_SINGLE, 4182 },
	{ 0x7b88, 0x7b88, PDF_CMAP_SINGLE, 10090 },
	{ 0x7b8a, 0x7b8a, PDF_CMAP_SINGLE, 10092 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 4175 },
	{ 0x7b8c, 0x7b8c, PDF_CMAP_SINGLE, 10097 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 10096 },
	{ 0x7b8e, 0x7b8e, PDF_CMAP_SINGLE, 10099 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 4180 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 10094 },
	{ 0x7b91, 0x7b91, PDF_CMAP_SINGLE, 10093 },
	{ 0x7b92, 0x7b92, PDF_CMAP_SINGLE, 14162 },
	{ 0x7b94, 0x7b94, PDF_CMAP_SINGLE, 4179 },
	{ 0x7b95, 0x7b95, PDF_CMAP_SINGLE, 4174 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 10095 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 4177 },
	{ 0x7b98, 0x7b98, PDF_CMAP_SINGLE, 10101 },
	{ 0x7b99, 0x7b99, PDF_CMAP_SINGLE, 10103 },
	{ 0x7b9b, 0x7b9b, PDF_CMAP_SINGLE, 10098 },
	{ 0x7b9c, 0x7b9c, PDF_CMAP_SINGLE, 10091 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 4178 },
	{ 0x7ba0, 0x7ba0, PDF_CMAP_SINGLE, 4562 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 4173 },
	{ 0x7ba2, 0x7ba2, PDF_CMAP_SINGLE, 14155 },
	{ 0x7ba3, 0x7ba3, PDF_CMAP_SINGLE, 16148 },
	{ 0x7ba4, 0x7ba4, PDF_CMAP_SINGLE, 10104 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 10666 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 4555 },
	{ 0x7baf, 0x7baf, PDF_CMAP_SINGLE, 10668 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 4556 },
	{ 0x7bb2, 0x7bb2, PDF_CMAP_SINGLE, 17948 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 4558 },
	{ 0x7bb5, 0x7bb5, PDF_CMAP_SINGLE, 10671 },
	{ 0x7bb7, 0x7bb7, PDF_CMAP_SINGLE, 10663 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 4181 },
	{ 0x7bb9, 0x7bb9, PDF_CMAP_SINGLE, 10669 },
	{ 0x7bbe, 0x7bbe, PDF_CMAP_SINGLE, 10665 },
	{ 0x7bc0, 0x7bc0, PDF_CMAP_SINGLE, 3767 },
	{ 0x7bc1, 0x7bc1, PDF_CMAP_SINGLE, 4561 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 4557 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_RANGE, 4559 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 4888 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 10670 },
	{ 0x7bcb, 0x7bcb, PDF_CMAP_SINGLE, 10664 },
	{ 0x7bcc, 0x7bcc, PDF_CMAP_SINGLE, 4563 },
	{ 0x7bce, 0x7bce, PDF_CMAP_SINGLE, 10667 },
	{ 0x7bcf, 0x7bcf, PDF_CMAP_SINGLE, 16152 },
	{ 0x7bd0, 0x7bd0, PDF_CMAP_SINGLE, 14160 },
	{ 0x7bd4, 0x7bd4, PDF_CMAP_SINGLE, 11271 },
	{ 0x7bd5, 0x7bd5, PDF_CMAP_SINGLE, 11266 },
	{ 0x7bd8, 0x7bd8, PDF_CMAP_SINGLE, 11276 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 4886 },
	{ 0x7bda, 0x7bda, PDF_CMAP_SINGLE, 11268 },
	{ 0x7bdb, 0x7bdb, PDF_CMAP_SINGLE, 4890 },
	{ 0x7bdc, 0x7bdc, PDF_CMAP_SINGLE, 11274 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 11265 },
	{ 0x7bde, 0x7bde, PDF_CMAP_SINGLE, 11262 },
	{ 0x7bdf, 0x7bdf, PDF_CMAP_SINGLE, 11277 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 5165 },
	{ 0x7be1, 0x7be1, PDF_CMAP_SINGLE, 4891 },
	{ 0x7be2, 0x7be2, PDF_CMAP_SINGLE, 11273 },
	{ 0x7be3, 0x7be3, PDF_CMAP_SINGLE, 11263 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 4889 },
	{ 0x7be5, 0x7be5, PDF_CMAP_SINGLE, 11267 },
	{ 0x7be6, 0x7be6, PDF_CMAP_SINGLE, 4893 },
	{ 0x7be7, 0x7be7, PDF_CMAP_SINGLE, 11264 },
	{ 0x7be8, 0x7be8, PDF_CMAP_SINGLE, 11269 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 4892 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 11272 },
	{ 0x7beb, 0x7beb, PDF_CMAP_SINGLE, 11275 },
	{ 0x7bf0, 0x7bf1, PDF_CMAP_RANGE, 11753 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 11738 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 11745 },
	{ 0x7bf4, 0x7bf4, PDF_CMAP_SINGLE, 11743 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 5163 },
	{ 0x7bf8, 0x7bf8, PDF_CMAP_SINGLE, 11750 },
	{ 0x7bf9, 0x7bf9, PDF_CMAP_SINGLE, 11270 },
	{ 0x7bfa, 0x7bfa, PDF_CMAP_SINGLE, 14167 },
	{ 0x7bfb, 0x7bfb, PDF_CMAP_SINGLE, 11741 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 16159 },
	{ 0x7bfd, 0x7bfd, PDF_CMAP_SINGLE, 11751 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 5162 },
	{ 0x7bff, 0x7bff, PDF_CMAP_SINGLE, 11740 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 11739 },
	{ 0x7c01, 0x7c01, PDF_CMAP_SINGLE, 11749 },
	{ 0x7c02, 0x7c02, PDF_CMAP_SINGLE, 11746 },
	{ 0x7c03, 0x7c03, PDF_CMAP_SINGLE, 11748 },
	{ 0x7c05, 0x7c05, PDF_CMAP_SINGLE, 11736 },
	{ 0x7c06, 0x7c06, PDF_CMAP_SINGLE, 11752 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 5160 },
	{ 0x7c09, 0x7c09, PDF_CMAP_SINGLE, 11747 },
	{ 0x7c0a, 0x7c0a, PDF_CMAP_SINGLE, 11756 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 11744 },
	{ 0x7c0c, 0x7c0c, PDF_CMAP_SINGLE, 5164 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 5161 },
	{ 0x7c0e, 0x7c0e, PDF_CMAP_SINGLE, 11742 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 11737 },
	{ 0x7c10, 0x7c10, PDF_CMAP_SINGLE, 11755 },
	{ 0x7c11, 0x7c11, PDF_CMAP_SINGLE, 4887 },
	{ 0x7c12, 0x7c12, PDF_CMAP_SINGLE, 18218 },
	{ 0x7c15, 0x7c15, PDF_CMAP_SINGLE, 16481 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 12152 },
	{ 0x7c1b, 0x7c1b, PDF_CMAP_SINGLE, 17369 },
	{ 0x7c1c, 0x7c1c, PDF_CMAP_SINGLE, 12150 },
	{ 0x7c1d, 0x7c1d, PDF_CMAP_SINGLE, 12156 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 5384 },
	{ 0x7c1f, 0x7c1f, PDF_CMAP_SINGLE, 12154 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 12153 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 5386 },
	{ 0x7c22, 0x7c22, PDF_CMAP_SINGLE, 12159 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 5385 },
	{ 0x7c25, 0x7c25, PDF_CMAP_SINGLE, 12160 },
	{ 0x7c26, 0x7c26, PDF_CMAP_SINGLE, 12157 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 5382 },
	{ 0x7c28, 0x7c28, PDF_CMAP_SINGLE, 12158 },
	{ 0x7c29, 0x7c29, PDF_CMAP_SINGLE, 12151 },
	{ 0x7c2a, 0x7c2a, PDF_CMAP_SINGLE, 5383 },
	{ 0x7c2b, 0x7c2b, PDF_CMAP_SINGLE, 5381 },
	{ 0x7c2c, 0x7c2c, PDF_CMAP_SINGLE, 12502 },
	{ 0x7c2d, 0x7c2d, PDF_CMAP_SINGLE, 12155 },
	{ 0x7c30, 0x7c30, PDF_CMAP_SINGLE, 12161 },
	{ 0x7c33, 0x7c33, PDF_CMAP_SINGLE, 12499 },
	{ 0x7c35, 0x7c35, PDF_CMAP_SINGLE, 14169 },
	{ 0x7c37, 0x7c37, PDF_CMAP_SINGLE, 5543 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 5541 },
	{ 0x7c39, 0x7c39, PDF_CMAP_SINGLE, 12501 },
	{ 0x7c3b, 0x7c3b, PDF_CMAP_SINGLE, 12503 },
	{ 0x7c3c, 0x7c3c, PDF_CMAP_SINGLE, 12500 },
	{ 0x7c3d, 0x7c3d, PDF_CMAP_SINGLE, 5542 },
	{ 0x7c3e, 0x7c3f, PDF_CMAP_RANGE, 5539 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 5544 },
	{ 0x7c42, 0x7c42, PDF_CMAP_SINGLE, 16156 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 5675 },
	{ 0x7c44, 0x7c44, PDF_CMAP_SINGLE, 14171 },
	{ 0x7c45, 0x7c45, PDF_CMAP_SINGLE, 12818 },
	{ 0x7c47, 0x7c47, PDF_CMAP_SINGLE, 12817 },
	{ 0x7c48, 0x7c48, PDF_CMAP_SINGLE, 12815 },
	{ 0x7c49, 0x7c49, PDF_CMAP_SINGLE, 12814 },
	{ 0x7c4a, 0x7c4a, PDF_CMAP_SINGLE, 12816 },
	{ 0x7c4c, 0x7c4c, PDF_CMAP_SINGLE, 5674 },
	{ 0x7c4d, 0x7c4d, PDF_CMAP_SINGLE, 5676 },
	{ 0x7c50, 0x7c50, PDF_CMAP_SINGLE, 5761 },
	{ 0x7c51, 0x7c51, PDF_CMAP_SINGLE, 16312 },
	{ 0x7c53, 0x7c53, PDF_CMAP_SINGLE, 13034 },
	{ 0x7c54, 0x7c54, PDF_CMAP_SINGLE, 13033 },
	{ 0x7c56, 0x7c56, PDF_CMAP_SINGLE, 16127 },
	{ 0x7c57, 0x7c57, PDF_CMAP_SINGLE, 13211 },
	{ 0x7c59, 0x7c59, PDF_CMAP_SINGLE, 13213 },
	{ 0x7c5a, 0x7c5a, PDF_CMAP_SINGLE, 13215 },
	{ 0x7c5b, 0x7c5b, PDF_CMAP_SINGLE, 13214 },
	{ 0x7c5c, 0x7c5c, PDF_CMAP_SINGLE, 13212 },
	{ 0x7c5d, 0x7c5d, PDF_CMAP_SINGLE, 15167 },
	{ 0x7c5f, 0x7c5f, PDF_CMAP_SINGLE, 5841 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 5840 },
	{ 0x7c63, 0x7c63, PDF_CMAP_SINGLE, 5889 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 5888 },
	{ 0x7c65, 0x7c65, PDF_CMAP_SINGLE, 5890 },
	{ 0x7c66, 0x7c66, PDF_CMAP_SINGLE, 13355 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 13354 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 13526 },
	{ 0x7c6a, 0x7c6a, PDF_CMAP_SINGLE, 13459 },
	{ 0x7c6b, 0x7c6b, PDF_CMAP_SINGLE, 13527 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 5957 },
	{ 0x7c6d, 0x7c6d, PDF_CMAP_SINGLE, 15168 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 5958 },
	{ 0x7c6f, 0x7c6f, PDF_CMAP_SINGLE, 13575 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 16210 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 5995 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 1015 },
	{ 0x7c74, 0x7c74, PDF_CMAP_SINGLE, 18076 },
	{ 0x7c75, 0x7c75, PDF_CMAP_SINGLE, 6693 },
	{ 0x7c78, 0x7c79, PDF_CMAP_RANGE, 7099 },
	{ 0x7c7a, 0x7c7a, PDF_CMAP_SINGLE, 7098 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 18221 },
	{ 0x7c7c, 0x7c7c, PDF_CMAP_SINGLE, 15177 },
	{ 0x7c7d, 0x7c7d, PDF_CMAP_SINGLE, 1996 },
	{ 0x7c7e, 0x7c7e, PDF_CMAP_SINGLE, 16154 },
	{ 0x7c7f, 0x7c81, PDF_CMAP_RANGE, 7101 },
	{ 0x7c83, 0x7c83, PDF_CMAP_SINGLE, 14172 },
	{ 0x7c84, 0x7c84, PDF_CMAP_SINGLE, 7574 },
	{ 0x7c85, 0x7c85, PDF_CMAP_SINGLE, 7580 },
	{ 0x7c86, 0x7c86, PDF_CMAP_SINGLE, 16157 },
	{ 0x7c88, 0x7c88, PDF_CMAP_SINGLE, 7578 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 2410 },
	{ 0x7c8a, 0x7c8a, PDF_CMAP_SINGLE, 7576 },
	{ 0x7c8c, 0x7c8c, PDF_CMAP_SINGLE, 7577 },
	{ 0x7c8d, 0x7c8d, PDF_CMAP_SINGLE, 7579 },
	{ 0x7c8e, 0x7c8e, PDF_CMAP_SINGLE, 15176 },
	{ 0x7c91, 0x7c91, PDF_CMAP_SINGLE, 7575 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 2896 },
	{ 0x7c94, 0x7c94, PDF_CMAP_SINGLE, 8160 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 2898 },
	{ 0x7c96, 0x7c96, PDF_CMAP_SINGLE, 8162 },
	{ 0x7c97, 0x7c97, PDF_CMAP_SINGLE, 2897 },
	{ 0x7c98, 0x7c98, PDF_CMAP_SINGLE, 8161 },
	{ 0x7c9c, 0x7c9c, PDF_CMAP_SINGLE, 18222 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 8797 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 3341 },
	{ 0x7ca1, 0x7ca1, PDF_CMAP_SINGLE, 8799 },
	{ 0x7ca2, 0x7ca2, PDF_CMAP_SINGLE, 8796 },
	{ 0x7ca3, 0x7ca3, PDF_CMAP_SINGLE, 8163 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 3342 },
	{ 0x7ca6, 0x7ca6, PDF_CMAP_SINGLE, 14174 },
	{ 0x7ca7, 0x7ca7, PDF_CMAP_SINGLE, 14061 },
	{ 0x7ca8, 0x7ca8, PDF_CMAP_SINGLE, 8798 },
	{ 0x7cac, 0x7cac, PDF_CMAP_SINGLE, 14469 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 15178 },
	{ 0x7caf, 0x7caf, PDF_CMAP_SINGLE, 9470 },
	{ 0x7cb1, 0x7cb1, PDF_CMAP_SINGLE, 3771 },
	{ 0x7cb2, 0x7cb2, PDF_CMAP_SINGLE, 9468 },
	{ 0x7cb3, 0x7cb3, PDF_CMAP_SINGLE, 3772 },
	{ 0x7cb4, 0x7cb4, PDF_CMAP_SINGLE, 9469 },
	{ 0x7cb5, 0x7cb5, PDF_CMAP_SINGLE, 3773 },
	{ 0x7cb8, 0x7cb8, PDF_CMAP_SINGLE, 18224 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 4184 },
	{ 0x7cba, 0x7cba, PDF_CMAP_SINGLE, 10109 },
	{ 0x7cbb, 0x7cbb, PDF_CMAP_SINGLE, 10106 },
	{ 0x7cbc, 0x7cbc, PDF_CMAP_SINGLE, 10108 },
	{ 0x7cbd, 0x7cbe, PDF_CMAP_RANGE, 4185 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 10107 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 16343 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 10672 },
	{ 0x7cc7, 0x7cc7, PDF_CMAP_SINGLE, 14177 },
	{ 0x7cc8, 0x7cc8, PDF_CMAP_SINGLE, 10673 },
	{ 0x7cc9, 0x7cc9, PDF_CMAP_SINGLE, 14176 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 4564 },
	{ 0x7ccb, 0x7ccb, PDF_CMAP_SINGLE, 10675 },
	{ 0x7ccc, 0x7ccc, PDF_CMAP_SINGLE, 10674 },
	{ 0x7ccd, 0x7ccd, PDF_CMAP_SINGLE, 15173 },
	{ 0x7cce, 0x7cce, PDF_CMAP_SINGLE, 289 },
	{ 0x7cd0, 0x7cd1, PDF_CMAP_RANGE, 11281 },
	{ 0x7cd2, 0x7cd2, PDF_CMAP_SINGLE, 11278 },
	{ 0x7cd3, 0x7cd3, PDF_CMAP_SINGLE, 15708 },
	{ 0x7cd4, 0x7cd4, PDF_CMAP_SINGLE, 11279 },
	{ 0x7cd5, 0x7cd6, PDF_CMAP_RANGE, 4894 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 11280 },
	{ 0x7cd9, 0x7cd9, PDF_CMAP_SINGLE, 5171 },
	{ 0x7cda, 0x7cda, PDF_CMAP_SINGLE, 16345 },
	{ 0x7cdc, 0x7cdc, PDF_CMAP_SINGLE, 5167 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 5172 },
	{ 0x7cde, 0x7cde, PDF_CMAP_SINGLE, 5168 },
	{ 0x7cdf, 0x7cdf, PDF_CMAP_SINGLE, 5170 },
	{ 0x7ce0, 0x7ce0, PDF_CMAP_SINGLE, 5166 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 5169 },
	{ 0x7ce6, 0x7ce6, PDF_CMAP_SINGLE, 14178 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 5387 },
	{ 0x7ce8, 0x7ce8, PDF_CMAP_SINGLE, 11757 },
	{ 0x7cea, 0x7cea, PDF_CMAP_SINGLE, 12505 },
	{ 0x7cec, 0x7cec, PDF_CMAP_SINGLE, 12504 },
	{ 0x7ced, 0x7ced, PDF_CMAP_SINGLE, 17370 },
	{ 0x7cee, 0x7cee, PDF_CMAP_SINGLE, 12819 },
	{ 0x7cef, 0x7cf0, PDF_CMAP_RANGE, 5677 },
	{ 0x7cf1, 0x7cf1, PDF_CMAP_SINGLE, 13217 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 13035 },
	{ 0x7cf3, 0x7cf3, PDF_CMAP_SINGLE, 14180 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 13216 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 14181 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 13528 },
	{ 0x7cf7, 0x7cf7, PDF_CMAP_SINGLE, 13597 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 1016 },
	{ 0x7cf9, 0x7cf9, PDF_CMAP_SINGLE, 17656 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 1275 },
	{ 0x7cfc, 0x7cfc, PDF_CMAP_SINGLE, 17773 },
	{ 0x7cfd, 0x7cfd, PDF_CMAP_SINGLE, 6694 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 1639 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 1999 },
	{ 0x7d01, 0x7d01, PDF_CMAP_SINGLE, 7106 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 1997 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 7104 },
	{ 0x7d04, 0x7d04, PDF_CMAP_SINGLE, 2002 },
	{ 0x7d05, 0x7d05, PDF_CMAP_SINGLE, 1998 },
	{ 0x7d06, 0x7d06, PDF_CMAP_SINGLE, 2003 },
	{ 0x7d07, 0x7d07, PDF_CMAP_SINGLE, 2001 },
	{ 0x7d08, 0x7d08, PDF_CMAP_SINGLE, 7105 },
	{ 0x7d09, 0x7d09, PDF_CMAP_SINGLE, 2000 },
	{ 0x7d0a, 0x7d0a, PDF_CMAP_SINGLE, 2414 },
	{ 0x7d0b, 0x7d0b, PDF_CMAP_SINGLE, 2413 },
	{ 0x7d0c, 0x7d0c, PDF_CMAP_SINGLE, 7591 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 2422 },
	{ 0x7d0e, 0x7d0e, PDF_CMAP_SINGLE, 7584 },
	{ 0x7d0f, 0x7d0f, PDF_CMAP_SINGLE, 7590 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 2418 },
	{ 0x7d11, 0x7d11, PDF_CMAP_SINGLE, 7583 },
	{ 0x7d12, 0x7d12, PDF_CMAP_SINGLE, 7589 },
	{ 0x7d13, 0x7d13, PDF_CMAP_SINGLE, 7587 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 2417 },
	{ 0x7d15, 0x7d15, PDF_CMAP_SINGLE, 2419 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 7586 },
	{ 0x7d17, 0x7d17, PDF_CMAP_SINGLE, 2412 },
	{ 0x7d18, 0x7d18, PDF_CMAP_SINGLE, 7585 },
	{ 0x7d19, 0x7d19, PDF_CMAP_SINGLE, 2423 },
	{ 0x7d1a, 0x7d1a, PDF_CMAP_SINGLE, 2420 },
	{ 0x7d1b, 0x7d1b, PDF_CMAP_SINGLE, 2424 },
	{ 0x7d1c, 0x7d1c, PDF_CMAP_SINGLE, 2421 },
	{ 0x7d1d, 0x7d1d, PDF_CMAP_SINGLE, 7582 },
	{ 0x7d1e, 0x7d1e, PDF_CMAP_SINGLE, 7581 },
	{ 0x7d1f, 0x7d1f, PDF_CMAP_SINGLE, 7588 },
	{ 0x7d20, 0x7d20, PDF_CMAP_SINGLE, 2415 },
	{ 0x7d21, 0x7d21, PDF_CMAP_SINGLE, 2411 },
	{ 0x7d22, 0x7d22, PDF_CMAP_SINGLE, 2416 },
	{ 0x7d25, 0x7d25, PDF_CMAP_SINGLE, 16094 },
	{ 0x7d28, 0x7d28, PDF_CMAP_SINGLE, 8178 },
	{ 0x7d29, 0x7d29, PDF_CMAP_SINGLE, 8171 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 3347 },
	{ 0x7d2c, 0x7d2c, PDF_CMAP_SINGLE, 8170 },
	{ 0x7d2e, 0x7d2e, PDF_CMAP_SINGLE, 2902 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 2909 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 2906 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 2912 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 2911 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 2907 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 8164 },
	{ 0x7d36, 0x7d36, PDF_CMAP_SINGLE, 8167 },
	{ 0x7d38, 0x7d38, PDF_CMAP_SINGLE, 8166 },
	{ 0x7d39, 0x7d39, PDF_CMAP_SINGLE, 2903 },
	{ 0x7d3a, 0x7d3a, PDF_CMAP_SINGLE, 8168 },
	{ 0x7d3b, 0x7d3b, PDF_CMAP_SINGLE, 8177 },
	{ 0x7d3c, 0x7d3c, PDF_CMAP_SINGLE, 2904 },
	{ 0x7d3d, 0x7d3d, PDF_CMAP_SINGLE, 8165 },
	{ 0x7d3e, 0x7d3f, PDF_CMAP_RANGE, 8174 },
	{ 0x7d40, 0x7d40, PDF_CMAP_SINGLE, 2905 },
	{ 0x7d41, 0x7d41, PDF_CMAP_SINGLE, 8172 },
	{ 0x7d42, 0x7d42, PDF_CMAP_SINGLE, 2910 },
	{ 0x7d43, 0x7d43, PDF_CMAP_SINGLE, 2900 },
	{ 0x7d44, 0x7d44, PDF_CMAP_SINGLE, 2908 },
	{ 0x7d45, 0x7d45, PDF_CMAP_SINGLE, 8169 },
	{ 0x7d46, 0x7d46, PDF_CMAP_SINGLE, 2899 },
	{ 0x7d47, 0x7d47, PDF_CMAP_SINGLE, 8173 },
	{ 0x7d4a, 0x7d4a, PDF_CMAP_SINGLE, 8176 },
	{ 0x7d4d, 0x7d4d, PDF_CMAP_SINGLE, 16349 },
	{ 0x7d4e, 0x7d4e, PDF_CMAP_SINGLE, 8816 },
	{ 0x7d4f, 0x7d4f, PDF_CMAP_SINGLE, 8807 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 3344 },
	{ 0x7d51, 0x7d51, PDF_CMAP_SINGLE, 8814 },
	{ 0x7d52, 0x7d52, PDF_CMAP_SINGLE, 8811 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 8803 },
	{ 0x7d54, 0x7d54, PDF_CMAP_SINGLE, 8812 },
	{ 0x7d55, 0x7d55, PDF_CMAP_SINGLE, 3346 },
	{ 0x7d56, 0x7d56, PDF_CMAP_SINGLE, 8804 },
	{ 0x7d58, 0x7d58, PDF_CMAP_SINGLE, 8800 },
	{ 0x7d5a, 0x7d5a, PDF_CMAP_SINGLE, 16019 },
	{ 0x7d5b, 0x7d5b, PDF_CMAP_SINGLE, 3779 },
	{ 0x7d5c, 0x7d5c, PDF_CMAP_SINGLE, 8809 },
	{ 0x7d5d, 0x7d5d, PDF_CMAP_SINGLE, 14185 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 3343 },
	{ 0x7d5f, 0x7d5f, PDF_CMAP_SINGLE, 8815 },
	{ 0x7d61, 0x7d61, PDF_CMAP_SINGLE, 3350 },
	{ 0x7d62, 0x7d62, PDF_CMAP_SINGLE, 3352 },
	{ 0x7d63, 0x7d63, PDF_CMAP_SINGLE, 8802 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 3351 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 8805 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 3345 },
	{ 0x7d69, 0x7d69, PDF_CMAP_SINGLE, 8813 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 8806 },
	{ 0x7d6b, 0x7d6b, PDF_CMAP_SINGLE, 8810 },
	{ 0x7d6d, 0x7d6d, PDF_CMAP_SINGLE, 8808 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 3348 },
	{ 0x7d6f, 0x7d6f, PDF_CMAP_SINGLE, 8801 },
	{ 0x7d70, 0x7d70, PDF_CMAP_SINGLE, 3353 },
	{ 0x7d71, 0x7d71, PDF_CMAP_SINGLE, 2901 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 3349 },
	{ 0x7d73, 0x7d73, PDF_CMAP_SINGLE, 3354 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 3775 },
	{ 0x7d7a, 0x7d7a, PDF_CMAP_SINGLE, 9477 },
	{ 0x7d7b, 0x7d7b, PDF_CMAP_SINGLE, 9479 },
	{ 0x7d7c, 0x7d7c, PDF_CMAP_SINGLE, 9481 },
	{ 0x7d7d, 0x7d7d, PDF_CMAP_SINGLE, 9485 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 9475 },
	{ 0x7d80, 0x7d80, PDF_CMAP_SINGLE, 9473 },
	{ 0x7d81, 0x7d81, PDF_CMAP_SINGLE, 3777 },
	{ 0x7d83, 0x7d83, PDF_CMAP_SINGLE, 9480 },
	{ 0x7d84, 0x7d84, PDF_CMAP_SINGLE, 9484 },
	{ 0x7d85, 0x7d85, PDF_CMAP_SINGLE, 9476 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 9472 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 9471 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 14187 },
	{ 0x7d8c, 0x7d8c, PDF_CMAP_SINGLE, 9482 },
	{ 0x7d8d, 0x7d8d, PDF_CMAP_SINGLE, 9474 },
	{ 0x7d8e, 0x7d8e, PDF_CMAP_SINGLE, 9478 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 3778 },
	{ 0x7d91, 0x7d91, PDF_CMAP_SINGLE, 3776 },
	{ 0x7d92, 0x7d92, PDF_CMAP_SINGLE, 9486 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 3774 },
	{ 0x7d94, 0x7d94, PDF_CMAP_SINGLE, 9483 },
	{ 0x7d96, 0x7d96, PDF_CMAP_SINGLE, 10126 },
	{ 0x7d97, 0x7d97, PDF_CMAP_SINGLE, 15187 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 4189 },
	{ 0x7d9d, 0x7d9d, PDF_CMAP_SINGLE, 10118 },
	{ 0x7d9e, 0x7d9e, PDF_CMAP_SINGLE, 4577 },
	{ 0x7d9f, 0x7d9f, PDF_CMAP_SINGLE, 10128 },
	{ 0x7da0, 0x7da0, PDF_CMAP_SINGLE, 4192 },
	{ 0x7da1, 0x7da1, PDF_CMAP_SINGLE, 10132 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 4198 },
	{ 0x7da3, 0x7da3, PDF_CMAP_SINGLE, 10113 },
	{ 0x7da4, 0x7da4, PDF_CMAP_SINGLE, 18135 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 10129 },
	{ 0x7da7, 0x7da7, PDF_CMAP_SINGLE, 10110 },
	{ 0x7da8, 0x7da8, PDF_CMAP_SINGLE, 15516 },
	{ 0x7da9, 0x7da9, PDF_CMAP_SINGLE, 10131 },
	{ 0x7daa, 0x7daa, PDF_CMAP_SINGLE, 10114 },
	{ 0x7dab, 0x7dab, PDF_CMAP_SINGLE, 14188 },
	{ 0x7dac, 0x7dac, PDF_CMAP_SINGLE, 4205 },
	{ 0x7dad, 0x7dad, PDF_CMAP_SINGLE, 4202 },
	{ 0x7dae, 0x7dae, PDF_CMAP_SINGLE, 10130 },
	{ 0x7daf, 0x7daf, PDF_CMAP_SINGLE, 10124 },
	{ 0x7db0, 0x7db0, PDF_CMAP_SINGLE, 4188 },
	{ 0x7db1, 0x7db1, PDF_CMAP_SINGLE, 4196 },
	{ 0x7db2, 0x7db2, PDF_CMAP_SINGLE, 4195 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 14190 },
	{ 0x7db4, 0x7db4, PDF_CMAP_SINGLE, 4194 },
	{ 0x7db5, 0x7db5, PDF_CMAP_SINGLE, 4200 },
	{ 0x7db7, 0x7db7, PDF_CMAP_SINGLE, 10111 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 4201 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 10125 },
	{ 0x7dba, 0x7dba, PDF_CMAP_SINGLE, 4197 },
	{ 0x7dbb, 0x7dbb, PDF_CMAP_SINGLE, 4187 },
	{ 0x7dbc, 0x7dbc, PDF_CMAP_SINGLE, 10127 },
	{ 0x7dbd, 0x7dbe, PDF_CMAP_RANGE, 4190 },
	{ 0x7dbf, 0x7dbf, PDF_CMAP_SINGLE, 4199 },
	{ 0x7dc0, 0x7dc0, PDF_CMAP_SINGLE, 10116 },
	{ 0x7dc1, 0x7dc1, PDF_CMAP_SINGLE, 10115 },
	{ 0x7dc2, 0x7dc2, PDF_CMAP_SINGLE, 10112 },
	{ 0x7dc4, 0x7dc4, PDF_CMAP_SINGLE, 10120 },
	{ 0x7dc5, 0x7dc5, PDF_CMAP_SINGLE, 10117 },
	{ 0x7dc6, 0x7dc6, PDF_CMAP_SINGLE, 10121 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 4204 },
	{ 0x7dc9, 0x7dc9, PDF_CMAP_SINGLE, 10133 },
	{ 0x7dca, 0x7dca, PDF_CMAP_SINGLE, 4193 },
	{ 0x7dcb, 0x7dcc, PDF_CMAP_RANGE, 10122 },
	{ 0x7dcd, 0x7dcd, PDF_CMAP_SINGLE, 17774 },
	{ 0x7dce, 0x7dce, PDF_CMAP_SINGLE, 10119 },
	{ 0x7dcf, 0x7dcf, PDF_CMAP_SINGLE, 18229 },
	{ 0x7dd0, 0x7dd0, PDF_CMAP_SINGLE, 18231 },
	{ 0x7dd2, 0x7dd2, PDF_CMAP_SINGLE, 4203 },
	{ 0x7dd3, 0x7dd3, PDF_CMAP_SINGLE, 15182 },
	{ 0x7dd4, 0x7dd4, PDF_CMAP_SINGLE, 18230 },
	{ 0x7dd6, 0x7dd6, PDF_CMAP_SINGLE, 14191 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 10680 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 4569 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 4578 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 4574 },
	{ 0x7ddb, 0x7ddb, PDF_CMAP_SINGLE, 10677 },
	{ 0x7ddc, 0x7ddc, PDF_CMAP_SINGLE, 15180 },
	{ 0x7ddd, 0x7ddd, PDF_CMAP_SINGLE, 4571 },
	{ 0x7dde, 0x7dde, PDF_CMAP_SINGLE, 4575 },
	{ 0x7ddf, 0x7ddf, PDF_CMAP_SINGLE, 10689 },
	{ 0x7de0, 0x7de0, PDF_CMAP_SINGLE, 4565 },
	{ 0x7de1, 0x7de1, PDF_CMAP_SINGLE, 10681 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 4573 },
	{ 0x7de4, 0x7de4, PDF_CMAP_SINGLE, 14194 },
	{ 0x7de5, 0x7de5, PDF_CMAP_SINGLE, 14198 },
	{ 0x7de6, 0x7de6, PDF_CMAP_SINGLE, 10684 },
	{ 0x7de7, 0x7de7, PDF_CMAP_SINGLE, 10679 },
	{ 0x7de8, 0x7de8, PDF_CMAP_SINGLE, 4572 },
	{ 0x7de9, 0x7de9, PDF_CMAP_SINGLE, 4576 },
	{ 0x7dea, 0x7dea, PDF_CMAP_SINGLE, 10678 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 4570 },
	{ 0x7dee, 0x7dee, PDF_CMAP_SINGLE, 10688 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 4567 },
	{ 0x7df0, 0x7df0, PDF_CMAP_SINGLE, 10687 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 10686 },
	{ 0x7df2, 0x7df2, PDF_CMAP_SINGLE, 4579 },
	{ 0x7df3, 0x7df3, PDF_CMAP_SINGLE, 10440 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4566 },
	{ 0x7df5, 0x7df5, PDF_CMAP_SINGLE, 14196 },
	{ 0x7df6, 0x7df6, PDF_CMAP_SINGLE, 10685 },
	{ 0x7df7, 0x7df7, PDF_CMAP_SINGLE, 10676 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 4580 },
	{ 0x7dfa, 0x7dfa, PDF_CMAP_SINGLE, 10683 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 4568 },
	{ 0x7dfd, 0x7dfd, PDF_CMAP_SINGLE, 18232 },
	{ 0x7dfe, 0x7dfe, PDF_CMAP_SINGLE, 15567 },
	{ 0x7e03, 0x7e03, PDF_CMAP_SINGLE, 10682 },
	{ 0x7e07, 0x7e07, PDF_CMAP_SINGLE, 15181 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 4898 },
	{ 0x7e09, 0x7e09, PDF_CMAP_SINGLE, 4903 },
	{ 0x7e0a, 0x7e0a, PDF_CMAP_SINGLE, 4896 },
	{ 0x7e0b, 0x7e0b, PDF_CMAP_SINGLE, 11295 },
	{ 0x7e0c, 0x7e0c, PDF_CMAP_SINGLE, 11286 },
	{ 0x7e0d, 0x7e0d, PDF_CMAP_SINGLE, 11298 },
	{ 0x7e0e, 0x7e0e, PDF_CMAP_SINGLE, 11290 },
	{ 0x7e0f, 0x7e0f, PDF_CMAP_SINGLE, 11296 },
	{ 0x7e10, 0x7e10, PDF_CMAP_SINGLE, 4904 },
	{ 0x7e11, 0x7e11, PDF_CMAP_SINGLE, 4897 },
	{ 0x7e12, 0x7e12, PDF_CMAP_SINGLE, 11283 },
	{ 0x7e13, 0x7e13, PDF_CMAP_SINGLE, 11289 },
	{ 0x7e14, 0x7e14, PDF_CMAP_SINGLE, 11299 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 11292 },
	{ 0x7e16, 0x7e16, PDF_CMAP_SINGLE, 11297 },
	{ 0x7e17, 0x7e17, PDF_CMAP_SINGLE, 11285 },
	{ 0x7e1a, 0x7e1a, PDF_CMAP_SINGLE, 11293 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 4899 },
	{ 0x7e1c, 0x7e1c, PDF_CMAP_SINGLE, 11291 },
	{ 0x7e1d, 0x7e1d, PDF_CMAP_SINGLE, 4902 },
	{ 0x7e1e, 0x7e1e, PDF_CMAP_SINGLE, 4901 },
	{ 0x7e1f, 0x7e20, PDF_CMAP_RANGE, 11287 },
	{ 0x7e21, 0x7e21, PDF_CMAP_SINGLE, 11284 },
	{ 0x7e22, 0x7e22, PDF_CMAP_SINGLE, 11294 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 4900 },
	{ 0x7e24, 0x7e24, PDF_CMAP_SINGLE, 11301 },
	{ 0x7e25, 0x7e25, PDF_CMAP_SINGLE, 11300 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 14207 },
	{ 0x7e29, 0x7e29, PDF_CMAP_SINGLE, 11768 },
	{ 0x7e2a, 0x7e2a, PDF_CMAP_SINGLE, 11764 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 5179 },
	{ 0x7e2d, 0x7e2d, PDF_CMAP_SINGLE, 11758 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 5173 },
	{ 0x7e2f, 0x7e2f, PDF_CMAP_SINGLE, 5189 },
	{ 0x7e30, 0x7e30, PDF_CMAP_SINGLE, 11770 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 5181 },
	{ 0x7e32, 0x7e32, PDF_CMAP_SINGLE, 5177 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 11761 },
	{ 0x7e34, 0x7e34, PDF_CMAP_SINGLE, 5184 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 5187 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 11772 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 5176 },
	{ 0x7e38, 0x7e38, PDF_CMAP_SINGLE, 11763 },
	{ 0x7e39, 0x7e39, PDF_CMAP_SINGLE, 5185 },
	{ 0x7e3a, 0x7e3a, PDF_CMAP_SINGLE, 11774 },
	{ 0x7e3b, 0x7e3b, PDF_CMAP_SINGLE, 11771 },
	{ 0x7e3c, 0x7e3c, PDF_CMAP_SINGLE, 11759 },
	{ 0x7e3d, 0x7e3d, PDF_CMAP_SINGLE, 5180 },
	{ 0x7e3e, 0x7e3e, PDF_CMAP_SINGLE, 5174 },
	{ 0x7e3f, 0x7e3f, PDF_CMAP_SINGLE, 5188 },
	{ 0x7e40, 0x7e40, PDF_CMAP_SINGLE, 11766 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 5183 },
	{ 0x7e42, 0x7e42, PDF_CMAP_SINGLE, 11760 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 5178 },
	{ 0x7e44, 0x7e44, PDF_CMAP_SINGLE, 11773 },
	{ 0x7e45, 0x7e45, PDF_CMAP_SINGLE, 5182 },
	{ 0x7e46, 0x7e46, PDF_CMAP_SINGLE, 5175 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 11767 },
	{ 0x7e48, 0x7e48, PDF_CMAP_SINGLE, 5186 },
	{ 0x7e49, 0x7e49, PDF_CMAP_SINGLE, 11765 },
	{ 0x7e4c, 0x7e4c, PDF_CMAP_SINGLE, 11769 },
	{ 0x7e50, 0x7e50, PDF_CMAP_SINGLE, 12163 },
	{ 0x7e51, 0x7e51, PDF_CMAP_SINGLE, 12169 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 5393 },
	{ 0x7e53, 0x7e53, PDF_CMAP_SINGLE, 12172 },
	{ 0x7e54, 0x7e55, PDF_CMAP_RANGE, 5388 },
	{ 0x7e56, 0x7e56, PDF_CMAP_SINGLE, 12164 },
	{ 0x7e57, 0x7e57, PDF_CMAP_SINGLE, 12171 },
	{ 0x7e58, 0x7e58, PDF_CMAP_SINGLE, 12166 },
	{ 0x7e59, 0x7e59, PDF_CMAP_SINGLE, 5394 },
	{ 0x7e5a, 0x7e5a, PDF_CMAP_SINGLE, 5391 },
	{ 0x7e5c, 0x7e5c, PDF_CMAP_SINGLE, 12162 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 5390 },
	{ 0x7e5f, 0x7e5f, PDF_CMAP_SINGLE, 12168 },
	{ 0x7e60, 0x7e60, PDF_CMAP_SINGLE, 12170 },
	{ 0x7e61, 0x7e61, PDF_CMAP_SINGLE, 5392 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 12167 },
	{ 0x7e63, 0x7e63, PDF_CMAP_SINGLE, 12165 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 18130 },
	{ 0x7e67, 0x7e67, PDF_CMAP_SINGLE, 14182 },
	{ 0x7e68, 0x7e68, PDF_CMAP_SINGLE, 12515 },
	{ 0x7e69, 0x7e6a, PDF_CMAP_RANGE, 5548 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 5545 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 5546 },
	{ 0x7e6e, 0x7e6e, PDF_CMAP_SINGLE, 14203 },
	{ 0x7e6f, 0x7e6f, PDF_CMAP_SINGLE, 12511 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 12509 },
	{ 0x7e72, 0x7e72, PDF_CMAP_SINGLE, 12513 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 5550 },
	{ 0x7e74, 0x7e74, PDF_CMAP_SINGLE, 12514 },
	{ 0x7e75, 0x7e75, PDF_CMAP_SINGLE, 12507 },
	{ 0x7e76, 0x7e76, PDF_CMAP_SINGLE, 12506 },
	{ 0x7e77, 0x7e77, PDF_CMAP_SINGLE, 12510 },
	{ 0x7e78, 0x7e78, PDF_CMAP_SINGLE, 12508 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 5547 },
	{ 0x7e7a, 0x7e7a, PDF_CMAP_SINGLE, 12512 },
	{ 0x7e7b, 0x7e7b, PDF_CMAP_SINGLE, 12820 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 5681 },
	{ 0x7e7d, 0x7e7d, PDF_CMAP_SINGLE, 5680 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 12821 },
	{ 0x7e7f, 0x7e7f, PDF_CMAP_SINGLE, 15953 },
	{ 0x7e80, 0x7e80, PDF_CMAP_SINGLE, 12823 },
	{ 0x7e81, 0x7e81, PDF_CMAP_SINGLE, 12822 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 5682 },
	{ 0x7e86, 0x7e86, PDF_CMAP_SINGLE, 13040 },
	{ 0x7e87, 0x7e88, PDF_CMAP_RANGE, 13037 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 13036 },
	{ 0x7e8b, 0x7e8b, PDF_CMAP_SINGLE, 13039 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 5763 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 13041 },
	{ 0x7e8e, 0x7e8e, PDF_CMAP_SINGLE, 16073 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 5762 },
	{ 0x7e91, 0x7e91, PDF_CMAP_SINGLE, 13218 },
	{ 0x7e92, 0x7e92, PDF_CMAP_SINGLE, 18078 },
	{ 0x7e93, 0x7e93, PDF_CMAP_SINGLE, 5891 },
	{ 0x7e94, 0x7e94, PDF_CMAP_SINGLE, 5893 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 13356 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 5892 },
	{ 0x7e97, 0x7e97, PDF_CMAP_SINGLE, 13460 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 13530 },
	{ 0x7e99, 0x7e99, PDF_CMAP_SINGLE, 13532 },
	{ 0x7e9a, 0x7e9a, PDF_CMAP_SINGLE, 13529 },
	{ 0x7e9b, 0x7e9b, PDF_CMAP_SINGLE, 13531 },
	{ 0x7e9c, 0x7e9c, PDF_CMAP_SINGLE, 5977 },
	{ 0x7e9f, 0x7e9f, PDF_CMAP_SINGLE, 18619 },
	{ 0x7ea4, 0x7ea4, PDF_CMAP_SINGLE, 17755 },
	{ 0x7eac, 0x7eac, PDF_CMAP_SINGLE, 17756 },
	{ 0x7eba, 0x7eba, PDF_CMAP_SINGLE, 17757 },
	{ 0x7ec7, 0x7ec7, PDF_CMAP_SINGLE, 17758 },
	{ 0x7ecf, 0x7ecf, PDF_CMAP_SINGLE, 17759 },
	{ 0x7edf, 0x7edf, PDF_CMAP_SINGLE, 17760 },
	{ 0x7f06, 0x7f06, PDF_CMAP_SINGLE, 17761 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 1017 },
	{ 0x7f37, 0x7f37, PDF_CMAP_SINGLE, 17762 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 2004 },
	{ 0x7f39, 0x7f39, PDF_CMAP_SINGLE, 7455 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 2425 },
	{ 0x7f3d, 0x7f3d, PDF_CMAP_SINGLE, 2913 },
	{ 0x7f3e, 0x7f3f, PDF_CMAP_RANGE, 8817 },
	{ 0x7f40, 0x7f41, PDF_CMAP_RANGE, 14208 },
	{ 0x7f43, 0x7f43, PDF_CMAP_SINGLE, 11302 },
	{ 0x7f44, 0x7f44, PDF_CMAP_SINGLE, 5190 },
	{ 0x7f45, 0x7f45, PDF_CMAP_SINGLE, 11775 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 14210 },
	{ 0x7f48, 0x7f48, PDF_CMAP_SINGLE, 5395 },
	{ 0x7f49, 0x7f49, PDF_CMAP_SINGLE, 15977 },
	{ 0x7f4a, 0x7f4a, PDF_CMAP_SINGLE, 12517 },
	{ 0x7f4b, 0x7f4b, PDF_CMAP_SINGLE, 12516 },
	{ 0x7f4c, 0x7f4c, PDF_CMAP_SINGLE, 5683 },
	{ 0x7f4d, 0x7f4d, PDF_CMAP_SINGLE, 13042 },
	{ 0x7f4e, 0x7f4e, PDF_CMAP_SINGLE, 15183 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 13219 },
	{ 0x7f50, 0x7f50, PDF_CMAP_SINGLE, 5926 },
	{ 0x7f51, 0x7f51, PDF_CMAP_SINGLE, 6162 },
	{ 0x7f52, 0x7f52, PDF_CMAP_SINGLE, 17654 },
	{ 0x7f53, 0x7f53, PDF_CMAP_SINGLE, 17657 },
	{ 0x7f54, 0x7f54, PDF_CMAP_SINGLE, 1640 },
	{ 0x7f55, 0x7f55, PDF_CMAP_SINGLE, 1276 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 7107 },
	{ 0x7f5b, 0x7f5b, PDF_CMAP_SINGLE, 7597 },
	{ 0x7f5c, 0x7f5c, PDF_CMAP_SINGLE, 7592 },
	{ 0x7f5d, 0x7f5d, PDF_CMAP_SINGLE, 7596 },
	{ 0x7f5e, 0x7f5e, PDF_CMAP_SINGLE, 7594 },
	{ 0x7f5f, 0x7f5f, PDF_CMAP_SINGLE, 2426 },
	{ 0x7f60, 0x7f60, PDF_CMAP_SINGLE, 7595 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 7593 },
	{ 0x7f63, 0x7f63, PDF_CMAP_SINGLE, 8179 },
	{ 0x7f65, 0x7f66, PDF_CMAP_RANGE, 8819 },
	{ 0x7f67, 0x7f68, PDF_CMAP_RANGE, 9489 },
	{ 0x7f69, 0x7f6a, PDF_CMAP_RANGE, 3781 },
	{ 0x7f6b, 0x7f6b, PDF_CMAP_SINGLE, 9488 },
	{ 0x7f6c, 0x7f6c, PDF_CMAP_SINGLE, 9491 },
	{ 0x7f6d, 0x7f6d, PDF_CMAP_SINGLE, 9487 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 3780 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 4206 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 16391 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 3783 },
	{ 0x7f73, 0x7f73, PDF_CMAP_SINGLE, 10134 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 4581 },
	{ 0x7f76, 0x7f76, PDF_CMAP_SINGLE, 10690 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 4582 },
	{ 0x7f78, 0x7f78, PDF_CMAP_SINGLE, 16251 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 4905 },
	{ 0x7f7a, 0x7f7a, PDF_CMAP_SINGLE, 11305 },
	{ 0x7f7b, 0x7f7c, PDF_CMAP_RANGE, 11303 },
	{ 0x7f7d, 0x7f7d, PDF_CMAP_SINGLE, 11778 },
	{ 0x7f7e, 0x7f7e, PDF_CMAP_SINGLE, 11777 },
	{ 0x7f7f, 0x7f7f, PDF_CMAP_SINGLE, 11776 },
	{ 0x7f83, 0x7f83, PDF_CMAP_SINGLE, 12518 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 5551 },
	{ 0x7f86, 0x7f86, PDF_CMAP_SINGLE, 12519 },
	{ 0x7f87, 0x7f87, PDF_CMAP_SINGLE, 13220 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 5927 },
	{ 0x7f89, 0x7f89, PDF_CMAP_SINGLE, 13461 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 1018 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 1642 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 1641 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 7109 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 2005 },
	{ 0x7f8f, 0x7f8f, PDF_CMAP_SINGLE, 16924 },
	{ 0x7f91, 0x7f91, PDF_CMAP_SINGLE, 7108 },
	{ 0x7f92, 0x7f92, PDF_CMAP_SINGLE, 7599 },
	{ 0x7f93, 0x7f93, PDF_CMAP_SINGLE, 17371 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 2427 },
	{ 0x7f95, 0x7f95, PDF_CMAP_SINGLE, 8180 },
	{ 0x7f96, 0x7f96, PDF_CMAP_SINGLE, 7598 },
	{ 0x7f97, 0x7f97, PDF_CMAP_SINGLE, 14214 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 2915 },
	{ 0x7f9b, 0x7f9b, PDF_CMAP_SINGLE, 8183 },
	{ 0x7f9c, 0x7f9d, PDF_CMAP_RANGE, 8181 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 2914 },
	{ 0x7fa0, 0x7fa1, PDF_CMAP_RANGE, 8822 },
	{ 0x7fa2, 0x7fa2, PDF_CMAP_SINGLE, 8821 },
	{ 0x7fa3, 0x7fa3, PDF_CMAP_SINGLE, 14216 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 3786 },
	{ 0x7fa5, 0x7fa5, PDF_CMAP_SINGLE, 9493 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 9492 },
	{ 0x7fa7, 0x7fa7, PDF_CMAP_SINGLE, 9494 },
	{ 0x7fa8, 0x7fa8, PDF_CMAP_SINGLE, 3785 },
	{ 0x7fa9, 0x7fa9, PDF_CMAP_SINGLE, 3784 },
	{ 0x7fac, 0x7fac, PDF_CMAP_SINGLE, 10691 },
	{ 0x7fad, 0x7fad, PDF_CMAP_SINGLE, 10693 },
	{ 0x7fae, 0x7fae, PDF_CMAP_SINGLE, 17372 },
	{ 0x7faf, 0x7faf, PDF_CMAP_SINGLE, 4583 },
	{ 0x7fb0, 0x7fb0, PDF_CMAP_SINGLE, 10692 },
	{ 0x7fb1, 0x7fb1, PDF_CMAP_SINGLE, 11306 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 4906 },
	{ 0x7fb3, 0x7fb3, PDF_CMAP_SINGLE, 12174 },
	{ 0x7fb4, 0x7fb4, PDF_CMAP_SINGLE, 18233 },
	{ 0x7fb5, 0x7fb5, PDF_CMAP_SINGLE, 12173 },
	{ 0x7fb6, 0x7fb6, PDF_CMAP_SINGLE, 5552 },
	{ 0x7fb7, 0x7fb7, PDF_CMAP_SINGLE, 12520 },
	{ 0x7fb8, 0x7fb8, PDF_CMAP_SINGLE, 5554 },
	{ 0x7fb9, 0x7fb9, PDF_CMAP_SINGLE, 5553 },
	{ 0x7fba, 0x7fba, PDF_CMAP_SINGLE, 12824 },
	{ 0x7fbb, 0x7fbb, PDF_CMAP_SINGLE, 13043 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 5764 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 1019 },
	{ 0x7fbe, 0x7fbe, PDF_CMAP_SINGLE, 7110 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 2006 },
	{ 0x7fc0, 0x7fc0, PDF_CMAP_SINGLE, 7602 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 2429 },
	{ 0x7fc2, 0x7fc2, PDF_CMAP_SINGLE, 7601 },
	{ 0x7fc3, 0x7fc3, PDF_CMAP_SINGLE, 7600 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 2428 },
	{ 0x7fc7, 0x7fc7, PDF_CMAP_SINGLE, 8189 },
	{ 0x7fc9, 0x7fc9, PDF_CMAP_SINGLE, 8191 },
	{ 0x7fca, 0x7fcb, PDF_CMAP_RANGE, 8184 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 2916 },
	{ 0x7fcd, 0x7fcd, PDF_CMAP_SINGLE, 8186 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 2917 },
	{ 0x7fcf, 0x7fcf, PDF_CMAP_SINGLE, 8190 },
	{ 0x7fd0, 0x7fd1, PDF_CMAP_RANGE, 8187 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2918 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_RANGE, 3356 },
	{ 0x7fd7, 0x7fd7, PDF_CMAP_SINGLE, 8824 },
	{ 0x7fdb, 0x7fdc, PDF_CMAP_RANGE, 9495 },
	{ 0x7fdd, 0x7fdd, PDF_CMAP_SINGLE, 15198 },
	{ 0x7fde, 0x7fde, PDF_CMAP_SINGLE, 10138 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 4209 },
	{ 0x7fe0, 0x7fe1, PDF_CMAP_RANGE, 4207 },
	{ 0x7fe2, 0x7fe3, PDF_CMAP_RANGE, 10135 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 10137 },
	{ 0x7fe6, 0x7fe6, PDF_CMAP_SINGLE, 10698 },
	{ 0x7fe7, 0x7fe7, PDF_CMAP_SINGLE, 18153 },
	{ 0x7fe8, 0x7fe8, PDF_CMAP_SINGLE, 10699 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 4584 },
	{ 0x7fea, 0x7fea, PDF_CMAP_SINGLE, 10696 },
	{ 0x7feb, 0x7feb, PDF_CMAP_SINGLE, 10695 },
	{ 0x7fec, 0x7fec, PDF_CMAP_SINGLE, 10697 },
	{ 0x7fed, 0x7fed, PDF_CMAP_SINGLE, 10694 },
	{ 0x7fee, 0x7fee, PDF_CMAP_SINGLE, 4909 },
	{ 0x7fef, 0x7fef, PDF_CMAP_SINGLE, 11307 },
	{ 0x7ff0, 0x7ff1, PDF_CMAP_RANGE, 4907 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 11780 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 5191 },
	{ 0x7ff4, 0x7ff4, PDF_CMAP_SINGLE, 11779 },
	{ 0x7ff5, 0x7ff5, PDF_CMAP_SINGLE, 12031 },
	{ 0x7ff7, 0x7ff8, PDF_CMAP_RANGE, 12175 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 5396 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 14222 },
	{ 0x7ffb, 0x7ffb, PDF_CMAP_SINGLE, 5397 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 5192 },
	{ 0x7ffd, 0x7ffe, PDF_CMAP_RANGE, 12521 },
	{ 0x7fff, 0x7fff, PDF_CMAP_SINGLE, 12825 },
	{ 0x8000, 0x8000, PDF_CMAP_SINGLE, 5684 },
	{ 0x8001, 0x8001, PDF_CMAP_SINGLE, 1020 },
	{ 0x8002, 0x8002, PDF_CMAP_SINGLE, 17661 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 1021 },
	{ 0x8004, 0x8004, PDF_CMAP_SINGLE, 2431 },
	{ 0x8005, 0x8005, PDF_CMAP_SINGLE, 1643 },
	{ 0x8006, 0x8006, PDF_CMAP_SINGLE, 2430 },
	{ 0x8007, 0x8007, PDF_CMAP_SINGLE, 7111 },
	{ 0x8008, 0x8008, PDF_CMAP_SINGLE, 14225 },
	{ 0x800b, 0x800b, PDF_CMAP_SINGLE, 3358 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 1022 },
	{ 0x800d, 0x800d, PDF_CMAP_SINGLE, 2008 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 7112 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 2007 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 2009 },
	{ 0x8012, 0x8012, PDF_CMAP_SINGLE, 1023 },
	{ 0x8014, 0x8014, PDF_CMAP_SINGLE, 7114 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 2433 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 7603 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 2435 },
	{ 0x8018, 0x8018, PDF_CMAP_SINGLE, 2432 },
	{ 0x8019, 0x8019, PDF_CMAP_SINGLE, 2434 },
	{ 0x801b, 0x801b, PDF_CMAP_SINGLE, 8194 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 2919 },
	{ 0x801d, 0x801d, PDF_CMAP_SINGLE, 14226 },
	{ 0x801e, 0x801e, PDF_CMAP_SINGLE, 8193 },
	{ 0x801f, 0x801f, PDF_CMAP_SINGLE, 8192 },
	{ 0x8020, 0x8020, PDF_CMAP_SINGLE, 18236 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 9497 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 10139 },
	{ 0x8025, 0x8025, PDF_CMAP_SINGLE, 18237 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 4585 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 4910 },
	{ 0x8029, 0x8029, PDF_CMAP_SINGLE, 11309 },
	{ 0x802a, 0x802a, PDF_CMAP_SINGLE, 11308 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 11781 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 18238 },
	{ 0x802f, 0x802f, PDF_CMAP_SINGLE, 14228 },
	{ 0x8030, 0x8030, PDF_CMAP_SINGLE, 13044 },
	{ 0x8031, 0x8031, PDF_CMAP_SINGLE, 18239 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 1024 },
	{ 0x8034, 0x8034, PDF_CMAP_SINGLE, 6360 },
	{ 0x8035, 0x8035, PDF_CMAP_SINGLE, 6695 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 2010 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 7115 },
	{ 0x8039, 0x8039, PDF_CMAP_SINGLE, 7605 },
	{ 0x803b, 0x803c, PDF_CMAP_RANGE, 14231 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 2436 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 7604 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 2437 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 8196 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 2921 },
	{ 0x8047, 0x8047, PDF_CMAP_SINGLE, 8195 },
	{ 0x8048, 0x8048, PDF_CMAP_SINGLE, 8197 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 2920 },
	{ 0x804f, 0x8050, PDF_CMAP_RANGE, 8826 },
	{ 0x8051, 0x8051, PDF_CMAP_SINGLE, 8825 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 3359 },
	{ 0x8054, 0x8054, PDF_CMAP_SINGLE, 18240 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 3787 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 3788 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 4211 },
	{ 0x805b, 0x805b, PDF_CMAP_SINGLE, 16357 },
	{ 0x805c, 0x805c, PDF_CMAP_SINGLE, 10141 },
	{ 0x805d, 0x805d, PDF_CMAP_SINGLE, 10140 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 4210 },
	{ 0x8061, 0x8061, PDF_CMAP_SINGLE, 14233 },
	{ 0x8062, 0x8062, PDF_CMAP_SINGLE, 16355 },
	{ 0x8063, 0x8063, PDF_CMAP_SINGLE, 16348 },
	{ 0x8064, 0x8064, PDF_CMAP_SINGLE, 10700 },
	{ 0x8066, 0x8066, PDF_CMAP_SINGLE, 16347 },
	{ 0x8067, 0x8067, PDF_CMAP_SINGLE, 10701 },
	{ 0x806c, 0x806c, PDF_CMAP_SINGLE, 11310 },
	{ 0x806f, 0x806f, PDF_CMAP_SINGLE, 5196 },
	{ 0x8070, 0x8070, PDF_CMAP_SINGLE, 5195 },
	{ 0x8071, 0x8072, PDF_CMAP_RANGE, 5193 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 5197 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 12177 },
	{ 0x8076, 0x8076, PDF_CMAP_SINGLE, 5399 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 5398 },
	{ 0x8078, 0x8078, PDF_CMAP_SINGLE, 12523 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 12826 },
	{ 0x807d, 0x807d, PDF_CMAP_SINGLE, 5843 },
	{ 0x807e, 0x807e, PDF_CMAP_SINGLE, 5842 },
	{ 0x807f, 0x807f, PDF_CMAP_SINGLE, 1025 },
	{ 0x8080, 0x8080, PDF_CMAP_SINGLE, 17662 },
	{ 0x8082, 0x8082, PDF_CMAP_SINGLE, 7404 },
	{ 0x8084, 0x8084, PDF_CMAP_SINGLE, 3790 },
	{ 0x8085, 0x8085, PDF_CMAP_SINGLE, 3360 },
	{ 0x8086, 0x8086, PDF_CMAP_SINGLE, 3789 },
	{ 0x8087, 0x8087, PDF_CMAP_SINGLE, 4212 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 1026 },
	{ 0x808a, 0x808a, PDF_CMAP_SINGLE, 6067 },
	{ 0x808b, 0x808c, PDF_CMAP_RANGE, 1027 },
	{ 0x808f, 0x808f, PDF_CMAP_SINGLE, 6696 },
	{ 0x8090, 0x8090, PDF_CMAP_SINGLE, 6363 },
	{ 0x8092, 0x8092, PDF_CMAP_SINGLE, 6364 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 1278 },
	{ 0x8095, 0x8095, PDF_CMAP_SINGLE, 6361 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 1277 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 1280 },
	{ 0x8099, 0x8099, PDF_CMAP_SINGLE, 6362 },
	{ 0x809a, 0x809a, PDF_CMAP_SINGLE, 1282 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 1281 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 6365 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 1279 },
	{ 0x809f, 0x809f, PDF_CMAP_SINGLE, 17782 },
	{ 0x80a1, 0x80a1, PDF_CMAP_SINGLE, 1648 },
	{ 0x80a2, 0x80a2, PDF_CMAP_SINGLE, 1646 },
	{ 0x80a3, 0x80a3, PDF_CMAP_SINGLE, 6698 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 1645 },
	{ 0x80a7, 0x80a7, PDF_CMAP_SINGLE, 14240 },
	{ 0x80a9, 0x80a9, PDF_CMAP_SINGLE, 1650 },
	{ 0x80aa, 0x80aa, PDF_CMAP_SINGLE, 1652 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 1649 },
	{ 0x80ad, 0x80ad, PDF_CMAP_SINGLE, 6701 },
	{ 0x80ae, 0x80ae, PDF_CMAP_SINGLE, 6697 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 1653 },
	{ 0x80b1, 0x80b1, PDF_CMAP_SINGLE, 1647 },
	{ 0x80b2, 0x80b2, PDF_CMAP_SINGLE, 1283 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 1651 },
	{ 0x80b5, 0x80b5, PDF_CMAP_SINGLE, 6700 },
	{ 0x80b6, 0x80b6, PDF_CMAP_SINGLE, 16046 },
	{ 0x80b7, 0x80b7, PDF_CMAP_SINGLE, 18243 },
	{ 0x80b8, 0x80b8, PDF_CMAP_SINGLE, 6699 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 1644 },
	{ 0x80bc, 0x80bc, PDF_CMAP_SINGLE, 17778 },
	{ 0x80bd, 0x80bd, PDF_CMAP_SINGLE, 15995 },
	{ 0x80c2, 0x80c2, PDF_CMAP_SINGLE, 7121 },
	{ 0x80c3, 0x80c4, PDF_CMAP_RANGE, 2014 },
	{ 0x80c5, 0x80c5, PDF_CMAP_SINGLE, 7123 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 15220 },
	{ 0x80c7, 0x80c7, PDF_CMAP_SINGLE, 7117 },
	{ 0x80c8, 0x80c8, PDF_CMAP_SINGLE, 7120 },
	{ 0x80c9, 0x80c9, PDF_CMAP_SINGLE, 7129 },
	{ 0x80ca, 0x80ca, PDF_CMAP_SINGLE, 7127 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 2016 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 7133 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 2019 },
	{ 0x80cf, 0x80cf, PDF_CMAP_SINGLE, 7130 },
	{ 0x80d0, 0x80d0, PDF_CMAP_SINGLE, 7122 },
	{ 0x80d1, 0x80d1, PDF_CMAP_SINGLE, 7119 },
	{ 0x80d4, 0x80d4, PDF_CMAP_SINGLE, 8829 },
	{ 0x80d5, 0x80d5, PDF_CMAP_SINGLE, 7128 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 2011 },
	{ 0x80d7, 0x80d7, PDF_CMAP_SINGLE, 7131 },
	{ 0x80d8, 0x80d8, PDF_CMAP_SINGLE, 7116 },
	{ 0x80d9, 0x80d9, PDF_CMAP_SINGLE, 7125 },
	{ 0x80da, 0x80da, PDF_CMAP_SINGLE, 2013 },
	{ 0x80db, 0x80db, PDF_CMAP_SINGLE, 2018 },
	{ 0x80dc, 0x80dc, PDF_CMAP_SINGLE, 7126 },
	{ 0x80dd, 0x80dd, PDF_CMAP_SINGLE, 2022 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 2020 },
	{ 0x80e0, 0x80e0, PDF_CMAP_SINGLE, 7118 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 2017 },
	{ 0x80e3, 0x80e3, PDF_CMAP_SINGLE, 7124 },
	{ 0x80e4, 0x80e4, PDF_CMAP_SINGLE, 2021 },
	{ 0x80e5, 0x80e5, PDF_CMAP_SINGLE, 2012 },
	{ 0x80e6, 0x80e6, PDF_CMAP_SINGLE, 7132 },
	{ 0x80e9, 0x80e9, PDF_CMAP_SINGLE, 18244 },
	{ 0x80ec, 0x80ec, PDF_CMAP_SINGLE, 17897 },
	{ 0x80ed, 0x80ed, PDF_CMAP_SINGLE, 2442 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 2451 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 2440 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 2438 },
	{ 0x80f2, 0x80f2, PDF_CMAP_SINGLE, 7607 },
	{ 0x80f3, 0x80f3, PDF_CMAP_SINGLE, 2446 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 2443 },
	{ 0x80f5, 0x80f5, PDF_CMAP_SINGLE, 7609 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 17780 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 2445 },
	{ 0x80f9, 0x80f9, PDF_CMAP_SINGLE, 7608 },
	{ 0x80fa, 0x80fa, PDF_CMAP_SINGLE, 7606 },
	{ 0x80fb, 0x80fb, PDF_CMAP_SINGLE, 7611 },
	{ 0x80fc, 0x80fc, PDF_CMAP_SINGLE, 2450 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 2448 },
	{ 0x80fe, 0x80fe, PDF_CMAP_SINGLE, 8828 },
	{ 0x8100, 0x8100, PDF_CMAP_SINGLE, 7612 },
	{ 0x8101, 0x8101, PDF_CMAP_SINGLE, 7610 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 2439 },
	{ 0x8103, 0x8103, PDF_CMAP_SINGLE, 16359 },
	{ 0x8105, 0x8105, PDF_CMAP_SINGLE, 2441 },
	{ 0x8106, 0x8106, PDF_CMAP_SINGLE, 2444 },
	{ 0x8107, 0x8107, PDF_CMAP_SINGLE, 14242 },
	{ 0x8108, 0x8108, PDF_CMAP_SINGLE, 2447 },
	{ 0x8109, 0x8109, PDF_CMAP_SINGLE, 15221 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 2449 },
	{ 0x810c, 0x810c, PDF_CMAP_SINGLE, 18246 },
	{ 0x810e, 0x810e, PDF_CMAP_SINGLE, 18247 },
	{ 0x8112, 0x8112, PDF_CMAP_SINGLE, 18248 },
	{ 0x8114, 0x8114, PDF_CMAP_SINGLE, 18249 },
	{ 0x8115, 0x8115, PDF_CMAP_SINGLE, 8207 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 2923 },
	{ 0x8117, 0x8117, PDF_CMAP_SINGLE, 15981 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 8198 },
	{ 0x8119, 0x8119, PDF_CMAP_SINGLE, 8200 },
	{ 0x811a, 0x811a, PDF_CMAP_SINGLE, 14243 },
	{ 0x811b, 0x811b, PDF_CMAP_SINGLE, 8201 },
	{ 0x811d, 0x811d, PDF_CMAP_SINGLE, 8209 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 8205 },
	{ 0x811f, 0x811f, PDF_CMAP_SINGLE, 8203 },
	{ 0x8121, 0x8121, PDF_CMAP_SINGLE, 8206 },
	{ 0x8122, 0x8122, PDF_CMAP_SINGLE, 8210 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 2924 },
	{ 0x8124, 0x8124, PDF_CMAP_SINGLE, 2928 },
	{ 0x8125, 0x8125, PDF_CMAP_SINGLE, 8199 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 8208 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 2926 },
	{ 0x812a, 0x812a, PDF_CMAP_SINGLE, 14905 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 2925 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 8204 },
	{ 0x812d, 0x812d, PDF_CMAP_SINGLE, 8202 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 2922 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 2927 },
	{ 0x8132, 0x8132, PDF_CMAP_SINGLE, 15993 },
	{ 0x8134, 0x8134, PDF_CMAP_SINGLE, 17900 },
	{ 0x8137, 0x8137, PDF_CMAP_SINGLE, 15218 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 3366 },
	{ 0x813a, 0x813a, PDF_CMAP_SINGLE, 8837 },
	{ 0x813d, 0x813d, PDF_CMAP_SINGLE, 8835 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 3368 },
	{ 0x8142, 0x8142, PDF_CMAP_SINGLE, 15222 },
	{ 0x8143, 0x8143, PDF_CMAP_SINGLE, 8830 },
	{ 0x8144, 0x8144, PDF_CMAP_SINGLE, 9511 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 3367 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 8834 },
	{ 0x8148, 0x8148, PDF_CMAP_SINGLE, 15990 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 8831 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 3363 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 3369 },
	{ 0x814d, 0x814d, PDF_CMAP_SINGLE, 8836 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 3365 },
	{ 0x814f, 0x814f, PDF_CMAP_SINGLE, 8833 },
	{ 0x8150, 0x8150, PDF_CMAP_SINGLE, 4213 },
	{ 0x8151, 0x8151, PDF_CMAP_SINGLE, 3364 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 8832 },
	{ 0x8153, 0x8153, PDF_CMAP_SINGLE, 3370 },
	{ 0x8154, 0x8154, PDF_CMAP_SINGLE, 3362 },
	{ 0x8155, 0x8155, PDF_CMAP_SINGLE, 3361 },
	{ 0x8156, 0x8156, PDF_CMAP_SINGLE, 18252 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 18253 },
	{ 0x815b, 0x815b, PDF_CMAP_SINGLE, 9503 },
	{ 0x815c, 0x815c, PDF_CMAP_SINGLE, 9501 },
	{ 0x815e, 0x815e, PDF_CMAP_SINGLE, 9507 },
	{ 0x8160, 0x8160, PDF_CMAP_SINGLE, 9499 },
	{ 0x8161, 0x8161, PDF_CMAP_SINGLE, 9512 },
	{ 0x8162, 0x8162, PDF_CMAP_SINGLE, 9504 },
	{ 0x8164, 0x8164, PDF_CMAP_SINGLE, 9498 },
	{ 0x8165, 0x8165, PDF_CMAP_SINGLE, 3794 },
	{ 0x8166, 0x8166, PDF_CMAP_SINGLE, 3800 },
	{ 0x8167, 0x8167, PDF_CMAP_SINGLE, 9509 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 9502 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 3797 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 17348 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 3795 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 9510 },
	{ 0x8170, 0x8170, PDF_CMAP_SINGLE, 3792 },
	{ 0x8171, 0x8171, PDF_CMAP_SINGLE, 3791 },
	{ 0x8172, 0x8172, PDF_CMAP_SINGLE, 9505 },
	{ 0x8173, 0x8173, PDF_CMAP_SINGLE, 3796 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 3371 },
	{ 0x8176, 0x8176, PDF_CMAP_SINGLE, 9508 },
	{ 0x8177, 0x8177, PDF_CMAP_SINGLE, 9500 },
	{ 0x8178, 0x8178, PDF_CMAP_SINGLE, 3793 },
	{ 0x8179, 0x817a, PDF_CMAP_RANGE, 3798 },
	{ 0x817c, 0x817c, PDF_CMAP_SINGLE, 18256 },
	{ 0x817f, 0x817f, PDF_CMAP_SINGLE, 4218 },
	{ 0x8180, 0x8180, PDF_CMAP_SINGLE, 4214 },
	{ 0x8182, 0x8182, PDF_CMAP_SINGLE, 4219 },
	{ 0x8183, 0x8183, PDF_CMAP_SINGLE, 10144 },
	{ 0x8184, 0x8184, PDF_CMAP_SINGLE, 17349 },
	{ 0x8186, 0x8186, PDF_CMAP_SINGLE, 10143 },
	{ 0x8187, 0x8187, PDF_CMAP_SINGLE, 10145 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 4216 },
	{ 0x8189, 0x8189, PDF_CMAP_SINGLE, 10142 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 4217 },
	{ 0x818b, 0x818b, PDF_CMAP_SINGLE, 10148 },
	{ 0x818c, 0x818c, PDF_CMAP_SINGLE, 10147 },
	{ 0x818d, 0x818d, PDF_CMAP_SINGLE, 10146 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 4215 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 17350 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 10705 },
	{ 0x8197, 0x8197, PDF_CMAP_SINGLE, 10708 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 4591 },
	{ 0x8199, 0x8199, PDF_CMAP_SINGLE, 10707 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 4590 },
	{ 0x819b, 0x819d, PDF_CMAP_RANGE, 4586 },
	{ 0x819e, 0x819e, PDF_CMAP_SINGLE, 10704 },
	{ 0x819f, 0x819f, PDF_CMAP_SINGLE, 10703 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 4589 },
	{ 0x81a2, 0x81a2, PDF_CMAP_SINGLE, 10706 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 10702 },
	{ 0x81a5, 0x81a5, PDF_CMAP_SINGLE, 18259 },
	{ 0x81a6, 0x81a6, PDF_CMAP_SINGLE, 11312 },
	{ 0x81a7, 0x81a7, PDF_CMAP_SINGLE, 11322 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 4913 },
	{ 0x81a9, 0x81a9, PDF_CMAP_SINGLE, 4912 },
	{ 0x81aa, 0x81aa, PDF_CMAP_SINGLE, 17252 },
	{ 0x81ab, 0x81ab, PDF_CMAP_SINGLE, 11316 },
	{ 0x81ac, 0x81ac, PDF_CMAP_SINGLE, 11318 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 11313 },
	{ 0x81b0, 0x81b0, PDF_CMAP_SINGLE, 11317 },
	{ 0x81b1, 0x81b1, PDF_CMAP_SINGLE, 11311 },
	{ 0x81b2, 0x81b2, PDF_CMAP_SINGLE, 11320 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 4911 },
	{ 0x81b4, 0x81b4, PDF_CMAP_SINGLE, 11319 },
	{ 0x81b5, 0x81b5, PDF_CMAP_SINGLE, 11315 },
	{ 0x81b6, 0x81b6, PDF_CMAP_SINGLE, 16050 },
	{ 0x81b7, 0x81b7, PDF_CMAP_SINGLE, 11321 },
	{ 0x81b9, 0x81b9, PDF_CMAP_SINGLE, 11314 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 5200 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 11782 },
	{ 0x81bc, 0x81bc, PDF_CMAP_SINGLE, 11788 },
	{ 0x81bd, 0x81bd, PDF_CMAP_SINGLE, 5204 },
	{ 0x81be, 0x81be, PDF_CMAP_SINGLE, 5206 },
	{ 0x81bf, 0x81bf, PDF_CMAP_SINGLE, 5203 },
	{ 0x81c0, 0x81c0, PDF_CMAP_SINGLE, 5202 },
	{ 0x81c1, 0x81c1, PDF_CMAP_SINGLE, 18261 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 5201 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 5199 },
	{ 0x81c4, 0x81c4, PDF_CMAP_SINGLE, 11783 },
	{ 0x81c5, 0x81c5, PDF_CMAP_SINGLE, 11786 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 5198 },
	{ 0x81c7, 0x81c7, PDF_CMAP_SINGLE, 11787 },
	{ 0x81c8, 0x81c8, PDF_CMAP_SINGLE, 16366 },
	{ 0x81c9, 0x81c9, PDF_CMAP_SINGLE, 5205 },
	{ 0x81ca, 0x81ca, PDF_CMAP_SINGLE, 11785 },
	{ 0x81cc, 0x81cc, PDF_CMAP_SINGLE, 11784 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 5400 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 5401 },
	{ 0x81d0, 0x81d0, PDF_CMAP_SINGLE, 12180 },
	{ 0x81d1, 0x81d2, PDF_CMAP_RANGE, 12178 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 12525 },
	{ 0x81d7, 0x81d7, PDF_CMAP_SINGLE, 12524 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 5555 },
	{ 0x81d9, 0x81d9, PDF_CMAP_SINGLE, 12828 },
	{ 0x81da, 0x81da, PDF_CMAP_SINGLE, 5685 },
	{ 0x81db, 0x81db, PDF_CMAP_SINGLE, 12827 },
	{ 0x81dd, 0x81dd, PDF_CMAP_SINGLE, 13045 },
	{ 0x81de, 0x81de, PDF_CMAP_SINGLE, 13221 },
	{ 0x81df, 0x81df, PDF_CMAP_SINGLE, 5844 },
	{ 0x81e0, 0x81e1, PDF_CMAP_RANGE, 13533 },
	{ 0x81e2, 0x81e2, PDF_CMAP_SINGLE, 5894 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 1029 },
	{ 0x81e4, 0x81e4, PDF_CMAP_SINGLE, 18262 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 1654 },
	{ 0x81e6, 0x81e6, PDF_CMAP_SINGLE, 8838 },
	{ 0x81e7, 0x81e7, PDF_CMAP_SINGLE, 4220 },
	{ 0x81e8, 0x81e8, PDF_CMAP_SINGLE, 5207 },
	{ 0x81e9, 0x81e9, PDF_CMAP_SINGLE, 11789 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 1030 },
	{ 0x81ec, 0x81ec, PDF_CMAP_SINGLE, 2453 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 2452 },
	{ 0x81ee, 0x81ee, PDF_CMAP_SINGLE, 8839 },
	{ 0x81ef, 0x81ef, PDF_CMAP_SINGLE, 17105 },
	{ 0x81f2, 0x81f2, PDF_CMAP_SINGLE, 11323 },
	{ 0x81f3, 0x81f3, PDF_CMAP_SINGLE, 1031 },
	{ 0x81f4, 0x81f4, PDF_CMAP_SINGLE, 2023 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 15713 },
	{ 0x81f7, 0x81f9, PDF_CMAP_RANGE, 8840 },
	{ 0x81fa, 0x81fa, PDF_CMAP_SINGLE, 4221 },
	{ 0x81fb, 0x81fb, PDF_CMAP_SINGLE, 4914 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 1032 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 1655 },
	{ 0x81ff, 0x81ff, PDF_CMAP_SINGLE, 7134 },
	{ 0x8200, 0x8200, PDF_CMAP_SINGLE, 2454 },
	{ 0x8201, 0x8201, PDF_CMAP_SINGLE, 7613 },
	{ 0x8202, 0x8202, PDF_CMAP_SINGLE, 2929 },
	{ 0x8204, 0x8204, PDF_CMAP_SINGLE, 8843 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 3801 },
	{ 0x8207, 0x8207, PDF_CMAP_SINGLE, 4222 },
	{ 0x8208, 0x8208, PDF_CMAP_SINGLE, 4915 },
	{ 0x8209, 0x8209, PDF_CMAP_SINGLE, 5208 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 5402 },
	{ 0x820b, 0x820b, PDF_CMAP_SINGLE, 12526 },
	{ 0x820c, 0x820c, PDF_CMAP_SINGLE, 1033 },
	{ 0x820d, 0x820d, PDF_CMAP_SINGLE, 1656 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 2455 },
	{ 0x8211, 0x8211, PDF_CMAP_SINGLE, 8211 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 3372 },
	{ 0x8214, 0x8214, PDF_CMAP_SINGLE, 4223 },
	{ 0x8215, 0x8215, PDF_CMAP_SINGLE, 10149 },
	{ 0x8216, 0x8216, PDF_CMAP_SINGLE, 10709 },
	{ 0x8218, 0x8218, PDF_CMAP_SINGLE, 14250 },
	{ 0x821a, 0x821a, PDF_CMAP_SINGLE, 16353 },
	{ 0x821b, 0x821b, PDF_CMAP_SINGLE, 1034 },
	{ 0x821c, 0x821c, PDF_CMAP_SINGLE, 3373 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 9513 },
	{ 0x821e, 0x821e, PDF_CMAP_SINGLE, 4224 },
	{ 0x821f, 0x821f, PDF_CMAP_SINGLE, 1035 },
	{ 0x8220, 0x8220, PDF_CMAP_SINGLE, 6702 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 7135 },
	{ 0x8222, 0x8222, PDF_CMAP_SINGLE, 2024 },
	{ 0x8225, 0x8225, PDF_CMAP_SINGLE, 7615 },
	{ 0x8226, 0x8226, PDF_CMAP_SINGLE, 16909 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 2458 },
	{ 0x8229, 0x8229, PDF_CMAP_SINGLE, 14256 },
	{ 0x822a, 0x822b, PDF_CMAP_RANGE, 2456 },
	{ 0x822c, 0x822c, PDF_CMAP_SINGLE, 2459 },
	{ 0x822d, 0x822d, PDF_CMAP_SINGLE, 16080 },
	{ 0x822f, 0x822f, PDF_CMAP_SINGLE, 7614 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 8216 },
	{ 0x8233, 0x8233, PDF_CMAP_SINGLE, 8213 },
	{ 0x8234, 0x8234, PDF_CMAP_SINGLE, 8215 },
	{ 0x8235, 0x8235, PDF_CMAP_SINGLE, 2930 },
	{ 0x8236, 0x8236, PDF_CMAP_SINGLE, 2932 },
	{ 0x8237, 0x8237, PDF_CMAP_SINGLE, 2931 },
	{ 0x8238, 0x8238, PDF_CMAP_SINGLE, 8212 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 2933 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 8214 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 8844 },
	{ 0x823e, 0x823e, PDF_CMAP_SINGLE, 17794 },
	{ 0x823f, 0x823f, PDF_CMAP_SINGLE, 8846 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 9516 },
	{ 0x8242, 0x8242, PDF_CMAP_SINGLE, 9517 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 9515 },
	{ 0x8245, 0x8245, PDF_CMAP_SINGLE, 9518 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3802 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 9514 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 4225 },
	{ 0x824e, 0x824e, PDF_CMAP_SINGLE, 10714 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 10710 },
	{ 0x8250, 0x8250, PDF_CMAP_SINGLE, 10713 },
	{ 0x8251, 0x8251, PDF_CMAP_SINGLE, 10715 },
	{ 0x8252, 0x8252, PDF_CMAP_SINGLE, 10712 },
	{ 0x8253, 0x8253, PDF_CMAP_SINGLE, 10711 },
	{ 0x8254, 0x8254, PDF_CMAP_SINGLE, 18263 },
	{ 0x8255, 0x8257, PDF_CMAP_RANGE, 11324 },
	{ 0x8258, 0x8259, PDF_CMAP_RANGE, 4916 },
	{ 0x825a, 0x825a, PDF_CMAP_SINGLE, 11791 },
	{ 0x825b, 0x825b, PDF_CMAP_SINGLE, 11790 },
	{ 0x825c, 0x825c, PDF_CMAP_SINGLE, 11792 },
	{ 0x825e, 0x825e, PDF_CMAP_SINGLE, 12182 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 12181 },
	{ 0x8261, 0x8261, PDF_CMAP_SINGLE, 12528 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 15226 },
	{ 0x8263, 0x8263, PDF_CMAP_SINGLE, 12529 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 12527 },
	{ 0x8265, 0x8265, PDF_CMAP_SINGLE, 15227 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 5686 },
	{ 0x8268, 0x8269, PDF_CMAP_RANGE, 12829 },
	{ 0x826b, 0x826b, PDF_CMAP_SINGLE, 13222 },
	{ 0x826c, 0x826c, PDF_CMAP_SINGLE, 13357 },
	{ 0x826d, 0x826d, PDF_CMAP_SINGLE, 13462 },
	{ 0x826e, 0x826e, PDF_CMAP_SINGLE, 1036 },
	{ 0x826f, 0x826f, PDF_CMAP_SINGLE, 1284 },
	{ 0x8271, 0x8271, PDF_CMAP_SINGLE, 5209 },
	{ 0x8272, 0x8272, PDF_CMAP_SINGLE, 1037 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 8217 },
	{ 0x8275, 0x8275, PDF_CMAP_SINGLE, 8847 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 18265 },
	{ 0x8277, 0x8277, PDF_CMAP_SINGLE, 5934 },
	{ 0x8278, 0x8278, PDF_CMAP_SINGLE, 6163 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 18210 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 17763 },
	{ 0x827b, 0x827b, PDF_CMAP_SINGLE, 14280 },
	{ 0x827c, 0x827c, PDF_CMAP_SINGLE, 6164 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 6166 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 1038 },
	{ 0x827f, 0x827f, PDF_CMAP_SINGLE, 6167 },
	{ 0x8280, 0x8280, PDF_CMAP_SINGLE, 6165 },
	{ 0x8283, 0x8284, PDF_CMAP_RANGE, 6373 },
	{ 0x8285, 0x8285, PDF_CMAP_SINGLE, 6368 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 16690 },
	{ 0x828a, 0x828a, PDF_CMAP_SINGLE, 6372 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 1286 },
	{ 0x828d, 0x828d, PDF_CMAP_SINGLE, 1287 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 6369 },
	{ 0x828f, 0x828f, PDF_CMAP_SINGLE, 6367 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 6366 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 6370 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 1285 },
	{ 0x8293, 0x8293, PDF_CMAP_SINGLE, 6371 },
	{ 0x8294, 0x8294, PDF_CMAP_SINGLE, 7136 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 6707 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 1659 },
	{ 0x829a, 0x829a, PDF_CMAP_SINGLE, 6706 },
	{ 0x829b, 0x829b, PDF_CMAP_SINGLE, 6708 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 1658 },
	{ 0x829e, 0x829e, PDF_CMAP_SINGLE, 6713 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 1662 },
	{ 0x82a0, 0x82a0, PDF_CMAP_SINGLE, 6703 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 6717 },
	{ 0x82a2, 0x82a2, PDF_CMAP_SINGLE, 6723 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 1669 },
	{ 0x82a4, 0x82a4, PDF_CMAP_SINGLE, 6720 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 1666 },
	{ 0x82a6, 0x82a6, PDF_CMAP_SINGLE, 15731 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 6710 },
	{ 0x82a8, 0x82a8, PDF_CMAP_SINGLE, 6716 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 6718 },
	{ 0x82aa, 0x82aa, PDF_CMAP_SINGLE, 14268 },
	{ 0x82ab, 0x82ab, PDF_CMAP_SINGLE, 6705 },
	{ 0x82ac, 0x82ac, PDF_CMAP_SINGLE, 1665 },
	{ 0x82ad, 0x82ad, PDF_CMAP_SINGLE, 1660 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 6711 },
	{ 0x82af, 0x82af, PDF_CMAP_SINGLE, 1667 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 1670 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1664 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 1657 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 6715 },
	{ 0x82b5, 0x82b5, PDF_CMAP_SINGLE, 6709 },
	{ 0x82b6, 0x82b6, PDF_CMAP_SINGLE, 6722 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 1672 },
	{ 0x82b8, 0x82b8, PDF_CMAP_SINGLE, 1668 },
	{ 0x82b9, 0x82b9, PDF_CMAP_SINGLE, 1663 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 6714 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 2460 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 6712 },
	{ 0x82bd, 0x82bd, PDF_CMAP_SINGLE, 1661 },
	{ 0x82be, 0x82be, PDF_CMAP_SINGLE, 1671 },
	{ 0x82c0, 0x82c0, PDF_CMAP_SINGLE, 6704 },
	{ 0x82c2, 0x82c2, PDF_CMAP_SINGLE, 6719 },
	{ 0x82c3, 0x82c3, PDF_CMAP_SINGLE, 6721 },
	{ 0x82c4, 0x82c4, PDF_CMAP_SINGLE, 17891 },
	{ 0x82ca, 0x82ca, PDF_CMAP_SINGLE, 18266 },
	{ 0x82cf, 0x82cf, PDF_CMAP_SINGLE, 17764 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 14273 },
	{ 0x82d1, 0x82d1, PDF_CMAP_SINGLE, 2041 },
	{ 0x82d2, 0x82d2, PDF_CMAP_SINGLE, 2035 },
	{ 0x82d3, 0x82d3, PDF_CMAP_SINGLE, 2043 },
	{ 0x82d4, 0x82d4, PDF_CMAP_SINGLE, 2040 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 7143 },
	{ 0x82d6, 0x82d6, PDF_CMAP_SINGLE, 7146 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 2036 },
	{ 0x82d8, 0x82d8, PDF_CMAP_SINGLE, 18267 },
	{ 0x82d9, 0x82d9, PDF_CMAP_SINGLE, 7137 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 2029 },
	{ 0x82dc, 0x82dc, PDF_CMAP_SINGLE, 2039 },
	{ 0x82de, 0x82de, PDF_CMAP_SINGLE, 2042 },
	{ 0x82df, 0x82df, PDF_CMAP_SINGLE, 2044 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 7158 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7149 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 14281 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 2028 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 7157 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 2032 },
	{ 0x82e6, 0x82e6, PDF_CMAP_SINGLE, 2030 },
	{ 0x82e7, 0x82e7, PDF_CMAP_SINGLE, 2025 },
	{ 0x82e8, 0x82e8, PDF_CMAP_SINGLE, 7141 },
	{ 0x82ea, 0x82ea, PDF_CMAP_SINGLE, 7156 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 7145 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7148 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 7161 },
	{ 0x82ee, 0x82ee, PDF_CMAP_SINGLE, 16408 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 2045 },
	{ 0x82f0, 0x82f0, PDF_CMAP_SINGLE, 7155 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 2037 },
	{ 0x82f2, 0x82f2, PDF_CMAP_SINGLE, 7150 },
	{ 0x82f3, 0x82f3, PDF_CMAP_SINGLE, 7160 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 7147 },
	{ 0x82f5, 0x82f5, PDF_CMAP_SINGLE, 7151 },
	{ 0x82f6, 0x82f6, PDF_CMAP_SINGLE, 7154 },
	{ 0x82f7, 0x82f7, PDF_CMAP_SINGLE, 15991 },
	{ 0x82f9, 0x82f9, PDF_CMAP_SINGLE, 7139 },
	{ 0x82fa, 0x82fa, PDF_CMAP_SINGLE, 7159 },
	{ 0x82fb, 0x82fb, PDF_CMAP_SINGLE, 7153 },
	{ 0x82fc, 0x82fc, PDF_CMAP_SINGLE, 16685 },
	{ 0x82fd, 0x82fd, PDF_CMAP_SINGLE, 15762 },
	{ 0x82fe, 0x82fe, PDF_CMAP_SINGLE, 7138 },
	{ 0x82ff, 0x82ff, PDF_CMAP_SINGLE, 17374 },
	{ 0x8300, 0x8300, PDF_CMAP_SINGLE, 7142 },
	{ 0x8301, 0x8301, PDF_CMAP_SINGLE, 2038 },
	{ 0x8302, 0x8302, PDF_CMAP_SINGLE, 2033 },
	{ 0x8303, 0x8303, PDF_CMAP_SINGLE, 2026 },
	{ 0x8304, 0x8304, PDF_CMAP_SINGLE, 2031 },
	{ 0x8305, 0x8305, PDF_CMAP_SINGLE, 2027 },
	{ 0x8306, 0x8306, PDF_CMAP_SINGLE, 2046 },
	{ 0x8307, 0x8307, PDF_CMAP_SINGLE, 7140 },
	{ 0x8308, 0x8308, PDF_CMAP_SINGLE, 7632 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 2034 },
	{ 0x830b, 0x830b, PDF_CMAP_SINGLE, 16054 },
	{ 0x830c, 0x830c, PDF_CMAP_SINGLE, 7152 },
	{ 0x830d, 0x830d, PDF_CMAP_SINGLE, 6781 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 7635 },
	{ 0x8317, 0x8317, PDF_CMAP_SINGLE, 2474 },
	{ 0x8318, 0x8318, PDF_CMAP_SINGLE, 14282 },
	{ 0x8319, 0x8319, PDF_CMAP_SINGLE, 7619 },
	{ 0x831a, 0x831a, PDF_CMAP_SINGLE, 14274 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 7630 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 7626 },
	{ 0x831d, 0x831d, PDF_CMAP_SINGLE, 14289 },
	{ 0x831e, 0x831e, PDF_CMAP_SINGLE, 7645 },
	{ 0x8320, 0x8320, PDF_CMAP_SINGLE, 7637 },
	{ 0x8322, 0x8322, PDF_CMAP_SINGLE, 7627 },
	{ 0x8324, 0x8324, PDF_CMAP_SINGLE, 7636 },
	{ 0x8325, 0x8325, PDF_CMAP_SINGLE, 7621 },
	{ 0x8326, 0x8326, PDF_CMAP_SINGLE, 7625 },
	{ 0x8327, 0x8327, PDF_CMAP_SINGLE, 7648 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 2477 },
	{ 0x8329, 0x8329, PDF_CMAP_SINGLE, 7640 },
	{ 0x832a, 0x832a, PDF_CMAP_SINGLE, 7631 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 2461 },
	{ 0x832c, 0x832c, PDF_CMAP_SINGLE, 7646 },
	{ 0x832d, 0x832d, PDF_CMAP_SINGLE, 7617 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 7639 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 2476 },
	{ 0x8332, 0x8332, PDF_CMAP_SINGLE, 2471 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 7616 },
	{ 0x8334, 0x8334, PDF_CMAP_SINGLE, 2469 },
	{ 0x8335, 0x8335, PDF_CMAP_SINGLE, 2468 },
	{ 0x8336, 0x8336, PDF_CMAP_SINGLE, 2473 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 7638 },
	{ 0x8338, 0x8338, PDF_CMAP_SINGLE, 2465 },
	{ 0x8339, 0x8339, PDF_CMAP_SINGLE, 2472 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 7144 },
	{ 0x833b, 0x833b, PDF_CMAP_SINGLE, 8848 },
	{ 0x833c, 0x833c, PDF_CMAP_SINGLE, 7633 },
	{ 0x833d, 0x833d, PDF_CMAP_SINGLE, 15658 },
	{ 0x833f, 0x833f, PDF_CMAP_SINGLE, 7623 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 2475 },
	{ 0x8341, 0x8341, PDF_CMAP_SINGLE, 7624 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7628 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 2478 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7618 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 7642 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 7641 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7649 },
	{ 0x8349, 0x8349, PDF_CMAP_SINGLE, 2467 },
	{ 0x834a, 0x834a, PDF_CMAP_SINGLE, 2464 },
	{ 0x834b, 0x834b, PDF_CMAP_SINGLE, 7647 },
	{ 0x834c, 0x834c, PDF_CMAP_SINGLE, 7643 },
	{ 0x834d, 0x834d, PDF_CMAP_SINGLE, 7634 },
	{ 0x834e, 0x834e, PDF_CMAP_SINGLE, 7629 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 2470 },
	{ 0x8350, 0x8350, PDF_CMAP_SINGLE, 2466 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 7620 },
	{ 0x8352, 0x8352, PDF_CMAP_SINGLE, 2462 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 7644 },
	{ 0x8354, 0x8354, PDF_CMAP_SINGLE, 2463 },
	{ 0x8356, 0x8356, PDF_CMAP_SINGLE, 7622 },
	{ 0x8357, 0x8357, PDF_CMAP_SINGLE, 18269 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 14262 },
	{ 0x8363, 0x8363, PDF_CMAP_SINGLE, 16875 },
	{ 0x8366, 0x8366, PDF_CMAP_SINGLE, 15902 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 17765 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 8223 },
	{ 0x8374, 0x8374, PDF_CMAP_SINGLE, 8225 },
	{ 0x8375, 0x8375, PDF_CMAP_SINGLE, 8230 },
	{ 0x8376, 0x8376, PDF_CMAP_SINGLE, 8250 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 2947 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 2937 },
	{ 0x837a, 0x837a, PDF_CMAP_SINGLE, 8222 },
	{ 0x837b, 0x837c, PDF_CMAP_RANGE, 2948 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 8233 },
	{ 0x837e, 0x837e, PDF_CMAP_SINGLE, 8240 },
	{ 0x837f, 0x837f, PDF_CMAP_SINGLE, 8246 },
	{ 0x8381, 0x8381, PDF_CMAP_SINGLE, 8227 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 8234 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 14291 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 2950 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 8248 },
	{ 0x8388, 0x8388, PDF_CMAP_SINGLE, 8243 },
	{ 0x8389, 0x8389, PDF_CMAP_SINGLE, 2945 },
	{ 0x838a, 0x838a, PDF_CMAP_SINGLE, 2943 },
	{ 0x838b, 0x838b, PDF_CMAP_SINGLE, 8239 },
	{ 0x838c, 0x838c, PDF_CMAP_SINGLE, 8235 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 8221 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 2934 },
	{ 0x838f, 0x838f, PDF_CMAP_SINGLE, 8226 },
	{ 0x8390, 0x8390, PDF_CMAP_SINGLE, 8218 },
	{ 0x8391, 0x8391, PDF_CMAP_SINGLE, 14316 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 2942 },
	{ 0x8393, 0x8393, PDF_CMAP_SINGLE, 2944 },
	{ 0x8394, 0x8394, PDF_CMAP_SINGLE, 8231 },
	{ 0x8395, 0x8395, PDF_CMAP_SINGLE, 8228 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 2939 },
	{ 0x8397, 0x8397, PDF_CMAP_SINGLE, 8244 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 2936 },
	{ 0x8399, 0x8399, PDF_CMAP_SINGLE, 8229 },
	{ 0x839a, 0x839a, PDF_CMAP_SINGLE, 8889 },
	{ 0x839b, 0x839b, PDF_CMAP_SINGLE, 8237 },
	{ 0x839c, 0x839c, PDF_CMAP_SINGLE, 14157 },
	{ 0x839d, 0x839d, PDF_CMAP_SINGLE, 8236 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 2935 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 2946 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 2938 },
	{ 0x83a3, 0x83a3, PDF_CMAP_SINGLE, 8219 },
	{ 0x83a4, 0x83a4, PDF_CMAP_SINGLE, 8224 },
	{ 0x83a5, 0x83a5, PDF_CMAP_SINGLE, 8241 },
	{ 0x83a6, 0x83a6, PDF_CMAP_SINGLE, 8247 },
	{ 0x83a7, 0x83a7, PDF_CMAP_SINGLE, 2951 },
	{ 0x83a8, 0x83a8, PDF_CMAP_SINGLE, 8220 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 8232 },
	{ 0x83aa, 0x83aa, PDF_CMAP_SINGLE, 8238 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 2941 },
	{ 0x83ac, 0x83ac, PDF_CMAP_SINGLE, 14294 },
	{ 0x83ae, 0x83ae, PDF_CMAP_SINGLE, 8249 },
	{ 0x83af, 0x83af, PDF_CMAP_SINGLE, 8242 },
	{ 0x83b0, 0x83b0, PDF_CMAP_SINGLE, 8245 },
	{ 0x83b9, 0x83b9, PDF_CMAP_SINGLE, 17063 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 2940 },
	{ 0x83be, 0x83be, PDF_CMAP_SINGLE, 16217 },
	{ 0x83bf, 0x83bf, PDF_CMAP_SINGLE, 8864 },
	{ 0x83c0, 0x83c0, PDF_CMAP_SINGLE, 8852 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 3381 },
	{ 0x83c2, 0x83c2, PDF_CMAP_SINGLE, 8881 },
	{ 0x83c3, 0x83c3, PDF_CMAP_SINGLE, 8890 },
	{ 0x83c4, 0x83c4, PDF_CMAP_SINGLE, 8893 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 3379 },
	{ 0x83c6, 0x83c6, PDF_CMAP_SINGLE, 8860 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 8885 },
	{ 0x83c8, 0x83c8, PDF_CMAP_SINGLE, 8861 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 8875 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 3392 },
	{ 0x83cb, 0x83cb, PDF_CMAP_SINGLE, 8871 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 3389 },
	{ 0x83cd, 0x83cd, PDF_CMAP_SINGLE, 16469 },
	{ 0x83ce, 0x83ce, PDF_CMAP_SINGLE, 8872 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 8849 },
	{ 0x83d1, 0x83d1, PDF_CMAP_SINGLE, 8886 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 14296 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 3398 },
	{ 0x83d5, 0x83d5, PDF_CMAP_SINGLE, 8883 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 8873 },
	{ 0x83d7, 0x83d7, PDF_CMAP_SINGLE, 8895 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 8868 },
	{ 0x83d9, 0x83d9, PDF_CMAP_SINGLE, 9550 },
	{ 0x83db, 0x83db, PDF_CMAP_SINGLE, 8898 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 3396 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 8866 },
	{ 0x83de, 0x83de, PDF_CMAP_SINGLE, 8878 },
	{ 0x83df, 0x83df, PDF_CMAP_SINGLE, 3399 },
	{ 0x83e0, 0x83e0, PDF_CMAP_SINGLE, 3378 },
	{ 0x83e1, 0x83e1, PDF_CMAP_SINGLE, 8870 },
	{ 0x83e2, 0x83e2, PDF_CMAP_SINGLE, 8896 },
	{ 0x83e3, 0x83e3, PDF_CMAP_SINGLE, 8863 },
	{ 0x83e4, 0x83e4, PDF_CMAP_SINGLE, 8856 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 8867 },
	{ 0x83e7, 0x83e7, PDF_CMAP_SINGLE, 8855 },
	{ 0x83e8, 0x83e8, PDF_CMAP_SINGLE, 8853 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 3374 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 8887 },
	{ 0x83eb, 0x83eb, PDF_CMAP_SINGLE, 8862 },
	{ 0x83ec, 0x83ec, PDF_CMAP_SINGLE, 8891 },
	{ 0x83ed, 0x83ed, PDF_CMAP_SINGLE, 15660 },
	{ 0x83ee, 0x83ee, PDF_CMAP_SINGLE, 8892 },
	{ 0x83ef, 0x83ef, PDF_CMAP_SINGLE, 3382 },
	{ 0x83f0, 0x83f0, PDF_CMAP_SINGLE, 3387 },
	{ 0x83f1, 0x83f1, PDF_CMAP_SINGLE, 3383 },
	{ 0x83f2, 0x83f2, PDF_CMAP_SINGLE, 3391 },
	{ 0x83f3, 0x83f3, PDF_CMAP_SINGLE, 8882 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 3384 },
	{ 0x83f5, 0x83f5, PDF_CMAP_SINGLE, 8874 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 8858 },
	{ 0x83f8, 0x83f8, PDF_CMAP_SINGLE, 3376 },
	{ 0x83f9, 0x83f9, PDF_CMAP_SINGLE, 8850 },
	{ 0x83fa, 0x83fa, PDF_CMAP_SINGLE, 8884 },
	{ 0x83fb, 0x83fb, PDF_CMAP_SINGLE, 8894 },
	{ 0x83fc, 0x83fc, PDF_CMAP_SINGLE, 8857 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 3390 },
	{ 0x83fe, 0x83fe, PDF_CMAP_SINGLE, 8899 },
	{ 0x83ff, 0x83ff, PDF_CMAP_SINGLE, 8869 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 8865 },
	{ 0x8403, 0x8403, PDF_CMAP_SINGLE, 3375 },
	{ 0x8404, 0x8404, PDF_CMAP_SINGLE, 3395 },
	{ 0x8405, 0x8405, PDF_CMAP_SINGLE, 18271 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 8880 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 3397 },
	{ 0x8409, 0x8409, PDF_CMAP_SINGLE, 8876 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 3386 },
	{ 0x840b, 0x840b, PDF_CMAP_SINGLE, 3380 },
	{ 0x840c, 0x840c, PDF_CMAP_SINGLE, 3388 },
	{ 0x840d, 0x840d, PDF_CMAP_SINGLE, 3377 },
	{ 0x840e, 0x840e, PDF_CMAP_SINGLE, 3394 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 8877 },
	{ 0x8410, 0x8410, PDF_CMAP_SINGLE, 8859 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 8879 },
	{ 0x8412, 0x8412, PDF_CMAP_SINGLE, 8854 },
	{ 0x8413, 0x8413, PDF_CMAP_SINGLE, 8888 },
	{ 0x8414, 0x8414, PDF_CMAP_SINGLE, 15376 },
	{ 0x8416, 0x8416, PDF_CMAP_SINGLE, 16439 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 15994 },
	{ 0x841b, 0x841b, PDF_CMAP_SINGLE, 8897 },
	{ 0x841c, 0x841c, PDF_CMAP_SINGLE, 16051 },
	{ 0x8420, 0x8420, PDF_CMAP_SINGLE, 15445 },
	{ 0x8421, 0x8421, PDF_CMAP_SINGLE, 14161 },
	{ 0x8423, 0x8423, PDF_CMAP_SINGLE, 8851 },
	{ 0x8424, 0x8424, PDF_CMAP_SINGLE, 18769 },
	{ 0x8426, 0x8426, PDF_CMAP_SINGLE, 18136 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 9549 },
	{ 0x842b, 0x842b, PDF_CMAP_SINGLE, 9571 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 3756 },
	{ 0x842d, 0x842d, PDF_CMAP_SINGLE, 9554 },
	{ 0x842e, 0x842e, PDF_CMAP_SINGLE, 16941 },
	{ 0x842f, 0x842f, PDF_CMAP_SINGLE, 9552 },
	{ 0x8430, 0x8430, PDF_CMAP_SINGLE, 9531 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 3806 },
	{ 0x8432, 0x8432, PDF_CMAP_SINGLE, 9547 },
	{ 0x8433, 0x8433, PDF_CMAP_SINGLE, 9567 },
	{ 0x8434, 0x8434, PDF_CMAP_SINGLE, 9543 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 3814 },
	{ 0x8436, 0x8436, PDF_CMAP_SINGLE, 9566 },
	{ 0x8437, 0x8437, PDF_CMAP_SINGLE, 9541 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 3393 },
	{ 0x8439, 0x8439, PDF_CMAP_SINGLE, 9557 },
	{ 0x843a, 0x843a, PDF_CMAP_SINGLE, 9542 },
	{ 0x843b, 0x843b, PDF_CMAP_SINGLE, 9564 },
	{ 0x843c, 0x843c, PDF_CMAP_SINGLE, 3813 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 3805 },
	{ 0x843e, 0x843e, PDF_CMAP_SINGLE, 15649 },
	{ 0x843f, 0x843f, PDF_CMAP_SINGLE, 9520 },
	{ 0x8440, 0x8440, PDF_CMAP_SINGLE, 9528 },
	{ 0x8442, 0x8442, PDF_CMAP_SINGLE, 9553 },
	{ 0x8443, 0x8443, PDF_CMAP_SINGLE, 9545 },
	{ 0x8444, 0x8444, PDF_CMAP_SINGLE, 9570 },
	{ 0x8445, 0x8445, PDF_CMAP_SINGLE, 9548 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 3819 },
	{ 0x8447, 0x8447, PDF_CMAP_SINGLE, 9565 },
	{ 0x8448, 0x8448, PDF_CMAP_SINGLE, 18200 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 3810 },
	{ 0x844a, 0x844a, PDF_CMAP_SINGLE, 14308 },
	{ 0x844b, 0x844b, PDF_CMAP_SINGLE, 9551 },
	{ 0x844c, 0x844c, PDF_CMAP_SINGLE, 9559 },
	{ 0x844d, 0x844d, PDF_CMAP_SINGLE, 9532 },
	{ 0x844e, 0x844e, PDF_CMAP_SINGLE, 9558 },
	{ 0x8450, 0x8450, PDF_CMAP_SINGLE, 9575 },
	{ 0x8451, 0x8451, PDF_CMAP_SINGLE, 9527 },
	{ 0x8452, 0x8452, PDF_CMAP_SINGLE, 9560 },
	{ 0x8453, 0x8453, PDF_CMAP_SINGLE, 15229 },
	{ 0x8454, 0x8454, PDF_CMAP_SINGLE, 9573 },
	{ 0x8455, 0x8455, PDF_CMAP_SINGLE, 14731 },
	{ 0x8456, 0x8456, PDF_CMAP_SINGLE, 9521 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 3385 },
	{ 0x8458, 0x8458, PDF_CMAP_SINGLE, 14302 },
	{ 0x8459, 0x8459, PDF_CMAP_SINGLE, 9535 },
	{ 0x845a, 0x845a, PDF_CMAP_SINGLE, 9534 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 3812 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 16091 },
	{ 0x845d, 0x845d, PDF_CMAP_SINGLE, 9538 },
	{ 0x845e, 0x845e, PDF_CMAP_SINGLE, 9540 },
	{ 0x845f, 0x845f, PDF_CMAP_SINGLE, 9555 },
	{ 0x8460, 0x8460, PDF_CMAP_SINGLE, 9572 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 3815 },
	{ 0x8462, 0x8462, PDF_CMAP_SINGLE, 17114 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 3816 },
	{ 0x8464, 0x8464, PDF_CMAP_SINGLE, 18273 },
	{ 0x8465, 0x8465, PDF_CMAP_SINGLE, 9526 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 3808 },
	{ 0x8467, 0x8467, PDF_CMAP_SINGLE, 9530 },
	{ 0x8468, 0x8468, PDF_CMAP_SINGLE, 9568 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 3817 },
	{ 0x846b, 0x846b, PDF_CMAP_SINGLE, 3809 },
	{ 0x846c, 0x846c, PDF_CMAP_SINGLE, 3811 },
	{ 0x846d, 0x846d, PDF_CMAP_SINGLE, 3818 },
	{ 0x846e, 0x846e, PDF_CMAP_SINGLE, 9574 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 9561 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 9556 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 14304 },
	{ 0x8472, 0x8472, PDF_CMAP_SINGLE, 15507 },
	{ 0x8473, 0x8473, PDF_CMAP_SINGLE, 9537 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 9536 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 3807 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 9522 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 3804 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 9546 },
	{ 0x8479, 0x8479, PDF_CMAP_SINGLE, 9523 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 9544 },
	{ 0x847d, 0x847d, PDF_CMAP_SINGLE, 9533 },
	{ 0x847e, 0x847e, PDF_CMAP_SINGLE, 9569 },
	{ 0x847f, 0x847f, PDF_CMAP_SINGLE, 14670 },
	{ 0x8480, 0x8480, PDF_CMAP_SINGLE, 15214 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 3803 },
	{ 0x8486, 0x8486, PDF_CMAP_SINGLE, 9529 },
	{ 0x8488, 0x8488, PDF_CMAP_SINGLE, 18274 },
	{ 0x848d, 0x848d, PDF_CMAP_SINGLE, 9525 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 9563 },
	{ 0x848f, 0x848f, PDF_CMAP_SINGLE, 9524 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 4238 },
	{ 0x8491, 0x8491, PDF_CMAP_SINGLE, 10189 },
	{ 0x8492, 0x8492, PDF_CMAP_SINGLE, 14166 },
	{ 0x8493, 0x8493, PDF_CMAP_SINGLE, 14318 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 10172 },
	{ 0x8496, 0x8496, PDF_CMAP_SINGLE, 15133 },
	{ 0x8497, 0x8497, PDF_CMAP_SINGLE, 10150 },
	{ 0x8498, 0x8498, PDF_CMAP_SINGLE, 10180 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 4230 },
	{ 0x849a, 0x849a, PDF_CMAP_SINGLE, 10165 },
	{ 0x849b, 0x849b, PDF_CMAP_SINGLE, 10175 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 4233 },
	{ 0x849d, 0x849d, PDF_CMAP_SINGLE, 10168 },
	{ 0x849e, 0x849e, PDF_CMAP_SINGLE, 4231 },
	{ 0x849f, 0x849f, PDF_CMAP_SINGLE, 10153 },
	{ 0x84a0, 0x84a0, PDF_CMAP_SINGLE, 10183 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 10152 },
	{ 0x84a2, 0x84a2, PDF_CMAP_SINGLE, 10171 },
	{ 0x84a3, 0x84a3, PDF_CMAP_SINGLE, 15262 },
	{ 0x84a4, 0x84a4, PDF_CMAP_SINGLE, 10151 },
	{ 0x84a7, 0x84a7, PDF_CMAP_SINGLE, 10169 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 10178 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 10176 },
	{ 0x84aa, 0x84aa, PDF_CMAP_SINGLE, 10164 },
	{ 0x84ab, 0x84ab, PDF_CMAP_SINGLE, 10159 },
	{ 0x84ac, 0x84ac, PDF_CMAP_SINGLE, 10157 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 15756 },
	{ 0x84ae, 0x84ae, PDF_CMAP_SINGLE, 10158 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 10177 },
	{ 0x84b0, 0x84b0, PDF_CMAP_SINGLE, 10188 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 10166 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 4232 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 10161 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 10181 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 4235 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 10160 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 10154 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 10170 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 4239 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 15436 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 18276 },
	{ 0x84bf, 0x84bf, PDF_CMAP_SINGLE, 4227 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 4236 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 10162 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 10156 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 4229 },
	{ 0x84c5, 0x84c5, PDF_CMAP_SINGLE, 9562 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 4228 },
	{ 0x84c7, 0x84c7, PDF_CMAP_SINGLE, 10173 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 4226 },
	{ 0x84ca, 0x84ca, PDF_CMAP_SINGLE, 4241 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 4234 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 10174 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 10163 },
	{ 0x84ce, 0x84ce, PDF_CMAP_SINGLE, 10155 },
	{ 0x84cf, 0x84cf, PDF_CMAP_SINGLE, 10182 },
	{ 0x84d0, 0x84d0, PDF_CMAP_SINGLE, 10167 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 4240 },
	{ 0x84d2, 0x84d2, PDF_CMAP_SINGLE, 10186 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 4237 },
	{ 0x84d4, 0x84d4, PDF_CMAP_SINGLE, 10185 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 10179 },
	{ 0x84d7, 0x84d7, PDF_CMAP_SINGLE, 10184 },
	{ 0x84da, 0x84da, PDF_CMAP_SINGLE, 16373 },
	{ 0x84db, 0x84db, PDF_CMAP_SINGLE, 10187 },
	{ 0x84de, 0x84de, PDF_CMAP_SINGLE, 14314 },
	{ 0x84e1, 0x84e1, PDF_CMAP_SINGLE, 18277 },
	{ 0x84e2, 0x84e2, PDF_CMAP_SINGLE, 14261 },
	{ 0x84e4, 0x84e4, PDF_CMAP_SINGLE, 14319 },
	{ 0x84e5, 0x84e5, PDF_CMAP_SINGLE, 15312 },
	{ 0x84e7, 0x84e8, PDF_CMAP_RANGE, 10755 },
	{ 0x84e9, 0x84e9, PDF_CMAP_SINGLE, 10743 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 10742 },
	{ 0x84eb, 0x84eb, PDF_CMAP_SINGLE, 10738 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 4603 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4595 },
	{ 0x84ef, 0x84ef, PDF_CMAP_SINGLE, 10758 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 10757 },
	{ 0x84f1, 0x84f1, PDF_CMAP_SINGLE, 9519 },
	{ 0x84f2, 0x84f2, PDF_CMAP_SINGLE, 10735 },
	{ 0x84f3, 0x84f3, PDF_CMAP_SINGLE, 10739 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 10733 },
	{ 0x84f6, 0x84f6, PDF_CMAP_SINGLE, 10752 },
	{ 0x84f7, 0x84f7, PDF_CMAP_SINGLE, 10737 },
	{ 0x84f8, 0x84f8, PDF_CMAP_SINGLE, 18278 },
	{ 0x84f9, 0x84f9, PDF_CMAP_SINGLE, 10759 },
	{ 0x84fa, 0x84fa, PDF_CMAP_SINGLE, 10730 },
	{ 0x84fb, 0x84fb, PDF_CMAP_SINGLE, 10728 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 10740 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 10750 },
	{ 0x84fe, 0x84fe, PDF_CMAP_SINGLE, 10745 },
	{ 0x84ff, 0x84ff, PDF_CMAP_SINGLE, 4605 },
	{ 0x8500, 0x8500, PDF_CMAP_SINGLE, 10719 },
	{ 0x8502, 0x8502, PDF_CMAP_SINGLE, 10749 },
	{ 0x8503, 0x8503, PDF_CMAP_SINGLE, 14730 },
	{ 0x8505, 0x8505, PDF_CMAP_SINGLE, 16619 },
	{ 0x8506, 0x8506, PDF_CMAP_SINGLE, 4606 },
	{ 0x8507, 0x8507, PDF_CMAP_SINGLE, 9539 },
	{ 0x8508, 0x8508, PDF_CMAP_SINGLE, 10731 },
	{ 0x8509, 0x8509, PDF_CMAP_SINGLE, 10722 },
	{ 0x850a, 0x850a, PDF_CMAP_SINGLE, 10725 },
	{ 0x850b, 0x850b, PDF_CMAP_SINGLE, 10763 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 10732 },
	{ 0x850d, 0x850d, PDF_CMAP_SINGLE, 10723 },
	{ 0x850e, 0x850e, PDF_CMAP_SINGLE, 10721 },
	{ 0x850f, 0x850f, PDF_CMAP_SINGLE, 10718 },
	{ 0x8510, 0x8510, PDF_CMAP_SINGLE, 18279 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 4599 },
	{ 0x8512, 0x8512, PDF_CMAP_SINGLE, 10741 },
	{ 0x8513, 0x8513, PDF_CMAP_SINGLE, 4598 },
	{ 0x8514, 0x8514, PDF_CMAP_SINGLE, 4602 },
	{ 0x8515, 0x8515, PDF_CMAP_SINGLE, 10736 },
	{ 0x8516, 0x8516, PDF_CMAP_SINGLE, 10744 },
	{ 0x8517, 0x8517, PDF_CMAP_SINGLE, 4592 },
	{ 0x8518, 0x8518, PDF_CMAP_SINGLE, 10760 },
	{ 0x8519, 0x8519, PDF_CMAP_SINGLE, 10764 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 4594 },
	{ 0x851c, 0x851c, PDF_CMAP_SINGLE, 10727 },
	{ 0x851d, 0x851d, PDF_CMAP_SINGLE, 10747 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 10751 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 10724 },
	{ 0x8520, 0x8520, PDF_CMAP_SINGLE, 10761 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 4601 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 4600 },
	{ 0x8524, 0x8524, PDF_CMAP_SINGLE, 10716 },
	{ 0x8525, 0x8525, PDF_CMAP_SINGLE, 4604 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 10754 },
	{ 0x8527, 0x8527, PDF_CMAP_SINGLE, 10726 },
	{ 0x8528, 0x8528, PDF_CMAP_SINGLE, 10746 },
	{ 0x8529, 0x8529, PDF_CMAP_SINGLE, 10720 },
	{ 0x852a, 0x852a, PDF_CMAP_SINGLE, 10734 },
	{ 0x852b, 0x852b, PDF_CMAP_SINGLE, 10729 },
	{ 0x852c, 0x852d, PDF_CMAP_RANGE, 4596 },
	{ 0x852e, 0x852e, PDF_CMAP_SINGLE, 10748 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 10765 },
	{ 0x8530, 0x8530, PDF_CMAP_SINGLE, 10762 },
	{ 0x8531, 0x8531, PDF_CMAP_SINGLE, 10753 },
	{ 0x8533, 0x8533, PDF_CMAP_SINGLE, 16936 },
	{ 0x8534, 0x8534, PDF_CMAP_SINGLE, 14324 },
	{ 0x8538, 0x8538, PDF_CMAP_SINGLE, 18280 },
	{ 0x853b, 0x853b, PDF_CMAP_SINGLE, 10717 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 4593 },
	{ 0x853e, 0x853e, PDF_CMAP_SINGLE, 11343 },
	{ 0x8540, 0x8540, PDF_CMAP_SINGLE, 11334 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 11337 },
	{ 0x8542, 0x8542, PDF_CMAP_SINGLE, 16482 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 4923 },
	{ 0x8544, 0x8544, PDF_CMAP_SINGLE, 11339 },
	{ 0x8545, 0x8545, PDF_CMAP_SINGLE, 11328 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 11335 },
	{ 0x8547, 0x8547, PDF_CMAP_SINGLE, 11341 },
	{ 0x8548, 0x8548, PDF_CMAP_SINGLE, 4920 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 4924 },
	{ 0x854a, 0x854a, PDF_CMAP_SINGLE, 4918 },
	{ 0x854b, 0x854b, PDF_CMAP_SINGLE, 14272 },
	{ 0x854c, 0x854c, PDF_CMAP_SINGLE, 16399 },
	{ 0x854d, 0x854d, PDF_CMAP_SINGLE, 11330 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 11346 },
	{ 0x8551, 0x8551, PDF_CMAP_SINGLE, 11340 },
	{ 0x8552, 0x8552, PDF_CMAP_SINGLE, 18281 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 11331 },
	{ 0x8554, 0x8554, PDF_CMAP_SINGLE, 11355 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 11349 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 11327 },
	{ 0x8557, 0x8557, PDF_CMAP_SINGLE, 11811 },
	{ 0x8558, 0x8558, PDF_CMAP_SINGLE, 11333 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 4919 },
	{ 0x855a, 0x855a, PDF_CMAP_SINGLE, 14299 },
	{ 0x855b, 0x855b, PDF_CMAP_SINGLE, 11344 },
	{ 0x855d, 0x855d, PDF_CMAP_SINGLE, 11354 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 4927 },
	{ 0x8560, 0x8560, PDF_CMAP_SINGLE, 11351 },
	{ 0x8561, 0x8561, PDF_CMAP_SINGLE, 11332 },
	{ 0x8562, 0x8562, PDF_CMAP_SINGLE, 11338 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 11342 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 11336 },
	{ 0x8565, 0x8565, PDF_CMAP_SINGLE, 11356 },
	{ 0x8566, 0x8566, PDF_CMAP_SINGLE, 11353 },
	{ 0x8567, 0x8567, PDF_CMAP_SINGLE, 11350 },
	{ 0x8568, 0x8569, PDF_CMAP_RANGE, 4921 },
	{ 0x856a, 0x856a, PDF_CMAP_SINGLE, 4926 },
	{ 0x856b, 0x856b, PDF_CMAP_SINGLE, 11329 },
	{ 0x856c, 0x856c, PDF_CMAP_SINGLE, 11357 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 4925 },
	{ 0x856e, 0x856e, PDF_CMAP_SINGLE, 11347 },
	{ 0x856f, 0x8570, PDF_CMAP_RANGE, 18283 },
	{ 0x8571, 0x8571, PDF_CMAP_SINGLE, 11345 },
	{ 0x8573, 0x8573, PDF_CMAP_SINGLE, 14328 },
	{ 0x8575, 0x8575, PDF_CMAP_SINGLE, 11348 },
	{ 0x8576, 0x8576, PDF_CMAP_SINGLE, 11824 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 11805 },
	{ 0x8578, 0x8578, PDF_CMAP_SINGLE, 11810 },
	{ 0x8579, 0x8579, PDF_CMAP_SINGLE, 11823 },
	{ 0x857a, 0x857a, PDF_CMAP_SINGLE, 11809 },
	{ 0x857b, 0x857b, PDF_CMAP_SINGLE, 11801 },
	{ 0x857c, 0x857c, PDF_CMAP_SINGLE, 11806 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 5212 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 11794 },
	{ 0x8581, 0x8581, PDF_CMAP_SINGLE, 11818 },
	{ 0x8582, 0x8582, PDF_CMAP_SINGLE, 11820 },
	{ 0x8583, 0x8583, PDF_CMAP_SINGLE, 11793 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 5211 },
	{ 0x8585, 0x8585, PDF_CMAP_SINGLE, 11822 },
	{ 0x8586, 0x8586, PDF_CMAP_SINGLE, 11814 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 5218 },
	{ 0x8588, 0x8588, PDF_CMAP_SINGLE, 11821 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 11807 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 5220 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 11799 },
	{ 0x858c, 0x858c, PDF_CMAP_SINGLE, 11352 },
	{ 0x858d, 0x858d, PDF_CMAP_SINGLE, 11815 },
	{ 0x858e, 0x858e, PDF_CMAP_SINGLE, 11812 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 11795 },
	{ 0x8590, 0x8590, PDF_CMAP_SINGLE, 11826 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 5214 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 5215 },
	{ 0x8595, 0x8595, PDF_CMAP_SINGLE, 11797 },
	{ 0x8596, 0x8596, PDF_CMAP_SINGLE, 11813 },
	{ 0x8598, 0x8598, PDF_CMAP_SINGLE, 11825 },
	{ 0x8599, 0x8599, PDF_CMAP_SINGLE, 11816 },
	{ 0x859a, 0x859a, PDF_CMAP_SINGLE, 11803 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 5217 },
	{ 0x859c, 0x859c, PDF_CMAP_SINGLE, 5213 },
	{ 0x859d, 0x859d, PDF_CMAP_SINGLE, 11817 },
	{ 0x859e, 0x859e, PDF_CMAP_SINGLE, 11804 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 11827 },
	{ 0x85a0, 0x85a0, PDF_CMAP_SINGLE, 11798 },
	{ 0x85a1, 0x85a1, PDF_CMAP_SINGLE, 11808 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 11819 },
	{ 0x85a3, 0x85a3, PDF_CMAP_SINGLE, 11800 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 11802 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 5221 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 11796 },
	{ 0x85a8, 0x85a8, PDF_CMAP_SINGLE, 5219 },
	{ 0x85a9, 0x85a9, PDF_CMAP_SINGLE, 5404 },
	{ 0x85aa, 0x85aa, PDF_CMAP_SINGLE, 5210 },
	{ 0x85af, 0x85af, PDF_CMAP_SINGLE, 5216 },
	{ 0x85b0, 0x85b0, PDF_CMAP_SINGLE, 5408 },
	{ 0x85b1, 0x85b1, PDF_CMAP_SINGLE, 12198 },
	{ 0x85b3, 0x85b3, PDF_CMAP_SINGLE, 12188 },
	{ 0x85b4, 0x85b4, PDF_CMAP_SINGLE, 12183 },
	{ 0x85b5, 0x85b5, PDF_CMAP_SINGLE, 12189 },
	{ 0x85b6, 0x85b6, PDF_CMAP_SINGLE, 12199 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 12203 },
	{ 0x85b8, 0x85b8, PDF_CMAP_SINGLE, 12202 },
	{ 0x85b9, 0x85b9, PDF_CMAP_SINGLE, 5410 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 5409 },
	{ 0x85bd, 0x85bd, PDF_CMAP_SINGLE, 12190 },
	{ 0x85be, 0x85be, PDF_CMAP_SINGLE, 12204 },
	{ 0x85bf, 0x85bf, PDF_CMAP_SINGLE, 12193 },
	{ 0x85c0, 0x85c0, PDF_CMAP_SINGLE, 12185 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 14334 },
	{ 0x85c2, 0x85c2, PDF_CMAP_SINGLE, 12187 },
	{ 0x85c3, 0x85c3, PDF_CMAP_SINGLE, 12186 },
	{ 0x85c4, 0x85c4, PDF_CMAP_SINGLE, 12192 },
	{ 0x85c5, 0x85c5, PDF_CMAP_SINGLE, 12197 },
	{ 0x85c6, 0x85c6, PDF_CMAP_SINGLE, 12184 },
	{ 0x85c7, 0x85c7, PDF_CMAP_SINGLE, 12191 },
	{ 0x85c8, 0x85c8, PDF_CMAP_SINGLE, 12196 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 5407 },
	{ 0x85cb, 0x85cb, PDF_CMAP_SINGLE, 12194 },
	{ 0x85cd, 0x85cd, PDF_CMAP_SINGLE, 5405 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 12195 },
	{ 0x85cf, 0x85cf, PDF_CMAP_SINGLE, 5403 },
	{ 0x85d0, 0x85d0, PDF_CMAP_SINGLE, 5406 },
	{ 0x85d1, 0x85d1, PDF_CMAP_SINGLE, 12545 },
	{ 0x85d2, 0x85d2, PDF_CMAP_SINGLE, 12200 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 5559 },
	{ 0x85d7, 0x85d7, PDF_CMAP_SINGLE, 12537 },
	{ 0x85d8, 0x85d8, PDF_CMAP_SINGLE, 12541 },
	{ 0x85d9, 0x85d9, PDF_CMAP_SINGLE, 12533 },
	{ 0x85da, 0x85da, PDF_CMAP_SINGLE, 12536 },
	{ 0x85dc, 0x85dc, PDF_CMAP_SINGLE, 12544 },
	{ 0x85dd, 0x85dd, PDF_CMAP_SINGLE, 5557 },
	{ 0x85de, 0x85de, PDF_CMAP_SINGLE, 12549 },
	{ 0x85df, 0x85df, PDF_CMAP_SINGLE, 12542 },
	{ 0x85e0, 0x85e0, PDF_CMAP_SINGLE, 18285 },
	{ 0x85e1, 0x85e1, PDF_CMAP_SINGLE, 12534 },
	{ 0x85e2, 0x85e2, PDF_CMAP_SINGLE, 12550 },
	{ 0x85e3, 0x85e3, PDF_CMAP_SINGLE, 12543 },
	{ 0x85e4, 0x85e5, PDF_CMAP_RANGE, 5560 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 12547 },
	{ 0x85e8, 0x85e8, PDF_CMAP_SINGLE, 12535 },
	{ 0x85e9, 0x85e9, PDF_CMAP_SINGLE, 5556 },
	{ 0x85ea, 0x85ea, PDF_CMAP_SINGLE, 5558 },
	{ 0x85eb, 0x85eb, PDF_CMAP_SINGLE, 12530 },
	{ 0x85ec, 0x85ec, PDF_CMAP_SINGLE, 12538 },
	{ 0x85ed, 0x85ed, PDF_CMAP_SINGLE, 12532 },
	{ 0x85ee, 0x85ee, PDF_CMAP_SINGLE, 15446 },
	{ 0x85ef, 0x85ef, PDF_CMAP_SINGLE, 12548 },
	{ 0x85f0, 0x85f0, PDF_CMAP_SINGLE, 12546 },
	{ 0x85f1, 0x85f1, PDF_CMAP_SINGLE, 12531 },
	{ 0x85f2, 0x85f2, PDF_CMAP_SINGLE, 12539 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 12837 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 5562 },
	{ 0x85f8, 0x85f8, PDF_CMAP_SINGLE, 12540 },
	{ 0x85f9, 0x85f9, PDF_CMAP_SINGLE, 5688 },
	{ 0x85fa, 0x85fa, PDF_CMAP_SINGLE, 5690 },
	{ 0x85fb, 0x85fb, PDF_CMAP_SINGLE, 5687 },
	{ 0x85fc, 0x85fc, PDF_CMAP_SINGLE, 15440 },
	{ 0x85fd, 0x85fd, PDF_CMAP_SINGLE, 12842 },
	{ 0x85fe, 0x85fe, PDF_CMAP_SINGLE, 12834 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 12832 },
	{ 0x8600, 0x8600, PDF_CMAP_SINGLE, 12836 },
	{ 0x8601, 0x8601, PDF_CMAP_SINGLE, 12833 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 14336 },
	{ 0x8604, 0x8604, PDF_CMAP_SINGLE, 12838 },
	{ 0x8605, 0x8605, PDF_CMAP_SINGLE, 12840 },
	{ 0x8606, 0x8606, PDF_CMAP_SINGLE, 5691 },
	{ 0x8607, 0x8607, PDF_CMAP_SINGLE, 5693 },
	{ 0x8609, 0x8609, PDF_CMAP_SINGLE, 12839 },
	{ 0x860a, 0x860a, PDF_CMAP_SINGLE, 5694 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 5692 },
	{ 0x860c, 0x860c, PDF_CMAP_SINGLE, 12841 },
	{ 0x860d, 0x860d, PDF_CMAP_SINGLE, 16484 },
	{ 0x8610, 0x8610, PDF_CMAP_SINGLE, 15231 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 5689 },
	{ 0x8614, 0x8614, PDF_CMAP_SINGLE, 18110 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 14331 },
	{ 0x8617, 0x8617, PDF_CMAP_SINGLE, 5765 },
	{ 0x8618, 0x8618, PDF_CMAP_SINGLE, 13046 },
	{ 0x8619, 0x8619, PDF_CMAP_SINGLE, 13052 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 5767 },
	{ 0x861b, 0x861b, PDF_CMAP_SINGLE, 12835 },
	{ 0x861c, 0x861c, PDF_CMAP_SINGLE, 13051 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 13058 },
	{ 0x861f, 0x861f, PDF_CMAP_SINGLE, 13049 },
	{ 0x8620, 0x8620, PDF_CMAP_SINGLE, 13056 },
	{ 0x8621, 0x8621, PDF_CMAP_SINGLE, 13055 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 12831 },
	{ 0x8623, 0x8623, PDF_CMAP_SINGLE, 13050 },
	{ 0x8624, 0x8624, PDF_CMAP_SINGLE, 12201 },
	{ 0x8625, 0x8625, PDF_CMAP_SINGLE, 13059 },
	{ 0x8626, 0x8626, PDF_CMAP_SINGLE, 13048 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 13053 },
	{ 0x8628, 0x8628, PDF_CMAP_SINGLE, 14341 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 13057 },
	{ 0x862a, 0x862a, PDF_CMAP_SINGLE, 13047 },
	{ 0x862c, 0x862c, PDF_CMAP_SINGLE, 13226 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 5766 },
	{ 0x862e, 0x862e, PDF_CMAP_SINGLE, 13054 },
	{ 0x862f, 0x862f, PDF_CMAP_SINGLE, 16286 },
	{ 0x8631, 0x8631, PDF_CMAP_SINGLE, 13362 },
	{ 0x8632, 0x8632, PDF_CMAP_SINGLE, 13227 },
	{ 0x8633, 0x8633, PDF_CMAP_SINGLE, 13225 },
	{ 0x8634, 0x8635, PDF_CMAP_RANGE, 13223 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 13228 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 5895 },
	{ 0x8639, 0x8639, PDF_CMAP_SINGLE, 13360 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 13358 },
	{ 0x863b, 0x863b, PDF_CMAP_SINGLE, 13363 },
	{ 0x863c, 0x863c, PDF_CMAP_SINGLE, 13361 },
	{ 0x863e, 0x863e, PDF_CMAP_SINGLE, 13364 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 5896 },
	{ 0x8640, 0x8640, PDF_CMAP_SINGLE, 13359 },
	{ 0x8642, 0x8642, PDF_CMAP_SINGLE, 14529 },
	{ 0x8643, 0x8643, PDF_CMAP_SINGLE, 13463 },
	{ 0x8645, 0x8645, PDF_CMAP_SINGLE, 15206 },
	{ 0x8646, 0x8648, PDF_CMAP_RANGE, 13535 },
	{ 0x864b, 0x864b, PDF_CMAP_SINGLE, 13630 },
	{ 0x864c, 0x864c, PDF_CMAP_SINGLE, 13622 },
	{ 0x864d, 0x864d, PDF_CMAP_SINGLE, 6168 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 1673 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 2047 },
	{ 0x8652, 0x8652, PDF_CMAP_SINGLE, 7651 },
	{ 0x8653, 0x8653, PDF_CMAP_SINGLE, 7650 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 2479 },
	{ 0x8655, 0x8655, PDF_CMAP_SINGLE, 2952 },
	{ 0x8656, 0x8656, PDF_CMAP_SINGLE, 8252 },
	{ 0x8659, 0x8659, PDF_CMAP_SINGLE, 8251 },
	{ 0x865b, 0x865b, PDF_CMAP_SINGLE, 3400 },
	{ 0x865c, 0x865c, PDF_CMAP_SINGLE, 3821 },
	{ 0x865e, 0x865e, PDF_CMAP_SINGLE, 3820 },
	{ 0x865f, 0x865f, PDF_CMAP_SINGLE, 3822 },
	{ 0x8661, 0x8661, PDF_CMAP_SINGLE, 10190 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 10766 },
	{ 0x8663, 0x8663, PDF_CMAP_SINGLE, 11358 },
	{ 0x8664, 0x8664, PDF_CMAP_SINGLE, 11360 },
	{ 0x8665, 0x8665, PDF_CMAP_SINGLE, 11359 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 5222 },
	{ 0x8668, 0x8668, PDF_CMAP_SINGLE, 11828 },
	{ 0x8669, 0x8669, PDF_CMAP_SINGLE, 12205 },
	{ 0x866a, 0x866a, PDF_CMAP_SINGLE, 13598 },
	{ 0x866b, 0x866b, PDF_CMAP_SINGLE, 1039 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 14355 },
	{ 0x866d, 0x866e, PDF_CMAP_RANGE, 6726 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 6725 },
	{ 0x8670, 0x8670, PDF_CMAP_SINGLE, 6724 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 1674 },
	{ 0x8672, 0x8672, PDF_CMAP_SINGLE, 18287 },
	{ 0x8673, 0x8673, PDF_CMAP_SINGLE, 7165 },
	{ 0x8674, 0x8674, PDF_CMAP_SINGLE, 7163 },
	{ 0x8677, 0x8677, PDF_CMAP_SINGLE, 7162 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 2048 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 2050 },
	{ 0x867b, 0x867b, PDF_CMAP_SINGLE, 2049 },
	{ 0x867c, 0x867c, PDF_CMAP_SINGLE, 7164 },
	{ 0x867e, 0x867e, PDF_CMAP_SINGLE, 14346 },
	{ 0x8685, 0x8685, PDF_CMAP_SINGLE, 7663 },
	{ 0x8686, 0x8686, PDF_CMAP_SINGLE, 7660 },
	{ 0x8687, 0x8687, PDF_CMAP_SINGLE, 7658 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 2480 },
	{ 0x868b, 0x868b, PDF_CMAP_SINGLE, 7661 },
	{ 0x868c, 0x868c, PDF_CMAP_SINGLE, 2485 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 7655 },
	{ 0x868e, 0x868e, PDF_CMAP_SINGLE, 7670 },
	{ 0x8690, 0x8690, PDF_CMAP_SINGLE, 7672 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 7656 },
	{ 0x8692, 0x8692, PDF_CMAP_SINGLE, 18288 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 2482 },
	{ 0x8694, 0x8694, PDF_CMAP_SINGLE, 7673 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 7668 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 7654 },
	{ 0x8697, 0x8697, PDF_CMAP_SINGLE, 7659 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 7669 },
	{ 0x8699, 0x8699, PDF_CMAP_SINGLE, 7665 },
	{ 0x869a, 0x869a, PDF_CMAP_SINGLE, 7662 },
	{ 0x869c, 0x869c, PDF_CMAP_SINGLE, 2487 },
	{ 0x869d, 0x869d, PDF_CMAP_SINGLE, 7671 },
	{ 0x869e, 0x869e, PDF_CMAP_SINGLE, 7657 },
	{ 0x86a0, 0x86a0, PDF_CMAP_SINGLE, 15240 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 7666 },
	{ 0x86a2, 0x86a2, PDF_CMAP_SINGLE, 7652 },
	{ 0x86a3, 0x86a3, PDF_CMAP_SINGLE, 2486 },
	{ 0x86a4, 0x86a4, PDF_CMAP_SINGLE, 2483 },
	{ 0x86a5, 0x86a5, PDF_CMAP_SINGLE, 7664 },
	{ 0x86a7, 0x86a7, PDF_CMAP_SINGLE, 7667 },
	{ 0x86a8, 0x86a8, PDF_CMAP_SINGLE, 7653 },
	{ 0x86a9, 0x86a9, PDF_CMAP_SINGLE, 2484 },
	{ 0x86aa, 0x86aa, PDF_CMAP_SINGLE, 2481 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 16134 },
	{ 0x86af, 0x86af, PDF_CMAP_SINGLE, 2962 },
	{ 0x86b0, 0x86b0, PDF_CMAP_SINGLE, 8259 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 2961 },
	{ 0x86b2, 0x86b2, PDF_CMAP_SINGLE, 17376 },
	{ 0x86b3, 0x86b3, PDF_CMAP_SINGLE, 8262 },
	{ 0x86b4, 0x86b4, PDF_CMAP_SINGLE, 8265 },
	{ 0x86b5, 0x86b5, PDF_CMAP_SINGLE, 2958 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 2956 },
	{ 0x86b7, 0x86b7, PDF_CMAP_SINGLE, 8254 },
	{ 0x86b8, 0x86b8, PDF_CMAP_SINGLE, 8263 },
	{ 0x86b9, 0x86b9, PDF_CMAP_SINGLE, 8261 },
	{ 0x86ba, 0x86ba, PDF_CMAP_SINGLE, 8258 },
	{ 0x86bb, 0x86bc, PDF_CMAP_RANGE, 8266 },
	{ 0x86bd, 0x86be, PDF_CMAP_RANGE, 8269 },
	{ 0x86bf, 0x86bf, PDF_CMAP_SINGLE, 8253 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 2955 },
	{ 0x86c1, 0x86c1, PDF_CMAP_SINGLE, 8256 },
	{ 0x86c2, 0x86c2, PDF_CMAP_SINGLE, 8255 },
	{ 0x86c3, 0x86c3, PDF_CMAP_SINGLE, 8268 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 2957 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 8257 },
	{ 0x86c6, 0x86c6, PDF_CMAP_SINGLE, 2959 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 2954 },
	{ 0x86c8, 0x86c8, PDF_CMAP_SINGLE, 8260 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 2963 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 2960 },
	{ 0x86cc, 0x86cc, PDF_CMAP_SINGLE, 8264 },
	{ 0x86d0, 0x86d0, PDF_CMAP_SINGLE, 3407 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 8914 },
	{ 0x86d3, 0x86d3, PDF_CMAP_SINGLE, 8903 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 3404 },
	{ 0x86d6, 0x86d6, PDF_CMAP_SINGLE, 9581 },
	{ 0x86d7, 0x86d7, PDF_CMAP_SINGLE, 8912 },
	{ 0x86d8, 0x86d8, PDF_CMAP_SINGLE, 8900 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 3402 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 8905 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 3405 },
	{ 0x86dc, 0x86dc, PDF_CMAP_SINGLE, 8909 },
	{ 0x86dd, 0x86dd, PDF_CMAP_SINGLE, 8907 },
	{ 0x86de, 0x86de, PDF_CMAP_SINGLE, 3408 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 3401 },
	{ 0x86e2, 0x86e2, PDF_CMAP_SINGLE, 8901 },
	{ 0x86e3, 0x86e3, PDF_CMAP_SINGLE, 8904 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 3406 },
	{ 0x86e6, 0x86e6, PDF_CMAP_SINGLE, 8902 },
	{ 0x86e8, 0x86e8, PDF_CMAP_SINGLE, 8913 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 8911 },
	{ 0x86ea, 0x86ea, PDF_CMAP_SINGLE, 8906 },
	{ 0x86eb, 0x86eb, PDF_CMAP_SINGLE, 8908 },
	{ 0x86ec, 0x86ec, PDF_CMAP_SINGLE, 8910 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 3403 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 18289 },
	{ 0x86f5, 0x86f5, PDF_CMAP_SINGLE, 9582 },
	{ 0x86f6, 0x86f6, PDF_CMAP_SINGLE, 9588 },
	{ 0x86f7, 0x86f7, PDF_CMAP_SINGLE, 9578 },
	{ 0x86f8, 0x86f8, PDF_CMAP_SINGLE, 9584 },
	{ 0x86f9, 0x86f9, PDF_CMAP_SINGLE, 3823 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 9580 },
	{ 0x86fb, 0x86fb, PDF_CMAP_SINGLE, 3829 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 3828 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 3827 },
	{ 0x8701, 0x8701, PDF_CMAP_SINGLE, 9587 },
	{ 0x8702, 0x8703, PDF_CMAP_RANGE, 3830 },
	{ 0x8704, 0x8704, PDF_CMAP_SINGLE, 9577 },
	{ 0x8705, 0x8705, PDF_CMAP_SINGLE, 9590 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 3832 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 3826 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 3825 },
	{ 0x8709, 0x8709, PDF_CMAP_SINGLE, 9586 },
	{ 0x870a, 0x870a, PDF_CMAP_SINGLE, 3833 },
	{ 0x870b, 0x870b, PDF_CMAP_SINGLE, 9576 },
	{ 0x870c, 0x870c, PDF_CMAP_SINGLE, 9579 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 9589 },
	{ 0x870e, 0x870e, PDF_CMAP_SINGLE, 9585 },
	{ 0x8711, 0x8711, PDF_CMAP_SINGLE, 10222 },
	{ 0x8712, 0x8712, PDF_CMAP_SINGLE, 10211 },
	{ 0x8713, 0x8713, PDF_CMAP_SINGLE, 3824 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 4248 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 10199 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 10220 },
	{ 0x871b, 0x871b, PDF_CMAP_SINGLE, 10200 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 4243 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 10197 },
	{ 0x8720, 0x8720, PDF_CMAP_SINGLE, 10206 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 10198 },
	{ 0x8722, 0x8722, PDF_CMAP_SINGLE, 4245 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 10192 },
	{ 0x8724, 0x8724, PDF_CMAP_SINGLE, 10219 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 4246 },
	{ 0x8726, 0x8727, PDF_CMAP_RANGE, 10216 },
	{ 0x8728, 0x8728, PDF_CMAP_SINGLE, 10193 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 4251 },
	{ 0x872a, 0x872a, PDF_CMAP_SINGLE, 10208 },
	{ 0x872c, 0x872c, PDF_CMAP_SINGLE, 10202 },
	{ 0x872d, 0x872d, PDF_CMAP_SINGLE, 10209 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 10196 },
	{ 0x8730, 0x8730, PDF_CMAP_SINGLE, 10221 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 10213 },
	{ 0x8732, 0x8732, PDF_CMAP_SINGLE, 10207 },
	{ 0x8733, 0x8733, PDF_CMAP_SINGLE, 10191 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 4247 },
	{ 0x8735, 0x8735, PDF_CMAP_SINGLE, 10214 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 4250 },
	{ 0x8738, 0x8738, PDF_CMAP_SINGLE, 10218 },
	{ 0x873a, 0x873a, PDF_CMAP_SINGLE, 10212 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 4244 },
	{ 0x873c, 0x873c, PDF_CMAP_SINGLE, 10210 },
	{ 0x873e, 0x873e, PDF_CMAP_SINGLE, 10204 },
	{ 0x873f, 0x873f, PDF_CMAP_SINGLE, 4242 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 10195 },
	{ 0x8741, 0x8741, PDF_CMAP_SINGLE, 10203 },
	{ 0x8742, 0x8742, PDF_CMAP_SINGLE, 10215 },
	{ 0x8743, 0x8743, PDF_CMAP_SINGLE, 10201 },
	{ 0x8746, 0x8746, PDF_CMAP_SINGLE, 10205 },
	{ 0x874c, 0x874c, PDF_CMAP_SINGLE, 4616 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 9583 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 10784 },
	{ 0x874f, 0x874f, PDF_CMAP_SINGLE, 10793 },
	{ 0x8750, 0x8750, PDF_CMAP_SINGLE, 10783 },
	{ 0x8751, 0x8751, PDF_CMAP_SINGLE, 10779 },
	{ 0x8752, 0x8752, PDF_CMAP_SINGLE, 10776 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 4617 },
	{ 0x8754, 0x8754, PDF_CMAP_SINGLE, 10774 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 4249 },
	{ 0x8756, 0x8756, PDF_CMAP_SINGLE, 10767 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 4615 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 10773 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 4614 },
	{ 0x875a, 0x875a, PDF_CMAP_SINGLE, 10778 },
	{ 0x875b, 0x875b, PDF_CMAP_SINGLE, 10775 },
	{ 0x875c, 0x875c, PDF_CMAP_SINGLE, 10791 },
	{ 0x875d, 0x875d, PDF_CMAP_SINGLE, 10786 },
	{ 0x875e, 0x875e, PDF_CMAP_SINGLE, 10780 },
	{ 0x875f, 0x875f, PDF_CMAP_SINGLE, 10785 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 4610 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 10777 },
	{ 0x8762, 0x8762, PDF_CMAP_SINGLE, 10796 },
	{ 0x8763, 0x8764, PDF_CMAP_RANGE, 10768 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 10792 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 4611 },
	{ 0x8767, 0x8767, PDF_CMAP_SINGLE, 10797 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 4613 },
	{ 0x8769, 0x8769, PDF_CMAP_SINGLE, 10798 },
	{ 0x876a, 0x876a, PDF_CMAP_SINGLE, 10782 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 10194 },
	{ 0x876c, 0x876c, PDF_CMAP_SINGLE, 10788 },
	{ 0x876d, 0x876d, PDF_CMAP_SINGLE, 10781 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 10790 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 10787 },
	{ 0x8770, 0x8770, PDF_CMAP_SINGLE, 15980 },
	{ 0x8771, 0x8771, PDF_CMAP_SINGLE, 14347 },
	{ 0x8773, 0x8773, PDF_CMAP_SINGLE, 10772 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 4608 },
	{ 0x8775, 0x8775, PDF_CMAP_SINGLE, 10795 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 4609 },
	{ 0x8777, 0x8777, PDF_CMAP_SINGLE, 10770 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 4612 },
	{ 0x8779, 0x8779, PDF_CMAP_SINGLE, 11371 },
	{ 0x877a, 0x877a, PDF_CMAP_SINGLE, 10789 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 10794 },
	{ 0x877d, 0x877d, PDF_CMAP_SINGLE, 16453 },
	{ 0x8781, 0x8781, PDF_CMAP_SINGLE, 11368 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 4607 },
	{ 0x8783, 0x8783, PDF_CMAP_SINGLE, 4928 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 11378 },
	{ 0x8785, 0x8785, PDF_CMAP_SINGLE, 11374 },
	{ 0x8786, 0x8786, PDF_CMAP_SINGLE, 16048 },
	{ 0x8787, 0x8787, PDF_CMAP_SINGLE, 11372 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 11367 },
	{ 0x8789, 0x8789, PDF_CMAP_SINGLE, 11382 },
	{ 0x878b, 0x878b, PDF_CMAP_SINGLE, 17378 },
	{ 0x878c, 0x878c, PDF_CMAP_SINGLE, 16367 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 4932 },
	{ 0x878f, 0x878f, PDF_CMAP_SINGLE, 11363 },
	{ 0x8790, 0x8791, PDF_CMAP_RANGE, 11375 },
	{ 0x8792, 0x8792, PDF_CMAP_SINGLE, 11366 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 11365 },
	{ 0x8794, 0x8794, PDF_CMAP_SINGLE, 11379 },
	{ 0x8796, 0x8796, PDF_CMAP_SINGLE, 11369 },
	{ 0x8797, 0x8797, PDF_CMAP_SINGLE, 11364 },
	{ 0x8798, 0x8798, PDF_CMAP_SINGLE, 11370 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 11381 },
	{ 0x879b, 0x879b, PDF_CMAP_SINGLE, 11362 },
	{ 0x879c, 0x879c, PDF_CMAP_SINGLE, 11380 },
	{ 0x879d, 0x879d, PDF_CMAP_SINGLE, 11377 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 4930 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 4929 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 4931 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 11373 },
	{ 0x87a4, 0x87a4, PDF_CMAP_SINGLE, 11361 },
	{ 0x87a5, 0x87a5, PDF_CMAP_SINGLE, 15752 },
	{ 0x87a9, 0x87a9, PDF_CMAP_SINGLE, 16364 },
	{ 0x87aa, 0x87aa, PDF_CMAP_SINGLE, 11830 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 5228 },
	{ 0x87ac, 0x87ac, PDF_CMAP_SINGLE, 11834 },
	{ 0x87ad, 0x87ad, PDF_CMAP_SINGLE, 11831 },
	{ 0x87ae, 0x87ae, PDF_CMAP_SINGLE, 11838 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 11844 },
	{ 0x87b0, 0x87b0, PDF_CMAP_SINGLE, 11833 },
	{ 0x87b1, 0x87b1, PDF_CMAP_SINGLE, 14351 },
	{ 0x87b2, 0x87b2, PDF_CMAP_SINGLE, 11853 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 5225 },
	{ 0x87b4, 0x87b4, PDF_CMAP_SINGLE, 11847 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 11836 },
	{ 0x87b6, 0x87b6, PDF_CMAP_SINGLE, 11848 },
	{ 0x87b7, 0x87b7, PDF_CMAP_SINGLE, 11843 },
	{ 0x87b8, 0x87b8, PDF_CMAP_SINGLE, 11850 },
	{ 0x87b9, 0x87b9, PDF_CMAP_SINGLE, 11835 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 5230 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 5229 },
	{ 0x87bc, 0x87bc, PDF_CMAP_SINGLE, 11837 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 11851 },
	{ 0x87be, 0x87be, PDF_CMAP_SINGLE, 11829 },
	{ 0x87bf, 0x87bf, PDF_CMAP_SINGLE, 11849 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 5223 },
	{ 0x87c1, 0x87c1, PDF_CMAP_SINGLE, 16380 },
	{ 0x87c2, 0x87c2, PDF_CMAP_SINGLE, 11841 },
	{ 0x87c3, 0x87c3, PDF_CMAP_SINGLE, 11840 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 11845 },
	{ 0x87c5, 0x87c5, PDF_CMAP_SINGLE, 11832 },
	{ 0x87c6, 0x87c6, PDF_CMAP_SINGLE, 5227 },
	{ 0x87c8, 0x87c8, PDF_CMAP_SINGLE, 5231 },
	{ 0x87c9, 0x87c9, PDF_CMAP_SINGLE, 11839 },
	{ 0x87ca, 0x87ca, PDF_CMAP_SINGLE, 11846 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 5232 },
	{ 0x87cc, 0x87cc, PDF_CMAP_SINGLE, 11842 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 16365 },
	{ 0x87d1, 0x87d1, PDF_CMAP_SINGLE, 5224 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 5226 },
	{ 0x87d3, 0x87d3, PDF_CMAP_SINGLE, 12218 },
	{ 0x87d4, 0x87d4, PDF_CMAP_SINGLE, 12216 },
	{ 0x87d6, 0x87d6, PDF_CMAP_SINGLE, 16274 },
	{ 0x87d7, 0x87d7, PDF_CMAP_SINGLE, 12222 },
	{ 0x87d8, 0x87d8, PDF_CMAP_SINGLE, 12220 },
	{ 0x87d9, 0x87d9, PDF_CMAP_SINGLE, 12223 },
	{ 0x87da, 0x87da, PDF_CMAP_SINGLE, 14352 },
	{ 0x87db, 0x87db, PDF_CMAP_SINGLE, 12209 },
	{ 0x87dc, 0x87dc, PDF_CMAP_SINGLE, 12217 },
	{ 0x87dd, 0x87dd, PDF_CMAP_SINGLE, 12227 },
	{ 0x87de, 0x87de, PDF_CMAP_SINGLE, 11852 },
	{ 0x87df, 0x87df, PDF_CMAP_SINGLE, 12213 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 5414 },
	{ 0x87e1, 0x87e1, PDF_CMAP_SINGLE, 10771 },
	{ 0x87e2, 0x87e2, PDF_CMAP_SINGLE, 12208 },
	{ 0x87e3, 0x87e3, PDF_CMAP_SINGLE, 12221 },
	{ 0x87e4, 0x87e4, PDF_CMAP_SINGLE, 12215 },
	{ 0x87e5, 0x87e5, PDF_CMAP_SINGLE, 12212 },
	{ 0x87e6, 0x87e6, PDF_CMAP_SINGLE, 12207 },
	{ 0x87e7, 0x87e7, PDF_CMAP_SINGLE, 12206 },
	{ 0x87e8, 0x87e8, PDF_CMAP_SINGLE, 12226 },
	{ 0x87ea, 0x87ea, PDF_CMAP_SINGLE, 12211 },
	{ 0x87eb, 0x87eb, PDF_CMAP_SINGLE, 12210 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 5412 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 12219 },
	{ 0x87ee, 0x87ee, PDF_CMAP_SINGLE, 14349 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 5411 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 5413 },
	{ 0x87f3, 0x87f3, PDF_CMAP_SINGLE, 12214 },
	{ 0x87f4, 0x87f4, PDF_CMAP_SINGLE, 12225 },
	{ 0x87f5, 0x87f5, PDF_CMAP_SINGLE, 16302 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 12554 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 5566 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 12552 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 5563 },
	{ 0x87fc, 0x87fc, PDF_CMAP_SINGLE, 12560 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 5567 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 12562 },
	{ 0x8800, 0x8800, PDF_CMAP_SINGLE, 12551 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 12224 },
	{ 0x8802, 0x8802, PDF_CMAP_SINGLE, 12564 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 12553 },
	{ 0x8804, 0x8804, PDF_CMAP_SINGLE, 16047 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 5564 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 12559 },
	{ 0x8808, 0x8808, PDF_CMAP_SINGLE, 12561 },
	{ 0x8809, 0x8809, PDF_CMAP_SINGLE, 12556 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 12563 },
	{ 0x880b, 0x880b, PDF_CMAP_SINGLE, 12558 },
	{ 0x880c, 0x880c, PDF_CMAP_SINGLE, 12557 },
	{ 0x880d, 0x880d, PDF_CMAP_SINGLE, 5565 },
	{ 0x880f, 0x880f, PDF_CMAP_SINGLE, 14353 },
	{ 0x8810, 0x8811, PDF_CMAP_RANGE, 12844 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 12847 },
	{ 0x8814, 0x8815, PDF_CMAP_RANGE, 5695 },
	{ 0x8816, 0x8816, PDF_CMAP_SINGLE, 12848 },
	{ 0x8817, 0x8817, PDF_CMAP_SINGLE, 12846 },
	{ 0x8818, 0x8818, PDF_CMAP_SINGLE, 16250 },
	{ 0x8819, 0x8819, PDF_CMAP_SINGLE, 12843 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 13062 },
	{ 0x881c, 0x881c, PDF_CMAP_SINGLE, 13065 },
	{ 0x881d, 0x881d, PDF_CMAP_SINGLE, 13061 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 5771 },
	{ 0x8820, 0x8820, PDF_CMAP_SINGLE, 13063 },
	{ 0x8821, 0x8821, PDF_CMAP_SINGLE, 5770 },
	{ 0x8822, 0x8822, PDF_CMAP_SINGLE, 5769 },
	{ 0x8823, 0x8823, PDF_CMAP_SINGLE, 5768 },
	{ 0x8824, 0x8824, PDF_CMAP_SINGLE, 13064 },
	{ 0x8825, 0x8825, PDF_CMAP_SINGLE, 13233 },
	{ 0x8826, 0x8826, PDF_CMAP_SINGLE, 13231 },
	{ 0x8827, 0x8827, PDF_CMAP_SINGLE, 15879 },
	{ 0x8828, 0x8828, PDF_CMAP_SINGLE, 13230 },
	{ 0x8829, 0x8829, PDF_CMAP_SINGLE, 13060 },
	{ 0x882a, 0x882a, PDF_CMAP_SINGLE, 13232 },
	{ 0x882b, 0x882b, PDF_CMAP_SINGLE, 13066 },
	{ 0x882c, 0x882c, PDF_CMAP_SINGLE, 13229 },
	{ 0x882d, 0x882d, PDF_CMAP_SINGLE, 15243 },
	{ 0x882e, 0x882e, PDF_CMAP_SINGLE, 13367 },
	{ 0x882f, 0x882f, PDF_CMAP_SINGLE, 13333 },
	{ 0x8830, 0x8830, PDF_CMAP_SINGLE, 13365 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 5897 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 13366 },
	{ 0x8833, 0x8833, PDF_CMAP_SINGLE, 13368 },
	{ 0x8835, 0x8835, PDF_CMAP_SINGLE, 13466 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 5928 },
	{ 0x8837, 0x8837, PDF_CMAP_SINGLE, 13465 },
	{ 0x8838, 0x8838, PDF_CMAP_SINGLE, 13464 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 5929 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 5959 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 13576 },
	{ 0x883d, 0x883d, PDF_CMAP_SINGLE, 13600 },
	{ 0x883e, 0x883e, PDF_CMAP_SINGLE, 13599 },
	{ 0x883f, 0x883f, PDF_CMAP_SINGLE, 13601 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 1040 },
	{ 0x8841, 0x8841, PDF_CMAP_SINGLE, 7166 },
	{ 0x8842, 0x8842, PDF_CMAP_SINGLE, 16218 },
	{ 0x8843, 0x8844, PDF_CMAP_RANGE, 7674 },
	{ 0x8845, 0x8846, PDF_CMAP_RANGE, 14358 },
	{ 0x8848, 0x8848, PDF_CMAP_SINGLE, 8915 },
	{ 0x884a, 0x884a, PDF_CMAP_SINGLE, 13067 },
	{ 0x884b, 0x884b, PDF_CMAP_SINGLE, 13467 },
	{ 0x884c, 0x884c, PDF_CMAP_SINGLE, 1041 },
	{ 0x884d, 0x884d, PDF_CMAP_SINGLE, 2051 },
	{ 0x884e, 0x884e, PDF_CMAP_SINGLE, 7167 },
	{ 0x884f, 0x884f, PDF_CMAP_SINGLE, 14404 },
	{ 0x8852, 0x8852, PDF_CMAP_SINGLE, 8271 },
	{ 0x8853, 0x8853, PDF_CMAP_SINGLE, 2964 },
	{ 0x8855, 0x8855, PDF_CMAP_SINGLE, 8917 },
	{ 0x8856, 0x8856, PDF_CMAP_SINGLE, 8916 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 3409 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 3834 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 10799 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 4618 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 4619 },
	{ 0x885e, 0x885e, PDF_CMAP_SINGLE, 14363 },
	{ 0x8860, 0x8860, PDF_CMAP_SINGLE, 14918 },
	{ 0x8861, 0x8861, PDF_CMAP_SINGLE, 4933 },
	{ 0x8862, 0x8862, PDF_CMAP_SINGLE, 5930 },
	{ 0x8863, 0x8863, PDF_CMAP_SINGLE, 1042 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 17666 },
	{ 0x8865, 0x8865, PDF_CMAP_SINGLE, 15868 },
	{ 0x8867, 0x8867, PDF_CMAP_SINGLE, 7168 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 1676 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 7170 },
	{ 0x886a, 0x886a, PDF_CMAP_SINGLE, 7169 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 2052 },
	{ 0x886d, 0x886d, PDF_CMAP_SINGLE, 7676 },
	{ 0x886e, 0x886e, PDF_CMAP_SINGLE, 15246 },
	{ 0x886f, 0x886f, PDF_CMAP_SINGLE, 7683 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 2488 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 7681 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 7679 },
	{ 0x8874, 0x8874, PDF_CMAP_SINGLE, 7686 },
	{ 0x8875, 0x8876, PDF_CMAP_RANGE, 7677 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2489 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 2493 },
	{ 0x887c, 0x887c, PDF_CMAP_SINGLE, 7687 },
	{ 0x887d, 0x887d, PDF_CMAP_SINGLE, 2492 },
	{ 0x887e, 0x887e, PDF_CMAP_SINGLE, 7685 },
	{ 0x887f, 0x887f, PDF_CMAP_SINGLE, 7682 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 7680 },
	{ 0x8881, 0x8882, PDF_CMAP_RANGE, 2490 },
	{ 0x8883, 0x8883, PDF_CMAP_SINGLE, 7684 },
	{ 0x8884, 0x8884, PDF_CMAP_SINGLE, 17143 },
	{ 0x8887, 0x8887, PDF_CMAP_SINGLE, 15248 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 2966 },
	{ 0x8889, 0x8889, PDF_CMAP_SINGLE, 8272 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 2971 },
	{ 0x888c, 0x888c, PDF_CMAP_SINGLE, 8288 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 2970 },
	{ 0x888e, 0x888e, PDF_CMAP_SINGLE, 8290 },
	{ 0x8891, 0x8891, PDF_CMAP_SINGLE, 8278 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 2968 },
	{ 0x8893, 0x8893, PDF_CMAP_SINGLE, 8289 },
	{ 0x8895, 0x8895, PDF_CMAP_SINGLE, 8273 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 2969 },
	{ 0x8897, 0x8897, PDF_CMAP_SINGLE, 8285 },
	{ 0x8898, 0x8898, PDF_CMAP_SINGLE, 8281 },
	{ 0x8899, 0x8899, PDF_CMAP_SINGLE, 8283 },
	{ 0x889a, 0x889a, PDF_CMAP_SINGLE, 8277 },
	{ 0x889b, 0x889b, PDF_CMAP_SINGLE, 8284 },
	{ 0x889c, 0x889c, PDF_CMAP_SINGLE, 14364 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 2965 },
	{ 0x889f, 0x889f, PDF_CMAP_SINGLE, 8280 },
	{ 0x88a0, 0x88a0, PDF_CMAP_SINGLE, 16288 },
	{ 0x88a1, 0x88a1, PDF_CMAP_SINGLE, 8279 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 8275 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 8286 },
	{ 0x88a7, 0x88a7, PDF_CMAP_SINGLE, 8282 },
	{ 0x88a8, 0x88a8, PDF_CMAP_SINGLE, 8274 },
	{ 0x88aa, 0x88aa, PDF_CMAP_SINGLE, 8276 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 2967 },
	{ 0x88ac, 0x88ac, PDF_CMAP_SINGLE, 8287 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 18293 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 3412 },
	{ 0x88b2, 0x88b2, PDF_CMAP_SINGLE, 8928 },
	{ 0x88b4, 0x88b5, PDF_CMAP_RANGE, 14366 },
	{ 0x88b6, 0x88b6, PDF_CMAP_SINGLE, 8924 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 8926 },
	{ 0x88b8, 0x88b8, PDF_CMAP_SINGLE, 8921 },
	{ 0x88b9, 0x88b9, PDF_CMAP_SINGLE, 8920 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 8918 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 8925 },
	{ 0x88bd, 0x88bd, PDF_CMAP_SINGLE, 8927 },
	{ 0x88be, 0x88be, PDF_CMAP_SINGLE, 8923 },
	{ 0x88bf, 0x88bf, PDF_CMAP_SINGLE, 15249 },
	{ 0x88c0, 0x88c0, PDF_CMAP_SINGLE, 8922 },
	{ 0x88c1, 0x88c2, PDF_CMAP_RANGE, 3410 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 14368 },
	{ 0x88c7, 0x88c7, PDF_CMAP_SINGLE, 16049 },
	{ 0x88c9, 0x88c9, PDF_CMAP_SINGLE, 8930 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 3842 },
	{ 0x88cb, 0x88cb, PDF_CMAP_SINGLE, 9592 },
	{ 0x88cc, 0x88cc, PDF_CMAP_SINGLE, 9598 },
	{ 0x88cd, 0x88ce, PDF_CMAP_RANGE, 9593 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 14058 },
	{ 0x88d0, 0x88d0, PDF_CMAP_SINGLE, 9599 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 3844 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 3836 },
	{ 0x88d5, 0x88d5, PDF_CMAP_SINGLE, 3843 },
	{ 0x88d6, 0x88d6, PDF_CMAP_SINGLE, 9591 },
	{ 0x88d7, 0x88d7, PDF_CMAP_SINGLE, 8919 },
	{ 0x88d8, 0x88d8, PDF_CMAP_SINGLE, 3839 },
	{ 0x88d9, 0x88d9, PDF_CMAP_SINGLE, 3837 },
	{ 0x88da, 0x88da, PDF_CMAP_SINGLE, 9597 },
	{ 0x88db, 0x88db, PDF_CMAP_SINGLE, 9596 },
	{ 0x88dc, 0x88dc, PDF_CMAP_SINGLE, 3838 },
	{ 0x88dd, 0x88dd, PDF_CMAP_SINGLE, 3840 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 9595 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 3835 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3841 },
	{ 0x88e6, 0x88e6, PDF_CMAP_SINGLE, 15250 },
	{ 0x88e7, 0x88e7, PDF_CMAP_SINGLE, 10224 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 4258 },
	{ 0x88eb, 0x88eb, PDF_CMAP_SINGLE, 10235 },
	{ 0x88ec, 0x88ec, PDF_CMAP_SINGLE, 10234 },
	{ 0x88ee, 0x88ee, PDF_CMAP_SINGLE, 10229 },
	{ 0x88ef, 0x88ef, PDF_CMAP_SINGLE, 4260 },
	{ 0x88f0, 0x88f0, PDF_CMAP_SINGLE, 10233 },
	{ 0x88f1, 0x88f2, PDF_CMAP_RANGE, 10225 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 4252 },
	{ 0x88f4, 0x88f4, PDF_CMAP_SINGLE, 4254 },
	{ 0x88f5, 0x88f5, PDF_CMAP_SINGLE, 15880 },
	{ 0x88f6, 0x88f6, PDF_CMAP_SINGLE, 10231 },
	{ 0x88f7, 0x88f7, PDF_CMAP_SINGLE, 10223 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 4256 },
	{ 0x88f9, 0x88f9, PDF_CMAP_SINGLE, 4255 },
	{ 0x88fa, 0x88fa, PDF_CMAP_SINGLE, 10227 },
	{ 0x88fb, 0x88fb, PDF_CMAP_SINGLE, 10232 },
	{ 0x88fc, 0x88fc, PDF_CMAP_SINGLE, 10230 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 4257 },
	{ 0x88fe, 0x88fe, PDF_CMAP_SINGLE, 10228 },
	{ 0x88ff, 0x88ff, PDF_CMAP_SINGLE, 18294 },
	{ 0x8900, 0x8900, PDF_CMAP_SINGLE, 17146 },
	{ 0x8901, 0x8901, PDF_CMAP_SINGLE, 8929 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 4253 },
	{ 0x8905, 0x8905, PDF_CMAP_SINGLE, 10800 },
	{ 0x8906, 0x8906, PDF_CMAP_SINGLE, 10807 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 4621 },
	{ 0x8909, 0x8909, PDF_CMAP_SINGLE, 10811 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 4625 },
	{ 0x890b, 0x890b, PDF_CMAP_SINGLE, 10803 },
	{ 0x890c, 0x890c, PDF_CMAP_SINGLE, 10801 },
	{ 0x890e, 0x890e, PDF_CMAP_SINGLE, 10810 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 4620 },
	{ 0x8911, 0x8911, PDF_CMAP_SINGLE, 10809 },
	{ 0x8912, 0x8913, PDF_CMAP_RANGE, 4622 },
	{ 0x8914, 0x8914, PDF_CMAP_SINGLE, 10802 },
	{ 0x8915, 0x8915, PDF_CMAP_SINGLE, 4624 },
	{ 0x8916, 0x8916, PDF_CMAP_SINGLE, 10808 },
	{ 0x8917, 0x8919, PDF_CMAP_RANGE, 10804 },
	{ 0x891a, 0x891a, PDF_CMAP_SINGLE, 4259 },
	{ 0x891e, 0x891e, PDF_CMAP_SINGLE, 11383 },
	{ 0x891f, 0x891f, PDF_CMAP_SINGLE, 11395 },
	{ 0x8921, 0x8921, PDF_CMAP_SINGLE, 4938 },
	{ 0x8922, 0x8922, PDF_CMAP_SINGLE, 11390 },
	{ 0x8923, 0x8923, PDF_CMAP_SINGLE, 11392 },
	{ 0x8924, 0x8924, PDF_CMAP_SINGLE, 18295 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 4936 },
	{ 0x8926, 0x8926, PDF_CMAP_SINGLE, 11384 },
	{ 0x8927, 0x8927, PDF_CMAP_SINGLE, 11388 },
	{ 0x8929, 0x8929, PDF_CMAP_SINGLE, 11391 },
	{ 0x892a, 0x892a, PDF_CMAP_SINGLE, 4934 },
	{ 0x892b, 0x892b, PDF_CMAP_SINGLE, 4937 },
	{ 0x892c, 0x892c, PDF_CMAP_SINGLE, 11394 },
	{ 0x892d, 0x892e, PDF_CMAP_RANGE, 11386 },
	{ 0x892f, 0x892f, PDF_CMAP_SINGLE, 11393 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 11385 },
	{ 0x8931, 0x8931, PDF_CMAP_SINGLE, 11389 },
	{ 0x8932, 0x8932, PDF_CMAP_SINGLE, 4935 },
	{ 0x8933, 0x8933, PDF_CMAP_SINGLE, 11855 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 11854 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 5234 },
	{ 0x8937, 0x8937, PDF_CMAP_SINGLE, 11860 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 5236 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 5233 },
	{ 0x893c, 0x893c, PDF_CMAP_SINGLE, 11856 },
	{ 0x893d, 0x893d, PDF_CMAP_SINGLE, 5237 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 11857 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 11858 },
	{ 0x8942, 0x8942, PDF_CMAP_SINGLE, 11861 },
	{ 0x8943, 0x8943, PDF_CMAP_SINGLE, 15894 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 5235 },
	{ 0x8946, 0x8946, PDF_CMAP_SINGLE, 12232 },
	{ 0x8947, 0x8947, PDF_CMAP_SINGLE, 17379 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 12235 },
	{ 0x894b, 0x894b, PDF_CMAP_SINGLE, 12229 },
	{ 0x894c, 0x894c, PDF_CMAP_SINGLE, 12231 },
	{ 0x894d, 0x894d, PDF_CMAP_SINGLE, 15252 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 12230 },
	{ 0x8950, 0x8951, PDF_CMAP_RANGE, 12233 },
	{ 0x8952, 0x8952, PDF_CMAP_SINGLE, 11859 },
	{ 0x8953, 0x8953, PDF_CMAP_SINGLE, 12228 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 15254 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 5570 },
	{ 0x8957, 0x8957, PDF_CMAP_SINGLE, 12568 },
	{ 0x8958, 0x8958, PDF_CMAP_SINGLE, 12571 },
	{ 0x8959, 0x8959, PDF_CMAP_SINGLE, 12573 },
	{ 0x895a, 0x895b, PDF_CMAP_RANGE, 12566 },
	{ 0x895c, 0x895c, PDF_CMAP_SINGLE, 12570 },
	{ 0x895d, 0x895d, PDF_CMAP_SINGLE, 12572 },
	{ 0x895e, 0x895e, PDF_CMAP_SINGLE, 5571 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 5569 },
	{ 0x8960, 0x8960, PDF_CMAP_SINGLE, 5568 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 12569 },
	{ 0x8962, 0x8962, PDF_CMAP_SINGLE, 12565 },
	{ 0x8963, 0x8963, PDF_CMAP_SINGLE, 12849 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 5697 },
	{ 0x8965, 0x8965, PDF_CMAP_SINGLE, 15251 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 12850 },
	{ 0x8969, 0x8969, PDF_CMAP_SINGLE, 13069 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 5772 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 13071 },
	{ 0x896c, 0x896c, PDF_CMAP_SINGLE, 5773 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 13068 },
	{ 0x896e, 0x896e, PDF_CMAP_SINGLE, 13070 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 5846 },
	{ 0x8971, 0x8971, PDF_CMAP_SINGLE, 13234 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 5845 },
	{ 0x8973, 0x8973, PDF_CMAP_SINGLE, 13371 },
	{ 0x8974, 0x8974, PDF_CMAP_SINGLE, 13370 },
	{ 0x8976, 0x8976, PDF_CMAP_SINGLE, 13369 },
	{ 0x8977, 0x8977, PDF_CMAP_SINGLE, 15850 },
	{ 0x8979, 0x897a, PDF_CMAP_RANGE, 13538 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 13541 },
	{ 0x897c, 0x897c, PDF_CMAP_SINGLE, 13540 },
	{ 0x897e, 0x897e, PDF_CMAP_SINGLE, 6169 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 1043 },
	{ 0x8980, 0x8980, PDF_CMAP_SINGLE, 18184 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 2053 },
	{ 0x8982, 0x8982, PDF_CMAP_SINGLE, 8291 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 3413 },
	{ 0x8985, 0x8985, PDF_CMAP_SINGLE, 9600 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 5415 },
	{ 0x8987, 0x8987, PDF_CMAP_SINGLE, 14370 },
	{ 0x8988, 0x8988, PDF_CMAP_SINGLE, 12574 },
	{ 0x8989, 0x8989, PDF_CMAP_SINGLE, 16361 },
	{ 0x898a, 0x898a, PDF_CMAP_SINGLE, 14371 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 1288 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 2973 },
	{ 0x8991, 0x8991, PDF_CMAP_SINGLE, 18296 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 2972 },
	{ 0x8994, 0x8994, PDF_CMAP_SINGLE, 16352 },
	{ 0x8995, 0x8995, PDF_CMAP_SINGLE, 8931 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 3414 },
	{ 0x8997, 0x8997, PDF_CMAP_SINGLE, 8933 },
	{ 0x8998, 0x8998, PDF_CMAP_SINGLE, 8932 },
	{ 0x899b, 0x899b, PDF_CMAP_SINGLE, 9601 },
	{ 0x899c, 0x899c, PDF_CMAP_SINGLE, 3845 },
	{ 0x899d, 0x899d, PDF_CMAP_SINGLE, 10236 },
	{ 0x899e, 0x899e, PDF_CMAP_SINGLE, 10239 },
	{ 0x899f, 0x899f, PDF_CMAP_SINGLE, 10238 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 10237 },
	{ 0x89a2, 0x89a2, PDF_CMAP_SINGLE, 10812 },
	{ 0x89a3, 0x89a3, PDF_CMAP_SINGLE, 10814 },
	{ 0x89a4, 0x89a4, PDF_CMAP_SINGLE, 10813 },
	{ 0x89a5, 0x89a5, PDF_CMAP_SINGLE, 15763 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 4940 },
	{ 0x89a7, 0x89a7, PDF_CMAP_SINGLE, 14374 },
	{ 0x89a9, 0x89a9, PDF_CMAP_SINGLE, 14373 },
	{ 0x89aa, 0x89aa, PDF_CMAP_SINGLE, 4939 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 5238 },
	{ 0x89ad, 0x89ad, PDF_CMAP_SINGLE, 11862 },
	{ 0x89ae, 0x89ae, PDF_CMAP_SINGLE, 11864 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 11863 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 5416 },
	{ 0x89b6, 0x89b6, PDF_CMAP_SINGLE, 12576 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 12575 },
	{ 0x89b9, 0x89b9, PDF_CMAP_SINGLE, 12851 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 5698 },
	{ 0x89bc, 0x89bc, PDF_CMAP_SINGLE, 14375 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 5774 },
	{ 0x89be, 0x89be, PDF_CMAP_SINGLE, 13236 },
	{ 0x89bf, 0x89bf, PDF_CMAP_SINGLE, 13235 },
	{ 0x89c0, 0x89c0, PDF_CMAP_SINGLE, 5960 },
	{ 0x89c1, 0x89c1, PDF_CMAP_SINGLE, 17667 },
	{ 0x89c6, 0x89c6, PDF_CMAP_SINGLE, 17766 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 1289 },
	{ 0x89d3, 0x89d3, PDF_CMAP_SINGLE, 7171 },
	{ 0x89d4, 0x89d4, PDF_CMAP_SINGLE, 2054 },
	{ 0x89d5, 0x89d5, PDF_CMAP_SINGLE, 8294 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 8292 },
	{ 0x89d9, 0x89d9, PDF_CMAP_SINGLE, 8293 },
	{ 0x89da, 0x89db, PDF_CMAP_RANGE, 8935 },
	{ 0x89dc, 0x89dc, PDF_CMAP_SINGLE, 9608 },
	{ 0x89dd, 0x89dd, PDF_CMAP_SINGLE, 8934 },
	{ 0x89df, 0x89df, PDF_CMAP_SINGLE, 9602 },
	{ 0x89e0, 0x89e0, PDF_CMAP_SINGLE, 9606 },
	{ 0x89e1, 0x89e1, PDF_CMAP_SINGLE, 9605 },
	{ 0x89e2, 0x89e2, PDF_CMAP_SINGLE, 9607 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 3846 },
	{ 0x89e4, 0x89e4, PDF_CMAP_SINGLE, 9604 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 9603 },
	{ 0x89e6, 0x89e6, PDF_CMAP_SINGLE, 9609 },
	{ 0x89e7, 0x89e7, PDF_CMAP_SINGLE, 14377 },
	{ 0x89e8, 0x89e8, PDF_CMAP_SINGLE, 10242 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 10240 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 10241 },
	{ 0x89ec, 0x89ec, PDF_CMAP_SINGLE, 10817 },
	{ 0x89ed, 0x89ed, PDF_CMAP_SINGLE, 10815 },
	{ 0x89f0, 0x89f0, PDF_CMAP_SINGLE, 10816 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 11396 },
	{ 0x89f2, 0x89f3, PDF_CMAP_RANGE, 11865 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 5417 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 12577 },
	{ 0x89f7, 0x89f7, PDF_CMAP_SINGLE, 12852 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 5699 },
	{ 0x89fa, 0x89fa, PDF_CMAP_SINGLE, 13072 },
	{ 0x89fb, 0x89fb, PDF_CMAP_SINGLE, 13237 },
	{ 0x89fc, 0x89fc, PDF_CMAP_SINGLE, 5847 },
	{ 0x89fe, 0x89fe, PDF_CMAP_SINGLE, 13372 },
	{ 0x89ff, 0x89ff, PDF_CMAP_SINGLE, 13542 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 1290 },
	{ 0x8a02, 0x8a03, PDF_CMAP_RANGE, 2056 },
	{ 0x8a04, 0x8a04, PDF_CMAP_SINGLE, 7172 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 7173 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 2055 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2499 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 2496 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 2503 },
	{ 0x8a10, 0x8a10, PDF_CMAP_SINGLE, 2495 },
	{ 0x8a11, 0x8a11, PDF_CMAP_SINGLE, 2504 },
	{ 0x8a12, 0x8a12, PDF_CMAP_SINGLE, 7688 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 2501 },
	{ 0x8a15, 0x8a15, PDF_CMAP_SINGLE, 2498 },
	{ 0x8a16, 0x8a16, PDF_CMAP_SINGLE, 2502 },
	{ 0x8a17, 0x8a17, PDF_CMAP_SINGLE, 2500 },
	{ 0x8a18, 0x8a18, PDF_CMAP_SINGLE, 2494 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 2981 },
	{ 0x8a1c, 0x8a1c, PDF_CMAP_SINGLE, 15267 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 2975 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 8298 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 2980 },
	{ 0x8a22, 0x8a22, PDF_CMAP_SINGLE, 2982 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 2976 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 2977 },
	{ 0x8a27, 0x8a27, PDF_CMAP_SINGLE, 8296 },
	{ 0x8a29, 0x8a29, PDF_CMAP_SINGLE, 17381 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 2974 },
	{ 0x8a2b, 0x8a2b, PDF_CMAP_SINGLE, 15313 },
	{ 0x8a2c, 0x8a2c, PDF_CMAP_SINGLE, 8297 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 2979 },
	{ 0x8a30, 0x8a30, PDF_CMAP_SINGLE, 8295 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 2978 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 3425 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 3427 },
	{ 0x8a38, 0x8a38, PDF_CMAP_SINGLE, 18298 },
	{ 0x8a39, 0x8a39, PDF_CMAP_SINGLE, 8939 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 3426 },
	{ 0x8a3b, 0x8a3b, PDF_CMAP_SINGLE, 3415 },
	{ 0x8a3c, 0x8a3c, PDF_CMAP_SINGLE, 3419 },
	{ 0x8a3d, 0x8a3d, PDF_CMAP_SINGLE, 15266 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 3865 },
	{ 0x8a3f, 0x8a3f, PDF_CMAP_SINGLE, 9614 },
	{ 0x8a40, 0x8a40, PDF_CMAP_SINGLE, 8941 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 3420 },
	{ 0x8a44, 0x8a45, PDF_CMAP_RANGE, 8944 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 3424 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 8947 },
	{ 0x8a49, 0x8a49, PDF_CMAP_SINGLE, 16368 },
	{ 0x8a4a, 0x8a4a, PDF_CMAP_SINGLE, 8949 },
	{ 0x8a4c, 0x8a4c, PDF_CMAP_SINGLE, 8950 },
	{ 0x8a4d, 0x8a4d, PDF_CMAP_SINGLE, 8938 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 8937 },
	{ 0x8a4f, 0x8a4f, PDF_CMAP_SINGLE, 8951 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 3423 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 8948 },
	{ 0x8a52, 0x8a52, PDF_CMAP_SINGLE, 8946 },
	{ 0x8a54, 0x8a54, PDF_CMAP_SINGLE, 3421 },
	{ 0x8a55, 0x8a55, PDF_CMAP_SINGLE, 3417 },
	{ 0x8a56, 0x8a56, PDF_CMAP_SINGLE, 3428 },
	{ 0x8a57, 0x8a58, PDF_CMAP_RANGE, 8942 },
	{ 0x8a59, 0x8a59, PDF_CMAP_SINGLE, 8940 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 3422 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 3418 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 3416 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 9613 },
	{ 0x8a62, 0x8a62, PDF_CMAP_SINGLE, 3860 },
	{ 0x8a63, 0x8a63, PDF_CMAP_SINGLE, 3855 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 3850 },
	{ 0x8a67, 0x8a67, PDF_CMAP_SINGLE, 16270 },
	{ 0x8a68, 0x8a68, PDF_CMAP_SINGLE, 3866 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 3851 },
	{ 0x8a6b, 0x8a6b, PDF_CMAP_SINGLE, 3847 },
	{ 0x8a6c, 0x8a6c, PDF_CMAP_SINGLE, 3862 },
	{ 0x8a6d, 0x8a6d, PDF_CMAP_SINGLE, 3859 },
	{ 0x8a6e, 0x8a6e, PDF_CMAP_SINGLE, 3861 },
	{ 0x8a70, 0x8a70, PDF_CMAP_SINGLE, 3852 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 3857 },
	{ 0x8a72, 0x8a73, PDF_CMAP_RANGE, 3848 },
	{ 0x8a74, 0x8a74, PDF_CMAP_SINGLE, 9621 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 9618 },
	{ 0x8a76, 0x8a76, PDF_CMAP_SINGLE, 9610 },
	{ 0x8a77, 0x8a77, PDF_CMAP_SINGLE, 9615 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 3863 },
	{ 0x8a7a, 0x8a7a, PDF_CMAP_SINGLE, 9622 },
	{ 0x8a7b, 0x8a7b, PDF_CMAP_SINGLE, 3864 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 3854 },
	{ 0x8a7e, 0x8a7e, PDF_CMAP_SINGLE, 16293 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 9612 },
	{ 0x8a81, 0x8a81, PDF_CMAP_SINGLE, 9620 },
	{ 0x8a82, 0x8a82, PDF_CMAP_SINGLE, 9616 },
	{ 0x8a83, 0x8a83, PDF_CMAP_SINGLE, 9619 },
	{ 0x8a84, 0x8a84, PDF_CMAP_SINGLE, 9617 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 3858 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 9611 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 3853 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 10245 },
	{ 0x8a8c, 0x8a8c, PDF_CMAP_SINGLE, 4262 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 4265 },
	{ 0x8a8f, 0x8a8f, PDF_CMAP_SINGLE, 10247 },
	{ 0x8a90, 0x8a90, PDF_CMAP_SINGLE, 14383 },
	{ 0x8a91, 0x8a91, PDF_CMAP_SINGLE, 4273 },
	{ 0x8a92, 0x8a92, PDF_CMAP_SINGLE, 10246 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 4267 },
	{ 0x8a94, 0x8a94, PDF_CMAP_SINGLE, 18299 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 4630 },
	{ 0x8a96, 0x8a96, PDF_CMAP_SINGLE, 10248 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 4272 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 10244 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 4274 },
	{ 0x8a9c, 0x8a9c, PDF_CMAP_SINGLE, 14380 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 4263 },
	{ 0x8aa0, 0x8aa0, PDF_CMAP_SINGLE, 3856 },
	{ 0x8aa1, 0x8aa1, PDF_CMAP_SINGLE, 4266 },
	{ 0x8aa3, 0x8aa3, PDF_CMAP_SINGLE, 4264 },
	{ 0x8aa4, 0x8aa4, PDF_CMAP_SINGLE, 4268 },
	{ 0x8aa5, 0x8aa5, PDF_CMAP_SINGLE, 4270 },
	{ 0x8aa6, 0x8aa6, PDF_CMAP_SINGLE, 4261 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 4275 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 4271 },
	{ 0x8aa9, 0x8aa9, PDF_CMAP_SINGLE, 14397 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 4269 },
	{ 0x8aab, 0x8aab, PDF_CMAP_SINGLE, 10243 },
	{ 0x8aaf, 0x8aaf, PDF_CMAP_SINGLE, 14401 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 4637 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 4633 },
	{ 0x8ab4, 0x8ab4, PDF_CMAP_SINGLE, 18300 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 4640 },
	{ 0x8ab8, 0x8ab8, PDF_CMAP_SINGLE, 10820 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 4641 },
	{ 0x8aba, 0x8aba, PDF_CMAP_SINGLE, 10832 },
	{ 0x8abb, 0x8abb, PDF_CMAP_SINGLE, 10825 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 4626 },
	{ 0x8abd, 0x8abd, PDF_CMAP_SINGLE, 10833 },
	{ 0x8abe, 0x8abe, PDF_CMAP_SINGLE, 10827 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 4636 },
	{ 0x8ac0, 0x8ac0, PDF_CMAP_SINGLE, 10828 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 4635 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 10831 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 4629 },
	{ 0x8ac5, 0x8ac5, PDF_CMAP_SINGLE, 10829 },
	{ 0x8ac6, 0x8ac6, PDF_CMAP_SINGLE, 10819 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 4628 },
	{ 0x8ac8, 0x8ac8, PDF_CMAP_SINGLE, 11407 },
	{ 0x8ac9, 0x8ac9, PDF_CMAP_SINGLE, 4634 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 4631 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 4639 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 10818 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 10822 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 4627 },
	{ 0x8ad3, 0x8ad3, PDF_CMAP_SINGLE, 10821 },
	{ 0x8ad4, 0x8ad5, PDF_CMAP_RANGE, 10823 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4638 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 10826 },
	{ 0x8ad8, 0x8ad8, PDF_CMAP_SINGLE, 10830 },
	{ 0x8ad9, 0x8ad9, PDF_CMAP_SINGLE, 10834 },
	{ 0x8ada, 0x8ada, PDF_CMAP_SINGLE, 14786 },
	{ 0x8adb, 0x8adb, PDF_CMAP_SINGLE, 4642 },
	{ 0x8adc, 0x8adc, PDF_CMAP_SINGLE, 4946 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 11402 },
	{ 0x8ade, 0x8ade, PDF_CMAP_SINGLE, 11408 },
	{ 0x8adf, 0x8adf, PDF_CMAP_SINGLE, 11405 },
	{ 0x8ae0, 0x8ae0, PDF_CMAP_SINGLE, 11397 },
	{ 0x8ae1, 0x8ae1, PDF_CMAP_SINGLE, 11409 },
	{ 0x8ae2, 0x8ae2, PDF_CMAP_SINGLE, 11398 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 11404 },
	{ 0x8ae6, 0x8ae6, PDF_CMAP_SINGLE, 4941 },
	{ 0x8ae7, 0x8ae7, PDF_CMAP_SINGLE, 4947 },
	{ 0x8ae8, 0x8ae8, PDF_CMAP_SINGLE, 11410 },
	{ 0x8aea, 0x8aea, PDF_CMAP_SINGLE, 15833 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 4943 },
	{ 0x8aed, 0x8aed, PDF_CMAP_SINGLE, 4953 },
	{ 0x8aee, 0x8aee, PDF_CMAP_SINGLE, 4948 },
	{ 0x8aef, 0x8aef, PDF_CMAP_SINGLE, 11412 },
	{ 0x8af0, 0x8af0, PDF_CMAP_SINGLE, 11406 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 4944 },
	{ 0x8af2, 0x8af2, PDF_CMAP_SINGLE, 11399 },
	{ 0x8af3, 0x8af3, PDF_CMAP_SINGLE, 4954 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 11400 },
	{ 0x8af6, 0x8af6, PDF_CMAP_SINGLE, 4955 },
	{ 0x8af7, 0x8af7, PDF_CMAP_SINGLE, 4952 },
	{ 0x8af8, 0x8af8, PDF_CMAP_SINGLE, 4632 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 4942 },
	{ 0x8afb, 0x8afb, PDF_CMAP_SINGLE, 11413 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 4956 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 4949 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 11411 },
	{ 0x8b00, 0x8b00, PDF_CMAP_SINGLE, 4945 },
	{ 0x8b01, 0x8b02, PDF_CMAP_RANGE, 4950 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 5246 },
	{ 0x8b05, 0x8b05, PDF_CMAP_SINGLE, 11871 },
	{ 0x8b06, 0x8b06, PDF_CMAP_SINGLE, 11880 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 11877 },
	{ 0x8b08, 0x8b08, PDF_CMAP_SINGLE, 11879 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 5243 },
	{ 0x8b0b, 0x8b0b, PDF_CMAP_SINGLE, 11872 },
	{ 0x8b0c, 0x8b0c, PDF_CMAP_SINGLE, 14390 },
	{ 0x8b0d, 0x8b0d, PDF_CMAP_SINGLE, 11878 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 5239 },
	{ 0x8b0f, 0x8b0f, PDF_CMAP_SINGLE, 11874 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 5247 },
	{ 0x8b11, 0x8b11, PDF_CMAP_SINGLE, 11870 },
	{ 0x8b12, 0x8b12, PDF_CMAP_SINGLE, 11875 },
	{ 0x8b13, 0x8b13, PDF_CMAP_SINGLE, 11882 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 11403 },
	{ 0x8b15, 0x8b15, PDF_CMAP_SINGLE, 11876 },
	{ 0x8b16, 0x8b16, PDF_CMAP_SINGLE, 11869 },
	{ 0x8b17, 0x8b17, PDF_CMAP_SINGLE, 5240 },
	{ 0x8b18, 0x8b18, PDF_CMAP_SINGLE, 11868 },
	{ 0x8b19, 0x8b19, PDF_CMAP_SINGLE, 5241 },
	{ 0x8b1a, 0x8b1a, PDF_CMAP_SINGLE, 11883 },
	{ 0x8b1b, 0x8b1b, PDF_CMAP_SINGLE, 5242 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 11881 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 5245 },
	{ 0x8b1e, 0x8b1e, PDF_CMAP_SINGLE, 11867 },
	{ 0x8b1f, 0x8b1f, PDF_CMAP_SINGLE, 14391 },
	{ 0x8b20, 0x8b20, PDF_CMAP_SINGLE, 5244 },
	{ 0x8b22, 0x8b22, PDF_CMAP_SINGLE, 11873 },
	{ 0x8b23, 0x8b23, PDF_CMAP_SINGLE, 12238 },
	{ 0x8b24, 0x8b24, PDF_CMAP_SINGLE, 12252 },
	{ 0x8b25, 0x8b25, PDF_CMAP_SINGLE, 12247 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 12249 },
	{ 0x8b27, 0x8b27, PDF_CMAP_SINGLE, 12237 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 5418 },
	{ 0x8b2a, 0x8b2a, PDF_CMAP_SINGLE, 12236 },
	{ 0x8b2b, 0x8b2b, PDF_CMAP_SINGLE, 5421 },
	{ 0x8b2c, 0x8b2c, PDF_CMAP_SINGLE, 5420 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 16281 },
	{ 0x8b2e, 0x8b2e, PDF_CMAP_SINGLE, 12251 },
	{ 0x8b2f, 0x8b2f, PDF_CMAP_SINGLE, 12243 },
	{ 0x8b30, 0x8b30, PDF_CMAP_SINGLE, 12240 },
	{ 0x8b31, 0x8b31, PDF_CMAP_SINGLE, 12246 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 12239 },
	{ 0x8b35, 0x8b35, PDF_CMAP_SINGLE, 12241 },
	{ 0x8b36, 0x8b36, PDF_CMAP_SINGLE, 12250 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 12248 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 5419 },
	{ 0x8b3a, 0x8b3a, PDF_CMAP_SINGLE, 12255 },
	{ 0x8b3b, 0x8b3b, PDF_CMAP_SINGLE, 12253 },
	{ 0x8b3c, 0x8b3c, PDF_CMAP_SINGLE, 12244 },
	{ 0x8b3d, 0x8b3d, PDF_CMAP_SINGLE, 12254 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 12245 },
	{ 0x8b3f, 0x8b3f, PDF_CMAP_SINGLE, 14394 },
	{ 0x8b40, 0x8b40, PDF_CMAP_SINGLE, 12581 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 5572 },
	{ 0x8b42, 0x8b42, PDF_CMAP_SINGLE, 12588 },
	{ 0x8b43, 0x8b43, PDF_CMAP_SINGLE, 16370 },
	{ 0x8b45, 0x8b45, PDF_CMAP_SINGLE, 13075 },
	{ 0x8b46, 0x8b46, PDF_CMAP_SINGLE, 5579 },
	{ 0x8b47, 0x8b47, PDF_CMAP_SINGLE, 12242 },
	{ 0x8b48, 0x8b48, PDF_CMAP_SINGLE, 12579 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 5575 },
	{ 0x8b4a, 0x8b4a, PDF_CMAP_SINGLE, 12580 },
	{ 0x8b4b, 0x8b4b, PDF_CMAP_SINGLE, 12585 },
	{ 0x8b4c, 0x8b4d, PDF_CMAP_RANGE, 14395 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_RANGE, 5577 },
	{ 0x8b50, 0x8b50, PDF_CMAP_SINGLE, 12578 },
	{ 0x8b51, 0x8b51, PDF_CMAP_SINGLE, 12587 },
	{ 0x8b52, 0x8b52, PDF_CMAP_SINGLE, 12589 },
	{ 0x8b53, 0x8b53, PDF_CMAP_SINGLE, 12582 },
	{ 0x8b54, 0x8b54, PDF_CMAP_SINGLE, 12584 },
	{ 0x8b55, 0x8b55, PDF_CMAP_SINGLE, 12586 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 12583 },
	{ 0x8b57, 0x8b57, PDF_CMAP_SINGLE, 12590 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 5574 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 5580 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 5576 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 5573 },
	{ 0x8b5d, 0x8b5d, PDF_CMAP_SINGLE, 12855 },
	{ 0x8b5e, 0x8b5e, PDF_CMAP_SINGLE, 15800 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 5704 },
	{ 0x8b60, 0x8b60, PDF_CMAP_SINGLE, 12853 },
	{ 0x8b62, 0x8b62, PDF_CMAP_SINGLE, 15828 },
	{ 0x8b63, 0x8b63, PDF_CMAP_SINGLE, 12857 },
	{ 0x8b65, 0x8b65, PDF_CMAP_SINGLE, 12858 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 5702 },
	{ 0x8b67, 0x8b67, PDF_CMAP_SINGLE, 12859 },
	{ 0x8b68, 0x8b68, PDF_CMAP_SINGLE, 12856 },
	{ 0x8b69, 0x8b69, PDF_CMAP_SINGLE, 15351 },
	{ 0x8b6a, 0x8b6a, PDF_CMAP_SINGLE, 12854 },
	{ 0x8b6b, 0x8b6b, PDF_CMAP_SINGLE, 5705 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 5701 },
	{ 0x8b6d, 0x8b6d, PDF_CMAP_SINGLE, 12860 },
	{ 0x8b6f, 0x8b6f, PDF_CMAP_SINGLE, 5703 },
	{ 0x8b70, 0x8b70, PDF_CMAP_SINGLE, 5700 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 5775 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 5776 },
	{ 0x8b78, 0x8b78, PDF_CMAP_SINGLE, 13074 },
	{ 0x8b79, 0x8b79, PDF_CMAP_SINGLE, 13073 },
	{ 0x8b7a, 0x8b7b, PDF_CMAP_RANGE, 13076 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 5777 },
	{ 0x8b7e, 0x8b7e, PDF_CMAP_SINGLE, 13238 },
	{ 0x8b7f, 0x8b7f, PDF_CMAP_SINGLE, 13243 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 5848 },
	{ 0x8b81, 0x8b81, PDF_CMAP_SINGLE, 15757 },
	{ 0x8b82, 0x8b82, PDF_CMAP_SINGLE, 13240 },
	{ 0x8b84, 0x8b84, PDF_CMAP_SINGLE, 13239 },
	{ 0x8b85, 0x8b85, PDF_CMAP_SINGLE, 13242 },
	{ 0x8b86, 0x8b86, PDF_CMAP_SINGLE, 13241 },
	{ 0x8b88, 0x8b88, PDF_CMAP_SINGLE, 13376 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5898 },
	{ 0x8b8b, 0x8b8b, PDF_CMAP_SINGLE, 13375 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 13373 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 13374 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 14399 },
	{ 0x8b92, 0x8b92, PDF_CMAP_SINGLE, 5932 },
	{ 0x8b93, 0x8b93, PDF_CMAP_SINGLE, 5931 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 13468 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 5933 },
	{ 0x8b98, 0x8b99, PDF_CMAP_RANGE, 13543 },
	{ 0x8b9a, 0x8b9a, PDF_CMAP_SINGLE, 5972 },
	{ 0x8b9b, 0x8b9b, PDF_CMAP_SINGLE, 14400 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 5978 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 13602 },
	{ 0x8b9f, 0x8b9f, PDF_CMAP_SINGLE, 13631 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 17669 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 17767 },
	{ 0x8be2, 0x8be2, PDF_CMAP_SINGLE, 17768 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1291 },
	{ 0x8c39, 0x8c39, PDF_CMAP_SINGLE, 8299 },
	{ 0x8c3b, 0x8c3b, PDF_CMAP_SINGLE, 8300 },
	{ 0x8c3c, 0x8c3c, PDF_CMAP_SINGLE, 9623 },
	{ 0x8c3d, 0x8c3d, PDF_CMAP_SINGLE, 10249 },
	{ 0x8c3e, 0x8c3e, PDF_CMAP_SINGLE, 10835 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 5249 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 5248 },
	{ 0x8c42, 0x8c42, PDF_CMAP_SINGLE, 12256 },
	{ 0x8c43, 0x8c43, PDF_CMAP_SINGLE, 12591 },
	{ 0x8c45, 0x8c45, PDF_CMAP_SINGLE, 13377 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 1292 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 7689 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 2505 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 2983 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 9625 },
	{ 0x8c4b, 0x8c4b, PDF_CMAP_SINGLE, 9624 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 4643 },
	{ 0x8c4d, 0x8c4d, PDF_CMAP_SINGLE, 10836 },
	{ 0x8c4e, 0x8c4e, PDF_CMAP_SINGLE, 4644 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 11884 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 5422 },
	{ 0x8c51, 0x8c51, PDF_CMAP_SINGLE, 18301 },
	{ 0x8c54, 0x8c54, PDF_CMAP_SINGLE, 5987 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 1293 },
	{ 0x8c56, 0x8c56, PDF_CMAP_SINGLE, 6728 },
	{ 0x8c57, 0x8c57, PDF_CMAP_SINGLE, 7690 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 2984 },
	{ 0x8c5c, 0x8c5d, PDF_CMAP_RANGE, 8301 },
	{ 0x8c5f, 0x8c5f, PDF_CMAP_SINGLE, 8952 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 3429 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 3867 },
	{ 0x8c64, 0x8c64, PDF_CMAP_SINGLE, 9627 },
	{ 0x8c65, 0x8c65, PDF_CMAP_SINGLE, 9626 },
	{ 0x8c66, 0x8c66, PDF_CMAP_SINGLE, 9628 },
	{ 0x8c68, 0x8c69, PDF_CMAP_RANGE, 10250 },
	{ 0x8c6a, 0x8c6a, PDF_CMAP_SINGLE, 4276 },
	{ 0x8c6b, 0x8c6b, PDF_CMAP_SINGLE, 4957 },
	{ 0x8c6c, 0x8c6c, PDF_CMAP_SINGLE, 4645 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 4958 },
	{ 0x8c6f, 0x8c6f, PDF_CMAP_SINGLE, 11888 },
	{ 0x8c70, 0x8c70, PDF_CMAP_SINGLE, 11885 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 11887 },
	{ 0x8c72, 0x8c72, PDF_CMAP_SINGLE, 11886 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 5250 },
	{ 0x8c75, 0x8c75, PDF_CMAP_SINGLE, 12257 },
	{ 0x8c76, 0x8c76, PDF_CMAP_SINGLE, 12593 },
	{ 0x8c77, 0x8c77, PDF_CMAP_SINGLE, 12592 },
	{ 0x8c78, 0x8c78, PDF_CMAP_SINGLE, 6375 },
	{ 0x8c79, 0x8c79, PDF_CMAP_SINGLE, 2507 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 2506 },
	{ 0x8c7b, 0x8c7b, PDF_CMAP_SINGLE, 7691 },
	{ 0x8c7d, 0x8c7d, PDF_CMAP_SINGLE, 8303 },
	{ 0x8c80, 0x8c80, PDF_CMAP_SINGLE, 8954 },
	{ 0x8c81, 0x8c81, PDF_CMAP_SINGLE, 8953 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 3430 },
	{ 0x8c84, 0x8c85, PDF_CMAP_RANGE, 9630 },
	{ 0x8c86, 0x8c86, PDF_CMAP_SINGLE, 9629 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 3869 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 3868 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 4278 },
	{ 0x8c8d, 0x8c8d, PDF_CMAP_SINGLE, 4277 },
	{ 0x8c8f, 0x8c8f, PDF_CMAP_SINGLE, 10837 },
	{ 0x8c90, 0x8c90, PDF_CMAP_SINGLE, 11416 },
	{ 0x8c91, 0x8c92, PDF_CMAP_RANGE, 11414 },
	{ 0x8c93, 0x8c93, PDF_CMAP_SINGLE, 4959 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 11890 },
	{ 0x8c95, 0x8c95, PDF_CMAP_SINGLE, 11889 },
	{ 0x8c97, 0x8c97, PDF_CMAP_SINGLE, 12260 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 12259 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 12258 },
	{ 0x8c9a, 0x8c9a, PDF_CMAP_SINGLE, 12594 },
	{ 0x8c9b, 0x8c9b, PDF_CMAP_SINGLE, 14405 },
	{ 0x8c9c, 0x8c9c, PDF_CMAP_SINGLE, 13603 },
	{ 0x8c9d, 0x8c9d, PDF_CMAP_SINGLE, 1294 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 2058 },
	{ 0x8c9f, 0x8c9f, PDF_CMAP_SINGLE, 15277 },
	{ 0x8ca0, 0x8ca0, PDF_CMAP_SINGLE, 2059 },
	{ 0x8ca1, 0x8ca2, PDF_CMAP_RANGE, 2508 },
	{ 0x8ca3, 0x8ca3, PDF_CMAP_SINGLE, 7693 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 7692 },
	{ 0x8ca5, 0x8ca5, PDF_CMAP_SINGLE, 8304 },
	{ 0x8ca7, 0x8ca7, PDF_CMAP_SINGLE, 2990 },
	{ 0x8ca8, 0x8ca8, PDF_CMAP_SINGLE, 2988 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 2985 },
	{ 0x8caa, 0x8caa, PDF_CMAP_SINGLE, 2989 },
	{ 0x8cab, 0x8cab, PDF_CMAP_SINGLE, 2987 },
	{ 0x8cac, 0x8cac, PDF_CMAP_SINGLE, 2986 },
	{ 0x8cad, 0x8cad, PDF_CMAP_SINGLE, 16369 },
	{ 0x8caf, 0x8caf, PDF_CMAP_SINGLE, 3431 },
	{ 0x8cb0, 0x8cb0, PDF_CMAP_SINGLE, 8957 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 3874 },
	{ 0x8cb3, 0x8cb3, PDF_CMAP_SINGLE, 3433 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 3438 },
	{ 0x8cb5, 0x8cb5, PDF_CMAP_SINGLE, 8959 },
	{ 0x8cb6, 0x8cb6, PDF_CMAP_SINGLE, 3440 },
	{ 0x8cb7, 0x8cb7, PDF_CMAP_SINGLE, 3439 },
	{ 0x8cb8, 0x8cb8, PDF_CMAP_SINGLE, 3442 },
	{ 0x8cb9, 0x8cb9, PDF_CMAP_SINGLE, 8958 },
	{ 0x8cba, 0x8cba, PDF_CMAP_SINGLE, 8955 },
	{ 0x8cbb, 0x8cbb, PDF_CMAP_SINGLE, 3436 },
	{ 0x8cbc, 0x8cbc, PDF_CMAP_SINGLE, 3432 },
	{ 0x8cbd, 0x8cbd, PDF_CMAP_SINGLE, 3434 },
	{ 0x8cbe, 0x8cbe, PDF_CMAP_SINGLE, 8956 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 3441 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 3437 },
	{ 0x8cc1, 0x8cc1, PDF_CMAP_SINGLE, 3435 },
	{ 0x8cc2, 0x8cc2, PDF_CMAP_SINGLE, 3876 },
	{ 0x8cc3, 0x8cc3, PDF_CMAP_SINGLE, 3875 },
	{ 0x8cc4, 0x8cc4, PDF_CMAP_SINGLE, 3873 },
	{ 0x8cc5, 0x8cc5, PDF_CMAP_SINGLE, 3877 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_RANGE, 3871 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 3870 },
	{ 0x8ccc, 0x8ccc, PDF_CMAP_SINGLE, 9632 },
	{ 0x8ccd, 0x8ccd, PDF_CMAP_SINGLE, 16025 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 10253 },
	{ 0x8cd1, 0x8cd2, PDF_CMAP_RANGE, 4280 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 4279 },
	{ 0x8cd4, 0x8cd4, PDF_CMAP_SINGLE, 18302 },
	{ 0x8cd5, 0x8cd5, PDF_CMAP_SINGLE, 10252 },
	{ 0x8cd6, 0x8cd6, PDF_CMAP_SINGLE, 14411 },
	{ 0x8cd7, 0x8cd7, PDF_CMAP_SINGLE, 10254 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 10840 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 10842 },
	{ 0x8cdb, 0x8cdb, PDF_CMAP_SINGLE, 14417 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 4654 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 10843 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 4647 },
	{ 0x8cdf, 0x8cdf, PDF_CMAP_SINGLE, 10839 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 4646 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 4656 },
	{ 0x8ce2, 0x8ce3, PDF_CMAP_RANGE, 4652 },
	{ 0x8ce4, 0x8ce4, PDF_CMAP_SINGLE, 4649 },
	{ 0x8ce5, 0x8ce5, PDF_CMAP_SINGLE, 10838 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 4648 },
	{ 0x8ce7, 0x8ce7, PDF_CMAP_SINGLE, 10844 },
	{ 0x8ce8, 0x8ce8, PDF_CMAP_SINGLE, 10841 },
	{ 0x8ce9, 0x8ce9, PDF_CMAP_SINGLE, 15278 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 4655 },
	{ 0x8ceb, 0x8ceb, PDF_CMAP_SINGLE, 16284 },
	{ 0x8cec, 0x8ced, PDF_CMAP_RANGE, 4650 },
	{ 0x8cee, 0x8cee, PDF_CMAP_SINGLE, 11418 },
	{ 0x8cf0, 0x8cf0, PDF_CMAP_SINGLE, 11420 },
	{ 0x8cf1, 0x8cf1, PDF_CMAP_SINGLE, 11419 },
	{ 0x8cf2, 0x8cf2, PDF_CMAP_SINGLE, 18303 },
	{ 0x8cf3, 0x8cf3, PDF_CMAP_SINGLE, 11421 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 4960 },
	{ 0x8cf5, 0x8cf5, PDF_CMAP_SINGLE, 11417 },
	{ 0x8cf7, 0x8cf7, PDF_CMAP_SINGLE, 16305 },
	{ 0x8cf8, 0x8cf8, PDF_CMAP_SINGLE, 5254 },
	{ 0x8cf9, 0x8cf9, PDF_CMAP_SINGLE, 11891 },
	{ 0x8cfa, 0x8cfa, PDF_CMAP_SINGLE, 5251 },
	{ 0x8cfb, 0x8cfb, PDF_CMAP_SINGLE, 5255 },
	{ 0x8cfc, 0x8cfc, PDF_CMAP_SINGLE, 5253 },
	{ 0x8cfd, 0x8cfd, PDF_CMAP_SINGLE, 5252 },
	{ 0x8cfe, 0x8cfe, PDF_CMAP_SINGLE, 12261 },
	{ 0x8d00, 0x8d00, PDF_CMAP_SINGLE, 12264 },
	{ 0x8d02, 0x8d02, PDF_CMAP_SINGLE, 12263 },
	{ 0x8d03, 0x8d03, PDF_CMAP_SINGLE, 14415 },
	{ 0x8d04, 0x8d04, PDF_CMAP_SINGLE, 12262 },
	{ 0x8d05, 0x8d05, PDF_CMAP_SINGLE, 5423 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 12595 },
	{ 0x8d08, 0x8d08, PDF_CMAP_SINGLE, 5581 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 12597 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 5582 },
	{ 0x8d0b, 0x8d0b, PDF_CMAP_SINGLE, 17779 },
	{ 0x8d0c, 0x8d0c, PDF_CMAP_SINGLE, 16278 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 5707 },
	{ 0x8d0f, 0x8d0f, PDF_CMAP_SINGLE, 5706 },
	{ 0x8d10, 0x8d10, PDF_CMAP_SINGLE, 13078 },
	{ 0x8d11, 0x8d11, PDF_CMAP_SINGLE, 14419 },
	{ 0x8d12, 0x8d12, PDF_CMAP_SINGLE, 14414 },
	{ 0x8d13, 0x8d13, PDF_CMAP_SINGLE, 5778 },
	{ 0x8d14, 0x8d14, PDF_CMAP_SINGLE, 13079 },
	{ 0x8d15, 0x8d15, PDF_CMAP_SINGLE, 13244 },
	{ 0x8d16, 0x8d17, PDF_CMAP_RANGE, 5849 },
	{ 0x8d18, 0x8d18, PDF_CMAP_SINGLE, 15983 },
	{ 0x8d19, 0x8d19, PDF_CMAP_SINGLE, 13378 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 5935 },
	{ 0x8d1c, 0x8d1c, PDF_CMAP_SINGLE, 18304 },
	{ 0x8d1d, 0x8d1d, PDF_CMAP_SINGLE, 17670 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 1295 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 2992 },
	{ 0x8d67, 0x8d67, PDF_CMAP_SINGLE, 2991 },
	{ 0x8d68, 0x8d69, PDF_CMAP_RANGE, 9633 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 4282 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 11422 },
	{ 0x8d6d, 0x8d6d, PDF_CMAP_SINGLE, 4657 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 11423 },
	{ 0x8d6f, 0x8d6f, PDF_CMAP_SINGLE, 11892 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 1296 },
	{ 0x8d72, 0x8d72, PDF_CMAP_SINGLE, 7174 },
	{ 0x8d73, 0x8d73, PDF_CMAP_SINGLE, 2061 },
	{ 0x8d74, 0x8d74, PDF_CMAP_SINGLE, 2060 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 7694 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 2510 },
	{ 0x8d78, 0x8d78, PDF_CMAP_SINGLE, 7695 },
	{ 0x8d79, 0x8d79, PDF_CMAP_SINGLE, 8307 },
	{ 0x8d7a, 0x8d7a, PDF_CMAP_SINGLE, 14817 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 8306 },
	{ 0x8d7d, 0x8d7d, PDF_CMAP_SINGLE, 8305 },
	{ 0x8d80, 0x8d80, PDF_CMAP_SINGLE, 8961 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 3445 },
	{ 0x8d82, 0x8d82, PDF_CMAP_SINGLE, 16233 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 8960 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 3444 },
	{ 0x8d89, 0x8d89, PDF_CMAP_SINGLE, 8962 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 3443 },
	{ 0x8d8c, 0x8d8c, PDF_CMAP_SINGLE, 9636 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 9639 },
	{ 0x8d8e, 0x8d8f, PDF_CMAP_RANGE, 9637 },
	{ 0x8d90, 0x8d90, PDF_CMAP_SINGLE, 9642 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 9635 },
	{ 0x8d92, 0x8d92, PDF_CMAP_SINGLE, 9643 },
	{ 0x8d93, 0x8d94, PDF_CMAP_RANGE, 9640 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 4284 },
	{ 0x8d96, 0x8d96, PDF_CMAP_SINGLE, 10255 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 4283 },
	{ 0x8d9b, 0x8d9b, PDF_CMAP_SINGLE, 10848 },
	{ 0x8d9c, 0x8d9c, PDF_CMAP_SINGLE, 10846 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 4658 },
	{ 0x8da0, 0x8da0, PDF_CMAP_SINGLE, 10845 },
	{ 0x8da1, 0x8da1, PDF_CMAP_SINGLE, 10847 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 4659 },
	{ 0x8da5, 0x8da5, PDF_CMAP_SINGLE, 11424 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 16375 },
	{ 0x8da7, 0x8da7, PDF_CMAP_SINGLE, 11425 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 5256 },
	{ 0x8da9, 0x8da9, PDF_CMAP_SINGLE, 14422 },
	{ 0x8daa, 0x8daa, PDF_CMAP_SINGLE, 12599 },
	{ 0x8dab, 0x8dab, PDF_CMAP_SINGLE, 12601 },
	{ 0x8dac, 0x8dac, PDF_CMAP_SINGLE, 12598 },
	{ 0x8dad, 0x8dad, PDF_CMAP_SINGLE, 12600 },
	{ 0x8dae, 0x8dae, PDF_CMAP_SINGLE, 12861 },
	{ 0x8daf, 0x8daf, PDF_CMAP_SINGLE, 13080 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 13577 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 1297 },
	{ 0x8db4, 0x8db4, PDF_CMAP_SINGLE, 2062 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 7696 },
	{ 0x8db6, 0x8db6, PDF_CMAP_SINGLE, 7698 },
	{ 0x8db7, 0x8db7, PDF_CMAP_SINGLE, 7697 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 8310 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 2994 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 8308 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 2993 },
	{ 0x8dbf, 0x8dbf, PDF_CMAP_SINGLE, 8311 },
	{ 0x8dc0, 0x8dc0, PDF_CMAP_SINGLE, 17258 },
	{ 0x8dc1, 0x8dc1, PDF_CMAP_SINGLE, 8312 },
	{ 0x8dc2, 0x8dc2, PDF_CMAP_SINGLE, 8309 },
	{ 0x8dc3, 0x8dc3, PDF_CMAP_SINGLE, 18306 },
	{ 0x8dc5, 0x8dc5, PDF_CMAP_SINGLE, 8974 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 3453 },
	{ 0x8dc7, 0x8dc7, PDF_CMAP_SINGLE, 8966 },
	{ 0x8dc8, 0x8dc8, PDF_CMAP_SINGLE, 8972 },
	{ 0x8dcb, 0x8dcb, PDF_CMAP_SINGLE, 3448 },
	{ 0x8dcc, 0x8dcc, PDF_CMAP_SINGLE, 3451 },
	{ 0x8dcd, 0x8dcd, PDF_CMAP_SINGLE, 8965 },
	{ 0x8dce, 0x8dce, PDF_CMAP_SINGLE, 3446 },
	{ 0x8dcf, 0x8dcf, PDF_CMAP_SINGLE, 8969 },
	{ 0x8dd0, 0x8dd0, PDF_CMAP_SINGLE, 9649 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 3450 },
	{ 0x8dd3, 0x8dd3, PDF_CMAP_SINGLE, 8964 },
	{ 0x8dd4, 0x8dd4, PDF_CMAP_SINGLE, 16258 },
	{ 0x8dd5, 0x8dd5, PDF_CMAP_SINGLE, 8970 },
	{ 0x8dd6, 0x8dd6, PDF_CMAP_SINGLE, 8967 },
	{ 0x8dd7, 0x8dd7, PDF_CMAP_SINGLE, 8973 },
	{ 0x8dd8, 0x8dd8, PDF_CMAP_SINGLE, 8963 },
	{ 0x8dd9, 0x8dd9, PDF_CMAP_SINGLE, 8971 },
	{ 0x8dda, 0x8dda, PDF_CMAP_SINGLE, 3449 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 3452 },
	{ 0x8ddc, 0x8ddc, PDF_CMAP_SINGLE, 8968 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 3447 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 3879 },
	{ 0x8de0, 0x8de0, PDF_CMAP_SINGLE, 9645 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 3878 },
	{ 0x8de2, 0x8de2, PDF_CMAP_SINGLE, 9652 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 9651 },
	{ 0x8de4, 0x8de4, PDF_CMAP_SINGLE, 3885 },
	{ 0x8de6, 0x8de6, PDF_CMAP_SINGLE, 3886 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 9653 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 3880 },
	{ 0x8de9, 0x8de9, PDF_CMAP_SINGLE, 9650 },
	{ 0x8dea, 0x8dea, PDF_CMAP_SINGLE, 3884 },
	{ 0x8deb, 0x8deb, PDF_CMAP_SINGLE, 9655 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 9646 },
	{ 0x8dee, 0x8dee, PDF_CMAP_SINGLE, 9648 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 3881 },
	{ 0x8df0, 0x8df0, PDF_CMAP_SINGLE, 9644 },
	{ 0x8df1, 0x8df1, PDF_CMAP_SINGLE, 9647 },
	{ 0x8df2, 0x8df2, PDF_CMAP_SINGLE, 9654 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3882 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 9656 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 3883 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 4285 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 10260 },
	{ 0x8dfe, 0x8dfe, PDF_CMAP_SINGLE, 10266 },
	{ 0x8dff, 0x8dff, PDF_CMAP_SINGLE, 10258 },
	{ 0x8e00, 0x8e00, PDF_CMAP_SINGLE, 10267 },
	{ 0x8e01, 0x8e01, PDF_CMAP_SINGLE, 16261 },
	{ 0x8e02, 0x8e02, PDF_CMAP_SINGLE, 10257 },
	{ 0x8e03, 0x8e03, PDF_CMAP_SINGLE, 10262 },
	{ 0x8e04, 0x8e04, PDF_CMAP_SINGLE, 10268 },
	{ 0x8e05, 0x8e05, PDF_CMAP_SINGLE, 10265 },
	{ 0x8e06, 0x8e06, PDF_CMAP_SINGLE, 10264 },
	{ 0x8e07, 0x8e07, PDF_CMAP_SINGLE, 10263 },
	{ 0x8e09, 0x8e09, PDF_CMAP_SINGLE, 10256 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 10261 },
	{ 0x8e0d, 0x8e0d, PDF_CMAP_SINGLE, 10259 },
	{ 0x8e0e, 0x8e0e, PDF_CMAP_SINGLE, 15966 },
	{ 0x8e0f, 0x8e0f, PDF_CMAP_SINGLE, 4664 },
	{ 0x8e10, 0x8e10, PDF_CMAP_SINGLE, 4661 },
	{ 0x8e11, 0x8e11, PDF_CMAP_SINGLE, 10857 },
	{ 0x8e12, 0x8e12, PDF_CMAP_SINGLE, 10862 },
	{ 0x8e13, 0x8e13, PDF_CMAP_SINGLE, 10864 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 10861 },
	{ 0x8e15, 0x8e15, PDF_CMAP_SINGLE, 10854 },
	{ 0x8e16, 0x8e16, PDF_CMAP_SINGLE, 10856 },
	{ 0x8e17, 0x8e17, PDF_CMAP_SINGLE, 10866 },
	{ 0x8e18, 0x8e18, PDF_CMAP_SINGLE, 10863 },
	{ 0x8e19, 0x8e19, PDF_CMAP_SINGLE, 10858 },
	{ 0x8e1a, 0x8e1a, PDF_CMAP_SINGLE, 10867 },
	{ 0x8e1b, 0x8e1b, PDF_CMAP_SINGLE, 10855 },
	{ 0x8e1c, 0x8e1c, PDF_CMAP_SINGLE, 10865 },
	{ 0x8e1d, 0x8e1d, PDF_CMAP_SINGLE, 4662 },
	{ 0x8e1e, 0x8e1e, PDF_CMAP_SINGLE, 4668 },
	{ 0x8e1f, 0x8e1f, PDF_CMAP_SINGLE, 4666 },
	{ 0x8e20, 0x8e20, PDF_CMAP_SINGLE, 10849 },
	{ 0x8e21, 0x8e21, PDF_CMAP_SINGLE, 4667 },
	{ 0x8e22, 0x8e22, PDF_CMAP_SINGLE, 4663 },
	{ 0x8e23, 0x8e23, PDF_CMAP_SINGLE, 10850 },
	{ 0x8e24, 0x8e24, PDF_CMAP_SINGLE, 10852 },
	{ 0x8e25, 0x8e25, PDF_CMAP_SINGLE, 10851 },
	{ 0x8e26, 0x8e27, PDF_CMAP_RANGE, 10859 },
	{ 0x8e28, 0x8e28, PDF_CMAP_SINGLE, 16264 },
	{ 0x8e29, 0x8e29, PDF_CMAP_SINGLE, 4665 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 15284 },
	{ 0x8e2b, 0x8e2b, PDF_CMAP_SINGLE, 4660 },
	{ 0x8e2d, 0x8e2d, PDF_CMAP_SINGLE, 16043 },
	{ 0x8e2e, 0x8e2e, PDF_CMAP_SINGLE, 10853 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 11435 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 4962 },
	{ 0x8e33, 0x8e33, PDF_CMAP_SINGLE, 11426 },
	{ 0x8e34, 0x8e34, PDF_CMAP_SINGLE, 4963 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 4966 },
	{ 0x8e36, 0x8e36, PDF_CMAP_SINGLE, 11431 },
	{ 0x8e38, 0x8e38, PDF_CMAP_SINGLE, 11428 },
	{ 0x8e39, 0x8e39, PDF_CMAP_SINGLE, 4965 },
	{ 0x8e3a, 0x8e3a, PDF_CMAP_SINGLE, 18308 },
	{ 0x8e3c, 0x8e3d, PDF_CMAP_RANGE, 11432 },
	{ 0x8e3e, 0x8e3e, PDF_CMAP_SINGLE, 11427 },
	{ 0x8e3f, 0x8e3f, PDF_CMAP_SINGLE, 11436 },
	{ 0x8e40, 0x8e40, PDF_CMAP_SINGLE, 11429 },
	{ 0x8e41, 0x8e41, PDF_CMAP_SINGLE, 11434 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 4964 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 4961 },
	{ 0x8e45, 0x8e45, PDF_CMAP_SINGLE, 11430 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 17170 },
	{ 0x8e47, 0x8e47, PDF_CMAP_SINGLE, 11898 },
	{ 0x8e48, 0x8e48, PDF_CMAP_SINGLE, 5259 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 5257 },
	{ 0x8e4a, 0x8e4a, PDF_CMAP_SINGLE, 5260 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 5258 },
	{ 0x8e4c, 0x8e4c, PDF_CMAP_SINGLE, 11897 },
	{ 0x8e4d, 0x8e4d, PDF_CMAP_SINGLE, 11894 },
	{ 0x8e4e, 0x8e4e, PDF_CMAP_SINGLE, 11893 },
	{ 0x8e4f, 0x8e4f, PDF_CMAP_SINGLE, 16259 },
	{ 0x8e50, 0x8e50, PDF_CMAP_SINGLE, 11896 },
	{ 0x8e53, 0x8e53, PDF_CMAP_SINGLE, 11895 },
	{ 0x8e54, 0x8e54, PDF_CMAP_SINGLE, 12278 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 5429 },
	{ 0x8e56, 0x8e56, PDF_CMAP_SINGLE, 12269 },
	{ 0x8e57, 0x8e57, PDF_CMAP_SINGLE, 12268 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 5424 },
	{ 0x8e5a, 0x8e5a, PDF_CMAP_SINGLE, 12274 },
	{ 0x8e5b, 0x8e5b, PDF_CMAP_SINGLE, 12273 },
	{ 0x8e5c, 0x8e5c, PDF_CMAP_SINGLE, 12265 },
	{ 0x8e5d, 0x8e5d, PDF_CMAP_SINGLE, 12276 },
	{ 0x8e5e, 0x8e5e, PDF_CMAP_SINGLE, 12270 },
	{ 0x8e5f, 0x8e5f, PDF_CMAP_SINGLE, 5428 },
	{ 0x8e60, 0x8e60, PDF_CMAP_SINGLE, 12267 },
	{ 0x8e61, 0x8e61, PDF_CMAP_SINGLE, 12275 },
	{ 0x8e62, 0x8e62, PDF_CMAP_SINGLE, 12266 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 5425 },
	{ 0x8e64, 0x8e64, PDF_CMAP_SINGLE, 5427 },
	{ 0x8e65, 0x8e65, PDF_CMAP_SINGLE, 12271 },
	{ 0x8e66, 0x8e66, PDF_CMAP_SINGLE, 5426 },
	{ 0x8e67, 0x8e67, PDF_CMAP_SINGLE, 12272 },
	{ 0x8e68, 0x8e68, PDF_CMAP_SINGLE, 17229 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 12277 },
	{ 0x8e6a, 0x8e6a, PDF_CMAP_SINGLE, 12605 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 5587 },
	{ 0x8e6d, 0x8e6d, PDF_CMAP_SINGLE, 12602 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 12606 },
	{ 0x8e71, 0x8e71, PDF_CMAP_SINGLE, 17383 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 5584 },
	{ 0x8e73, 0x8e73, PDF_CMAP_SINGLE, 12604 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 5589 },
	{ 0x8e75, 0x8e75, PDF_CMAP_SINGLE, 16265 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 5586 },
	{ 0x8e77, 0x8e77, PDF_CMAP_SINGLE, 16255 },
	{ 0x8e78, 0x8e78, PDF_CMAP_SINGLE, 12603 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 5588 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 12607 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 5583 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 17188 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 15959 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 5709 },
	{ 0x8e82, 0x8e82, PDF_CMAP_SINGLE, 5711 },
	{ 0x8e84, 0x8e84, PDF_CMAP_SINGLE, 12864 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 5710 },
	{ 0x8e86, 0x8e86, PDF_CMAP_SINGLE, 12862 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 5585 },
	{ 0x8e88, 0x8e88, PDF_CMAP_SINGLE, 12863 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 5708 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 5779 },
	{ 0x8e8b, 0x8e8b, PDF_CMAP_SINGLE, 5781 },
	{ 0x8e8c, 0x8e8c, PDF_CMAP_SINGLE, 13082 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 5780 },
	{ 0x8e8e, 0x8e8e, PDF_CMAP_SINGLE, 13081 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 13249 },
	{ 0x8e91, 0x8e91, PDF_CMAP_SINGLE, 5851 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 13248 },
	{ 0x8e93, 0x8e93, PDF_CMAP_SINGLE, 5852 },
	{ 0x8e94, 0x8e94, PDF_CMAP_SINGLE, 13246 },
	{ 0x8e95, 0x8e95, PDF_CMAP_SINGLE, 13245 },
	{ 0x8e96, 0x8e97, PDF_CMAP_RANGE, 13250 },
	{ 0x8e98, 0x8e98, PDF_CMAP_SINGLE, 13379 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 13247 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 13473 },
	{ 0x8e9e, 0x8ea0, PDF_CMAP_RANGE, 13470 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 5961 },
	{ 0x8ea3, 0x8ea3, PDF_CMAP_SINGLE, 13547 },
	{ 0x8ea4, 0x8ea4, PDF_CMAP_SINGLE, 13546 },
	{ 0x8ea5, 0x8ea5, PDF_CMAP_SINGLE, 13545 },
	{ 0x8ea6, 0x8ea6, PDF_CMAP_SINGLE, 13578 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 15285 },
	{ 0x8ea8, 0x8ea8, PDF_CMAP_SINGLE, 13623 },
	{ 0x8ea9, 0x8ea9, PDF_CMAP_SINGLE, 13604 },
	{ 0x8eaa, 0x8eaa, PDF_CMAP_SINGLE, 5979 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 1298 },
	{ 0x8eac, 0x8eac, PDF_CMAP_SINGLE, 2511 },
	{ 0x8ead, 0x8ead, PDF_CMAP_SINGLE, 14430 },
	{ 0x8eb0, 0x8eb0, PDF_CMAP_SINGLE, 17385 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 3887 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14431 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 4669 },
	{ 0x8ebc, 0x8ebc, PDF_CMAP_SINGLE, 14496 },
	{ 0x8ebd, 0x8ebd, PDF_CMAP_SINGLE, 11437 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 5430 },
	{ 0x8ec2, 0x8ec2, PDF_CMAP_SINGLE, 12608 },
	{ 0x8ec3, 0x8ec3, PDF_CMAP_SINGLE, 14432 },
	{ 0x8ec9, 0x8ec9, PDF_CMAP_SINGLE, 13605 },
	{ 0x8eca, 0x8eca, PDF_CMAP_SINGLE, 1299 },
	{ 0x8ecb, 0x8ecb, PDF_CMAP_SINGLE, 1677 },
	{ 0x8ecc, 0x8ecc, PDF_CMAP_SINGLE, 2064 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 2063 },
	{ 0x8ece, 0x8ece, PDF_CMAP_SINGLE, 18312 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 2514 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 7699 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 2512 },
	{ 0x8ed3, 0x8ed3, PDF_CMAP_SINGLE, 7700 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 2513 },
	{ 0x8ed7, 0x8ed7, PDF_CMAP_SINGLE, 8317 },
	{ 0x8ed8, 0x8ed8, PDF_CMAP_SINGLE, 8313 },
	{ 0x8eda, 0x8eda, PDF_CMAP_SINGLE, 15806 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 2995 },
	{ 0x8edc, 0x8edc, PDF_CMAP_SINGLE, 8316 },
	{ 0x8edd, 0x8edd, PDF_CMAP_SINGLE, 8315 },
	{ 0x8ede, 0x8ede, PDF_CMAP_SINGLE, 8314 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 2996 },
	{ 0x8ee0, 0x8ee1, PDF_CMAP_RANGE, 8318 },
	{ 0x8ee2, 0x8ee2, PDF_CMAP_SINGLE, 18313 },
	{ 0x8ee4, 0x8ee4, PDF_CMAP_SINGLE, 18314 },
	{ 0x8ee5, 0x8ee5, PDF_CMAP_SINGLE, 8981 },
	{ 0x8ee6, 0x8ee6, PDF_CMAP_SINGLE, 8979 },
	{ 0x8ee7, 0x8ee8, PDF_CMAP_RANGE, 8983 },
	{ 0x8ee9, 0x8ee9, PDF_CMAP_SINGLE, 8990 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 8986 },
	{ 0x8eec, 0x8eec, PDF_CMAP_SINGLE, 8988 },
	{ 0x8eed, 0x8eed, PDF_CMAP_SINGLE, 18315 },
	{ 0x8eee, 0x8eee, PDF_CMAP_SINGLE, 8980 },
	{ 0x8eef, 0x8eef, PDF_CMAP_SINGLE, 8975 },
	{ 0x8ef1, 0x8ef1, PDF_CMAP_SINGLE, 8987 },
	{ 0x8ef2, 0x8ef2, PDF_CMAP_SINGLE, 18316 },
	{ 0x8ef4, 0x8ef4, PDF_CMAP_SINGLE, 8989 },
	{ 0x8ef5, 0x8ef5, PDF_CMAP_SINGLE, 8982 },
	{ 0x8ef6, 0x8ef6, PDF_CMAP_SINGLE, 8985 },
	{ 0x8ef7, 0x8ef7, PDF_CMAP_SINGLE, 8976 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 3455 },
	{ 0x8ef9, 0x8ef9, PDF_CMAP_SINGLE, 8978 },
	{ 0x8efa, 0x8efa, PDF_CMAP_SINGLE, 8977 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 3454 },
	{ 0x8efc, 0x8efc, PDF_CMAP_SINGLE, 3456 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 3890 },
	{ 0x8eff, 0x8eff, PDF_CMAP_SINGLE, 9658 },
	{ 0x8f00, 0x8f00, PDF_CMAP_SINGLE, 9660 },
	{ 0x8f01, 0x8f01, PDF_CMAP_SINGLE, 9659 },
	{ 0x8f02, 0x8f02, PDF_CMAP_SINGLE, 9664 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 3888 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 9661 },
	{ 0x8f06, 0x8f06, PDF_CMAP_SINGLE, 9657 },
	{ 0x8f07, 0x8f08, PDF_CMAP_RANGE, 9662 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 3889 },
	{ 0x8f0a, 0x8f0a, PDF_CMAP_SINGLE, 3891 },
	{ 0x8f0b, 0x8f0b, PDF_CMAP_SINGLE, 9665 },
	{ 0x8f0d, 0x8f0d, PDF_CMAP_SINGLE, 10272 },
	{ 0x8f0e, 0x8f0e, PDF_CMAP_SINGLE, 10271 },
	{ 0x8f10, 0x8f11, PDF_CMAP_RANGE, 10269 },
	{ 0x8f12, 0x8f12, PDF_CMAP_SINGLE, 4287 },
	{ 0x8f13, 0x8f13, PDF_CMAP_SINGLE, 4289 },
	{ 0x8f14, 0x8f14, PDF_CMAP_SINGLE, 4286 },
	{ 0x8f15, 0x8f15, PDF_CMAP_SINGLE, 4288 },
	{ 0x8f16, 0x8f17, PDF_CMAP_RANGE, 10874 },
	{ 0x8f18, 0x8f18, PDF_CMAP_SINGLE, 10870 },
	{ 0x8f19, 0x8f19, PDF_CMAP_SINGLE, 14434 },
	{ 0x8f1a, 0x8f1a, PDF_CMAP_SINGLE, 10871 },
	{ 0x8f1b, 0x8f1b, PDF_CMAP_SINGLE, 4671 },
	{ 0x8f1c, 0x8f1c, PDF_CMAP_SINGLE, 4676 },
	{ 0x8f1d, 0x8f1d, PDF_CMAP_SINGLE, 4670 },
	{ 0x8f1e, 0x8f1e, PDF_CMAP_SINGLE, 4677 },
	{ 0x8f1f, 0x8f1f, PDF_CMAP_SINGLE, 4672 },
	{ 0x8f20, 0x8f20, PDF_CMAP_SINGLE, 10872 },
	{ 0x8f23, 0x8f23, PDF_CMAP_SINGLE, 10873 },
	{ 0x8f24, 0x8f24, PDF_CMAP_SINGLE, 10869 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 4678 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 4674 },
	{ 0x8f29, 0x8f29, PDF_CMAP_SINGLE, 4673 },
	{ 0x8f2a, 0x8f2a, PDF_CMAP_SINGLE, 4675 },
	{ 0x8f2c, 0x8f2c, PDF_CMAP_SINGLE, 10868 },
	{ 0x8f2d, 0x8f2d, PDF_CMAP_SINGLE, 14435 },
	{ 0x8f2e, 0x8f2e, PDF_CMAP_SINGLE, 11439 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 4968 },
	{ 0x8f30, 0x8f30, PDF_CMAP_SINGLE, 15287 },
	{ 0x8f32, 0x8f32, PDF_CMAP_SINGLE, 11441 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 4970 },
	{ 0x8f34, 0x8f34, PDF_CMAP_SINGLE, 11444 },
	{ 0x8f35, 0x8f35, PDF_CMAP_SINGLE, 11440 },
	{ 0x8f36, 0x8f36, PDF_CMAP_SINGLE, 11438 },
	{ 0x8f37, 0x8f37, PDF_CMAP_SINGLE, 11443 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 4969 },
	{ 0x8f39, 0x8f39, PDF_CMAP_SINGLE, 11442 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 4967 },
	{ 0x8f3e, 0x8f3e, PDF_CMAP_SINGLE, 5262 },
	{ 0x8f3f, 0x8f3f, PDF_CMAP_SINGLE, 5265 },
	{ 0x8f40, 0x8f40, PDF_CMAP_SINGLE, 11900 },
	{ 0x8f41, 0x8f41, PDF_CMAP_SINGLE, 15623 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 5263 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 11899 },
	{ 0x8f44, 0x8f44, PDF_CMAP_SINGLE, 5261 },
	{ 0x8f45, 0x8f45, PDF_CMAP_SINGLE, 5264 },
	{ 0x8f46, 0x8f48, PDF_CMAP_RANGE, 12279 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 5431 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 15288 },
	{ 0x8f4b, 0x8f4b, PDF_CMAP_SINGLE, 12282 },
	{ 0x8f4d, 0x8f4d, PDF_CMAP_SINGLE, 5432 },
	{ 0x8f4e, 0x8f4e, PDF_CMAP_SINGLE, 5591 },
	{ 0x8f4f, 0x8f50, PDF_CMAP_RANGE, 12611 },
	{ 0x8f51, 0x8f51, PDF_CMAP_SINGLE, 12610 },
	{ 0x8f52, 0x8f52, PDF_CMAP_SINGLE, 12609 },
	{ 0x8f53, 0x8f53, PDF_CMAP_SINGLE, 12613 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 5590 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 12868 },
	{ 0x8f56, 0x8f57, PDF_CMAP_RANGE, 12866 },
	{ 0x8f58, 0x8f58, PDF_CMAP_SINGLE, 12869 },
	{ 0x8f59, 0x8f59, PDF_CMAP_SINGLE, 12865 },
	{ 0x8f5a, 0x8f5a, PDF_CMAP_SINGLE, 12870 },
	{ 0x8f5b, 0x8f5b, PDF_CMAP_SINGLE, 13084 },
	{ 0x8f5c, 0x8f5c, PDF_CMAP_SINGLE, 16177 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 13085 },
	{ 0x8f5e, 0x8f5e, PDF_CMAP_SINGLE, 13083 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 5782 },
	{ 0x8f60, 0x8f60, PDF_CMAP_SINGLE, 13252 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 5853 },
	{ 0x8f62, 0x8f62, PDF_CMAP_SINGLE, 13253 },
	{ 0x8f63, 0x8f63, PDF_CMAP_SINGLE, 13381 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 13380 },
	{ 0x8f66, 0x8f67, PDF_CMAP_RANGE, 17769 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 17771 },
	{ 0x8f93, 0x8f93, PDF_CMAP_SINGLE, 18226 },
	{ 0x8f9b, 0x8f9b, PDF_CMAP_SINGLE, 1300 },
	{ 0x8f9c, 0x8f9c, PDF_CMAP_SINGLE, 3457 },
	{ 0x8f9f, 0x8f9f, PDF_CMAP_SINGLE, 3892 },
	{ 0x8fa0, 0x8fa0, PDF_CMAP_SINGLE, 15764 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 4290 },
	{ 0x8fa5, 0x8fa5, PDF_CMAP_SINGLE, 14438 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 4972 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 4971 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 5592 },
	{ 0x8fae, 0x8fae, PDF_CMAP_SINGLE, 5679 },
	{ 0x8faf, 0x8faf, PDF_CMAP_SINGLE, 5783 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 1301 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 2515 },
	{ 0x8fb2, 0x8fb2, PDF_CMAP_SINGLE, 3893 },
	{ 0x8fb3, 0x8fb3, PDF_CMAP_SINGLE, 14442 },
	{ 0x8fb4, 0x8fb4, PDF_CMAP_SINGLE, 12614 },
	{ 0x8fb5, 0x8fb5, PDF_CMAP_SINGLE, 560 },
	{ 0x8fb6, 0x8fb6, PDF_CMAP_SINGLE, 18387 },
	{ 0x8fb7, 0x8fb7, PDF_CMAP_SINGLE, 17405 },
	{ 0x8fb8, 0x8fb8, PDF_CMAP_SINGLE, 17142 },
	{ 0x8fb9, 0x8fb9, PDF_CMAP_SINGLE, 18633 },
	{ 0x8fba, 0x8fba, PDF_CMAP_SINGLE, 18631 },
	{ 0x8fbb, 0x8fbc, PDF_CMAP_RANGE, 15723 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 18630 },
	{ 0x8fbf, 0x8fbf, PDF_CMAP_SINGLE, 6377 },
	{ 0x8fc1, 0x8fc1, PDF_CMAP_SINGLE, 18317 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 1302 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 1305 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 1304 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 1303 },
	{ 0x8fc9, 0x8fc9, PDF_CMAP_SINGLE, 6376 },
	{ 0x8fca, 0x8fca, PDF_CMAP_SINGLE, 18318 },
	{ 0x8fcb, 0x8fcb, PDF_CMAP_SINGLE, 6730 },
	{ 0x8fcc, 0x8fcc, PDF_CMAP_SINGLE, 18319 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 6732 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 1678 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 18606 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 1680 },
	{ 0x8fd2, 0x8fd2, PDF_CMAP_SINGLE, 6729 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 6731 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 1679 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 6734 },
	{ 0x8fd6, 0x8fd6, PDF_CMAP_SINGLE, 6733 },
	{ 0x8fd7, 0x8fd7, PDF_CMAP_SINGLE, 6735 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 16331 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 7178 },
	{ 0x8fe1, 0x8fe1, PDF_CMAP_SINGLE, 7176 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 2067 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 7175 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 2072 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 2069 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 2066 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 2073 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 2068 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 2071 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 2070 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 7177 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 2065 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 2521 },
	{ 0x8ff5, 0x8ff5, PDF_CMAP_SINGLE, 7702 },
	{ 0x8ff6, 0x8ff6, PDF_CMAP_SINGLE, 7708 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 2518 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 2525 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 14449 },
	{ 0x8ffa, 0x8ffa, PDF_CMAP_SINGLE, 2520 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 7705 },
	{ 0x8ffc, 0x8ffc, PDF_CMAP_SINGLE, 7707 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 2523 },
	{ 0x8ffe, 0x8ffe, PDF_CMAP_SINGLE, 7701 },
	{ 0x8fff, 0x8fff, PDF_CMAP_SINGLE, 7704 },
	{ 0x9000, 0x9000, PDF_CMAP_SINGLE, 2519 },
	{ 0x9001, 0x9001, PDF_CMAP_SINGLE, 2516 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 7703 },
	{ 0x9003, 0x9003, PDF_CMAP_SINGLE, 2522 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 7706 },
	{ 0x9005, 0x9005, PDF_CMAP_SINGLE, 2524 },
	{ 0x9006, 0x9006, PDF_CMAP_SINGLE, 2517 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 16096 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 8321 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 8324 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 2998 },
	{ 0x900f, 0x900f, PDF_CMAP_SINGLE, 3008 },
	{ 0x9010, 0x9010, PDF_CMAP_SINGLE, 3004 },
	{ 0x9011, 0x9011, PDF_CMAP_SINGLE, 8322 },
	{ 0x9012, 0x9012, PDF_CMAP_SINGLE, 15950 },
	{ 0x9014, 0x9014, PDF_CMAP_SINGLE, 3012 },
	{ 0x9015, 0x9015, PDF_CMAP_SINGLE, 3005 },
	{ 0x9016, 0x9016, PDF_CMAP_SINGLE, 3010 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 3000 },
	{ 0x9019, 0x9019, PDF_CMAP_SINGLE, 2997 },
	{ 0x901a, 0x901a, PDF_CMAP_SINGLE, 2999 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 3011 },
	{ 0x901c, 0x901c, PDF_CMAP_SINGLE, 8323 },
	{ 0x901d, 0x901d, PDF_CMAP_SINGLE, 3003 },
	{ 0x901e, 0x901e, PDF_CMAP_SINGLE, 3006 },
	{ 0x901f, 0x901f, PDF_CMAP_SINGLE, 3002 },
	{ 0x9020, 0x9020, PDF_CMAP_SINGLE, 3007 },
	{ 0x9021, 0x9021, PDF_CMAP_SINGLE, 8325 },
	{ 0x9022, 0x9022, PDF_CMAP_SINGLE, 3009 },
	{ 0x9023, 0x9023, PDF_CMAP_SINGLE, 3001 },
	{ 0x9024, 0x9024, PDF_CMAP_SINGLE, 8320 },
	{ 0x902d, 0x902d, PDF_CMAP_SINGLE, 8991 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 3458 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 8993 },
	{ 0x9031, 0x9031, PDF_CMAP_SINGLE, 3460 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 3462 },
	{ 0x9033, 0x9033, PDF_CMAP_SINGLE, 18320 },
	{ 0x9034, 0x9034, PDF_CMAP_SINGLE, 8992 },
	{ 0x9035, 0x9035, PDF_CMAP_SINGLE, 3459 },
	{ 0x9036, 0x9036, PDF_CMAP_SINGLE, 3463 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 14454 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 3461 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 3899 },
	{ 0x903d, 0x903d, PDF_CMAP_SINGLE, 9670 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 3907 },
	{ 0x903f, 0x903f, PDF_CMAP_SINGLE, 9667 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 3908 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 3897 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 9668 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 3902 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 9669 },
	{ 0x904a, 0x904a, PDF_CMAP_SINGLE, 3895 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 3894 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 16144 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 3905 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 3904 },
	{ 0x904f, 0x904f, PDF_CMAP_SINGLE, 3903 },
	{ 0x9050, 0x9050, PDF_CMAP_SINGLE, 3901 },
	{ 0x9051, 0x9051, PDF_CMAP_SINGLE, 3906 },
	{ 0x9052, 0x9052, PDF_CMAP_SINGLE, 9666 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 3896 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 3898 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 3900 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 15826 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 4292 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 4295 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 4299 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 4293 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 4298 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 4296 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 4291 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 14457 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 4297 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 4294 },
	{ 0x9064, 0x9064, PDF_CMAP_SINGLE, 15464 },
	{ 0x9067, 0x9067, PDF_CMAP_SINGLE, 10879 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 4681 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 4679 },
	{ 0x906b, 0x906b, PDF_CMAP_SINGLE, 10880 },
	{ 0x906c, 0x906c, PDF_CMAP_SINGLE, 15712 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 4682 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 4680 },
	{ 0x906f, 0x906f, PDF_CMAP_SINGLE, 10878 },
	{ 0x9070, 0x9070, PDF_CMAP_SINGLE, 10877 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 4976 },
	{ 0x9073, 0x9073, PDF_CMAP_SINGLE, 10876 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 4974 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 4973 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 11445 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 4683 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 4975 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 11446 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 4978 },
	{ 0x907b, 0x907b, PDF_CMAP_SINGLE, 11447 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 4977 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 5267 },
	{ 0x907e, 0x907e, PDF_CMAP_SINGLE, 11902 },
	{ 0x907f, 0x907f, PDF_CMAP_SINGLE, 5266 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 5271 },
	{ 0x9081, 0x9082, PDF_CMAP_RANGE, 5269 },
	{ 0x9083, 0x9083, PDF_CMAP_SINGLE, 5434 },
	{ 0x9084, 0x9084, PDF_CMAP_SINGLE, 5268 },
	{ 0x9085, 0x9085, PDF_CMAP_SINGLE, 11901 },
	{ 0x9086, 0x9086, PDF_CMAP_SINGLE, 11448 },
	{ 0x9087, 0x9087, PDF_CMAP_SINGLE, 5433 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 5435 },
	{ 0x908a, 0x908b, PDF_CMAP_RANGE, 5593 },
	{ 0x908d, 0x908d, PDF_CMAP_SINGLE, 12871 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 5900 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 5899 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 1307 },
	{ 0x9094, 0x9094, PDF_CMAP_SINGLE, 6174 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 2526 },
	{ 0x9097, 0x9098, PDF_CMAP_RANGE, 6171 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 6170 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 6173 },
	{ 0x909e, 0x909e, PDF_CMAP_SINGLE, 6381 },
	{ 0x909f, 0x909f, PDF_CMAP_SINGLE, 6378 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 6383 },
	{ 0x90a1, 0x90a1, PDF_CMAP_SINGLE, 6379 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 1308 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 1311 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 6380 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 1310 },
	{ 0x90a7, 0x90a7, PDF_CMAP_SINGLE, 6382 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 14460 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 1309 },
	{ 0x90ae, 0x90ae, PDF_CMAP_SINGLE, 14464 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 6738 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 6740 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 1683 },
	{ 0x90b2, 0x90b2, PDF_CMAP_SINGLE, 6736 },
	{ 0x90b3, 0x90b3, PDF_CMAP_SINGLE, 6739 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 6737 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 1681 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 1684 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 1682 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 15300 },
	{ 0x90bd, 0x90bd, PDF_CMAP_SINGLE, 7180 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 7184 },
	{ 0x90bf, 0x90bf, PDF_CMAP_SINGLE, 7181 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 2076 },
	{ 0x90c3, 0x90c3, PDF_CMAP_SINGLE, 2077 },
	{ 0x90c4, 0x90c4, PDF_CMAP_SINGLE, 14462 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 7183 },
	{ 0x90c7, 0x90c7, PDF_CMAP_SINGLE, 7185 },
	{ 0x90c8, 0x90c8, PDF_CMAP_SINGLE, 7187 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2074 },
	{ 0x90cb, 0x90cb, PDF_CMAP_SINGLE, 7186 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 2075 },
	{ 0x90d4, 0x90d4, PDF_CMAP_SINGLE, 8332 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 7182 },
	{ 0x90d6, 0x90d6, PDF_CMAP_SINGLE, 7709 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 7718 },
	{ 0x90d8, 0x90d8, PDF_CMAP_SINGLE, 7716 },
	{ 0x90d9, 0x90da, PDF_CMAP_RANGE, 7711 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 7717 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 7719 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 2528 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 7714 },
	{ 0x90e0, 0x90e0, PDF_CMAP_SINGLE, 7710 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 2527 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 2529 },
	{ 0x90e3, 0x90e3, PDF_CMAP_SINGLE, 7713 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 7720 },
	{ 0x90e5, 0x90e5, PDF_CMAP_SINGLE, 7715 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 3013 },
	{ 0x90e9, 0x90e9, PDF_CMAP_SINGLE, 8335 },
	{ 0x90ea, 0x90ea, PDF_CMAP_SINGLE, 8327 },
	{ 0x90eb, 0x90ec, PDF_CMAP_RANGE, 8333 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 3014 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 8326 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 8328 },
	{ 0x90f1, 0x90f1, PDF_CMAP_SINGLE, 7179 },
	{ 0x90f2, 0x90f3, PDF_CMAP_RANGE, 8330 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 8329 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 3465 },
	{ 0x90f9, 0x90f9, PDF_CMAP_SINGLE, 9000 },
	{ 0x90fa, 0x90fa, PDF_CMAP_SINGLE, 11449 },
	{ 0x90fb, 0x90fb, PDF_CMAP_SINGLE, 9001 },
	{ 0x90fc, 0x90fc, PDF_CMAP_SINGLE, 8998 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 3015 },
	{ 0x90fe, 0x90fe, PDF_CMAP_SINGLE, 3467 },
	{ 0x90ff, 0x90ff, PDF_CMAP_SINGLE, 8997 },
	{ 0x9100, 0x9100, PDF_CMAP_SINGLE, 9003 },
	{ 0x9101, 0x9101, PDF_CMAP_SINGLE, 9002 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 3464 },
	{ 0x9103, 0x9103, PDF_CMAP_SINGLE, 9006 },
	{ 0x9104, 0x9104, PDF_CMAP_SINGLE, 8996 },
	{ 0x9105, 0x9105, PDF_CMAP_SINGLE, 9005 },
	{ 0x9106, 0x9106, PDF_CMAP_SINGLE, 8994 },
	{ 0x9107, 0x9107, PDF_CMAP_SINGLE, 9004 },
	{ 0x9108, 0x9108, PDF_CMAP_SINGLE, 8999 },
	{ 0x9109, 0x9109, PDF_CMAP_SINGLE, 3466 },
	{ 0x910b, 0x910b, PDF_CMAP_SINGLE, 9677 },
	{ 0x910d, 0x910d, PDF_CMAP_SINGLE, 9672 },
	{ 0x910e, 0x910e, PDF_CMAP_SINGLE, 9678 },
	{ 0x910f, 0x910f, PDF_CMAP_SINGLE, 9673 },
	{ 0x9110, 0x9110, PDF_CMAP_SINGLE, 9671 },
	{ 0x9111, 0x9111, PDF_CMAP_SINGLE, 9674 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 3909 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 9676 },
	{ 0x9116, 0x9116, PDF_CMAP_SINGLE, 9675 },
	{ 0x9117, 0x9117, PDF_CMAP_SINGLE, 3910 },
	{ 0x9118, 0x9118, PDF_CMAP_SINGLE, 4301 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 4300 },
	{ 0x911a, 0x911a, PDF_CMAP_SINGLE, 10279 },
	{ 0x911b, 0x911b, PDF_CMAP_SINGLE, 10282 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 10274 },
	{ 0x911d, 0x911d, PDF_CMAP_SINGLE, 10278 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 4302 },
	{ 0x911f, 0x911f, PDF_CMAP_SINGLE, 10277 },
	{ 0x9120, 0x9120, PDF_CMAP_SINGLE, 10275 },
	{ 0x9121, 0x9121, PDF_CMAP_SINGLE, 10281 },
	{ 0x9122, 0x9122, PDF_CMAP_SINGLE, 10276 },
	{ 0x9123, 0x9123, PDF_CMAP_SINGLE, 10273 },
	{ 0x9124, 0x9124, PDF_CMAP_SINGLE, 10280 },
	{ 0x9126, 0x9126, PDF_CMAP_SINGLE, 10886 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 4686 },
	{ 0x9128, 0x9128, PDF_CMAP_SINGLE, 12283 },
	{ 0x9129, 0x912a, PDF_CMAP_RANGE, 10883 },
	{ 0x912b, 0x912b, PDF_CMAP_SINGLE, 10882 },
	{ 0x912c, 0x912c, PDF_CMAP_SINGLE, 8995 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 4685 },
	{ 0x912e, 0x912e, PDF_CMAP_SINGLE, 10887 },
	{ 0x912f, 0x912f, PDF_CMAP_SINGLE, 10881 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 4684 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 4687 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 10885 },
	{ 0x9133, 0x9133, PDF_CMAP_SINGLE, 11450 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 4979 },
	{ 0x9135, 0x9136, PDF_CMAP_RANGE, 11451 },
	{ 0x9138, 0x9138, PDF_CMAP_SINGLE, 11903 },
	{ 0x9139, 0x9139, PDF_CMAP_SINGLE, 5272 },
	{ 0x913a, 0x913b, PDF_CMAP_RANGE, 12284 },
	{ 0x913e, 0x913e, PDF_CMAP_SINGLE, 12286 },
	{ 0x913f, 0x913f, PDF_CMAP_SINGLE, 12616 },
	{ 0x9140, 0x9140, PDF_CMAP_SINGLE, 12615 },
	{ 0x9141, 0x9141, PDF_CMAP_SINGLE, 12873 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 12872 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 13087 },
	{ 0x9146, 0x9146, PDF_CMAP_SINGLE, 13086 },
	{ 0x9147, 0x9147, PDF_CMAP_SINGLE, 13254 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 5854 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 1312 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 2079 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 2078 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 2532 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 2531 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7722 },
	{ 0x9150, 0x9150, PDF_CMAP_SINGLE, 7721 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 16201 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2530 },
	{ 0x9153, 0x9153, PDF_CMAP_SINGLE, 8339 },
	{ 0x9155, 0x9155, PDF_CMAP_SINGLE, 8340 },
	{ 0x9156, 0x9156, PDF_CMAP_SINGLE, 8336 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 3016 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 8337 },
	{ 0x9159, 0x9159, PDF_CMAP_SINGLE, 16197 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 8338 },
	{ 0x915c, 0x915c, PDF_CMAP_SINGLE, 16199 },
	{ 0x915e, 0x915e, PDF_CMAP_SINGLE, 17387 },
	{ 0x915f, 0x915f, PDF_CMAP_SINGLE, 9009 },
	{ 0x9160, 0x9160, PDF_CMAP_SINGLE, 9011 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 9007 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 9010 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 3468 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 9008 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 3469 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 14466 },
	{ 0x9168, 0x9168, PDF_CMAP_SINGLE, 9186 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 3913 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 3912 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 3911 },
	{ 0x916e, 0x916f, PDF_CMAP_RANGE, 9679 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 16056 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 10284 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 10286 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 4306 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 4303 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 16027 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 4305 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 4304 },
	{ 0x9179, 0x9179, PDF_CMAP_SINGLE, 10285 },
	{ 0x917a, 0x917a, PDF_CMAP_SINGLE, 10283 },
	{ 0x917c, 0x917c, PDF_CMAP_SINGLE, 16239 },
	{ 0x9180, 0x9180, PDF_CMAP_SINGLE, 10894 },
	{ 0x9181, 0x9182, PDF_CMAP_RANGE, 10891 },
	{ 0x9183, 0x9183, PDF_CMAP_SINGLE, 4691 },
	{ 0x9184, 0x9184, PDF_CMAP_SINGLE, 10893 },
	{ 0x9185, 0x9186, PDF_CMAP_RANGE, 10888 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 4688 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 4689 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 10890 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 4690 },
	{ 0x918c, 0x918c, PDF_CMAP_SINGLE, 17388 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 11456 },
	{ 0x918e, 0x918e, PDF_CMAP_SINGLE, 16206 },
	{ 0x918f, 0x918f, PDF_CMAP_SINGLE, 11457 },
	{ 0x9190, 0x9191, PDF_CMAP_RANGE, 11454 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 4980 },
	{ 0x9193, 0x9193, PDF_CMAP_SINGLE, 11453 },
	{ 0x9199, 0x9199, PDF_CMAP_SINGLE, 11907 },
	{ 0x919a, 0x919a, PDF_CMAP_SINGLE, 11904 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 11906 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 5275 },
	{ 0x919d, 0x919d, PDF_CMAP_SINGLE, 11910 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 5274 },
	{ 0x919f, 0x919f, PDF_CMAP_SINGLE, 11908 },
	{ 0x91a0, 0x91a0, PDF_CMAP_SINGLE, 11911 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 11909 },
	{ 0x91a2, 0x91a2, PDF_CMAP_SINGLE, 11905 },
	{ 0x91a3, 0x91a3, PDF_CMAP_SINGLE, 5273 },
	{ 0x91a5, 0x91a5, PDF_CMAP_SINGLE, 12288 },
	{ 0x91a7, 0x91a7, PDF_CMAP_SINGLE, 12289 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 12287 },
	{ 0x91a9, 0x91a9, PDF_CMAP_SINGLE, 14467 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 12291 },
	{ 0x91ab, 0x91ac, PDF_CMAP_RANGE, 5436 },
	{ 0x91ad, 0x91ad, PDF_CMAP_SINGLE, 12618 },
	{ 0x91ae, 0x91ae, PDF_CMAP_SINGLE, 5596 },
	{ 0x91af, 0x91af, PDF_CMAP_SINGLE, 12290 },
	{ 0x91b0, 0x91b0, PDF_CMAP_SINGLE, 12617 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 5595 },
	{ 0x91b2, 0x91b3, PDF_CMAP_RANGE, 12876 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 5712 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 12875 },
	{ 0x91b6, 0x91b6, PDF_CMAP_SINGLE, 17903 },
	{ 0x91b7, 0x91b7, PDF_CMAP_SINGLE, 12874 },
	{ 0x91b9, 0x91b9, PDF_CMAP_SINGLE, 13089 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 5784 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 15304 },
	{ 0x91bc, 0x91bc, PDF_CMAP_SINGLE, 13382 },
	{ 0x91bd, 0x91bd, PDF_CMAP_SINGLE, 13475 },
	{ 0x91be, 0x91be, PDF_CMAP_SINGLE, 13474 },
	{ 0x91c0, 0x91c0, PDF_CMAP_SINGLE, 5936 },
	{ 0x91c1, 0x91c1, PDF_CMAP_SINGLE, 5962 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 13476 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 13579 },
	{ 0x91c4, 0x91c4, PDF_CMAP_SINGLE, 14468 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 5980 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 1313 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 1685 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 3914 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 5713 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 1314 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 2080 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 3017 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 3470 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 5438 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 1686 },
	{ 0x91d3, 0x91d3, PDF_CMAP_SINGLE, 7189 },
	{ 0x91d4, 0x91d4, PDF_CMAP_SINGLE, 7188 },
	{ 0x91d5, 0x91d5, PDF_CMAP_SINGLE, 7724 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 16734 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 2535 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 2533 },
	{ 0x91d9, 0x91d9, PDF_CMAP_SINGLE, 2537 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 7726 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 2536 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 2534 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 17722 },
	{ 0x91e2, 0x91e2, PDF_CMAP_SINGLE, 7725 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 3020 },
	{ 0x91e4, 0x91e4, PDF_CMAP_SINGLE, 8346 },
	{ 0x91e5, 0x91e5, PDF_CMAP_SINGLE, 15366 },
	{ 0x91e6, 0x91e6, PDF_CMAP_SINGLE, 3019 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 3021 },
	{ 0x91e8, 0x91e8, PDF_CMAP_SINGLE, 8351 },
	{ 0x91e9, 0x91e9, PDF_CMAP_SINGLE, 3023 },
	{ 0x91ea, 0x91eb, PDF_CMAP_RANGE, 8348 },
	{ 0x91ec, 0x91ec, PDF_CMAP_SINGLE, 8341 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 3022 },
	{ 0x91ee, 0x91ee, PDF_CMAP_SINGLE, 8352 },
	{ 0x91f1, 0x91f1, PDF_CMAP_SINGLE, 8343 },
	{ 0x91f3, 0x91f3, PDF_CMAP_SINGLE, 8344 },
	{ 0x91f4, 0x91f4, PDF_CMAP_SINGLE, 8342 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 3018 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 8350 },
	{ 0x91f8, 0x91f8, PDF_CMAP_SINGLE, 8345 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 8347 },
	{ 0x91fa, 0x91fa, PDF_CMAP_SINGLE, 15096 },
	{ 0x91fd, 0x91fd, PDF_CMAP_SINGLE, 9023 },
	{ 0x91fe, 0x91fe, PDF_CMAP_SINGLE, 14382 },
	{ 0x91ff, 0x91ff, PDF_CMAP_SINGLE, 9022 },
	{ 0x9200, 0x9200, PDF_CMAP_SINGLE, 9020 },
	{ 0x9201, 0x9201, PDF_CMAP_SINGLE, 9012 },
	{ 0x9202, 0x9202, PDF_CMAP_SINGLE, 9027 },
	{ 0x9203, 0x9203, PDF_CMAP_SINGLE, 9015 },
	{ 0x9204, 0x9204, PDF_CMAP_SINGLE, 9025 },
	{ 0x9205, 0x9205, PDF_CMAP_SINGLE, 9032 },
	{ 0x9206, 0x9206, PDF_CMAP_SINGLE, 9024 },
	{ 0x9207, 0x9207, PDF_CMAP_SINGLE, 3478 },
	{ 0x9208, 0x9208, PDF_CMAP_SINGLE, 16057 },
	{ 0x9209, 0x9209, PDF_CMAP_SINGLE, 3474 },
	{ 0x920a, 0x920a, PDF_CMAP_SINGLE, 9013 },
	{ 0x920c, 0x920c, PDF_CMAP_SINGLE, 9019 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 3476 },
	{ 0x920e, 0x920e, PDF_CMAP_SINGLE, 14472 },
	{ 0x920f, 0x920f, PDF_CMAP_SINGLE, 9018 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 3477 },
	{ 0x9211, 0x9211, PDF_CMAP_SINGLE, 3479 },
	{ 0x9212, 0x9212, PDF_CMAP_SINGLE, 9021 },
	{ 0x9213, 0x9213, PDF_CMAP_SINGLE, 18323 },
	{ 0x9214, 0x9215, PDF_CMAP_RANGE, 3471 },
	{ 0x9216, 0x9216, PDF_CMAP_SINGLE, 9033 },
	{ 0x9217, 0x9217, PDF_CMAP_SINGLE, 9031 },
	{ 0x9219, 0x9219, PDF_CMAP_SINGLE, 9030 },
	{ 0x921a, 0x921a, PDF_CMAP_SINGLE, 9016 },
	{ 0x921c, 0x921c, PDF_CMAP_SINGLE, 9028 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 3475 },
	{ 0x9223, 0x9223, PDF_CMAP_SINGLE, 3473 },
	{ 0x9224, 0x9224, PDF_CMAP_SINGLE, 9029 },
	{ 0x9225, 0x9225, PDF_CMAP_SINGLE, 9014 },
	{ 0x9226, 0x9226, PDF_CMAP_SINGLE, 9017 },
	{ 0x9227, 0x9227, PDF_CMAP_SINGLE, 9026 },
	{ 0x9228, 0x9228, PDF_CMAP_SINGLE, 18325 },
	{ 0x922a, 0x922a, PDF_CMAP_SINGLE, 15374 },
	{ 0x922b, 0x922b, PDF_CMAP_SINGLE, 15270 },
	{ 0x922d, 0x922d, PDF_CMAP_SINGLE, 10321 },
	{ 0x922e, 0x922e, PDF_CMAP_SINGLE, 9690 },
	{ 0x9230, 0x9230, PDF_CMAP_SINGLE, 9683 },
	{ 0x9231, 0x9231, PDF_CMAP_SINGLE, 9702 },
	{ 0x9232, 0x9232, PDF_CMAP_SINGLE, 9711 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 9686 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 3925 },
	{ 0x9235, 0x9235, PDF_CMAP_SINGLE, 15142 },
	{ 0x9236, 0x9236, PDF_CMAP_SINGLE, 9699 },
	{ 0x9237, 0x9237, PDF_CMAP_SINGLE, 3915 },
	{ 0x9238, 0x9238, PDF_CMAP_SINGLE, 3917 },
	{ 0x9239, 0x9239, PDF_CMAP_SINGLE, 3929 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 9684 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 16850 },
	{ 0x923d, 0x923d, PDF_CMAP_SINGLE, 3918 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 3920 },
	{ 0x923f, 0x923f, PDF_CMAP_SINGLE, 3930 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 3919 },
	{ 0x9241, 0x9241, PDF_CMAP_SINGLE, 14474 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 14490 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 3928 },
	{ 0x9246, 0x9246, PDF_CMAP_SINGLE, 9692 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 9681 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 3926 },
	{ 0x924a, 0x924a, PDF_CMAP_SINGLE, 9691 },
	{ 0x924b, 0x924b, PDF_CMAP_SINGLE, 3922 },
	{ 0x924c, 0x924c, PDF_CMAP_SINGLE, 9709 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 3927 },
	{ 0x924e, 0x924e, PDF_CMAP_SINGLE, 9707 },
	{ 0x924f, 0x924f, PDF_CMAP_SINGLE, 9695 },
	{ 0x9250, 0x9250, PDF_CMAP_SINGLE, 9705 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 3924 },
	{ 0x9252, 0x9252, PDF_CMAP_SINGLE, 9682 },
	{ 0x9253, 0x9253, PDF_CMAP_SINGLE, 9708 },
	{ 0x9254, 0x9254, PDF_CMAP_SINGLE, 9703 },
	{ 0x9255, 0x9255, PDF_CMAP_SINGLE, 15943 },
	{ 0x9256, 0x9256, PDF_CMAP_SINGLE, 9710 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 3916 },
	{ 0x9258, 0x9258, PDF_CMAP_SINGLE, 18326 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 3931 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 3921 },
	{ 0x925d, 0x925d, PDF_CMAP_SINGLE, 14851 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 9688 },
	{ 0x925f, 0x925f, PDF_CMAP_SINGLE, 15584 },
	{ 0x9260, 0x9260, PDF_CMAP_SINGLE, 9696 },
	{ 0x9261, 0x9261, PDF_CMAP_SINGLE, 9700 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 14475 },
	{ 0x9263, 0x9263, PDF_CMAP_SINGLE, 9704 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 3923 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 9687 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 9685 },
	{ 0x9267, 0x9267, PDF_CMAP_SINGLE, 9697 },
	{ 0x926b, 0x926b, PDF_CMAP_SINGLE, 18327 },
	{ 0x926c, 0x926c, PDF_CMAP_SINGLE, 9694 },
	{ 0x926d, 0x926d, PDF_CMAP_SINGLE, 9693 },
	{ 0x926e, 0x926e, PDF_CMAP_SINGLE, 15621 },
	{ 0x926f, 0x926f, PDF_CMAP_SINGLE, 9698 },
	{ 0x9270, 0x9270, PDF_CMAP_SINGLE, 9701 },
	{ 0x9272, 0x9272, PDF_CMAP_SINGLE, 9706 },
	{ 0x9276, 0x9276, PDF_CMAP_SINGLE, 10289 },
	{ 0x9277, 0x9277, PDF_CMAP_SINGLE, 15460 },
	{ 0x9278, 0x9278, PDF_CMAP_SINGLE, 4307 },
	{ 0x9279, 0x9279, PDF_CMAP_SINGLE, 10299 },
	{ 0x927a, 0x927a, PDF_CMAP_SINGLE, 10291 },
	{ 0x927b, 0x927b, PDF_CMAP_SINGLE, 4313 },
	{ 0x927c, 0x927c, PDF_CMAP_SINGLE, 4317 },
	{ 0x927d, 0x927d, PDF_CMAP_SINGLE, 10308 },
	{ 0x927e, 0x927e, PDF_CMAP_SINGLE, 10316 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 10301 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 4309 },
	{ 0x9281, 0x9281, PDF_CMAP_SINGLE, 15873 },
	{ 0x9282, 0x9282, PDF_CMAP_SINGLE, 10305 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 9689 },
	{ 0x9284, 0x9284, PDF_CMAP_SINGLE, 15545 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 4310 },
	{ 0x9286, 0x9286, PDF_CMAP_SINGLE, 10312 },
	{ 0x9287, 0x9287, PDF_CMAP_SINGLE, 10317 },
	{ 0x9288, 0x9288, PDF_CMAP_SINGLE, 10309 },
	{ 0x9289, 0x9289, PDF_CMAP_SINGLE, 15029 },
	{ 0x928a, 0x928a, PDF_CMAP_SINGLE, 10311 },
	{ 0x928b, 0x928b, PDF_CMAP_SINGLE, 10320 },
	{ 0x928c, 0x928c, PDF_CMAP_SINGLE, 10313 },
	{ 0x928d, 0x928d, PDF_CMAP_SINGLE, 10295 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 10304 },
	{ 0x928f, 0x928f, PDF_CMAP_SINGLE, 18109 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 4318 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 4314 },
	{ 0x9294, 0x9294, PDF_CMAP_SINGLE, 10293 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 10306 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 4312 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 10300 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 4311 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 10314 },
	{ 0x929a, 0x929a, PDF_CMAP_SINGLE, 10297 },
	{ 0x929b, 0x929b, PDF_CMAP_SINGLE, 10290 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 4315 },
	{ 0x929d, 0x929d, PDF_CMAP_SINGLE, 10319 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 10292 },
	{ 0x92a1, 0x92a1, PDF_CMAP_SINGLE, 10310 },
	{ 0x92a2, 0x92a2, PDF_CMAP_SINGLE, 10307 },
	{ 0x92a3, 0x92a3, PDF_CMAP_SINGLE, 10302 },
	{ 0x92a4, 0x92a4, PDF_CMAP_SINGLE, 10288 },
	{ 0x92a5, 0x92a5, PDF_CMAP_SINGLE, 10287 },
	{ 0x92a6, 0x92a6, PDF_CMAP_SINGLE, 10296 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 10315 },
	{ 0x92a8, 0x92a8, PDF_CMAP_SINGLE, 4316 },
	{ 0x92a9, 0x92a9, PDF_CMAP_SINGLE, 10318 },
	{ 0x92aa, 0x92aa, PDF_CMAP_SINGLE, 10294 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 10298 },
	{ 0x92ac, 0x92ac, PDF_CMAP_SINGLE, 4308 },
	{ 0x92ae, 0x92ae, PDF_CMAP_SINGLE, 18329 },
	{ 0x92b1, 0x92b1, PDF_CMAP_SINGLE, 18328 },
	{ 0x92b2, 0x92b2, PDF_CMAP_SINGLE, 4703 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 4698 },
	{ 0x92b4, 0x92b4, PDF_CMAP_SINGLE, 10928 },
	{ 0x92b5, 0x92b5, PDF_CMAP_SINGLE, 10924 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 10900 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 4694 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 14057 },
	{ 0x92ba, 0x92ba, PDF_CMAP_SINGLE, 15349 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 4693 },
	{ 0x92bc, 0x92bc, PDF_CMAP_SINGLE, 4699 },
	{ 0x92be, 0x92be, PDF_CMAP_SINGLE, 15456 },
	{ 0x92bf, 0x92bf, PDF_CMAP_SINGLE, 18330 },
	{ 0x92c0, 0x92c0, PDF_CMAP_SINGLE, 10898 },
	{ 0x92c1, 0x92c1, PDF_CMAP_SINGLE, 4697 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 10910 },
	{ 0x92c3, 0x92c4, PDF_CMAP_RANGE, 10896 },
	{ 0x92c5, 0x92c5, PDF_CMAP_SINGLE, 4692 },
	{ 0x92c6, 0x92c6, PDF_CMAP_SINGLE, 10927 },
	{ 0x92c7, 0x92c7, PDF_CMAP_SINGLE, 4701 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 10913 },
	{ 0x92c9, 0x92c9, PDF_CMAP_SINGLE, 10918 },
	{ 0x92ca, 0x92ca, PDF_CMAP_SINGLE, 10912 },
	{ 0x92cb, 0x92cb, PDF_CMAP_SINGLE, 11475 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 10908 },
	{ 0x92cd, 0x92cd, PDF_CMAP_SINGLE, 10916 },
	{ 0x92ce, 0x92ce, PDF_CMAP_SINGLE, 10914 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 10901 },
	{ 0x92d0, 0x92d0, PDF_CMAP_SINGLE, 10895 },
	{ 0x92d1, 0x92d1, PDF_CMAP_SINGLE, 10922 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 4700 },
	{ 0x92d3, 0x92d3, PDF_CMAP_SINGLE, 10923 },
	{ 0x92d4, 0x92d4, PDF_CMAP_SINGLE, 14433 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 10917 },
	{ 0x92d7, 0x92d7, PDF_CMAP_SINGLE, 10906 },
	{ 0x92d8, 0x92d8, PDF_CMAP_SINGLE, 10904 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 10899 },
	{ 0x92db, 0x92db, PDF_CMAP_SINGLE, 16396 },
	{ 0x92dd, 0x92dd, PDF_CMAP_SINGLE, 10907 },
	{ 0x92de, 0x92de, PDF_CMAP_SINGLE, 10920 },
	{ 0x92df, 0x92df, PDF_CMAP_SINGLE, 10903 },
	{ 0x92e0, 0x92e0, PDF_CMAP_SINGLE, 10919 },
	{ 0x92e1, 0x92e1, PDF_CMAP_SINGLE, 10925 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 18331 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 4696 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 15522 },
	{ 0x92e6, 0x92e6, PDF_CMAP_SINGLE, 10915 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 10921 },
	{ 0x92e8, 0x92e8, PDF_CMAP_SINGLE, 10911 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 10905 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 4695 },
	{ 0x92eb, 0x92eb, PDF_CMAP_SINGLE, 18332 },
	{ 0x92ec, 0x92ec, PDF_CMAP_SINGLE, 14692 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 10303 },
	{ 0x92ef, 0x92ef, PDF_CMAP_SINGLE, 10909 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 4702 },
	{ 0x92f1, 0x92f1, PDF_CMAP_SINGLE, 10902 },
	{ 0x92f2, 0x92f2, PDF_CMAP_SINGLE, 15026 },
	{ 0x92f3, 0x92f4, PDF_CMAP_RANGE, 18333 },
	{ 0x92f6, 0x92f6, PDF_CMAP_SINGLE, 15364 },
	{ 0x92f7, 0x92f7, PDF_CMAP_SINGLE, 11480 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 4983 },
	{ 0x92f9, 0x92f9, PDF_CMAP_SINGLE, 11479 },
	{ 0x92fa, 0x92fa, PDF_CMAP_SINGLE, 11477 },
	{ 0x92fb, 0x92fb, PDF_CMAP_SINGLE, 11495 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 4987 },
	{ 0x92fd, 0x92fd, PDF_CMAP_SINGLE, 18335 },
	{ 0x92fe, 0x92fe, PDF_CMAP_SINGLE, 11492 },
	{ 0x92ff, 0x92ff, PDF_CMAP_SINGLE, 11484 },
	{ 0x9300, 0x9300, PDF_CMAP_SINGLE, 11494 },
	{ 0x9301, 0x9301, PDF_CMAP_SINGLE, 11470 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 11482 },
	{ 0x9303, 0x9303, PDF_CMAP_SINGLE, 14439 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 4989 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 11462 },
	{ 0x9307, 0x9307, PDF_CMAP_SINGLE, 15121 },
	{ 0x9308, 0x9308, PDF_CMAP_SINGLE, 11460 },
	{ 0x9309, 0x9309, PDF_CMAP_SINGLE, 11493 },
	{ 0x930b, 0x930b, PDF_CMAP_SINGLE, 11491 },
	{ 0x930c, 0x930c, PDF_CMAP_SINGLE, 11490 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 11474 },
	{ 0x930e, 0x930e, PDF_CMAP_SINGLE, 11473 },
	{ 0x930f, 0x930f, PDF_CMAP_SINGLE, 11463 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 4991 },
	{ 0x9312, 0x9312, PDF_CMAP_SINGLE, 11469 },
	{ 0x9313, 0x9313, PDF_CMAP_SINGLE, 11478 },
	{ 0x9314, 0x9314, PDF_CMAP_SINGLE, 11489 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 4994 },
	{ 0x9316, 0x9316, PDF_CMAP_SINGLE, 11496 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 5283 },
	{ 0x9319, 0x9319, PDF_CMAP_SINGLE, 4996 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 4990 },
	{ 0x931b, 0x931b, PDF_CMAP_SINGLE, 11467 },
	{ 0x931d, 0x931d, PDF_CMAP_SINGLE, 11476 },
	{ 0x931e, 0x931e, PDF_CMAP_SINGLE, 11459 },
	{ 0x931f, 0x931f, PDF_CMAP_SINGLE, 11461 },
	{ 0x9320, 0x9320, PDF_CMAP_SINGLE, 4981 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 4993 },
	{ 0x9322, 0x9322, PDF_CMAP_SINGLE, 4986 },
	{ 0x9323, 0x9323, PDF_CMAP_SINGLE, 11468 },
	{ 0x9324, 0x9324, PDF_CMAP_SINGLE, 11483 },
	{ 0x9325, 0x9325, PDF_CMAP_SINGLE, 10926 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 4992 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 11458 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 5278 },
	{ 0x9329, 0x9329, PDF_CMAP_SINGLE, 11485 },
	{ 0x932a, 0x932a, PDF_CMAP_SINGLE, 11488 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 4988 },
	{ 0x932c, 0x932c, PDF_CMAP_SINGLE, 14482 },
	{ 0x932d, 0x932d, PDF_CMAP_SINGLE, 11472 },
	{ 0x932e, 0x932e, PDF_CMAP_SINGLE, 4995 },
	{ 0x932f, 0x932f, PDF_CMAP_SINGLE, 4985 },
	{ 0x9330, 0x9330, PDF_CMAP_SINGLE, 15875 },
	{ 0x9331, 0x9331, PDF_CMAP_SINGLE, 15549 },
	{ 0x9333, 0x9333, PDF_CMAP_SINGLE, 4984 },
	{ 0x9334, 0x9334, PDF_CMAP_SINGLE, 11481 },
	{ 0x9335, 0x9335, PDF_CMAP_SINGLE, 11487 },
	{ 0x9336, 0x9336, PDF_CMAP_SINGLE, 4982 },
	{ 0x9338, 0x9338, PDF_CMAP_SINGLE, 11465 },
	{ 0x9339, 0x9339, PDF_CMAP_SINGLE, 11486 },
	{ 0x933c, 0x933c, PDF_CMAP_SINGLE, 11466 },
	{ 0x9340, 0x9340, PDF_CMAP_SINGLE, 16725 },
	{ 0x9341, 0x9341, PDF_CMAP_SINGLE, 17015 },
	{ 0x9342, 0x9342, PDF_CMAP_SINGLE, 15098 },
	{ 0x9343, 0x9343, PDF_CMAP_SINGLE, 18336 },
	{ 0x9344, 0x9344, PDF_CMAP_SINGLE, 16406 },
	{ 0x9345, 0x9345, PDF_CMAP_SINGLE, 15988 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 11471 },
	{ 0x9347, 0x9347, PDF_CMAP_SINGLE, 11917 },
	{ 0x9348, 0x9348, PDF_CMAP_SINGLE, 15457 },
	{ 0x9349, 0x9349, PDF_CMAP_SINGLE, 11922 },
	{ 0x934a, 0x934a, PDF_CMAP_SINGLE, 5280 },
	{ 0x934b, 0x934b, PDF_CMAP_SINGLE, 5282 },
	{ 0x934c, 0x934c, PDF_CMAP_SINGLE, 11928 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 5276 },
	{ 0x934e, 0x934e, PDF_CMAP_SINGLE, 11942 },
	{ 0x934f, 0x934f, PDF_CMAP_SINGLE, 11934 },
	{ 0x9350, 0x9351, PDF_CMAP_RANGE, 11923 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 11933 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 5289 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 11932 },
	{ 0x9356, 0x9356, PDF_CMAP_SINGLE, 11916 },
	{ 0x9357, 0x9357, PDF_CMAP_SINGLE, 11931 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 11919 },
	{ 0x9359, 0x9359, PDF_CMAP_SINGLE, 11943 },
	{ 0x935a, 0x935a, PDF_CMAP_SINGLE, 5288 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 5286 },
	{ 0x935c, 0x935c, PDF_CMAP_SINGLE, 11920 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 11939 },
	{ 0x935f, 0x935f, PDF_CMAP_SINGLE, 15547 },
	{ 0x9360, 0x9360, PDF_CMAP_SINGLE, 11925 },
	{ 0x9361, 0x9361, PDF_CMAP_SINGLE, 11938 },
	{ 0x9362, 0x9362, PDF_CMAP_SINGLE, 17000 },
	{ 0x9363, 0x9363, PDF_CMAP_SINGLE, 11940 },
	{ 0x9364, 0x9364, PDF_CMAP_SINGLE, 11915 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 5281 },
	{ 0x9366, 0x9366, PDF_CMAP_SINGLE, 18108 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 11941 },
	{ 0x9368, 0x9368, PDF_CMAP_SINGLE, 15344 },
	{ 0x9369, 0x9369, PDF_CMAP_SINGLE, 14689 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 11929 },
	{ 0x936b, 0x936b, PDF_CMAP_SINGLE, 14483 },
	{ 0x936c, 0x936c, PDF_CMAP_SINGLE, 5285 },
	{ 0x936d, 0x936d, PDF_CMAP_SINGLE, 11926 },
	{ 0x936e, 0x936e, PDF_CMAP_SINGLE, 14824 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 5287 },
	{ 0x9371, 0x9371, PDF_CMAP_SINGLE, 11935 },
	{ 0x9373, 0x9373, PDF_CMAP_SINGLE, 14494 },
	{ 0x9374, 0x9374, PDF_CMAP_SINGLE, 15588 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 5279 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 11921 },
	{ 0x9377, 0x9377, PDF_CMAP_SINGLE, 11936 },
	{ 0x9378, 0x9378, PDF_CMAP_SINGLE, 14585 },
	{ 0x9379, 0x9379, PDF_CMAP_SINGLE, 11930 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 11464 },
	{ 0x937b, 0x937b, PDF_CMAP_SINGLE, 11937 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 11918 },
	{ 0x937d, 0x937d, PDF_CMAP_SINGLE, 15397 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 5284 },
	{ 0x9380, 0x9380, PDF_CMAP_SINGLE, 12308 },
	{ 0x9381, 0x9381, PDF_CMAP_SINGLE, 14693 },
	{ 0x9382, 0x9382, PDF_CMAP_SINGLE, 5277 },
	{ 0x9383, 0x9383, PDF_CMAP_SINGLE, 11913 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 18337 },
	{ 0x9385, 0x9385, PDF_CMAP_SINGLE, 17085 },
	{ 0x9386, 0x9386, PDF_CMAP_SINGLE, 15552 },
	{ 0x9387, 0x9387, PDF_CMAP_SINGLE, 15072 },
	{ 0x9388, 0x9388, PDF_CMAP_SINGLE, 12305 },
	{ 0x9389, 0x9389, PDF_CMAP_SINGLE, 12298 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 5440 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 12293 },
	{ 0x938d, 0x938d, PDF_CMAP_SINGLE, 12309 },
	{ 0x938e, 0x938e, PDF_CMAP_SINGLE, 12300 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 11927 },
	{ 0x9390, 0x9390, PDF_CMAP_SINGLE, 15676 },
	{ 0x9391, 0x9391, PDF_CMAP_SINGLE, 12311 },
	{ 0x9392, 0x9392, PDF_CMAP_SINGLE, 12294 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 5439 },
	{ 0x9395, 0x9395, PDF_CMAP_SINGLE, 12304 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 5441 },
	{ 0x9397, 0x9397, PDF_CMAP_SINGLE, 5449 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 5447 },
	{ 0x9399, 0x9399, PDF_CMAP_SINGLE, 12306 },
	{ 0x939a, 0x939a, PDF_CMAP_SINGLE, 5448 },
	{ 0x939b, 0x939b, PDF_CMAP_SINGLE, 12296 },
	{ 0x939c, 0x939c, PDF_CMAP_SINGLE, 14764 },
	{ 0x939d, 0x939d, PDF_CMAP_SINGLE, 12297 },
	{ 0x939e, 0x939e, PDF_CMAP_SINGLE, 12302 },
	{ 0x939f, 0x939f, PDF_CMAP_SINGLE, 12307 },
	{ 0x93a0, 0x93a0, PDF_CMAP_SINGLE, 15025 },
	{ 0x93a1, 0x93a1, PDF_CMAP_SINGLE, 11912 },
	{ 0x93a2, 0x93a2, PDF_CMAP_SINGLE, 5442 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 12316 },
	{ 0x93a4, 0x93a4, PDF_CMAP_SINGLE, 12313 },
	{ 0x93a5, 0x93a5, PDF_CMAP_SINGLE, 12640 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 12303 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 12299 },
	{ 0x93a8, 0x93a8, PDF_CMAP_SINGLE, 12314 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 12629 },
	{ 0x93aa, 0x93aa, PDF_CMAP_SINGLE, 12301 },
	{ 0x93ac, 0x93ac, PDF_CMAP_SINGLE, 5445 },
	{ 0x93ad, 0x93ad, PDF_CMAP_SINGLE, 18338 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 5444 },
	{ 0x93af, 0x93af, PDF_CMAP_SINGLE, 11914 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 5446 },
	{ 0x93b1, 0x93b1, PDF_CMAP_SINGLE, 12310 },
	{ 0x93b2, 0x93b2, PDF_CMAP_SINGLE, 12312 },
	{ 0x93b3, 0x93b3, PDF_CMAP_SINGLE, 5443 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 12315 },
	{ 0x93b5, 0x93b5, PDF_CMAP_SINGLE, 12292 },
	{ 0x93b7, 0x93b7, PDF_CMAP_SINGLE, 12295 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 15504 },
	{ 0x93ba, 0x93ba, PDF_CMAP_SINGLE, 17399 },
	{ 0x93bb, 0x93bb, PDF_CMAP_SINGLE, 15503 },
	{ 0x93bd, 0x93bd, PDF_CMAP_SINGLE, 14887 },
	{ 0x93bf, 0x93bf, PDF_CMAP_SINGLE, 15917 },
	{ 0x93c0, 0x93c0, PDF_CMAP_SINGLE, 12638 },
	{ 0x93c2, 0x93c2, PDF_CMAP_SINGLE, 12622 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 5600 },
	{ 0x93c4, 0x93c4, PDF_CMAP_SINGLE, 12636 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 14762 },
	{ 0x93c7, 0x93c7, PDF_CMAP_SINGLE, 12620 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 5601 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 12631 },
	{ 0x93cb, 0x93cb, PDF_CMAP_SINGLE, 14694 },
	{ 0x93cc, 0x93cc, PDF_CMAP_SINGLE, 12627 },
	{ 0x93cd, 0x93cd, PDF_CMAP_SINGLE, 5606 },
	{ 0x93ce, 0x93ce, PDF_CMAP_SINGLE, 12637 },
	{ 0x93cf, 0x93cf, PDF_CMAP_SINGLE, 12621 },
	{ 0x93d0, 0x93d0, PDF_CMAP_SINGLE, 12624 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 5598 },
	{ 0x93d2, 0x93d2, PDF_CMAP_SINGLE, 12639 },
	{ 0x93d3, 0x93d3, PDF_CMAP_SINGLE, 16609 },
	{ 0x93d4, 0x93d4, PDF_CMAP_SINGLE, 12632 },
	{ 0x93d5, 0x93d5, PDF_CMAP_SINGLE, 12635 },
	{ 0x93d6, 0x93d6, PDF_CMAP_SINGLE, 5604 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 5609 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 5607 },
	{ 0x93d9, 0x93d9, PDF_CMAP_SINGLE, 12628 },
	{ 0x93da, 0x93da, PDF_CMAP_SINGLE, 12623 },
	{ 0x93db, 0x93db, PDF_CMAP_SINGLE, 15455 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 5602 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 12619 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 5599 },
	{ 0x93e0, 0x93e0, PDF_CMAP_SINGLE, 14880 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 5597 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 5605 },
	{ 0x93e3, 0x93e3, PDF_CMAP_SINGLE, 12634 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 5608 },
	{ 0x93e6, 0x93e6, PDF_CMAP_SINGLE, 12630 },
	{ 0x93e7, 0x93e7, PDF_CMAP_SINGLE, 12641 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 5610 },
	{ 0x93ec, 0x93ec, PDF_CMAP_SINGLE, 12626 },
	{ 0x93ee, 0x93ee, PDF_CMAP_SINGLE, 12633 },
	{ 0x93f0, 0x93f0, PDF_CMAP_SINGLE, 16959 },
	{ 0x93f1, 0x93f1, PDF_CMAP_SINGLE, 17066 },
	{ 0x93f3, 0x93f3, PDF_CMAP_SINGLE, 14690 },
	{ 0x93f5, 0x93f5, PDF_CMAP_SINGLE, 12890 },
	{ 0x93f6, 0x93f6, PDF_CMAP_SINGLE, 12905 },
	{ 0x93f7, 0x93f7, PDF_CMAP_SINGLE, 12892 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 12899 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 12625 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 12897 },
	{ 0x93fb, 0x93fb, PDF_CMAP_SINGLE, 12880 },
	{ 0x93fc, 0x93fc, PDF_CMAP_SINGLE, 12903 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 5716 },
	{ 0x93fe, 0x93fe, PDF_CMAP_SINGLE, 12884 },
	{ 0x93ff, 0x93ff, PDF_CMAP_SINGLE, 12902 },
	{ 0x9400, 0x9400, PDF_CMAP_SINGLE, 12891 },
	{ 0x9401, 0x9401, PDF_CMAP_SINGLE, 15572 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 5715 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 15305 },
	{ 0x9406, 0x9406, PDF_CMAP_SINGLE, 12907 },
	{ 0x9407, 0x9407, PDF_CMAP_SINGLE, 12893 },
	{ 0x9408, 0x9408, PDF_CMAP_SINGLE, 14885 },
	{ 0x9409, 0x9409, PDF_CMAP_SINGLE, 12898 },
	{ 0x940a, 0x940a, PDF_CMAP_SINGLE, 12901 },
	{ 0x940b, 0x940b, PDF_CMAP_SINGLE, 12878 },
	{ 0x940c, 0x940c, PDF_CMAP_SINGLE, 12904 },
	{ 0x940d, 0x940d, PDF_CMAP_SINGLE, 12889 },
	{ 0x940e, 0x940e, PDF_CMAP_SINGLE, 12894 },
	{ 0x940f, 0x940f, PDF_CMAP_SINGLE, 12882 },
	{ 0x9410, 0x9410, PDF_CMAP_SINGLE, 12886 },
	{ 0x9411, 0x9411, PDF_CMAP_SINGLE, 12906 },
	{ 0x9412, 0x9412, PDF_CMAP_SINGLE, 12896 },
	{ 0x9413, 0x9413, PDF_CMAP_SINGLE, 12879 },
	{ 0x9414, 0x9414, PDF_CMAP_SINGLE, 12883 },
	{ 0x9415, 0x9415, PDF_CMAP_SINGLE, 12885 },
	{ 0x9416, 0x9416, PDF_CMAP_SINGLE, 12895 },
	{ 0x9417, 0x9417, PDF_CMAP_SINGLE, 18341 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 5714 },
	{ 0x9419, 0x9419, PDF_CMAP_SINGLE, 12888 },
	{ 0x941b, 0x941b, PDF_CMAP_SINGLE, 16712 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 18342 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 12881 },
	{ 0x9424, 0x9424, PDF_CMAP_SINGLE, 14659 },
	{ 0x9425, 0x9425, PDF_CMAP_SINGLE, 14544 },
	{ 0x9426, 0x9426, PDF_CMAP_SINGLE, 14499 },
	{ 0x9427, 0x9427, PDF_CMAP_SINGLE, 15309 },
	{ 0x9428, 0x9428, PDF_CMAP_SINGLE, 12887 },
	{ 0x9429, 0x9429, PDF_CMAP_SINGLE, 13093 },
	{ 0x942a, 0x942a, PDF_CMAP_SINGLE, 13097 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 5791 },
	{ 0x942c, 0x942c, PDF_CMAP_SINGLE, 13099 },
	{ 0x942d, 0x942d, PDF_CMAP_SINGLE, 18343 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 5785 },
	{ 0x9430, 0x9430, PDF_CMAP_SINGLE, 13095 },
	{ 0x9431, 0x9431, PDF_CMAP_SINGLE, 13101 },
	{ 0x9432, 0x9432, PDF_CMAP_SINGLE, 5790 },
	{ 0x9433, 0x9433, PDF_CMAP_SINGLE, 5786 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 5787 },
	{ 0x9436, 0x9436, PDF_CMAP_SINGLE, 13092 },
	{ 0x9437, 0x9437, PDF_CMAP_SINGLE, 13098 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 5789 },
	{ 0x9439, 0x9439, PDF_CMAP_SINGLE, 13096 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 5788 },
	{ 0x943b, 0x943b, PDF_CMAP_SINGLE, 13091 },
	{ 0x943c, 0x943c, PDF_CMAP_SINGLE, 12900 },
	{ 0x943d, 0x943d, PDF_CMAP_SINGLE, 13094 },
	{ 0x943e, 0x943e, PDF_CMAP_SINGLE, 18344 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 13090 },
	{ 0x9440, 0x9440, PDF_CMAP_SINGLE, 13100 },
	{ 0x9442, 0x9442, PDF_CMAP_SINGLE, 17010 },
	{ 0x9443, 0x9443, PDF_CMAP_SINGLE, 17013 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 5855 },
	{ 0x9445, 0x9445, PDF_CMAP_SINGLE, 13261 },
	{ 0x9446, 0x9446, PDF_CMAP_SINGLE, 13264 },
	{ 0x9447, 0x9447, PDF_CMAP_SINGLE, 13260 },
	{ 0x9448, 0x9449, PDF_CMAP_RANGE, 13262 },
	{ 0x944a, 0x944b, PDF_CMAP_RANGE, 13257 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 13255 },
	{ 0x944d, 0x944d, PDF_CMAP_SINGLE, 15461 },
	{ 0x944f, 0x944f, PDF_CMAP_SINGLE, 13259 },
	{ 0x9450, 0x9450, PDF_CMAP_SINGLE, 13256 },
	{ 0x9451, 0x9452, PDF_CMAP_RANGE, 5856 },
	{ 0x9454, 0x9454, PDF_CMAP_SINGLE, 18346 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 13384 },
	{ 0x9457, 0x9457, PDF_CMAP_SINGLE, 13386 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 15408 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 14495 },
	{ 0x945d, 0x945d, PDF_CMAP_SINGLE, 13385 },
	{ 0x945e, 0x945e, PDF_CMAP_SINGLE, 13387 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 5902 },
	{ 0x9462, 0x9462, PDF_CMAP_SINGLE, 13383 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 5901 },
	{ 0x9464, 0x9464, PDF_CMAP_SINGLE, 5903 },
	{ 0x9465, 0x9465, PDF_CMAP_SINGLE, 15838 },
	{ 0x9467, 0x9467, PDF_CMAP_SINGLE, 15032 },
	{ 0x9468, 0x9469, PDF_CMAP_RANGE, 13478 },
	{ 0x946a, 0x946a, PDF_CMAP_SINGLE, 5937 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 13477 },
	{ 0x946c, 0x946c, PDF_CMAP_SINGLE, 14206 },
	{ 0x946d, 0x946d, PDF_CMAP_SINGLE, 13549 },
	{ 0x946e, 0x946e, PDF_CMAP_SINGLE, 13548 },
	{ 0x946f, 0x946f, PDF_CMAP_SINGLE, 13550 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 5964 },
	{ 0x9471, 0x9471, PDF_CMAP_SINGLE, 13551 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 5963 },
	{ 0x9473, 0x9473, PDF_CMAP_SINGLE, 13552 },
	{ 0x9474, 0x9474, PDF_CMAP_SINGLE, 13580 },
	{ 0x9475, 0x9475, PDF_CMAP_SINGLE, 13583 },
	{ 0x9476, 0x9476, PDF_CMAP_SINGLE, 13582 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 5973 },
	{ 0x9478, 0x9478, PDF_CMAP_SINGLE, 13581 },
	{ 0x9479, 0x9479, PDF_CMAP_SINGLE, 18347 },
	{ 0x947b, 0x947b, PDF_CMAP_SINGLE, 16795 },
	{ 0x947c, 0x947c, PDF_CMAP_SINGLE, 5983 },
	{ 0x947d, 0x947e, PDF_CMAP_RANGE, 5981 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 5988 },
	{ 0x9480, 0x9481, PDF_CMAP_RANGE, 13625 },
	{ 0x9482, 0x9482, PDF_CMAP_SINGLE, 13624 },
	{ 0x9483, 0x9483, PDF_CMAP_SINGLE, 13632 },
	{ 0x9485, 0x9485, PDF_CMAP_SINGLE, 17671 },
	{ 0x949f, 0x949f, PDF_CMAP_SINGLE, 18565 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 17691 },
	{ 0x94c1, 0x94c1, PDF_CMAP_SINGLE, 18401 },
	{ 0x94c3, 0x94c3, PDF_CMAP_SINGLE, 18399 },
	{ 0x94dc, 0x94dc, PDF_CMAP_SINGLE, 18391 },
	{ 0x94f6, 0x94f6, PDF_CMAP_SINGLE, 18386 },
	{ 0x952d, 0x952d, PDF_CMAP_SINGLE, 18348 },
	{ 0x9547, 0x9547, PDF_CMAP_SINGLE, 18623 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 1687 },
	{ 0x9578, 0x9578, PDF_CMAP_SINGLE, 17672 },
	{ 0x957a, 0x957a, PDF_CMAP_SINGLE, 8353 },
	{ 0x957b, 0x957b, PDF_CMAP_SINGLE, 9034 },
	{ 0x957c, 0x957c, PDF_CMAP_SINGLE, 10929 },
	{ 0x957d, 0x957d, PDF_CMAP_SINGLE, 12642 },
	{ 0x957f, 0x957f, PDF_CMAP_SINGLE, 17673 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 1688 },
	{ 0x9582, 0x9582, PDF_CMAP_SINGLE, 2081 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 2538 },
	{ 0x9585, 0x9585, PDF_CMAP_SINGLE, 14497 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 8354 },
	{ 0x9588, 0x9588, PDF_CMAP_SINGLE, 8355 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 3024 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 3482 },
	{ 0x958c, 0x958c, PDF_CMAP_SINGLE, 9036 },
	{ 0x958d, 0x958d, PDF_CMAP_SINGLE, 9035 },
	{ 0x958e, 0x958e, PDF_CMAP_SINGLE, 3486 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 3481 },
	{ 0x9590, 0x9590, PDF_CMAP_SINGLE, 9037 },
	{ 0x9591, 0x9591, PDF_CMAP_SINGLE, 3483 },
	{ 0x9592, 0x9592, PDF_CMAP_SINGLE, 3485 },
	{ 0x9593, 0x9593, PDF_CMAP_SINGLE, 3484 },
	{ 0x9594, 0x9594, PDF_CMAP_SINGLE, 3480 },
	{ 0x9596, 0x9596, PDF_CMAP_SINGLE, 15317 },
	{ 0x9597, 0x9597, PDF_CMAP_SINGLE, 15316 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 3932 },
	{ 0x9599, 0x9599, PDF_CMAP_SINGLE, 15314 },
	{ 0x959b, 0x959b, PDF_CMAP_SINGLE, 9715 },
	{ 0x959c, 0x959c, PDF_CMAP_SINGLE, 9713 },
	{ 0x959e, 0x959e, PDF_CMAP_SINGLE, 9714 },
	{ 0x959f, 0x959f, PDF_CMAP_SINGLE, 9712 },
	{ 0x95a0, 0x95a0, PDF_CMAP_SINGLE, 14500 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 4319 },
	{ 0x95a2, 0x95a2, PDF_CMAP_SINGLE, 18349 },
	{ 0x95a3, 0x95a3, PDF_CMAP_SINGLE, 4322 },
	{ 0x95a4, 0x95a4, PDF_CMAP_SINGLE, 4324 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 4323 },
	{ 0x95a6, 0x95a6, PDF_CMAP_SINGLE, 14498 },
	{ 0x95a7, 0x95a7, PDF_CMAP_SINGLE, 15315 },
	{ 0x95a8, 0x95a9, PDF_CMAP_RANGE, 4320 },
	{ 0x95aa, 0x95aa, PDF_CMAP_SINGLE, 17301 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 10931 },
	{ 0x95ac, 0x95ac, PDF_CMAP_SINGLE, 10930 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 4704 },
	{ 0x95ae, 0x95ae, PDF_CMAP_SINGLE, 10932 },
	{ 0x95b0, 0x95b0, PDF_CMAP_SINGLE, 10933 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 4705 },
	{ 0x95b5, 0x95b5, PDF_CMAP_SINGLE, 11504 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 11502 },
	{ 0x95b7, 0x95b7, PDF_CMAP_SINGLE, 11949 },
	{ 0x95b9, 0x95ba, PDF_CMAP_RANGE, 11500 },
	{ 0x95bb, 0x95bb, PDF_CMAP_SINGLE, 4997 },
	{ 0x95bc, 0x95bc, PDF_CMAP_SINGLE, 11497 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 11505 },
	{ 0x95be, 0x95be, PDF_CMAP_SINGLE, 11499 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 11503 },
	{ 0x95c0, 0x95c0, PDF_CMAP_SINGLE, 11945 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 11947 },
	{ 0x95c5, 0x95c5, PDF_CMAP_SINGLE, 11948 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 5294 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 11944 },
	{ 0x95c8, 0x95c8, PDF_CMAP_SINGLE, 5293 },
	{ 0x95c9, 0x95c9, PDF_CMAP_SINGLE, 11946 },
	{ 0x95ca, 0x95cc, PDF_CMAP_RANGE, 5290 },
	{ 0x95cd, 0x95cd, PDF_CMAP_SINGLE, 11498 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 5452 },
	{ 0x95d1, 0x95d1, PDF_CMAP_SINGLE, 12319 },
	{ 0x95d2, 0x95d3, PDF_CMAP_RANGE, 12317 },
	{ 0x95d4, 0x95d4, PDF_CMAP_SINGLE, 5450 },
	{ 0x95d5, 0x95d5, PDF_CMAP_SINGLE, 5453 },
	{ 0x95d6, 0x95d6, PDF_CMAP_SINGLE, 5451 },
	{ 0x95da, 0x95db, PDF_CMAP_RANGE, 12643 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 5611 },
	{ 0x95de, 0x95de, PDF_CMAP_SINGLE, 12908 },
	{ 0x95df, 0x95df, PDF_CMAP_SINGLE, 12910 },
	{ 0x95e0, 0x95e0, PDF_CMAP_SINGLE, 12909 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 5717 },
	{ 0x95e2, 0x95e2, PDF_CMAP_SINGLE, 5792 },
	{ 0x95e3, 0x95e3, PDF_CMAP_SINGLE, 13104 },
	{ 0x95e4, 0x95e4, PDF_CMAP_SINGLE, 13103 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 13102 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 17674 },
	{ 0x95f4, 0x95f4, PDF_CMAP_SINGLE, 18351 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 1689 },
	{ 0x961d, 0x961d, PDF_CMAP_SINGLE, 17689 },
	{ 0x961e, 0x961e, PDF_CMAP_SINGLE, 6068 },
	{ 0x9620, 0x9620, PDF_CMAP_SINGLE, 6177 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 1044 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 6175 },
	{ 0x9623, 0x9623, PDF_CMAP_SINGLE, 6178 },
	{ 0x9624, 0x9624, PDF_CMAP_SINGLE, 6176 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 6385 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 1318 },
	{ 0x962c, 0x962c, PDF_CMAP_SINGLE, 1319 },
	{ 0x962d, 0x962d, PDF_CMAP_SINGLE, 6387 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 1316 },
	{ 0x962f, 0x962f, PDF_CMAP_SINGLE, 6386 },
	{ 0x9630, 0x9630, PDF_CMAP_SINGLE, 6384 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 1317 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 1315 },
	{ 0x9633, 0x9633, PDF_CMAP_SINGLE, 18352 },
	{ 0x9638, 0x9638, PDF_CMAP_SINGLE, 16337 },
	{ 0x9639, 0x9639, PDF_CMAP_SINGLE, 6741 },
	{ 0x963a, 0x963a, PDF_CMAP_SINGLE, 6744 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 1692 },
	{ 0x963c, 0x963c, PDF_CMAP_SINGLE, 6743 },
	{ 0x963d, 0x963d, PDF_CMAP_SINGLE, 6742 },
	{ 0x963f, 0x963f, PDF_CMAP_SINGLE, 1691 },
	{ 0x9640, 0x9640, PDF_CMAP_SINGLE, 1690 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 16189 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 1694 },
	{ 0x9643, 0x9643, PDF_CMAP_SINGLE, 6745 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 1693 },
	{ 0x9645, 0x9645, PDF_CMAP_SINGLE, 18290 },
	{ 0x964a, 0x964a, PDF_CMAP_SINGLE, 7194 },
	{ 0x964b, 0x964d, PDF_CMAP_RANGE, 2083 },
	{ 0x964e, 0x964e, PDF_CMAP_SINGLE, 7195 },
	{ 0x964f, 0x964f, PDF_CMAP_SINGLE, 7191 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 2082 },
	{ 0x9651, 0x9651, PDF_CMAP_SINGLE, 7192 },
	{ 0x9653, 0x9653, PDF_CMAP_SINGLE, 7193 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 7190 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 15847 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 2545 },
	{ 0x965b, 0x965b, PDF_CMAP_SINGLE, 2542 },
	{ 0x965c, 0x965c, PDF_CMAP_SINGLE, 7727 },
	{ 0x965d, 0x965d, PDF_CMAP_SINGLE, 2543 },
	{ 0x965e, 0x965e, PDF_CMAP_SINGLE, 2546 },
	{ 0x965f, 0x965f, PDF_CMAP_SINGLE, 7728 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 2541 },
	{ 0x9662, 0x9663, PDF_CMAP_RANGE, 2539 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 2544 },
	{ 0x9669, 0x9669, PDF_CMAP_SINGLE, 18270 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 3025 },
	{ 0x966b, 0x966b, PDF_CMAP_SINGLE, 8358 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 3033 },
	{ 0x966d, 0x966d, PDF_CMAP_SINGLE, 8357 },
	{ 0x966f, 0x966f, PDF_CMAP_SINGLE, 8360 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 3029 },
	{ 0x9671, 0x9671, PDF_CMAP_SINGLE, 8359 },
	{ 0x9672, 0x9672, PDF_CMAP_SINGLE, 3494 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 3027 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 3030 },
	{ 0x9675, 0x9675, PDF_CMAP_SINGLE, 3026 },
	{ 0x9676, 0x9677, PDF_CMAP_RANGE, 3031 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 3028 },
	{ 0x967b, 0x967b, PDF_CMAP_SINGLE, 14508 },
	{ 0x967c, 0x967c, PDF_CMAP_SINGLE, 8356 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 3490 },
	{ 0x967e, 0x967e, PDF_CMAP_SINGLE, 9039 },
	{ 0x9680, 0x9680, PDF_CMAP_SINGLE, 9043 },
	{ 0x9681, 0x9681, PDF_CMAP_SINGLE, 16198 },
	{ 0x9683, 0x9683, PDF_CMAP_SINGLE, 9042 },
	{ 0x9684, 0x9684, PDF_CMAP_SINGLE, 3495 },
	{ 0x9685, 0x9686, PDF_CMAP_RANGE, 3491 },
	{ 0x9687, 0x9687, PDF_CMAP_SINGLE, 9038 },
	{ 0x9688, 0x9689, PDF_CMAP_RANGE, 9040 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 3487 },
	{ 0x968b, 0x968b, PDF_CMAP_SINGLE, 3489 },
	{ 0x968d, 0x968d, PDF_CMAP_SINGLE, 3493 },
	{ 0x968e, 0x968e, PDF_CMAP_SINGLE, 3488 },
	{ 0x968f, 0x968f, PDF_CMAP_SINGLE, 15326 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 9718 },
	{ 0x9692, 0x9693, PDF_CMAP_RANGE, 9716 },
	{ 0x9694, 0x9695, PDF_CMAP_RANGE, 3934 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 14509 },
	{ 0x9697, 0x9697, PDF_CMAP_SINGLE, 9719 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 3933 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 4325 },
	{ 0x969b, 0x969b, PDF_CMAP_SINGLE, 4327 },
	{ 0x969c, 0x969c, PDF_CMAP_SINGLE, 4326 },
	{ 0x969e, 0x969e, PDF_CMAP_SINGLE, 10322 },
	{ 0x96a1, 0x96a1, PDF_CMAP_SINGLE, 10323 },
	{ 0x96a2, 0x96a2, PDF_CMAP_SINGLE, 10935 },
	{ 0x96a3, 0x96a3, PDF_CMAP_SINGLE, 14511 },
	{ 0x96a4, 0x96a4, PDF_CMAP_SINGLE, 10934 },
	{ 0x96a5, 0x96a5, PDF_CMAP_SINGLE, 17135 },
	{ 0x96a7, 0x96a8, PDF_CMAP_RANGE, 4998 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 11506 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 5000 },
	{ 0x96ac, 0x96ac, PDF_CMAP_SINGLE, 11952 },
	{ 0x96ae, 0x96ae, PDF_CMAP_SINGLE, 11950 },
	{ 0x96b0, 0x96b0, PDF_CMAP_SINGLE, 11951 },
	{ 0x96b1, 0x96b1, PDF_CMAP_SINGLE, 5295 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 12320 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 5612 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 561 },
	{ 0x96b8, 0x96b8, PDF_CMAP_SINGLE, 5296 },
	{ 0x96b9, 0x96b9, PDF_CMAP_SINGLE, 1695 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 2547 },
	{ 0x96bc, 0x96bc, PDF_CMAP_SINGLE, 7729 },
	{ 0x96bd, 0x96bd, PDF_CMAP_SINGLE, 14517 },
	{ 0x96bf, 0x96bf, PDF_CMAP_SINGLE, 8361 },
	{ 0x96c0, 0x96c0, PDF_CMAP_SINGLE, 3034 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 3496 },
	{ 0x96c2, 0x96c2, PDF_CMAP_SINGLE, 9044 },
	{ 0x96c3, 0x96c3, PDF_CMAP_SINGLE, 9046 },
	{ 0x96c4, 0x96c4, PDF_CMAP_SINGLE, 3498 },
	{ 0x96c5, 0x96c5, PDF_CMAP_SINGLE, 3497 },
	{ 0x96c6, 0x96c7, PDF_CMAP_RANGE, 3499 },
	{ 0x96c8, 0x96c8, PDF_CMAP_SINGLE, 9045 },
	{ 0x96c9, 0x96ca, PDF_CMAP_RANGE, 3938 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 3937 },
	{ 0x96cc, 0x96cc, PDF_CMAP_SINGLE, 4328 },
	{ 0x96cd, 0x96cd, PDF_CMAP_SINGLE, 3936 },
	{ 0x96ce, 0x96ce, PDF_CMAP_SINGLE, 9720 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 4329 },
	{ 0x96d3, 0x96d3, PDF_CMAP_SINGLE, 10936 },
	{ 0x96d4, 0x96d4, PDF_CMAP_SINGLE, 11507 },
	{ 0x96d5, 0x96d5, PDF_CMAP_SINGLE, 5001 },
	{ 0x96d6, 0x96d6, PDF_CMAP_SINGLE, 5297 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 12321 },
	{ 0x96d8, 0x96d8, PDF_CMAP_SINGLE, 12325 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 5456 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 12322 },
	{ 0x96db, 0x96db, PDF_CMAP_SINGLE, 5457 },
	{ 0x96dc, 0x96dc, PDF_CMAP_SINGLE, 5455 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 12326 },
	{ 0x96de, 0x96de, PDF_CMAP_SINGLE, 5458 },
	{ 0x96df, 0x96df, PDF_CMAP_SINGLE, 12324 },
	{ 0x96e1, 0x96e1, PDF_CMAP_SINGLE, 12645 },
	{ 0x96e2, 0x96e2, PDF_CMAP_SINGLE, 5454 },
	{ 0x96e3, 0x96e3, PDF_CMAP_SINGLE, 5613 },
	{ 0x96e5, 0x96e5, PDF_CMAP_SINGLE, 13480 },
	{ 0x96e8, 0x96e8, PDF_CMAP_SINGLE, 1696 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 3036 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 3035 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 3501 },
	{ 0x96f0, 0x96f0, PDF_CMAP_SINGLE, 9048 },
	{ 0x96f1, 0x96f1, PDF_CMAP_SINGLE, 9047 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 3502 },
	{ 0x96f4, 0x96f4, PDF_CMAP_SINGLE, 14852 },
	{ 0x96f5, 0x96f5, PDF_CMAP_SINGLE, 9724 },
	{ 0x96f6, 0x96f6, PDF_CMAP_SINGLE, 3943 },
	{ 0x96f7, 0x96f7, PDF_CMAP_SINGLE, 3940 },
	{ 0x96f8, 0x96f8, PDF_CMAP_SINGLE, 9723 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 3942 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 9721 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 3941 },
	{ 0x96fd, 0x96fd, PDF_CMAP_SINGLE, 9722 },
	{ 0x96ff, 0x96ff, PDF_CMAP_SINGLE, 10324 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 4330 },
	{ 0x9702, 0x9702, PDF_CMAP_SINGLE, 10939 },
	{ 0x9703, 0x9703, PDF_CMAP_SINGLE, 16141 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 4706 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 10937 },
	{ 0x9706, 0x9707, PDF_CMAP_RANGE, 4707 },
	{ 0x9708, 0x9708, PDF_CMAP_SINGLE, 10938 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 4709 },
	{ 0x970b, 0x970b, PDF_CMAP_SINGLE, 11508 },
	{ 0x970d, 0x970d, PDF_CMAP_SINGLE, 5005 },
	{ 0x970e, 0x970e, PDF_CMAP_SINGLE, 5002 },
	{ 0x970f, 0x970f, PDF_CMAP_SINGLE, 5007 },
	{ 0x9710, 0x9710, PDF_CMAP_SINGLE, 11510 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 5003 },
	{ 0x9712, 0x9712, PDF_CMAP_SINGLE, 11509 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 5006 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 5004 },
	{ 0x9718, 0x9718, PDF_CMAP_SINGLE, 11955 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 11957 },
	{ 0x971b, 0x971b, PDF_CMAP_SINGLE, 14537 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 5298 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 11956 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 5299 },
	{ 0x971f, 0x971f, PDF_CMAP_SINGLE, 11954 },
	{ 0x9720, 0x9720, PDF_CMAP_SINGLE, 11953 },
	{ 0x9721, 0x9721, PDF_CMAP_SINGLE, 16307 },
	{ 0x9722, 0x9722, PDF_CMAP_SINGLE, 12328 },
	{ 0x9723, 0x9723, PDF_CMAP_SINGLE, 12327 },
	{ 0x9724, 0x9724, PDF_CMAP_SINGLE, 5459 },
	{ 0x9725, 0x9725, PDF_CMAP_SINGLE, 12329 },
	{ 0x9726, 0x9726, PDF_CMAP_SINGLE, 12650 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 5615 },
	{ 0x9728, 0x9728, PDF_CMAP_SINGLE, 12649 },
	{ 0x9729, 0x9729, PDF_CMAP_SINGLE, 12646 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 5614 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 12647 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 12911 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 5718 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 14528 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 5795 },
	{ 0x9735, 0x9735, PDF_CMAP_SINGLE, 13105 },
	{ 0x9736, 0x9736, PDF_CMAP_SINGLE, 14530 },
	{ 0x9738, 0x9739, PDF_CMAP_RANGE, 5793 },
	{ 0x973a, 0x973a, PDF_CMAP_SINGLE, 13106 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 5858 },
	{ 0x973f, 0x973f, PDF_CMAP_SINGLE, 13265 },
	{ 0x9740, 0x9740, PDF_CMAP_SINGLE, 18355 },
	{ 0x9741, 0x9741, PDF_CMAP_SINGLE, 15337 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 5938 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 13482 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 5940 },
	{ 0x9746, 0x9746, PDF_CMAP_SINGLE, 13481 },
	{ 0x9747, 0x9747, PDF_CMAP_SINGLE, 13483 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 5939 },
	{ 0x9749, 0x9749, PDF_CMAP_SINGLE, 13553 },
	{ 0x974b, 0x974b, PDF_CMAP_SINGLE, 13606 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 17635 },
	{ 0x9752, 0x9752, PDF_CMAP_SINGLE, 1697 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 3944 },
	{ 0x9757, 0x9757, PDF_CMAP_SINGLE, 14540 },
	{ 0x9758, 0x9758, PDF_CMAP_SINGLE, 10325 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 10940 },
	{ 0x975b, 0x975c, PDF_CMAP_RANGE, 5008 },
	{ 0x975d, 0x975d, PDF_CMAP_SINGLE, 15343 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 1698 },
	{ 0x975f, 0x975f, PDF_CMAP_SINGLE, 14543 },
	{ 0x9760, 0x9760, PDF_CMAP_SINGLE, 4710 },
	{ 0x9761, 0x9761, PDF_CMAP_SINGLE, 5616 },
	{ 0x9762, 0x9762, PDF_CMAP_SINGLE, 2086 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 5010 },
	{ 0x9768, 0x9768, PDF_CMAP_SINGLE, 5904 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 2087 },
	{ 0x976a, 0x976a, PDF_CMAP_SINGLE, 8362 },
	{ 0x976c, 0x976c, PDF_CMAP_SINGLE, 9049 },
	{ 0x976d, 0x976d, PDF_CMAP_SINGLE, 16023 },
	{ 0x976e, 0x976e, PDF_CMAP_SINGLE, 9051 },
	{ 0x9770, 0x9770, PDF_CMAP_SINGLE, 9050 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 15356 },
	{ 0x9772, 0x9772, PDF_CMAP_SINGLE, 9728 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 9725 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 3945 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 3946 },
	{ 0x9777, 0x9778, PDF_CMAP_RANGE, 9726 },
	{ 0x977a, 0x977a, PDF_CMAP_SINGLE, 10327 },
	{ 0x977b, 0x977b, PDF_CMAP_SINGLE, 10332 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 4331 },
	{ 0x977d, 0x977d, PDF_CMAP_SINGLE, 10326 },
	{ 0x977e, 0x977e, PDF_CMAP_SINGLE, 10328 },
	{ 0x977f, 0x977f, PDF_CMAP_SINGLE, 10335 },
	{ 0x9780, 0x9780, PDF_CMAP_SINGLE, 10330 },
	{ 0x9781, 0x9781, PDF_CMAP_SINGLE, 10334 },
	{ 0x9782, 0x9782, PDF_CMAP_SINGLE, 10331 },
	{ 0x9783, 0x9783, PDF_CMAP_SINGLE, 10329 },
	{ 0x9784, 0x9784, PDF_CMAP_SINGLE, 10333 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 4332 },
	{ 0x9787, 0x9787, PDF_CMAP_SINGLE, 16576 },
	{ 0x9788, 0x9788, PDF_CMAP_SINGLE, 10943 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 14548 },
	{ 0x978a, 0x978a, PDF_CMAP_SINGLE, 10941 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 4712 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 4711 },
	{ 0x978e, 0x978e, PDF_CMAP_SINGLE, 10942 },
	{ 0x978f, 0x978f, PDF_CMAP_SINGLE, 4713 },
	{ 0x9794, 0x9794, PDF_CMAP_SINGLE, 11513 },
	{ 0x9797, 0x9797, PDF_CMAP_SINGLE, 11512 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 5011 },
	{ 0x9799, 0x9799, PDF_CMAP_SINGLE, 11511 },
	{ 0x979a, 0x979a, PDF_CMAP_SINGLE, 11958 },
	{ 0x979b, 0x979b, PDF_CMAP_SINGLE, 15355 },
	{ 0x979c, 0x979c, PDF_CMAP_SINGLE, 11960 },
	{ 0x979d, 0x979d, PDF_CMAP_SINGLE, 11962 },
	{ 0x979e, 0x979e, PDF_CMAP_SINGLE, 11961 },
	{ 0x979f, 0x979f, PDF_CMAP_SINGLE, 14549 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 5300 },
	{ 0x97a1, 0x97a1, PDF_CMAP_SINGLE, 11959 },
	{ 0x97a2, 0x97a2, PDF_CMAP_SINGLE, 12336 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 5460 },
	{ 0x97a4, 0x97a4, PDF_CMAP_SINGLE, 12334 },
	{ 0x97a5, 0x97a5, PDF_CMAP_SINGLE, 12337 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 5461 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 12332 },
	{ 0x97aa, 0x97aa, PDF_CMAP_SINGLE, 12335 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 12333 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 12330 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 5462 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 12331 },
	{ 0x97b1, 0x97b1, PDF_CMAP_SINGLE, 14550 },
	{ 0x97b2, 0x97b2, PDF_CMAP_SINGLE, 18357 },
	{ 0x97b3, 0x97b3, PDF_CMAP_SINGLE, 12651 },
	{ 0x97b4, 0x97b4, PDF_CMAP_SINGLE, 16179 },
	{ 0x97b6, 0x97b6, PDF_CMAP_SINGLE, 12653 },
	{ 0x97b7, 0x97b7, PDF_CMAP_SINGLE, 12652 },
	{ 0x97b8, 0x97b8, PDF_CMAP_SINGLE, 15765 },
	{ 0x97b9, 0x97b9, PDF_CMAP_SINGLE, 12913 },
	{ 0x97ba, 0x97ba, PDF_CMAP_SINGLE, 16181 },
	{ 0x97bb, 0x97bb, PDF_CMAP_SINGLE, 12914 },
	{ 0x97bd, 0x97bd, PDF_CMAP_SINGLE, 18766 },
	{ 0x97be, 0x97be, PDF_CMAP_SINGLE, 14551 },
	{ 0x97bf, 0x97bf, PDF_CMAP_SINGLE, 13107 },
	{ 0x97c0, 0x97c0, PDF_CMAP_SINGLE, 14552 },
	{ 0x97c1, 0x97c1, PDF_CMAP_SINGLE, 5861 },
	{ 0x97c2, 0x97c2, PDF_CMAP_SINGLE, 18358 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 5860 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 13388 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 5941 },
	{ 0x97c7, 0x97c7, PDF_CMAP_SINGLE, 13484 },
	{ 0x97c8, 0x97c8, PDF_CMAP_SINGLE, 16175 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 5974 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 2088 },
	{ 0x97cc, 0x97cc, PDF_CMAP_SINGLE, 3503 },
	{ 0x97cd, 0x97cd, PDF_CMAP_SINGLE, 10337 },
	{ 0x97ce, 0x97ce, PDF_CMAP_SINGLE, 10336 },
	{ 0x97cf, 0x97cf, PDF_CMAP_SINGLE, 10945 },
	{ 0x97d0, 0x97d0, PDF_CMAP_SINGLE, 10944 },
	{ 0x97d2, 0x97d2, PDF_CMAP_SINGLE, 14553 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 5301 },
	{ 0x97d4, 0x97d4, PDF_CMAP_SINGLE, 11964 },
	{ 0x97d5, 0x97d5, PDF_CMAP_SINGLE, 11963 },
	{ 0x97d6, 0x97d6, PDF_CMAP_SINGLE, 12340 },
	{ 0x97d7, 0x97d7, PDF_CMAP_SINGLE, 12338 },
	{ 0x97d8, 0x97d8, PDF_CMAP_SINGLE, 12341 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 12339 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 5617 },
	{ 0x97dd, 0x97df, PDF_CMAP_RANGE, 12654 },
	{ 0x97e0, 0x97e0, PDF_CMAP_SINGLE, 14554 },
	{ 0x97e1, 0x97e1, PDF_CMAP_SINGLE, 13108 },
	{ 0x97e3, 0x97e3, PDF_CMAP_SINGLE, 13266 },
	{ 0x97e5, 0x97e5, PDF_CMAP_SINGLE, 13485 },
	{ 0x97e6, 0x97e6, PDF_CMAP_SINGLE, 17676 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 2089 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 14556 },
	{ 0x97f0, 0x97f0, PDF_CMAP_SINGLE, 11514 },
	{ 0x97f1, 0x97f1, PDF_CMAP_SINGLE, 11965 },
	{ 0x97f2, 0x97f2, PDF_CMAP_SINGLE, 14669 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 2090 },
	{ 0x97f5, 0x97f5, PDF_CMAP_SINGLE, 14560 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 4333 },
	{ 0x97f8, 0x97f8, PDF_CMAP_SINGLE, 11515 },
	{ 0x97f9, 0x97f9, PDF_CMAP_SINGLE, 5463 },
	{ 0x97fa, 0x97fa, PDF_CMAP_SINGLE, 12342 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 5618 },
	{ 0x97fd, 0x97fe, PDF_CMAP_RANGE, 12915 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 5796 },
	{ 0x9800, 0x9800, PDF_CMAP_SINGLE, 13390 },
	{ 0x9801, 0x9801, PDF_CMAP_SINGLE, 2091 },
	{ 0x9802, 0x9803, PDF_CMAP_RANGE, 3039 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 8363 },
	{ 0x9805, 0x9806, PDF_CMAP_RANGE, 3504 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 9052 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 3506 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 3950 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 3952 },
	{ 0x980d, 0x980e, PDF_CMAP_RANGE, 9730 },
	{ 0x980f, 0x980f, PDF_CMAP_SINGLE, 9729 },
	{ 0x9810, 0x9811, PDF_CMAP_RANGE, 3947 },
	{ 0x9812, 0x9812, PDF_CMAP_SINGLE, 3951 },
	{ 0x9813, 0x9813, PDF_CMAP_SINGLE, 3949 },
	{ 0x9814, 0x9814, PDF_CMAP_SINGLE, 18770 },
	{ 0x9815, 0x9815, PDF_CMAP_SINGLE, 17212 },
	{ 0x9816, 0x9816, PDF_CMAP_SINGLE, 10338 },
	{ 0x9817, 0x9818, PDF_CMAP_RANGE, 4334 },
	{ 0x981b, 0x981b, PDF_CMAP_SINGLE, 10952 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 4716 },
	{ 0x981d, 0x981d, PDF_CMAP_SINGLE, 10947 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 10946 },
	{ 0x981f, 0x981f, PDF_CMAP_SINGLE, 15362 },
	{ 0x9820, 0x9820, PDF_CMAP_SINGLE, 10951 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 4714 },
	{ 0x9823, 0x9823, PDF_CMAP_SINGLE, 15927 },
	{ 0x9824, 0x9824, PDF_CMAP_SINGLE, 5018 },
	{ 0x9826, 0x9826, PDF_CMAP_SINGLE, 10948 },
	{ 0x9827, 0x9827, PDF_CMAP_SINGLE, 10953 },
	{ 0x9828, 0x9828, PDF_CMAP_SINGLE, 10950 },
	{ 0x9829, 0x9829, PDF_CMAP_SINGLE, 10949 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 4715 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 5016 },
	{ 0x982e, 0x982e, PDF_CMAP_SINGLE, 16242 },
	{ 0x982f, 0x982f, PDF_CMAP_SINGLE, 11517 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 5012 },
	{ 0x9832, 0x9832, PDF_CMAP_SINGLE, 11518 },
	{ 0x9833, 0x9833, PDF_CMAP_SINGLE, 14565 },
	{ 0x9834, 0x9834, PDF_CMAP_SINGLE, 14564 },
	{ 0x9835, 0x9835, PDF_CMAP_SINGLE, 11516 },
	{ 0x9837, 0x9837, PDF_CMAP_SINGLE, 5015 },
	{ 0x9838, 0x9838, PDF_CMAP_SINGLE, 5013 },
	{ 0x9839, 0x9839, PDF_CMAP_SINGLE, 5017 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 5014 },
	{ 0x9841, 0x9841, PDF_CMAP_SINGLE, 11966 },
	{ 0x9843, 0x9843, PDF_CMAP_SINGLE, 11971 },
	{ 0x9844, 0x9844, PDF_CMAP_SINGLE, 11967 },
	{ 0x9845, 0x9845, PDF_CMAP_SINGLE, 11970 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 5302 },
	{ 0x9847, 0x9847, PDF_CMAP_SINGLE, 15766 },
	{ 0x9848, 0x9848, PDF_CMAP_SINGLE, 11762 },
	{ 0x9849, 0x9849, PDF_CMAP_SINGLE, 11969 },
	{ 0x984a, 0x984a, PDF_CMAP_SINGLE, 11968 },
	{ 0x984b, 0x984b, PDF_CMAP_SINGLE, 14566 },
	{ 0x984c, 0x984c, PDF_CMAP_SINGLE, 5466 },
	{ 0x984d, 0x984d, PDF_CMAP_SINGLE, 5464 },
	{ 0x984e, 0x984e, PDF_CMAP_SINGLE, 5467 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 5465 },
	{ 0x9850, 0x9852, PDF_CMAP_RANGE, 12343 },
	{ 0x9853, 0x9853, PDF_CMAP_SINGLE, 5468 },
	{ 0x9857, 0x9857, PDF_CMAP_SINGLE, 12660 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 5620 },
	{ 0x9859, 0x9859, PDF_CMAP_SINGLE, 12658 },
	{ 0x985b, 0x985b, PDF_CMAP_SINGLE, 5621 },
	{ 0x985c, 0x985c, PDF_CMAP_SINGLE, 12657 },
	{ 0x985d, 0x985d, PDF_CMAP_SINGLE, 12659 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 5619 },
	{ 0x985f, 0x985f, PDF_CMAP_SINGLE, 12920 },
	{ 0x9860, 0x9860, PDF_CMAP_SINGLE, 12917 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 12918 },
	{ 0x9864, 0x9864, PDF_CMAP_SINGLE, 13109 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 5798 },
	{ 0x9866, 0x9866, PDF_CMAP_SINGLE, 14567 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 5797 },
	{ 0x9869, 0x9869, PDF_CMAP_SINGLE, 13268 },
	{ 0x986a, 0x986a, PDF_CMAP_SINGLE, 13267 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 5862 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 16437 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 5905 },
	{ 0x9870, 0x9870, PDF_CMAP_SINGLE, 5942 },
	{ 0x9871, 0x9871, PDF_CMAP_SINGLE, 5965 },
	{ 0x9872, 0x9872, PDF_CMAP_SINGLE, 13554 },
	{ 0x9873, 0x9874, PDF_CMAP_RANGE, 13607 },
	{ 0x9875, 0x9875, PDF_CMAP_SINGLE, 17677 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 2092 },
	{ 0x98a9, 0x98a9, PDF_CMAP_SINGLE, 9053 },
	{ 0x98ac, 0x98ac, PDF_CMAP_SINGLE, 9732 },
	{ 0x98ad, 0x98ae, PDF_CMAP_RANGE, 10339 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 4336 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 4337 },
	{ 0x98b2, 0x98b2, PDF_CMAP_SINGLE, 10954 },
	{ 0x98b3, 0x98b3, PDF_CMAP_SINGLE, 4717 },
	{ 0x98b4, 0x98b4, PDF_CMAP_SINGLE, 16243 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 5303 },
	{ 0x98b7, 0x98b7, PDF_CMAP_SINGLE, 14575 },
	{ 0x98b8, 0x98b8, PDF_CMAP_SINGLE, 12346 },
	{ 0x98b9, 0x98b9, PDF_CMAP_SINGLE, 18360 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 5469 },
	{ 0x98bb, 0x98bb, PDF_CMAP_SINGLE, 12663 },
	{ 0x98bc, 0x98bc, PDF_CMAP_SINGLE, 5622 },
	{ 0x98bd, 0x98bd, PDF_CMAP_SINGLE, 12662 },
	{ 0x98be, 0x98be, PDF_CMAP_SINGLE, 12664 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 12661 },
	{ 0x98c0, 0x98c0, PDF_CMAP_SINGLE, 13112 },
	{ 0x98c1, 0x98c2, PDF_CMAP_RANGE, 12921 },
	{ 0x98c3, 0x98c3, PDF_CMAP_SINGLE, 15224 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 5719 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 13111 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 14577 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 14576 },
	{ 0x98c9, 0x98c9, PDF_CMAP_SINGLE, 13110 },
	{ 0x98ca, 0x98ca, PDF_CMAP_SINGLE, 14574 },
	{ 0x98cb, 0x98cb, PDF_CMAP_SINGLE, 13269 },
	{ 0x98cc, 0x98cc, PDF_CMAP_SINGLE, 13609 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 17678 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 2093 },
	{ 0x98dc, 0x98dc, PDF_CMAP_SINGLE, 15371 },
	{ 0x98de, 0x98de, PDF_CMAP_SINGLE, 17679 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 2094 },
	{ 0x98e0, 0x98e0, PDF_CMAP_SINGLE, 18322 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 14582 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 2548 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 7730 },
	{ 0x98e5, 0x98e5, PDF_CMAP_SINGLE, 8364 },
	{ 0x98e6, 0x98e6, PDF_CMAP_SINGLE, 14583 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 3507 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 3510 },
	{ 0x98ea, 0x98ea, PDF_CMAP_SINGLE, 3508 },
	{ 0x98eb, 0x98eb, PDF_CMAP_SINGLE, 9054 },
	{ 0x98ec, 0x98ec, PDF_CMAP_SINGLE, 14584 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 3512 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 3509 },
	{ 0x98f1, 0x98f1, PDF_CMAP_SINGLE, 18362 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 3511 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 3954 },
	{ 0x98f5, 0x98f5, PDF_CMAP_SINGLE, 17253 },
	{ 0x98f6, 0x98f6, PDF_CMAP_SINGLE, 9733 },
	{ 0x98f9, 0x98f9, PDF_CMAP_SINGLE, 9734 },
	{ 0x98fa, 0x98fa, PDF_CMAP_SINGLE, 10956 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 3953 },
	{ 0x98fd, 0x98fe, PDF_CMAP_RANGE, 3955 },
	{ 0x9900, 0x9900, PDF_CMAP_SINGLE, 10342 },
	{ 0x9902, 0x9902, PDF_CMAP_SINGLE, 10341 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 4338 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 4339 },
	{ 0x9907, 0x9907, PDF_CMAP_SINGLE, 10343 },
	{ 0x9908, 0x9908, PDF_CMAP_SINGLE, 10955 },
	{ 0x9909, 0x9909, PDF_CMAP_SINGLE, 4341 },
	{ 0x990a, 0x990a, PDF_CMAP_SINGLE, 4718 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 4340 },
	{ 0x990e, 0x990e, PDF_CMAP_SINGLE, 17389 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 5019 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 10957 },
	{ 0x9912, 0x9912, PDF_CMAP_SINGLE, 4720 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 4719 },
	{ 0x9914, 0x9914, PDF_CMAP_SINGLE, 10958 },
	{ 0x9915, 0x9915, PDF_CMAP_SINGLE, 10961 },
	{ 0x9916, 0x9917, PDF_CMAP_RANGE, 10959 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 4721 },
	{ 0x9919, 0x9919, PDF_CMAP_SINGLE, 18364 },
	{ 0x991a, 0x991a, PDF_CMAP_SINGLE, 5024 },
	{ 0x991b, 0x991b, PDF_CMAP_SINGLE, 5022 },
	{ 0x991c, 0x991c, PDF_CMAP_SINGLE, 17390 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 5021 },
	{ 0x991f, 0x991f, PDF_CMAP_SINGLE, 11520 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 5023 },
	{ 0x9924, 0x9924, PDF_CMAP_SINGLE, 11519 },
	{ 0x9925, 0x9925, PDF_CMAP_SINGLE, 11972 },
	{ 0x9927, 0x9927, PDF_CMAP_SINGLE, 11521 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 5020 },
	{ 0x9929, 0x9929, PDF_CMAP_SINGLE, 11522 },
	{ 0x992a, 0x992a, PDF_CMAP_SINGLE, 11975 },
	{ 0x992b, 0x992c, PDF_CMAP_RANGE, 11973 },
	{ 0x992d, 0x992d, PDF_CMAP_SINGLE, 11979 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 5473 },
	{ 0x992f, 0x992f, PDF_CMAP_SINGLE, 11978 },
	{ 0x9930, 0x9930, PDF_CMAP_SINGLE, 11981 },
	{ 0x9931, 0x9931, PDF_CMAP_SINGLE, 11980 },
	{ 0x9932, 0x9932, PDF_CMAP_SINGLE, 11977 },
	{ 0x9933, 0x9933, PDF_CMAP_SINGLE, 11976 },
	{ 0x9935, 0x9935, PDF_CMAP_SINGLE, 5304 },
	{ 0x9937, 0x9937, PDF_CMAP_SINGLE, 18365 },
	{ 0x9938, 0x9938, PDF_CMAP_SINGLE, 15357 },
	{ 0x9939, 0x9939, PDF_CMAP_SINGLE, 14586 },
	{ 0x993a, 0x993a, PDF_CMAP_SINGLE, 12349 },
	{ 0x993b, 0x993b, PDF_CMAP_SINGLE, 15377 },
	{ 0x993c, 0x993c, PDF_CMAP_SINGLE, 12348 },
	{ 0x993d, 0x993d, PDF_CMAP_SINGLE, 5472 },
	{ 0x993e, 0x993f, PDF_CMAP_RANGE, 5470 },
	{ 0x9940, 0x9940, PDF_CMAP_SINGLE, 16180 },
	{ 0x9941, 0x9941, PDF_CMAP_SINGLE, 12347 },
	{ 0x9942, 0x9942, PDF_CMAP_SINGLE, 17407 },
	{ 0x9943, 0x9943, PDF_CMAP_SINGLE, 12667 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 5623 },
	{ 0x9947, 0x9947, PDF_CMAP_SINGLE, 12666 },
	{ 0x9948, 0x9948, PDF_CMAP_SINGLE, 12665 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 5624 },
	{ 0x994a, 0x994a, PDF_CMAP_SINGLE, 16313 },
	{ 0x994b, 0x994b, PDF_CMAP_SINGLE, 12927 },
	{ 0x994c, 0x994c, PDF_CMAP_SINGLE, 12926 },
	{ 0x994d, 0x994d, PDF_CMAP_SINGLE, 15378 },
	{ 0x994e, 0x994e, PDF_CMAP_SINGLE, 12924 },
	{ 0x9950, 0x9950, PDF_CMAP_SINGLE, 12923 },
	{ 0x9951, 0x9951, PDF_CMAP_SINGLE, 5721 },
	{ 0x9952, 0x9952, PDF_CMAP_SINGLE, 5720 },
	{ 0x9953, 0x9953, PDF_CMAP_SINGLE, 12928 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 13270 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 5863 },
	{ 0x9956, 0x9956, PDF_CMAP_SINGLE, 13114 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 5799 },
	{ 0x9958, 0x9958, PDF_CMAP_SINGLE, 13113 },
	{ 0x9959, 0x9959, PDF_CMAP_SINGLE, 12925 },
	{ 0x995b, 0x995b, PDF_CMAP_SINGLE, 13271 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 5906 },
	{ 0x995d, 0x995d, PDF_CMAP_SINGLE, 18366 },
	{ 0x995e, 0x995e, PDF_CMAP_SINGLE, 5966 },
	{ 0x995f, 0x995f, PDF_CMAP_SINGLE, 13555 },
	{ 0x9961, 0x9961, PDF_CMAP_SINGLE, 13610 },
	{ 0x9962, 0x9962, PDF_CMAP_SINGLE, 17391 },
	{ 0x9963, 0x9963, PDF_CMAP_SINGLE, 17680 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 2095 },
	{ 0x9997, 0x9997, PDF_CMAP_SINGLE, 8365 },
	{ 0x9998, 0x9998, PDF_CMAP_SINGLE, 11982 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 2096 },
	{ 0x999b, 0x999b, PDF_CMAP_SINGLE, 15381 },
	{ 0x999c, 0x999c, PDF_CMAP_SINGLE, 10345 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 10344 },
	{ 0x999e, 0x999e, PDF_CMAP_SINGLE, 11523 },
	{ 0x99a1, 0x99a1, PDF_CMAP_SINGLE, 11984 },
	{ 0x99a3, 0x99a3, PDF_CMAP_SINGLE, 11983 },
	{ 0x99a4, 0x99a4, PDF_CMAP_SINGLE, 16835 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 5474 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 12668 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5722 },
	{ 0x99aa, 0x99aa, PDF_CMAP_SINGLE, 15383 },
	{ 0x99ab, 0x99ab, PDF_CMAP_SINGLE, 13611 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 2549 },
	{ 0x99ad, 0x99ad, PDF_CMAP_SINGLE, 3514 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 3513 },
	{ 0x99af, 0x99af, PDF_CMAP_SINGLE, 9735 },
	{ 0x99b0, 0x99b0, PDF_CMAP_SINGLE, 9737 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 3958 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 9736 },
	{ 0x99b3, 0x99b3, PDF_CMAP_SINGLE, 3957 },
	{ 0x99b4, 0x99b4, PDF_CMAP_SINGLE, 3959 },
	{ 0x99b5, 0x99b5, PDF_CMAP_SINGLE, 9738 },
	{ 0x99b8, 0x99b8, PDF_CMAP_SINGLE, 14667 },
	{ 0x99b9, 0x99b9, PDF_CMAP_SINGLE, 10347 },
	{ 0x99ba, 0x99ba, PDF_CMAP_SINGLE, 10349 },
	{ 0x99bb, 0x99bb, PDF_CMAP_SINGLE, 10348 },
	{ 0x99bc, 0x99bc, PDF_CMAP_SINGLE, 15394 },
	{ 0x99bd, 0x99bd, PDF_CMAP_SINGLE, 10351 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 4342 },
	{ 0x99c2, 0x99c2, PDF_CMAP_SINGLE, 10350 },
	{ 0x99c3, 0x99c3, PDF_CMAP_SINGLE, 10346 },
	{ 0x99c4, 0x99c4, PDF_CMAP_SINGLE, 15735 },
	{ 0x99c5, 0x99c5, PDF_CMAP_SINGLE, 18369 },
	{ 0x99c7, 0x99c7, PDF_CMAP_SINGLE, 10352 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 10968 },
	{ 0x99cb, 0x99cb, PDF_CMAP_SINGLE, 10971 },
	{ 0x99cc, 0x99cc, PDF_CMAP_SINGLE, 10973 },
	{ 0x99cd, 0x99cd, PDF_CMAP_SINGLE, 10963 },
	{ 0x99ce, 0x99ce, PDF_CMAP_SINGLE, 10967 },
	{ 0x99cf, 0x99cf, PDF_CMAP_SINGLE, 10964 },
	{ 0x99d0, 0x99d0, PDF_CMAP_SINGLE, 4723 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 4726 },
	{ 0x99d2, 0x99d2, PDF_CMAP_SINGLE, 4728 },
	{ 0x99d3, 0x99d4, PDF_CMAP_RANGE, 10965 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 4727 },
	{ 0x99d6, 0x99d6, PDF_CMAP_SINGLE, 10969 },
	{ 0x99d7, 0x99d7, PDF_CMAP_SINGLE, 10972 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 10970 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 4729 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 16194 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 4725 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 10962 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 4722 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 4724 },
	{ 0x99e1, 0x99e1, PDF_CMAP_SINGLE, 14213 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 5026 },
	{ 0x99e3, 0x99e3, PDF_CMAP_SINGLE, 11529 },
	{ 0x99e4, 0x99e4, PDF_CMAP_SINGLE, 11527 },
	{ 0x99e5, 0x99e5, PDF_CMAP_SINGLE, 11526 },
	{ 0x99e6, 0x99e6, PDF_CMAP_SINGLE, 15205 },
	{ 0x99e7, 0x99e7, PDF_CMAP_SINGLE, 11532 },
	{ 0x99e9, 0x99e9, PDF_CMAP_SINGLE, 11531 },
	{ 0x99ea, 0x99ea, PDF_CMAP_SINGLE, 11530 },
	{ 0x99ec, 0x99ec, PDF_CMAP_SINGLE, 11525 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 5025 },
	{ 0x99ee, 0x99ee, PDF_CMAP_SINGLE, 11524 },
	{ 0x99f0, 0x99f0, PDF_CMAP_SINGLE, 11528 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 5027 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 11987 },
	{ 0x99f5, 0x99f5, PDF_CMAP_SINGLE, 14591 },
	{ 0x99f6, 0x99f6, PDF_CMAP_SINGLE, 11991 },
	{ 0x99f7, 0x99f7, PDF_CMAP_SINGLE, 11988 },
	{ 0x99f8, 0x99f8, PDF_CMAP_SINGLE, 11990 },
	{ 0x99f9, 0x99f9, PDF_CMAP_SINGLE, 11989 },
	{ 0x99fa, 0x99fa, PDF_CMAP_SINGLE, 11986 },
	{ 0x99fb, 0x99fb, PDF_CMAP_SINGLE, 11992 },
	{ 0x99fc, 0x99fc, PDF_CMAP_SINGLE, 11995 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 11993 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 5306 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 5305 },
	{ 0x9a02, 0x9a02, PDF_CMAP_SINGLE, 11985 },
	{ 0x9a03, 0x9a03, PDF_CMAP_SINGLE, 11996 },
	{ 0x9a04, 0x9a04, PDF_CMAP_SINGLE, 12354 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 12357 },
	{ 0x9a06, 0x9a06, PDF_CMAP_SINGLE, 12359 },
	{ 0x9a07, 0x9a07, PDF_CMAP_SINGLE, 12358 },
	{ 0x9a09, 0x9a09, PDF_CMAP_SINGLE, 12352 },
	{ 0x9a0a, 0x9a0a, PDF_CMAP_SINGLE, 12356 },
	{ 0x9a0b, 0x9a0b, PDF_CMAP_SINGLE, 12351 },
	{ 0x9a0c, 0x9a0c, PDF_CMAP_SINGLE, 14592 },
	{ 0x9a0d, 0x9a0d, PDF_CMAP_SINGLE, 12353 },
	{ 0x9a0e, 0x9a0e, PDF_CMAP_SINGLE, 5475 },
	{ 0x9a0f, 0x9a0f, PDF_CMAP_SINGLE, 12350 },
	{ 0x9a10, 0x9a10, PDF_CMAP_SINGLE, 14594 },
	{ 0x9a11, 0x9a11, PDF_CMAP_SINGLE, 12355 },
	{ 0x9a14, 0x9a14, PDF_CMAP_SINGLE, 12682 },
	{ 0x9a15, 0x9a15, PDF_CMAP_SINGLE, 12671 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 5625 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 5626 },
	{ 0x9a1a, 0x9a1a, PDF_CMAP_SINGLE, 12670 },
	{ 0x9a1b, 0x9a1b, PDF_CMAP_SINGLE, 12675 },
	{ 0x9a1c, 0x9a1c, PDF_CMAP_SINGLE, 12681 },
	{ 0x9a1d, 0x9a1d, PDF_CMAP_SINGLE, 12673 },
	{ 0x9a1e, 0x9a1e, PDF_CMAP_SINGLE, 12680 },
	{ 0x9a1f, 0x9a1f, PDF_CMAP_SINGLE, 15212 },
	{ 0x9a20, 0x9a20, PDF_CMAP_SINGLE, 12677 },
	{ 0x9a21, 0x9a21, PDF_CMAP_SINGLE, 15388 },
	{ 0x9a22, 0x9a22, PDF_CMAP_SINGLE, 12676 },
	{ 0x9a23, 0x9a23, PDF_CMAP_SINGLE, 12679 },
	{ 0x9a24, 0x9a24, PDF_CMAP_SINGLE, 12674 },
	{ 0x9a25, 0x9a25, PDF_CMAP_SINGLE, 12672 },
	{ 0x9a26, 0x9a26, PDF_CMAP_SINGLE, 15858 },
	{ 0x9a27, 0x9a27, PDF_CMAP_SINGLE, 12678 },
	{ 0x9a29, 0x9a29, PDF_CMAP_SINGLE, 12935 },
	{ 0x9a2a, 0x9a2a, PDF_CMAP_SINGLE, 12933 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 5723 },
	{ 0x9a2c, 0x9a2c, PDF_CMAP_SINGLE, 12932 },
	{ 0x9a2d, 0x9a2d, PDF_CMAP_SINGLE, 12938 },
	{ 0x9a2e, 0x9a2e, PDF_CMAP_SINGLE, 12936 },
	{ 0x9a2f, 0x9a2f, PDF_CMAP_SINGLE, 15390 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 5724 },
	{ 0x9a31, 0x9a31, PDF_CMAP_SINGLE, 12931 },
	{ 0x9a32, 0x9a32, PDF_CMAP_SINGLE, 12929 },
	{ 0x9a34, 0x9a34, PDF_CMAP_SINGLE, 12930 },
	{ 0x9a35, 0x9a35, PDF_CMAP_SINGLE, 5726 },
	{ 0x9a36, 0x9a36, PDF_CMAP_SINGLE, 12934 },
	{ 0x9a37, 0x9a37, PDF_CMAP_SINGLE, 5725 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 12937 },
	{ 0x9a39, 0x9a39, PDF_CMAP_SINGLE, 13115 },
	{ 0x9a3a, 0x9a3a, PDF_CMAP_SINGLE, 13121 },
	{ 0x9a3b, 0x9a3b, PDF_CMAP_SINGLE, 14593 },
	{ 0x9a3c, 0x9a3c, PDF_CMAP_SINGLE, 18371 },
	{ 0x9a3d, 0x9a3d, PDF_CMAP_SINGLE, 13116 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 5803 },
	{ 0x9a3f, 0x9a3f, PDF_CMAP_SINGLE, 13122 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 5802 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 13120 },
	{ 0x9a42, 0x9a42, PDF_CMAP_SINGLE, 13119 },
	{ 0x9a43, 0x9a43, PDF_CMAP_SINGLE, 5801 },
	{ 0x9a44, 0x9a44, PDF_CMAP_SINGLE, 13118 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 5800 },
	{ 0x9a46, 0x9a46, PDF_CMAP_SINGLE, 13117 },
	{ 0x9a48, 0x9a48, PDF_CMAP_SINGLE, 13277 },
	{ 0x9a49, 0x9a49, PDF_CMAP_SINGLE, 13279 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 13278 },
	{ 0x9a4c, 0x9a4c, PDF_CMAP_SINGLE, 13275 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 5865 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 13272 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 13276 },
	{ 0x9a50, 0x9a50, PDF_CMAP_SINGLE, 13281 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 13280 },
	{ 0x9a53, 0x9a54, PDF_CMAP_RANGE, 13273 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 5864 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 13391 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 5909 },
	{ 0x9a58, 0x9a58, PDF_CMAP_SINGLE, 14595 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 13392 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_RANGE, 5907 },
	{ 0x9a5c, 0x9a5c, PDF_CMAP_SINGLE, 15384 },
	{ 0x9a5e, 0x9a5e, PDF_CMAP_SINGLE, 13486 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 5943 },
	{ 0x9a60, 0x9a60, PDF_CMAP_SINGLE, 13584 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 5975 },
	{ 0x9a63, 0x9a63, PDF_CMAP_SINGLE, 15207 },
	{ 0x9a64, 0x9a64, PDF_CMAP_SINGLE, 13612 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 5976 },
	{ 0x9a66, 0x9a67, PDF_CMAP_RANGE, 13613 },
	{ 0x9a68, 0x9a68, PDF_CMAP_SINGLE, 13628 },
	{ 0x9a69, 0x9a69, PDF_CMAP_SINGLE, 13627 },
	{ 0x9a6a, 0x9a6a, PDF_CMAP_SINGLE, 5991 },
	{ 0x9a6b, 0x9a6b, PDF_CMAP_SINGLE, 13636 },
	{ 0x9a6c, 0x9a6c, PDF_CMAP_SINGLE, 17797 },
	{ 0x9a8f, 0x9a8f, PDF_CMAP_SINGLE, 17798 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2550 },
	{ 0x9aab, 0x9aab, PDF_CMAP_SINGLE, 9740 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 9739 },
	{ 0x9aaf, 0x9ab0, PDF_CMAP_RANGE, 4343 },
	{ 0x9ab1, 0x9ab1, PDF_CMAP_SINGLE, 10353 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 17202 },
	{ 0x9ab3, 0x9ab3, PDF_CMAP_SINGLE, 10974 },
	{ 0x9ab4, 0x9ab4, PDF_CMAP_SINGLE, 11535 },
	{ 0x9ab6, 0x9ab6, PDF_CMAP_SINGLE, 17392 },
	{ 0x9ab7, 0x9ab7, PDF_CMAP_SINGLE, 4730 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 5028 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 11533 },
	{ 0x9aba, 0x9aba, PDF_CMAP_SINGLE, 16244 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 11536 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 5029 },
	{ 0x9abd, 0x9abd, PDF_CMAP_SINGLE, 15767 },
	{ 0x9abe, 0x9abe, PDF_CMAP_SINGLE, 11997 },
	{ 0x9abf, 0x9abf, PDF_CMAP_SINGLE, 11534 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 12360 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 5476 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 12683 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 12941 },
	{ 0x9ac7, 0x9ac7, PDF_CMAP_SINGLE, 12939 },
	{ 0x9aca, 0x9aca, PDF_CMAP_SINGLE, 12940 },
	{ 0x9acd, 0x9acd, PDF_CMAP_SINGLE, 13123 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 5804 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 13282 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 5912 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 5866 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 5910 },
	{ 0x9ad5, 0x9ad5, PDF_CMAP_SINGLE, 13487 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 5967 },
	{ 0x9ad7, 0x9ad7, PDF_CMAP_SINGLE, 16298 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 2551 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 12361 },
	{ 0x9adf, 0x9adf, PDF_CMAP_SINGLE, 7731 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 14600 },
	{ 0x9ae1, 0x9ae1, PDF_CMAP_SINGLE, 3960 },
	{ 0x9ae2, 0x9ae2, PDF_CMAP_SINGLE, 14601 },
	{ 0x9ae3, 0x9ae3, PDF_CMAP_SINGLE, 10354 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 4345 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 10355 },
	{ 0x9aeb, 0x9aeb, PDF_CMAP_SINGLE, 10976 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 10975 },
	{ 0x9aed, 0x9aed, PDF_CMAP_SINGLE, 5031 },
	{ 0x9aee, 0x9aef, PDF_CMAP_RANGE, 4731 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 10979 },
	{ 0x9af2, 0x9af2, PDF_CMAP_SINGLE, 10978 },
	{ 0x9af3, 0x9af3, PDF_CMAP_SINGLE, 10977 },
	{ 0x9af4, 0x9af4, PDF_CMAP_SINGLE, 14603 },
	{ 0x9af6, 0x9af6, PDF_CMAP_SINGLE, 11537 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 11540 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 11539 },
	{ 0x9afa, 0x9afa, PDF_CMAP_SINGLE, 11538 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 5030 },
	{ 0x9afc, 0x9afc, PDF_CMAP_SINGLE, 12001 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 11999 },
	{ 0x9afe, 0x9afe, PDF_CMAP_SINGLE, 11998 },
	{ 0x9aff, 0x9aff, PDF_CMAP_SINGLE, 16151 },
	{ 0x9b01, 0x9b01, PDF_CMAP_SINGLE, 12000 },
	{ 0x9b02, 0x9b02, PDF_CMAP_SINGLE, 16146 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 5477 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 12363 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 5478 },
	{ 0x9b08, 0x9b08, PDF_CMAP_SINGLE, 12362 },
	{ 0x9b09, 0x9b09, PDF_CMAP_SINGLE, 16160 },
	{ 0x9b0a, 0x9b0a, PDF_CMAP_SINGLE, 12685 },
	{ 0x9b0b, 0x9b0b, PDF_CMAP_SINGLE, 12684 },
	{ 0x9b0c, 0x9b0c, PDF_CMAP_SINGLE, 12687 },
	{ 0x9b0d, 0x9b0d, PDF_CMAP_SINGLE, 5627 },
	{ 0x9b0e, 0x9b0e, PDF_CMAP_SINGLE, 12686 },
	{ 0x9b0f, 0x9b0f, PDF_CMAP_SINGLE, 18372 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 12942 },
	{ 0x9b11, 0x9b11, PDF_CMAP_SINGLE, 12944 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 12943 },
	{ 0x9b14, 0x9b14, PDF_CMAP_SINGLE, 14605 },
	{ 0x9b15, 0x9b15, PDF_CMAP_SINGLE, 13124 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 13127 },
	{ 0x9b17, 0x9b18, PDF_CMAP_RANGE, 13125 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 13283 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 5867 },
	{ 0x9b1e, 0x9b20, PDF_CMAP_RANGE, 13393 },
	{ 0x9b22, 0x9b22, PDF_CMAP_SINGLE, 5944 },
	{ 0x9b23, 0x9b23, PDF_CMAP_SINGLE, 5968 },
	{ 0x9b24, 0x9b24, PDF_CMAP_SINGLE, 13615 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 2552 },
	{ 0x9b27, 0x9b27, PDF_CMAP_SINGLE, 4733 },
	{ 0x9b28, 0x9b28, PDF_CMAP_SINGLE, 5032 },
	{ 0x9b29, 0x9b29, PDF_CMAP_SINGLE, 12365 },
	{ 0x9b2a, 0x9b2a, PDF_CMAP_SINGLE, 16153 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 13284 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 14606 },
	{ 0x9b2e, 0x9b2e, PDF_CMAP_SINGLE, 13585 },
	{ 0x9b2f, 0x9b2f, PDF_CMAP_SINGLE, 7732 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 5992 },
	{ 0x9b32, 0x9b32, PDF_CMAP_SINGLE, 2553 },
	{ 0x9b33, 0x9b33, PDF_CMAP_SINGLE, 11541 },
	{ 0x9b34, 0x9b34, PDF_CMAP_SINGLE, 14609 },
	{ 0x9b35, 0x9b35, PDF_CMAP_SINGLE, 12366 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 12688 },
	{ 0x9b39, 0x9b39, PDF_CMAP_SINGLE, 16128 },
	{ 0x9b3a, 0x9b3a, PDF_CMAP_SINGLE, 13128 },
	{ 0x9b3b, 0x9b3b, PDF_CMAP_SINGLE, 13285 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 2554 },
	{ 0x9b3e, 0x9b3f, PDF_CMAP_RANGE, 10356 },
	{ 0x9b40, 0x9b40, PDF_CMAP_SINGLE, 14613 },
	{ 0x9b41, 0x9b42, PDF_CMAP_RANGE, 4346 },
	{ 0x9b43, 0x9b43, PDF_CMAP_SINGLE, 10981 },
	{ 0x9b44, 0x9b44, PDF_CMAP_SINGLE, 4735 },
	{ 0x9b45, 0x9b45, PDF_CMAP_SINGLE, 4734 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 10980 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 12002 },
	{ 0x9b4a, 0x9b4a, PDF_CMAP_SINGLE, 12367 },
	{ 0x9b4b, 0x9b4b, PDF_CMAP_SINGLE, 12369 },
	{ 0x9b4c, 0x9b4c, PDF_CMAP_SINGLE, 12368 },
	{ 0x9b4d, 0x9b4d, PDF_CMAP_SINGLE, 5481 },
	{ 0x9b4e, 0x9b4e, PDF_CMAP_SINGLE, 5480 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 5479 },
	{ 0x9b50, 0x9b50, PDF_CMAP_SINGLE, 14612 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 5806 },
	{ 0x9b52, 0x9b52, PDF_CMAP_SINGLE, 13129 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 5805 },
	{ 0x9b55, 0x9b55, PDF_CMAP_SINGLE, 13287 },
	{ 0x9b56, 0x9b56, PDF_CMAP_SINGLE, 13286 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 5945 },
	{ 0x9b59, 0x9b59, PDF_CMAP_SINGLE, 13488 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 3041 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 9741 },
	{ 0x9b5f, 0x9b5f, PDF_CMAP_SINGLE, 10360 },
	{ 0x9b60, 0x9b61, PDF_CMAP_RANGE, 10358 },
	{ 0x9b64, 0x9b64, PDF_CMAP_SINGLE, 10990 },
	{ 0x9b66, 0x9b66, PDF_CMAP_SINGLE, 10985 },
	{ 0x9b67, 0x9b67, PDF_CMAP_SINGLE, 10982 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 10989 },
	{ 0x9b69, 0x9b69, PDF_CMAP_SINGLE, 18373 },
	{ 0x9b6c, 0x9b6c, PDF_CMAP_SINGLE, 10991 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4737 },
	{ 0x9b70, 0x9b70, PDF_CMAP_SINGLE, 10988 },
	{ 0x9b71, 0x9b71, PDF_CMAP_SINGLE, 10984 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 10983 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 10987 },
	{ 0x9b76, 0x9b76, PDF_CMAP_SINGLE, 10986 },
	{ 0x9b77, 0x9b77, PDF_CMAP_SINGLE, 4736 },
	{ 0x9b7a, 0x9b7a, PDF_CMAP_SINGLE, 11552 },
	{ 0x9b7b, 0x9b7b, PDF_CMAP_SINGLE, 11547 },
	{ 0x9b7c, 0x9b7c, PDF_CMAP_SINGLE, 11545 },
	{ 0x9b7d, 0x9b7d, PDF_CMAP_SINGLE, 11554 },
	{ 0x9b7e, 0x9b7e, PDF_CMAP_SINGLE, 11546 },
	{ 0x9b7f, 0x9b7f, PDF_CMAP_SINGLE, 15419 },
	{ 0x9b80, 0x9b80, PDF_CMAP_SINGLE, 11542 },
	{ 0x9b81, 0x9b81, PDF_CMAP_SINGLE, 17393 },
	{ 0x9b82, 0x9b82, PDF_CMAP_SINGLE, 11548 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 16921 },
	{ 0x9b85, 0x9b85, PDF_CMAP_SINGLE, 11543 },
	{ 0x9b86, 0x9b86, PDF_CMAP_SINGLE, 12011 },
	{ 0x9b87, 0x9b87, PDF_CMAP_SINGLE, 11544 },
	{ 0x9b88, 0x9b88, PDF_CMAP_SINGLE, 11555 },
	{ 0x9b8b, 0x9b8b, PDF_CMAP_SINGLE, 16110 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 17955 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 14617 },
	{ 0x9b8f, 0x9b8f, PDF_CMAP_SINGLE, 16301 },
	{ 0x9b90, 0x9b90, PDF_CMAP_SINGLE, 11551 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 5033 },
	{ 0x9b92, 0x9b92, PDF_CMAP_SINGLE, 11550 },
	{ 0x9b93, 0x9b93, PDF_CMAP_SINGLE, 11549 },
	{ 0x9b95, 0x9b95, PDF_CMAP_SINGLE, 11553 },
	{ 0x9b97, 0x9b97, PDF_CMAP_SINGLE, 16241 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 12003 },
	{ 0x9b9b, 0x9b9b, PDF_CMAP_SINGLE, 12006 },
	{ 0x9b9d, 0x9b9d, PDF_CMAP_SINGLE, 16214 },
	{ 0x9b9e, 0x9b9e, PDF_CMAP_SINGLE, 12005 },
	{ 0x9b9f, 0x9b9f, PDF_CMAP_SINGLE, 15422 },
	{ 0x9ba0, 0x9ba0, PDF_CMAP_SINGLE, 12013 },
	{ 0x9ba1, 0x9ba1, PDF_CMAP_SINGLE, 12008 },
	{ 0x9ba2, 0x9ba2, PDF_CMAP_SINGLE, 12012 },
	{ 0x9ba4, 0x9ba4, PDF_CMAP_SINGLE, 12010 },
	{ 0x9ba5, 0x9ba5, PDF_CMAP_SINGLE, 12009 },
	{ 0x9ba6, 0x9ba6, PDF_CMAP_SINGLE, 12007 },
	{ 0x9ba8, 0x9ba8, PDF_CMAP_SINGLE, 12004 },
	{ 0x9baa, 0x9baa, PDF_CMAP_SINGLE, 5309 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 5308 },
	{ 0x9bad, 0x9bad, PDF_CMAP_SINGLE, 5310 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 5307 },
	{ 0x9baf, 0x9baf, PDF_CMAP_SINGLE, 12014 },
	{ 0x9bb0, 0x9bb0, PDF_CMAP_SINGLE, 16308 },
	{ 0x9bb5, 0x9bb5, PDF_CMAP_SINGLE, 12375 },
	{ 0x9bb6, 0x9bb6, PDF_CMAP_SINGLE, 12378 },
	{ 0x9bb8, 0x9bb8, PDF_CMAP_SINGLE, 12376 },
	{ 0x9bb9, 0x9bb9, PDF_CMAP_SINGLE, 12380 },
	{ 0x9bbd, 0x9bbd, PDF_CMAP_SINGLE, 12381 },
	{ 0x9bbf, 0x9bbf, PDF_CMAP_SINGLE, 12373 },
	{ 0x9bc0, 0x9bc0, PDF_CMAP_SINGLE, 5486 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 12374 },
	{ 0x9bc3, 0x9bc3, PDF_CMAP_SINGLE, 12372 },
	{ 0x9bc4, 0x9bc4, PDF_CMAP_SINGLE, 12379 },
	{ 0x9bc6, 0x9bc6, PDF_CMAP_SINGLE, 12371 },
	{ 0x9bc7, 0x9bc7, PDF_CMAP_SINGLE, 12370 },
	{ 0x9bc8, 0x9bc8, PDF_CMAP_SINGLE, 5485 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 5483 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 5482 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 15420 },
	{ 0x9bd3, 0x9bd3, PDF_CMAP_SINGLE, 12377 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 12697 },
	{ 0x9bd5, 0x9bd5, PDF_CMAP_SINGLE, 12703 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 5630 },
	{ 0x9bd7, 0x9bd7, PDF_CMAP_SINGLE, 12698 },
	{ 0x9bd9, 0x9bd9, PDF_CMAP_SINGLE, 12701 },
	{ 0x9bda, 0x9bda, PDF_CMAP_SINGLE, 12705 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 5631 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 12700 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 18374 },
	{ 0x9bde, 0x9bde, PDF_CMAP_SINGLE, 12692 },
	{ 0x9be0, 0x9be0, PDF_CMAP_SINGLE, 12691 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 12704 },
	{ 0x9be2, 0x9be2, PDF_CMAP_SINGLE, 12695 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 12693 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 12702 },
	{ 0x9be6, 0x9be6, PDF_CMAP_SINGLE, 12694 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 5629 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 5628 },
	{ 0x9be9, 0x9be9, PDF_CMAP_SINGLE, 15297 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 12689 },
	{ 0x9bec, 0x9bec, PDF_CMAP_SINGLE, 12699 },
	{ 0x9bed, 0x9bed, PDF_CMAP_SINGLE, 16087 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 12696 },
	{ 0x9bf1, 0x9bf1, PDF_CMAP_SINGLE, 18375 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 18376 },
	{ 0x9bf7, 0x9bf7, PDF_CMAP_SINGLE, 12947 },
	{ 0x9bf8, 0x9bf8, PDF_CMAP_SINGLE, 12950 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5484 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 14619 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 14618 },
	{ 0x9c05, 0x9c05, PDF_CMAP_SINGLE, 12948 },
	{ 0x9c06, 0x9c06, PDF_CMAP_SINGLE, 12954 },
	{ 0x9c07, 0x9c07, PDF_CMAP_SINGLE, 12952 },
	{ 0x9c08, 0x9c08, PDF_CMAP_SINGLE, 12946 },
	{ 0x9c09, 0x9c09, PDF_CMAP_SINGLE, 12957 },
	{ 0x9c0a, 0x9c0a, PDF_CMAP_SINGLE, 16186 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 12945 },
	{ 0x9c0c, 0x9c0c, PDF_CMAP_SINGLE, 14620 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 5728 },
	{ 0x9c0e, 0x9c0e, PDF_CMAP_SINGLE, 12953 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 15418 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 12949 },
	{ 0x9c13, 0x9c13, PDF_CMAP_SINGLE, 5727 },
	{ 0x9c14, 0x9c14, PDF_CMAP_SINGLE, 12956 },
	{ 0x9c15, 0x9c15, PDF_CMAP_SINGLE, 16158 },
	{ 0x9c17, 0x9c17, PDF_CMAP_SINGLE, 12955 },
	{ 0x9c1b, 0x9c1b, PDF_CMAP_SINGLE, 18715 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 13132 },
	{ 0x9c1d, 0x9c1d, PDF_CMAP_SINGLE, 13131 },
	{ 0x9c1f, 0x9c1f, PDF_CMAP_SINGLE, 17954 },
	{ 0x9c20, 0x9c20, PDF_CMAP_SINGLE, 18378 },
	{ 0x9c21, 0x9c21, PDF_CMAP_SINGLE, 13138 },
	{ 0x9c23, 0x9c23, PDF_CMAP_SINGLE, 13134 },
	{ 0x9c24, 0x9c24, PDF_CMAP_SINGLE, 13137 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 5808 },
	{ 0x9c26, 0x9c26, PDF_CMAP_SINGLE, 17894 },
	{ 0x9c28, 0x9c29, PDF_CMAP_RANGE, 13135 },
	{ 0x9c2b, 0x9c2b, PDF_CMAP_SINGLE, 13130 },
	{ 0x9c2c, 0x9c2c, PDF_CMAP_SINGLE, 13133 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 5807 },
	{ 0x9c2e, 0x9c2e, PDF_CMAP_SINGLE, 16162 },
	{ 0x9c2f, 0x9c2f, PDF_CMAP_SINGLE, 15751 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 5869 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 13298 },
	{ 0x9c33, 0x9c33, PDF_CMAP_SINGLE, 13293 },
	{ 0x9c34, 0x9c34, PDF_CMAP_SINGLE, 13297 },
	{ 0x9c35, 0x9c35, PDF_CMAP_SINGLE, 15417 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 13300 },
	{ 0x9c37, 0x9c37, PDF_CMAP_SINGLE, 13296 },
	{ 0x9c39, 0x9c39, PDF_CMAP_SINGLE, 13292 },
	{ 0x9c3a, 0x9c3a, PDF_CMAP_SINGLE, 15742 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 5871 },
	{ 0x9c3c, 0x9c3c, PDF_CMAP_SINGLE, 13295 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 13299 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 5870 },
	{ 0x9c3f, 0x9c3f, PDF_CMAP_SINGLE, 13290 },
	{ 0x9c40, 0x9c40, PDF_CMAP_SINGLE, 12951 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 13294 },
	{ 0x9c44, 0x9c44, PDF_CMAP_SINGLE, 13291 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 15985 },
	{ 0x9c46, 0x9c46, PDF_CMAP_SINGLE, 13288 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 13289 },
	{ 0x9c49, 0x9c49, PDF_CMAP_SINGLE, 5868 },
	{ 0x9c4a, 0x9c4a, PDF_CMAP_SINGLE, 13399 },
	{ 0x9c4b, 0x9c4b, PDF_CMAP_SINGLE, 13401 },
	{ 0x9c4c, 0x9c4c, PDF_CMAP_SINGLE, 13404 },
	{ 0x9c4d, 0x9c4d, PDF_CMAP_SINGLE, 13400 },
	{ 0x9c4e, 0x9c4e, PDF_CMAP_SINGLE, 13405 },
	{ 0x9c4f, 0x9c4f, PDF_CMAP_SINGLE, 15414 },
	{ 0x9c50, 0x9c50, PDF_CMAP_SINGLE, 13398 },
	{ 0x9c52, 0x9c52, PDF_CMAP_SINGLE, 13396 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 15415 },
	{ 0x9c54, 0x9c54, PDF_CMAP_SINGLE, 5913 },
	{ 0x9c55, 0x9c55, PDF_CMAP_SINGLE, 13402 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 5915 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 5914 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 13397 },
	{ 0x9c59, 0x9c59, PDF_CMAP_SINGLE, 13403 },
	{ 0x9c5d, 0x9c5d, PDF_CMAP_SINGLE, 16063 },
	{ 0x9c5e, 0x9c5e, PDF_CMAP_SINGLE, 13493 },
	{ 0x9c5f, 0x9c5f, PDF_CMAP_SINGLE, 5946 },
	{ 0x9c60, 0x9c60, PDF_CMAP_SINGLE, 13494 },
	{ 0x9c62, 0x9c62, PDF_CMAP_SINGLE, 13492 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 13489 },
	{ 0x9c66, 0x9c66, PDF_CMAP_SINGLE, 13491 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 13490 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 13556 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 13558 },
	{ 0x9c6e, 0x9c6e, PDF_CMAP_SINGLE, 13557 },
	{ 0x9c71, 0x9c71, PDF_CMAP_SINGLE, 13588 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 16033 },
	{ 0x9c73, 0x9c73, PDF_CMAP_SINGLE, 13587 },
	{ 0x9c74, 0x9c74, PDF_CMAP_SINGLE, 13586 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 13589 },
	{ 0x9c77, 0x9c78, PDF_CMAP_RANGE, 5984 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 13633 },
	{ 0x9c7a, 0x9c7a, PDF_CMAP_SINGLE, 13637 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 15416 },
	{ 0x9c7c, 0x9c7c, PDF_CMAP_SINGLE, 17682 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 3042 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 9055 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 9744 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3961 },
	{ 0x9cea, 0x9cea, PDF_CMAP_SINGLE, 9742 },
	{ 0x9ced, 0x9ced, PDF_CMAP_SINGLE, 9743 },
	{ 0x9cf1, 0x9cf2, PDF_CMAP_RANGE, 10361 },
	{ 0x9cf3, 0x9cf3, PDF_CMAP_SINGLE, 4350 },
	{ 0x9cf4, 0x9cf4, PDF_CMAP_SINGLE, 4348 },
	{ 0x9cf5, 0x9cf5, PDF_CMAP_SINGLE, 10363 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 4349 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 10996 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 10999 },
	{ 0x9cfa, 0x9cfa, PDF_CMAP_SINGLE, 10993 },
	{ 0x9cfb, 0x9cfb, PDF_CMAP_SINGLE, 11000 },
	{ 0x9cfc, 0x9cfc, PDF_CMAP_SINGLE, 10992 },
	{ 0x9cfd, 0x9cfd, PDF_CMAP_SINGLE, 10994 },
	{ 0x9cff, 0x9cff, PDF_CMAP_SINGLE, 10995 },
	{ 0x9d00, 0x9d00, PDF_CMAP_SINGLE, 10998 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 16187 },
	{ 0x9d03, 0x9d03, PDF_CMAP_SINGLE, 4740 },
	{ 0x9d04, 0x9d04, PDF_CMAP_SINGLE, 11003 },
	{ 0x9d05, 0x9d05, PDF_CMAP_SINGLE, 11002 },
	{ 0x9d06, 0x9d06, PDF_CMAP_SINGLE, 4738 },
	{ 0x9d07, 0x9d07, PDF_CMAP_SINGLE, 10997 },
	{ 0x9d08, 0x9d08, PDF_CMAP_SINGLE, 11001 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 4739 },
	{ 0x9d0c, 0x9d0c, PDF_CMAP_SINGLE, 15430 },
	{ 0x9d10, 0x9d10, PDF_CMAP_SINGLE, 11565 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 5038 },
	{ 0x9d14, 0x9d14, PDF_CMAP_SINGLE, 11560 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 5034 },
	{ 0x9d16, 0x9d16, PDF_CMAP_SINGLE, 15484 },
	{ 0x9d17, 0x9d17, PDF_CMAP_SINGLE, 11557 },
	{ 0x9d18, 0x9d18, PDF_CMAP_SINGLE, 11563 },
	{ 0x9d19, 0x9d19, PDF_CMAP_SINGLE, 11566 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 5039 },
	{ 0x9d1d, 0x9d1d, PDF_CMAP_SINGLE, 11562 },
	{ 0x9d1e, 0x9d1e, PDF_CMAP_SINGLE, 11559 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 11567 },
	{ 0x9d20, 0x9d20, PDF_CMAP_SINGLE, 11558 },
	{ 0x9d21, 0x9d21, PDF_CMAP_SINGLE, 15425 },
	{ 0x9d22, 0x9d22, PDF_CMAP_SINGLE, 11564 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 5035 },
	{ 0x9d25, 0x9d25, PDF_CMAP_SINGLE, 11556 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 5036 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 5037 },
	{ 0x9d29, 0x9d29, PDF_CMAP_SINGLE, 11561 },
	{ 0x9d2d, 0x9d2d, PDF_CMAP_SINGLE, 12032 },
	{ 0x9d2e, 0x9d2f, PDF_CMAP_RANGE, 12019 },
	{ 0x9d30, 0x9d30, PDF_CMAP_SINGLE, 12023 },
	{ 0x9d31, 0x9d31, PDF_CMAP_SINGLE, 12021 },
	{ 0x9d33, 0x9d33, PDF_CMAP_SINGLE, 12015 },
	{ 0x9d34, 0x9d34, PDF_CMAP_SINGLE, 16458 },
	{ 0x9d36, 0x9d36, PDF_CMAP_SINGLE, 12018 },
	{ 0x9d37, 0x9d37, PDF_CMAP_SINGLE, 12028 },
	{ 0x9d38, 0x9d38, PDF_CMAP_SINGLE, 12022 },
	{ 0x9d39, 0x9d39, PDF_CMAP_SINGLE, 14638 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 5311 },
	{ 0x9d3d, 0x9d3d, PDF_CMAP_SINGLE, 12030 },
	{ 0x9d3e, 0x9d3e, PDF_CMAP_SINGLE, 12027 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 5312 },
	{ 0x9d40, 0x9d40, PDF_CMAP_SINGLE, 12029 },
	{ 0x9d41, 0x9d41, PDF_CMAP_SINGLE, 12016 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 12025 },
	{ 0x9d44, 0x9d44, PDF_CMAP_SINGLE, 16299 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 12024 },
	{ 0x9d49, 0x9d49, PDF_CMAP_SINGLE, 18381 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 12385 },
	{ 0x9d4b, 0x9d4b, PDF_CMAP_SINGLE, 12387 },
	{ 0x9d4c, 0x9d4c, PDF_CMAP_SINGLE, 12390 },
	{ 0x9d4e, 0x9d4e, PDF_CMAP_SINGLE, 17721 },
	{ 0x9d4f, 0x9d4f, PDF_CMAP_SINGLE, 12384 },
	{ 0x9d50, 0x9d50, PDF_CMAP_SINGLE, 16074 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 5487 },
	{ 0x9d52, 0x9d52, PDF_CMAP_SINGLE, 12392 },
	{ 0x9d53, 0x9d53, PDF_CMAP_SINGLE, 12383 },
	{ 0x9d54, 0x9d54, PDF_CMAP_SINGLE, 12393 },
	{ 0x9d56, 0x9d56, PDF_CMAP_SINGLE, 12389 },
	{ 0x9d57, 0x9d57, PDF_CMAP_SINGLE, 12391 },
	{ 0x9d58, 0x9d58, PDF_CMAP_SINGLE, 12395 },
	{ 0x9d59, 0x9d59, PDF_CMAP_SINGLE, 12388 },
	{ 0x9d5a, 0x9d5a, PDF_CMAP_SINGLE, 12396 },
	{ 0x9d5b, 0x9d5b, PDF_CMAP_SINGLE, 12386 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 12382 },
	{ 0x9d5d, 0x9d5d, PDF_CMAP_SINGLE, 5488 },
	{ 0x9d5e, 0x9d5e, PDF_CMAP_SINGLE, 15982 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 12394 },
	{ 0x9d60, 0x9d60, PDF_CMAP_SINGLE, 5489 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 5633 },
	{ 0x9d67, 0x9d67, PDF_CMAP_SINGLE, 12017 },
	{ 0x9d68, 0x9d68, PDF_CMAP_SINGLE, 12731 },
	{ 0x9d69, 0x9d69, PDF_CMAP_SINGLE, 12722 },
	{ 0x9d6a, 0x9d6a, PDF_CMAP_SINGLE, 5635 },
	{ 0x9d6b, 0x9d6b, PDF_CMAP_SINGLE, 12718 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 5636 },
	{ 0x9d6d, 0x9d6d, PDF_CMAP_SINGLE, 15276 },
	{ 0x9d6e, 0x9d6e, PDF_CMAP_SINGLE, 17211 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 12727 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 12721 },
	{ 0x9d71, 0x9d71, PDF_CMAP_SINGLE, 12711 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 5634 },
	{ 0x9d73, 0x9d73, PDF_CMAP_SINGLE, 12724 },
	{ 0x9d74, 0x9d75, PDF_CMAP_RANGE, 12719 },
	{ 0x9d77, 0x9d77, PDF_CMAP_SINGLE, 12706 },
	{ 0x9d78, 0x9d78, PDF_CMAP_SINGLE, 12713 },
	{ 0x9d79, 0x9d79, PDF_CMAP_SINGLE, 12728 },
	{ 0x9d7b, 0x9d7b, PDF_CMAP_SINGLE, 12725 },
	{ 0x9d7c, 0x9d7c, PDF_CMAP_SINGLE, 16126 },
	{ 0x9d7d, 0x9d7d, PDF_CMAP_SINGLE, 12717 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 14629 },
	{ 0x9d7f, 0x9d7f, PDF_CMAP_SINGLE, 12729 },
	{ 0x9d80, 0x9d80, PDF_CMAP_SINGLE, 12712 },
	{ 0x9d81, 0x9d81, PDF_CMAP_SINGLE, 12707 },
	{ 0x9d82, 0x9d82, PDF_CMAP_SINGLE, 12726 },
	{ 0x9d83, 0x9d83, PDF_CMAP_SINGLE, 14630 },
	{ 0x9d84, 0x9d84, PDF_CMAP_SINGLE, 12709 },
	{ 0x9d85, 0x9d85, PDF_CMAP_SINGLE, 12723 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 12714 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 12730 },
	{ 0x9d88, 0x9d88, PDF_CMAP_SINGLE, 12710 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 5632 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 12708 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 12715 },
	{ 0x9d90, 0x9d90, PDF_CMAP_SINGLE, 12964 },
	{ 0x9d92, 0x9d92, PDF_CMAP_SINGLE, 12962 },
	{ 0x9d93, 0x9d93, PDF_CMAP_SINGLE, 17395 },
	{ 0x9d94, 0x9d94, PDF_CMAP_SINGLE, 12967 },
	{ 0x9d96, 0x9d96, PDF_CMAP_SINGLE, 12979 },
	{ 0x9d97, 0x9d97, PDF_CMAP_SINGLE, 12970 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 12963 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 12959 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 12972 },
	{ 0x9d9b, 0x9d9b, PDF_CMAP_SINGLE, 12965 },
	{ 0x9d9c, 0x9d9c, PDF_CMAP_SINGLE, 12968 },
	{ 0x9d9d, 0x9d9d, PDF_CMAP_SINGLE, 12961 },
	{ 0x9d9e, 0x9d9e, PDF_CMAP_SINGLE, 12975 },
	{ 0x9d9f, 0x9d9f, PDF_CMAP_SINGLE, 12958 },
	{ 0x9da0, 0x9da0, PDF_CMAP_SINGLE, 12966 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 12971 },
	{ 0x9da2, 0x9da2, PDF_CMAP_SINGLE, 12973 },
	{ 0x9da3, 0x9da3, PDF_CMAP_SINGLE, 12976 },
	{ 0x9da4, 0x9da4, PDF_CMAP_SINGLE, 12960 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 15435 },
	{ 0x9da6, 0x9da7, PDF_CMAP_RANGE, 12980 },
	{ 0x9da8, 0x9da8, PDF_CMAP_SINGLE, 12974 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 12978 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 12969 },
	{ 0x9dab, 0x9dab, PDF_CMAP_SINGLE, 16176 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 13154 },
	{ 0x9dad, 0x9dad, PDF_CMAP_SINGLE, 13157 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 5809 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 13156 },
	{ 0x9db2, 0x9db2, PDF_CMAP_SINGLE, 13161 },
	{ 0x9db3, 0x9db3, PDF_CMAP_SINGLE, 13159 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 5810 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 13150 },
	{ 0x9db6, 0x9db6, PDF_CMAP_SINGLE, 13140 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 13139 },
	{ 0x9db8, 0x9db8, PDF_CMAP_SINGLE, 5812 },
	{ 0x9db9, 0x9dba, PDF_CMAP_RANGE, 13152 },
	{ 0x9dbb, 0x9dbb, PDF_CMAP_SINGLE, 13149 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 13141 },
	{ 0x9dbd, 0x9dbd, PDF_CMAP_SINGLE, 18384 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 13146 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 12977 },
	{ 0x9dc0, 0x9dc0, PDF_CMAP_SINGLE, 17394 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 13142 },
	{ 0x9dc2, 0x9dc2, PDF_CMAP_SINGLE, 5811 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 13148 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 14633 },
	{ 0x9dc5, 0x9dc5, PDF_CMAP_SINGLE, 13147 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 13143 },
	{ 0x9dc8, 0x9dc8, PDF_CMAP_SINGLE, 13155 },
	{ 0x9dc9, 0x9dc9, PDF_CMAP_SINGLE, 17785 },
	{ 0x9dca, 0x9dca, PDF_CMAP_SINGLE, 13144 },
	{ 0x9dcb, 0x9dcb, PDF_CMAP_SINGLE, 13305 },
	{ 0x9dcc, 0x9dcc, PDF_CMAP_SINGLE, 13158 },
	{ 0x9dcd, 0x9dcd, PDF_CMAP_SINGLE, 13160 },
	{ 0x9dce, 0x9dce, PDF_CMAP_SINGLE, 13151 },
	{ 0x9dcf, 0x9dcf, PDF_CMAP_SINGLE, 13145 },
	{ 0x9dd0, 0x9dd0, PDF_CMAP_SINGLE, 13306 },
	{ 0x9dd1, 0x9dd1, PDF_CMAP_SINGLE, 13308 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 13302 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 5872 },
	{ 0x9dd4, 0x9dd4, PDF_CMAP_SINGLE, 14622 },
	{ 0x9dd5, 0x9dd5, PDF_CMAP_SINGLE, 13315 },
	{ 0x9dd6, 0x9dd6, PDF_CMAP_SINGLE, 13313 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 5873 },
	{ 0x9dd8, 0x9dd8, PDF_CMAP_SINGLE, 13312 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 13311 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 13304 },
	{ 0x9ddb, 0x9ddb, PDF_CMAP_SINGLE, 13301 },
	{ 0x9ddc, 0x9ddc, PDF_CMAP_SINGLE, 13307 },
	{ 0x9ddd, 0x9ddd, PDF_CMAP_SINGLE, 13316 },
	{ 0x9dde, 0x9dde, PDF_CMAP_SINGLE, 13303 },
	{ 0x9ddf, 0x9ddf, PDF_CMAP_SINGLE, 13309 },
	{ 0x9de1, 0x9de1, PDF_CMAP_SINGLE, 13414 },
	{ 0x9de2, 0x9de2, PDF_CMAP_SINGLE, 13419 },
	{ 0x9de3, 0x9de3, PDF_CMAP_SINGLE, 13409 },
	{ 0x9de4, 0x9de4, PDF_CMAP_SINGLE, 13412 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 5916 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 13416 },
	{ 0x9de8, 0x9de8, PDF_CMAP_SINGLE, 13423 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 13310 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 13410 },
	{ 0x9dec, 0x9dec, PDF_CMAP_SINGLE, 13420 },
	{ 0x9ded, 0x9ded, PDF_CMAP_SINGLE, 13424 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 13415 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 13408 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 13418 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 13417 },
	{ 0x9df3, 0x9df3, PDF_CMAP_SINGLE, 13422 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 13421 },
	{ 0x9df5, 0x9df5, PDF_CMAP_SINGLE, 13314 },
	{ 0x9df6, 0x9df6, PDF_CMAP_SINGLE, 13413 },
	{ 0x9df7, 0x9df7, PDF_CMAP_SINGLE, 13407 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 13411 },
	{ 0x9df9, 0x9dfa, PDF_CMAP_RANGE, 5947 },
	{ 0x9dfb, 0x9dfb, PDF_CMAP_SINGLE, 13406 },
	{ 0x9dfc, 0x9dfc, PDF_CMAP_SINGLE, 18385 },
	{ 0x9dfd, 0x9dfd, PDF_CMAP_SINGLE, 13505 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 13496 },
	{ 0x9dff, 0x9dff, PDF_CMAP_SINGLE, 13504 },
	{ 0x9e00, 0x9e01, PDF_CMAP_RANGE, 13501 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 13495 },
	{ 0x9e03, 0x9e03, PDF_CMAP_SINGLE, 13498 },
	{ 0x9e04, 0x9e04, PDF_CMAP_SINGLE, 13506 },
	{ 0x9e05, 0x9e05, PDF_CMAP_SINGLE, 13500 },
	{ 0x9e06, 0x9e06, PDF_CMAP_SINGLE, 13499 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 13497 },
	{ 0x9e09, 0x9e09, PDF_CMAP_SINGLE, 13503 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 17790 },
	{ 0x9e0b, 0x9e0b, PDF_CMAP_SINGLE, 13559 },
	{ 0x9e0c, 0x9e0c, PDF_CMAP_SINGLE, 17786 },
	{ 0x9e0d, 0x9e0d, PDF_CMAP_SINGLE, 13560 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 14632 },
	{ 0x9e0f, 0x9e0f, PDF_CMAP_SINGLE, 13562 },
	{ 0x9e10, 0x9e10, PDF_CMAP_SINGLE, 13561 },
	{ 0x9e11, 0x9e11, PDF_CMAP_SINGLE, 13564 },
	{ 0x9e12, 0x9e12, PDF_CMAP_SINGLE, 13563 },
	{ 0x9e13, 0x9e13, PDF_CMAP_SINGLE, 13591 },
	{ 0x9e14, 0x9e14, PDF_CMAP_SINGLE, 13590 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 13616 },
	{ 0x9e17, 0x9e17, PDF_CMAP_SINGLE, 13617 },
	{ 0x9e18, 0x9e18, PDF_CMAP_SINGLE, 15428 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 13629 },
	{ 0x9e1a, 0x9e1a, PDF_CMAP_SINGLE, 5989 },
	{ 0x9e1b, 0x9e1b, PDF_CMAP_SINGLE, 5993 },
	{ 0x9e1c, 0x9e1c, PDF_CMAP_SINGLE, 16260 },
	{ 0x9e1d, 0x9e1d, PDF_CMAP_SINGLE, 13638 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 5994 },
	{ 0x9e1f, 0x9e1f, PDF_CMAP_SINGLE, 17683 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 3043 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 5729 },
	{ 0x9e7a, 0x9e7a, PDF_CMAP_SINGLE, 13162 },
	{ 0x9e7b, 0x9e7b, PDF_CMAP_SINGLE, 17400 },
	{ 0x9e7c, 0x9e7d, PDF_CMAP_RANGE, 5949 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 3044 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 9745 },
	{ 0x9e81, 0x9e81, PDF_CMAP_SINGLE, 16143 },
	{ 0x9e82, 0x9e82, PDF_CMAP_SINGLE, 3962 },
	{ 0x9e83, 0x9e83, PDF_CMAP_SINGLE, 11004 },
	{ 0x9e84, 0x9e84, PDF_CMAP_SINGLE, 16246 },
	{ 0x9e85, 0x9e85, PDF_CMAP_SINGLE, 16114 },
	{ 0x9e86, 0x9e87, PDF_CMAP_RANGE, 11569 },
	{ 0x9e88, 0x9e88, PDF_CMAP_SINGLE, 11568 },
	{ 0x9e89, 0x9e89, PDF_CMAP_SINGLE, 12034 },
	{ 0x9e8a, 0x9e8a, PDF_CMAP_SINGLE, 12033 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 5313 },
	{ 0x9e8c, 0x9e8c, PDF_CMAP_SINGLE, 12398 },
	{ 0x9e8d, 0x9e8d, PDF_CMAP_SINGLE, 12035 },
	{ 0x9e8e, 0x9e8e, PDF_CMAP_SINGLE, 12397 },
	{ 0x9e90, 0x9e90, PDF_CMAP_SINGLE, 14641 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 12733 },
	{ 0x9e92, 0x9e92, PDF_CMAP_SINGLE, 5637 },
	{ 0x9e93, 0x9e93, PDF_CMAP_SINGLE, 5639 },
	{ 0x9e94, 0x9e94, PDF_CMAP_SINGLE, 12732 },
	{ 0x9e95, 0x9e95, PDF_CMAP_SINGLE, 14642 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 16316 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 5638 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 16167 },
	{ 0x9e99, 0x9e99, PDF_CMAP_SINGLE, 12982 },
	{ 0x9e9a, 0x9e9a, PDF_CMAP_SINGLE, 12984 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 12983 },
	{ 0x9e9c, 0x9e9c, PDF_CMAP_SINGLE, 13163 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 5813 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 14643 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 5917 },
	{ 0x9ea0, 0x9ea0, PDF_CMAP_SINGLE, 13507 },
	{ 0x9ea1, 0x9ea1, PDF_CMAP_SINGLE, 13565 },
	{ 0x9ea2, 0x9ea2, PDF_CMAP_SINGLE, 14644 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 13642 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 3045 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 18756 },
	{ 0x9ea7, 0x9ea7, PDF_CMAP_SINGLE, 10364 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 16245 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 4741 },
	{ 0x9eaa, 0x9eaa, PDF_CMAP_SINGLE, 14646 },
	{ 0x9eab, 0x9eab, PDF_CMAP_SINGLE, 16018 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 17396 },
	{ 0x9ead, 0x9ead, PDF_CMAP_SINGLE, 11572 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 11571 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 14647 },
	{ 0x9eb0, 0x9eb0, PDF_CMAP_SINGLE, 12036 },
	{ 0x9eb1, 0x9eb1, PDF_CMAP_SINGLE, 18388 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 5640 },
	{ 0x9eb5, 0x9eb5, PDF_CMAP_SINGLE, 5730 },
	{ 0x9eb6, 0x9eb6, PDF_CMAP_SINGLE, 13317 },
	{ 0x9eb7, 0x9eb7, PDF_CMAP_SINGLE, 13634 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 3046 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 4351 },
	{ 0x9ebd, 0x9ebd, PDF_CMAP_SINGLE, 18389 },
	{ 0x9ebe, 0x9ebe, PDF_CMAP_SINGLE, 4742 },
	{ 0x9ebf, 0x9ebf, PDF_CMAP_SINGLE, 15736 },
	{ 0x9ec0, 0x9ec0, PDF_CMAP_SINGLE, 12734 },
	{ 0x9ec1, 0x9ec1, PDF_CMAP_SINGLE, 14649 },
	{ 0x9ec2, 0x9ec2, PDF_CMAP_SINGLE, 13425 },
	{ 0x9ec3, 0x9ec3, PDF_CMAP_SINGLE, 3515 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 17684 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 18390 },
	{ 0x9ec7, 0x9ec7, PDF_CMAP_SINGLE, 17783 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 12037 },
	{ 0x9ecc, 0x9ecc, PDF_CMAP_SINGLE, 5969 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 3516 },
	{ 0x9ece, 0x9ece, PDF_CMAP_SINGLE, 4743 },
	{ 0x9ecf, 0x9ecf, PDF_CMAP_SINGLE, 5314 },
	{ 0x9ed0, 0x9ed0, PDF_CMAP_SINGLE, 13426 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 3517 },
	{ 0x9ed3, 0x9ed3, PDF_CMAP_SINGLE, 11005 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 5041 },
	{ 0x9ed5, 0x9ed6, PDF_CMAP_RANGE, 11573 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 5040 },
	{ 0x9eda, 0x9eda, PDF_CMAP_SINGLE, 12038 },
	{ 0x9edb, 0x9edb, PDF_CMAP_SINGLE, 5318 },
	{ 0x9edc, 0x9edd, PDF_CMAP_RANGE, 5316 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 5315 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 12399 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 5490 },
	{ 0x9ee2, 0x9ee2, PDF_CMAP_SINGLE, 18392 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 12986 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 12985 },
	{ 0x9ee6, 0x9ee6, PDF_CMAP_SINGLE, 12988 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 12987 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 5731 },
	{ 0x9eeb, 0x9eeb, PDF_CMAP_SINGLE, 13164 },
	{ 0x9eed, 0x9eed, PDF_CMAP_SINGLE, 13166 },
	{ 0x9eee, 0x9eee, PDF_CMAP_SINGLE, 13165 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 5814 },
	{ 0x9ef0, 0x9ef0, PDF_CMAP_SINGLE, 13318 },
	{ 0x9ef1, 0x9ef1, PDF_CMAP_SINGLE, 18393 },
	{ 0x9ef2, 0x9ef3, PDF_CMAP_RANGE, 13427 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 5918 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 13566 },
	{ 0x9ef6, 0x9ef6, PDF_CMAP_SINGLE, 13592 },
	{ 0x9ef7, 0x9ef7, PDF_CMAP_SINGLE, 5986 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 18394 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 9056 },
	{ 0x9efa, 0x9efa, PDF_CMAP_SINGLE, 11575 },
	{ 0x9efb, 0x9efb, PDF_CMAP_SINGLE, 12039 },
	{ 0x9efc, 0x9efc, PDF_CMAP_SINGLE, 12735 },
	{ 0x9efd, 0x9efd, PDF_CMAP_SINGLE, 9746 },
	{ 0x9efe, 0x9efe, PDF_CMAP_SINGLE, 18382 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 12040 },
	{ 0x9f00, 0x9f00, PDF_CMAP_SINGLE, 12401 },
	{ 0x9f01, 0x9f01, PDF_CMAP_SINGLE, 12400 },
	{ 0x9f02, 0x9f02, PDF_CMAP_SINGLE, 14656 },
	{ 0x9f06, 0x9f06, PDF_CMAP_SINGLE, 13429 },
	{ 0x9f07, 0x9f07, PDF_CMAP_SINGLE, 5951 },
	{ 0x9f08, 0x9f08, PDF_CMAP_SINGLE, 14657 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 13567 },
	{ 0x9f0a, 0x9f0a, PDF_CMAP_SINGLE, 13593 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3963 },
	{ 0x9f0f, 0x9f10, PDF_CMAP_RANGE, 11006 },
	{ 0x9f12, 0x9f12, PDF_CMAP_SINGLE, 11576 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 3964 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 5491 },
	{ 0x9f16, 0x9f16, PDF_CMAP_SINGLE, 12402 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 14661 },
	{ 0x9f18, 0x9f18, PDF_CMAP_SINGLE, 13168 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 5815 },
	{ 0x9f1a, 0x9f1a, PDF_CMAP_SINGLE, 13169 },
	{ 0x9f1b, 0x9f1b, PDF_CMAP_SINGLE, 13167 },
	{ 0x9f1c, 0x9f1c, PDF_CMAP_SINGLE, 13430 },
	{ 0x9f1e, 0x9f1e, PDF_CMAP_SINGLE, 13508 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 3965 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 12043 },
	{ 0x9f23, 0x9f23, PDF_CMAP_SINGLE, 12042 },
	{ 0x9f24, 0x9f24, PDF_CMAP_SINGLE, 12041 },
	{ 0x9f25, 0x9f25, PDF_CMAP_SINGLE, 12403 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 16272 },
	{ 0x9f27, 0x9f27, PDF_CMAP_SINGLE, 17952 },
	{ 0x9f28, 0x9f28, PDF_CMAP_SINGLE, 12407 },
	{ 0x9f29, 0x9f29, PDF_CMAP_SINGLE, 12406 },
	{ 0x9f2a, 0x9f2a, PDF_CMAP_SINGLE, 12405 },
	{ 0x9f2b, 0x9f2b, PDF_CMAP_SINGLE, 12404 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 5492 },
	{ 0x9f2d, 0x9f2d, PDF_CMAP_SINGLE, 12736 },
	{ 0x9f2e, 0x9f2e, PDF_CMAP_SINGLE, 12990 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 5732 },
	{ 0x9f30, 0x9f30, PDF_CMAP_SINGLE, 12989 },
	{ 0x9f31, 0x9f31, PDF_CMAP_SINGLE, 13170 },
	{ 0x9f32, 0x9f32, PDF_CMAP_SINGLE, 13321 },
	{ 0x9f33, 0x9f33, PDF_CMAP_SINGLE, 13320 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 5874 },
	{ 0x9f35, 0x9f35, PDF_CMAP_SINGLE, 13319 },
	{ 0x9f36, 0x9f36, PDF_CMAP_SINGLE, 13433 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 13432 },
	{ 0x9f38, 0x9f38, PDF_CMAP_SINGLE, 13431 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 14663 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 4352 },
	{ 0x9f3d, 0x9f3d, PDF_CMAP_SINGLE, 11577 },
	{ 0x9f3e, 0x9f3e, PDF_CMAP_SINGLE, 5319 },
	{ 0x9f40, 0x9f41, PDF_CMAP_RANGE, 12737 },
	{ 0x9f42, 0x9f42, PDF_CMAP_SINGLE, 13322 },
	{ 0x9f43, 0x9f43, PDF_CMAP_SINGLE, 13434 },
	{ 0x9f44, 0x9f44, PDF_CMAP_SINGLE, 18395 },
	{ 0x9f45, 0x9f45, PDF_CMAP_SINGLE, 14666 },
	{ 0x9f46, 0x9f46, PDF_CMAP_SINGLE, 13509 },
	{ 0x9f47, 0x9f47, PDF_CMAP_SINGLE, 13568 },
	{ 0x9f48, 0x9f48, PDF_CMAP_SINGLE, 13618 },
	{ 0x9f49, 0x9f49, PDF_CMAP_SINGLE, 13644 },
	{ 0x9f4a, 0x9f4a, PDF_CMAP_SINGLE, 4353 },
	{ 0x9f4b, 0x9f4b, PDF_CMAP_SINGLE, 5320 },
	{ 0x9f4c, 0x9f4c, PDF_CMAP_SINGLE, 12408 },
	{ 0x9f4d, 0x9f4d, PDF_CMAP_SINGLE, 12739 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 13171 },
	{ 0x9f4f, 0x9f4f, PDF_CMAP_SINGLE, 13435 },
	{ 0x9f50, 0x9f50, PDF_CMAP_SINGLE, 18755 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 4745 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 16256 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 12044 },
	{ 0x9f55, 0x9f55, PDF_CMAP_SINGLE, 12409 },
	{ 0x9f56, 0x9f58, PDF_CMAP_RANGE, 12740 },
	{ 0x9f59, 0x9f59, PDF_CMAP_SINGLE, 12995 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 16163 },
	{ 0x9f5b, 0x9f5b, PDF_CMAP_SINGLE, 12991 },
	{ 0x9f5c, 0x9f5c, PDF_CMAP_SINGLE, 5816 },
	{ 0x9f5d, 0x9f5d, PDF_CMAP_SINGLE, 12994 },
	{ 0x9f5e, 0x9f5e, PDF_CMAP_SINGLE, 12993 },
	{ 0x9f5f, 0x9f5f, PDF_CMAP_SINGLE, 5733 },
	{ 0x9f60, 0x9f60, PDF_CMAP_SINGLE, 12992 },
	{ 0x9f61, 0x9f61, PDF_CMAP_SINGLE, 5735 },
	{ 0x9f62, 0x9f62, PDF_CMAP_SINGLE, 14671 },
	{ 0x9f63, 0x9f63, PDF_CMAP_SINGLE, 5734 },
	{ 0x9f64, 0x9f64, PDF_CMAP_SINGLE, 13173 },
	{ 0x9f65, 0x9f65, PDF_CMAP_SINGLE, 13172 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 5817 },
	{ 0x9f69, 0x9f69, PDF_CMAP_SINGLE, 14672 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 5876 },
	{ 0x9f6b, 0x9f6b, PDF_CMAP_SINGLE, 13323 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 5875 },
	{ 0x9f6e, 0x9f6f, PDF_CMAP_RANGE, 13438 },
	{ 0x9f70, 0x9f70, PDF_CMAP_SINGLE, 13437 },
	{ 0x9f71, 0x9f71, PDF_CMAP_SINGLE, 13436 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 5953 },
	{ 0x9f74, 0x9f76, PDF_CMAP_RANGE, 13510 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 5952 },
	{ 0x9f78, 0x9f78, PDF_CMAP_SINGLE, 13569 },
	{ 0x9f79, 0x9f79, PDF_CMAP_SINGLE, 13572 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 13571 },
	{ 0x9f7b, 0x9f7b, PDF_CMAP_SINGLE, 13570 },
	{ 0x9f7e, 0x9f7e, PDF_CMAP_SINGLE, 13643 },
	{ 0x9f7f, 0x9f7f, PDF_CMAP_SINGLE, 18048 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 5042 },
	{ 0x9f8e, 0x9f8e, PDF_CMAP_SINGLE, 14674 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 5500 },
	{ 0x9f91, 0x9f91, PDF_CMAP_SINGLE, 12996 },
	{ 0x9f92, 0x9f92, PDF_CMAP_SINGLE, 13174 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 5877 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 13324 },
	{ 0x9f98, 0x9f98, PDF_CMAP_SINGLE, 13645 },
	{ 0x9f99, 0x9f99, PDF_CMAP_SINGLE, 17799 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 5043 },
	{ 0x9f9f, 0x9f9f, PDF_CMAP_SINGLE, 17990 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 12045 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 13325 },
	{ 0x9fa4, 0x9fa4, PDF_CMAP_SINGLE, 13594 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 16240 },
	{ 0xe000, 0xe000, PDF_CMAP_SINGLE, 16395 },
	{ 0xe002, 0xe003, PDF_CMAP_RANGE, 16397 },
	{ 0xe009, 0xe00a, PDF_CMAP_RANGE, 16404 },
	{ 0xe00c, 0xe00c, PDF_CMAP_SINGLE, 16407 },
	{ 0xe00e, 0xe00e, PDF_CMAP_SINGLE, 16409 },
	{ 0xe012, 0xe016, PDF_CMAP_RANGE, 16413 },
	{ 0xe018, 0xe018, PDF_CMAP_SINGLE, 16419 },
	{ 0xe01a, 0xe01e, PDF_CMAP_RANGE, 16421 },
	{ 0xe020, 0xe020, PDF_CMAP_SINGLE, 16427 },
	{ 0xe022, 0xe023, PDF_CMAP_RANGE, 16429 },
	{ 0xe025, 0xe025, PDF_CMAP_SINGLE, 16432 },
	{ 0xe02d, 0xe02e, PDF_CMAP_RANGE, 16440 },
	{ 0xe030, 0xe030, PDF_CMAP_SINGLE, 16443 },
	{ 0xe032, 0xe038, PDF_CMAP_RANGE, 16445 },
	{ 0xe03b, 0xe03c, PDF_CMAP_RANGE, 16454 },
	{ 0xe045, 0xe045, PDF_CMAP_SINGLE, 16464 },
	{ 0xe04c, 0xe04c, PDF_CMAP_SINGLE, 16471 },
	{ 0xe05a, 0xe05a, PDF_CMAP_SINGLE, 16485 },
	{ 0xe05c, 0xe05c, PDF_CMAP_SINGLE, 16487 },
	{ 0xe05f, 0xe061, PDF_CMAP_RANGE, 16490 },
	{ 0xe064, 0xe064, PDF_CMAP_SINGLE, 16495 },
	{ 0xe066, 0xe066, PDF_CMAP_SINGLE, 16497 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 16503 },
	{ 0xe071, 0xe071, PDF_CMAP_SINGLE, 16508 },
	{ 0xe074, 0xe075, PDF_CMAP_RANGE, 16511 },
	{ 0xe078, 0xe078, PDF_CMAP_SINGLE, 16515 },
	{ 0xe07d, 0xe07d, PDF_CMAP_SINGLE, 16520 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 16530 },
	{ 0xe089, 0xe08a, PDF_CMAP_RANGE, 16532 },
	{ 0xe08c, 0xe08c, PDF_CMAP_SINGLE, 16535 },
	{ 0xe093, 0xe094, PDF_CMAP_RANGE, 16542 },
	{ 0xe099, 0xe09a, PDF_CMAP_RANGE, 16548 },
	{ 0xe09c, 0xe09e, PDF_CMAP_RANGE, 16551 },
	{ 0xe0a6, 0xe0a7, PDF_CMAP_RANGE, 16561 },
	{ 0xe0a9, 0xe0a9, PDF_CMAP_SINGLE, 16564 },
	{ 0xe0ab, 0xe0ab, PDF_CMAP_SINGLE, 16566 },
	{ 0xe0ae, 0xe0ae, PDF_CMAP_SINGLE, 16569 },
	{ 0xe0ba, 0xe0ba, PDF_CMAP_SINGLE, 16581 },
	{ 0xe0bc, 0xe0bc, PDF_CMAP_SINGLE, 16583 },
	{ 0xe0be, 0xe0be, PDF_CMAP_SINGLE, 16585 },
	{ 0xe0c3, 0xe0c3, PDF_CMAP_SINGLE, 16590 },
	{ 0xe0c5, 0xe0c5, PDF_CMAP_SINGLE, 16592 },
	{ 0xe0c7, 0xe0c8, PDF_CMAP_RANGE, 16594 },
	{ 0xe0ca, 0xe0ca, PDF_CMAP_SINGLE, 16597 },
	{ 0xe0d0, 0xe0d0, PDF_CMAP_SINGLE, 16603 },
	{ 0xe0d4, 0xe0d5, PDF_CMAP_RANGE, 16607 },
	{ 0xe0dc, 0xe0dc, PDF_CMAP_SINGLE, 16615 },
	{ 0xe0df, 0xe0df, PDF_CMAP_SINGLE, 16618 },
	{ 0xe0e2, 0xe0e5, PDF_CMAP_RANGE, 16621 },
	{ 0xe0e7, 0xe0e9, PDF_CMAP_RANGE, 16626 },
	{ 0xe0eb, 0xe0ec, PDF_CMAP_RANGE, 16630 },
	{ 0xe0ee, 0xe0ee, PDF_CMAP_SINGLE, 16633 },
	{ 0xe0f2, 0xe0f2, PDF_CMAP_SINGLE, 16637 },
	{ 0xe0f8, 0xe0f9, PDF_CMAP_RANGE, 16643 },
	{ 0xe0fb, 0xe0fb, PDF_CMAP_SINGLE, 16645 },
	{ 0xe0fd, 0xe0fd, PDF_CMAP_SINGLE, 16647 },
	{ 0xe103, 0xe103, PDF_CMAP_SINGLE, 16653 },
	{ 0xe106, 0xe106, PDF_CMAP_SINGLE, 16656 },
	{ 0xe109, 0xe10b, PDF_CMAP_RANGE, 16658 },
	{ 0xe110, 0xe110, PDF_CMAP_SINGLE, 16665 },
	{ 0xe114, 0xe115, PDF_CMAP_RANGE, 16669 },
	{ 0xe11c, 0xe11d, PDF_CMAP_RANGE, 16677 },
	{ 0xe121, 0xe121, PDF_CMAP_SINGLE, 16682 },
	{ 0xe126, 0xe128, PDF_CMAP_RANGE, 16687 },
	{ 0xe130, 0xe131, PDF_CMAP_RANGE, 16697 },
	{ 0xe136, 0xe136, PDF_CMAP_SINGLE, 16703 },
	{ 0xe139, 0xe139, PDF_CMAP_SINGLE, 16706 },
	{ 0xe141, 0xe142, PDF_CMAP_RANGE, 16714 },
	{ 0xe145, 0xe145, PDF_CMAP_SINGLE, 16717 },
	{ 0xe147, 0xe148, PDF_CMAP_RANGE, 16719 },
	{ 0xe14a, 0xe14a, PDF_CMAP_SINGLE, 16721 },
	{ 0xe14d, 0xe14d, PDF_CMAP_SINGLE, 16723 },
	{ 0xe150, 0xe150, PDF_CMAP_SINGLE, 16726 },
	{ 0xe154, 0xe157, PDF_CMAP_RANGE, 16730 },
	{ 0xe159, 0xe15a, PDF_CMAP_RANGE, 16735 },
	{ 0xe15d, 0xe15d, PDF_CMAP_SINGLE, 18767 },
	{ 0xe162, 0xe162, PDF_CMAP_SINGLE, 16743 },
	{ 0xe168, 0xe16a, PDF_CMAP_RANGE, 16749 },
	{ 0xe171, 0xe172, PDF_CMAP_RANGE, 16757 },
	{ 0xe175, 0xe175, PDF_CMAP_SINGLE, 16761 },
	{ 0xe178, 0xe179, PDF_CMAP_RANGE, 16764 },
	{ 0xe17b, 0xe17c, PDF_CMAP_RANGE, 16767 },
	{ 0xe17f, 0xe180, PDF_CMAP_RANGE, 16771 },
	{ 0xe185, 0xe185, PDF_CMAP_SINGLE, 16777 },
	{ 0xe187, 0xe187, PDF_CMAP_SINGLE, 16779 },
	{ 0xe189, 0xe189, PDF_CMAP_SINGLE, 16781 },
	{ 0xe18b, 0xe18b, PDF_CMAP_SINGLE, 16783 },
	{ 0xe18f, 0xe190, PDF_CMAP_RANGE, 16787 },
	{ 0xe193, 0xe193, PDF_CMAP_SINGLE, 16790 },
	{ 0xe195, 0xe195, PDF_CMAP_SINGLE, 18771 },
	{ 0xe196, 0xe196, PDF_CMAP_SINGLE, 16792 },
	{ 0xe19f, 0xe1a0, PDF_CMAP_RANGE, 16801 },
	{ 0xe1a4, 0xe1a4, PDF_CMAP_SINGLE, 18772 },
	{ 0xe1a6, 0xe1a6, PDF_CMAP_SINGLE, 16808 },
	{ 0xe1aa, 0xe1aa, PDF_CMAP_SINGLE, 16812 },
	{ 0xe1ae, 0xe1ae, PDF_CMAP_SINGLE, 16816 },
	{ 0xe1b0, 0xe1b1, PDF_CMAP_RANGE, 16818 },
	{ 0xe1b3, 0xe1b4, PDF_CMAP_RANGE, 16821 },
	{ 0xe1b6, 0xe1b6, PDF_CMAP_SINGLE, 16824 },
	{ 0xe1b8, 0xe1b8, PDF_CMAP_SINGLE, 16826 },
	{ 0xe1bb, 0xe1bb, PDF_CMAP_SINGLE, 18773 },
	{ 0xe1bc, 0xe1bf, PDF_CMAP_RANGE, 16829 },
	{ 0xe1c1, 0xe1c1, PDF_CMAP_SINGLE, 16834 },
	{ 0xe1c4, 0xe1c5, PDF_CMAP_RANGE, 16837 },
	{ 0xe1c8, 0xe1c8, PDF_CMAP_SINGLE, 16840 },
	{ 0xe1cc, 0xe1cd, PDF_CMAP_RANGE, 16844 },
	{ 0xe1d4, 0xe1d5, PDF_CMAP_RANGE, 16852 },
	{ 0xe1d7, 0xe1d7, PDF_CMAP_SINGLE, 16855 },
	{ 0xe1da, 0xe1da, PDF_CMAP_SINGLE, 16858 },
	{ 0xe1dd, 0xe1dd, PDF_CMAP_SINGLE, 16861 },
	{ 0xe1e2, 0xe1e2, PDF_CMAP_SINGLE, 16865 },
	{ 0xe1e6, 0xe1e6, PDF_CMAP_SINGLE, 16869 },
	{ 0xe1eb, 0xe1eb, PDF_CMAP_SINGLE, 16874 },
	{ 0xe1ef, 0xe1f0, PDF_CMAP_RANGE, 16878 },
	{ 0xe1f2, 0xe1f2, PDF_CMAP_SINGLE, 16881 },
	{ 0xe1f7, 0xe1f7, PDF_CMAP_SINGLE, 16886 },
	{ 0xe1fa, 0xe1fa, PDF_CMAP_SINGLE, 16889 },
	{ 0xe1fe, 0xe1fe, PDF_CMAP_SINGLE, 16893 },
	{ 0xe200, 0xe200, PDF_CMAP_SINGLE, 16895 },
	{ 0xe201, 0xe201, PDF_CMAP_SINGLE, 18776 },
	{ 0xe202, 0xe202, PDF_CMAP_SINGLE, 16897 },
	{ 0xe205, 0xe205, PDF_CMAP_SINGLE, 16900 },
	{ 0xe207, 0xe207, PDF_CMAP_SINGLE, 16902 },
	{ 0xe20a, 0xe20c, PDF_CMAP_RANGE, 16905 },
	{ 0xe210, 0xe210, PDF_CMAP_SINGLE, 16911 },
	{ 0xe212, 0xe216, PDF_CMAP_RANGE, 16913 },
	{ 0xe219, 0xe219, PDF_CMAP_SINGLE, 16920 },
	{ 0xe21b, 0xe21c, PDF_CMAP_RANGE, 16922 },
	{ 0xe21f, 0xe221, PDF_CMAP_RANGE, 16926 },
	{ 0xe226, 0xe228, PDF_CMAP_RANGE, 16933 },
	{ 0xe22a, 0xe22a, PDF_CMAP_SINGLE, 16937 },
	{ 0xe22f, 0xe22f, PDF_CMAP_SINGLE, 16942 },
	{ 0xe231, 0xe231, PDF_CMAP_SINGLE, 16944 },
	{ 0xe233, 0xe233, PDF_CMAP_SINGLE, 16946 },
	{ 0xe237, 0xe237, PDF_CMAP_SINGLE, 16950 },
	{ 0xe23b, 0xe23b, PDF_CMAP_SINGLE, 16954 },
	{ 0xe23d, 0xe23e, PDF_CMAP_RANGE, 16956 },
	{ 0xe245, 0xe245, PDF_CMAP_SINGLE, 16964 },
	{ 0xe248, 0xe248, PDF_CMAP_SINGLE, 16967 },
	{ 0xe24a, 0xe24f, PDF_CMAP_RANGE, 16969 },
	{ 0xe251, 0xe254, PDF_CMAP_RANGE, 16976 },
	{ 0xe261, 0xe261, PDF_CMAP_SINGLE, 16992 },
	{ 0xe268, 0xe268, PDF_CMAP_SINGLE, 16998 },
	{ 0xe26b, 0xe26b, PDF_CMAP_SINGLE, 17001 },
	{ 0xe26d, 0xe26f, PDF_CMAP_RANGE, 17003 },
	{ 0xe271, 0xe271, PDF_CMAP_SINGLE, 17007 },
	{ 0xe27a, 0xe27a, PDF_CMAP_SINGLE, 17016 },
	{ 0xe27d, 0xe27d, PDF_CMAP_SINGLE, 17019 },
	{ 0xe282, 0xe282, PDF_CMAP_SINGLE, 17024 },
	{ 0xe288, 0xe289, PDF_CMAP_RANGE, 17030 },
	{ 0xe28b, 0xe28b, PDF_CMAP_SINGLE, 17033 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 17038 },
	{ 0xe294, 0xe296, PDF_CMAP_RANGE, 17042 },
	{ 0xe299, 0xe29b, PDF_CMAP_RANGE, 17047 },
	{ 0xe29f, 0xe29f, PDF_CMAP_SINGLE, 17053 },
	{ 0xe2a1, 0xe2a1, PDF_CMAP_SINGLE, 18779 },
	{ 0xe2a4, 0xe2a4, PDF_CMAP_SINGLE, 17057 },
	{ 0xe2a6, 0xe2a6, PDF_CMAP_SINGLE, 17059 },
	{ 0xe2b2, 0xe2b5, PDF_CMAP_RANGE, 17070 },
	{ 0xe2b7, 0xe2b8, PDF_CMAP_RANGE, 17075 },
	{ 0xe2ba, 0xe2ba, PDF_CMAP_SINGLE, 17078 },
	{ 0xe2be, 0xe2bf, PDF_CMAP_RANGE, 17081 },
	{ 0xe2c1, 0xe2c1, PDF_CMAP_SINGLE, 17084 },
	{ 0xe2c7, 0xe2c7, PDF_CMAP_SINGLE, 17090 },
	{ 0xe2c9, 0xe2c9, PDF_CMAP_SINGLE, 17092 },
	{ 0xe2d4, 0xe2d4, PDF_CMAP_SINGLE, 17103 },
	{ 0xe2d8, 0xe2d9, PDF_CMAP_RANGE, 17107 },
	{ 0xe2de, 0xe2de, PDF_CMAP_SINGLE, 17113 },
	{ 0xe2e0, 0xe2e2, PDF_CMAP_RANGE, 17115 },
	{ 0xe2e7, 0xe2e7, PDF_CMAP_SINGLE, 17122 },
	{ 0xe2e9, 0xe2e9, PDF_CMAP_SINGLE, 17124 },
	{ 0xe2eb, 0xe2ed, PDF_CMAP_RANGE, 17126 },
	{ 0xe2f0, 0xe2f1, PDF_CMAP_RANGE, 18781 },
	{ 0xe2f8, 0xe2f8, PDF_CMAP_SINGLE, 17137 },
	{ 0xe2fc, 0xe2fc, PDF_CMAP_SINGLE, 17141 },
	{ 0xe2ff, 0xe2ff, PDF_CMAP_SINGLE, 18783 },
	{ 0xe300, 0xe300, PDF_CMAP_SINGLE, 18784 },
	{ 0xe301, 0xe301, PDF_CMAP_SINGLE, 17144 },
	{ 0xe306, 0xe306, PDF_CMAP_SINGLE, 17149 },
	{ 0xe308, 0xe308, PDF_CMAP_SINGLE, 17151 },
	{ 0xe30c, 0xe30c, PDF_CMAP_SINGLE, 17155 },
	{ 0xe30f, 0xe30f, PDF_CMAP_SINGLE, 17158 },
	{ 0xe311, 0xe311, PDF_CMAP_SINGLE, 14123 },
	{ 0xe313, 0xe313, PDF_CMAP_SINGLE, 14125 },
	{ 0xe315, 0xe315, PDF_CMAP_SINGLE, 14127 },
	{ 0xe318, 0xe318, PDF_CMAP_SINGLE, 14129 },
	{ 0xe31b, 0xe31c, PDF_CMAP_RANGE, 14132 },
	{ 0xe31f, 0xe31f, PDF_CMAP_SINGLE, 14136 },
	{ 0xe325, 0xe325, PDF_CMAP_SINGLE, 14142 },
	{ 0xe32c, 0xe32c, PDF_CMAP_SINGLE, 14149 },
	{ 0xe330, 0xe331, PDF_CMAP_RANGE, 14153 },
	{ 0xe335, 0xe336, PDF_CMAP_RANGE, 14158 },
	{ 0xe33b, 0xe33b, PDF_CMAP_SINGLE, 14164 },
	{ 0xe33d, 0xe33d, PDF_CMAP_SINGLE, 18074 },
	{ 0xe342, 0xe342, PDF_CMAP_SINGLE, 14170 },
	{ 0xe345, 0xe345, PDF_CMAP_SINGLE, 14173 },
	{ 0xe348, 0xe348, PDF_CMAP_SINGLE, 14175 },
	{ 0xe352, 0xe352, PDF_CMAP_SINGLE, 14184 },
	{ 0xe354, 0xe354, PDF_CMAP_SINGLE, 14186 },
	{ 0xe35b, 0xe35c, PDF_CMAP_RANGE, 14192 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 14197 },
	{ 0xe362, 0xe362, PDF_CMAP_SINGLE, 14199 },
	{ 0xe364, 0xe365, PDF_CMAP_RANGE, 14201 },
	{ 0xe36f, 0xe36f, PDF_CMAP_SINGLE, 14212 },
	{ 0xe372, 0xe372, PDF_CMAP_SINGLE, 14215 },
	{ 0xe374, 0xe375, PDF_CMAP_RANGE, 14217 },
	{ 0xe377, 0xe377, PDF_CMAP_SINGLE, 14220 },
	{ 0xe379, 0xe379, PDF_CMAP_SINGLE, 14221 },
	{ 0xe37b, 0xe37b, PDF_CMAP_SINGLE, 14223 },
	{ 0xe381, 0xe382, PDF_CMAP_RANGE, 14229 },
	{ 0xe386, 0xe386, PDF_CMAP_SINGLE, 14234 },
	{ 0xe388, 0xe38a, PDF_CMAP_RANGE, 14236 },
	{ 0xe38d, 0xe38d, PDF_CMAP_SINGLE, 14241 },
	{ 0xe391, 0xe391, PDF_CMAP_SINGLE, 14245 },
	{ 0xe393, 0xe393, PDF_CMAP_SINGLE, 14247 },
	{ 0xe394, 0xe394, PDF_CMAP_SINGLE, 18080 },
	{ 0xe396, 0xe396, PDF_CMAP_SINGLE, 14249 },
	{ 0xe398, 0xe398, PDF_CMAP_SINGLE, 14251 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 14253 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 18081 },
	{ 0xe39f, 0xe3a0, PDF_CMAP_RANGE, 14257 },
	{ 0xe3a2, 0xe3a2, PDF_CMAP_SINGLE, 14260 },
	{ 0xe3a5, 0xe3a5, PDF_CMAP_SINGLE, 18082 },
	{ 0xe3a6, 0xe3a9, PDF_CMAP_RANGE, 14264 },
	{ 0xe3ac, 0xe3ac, PDF_CMAP_SINGLE, 14270 },
	{ 0xe3b1, 0xe3b2, PDF_CMAP_RANGE, 14275 },
	{ 0xe3b4, 0xe3b5, PDF_CMAP_RANGE, 14278 },
	{ 0xe3b9, 0xe3bd, PDF_CMAP_RANGE, 14283 },
	{ 0xe3c3, 0xe3c3, PDF_CMAP_SINGLE, 14293 },
	{ 0xe3c8, 0xe3c8, PDF_CMAP_SINGLE, 14297 },
	{ 0xe3cc, 0xe3cc, PDF_CMAP_SINGLE, 14301 },
	{ 0xe3cd, 0xe3cd, PDF_CMAP_SINGLE, 18084 },
	{ 0xe3cf, 0xe3cf, PDF_CMAP_SINGLE, 14303 },
	{ 0xe3d5, 0xe3d5, PDF_CMAP_SINGLE, 18085 },
	{ 0xe3d8, 0xe3da, PDF_CMAP_RANGE, 14311 },
	{ 0xe3e1, 0xe3e1, PDF_CMAP_SINGLE, 14320 },
	{ 0xe3e3, 0xe3e3, PDF_CMAP_SINGLE, 14322 },
	{ 0xe3e7, 0xe3e7, PDF_CMAP_SINGLE, 18086 },
	{ 0xe3ed, 0xe3ee, PDF_CMAP_RANGE, 14332 },
	{ 0xe3f0, 0xe3f0, PDF_CMAP_SINGLE, 14335 },
	{ 0xe3f2, 0xe3f4, PDF_CMAP_RANGE, 14337 },
	{ 0xe3f8, 0xe3f8, PDF_CMAP_SINGLE, 14343 },
	{ 0xe3fa, 0xe3fa, PDF_CMAP_SINGLE, 14345 },
	{ 0xe3fd, 0xe3fd, PDF_CMAP_SINGLE, 14348 },
	{ 0xe3ff, 0xe3ff, PDF_CMAP_SINGLE, 14350 },
	{ 0xe409, 0xe40b, PDF_CMAP_RANGE, 14360 },
	{ 0xe414, 0xe414, PDF_CMAP_SINGLE, 14369 },
	{ 0xe41b, 0xe41b, PDF_CMAP_SINGLE, 14376 },
	{ 0xe41d, 0xe41e, PDF_CMAP_RANGE, 14378 },
	{ 0xe423, 0xe423, PDF_CMAP_SINGLE, 14384 },
	{ 0xe425, 0xe425, PDF_CMAP_SINGLE, 14386 },
	{ 0xe426, 0xe426, PDF_CMAP_SINGLE, 18091 },
	{ 0xe428, 0xe428, PDF_CMAP_SINGLE, 14388 },
	{ 0xe42c, 0xe42d, PDF_CMAP_RANGE, 14392 },
	{ 0xe432, 0xe432, PDF_CMAP_SINGLE, 14398 },
	{ 0xe436, 0xe436, PDF_CMAP_SINGLE, 14402 },
	{ 0xe43a, 0xe43c, PDF_CMAP_RANGE, 14406 },
	{ 0xe43e, 0xe43e, PDF_CMAP_SINGLE, 14410 },
	{ 0xe440, 0xe441, PDF_CMAP_RANGE, 14412 },
	{ 0xe444, 0xe444, PDF_CMAP_SINGLE, 14416 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 18093 },
	{ 0xe44c, 0xe44e, PDF_CMAP_RANGE, 14423 },
	{ 0xe450, 0xe451, PDF_CMAP_RANGE, 14427 },
	{ 0xe459, 0xe45a, PDF_CMAP_RANGE, 14436 },
	{ 0xe45d, 0xe45e, PDF_CMAP_RANGE, 14440 },
	{ 0xe461, 0xe463, PDF_CMAP_RANGE, 14444 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 14448 },
	{ 0xe467, 0xe469, PDF_CMAP_RANGE, 18095 },
	{ 0xe46a, 0xe46a, PDF_CMAP_SINGLE, 14451 },
	{ 0xe46c, 0xe46c, PDF_CMAP_SINGLE, 14453 },
	{ 0xe46e, 0xe46f, PDF_CMAP_RANGE, 14455 },
	{ 0xe471, 0xe472, PDF_CMAP_RANGE, 14458 },
	{ 0xe474, 0xe474, PDF_CMAP_SINGLE, 14461 },
	{ 0xe476, 0xe476, PDF_CMAP_SINGLE, 14463 },
	{ 0xe47e, 0xe47f, PDF_CMAP_RANGE, 14470 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 14476 },
	{ 0xe486, 0xe48a, PDF_CMAP_RANGE, 14477 },
	{ 0xe48d, 0xe48e, PDF_CMAP_RANGE, 14484 },
	{ 0xe491, 0xe492, PDF_CMAP_RANGE, 14487 },
	{ 0xe495, 0xe497, PDF_CMAP_RANGE, 14491 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 18100 },
	{ 0xe4a2, 0xe4a4, PDF_CMAP_RANGE, 14503 },
	{ 0xe4ab, 0xe4ab, PDF_CMAP_SINGLE, 14512 },
	{ 0xe4af, 0xe4af, PDF_CMAP_SINGLE, 14516 },
	{ 0xe4b3, 0xe4ba, PDF_CMAP_RANGE, 14520 },
	{ 0xe4c1, 0xe4c1, PDF_CMAP_SINGLE, 14534 },
	{ 0xe4c6, 0xe4c6, PDF_CMAP_SINGLE, 14539 },
	{ 0xe4c9, 0xe4c9, PDF_CMAP_SINGLE, 14542 },
	{ 0xe4cd, 0xe4ce, PDF_CMAP_RANGE, 14546 },
	{ 0xe4d6, 0xe4d6, PDF_CMAP_SINGLE, 14555 },
	{ 0xe4d9, 0xe4d9, PDF_CMAP_SINGLE, 14558 },
	{ 0xe4dc, 0xe4dd, PDF_CMAP_RANGE, 14561 },
	{ 0xe4e4, 0xe4e4, PDF_CMAP_SINGLE, 14569 },
	{ 0xe4e6, 0xe4e8, PDF_CMAP_RANGE, 14571 },
	{ 0xe4ee, 0xe4ef, PDF_CMAP_RANGE, 14579 },
	{ 0xe4f6, 0xe4f6, PDF_CMAP_SINGLE, 14587 },
	{ 0xe4f8, 0xe4f9, PDF_CMAP_RANGE, 14589 },
	{ 0xe4ff, 0xe4ff, PDF_CMAP_SINGLE, 14596 },
	{ 0xe501, 0xe502, PDF_CMAP_RANGE, 14598 },
	{ 0xe505, 0xe505, PDF_CMAP_SINGLE, 14602 },
	{ 0xe50a, 0xe50a, PDF_CMAP_SINGLE, 14607 },
	{ 0xe50d, 0xe50d, PDF_CMAP_SINGLE, 18101 },
	{ 0xe50f, 0xe50f, PDF_CMAP_SINGLE, 14611 },
	{ 0xe512, 0xe512, PDF_CMAP_SINGLE, 14614 },
	{ 0xe514, 0xe514, PDF_CMAP_SINGLE, 14616 },
	{ 0xe516, 0xe516, PDF_CMAP_SINGLE, 18102 },
	{ 0xe51a, 0xe51a, PDF_CMAP_SINGLE, 14621 },
	{ 0xe51c, 0xe521, PDF_CMAP_RANGE, 14623 },
	{ 0xe524, 0xe524, PDF_CMAP_SINGLE, 14631 },
	{ 0xe528, 0xe528, PDF_CMAP_SINGLE, 18104 },
	{ 0xe529, 0xe52c, PDF_CMAP_RANGE, 14634 },
	{ 0xe52e, 0xe52f, PDF_CMAP_RANGE, 14639 },
	{ 0xe537, 0xe537, PDF_CMAP_SINGLE, 14648 },
	{ 0xe53e, 0xe53e, PDF_CMAP_SINGLE, 14655 },
	{ 0xe543, 0xe543, PDF_CMAP_SINGLE, 14660 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 14668 },
	{ 0xe554, 0xe555, PDF_CMAP_RANGE, 14677 },
	{ 0xe557, 0xe559, PDF_CMAP_RANGE, 14680 },
	{ 0xe55b, 0xe55c, PDF_CMAP_RANGE, 14684 },
	{ 0xe55e, 0xe55f, PDF_CMAP_RANGE, 14687 },
	{ 0xe562, 0xe562, PDF_CMAP_SINGLE, 14691 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 14695 },
	{ 0xe56d, 0xe56e, PDF_CMAP_RANGE, 14702 },
	{ 0xe570, 0xe570, PDF_CMAP_SINGLE, 14705 },
	{ 0xe572, 0xe573, PDF_CMAP_RANGE, 14707 },
	{ 0xe575, 0xe577, PDF_CMAP_RANGE, 14710 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 14714 },
	{ 0xe57c, 0xe57e, PDF_CMAP_RANGE, 14717 },
	{ 0xe580, 0xe580, PDF_CMAP_SINGLE, 14721 },
	{ 0xe582, 0xe586, PDF_CMAP_RANGE, 14723 },
	{ 0xe588, 0xe588, PDF_CMAP_SINGLE, 14729 },
	{ 0xe58b, 0xe590, PDF_CMAP_RANGE, 14732 },
	{ 0xe592, 0xe594, PDF_CMAP_RANGE, 14739 },
	{ 0xe59a, 0xe59a, PDF_CMAP_SINGLE, 14747 },
	{ 0xe59d, 0xe5a1, PDF_CMAP_RANGE, 14750 },
	{ 0xe5a4, 0xe5a4, PDF_CMAP_SINGLE, 14757 },
	{ 0xe5a6, 0xe5a8, PDF_CMAP_RANGE, 14759 },
	{ 0xe5aa, 0xe5aa, PDF_CMAP_SINGLE, 14763 },
	{ 0xe5af, 0xe5af, PDF_CMAP_SINGLE, 14768 },
	{ 0xe5b1, 0xe5b1, PDF_CMAP_SINGLE, 14770 },
	{ 0xe5b2, 0xe5b2, PDF_CMAP_SINGLE, 18107 },
	{ 0xe5b5, 0xe5b5, PDF_CMAP_SINGLE, 14774 },
	{ 0xe5b7, 0xe5b8, PDF_CMAP_RANGE, 14776 },
	{ 0xe5ba, 0xe5bb, PDF_CMAP_RANGE, 14779 },
	{ 0xe5be, 0xe5c0, PDF_CMAP_RANGE, 14783 },
	{ 0xe5c2, 0xe5c2, PDF_CMAP_SINGLE, 14787 },
	{ 0xe5c8, 0xe5c9, PDF_CMAP_RANGE, 14793 },
	{ 0xe5d7, 0xe5d9, PDF_CMAP_RANGE, 14804 },
	{ 0xe5db, 0xe5dc, PDF_CMAP_RANGE, 14808 },
	{ 0xe5df, 0xe5df, PDF_CMAP_SINGLE, 14812 },
	{ 0xe5e1, 0xe5e3, PDF_CMAP_RANGE, 14814 },
	{ 0xe5e5, 0xe5e6, PDF_CMAP_RANGE, 14818 },
	{ 0xe5e9, 0xe5e9, PDF_CMAP_SINGLE, 14822 },
	{ 0xe5ee, 0xe5f1, PDF_CMAP_RANGE, 14827 },
	{ 0xe5f3, 0xe5f3, PDF_CMAP_SINGLE, 14832 },
	{ 0xe5fb, 0xe5fc, PDF_CMAP_RANGE, 14840 },
	{ 0xe600, 0xe600, PDF_CMAP_SINGLE, 14845 },
	{ 0xe602, 0xe602, PDF_CMAP_SINGLE, 14847 },
	{ 0xe604, 0xe604, PDF_CMAP_SINGLE, 14849 },
	{ 0xe60f, 0xe612, PDF_CMAP_RANGE, 14859 },
	{ 0xe619, 0xe61c, PDF_CMAP_RANGE, 14868 },
	{ 0xe621, 0xe621, PDF_CMAP_SINGLE, 14876 },
	{ 0xe623, 0xe624, PDF_CMAP_RANGE, 14878 },
	{ 0xe627, 0xe627, PDF_CMAP_SINGLE, 14882 },
	{ 0xe62b, 0xe62b, PDF_CMAP_SINGLE, 14886 },
	{ 0xe632, 0xe634, PDF_CMAP_RANGE, 14893 },
	{ 0xe637, 0xe638, PDF_CMAP_RANGE, 14898 },
	{ 0xe63a, 0xe63a, PDF_CMAP_SINGLE, 14901 },
	{ 0xe63c, 0xe63c, PDF_CMAP_SINGLE, 14903 },
	{ 0xe63f, 0xe642, PDF_CMAP_RANGE, 14906 },
	{ 0xe645, 0xe646, PDF_CMAP_RANGE, 14912 },
	{ 0xe648, 0xe648, PDF_CMAP_SINGLE, 14915 },
	{ 0xe64b, 0xe64b, PDF_CMAP_SINGLE, 14917 },
	{ 0xe64e, 0xe64f, PDF_CMAP_RANGE, 14920 },
	{ 0xe652, 0xe655, PDF_CMAP_RANGE, 14924 },
	{ 0xe657, 0xe659, PDF_CMAP_RANGE, 14929 },
	{ 0xe65d, 0xe660, PDF_CMAP_RANGE, 14935 },
	{ 0xe662, 0xe663, PDF_CMAP_RANGE, 14940 },
	{ 0xe665, 0xe668, PDF_CMAP_RANGE, 14943 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 18111 },
	{ 0xe66e, 0xe66f, PDF_CMAP_RANGE, 14951 },
	{ 0xe671, 0xe671, PDF_CMAP_SINGLE, 14954 },
	{ 0xe676, 0xe679, PDF_CMAP_RANGE, 14959 },
	{ 0xe67b, 0xe67d, PDF_CMAP_RANGE, 14964 },
	{ 0xe67f, 0xe680, PDF_CMAP_RANGE, 14968 },
	{ 0xe683, 0xe688, PDF_CMAP_RANGE, 14971 },
	{ 0xe68a, 0xe68c, PDF_CMAP_RANGE, 14978 },
	{ 0xe68e, 0xe690, PDF_CMAP_RANGE, 14982 },
	{ 0xe692, 0xe692, PDF_CMAP_SINGLE, 14986 },
	{ 0xe696, 0xe697, PDF_CMAP_RANGE, 14990 },
	{ 0xe699, 0xe69a, PDF_CMAP_RANGE, 14993 },
	{ 0xe69c, 0xe69c, PDF_CMAP_SINGLE, 14996 },
	{ 0xe69e, 0xe69f, PDF_CMAP_RANGE, 14998 },
	{ 0xe6a1, 0xe6a2, PDF_CMAP_RANGE, 15001 },
	{ 0xe6a6, 0xe6a6, PDF_CMAP_SINGLE, 15005 },
	{ 0xe6ac, 0xe6ad, PDF_CMAP_RANGE, 15010 },
	{ 0xe6b2, 0xe6b2, PDF_CMAP_SINGLE, 15016 },
	{ 0xe6b9, 0xe6ba, PDF_CMAP_RANGE, 15023 },
	{ 0xe6bd, 0xe6be, PDF_CMAP_RANGE, 15027 },
	{ 0xe6c0, 0xe6c1, PDF_CMAP_RANGE, 15030 },
	{ 0xe6c5, 0xe6c5, PDF_CMAP_SINGLE, 18116 },
	{ 0xe6c7, 0xe6c7, PDF_CMAP_SINGLE, 15036 },
	{ 0xe6ca, 0xe6ca, PDF_CMAP_SINGLE, 15039 },
	{ 0xe6d1, 0xe6d2, PDF_CMAP_RANGE, 15046 },
	{ 0xe6d7, 0xe6d8, PDF_CMAP_RANGE, 15052 },
	{ 0xe6da, 0xe6db, PDF_CMAP_RANGE, 15055 },
	{ 0xe6dd, 0xe6e1, PDF_CMAP_RANGE, 15058 },
	{ 0xe6e4, 0xe6e6, PDF_CMAP_RANGE, 15065 },
	{ 0xe6ee, 0xe6ee, PDF_CMAP_SINGLE, 15075 },
	{ 0xe6f2, 0xe6f3, PDF_CMAP_RANGE, 15079 },
	{ 0xe6f7, 0xe6f7, PDF_CMAP_SINGLE, 15084 },
	{ 0xe6fa, 0xe6fc, PDF_CMAP_RANGE, 15087 },
	{ 0xe6ff, 0xe6ff, PDF_CMAP_SINGLE, 15092 },
	{ 0xe701, 0xe702, PDF_CMAP_RANGE, 15094 },
	{ 0xe706, 0xe707, PDF_CMAP_RANGE, 15099 },
	{ 0xe709, 0xe70a, PDF_CMAP_RANGE, 15102 },
	{ 0xe70c, 0xe70c, PDF_CMAP_SINGLE, 15105 },
	{ 0xe710, 0xe711, PDF_CMAP_RANGE, 15109 },
	{ 0xe713, 0xe713, PDF_CMAP_SINGLE, 15112 },
	{ 0xe717, 0xe718, PDF_CMAP_RANGE, 15116 },
	{ 0xe71a, 0xe71b, PDF_CMAP_RANGE, 15119 },
	{ 0xe71d, 0xe71d, PDF_CMAP_SINGLE, 15122 },
	{ 0xe724, 0xe725, PDF_CMAP_RANGE, 15129 },
	{ 0xe72b, 0xe72b, PDF_CMAP_SINGLE, 15136 },
	{ 0xe72f, 0xe72f, PDF_CMAP_SINGLE, 15140 },
	{ 0xe733, 0xe737, PDF_CMAP_RANGE, 15144 },
	{ 0xe73c, 0xe73d, PDF_CMAP_RANGE, 15153 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 15157 },
	{ 0xe744, 0xe744, PDF_CMAP_SINGLE, 15161 },
	{ 0xe747, 0xe747, PDF_CMAP_SINGLE, 15164 },
	{ 0xe74d, 0xe74f, PDF_CMAP_RANGE, 15170 },
	{ 0xe751, 0xe752, PDF_CMAP_RANGE, 15174 },
	{ 0xe75b, 0xe75d, PDF_CMAP_RANGE, 15184 },
	{ 0xe75f, 0xe75f, PDF_CMAP_SINGLE, 15188 },
	{ 0xe761, 0xe762, PDF_CMAP_RANGE, 15190 },
	{ 0xe764, 0xe765, PDF_CMAP_RANGE, 15193 },
	{ 0xe767, 0xe768, PDF_CMAP_RANGE, 15196 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 18119 },
	{ 0xe76c, 0xe76e, PDF_CMAP_RANGE, 15200 },
	{ 0xe770, 0xe770, PDF_CMAP_SINGLE, 15204 },
	{ 0xe775, 0xe775, PDF_CMAP_SINGLE, 15209 },
	{ 0xe777, 0xe778, PDF_CMAP_RANGE, 15210 },
	{ 0xe77a, 0xe77a, PDF_CMAP_SINGLE, 15213 },
	{ 0xe77c, 0xe77d, PDF_CMAP_RANGE, 15215 },
	{ 0xe784, 0xe784, PDF_CMAP_SINGLE, 15223 },
	{ 0xe786, 0xe786, PDF_CMAP_SINGLE, 15225 },
	{ 0xe789, 0xe789, PDF_CMAP_SINGLE, 15228 },
	{ 0xe78b, 0xe78b, PDF_CMAP_SINGLE, 15230 },
	{ 0xe78d, 0xe78d, PDF_CMAP_SINGLE, 15232 },
	{ 0xe790, 0xe790, PDF_CMAP_SINGLE, 15235 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 15237 },
	{ 0xe794, 0xe794, PDF_CMAP_SINGLE, 15239 },
	{ 0xe796, 0xe797, PDF_CMAP_RANGE, 15241 },
	{ 0xe799, 0xe799, PDF_CMAP_SINGLE, 15244 },
	{ 0xe7a2, 0xe7a2, PDF_CMAP_SINGLE, 15253 },
	{ 0xe7a4, 0xe7a9, PDF_CMAP_RANGE, 15255 },
	{ 0xe7b1, 0xe7b1, PDF_CMAP_SINGLE, 15268 },
	{ 0xe7b4, 0xe7b4, PDF_CMAP_SINGLE, 15271 },
	{ 0xe7b8, 0xe7b8, PDF_CMAP_SINGLE, 15275 },
	{ 0xe7bc, 0xe7bc, PDF_CMAP_SINGLE, 15279 },
	{ 0xe7c4, 0xe7c4, PDF_CMAP_SINGLE, 15286 },
	{ 0xe7ca, 0xe7ca, PDF_CMAP_SINGLE, 15292 },
	{ 0xe7ce, 0xe7ce, PDF_CMAP_SINGLE, 15296 },
	{ 0xe7d1, 0xe7d1, PDF_CMAP_SINGLE, 15299 },
	{ 0xe7d8, 0xe7d8, PDF_CMAP_SINGLE, 15306 },
	{ 0xe7da, 0xe7da, PDF_CMAP_SINGLE, 15308 },
	{ 0xe7dc, 0xe7dd, PDF_CMAP_RANGE, 15310 },
	{ 0xe7e4, 0xe7e4, PDF_CMAP_SINGLE, 15318 },
	{ 0xe7e7, 0xe7e9, PDF_CMAP_RANGE, 15321 },
	{ 0xe7eb, 0xe7eb, PDF_CMAP_SINGLE, 15325 },
	{ 0xe7ed, 0xe7ef, PDF_CMAP_RANGE, 15327 },
	{ 0xe7f1, 0xe7f3, PDF_CMAP_RANGE, 15331 },
	{ 0xe7f8, 0xe7f8, PDF_CMAP_SINGLE, 15338 },
	{ 0xe7fb, 0xe7fc, PDF_CMAP_RANGE, 15341 },
	{ 0xe7ff, 0xe7ff, PDF_CMAP_SINGLE, 15345 },
	{ 0xe800, 0xe802, PDF_CMAP_RANGE, 15346 },
	{ 0xe808, 0xe808, PDF_CMAP_SINGLE, 15354 },
	{ 0xe80c, 0xe80c, PDF_CMAP_SINGLE, 15358 },
	{ 0xe80e, 0xe80f, PDF_CMAP_RANGE, 15360 },
	{ 0xe811, 0xe811, PDF_CMAP_SINGLE, 15363 },
	{ 0xe813, 0xe813, PDF_CMAP_SINGLE, 15365 },
	{ 0xe816, 0xe818, PDF_CMAP_RANGE, 15368 },
	{ 0xe81a, 0xe81a, PDF_CMAP_SINGLE, 15372 },
	{ 0xe821, 0xe821, PDF_CMAP_SINGLE, 15379 },
	{ 0xe827, 0xe828, PDF_CMAP_RANGE, 15385 },
	{ 0xe82d, 0xe82d, PDF_CMAP_SINGLE, 15391 },
	{ 0xe82f, 0xe82f, PDF_CMAP_SINGLE, 15393 },
	{ 0xe835, 0xe835, PDF_CMAP_SINGLE, 15399 },
	{ 0xe837, 0xe838, PDF_CMAP_RANGE, 15401 },
	{ 0xe83a, 0xe83b, PDF_CMAP_RANGE, 15404 },
	{ 0xe841, 0xe842, PDF_CMAP_RANGE, 15411 },
	{ 0xe84b, 0xe84b, PDF_CMAP_SINGLE, 15421 },
	{ 0xe84d, 0xe84e, PDF_CMAP_RANGE, 15423 },
	{ 0xe851, 0xe851, PDF_CMAP_SINGLE, 15427 },
	{ 0xe855, 0xe858, PDF_CMAP_RANGE, 15431 },
	{ 0xe85b, 0xe85d, PDF_CMAP_RANGE, 15437 },
	{ 0xe860, 0xe862, PDF_CMAP_RANGE, 15442 },
	{ 0xe865, 0xe867, PDF_CMAP_RANGE, 15447 },
	{ 0xe869, 0xe86a, PDF_CMAP_RANGE, 15451 },
	{ 0xe86c, 0xe86c, PDF_CMAP_SINGLE, 15453 },
	{ 0xe871, 0xe871, PDF_CMAP_SINGLE, 15458 },
	{ 0xe878, 0xe878, PDF_CMAP_SINGLE, 15465 },
	{ 0xe87d, 0xe87f, PDF_CMAP_RANGE, 15470 },
	{ 0xe887, 0xe887, PDF_CMAP_SINGLE, 15480 },
	{ 0xe88f, 0xe88f, PDF_CMAP_SINGLE, 15488 },
	{ 0xe891, 0xe892, PDF_CMAP_RANGE, 15490 },
	{ 0xe894, 0xe895, PDF_CMAP_RANGE, 15493 },
	{ 0xe89a, 0xe89a, PDF_CMAP_SINGLE, 15499 },
	{ 0xe89d, 0xe89d, PDF_CMAP_SINGLE, 15502 },
	{ 0xe8a0, 0xe8a1, PDF_CMAP_RANGE, 15505 },
	{ 0xe8a3, 0xe8a7, PDF_CMAP_RANGE, 15508 },
	{ 0xe8a9, 0xe8aa, PDF_CMAP_RANGE, 15514 },
	{ 0xe8ac, 0xe8b0, PDF_CMAP_RANGE, 15517 },
	{ 0xe8b5, 0xe8b6, PDF_CMAP_RANGE, 15526 },
	{ 0xe8b8, 0xe8b8, PDF_CMAP_SINGLE, 15529 },
	{ 0xe8bd, 0xe8c3, PDF_CMAP_RANGE, 15534 },
	{ 0xe8c5, 0xe8c5, PDF_CMAP_SINGLE, 15542 },
	{ 0xe8c7, 0xe8c7, PDF_CMAP_SINGLE, 15544 },
	{ 0xe8cb, 0xe8cb, PDF_CMAP_SINGLE, 15548 },
	{ 0xe8cd, 0xe8ce, PDF_CMAP_RANGE, 15550 },
	{ 0xe8d0, 0xe8d1, PDF_CMAP_RANGE, 15553 },
	{ 0xe8d3, 0xe8d3, PDF_CMAP_SINGLE, 15556 },
	{ 0xe8d5, 0xe8d6, PDF_CMAP_RANGE, 15558 },
	{ 0xe8d8, 0xe8da, PDF_CMAP_RANGE, 15561 },
	{ 0xe8dd, 0xe8dd, PDF_CMAP_SINGLE, 15566 },
	{ 0xe8df, 0xe8e1, PDF_CMAP_RANGE, 15568 },
	{ 0xe8e5, 0xe8e6, PDF_CMAP_RANGE, 15574 },
	{ 0xe8e8, 0xe8ea, PDF_CMAP_RANGE, 15577 },
	{ 0xe8ed, 0xe8ed, PDF_CMAP_SINGLE, 15582 },
	{ 0xe8f1, 0xe8f2, PDF_CMAP_RANGE, 15586 },
	{ 0xe8f5, 0xe8f5, PDF_CMAP_SINGLE, 15590 },
	{ 0xe8fb, 0xe8fb, PDF_CMAP_SINGLE, 15596 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 15599 },
	{ 0xe900, 0xe900, PDF_CMAP_SINGLE, 15601 },
	{ 0xe903, 0xe903, PDF_CMAP_SINGLE, 15604 },
	{ 0xe907, 0xe908, PDF_CMAP_RANGE, 15608 },
	{ 0xe90c, 0xe90c, PDF_CMAP_SINGLE, 15613 },
	{ 0xe917, 0xe918, PDF_CMAP_RANGE, 15624 },
	{ 0xe91c, 0xe91c, PDF_CMAP_SINGLE, 15629 },
	{ 0xe91e, 0xe922, PDF_CMAP_RANGE, 15631 },
	{ 0xe925, 0xe92c, PDF_CMAP_RANGE, 15638 },
	{ 0xe92e, 0xe92f, PDF_CMAP_RANGE, 15647 },
	{ 0xe933, 0xe936, PDF_CMAP_RANGE, 15652 },
	{ 0xe938, 0xe938, PDF_CMAP_SINGLE, 18127 },
	{ 0xe93a, 0xe93a, PDF_CMAP_SINGLE, 15659 },
	{ 0xe93c, 0xe93e, PDF_CMAP_RANGE, 15661 },
	{ 0xe941, 0xe941, PDF_CMAP_SINGLE, 15666 },
	{ 0xe944, 0xe945, PDF_CMAP_RANGE, 15669 },
	{ 0xe948, 0xe948, PDF_CMAP_SINGLE, 15673 },
	{ 0xe94d, 0xe94d, PDF_CMAP_SINGLE, 15678 },
	{ 0xe950, 0xe950, PDF_CMAP_SINGLE, 15681 },
	{ 0xe952, 0xe955, PDF_CMAP_RANGE, 15683 },
	{ 0xe957, 0xe958, PDF_CMAP_RANGE, 15688 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 15691 },
	{ 0xe95c, 0xe95e, PDF_CMAP_RANGE, 15693 },
	{ 0xe960, 0xe961, PDF_CMAP_RANGE, 15697 },
	{ 0xe964, 0xe964, PDF_CMAP_SINGLE, 15701 },
	{ 0xe967, 0xe967, PDF_CMAP_SINGLE, 15704 },
	{ 0xe968, 0xe968, PDF_CMAP_SINGLE, 18129 },
	{ 0xe96e, 0xe96f, PDF_CMAP_RANGE, 18133 },
	{ 0xe973, 0xe973, PDF_CMAP_SINGLE, 18138 },
	{ 0xe974, 0xe974, PDF_CMAP_SINGLE, 15707 },
	{ 0xe975, 0xe975, PDF_CMAP_SINGLE, 18139 },
	{ 0xe977, 0xe978, PDF_CMAP_RANGE, 18141 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 18144 },
	{ 0xe97d, 0xe97d, PDF_CMAP_SINGLE, 18147 },
	{ 0xe981, 0xe982, PDF_CMAP_RANGE, 18150 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 15709 },
	{ 0xe98a, 0xe98a, PDF_CMAP_SINGLE, 15710 },
	{ 0xe98e, 0xe98e, PDF_CMAP_SINGLE, 15711 },
	{ 0xe991, 0xe991, PDF_CMAP_SINGLE, 18159 },
	{ 0xe9b0, 0xe9b0, PDF_CMAP_SINGLE, 17364 },
	{ 0xe9cb, 0xe9cb, PDF_CMAP_SINGLE, 18201 },
	{ 0xe9ef, 0xe9ef, PDF_CMAP_SINGLE, 18228 },
	{ 0xea43, 0xea43, PDF_CMAP_SINGLE, 18297 },
	{ 0xeaa9, 0xeaa9, PDF_CMAP_SINGLE, 18380 },
	{ 0xeabf, 0xeac1, PDF_CMAP_RANGE, 18396 },
	{ 0xeac5, 0xeac5, PDF_CMAP_SINGLE, 18400 },
	{ 0xeac9, 0xeacc, PDF_CMAP_RANGE, 18402 },
	{ 0xeacf, 0xead1, PDF_CMAP_RANGE, 18408 },
	{ 0xead3, 0xead4, PDF_CMAP_RANGE, 18411 },
	{ 0xead6, 0xeada, PDF_CMAP_RANGE, 18414 },
	{ 0xeadc, 0xeadc, PDF_CMAP_SINGLE, 18419 },
	{ 0xeade, 0xeae3, PDF_CMAP_RANGE, 18420 },
	{ 0xeae7, 0xeaec, PDF_CMAP_RANGE, 18426 },
	{ 0xeaee, 0xeaee, PDF_CMAP_SINGLE, 18432 },
	{ 0xeaf0, 0xeaf2, PDF_CMAP_RANGE, 18433 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 17328 },
	{ 0xeaf5, 0xeaf6, PDF_CMAP_RANGE, 18436 },
	{ 0xeaf8, 0xeaf8, PDF_CMAP_SINGLE, 18439 },
	{ 0xeaf9, 0xeaf9, PDF_CMAP_SINGLE, 17330 },
	{ 0xeafa, 0xeafb, PDF_CMAP_RANGE, 18440 },
	{ 0xeafe, 0xeaff, PDF_CMAP_RANGE, 18443 },
	{ 0xeb00, 0xeb00, PDF_CMAP_SINGLE, 18445 },
	{ 0xeb02, 0xeb05, PDF_CMAP_RANGE, 18446 },
	{ 0xeb08, 0xeb0f, PDF_CMAP_RANGE, 18451 },
	{ 0xeb11, 0xeb12, PDF_CMAP_RANGE, 18459 },
	{ 0xeb15, 0xeb15, PDF_CMAP_SINGLE, 18462 },
	{ 0xeb18, 0xeb1d, PDF_CMAP_RANGE, 18465 },
	{ 0xeb1f, 0xeb21, PDF_CMAP_RANGE, 18471 },
	{ 0xeb23, 0xeb23, PDF_CMAP_SINGLE, 18474 },
	{ 0xeb25, 0xeb25, PDF_CMAP_SINGLE, 18475 },
	{ 0xeb27, 0xeb27, PDF_CMAP_SINGLE, 18476 },
	{ 0xeb2a, 0xeb2a, PDF_CMAP_SINGLE, 18477 },
	{ 0xeb2c, 0xeb39, PDF_CMAP_RANGE, 18478 },
	{ 0xeb3d, 0xeb3d, PDF_CMAP_SINGLE, 15771 },
	{ 0xeb3e, 0xeb3f, PDF_CMAP_RANGE, 18492 },
	{ 0xeb41, 0xeb41, PDF_CMAP_SINGLE, 15773 },
	{ 0xeb43, 0xeb44, PDF_CMAP_RANGE, 18494 },
	{ 0xeb46, 0xeb46, PDF_CMAP_SINGLE, 15776 },
	{ 0xeb4a, 0xeb4a, PDF_CMAP_SINGLE, 18497 },
	{ 0xeb4b, 0xeb4c, PDF_CMAP_RANGE, 15779 },
	{ 0xeb4d, 0xeb4e, PDF_CMAP_RANGE, 18498 },
	{ 0xeb4f, 0xeb50, PDF_CMAP_RANGE, 15781 },
	{ 0xeb53, 0xeb53, PDF_CMAP_SINGLE, 18500 },
	{ 0xeb54, 0xeb54, PDF_CMAP_SINGLE, 15785 },
	{ 0xeb55, 0xeb56, PDF_CMAP_RANGE, 18501 },
	{ 0xeb57, 0xeb57, PDF_CMAP_SINGLE, 15786 },
	{ 0xeb58, 0xeb59, PDF_CMAP_RANGE, 18503 },
	{ 0xeb5a, 0xeb5b, PDF_CMAP_RANGE, 15787 },
	{ 0xeb5d, 0xeb5d, PDF_CMAP_SINGLE, 15790 },
	{ 0xeb5f, 0xeb5f, PDF_CMAP_SINGLE, 15792 },
	{ 0xeb61, 0xeb61, PDF_CMAP_SINGLE, 15794 },
	{ 0xeb62, 0xeb63, PDF_CMAP_RANGE, 18505 },
	{ 0xeb65, 0xeb65, PDF_CMAP_SINGLE, 18508 },
	{ 0xeb67, 0xeb67, PDF_CMAP_SINGLE, 18509 },
	{ 0xeb68, 0xeb69, PDF_CMAP_RANGE, 15796 },
	{ 0xeb6b, 0xeb6b, PDF_CMAP_SINGLE, 15799 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 15801 },
	{ 0xeb6f, 0xeb71, PDF_CMAP_RANGE, 18510 },
	{ 0xeb73, 0xeb73, PDF_CMAP_SINGLE, 18513 },
	{ 0xeb74, 0xeb74, PDF_CMAP_SINGLE, 15804 },
	{ 0xeb76, 0xeb76, PDF_CMAP_SINGLE, 17360 },
	{ 0xeb77, 0xeb77, PDF_CMAP_SINGLE, 18515 },
	{ 0xeb78, 0xeb78, PDF_CMAP_SINGLE, 15805 },
	{ 0xeb7a, 0xeb7a, PDF_CMAP_SINGLE, 18516 },
	{ 0xeb7e, 0xeb7f, PDF_CMAP_RANGE, 18518 },
	{ 0xeb80, 0xeb81, PDF_CMAP_RANGE, 15809 },
	{ 0xeb83, 0xeb84, PDF_CMAP_RANGE, 18520 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 15812 },
	{ 0xeb88, 0xeb88, PDF_CMAP_SINGLE, 18522 },
	{ 0xeb89, 0xeb89, PDF_CMAP_SINGLE, 15814 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 18523 },
	{ 0xeb8c, 0xeb8e, PDF_CMAP_RANGE, 18524 },
	{ 0xeb8f, 0xeb8f, PDF_CMAP_SINGLE, 15816 },
	{ 0xeb90, 0xeb90, PDF_CMAP_SINGLE, 18527 },
	{ 0xeb93, 0xeb93, PDF_CMAP_SINGLE, 15819 },
	{ 0xeb95, 0xeb95, PDF_CMAP_SINGLE, 18528 },
	{ 0xeb97, 0xeb97, PDF_CMAP_SINGLE, 15822 },
	{ 0xeb98, 0xeb9a, PDF_CMAP_RANGE, 18529 },
	{ 0xeb9b, 0xeb9b, PDF_CMAP_SINGLE, 15823 },
	{ 0xeb9f, 0xeb9f, PDF_CMAP_SINGLE, 18534 },
	{ 0xeba0, 0xeba0, PDF_CMAP_SINGLE, 15825 },
	{ 0xeba2, 0xeba2, PDF_CMAP_SINGLE, 18535 },
	{ 0xeba3, 0xeba3, PDF_CMAP_SINGLE, 15827 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 18536 },
	{ 0xeba8, 0xeba8, PDF_CMAP_SINGLE, 15830 },
	{ 0xebaa, 0xebaa, PDF_CMAP_SINGLE, 18538 },
	{ 0xebab, 0xebab, PDF_CMAP_SINGLE, 15832 },
	{ 0xebad, 0xebad, PDF_CMAP_SINGLE, 15834 },
	{ 0xebae, 0xebae, PDF_CMAP_SINGLE, 18539 },
	{ 0xebaf, 0xebaf, PDF_CMAP_SINGLE, 15835 },
	{ 0xebb0, 0xebb0, PDF_CMAP_SINGLE, 18540 },
	{ 0xebb2, 0xebb3, PDF_CMAP_RANGE, 18541 },
	{ 0xebb4, 0xebb4, PDF_CMAP_SINGLE, 15837 },
	{ 0xebb6, 0xebb6, PDF_CMAP_SINGLE, 15839 },
	{ 0xebb9, 0xebb9, PDF_CMAP_SINGLE, 15842 },
	{ 0xebbc, 0xebbc, PDF_CMAP_SINGLE, 15844 },
	{ 0xebbd, 0xebbd, PDF_CMAP_SINGLE, 18544 },
	{ 0xebbf, 0xebbf, PDF_CMAP_SINGLE, 15846 },
	{ 0xebc2, 0xebc2, PDF_CMAP_SINGLE, 15849 },
	{ 0xebc5, 0xebc6, PDF_CMAP_RANGE, 18546 },
	{ 0xebc7, 0xebc7, PDF_CMAP_SINGLE, 15851 },
	{ 0xebc8, 0xebc8, PDF_CMAP_SINGLE, 18548 },
	{ 0xebca, 0xebca, PDF_CMAP_SINGLE, 18549 },
	{ 0xebcc, 0xebcd, PDF_CMAP_RANGE, 15854 },
	{ 0xebce, 0xebce, PDF_CMAP_SINGLE, 18550 },
	{ 0xebd0, 0xebd0, PDF_CMAP_SINGLE, 18551 },
	{ 0xebd3, 0xebd3, PDF_CMAP_SINGLE, 18552 },
	{ 0xebd5, 0xebd5, PDF_CMAP_SINGLE, 18553 },
	{ 0xebd7, 0xebd8, PDF_CMAP_RANGE, 15860 },
	{ 0xebda, 0xebdb, PDF_CMAP_RANGE, 18554 },
	{ 0xebdd, 0xebdd, PDF_CMAP_SINGLE, 15864 },
	{ 0xebdf, 0xebdf, PDF_CMAP_SINGLE, 18556 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 18557 },
	{ 0xebe4, 0xebe4, PDF_CMAP_SINGLE, 18558 },
	{ 0xebe7, 0xebe7, PDF_CMAP_SINGLE, 15871 },
	{ 0xebe8, 0xebe9, PDF_CMAP_RANGE, 18559 },
	{ 0xebeb, 0xebec, PDF_CMAP_RANGE, 18561 },
	{ 0xebee, 0xebee, PDF_CMAP_SINGLE, 15874 },
	{ 0xebef, 0xebef, PDF_CMAP_SINGLE, 18563 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 18564 },
	{ 0xebf2, 0xebf2, PDF_CMAP_SINGLE, 15876 },
	{ 0xebf5, 0xebf5, PDF_CMAP_SINGLE, 18566 },
	{ 0xebf6, 0xebf6, PDF_CMAP_SINGLE, 15878 },
	{ 0xebf9, 0xebfb, PDF_CMAP_RANGE, 15881 },
	{ 0xebfd, 0xebfe, PDF_CMAP_RANGE, 15885 },
	{ 0xec05, 0xec06, PDF_CMAP_RANGE, 15892 },
	{ 0xec08, 0xec09, PDF_CMAP_RANGE, 18568 },
	{ 0xec0c, 0xec0c, PDF_CMAP_SINGLE, 15896 },
	{ 0xec0e, 0xec10, PDF_CMAP_RANGE, 15898 },
	{ 0xec13, 0xec13, PDF_CMAP_SINGLE, 18571 },
	{ 0xec14, 0xec14, PDF_CMAP_SINGLE, 15903 },
	{ 0xec17, 0xec18, PDF_CMAP_RANGE, 15906 },
	{ 0xec19, 0xec1c, PDF_CMAP_RANGE, 18572 },
	{ 0xec1d, 0xec1d, PDF_CMAP_SINGLE, 15908 },
	{ 0xec20, 0xec20, PDF_CMAP_SINGLE, 18576 },
	{ 0xec22, 0xec22, PDF_CMAP_SINGLE, 15912 },
	{ 0xec24, 0xec24, PDF_CMAP_SINGLE, 15914 },
	{ 0xec26, 0xec26, PDF_CMAP_SINGLE, 15916 },
	{ 0xec2a, 0xec2a, PDF_CMAP_SINGLE, 15920 },
	{ 0xec2b, 0xec2b, PDF_CMAP_SINGLE, 18577 },
	{ 0xec2c, 0xec2e, PDF_CMAP_RANGE, 15921 },
	{ 0xec2f, 0xec2f, PDF_CMAP_SINGLE, 18578 },
	{ 0xec31, 0xec31, PDF_CMAP_SINGLE, 15925 },
	{ 0xec32, 0xec32, PDF_CMAP_SINGLE, 18579 },
	{ 0xec35, 0xec35, PDF_CMAP_SINGLE, 15928 },
	{ 0xec37, 0xec37, PDF_CMAP_SINGLE, 15930 },
	{ 0xec3b, 0xec3c, PDF_CMAP_RANGE, 15934 },
	{ 0xec3f, 0xec41, PDF_CMAP_RANGE, 18580 },
	{ 0xec42, 0xec42, PDF_CMAP_SINGLE, 15938 },
	{ 0xec43, 0xec43, PDF_CMAP_SINGLE, 18583 },
	{ 0xec44, 0xec44, PDF_CMAP_SINGLE, 17384 },
	{ 0xec45, 0xec49, PDF_CMAP_RANGE, 18584 },
	{ 0xec4b, 0xec4c, PDF_CMAP_RANGE, 18589 },
	{ 0xec4f, 0xec4f, PDF_CMAP_SINGLE, 18592 },
	{ 0xec52, 0xec52, PDF_CMAP_SINGLE, 18593 },
	{ 0xec54, 0xec54, PDF_CMAP_SINGLE, 18594 },
	{ 0xec56, 0xec58, PDF_CMAP_RANGE, 18595 },
	{ 0xec5a, 0xec5a, PDF_CMAP_SINGLE, 18598 },
	{ 0xec5c, 0xec5d, PDF_CMAP_RANGE, 18599 },
	{ 0xec5f, 0xec60, PDF_CMAP_RANGE, 18601 },
	{ 0xec62, 0xec63, PDF_CMAP_RANGE, 18604 },
	{ 0xec67, 0xec67, PDF_CMAP_SINGLE, 18607 },
	{ 0xec68, 0xec68, PDF_CMAP_SINGLE, 15949 },
	{ 0xec69, 0xec73, PDF_CMAP_RANGE, 18608 },
	{ 0xec75, 0xec76, PDF_CMAP_RANGE, 18620 },
	{ 0xec79, 0xec79, PDF_CMAP_SINGLE, 18624 },
	{ 0xec7b, 0xec7b, PDF_CMAP_SINGLE, 18626 },
	{ 0xec7e, 0xec7e, PDF_CMAP_SINGLE, 18627 },
	{ 0xec80, 0xec80, PDF_CMAP_SINGLE, 18629 },
	{ 0xec84, 0xec84, PDF_CMAP_SINGLE, 18632 },
	{ 0xec86, 0xec86, PDF_CMAP_SINGLE, 18634 },
	{ 0xec8d, 0xec8d, PDF_CMAP_SINGLE, 18636 },
	{ 0xec8e, 0xec8e, PDF_CMAP_SINGLE, 15957 },
	{ 0xec91, 0xec91, PDF_CMAP_SINGLE, 18638 },
	{ 0xec92, 0xec92, PDF_CMAP_SINGLE, 15960 },
	{ 0xec93, 0xec94, PDF_CMAP_RANGE, 18639 },
	{ 0xec95, 0xec95, PDF_CMAP_SINGLE, 15961 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 18641 },
	{ 0xec98, 0xec98, PDF_CMAP_SINGLE, 18642 },
	{ 0xec99, 0xec99, PDF_CMAP_SINGLE, 15963 },
	{ 0xec9a, 0xec9b, PDF_CMAP_RANGE, 18643 },
	{ 0xec9d, 0xec9d, PDF_CMAP_SINGLE, 18646 },
	{ 0xec9e, 0xec9e, PDF_CMAP_SINGLE, 15964 },
	{ 0xeca0, 0xeca0, PDF_CMAP_SINGLE, 18647 },
	{ 0xeca3, 0xeca4, PDF_CMAP_RANGE, 18648 },
	{ 0xeca7, 0xeca8, PDF_CMAP_RANGE, 18650 },
	{ 0xecab, 0xecab, PDF_CMAP_SINGLE, 18652 },
	{ 0xecad, 0xecad, PDF_CMAP_SINGLE, 18653 },
	{ 0xecaf, 0xecaf, PDF_CMAP_SINGLE, 15973 },
	{ 0xecb4, 0xecb4, PDF_CMAP_SINGLE, 18655 },
	{ 0xecb6, 0xecb6, PDF_CMAP_SINGLE, 15979 },
	{ 0xecb9, 0xecbb, PDF_CMAP_RANGE, 18657 },
	{ 0xeccf, 0xeccf, PDF_CMAP_SINGLE, 16000 },
	{ 0xecd4, 0xecd4, PDF_CMAP_SINGLE, 16005 },
	{ 0xecdb, 0xecdc, PDF_CMAP_RANGE, 16012 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 18660 },
	{ 0xecdf, 0xece1, PDF_CMAP_RANGE, 16015 },
	{ 0xece5, 0xece5, PDF_CMAP_SINGLE, 16021 },
	{ 0xecf0, 0xecf0, PDF_CMAP_SINGLE, 16032 },
	{ 0xecf2, 0xecf2, PDF_CMAP_SINGLE, 16034 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 16037 },
	{ 0xecfc, 0xecfc, PDF_CMAP_SINGLE, 16044 },
	{ 0xed04, 0xed04, PDF_CMAP_SINGLE, 18663 },
	{ 0xed07, 0xed07, PDF_CMAP_SINGLE, 18664 },
	{ 0xed0a, 0xed0a, PDF_CMAP_SINGLE, 18666 },
	{ 0xed0c, 0xed11, PDF_CMAP_RANGE, 18667 },
	{ 0xed13, 0xed13, PDF_CMAP_SINGLE, 18673 },
	{ 0xed15, 0xed17, PDF_CMAP_RANGE, 18674 },
	{ 0xed18, 0xed18, PDF_CMAP_SINGLE, 16058 },
	{ 0xed1a, 0xed1a, PDF_CMAP_SINGLE, 18677 },
	{ 0xed1c, 0xed1c, PDF_CMAP_SINGLE, 18678 },
	{ 0xed1d, 0xed1d, PDF_CMAP_SINGLE, 16061 },
	{ 0xed1e, 0xed1e, PDF_CMAP_SINGLE, 18679 },
	{ 0xed22, 0xed22, PDF_CMAP_SINGLE, 18680 },
	{ 0xed24, 0xed25, PDF_CMAP_RANGE, 16066 },
	{ 0xed26, 0xed27, PDF_CMAP_RANGE, 18681 },
	{ 0xed29, 0xed29, PDF_CMAP_SINGLE, 16069 },
	{ 0xed2a, 0xed2a, PDF_CMAP_SINGLE, 18683 },
	{ 0xed2c, 0xed2c, PDF_CMAP_SINGLE, 16071 },
	{ 0xed2d, 0xed2d, PDF_CMAP_SINGLE, 18684 },
	{ 0xed3b, 0xed3b, PDF_CMAP_SINGLE, 16079 },
	{ 0xed3d, 0xed3d, PDF_CMAP_SINGLE, 18692 },
	{ 0xed41, 0xed42, PDF_CMAP_RANGE, 18694 },
	{ 0xed45, 0xed45, PDF_CMAP_SINGLE, 18696 },
	{ 0xed47, 0xed47, PDF_CMAP_SINGLE, 16085 },
	{ 0xed4a, 0xed4b, PDF_CMAP_RANGE, 18697 },
	{ 0xed4d, 0xed4e, PDF_CMAP_RANGE, 18699 },
	{ 0xed4f, 0xed4f, PDF_CMAP_SINGLE, 16090 },
	{ 0xed51, 0xed51, PDF_CMAP_SINGLE, 18701 },
	{ 0xed52, 0xed52, PDF_CMAP_SINGLE, 16092 },
	{ 0xed53, 0xed54, PDF_CMAP_RANGE, 18702 },
	{ 0xed57, 0xed58, PDF_CMAP_RANGE, 18704 },
	{ 0xed5f, 0xed5f, PDF_CMAP_SINGLE, 16101 },
	{ 0xed62, 0xed62, PDF_CMAP_SINGLE, 16104 },
	{ 0xed6b, 0xed6b, PDF_CMAP_SINGLE, 16113 },
	{ 0xed71, 0xed71, PDF_CMAP_SINGLE, 18706 },
	{ 0xed72, 0xed72, PDF_CMAP_SINGLE, 16119 },
	{ 0xed75, 0xed75, PDF_CMAP_SINGLE, 18707 },
	{ 0xed77, 0xed77, PDF_CMAP_SINGLE, 18709 },
	{ 0xed78, 0xed78, PDF_CMAP_SINGLE, 16122 },
	{ 0xed7a, 0xed7a, PDF_CMAP_SINGLE, 18710 },
	{ 0xed81, 0xed81, PDF_CMAP_SINGLE, 18711 },
	{ 0xed84, 0xed85, PDF_CMAP_RANGE, 16132 },
	{ 0xed86, 0xed86, PDF_CMAP_SINGLE, 18712 },
	{ 0xed88, 0xed88, PDF_CMAP_SINGLE, 16135 },
	{ 0xed89, 0xed8a, PDF_CMAP_RANGE, 18713 },
	{ 0xed8b, 0xed8b, PDF_CMAP_SINGLE, 16136 },
	{ 0xed8e, 0xed8e, PDF_CMAP_SINGLE, 16139 },
	{ 0xed91, 0xed91, PDF_CMAP_SINGLE, 16142 },
	{ 0xeda6, 0xeda6, PDF_CMAP_SINGLE, 18716 },
	{ 0xedc6, 0xedc6, PDF_CMAP_SINGLE, 16193 },
	{ 0xedcf, 0xedcf, PDF_CMAP_SINGLE, 16202 },
	{ 0xedd1, 0xedd1, PDF_CMAP_SINGLE, 18718 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 18719 },
	{ 0xedda, 0xedda, PDF_CMAP_SINGLE, 16211 },
	{ 0xeddb, 0xeddb, PDF_CMAP_SINGLE, 18720 },
	{ 0xeddd, 0xeddd, PDF_CMAP_SINGLE, 18721 },
	{ 0xede1, 0xede1, PDF_CMAP_SINGLE, 16216 },
	{ 0xeded, 0xeded, PDF_CMAP_SINGLE, 16228 },
	{ 0xee08, 0xee08, PDF_CMAP_SINGLE, 16254 },
	{ 0xee11, 0xee11, PDF_CMAP_SINGLE, 16263 },
	{ 0xee15, 0xee15, PDF_CMAP_SINGLE, 18725 },
	{ 0xee1b, 0xee1b, PDF_CMAP_SINGLE, 18726 },
	{ 0xee20, 0xee20, PDF_CMAP_SINGLE, 16276 },
	{ 0xee22, 0xee22, PDF_CMAP_SINGLE, 18727 },
	{ 0xee2a, 0xee2a, PDF_CMAP_SINGLE, 16285 },
	{ 0xee30, 0xee30, PDF_CMAP_SINGLE, 18728 },
	{ 0xee31, 0xee31, PDF_CMAP_SINGLE, 16291 },
	{ 0xee34, 0xee34, PDF_CMAP_SINGLE, 16294 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 18729 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 18730 },
	{ 0xee5b, 0xee5b, PDF_CMAP_SINGLE, 18731 },
	{ 0xee5d, 0xee5d, PDF_CMAP_SINGLE, 16332 },
	{ 0xee60, 0xee60, PDF_CMAP_SINGLE, 18732 },
	{ 0xee64, 0xee64, PDF_CMAP_SINGLE, 16338 },
	{ 0xee6a, 0xee6a, PDF_CMAP_SINGLE, 16344 },
	{ 0xee76, 0xee76, PDF_CMAP_SINGLE, 16356 },
	{ 0xee7c, 0xee7c, PDF_CMAP_SINGLE, 16362 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 18733 },
	{ 0xee91, 0xee91, PDF_CMAP_SINGLE, 16381 },
	{ 0xee9c, 0xee9c, PDF_CMAP_SINGLE, 18735 },
	{ 0xeea1, 0xeea6, PDF_CMAP_RANGE, 18737 },
	{ 0xeea7, 0xeea7, PDF_CMAP_SINGLE, 17332 },
	{ 0xeea8, 0xeeaa, PDF_CMAP_RANGE, 18743 },
	{ 0xeeac, 0xeeb1, PDF_CMAP_RANGE, 18746 },
	{ 0xeeb4, 0xeeb4, PDF_CMAP_SINGLE, 18754 },
	{ 0xeeb7, 0xeeb7, PDF_CMAP_SINGLE, 18757 },
	{ 0xf303, 0xf318, PDF_CMAP_RANGE, 17609 },
	{ 0xf325, 0xf325, PDF_CMAP_SINGLE, 18797 },
	{ 0xf327, 0xf327, PDF_CMAP_SINGLE, 18799 },
	{ 0xf344, 0xf344, PDF_CMAP_SINGLE, 18828 },
	{ 0xf346, 0xf346, PDF_CMAP_SINGLE, 18830 },
	{ 0xf34a, 0xf34b, PDF_CMAP_RANGE, 18844 },
	{ 0xf3a0, 0xf3a1, PDF_CMAP_RANGE, 17716 },
	{ 0xf3ac, 0xf3ac, PDF_CMAP_SINGLE, 17723 },
	{ 0xf3f0, 0xf3f0, PDF_CMAP_SINGLE, 17781 },
	{ 0xf3f9, 0xf3fa, PDF_CMAP_RANGE, 17788 },
	{ 0xf3fc, 0xf3fc, PDF_CMAP_SINGLE, 17791 },
	{ 0xf408, 0xf40b, PDF_CMAP_RANGE, 17801 },
	{ 0xf43d, 0xf43d, PDF_CMAP_SINGLE, 17845 },
	{ 0xf440, 0xf442, PDF_CMAP_RANGE, 17163 },
	{ 0xf444, 0xf444, PDF_CMAP_SINGLE, 17167 },
	{ 0xf446, 0xf446, PDF_CMAP_SINGLE, 17169 },
	{ 0xf448, 0xf449, PDF_CMAP_RANGE, 17171 },
	{ 0xf44b, 0xf44b, PDF_CMAP_SINGLE, 17174 },
	{ 0xf44d, 0xf44d, PDF_CMAP_SINGLE, 17176 },
	{ 0xf450, 0xf452, PDF_CMAP_RANGE, 17179 },
	{ 0xf454, 0xf456, PDF_CMAP_RANGE, 17183 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 17847 },
	{ 0xf45a, 0xf45a, PDF_CMAP_SINGLE, 17189 },
	{ 0xf45b, 0xf45b, PDF_CMAP_SINGLE, 17848 },
	{ 0xf45c, 0xf45f, PDF_CMAP_RANGE, 17191 },
	{ 0xf461, 0xf463, PDF_CMAP_RANGE, 17196 },
	{ 0xf465, 0xf466, PDF_CMAP_RANGE, 17200 },
	{ 0xf469, 0xf469, PDF_CMAP_SINGLE, 17204 },
	{ 0xf46c, 0xf46d, PDF_CMAP_RANGE, 17207 },
	{ 0xf46e, 0xf46e, PDF_CMAP_SINGLE, 17849 },
	{ 0xf46f, 0xf46f, PDF_CMAP_SINGLE, 17210 },
	{ 0xf474, 0xf474, PDF_CMAP_SINGLE, 17850 },
	{ 0xf477, 0xf477, PDF_CMAP_SINGLE, 17851 },
	{ 0xf478, 0xf478, PDF_CMAP_SINGLE, 17219 },
	{ 0xf479, 0xf479, PDF_CMAP_SINGLE, 17852 },
	{ 0xf47b, 0xf47b, PDF_CMAP_SINGLE, 17853 },
	{ 0xf47c, 0xf47e, PDF_CMAP_RANGE, 17223 },
	{ 0xf480, 0xf481, PDF_CMAP_RANGE, 17227 },
	{ 0xf483, 0xf483, PDF_CMAP_SINGLE, 17854 },
	{ 0xf484, 0xf485, PDF_CMAP_RANGE, 17231 },
	{ 0xf488, 0xf488, PDF_CMAP_SINGLE, 17235 },
	{ 0xf48b, 0xf48b, PDF_CMAP_SINGLE, 17238 },
	{ 0xf48d, 0xf490, PDF_CMAP_RANGE, 17240 },
	{ 0xf491, 0xf491, PDF_CMAP_SINGLE, 17855 },
	{ 0xf497, 0xf498, PDF_CMAP_RANGE, 17250 },
	{ 0xf49b, 0xf49b, PDF_CMAP_SINGLE, 17254 },
	{ 0xf49e, 0xf49e, PDF_CMAP_SINGLE, 17257 },
	{ 0xf4a4, 0xf4a6, PDF_CMAP_RANGE, 17263 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 17857 },
	{ 0xf4a9, 0xf4aa, PDF_CMAP_RANGE, 17268 },
	{ 0xf4ad, 0xf4ad, PDF_CMAP_SINGLE, 17272 },
	{ 0xf4af, 0xf4b0, PDF_CMAP_RANGE, 17274 },
	{ 0xf4b2, 0xf4b2, PDF_CMAP_SINGLE, 17859 },
	{ 0xf4b3, 0xf4b4, PDF_CMAP_RANGE, 17278 },
	{ 0xf4b7, 0xf4b7, PDF_CMAP_SINGLE, 17282 },
	{ 0xf4ba, 0xf4c0, PDF_CMAP_RANGE, 17285 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 17860 },
	{ 0xf4c3, 0xf4c5, PDF_CMAP_RANGE, 17861 },
	{ 0xf4c7, 0xf4ca, PDF_CMAP_RANGE, 17865 },
	{ 0xf4cc, 0xf4cf, PDF_CMAP_RANGE, 17870 },
	{ 0xf4d1, 0xf4d5, PDF_CMAP_RANGE, 17874 },
	{ 0xf4d7, 0xf4d7, PDF_CMAP_SINGLE, 17880 },
	{ 0xf4d9, 0xf4da, PDF_CMAP_RANGE, 17882 },
	{ 0xf4db, 0xf4dc, PDF_CMAP_RANGE, 17292 },
	{ 0xf4dd, 0xf4de, PDF_CMAP_RANGE, 17884 },
	{ 0xf4df, 0xf4df, PDF_CMAP_SINGLE, 17294 },
	{ 0xf4e0, 0xf4e0, PDF_CMAP_SINGLE, 17886 },
	{ 0xf4e2, 0xf4e2, PDF_CMAP_SINGLE, 17887 },
	{ 0xf4e3, 0xf4e3, PDF_CMAP_SINGLE, 17296 },
	{ 0xf4e4, 0xf4e4, PDF_CMAP_SINGLE, 17888 },
	{ 0xf4e5, 0xf4e5, PDF_CMAP_SINGLE, 17297 },
	{ 0xf4e6, 0xf4e6, PDF_CMAP_SINGLE, 17889 },
	{ 0xf4e7, 0xf4e9, PDF_CMAP_RANGE, 17298 },
	{ 0xf4ef, 0xf4ef, PDF_CMAP_SINGLE, 17893 },
	{ 0xf4f2, 0xf4f2, PDF_CMAP_SINGLE, 17303 },
	{ 0xf4f3, 0xf4f3, PDF_CMAP_SINGLE, 17896 },
	{ 0xf4f8, 0xf4f8, PDF_CMAP_SINGLE, 17899 },
	{ 0xf4fc, 0xf4fc, PDF_CMAP_SINGLE, 17902 },
	{ 0xf4fe, 0xf4ff, PDF_CMAP_RANGE, 17904 },
	{ 0xf500, 0xf501, PDF_CMAP_RANGE, 17906 },
	{ 0xf503, 0xf503, PDF_CMAP_SINGLE, 17908 },
	{ 0xf505, 0xf505, PDF_CMAP_SINGLE, 17910 },
	{ 0xf507, 0xf511, PDF_CMAP_RANGE, 17912 },
	{ 0xf513, 0xf516, PDF_CMAP_RANGE, 17924 },
	{ 0xf518, 0xf51d, PDF_CMAP_RANGE, 17929 },
	{ 0xf51f, 0xf526, PDF_CMAP_RANGE, 17936 },
	{ 0xf52a, 0xf52a, PDF_CMAP_SINGLE, 17947 },
	{ 0xf533, 0xf534, PDF_CMAP_RANGE, 17956 },
	{ 0xf53a, 0xf53b, PDF_CMAP_RANGE, 17633 },
	{ 0xf53d, 0xf53d, PDF_CMAP_SINGLE, 17636 },
	{ 0xf54e, 0xf54e, PDF_CMAP_SINGLE, 17653 },
	{ 0xf553, 0xf553, PDF_CMAP_SINGLE, 17658 },
	{ 0xf556, 0xf556, PDF_CMAP_SINGLE, 17660 },
	{ 0xf559, 0xf55a, PDF_CMAP_RANGE, 17663 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 17668 },
	{ 0xf565, 0xf565, PDF_CMAP_SINGLE, 17675 },
	{ 0xf56b, 0xf56b, PDF_CMAP_SINGLE, 17681 },
	{ 0xf572, 0xf572, PDF_CMAP_SINGLE, 17688 },
	{ 0xf694, 0xf694, PDF_CMAP_SINGLE, 18044 },
	{ 0xf69e, 0xf69e, PDF_CMAP_SINGLE, 18054 },
	{ 0xf6ad, 0xf6ad, PDF_CMAP_SINGLE, 18068 },
	{ 0xf6b0, 0xf6b0, PDF_CMAP_SINGLE, 18071 },
	{ 0xf7e5, 0xf7e6, PDF_CMAP_RANGE, 13999 },
	{ 0xf7e8, 0xf7e8, PDF_CMAP_SINGLE, 14002 },
	{ 0xf7ea, 0xf7eb, PDF_CMAP_RANGE, 14004 },
	{ 0xf7ed, 0xf7ee, PDF_CMAP_RANGE, 14007 },
	{ 0xfa0c, 0xfa0c, PDF_CMAP_SINGLE, 628 },
	{ 0xfa0d, 0xfa0d, PDF_CMAP_SINGLE, 9089 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 109 },
	{ 0xfe31, 0xfe31, PDF_CMAP_SINGLE, 122 },
	{ 0xfe33, 0xfe33, PDF_CMAP_SINGLE, 13743 },
	{ 0xfe34, 0xfe34, PDF_CMAP_SINGLE, 13745 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 130 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 134 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 138 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 142 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 146 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 150 },
	{ 0xfe41, 0xfe42, PDF_CMAP_RANGE, 154 },
	{ 0xfe43, 0xfe44, PDF_CMAP_RANGE, 158 },
	{ 0xfe49, 0xfe4a, PDF_CMAP_RANGE, 199 },
	{ 0xfe4b, 0xfe4c, PDF_CMAP_RANGE, 203 },
	{ 0xfe4d, 0xfe4e, PDF_CMAP_RANGE, 201 },
	{ 0xfe4f, 0xfe4f, PDF_CMAP_SINGLE, 13746 },
	{ 0xfe50, 0xfe50, PDF_CMAP_SINGLE, 112 },
	{ 0xfe52, 0xfe52, PDF_CMAP_SINGLE, 114 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 116 },
	{ 0xfe59, 0xfe5e, PDF_CMAP_RANGE, 160 },
	{ 0xfe5f, 0xfe61, PDF_CMAP_RANGE, 205 },
	{ 0xfe62, 0xfe66, PDF_CMAP_RANGE, 223 },
	{ 0xfe69, 0xfe6b, PDF_CMAP_RANGE, 268 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 108 },
	{ 0xff02, 0xff02, PDF_CMAP_SINGLE, 14052 },
	{ 0xff03, 0xff03, PDF_CMAP_SINGLE, 174 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 259 },
	{ 0xff05, 0xff05, PDF_CMAP_SINGLE, 264 },
	{ 0xff06, 0xff06, PDF_CMAP_SINGLE, 175 },
	{ 0xff07, 0xff07, PDF_CMAP_SINGLE, 14051 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 128 },
	{ 0xff0a, 0xff0a, PDF_CMAP_SINGLE, 176 },
	{ 0xff0b, 0xff0b, PDF_CMAP_SINGLE, 208 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 100 },
	{ 0xff0d, 0xff0d, PDF_CMAP_SINGLE, 209 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 103 },
	{ 0xff0f, 0xff0f, PDF_CMAP_SINGLE, 257 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 333 },
	{ 0xff1a, 0xff1a, PDF_CMAP_SINGLE, 106 },
	{ 0xff1b, 0xff1b, PDF_CMAP_SINGLE, 105 },
	{ 0xff1c, 0xff1c, PDF_CMAP_SINGLE, 214 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 216 },
	{ 0xff1e, 0xff1e, PDF_CMAP_SINGLE, 215 },
	{ 0xff1f, 0xff1f, PDF_CMAP_SINGLE, 107 },
	{ 0xff20, 0xff20, PDF_CMAP_SINGLE, 265 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 365 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 13758 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 258 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 13759 },
	{ 0xff3e, 0xff3e, PDF_CMAP_SINGLE, 13748 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 197 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 391 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 132 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 120 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 133 },
	{ 0xff64, 0xff64, PDF_CMAP_SINGLE, 113 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 14049 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 14050 },
};

static const unsigned short pdf_cmap_UniCNS_UCS2_H_table[13509] =
{
	178,13747,280,212,18788,18786,18792,18790,
	18801,18796,18794,18805,18803,18810,18808,18832,
	18814,18812,18818,18816,211,18840,18822,18820,
	18785,18802,18789,18807,18791,18809,18841,18793,
	18815,18787,18804,18813,18795,18817,18835,18806,
	13748,504,503,505,18798,18829,18800,18831,
	121,123,111,110,248,245,247,246,
	252,251,220,233,232,240,239,219,
	222,322,14090,14072,14081,14063,14074,14083,
	14065,14078,14087,14069,14080,14089,14071,323,
	14084,14066,325,14086,14068,14073,14082,14064,
	14079,14088,14070,324,14085,14067,321,320,
	304,303,302,301,300,299,298,14096,
	310,313,190,189,183,182,192,191,
	188,187,184,181,329,328,186,185,
	241,243,242,17700,732,236,235,17815,
	18083,16075,17273,15149,17840,18070,17959,16236,
	18438,14342,16403,15583,16589,16613,14597,17046,
	15674,18464,14904,15283,14716,15581,16918,16636,
	14713,14720,15500,15671,16646,14743,17976,15592,
	16784,17295,14942,16095,16796,17203,18006,18005,
	18007,14426,18022,16883,14958,15594,18057,18654,
	16740,15727,15091,14745,16989,17963,15380,17156,
	18157,16501,14420,18162,15533,17055,15528,18166,
	15069,18168,15093,15373,15081,16521,17879,17236,
	17367,16988,15598,16870,18250,15219,15074,14317,
	16847,15217,15602,16052,18079,18275,15076,16563,
	15264,15866,18321,18765,15382,18368,14658,15396,
	15395,14676,595,597,598,18686,6001,617,
	615,618,616,6002,660,6010,6008,659,
	658,754,753,752,755,751,18685,16215,
	619,17651,661,6011,662,6030,1045,6388,
	537,17687,620,663,756,6031,17719,13999,
	5996,599,6003,623,16526,757,759,758,
	2097,596,14001,17637,5997,600,625,624,
	882,15845,2555,7733,17806,17340,3518,539,
	601,17807,666,17820,1323,602,6004,626,
	667,669,6012,670,668,14788,883,18645,
	1324,1325,1699,540,627,671,884,886,
	885,9059,16700,13175,603,17638,14769,17808,
	673,672,6013,674,680,6015,6014,677,
	16226,761,768,6037,6033,6036,769,785,
	6032,6034,18635,17809,899,6074,896,900,
	6084,6072,897,6076,14765,887,6075,902,
	6079,14792,6083,6080,6077,888,890,903,
	892,6069,895,6085,6082,6073,891,6070,
	889,17724,17725,14782,6071,6191,1068,1055,
	14653,6192,1052,1070,17810,1060,16229,1059,
	6180,1063,1058,6185,6193,6189,14795,1061,
	15247,1064,1049,1073,6182,14772,6400,1047,
	1069,1048,6187,6183,1062,1054,6179,1050,
	6190,1071,1074,1053,1066,1072,1051,6188,
	1067,6194,6181,1065,6184,1341,6402,6414,
	1333,6415,1328,1338,15474,1331,6396,1339,
	6395,6399,6406,6404,1347,1342,6390,6392,
	1344,1332,6393,6405,6412,1337,6398,6391,
	1336,6394,1340,6397,14773,1335,1345,6407,
	1346,6411,6409,6413,1343,6401,6389,6403,
	1334,6408,1329,6410,1716,1704,15690,6749,
	6757,1703,1711,1723,6763,6761,6756,1705,
	6762,6752,1719,1710,1718,6747,6759,18152,
	6750,1714,6751,14799,6746,1721,1708,1717,
	1707,6748,6753,6760,1715,1712,6755,1720,
	6758,6754,1709,1722,1713,1706,1702,15462,
	14800,2124,2102,2118,16168,2123,7204,7203,
	2105,14259,2114,16166,2127,7197,2108,7198,
	16420,2129,7217,2572,2113,2112,7199,2116,
	7749,2107,7210,2122,2121,2111,7202,7211,
	7196,2110,7212,2119,7200,2101,2104,2103,
	7213,2117,2106,2126,2128,7207,2125,14796,
	7215,7201,7216,7205,14608,7214,7757,7745,
	7750,7756,2559,7743,2563,7747,7740,2561,
	7744,2566,2571,2567,16172,7760,14801,2562,
	7746,2558,7741,7737,7751,7738,7736,7748,
	2564,15817,8376,7752,7735,7753,14797,2574,
	7758,2573,7734,7742,7759,2569,2568,2565,
	2570,3052,14798,8379,8372,8380,17308,8370,
	8373,3048,8374,16876,3051,8378,8368,8367,
	3053,3054,3050,3055,8366,8377,8375,8369,
	3047,8371,3525,3519,9063,3528,9069,9072,
	9066,3520,9075,3526,9076,9071,3527,9060,
	3524,9061,9070,9068,9064,3523,9062,3529,
	9067,9074,9065,9073,14789,3976,3973,14545,
	3974,9756,9748,3972,3969,9749,16334,3971,
	9752,9754,16433,15101,9761,16753,9760,9755,
	3968,9747,3966,9750,3977,9753,9757,3970,
	3967,9759,9758,3975,9751,15593,4357,10371,
	10368,4358,4356,14802,10375,10372,10365,4355,
	16434,4359,10366,4354,4362,14790,16436,4749,
	11012,4746,11009,4748,4750,11010,4747,5045,
	11582,5046,11580,11581,11579,11578,11583,5044,
	14766,10367,12046,5047,5493,12998,12997,5820,
	5819,13326,604,628,682,681,771,770,
	906,905,907,904,1076,1075,1077,1348,
	1350,17306,1724,281,7218,282,2575,284,
	283,8381,16442,285,3978,286,683,908,
	1351,16444,606,686,17727,1078,1353,1352,
	1354,15054,17311,6009,14006,1726,1725,7220,
	7219,2576,542,687,6016,16038,14810,4752,
	543,774,911,6086,16456,16465,15740,16402,
	16457,6196,6417,1355,6418,2136,16938,7224,
	16459,7221,2137,2135,2134,7225,16460,7762,
	16461,15789,16463,8382,9077,9762,17818,4363,
	4753,11013,16975,607,621,16466,2577,3057,
	3979,16760,5998,688,17819,1356,16138,14003,
	629,6039,778,16473,6038,916,912,6197,
	1675,6198,16224,18736,6199,1082,1081,16474,
	17314,1085,1361,6423,16123,6419,1363,1359,
	1358,1360,1357,16475,1729,6765,6422,1734,
	6766,1730,1733,1732,1731,1728,2140,7231,
	2138,16477,14820,7227,2141,2139,2142,16169,
	7226,7232,17945,3061,2578,7763,7765,7764,
	7766,2579,3058,16195,3531,9079,17315,9078,
	3981,3980,10102,15807,17821,4754,4755,16478,
	12412,13000,12999,13327,16232,610,780,779,
	17730,917,14821,1086,1089,1366,6424,6767,
	1738,6768,1737,16097,7233,16649,16170,2580,
	7768,2583,7767,2767,2582,2581,3063,3062,
	3533,16483,14823,10380,4369,10379,4756,11584,
	5048,12413,5641,544,630,692,6017,918,
	6202,1739,7234,2585,2584,8384,8383,611,
	695,783,2586,5999,6040,784,6091,920,
	919,6090,1090,16182,7769,3538,9767,3982,
	16336,16493,11014,12743,545,696,17732,6427,
	6769,2589,612,18759,631,363,699,698,
	697,787,786,6041,6092,17733,1370,1367,
	1369,1368,613,17665,700,17319,789,788,
	6203,18442,1371,16077,6018,16506,791,790,
	6204,1091,16507,1092,1741,6770,18689,16509,
	2144,6428,16514,6429,6772,6771,6774,6773,
	1742,7770,2146,7235,2145,16516,8385,3066,
	16699,8386,16702,17824,16326,16517,9768,3983,
	16518,16523,17318,2590,614,632,703,702,
	14518,704,14842,17734,15933,1375,1377,17735,
	16528,16185,1743,2147,16531,11015,5321,633,
	794,809,804,799,798,805,803,796,
	810,797,793,808,807,806,795,802,
	16534,14344,801,811,800,934,930,936,
	6094,933,923,926,929,925,932,935,
	924,927,931,937,16536,15479,6215,6212,
	15972,1105,6213,1093,1095,1119,1114,16899,
	17827,6220,6214,1097,1099,6219,1106,6209,
	1118,1120,1094,1111,16556,6207,1117,15989,
	1102,6208,1110,1108,1109,1115,6216,1096,
	1116,6218,1104,1101,6211,1100,6221,1103,
	1107,16081,16039,1098,6217,6446,6445,1395,
	6448,6450,6439,6442,6449,1396,16178,15747,
	6436,6440,16541,6444,6789,1391,6793,1378,
	6441,1379,1392,1385,1381,6437,1384,1389,
	1398,6438,6431,1383,6432,6434,1386,18137,
	1388,6430,6435,1397,1393,6443,1399,16545,
	1390,6433,1387,17831,16024,1382,1380,16010,
	6447,6788,6776,6791,17832,16546,6778,1750,
	1764,1746,1763,1755,1760,1744,6777,6783,
	1759,6795,1761,14148,1751,17833,6785,6782,
	1749,17834,6775,1762,6790,1754,6792,1765,
	1745,1756,1753,6780,1757,1752,1758,1748,
	14755,17320,14848,1747,6779,17835,7255,2160,
	7237,17217,7242,2153,2164,7240,2148,2158,
	2163,7247,2159,2162,17836,7250,2154,7241,
	2156,7252,2152,2167,2150,15564,7257,7245,
	7249,2155,2166,7246,2161,7251,7258,7785,
	14963,7256,2168,2149,7248,7239,16041,2157,
	7238,17837,17161,7236,2165,17839,7777,16538,
	2609,7791,2607,2604,7782,2600,7786,2611,
	7776,7781,7780,2151,2606,7789,7792,7773,
	3084,17321,7793,2612,2598,2595,7784,2592,
	16554,7790,16488,2599,7794,7774,7788,2602,
	7775,7778,7783,17326,2603,2601,2613,2608,
	16544,2596,2748,2597,7779,2610,2605,7787,
	2594,16031,2593,15150,15302,16011,16164,3067,
	3070,8402,3090,8405,3068,8398,3074,3080,
	3355,16558,3078,8395,3091,3071,3079,8403,
	9095,8408,8396,17166,8387,8400,8394,3077,
	8406,3073,3093,3086,3075,3072,8407,8393,
	8399,8401,8389,8404,3069,8388,16559,3076,
	3092,3088,8390,3082,14581,3089,3085,3081,
	14846,8397,17322,16009,3087,15811,9093,9089,
	15951,9104,9082,9091,3545,9100,3555,9087,
	9086,9084,9102,3543,9096,9085,3546,9094,
	3541,9090,9097,9099,17843,17323,9103,3550,
	9083,3544,9088,16020,3539,3551,9098,3554,
	3542,287,3540,9092,15996,3549,16008,16003,
	9101,18310,3999,3994,9783,9779,17958,9773,
	3988,3984,9782,3985,9776,9778,9781,16002,
	3990,3997,3991,9774,3998,16205,9772,9777,
	3989,9770,3995,3987,3986,9775,9780,15976,
	3996,16253,17325,15975,18183,4377,10397,10394,
	15971,4371,4374,10392,4376,10384,4382,16567,
	10396,4373,10398,4372,10393,10395,4375,10391,
	10385,10382,15998,8392,10388,10381,10387,10386,
	10383,16572,4379,16573,18383,17858,4378,17327,
	4380,10389,4757,10390,16121,11024,11017,14354,
	4769,11021,4761,4765,11020,4764,4760,4763,
	4758,4768,11022,11018,4767,11016,4766,11023,
	11019,4381,4770,11025,4762,4759,15749,16055,
	16109,11591,11593,5050,11594,11592,11590,11589,
	5052,11588,5053,17960,14665,15948,5049,5054,
	5051,14758,11585,12051,12049,12054,12052,12050,
	12053,14756,14664,15560,5494,12419,12415,12420,
	12416,5323,17911,17983,12744,12746,5644,12745,
	5643,5642,15965,5739,5738,5740,13001,13177,
	13176,5821,5823,5822,13178,5878,13441,13440,
	13513,18570,6005,813,812,940,939,6096,
	938,6095,17736,1123,6224,1121,1124,16580,
	6222,16579,1122,16575,6452,1400,1766,16578,
	7796,2614,2616,7795,2615,8409,3094,9105,
	15926,4001,10399,11027,11026,14850,13573,634,
	6020,943,947,6098,945,944,6097,946,
	942,15097,941,6099,1131,18517,1127,6225,
	16588,16606,6226,16584,1129,6228,1125,6229,
	6227,1128,1130,1133,1132,1126,6230,1405,
	6465,1406,16171,6466,1404,1403,6456,6453,
	6458,6457,6454,6461,6459,1402,16223,6462,
	1408,6467,18140,14855,6460,1767,1401,15483,
	1768,6800,14854,6806,16742,1774,6804,6810,
	6801,6805,6807,6809,6803,17928,6802,1769,
	18532,1771,1770,6799,6808,18241,6796,7270,
	7269,7274,7265,15481,7268,7267,14124,7271,
	7275,2171,2174,15195,7266,7272,15627,2173,
	7261,1772,7810,7273,2172,7263,7801,14853,
	16129,2617,2621,7817,7799,7813,2622,7815,
	16591,7821,7816,7804,7812,18260,7826,7824,
	7814,7809,7802,8417,7800,2626,7807,2627,
	2623,7797,7819,7805,16527,7803,7822,2624,
	16593,7828,2618,2620,7811,7806,2175,2619,
	7808,7823,7827,7818,8421,8420,3102,8413,
	3103,3101,15400,8415,3098,8419,16596,8414,
	8416,8410,3096,8429,8428,8425,3095,7262,
	8422,3097,2625,8424,8411,8427,8426,15730,
	8430,8423,15746,15042,8418,9111,3567,3570,
	3565,9110,9114,9109,3559,3569,9106,3563,
	9113,3561,3560,9116,3562,9118,18761,9115,
	3558,17375,3568,9120,9108,9117,9107,15717,
	3564,14746,3566,9121,15398,17237,9791,4002,
	9797,4007,9793,9800,9785,4009,4003,9790,
	4385,9789,9798,4004,4008,9788,9794,9799,
	9784,4006,9792,9796,9802,9786,9795,4005,
	9801,9787,17944,18691,4389,10401,4387,4386,
	10403,10408,10407,4392,16107,4744,4391,16599,
	10400,10406,4390,10405,14244,10402,18496,15869,
	11032,14059,11034,11029,4772,11031,4771,11033,
	4774,11035,4773,11028,11030,5058,11597,5057,
	11598,5056,11596,5055,11595,5325,5324,12421,
	12423,16602,12422,5498,12747,5646,13329,13328,
	5920,635,705,1135,17737,15720,15482,6811,
	16604,9122,4010,9803,10409,548,6021,17738,
	17817,6231,17739,6468,6812,7276,2176,17012,
	12424,5741,636,814,6044,16611,2628,4013,
	4011,706,708,707,6022,709,815,6045,
	816,17740,1413,6469,1410,7277,1778,1777,
	1779,1776,6813,1414,1775,3106,8431,2629,
	4015,4014,9804,4393,4775,12055,13179,13330,
	638,817,15978,818,6106,953,956,6104,
	6102,955,6105,6107,6101,958,957,954,
	952,6103,15664,1147,1144,6237,6240,6239,
	1138,1146,15513,1143,6236,6234,1142,15280,
	1137,1140,15791,6235,6242,6238,1141,1145,
	1148,6233,6241,1139,1419,1427,15576,6481,
	6474,1428,6485,6470,6477,16173,14864,1418,
	6471,1416,6478,6482,1415,6483,6476,6479,
	15620,1430,1421,6486,6484,16620,1426,1424,
	6475,1423,6473,6472,1422,1420,1429,1425,
	1417,1781,16625,1788,6827,1780,6819,6815,
	6831,6814,1783,6824,1786,1789,1784,6828,
	1787,15705,2186,6834,6816,15672,15282,6818,
	6825,6829,6833,6830,6823,6826,15946,16632,
	6820,1791,6822,6821,6832,1782,6817,1790,
	15245,1785,2192,7285,15497,2191,15160,2180,
	14867,2185,7282,7279,7284,2181,7278,2184,
	2182,7286,2183,2187,14875,2188,16616,2190,
	2189,15163,15680,14863,7836,2630,8445,7835,
	7854,2636,15040,7852,2635,2631,7861,7845,
	7848,14615,2639,7858,7850,2632,2640,7855,
	7853,7838,7859,7847,7842,17923,7832,7860,
	7844,7831,15156,2638,7849,7862,7846,7834,
	7839,7830,15667,2637,16635,7843,7840,2633,
	7833,14775,7857,2634,14914,7841,7837,15679,
	7856,15668,3107,8438,15585,8435,8452,8440,
	8453,14865,3109,14877,15665,8449,8446,8447,
	8450,8454,7851,8456,9142,16949,3110,8457,
	8433,8443,16640,8448,14858,3108,3111,8455,
	8458,8437,8434,16754,15580,8436,8441,8439,
	3112,16634,8442,8444,8432,9130,9128,3580,
	3578,9138,9129,9139,9135,9127,9141,9126,
	9133,3577,3576,3575,9131,9136,3572,3579,
	14486,9124,9134,9123,9132,3573,9137,9125,
	3574,9140,15496,17333,9808,4020,4019,4021,
	9820,9810,9816,9805,9819,9818,9822,9815,
	4016,9814,4022,15238,9807,4017,9821,4018,
	9809,9812,9817,9811,9806,14325,9813,10411,
	4397,10414,10413,10416,10418,14062,4395,10412,
	15389,10410,10419,10417,10415,10420,4399,4394,
	4396,4398,18687,10422,11042,15350,11041,11040,
	11043,11036,11037,11045,11038,4776,11047,18753,
	11046,11039,11602,5061,11600,11605,11604,11044,
	5060,15236,11603,11599,11607,11606,5059,5647,
	14536,5648,14916,12748,17074,13005,13004,13331,
	13442,14897,641,710,819,6108,1150,1149,
	6487,1433,1432,6488,17742,17949,1792,2193,
	7288,15967,7864,2641,3114,7863,3113,14130,
	4023,16650,9823,4778,5062,11608,16652,5649,
	5824,549,6046,16452,820,6047,963,1154,
	1153,6243,1155,18665,6244,6489,1434,1436,
	1438,1435,1439,1437,16662,1796,1793,1795,
	1797,1794,7289,6835,7291,7290,2199,2196,
	2198,2200,2197,2202,2201,2646,7866,7865,
	2645,16165,2644,2643,2647,2642,8464,8461,
	3116,8460,8465,3118,8463,3115,3117,8462,
	16657,9143,17083,4024,4032,9824,4026,4030,
	9825,4031,4025,4029,4402,8459,4403,4401,
	4400,11048,4779,5499,5650,7292,2203,16664,
	2650,2649,2648,8466,4033,4780,643,6023,
	711,6048,16666,16668,6109,966,14889,9146,
	15753,644,17640,712,6110,8467,3121,5063,
	645,665,713,6049,821,1160,1159,6492,
	1441,16671,16220,1802,6836,1801,2206,2204,
	16130,7294,2205,7293,2194,7867,4034,9826,
	10423,5064,5742,13443,6006,714,6051,6050,
	16673,6254,6249,6251,6253,6250,1164,6247,
	6252,6255,1163,6256,18060,17745,18778,6506,
	14890,6500,6495,1445,6503,6501,6494,6507,
	6510,6505,6498,1447,6504,1448,6499,6502,
	6493,6497,6509,1449,6496,6508,1444,1446,
	17336,15692,6853,6846,6852,6845,6847,6841,
	6839,6854,6851,6837,16147,1804,6848,6840,
	6838,1803,6844,6842,6843,6855,2211,16674,
	2210,7295,2207,7297,16675,2212,7298,2213,
	7299,6856,7301,7296,7300,2214,2655,2654,
	7869,7873,7875,2656,7878,16679,2660,7880,
	2662,17337,2658,2666,16680,2663,7871,2657,
	7870,7868,7882,7872,2659,7881,2664,7877,
	7876,2665,7874,2661,7883,18131,7879,8482,
	8489,8478,3124,8474,8490,8468,8487,8485,
	8479,8488,8481,17961,8473,8491,8471,8484,
	8469,8492,16681,3125,17962,8486,9150,8472,
	3122,3123,8475,8480,8477,17338,16330,9157,
	9149,9156,9155,3581,8470,9153,3582,18148,
	9148,9152,9840,9831,9838,9829,9837,9841,
	9828,9832,9827,4035,15924,9830,4036,9836,
	9835,9834,16676,9839,16140,10427,10429,4407,
	10430,10424,10434,10432,4406,10435,10426,10431,
	10433,10428,11052,11057,11051,11056,15929,11049,
	11059,11055,11060,11054,11050,11058,11061,11053,
	16728,11611,5068,17966,5066,5065,5067,12059,
	15359,12323,12428,13183,5826,5825,13182,5879,
	16684,13332,647,968,6115,6257,1306,2667,
	648,824,823,822,715,16225,3126,652,
	6024,16704,6258,15754,1166,1456,16174,6512,
	1454,1453,6511,1451,6513,1452,1455,1808,
	6861,6862,1807,7303,7302,2217,16693,2670,
	7885,2669,2671,2668,3128,7884,3129,8494,
	17312,3130,8493,3127,9161,9163,3583,9162,
	9160,9159,9844,4041,4039,4037,10439,10437,
	14891,4409,10438,4410,4408,4038,10436,11613,
	5069,11612,11065,12429,653,827,971,6116,
	970,1457,3584,551,716,828,1809,3131,
	552,6053,6054,16648,6117,17746,1168,6263,
	6261,1169,6260,6262,6264,1458,6866,1460,
	6865,6867,6864,6868,1811,2221,7304,7306,
	2219,7307,2220,7305,6863,7886,7887,2675,
	2674,17341,2676,3135,9164,9166,9168,3586,
	3585,3132,9167,9165,14896,9848,16701,9846,
	9852,4042,9853,9850,4043,9847,9845,9851,
	4412,10441,9849,4414,10442,4413,4416,10443,
	4411,4415,11070,11068,11067,11069,11066,13008,
	13184,5954,553,1463,1170,14447,16707,1812,
	16708,16710,654,717,829,6118,1171,6265,
	6869,1813,4044,655,14910,16713,14911,972,
	3587,656,831,830,6119,973,6514,1466,
	1814,6870,7309,2222,2677,7891,2678,7892,
	9855,9854,4417,10444,4781,11071,5070,15269,
	5827,13334,554,17641,6871,2679,8495,3588,
	15706,5327,555,1174,16722,1173,1815,2681,
	2953,2682,2680,3137,9856,4046,4418,16724,
	6007,6120,6268,1175,6267,1176,16472,1470,
	6521,6520,1469,6519,17969,1817,6872,1820,
	1816,1822,2225,2224,2223,7894,2683,2686,
	2684,7893,2689,2685,7895,2688,2687,16428,
	8497,8496,16727,3140,8498,3589,9171,3590,
	9170,15768,10445,4420,9857,4419,4047,16287,
	11073,11072,5071,11614,12430,720,17642,832,
	1178,1180,6123,6271,6270,6269,6122,6121,
	975,1179,1177,974,1471,6522,16815,1472,
	6276,6278,6277,6523,1184,1182,6272,6274,
	6280,6275,6284,1474,6281,1183,6279,6282,
	1475,6283,6543,6530,6528,6542,6537,1827,
	1476,6538,6541,1823,6540,1477,1482,1480,
	6531,6526,6533,1487,6544,1824,6534,1825,
	1483,6536,16773,6887,1826,6525,1484,1828,
	1485,1481,1486,6535,6524,6539,1478,16737,
	6527,6532,6529,1479,6884,7314,6885,1834,
	6888,6878,1833,6880,6881,16500,6883,1829,
	14060,6879,6875,2230,6889,2226,7312,6882,
	7311,6877,6886,1832,2227,1838,2228,6890,
	7313,1831,2232,1837,1836,1835,2231,6891,
	2233,1830,14919,2690,7317,7319,7321,2234,
	2240,7898,7325,7316,2692,7896,2239,2237,
	7327,2238,7322,2241,7324,2236,7323,7326,
	7320,16746,2235,2693,7315,2691,16385,15068,
	2694,7900,7909,3143,16748,2696,2701,3144,
	7911,2708,8500,7901,2700,2703,2698,7899,
	7913,7916,8515,7914,8504,2699,2706,2710,
	7908,8499,7912,2695,8501,7915,8503,7904,
	17973,2704,7906,2709,7910,2702,7907,15820,
	2707,3145,3142,8502,15733,7905,2697,15748,
	15739,16393,3150,3154,8506,3595,3152,8510,
	3156,9172,8512,3597,3148,3151,8513,16296,
	8514,8520,3158,3598,8516,8521,8509,3608,
	15413,3599,3157,8507,8522,15495,3607,3155,
	3592,8519,3159,8511,8505,3149,8508,8517,
	16118,3591,3596,3146,8518,3606,9859,9183,
	9175,9858,9179,9181,16183,9177,3609,17974,
	3604,4049,9184,9860,16756,3602,9176,9180,
	4048,3593,4050,3601,3603,3600,9182,15410,
	9870,4425,9877,4056,16314,4055,9872,16774,
	4424,9861,4054,18764,9876,16755,9873,10462,
	4422,3153,9875,4429,9867,4423,16320,4428,
	9862,9866,9863,9869,4057,4421,4051,10448,
	9871,4427,16317,4430,9868,16759,4426,10447,
	10452,4785,10446,11085,10461,4434,4432,4783,
	10458,10454,4438,15840,11076,14933,14932,4436,
	10453,16386,11074,10460,10451,4431,11075,4784,
	10459,4433,4435,10456,4439,10455,10450,10449,
	4782,10463,11078,11616,4787,17977,11084,17343,
	11082,4788,11083,14928,11080,5073,11615,11077,
	11079,5074,4790,5072,4789,5076,11081,4786,
	17978,16769,11623,12063,11619,16325,5328,11621,
	11620,5075,11618,11622,12071,12067,12069,12068,
	12066,5502,5505,5504,5503,5652,12755,5653,
	12431,5744,13009,5745,5828,5880,13335,13619,
	721,6056,833,979,1186,1185,6545,1489,
	1488,3160,9185,9187,9189,9188,9878,4058,
	9880,17980,10464,4440,5077,5329,5078,722,
	17690,16322,6285,6546,1491,1490,1492,1839,
	7329,2242,2713,3161,8523,723,17644,657,
	724,6057,837,836,982,6130,6132,981,
	6124,6131,6127,6129,980,6126,6133,6128,
	1193,1202,1200,6295,6292,6300,1198,6301,
	1191,6298,1197,6294,6293,1195,6299,1196,
	1493,1190,6296,16262,6286,1187,16208,1206,
	6291,6297,1192,1194,17344,1205,1199,1204,
	6290,1203,1189,1188,1201,1507,6558,6551,
	1502,1521,6547,6554,6556,6559,1516,6555,
	6548,1514,6552,6560,1499,6557,1509,1508,
	6550,1497,1498,15759,1496,16778,1520,1512,
	1506,1494,6553,1505,1495,1513,1522,6894,
	1510,6549,1500,1517,1511,1515,1501,1840,
	6900,1852,1844,1846,1851,6912,1849,7330,
	2243,1854,6897,6905,1850,6904,6901,6909,
	6911,1843,1847,1853,2245,6906,1845,1856,
	6899,1848,2244,1842,6896,6903,6895,6898,
	6902,7331,1855,6910,1841,2259,7343,7333,
	7349,16093,2248,14949,7917,7351,7345,16150,
	7336,14947,7354,7339,2254,7940,7342,2256,
	2247,7356,7344,2250,7337,2251,7350,7357,
	2253,7341,7340,2261,2260,2246,2252,2255,
	7353,7347,2249,7332,7355,7346,7348,7352,
	16327,7918,2721,2741,2740,2729,16782,7936,
	7932,7929,7947,2724,2716,7934,3177,2720,
	7942,17255,2742,2739,7938,7921,16203,2738,
	7944,7920,2727,2731,7943,7930,2732,2726,
	7919,16776,3163,7946,7928,2737,7931,7945,
	2736,2717,7926,2722,2733,2728,7935,7925,
	7923,7941,2714,2734,2718,3162,7939,2719,
	2715,2730,2725,2723,7927,2735,7924,7937,
	7933,8526,8525,7922,8542,3165,18088,8548,
	8531,8545,9190,3168,8549,8552,3167,8534,
	8551,8550,3169,8527,3164,3172,14950,3170,
	8547,8537,3174,8554,8546,8555,3182,3180,
	8544,8543,15968,8541,8535,3173,17985,3171,
	7338,8528,18591,9220,8529,3166,3179,9192,
	8530,8553,9191,8538,8533,3178,8539,8536,
	16786,3183,17986,16028,3626,17987,9195,9214,
	9219,9207,3622,9224,3619,9193,9194,3612,
	3621,9201,9215,9221,3620,3614,9200,9196,
	9223,9197,9217,9208,9218,9210,3615,9881,
	3618,3616,9222,9213,9209,17881,9198,4068,
	9212,3623,9204,16785,9203,16797,14955,3617,
	3613,9902,9216,9211,9882,9893,9900,4066,
	3181,9898,4061,4060,9905,17346,9883,9896,
	9884,9895,4064,9899,8540,9906,9891,4067,
	10478,4441,9904,9903,4069,10465,4442,10466,
	14939,9885,9890,9886,9907,4063,4443,4065,
	4070,17260,9889,4059,4446,11092,10474,10473,
	10475,17187,10472,4447,17347,4453,4450,4451,
	10467,10470,10480,4459,4457,10471,8532,4444,
	10477,10468,4460,10476,4062,4449,9892,4456,
	4458,4455,4454,17121,4448,10479,4445,4461,
	17216,11091,4805,4802,17989,4793,4800,11093,
	4798,4792,11090,5080,4794,4801,5079,11089,
	11088,4803,16131,4804,11087,11086,5081,11096,
	4797,11094,5082,16793,5087,11626,11628,16794,
	5084,14248,11629,11625,17226,11627,5085,5088,
	11624,5083,5086,5331,11095,5330,5336,12075,
	12078,12074,5332,12072,5506,12076,12077,12073,
	5333,12432,12435,17718,12436,12434,12437,5507,
	12433,15017,12759,5655,12758,12757,12756,5654,
	5656,13010,5747,5746,16775,16064,13186,13185,
	13188,5881,5829,13339,13187,13338,5883,5882,
	5921,13444,13514,725,15970,8556,11097,556,
	17645,983,6134,1209,1207,6561,1523,1857,
	1858,7359,7948,2752,2751,2744,2747,2750,
	2746,8560,2743,3184,16805,8559,8562,3185,
	8557,8561,8558,14956,3627,16804,4071,9908,
	4806,4462,10481,10483,10482,10484,11099,11098,
	11631,11630,12438,16803,17992,726,16651,17748,
	3189,3188,9226,12079,5748,13340,727,2754,
	2753,3628,9909,4072,11100,728,838,1524,
	6562,6915,1859,2755,8566,3190,3629,16807,
	11632,5337,13515,729,6563,1525,1860,15931,
	6916,2265,7363,2266,7360,7949,2759,2756,
	8567,16810,8568,9227,15272,4074,4073,11633,
	12080,12440,12439,12760,557,6025,1861,16814,
	730,839,985,984,6136,6135,6302,1210,
	6305,6304,6303,6306,1526,6569,6565,6574,
	17994,1533,1531,6570,6566,6573,1530,1537,
	6568,6564,1536,6571,1529,6572,1532,1534,
	6576,6575,6567,1528,1527,1535,6921,6927,
	14995,1867,1864,6918,6923,17045,1870,1862,
	6922,1865,1868,1863,6930,1866,14989,1869,
	6919,6924,6928,6920,6917,6929,6926,2274,
	2267,2270,16820,2273,7366,15769,2268,7364,
	16823,2272,7367,2271,17995,7953,2761,7952,
	7954,2760,2765,7365,7951,7955,14992,2762,
	7950,2764,16825,2763,8575,14987,8570,3191,
	3195,3192,8573,8576,16827,3193,3194,3199,
	9236,8574,3197,8571,8569,18075,3198,3635,
	16738,9228,3631,9233,9232,9229,3637,14985,
	9231,3196,17997,9235,9230,3634,3630,3636,
	9234,15677,9912,9911,9910,4075,4076,10489,
	10492,4466,18760,4465,10493,10488,4468,10490,
	16565,4467,10487,15891,10491,4812,4809,11109,
	11108,11107,11102,15770,11103,4807,4811,11101,
	4808,11110,16568,14967,11106,12083,5091,5338,
	5509,12441,12761,5749,13341,5884,731,16004,
	6577,1871,2275,2766,3056,8577,16836,3638,
	9914,9913,732,990,6578,1539,16354,1538,
	15777,17999,2278,7956,3203,15755,3202,9237,
	9506,9915,11111,14239,5339,5658,733,841,
	844,840,6144,992,994,15937,6139,6138,
	6141,6140,6143,991,6137,6142,16411,6307,
	1221,6308,6312,1220,6316,6311,6314,6310,
	1218,1224,6309,6315,1217,6313,1219,1212,
	1222,15718,16843,6580,1540,1543,1561,1545,
	6588,1556,6583,1546,6591,6584,1559,6596,
	1554,1552,6595,6587,6579,6586,1547,1541,
	6590,6589,6581,18113,1555,6593,6582,1558,
	6600,1542,1549,6585,6594,1557,1544,1548,
	6592,15738,6964,1879,1892,16884,6959,6949,
	1885,6947,1878,6942,1888,6955,6936,6938,
	14330,6941,6933,6954,6970,1883,6943,6961,
	6972,6963,6948,6956,1889,1876,1884,1896,
	1873,1875,15192,16846,6940,1893,1886,6966,
	6937,1895,1890,6946,1894,6953,6945,1887,
	6965,6957,1881,6971,6944,1877,6962,6952,
	1882,6958,1874,6934,1891,2293,1880,6951,
	6950,6939,14742,6967,17749,1872,17351,16851,
	16328,15722,17750,7389,2300,7390,7378,2289,
	2301,7371,16498,7380,7388,18114,7370,7397,
	2279,16848,7396,7391,7392,2287,2298,7382,
	7383,7393,7384,7387,7381,15918,16925,2280,
	2284,7395,7375,2296,2292,16711,2295,2302,
	2285,2297,7386,2299,2282,15339,2281,7372,
	18003,7376,2290,7394,7385,7377,2294,2291,
	2283,2286,15760,14356,7981,7978,7971,7970,
	2775,7974,7989,7957,7990,7976,7994,7982,
	2774,7980,2769,2790,2780,18004,2784,2783,
	7958,18103,7987,7993,7985,7992,7973,7959,
	7991,7975,2772,2785,7984,2778,15018,7983,
	7960,2786,2788,7986,2789,2771,7967,7988,
	16612,2777,2787,7969,7979,7964,7968,2782,
	7963,2770,2779,2776,7972,2288,8600,2773,
	15006,16860,2781,15406,8603,8625,8614,3223,
	17131,3218,8597,3219,8613,8606,8619,3215,
	8579,8621,3205,8587,3208,3207,3210,3206,
	8595,3217,8590,8617,3213,8609,8622,8584,
	8589,8583,3212,8602,9259,8585,3216,8594,
	8618,3211,8591,8588,8605,3214,3204,3226,
	8608,8607,16863,16862,17147,17052,8580,3209,
	8620,8596,8598,18008,8611,8610,8578,3220,
	3222,8586,8593,8601,3221,8592,8604,8623,
	8612,15778,15744,14269,9279,3646,9277,9256,
	9261,9254,16871,9273,9240,9247,9271,9281,
	9257,16470,9244,9263,9268,9248,18009,9265,
	9245,17775,3648,9272,9270,9241,9275,9276,
	9278,3652,3644,15750,7965,9249,9267,9250,
	3641,3657,9274,3655,3651,9239,3643,16868,
	9242,3656,18011,9258,9238,15637,3649,9262,
	9246,3650,9255,3640,3225,9264,8599,9243,
	14957,16859,9269,3645,9266,3642,9260,3653,
	9251,9280,3647,9391,18013,3654,9929,9927,
	9949,9925,9943,3639,4080,9921,9948,16872,
	9926,9940,4085,4078,9946,18012,9919,9945,
	14128,4096,9936,9952,4094,9928,4079,9930,
	9944,4088,9923,4092,4082,9932,9922,9916,
	16374,9942,4089,9951,9917,4086,4087,9924,
	9935,9931,9933,16221,4081,9950,4095,9934,
	9918,9939,4084,4093,4091,9920,9941,4090,
	14831,4083,9937,15932,18014,16856,10505,10501,
	10497,10517,10514,4471,15761,10521,10504,4482,
	10511,10500,10510,4479,10530,10527,10513,10498,
	16877,16839,10518,10520,4475,10508,10503,10494,
	4472,4480,15913,10523,4481,10495,10529,10516,
	10522,4478,16857,15696,10519,4477,10512,10499,
	10531,10496,10524,4474,15646,10506,15208,10507,
	4473,4470,10502,4476,10525,10528,10509,11124,
	15682,17067,15135,11116,15012,11112,4825,4814,
	4819,4815,11126,11137,4813,11118,11127,10515,
	11128,4820,16880,11143,4824,4827,11114,4823,
	15019,11132,11131,11123,15013,14252,11133,11140,
	11139,14298,4818,4816,11125,11122,11119,11138,
	4826,11136,4822,4821,11130,18016,11113,11115,
	11117,11135,11129,4817,5100,11652,5093,11637,
	18019,11648,5095,11655,11639,16694,11656,11635,
	11646,15387,5102,11651,11657,11645,5094,11647,
	11636,5101,18021,11654,11641,5097,16882,11643,
	11640,5103,11642,5096,5099,11650,11638,11653,
	14307,11649,18023,5341,12094,15955,15179,5340,
	12093,18024,5344,12085,12089,5343,12086,15009,
	5345,5342,12084,12088,12092,16885,15014,16204,
	12447,12444,12453,12452,12450,12445,5513,18025,
	15530,12446,5512,5098,12449,5511,12454,12448,
	12443,12087,12442,12767,12766,12451,5659,12765,
	12769,13014,12768,5752,5750,13012,13011,13013,
	5751,15015,5830,13189,13344,5885,5955,13518,
	16888,13620,734,995,7398,7400,7401,2791,
	7996,7402,16891,7995,8626,3228,8627,8629,
	3229,3227,8628,9287,9285,9284,9283,9282,
	3658,9286,4097,9954,9956,4098,9955,4485,
	4484,10532,16238,11145,11144,11146,15022,11658,
	5104,12095,5348,12455,5831,735,845,996,
	1225,10658,4829,5349,736,17646,997,6603,
	1565,6602,18027,6974,1898,6975,7405,2304,
	2303,8630,8633,3231,8631,3230,8632,10535,
	10534,4486,10533,10536,11147,11659,5105,5350,
	12456,5753,6026,1900,6976,2305,3232,8634,
	3661,3660,18028,11151,11150,12096,13345,737,
	6027,846,1226,6317,1901,3662,738,6977,
	1902,6978,5106,739,16903,6979,16901,7410,
	7409,7407,7406,7411,7408,3233,8635,9291,
	16249,3663,9962,10539,10538,10537,10540,11153,
	11152,4830,11154,5107,11660,12097,12457,13192,
	740,848,847,1566,6028,6058,998,1567,
	14507,6605,2306,2310,7412,2309,8004,2795,
	3236,8638,16227,4099,741,17647,6059,16904,
	849,15877,17648,6062,17281,852,6061,851,
	850,1227,6060,16525,6150,6149,1004,6151,
	1005,1000,15290,1001,16908,1007,6148,999,
	1228,6336,6334,6321,1240,6328,1235,6322,
	6330,16586,6327,1238,6326,1245,6335,1247,
	18030,1249,6320,16910,1236,6337,1243,1246,
	14246,6628,1230,1253,1244,6323,1233,1248,
	6331,6626,6324,1239,1250,6338,6325,1237,
	1242,6606,1251,6332,1241,1252,1229,6329,
	1234,6333,6627,18031,1579,1595,6613,1586,
	15606,6634,1571,16076,1574,6625,6611,6615,
	1582,1584,6618,1591,1577,1590,6630,1585,
	6617,6619,1583,1588,6620,6633,1904,1594,
	16496,1572,6632,16273,6636,6616,6638,6623,
	1581,6612,1580,1598,1587,6610,6990,1593,
	1597,6624,6629,15703,1599,1592,1578,1568,
	6614,1569,6637,16269,6608,6607,6621,6609,
	1596,2311,1589,6622,1570,6997,7000,18033,
	7003,6992,14873,18034,7006,7009,7011,6989,
	1905,1910,7417,1926,7004,7012,6996,6988,
	7444,1913,7001,6993,6995,1918,6998,1912,
	6985,7007,15043,1909,18035,1921,6982,1923,
	1907,1927,7008,6984,1924,7438,6631,1911,
	1906,6991,6983,1925,1917,7002,1922,1920,
	6994,1914,6986,6987,7005,1908,6999,15603,
	7420,2319,2324,16540,16930,7429,7435,7431,
	7418,7421,2331,2316,7430,7439,2326,2312,
	2321,7422,2323,7423,7433,16629,14807,2325,
	7448,7416,2318,2317,2329,7413,7446,7434,
	7447,7441,7427,7442,18036,7436,7445,7443,
	2330,7428,2315,2314,2322,2328,7440,2327,
	7425,2796,15848,7449,7424,7419,2320,2332,
	2313,16929,7432,7437,18038,15033,2830,8006,
	8010,2811,2809,18039,2800,8008,8007,2820,
	8013,2814,16379,8031,8043,2797,8028,8021,
	2832,8005,2829,2817,2828,2807,8017,8035,
	8027,2808,2802,8041,8034,8036,2810,2818,
	8038,8015,8032,8020,8040,2823,2799,2821,
	8025,2812,8019,8018,2801,8012,8042,2803,
	8022,2833,2827,8011,2824,2822,2831,8029,
	2826,7426,8030,2825,8037,2798,8026,2816,
	8014,2815,2813,2805,2804,8639,8039,15610,
	8660,15038,2806,18041,3265,2819,3247,8654,
	3261,8642,3244,3240,8672,3246,3250,3245,
	3254,8676,8667,18042,17154,8649,3259,3253,
	8661,3237,8673,8666,3241,8655,3256,3238,
	8671,3258,8659,3262,8650,8657,8651,3260,
	3268,8647,8643,3243,8656,3257,3266,16948,
	16849,8658,8682,8674,3239,8680,3251,3249,
	16614,8686,3248,8664,8652,8662,3271,8668,
	8665,8648,3267,8678,8675,8683,3242,8663,
	3270,3252,3255,8669,3269,8653,8009,14803,
	8679,8677,8681,8644,3264,9324,9323,9297,
	3669,3679,9331,3674,9321,17793,9294,3680,
	3670,9309,9299,9302,9326,3664,9333,9325,
	3673,9314,3684,3683,3677,9332,3665,9312,
	9303,9316,9328,3685,16894,3667,9311,15884,
	9304,3676,18045,3675,9307,9317,9327,9298,
	9308,3668,9318,3681,3672,9305,3671,9296,
	9310,9330,3263,4127,9313,9963,9315,9329,
	3678,9306,3666,3682,4506,9295,9320,16489,
	18047,16431,9319,17720,9991,4124,9980,9985,
	4113,9965,4126,4104,9964,9997,4128,9968,
	9984,16955,9970,10000,9996,4102,4112,10566,
	4125,4110,9966,16932,9995,4114,9992,10005,
	9972,9981,15034,15611,10003,9993,4109,9979,
	4103,4101,4119,16800,18126,9977,10004,10002,
	4107,10006,4111,4118,18051,9967,10542,9976,
	4105,4123,4120,4108,9986,9971,4121,9988,
	4115,4117,4100,18052,9990,9982,9969,4116,
	9998,10001,9978,9989,17026,4106,4489,9987,
	10541,9994,16960,9973,10551,10561,4492,10560,
	10572,4494,10558,15954,10562,4505,10553,4497,
	16961,10565,11162,4509,4508,10567,10550,10577,
	4503,4493,10570,10574,10579,10568,10556,4496,
	4499,4507,4502,10559,9983,16963,10555,10578,
	4498,15294,4501,10580,4490,10569,10543,10575,
	5116,18055,10557,4491,10552,4495,10544,4122,
	10548,16789,10573,10549,10547,4500,10546,10564,
	10563,10576,10554,4504,16953,11165,16966,4843,
	4833,11177,11157,4835,11169,4842,4837,11167,
	11662,11173,11179,11172,11155,11170,11181,11183,
	4832,11182,4838,4844,15911,4841,11176,4840,
	11171,15291,11159,11164,18056,11175,4839,4836,
	4831,4834,11163,10545,11158,11161,11178,11166,
	11156,11661,11180,11160,11174,16968,11664,5120,
	5108,15476,15475,5112,11666,11672,11670,5118,
	11673,11665,5113,11669,11668,11674,5119,5114,
	5117,11667,5121,5115,5122,5109,11671,14501,
	12110,16951,5355,12107,12109,5353,12105,12106,
	12101,18146,11168,12102,5354,12098,5351,12111,
	5352,12104,5357,12103,5519,12461,12467,5520,
	12458,5517,5514,12469,5518,15478,5515,12460,
	12108,12459,5516,12466,12783,12462,18062,12774,
	5661,12777,5662,12786,12776,12773,12775,12779,
	12782,12781,12785,12772,5660,12780,12784,12787,
	12778,13018,16980,5754,18064,5832,13196,13347,
	13346,14418,13450,5922,13448,13451,13449,13520,
	5956,5970,13596,13574,15605,13595,742,17649,
	18066,1010,6154,6339,14535,1254,1257,16537,
	6340,16981,18067,18777,6647,6649,6644,6641,
	6643,17401,1603,16587,1601,18099,6645,1602,
	6642,6639,1600,6646,6640,1604,6648,7014,
	18242,7018,14706,1936,15543,15699,16982,1928,
	1931,1933,1935,1932,7017,7016,7013,1934,
	15721,1929,18450,17753,17008,7015,2333,7454,
	16984,7462,7467,2338,14701,7452,7458,7451,
	7461,7457,2334,2336,7466,7450,7453,7459,
	16987,7460,7468,7456,2838,8050,15048,8052,
	8048,18272,8060,8046,8059,2834,8044,8690,
	8054,2837,15630,8055,8058,8051,14883,8056,
	8061,8049,8045,8063,8700,3279,15687,8688,
	8697,8687,3276,8695,8693,8694,3274,8698,
	15273,16993,17002,8691,8689,3275,8692,8696,
	15700,15486,14189,3277,8699,15045,17304,9339,
	9346,9348,9354,16994,3698,9334,3690,16996,
	9349,3695,9355,3686,16999,9352,18533,9336,
	9353,9335,9347,3687,9356,3692,9340,3697,
	9351,9338,9345,9341,9337,3689,3696,3694,
	3691,3699,3688,9344,15557,3693,9350,15808,
	9342,18407,15597,16384,14679,9343,18724,10014,
	14389,4131,14704,10010,10016,10012,4133,10011,
	10015,10007,18182,10009,4132,15675,10013,10008,
	16991,10017,18461,4130,10586,10583,10595,10589,
	10591,4510,10585,10593,18625,16072,10592,10590,
	10596,4513,10587,10594,4511,15901,10582,10584,
	4512,10581,10597,16990,10588,18059,11187,4851,
	17006,11198,11196,4845,11186,11184,11199,11194,
	4849,4846,11193,11191,4852,11195,4847,14141,
	4848,11192,4850,11188,11197,4853,4854,16504,
	5124,5131,11675,11681,11679,5127,5126,5123,
	11677,15489,5129,5128,5125,11680,11676,11678,
	5130,18158,16026,5361,12114,12116,5360,12113,
	12112,12473,12115,12474,5521,12472,15803,12471,
	12470,5522,17009,14844,5663,17011,14675,14697,
	13025,13024,5755,13348,13452,17014,13521,13621,
	5990,13641,743,17650,5132,744,1607,2339,
	3701,745,2839,4135,6029,17018,7022,7469,
	5133,746,1608,8701,3280,9357,15049,3702,
	10019,17021,4514,17022,5523,747,8702,748,
	18010,1012,6155,1011,1260,1259,1258,6341,
	16059,1609,1610,6650,7028,1939,7026,1938,
	7027,1940,7472,7471,7470,2340,2840,8064,
	8067,3282,2841,17027,3281,8706,8705,15050,
	8707,9359,9358,9362,17353,9361,9360,4136,
	10022,15051,10021,4137,10020,10598,10599,4515,
	12479,5524,12478,12477,5756,12790,13199,13522,
	749,17652,6063,853,6064,6342,1611,6346,
	1262,6344,1261,6348,6345,6654,7029,6652,
	6659,17354,1612,1615,6660,6656,6651,1614,
	6653,1613,6658,16279,7036,7030,7031,1941,
	7034,7032,7478,7475,7477,2345,2344,2342,
	7474,7479,2341,2343,7476,8075,7473,7480,
	16282,8070,8074,8073,8709,8720,8076,18154,
	8071,8708,2845,2844,8069,8072,2846,2843,
	2842,8068,8077,8711,8718,3284,8717,8714,
	17034,3286,15728,17029,16306,8716,17035,8710,
	8712,8715,8713,3285,8719,3283,3703,18155,
	17356,9366,9365,9363,3706,3705,9367,18156,
	9364,10023,4138,3704,17402,9369,9368,10026,
	10024,4516,16209,4139,10025,10601,4517,10600,
	10609,10608,10606,10607,10610,4857,11204,11210,
	11209,11207,5134,16280,5135,11682,16084,5363,
	12117,5362,5525,5526,5664,12792,12791,13026,
	13200,5834,13350,13349,13351,854,7037,2346,
	2847,8078,855,6065,750,6158,18149,6349,
	1263,6350,14838,16522,6668,14683,1618,6666,
	6662,6665,6661,1620,6664,16519,1617,1616,
	17037,1619,6667,6663,1947,1950,7045,7044,
	7043,1944,7052,7042,1946,7485,7049,7047,
	1949,7039,7050,7041,7051,7048,7040,17040,
	1945,7053,7038,1948,16667,17036,15127,7490,
	7481,7492,16499,7484,7482,7494,7491,7489,
	7493,2352,2350,18160,15523,7487,17061,7483,
	15546,7486,7497,7488,2351,7046,2347,2349,
	16401,15531,8092,8081,8079,17050,8080,15353,
	8087,15532,8088,8084,2852,8089,8086,8083,
	2850,8082,2848,2851,8085,8093,2348,2849,
	8091,8090,2853,17772,15334,14836,17051,8725,
	16557,18163,8726,3296,14557,8732,8734,8727,
	3290,8731,8730,3291,3297,3298,8733,3288,
	8724,8722,8728,8721,3295,8723,8729,8735,
	3289,3294,15073,15113,17054,3287,3713,9376,
	3712,9380,14874,9370,15319,9381,17060,15541,
	17041,9382,9379,9374,9373,16945,9383,3709,
	14837,9375,3714,3708,17094,3711,3710,17058,
	10027,4141,4140,16641,10032,15086,4518,4142,
	15335,17064,4144,10033,3707,4143,10029,10031,
	10028,17065,10030,17062,10620,15057,10619,10616,
	4521,11211,4522,10615,17068,4520,10617,10614,
	10611,10618,4519,14881,18077,11214,11216,11685,
	4860,14835,11212,15114,4858,4862,4861,11213,
	11217,4859,16752,11692,5138,5364,5139,5136,
	11689,11687,11693,5137,11691,11686,15525,12120,
	12123,12482,12118,12793,12124,5527,12122,5365,
	12119,12121,12125,12483,5528,12480,15070,5665,
	15003,18170,5758,13202,5757,13204,13201,13203,
	5886,13453,856,6669,2854,9384,4863,5529,
	5835,13454,857,16382,6670,288,16358,16283,
	16351,9387,9386,9385,10034,4145,16350,10622,
	10621,17079,11697,12127,12126,5366,12484,13027,
	858,17795,1951,2857,9388,17080,859,7502,
	2858,6159,1265,1264,1952,7056,8737,16007,
	17754,6355,6354,15867,1621,6673,17087,1956,
	1954,17086,1955,1953,16363,2353,15082,2357,
	2356,7503,2355,2354,7504,15729,2861,8095,
	8094,3302,3301,8739,18173,17089,8738,2862,
	3717,9390,3718,9389,16207,10037,10623,4523,
	11219,15077,11698,5531,5530,5836,866,6674,
	18175,17093,2863,10038,4146,558,1622,17095,
	7060,1957,7505,16045,17096,2866,7511,2367,
	2365,7512,7507,2364,2363,2358,7510,7509,
	7506,2366,16275,7508,2359,2360,18177,2868,
	8096,8098,8102,8101,8097,8747,3307,3306,
	8741,3304,17358,8744,3308,8745,3309,8742,
	3303,3305,8746,8743,8740,9392,3720,3723,
	3722,3727,3726,9404,9402,9395,9399,9398,
	3724,9401,9397,9405,9396,3725,3719,3721,
	17097,9394,16271,10040,4150,10044,4149,10041,
	4148,9393,9400,10043,4151,10045,10042,10039,
	10626,10630,10632,10628,10627,10625,4526,4524,
	10629,4527,10624,4528,4147,10631,4525,17098,
	11221,11220,11228,11222,11227,11224,4866,11226,
	4868,15984,11225,11223,5141,11699,5140,11702,
	5142,18178,15986,17100,12131,17099,5369,12132,
	16112,5367,12133,5368,12130,12134,5532,12485,
	5533,5666,12129,5667,16042,16297,5759,13028,
	5838,13205,5837,17101,13352,15083,18179,13635,
	559,1962,7061,18181,867,1013,6160,17363,
	6358,1268,8104,7513,2368,17364,2869,8103,
	17104,8749,3313,8748,3312,9406,4531,10636,
	11703,17110,12135,12795,12794,13206,868,6676,
	2369,3314,9407,17111,10046,4532,11229,12136,
	12796,869,1626,1968,7062,1969,1967,1966,
	7514,2370,2871,8105,2870,8750,2872,3315,
	9408,4153,4533,4870,11230,4869,11704,5143,
	12137,12797,870,1269,6679,6678,1627,6677,
	1628,6680,7068,1972,1971,7070,7069,1976,
	1977,7063,1973,7067,1974,15714,7517,7521,
	7519,7518,7515,17118,7525,7520,16360,8114,
	17119,7526,2376,2373,8112,8111,8116,2876,
	2873,2877,8106,2878,8115,2875,8113,2874,
	18186,8753,8755,8754,8757,3316,9417,9409,
	9413,17123,9418,9414,3730,3738,4157,3741,
	3734,3739,3732,9416,3740,9415,3736,3731,
	3737,9419,10051,10053,18089,15104,3735,17175,
	15108,10048,4155,10054,4156,10052,10047,10050,
	10055,4154,10049,4535,10640,10639,4538,4536,
	10637,4534,10638,11240,18190,11239,11231,11235,
	11234,11232,4871,11233,5148,11712,5145,11706,
	5147,5149,17125,5146,11711,11707,5144,11710,
	11705,11709,11708,15987,12139,5370,18193,5371,
	12138,15281,12488,16833,18194,12798,12801,5760,
	13353,13455,5971,871,1978,872,1270,1629,
	16513,8761,3317,3742,5150,11713,12489,12802,
	873,17129,6686,6681,6683,6685,6684,6682,
	1630,16341,1979,7081,7076,7073,15942,7080,
	7079,7078,7077,7082,1980,18196,2381,15123,
	2387,2386,7535,7529,7527,2384,8123,2379,
	7532,7536,7538,7534,7528,2385,7533,7531,
	2378,7539,2388,7537,2382,7530,2383,2380,
	17366,16410,15466,15044,2880,15403,8124,18197,
	8117,8119,8121,15469,8122,8120,2881,8125,
	18198,8118,16277,15467,8765,3318,8773,8772,
	8771,8768,2879,3319,8766,8769,3320,8770,
	8767,9431,16105,9435,3749,10061,3752,9433,
	14056,9430,9425,9421,15125,3748,3747,15124,
	3743,9424,3751,9423,9426,9434,9432,9422,
	10063,4159,10065,9429,10067,4163,10068,10064,
	4160,10060,4162,10057,10062,10066,10059,17132,
	3744,17140,10056,4161,10058,17777,15459,4541,
	10642,4545,17133,10648,10651,4542,4539,10644,
	10641,10646,10643,4546,10645,15126,17134,10647,
	4544,17138,16268,4876,11247,11241,11245,4878,
	4875,11242,11244,4877,11250,14977,5154,15162,
	16574,11720,11714,5153,14515,11722,5152,11715,
	11723,11719,5155,11716,11724,11721,16335,12144,
	12146,12145,12491,12493,12490,12495,12804,12807,
	12803,5668,12808,5669,5671,5670,13030,17136,
	13032,13031,18203,18202,17139,13207,14211,13456,
	13525,874,17655,17152,6359,1631,6688,7087,
	7086,1983,1986,1985,1984,7083,7085,7084,
	7542,2390,7546,7545,7544,7540,2389,2393,
	2396,14309,15128,2397,7541,7543,2395,2394,
	2392,2391,8132,15997,8130,8126,2882,8127,
	2883,8778,16744,8777,8776,8775,9439,3753,
	9436,9438,15910,3754,17148,3755,9437,15862,
	17145,17800,10077,10079,18205,10078,10075,10074,
	10073,10072,10076,10069,17404,10652,10657,10655,
	17150,14686,10654,10653,10656,11253,15134,4879,
	5156,11726,17153,5157,11725,12148,12147,5375,
	12496,5536,12809,5839,13208,13458,13457,6066,
	3757,875,1273,1272,1271,15565,6689,17157,
	1635,1634,1991,16947,7089,7092,7090,17159,
	7088,7091,7553,7555,7554,2401,7549,15450,
	2399,2398,2402,2400,2403,7552,7551,7550,
	8134,2885,8779,8781,8783,3324,18206,8780,
	3323,8782,3321,9449,9440,9443,9447,3761,
	9445,3759,9448,3758,3763,3762,3760,16346,
	4167,10083,10082,4168,10659,15137,4552,4548,
	4550,16267,4547,4549,14700,11254,15857,4882,
	11256,11255,14131,10081,4884,4883,11729,5158,
	11730,11732,11728,11727,12149,5378,876,6161,
	1274,6690,1637,1636,6691,14135,7094,1992,
	7093,1993,15741,2405,7558,7556,7562,2406,
	7557,7561,8138,8140,17368,2886,8139,2887,
	3328,3327,3326,8784,9451,9450,10086,4170,
	4169,4554,4553,14138,11260,10660,10662,10661,
	11259,11257,11261,11258,4885,14137,18209,11733,
	5159,5672,15743,17776,5887,877,7563,2407,
	14144,18213,14673,14145,3038,3037,15155,3330,
	8786,3329,8785,14429,14385,14146,9453,4171,
	10089,4172,5673,12813,1014,1638,6692,1995,
	15165,1994,7565,7567,2408,7569,7572,7570,
	14150,15203,7571,7568,7564,2409,7573,7566,
	16247,8149,2893,8158,2890,8151,2894,16145,
	2888,8146,8159,8147,8144,2892,15199,2889,
	14152,8150,8153,2891,8154,2895,8155,8145,
	8152,8156,8148,8143,8141,8157,15726,15853,
	15909,8789,8795,3333,8788,8790,3331,8787,
	3338,8791,3337,8792,3339,3334,3340,3335,
	3336,14151,3332,3768,9464,18215,9467,9455,
	9460,9454,3770,18216,9456,3769,14156,9463,
	9462,9459,9461,9457,4176,9466,3766,9465,
	4183,10100,4182,10090,10092,4175,10097,10096,
	10099,4180,10094,10093,14162,4179,4174,10095,
	4177,10101,10103,10098,10091,4178,4562,4173,
	14155,16148,10104,10666,4555,4556,17948,4558,
	10671,10663,4181,10669,3767,4561,4888,10670,
	10664,4563,10667,16152,14160,11271,11266,11276,
	4886,11268,4890,11274,11265,11262,11277,5165,
	4891,11273,11263,4889,11267,4893,11264,11269,
	4892,11272,11275,11738,11745,11743,5163,11750,
	11270,14167,11741,16159,11751,5162,11740,11739,
	11749,11746,11748,11736,11752,5160,11747,11756,
	11744,5164,5161,11742,11737,11755,4887,18218,
	17369,12150,12156,5384,12154,12153,5386,12159,
	5385,12160,12157,5382,12158,12151,5383,5381,
	12502,12155,5543,5541,12501,12503,12500,5542,
	16156,5675,14171,12818,12817,12815,12814,12816,
	5674,5676,5761,16312,13034,13033,16127,13211,
	13213,13215,13214,13212,15167,5841,5840,5889,
	5888,5890,13355,13354,13526,13459,13527,5957,
	15168,5958,13575,16210,5995,1015,18076,6693,
	7098,18221,15177,1996,16154,14172,7574,7580,
	16157,7578,2410,7576,7577,7579,15176,7575,
	2896,8160,2898,8162,2897,8161,8797,3341,
	8799,8796,8163,3342,14174,14061,8798,15178,
	9470,3771,9468,3772,9469,3773,18224,4184,
	10109,10106,10108,14177,10673,14176,4564,10675,
	10674,15173,289,11278,15708,11279,5171,16345,
	5167,5172,5168,5170,5166,14178,5387,11757,
	12504,17370,12819,13217,13035,14180,13216,14181,
	13528,13597,1016,17656,1275,17773,6694,1639,
	1999,7106,1997,7104,2002,1998,2003,2001,
	7105,2000,2414,2413,7591,2422,7584,7590,
	2418,7583,7589,7587,2417,2419,7586,2412,
	7585,2423,2420,2424,2421,7582,7581,7588,
	2415,2411,2416,8178,8171,3347,8170,2902,
	2909,2906,2912,2911,2907,8164,8167,8166,
	2903,8168,8177,2904,8165,2905,8172,2910,
	2900,2908,8169,2899,8173,16349,8816,8807,
	3344,8814,8811,8803,8812,3346,8804,16019,
	3779,8809,14185,3343,8815,3350,3352,8802,
	3351,8805,3345,8813,8806,8810,8808,3348,
	8801,3353,2901,3349,3354,3775,9477,9479,
	9481,9485,9475,9473,3777,9480,9484,9476,
	9472,9471,14187,9482,9474,9478,3778,3776,
	9486,3774,9483,10126,15187,4189,10118,4577,
	10128,4192,10132,4198,10113,18135,10129,10110,
	15516,10131,10114,14188,4205,4202,10130,10124,
	4188,4196,4195,14190,4194,4200,10111,4201,
	10125,4197,4187,10127,4199,10116,10115,10112,
	10120,10117,10121,4204,10133,4193,17774,10119,
	18229,18231,4203,15182,18230,14191,10680,4569,
	4578,4574,10677,15180,4571,4575,10689,4565,
	10681,4573,14194,14198,10684,10679,4572,4576,
	10678,10688,4567,10687,10686,4579,10440,4566,
	14196,10685,10676,4580,10683,4568,18232,15567,
	15181,4898,4903,4896,11295,11286,11298,11290,
	11296,4904,4897,11283,11289,11299,11292,11297,
	11285,11293,4899,11291,4902,4901,11284,11294,
	4900,11301,11300,11768,11764,5179,11758,5173,
	5189,11770,5181,5177,11761,5184,5187,11772,
	5176,11763,5185,11774,11771,11759,5180,5174,
	5188,11766,5183,11760,5178,11773,5182,5175,
	11767,5186,11765,12163,12169,5393,12172,12164,
	12171,12166,5394,5391,5390,12168,12170,5392,
	12167,12165,14182,12515,5546,14203,12511,12509,
	12513,5550,12514,12507,12506,12510,12508,5547,
	12512,12820,5681,5680,12821,15953,12823,12822,
	5682,13036,13039,5763,13041,16073,5762,13218,
	18078,5891,5893,13356,5892,13460,13530,13532,
	13529,13531,5977,1017,17762,2004,7455,2425,
	11302,5190,11775,14210,5395,15977,12517,12516,
	5683,13042,15183,13219,5926,6162,17654,17657,
	1640,1276,7597,7592,7596,7594,2426,7595,
	7593,9488,9491,9487,3780,4206,16391,3783,
	10134,4581,10690,4582,16251,4905,11305,11778,
	11777,11776,5551,12519,13220,5927,13461,1018,
	1642,1641,7109,2005,16924,7108,7599,17371,
	2427,8180,7598,14214,2915,8183,8821,14216,
	3786,9493,9492,9494,3785,3784,10691,10693,
	17372,4583,10692,11306,4906,12174,18233,12173,
	5552,12520,5554,5553,12824,13043,5764,1019,
	7110,2006,7602,2429,7601,7600,2916,8186,
	2917,8190,15198,10138,4209,10137,10698,18153,
	10699,4584,10696,10695,10697,10694,4909,11307,
	11780,5191,11779,12031,5396,14222,5397,5192,
	12825,5684,1020,17661,1021,2431,1643,2430,
	7111,14225,3358,1022,2008,2007,2009,1023,
	7114,2433,7603,2435,2432,2434,8194,2919,
	14226,8193,8192,18236,9497,10139,18237,4585,
	4910,11309,11308,18238,14228,13044,18239,1024,
	6360,6695,2010,7115,2436,7604,2437,2921,
	8195,8197,8825,3359,4211,16357,10141,10140,
	4210,14233,16355,16348,10700,16347,10701,5196,
	5195,12177,5399,5398,12523,12826,5843,5842,
	1025,17662,3790,3360,3789,4212,1026,6067,
	6696,6363,6364,1278,6361,1277,1280,6362,
	1282,1281,6365,1279,1648,1646,6698,1650,
	1652,1649,6701,6697,1653,1647,1283,1651,
	6700,16046,18243,6699,17778,15995,7123,15220,
	7117,7120,7129,7127,2016,7133,2019,7130,
	7122,7119,8829,7128,2011,7131,7116,7125,
	2013,2018,7126,2022,2020,7118,2017,7124,
	2021,2012,7132,17897,2442,2451,2440,2438,
	7607,2446,2443,7609,17780,2445,7608,7606,
	7611,2450,2448,8828,7612,7610,2439,16359,
	2441,2444,14242,2447,15221,2449,18249,8207,
	2923,15981,8198,8200,14243,8201,8209,8205,
	8203,8206,8210,2924,2928,8199,2926,14905,
	2925,8204,8202,2922,2927,3366,8837,8835,
	3368,15222,8830,9511,3367,8834,15990,8831,
	3363,3369,8836,3365,8833,4213,3364,8832,
	3370,3362,3361,18252,9503,9501,9499,9512,
	9504,9498,3794,3800,9509,17348,3795,9510,
	3792,3791,9505,3796,3371,9508,9500,3793,
	4218,4214,4219,10144,17349,10143,10145,4216,
	10142,4217,10148,10147,10146,10708,4591,10707,
	4590,10704,10703,4589,10706,10702,18259,11312,
	11322,4913,4912,17252,11316,11318,11317,11311,
	11320,4911,11319,11315,16050,11321,11314,5200,
	11782,11788,5204,5206,5203,5202,18261,5201,
	5199,11783,11786,5198,11787,16366,5205,11785,
	11784,5400,5401,12180,12524,5555,12828,5685,
	12827,13045,13221,5844,5894,1029,18262,1654,
	8838,4220,5207,11789,1030,2453,2452,8839,
	17105,11323,1031,2023,4221,4914,1032,1655,
	7134,2454,7613,2929,8843,3801,4222,4915,
	5208,5402,12526,1033,1656,2455,8211,3372,
	4223,10149,10709,16353,1034,3373,9513,4224,
	1035,6702,7135,2024,7615,16909,2458,14256,
	2459,16080,8216,8213,8215,2930,2932,2931,
	8212,2933,8214,17794,8846,9516,9515,9518,
	10714,10710,10713,10715,10712,10711,18263,11791,
	11790,11792,12182,12181,12528,15226,12529,12527,
	15227,5686,13222,13357,13462,1036,1284,5209,
	1037,8217,8847,18265,5934,6163,18210,17763,
	14280,6164,6166,1038,6167,6165,6372,1286,
	1287,6369,6367,6366,6370,1285,6371,7136,
	6707,1659,6706,6708,1658,6713,1662,6703,
	6717,6723,1669,6720,1666,15731,6710,6716,
	6718,14268,6705,1665,1660,6711,1667,1670,
	1664,1657,6715,6709,6722,1672,1668,1663,
	6714,2460,6712,1661,1671,6719,6721,17891,
	17764,14273,2041,2035,2043,2040,7143,7146,
	2036,18267,7137,2029,2039,2042,2044,7158,
	7149,14281,2028,7157,2032,2030,2025,7141,
	7156,7145,7148,7161,16408,2045,7155,2037,
	7150,7160,7147,7151,7154,15991,7139,7159,
	7153,16685,15762,7138,17374,7142,2038,2033,
	2026,2031,2027,2046,7140,7632,2034,16054,
	7152,6781,7635,2474,14282,7619,14274,7630,
	7626,14289,7645,7636,7621,7625,7648,2477,
	7640,7631,2461,7646,7617,2476,2471,7616,
	2469,2468,2473,7638,2465,2472,7144,8848,
	7633,15658,7623,2475,7624,7628,2478,7618,
	7642,7641,7649,2467,2464,7647,7643,7634,
	7629,2470,2466,7620,2462,7644,2463,7622,
	18269,14262,16875,8223,8225,8230,8250,2947,
	2937,8233,8240,8246,14291,2950,8248,8243,
	2945,2943,8239,8235,8221,2934,8226,8218,
	14316,2942,2944,8231,8228,2939,8244,2936,
	8229,8889,8237,14157,8236,2935,2938,8219,
	8224,8241,8247,2951,8220,8232,8238,2941,
	14294,8249,8242,8245,2940,16217,8864,8852,
	3381,8881,8890,8893,3379,8860,8885,8861,
	8875,3392,8871,3389,16469,8872,8849,14296,
	3398,8883,8873,8895,8868,9550,8898,3396,
	8866,8878,3399,3378,8870,8896,8863,8856,
	8867,8855,8853,3374,8887,8862,8891,15660,
	8892,3382,3387,3383,3391,8882,3384,8874,
	8858,3376,8850,8884,8894,8857,3390,8899,
	8869,3375,3395,18271,8880,3397,8876,3386,
	3380,3388,3377,3394,8877,8859,8879,8854,
	8888,15376,8897,16051,15445,14161,8851,18769,
	9571,3756,9554,16941,9552,9531,3806,9547,
	9567,9543,3814,9566,9541,3393,9557,9542,
	9564,3813,3805,15649,9520,9528,9553,9545,
	9570,9548,3819,9565,18200,3810,14308,9551,
	9559,9532,9558,9575,9527,9560,15229,9573,
	14731,9521,3385,14302,9535,9534,3812,16091,
	9538,9540,9555,9572,3815,17114,3816,18273,
	9526,3808,9530,9568,3817,3809,3811,3818,
	9574,9561,9556,14304,15507,9537,9536,3807,
	9522,3804,9546,9523,9544,9533,9569,14670,
	15214,9525,9563,9524,4238,10189,14166,14318,
	10172,15133,10150,10180,4230,10165,10175,4233,
	10168,4231,10153,10183,10152,10171,15262,10151,
	10169,10178,10176,10164,10159,10157,15756,10158,
	10177,10188,10166,4232,4235,10160,10154,10170,
	4239,15436,18276,4227,4236,10162,10156,4229,
	9562,4228,10173,4226,4241,4234,10174,10163,
	10155,10182,10167,4240,10186,4237,10185,10179,
	10184,16373,10187,18277,14261,14319,15312,10743,
	10742,10738,4603,4595,10758,10757,9519,10735,
	10739,10733,10752,10737,18278,10759,10730,10728,
	10740,10750,10745,4605,10719,10749,14730,16619,
	4606,9539,10731,10722,10725,10763,10732,10723,
	10721,10718,18279,4599,10741,4598,4602,10736,
	10744,4592,10760,10764,4594,10727,10747,10751,
	10724,10761,4601,4600,10716,4604,10754,10726,
	10746,10720,10734,10729,10748,10765,10762,10753,
	16936,14324,4593,11343,11334,11337,16482,4923,
	11339,11328,11335,11341,4920,4924,4918,14272,
	16399,11330,11346,11340,18281,11331,11355,11349,
	11327,11811,11333,4919,14299,11344,11354,4927,
	11351,11332,11338,11342,11336,11356,11353,11350,
	4926,11329,11357,4925,11347,11348,11824,11805,
	11810,11823,11809,11801,11806,11794,11818,11820,
	11793,5211,11822,11814,5218,11821,11807,5220,
	11799,11352,11815,11812,11795,11826,5214,5215,
	11797,11813,11825,11816,11803,5217,5213,11817,
	11804,11827,11798,11808,11819,11800,11802,5221,
	11796,5219,5404,5210,5216,5408,12198,12188,
	12183,12189,12199,12203,12202,5410,5409,12190,
	12204,12193,12185,14334,12187,12186,12192,12197,
	12184,12191,12196,5407,5405,12195,5403,5406,
	12545,12200,12537,12541,12533,12536,12544,5557,
	12549,12542,18285,12534,12550,12543,12535,5556,
	5558,12530,12538,12532,15446,12548,12546,12531,
	12539,12837,5562,12540,5688,5690,5687,15440,
	12842,12834,12832,12836,12833,14336,12838,12840,
	5691,5693,12839,5694,5692,12841,16484,15231,
	5689,14331,5765,13046,13052,5767,12835,13051,
	13058,13049,13056,13055,12831,13050,12201,13059,
	13048,13053,14341,13057,13047,13226,5766,13054,
	16286,13362,13227,13225,5895,13360,13358,13363,
	13361,13364,5896,13359,14529,13463,13630,13622,
	6168,1673,7651,7650,2479,2952,8252,3400,
	3821,3820,3822,10190,10766,11358,11360,11359,
	5222,11828,12205,13598,1039,14355,6725,6724,
	1674,18287,7165,7163,2048,2050,2049,7164,
	7663,7660,7658,2480,7661,2485,7655,7670,
	7672,7656,18288,2482,7673,7668,7654,7659,
	7669,7665,7662,2487,7671,7657,15240,7666,
	7652,2486,2483,7664,7667,7653,2484,2481,
	2962,8259,2961,17376,8262,8265,2958,2956,
	8254,8263,8261,8258,8253,2955,8256,8255,
	8268,2957,8257,2959,2954,8260,2963,2960,
	8264,3407,8914,8903,3404,9581,8912,8900,
	3402,8905,3405,8909,8907,3408,3401,8901,
	8904,3406,8913,8911,8906,8908,8910,3403,
	9582,9588,9578,9584,3823,9580,3829,3827,
	9587,9577,9590,3832,3826,3825,9586,3833,
	9576,9579,9589,9585,10222,10211,3824,4248,
	10199,10220,10200,4243,10206,10198,4245,10192,
	10219,4246,10193,4251,10208,10202,10209,10196,
	10221,10213,10207,10191,4247,10214,4250,10218,
	10212,4244,10210,10204,4242,10195,10203,10215,
	10201,4616,9583,10784,10793,10783,10779,10776,
	4617,10774,4249,10767,4615,10773,4614,10778,
	10775,10791,10786,10780,10785,4610,10777,10796,
	10792,4611,10797,4613,10798,10782,10194,10788,
	10781,10790,10787,15980,14347,10772,4608,10795,
	4609,10770,4612,11371,10789,10794,11368,4607,
	4928,11378,11374,16048,11372,11367,11382,17378,
	16367,4932,11366,11365,11379,11369,11364,11370,
	11381,11362,11380,11377,4930,4929,4931,11373,
	11361,15752,16364,11830,5228,11834,11831,11838,
	11844,11833,14351,11853,5225,11847,11836,11848,
	11843,11850,11835,5230,5229,11837,11851,11829,
	11849,5223,16380,11841,11840,11845,11832,5227,
	5231,11839,11846,5232,11842,5224,5226,12218,
	12216,16274,12222,12220,12223,14352,12209,12217,
	12227,11852,12213,5414,10771,12208,12221,12215,
	12212,12207,12206,12226,12211,12210,5412,12219,
	14349,5411,5413,12214,12225,16302,5566,12552,
	5563,12560,5567,12562,12551,12224,12564,12553,
	16047,5564,12559,12561,12556,12563,12558,12557,
	5565,12848,12846,16250,12843,13062,13065,13061,
	5771,13063,5770,5769,5768,13064,13233,13231,
	15879,13230,13060,13232,13066,13229,15243,13367,
	13333,13365,5897,13366,13368,13466,5928,13465,
	13464,5929,5959,13576,13600,13599,13601,1040,
	7166,16218,13067,13467,1041,2051,7167,14404,
	8271,2964,8917,8916,3409,3834,10799,4618,
	4619,14363,14918,4933,5930,1042,17666,15868,
	7168,1676,7170,7169,2052,7676,15246,7683,
	2488,7681,7679,7687,2492,7685,7682,7680,
	7684,17143,15248,2966,8272,2971,8288,2970,
	8290,8278,2968,8289,8273,2969,8285,8281,
	8283,8277,8284,14364,2965,8280,16288,8279,
	8275,8282,8274,8276,2967,8287,3412,8928,
	8924,8926,8921,8920,8918,8925,8927,8923,
	15249,8922,8930,3842,9592,9598,14058,9599,
	3836,3843,9591,8919,3839,3837,9597,9596,
	3838,3840,9595,3835,15250,10224,4258,10235,
	10234,10229,4260,10233,4252,4254,15880,10231,
	10223,4256,4255,10227,10232,10230,4257,10228,
	18294,17146,8929,4253,10800,10807,4621,10811,
	4625,10803,10801,4620,10809,10802,4624,10808,
	11383,11395,4938,11390,11392,18295,4936,11384,
	11388,11391,4934,4937,11394,11393,11385,11389,
	4935,11855,11854,5234,11860,5236,5233,11856,
	5237,11857,11858,11861,15894,5235,12232,17379,
	12229,12231,15252,11859,12228,15254,5570,12568,
	12571,12573,12570,12572,5571,5569,5568,12569,
	12565,12849,5697,15251,12850,13069,5772,13071,
	5773,13068,13070,5846,13234,5845,13371,13370,
	13369,15850,13541,13540,6169,1043,18184,2053,
	8291,3413,9600,5415,14370,12574,16361,14371,
	1288,2972,16352,8931,3414,8933,8932,9601,
	3845,10236,10239,10238,10237,10812,10814,10813,
	15763,4940,14374,14373,4939,5238,11862,11864,
	11863,12576,12575,12851,5698,14375,5774,13236,
	13235,5960,17667,1289,7171,2054,8294,8292,
	9608,8934,9602,9606,9605,9607,3846,9604,
	9603,9609,14377,10242,10240,10241,10817,10815,
	10816,11396,12577,12852,5699,13072,13237,5847,
	13372,13542,1290,7173,2055,2496,2503,2495,
	2504,7688,2501,2498,2502,2500,2494,2981,
	15267,2975,8298,2980,2982,2976,17381,2974,
	15313,8297,2979,8295,2978,18298,8939,3426,
	3415,3419,15266,3865,9614,8941,3420,8947,
	16368,8949,8950,8938,8937,8951,3423,8948,
	8946,3421,3417,3428,3416,9613,3860,3855,
	3850,16270,3866,3851,3847,3862,3859,3861,
	3852,3857,9621,9618,9610,9615,3863,9622,
	3864,3854,16293,9612,9620,9616,9619,9617,
	3858,9611,3853,10245,4262,4265,10247,14383,
	4273,10246,4267,18299,4630,10248,4272,10244,
	4274,3856,4266,4264,4268,4270,4261,4275,
	4271,14397,4269,10243,14401,4637,10820,4641,
	10832,10825,4626,10833,10827,4636,10828,4635,
	10831,4629,10829,10819,4628,11407,4634,10822,
	4627,10821,4638,10826,10830,10834,14786,4642,
	4946,11402,11408,11405,11397,11409,11398,4941,
	4947,11410,15833,4943,4953,4948,11412,11406,
	4944,11399,4954,4955,4952,4632,4942,11413,
	4956,4949,11411,4945,5246,11871,11880,11877,
	11879,5243,11872,14390,11878,5239,11874,5247,
	11870,11875,11882,11403,11876,11869,5240,11868,
	5241,11883,5242,11881,5245,11867,14391,5244,
	11873,12238,12252,12247,12249,12237,5418,12236,
	5421,5420,16281,12251,12243,12240,12246,12241,
	12250,12248,5419,12255,12253,12244,12254,12245,
	14394,12581,5572,12588,16370,13075,5579,12242,
	12579,5575,12580,12585,12578,12587,12589,12582,
	12584,12586,12583,12590,5574,5580,5576,5573,
	12855,15800,5704,12853,15828,12857,12858,5702,
	12859,12856,15351,12854,5705,5701,12860,5703,
	5700,5776,13074,13073,5777,13238,13243,5848,
	15757,13240,13239,13242,13241,5898,13375,13373,
	5932,5931,5972,14400,5978,13602,13631,17669,
	8300,9623,10249,10835,5249,5248,12256,12591,
	13377,1292,7689,2505,2983,9625,9624,4643,
	10836,4644,11884,5422,18301,5987,1293,6728,
	7690,3429,3867,9627,9626,9628,4276,4957,
	4645,4958,11888,11885,11887,11886,5250,12257,
	12593,12592,6375,2507,2506,7691,8954,8953,
	3430,3869,3868,4278,4277,10837,11416,4959,
	11890,11889,12260,12259,12258,12594,14405,13603,
	1294,2058,15277,2059,7693,7692,8304,2990,
	2988,2985,2989,2987,2986,16369,3431,8957,
	3874,3433,3438,8959,3440,3439,3442,8958,
	8955,3436,3432,3434,8956,3441,3437,3435,
	3876,3875,3873,3877,9632,16025,4279,18302,
	10252,14411,10254,10840,10842,14417,4654,10843,
	4647,10839,4646,4656,4649,10838,4648,10844,
	10841,15278,4655,16284,11420,11419,18303,11421,
	4960,11417,16305,5254,11891,5251,5255,5253,
	5252,12261,12263,14415,12262,5423,5581,12597,
	5582,17779,16278,5707,5706,13078,14419,14414,
	5778,13079,13244,15983,13378,5935,18304,17670,
	2992,2991,4282,11422,4657,11423,11892,1296,
	7174,2061,2060,7694,2510,7695,8307,14817,
	8306,8961,3445,16233,8960,3444,8962,3443,
	9636,9639,9642,9635,9643,4284,10255,10848,
	10846,4658,10845,10847,11424,16375,11425,5256,
	14422,12599,12601,12598,12600,12861,13080,13577,
	1297,2062,7696,7698,7697,8310,2994,2993,
	8311,17258,8312,8309,18306,8974,3453,8966,
	8972,3448,3451,8965,3446,8969,9649,3450,
	8964,16258,8970,8967,8973,8963,8971,3449,
	3452,8968,3447,3879,9645,3878,9652,9651,
	3885,3886,9653,3880,9650,3884,9655,9646,
	9648,3881,9644,9647,9654,3882,9656,4285,
	10260,10266,10258,10267,16261,10257,10262,10268,
	10265,10264,10263,10256,10261,10259,15966,4664,
	4661,10857,10862,10864,10861,10854,10856,10866,
	10863,10858,10867,10855,10865,4662,4668,4666,
	10849,4667,4663,10850,10852,10851,16264,4665,
	15284,4660,16043,10853,11435,4962,11426,4963,
	4966,11431,11428,4965,18308,11427,11436,11429,
	11434,4964,4961,11430,17170,11898,5259,5257,
	5260,5258,11897,11894,11893,16259,11896,11895,
	12278,5429,12269,12268,5424,12274,12273,12265,
	12276,12270,5428,12267,12275,12266,5425,5427,
	12271,5426,12272,17229,12277,12605,5587,12602,
	17383,5584,12604,5589,16265,5586,16255,12603,
	5588,12607,5583,15959,5709,5711,12864,5710,
	12862,5585,12863,5708,5779,5781,13082,5780,
	13081,13249,5851,13248,5852,13246,13245,13547,
	13546,13545,13578,15285,13623,13604,5979,1298,
	2511,14430,14496,11437,12608,14432,13605,1299,
	1677,2064,2063,18312,2514,7699,2512,7700,
	2513,8317,8313,15806,2995,8316,8315,8314,
	2996,18314,8981,8979,8986,8988,18315,8980,
	8975,8987,18316,8989,8982,8985,8976,3455,
	8978,8977,3454,3456,3890,9658,9660,9659,
	9664,3888,9661,9657,3889,3891,9665,10272,
	10271,4287,4289,4286,4288,10870,14434,10871,
	4671,4676,4670,4677,4672,10872,10873,10869,
	4678,4674,4673,4675,10868,14435,11439,4968,
	15287,11441,4970,11444,11440,11438,11443,4969,
	11442,5262,5265,11900,15623,5263,11899,5261,
	5264,5431,15288,12282,5432,5591,12610,12609,
	12613,5590,12868,12869,12865,12870,13084,16177,
	13085,13083,5782,13252,5853,13253,13381,13380,
	1300,3457,3892,15764,14438,4972,5592,5679,
	5783,1301,2515,3893,14442,12614,560,18387,
	17405,17142,18633,18631,18630,6377,18317,1302,
	1305,1304,1303,6376,18318,6730,18319,6732,
	1678,18606,1680,6729,6731,1679,6734,6733,
	6735,7178,7176,2067,7175,2072,2069,2066,
	2068,2071,2070,7177,2521,7702,7708,2518,
	2525,14449,2520,7705,7707,2523,7701,7704,
	2519,2516,7703,2522,7706,2524,2517,8321,
	8324,2998,3008,3004,8322,15950,3012,3005,
	3010,3000,2997,2999,3011,8323,3003,3006,
	3002,3007,8325,3009,3001,8320,8991,3458,
	8993,3460,3462,18320,8992,3459,3463,14454,
	3461,3899,9670,3907,9667,3908,3897,9669,
	3895,3894,16144,3905,3904,3903,3901,3906,
	9666,3896,3898,3900,15826,4292,4295,4299,
	4293,4298,4296,4291,14457,4297,4294,15464,
	10879,4681,4679,10880,15712,4682,4680,10878,
	10877,4976,10876,4974,4973,11445,4683,4975,
	11446,4978,11447,4977,5267,11902,5266,5271,
	5434,5268,11901,11448,5433,5435,5900,5899,
	1307,6174,2526,6381,6378,6383,6379,1308,
	1311,6380,1310,6382,14460,14464,6738,6740,
	1683,6736,6739,6737,1681,1684,7180,7184,
	7181,2077,14462,7183,7185,7187,2074,7186,
	8332,7182,7709,7718,7716,7717,7719,2528,
	7714,7710,2527,2529,7713,7720,7715,3013,
	8335,8327,8326,8328,7179,8329,3465,9000,
	11449,9001,8998,3015,3467,8997,9003,9002,
	3464,9006,8996,9005,8994,9004,8999,3466,
	9672,9678,9673,9671,9674,3909,9675,3910,
	4301,4300,10279,10282,10274,10278,4302,10277,
	10275,10281,10276,10273,10280,10886,4686,12283,
	10882,8995,4685,10887,10881,4684,4687,10885,
	11450,4979,11903,5272,12286,12616,12615,12873,
	13086,13254,5854,1312,2079,2078,2532,2531,
	7721,16201,2530,8339,8340,8336,3016,8337,
	16197,8338,17387,9009,9011,9007,9010,3468,
	9008,3469,14466,9186,3913,3912,10284,10286,
	4306,4303,16027,4305,4304,10285,10283,4691,
	10893,4689,10890,4690,17388,11456,16206,11457,
	4980,11453,11907,11904,11906,5275,11910,5274,
	11908,11911,11909,11905,5273,12289,12287,14467,
	12291,12618,5596,12290,12617,5595,5712,12875,
	17903,12874,13089,5784,15304,13382,13475,13474,
	5936,5962,13476,13579,14468,5980,1313,1685,
	5713,1314,2080,3017,3470,5438,1686,7189,
	7188,7724,16734,2535,2533,2537,7726,2536,
	2534,7725,3020,8346,15366,3019,3021,8351,
	3023,8341,3022,8352,8344,8342,3018,8350,
	8345,8347,15096,9023,14382,9022,9020,9012,
	9027,9015,9025,9032,9024,3478,16057,3474,
	9013,9019,3476,14472,9018,3477,3479,9021,
	18323,9033,9031,9030,9016,3473,9029,9014,
	9017,9026,18325,15374,15270,10321,9690,9683,
	9702,9711,9686,3925,15142,9699,3915,3917,
	3929,9684,16850,3918,3920,3930,3919,14474,
	14490,3928,9692,9681,3926,9691,3922,9709,
	3927,9707,9695,9705,3924,9682,9708,9703,
	15943,9710,3916,18326,3931,3921,14851,9688,
	15584,9696,9700,14475,9704,3923,9687,9685,
	9697,18327,9694,9693,15621,9698,9701,10289,
	15460,4307,10299,10291,4313,4317,10308,10316,
	10301,4309,15873,10305,9689,15545,4310,10312,
	10317,10309,15029,10311,10320,10313,10295,10304,
	18109,4314,10293,10306,4312,10300,4311,10314,
	10297,10290,4315,10319,10292,10310,10307,10302,
	10288,10287,10296,10315,4316,10318,10294,10298,
	4308,18328,4703,4698,10928,10924,10900,4694,
	14057,15349,4693,4699,15456,18330,10898,4697,
	10910,4692,10927,4701,10913,10918,10912,11475,
	10908,10916,10914,10901,10895,10922,4700,10923,
	14433,10917,10906,10904,10899,10907,10920,10903,
	10919,10925,18331,4696,15522,10915,10921,10911,
	10905,4695,18332,14692,10303,10909,4702,10902,
	15026,15364,11480,4983,11479,11477,11495,4987,
	18335,11492,11484,11494,11470,11482,14439,4989,
	11462,15121,11460,11493,11491,11490,11474,11473,
	11463,4991,11469,11478,11489,4994,11496,5283,
	4996,4990,11467,11476,11459,11461,4981,4993,
	4986,11468,11483,10926,4992,11458,5278,11485,
	11488,4988,14482,11472,4995,4985,15875,15549,
	4984,11481,11487,4982,11465,11486,16725,17015,
	15098,18336,16406,15988,11471,11917,15457,11922,
	5280,5282,11928,5276,11942,11934,5289,11932,
	11916,11931,11919,11943,5288,5286,11920,11939,
	15547,11925,11938,17000,11940,11915,5281,18108,
	11941,15344,14689,11929,14483,5285,11926,14824,
	5287,11935,14494,15588,5279,11921,11936,14585,
	11930,11464,11937,11918,15397,5284,12308,14693,
	5277,11913,18337,17085,15552,15072,12305,12298,
	5440,12293,12309,12300,11927,15676,12311,12294,
	5439,12304,5441,5449,5447,12306,5448,12296,
	14764,12297,12302,12307,15025,11912,5442,12316,
	12313,12640,12303,12299,12314,12629,12301,5445,
	18338,5444,11914,5446,12310,12312,5443,12315,
	12292,12295,15504,17399,15503,15917,12638,12622,
	5600,12636,14762,12620,5601,12631,14694,12627,
	5606,12637,12621,12624,5598,12639,16609,12632,
	12635,5604,5609,5607,12628,12623,15455,12619,
	5599,14880,5597,5605,12634,5608,12630,12641,
	5610,16959,17066,12890,12905,12892,12899,12625,
	12897,12880,12903,5716,12884,12902,12891,15572,
	5715,15305,12907,12893,14885,12898,12901,12878,
	12904,12889,12894,12882,12886,12906,12896,12879,
	12883,12885,12895,18341,5714,12888,14659,14544,
	14499,15309,12887,13093,13097,5791,13099,18343,
	5785,13095,13101,5790,5786,5787,13092,13098,
	5789,13096,5788,13091,12900,13094,18344,13090,
	13100,17010,17013,5855,13261,13264,13260,13255,
	15461,13259,13256,18346,13384,13386,15408,13385,
	13387,13383,5901,5903,15838,5937,13477,14206,
	13549,13548,13550,5964,13551,5963,13552,13580,
	13583,13582,5973,13581,18347,16795,5983,13624,
	13632,1687,17672,8353,9034,10929,12642,17673,
	1688,2081,2538,14497,8354,8355,3024,3482,
	9036,9035,3486,3481,9037,3483,3485,3484,
	3480,15317,15316,3932,15314,9715,9713,9714,
	9712,14500,4319,18349,4322,4324,4323,14498,
	15315,17301,10931,10930,4704,10932,10933,4705,
	11504,11502,11949,4997,11497,11505,11499,11503,
	11945,11948,5294,11944,5293,11946,5452,12319,
	5450,5453,5451,12908,12910,12909,5717,5792,
	13104,13103,13102,1689,17689,6068,6177,1044,
	6175,6178,6176,1319,6387,1316,6386,6384,
	1317,1315,18352,16337,6741,6744,1692,6743,
	6742,1691,1690,16189,1694,6745,1693,18290,
	7195,7191,2082,7192,7193,7190,2542,7727,
	2543,2546,7728,18270,3025,8358,3033,8357,
	8360,3029,8359,3494,3027,3030,3026,14508,
	8356,3490,9039,9043,16198,9042,3495,3487,
	3489,3493,3488,15326,14509,9719,3933,4325,
	4327,4326,10323,10935,14511,10934,17135,11506,
	5000,11951,5295,12320,5612,5296,1695,2547,
	7729,14517,8361,3034,3496,9044,9046,3498,
	3497,3937,4328,3936,9720,4329,10936,11507,
	5001,5297,12321,12325,5456,12322,5457,5455,
	12326,5458,12324,12645,5454,5613,1696,3036,
	3035,3501,9048,9047,3502,14852,9724,3943,
	3940,9723,3942,9721,3941,10324,4330,10939,
	16141,4706,10937,10938,4709,5005,5002,5007,
	11510,5003,11509,5006,11955,11957,14537,5298,
	11956,5299,11954,11953,16307,12328,12327,5459,
	12329,12650,5615,12649,12646,5614,5718,14528,
	5795,13105,14530,13265,18355,15337,5938,13482,
	5940,13481,13483,5939,13553,17635,1697,3944,
	14540,10325,15343,1698,14543,4710,5616,2086,
	5904,2087,8362,9049,16023,9051,9050,15356,
	9728,9725,3945,10327,10332,4331,10326,10328,
	10335,10330,10334,10331,10329,10333,4332,16576,
	10943,14548,10941,4712,4711,10942,4713,11512,
	5011,11511,11958,15355,11960,11962,11961,14549,
	5300,11959,12336,5460,12334,12337,5461,12335,
	12333,12330,5462,12331,14550,18357,12651,16179,
	12653,12652,15765,12913,16181,12914,18766,14551,
	13107,14552,5861,18358,5860,5941,13484,16175,
	5974,2088,3503,10337,10336,10945,10944,14553,
	5301,11964,11963,12340,12338,12341,12339,14554,
	13108,13485,17676,2089,14556,11514,11965,14669,
	2090,14560,4333,11515,5463,12342,5618,5796,
	13390,2091,9052,3506,3951,3949,18770,17212,
	10338,10952,4716,10947,10946,15362,10951,4714,
	15927,5018,10948,10953,10950,10949,5016,16242,
	11517,5012,11518,14565,14564,11516,5015,5013,
	5017,11971,11967,11970,5302,15766,11762,11969,
	11968,14566,5466,5464,5467,5465,12660,5620,
	12658,5621,12657,12659,5619,12920,12917,13109,
	5798,14567,5797,13268,13267,5862,16437,5905,
	5942,5965,13554,2092,9053,4337,10954,4717,
	16243,5303,14575,12346,18360,5469,12663,5622,
	12662,12664,12661,13112,15224,5719,13111,14577,
	14576,13110,14574,13269,13609,2093,15371,17679,
	2094,18322,14582,2548,7730,8364,14583,3507,
	3510,3508,9054,14584,3512,18362,3511,3954,
	17253,9733,9734,10956,10341,4338,10343,10955,
	4341,4718,5019,10957,4720,4719,10958,10961,
	4721,18364,5024,5022,17390,5021,11520,11519,
	11972,11521,5020,11522,11975,11979,5473,11978,
	11981,11980,11977,11976,18365,15357,14586,12349,
	15377,12348,5472,16180,12347,17407,12667,12666,
	12665,5624,16313,12927,12926,15378,12924,12923,
	5721,5720,12928,13270,5863,13114,5799,13113,
	12925,13271,5906,18366,5966,13555,13610,17391,
	17680,2095,8365,11982,2096,15381,10345,10344,
	11523,11983,16835,5474,15383,13611,2549,3514,
	3513,9735,9737,3958,9736,3957,3959,9738,
	14667,10347,10349,10348,15394,10351,4342,10350,
	10346,15735,18369,10971,10973,10963,10967,10964,
	4723,4726,4728,4727,10969,10972,10970,4729,
	16194,4725,10962,4722,14213,5026,11529,11527,
	11526,15205,11532,11531,11530,11525,5025,11524,
	11528,5027,11987,14591,11991,11988,11990,11989,
	11986,11992,11995,5305,11985,11996,12354,12357,
	12359,12358,12352,12356,12351,14592,12353,5475,
	12350,14594,12355,12682,12671,5625,5626,12670,
	12675,12681,12673,12680,15212,12677,15388,12676,
	12679,12674,12672,15858,12678,12935,12933,5723,
	12932,12938,12936,15390,5724,12931,12929,12930,
	5726,12934,5725,12937,13115,13121,14593,18371,
	13116,5803,13122,5802,13120,13119,5801,13118,
	5800,13117,13277,13279,13278,13275,5865,13272,
	13276,13281,5864,13391,5909,14595,13392,13486,
	5943,13584,5975,15207,13612,5976,13628,13627,
	5991,13636,17797,10353,17202,10974,11535,17392,
	4730,5028,11533,16244,11536,5029,15767,11997,
	11534,12360,5476,12683,12941,12939,5804,13282,
	5912,5866,13487,5967,16298,2551,7731,14600,
	3960,14601,10354,4345,10355,10976,10975,5031,
	10979,10978,10977,14603,11537,11540,11539,11538,
	5030,12001,11999,11998,16151,12000,16146,5477,
	12362,16160,12685,12684,12687,5627,12686,18372,
	12942,12944,12943,14605,13124,13127,13283,5867,
	5944,5968,13615,2552,4733,5032,12365,16153,
	13284,14606,13585,7732,5992,2553,11541,14609,
	12366,16128,13128,13285,2554,10981,4735,4734,
	10980,12367,12369,12368,5481,5480,5479,14612,
	5806,13129,5805,13287,13286,5945,13488,3041,
	9741,10985,10982,10989,18373,4737,10988,10984,
	10983,10987,10986,4736,11552,11547,11545,11554,
	11546,15419,11542,17393,11548,16921,11543,12011,
	11544,11555,17955,14617,16301,11551,5033,11550,
	11549,12003,12006,16214,12005,15422,12013,12008,
	12012,12010,12009,12007,5309,5308,5310,5307,
	12014,16308,12375,12378,12376,12380,12373,5486,
	12374,12372,12379,12371,12370,5485,5483,5482,
	12377,12697,12703,5630,12698,12701,12705,5631,
	12700,18374,12692,12691,12704,12695,12693,12702,
	12694,5629,5628,15297,12699,16087,12696,18375,
	12947,12950,12948,12954,12952,12946,12957,16186,
	12945,14620,5728,12953,12949,5727,12956,16158,
	18715,13132,13131,17954,18378,13138,13134,13137,
	5808,17894,13130,13133,5807,16162,15751,5869,
	13298,13293,13297,15417,13300,13296,13292,15742,
	5871,13295,13299,5870,13290,12951,13294,13291,
	15985,13288,13289,5868,13399,13401,13404,13400,
	13405,15414,13398,13396,15415,5913,13402,5915,
	5914,13397,13403,16063,13493,5946,13494,13492,
	13489,13491,13490,13556,13558,13557,13588,16033,
	13587,13586,13589,13633,13637,15416,17682,3042,
	9055,9744,3961,9742,4350,4348,10363,4349,
	10996,10999,10993,11000,10992,10994,10995,10998,
	16187,4740,11003,11002,4738,10997,11001,4739,
	11560,5034,15484,11557,11563,11566,11562,11559,
	11567,11558,15425,11564,5035,11556,5036,5037,
	11561,12023,12021,12015,16458,12018,12028,12022,
	14638,12030,12027,5312,12029,12016,16299,12024,
	18381,12385,12387,12390,17721,12384,16074,5487,
	12392,12383,12393,12389,12391,12395,12388,12396,
	12386,12382,5488,15982,12394,5489,5633,12017,
	12731,12722,5635,12718,5636,15276,17211,12727,
	12721,12711,5634,12724,12706,12713,12728,12725,
	16126,12717,14629,12729,12712,12707,12726,14630,
	12709,12723,12714,12730,12710,5632,12708,12962,
	17395,12967,12979,12970,12963,12959,12972,12965,
	12968,12961,12975,12958,12966,12971,12973,12976,
	12960,15435,12974,12978,12969,16176,13154,13157,
	13156,13161,13159,5810,13150,13140,13139,5812,
	13149,13141,18384,13146,12977,17394,13142,5811,
	13148,14633,13147,13143,13155,17785,13144,13305,
	13158,13160,13151,13145,13306,13308,13302,5872,
	14622,13315,13313,5873,13312,13311,13304,13301,
	13307,13316,13303,13309,13414,13419,13409,13412,
	5916,13416,13423,13310,13410,13420,13424,13415,
	13408,13418,13417,13422,13421,13314,13413,13407,
	13411,13406,18385,13505,13496,13504,13495,13498,
	13506,13500,13499,13497,13503,17790,13559,17786,
	13560,14632,13562,13561,13564,13563,13591,13590,
	13616,13617,15428,13629,5989,5993,16260,13638,
	5994,17683,5729,13162,17400,3044,9745,16143,
	3962,11004,16246,16114,11568,12034,12033,5313,
	12398,12035,12397,14641,12733,5637,5639,12732,
	14642,16316,5638,16167,12982,12984,12983,13163,
	5813,14643,5917,13507,13565,14644,13642,3045,
	18756,10364,16245,4741,14646,16018,17396,11572,
	11571,14647,12036,18388,5640,5730,13317,13634,
	3046,4351,18389,4742,15736,12734,14649,13425,
	3515,17684,18390,17783,12037,5969,3516,4743,
	5314,13426,3517,11005,5041,12038,5318,5315,
	12399,5490,12986,12985,12988,12987,5731,13166,
	13165,5814,13318,18393,5918,13566,13592,5986,
	18394,9056,11575,12039,12735,9746,18382,12040,
	12401,12400,14656,13429,5951,14657,13567,13593,
	11576,3964,5491,12402,14661,13168,5815,13169,
	13167,13430,12043,12042,12041,12403,16272,17952,
	12407,12406,12405,12404,5492,12736,12990,5732,
	12989,13170,13321,13320,5874,13319,13433,13432,
	13431,14663,11577,5319,13322,13434,18395,14666,
	13509,13568,13618,13644,4353,5320,12408,12739,
	13171,13435,18755,4745,16256,12044,12409,12995,
	16163,12991,5816,12994,12993,5733,12992,5735,
	14671,5734,13173,13172,14672,5876,13323,5875,
	13437,13436,5953,5952,13569,13572,13571,13570,
	13643,18048,5042,14674,5500,12996,13174,5877,
	13324,13645,17799,17990,12045,13594,16240,18771,
	16792,16895,18776,16897,14247,18080,14301,18084,
	14386,18091,14770,18107,15704,18129,18138,15707,
	18139,18439,17330,18500,15785,18513,15804,17360,
	18515,15805,18522,15814,18523,15816,18527,18534,
	15825,18535,15827,18538,15832,15834,18539,15835,
	18540,15844,18544,15851,18548,15874,18563,18564,
	15876,18566,15878,18571,15903,15920,18577,15925,
	18579,15938,18583,17384,18607,15949,18636,15957,
	18638,15960,15961,18641,18642,15963,18646,15964,
	18678,16061,18679,16069,18683,16071,18684,18701,
	16092,18706,16119,18709,16122,16211,18720,18728,
	16291,17189,17848,17849,17210,17851,17219,17852,
	17294,17886,17887,17296,17888,17297,17889,17303,
	17896,628,9089,109,122,13743,13745,13746,
	112,108,14052,174,259,264,175,14051,
	176,208,100,209,103,257,106,105,
	214,216,215,107,265,13758,258,13759,
	13748,197,132,120,133,
};

pdf_cmap pdf_cmap_UniCNS_UCS2_H =
{
	-1, "UniCNS-UCS2-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 2, 0xe000, 0xffff },
	},
	16418, 16418, (pdf_range*) pdf_cmap_UniCNS_UCS2_H_ranges,
	13509, 13509, (unsigned short*) pdf_cmap_UniCNS_UCS2_H_table,
};

/* UniCNS-UCS2-V */

static const pdf_range pdf_cmap_UniCNS_UCS2_V_ranges[] =
{
	{ 0x2013, 0x2014, PDF_CMAP_TABLE, 0 },
	{ 0x2025, 0x2025, PDF_CMAP_SINGLE, 109 },
	{ 0x3008, 0x3009, PDF_CMAP_RANGE, 150 },
	{ 0x300a, 0x300b, PDF_CMAP_RANGE, 146 },
	{ 0x300c, 0x300d, PDF_CMAP_RANGE, 154 },
	{ 0x300e, 0x300f, PDF_CMAP_RANGE, 158 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 142 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 138 },
	{ 0xfe4f, 0xfe4f, PDF_CMAP_SINGLE, 13745 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 130 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 134 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 135 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 135 },
};

static const unsigned short pdf_cmap_UniCNS_UCS2_V_table[2] =
{
	120,122,
};

pdf_cmap pdf_cmap_UniCNS_UCS2_V =
{
	-1, "UniCNS-UCS2-V", "UniCNS-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	13, 13, (pdf_range*) pdf_cmap_UniCNS_UCS2_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_UniCNS_UCS2_V_table,
};

/* UniCNS-UTF16-H */

static const pdf_range pdf_cmap_UniCNS_UTF16_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a2, 0x00a3, PDF_CMAP_RANGE, 262 },
	{ 0x00a5, 0x00a5, PDF_CMAP_SINGLE, 260 },
	{ 0x00a7, 0x00a8, PDF_CMAP_TABLE, 0 },
	{ 0x00ac, 0x00ac, PDF_CMAP_SINGLE, 14049 },
	{ 0x00af, 0x00b1, PDF_CMAP_TABLE, 2 },
	{ 0x00b7, 0x00b7, PDF_CMAP_SINGLE, 115 },
	{ 0x00c0, 0x00c1, PDF_CMAP_TABLE, 5 },
	{ 0x00c8, 0x00ca, PDF_CMAP_TABLE, 7 },
	{ 0x00d2, 0x00d3, PDF_CMAP_TABLE, 10 },
	{ 0x00d7, 0x00d7, PDF_CMAP_SINGLE, 210 },
	{ 0x00e0, 0x00e1, PDF_CMAP_TABLE, 12 },
	{ 0x00e8, 0x00ea, PDF_CMAP_TABLE, 14 },
	{ 0x00ec, 0x00ed, PDF_CMAP_TABLE, 17 },
	{ 0x00f2, 0x00f3, PDF_CMAP_TABLE, 19 },
	{ 0x00f7, 0x00fa, PDF_CMAP_TABLE, 21 },
	{ 0x00fc, 0x00fc, PDF_CMAP_SINGLE, 18827 },
	{ 0x0100, 0x0101, PDF_CMAP_TABLE, 25 },
	{ 0x0112, 0x0113, PDF_CMAP_TABLE, 27 },
	{ 0x011a, 0x011b, PDF_CMAP_TABLE, 29 },
	{ 0x012b, 0x012b, PDF_CMAP_SINGLE, 18811 },
	{ 0x014b, 0x014d, PDF_CMAP_TABLE, 31 },
	{ 0x0153, 0x0153, PDF_CMAP_SINGLE, 18839 },
	{ 0x016b, 0x016b, PDF_CMAP_SINGLE, 18819 },
	{ 0x01cd, 0x01ce, PDF_CMAP_TABLE, 34 },
	{ 0x01d0, 0x01d2, PDF_CMAP_TABLE, 36 },
	{ 0x01d4, 0x01d4, PDF_CMAP_SINGLE, 18821 },
	{ 0x01d6, 0x01d6, PDF_CMAP_SINGLE, 18823 },
	{ 0x01d8, 0x01d8, PDF_CMAP_SINGLE, 18824 },
	{ 0x01da, 0x01da, PDF_CMAP_SINGLE, 18825 },
	{ 0x01dc, 0x01dc, PDF_CMAP_SINGLE, 18826 },
	{ 0x0250, 0x0251, PDF_CMAP_TABLE, 39 },
	{ 0x0254, 0x0254, PDF_CMAP_SINGLE, 18837 },
	{ 0x025b, 0x025b, PDF_CMAP_SINGLE, 18836 },
	{ 0x0261, 0x0261, PDF_CMAP_SINGLE, 18833 },
	{ 0x026a, 0x026a, PDF_CMAP_SINGLE, 18843 },
	{ 0x0275, 0x0275, PDF_CMAP_SINGLE, 18838 },
	{ 0x0283, 0x0283, PDF_CMAP_SINGLE, 18834 },
	{ 0x028a, 0x028a, PDF_CMAP_SINGLE, 18842 },
	{ 0x02c6, 0x02c7, PDF_CMAP_TABLE, 41 },
	{ 0x02ca, 0x02cb, PDF_CMAP_TABLE, 43 },
	{ 0x02cd, 0x02cd, PDF_CMAP_SINGLE, 198 },
	{ 0x02d9, 0x02d9, PDF_CMAP_SINGLE, 502 },
	{ 0x0304, 0x0304, PDF_CMAP_SINGLE, 195 },
	{ 0x0308, 0x0308, PDF_CMAP_SINGLE, 13747 },
	{ 0x030c, 0x030c, PDF_CMAP_SINGLE, 504 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 417 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 434 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 441 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 458 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 13936 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 13930 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 13937 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 13970 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 13969 },
	{ 0x1ebe, 0x1ec1, PDF_CMAP_TABLE, 45 },
	{ 0x2013, 0x2014, PDF_CMAP_TABLE, 49 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 166 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 168 },
	{ 0x2022, 0x2022, PDF_CMAP_SINGLE, 104 },
	{ 0x2025, 0x2027, PDF_CMAP_TABLE, 51 },
	{ 0x2032, 0x2032, PDF_CMAP_SINGLE, 173 },
	{ 0x2035, 0x2035, PDF_CMAP_SINGLE, 172 },
	{ 0x203b, 0x203b, PDF_CMAP_SINGLE, 177 },
	{ 0x203e, 0x203e, PDF_CMAP_SINGLE, 195 },
	{ 0x20ac, 0x20ac, PDF_CMAP_SINGLE, 17601 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 266 },
	{ 0x2105, 0x2105, PDF_CMAP_SINGLE, 194 },
	{ 0x2109, 0x2109, PDF_CMAP_SINGLE, 267 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 14054 },
	{ 0x2121, 0x2121, PDF_CMAP_SINGLE, 14055 },
	{ 0x2160, 0x2169, PDF_CMAP_RANGE, 343 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 526 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 54 },
	{ 0x2196, 0x2197, PDF_CMAP_RANGE, 249 },
	{ 0x2198, 0x2199, PDF_CMAP_TABLE, 58 },
	{ 0x21b8, 0x21b9, PDF_CMAP_RANGE, 13997 },
	{ 0x21e7, 0x21e7, PDF_CMAP_SINGLE, 13996 },
	{ 0x2215, 0x2215, PDF_CMAP_SINGLE, 257 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 213 },
	{ 0x221e, 0x2220, PDF_CMAP_TABLE, 60 },
	{ 0x2223, 0x2223, PDF_CMAP_SINGLE, 254 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 253 },
	{ 0x2229, 0x222a, PDF_CMAP_RANGE, 229 },
	{ 0x222b, 0x222b, PDF_CMAP_SINGLE, 237 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 238 },
	{ 0x2234, 0x2235, PDF_CMAP_TABLE, 63 },
	{ 0x223c, 0x223c, PDF_CMAP_SINGLE, 228 },
	{ 0x2252, 0x2252, PDF_CMAP_SINGLE, 221 },
	{ 0x2260, 0x2261, PDF_CMAP_TABLE, 65 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 217 },
	{ 0x2295, 0x2295, PDF_CMAP_SINGLE, 243 },
	{ 0x2299, 0x2299, PDF_CMAP_SINGLE, 244 },
	{ 0x22a5, 0x22a5, PDF_CMAP_SINGLE, 231 },
	{ 0x22bf, 0x22bf, PDF_CMAP_SINGLE, 234 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 110 },
	{ 0x23da, 0x23db, PDF_CMAP_RANGE, 18844 },
	{ 0x2400, 0x241f, PDF_CMAP_RANGE, 562 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 594 },
	{ 0x2460, 0x2469, PDF_CMAP_RANGE, 506 },
	{ 0x2474, 0x247d, PDF_CMAP_RANGE, 516 },
	{ 0x2500, 0x2500, PDF_CMAP_SINGLE, 311 },
	{ 0x2502, 0x2502, PDF_CMAP_SINGLE, 312 },
	{ 0x250c, 0x250c, PDF_CMAP_SINGLE, 314 },
	{ 0x2510, 0x2510, PDF_CMAP_SINGLE, 315 },
	{ 0x2514, 0x2514, PDF_CMAP_SINGLE, 316 },
	{ 0x2518, 0x2518, PDF_CMAP_SINGLE, 317 },
	{ 0x251c, 0x251c, PDF_CMAP_SINGLE, 309 },
	{ 0x2524, 0x2524, PDF_CMAP_SINGLE, 308 },
	{ 0x252c, 0x252c, PDF_CMAP_SINGLE, 307 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 306 },
	{ 0x253c, 0x253c, PDF_CMAP_SINGLE, 305 },
	{ 0x2550, 0x256c, PDF_CMAP_TABLE, 67 },
	{ 0x256d, 0x256e, PDF_CMAP_RANGE, 318 },
	{ 0x256f, 0x2570, PDF_CMAP_TABLE, 96 },
	{ 0x2571, 0x2573, PDF_CMAP_RANGE, 330 },
	{ 0x2574, 0x2574, PDF_CMAP_SINGLE, 13744 },
	{ 0x2581, 0x2588, PDF_CMAP_RANGE, 290 },
	{ 0x2589, 0x258f, PDF_CMAP_TABLE, 98 },
	{ 0x2593, 0x2595, PDF_CMAP_TABLE, 105 },
	{ 0x25a0, 0x25a1, PDF_CMAP_TABLE, 108 },
	{ 0x25b2, 0x25b3, PDF_CMAP_TABLE, 110 },
	{ 0x25bc, 0x25bd, PDF_CMAP_TABLE, 112 },
	{ 0x25c6, 0x25c7, PDF_CMAP_TABLE, 114 },
	{ 0x25cb, 0x25cb, PDF_CMAP_SINGLE, 180 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 116 },
	{ 0x25e2, 0x25e3, PDF_CMAP_RANGE, 326 },
	{ 0x25e4, 0x25e5, PDF_CMAP_TABLE, 118 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 120 },
	{ 0x2609, 0x2609, PDF_CMAP_SINGLE, 244 },
	{ 0x2640, 0x2642, PDF_CMAP_TABLE, 122 },
	{ 0x273d, 0x273d, PDF_CMAP_SINGLE, 13760 },
	{ 0x2e80, 0x2e80, PDF_CMAP_SINGLE, 17608 },
	{ 0x2e84, 0x2e84, PDF_CMAP_SINGLE, 17692 },
	{ 0x2e86, 0x2e88, PDF_CMAP_RANGE, 17693 },
	{ 0x2e8a, 0x2e8a, PDF_CMAP_SINGLE, 17696 },
	{ 0x2e8c, 0x2e8d, PDF_CMAP_RANGE, 17697 },
	{ 0x2e95, 0x2e95, PDF_CMAP_SINGLE, 17699 },
	{ 0x2e9c, 0x2e9d, PDF_CMAP_TABLE, 125 },
	{ 0x2ea5, 0x2ea5, PDF_CMAP_SINGLE, 17701 },
	{ 0x2ea7, 0x2ea7, PDF_CMAP_SINGLE, 17702 },
	{ 0x2eaa, 0x2eaa, PDF_CMAP_SINGLE, 17703 },
	{ 0x2eac, 0x2eac, PDF_CMAP_SINGLE, 17704 },
	{ 0x2eae, 0x2eae, PDF_CMAP_SINGLE, 17705 },
	{ 0x2eb6, 0x2eb6, PDF_CMAP_SINGLE, 17706 },
	{ 0x2ebc, 0x2ebc, PDF_CMAP_SINGLE, 17707 },
	{ 0x2ebe, 0x2ebe, PDF_CMAP_SINGLE, 17708 },
	{ 0x2ec6, 0x2ec6, PDF_CMAP_SINGLE, 18847 },
	{ 0x2eca, 0x2eca, PDF_CMAP_SINGLE, 17709 },
	{ 0x2ecc, 0x2ecd, PDF_CMAP_RANGE, 17710 },
	{ 0x2ecf, 0x2ecf, PDF_CMAP_SINGLE, 17712 },
	{ 0x2ed6, 0x2ed7, PDF_CMAP_RANGE, 17713 },
	{ 0x2ede, 0x2ede, PDF_CMAP_SINGLE, 17715 },
	{ 0x2ee3, 0x2ee3, PDF_CMAP_SINGLE, 18848 },
	{ 0x2f00, 0x2f00, PDF_CMAP_SINGLE, 595 },
	{ 0x2f01, 0x2f03, PDF_CMAP_RANGE, 536 },
	{ 0x2f04, 0x2f07, PDF_CMAP_TABLE, 127 },
	{ 0x2f08, 0x2f0b, PDF_CMAP_RANGE, 603 },
	{ 0x2f0c, 0x2f0e, PDF_CMAP_RANGE, 541 },
	{ 0x2f0f, 0x2f16, PDF_CMAP_TABLE, 131 },
	{ 0x2f17, 0x2f18, PDF_CMAP_RANGE, 612 },
	{ 0x2f19, 0x2f1e, PDF_CMAP_TABLE, 139 },
	{ 0x2f1f, 0x2f20, PDF_CMAP_RANGE, 634 },
	{ 0x2f21, 0x2f22, PDF_CMAP_TABLE, 145 },
	{ 0x2f23, 0x2f26, PDF_CMAP_RANGE, 636 },
	{ 0x2f27, 0x2f27, PDF_CMAP_SINGLE, 549 },
	{ 0x2f28, 0x2f2b, PDF_CMAP_RANGE, 642 },
	{ 0x2f2c, 0x2f2e, PDF_CMAP_TABLE, 147 },
	{ 0x2f2f, 0x2f30, PDF_CMAP_RANGE, 648 },
	{ 0x2f31, 0x2f32, PDF_CMAP_RANGE, 652 },
	{ 0x2f33, 0x2f35, PDF_CMAP_RANGE, 551 },
	{ 0x2f36, 0x2f38, PDF_CMAP_RANGE, 654 },
	{ 0x2f39, 0x2f3a, PDF_CMAP_RANGE, 554 },
	{ 0x2f3b, 0x2f3b, PDF_CMAP_SINGLE, 6007 },
	{ 0x2f3c, 0x2f3f, PDF_CMAP_RANGE, 720 },
	{ 0x2f40, 0x2f41, PDF_CMAP_TABLE, 150 },
	{ 0x2f42, 0x2f45, PDF_CMAP_RANGE, 726 },
	{ 0x2f46, 0x2f46, PDF_CMAP_SINGLE, 557 },
	{ 0x2f47, 0x2f4d, PDF_CMAP_RANGE, 730 },
	{ 0x2f4e, 0x2f4e, PDF_CMAP_SINGLE, 6026 },
	{ 0x2f4f, 0x2f52, PDF_CMAP_RANGE, 737 },
	{ 0x2f53, 0x2f53, PDF_CMAP_SINGLE, 6028 },
	{ 0x2f54, 0x2f58, PDF_CMAP_RANGE, 741 },
	{ 0x2f59, 0x2f59, PDF_CMAP_SINGLE, 6029 },
	{ 0x2f5a, 0x2f5d, PDF_CMAP_RANGE, 746 },
	{ 0x2f5e, 0x2f64, PDF_CMAP_RANGE, 854 },
	{ 0x2f65, 0x2f66, PDF_CMAP_TABLE, 152 },
	{ 0x2f67, 0x2f68, PDF_CMAP_RANGE, 558 },
	{ 0x2f69, 0x2f70, PDF_CMAP_RANGE, 867 },
	{ 0x2f71, 0x2f71, PDF_CMAP_SINGLE, 6066 },
	{ 0x2f72, 0x2f74, PDF_CMAP_RANGE, 875 },
	{ 0x2f75, 0x2f78, PDF_CMAP_RANGE, 1014 },
	{ 0x2f79, 0x2f79, PDF_CMAP_SINGLE, 6162 },
	{ 0x2f7a, 0x2f7c, PDF_CMAP_RANGE, 1018 },
	{ 0x2f7d, 0x2f81, PDF_CMAP_RANGE, 1022 },
	{ 0x2f82, 0x2f8a, PDF_CMAP_RANGE, 1029 },
	{ 0x2f8b, 0x2f8c, PDF_CMAP_TABLE, 154 },
	{ 0x2f8d, 0x2f90, PDF_CMAP_RANGE, 1039 },
	{ 0x2f91, 0x2f91, PDF_CMAP_SINGLE, 6169 },
	{ 0x2f92, 0x2f97, PDF_CMAP_RANGE, 1288 },
	{ 0x2f98, 0x2f98, PDF_CMAP_SINGLE, 6375 },
	{ 0x2f99, 0x2fa0, PDF_CMAP_RANGE, 1294 },
	{ 0x2fa1, 0x2fa2, PDF_CMAP_TABLE, 156 },
	{ 0x2fa3, 0x2fa5, PDF_CMAP_RANGE, 1312 },
	{ 0x2fa6, 0x2fa9, PDF_CMAP_RANGE, 1686 },
	{ 0x2faa, 0x2faa, PDF_CMAP_SINGLE, 561 },
	{ 0x2fab, 0x2fac, PDF_CMAP_RANGE, 1695 },
	{ 0x2fad, 0x2fae, PDF_CMAP_TABLE, 158 },
	{ 0x2faf, 0x2fb9, PDF_CMAP_RANGE, 2086 },
	{ 0x2fba, 0x2fbc, PDF_CMAP_RANGE, 2549 },
	{ 0x2fbd, 0x2fbf, PDF_CMAP_TABLE, 160 },
	{ 0x2fc0, 0x2fc1, PDF_CMAP_RANGE, 2553 },
	{ 0x2fc2, 0x2fc7, PDF_CMAP_RANGE, 3041 },
	{ 0x2fc8, 0x2fca, PDF_CMAP_RANGE, 3515 },
	{ 0x2fcb, 0x2fcc, PDF_CMAP_TABLE, 163 },
	{ 0x2fcd, 0x2fcf, PDF_CMAP_RANGE, 3963 },
	{ 0x2fd0, 0x2fd1, PDF_CMAP_RANGE, 4352 },
	{ 0x2fd2, 0x2fd2, PDF_CMAP_SINGLE, 4745 },
	{ 0x2fd3, 0x2fd4, PDF_CMAP_RANGE, 5042 },
	{ 0x2fd5, 0x2fd5, PDF_CMAP_SINGLE, 12045 },
	{ 0x3000, 0x3000, PDF_CMAP_SINGLE, 99 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 101 },
	{ 0x3003, 0x3003, PDF_CMAP_SINGLE, 179 },
	{ 0x3005, 0x3007, PDF_CMAP_RANGE, 13754 },
	{ 0x3008, 0x3009, PDF_CMAP_RANGE, 148 },
	{ 0x300a, 0x300b, PDF_CMAP_RANGE, 144 },
	{ 0x300c, 0x300d, PDF_CMAP_RANGE, 152 },
	{ 0x300e, 0x300f, PDF_CMAP_RANGE, 156 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 140 },
	{ 0x3012, 0x3012, PDF_CMAP_SINGLE, 261 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 136 },
	{ 0x301d, 0x301e, PDF_CMAP_RANGE, 170 },
	{ 0x3021, 0x3029, PDF_CMAP_RANGE, 353 },
	{ 0x3038, 0x303a, PDF_CMAP_RANGE, 362 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 13761 },
	{ 0x309b, 0x309c, PDF_CMAP_RANGE, 17606 },
	{ 0x309d, 0x309e, PDF_CMAP_RANGE, 13751 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 13844 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 13757 },
	{ 0x30fd, 0x30fe, PDF_CMAP_RANGE, 13749 },
	{ 0x3105, 0x3129, PDF_CMAP_RANGE, 465 },
	{ 0x31c0, 0x31c4, PDF_CMAP_RANGE, 17609 },
	{ 0x31c5, 0x31c5, PDF_CMAP_SINGLE, 17615 },
	{ 0x31c6, 0x31c7, PDF_CMAP_RANGE, 17618 },
	{ 0x31c8, 0x31c8, PDF_CMAP_SINGLE, 17622 },
	{ 0x31c9, 0x31cc, PDF_CMAP_RANGE, 17624 },
	{ 0x31cd, 0x31ce, PDF_CMAP_RANGE, 17629 },
	{ 0x31cf, 0x31cf, PDF_CMAP_SINGLE, 13999 },
	{ 0x3231, 0x3231, PDF_CMAP_SINGLE, 14053 },
	{ 0x32a3, 0x32a3, PDF_CMAP_SINGLE, 193 },
	{ 0x338e, 0x338f, PDF_CMAP_RANGE, 277 },
	{ 0x339c, 0x339e, PDF_CMAP_RANGE, 272 },
	{ 0x33a1, 0x33a1, PDF_CMAP_SINGLE, 276 },
	{ 0x33c4, 0x33c4, PDF_CMAP_SINGLE, 279 },
	{ 0x33ce, 0x33ce, PDF_CMAP_SINGLE, 275 },
	{ 0x33d1, 0x33d2, PDF_CMAP_TABLE, 165 },
	{ 0x33d5, 0x33d5, PDF_CMAP_SINGLE, 271 },
	{ 0x3435, 0x3435, PDF_CMAP_SINGLE, 14781 },
	{ 0x3440, 0x3440, PDF_CMAP_SINGLE, 15463 },
	{ 0x344a, 0x344a, PDF_CMAP_SINGLE, 19046 },
	{ 0x344c, 0x344c, PDF_CMAP_SINGLE, 17811 },
	{ 0x3464, 0x3464, PDF_CMAP_SINGLE, 14981 },
	{ 0x3473, 0x3473, PDF_CMAP_SINGLE, 15813 },
	{ 0x347a, 0x347a, PDF_CMAP_SINGLE, 16435 },
	{ 0x347d, 0x347e, PDF_CMAP_TABLE, 167 },
	{ 0x3493, 0x3493, PDF_CMAP_SINGLE, 17310 },
	{ 0x3496, 0x3496, PDF_CMAP_SINGLE, 14300 },
	{ 0x34a5, 0x34a5, PDF_CMAP_SINGLE, 17816 },
	{ 0x34af, 0x34af, PDF_CMAP_SINGLE, 15487 },
	{ 0x34bc, 0x34bc, PDF_CMAP_SINGLE, 17280 },
	{ 0x34c1, 0x34c1, PDF_CMAP_SINGLE, 17659 },
	{ 0x34c8, 0x34c8, PDF_CMAP_SINGLE, 15616 },
	{ 0x34df, 0x34df, PDF_CMAP_SINGLE, 16036 },
	{ 0x34e4, 0x34e4, PDF_CMAP_SINGLE, 15956 },
	{ 0x34e6, 0x34e6, PDF_CMAP_SINGLE, 19122 },
	{ 0x34fb, 0x34fb, PDF_CMAP_SINGLE, 15818 },
	{ 0x3506, 0x3506, PDF_CMAP_SINGLE, 17206 },
	{ 0x353e, 0x353e, PDF_CMAP_SINGLE, 17639 },
	{ 0x3551, 0x3551, PDF_CMAP_SINGLE, 17825 },
	{ 0x3553, 0x3553, PDF_CMAP_SINGLE, 17317 },
	{ 0x3559, 0x3559, PDF_CMAP_SINGLE, 18860 },
	{ 0x3561, 0x3561, PDF_CMAP_SINGLE, 16600 },
	{ 0x356d, 0x356d, PDF_CMAP_SINGLE, 17828 },
	{ 0x3570, 0x3570, PDF_CMAP_SINGLE, 15151 },
	{ 0x3572, 0x3572, PDF_CMAP_SINGLE, 17829 },
	{ 0x3577, 0x3578, PDF_CMAP_TABLE, 169 },
	{ 0x3584, 0x3584, PDF_CMAP_SINGLE, 14843 },
	{ 0x3597, 0x3598, PDF_CMAP_TABLE, 171 },
	{ 0x35a1, 0x35a1, PDF_CMAP_SINGLE, 16610 },
	{ 0x35a5, 0x35a5, PDF_CMAP_SINGLE, 17841 },
	{ 0x35ad, 0x35ad, PDF_CMAP_SINGLE, 16124 },
	{ 0x35bf, 0x35bf, PDF_CMAP_SINGLE, 17842 },
	{ 0x35c1, 0x35c1, PDF_CMAP_SINGLE, 17792 },
	{ 0x35c5, 0x35c5, PDF_CMAP_SINGLE, 17844 },
	{ 0x35c7, 0x35c7, PDF_CMAP_SINGLE, 17823 },
	{ 0x35ca, 0x35ca, PDF_CMAP_SINGLE, 15939 },
	{ 0x35ce, 0x35ce, PDF_CMAP_SINGLE, 16001 },
	{ 0x35d2, 0x35d2, PDF_CMAP_SINGLE, 16329 },
	{ 0x35d6, 0x35d6, PDF_CMAP_SINGLE, 16309 },
	{ 0x35db, 0x35db, PDF_CMAP_SINGLE, 18189 },
	{ 0x35dd, 0x35dd, PDF_CMAP_SINGLE, 17324 },
	{ 0x35f1, 0x35f3, PDF_CMAP_TABLE, 173 },
	{ 0x35fb, 0x35fb, PDF_CMAP_SINGLE, 17864 },
	{ 0x35fe, 0x35fe, PDF_CMAP_SINGLE, 16234 },
	{ 0x3609, 0x3609, PDF_CMAP_SINGLE, 17909 },
	{ 0x3618, 0x3618, PDF_CMAP_SINGLE, 18545 },
	{ 0x361a, 0x361a, PDF_CMAP_SINGLE, 17946 },
	{ 0x361d, 0x361d, PDF_CMAP_SINGLE, 14007 },
	{ 0x3623, 0x3623, PDF_CMAP_SINGLE, 16582 },
	{ 0x3625, 0x3625, PDF_CMAP_SINGLE, 19012 },
	{ 0x362d, 0x362d, PDF_CMAP_SINGLE, 16006 },
	{ 0x3635, 0x3635, PDF_CMAP_SINGLE, 18734 },
	{ 0x3639, 0x3639, PDF_CMAP_SINGLE, 16741 },
	{ 0x363e, 0x363e, PDF_CMAP_SINGLE, 14856 },
	{ 0x3647, 0x3649, PDF_CMAP_TABLE, 176 },
	{ 0x364e, 0x364e, PDF_CMAP_SINGLE, 18072 },
	{ 0x365f, 0x365f, PDF_CMAP_SINGLE, 15859 },
	{ 0x3661, 0x3661, PDF_CMAP_SINGLE, 19084 },
	{ 0x367a, 0x367a, PDF_CMAP_SINGLE, 16099 },
	{ 0x3681, 0x3681, PDF_CMAP_SINGLE, 17830 },
	{ 0x369a, 0x369a, PDF_CMAP_SINGLE, 15473 },
	{ 0x36a5, 0x36a5, PDF_CMAP_SINGLE, 18690 },
	{ 0x36aa, 0x36aa, PDF_CMAP_SINGLE, 15152 },
	{ 0x36ac, 0x36ac, PDF_CMAP_SINGLE, 18688 },
	{ 0x36b0, 0x36b1, PDF_CMAP_TABLE, 179 },
	{ 0x36b5, 0x36b5, PDF_CMAP_SINGLE, 15298 },
	{ 0x36b9, 0x36b9, PDF_CMAP_SINGLE, 18567 },
	{ 0x36bc, 0x36bc, PDF_CMAP_SINGLE, 15615 },
	{ 0x36c1, 0x36c1, PDF_CMAP_SINGLE, 14277 },
	{ 0x36c3, 0x36c5, PDF_CMAP_TABLE, 181 },
	{ 0x36c7, 0x36c8, PDF_CMAP_TABLE, 184 },
	{ 0x36d3, 0x36d4, PDF_CMAP_TABLE, 186 },
	{ 0x36d6, 0x36d6, PDF_CMAP_SINGLE, 15628 },
	{ 0x36dd, 0x36dd, PDF_CMAP_SINGLE, 14902 },
	{ 0x36e1, 0x36e2, PDF_CMAP_TABLE, 188 },
	{ 0x36e5, 0x36e6, PDF_CMAP_TABLE, 190 },
	{ 0x36f5, 0x36f5, PDF_CMAP_SINGLE, 14872 },
	{ 0x3701, 0x3701, PDF_CMAP_SINGLE, 14900 },
	{ 0x3703, 0x3703, PDF_CMAP_SINGLE, 17935 },
	{ 0x3708, 0x3708, PDF_CMAP_SINGLE, 16639 },
	{ 0x370a, 0x370a, PDF_CMAP_SINGLE, 15573 },
	{ 0x370d, 0x370d, PDF_CMAP_SINGLE, 16952 },
	{ 0x371c, 0x371c, PDF_CMAP_SINGLE, 15870 },
	{ 0x3722, 0x3723, PDF_CMAP_TABLE, 192 },
	{ 0x3725, 0x3725, PDF_CMAP_SINGLE, 14409 },
	{ 0x372c, 0x372d, PDF_CMAP_TABLE, 194 },
	{ 0x3730, 0x3730, PDF_CMAP_SINGLE, 18780 },
	{ 0x3732, 0x3733, PDF_CMAP_TABLE, 196 },
	{ 0x373a, 0x373a, PDF_CMAP_SINGLE, 15958 },
	{ 0x3740, 0x3740, PDF_CMAP_SINGLE, 16898 },
	{ 0x3743, 0x3743, PDF_CMAP_SINGLE, 16438 },
	{ 0x3762, 0x3762, PDF_CMAP_SINGLE, 15798 },
	{ 0x376f, 0x376f, PDF_CMAP_SINGLE, 18379 },
	{ 0x3797, 0x3797, PDF_CMAP_SINGLE, 17901 },
	{ 0x37a0, 0x37a0, PDF_CMAP_SINGLE, 14888 },
	{ 0x37b9, 0x37b9, PDF_CMAP_SINGLE, 17335 },
	{ 0x37be, 0x37be, PDF_CMAP_SINGLE, 14654 },
	{ 0x37d6, 0x37d6, PDF_CMAP_SINGLE, 19067 },
	{ 0x37f2, 0x37f2, PDF_CMAP_SINGLE, 15265 },
	{ 0x37f8, 0x37f8, PDF_CMAP_SINGLE, 17106 },
	{ 0x37fb, 0x37fb, PDF_CMAP_SINGLE, 16117 },
	{ 0x380f, 0x380f, PDF_CMAP_SINGLE, 17964 },
	{ 0x3819, 0x3819, PDF_CMAP_SINGLE, 14767 },
	{ 0x3820, 0x3820, PDF_CMAP_SINGLE, 17967 },
	{ 0x382d, 0x382d, PDF_CMAP_SINGLE, 16686 },
	{ 0x3836, 0x3836, PDF_CMAP_SINGLE, 16691 },
	{ 0x3838, 0x3838, PDF_CMAP_SINGLE, 17339 },
	{ 0x3863, 0x3863, PDF_CMAP_SINGLE, 18115 },
	{ 0x3875, 0x3875, PDF_CMAP_SINGLE, 19088 },
	{ 0x38a0, 0x38a0, PDF_CMAP_SINGLE, 16709 },
	{ 0x38c3, 0x38c3, PDF_CMAP_SINGLE, 14610 },
	{ 0x38cc, 0x38cc, PDF_CMAP_SINGLE, 16502 },
	{ 0x38d1, 0x38d1, PDF_CMAP_SINGLE, 14997 },
	{ 0x38d4, 0x38d4, PDF_CMAP_SINGLE, 19074 },
	{ 0x38fa, 0x38fa, PDF_CMAP_SINGLE, 17643 },
	{ 0x3908, 0x3908, PDF_CMAP_SINGLE, 17970 },
	{ 0x3914, 0x3914, PDF_CMAP_SINGLE, 17342 },
	{ 0x3927, 0x3927, PDF_CMAP_SINGLE, 15409 },
	{ 0x3932, 0x3932, PDF_CMAP_SINGLE, 16770 },
	{ 0x393f, 0x393f, PDF_CMAP_SINGLE, 17971 },
	{ 0x394d, 0x394d, PDF_CMAP_SINGLE, 17972 },
	{ 0x3963, 0x3963, PDF_CMAP_SINGLE, 16739 },
	{ 0x3978, 0x3978, PDF_CMAP_SINGLE, 19045 },
	{ 0x3980, 0x3980, PDF_CMAP_SINGLE, 14452 },
	{ 0x3989, 0x398a, PDF_CMAP_TABLE, 198 },
	{ 0x3992, 0x3992, PDF_CMAP_SINGLE, 17270 },
	{ 0x3999, 0x3999, PDF_CMAP_SINGLE, 14778 },
	{ 0x399b, 0x399b, PDF_CMAP_SINGLE, 15795 },
	{ 0x39a1, 0x39a1, PDF_CMAP_SINGLE, 15897 },
	{ 0x39a4, 0x39a4, PDF_CMAP_SINGLE, 15887 },
	{ 0x39b8, 0x39b8, PDF_CMAP_SINGLE, 17979 },
	{ 0x39dc, 0x39dc, PDF_CMAP_SINGLE, 16078 },
	{ 0x39e2, 0x39e2, PDF_CMAP_SINGLE, 18120 },
	{ 0x39e5, 0x39e5, PDF_CMAP_SINGLE, 14651 },
	{ 0x39ec, 0x39ec, PDF_CMAP_SINGLE, 17168 },
	{ 0x39f8, 0x39f8, PDF_CMAP_SINGLE, 17982 },
	{ 0x39fb, 0x39fb, PDF_CMAP_SINGLE, 17221 },
	{ 0x39fe, 0x39fe, PDF_CMAP_SINGLE, 17256 },
	{ 0x3a01, 0x3a01, PDF_CMAP_SINGLE, 16864 },
	{ 0x3a03, 0x3a03, PDF_CMAP_SINGLE, 17984 },
	{ 0x3a06, 0x3a06, PDF_CMAP_SINGLE, 17271 },
	{ 0x3a17, 0x3a18, PDF_CMAP_TABLE, 200 },
	{ 0x3a29, 0x3a2a, PDF_CMAP_TABLE, 202 },
	{ 0x3a34, 0x3a34, PDF_CMAP_SINGLE, 17177 },
	{ 0x3a4b, 0x3a4b, PDF_CMAP_SINGLE, 17988 },
	{ 0x3a52, 0x3a52, PDF_CMAP_SINGLE, 16083 },
	{ 0x3a57, 0x3a57, PDF_CMAP_SINGLE, 16798 },
	{ 0x3a5c, 0x3a5c, PDF_CMAP_SINGLE, 16324 },
	{ 0x3a5e, 0x3a5e, PDF_CMAP_SINGLE, 15111 },
	{ 0x3a66, 0x3a67, PDF_CMAP_TABLE, 204 },
	{ 0x3a97, 0x3a97, PDF_CMAP_SINGLE, 17991 },
	{ 0x3aab, 0x3aab, PDF_CMAP_SINGLE, 16529 },
	{ 0x3abd, 0x3abd, PDF_CMAP_SINGLE, 17993 },
	{ 0x3ade, 0x3ade, PDF_CMAP_SINGLE, 16716 },
	{ 0x3ae0, 0x3ae0, PDF_CMAP_SINGLE, 14970 },
	{ 0x3af0, 0x3af0, PDF_CMAP_SINGLE, 18098 },
	{ 0x3af2, 0x3af2, PDF_CMAP_SINGLE, 17996 },
	{ 0x3af5, 0x3af5, PDF_CMAP_SINGLE, 19094 },
	{ 0x3afb, 0x3afb, PDF_CMAP_SINGLE, 15090 },
	{ 0x3b0e, 0x3b0e, PDF_CMAP_SINGLE, 14568 },
	{ 0x3b19, 0x3b19, PDF_CMAP_SINGLE, 18117 },
	{ 0x3b22, 0x3b22, PDF_CMAP_SINGLE, 17998 },
	{ 0x3b2b, 0x3b2b, PDF_CMAP_SINGLE, 18774 },
	{ 0x3b39, 0x3b39, PDF_CMAP_SINGLE, 18251 },
	{ 0x3b42, 0x3b42, PDF_CMAP_SINGLE, 18000 },
	{ 0x3b58, 0x3b58, PDF_CMAP_SINGLE, 18002 },
	{ 0x3b60, 0x3b60, PDF_CMAP_SINGLE, 14650 },
	{ 0x3b71, 0x3b72, PDF_CMAP_TABLE, 206 },
	{ 0x3b7b, 0x3b7c, PDF_CMAP_TABLE, 208 },
	{ 0x3b80, 0x3b80, PDF_CMAP_SINGLE, 16866 },
	{ 0x3b95, 0x3b96, PDF_CMAP_TABLE, 210 },
	{ 0x3b99, 0x3b99, PDF_CMAP_SINGLE, 15000 },
	{ 0x3ba1, 0x3ba1, PDF_CMAP_SINGLE, 16873 },
	{ 0x3bbc, 0x3bbc, PDF_CMAP_SINGLE, 17352 },
	{ 0x3bbe, 0x3bbe, PDF_CMAP_SINGLE, 15793 },
	{ 0x3bc2, 0x3bc2, PDF_CMAP_SINGLE, 16692 },
	{ 0x3bc4, 0x3bc4, PDF_CMAP_SINGLE, 15008 },
	{ 0x3bd7, 0x3bd7, PDF_CMAP_SINGLE, 15020 },
	{ 0x3bdd, 0x3bdd, PDF_CMAP_SINGLE, 18015 },
	{ 0x3bec, 0x3bec, PDF_CMAP_SINGLE, 18020 },
	{ 0x3bf2, 0x3bf4, PDF_CMAP_TABLE, 212 },
	{ 0x3c0d, 0x3c0d, PDF_CMAP_SINGLE, 16887 },
	{ 0x3c11, 0x3c11, PDF_CMAP_SINGLE, 15936 },
	{ 0x3c15, 0x3c15, PDF_CMAP_SINGLE, 14744 },
	{ 0x3c18, 0x3c18, PDF_CMAP_SINGLE, 18737 },
	{ 0x3c54, 0x3c54, PDF_CMAP_SINGLE, 15872 },
	{ 0x3c8b, 0x3c8b, PDF_CMAP_SINGLE, 18871 },
	{ 0x3ccb, 0x3ccb, PDF_CMAP_SINGLE, 18032 },
	{ 0x3ccd, 0x3ccd, PDF_CMAP_SINGLE, 15589 },
	{ 0x3cd1, 0x3cd1, PDF_CMAP_SINGLE, 16387 },
	{ 0x3cd6, 0x3cd6, PDF_CMAP_SINGLE, 15607 },
	{ 0x3cdc, 0x3cdc, PDF_CMAP_SINGLE, 16462 },
	{ 0x3ceb, 0x3ceb, PDF_CMAP_SINGLE, 16919 },
	{ 0x3cef, 0x3cef, PDF_CMAP_SINGLE, 18037 },
	{ 0x3d12, 0x3d13, PDF_CMAP_TABLE, 215 },
	{ 0x3d1d, 0x3d1d, PDF_CMAP_SINGLE, 14652 },
	{ 0x3d32, 0x3d32, PDF_CMAP_SINGLE, 18775 },
	{ 0x3d3b, 0x3d3b, PDF_CMAP_SINGLE, 16965 },
	{ 0x3d46, 0x3d46, PDF_CMAP_SINGLE, 18053 },
	{ 0x3d4c, 0x3d4c, PDF_CMAP_SINGLE, 15595 },
	{ 0x3d4e, 0x3d4e, PDF_CMAP_SINGLE, 16962 },
	{ 0x3d51, 0x3d51, PDF_CMAP_SINGLE, 14570 },
	{ 0x3d5f, 0x3d5f, PDF_CMAP_SINGLE, 16729 },
	{ 0x3d62, 0x3d62, PDF_CMAP_SINGLE, 15454 },
	{ 0x3d69, 0x3d6a, PDF_CMAP_TABLE, 217 },
	{ 0x3d6f, 0x3d6f, PDF_CMAP_SINGLE, 15612 },
	{ 0x3d75, 0x3d75, PDF_CMAP_SINGLE, 18058 },
	{ 0x3d7d, 0x3d7d, PDF_CMAP_SINGLE, 15407 },
	{ 0x3d85, 0x3d85, PDF_CMAP_SINGLE, 18763 },
	{ 0x3d88, 0x3d88, PDF_CMAP_SINGLE, 19026 },
	{ 0x3d8a, 0x3d8a, PDF_CMAP_SINGLE, 18061 },
	{ 0x3d8f, 0x3d8f, PDF_CMAP_SINGLE, 15037 },
	{ 0x3d91, 0x3d91, PDF_CMAP_SINGLE, 18063 },
	{ 0x3da5, 0x3da5, PDF_CMAP_SINGLE, 15702 },
	{ 0x3dad, 0x3dad, PDF_CMAP_SINGLE, 18073 },
	{ 0x3db4, 0x3db4, PDF_CMAP_SINGLE, 16550 },
	{ 0x3dbf, 0x3dbf, PDF_CMAP_SINGLE, 14288 },
	{ 0x3dc6, 0x3dc7, PDF_CMAP_TABLE, 219 },
	{ 0x3dc9, 0x3dc9, PDF_CMAP_SINGLE, 18853 },
	{ 0x3dcc, 0x3dcd, PDF_CMAP_TABLE, 221 },
	{ 0x3dd3, 0x3dd3, PDF_CMAP_SINGLE, 14305 },
	{ 0x3ddb, 0x3ddb, PDF_CMAP_SINGLE, 16383 },
	{ 0x3de7, 0x3de8, PDF_CMAP_TABLE, 223 },
	{ 0x3deb, 0x3deb, PDF_CMAP_SINGLE, 15962 },
	{ 0x3df3, 0x3df4, PDF_CMAP_TABLE, 225 },
	{ 0x3df7, 0x3df7, PDF_CMAP_SINGLE, 18603 },
	{ 0x3dfc, 0x3dfd, PDF_CMAP_TABLE, 227 },
	{ 0x3e06, 0x3e06, PDF_CMAP_SINGLE, 18717 },
	{ 0x3e40, 0x3e40, PDF_CMAP_SINGLE, 16745 },
	{ 0x3e43, 0x3e43, PDF_CMAP_SINGLE, 17261 },
	{ 0x3e48, 0x3e48, PDF_CMAP_SINGLE, 17813 },
	{ 0x3e55, 0x3e55, PDF_CMAP_SINGLE, 17023 },
	{ 0x3e74, 0x3e74, PDF_CMAP_SINGLE, 16098 },
	{ 0x3ea8, 0x3eaa, PDF_CMAP_TABLE, 229 },
	{ 0x3ead, 0x3ead, PDF_CMAP_SINGLE, 15261 },
	{ 0x3eb1, 0x3eb1, PDF_CMAP_SINGLE, 15064 },
	{ 0x3eb8, 0x3eb8, PDF_CMAP_SINGLE, 14923 },
	{ 0x3ebf, 0x3ebf, PDF_CMAP_SINGLE, 15115 },
	{ 0x3ec2, 0x3ec2, PDF_CMAP_SINGLE, 15320 },
	{ 0x3ec7, 0x3ec7, PDF_CMAP_SINGLE, 14709 },
	{ 0x3eca, 0x3eca, PDF_CMAP_SINGLE, 18161 },
	{ 0x3ecc, 0x3ecc, PDF_CMAP_SINGLE, 15330 },
	{ 0x3ed0, 0x3ed1, PDF_CMAP_TABLE, 232 },
	{ 0x3ed6, 0x3ed7, PDF_CMAP_TABLE, 234 },
	{ 0x3eda, 0x3edb, PDF_CMAP_TABLE, 236 },
	{ 0x3ede, 0x3ede, PDF_CMAP_SINGLE, 14834 },
	{ 0x3ee1, 0x3ee2, PDF_CMAP_TABLE, 238 },
	{ 0x3ee7, 0x3ee7, PDF_CMAP_SINGLE, 15324 },
	{ 0x3ee9, 0x3ee9, PDF_CMAP_SINGLE, 15524 },
	{ 0x3eeb, 0x3eec, PDF_CMAP_TABLE, 240 },
	{ 0x3ef0, 0x3ef0, PDF_CMAP_SINGLE, 18167 },
	{ 0x3ef3, 0x3ef4, PDF_CMAP_TABLE, 242 },
	{ 0x3efa, 0x3efa, PDF_CMAP_SINGLE, 18169 },
	{ 0x3efc, 0x3efc, PDF_CMAP_SINGLE, 15336 },
	{ 0x3eff, 0x3f00, PDF_CMAP_TABLE, 244 },
	{ 0x3f04, 0x3f04, PDF_CMAP_SINGLE, 17091 },
	{ 0x3f06, 0x3f07, PDF_CMAP_TABLE, 246 },
	{ 0x3f0e, 0x3f0e, PDF_CMAP_SINGLE, 18171 },
	{ 0x3f53, 0x3f53, PDF_CMAP_SINGLE, 18172 },
	{ 0x3f58, 0x3f59, PDF_CMAP_TABLE, 248 },
	{ 0x3f63, 0x3f63, PDF_CMAP_SINGLE, 15078 },
	{ 0x3f7c, 0x3f7c, PDF_CMAP_SINGLE, 18176 },
	{ 0x3f93, 0x3f93, PDF_CMAP_SINGLE, 17869 },
	{ 0x3fc0, 0x3fc0, PDF_CMAP_SINGLE, 17359 },
	{ 0x3fc8, 0x3fc8, PDF_CMAP_SINGLE, 19082 },
	{ 0x3fd7, 0x3fd7, PDF_CMAP_SINGLE, 17361 },
	{ 0x3fdc, 0x3fdc, PDF_CMAP_SINGLE, 18180 },
	{ 0x3fe5, 0x3fe5, PDF_CMAP_SINGLE, 18143 },
	{ 0x3fed, 0x3fed, PDF_CMAP_SINGLE, 17205 },
	{ 0x3ff9, 0x3ffa, PDF_CMAP_TABLE, 250 },
	{ 0x4004, 0x4004, PDF_CMAP_SINGLE, 16654 },
	{ 0x4009, 0x4009, PDF_CMAP_SINGLE, 18878 },
	{ 0x401d, 0x401d, PDF_CMAP_SINGLE, 18185 },
	{ 0x4039, 0x4039, PDF_CMAP_SINGLE, 18187 },
	{ 0x4045, 0x4045, PDF_CMAP_SINGLE, 18188 },
	{ 0x4053, 0x4053, PDF_CMAP_SINGLE, 17846 },
	{ 0x4057, 0x4057, PDF_CMAP_SINGLE, 14749 },
	{ 0x4062, 0x4062, PDF_CMAP_SINGLE, 15307 },
	{ 0x4065, 0x4065, PDF_CMAP_SINGLE, 16339 },
	{ 0x406a, 0x406a, PDF_CMAP_SINGLE, 18191 },
	{ 0x406f, 0x406f, PDF_CMAP_SINGLE, 18192 },
	{ 0x4071, 0x4071, PDF_CMAP_SINGLE, 18944 },
	{ 0x40a8, 0x40a8, PDF_CMAP_SINGLE, 17365 },
	{ 0x40b4, 0x40b4, PDF_CMAP_SINGLE, 19149 },
	{ 0x40bb, 0x40bb, PDF_CMAP_SINGLE, 17856 },
	{ 0x40bf, 0x40bf, PDF_CMAP_SINGLE, 16108 },
	{ 0x40c8, 0x40c8, PDF_CMAP_SINGLE, 15118 },
	{ 0x40d8, 0x40d8, PDF_CMAP_SINGLE, 16811 },
	{ 0x40df, 0x40df, PDF_CMAP_SINGLE, 15895 },
	{ 0x40f8, 0x40f8, PDF_CMAP_SINGLE, 19066 },
	{ 0x40fa, 0x40fa, PDF_CMAP_SINGLE, 16062 },
	{ 0x4102, 0x4104, PDF_CMAP_TABLE, 252 },
	{ 0x4109, 0x4109, PDF_CMAP_SINGLE, 18204 },
	{ 0x410e, 0x410e, PDF_CMAP_SINGLE, 15131 },
	{ 0x4131, 0x4132, PDF_CMAP_TABLE, 255 },
	{ 0x4167, 0x4167, PDF_CMAP_SINGLE, 18207 },
	{ 0x416c, 0x416c, PDF_CMAP_SINGLE, 14510 },
	{ 0x416e, 0x416e, PDF_CMAP_SINGLE, 15139 },
	{ 0x417c, 0x417c, PDF_CMAP_SINGLE, 19070 },
	{ 0x417f, 0x417f, PDF_CMAP_SINGLE, 15234 },
	{ 0x4181, 0x4181, PDF_CMAP_SINGLE, 19138 },
	{ 0x4190, 0x4190, PDF_CMAP_SINGLE, 18112 },
	{ 0x41b2, 0x41b2, PDF_CMAP_SINGLE, 18208 },
	{ 0x41c4, 0x41c4, PDF_CMAP_SINGLE, 18211 },
	{ 0x41ca, 0x41ca, PDF_CMAP_SINGLE, 14143 },
	{ 0x41cf, 0x41cf, PDF_CMAP_SINGLE, 18214 },
	{ 0x41db, 0x41db, PDF_CMAP_SINGLE, 14271 },
	{ 0x41ed, 0x41ed, PDF_CMAP_SINGLE, 19087 },
	{ 0x41ef, 0x41ef, PDF_CMAP_SINGLE, 14147 },
	{ 0x41f9, 0x41f9, PDF_CMAP_SINGLE, 15166 },
	{ 0x4211, 0x4211, PDF_CMAP_SINGLE, 15169 },
	{ 0x4223, 0x4223, PDF_CMAP_SINGLE, 18881 },
	{ 0x4240, 0x4240, PDF_CMAP_SINGLE, 14321 },
	{ 0x4260, 0x4260, PDF_CMAP_SINGLE, 18219 },
	{ 0x426a, 0x426a, PDF_CMAP_SINGLE, 15189 },
	{ 0x4276, 0x4276, PDF_CMAP_SINGLE, 18949 },
	{ 0x427a, 0x427a, PDF_CMAP_SINGLE, 18220 },
	{ 0x428c, 0x428c, PDF_CMAP_SINGLE, 18223 },
	{ 0x4294, 0x4294, PDF_CMAP_SINGLE, 18225 },
	{ 0x42a2, 0x42a2, PDF_CMAP_SINGLE, 18945 },
	{ 0x42b5, 0x42b5, PDF_CMAP_SINGLE, 16400 },
	{ 0x42b9, 0x42b9, PDF_CMAP_SINGLE, 14502 },
	{ 0x42bc, 0x42bc, PDF_CMAP_SINGLE, 15498 },
	{ 0x42f4, 0x42f4, PDF_CMAP_SINGLE, 15289 },
	{ 0x42fb, 0x42fc, PDF_CMAP_TABLE, 257 },
	{ 0x430a, 0x430a, PDF_CMAP_SINGLE, 19102 },
	{ 0x432b, 0x432b, PDF_CMAP_SINGLE, 14205 },
	{ 0x436e, 0x436e, PDF_CMAP_SINGLE, 18122 },
	{ 0x4397, 0x4397, PDF_CMAP_SINGLE, 18235 },
	{ 0x439a, 0x439a, PDF_CMAP_SINGLE, 19030 },
	{ 0x43ba, 0x43ba, PDF_CMAP_SINGLE, 17247 },
	{ 0x43c1, 0x43c1, PDF_CMAP_SINGLE, 18069 },
	{ 0x43d9, 0x43d9, PDF_CMAP_SINGLE, 17214 },
	{ 0x43df, 0x43df, PDF_CMAP_SINGLE, 15945 },
	{ 0x43ed, 0x43ed, PDF_CMAP_SINGLE, 18245 },
	{ 0x43f0, 0x43f0, PDF_CMAP_SINGLE, 18965 },
	{ 0x43f2, 0x43f2, PDF_CMAP_SINGLE, 15944 },
	{ 0x4401, 0x4402, PDF_CMAP_TABLE, 259 },
	{ 0x4413, 0x4413, PDF_CMAP_SINGLE, 18255 },
	{ 0x4425, 0x4425, PDF_CMAP_SINGLE, 18257 },
	{ 0x442d, 0x442d, PDF_CMAP_SINGLE, 18258 },
	{ 0x447a, 0x447a, PDF_CMAP_SINGLE, 14255 },
	{ 0x448f, 0x448f, PDF_CMAP_SINGLE, 18264 },
	{ 0x4491, 0x4491, PDF_CMAP_SINGLE, 14004 },
	{ 0x449f, 0x44a0, PDF_CMAP_TABLE, 261 },
	{ 0x44a2, 0x44a2, PDF_CMAP_SINGLE, 16505 },
	{ 0x44b0, 0x44b0, PDF_CMAP_SINGLE, 18268 },
	{ 0x44b7, 0x44b7, PDF_CMAP_SINGLE, 16289 },
	{ 0x44bd, 0x44bd, PDF_CMAP_SINGLE, 19086 },
	{ 0x44c0, 0x44c0, PDF_CMAP_SINGLE, 15367 },
	{ 0x44c3, 0x44c3, PDF_CMAP_SINGLE, 19027 },
	{ 0x44c5, 0x44c5, PDF_CMAP_SINGLE, 16912 },
	{ 0x44ce, 0x44ce, PDF_CMAP_SINGLE, 15651 },
	{ 0x44dd, 0x44df, PDF_CMAP_TABLE, 263 },
	{ 0x44e1, 0x44e1, PDF_CMAP_SINGLE, 19151 },
	{ 0x44e4, 0x44e4, PDF_CMAP_SINGLE, 14306 },
	{ 0x44e9, 0x44ec, PDF_CMAP_TABLE, 266 },
	{ 0x44f4, 0x44f4, PDF_CMAP_SINGLE, 14738 },
	{ 0x4503, 0x4504, PDF_CMAP_TABLE, 270 },
	{ 0x4509, 0x4509, PDF_CMAP_SINGLE, 15952 },
	{ 0x450b, 0x450b, PDF_CMAP_SINGLE, 14292 },
	{ 0x4516, 0x4516, PDF_CMAP_SINGLE, 14329 },
	{ 0x451b, 0x451b, PDF_CMAP_SINGLE, 19077 },
	{ 0x451d, 0x451d, PDF_CMAP_SINGLE, 14183 },
	{ 0x4523, 0x4523, PDF_CMAP_SINGLE, 14326 },
	{ 0x4527, 0x4527, PDF_CMAP_SINGLE, 14327 },
	{ 0x452e, 0x452e, PDF_CMAP_SINGLE, 15571 },
	{ 0x4533, 0x4533, PDF_CMAP_SINGLE, 15441 },
	{ 0x4536, 0x4536, PDF_CMAP_SINGLE, 19031 },
	{ 0x453b, 0x453b, PDF_CMAP_SINGLE, 18282 },
	{ 0x453d, 0x453d, PDF_CMAP_SINGLE, 14532 },
	{ 0x453f, 0x453f, PDF_CMAP_SINGLE, 15890 },
	{ 0x4543, 0x4543, PDF_CMAP_SINGLE, 14323 },
	{ 0x4551, 0x4552, PDF_CMAP_TABLE, 272 },
	{ 0x4555, 0x4555, PDF_CMAP_SINGLE, 16958 },
	{ 0x4558, 0x4558, PDF_CMAP_SINGLE, 19041 },
	{ 0x455c, 0x455c, PDF_CMAP_SINGLE, 14219 },
	{ 0x4561, 0x4562, PDF_CMAP_TABLE, 274 },
	{ 0x456a, 0x456a, PDF_CMAP_SINGLE, 14340 },
	{ 0x456d, 0x456d, PDF_CMAP_SINGLE, 19073 },
	{ 0x4577, 0x4578, PDF_CMAP_TABLE, 276 },
	{ 0x4585, 0x4585, PDF_CMAP_SINGLE, 14533 },
	{ 0x45a6, 0x45a6, PDF_CMAP_SINGLE, 18968 },
	{ 0x45ac, 0x45ac, PDF_CMAP_SINGLE, 17377 },
	{ 0x45b3, 0x45b3, PDF_CMAP_SINGLE, 18972 },
	{ 0x45da, 0x45da, PDF_CMAP_SINGLE, 18989 },
	{ 0x45e9, 0x45ea, PDF_CMAP_TABLE, 278 },
	{ 0x4603, 0x4603, PDF_CMAP_SINGLE, 18967 },
	{ 0x4606, 0x4606, PDF_CMAP_SINGLE, 18291 },
	{ 0x460f, 0x460f, PDF_CMAP_SINGLE, 14357 },
	{ 0x4615, 0x4615, PDF_CMAP_SINGLE, 14403 },
	{ 0x4617, 0x4617, PDF_CMAP_SINGLE, 18292 },
	{ 0x465b, 0x465b, PDF_CMAP_SINGLE, 14365 },
	{ 0x467a, 0x467a, PDF_CMAP_SINGLE, 14825 },
	{ 0x4680, 0x4680, PDF_CMAP_SINGLE, 15617 },
	{ 0x46a1, 0x46a1, PDF_CMAP_SINGLE, 18994 },
	{ 0x46ae, 0x46ae, PDF_CMAP_SINGLE, 19136 },
	{ 0x46bb, 0x46bb, PDF_CMAP_SINGLE, 19061 },
	{ 0x46cf, 0x46d0, PDF_CMAP_TABLE, 280 },
	{ 0x46f5, 0x46f5, PDF_CMAP_SINGLE, 15263 },
	{ 0x46f7, 0x46f7, PDF_CMAP_SINGLE, 19068 },
	{ 0x4713, 0x4713, PDF_CMAP_SINGLE, 14387 },
	{ 0x4718, 0x4718, PDF_CMAP_SINGLE, 15815 },
	{ 0x4736, 0x4736, PDF_CMAP_SINGLE, 18977 },
	{ 0x4744, 0x4744, PDF_CMAP_SINGLE, 18978 },
	{ 0x474e, 0x474f, PDF_CMAP_TABLE, 282 },
	{ 0x477c, 0x477c, PDF_CMAP_SINGLE, 15821 },
	{ 0x4798, 0x4798, PDF_CMAP_SINGLE, 18305 },
	{ 0x47a6, 0x47a6, PDF_CMAP_SINGLE, 16547 },
	{ 0x47b6, 0x47b6, PDF_CMAP_SINGLE, 16106 },
	{ 0x47d5, 0x47d5, PDF_CMAP_SINGLE, 17178 },
	{ 0x47ed, 0x47ed, PDF_CMAP_SINGLE, 18307 },
	{ 0x47f4, 0x47f4, PDF_CMAP_SINGLE, 17199 },
	{ 0x4800, 0x4800, PDF_CMAP_SINGLE, 17950 },
	{ 0x480b, 0x480b, PDF_CMAP_SINGLE, 17234 },
	{ 0x4837, 0x4837, PDF_CMAP_SINGLE, 18311 },
	{ 0x485d, 0x485d, PDF_CMAP_SINGLE, 16655 },
	{ 0x4871, 0x4871, PDF_CMAP_SINGLE, 15619 },
	{ 0x489b, 0x489b, PDF_CMAP_SINGLE, 15293 },
	{ 0x48ad, 0x48ae, PDF_CMAP_TABLE, 284 },
	{ 0x48b4, 0x48b4, PDF_CMAP_SINGLE, 15301 },
	{ 0x48d0, 0x48d0, PDF_CMAP_SINGLE, 15783 },
	{ 0x48dd, 0x48dd, PDF_CMAP_SINGLE, 16672 },
	{ 0x48ed, 0x48ed, PDF_CMAP_SINGLE, 17032 },
	{ 0x48f3, 0x48f3, PDF_CMAP_SINGLE, 16065 },
	{ 0x48fa, 0x48fa, PDF_CMAP_SINGLE, 15940 },
	{ 0x4906, 0x4906, PDF_CMAP_SINGLE, 15303 },
	{ 0x4911, 0x4911, PDF_CMAP_SINGLE, 17796 },
	{ 0x491e, 0x491e, PDF_CMAP_SINGLE, 18324 },
	{ 0x4925, 0x4925, PDF_CMAP_SINGLE, 15375 },
	{ 0x492a, 0x492a, PDF_CMAP_SINGLE, 18094 },
	{ 0x492d, 0x492d, PDF_CMAP_SINGLE, 18125 },
	{ 0x492f, 0x4930, PDF_CMAP_TABLE, 286 },
	{ 0x4935, 0x4935, PDF_CMAP_SINGLE, 15555 },
	{ 0x493c, 0x493c, PDF_CMAP_SINGLE, 15352 },
	{ 0x493e, 0x493e, PDF_CMAP_SINGLE, 15622 },
	{ 0x4945, 0x4945, PDF_CMAP_SINGLE, 18339 },
	{ 0x4951, 0x4951, PDF_CMAP_SINGLE, 18340 },
	{ 0x4953, 0x4953, PDF_CMAP_SINGLE, 17069 },
	{ 0x4965, 0x4965, PDF_CMAP_SINGLE, 14489 },
	{ 0x496a, 0x496a, PDF_CMAP_SINGLE, 18345 },
	{ 0x4972, 0x4972, PDF_CMAP_SINGLE, 14884 },
	{ 0x4989, 0x4989, PDF_CMAP_SINGLE, 14235 },
	{ 0x49a1, 0x49a1, PDF_CMAP_SINGLE, 14519 },
	{ 0x49a7, 0x49a7, PDF_CMAP_SINGLE, 18350 },
	{ 0x49df, 0x49df, PDF_CMAP_SINGLE, 14506 },
	{ 0x49e5, 0x49e5, PDF_CMAP_SINGLE, 18353 },
	{ 0x49e7, 0x49e7, PDF_CMAP_SINGLE, 17953 },
	{ 0x4a0f, 0x4a0f, PDF_CMAP_SINGLE, 14531 },
	{ 0x4a1d, 0x4a1d, PDF_CMAP_SINGLE, 15340 },
	{ 0x4a24, 0x4a24, PDF_CMAP_SINGLE, 18354 },
	{ 0x4a35, 0x4a35, PDF_CMAP_SINGLE, 18356 },
	{ 0x4a96, 0x4a96, PDF_CMAP_SINGLE, 15591 },
	{ 0x4aa4, 0x4aa4, PDF_CMAP_SINGLE, 18993 },
	{ 0x4ab4, 0x4ab4, PDF_CMAP_SINGLE, 17249 },
	{ 0x4ab8, 0x4ab8, PDF_CMAP_SINGLE, 15784 },
	{ 0x4ad1, 0x4ad1, PDF_CMAP_SINGLE, 14563 },
	{ 0x4ae4, 0x4ae4, PDF_CMAP_SINGLE, 18359 },
	{ 0x4aff, 0x4aff, PDF_CMAP_SINGLE, 14578 },
	{ 0x4b10, 0x4b10, PDF_CMAP_SINGLE, 18873 },
	{ 0x4b19, 0x4b19, PDF_CMAP_SINGLE, 18361 },
	{ 0x4b20, 0x4b20, PDF_CMAP_SINGLE, 18858 },
	{ 0x4b2c, 0x4b2c, PDF_CMAP_SINGLE, 17951 },
	{ 0x4b37, 0x4b37, PDF_CMAP_SINGLE, 16809 },
	{ 0x4b6f, 0x4b70, PDF_CMAP_TABLE, 288 },
	{ 0x4b72, 0x4b72, PDF_CMAP_SINGLE, 14588 },
	{ 0x4b7b, 0x4b7b, PDF_CMAP_SINGLE, 15501 },
	{ 0x4b7e, 0x4b7e, PDF_CMAP_SINGLE, 16394 },
	{ 0x4b8e, 0x4b8e, PDF_CMAP_SINGLE, 14839 },
	{ 0x4b90, 0x4b90, PDF_CMAP_SINGLE, 15392 },
	{ 0x4b93, 0x4b93, PDF_CMAP_SINGLE, 14988 },
	{ 0x4b96, 0x4b97, PDF_CMAP_TABLE, 290 },
	{ 0x4b9d, 0x4b9d, PDF_CMAP_SINGLE, 18370 },
	{ 0x4bbd, 0x4bbe, PDF_CMAP_TABLE, 292 },
	{ 0x4bc0, 0x4bc0, PDF_CMAP_SINGLE, 15836 },
	{ 0x4c04, 0x4c04, PDF_CMAP_SINGLE, 16315 },
	{ 0x4c07, 0x4c07, PDF_CMAP_SINGLE, 16311 },
	{ 0x4c0e, 0x4c0e, PDF_CMAP_SINGLE, 14604 },
	{ 0x4c32, 0x4c32, PDF_CMAP_SINGLE, 18966 },
	{ 0x4c3b, 0x4c3b, PDF_CMAP_SINGLE, 16188 },
	{ 0x4c3e, 0x4c3e, PDF_CMAP_SINGLE, 17787 },
	{ 0x4c40, 0x4c40, PDF_CMAP_SINGLE, 18980 },
	{ 0x4c47, 0x4c47, PDF_CMAP_SINGLE, 18979 },
	{ 0x4c57, 0x4c57, PDF_CMAP_SINGLE, 18986 },
	{ 0x4c5b, 0x4c5b, PDF_CMAP_SINGLE, 16089 },
	{ 0x4c6d, 0x4c6d, PDF_CMAP_SINGLE, 18377 },
	{ 0x4c77, 0x4c77, PDF_CMAP_SINGLE, 19044 },
	{ 0x4c7b, 0x4c7b, PDF_CMAP_SINGLE, 19079 },
	{ 0x4c7d, 0x4c7d, PDF_CMAP_SINGLE, 15974 },
	{ 0x4c81, 0x4c81, PDF_CMAP_SINGLE, 18955 },
	{ 0x4c85, 0x4c85, PDF_CMAP_SINGLE, 18990 },
	{ 0x4ca4, 0x4ca4, PDF_CMAP_SINGLE, 18622 },
	{ 0x4cae, 0x4cae, PDF_CMAP_SINGLE, 15426 },
	{ 0x4cb0, 0x4cb0, PDF_CMAP_SINGLE, 15429 },
	{ 0x4cb7, 0x4cb7, PDF_CMAP_SINGLE, 15905 },
	{ 0x4ccd, 0x4ccd, PDF_CMAP_SINGLE, 17784 },
	{ 0x4ce1, 0x4ce2, PDF_CMAP_TABLE, 294 },
	{ 0x4ced, 0x4ced, PDF_CMAP_SINGLE, 16555 },
	{ 0x4d07, 0x4d07, PDF_CMAP_SINGLE, 18992 },
	{ 0x4d09, 0x4d09, PDF_CMAP_SINGLE, 16086 },
	{ 0x4d10, 0x4d10, PDF_CMAP_SINGLE, 16663 },
	{ 0x4d34, 0x4d34, PDF_CMAP_SINGLE, 14645 },
	{ 0x4d76, 0x4d77, PDF_CMAP_TABLE, 296 },
	{ 0x4d89, 0x4d89, PDF_CMAP_SINGLE, 19078 },
	{ 0x4d91, 0x4d91, PDF_CMAP_SINGLE, 17397 },
	{ 0x4d9c, 0x4d9c, PDF_CMAP_SINGLE, 18628 },
	{ 0x4e00, 0x4e01, PDF_CMAP_TABLE, 298 },
	{ 0x4e03, 0x4e04, PDF_CMAP_TABLE, 300 },
	{ 0x4e07, 0x4e11, PDF_CMAP_TABLE, 302 },
	{ 0x4e14, 0x4e16, PDF_CMAP_TABLE, 313 },
	{ 0x4e18, 0x4e1a, PDF_CMAP_TABLE, 316 },
	{ 0x4e1c, 0x4e1c, PDF_CMAP_SINGLE, 18656 },
	{ 0x4e1e, 0x4e1f, PDF_CMAP_RANGE, 878 },
	{ 0x4e21, 0x4e21, PDF_CMAP_SINGLE, 15725 },
	{ 0x4e24, 0x4e24, PDF_CMAP_SINGLE, 17805 },
	{ 0x4e26, 0x4e26, PDF_CMAP_SINGLE, 1320 },
	{ 0x4e28, 0x4e28, PDF_CMAP_SINGLE, 536 },
	{ 0x4e2a, 0x4e33, PDF_CMAP_TABLE, 319 },
	{ 0x4e36, 0x4e39, PDF_CMAP_TABLE, 329 },
	{ 0x4e3b, 0x4e3d, PDF_CMAP_TABLE, 333 },
	{ 0x4e3f, 0x4e3f, PDF_CMAP_SINGLE, 538 },
	{ 0x4e41, 0x4e43, PDF_CMAP_TABLE, 336 },
	{ 0x4e45, 0x4e45, PDF_CMAP_SINGLE, 622 },
	{ 0x4e47, 0x4e49, PDF_CMAP_TABLE, 339 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 664 },
	{ 0x4e4d, 0x4e4f, PDF_CMAP_TABLE, 342 },
	{ 0x4e52, 0x4e53, PDF_CMAP_RANGE, 880 },
	{ 0x4e56, 0x4e56, PDF_CMAP_SINGLE, 1321 },
	{ 0x4e58, 0x4e5f, PDF_CMAP_TABLE, 345 },
	{ 0x4e69, 0x4e6a, PDF_CMAP_TABLE, 353 },
	{ 0x4e73, 0x4e73, PDF_CMAP_SINGLE, 1322 },
	{ 0x4e78, 0x4e78, PDF_CMAP_SINGLE, 15758 },
	{ 0x4e7e, 0x4e82, PDF_CMAP_TABLE, 355 },
	{ 0x4e83, 0x4e84, PDF_CMAP_RANGE, 9057 },
	{ 0x4e85, 0x4e89, PDF_CMAP_TABLE, 360 },
	{ 0x4e8b, 0x4e8e, PDF_CMAP_TABLE, 365 },
	{ 0x4e91, 0x4e95, PDF_CMAP_TABLE, 369 },
	{ 0x4e98, 0x4e9b, PDF_CMAP_TABLE, 374 },
	{ 0x4e9e, 0x4ea6, PDF_CMAP_TABLE, 378 },
	{ 0x4ea8, 0x4ea8, PDF_CMAP_SINGLE, 1046 },
	{ 0x4eab, 0x4eac, PDF_CMAP_RANGE, 1326 },
	{ 0x4ead, 0x4eae, PDF_CMAP_RANGE, 1700 },
	{ 0x4eb3, 0x4eb3, PDF_CMAP_SINGLE, 2098 },
	{ 0x4eb6, 0x4eb7, PDF_CMAP_TABLE, 387 },
	{ 0x4eb9, 0x4ebc, PDF_CMAP_TABLE, 389 },
	{ 0x4ebf, 0x4ec4, PDF_CMAP_TABLE, 393 },
	{ 0x4ec6, 0x4ec7, PDF_CMAP_RANGE, 675 },
	{ 0x4ec8, 0x4ec9, PDF_CMAP_TABLE, 399 },
	{ 0x4eca, 0x4ecb, PDF_CMAP_RANGE, 678 },
	{ 0x4ecd, 0x4ece, PDF_CMAP_TABLE, 401 },
	{ 0x4ed4, 0x4ed7, PDF_CMAP_RANGE, 762 },
	{ 0x4ed8, 0x4eda, PDF_CMAP_TABLE, 403 },
	{ 0x4edc, 0x4edf, PDF_CMAP_TABLE, 406 },
	{ 0x4ee1, 0x4ee1, PDF_CMAP_SINGLE, 6035 },
	{ 0x4ee3, 0x4ee4, PDF_CMAP_RANGE, 766 },
	{ 0x4ee5, 0x4ee5, PDF_CMAP_SINGLE, 760 },
	{ 0x4ee8, 0x4eeb, PDF_CMAP_TABLE, 410 },
	{ 0x4eee, 0x4eee, PDF_CMAP_SINGLE, 15734 },
	{ 0x4ef0, 0x4ef8, PDF_CMAP_TABLE, 414 },
	{ 0x4efb, 0x4efb, PDF_CMAP_SINGLE, 898 },
	{ 0x4efd, 0x4f05, PDF_CMAP_TABLE, 423 },
	{ 0x4f08, 0x4f0b, PDF_CMAP_TABLE, 432 },
	{ 0x4f0d, 0x4f0f, PDF_CMAP_TABLE, 436 },
	{ 0x4f10, 0x4f11, PDF_CMAP_RANGE, 893 },
	{ 0x4f12, 0x4f15, PDF_CMAP_TABLE, 439 },
	{ 0x4f17, 0x4f1a, PDF_CMAP_TABLE, 443 },
	{ 0x4f1d, 0x4f1d, PDF_CMAP_SINGLE, 6078 },
	{ 0x4f22, 0x4f22, PDF_CMAP_SINGLE, 6081 },
	{ 0x4f28, 0x4f29, PDF_CMAP_TABLE, 447 },
	{ 0x4f2c, 0x4f2d, PDF_CMAP_TABLE, 449 },
	{ 0x4f2f, 0x4f30, PDF_CMAP_TABLE, 451 },
	{ 0x4f32, 0x4f34, PDF_CMAP_TABLE, 453 },
	{ 0x4f36, 0x4f3f, PDF_CMAP_TABLE, 456 },
	{ 0x4f41, 0x4f43, PDF_CMAP_TABLE, 466 },
	{ 0x4f45, 0x4f49, PDF_CMAP_TABLE, 469 },
	{ 0x4f4b, 0x4f4f, PDF_CMAP_TABLE, 474 },
	{ 0x4f50, 0x4f51, PDF_CMAP_RANGE, 1056 },
	{ 0x4f52, 0x4f64, PDF_CMAP_TABLE, 479 },
	{ 0x4f67, 0x4f67, PDF_CMAP_SINGLE, 6186 },
	{ 0x4f69, 0x4f6c, PDF_CMAP_TABLE, 498 },
	{ 0x4f6e, 0x4f70, PDF_CMAP_TABLE, 502 },
	{ 0x4f72, 0x4f8b, PDF_CMAP_TABLE, 505 },
	{ 0x4f8d, 0x4f8d, PDF_CMAP_SINGLE, 1330 },
	{ 0x4f8f, 0x4f92, PDF_CMAP_TABLE, 531 },
	{ 0x4f94, 0x4f98, PDF_CMAP_TABLE, 535 },
	{ 0x4f9a, 0x4f9e, PDF_CMAP_TABLE, 540 },
	{ 0x4fa2, 0x4fa2, PDF_CMAP_SINGLE, 14791 },
	{ 0x4fa8, 0x4fa8, PDF_CMAP_SINGLE, 17726 },
	{ 0x4fab, 0x4fab, PDF_CMAP_SINGLE, 16418 },
	{ 0x4fae, 0x4fb0, PDF_CMAP_TABLE, 545 },
	{ 0x4fb2, 0x4fb7, PDF_CMAP_TABLE, 548 },
	{ 0x4fb9, 0x4fbb, PDF_CMAP_TABLE, 554 },
	{ 0x4fbd, 0x4fbd, PDF_CMAP_SINGLE, 17812 },
	{ 0x4fbf, 0x4fc5, PDF_CMAP_TABLE, 557 },
	{ 0x4fc7, 0x4fd1, PDF_CMAP_TABLE, 564 },
	{ 0x4fd3, 0x4fd4, PDF_CMAP_TABLE, 575 },
	{ 0x4fd6, 0x4fe1, PDF_CMAP_TABLE, 577 },
	{ 0x4fe4, 0x4fe5, PDF_CMAP_TABLE, 589 },
	{ 0x4fec, 0x4fec, PDF_CMAP_SINGLE, 6764 },
	{ 0x4fee, 0x4ff5, PDF_CMAP_TABLE, 591 },
	{ 0x4ff6, 0x4ff7, PDF_CMAP_RANGE, 7208 },
	{ 0x4ff8, 0x4ffa, PDF_CMAP_TABLE, 599 },
	{ 0x4ffd, 0x4ffe, PDF_CMAP_TABLE, 602 },
	{ 0x5000, 0x5000, PDF_CMAP_SINGLE, 2115 },
	{ 0x5003, 0x5003, PDF_CMAP_SINGLE, 17814 },
	{ 0x5005, 0x5009, PDF_CMAP_TABLE, 604 },
	{ 0x500b, 0x500b, PDF_CMAP_SINGLE, 2120 },
	{ 0x500c, 0x500d, PDF_CMAP_RANGE, 2099 },
	{ 0x500e, 0x500f, PDF_CMAP_TABLE, 609 },
	{ 0x5011, 0x501c, PDF_CMAP_TABLE, 611 },
	{ 0x501e, 0x5023, PDF_CMAP_TABLE, 623 },
	{ 0x5025, 0x5031, PDF_CMAP_TABLE, 629 },
	{ 0x5033, 0x5035, PDF_CMAP_TABLE, 642 },
	{ 0x5037, 0x5037, PDF_CMAP_SINGLE, 7206 },
	{ 0x503b, 0x503c, PDF_CMAP_TABLE, 645 },
	{ 0x5040, 0x5041, PDF_CMAP_TABLE, 647 },
	{ 0x5043, 0x5043, PDF_CMAP_SINGLE, 2560 },
	{ 0x5045, 0x504f, PDF_CMAP_TABLE, 649 },
	{ 0x5051, 0x5051, PDF_CMAP_SINGLE, 7761 },
	{ 0x5053, 0x5053, PDF_CMAP_SINGLE, 7739 },
	{ 0x5055, 0x5058, PDF_CMAP_TABLE, 660 },
	{ 0x505a, 0x5062, PDF_CMAP_TABLE, 664 },
	{ 0x5063, 0x5064, PDF_CMAP_RANGE, 7754 },
	{ 0x5065, 0x5066, PDF_CMAP_TABLE, 673 },
	{ 0x5068, 0x5070, PDF_CMAP_TABLE, 675 },
	{ 0x5072, 0x5077, PDF_CMAP_TABLE, 684 },
	{ 0x507a, 0x507a, PDF_CMAP_SINGLE, 2556 },
	{ 0x507d, 0x507d, PDF_CMAP_SINGLE, 2557 },
	{ 0x5080, 0x5083, PDF_CMAP_TABLE, 690 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 3049 },
	{ 0x5087, 0x5088, PDF_CMAP_TABLE, 694 },
	{ 0x508b, 0x508e, PDF_CMAP_TABLE, 696 },
	{ 0x5090, 0x5092, PDF_CMAP_TABLE, 700 },
	{ 0x5094, 0x5096, PDF_CMAP_TABLE, 703 },
	{ 0x5098, 0x509e, PDF_CMAP_TABLE, 706 },
	{ 0x50a2, 0x50a3, PDF_CMAP_TABLE, 713 },
	{ 0x50a6, 0x50a6, PDF_CMAP_SINGLE, 16191 },
	{ 0x50ac, 0x50b1, PDF_CMAP_TABLE, 715 },
	{ 0x50b2, 0x50b3, PDF_CMAP_RANGE, 3521 },
	{ 0x50b4, 0x50b8, PDF_CMAP_TABLE, 721 },
	{ 0x50ba, 0x50bf, PDF_CMAP_TABLE, 726 },
	{ 0x50c1, 0x50c2, PDF_CMAP_TABLE, 732 },
	{ 0x50c4, 0x50cb, PDF_CMAP_TABLE, 734 },
	{ 0x50cd, 0x50d1, PDF_CMAP_TABLE, 742 },
	{ 0x50d3, 0x50d7, PDF_CMAP_TABLE, 747 },
	{ 0x50d9, 0x50db, PDF_CMAP_TABLE, 752 },
	{ 0x50dd, 0x50e1, PDF_CMAP_TABLE, 755 },
	{ 0x50e3, 0x50ea, PDF_CMAP_TABLE, 760 },
	{ 0x50ec, 0x50f1, PDF_CMAP_TABLE, 768 },
	{ 0x50f3, 0x50f6, PDF_CMAP_TABLE, 774 },
	{ 0x50f8, 0x50f9, PDF_CMAP_TABLE, 778 },
	{ 0x50fb, 0x5105, PDF_CMAP_TABLE, 780 },
	{ 0x5106, 0x5107, PDF_CMAP_RANGE, 10369 },
	{ 0x5108, 0x5109, PDF_CMAP_RANGE, 4360 },
	{ 0x510a, 0x510a, PDF_CMAP_SINGLE, 10376 },
	{ 0x510b, 0x510c, PDF_CMAP_RANGE, 10373 },
	{ 0x510d, 0x510e, PDF_CMAP_TABLE, 791 },
	{ 0x5110, 0x5115, PDF_CMAP_TABLE, 793 },
	{ 0x5117, 0x5118, PDF_CMAP_TABLE, 799 },
	{ 0x511a, 0x511a, PDF_CMAP_SINGLE, 11011 },
	{ 0x511c, 0x511c, PDF_CMAP_SINGLE, 11008 },
	{ 0x511f, 0x5122, PDF_CMAP_TABLE, 801 },
	{ 0x5124, 0x5126, PDF_CMAP_TABLE, 805 },
	{ 0x5129, 0x512b, PDF_CMAP_TABLE, 808 },
	{ 0x512d, 0x512e, PDF_CMAP_RANGE, 12047 },
	{ 0x5130, 0x5133, PDF_CMAP_TABLE, 811 },
	{ 0x5134, 0x5135, PDF_CMAP_RANGE, 12410 },
	{ 0x5137, 0x5138, PDF_CMAP_RANGE, 5736 },
	{ 0x5139, 0x513d, PDF_CMAP_TABLE, 815 },
	{ 0x513f, 0x5141, PDF_CMAP_TABLE, 820 },
	{ 0x5143, 0x5149, PDF_CMAP_TABLE, 823 },
	{ 0x514b, 0x514d, PDF_CMAP_TABLE, 830 },
	{ 0x5152, 0x5152, PDF_CMAP_SINGLE, 1349 },
	{ 0x5154, 0x5157, PDF_CMAP_TABLE, 833 },
	{ 0x5159, 0x5163, PDF_CMAP_TABLE, 837 },
	{ 0x5165, 0x5165, PDF_CMAP_SINGLE, 605 },
	{ 0x5167, 0x516c, PDF_CMAP_TABLE, 848 },
	{ 0x516d, 0x516e, PDF_CMAP_RANGE, 684 },
	{ 0x5171, 0x5171, PDF_CMAP_SINGLE, 909 },
	{ 0x5174, 0x5179, PDF_CMAP_TABLE, 854 },
	{ 0x517c, 0x517c, PDF_CMAP_SINGLE, 2130 },
	{ 0x5180, 0x5180, PDF_CMAP_SINGLE, 4751 },
	{ 0x5182, 0x5182, PDF_CMAP_SINGLE, 541 },
	{ 0x5186, 0x5188, PDF_CMAP_TABLE, 860 },
	{ 0x5189, 0x518a, PDF_CMAP_RANGE, 772 },
	{ 0x518d, 0x518d, PDF_CMAP_SINGLE, 910 },
	{ 0x518f, 0x518f, PDF_CMAP_SINGLE, 6195 },
	{ 0x5191, 0x5198, PDF_CMAP_TABLE, 863 },
	{ 0x519a, 0x519a, PDF_CMAP_SINGLE, 16035 },
	{ 0x519c, 0x519c, PDF_CMAP_SINGLE, 17728 },
	{ 0x519e, 0x519e, PDF_CMAP_SINGLE, 6416 },
	{ 0x51a0, 0x51a0, PDF_CMAP_SINGLE, 1727 },
	{ 0x51a2, 0x51a2, PDF_CMAP_SINGLE, 2133 },
	{ 0x51a4, 0x51a5, PDF_CMAP_RANGE, 2131 },
	{ 0x51a7, 0x51a8, PDF_CMAP_TABLE, 871 },
	{ 0x51aa, 0x51ac, PDF_CMAP_TABLE, 873 },
	{ 0x51ae, 0x51ae, PDF_CMAP_SINGLE, 18876 },
	{ 0x51b0, 0x51b5, PDF_CMAP_TABLE, 876 },
	{ 0x51b6, 0x51b7, PDF_CMAP_RANGE, 1079 },
	{ 0x51b8, 0x51b9, PDF_CMAP_TABLE, 882 },
	{ 0x51bc, 0x51be, PDF_CMAP_TABLE, 884 },
	{ 0x51c3, 0x51c3, PDF_CMAP_SINGLE, 14811 },
	{ 0x51c4, 0x51c5, PDF_CMAP_RANGE, 7222 },
	{ 0x51c6, 0x51d4, PDF_CMAP_TABLE, 887 },
	{ 0x51d7, 0x51d8, PDF_CMAP_TABLE, 902 },
	{ 0x51db, 0x51e2, PDF_CMAP_TABLE, 904 },
	{ 0x51e4, 0x51e4, PDF_CMAP_SINGLE, 17729 },
	{ 0x51ed, 0x51ed, PDF_CMAP_SINGLE, 16468 },
	{ 0x51f0, 0x51f1, PDF_CMAP_TABLE, 912 },
	{ 0x51f3, 0x51f6, PDF_CMAP_TABLE, 914 },
	{ 0x51f8, 0x51f8, PDF_CMAP_SINGLE, 777 },
	{ 0x51f9, 0x51fa, PDF_CMAP_RANGE, 775 },
	{ 0x51fc, 0x51fe, PDF_CMAP_TABLE, 918 },
	{ 0x5200, 0x5201, PDF_CMAP_RANGE, 608 },
	{ 0x5202, 0x5203, PDF_CMAP_TABLE, 921 },
	{ 0x5205, 0x5205, PDF_CMAP_SINGLE, 17313 },
	{ 0x5206, 0x5208, PDF_CMAP_RANGE, 689 },
	{ 0x5209, 0x520c, PDF_CMAP_TABLE, 923 },
	{ 0x520e, 0x520e, PDF_CMAP_SINGLE, 915 },
	{ 0x5210, 0x5210, PDF_CMAP_SINGLE, 6088 },
	{ 0x5211, 0x5212, PDF_CMAP_RANGE, 913 },
	{ 0x5213, 0x5213, PDF_CMAP_SINGLE, 6087 },
	{ 0x5216, 0x5217, PDF_CMAP_TABLE, 927 },
	{ 0x521c, 0x5221, PDF_CMAP_TABLE, 929 },
	{ 0x5224, 0x5228, PDF_CMAP_TABLE, 935 },
	{ 0x5229, 0x522a, PDF_CMAP_RANGE, 1083 },
	{ 0x522e, 0x522e, PDF_CMAP_SINGLE, 1362 },
	{ 0x5230, 0x5231, PDF_CMAP_TABLE, 940 },
	{ 0x5232, 0x5233, PDF_CMAP_RANGE, 6420 },
	{ 0x5234, 0x5238, PDF_CMAP_TABLE, 942 },
	{ 0x523a, 0x523c, PDF_CMAP_TABLE, 947 },
	{ 0x5241, 0x5241, PDF_CMAP_SINGLE, 1364 },
	{ 0x5243, 0x5244, PDF_CMAP_TABLE, 950 },
	{ 0x5246, 0x5247, PDF_CMAP_TABLE, 952 },
	{ 0x5249, 0x524f, PDF_CMAP_TABLE, 954 },
	{ 0x5252, 0x5252, PDF_CMAP_SINGLE, 7228 },
	{ 0x5254, 0x5257, PDF_CMAP_TABLE, 961 },
	{ 0x5259, 0x525d, PDF_CMAP_TABLE, 965 },
	{ 0x525e, 0x525f, PDF_CMAP_RANGE, 7229 },
	{ 0x5260, 0x5262, PDF_CMAP_TABLE, 970 },
	{ 0x5268, 0x526f, PDF_CMAP_TABLE, 973 },
	{ 0x5272, 0x5273, PDF_CMAP_TABLE, 981 },
	{ 0x5274, 0x5275, PDF_CMAP_RANGE, 3059 },
	{ 0x5277, 0x527a, PDF_CMAP_TABLE, 983 },
	{ 0x527b, 0x527c, PDF_CMAP_RANGE, 9080 },
	{ 0x527d, 0x527d, PDF_CMAP_SINGLE, 3532 },
	{ 0x527f, 0x527f, PDF_CMAP_SINGLE, 3530 },
	{ 0x5280, 0x5281, PDF_CMAP_RANGE, 9763 },
	{ 0x5282, 0x5284, PDF_CMAP_TABLE, 987 },
	{ 0x5287, 0x5289, PDF_CMAP_RANGE, 4364 },
	{ 0x528a, 0x528a, PDF_CMAP_SINGLE, 4368 },
	{ 0x528b, 0x528c, PDF_CMAP_RANGE, 10377 },
	{ 0x528d, 0x528d, PDF_CMAP_SINGLE, 4367 },
	{ 0x528f, 0x5291, PDF_CMAP_TABLE, 990 },
	{ 0x5293, 0x5294, PDF_CMAP_TABLE, 993 },
	{ 0x5296, 0x529b, PDF_CMAP_TABLE, 995 },
	{ 0x529f, 0x52a1, PDF_CMAP_TABLE, 1001 },
	{ 0x52a3, 0x52a4, PDF_CMAP_TABLE, 1004 },
	{ 0x52a6, 0x52a6, PDF_CMAP_SINGLE, 6089 },
	{ 0x52a8, 0x52a8, PDF_CMAP_SINGLE, 17731 },
	{ 0x52a9, 0x52aa, PDF_CMAP_RANGE, 1087 },
	{ 0x52ab, 0x52ac, PDF_CMAP_TABLE, 1006 },
	{ 0x52ad, 0x52ae, PDF_CMAP_RANGE, 6200 },
	{ 0x52b5, 0x52b5, PDF_CMAP_SINGLE, 16476 },
	{ 0x52b9, 0x52b9, PDF_CMAP_SINGLE, 16479 },
	{ 0x52bb, 0x52bc, PDF_CMAP_TABLE, 1008 },
	{ 0x52be, 0x52be, PDF_CMAP_SINGLE, 1365 },
	{ 0x52c0, 0x52c3, PDF_CMAP_TABLE, 1010 },
	{ 0x52c5, 0x52c5, PDF_CMAP_SINGLE, 16480 },
	{ 0x52c7, 0x52c7, PDF_CMAP_SINGLE, 1735 },
	{ 0x52c9, 0x52c9, PDF_CMAP_SINGLE, 1736 },
	{ 0x52cc, 0x52cd, PDF_CMAP_TABLE, 1014 },
	{ 0x52d0, 0x52d3, PDF_CMAP_TABLE, 1016 },
	{ 0x52d5, 0x52d9, PDF_CMAP_TABLE, 1020 },
	{ 0x52db, 0x52db, PDF_CMAP_SINGLE, 3064 },
	{ 0x52dd, 0x52e1, PDF_CMAP_TABLE, 1025 },
	{ 0x52e2, 0x52e3, PDF_CMAP_RANGE, 3536 },
	{ 0x52e4, 0x52e4, PDF_CMAP_SINGLE, 3535 },
	{ 0x52e6, 0x52e6, PDF_CMAP_SINGLE, 3534 },
	{ 0x52e9, 0x52e9, PDF_CMAP_SINGLE, 9765 },
	{ 0x52eb, 0x52eb, PDF_CMAP_SINGLE, 9766 },
	{ 0x52ef, 0x52f1, PDF_CMAP_TABLE, 1030 },
	{ 0x52f3, 0x52f5, PDF_CMAP_TABLE, 1033 },
	{ 0x52f7, 0x52fc, PDF_CMAP_TABLE, 1036 },
	{ 0x52fe, 0x52ff, PDF_CMAP_RANGE, 693 },
	{ 0x5301, 0x5301, PDF_CMAP_SINGLE, 15737 },
	{ 0x5305, 0x5306, PDF_CMAP_RANGE, 781 },
	{ 0x5308, 0x5309, PDF_CMAP_TABLE, 1042 },
	{ 0x530a, 0x530b, PDF_CMAP_RANGE, 6425 },
	{ 0x530d, 0x5312, PDF_CMAP_TABLE, 1044 },
	{ 0x5315, 0x5317, PDF_CMAP_TABLE, 1050 },
	{ 0x5319, 0x531a, PDF_CMAP_TABLE, 1053 },
	{ 0x531c, 0x531d, PDF_CMAP_TABLE, 1055 },
	{ 0x531f, 0x5324, PDF_CMAP_TABLE, 1057 },
	{ 0x5327, 0x5327, PDF_CMAP_SINGLE, 17822 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 2143 },
	{ 0x532c, 0x532d, PDF_CMAP_TABLE, 1063 },
	{ 0x532f, 0x5334, PDF_CMAP_TABLE, 1065 },
	{ 0x5337, 0x5339, PDF_CMAP_TABLE, 1071 },
	{ 0x533b, 0x533e, PDF_CMAP_TABLE, 1074 },
	{ 0x533f, 0x5340, PDF_CMAP_RANGE, 2587 },
	{ 0x5341, 0x5345, PDF_CMAP_TABLE, 1078 },
	{ 0x5347, 0x534a, PDF_CMAP_TABLE, 1083 },
	{ 0x534c, 0x534e, PDF_CMAP_TABLE, 1087 },
	{ 0x5351, 0x5354, PDF_CMAP_TABLE, 1090 },
	{ 0x5357, 0x5357, PDF_CMAP_SINGLE, 1740 },
	{ 0x535a, 0x535a, PDF_CMAP_SINGLE, 3065 },
	{ 0x535c, 0x5361, PDF_CMAP_TABLE, 1094 },
	{ 0x5363, 0x5364, PDF_CMAP_TABLE, 1100 },
	{ 0x5366, 0x5367, PDF_CMAP_TABLE, 1102 },
	{ 0x5369, 0x5369, PDF_CMAP_SINGLE, 546 },
	{ 0x536c, 0x536f, PDF_CMAP_TABLE, 1104 },
	{ 0x5370, 0x5371, PDF_CMAP_RANGE, 921 },
	{ 0x5372, 0x5375, PDF_CMAP_TABLE, 1108 },
	{ 0x5377, 0x5379, PDF_CMAP_RANGE, 1372 },
	{ 0x537b, 0x537f, PDF_CMAP_TABLE, 1112 },
	{ 0x5382, 0x5382, PDF_CMAP_SINGLE, 6000 },
	{ 0x5384, 0x5384, PDF_CMAP_SINGLE, 701 },
	{ 0x538a, 0x538a, PDF_CMAP_SINGLE, 6093 },
	{ 0x538e, 0x538f, PDF_CMAP_RANGE, 6205 },
	{ 0x5392, 0x5394, PDF_CMAP_TABLE, 1117 },
	{ 0x5396, 0x539a, PDF_CMAP_TABLE, 1120 },
	{ 0x539c, 0x53a0, PDF_CMAP_TABLE, 1125 },
	{ 0x53a2, 0x53a2, PDF_CMAP_SINGLE, 15915 },
	{ 0x53a4, 0x53ae, PDF_CMAP_TABLE, 1130 },
	{ 0x53b0, 0x53b0, PDF_CMAP_SINGLE, 17826 },
	{ 0x53b2, 0x53b2, PDF_CMAP_SINGLE, 4370 },
	{ 0x53b4, 0x53b4, PDF_CMAP_SINGLE, 12414 },
	{ 0x53b6, 0x53b6, PDF_CMAP_SINGLE, 547 },
	{ 0x53b9, 0x53b9, PDF_CMAP_SINGLE, 6019 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 792 },
	{ 0x53c1, 0x53c3, PDF_CMAP_TABLE, 1141 },
	{ 0x53c5, 0x53c5, PDF_CMAP_SINGLE, 16524 },
	{ 0x53c8, 0x53cd, PDF_CMAP_TABLE, 1144 },
	{ 0x53d0, 0x53d2, PDF_CMAP_TABLE, 1150 },
	{ 0x53d4, 0x53d4, PDF_CMAP_SINGLE, 1376 },
	{ 0x53d6, 0x53db, PDF_CMAP_TABLE, 1153 },
	{ 0x53df, 0x53e6, PDF_CMAP_TABLE, 1159 },
	{ 0x53e8, 0x53f3, PDF_CMAP_TABLE, 1167 },
	{ 0x53f5, 0x53f8, PDF_CMAP_TABLE, 1179 },
	{ 0x53fb, 0x53fc, PDF_CMAP_TABLE, 1183 },
	{ 0x53fe, 0x53fe, PDF_CMAP_SINGLE, 16577 },
	{ 0x5401, 0x5401, PDF_CMAP_SINGLE, 928 },
	{ 0x5403, 0x5404, PDF_CMAP_TABLE, 1185 },
	{ 0x5406, 0x5414, PDF_CMAP_TABLE, 1187 },
	{ 0x5416, 0x5416, PDF_CMAP_SINGLE, 15999 },
	{ 0x5418, 0x5421, PDF_CMAP_TABLE, 1202 },
	{ 0x5423, 0x5434, PDF_CMAP_TABLE, 1212 },
	{ 0x5435, 0x5436, PDF_CMAP_RANGE, 1112 },
	{ 0x5437, 0x5439, PDF_CMAP_TABLE, 1230 },
	{ 0x543b, 0x543e, PDF_CMAP_TABLE, 1233 },
	{ 0x5440, 0x5443, PDF_CMAP_TABLE, 1237 },
	{ 0x5445, 0x5448, PDF_CMAP_TABLE, 1241 },
	{ 0x544a, 0x544f, PDF_CMAP_TABLE, 1245 },
	{ 0x5454, 0x5454, PDF_CMAP_SINGLE, 6210 },
	{ 0x5460, 0x546d, PDF_CMAP_TABLE, 1251 },
	{ 0x546f, 0x5478, PDF_CMAP_TABLE, 1265 },
	{ 0x547a, 0x5482, PDF_CMAP_TABLE, 1275 },
	{ 0x5484, 0x5488, PDF_CMAP_TABLE, 1284 },
	{ 0x548b, 0x5498, PDF_CMAP_TABLE, 1289 },
	{ 0x549a, 0x549a, PDF_CMAP_SINGLE, 1394 },
	{ 0x549c, 0x549c, PDF_CMAP_SINGLE, 16230 },
	{ 0x549e, 0x549e, PDF_CMAP_SINGLE, 18413 },
	{ 0x54a0, 0x54b4, PDF_CMAP_TABLE, 1303 },
	{ 0x54b6, 0x54c4, PDF_CMAP_TABLE, 1324 },
	{ 0x54c5, 0x54c6, PDF_CMAP_RANGE, 6786 },
	{ 0x54c7, 0x54c9, PDF_CMAP_TABLE, 1339 },
	{ 0x54cb, 0x54d0, PDF_CMAP_TABLE, 1342 },
	{ 0x54d6, 0x54d6, PDF_CMAP_SINGLE, 6784 },
	{ 0x54da, 0x54da, PDF_CMAP_SINGLE, 18723 },
	{ 0x54de, 0x54de, PDF_CMAP_SINGLE, 6794 },
	{ 0x54e0, 0x54eb, PDF_CMAP_TABLE, 1348 },
	{ 0x54ed, 0x54ef, PDF_CMAP_TABLE, 1360 },
	{ 0x54f1, 0x54f3, PDF_CMAP_TABLE, 1363 },
	{ 0x54f7, 0x54f8, PDF_CMAP_RANGE, 7253 },
	{ 0x54fa, 0x54fd, PDF_CMAP_TABLE, 1366 },
	{ 0x54ff, 0x54ff, PDF_CMAP_SINGLE, 7244 },
	{ 0x5501, 0x5514, PDF_CMAP_TABLE, 1370 },
	{ 0x5517, 0x5518, PDF_CMAP_TABLE, 1390 },
	{ 0x551a, 0x551a, PDF_CMAP_SINGLE, 7243 },
	{ 0x551e, 0x551e, PDF_CMAP_SINGLE, 16040 },
	{ 0x5523, 0x5523, PDF_CMAP_SINGLE, 17838 },
	{ 0x5525, 0x5528, PDF_CMAP_TABLE, 1392 },
	{ 0x552a, 0x5539, PDF_CMAP_TABLE, 1396 },
	{ 0x553b, 0x553c, PDF_CMAP_TABLE, 1412 },
	{ 0x553e, 0x5541, PDF_CMAP_TABLE, 1414 },
	{ 0x5543, 0x554b, PDF_CMAP_TABLE, 1418 },
	{ 0x554d, 0x5553, PDF_CMAP_TABLE, 1427 },
	{ 0x5555, 0x5557, PDF_CMAP_TABLE, 1434 },
	{ 0x555c, 0x555f, PDF_CMAP_TABLE, 1437 },
	{ 0x5561, 0x5566, PDF_CMAP_TABLE, 1441 },
	{ 0x5569, 0x556b, PDF_CMAP_TABLE, 1447 },
	{ 0x5571, 0x5573, PDF_CMAP_TABLE, 1450 },
	{ 0x5575, 0x5576, PDF_CMAP_RANGE, 7771 },
	{ 0x5577, 0x5577, PDF_CMAP_SINGLE, 8391 },
	{ 0x5579, 0x5579, PDF_CMAP_SINGLE, 17245 },
	{ 0x557b, 0x5584, PDF_CMAP_TABLE, 1453 },
	{ 0x5586, 0x5595, PDF_CMAP_TABLE, 1463 },
	{ 0x5598, 0x559a, PDF_CMAP_TABLE, 1479 },
	{ 0x559c, 0x559d, PDF_CMAP_TABLE, 1482 },
	{ 0x559f, 0x559f, PDF_CMAP_SINGLE, 3083 },
	{ 0x55a1, 0x55ae, PDF_CMAP_TABLE, 1484 },
	{ 0x55b0, 0x55b5, PDF_CMAP_TABLE, 1498 },
	{ 0x55b9, 0x55bc, PDF_CMAP_TABLE, 1504 },
	{ 0x55bf, 0x55c4, PDF_CMAP_TABLE, 1508 },
	{ 0x55c5, 0x55c6, PDF_CMAP_RANGE, 3552 },
	{ 0x55c7, 0x55df, PDF_CMAP_TABLE, 1514 },
	{ 0x55e1, 0x55e2, PDF_CMAP_TABLE, 1539 },
	{ 0x55e3, 0x55e4, PDF_CMAP_RANGE, 3547 },
	{ 0x55e5, 0x55ea, PDF_CMAP_TABLE, 1541 },
	{ 0x55ec, 0x55ec, PDF_CMAP_SINGLE, 14290 },
	{ 0x55ef, 0x55f2, PDF_CMAP_TABLE, 1547 },
	{ 0x55f5, 0x55f7, PDF_CMAP_TABLE, 1551 },
	{ 0x55f9, 0x5602, PDF_CMAP_TABLE, 1554 },
	{ 0x5604, 0x5606, PDF_CMAP_TABLE, 1564 },
	{ 0x5608, 0x5609, PDF_CMAP_TABLE, 1567 },
	{ 0x560c, 0x560c, PDF_CMAP_SINGLE, 9771 },
	{ 0x560d, 0x560e, PDF_CMAP_RANGE, 3992 },
	{ 0x560f, 0x5617, PDF_CMAP_TABLE, 1569 },
	{ 0x561b, 0x5623, PDF_CMAP_TABLE, 1578 },
	{ 0x5625, 0x5625, PDF_CMAP_SINGLE, 15992 },
	{ 0x5627, 0x5627, PDF_CMAP_SINGLE, 9769 },
	{ 0x5629, 0x562a, PDF_CMAP_TABLE, 1587 },
	{ 0x562c, 0x562e, PDF_CMAP_TABLE, 1589 },
	{ 0x562f, 0x5630, PDF_CMAP_RANGE, 4383 },
	{ 0x5632, 0x563b, PDF_CMAP_TABLE, 1592 },
	{ 0x563d, 0x5643, PDF_CMAP_TABLE, 1602 },
	{ 0x5645, 0x5646, PDF_CMAP_TABLE, 1609 },
	{ 0x5648, 0x564a, PDF_CMAP_TABLE, 1611 },
	{ 0x564c, 0x5650, PDF_CMAP_TABLE, 1614 },
	{ 0x5652, 0x5654, PDF_CMAP_TABLE, 1619 },
	{ 0x5657, 0x565a, PDF_CMAP_TABLE, 1622 },
	{ 0x565d, 0x565e, PDF_CMAP_TABLE, 1626 },
	{ 0x5660, 0x5666, PDF_CMAP_TABLE, 1628 },
	{ 0x5668, 0x5674, PDF_CMAP_TABLE, 1635 },
	{ 0x5676, 0x567c, PDF_CMAP_TABLE, 1648 },
	{ 0x567e, 0x5687, PDF_CMAP_TABLE, 1655 },
	{ 0x5689, 0x568b, PDF_CMAP_TABLE, 1665 },
	{ 0x568c, 0x568d, PDF_CMAP_RANGE, 11586 },
	{ 0x568e, 0x5690, PDF_CMAP_TABLE, 1668 },
	{ 0x5692, 0x5693, PDF_CMAP_TABLE, 1671 },
	{ 0x5695, 0x5695, PDF_CMAP_SINGLE, 5322 },
	{ 0x5697, 0x569a, PDF_CMAP_TABLE, 1673 },
	{ 0x569c, 0x569f, PDF_CMAP_TABLE, 1677 },
	{ 0x56a1, 0x56a1, PDF_CMAP_SINGLE, 17259 },
	{ 0x56a4, 0x56a5, PDF_CMAP_TABLE, 1681 },
	{ 0x56a6, 0x56a7, PDF_CMAP_RANGE, 12417 },
	{ 0x56a8, 0x56a8, PDF_CMAP_SINGLE, 5495 },
	{ 0x56aa, 0x56af, PDF_CMAP_TABLE, 1683 },
	{ 0x56b1, 0x56b7, PDF_CMAP_TABLE, 1689 },
	{ 0x56b9, 0x56b9, PDF_CMAP_SINGLE, 18543 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 5645 },
	{ 0x56bd, 0x56be, PDF_CMAP_RANGE, 13002 },
	{ 0x56bf, 0x56c3, PDF_CMAP_TABLE, 1696 },
	{ 0x56c5, 0x56c6, PDF_CMAP_TABLE, 1701 },
	{ 0x56c8, 0x56cd, PDF_CMAP_TABLE, 1703 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 5919 },
	{ 0x56d3, 0x56d4, PDF_CMAP_TABLE, 1709 },
	{ 0x56d6, 0x56d7, PDF_CMAP_TABLE, 1711 },
	{ 0x56da, 0x56db, PDF_CMAP_TABLE, 1713 },
	{ 0x56dd, 0x56e2, PDF_CMAP_TABLE, 1715 },
	{ 0x56e4, 0x56e5, PDF_CMAP_TABLE, 1721 },
	{ 0x56e7, 0x56e7, PDF_CMAP_SINGLE, 6223 },
	{ 0x56ea, 0x56eb, PDF_CMAP_TABLE, 1723 },
	{ 0x56ed, 0x56f1, PDF_CMAP_TABLE, 1725 },
	{ 0x56f7, 0x56f7, PDF_CMAP_SINGLE, 6451 },
	{ 0x56f9, 0x56fb, PDF_CMAP_TABLE, 1730 },
	{ 0x56fd, 0x56fd, PDF_CMAP_SINGLE, 15716 },
	{ 0x56ff, 0x5700, PDF_CMAP_TABLE, 1733 },
	{ 0x5701, 0x5702, PDF_CMAP_RANGE, 7259 },
	{ 0x5703, 0x5704, PDF_CMAP_RANGE, 2169 },
	{ 0x5707, 0x570d, PDF_CMAP_TABLE, 1735 },
	{ 0x5712, 0x5713, PDF_CMAP_RANGE, 3556 },
	{ 0x5714, 0x5716, PDF_CMAP_TABLE, 1742 },
	{ 0x5718, 0x5718, PDF_CMAP_SINGLE, 4000 },
	{ 0x571a, 0x5720, PDF_CMAP_TABLE, 1745 },
	{ 0x5722, 0x5723, PDF_CMAP_RANGE, 6042 },
	{ 0x5728, 0x572a, PDF_CMAP_TABLE, 1752 },
	{ 0x572c, 0x5730, PDF_CMAP_TABLE, 1755 },
	{ 0x5732, 0x5734, PDF_CMAP_TABLE, 1760 },
	{ 0x573b, 0x573b, PDF_CMAP_SINGLE, 1134 },
	{ 0x573d, 0x5743, PDF_CMAP_TABLE, 1763 },
	{ 0x5745, 0x5747, PDF_CMAP_TABLE, 1770 },
	{ 0x5749, 0x5752, PDF_CMAP_TABLE, 1773 },
	{ 0x5754, 0x5754, PDF_CMAP_SINGLE, 18309 },
	{ 0x5757, 0x5757, PDF_CMAP_SINGLE, 18406 },
	{ 0x575b, 0x575b, PDF_CMAP_SINGLE, 14722 },
	{ 0x575f, 0x575f, PDF_CMAP_SINGLE, 16319 },
	{ 0x5761, 0x5762, PDF_CMAP_TABLE, 1783 },
	{ 0x5764, 0x5764, PDF_CMAP_SINGLE, 1407 },
	{ 0x5766, 0x576b, PDF_CMAP_TABLE, 1785 },
	{ 0x576d, 0x576d, PDF_CMAP_SINGLE, 6455 },
	{ 0x576f, 0x5772, PDF_CMAP_TABLE, 1791 },
	{ 0x5773, 0x5774, PDF_CMAP_RANGE, 6463 },
	{ 0x5775, 0x5777, PDF_CMAP_TABLE, 1795 },
	{ 0x577a, 0x5780, PDF_CMAP_TABLE, 1798 },
	{ 0x5782, 0x5783, PDF_CMAP_TABLE, 1805 },
	{ 0x5788, 0x5788, PDF_CMAP_SINGLE, 18507 },
	{ 0x578a, 0x578d, PDF_CMAP_TABLE, 1807 },
	{ 0x578f, 0x5790, PDF_CMAP_TABLE, 1811 },
	{ 0x5793, 0x5795, PDF_CMAP_TABLE, 1813 },
	{ 0x5797, 0x579d, PDF_CMAP_TABLE, 1816 },
	{ 0x579e, 0x579f, PDF_CMAP_RANGE, 6797 },
	{ 0x57a0, 0x57a5, PDF_CMAP_TABLE, 1823 },
	{ 0x57a7, 0x57a7, PDF_CMAP_SINGLE, 18708 },
	{ 0x57aa, 0x57aa, PDF_CMAP_SINGLE, 18693 },
	{ 0x57ae, 0x57ae, PDF_CMAP_SINGLE, 1773 },
	{ 0x57b3, 0x57b6, PDF_CMAP_TABLE, 1829 },
	{ 0x57b8, 0x57bf, PDF_CMAP_TABLE, 1833 },
	{ 0x57c1, 0x57c4, PDF_CMAP_TABLE, 1841 },
	{ 0x57c6, 0x57c8, PDF_CMAP_TABLE, 1845 },
	{ 0x57cb, 0x57cc, PDF_CMAP_TABLE, 1848 },
	{ 0x57ce, 0x57d0, PDF_CMAP_TABLE, 1850 },
	{ 0x57d2, 0x57d2, PDF_CMAP_SINGLE, 7264 },
	{ 0x57d4, 0x57d5, PDF_CMAP_TABLE, 1853 },
	{ 0x57d7, 0x57d7, PDF_CMAP_SINGLE, 15485 },
	{ 0x57dc, 0x57e7, PDF_CMAP_TABLE, 1855 },
	{ 0x57e9, 0x57e9, PDF_CMAP_SINGLE, 7825 },
	{ 0x57ec, 0x57fe, PDF_CMAP_TABLE, 1867 },
	{ 0x5800, 0x580e, PDF_CMAP_TABLE, 1886 },
	{ 0x5810, 0x5810, PDF_CMAP_SINGLE, 7820 },
	{ 0x5812, 0x5812, PDF_CMAP_SINGLE, 15626 },
	{ 0x5814, 0x5814, PDF_CMAP_SINGLE, 7798 },
	{ 0x5818, 0x5819, PDF_CMAP_TABLE, 1901 },
	{ 0x581b, 0x582a, PDF_CMAP_TABLE, 1903 },
	{ 0x582c, 0x582f, PDF_CMAP_TABLE, 1919 },
	{ 0x5830, 0x5831, PDF_CMAP_RANGE, 3099 },
	{ 0x5832, 0x583b, PDF_CMAP_TABLE, 1923 },
	{ 0x583d, 0x583d, PDF_CMAP_SINGLE, 9119 },
	{ 0x583f, 0x5840, PDF_CMAP_TABLE, 1933 },
	{ 0x5844, 0x5844, PDF_CMAP_SINGLE, 18363 },
	{ 0x5847, 0x584f, PDF_CMAP_TABLE, 1935 },
	{ 0x5851, 0x5855, PDF_CMAP_TABLE, 1944 },
	{ 0x5857, 0x585f, PDF_CMAP_TABLE, 1949 },
	{ 0x5862, 0x5865, PDF_CMAP_TABLE, 1958 },
	{ 0x5868, 0x5869, PDF_CMAP_TABLE, 1962 },
	{ 0x586b, 0x586d, PDF_CMAP_TABLE, 1964 },
	{ 0x586f, 0x586f, PDF_CMAP_SINGLE, 9112 },
	{ 0x5871, 0x5876, PDF_CMAP_TABLE, 1967 },
	{ 0x5879, 0x5883, PDF_CMAP_TABLE, 1973 },
	{ 0x5885, 0x588b, PDF_CMAP_TABLE, 1984 },
	{ 0x588e, 0x5894, PDF_CMAP_TABLE, 1991 },
	{ 0x5896, 0x5896, PDF_CMAP_SINGLE, 19108 },
	{ 0x5898, 0x589a, PDF_CMAP_TABLE, 1998 },
	{ 0x589c, 0x58a1, PDF_CMAP_TABLE, 2001 },
	{ 0x58a3, 0x58a3, PDF_CMAP_SINGLE, 10404 },
	{ 0x58a5, 0x58ac, PDF_CMAP_TABLE, 2007 },
	{ 0x58ae, 0x58b1, PDF_CMAP_TABLE, 2015 },
	{ 0x58b3, 0x58b3, PDF_CMAP_SINGLE, 4388 },
	{ 0x58b5, 0x58b6, PDF_CMAP_TABLE, 2019 },
	{ 0x58ba, 0x58bf, PDF_CMAP_TABLE, 2021 },
	{ 0x58c1, 0x58c2, PDF_CMAP_TABLE, 2027 },
	{ 0x58c5, 0x58c9, PDF_CMAP_TABLE, 2029 },
	{ 0x58cb, 0x58cb, PDF_CMAP_SINGLE, 14857 },
	{ 0x58ce, 0x58d6, PDF_CMAP_TABLE, 2034 },
	{ 0x58d8, 0x58dd, PDF_CMAP_TABLE, 2043 },
	{ 0x58de, 0x58df, PDF_CMAP_RANGE, 5496 },
	{ 0x58e0, 0x58e0, PDF_CMAP_SINGLE, 16601 },
	{ 0x58e2, 0x58e4, PDF_CMAP_TABLE, 2049 },
	{ 0x58e7, 0x58e9, PDF_CMAP_TABLE, 2052 },
	{ 0x58eb, 0x58ec, PDF_CMAP_TABLE, 2055 },
	{ 0x58ef, 0x58f0, PDF_CMAP_TABLE, 2057 },
	{ 0x58f2, 0x58f4, PDF_CMAP_TABLE, 2059 },
	{ 0x58f9, 0x58fa, PDF_CMAP_RANGE, 3104 },
	{ 0x58fb, 0x58ff, PDF_CMAP_TABLE, 2062 },
	{ 0x5902, 0x5907, PDF_CMAP_TABLE, 2067 },
	{ 0x590a, 0x590a, PDF_CMAP_SINGLE, 548 },
	{ 0x590c, 0x590f, PDF_CMAP_TABLE, 2073 },
	{ 0x5911, 0x5912, PDF_CMAP_TABLE, 2077 },
	{ 0x5914, 0x5917, PDF_CMAP_TABLE, 2079 },
	{ 0x5919, 0x591a, PDF_CMAP_RANGE, 948 },
	{ 0x591c, 0x591d, PDF_CMAP_TABLE, 2083 },
	{ 0x591f, 0x5920, PDF_CMAP_TABLE, 2085 },
	{ 0x5922, 0x5922, PDF_CMAP_SINGLE, 4012 },
	{ 0x5924, 0x5925, PDF_CMAP_TABLE, 2087 },
	{ 0x5927, 0x5927, PDF_CMAP_SINGLE, 637 },
	{ 0x5929, 0x592f, PDF_CMAP_TABLE, 2089 },
	{ 0x5931, 0x5932, PDF_CMAP_TABLE, 2096 },
	{ 0x5934, 0x5934, PDF_CMAP_SINGLE, 17741 },
	{ 0x5937, 0x5938, PDF_CMAP_RANGE, 950 },
	{ 0x593c, 0x593c, PDF_CMAP_SINGLE, 6100 },
	{ 0x593e, 0x593e, PDF_CMAP_SINGLE, 1136 },
	{ 0x5940, 0x5940, PDF_CMAP_SINGLE, 6232 },
	{ 0x5944, 0x5945, PDF_CMAP_TABLE, 2098 },
	{ 0x5947, 0x5948, PDF_CMAP_RANGE, 1411 },
	{ 0x5949, 0x594a, PDF_CMAP_TABLE, 2100 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 2102 },
	{ 0x5953, 0x5955, PDF_CMAP_TABLE, 2106 },
	{ 0x5957, 0x5958, PDF_CMAP_RANGE, 2177 },
	{ 0x595a, 0x595a, PDF_CMAP_SINGLE, 2179 },
	{ 0x595c, 0x595c, PDF_CMAP_SINGLE, 7829 },
	{ 0x5960, 0x5962, PDF_CMAP_TABLE, 2109 },
	{ 0x5965, 0x5965, PDF_CMAP_SINGLE, 18514 },
	{ 0x5967, 0x5967, PDF_CMAP_SINGLE, 3571 },
	{ 0x5969, 0x596b, PDF_CMAP_TABLE, 2112 },
	{ 0x596d, 0x596e, PDF_CMAP_TABLE, 2115 },
	{ 0x5970, 0x5979, PDF_CMAP_TABLE, 2117 },
	{ 0x597b, 0x5985, PDF_CMAP_TABLE, 2127 },
	{ 0x5989, 0x598a, PDF_CMAP_TABLE, 2138 },
	{ 0x598d, 0x5990, PDF_CMAP_TABLE, 2140 },
	{ 0x5992, 0x5994, PDF_CMAP_TABLE, 2144 },
	{ 0x5996, 0x599a, PDF_CMAP_TABLE, 2147 },
	{ 0x599d, 0x59a8, PDF_CMAP_TABLE, 2152 },
	{ 0x59ac, 0x59ac, PDF_CMAP_SINGLE, 15745 },
	{ 0x59ae, 0x59c1, PDF_CMAP_TABLE, 2164 },
	{ 0x59c3, 0x59d4, PDF_CMAP_TABLE, 2184 },
	{ 0x59d6, 0x59d6, PDF_CMAP_SINGLE, 6480 },
	{ 0x59d8, 0x59de, PDF_CMAP_TABLE, 2202 },
	{ 0x59e0, 0x59e1, PDF_CMAP_TABLE, 2209 },
	{ 0x59e3, 0x59e6, PDF_CMAP_TABLE, 2211 },
	{ 0x59e8, 0x5a03, PDF_CMAP_TABLE, 2215 },
	{ 0x5a09, 0x5a0d, PDF_CMAP_TABLE, 2243 },
	{ 0x5a0f, 0x5a0f, PDF_CMAP_SINGLE, 7283 },
	{ 0x5a11, 0x5a13, PDF_CMAP_TABLE, 2248 },
	{ 0x5a15, 0x5a1c, PDF_CMAP_TABLE, 2251 },
	{ 0x5a1e, 0x5a21, PDF_CMAP_TABLE, 2259 },
	{ 0x5a23, 0x5a25, PDF_CMAP_TABLE, 2263 },
	{ 0x5a27, 0x5a27, PDF_CMAP_SINGLE, 15841 },
	{ 0x5a29, 0x5a2c, PDF_CMAP_TABLE, 2266 },
	{ 0x5a2d, 0x5a2e, PDF_CMAP_RANGE, 7280 },
	{ 0x5a33, 0x5a33, PDF_CMAP_SINGLE, 7287 },
	{ 0x5a35, 0x5a39, PDF_CMAP_TABLE, 2270 },
	{ 0x5a3c, 0x5a3e, PDF_CMAP_TABLE, 2275 },
	{ 0x5a40, 0x5a4a, PDF_CMAP_TABLE, 2278 },
	{ 0x5a4c, 0x5a4d, PDF_CMAP_TABLE, 2289 },
	{ 0x5a50, 0x5a6e, PDF_CMAP_TABLE, 2291 },
	{ 0x5a70, 0x5a71, PDF_CMAP_TABLE, 2322 },
	{ 0x5a77, 0x5a7f, PDF_CMAP_TABLE, 2324 },
	{ 0x5a81, 0x5a84, PDF_CMAP_TABLE, 2333 },
	{ 0x5a86, 0x5a86, PDF_CMAP_SINGLE, 15233 },
	{ 0x5a88, 0x5a88, PDF_CMAP_SINGLE, 16995 },
	{ 0x5a8a, 0x5a8c, PDF_CMAP_TABLE, 2337 },
	{ 0x5a8e, 0x5a97, PDF_CMAP_TABLE, 2340 },
	{ 0x5a99, 0x5aa2, PDF_CMAP_TABLE, 2350 },
	{ 0x5aa4, 0x5aa7, PDF_CMAP_TABLE, 2360 },
	{ 0x5aa9, 0x5aa9, PDF_CMAP_SINGLE, 8451 },
	{ 0x5aab, 0x5aac, PDF_CMAP_TABLE, 2364 },
	{ 0x5aae, 0x5ac4, PDF_CMAP_TABLE, 2366 },
	{ 0x5ac6, 0x5acf, PDF_CMAP_TABLE, 2389 },
	{ 0x5ad1, 0x5ad1, PDF_CMAP_SINGLE, 19014 },
	{ 0x5ad3, 0x5ad3, PDF_CMAP_SINGLE, 16642 },
	{ 0x5ad5, 0x5ae6, PDF_CMAP_TABLE, 2399 },
	{ 0x5ae8, 0x5aee, PDF_CMAP_TABLE, 2417 },
	{ 0x5af0, 0x5af0, PDF_CMAP_SINGLE, 16103 },
	{ 0x5af2, 0x5afb, PDF_CMAP_TABLE, 2424 },
	{ 0x5afd, 0x5aff, PDF_CMAP_TABLE, 2434 },
	{ 0x5b01, 0x5b03, PDF_CMAP_TABLE, 2437 },
	{ 0x5b05, 0x5b05, PDF_CMAP_SINGLE, 10421 },
	{ 0x5b07, 0x5b09, PDF_CMAP_TABLE, 2440 },
	{ 0x5b0b, 0x5b0d, PDF_CMAP_TABLE, 2443 },
	{ 0x5b0f, 0x5b11, PDF_CMAP_TABLE, 2446 },
	{ 0x5b13, 0x5b17, PDF_CMAP_TABLE, 2449 },
	{ 0x5b19, 0x5b1b, PDF_CMAP_TABLE, 2454 },
	{ 0x5b1d, 0x5b21, PDF_CMAP_TABLE, 2457 },
	{ 0x5b23, 0x5b28, PDF_CMAP_TABLE, 2462 },
	{ 0x5b2a, 0x5b30, PDF_CMAP_TABLE, 2468 },
	{ 0x5b32, 0x5b32, PDF_CMAP_SINGLE, 11601 },
	{ 0x5b34, 0x5b34, PDF_CMAP_SINGLE, 4777 },
	{ 0x5b38, 0x5b38, PDF_CMAP_SINGLE, 5326 },
	{ 0x5b3c, 0x5b3c, PDF_CMAP_SINGLE, 12056 },
	{ 0x5b3d, 0x5b3f, PDF_CMAP_RANGE, 12425 },
	{ 0x5b40, 0x5b41, PDF_CMAP_TABLE, 2475 },
	{ 0x5b43, 0x5b48, PDF_CMAP_TABLE, 2477 },
	{ 0x5b4a, 0x5b4a, PDF_CMAP_SINGLE, 14541 },
	{ 0x5b4b, 0x5b4c, PDF_CMAP_RANGE, 13180 },
	{ 0x5b4d, 0x5b4f, PDF_CMAP_TABLE, 2483 },
	{ 0x5b50, 0x5b51, PDF_CMAP_RANGE, 639 },
	{ 0x5b53, 0x5b56, PDF_CMAP_TABLE, 2486 },
	{ 0x5b57, 0x5b58, PDF_CMAP_RANGE, 959 },
	{ 0x5b5a, 0x5b5b, PDF_CMAP_RANGE, 1151 },
	{ 0x5b5c, 0x5b5d, PDF_CMAP_TABLE, 2490 },
	{ 0x5b5f, 0x5b5f, PDF_CMAP_SINGLE, 1431 },
	{ 0x5b62, 0x5b66, PDF_CMAP_TABLE, 2492 },
	{ 0x5b68, 0x5b69, PDF_CMAP_TABLE, 2497 },
	{ 0x5b6b, 0x5b6e, PDF_CMAP_TABLE, 2499 },
	{ 0x5b70, 0x5b78, PDF_CMAP_TABLE, 2503 },
	{ 0x5b7a, 0x5b7d, PDF_CMAP_TABLE, 2512 },
	{ 0x5b7f, 0x5b85, PDF_CMAP_TABLE, 2516 },
	{ 0x5b87, 0x5b88, PDF_CMAP_RANGE, 961 },
	{ 0x5b89, 0x5b89, PDF_CMAP_SINGLE, 964 },
	{ 0x5b8b, 0x5b8c, PDF_CMAP_TABLE, 2523 },
	{ 0x5b8e, 0x5b90, PDF_CMAP_TABLE, 2525 },
	{ 0x5b92, 0x5b93, PDF_CMAP_TABLE, 2528 },
	{ 0x5b95, 0x5b9d, PDF_CMAP_TABLE, 2530 },
	{ 0x5b9e, 0x5b9f, PDF_CMAP_RANGE, 17743 },
	{ 0x5ba2, 0x5ba8, PDF_CMAP_TABLE, 2539 },
	{ 0x5baa, 0x5baa, PDF_CMAP_SINGLE, 16763 },
	{ 0x5bac, 0x5bae, PDF_CMAP_TABLE, 2546 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 2195 },
	{ 0x5bb3, 0x5bb9, PDF_CMAP_TABLE, 2549 },
	{ 0x5bbf, 0x5bc7, PDF_CMAP_TABLE, 2556 },
	{ 0x5bca, 0x5bce, PDF_CMAP_TABLE, 2565 },
	{ 0x5bd0, 0x5bd7, PDF_CMAP_TABLE, 2570 },
	{ 0x5bd8, 0x5bd9, PDF_CMAP_RANGE, 9144 },
	{ 0x5bdb, 0x5bdb, PDF_CMAP_SINGLE, 19009 },
	{ 0x5bde, 0x5be4, PDF_CMAP_TABLE, 2578 },
	{ 0x5be5, 0x5be6, PDF_CMAP_RANGE, 4027 },
	{ 0x5be7, 0x5bec, PDF_CMAP_TABLE, 2585 },
	{ 0x5bee, 0x5bf0, PDF_CMAP_TABLE, 2591 },
	{ 0x5bf1, 0x5bf2, PDF_CMAP_RANGE, 11609 },
	{ 0x5bf3, 0x5bf3, PDF_CMAP_SINGLE, 16661 },
	{ 0x5bf5, 0x5bf6, PDF_CMAP_TABLE, 2594 },
	{ 0x5bf8, 0x5bf8, PDF_CMAP_SINGLE, 642 },
	{ 0x5bfa, 0x5bfa, PDF_CMAP_SINGLE, 965 },
	{ 0x5bff, 0x5bff, PDF_CMAP_SINGLE, 16605 },
	{ 0x5c01, 0x5c01, PDF_CMAP_SINGLE, 1798 },
	{ 0x5c03, 0x5c05, PDF_CMAP_TABLE, 2596 },
	{ 0x5c07, 0x5c09, PDF_CMAP_TABLE, 2599 },
	{ 0x5c0a, 0x5c0b, PDF_CMAP_RANGE, 3119 },
	{ 0x5c0c, 0x5c16, PDF_CMAP_TABLE, 2602 },
	{ 0x5c1a, 0x5c1a, PDF_CMAP_SINGLE, 1440 },
	{ 0x5c1c, 0x5c1c, PDF_CMAP_SINGLE, 17898 },
	{ 0x5c1e, 0x5c20, PDF_CMAP_TABLE, 2613 },
	{ 0x5c22, 0x5c25, PDF_CMAP_TABLE, 2616 },
	{ 0x5c28, 0x5c28, PDF_CMAP_SINGLE, 6245 },
	{ 0x5c2a, 0x5c2a, PDF_CMAP_SINGLE, 6246 },
	{ 0x5c2c, 0x5c2c, PDF_CMAP_SINGLE, 1156 },
	{ 0x5c30, 0x5c31, PDF_CMAP_TABLE, 2620 },
	{ 0x5c33, 0x5c33, PDF_CMAP_SINGLE, 9147 },
	{ 0x5c37, 0x5c3c, PDF_CMAP_TABLE, 2622 },
	{ 0x5c3e, 0x5c3f, PDF_CMAP_TABLE, 2628 },
	{ 0x5c40, 0x5c41, PDF_CMAP_RANGE, 1157 },
	{ 0x5c44, 0x5c44, PDF_CMAP_SINGLE, 6491 },
	{ 0x5c45, 0x5c46, PDF_CMAP_RANGE, 1442 },
	{ 0x5c47, 0x5c4d, PDF_CMAP_TABLE, 2630 },
	{ 0x5c4e, 0x5c4f, PDF_CMAP_RANGE, 1799 },
	{ 0x5c50, 0x5c51, PDF_CMAP_TABLE, 2637 },
	{ 0x5c53, 0x5c56, PDF_CMAP_TABLE, 2639 },
	{ 0x5c58, 0x5c59, PDF_CMAP_TABLE, 2643 },
	{ 0x5c5c, 0x5c5d, PDF_CMAP_RANGE, 2652 },
	{ 0x5c5e, 0x5c5e, PDF_CMAP_SINGLE, 15719 },
	{ 0x5c60, 0x5c60, PDF_CMAP_SINGLE, 2651 },
	{ 0x5c62, 0x5c63, PDF_CMAP_TABLE, 2645 },
	{ 0x5c64, 0x5c65, PDF_CMAP_RANGE, 4404 },
	{ 0x5c67, 0x5c68, PDF_CMAP_TABLE, 2647 },
	{ 0x5c69, 0x5c6a, PDF_CMAP_RANGE, 12057 },
	{ 0x5c6c, 0x5c6f, PDF_CMAP_TABLE, 2649 },
	{ 0x5c71, 0x5c71, PDF_CMAP_SINGLE, 646 },
	{ 0x5c73, 0x5c74, PDF_CMAP_TABLE, 2653 },
	{ 0x5c78, 0x5c79, PDF_CMAP_TABLE, 2655 },
	{ 0x5c7a, 0x5c7b, PDF_CMAP_RANGE, 6112 },
	{ 0x5c7c, 0x5c7c, PDF_CMAP_SINGLE, 6111 },
	{ 0x5c7e, 0x5c7e, PDF_CMAP_SINGLE, 6114 },
	{ 0x5c85, 0x5c86, PDF_CMAP_TABLE, 2657 },
	{ 0x5c88, 0x5c8d, PDF_CMAP_TABLE, 2659 },
	{ 0x5c8f, 0x5c8f, PDF_CMAP_SINGLE, 6248 },
	{ 0x5c90, 0x5c91, PDF_CMAP_RANGE, 1161 },
	{ 0x5c92, 0x5c95, PDF_CMAP_TABLE, 2665 },
	{ 0x5c99, 0x5c9a, PDF_CMAP_TABLE, 2669 },
	{ 0x5c9c, 0x5cb1, PDF_CMAP_TABLE, 2671 },
	{ 0x5cb3, 0x5cb3, PDF_CMAP_SINGLE, 1450 },
	{ 0x5cb5, 0x5cb8, PDF_CMAP_TABLE, 2693 },
	{ 0x5cba, 0x5cba, PDF_CMAP_SINGLE, 16683 },
	{ 0x5cc1, 0x5cc2, PDF_CMAP_TABLE, 2697 },
	{ 0x5cc6, 0x5ccc, PDF_CMAP_TABLE, 2699 },
	{ 0x5cce, 0x5cd2, PDF_CMAP_TABLE, 2706 },
	{ 0x5cd3, 0x5cd4, PDF_CMAP_RANGE, 6849 },
	{ 0x5cd5, 0x5cdb, PDF_CMAP_TABLE, 2711 },
	{ 0x5cde, 0x5cdf, PDF_CMAP_TABLE, 2718 },
	{ 0x5ce5, 0x5ce5, PDF_CMAP_SINGLE, 17975 },
	{ 0x5ce8, 0x5cea, PDF_CMAP_TABLE, 2720 },
	{ 0x5cec, 0x5cf1, PDF_CMAP_TABLE, 2723 },
	{ 0x5cf4, 0x5cf9, PDF_CMAP_TABLE, 2729 },
	{ 0x5cfb, 0x5cfd, PDF_CMAP_TABLE, 2735 },
	{ 0x5cff, 0x5d01, PDF_CMAP_TABLE, 2738 },
	{ 0x5d06, 0x5d07, PDF_CMAP_TABLE, 2741 },
	{ 0x5d0b, 0x5d12, PDF_CMAP_TABLE, 2743 },
	{ 0x5d14, 0x5d1b, PDF_CMAP_TABLE, 2751 },
	{ 0x5d1d, 0x5d20, PDF_CMAP_TABLE, 2759 },
	{ 0x5d22, 0x5d29, PDF_CMAP_TABLE, 2763 },
	{ 0x5d2c, 0x5d2c, PDF_CMAP_SINGLE, 15863 },
	{ 0x5d2e, 0x5d3a, PDF_CMAP_TABLE, 2771 },
	{ 0x5d3c, 0x5d43, PDF_CMAP_TABLE, 2784 },
	{ 0x5d45, 0x5d4c, PDF_CMAP_TABLE, 2792 },
	{ 0x5d4e, 0x5d4e, PDF_CMAP_SINGLE, 8476 },
	{ 0x5d50, 0x5d52, PDF_CMAP_TABLE, 2800 },
	{ 0x5d55, 0x5d57, PDF_CMAP_TABLE, 2803 },
	{ 0x5d59, 0x5d59, PDF_CMAP_SINGLE, 8483 },
	{ 0x5d5b, 0x5d5b, PDF_CMAP_SINGLE, 15829 },
	{ 0x5d5e, 0x5d5e, PDF_CMAP_SINGLE, 9154 },
	{ 0x5d62, 0x5d63, PDF_CMAP_TABLE, 2806 },
	{ 0x5d65, 0x5d65, PDF_CMAP_SINGLE, 9151 },
	{ 0x5d67, 0x5d69, PDF_CMAP_TABLE, 2808 },
	{ 0x5d6b, 0x5d6c, PDF_CMAP_TABLE, 2811 },
	{ 0x5d6f, 0x5d72, PDF_CMAP_TABLE, 2813 },
	{ 0x5d74, 0x5d74, PDF_CMAP_SINGLE, 16111 },
	{ 0x5d77, 0x5d82, PDF_CMAP_TABLE, 2817 },
	{ 0x5d84, 0x5d8b, PDF_CMAP_TABLE, 2829 },
	{ 0x5d8d, 0x5d8e, PDF_CMAP_TABLE, 2837 },
	{ 0x5d92, 0x5d95, PDF_CMAP_TABLE, 2839 },
	{ 0x5d97, 0x5d97, PDF_CMAP_SINGLE, 10425 },
	{ 0x5d99, 0x5d9a, PDF_CMAP_TABLE, 2843 },
	{ 0x5d9c, 0x5da2, PDF_CMAP_TABLE, 2845 },
	{ 0x5da4, 0x5da4, PDF_CMAP_SINGLE, 17965 },
	{ 0x5da7, 0x5db2, PDF_CMAP_TABLE, 2852 },
	{ 0x5db4, 0x5dba, PDF_CMAP_TABLE, 2864 },
	{ 0x5dbc, 0x5dbd, PDF_CMAP_TABLE, 2871 },
	{ 0x5dc0, 0x5dc3, PDF_CMAP_TABLE, 2873 },
	{ 0x5dc6, 0x5dc7, PDF_CMAP_RANGE, 12749 },
	{ 0x5dc9, 0x5dc9, PDF_CMAP_SINGLE, 5651 },
	{ 0x5dcb, 0x5dcb, PDF_CMAP_SINGLE, 13006 },
	{ 0x5dcd, 0x5dcd, PDF_CMAP_SINGLE, 5743 },
	{ 0x5dcf, 0x5dcf, PDF_CMAP_SINGLE, 13007 },
	{ 0x5dd1, 0x5dd2, PDF_CMAP_TABLE, 2877 },
	{ 0x5dd4, 0x5dd8, PDF_CMAP_TABLE, 2879 },
	{ 0x5ddb, 0x5ddb, PDF_CMAP_SINGLE, 550 },
	{ 0x5ddd, 0x5de2, PDF_CMAP_TABLE, 2884 },
	{ 0x5de5, 0x5de8, PDF_CMAP_TABLE, 2890 },
	{ 0x5deb, 0x5deb, PDF_CMAP_SINGLE, 1165 },
	{ 0x5dee, 0x5dee, PDF_CMAP_SINGLE, 2216 },
	{ 0x5df0, 0x5df0, PDF_CMAP_SINGLE, 9158 },
	{ 0x5df1, 0x5df3, PDF_CMAP_RANGE, 649 },
	{ 0x5df4, 0x5df5, PDF_CMAP_TABLE, 2894 },
	{ 0x5df7, 0x5df7, PDF_CMAP_SINGLE, 1805 },
	{ 0x5df9, 0x5df9, PDF_CMAP_SINGLE, 6857 },
	{ 0x5dfd, 0x5dff, PDF_CMAP_TABLE, 2896 },
	{ 0x5e02, 0x5e03, PDF_CMAP_RANGE, 825 },
	{ 0x5e04, 0x5e04, PDF_CMAP_SINGLE, 6052 },
	{ 0x5e06, 0x5e06, PDF_CMAP_SINGLE, 969 },
	{ 0x5e09, 0x5e0c, PDF_CMAP_TABLE, 2899 },
	{ 0x5e0e, 0x5e0e, PDF_CMAP_SINGLE, 6259 },
	{ 0x5e11, 0x5e12, PDF_CMAP_TABLE, 2903 },
	{ 0x5e14, 0x5e1b, PDF_CMAP_TABLE, 2905 },
	{ 0x5e1d, 0x5e1d, PDF_CMAP_SINGLE, 1806 },
	{ 0x5e1f, 0x5e20, PDF_CMAP_TABLE, 2913 },
	{ 0x5e21, 0x5e23, PDF_CMAP_RANGE, 6858 },
	{ 0x5e24, 0x5e25, PDF_CMAP_TABLE, 2915 },
	{ 0x5e28, 0x5e29, PDF_CMAP_TABLE, 2917 },
	{ 0x5e2b, 0x5e2b, PDF_CMAP_SINGLE, 2218 },
	{ 0x5e2d, 0x5e2e, PDF_CMAP_TABLE, 2919 },
	{ 0x5e33, 0x5e34, PDF_CMAP_TABLE, 2921 },
	{ 0x5e36, 0x5e38, PDF_CMAP_TABLE, 2923 },
	{ 0x5e3d, 0x5e3e, PDF_CMAP_TABLE, 2926 },
	{ 0x5e40, 0x5e45, PDF_CMAP_TABLE, 2928 },
	{ 0x5e48, 0x5e48, PDF_CMAP_SINGLE, 14892 },
	{ 0x5e4a, 0x5e4f, PDF_CMAP_TABLE, 2934 },
	{ 0x5e53, 0x5e55, PDF_CMAP_TABLE, 2940 },
	{ 0x5e57, 0x5e57, PDF_CMAP_SINGLE, 4040 },
	{ 0x5e58, 0x5e59, PDF_CMAP_RANGE, 9842 },
	{ 0x5e5b, 0x5e63, PDF_CMAP_TABLE, 2943 },
	{ 0x5e66, 0x5e66, PDF_CMAP_SINGLE, 11064 },
	{ 0x5e67, 0x5e68, PDF_CMAP_RANGE, 11062 },
	{ 0x5e69, 0x5e6c, PDF_CMAP_TABLE, 2952 },
	{ 0x5e6d, 0x5e6e, PDF_CMAP_RANGE, 12060 },
	{ 0x5e6f, 0x5e70, PDF_CMAP_TABLE, 2956 },
	{ 0x5e72, 0x5e76, PDF_CMAP_TABLE, 2958 },
	{ 0x5e78, 0x5e80, PDF_CMAP_TABLE, 2963 },
	{ 0x5e82, 0x5e84, PDF_CMAP_TABLE, 2972 },
	{ 0x5e86, 0x5e8d, PDF_CMAP_TABLE, 2975 },
	{ 0x5e8f, 0x5e8f, PDF_CMAP_SINGLE, 1167 },
	{ 0x5e92, 0x5e92, PDF_CMAP_SINGLE, 18906 },
	{ 0x5e95, 0x5e96, PDF_CMAP_RANGE, 1461 },
	{ 0x5e97, 0x5e97, PDF_CMAP_SINGLE, 1459 },
	{ 0x5e99, 0x5e9c, PDF_CMAP_TABLE, 2983 },
	{ 0x5ea0, 0x5ea0, PDF_CMAP_SINGLE, 1810 },
	{ 0x5ea2, 0x5ea8, PDF_CMAP_TABLE, 2987 },
	{ 0x5eaa, 0x5eae, PDF_CMAP_TABLE, 2994 },
	{ 0x5eb0, 0x5eb1, PDF_CMAP_TABLE, 2999 },
	{ 0x5eb2, 0x5eb3, PDF_CMAP_RANGE, 7889 },
	{ 0x5eb4, 0x5eb6, PDF_CMAP_TABLE, 3001 },
	{ 0x5eb7, 0x5eb8, PDF_CMAP_RANGE, 2672 },
	{ 0x5eb9, 0x5eb9, PDF_CMAP_SINGLE, 7888 },
	{ 0x5ebd, 0x5ebe, PDF_CMAP_TABLE, 3004 },
	{ 0x5ec1, 0x5ec2, PDF_CMAP_RANGE, 3133 },
	{ 0x5ec4, 0x5ece, PDF_CMAP_TABLE, 3006 },
	{ 0x5ed0, 0x5ee3, PDF_CMAP_TABLE, 3017 },
	{ 0x5ee5, 0x5ee9, PDF_CMAP_TABLE, 3037 },
	{ 0x5eec, 0x5eec, PDF_CMAP_SINGLE, 5501 },
	{ 0x5eee, 0x5eef, PDF_CMAP_RANGE, 12751 },
	{ 0x5ef1, 0x5ef4, PDF_CMAP_TABLE, 3042 },
	{ 0x5ef6, 0x5efc, PDF_CMAP_TABLE, 3046 },
	{ 0x5efe, 0x5eff, PDF_CMAP_TABLE, 3053 },
	{ 0x5f01, 0x5f02, PDF_CMAP_TABLE, 3055 },
	{ 0x5f04, 0x5f05, PDF_CMAP_TABLE, 3057 },
	{ 0x5f07, 0x5f08, PDF_CMAP_TABLE, 3059 },
	{ 0x5f0a, 0x5f0f, PDF_CMAP_TABLE, 3061 },
	{ 0x5f12, 0x5f13, PDF_CMAP_TABLE, 3067 },
	{ 0x5f14, 0x5f15, PDF_CMAP_RANGE, 718 },
	{ 0x5f17, 0x5f18, PDF_CMAP_TABLE, 3069 },
	{ 0x5f1a, 0x5f1b, PDF_CMAP_TABLE, 3071 },
	{ 0x5f1d, 0x5f1d, PDF_CMAP_SINGLE, 6266 },
	{ 0x5f1f, 0x5f1f, PDF_CMAP_SINGLE, 1172 },
	{ 0x5f22, 0x5f24, PDF_CMAP_RANGE, 6515 },
	{ 0x5f25, 0x5f25, PDF_CMAP_SINGLE, 17968 },
	{ 0x5f26, 0x5f27, PDF_CMAP_RANGE, 1464 },
	{ 0x5f28, 0x5f29, PDF_CMAP_TABLE, 3073 },
	{ 0x5f2d, 0x5f2e, PDF_CMAP_TABLE, 3075 },
	{ 0x5f30, 0x5f31, PDF_CMAP_TABLE, 3077 },
	{ 0x5f33, 0x5f33, PDF_CMAP_SINGLE, 7308 },
	{ 0x5f35, 0x5f38, PDF_CMAP_TABLE, 3079 },
	{ 0x5f3a, 0x5f3c, PDF_CMAP_TABLE, 3083 },
	{ 0x5f40, 0x5f40, PDF_CMAP_SINGLE, 9169 },
	{ 0x5f43, 0x5f44, PDF_CMAP_TABLE, 3086 },
	{ 0x5f46, 0x5f46, PDF_CMAP_SINGLE, 4045 },
	{ 0x5f48, 0x5f51, PDF_CMAP_TABLE, 3088 },
	{ 0x5f54, 0x5f54, PDF_CMAP_SINGLE, 6518 },
	{ 0x5f56, 0x5f59, PDF_CMAP_TABLE, 3098 },
	{ 0x5f5c, 0x5f5d, PDF_CMAP_TABLE, 3102 },
	{ 0x5f61, 0x5f65, PDF_CMAP_TABLE, 3104 },
	{ 0x5f67, 0x5f67, PDF_CMAP_SINGLE, 7310 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 3109 },
	{ 0x5f6f, 0x5f74, PDF_CMAP_TABLE, 3114 },
	{ 0x5f76, 0x5f79, PDF_CMAP_TABLE, 3120 },
	{ 0x5f7b, 0x5f7f, PDF_CMAP_TABLE, 3124 },
	{ 0x5f80, 0x5f81, PDF_CMAP_RANGE, 1467 },
	{ 0x5f82, 0x5f83, PDF_CMAP_TABLE, 3129 },
	{ 0x5f85, 0x5f89, PDF_CMAP_TABLE, 3131 },
	{ 0x5f8a, 0x5f8b, PDF_CMAP_RANGE, 1818 },
	{ 0x5f8c, 0x5f8c, PDF_CMAP_SINGLE, 1821 },
	{ 0x5f90, 0x5f92, PDF_CMAP_TABLE, 3136 },
	{ 0x5f96, 0x5f99, PDF_CMAP_TABLE, 3139 },
	{ 0x5f9b, 0x5f9c, PDF_CMAP_TABLE, 3143 },
	{ 0x5f9e, 0x5fa1, PDF_CMAP_TABLE, 3145 },
	{ 0x5fa4, 0x5fa8, PDF_CMAP_TABLE, 3149 },
	{ 0x5fa9, 0x5faa, PDF_CMAP_RANGE, 3138 },
	{ 0x5fab, 0x5faf, PDF_CMAP_TABLE, 3154 },
	{ 0x5fb1, 0x5fb2, PDF_CMAP_TABLE, 3159 },
	{ 0x5fb5, 0x5fb7, PDF_CMAP_TABLE, 3161 },
	{ 0x5fb9, 0x5fbf, PDF_CMAP_TABLE, 3164 },
	{ 0x5fc0, 0x5fc1, PDF_CMAP_RANGE, 12753 },
	{ 0x5fc2, 0x5fc5, PDF_CMAP_TABLE, 3171 },
	{ 0x5fc9, 0x5fc9, PDF_CMAP_SINGLE, 6055 },
	{ 0x5fcc, 0x5fcd, PDF_CMAP_TABLE, 3175 },
	{ 0x5fcf, 0x5fd2, PDF_CMAP_TABLE, 3177 },
	{ 0x5fd4, 0x5fd9, PDF_CMAP_TABLE, 3181 },
	{ 0x5fdb, 0x5fdb, PDF_CMAP_SINGLE, 14922 },
	{ 0x5fdd, 0x5fe1, PDF_CMAP_TABLE, 3187 },
	{ 0x5fe3, 0x5fe5, PDF_CMAP_TABLE, 3192 },
	{ 0x5fe8, 0x5fe8, PDF_CMAP_SINGLE, 6273 },
	{ 0x5fea, 0x5feb, PDF_CMAP_TABLE, 3195 },
	{ 0x5fed, 0x5fef, PDF_CMAP_TABLE, 3197 },
	{ 0x5ff1, 0x5ff1, PDF_CMAP_SINGLE, 1181 },
	{ 0x5ff3, 0x5ff5, PDF_CMAP_TABLE, 3200 },
	{ 0x5ff7, 0x5ff8, PDF_CMAP_TABLE, 3203 },
	{ 0x5ffa, 0x5ffb, PDF_CMAP_TABLE, 3205 },
	{ 0x5ffd, 0x5ffd, PDF_CMAP_SINGLE, 1473 },
	{ 0x5fff, 0x6000, PDF_CMAP_TABLE, 3207 },
	{ 0x6009, 0x6017, PDF_CMAP_TABLE, 3209 },
	{ 0x6019, 0x601e, PDF_CMAP_TABLE, 3224 },
	{ 0x6020, 0x602f, PDF_CMAP_TABLE, 3230 },
	{ 0x6031, 0x6035, PDF_CMAP_TABLE, 3246 },
	{ 0x6037, 0x6037, PDF_CMAP_SINGLE, 6873 },
	{ 0x6039, 0x6039, PDF_CMAP_SINGLE, 6874 },
	{ 0x603b, 0x603b, PDF_CMAP_SINGLE, 17747 },
	{ 0x6040, 0x6047, PDF_CMAP_TABLE, 3251 },
	{ 0x6049, 0x604a, PDF_CMAP_TABLE, 3259 },
	{ 0x604c, 0x604d, PDF_CMAP_TABLE, 3261 },
	{ 0x6050, 0x6050, PDF_CMAP_SINGLE, 2229 },
	{ 0x6052, 0x6055, PDF_CMAP_TABLE, 3263 },
	{ 0x6058, 0x605b, PDF_CMAP_TABLE, 3267 },
	{ 0x605d, 0x605f, PDF_CMAP_TABLE, 3271 },
	{ 0x6062, 0x6070, PDF_CMAP_TABLE, 3274 },
	{ 0x6072, 0x6072, PDF_CMAP_SINGLE, 6876 },
	{ 0x6075, 0x6075, PDF_CMAP_SINGLE, 14934 },
	{ 0x6077, 0x6077, PDF_CMAP_SINGLE, 16389 },
	{ 0x607e, 0x6081, PDF_CMAP_TABLE, 3289 },
	{ 0x6083, 0x608a, PDF_CMAP_TABLE, 3293 },
	{ 0x608c, 0x608e, PDF_CMAP_TABLE, 3301 },
	{ 0x6090, 0x6090, PDF_CMAP_SINGLE, 7897 },
	{ 0x6092, 0x6092, PDF_CMAP_SINGLE, 7318 },
	{ 0x6094, 0x6097, PDF_CMAP_TABLE, 3304 },
	{ 0x609a, 0x60a0, PDF_CMAP_TABLE, 3308 },
	{ 0x60a2, 0x60a4, PDF_CMAP_TABLE, 3315 },
	{ 0x60a7, 0x60a8, PDF_CMAP_TABLE, 3318 },
	{ 0x60b0, 0x60c1, PDF_CMAP_TABLE, 3320 },
	{ 0x60c3, 0x60cf, PDF_CMAP_TABLE, 3338 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 3141 },
	{ 0x60d3, 0x60d4, PDF_CMAP_RANGE, 7902 },
	{ 0x60d5, 0x60d5, PDF_CMAP_SINGLE, 2705 },
	{ 0x60d7, 0x60e4, PDF_CMAP_TABLE, 3351 },
	{ 0x60e6, 0x60ea, PDF_CMAP_TABLE, 3365 },
	{ 0x60f0, 0x6101, PDF_CMAP_TABLE, 3370 },
	{ 0x6103, 0x6110, PDF_CMAP_TABLE, 3388 },
	{ 0x6112, 0x6116, PDF_CMAP_TABLE, 3402 },
	{ 0x6118, 0x611d, PDF_CMAP_TABLE, 3407 },
	{ 0x611f, 0x611f, PDF_CMAP_SINGLE, 3594 },
	{ 0x6122, 0x6123, PDF_CMAP_TABLE, 3413 },
	{ 0x6127, 0x6129, PDF_CMAP_TABLE, 3415 },
	{ 0x612b, 0x612c, PDF_CMAP_TABLE, 3418 },
	{ 0x612e, 0x6130, PDF_CMAP_TABLE, 3420 },
	{ 0x6132, 0x6132, PDF_CMAP_SINGLE, 9178 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 3605 },
	{ 0x6136, 0x6137, PDF_CMAP_TABLE, 3423 },
	{ 0x613b, 0x613b, PDF_CMAP_SINGLE, 9874 },
	{ 0x613d, 0x6142, PDF_CMAP_TABLE, 3425 },
	{ 0x6144, 0x6148, PDF_CMAP_TABLE, 3431 },
	{ 0x6149, 0x614a, PDF_CMAP_RANGE, 9173 },
	{ 0x614b, 0x6150, PDF_CMAP_TABLE, 3436 },
	{ 0x6152, 0x6153, PDF_CMAP_RANGE, 9864 },
	{ 0x6154, 0x6156, PDF_CMAP_TABLE, 3442 },
	{ 0x6158, 0x6161, PDF_CMAP_TABLE, 3445 },
	{ 0x6162, 0x6163, PDF_CMAP_RANGE, 4052 },
	{ 0x6164, 0x6168, PDF_CMAP_TABLE, 3455 },
	{ 0x616a, 0x616c, PDF_CMAP_TABLE, 3460 },
	{ 0x616e, 0x6177, PDF_CMAP_TABLE, 3463 },
	{ 0x6179, 0x617a, PDF_CMAP_TABLE, 3473 },
	{ 0x617c, 0x617e, PDF_CMAP_TABLE, 3475 },
	{ 0x6180, 0x6183, PDF_CMAP_TABLE, 3478 },
	{ 0x6187, 0x6187, PDF_CMAP_SINGLE, 16762 },
	{ 0x6189, 0x618e, PDF_CMAP_TABLE, 3482 },
	{ 0x6190, 0x6196, PDF_CMAP_TABLE, 3488 },
	{ 0x6198, 0x619d, PDF_CMAP_TABLE, 3495 },
	{ 0x619f, 0x619f, PDF_CMAP_SINGLE, 10457 },
	{ 0x61a1, 0x61a2, PDF_CMAP_TABLE, 3501 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 4437 },
	{ 0x61a7, 0x61ba, PDF_CMAP_TABLE, 3503 },
	{ 0x61bc, 0x61bc, PDF_CMAP_SINGLE, 11617 },
	{ 0x61be, 0x61c3, PDF_CMAP_TABLE, 3523 },
	{ 0x61c5, 0x61cd, PDF_CMAP_TABLE, 3529 },
	{ 0x61cf, 0x61d0, PDF_CMAP_TABLE, 3538 },
	{ 0x61d3, 0x61d3, PDF_CMAP_SINGLE, 16766 },
	{ 0x61d6, 0x61d6, PDF_CMAP_SINGLE, 12070 },
	{ 0x61d8, 0x61d8, PDF_CMAP_SINGLE, 12062 },
	{ 0x61da, 0x61da, PDF_CMAP_SINGLE, 14513 },
	{ 0x61de, 0x61e0, PDF_CMAP_TABLE, 3540 },
	{ 0x61e2, 0x61eb, PDF_CMAP_TABLE, 3543 },
	{ 0x61ed, 0x61ee, PDF_CMAP_RANGE, 12064 },
	{ 0x61f0, 0x61f2, PDF_CMAP_TABLE, 3553 },
	{ 0x61f5, 0x6201, PDF_CMAP_TABLE, 3556 },
	{ 0x6203, 0x6204, PDF_CMAP_RANGE, 13336 },
	{ 0x6207, 0x620a, PDF_CMAP_TABLE, 3569 },
	{ 0x620c, 0x620d, PDF_CMAP_RANGE, 977 },
	{ 0x620e, 0x620e, PDF_CMAP_SINGLE, 976 },
	{ 0x6210, 0x6212, PDF_CMAP_TABLE, 3573 },
	{ 0x6214, 0x6216, PDF_CMAP_TABLE, 3576 },
	{ 0x6219, 0x6219, PDF_CMAP_SINGLE, 7328 },
	{ 0x621a, 0x621b, PDF_CMAP_RANGE, 2711 },
	{ 0x621f, 0x6220, PDF_CMAP_TABLE, 3579 },
	{ 0x6221, 0x6222, PDF_CMAP_RANGE, 3610 },
	{ 0x6223, 0x6225, PDF_CMAP_TABLE, 3581 },
	{ 0x6227, 0x6227, PDF_CMAP_SINGLE, 9879 },
	{ 0x6229, 0x622e, PDF_CMAP_TABLE, 3584 },
	{ 0x6230, 0x6230, PDF_CMAP_SINGLE, 4791 },
	{ 0x6232, 0x6234, PDF_CMAP_TABLE, 3590 },
	{ 0x6236, 0x6237, PDF_CMAP_TABLE, 3593 },
	{ 0x6239, 0x623a, PDF_CMAP_TABLE, 3595 },
	{ 0x623d, 0x6241, PDF_CMAP_TABLE, 3597 },
	{ 0x6242, 0x6243, PDF_CMAP_RANGE, 6892 },
	{ 0x6246, 0x624e, PDF_CMAP_TABLE, 3602 },
	{ 0x6250, 0x6252, PDF_CMAP_TABLE, 3611 },
	{ 0x6253, 0x6254, PDF_CMAP_RANGE, 834 },
	{ 0x6258, 0x625c, PDF_CMAP_TABLE, 3614 },
	{ 0x625e, 0x625e, PDF_CMAP_SINGLE, 6125 },
	{ 0x6260, 0x6266, PDF_CMAP_TABLE, 3619 },
	{ 0x6268, 0x6268, PDF_CMAP_SINGLE, 16149 },
	{ 0x626d, 0x6274, PDF_CMAP_TABLE, 3626 },
	{ 0x6276, 0x6277, PDF_CMAP_TABLE, 3634 },
	{ 0x6279, 0x628a, PDF_CMAP_TABLE, 3636 },
	{ 0x628c, 0x628c, PDF_CMAP_SINGLE, 6287 },
	{ 0x628e, 0x628f, PDF_CMAP_RANGE, 6288 },
	{ 0x6290, 0x6298, PDF_CMAP_TABLE, 3654 },
	{ 0x629d, 0x629d, PDF_CMAP_SINGLE, 16022 },
	{ 0x62a4, 0x62a4, PDF_CMAP_SINGLE, 14953 },
	{ 0x62a6, 0x62a6, PDF_CMAP_SINGLE, 16321 },
	{ 0x62a8, 0x62b1, PDF_CMAP_TABLE, 3663 },
	{ 0x62b3, 0x62b6, PDF_CMAP_TABLE, 3673 },
	{ 0x62b8, 0x62b9, PDF_CMAP_TABLE, 3677 },
	{ 0x62bb, 0x62bf, PDF_CMAP_TABLE, 3679 },
	{ 0x62c1, 0x62d2, PDF_CMAP_TABLE, 3684 },
	{ 0x62d3, 0x62d4, PDF_CMAP_RANGE, 1503 },
	{ 0x62d5, 0x62d5, PDF_CMAP_SINGLE, 16780 },
	{ 0x62d6, 0x62d7, PDF_CMAP_RANGE, 1518 },
	{ 0x62d8, 0x62dc, PDF_CMAP_TABLE, 3702 },
	{ 0x62df, 0x62df, PDF_CMAP_SINGLE, 16412 },
	{ 0x62e5, 0x62e5, PDF_CMAP_SINGLE, 17981 },
	{ 0x62eb, 0x6303, PDF_CMAP_TABLE, 3707 },
	{ 0x6307, 0x6309, PDF_CMAP_TABLE, 3732 },
	{ 0x630b, 0x6311, PDF_CMAP_TABLE, 3735 },
	{ 0x6313, 0x6314, PDF_CMAP_RANGE, 6907 },
	{ 0x6315, 0x6316, PDF_CMAP_TABLE, 3742 },
	{ 0x6318, 0x6318, PDF_CMAP_SINGLE, 17345 },
	{ 0x6328, 0x6329, PDF_CMAP_TABLE, 3744 },
	{ 0x632a, 0x632b, PDF_CMAP_RANGE, 2257 },
	{ 0x632c, 0x632f, PDF_CMAP_TABLE, 3746 },
	{ 0x6331, 0x633e, PDF_CMAP_TABLE, 3750 },
	{ 0x6340, 0x6343, PDF_CMAP_TABLE, 3764 },
	{ 0x6344, 0x6345, PDF_CMAP_RANGE, 7334 },
	{ 0x6346, 0x6351, PDF_CMAP_TABLE, 3768 },
	{ 0x6354, 0x635a, PDF_CMAP_TABLE, 3780 },
	{ 0x6364, 0x6365, PDF_CMAP_TABLE, 3787 },
	{ 0x6367, 0x6369, PDF_CMAP_TABLE, 3789 },
	{ 0x636b, 0x6372, PDF_CMAP_TABLE, 3792 },
	{ 0x6375, 0x637d, PDF_CMAP_TABLE, 3800 },
	{ 0x637f, 0x6385, PDF_CMAP_TABLE, 3809 },
	{ 0x6387, 0x6392, PDF_CMAP_TABLE, 3816 },
	{ 0x6394, 0x6394, PDF_CMAP_SINGLE, 8524 },
	{ 0x6396, 0x6399, PDF_CMAP_TABLE, 3828 },
	{ 0x639b, 0x63a5, PDF_CMAP_TABLE, 3832 },
	{ 0x63a7, 0x63b1, PDF_CMAP_TABLE, 3843 },
	{ 0x63b9, 0x63b9, PDF_CMAP_SINGLE, 16029 },
	{ 0x63bd, 0x63be, PDF_CMAP_TABLE, 3854 },
	{ 0x63c0, 0x63d3, PDF_CMAP_TABLE, 3856 },
	{ 0x63d5, 0x63eb, PDF_CMAP_TABLE, 3876 },
	{ 0x63ed, 0x63ee, PDF_CMAP_RANGE, 3175 },
	{ 0x63ef, 0x63f6, PDF_CMAP_TABLE, 3899 },
	{ 0x63f8, 0x63f9, PDF_CMAP_TABLE, 3907 },
	{ 0x63fb, 0x63fc, PDF_CMAP_TABLE, 3909 },
	{ 0x63fe, 0x63fe, PDF_CMAP_SINGLE, 16030 },
	{ 0x6406, 0x6407, PDF_CMAP_TABLE, 3911 },
	{ 0x6409, 0x6410, PDF_CMAP_TABLE, 3913 },
	{ 0x6412, 0x6415, PDF_CMAP_TABLE, 3921 },
	{ 0x6416, 0x6417, PDF_CMAP_RANGE, 3624 },
	{ 0x6418, 0x6418, PDF_CMAP_SINGLE, 9202 },
	{ 0x641a, 0x641c, PDF_CMAP_TABLE, 3925 },
	{ 0x641e, 0x6421, PDF_CMAP_TABLE, 3928 },
	{ 0x6422, 0x6423, PDF_CMAP_RANGE, 9205 },
	{ 0x6424, 0x6428, PDF_CMAP_TABLE, 3932 },
	{ 0x642a, 0x6430, PDF_CMAP_TABLE, 3937 },
	{ 0x6432, 0x643b, PDF_CMAP_TABLE, 3944 },
	{ 0x643d, 0x6441, PDF_CMAP_TABLE, 3954 },
	{ 0x6443, 0x6443, PDF_CMAP_SINGLE, 9199 },
	{ 0x644b, 0x644b, PDF_CMAP_SINGLE, 9897 },
	{ 0x644d, 0x644e, PDF_CMAP_TABLE, 3959 },
	{ 0x6450, 0x6454, PDF_CMAP_TABLE, 3961 },
	{ 0x6458, 0x6461, PDF_CMAP_TABLE, 3966 },
	{ 0x6465, 0x6469, PDF_CMAP_TABLE, 3976 },
	{ 0x646b, 0x6471, PDF_CMAP_TABLE, 3981 },
	{ 0x6472, 0x6473, PDF_CMAP_RANGE, 9887 },
	{ 0x6474, 0x647d, PDF_CMAP_TABLE, 3988 },
	{ 0x647f, 0x647f, PDF_CMAP_SINGLE, 9901 },
	{ 0x6482, 0x6482, PDF_CMAP_SINGLE, 9894 },
	{ 0x6485, 0x6485, PDF_CMAP_SINGLE, 10469 },
	{ 0x6487, 0x648d, PDF_CMAP_TABLE, 3998 },
	{ 0x648f, 0x6493, PDF_CMAP_TABLE, 4005 },
	{ 0x6495, 0x649a, PDF_CMAP_TABLE, 4010 },
	{ 0x649c, 0x64a0, PDF_CMAP_TABLE, 4016 },
	{ 0x64a2, 0x64a6, PDF_CMAP_TABLE, 4021 },
	{ 0x64a9, 0x64a9, PDF_CMAP_SINGLE, 4452 },
	{ 0x64ab, 0x64b4, PDF_CMAP_TABLE, 4026 },
	{ 0x64b6, 0x64b6, PDF_CMAP_SINGLE, 14948 },
	{ 0x64bb, 0x64bc, PDF_CMAP_RANGE, 4795 },
	{ 0x64bd, 0x64c5, PDF_CMAP_TABLE, 4036 },
	{ 0x64c7, 0x64c7, PDF_CMAP_SINGLE, 4799 },
	{ 0x64c9, 0x64cb, PDF_CMAP_TABLE, 4045 },
	{ 0x64cd, 0x64d0, PDF_CMAP_TABLE, 4048 },
	{ 0x64d2, 0x64d4, PDF_CMAP_TABLE, 4052 },
	{ 0x64d6, 0x64db, PDF_CMAP_TABLE, 4055 },
	{ 0x64dd, 0x64dd, PDF_CMAP_SINGLE, 17182 },
	{ 0x64e0, 0x64ed, PDF_CMAP_TABLE, 4061 },
	{ 0x64ef, 0x64f4, PDF_CMAP_TABLE, 4075 },
	{ 0x64f7, 0x64f8, PDF_CMAP_TABLE, 4081 },
	{ 0x64fa, 0x64fb, PDF_CMAP_RANGE, 5334 },
	{ 0x64fc, 0x6501, PDF_CMAP_TABLE, 4083 },
	{ 0x6503, 0x6504, PDF_CMAP_TABLE, 4089 },
	{ 0x6506, 0x6507, PDF_CMAP_TABLE, 4091 },
	{ 0x6509, 0x650a, PDF_CMAP_TABLE, 4093 },
	{ 0x650c, 0x6511, PDF_CMAP_TABLE, 4095 },
	{ 0x6513, 0x6519, PDF_CMAP_TABLE, 4101 },
	{ 0x651b, 0x6526, PDF_CMAP_TABLE, 4108 },
	{ 0x6529, 0x6530, PDF_CMAP_TABLE, 4120 },
	{ 0x6532, 0x6539, PDF_CMAP_TABLE, 4128 },
	{ 0x653b, 0x653b, PDF_CMAP_SINGLE, 1208 },
	{ 0x653d, 0x653f, PDF_CMAP_TABLE, 4136 },
	{ 0x6541, 0x6541, PDF_CMAP_SINGLE, 6913 },
	{ 0x6543, 0x6543, PDF_CMAP_SINGLE, 6914 },
	{ 0x6545, 0x6546, PDF_CMAP_TABLE, 4139 },
	{ 0x6548, 0x6549, PDF_CMAP_RANGE, 2262 },
	{ 0x654a, 0x654a, PDF_CMAP_SINGLE, 7358 },
	{ 0x654d, 0x654d, PDF_CMAP_SINGLE, 16799 },
	{ 0x654f, 0x654f, PDF_CMAP_SINGLE, 2749 },
	{ 0x6551, 0x6551, PDF_CMAP_SINGLE, 2745 },
	{ 0x6553, 0x6559, PDF_CMAP_TABLE, 4141 },
	{ 0x655c, 0x655f, PDF_CMAP_TABLE, 4148 },
	{ 0x6562, 0x6563, PDF_CMAP_RANGE, 3186 },
	{ 0x6564, 0x6568, PDF_CMAP_TABLE, 4152 },
	{ 0x656a, 0x656d, PDF_CMAP_TABLE, 4157 },
	{ 0x656f, 0x656f, PDF_CMAP_SINGLE, 9225 },
	{ 0x6572, 0x6576, PDF_CMAP_TABLE, 4161 },
	{ 0x6577, 0x6578, PDF_CMAP_RANGE, 4463 },
	{ 0x6579, 0x657c, PDF_CMAP_TABLE, 4166 },
	{ 0x657f, 0x6581, PDF_CMAP_TABLE, 4170 },
	{ 0x6582, 0x6583, PDF_CMAP_RANGE, 5089 },
	{ 0x6584, 0x6589, PDF_CMAP_TABLE, 4173 },
	{ 0x658b, 0x658c, PDF_CMAP_TABLE, 4179 },
	{ 0x6590, 0x6592, PDF_CMAP_TABLE, 4181 },
	{ 0x6594, 0x6597, PDF_CMAP_TABLE, 4184 },
	{ 0x6599, 0x6599, PDF_CMAP_SINGLE, 2264 },
	{ 0x659b, 0x659c, PDF_CMAP_TABLE, 4188 },
	{ 0x659d, 0x659e, PDF_CMAP_RANGE, 8564 },
	{ 0x659f, 0x65a2, PDF_CMAP_TABLE, 4190 },
	{ 0x65a4, 0x65a5, PDF_CMAP_TABLE, 4194 },
	{ 0x65a7, 0x65a8, PDF_CMAP_TABLE, 4196 },
	{ 0x65aa, 0x65ac, PDF_CMAP_TABLE, 4198 },
	{ 0x65ae, 0x65b0, PDF_CMAP_TABLE, 4201 },
	{ 0x65b2, 0x65b3, PDF_CMAP_RANGE, 10485 },
	{ 0x65b5, 0x65b9, PDF_CMAP_TABLE, 4204 },
	{ 0x65bb, 0x65bf, PDF_CMAP_TABLE, 4209 },
	{ 0x65c1, 0x65c2, PDF_CMAP_TABLE, 4214 },
	{ 0x65c3, 0x65c4, PDF_CMAP_RANGE, 7361 },
	{ 0x65c5, 0x65c6, PDF_CMAP_TABLE, 4216 },
	{ 0x65cb, 0x65cc, PDF_CMAP_RANGE, 2757 },
	{ 0x65cd, 0x65d4, PDF_CMAP_TABLE, 4218 },
	{ 0x65d6, 0x65d7, PDF_CMAP_TABLE, 4226 },
	{ 0x65da, 0x65db, PDF_CMAP_TABLE, 4228 },
	{ 0x65dd, 0x65e3, PDF_CMAP_TABLE, 4230 },
	{ 0x65e5, 0x65e6, PDF_CMAP_TABLE, 4237 },
	{ 0x65e8, 0x65e9, PDF_CMAP_TABLE, 4239 },
	{ 0x65ec, 0x65ed, PDF_CMAP_RANGE, 986 },
	{ 0x65ee, 0x65f5, PDF_CMAP_TABLE, 4241 },
	{ 0x65fa, 0x65fd, PDF_CMAP_TABLE, 4249 },
	{ 0x65ff, 0x6600, PDF_CMAP_TABLE, 4253 },
	{ 0x6602, 0x6615, PDF_CMAP_TABLE, 4255 },
	{ 0x6618, 0x6618, PDF_CMAP_SINGLE, 16817 },
	{ 0x661c, 0x6628, PDF_CMAP_TABLE, 4275 },
	{ 0x662b, 0x662b, PDF_CMAP_SINGLE, 6925 },
	{ 0x662d, 0x6636, PDF_CMAP_TABLE, 4288 },
	{ 0x6639, 0x663a, PDF_CMAP_TABLE, 4298 },
	{ 0x6641, 0x6645, PDF_CMAP_TABLE, 4300 },
	{ 0x6647, 0x664d, PDF_CMAP_TABLE, 4305 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 2269 },
	{ 0x6651, 0x6653, PDF_CMAP_TABLE, 4312 },
	{ 0x6657, 0x6657, PDF_CMAP_SINGLE, 16539 },
	{ 0x6659, 0x6668, PDF_CMAP_TABLE, 4315 },
	{ 0x666a, 0x666c, PDF_CMAP_TABLE, 4331 },
	{ 0x666e, 0x6674, PDF_CMAP_TABLE, 4334 },
	{ 0x6676, 0x667e, PDF_CMAP_TABLE, 4341 },
	{ 0x6680, 0x6680, PDF_CMAP_SINGLE, 8572 },
	{ 0x6684, 0x6686, PDF_CMAP_TABLE, 4350 },
	{ 0x6687, 0x6688, PDF_CMAP_RANGE, 3632 },
	{ 0x6689, 0x668e, PDF_CMAP_TABLE, 4353 },
	{ 0x6690, 0x6692, PDF_CMAP_TABLE, 4359 },
	{ 0x6694, 0x669a, PDF_CMAP_TABLE, 4362 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 4077 },
	{ 0x669f, 0x66a2, PDF_CMAP_TABLE, 4369 },
	{ 0x66a4, 0x66a4, PDF_CMAP_SINGLE, 16570 },
	{ 0x66a8, 0x66ab, PDF_CMAP_TABLE, 4373 },
	{ 0x66ad, 0x66bb, PDF_CMAP_TABLE, 4377 },
	{ 0x66bd, 0x66c0, PDF_CMAP_TABLE, 4392 },
	{ 0x66c4, 0x66c4, PDF_CMAP_SINGLE, 4810 },
	{ 0x66c6, 0x66c9, PDF_CMAP_TABLE, 4396 },
	{ 0x66ca, 0x66cb, PDF_CMAP_RANGE, 11104 },
	{ 0x66cc, 0x66cf, PDF_CMAP_TABLE, 4400 },
	{ 0x66d2, 0x66d2, PDF_CMAP_SINGLE, 11634 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 5092 },
	{ 0x66d8, 0x66d9, PDF_CMAP_TABLE, 4404 },
	{ 0x66da, 0x66db, PDF_CMAP_RANGE, 12081 },
	{ 0x66dc, 0x66de, PDF_CMAP_TABLE, 4406 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 5508 },
	{ 0x66e3, 0x66e4, PDF_CMAP_RANGE, 12762 },
	{ 0x66e6, 0x66e9, PDF_CMAP_TABLE, 4409 },
	{ 0x66eb, 0x66ec, PDF_CMAP_TABLE, 4413 },
	{ 0x66ed, 0x66ee, PDF_CMAP_RANGE, 13445 },
	{ 0x66f0, 0x66f1, PDF_CMAP_TABLE, 4415 },
	{ 0x66f2, 0x66f3, PDF_CMAP_RANGE, 988 },
	{ 0x66f4, 0x66f4, PDF_CMAP_SINGLE, 1211 },
	{ 0x66f6, 0x66f9, PDF_CMAP_TABLE, 4417 },
	{ 0x66fc, 0x66fc, PDF_CMAP_SINGLE, 2591 },
	{ 0x66fe, 0x66ff, PDF_CMAP_RANGE, 3200 },
	{ 0x6700, 0x6705, PDF_CMAP_TABLE, 4421 },
	{ 0x6708, 0x670e, PDF_CMAP_TABLE, 4427 },
	{ 0x670f, 0x6710, PDF_CMAP_RANGE, 6931 },
	{ 0x6712, 0x6713, PDF_CMAP_RANGE, 7368 },
	{ 0x6714, 0x6715, PDF_CMAP_RANGE, 2276 },
	{ 0x6716, 0x6719, PDF_CMAP_TABLE, 4434 },
	{ 0x671b, 0x671b, PDF_CMAP_SINGLE, 2768 },
	{ 0x671d, 0x6723, PDF_CMAP_TABLE, 4438 },
	{ 0x6725, 0x6728, PDF_CMAP_TABLE, 4445 },
	{ 0x672a, 0x672b, PDF_CMAP_RANGE, 842 },
	{ 0x672c, 0x672e, PDF_CMAP_TABLE, 4449 },
	{ 0x6731, 0x6731, PDF_CMAP_SINGLE, 993 },
	{ 0x6733, 0x6736, PDF_CMAP_TABLE, 4452 },
	{ 0x6738, 0x673f, PDF_CMAP_TABLE, 4456 },
	{ 0x6744, 0x6749, PDF_CMAP_TABLE, 4464 },
	{ 0x674b, 0x674d, PDF_CMAP_TABLE, 4470 },
	{ 0x674e, 0x6751, PDF_CMAP_RANGE, 1213 },
	{ 0x6753, 0x6753, PDF_CMAP_SINGLE, 1223 },
	{ 0x6755, 0x6757, PDF_CMAP_TABLE, 4473 },
	{ 0x6759, 0x675a, PDF_CMAP_TABLE, 4476 },
	{ 0x675c, 0x6762, PDF_CMAP_TABLE, 4478 },
	{ 0x6767, 0x6767, PDF_CMAP_SINGLE, 16842 },
	{ 0x676a, 0x676e, PDF_CMAP_TABLE, 4485 },
	{ 0x676f, 0x6770, PDF_CMAP_RANGE, 1550 },
	{ 0x6771, 0x6777, PDF_CMAP_TABLE, 4490 },
	{ 0x6778, 0x6779, PDF_CMAP_RANGE, 6598 },
	{ 0x677a, 0x677f, PDF_CMAP_TABLE, 4497 },
	{ 0x6781, 0x6787, PDF_CMAP_TABLE, 4503 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 1553 },
	{ 0x678b, 0x6795, PDF_CMAP_TABLE, 4510 },
	{ 0x6797, 0x679a, PDF_CMAP_TABLE, 4521 },
	{ 0x679c, 0x679d, PDF_CMAP_TABLE, 4525 },
	{ 0x679f, 0x67a0, PDF_CMAP_TABLE, 4527 },
	{ 0x67a4, 0x67a4, PDF_CMAP_SINGLE, 18001 },
	{ 0x67ac, 0x67ac, PDF_CMAP_SINGLE, 16867 },
	{ 0x67ae, 0x67bb, PDF_CMAP_TABLE, 4529 },
	{ 0x67bf, 0x67c6, PDF_CMAP_TABLE, 4543 },
	{ 0x67c8, 0x67c8, PDF_CMAP_SINGLE, 6935 },
	{ 0x67c9, 0x67ca, PDF_CMAP_RANGE, 6968 },
	{ 0x67cb, 0x67d4, PDF_CMAP_TABLE, 4551 },
	{ 0x67d6, 0x67df, PDF_CMAP_TABLE, 4561 },
	{ 0x67e2, 0x67e7, PDF_CMAP_TABLE, 4571 },
	{ 0x67e9, 0x67fa, PDF_CMAP_TABLE, 4577 },
	{ 0x67fc, 0x67fc, PDF_CMAP_SINGLE, 6960 },
	{ 0x67fe, 0x6804, PDF_CMAP_TABLE, 4595 },
	{ 0x680d, 0x680d, PDF_CMAP_SINGLE, 17025 },
	{ 0x6810, 0x6810, PDF_CMAP_SINGLE, 14748 },
	{ 0x6812, 0x6814, PDF_CMAP_TABLE, 4602 },
	{ 0x6816, 0x6818, PDF_CMAP_TABLE, 4605 },
	{ 0x681a, 0x6822, PDF_CMAP_TABLE, 4608 },
	{ 0x6825, 0x6826, PDF_CMAP_TABLE, 4617 },
	{ 0x6828, 0x682b, PDF_CMAP_TABLE, 4619 },
	{ 0x682d, 0x682f, PDF_CMAP_TABLE, 4623 },
	{ 0x6831, 0x6831, PDF_CMAP_SINGLE, 7379 },
	{ 0x6832, 0x6833, PDF_CMAP_RANGE, 7373 },
	{ 0x6834, 0x683e, PDF_CMAP_TABLE, 4626 },
	{ 0x6840, 0x6851, PDF_CMAP_TABLE, 4637 },
	{ 0x6853, 0x6856, PDF_CMAP_TABLE, 4655 },
	{ 0x685d, 0x685d, PDF_CMAP_SINGLE, 16854 },
	{ 0x6865, 0x6865, PDF_CMAP_SINGLE, 17751 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 7977 },
	{ 0x686d, 0x686e, PDF_CMAP_RANGE, 7961 },
	{ 0x686f, 0x686f, PDF_CMAP_SINGLE, 7966 },
	{ 0x6871, 0x6872, PDF_CMAP_TABLE, 4659 },
	{ 0x6874, 0x6879, PDF_CMAP_TABLE, 4661 },
	{ 0x687b, 0x688c, PDF_CMAP_TABLE, 4667 },
	{ 0x688f, 0x6894, PDF_CMAP_TABLE, 4685 },
	{ 0x6896, 0x6898, PDF_CMAP_TABLE, 4691 },
	{ 0x689b, 0x689d, PDF_CMAP_TABLE, 4694 },
	{ 0x689f, 0x68a4, PDF_CMAP_TABLE, 4697 },
	{ 0x68a6, 0x68b6, PDF_CMAP_TABLE, 4703 },
	{ 0x68b9, 0x68b9, PDF_CMAP_SINGLE, 15636 },
	{ 0x68bd, 0x68bd, PDF_CMAP_SINGLE, 17020 },
	{ 0x68c3, 0x68ce, PDF_CMAP_TABLE, 4720 },
	{ 0x68d0, 0x68d8, PDF_CMAP_TABLE, 4732 },
	{ 0x68da, 0x68da, PDF_CMAP_SINGLE, 3224 },
	{ 0x68dc, 0x68dc, PDF_CMAP_SINGLE, 8581 },
	{ 0x68dd, 0x68de, PDF_CMAP_RANGE, 8615 },
	{ 0x68df, 0x68e1, PDF_CMAP_TABLE, 4741 },
	{ 0x68e3, 0x68e4, PDF_CMAP_TABLE, 4744 },
	{ 0x68e6, 0x68ec, PDF_CMAP_TABLE, 4746 },
	{ 0x68ee, 0x68fd, PDF_CMAP_TABLE, 4753 },
	{ 0x6900, 0x6915, PDF_CMAP_TABLE, 4769 },
	{ 0x6917, 0x691b, PDF_CMAP_TABLE, 4791 },
	{ 0x6925, 0x6925, PDF_CMAP_SINGLE, 8624 },
	{ 0x692a, 0x692a, PDF_CMAP_SINGLE, 8582 },
	{ 0x692c, 0x692c, PDF_CMAP_SINGLE, 16184 },
	{ 0x692f, 0x6930, PDF_CMAP_TABLE, 4796 },
	{ 0x6932, 0x6939, PDF_CMAP_TABLE, 4798 },
	{ 0x693b, 0x6946, PDF_CMAP_TABLE, 4806 },
	{ 0x6948, 0x6949, PDF_CMAP_RANGE, 9252 },
	{ 0x694a, 0x694c, PDF_CMAP_TABLE, 4818 },
	{ 0x694e, 0x694f, PDF_CMAP_TABLE, 4821 },
	{ 0x6951, 0x697b, PDF_CMAP_TABLE, 4823 },
	{ 0x6980, 0x6980, PDF_CMAP_SINGLE, 16190 },
	{ 0x6982, 0x6983, PDF_CMAP_TABLE, 4866 },
	{ 0x6985, 0x6986, PDF_CMAP_TABLE, 4868 },
	{ 0x698a, 0x698a, PDF_CMAP_SINGLE, 15732 },
	{ 0x698d, 0x698e, PDF_CMAP_TABLE, 4870 },
	{ 0x6990, 0x6991, PDF_CMAP_TABLE, 4872 },
	{ 0x6993, 0x699c, PDF_CMAP_TABLE, 4874 },
	{ 0x699e, 0x69b7, PDF_CMAP_TABLE, 4884 },
	{ 0x69b9, 0x69b9, PDF_CMAP_SINGLE, 9938 },
	{ 0x69bb, 0x69c4, PDF_CMAP_TABLE, 4910 },
	{ 0x69c6, 0x69c6, PDF_CMAP_SINGLE, 9953 },
	{ 0x69c9, 0x69d1, PDF_CMAP_TABLE, 4920 },
	{ 0x69d3, 0x69d6, PDF_CMAP_TABLE, 4929 },
	{ 0x69d9, 0x69d9, PDF_CMAP_SINGLE, 9947 },
	{ 0x69e1, 0x69e2, PDF_CMAP_TABLE, 4933 },
	{ 0x69e4, 0x69e9, PDF_CMAP_TABLE, 4935 },
	{ 0x69eb, 0x69ee, PDF_CMAP_TABLE, 4941 },
	{ 0x69f1, 0x69f4, PDF_CMAP_TABLE, 4945 },
	{ 0x69f6, 0x6a0d, PDF_CMAP_TABLE, 4949 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 10526 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 4483 },
	{ 0x6a13, 0x6a21, PDF_CMAP_TABLE, 4973 },
	{ 0x6a23, 0x6a23, PDF_CMAP_SINGLE, 4469 },
	{ 0x6a25, 0x6a29, PDF_CMAP_TABLE, 4988 },
	{ 0x6a2b, 0x6a2d, PDF_CMAP_TABLE, 4993 },
	{ 0x6a32, 0x6a35, PDF_CMAP_TABLE, 4996 },
	{ 0x6a38, 0x6a41, PDF_CMAP_TABLE, 5000 },
	{ 0x6a43, 0x6a49, PDF_CMAP_TABLE, 5010 },
	{ 0x6a4b, 0x6a4c, PDF_CMAP_TABLE, 5017 },
	{ 0x6a4d, 0x6a4e, PDF_CMAP_RANGE, 11141 },
	{ 0x6a4f, 0x6a5b, PDF_CMAP_TABLE, 5019 },
	{ 0x6a5d, 0x6a6b, PDF_CMAP_TABLE, 5032 },
	{ 0x6a6d, 0x6a6d, PDF_CMAP_SINGLE, 11120 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 11134 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 18018 },
	{ 0x6a74, 0x6a74, PDF_CMAP_SINGLE, 18017 },
	{ 0x6a76, 0x6a76, PDF_CMAP_SINGLE, 11121 },
	{ 0x6a7a, 0x6a7a, PDF_CMAP_SINGLE, 15021 },
	{ 0x6a7e, 0x6a85, PDF_CMAP_TABLE, 5047 },
	{ 0x6a87, 0x6a87, PDF_CMAP_SINGLE, 11644 },
	{ 0x6a89, 0x6a8a, PDF_CMAP_TABLE, 5055 },
	{ 0x6a8c, 0x6a97, PDF_CMAP_TABLE, 5057 },
	{ 0x6a99, 0x6aa8, PDF_CMAP_TABLE, 5069 },
	{ 0x6aab, 0x6aad, PDF_CMAP_TABLE, 5085 },
	{ 0x6aae, 0x6aaf, PDF_CMAP_RANGE, 5346 },
	{ 0x6ab1, 0x6ab5, PDF_CMAP_TABLE, 5088 },
	{ 0x6ab6, 0x6ab7, PDF_CMAP_RANGE, 12090 },
	{ 0x6ab8, 0x6abb, PDF_CMAP_TABLE, 5093 },
	{ 0x6abd, 0x6abe, PDF_CMAP_TABLE, 5097 },
	{ 0x6ac2, 0x6ac3, PDF_CMAP_TABLE, 5099 },
	{ 0x6ac5, 0x6acd, PDF_CMAP_TABLE, 5101 },
	{ 0x6acf, 0x6ad1, PDF_CMAP_TABLE, 5110 },
	{ 0x6ad3, 0x6ad4, PDF_CMAP_TABLE, 5113 },
	{ 0x6ad8, 0x6ae1, PDF_CMAP_TABLE, 5115 },
	{ 0x6ae5, 0x6ae5, PDF_CMAP_SINGLE, 5510 },
	{ 0x6ae7, 0x6ae8, PDF_CMAP_TABLE, 5125 },
	{ 0x6aea, 0x6aec, PDF_CMAP_TABLE, 5127 },
	{ 0x6aee, 0x6aef, PDF_CMAP_RANGE, 12770 },
	{ 0x6af0, 0x6af1, PDF_CMAP_TABLE, 5130 },
	{ 0x6af3, 0x6af3, PDF_CMAP_SINGLE, 12764 },
	{ 0x6af6, 0x6af6, PDF_CMAP_SINGLE, 18026 },
	{ 0x6af8, 0x6afc, PDF_CMAP_TABLE, 5132 },
	{ 0x6b00, 0x6b00, PDF_CMAP_SINGLE, 13015 },
	{ 0x6b02, 0x6b05, PDF_CMAP_TABLE, 5137 },
	{ 0x6b08, 0x6b09, PDF_CMAP_RANGE, 13190 },
	{ 0x6b0a, 0x6b0b, PDF_CMAP_TABLE, 5141 },
	{ 0x6b0f, 0x6b10, PDF_CMAP_TABLE, 5143 },
	{ 0x6b11, 0x6b12, PDF_CMAP_RANGE, 13342 },
	{ 0x6b13, 0x6b13, PDF_CMAP_SINGLE, 13447 },
	{ 0x6b16, 0x6b17, PDF_CMAP_TABLE, 5145 },
	{ 0x6b18, 0x6b19, PDF_CMAP_RANGE, 13516 },
	{ 0x6b1a, 0x6b1a, PDF_CMAP_SINGLE, 13519 },
	{ 0x6b1d, 0x6b1e, PDF_CMAP_TABLE, 5147 },
	{ 0x6b20, 0x6b21, PDF_CMAP_TABLE, 5149 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 1562 },
	{ 0x6b25, 0x6b25, PDF_CMAP_SINGLE, 6601 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 6973 },
	{ 0x6b2c, 0x6b2d, PDF_CMAP_TABLE, 5151 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 7399 },
	{ 0x6b31, 0x6b36, PDF_CMAP_TABLE, 5153 },
	{ 0x6b37, 0x6b38, PDF_CMAP_RANGE, 7997 },
	{ 0x6b39, 0x6b3f, PDF_CMAP_TABLE, 5159 },
	{ 0x6b41, 0x6b43, PDF_CMAP_TABLE, 5166 },
	{ 0x6b45, 0x6b4e, PDF_CMAP_TABLE, 5169 },
	{ 0x6b50, 0x6b52, PDF_CMAP_TABLE, 5179 },
	{ 0x6b54, 0x6b57, PDF_CMAP_TABLE, 5182 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 4828 },
	{ 0x6b5b, 0x6b5c, PDF_CMAP_TABLE, 5186 },
	{ 0x6b5e, 0x6b65, PDF_CMAP_TABLE, 5188 },
	{ 0x6b66, 0x6b67, PDF_CMAP_RANGE, 1563 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 1897 },
	{ 0x6b6d, 0x6b6d, PDF_CMAP_SINGLE, 7403 },
	{ 0x6b6f, 0x6b6f, PDF_CMAP_SINGLE, 17685 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 3659 },
	{ 0x6b74, 0x6b74, PDF_CMAP_SINGLE, 16892 },
	{ 0x6b76, 0x6b7b, PDF_CMAP_TABLE, 5196 },
	{ 0x6b7e, 0x6b84, PDF_CMAP_TABLE, 5202 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 1899 },
	{ 0x6b88, 0x6b8a, PDF_CMAP_TABLE, 5209 },
	{ 0x6b8c, 0x6b8c, PDF_CMAP_SINGLE, 8003 },
	{ 0x6b8d, 0x6b8e, PDF_CMAP_RANGE, 8001 },
	{ 0x6b8f, 0x6b8f, PDF_CMAP_SINGLE, 8000 },
	{ 0x6b91, 0x6b91, PDF_CMAP_SINGLE, 7999 },
	{ 0x6b94, 0x6b99, PDF_CMAP_TABLE, 5212 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 9288 },
	{ 0x6b9e, 0x6ba0, PDF_CMAP_RANGE, 9957 },
	{ 0x6ba2, 0x6ba7, PDF_CMAP_TABLE, 5218 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 11148 },
	{ 0x6bad, 0x6bb0, PDF_CMAP_TABLE, 5224 },
	{ 0x6bb2, 0x6bb3, PDF_CMAP_TABLE, 5228 },
	{ 0x6bb5, 0x6bb7, PDF_CMAP_TABLE, 5230 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 2792 },
	{ 0x6bbc, 0x6bbd, PDF_CMAP_TABLE, 5233 },
	{ 0x6bbf, 0x6bc1, PDF_CMAP_TABLE, 5235 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 9960 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_RANGE, 4487 },
	{ 0x6bc7, 0x6bcd, PDF_CMAP_TABLE, 5238 },
	{ 0x6bcf, 0x6bd0, PDF_CMAP_TABLE, 5245 },
	{ 0x6bd2, 0x6bd4, PDF_CMAP_TABLE, 5247 },
	{ 0x6bd6, 0x6bd8, PDF_CMAP_TABLE, 5250 },
	{ 0x6bda, 0x6bdc, PDF_CMAP_TABLE, 5253 },
	{ 0x6bde, 0x6bde, PDF_CMAP_SINGLE, 6604 },
	{ 0x6be0, 0x6be4, PDF_CMAP_TABLE, 5256 },
	{ 0x6be6, 0x6be8, PDF_CMAP_TABLE, 5261 },
	{ 0x6bea, 0x6bea, PDF_CMAP_SINGLE, 18029 },
	{ 0x6beb, 0x6bec, PDF_CMAP_RANGE, 2793 },
	{ 0x6bef, 0x6bf0, PDF_CMAP_TABLE, 5264 },
	{ 0x6bf2, 0x6bf3, PDF_CMAP_RANGE, 8636 },
	{ 0x6bf7, 0x6bf8, PDF_CMAP_RANGE, 9292 },
	{ 0x6bf9, 0x6bfa, PDF_CMAP_TABLE, 5266 },
	{ 0x6bfb, 0x6bfc, PDF_CMAP_RANGE, 9289 },
	{ 0x6bfd, 0x6c06, PDF_CMAP_TABLE, 5268 },
	{ 0x6c08, 0x6c09, PDF_CMAP_TABLE, 5278 },
	{ 0x6c0b, 0x6c0d, PDF_CMAP_TABLE, 5280 },
	{ 0x6c0f, 0x6c11, PDF_CMAP_TABLE, 5283 },
	{ 0x6c13, 0x6c16, PDF_CMAP_TABLE, 5286 },
	{ 0x6c18, 0x6c18, PDF_CMAP_SINGLE, 6145 },
	{ 0x6c19, 0x6c1a, PDF_CMAP_RANGE, 6318 },
	{ 0x6c1b, 0x6c1d, PDF_CMAP_TABLE, 5290 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 1903 },
	{ 0x6c20, 0x6c21, PDF_CMAP_RANGE, 6980 },
	{ 0x6c23, 0x6c26, PDF_CMAP_TABLE, 5293 },
	{ 0x6c27, 0x6c28, PDF_CMAP_RANGE, 2307 },
	{ 0x6c2a, 0x6c2c, PDF_CMAP_TABLE, 5297 },
	{ 0x6c2e, 0x6c2f, PDF_CMAP_RANGE, 3234 },
	{ 0x6c30, 0x6c31, PDF_CMAP_TABLE, 5300 },
	{ 0x6c33, 0x6c3b, PDF_CMAP_TABLE, 5302 },
	{ 0x6c3d, 0x6c43, PDF_CMAP_TABLE, 5311 },
	{ 0x6c46, 0x6c46, PDF_CMAP_SINGLE, 6146 },
	{ 0x6c49, 0x6c4a, PDF_CMAP_TABLE, 5318 },
	{ 0x6c4b, 0x6c4c, PDF_CMAP_RANGE, 6152 },
	{ 0x6c4d, 0x6c4e, PDF_CMAP_RANGE, 1008 },
	{ 0x6c4f, 0x6c50, PDF_CMAP_TABLE, 5320 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 6147 },
	{ 0x6c54, 0x6c55, PDF_CMAP_TABLE, 5322 },
	{ 0x6c57, 0x6c5e, PDF_CMAP_TABLE, 5324 },
	{ 0x6c5f, 0x6c60, PDF_CMAP_RANGE, 1002 },
	{ 0x6c61, 0x6c61, PDF_CMAP_SINGLE, 1006 },
	{ 0x6c65, 0x6c6b, PDF_CMAP_TABLE, 5332 },
	{ 0x6c6d, 0x6c76, PDF_CMAP_TABLE, 5339 },
	{ 0x6c78, 0x6c7b, PDF_CMAP_TABLE, 5349 },
	{ 0x6c7d, 0x6c87, PDF_CMAP_TABLE, 5353 },
	{ 0x6c88, 0x6c89, PDF_CMAP_RANGE, 1231 },
	{ 0x6c8a, 0x6c90, PDF_CMAP_TABLE, 5364 },
	{ 0x6c92, 0x6c96, PDF_CMAP_TABLE, 5371 },
	{ 0x6c98, 0x6c9d, PDF_CMAP_TABLE, 5376 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 14473 },
	{ 0x6ca2, 0x6ca2, PDF_CMAP_SINGLE, 15715 },
	{ 0x6caa, 0x6cb4, PDF_CMAP_TABLE, 5382 },
	{ 0x6cb6, 0x6cbc, PDF_CMAP_TABLE, 5393 },
	{ 0x6cbd, 0x6cbe, PDF_CMAP_RANGE, 1575 },
	{ 0x6cbf, 0x6cc7, PDF_CMAP_TABLE, 5400 },
	{ 0x6cc9, 0x6cd7, PDF_CMAP_TABLE, 5409 },
	{ 0x6cd9, 0x6ce3, PDF_CMAP_TABLE, 5424 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 1573 },
	{ 0x6ce7, 0x6cf3, PDF_CMAP_TABLE, 5435 },
	{ 0x6cf5, 0x6cf5, PDF_CMAP_SINGLE, 1919 },
	{ 0x6cf9, 0x6cf9, PDF_CMAP_SINGLE, 6635 },
	{ 0x6cff, 0x6d12, PDF_CMAP_TABLE, 5448 },
	{ 0x6d16, 0x6d1b, PDF_CMAP_TABLE, 5468 },
	{ 0x6d1d, 0x6d20, PDF_CMAP_TABLE, 5474 },
	{ 0x6d22, 0x6d22, PDF_CMAP_SINGLE, 7010 },
	{ 0x6d24, 0x6d3c, PDF_CMAP_TABLE, 5478 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_RANGE, 1915 },
	{ 0x6d3f, 0x6d42, PDF_CMAP_TABLE, 5503 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 17752 },
	{ 0x6d57, 0x6d5c, PDF_CMAP_TABLE, 5507 },
	{ 0x6d5e, 0x6d62, PDF_CMAP_TABLE, 5513 },
	{ 0x6d63, 0x6d64, PDF_CMAP_RANGE, 7414 },
	{ 0x6d65, 0x6d6a, PDF_CMAP_TABLE, 5518 },
	{ 0x6d6c, 0x6d72, PDF_CMAP_TABLE, 5524 },
	{ 0x6d74, 0x6d98, PDF_CMAP_TABLE, 5531 },
	{ 0x6da4, 0x6da5, PDF_CMAP_TABLE, 5568 },
	{ 0x6daa, 0x6dac, PDF_CMAP_TABLE, 5570 },
	{ 0x6dae, 0x6daf, PDF_CMAP_TABLE, 5573 },
	{ 0x6db1, 0x6db5, PDF_CMAP_TABLE, 5575 },
	{ 0x6db7, 0x6dc0, PDF_CMAP_TABLE, 5580 },
	{ 0x6dc2, 0x6dc2, PDF_CMAP_SINGLE, 8033 },
	{ 0x6dc4, 0x6dcd, PDF_CMAP_TABLE, 5590 },
	{ 0x6dcf, 0x6ddb, PDF_CMAP_TABLE, 5600 },
	{ 0x6ddc, 0x6ddd, PDF_CMAP_RANGE, 8023 },
	{ 0x6dde, 0x6de6, PDF_CMAP_TABLE, 5613 },
	{ 0x6de8, 0x6df7, PDF_CMAP_TABLE, 5622 },
	{ 0x6df9, 0x6dfe, PDF_CMAP_TABLE, 5638 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 8016 },
	{ 0x6e02, 0x6e05, PDF_CMAP_TABLE, 5644 },
	{ 0x6e0a, 0x6e0a, PDF_CMAP_SINGLE, 15041 },
	{ 0x6e0f, 0x6e0f, PDF_CMAP_SINGLE, 15600 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 18040 },
	{ 0x6e18, 0x6e1d, PDF_CMAP_TABLE, 5648 },
	{ 0x6e1f, 0x6e34, PDF_CMAP_TABLE, 5654 },
	{ 0x6e35, 0x6e36, PDF_CMAP_RANGE, 8684 },
	{ 0x6e38, 0x6e3b, PDF_CMAP_TABLE, 5676 },
	{ 0x6e3c, 0x6e3d, PDF_CMAP_RANGE, 8645 },
	{ 0x6e3e, 0x6e41, PDF_CMAP_TABLE, 5680 },
	{ 0x6e43, 0x6e45, PDF_CMAP_TABLE, 5684 },
	{ 0x6e46, 0x6e47, PDF_CMAP_RANGE, 8640 },
	{ 0x6e49, 0x6e4b, PDF_CMAP_TABLE, 5687 },
	{ 0x6e4d, 0x6e69, PDF_CMAP_TABLE, 5690 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 8670 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 5719 },
	{ 0x6e71, 0x6e74, PDF_CMAP_TABLE, 5721 },
	{ 0x6e76, 0x6e79, PDF_CMAP_TABLE, 5725 },
	{ 0x6e7c, 0x6e7c, PDF_CMAP_SINGLE, 16931 },
	{ 0x6e86, 0x6e86, PDF_CMAP_SINGLE, 18043 },
	{ 0x6e88, 0x6e89, PDF_CMAP_TABLE, 5729 },
	{ 0x6e8b, 0x6e8b, PDF_CMAP_SINGLE, 15295 },
	{ 0x6e8d, 0x6e90, PDF_CMAP_TABLE, 5731 },
	{ 0x6e92, 0x6e92, PDF_CMAP_SINGLE, 9322 },
	{ 0x6e93, 0x6e94, PDF_CMAP_RANGE, 9300 },
	{ 0x6e96, 0x6ea7, PDF_CMAP_TABLE, 5735 },
	{ 0x6eaa, 0x6eab, PDF_CMAP_TABLE, 5753 },
	{ 0x6eae, 0x6ed6, PDF_CMAP_TABLE, 5755 },
	{ 0x6ed8, 0x6edd, PDF_CMAP_TABLE, 5796 },
	{ 0x6ee2, 0x6ee2, PDF_CMAP_SINGLE, 18046 },
	{ 0x6ee8, 0x6ee9, PDF_CMAP_RANGE, 18049 },
	{ 0x6eeb, 0x6eef, PDF_CMAP_TABLE, 5802 },
	{ 0x6ef1, 0x6ef2, PDF_CMAP_TABLE, 5807 },
	{ 0x6ef4, 0x6f0f, PDF_CMAP_TABLE, 5809 },
	{ 0x6f12, 0x6f18, PDF_CMAP_TABLE, 5837 },
	{ 0x6f19, 0x6f1a, PDF_CMAP_RANGE, 9974 },
	{ 0x6f1c, 0x6f1c, PDF_CMAP_SINGLE, 9999 },
	{ 0x6f1e, 0x6f27, PDF_CMAP_TABLE, 5844 },
	{ 0x6f29, 0x6f41, PDF_CMAP_TABLE, 5854 },
	{ 0x6f43, 0x6f44, PDF_CMAP_TABLE, 5879 },
	{ 0x6f4e, 0x6f58, PDF_CMAP_TABLE, 5881 },
	{ 0x6f5a, 0x6f64, PDF_CMAP_TABLE, 5892 },
	{ 0x6f66, 0x6f67, PDF_CMAP_TABLE, 5903 },
	{ 0x6f69, 0x6f70, PDF_CMAP_TABLE, 5905 },
	{ 0x6f72, 0x6f74, PDF_CMAP_TABLE, 5913 },
	{ 0x6f76, 0x6f82, PDF_CMAP_TABLE, 5916 },
	{ 0x6f84, 0x6f8e, PDF_CMAP_TABLE, 5929 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 10571 },
	{ 0x6f92, 0x6f97, PDF_CMAP_TABLE, 5940 },
	{ 0x6f9d, 0x6fb6, PDF_CMAP_TABLE, 5946 },
	{ 0x6fb8, 0x6fc4, PDF_CMAP_TABLE, 5972 },
	{ 0x6fc6, 0x6fcf, PDF_CMAP_TABLE, 5985 },
	{ 0x6fd3, 0x6fd5, PDF_CMAP_TABLE, 5995 },
	{ 0x6fd8, 0x6fde, PDF_CMAP_TABLE, 5998 },
	{ 0x6fdf, 0x6fe0, PDF_CMAP_RANGE, 5110 },
	{ 0x6fe1, 0x6fe4, PDF_CMAP_TABLE, 6005 },
	{ 0x6fe6, 0x6fe9, PDF_CMAP_TABLE, 6009 },
	{ 0x6feb, 0x6ff2, PDF_CMAP_TABLE, 6013 },
	{ 0x6ff4, 0x6ff4, PDF_CMAP_SINGLE, 11663 },
	{ 0x6ff6, 0x6ff8, PDF_CMAP_TABLE, 6021 },
	{ 0x6ffa, 0x6ffc, PDF_CMAP_TABLE, 6024 },
	{ 0x6ffe, 0x7001, PDF_CMAP_TABLE, 6027 },
	{ 0x7003, 0x7007, PDF_CMAP_TABLE, 6031 },
	{ 0x7009, 0x700b, PDF_CMAP_TABLE, 6036 },
	{ 0x700c, 0x700d, PDF_CMAP_RANGE, 12099 },
	{ 0x700e, 0x700f, PDF_CMAP_TABLE, 6039 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5356 },
	{ 0x7014, 0x7020, PDF_CMAP_TABLE, 6041 },
	{ 0x7021, 0x7023, PDF_CMAP_RANGE, 12463 },
	{ 0x7024, 0x7024, PDF_CMAP_SINGLE, 12468 },
	{ 0x7026, 0x702c, PDF_CMAP_TABLE, 6054 },
	{ 0x702f, 0x7035, PDF_CMAP_TABLE, 6061 },
	{ 0x7037, 0x703c, PDF_CMAP_TABLE, 6068 },
	{ 0x703e, 0x7042, PDF_CMAP_TABLE, 6074 },
	{ 0x7043, 0x7044, PDF_CMAP_RANGE, 13016 },
	{ 0x7045, 0x7046, PDF_CMAP_RANGE, 13021 },
	{ 0x7048, 0x7049, PDF_CMAP_RANGE, 13019 },
	{ 0x704a, 0x704d, PDF_CMAP_TABLE, 6079 },
	{ 0x7050, 0x7052, PDF_CMAP_TABLE, 6083 },
	{ 0x7054, 0x7054, PDF_CMAP_SINGLE, 18065 },
	{ 0x7055, 0x7057, PDF_CMAP_RANGE, 13193 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 5833 },
	{ 0x705a, 0x7068, PDF_CMAP_TABLE, 6086 },
	{ 0x7069, 0x706a, PDF_CMAP_RANGE, 13639 },
	{ 0x706b, 0x706c, PDF_CMAP_TABLE, 6101 },
	{ 0x706e, 0x7071, PDF_CMAP_TABLE, 6103 },
	{ 0x7074, 0x707a, PDF_CMAP_TABLE, 6107 },
	{ 0x707c, 0x707d, PDF_CMAP_RANGE, 1255 },
	{ 0x707e, 0x707f, PDF_CMAP_TABLE, 6114 },
	{ 0x7081, 0x7086, PDF_CMAP_TABLE, 6116 },
	{ 0x7089, 0x708b, PDF_CMAP_TABLE, 6122 },
	{ 0x708e, 0x708f, PDF_CMAP_TABLE, 6125 },
	{ 0x7091, 0x7096, PDF_CMAP_TABLE, 6127 },
	{ 0x7098, 0x709a, PDF_CMAP_TABLE, 6133 },
	{ 0x709f, 0x70a1, PDF_CMAP_TABLE, 6136 },
	{ 0x70a3, 0x70a7, PDF_CMAP_TABLE, 6139 },
	{ 0x70a9, 0x70a9, PDF_CMAP_SINGLE, 7021 },
	{ 0x70ab, 0x70b1, PDF_CMAP_TABLE, 6144 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 1930 },
	{ 0x70b4, 0x70b5, PDF_CMAP_RANGE, 7019 },
	{ 0x70b7, 0x70be, PDF_CMAP_TABLE, 6151 },
	{ 0x70c0, 0x70c0, PDF_CMAP_SINGLE, 18227 },
	{ 0x70c4, 0x70c4, PDF_CMAP_SINGLE, 15274 },
	{ 0x70c5, 0x70c7, PDF_CMAP_RANGE, 7463 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 2337 },
	{ 0x70ca, 0x70d4, PDF_CMAP_TABLE, 6159 },
	{ 0x70d5, 0x70d6, PDF_CMAP_RANGE, 16985 },
	{ 0x70d7, 0x70da, PDF_CMAP_TABLE, 6170 },
	{ 0x70dc, 0x70e2, PDF_CMAP_TABLE, 6174 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2335 },
	{ 0x70ef, 0x70f1, PDF_CMAP_TABLE, 6181 },
	{ 0x70f3, 0x7100, PDF_CMAP_TABLE, 6184 },
	{ 0x7102, 0x7102, PDF_CMAP_SINGLE, 8062 },
	{ 0x7104, 0x7106, PDF_CMAP_TABLE, 6198 },
	{ 0x7109, 0x710a, PDF_CMAP_RANGE, 2835 },
	{ 0x710b, 0x710e, PDF_CMAP_TABLE, 6201 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 8053 },
	{ 0x7113, 0x7113, PDF_CMAP_SINGLE, 8057 },
	{ 0x7117, 0x7117, PDF_CMAP_SINGLE, 8047 },
	{ 0x7119, 0x711a, PDF_CMAP_RANGE, 3272 },
	{ 0x711b, 0x7123, PDF_CMAP_TABLE, 6205 },
	{ 0x7125, 0x7126, PDF_CMAP_TABLE, 6214 },
	{ 0x7128, 0x7129, PDF_CMAP_TABLE, 6216 },
	{ 0x712b, 0x712c, PDF_CMAP_TABLE, 6218 },
	{ 0x712e, 0x7136, PDF_CMAP_TABLE, 6220 },
	{ 0x713a, 0x713b, PDF_CMAP_TABLE, 6229 },
	{ 0x713e, 0x713e, PDF_CMAP_SINGLE, 14126 },
	{ 0x7140, 0x7147, PDF_CMAP_TABLE, 6231 },
	{ 0x7149, 0x7154, PDF_CMAP_TABLE, 6239 },
	{ 0x7156, 0x715a, PDF_CMAP_TABLE, 6251 },
	{ 0x715c, 0x716c, PDF_CMAP_TABLE, 6256 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 3278 },
	{ 0x7170, 0x7173, PDF_CMAP_TABLE, 6273 },
	{ 0x7175, 0x7178, PDF_CMAP_TABLE, 6277 },
	{ 0x717a, 0x717e, PDF_CMAP_TABLE, 6281 },
	{ 0x7180, 0x7182, PDF_CMAP_TABLE, 6286 },
	{ 0x7184, 0x718a, PDF_CMAP_TABLE, 6289 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 14826 },
	{ 0x718e, 0x7192, PDF_CMAP_TABLE, 6296 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 4129 },
	{ 0x7196, 0x71a5, PDF_CMAP_TABLE, 6301 },
	{ 0x71a7, 0x71aa, PDF_CMAP_TABLE, 6317 },
	{ 0x71ac, 0x71ad, PDF_CMAP_TABLE, 6321 },
	{ 0x71af, 0x71b5, PDF_CMAP_TABLE, 6323 },
	{ 0x71b7, 0x71ba, PDF_CMAP_TABLE, 6330 },
	{ 0x71bc, 0x71bf, PDF_CMAP_TABLE, 6334 },
	{ 0x71c0, 0x71c1, PDF_CMAP_RANGE, 11189 },
	{ 0x71c2, 0x71c2, PDF_CMAP_SINGLE, 11185 },
	{ 0x71c3, 0x71c4, PDF_CMAP_RANGE, 4855 },
	{ 0x71c5, 0x71cb, PDF_CMAP_TABLE, 6338 },
	{ 0x71ce, 0x71d2, PDF_CMAP_TABLE, 6345 },
	{ 0x71d4, 0x71d6, PDF_CMAP_TABLE, 6350 },
	{ 0x71d8, 0x71d9, PDF_CMAP_TABLE, 6353 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 11200 },
	{ 0x71dc, 0x71dd, PDF_CMAP_TABLE, 6355 },
	{ 0x71df, 0x71e2, PDF_CMAP_TABLE, 6357 },
	{ 0x71e4, 0x71e8, PDF_CMAP_TABLE, 6361 },
	{ 0x71eb, 0x71ee, PDF_CMAP_TABLE, 6366 },
	{ 0x71f0, 0x71f2, PDF_CMAP_TABLE, 6370 },
	{ 0x71f4, 0x71f6, PDF_CMAP_TABLE, 6373 },
	{ 0x71f8, 0x71f9, PDF_CMAP_TABLE, 6376 },
	{ 0x71fb, 0x71fc, PDF_CMAP_RANGE, 5358 },
	{ 0x71fd, 0x7203, PDF_CMAP_TABLE, 6378 },
	{ 0x7205, 0x7207, PDF_CMAP_TABLE, 6385 },
	{ 0x7209, 0x720a, PDF_CMAP_TABLE, 6388 },
	{ 0x720c, 0x7210, PDF_CMAP_TABLE, 6390 },
	{ 0x7213, 0x7214, PDF_CMAP_RANGE, 12788 },
	{ 0x7215, 0x7217, PDF_CMAP_TABLE, 6395 },
	{ 0x7219, 0x721b, PDF_CMAP_TABLE, 6398 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 13023 },
	{ 0x721e, 0x721f, PDF_CMAP_RANGE, 13197 },
	{ 0x7222, 0x722b, PDF_CMAP_TABLE, 6401 },
	{ 0x722c, 0x722d, PDF_CMAP_RANGE, 1605 },
	{ 0x722e, 0x722e, PDF_CMAP_SINGLE, 17017 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 1937 },
	{ 0x7235, 0x7236, PDF_CMAP_TABLE, 6411 },
	{ 0x7238, 0x723b, PDF_CMAP_TABLE, 6413 },
	{ 0x723d, 0x7242, PDF_CMAP_TABLE, 6417 },
	{ 0x7244, 0x7244, PDF_CMAP_SINGLE, 10018 },
	{ 0x7246, 0x7248, PDF_CMAP_TABLE, 6423 },
	{ 0x7249, 0x724a, PDF_CMAP_RANGE, 7023 },
	{ 0x724b, 0x724c, PDF_CMAP_TABLE, 6426 },
	{ 0x724f, 0x7250, PDF_CMAP_TABLE, 6428 },
	{ 0x7252, 0x7253, PDF_CMAP_TABLE, 6430 },
	{ 0x7255, 0x7263, PDF_CMAP_TABLE, 6432 },
	{ 0x7266, 0x7267, PDF_CMAP_TABLE, 6447 },
	{ 0x7269, 0x726a, PDF_CMAP_TABLE, 6449 },
	{ 0x726c, 0x726c, PDF_CMAP_SINGLE, 7025 },
	{ 0x726e, 0x7270, PDF_CMAP_TABLE, 6451 },
	{ 0x7272, 0x7274, PDF_CMAP_TABLE, 6454 },
	{ 0x7276, 0x7279, PDF_CMAP_TABLE, 6457 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 8065 },
	{ 0x727d, 0x7282, PDF_CMAP_TABLE, 6461 },
	{ 0x7284, 0x7287, PDF_CMAP_TABLE, 6467 },
	{ 0x7288, 0x7289, PDF_CMAP_RANGE, 8703 },
	{ 0x728b, 0x7298, PDF_CMAP_TABLE, 6471 },
	{ 0x729a, 0x729b, PDF_CMAP_TABLE, 6485 },
	{ 0x729d, 0x729e, PDF_CMAP_RANGE, 11202 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 18234 },
	{ 0x72a1, 0x72a4, PDF_CMAP_TABLE, 6487 },
	{ 0x72a5, 0x72a6, PDF_CMAP_RANGE, 12475 },
	{ 0x72a7, 0x72aa, PDF_CMAP_TABLE, 6491 },
	{ 0x72ac, 0x72b0, PDF_CMAP_TABLE, 6495 },
	{ 0x72b2, 0x72b2, PDF_CMAP_SINGLE, 16310 },
	{ 0x72b4, 0x72b5, PDF_CMAP_RANGE, 6156 },
	{ 0x72ba, 0x72ba, PDF_CMAP_SINGLE, 6347 },
	{ 0x72bd, 0x72bd, PDF_CMAP_SINGLE, 6343 },
	{ 0x72bf, 0x72c6, PDF_CMAP_TABLE, 6500 },
	{ 0x72c9, 0x72ce, PDF_CMAP_TABLE, 6508 },
	{ 0x72d0, 0x72d2, PDF_CMAP_TABLE, 6514 },
	{ 0x72d4, 0x72d4, PDF_CMAP_SINGLE, 6657 },
	{ 0x72d6, 0x72da, PDF_CMAP_TABLE, 6517 },
	{ 0x72dc, 0x72dc, PDF_CMAP_SINGLE, 6655 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 7033 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 1942 },
	{ 0x72e2, 0x72e4, PDF_CMAP_TABLE, 6522 },
	{ 0x72e6, 0x72e6, PDF_CMAP_SINGLE, 7035 },
	{ 0x72e8, 0x72eb, PDF_CMAP_TABLE, 6525 },
	{ 0x72f3, 0x72f4, PDF_CMAP_TABLE, 6529 },
	{ 0x72f6, 0x7302, PDF_CMAP_TABLE, 6531 },
	{ 0x7304, 0x7304, PDF_CMAP_SINGLE, 16053 },
	{ 0x7307, 0x7308, PDF_CMAP_TABLE, 6544 },
	{ 0x730a, 0x730c, PDF_CMAP_TABLE, 6546 },
	{ 0x730f, 0x7313, PDF_CMAP_TABLE, 6549 },
	{ 0x7316, 0x7319, PDF_CMAP_TABLE, 6554 },
	{ 0x731b, 0x731e, PDF_CMAP_TABLE, 6558 },
	{ 0x7322, 0x7323, PDF_CMAP_TABLE, 6562 },
	{ 0x7325, 0x732e, PDF_CMAP_TABLE, 6564 },
	{ 0x7330, 0x733c, PDF_CMAP_TABLE, 6574 },
	{ 0x733e, 0x7345, PDF_CMAP_TABLE, 6587 },
	{ 0x7348, 0x734a, PDF_CMAP_TABLE, 6595 },
	{ 0x734c, 0x7352, PDF_CMAP_TABLE, 6598 },
	{ 0x7357, 0x735b, PDF_CMAP_TABLE, 6605 },
	{ 0x735d, 0x735d, PDF_CMAP_SINGLE, 10605 },
	{ 0x735e, 0x7360, PDF_CMAP_RANGE, 10602 },
	{ 0x7361, 0x7362, PDF_CMAP_TABLE, 6610 },
	{ 0x7365, 0x7365, PDF_CMAP_SINGLE, 11208 },
	{ 0x7366, 0x7367, PDF_CMAP_RANGE, 11205 },
	{ 0x7368, 0x736c, PDF_CMAP_TABLE, 6612 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 11683 },
	{ 0x7370, 0x7378, PDF_CMAP_TABLE, 6617 },
	{ 0x737a, 0x738c, PDF_CMAP_TABLE, 6626 },
	{ 0x738e, 0x738f, PDF_CMAP_TABLE, 6645 },
	{ 0x7392, 0x7392, PDF_CMAP_SINGLE, 6353 },
	{ 0x7393, 0x7394, PDF_CMAP_RANGE, 6351 },
	{ 0x7395, 0x7398, PDF_CMAP_TABLE, 6647 },
	{ 0x739c, 0x73a2, PDF_CMAP_TABLE, 6651 },
	{ 0x73a4, 0x73ad, PDF_CMAP_TABLE, 6658 },
	{ 0x73b2, 0x73bc, PDF_CMAP_TABLE, 6668 },
	{ 0x73be, 0x73c0, PDF_CMAP_TABLE, 6679 },
	{ 0x73c2, 0x73d0, PDF_CMAP_TABLE, 6682 },
	{ 0x73d2, 0x73d6, PDF_CMAP_TABLE, 6697 },
	{ 0x73d7, 0x73d8, PDF_CMAP_RANGE, 7495 },
	{ 0x73d9, 0x73de, PDF_CMAP_TABLE, 6702 },
	{ 0x73e0, 0x73eb, PDF_CMAP_TABLE, 6708 },
	{ 0x73ed, 0x73ef, PDF_CMAP_TABLE, 6720 },
	{ 0x73f3, 0x740d, PDF_CMAP_TABLE, 6723 },
	{ 0x7411, 0x7412, PDF_CMAP_TABLE, 6750 },
	{ 0x7414, 0x7417, PDF_CMAP_TABLE, 6752 },
	{ 0x7419, 0x741d, PDF_CMAP_TABLE, 6756 },
	{ 0x741e, 0x741f, PDF_CMAP_RANGE, 18164 },
	{ 0x7420, 0x7426, PDF_CMAP_TABLE, 6761 },
	{ 0x7428, 0x7434, PDF_CMAP_TABLE, 6768 },
	{ 0x7435, 0x7436, PDF_CMAP_RANGE, 3292 },
	{ 0x7437, 0x743a, PDF_CMAP_TABLE, 6781 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 15071 },
	{ 0x743f, 0x7449, PDF_CMAP_TABLE, 6785 },
	{ 0x744a, 0x744b, PDF_CMAP_RANGE, 9371 },
	{ 0x744c, 0x744e, PDF_CMAP_TABLE, 6796 },
	{ 0x744f, 0x7450, PDF_CMAP_RANGE, 9377 },
	{ 0x7451, 0x7457, PDF_CMAP_TABLE, 6799 },
	{ 0x7459, 0x745a, PDF_CMAP_TABLE, 6806 },
	{ 0x745b, 0x745c, PDF_CMAP_RANGE, 3715 },
	{ 0x745d, 0x7465, PDF_CMAP_TABLE, 6808 },
	{ 0x7467, 0x7476, PDF_CMAP_TABLE, 6817 },
	{ 0x7479, 0x747a, PDF_CMAP_TABLE, 6833 },
	{ 0x747c, 0x7483, PDF_CMAP_TABLE, 6835 },
	{ 0x7485, 0x7488, PDF_CMAP_TABLE, 6843 },
	{ 0x7489, 0x748a, PDF_CMAP_RANGE, 10612 },
	{ 0x748b, 0x748d, PDF_CMAP_TABLE, 6847 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 11688 },
	{ 0x7492, 0x7492, PDF_CMAP_SINGLE, 11215 },
	{ 0x7494, 0x7495, PDF_CMAP_TABLE, 6850 },
	{ 0x7497, 0x74a1, PDF_CMAP_TABLE, 6852 },
	{ 0x74a3, 0x74ab, PDF_CMAP_TABLE, 6863 },
	{ 0x74ad, 0x74ad, PDF_CMAP_SINGLE, 11690 },
	{ 0x74af, 0x74b2, PDF_CMAP_TABLE, 6872 },
	{ 0x74b4, 0x74bb, PDF_CMAP_TABLE, 6876 },
	{ 0x74bd, 0x74c3, PDF_CMAP_TABLE, 6884 },
	{ 0x74c5, 0x74c6, PDF_CMAP_TABLE, 6891 },
	{ 0x74c8, 0x74c8, PDF_CMAP_SINGLE, 17077 },
	{ 0x74ca, 0x74cc, PDF_CMAP_TABLE, 6893 },
	{ 0x74cf, 0x74d0, PDF_CMAP_TABLE, 6896 },
	{ 0x74d3, 0x74dd, PDF_CMAP_TABLE, 6898 },
	{ 0x74de, 0x74df, PDF_CMAP_RANGE, 7498 },
	{ 0x74e0, 0x74e9, PDF_CMAP_TABLE, 6909 },
	{ 0x74ec, 0x74ec, PDF_CMAP_SINGLE, 7054 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 7055 },
	{ 0x74f0, 0x74f2, PDF_CMAP_TABLE, 6919 },
	{ 0x74f4, 0x74f5, PDF_CMAP_RANGE, 7500 },
	{ 0x74f6, 0x74f7, PDF_CMAP_RANGE, 2855 },
	{ 0x74f8, 0x74f8, PDF_CMAP_SINGLE, 16372 },
	{ 0x74fb, 0x74fb, PDF_CMAP_SINGLE, 8736 },
	{ 0x74fd, 0x7500, PDF_CMAP_TABLE, 6922 },
	{ 0x7502, 0x7503, PDF_CMAP_RANGE, 10035 },
	{ 0x7504, 0x7505, PDF_CMAP_TABLE, 6926 },
	{ 0x7507, 0x7508, PDF_CMAP_TABLE, 6928 },
	{ 0x750b, 0x750b, PDF_CMAP_SINGLE, 11218 },
	{ 0x750c, 0x750d, PDF_CMAP_RANGE, 4864 },
	{ 0x750e, 0x750f, PDF_CMAP_TABLE, 6930 },
	{ 0x7510, 0x7512, PDF_CMAP_RANGE, 11694 },
	{ 0x7513, 0x751a, PDF_CMAP_TABLE, 6932 },
	{ 0x751c, 0x751f, PDF_CMAP_TABLE, 6940 },
	{ 0x7521, 0x7522, PDF_CMAP_TABLE, 6944 },
	{ 0x7525, 0x7526, PDF_CMAP_RANGE, 3299 },
	{ 0x7528, 0x7529, PDF_CMAP_RANGE, 860 },
	{ 0x752a, 0x752f, PDF_CMAP_TABLE, 6946 },
	{ 0x7530, 0x7533, PDF_CMAP_RANGE, 862 },
	{ 0x7534, 0x7535, PDF_CMAP_TABLE, 6952 },
	{ 0x7537, 0x7538, PDF_CMAP_RANGE, 1266 },
	{ 0x7539, 0x753b, PDF_CMAP_TABLE, 6954 },
	{ 0x753d, 0x753e, PDF_CMAP_TABLE, 6957 },
	{ 0x753f, 0x7540, PDF_CMAP_RANGE, 6671 },
	{ 0x7542, 0x7542, PDF_CMAP_SINGLE, 17357 },
	{ 0x7546, 0x7546, PDF_CMAP_SINGLE, 16192 },
	{ 0x7547, 0x7548, PDF_CMAP_RANGE, 7057 },
	{ 0x754a, 0x754f, PDF_CMAP_TABLE, 6959 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 15614 },
	{ 0x7553, 0x7555, PDF_CMAP_TABLE, 6965 },
	{ 0x7559, 0x755d, PDF_CMAP_TABLE, 6968 },
	{ 0x755f, 0x7560, PDF_CMAP_TABLE, 6973 },
	{ 0x7562, 0x7564, PDF_CMAP_TABLE, 6975 },
	{ 0x7565, 0x7566, PDF_CMAP_RANGE, 2859 },
	{ 0x7567, 0x7567, PDF_CMAP_SINGLE, 17088 },
	{ 0x756a, 0x7570, PDF_CMAP_TABLE, 6978 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 18174 },
	{ 0x7576, 0x757a, PDF_CMAP_TABLE, 6985 },
	{ 0x757d, 0x7580, PDF_CMAP_TABLE, 6990 },
	{ 0x7583, 0x7584, PDF_CMAP_TABLE, 6994 },
	{ 0x7586, 0x7587, PDF_CMAP_TABLE, 6996 },
	{ 0x758a, 0x7592, PDF_CMAP_TABLE, 6998 },
	{ 0x7594, 0x7595, PDF_CMAP_RANGE, 6356 },
	{ 0x7598, 0x7598, PDF_CMAP_SINGLE, 6675 },
	{ 0x7599, 0x759a, PDF_CMAP_RANGE, 1623 },
	{ 0x759d, 0x759e, PDF_CMAP_TABLE, 7007 },
	{ 0x75a2, 0x75a3, PDF_CMAP_RANGE, 1960 },
	{ 0x75a4, 0x75a5, PDF_CMAP_RANGE, 1958 },
	{ 0x75a7, 0x75a7, PDF_CMAP_SINGLE, 7059 },
	{ 0x75aa, 0x75ab, PDF_CMAP_TABLE, 7009 },
	{ 0x75b0, 0x75b1, PDF_CMAP_TABLE, 7011 },
	{ 0x75b2, 0x75b3, PDF_CMAP_RANGE, 2361 },
	{ 0x75b4, 0x75b6, PDF_CMAP_TABLE, 7013 },
	{ 0x75b8, 0x75c5, PDF_CMAP_TABLE, 7016 },
	{ 0x75c7, 0x75c8, PDF_CMAP_TABLE, 7030 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 2867 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 8099 },
	{ 0x75cd, 0x75d2, PDF_CMAP_TABLE, 7032 },
	{ 0x75d4, 0x75d5, PDF_CMAP_RANGE, 2864 },
	{ 0x75d7, 0x75e4, PDF_CMAP_TABLE, 7038 },
	{ 0x75e6, 0x75e7, PDF_CMAP_TABLE, 7052 },
	{ 0x75ed, 0x75ed, PDF_CMAP_SINGLE, 9403 },
	{ 0x75ef, 0x7603, PDF_CMAP_TABLE, 7054 },
	{ 0x7607, 0x760d, PDF_CMAP_TABLE, 7075 },
	{ 0x760f, 0x7611, PDF_CMAP_TABLE, 7082 },
	{ 0x7613, 0x7616, PDF_CMAP_TABLE, 7085 },
	{ 0x7619, 0x7620, PDF_CMAP_TABLE, 7089 },
	{ 0x7621, 0x7622, PDF_CMAP_RANGE, 4529 },
	{ 0x7623, 0x7629, PDF_CMAP_TABLE, 7097 },
	{ 0x762c, 0x762d, PDF_CMAP_TABLE, 7104 },
	{ 0x762f, 0x7635, PDF_CMAP_TABLE, 7106 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 4867 },
	{ 0x763a, 0x763d, PDF_CMAP_TABLE, 7113 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 16102 },
	{ 0x7642, 0x7643, PDF_CMAP_TABLE, 7117 },
	{ 0x7646, 0x7647, PDF_CMAP_TABLE, 7119 },
	{ 0x7648, 0x7649, PDF_CMAP_RANGE, 11700 },
	{ 0x764c, 0x7654, PDF_CMAP_TABLE, 7121 },
	{ 0x7656, 0x765a, PDF_CMAP_TABLE, 7130 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 12128 },
	{ 0x765f, 0x7662, PDF_CMAP_TABLE, 7135 },
	{ 0x7664, 0x7667, PDF_CMAP_TABLE, 7139 },
	{ 0x7669, 0x766a, PDF_CMAP_TABLE, 7143 },
	{ 0x766c, 0x7670, PDF_CMAP_TABLE, 7145 },
	{ 0x7671, 0x7672, PDF_CMAP_RANGE, 5923 },
	{ 0x7673, 0x7676, PDF_CMAP_TABLE, 7150 },
	{ 0x7678, 0x767a, PDF_CMAP_TABLE, 7154 },
	{ 0x767b, 0x767c, PDF_CMAP_RANGE, 3310 },
	{ 0x767d, 0x7682, PDF_CMAP_TABLE, 7157 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 1625 },
	{ 0x7686, 0x7688, PDF_CMAP_RANGE, 1963 },
	{ 0x7689, 0x768c, PDF_CMAP_TABLE, 7163 },
	{ 0x768e, 0x7690, PDF_CMAP_TABLE, 7167 },
	{ 0x7692, 0x7693, PDF_CMAP_TABLE, 7170 },
	{ 0x7695, 0x7696, PDF_CMAP_TABLE, 7172 },
	{ 0x7699, 0x769b, PDF_CMAP_TABLE, 7174 },
	{ 0x769c, 0x769e, PDF_CMAP_RANGE, 10633 },
	{ 0x76a1, 0x76a1, PDF_CMAP_SINGLE, 17109 },
	{ 0x76a4, 0x76a6, PDF_CMAP_TABLE, 7177 },
	{ 0x76aa, 0x76ab, PDF_CMAP_TABLE, 7180 },
	{ 0x76ad, 0x76b0, PDF_CMAP_TABLE, 7182 },
	{ 0x76b4, 0x76b5, PDF_CMAP_TABLE, 7186 },
	{ 0x76b7, 0x76b8, PDF_CMAP_TABLE, 7188 },
	{ 0x76ba, 0x76bb, PDF_CMAP_TABLE, 7190 },
	{ 0x76bd, 0x76bf, PDF_CMAP_TABLE, 7192 },
	{ 0x76c2, 0x76c6, PDF_CMAP_TABLE, 7195 },
	{ 0x76c8, 0x76ca, PDF_CMAP_TABLE, 7200 },
	{ 0x76cc, 0x76cc, PDF_CMAP_SINGLE, 17112 },
	{ 0x76cd, 0x76ce, PDF_CMAP_RANGE, 2371 },
	{ 0x76d2, 0x76d4, PDF_CMAP_TABLE, 7203 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 16213 },
	{ 0x76d9, 0x76dd, PDF_CMAP_TABLE, 7206 },
	{ 0x76de, 0x76df, PDF_CMAP_RANGE, 3728 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 4152 },
	{ 0x76e3, 0x76e7, PDF_CMAP_TABLE, 7211 },
	{ 0x76e9, 0x76ea, PDF_CMAP_TABLE, 7216 },
	{ 0x76ec, 0x76f5, PDF_CMAP_TABLE, 7218 },
	{ 0x76f7, 0x76fc, PDF_CMAP_TABLE, 7228 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 1975 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 1970 },
	{ 0x7703, 0x7705, PDF_CMAP_RANGE, 7064 },
	{ 0x7707, 0x770c, PDF_CMAP_TABLE, 7234 },
	{ 0x770e, 0x770f, PDF_CMAP_RANGE, 15106 },
	{ 0x7710, 0x7713, PDF_CMAP_TABLE, 7240 },
	{ 0x7715, 0x7715, PDF_CMAP_SINGLE, 7522 },
	{ 0x7719, 0x771a, PDF_CMAP_RANGE, 7523 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 7516 },
	{ 0x771d, 0x771e, PDF_CMAP_TABLE, 7244 },
	{ 0x771f, 0x7720, PDF_CMAP_RANGE, 2374 },
	{ 0x7722, 0x7729, PDF_CMAP_TABLE, 7246 },
	{ 0x772b, 0x772b, PDF_CMAP_SINGLE, 16371 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 8108 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 8107 },
	{ 0x7731, 0x7732, PDF_CMAP_RANGE, 8109 },
	{ 0x7733, 0x773e, PDF_CMAP_TABLE, 7254 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 17120 },
	{ 0x7743, 0x7745, PDF_CMAP_TABLE, 7266 },
	{ 0x7746, 0x7747, PDF_CMAP_RANGE, 8751 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 8756 },
	{ 0x774b, 0x774c, PDF_CMAP_RANGE, 8758 },
	{ 0x774d, 0x774f, PDF_CMAP_TABLE, 7269 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 9412 },
	{ 0x7754, 0x7756, PDF_CMAP_TABLE, 7272 },
	{ 0x7758, 0x775c, PDF_CMAP_TABLE, 7275 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 3733 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 9410 },
	{ 0x7761, 0x7763, PDF_CMAP_TABLE, 7280 },
	{ 0x7765, 0x776f, PDF_CMAP_TABLE, 7283 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14699 },
	{ 0x7777, 0x7785, PDF_CMAP_TABLE, 7294 },
	{ 0x7787, 0x7789, PDF_CMAP_TABLE, 7309 },
	{ 0x778b, 0x778f, PDF_CMAP_TABLE, 7312 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 4537 },
	{ 0x7793, 0x7793, PDF_CMAP_SINGLE, 14381 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 11238 },
	{ 0x7797, 0x779d, PDF_CMAP_TABLE, 7317 },
	{ 0x779e, 0x779f, PDF_CMAP_RANGE, 4872 },
	{ 0x77a0, 0x77a1, PDF_CMAP_TABLE, 7324 },
	{ 0x77a2, 0x77a3, PDF_CMAP_RANGE, 11236 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 4874 },
	{ 0x77a7, 0x77a8, PDF_CMAP_TABLE, 7326 },
	{ 0x77aa, 0x77ad, PDF_CMAP_TABLE, 7328 },
	{ 0x77af, 0x77b7, PDF_CMAP_TABLE, 7332 },
	{ 0x77b9, 0x77ba, PDF_CMAP_TABLE, 7341 },
	{ 0x77bb, 0x77bc, PDF_CMAP_RANGE, 5372 },
	{ 0x77bd, 0x77bf, PDF_CMAP_TABLE, 7343 },
	{ 0x77c2, 0x77c5, PDF_CMAP_TABLE, 7346 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5534 },
	{ 0x77c9, 0x77ca, PDF_CMAP_RANGE, 12486 },
	{ 0x77cb, 0x77cd, PDF_CMAP_TABLE, 7350 },
	{ 0x77ce, 0x77cf, PDF_CMAP_RANGE, 12799 },
	{ 0x77d0, 0x77d0, PDF_CMAP_SINGLE, 13029 },
	{ 0x77d3, 0x77d5, PDF_CMAP_TABLE, 7353 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5925 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 13523 },
	{ 0x77da, 0x77de, PDF_CMAP_TABLE, 7356 },
	{ 0x77e0, 0x77e0, PDF_CMAP_SINGLE, 9420 },
	{ 0x77e2, 0x77e3, PDF_CMAP_TABLE, 7361 },
	{ 0x77e5, 0x77e6, PDF_CMAP_TABLE, 7363 },
	{ 0x77e7, 0x77e8, PDF_CMAP_RANGE, 7071 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 2377 },
	{ 0x77ec, 0x77f4, PDF_CMAP_TABLE, 7365 },
	{ 0x77f7, 0x77fe, PDF_CMAP_TABLE, 7374 },
	{ 0x7802, 0x7803, PDF_CMAP_TABLE, 7382 },
	{ 0x7805, 0x7806, PDF_CMAP_TABLE, 7384 },
	{ 0x7808, 0x7809, PDF_CMAP_TABLE, 7386 },
	{ 0x780c, 0x780d, PDF_CMAP_RANGE, 1981 },
	{ 0x780e, 0x7810, PDF_CMAP_TABLE, 7388 },
	{ 0x7811, 0x7812, PDF_CMAP_RANGE, 7074 },
	{ 0x7813, 0x7814, PDF_CMAP_TABLE, 7391 },
	{ 0x7818, 0x7818, PDF_CMAP_SINGLE, 18195 },
	{ 0x781c, 0x7823, PDF_CMAP_TABLE, 7393 },
	{ 0x7825, 0x7835, PDF_CMAP_TABLE, 7401 },
	{ 0x7837, 0x7839, PDF_CMAP_TABLE, 7418 },
	{ 0x783c, 0x783d, PDF_CMAP_TABLE, 7421 },
	{ 0x7842, 0x7845, PDF_CMAP_TABLE, 7423 },
	{ 0x7847, 0x784e, PDF_CMAP_TABLE, 7427 },
	{ 0x7850, 0x7854, PDF_CMAP_TABLE, 7435 },
	{ 0x785c, 0x785e, PDF_CMAP_TABLE, 7440 },
	{ 0x7860, 0x7860, PDF_CMAP_SINGLE, 8762 },
	{ 0x7862, 0x7862, PDF_CMAP_SINGLE, 8774 },
	{ 0x7864, 0x7865, PDF_CMAP_RANGE, 8763 },
	{ 0x7866, 0x7866, PDF_CMAP_SINGLE, 18199 },
	{ 0x7868, 0x7871, PDF_CMAP_TABLE, 7443 },
	{ 0x7879, 0x787c, PDF_CMAP_TABLE, 7453 },
	{ 0x787e, 0x7881, PDF_CMAP_TABLE, 7457 },
	{ 0x7883, 0x7884, PDF_CMAP_TABLE, 7461 },
	{ 0x7885, 0x7886, PDF_CMAP_RANGE, 9427 },
	{ 0x7887, 0x7889, PDF_CMAP_TABLE, 7463 },
	{ 0x788c, 0x788f, PDF_CMAP_TABLE, 7466 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3750 },
	{ 0x7893, 0x7896, PDF_CMAP_TABLE, 7470 },
	{ 0x7897, 0x7898, PDF_CMAP_RANGE, 3745 },
	{ 0x7899, 0x789a, PDF_CMAP_TABLE, 7474 },
	{ 0x789e, 0x78a5, PDF_CMAP_TABLE, 7476 },
	{ 0x78a7, 0x78ad, PDF_CMAP_TABLE, 7484 },
	{ 0x78af, 0x78b4, PDF_CMAP_TABLE, 7491 },
	{ 0x78b6, 0x78b6, PDF_CMAP_SINGLE, 15468 },
	{ 0x78b8, 0x78bc, PDF_CMAP_TABLE, 7497 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 4543 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 4158 },
	{ 0x78c3, 0x78c4, PDF_CMAP_RANGE, 10649 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 4540 },
	{ 0x78c7, 0x78d5, PDF_CMAP_TABLE, 7502 },
	{ 0x78d7, 0x78d8, PDF_CMAP_TABLE, 7517 },
	{ 0x78da, 0x78db, PDF_CMAP_TABLE, 7519 },
	{ 0x78dd, 0x78de, PDF_CMAP_TABLE, 7521 },
	{ 0x78df, 0x78e0, PDF_CMAP_RANGE, 11251 },
	{ 0x78e1, 0x78e2, PDF_CMAP_RANGE, 11248 },
	{ 0x78e3, 0x78e5, PDF_CMAP_TABLE, 7523 },
	{ 0x78e7, 0x78ea, PDF_CMAP_TABLE, 7526 },
	{ 0x78ec, 0x78f5, PDF_CMAP_TABLE, 7530 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 5151 },
	{ 0x78f9, 0x78fa, PDF_CMAP_TABLE, 7540 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 11717 },
	{ 0x78fd, 0x78ff, PDF_CMAP_TABLE, 7542 },
	{ 0x7901, 0x7902, PDF_CMAP_TABLE, 7545 },
	{ 0x7904, 0x7906, PDF_CMAP_TABLE, 7547 },
	{ 0x7909, 0x7909, PDF_CMAP_SINGLE, 12143 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 12140 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 5374 },
	{ 0x7910, 0x7912, PDF_CMAP_TABLE, 7550 },
	{ 0x7913, 0x7914, PDF_CMAP_RANGE, 12141 },
	{ 0x7917, 0x7917, PDF_CMAP_SINGLE, 12494 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5535 },
	{ 0x791b, 0x791e, PDF_CMAP_TABLE, 7553 },
	{ 0x7921, 0x7921, PDF_CMAP_SINGLE, 12492 },
	{ 0x7923, 0x7926, PDF_CMAP_TABLE, 7557 },
	{ 0x7927, 0x7928, PDF_CMAP_RANGE, 12805 },
	{ 0x7929, 0x792f, PDF_CMAP_TABLE, 7561 },
	{ 0x7931, 0x7936, PDF_CMAP_TABLE, 7568 },
	{ 0x7938, 0x793f, PDF_CMAP_TABLE, 7574 },
	{ 0x7940, 0x7941, PDF_CMAP_RANGE, 1632 },
	{ 0x7942, 0x7942, PDF_CMAP_SINGLE, 6687 },
	{ 0x7944, 0x794c, PDF_CMAP_TABLE, 7582 },
	{ 0x794f, 0x7965, PDF_CMAP_TABLE, 7591 },
	{ 0x7967, 0x7968, PDF_CMAP_TABLE, 7614 },
	{ 0x7969, 0x796a, PDF_CMAP_RANGE, 8128 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 8131 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2884 },
	{ 0x7970, 0x7974, PDF_CMAP_TABLE, 7616 },
	{ 0x7979, 0x797a, PDF_CMAP_TABLE, 7621 },
	{ 0x797c, 0x7983, PDF_CMAP_TABLE, 7623 },
	{ 0x7986, 0x7988, PDF_CMAP_TABLE, 7631 },
	{ 0x798a, 0x798b, PDF_CMAP_RANGE, 10070 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 4166 },
	{ 0x798e, 0x798f, PDF_CMAP_RANGE, 4164 },
	{ 0x7990, 0x799d, PDF_CMAP_TABLE, 7634 },
	{ 0x799f, 0x79a2, PDF_CMAP_TABLE, 7648 },
	{ 0x79a4, 0x79ae, PDF_CMAP_TABLE, 7652 },
	{ 0x79b0, 0x79b4, PDF_CMAP_TABLE, 7663 },
	{ 0x79b6, 0x79b8, PDF_CMAP_TABLE, 7668 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 1987 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 8133 },
	{ 0x79bd, 0x79c1, PDF_CMAP_TABLE, 7671 },
	{ 0x79c4, 0x79c6, PDF_CMAP_TABLE, 7676 },
	{ 0x79c8, 0x79d0, PDF_CMAP_TABLE, 7679 },
	{ 0x79d1, 0x79d2, PDF_CMAP_RANGE, 1989 },
	{ 0x79d4, 0x79d6, PDF_CMAP_TABLE, 7688 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2404 },
	{ 0x79dc, 0x79e0, PDF_CMAP_TABLE, 7691 },
	{ 0x79e2, 0x79e4, PDF_CMAP_TABLE, 7696 },
	{ 0x79e6, 0x79e7, PDF_CMAP_TABLE, 7699 },
	{ 0x79e9, 0x79ea, PDF_CMAP_TABLE, 7701 },
	{ 0x79eb, 0x79ec, PDF_CMAP_RANGE, 7547 },
	{ 0x79ed, 0x79ee, PDF_CMAP_TABLE, 7703 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 15143 },
	{ 0x79f4, 0x79f4, PDF_CMAP_SINGLE, 15138 },
	{ 0x79f6, 0x79f7, PDF_CMAP_RANGE, 8136 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 8135 },
	{ 0x79fa, 0x79fb, PDF_CMAP_TABLE, 7705 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 3325 },
	{ 0x7a02, 0x7a06, PDF_CMAP_TABLE, 7707 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 3322 },
	{ 0x7a0a, 0x7a0d, PDF_CMAP_TABLE, 7712 },
	{ 0x7a10, 0x7a15, PDF_CMAP_TABLE, 7716 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 9444 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 9441 },
	{ 0x7a1a, 0x7a1c, PDF_CMAP_TABLE, 7722 },
	{ 0x7a1e, 0x7a20, PDF_CMAP_TABLE, 7725 },
	{ 0x7a22, 0x7a22, PDF_CMAP_SINGLE, 9446 },
	{ 0x7a26, 0x7a26, PDF_CMAP_SINGLE, 10085 },
	{ 0x7a28, 0x7a28, PDF_CMAP_SINGLE, 10084 },
	{ 0x7a2a, 0x7a32, PDF_CMAP_TABLE, 7728 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 4551 },
	{ 0x7a39, 0x7a40, PDF_CMAP_TABLE, 7737 },
	{ 0x7a43, 0x7a4c, PDF_CMAP_TABLE, 7745 },
	{ 0x7a4d, 0x7a4e, PDF_CMAP_RANGE, 4880 },
	{ 0x7a54, 0x7a54, PDF_CMAP_SINGLE, 11731 },
	{ 0x7a56, 0x7a58, PDF_CMAP_TABLE, 7755 },
	{ 0x7a5a, 0x7a5c, PDF_CMAP_TABLE, 7758 },
	{ 0x7a5f, 0x7a60, PDF_CMAP_TABLE, 7761 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5376 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 14134 },
	{ 0x7a67, 0x7a68, PDF_CMAP_RANGE, 12497 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5538 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 5537 },
	{ 0x7a6c, 0x7a6d, PDF_CMAP_RANGE, 12811 },
	{ 0x7a6e, 0x7a6e, PDF_CMAP_SINGLE, 12810 },
	{ 0x7a70, 0x7a71, PDF_CMAP_RANGE, 13209 },
	{ 0x7a72, 0x7a72, PDF_CMAP_SINGLE, 19133 },
	{ 0x7a74, 0x7a76, PDF_CMAP_TABLE, 7763 },
	{ 0x7a78, 0x7a7b, PDF_CMAP_TABLE, 7766 },
	{ 0x7a7d, 0x7a81, PDF_CMAP_TABLE, 7770 },
	{ 0x7a83, 0x7a8a, PDF_CMAP_TABLE, 7775 },
	{ 0x7a8b, 0x7a8c, PDF_CMAP_RANGE, 7559 },
	{ 0x7a8f, 0x7a99, PDF_CMAP_TABLE, 7783 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 9452 },
	{ 0x7a9f, 0x7aa0, PDF_CMAP_RANGE, 3764 },
	{ 0x7aa2, 0x7aa3, PDF_CMAP_TABLE, 7794 },
	{ 0x7aa8, 0x7aaa, PDF_CMAP_TABLE, 7796 },
	{ 0x7aab, 0x7aac, PDF_CMAP_RANGE, 10087 },
	{ 0x7aae, 0x7ab8, PDF_CMAP_TABLE, 7799 },
	{ 0x7aba, 0x7abc, PDF_CMAP_TABLE, 7810 },
	{ 0x7abe, 0x7abf, PDF_CMAP_TABLE, 7813 },
	{ 0x7ac0, 0x7ac1, PDF_CMAP_RANGE, 11734 },
	{ 0x7ac2, 0x7ac3, PDF_CMAP_RANGE, 14139 },
	{ 0x7ac4, 0x7ac5, PDF_CMAP_RANGE, 5379 },
	{ 0x7ac7, 0x7acb, PDF_CMAP_TABLE, 7815 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 18212 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 7095 },
	{ 0x7ad3, 0x7ad3, PDF_CMAP_SINGLE, 16266 },
	{ 0x7ad8, 0x7add, PDF_CMAP_TABLE, 7820 },
	{ 0x7adf, 0x7ae0, PDF_CMAP_TABLE, 7826 },
	{ 0x7ae2, 0x7ae7, PDF_CMAP_TABLE, 7828 },
	{ 0x7ae9, 0x7aeb, PDF_CMAP_TABLE, 7834 },
	{ 0x7aed, 0x7aef, PDF_CMAP_TABLE, 7837 },
	{ 0x7af6, 0x7af7, PDF_CMAP_TABLE, 7840 },
	{ 0x7af9, 0x7aff, PDF_CMAP_TABLE, 7842 },
	{ 0x7b00, 0x7b01, PDF_CMAP_RANGE, 7096 },
	{ 0x7b04, 0x7b06, PDF_CMAP_TABLE, 7849 },
	{ 0x7b08, 0x7b0c, PDF_CMAP_TABLE, 7852 },
	{ 0x7b0e, 0x7b14, PDF_CMAP_TABLE, 7857 },
	{ 0x7b18, 0x7b1b, PDF_CMAP_TABLE, 7864 },
	{ 0x7b1d, 0x7b20, PDF_CMAP_TABLE, 7868 },
	{ 0x7b22, 0x7b35, PDF_CMAP_TABLE, 7872 },
	{ 0x7b38, 0x7b39, PDF_CMAP_TABLE, 7892 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 8142 },
	{ 0x7b40, 0x7b40, PDF_CMAP_SINGLE, 8793 },
	{ 0x7b42, 0x7b52, PDF_CMAP_TABLE, 7894 },
	{ 0x7b54, 0x7b56, PDF_CMAP_TABLE, 7911 },
	{ 0x7b58, 0x7b58, PDF_CMAP_SINGLE, 8794 },
	{ 0x7b60, 0x7b67, PDF_CMAP_TABLE, 7914 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 9458 },
	{ 0x7b6c, 0x7b78, PDF_CMAP_TABLE, 7922 },
	{ 0x7b7b, 0x7b7b, PDF_CMAP_SINGLE, 18217 },
	{ 0x7b82, 0x7b82, PDF_CMAP_SINGLE, 10105 },
	{ 0x7b84, 0x7b85, PDF_CMAP_TABLE, 7935 },
	{ 0x7b87, 0x7b88, PDF_CMAP_TABLE, 7937 },
	{ 0x7b8a, 0x7b92, PDF_CMAP_TABLE, 7939 },
	{ 0x7b94, 0x7b99, PDF_CMAP_TABLE, 7948 },
	{ 0x7b9b, 0x7b9d, PDF_CMAP_TABLE, 7954 },
	{ 0x7ba0, 0x7ba4, PDF_CMAP_TABLE, 7957 },
	{ 0x7bac, 0x7baf, PDF_CMAP_TABLE, 7962 },
	{ 0x7bb1, 0x7bb2, PDF_CMAP_TABLE, 7966 },
	{ 0x7bb4, 0x7bb5, PDF_CMAP_TABLE, 7968 },
	{ 0x7bb7, 0x7bb9, PDF_CMAP_TABLE, 7970 },
	{ 0x7bbe, 0x7bbe, PDF_CMAP_SINGLE, 10665 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 7973 },
	{ 0x7bc4, 0x7bc5, PDF_CMAP_TABLE, 7975 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_RANGE, 4559 },
	{ 0x7bc9, 0x7bcc, PDF_CMAP_TABLE, 7977 },
	{ 0x7bce, 0x7bd0, PDF_CMAP_TABLE, 7981 },
	{ 0x7bd4, 0x7bd5, PDF_CMAP_TABLE, 7984 },
	{ 0x7bd8, 0x7bec, PDF_CMAP_TABLE, 7986 },
	{ 0x7bf0, 0x7bf1, PDF_CMAP_RANGE, 11753 },
	{ 0x7bf2, 0x7bf4, PDF_CMAP_TABLE, 8007 },
	{ 0x7bf7, 0x7c03, PDF_CMAP_TABLE, 8010 },
	{ 0x7c05, 0x7c07, PDF_CMAP_TABLE, 8023 },
	{ 0x7c09, 0x7c12, PDF_CMAP_TABLE, 8026 },
	{ 0x7c15, 0x7c15, PDF_CMAP_SINGLE, 16481 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 12152 },
	{ 0x7c1b, 0x7c23, PDF_CMAP_TABLE, 8036 },
	{ 0x7c25, 0x7c2d, PDF_CMAP_TABLE, 8045 },
	{ 0x7c30, 0x7c30, PDF_CMAP_SINGLE, 12161 },
	{ 0x7c33, 0x7c33, PDF_CMAP_SINGLE, 12499 },
	{ 0x7c35, 0x7c35, PDF_CMAP_SINGLE, 14169 },
	{ 0x7c37, 0x7c39, PDF_CMAP_TABLE, 8054 },
	{ 0x7c3b, 0x7c3d, PDF_CMAP_TABLE, 8057 },
	{ 0x7c3e, 0x7c3f, PDF_CMAP_RANGE, 5539 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 5544 },
	{ 0x7c42, 0x7c45, PDF_CMAP_TABLE, 8060 },
	{ 0x7c47, 0x7c4a, PDF_CMAP_TABLE, 8064 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 8068 },
	{ 0x7c50, 0x7c51, PDF_CMAP_TABLE, 8070 },
	{ 0x7c53, 0x7c54, PDF_CMAP_TABLE, 8072 },
	{ 0x7c56, 0x7c57, PDF_CMAP_TABLE, 8074 },
	{ 0x7c59, 0x7c5d, PDF_CMAP_TABLE, 8076 },
	{ 0x7c5f, 0x7c60, PDF_CMAP_TABLE, 8081 },
	{ 0x7c63, 0x7c67, PDF_CMAP_TABLE, 8083 },
	{ 0x7c69, 0x7c70, PDF_CMAP_TABLE, 8088 },
	{ 0x7c72, 0x7c75, PDF_CMAP_TABLE, 8096 },
	{ 0x7c78, 0x7c79, PDF_CMAP_RANGE, 7099 },
	{ 0x7c7a, 0x7c7e, PDF_CMAP_TABLE, 8100 },
	{ 0x7c7f, 0x7c81, PDF_CMAP_RANGE, 7101 },
	{ 0x7c83, 0x7c86, PDF_CMAP_TABLE, 8105 },
	{ 0x7c88, 0x7c8a, PDF_CMAP_TABLE, 8109 },
	{ 0x7c8c, 0x7c8e, PDF_CMAP_TABLE, 8112 },
	{ 0x7c91, 0x7c92, PDF_CMAP_TABLE, 8115 },
	{ 0x7c94, 0x7c98, PDF_CMAP_TABLE, 8117 },
	{ 0x7c9c, 0x7c9c, PDF_CMAP_SINGLE, 18222 },
	{ 0x7c9e, 0x7c9f, PDF_CMAP_TABLE, 8122 },
	{ 0x7ca1, 0x7ca3, PDF_CMAP_TABLE, 8124 },
	{ 0x7ca5, 0x7ca8, PDF_CMAP_TABLE, 8127 },
	{ 0x7cac, 0x7cac, PDF_CMAP_SINGLE, 14469 },
	{ 0x7cae, 0x7caf, PDF_CMAP_TABLE, 8131 },
	{ 0x7cb1, 0x7cb5, PDF_CMAP_TABLE, 8133 },
	{ 0x7cb8, 0x7cbc, PDF_CMAP_TABLE, 8138 },
	{ 0x7cbd, 0x7cbe, PDF_CMAP_RANGE, 4185 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 10107 },
	{ 0x7cc2, 0x7cc3, PDF_CMAP_TABLE, 8143 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 10672 },
	{ 0x7cc7, 0x7cce, PDF_CMAP_TABLE, 8145 },
	{ 0x7cd0, 0x7cd1, PDF_CMAP_RANGE, 11281 },
	{ 0x7cd2, 0x7cd4, PDF_CMAP_TABLE, 8153 },
	{ 0x7cd5, 0x7cd6, PDF_CMAP_RANGE, 4894 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 11280 },
	{ 0x7cd9, 0x7cda, PDF_CMAP_TABLE, 8156 },
	{ 0x7cdc, 0x7ce0, PDF_CMAP_TABLE, 8158 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 5169 },
	{ 0x7ce6, 0x7ce8, PDF_CMAP_TABLE, 8163 },
	{ 0x7cea, 0x7cea, PDF_CMAP_SINGLE, 12505 },
	{ 0x7cec, 0x7cee, PDF_CMAP_TABLE, 8166 },
	{ 0x7cef, 0x7cf0, PDF_CMAP_RANGE, 5677 },
	{ 0x7cf1, 0x7cf9, PDF_CMAP_TABLE, 8169 },
	{ 0x7cfb, 0x7cfe, PDF_CMAP_TABLE, 8178 },
	{ 0x7d00, 0x7d22, PDF_CMAP_TABLE, 8182 },
	{ 0x7d25, 0x7d25, PDF_CMAP_SINGLE, 16094 },
	{ 0x7d28, 0x7d29, PDF_CMAP_TABLE, 8217 },
	{ 0x7d2b, 0x7d2c, PDF_CMAP_TABLE, 8219 },
	{ 0x7d2e, 0x7d33, PDF_CMAP_TABLE, 8221 },
	{ 0x7d35, 0x7d36, PDF_CMAP_TABLE, 8227 },
	{ 0x7d38, 0x7d3d, PDF_CMAP_TABLE, 8229 },
	{ 0x7d3e, 0x7d3f, PDF_CMAP_RANGE, 8174 },
	{ 0x7d40, 0x7d47, PDF_CMAP_TABLE, 8235 },
	{ 0x7d4a, 0x7d4a, PDF_CMAP_SINGLE, 8176 },
	{ 0x7d4d, 0x7d56, PDF_CMAP_TABLE, 8243 },
	{ 0x7d58, 0x7d58, PDF_CMAP_SINGLE, 8800 },
	{ 0x7d5a, 0x7d5f, PDF_CMAP_TABLE, 8253 },
	{ 0x7d61, 0x7d63, PDF_CMAP_TABLE, 8259 },
	{ 0x7d66, 0x7d6b, PDF_CMAP_TABLE, 8262 },
	{ 0x7d6d, 0x7d73, PDF_CMAP_TABLE, 8268 },
	{ 0x7d79, 0x7d7d, PDF_CMAP_TABLE, 8275 },
	{ 0x7d7f, 0x7d81, PDF_CMAP_TABLE, 8280 },
	{ 0x7d83, 0x7d86, PDF_CMAP_TABLE, 8283 },
	{ 0x7d88, 0x7d89, PDF_CMAP_TABLE, 8287 },
	{ 0x7d8b, 0x7d8f, PDF_CMAP_TABLE, 8289 },
	{ 0x7d91, 0x7d97, PDF_CMAP_TABLE, 8294 },
	{ 0x7d9c, 0x7da4, PDF_CMAP_TABLE, 8301 },
	{ 0x7da6, 0x7db5, PDF_CMAP_TABLE, 8310 },
	{ 0x7db7, 0x7dbc, PDF_CMAP_TABLE, 8326 },
	{ 0x7dbd, 0x7dbe, PDF_CMAP_RANGE, 4190 },
	{ 0x7dbf, 0x7dc2, PDF_CMAP_TABLE, 8332 },
	{ 0x7dc4, 0x7dc7, PDF_CMAP_TABLE, 8336 },
	{ 0x7dc9, 0x7dca, PDF_CMAP_TABLE, 8340 },
	{ 0x7dcb, 0x7dcc, PDF_CMAP_RANGE, 10122 },
	{ 0x7dcd, 0x7dd0, PDF_CMAP_TABLE, 8342 },
	{ 0x7dd2, 0x7dd4, PDF_CMAP_TABLE, 8346 },
	{ 0x7dd6, 0x7de1, PDF_CMAP_TABLE, 8349 },
	{ 0x7de3, 0x7dea, PDF_CMAP_TABLE, 8361 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 4570 },
	{ 0x7dee, 0x7df7, PDF_CMAP_TABLE, 8369 },
	{ 0x7df9, 0x7dfb, PDF_CMAP_TABLE, 8379 },
	{ 0x7dfd, 0x7dfe, PDF_CMAP_TABLE, 8382 },
	{ 0x7e03, 0x7e03, PDF_CMAP_SINGLE, 10682 },
	{ 0x7e07, 0x7e17, PDF_CMAP_TABLE, 8384 },
	{ 0x7e1a, 0x7e1e, PDF_CMAP_TABLE, 8401 },
	{ 0x7e1f, 0x7e20, PDF_CMAP_RANGE, 11287 },
	{ 0x7e21, 0x7e25, PDF_CMAP_TABLE, 8406 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 14207 },
	{ 0x7e29, 0x7e2b, PDF_CMAP_TABLE, 8411 },
	{ 0x7e2d, 0x7e49, PDF_CMAP_TABLE, 8414 },
	{ 0x7e4c, 0x7e4c, PDF_CMAP_SINGLE, 11769 },
	{ 0x7e50, 0x7e53, PDF_CMAP_TABLE, 8443 },
	{ 0x7e54, 0x7e55, PDF_CMAP_RANGE, 5388 },
	{ 0x7e56, 0x7e5c, PDF_CMAP_TABLE, 8447 },
	{ 0x7e5e, 0x7e63, PDF_CMAP_TABLE, 8454 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 18130 },
	{ 0x7e67, 0x7e68, PDF_CMAP_TABLE, 8460 },
	{ 0x7e69, 0x7e6a, PDF_CMAP_RANGE, 5548 },
	{ 0x7e6b, 0x7e70, PDF_CMAP_TABLE, 8462 },
	{ 0x7e72, 0x7e82, PDF_CMAP_TABLE, 8468 },
	{ 0x7e86, 0x7e86, PDF_CMAP_SINGLE, 13040 },
	{ 0x7e87, 0x7e88, PDF_CMAP_RANGE, 13037 },
	{ 0x7e8a, 0x7e8f, PDF_CMAP_TABLE, 8485 },
	{ 0x7e91, 0x7e9c, PDF_CMAP_TABLE, 8491 },
	{ 0x7e9f, 0x7e9f, PDF_CMAP_SINGLE, 18619 },
	{ 0x7ea4, 0x7ea4, PDF_CMAP_SINGLE, 17755 },
	{ 0x7eac, 0x7eac, PDF_CMAP_SINGLE, 17756 },
	{ 0x7eba, 0x7eba, PDF_CMAP_SINGLE, 17757 },
	{ 0x7ec7, 0x7ec7, PDF_CMAP_SINGLE, 17758 },
	{ 0x7ecf, 0x7ecf, PDF_CMAP_SINGLE, 17759 },
	{ 0x7edf, 0x7edf, PDF_CMAP_SINGLE, 17760 },
	{ 0x7f06, 0x7f06, PDF_CMAP_SINGLE, 17761 },
	{ 0x7f36, 0x7f3a, PDF_CMAP_TABLE, 8503 },
	{ 0x7f3d, 0x7f3d, PDF_CMAP_SINGLE, 2913 },
	{ 0x7f3e, 0x7f3f, PDF_CMAP_RANGE, 8817 },
	{ 0x7f40, 0x7f41, PDF_CMAP_RANGE, 14208 },
	{ 0x7f43, 0x7f45, PDF_CMAP_TABLE, 8508 },
	{ 0x7f47, 0x7f55, PDF_CMAP_TABLE, 8511 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 7107 },
	{ 0x7f5b, 0x7f61, PDF_CMAP_TABLE, 8526 },
	{ 0x7f63, 0x7f63, PDF_CMAP_SINGLE, 8179 },
	{ 0x7f65, 0x7f66, PDF_CMAP_RANGE, 8819 },
	{ 0x7f67, 0x7f68, PDF_CMAP_RANGE, 9489 },
	{ 0x7f69, 0x7f6a, PDF_CMAP_RANGE, 3781 },
	{ 0x7f6b, 0x7f6e, PDF_CMAP_TABLE, 8533 },
	{ 0x7f70, 0x7f73, PDF_CMAP_TABLE, 8537 },
	{ 0x7f75, 0x7f7a, PDF_CMAP_TABLE, 8541 },
	{ 0x7f7b, 0x7f7c, PDF_CMAP_RANGE, 11303 },
	{ 0x7f7d, 0x7f7f, PDF_CMAP_TABLE, 8547 },
	{ 0x7f83, 0x7f83, PDF_CMAP_SINGLE, 12518 },
	{ 0x7f85, 0x7f8f, PDF_CMAP_TABLE, 8550 },
	{ 0x7f91, 0x7f97, PDF_CMAP_TABLE, 8561 },
	{ 0x7f9a, 0x7f9b, PDF_CMAP_TABLE, 8568 },
	{ 0x7f9c, 0x7f9d, PDF_CMAP_RANGE, 8181 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 2914 },
	{ 0x7fa0, 0x7fa1, PDF_CMAP_RANGE, 8822 },
	{ 0x7fa2, 0x7fa9, PDF_CMAP_TABLE, 8570 },
	{ 0x7fac, 0x7fc3, PDF_CMAP_TABLE, 8578 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 2428 },
	{ 0x7fc7, 0x7fc7, PDF_CMAP_SINGLE, 8189 },
	{ 0x7fc9, 0x7fc9, PDF_CMAP_SINGLE, 8191 },
	{ 0x7fca, 0x7fcb, PDF_CMAP_RANGE, 8184 },
	{ 0x7fcc, 0x7fcf, PDF_CMAP_TABLE, 8602 },
	{ 0x7fd0, 0x7fd1, PDF_CMAP_RANGE, 8187 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2918 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_RANGE, 3356 },
	{ 0x7fd7, 0x7fd7, PDF_CMAP_SINGLE, 8824 },
	{ 0x7fdb, 0x7fdc, PDF_CMAP_RANGE, 9495 },
	{ 0x7fdd, 0x7fdf, PDF_CMAP_TABLE, 8606 },
	{ 0x7fe0, 0x7fe1, PDF_CMAP_RANGE, 4207 },
	{ 0x7fe2, 0x7fe3, PDF_CMAP_RANGE, 10135 },
	{ 0x7fe5, 0x7fef, PDF_CMAP_TABLE, 8609 },
	{ 0x7ff0, 0x7ff1, PDF_CMAP_RANGE, 4907 },
	{ 0x7ff2, 0x7ff5, PDF_CMAP_TABLE, 8620 },
	{ 0x7ff7, 0x7ff8, PDF_CMAP_RANGE, 12175 },
	{ 0x7ff9, 0x7ffc, PDF_CMAP_TABLE, 8624 },
	{ 0x7ffd, 0x7ffe, PDF_CMAP_RANGE, 12521 },
	{ 0x7fff, 0x8008, PDF_CMAP_TABLE, 8628 },
	{ 0x800b, 0x800d, PDF_CMAP_TABLE, 8638 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 7112 },
	{ 0x8010, 0x8012, PDF_CMAP_TABLE, 8641 },
	{ 0x8014, 0x8019, PDF_CMAP_TABLE, 8644 },
	{ 0x801b, 0x8021, PDF_CMAP_TABLE, 8650 },
	{ 0x8024, 0x8026, PDF_CMAP_TABLE, 8657 },
	{ 0x8028, 0x802a, PDF_CMAP_TABLE, 8660 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 11781 },
	{ 0x802e, 0x8031, PDF_CMAP_TABLE, 8663 },
	{ 0x8033, 0x8037, PDF_CMAP_TABLE, 8667 },
	{ 0x8039, 0x8039, PDF_CMAP_SINGLE, 7605 },
	{ 0x803b, 0x803c, PDF_CMAP_RANGE, 14231 },
	{ 0x803d, 0x803f, PDF_CMAP_TABLE, 8672 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 8196 },
	{ 0x8046, 0x8048, PDF_CMAP_TABLE, 8675 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 2920 },
	{ 0x804f, 0x8050, PDF_CMAP_RANGE, 8826 },
	{ 0x8051, 0x8052, PDF_CMAP_TABLE, 8678 },
	{ 0x8054, 0x8054, PDF_CMAP_SINGLE, 18240 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 3787 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 3788 },
	{ 0x805a, 0x805e, PDF_CMAP_TABLE, 8680 },
	{ 0x8061, 0x8064, PDF_CMAP_TABLE, 8685 },
	{ 0x8066, 0x8067, PDF_CMAP_TABLE, 8689 },
	{ 0x806c, 0x806c, PDF_CMAP_SINGLE, 11310 },
	{ 0x806f, 0x8070, PDF_CMAP_TABLE, 8691 },
	{ 0x8071, 0x8072, PDF_CMAP_RANGE, 5193 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 5197 },
	{ 0x8075, 0x8079, PDF_CMAP_TABLE, 8693 },
	{ 0x807d, 0x8080, PDF_CMAP_TABLE, 8698 },
	{ 0x8082, 0x8082, PDF_CMAP_SINGLE, 7404 },
	{ 0x8084, 0x8087, PDF_CMAP_TABLE, 8702 },
	{ 0x8089, 0x808a, PDF_CMAP_TABLE, 8706 },
	{ 0x808b, 0x808c, PDF_CMAP_RANGE, 1027 },
	{ 0x808f, 0x8090, PDF_CMAP_TABLE, 8708 },
	{ 0x8092, 0x8093, PDF_CMAP_TABLE, 8710 },
	{ 0x8095, 0x8096, PDF_CMAP_TABLE, 8712 },
	{ 0x8098, 0x809d, PDF_CMAP_TABLE, 8714 },
	{ 0x809f, 0x809f, PDF_CMAP_SINGLE, 17782 },
	{ 0x80a1, 0x80a3, PDF_CMAP_TABLE, 8720 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 1645 },
	{ 0x80a7, 0x80a7, PDF_CMAP_SINGLE, 14240 },
	{ 0x80a9, 0x80ab, PDF_CMAP_TABLE, 8723 },
	{ 0x80ad, 0x80af, PDF_CMAP_TABLE, 8726 },
	{ 0x80b1, 0x80b2, PDF_CMAP_TABLE, 8729 },
	{ 0x80b4, 0x80b8, PDF_CMAP_TABLE, 8731 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 1644 },
	{ 0x80bc, 0x80bd, PDF_CMAP_TABLE, 8736 },
	{ 0x80c2, 0x80c2, PDF_CMAP_SINGLE, 7121 },
	{ 0x80c3, 0x80c4, PDF_CMAP_RANGE, 2014 },
	{ 0x80c5, 0x80ca, PDF_CMAP_TABLE, 8738 },
	{ 0x80cc, 0x80d1, PDF_CMAP_TABLE, 8744 },
	{ 0x80d4, 0x80de, PDF_CMAP_TABLE, 8750 },
	{ 0x80e0, 0x80e1, PDF_CMAP_TABLE, 8761 },
	{ 0x80e3, 0x80e6, PDF_CMAP_TABLE, 8763 },
	{ 0x80e9, 0x80e9, PDF_CMAP_SINGLE, 18244 },
	{ 0x80ec, 0x80ed, PDF_CMAP_TABLE, 8767 },
	{ 0x80ef, 0x80f6, PDF_CMAP_TABLE, 8769 },
	{ 0x80f8, 0x80fe, PDF_CMAP_TABLE, 8777 },
	{ 0x8100, 0x8103, PDF_CMAP_TABLE, 8784 },
	{ 0x8105, 0x810a, PDF_CMAP_TABLE, 8788 },
	{ 0x810c, 0x810c, PDF_CMAP_SINGLE, 18246 },
	{ 0x810e, 0x810e, PDF_CMAP_SINGLE, 18247 },
	{ 0x8112, 0x8112, PDF_CMAP_SINGLE, 18248 },
	{ 0x8114, 0x811b, PDF_CMAP_TABLE, 8794 },
	{ 0x811d, 0x811f, PDF_CMAP_TABLE, 8802 },
	{ 0x8121, 0x8125, PDF_CMAP_TABLE, 8805 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 8208 },
	{ 0x8129, 0x812d, PDF_CMAP_TABLE, 8810 },
	{ 0x812f, 0x8130, PDF_CMAP_TABLE, 8815 },
	{ 0x8132, 0x8132, PDF_CMAP_SINGLE, 15993 },
	{ 0x8134, 0x8134, PDF_CMAP_SINGLE, 17900 },
	{ 0x8137, 0x8137, PDF_CMAP_SINGLE, 15218 },
	{ 0x8139, 0x813a, PDF_CMAP_TABLE, 8817 },
	{ 0x813d, 0x813e, PDF_CMAP_TABLE, 8819 },
	{ 0x8142, 0x8144, PDF_CMAP_TABLE, 8821 },
	{ 0x8146, 0x8148, PDF_CMAP_TABLE, 8824 },
	{ 0x814a, 0x8156, PDF_CMAP_TABLE, 8827 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 18253 },
	{ 0x815b, 0x815c, PDF_CMAP_TABLE, 8840 },
	{ 0x815e, 0x815e, PDF_CMAP_SINGLE, 9507 },
	{ 0x8160, 0x8162, PDF_CMAP_TABLE, 8842 },
	{ 0x8164, 0x8167, PDF_CMAP_TABLE, 8845 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 9502 },
	{ 0x816b, 0x8174, PDF_CMAP_TABLE, 8849 },
	{ 0x8176, 0x8178, PDF_CMAP_TABLE, 8859 },
	{ 0x8179, 0x817a, PDF_CMAP_RANGE, 3798 },
	{ 0x817c, 0x817c, PDF_CMAP_SINGLE, 18256 },
	{ 0x817f, 0x8180, PDF_CMAP_TABLE, 8862 },
	{ 0x8182, 0x8184, PDF_CMAP_TABLE, 8864 },
	{ 0x8186, 0x818d, PDF_CMAP_TABLE, 8867 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 4215 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 17350 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 10705 },
	{ 0x8197, 0x819a, PDF_CMAP_TABLE, 8875 },
	{ 0x819b, 0x819d, PDF_CMAP_RANGE, 4586 },
	{ 0x819e, 0x81a0, PDF_CMAP_TABLE, 8879 },
	{ 0x81a2, 0x81a3, PDF_CMAP_TABLE, 8882 },
	{ 0x81a5, 0x81ac, PDF_CMAP_TABLE, 8884 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 11313 },
	{ 0x81b0, 0x81b7, PDF_CMAP_TABLE, 8892 },
	{ 0x81b9, 0x81ca, PDF_CMAP_TABLE, 8900 },
	{ 0x81cc, 0x81cd, PDF_CMAP_TABLE, 8918 },
	{ 0x81cf, 0x81d0, PDF_CMAP_TABLE, 8920 },
	{ 0x81d1, 0x81d2, PDF_CMAP_RANGE, 12178 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 12525 },
	{ 0x81d7, 0x81db, PDF_CMAP_TABLE, 8922 },
	{ 0x81dd, 0x81df, PDF_CMAP_TABLE, 8927 },
	{ 0x81e0, 0x81e1, PDF_CMAP_RANGE, 13533 },
	{ 0x81e2, 0x81ea, PDF_CMAP_TABLE, 8930 },
	{ 0x81ec, 0x81ef, PDF_CMAP_TABLE, 8939 },
	{ 0x81f2, 0x81f4, PDF_CMAP_TABLE, 8943 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 15713 },
	{ 0x81f7, 0x81f9, PDF_CMAP_RANGE, 8840 },
	{ 0x81fa, 0x81fc, PDF_CMAP_TABLE, 8946 },
	{ 0x81fe, 0x8202, PDF_CMAP_TABLE, 8949 },
	{ 0x8204, 0x8205, PDF_CMAP_TABLE, 8954 },
	{ 0x8207, 0x820d, PDF_CMAP_TABLE, 8956 },
	{ 0x8210, 0x8212, PDF_CMAP_TABLE, 8963 },
	{ 0x8214, 0x8216, PDF_CMAP_TABLE, 8966 },
	{ 0x8218, 0x8218, PDF_CMAP_SINGLE, 14250 },
	{ 0x821a, 0x8222, PDF_CMAP_TABLE, 8969 },
	{ 0x8225, 0x8226, PDF_CMAP_TABLE, 8978 },
	{ 0x8228, 0x8229, PDF_CMAP_TABLE, 8980 },
	{ 0x822a, 0x822b, PDF_CMAP_RANGE, 2456 },
	{ 0x822c, 0x822d, PDF_CMAP_TABLE, 8982 },
	{ 0x822f, 0x822f, PDF_CMAP_SINGLE, 7614 },
	{ 0x8232, 0x823a, PDF_CMAP_TABLE, 8984 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 8844 },
	{ 0x823e, 0x8240, PDF_CMAP_TABLE, 8993 },
	{ 0x8242, 0x8242, PDF_CMAP_SINGLE, 9517 },
	{ 0x8244, 0x8245, PDF_CMAP_TABLE, 8996 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3802 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 9514 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 4225 },
	{ 0x824e, 0x8254, PDF_CMAP_TABLE, 8998 },
	{ 0x8255, 0x8257, PDF_CMAP_RANGE, 11324 },
	{ 0x8258, 0x8259, PDF_CMAP_RANGE, 4916 },
	{ 0x825a, 0x825c, PDF_CMAP_TABLE, 9005 },
	{ 0x825e, 0x825f, PDF_CMAP_TABLE, 9008 },
	{ 0x8261, 0x8266, PDF_CMAP_TABLE, 9010 },
	{ 0x8268, 0x8269, PDF_CMAP_RANGE, 12829 },
	{ 0x826b, 0x826f, PDF_CMAP_TABLE, 9016 },
	{ 0x8271, 0x8272, PDF_CMAP_TABLE, 9021 },
	{ 0x8274, 0x8280, PDF_CMAP_TABLE, 9023 },
	{ 0x8283, 0x8284, PDF_CMAP_RANGE, 6373 },
	{ 0x8285, 0x8285, PDF_CMAP_SINGLE, 6368 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 16690 },
	{ 0x828a, 0x828b, PDF_CMAP_TABLE, 9036 },
	{ 0x828d, 0x8294, PDF_CMAP_TABLE, 9038 },
	{ 0x8298, 0x829b, PDF_CMAP_TABLE, 9046 },
	{ 0x829d, 0x82b1, PDF_CMAP_TABLE, 9050 },
	{ 0x82b3, 0x82c0, PDF_CMAP_TABLE, 9071 },
	{ 0x82c2, 0x82c4, PDF_CMAP_TABLE, 9085 },
	{ 0x82ca, 0x82ca, PDF_CMAP_SINGLE, 18266 },
	{ 0x82cf, 0x82d9, PDF_CMAP_TABLE, 9088 },
	{ 0x82db, 0x82dc, PDF_CMAP_TABLE, 9099 },
	{ 0x82de, 0x82e8, PDF_CMAP_TABLE, 9101 },
	{ 0x82ea, 0x8309, PDF_CMAP_TABLE, 9112 },
	{ 0x830b, 0x830d, PDF_CMAP_TABLE, 9144 },
	{ 0x8316, 0x831e, PDF_CMAP_TABLE, 9147 },
	{ 0x8320, 0x8320, PDF_CMAP_SINGLE, 7637 },
	{ 0x8322, 0x8322, PDF_CMAP_SINGLE, 7627 },
	{ 0x8324, 0x832d, PDF_CMAP_TABLE, 9156 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 7639 },
	{ 0x8331, 0x833d, PDF_CMAP_TABLE, 9166 },
	{ 0x833f, 0x8345, PDF_CMAP_TABLE, 9179 },
	{ 0x8347, 0x8354, PDF_CMAP_TABLE, 9186 },
	{ 0x8356, 0x8357, PDF_CMAP_TABLE, 9200 },
	{ 0x8362, 0x8363, PDF_CMAP_TABLE, 9202 },
	{ 0x8366, 0x8366, PDF_CMAP_SINGLE, 15902 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 17765 },
	{ 0x8373, 0x8378, PDF_CMAP_TABLE, 9204 },
	{ 0x837a, 0x837a, PDF_CMAP_SINGLE, 8222 },
	{ 0x837b, 0x837c, PDF_CMAP_RANGE, 2948 },
	{ 0x837d, 0x837f, PDF_CMAP_TABLE, 9210 },
	{ 0x8381, 0x8381, PDF_CMAP_SINGLE, 8227 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 8234 },
	{ 0x8385, 0x839e, PDF_CMAP_TABLE, 9213 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 2946 },
	{ 0x83a2, 0x83ac, PDF_CMAP_TABLE, 9239 },
	{ 0x83ae, 0x83b0, PDF_CMAP_TABLE, 9250 },
	{ 0x83b9, 0x83b9, PDF_CMAP_SINGLE, 17063 },
	{ 0x83bd, 0x83cf, PDF_CMAP_TABLE, 9253 },
	{ 0x83d1, 0x83d1, PDF_CMAP_SINGLE, 8886 },
	{ 0x83d3, 0x83d9, PDF_CMAP_TABLE, 9272 },
	{ 0x83db, 0x83e5, PDF_CMAP_TABLE, 9279 },
	{ 0x83e7, 0x83f6, PDF_CMAP_TABLE, 9290 },
	{ 0x83f8, 0x83ff, PDF_CMAP_TABLE, 9306 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 8865 },
	{ 0x8403, 0x8407, PDF_CMAP_TABLE, 9314 },
	{ 0x8409, 0x8414, PDF_CMAP_TABLE, 9319 },
	{ 0x8416, 0x8416, PDF_CMAP_SINGLE, 16439 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 15994 },
	{ 0x841b, 0x841c, PDF_CMAP_TABLE, 9331 },
	{ 0x8420, 0x8421, PDF_CMAP_TABLE, 9333 },
	{ 0x8423, 0x8424, PDF_CMAP_TABLE, 9335 },
	{ 0x8426, 0x8426, PDF_CMAP_SINGLE, 18136 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 9549 },
	{ 0x842b, 0x8440, PDF_CMAP_TABLE, 9337 },
	{ 0x8442, 0x844e, PDF_CMAP_TABLE, 9359 },
	{ 0x8450, 0x8469, PDF_CMAP_TABLE, 9372 },
	{ 0x846b, 0x847a, PDF_CMAP_TABLE, 9398 },
	{ 0x847d, 0x8480, PDF_CMAP_TABLE, 9414 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 3803 },
	{ 0x8484, 0x8484, PDF_CMAP_SINGLE, 19103 },
	{ 0x8486, 0x8486, PDF_CMAP_SINGLE, 9529 },
	{ 0x8488, 0x8488, PDF_CMAP_SINGLE, 18274 },
	{ 0x848d, 0x84a4, PDF_CMAP_TABLE, 9418 },
	{ 0x84a7, 0x84b2, PDF_CMAP_TABLE, 9442 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 10161 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 10181 },
	{ 0x84b8, 0x84c2, PDF_CMAP_TABLE, 9454 },
	{ 0x84c4, 0x84c7, PDF_CMAP_TABLE, 9465 },
	{ 0x84c9, 0x84d4, PDF_CMAP_TABLE, 9469 },
	{ 0x84d6, 0x84d7, PDF_CMAP_TABLE, 9481 },
	{ 0x84da, 0x84db, PDF_CMAP_TABLE, 9483 },
	{ 0x84de, 0x84de, PDF_CMAP_SINGLE, 14314 },
	{ 0x84e1, 0x84e2, PDF_CMAP_TABLE, 9485 },
	{ 0x84e4, 0x84e5, PDF_CMAP_TABLE, 9487 },
	{ 0x84e7, 0x84e8, PDF_CMAP_RANGE, 10755 },
	{ 0x84e9, 0x84ec, PDF_CMAP_TABLE, 9489 },
	{ 0x84ee, 0x84f4, PDF_CMAP_TABLE, 9493 },
	{ 0x84f6, 0x8500, PDF_CMAP_TABLE, 9500 },
	{ 0x8502, 0x851a, PDF_CMAP_TABLE, 9511 },
	{ 0x851c, 0x8521, PDF_CMAP_TABLE, 9536 },
	{ 0x8523, 0x852b, PDF_CMAP_TABLE, 9542 },
	{ 0x852c, 0x852d, PDF_CMAP_RANGE, 4596 },
	{ 0x852e, 0x8531, PDF_CMAP_TABLE, 9551 },
	{ 0x8533, 0x8534, PDF_CMAP_TABLE, 9555 },
	{ 0x8538, 0x8538, PDF_CMAP_SINGLE, 18280 },
	{ 0x853b, 0x853b, PDF_CMAP_SINGLE, 10717 },
	{ 0x853d, 0x853e, PDF_CMAP_TABLE, 9557 },
	{ 0x8540, 0x854e, PDF_CMAP_TABLE, 9559 },
	{ 0x8551, 0x855b, PDF_CMAP_TABLE, 9574 },
	{ 0x855d, 0x8567, PDF_CMAP_TABLE, 9585 },
	{ 0x8568, 0x8569, PDF_CMAP_RANGE, 4921 },
	{ 0x856a, 0x856e, PDF_CMAP_TABLE, 9596 },
	{ 0x856f, 0x8570, PDF_CMAP_RANGE, 18283 },
	{ 0x8571, 0x8571, PDF_CMAP_SINGLE, 11345 },
	{ 0x8573, 0x8573, PDF_CMAP_SINGLE, 14328 },
	{ 0x8575, 0x857c, PDF_CMAP_TABLE, 9601 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 5212 },
	{ 0x8580, 0x8591, PDF_CMAP_TABLE, 9609 },
	{ 0x8593, 0x85a4, PDF_CMAP_TABLE, 9627 },
	{ 0x85a6, 0x85aa, PDF_CMAP_TABLE, 9645 },
	{ 0x85af, 0x85b1, PDF_CMAP_TABLE, 9650 },
	{ 0x85b3, 0x85ba, PDF_CMAP_TABLE, 9653 },
	{ 0x85bd, 0x85c9, PDF_CMAP_TABLE, 9661 },
	{ 0x85cb, 0x85cb, PDF_CMAP_SINGLE, 12194 },
	{ 0x85cd, 0x85d2, PDF_CMAP_TABLE, 9674 },
	{ 0x85d5, 0x85da, PDF_CMAP_TABLE, 9680 },
	{ 0x85dc, 0x85e3, PDF_CMAP_TABLE, 9686 },
	{ 0x85e4, 0x85e5, PDF_CMAP_RANGE, 5560 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 12547 },
	{ 0x85e8, 0x85f2, PDF_CMAP_TABLE, 9694 },
	{ 0x85f6, 0x8602, PDF_CMAP_TABLE, 9705 },
	{ 0x8604, 0x8607, PDF_CMAP_TABLE, 9718 },
	{ 0x8609, 0x860d, PDF_CMAP_TABLE, 9722 },
	{ 0x860f, 0x8611, PDF_CMAP_TABLE, 9727 },
	{ 0x8613, 0x8614, PDF_CMAP_TABLE, 9730 },
	{ 0x8616, 0x861c, PDF_CMAP_TABLE, 9732 },
	{ 0x861e, 0x862a, PDF_CMAP_TABLE, 9739 },
	{ 0x862c, 0x862f, PDF_CMAP_TABLE, 9752 },
	{ 0x8631, 0x8633, PDF_CMAP_TABLE, 9756 },
	{ 0x8634, 0x8635, PDF_CMAP_RANGE, 13223 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 13228 },
	{ 0x8638, 0x863c, PDF_CMAP_TABLE, 9759 },
	{ 0x863e, 0x8640, PDF_CMAP_TABLE, 9764 },
	{ 0x8642, 0x8643, PDF_CMAP_TABLE, 9767 },
	{ 0x8645, 0x8645, PDF_CMAP_SINGLE, 15206 },
	{ 0x8646, 0x8648, PDF_CMAP_RANGE, 13535 },
	{ 0x864b, 0x864e, PDF_CMAP_TABLE, 9769 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 2047 },
	{ 0x8652, 0x8656, PDF_CMAP_TABLE, 9773 },
	{ 0x8659, 0x8659, PDF_CMAP_SINGLE, 8251 },
	{ 0x865b, 0x865c, PDF_CMAP_TABLE, 9778 },
	{ 0x865e, 0x865f, PDF_CMAP_TABLE, 9780 },
	{ 0x8661, 0x8665, PDF_CMAP_TABLE, 9782 },
	{ 0x8667, 0x866c, PDF_CMAP_TABLE, 9787 },
	{ 0x866d, 0x866e, PDF_CMAP_RANGE, 6726 },
	{ 0x866f, 0x8674, PDF_CMAP_TABLE, 9793 },
	{ 0x8677, 0x8677, PDF_CMAP_SINGLE, 7162 },
	{ 0x8679, 0x867c, PDF_CMAP_TABLE, 9799 },
	{ 0x867e, 0x867e, PDF_CMAP_SINGLE, 14346 },
	{ 0x8685, 0x8687, PDF_CMAP_TABLE, 9803 },
	{ 0x868a, 0x868e, PDF_CMAP_TABLE, 9806 },
	{ 0x8690, 0x869a, PDF_CMAP_TABLE, 9811 },
	{ 0x869c, 0x869e, PDF_CMAP_TABLE, 9822 },
	{ 0x86a0, 0x86a5, PDF_CMAP_TABLE, 9825 },
	{ 0x86a7, 0x86aa, PDF_CMAP_TABLE, 9831 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 16134 },
	{ 0x86af, 0x86ba, PDF_CMAP_TABLE, 9835 },
	{ 0x86bb, 0x86bc, PDF_CMAP_RANGE, 8266 },
	{ 0x86bd, 0x86be, PDF_CMAP_RANGE, 8269 },
	{ 0x86bf, 0x86c9, PDF_CMAP_TABLE, 9847 },
	{ 0x86cb, 0x86cc, PDF_CMAP_TABLE, 9858 },
	{ 0x86d0, 0x86d1, PDF_CMAP_TABLE, 9860 },
	{ 0x86d3, 0x86d4, PDF_CMAP_TABLE, 9862 },
	{ 0x86d6, 0x86df, PDF_CMAP_TABLE, 9864 },
	{ 0x86e2, 0x86e4, PDF_CMAP_TABLE, 9874 },
	{ 0x86e6, 0x86e6, PDF_CMAP_SINGLE, 8902 },
	{ 0x86e8, 0x86ed, PDF_CMAP_TABLE, 9877 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 18289 },
	{ 0x86f5, 0x86fb, PDF_CMAP_TABLE, 9883 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 3828 },
	{ 0x8700, 0x8701, PDF_CMAP_TABLE, 9890 },
	{ 0x8702, 0x8703, PDF_CMAP_RANGE, 3830 },
	{ 0x8704, 0x870e, PDF_CMAP_TABLE, 9892 },
	{ 0x8711, 0x8713, PDF_CMAP_TABLE, 9903 },
	{ 0x8718, 0x871c, PDF_CMAP_TABLE, 9906 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 10197 },
	{ 0x8720, 0x8725, PDF_CMAP_TABLE, 9911 },
	{ 0x8726, 0x8727, PDF_CMAP_RANGE, 10216 },
	{ 0x8728, 0x872a, PDF_CMAP_TABLE, 9917 },
	{ 0x872c, 0x872e, PDF_CMAP_TABLE, 9920 },
	{ 0x8730, 0x8735, PDF_CMAP_TABLE, 9923 },
	{ 0x8737, 0x8738, PDF_CMAP_TABLE, 9929 },
	{ 0x873a, 0x873c, PDF_CMAP_TABLE, 9931 },
	{ 0x873e, 0x8743, PDF_CMAP_TABLE, 9934 },
	{ 0x8746, 0x8746, PDF_CMAP_SINGLE, 10205 },
	{ 0x874c, 0x8762, PDF_CMAP_TABLE, 9940 },
	{ 0x8763, 0x8764, PDF_CMAP_RANGE, 10768 },
	{ 0x8765, 0x8771, PDF_CMAP_TABLE, 9963 },
	{ 0x8773, 0x877b, PDF_CMAP_TABLE, 9976 },
	{ 0x877d, 0x877d, PDF_CMAP_SINGLE, 16453 },
	{ 0x8781, 0x8789, PDF_CMAP_TABLE, 9985 },
	{ 0x878b, 0x878d, PDF_CMAP_TABLE, 9994 },
	{ 0x878f, 0x878f, PDF_CMAP_SINGLE, 11363 },
	{ 0x8790, 0x8791, PDF_CMAP_RANGE, 11375 },
	{ 0x8792, 0x8794, PDF_CMAP_TABLE, 9997 },
	{ 0x8796, 0x8798, PDF_CMAP_TABLE, 10000 },
	{ 0x879a, 0x879f, PDF_CMAP_TABLE, 10003 },
	{ 0x87a2, 0x87a5, PDF_CMAP_TABLE, 10009 },
	{ 0x87a9, 0x87c6, PDF_CMAP_TABLE, 10013 },
	{ 0x87c8, 0x87cc, PDF_CMAP_TABLE, 10043 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 16365 },
	{ 0x87d1, 0x87d4, PDF_CMAP_TABLE, 10048 },
	{ 0x87d6, 0x87e8, PDF_CMAP_TABLE, 10052 },
	{ 0x87ea, 0x87ef, PDF_CMAP_TABLE, 10071 },
	{ 0x87f2, 0x87f5, PDF_CMAP_TABLE, 10077 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 12554 },
	{ 0x87f9, 0x87fc, PDF_CMAP_TABLE, 10081 },
	{ 0x87fe, 0x8806, PDF_CMAP_TABLE, 10085 },
	{ 0x8808, 0x880d, PDF_CMAP_TABLE, 10094 },
	{ 0x880f, 0x880f, PDF_CMAP_SINGLE, 14353 },
	{ 0x8810, 0x8811, PDF_CMAP_RANGE, 12844 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 12847 },
	{ 0x8814, 0x8815, PDF_CMAP_RANGE, 5695 },
	{ 0x8816, 0x8819, PDF_CMAP_TABLE, 10100 },
	{ 0x881b, 0x881d, PDF_CMAP_TABLE, 10104 },
	{ 0x881f, 0x8833, PDF_CMAP_TABLE, 10107 },
	{ 0x8835, 0x8839, PDF_CMAP_TABLE, 10128 },
	{ 0x883b, 0x8842, PDF_CMAP_TABLE, 10133 },
	{ 0x8843, 0x8844, PDF_CMAP_RANGE, 7674 },
	{ 0x8845, 0x8846, PDF_CMAP_RANGE, 14358 },
	{ 0x8848, 0x8848, PDF_CMAP_SINGLE, 8915 },
	{ 0x884a, 0x884f, PDF_CMAP_TABLE, 10141 },
	{ 0x8852, 0x8853, PDF_CMAP_TABLE, 10147 },
	{ 0x8855, 0x8857, PDF_CMAP_TABLE, 10149 },
	{ 0x8859, 0x885b, PDF_CMAP_TABLE, 10152 },
	{ 0x885d, 0x885e, PDF_CMAP_TABLE, 10155 },
	{ 0x8860, 0x8865, PDF_CMAP_TABLE, 10157 },
	{ 0x8867, 0x886b, PDF_CMAP_TABLE, 10163 },
	{ 0x886d, 0x8872, PDF_CMAP_TABLE, 10168 },
	{ 0x8874, 0x8874, PDF_CMAP_SINGLE, 7686 },
	{ 0x8875, 0x8876, PDF_CMAP_RANGE, 7677 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2489 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 2493 },
	{ 0x887c, 0x8880, PDF_CMAP_TABLE, 10174 },
	{ 0x8881, 0x8882, PDF_CMAP_RANGE, 2490 },
	{ 0x8883, 0x8884, PDF_CMAP_TABLE, 10179 },
	{ 0x8887, 0x8889, PDF_CMAP_TABLE, 10181 },
	{ 0x888b, 0x8893, PDF_CMAP_TABLE, 10184 },
	{ 0x8895, 0x88a2, PDF_CMAP_TABLE, 10193 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 8286 },
	{ 0x88a7, 0x88a8, PDF_CMAP_TABLE, 10207 },
	{ 0x88aa, 0x88ac, PDF_CMAP_TABLE, 10209 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 18293 },
	{ 0x88b1, 0x88b2, PDF_CMAP_TABLE, 10212 },
	{ 0x88b4, 0x88b5, PDF_CMAP_RANGE, 14366 },
	{ 0x88b6, 0x88ba, PDF_CMAP_TABLE, 10214 },
	{ 0x88bc, 0x88c0, PDF_CMAP_TABLE, 10219 },
	{ 0x88c1, 0x88c2, PDF_CMAP_RANGE, 3410 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 14368 },
	{ 0x88c7, 0x88c7, PDF_CMAP_SINGLE, 16049 },
	{ 0x88c9, 0x88cc, PDF_CMAP_TABLE, 10224 },
	{ 0x88cd, 0x88ce, PDF_CMAP_RANGE, 9593 },
	{ 0x88cf, 0x88d0, PDF_CMAP_TABLE, 10228 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 3844 },
	{ 0x88d4, 0x88df, PDF_CMAP_TABLE, 10230 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3841 },
	{ 0x88e6, 0x88e8, PDF_CMAP_TABLE, 10242 },
	{ 0x88eb, 0x88ec, PDF_CMAP_TABLE, 10245 },
	{ 0x88ee, 0x88f0, PDF_CMAP_TABLE, 10247 },
	{ 0x88f1, 0x88f2, PDF_CMAP_RANGE, 10225 },
	{ 0x88f3, 0x8902, PDF_CMAP_TABLE, 10250 },
	{ 0x8905, 0x8907, PDF_CMAP_TABLE, 10266 },
	{ 0x8909, 0x890c, PDF_CMAP_TABLE, 10269 },
	{ 0x890e, 0x890e, PDF_CMAP_SINGLE, 10810 },
	{ 0x8910, 0x8911, PDF_CMAP_TABLE, 10273 },
	{ 0x8912, 0x8913, PDF_CMAP_RANGE, 4622 },
	{ 0x8914, 0x8916, PDF_CMAP_TABLE, 10275 },
	{ 0x8917, 0x8919, PDF_CMAP_RANGE, 10804 },
	{ 0x891a, 0x891a, PDF_CMAP_SINGLE, 4259 },
	{ 0x891e, 0x891f, PDF_CMAP_TABLE, 10278 },
	{ 0x8921, 0x8927, PDF_CMAP_TABLE, 10280 },
	{ 0x8929, 0x892c, PDF_CMAP_TABLE, 10287 },
	{ 0x892d, 0x892e, PDF_CMAP_RANGE, 11386 },
	{ 0x892f, 0x8933, PDF_CMAP_TABLE, 10291 },
	{ 0x8935, 0x8938, PDF_CMAP_TABLE, 10296 },
	{ 0x893b, 0x893e, PDF_CMAP_TABLE, 10300 },
	{ 0x8941, 0x8944, PDF_CMAP_TABLE, 10304 },
	{ 0x8946, 0x8947, PDF_CMAP_TABLE, 10308 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 12235 },
	{ 0x894b, 0x894d, PDF_CMAP_TABLE, 10310 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 12230 },
	{ 0x8950, 0x8951, PDF_CMAP_RANGE, 12233 },
	{ 0x8952, 0x8954, PDF_CMAP_TABLE, 10313 },
	{ 0x8956, 0x8959, PDF_CMAP_TABLE, 10316 },
	{ 0x895a, 0x895b, PDF_CMAP_RANGE, 12566 },
	{ 0x895c, 0x8966, PDF_CMAP_TABLE, 10320 },
	{ 0x8969, 0x896f, PDF_CMAP_TABLE, 10331 },
	{ 0x8971, 0x8974, PDF_CMAP_TABLE, 10338 },
	{ 0x8976, 0x8977, PDF_CMAP_TABLE, 10342 },
	{ 0x8979, 0x897a, PDF_CMAP_RANGE, 13538 },
	{ 0x897b, 0x897c, PDF_CMAP_TABLE, 10344 },
	{ 0x897e, 0x8983, PDF_CMAP_TABLE, 10346 },
	{ 0x8985, 0x898b, PDF_CMAP_TABLE, 10352 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 2973 },
	{ 0x8991, 0x8991, PDF_CMAP_SINGLE, 18296 },
	{ 0x8993, 0x8998, PDF_CMAP_TABLE, 10359 },
	{ 0x899b, 0x899f, PDF_CMAP_TABLE, 10365 },
	{ 0x89a1, 0x89a7, PDF_CMAP_TABLE, 10370 },
	{ 0x89a9, 0x89aa, PDF_CMAP_TABLE, 10377 },
	{ 0x89ac, 0x89af, PDF_CMAP_TABLE, 10379 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 5416 },
	{ 0x89b6, 0x89b7, PDF_CMAP_TABLE, 10383 },
	{ 0x89b9, 0x89ba, PDF_CMAP_TABLE, 10385 },
	{ 0x89bc, 0x89c1, PDF_CMAP_TABLE, 10387 },
	{ 0x89c6, 0x89c6, PDF_CMAP_SINGLE, 17766 },
	{ 0x89d2, 0x89d6, PDF_CMAP_TABLE, 10393 },
	{ 0x89d9, 0x89d9, PDF_CMAP_SINGLE, 8293 },
	{ 0x89da, 0x89db, PDF_CMAP_RANGE, 8935 },
	{ 0x89dc, 0x89dd, PDF_CMAP_TABLE, 10398 },
	{ 0x89df, 0x89e9, PDF_CMAP_TABLE, 10400 },
	{ 0x89eb, 0x89ed, PDF_CMAP_TABLE, 10411 },
	{ 0x89f0, 0x89f1, PDF_CMAP_TABLE, 10414 },
	{ 0x89f2, 0x89f3, PDF_CMAP_RANGE, 11865 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 5417 },
	{ 0x89f6, 0x89f8, PDF_CMAP_TABLE, 10416 },
	{ 0x89fa, 0x89fc, PDF_CMAP_TABLE, 10419 },
	{ 0x89fe, 0x8a00, PDF_CMAP_TABLE, 10422 },
	{ 0x8a02, 0x8a03, PDF_CMAP_RANGE, 2056 },
	{ 0x8a04, 0x8a04, PDF_CMAP_SINGLE, 7172 },
	{ 0x8a07, 0x8a08, PDF_CMAP_TABLE, 10425 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2499 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a0e, 0x8a13, PDF_CMAP_TABLE, 10427 },
	{ 0x8a15, 0x8a18, PDF_CMAP_TABLE, 10433 },
	{ 0x8a1b, 0x8a1f, PDF_CMAP_TABLE, 10437 },
	{ 0x8a22, 0x8a23, PDF_CMAP_TABLE, 10442 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 2977 },
	{ 0x8a27, 0x8a27, PDF_CMAP_SINGLE, 8296 },
	{ 0x8a29, 0x8a2d, PDF_CMAP_TABLE, 10444 },
	{ 0x8a30, 0x8a31, PDF_CMAP_TABLE, 10449 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 3425 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 3427 },
	{ 0x8a38, 0x8a41, PDF_CMAP_TABLE, 10451 },
	{ 0x8a44, 0x8a45, PDF_CMAP_RANGE, 8944 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 3424 },
	{ 0x8a48, 0x8a4a, PDF_CMAP_TABLE, 10461 },
	{ 0x8a4c, 0x8a52, PDF_CMAP_TABLE, 10464 },
	{ 0x8a54, 0x8a56, PDF_CMAP_TABLE, 10471 },
	{ 0x8a57, 0x8a58, PDF_CMAP_RANGE, 8942 },
	{ 0x8a59, 0x8a59, PDF_CMAP_SINGLE, 8940 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 3422 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 3418 },
	{ 0x8a60, 0x8a63, PDF_CMAP_TABLE, 10474 },
	{ 0x8a66, 0x8a69, PDF_CMAP_TABLE, 10478 },
	{ 0x8a6b, 0x8a6e, PDF_CMAP_TABLE, 10482 },
	{ 0x8a70, 0x8a71, PDF_CMAP_TABLE, 10486 },
	{ 0x8a72, 0x8a73, PDF_CMAP_RANGE, 3848 },
	{ 0x8a74, 0x8a77, PDF_CMAP_TABLE, 10488 },
	{ 0x8a79, 0x8a7c, PDF_CMAP_TABLE, 10492 },
	{ 0x8a7e, 0x8a7f, PDF_CMAP_TABLE, 10496 },
	{ 0x8a81, 0x8a87, PDF_CMAP_TABLE, 10498 },
	{ 0x8a8b, 0x8a8d, PDF_CMAP_TABLE, 10505 },
	{ 0x8a8f, 0x8a96, PDF_CMAP_TABLE, 10508 },
	{ 0x8a98, 0x8a9a, PDF_CMAP_TABLE, 10516 },
	{ 0x8a9c, 0x8a9c, PDF_CMAP_SINGLE, 14380 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 4263 },
	{ 0x8aa0, 0x8aa1, PDF_CMAP_TABLE, 10519 },
	{ 0x8aa3, 0x8aab, PDF_CMAP_TABLE, 10521 },
	{ 0x8aaf, 0x8ab0, PDF_CMAP_TABLE, 10530 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 4633 },
	{ 0x8ab4, 0x8ab4, PDF_CMAP_SINGLE, 18300 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 4640 },
	{ 0x8ab8, 0x8ac0, PDF_CMAP_TABLE, 10532 },
	{ 0x8ac2, 0x8ac9, PDF_CMAP_TABLE, 10541 },
	{ 0x8acb, 0x8acd, PDF_CMAP_TABLE, 10549 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 10818 },
	{ 0x8ad1, 0x8ad3, PDF_CMAP_TABLE, 10552 },
	{ 0x8ad4, 0x8ad5, PDF_CMAP_RANGE, 10823 },
	{ 0x8ad6, 0x8ae2, PDF_CMAP_TABLE, 10555 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 11404 },
	{ 0x8ae6, 0x8ae8, PDF_CMAP_TABLE, 10568 },
	{ 0x8aea, 0x8aeb, PDF_CMAP_TABLE, 10571 },
	{ 0x8aed, 0x8af3, PDF_CMAP_TABLE, 10573 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 11400 },
	{ 0x8af6, 0x8afc, PDF_CMAP_TABLE, 10580 },
	{ 0x8afe, 0x8b00, PDF_CMAP_TABLE, 10587 },
	{ 0x8b01, 0x8b02, PDF_CMAP_RANGE, 4950 },
	{ 0x8b04, 0x8b08, PDF_CMAP_TABLE, 10590 },
	{ 0x8b0a, 0x8b20, PDF_CMAP_TABLE, 10595 },
	{ 0x8b22, 0x8b28, PDF_CMAP_TABLE, 10618 },
	{ 0x8b2a, 0x8b31, PDF_CMAP_TABLE, 10625 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 12239 },
	{ 0x8b35, 0x8b37, PDF_CMAP_TABLE, 10633 },
	{ 0x8b39, 0x8b43, PDF_CMAP_TABLE, 10636 },
	{ 0x8b45, 0x8b4d, PDF_CMAP_TABLE, 10647 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_RANGE, 5577 },
	{ 0x8b50, 0x8b5a, PDF_CMAP_TABLE, 10656 },
	{ 0x8b5c, 0x8b60, PDF_CMAP_TABLE, 10667 },
	{ 0x8b62, 0x8b63, PDF_CMAP_TABLE, 10672 },
	{ 0x8b65, 0x8b6d, PDF_CMAP_TABLE, 10674 },
	{ 0x8b6f, 0x8b70, PDF_CMAP_TABLE, 10683 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 5775 },
	{ 0x8b77, 0x8b79, PDF_CMAP_TABLE, 10685 },
	{ 0x8b7a, 0x8b7b, PDF_CMAP_RANGE, 13076 },
	{ 0x8b7d, 0x8b86, PDF_CMAP_TABLE, 10688 },
	{ 0x8b88, 0x8b88, PDF_CMAP_SINGLE, 13376 },
	{ 0x8b8a, 0x8b8c, PDF_CMAP_TABLE, 10698 },
	{ 0x8b8e, 0x8b90, PDF_CMAP_TABLE, 10701 },
	{ 0x8b92, 0x8b93, PDF_CMAP_TABLE, 10704 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 13468 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 5933 },
	{ 0x8b98, 0x8b99, PDF_CMAP_RANGE, 13543 },
	{ 0x8b9a, 0x8b9c, PDF_CMAP_TABLE, 10706 },
	{ 0x8b9e, 0x8ba0, PDF_CMAP_TABLE, 10709 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 17767 },
	{ 0x8be2, 0x8be2, PDF_CMAP_SINGLE, 17768 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1291 },
	{ 0x8c39, 0x8c39, PDF_CMAP_SINGLE, 8299 },
	{ 0x8c3b, 0x8c3f, PDF_CMAP_TABLE, 10712 },
	{ 0x8c41, 0x8c43, PDF_CMAP_TABLE, 10717 },
	{ 0x8c45, 0x8c51, PDF_CMAP_TABLE, 10720 },
	{ 0x8c54, 0x8c57, PDF_CMAP_TABLE, 10733 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 2984 },
	{ 0x8c5c, 0x8c5d, PDF_CMAP_RANGE, 8301 },
	{ 0x8c5f, 0x8c5f, PDF_CMAP_SINGLE, 8952 },
	{ 0x8c61, 0x8c62, PDF_CMAP_TABLE, 10737 },
	{ 0x8c64, 0x8c66, PDF_CMAP_TABLE, 10739 },
	{ 0x8c68, 0x8c69, PDF_CMAP_RANGE, 10250 },
	{ 0x8c6a, 0x8c6d, PDF_CMAP_TABLE, 10742 },
	{ 0x8c6f, 0x8c73, PDF_CMAP_TABLE, 10746 },
	{ 0x8c75, 0x8c7b, PDF_CMAP_TABLE, 10751 },
	{ 0x8c7d, 0x8c7d, PDF_CMAP_SINGLE, 8303 },
	{ 0x8c80, 0x8c82, PDF_CMAP_TABLE, 10758 },
	{ 0x8c84, 0x8c85, PDF_CMAP_RANGE, 9630 },
	{ 0x8c86, 0x8c86, PDF_CMAP_SINGLE, 9629 },
	{ 0x8c89, 0x8c8a, PDF_CMAP_TABLE, 10761 },
	{ 0x8c8c, 0x8c8d, PDF_CMAP_TABLE, 10763 },
	{ 0x8c8f, 0x8c90, PDF_CMAP_TABLE, 10765 },
	{ 0x8c91, 0x8c92, PDF_CMAP_RANGE, 11414 },
	{ 0x8c93, 0x8c95, PDF_CMAP_TABLE, 10767 },
	{ 0x8c97, 0x8ca0, PDF_CMAP_TABLE, 10770 },
	{ 0x8ca1, 0x8ca2, PDF_CMAP_RANGE, 2508 },
	{ 0x8ca3, 0x8ca5, PDF_CMAP_TABLE, 10780 },
	{ 0x8ca7, 0x8cad, PDF_CMAP_TABLE, 10783 },
	{ 0x8caf, 0x8cb0, PDF_CMAP_TABLE, 10790 },
	{ 0x8cb2, 0x8cc5, PDF_CMAP_TABLE, 10792 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_RANGE, 3871 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 3870 },
	{ 0x8ccc, 0x8ccd, PDF_CMAP_TABLE, 10812 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 10253 },
	{ 0x8cd1, 0x8cd2, PDF_CMAP_RANGE, 4280 },
	{ 0x8cd3, 0x8cd7, PDF_CMAP_TABLE, 10814 },
	{ 0x8cd9, 0x8ce1, PDF_CMAP_TABLE, 10819 },
	{ 0x8ce2, 0x8ce3, PDF_CMAP_RANGE, 4652 },
	{ 0x8ce4, 0x8ceb, PDF_CMAP_TABLE, 10828 },
	{ 0x8cec, 0x8ced, PDF_CMAP_RANGE, 4650 },
	{ 0x8cee, 0x8cee, PDF_CMAP_SINGLE, 11418 },
	{ 0x8cf0, 0x8cf5, PDF_CMAP_TABLE, 10836 },
	{ 0x8cf7, 0x8cfe, PDF_CMAP_TABLE, 10842 },
	{ 0x8d00, 0x8d00, PDF_CMAP_SINGLE, 12264 },
	{ 0x8d02, 0x8d05, PDF_CMAP_TABLE, 10850 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 12595 },
	{ 0x8d08, 0x8d0d, PDF_CMAP_TABLE, 10854 },
	{ 0x8d0f, 0x8d15, PDF_CMAP_TABLE, 10860 },
	{ 0x8d16, 0x8d17, PDF_CMAP_RANGE, 5849 },
	{ 0x8d18, 0x8d19, PDF_CMAP_TABLE, 10867 },
	{ 0x8d1b, 0x8d1d, PDF_CMAP_TABLE, 10869 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 1295 },
	{ 0x8d66, 0x8d67, PDF_CMAP_TABLE, 10872 },
	{ 0x8d68, 0x8d69, PDF_CMAP_RANGE, 9633 },
	{ 0x8d6b, 0x8d70, PDF_CMAP_TABLE, 10874 },
	{ 0x8d72, 0x8d74, PDF_CMAP_TABLE, 10880 },
	{ 0x8d76, 0x8d7b, PDF_CMAP_TABLE, 10883 },
	{ 0x8d7d, 0x8d7d, PDF_CMAP_SINGLE, 8305 },
	{ 0x8d80, 0x8d82, PDF_CMAP_TABLE, 10889 },
	{ 0x8d84, 0x8d85, PDF_CMAP_TABLE, 10892 },
	{ 0x8d89, 0x8d8a, PDF_CMAP_TABLE, 10894 },
	{ 0x8d8c, 0x8d8d, PDF_CMAP_TABLE, 10896 },
	{ 0x8d8e, 0x8d8f, PDF_CMAP_RANGE, 9637 },
	{ 0x8d90, 0x8d92, PDF_CMAP_TABLE, 10898 },
	{ 0x8d93, 0x8d94, PDF_CMAP_RANGE, 9640 },
	{ 0x8d95, 0x8d96, PDF_CMAP_TABLE, 10901 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 4283 },
	{ 0x8d9b, 0x8d9c, PDF_CMAP_TABLE, 10903 },
	{ 0x8d9f, 0x8da1, PDF_CMAP_TABLE, 10905 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 4659 },
	{ 0x8da5, 0x8daf, PDF_CMAP_TABLE, 10908 },
	{ 0x8db2, 0x8db7, PDF_CMAP_TABLE, 10919 },
	{ 0x8db9, 0x8dba, PDF_CMAP_TABLE, 10925 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 8308 },
	{ 0x8dbe, 0x8dc3, PDF_CMAP_TABLE, 10927 },
	{ 0x8dc5, 0x8dc8, PDF_CMAP_TABLE, 10933 },
	{ 0x8dcb, 0x8dd1, PDF_CMAP_TABLE, 10937 },
	{ 0x8dd3, 0x8ddd, PDF_CMAP_TABLE, 10944 },
	{ 0x8ddf, 0x8de4, PDF_CMAP_TABLE, 10955 },
	{ 0x8de6, 0x8dec, PDF_CMAP_TABLE, 10961 },
	{ 0x8dee, 0x8df4, PDF_CMAP_TABLE, 10968 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 3883 },
	{ 0x8dfc, 0x8e07, PDF_CMAP_TABLE, 10975 },
	{ 0x8e09, 0x8e0a, PDF_CMAP_TABLE, 10987 },
	{ 0x8e0d, 0x8e25, PDF_CMAP_TABLE, 10989 },
	{ 0x8e26, 0x8e27, PDF_CMAP_RANGE, 10859 },
	{ 0x8e28, 0x8e2b, PDF_CMAP_TABLE, 11014 },
	{ 0x8e2d, 0x8e2e, PDF_CMAP_TABLE, 11018 },
	{ 0x8e30, 0x8e31, PDF_CMAP_TABLE, 11020 },
	{ 0x8e33, 0x8e36, PDF_CMAP_TABLE, 11022 },
	{ 0x8e38, 0x8e3a, PDF_CMAP_TABLE, 11026 },
	{ 0x8e3c, 0x8e3d, PDF_CMAP_RANGE, 11432 },
	{ 0x8e3e, 0x8e42, PDF_CMAP_TABLE, 11029 },
	{ 0x8e44, 0x8e50, PDF_CMAP_TABLE, 11034 },
	{ 0x8e53, 0x8e57, PDF_CMAP_TABLE, 11047 },
	{ 0x8e59, 0x8e6a, PDF_CMAP_TABLE, 11052 },
	{ 0x8e6c, 0x8e6d, PDF_CMAP_TABLE, 11070 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 12606 },
	{ 0x8e71, 0x8e78, PDF_CMAP_TABLE, 11072 },
	{ 0x8e7a, 0x8e7c, PDF_CMAP_TABLE, 11080 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 17188 },
	{ 0x8e80, 0x8e82, PDF_CMAP_TABLE, 11083 },
	{ 0x8e84, 0x8e8e, PDF_CMAP_TABLE, 11086 },
	{ 0x8e90, 0x8e95, PDF_CMAP_TABLE, 11097 },
	{ 0x8e96, 0x8e97, PDF_CMAP_RANGE, 13250 },
	{ 0x8e98, 0x8e98, PDF_CMAP_SINGLE, 13379 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 13247 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 13473 },
	{ 0x8e9e, 0x8ea0, PDF_CMAP_RANGE, 13470 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 5961 },
	{ 0x8ea3, 0x8ead, PDF_CMAP_TABLE, 11103 },
	{ 0x8eb0, 0x8eb0, PDF_CMAP_SINGLE, 17385 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 3887 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14431 },
	{ 0x8eb9, 0x8eba, PDF_CMAP_TABLE, 11114 },
	{ 0x8ebc, 0x8ebd, PDF_CMAP_TABLE, 11116 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 5430 },
	{ 0x8ec2, 0x8ec3, PDF_CMAP_TABLE, 11118 },
	{ 0x8ec9, 0x8ecf, PDF_CMAP_TABLE, 11120 },
	{ 0x8ed1, 0x8ed4, PDF_CMAP_TABLE, 11127 },
	{ 0x8ed7, 0x8ed8, PDF_CMAP_TABLE, 11131 },
	{ 0x8eda, 0x8edf, PDF_CMAP_TABLE, 11133 },
	{ 0x8ee0, 0x8ee1, PDF_CMAP_RANGE, 8318 },
	{ 0x8ee2, 0x8ee2, PDF_CMAP_SINGLE, 18313 },
	{ 0x8ee4, 0x8ee6, PDF_CMAP_TABLE, 11139 },
	{ 0x8ee7, 0x8ee8, PDF_CMAP_RANGE, 8983 },
	{ 0x8ee9, 0x8ee9, PDF_CMAP_SINGLE, 8990 },
	{ 0x8eeb, 0x8eef, PDF_CMAP_TABLE, 11142 },
	{ 0x8ef1, 0x8ef2, PDF_CMAP_TABLE, 11147 },
	{ 0x8ef4, 0x8efc, PDF_CMAP_TABLE, 11149 },
	{ 0x8efe, 0x8f03, PDF_CMAP_TABLE, 11158 },
	{ 0x8f05, 0x8f06, PDF_CMAP_TABLE, 11164 },
	{ 0x8f07, 0x8f08, PDF_CMAP_RANGE, 9662 },
	{ 0x8f09, 0x8f0b, PDF_CMAP_TABLE, 11166 },
	{ 0x8f0d, 0x8f0e, PDF_CMAP_TABLE, 11169 },
	{ 0x8f10, 0x8f11, PDF_CMAP_RANGE, 10269 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 11171 },
	{ 0x8f16, 0x8f17, PDF_CMAP_RANGE, 10874 },
	{ 0x8f18, 0x8f20, PDF_CMAP_TABLE, 11175 },
	{ 0x8f23, 0x8f26, PDF_CMAP_TABLE, 11184 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 11188 },
	{ 0x8f2c, 0x8f30, PDF_CMAP_TABLE, 11190 },
	{ 0x8f32, 0x8f39, PDF_CMAP_TABLE, 11195 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 4967 },
	{ 0x8f3e, 0x8f45, PDF_CMAP_TABLE, 11203 },
	{ 0x8f46, 0x8f48, PDF_CMAP_RANGE, 12279 },
	{ 0x8f49, 0x8f4b, PDF_CMAP_TABLE, 11211 },
	{ 0x8f4d, 0x8f4e, PDF_CMAP_TABLE, 11214 },
	{ 0x8f4f, 0x8f50, PDF_CMAP_RANGE, 12611 },
	{ 0x8f51, 0x8f55, PDF_CMAP_TABLE, 11216 },
	{ 0x8f56, 0x8f57, PDF_CMAP_RANGE, 12866 },
	{ 0x8f58, 0x8f64, PDF_CMAP_TABLE, 11221 },
	{ 0x8f66, 0x8f67, PDF_CMAP_RANGE, 17769 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 17771 },
	{ 0x8f93, 0x8f93, PDF_CMAP_SINGLE, 18226 },
	{ 0x8f9b, 0x8f9c, PDF_CMAP_TABLE, 11234 },
	{ 0x8f9f, 0x8fa0, PDF_CMAP_TABLE, 11236 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 4290 },
	{ 0x8fa5, 0x8fa8, PDF_CMAP_TABLE, 11238 },
	{ 0x8fad, 0x8fba, PDF_CMAP_TABLE, 11242 },
	{ 0x8fbb, 0x8fbc, PDF_CMAP_RANGE, 15723 },
	{ 0x8fbe, 0x8fbf, PDF_CMAP_TABLE, 11256 },
	{ 0x8fc1, 0x8fc2, PDF_CMAP_TABLE, 11258 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 11260 },
	{ 0x8fc9, 0x8fd7, PDF_CMAP_TABLE, 11263 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 16331 },
	{ 0x8fe0, 0x8fe6, PDF_CMAP_TABLE, 11278 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 2073 },
	{ 0x8fea, 0x8feb, PDF_CMAP_TABLE, 11285 },
	{ 0x8fed, 0x8fee, PDF_CMAP_TABLE, 11287 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 2065 },
	{ 0x8ff4, 0x9006, PDF_CMAP_TABLE, 11289 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 16096 },
	{ 0x900b, 0x900d, PDF_CMAP_TABLE, 11308 },
	{ 0x900f, 0x9012, PDF_CMAP_TABLE, 11311 },
	{ 0x9014, 0x9017, PDF_CMAP_TABLE, 11315 },
	{ 0x9019, 0x9024, PDF_CMAP_TABLE, 11319 },
	{ 0x902d, 0x902f, PDF_CMAP_TABLE, 11331 },
	{ 0x9031, 0x9038, PDF_CMAP_TABLE, 11334 },
	{ 0x903c, 0x903f, PDF_CMAP_TABLE, 11342 },
	{ 0x9041, 0x9042, PDF_CMAP_TABLE, 11346 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 9668 },
	{ 0x9046, 0x9047, PDF_CMAP_TABLE, 11348 },
	{ 0x9049, 0x9056, PDF_CMAP_TABLE, 11350 },
	{ 0x9058, 0x9059, PDF_CMAP_TABLE, 11364 },
	{ 0x905b, 0x905e, PDF_CMAP_TABLE, 11366 },
	{ 0x9060, 0x9064, PDF_CMAP_TABLE, 11370 },
	{ 0x9067, 0x9069, PDF_CMAP_TABLE, 11375 },
	{ 0x906b, 0x9070, PDF_CMAP_TABLE, 11378 },
	{ 0x9072, 0x9080, PDF_CMAP_TABLE, 11384 },
	{ 0x9081, 0x9082, PDF_CMAP_RANGE, 5269 },
	{ 0x9083, 0x9088, PDF_CMAP_TABLE, 11399 },
	{ 0x908a, 0x908b, PDF_CMAP_RANGE, 5593 },
	{ 0x908d, 0x908d, PDF_CMAP_SINGLE, 12871 },
	{ 0x908f, 0x9091, PDF_CMAP_TABLE, 11405 },
	{ 0x9094, 0x9095, PDF_CMAP_TABLE, 11408 },
	{ 0x9097, 0x9098, PDF_CMAP_RANGE, 6171 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 6170 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 6173 },
	{ 0x909e, 0x90a3, PDF_CMAP_TABLE, 11410 },
	{ 0x90a5, 0x90a8, PDF_CMAP_TABLE, 11416 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 1309 },
	{ 0x90ae, 0x90b6, PDF_CMAP_TABLE, 11420 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 1682 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 15300 },
	{ 0x90bd, 0x90bf, PDF_CMAP_TABLE, 11429 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 2076 },
	{ 0x90c3, 0x90c5, PDF_CMAP_TABLE, 11432 },
	{ 0x90c7, 0x90c8, PDF_CMAP_TABLE, 11435 },
	{ 0x90ca, 0x90cb, PDF_CMAP_TABLE, 11437 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 2075 },
	{ 0x90d4, 0x90d8, PDF_CMAP_TABLE, 11439 },
	{ 0x90d9, 0x90da, PDF_CMAP_RANGE, 7711 },
	{ 0x90db, 0x90dd, PDF_CMAP_TABLE, 11444 },
	{ 0x90df, 0x90e5, PDF_CMAP_TABLE, 11447 },
	{ 0x90e8, 0x90ea, PDF_CMAP_TABLE, 11454 },
	{ 0x90eb, 0x90ec, PDF_CMAP_RANGE, 8333 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 3014 },
	{ 0x90ef, 0x90f1, PDF_CMAP_TABLE, 11457 },
	{ 0x90f2, 0x90f3, PDF_CMAP_RANGE, 8330 },
	{ 0x90f4, 0x90f5, PDF_CMAP_TABLE, 11460 },
	{ 0x90f9, 0x9109, PDF_CMAP_TABLE, 11462 },
	{ 0x910b, 0x910b, PDF_CMAP_SINGLE, 9677 },
	{ 0x910d, 0x9112, PDF_CMAP_TABLE, 11479 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 9676 },
	{ 0x9116, 0x9124, PDF_CMAP_TABLE, 11485 },
	{ 0x9126, 0x9128, PDF_CMAP_TABLE, 11500 },
	{ 0x9129, 0x912a, PDF_CMAP_RANGE, 10883 },
	{ 0x912b, 0x9134, PDF_CMAP_TABLE, 11503 },
	{ 0x9135, 0x9136, PDF_CMAP_RANGE, 11451 },
	{ 0x9138, 0x9139, PDF_CMAP_TABLE, 11513 },
	{ 0x913a, 0x913b, PDF_CMAP_RANGE, 12284 },
	{ 0x913e, 0x9141, PDF_CMAP_TABLE, 11515 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 12872 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 13087 },
	{ 0x9146, 0x914d, PDF_CMAP_TABLE, 11519 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7722 },
	{ 0x9150, 0x9153, PDF_CMAP_TABLE, 11527 },
	{ 0x9155, 0x915a, PDF_CMAP_TABLE, 11531 },
	{ 0x915c, 0x915c, PDF_CMAP_SINGLE, 16199 },
	{ 0x915e, 0x9165, PDF_CMAP_TABLE, 11537 },
	{ 0x9167, 0x916a, PDF_CMAP_TABLE, 11545 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 3911 },
	{ 0x916e, 0x916f, PDF_CMAP_RANGE, 9679 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 16056 },
	{ 0x9172, 0x917a, PDF_CMAP_TABLE, 11549 },
	{ 0x917c, 0x917c, PDF_CMAP_SINGLE, 16239 },
	{ 0x9180, 0x9180, PDF_CMAP_SINGLE, 10894 },
	{ 0x9181, 0x9182, PDF_CMAP_RANGE, 10891 },
	{ 0x9183, 0x9184, PDF_CMAP_TABLE, 11558 },
	{ 0x9185, 0x9186, PDF_CMAP_RANGE, 10888 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 4688 },
	{ 0x9189, 0x918f, PDF_CMAP_TABLE, 11560 },
	{ 0x9190, 0x9191, PDF_CMAP_RANGE, 11454 },
	{ 0x9192, 0x9193, PDF_CMAP_TABLE, 11567 },
	{ 0x9199, 0x91a3, PDF_CMAP_TABLE, 11569 },
	{ 0x91a5, 0x91a5, PDF_CMAP_SINGLE, 12288 },
	{ 0x91a7, 0x91aa, PDF_CMAP_TABLE, 11580 },
	{ 0x91ab, 0x91ac, PDF_CMAP_RANGE, 5436 },
	{ 0x91ad, 0x91b1, PDF_CMAP_TABLE, 11584 },
	{ 0x91b2, 0x91b3, PDF_CMAP_RANGE, 12876 },
	{ 0x91b4, 0x91b7, PDF_CMAP_TABLE, 11589 },
	{ 0x91b9, 0x91be, PDF_CMAP_TABLE, 11593 },
	{ 0x91c0, 0x91c7, PDF_CMAP_TABLE, 11599 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 3914 },
	{ 0x91cb, 0x91d1, PDF_CMAP_TABLE, 11607 },
	{ 0x91d3, 0x91da, PDF_CMAP_TABLE, 11614 },
	{ 0x91dc, 0x91dd, PDF_CMAP_TABLE, 11622 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 17722 },
	{ 0x91e2, 0x91e9, PDF_CMAP_TABLE, 11624 },
	{ 0x91ea, 0x91eb, PDF_CMAP_RANGE, 8348 },
	{ 0x91ec, 0x91ee, PDF_CMAP_TABLE, 11632 },
	{ 0x91f1, 0x91f1, PDF_CMAP_SINGLE, 8343 },
	{ 0x91f3, 0x91fa, PDF_CMAP_TABLE, 11635 },
	{ 0x91fd, 0x920a, PDF_CMAP_TABLE, 11643 },
	{ 0x920c, 0x9213, PDF_CMAP_TABLE, 11657 },
	{ 0x9214, 0x9215, PDF_CMAP_RANGE, 3471 },
	{ 0x9216, 0x921a, PDF_CMAP_TABLE, 11665 },
	{ 0x921c, 0x921c, PDF_CMAP_SINGLE, 9028 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 3475 },
	{ 0x9221, 0x9221, PDF_CMAP_SINGLE, 18940 },
	{ 0x9223, 0x9228, PDF_CMAP_TABLE, 11670 },
	{ 0x922a, 0x922b, PDF_CMAP_TABLE, 11676 },
	{ 0x922d, 0x922e, PDF_CMAP_TABLE, 11678 },
	{ 0x9230, 0x923a, PDF_CMAP_TABLE, 11680 },
	{ 0x923c, 0x9241, PDF_CMAP_TABLE, 11691 },
	{ 0x9244, 0x9246, PDF_CMAP_TABLE, 11697 },
	{ 0x9248, 0x9258, PDF_CMAP_TABLE, 11700 },
	{ 0x925a, 0x925b, PDF_CMAP_TABLE, 11717 },
	{ 0x925d, 0x9267, PDF_CMAP_TABLE, 11719 },
	{ 0x926b, 0x9270, PDF_CMAP_TABLE, 11730 },
	{ 0x9272, 0x9272, PDF_CMAP_SINGLE, 9706 },
	{ 0x9276, 0x928f, PDF_CMAP_TABLE, 11736 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 4318 },
	{ 0x9293, 0x929d, PDF_CMAP_TABLE, 11762 },
	{ 0x92a0, 0x92ac, PDF_CMAP_TABLE, 11773 },
	{ 0x92ae, 0x92ae, PDF_CMAP_SINGLE, 18329 },
	{ 0x92b1, 0x92b7, PDF_CMAP_TABLE, 11786 },
	{ 0x92b9, 0x92bc, PDF_CMAP_TABLE, 11793 },
	{ 0x92be, 0x92c2, PDF_CMAP_TABLE, 11797 },
	{ 0x92c3, 0x92c4, PDF_CMAP_RANGE, 10896 },
	{ 0x92c5, 0x92d5, PDF_CMAP_TABLE, 11802 },
	{ 0x92d7, 0x92d9, PDF_CMAP_TABLE, 11819 },
	{ 0x92db, 0x92db, PDF_CMAP_SINGLE, 16396 },
	{ 0x92dd, 0x92e1, PDF_CMAP_TABLE, 11822 },
	{ 0x92e3, 0x92ec, PDF_CMAP_TABLE, 11827 },
	{ 0x92ee, 0x92f2, PDF_CMAP_TABLE, 11837 },
	{ 0x92f3, 0x92f4, PDF_CMAP_RANGE, 18333 },
	{ 0x92f6, 0x9304, PDF_CMAP_TABLE, 11842 },
	{ 0x9306, 0x9309, PDF_CMAP_TABLE, 11857 },
	{ 0x930b, 0x9310, PDF_CMAP_TABLE, 11861 },
	{ 0x9312, 0x9316, PDF_CMAP_TABLE, 11867 },
	{ 0x9318, 0x931b, PDF_CMAP_TABLE, 11872 },
	{ 0x931d, 0x9331, PDF_CMAP_TABLE, 11876 },
	{ 0x9333, 0x9336, PDF_CMAP_TABLE, 11897 },
	{ 0x9338, 0x9339, PDF_CMAP_TABLE, 11901 },
	{ 0x933c, 0x933c, PDF_CMAP_SINGLE, 11466 },
	{ 0x9340, 0x934f, PDF_CMAP_TABLE, 11903 },
	{ 0x9350, 0x9351, PDF_CMAP_RANGE, 11923 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 11933 },
	{ 0x9354, 0x935c, PDF_CMAP_TABLE, 11919 },
	{ 0x935e, 0x936e, PDF_CMAP_TABLE, 11928 },
	{ 0x9370, 0x9371, PDF_CMAP_TABLE, 11945 },
	{ 0x9373, 0x937e, PDF_CMAP_TABLE, 11947 },
	{ 0x9380, 0x938a, PDF_CMAP_TABLE, 11959 },
	{ 0x938c, 0x9392, PDF_CMAP_TABLE, 11970 },
	{ 0x9394, 0x93aa, PDF_CMAP_TABLE, 11977 },
	{ 0x93ac, 0x93b5, PDF_CMAP_TABLE, 12000 },
	{ 0x93b7, 0x93b8, PDF_CMAP_TABLE, 12010 },
	{ 0x93ba, 0x93bb, PDF_CMAP_TABLE, 12012 },
	{ 0x93bd, 0x93bd, PDF_CMAP_SINGLE, 14887 },
	{ 0x93bf, 0x93c0, PDF_CMAP_TABLE, 12014 },
	{ 0x93c2, 0x93c4, PDF_CMAP_TABLE, 12016 },
	{ 0x93c6, 0x93c8, PDF_CMAP_TABLE, 12019 },
	{ 0x93ca, 0x93db, PDF_CMAP_TABLE, 12022 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 5602 },
	{ 0x93de, 0x93e4, PDF_CMAP_TABLE, 12040 },
	{ 0x93e6, 0x93e8, PDF_CMAP_TABLE, 12047 },
	{ 0x93ec, 0x93ec, PDF_CMAP_SINGLE, 12626 },
	{ 0x93ee, 0x93ee, PDF_CMAP_SINGLE, 12633 },
	{ 0x93f0, 0x93f1, PDF_CMAP_TABLE, 12050 },
	{ 0x93f3, 0x9401, PDF_CMAP_TABLE, 12052 },
	{ 0x9403, 0x9404, PDF_CMAP_TABLE, 12067 },
	{ 0x9406, 0x9419, PDF_CMAP_TABLE, 12069 },
	{ 0x941b, 0x941b, PDF_CMAP_SINGLE, 16712 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 18342 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 12881 },
	{ 0x9424, 0x9433, PDF_CMAP_TABLE, 12089 },
	{ 0x9435, 0x9440, PDF_CMAP_TABLE, 12105 },
	{ 0x9442, 0x9447, PDF_CMAP_TABLE, 12117 },
	{ 0x9448, 0x9449, PDF_CMAP_RANGE, 13262 },
	{ 0x944a, 0x944b, PDF_CMAP_RANGE, 13257 },
	{ 0x944c, 0x944d, PDF_CMAP_TABLE, 12123 },
	{ 0x944f, 0x9450, PDF_CMAP_TABLE, 12125 },
	{ 0x9451, 0x9452, PDF_CMAP_RANGE, 5856 },
	{ 0x9454, 0x9455, PDF_CMAP_TABLE, 12127 },
	{ 0x9457, 0x9458, PDF_CMAP_TABLE, 12129 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 14495 },
	{ 0x945d, 0x945e, PDF_CMAP_TABLE, 12131 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 5902 },
	{ 0x9462, 0x9465, PDF_CMAP_TABLE, 12133 },
	{ 0x9467, 0x9467, PDF_CMAP_SINGLE, 15032 },
	{ 0x9468, 0x9469, PDF_CMAP_RANGE, 13478 },
	{ 0x946a, 0x9479, PDF_CMAP_TABLE, 12137 },
	{ 0x947b, 0x947c, PDF_CMAP_TABLE, 12153 },
	{ 0x947d, 0x947e, PDF_CMAP_RANGE, 5981 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 5988 },
	{ 0x9480, 0x9481, PDF_CMAP_RANGE, 13625 },
	{ 0x9482, 0x9483, PDF_CMAP_TABLE, 12155 },
	{ 0x9485, 0x9485, PDF_CMAP_SINGLE, 17671 },
	{ 0x949f, 0x949f, PDF_CMAP_SINGLE, 18565 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 17691 },
	{ 0x94c1, 0x94c1, PDF_CMAP_SINGLE, 18401 },
	{ 0x94c3, 0x94c3, PDF_CMAP_SINGLE, 18399 },
	{ 0x94dc, 0x94dc, PDF_CMAP_SINGLE, 18391 },
	{ 0x94f6, 0x94f6, PDF_CMAP_SINGLE, 18386 },
	{ 0x952d, 0x952d, PDF_CMAP_SINGLE, 18348 },
	{ 0x9547, 0x9547, PDF_CMAP_SINGLE, 18623 },
	{ 0x9577, 0x9578, PDF_CMAP_TABLE, 12157 },
	{ 0x957a, 0x957d, PDF_CMAP_TABLE, 12159 },
	{ 0x957f, 0x9580, PDF_CMAP_TABLE, 12163 },
	{ 0x9582, 0x9583, PDF_CMAP_TABLE, 12165 },
	{ 0x9585, 0x9586, PDF_CMAP_TABLE, 12167 },
	{ 0x9588, 0x9589, PDF_CMAP_TABLE, 12169 },
	{ 0x958b, 0x9594, PDF_CMAP_TABLE, 12171 },
	{ 0x9596, 0x9599, PDF_CMAP_TABLE, 12181 },
	{ 0x959b, 0x959c, PDF_CMAP_TABLE, 12185 },
	{ 0x959e, 0x95a7, PDF_CMAP_TABLE, 12187 },
	{ 0x95a8, 0x95a9, PDF_CMAP_RANGE, 4320 },
	{ 0x95aa, 0x95ae, PDF_CMAP_TABLE, 12197 },
	{ 0x95b0, 0x95b1, PDF_CMAP_TABLE, 12202 },
	{ 0x95b5, 0x95b7, PDF_CMAP_TABLE, 12204 },
	{ 0x95b9, 0x95ba, PDF_CMAP_RANGE, 11500 },
	{ 0x95bb, 0x95c0, PDF_CMAP_TABLE, 12207 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 11947 },
	{ 0x95c5, 0x95c9, PDF_CMAP_TABLE, 12213 },
	{ 0x95ca, 0x95cc, PDF_CMAP_RANGE, 5290 },
	{ 0x95cd, 0x95cd, PDF_CMAP_SINGLE, 11498 },
	{ 0x95d0, 0x95d1, PDF_CMAP_TABLE, 12218 },
	{ 0x95d2, 0x95d3, PDF_CMAP_RANGE, 12317 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 12220 },
	{ 0x95da, 0x95db, PDF_CMAP_RANGE, 12643 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 5611 },
	{ 0x95de, 0x95e5, PDF_CMAP_TABLE, 12223 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 17674 },
	{ 0x95f4, 0x95f4, PDF_CMAP_SINGLE, 18351 },
	{ 0x961c, 0x961e, PDF_CMAP_TABLE, 12231 },
	{ 0x9620, 0x9624, PDF_CMAP_TABLE, 12234 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 6385 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 1318 },
	{ 0x962c, 0x9633, PDF_CMAP_TABLE, 12239 },
	{ 0x9638, 0x963d, PDF_CMAP_TABLE, 12247 },
	{ 0x963f, 0x9645, PDF_CMAP_TABLE, 12253 },
	{ 0x964a, 0x964a, PDF_CMAP_SINGLE, 7194 },
	{ 0x964b, 0x964d, PDF_CMAP_RANGE, 2083 },
	{ 0x964e, 0x9651, PDF_CMAP_TABLE, 12260 },
	{ 0x9653, 0x9654, PDF_CMAP_TABLE, 12264 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 15847 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 2545 },
	{ 0x965b, 0x965f, PDF_CMAP_TABLE, 12266 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 2541 },
	{ 0x9662, 0x9663, PDF_CMAP_RANGE, 2539 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 2544 },
	{ 0x9669, 0x966d, PDF_CMAP_TABLE, 12271 },
	{ 0x966f, 0x9675, PDF_CMAP_TABLE, 12276 },
	{ 0x9676, 0x9677, PDF_CMAP_RANGE, 3031 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 3028 },
	{ 0x967b, 0x967e, PDF_CMAP_TABLE, 12283 },
	{ 0x9680, 0x9681, PDF_CMAP_TABLE, 12287 },
	{ 0x9683, 0x9684, PDF_CMAP_TABLE, 12289 },
	{ 0x9685, 0x9686, PDF_CMAP_RANGE, 3491 },
	{ 0x9687, 0x9687, PDF_CMAP_SINGLE, 9038 },
	{ 0x9688, 0x9689, PDF_CMAP_RANGE, 9040 },
	{ 0x968a, 0x968b, PDF_CMAP_TABLE, 12291 },
	{ 0x968d, 0x968f, PDF_CMAP_TABLE, 12293 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 9718 },
	{ 0x9692, 0x9693, PDF_CMAP_RANGE, 9716 },
	{ 0x9694, 0x9695, PDF_CMAP_RANGE, 3934 },
	{ 0x9696, 0x9699, PDF_CMAP_TABLE, 12296 },
	{ 0x969b, 0x969c, PDF_CMAP_TABLE, 12300 },
	{ 0x969e, 0x969e, PDF_CMAP_SINGLE, 10322 },
	{ 0x96a1, 0x96a5, PDF_CMAP_TABLE, 12302 },
	{ 0x96a7, 0x96a8, PDF_CMAP_RANGE, 4998 },
	{ 0x96a9, 0x96aa, PDF_CMAP_TABLE, 12307 },
	{ 0x96ac, 0x96ac, PDF_CMAP_SINGLE, 11952 },
	{ 0x96ae, 0x96ae, PDF_CMAP_SINGLE, 11950 },
	{ 0x96b0, 0x96b1, PDF_CMAP_TABLE, 12309 },
	{ 0x96b3, 0x96b4, PDF_CMAP_TABLE, 12311 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 561 },
	{ 0x96b8, 0x96b9, PDF_CMAP_TABLE, 12313 },
	{ 0x96bb, 0x96bd, PDF_CMAP_TABLE, 12315 },
	{ 0x96bf, 0x96c5, PDF_CMAP_TABLE, 12318 },
	{ 0x96c6, 0x96c7, PDF_CMAP_RANGE, 3499 },
	{ 0x96c8, 0x96c8, PDF_CMAP_SINGLE, 9045 },
	{ 0x96c9, 0x96ca, PDF_CMAP_RANGE, 3938 },
	{ 0x96cb, 0x96ce, PDF_CMAP_TABLE, 12325 },
	{ 0x96d2, 0x96df, PDF_CMAP_TABLE, 12329 },
	{ 0x96e1, 0x96e3, PDF_CMAP_TABLE, 12343 },
	{ 0x96e5, 0x96e5, PDF_CMAP_SINGLE, 13480 },
	{ 0x96e8, 0x96ea, PDF_CMAP_TABLE, 12346 },
	{ 0x96ef, 0x96f2, PDF_CMAP_TABLE, 12349 },
	{ 0x96f4, 0x96fb, PDF_CMAP_TABLE, 12353 },
	{ 0x96fd, 0x96fd, PDF_CMAP_SINGLE, 9722 },
	{ 0x96ff, 0x9700, PDF_CMAP_TABLE, 12361 },
	{ 0x9702, 0x9705, PDF_CMAP_TABLE, 12363 },
	{ 0x9706, 0x9707, PDF_CMAP_RANGE, 4707 },
	{ 0x9708, 0x9709, PDF_CMAP_TABLE, 12367 },
	{ 0x970b, 0x970b, PDF_CMAP_SINGLE, 11508 },
	{ 0x970d, 0x9713, PDF_CMAP_TABLE, 12369 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 5004 },
	{ 0x9718, 0x9719, PDF_CMAP_TABLE, 12376 },
	{ 0x971b, 0x972a, PDF_CMAP_TABLE, 12378 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 12647 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 12911 },
	{ 0x9730, 0x9732, PDF_CMAP_TABLE, 12394 },
	{ 0x9734, 0x9736, PDF_CMAP_TABLE, 12397 },
	{ 0x9738, 0x9739, PDF_CMAP_RANGE, 5793 },
	{ 0x973a, 0x973a, PDF_CMAP_SINGLE, 13106 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 5858 },
	{ 0x973f, 0x9744, PDF_CMAP_TABLE, 12400 },
	{ 0x9746, 0x974b, PDF_CMAP_TABLE, 12406 },
	{ 0x9751, 0x9752, PDF_CMAP_TABLE, 12412 },
	{ 0x9755, 0x9758, PDF_CMAP_TABLE, 12414 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 10940 },
	{ 0x975b, 0x975c, PDF_CMAP_RANGE, 5008 },
	{ 0x975d, 0x9762, PDF_CMAP_TABLE, 12418 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 5010 },
	{ 0x9768, 0x976a, PDF_CMAP_TABLE, 12424 },
	{ 0x976c, 0x976e, PDF_CMAP_TABLE, 12427 },
	{ 0x9770, 0x9774, PDF_CMAP_TABLE, 12430 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 3946 },
	{ 0x9777, 0x9778, PDF_CMAP_RANGE, 9726 },
	{ 0x977a, 0x9785, PDF_CMAP_TABLE, 12435 },
	{ 0x9787, 0x978b, PDF_CMAP_TABLE, 12447 },
	{ 0x978d, 0x978f, PDF_CMAP_TABLE, 12452 },
	{ 0x9794, 0x9794, PDF_CMAP_SINGLE, 11513 },
	{ 0x9797, 0x97a6, PDF_CMAP_TABLE, 12455 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 12332 },
	{ 0x97aa, 0x97ae, PDF_CMAP_TABLE, 12471 },
	{ 0x97b1, 0x97b4, PDF_CMAP_TABLE, 12476 },
	{ 0x97b6, 0x97bb, PDF_CMAP_TABLE, 12480 },
	{ 0x97bd, 0x97c3, PDF_CMAP_TABLE, 12486 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 13388 },
	{ 0x97c6, 0x97c9, PDF_CMAP_TABLE, 12493 },
	{ 0x97cb, 0x97d0, PDF_CMAP_TABLE, 12497 },
	{ 0x97d2, 0x97d9, PDF_CMAP_TABLE, 12503 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 5617 },
	{ 0x97dd, 0x97df, PDF_CMAP_RANGE, 12654 },
	{ 0x97e0, 0x97e1, PDF_CMAP_TABLE, 12511 },
	{ 0x97e3, 0x97e3, PDF_CMAP_SINGLE, 13266 },
	{ 0x97e5, 0x97e6, PDF_CMAP_TABLE, 12513 },
	{ 0x97ed, 0x97ee, PDF_CMAP_TABLE, 12515 },
	{ 0x97f0, 0x97f3, PDF_CMAP_TABLE, 12517 },
	{ 0x97f5, 0x97f6, PDF_CMAP_TABLE, 12521 },
	{ 0x97f8, 0x97fb, PDF_CMAP_TABLE, 12523 },
	{ 0x97fd, 0x97fe, PDF_CMAP_RANGE, 12915 },
	{ 0x97ff, 0x9801, PDF_CMAP_TABLE, 12527 },
	{ 0x9802, 0x9803, PDF_CMAP_RANGE, 3039 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 8363 },
	{ 0x9805, 0x9806, PDF_CMAP_RANGE, 3504 },
	{ 0x9807, 0x9808, PDF_CMAP_TABLE, 12530 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 3950 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 3952 },
	{ 0x980d, 0x980e, PDF_CMAP_RANGE, 9730 },
	{ 0x980f, 0x980f, PDF_CMAP_SINGLE, 9729 },
	{ 0x9810, 0x9811, PDF_CMAP_RANGE, 3947 },
	{ 0x9812, 0x9816, PDF_CMAP_TABLE, 12532 },
	{ 0x9817, 0x9818, PDF_CMAP_RANGE, 4334 },
	{ 0x981b, 0x9821, PDF_CMAP_TABLE, 12537 },
	{ 0x9823, 0x9824, PDF_CMAP_TABLE, 12544 },
	{ 0x9826, 0x9829, PDF_CMAP_TABLE, 12546 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 4715 },
	{ 0x982d, 0x9830, PDF_CMAP_TABLE, 12550 },
	{ 0x9832, 0x9835, PDF_CMAP_TABLE, 12554 },
	{ 0x9837, 0x9839, PDF_CMAP_TABLE, 12558 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 5014 },
	{ 0x9841, 0x9841, PDF_CMAP_SINGLE, 11966 },
	{ 0x9843, 0x984f, PDF_CMAP_TABLE, 12561 },
	{ 0x9850, 0x9852, PDF_CMAP_RANGE, 12343 },
	{ 0x9853, 0x9853, PDF_CMAP_SINGLE, 5468 },
	{ 0x9856, 0x9859, PDF_CMAP_TABLE, 12574 },
	{ 0x985b, 0x9860, PDF_CMAP_TABLE, 12578 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 12918 },
	{ 0x9864, 0x986c, PDF_CMAP_TABLE, 12584 },
	{ 0x986f, 0x9872, PDF_CMAP_TABLE, 12593 },
	{ 0x9873, 0x9874, PDF_CMAP_RANGE, 13607 },
	{ 0x9875, 0x9875, PDF_CMAP_SINGLE, 17677 },
	{ 0x98a8, 0x98a9, PDF_CMAP_TABLE, 12597 },
	{ 0x98ac, 0x98ac, PDF_CMAP_SINGLE, 9732 },
	{ 0x98ad, 0x98ae, PDF_CMAP_RANGE, 10339 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 4336 },
	{ 0x98b1, 0x98b4, PDF_CMAP_TABLE, 12599 },
	{ 0x98b6, 0x98c0, PDF_CMAP_TABLE, 12603 },
	{ 0x98c1, 0x98c2, PDF_CMAP_RANGE, 12921 },
	{ 0x98c3, 0x98c4, PDF_CMAP_TABLE, 12614 },
	{ 0x98c6, 0x98cc, PDF_CMAP_TABLE, 12616 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 17678 },
	{ 0x98db, 0x98dc, PDF_CMAP_TABLE, 12623 },
	{ 0x98de, 0x98e3, PDF_CMAP_TABLE, 12625 },
	{ 0x98e5, 0x98e7, PDF_CMAP_TABLE, 12631 },
	{ 0x98e9, 0x98ed, PDF_CMAP_TABLE, 12634 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 3509 },
	{ 0x98f1, 0x98f2, PDF_CMAP_TABLE, 12639 },
	{ 0x98f4, 0x98f6, PDF_CMAP_TABLE, 12641 },
	{ 0x98f9, 0x98fa, PDF_CMAP_TABLE, 12644 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 3953 },
	{ 0x98fd, 0x98fe, PDF_CMAP_RANGE, 3955 },
	{ 0x9900, 0x9900, PDF_CMAP_SINGLE, 10342 },
	{ 0x9902, 0x9903, PDF_CMAP_TABLE, 12646 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 4339 },
	{ 0x9907, 0x990a, PDF_CMAP_TABLE, 12648 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 4340 },
	{ 0x990e, 0x990e, PDF_CMAP_SINGLE, 17389 },
	{ 0x9910, 0x9915, PDF_CMAP_TABLE, 12652 },
	{ 0x9916, 0x9917, PDF_CMAP_RANGE, 10959 },
	{ 0x9918, 0x991c, PDF_CMAP_TABLE, 12658 },
	{ 0x991e, 0x991f, PDF_CMAP_TABLE, 12663 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 5023 },
	{ 0x9924, 0x9925, PDF_CMAP_TABLE, 12665 },
	{ 0x9927, 0x992a, PDF_CMAP_TABLE, 12667 },
	{ 0x992b, 0x992c, PDF_CMAP_RANGE, 11973 },
	{ 0x992d, 0x9933, PDF_CMAP_TABLE, 12671 },
	{ 0x9935, 0x9935, PDF_CMAP_SINGLE, 5304 },
	{ 0x9937, 0x993d, PDF_CMAP_TABLE, 12678 },
	{ 0x993e, 0x993f, PDF_CMAP_RANGE, 5470 },
	{ 0x9940, 0x9943, PDF_CMAP_TABLE, 12685 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 5623 },
	{ 0x9947, 0x994e, PDF_CMAP_TABLE, 12689 },
	{ 0x9950, 0x9959, PDF_CMAP_TABLE, 12697 },
	{ 0x995b, 0x995f, PDF_CMAP_TABLE, 12707 },
	{ 0x9961, 0x9963, PDF_CMAP_TABLE, 12712 },
	{ 0x9996, 0x9999, PDF_CMAP_TABLE, 12715 },
	{ 0x999b, 0x999e, PDF_CMAP_TABLE, 12719 },
	{ 0x99a1, 0x99a1, PDF_CMAP_SINGLE, 11984 },
	{ 0x99a3, 0x99a5, PDF_CMAP_TABLE, 12723 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 12668 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5722 },
	{ 0x99aa, 0x99b5, PDF_CMAP_TABLE, 12726 },
	{ 0x99b8, 0x99bd, PDF_CMAP_TABLE, 12738 },
	{ 0x99c1, 0x99c5, PDF_CMAP_TABLE, 12744 },
	{ 0x99c7, 0x99c7, PDF_CMAP_SINGLE, 10352 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 10968 },
	{ 0x99cb, 0x99d2, PDF_CMAP_TABLE, 12749 },
	{ 0x99d3, 0x99d4, PDF_CMAP_RANGE, 10965 },
	{ 0x99d5, 0x99dd, PDF_CMAP_TABLE, 12757 },
	{ 0x99df, 0x99e7, PDF_CMAP_TABLE, 12766 },
	{ 0x99e9, 0x99ea, PDF_CMAP_TABLE, 12775 },
	{ 0x99ec, 0x99ee, PDF_CMAP_TABLE, 12777 },
	{ 0x99f0, 0x99f1, PDF_CMAP_TABLE, 12780 },
	{ 0x99f4, 0x99fc, PDF_CMAP_TABLE, 12782 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 11993 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 5306 },
	{ 0x9a01, 0x9a07, PDF_CMAP_TABLE, 12791 },
	{ 0x9a09, 0x9a11, PDF_CMAP_TABLE, 12798 },
	{ 0x9a14, 0x9a16, PDF_CMAP_TABLE, 12807 },
	{ 0x9a19, 0x9a27, PDF_CMAP_TABLE, 12810 },
	{ 0x9a29, 0x9a32, PDF_CMAP_TABLE, 12825 },
	{ 0x9a34, 0x9a46, PDF_CMAP_TABLE, 12835 },
	{ 0x9a48, 0x9a4a, PDF_CMAP_TABLE, 12854 },
	{ 0x9a4c, 0x9a50, PDF_CMAP_TABLE, 12857 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 13280 },
	{ 0x9a53, 0x9a54, PDF_CMAP_RANGE, 13273 },
	{ 0x9a55, 0x9a59, PDF_CMAP_TABLE, 12862 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_RANGE, 5907 },
	{ 0x9a5c, 0x9a5c, PDF_CMAP_SINGLE, 15384 },
	{ 0x9a5e, 0x9a60, PDF_CMAP_TABLE, 12867 },
	{ 0x9a62, 0x9a65, PDF_CMAP_TABLE, 12870 },
	{ 0x9a66, 0x9a67, PDF_CMAP_RANGE, 13613 },
	{ 0x9a68, 0x9a6c, PDF_CMAP_TABLE, 12874 },
	{ 0x9a8f, 0x9a8f, PDF_CMAP_SINGLE, 17798 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2550 },
	{ 0x9aab, 0x9aab, PDF_CMAP_SINGLE, 9740 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 9739 },
	{ 0x9aaf, 0x9ab0, PDF_CMAP_RANGE, 4343 },
	{ 0x9ab1, 0x9ab4, PDF_CMAP_TABLE, 12879 },
	{ 0x9ab6, 0x9ac2, PDF_CMAP_TABLE, 12883 },
	{ 0x9ac6, 0x9ac7, PDF_CMAP_TABLE, 12896 },
	{ 0x9aca, 0x9aca, PDF_CMAP_SINGLE, 12940 },
	{ 0x9acd, 0x9acd, PDF_CMAP_SINGLE, 13123 },
	{ 0x9acf, 0x9ad2, PDF_CMAP_TABLE, 12898 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 5910 },
	{ 0x9ad5, 0x9ad8, PDF_CMAP_TABLE, 12902 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 12361 },
	{ 0x9adf, 0x9ae3, PDF_CMAP_TABLE, 12906 },
	{ 0x9ae6, 0x9ae7, PDF_CMAP_TABLE, 12911 },
	{ 0x9aeb, 0x9aed, PDF_CMAP_TABLE, 12913 },
	{ 0x9aee, 0x9aef, PDF_CMAP_RANGE, 4731 },
	{ 0x9af1, 0x9af4, PDF_CMAP_TABLE, 12916 },
	{ 0x9af6, 0x9af7, PDF_CMAP_TABLE, 12920 },
	{ 0x9af9, 0x9aff, PDF_CMAP_TABLE, 12922 },
	{ 0x9b01, 0x9b03, PDF_CMAP_TABLE, 12929 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 12363 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 5478 },
	{ 0x9b08, 0x9b12, PDF_CMAP_TABLE, 12932 },
	{ 0x9b14, 0x9b16, PDF_CMAP_TABLE, 12943 },
	{ 0x9b17, 0x9b18, PDF_CMAP_RANGE, 13125 },
	{ 0x9b19, 0x9b1a, PDF_CMAP_TABLE, 12946 },
	{ 0x9b1e, 0x9b20, PDF_CMAP_RANGE, 13393 },
	{ 0x9b22, 0x9b25, PDF_CMAP_TABLE, 12948 },
	{ 0x9b27, 0x9b2b, PDF_CMAP_TABLE, 12952 },
	{ 0x9b2d, 0x9b2f, PDF_CMAP_TABLE, 12957 },
	{ 0x9b31, 0x9b35, PDF_CMAP_TABLE, 12960 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 12688 },
	{ 0x9b39, 0x9b3c, PDF_CMAP_TABLE, 12965 },
	{ 0x9b3e, 0x9b3f, PDF_CMAP_RANGE, 10356 },
	{ 0x9b40, 0x9b40, PDF_CMAP_SINGLE, 14613 },
	{ 0x9b41, 0x9b42, PDF_CMAP_RANGE, 4346 },
	{ 0x9b43, 0x9b46, PDF_CMAP_TABLE, 12969 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 12002 },
	{ 0x9b4a, 0x9b52, PDF_CMAP_TABLE, 12973 },
	{ 0x9b54, 0x9b56, PDF_CMAP_TABLE, 12982 },
	{ 0x9b58, 0x9b5b, PDF_CMAP_TABLE, 12985 },
	{ 0x9b5f, 0x9b5f, PDF_CMAP_SINGLE, 10360 },
	{ 0x9b60, 0x9b61, PDF_CMAP_RANGE, 10358 },
	{ 0x9b64, 0x9b64, PDF_CMAP_SINGLE, 10990 },
	{ 0x9b66, 0x9b69, PDF_CMAP_TABLE, 12989 },
	{ 0x9b6c, 0x9b6c, PDF_CMAP_SINGLE, 10991 },
	{ 0x9b6f, 0x9b71, PDF_CMAP_TABLE, 12993 },
	{ 0x9b74, 0x9b77, PDF_CMAP_TABLE, 12996 },
	{ 0x9b7a, 0x9b83, PDF_CMAP_TABLE, 13000 },
	{ 0x9b85, 0x9b88, PDF_CMAP_TABLE, 13010 },
	{ 0x9b8b, 0x9b8b, PDF_CMAP_SINGLE, 16110 },
	{ 0x9b8d, 0x9b93, PDF_CMAP_TABLE, 13014 },
	{ 0x9b95, 0x9b95, PDF_CMAP_SINGLE, 11553 },
	{ 0x9b97, 0x9b97, PDF_CMAP_SINGLE, 16241 },
	{ 0x9b9a, 0x9b9b, PDF_CMAP_TABLE, 13021 },
	{ 0x9b9d, 0x9ba2, PDF_CMAP_TABLE, 13023 },
	{ 0x9ba4, 0x9ba6, PDF_CMAP_TABLE, 13029 },
	{ 0x9ba8, 0x9ba8, PDF_CMAP_SINGLE, 12004 },
	{ 0x9baa, 0x9bab, PDF_CMAP_TABLE, 13032 },
	{ 0x9bad, 0x9bb0, PDF_CMAP_TABLE, 13034 },
	{ 0x9bb5, 0x9bb6, PDF_CMAP_TABLE, 13038 },
	{ 0x9bb8, 0x9bb9, PDF_CMAP_TABLE, 13040 },
	{ 0x9bbd, 0x9bbd, PDF_CMAP_SINGLE, 12381 },
	{ 0x9bbf, 0x9bc1, PDF_CMAP_TABLE, 13042 },
	{ 0x9bc3, 0x9bc4, PDF_CMAP_TABLE, 13045 },
	{ 0x9bc6, 0x9bca, PDF_CMAP_TABLE, 13047 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 15420 },
	{ 0x9bd3, 0x9bd7, PDF_CMAP_TABLE, 13052 },
	{ 0x9bd9, 0x9bde, PDF_CMAP_TABLE, 13057 },
	{ 0x9be0, 0x9be2, PDF_CMAP_TABLE, 13063 },
	{ 0x9be4, 0x9be9, PDF_CMAP_TABLE, 13066 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 12689 },
	{ 0x9bec, 0x9bed, PDF_CMAP_TABLE, 13072 },
	{ 0x9bf0, 0x9bf1, PDF_CMAP_TABLE, 13074 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 18376 },
	{ 0x9bf7, 0x9bf8, PDF_CMAP_TABLE, 13076 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5484 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 14619 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 14618 },
	{ 0x9c05, 0x9c0e, PDF_CMAP_TABLE, 13078 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 15418 },
	{ 0x9c12, 0x9c15, PDF_CMAP_TABLE, 13088 },
	{ 0x9c17, 0x9c17, PDF_CMAP_SINGLE, 12955 },
	{ 0x9c1b, 0x9c1d, PDF_CMAP_TABLE, 13092 },
	{ 0x9c1f, 0x9c21, PDF_CMAP_TABLE, 13095 },
	{ 0x9c23, 0x9c26, PDF_CMAP_TABLE, 13098 },
	{ 0x9c28, 0x9c29, PDF_CMAP_RANGE, 13135 },
	{ 0x9c2b, 0x9c2f, PDF_CMAP_TABLE, 13102 },
	{ 0x9c31, 0x9c37, PDF_CMAP_TABLE, 13107 },
	{ 0x9c39, 0x9c41, PDF_CMAP_TABLE, 13114 },
	{ 0x9c44, 0x9c46, PDF_CMAP_TABLE, 13123 },
	{ 0x9c48, 0x9c50, PDF_CMAP_TABLE, 13126 },
	{ 0x9c52, 0x9c59, PDF_CMAP_TABLE, 13135 },
	{ 0x9c5d, 0x9c60, PDF_CMAP_TABLE, 13143 },
	{ 0x9c62, 0x9c63, PDF_CMAP_TABLE, 13147 },
	{ 0x9c66, 0x9c68, PDF_CMAP_TABLE, 13149 },
	{ 0x9c6d, 0x9c6e, PDF_CMAP_TABLE, 13152 },
	{ 0x9c71, 0x9c75, PDF_CMAP_TABLE, 13154 },
	{ 0x9c77, 0x9c78, PDF_CMAP_RANGE, 5984 },
	{ 0x9c79, 0x9c7c, PDF_CMAP_TABLE, 13159 },
	{ 0x9ce5, 0x9ce7, PDF_CMAP_TABLE, 13163 },
	{ 0x9ce9, 0x9cea, PDF_CMAP_TABLE, 13166 },
	{ 0x9ced, 0x9ced, PDF_CMAP_SINGLE, 9743 },
	{ 0x9cf1, 0x9cf2, PDF_CMAP_RANGE, 10361 },
	{ 0x9cf3, 0x9cf7, PDF_CMAP_TABLE, 13168 },
	{ 0x9cf9, 0x9cfd, PDF_CMAP_TABLE, 13173 },
	{ 0x9cff, 0x9d00, PDF_CMAP_TABLE, 13178 },
	{ 0x9d02, 0x9d09, PDF_CMAP_TABLE, 13180 },
	{ 0x9d0c, 0x9d0c, PDF_CMAP_SINGLE, 15430 },
	{ 0x9d10, 0x9d10, PDF_CMAP_SINGLE, 11565 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 5038 },
	{ 0x9d14, 0x9d19, PDF_CMAP_TABLE, 13188 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 5039 },
	{ 0x9d1d, 0x9d23, PDF_CMAP_TABLE, 13194 },
	{ 0x9d25, 0x9d26, PDF_CMAP_TABLE, 13201 },
	{ 0x9d28, 0x9d29, PDF_CMAP_TABLE, 13203 },
	{ 0x9d2d, 0x9d2d, PDF_CMAP_SINGLE, 12032 },
	{ 0x9d2e, 0x9d2f, PDF_CMAP_RANGE, 12019 },
	{ 0x9d30, 0x9d31, PDF_CMAP_TABLE, 13205 },
	{ 0x9d33, 0x9d34, PDF_CMAP_TABLE, 13207 },
	{ 0x9d36, 0x9d39, PDF_CMAP_TABLE, 13209 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 5311 },
	{ 0x9d3d, 0x9d41, PDF_CMAP_TABLE, 13213 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 12025 },
	{ 0x9d44, 0x9d45, PDF_CMAP_TABLE, 13218 },
	{ 0x9d49, 0x9d4c, PDF_CMAP_TABLE, 13220 },
	{ 0x9d4e, 0x9d54, PDF_CMAP_TABLE, 13224 },
	{ 0x9d56, 0x9d61, PDF_CMAP_TABLE, 13231 },
	{ 0x9d67, 0x9d73, PDF_CMAP_TABLE, 13243 },
	{ 0x9d74, 0x9d75, PDF_CMAP_RANGE, 12719 },
	{ 0x9d77, 0x9d79, PDF_CMAP_TABLE, 13256 },
	{ 0x9d7b, 0x9d8a, PDF_CMAP_TABLE, 13259 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 12715 },
	{ 0x9d90, 0x9d90, PDF_CMAP_SINGLE, 12964 },
	{ 0x9d92, 0x9d94, PDF_CMAP_TABLE, 13275 },
	{ 0x9d96, 0x9da5, PDF_CMAP_TABLE, 13278 },
	{ 0x9da6, 0x9da7, PDF_CMAP_RANGE, 12980 },
	{ 0x9da8, 0x9dad, PDF_CMAP_TABLE, 13294 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 5809 },
	{ 0x9db1, 0x9db8, PDF_CMAP_TABLE, 13300 },
	{ 0x9db9, 0x9dba, PDF_CMAP_RANGE, 13152 },
	{ 0x9dbb, 0x9dc5, PDF_CMAP_TABLE, 13308 },
	{ 0x9dc7, 0x9ddf, PDF_CMAP_TABLE, 13319 },
	{ 0x9de1, 0x9de6, PDF_CMAP_TABLE, 13344 },
	{ 0x9de8, 0x9de9, PDF_CMAP_TABLE, 13350 },
	{ 0x9deb, 0x9df0, PDF_CMAP_TABLE, 13352 },
	{ 0x9df2, 0x9df8, PDF_CMAP_TABLE, 13358 },
	{ 0x9df9, 0x9dfa, PDF_CMAP_RANGE, 5947 },
	{ 0x9dfb, 0x9dff, PDF_CMAP_TABLE, 13365 },
	{ 0x9e00, 0x9e01, PDF_CMAP_RANGE, 13501 },
	{ 0x9e02, 0x9e07, PDF_CMAP_TABLE, 13370 },
	{ 0x9e09, 0x9e15, PDF_CMAP_TABLE, 13376 },
	{ 0x9e17, 0x9e1f, PDF_CMAP_TABLE, 13389 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 3043 },
	{ 0x9e79, 0x9e7b, PDF_CMAP_TABLE, 13398 },
	{ 0x9e7c, 0x9e7d, PDF_CMAP_RANGE, 5949 },
	{ 0x9e7f, 0x9e85, PDF_CMAP_TABLE, 13401 },
	{ 0x9e86, 0x9e87, PDF_CMAP_RANGE, 11569 },
	{ 0x9e88, 0x9e8e, PDF_CMAP_TABLE, 13408 },
	{ 0x9e90, 0x9ea2, PDF_CMAP_TABLE, 13415 },
	{ 0x9ea4, 0x9eb1, PDF_CMAP_TABLE, 13434 },
	{ 0x9eb4, 0x9eb7, PDF_CMAP_TABLE, 13448 },
	{ 0x9ebb, 0x9ec4, PDF_CMAP_TABLE, 13452 },
	{ 0x9ec6, 0x9ec8, PDF_CMAP_TABLE, 13462 },
	{ 0x9ecc, 0x9ed1, PDF_CMAP_TABLE, 13465 },
	{ 0x9ed3, 0x9ed4, PDF_CMAP_TABLE, 13471 },
	{ 0x9ed5, 0x9ed6, PDF_CMAP_RANGE, 11573 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 5040 },
	{ 0x9eda, 0x9edb, PDF_CMAP_TABLE, 13473 },
	{ 0x9edc, 0x9edd, PDF_CMAP_RANGE, 5316 },
	{ 0x9ede, 0x9ee0, PDF_CMAP_TABLE, 13475 },
	{ 0x9ee2, 0x9ee2, PDF_CMAP_SINGLE, 18392 },
	{ 0x9ee4, 0x9ee8, PDF_CMAP_TABLE, 13478 },
	{ 0x9eeb, 0x9eeb, PDF_CMAP_SINGLE, 13164 },
	{ 0x9eed, 0x9ef1, PDF_CMAP_TABLE, 13483 },
	{ 0x9ef2, 0x9ef3, PDF_CMAP_RANGE, 13427 },
	{ 0x9ef4, 0x9f02, PDF_CMAP_TABLE, 13488 },
	{ 0x9f06, 0x9f0a, PDF_CMAP_TABLE, 13503 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3963 },
	{ 0x9f0f, 0x9f10, PDF_CMAP_RANGE, 11006 },
	{ 0x9f12, 0x9f13, PDF_CMAP_TABLE, 13508 },
	{ 0x9f15, 0x9f1c, PDF_CMAP_TABLE, 13510 },
	{ 0x9f1e, 0x9f1e, PDF_CMAP_SINGLE, 13508 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 3965 },
	{ 0x9f22, 0x9f39, PDF_CMAP_TABLE, 13518 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 4352 },
	{ 0x9f3d, 0x9f3e, PDF_CMAP_TABLE, 13542 },
	{ 0x9f40, 0x9f41, PDF_CMAP_RANGE, 12737 },
	{ 0x9f42, 0x9f50, PDF_CMAP_TABLE, 13544 },
	{ 0x9f52, 0x9f55, PDF_CMAP_TABLE, 13559 },
	{ 0x9f56, 0x9f58, PDF_CMAP_RANGE, 12740 },
	{ 0x9f59, 0x9f65, PDF_CMAP_TABLE, 13563 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 5817 },
	{ 0x9f69, 0x9f6c, PDF_CMAP_TABLE, 13576 },
	{ 0x9f6e, 0x9f6f, PDF_CMAP_RANGE, 13438 },
	{ 0x9f70, 0x9f72, PDF_CMAP_TABLE, 13580 },
	{ 0x9f74, 0x9f76, PDF_CMAP_RANGE, 13510 },
	{ 0x9f77, 0x9f7b, PDF_CMAP_TABLE, 13583 },
	{ 0x9f7e, 0x9f7f, PDF_CMAP_TABLE, 13588 },
	{ 0x9f8d, 0x9f8e, PDF_CMAP_TABLE, 13590 },
	{ 0x9f90, 0x9f92, PDF_CMAP_TABLE, 13592 },
	{ 0x9f94, 0x9f99, PDF_CMAP_TABLE, 13595 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 5043 },
	{ 0x9f9f, 0x9fa0, PDF_CMAP_TABLE, 13601 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 13325 },
	{ 0x9fa4, 0x9fac, PDF_CMAP_TABLE, 13603 },
	{ 0x9fad, 0x9fae, PDF_CMAP_RANGE, 18960 },
	{ 0x9faf, 0x9fb3, PDF_CMAP_TABLE, 13612 },
	{ 0x9fc7, 0x9fcb, PDF_CMAP_TABLE, 13617 },
	{ 0xdc00, 0xdc02, PDF_CMAP_TABLE, 13622 },
	{ 0xdc02, 0xdc03, PDF_CMAP_TABLE, 13625 },
	{ 0xdc03, 0xdc04, PDF_CMAP_TABLE, 13627 },
	{ 0xdc04, 0xdc04, PDF_CMAP_SINGLE, 17033 },
	{ 0xdc06, 0xdc06, PDF_CMAP_SINGLE, 18531 },
	{ 0xdc06, 0xdc06, PDF_CMAP_SINGLE, 15843 },
	{ 0xdc09, 0xdc09, PDF_CMAP_SINGLE, 15286 },
	{ 0xdc09, 0xdc09, PDF_CMAP_SINGLE, 18576 },
	{ 0xdc09, 0xdc09, PDF_CMAP_SINGLE, 18598 },
	{ 0xdc0b, 0xdc0b, PDF_CMAP_SINGLE, 17848 },
	{ 0xdc0d, 0xdc0e, PDF_CMAP_TABLE, 13629 },
	{ 0xdc10, 0xdc10, PDF_CMAP_SINGLE, 14522 },
	{ 0xdc10, 0xdc11, PDF_CMAP_TABLE, 13631 },
	{ 0xdc11, 0xdc12, PDF_CMAP_TABLE, 13633 },
	{ 0xdc12, 0xdc12, PDF_CMAP_SINGLE, 15196 },
	{ 0xdc12, 0xdc12, PDF_CMAP_SINGLE, 15825 },
	{ 0xdc12, 0xdc13, PDF_CMAP_TABLE, 13635 },
	{ 0xdc13, 0xdc14, PDF_CMAP_TABLE, 13637 },
	{ 0xdc14, 0xdc14, PDF_CMAP_SINGLE, 14424 },
	{ 0xdc14, 0xdc14, PDF_CMAP_SINGLE, 18095 },
	{ 0xdc16, 0xdc16, PDF_CMAP_SINGLE, 16592 },
	{ 0xdc19, 0xdc19, PDF_CMAP_SINGLE, 18519 },
	{ 0xdc1c, 0xdc1c, PDF_CMAP_SINGLE, 17860 },
	{ 0xdc1c, 0xdc1d, PDF_CMAP_TABLE, 13639 },
	{ 0xdc1d, 0xdc1d, PDF_CMAP_SINGLE, 14561 },
	{ 0xdc1d, 0xdc1e, PDF_CMAP_RANGE, 17924 },
	{ 0xdc1e, 0xdc20, PDF_CMAP_TABLE, 13641 },
	{ 0xdc20, 0xdc20, PDF_CMAP_SINGLE, 15886 },
	{ 0xdc20, 0xdc21, PDF_CMAP_TABLE, 13644 },
	{ 0xdc21, 0xdc21, PDF_CMAP_SINGLE, 16211 },
	{ 0xdc21, 0xdc21, PDF_CMAP_SINGLE, 19091 },
	{ 0xdc21, 0xdc23, PDF_CMAP_TABLE, 13646 },
	{ 0xdc23, 0xdc23, PDF_CMAP_SINGLE, 18577 },
	{ 0xdc23, 0xdc24, PDF_CMAP_TABLE, 13649 },
	{ 0xdc24, 0xdc25, PDF_CMAP_TABLE, 13651 },
	{ 0xdc25, 0xdc26, PDF_CMAP_TABLE, 13653 },
	{ 0xdc26, 0xdc28, PDF_CMAP_TABLE, 13655 },
	{ 0xdc28, 0xdc29, PDF_CMAP_TABLE, 13658 },
	{ 0xdc29, 0xdc29, PDF_CMAP_SINGLE, 14193 },
	{ 0xdc29, 0xdc2a, PDF_CMAP_TABLE, 13660 },
	{ 0xdc2a, 0xdc2b, PDF_CMAP_TABLE, 13662 },
	{ 0xdc2b, 0xdc2c, PDF_CMAP_TABLE, 13664 },
	{ 0xdc2c, 0xdc2d, PDF_CMAP_TABLE, 13666 },
	{ 0xdc2f, 0xdc30, PDF_CMAP_TABLE, 13668 },
	{ 0xdc30, 0xdc30, PDF_CMAP_SINGLE, 15925 },
	{ 0xdc32, 0xdc33, PDF_CMAP_TABLE, 13670 },
	{ 0xdc33, 0xdc34, PDF_CMAP_TABLE, 13672 },
	{ 0xdc34, 0xdc36, PDF_CMAP_TABLE, 13674 },
	{ 0xdc38, 0xdc39, PDF_CMAP_TABLE, 13677 },
	{ 0xdc39, 0xdc39, PDF_CMAP_SINGLE, 16971 },
	{ 0xdc39, 0xdc3a, PDF_CMAP_TABLE, 13679 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 17941 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 14998 },
	{ 0xdc3a, 0xdc3b, PDF_CMAP_TABLE, 13681 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 18728 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 15786 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 15311 },
	{ 0xdc3b, 0xdc3f, PDF_CMAP_TABLE, 13683 },
	{ 0xdc3f, 0xdc3f, PDF_CMAP_SINGLE, 15209 },
	{ 0xdc3f, 0xdc40, PDF_CMAP_TABLE, 13688 },
	{ 0xdc40, 0xdc40, PDF_CMAP_SINGLE, 15490 },
	{ 0xdc40, 0xdc41, PDF_CMAP_TABLE, 13690 },
	{ 0xdc41, 0xdc45, PDF_CMAP_TABLE, 13692 },
	{ 0xdc45, 0xdc46, PDF_CMAP_TABLE, 13697 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 18520 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 18750 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 19069 },
	{ 0xdc48, 0xdc48, PDF_CMAP_SINGLE, 16071 },
	{ 0xdc48, 0xdc48, PDF_CMAP_SINGLE, 18578 },
	{ 0xdc48, 0xdc4a, PDF_CMAP_TABLE, 13699 },
	{ 0xdc4a, 0xdc4b, PDF_CMAP_TABLE, 13702 },
	{ 0xdc4b, 0xdc4c, PDF_CMAP_TABLE, 13704 },
	{ 0xdc4e, 0xdc50, PDF_CMAP_TABLE, 13706 },
	{ 0xdc50, 0xdc52, PDF_CMAP_TABLE, 13709 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 15570 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 17227 },
	{ 0xdc52, 0xdc53, PDF_CMAP_TABLE, 13712 },
	{ 0xdc53, 0xdc55, PDF_CMAP_TABLE, 13714 },
	{ 0xdc55, 0xdc55, PDF_CMAP_SINGLE, 17886 },
	{ 0xdc57, 0xdc57, PDF_CMAP_SINGLE, 14192 },
	{ 0xdc57, 0xdc57, PDF_CMAP_SINGLE, 17149 },
	{ 0xdc57, 0xdc58, PDF_CMAP_TABLE, 13717 },
	{ 0xdc5a, 0xdc5b, PDF_CMAP_TABLE, 13719 },
	{ 0xdc5b, 0xdc5e, PDF_CMAP_TABLE, 13721 },
	{ 0xdc5e, 0xdc62, PDF_CMAP_TABLE, 13725 },
	{ 0xdc62, 0xdc63, PDF_CMAP_TABLE, 13730 },
	{ 0xdc63, 0xdc64, PDF_CMAP_TABLE, 13732 },
	{ 0xdc64, 0xdc65, PDF_CMAP_TABLE, 13734 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 18417 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 18696 },
	{ 0xdc65, 0xdc66, PDF_CMAP_TABLE, 13736 },
	{ 0xdc68, 0xdc68, PDF_CMAP_SINGLE, 14218 },
	{ 0xdc68, 0xdc69, PDF_CMAP_TABLE, 13738 },
	{ 0xdc6c, 0xdc6c, PDF_CMAP_SINGLE, 14555 },
	{ 0xdc6c, 0xdc6c, PDF_CMAP_SINGLE, 18991 },
	{ 0xdc6c, 0xdc6c, PDF_CMAP_SINGLE, 17801 },
	{ 0xdc6e, 0xdc6e, PDF_CMAP_SINGLE, 18521 },
	{ 0xdc6e, 0xdc6f, PDF_CMAP_TABLE, 13740 },
	{ 0xdc6f, 0xdc70, PDF_CMAP_TABLE, 13742 },
	{ 0xdc70, 0xdc71, PDF_CMAP_TABLE, 13744 },
	{ 0xdc71, 0xdc72, PDF_CMAP_TABLE, 13746 },
	{ 0xdc72, 0xdc73, PDF_CMAP_TABLE, 13748 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 14702 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 15882 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 18851 },
	{ 0xdc75, 0xdc77, PDF_CMAP_TABLE, 13750 },
	{ 0xdc77, 0xdc77, PDF_CMAP_SINGLE, 17852 },
	{ 0xdc77, 0xdc78, PDF_CMAP_TABLE, 13753 },
	{ 0xdc78, 0xdc78, PDF_CMAP_SINGLE, 17916 },
	{ 0xdc78, 0xdc79, PDF_CMAP_TABLE, 13755 },
	{ 0xdc7b, 0xdc7b, PDF_CMAP_SINGLE, 17328 },
	{ 0xdc7b, 0xdc7e, PDF_CMAP_TABLE, 13757 },
	{ 0xdc7e, 0xdc7e, PDF_CMAP_SINGLE, 18772 },
	{ 0xdc7e, 0xdc82, PDF_CMAP_TABLE, 13761 },
	{ 0xdc82, 0xdc82, PDF_CMAP_SINGLE, 15654 },
	{ 0xdc82, 0xdc83, PDF_CMAP_TABLE, 13766 },
	{ 0xdc83, 0xdc83, PDF_CMAP_SINGLE, 18784 },
	{ 0xdc83, 0xdc84, PDF_CMAP_TABLE, 13768 },
	{ 0xdc84, 0xdc84, PDF_CMAP_SINGLE, 18538 },
	{ 0xdc84, 0xdc85, PDF_CMAP_TABLE, 13770 },
	{ 0xdc85, 0xdc86, PDF_CMAP_TABLE, 13772 },
	{ 0xdc86, 0xdc87, PDF_CMAP_TABLE, 13774 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 16441 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 15011 },
	{ 0xdc87, 0xdc88, PDF_CMAP_TABLE, 13776 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 18478 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 15499 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 18539 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 16719 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 14335 },
	{ 0xdc88, 0xdc8a, PDF_CMAP_TABLE, 13778 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 14249 },
	{ 0xdc8a, 0xdc8b, PDF_CMAP_TABLE, 13781 },
	{ 0xdc8b, 0xdc8b, PDF_CMAP_SINGLE, 14752 },
	{ 0xdc8b, 0xdc8d, PDF_CMAP_TABLE, 13783 },
	{ 0xdc8d, 0xdc8d, PDF_CMAP_SINGLE, 14841 },
	{ 0xdc8d, 0xdc8e, PDF_CMAP_TABLE, 13786 },
	{ 0xdc8e, 0xdc8e, PDF_CMAP_SINGLE, 14959 },
	{ 0xdc90, 0xdc91, PDF_CMAP_TABLE, 13788 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 16923 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 16969 },
	{ 0xdc91, 0xdc93, PDF_CMAP_TABLE, 13790 },
	{ 0xdc93, 0xdc94, PDF_CMAP_TABLE, 13793 },
	{ 0xdc94, 0xdc94, PDF_CMAP_SINGLE, 16715 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 16013 },
	{ 0xdc96, 0xdc97, PDF_CMAP_TABLE, 13795 },
	{ 0xdc97, 0xdc9a, PDF_CMAP_TABLE, 13797 },
	{ 0xdc9a, 0xdc9a, PDF_CMAP_SINGLE, 15024 },
	{ 0xdc9a, 0xdc9b, PDF_CMAP_TABLE, 13801 },
	{ 0xdc9b, 0xdc9d, PDF_CMAP_TABLE, 13803 },
	{ 0xdc9d, 0xdc9d, PDF_CMAP_SINGLE, 17115 },
	{ 0xdc9d, 0xdc9e, PDF_CMAP_TABLE, 13806 },
	{ 0xdc9e, 0xdc9f, PDF_CMAP_TABLE, 13808 },
	{ 0xdc9f, 0xdca1, PDF_CMAP_TABLE, 13810 },
	{ 0xdca1, 0xdca2, PDF_CMAP_TABLE, 13813 },
	{ 0xdca2, 0xdca4, PDF_CMAP_TABLE, 13815 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 15518 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 18557 },
	{ 0xdca4, 0xdca5, PDF_CMAP_TABLE, 13818 },
	{ 0xdca5, 0xdca6, PDF_CMAP_TABLE, 13820 },
	{ 0xdca9, 0xdca9, PDF_CMAP_SINGLE, 17192 },
	{ 0xdcac, 0xdcad, PDF_CMAP_TABLE, 13822 },
	{ 0xdcad, 0xdcad, PDF_CMAP_SINGLE, 18745 },
	{ 0xdcad, 0xdcaf, PDF_CMAP_TABLE, 13824 },
	{ 0xdcaf, 0xdcb0, PDF_CMAP_TABLE, 13827 },
	{ 0xdcb0, 0xdcb1, PDF_CMAP_TABLE, 13829 },
	{ 0xdcb1, 0xdcb2, PDF_CMAP_TABLE, 13831 },
	{ 0xdcb2, 0xdcb2, PDF_CMAP_SINGLE, 17668 },
	{ 0xdcb4, 0xdcb5, PDF_CMAP_TABLE, 13833 },
	{ 0xdcb5, 0xdcb7, PDF_CMAP_TABLE, 13835 },
	{ 0xdcb7, 0xdcb7, PDF_CMAP_SINGLE, 14546 },
	{ 0xdcb7, 0xdcb7, PDF_CMAP_SINGLE, 15963 },
	{ 0xdcb7, 0xdcb8, PDF_CMAP_TABLE, 13838 },
	{ 0xdcb8, 0xdcb9, PDF_CMAP_TABLE, 13840 },
	{ 0xdcb9, 0xdcb9, PDF_CMAP_SINGLE, 14639 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 19141 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 14547 },
	{ 0xdcbc, 0xdcbd, PDF_CMAP_TABLE, 13842 },
	{ 0xdcbd, 0xdcbe, PDF_CMAP_TABLE, 13844 },
	{ 0xdcbe, 0xdcbf, PDF_CMAP_TABLE, 13846 },
	{ 0xdcbf, 0xdcc0, PDF_CMAP_TABLE, 13848 },
	{ 0xdcc0, 0xdcc0, PDF_CMAP_SINGLE, 14322 },
	{ 0xdcc0, 0xdcc0, PDF_CMAP_SINGLE, 15508 },
	{ 0xdcc0, 0xdcc1, PDF_CMAP_TABLE, 13850 },
	{ 0xdcc1, 0xdcc1, PDF_CMAP_SINGLE, 19143 },
	{ 0xdcc1, 0xdcc1, PDF_CMAP_SINGLE, 16772 },
	{ 0xdcc1, 0xdcc3, PDF_CMAP_TABLE, 13852 },
	{ 0xdcc5, 0xdcc7, PDF_CMAP_TABLE, 13855 },
	{ 0xdcc7, 0xdcc7, PDF_CMAP_SINGLE, 17275 },
	{ 0xdcc7, 0xdcc7, PDF_CMAP_SINGLE, 18738 },
	{ 0xdcc7, 0xdcc8, PDF_CMAP_TABLE, 13858 },
	{ 0xdcc8, 0xdcc8, PDF_CMAP_SINGLE, 16409 },
	{ 0xdcc8, 0xdcc8, PDF_CMAP_SINGLE, 15386 },
	{ 0xdcc8, 0xdcc9, PDF_CMAP_TABLE, 13860 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 19144 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 17857 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 16917 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 14845 },
	{ 0xdcc9, 0xdcca, PDF_CMAP_TABLE, 13862 },
	{ 0xdcca, 0xdccc, PDF_CMAP_TABLE, 13864 },
	{ 0xdccc, 0xdccd, PDF_CMAP_TABLE, 13867 },
	{ 0xdccd, 0xdccd, PDF_CMAP_SINGLE, 14338 },
	{ 0xdccd, 0xdccd, PDF_CMAP_SINGLE, 17289 },
	{ 0xdccd, 0xdccf, PDF_CMAP_TABLE, 13869 },
	{ 0xdccf, 0xdcd1, PDF_CMAP_TABLE, 13872 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 14718 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 15216 },
	{ 0xdcd1, 0xdcd2, PDF_CMAP_TABLE, 13875 },
	{ 0xdcd2, 0xdcd3, PDF_CMAP_TABLE, 13877 },
	{ 0xdcd3, 0xdcd3, PDF_CMAP_SINGLE, 17330 },
	{ 0xdcd3, 0xdcd5, PDF_CMAP_TABLE, 13879 },
	{ 0xdcd5, 0xdcd6, PDF_CMAP_TABLE, 13882 },
	{ 0xdcd6, 0xdcd6, PDF_CMAP_SINGLE, 15505 },
	{ 0xdcd6, 0xdcd7, PDF_CMAP_TABLE, 13884 },
	{ 0xdcd7, 0xdcd9, PDF_CMAP_TABLE, 13886 },
	{ 0xdcd9, 0xdcd9, PDF_CMAP_SINGLE, 15079 },
	{ 0xdcd9, 0xdcda, PDF_CMAP_TABLE, 13889 },
	{ 0xdcda, 0xdcdb, PDF_CMAP_TABLE, 13891 },
	{ 0xdcdd, 0xdcde, PDF_CMAP_TABLE, 13893 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14751 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14132 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14626 },
	{ 0xdce4, 0xdce4, PDF_CMAP_SINGLE, 14830 },
	{ 0xdce4, 0xdce5, PDF_CMAP_TABLE, 13895 },
	{ 0xdce5, 0xdce5, PDF_CMAP_SINGLE, 14524 },
	{ 0xdce5, 0xdce5, PDF_CMAP_SINGLE, 17172 },
	{ 0xdce7, 0xdce8, PDF_CMAP_TABLE, 13897 },
	{ 0xdce8, 0xdce9, PDF_CMAP_TABLE, 13899 },
	{ 0xdce9, 0xdcea, PDF_CMAP_TABLE, 13901 },
	{ 0xdcec, 0xdcec, PDF_CMAP_SINGLE, 19042 },
	{ 0xdcec, 0xdced, PDF_CMAP_TABLE, 13903 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 16419 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 16845 },
	{ 0xdced, 0xdcee, PDF_CMAP_TABLE, 13905 },
	{ 0xdcf0, 0xdcf1, PDF_CMAP_TABLE, 13907 },
	{ 0xdcf1, 0xdcf3, PDF_CMAP_TABLE, 13909 },
	{ 0xdcf3, 0xdcf4, PDF_CMAP_TABLE, 13912 },
	{ 0xdcf4, 0xdcf4, PDF_CMAP_SINGLE, 18581 },
	{ 0xdcf5, 0xdcf6, PDF_CMAP_RANGE, 17926 },
	{ 0xdcf7, 0xdcf8, PDF_CMAP_TABLE, 13914 },
	{ 0xdcf8, 0xdcf8, PDF_CMAP_SINGLE, 14348 },
	{ 0xdcfa, 0xdcfa, PDF_CMAP_SINGLE, 14685 },
	{ 0xdcfa, 0xdcfc, PDF_CMAP_TABLE, 13916 },
	{ 0xdcfc, 0xdcfc, PDF_CMAP_SINGLE, 16446 },
	{ 0xdcfc, 0xdcfd, PDF_CMAP_TABLE, 13919 },
	{ 0xdcfd, 0xdcfe, PDF_CMAP_TABLE, 13921 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 18697 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 14154 },
	{ 0xdcfe, 0xdcff, PDF_CMAP_TABLE, 13923 },
	{ 0xdcff, 0xdcff, PDF_CMAP_SINGLE, 15794 },
	{ 0xdcff, 0xdd01, PDF_CMAP_TABLE, 13925 },
	{ 0xdd01, 0xdd03, PDF_CMAP_TABLE, 13928 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 18528 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 18743 },
	{ 0xdd03, 0xdd04, PDF_CMAP_RANGE, 14898 },
	{ 0xdd04, 0xdd05, PDF_CMAP_TABLE, 13931 },
	{ 0xdd05, 0xdd05, PDF_CMAP_SINGLE, 18917 },
	{ 0xdd05, 0xdd07, PDF_CMAP_TABLE, 13933 },
	{ 0xdd07, 0xdd08, PDF_CMAP_TABLE, 13936 },
	{ 0xdd08, 0xdd09, PDF_CMAP_TABLE, 13938 },
	{ 0xdd0c, 0xdd0c, PDF_CMAP_SINGLE, 17614 },
	{ 0xdd0c, 0xdd0c, PDF_CMAP_SINGLE, 16915 },
	{ 0xdd0c, 0xdd0d, PDF_CMAP_TABLE, 13940 },
	{ 0xdd0d, 0xdd0d, PDF_CMAP_SINGLE, 19098 },
	{ 0xdd0d, 0xdd0e, PDF_CMAP_TABLE, 13942 },
	{ 0xdd0e, 0xdd10, PDF_CMAP_TABLE, 13944 },
	{ 0xdd10, 0xdd11, PDF_CMAP_TABLE, 13947 },
	{ 0xdd11, 0xdd12, PDF_CMAP_TABLE, 13949 },
	{ 0xdd12, 0xdd12, PDF_CMAP_SINGLE, 18482 },
	{ 0xdd12, 0xdd13, PDF_CMAP_TABLE, 13951 },
	{ 0xdd13, 0xdd13, PDF_CMAP_SINGLE, 17092 },
	{ 0xdd15, 0xdd15, PDF_CMAP_SINGLE, 17164 },
	{ 0xdd15, 0xdd16, PDF_CMAP_TABLE, 13953 },
	{ 0xdd16, 0xdd19, PDF_CMAP_TABLE, 13955 },
	{ 0xdd19, 0xdd19, PDF_CMAP_SINGLE, 18500 },
	{ 0xdd19, 0xdd19, PDF_CMAP_SINGLE, 18721 },
	{ 0xdd1b, 0xdd1c, PDF_CMAP_TABLE, 13959 },
	{ 0xdd20, 0xdd21, PDF_CMAP_TABLE, 13961 },
	{ 0xdd21, 0xdd22, PDF_CMAP_TABLE, 13963 },
	{ 0xdd22, 0xdd22, PDF_CMAP_SINGLE, 18713 },
	{ 0xdd22, 0xdd23, PDF_CMAP_TABLE, 13965 },
	{ 0xdd23, 0xdd23, PDF_CMAP_SINGLE, 18718 },
	{ 0xdd23, 0xdd24, PDF_CMAP_TABLE, 13967 },
	{ 0xdd24, 0xdd26, PDF_CMAP_TABLE, 13969 },
	{ 0xdd26, 0xdd26, PDF_CMAP_SINGLE, 18550 },
	{ 0xdd26, 0xdd26, PDF_CMAP_SINGLE, 15652 },
	{ 0xdd26, 0xdd27, PDF_CMAP_TABLE, 13972 },
	{ 0xdd27, 0xdd27, PDF_CMAP_SINGLE, 14579 },
	{ 0xdd27, 0xdd28, PDF_CMAP_TABLE, 13974 },
	{ 0xdd28, 0xdd2a, PDF_CMAP_TABLE, 13976 },
	{ 0xdd2a, 0xdd2b, PDF_CMAP_TABLE, 13979 },
	{ 0xdd2e, 0xdd2f, PDF_CMAP_TABLE, 13981 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 15299 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 18668 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 15434 },
	{ 0xdd2f, 0xdd30, PDF_CMAP_TABLE, 13983 },
	{ 0xdd30, 0xdd30, PDF_CMAP_SINGLE, 14265 },
	{ 0xdd30, 0xdd31, PDF_CMAP_TABLE, 13985 },
	{ 0xdd31, 0xdd33, PDF_CMAP_TABLE, 13987 },
	{ 0xdd33, 0xdd33, PDF_CMAP_SINGLE, 14470 },
	{ 0xdd33, 0xdd34, PDF_CMAP_TABLE, 13990 },
	{ 0xdd34, 0xdd34, PDF_CMAP_SINGLE, 15318 },
	{ 0xdd34, 0xdd35, PDF_CMAP_TABLE, 13992 },
	{ 0xdd39, 0xdd39, PDF_CMAP_SINGLE, 17863 },
	{ 0xdd39, 0xdd39, PDF_CMAP_SINGLE, 18551 },
	{ 0xdd3b, 0xdd3b, PDF_CMAP_SINGLE, 18441 },
	{ 0xdd3b, 0xdd3c, PDF_CMAP_TABLE, 13994 },
	{ 0xdd3c, 0xdd3d, PDF_CMAP_TABLE, 13996 },
	{ 0xdd3d, 0xdd3d, PDF_CMAP_SINGLE, 17957 },
	{ 0xdd3d, 0xdd3e, PDF_CMAP_TABLE, 13998 },
	{ 0xdd3e, 0xdd3f, PDF_CMAP_TABLE, 14000 },
	{ 0xdd3f, 0xdd3f, PDF_CMAP_SINGLE, 15816 },
	{ 0xdd3f, 0xdd40, PDF_CMAP_TABLE, 14002 },
	{ 0xdd40, 0xdd40, PDF_CMAP_SINGLE, 16978 },
	{ 0xdd40, 0xdd43, PDF_CMAP_TABLE, 14004 },
	{ 0xdd43, 0xdd44, PDF_CMAP_TABLE, 14008 },
	{ 0xdd44, 0xdd44, PDF_CMAP_SINGLE, 17210 },
	{ 0xdd44, 0xdd45, PDF_CMAP_TABLE, 14010 },
	{ 0xdd45, 0xdd46, PDF_CMAP_TABLE, 14012 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 18732 },
	{ 0xdd46, 0xdd47, PDF_CMAP_TABLE, 14014 },
	{ 0xdd47, 0xdd47, PDF_CMAP_SINGLE, 17871 },
	{ 0xdd47, 0xdd48, PDF_CMAP_TABLE, 14016 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 17917 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 18660 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 15822 },
	{ 0xdd48, 0xdd49, PDF_CMAP_TABLE, 14018 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 15928 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 18612 },
	{ 0xdd4c, 0xdd4c, PDF_CMAP_SINGLE, 15144 },
	{ 0xdd4c, 0xdd4d, PDF_CMAP_TABLE, 14020 },
	{ 0xdd4d, 0xdd4e, PDF_CMAP_TABLE, 14022 },
	{ 0xdd4e, 0xdd4f, PDF_CMAP_TABLE, 14024 },
	{ 0xdd4f, 0xdd4f, PDF_CMAP_SINGLE, 18470 },
	{ 0xdd4f, 0xdd51, PDF_CMAP_TABLE, 14026 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 17664 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 15653 },
	{ 0xdd53, 0xdd53, PDF_CMAP_SINGLE, 19089 },
	{ 0xdd53, 0xdd54, PDF_CMAP_TABLE, 14029 },
	{ 0xdd54, 0xdd54, PDF_CMAP_SINGLE, 16644 },
	{ 0xdd54, 0xdd56, PDF_CMAP_TABLE, 14031 },
	{ 0xdd56, 0xdd56, PDF_CMAP_SINGLE, 18592 },
	{ 0xdd56, 0xdd57, PDF_CMAP_RANGE, 16976 },
	{ 0xdd58, 0xdd5a, PDF_CMAP_TABLE, 14034 },
	{ 0xdd5a, 0xdd5a, PDF_CMAP_SINGLE, 14202 },
	{ 0xdd5a, 0xdd5b, PDF_CMAP_TABLE, 14037 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 15582 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 14975 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 15517 },
	{ 0xdd5b, 0xdd5c, PDF_CMAP_TABLE, 14039 },
	{ 0xdd5c, 0xdd5c, PDF_CMAP_SINGLE, 14735 },
	{ 0xdd5c, 0xdd5d, PDF_CMAP_TABLE, 14041 },
	{ 0xdd5d, 0xdd5d, PDF_CMAP_SINGLE, 17179 },
	{ 0xdd5d, 0xdd5e, PDF_CMAP_TABLE, 14043 },
	{ 0xdd5e, 0xdd5e, PDF_CMAP_SINGLE, 18523 },
	{ 0xdd5e, 0xdd5f, PDF_CMAP_TABLE, 14045 },
	{ 0xdd61, 0xdd62, PDF_CMAP_TABLE, 14047 },
	{ 0xdd62, 0xdd64, PDF_CMAP_TABLE, 14049 },
	{ 0xdd64, 0xdd65, PDF_CMAP_TABLE, 14052 },
	{ 0xdd65, 0xdd66, PDF_CMAP_TABLE, 14054 },
	{ 0xdd66, 0xdd67, PDF_CMAP_TABLE, 14056 },
	{ 0xdd67, 0xdd67, PDF_CMAP_SINGLE, 14952 },
	{ 0xdd67, 0xdd67, PDF_CMAP_SINGLE, 18492 },
	{ 0xdd67, 0xdd68, PDF_CMAP_TABLE, 14058 },
	{ 0xdd6a, 0xdd6a, PDF_CMAP_SINGLE, 18452 },
	{ 0xdd6a, 0xdd6c, PDF_CMAP_TABLE, 14060 },
	{ 0xdd6c, 0xdd6d, PDF_CMAP_TABLE, 14063 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 18423 },
	{ 0xdd6f, 0xdd73, PDF_CMAP_TABLE, 14065 },
	{ 0xdd73, 0xdd74, PDF_CMAP_TABLE, 14070 },
	{ 0xdd74, 0xdd74, PDF_CMAP_SINGLE, 17019 },
	{ 0xdd74, 0xdd74, PDF_CMAP_SINGLE, 18956 },
	{ 0xdd74, 0xdd75, PDF_CMAP_TABLE, 14072 },
	{ 0xdd75, 0xdd77, PDF_CMAP_TABLE, 14074 },
	{ 0xdd77, 0xdd7a, PDF_CMAP_TABLE, 14077 },
	{ 0xdd7a, 0xdd7a, PDF_CMAP_SINGLE, 15908 },
	{ 0xdd7a, 0xdd7c, PDF_CMAP_TABLE, 14081 },
	{ 0xdd7c, 0xdd7c, PDF_CMAP_SINGLE, 14894 },
	{ 0xdd7c, 0xdd7d, PDF_CMAP_TABLE, 14084 },
	{ 0xdd7d, 0xdd7e, PDF_CMAP_TABLE, 14086 },
	{ 0xdd7e, 0xdd7e, PDF_CMAP_SINGLE, 17934 },
	{ 0xdd7e, 0xdd7e, PDF_CMAP_SINGLE, 14687 },
	{ 0xdd7e, 0xdd7f, PDF_CMAP_RANGE, 17884 },
	{ 0xdd7f, 0xdd80, PDF_CMAP_TABLE, 14088 },
	{ 0xdd80, 0xdd80, PDF_CMAP_SINGLE, 18872 },
	{ 0xdd80, 0xdd82, PDF_CMAP_TABLE, 14090 },
	{ 0xdd82, 0xdd82, PDF_CMAP_SINGLE, 14983 },
	{ 0xdd82, 0xdd84, PDF_CMAP_TABLE, 14093 },
	{ 0xdd84, 0xdd84, PDF_CMAP_SINGLE, 17845 },
	{ 0xdd87, 0xdd88, PDF_CMAP_TABLE, 14096 },
	{ 0xdd88, 0xdd89, PDF_CMAP_TABLE, 14098 },
	{ 0xdd89, 0xdd89, PDF_CMAP_SINGLE, 17384 },
	{ 0xdd89, 0xdd8d, PDF_CMAP_TABLE, 14100 },
	{ 0xdd8d, 0xdd8d, PDF_CMAP_SINGLE, 17238 },
	{ 0xdd8d, 0xdd8e, PDF_CMAP_TABLE, 14105 },
	{ 0xdd8e, 0xdd8f, PDF_CMAP_TABLE, 14107 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 16750 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 14973 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 14407 },
	{ 0xdd8f, 0xdd90, PDF_CMAP_TABLE, 14109 },
	{ 0xdd92, 0xdd93, PDF_CMAP_TABLE, 14111 },
	{ 0xdd93, 0xdd94, PDF_CMAP_TABLE, 14113 },
	{ 0xdd94, 0xdd95, PDF_CMAP_TABLE, 14115 },
	{ 0xdd95, 0xdd96, PDF_CMAP_TABLE, 14117 },
	{ 0xdd96, 0xdd96, PDF_CMAP_SINGLE, 16511 },
	{ 0xdd96, 0xdd96, PDF_CMAP_SINGLE, 14893 },
	{ 0xdd98, 0xdd98, PDF_CMAP_SINGLE, 14412 },
	{ 0xdd98, 0xdd99, PDF_CMAP_TABLE, 14119 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 19139 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 17299 },
	{ 0xdd9b, 0xdd9c, PDF_CMAP_TABLE, 14121 },
	{ 0xdd9c, 0xdd9c, PDF_CMAP_SINGLE, 15451 },
	{ 0xdd9c, 0xdd9e, PDF_CMAP_TABLE, 14123 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 18680 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 15213 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 16853 },
	{ 0xdda0, 0xdda1, PDF_CMAP_TABLE, 14126 },
	{ 0xdda1, 0xdda3, PDF_CMAP_TABLE, 14128 },
	{ 0xdda3, 0xdda3, PDF_CMAP_SINGLE, 18669 },
	{ 0xdda3, 0xdda3, PDF_CMAP_SINGLE, 14284 },
	{ 0xdda3, 0xdda4, PDF_CMAP_TABLE, 14131 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 17922 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 16914 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 16440 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 15442 },
	{ 0xdda4, 0xdda5, PDF_CMAP_RANGE, 16821 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 16660 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 16585 },
	{ 0xdda5, 0xdda7, PDF_CMAP_TABLE, 14133 },
	{ 0xdda7, 0xdda7, PDF_CMAP_SINGLE, 14264 },
	{ 0xdda7, 0xdda8, PDF_CMAP_TABLE, 14136 },
	{ 0xdda8, 0xdda8, PDF_CMAP_SINGLE, 14909 },
	{ 0xdda8, 0xdda9, PDF_CMAP_TABLE, 14138 },
	{ 0xdda9, 0xddab, PDF_CMAP_TABLE, 14140 },
	{ 0xddab, 0xddab, PDF_CMAP_SINGLE, 15088 },
	{ 0xddab, 0xddab, PDF_CMAP_SINGLE, 15907 },
	{ 0xddab, 0xddac, PDF_CMAP_TABLE, 14143 },
	{ 0xddac, 0xddac, PDF_CMAP_SINGLE, 14829 },
	{ 0xddac, 0xddad, PDF_CMAP_TABLE, 14145 },
	{ 0xddad, 0xddad, PDF_CMAP_SINGLE, 18746 },
	{ 0xddad, 0xddae, PDF_CMAP_TABLE, 14147 },
	{ 0xddae, 0xddaf, PDF_CMAP_TABLE, 14149 },
	{ 0xddaf, 0xddb0, PDF_CMAP_TABLE, 14151 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 18424 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 18535 },
	{ 0xddb2, 0xddb3, PDF_CMAP_TABLE, 14153 },
	{ 0xddb3, 0xddb5, PDF_CMAP_TABLE, 14155 },
	{ 0xddb5, 0xddb5, PDF_CMAP_SINGLE, 15921 },
	{ 0xddb5, 0xddb6, PDF_CMAP_TABLE, 14158 },
	{ 0xddb6, 0xddb7, PDF_CMAP_TABLE, 14160 },
	{ 0xddb7, 0xddb7, PDF_CMAP_SINGLE, 15514 },
	{ 0xddb7, 0xddb8, PDF_CMAP_TABLE, 14162 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 15401 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 17171 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 18593 },
	{ 0xddb8, 0xddb9, PDF_CMAP_TABLE, 14164 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 15519 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 14476 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 19124 },
	{ 0xddb9, 0xddba, PDF_CMAP_TABLE, 14166 },
	{ 0xddba, 0xddba, PDF_CMAP_SINGLE, 15147 },
	{ 0xddba, 0xddbb, PDF_CMAP_TABLE, 14168 },
	{ 0xddbb, 0xddbb, PDF_CMAP_SINGLE, 14446 },
	{ 0xddbb, 0xddbc, PDF_CMAP_TABLE, 14170 },
	{ 0xddbc, 0xddbc, PDF_CMAP_SINGLE, 14427 },
	{ 0xddbc, 0xddbc, PDF_CMAP_SINGLE, 16552 },
	{ 0xddbc, 0xddbd, PDF_CMAP_TABLE, 14172 },
	{ 0xddbd, 0xddc1, PDF_CMAP_TABLE, 14174 },
	{ 0xddc1, 0xddc3, PDF_CMAP_TABLE, 14179 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 15799 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 18614 },
	{ 0xddc5, 0xddc6, PDF_CMAP_TABLE, 14182 },
	{ 0xddc8, 0xddc8, PDF_CMAP_SINGLE, 18425 },
	{ 0xddc8, 0xddc8, PDF_CMAP_SINGLE, 16831 },
	{ 0xddc8, 0xddc9, PDF_CMAP_TABLE, 14184 },
	{ 0xddc9, 0xddca, PDF_CMAP_TABLE, 14186 },
	{ 0xddca, 0xddcd, PDF_CMAP_TABLE, 14188 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 16844 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 18859 },
	{ 0xddcd, 0xddd0, PDF_CMAP_TABLE, 14192 },
	{ 0xddd0, 0xddd1, PDF_CMAP_TABLE, 14196 },
	{ 0xddd4, 0xddd4, PDF_CMAP_SINGLE, 15153 },
	{ 0xddd4, 0xddd5, PDF_CMAP_TABLE, 14198 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 15554 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 14539 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 15365 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 14421 },
	{ 0xddd9, 0xdddb, PDF_CMAP_TABLE, 14200 },
	{ 0xdddb, 0xdddc, PDF_CMAP_TABLE, 14203 },
	{ 0xdddc, 0xdddc, PDF_CMAP_SINGLE, 15279 },
	{ 0xdddc, 0xdddd, PDF_CMAP_TABLE, 14205 },
	{ 0xdddd, 0xddde, PDF_CMAP_TABLE, 14207 },
	{ 0xddde, 0xdddf, PDF_CMAP_TABLE, 14209 },
	{ 0xdddf, 0xdddf, PDF_CMAP_SINGLE, 16783 },
	{ 0xdddf, 0xdde0, PDF_CMAP_TABLE, 14211 },
	{ 0xdde0, 0xdde0, PDF_CMAP_SINGLE, 17137 },
	{ 0xdde0, 0xdde3, PDF_CMAP_TABLE, 14213 },
	{ 0xdde3, 0xdde3, PDF_CMAP_SINGLE, 14951 },
	{ 0xdde3, 0xdde3, PDF_CMAP_SINGLE, 17865 },
	{ 0xdde3, 0xdde4, PDF_CMAP_TABLE, 14217 },
	{ 0xdde4, 0xdde7, PDF_CMAP_TABLE, 14219 },
	{ 0xdde7, 0xdde7, PDF_CMAP_SINGLE, 15103 },
	{ 0xdde7, 0xdde8, PDF_CMAP_RANGE, 18594 },
	{ 0xdde8, 0xddea, PDF_CMAP_TABLE, 14223 },
	{ 0xddea, 0xddea, PDF_CMAP_SINGLE, 17181 },
	{ 0xddea, 0xddeb, PDF_CMAP_TABLE, 14226 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 14991 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 17908 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 16928 },
	{ 0xddeb, 0xddee, PDF_CMAP_TABLE, 14228 },
	{ 0xddf0, 0xddf0, PDF_CMAP_SINGLE, 15444 },
	{ 0xddf2, 0xddf2, PDF_CMAP_SINGLE, 16228 },
	{ 0xddf2, 0xddf3, PDF_CMAP_TABLE, 14232 },
	{ 0xddf3, 0xddf4, PDF_CMAP_TABLE, 14234 },
	{ 0xddf4, 0xddf6, PDF_CMAP_TABLE, 14236 },
	{ 0xddf6, 0xddf7, PDF_CMAP_TABLE, 14239 },
	{ 0xddf7, 0xddf8, PDF_CMAP_RANGE, 16818 },
	{ 0xddf8, 0xddf8, PDF_CMAP_SINGLE, 14267 },
	{ 0xddf8, 0xddf9, PDF_CMAP_TABLE, 14241 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 16922 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 15174 },
	{ 0xddf9, 0xddfa, PDF_CMAP_TABLE, 14243 },
	{ 0xddfa, 0xddfb, PDF_CMAP_TABLE, 14245 },
	{ 0xddfb, 0xddfc, PDF_CMAP_TABLE, 14247 },
	{ 0xddfc, 0xddfd, PDF_CMAP_TABLE, 14249 },
	{ 0xddfd, 0xde00, PDF_CMAP_TABLE, 14251 },
	{ 0xde00, 0xde00, PDF_CMAP_SINGLE, 15447 },
	{ 0xde00, 0xde01, PDF_CMAP_TABLE, 14255 },
	{ 0xde04, 0xde04, PDF_CMAP_SINGLE, 16656 },
	{ 0xde04, 0xde05, PDF_CMAP_TABLE, 14257 },
	{ 0xde05, 0xde06, PDF_CMAP_TABLE, 14259 },
	{ 0xde06, 0xde07, PDF_CMAP_TABLE, 14261 },
	{ 0xde07, 0xde07, PDF_CMAP_SINGLE, 14008 },
	{ 0xde07, 0xde0a, PDF_CMAP_TABLE, 14263 },
	{ 0xde0a, 0xde0a, PDF_CMAP_SINGLE, 15896 },
	{ 0xde0a, 0xde0c, PDF_CMAP_TABLE, 14267 },
	{ 0xde0c, 0xde0d, PDF_CMAP_TABLE, 14270 },
	{ 0xde0d, 0xde0e, PDF_CMAP_TABLE, 14272 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 14793 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 15204 },
	{ 0xde0e, 0xde0f, PDF_CMAP_TABLE, 14274 },
	{ 0xde0f, 0xde0f, PDF_CMAP_SINGLE, 14982 },
	{ 0xde0f, 0xde0f, PDF_CMAP_SINGLE, 18747 },
	{ 0xde0f, 0xde11, PDF_CMAP_TABLE, 14276 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 15855 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 17929 },
	{ 0xde11, 0xde12, PDF_CMAP_TABLE, 14279 },
	{ 0xde12, 0xde12, PDF_CMAP_SINGLE, 17663 },
	{ 0xde12, 0xde13, PDF_CMAP_TABLE, 14281 },
	{ 0xde13, 0xde15, PDF_CMAP_TABLE, 14283 },
	{ 0xde15, 0xde16, PDF_CMAP_TABLE, 14286 },
	{ 0xde16, 0xde18, PDF_CMAP_TABLE, 14288 },
	{ 0xde18, 0xde19, PDF_CMAP_TABLE, 14291 },
	{ 0xde19, 0xde1a, PDF_CMAP_TABLE, 14293 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 18494 },
	{ 0xde1a, 0xde1b, PDF_CMAP_TABLE, 14295 },
	{ 0xde1b, 0xde1b, PDF_CMAP_SINGLE, 15354 },
	{ 0xde1b, 0xde1b, PDF_CMAP_SINGLE, 15232 },
	{ 0xde1b, 0xde1c, PDF_CMAP_TABLE, 14297 },
	{ 0xde1c, 0xde1d, PDF_CMAP_TABLE, 14299 },
	{ 0xde20, 0xde20, PDF_CMAP_SINGLE, 17122 },
	{ 0xde20, 0xde21, PDF_CMAP_TABLE, 14301 },
	{ 0xde21, 0xde23, PDF_CMAP_TABLE, 14303 },
	{ 0xde23, 0xde25, PDF_CMAP_TABLE, 14306 },
	{ 0xde25, 0xde25, PDF_CMAP_SINGLE, 14723 },
	{ 0xde25, 0xde25, PDF_CMAP_SINGLE, 14332 },
	{ 0xde25, 0xde26, PDF_CMAP_TABLE, 14309 },
	{ 0xde26, 0xde28, PDF_CMAP_TABLE, 14311 },
	{ 0xde28, 0xde29, PDF_CMAP_TABLE, 14314 },
	{ 0xde29, 0xde2a, PDF_CMAP_TABLE, 14316 },
	{ 0xde2a, 0xde2d, PDF_CMAP_TABLE, 14318 },
	{ 0xde2d, 0xde2d, PDF_CMAP_SINGLE, 15609 },
	{ 0xde2d, 0xde2d, PDF_CMAP_SINGLE, 18553 },
	{ 0xde2d, 0xde2e, PDF_CMAP_TABLE, 14322 },
	{ 0xde2e, 0xde2f, PDF_CMAP_TABLE, 14324 },
	{ 0xde2f, 0xde30, PDF_CMAP_TABLE, 14326 },
	{ 0xde30, 0xde31, PDF_CMAP_TABLE, 14328 },
	{ 0xde31, 0xde32, PDF_CMAP_TABLE, 14330 },
	{ 0xde32, 0xde32, PDF_CMAP_SINGLE, 15814 },
	{ 0xde32, 0xde32, PDF_CMAP_SINGLE, 18863 },
	{ 0xde32, 0xde33, PDF_CMAP_TABLE, 14332 },
	{ 0xde33, 0xde34, PDF_CMAP_TABLE, 14334 },
	{ 0xde34, 0xde34, PDF_CMAP_SINGLE, 14980 },
	{ 0xde34, 0xde36, PDF_CMAP_TABLE, 14336 },
	{ 0xde36, 0xde36, PDF_CMAP_SINGLE, 16726 },
	{ 0xde36, 0xde37, PDF_CMAP_TABLE, 14339 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 19051 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 14805 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 17292 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 16332 },
	{ 0xde3b, 0xde3b, PDF_CMAP_SINGLE, 17282 },
	{ 0xde3b, 0xde3c, PDF_CMAP_TABLE, 14341 },
	{ 0xde3c, 0xde3e, PDF_CMAP_TABLE, 14343 },
	{ 0xde3e, 0xde3e, PDF_CMAP_SINGLE, 15534 },
	{ 0xde3e, 0xde3e, PDF_CMAP_SINGLE, 15961 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 15452 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 18485 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 15322 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 15225 },
	{ 0xde42, 0xde44, PDF_CMAP_TABLE, 14346 },
	{ 0xde44, 0xde45, PDF_CMAP_TABLE, 14349 },
	{ 0xde45, 0xde45, PDF_CMAP_SINGLE, 15787 },
	{ 0xde45, 0xde45, PDF_CMAP_SINGLE, 16633 },
	{ 0xde45, 0xde49, PDF_CMAP_TABLE, 14351 },
	{ 0xde49, 0xde4a, PDF_CMAP_TABLE, 14356 },
	{ 0xde4a, 0xde4a, PDF_CMAP_SINGLE, 16869 },
	{ 0xde4a, 0xde4b, PDF_CMAP_TABLE, 14358 },
	{ 0xde4b, 0xde4b, PDF_CMAP_SINGLE, 18102 },
	{ 0xde4b, 0xde4d, PDF_CMAP_TABLE, 14360 },
	{ 0xde4d, 0xde4f, PDF_CMAP_TABLE, 14363 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 18129 },
	{ 0xde4f, 0xde50, PDF_CMAP_TABLE, 14366 },
	{ 0xde50, 0xde50, PDF_CMAP_SINGLE, 15569 },
	{ 0xde50, 0xde50, PDF_CMAP_SINGLE, 17921 },
	{ 0xde50, 0xde51, PDF_CMAP_TABLE, 14368 },
	{ 0xde51, 0xde51, PDF_CMAP_SINGLE, 14774 },
	{ 0xde51, 0xde56, PDF_CMAP_TABLE, 14370 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 19028 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 15697 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 18740 },
	{ 0xde56, 0xde59, PDF_CMAP_TABLE, 14376 },
	{ 0xde59, 0xde59, PDF_CMAP_SINGLE, 15688 },
	{ 0xde59, 0xde59, PDF_CMAP_SINGLE, 14384 },
	{ 0xde59, 0xde5b, PDF_CMAP_TABLE, 14380 },
	{ 0xde5b, 0xde5b, PDF_CMAP_SINGLE, 17288 },
	{ 0xde5d, 0xde5f, PDF_CMAP_TABLE, 14383 },
	{ 0xde61, 0xde61, PDF_CMAP_SINGLE, 14217 },
	{ 0xde61, 0xde63, PDF_CMAP_TABLE, 14386 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 14940 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 14253 },
	{ 0xde65, 0xde66, PDF_CMAP_TABLE, 14389 },
	{ 0xde66, 0xde66, PDF_CMAP_SINGLE, 15537 },
	{ 0xde66, 0xde67, PDF_CMAP_TABLE, 14391 },
	{ 0xde67, 0xde68, PDF_CMAP_TABLE, 14393 },
	{ 0xde68, 0xde68, PDF_CMAP_SINGLE, 14621 },
	{ 0xde6a, 0xde6b, PDF_CMAP_TABLE, 14395 },
	{ 0xde6d, 0xde6f, PDF_CMAP_TABLE, 14397 },
	{ 0xde71, 0xde74, PDF_CMAP_TABLE, 14400 },
	{ 0xde74, 0xde75, PDF_CMAP_TABLE, 14404 },
	{ 0xde75, 0xde76, PDF_CMAP_TABLE, 14406 },
	{ 0xde76, 0xde76, PDF_CMAP_SINGLE, 16015 },
	{ 0xde76, 0xde77, PDF_CMAP_TABLE, 14408 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 18748 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 16992 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 18725 },
	{ 0xde77, 0xde78, PDF_CMAP_TABLE, 14410 },
	{ 0xde78, 0xde78, PDF_CMAP_SINGLE, 14946 },
	{ 0xde78, 0xde78, PDF_CMAP_SINGLE, 16608 },
	{ 0xde78, 0xde79, PDF_CMAP_TABLE, 14412 },
	{ 0xde79, 0xde79, PDF_CMAP_SINGLE, 17072 },
	{ 0xde79, 0xde7a, PDF_CMAP_TABLE, 14414 },
	{ 0xde7a, 0xde7a, PDF_CMAP_SINGLE, 18100 },
	{ 0xde7a, 0xde7c, PDF_CMAP_TABLE, 14416 },
	{ 0xde7c, 0xde7c, PDF_CMAP_SINGLE, 18465 },
	{ 0xde7c, 0xde7c, PDF_CMAP_SINGLE, 17943 },
	{ 0xde80, 0xde81, PDF_CMAP_TABLE, 14419 },
	{ 0xde81, 0xde81, PDF_CMAP_SINGLE, 18646 },
	{ 0xde81, 0xde82, PDF_CMAP_TABLE, 14421 },
	{ 0xde82, 0xde83, PDF_CMAP_TABLE, 14423 },
	{ 0xde83, 0xde83, PDF_CMAP_SINGLE, 14876 },
	{ 0xde83, 0xde84, PDF_CMAP_TABLE, 14425 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 15443 },
	{ 0xde84, 0xde85, PDF_CMAP_TABLE, 14427 },
	{ 0xde88, 0xde88, PDF_CMAP_SINGLE, 18648 },
	{ 0xde88, 0xde89, PDF_CMAP_TABLE, 14429 },
	{ 0xde89, 0xde89, PDF_CMAP_SINGLE, 14968 },
	{ 0xde89, 0xde8a, PDF_CMAP_TABLE, 14431 },
	{ 0xde8a, 0xde8a, PDF_CMAP_SINGLE, 16569 },
	{ 0xde8a, 0xde8b, PDF_CMAP_TABLE, 14433 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 14283 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 18984 },
	{ 0xde8b, 0xde8c, PDF_CMAP_TABLE, 14435 },
	{ 0xde8c, 0xde8c, PDF_CMAP_SINGLE, 17070 },
	{ 0xde8e, 0xde8e, PDF_CMAP_SINGLE, 19131 },
	{ 0xde92, 0xde93, PDF_CMAP_TABLE, 14437 },
	{ 0xde93, 0xde93, PDF_CMAP_SINGLE, 18634 },
	{ 0xde93, 0xde94, PDF_CMAP_TABLE, 14439 },
	{ 0xde94, 0xde95, PDF_CMAP_TABLE, 14441 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 14254 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 18925 },
	{ 0xde95, 0xde96, PDF_CMAP_TABLE, 14443 },
	{ 0xde96, 0xde96, PDF_CMAP_SINGLE, 17880 },
	{ 0xde96, 0xde97, PDF_CMAP_TABLE, 14445 },
	{ 0xde97, 0xde98, PDF_CMAP_TABLE, 14447 },
	{ 0xde98, 0xde99, PDF_CMAP_TABLE, 14449 },
	{ 0xde99, 0xde99, PDF_CMAP_SINGLE, 18518 },
	{ 0xde99, 0xde9b, PDF_CMAP_TABLE, 14451 },
	{ 0xde9b, 0xde9c, PDF_CMAP_TABLE, 14454 },
	{ 0xde9c, 0xde9e, PDF_CMAP_TABLE, 14456 },
	{ 0xde9e, 0xdea0, PDF_CMAP_TABLE, 14459 },
	{ 0xdea2, 0xdea2, PDF_CMAP_SINGLE, 16017 },
	{ 0xdea4, 0xdea4, PDF_CMAP_SINGLE, 16682 },
	{ 0xdea4, 0xdea5, PDF_CMAP_TABLE, 14462 },
	{ 0xdea5, 0xdea5, PDF_CMAP_SINGLE, 15934 },
	{ 0xdea5, 0xdea6, PDF_CMAP_TABLE, 14464 },
	{ 0xdea6, 0xdea6, PDF_CMAP_SINGLE, 16464 },
	{ 0xdea6, 0xdea7, PDF_CMAP_TABLE, 14466 },
	{ 0xdea7, 0xdea7, PDF_CMAP_SINGLE, 15810 },
	{ 0xdea7, 0xdea8, PDF_CMAP_TABLE, 14468 },
	{ 0xdea8, 0xdea9, PDF_CMAP_TABLE, 14470 },
	{ 0xdea9, 0xdeaa, PDF_CMAP_TABLE, 14472 },
	{ 0xdeaa, 0xdeab, PDF_CMAP_TABLE, 14474 },
	{ 0xdeab, 0xdeab, PDF_CMAP_SINGLE, 14962 },
	{ 0xdeab, 0xdeac, PDF_CMAP_TABLE, 14476 },
	{ 0xdeac, 0xdeac, PDF_CMAP_SINGLE, 17124 },
	{ 0xdeac, 0xdeac, PDF_CMAP_SINGLE, 17084 },
	{ 0xdeac, 0xdead, PDF_CMAP_TABLE, 14478 },
	{ 0xdead, 0xdeaf, PDF_CMAP_TABLE, 14480 },
	{ 0xdeaf, 0xdeb0, PDF_CMAP_TABLE, 14483 },
	{ 0xdeb0, 0xdeb1, PDF_CMAP_TABLE, 14485 },
	{ 0xdeb1, 0xdeb2, PDF_CMAP_TABLE, 14487 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 16092 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 19071 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 17059 },
	{ 0xdeb2, 0xdeb3, PDF_CMAP_TABLE, 14489 },
	{ 0xdeb3, 0xdeb3, PDF_CMAP_SINGLE, 15529 },
	{ 0xdeb3, 0xdeb4, PDF_CMAP_TABLE, 14491 },
	{ 0xdeb4, 0xdeb6, PDF_CMAP_TABLE, 14493 },
	{ 0xdeb6, 0xdeb6, PDF_CMAP_SINGLE, 18672 },
	{ 0xdeb6, 0xdeba, PDF_CMAP_TABLE, 14496 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 18659 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 15116 },
	{ 0xdeba, 0xdebc, PDF_CMAP_TABLE, 14501 },
	{ 0xdebc, 0xdebc, PDF_CMAP_SINGLE, 15171 },
	{ 0xdebc, 0xdebd, PDF_CMAP_TABLE, 14504 },
	{ 0xdebd, 0xdebf, PDF_CMAP_TABLE, 14506 },
	{ 0xdebf, 0xdebf, PDF_CMAP_SINGLE, 14806 },
	{ 0xdebf, 0xdec0, PDF_CMAP_TABLE, 14509 },
	{ 0xdec0, 0xdec0, PDF_CMAP_SINGLE, 14721 },
	{ 0xdec0, 0xdec0, PDF_CMAP_SINGLE, 18513 },
	{ 0xdec0, 0xdec1, PDF_CMAP_TABLE, 14511 },
	{ 0xdec1, 0xdec2, PDF_CMAP_TABLE, 14513 },
	{ 0xdec2, 0xdec3, PDF_CMAP_TABLE, 14515 },
	{ 0xdec6, 0xdec7, PDF_CMAP_TABLE, 14517 },
	{ 0xdec7, 0xdeca, PDF_CMAP_TABLE, 14519 },
	{ 0xdeca, 0xdecd, PDF_CMAP_TABLE, 14523 },
	{ 0xdecd, 0xded0, PDF_CMAP_TABLE, 14527 },
	{ 0xded0, 0xded1, PDF_CMAP_TABLE, 14531 },
	{ 0xded1, 0xded2, PDF_CMAP_TABLE, 14533 },
	{ 0xded2, 0xded3, PDF_CMAP_TABLE, 14535 },
	{ 0xded3, 0xded4, PDF_CMAP_TABLE, 14537 },
	{ 0xded4, 0xded5, PDF_CMAP_TABLE, 14539 },
	{ 0xded5, 0xded5, PDF_CMAP_SINGLE, 18459 },
	{ 0xded5, 0xded5, PDF_CMAP_SINGLE, 18473 },
	{ 0xded5, 0xded7, PDF_CMAP_TABLE, 14541 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 14123 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 18739 },
	{ 0xded7, 0xded8, PDF_CMAP_TABLE, 14544 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 17938 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 18655 },
	{ 0xded8, 0xdedb, PDF_CMAP_TABLE, 14546 },
	{ 0xdedd, 0xdedf, PDF_CMAP_TABLE, 14550 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 15641 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 15065 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 16858 },
	{ 0xdedf, 0xdee0, PDF_CMAP_TABLE, 14553 },
	{ 0xdee0, 0xdee1, PDF_CMAP_TABLE, 14555 },
	{ 0xdee1, 0xdee2, PDF_CMAP_TABLE, 14557 },
	{ 0xdee2, 0xdee2, PDF_CMAP_SINGLE, 16005 },
	{ 0xdee2, 0xdee3, PDF_CMAP_TABLE, 14559 },
	{ 0xdee3, 0xdee3, PDF_CMAP_SINGLE, 15130 },
	{ 0xdee3, 0xdee3, PDF_CMAP_SINGLE, 15099 },
	{ 0xdee5, 0xdee5, PDF_CMAP_SINGLE, 16447 },
	{ 0xdee5, 0xdee6, PDF_CMAP_TABLE, 14561 },
	{ 0xdee6, 0xdee6, PDF_CMAP_SINGLE, 17804 },
	{ 0xdee6, 0xdee6, PDF_CMAP_SINGLE, 18564 },
	{ 0xdee6, 0xdee7, PDF_CMAP_TABLE, 14563 },
	{ 0xdee7, 0xdee8, PDF_CMAP_TABLE, 14565 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 18474 },
	{ 0xdee8, 0xdee9, PDF_CMAP_TABLE, 14567 },
	{ 0xdee9, 0xdeea, PDF_CMAP_TABLE, 14569 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 14739 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 14895 },
	{ 0xdeee, 0xdeef, PDF_CMAP_TABLE, 14571 },
	{ 0xdef4, 0xdef4, PDF_CMAP_SINGLE, 18667 },
	{ 0xdef4, 0xdef6, PDF_CMAP_TABLE, 14573 },
	{ 0xdef6, 0xdef8, PDF_CMAP_TABLE, 14576 },
	{ 0xdef8, 0xdef8, PDF_CMAP_SINGLE, 15058 },
	{ 0xdef8, 0xdef9, PDF_CMAP_TABLE, 14579 },
	{ 0xdef9, 0xdefa, PDF_CMAP_TABLE, 14581 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 14860 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 18138 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 15136 },
	{ 0xdefa, 0xdefb, PDF_CMAP_TABLE, 14583 },
	{ 0xdefb, 0xdefb, PDF_CMAP_SINGLE, 14714 },
	{ 0xdefb, 0xdefc, PDF_CMAP_TABLE, 14585 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 16902 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 18544 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 15308 },
	{ 0xdefe, 0xdefe, PDF_CMAP_SINGLE, 17332 },
	{ 0xdefe, 0xdf00, PDF_CMAP_TABLE, 14587 },
	{ 0xdf02, 0xdf03, PDF_CMAP_TABLE, 14590 },
	{ 0xdf03, 0xdf05, PDF_CMAP_TABLE, 14592 },
	{ 0xdf05, 0xdf05, PDF_CMAP_SINGLE, 14285 },
	{ 0xdf05, 0xdf06, PDF_CMAP_TABLE, 14595 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 14573 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 15379 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 18586 },
	{ 0xdf0a, 0xdf0a, PDF_CMAP_SINGLE, 14779 },
	{ 0xdf0a, 0xdf0a, PDF_CMAP_SINGLE, 14527 },
	{ 0xdf0a, 0xdf0d, PDF_CMAP_TABLE, 14597 },
	{ 0xdf0d, 0xdf0d, PDF_CMAP_SINGLE, 16548 },
	{ 0xdf0d, 0xdf0e, PDF_CMAP_TABLE, 14601 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18475 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18751 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 14153 },
	{ 0xdf0e, 0xdf0f, PDF_CMAP_TABLE, 14603 },
	{ 0xdf0f, 0xdf0f, PDF_CMAP_SINGLE, 19083 },
	{ 0xdf0f, 0xdf0f, PDF_CMAP_SINGLE, 14369 },
	{ 0xdf0f, 0xdf13, PDF_CMAP_TABLE, 14605 },
	{ 0xdf13, 0xdf13, PDF_CMAP_SINGLE, 16485 },
	{ 0xdf13, 0xdf13, PDF_CMAP_SINGLE, 14461 },
	{ 0xdf13, 0xdf15, PDF_CMAP_TABLE, 14610 },
	{ 0xdf15, 0xdf16, PDF_CMAP_TABLE, 14613 },
	{ 0xdf16, 0xdf16, PDF_CMAP_SINGLE, 17224 },
	{ 0xdf18, 0xdf18, PDF_CMAP_SINGLE, 15711 },
	{ 0xdf18, 0xdf18, PDF_CMAP_SINGLE, 15914 },
	{ 0xdf1a, 0xdf1a, PDF_CMAP_SINGLE, 19033 },
	{ 0xdf1a, 0xdf1a, PDF_CMAP_SINGLE, 15046 },
	{ 0xdf1a, 0xdf1b, PDF_CMAP_TABLE, 14615 },
	{ 0xdf1d, 0xdf1d, PDF_CMAP_SINGLE, 15506 },
	{ 0xdf1d, 0xdf1d, PDF_CMAP_SINGLE, 16792 },
	{ 0xdf1f, 0xdf1f, PDF_CMAP_SINGLE, 15404 },
	{ 0xdf1f, 0xdf20, PDF_CMAP_TABLE, 14617 },
	{ 0xdf20, 0xdf21, PDF_CMAP_TABLE, 14619 },
	{ 0xdf21, 0xdf22, PDF_CMAP_TABLE, 14621 },
	{ 0xdf22, 0xdf23, PDF_CMAP_TABLE, 14623 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18621 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 18068 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 14451 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 15053 },
	{ 0xdf25, 0xdf26, PDF_CMAP_TABLE, 14625 },
	{ 0xdf26, 0xdf26, PDF_CMAP_SINGLE, 16497 },
	{ 0xdf26, 0xdf26, PDF_CMAP_SINGLE, 14741 },
	{ 0xdf28, 0xdf28, PDF_CMAP_SINGLE, 16670 },
	{ 0xdf2a, 0xdf2d, PDF_CMAP_TABLE, 14627 },
	{ 0xdf2d, 0xdf2d, PDF_CMAP_SINGLE, 16471 },
	{ 0xdf2d, 0xdf2e, PDF_CMAP_TABLE, 14631 },
	{ 0xdf2e, 0xdf2f, PDF_CMAP_TABLE, 14633 },
	{ 0xdf2f, 0xdf2f, PDF_CMAP_SINGLE, 18509 },
	{ 0xdf2f, 0xdf2f, PDF_CMAP_SINGLE, 18587 },
	{ 0xdf2f, 0xdf30, PDF_CMAP_TABLE, 14635 },
	{ 0xdf30, 0xdf31, PDF_CMAP_TABLE, 14637 },
	{ 0xdf31, 0xdf32, PDF_CMAP_TABLE, 14639 },
	{ 0xdf34, 0xdf35, PDF_CMAP_TABLE, 14641 },
	{ 0xdf35, 0xdf35, PDF_CMAP_SINGLE, 16133 },
	{ 0xdf35, 0xdf36, PDF_CMAP_TABLE, 14643 },
	{ 0xdf36, 0xdf36, PDF_CMAP_SINGLE, 15684 },
	{ 0xdf38, 0xdf3a, PDF_CMAP_TABLE, 14645 },
	{ 0xdf3a, 0xdf3a, PDF_CMAP_SINGLE, 15659 },
	{ 0xdf3a, 0xdf3a, PDF_CMAP_SINGLE, 14386 },
	{ 0xdf3a, 0xdf3b, PDF_CMAP_TABLE, 14648 },
	{ 0xdf3b, 0xdf3d, PDF_CMAP_TABLE, 14650 },
	{ 0xdf3f, 0xdf3f, PDF_CMAP_SINGLE, 18476 },
	{ 0xdf3f, 0xdf3f, PDF_CMAP_SINGLE, 14750 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 17169 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 18054 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 15515 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 15129 },
	{ 0xdf43, 0xdf46, PDF_CMAP_TABLE, 14653 },
	{ 0xdf46, 0xdf48, PDF_CMAP_TABLE, 14657 },
	{ 0xdf48, 0xdf48, PDF_CMAP_SINGLE, 15186 },
	{ 0xdf4a, 0xdf4b, PDF_CMAP_TABLE, 14660 },
	{ 0xdf4b, 0xdf4c, PDF_CMAP_TABLE, 14662 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 18754 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 17231 },
	{ 0xdf4e, 0xdf4e, PDF_CMAP_SINGLE, 15801 },
	{ 0xdf50, 0xdf50, PDF_CMAP_SINGLE, 17889 },
	{ 0xdf50, 0xdf51, PDF_CMAP_TABLE, 14664 },
	{ 0xdf51, 0xdf51, PDF_CMAP_SINGLE, 16907 },
	{ 0xdf51, 0xdf52, PDF_CMAP_TABLE, 14666 },
	{ 0xdf52, 0xdf53, PDF_CMAP_TABLE, 14668 },
	{ 0xdf57, 0xdf57, PDF_CMAP_SINGLE, 15470 },
	{ 0xdf5a, 0xdf5c, PDF_CMAP_TABLE, 14670 },
	{ 0xdf5e, 0xdf5e, PDF_CMAP_SINGLE, 17303 },
	{ 0xdf61, 0xdf63, PDF_CMAP_TABLE, 14673 },
	{ 0xdf63, 0xdf64, PDF_CMAP_TABLE, 14676 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 14648 },
	{ 0xdf64, 0xdf65, PDF_CMAP_TABLE, 14678 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 19134 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 14436 },
	{ 0xdf65, 0xdf66, PDF_CMAP_TABLE, 14680 },
	{ 0xdf66, 0xdf66, PDF_CMAP_SINGLE, 16581 },
	{ 0xdf6a, 0xdf6a, PDF_CMAP_SINGLE, 16731 },
	{ 0xdf6a, 0xdf6a, PDF_CMAP_SINGLE, 14941 },
	{ 0xdf6c, 0xdf6c, PDF_CMAP_SINGLE, 18711 },
	{ 0xdf6c, 0xdf6d, PDF_CMAP_TABLE, 14682 },
	{ 0xdf6d, 0xdf6e, PDF_CMAP_TABLE, 14684 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_TABLE, 14686 },
	{ 0xdf73, 0xdf74, PDF_CMAP_TABLE, 14692 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 18486 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 17956 },
	{ 0xdf74, 0xdf75, PDF_CMAP_TABLE, 14694 },
	{ 0xdf75, 0xdf75, PDF_CMAP_SINGLE, 17257 },
	{ 0xdf75, 0xdf76, PDF_CMAP_TABLE, 14696 },
	{ 0xdf79, 0xdf79, PDF_CMAP_SINGLE, 15805 },
	{ 0xdf7d, 0xdf7f, PDF_CMAP_TABLE, 14698 },
	{ 0xdf7f, 0xdf80, PDF_CMAP_TABLE, 14701 },
	{ 0xdf80, 0xdf81, PDF_CMAP_TABLE, 14703 },
	{ 0xdf81, 0xdf81, PDF_CMAP_SINGLE, 15110 },
	{ 0xdf81, 0xdf82, PDF_CMAP_TABLE, 14705 },
	{ 0xdf82, 0xdf82, PDF_CMAP_SINGLE, 17090 },
	{ 0xdf82, 0xdf82, PDF_CMAP_SINGLE, 15439 },
	{ 0xdf84, 0xdf86, PDF_CMAP_TABLE, 14707 },
	{ 0xdf89, 0xdf8a, PDF_CMAP_TABLE, 14710 },
	{ 0xdf8c, 0xdf8f, PDF_CMAP_TABLE, 14712 },
	{ 0xdf8f, 0xdf8f, PDF_CMAP_SINGLE, 18091 },
	{ 0xdf8f, 0xdf8f, PDF_CMAP_SINGLE, 15052 },
	{ 0xdf8f, 0xdf90, PDF_CMAP_TABLE, 14716 },
	{ 0xdf96, 0xdf96, PDF_CMAP_SINGLE, 19029 },
	{ 0xdf96, 0xdf96, PDF_CMAP_SINGLE, 18504 },
	{ 0xdf96, 0xdf97, PDF_CMAP_TABLE, 14718 },
	{ 0xdf97, 0xdf99, PDF_CMAP_TABLE, 14720 },
	{ 0xdf99, 0xdf9a, PDF_CMAP_TABLE, 14723 },
	{ 0xdf9a, 0xdf9a, PDF_CMAP_SINGLE, 14345 },
	{ 0xdf9c, 0xdf9c, PDF_CMAP_SINGLE, 15596 },
	{ 0xdf9c, 0xdf9c, PDF_CMAP_SINGLE, 16837 },
	{ 0xdf9c, 0xdf9d, PDF_CMAP_TABLE, 14725 },
	{ 0xdf9d, 0xdf9f, PDF_CMAP_TABLE, 14727 },
	{ 0xdf9f, 0xdfa1, PDF_CMAP_TABLE, 14730 },
	{ 0xdfa1, 0xdfa2, PDF_CMAP_TABLE, 14733 },
	{ 0xdfa7, 0xdfa9, PDF_CMAP_TABLE, 14735 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 18699 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 17716 },
	{ 0xdfab, 0xdfab, PDF_CMAP_SINGLE, 14913 },
	{ 0xdfad, 0xdfad, PDF_CMAP_SINGLE, 17882 },
	{ 0xdfb0, 0xdfb3, PDF_CMAP_TABLE, 14738 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 15170 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 18702 },
	{ 0xdfb3, 0xdfb4, PDF_CMAP_TABLE, 14742 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 14276 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 16743 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 16443 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 16973 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 14158 },
	{ 0xdfb4, 0xdfb5, PDF_CMAP_TABLE, 14744 },
	{ 0xdfb5, 0xdfb5, PDF_CMAP_SINGLE, 17932 },
	{ 0xdfb5, 0xdfb5, PDF_CMAP_SINGLE, 15493 },
	{ 0xdfb5, 0xdfb7, PDF_CMAP_TABLE, 14746 },
	{ 0xdfb7, 0xdfb9, PDF_CMAP_TABLE, 14749 },
	{ 0xdfbc, 0xdfbc, PDF_CMAP_SINGLE, 18432 },
	{ 0xdfbc, 0xdfbd, PDF_CMAP_TABLE, 14752 },
	{ 0xdfbd, 0xdfbe, PDF_CMAP_TABLE, 14754 },
	{ 0xdfbe, 0xdfc0, PDF_CMAP_TABLE, 14756 },
	{ 0xdfc0, 0xdfc1, PDF_CMAP_TABLE, 14759 },
	{ 0xdfc1, 0xdfc2, PDF_CMAP_TABLE, 14761 },
	{ 0xdfc2, 0xdfc2, PDF_CMAP_SINGLE, 16840 },
	{ 0xdfc2, 0xdfc2, PDF_CMAP_SINGLE, 18678 },
	{ 0xdfc2, 0xdfc3, PDF_CMAP_TABLE, 14763 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 15900 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 16398 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 15360 },
	{ 0xdfc5, 0xdfc6, PDF_CMAP_TABLE, 14765 },
	{ 0xdfc6, 0xdfc6, PDF_CMAP_SINGLE, 14726 },
	{ 0xdfc6, 0xdfc7, PDF_CMAP_TABLE, 14767 },
	{ 0xdfc7, 0xdfc7, PDF_CMAP_SINGLE, 18526 },
	{ 0xdfc9, 0xdfcb, PDF_CMAP_TABLE, 14769 },
	{ 0xdfcb, 0xdfcc, PDF_CMAP_TABLE, 14772 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 15819 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 16362 },
	{ 0xdfcc, 0xdfce, PDF_CMAP_TABLE, 14774 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 16720 },
	{ 0xdfd4, 0xdfd4, PDF_CMAP_SINGLE, 18597 },
	{ 0xdfd4, 0xdfd5, PDF_CMAP_TABLE, 14777 },
	{ 0xdfd5, 0xdfd7, PDF_CMAP_TABLE, 14779 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 15448 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 14878 },
	{ 0xdfd9, 0xdfd9, PDF_CMAP_SINGLE, 15536 },
	{ 0xdfd9, 0xdfda, PDF_CMAP_TABLE, 14782 },
	{ 0xdfda, 0xdfdc, PDF_CMAP_TABLE, 14784 },
	{ 0xdfde, 0xdfde, PDF_CMAP_SINGLE, 18491 },
	{ 0xdfde, 0xdfde, PDF_CMAP_SINGLE, 15655 },
	{ 0xdfde, 0xdfdf, PDF_CMAP_TABLE, 14787 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 18134 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 15342 },
	{ 0xdfdf, 0xdfe0, PDF_CMAP_TABLE, 14789 },
	{ 0xdfe0, 0xdfe1, PDF_CMAP_TABLE, 14791 },
	{ 0xdfe1, 0xdfe2, PDF_CMAP_TABLE, 14793 },
	{ 0xdfe2, 0xdfe4, PDF_CMAP_TABLE, 14795 },
	{ 0xdfe6, 0xdfe8, PDF_CMAP_TABLE, 14798 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 18480 },
	{ 0xdfe8, 0xdfe9, PDF_CMAP_TABLE, 14801 },
	{ 0xdfe9, 0xdfea, PDF_CMAP_TABLE, 14803 },
	{ 0xdfea, 0xdfea, PDF_CMAP_SINGLE, 15323 },
	{ 0xdfea, 0xdfeb, PDF_CMAP_TABLE, 14805 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 18679 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 16532 },
	{ 0xdfeb, 0xdfec, PDF_CMAP_TABLE, 14807 },
	{ 0xdfec, 0xdfec, PDF_CMAP_SINGLE, 16492 },
	{ 0xdfec, 0xdfed, PDF_CMAP_TABLE, 14809 },
	{ 0xdfed, 0xdfed, PDF_CMAP_SINGLE, 15922 },
	{ 0xdfed, 0xdfef, PDF_CMAP_TABLE, 14811 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 15835 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 17355 },
	{ 0xdfef, 0xdff0, PDF_CMAP_TABLE, 14814 },
	{ 0xdff2, 0xdff2, PDF_CMAP_SINGLE, 16119 },
	{ 0xdff4, 0xdff4, PDF_CMAP_SINGLE, 17883 },
	{ 0xdff4, 0xdff5, PDF_CMAP_TABLE, 14816 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 18638 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 18730 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 16597 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 17103 },
	{ 0xdff5, 0xdff8, PDF_CMAP_TABLE, 14818 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 14703 },
	{ 0xdff8, 0xdff9, PDF_CMAP_TABLE, 14822 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 18911 },
	{ 0xdff9, 0xdffa, PDF_CMAP_TABLE, 14824 },
	{ 0xdffa, 0xdffa, PDF_CMAP_SINGLE, 18467 },
	{ 0xdffa, 0xdffb, PDF_CMAP_TABLE, 14826 },
	{ 0xdffb, 0xdffc, PDF_CMAP_TABLE, 14828 },
	{ 0xdffc, 0xdffc, PDF_CMAP_SINGLE, 16788 },
	{ 0xdffe, 0xdffe, PDF_CMAP_SINGLE, 14724 },
	{ 0xdffe, 0xdfff, PDF_CMAP_TABLE, 14830 },
	{ 0xdfff, 0xdfff, PDF_CMAP_SINGLE, 19072 },
	{ 0xdfff, 0xdfff, PDF_CMAP_SINGLE, 14493 },
	{ 0xe000, 0xe01e, PDF_CMAP_RANGE, 16395 },
	{ 0xe01f, 0xe01f, PDF_CMAP_SINGLE, 2106 },
	{ 0xe020, 0xe025, PDF_CMAP_RANGE, 16427 },
	{ 0xe026, 0xe026, PDF_CMAP_SINGLE, 2557 },
	{ 0xe027, 0xe046, PDF_CMAP_RANGE, 16434 },
	{ 0xe047, 0xe048, PDF_CMAP_RANGE, 18758 },
	{ 0xe049, 0xe05a, PDF_CMAP_RANGE, 16468 },
	{ 0xe05b, 0xe05b, PDF_CMAP_SINGLE, 781 },
	{ 0xe05c, 0xe062, PDF_CMAP_RANGE, 16487 },
	{ 0xe063, 0xe063, PDF_CMAP_SINGLE, 363 },
	{ 0xe064, 0xe072, PDF_CMAP_RANGE, 16495 },
	{ 0xe073, 0xe073, PDF_CMAP_SINGLE, 2144 },
	{ 0xe074, 0xe0a4, PDF_CMAP_RANGE, 16511 },
	{ 0xe0a5, 0xe0a5, PDF_CMAP_SINGLE, 16002 },
	{ 0xe0a6, 0xe0af, PDF_CMAP_RANGE, 16561 },
	{ 0xe0b0, 0xe0b0, PDF_CMAP_SINGLE, 18760 },
	{ 0xe0b1, 0xe0ca, PDF_CMAP_RANGE, 16572 },
	{ 0xe0cb, 0xe0cb, PDF_CMAP_SINGLE, 18761 },
	{ 0xe0cc, 0xe0dd, PDF_CMAP_RANGE, 16599 },
	{ 0xe0de, 0xe0de, PDF_CMAP_SINGLE, 18762 },
	{ 0xe0df, 0xe0f2, PDF_CMAP_RANGE, 16618 },
	{ 0xe0f3, 0xe0f3, PDF_CMAP_SINGLE, 3107 },
	{ 0xe0f4, 0xe0f9, PDF_CMAP_RANGE, 16639 },
	{ 0xe0fa, 0xe0fa, PDF_CMAP_SINGLE, 18763 },
	{ 0xe0fb, 0xe107, PDF_CMAP_RANGE, 16645 },
	{ 0xe108, 0xe108, PDF_CMAP_SINGLE, 18764 },
	{ 0xe109, 0xe12d, PDF_CMAP_RANGE, 16658 },
	{ 0xe12e, 0xe12e, PDF_CMAP_SINGLE, 6116 },
	{ 0xe12f, 0xe133, PDF_CMAP_RANGE, 16696 },
	{ 0xe134, 0xe134, PDF_CMAP_SINGLE, 16014 },
	{ 0xe135, 0xe143, PDF_CMAP_RANGE, 16702 },
	{ 0xe144, 0xe144, PDF_CMAP_SINGLE, 18765 },
	{ 0xe145, 0xe148, PDF_CMAP_RANGE, 16717 },
	{ 0xe149, 0xe149, PDF_CMAP_SINGLE, 8495 },
	{ 0xe14a, 0xe14b, PDF_CMAP_RANGE, 16721 },
	{ 0xe14c, 0xe14c, PDF_CMAP_SINGLE, 18766 },
	{ 0xe14d, 0xe15c, PDF_CMAP_RANGE, 16723 },
	{ 0xe15d, 0xe15d, PDF_CMAP_SINGLE, 18767 },
	{ 0xe15e, 0xe165, PDF_CMAP_RANGE, 16739 },
	{ 0xe166, 0xe167, PDF_CMAP_TABLE, 14832 },
	{ 0xe168, 0xe16e, PDF_CMAP_RANGE, 16749 },
	{ 0xe16f, 0xe16f, PDF_CMAP_SINGLE, 18769 },
	{ 0xe170, 0xe190, PDF_CMAP_RANGE, 16756 },
	{ 0xe191, 0xe191, PDF_CMAP_SINGLE, 4447 },
	{ 0xe192, 0xe193, PDF_CMAP_RANGE, 16789 },
	{ 0xe194, 0xe195, PDF_CMAP_RANGE, 18770 },
	{ 0xe196, 0xe1a3, PDF_CMAP_RANGE, 16792 },
	{ 0xe1a4, 0xe1a4, PDF_CMAP_SINGLE, 18772 },
	{ 0xe1a5, 0xe1b9, PDF_CMAP_RANGE, 16807 },
	{ 0xe1ba, 0xe1bb, PDF_CMAP_TABLE, 14834 },
	{ 0xe1bc, 0xe1c5, PDF_CMAP_RANGE, 16829 },
	{ 0xe1c6, 0xe1c6, PDF_CMAP_SINGLE, 18774 },
	{ 0xe1c7, 0xe1c8, PDF_CMAP_RANGE, 16839 },
	{ 0xe1c9, 0xe1c9, PDF_CMAP_SINGLE, 1219 },
	{ 0xe1ca, 0xe1df, PDF_CMAP_RANGE, 16842 },
	{ 0xe1e0, 0xe1e0, PDF_CMAP_SINGLE, 18775 },
	{ 0xe1e1, 0xe200, PDF_CMAP_RANGE, 16864 },
	{ 0xe201, 0xe201, PDF_CMAP_SINGLE, 18776 },
	{ 0xe202, 0xe22b, PDF_CMAP_RANGE, 16897 },
	{ 0xe22c, 0xe22d, PDF_CMAP_TABLE, 14836 },
	{ 0xe22e, 0xe22f, PDF_CMAP_RANGE, 16941 },
	{ 0xe230, 0xe230, PDF_CMAP_SINGLE, 3237 },
	{ 0xe231, 0xe257, PDF_CMAP_RANGE, 16944 },
	{ 0xe258, 0xe258, PDF_CMAP_SINGLE, 18777 },
	{ 0xe259, 0xe265, PDF_CMAP_RANGE, 16984 },
	{ 0xe266, 0xe267, PDF_CMAP_TABLE, 14838 },
	{ 0xe268, 0xe285, PDF_CMAP_RANGE, 16998 },
	{ 0xe286, 0xe286, PDF_CMAP_SINGLE, 15728 },
	{ 0xe287, 0xe2a0, PDF_CMAP_RANGE, 17029 },
	{ 0xe2a1, 0xe2a3, PDF_CMAP_TABLE, 14840 },
	{ 0xe2a4, 0xe2ab, PDF_CMAP_RANGE, 17057 },
	{ 0xe2ac, 0xe2ac, PDF_CMAP_SINGLE, 18780 },
	{ 0xe2ad, 0xe2bb, PDF_CMAP_RANGE, 17065 },
	{ 0xe2bc, 0xe2bc, PDF_CMAP_SINGLE, 288 },
	{ 0xe2bd, 0xe2ee, PDF_CMAP_RANGE, 17080 },
	{ 0xe2ef, 0xe2ef, PDF_CMAP_SINGLE, 7080 },
	{ 0xe2f0, 0xe2f1, PDF_CMAP_RANGE, 18781 },
	{ 0xe2f2, 0xe2fe, PDF_CMAP_RANGE, 17131 },
	{ 0xe2ff, 0xe300, PDF_CMAP_RANGE, 18783 },
	{ 0xe301, 0xe310, PDF_CMAP_RANGE, 17144 },
	{ 0xe311, 0xe315, PDF_CMAP_RANGE, 14123 },
	{ 0xe316, 0xe316, PDF_CMAP_SINGLE, 18072 },
	{ 0xe317, 0xe339, PDF_CMAP_RANGE, 14128 },
	{ 0xe33a, 0xe33b, PDF_CMAP_TABLE, 14843 },
	{ 0xe33c, 0xe33d, PDF_CMAP_RANGE, 18073 },
	{ 0xe33e, 0xe33f, PDF_CMAP_RANGE, 14166 },
	{ 0xe340, 0xe340, PDF_CMAP_SINGLE, 11752 },
	{ 0xe341, 0xe346, PDF_CMAP_RANGE, 14169 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 18075 },
	{ 0xe348, 0xe34b, PDF_CMAP_RANGE, 14175 },
	{ 0xe34c, 0xe34c, PDF_CMAP_SINGLE, 18076 },
	{ 0xe34d, 0xe34e, PDF_CMAP_RANGE, 14180 },
	{ 0xe34f, 0xe34f, PDF_CMAP_SINGLE, 289 },
	{ 0xe350, 0xe354, PDF_CMAP_RANGE, 14182 },
	{ 0xe355, 0xe355, PDF_CMAP_SINGLE, 18077 },
	{ 0xe356, 0xe359, PDF_CMAP_RANGE, 14187 },
	{ 0xe35a, 0xe35a, PDF_CMAP_SINGLE, 4203 },
	{ 0xe35b, 0xe362, PDF_CMAP_RANGE, 14192 },
	{ 0xe363, 0xe363, PDF_CMAP_SINGLE, 4902 },
	{ 0xe364, 0xe366, PDF_CMAP_RANGE, 14201 },
	{ 0xe367, 0xe367, PDF_CMAP_SINGLE, 18078 },
	{ 0xe368, 0xe377, PDF_CMAP_RANGE, 14205 },
	{ 0xe378, 0xe378, PDF_CMAP_SINGLE, 18079 },
	{ 0xe379, 0xe37b, PDF_CMAP_RANGE, 14221 },
	{ 0xe37c, 0xe37c, PDF_CMAP_SINGLE, 1643 },
	{ 0xe37d, 0xe37e, PDF_CMAP_RANGE, 14225 },
	{ 0xe37f, 0xe37f, PDF_CMAP_SINGLE, 4910 },
	{ 0xe380, 0xe393, PDF_CMAP_RANGE, 14228 },
	{ 0xe394, 0xe394, PDF_CMAP_SINGLE, 18080 },
	{ 0xe395, 0xe39d, PDF_CMAP_RANGE, 14248 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 18081 },
	{ 0xe39f, 0xe3a4, PDF_CMAP_RANGE, 14257 },
	{ 0xe3a5, 0xe3a5, PDF_CMAP_SINGLE, 18082 },
	{ 0xe3a6, 0xe3c4, PDF_CMAP_RANGE, 14264 },
	{ 0xe3c5, 0xe3c7, PDF_CMAP_TABLE, 14845 },
	{ 0xe3c8, 0xe3cc, PDF_CMAP_RANGE, 14297 },
	{ 0xe3cd, 0xe3cd, PDF_CMAP_SINGLE, 18084 },
	{ 0xe3ce, 0xe3d4, PDF_CMAP_RANGE, 14302 },
	{ 0xe3d5, 0xe3d7, PDF_CMAP_TABLE, 14848 },
	{ 0xe3d8, 0xe3db, PDF_CMAP_RANGE, 14311 },
	{ 0xe3dc, 0xe3dc, PDF_CMAP_SINGLE, 8877 },
	{ 0xe3dd, 0xe3e6, PDF_CMAP_RANGE, 14316 },
	{ 0xe3e7, 0xe3e7, PDF_CMAP_SINGLE, 18086 },
	{ 0xe3e8, 0xe3fb, PDF_CMAP_RANGE, 14327 },
	{ 0xe3fc, 0xe3fc, PDF_CMAP_SINGLE, 18087 },
	{ 0xe3fd, 0xe410, PDF_CMAP_RANGE, 14348 },
	{ 0xe411, 0xe413, PDF_CMAP_TABLE, 14851 },
	{ 0xe414, 0xe415, PDF_CMAP_RANGE, 14369 },
	{ 0xe416, 0xe418, PDF_CMAP_TABLE, 14854 },
	{ 0xe419, 0xe425, PDF_CMAP_RANGE, 14374 },
	{ 0xe426, 0xe426, PDF_CMAP_SINGLE, 18091 },
	{ 0xe427, 0xe42e, PDF_CMAP_RANGE, 14387 },
	{ 0xe42f, 0xe42f, PDF_CMAP_SINGLE, 18092 },
	{ 0xe430, 0xe447, PDF_CMAP_RANGE, 14396 },
	{ 0xe448, 0xe44a, PDF_CMAP_TABLE, 14857 },
	{ 0xe44b, 0xe45f, PDF_CMAP_RANGE, 14422 },
	{ 0xe460, 0xe460, PDF_CMAP_SINGLE, 18094 },
	{ 0xe461, 0xe466, PDF_CMAP_RANGE, 14444 },
	{ 0xe467, 0xe469, PDF_CMAP_RANGE, 18095 },
	{ 0xe46a, 0xe477, PDF_CMAP_RANGE, 14451 },
	{ 0xe478, 0xe47a, PDF_CMAP_TABLE, 14860 },
	{ 0xe47b, 0xe484, PDF_CMAP_RANGE, 14467 },
	{ 0xe485, 0xe485, PDF_CMAP_SINGLE, 14057 },
	{ 0xe486, 0xe48e, PDF_CMAP_RANGE, 14477 },
	{ 0xe48f, 0xe48f, PDF_CMAP_SINGLE, 18099 },
	{ 0xe490, 0xe4a0, PDF_CMAP_RANGE, 14486 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 18100 },
	{ 0xe4a2, 0xe4c4, PDF_CMAP_RANGE, 14503 },
	{ 0xe4c5, 0xe4c5, PDF_CMAP_SINGLE, 5009 },
	{ 0xe4c6, 0xe4d9, PDF_CMAP_RANGE, 14539 },
	{ 0xe4da, 0xe4da, PDF_CMAP_SINGLE, 5796 },
	{ 0xe4db, 0xe50c, PDF_CMAP_RANGE, 14560 },
	{ 0xe50d, 0xe50d, PDF_CMAP_SINGLE, 18101 },
	{ 0xe50e, 0xe515, PDF_CMAP_RANGE, 14610 },
	{ 0xe516, 0xe516, PDF_CMAP_SINGLE, 18102 },
	{ 0xe517, 0xe525, PDF_CMAP_RANGE, 14618 },
	{ 0xe526, 0xe528, PDF_CMAP_TABLE, 14863 },
	{ 0xe529, 0xe544, PDF_CMAP_RANGE, 14634 },
	{ 0xe545, 0xe545, PDF_CMAP_SINGLE, 12402 },
	{ 0xe546, 0xe54d, PDF_CMAP_RANGE, 14663 },
	{ 0xe54e, 0xe54e, PDF_CMAP_SINGLE, 18105 },
	{ 0xe54f, 0xe588, PDF_CMAP_RANGE, 14672 },
	{ 0xe589, 0xe589, PDF_CMAP_SINGLE, 14728 },
	{ 0xe58a, 0xe5a8, PDF_CMAP_RANGE, 14731 },
	{ 0xe5a9, 0xe5a9, PDF_CMAP_SINGLE, 18106 },
	{ 0xe5aa, 0xe5b1, PDF_CMAP_RANGE, 14763 },
	{ 0xe5b2, 0xe5b2, PDF_CMAP_SINGLE, 18107 },
	{ 0xe5b3, 0xe5d1, PDF_CMAP_RANGE, 14772 },
	{ 0xe5d2, 0xe5d3, PDF_CMAP_RANGE, 281 },
	{ 0xe5d4, 0xe5d5, PDF_CMAP_TABLE, 14866 },
	{ 0xe5d6, 0xe5ea, PDF_CMAP_RANGE, 14803 },
	{ 0xe5eb, 0xe5eb, PDF_CMAP_SINGLE, 16300 },
	{ 0xe5ec, 0xe5f3, PDF_CMAP_RANGE, 14825 },
	{ 0xe5f4, 0xe5f4, PDF_CMAP_SINGLE, 10620 },
	{ 0xe5f5, 0xe607, PDF_CMAP_RANGE, 14834 },
	{ 0xe608, 0xe608, PDF_CMAP_SINGLE, 18108 },
	{ 0xe609, 0xe614, PDF_CMAP_RANGE, 14853 },
	{ 0xe615, 0xe615, PDF_CMAP_SINGLE, 18109 },
	{ 0xe616, 0xe649, PDF_CMAP_RANGE, 14865 },
	{ 0xe64a, 0xe64a, PDF_CMAP_SINGLE, 18110 },
	{ 0xe64b, 0xe669, PDF_CMAP_RANGE, 14917 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 18111 },
	{ 0xe66b, 0xe681, PDF_CMAP_RANGE, 14948 },
	{ 0xe682, 0xe682, PDF_CMAP_SINGLE, 18112 },
	{ 0xe683, 0xe6a4, PDF_CMAP_RANGE, 14971 },
	{ 0xe6a5, 0xe6a5, PDF_CMAP_SINGLE, 18113 },
	{ 0xe6a6, 0xe6a7, PDF_CMAP_RANGE, 15005 },
	{ 0xe6a8, 0xe6a8, PDF_CMAP_SINGLE, 18114 },
	{ 0xe6a9, 0xe6aa, PDF_CMAP_RANGE, 15008 },
	{ 0xe6ab, 0xe6ab, PDF_CMAP_SINGLE, 18115 },
	{ 0xe6ac, 0xe6c4, PDF_CMAP_RANGE, 15010 },
	{ 0xe6c5, 0xe6c6, PDF_CMAP_TABLE, 14868 },
	{ 0xe6c7, 0xe6f7, PDF_CMAP_RANGE, 15036 },
	{ 0xe6f8, 0xe6f8, PDF_CMAP_SINGLE, 18117 },
	{ 0xe6f9, 0xe726, PDF_CMAP_RANGE, 15086 },
	{ 0xe727, 0xe728, PDF_CMAP_TABLE, 14870 },
	{ 0xe729, 0xe768, PDF_CMAP_RANGE, 15134 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 18119 },
	{ 0xe76a, 0xe775, PDF_CMAP_RANGE, 15198 },
	{ 0xe776, 0xe776, PDF_CMAP_SINGLE, 18120 },
	{ 0xe777, 0xe77a, PDF_CMAP_RANGE, 15210 },
	{ 0xe77b, 0xe77b, PDF_CMAP_SINGLE, 18121 },
	{ 0xe77c, 0xe7bf, PDF_CMAP_RANGE, 15215 },
	{ 0xe7c0, 0xe7c0, PDF_CMAP_SINGLE, 18122 },
	{ 0xe7c1, 0xe7d2, PDF_CMAP_RANGE, 15283 },
	{ 0xe7d3, 0xe7d3, PDF_CMAP_SINGLE, 6171 },
	{ 0xe7d4, 0xe7fc, PDF_CMAP_RANGE, 15302 },
	{ 0xe7fd, 0xe7fd, PDF_CMAP_SINGLE, 15919 },
	{ 0xe7fe, 0xe809, PDF_CMAP_RANGE, 15344 },
	{ 0xe80a, 0xe80a, PDF_CMAP_SINGLE, 18123 },
	{ 0xe80b, 0xe822, PDF_CMAP_RANGE, 15357 },
	{ 0xe823, 0xe823, PDF_CMAP_SINGLE, 18124 },
	{ 0xe824, 0xe86a, PDF_CMAP_RANGE, 15382 },
	{ 0xe86b, 0xe86b, PDF_CMAP_SINGLE, 18125 },
	{ 0xe86c, 0xe883, PDF_CMAP_RANGE, 15453 },
	{ 0xe884, 0xe884, PDF_CMAP_SINGLE, 15478 },
	{ 0xe885, 0xe892, PDF_CMAP_RANGE, 15478 },
	{ 0xe893, 0xe893, PDF_CMAP_SINGLE, 11044 },
	{ 0xe894, 0xe936, PDF_CMAP_RANGE, 15493 },
	{ 0xe937, 0xe938, PDF_CMAP_RANGE, 18126 },
	{ 0xe939, 0xe961, PDF_CMAP_RANGE, 15658 },
	{ 0xe962, 0xe962, PDF_CMAP_SINGLE, 18128 },
	{ 0xe963, 0xe967, PDF_CMAP_RANGE, 15700 },
	{ 0xe968, 0xe969, PDF_CMAP_RANGE, 18129 },
	{ 0xe96a, 0xe96a, PDF_CMAP_SINGLE, 15705 },
	{ 0xe96b, 0xe96c, PDF_CMAP_RANGE, 18131 },
	{ 0xe96d, 0xe96d, PDF_CMAP_SINGLE, 15706 },
	{ 0xe96e, 0xe973, PDF_CMAP_RANGE, 18133 },
	{ 0xe974, 0xe974, PDF_CMAP_SINGLE, 15707 },
	{ 0xe975, 0xe97f, PDF_CMAP_RANGE, 18139 },
	{ 0xe980, 0xe980, PDF_CMAP_SINGLE, 15708 },
	{ 0xe981, 0xe984, PDF_CMAP_RANGE, 18150 },
	{ 0xe985, 0xe98d, PDF_CMAP_TABLE, 14872 },
	{ 0xe98e, 0xe98f, PDF_CMAP_RANGE, 15711 },
	{ 0xe990, 0xe992, PDF_CMAP_RANGE, 18158 },
	{ 0xe993, 0xe996, PDF_CMAP_TABLE, 14881 },
	{ 0xe997, 0xe998, PDF_CMAP_RANGE, 15715 },
	{ 0xe999, 0xe9a2, PDF_CMAP_RANGE, 18163 },
	{ 0xe9a3, 0xe9a3, PDF_CMAP_SINGLE, 17357 },
	{ 0xe9a4, 0xe9a8, PDF_CMAP_RANGE, 18173 },
	{ 0xe9a9, 0xe9aa, PDF_CMAP_RANGE, 17358 },
	{ 0xe9ab, 0xe9ac, PDF_CMAP_TABLE, 14885 },
	{ 0xe9ad, 0xe9af, PDF_CMAP_RANGE, 18179 },
	{ 0xe9b0, 0xe9b0, PDF_CMAP_SINGLE, 17364 },
	{ 0xe9b1, 0xe9b3, PDF_CMAP_RANGE, 18182 },
	{ 0xe9b4, 0xe9b4, PDF_CMAP_SINGLE, 15717 },
	{ 0xe9b5, 0xe9b7, PDF_CMAP_RANGE, 18185 },
	{ 0xe9b8, 0xe9b8, PDF_CMAP_SINGLE, 15718 },
	{ 0xe9b9, 0xe9bd, PDF_CMAP_RANGE, 18188 },
	{ 0xe9be, 0xe9be, PDF_CMAP_SINGLE, 15719 },
	{ 0xe9bf, 0xe9c0, PDF_CMAP_RANGE, 18193 },
	{ 0xe9c1, 0xe9c4, PDF_CMAP_TABLE, 14887 },
	{ 0xe9c5, 0xe9c6, PDF_CMAP_RANGE, 17365 },
	{ 0xe9c7, 0xe9cc, PDF_CMAP_RANGE, 18197 },
	{ 0xe9cd, 0xe9cf, PDF_CMAP_TABLE, 14891 },
	{ 0xe9d0, 0xe9d1, PDF_CMAP_RANGE, 18204 },
	{ 0xe9d2, 0xe9d7, PDF_CMAP_TABLE, 14894 },
	{ 0xe9d8, 0xe9de, PDF_CMAP_RANGE, 18208 },
	{ 0xe9df, 0xe9df, PDF_CMAP_SINGLE, 15725 },
	{ 0xe9e0, 0xe9e3, PDF_CMAP_RANGE, 18215 },
	{ 0xe9e4, 0xe9e4, PDF_CMAP_SINGLE, 17369 },
	{ 0xe9e5, 0xe9eb, PDF_CMAP_RANGE, 18219 },
	{ 0xe9ec, 0xe9ec, PDF_CMAP_SINGLE, 17370 },
	{ 0xe9ed, 0xe9f3, PDF_CMAP_RANGE, 18226 },
	{ 0xe9f4, 0xe9f4, PDF_CMAP_SINGLE, 17372 },
	{ 0xe9f5, 0xe9f9, PDF_CMAP_RANGE, 18233 },
	{ 0xe9fa, 0xe9fa, PDF_CMAP_SINGLE, 15726 },
	{ 0xe9fb, 0xe9fd, PDF_CMAP_RANGE, 18238 },
	{ 0xe9fe, 0xe9ff, PDF_CMAP_TABLE, 14900 },
	{ 0xea00, 0xea04, PDF_CMAP_RANGE, 18242 },
	{ 0xea05, 0xea05, PDF_CMAP_SINGLE, 15728 },
	{ 0xea06, 0xea07, PDF_CMAP_RANGE, 18247 },
	{ 0xea08, 0xea08, PDF_CMAP_SINGLE, 15729 },
	{ 0xea09, 0xea0f, PDF_CMAP_RANGE, 18249 },
	{ 0xea10, 0xea14, PDF_CMAP_TABLE, 14902 },
	{ 0xea15, 0xea1b, PDF_CMAP_RANGE, 18258 },
	{ 0xea1c, 0xea1c, PDF_CMAP_SINGLE, 15731 },
	{ 0xea1d, 0xea1f, PDF_CMAP_RANGE, 18265 },
	{ 0xea20, 0xea20, PDF_CMAP_SINGLE, 17374 },
	{ 0xea21, 0xea23, PDF_CMAP_RANGE, 18268 },
	{ 0xea24, 0xea24, PDF_CMAP_SINGLE, 15732 },
	{ 0xea25, 0xea27, PDF_CMAP_RANGE, 18271 },
	{ 0xea28, 0xea28, PDF_CMAP_SINGLE, 15733 },
	{ 0xea29, 0xea37, PDF_CMAP_RANGE, 18274 },
	{ 0xea38, 0xea38, PDF_CMAP_SINGLE, 17376 },
	{ 0xea39, 0xea3a, PDF_CMAP_RANGE, 18289 },
	{ 0xea3b, 0xea3b, PDF_CMAP_SINGLE, 17378 },
	{ 0xea3c, 0xea40, PDF_CMAP_RANGE, 18291 },
	{ 0xea41, 0xea41, PDF_CMAP_SINGLE, 17379 },
	{ 0xea42, 0xea43, PDF_CMAP_RANGE, 18296 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 17381 },
	{ 0xea45, 0xea4c, PDF_CMAP_RANGE, 18298 },
	{ 0xea4d, 0xea4d, PDF_CMAP_SINGLE, 17375 },
	{ 0xea4e, 0xea4f, PDF_CMAP_RANGE, 18306 },
	{ 0xea50, 0xea56, PDF_CMAP_TABLE, 14907 },
	{ 0xea57, 0xea5c, PDF_CMAP_RANGE, 18311 },
	{ 0xea5d, 0xea5d, PDF_CMAP_SINGLE, 17405 },
	{ 0xea5e, 0xea61, PDF_CMAP_RANGE, 18317 },
	{ 0xea62, 0xea62, PDF_CMAP_SINGLE, 15735 },
	{ 0xea63, 0xea76, PDF_CMAP_RANGE, 18321 },
	{ 0xea77, 0xea79, PDF_CMAP_TABLE, 14914 },
	{ 0xea7a, 0xea85, PDF_CMAP_RANGE, 18342 },
	{ 0xea86, 0xea86, PDF_CMAP_SINGLE, 15738 },
	{ 0xea87, 0xea8b, PDF_CMAP_RANGE, 18354 },
	{ 0xea8c, 0xea8e, PDF_CMAP_TABLE, 14917 },
	{ 0xea8f, 0xea92, PDF_CMAP_RANGE, 18360 },
	{ 0xea93, 0xea98, PDF_CMAP_TABLE, 14920 },
	{ 0xea99, 0xea9f, PDF_CMAP_RANGE, 18366 },
	{ 0xeaa0, 0xeaa2, PDF_CMAP_TABLE, 14926 },
	{ 0xeaa3, 0xeaaa, PDF_CMAP_RANGE, 18374 },
	{ 0xeaab, 0xeaab, PDF_CMAP_SINGLE, 15742 },
	{ 0xeaac, 0xeaad, PDF_CMAP_RANGE, 18382 },
	{ 0xeaae, 0xeab0, PDF_CMAP_TABLE, 14929 },
	{ 0xeab1, 0xeab3, PDF_CMAP_RANGE, 18385 },
	{ 0xeab4, 0xeab5, PDF_CMAP_TABLE, 14932 },
	{ 0xeab6, 0xeabc, PDF_CMAP_RANGE, 18388 },
	{ 0xeabd, 0xeabd, PDF_CMAP_SINGLE, 15743 },
	{ 0xeabe, 0xeac1, PDF_CMAP_RANGE, 18395 },
	{ 0xeac2, 0xeac8, PDF_CMAP_TABLE, 14934 },
	{ 0xeac9, 0xead1, PDF_CMAP_RANGE, 18402 },
	{ 0xead2, 0xead2, PDF_CMAP_SINGLE, 15747 },
	{ 0xead3, 0xeada, PDF_CMAP_RANGE, 18411 },
	{ 0xeadb, 0xeadd, PDF_CMAP_TABLE, 14941 },
	{ 0xeade, 0xeae3, PDF_CMAP_RANGE, 18420 },
	{ 0xeae4, 0xeae6, PDF_CMAP_RANGE, 15750 },
	{ 0xeae7, 0xeaec, PDF_CMAP_RANGE, 18426 },
	{ 0xeaed, 0xeaef, PDF_CMAP_TABLE, 14944 },
	{ 0xeaf0, 0xeaf2, PDF_CMAP_RANGE, 18433 },
	{ 0xeaf3, 0xeaf4, PDF_CMAP_TABLE, 14947 },
	{ 0xeaf5, 0xeaf8, PDF_CMAP_RANGE, 18436 },
	{ 0xeaf9, 0xeaf9, PDF_CMAP_SINGLE, 17330 },
	{ 0xeafa, 0xeafc, PDF_CMAP_RANGE, 18440 },
	{ 0xeafd, 0xeafd, PDF_CMAP_SINGLE, 15756 },
	{ 0xeafe, 0xeb00, PDF_CMAP_RANGE, 18443 },
	{ 0xeb01, 0xeb01, PDF_CMAP_SINGLE, 15757 },
	{ 0xeb02, 0xeb05, PDF_CMAP_RANGE, 18446 },
	{ 0xeb06, 0xeb06, PDF_CMAP_SINGLE, 15758 },
	{ 0xeb07, 0xeb0f, PDF_CMAP_RANGE, 18450 },
	{ 0xeb10, 0xeb10, PDF_CMAP_SINGLE, 15759 },
	{ 0xeb11, 0xeb13, PDF_CMAP_RANGE, 18459 },
	{ 0xeb14, 0xeb14, PDF_CMAP_SINGLE, 15760 },
	{ 0xeb15, 0xeb1d, PDF_CMAP_RANGE, 18462 },
	{ 0xeb1e, 0xeb1e, PDF_CMAP_SINGLE, 15762 },
	{ 0xeb1f, 0xeb21, PDF_CMAP_RANGE, 18471 },
	{ 0xeb22, 0xeb27, PDF_CMAP_TABLE, 14949 },
	{ 0xeb28, 0xeb29, PDF_CMAP_RANGE, 15766 },
	{ 0xeb2a, 0xeb2a, PDF_CMAP_SINGLE, 18477 },
	{ 0xeb2c, 0xeb39, PDF_CMAP_RANGE, 18478 },
	{ 0xeb3a, 0xeb3d, PDF_CMAP_RANGE, 15768 },
	{ 0xeb3e, 0xeb3f, PDF_CMAP_RANGE, 18492 },
	{ 0xeb40, 0xeb42, PDF_CMAP_TABLE, 14955 },
	{ 0xeb43, 0xeb44, PDF_CMAP_RANGE, 18494 },
	{ 0xeb45, 0xeb47, PDF_CMAP_TABLE, 14958 },
	{ 0xeb48, 0xeb49, PDF_CMAP_RANGE, 15777 },
	{ 0xeb4a, 0xeb4a, PDF_CMAP_SINGLE, 18497 },
	{ 0xeb4b, 0xeb4c, PDF_CMAP_RANGE, 15779 },
	{ 0xeb4d, 0xeb4e, PDF_CMAP_RANGE, 18498 },
	{ 0xeb4f, 0xeb52, PDF_CMAP_RANGE, 15781 },
	{ 0xeb53, 0xeb54, PDF_CMAP_TABLE, 14961 },
	{ 0xeb55, 0xeb56, PDF_CMAP_RANGE, 18501 },
	{ 0xeb57, 0xeb57, PDF_CMAP_SINGLE, 15786 },
	{ 0xeb58, 0xeb59, PDF_CMAP_RANGE, 18503 },
	{ 0xeb5a, 0xeb61, PDF_CMAP_RANGE, 15787 },
	{ 0xeb62, 0xeb65, PDF_CMAP_RANGE, 18505 },
	{ 0xeb66, 0xeb67, PDF_CMAP_TABLE, 14963 },
	{ 0xeb68, 0xeb6d, PDF_CMAP_RANGE, 15796 },
	{ 0xeb6e, 0xeb6e, PDF_CMAP_SINGLE, 10969 },
	{ 0xeb6f, 0xeb71, PDF_CMAP_RANGE, 18510 },
	{ 0xeb72, 0xeb77, PDF_CMAP_TABLE, 14965 },
	{ 0xeb78, 0xeb79, PDF_CMAP_RANGE, 15805 },
	{ 0xeb7a, 0xeb7d, PDF_CMAP_TABLE, 14971 },
	{ 0xeb7e, 0xeb7f, PDF_CMAP_RANGE, 18518 },
	{ 0xeb80, 0xeb82, PDF_CMAP_RANGE, 15809 },
	{ 0xeb83, 0xeb84, PDF_CMAP_RANGE, 18520 },
	{ 0xeb85, 0xeb8b, PDF_CMAP_TABLE, 14975 },
	{ 0xeb8c, 0xeb8e, PDF_CMAP_RANGE, 18524 },
	{ 0xeb8f, 0xeb90, PDF_CMAP_TABLE, 14982 },
	{ 0xeb91, 0xeb93, PDF_CMAP_RANGE, 15817 },
	{ 0xeb94, 0xeb95, PDF_CMAP_TABLE, 14984 },
	{ 0xeb96, 0xeb97, PDF_CMAP_RANGE, 15821 },
	{ 0xeb98, 0xeb9a, PDF_CMAP_RANGE, 18529 },
	{ 0xeb9b, 0xeb9b, PDF_CMAP_SINGLE, 15823 },
	{ 0xeb9c, 0xeb9d, PDF_CMAP_RANGE, 18532 },
	{ 0xeb9e, 0xeb9f, PDF_CMAP_TABLE, 14986 },
	{ 0xeba0, 0xeba1, PDF_CMAP_RANGE, 15825 },
	{ 0xeba2, 0xeba2, PDF_CMAP_SINGLE, 18535 },
	{ 0xeba3, 0xeba4, PDF_CMAP_RANGE, 15827 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 18536 },
	{ 0xeba7, 0xeba8, PDF_CMAP_RANGE, 15829 },
	{ 0xeba9, 0xebaa, PDF_CMAP_TABLE, 14988 },
	{ 0xebab, 0xebad, PDF_CMAP_RANGE, 15832 },
	{ 0xebae, 0xebb1, PDF_CMAP_TABLE, 14990 },
	{ 0xebb2, 0xebb3, PDF_CMAP_RANGE, 18541 },
	{ 0xebb4, 0xebb9, PDF_CMAP_RANGE, 15837 },
	{ 0xebba, 0xebbd, PDF_CMAP_TABLE, 14994 },
	{ 0xebbe, 0xebc2, PDF_CMAP_RANGE, 15845 },
	{ 0xebc3, 0xebc4, PDF_CMAP_TABLE, 14998 },
	{ 0xebc5, 0xebc6, PDF_CMAP_RANGE, 18546 },
	{ 0xebc7, 0xebca, PDF_CMAP_TABLE, 15000 },
	{ 0xebcb, 0xebcd, PDF_CMAP_RANGE, 15853 },
	{ 0xebce, 0xebd5, PDF_CMAP_TABLE, 15004 },
	{ 0xebd6, 0xebd9, PDF_CMAP_RANGE, 15859 },
	{ 0xebda, 0xebdb, PDF_CMAP_RANGE, 18554 },
	{ 0xebdc, 0xebdd, PDF_CMAP_RANGE, 15863 },
	{ 0xebde, 0xebe1, PDF_CMAP_TABLE, 15012 },
	{ 0xebe2, 0xebe3, PDF_CMAP_RANGE, 15867 },
	{ 0xebe4, 0xebe4, PDF_CMAP_SINGLE, 18558 },
	{ 0xebe5, 0xebe7, PDF_CMAP_RANGE, 15869 },
	{ 0xebe8, 0xebe9, PDF_CMAP_RANGE, 18559 },
	{ 0xebea, 0xebea, PDF_CMAP_SINGLE, 15872 },
	{ 0xebeb, 0xebec, PDF_CMAP_RANGE, 18561 },
	{ 0xebed, 0xebee, PDF_CMAP_RANGE, 15873 },
	{ 0xebef, 0xebf1, PDF_CMAP_TABLE, 15016 },
	{ 0xebf2, 0xebf3, PDF_CMAP_RANGE, 15876 },
	{ 0xebf4, 0xebf5, PDF_CMAP_RANGE, 18565 },
	{ 0xebf6, 0xebff, PDF_CMAP_RANGE, 15878 },
	{ 0xec00, 0xec02, PDF_CMAP_TABLE, 15019 },
	{ 0xec03, 0xec07, PDF_CMAP_RANGE, 15890 },
	{ 0xec08, 0xec0a, PDF_CMAP_RANGE, 18568 },
	{ 0xec0b, 0xec12, PDF_CMAP_RANGE, 15895 },
	{ 0xec13, 0xec15, PDF_CMAP_TABLE, 15022 },
	{ 0xec16, 0xec18, PDF_CMAP_RANGE, 15905 },
	{ 0xec19, 0xec1c, PDF_CMAP_RANGE, 18572 },
	{ 0xec1d, 0xec1f, PDF_CMAP_RANGE, 15908 },
	{ 0xec20, 0xec20, PDF_CMAP_SINGLE, 18576 },
	{ 0xec21, 0xec2a, PDF_CMAP_RANGE, 15911 },
	{ 0xec2b, 0xec2b, PDF_CMAP_SINGLE, 18577 },
	{ 0xec2c, 0xec2e, PDF_CMAP_RANGE, 15921 },
	{ 0xec2f, 0xec2f, PDF_CMAP_SINGLE, 18578 },
	{ 0xec30, 0xec31, PDF_CMAP_RANGE, 15924 },
	{ 0xec32, 0xec32, PDF_CMAP_SINGLE, 18579 },
	{ 0xec33, 0xec3e, PDF_CMAP_RANGE, 15926 },
	{ 0xec3f, 0xec41, PDF_CMAP_RANGE, 18580 },
	{ 0xec42, 0xec44, PDF_CMAP_TABLE, 15025 },
	{ 0xec45, 0xec49, PDF_CMAP_RANGE, 18584 },
	{ 0xec4a, 0xec4a, PDF_CMAP_SINGLE, 15939 },
	{ 0xec4b, 0xec4c, PDF_CMAP_RANGE, 18589 },
	{ 0xec4d, 0xec4d, PDF_CMAP_SINGLE, 15940 },
	{ 0xec4e, 0xec4f, PDF_CMAP_RANGE, 18591 },
	{ 0xec50, 0xec51, PDF_CMAP_RANGE, 15942 },
	{ 0xec52, 0xec55, PDF_CMAP_TABLE, 15028 },
	{ 0xec56, 0xec58, PDF_CMAP_RANGE, 18595 },
	{ 0xec59, 0xec5b, PDF_CMAP_TABLE, 15032 },
	{ 0xec5c, 0xec5d, PDF_CMAP_RANGE, 18599 },
	{ 0xec5e, 0xec5e, PDF_CMAP_SINGLE, 3340 },
	{ 0xec5f, 0xec64, PDF_CMAP_RANGE, 18601 },
	{ 0xec65, 0xec68, PDF_CMAP_TABLE, 15035 },
	{ 0xec69, 0xec7b, PDF_CMAP_RANGE, 18608 },
	{ 0xec7c, 0xec7d, PDF_CMAP_TABLE, 15039 },
	{ 0xec7e, 0xec81, PDF_CMAP_RANGE, 18627 },
	{ 0xec82, 0xec82, PDF_CMAP_SINGLE, 15951 },
	{ 0xec83, 0xec86, PDF_CMAP_RANGE, 18631 },
	{ 0xec87, 0xec8a, PDF_CMAP_RANGE, 15952 },
	{ 0xec8b, 0xec8d, PDF_CMAP_TABLE, 15041 },
	{ 0xec8e, 0xec8f, PDF_CMAP_RANGE, 15957 },
	{ 0xec90, 0xec91, PDF_CMAP_RANGE, 18637 },
	{ 0xec92, 0xec92, PDF_CMAP_SINGLE, 15960 },
	{ 0xec93, 0xec94, PDF_CMAP_RANGE, 18639 },
	{ 0xec95, 0xec99, PDF_CMAP_TABLE, 15044 },
	{ 0xec9a, 0xec9d, PDF_CMAP_RANGE, 18643 },
	{ 0xec9e, 0xec9f, PDF_CMAP_RANGE, 15964 },
	{ 0xeca0, 0xeca0, PDF_CMAP_SINGLE, 18647 },
	{ 0xeca1, 0xeca2, PDF_CMAP_RANGE, 15966 },
	{ 0xeca3, 0xeca4, PDF_CMAP_RANGE, 18648 },
	{ 0xeca5, 0xeca6, PDF_CMAP_TABLE, 15049 },
	{ 0xeca7, 0xeca8, PDF_CMAP_RANGE, 18650 },
	{ 0xeca9, 0xecaa, PDF_CMAP_RANGE, 15970 },
	{ 0xecab, 0xecac, PDF_CMAP_TABLE, 15051 },
	{ 0xecad, 0xecae, PDF_CMAP_RANGE, 18653 },
	{ 0xecaf, 0xecb3, PDF_CMAP_RANGE, 15973 },
	{ 0xecb4, 0xecb4, PDF_CMAP_SINGLE, 18655 },
	{ 0xecb5, 0xecb7, PDF_CMAP_RANGE, 15978 },
	{ 0xecb8, 0xecbb, PDF_CMAP_RANGE, 18656 },
	{ 0xecbc, 0xecdd, PDF_CMAP_RANGE, 15981 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 18660 },
	{ 0xecdf, 0xece3, PDF_CMAP_RANGE, 16015 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 18661 },
	{ 0xece5, 0xecff, PDF_CMAP_RANGE, 16021 },
	{ 0xed00, 0xed00, PDF_CMAP_SINGLE, 18662 },
	{ 0xed01, 0xed03, PDF_CMAP_RANGE, 16049 },
	{ 0xed04, 0xed04, PDF_CMAP_SINGLE, 18663 },
	{ 0xed05, 0xed06, PDF_CMAP_RANGE, 16052 },
	{ 0xed07, 0xed08, PDF_CMAP_RANGE, 18664 },
	{ 0xed09, 0xed0b, PDF_CMAP_TABLE, 15053 },
	{ 0xed0c, 0xed11, PDF_CMAP_RANGE, 18667 },
	{ 0xed12, 0xed14, PDF_CMAP_TABLE, 15056 },
	{ 0xed15, 0xed17, PDF_CMAP_RANGE, 18674 },
	{ 0xed18, 0xed19, PDF_CMAP_RANGE, 16058 },
	{ 0xed1a, 0xed1e, PDF_CMAP_TABLE, 15059 },
	{ 0xed1f, 0xed21, PDF_CMAP_RANGE, 16062 },
	{ 0xed22, 0xed22, PDF_CMAP_SINGLE, 18680 },
	{ 0xed23, 0xed25, PDF_CMAP_RANGE, 16065 },
	{ 0xed26, 0xed27, PDF_CMAP_RANGE, 18681 },
	{ 0xed28, 0xed2a, PDF_CMAP_TABLE, 15064 },
	{ 0xed2b, 0xed2c, PDF_CMAP_RANGE, 16070 },
	{ 0xed2d, 0xed2d, PDF_CMAP_SINGLE, 18684 },
	{ 0xed2e, 0xed30, PDF_CMAP_RANGE, 16072 },
	{ 0xed31, 0xed32, PDF_CMAP_RANGE, 18685 },
	{ 0xed33, 0xed34, PDF_CMAP_TABLE, 15067 },
	{ 0xed35, 0xed36, PDF_CMAP_RANGE, 16076 },
	{ 0xed37, 0xed38, PDF_CMAP_TABLE, 15069 },
	{ 0xed39, 0xed3a, PDF_CMAP_RANGE, 18689 },
	{ 0xed3b, 0xed3b, PDF_CMAP_SINGLE, 16079 },
	{ 0xed3c, 0xed3d, PDF_CMAP_RANGE, 18691 },
	{ 0xed3e, 0xed3f, PDF_CMAP_RANGE, 16080 },
	{ 0xed40, 0xed42, PDF_CMAP_RANGE, 18693 },
	{ 0xed43, 0xed44, PDF_CMAP_RANGE, 16082 },
	{ 0xed45, 0xed45, PDF_CMAP_SINGLE, 18696 },
	{ 0xed46, 0xed49, PDF_CMAP_RANGE, 16084 },
	{ 0xed4a, 0xed4b, PDF_CMAP_RANGE, 18697 },
	{ 0xed4c, 0xed4c, PDF_CMAP_SINGLE, 16089 },
	{ 0xed4d, 0xed4e, PDF_CMAP_RANGE, 18699 },
	{ 0xed4f, 0xed50, PDF_CMAP_RANGE, 16090 },
	{ 0xed51, 0xed52, PDF_CMAP_TABLE, 15071 },
	{ 0xed53, 0xed54, PDF_CMAP_RANGE, 18702 },
	{ 0xed55, 0xed56, PDF_CMAP_RANGE, 16093 },
	{ 0xed57, 0xed58, PDF_CMAP_RANGE, 18704 },
	{ 0xed59, 0xed6d, PDF_CMAP_RANGE, 16095 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 9398 },
	{ 0xed6f, 0xed70, PDF_CMAP_RANGE, 16117 },
	{ 0xed71, 0xed71, PDF_CMAP_SINGLE, 18706 },
	{ 0xed72, 0xed74, PDF_CMAP_RANGE, 16119 },
	{ 0xed75, 0xed77, PDF_CMAP_RANGE, 18707 },
	{ 0xed78, 0xed79, PDF_CMAP_RANGE, 16122 },
	{ 0xed7a, 0xed7c, PDF_CMAP_TABLE, 15073 },
	{ 0xed7d, 0xed80, PDF_CMAP_RANGE, 16126 },
	{ 0xed81, 0xed81, PDF_CMAP_SINGLE, 18711 },
	{ 0xed82, 0xed85, PDF_CMAP_RANGE, 16130 },
	{ 0xed86, 0xed86, PDF_CMAP_SINGLE, 18712 },
	{ 0xed87, 0xed88, PDF_CMAP_RANGE, 16134 },
	{ 0xed89, 0xed8a, PDF_CMAP_RANGE, 18713 },
	{ 0xed8b, 0xed9d, PDF_CMAP_RANGE, 16136 },
	{ 0xed9e, 0xed9e, PDF_CMAP_SINGLE, 13585 },
	{ 0xed9f, 0xeda3, PDF_CMAP_RANGE, 16156 },
	{ 0xeda4, 0xeda4, PDF_CMAP_SINGLE, 14661 },
	{ 0xeda5, 0xeda6, PDF_CMAP_RANGE, 18715 },
	{ 0xeda7, 0xedad, PDF_CMAP_RANGE, 16163 },
	{ 0xedae, 0xedae, PDF_CMAP_SINGLE, 18717 },
	{ 0xedaf, 0xedd0, PDF_CMAP_RANGE, 16170 },
	{ 0xedd1, 0xedd1, PDF_CMAP_SINGLE, 18718 },
	{ 0xedd2, 0xedd6, PDF_CMAP_RANGE, 16204 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 18719 },
	{ 0xedd8, 0xedda, PDF_CMAP_RANGE, 16209 },
	{ 0xeddb, 0xeddd, PDF_CMAP_TABLE, 15076 },
	{ 0xedde, 0xede6, PDF_CMAP_RANGE, 16213 },
	{ 0xede7, 0xede8, PDF_CMAP_TABLE, 15079 },
	{ 0xede9, 0xedef, PDF_CMAP_RANGE, 16224 },
	{ 0xedf0, 0xedf0, PDF_CMAP_SINGLE, 18723 },
	{ 0xedf1, 0xedf3, PDF_CMAP_RANGE, 16232 },
	{ 0xedf4, 0xedf4, PDF_CMAP_SINGLE, 14910 },
	{ 0xedf5, 0xedff, PDF_CMAP_RANGE, 16236 },
	{ 0xee00, 0xee00, PDF_CMAP_SINGLE, 18724 },
	{ 0xee01, 0xee14, PDF_CMAP_RANGE, 16247 },
	{ 0xee15, 0xee15, PDF_CMAP_SINGLE, 18725 },
	{ 0xee16, 0xee1a, PDF_CMAP_RANGE, 16267 },
	{ 0xee1b, 0xee1b, PDF_CMAP_SINGLE, 18726 },
	{ 0xee1c, 0xee21, PDF_CMAP_RANGE, 16272 },
	{ 0xee22, 0xee22, PDF_CMAP_SINGLE, 18727 },
	{ 0xee23, 0xee2f, PDF_CMAP_RANGE, 16278 },
	{ 0xee30, 0xee30, PDF_CMAP_SINGLE, 18728 },
	{ 0xee31, 0xee3d, PDF_CMAP_RANGE, 16291 },
	{ 0xee3e, 0xee3e, PDF_CMAP_SINGLE, 14353 },
	{ 0xee3f, 0xee47, PDF_CMAP_RANGE, 16305 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 18729 },
	{ 0xee49, 0xee51, PDF_CMAP_RANGE, 16314 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 1522 },
	{ 0xee53, 0xee55, PDF_CMAP_RANGE, 16324 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 18730 },
	{ 0xee57, 0xee5a, PDF_CMAP_RANGE, 16327 },
	{ 0xee5b, 0xee5b, PDF_CMAP_SINGLE, 18731 },
	{ 0xee5c, 0xee5f, PDF_CMAP_RANGE, 16331 },
	{ 0xee60, 0xee60, PDF_CMAP_SINGLE, 18732 },
	{ 0xee61, 0xee7d, PDF_CMAP_RANGE, 16335 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 18733 },
	{ 0xee7f, 0xee88, PDF_CMAP_RANGE, 16364 },
	{ 0xee89, 0xee89, PDF_CMAP_SINGLE, 18734 },
	{ 0xee8a, 0xee8d, PDF_CMAP_RANGE, 16374 },
	{ 0xee8e, 0xee8e, PDF_CMAP_SINGLE, 15086 },
	{ 0xee8f, 0xee97, PDF_CMAP_RANGE, 16379 },
	{ 0xee98, 0xee98, PDF_CMAP_SINGLE, 9341 },
	{ 0xee99, 0xee9b, PDF_CMAP_RANGE, 16389 },
	{ 0xee9c, 0xee9d, PDF_CMAP_TABLE, 15081 },
	{ 0xee9e, 0xee9f, PDF_CMAP_RANGE, 16393 },
	{ 0xeea0, 0xeea6, PDF_CMAP_RANGE, 18736 },
	{ 0xeea7, 0xeea7, PDF_CMAP_SINGLE, 17332 },
	{ 0xeea8, 0xeeaa, PDF_CMAP_RANGE, 18743 },
	{ 0xeeab, 0xeeab, PDF_CMAP_SINGLE, 17336 },
	{ 0xeeac, 0xeeb7, PDF_CMAP_RANGE, 18746 },
	{ 0xf303, 0xf318, PDF_CMAP_RANGE, 17609 },
	{ 0xf319, 0xf349, PDF_CMAP_RANGE, 18785 },
	{ 0xf34a, 0xf34b, PDF_CMAP_RANGE, 18844 },
	{ 0xf3a0, 0xf3a1, PDF_CMAP_RANGE, 17716 },
	{ 0xf3a3, 0xf3a3, PDF_CMAP_SINGLE, 17718 },
	{ 0xf3a6, 0xf3a9, PDF_CMAP_RANGE, 17719 },
	{ 0xf3ac, 0xf3ad, PDF_CMAP_TABLE, 15083 },
	{ 0xf3ae, 0xf3b0, PDF_CMAP_RANGE, 17724 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 17306 },
	{ 0xf3b2, 0xf3e3, PDF_CMAP_RANGE, 17727 },
	{ 0xf3e4, 0xf3e4, PDF_CMAP_SINGLE, 17314 },
	{ 0xf3e9, 0xf3ec, PDF_CMAP_TABLE, 15085 },
	{ 0xf3ee, 0xf3f0, PDF_CMAP_RANGE, 17779 },
	{ 0xf3f3, 0xf3fd, PDF_CMAP_RANGE, 17782 },
	{ 0xf3ff, 0xf401, PDF_CMAP_RANGE, 17793 },
	{ 0xf403, 0xf40c, PDF_CMAP_RANGE, 17796 },
	{ 0xf40d, 0xf40d, PDF_CMAP_SINGLE, 17340 },
	{ 0xf40e, 0xf416, PDF_CMAP_RANGE, 17806 },
	{ 0xf417, 0xf41b, PDF_CMAP_TABLE, 15089 },
	{ 0xf41c, 0xf41e, PDF_CMAP_RANGE, 17817 },
	{ 0xf41f, 0xf421, PDF_CMAP_TABLE, 15094 },
	{ 0xf422, 0xf427, PDF_CMAP_RANGE, 17821 },
	{ 0xf428, 0xf429, PDF_CMAP_RANGE, 17317 },
	{ 0xf42a, 0xf437, PDF_CMAP_RANGE, 17827 },
	{ 0xf438, 0xf438, PDF_CMAP_SINGLE, 17321 },
	{ 0xf439, 0xf43d, PDF_CMAP_RANGE, 17841 },
	{ 0xf43e, 0xf43e, PDF_CMAP_SINGLE, 17161 },
	{ 0xf440, 0xf449, PDF_CMAP_RANGE, 17163 },
	{ 0xf44a, 0xf44a, PDF_CMAP_SINGLE, 17846 },
	{ 0xf44b, 0xf456, PDF_CMAP_RANGE, 17174 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 17847 },
	{ 0xf458, 0xf45a, PDF_CMAP_RANGE, 17187 },
	{ 0xf45b, 0xf45b, PDF_CMAP_SINGLE, 17848 },
	{ 0xf45c, 0xf45f, PDF_CMAP_RANGE, 17191 },
	{ 0xf461, 0xf46d, PDF_CMAP_RANGE, 17196 },
	{ 0xf46e, 0xf46e, PDF_CMAP_SINGLE, 17849 },
	{ 0xf46f, 0xf471, PDF_CMAP_RANGE, 17210 },
	{ 0xf473, 0xf474, PDF_CMAP_TABLE, 15097 },
	{ 0xf475, 0xf476, PDF_CMAP_RANGE, 17216 },
	{ 0xf477, 0xf47b, PDF_CMAP_TABLE, 15099 },
	{ 0xf47c, 0xf482, PDF_CMAP_RANGE, 17223 },
	{ 0xf483, 0xf483, PDF_CMAP_SINGLE, 17854 },
	{ 0xf484, 0xf485, PDF_CMAP_RANGE, 17231 },
	{ 0xf487, 0xf48b, PDF_CMAP_RANGE, 17234 },
	{ 0xf48d, 0xf490, PDF_CMAP_RANGE, 17240 },
	{ 0xf491, 0xf494, PDF_CMAP_TABLE, 15104 },
	{ 0xf496, 0xf4a2, PDF_CMAP_RANGE, 17249 },
	{ 0xf4a4, 0xf4a6, PDF_CMAP_RANGE, 17263 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 17857 },
	{ 0xf4a9, 0xf4b0, PDF_CMAP_RANGE, 17268 },
	{ 0xf4b1, 0xf4b2, PDF_CMAP_RANGE, 17858 },
	{ 0xf4b3, 0xf4b7, PDF_CMAP_RANGE, 17278 },
	{ 0xf4ba, 0xf4c0, PDF_CMAP_RANGE, 17285 },
	{ 0xf4c1, 0xf4c2, PDF_CMAP_TABLE, 15108 },
	{ 0xf4c3, 0xf4cf, PDF_CMAP_RANGE, 17861 },
	{ 0xf4d1, 0xf4da, PDF_CMAP_RANGE, 17874 },
	{ 0xf4db, 0xf4dc, PDF_CMAP_RANGE, 17292 },
	{ 0xf4dd, 0xf4de, PDF_CMAP_RANGE, 17884 },
	{ 0xf4df, 0xf4e6, PDF_CMAP_TABLE, 15110 },
	{ 0xf4e7, 0xf4ea, PDF_CMAP_RANGE, 17298 },
	{ 0xf4eb, 0xf4ed, PDF_CMAP_RANGE, 17890 },
	{ 0xf4ef, 0xf4f1, PDF_CMAP_RANGE, 17893 },
	{ 0xf4f2, 0xf4f5, PDF_CMAP_TABLE, 15118 },
	{ 0xf4f6, 0xf4fa, PDF_CMAP_RANGE, 17897 },
	{ 0xf4fb, 0xf4fb, PDF_CMAP_SINGLE, 17319 },
	{ 0xf4fc, 0xf501, PDF_CMAP_RANGE, 17902 },
	{ 0xf502, 0xf502, PDF_CMAP_SINGLE, 17324 },
	{ 0xf503, 0xf537, PDF_CMAP_RANGE, 17908 },
	{ 0xf538, 0xf554, PDF_CMAP_RANGE, 17631 },
	{ 0xf556, 0xf575, PDF_CMAP_RANGE, 17660 },
	{ 0xf579, 0xf57b, PDF_CMAP_RANGE, 18851 },
	{ 0xf57d, 0xf57d, PDF_CMAP_SINGLE, 18855 },
	{ 0xf580, 0xf583, PDF_CMAP_RANGE, 18858 },
	{ 0xf585, 0xf585, PDF_CMAP_SINGLE, 18863 },
	{ 0xf58b, 0xf58f, PDF_CMAP_RANGE, 18869 },
	{ 0xf593, 0xf594, PDF_CMAP_RANGE, 18877 },
	{ 0xf596, 0xf597, PDF_CMAP_RANGE, 18880 },
	{ 0xf59b, 0xf59c, PDF_CMAP_RANGE, 18884 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 18889 },
	{ 0xf5a3, 0xf5a4, PDF_CMAP_RANGE, 18892 },
	{ 0xf5ab, 0xf5ab, PDF_CMAP_SINGLE, 18900 },
	{ 0xf5b0, 0xf5b0, PDF_CMAP_SINGLE, 18905 },
	{ 0xf5b4, 0xf5b4, PDF_CMAP_SINGLE, 18909 },
	{ 0xf5b6, 0xf5b6, PDF_CMAP_SINGLE, 18911 },
	{ 0xf5b9, 0xf5b9, PDF_CMAP_SINGLE, 18914 },
	{ 0xf5bd, 0xf5bd, PDF_CMAP_SINGLE, 18917 },
	{ 0xf5bf, 0xf5c0, PDF_CMAP_RANGE, 18919 },
	{ 0xf5c4, 0xf5c5, PDF_CMAP_RANGE, 18924 },
	{ 0xf5cc, 0xf5cc, PDF_CMAP_SINGLE, 18932 },
	{ 0xf5ce, 0xf5ce, PDF_CMAP_SINGLE, 18934 },
	{ 0xf5d0, 0xf5d0, PDF_CMAP_SINGLE, 18936 },
	{ 0xf5d5, 0xf5d9, PDF_CMAP_RANGE, 18941 },
	{ 0xf5e0, 0xf5e0, PDF_CMAP_SINGLE, 18949 },
	{ 0xf5e4, 0xf5e4, PDF_CMAP_SINGLE, 18952 },
	{ 0xf5e6, 0xf5e8, PDF_CMAP_RANGE, 18954 },
	{ 0xf5ea, 0xf5ed, PDF_CMAP_RANGE, 18958 },
	{ 0xf5ef, 0xf5ef, PDF_CMAP_SINGLE, 18963 },
	{ 0xf634, 0xf635, PDF_CMAP_RANGE, 17961 },
	{ 0xf636, 0xf636, PDF_CMAP_SINGLE, 17338 },
	{ 0xf637, 0xf63b, PDF_CMAP_RANGE, 17963 },
	{ 0xf63c, 0xf63e, PDF_CMAP_TABLE, 15122 },
	{ 0xf63f, 0xf641, PDF_CMAP_RANGE, 17968 },
	{ 0xf642, 0xf642, PDF_CMAP_SINGLE, 17342 },
	{ 0xf643, 0xf649, PDF_CMAP_RANGE, 17971 },
	{ 0xf64a, 0xf64a, PDF_CMAP_SINGLE, 17343 },
	{ 0xf64b, 0xf64d, PDF_CMAP_RANGE, 17978 },
	{ 0xf64e, 0xf650, PDF_CMAP_TABLE, 15125 },
	{ 0xf651, 0xf656, PDF_CMAP_RANGE, 17982 },
	{ 0xf657, 0xf657, PDF_CMAP_SINGLE, 17346 },
	{ 0xf658, 0xf659, PDF_CMAP_RANGE, 17988 },
	{ 0xf65a, 0xf65b, PDF_CMAP_TABLE, 15128 },
	{ 0xf65c, 0xf667, PDF_CMAP_RANGE, 17990 },
	{ 0xf668, 0xf668, PDF_CMAP_SINGLE, 17351 },
	{ 0xf669, 0xf674, PDF_CMAP_RANGE, 18002 },
	{ 0xf675, 0xf675, PDF_CMAP_SINGLE, 17352 },
	{ 0xf676, 0xf6ab, PDF_CMAP_RANGE, 18014 },
	{ 0xf6ac, 0xf6ac, PDF_CMAP_SINGLE, 17401 },
	{ 0xf6ad, 0xf6b0, PDF_CMAP_RANGE, 18068 },
	{ 0xf6b1, 0xf6ce, PDF_CMAP_RANGE, 506 },
	{ 0xf6cf, 0xf6de, PDF_CMAP_RANGE, 537 },
	{ 0xf6e0, 0xf6e2, PDF_CMAP_RANGE, 554 },
	{ 0xf6e4, 0xf6e4, PDF_CMAP_SINGLE, 558 },
	{ 0xf6e6, 0xf6e6, PDF_CMAP_SINGLE, 560 },
	{ 0xf6e8, 0xf6ed, PDF_CMAP_RANGE, 13747 },
	{ 0xf6f0, 0xf7ee, PDF_CMAP_RANGE, 13754 },
	{ 0xf817, 0xf81d, PDF_CMAP_RANGE, 14049 },
	{ 0xf81e, 0xf820, PDF_CMAP_RANGE, 17606 },
	{ 0xf821, 0xf829, PDF_CMAP_RANGE, 17692 },
	{ 0xf82a, 0xf82a, PDF_CMAP_SINGLE, 18846 },
	{ 0xf82b, 0xf832, PDF_CMAP_RANGE, 17701 },
	{ 0xf833, 0xf833, PDF_CMAP_SINGLE, 18847 },
	{ 0xf834, 0xf83a, PDF_CMAP_RANGE, 17709 },
	{ 0xf83b, 0xf83b, PDF_CMAP_SINGLE, 18848 },
	{ 0xf83f, 0xf848, PDF_CMAP_RANGE, 18834 },
	{ 0xf907, 0xf907, PDF_CMAP_SINGLE, 17686 },
	{ 0xfa0c, 0xfa0d, PDF_CMAP_TABLE, 15130 },
	{ 0xfa12, 0xfa12, PDF_CMAP_SINGLE, 16828 },
	{ 0xfa26, 0xfa26, PDF_CMAP_SINGLE, 14465 },
	{ 0xfa48, 0xfa48, PDF_CMAP_SINGLE, 16997 },
	{ 0xfa5b, 0xfa5b, PDF_CMAP_SINGLE, 14224 },
	{ 0xfe10, 0xfe12, PDF_CMAP_RANGE, 14099 },
	{ 0xfe13, 0xfe16, PDF_CMAP_TABLE, 15132 },
	{ 0xfe19, 0xfe19, PDF_CMAP_SINGLE, 13646 },
	{ 0xfe30, 0xfe31, PDF_CMAP_TABLE, 15136 },
	{ 0xfe33, 0xfe34, PDF_CMAP_TABLE, 15138 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 130 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 134 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 138 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 142 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 146 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 150 },
	{ 0xfe41, 0xfe42, PDF_CMAP_RANGE, 154 },
	{ 0xfe43, 0xfe44, PDF_CMAP_RANGE, 158 },
	{ 0xfe49, 0xfe4a, PDF_CMAP_RANGE, 199 },
	{ 0xfe4b, 0xfe4c, PDF_CMAP_RANGE, 203 },
	{ 0xfe4d, 0xfe4e, PDF_CMAP_RANGE, 201 },
	{ 0xfe4f, 0xfe4f, PDF_CMAP_SINGLE, 13746 },
	{ 0xfe50, 0xfe52, PDF_CMAP_RANGE, 112 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 116 },
	{ 0xfe59, 0xfe5e, PDF_CMAP_RANGE, 160 },
	{ 0xfe5f, 0xfe61, PDF_CMAP_RANGE, 205 },
	{ 0xfe62, 0xfe66, PDF_CMAP_RANGE, 223 },
	{ 0xfe68, 0xfe68, PDF_CMAP_SINGLE, 258 },
	{ 0xfe69, 0xfe6b, PDF_CMAP_RANGE, 268 },
	{ 0xff01, 0xff07, PDF_CMAP_TABLE, 15140 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 128 },
	{ 0xff0a, 0xff0f, PDF_CMAP_TABLE, 15147 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 333 },
	{ 0xff1a, 0xff20, PDF_CMAP_TABLE, 15153 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 365 },
	{ 0xff3b, 0xff3f, PDF_CMAP_TABLE, 15160 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 391 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 15165 },
	{ 0xff64, 0xff64, PDF_CMAP_SINGLE, 113 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 262 },
	{ 0xffe2, 0xffe5, PDF_CMAP_TABLE, 15169 },
	{ 0xffed, 0xffed, PDF_CMAP_SINGLE, 14096 },
	{ 0x6b03, 0x6b03, PDF_CMAP_SINGLE, 13013 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 5751 },
	{ 0x6b05, 0x6b05, PDF_CMAP_SINGLE, 15015 },
	{ 0x6b08, 0x6b09, PDF_CMAP_RANGE, 13190 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 5830 },
	{ 0x6b0b, 0x6b0b, PDF_CMAP_SINGLE, 13189 },
	{ 0x6b0f, 0x6b0f, PDF_CMAP_SINGLE, 13344 },
	{ 0x6b10, 0x6b10, PDF_CMAP_SINGLE, 5885 },
	{ 0x6b11, 0x6b12, PDF_CMAP_RANGE, 13342 },
	{ 0x6b13, 0x6b13, PDF_CMAP_SINGLE, 13447 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 5955 },
	{ 0x6b17, 0x6b17, PDF_CMAP_SINGLE, 13518 },
	{ 0x6b18, 0x6b19, PDF_CMAP_RANGE, 13516 },
	{ 0x6b1a, 0x6b1a, PDF_CMAP_SINGLE, 13519 },
	{ 0x6b1d, 0x6b1d, PDF_CMAP_SINGLE, 16888 },
	{ 0x6b1e, 0x6b1e, PDF_CMAP_SINGLE, 13620 },
	{ 0x6b20, 0x6b20, PDF_CMAP_SINGLE, 734 },
	{ 0x6b21, 0x6b21, PDF_CMAP_SINGLE, 995 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 1562 },
	{ 0x6b25, 0x6b25, PDF_CMAP_SINGLE, 6601 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 6973 },
	{ 0x6b2c, 0x6b2c, PDF_CMAP_SINGLE, 7398 },
	{ 0x6b2d, 0x6b2d, PDF_CMAP_SINGLE, 7400 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 7399 },
	{ 0x6b31, 0x6b31, PDF_CMAP_SINGLE, 7401 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 2791 },
	{ 0x6b33, 0x6b33, PDF_CMAP_SINGLE, 7996 },
	{ 0x6b34, 0x6b34, PDF_CMAP_SINGLE, 7402 },
	{ 0x6b35, 0x6b35, PDF_CMAP_SINGLE, 16891 },
	{ 0x6b36, 0x6b36, PDF_CMAP_SINGLE, 7995 },
	{ 0x6b37, 0x6b38, PDF_CMAP_RANGE, 7997 },
	{ 0x6b39, 0x6b39, PDF_CMAP_SINGLE, 8626 },
	{ 0x6b3a, 0x6b3a, PDF_CMAP_SINGLE, 3228 },
	{ 0x6b3b, 0x6b3b, PDF_CMAP_SINGLE, 8627 },
	{ 0x6b3c, 0x6b3c, PDF_CMAP_SINGLE, 8629 },
	{ 0x6b3d, 0x6b3d, PDF_CMAP_SINGLE, 3229 },
	{ 0x6b3e, 0x6b3e, PDF_CMAP_SINGLE, 3227 },
	{ 0x6b3f, 0x6b3f, PDF_CMAP_SINGLE, 8628 },
	{ 0x6b41, 0x6b41, PDF_CMAP_SINGLE, 9287 },
	{ 0x6b42, 0x6b42, PDF_CMAP_SINGLE, 9285 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 9284 },
	{ 0x6b45, 0x6b45, PDF_CMAP_SINGLE, 9283 },
	{ 0x6b46, 0x6b46, PDF_CMAP_SINGLE, 9282 },
	{ 0x6b47, 0x6b47, PDF_CMAP_SINGLE, 3658 },
	{ 0x6b48, 0x6b48, PDF_CMAP_SINGLE, 9286 },
	{ 0x6b49, 0x6b49, PDF_CMAP_SINGLE, 4097 },
	{ 0x6b4a, 0x6b4a, PDF_CMAP_SINGLE, 9954 },
	{ 0x6b4b, 0x6b4b, PDF_CMAP_SINGLE, 9956 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 4098 },
	{ 0x6b4d, 0x6b4d, PDF_CMAP_SINGLE, 9955 },
	{ 0x6b4e, 0x6b4e, PDF_CMAP_SINGLE, 4485 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 4484 },
	{ 0x6b51, 0x6b51, PDF_CMAP_SINGLE, 10532 },
	{ 0x6b52, 0x6b52, PDF_CMAP_SINGLE, 16238 },
	{ 0x6b54, 0x6b54, PDF_CMAP_SINGLE, 11145 },
	{ 0x6b55, 0x6b55, PDF_CMAP_SINGLE, 11144 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 11146 },
	{ 0x6b57, 0x6b57, PDF_CMAP_SINGLE, 15022 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 4828 },
	{ 0x6b5b, 0x6b5b, PDF_CMAP_SINGLE, 11658 },
	{ 0x6b5c, 0x6b5c, PDF_CMAP_SINGLE, 5104 },
	{ 0x6b5e, 0x6b5e, PDF_CMAP_SINGLE, 12095 },
	{ 0x6b5f, 0x6b5f, PDF_CMAP_SINGLE, 5348 },
	{ 0x6b60, 0x6b60, PDF_CMAP_SINGLE, 12455 },
	{ 0x6b61, 0x6b61, PDF_CMAP_SINGLE, 5831 },
	{ 0x6b62, 0x6b62, PDF_CMAP_SINGLE, 735 },
	{ 0x6b63, 0x6b63, PDF_CMAP_SINGLE, 845 },
	{ 0x6b64, 0x6b64, PDF_CMAP_SINGLE, 996 },
	{ 0x6b65, 0x6b65, PDF_CMAP_SINGLE, 1225 },
	{ 0x6b66, 0x6b67, PDF_CMAP_RANGE, 1563 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 1897 },
	{ 0x6b6d, 0x6b6d, PDF_CMAP_SINGLE, 7403 },
	{ 0x6b6f, 0x6b6f, PDF_CMAP_SINGLE, 17685 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 3659 },
	{ 0x6b74, 0x6b74, PDF_CMAP_SINGLE, 16892 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 10658 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 4829 },
	{ 0x6b78, 0x6b78, PDF_CMAP_SINGLE, 5349 },
	{ 0x6b79, 0x6b79, PDF_CMAP_SINGLE, 736 },
	{ 0x6b7a, 0x6b7a, PDF_CMAP_SINGLE, 17646 },
	{ 0x6b7b, 0x6b7b, PDF_CMAP_SINGLE, 997 },
	{ 0x6b7e, 0x6b7e, PDF_CMAP_SINGLE, 6603 },
	{ 0x6b7f, 0x6b7f, PDF_CMAP_SINGLE, 1565 },
	{ 0x6b80, 0x6b80, PDF_CMAP_SINGLE, 6602 },
	{ 0x6b81, 0x6b81, PDF_CMAP_SINGLE, 18027 },
	{ 0x6b82, 0x6b82, PDF_CMAP_SINGLE, 6974 },
	{ 0x6b83, 0x6b83, PDF_CMAP_SINGLE, 1898 },
	{ 0x6b84, 0x6b84, PDF_CMAP_SINGLE, 6975 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 1899 },
	{ 0x6b88, 0x6b88, PDF_CMAP_SINGLE, 7405 },
	{ 0x6b89, 0x6b89, PDF_CMAP_SINGLE, 2304 },
	{ 0x6b8a, 0x6b8a, PDF_CMAP_SINGLE, 2303 },
	{ 0x6b8c, 0x6b8c, PDF_CMAP_SINGLE, 8003 },
	{ 0x6b8d, 0x6b8e, PDF_CMAP_RANGE, 8001 },
	{ 0x6b8f, 0x6b8f, PDF_CMAP_SINGLE, 8000 },
	{ 0x6b91, 0x6b91, PDF_CMAP_SINGLE, 7999 },
	{ 0x6b94, 0x6b94, PDF_CMAP_SINGLE, 8630 },
	{ 0x6b95, 0x6b95, PDF_CMAP_SINGLE, 8633 },
	{ 0x6b96, 0x6b96, PDF_CMAP_SINGLE, 3231 },
	{ 0x6b97, 0x6b97, PDF_CMAP_SINGLE, 8631 },
	{ 0x6b98, 0x6b98, PDF_CMAP_SINGLE, 3230 },
	{ 0x6b99, 0x6b99, PDF_CMAP_SINGLE, 8632 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 9288 },
	{ 0x6b9e, 0x6ba0, PDF_CMAP_RANGE, 9957 },
	{ 0x6ba2, 0x6ba2, PDF_CMAP_SINGLE, 10535 },
	{ 0x6ba3, 0x6ba3, PDF_CMAP_SINGLE, 10534 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 4486 },
	{ 0x6ba5, 0x6ba5, PDF_CMAP_SINGLE, 10533 },
	{ 0x6ba6, 0x6ba6, PDF_CMAP_SINGLE, 10536 },
	{ 0x6ba7, 0x6ba7, PDF_CMAP_SINGLE, 11147 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 11148 },
	{ 0x6bad, 0x6bad, PDF_CMAP_SINGLE, 11659 },
	{ 0x6bae, 0x6bae, PDF_CMAP_SINGLE, 5105 },
	{ 0x6baf, 0x6baf, PDF_CMAP_SINGLE, 5350 },
	{ 0x6bb0, 0x6bb0, PDF_CMAP_SINGLE, 12456 },
	{ 0x6bb2, 0x6bb2, PDF_CMAP_SINGLE, 5753 },
	{ 0x6bb3, 0x6bb3, PDF_CMAP_SINGLE, 6026 },
	{ 0x6bb5, 0x6bb5, PDF_CMAP_SINGLE, 1900 },
	{ 0x6bb6, 0x6bb6, PDF_CMAP_SINGLE, 6976 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 2305 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 2792 },
	{ 0x6bbc, 0x6bbc, PDF_CMAP_SINGLE, 3232 },
	{ 0x6bbd, 0x6bbd, PDF_CMAP_SINGLE, 8634 },
	{ 0x6bbf, 0x6bbf, PDF_CMAP_SINGLE, 3661 },
	{ 0x6bc0, 0x6bc0, PDF_CMAP_SINGLE, 3660 },
	{ 0x6bc1, 0x6bc1, PDF_CMAP_SINGLE, 18028 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 9960 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_RANGE, 4487 },
	{ 0x6bc7, 0x6bc7, PDF_CMAP_SINGLE, 11151 },
	{ 0x6bc8, 0x6bc8, PDF_CMAP_SINGLE, 11150 },
	{ 0x6bc9, 0x6bc9, PDF_CMAP_SINGLE, 12096 },
	{ 0x6bca, 0x6bca, PDF_CMAP_SINGLE, 13345 },
	{ 0x6bcb, 0x6bcb, PDF_CMAP_SINGLE, 737 },
	{ 0x6bcc, 0x6bcc, PDF_CMAP_SINGLE, 6027 },
	{ 0x6bcd, 0x6bcd, PDF_CMAP_SINGLE, 846 },
	{ 0x6bcf, 0x6bcf, PDF_CMAP_SINGLE, 1226 },
	{ 0x6bd0, 0x6bd0, PDF_CMAP_SINGLE, 6317 },
	{ 0x6bd2, 0x6bd2, PDF_CMAP_SINGLE, 1901 },
	{ 0x6bd3, 0x6bd3, PDF_CMAP_SINGLE, 3662 },
	{ 0x6bd4, 0x6bd4, PDF_CMAP_SINGLE, 738 },
	{ 0x6bd6, 0x6bd6, PDF_CMAP_SINGLE, 6977 },
	{ 0x6bd7, 0x6bd7, PDF_CMAP_SINGLE, 1902 },
	{ 0x6bd8, 0x6bd8, PDF_CMAP_SINGLE, 6978 },
	{ 0x6bda, 0x6bda, PDF_CMAP_SINGLE, 5106 },
	{ 0x6bdb, 0x6bdb, PDF_CMAP_SINGLE, 739 },
	{ 0x6bdc, 0x6bdc, PDF_CMAP_SINGLE, 16903 },
	{ 0x6bde, 0x6bde, PDF_CMAP_SINGLE, 6604 },
	{ 0x6be0, 0x6be0, PDF_CMAP_SINGLE, 6979 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 16901 },
	{ 0x6be2, 0x6be2, PDF_CMAP_SINGLE, 7410 },
	{ 0x6be3, 0x6be3, PDF_CMAP_SINGLE, 7409 },
	{ 0x6be4, 0x6be4, PDF_CMAP_SINGLE, 7407 },
	{ 0x6be6, 0x6be6, PDF_CMAP_SINGLE, 7406 },
	{ 0x6be7, 0x6be7, PDF_CMAP_SINGLE, 7411 },
	{ 0x6be8, 0x6be8, PDF_CMAP_SINGLE, 7408 },
	{ 0x6bea, 0x6bea, PDF_CMAP_SINGLE, 18029 },
	{ 0x6beb, 0x6bec, PDF_CMAP_RANGE, 2793 },
	{ 0x6bef, 0x6bef, PDF_CMAP_SINGLE, 3233 },
	{ 0x6bf0, 0x6bf0, PDF_CMAP_SINGLE, 8635 },
	{ 0x6bf2, 0x6bf3, PDF_CMAP_RANGE, 8636 },
	{ 0x6bf7, 0x6bf8, PDF_CMAP_RANGE, 9292 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 9291 },
	{ 0x6bfa, 0x6bfa, PDF_CMAP_SINGLE, 16249 },
	{ 0x6bfb, 0x6bfc, PDF_CMAP_RANGE, 9289 },
	{ 0x6bfd, 0x6bfd, PDF_CMAP_SINGLE, 3663 },
	{ 0x6bfe, 0x6bfe, PDF_CMAP_SINGLE, 9962 },
	{ 0x6bff, 0x6bff, PDF_CMAP_SINGLE, 10539 },
	{ 0x6c00, 0x6c00, PDF_CMAP_SINGLE, 10538 },
	{ 0x6c01, 0x6c01, PDF_CMAP_SINGLE, 10537 },
	{ 0x6c02, 0x6c02, PDF_CMAP_SINGLE, 10540 },
	{ 0x6c03, 0x6c03, PDF_CMAP_SINGLE, 11153 },
	{ 0x6c04, 0x6c04, PDF_CMAP_SINGLE, 11152 },
	{ 0x6c05, 0x6c05, PDF_CMAP_SINGLE, 4830 },
	{ 0x6c06, 0x6c06, PDF_CMAP_SINGLE, 11154 },
	{ 0x6c08, 0x6c08, PDF_CMAP_SINGLE, 5107 },
	{ 0x6c09, 0x6c09, PDF_CMAP_SINGLE, 11660 },
	{ 0x6c0b, 0x6c0b, PDF_CMAP_SINGLE, 12097 },
	{ 0x6c0c, 0x6c0c, PDF_CMAP_SINGLE, 12457 },
	{ 0x6c0d, 0x6c0d, PDF_CMAP_SINGLE, 13192 },
	{ 0x6c0f, 0x6c0f, PDF_CMAP_SINGLE, 740 },
	{ 0x6c10, 0x6c10, PDF_CMAP_SINGLE, 848 },
	{ 0x6c11, 0x6c11, PDF_CMAP_SINGLE, 847 },
	{ 0x6c13, 0x6c13, PDF_CMAP_SINGLE, 1566 },
	{ 0x6c14, 0x6c14, PDF_CMAP_SINGLE, 6028 },
	{ 0x6c15, 0x6c15, PDF_CMAP_SINGLE, 6058 },
	{ 0x6c16, 0x6c16, PDF_CMAP_SINGLE, 998 },
	{ 0x6c18, 0x6c18, PDF_CMAP_SINGLE, 6145 },
	{ 0x6c19, 0x6c1a, PDF_CMAP_RANGE, 6318 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 1567 },
	{ 0x6c1c, 0x6c1c, PDF_CMAP_SINGLE, 14507 },
	{ 0x6c1d, 0x6c1d, PDF_CMAP_SINGLE, 6605 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 1903 },
	{ 0x6c20, 0x6c21, PDF_CMAP_RANGE, 6980 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 2306 },
	{ 0x6c24, 0x6c24, PDF_CMAP_SINGLE, 2310 },
	{ 0x6c25, 0x6c25, PDF_CMAP_SINGLE, 7412 },
	{ 0x6c26, 0x6c26, PDF_CMAP_SINGLE, 2309 },
	{ 0x6c27, 0x6c28, PDF_CMAP_RANGE, 2307 },
	{ 0x6c2a, 0x6c2a, PDF_CMAP_SINGLE, 8004 },
	{ 0x6c2b, 0x6c2b, PDF_CMAP_SINGLE, 2795 },
	{ 0x6c2c, 0x6c2c, PDF_CMAP_SINGLE, 3236 },
	{ 0x6c2e, 0x6c2f, PDF_CMAP_RANGE, 3234 },
	{ 0x6c30, 0x6c30, PDF_CMAP_SINGLE, 8638 },
	{ 0x6c31, 0x6c31, PDF_CMAP_SINGLE, 16227 },
	{ 0x6c33, 0x6c33, PDF_CMAP_SINGLE, 4099 },
	{ 0x6c34, 0x6c34, PDF_CMAP_SINGLE, 741 },
	{ 0x6c35, 0x6c35, PDF_CMAP_SINGLE, 17647 },
	{ 0x6c36, 0x6c36, PDF_CMAP_SINGLE, 6059 },
	{ 0x6c37, 0x6c37, PDF_CMAP_SINGLE, 16904 },
	{ 0x6c38, 0x6c38, PDF_CMAP_SINGLE, 849 },
	{ 0x6c39, 0x6c39, PDF_CMAP_SINGLE, 15877 },
	{ 0x6c3a, 0x6c3a, PDF_CMAP_SINGLE, 17648 },
	{ 0x6c3b, 0x6c3b, PDF_CMAP_SINGLE, 6062 },
	{ 0x6c3d, 0x6c3d, PDF_CMAP_SINGLE, 17281 },
	{ 0x6c3e, 0x6c3e, PDF_CMAP_SINGLE, 852 },
	{ 0x6c3f, 0x6c3f, PDF_CMAP_SINGLE, 6061 },
	{ 0x6c40, 0x6c40, PDF_CMAP_SINGLE, 851 },
	{ 0x6c41, 0x6c41, PDF_CMAP_SINGLE, 850 },
	{ 0x6c42, 0x6c42, PDF_CMAP_SINGLE, 1227 },
	{ 0x6c43, 0x6c43, PDF_CMAP_SINGLE, 6060 },
	{ 0x6c46, 0x6c46, PDF_CMAP_SINGLE, 6146 },
	{ 0x6c49, 0x6c49, PDF_CMAP_SINGLE, 16525 },
	{ 0x6c4a, 0x6c4a, PDF_CMAP_SINGLE, 6150 },
	{ 0x6c4b, 0x6c4c, PDF_CMAP_RANGE, 6152 },
	{ 0x6c4d, 0x6c4e, PDF_CMAP_RANGE, 1008 },
	{ 0x6c4f, 0x6c4f, PDF_CMAP_SINGLE, 6149 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 1004 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 6147 },
	{ 0x6c54, 0x6c54, PDF_CMAP_SINGLE, 6151 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 1005 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 1000 },
	{ 0x6c58, 0x6c58, PDF_CMAP_SINGLE, 15290 },
	{ 0x6c59, 0x6c59, PDF_CMAP_SINGLE, 1001 },
	{ 0x6c5a, 0x6c5a, PDF_CMAP_SINGLE, 16908 },
	{ 0x6c5b, 0x6c5b, PDF_CMAP_SINGLE, 1007 },
	{ 0x6c5c, 0x6c5c, PDF_CMAP_SINGLE, 6148 },
	{ 0x6c5d, 0x6c5d, PDF_CMAP_SINGLE, 999 },
	{ 0x6c5e, 0x6c5e, PDF_CMAP_SINGLE, 1228 },
	{ 0x6c5f, 0x6c60, PDF_CMAP_RANGE, 1002 },
	{ 0x6c61, 0x6c61, PDF_CMAP_SINGLE, 1006 },
	{ 0x6c65, 0x6c65, PDF_CMAP_SINGLE, 6336 },
	{ 0x6c66, 0x6c66, PDF_CMAP_SINGLE, 6334 },
	{ 0x6c67, 0x6c67, PDF_CMAP_SINGLE, 6321 },
	{ 0x6c68, 0x6c68, PDF_CMAP_SINGLE, 1240 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 6328 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 1235 },
	{ 0x6c6b, 0x6c6b, PDF_CMAP_SINGLE, 6322 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 6330 },
	{ 0x6c6e, 0x6c6e, PDF_CMAP_SINGLE, 16586 },
	{ 0x6c6f, 0x6c6f, PDF_CMAP_SINGLE, 6327 },
	{ 0x6c70, 0x6c70, PDF_CMAP_SINGLE, 1238 },
	{ 0x6c71, 0x6c71, PDF_CMAP_SINGLE, 6326 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 1245 },
	{ 0x6c73, 0x6c73, PDF_CMAP_SINGLE, 6335 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 1247 },
	{ 0x6c75, 0x6c75, PDF_CMAP_SINGLE, 18030 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 1249 },
	{ 0x6c78, 0x6c78, PDF_CMAP_SINGLE, 6320 },
	{ 0x6c79, 0x6c79, PDF_CMAP_SINGLE, 16910 },
	{ 0x6c7a, 0x6c7a, PDF_CMAP_SINGLE, 1236 },
	{ 0x6c7b, 0x6c7b, PDF_CMAP_SINGLE, 6337 },
	{ 0x6c7d, 0x6c7d, PDF_CMAP_SINGLE, 1243 },
	{ 0x6c7e, 0x6c7e, PDF_CMAP_SINGLE, 1246 },
	{ 0x6c7f, 0x6c7f, PDF_CMAP_SINGLE, 14246 },
	{ 0x6c80, 0x6c80, PDF_CMAP_SINGLE, 6628 },
	{ 0x6c81, 0x6c81, PDF_CMAP_SINGLE, 1230 },
	{ 0x6c82, 0x6c82, PDF_CMAP_SINGLE, 1253 },
	{ 0x6c83, 0x6c83, PDF_CMAP_SINGLE, 1244 },
	{ 0x6c84, 0x6c84, PDF_CMAP_SINGLE, 6323 },
	{ 0x6c85, 0x6c85, PDF_CMAP_SINGLE, 1233 },
	{ 0x6c86, 0x6c86, PDF_CMAP_SINGLE, 1248 },
	{ 0x6c87, 0x6c87, PDF_CMAP_SINGLE, 6331 },
	{ 0x6c88, 0x6c89, PDF_CMAP_RANGE, 1231 },
	{ 0x6c8a, 0x6c8a, PDF_CMAP_SINGLE, 6626 },
	{ 0x6c8b, 0x6c8b, PDF_CMAP_SINGLE, 6324 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 1239 },
	{ 0x6c8d, 0x6c8d, PDF_CMAP_SINGLE, 1250 },
	{ 0x6c8e, 0x6c8e, PDF_CMAP_SINGLE, 6338 },
	{ 0x6c8f, 0x6c8f, PDF_CMAP_SINGLE, 6325 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 1237 },
	{ 0x6c92, 0x6c92, PDF_CMAP_SINGLE, 1242 },
	{ 0x6c93, 0x6c93, PDF_CMAP_SINGLE, 6606 },
	{ 0x6c94, 0x6c94, PDF_CMAP_SINGLE, 1251 },
	{ 0x6c95, 0x6c95, PDF_CMAP_SINGLE, 6332 },
	{ 0x6c96, 0x6c96, PDF_CMAP_SINGLE, 1241 },
	{ 0x6c98, 0x6c98, PDF_CMAP_SINGLE, 1252 },
	{ 0x6c99, 0x6c99, PDF_CMAP_SINGLE, 1229 },
	{ 0x6c9a, 0x6c9a, PDF_CMAP_SINGLE, 6329 },
	{ 0x6c9b, 0x6c9b, PDF_CMAP_SINGLE, 1234 },
	{ 0x6c9c, 0x6c9c, PDF_CMAP_SINGLE, 6333 },
	{ 0x6c9d, 0x6c9d, PDF_CMAP_SINGLE, 6627 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 14473 },
	{ 0x6ca2, 0x6ca2, PDF_CMAP_SINGLE, 15715 },
	{ 0x6caa, 0x6caa, PDF_CMAP_SINGLE, 18031 },
	{ 0x6cab, 0x6cab, PDF_CMAP_SINGLE, 1579 },
	{ 0x6cac, 0x6cac, PDF_CMAP_SINGLE, 1595 },
	{ 0x6cad, 0x6cad, PDF_CMAP_SINGLE, 6613 },
	{ 0x6cae, 0x6cae, PDF_CMAP_SINGLE, 1586 },
	{ 0x6caf, 0x6caf, PDF_CMAP_SINGLE, 15606 },
	{ 0x6cb0, 0x6cb0, PDF_CMAP_SINGLE, 6634 },
	{ 0x6cb1, 0x6cb1, PDF_CMAP_SINGLE, 1571 },
	{ 0x6cb2, 0x6cb2, PDF_CMAP_SINGLE, 16076 },
	{ 0x6cb3, 0x6cb3, PDF_CMAP_SINGLE, 1574 },
	{ 0x6cb4, 0x6cb4, PDF_CMAP_SINGLE, 6625 },
	{ 0x6cb6, 0x6cb6, PDF_CMAP_SINGLE, 6611 },
	{ 0x6cb7, 0x6cb7, PDF_CMAP_SINGLE, 6615 },
	{ 0x6cb8, 0x6cb8, PDF_CMAP_SINGLE, 1582 },
	{ 0x6cb9, 0x6cb9, PDF_CMAP_SINGLE, 1584 },
	{ 0x6cba, 0x6cba, PDF_CMAP_SINGLE, 6618 },
	{ 0x6cbb, 0x6cbb, PDF_CMAP_SINGLE, 1591 },
	{ 0x6cbc, 0x6cbc, PDF_CMAP_SINGLE, 1577 },
	{ 0x6cbd, 0x6cbe, PDF_CMAP_RANGE, 1575 },
	{ 0x6cbf, 0x6cbf, PDF_CMAP_SINGLE, 1590 },
	{ 0x6cc0, 0x6cc0, PDF_CMAP_SINGLE, 6630 },
	{ 0x6cc1, 0x6cc1, PDF_CMAP_SINGLE, 1585 },
	{ 0x6cc2, 0x6cc2, PDF_CMAP_SINGLE, 6617 },
	{ 0x6cc3, 0x6cc3, PDF_CMAP_SINGLE, 6619 },
	{ 0x6cc4, 0x6cc4, PDF_CMAP_SINGLE, 1583 },
	{ 0x6cc5, 0x6cc5, PDF_CMAP_SINGLE, 1588 },
	{ 0x6cc6, 0x6cc6, PDF_CMAP_SINGLE, 6620 },
	{ 0x6cc7, 0x6cc7, PDF_CMAP_SINGLE, 6633 },
	{ 0x6cc9, 0x6cc9, PDF_CMAP_SINGLE, 1904 },
	{ 0x6cca, 0x6cca, PDF_CMAP_SINGLE, 1594 },
	{ 0x6ccb, 0x6ccb, PDF_CMAP_SINGLE, 16496 },
	{ 0x6ccc, 0x6ccc, PDF_CMAP_SINGLE, 1572 },
	{ 0x6ccd, 0x6ccd, PDF_CMAP_SINGLE, 6632 },
	{ 0x6cce, 0x6cce, PDF_CMAP_SINGLE, 16273 },
	{ 0x6ccf, 0x6ccf, PDF_CMAP_SINGLE, 6636 },
	{ 0x6cd0, 0x6cd0, PDF_CMAP_SINGLE, 6616 },
	{ 0x6cd1, 0x6cd1, PDF_CMAP_SINGLE, 6638 },
	{ 0x6cd2, 0x6cd2, PDF_CMAP_SINGLE, 6623 },
	{ 0x6cd3, 0x6cd3, PDF_CMAP_SINGLE, 1581 },
	{ 0x6cd4, 0x6cd4, PDF_CMAP_SINGLE, 6612 },
	{ 0x6cd5, 0x6cd5, PDF_CMAP_SINGLE, 1580 },
	{ 0x6cd6, 0x6cd6, PDF_CMAP_SINGLE, 1598 },
	{ 0x6cd7, 0x6cd7, PDF_CMAP_SINGLE, 1587 },
	{ 0x6cd9, 0x6cd9, PDF_CMAP_SINGLE, 6610 },
	{ 0x6cda, 0x6cda, PDF_CMAP_SINGLE, 6990 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 1593 },
	{ 0x6cdc, 0x6cdc, PDF_CMAP_SINGLE, 1597 },
	{ 0x6cdd, 0x6cdd, PDF_CMAP_SINGLE, 6624 },
	{ 0x6cde, 0x6cde, PDF_CMAP_SINGLE, 6629 },
	{ 0x6cdf, 0x6cdf, PDF_CMAP_SINGLE, 15703 },
	{ 0x6ce0, 0x6ce0, PDF_CMAP_SINGLE, 1599 },
	{ 0x6ce1, 0x6ce1, PDF_CMAP_SINGLE, 1592 },
	{ 0x6ce2, 0x6ce2, PDF_CMAP_SINGLE, 1578 },
	{ 0x6ce3, 0x6ce3, PDF_CMAP_SINGLE, 1568 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 1573 },
	{ 0x6ce7, 0x6ce7, PDF_CMAP_SINGLE, 6614 },
	{ 0x6ce8, 0x6ce8, PDF_CMAP_SINGLE, 1569 },
	{ 0x6ce9, 0x6ce9, PDF_CMAP_SINGLE, 6637 },
	{ 0x6cea, 0x6cea, PDF_CMAP_SINGLE, 16269 },
	{ 0x6ceb, 0x6ceb, PDF_CMAP_SINGLE, 6608 },
	{ 0x6cec, 0x6cec, PDF_CMAP_SINGLE, 6607 },
	{ 0x6ced, 0x6ced, PDF_CMAP_SINGLE, 6621 },
	{ 0x6cee, 0x6cee, PDF_CMAP_SINGLE, 6609 },
	{ 0x6cef, 0x6cef, PDF_CMAP_SINGLE, 1596 },
	{ 0x6cf0, 0x6cf0, PDF_CMAP_SINGLE, 2311 },
	{ 0x6cf1, 0x6cf1, PDF_CMAP_SINGLE, 1589 },
	{ 0x6cf2, 0x6cf2, PDF_CMAP_SINGLE, 6622 },
	{ 0x6cf3, 0x6cf3, PDF_CMAP_SINGLE, 1570 },
	{ 0x6cf5, 0x6cf5, PDF_CMAP_SINGLE, 1919 },
	{ 0x6cf9, 0x6cf9, PDF_CMAP_SINGLE, 6635 },
	{ 0x6cff, 0x6cff, PDF_CMAP_SINGLE, 18915 },
	{ 0x6d00, 0x6d00, PDF_CMAP_SINGLE, 6997 },
	{ 0x6d01, 0x6d01, PDF_CMAP_SINGLE, 7000 },
	{ 0x6d02, 0x6d02, PDF_CMAP_SINGLE, 18033 },
	{ 0x6d03, 0x6d03, PDF_CMAP_SINGLE, 7003 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 6992 },
	{ 0x6d05, 0x6d05, PDF_CMAP_SINGLE, 14873 },
	{ 0x6d06, 0x6d06, PDF_CMAP_SINGLE, 18034 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 7006 },
	{ 0x6d08, 0x6d08, PDF_CMAP_SINGLE, 7009 },
	{ 0x6d09, 0x6d09, PDF_CMAP_SINGLE, 7011 },
	{ 0x6d0a, 0x6d0a, PDF_CMAP_SINGLE, 6989 },
	{ 0x6d0b, 0x6d0b, PDF_CMAP_SINGLE, 1905 },
	{ 0x6d0c, 0x6d0c, PDF_CMAP_SINGLE, 1910 },
	{ 0x6d0d, 0x6d0d, PDF_CMAP_SINGLE, 7417 },
	{ 0x6d0e, 0x6d0e, PDF_CMAP_SINGLE, 1926 },
	{ 0x6d0f, 0x6d0f, PDF_CMAP_SINGLE, 7004 },
	{ 0x6d10, 0x6d10, PDF_CMAP_SINGLE, 7012 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 6996 },
	{ 0x6d12, 0x6d12, PDF_CMAP_SINGLE, 6988 },
	{ 0x6d16, 0x6d16, PDF_CMAP_SINGLE, 7444 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 1913 },
	{ 0x6d18, 0x6d18, PDF_CMAP_SINGLE, 7001 },
	{ 0x6d19, 0x6d19, PDF_CMAP_SINGLE, 6993 },
	{ 0x6d1a, 0x6d1a, PDF_CMAP_SINGLE, 6995 },
	{ 0x6d1b, 0x6d1b, PDF_CMAP_SINGLE, 1918 },
	{ 0x6d1d, 0x6d1d, PDF_CMAP_SINGLE, 6998 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 1912 },
	{ 0x6d1f, 0x6d1f, PDF_CMAP_SINGLE, 6985 },
	{ 0x6d20, 0x6d20, PDF_CMAP_SINGLE, 7007 },
	{ 0x6d22, 0x6d22, PDF_CMAP_SINGLE, 7010 },
	{ 0x6d24, 0x6d24, PDF_CMAP_SINGLE, 15043 },
	{ 0x6d25, 0x6d25, PDF_CMAP_SINGLE, 1909 },
	{ 0x6d26, 0x6d26, PDF_CMAP_SINGLE, 18035 },
	{ 0x6d27, 0x6d27, PDF_CMAP_SINGLE, 1921 },
	{ 0x6d28, 0x6d28, PDF_CMAP_SINGLE, 6982 },
	{ 0x6d29, 0x6d29, PDF_CMAP_SINGLE, 1923 },
	{ 0x6d2a, 0x6d2a, PDF_CMAP_SINGLE, 1907 },
	{ 0x6d2b, 0x6d2b, PDF_CMAP_SINGLE, 1927 },
	{ 0x6d2c, 0x6d2c, PDF_CMAP_SINGLE, 7008 },
	{ 0x6d2d, 0x6d2d, PDF_CMAP_SINGLE, 6984 },
	{ 0x6d2e, 0x6d2e, PDF_CMAP_SINGLE, 1924 },
	{ 0x6d2f, 0x6d2f, PDF_CMAP_SINGLE, 7438 },
	{ 0x6d30, 0x6d30, PDF_CMAP_SINGLE, 6631 },
	{ 0x6d31, 0x6d31, PDF_CMAP_SINGLE, 1911 },
	{ 0x6d32, 0x6d32, PDF_CMAP_SINGLE, 1906 },
	{ 0x6d33, 0x6d33, PDF_CMAP_SINGLE, 6991 },
	{ 0x6d34, 0x6d34, PDF_CMAP_SINGLE, 6983 },
	{ 0x6d35, 0x6d35, PDF_CMAP_SINGLE, 1925 },
	{ 0x6d36, 0x6d36, PDF_CMAP_SINGLE, 1917 },
	{ 0x6d37, 0x6d37, PDF_CMAP_SINGLE, 7002 },
	{ 0x6d38, 0x6d38, PDF_CMAP_SINGLE, 1922 },
	{ 0x6d39, 0x6d39, PDF_CMAP_SINGLE, 1920 },
	{ 0x6d3a, 0x6d3a, PDF_CMAP_SINGLE, 6994 },
	{ 0x6d3b, 0x6d3b, PDF_CMAP_SINGLE, 1914 },
	{ 0x6d3c, 0x6d3c, PDF_CMAP_SINGLE, 6986 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_RANGE, 1915 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 6987 },
	{ 0x6d40, 0x6d40, PDF_CMAP_SINGLE, 7005 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 1908 },
	{ 0x6d42, 0x6d42, PDF_CMAP_SINGLE, 6999 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 17752 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 15603 },
	{ 0x6d58, 0x6d58, PDF_CMAP_SINGLE, 7420 },
	{ 0x6d59, 0x6d59, PDF_CMAP_SINGLE, 2319 },
	{ 0x6d5a, 0x6d5a, PDF_CMAP_SINGLE, 2324 },
	{ 0x6d5b, 0x6d5b, PDF_CMAP_SINGLE, 16540 },
	{ 0x6d5c, 0x6d5c, PDF_CMAP_SINGLE, 16930 },
	{ 0x6d5e, 0x6d5e, PDF_CMAP_SINGLE, 7429 },
	{ 0x6d5f, 0x6d5f, PDF_CMAP_SINGLE, 7435 },
	{ 0x6d60, 0x6d60, PDF_CMAP_SINGLE, 7431 },
	{ 0x6d61, 0x6d61, PDF_CMAP_SINGLE, 7418 },
	{ 0x6d62, 0x6d62, PDF_CMAP_SINGLE, 7421 },
	{ 0x6d63, 0x6d64, PDF_CMAP_RANGE, 7414 },
	{ 0x6d65, 0x6d65, PDF_CMAP_SINGLE, 2331 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 2316 },
	{ 0x6d67, 0x6d67, PDF_CMAP_SINGLE, 7430 },
	{ 0x6d68, 0x6d68, PDF_CMAP_SINGLE, 7439 },
	{ 0x6d69, 0x6d69, PDF_CMAP_SINGLE, 2326 },
	{ 0x6d6a, 0x6d6a, PDF_CMAP_SINGLE, 2312 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 2321 },
	{ 0x6d6d, 0x6d6d, PDF_CMAP_SINGLE, 7422 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 2323 },
	{ 0x6d6f, 0x6d6f, PDF_CMAP_SINGLE, 7423 },
	{ 0x6d70, 0x6d70, PDF_CMAP_SINGLE, 7433 },
	{ 0x6d71, 0x6d71, PDF_CMAP_SINGLE, 16629 },
	{ 0x6d72, 0x6d72, PDF_CMAP_SINGLE, 14807 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 2325 },
	{ 0x6d75, 0x6d75, PDF_CMAP_SINGLE, 7448 },
	{ 0x6d76, 0x6d76, PDF_CMAP_SINGLE, 7416 },
	{ 0x6d77, 0x6d77, PDF_CMAP_SINGLE, 2318 },
	{ 0x6d78, 0x6d78, PDF_CMAP_SINGLE, 2317 },
	{ 0x6d79, 0x6d79, PDF_CMAP_SINGLE, 2329 },
	{ 0x6d7a, 0x6d7a, PDF_CMAP_SINGLE, 7413 },
	{ 0x6d7b, 0x6d7b, PDF_CMAP_SINGLE, 7446 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 7434 },
	{ 0x6d7d, 0x6d7d, PDF_CMAP_SINGLE, 7447 },
	{ 0x6d7e, 0x6d7e, PDF_CMAP_SINGLE, 7441 },
	{ 0x6d7f, 0x6d7f, PDF_CMAP_SINGLE, 7427 },
	{ 0x6d80, 0x6d80, PDF_CMAP_SINGLE, 7442 },
	{ 0x6d81, 0x6d81, PDF_CMAP_SINGLE, 18036 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 7436 },
	{ 0x6d83, 0x6d83, PDF_CMAP_SINGLE, 7445 },
	{ 0x6d84, 0x6d84, PDF_CMAP_SINGLE, 7443 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 2330 },
	{ 0x6d86, 0x6d86, PDF_CMAP_SINGLE, 7428 },
	{ 0x6d87, 0x6d87, PDF_CMAP_SINGLE, 2315 },
	{ 0x6d88, 0x6d88, PDF_CMAP_SINGLE, 2314 },
	{ 0x6d89, 0x6d89, PDF_CMAP_SINGLE, 2322 },
	{ 0x6d8a, 0x6d8a, PDF_CMAP_SINGLE, 2328 },
	{ 0x6d8b, 0x6d8b, PDF_CMAP_SINGLE, 7440 },
	{ 0x6d8c, 0x6d8c, PDF_CMAP_SINGLE, 2327 },
	{ 0x6d8d, 0x6d8d, PDF_CMAP_SINGLE, 7425 },
	{ 0x6d8e, 0x6d8e, PDF_CMAP_SINGLE, 2796 },
	{ 0x6d8f, 0x6d8f, PDF_CMAP_SINGLE, 15848 },
	{ 0x6d90, 0x6d90, PDF_CMAP_SINGLE, 7449 },
	{ 0x6d91, 0x6d91, PDF_CMAP_SINGLE, 7424 },
	{ 0x6d92, 0x6d92, PDF_CMAP_SINGLE, 7419 },
	{ 0x6d93, 0x6d93, PDF_CMAP_SINGLE, 2320 },
	{ 0x6d94, 0x6d94, PDF_CMAP_SINGLE, 2332 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 2313 },
	{ 0x6d96, 0x6d96, PDF_CMAP_SINGLE, 16929 },
	{ 0x6d97, 0x6d97, PDF_CMAP_SINGLE, 7432 },
	{ 0x6d98, 0x6d98, PDF_CMAP_SINGLE, 7437 },
	{ 0x6da4, 0x6da4, PDF_CMAP_SINGLE, 18038 },
	{ 0x6da5, 0x6da5, PDF_CMAP_SINGLE, 15033 },
	{ 0x6daa, 0x6daa, PDF_CMAP_SINGLE, 2830 },
	{ 0x6dab, 0x6dab, PDF_CMAP_SINGLE, 8006 },
	{ 0x6dac, 0x6dac, PDF_CMAP_SINGLE, 8010 },
	{ 0x6dae, 0x6dae, PDF_CMAP_SINGLE, 2811 },
	{ 0x6daf, 0x6daf, PDF_CMAP_SINGLE, 2809 },
	{ 0x6db1, 0x6db1, PDF_CMAP_SINGLE, 18039 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 2800 },
	{ 0x6db3, 0x6db3, PDF_CMAP_SINGLE, 8008 },
	{ 0x6db4, 0x6db4, PDF_CMAP_SINGLE, 8007 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 2820 },
	{ 0x6db7, 0x6db7, PDF_CMAP_SINGLE, 8013 },
	{ 0x6db8, 0x6db8, PDF_CMAP_SINGLE, 2814 },
	{ 0x6db9, 0x6db9, PDF_CMAP_SINGLE, 16379 },
	{ 0x6dba, 0x6dba, PDF_CMAP_SINGLE, 8031 },
	{ 0x6dbb, 0x6dbb, PDF_CMAP_SINGLE, 8043 },
	{ 0x6dbc, 0x6dbc, PDF_CMAP_SINGLE, 2797 },
	{ 0x6dbd, 0x6dbd, PDF_CMAP_SINGLE, 8028 },
	{ 0x6dbe, 0x6dbe, PDF_CMAP_SINGLE, 8021 },
	{ 0x6dbf, 0x6dbf, PDF_CMAP_SINGLE, 2832 },
	{ 0x6dc0, 0x6dc0, PDF_CMAP_SINGLE, 8005 },
	{ 0x6dc2, 0x6dc2, PDF_CMAP_SINGLE, 8033 },
	{ 0x6dc4, 0x6dc4, PDF_CMAP_SINGLE, 2829 },
	{ 0x6dc5, 0x6dc5, PDF_CMAP_SINGLE, 2817 },
	{ 0x6dc6, 0x6dc6, PDF_CMAP_SINGLE, 2828 },
	{ 0x6dc7, 0x6dc7, PDF_CMAP_SINGLE, 2807 },
	{ 0x6dc8, 0x6dc8, PDF_CMAP_SINGLE, 8017 },
	{ 0x6dc9, 0x6dc9, PDF_CMAP_SINGLE, 8035 },
	{ 0x6dca, 0x6dca, PDF_CMAP_SINGLE, 8027 },
	{ 0x6dcb, 0x6dcb, PDF_CMAP_SINGLE, 2808 },
	{ 0x6dcc, 0x6dcc, PDF_CMAP_SINGLE, 2802 },
	{ 0x6dcd, 0x6dcd, PDF_CMAP_SINGLE, 8041 },
	{ 0x6dcf, 0x6dcf, PDF_CMAP_SINGLE, 8034 },
	{ 0x6dd0, 0x6dd0, PDF_CMAP_SINGLE, 8036 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 2810 },
	{ 0x6dd2, 0x6dd2, PDF_CMAP_SINGLE, 2818 },
	{ 0x6dd3, 0x6dd3, PDF_CMAP_SINGLE, 8038 },
	{ 0x6dd4, 0x6dd4, PDF_CMAP_SINGLE, 8015 },
	{ 0x6dd5, 0x6dd5, PDF_CMAP_SINGLE, 8032 },
	{ 0x6dd6, 0x6dd6, PDF_CMAP_SINGLE, 8020 },
	{ 0x6dd7, 0x6dd7, PDF_CMAP_SINGLE, 8040 },
	{ 0x6dd8, 0x6dd8, PDF_CMAP_SINGLE, 2823 },
	{ 0x6dd9, 0x6dd9, PDF_CMAP_SINGLE, 2799 },
	{ 0x6dda, 0x6dda, PDF_CMAP_SINGLE, 2821 },
	{ 0x6ddb, 0x6ddb, PDF_CMAP_SINGLE, 8025 },
	{ 0x6ddc, 0x6ddd, PDF_CMAP_RANGE, 8023 },
	{ 0x6dde, 0x6dde, PDF_CMAP_SINGLE, 2812 },
	{ 0x6ddf, 0x6ddf, PDF_CMAP_SINGLE, 8019 },
	{ 0x6de0, 0x6de0, PDF_CMAP_SINGLE, 8018 },
	{ 0x6de1, 0x6de1, PDF_CMAP_SINGLE, 2801 },
	{ 0x6de2, 0x6de2, PDF_CMAP_SINGLE, 8012 },
	{ 0x6de3, 0x6de3, PDF_CMAP_SINGLE, 8042 },
	{ 0x6de4, 0x6de4, PDF_CMAP_SINGLE, 2803 },
	{ 0x6de5, 0x6de5, PDF_CMAP_SINGLE, 8022 },
	{ 0x6de6, 0x6de6, PDF_CMAP_SINGLE, 2833 },
	{ 0x6de8, 0x6de8, PDF_CMAP_SINGLE, 2827 },
	{ 0x6de9, 0x6de9, PDF_CMAP_SINGLE, 8011 },
	{ 0x6dea, 0x6dea, PDF_CMAP_SINGLE, 2824 },
	{ 0x6deb, 0x6deb, PDF_CMAP_SINGLE, 2822 },
	{ 0x6dec, 0x6dec, PDF_CMAP_SINGLE, 2831 },
	{ 0x6ded, 0x6ded, PDF_CMAP_SINGLE, 8029 },
	{ 0x6dee, 0x6dee, PDF_CMAP_SINGLE, 2826 },
	{ 0x6def, 0x6def, PDF_CMAP_SINGLE, 7426 },
	{ 0x6df0, 0x6df0, PDF_CMAP_SINGLE, 8030 },
	{ 0x6df1, 0x6df1, PDF_CMAP_SINGLE, 2825 },
	{ 0x6df2, 0x6df2, PDF_CMAP_SINGLE, 8037 },
	{ 0x6df3, 0x6df3, PDF_CMAP_SINGLE, 2798 },
	{ 0x6df4, 0x6df4, PDF_CMAP_SINGLE, 8026 },
	{ 0x6df5, 0x6df5, PDF_CMAP_SINGLE, 2816 },
	{ 0x6df6, 0x6df6, PDF_CMAP_SINGLE, 8014 },
	{ 0x6df7, 0x6df7, PDF_CMAP_SINGLE, 2815 },
	{ 0x6df9, 0x6df9, PDF_CMAP_SINGLE, 2813 },
	{ 0x6dfa, 0x6dfa, PDF_CMAP_SINGLE, 2805 },
	{ 0x6dfb, 0x6dfb, PDF_CMAP_SINGLE, 2804 },
	{ 0x6dfc, 0x6dfc, PDF_CMAP_SINGLE, 8639 },
	{ 0x6dfd, 0x6dfd, PDF_CMAP_SINGLE, 8039 },
	{ 0x6dfe, 0x6dfe, PDF_CMAP_SINGLE, 18850 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 8016 },
	{ 0x6e02, 0x6e02, PDF_CMAP_SINGLE, 15610 },
	{ 0x6e03, 0x6e03, PDF_CMAP_SINGLE, 8660 },
	{ 0x6e04, 0x6e04, PDF_CMAP_SINGLE, 15038 },
	{ 0x6e05, 0x6e05, PDF_CMAP_SINGLE, 2806 },
	{ 0x6e0a, 0x6e0a, PDF_CMAP_SINGLE, 15041 },
	{ 0x6e0f, 0x6e0f, PDF_CMAP_SINGLE, 15600 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 18040 },
	{ 0x6e18, 0x6e18, PDF_CMAP_SINGLE, 18041 },
	{ 0x6e19, 0x6e19, PDF_CMAP_SINGLE, 3265 },
	{ 0x6e1a, 0x6e1a, PDF_CMAP_SINGLE, 2819 },
	{ 0x6e1b, 0x6e1b, PDF_CMAP_SINGLE, 3247 },
	{ 0x6e1c, 0x6e1c, PDF_CMAP_SINGLE, 8654 },
	{ 0x6e1d, 0x6e1d, PDF_CMAP_SINGLE, 3261 },
	{ 0x6e1f, 0x6e1f, PDF_CMAP_SINGLE, 8642 },
	{ 0x6e20, 0x6e20, PDF_CMAP_SINGLE, 3244 },
	{ 0x6e21, 0x6e21, PDF_CMAP_SINGLE, 3240 },
	{ 0x6e22, 0x6e22, PDF_CMAP_SINGLE, 8672 },
	{ 0x6e23, 0x6e23, PDF_CMAP_SINGLE, 3246 },
	{ 0x6e24, 0x6e24, PDF_CMAP_SINGLE, 3250 },
	{ 0x6e25, 0x6e25, PDF_CMAP_SINGLE, 3245 },
	{ 0x6e26, 0x6e26, PDF_CMAP_SINGLE, 3254 },
	{ 0x6e27, 0x6e27, PDF_CMAP_SINGLE, 8676 },
	{ 0x6e28, 0x6e28, PDF_CMAP_SINGLE, 8667 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 18042 },
	{ 0x6e2a, 0x6e2a, PDF_CMAP_SINGLE, 17154 },
	{ 0x6e2b, 0x6e2b, PDF_CMAP_SINGLE, 8649 },
	{ 0x6e2c, 0x6e2c, PDF_CMAP_SINGLE, 3259 },
	{ 0x6e2d, 0x6e2d, PDF_CMAP_SINGLE, 3253 },
	{ 0x6e2e, 0x6e2e, PDF_CMAP_SINGLE, 8661 },
	{ 0x6e2f, 0x6e2f, PDF_CMAP_SINGLE, 3237 },
	{ 0x6e30, 0x6e30, PDF_CMAP_SINGLE, 8673 },
	{ 0x6e31, 0x6e31, PDF_CMAP_SINGLE, 8666 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 3241 },
	{ 0x6e33, 0x6e33, PDF_CMAP_SINGLE, 8655 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 3256 },
	{ 0x6e35, 0x6e36, PDF_CMAP_RANGE, 8684 },
	{ 0x6e38, 0x6e38, PDF_CMAP_SINGLE, 3238 },
	{ 0x6e39, 0x6e39, PDF_CMAP_SINGLE, 8671 },
	{ 0x6e3a, 0x6e3a, PDF_CMAP_SINGLE, 3258 },
	{ 0x6e3b, 0x6e3b, PDF_CMAP_SINGLE, 8659 },
	{ 0x6e3c, 0x6e3d, PDF_CMAP_RANGE, 8645 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 3262 },
	{ 0x6e3f, 0x6e3f, PDF_CMAP_SINGLE, 8650 },
	{ 0x6e40, 0x6e40, PDF_CMAP_SINGLE, 8657 },
	{ 0x6e41, 0x6e41, PDF_CMAP_SINGLE, 8651 },
	{ 0x6e43, 0x6e43, PDF_CMAP_SINGLE, 3260 },
	{ 0x6e44, 0x6e44, PDF_CMAP_SINGLE, 3268 },
	{ 0x6e45, 0x6e45, PDF_CMAP_SINGLE, 8647 },
	{ 0x6e46, 0x6e47, PDF_CMAP_RANGE, 8640 },
	{ 0x6e49, 0x6e49, PDF_CMAP_SINGLE, 8643 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 3243 },
	{ 0x6e4b, 0x6e4b, PDF_CMAP_SINGLE, 8656 },
	{ 0x6e4d, 0x6e4d, PDF_CMAP_SINGLE, 3257 },
	{ 0x6e4e, 0x6e4e, PDF_CMAP_SINGLE, 3266 },
	{ 0x6e4f, 0x6e4f, PDF_CMAP_SINGLE, 16948 },
	{ 0x6e50, 0x6e50, PDF_CMAP_SINGLE, 16849 },
	{ 0x6e51, 0x6e51, PDF_CMAP_SINGLE, 8658 },
	{ 0x6e52, 0x6e52, PDF_CMAP_SINGLE, 8682 },
	{ 0x6e53, 0x6e53, PDF_CMAP_SINGLE, 8674 },
	{ 0x6e54, 0x6e54, PDF_CMAP_SINGLE, 3239 },
	{ 0x6e55, 0x6e55, PDF_CMAP_SINGLE, 8680 },
	{ 0x6e56, 0x6e56, PDF_CMAP_SINGLE, 3251 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 19024 },
	{ 0x6e58, 0x6e58, PDF_CMAP_SINGLE, 3249 },
	{ 0x6e59, 0x6e59, PDF_CMAP_SINGLE, 16614 },
	{ 0x6e5a, 0x6e5a, PDF_CMAP_SINGLE, 8686 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 3248 },
	{ 0x6e5c, 0x6e5c, PDF_CMAP_SINGLE, 8664 },
	{ 0x6e5d, 0x6e5d, PDF_CMAP_SINGLE, 8652 },
	{ 0x6e5e, 0x6e5e, PDF_CMAP_SINGLE, 8662 },
	{ 0x6e5f, 0x6e5f, PDF_CMAP_SINGLE, 3271 },
	{ 0x6e60, 0x6e60, PDF_CMAP_SINGLE, 8668 },
	{ 0x6e61, 0x6e61, PDF_CMAP_SINGLE, 8665 },
	{ 0x6e62, 0x6e62, PDF_CMAP_SINGLE, 8648 },
	{ 0x6e63, 0x6e63, PDF_CMAP_SINGLE, 3267 },
	{ 0x6e64, 0x6e64, PDF_CMAP_SINGLE, 8678 },
	{ 0x6e65, 0x6e65, PDF_CMAP_SINGLE, 8675 },
	{ 0x6e66, 0x6e66, PDF_CMAP_SINGLE, 8683 },
	{ 0x6e67, 0x6e67, PDF_CMAP_SINGLE, 3242 },
	{ 0x6e68, 0x6e68, PDF_CMAP_SINGLE, 8663 },
	{ 0x6e69, 0x6e69, PDF_CMAP_SINGLE, 3270 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 8670 },
	{ 0x6e6e, 0x6e6e, PDF_CMAP_SINGLE, 3252 },
	{ 0x6e6f, 0x6e6f, PDF_CMAP_SINGLE, 3255 },
	{ 0x6e71, 0x6e71, PDF_CMAP_SINGLE, 8669 },
	{ 0x6e72, 0x6e72, PDF_CMAP_SINGLE, 3269 },
	{ 0x6e73, 0x6e73, PDF_CMAP_SINGLE, 8653 },
	{ 0x6e74, 0x6e74, PDF_CMAP_SINGLE, 8009 },
	{ 0x6e76, 0x6e76, PDF_CMAP_SINGLE, 14803 },
	{ 0x6e77, 0x6e77, PDF_CMAP_SINGLE, 8679 },
	{ 0x6e78, 0x6e78, PDF_CMAP_SINGLE, 8677 },
	{ 0x6e79, 0x6e79, PDF_CMAP_SINGLE, 8681 },
	{ 0x6e7c, 0x6e7c, PDF_CMAP_SINGLE, 16931 },
	{ 0x6e86, 0x6e86, PDF_CMAP_SINGLE, 18043 },
	{ 0x6e88, 0x6e88, PDF_CMAP_SINGLE, 8644 },
	{ 0x6e89, 0x6e89, PDF_CMAP_SINGLE, 3264 },
	{ 0x6e8b, 0x6e8b, PDF_CMAP_SINGLE, 15295 },
	{ 0x6e8d, 0x6e8d, PDF_CMAP_SINGLE, 9324 },
	{ 0x6e8e, 0x6e8e, PDF_CMAP_SINGLE, 9323 },
	{ 0x6e8f, 0x6e8f, PDF_CMAP_SINGLE, 9297 },
	{ 0x6e90, 0x6e90, PDF_CMAP_SINGLE, 3669 },
	{ 0x6e92, 0x6e92, PDF_CMAP_SINGLE, 9322 },
	{ 0x6e93, 0x6e94, PDF_CMAP_RANGE, 9300 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 3679 },
	{ 0x6e97, 0x6e97, PDF_CMAP_SINGLE, 9331 },
	{ 0x6e98, 0x6e98, PDF_CMAP_SINGLE, 3674 },
	{ 0x6e99, 0x6e99, PDF_CMAP_SINGLE, 9321 },
	{ 0x6e9a, 0x6e9a, PDF_CMAP_SINGLE, 17793 },
	{ 0x6e9b, 0x6e9b, PDF_CMAP_SINGLE, 9294 },
	{ 0x6e9c, 0x6e9c, PDF_CMAP_SINGLE, 3680 },
	{ 0x6e9d, 0x6e9d, PDF_CMAP_SINGLE, 3670 },
	{ 0x6e9e, 0x6e9e, PDF_CMAP_SINGLE, 9309 },
	{ 0x6e9f, 0x6e9f, PDF_CMAP_SINGLE, 9299 },
	{ 0x6ea0, 0x6ea0, PDF_CMAP_SINGLE, 9302 },
	{ 0x6ea1, 0x6ea1, PDF_CMAP_SINGLE, 9326 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 3664 },
	{ 0x6ea3, 0x6ea3, PDF_CMAP_SINGLE, 9333 },
	{ 0x6ea4, 0x6ea4, PDF_CMAP_SINGLE, 9325 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 3673 },
	{ 0x6ea6, 0x6ea6, PDF_CMAP_SINGLE, 9314 },
	{ 0x6ea7, 0x6ea7, PDF_CMAP_SINGLE, 3684 },
	{ 0x6eaa, 0x6eaa, PDF_CMAP_SINGLE, 3683 },
	{ 0x6eab, 0x6eab, PDF_CMAP_SINGLE, 3677 },
	{ 0x6eae, 0x6eae, PDF_CMAP_SINGLE, 9332 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 3665 },
	{ 0x6eb0, 0x6eb0, PDF_CMAP_SINGLE, 9312 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 9303 },
	{ 0x6eb2, 0x6eb2, PDF_CMAP_SINGLE, 9316 },
	{ 0x6eb3, 0x6eb3, PDF_CMAP_SINGLE, 9328 },
	{ 0x6eb4, 0x6eb4, PDF_CMAP_SINGLE, 3685 },
	{ 0x6eb5, 0x6eb5, PDF_CMAP_SINGLE, 16894 },
	{ 0x6eb6, 0x6eb6, PDF_CMAP_SINGLE, 3667 },
	{ 0x6eb7, 0x6eb7, PDF_CMAP_SINGLE, 9311 },
	{ 0x6eb8, 0x6eb8, PDF_CMAP_SINGLE, 15884 },
	{ 0x6eb9, 0x6eb9, PDF_CMAP_SINGLE, 9304 },
	{ 0x6eba, 0x6eba, PDF_CMAP_SINGLE, 3676 },
	{ 0x6ebb, 0x6ebb, PDF_CMAP_SINGLE, 18045 },
	{ 0x6ebc, 0x6ebc, PDF_CMAP_SINGLE, 3675 },
	{ 0x6ebd, 0x6ebd, PDF_CMAP_SINGLE, 9307 },
	{ 0x6ebe, 0x6ebe, PDF_CMAP_SINGLE, 9317 },
	{ 0x6ebf, 0x6ebf, PDF_CMAP_SINGLE, 9327 },
	{ 0x6ec0, 0x6ec0, PDF_CMAP_SINGLE, 9298 },
	{ 0x6ec1, 0x6ec1, PDF_CMAP_SINGLE, 9308 },
	{ 0x6ec2, 0x6ec2, PDF_CMAP_SINGLE, 3668 },
	{ 0x6ec3, 0x6ec3, PDF_CMAP_SINGLE, 9318 },
	{ 0x6ec4, 0x6ec4, PDF_CMAP_SINGLE, 3681 },
	{ 0x6ec5, 0x6ec5, PDF_CMAP_SINGLE, 3672 },
	{ 0x6ec6, 0x6ec6, PDF_CMAP_SINGLE, 9305 },
	{ 0x6ec7, 0x6ec7, PDF_CMAP_SINGLE, 3671 },
	{ 0x6ec8, 0x6ec8, PDF_CMAP_SINGLE, 9296 },
	{ 0x6ec9, 0x6ec9, PDF_CMAP_SINGLE, 9310 },
	{ 0x6eca, 0x6eca, PDF_CMAP_SINGLE, 9330 },
	{ 0x6ecb, 0x6ecb, PDF_CMAP_SINGLE, 3263 },
	{ 0x6ecc, 0x6ecc, PDF_CMAP_SINGLE, 4127 },
	{ 0x6ecd, 0x6ecd, PDF_CMAP_SINGLE, 9313 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 9963 },
	{ 0x6ecf, 0x6ecf, PDF_CMAP_SINGLE, 9315 },
	{ 0x6ed0, 0x6ed0, PDF_CMAP_SINGLE, 9329 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 3678 },
	{ 0x6ed2, 0x6ed2, PDF_CMAP_SINGLE, 9306 },
	{ 0x6ed3, 0x6ed3, PDF_CMAP_SINGLE, 3666 },
	{ 0x6ed4, 0x6ed4, PDF_CMAP_SINGLE, 3682 },
	{ 0x6ed5, 0x6ed5, PDF_CMAP_SINGLE, 4506 },
	{ 0x6ed6, 0x6ed6, PDF_CMAP_SINGLE, 9295 },
	{ 0x6ed8, 0x6ed8, PDF_CMAP_SINGLE, 9320 },
	{ 0x6ed9, 0x6ed9, PDF_CMAP_SINGLE, 16489 },
	{ 0x6eda, 0x6eda, PDF_CMAP_SINGLE, 18047 },
	{ 0x6edb, 0x6edb, PDF_CMAP_SINGLE, 16431 },
	{ 0x6edc, 0x6edc, PDF_CMAP_SINGLE, 9319 },
	{ 0x6edd, 0x6edd, PDF_CMAP_SINGLE, 17720 },
	{ 0x6ee2, 0x6ee2, PDF_CMAP_SINGLE, 18046 },
	{ 0x6ee8, 0x6ee9, PDF_CMAP_RANGE, 18049 },
	{ 0x6eeb, 0x6eeb, PDF_CMAP_SINGLE, 9991 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 4124 },
	{ 0x6eed, 0x6eed, PDF_CMAP_SINGLE, 9980 },
	{ 0x6eee, 0x6eee, PDF_CMAP_SINGLE, 9985 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 4113 },
	{ 0x6ef1, 0x6ef1, PDF_CMAP_SINGLE, 9965 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 4126 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 4104 },
	{ 0x6ef5, 0x6ef5, PDF_CMAP_SINGLE, 9964 },
	{ 0x6ef6, 0x6ef6, PDF_CMAP_SINGLE, 9997 },
	{ 0x6ef7, 0x6ef7, PDF_CMAP_SINGLE, 4128 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 9968 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 9984 },
	{ 0x6efa, 0x6efa, PDF_CMAP_SINGLE, 16955 },
	{ 0x6efb, 0x6efb, PDF_CMAP_SINGLE, 9970 },
	{ 0x6efc, 0x6efc, PDF_CMAP_SINGLE, 10000 },
	{ 0x6efd, 0x6efd, PDF_CMAP_SINGLE, 9996 },
	{ 0x6efe, 0x6efe, PDF_CMAP_SINGLE, 4102 },
	{ 0x6eff, 0x6eff, PDF_CMAP_SINGLE, 4112 },
	{ 0x6f00, 0x6f00, PDF_CMAP_SINGLE, 10566 },
	{ 0x6f01, 0x6f01, PDF_CMAP_SINGLE, 4125 },
	{ 0x6f02, 0x6f02, PDF_CMAP_SINGLE, 4110 },
	{ 0x6f03, 0x6f03, PDF_CMAP_SINGLE, 9966 },
	{ 0x6f04, 0x6f04, PDF_CMAP_SINGLE, 16932 },
	{ 0x6f05, 0x6f05, PDF_CMAP_SINGLE, 9995 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 4114 },
	{ 0x6f07, 0x6f07, PDF_CMAP_SINGLE, 9992 },
	{ 0x6f08, 0x6f08, PDF_CMAP_SINGLE, 10005 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 9972 },
	{ 0x6f0a, 0x6f0a, PDF_CMAP_SINGLE, 9981 },
	{ 0x6f0b, 0x6f0b, PDF_CMAP_SINGLE, 15034 },
	{ 0x6f0c, 0x6f0c, PDF_CMAP_SINGLE, 15611 },
	{ 0x6f0d, 0x6f0d, PDF_CMAP_SINGLE, 10003 },
	{ 0x6f0e, 0x6f0e, PDF_CMAP_SINGLE, 9993 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 4109 },
	{ 0x6f12, 0x6f12, PDF_CMAP_SINGLE, 9979 },
	{ 0x6f13, 0x6f13, PDF_CMAP_SINGLE, 4103 },
	{ 0x6f14, 0x6f14, PDF_CMAP_SINGLE, 4101 },
	{ 0x6f15, 0x6f15, PDF_CMAP_SINGLE, 4119 },
	{ 0x6f16, 0x6f16, PDF_CMAP_SINGLE, 16800 },
	{ 0x6f17, 0x6f17, PDF_CMAP_SINGLE, 18126 },
	{ 0x6f18, 0x6f18, PDF_CMAP_SINGLE, 9977 },
	{ 0x6f19, 0x6f1a, PDF_CMAP_RANGE, 9974 },
	{ 0x6f1c, 0x6f1c, PDF_CMAP_SINGLE, 9999 },
	{ 0x6f1e, 0x6f1e, PDF_CMAP_SINGLE, 10004 },
	{ 0x6f1f, 0x6f1f, PDF_CMAP_SINGLE, 10002 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 4107 },
	{ 0x6f21, 0x6f21, PDF_CMAP_SINGLE, 10006 },
	{ 0x6f22, 0x6f22, PDF_CMAP_SINGLE, 4111 },
	{ 0x6f23, 0x6f23, PDF_CMAP_SINGLE, 4118 },
	{ 0x6f24, 0x6f24, PDF_CMAP_SINGLE, 18051 },
	{ 0x6f25, 0x6f25, PDF_CMAP_SINGLE, 9967 },
	{ 0x6f26, 0x6f26, PDF_CMAP_SINGLE, 10542 },
	{ 0x6f27, 0x6f27, PDF_CMAP_SINGLE, 9976 },
	{ 0x6f29, 0x6f29, PDF_CMAP_SINGLE, 4105 },
	{ 0x6f2a, 0x6f2a, PDF_CMAP_SINGLE, 4123 },
	{ 0x6f2b, 0x6f2b, PDF_CMAP_SINGLE, 4120 },
	{ 0x6f2c, 0x6f2c, PDF_CMAP_SINGLE, 4108 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 9986 },
	{ 0x6f2e, 0x6f2e, PDF_CMAP_SINGLE, 9971 },
	{ 0x6f2f, 0x6f2f, PDF_CMAP_SINGLE, 4121 },
	{ 0x6f30, 0x6f30, PDF_CMAP_SINGLE, 9988 },
	{ 0x6f31, 0x6f31, PDF_CMAP_SINGLE, 4115 },
	{ 0x6f32, 0x6f32, PDF_CMAP_SINGLE, 4117 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 4100 },
	{ 0x6f34, 0x6f34, PDF_CMAP_SINGLE, 18052 },
	{ 0x6f35, 0x6f35, PDF_CMAP_SINGLE, 9990 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 9982 },
	{ 0x6f37, 0x6f37, PDF_CMAP_SINGLE, 9969 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 4116 },
	{ 0x6f39, 0x6f39, PDF_CMAP_SINGLE, 9998 },
	{ 0x6f3a, 0x6f3a, PDF_CMAP_SINGLE, 10001 },
	{ 0x6f3b, 0x6f3b, PDF_CMAP_SINGLE, 9978 },
	{ 0x6f3c, 0x6f3c, PDF_CMAP_SINGLE, 9989 },
	{ 0x6f3d, 0x6f3d, PDF_CMAP_SINGLE, 17026 },
	{ 0x6f3e, 0x6f3e, PDF_CMAP_SINGLE, 4106 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 4489 },
	{ 0x6f40, 0x6f40, PDF_CMAP_SINGLE, 9987 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 10541 },
	{ 0x6f43, 0x6f43, PDF_CMAP_SINGLE, 9994 },
	{ 0x6f44, 0x6f44, PDF_CMAP_SINGLE, 16960 },
	{ 0x6f4e, 0x6f4e, PDF_CMAP_SINGLE, 9973 },
	{ 0x6f4f, 0x6f4f, PDF_CMAP_SINGLE, 10551 },
	{ 0x6f50, 0x6f50, PDF_CMAP_SINGLE, 10561 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 4492 },
	{ 0x6f52, 0x6f52, PDF_CMAP_SINGLE, 10560 },
	{ 0x6f53, 0x6f53, PDF_CMAP_SINGLE, 10572 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 4494 },
	{ 0x6f55, 0x6f55, PDF_CMAP_SINGLE, 10558 },
	{ 0x6f56, 0x6f56, PDF_CMAP_SINGLE, 15954 },
	{ 0x6f57, 0x6f57, PDF_CMAP_SINGLE, 10562 },
	{ 0x6f58, 0x6f58, PDF_CMAP_SINGLE, 4505 },
	{ 0x6f5a, 0x6f5a, PDF_CMAP_SINGLE, 10553 },
	{ 0x6f5b, 0x6f5b, PDF_CMAP_SINGLE, 4497 },
	{ 0x6f5c, 0x6f5c, PDF_CMAP_SINGLE, 16961 },
	{ 0x6f5d, 0x6f5d, PDF_CMAP_SINGLE, 10565 },
	{ 0x6f5e, 0x6f5e, PDF_CMAP_SINGLE, 11162 },
	{ 0x6f5f, 0x6f5f, PDF_CMAP_SINGLE, 4509 },
	{ 0x6f60, 0x6f60, PDF_CMAP_SINGLE, 4508 },
	{ 0x6f61, 0x6f61, PDF_CMAP_SINGLE, 10567 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 10550 },
	{ 0x6f63, 0x6f63, PDF_CMAP_SINGLE, 10577 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 4503 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 4493 },
	{ 0x6f67, 0x6f67, PDF_CMAP_SINGLE, 10570 },
	{ 0x6f69, 0x6f69, PDF_CMAP_SINGLE, 10574 },
	{ 0x6f6a, 0x6f6a, PDF_CMAP_SINGLE, 10579 },
	{ 0x6f6b, 0x6f6b, PDF_CMAP_SINGLE, 10568 },
	{ 0x6f6c, 0x6f6c, PDF_CMAP_SINGLE, 10556 },
	{ 0x6f6d, 0x6f6d, PDF_CMAP_SINGLE, 4496 },
	{ 0x6f6e, 0x6f6e, PDF_CMAP_SINGLE, 4499 },
	{ 0x6f6f, 0x6f6f, PDF_CMAP_SINGLE, 4507 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 4502 },
	{ 0x6f72, 0x6f72, PDF_CMAP_SINGLE, 10559 },
	{ 0x6f73, 0x6f73, PDF_CMAP_SINGLE, 9983 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 16963 },
	{ 0x6f76, 0x6f76, PDF_CMAP_SINGLE, 10555 },
	{ 0x6f77, 0x6f77, PDF_CMAP_SINGLE, 10578 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 4498 },
	{ 0x6f79, 0x6f79, PDF_CMAP_SINGLE, 15294 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 4501 },
	{ 0x6f7b, 0x6f7b, PDF_CMAP_SINGLE, 10580 },
	{ 0x6f7c, 0x6f7c, PDF_CMAP_SINGLE, 4490 },
	{ 0x6f7d, 0x6f7d, PDF_CMAP_SINGLE, 10569 },
	{ 0x6f7e, 0x6f7e, PDF_CMAP_SINGLE, 10543 },
	{ 0x6f7f, 0x6f7f, PDF_CMAP_SINGLE, 10575 },
	{ 0x6f80, 0x6f80, PDF_CMAP_SINGLE, 5116 },
	{ 0x6f81, 0x6f81, PDF_CMAP_SINGLE, 18055 },
	{ 0x6f82, 0x6f82, PDF_CMAP_SINGLE, 10557 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 4491 },
	{ 0x6f85, 0x6f85, PDF_CMAP_SINGLE, 10552 },
	{ 0x6f86, 0x6f86, PDF_CMAP_SINGLE, 4495 },
	{ 0x6f87, 0x6f87, PDF_CMAP_SINGLE, 10544 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 4122 },
	{ 0x6f89, 0x6f89, PDF_CMAP_SINGLE, 10548 },
	{ 0x6f8a, 0x6f8a, PDF_CMAP_SINGLE, 16789 },
	{ 0x6f8b, 0x6f8b, PDF_CMAP_SINGLE, 10573 },
	{ 0x6f8c, 0x6f8c, PDF_CMAP_SINGLE, 10549 },
	{ 0x6f8d, 0x6f8d, PDF_CMAP_SINGLE, 10547 },
	{ 0x6f8e, 0x6f8e, PDF_CMAP_SINGLE, 4500 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 10571 },
	{ 0x6f92, 0x6f92, PDF_CMAP_SINGLE, 10546 },
	{ 0x6f93, 0x6f93, PDF_CMAP_SINGLE, 10564 },
	{ 0x6f94, 0x6f94, PDF_CMAP_SINGLE, 10563 },
	{ 0x6f95, 0x6f95, PDF_CMAP_SINGLE, 10576 },
	{ 0x6f96, 0x6f96, PDF_CMAP_SINGLE, 10554 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 4504 },
	{ 0x6f9d, 0x6f9d, PDF_CMAP_SINGLE, 16953 },
	{ 0x6f9e, 0x6f9e, PDF_CMAP_SINGLE, 11165 },
	{ 0x6f9f, 0x6f9f, PDF_CMAP_SINGLE, 16966 },
	{ 0x6fa0, 0x6fa0, PDF_CMAP_SINGLE, 4843 },
	{ 0x6fa1, 0x6fa1, PDF_CMAP_SINGLE, 4833 },
	{ 0x6fa2, 0x6fa2, PDF_CMAP_SINGLE, 11177 },
	{ 0x6fa3, 0x6fa3, PDF_CMAP_SINGLE, 11157 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 4835 },
	{ 0x6fa5, 0x6fa5, PDF_CMAP_SINGLE, 11169 },
	{ 0x6fa6, 0x6fa6, PDF_CMAP_SINGLE, 4842 },
	{ 0x6fa7, 0x6fa7, PDF_CMAP_SINGLE, 4837 },
	{ 0x6fa8, 0x6fa8, PDF_CMAP_SINGLE, 11167 },
	{ 0x6fa9, 0x6fa9, PDF_CMAP_SINGLE, 11662 },
	{ 0x6faa, 0x6faa, PDF_CMAP_SINGLE, 11173 },
	{ 0x6fab, 0x6fab, PDF_CMAP_SINGLE, 11179 },
	{ 0x6fac, 0x6fac, PDF_CMAP_SINGLE, 11172 },
	{ 0x6fad, 0x6fad, PDF_CMAP_SINGLE, 11155 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 11170 },
	{ 0x6faf, 0x6faf, PDF_CMAP_SINGLE, 11181 },
	{ 0x6fb0, 0x6fb0, PDF_CMAP_SINGLE, 11183 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 4832 },
	{ 0x6fb2, 0x6fb2, PDF_CMAP_SINGLE, 11182 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 4838 },
	{ 0x6fb4, 0x6fb4, PDF_CMAP_SINGLE, 4844 },
	{ 0x6fb5, 0x6fb5, PDF_CMAP_SINGLE, 15911 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 4841 },
	{ 0x6fb8, 0x6fb8, PDF_CMAP_SINGLE, 11176 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 4840 },
	{ 0x6fba, 0x6fba, PDF_CMAP_SINGLE, 11171 },
	{ 0x6fbb, 0x6fbb, PDF_CMAP_SINGLE, 15291 },
	{ 0x6fbc, 0x6fbc, PDF_CMAP_SINGLE, 11159 },
	{ 0x6fbd, 0x6fbd, PDF_CMAP_SINGLE, 11164 },
	{ 0x6fbe, 0x6fbe, PDF_CMAP_SINGLE, 18056 },
	{ 0x6fbf, 0x6fbf, PDF_CMAP_SINGLE, 11175 },
	{ 0x6fc0, 0x6fc0, PDF_CMAP_SINGLE, 4839 },
	{ 0x6fc1, 0x6fc1, PDF_CMAP_SINGLE, 4836 },
	{ 0x6fc2, 0x6fc2, PDF_CMAP_SINGLE, 4831 },
	{ 0x6fc3, 0x6fc3, PDF_CMAP_SINGLE, 4834 },
	{ 0x6fc4, 0x6fc4, PDF_CMAP_SINGLE, 11163 },
	{ 0x6fc6, 0x6fc6, PDF_CMAP_SINGLE, 10545 },
	{ 0x6fc7, 0x6fc7, PDF_CMAP_SINGLE, 11158 },
	{ 0x6fc8, 0x6fc8, PDF_CMAP_SINGLE, 11161 },
	{ 0x6fc9, 0x6fc9, PDF_CMAP_SINGLE, 11178 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 11166 },
	{ 0x6fcb, 0x6fcb, PDF_CMAP_SINGLE, 11156 },
	{ 0x6fcc, 0x6fcc, PDF_CMAP_SINGLE, 11661 },
	{ 0x6fcd, 0x6fcd, PDF_CMAP_SINGLE, 11180 },
	{ 0x6fce, 0x6fce, PDF_CMAP_SINGLE, 11160 },
	{ 0x6fcf, 0x6fcf, PDF_CMAP_SINGLE, 11174 },
	{ 0x6fd3, 0x6fd3, PDF_CMAP_SINGLE, 16968 },
	{ 0x6fd4, 0x6fd4, PDF_CMAP_SINGLE, 11664 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 5120 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 5108 },
	{ 0x6fd9, 0x6fd9, PDF_CMAP_SINGLE, 15476 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 15475 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 5112 },
	{ 0x6fdc, 0x6fdc, PDF_CMAP_SINGLE, 11666 },
	{ 0x6fdd, 0x6fdd, PDF_CMAP_SINGLE, 11672 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 11670 },
	{ 0x6fdf, 0x6fe0, PDF_CMAP_RANGE, 5110 },
	{ 0x6fe1, 0x6fe1, PDF_CMAP_SINGLE, 5118 },
	{ 0x6fe2, 0x6fe2, PDF_CMAP_SINGLE, 11673 },
	{ 0x6fe3, 0x6fe3, PDF_CMAP_SINGLE, 11665 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 5113 },
	{ 0x6fe6, 0x6fe6, PDF_CMAP_SINGLE, 11669 },
	{ 0x6fe7, 0x6fe7, PDF_CMAP_SINGLE, 11668 },
	{ 0x6fe8, 0x6fe8, PDF_CMAP_SINGLE, 11674 },
	{ 0x6fe9, 0x6fe9, PDF_CMAP_SINGLE, 5119 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 5114 },
	{ 0x6fec, 0x6fec, PDF_CMAP_SINGLE, 5117 },
	{ 0x6fed, 0x6fed, PDF_CMAP_SINGLE, 11667 },
	{ 0x6fee, 0x6fee, PDF_CMAP_SINGLE, 5121 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 5115 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 5122 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 5109 },
	{ 0x6ff2, 0x6ff2, PDF_CMAP_SINGLE, 11671 },
	{ 0x6ff4, 0x6ff4, PDF_CMAP_SINGLE, 11663 },
	{ 0x6ff6, 0x6ff6, PDF_CMAP_SINGLE, 14501 },
	{ 0x6ff7, 0x6ff7, PDF_CMAP_SINGLE, 12110 },
	{ 0x6ff8, 0x6ff8, PDF_CMAP_SINGLE, 16951 },
	{ 0x6ffa, 0x6ffa, PDF_CMAP_SINGLE, 5355 },
	{ 0x6ffb, 0x6ffb, PDF_CMAP_SINGLE, 12107 },
	{ 0x6ffc, 0x6ffc, PDF_CMAP_SINGLE, 12109 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 5353 },
	{ 0x6fff, 0x6fff, PDF_CMAP_SINGLE, 12105 },
	{ 0x7000, 0x7000, PDF_CMAP_SINGLE, 12106 },
	{ 0x7001, 0x7001, PDF_CMAP_SINGLE, 12101 },
	{ 0x7003, 0x7003, PDF_CMAP_SINGLE, 18146 },
	{ 0x7004, 0x7004, PDF_CMAP_SINGLE, 11168 },
	{ 0x7005, 0x7005, PDF_CMAP_SINGLE, 12102 },
	{ 0x7006, 0x7006, PDF_CMAP_SINGLE, 5354 },
	{ 0x7007, 0x7007, PDF_CMAP_SINGLE, 12098 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 5351 },
	{ 0x700a, 0x700a, PDF_CMAP_SINGLE, 12111 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 5352 },
	{ 0x700c, 0x700d, PDF_CMAP_RANGE, 12099 },
	{ 0x700e, 0x700e, PDF_CMAP_SINGLE, 12104 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 5357 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5356 },
	{ 0x7014, 0x7014, PDF_CMAP_SINGLE, 12103 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 5519 },
	{ 0x7016, 0x7016, PDF_CMAP_SINGLE, 12461 },
	{ 0x7017, 0x7017, PDF_CMAP_SINGLE, 12467 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 5520 },
	{ 0x7019, 0x7019, PDF_CMAP_SINGLE, 12458 },
	{ 0x701a, 0x701a, PDF_CMAP_SINGLE, 5517 },
	{ 0x701b, 0x701b, PDF_CMAP_SINGLE, 5514 },
	{ 0x701c, 0x701c, PDF_CMAP_SINGLE, 12469 },
	{ 0x701d, 0x701d, PDF_CMAP_SINGLE, 5518 },
	{ 0x701e, 0x701e, PDF_CMAP_SINGLE, 15478 },
	{ 0x701f, 0x701f, PDF_CMAP_SINGLE, 5515 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 12460 },
	{ 0x7021, 0x7023, PDF_CMAP_RANGE, 12463 },
	{ 0x7024, 0x7024, PDF_CMAP_SINGLE, 12468 },
	{ 0x7026, 0x7026, PDF_CMAP_SINGLE, 12108 },
	{ 0x7027, 0x7027, PDF_CMAP_SINGLE, 12459 },
	{ 0x7028, 0x7028, PDF_CMAP_SINGLE, 5516 },
	{ 0x7029, 0x7029, PDF_CMAP_SINGLE, 12466 },
	{ 0x702a, 0x702a, PDF_CMAP_SINGLE, 12783 },
	{ 0x702b, 0x702b, PDF_CMAP_SINGLE, 12462 },
	{ 0x702c, 0x702c, PDF_CMAP_SINGLE, 18062 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 12774 },
	{ 0x7030, 0x7030, PDF_CMAP_SINGLE, 5661 },
	{ 0x7031, 0x7031, PDF_CMAP_SINGLE, 12777 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 5662 },
	{ 0x7033, 0x7033, PDF_CMAP_SINGLE, 12786 },
	{ 0x7034, 0x7034, PDF_CMAP_SINGLE, 12776 },
	{ 0x7035, 0x7035, PDF_CMAP_SINGLE, 12773 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 12775 },
	{ 0x7038, 0x7038, PDF_CMAP_SINGLE, 12779 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 12782 },
	{ 0x703a, 0x703a, PDF_CMAP_SINGLE, 12781 },
	{ 0x703b, 0x703b, PDF_CMAP_SINGLE, 12785 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 12772 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 5660 },
	{ 0x703f, 0x703f, PDF_CMAP_SINGLE, 12780 },
	{ 0x7040, 0x7040, PDF_CMAP_SINGLE, 12784 },
	{ 0x7041, 0x7041, PDF_CMAP_SINGLE, 12787 },
	{ 0x7042, 0x7042, PDF_CMAP_SINGLE, 12778 },
	{ 0x7043, 0x7044, PDF_CMAP_RANGE, 13016 },
	{ 0x7045, 0x7046, PDF_CMAP_RANGE, 13021 },
	{ 0x7048, 0x7049, PDF_CMAP_RANGE, 13019 },
	{ 0x704a, 0x704a, PDF_CMAP_SINGLE, 13018 },
	{ 0x704b, 0x704b, PDF_CMAP_SINGLE, 16980 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 5754 },
	{ 0x704d, 0x704d, PDF_CMAP_SINGLE, 18887 },
	{ 0x7050, 0x7050, PDF_CMAP_SINGLE, 18064 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 5832 },
	{ 0x7052, 0x7052, PDF_CMAP_SINGLE, 13196 },
	{ 0x7054, 0x7054, PDF_CMAP_SINGLE, 18065 },
	{ 0x7055, 0x7057, PDF_CMAP_RANGE, 13193 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 5833 },
	{ 0x705a, 0x705a, PDF_CMAP_SINGLE, 13347 },
	{ 0x705b, 0x705b, PDF_CMAP_SINGLE, 13346 },
	{ 0x705c, 0x705c, PDF_CMAP_SINGLE, 14418 },
	{ 0x705d, 0x705d, PDF_CMAP_SINGLE, 13450 },
	{ 0x705e, 0x705e, PDF_CMAP_SINGLE, 5922 },
	{ 0x705f, 0x705f, PDF_CMAP_SINGLE, 13448 },
	{ 0x7060, 0x7060, PDF_CMAP_SINGLE, 13451 },
	{ 0x7061, 0x7061, PDF_CMAP_SINGLE, 13449 },
	{ 0x7062, 0x7062, PDF_CMAP_SINGLE, 13520 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 5956 },
	{ 0x7064, 0x7064, PDF_CMAP_SINGLE, 5970 },
	{ 0x7065, 0x7065, PDF_CMAP_SINGLE, 13596 },
	{ 0x7066, 0x7066, PDF_CMAP_SINGLE, 13574 },
	{ 0x7067, 0x7067, PDF_CMAP_SINGLE, 15605 },
	{ 0x7068, 0x7068, PDF_CMAP_SINGLE, 13595 },
	{ 0x7069, 0x706a, PDF_CMAP_RANGE, 13639 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 742 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 17649 },
	{ 0x706e, 0x706e, PDF_CMAP_SINGLE, 19154 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 18066 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 1010 },
	{ 0x7071, 0x7071, PDF_CMAP_SINGLE, 6154 },
	{ 0x7074, 0x7074, PDF_CMAP_SINGLE, 6339 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 14535 },
	{ 0x7076, 0x7076, PDF_CMAP_SINGLE, 1254 },
	{ 0x7077, 0x7077, PDF_CMAP_SINGLE, 18856 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 1257 },
	{ 0x7079, 0x7079, PDF_CMAP_SINGLE, 16537 },
	{ 0x707a, 0x707a, PDF_CMAP_SINGLE, 6340 },
	{ 0x707c, 0x707d, PDF_CMAP_RANGE, 1255 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 16981 },
	{ 0x707f, 0x707f, PDF_CMAP_SINGLE, 18067 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 18777 },
	{ 0x7082, 0x7082, PDF_CMAP_SINGLE, 6647 },
	{ 0x7083, 0x7083, PDF_CMAP_SINGLE, 6649 },
	{ 0x7084, 0x7084, PDF_CMAP_SINGLE, 6644 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 6641 },
	{ 0x7086, 0x7086, PDF_CMAP_SINGLE, 6643 },
	{ 0x7089, 0x7089, PDF_CMAP_SINGLE, 17401 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 1603 },
	{ 0x708b, 0x708b, PDF_CMAP_SINGLE, 16587 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 1601 },
	{ 0x708f, 0x708f, PDF_CMAP_SINGLE, 18099 },
	{ 0x7091, 0x7091, PDF_CMAP_SINGLE, 6645 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 1602 },
	{ 0x7093, 0x7093, PDF_CMAP_SINGLE, 6642 },
	{ 0x7094, 0x7094, PDF_CMAP_SINGLE, 6639 },
	{ 0x7095, 0x7095, PDF_CMAP_SINGLE, 1600 },
	{ 0x7096, 0x7096, PDF_CMAP_SINGLE, 6646 },
	{ 0x7098, 0x7098, PDF_CMAP_SINGLE, 6640 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 1604 },
	{ 0x709a, 0x709a, PDF_CMAP_SINGLE, 6648 },
	{ 0x709f, 0x709f, PDF_CMAP_SINGLE, 7014 },
	{ 0x70a0, 0x70a0, PDF_CMAP_SINGLE, 18242 },
	{ 0x70a1, 0x70a1, PDF_CMAP_SINGLE, 7018 },
	{ 0x70a3, 0x70a3, PDF_CMAP_SINGLE, 14706 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 1936 },
	{ 0x70a5, 0x70a5, PDF_CMAP_SINGLE, 15543 },
	{ 0x70a6, 0x70a6, PDF_CMAP_SINGLE, 18128 },
	{ 0x70a7, 0x70a7, PDF_CMAP_SINGLE, 16982 },
	{ 0x70a9, 0x70a9, PDF_CMAP_SINGLE, 7021 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 1928 },
	{ 0x70ac, 0x70ac, PDF_CMAP_SINGLE, 1931 },
	{ 0x70ad, 0x70ad, PDF_CMAP_SINGLE, 1933 },
	{ 0x70ae, 0x70ae, PDF_CMAP_SINGLE, 1935 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 1932 },
	{ 0x70b0, 0x70b0, PDF_CMAP_SINGLE, 7017 },
	{ 0x70b1, 0x70b1, PDF_CMAP_SINGLE, 7016 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 1930 },
	{ 0x70b4, 0x70b5, PDF_CMAP_RANGE, 7019 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 7013 },
	{ 0x70b8, 0x70b8, PDF_CMAP_SINGLE, 1934 },
	{ 0x70b9, 0x70b9, PDF_CMAP_SINGLE, 15721 },
	{ 0x70ba, 0x70ba, PDF_CMAP_SINGLE, 1929 },
	{ 0x70bb, 0x70bb, PDF_CMAP_SINGLE, 18450 },
	{ 0x70bc, 0x70bc, PDF_CMAP_SINGLE, 17753 },
	{ 0x70bd, 0x70bd, PDF_CMAP_SINGLE, 17008 },
	{ 0x70be, 0x70be, PDF_CMAP_SINGLE, 7015 },
	{ 0x70c0, 0x70c0, PDF_CMAP_SINGLE, 18227 },
	{ 0x70c4, 0x70c4, PDF_CMAP_SINGLE, 15274 },
	{ 0x70c5, 0x70c7, PDF_CMAP_RANGE, 7463 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 2337 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 2333 },
	{ 0x70cb, 0x70cb, PDF_CMAP_SINGLE, 7454 },
	{ 0x70cc, 0x70cc, PDF_CMAP_SINGLE, 16984 },
	{ 0x70cd, 0x70cd, PDF_CMAP_SINGLE, 7462 },
	{ 0x70ce, 0x70ce, PDF_CMAP_SINGLE, 7467 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 2338 },
	{ 0x70d0, 0x70d0, PDF_CMAP_SINGLE, 14701 },
	{ 0x70d1, 0x70d1, PDF_CMAP_SINGLE, 7452 },
	{ 0x70d2, 0x70d2, PDF_CMAP_SINGLE, 7458 },
	{ 0x70d3, 0x70d3, PDF_CMAP_SINGLE, 7451 },
	{ 0x70d4, 0x70d4, PDF_CMAP_SINGLE, 7461 },
	{ 0x70d5, 0x70d6, PDF_CMAP_RANGE, 16985 },
	{ 0x70d7, 0x70d7, PDF_CMAP_SINGLE, 7457 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 2334 },
	{ 0x70d9, 0x70d9, PDF_CMAP_SINGLE, 2336 },
	{ 0x70da, 0x70da, PDF_CMAP_SINGLE, 7466 },
	{ 0x70dc, 0x70dc, PDF_CMAP_SINGLE, 7450 },
	{ 0x70dd, 0x70dd, PDF_CMAP_SINGLE, 7453 },
	{ 0x70de, 0x70de, PDF_CMAP_SINGLE, 7459 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 16987 },
	{ 0x70e0, 0x70e0, PDF_CMAP_SINGLE, 7460 },
	{ 0x70e1, 0x70e1, PDF_CMAP_SINGLE, 7468 },
	{ 0x70e2, 0x70e2, PDF_CMAP_SINGLE, 7456 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2335 },
	{ 0x70ef, 0x70ef, PDF_CMAP_SINGLE, 2838 },
	{ 0x70f0, 0x70f0, PDF_CMAP_SINGLE, 8050 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 15048 },
	{ 0x70f3, 0x70f3, PDF_CMAP_SINGLE, 8052 },
	{ 0x70f4, 0x70f4, PDF_CMAP_SINGLE, 8048 },
	{ 0x70f5, 0x70f5, PDF_CMAP_SINGLE, 18272 },
	{ 0x70f6, 0x70f6, PDF_CMAP_SINGLE, 8060 },
	{ 0x70f7, 0x70f7, PDF_CMAP_SINGLE, 8046 },
	{ 0x70f8, 0x70f8, PDF_CMAP_SINGLE, 8059 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 2834 },
	{ 0x70fa, 0x70fa, PDF_CMAP_SINGLE, 8044 },
	{ 0x70fb, 0x70fb, PDF_CMAP_SINGLE, 8690 },
	{ 0x70fc, 0x70fc, PDF_CMAP_SINGLE, 8054 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 2837 },
	{ 0x70fe, 0x70fe, PDF_CMAP_SINGLE, 15630 },
	{ 0x70ff, 0x70ff, PDF_CMAP_SINGLE, 8055 },
	{ 0x7100, 0x7100, PDF_CMAP_SINGLE, 8058 },
	{ 0x7102, 0x7102, PDF_CMAP_SINGLE, 8062 },
	{ 0x7104, 0x7104, PDF_CMAP_SINGLE, 8051 },
	{ 0x7105, 0x7105, PDF_CMAP_SINGLE, 14883 },
	{ 0x7106, 0x7106, PDF_CMAP_SINGLE, 8056 },
	{ 0x7109, 0x710a, PDF_CMAP_RANGE, 2835 },
	{ 0x710b, 0x710b, PDF_CMAP_SINGLE, 8061 },
	{ 0x710c, 0x710c, PDF_CMAP_SINGLE, 8049 },
	{ 0x710d, 0x710d, PDF_CMAP_SINGLE, 8045 },
	{ 0x710e, 0x710e, PDF_CMAP_SINGLE, 8063 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 8053 },
	{ 0x7113, 0x7113, PDF_CMAP_SINGLE, 8057 },
	{ 0x7117, 0x7117, PDF_CMAP_SINGLE, 8047 },
	{ 0x7119, 0x711a, PDF_CMAP_RANGE, 3272 },
	{ 0x711b, 0x711b, PDF_CMAP_SINGLE, 8700 },
	{ 0x711c, 0x711c, PDF_CMAP_SINGLE, 3279 },
	{ 0x711d, 0x711d, PDF_CMAP_SINGLE, 15687 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 8688 },
	{ 0x711f, 0x711f, PDF_CMAP_SINGLE, 8697 },
	{ 0x7120, 0x7120, PDF_CMAP_SINGLE, 8687 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 3276 },
	{ 0x7122, 0x7122, PDF_CMAP_SINGLE, 8695 },
	{ 0x7123, 0x7123, PDF_CMAP_SINGLE, 8693 },
	{ 0x7125, 0x7125, PDF_CMAP_SINGLE, 8694 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 3274 },
	{ 0x7128, 0x7128, PDF_CMAP_SINGLE, 8698 },
	{ 0x7129, 0x7129, PDF_CMAP_SINGLE, 15273 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 16993 },
	{ 0x712c, 0x712c, PDF_CMAP_SINGLE, 17002 },
	{ 0x712e, 0x712e, PDF_CMAP_SINGLE, 8691 },
	{ 0x712f, 0x712f, PDF_CMAP_SINGLE, 8689 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 3275 },
	{ 0x7131, 0x7131, PDF_CMAP_SINGLE, 8692 },
	{ 0x7132, 0x7132, PDF_CMAP_SINGLE, 8696 },
	{ 0x7133, 0x7133, PDF_CMAP_SINGLE, 15700 },
	{ 0x7134, 0x7134, PDF_CMAP_SINGLE, 15486 },
	{ 0x7135, 0x7135, PDF_CMAP_SINGLE, 14189 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 3277 },
	{ 0x713a, 0x713a, PDF_CMAP_SINGLE, 8699 },
	{ 0x713b, 0x713b, PDF_CMAP_SINGLE, 15045 },
	{ 0x713e, 0x713e, PDF_CMAP_SINGLE, 14126 },
	{ 0x7140, 0x7140, PDF_CMAP_SINGLE, 17304 },
	{ 0x7141, 0x7141, PDF_CMAP_SINGLE, 9339 },
	{ 0x7142, 0x7142, PDF_CMAP_SINGLE, 9346 },
	{ 0x7143, 0x7143, PDF_CMAP_SINGLE, 9348 },
	{ 0x7144, 0x7144, PDF_CMAP_SINGLE, 9354 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 16994 },
	{ 0x7146, 0x7146, PDF_CMAP_SINGLE, 3698 },
	{ 0x7147, 0x7147, PDF_CMAP_SINGLE, 9334 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 3690 },
	{ 0x714a, 0x714a, PDF_CMAP_SINGLE, 16996 },
	{ 0x714b, 0x714b, PDF_CMAP_SINGLE, 9349 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 3695 },
	{ 0x714d, 0x714d, PDF_CMAP_SINGLE, 9355 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 3686 },
	{ 0x714f, 0x714f, PDF_CMAP_SINGLE, 16999 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 9352 },
	{ 0x7151, 0x7151, PDF_CMAP_SINGLE, 18533 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 9336 },
	{ 0x7153, 0x7153, PDF_CMAP_SINGLE, 9353 },
	{ 0x7154, 0x7154, PDF_CMAP_SINGLE, 9335 },
	{ 0x7156, 0x7156, PDF_CMAP_SINGLE, 3700 },
	{ 0x7157, 0x7157, PDF_CMAP_SINGLE, 19040 },
	{ 0x7158, 0x7158, PDF_CMAP_SINGLE, 9347 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 3687 },
	{ 0x715a, 0x715a, PDF_CMAP_SINGLE, 9356 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 3692 },
	{ 0x715d, 0x715d, PDF_CMAP_SINGLE, 9340 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 3697 },
	{ 0x715f, 0x715f, PDF_CMAP_SINGLE, 9351 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 9338 },
	{ 0x7161, 0x7161, PDF_CMAP_SINGLE, 9345 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 9341 },
	{ 0x7163, 0x7163, PDF_CMAP_SINGLE, 9337 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 3689 },
	{ 0x7165, 0x7165, PDF_CMAP_SINGLE, 3696 },
	{ 0x7166, 0x7166, PDF_CMAP_SINGLE, 3694 },
	{ 0x7167, 0x7167, PDF_CMAP_SINGLE, 3691 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 3699 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 3688 },
	{ 0x716a, 0x716a, PDF_CMAP_SINGLE, 9344 },
	{ 0x716b, 0x716b, PDF_CMAP_SINGLE, 15557 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 3693 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 3278 },
	{ 0x7170, 0x7170, PDF_CMAP_SINGLE, 9350 },
	{ 0x7171, 0x7171, PDF_CMAP_SINGLE, 15808 },
	{ 0x7172, 0x7172, PDF_CMAP_SINGLE, 9342 },
	{ 0x7173, 0x7173, PDF_CMAP_SINGLE, 18407 },
	{ 0x7175, 0x7175, PDF_CMAP_SINGLE, 15597 },
	{ 0x7176, 0x7176, PDF_CMAP_SINGLE, 16384 },
	{ 0x7177, 0x7177, PDF_CMAP_SINGLE, 14679 },
	{ 0x7178, 0x7178, PDF_CMAP_SINGLE, 9343 },
	{ 0x717a, 0x717a, PDF_CMAP_SINGLE, 18724 },
	{ 0x717b, 0x717b, PDF_CMAP_SINGLE, 10014 },
	{ 0x717c, 0x717c, PDF_CMAP_SINGLE, 14389 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 4131 },
	{ 0x717e, 0x717e, PDF_CMAP_SINGLE, 14704 },
	{ 0x7180, 0x7180, PDF_CMAP_SINGLE, 10010 },
	{ 0x7181, 0x7181, PDF_CMAP_SINGLE, 10016 },
	{ 0x7182, 0x7182, PDF_CMAP_SINGLE, 10012 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 4133 },
	{ 0x7185, 0x7185, PDF_CMAP_SINGLE, 10011 },
	{ 0x7186, 0x7186, PDF_CMAP_SINGLE, 10015 },
	{ 0x7187, 0x7187, PDF_CMAP_SINGLE, 10007 },
	{ 0x7188, 0x7188, PDF_CMAP_SINGLE, 18182 },
	{ 0x7189, 0x7189, PDF_CMAP_SINGLE, 10009 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 4132 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 14826 },
	{ 0x718e, 0x718e, PDF_CMAP_SINGLE, 15675 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 10013 },
	{ 0x7190, 0x7190, PDF_CMAP_SINGLE, 10008 },
	{ 0x7191, 0x7191, PDF_CMAP_SINGLE, 18866 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 4134 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 4129 },
	{ 0x7196, 0x7196, PDF_CMAP_SINGLE, 16991 },
	{ 0x7197, 0x7197, PDF_CMAP_SINGLE, 10017 },
	{ 0x7198, 0x7198, PDF_CMAP_SINGLE, 18461 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 4130 },
	{ 0x719a, 0x719a, PDF_CMAP_SINGLE, 10586 },
	{ 0x719b, 0x719b, PDF_CMAP_SINGLE, 10583 },
	{ 0x719c, 0x719c, PDF_CMAP_SINGLE, 10595 },
	{ 0x719d, 0x719d, PDF_CMAP_SINGLE, 10589 },
	{ 0x719e, 0x719e, PDF_CMAP_SINGLE, 10591 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 4510 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 10585 },
	{ 0x71a1, 0x71a1, PDF_CMAP_SINGLE, 10593 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 18625 },
	{ 0x71a3, 0x71a3, PDF_CMAP_SINGLE, 16072 },
	{ 0x71a4, 0x71a4, PDF_CMAP_SINGLE, 10592 },
	{ 0x71a5, 0x71a5, PDF_CMAP_SINGLE, 10590 },
	{ 0x71a7, 0x71a7, PDF_CMAP_SINGLE, 10596 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 4513 },
	{ 0x71a9, 0x71a9, PDF_CMAP_SINGLE, 10587 },
	{ 0x71aa, 0x71aa, PDF_CMAP_SINGLE, 10594 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 4511 },
	{ 0x71ad, 0x71ad, PDF_CMAP_SINGLE, 15901 },
	{ 0x71af, 0x71af, PDF_CMAP_SINGLE, 10582 },
	{ 0x71b0, 0x71b0, PDF_CMAP_SINGLE, 10584 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 4512 },
	{ 0x71b2, 0x71b2, PDF_CMAP_SINGLE, 10581 },
	{ 0x71b3, 0x71b3, PDF_CMAP_SINGLE, 10597 },
	{ 0x71b4, 0x71b4, PDF_CMAP_SINGLE, 16990 },
	{ 0x71b5, 0x71b5, PDF_CMAP_SINGLE, 10588 },
	{ 0x71b7, 0x71b7, PDF_CMAP_SINGLE, 18059 },
	{ 0x71b8, 0x71b8, PDF_CMAP_SINGLE, 11187 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 4851 },
	{ 0x71ba, 0x71ba, PDF_CMAP_SINGLE, 17006 },
	{ 0x71bc, 0x71bc, PDF_CMAP_SINGLE, 11198 },
	{ 0x71bd, 0x71bd, PDF_CMAP_SINGLE, 11196 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 4845 },
	{ 0x71bf, 0x71bf, PDF_CMAP_SINGLE, 11186 },
	{ 0x71c0, 0x71c1, PDF_CMAP_RANGE, 11189 },
	{ 0x71c2, 0x71c2, PDF_CMAP_SINGLE, 11185 },
	{ 0x71c3, 0x71c4, PDF_CMAP_RANGE, 4855 },
	{ 0x71c5, 0x71c5, PDF_CMAP_SINGLE, 11184 },
	{ 0x71c6, 0x71c6, PDF_CMAP_SINGLE, 11199 },
	{ 0x71c7, 0x71c7, PDF_CMAP_SINGLE, 11194 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 4849 },
	{ 0x71c9, 0x71c9, PDF_CMAP_SINGLE, 4846 },
	{ 0x71ca, 0x71ca, PDF_CMAP_SINGLE, 11193 },
	{ 0x71cb, 0x71cb, PDF_CMAP_SINGLE, 11191 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 4852 },
	{ 0x71cf, 0x71cf, PDF_CMAP_SINGLE, 11195 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 4847 },
	{ 0x71d1, 0x71d1, PDF_CMAP_SINGLE, 14141 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 4848 },
	{ 0x71d4, 0x71d4, PDF_CMAP_SINGLE, 11192 },
	{ 0x71d5, 0x71d5, PDF_CMAP_SINGLE, 4850 },
	{ 0x71d6, 0x71d6, PDF_CMAP_SINGLE, 11188 },
	{ 0x71d8, 0x71d8, PDF_CMAP_SINGLE, 11197 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 4853 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 11200 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 4854 },
	{ 0x71dd, 0x71dd, PDF_CMAP_SINGLE, 16504 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 5124 },
	{ 0x71e0, 0x71e0, PDF_CMAP_SINGLE, 5131 },
	{ 0x71e1, 0x71e1, PDF_CMAP_SINGLE, 11675 },
	{ 0x71e2, 0x71e2, PDF_CMAP_SINGLE, 11681 },
	{ 0x71e4, 0x71e4, PDF_CMAP_SINGLE, 11679 },
	{ 0x71e5, 0x71e5, PDF_CMAP_SINGLE, 5127 },
	{ 0x71e6, 0x71e6, PDF_CMAP_SINGLE, 5126 },
	{ 0x71e7, 0x71e7, PDF_CMAP_SINGLE, 5123 },
	{ 0x71e8, 0x71e8, PDF_CMAP_SINGLE, 11677 },
	{ 0x71eb, 0x71eb, PDF_CMAP_SINGLE, 15489 },
	{ 0x71ec, 0x71ec, PDF_CMAP_SINGLE, 5129 },
	{ 0x71ed, 0x71ed, PDF_CMAP_SINGLE, 5128 },
	{ 0x71ee, 0x71ee, PDF_CMAP_SINGLE, 5125 },
	{ 0x71f0, 0x71f0, PDF_CMAP_SINGLE, 11680 },
	{ 0x71f1, 0x71f1, PDF_CMAP_SINGLE, 11676 },
	{ 0x71f2, 0x71f2, PDF_CMAP_SINGLE, 11678 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 5130 },
	{ 0x71f5, 0x71f5, PDF_CMAP_SINGLE, 18158 },
	{ 0x71f6, 0x71f6, PDF_CMAP_SINGLE, 16026 },
	{ 0x71f8, 0x71f8, PDF_CMAP_SINGLE, 5361 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 12114 },
	{ 0x71fb, 0x71fc, PDF_CMAP_RANGE, 5358 },
	{ 0x71fd, 0x71fd, PDF_CMAP_SINGLE, 12116 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 5360 },
	{ 0x71ff, 0x71ff, PDF_CMAP_SINGLE, 12113 },
	{ 0x7200, 0x7200, PDF_CMAP_SINGLE, 18916 },
	{ 0x7201, 0x7201, PDF_CMAP_SINGLE, 12112 },
	{ 0x7202, 0x7202, PDF_CMAP_SINGLE, 12473 },
	{ 0x7203, 0x7203, PDF_CMAP_SINGLE, 12115 },
	{ 0x7205, 0x7205, PDF_CMAP_SINGLE, 12474 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 5521 },
	{ 0x7207, 0x7207, PDF_CMAP_SINGLE, 12472 },
	{ 0x7209, 0x7209, PDF_CMAP_SINGLE, 15803 },
	{ 0x720a, 0x720a, PDF_CMAP_SINGLE, 12471 },
	{ 0x720c, 0x720c, PDF_CMAP_SINGLE, 12470 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 5522 },
	{ 0x720e, 0x720e, PDF_CMAP_SINGLE, 17009 },
	{ 0x720f, 0x720f, PDF_CMAP_SINGLE, 14844 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 5663 },
	{ 0x7213, 0x7214, PDF_CMAP_RANGE, 12788 },
	{ 0x7215, 0x7215, PDF_CMAP_SINGLE, 17011 },
	{ 0x7216, 0x7216, PDF_CMAP_SINGLE, 14675 },
	{ 0x7217, 0x7217, PDF_CMAP_SINGLE, 14697 },
	{ 0x7219, 0x7219, PDF_CMAP_SINGLE, 13025 },
	{ 0x721a, 0x721a, PDF_CMAP_SINGLE, 13024 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 5755 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 13023 },
	{ 0x721e, 0x721f, PDF_CMAP_RANGE, 13197 },
	{ 0x7222, 0x7222, PDF_CMAP_SINGLE, 13348 },
	{ 0x7223, 0x7223, PDF_CMAP_SINGLE, 13452 },
	{ 0x7224, 0x7224, PDF_CMAP_SINGLE, 17014 },
	{ 0x7225, 0x7225, PDF_CMAP_SINGLE, 18996 },
	{ 0x7226, 0x7226, PDF_CMAP_SINGLE, 13521 },
	{ 0x7227, 0x7227, PDF_CMAP_SINGLE, 13621 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 5990 },
	{ 0x7229, 0x7229, PDF_CMAP_SINGLE, 13641 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 743 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 17650 },
	{ 0x722c, 0x722d, PDF_CMAP_RANGE, 1605 },
	{ 0x722e, 0x722e, PDF_CMAP_SINGLE, 17017 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 1937 },
	{ 0x7235, 0x7235, PDF_CMAP_SINGLE, 5132 },
	{ 0x7236, 0x7236, PDF_CMAP_SINGLE, 744 },
	{ 0x7238, 0x7238, PDF_CMAP_SINGLE, 1607 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 2339 },
	{ 0x723a, 0x723a, PDF_CMAP_SINGLE, 3701 },
	{ 0x723b, 0x723b, PDF_CMAP_SINGLE, 745 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 2839 },
	{ 0x723e, 0x723e, PDF_CMAP_SINGLE, 4135 },
	{ 0x723f, 0x723f, PDF_CMAP_SINGLE, 6029 },
	{ 0x7240, 0x7240, PDF_CMAP_SINGLE, 17018 },
	{ 0x7241, 0x7241, PDF_CMAP_SINGLE, 7022 },
	{ 0x7242, 0x7242, PDF_CMAP_SINGLE, 7469 },
	{ 0x7244, 0x7244, PDF_CMAP_SINGLE, 10018 },
	{ 0x7246, 0x7246, PDF_CMAP_SINGLE, 5133 },
	{ 0x7247, 0x7247, PDF_CMAP_SINGLE, 746 },
	{ 0x7248, 0x7248, PDF_CMAP_SINGLE, 1608 },
	{ 0x7249, 0x724a, PDF_CMAP_RANGE, 7023 },
	{ 0x724b, 0x724b, PDF_CMAP_SINGLE, 8701 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 3280 },
	{ 0x724f, 0x724f, PDF_CMAP_SINGLE, 9357 },
	{ 0x7250, 0x7250, PDF_CMAP_SINGLE, 15049 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 3702 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 10019 },
	{ 0x7255, 0x7255, PDF_CMAP_SINGLE, 17021 },
	{ 0x7256, 0x7256, PDF_CMAP_SINGLE, 4514 },
	{ 0x7257, 0x7257, PDF_CMAP_SINGLE, 17022 },
	{ 0x7258, 0x7258, PDF_CMAP_SINGLE, 5523 },
	{ 0x7259, 0x7259, PDF_CMAP_SINGLE, 747 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 8702 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 748 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 18010 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 1012 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 6155 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 1011 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 1260 },
	{ 0x7261, 0x7261, PDF_CMAP_SINGLE, 1259 },
	{ 0x7262, 0x7262, PDF_CMAP_SINGLE, 1258 },
	{ 0x7263, 0x7263, PDF_CMAP_SINGLE, 6341 },
	{ 0x7266, 0x7266, PDF_CMAP_SINGLE, 16059 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 1609 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 1610 },
	{ 0x726a, 0x726a, PDF_CMAP_SINGLE, 6650 },
	{ 0x726c, 0x726c, PDF_CMAP_SINGLE, 7025 },
	{ 0x726e, 0x726e, PDF_CMAP_SINGLE, 7028 },
	{ 0x726f, 0x726f, PDF_CMAP_SINGLE, 1939 },
	{ 0x7270, 0x7270, PDF_CMAP_SINGLE, 7026 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 1938 },
	{ 0x7273, 0x7273, PDF_CMAP_SINGLE, 7027 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1940 },
	{ 0x7276, 0x7276, PDF_CMAP_SINGLE, 7472 },
	{ 0x7277, 0x7277, PDF_CMAP_SINGLE, 7471 },
	{ 0x7278, 0x7278, PDF_CMAP_SINGLE, 7470 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 2340 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 8065 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 2840 },
	{ 0x727e, 0x727e, PDF_CMAP_SINGLE, 8064 },
	{ 0x727f, 0x727f, PDF_CMAP_SINGLE, 8067 },
	{ 0x7280, 0x7280, PDF_CMAP_SINGLE, 3282 },
	{ 0x7281, 0x7281, PDF_CMAP_SINGLE, 2841 },
	{ 0x7282, 0x7282, PDF_CMAP_SINGLE, 17027 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 3281 },
	{ 0x7285, 0x7285, PDF_CMAP_SINGLE, 8706 },
	{ 0x7286, 0x7286, PDF_CMAP_SINGLE, 8705 },
	{ 0x7287, 0x7287, PDF_CMAP_SINGLE, 15050 },
	{ 0x7288, 0x7289, PDF_CMAP_RANGE, 8703 },
	{ 0x728b, 0x728b, PDF_CMAP_SINGLE, 8707 },
	{ 0x728c, 0x728c, PDF_CMAP_SINGLE, 9359 },
	{ 0x728d, 0x728d, PDF_CMAP_SINGLE, 9358 },
	{ 0x728e, 0x728e, PDF_CMAP_SINGLE, 9362 },
	{ 0x728f, 0x728f, PDF_CMAP_SINGLE, 17353 },
	{ 0x7290, 0x7290, PDF_CMAP_SINGLE, 9361 },
	{ 0x7291, 0x7291, PDF_CMAP_SINGLE, 9360 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 4136 },
	{ 0x7293, 0x7293, PDF_CMAP_SINGLE, 10022 },
	{ 0x7294, 0x7294, PDF_CMAP_SINGLE, 15051 },
	{ 0x7295, 0x7295, PDF_CMAP_SINGLE, 10021 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 4137 },
	{ 0x7297, 0x7297, PDF_CMAP_SINGLE, 10020 },
	{ 0x7298, 0x7298, PDF_CMAP_SINGLE, 10598 },
	{ 0x729a, 0x729a, PDF_CMAP_SINGLE, 10599 },
	{ 0x729b, 0x729b, PDF_CMAP_SINGLE, 4515 },
	{ 0x729d, 0x729e, PDF_CMAP_RANGE, 11202 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 18234 },
	{ 0x72a1, 0x72a1, PDF_CMAP_SINGLE, 12479 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 5524 },
	{ 0x72a3, 0x72a3, PDF_CMAP_SINGLE, 12478 },
	{ 0x72a4, 0x72a4, PDF_CMAP_SINGLE, 12477 },
	{ 0x72a5, 0x72a6, PDF_CMAP_RANGE, 12475 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 5756 },
	{ 0x72a8, 0x72a8, PDF_CMAP_SINGLE, 12790 },
	{ 0x72a9, 0x72a9, PDF_CMAP_SINGLE, 13199 },
	{ 0x72aa, 0x72aa, PDF_CMAP_SINGLE, 13522 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 749 },
	{ 0x72ad, 0x72ad, PDF_CMAP_SINGLE, 17652 },
	{ 0x72ae, 0x72ae, PDF_CMAP_SINGLE, 6063 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 853 },
	{ 0x72b0, 0x72b0, PDF_CMAP_SINGLE, 6064 },
	{ 0x72b2, 0x72b2, PDF_CMAP_SINGLE, 16310 },
	{ 0x72b4, 0x72b5, PDF_CMAP_RANGE, 6156 },
	{ 0x72ba, 0x72ba, PDF_CMAP_SINGLE, 6347 },
	{ 0x72bd, 0x72bd, PDF_CMAP_SINGLE, 6343 },
	{ 0x72bf, 0x72bf, PDF_CMAP_SINGLE, 6342 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 1611 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 6346 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 1262 },
	{ 0x72c3, 0x72c3, PDF_CMAP_SINGLE, 6344 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 1261 },
	{ 0x72c5, 0x72c5, PDF_CMAP_SINGLE, 6348 },
	{ 0x72c6, 0x72c6, PDF_CMAP_SINGLE, 6345 },
	{ 0x72c9, 0x72c9, PDF_CMAP_SINGLE, 6654 },
	{ 0x72ca, 0x72ca, PDF_CMAP_SINGLE, 7029 },
	{ 0x72cb, 0x72cb, PDF_CMAP_SINGLE, 6652 },
	{ 0x72cc, 0x72cc, PDF_CMAP_SINGLE, 6659 },
	{ 0x72cd, 0x72cd, PDF_CMAP_SINGLE, 17354 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 1612 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1615 },
	{ 0x72d1, 0x72d1, PDF_CMAP_SINGLE, 6660 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 6656 },
	{ 0x72d4, 0x72d4, PDF_CMAP_SINGLE, 6657 },
	{ 0x72d6, 0x72d6, PDF_CMAP_SINGLE, 6651 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 1614 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 6653 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 1613 },
	{ 0x72da, 0x72da, PDF_CMAP_SINGLE, 6658 },
	{ 0x72dc, 0x72dc, PDF_CMAP_SINGLE, 6655 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 7033 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 1942 },
	{ 0x72e2, 0x72e2, PDF_CMAP_SINGLE, 16279 },
	{ 0x72e3, 0x72e3, PDF_CMAP_SINGLE, 7036 },
	{ 0x72e4, 0x72e4, PDF_CMAP_SINGLE, 7030 },
	{ 0x72e6, 0x72e6, PDF_CMAP_SINGLE, 7035 },
	{ 0x72e8, 0x72e8, PDF_CMAP_SINGLE, 7031 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 1941 },
	{ 0x72ea, 0x72ea, PDF_CMAP_SINGLE, 7034 },
	{ 0x72eb, 0x72eb, PDF_CMAP_SINGLE, 7032 },
	{ 0x72f3, 0x72f3, PDF_CMAP_SINGLE, 7478 },
	{ 0x72f4, 0x72f4, PDF_CMAP_SINGLE, 7475 },
	{ 0x72f6, 0x72f6, PDF_CMAP_SINGLE, 7477 },
	{ 0x72f7, 0x72f7, PDF_CMAP_SINGLE, 2345 },
	{ 0x72f8, 0x72f8, PDF_CMAP_SINGLE, 2344 },
	{ 0x72f9, 0x72f9, PDF_CMAP_SINGLE, 2342 },
	{ 0x72fa, 0x72fa, PDF_CMAP_SINGLE, 7474 },
	{ 0x72fb, 0x72fb, PDF_CMAP_SINGLE, 7479 },
	{ 0x72fc, 0x72fc, PDF_CMAP_SINGLE, 2341 },
	{ 0x72fd, 0x72fd, PDF_CMAP_SINGLE, 2343 },
	{ 0x72fe, 0x72fe, PDF_CMAP_SINGLE, 7476 },
	{ 0x72ff, 0x72ff, PDF_CMAP_SINGLE, 8075 },
	{ 0x7300, 0x7300, PDF_CMAP_SINGLE, 7473 },
	{ 0x7301, 0x7301, PDF_CMAP_SINGLE, 7480 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 16282 },
	{ 0x7304, 0x7304, PDF_CMAP_SINGLE, 16053 },
	{ 0x7307, 0x7307, PDF_CMAP_SINGLE, 8070 },
	{ 0x7308, 0x7308, PDF_CMAP_SINGLE, 8074 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 8073 },
	{ 0x730b, 0x730b, PDF_CMAP_SINGLE, 8709 },
	{ 0x730c, 0x730c, PDF_CMAP_SINGLE, 8720 },
	{ 0x730f, 0x730f, PDF_CMAP_SINGLE, 8076 },
	{ 0x7310, 0x7310, PDF_CMAP_SINGLE, 18154 },
	{ 0x7311, 0x7311, PDF_CMAP_SINGLE, 8071 },
	{ 0x7312, 0x7312, PDF_CMAP_SINGLE, 8708 },
	{ 0x7313, 0x7313, PDF_CMAP_SINGLE, 2845 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 2844 },
	{ 0x7317, 0x7317, PDF_CMAP_SINGLE, 8069 },
	{ 0x7318, 0x7318, PDF_CMAP_SINGLE, 8072 },
	{ 0x7319, 0x7319, PDF_CMAP_SINGLE, 2846 },
	{ 0x731b, 0x731b, PDF_CMAP_SINGLE, 2843 },
	{ 0x731c, 0x731c, PDF_CMAP_SINGLE, 2842 },
	{ 0x731d, 0x731d, PDF_CMAP_SINGLE, 8068 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 8077 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 8711 },
	{ 0x7323, 0x7323, PDF_CMAP_SINGLE, 8718 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 3284 },
	{ 0x7326, 0x7326, PDF_CMAP_SINGLE, 8717 },
	{ 0x7327, 0x7327, PDF_CMAP_SINGLE, 8714 },
	{ 0x7328, 0x7328, PDF_CMAP_SINGLE, 17034 },
	{ 0x7329, 0x7329, PDF_CMAP_SINGLE, 3286 },
	{ 0x732a, 0x732a, PDF_CMAP_SINGLE, 15728 },
	{ 0x732b, 0x732b, PDF_CMAP_SINGLE, 17029 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 16306 },
	{ 0x732d, 0x732d, PDF_CMAP_SINGLE, 8716 },
	{ 0x732e, 0x732e, PDF_CMAP_SINGLE, 17035 },
	{ 0x7330, 0x7330, PDF_CMAP_SINGLE, 8710 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 8712 },
	{ 0x7332, 0x7332, PDF_CMAP_SINGLE, 8715 },
	{ 0x7333, 0x7333, PDF_CMAP_SINGLE, 8713 },
	{ 0x7334, 0x7334, PDF_CMAP_SINGLE, 3285 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 8719 },
	{ 0x7336, 0x7336, PDF_CMAP_SINGLE, 3283 },
	{ 0x7337, 0x7337, PDF_CMAP_SINGLE, 3703 },
	{ 0x7338, 0x7338, PDF_CMAP_SINGLE, 18155 },
	{ 0x7339, 0x7339, PDF_CMAP_SINGLE, 17356 },
	{ 0x733a, 0x733a, PDF_CMAP_SINGLE, 9366 },
	{ 0x733b, 0x733b, PDF_CMAP_SINGLE, 9365 },
	{ 0x733c, 0x733c, PDF_CMAP_SINGLE, 9363 },
	{ 0x733e, 0x733e, PDF_CMAP_SINGLE, 3706 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 3705 },
	{ 0x7340, 0x7340, PDF_CMAP_SINGLE, 9367 },
	{ 0x7341, 0x7341, PDF_CMAP_SINGLE, 18156 },
	{ 0x7342, 0x7342, PDF_CMAP_SINGLE, 9364 },
	{ 0x7343, 0x7343, PDF_CMAP_SINGLE, 10023 },
	{ 0x7344, 0x7344, PDF_CMAP_SINGLE, 4138 },
	{ 0x7345, 0x7345, PDF_CMAP_SINGLE, 3704 },
	{ 0x7348, 0x7348, PDF_CMAP_SINGLE, 17402 },
	{ 0x7349, 0x7349, PDF_CMAP_SINGLE, 9369 },
	{ 0x734a, 0x734a, PDF_CMAP_SINGLE, 9368 },
	{ 0x734c, 0x734c, PDF_CMAP_SINGLE, 10026 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 10024 },
	{ 0x734e, 0x734e, PDF_CMAP_SINGLE, 4516 },
	{ 0x734f, 0x734f, PDF_CMAP_SINGLE, 16209 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 4139 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 10025 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 10601 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 4517 },
	{ 0x7358, 0x7358, PDF_CMAP_SINGLE, 10600 },
	{ 0x7359, 0x7359, PDF_CMAP_SINGLE, 10609 },
	{ 0x735a, 0x735a, PDF_CMAP_SINGLE, 10608 },
	{ 0x735b, 0x735b, PDF_CMAP_SINGLE, 10606 },
	{ 0x735d, 0x735d, PDF_CMAP_SINGLE, 10605 },
	{ 0x735e, 0x7360, PDF_CMAP_RANGE, 10602 },
	{ 0x7361, 0x7361, PDF_CMAP_SINGLE, 10607 },
	{ 0x7362, 0x7362, PDF_CMAP_SINGLE, 10610 },
	{ 0x7365, 0x7365, PDF_CMAP_SINGLE, 11208 },
	{ 0x7366, 0x7367, PDF_CMAP_RANGE, 11205 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 4857 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 11204 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 11210 },
	{ 0x736b, 0x736b, PDF_CMAP_SINGLE, 11209 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 11207 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 11683 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 5134 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 16280 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 5135 },
	{ 0x7373, 0x7373, PDF_CMAP_SINGLE, 11682 },
	{ 0x7374, 0x7374, PDF_CMAP_SINGLE, 16084 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 5363 },
	{ 0x7376, 0x7376, PDF_CMAP_SINGLE, 12117 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 5362 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 5525 },
	{ 0x737a, 0x737a, PDF_CMAP_SINGLE, 5526 },
	{ 0x737b, 0x737b, PDF_CMAP_SINGLE, 5664 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 12792 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 12791 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 13026 },
	{ 0x737f, 0x737f, PDF_CMAP_SINGLE, 13200 },
	{ 0x7380, 0x7380, PDF_CMAP_SINGLE, 5834 },
	{ 0x7381, 0x7381, PDF_CMAP_SINGLE, 13350 },
	{ 0x7382, 0x7382, PDF_CMAP_SINGLE, 13349 },
	{ 0x7383, 0x7383, PDF_CMAP_SINGLE, 13351 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 854 },
	{ 0x7385, 0x7385, PDF_CMAP_SINGLE, 7037 },
	{ 0x7386, 0x7386, PDF_CMAP_SINGLE, 2346 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 2847 },
	{ 0x7388, 0x7388, PDF_CMAP_SINGLE, 8078 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 855 },
	{ 0x738a, 0x738a, PDF_CMAP_SINGLE, 6065 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 750 },
	{ 0x738c, 0x738c, PDF_CMAP_SINGLE, 18918 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6158 },
	{ 0x738f, 0x738f, PDF_CMAP_SINGLE, 18149 },
	{ 0x7392, 0x7392, PDF_CMAP_SINGLE, 6353 },
	{ 0x7393, 0x7394, PDF_CMAP_RANGE, 6351 },
	{ 0x7395, 0x7395, PDF_CMAP_SINGLE, 6349 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 1263 },
	{ 0x7397, 0x7397, PDF_CMAP_SINGLE, 6350 },
	{ 0x7398, 0x7398, PDF_CMAP_SINGLE, 14838 },
	{ 0x739c, 0x739c, PDF_CMAP_SINGLE, 16522 },
	{ 0x739d, 0x739d, PDF_CMAP_SINGLE, 6668 },
	{ 0x739e, 0x739e, PDF_CMAP_SINGLE, 14683 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 1618 },
	{ 0x73a0, 0x73a0, PDF_CMAP_SINGLE, 6666 },
	{ 0x73a1, 0x73a1, PDF_CMAP_SINGLE, 6662 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 6665 },
	{ 0x73a4, 0x73a4, PDF_CMAP_SINGLE, 6661 },
	{ 0x73a5, 0x73a5, PDF_CMAP_SINGLE, 1620 },
	{ 0x73a6, 0x73a6, PDF_CMAP_SINGLE, 6664 },
	{ 0x73a7, 0x73a7, PDF_CMAP_SINGLE, 16519 },
	{ 0x73a8, 0x73a8, PDF_CMAP_SINGLE, 1617 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 1616 },
	{ 0x73aa, 0x73aa, PDF_CMAP_SINGLE, 17037 },
	{ 0x73ab, 0x73ab, PDF_CMAP_SINGLE, 1619 },
	{ 0x73ac, 0x73ac, PDF_CMAP_SINGLE, 6667 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 6663 },
	{ 0x73b2, 0x73b2, PDF_CMAP_SINGLE, 1947 },
	{ 0x73b3, 0x73b3, PDF_CMAP_SINGLE, 1950 },
	{ 0x73b4, 0x73b4, PDF_CMAP_SINGLE, 7045 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 7044 },
	{ 0x73b6, 0x73b6, PDF_CMAP_SINGLE, 7043 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 1944 },
	{ 0x73b8, 0x73b8, PDF_CMAP_SINGLE, 7052 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 7042 },
	{ 0x73ba, 0x73ba, PDF_CMAP_SINGLE, 18868 },
	{ 0x73bb, 0x73bb, PDF_CMAP_SINGLE, 1946 },
	{ 0x73bc, 0x73bc, PDF_CMAP_SINGLE, 7485 },
	{ 0x73be, 0x73be, PDF_CMAP_SINGLE, 7049 },
	{ 0x73bf, 0x73bf, PDF_CMAP_SINGLE, 7047 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 1949 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 7039 },
	{ 0x73c3, 0x73c3, PDF_CMAP_SINGLE, 7050 },
	{ 0x73c4, 0x73c4, PDF_CMAP_SINGLE, 19123 },
	{ 0x73c5, 0x73c5, PDF_CMAP_SINGLE, 7041 },
	{ 0x73c6, 0x73c6, PDF_CMAP_SINGLE, 7051 },
	{ 0x73c7, 0x73c7, PDF_CMAP_SINGLE, 7048 },
	{ 0x73c8, 0x73c8, PDF_CMAP_SINGLE, 7040 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 17040 },
	{ 0x73ca, 0x73ca, PDF_CMAP_SINGLE, 1945 },
	{ 0x73cb, 0x73cb, PDF_CMAP_SINGLE, 7053 },
	{ 0x73cc, 0x73cc, PDF_CMAP_SINGLE, 7038 },
	{ 0x73cd, 0x73cd, PDF_CMAP_SINGLE, 1948 },
	{ 0x73ce, 0x73ce, PDF_CMAP_SINGLE, 16667 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 17036 },
	{ 0x73d0, 0x73d0, PDF_CMAP_SINGLE, 15127 },
	{ 0x73d2, 0x73d2, PDF_CMAP_SINGLE, 7490 },
	{ 0x73d3, 0x73d3, PDF_CMAP_SINGLE, 7481 },
	{ 0x73d4, 0x73d4, PDF_CMAP_SINGLE, 7492 },
	{ 0x73d5, 0x73d5, PDF_CMAP_SINGLE, 16499 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 7484 },
	{ 0x73d7, 0x73d8, PDF_CMAP_RANGE, 7495 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 7482 },
	{ 0x73da, 0x73da, PDF_CMAP_SINGLE, 7494 },
	{ 0x73db, 0x73db, PDF_CMAP_SINGLE, 7491 },
	{ 0x73dc, 0x73dc, PDF_CMAP_SINGLE, 7489 },
	{ 0x73dd, 0x73dd, PDF_CMAP_SINGLE, 7493 },
	{ 0x73de, 0x73de, PDF_CMAP_SINGLE, 2352 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 2350 },
	{ 0x73e1, 0x73e1, PDF_CMAP_SINGLE, 18160 },
	{ 0x73e2, 0x73e2, PDF_CMAP_SINGLE, 15523 },
	{ 0x73e3, 0x73e3, PDF_CMAP_SINGLE, 7487 },
	{ 0x73e4, 0x73e4, PDF_CMAP_SINGLE, 17061 },
	{ 0x73e5, 0x73e5, PDF_CMAP_SINGLE, 7483 },
	{ 0x73e6, 0x73e6, PDF_CMAP_SINGLE, 15546 },
	{ 0x73e7, 0x73e7, PDF_CMAP_SINGLE, 7486 },
	{ 0x73e8, 0x73e8, PDF_CMAP_SINGLE, 7497 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 7488 },
	{ 0x73ea, 0x73ea, PDF_CMAP_SINGLE, 2351 },
	{ 0x73eb, 0x73eb, PDF_CMAP_SINGLE, 7046 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 2347 },
	{ 0x73ee, 0x73ee, PDF_CMAP_SINGLE, 2349 },
	{ 0x73ef, 0x73ef, PDF_CMAP_SINGLE, 16401 },
	{ 0x73f3, 0x73f3, PDF_CMAP_SINGLE, 15531 },
	{ 0x73f4, 0x73f4, PDF_CMAP_SINGLE, 8092 },
	{ 0x73f5, 0x73f5, PDF_CMAP_SINGLE, 8081 },
	{ 0x73f6, 0x73f6, PDF_CMAP_SINGLE, 8079 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 17050 },
	{ 0x73f8, 0x73f8, PDF_CMAP_SINGLE, 8080 },
	{ 0x73f9, 0x73f9, PDF_CMAP_SINGLE, 15353 },
	{ 0x73fa, 0x73fa, PDF_CMAP_SINGLE, 8087 },
	{ 0x73fb, 0x73fb, PDF_CMAP_SINGLE, 15532 },
	{ 0x73fc, 0x73fc, PDF_CMAP_SINGLE, 8088 },
	{ 0x73fd, 0x73fd, PDF_CMAP_SINGLE, 8084 },
	{ 0x73fe, 0x73fe, PDF_CMAP_SINGLE, 2852 },
	{ 0x73ff, 0x73ff, PDF_CMAP_SINGLE, 8089 },
	{ 0x7400, 0x7400, PDF_CMAP_SINGLE, 8086 },
	{ 0x7401, 0x7401, PDF_CMAP_SINGLE, 8083 },
	{ 0x7402, 0x7402, PDF_CMAP_SINGLE, 18875 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 2850 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 8082 },
	{ 0x7405, 0x7405, PDF_CMAP_SINGLE, 2848 },
	{ 0x7406, 0x7406, PDF_CMAP_SINGLE, 2851 },
	{ 0x7407, 0x7407, PDF_CMAP_SINGLE, 8085 },
	{ 0x7408, 0x7408, PDF_CMAP_SINGLE, 8093 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 2348 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 2849 },
	{ 0x740b, 0x740b, PDF_CMAP_SINGLE, 8091 },
	{ 0x740c, 0x740c, PDF_CMAP_SINGLE, 8090 },
	{ 0x740d, 0x740d, PDF_CMAP_SINGLE, 2853 },
	{ 0x7411, 0x7411, PDF_CMAP_SINGLE, 17772 },
	{ 0x7412, 0x7412, PDF_CMAP_SINGLE, 15334 },
	{ 0x7414, 0x7414, PDF_CMAP_SINGLE, 14836 },
	{ 0x7415, 0x7415, PDF_CMAP_SINGLE, 17051 },
	{ 0x7416, 0x7416, PDF_CMAP_SINGLE, 8725 },
	{ 0x7417, 0x7417, PDF_CMAP_SINGLE, 16557 },
	{ 0x7419, 0x7419, PDF_CMAP_SINGLE, 18163 },
	{ 0x741a, 0x741a, PDF_CMAP_SINGLE, 8726 },
	{ 0x741b, 0x741b, PDF_CMAP_SINGLE, 3296 },
	{ 0x741c, 0x741c, PDF_CMAP_SINGLE, 14557 },
	{ 0x741d, 0x741d, PDF_CMAP_SINGLE, 8732 },
	{ 0x741e, 0x741f, PDF_CMAP_RANGE, 18164 },
	{ 0x7420, 0x7420, PDF_CMAP_SINGLE, 8734 },
	{ 0x7421, 0x7421, PDF_CMAP_SINGLE, 8727 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 3290 },
	{ 0x7423, 0x7423, PDF_CMAP_SINGLE, 8731 },
	{ 0x7424, 0x7424, PDF_CMAP_SINGLE, 8730 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 3291 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 3297 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 3298 },
	{ 0x7429, 0x7429, PDF_CMAP_SINGLE, 8733 },
	{ 0x742a, 0x742a, PDF_CMAP_SINGLE, 3288 },
	{ 0x742b, 0x742b, PDF_CMAP_SINGLE, 8724 },
	{ 0x742c, 0x742c, PDF_CMAP_SINGLE, 8722 },
	{ 0x742d, 0x742d, PDF_CMAP_SINGLE, 8728 },
	{ 0x742e, 0x742e, PDF_CMAP_SINGLE, 8721 },
	{ 0x742f, 0x742f, PDF_CMAP_SINGLE, 3295 },
	{ 0x7430, 0x7430, PDF_CMAP_SINGLE, 8723 },
	{ 0x7431, 0x7431, PDF_CMAP_SINGLE, 8729 },
	{ 0x7432, 0x7432, PDF_CMAP_SINGLE, 8735 },
	{ 0x7433, 0x7433, PDF_CMAP_SINGLE, 3289 },
	{ 0x7434, 0x7434, PDF_CMAP_SINGLE, 3294 },
	{ 0x7435, 0x7436, PDF_CMAP_RANGE, 3292 },
	{ 0x7437, 0x7437, PDF_CMAP_SINGLE, 15073 },
	{ 0x7438, 0x7438, PDF_CMAP_SINGLE, 15113 },
	{ 0x7439, 0x7439, PDF_CMAP_SINGLE, 17054 },
	{ 0x743a, 0x743a, PDF_CMAP_SINGLE, 3287 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 15071 },
	{ 0x743f, 0x743f, PDF_CMAP_SINGLE, 3713 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 9376 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 3712 },
	{ 0x7442, 0x7442, PDF_CMAP_SINGLE, 9380 },
	{ 0x7443, 0x7443, PDF_CMAP_SINGLE, 14874 },
	{ 0x7444, 0x7444, PDF_CMAP_SINGLE, 9370 },
	{ 0x7445, 0x7445, PDF_CMAP_SINGLE, 15319 },
	{ 0x7446, 0x7446, PDF_CMAP_SINGLE, 9381 },
	{ 0x7447, 0x7447, PDF_CMAP_SINGLE, 17060 },
	{ 0x7448, 0x7448, PDF_CMAP_SINGLE, 15541 },
	{ 0x7449, 0x7449, PDF_CMAP_SINGLE, 17041 },
	{ 0x744a, 0x744b, PDF_CMAP_RANGE, 9371 },
	{ 0x744c, 0x744c, PDF_CMAP_SINGLE, 19116 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 9382 },
	{ 0x744e, 0x744e, PDF_CMAP_SINGLE, 9379 },
	{ 0x744f, 0x7450, PDF_CMAP_RANGE, 9377 },
	{ 0x7451, 0x7451, PDF_CMAP_SINGLE, 9374 },
	{ 0x7452, 0x7452, PDF_CMAP_SINGLE, 9373 },
	{ 0x7453, 0x7453, PDF_CMAP_SINGLE, 16945 },
	{ 0x7454, 0x7454, PDF_CMAP_SINGLE, 9383 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 3709 },
	{ 0x7456, 0x7456, PDF_CMAP_SINGLE, 14837 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 9375 },
	{ 0x7459, 0x7459, PDF_CMAP_SINGLE, 3714 },
	{ 0x745a, 0x745a, PDF_CMAP_SINGLE, 3708 },
	{ 0x745b, 0x745c, PDF_CMAP_RANGE, 3715 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 17094 },
	{ 0x745e, 0x745e, PDF_CMAP_SINGLE, 3711 },
	{ 0x745f, 0x745f, PDF_CMAP_SINGLE, 3710 },
	{ 0x7460, 0x7460, PDF_CMAP_SINGLE, 17058 },
	{ 0x7461, 0x7461, PDF_CMAP_SINGLE, 19020 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 10027 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 4141 },
	{ 0x7464, 0x7464, PDF_CMAP_SINGLE, 4140 },
	{ 0x7465, 0x7465, PDF_CMAP_SINGLE, 16641 },
	{ 0x7467, 0x7467, PDF_CMAP_SINGLE, 10032 },
	{ 0x7468, 0x7468, PDF_CMAP_SINGLE, 15086 },
	{ 0x7469, 0x7469, PDF_CMAP_SINGLE, 4518 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 4142 },
	{ 0x746b, 0x746b, PDF_CMAP_SINGLE, 15335 },
	{ 0x746c, 0x746c, PDF_CMAP_SINGLE, 17064 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 4144 },
	{ 0x746e, 0x746e, PDF_CMAP_SINGLE, 10033 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 3707 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 4143 },
	{ 0x7471, 0x7471, PDF_CMAP_SINGLE, 10029 },
	{ 0x7472, 0x7472, PDF_CMAP_SINGLE, 10031 },
	{ 0x7473, 0x7473, PDF_CMAP_SINGLE, 10028 },
	{ 0x7474, 0x7474, PDF_CMAP_SINGLE, 17065 },
	{ 0x7475, 0x7475, PDF_CMAP_SINGLE, 10030 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 17062 },
	{ 0x7479, 0x7479, PDF_CMAP_SINGLE, 10620 },
	{ 0x747a, 0x747a, PDF_CMAP_SINGLE, 15057 },
	{ 0x747c, 0x747c, PDF_CMAP_SINGLE, 10619 },
	{ 0x747d, 0x747d, PDF_CMAP_SINGLE, 10616 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 4521 },
	{ 0x747f, 0x747f, PDF_CMAP_SINGLE, 11211 },
	{ 0x7480, 0x7480, PDF_CMAP_SINGLE, 4522 },
	{ 0x7481, 0x7481, PDF_CMAP_SINGLE, 10615 },
	{ 0x7482, 0x7482, PDF_CMAP_SINGLE, 17068 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 4520 },
	{ 0x7485, 0x7485, PDF_CMAP_SINGLE, 10617 },
	{ 0x7486, 0x7486, PDF_CMAP_SINGLE, 10614 },
	{ 0x7487, 0x7487, PDF_CMAP_SINGLE, 10611 },
	{ 0x7488, 0x7488, PDF_CMAP_SINGLE, 10618 },
	{ 0x7489, 0x748a, PDF_CMAP_RANGE, 10612 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 4519 },
	{ 0x748c, 0x748c, PDF_CMAP_SINGLE, 14881 },
	{ 0x748d, 0x748d, PDF_CMAP_SINGLE, 18077 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 11688 },
	{ 0x7492, 0x7492, PDF_CMAP_SINGLE, 11215 },
	{ 0x7494, 0x7494, PDF_CMAP_SINGLE, 11214 },
	{ 0x7495, 0x7495, PDF_CMAP_SINGLE, 11216 },
	{ 0x7497, 0x7497, PDF_CMAP_SINGLE, 11685 },
	{ 0x7498, 0x7498, PDF_CMAP_SINGLE, 4860 },
	{ 0x7499, 0x7499, PDF_CMAP_SINGLE, 14835 },
	{ 0x749a, 0x749a, PDF_CMAP_SINGLE, 11212 },
	{ 0x749b, 0x749b, PDF_CMAP_SINGLE, 15114 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 4858 },
	{ 0x749d, 0x749d, PDF_CMAP_SINGLE, 19021 },
	{ 0x749e, 0x749e, PDF_CMAP_SINGLE, 4862 },
	{ 0x749f, 0x749f, PDF_CMAP_SINGLE, 4861 },
	{ 0x74a0, 0x74a0, PDF_CMAP_SINGLE, 11213 },
	{ 0x74a1, 0x74a1, PDF_CMAP_SINGLE, 11217 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 4859 },
	{ 0x74a4, 0x74a4, PDF_CMAP_SINGLE, 16752 },
	{ 0x74a5, 0x74a5, PDF_CMAP_SINGLE, 11692 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 5138 },
	{ 0x74a7, 0x74a7, PDF_CMAP_SINGLE, 5364 },
	{ 0x74a8, 0x74a8, PDF_CMAP_SINGLE, 5139 },
	{ 0x74a9, 0x74a9, PDF_CMAP_SINGLE, 5136 },
	{ 0x74aa, 0x74aa, PDF_CMAP_SINGLE, 11689 },
	{ 0x74ab, 0x74ab, PDF_CMAP_SINGLE, 11687 },
	{ 0x74ad, 0x74ad, PDF_CMAP_SINGLE, 11690 },
	{ 0x74af, 0x74af, PDF_CMAP_SINGLE, 11693 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 5137 },
	{ 0x74b1, 0x74b1, PDF_CMAP_SINGLE, 11691 },
	{ 0x74b2, 0x74b2, PDF_CMAP_SINGLE, 11686 },
	{ 0x74b4, 0x74b4, PDF_CMAP_SINGLE, 15525 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 12120 },
	{ 0x74b6, 0x74b6, PDF_CMAP_SINGLE, 12123 },
	{ 0x74b7, 0x74b7, PDF_CMAP_SINGLE, 12482 },
	{ 0x74b8, 0x74b8, PDF_CMAP_SINGLE, 12118 },
	{ 0x74b9, 0x74b9, PDF_CMAP_SINGLE, 18922 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 12793 },
	{ 0x74bb, 0x74bb, PDF_CMAP_SINGLE, 12124 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 5527 },
	{ 0x74be, 0x74be, PDF_CMAP_SINGLE, 12122 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 5365 },
	{ 0x74c0, 0x74c0, PDF_CMAP_SINGLE, 12119 },
	{ 0x74c1, 0x74c1, PDF_CMAP_SINGLE, 12121 },
	{ 0x74c2, 0x74c2, PDF_CMAP_SINGLE, 12125 },
	{ 0x74c3, 0x74c3, PDF_CMAP_SINGLE, 12483 },
	{ 0x74c5, 0x74c5, PDF_CMAP_SINGLE, 12481 },
	{ 0x74c6, 0x74c6, PDF_CMAP_SINGLE, 19125 },
	{ 0x74c8, 0x74c8, PDF_CMAP_SINGLE, 17077 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 5528 },
	{ 0x74cb, 0x74cb, PDF_CMAP_SINGLE, 12480 },
	{ 0x74cc, 0x74cc, PDF_CMAP_SINGLE, 15070 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 5665 },
	{ 0x74d0, 0x74d0, PDF_CMAP_SINGLE, 15003 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 18170 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 5758 },
	{ 0x74d5, 0x74d5, PDF_CMAP_SINGLE, 13202 },
	{ 0x74d6, 0x74d6, PDF_CMAP_SINGLE, 5757 },
	{ 0x74d7, 0x74d7, PDF_CMAP_SINGLE, 13204 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 13201 },
	{ 0x74d9, 0x74d9, PDF_CMAP_SINGLE, 13203 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 5886 },
	{ 0x74db, 0x74db, PDF_CMAP_SINGLE, 13453 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 856 },
	{ 0x74dd, 0x74dd, PDF_CMAP_SINGLE, 6669 },
	{ 0x74de, 0x74df, PDF_CMAP_RANGE, 7498 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 2854 },
	{ 0x74e1, 0x74e1, PDF_CMAP_SINGLE, 9384 },
	{ 0x74e2, 0x74e2, PDF_CMAP_SINGLE, 4863 },
	{ 0x74e3, 0x74e3, PDF_CMAP_SINGLE, 5529 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 5835 },
	{ 0x74e5, 0x74e5, PDF_CMAP_SINGLE, 13454 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 857 },
	{ 0x74e7, 0x74e7, PDF_CMAP_SINGLE, 16382 },
	{ 0x74e8, 0x74e8, PDF_CMAP_SINGLE, 6670 },
	{ 0x74e9, 0x74e9, PDF_CMAP_SINGLE, 288 },
	{ 0x74ec, 0x74ec, PDF_CMAP_SINGLE, 7054 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 7055 },
	{ 0x74f0, 0x74f0, PDF_CMAP_SINGLE, 16358 },
	{ 0x74f1, 0x74f1, PDF_CMAP_SINGLE, 16283 },
	{ 0x74f2, 0x74f2, PDF_CMAP_SINGLE, 16351 },
	{ 0x74f4, 0x74f5, PDF_CMAP_RANGE, 7500 },
	{ 0x74f6, 0x74f7, PDF_CMAP_RANGE, 2855 },
	{ 0x74f8, 0x74f8, PDF_CMAP_SINGLE, 16372 },
	{ 0x74fb, 0x74fb, PDF_CMAP_SINGLE, 8736 },
	{ 0x74fd, 0x74fd, PDF_CMAP_SINGLE, 9387 },
	{ 0x74fe, 0x74fe, PDF_CMAP_SINGLE, 9386 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 9385 },
	{ 0x7500, 0x7500, PDF_CMAP_SINGLE, 10034 },
	{ 0x7502, 0x7503, PDF_CMAP_RANGE, 10035 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 4145 },
	{ 0x7505, 0x7505, PDF_CMAP_SINGLE, 16350 },
	{ 0x7507, 0x7507, PDF_CMAP_SINGLE, 10622 },
	{ 0x7508, 0x7508, PDF_CMAP_SINGLE, 10621 },
	{ 0x750b, 0x750b, PDF_CMAP_SINGLE, 11218 },
	{ 0x750c, 0x750d, PDF_CMAP_RANGE, 4864 },
	{ 0x750e, 0x750e, PDF_CMAP_SINGLE, 17079 },
	{ 0x750f, 0x750f, PDF_CMAP_SINGLE, 11697 },
	{ 0x7510, 0x7512, PDF_CMAP_RANGE, 11694 },
	{ 0x7513, 0x7513, PDF_CMAP_SINGLE, 12127 },
	{ 0x7514, 0x7514, PDF_CMAP_SINGLE, 12126 },
	{ 0x7515, 0x7515, PDF_CMAP_SINGLE, 5366 },
	{ 0x7516, 0x7516, PDF_CMAP_SINGLE, 12484 },
	{ 0x7517, 0x7517, PDF_CMAP_SINGLE, 13027 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 858 },
	{ 0x7519, 0x7519, PDF_CMAP_SINGLE, 17795 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 1951 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 2857 },
	{ 0x751d, 0x751d, PDF_CMAP_SINGLE, 9388 },
	{ 0x751e, 0x751e, PDF_CMAP_SINGLE, 17080 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 859 },
	{ 0x7521, 0x7521, PDF_CMAP_SINGLE, 7502 },
	{ 0x7522, 0x7522, PDF_CMAP_SINGLE, 2858 },
	{ 0x7525, 0x7526, PDF_CMAP_RANGE, 3299 },
	{ 0x7528, 0x7529, PDF_CMAP_RANGE, 860 },
	{ 0x752a, 0x752a, PDF_CMAP_SINGLE, 6159 },
	{ 0x752b, 0x752b, PDF_CMAP_SINGLE, 1265 },
	{ 0x752c, 0x752c, PDF_CMAP_SINGLE, 1264 },
	{ 0x752d, 0x752d, PDF_CMAP_SINGLE, 1952 },
	{ 0x752e, 0x752e, PDF_CMAP_SINGLE, 7056 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 8737 },
	{ 0x7530, 0x7533, PDF_CMAP_RANGE, 862 },
	{ 0x7534, 0x7534, PDF_CMAP_SINGLE, 16007 },
	{ 0x7535, 0x7535, PDF_CMAP_SINGLE, 17754 },
	{ 0x7537, 0x7538, PDF_CMAP_RANGE, 1266 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 6355 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 6354 },
	{ 0x753b, 0x753b, PDF_CMAP_SINGLE, 15867 },
	{ 0x753d, 0x753d, PDF_CMAP_SINGLE, 1621 },
	{ 0x753e, 0x753e, PDF_CMAP_SINGLE, 6673 },
	{ 0x753f, 0x7540, PDF_CMAP_RANGE, 6671 },
	{ 0x7542, 0x7542, PDF_CMAP_SINGLE, 17357 },
	{ 0x7546, 0x7546, PDF_CMAP_SINGLE, 16192 },
	{ 0x7547, 0x7548, PDF_CMAP_RANGE, 7057 },
	{ 0x754a, 0x754a, PDF_CMAP_SINGLE, 17087 },
	{ 0x754b, 0x754b, PDF_CMAP_SINGLE, 1956 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 1954 },
	{ 0x754d, 0x754d, PDF_CMAP_SINGLE, 17086 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 1955 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 1953 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 15614 },
	{ 0x7553, 0x7553, PDF_CMAP_SINGLE, 16363 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 2353 },
	{ 0x7555, 0x7555, PDF_CMAP_SINGLE, 15082 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 2357 },
	{ 0x755a, 0x755a, PDF_CMAP_SINGLE, 2356 },
	{ 0x755b, 0x755b, PDF_CMAP_SINGLE, 7503 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 2355 },
	{ 0x755d, 0x755d, PDF_CMAP_SINGLE, 2354 },
	{ 0x755f, 0x755f, PDF_CMAP_SINGLE, 7504 },
	{ 0x7560, 0x7560, PDF_CMAP_SINGLE, 15729 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 2861 },
	{ 0x7563, 0x7563, PDF_CMAP_SINGLE, 8095 },
	{ 0x7564, 0x7564, PDF_CMAP_SINGLE, 8094 },
	{ 0x7565, 0x7566, PDF_CMAP_RANGE, 2859 },
	{ 0x7567, 0x7567, PDF_CMAP_SINGLE, 17088 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 3302 },
	{ 0x756b, 0x756b, PDF_CMAP_SINGLE, 3301 },
	{ 0x756c, 0x756c, PDF_CMAP_SINGLE, 8739 },
	{ 0x756d, 0x756d, PDF_CMAP_SINGLE, 18173 },
	{ 0x756e, 0x756e, PDF_CMAP_SINGLE, 17089 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 8738 },
	{ 0x7570, 0x7570, PDF_CMAP_SINGLE, 2862 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 18174 },
	{ 0x7576, 0x7576, PDF_CMAP_SINGLE, 3717 },
	{ 0x7577, 0x7577, PDF_CMAP_SINGLE, 9390 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 3718 },
	{ 0x7579, 0x7579, PDF_CMAP_SINGLE, 9389 },
	{ 0x757a, 0x757a, PDF_CMAP_SINGLE, 16207 },
	{ 0x757d, 0x757d, PDF_CMAP_SINGLE, 10037 },
	{ 0x757e, 0x757e, PDF_CMAP_SINGLE, 10623 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 4523 },
	{ 0x7580, 0x7580, PDF_CMAP_SINGLE, 11219 },
	{ 0x7583, 0x7583, PDF_CMAP_SINGLE, 15077 },
	{ 0x7584, 0x7584, PDF_CMAP_SINGLE, 11698 },
	{ 0x7586, 0x7586, PDF_CMAP_SINGLE, 5531 },
	{ 0x7587, 0x7587, PDF_CMAP_SINGLE, 5530 },
	{ 0x758a, 0x758a, PDF_CMAP_SINGLE, 5836 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 866 },
	{ 0x758c, 0x758c, PDF_CMAP_SINGLE, 6674 },
	{ 0x758d, 0x758d, PDF_CMAP_SINGLE, 18175 },
	{ 0x758e, 0x758e, PDF_CMAP_SINGLE, 17093 },
	{ 0x758f, 0x758f, PDF_CMAP_SINGLE, 2863 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 10038 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 4146 },
	{ 0x7592, 0x7592, PDF_CMAP_SINGLE, 558 },
	{ 0x7594, 0x7595, PDF_CMAP_RANGE, 6356 },
	{ 0x7598, 0x7598, PDF_CMAP_SINGLE, 6675 },
	{ 0x7599, 0x759a, PDF_CMAP_RANGE, 1623 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 1622 },
	{ 0x759e, 0x759e, PDF_CMAP_SINGLE, 17095 },
	{ 0x75a2, 0x75a3, PDF_CMAP_RANGE, 1960 },
	{ 0x75a4, 0x75a5, PDF_CMAP_RANGE, 1958 },
	{ 0x75a7, 0x75a7, PDF_CMAP_SINGLE, 7059 },
	{ 0x75aa, 0x75aa, PDF_CMAP_SINGLE, 7060 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 1957 },
	{ 0x75b0, 0x75b0, PDF_CMAP_SINGLE, 7505 },
	{ 0x75b1, 0x75b1, PDF_CMAP_SINGLE, 16045 },
	{ 0x75b2, 0x75b3, PDF_CMAP_RANGE, 2361 },
	{ 0x75b4, 0x75b4, PDF_CMAP_SINGLE, 17096 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 2866 },
	{ 0x75b6, 0x75b6, PDF_CMAP_SINGLE, 7511 },
	{ 0x75b8, 0x75b8, PDF_CMAP_SINGLE, 2367 },
	{ 0x75b9, 0x75b9, PDF_CMAP_SINGLE, 2365 },
	{ 0x75ba, 0x75ba, PDF_CMAP_SINGLE, 7512 },
	{ 0x75bb, 0x75bb, PDF_CMAP_SINGLE, 7507 },
	{ 0x75bc, 0x75bc, PDF_CMAP_SINGLE, 2364 },
	{ 0x75bd, 0x75bd, PDF_CMAP_SINGLE, 2363 },
	{ 0x75be, 0x75be, PDF_CMAP_SINGLE, 2358 },
	{ 0x75bf, 0x75bf, PDF_CMAP_SINGLE, 7510 },
	{ 0x75c0, 0x75c0, PDF_CMAP_SINGLE, 7509 },
	{ 0x75c1, 0x75c1, PDF_CMAP_SINGLE, 7506 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 2366 },
	{ 0x75c3, 0x75c3, PDF_CMAP_SINGLE, 16275 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 7508 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 2359 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 2360 },
	{ 0x75c8, 0x75c8, PDF_CMAP_SINGLE, 18177 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 2867 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 8099 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 2868 },
	{ 0x75ce, 0x75ce, PDF_CMAP_SINGLE, 8096 },
	{ 0x75cf, 0x75cf, PDF_CMAP_SINGLE, 8098 },
	{ 0x75d0, 0x75d0, PDF_CMAP_SINGLE, 8102 },
	{ 0x75d1, 0x75d1, PDF_CMAP_SINGLE, 8101 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 8097 },
	{ 0x75d4, 0x75d5, PDF_CMAP_RANGE, 2864 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 8747 },
	{ 0x75d8, 0x75d8, PDF_CMAP_SINGLE, 3307 },
	{ 0x75d9, 0x75d9, PDF_CMAP_SINGLE, 3306 },
	{ 0x75da, 0x75da, PDF_CMAP_SINGLE, 8741 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 3304 },
	{ 0x75dc, 0x75dc, PDF_CMAP_SINGLE, 17358 },
	{ 0x75dd, 0x75dd, PDF_CMAP_SINGLE, 8744 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 3308 },
	{ 0x75df, 0x75df, PDF_CMAP_SINGLE, 8745 },
	{ 0x75e0, 0x75e0, PDF_CMAP_SINGLE, 3309 },
	{ 0x75e1, 0x75e1, PDF_CMAP_SINGLE, 8742 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 3303 },
	{ 0x75e3, 0x75e3, PDF_CMAP_SINGLE, 3305 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 8746 },
	{ 0x75e6, 0x75e6, PDF_CMAP_SINGLE, 8743 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 8740 },
	{ 0x75ed, 0x75ed, PDF_CMAP_SINGLE, 9403 },
	{ 0x75ef, 0x75ef, PDF_CMAP_SINGLE, 9392 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 3720 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 3723 },
	{ 0x75f2, 0x75f2, PDF_CMAP_SINGLE, 3722 },
	{ 0x75f3, 0x75f3, PDF_CMAP_SINGLE, 3727 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 3726 },
	{ 0x75f5, 0x75f5, PDF_CMAP_SINGLE, 9404 },
	{ 0x75f6, 0x75f6, PDF_CMAP_SINGLE, 9402 },
	{ 0x75f7, 0x75f7, PDF_CMAP_SINGLE, 9395 },
	{ 0x75f8, 0x75f8, PDF_CMAP_SINGLE, 9399 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 9398 },
	{ 0x75fa, 0x75fa, PDF_CMAP_SINGLE, 3724 },
	{ 0x75fb, 0x75fb, PDF_CMAP_SINGLE, 9401 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 9397 },
	{ 0x75fd, 0x75fd, PDF_CMAP_SINGLE, 9405 },
	{ 0x75fe, 0x75fe, PDF_CMAP_SINGLE, 9396 },
	{ 0x75ff, 0x75ff, PDF_CMAP_SINGLE, 3725 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 3719 },
	{ 0x7601, 0x7601, PDF_CMAP_SINGLE, 3721 },
	{ 0x7602, 0x7602, PDF_CMAP_SINGLE, 17097 },
	{ 0x7603, 0x7603, PDF_CMAP_SINGLE, 9394 },
	{ 0x7607, 0x7607, PDF_CMAP_SINGLE, 16271 },
	{ 0x7608, 0x7608, PDF_CMAP_SINGLE, 10040 },
	{ 0x7609, 0x7609, PDF_CMAP_SINGLE, 4150 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 10044 },
	{ 0x760b, 0x760b, PDF_CMAP_SINGLE, 4149 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 10041 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 4148 },
	{ 0x760f, 0x760f, PDF_CMAP_SINGLE, 9393 },
	{ 0x7610, 0x7610, PDF_CMAP_SINGLE, 9400 },
	{ 0x7611, 0x7611, PDF_CMAP_SINGLE, 10043 },
	{ 0x7613, 0x7613, PDF_CMAP_SINGLE, 4151 },
	{ 0x7614, 0x7614, PDF_CMAP_SINGLE, 10045 },
	{ 0x7615, 0x7615, PDF_CMAP_SINGLE, 10042 },
	{ 0x7616, 0x7616, PDF_CMAP_SINGLE, 10039 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 10626 },
	{ 0x761a, 0x761a, PDF_CMAP_SINGLE, 10630 },
	{ 0x761b, 0x761b, PDF_CMAP_SINGLE, 10632 },
	{ 0x761c, 0x761c, PDF_CMAP_SINGLE, 10628 },
	{ 0x761d, 0x761d, PDF_CMAP_SINGLE, 10627 },
	{ 0x761e, 0x761e, PDF_CMAP_SINGLE, 10625 },
	{ 0x761f, 0x761f, PDF_CMAP_SINGLE, 4526 },
	{ 0x7620, 0x7620, PDF_CMAP_SINGLE, 4524 },
	{ 0x7621, 0x7622, PDF_CMAP_RANGE, 4529 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 10629 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 4527 },
	{ 0x7625, 0x7625, PDF_CMAP_SINGLE, 10624 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 4528 },
	{ 0x7627, 0x7627, PDF_CMAP_SINGLE, 4147 },
	{ 0x7628, 0x7628, PDF_CMAP_SINGLE, 10631 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 4525 },
	{ 0x762c, 0x762c, PDF_CMAP_SINGLE, 17098 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 11221 },
	{ 0x762f, 0x762f, PDF_CMAP_SINGLE, 11220 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 11228 },
	{ 0x7631, 0x7631, PDF_CMAP_SINGLE, 11222 },
	{ 0x7632, 0x7632, PDF_CMAP_SINGLE, 11227 },
	{ 0x7633, 0x7633, PDF_CMAP_SINGLE, 11224 },
	{ 0x7634, 0x7634, PDF_CMAP_SINGLE, 4866 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 11226 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 4867 },
	{ 0x763a, 0x763a, PDF_CMAP_SINGLE, 4868 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 15984 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 11225 },
	{ 0x763d, 0x763d, PDF_CMAP_SINGLE, 11223 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 16102 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 5141 },
	{ 0x7643, 0x7643, PDF_CMAP_SINGLE, 11699 },
	{ 0x7646, 0x7646, PDF_CMAP_SINGLE, 5140 },
	{ 0x7647, 0x7647, PDF_CMAP_SINGLE, 11702 },
	{ 0x7648, 0x7649, PDF_CMAP_RANGE, 11700 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 5142 },
	{ 0x764d, 0x764d, PDF_CMAP_SINGLE, 18178 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 15986 },
	{ 0x764f, 0x764f, PDF_CMAP_SINGLE, 17100 },
	{ 0x7650, 0x7650, PDF_CMAP_SINGLE, 12131 },
	{ 0x7651, 0x7651, PDF_CMAP_SINGLE, 17099 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 5369 },
	{ 0x7653, 0x7653, PDF_CMAP_SINGLE, 12132 },
	{ 0x7654, 0x7654, PDF_CMAP_SINGLE, 16112 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 5367 },
	{ 0x7657, 0x7657, PDF_CMAP_SINGLE, 12133 },
	{ 0x7658, 0x7658, PDF_CMAP_SINGLE, 5368 },
	{ 0x7659, 0x7659, PDF_CMAP_SINGLE, 12130 },
	{ 0x765a, 0x765a, PDF_CMAP_SINGLE, 12134 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 12128 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 5532 },
	{ 0x7660, 0x7660, PDF_CMAP_SINGLE, 12485 },
	{ 0x7661, 0x7661, PDF_CMAP_SINGLE, 5533 },
	{ 0x7662, 0x7662, PDF_CMAP_SINGLE, 5666 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 12129 },
	{ 0x7665, 0x7665, PDF_CMAP_SINGLE, 5667 },
	{ 0x7666, 0x7666, PDF_CMAP_SINGLE, 16042 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 16297 },
	{ 0x7669, 0x7669, PDF_CMAP_SINGLE, 5759 },
	{ 0x766a, 0x766a, PDF_CMAP_SINGLE, 13028 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 5838 },
	{ 0x766d, 0x766d, PDF_CMAP_SINGLE, 13205 },
	{ 0x766e, 0x766e, PDF_CMAP_SINGLE, 5837 },
	{ 0x766f, 0x766f, PDF_CMAP_SINGLE, 17101 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 13352 },
	{ 0x7671, 0x7672, PDF_CMAP_RANGE, 5923 },
	{ 0x7673, 0x7673, PDF_CMAP_SINGLE, 15083 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 18179 },
	{ 0x7675, 0x7675, PDF_CMAP_SINGLE, 13635 },
	{ 0x7676, 0x7676, PDF_CMAP_SINGLE, 559 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 1962 },
	{ 0x7679, 0x7679, PDF_CMAP_SINGLE, 7061 },
	{ 0x767a, 0x767a, PDF_CMAP_SINGLE, 18181 },
	{ 0x767b, 0x767c, PDF_CMAP_RANGE, 3310 },
	{ 0x767d, 0x767d, PDF_CMAP_SINGLE, 867 },
	{ 0x767e, 0x767e, PDF_CMAP_SINGLE, 1013 },
	{ 0x767f, 0x767f, PDF_CMAP_SINGLE, 6160 },
	{ 0x7680, 0x7680, PDF_CMAP_SINGLE, 17363 },
	{ 0x7681, 0x7681, PDF_CMAP_SINGLE, 6358 },
	{ 0x7682, 0x7682, PDF_CMAP_SINGLE, 1268 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 1625 },
	{ 0x7686, 0x7688, PDF_CMAP_RANGE, 1963 },
	{ 0x7689, 0x7689, PDF_CMAP_SINGLE, 8104 },
	{ 0x768a, 0x768a, PDF_CMAP_SINGLE, 7513 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 2368 },
	{ 0x768c, 0x768c, PDF_CMAP_SINGLE, 17364 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 2869 },
	{ 0x768f, 0x768f, PDF_CMAP_SINGLE, 8103 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 17104 },
	{ 0x7692, 0x7692, PDF_CMAP_SINGLE, 8749 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 3313 },
	{ 0x7695, 0x7695, PDF_CMAP_SINGLE, 8748 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 3312 },
	{ 0x7699, 0x7699, PDF_CMAP_SINGLE, 9406 },
	{ 0x769a, 0x769a, PDF_CMAP_SINGLE, 4531 },
	{ 0x769b, 0x769b, PDF_CMAP_SINGLE, 10636 },
	{ 0x769c, 0x769e, PDF_CMAP_RANGE, 10633 },
	{ 0x76a1, 0x76a1, PDF_CMAP_SINGLE, 17109 },
	{ 0x76a4, 0x76a4, PDF_CMAP_SINGLE, 11703 },
	{ 0x76a5, 0x76a5, PDF_CMAP_SINGLE, 17110 },
	{ 0x76a6, 0x76a6, PDF_CMAP_SINGLE, 12135 },
	{ 0x76aa, 0x76aa, PDF_CMAP_SINGLE, 12795 },
	{ 0x76ab, 0x76ab, PDF_CMAP_SINGLE, 12794 },
	{ 0x76ad, 0x76ad, PDF_CMAP_SINGLE, 13206 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 868 },
	{ 0x76af, 0x76af, PDF_CMAP_SINGLE, 6676 },
	{ 0x76b0, 0x76b0, PDF_CMAP_SINGLE, 2369 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 3314 },
	{ 0x76b5, 0x76b5, PDF_CMAP_SINGLE, 9407 },
	{ 0x76b7, 0x76b7, PDF_CMAP_SINGLE, 17111 },
	{ 0x76b8, 0x76b8, PDF_CMAP_SINGLE, 10046 },
	{ 0x76ba, 0x76ba, PDF_CMAP_SINGLE, 4532 },
	{ 0x76bb, 0x76bb, PDF_CMAP_SINGLE, 11229 },
	{ 0x76bd, 0x76bd, PDF_CMAP_SINGLE, 12136 },
	{ 0x76be, 0x76be, PDF_CMAP_SINGLE, 12796 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 869 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 1626 },
	{ 0x76c3, 0x76c3, PDF_CMAP_SINGLE, 1968 },
	{ 0x76c4, 0x76c4, PDF_CMAP_SINGLE, 7062 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 1969 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 1967 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 1966 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 7514 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 2370 },
	{ 0x76cc, 0x76cc, PDF_CMAP_SINGLE, 17112 },
	{ 0x76cd, 0x76ce, PDF_CMAP_RANGE, 2371 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 2871 },
	{ 0x76d3, 0x76d3, PDF_CMAP_SINGLE, 8105 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 2870 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 16213 },
	{ 0x76d9, 0x76d9, PDF_CMAP_SINGLE, 18891 },
	{ 0x76da, 0x76da, PDF_CMAP_SINGLE, 8750 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 2872 },
	{ 0x76dc, 0x76dc, PDF_CMAP_SINGLE, 3315 },
	{ 0x76dd, 0x76dd, PDF_CMAP_SINGLE, 9408 },
	{ 0x76de, 0x76df, PDF_CMAP_RANGE, 3728 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 4152 },
	{ 0x76e3, 0x76e3, PDF_CMAP_SINGLE, 4153 },
	{ 0x76e4, 0x76e4, PDF_CMAP_SINGLE, 4533 },
	{ 0x76e5, 0x76e5, PDF_CMAP_SINGLE, 4870 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 11230 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 4869 },
	{ 0x76e9, 0x76e9, PDF_CMAP_SINGLE, 11704 },
	{ 0x76ea, 0x76ea, PDF_CMAP_SINGLE, 5143 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 12137 },
	{ 0x76ed, 0x76ed, PDF_CMAP_SINGLE, 12797 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 870 },
	{ 0x76ef, 0x76ef, PDF_CMAP_SINGLE, 1269 },
	{ 0x76f0, 0x76f0, PDF_CMAP_SINGLE, 6679 },
	{ 0x76f1, 0x76f1, PDF_CMAP_SINGLE, 6678 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 1627 },
	{ 0x76f3, 0x76f3, PDF_CMAP_SINGLE, 6677 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 1628 },
	{ 0x76f5, 0x76f5, PDF_CMAP_SINGLE, 6680 },
	{ 0x76f7, 0x76f7, PDF_CMAP_SINGLE, 7068 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 1972 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 1971 },
	{ 0x76fa, 0x76fa, PDF_CMAP_SINGLE, 7070 },
	{ 0x76fb, 0x76fb, PDF_CMAP_SINGLE, 7069 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 1976 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 1975 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 1970 },
	{ 0x7703, 0x7705, PDF_CMAP_RANGE, 7064 },
	{ 0x7707, 0x7707, PDF_CMAP_SINGLE, 1977 },
	{ 0x7708, 0x7708, PDF_CMAP_SINGLE, 7063 },
	{ 0x7709, 0x7709, PDF_CMAP_SINGLE, 1973 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 7067 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 1974 },
	{ 0x770c, 0x770c, PDF_CMAP_SINGLE, 15714 },
	{ 0x770e, 0x770f, PDF_CMAP_RANGE, 15106 },
	{ 0x7710, 0x7710, PDF_CMAP_SINGLE, 7517 },
	{ 0x7711, 0x7711, PDF_CMAP_SINGLE, 7521 },
	{ 0x7712, 0x7712, PDF_CMAP_SINGLE, 7519 },
	{ 0x7713, 0x7713, PDF_CMAP_SINGLE, 7518 },
	{ 0x7715, 0x7715, PDF_CMAP_SINGLE, 7522 },
	{ 0x7719, 0x771a, PDF_CMAP_RANGE, 7523 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 7516 },
	{ 0x771d, 0x771d, PDF_CMAP_SINGLE, 7515 },
	{ 0x771e, 0x771e, PDF_CMAP_SINGLE, 17118 },
	{ 0x771f, 0x7720, PDF_CMAP_RANGE, 2374 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 7525 },
	{ 0x7723, 0x7723, PDF_CMAP_SINGLE, 7520 },
	{ 0x7724, 0x7724, PDF_CMAP_SINGLE, 16360 },
	{ 0x7725, 0x7725, PDF_CMAP_SINGLE, 8114 },
	{ 0x7726, 0x7726, PDF_CMAP_SINGLE, 17119 },
	{ 0x7727, 0x7727, PDF_CMAP_SINGLE, 7526 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 2376 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 2373 },
	{ 0x772b, 0x772b, PDF_CMAP_SINGLE, 16371 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 8108 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 8107 },
	{ 0x7731, 0x7732, PDF_CMAP_RANGE, 8109 },
	{ 0x7733, 0x7733, PDF_CMAP_SINGLE, 8112 },
	{ 0x7734, 0x7734, PDF_CMAP_SINGLE, 8111 },
	{ 0x7735, 0x7735, PDF_CMAP_SINGLE, 8116 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 2876 },
	{ 0x7737, 0x7737, PDF_CMAP_SINGLE, 2873 },
	{ 0x7738, 0x7738, PDF_CMAP_SINGLE, 2877 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 8106 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 2878 },
	{ 0x773b, 0x773b, PDF_CMAP_SINGLE, 8115 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 2875 },
	{ 0x773d, 0x773d, PDF_CMAP_SINGLE, 8113 },
	{ 0x773e, 0x773e, PDF_CMAP_SINGLE, 2874 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 17120 },
	{ 0x7743, 0x7743, PDF_CMAP_SINGLE, 18186 },
	{ 0x7744, 0x7744, PDF_CMAP_SINGLE, 8753 },
	{ 0x7745, 0x7745, PDF_CMAP_SINGLE, 8755 },
	{ 0x7746, 0x7747, PDF_CMAP_RANGE, 8751 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 8756 },
	{ 0x774b, 0x774c, PDF_CMAP_RANGE, 8758 },
	{ 0x774d, 0x774d, PDF_CMAP_SINGLE, 8754 },
	{ 0x774e, 0x774e, PDF_CMAP_SINGLE, 8757 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 3316 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 9412 },
	{ 0x7754, 0x7754, PDF_CMAP_SINGLE, 9417 },
	{ 0x7755, 0x7755, PDF_CMAP_SINGLE, 9409 },
	{ 0x7756, 0x7756, PDF_CMAP_SINGLE, 9413 },
	{ 0x7758, 0x7758, PDF_CMAP_SINGLE, 17123 },
	{ 0x7759, 0x7759, PDF_CMAP_SINGLE, 9418 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 9414 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 3730 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 3738 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 3733 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 9410 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 4157 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 3741 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 3734 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 3739 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 3732 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 9416 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 3740 },
	{ 0x7769, 0x7769, PDF_CMAP_SINGLE, 9415 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 3736 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 3731 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 3737 },
	{ 0x776d, 0x776d, PDF_CMAP_SINGLE, 9419 },
	{ 0x776e, 0x776e, PDF_CMAP_SINGLE, 10051 },
	{ 0x776f, 0x776f, PDF_CMAP_SINGLE, 10053 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14699 },
	{ 0x7777, 0x7777, PDF_CMAP_SINGLE, 18089 },
	{ 0x7778, 0x7778, PDF_CMAP_SINGLE, 15104 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 3735 },
	{ 0x777a, 0x777a, PDF_CMAP_SINGLE, 17175 },
	{ 0x777b, 0x777b, PDF_CMAP_SINGLE, 15108 },
	{ 0x777c, 0x777c, PDF_CMAP_SINGLE, 10048 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 4155 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 10054 },
	{ 0x777f, 0x777f, PDF_CMAP_SINGLE, 4156 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 10052 },
	{ 0x7781, 0x7781, PDF_CMAP_SINGLE, 10047 },
	{ 0x7782, 0x7782, PDF_CMAP_SINGLE, 10050 },
	{ 0x7783, 0x7783, PDF_CMAP_SINGLE, 10055 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 4154 },
	{ 0x7785, 0x7785, PDF_CMAP_SINGLE, 10049 },
	{ 0x7787, 0x7787, PDF_CMAP_SINGLE, 4535 },
	{ 0x7788, 0x7788, PDF_CMAP_SINGLE, 10640 },
	{ 0x7789, 0x7789, PDF_CMAP_SINGLE, 10639 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 4538 },
	{ 0x778c, 0x778c, PDF_CMAP_SINGLE, 4536 },
	{ 0x778d, 0x778d, PDF_CMAP_SINGLE, 10637 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 4534 },
	{ 0x778f, 0x778f, PDF_CMAP_SINGLE, 10638 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 4537 },
	{ 0x7793, 0x7793, PDF_CMAP_SINGLE, 14381 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 11238 },
	{ 0x7797, 0x7797, PDF_CMAP_SINGLE, 11240 },
	{ 0x7798, 0x7798, PDF_CMAP_SINGLE, 18190 },
	{ 0x7799, 0x7799, PDF_CMAP_SINGLE, 11239 },
	{ 0x779a, 0x779a, PDF_CMAP_SINGLE, 11231 },
	{ 0x779b, 0x779b, PDF_CMAP_SINGLE, 11235 },
	{ 0x779c, 0x779c, PDF_CMAP_SINGLE, 11234 },
	{ 0x779d, 0x779d, PDF_CMAP_SINGLE, 11232 },
	{ 0x779e, 0x779f, PDF_CMAP_RANGE, 4872 },
	{ 0x77a0, 0x77a0, PDF_CMAP_SINGLE, 4871 },
	{ 0x77a1, 0x77a1, PDF_CMAP_SINGLE, 11233 },
	{ 0x77a2, 0x77a3, PDF_CMAP_RANGE, 11236 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 4874 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 5148 },
	{ 0x77a8, 0x77a8, PDF_CMAP_SINGLE, 11712 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 5145 },
	{ 0x77ab, 0x77ab, PDF_CMAP_SINGLE, 11706 },
	{ 0x77ac, 0x77ac, PDF_CMAP_SINGLE, 5147 },
	{ 0x77ad, 0x77ad, PDF_CMAP_SINGLE, 5149 },
	{ 0x77af, 0x77af, PDF_CMAP_SINGLE, 17125 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 5146 },
	{ 0x77b1, 0x77b1, PDF_CMAP_SINGLE, 11711 },
	{ 0x77b2, 0x77b2, PDF_CMAP_SINGLE, 11707 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 5144 },
	{ 0x77b4, 0x77b4, PDF_CMAP_SINGLE, 11710 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 11705 },
	{ 0x77b6, 0x77b6, PDF_CMAP_SINGLE, 11709 },
	{ 0x77b7, 0x77b7, PDF_CMAP_SINGLE, 11708 },
	{ 0x77b9, 0x77b9, PDF_CMAP_SINGLE, 15987 },
	{ 0x77ba, 0x77ba, PDF_CMAP_SINGLE, 12139 },
	{ 0x77bb, 0x77bc, PDF_CMAP_RANGE, 5372 },
	{ 0x77bd, 0x77bd, PDF_CMAP_SINGLE, 5370 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 18193 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 5371 },
	{ 0x77c2, 0x77c2, PDF_CMAP_SINGLE, 12138 },
	{ 0x77c3, 0x77c3, PDF_CMAP_SINGLE, 15281 },
	{ 0x77c4, 0x77c4, PDF_CMAP_SINGLE, 12488 },
	{ 0x77c5, 0x77c5, PDF_CMAP_SINGLE, 16833 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5534 },
	{ 0x77c9, 0x77ca, PDF_CMAP_RANGE, 12486 },
	{ 0x77cb, 0x77cb, PDF_CMAP_SINGLE, 18194 },
	{ 0x77cc, 0x77cc, PDF_CMAP_SINGLE, 12798 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 12801 },
	{ 0x77ce, 0x77cf, PDF_CMAP_RANGE, 12799 },
	{ 0x77d0, 0x77d0, PDF_CMAP_SINGLE, 13029 },
	{ 0x77d3, 0x77d3, PDF_CMAP_SINGLE, 5760 },
	{ 0x77d4, 0x77d4, PDF_CMAP_SINGLE, 13353 },
	{ 0x77d5, 0x77d5, PDF_CMAP_SINGLE, 13455 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5925 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 13523 },
	{ 0x77da, 0x77da, PDF_CMAP_SINGLE, 5971 },
	{ 0x77db, 0x77db, PDF_CMAP_SINGLE, 871 },
	{ 0x77dc, 0x77dc, PDF_CMAP_SINGLE, 1978 },
	{ 0x77dd, 0x77dd, PDF_CMAP_SINGLE, 18894 },
	{ 0x77de, 0x77de, PDF_CMAP_SINGLE, 8760 },
	{ 0x77e0, 0x77e0, PDF_CMAP_SINGLE, 9420 },
	{ 0x77e2, 0x77e2, PDF_CMAP_SINGLE, 872 },
	{ 0x77e3, 0x77e3, PDF_CMAP_SINGLE, 1270 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 1629 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 16513 },
	{ 0x77e7, 0x77e8, PDF_CMAP_RANGE, 7071 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 2377 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 8761 },
	{ 0x77ed, 0x77ed, PDF_CMAP_SINGLE, 3317 },
	{ 0x77ee, 0x77ee, PDF_CMAP_SINGLE, 3742 },
	{ 0x77ef, 0x77ef, PDF_CMAP_SINGLE, 5150 },
	{ 0x77f0, 0x77f0, PDF_CMAP_SINGLE, 11713 },
	{ 0x77f1, 0x77f1, PDF_CMAP_SINGLE, 12489 },
	{ 0x77f2, 0x77f2, PDF_CMAP_SINGLE, 12802 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 873 },
	{ 0x77f4, 0x77f4, PDF_CMAP_SINGLE, 17129 },
	{ 0x77f7, 0x77f7, PDF_CMAP_SINGLE, 6686 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 6681 },
	{ 0x77f9, 0x77f9, PDF_CMAP_SINGLE, 6683 },
	{ 0x77fa, 0x77fa, PDF_CMAP_SINGLE, 6685 },
	{ 0x77fb, 0x77fb, PDF_CMAP_SINGLE, 6684 },
	{ 0x77fc, 0x77fc, PDF_CMAP_SINGLE, 6682 },
	{ 0x77fd, 0x77fd, PDF_CMAP_SINGLE, 1630 },
	{ 0x77fe, 0x77fe, PDF_CMAP_SINGLE, 16341 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 1979 },
	{ 0x7803, 0x7803, PDF_CMAP_SINGLE, 7081 },
	{ 0x7805, 0x7805, PDF_CMAP_SINGLE, 7076 },
	{ 0x7806, 0x7806, PDF_CMAP_SINGLE, 7073 },
	{ 0x7808, 0x7808, PDF_CMAP_SINGLE, 15942 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 7080 },
	{ 0x780c, 0x780d, PDF_CMAP_RANGE, 1981 },
	{ 0x780e, 0x780e, PDF_CMAP_SINGLE, 7079 },
	{ 0x780f, 0x780f, PDF_CMAP_SINGLE, 7078 },
	{ 0x7810, 0x7810, PDF_CMAP_SINGLE, 7077 },
	{ 0x7811, 0x7812, PDF_CMAP_RANGE, 7074 },
	{ 0x7813, 0x7813, PDF_CMAP_SINGLE, 7082 },
	{ 0x7814, 0x7814, PDF_CMAP_SINGLE, 1980 },
	{ 0x7818, 0x7818, PDF_CMAP_SINGLE, 18195 },
	{ 0x781c, 0x781c, PDF_CMAP_SINGLE, 18196 },
	{ 0x781d, 0x781d, PDF_CMAP_SINGLE, 2381 },
	{ 0x781e, 0x781e, PDF_CMAP_SINGLE, 15123 },
	{ 0x781f, 0x781f, PDF_CMAP_SINGLE, 2387 },
	{ 0x7820, 0x7820, PDF_CMAP_SINGLE, 2386 },
	{ 0x7821, 0x7821, PDF_CMAP_SINGLE, 7535 },
	{ 0x7822, 0x7822, PDF_CMAP_SINGLE, 7529 },
	{ 0x7823, 0x7823, PDF_CMAP_SINGLE, 7527 },
	{ 0x7825, 0x7825, PDF_CMAP_SINGLE, 2384 },
	{ 0x7826, 0x7826, PDF_CMAP_SINGLE, 8123 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 2379 },
	{ 0x7828, 0x7828, PDF_CMAP_SINGLE, 7532 },
	{ 0x7829, 0x7829, PDF_CMAP_SINGLE, 7536 },
	{ 0x782a, 0x782a, PDF_CMAP_SINGLE, 7538 },
	{ 0x782b, 0x782b, PDF_CMAP_SINGLE, 7534 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 7528 },
	{ 0x782d, 0x782d, PDF_CMAP_SINGLE, 2385 },
	{ 0x782e, 0x782e, PDF_CMAP_SINGLE, 7533 },
	{ 0x782f, 0x782f, PDF_CMAP_SINGLE, 7531 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 2378 },
	{ 0x7831, 0x7831, PDF_CMAP_SINGLE, 7539 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 2388 },
	{ 0x7833, 0x7833, PDF_CMAP_SINGLE, 7537 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 2382 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 7530 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 2383 },
	{ 0x7838, 0x7838, PDF_CMAP_SINGLE, 2380 },
	{ 0x7839, 0x7839, PDF_CMAP_SINGLE, 17366 },
	{ 0x783c, 0x783c, PDF_CMAP_SINGLE, 16410 },
	{ 0x783d, 0x783d, PDF_CMAP_SINGLE, 15466 },
	{ 0x7842, 0x7842, PDF_CMAP_SINGLE, 15044 },
	{ 0x7843, 0x7843, PDF_CMAP_SINGLE, 2880 },
	{ 0x7844, 0x7844, PDF_CMAP_SINGLE, 15403 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 8124 },
	{ 0x7847, 0x7847, PDF_CMAP_SINGLE, 18197 },
	{ 0x7848, 0x7848, PDF_CMAP_SINGLE, 8117 },
	{ 0x7849, 0x7849, PDF_CMAP_SINGLE, 8119 },
	{ 0x784a, 0x784a, PDF_CMAP_SINGLE, 8121 },
	{ 0x784b, 0x784b, PDF_CMAP_SINGLE, 15469 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 8122 },
	{ 0x784d, 0x784d, PDF_CMAP_SINGLE, 8120 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 2881 },
	{ 0x7850, 0x7850, PDF_CMAP_SINGLE, 8125 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 18198 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 8118 },
	{ 0x7853, 0x7853, PDF_CMAP_SINGLE, 16277 },
	{ 0x7854, 0x7854, PDF_CMAP_SINGLE, 15467 },
	{ 0x785c, 0x785c, PDF_CMAP_SINGLE, 8765 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3318 },
	{ 0x785e, 0x785e, PDF_CMAP_SINGLE, 8773 },
	{ 0x7860, 0x7860, PDF_CMAP_SINGLE, 8762 },
	{ 0x7862, 0x7862, PDF_CMAP_SINGLE, 8774 },
	{ 0x7864, 0x7865, PDF_CMAP_RANGE, 8763 },
	{ 0x7866, 0x7866, PDF_CMAP_SINGLE, 18199 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 8772 },
	{ 0x7869, 0x7869, PDF_CMAP_SINGLE, 8771 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 8768 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 2879 },
	{ 0x786c, 0x786c, PDF_CMAP_SINGLE, 3319 },
	{ 0x786d, 0x786d, PDF_CMAP_SINGLE, 8766 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 8769 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 3320 },
	{ 0x7870, 0x7870, PDF_CMAP_SINGLE, 8770 },
	{ 0x7871, 0x7871, PDF_CMAP_SINGLE, 8767 },
	{ 0x7879, 0x7879, PDF_CMAP_SINGLE, 9431 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 16105 },
	{ 0x787b, 0x787b, PDF_CMAP_SINGLE, 9435 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 3749 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 10061 },
	{ 0x787f, 0x787f, PDF_CMAP_SINGLE, 3752 },
	{ 0x7880, 0x7880, PDF_CMAP_SINGLE, 9433 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 14056 },
	{ 0x7883, 0x7883, PDF_CMAP_SINGLE, 9430 },
	{ 0x7884, 0x7884, PDF_CMAP_SINGLE, 9425 },
	{ 0x7885, 0x7886, PDF_CMAP_RANGE, 9427 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 9421 },
	{ 0x7888, 0x7888, PDF_CMAP_SINGLE, 15125 },
	{ 0x7889, 0x7889, PDF_CMAP_SINGLE, 3748 },
	{ 0x788c, 0x788c, PDF_CMAP_SINGLE, 3747 },
	{ 0x788d, 0x788d, PDF_CMAP_SINGLE, 15124 },
	{ 0x788e, 0x788e, PDF_CMAP_SINGLE, 3743 },
	{ 0x788f, 0x788f, PDF_CMAP_SINGLE, 9424 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3750 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 3751 },
	{ 0x7894, 0x7894, PDF_CMAP_SINGLE, 9423 },
	{ 0x7895, 0x7895, PDF_CMAP_SINGLE, 9426 },
	{ 0x7896, 0x7896, PDF_CMAP_SINGLE, 9434 },
	{ 0x7897, 0x7898, PDF_CMAP_RANGE, 3745 },
	{ 0x7899, 0x7899, PDF_CMAP_SINGLE, 9432 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 9422 },
	{ 0x789e, 0x789e, PDF_CMAP_SINGLE, 10063 },
	{ 0x789f, 0x789f, PDF_CMAP_SINGLE, 4159 },
	{ 0x78a0, 0x78a0, PDF_CMAP_SINGLE, 10065 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 9429 },
	{ 0x78a2, 0x78a2, PDF_CMAP_SINGLE, 10067 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 4163 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 10068 },
	{ 0x78a5, 0x78a5, PDF_CMAP_SINGLE, 10064 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 4160 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 10060 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 4162 },
	{ 0x78aa, 0x78aa, PDF_CMAP_SINGLE, 10057 },
	{ 0x78ab, 0x78ab, PDF_CMAP_SINGLE, 10062 },
	{ 0x78ac, 0x78ac, PDF_CMAP_SINGLE, 10066 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 10059 },
	{ 0x78af, 0x78af, PDF_CMAP_SINGLE, 17132 },
	{ 0x78b0, 0x78b0, PDF_CMAP_SINGLE, 3744 },
	{ 0x78b1, 0x78b1, PDF_CMAP_SINGLE, 17140 },
	{ 0x78b2, 0x78b2, PDF_CMAP_SINGLE, 10056 },
	{ 0x78b3, 0x78b3, PDF_CMAP_SINGLE, 4161 },
	{ 0x78b4, 0x78b4, PDF_CMAP_SINGLE, 10058 },
	{ 0x78b6, 0x78b6, PDF_CMAP_SINGLE, 15468 },
	{ 0x78b8, 0x78b8, PDF_CMAP_SINGLE, 17777 },
	{ 0x78b9, 0x78b9, PDF_CMAP_SINGLE, 15459 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 4541 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 10642 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 4545 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 4543 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 4158 },
	{ 0x78c3, 0x78c4, PDF_CMAP_RANGE, 10649 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 4540 },
	{ 0x78c7, 0x78c7, PDF_CMAP_SINGLE, 17133 },
	{ 0x78c8, 0x78c8, PDF_CMAP_SINGLE, 10648 },
	{ 0x78c9, 0x78c9, PDF_CMAP_SINGLE, 10651 },
	{ 0x78ca, 0x78ca, PDF_CMAP_SINGLE, 4542 },
	{ 0x78cb, 0x78cb, PDF_CMAP_SINGLE, 4539 },
	{ 0x78cc, 0x78cc, PDF_CMAP_SINGLE, 10644 },
	{ 0x78cd, 0x78cd, PDF_CMAP_SINGLE, 10641 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 10646 },
	{ 0x78cf, 0x78cf, PDF_CMAP_SINGLE, 10643 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 4546 },
	{ 0x78d1, 0x78d1, PDF_CMAP_SINGLE, 10645 },
	{ 0x78d2, 0x78d2, PDF_CMAP_SINGLE, 15126 },
	{ 0x78d3, 0x78d3, PDF_CMAP_SINGLE, 17134 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 10647 },
	{ 0x78d5, 0x78d5, PDF_CMAP_SINGLE, 4544 },
	{ 0x78d7, 0x78d7, PDF_CMAP_SINGLE, 17138 },
	{ 0x78d8, 0x78d8, PDF_CMAP_SINGLE, 16268 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 4876 },
	{ 0x78db, 0x78db, PDF_CMAP_SINGLE, 11247 },
	{ 0x78dd, 0x78dd, PDF_CMAP_SINGLE, 11241 },
	{ 0x78de, 0x78de, PDF_CMAP_SINGLE, 11245 },
	{ 0x78df, 0x78e0, PDF_CMAP_RANGE, 11251 },
	{ 0x78e1, 0x78e2, PDF_CMAP_RANGE, 11248 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 11246 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 18874 },
	{ 0x78e5, 0x78e5, PDF_CMAP_SINGLE, 11243 },
	{ 0x78e7, 0x78e7, PDF_CMAP_SINGLE, 4878 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 4875 },
	{ 0x78e9, 0x78e9, PDF_CMAP_SINGLE, 11242 },
	{ 0x78ea, 0x78ea, PDF_CMAP_SINGLE, 11244 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 4877 },
	{ 0x78ed, 0x78ed, PDF_CMAP_SINGLE, 11250 },
	{ 0x78ee, 0x78ee, PDF_CMAP_SINGLE, 14977 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 5154 },
	{ 0x78f0, 0x78f0, PDF_CMAP_SINGLE, 15162 },
	{ 0x78f1, 0x78f1, PDF_CMAP_SINGLE, 16574 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 11720 },
	{ 0x78f3, 0x78f3, PDF_CMAP_SINGLE, 11714 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 5153 },
	{ 0x78f5, 0x78f5, PDF_CMAP_SINGLE, 14515 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 5151 },
	{ 0x78f9, 0x78f9, PDF_CMAP_SINGLE, 11722 },
	{ 0x78fa, 0x78fa, PDF_CMAP_SINGLE, 5152 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 11717 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 11715 },
	{ 0x78fe, 0x78fe, PDF_CMAP_SINGLE, 11723 },
	{ 0x78ff, 0x78ff, PDF_CMAP_SINGLE, 11719 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 5155 },
	{ 0x7902, 0x7902, PDF_CMAP_SINGLE, 11716 },
	{ 0x7904, 0x7904, PDF_CMAP_SINGLE, 11724 },
	{ 0x7905, 0x7905, PDF_CMAP_SINGLE, 11721 },
	{ 0x7906, 0x7906, PDF_CMAP_SINGLE, 16335 },
	{ 0x7909, 0x7909, PDF_CMAP_SINGLE, 12143 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 12140 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 5374 },
	{ 0x7910, 0x7910, PDF_CMAP_SINGLE, 12144 },
	{ 0x7911, 0x7911, PDF_CMAP_SINGLE, 12146 },
	{ 0x7912, 0x7912, PDF_CMAP_SINGLE, 12145 },
	{ 0x7913, 0x7914, PDF_CMAP_RANGE, 12141 },
	{ 0x7917, 0x7917, PDF_CMAP_SINGLE, 12494 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5535 },
	{ 0x791b, 0x791b, PDF_CMAP_SINGLE, 12491 },
	{ 0x791c, 0x791c, PDF_CMAP_SINGLE, 12493 },
	{ 0x791d, 0x791d, PDF_CMAP_SINGLE, 12490 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 12495 },
	{ 0x7921, 0x7921, PDF_CMAP_SINGLE, 12492 },
	{ 0x7923, 0x7923, PDF_CMAP_SINGLE, 12804 },
	{ 0x7924, 0x7924, PDF_CMAP_SINGLE, 12807 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 12803 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 5668 },
	{ 0x7927, 0x7928, PDF_CMAP_RANGE, 12805 },
	{ 0x7929, 0x7929, PDF_CMAP_SINGLE, 12808 },
	{ 0x792a, 0x792a, PDF_CMAP_SINGLE, 5669 },
	{ 0x792b, 0x792b, PDF_CMAP_SINGLE, 5671 },
	{ 0x792c, 0x792c, PDF_CMAP_SINGLE, 5670 },
	{ 0x792d, 0x792d, PDF_CMAP_SINGLE, 13030 },
	{ 0x792e, 0x792e, PDF_CMAP_SINGLE, 17136 },
	{ 0x792f, 0x792f, PDF_CMAP_SINGLE, 13032 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 13031 },
	{ 0x7932, 0x7932, PDF_CMAP_SINGLE, 18203 },
	{ 0x7933, 0x7933, PDF_CMAP_SINGLE, 18202 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 17139 },
	{ 0x7935, 0x7935, PDF_CMAP_SINGLE, 13207 },
	{ 0x7936, 0x7936, PDF_CMAP_SINGLE, 14211 },
	{ 0x7938, 0x7938, PDF_CMAP_SINGLE, 13456 },
	{ 0x7939, 0x7939, PDF_CMAP_SINGLE, 13525 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 874 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 17655 },
	{ 0x793c, 0x793c, PDF_CMAP_SINGLE, 17152 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 6359 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 1631 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 6688 },
	{ 0x7940, 0x7941, PDF_CMAP_RANGE, 1632 },
	{ 0x7942, 0x7942, PDF_CMAP_SINGLE, 6687 },
	{ 0x7944, 0x7944, PDF_CMAP_SINGLE, 7087 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 7086 },
	{ 0x7946, 0x7946, PDF_CMAP_SINGLE, 1983 },
	{ 0x7947, 0x7947, PDF_CMAP_SINGLE, 1986 },
	{ 0x7948, 0x7948, PDF_CMAP_SINGLE, 1985 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 1984 },
	{ 0x794a, 0x794a, PDF_CMAP_SINGLE, 7083 },
	{ 0x794b, 0x794b, PDF_CMAP_SINGLE, 7085 },
	{ 0x794c, 0x794c, PDF_CMAP_SINGLE, 7084 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 7542 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 2390 },
	{ 0x7951, 0x7951, PDF_CMAP_SINGLE, 7546 },
	{ 0x7952, 0x7952, PDF_CMAP_SINGLE, 7545 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 7544 },
	{ 0x7954, 0x7954, PDF_CMAP_SINGLE, 7540 },
	{ 0x7955, 0x7955, PDF_CMAP_SINGLE, 2389 },
	{ 0x7956, 0x7956, PDF_CMAP_SINGLE, 2393 },
	{ 0x7957, 0x7957, PDF_CMAP_SINGLE, 2396 },
	{ 0x7958, 0x7958, PDF_CMAP_SINGLE, 14309 },
	{ 0x7959, 0x7959, PDF_CMAP_SINGLE, 15128 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 2397 },
	{ 0x795b, 0x795b, PDF_CMAP_SINGLE, 7541 },
	{ 0x795c, 0x795c, PDF_CMAP_SINGLE, 7543 },
	{ 0x795d, 0x795d, PDF_CMAP_SINGLE, 2395 },
	{ 0x795e, 0x795e, PDF_CMAP_SINGLE, 2394 },
	{ 0x795f, 0x795f, PDF_CMAP_SINGLE, 2392 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 2391 },
	{ 0x7961, 0x7961, PDF_CMAP_SINGLE, 8132 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 15997 },
	{ 0x7963, 0x7963, PDF_CMAP_SINGLE, 8130 },
	{ 0x7964, 0x7964, PDF_CMAP_SINGLE, 8126 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 2882 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 8127 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 2883 },
	{ 0x7969, 0x796a, PDF_CMAP_RANGE, 8128 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 8131 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2884 },
	{ 0x7970, 0x7970, PDF_CMAP_SINGLE, 8778 },
	{ 0x7971, 0x7971, PDF_CMAP_SINGLE, 16744 },
	{ 0x7972, 0x7972, PDF_CMAP_SINGLE, 8777 },
	{ 0x7973, 0x7973, PDF_CMAP_SINGLE, 8776 },
	{ 0x7974, 0x7974, PDF_CMAP_SINGLE, 8775 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 9439 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 3753 },
	{ 0x797c, 0x797c, PDF_CMAP_SINGLE, 9436 },
	{ 0x797d, 0x797d, PDF_CMAP_SINGLE, 9438 },
	{ 0x797e, 0x797e, PDF_CMAP_SINGLE, 15910 },
	{ 0x797f, 0x797f, PDF_CMAP_SINGLE, 3754 },
	{ 0x7980, 0x7980, PDF_CMAP_SINGLE, 17148 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 3755 },
	{ 0x7982, 0x7982, PDF_CMAP_SINGLE, 9437 },
	{ 0x7983, 0x7983, PDF_CMAP_SINGLE, 15862 },
	{ 0x7986, 0x7986, PDF_CMAP_SINGLE, 17145 },
	{ 0x7987, 0x7987, PDF_CMAP_SINGLE, 17800 },
	{ 0x7988, 0x7988, PDF_CMAP_SINGLE, 10077 },
	{ 0x798a, 0x798b, PDF_CMAP_RANGE, 10070 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 4166 },
	{ 0x798e, 0x798f, PDF_CMAP_RANGE, 4164 },
	{ 0x7990, 0x7990, PDF_CMAP_SINGLE, 10079 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 18205 },
	{ 0x7992, 0x7992, PDF_CMAP_SINGLE, 10078 },
	{ 0x7993, 0x7993, PDF_CMAP_SINGLE, 10075 },
	{ 0x7994, 0x7994, PDF_CMAP_SINGLE, 10074 },
	{ 0x7995, 0x7995, PDF_CMAP_SINGLE, 10073 },
	{ 0x7996, 0x7996, PDF_CMAP_SINGLE, 10072 },
	{ 0x7997, 0x7997, PDF_CMAP_SINGLE, 10076 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 10069 },
	{ 0x7999, 0x7999, PDF_CMAP_SINGLE, 17404 },
	{ 0x799a, 0x799a, PDF_CMAP_SINGLE, 10652 },
	{ 0x799b, 0x799b, PDF_CMAP_SINGLE, 10657 },
	{ 0x799c, 0x799c, PDF_CMAP_SINGLE, 10655 },
	{ 0x799d, 0x799d, PDF_CMAP_SINGLE, 17150 },
	{ 0x799f, 0x799f, PDF_CMAP_SINGLE, 14686 },
	{ 0x79a0, 0x79a0, PDF_CMAP_SINGLE, 10654 },
	{ 0x79a1, 0x79a1, PDF_CMAP_SINGLE, 10653 },
	{ 0x79a2, 0x79a2, PDF_CMAP_SINGLE, 10656 },
	{ 0x79a4, 0x79a4, PDF_CMAP_SINGLE, 11253 },
	{ 0x79a5, 0x79a5, PDF_CMAP_SINGLE, 15134 },
	{ 0x79a6, 0x79a6, PDF_CMAP_SINGLE, 4879 },
	{ 0x79a7, 0x79a7, PDF_CMAP_SINGLE, 5156 },
	{ 0x79a8, 0x79a8, PDF_CMAP_SINGLE, 11726 },
	{ 0x79a9, 0x79a9, PDF_CMAP_SINGLE, 17153 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5157 },
	{ 0x79ab, 0x79ab, PDF_CMAP_SINGLE, 11725 },
	{ 0x79ac, 0x79ac, PDF_CMAP_SINGLE, 12148 },
	{ 0x79ad, 0x79ad, PDF_CMAP_SINGLE, 12147 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 5375 },
	{ 0x79b0, 0x79b0, PDF_CMAP_SINGLE, 12496 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 5536 },
	{ 0x79b2, 0x79b2, PDF_CMAP_SINGLE, 12809 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5839 },
	{ 0x79b4, 0x79b4, PDF_CMAP_SINGLE, 13208 },
	{ 0x79b6, 0x79b6, PDF_CMAP_SINGLE, 13458 },
	{ 0x79b7, 0x79b7, PDF_CMAP_SINGLE, 13457 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 6066 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 1987 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 8133 },
	{ 0x79bd, 0x79bd, PDF_CMAP_SINGLE, 3757 },
	{ 0x79be, 0x79be, PDF_CMAP_SINGLE, 875 },
	{ 0x79bf, 0x79bf, PDF_CMAP_SINGLE, 1273 },
	{ 0x79c0, 0x79c0, PDF_CMAP_SINGLE, 1272 },
	{ 0x79c1, 0x79c1, PDF_CMAP_SINGLE, 1271 },
	{ 0x79c4, 0x79c4, PDF_CMAP_SINGLE, 15565 },
	{ 0x79c5, 0x79c5, PDF_CMAP_SINGLE, 6689 },
	{ 0x79c6, 0x79c6, PDF_CMAP_SINGLE, 17157 },
	{ 0x79c8, 0x79c8, PDF_CMAP_SINGLE, 1635 },
	{ 0x79c9, 0x79c9, PDF_CMAP_SINGLE, 1634 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 19025 },
	{ 0x79cb, 0x79cb, PDF_CMAP_SINGLE, 1991 },
	{ 0x79cc, 0x79cc, PDF_CMAP_SINGLE, 16947 },
	{ 0x79cd, 0x79cd, PDF_CMAP_SINGLE, 7089 },
	{ 0x79ce, 0x79ce, PDF_CMAP_SINGLE, 7092 },
	{ 0x79cf, 0x79cf, PDF_CMAP_SINGLE, 7090 },
	{ 0x79d0, 0x79d0, PDF_CMAP_SINGLE, 19111 },
	{ 0x79d1, 0x79d2, PDF_CMAP_RANGE, 1989 },
	{ 0x79d4, 0x79d4, PDF_CMAP_SINGLE, 17159 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 7088 },
	{ 0x79d6, 0x79d6, PDF_CMAP_SINGLE, 7091 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2404 },
	{ 0x79dc, 0x79dc, PDF_CMAP_SINGLE, 7553 },
	{ 0x79dd, 0x79dd, PDF_CMAP_SINGLE, 7555 },
	{ 0x79de, 0x79de, PDF_CMAP_SINGLE, 7554 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 2401 },
	{ 0x79e0, 0x79e0, PDF_CMAP_SINGLE, 7549 },
	{ 0x79e2, 0x79e2, PDF_CMAP_SINGLE, 15450 },
	{ 0x79e3, 0x79e3, PDF_CMAP_SINGLE, 2399 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 2398 },
	{ 0x79e6, 0x79e6, PDF_CMAP_SINGLE, 2402 },
	{ 0x79e7, 0x79e7, PDF_CMAP_SINGLE, 2400 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 2403 },
	{ 0x79ea, 0x79ea, PDF_CMAP_SINGLE, 7552 },
	{ 0x79eb, 0x79ec, PDF_CMAP_RANGE, 7547 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 7551 },
	{ 0x79ee, 0x79ee, PDF_CMAP_SINGLE, 7550 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 15143 },
	{ 0x79f4, 0x79f4, PDF_CMAP_SINGLE, 15138 },
	{ 0x79f6, 0x79f7, PDF_CMAP_RANGE, 8136 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 8135 },
	{ 0x79fa, 0x79fa, PDF_CMAP_SINGLE, 8134 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 2885 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 3325 },
	{ 0x7a02, 0x7a02, PDF_CMAP_SINGLE, 8779 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 8781 },
	{ 0x7a04, 0x7a04, PDF_CMAP_SINGLE, 8783 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 3324 },
	{ 0x7a06, 0x7a06, PDF_CMAP_SINGLE, 18206 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 3322 },
	{ 0x7a0a, 0x7a0a, PDF_CMAP_SINGLE, 8780 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 3323 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 8782 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 3321 },
	{ 0x7a10, 0x7a10, PDF_CMAP_SINGLE, 9449 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 9440 },
	{ 0x7a12, 0x7a12, PDF_CMAP_SINGLE, 9443 },
	{ 0x7a13, 0x7a13, PDF_CMAP_SINGLE, 9447 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 3761 },
	{ 0x7a15, 0x7a15, PDF_CMAP_SINGLE, 9445 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 9444 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 9441 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 3759 },
	{ 0x7a1b, 0x7a1b, PDF_CMAP_SINGLE, 9448 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 3758 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 3763 },
	{ 0x7a1f, 0x7a1f, PDF_CMAP_SINGLE, 3762 },
	{ 0x7a20, 0x7a20, PDF_CMAP_SINGLE, 3760 },
	{ 0x7a22, 0x7a22, PDF_CMAP_SINGLE, 9446 },
	{ 0x7a26, 0x7a26, PDF_CMAP_SINGLE, 10085 },
	{ 0x7a28, 0x7a28, PDF_CMAP_SINGLE, 10084 },
	{ 0x7a2a, 0x7a2a, PDF_CMAP_SINGLE, 18883 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 10080 },
	{ 0x7a2c, 0x7a2c, PDF_CMAP_SINGLE, 19119 },
	{ 0x7a2d, 0x7a2d, PDF_CMAP_SINGLE, 16346 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 4167 },
	{ 0x7a2f, 0x7a2f, PDF_CMAP_SINGLE, 10083 },
	{ 0x7a30, 0x7a30, PDF_CMAP_SINGLE, 10082 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 4168 },
	{ 0x7a32, 0x7a32, PDF_CMAP_SINGLE, 19112 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 4551 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 10659 },
	{ 0x7a3a, 0x7a3a, PDF_CMAP_SINGLE, 15137 },
	{ 0x7a3b, 0x7a3b, PDF_CMAP_SINGLE, 4552 },
	{ 0x7a3c, 0x7a3c, PDF_CMAP_SINGLE, 4548 },
	{ 0x7a3d, 0x7a3d, PDF_CMAP_SINGLE, 4550 },
	{ 0x7a3e, 0x7a3e, PDF_CMAP_SINGLE, 16267 },
	{ 0x7a3f, 0x7a3f, PDF_CMAP_SINGLE, 4547 },
	{ 0x7a40, 0x7a40, PDF_CMAP_SINGLE, 4549 },
	{ 0x7a43, 0x7a43, PDF_CMAP_SINGLE, 14700 },
	{ 0x7a44, 0x7a44, PDF_CMAP_SINGLE, 11254 },
	{ 0x7a45, 0x7a45, PDF_CMAP_SINGLE, 15857 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 4882 },
	{ 0x7a47, 0x7a47, PDF_CMAP_SINGLE, 11256 },
	{ 0x7a48, 0x7a48, PDF_CMAP_SINGLE, 11255 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 14131 },
	{ 0x7a4a, 0x7a4a, PDF_CMAP_SINGLE, 10081 },
	{ 0x7a4b, 0x7a4b, PDF_CMAP_SINGLE, 4884 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 4883 },
	{ 0x7a4d, 0x7a4e, PDF_CMAP_RANGE, 4880 },
	{ 0x7a54, 0x7a54, PDF_CMAP_SINGLE, 11731 },
	{ 0x7a56, 0x7a56, PDF_CMAP_SINGLE, 11729 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5158 },
	{ 0x7a58, 0x7a58, PDF_CMAP_SINGLE, 11730 },
	{ 0x7a5a, 0x7a5a, PDF_CMAP_SINGLE, 11732 },
	{ 0x7a5b, 0x7a5b, PDF_CMAP_SINGLE, 11728 },
	{ 0x7a5c, 0x7a5c, PDF_CMAP_SINGLE, 11727 },
	{ 0x7a5f, 0x7a5f, PDF_CMAP_SINGLE, 12149 },
	{ 0x7a60, 0x7a60, PDF_CMAP_SINGLE, 5378 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5376 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 14134 },
	{ 0x7a67, 0x7a68, PDF_CMAP_RANGE, 12497 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5538 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 5537 },
	{ 0x7a6c, 0x7a6d, PDF_CMAP_RANGE, 12811 },
	{ 0x7a6e, 0x7a6e, PDF_CMAP_SINGLE, 12810 },
	{ 0x7a70, 0x7a71, PDF_CMAP_RANGE, 13209 },
	{ 0x7a72, 0x7a72, PDF_CMAP_SINGLE, 19133 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 876 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 6161 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 1274 },
	{ 0x7a78, 0x7a78, PDF_CMAP_SINGLE, 6690 },
	{ 0x7a79, 0x7a79, PDF_CMAP_SINGLE, 1637 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 1636 },
	{ 0x7a7b, 0x7a7b, PDF_CMAP_SINGLE, 6691 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 14135 },
	{ 0x7a7e, 0x7a7e, PDF_CMAP_SINGLE, 7094 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 1992 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 7093 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 1993 },
	{ 0x7a83, 0x7a83, PDF_CMAP_SINGLE, 15741 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 2405 },
	{ 0x7a85, 0x7a85, PDF_CMAP_SINGLE, 7558 },
	{ 0x7a86, 0x7a86, PDF_CMAP_SINGLE, 7556 },
	{ 0x7a87, 0x7a87, PDF_CMAP_SINGLE, 7562 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 2406 },
	{ 0x7a89, 0x7a89, PDF_CMAP_SINGLE, 7557 },
	{ 0x7a8a, 0x7a8a, PDF_CMAP_SINGLE, 7561 },
	{ 0x7a8b, 0x7a8c, PDF_CMAP_RANGE, 7559 },
	{ 0x7a8f, 0x7a8f, PDF_CMAP_SINGLE, 8138 },
	{ 0x7a90, 0x7a90, PDF_CMAP_SINGLE, 8140 },
	{ 0x7a91, 0x7a91, PDF_CMAP_SINGLE, 17368 },
	{ 0x7a92, 0x7a92, PDF_CMAP_SINGLE, 2886 },
	{ 0x7a93, 0x7a93, PDF_CMAP_SINGLE, 18926 },
	{ 0x7a94, 0x7a94, PDF_CMAP_SINGLE, 8139 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 2887 },
	{ 0x7a96, 0x7a96, PDF_CMAP_SINGLE, 3328 },
	{ 0x7a97, 0x7a97, PDF_CMAP_SINGLE, 3327 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 3326 },
	{ 0x7a99, 0x7a99, PDF_CMAP_SINGLE, 8784 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 9452 },
	{ 0x7a9f, 0x7aa0, PDF_CMAP_RANGE, 3764 },
	{ 0x7aa2, 0x7aa2, PDF_CMAP_SINGLE, 9451 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 9450 },
	{ 0x7aa8, 0x7aa8, PDF_CMAP_SINGLE, 10086 },
	{ 0x7aa9, 0x7aa9, PDF_CMAP_SINGLE, 4170 },
	{ 0x7aaa, 0x7aaa, PDF_CMAP_SINGLE, 4169 },
	{ 0x7aab, 0x7aac, PDF_CMAP_RANGE, 10087 },
	{ 0x7aae, 0x7aae, PDF_CMAP_SINGLE, 4554 },
	{ 0x7aaf, 0x7aaf, PDF_CMAP_SINGLE, 4553 },
	{ 0x7ab0, 0x7ab0, PDF_CMAP_SINGLE, 14138 },
	{ 0x7ab1, 0x7ab1, PDF_CMAP_SINGLE, 11260 },
	{ 0x7ab2, 0x7ab2, PDF_CMAP_SINGLE, 10660 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 10662 },
	{ 0x7ab4, 0x7ab4, PDF_CMAP_SINGLE, 10661 },
	{ 0x7ab5, 0x7ab5, PDF_CMAP_SINGLE, 11259 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 11257 },
	{ 0x7ab7, 0x7ab7, PDF_CMAP_SINGLE, 11261 },
	{ 0x7ab8, 0x7ab8, PDF_CMAP_SINGLE, 11258 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 4885 },
	{ 0x7abb, 0x7abb, PDF_CMAP_SINGLE, 14137 },
	{ 0x7abc, 0x7abc, PDF_CMAP_SINGLE, 18209 },
	{ 0x7abe, 0x7abe, PDF_CMAP_SINGLE, 11733 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 5159 },
	{ 0x7ac0, 0x7ac1, PDF_CMAP_RANGE, 11734 },
	{ 0x7ac2, 0x7ac3, PDF_CMAP_RANGE, 14139 },
	{ 0x7ac4, 0x7ac5, PDF_CMAP_RANGE, 5379 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 5672 },
	{ 0x7ac8, 0x7ac8, PDF_CMAP_SINGLE, 15743 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 17776 },
	{ 0x7aca, 0x7aca, PDF_CMAP_SINGLE, 5887 },
	{ 0x7acb, 0x7acb, PDF_CMAP_SINGLE, 877 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 18212 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 7095 },
	{ 0x7ad3, 0x7ad3, PDF_CMAP_SINGLE, 16266 },
	{ 0x7ad8, 0x7ad8, PDF_CMAP_SINGLE, 7563 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 2407 },
	{ 0x7ada, 0x7ada, PDF_CMAP_SINGLE, 14144 },
	{ 0x7adb, 0x7adb, PDF_CMAP_SINGLE, 18213 },
	{ 0x7adc, 0x7adc, PDF_CMAP_SINGLE, 14673 },
	{ 0x7add, 0x7add, PDF_CMAP_SINGLE, 14145 },
	{ 0x7adf, 0x7adf, PDF_CMAP_SINGLE, 3038 },
	{ 0x7ae0, 0x7ae0, PDF_CMAP_SINGLE, 3037 },
	{ 0x7ae2, 0x7ae2, PDF_CMAP_SINGLE, 15155 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 3330 },
	{ 0x7ae4, 0x7ae4, PDF_CMAP_SINGLE, 8786 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 3329 },
	{ 0x7ae6, 0x7ae6, PDF_CMAP_SINGLE, 8785 },
	{ 0x7ae7, 0x7ae7, PDF_CMAP_SINGLE, 14429 },
	{ 0x7ae9, 0x7ae9, PDF_CMAP_SINGLE, 14385 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 14146 },
	{ 0x7aeb, 0x7aeb, PDF_CMAP_SINGLE, 9453 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 4171 },
	{ 0x7aee, 0x7aee, PDF_CMAP_SINGLE, 10089 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 4172 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 5673 },
	{ 0x7af7, 0x7af7, PDF_CMAP_SINGLE, 12813 },
	{ 0x7af9, 0x7af9, PDF_CMAP_SINGLE, 1014 },
	{ 0x7afa, 0x7afa, PDF_CMAP_SINGLE, 1638 },
	{ 0x7afb, 0x7afb, PDF_CMAP_SINGLE, 6692 },
	{ 0x7afc, 0x7afc, PDF_CMAP_SINGLE, 19095 },
	{ 0x7afd, 0x7afd, PDF_CMAP_SINGLE, 1995 },
	{ 0x7afe, 0x7afe, PDF_CMAP_SINGLE, 15165 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1994 },
	{ 0x7b00, 0x7b01, PDF_CMAP_RANGE, 7096 },
	{ 0x7b04, 0x7b04, PDF_CMAP_SINGLE, 7565 },
	{ 0x7b05, 0x7b05, PDF_CMAP_SINGLE, 7567 },
	{ 0x7b06, 0x7b06, PDF_CMAP_SINGLE, 2408 },
	{ 0x7b08, 0x7b08, PDF_CMAP_SINGLE, 7569 },
	{ 0x7b09, 0x7b09, PDF_CMAP_SINGLE, 7572 },
	{ 0x7b0a, 0x7b0a, PDF_CMAP_SINGLE, 7570 },
	{ 0x7b0b, 0x7b0b, PDF_CMAP_SINGLE, 14150 },
	{ 0x7b0c, 0x7b0c, PDF_CMAP_SINGLE, 15203 },
	{ 0x7b0e, 0x7b0e, PDF_CMAP_SINGLE, 7571 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 7568 },
	{ 0x7b10, 0x7b10, PDF_CMAP_SINGLE, 7564 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 2409 },
	{ 0x7b12, 0x7b12, PDF_CMAP_SINGLE, 7573 },
	{ 0x7b13, 0x7b13, PDF_CMAP_SINGLE, 7566 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 16247 },
	{ 0x7b18, 0x7b18, PDF_CMAP_SINGLE, 8149 },
	{ 0x7b19, 0x7b19, PDF_CMAP_SINGLE, 2893 },
	{ 0x7b1a, 0x7b1a, PDF_CMAP_SINGLE, 8158 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 2890 },
	{ 0x7b1d, 0x7b1d, PDF_CMAP_SINGLE, 8151 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 2894 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 16145 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 2888 },
	{ 0x7b22, 0x7b22, PDF_CMAP_SINGLE, 8146 },
	{ 0x7b23, 0x7b23, PDF_CMAP_SINGLE, 8159 },
	{ 0x7b24, 0x7b24, PDF_CMAP_SINGLE, 8147 },
	{ 0x7b25, 0x7b25, PDF_CMAP_SINGLE, 8144 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 2892 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 15199 },
	{ 0x7b28, 0x7b28, PDF_CMAP_SINGLE, 2889 },
	{ 0x7b29, 0x7b29, PDF_CMAP_SINGLE, 14152 },
	{ 0x7b2a, 0x7b2a, PDF_CMAP_SINGLE, 8150 },
	{ 0x7b2b, 0x7b2b, PDF_CMAP_SINGLE, 8153 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 2891 },
	{ 0x7b2d, 0x7b2d, PDF_CMAP_SINGLE, 8154 },
	{ 0x7b2e, 0x7b2e, PDF_CMAP_SINGLE, 2895 },
	{ 0x7b2f, 0x7b2f, PDF_CMAP_SINGLE, 8155 },
	{ 0x7b30, 0x7b30, PDF_CMAP_SINGLE, 8145 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 8152 },
	{ 0x7b32, 0x7b32, PDF_CMAP_SINGLE, 8156 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 8148 },
	{ 0x7b34, 0x7b34, PDF_CMAP_SINGLE, 8143 },
	{ 0x7b35, 0x7b35, PDF_CMAP_SINGLE, 8141 },
	{ 0x7b38, 0x7b38, PDF_CMAP_SINGLE, 8157 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 15726 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 8142 },
	{ 0x7b40, 0x7b40, PDF_CMAP_SINGLE, 8793 },
	{ 0x7b42, 0x7b42, PDF_CMAP_SINGLE, 15853 },
	{ 0x7b43, 0x7b43, PDF_CMAP_SINGLE, 15909 },
	{ 0x7b44, 0x7b44, PDF_CMAP_SINGLE, 8789 },
	{ 0x7b45, 0x7b45, PDF_CMAP_SINGLE, 8795 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 3333 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 8788 },
	{ 0x7b48, 0x7b48, PDF_CMAP_SINGLE, 8790 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 3331 },
	{ 0x7b4a, 0x7b4a, PDF_CMAP_SINGLE, 8787 },
	{ 0x7b4b, 0x7b4b, PDF_CMAP_SINGLE, 3338 },
	{ 0x7b4c, 0x7b4c, PDF_CMAP_SINGLE, 8791 },
	{ 0x7b4d, 0x7b4d, PDF_CMAP_SINGLE, 3337 },
	{ 0x7b4e, 0x7b4e, PDF_CMAP_SINGLE, 8792 },
	{ 0x7b4f, 0x7b4f, PDF_CMAP_SINGLE, 3339 },
	{ 0x7b50, 0x7b50, PDF_CMAP_SINGLE, 3334 },
	{ 0x7b51, 0x7b51, PDF_CMAP_SINGLE, 3340 },
	{ 0x7b52, 0x7b52, PDF_CMAP_SINGLE, 3335 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 3336 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 14151 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 3332 },
	{ 0x7b58, 0x7b58, PDF_CMAP_SINGLE, 8794 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 3768 },
	{ 0x7b61, 0x7b61, PDF_CMAP_SINGLE, 9464 },
	{ 0x7b62, 0x7b62, PDF_CMAP_SINGLE, 18215 },
	{ 0x7b63, 0x7b63, PDF_CMAP_SINGLE, 9467 },
	{ 0x7b64, 0x7b64, PDF_CMAP_SINGLE, 9455 },
	{ 0x7b65, 0x7b65, PDF_CMAP_SINGLE, 9460 },
	{ 0x7b66, 0x7b66, PDF_CMAP_SINGLE, 9454 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 3770 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 9458 },
	{ 0x7b6c, 0x7b6c, PDF_CMAP_SINGLE, 18216 },
	{ 0x7b6d, 0x7b6d, PDF_CMAP_SINGLE, 9456 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 3769 },
	{ 0x7b6f, 0x7b6f, PDF_CMAP_SINGLE, 14156 },
	{ 0x7b70, 0x7b70, PDF_CMAP_SINGLE, 9463 },
	{ 0x7b71, 0x7b71, PDF_CMAP_SINGLE, 9462 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 9459 },
	{ 0x7b73, 0x7b73, PDF_CMAP_SINGLE, 9461 },
	{ 0x7b74, 0x7b74, PDF_CMAP_SINGLE, 9457 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 4176 },
	{ 0x7b76, 0x7b76, PDF_CMAP_SINGLE, 9466 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 3766 },
	{ 0x7b78, 0x7b78, PDF_CMAP_SINGLE, 9465 },
	{ 0x7b7b, 0x7b7b, PDF_CMAP_SINGLE, 18217 },
	{ 0x7b82, 0x7b82, PDF_CMAP_SINGLE, 10105 },
	{ 0x7b84, 0x7b84, PDF_CMAP_SINGLE, 4183 },
	{ 0x7b85, 0x7b85, PDF_CMAP_SINGLE, 10100 },
	{ 0x7b87, 0x7b87, PDF_CMAP_SINGLE, 4182 },
	{ 0x7b88, 0x7b88, PDF_CMAP_SINGLE, 10090 },
	{ 0x7b8a, 0x7b8a, PDF_CMAP_SINGLE, 10092 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 4175 },
	{ 0x7b8c, 0x7b8c, PDF_CMAP_SINGLE, 10097 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 10096 },
	{ 0x7b8e, 0x7b8e, PDF_CMAP_SINGLE, 10099 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 4180 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 10094 },
	{ 0x7b91, 0x7b91, PDF_CMAP_SINGLE, 10093 },
	{ 0x7b92, 0x7b92, PDF_CMAP_SINGLE, 14162 },
	{ 0x7b94, 0x7b94, PDF_CMAP_SINGLE, 4179 },
	{ 0x7b95, 0x7b95, PDF_CMAP_SINGLE, 4174 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 10095 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 4177 },
	{ 0x7b98, 0x7b98, PDF_CMAP_SINGLE, 10101 },
	{ 0x7b99, 0x7b99, PDF_CMAP_SINGLE, 10103 },
	{ 0x7b9b, 0x7b9b, PDF_CMAP_SINGLE, 10098 },
	{ 0x7b9c, 0x7b9c, PDF_CMAP_SINGLE, 10091 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 4178 },
	{ 0x7ba0, 0x7ba0, PDF_CMAP_SINGLE, 4562 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 4173 },
	{ 0x7ba2, 0x7ba2, PDF_CMAP_SINGLE, 14155 },
	{ 0x7ba3, 0x7ba3, PDF_CMAP_SINGLE, 16148 },
	{ 0x7ba4, 0x7ba4, PDF_CMAP_SINGLE, 10104 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 10666 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 4555 },
	{ 0x7bae, 0x7bae, PDF_CMAP_SINGLE, 19140 },
	{ 0x7baf, 0x7baf, PDF_CMAP_SINGLE, 10668 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 4556 },
	{ 0x7bb2, 0x7bb2, PDF_CMAP_SINGLE, 17948 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 4558 },
	{ 0x7bb5, 0x7bb5, PDF_CMAP_SINGLE, 10671 },
	{ 0x7bb7, 0x7bb7, PDF_CMAP_SINGLE, 10663 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 4181 },
	{ 0x7bb9, 0x7bb9, PDF_CMAP_SINGLE, 10669 },
	{ 0x7bbe, 0x7bbe, PDF_CMAP_SINGLE, 10665 },
	{ 0x7bc0, 0x7bc0, PDF_CMAP_SINGLE, 3767 },
	{ 0x7bc1, 0x7bc1, PDF_CMAP_SINGLE, 4561 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 4557 },
	{ 0x7bc5, 0x7bc5, PDF_CMAP_SINGLE, 19117 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_RANGE, 4559 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 4888 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 10670 },
	{ 0x7bcb, 0x7bcb, PDF_CMAP_SINGLE, 10664 },
	{ 0x7bcc, 0x7bcc, PDF_CMAP_SINGLE, 4563 },
	{ 0x7bce, 0x7bce, PDF_CMAP_SINGLE, 10667 },
	{ 0x7bcf, 0x7bcf, PDF_CMAP_SINGLE, 16152 },
	{ 0x7bd0, 0x7bd0, PDF_CMAP_SINGLE, 14160 },
	{ 0x7bd4, 0x7bd4, PDF_CMAP_SINGLE, 11271 },
	{ 0x7bd5, 0x7bd5, PDF_CMAP_SINGLE, 11266 },
	{ 0x7bd8, 0x7bd8, PDF_CMAP_SINGLE, 11276 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 4886 },
	{ 0x7bda, 0x7bda, PDF_CMAP_SINGLE, 11268 },
	{ 0x7bdb, 0x7bdb, PDF_CMAP_SINGLE, 4890 },
	{ 0x7bdc, 0x7bdc, PDF_CMAP_SINGLE, 11274 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 11265 },
	{ 0x7bde, 0x7bde, PDF_CMAP_SINGLE, 11262 },
	{ 0x7bdf, 0x7bdf, PDF_CMAP_SINGLE, 11277 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 5165 },
	{ 0x7be1, 0x7be1, PDF_CMAP_SINGLE, 4891 },
	{ 0x7be2, 0x7be2, PDF_CMAP_SINGLE, 11273 },
	{ 0x7be3, 0x7be3, PDF_CMAP_SINGLE, 11263 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 4889 },
	{ 0x7be5, 0x7be5, PDF_CMAP_SINGLE, 11267 },
	{ 0x7be6, 0x7be6, PDF_CMAP_SINGLE, 4893 },
	{ 0x7be7, 0x7be7, PDF_CMAP_SINGLE, 11264 },
	{ 0x7be8, 0x7be8, PDF_CMAP_SINGLE, 11269 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 4892 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 11272 },
	{ 0x7beb, 0x7beb, PDF_CMAP_SINGLE, 11275 },
	{ 0x7bec, 0x7bec, PDF_CMAP_SINGLE, 18927 },
	{ 0x7bf0, 0x7bf1, PDF_CMAP_RANGE, 11753 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 11738 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 11745 },
	{ 0x7bf4, 0x7bf4, PDF_CMAP_SINGLE, 11743 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 5163 },
	{ 0x7bf8, 0x7bf8, PDF_CMAP_SINGLE, 11750 },
	{ 0x7bf9, 0x7bf9, PDF_CMAP_SINGLE, 11270 },
	{ 0x7bfa, 0x7bfa, PDF_CMAP_SINGLE, 14167 },
	{ 0x7bfb, 0x7bfb, PDF_CMAP_SINGLE, 11741 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 16159 },
	{ 0x7bfd, 0x7bfd, PDF_CMAP_SINGLE, 11751 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 5162 },
	{ 0x7bff, 0x7bff, PDF_CMAP_SINGLE, 11740 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 11739 },
	{ 0x7c01, 0x7c01, PDF_CMAP_SINGLE, 11749 },
	{ 0x7c02, 0x7c02, PDF_CMAP_SINGLE, 11746 },
	{ 0x7c03, 0x7c03, PDF_CMAP_SINGLE, 11748 },
	{ 0x7c05, 0x7c05, PDF_CMAP_SINGLE, 11736 },
	{ 0x7c06, 0x7c06, PDF_CMAP_SINGLE, 11752 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 5160 },
	{ 0x7c09, 0x7c09, PDF_CMAP_SINGLE, 11747 },
	{ 0x7c0a, 0x7c0a, PDF_CMAP_SINGLE, 11756 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 11744 },
	{ 0x7c0c, 0x7c0c, PDF_CMAP_SINGLE, 5164 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 5161 },
	{ 0x7c0e, 0x7c0e, PDF_CMAP_SINGLE, 11742 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 11737 },
	{ 0x7c10, 0x7c10, PDF_CMAP_SINGLE, 11755 },
	{ 0x7c11, 0x7c11, PDF_CMAP_SINGLE, 4887 },
	{ 0x7c12, 0x7c12, PDF_CMAP_SINGLE, 18218 },
	{ 0x7c15, 0x7c15, PDF_CMAP_SINGLE, 16481 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 12152 },
	{ 0x7c1b, 0x7c1b, PDF_CMAP_SINGLE, 17369 },
	{ 0x7c1c, 0x7c1c, PDF_CMAP_SINGLE, 12150 },
	{ 0x7c1d, 0x7c1d, PDF_CMAP_SINGLE, 12156 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 5384 },
	{ 0x7c1f, 0x7c1f, PDF_CMAP_SINGLE, 12154 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 12153 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 5386 },
	{ 0x7c22, 0x7c22, PDF_CMAP_SINGLE, 12159 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 5385 },
	{ 0x7c25, 0x7c25, PDF_CMAP_SINGLE, 12160 },
	{ 0x7c26, 0x7c26, PDF_CMAP_SINGLE, 12157 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 5382 },
	{ 0x7c28, 0x7c28, PDF_CMAP_SINGLE, 12158 },
	{ 0x7c29, 0x7c29, PDF_CMAP_SINGLE, 12151 },
	{ 0x7c2a, 0x7c2a, PDF_CMAP_SINGLE, 5383 },
	{ 0x7c2b, 0x7c2b, PDF_CMAP_SINGLE, 5381 },
	{ 0x7c2c, 0x7c2c, PDF_CMAP_SINGLE, 12502 },
	{ 0x7c2d, 0x7c2d, PDF_CMAP_SINGLE, 12155 },
	{ 0x7c30, 0x7c30, PDF_CMAP_SINGLE, 12161 },
	{ 0x7c33, 0x7c33, PDF_CMAP_SINGLE, 12499 },
	{ 0x7c35, 0x7c35, PDF_CMAP_SINGLE, 14169 },
	{ 0x7c37, 0x7c37, PDF_CMAP_SINGLE, 5543 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 5541 },
	{ 0x7c39, 0x7c39, PDF_CMAP_SINGLE, 12501 },
	{ 0x7c3b, 0x7c3b, PDF_CMAP_SINGLE, 12503 },
	{ 0x7c3c, 0x7c3c, PDF_CMAP_SINGLE, 12500 },
	{ 0x7c3d, 0x7c3d, PDF_CMAP_SINGLE, 5542 },
	{ 0x7c3e, 0x7c3f, PDF_CMAP_RANGE, 5539 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 5544 },
	{ 0x7c42, 0x7c42, PDF_CMAP_SINGLE, 16156 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 5675 },
	{ 0x7c44, 0x7c44, PDF_CMAP_SINGLE, 14171 },
	{ 0x7c45, 0x7c45, PDF_CMAP_SINGLE, 12818 },
	{ 0x7c47, 0x7c47, PDF_CMAP_SINGLE, 12817 },
	{ 0x7c48, 0x7c48, PDF_CMAP_SINGLE, 12815 },
	{ 0x7c49, 0x7c49, PDF_CMAP_SINGLE, 12814 },
	{ 0x7c4a, 0x7c4a, PDF_CMAP_SINGLE, 12816 },
	{ 0x7c4c, 0x7c4c, PDF_CMAP_SINGLE, 5674 },
	{ 0x7c4d, 0x7c4d, PDF_CMAP_SINGLE, 5676 },
	{ 0x7c50, 0x7c50, PDF_CMAP_SINGLE, 5761 },
	{ 0x7c51, 0x7c51, PDF_CMAP_SINGLE, 16312 },
	{ 0x7c53, 0x7c53, PDF_CMAP_SINGLE, 13034 },
	{ 0x7c54, 0x7c54, PDF_CMAP_SINGLE, 13033 },
	{ 0x7c56, 0x7c56, PDF_CMAP_SINGLE, 16127 },
	{ 0x7c57, 0x7c57, PDF_CMAP_SINGLE, 13211 },
	{ 0x7c59, 0x7c59, PDF_CMAP_SINGLE, 13213 },
	{ 0x7c5a, 0x7c5a, PDF_CMAP_SINGLE, 13215 },
	{ 0x7c5b, 0x7c5b, PDF_CMAP_SINGLE, 13214 },
	{ 0x7c5c, 0x7c5c, PDF_CMAP_SINGLE, 13212 },
	{ 0x7c5d, 0x7c5d, PDF_CMAP_SINGLE, 15167 },
	{ 0x7c5f, 0x7c5f, PDF_CMAP_SINGLE, 5841 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 5840 },
	{ 0x7c63, 0x7c63, PDF_CMAP_SINGLE, 5889 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 5888 },
	{ 0x7c65, 0x7c65, PDF_CMAP_SINGLE, 5890 },
	{ 0x7c66, 0x7c66, PDF_CMAP_SINGLE, 13355 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 13354 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 13526 },
	{ 0x7c6a, 0x7c6a, PDF_CMAP_SINGLE, 13459 },
	{ 0x7c6b, 0x7c6b, PDF_CMAP_SINGLE, 13527 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 5957 },
	{ 0x7c6d, 0x7c6d, PDF_CMAP_SINGLE, 15168 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 5958 },
	{ 0x7c6f, 0x7c6f, PDF_CMAP_SINGLE, 13575 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 16210 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 5995 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 1015 },
	{ 0x7c74, 0x7c74, PDF_CMAP_SINGLE, 18076 },
	{ 0x7c75, 0x7c75, PDF_CMAP_SINGLE, 6693 },
	{ 0x7c78, 0x7c79, PDF_CMAP_RANGE, 7099 },
	{ 0x7c7a, 0x7c7a, PDF_CMAP_SINGLE, 7098 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 18221 },
	{ 0x7c7c, 0x7c7c, PDF_CMAP_SINGLE, 15177 },
	{ 0x7c7d, 0x7c7d, PDF_CMAP_SINGLE, 1996 },
	{ 0x7c7e, 0x7c7e, PDF_CMAP_SINGLE, 16154 },
	{ 0x7c7f, 0x7c81, PDF_CMAP_RANGE, 7101 },
	{ 0x7c83, 0x7c83, PDF_CMAP_SINGLE, 14172 },
	{ 0x7c84, 0x7c84, PDF_CMAP_SINGLE, 7574 },
	{ 0x7c85, 0x7c85, PDF_CMAP_SINGLE, 7580 },
	{ 0x7c86, 0x7c86, PDF_CMAP_SINGLE, 16157 },
	{ 0x7c88, 0x7c88, PDF_CMAP_SINGLE, 7578 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 2410 },
	{ 0x7c8a, 0x7c8a, PDF_CMAP_SINGLE, 7576 },
	{ 0x7c8c, 0x7c8c, PDF_CMAP_SINGLE, 7577 },
	{ 0x7c8d, 0x7c8d, PDF_CMAP_SINGLE, 7579 },
	{ 0x7c8e, 0x7c8e, PDF_CMAP_SINGLE, 15176 },
	{ 0x7c91, 0x7c91, PDF_CMAP_SINGLE, 7575 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 2896 },
	{ 0x7c94, 0x7c94, PDF_CMAP_SINGLE, 8160 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 2898 },
	{ 0x7c96, 0x7c96, PDF_CMAP_SINGLE, 8162 },
	{ 0x7c97, 0x7c97, PDF_CMAP_SINGLE, 2897 },
	{ 0x7c98, 0x7c98, PDF_CMAP_SINGLE, 8161 },
	{ 0x7c9c, 0x7c9c, PDF_CMAP_SINGLE, 18222 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 8797 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 3341 },
	{ 0x7ca1, 0x7ca1, PDF_CMAP_SINGLE, 8799 },
	{ 0x7ca2, 0x7ca2, PDF_CMAP_SINGLE, 8796 },
	{ 0x7ca3, 0x7ca3, PDF_CMAP_SINGLE, 8163 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 3342 },
	{ 0x7ca6, 0x7ca6, PDF_CMAP_SINGLE, 14174 },
	{ 0x7ca7, 0x7ca7, PDF_CMAP_SINGLE, 14061 },
	{ 0x7ca8, 0x7ca8, PDF_CMAP_SINGLE, 8798 },
	{ 0x7cac, 0x7cac, PDF_CMAP_SINGLE, 14469 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 15178 },
	{ 0x7caf, 0x7caf, PDF_CMAP_SINGLE, 9470 },
	{ 0x7cb1, 0x7cb1, PDF_CMAP_SINGLE, 3771 },
	{ 0x7cb2, 0x7cb2, PDF_CMAP_SINGLE, 9468 },
	{ 0x7cb3, 0x7cb3, PDF_CMAP_SINGLE, 3772 },
	{ 0x7cb4, 0x7cb4, PDF_CMAP_SINGLE, 9469 },
	{ 0x7cb5, 0x7cb5, PDF_CMAP_SINGLE, 3773 },
	{ 0x7cb8, 0x7cb8, PDF_CMAP_SINGLE, 18224 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 4184 },
	{ 0x7cba, 0x7cba, PDF_CMAP_SINGLE, 10109 },
	{ 0x7cbb, 0x7cbb, PDF_CMAP_SINGLE, 10106 },
	{ 0x7cbc, 0x7cbc, PDF_CMAP_SINGLE, 10108 },
	{ 0x7cbd, 0x7cbe, PDF_CMAP_RANGE, 4185 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 10107 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 16343 },
	{ 0x7cc3, 0x7cc3, PDF_CMAP_SINGLE, 18928 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 10672 },
	{ 0x7cc7, 0x7cc7, PDF_CMAP_SINGLE, 14177 },
	{ 0x7cc8, 0x7cc8, PDF_CMAP_SINGLE, 10673 },
	{ 0x7cc9, 0x7cc9, PDF_CMAP_SINGLE, 14176 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 4564 },
	{ 0x7ccb, 0x7ccb, PDF_CMAP_SINGLE, 10675 },
	{ 0x7ccc, 0x7ccc, PDF_CMAP_SINGLE, 10674 },
	{ 0x7ccd, 0x7ccd, PDF_CMAP_SINGLE, 15173 },
	{ 0x7cce, 0x7cce, PDF_CMAP_SINGLE, 289 },
	{ 0x7cd0, 0x7cd1, PDF_CMAP_RANGE, 11281 },
	{ 0x7cd2, 0x7cd2, PDF_CMAP_SINGLE, 11278 },
	{ 0x7cd3, 0x7cd3, PDF_CMAP_SINGLE, 15708 },
	{ 0x7cd4, 0x7cd4, PDF_CMAP_SINGLE, 11279 },
	{ 0x7cd5, 0x7cd6, PDF_CMAP_RANGE, 4894 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 11280 },
	{ 0x7cd9, 0x7cd9, PDF_CMAP_SINGLE, 5171 },
	{ 0x7cda, 0x7cda, PDF_CMAP_SINGLE, 16345 },
	{ 0x7cdc, 0x7cdc, PDF_CMAP_SINGLE, 5167 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 5172 },
	{ 0x7cde, 0x7cde, PDF_CMAP_SINGLE, 5168 },
	{ 0x7cdf, 0x7cdf, PDF_CMAP_SINGLE, 5170 },
	{ 0x7ce0, 0x7ce0, PDF_CMAP_SINGLE, 5166 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 5169 },
	{ 0x7ce6, 0x7ce6, PDF_CMAP_SINGLE, 14178 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 5387 },
	{ 0x7ce8, 0x7ce8, PDF_CMAP_SINGLE, 11757 },
	{ 0x7cea, 0x7cea, PDF_CMAP_SINGLE, 12505 },
	{ 0x7cec, 0x7cec, PDF_CMAP_SINGLE, 12504 },
	{ 0x7ced, 0x7ced, PDF_CMAP_SINGLE, 17370 },
	{ 0x7cee, 0x7cee, PDF_CMAP_SINGLE, 12819 },
	{ 0x7cef, 0x7cf0, PDF_CMAP_RANGE, 5677 },
	{ 0x7cf1, 0x7cf1, PDF_CMAP_SINGLE, 13217 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 13035 },
	{ 0x7cf3, 0x7cf3, PDF_CMAP_SINGLE, 14180 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 13216 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 14181 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 13528 },
	{ 0x7cf7, 0x7cf7, PDF_CMAP_SINGLE, 13597 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 1016 },
	{ 0x7cf9, 0x7cf9, PDF_CMAP_SINGLE, 17656 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 1275 },
	{ 0x7cfc, 0x7cfc, PDF_CMAP_SINGLE, 17773 },
	{ 0x7cfd, 0x7cfd, PDF_CMAP_SINGLE, 6694 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 1639 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 1999 },
	{ 0x7d01, 0x7d01, PDF_CMAP_SINGLE, 7106 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 1997 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 7104 },
	{ 0x7d04, 0x7d04, PDF_CMAP_SINGLE, 2002 },
	{ 0x7d05, 0x7d05, PDF_CMAP_SINGLE, 1998 },
	{ 0x7d06, 0x7d06, PDF_CMAP_SINGLE, 2003 },
	{ 0x7d07, 0x7d07, PDF_CMAP_SINGLE, 2001 },
	{ 0x7d08, 0x7d08, PDF_CMAP_SINGLE, 7105 },
	{ 0x7d09, 0x7d09, PDF_CMAP_SINGLE, 2000 },
	{ 0x7d0a, 0x7d0a, PDF_CMAP_SINGLE, 2414 },
	{ 0x7d0b, 0x7d0b, PDF_CMAP_SINGLE, 2413 },
	{ 0x7d0c, 0x7d0c, PDF_CMAP_SINGLE, 7591 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 2422 },
	{ 0x7d0e, 0x7d0e, PDF_CMAP_SINGLE, 7584 },
	{ 0x7d0f, 0x7d0f, PDF_CMAP_SINGLE, 7590 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 2418 },
	{ 0x7d11, 0x7d11, PDF_CMAP_SINGLE, 7583 },
	{ 0x7d12, 0x7d12, PDF_CMAP_SINGLE, 7589 },
	{ 0x7d13, 0x7d13, PDF_CMAP_SINGLE, 7587 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 2417 },
	{ 0x7d15, 0x7d15, PDF_CMAP_SINGLE, 2419 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 7586 },
	{ 0x7d17, 0x7d17, PDF_CMAP_SINGLE, 2412 },
	{ 0x7d18, 0x7d18, PDF_CMAP_SINGLE, 7585 },
	{ 0x7d19, 0x7d19, PDF_CMAP_SINGLE, 2423 },
	{ 0x7d1a, 0x7d1a, PDF_CMAP_SINGLE, 2420 },
	{ 0x7d1b, 0x7d1b, PDF_CMAP_SINGLE, 2424 },
	{ 0x7d1c, 0x7d1c, PDF_CMAP_SINGLE, 2421 },
	{ 0x7d1d, 0x7d1d, PDF_CMAP_SINGLE, 7582 },
	{ 0x7d1e, 0x7d1e, PDF_CMAP_SINGLE, 7581 },
	{ 0x7d1f, 0x7d1f, PDF_CMAP_SINGLE, 7588 },
	{ 0x7d20, 0x7d20, PDF_CMAP_SINGLE, 2415 },
	{ 0x7d21, 0x7d21, PDF_CMAP_SINGLE, 2411 },
	{ 0x7d22, 0x7d22, PDF_CMAP_SINGLE, 2416 },
	{ 0x7d25, 0x7d25, PDF_CMAP_SINGLE, 16094 },
	{ 0x7d28, 0x7d28, PDF_CMAP_SINGLE, 8178 },
	{ 0x7d29, 0x7d29, PDF_CMAP_SINGLE, 8171 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 3347 },
	{ 0x7d2c, 0x7d2c, PDF_CMAP_SINGLE, 8170 },
	{ 0x7d2e, 0x7d2e, PDF_CMAP_SINGLE, 2902 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 2909 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 2906 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 2912 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 2911 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 2907 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 8164 },
	{ 0x7d36, 0x7d36, PDF_CMAP_SINGLE, 8167 },
	{ 0x7d38, 0x7d38, PDF_CMAP_SINGLE, 8166 },
	{ 0x7d39, 0x7d39, PDF_CMAP_SINGLE, 2903 },
	{ 0x7d3a, 0x7d3a, PDF_CMAP_SINGLE, 8168 },
	{ 0x7d3b, 0x7d3b, PDF_CMAP_SINGLE, 8177 },
	{ 0x7d3c, 0x7d3c, PDF_CMAP_SINGLE, 2904 },
	{ 0x7d3d, 0x7d3d, PDF_CMAP_SINGLE, 8165 },
	{ 0x7d3e, 0x7d3f, PDF_CMAP_RANGE, 8174 },
	{ 0x7d40, 0x7d40, PDF_CMAP_SINGLE, 2905 },
	{ 0x7d41, 0x7d41, PDF_CMAP_SINGLE, 8172 },
	{ 0x7d42, 0x7d42, PDF_CMAP_SINGLE, 2910 },
	{ 0x7d43, 0x7d43, PDF_CMAP_SINGLE, 2900 },
	{ 0x7d44, 0x7d44, PDF_CMAP_SINGLE, 2908 },
	{ 0x7d45, 0x7d45, PDF_CMAP_SINGLE, 8169 },
	{ 0x7d46, 0x7d46, PDF_CMAP_SINGLE, 2899 },
	{ 0x7d47, 0x7d47, PDF_CMAP_SINGLE, 8173 },
	{ 0x7d4a, 0x7d4a, PDF_CMAP_SINGLE, 8176 },
	{ 0x7d4d, 0x7d4d, PDF_CMAP_SINGLE, 16349 },
	{ 0x7d4e, 0x7d4e, PDF_CMAP_SINGLE, 8816 },
	{ 0x7d4f, 0x7d4f, PDF_CMAP_SINGLE, 8807 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 3344 },
	{ 0x7d51, 0x7d51, PDF_CMAP_SINGLE, 8814 },
	{ 0x7d52, 0x7d52, PDF_CMAP_SINGLE, 8811 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 8803 },
	{ 0x7d54, 0x7d54, PDF_CMAP_SINGLE, 8812 },
	{ 0x7d55, 0x7d55, PDF_CMAP_SINGLE, 3346 },
	{ 0x7d56, 0x7d56, PDF_CMAP_SINGLE, 8804 },
	{ 0x7d58, 0x7d58, PDF_CMAP_SINGLE, 8800 },
	{ 0x7d5a, 0x7d5a, PDF_CMAP_SINGLE, 16019 },
	{ 0x7d5b, 0x7d5b, PDF_CMAP_SINGLE, 3779 },
	{ 0x7d5c, 0x7d5c, PDF_CMAP_SINGLE, 8809 },
	{ 0x7d5d, 0x7d5d, PDF_CMAP_SINGLE, 14185 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 3343 },
	{ 0x7d5f, 0x7d5f, PDF_CMAP_SINGLE, 8815 },
	{ 0x7d61, 0x7d61, PDF_CMAP_SINGLE, 3350 },
	{ 0x7d62, 0x7d62, PDF_CMAP_SINGLE, 3352 },
	{ 0x7d63, 0x7d63, PDF_CMAP_SINGLE, 8802 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 3351 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 8805 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 3345 },
	{ 0x7d69, 0x7d69, PDF_CMAP_SINGLE, 8813 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 8806 },
	{ 0x7d6b, 0x7d6b, PDF_CMAP_SINGLE, 8810 },
	{ 0x7d6d, 0x7d6d, PDF_CMAP_SINGLE, 8808 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 3348 },
	{ 0x7d6f, 0x7d6f, PDF_CMAP_SINGLE, 8801 },
	{ 0x7d70, 0x7d70, PDF_CMAP_SINGLE, 3353 },
	{ 0x7d71, 0x7d71, PDF_CMAP_SINGLE, 2901 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 3349 },
	{ 0x7d73, 0x7d73, PDF_CMAP_SINGLE, 3354 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 3775 },
	{ 0x7d7a, 0x7d7a, PDF_CMAP_SINGLE, 9477 },
	{ 0x7d7b, 0x7d7b, PDF_CMAP_SINGLE, 9479 },
	{ 0x7d7c, 0x7d7c, PDF_CMAP_SINGLE, 9481 },
	{ 0x7d7d, 0x7d7d, PDF_CMAP_SINGLE, 9485 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 9475 },
	{ 0x7d80, 0x7d80, PDF_CMAP_SINGLE, 9473 },
	{ 0x7d81, 0x7d81, PDF_CMAP_SINGLE, 3777 },
	{ 0x7d83, 0x7d83, PDF_CMAP_SINGLE, 9480 },
	{ 0x7d84, 0x7d84, PDF_CMAP_SINGLE, 9484 },
	{ 0x7d85, 0x7d85, PDF_CMAP_SINGLE, 9476 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 9472 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 9471 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 14187 },
	{ 0x7d8b, 0x7d8b, PDF_CMAP_SINGLE, 18902 },
	{ 0x7d8c, 0x7d8c, PDF_CMAP_SINGLE, 9482 },
	{ 0x7d8d, 0x7d8d, PDF_CMAP_SINGLE, 9474 },
	{ 0x7d8e, 0x7d8e, PDF_CMAP_SINGLE, 9478 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 3778 },
	{ 0x7d91, 0x7d91, PDF_CMAP_SINGLE, 3776 },
	{ 0x7d92, 0x7d92, PDF_CMAP_SINGLE, 9486 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 3774 },
	{ 0x7d94, 0x7d94, PDF_CMAP_SINGLE, 9483 },
	{ 0x7d95, 0x7d95, PDF_CMAP_SINGLE, 19003 },
	{ 0x7d96, 0x7d96, PDF_CMAP_SINGLE, 10126 },
	{ 0x7d97, 0x7d97, PDF_CMAP_SINGLE, 15187 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 4189 },
	{ 0x7d9d, 0x7d9d, PDF_CMAP_SINGLE, 10118 },
	{ 0x7d9e, 0x7d9e, PDF_CMAP_SINGLE, 4577 },
	{ 0x7d9f, 0x7d9f, PDF_CMAP_SINGLE, 10128 },
	{ 0x7da0, 0x7da0, PDF_CMAP_SINGLE, 4192 },
	{ 0x7da1, 0x7da1, PDF_CMAP_SINGLE, 10132 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 4198 },
	{ 0x7da3, 0x7da3, PDF_CMAP_SINGLE, 10113 },
	{ 0x7da4, 0x7da4, PDF_CMAP_SINGLE, 18135 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 10129 },
	{ 0x7da7, 0x7da7, PDF_CMAP_SINGLE, 10110 },
	{ 0x7da8, 0x7da8, PDF_CMAP_SINGLE, 15516 },
	{ 0x7da9, 0x7da9, PDF_CMAP_SINGLE, 10131 },
	{ 0x7daa, 0x7daa, PDF_CMAP_SINGLE, 10114 },
	{ 0x7dab, 0x7dab, PDF_CMAP_SINGLE, 14188 },
	{ 0x7dac, 0x7dac, PDF_CMAP_SINGLE, 4205 },
	{ 0x7dad, 0x7dad, PDF_CMAP_SINGLE, 4202 },
	{ 0x7dae, 0x7dae, PDF_CMAP_SINGLE, 10130 },
	{ 0x7daf, 0x7daf, PDF_CMAP_SINGLE, 10124 },
	{ 0x7db0, 0x7db0, PDF_CMAP_SINGLE, 4188 },
	{ 0x7db1, 0x7db1, PDF_CMAP_SINGLE, 4196 },
	{ 0x7db2, 0x7db2, PDF_CMAP_SINGLE, 4195 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 14190 },
	{ 0x7db4, 0x7db4, PDF_CMAP_SINGLE, 4194 },
	{ 0x7db5, 0x7db5, PDF_CMAP_SINGLE, 4200 },
	{ 0x7db7, 0x7db7, PDF_CMAP_SINGLE, 10111 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 4201 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 10125 },
	{ 0x7dba, 0x7dba, PDF_CMAP_SINGLE, 4197 },
	{ 0x7dbb, 0x7dbb, PDF_CMAP_SINGLE, 4187 },
	{ 0x7dbc, 0x7dbc, PDF_CMAP_SINGLE, 10127 },
	{ 0x7dbd, 0x7dbe, PDF_CMAP_RANGE, 4190 },
	{ 0x7dbf, 0x7dbf, PDF_CMAP_SINGLE, 4199 },
	{ 0x7dc0, 0x7dc0, PDF_CMAP_SINGLE, 10116 },
	{ 0x7dc1, 0x7dc1, PDF_CMAP_SINGLE, 10115 },
	{ 0x7dc2, 0x7dc2, PDF_CMAP_SINGLE, 10112 },
	{ 0x7dc4, 0x7dc4, PDF_CMAP_SINGLE, 10120 },
	{ 0x7dc5, 0x7dc5, PDF_CMAP_SINGLE, 10117 },
	{ 0x7dc6, 0x7dc6, PDF_CMAP_SINGLE, 10121 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 4204 },
	{ 0x7dc9, 0x7dc9, PDF_CMAP_SINGLE, 10133 },
	{ 0x7dca, 0x7dca, PDF_CMAP_SINGLE, 4193 },
	{ 0x7dcb, 0x7dcc, PDF_CMAP_RANGE, 10122 },
	{ 0x7dcd, 0x7dcd, PDF_CMAP_SINGLE, 17774 },
	{ 0x7dce, 0x7dce, PDF_CMAP_SINGLE, 10119 },
	{ 0x7dcf, 0x7dcf, PDF_CMAP_SINGLE, 18229 },
	{ 0x7dd0, 0x7dd0, PDF_CMAP_SINGLE, 18231 },
	{ 0x7dd2, 0x7dd2, PDF_CMAP_SINGLE, 4203 },
	{ 0x7dd3, 0x7dd3, PDF_CMAP_SINGLE, 15182 },
	{ 0x7dd4, 0x7dd4, PDF_CMAP_SINGLE, 18230 },
	{ 0x7dd6, 0x7dd6, PDF_CMAP_SINGLE, 14191 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 10680 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 4569 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 4578 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 4574 },
	{ 0x7ddb, 0x7ddb, PDF_CMAP_SINGLE, 10677 },
	{ 0x7ddc, 0x7ddc, PDF_CMAP_SINGLE, 15180 },
	{ 0x7ddd, 0x7ddd, PDF_CMAP_SINGLE, 4571 },
	{ 0x7dde, 0x7dde, PDF_CMAP_SINGLE, 4575 },
	{ 0x7ddf, 0x7ddf, PDF_CMAP_SINGLE, 10689 },
	{ 0x7de0, 0x7de0, PDF_CMAP_SINGLE, 4565 },
	{ 0x7de1, 0x7de1, PDF_CMAP_SINGLE, 10681 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 4573 },
	{ 0x7de4, 0x7de4, PDF_CMAP_SINGLE, 14194 },
	{ 0x7de5, 0x7de5, PDF_CMAP_SINGLE, 14198 },
	{ 0x7de6, 0x7de6, PDF_CMAP_SINGLE, 10684 },
	{ 0x7de7, 0x7de7, PDF_CMAP_SINGLE, 10679 },
	{ 0x7de8, 0x7de8, PDF_CMAP_SINGLE, 4572 },
	{ 0x7de9, 0x7de9, PDF_CMAP_SINGLE, 4576 },
	{ 0x7dea, 0x7dea, PDF_CMAP_SINGLE, 10678 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 4570 },
	{ 0x7dee, 0x7dee, PDF_CMAP_SINGLE, 10688 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 4567 },
	{ 0x7df0, 0x7df0, PDF_CMAP_SINGLE, 10687 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 10686 },
	{ 0x7df2, 0x7df2, PDF_CMAP_SINGLE, 4579 },
	{ 0x7df3, 0x7df3, PDF_CMAP_SINGLE, 10440 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4566 },
	{ 0x7df5, 0x7df5, PDF_CMAP_SINGLE, 14196 },
	{ 0x7df6, 0x7df6, PDF_CMAP_SINGLE, 10685 },
	{ 0x7df7, 0x7df7, PDF_CMAP_SINGLE, 10676 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 4580 },
	{ 0x7dfa, 0x7dfa, PDF_CMAP_SINGLE, 10683 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 4568 },
	{ 0x7dfd, 0x7dfd, PDF_CMAP_SINGLE, 18232 },
	{ 0x7dfe, 0x7dfe, PDF_CMAP_SINGLE, 15567 },
	{ 0x7e03, 0x7e03, PDF_CMAP_SINGLE, 10682 },
	{ 0x7e07, 0x7e07, PDF_CMAP_SINGLE, 15181 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 4898 },
	{ 0x7e09, 0x7e09, PDF_CMAP_SINGLE, 4903 },
	{ 0x7e0a, 0x7e0a, PDF_CMAP_SINGLE, 4896 },
	{ 0x7e0b, 0x7e0b, PDF_CMAP_SINGLE, 11295 },
	{ 0x7e0c, 0x7e0c, PDF_CMAP_SINGLE, 11286 },
	{ 0x7e0d, 0x7e0d, PDF_CMAP_SINGLE, 11298 },
	{ 0x7e0e, 0x7e0e, PDF_CMAP_SINGLE, 11290 },
	{ 0x7e0f, 0x7e0f, PDF_CMAP_SINGLE, 11296 },
	{ 0x7e10, 0x7e10, PDF_CMAP_SINGLE, 4904 },
	{ 0x7e11, 0x7e11, PDF_CMAP_SINGLE, 4897 },
	{ 0x7e12, 0x7e12, PDF_CMAP_SINGLE, 11283 },
	{ 0x7e13, 0x7e13, PDF_CMAP_SINGLE, 11289 },
	{ 0x7e14, 0x7e14, PDF_CMAP_SINGLE, 11299 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 11292 },
	{ 0x7e16, 0x7e16, PDF_CMAP_SINGLE, 11297 },
	{ 0x7e17, 0x7e17, PDF_CMAP_SINGLE, 11285 },
	{ 0x7e1a, 0x7e1a, PDF_CMAP_SINGLE, 11293 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 4899 },
	{ 0x7e1c, 0x7e1c, PDF_CMAP_SINGLE, 11291 },
	{ 0x7e1d, 0x7e1d, PDF_CMAP_SINGLE, 4902 },
	{ 0x7e1e, 0x7e1e, PDF_CMAP_SINGLE, 4901 },
	{ 0x7e1f, 0x7e20, PDF_CMAP_RANGE, 11287 },
	{ 0x7e21, 0x7e21, PDF_CMAP_SINGLE, 11284 },
	{ 0x7e22, 0x7e22, PDF_CMAP_SINGLE, 11294 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 4900 },
	{ 0x7e24, 0x7e24, PDF_CMAP_SINGLE, 11301 },
	{ 0x7e25, 0x7e25, PDF_CMAP_SINGLE, 11300 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 14207 },
	{ 0x7e29, 0x7e29, PDF_CMAP_SINGLE, 11768 },
	{ 0x7e2a, 0x7e2a, PDF_CMAP_SINGLE, 11764 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 5179 },
	{ 0x7e2d, 0x7e2d, PDF_CMAP_SINGLE, 11758 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 5173 },
	{ 0x7e2f, 0x7e2f, PDF_CMAP_SINGLE, 5189 },
	{ 0x7e30, 0x7e30, PDF_CMAP_SINGLE, 11770 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 5181 },
	{ 0x7e32, 0x7e32, PDF_CMAP_SINGLE, 5177 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 11761 },
	{ 0x7e34, 0x7e34, PDF_CMAP_SINGLE, 5184 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 5187 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 11772 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 5176 },
	{ 0x7e38, 0x7e38, PDF_CMAP_SINGLE, 11763 },
	{ 0x7e39, 0x7e39, PDF_CMAP_SINGLE, 5185 },
	{ 0x7e3a, 0x7e3a, PDF_CMAP_SINGLE, 11774 },
	{ 0x7e3b, 0x7e3b, PDF_CMAP_SINGLE, 11771 },
	{ 0x7e3c, 0x7e3c, PDF_CMAP_SINGLE, 11759 },
	{ 0x7e3d, 0x7e3d, PDF_CMAP_SINGLE, 5180 },
	{ 0x7e3e, 0x7e3e, PDF_CMAP_SINGLE, 5174 },
	{ 0x7e3f, 0x7e3f, PDF_CMAP_SINGLE, 5188 },
	{ 0x7e40, 0x7e40, PDF_CMAP_SINGLE, 11766 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 5183 },
	{ 0x7e42, 0x7e42, PDF_CMAP_SINGLE, 11760 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 5178 },
	{ 0x7e44, 0x7e44, PDF_CMAP_SINGLE, 11773 },
	{ 0x7e45, 0x7e45, PDF_CMAP_SINGLE, 5182 },
	{ 0x7e46, 0x7e46, PDF_CMAP_SINGLE, 5175 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 11767 },
	{ 0x7e48, 0x7e48, PDF_CMAP_SINGLE, 5186 },
	{ 0x7e49, 0x7e49, PDF_CMAP_SINGLE, 11765 },
	{ 0x7e4c, 0x7e4c, PDF_CMAP_SINGLE, 11769 },
	{ 0x7e50, 0x7e50, PDF_CMAP_SINGLE, 12163 },
	{ 0x7e51, 0x7e51, PDF_CMAP_SINGLE, 12169 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 5393 },
	{ 0x7e53, 0x7e53, PDF_CMAP_SINGLE, 12172 },
	{ 0x7e54, 0x7e55, PDF_CMAP_RANGE, 5388 },
	{ 0x7e56, 0x7e56, PDF_CMAP_SINGLE, 12164 },
	{ 0x7e57, 0x7e57, PDF_CMAP_SINGLE, 12171 },
	{ 0x7e58, 0x7e58, PDF_CMAP_SINGLE, 12166 },
	{ 0x7e59, 0x7e59, PDF_CMAP_SINGLE, 5394 },
	{ 0x7e5a, 0x7e5a, PDF_CMAP_SINGLE, 5391 },
	{ 0x7e5b, 0x7e5b, PDF_CMAP_SINGLE, 18964 },
	{ 0x7e5c, 0x7e5c, PDF_CMAP_SINGLE, 12162 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 5390 },
	{ 0x7e5f, 0x7e5f, PDF_CMAP_SINGLE, 12168 },
	{ 0x7e60, 0x7e60, PDF_CMAP_SINGLE, 12170 },
	{ 0x7e61, 0x7e61, PDF_CMAP_SINGLE, 5392 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 12167 },
	{ 0x7e63, 0x7e63, PDF_CMAP_SINGLE, 12165 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 18130 },
	{ 0x7e67, 0x7e67, PDF_CMAP_SINGLE, 14182 },
	{ 0x7e68, 0x7e68, PDF_CMAP_SINGLE, 12515 },
	{ 0x7e69, 0x7e6a, PDF_CMAP_RANGE, 5548 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 5545 },
	{ 0x7e6c, 0x7e6c, PDF_CMAP_SINGLE, 18929 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 5546 },
	{ 0x7e6e, 0x7e6e, PDF_CMAP_SINGLE, 14203 },
	{ 0x7e6f, 0x7e6f, PDF_CMAP_SINGLE, 12511 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 12509 },
	{ 0x7e72, 0x7e72, PDF_CMAP_SINGLE, 12513 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 5550 },
	{ 0x7e74, 0x7e74, PDF_CMAP_SINGLE, 12514 },
	{ 0x7e75, 0x7e75, PDF_CMAP_SINGLE, 12507 },
	{ 0x7e76, 0x7e76, PDF_CMAP_SINGLE, 12506 },
	{ 0x7e77, 0x7e77, PDF_CMAP_SINGLE, 12510 },
	{ 0x7e78, 0x7e78, PDF_CMAP_SINGLE, 12508 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 5547 },
	{ 0x7e7a, 0x7e7a, PDF_CMAP_SINGLE, 12512 },
	{ 0x7e7b, 0x7e7b, PDF_CMAP_SINGLE, 12820 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 5681 },
	{ 0x7e7d, 0x7e7d, PDF_CMAP_SINGLE, 5680 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 12821 },
	{ 0x7e7f, 0x7e7f, PDF_CMAP_SINGLE, 15953 },
	{ 0x7e80, 0x7e80, PDF_CMAP_SINGLE, 12823 },
	{ 0x7e81, 0x7e81, PDF_CMAP_SINGLE, 12822 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 5682 },
	{ 0x7e86, 0x7e86, PDF_CMAP_SINGLE, 13040 },
	{ 0x7e87, 0x7e88, PDF_CMAP_RANGE, 13037 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 13036 },
	{ 0x7e8b, 0x7e8b, PDF_CMAP_SINGLE, 13039 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 5763 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 13041 },
	{ 0x7e8e, 0x7e8e, PDF_CMAP_SINGLE, 16073 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 5762 },
	{ 0x7e91, 0x7e91, PDF_CMAP_SINGLE, 13218 },
	{ 0x7e92, 0x7e92, PDF_CMAP_SINGLE, 18078 },
	{ 0x7e93, 0x7e93, PDF_CMAP_SINGLE, 5891 },
	{ 0x7e94, 0x7e94, PDF_CMAP_SINGLE, 5893 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 13356 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 5892 },
	{ 0x7e97, 0x7e97, PDF_CMAP_SINGLE, 13460 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 13530 },
	{ 0x7e99, 0x7e99, PDF_CMAP_SINGLE, 13532 },
	{ 0x7e9a, 0x7e9a, PDF_CMAP_SINGLE, 13529 },
	{ 0x7e9b, 0x7e9b, PDF_CMAP_SINGLE, 13531 },
	{ 0x7e9c, 0x7e9c, PDF_CMAP_SINGLE, 5977 },
	{ 0x7e9f, 0x7e9f, PDF_CMAP_SINGLE, 18619 },
	{ 0x7ea4, 0x7ea4, PDF_CMAP_SINGLE, 17755 },
	{ 0x7eac, 0x7eac, PDF_CMAP_SINGLE, 17756 },
	{ 0x7eba, 0x7eba, PDF_CMAP_SINGLE, 17757 },
	{ 0x7ec7, 0x7ec7, PDF_CMAP_SINGLE, 17758 },
	{ 0x7ecf, 0x7ecf, PDF_CMAP_SINGLE, 17759 },
	{ 0x7edf, 0x7edf, PDF_CMAP_SINGLE, 17760 },
	{ 0x7f06, 0x7f06, PDF_CMAP_SINGLE, 17761 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 1017 },
	{ 0x7f37, 0x7f37, PDF_CMAP_SINGLE, 17762 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 2004 },
	{ 0x7f39, 0x7f39, PDF_CMAP_SINGLE, 7455 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 2425 },
	{ 0x7f3d, 0x7f3d, PDF_CMAP_SINGLE, 2913 },
	{ 0x7f3e, 0x7f3f, PDF_CMAP_RANGE, 8817 },
	{ 0x7f40, 0x7f41, PDF_CMAP_RANGE, 14208 },
	{ 0x7f43, 0x7f43, PDF_CMAP_SINGLE, 11302 },
	{ 0x7f44, 0x7f44, PDF_CMAP_SINGLE, 5190 },
	{ 0x7f45, 0x7f45, PDF_CMAP_SINGLE, 11775 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 14210 },
	{ 0x7f48, 0x7f48, PDF_CMAP_SINGLE, 5395 },
	{ 0x7f49, 0x7f49, PDF_CMAP_SINGLE, 15977 },
	{ 0x7f4a, 0x7f4a, PDF_CMAP_SINGLE, 12517 },
	{ 0x7f4b, 0x7f4b, PDF_CMAP_SINGLE, 12516 },
	{ 0x7f4c, 0x7f4c, PDF_CMAP_SINGLE, 5683 },
	{ 0x7f4d, 0x7f4d, PDF_CMAP_SINGLE, 13042 },
	{ 0x7f4e, 0x7f4e, PDF_CMAP_SINGLE, 15183 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 13219 },
	{ 0x7f50, 0x7f50, PDF_CMAP_SINGLE, 5926 },
	{ 0x7f51, 0x7f51, PDF_CMAP_SINGLE, 6162 },
	{ 0x7f52, 0x7f52, PDF_CMAP_SINGLE, 17654 },
	{ 0x7f53, 0x7f53, PDF_CMAP_SINGLE, 17657 },
	{ 0x7f54, 0x7f54, PDF_CMAP_SINGLE, 1640 },
	{ 0x7f55, 0x7f55, PDF_CMAP_SINGLE, 1276 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 7107 },
	{ 0x7f5b, 0x7f5b, PDF_CMAP_SINGLE, 7597 },
	{ 0x7f5c, 0x7f5c, PDF_CMAP_SINGLE, 7592 },
	{ 0x7f5d, 0x7f5d, PDF_CMAP_SINGLE, 7596 },
	{ 0x7f5e, 0x7f5e, PDF_CMAP_SINGLE, 7594 },
	{ 0x7f5f, 0x7f5f, PDF_CMAP_SINGLE, 2426 },
	{ 0x7f60, 0x7f60, PDF_CMAP_SINGLE, 7595 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 7593 },
	{ 0x7f63, 0x7f63, PDF_CMAP_SINGLE, 8179 },
	{ 0x7f65, 0x7f66, PDF_CMAP_RANGE, 8819 },
	{ 0x7f67, 0x7f68, PDF_CMAP_RANGE, 9489 },
	{ 0x7f69, 0x7f6a, PDF_CMAP_RANGE, 3781 },
	{ 0x7f6b, 0x7f6b, PDF_CMAP_SINGLE, 9488 },
	{ 0x7f6c, 0x7f6c, PDF_CMAP_SINGLE, 9491 },
	{ 0x7f6d, 0x7f6d, PDF_CMAP_SINGLE, 9487 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 3780 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 4206 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 16391 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 3783 },
	{ 0x7f73, 0x7f73, PDF_CMAP_SINGLE, 10134 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 4581 },
	{ 0x7f76, 0x7f76, PDF_CMAP_SINGLE, 10690 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 4582 },
	{ 0x7f78, 0x7f78, PDF_CMAP_SINGLE, 16251 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 4905 },
	{ 0x7f7a, 0x7f7a, PDF_CMAP_SINGLE, 11305 },
	{ 0x7f7b, 0x7f7c, PDF_CMAP_RANGE, 11303 },
	{ 0x7f7d, 0x7f7d, PDF_CMAP_SINGLE, 11778 },
	{ 0x7f7e, 0x7f7e, PDF_CMAP_SINGLE, 11777 },
	{ 0x7f7f, 0x7f7f, PDF_CMAP_SINGLE, 11776 },
	{ 0x7f83, 0x7f83, PDF_CMAP_SINGLE, 12518 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 5551 },
	{ 0x7f86, 0x7f86, PDF_CMAP_SINGLE, 12519 },
	{ 0x7f87, 0x7f87, PDF_CMAP_SINGLE, 13220 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 5927 },
	{ 0x7f89, 0x7f89, PDF_CMAP_SINGLE, 13461 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 1018 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 1642 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 1641 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 7109 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 2005 },
	{ 0x7f8f, 0x7f8f, PDF_CMAP_SINGLE, 16924 },
	{ 0x7f91, 0x7f91, PDF_CMAP_SINGLE, 7108 },
	{ 0x7f92, 0x7f92, PDF_CMAP_SINGLE, 7599 },
	{ 0x7f93, 0x7f93, PDF_CMAP_SINGLE, 17371 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 2427 },
	{ 0x7f95, 0x7f95, PDF_CMAP_SINGLE, 8180 },
	{ 0x7f96, 0x7f96, PDF_CMAP_SINGLE, 7598 },
	{ 0x7f97, 0x7f97, PDF_CMAP_SINGLE, 14214 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 2915 },
	{ 0x7f9b, 0x7f9b, PDF_CMAP_SINGLE, 8183 },
	{ 0x7f9c, 0x7f9d, PDF_CMAP_RANGE, 8181 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 2914 },
	{ 0x7fa0, 0x7fa1, PDF_CMAP_RANGE, 8822 },
	{ 0x7fa2, 0x7fa2, PDF_CMAP_SINGLE, 8821 },
	{ 0x7fa3, 0x7fa3, PDF_CMAP_SINGLE, 14216 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 3786 },
	{ 0x7fa5, 0x7fa5, PDF_CMAP_SINGLE, 9493 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 9492 },
	{ 0x7fa7, 0x7fa7, PDF_CMAP_SINGLE, 9494 },
	{ 0x7fa8, 0x7fa8, PDF_CMAP_SINGLE, 3785 },
	{ 0x7fa9, 0x7fa9, PDF_CMAP_SINGLE, 3784 },
	{ 0x7fac, 0x7fac, PDF_CMAP_SINGLE, 10691 },
	{ 0x7fad, 0x7fad, PDF_CMAP_SINGLE, 10693 },
	{ 0x7fae, 0x7fae, PDF_CMAP_SINGLE, 17372 },
	{ 0x7faf, 0x7faf, PDF_CMAP_SINGLE, 4583 },
	{ 0x7fb0, 0x7fb0, PDF_CMAP_SINGLE, 10692 },
	{ 0x7fb1, 0x7fb1, PDF_CMAP_SINGLE, 11306 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 4906 },
	{ 0x7fb3, 0x7fb3, PDF_CMAP_SINGLE, 12174 },
	{ 0x7fb4, 0x7fb4, PDF_CMAP_SINGLE, 18233 },
	{ 0x7fb5, 0x7fb5, PDF_CMAP_SINGLE, 12173 },
	{ 0x7fb6, 0x7fb6, PDF_CMAP_SINGLE, 5552 },
	{ 0x7fb7, 0x7fb7, PDF_CMAP_SINGLE, 12520 },
	{ 0x7fb8, 0x7fb8, PDF_CMAP_SINGLE, 5554 },
	{ 0x7fb9, 0x7fb9, PDF_CMAP_SINGLE, 5553 },
	{ 0x7fba, 0x7fba, PDF_CMAP_SINGLE, 12824 },
	{ 0x7fbb, 0x7fbb, PDF_CMAP_SINGLE, 13043 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 5764 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 1019 },
	{ 0x7fbe, 0x7fbe, PDF_CMAP_SINGLE, 7110 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 2006 },
	{ 0x7fc0, 0x7fc0, PDF_CMAP_SINGLE, 7602 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 2429 },
	{ 0x7fc2, 0x7fc2, PDF_CMAP_SINGLE, 7601 },
	{ 0x7fc3, 0x7fc3, PDF_CMAP_SINGLE, 7600 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 2428 },
	{ 0x7fc7, 0x7fc7, PDF_CMAP_SINGLE, 8189 },
	{ 0x7fc9, 0x7fc9, PDF_CMAP_SINGLE, 8191 },
	{ 0x7fca, 0x7fcb, PDF_CMAP_RANGE, 8184 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 2916 },
	{ 0x7fcd, 0x7fcd, PDF_CMAP_SINGLE, 8186 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 2917 },
	{ 0x7fcf, 0x7fcf, PDF_CMAP_SINGLE, 8190 },
	{ 0x7fd0, 0x7fd1, PDF_CMAP_RANGE, 8187 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2918 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_RANGE, 3356 },
	{ 0x7fd7, 0x7fd7, PDF_CMAP_SINGLE, 8824 },
	{ 0x7fdb, 0x7fdc, PDF_CMAP_RANGE, 9495 },
	{ 0x7fdd, 0x7fdd, PDF_CMAP_SINGLE, 15198 },
	{ 0x7fde, 0x7fde, PDF_CMAP_SINGLE, 10138 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 4209 },
	{ 0x7fe0, 0x7fe1, PDF_CMAP_RANGE, 4207 },
	{ 0x7fe2, 0x7fe3, PDF_CMAP_RANGE, 10135 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 10137 },
	{ 0x7fe6, 0x7fe6, PDF_CMAP_SINGLE, 10698 },
	{ 0x7fe7, 0x7fe7, PDF_CMAP_SINGLE, 18153 },
	{ 0x7fe8, 0x7fe8, PDF_CMAP_SINGLE, 10699 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 4584 },
	{ 0x7fea, 0x7fea, PDF_CMAP_SINGLE, 10696 },
	{ 0x7feb, 0x7feb, PDF_CMAP_SINGLE, 10695 },
	{ 0x7fec, 0x7fec, PDF_CMAP_SINGLE, 10697 },
	{ 0x7fed, 0x7fed, PDF_CMAP_SINGLE, 10694 },
	{ 0x7fee, 0x7fee, PDF_CMAP_SINGLE, 4909 },
	{ 0x7fef, 0x7fef, PDF_CMAP_SINGLE, 11307 },
	{ 0x7ff0, 0x7ff1, PDF_CMAP_RANGE, 4907 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 11780 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 5191 },
	{ 0x7ff4, 0x7ff4, PDF_CMAP_SINGLE, 11779 },
	{ 0x7ff5, 0x7ff5, PDF_CMAP_SINGLE, 12031 },
	{ 0x7ff7, 0x7ff8, PDF_CMAP_RANGE, 12175 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 5396 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 14222 },
	{ 0x7ffb, 0x7ffb, PDF_CMAP_SINGLE, 5397 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 5192 },
	{ 0x7ffd, 0x7ffe, PDF_CMAP_RANGE, 12521 },
	{ 0x7fff, 0x7fff, PDF_CMAP_SINGLE, 12825 },
	{ 0x8000, 0x8000, PDF_CMAP_SINGLE, 5684 },
	{ 0x8001, 0x8001, PDF_CMAP_SINGLE, 1020 },
	{ 0x8002, 0x8002, PDF_CMAP_SINGLE, 17661 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 1021 },
	{ 0x8004, 0x8004, PDF_CMAP_SINGLE, 2431 },
	{ 0x8005, 0x8005, PDF_CMAP_SINGLE, 1643 },
	{ 0x8006, 0x8006, PDF_CMAP_SINGLE, 2430 },
	{ 0x8007, 0x8007, PDF_CMAP_SINGLE, 7111 },
	{ 0x8008, 0x8008, PDF_CMAP_SINGLE, 14225 },
	{ 0x800b, 0x800b, PDF_CMAP_SINGLE, 3358 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 1022 },
	{ 0x800d, 0x800d, PDF_CMAP_SINGLE, 2008 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 7112 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 2007 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 2009 },
	{ 0x8012, 0x8012, PDF_CMAP_SINGLE, 1023 },
	{ 0x8014, 0x8014, PDF_CMAP_SINGLE, 7114 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 2433 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 7603 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 2435 },
	{ 0x8018, 0x8018, PDF_CMAP_SINGLE, 2432 },
	{ 0x8019, 0x8019, PDF_CMAP_SINGLE, 2434 },
	{ 0x801b, 0x801b, PDF_CMAP_SINGLE, 8194 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 2919 },
	{ 0x801d, 0x801d, PDF_CMAP_SINGLE, 14226 },
	{ 0x801e, 0x801e, PDF_CMAP_SINGLE, 8193 },
	{ 0x801f, 0x801f, PDF_CMAP_SINGLE, 8192 },
	{ 0x8020, 0x8020, PDF_CMAP_SINGLE, 18236 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 9497 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 10139 },
	{ 0x8025, 0x8025, PDF_CMAP_SINGLE, 18237 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 4585 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 4910 },
	{ 0x8029, 0x8029, PDF_CMAP_SINGLE, 11309 },
	{ 0x802a, 0x802a, PDF_CMAP_SINGLE, 11308 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 11781 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 18238 },
	{ 0x802f, 0x802f, PDF_CMAP_SINGLE, 14228 },
	{ 0x8030, 0x8030, PDF_CMAP_SINGLE, 13044 },
	{ 0x8031, 0x8031, PDF_CMAP_SINGLE, 18239 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 1024 },
	{ 0x8034, 0x8034, PDF_CMAP_SINGLE, 6360 },
	{ 0x8035, 0x8035, PDF_CMAP_SINGLE, 6695 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 2010 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 7115 },
	{ 0x8039, 0x8039, PDF_CMAP_SINGLE, 7605 },
	{ 0x803b, 0x803c, PDF_CMAP_RANGE, 14231 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 2436 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 7604 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 2437 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 8196 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 2921 },
	{ 0x8047, 0x8047, PDF_CMAP_SINGLE, 8195 },
	{ 0x8048, 0x8048, PDF_CMAP_SINGLE, 8197 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 2920 },
	{ 0x804f, 0x8050, PDF_CMAP_RANGE, 8826 },
	{ 0x8051, 0x8051, PDF_CMAP_SINGLE, 8825 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 3359 },
	{ 0x8054, 0x8054, PDF_CMAP_SINGLE, 18240 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 3787 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 3788 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 4211 },
	{ 0x805b, 0x805b, PDF_CMAP_SINGLE, 16357 },
	{ 0x805c, 0x805c, PDF_CMAP_SINGLE, 10141 },
	{ 0x805d, 0x805d, PDF_CMAP_SINGLE, 10140 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 4210 },
	{ 0x8061, 0x8061, PDF_CMAP_SINGLE, 14233 },
	{ 0x8062, 0x8062, PDF_CMAP_SINGLE, 16355 },
	{ 0x8063, 0x8063, PDF_CMAP_SINGLE, 16348 },
	{ 0x8064, 0x8064, PDF_CMAP_SINGLE, 10700 },
	{ 0x8066, 0x8066, PDF_CMAP_SINGLE, 16347 },
	{ 0x8067, 0x8067, PDF_CMAP_SINGLE, 10701 },
	{ 0x806c, 0x806c, PDF_CMAP_SINGLE, 11310 },
	{ 0x806f, 0x806f, PDF_CMAP_SINGLE, 5196 },
	{ 0x8070, 0x8070, PDF_CMAP_SINGLE, 5195 },
	{ 0x8071, 0x8072, PDF_CMAP_RANGE, 5193 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 5197 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 12177 },
	{ 0x8076, 0x8076, PDF_CMAP_SINGLE, 5399 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 5398 },
	{ 0x8078, 0x8078, PDF_CMAP_SINGLE, 12523 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 12826 },
	{ 0x807d, 0x807d, PDF_CMAP_SINGLE, 5843 },
	{ 0x807e, 0x807e, PDF_CMAP_SINGLE, 5842 },
	{ 0x807f, 0x807f, PDF_CMAP_SINGLE, 1025 },
	{ 0x8080, 0x8080, PDF_CMAP_SINGLE, 17662 },
	{ 0x8082, 0x8082, PDF_CMAP_SINGLE, 7404 },
	{ 0x8084, 0x8084, PDF_CMAP_SINGLE, 3790 },
	{ 0x8085, 0x8085, PDF_CMAP_SINGLE, 3360 },
	{ 0x8086, 0x8086, PDF_CMAP_SINGLE, 3789 },
	{ 0x8087, 0x8087, PDF_CMAP_SINGLE, 4212 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 1026 },
	{ 0x808a, 0x808a, PDF_CMAP_SINGLE, 6067 },
	{ 0x808b, 0x808c, PDF_CMAP_RANGE, 1027 },
	{ 0x808f, 0x808f, PDF_CMAP_SINGLE, 6696 },
	{ 0x8090, 0x8090, PDF_CMAP_SINGLE, 6363 },
	{ 0x8092, 0x8092, PDF_CMAP_SINGLE, 6364 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 1278 },
	{ 0x8095, 0x8095, PDF_CMAP_SINGLE, 6361 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 1277 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 1280 },
	{ 0x8099, 0x8099, PDF_CMAP_SINGLE, 6362 },
	{ 0x809a, 0x809a, PDF_CMAP_SINGLE, 1282 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 1281 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 6365 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 1279 },
	{ 0x809f, 0x809f, PDF_CMAP_SINGLE, 17782 },
	{ 0x80a1, 0x80a1, PDF_CMAP_SINGLE, 1648 },
	{ 0x80a2, 0x80a2, PDF_CMAP_SINGLE, 1646 },
	{ 0x80a3, 0x80a3, PDF_CMAP_SINGLE, 6698 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 1645 },
	{ 0x80a7, 0x80a7, PDF_CMAP_SINGLE, 14240 },
	{ 0x80a9, 0x80a9, PDF_CMAP_SINGLE, 1650 },
	{ 0x80aa, 0x80aa, PDF_CMAP_SINGLE, 1652 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 1649 },
	{ 0x80ad, 0x80ad, PDF_CMAP_SINGLE, 6701 },
	{ 0x80ae, 0x80ae, PDF_CMAP_SINGLE, 6697 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 1653 },
	{ 0x80b1, 0x80b1, PDF_CMAP_SINGLE, 1647 },
	{ 0x80b2, 0x80b2, PDF_CMAP_SINGLE, 1283 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 1651 },
	{ 0x80b5, 0x80b5, PDF_CMAP_SINGLE, 6700 },
	{ 0x80b6, 0x80b6, PDF_CMAP_SINGLE, 16046 },
	{ 0x80b7, 0x80b7, PDF_CMAP_SINGLE, 18243 },
	{ 0x80b8, 0x80b8, PDF_CMAP_SINGLE, 6699 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 1644 },
	{ 0x80bc, 0x80bc, PDF_CMAP_SINGLE, 17778 },
	{ 0x80bd, 0x80bd, PDF_CMAP_SINGLE, 15995 },
	{ 0x80c2, 0x80c2, PDF_CMAP_SINGLE, 7121 },
	{ 0x80c3, 0x80c4, PDF_CMAP_RANGE, 2014 },
	{ 0x80c5, 0x80c5, PDF_CMAP_SINGLE, 7123 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 15220 },
	{ 0x80c7, 0x80c7, PDF_CMAP_SINGLE, 7117 },
	{ 0x80c8, 0x80c8, PDF_CMAP_SINGLE, 7120 },
	{ 0x80c9, 0x80c9, PDF_CMAP_SINGLE, 7129 },
	{ 0x80ca, 0x80ca, PDF_CMAP_SINGLE, 7127 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 2016 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 7133 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 2019 },
	{ 0x80cf, 0x80cf, PDF_CMAP_SINGLE, 7130 },
	{ 0x80d0, 0x80d0, PDF_CMAP_SINGLE, 7122 },
	{ 0x80d1, 0x80d1, PDF_CMAP_SINGLE, 7119 },
	{ 0x80d4, 0x80d4, PDF_CMAP_SINGLE, 8829 },
	{ 0x80d5, 0x80d5, PDF_CMAP_SINGLE, 7128 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 2011 },
	{ 0x80d7, 0x80d7, PDF_CMAP_SINGLE, 7131 },
	{ 0x80d8, 0x80d8, PDF_CMAP_SINGLE, 7116 },
	{ 0x80d9, 0x80d9, PDF_CMAP_SINGLE, 7125 },
	{ 0x80da, 0x80da, PDF_CMAP_SINGLE, 2013 },
	{ 0x80db, 0x80db, PDF_CMAP_SINGLE, 2018 },
	{ 0x80dc, 0x80dc, PDF_CMAP_SINGLE, 7126 },
	{ 0x80dd, 0x80dd, PDF_CMAP_SINGLE, 2022 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 2020 },
	{ 0x80e0, 0x80e0, PDF_CMAP_SINGLE, 7118 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 2017 },
	{ 0x80e3, 0x80e3, PDF_CMAP_SINGLE, 7124 },
	{ 0x80e4, 0x80e4, PDF_CMAP_SINGLE, 2021 },
	{ 0x80e5, 0x80e5, PDF_CMAP_SINGLE, 2012 },
	{ 0x80e6, 0x80e6, PDF_CMAP_SINGLE, 7132 },
	{ 0x80e9, 0x80e9, PDF_CMAP_SINGLE, 18244 },
	{ 0x80ec, 0x80ec, PDF_CMAP_SINGLE, 17897 },
	{ 0x80ed, 0x80ed, PDF_CMAP_SINGLE, 2442 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 2451 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 2440 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 2438 },
	{ 0x80f2, 0x80f2, PDF_CMAP_SINGLE, 7607 },
	{ 0x80f3, 0x80f3, PDF_CMAP_SINGLE, 2446 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 2443 },
	{ 0x80f5, 0x80f5, PDF_CMAP_SINGLE, 7609 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 17780 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 2445 },
	{ 0x80f9, 0x80f9, PDF_CMAP_SINGLE, 7608 },
	{ 0x80fa, 0x80fa, PDF_CMAP_SINGLE, 7606 },
	{ 0x80fb, 0x80fb, PDF_CMAP_SINGLE, 7611 },
	{ 0x80fc, 0x80fc, PDF_CMAP_SINGLE, 2450 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 2448 },
	{ 0x80fe, 0x80fe, PDF_CMAP_SINGLE, 8828 },
	{ 0x8100, 0x8100, PDF_CMAP_SINGLE, 7612 },
	{ 0x8101, 0x8101, PDF_CMAP_SINGLE, 7610 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 2439 },
	{ 0x8103, 0x8103, PDF_CMAP_SINGLE, 16359 },
	{ 0x8105, 0x8105, PDF_CMAP_SINGLE, 2441 },
	{ 0x8106, 0x8106, PDF_CMAP_SINGLE, 2444 },
	{ 0x8107, 0x8107, PDF_CMAP_SINGLE, 14242 },
	{ 0x8108, 0x8108, PDF_CMAP_SINGLE, 2447 },
	{ 0x8109, 0x8109, PDF_CMAP_SINGLE, 15221 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 2449 },
	{ 0x810c, 0x810c, PDF_CMAP_SINGLE, 18246 },
	{ 0x810e, 0x810e, PDF_CMAP_SINGLE, 18247 },
	{ 0x8112, 0x8112, PDF_CMAP_SINGLE, 18248 },
	{ 0x8114, 0x8114, PDF_CMAP_SINGLE, 18249 },
	{ 0x8115, 0x8115, PDF_CMAP_SINGLE, 8207 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 2923 },
	{ 0x8117, 0x8117, PDF_CMAP_SINGLE, 15981 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 8198 },
	{ 0x8119, 0x8119, PDF_CMAP_SINGLE, 8200 },
	{ 0x811a, 0x811a, PDF_CMAP_SINGLE, 14243 },
	{ 0x811b, 0x811b, PDF_CMAP_SINGLE, 8201 },
	{ 0x811d, 0x811d, PDF_CMAP_SINGLE, 8209 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 8205 },
	{ 0x811f, 0x811f, PDF_CMAP_SINGLE, 8203 },
	{ 0x8121, 0x8121, PDF_CMAP_SINGLE, 8206 },
	{ 0x8122, 0x8122, PDF_CMAP_SINGLE, 8210 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 2924 },
	{ 0x8124, 0x8124, PDF_CMAP_SINGLE, 2928 },
	{ 0x8125, 0x8125, PDF_CMAP_SINGLE, 8199 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 8208 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 2926 },
	{ 0x812a, 0x812a, PDF_CMAP_SINGLE, 14905 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 2925 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 8204 },
	{ 0x812d, 0x812d, PDF_CMAP_SINGLE, 8202 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 2922 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 2927 },
	{ 0x8132, 0x8132, PDF_CMAP_SINGLE, 15993 },
	{ 0x8134, 0x8134, PDF_CMAP_SINGLE, 17900 },
	{ 0x8137, 0x8137, PDF_CMAP_SINGLE, 15218 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 3366 },
	{ 0x813a, 0x813a, PDF_CMAP_SINGLE, 8837 },
	{ 0x813d, 0x813d, PDF_CMAP_SINGLE, 8835 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 3368 },
	{ 0x8142, 0x8142, PDF_CMAP_SINGLE, 15222 },
	{ 0x8143, 0x8143, PDF_CMAP_SINGLE, 8830 },
	{ 0x8144, 0x8144, PDF_CMAP_SINGLE, 9511 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 3367 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 8834 },
	{ 0x8148, 0x8148, PDF_CMAP_SINGLE, 15990 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 8831 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 3363 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 3369 },
	{ 0x814d, 0x814d, PDF_CMAP_SINGLE, 8836 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 3365 },
	{ 0x814f, 0x814f, PDF_CMAP_SINGLE, 8833 },
	{ 0x8150, 0x8150, PDF_CMAP_SINGLE, 4213 },
	{ 0x8151, 0x8151, PDF_CMAP_SINGLE, 3364 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 8832 },
	{ 0x8153, 0x8153, PDF_CMAP_SINGLE, 3370 },
	{ 0x8154, 0x8154, PDF_CMAP_SINGLE, 3362 },
	{ 0x8155, 0x8155, PDF_CMAP_SINGLE, 3361 },
	{ 0x8156, 0x8156, PDF_CMAP_SINGLE, 18252 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 18253 },
	{ 0x815b, 0x815b, PDF_CMAP_SINGLE, 9503 },
	{ 0x815c, 0x815c, PDF_CMAP_SINGLE, 9501 },
	{ 0x815e, 0x815e, PDF_CMAP_SINGLE, 9507 },
	{ 0x8160, 0x8160, PDF_CMAP_SINGLE, 9499 },
	{ 0x8161, 0x8161, PDF_CMAP_SINGLE, 9512 },
	{ 0x8162, 0x8162, PDF_CMAP_SINGLE, 9504 },
	{ 0x8164, 0x8164, PDF_CMAP_SINGLE, 9498 },
	{ 0x8165, 0x8165, PDF_CMAP_SINGLE, 3794 },
	{ 0x8166, 0x8166, PDF_CMAP_SINGLE, 3800 },
	{ 0x8167, 0x8167, PDF_CMAP_SINGLE, 9509 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 9502 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 3797 },
	{ 0x816c, 0x816c, PDF_CMAP_SINGLE, 19053 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 17348 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 3795 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 9510 },
	{ 0x8170, 0x8170, PDF_CMAP_SINGLE, 3792 },
	{ 0x8171, 0x8171, PDF_CMAP_SINGLE, 3791 },
	{ 0x8172, 0x8172, PDF_CMAP_SINGLE, 9505 },
	{ 0x8173, 0x8173, PDF_CMAP_SINGLE, 3796 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 3371 },
	{ 0x8176, 0x8176, PDF_CMAP_SINGLE, 9508 },
	{ 0x8177, 0x8177, PDF_CMAP_SINGLE, 9500 },
	{ 0x8178, 0x8178, PDF_CMAP_SINGLE, 3793 },
	{ 0x8179, 0x817a, PDF_CMAP_RANGE, 3798 },
	{ 0x817c, 0x817c, PDF_CMAP_SINGLE, 18256 },
	{ 0x817f, 0x817f, PDF_CMAP_SINGLE, 4218 },
	{ 0x8180, 0x8180, PDF_CMAP_SINGLE, 4214 },
	{ 0x8182, 0x8182, PDF_CMAP_SINGLE, 4219 },
	{ 0x8183, 0x8183, PDF_CMAP_SINGLE, 10144 },
	{ 0x8184, 0x8184, PDF_CMAP_SINGLE, 17349 },
	{ 0x8186, 0x8186, PDF_CMAP_SINGLE, 10143 },
	{ 0x8187, 0x8187, PDF_CMAP_SINGLE, 10145 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 4216 },
	{ 0x8189, 0x8189, PDF_CMAP_SINGLE, 10142 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 4217 },
	{ 0x818b, 0x818b, PDF_CMAP_SINGLE, 10148 },
	{ 0x818c, 0x818c, PDF_CMAP_SINGLE, 10147 },
	{ 0x818d, 0x818d, PDF_CMAP_SINGLE, 10146 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 4215 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 17350 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 10705 },
	{ 0x8197, 0x8197, PDF_CMAP_SINGLE, 10708 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 4591 },
	{ 0x8199, 0x8199, PDF_CMAP_SINGLE, 10707 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 4590 },
	{ 0x819b, 0x819d, PDF_CMAP_RANGE, 4586 },
	{ 0x819e, 0x819e, PDF_CMAP_SINGLE, 10704 },
	{ 0x819f, 0x819f, PDF_CMAP_SINGLE, 10703 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 4589 },
	{ 0x81a2, 0x81a2, PDF_CMAP_SINGLE, 10706 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 10702 },
	{ 0x81a5, 0x81a5, PDF_CMAP_SINGLE, 18259 },
	{ 0x81a6, 0x81a6, PDF_CMAP_SINGLE, 11312 },
	{ 0x81a7, 0x81a7, PDF_CMAP_SINGLE, 11322 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 4913 },
	{ 0x81a9, 0x81a9, PDF_CMAP_SINGLE, 4912 },
	{ 0x81aa, 0x81aa, PDF_CMAP_SINGLE, 17252 },
	{ 0x81ab, 0x81ab, PDF_CMAP_SINGLE, 11316 },
	{ 0x81ac, 0x81ac, PDF_CMAP_SINGLE, 11318 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 11313 },
	{ 0x81b0, 0x81b0, PDF_CMAP_SINGLE, 11317 },
	{ 0x81b1, 0x81b1, PDF_CMAP_SINGLE, 11311 },
	{ 0x81b2, 0x81b2, PDF_CMAP_SINGLE, 11320 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 4911 },
	{ 0x81b4, 0x81b4, PDF_CMAP_SINGLE, 11319 },
	{ 0x81b5, 0x81b5, PDF_CMAP_SINGLE, 11315 },
	{ 0x81b6, 0x81b6, PDF_CMAP_SINGLE, 16050 },
	{ 0x81b7, 0x81b7, PDF_CMAP_SINGLE, 11321 },
	{ 0x81b9, 0x81b9, PDF_CMAP_SINGLE, 11314 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 5200 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 11782 },
	{ 0x81bc, 0x81bc, PDF_CMAP_SINGLE, 11788 },
	{ 0x81bd, 0x81bd, PDF_CMAP_SINGLE, 5204 },
	{ 0x81be, 0x81be, PDF_CMAP_SINGLE, 5206 },
	{ 0x81bf, 0x81bf, PDF_CMAP_SINGLE, 5203 },
	{ 0x81c0, 0x81c0, PDF_CMAP_SINGLE, 5202 },
	{ 0x81c1, 0x81c1, PDF_CMAP_SINGLE, 18261 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 5201 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 5199 },
	{ 0x81c4, 0x81c4, PDF_CMAP_SINGLE, 11783 },
	{ 0x81c5, 0x81c5, PDF_CMAP_SINGLE, 11786 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 5198 },
	{ 0x81c7, 0x81c7, PDF_CMAP_SINGLE, 11787 },
	{ 0x81c8, 0x81c8, PDF_CMAP_SINGLE, 16366 },
	{ 0x81c9, 0x81c9, PDF_CMAP_SINGLE, 5205 },
	{ 0x81ca, 0x81ca, PDF_CMAP_SINGLE, 11785 },
	{ 0x81cc, 0x81cc, PDF_CMAP_SINGLE, 11784 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 5400 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 5401 },
	{ 0x81d0, 0x81d0, PDF_CMAP_SINGLE, 12180 },
	{ 0x81d1, 0x81d2, PDF_CMAP_RANGE, 12178 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 12525 },
	{ 0x81d7, 0x81d7, PDF_CMAP_SINGLE, 12524 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 5555 },
	{ 0x81d9, 0x81d9, PDF_CMAP_SINGLE, 12828 },
	{ 0x81da, 0x81da, PDF_CMAP_SINGLE, 5685 },
	{ 0x81db, 0x81db, PDF_CMAP_SINGLE, 12827 },
	{ 0x81dd, 0x81dd, PDF_CMAP_SINGLE, 13045 },
	{ 0x81de, 0x81de, PDF_CMAP_SINGLE, 13221 },
	{ 0x81df, 0x81df, PDF_CMAP_SINGLE, 5844 },
	{ 0x81e0, 0x81e1, PDF_CMAP_RANGE, 13533 },
	{ 0x81e2, 0x81e2, PDF_CMAP_SINGLE, 5894 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 1029 },
	{ 0x81e4, 0x81e4, PDF_CMAP_SINGLE, 18262 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 1654 },
	{ 0x81e6, 0x81e6, PDF_CMAP_SINGLE, 8838 },
	{ 0x81e7, 0x81e7, PDF_CMAP_SINGLE, 4220 },
	{ 0x81e8, 0x81e8, PDF_CMAP_SINGLE, 5207 },
	{ 0x81e9, 0x81e9, PDF_CMAP_SINGLE, 11789 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 1030 },
	{ 0x81ec, 0x81ec, PDF_CMAP_SINGLE, 2453 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 2452 },
	{ 0x81ee, 0x81ee, PDF_CMAP_SINGLE, 8839 },
	{ 0x81ef, 0x81ef, PDF_CMAP_SINGLE, 17105 },
	{ 0x81f2, 0x81f2, PDF_CMAP_SINGLE, 11323 },
	{ 0x81f3, 0x81f3, PDF_CMAP_SINGLE, 1031 },
	{ 0x81f4, 0x81f4, PDF_CMAP_SINGLE, 2023 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 15713 },
	{ 0x81f7, 0x81f9, PDF_CMAP_RANGE, 8840 },
	{ 0x81fa, 0x81fa, PDF_CMAP_SINGLE, 4221 },
	{ 0x81fb, 0x81fb, PDF_CMAP_SINGLE, 4914 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 1032 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 1655 },
	{ 0x81ff, 0x81ff, PDF_CMAP_SINGLE, 7134 },
	{ 0x8200, 0x8200, PDF_CMAP_SINGLE, 2454 },
	{ 0x8201, 0x8201, PDF_CMAP_SINGLE, 7613 },
	{ 0x8202, 0x8202, PDF_CMAP_SINGLE, 2929 },
	{ 0x8204, 0x8204, PDF_CMAP_SINGLE, 8843 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 3801 },
	{ 0x8207, 0x8207, PDF_CMAP_SINGLE, 4222 },
	{ 0x8208, 0x8208, PDF_CMAP_SINGLE, 4915 },
	{ 0x8209, 0x8209, PDF_CMAP_SINGLE, 5208 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 5402 },
	{ 0x820b, 0x820b, PDF_CMAP_SINGLE, 12526 },
	{ 0x820c, 0x820c, PDF_CMAP_SINGLE, 1033 },
	{ 0x820d, 0x820d, PDF_CMAP_SINGLE, 1656 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 2455 },
	{ 0x8211, 0x8211, PDF_CMAP_SINGLE, 8211 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 3372 },
	{ 0x8214, 0x8214, PDF_CMAP_SINGLE, 4223 },
	{ 0x8215, 0x8215, PDF_CMAP_SINGLE, 10149 },
	{ 0x8216, 0x8216, PDF_CMAP_SINGLE, 10709 },
	{ 0x8218, 0x8218, PDF_CMAP_SINGLE, 14250 },
	{ 0x821a, 0x821a, PDF_CMAP_SINGLE, 16353 },
	{ 0x821b, 0x821b, PDF_CMAP_SINGLE, 1034 },
	{ 0x821c, 0x821c, PDF_CMAP_SINGLE, 3373 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 9513 },
	{ 0x821e, 0x821e, PDF_CMAP_SINGLE, 4224 },
	{ 0x821f, 0x821f, PDF_CMAP_SINGLE, 1035 },
	{ 0x8220, 0x8220, PDF_CMAP_SINGLE, 6702 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 7135 },
	{ 0x8222, 0x8222, PDF_CMAP_SINGLE, 2024 },
	{ 0x8225, 0x8225, PDF_CMAP_SINGLE, 7615 },
	{ 0x8226, 0x8226, PDF_CMAP_SINGLE, 16909 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 2458 },
	{ 0x8229, 0x8229, PDF_CMAP_SINGLE, 14256 },
	{ 0x822a, 0x822b, PDF_CMAP_RANGE, 2456 },
	{ 0x822c, 0x822c, PDF_CMAP_SINGLE, 2459 },
	{ 0x822d, 0x822d, PDF_CMAP_SINGLE, 16080 },
	{ 0x822f, 0x822f, PDF_CMAP_SINGLE, 7614 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 8216 },
	{ 0x8233, 0x8233, PDF_CMAP_SINGLE, 8213 },
	{ 0x8234, 0x8234, PDF_CMAP_SINGLE, 8215 },
	{ 0x8235, 0x8235, PDF_CMAP_SINGLE, 2930 },
	{ 0x8236, 0x8236, PDF_CMAP_SINGLE, 2932 },
	{ 0x8237, 0x8237, PDF_CMAP_SINGLE, 2931 },
	{ 0x8238, 0x8238, PDF_CMAP_SINGLE, 8212 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 2933 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 8214 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 8844 },
	{ 0x823e, 0x823e, PDF_CMAP_SINGLE, 17794 },
	{ 0x823f, 0x823f, PDF_CMAP_SINGLE, 8846 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 9516 },
	{ 0x8242, 0x8242, PDF_CMAP_SINGLE, 9517 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 9515 },
	{ 0x8245, 0x8245, PDF_CMAP_SINGLE, 9518 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3802 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 9514 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 4225 },
	{ 0x824e, 0x824e, PDF_CMAP_SINGLE, 10714 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 10710 },
	{ 0x8250, 0x8250, PDF_CMAP_SINGLE, 10713 },
	{ 0x8251, 0x8251, PDF_CMAP_SINGLE, 10715 },
	{ 0x8252, 0x8252, PDF_CMAP_SINGLE, 10712 },
	{ 0x8253, 0x8253, PDF_CMAP_SINGLE, 10711 },
	{ 0x8254, 0x8254, PDF_CMAP_SINGLE, 18263 },
	{ 0x8255, 0x8257, PDF_CMAP_RANGE, 11324 },
	{ 0x8258, 0x8259, PDF_CMAP_RANGE, 4916 },
	{ 0x825a, 0x825a, PDF_CMAP_SINGLE, 11791 },
	{ 0x825b, 0x825b, PDF_CMAP_SINGLE, 11790 },
	{ 0x825c, 0x825c, PDF_CMAP_SINGLE, 11792 },
	{ 0x825e, 0x825e, PDF_CMAP_SINGLE, 12182 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 12181 },
	{ 0x8261, 0x8261, PDF_CMAP_SINGLE, 12528 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 15226 },
	{ 0x8263, 0x8263, PDF_CMAP_SINGLE, 12529 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 12527 },
	{ 0x8265, 0x8265, PDF_CMAP_SINGLE, 15227 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 5686 },
	{ 0x8268, 0x8269, PDF_CMAP_RANGE, 12829 },
	{ 0x826b, 0x826b, PDF_CMAP_SINGLE, 13222 },
	{ 0x826c, 0x826c, PDF_CMAP_SINGLE, 13357 },
	{ 0x826d, 0x826d, PDF_CMAP_SINGLE, 13462 },
	{ 0x826e, 0x826e, PDF_CMAP_SINGLE, 1036 },
	{ 0x826f, 0x826f, PDF_CMAP_SINGLE, 1284 },
	{ 0x8271, 0x8271, PDF_CMAP_SINGLE, 5209 },
	{ 0x8272, 0x8272, PDF_CMAP_SINGLE, 1037 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 8217 },
	{ 0x8275, 0x8275, PDF_CMAP_SINGLE, 8847 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 18265 },
	{ 0x8277, 0x8277, PDF_CMAP_SINGLE, 5934 },
	{ 0x8278, 0x8278, PDF_CMAP_SINGLE, 6163 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 18210 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 17763 },
	{ 0x827b, 0x827b, PDF_CMAP_SINGLE, 14280 },
	{ 0x827c, 0x827c, PDF_CMAP_SINGLE, 6164 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 6166 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 1038 },
	{ 0x827f, 0x827f, PDF_CMAP_SINGLE, 6167 },
	{ 0x8280, 0x8280, PDF_CMAP_SINGLE, 6165 },
	{ 0x8283, 0x8284, PDF_CMAP_RANGE, 6373 },
	{ 0x8285, 0x8285, PDF_CMAP_SINGLE, 6368 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 16690 },
	{ 0x828a, 0x828a, PDF_CMAP_SINGLE, 6372 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 1286 },
	{ 0x828d, 0x828d, PDF_CMAP_SINGLE, 1287 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 6369 },
	{ 0x828f, 0x828f, PDF_CMAP_SINGLE, 6367 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 6366 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 6370 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 1285 },
	{ 0x8293, 0x8293, PDF_CMAP_SINGLE, 6371 },
	{ 0x8294, 0x8294, PDF_CMAP_SINGLE, 7136 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 6707 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 1659 },
	{ 0x829a, 0x829a, PDF_CMAP_SINGLE, 6706 },
	{ 0x829b, 0x829b, PDF_CMAP_SINGLE, 6708 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 1658 },
	{ 0x829e, 0x829e, PDF_CMAP_SINGLE, 6713 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 1662 },
	{ 0x82a0, 0x82a0, PDF_CMAP_SINGLE, 6703 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 6717 },
	{ 0x82a2, 0x82a2, PDF_CMAP_SINGLE, 6723 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 1669 },
	{ 0x82a4, 0x82a4, PDF_CMAP_SINGLE, 6720 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 1666 },
	{ 0x82a6, 0x82a6, PDF_CMAP_SINGLE, 15731 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 6710 },
	{ 0x82a8, 0x82a8, PDF_CMAP_SINGLE, 6716 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 6718 },
	{ 0x82aa, 0x82aa, PDF_CMAP_SINGLE, 14268 },
	{ 0x82ab, 0x82ab, PDF_CMAP_SINGLE, 6705 },
	{ 0x82ac, 0x82ac, PDF_CMAP_SINGLE, 1665 },
	{ 0x82ad, 0x82ad, PDF_CMAP_SINGLE, 1660 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 6711 },
	{ 0x82af, 0x82af, PDF_CMAP_SINGLE, 1667 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 1670 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1664 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 1657 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 6715 },
	{ 0x82b5, 0x82b5, PDF_CMAP_SINGLE, 6709 },
	{ 0x82b6, 0x82b6, PDF_CMAP_SINGLE, 6722 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 1672 },
	{ 0x82b8, 0x82b8, PDF_CMAP_SINGLE, 1668 },
	{ 0x82b9, 0x82b9, PDF_CMAP_SINGLE, 1663 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 6714 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 2460 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 6712 },
	{ 0x82bd, 0x82bd, PDF_CMAP_SINGLE, 1661 },
	{ 0x82be, 0x82be, PDF_CMAP_SINGLE, 1671 },
	{ 0x82bf, 0x82bf, PDF_CMAP_SINGLE, 18953 },
	{ 0x82c0, 0x82c0, PDF_CMAP_SINGLE, 6704 },
	{ 0x82c2, 0x82c2, PDF_CMAP_SINGLE, 6719 },
	{ 0x82c3, 0x82c3, PDF_CMAP_SINGLE, 6721 },
	{ 0x82c4, 0x82c4, PDF_CMAP_SINGLE, 17891 },
	{ 0x82ca, 0x82ca, PDF_CMAP_SINGLE, 18266 },
	{ 0x82cf, 0x82cf, PDF_CMAP_SINGLE, 17764 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 14273 },
	{ 0x82d1, 0x82d1, PDF_CMAP_SINGLE, 2041 },
	{ 0x82d2, 0x82d2, PDF_CMAP_SINGLE, 2035 },
	{ 0x82d3, 0x82d3, PDF_CMAP_SINGLE, 2043 },
	{ 0x82d4, 0x82d4, PDF_CMAP_SINGLE, 2040 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 7143 },
	{ 0x82d6, 0x82d6, PDF_CMAP_SINGLE, 7146 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 2036 },
	{ 0x82d8, 0x82d8, PDF_CMAP_SINGLE, 18267 },
	{ 0x82d9, 0x82d9, PDF_CMAP_SINGLE, 7137 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 2029 },
	{ 0x82dc, 0x82dc, PDF_CMAP_SINGLE, 2039 },
	{ 0x82de, 0x82de, PDF_CMAP_SINGLE, 2042 },
	{ 0x82df, 0x82df, PDF_CMAP_SINGLE, 2044 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 7158 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7149 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 14281 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 2028 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 7157 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 2032 },
	{ 0x82e6, 0x82e6, PDF_CMAP_SINGLE, 2030 },
	{ 0x82e7, 0x82e7, PDF_CMAP_SINGLE, 2025 },
	{ 0x82e8, 0x82e8, PDF_CMAP_SINGLE, 7141 },
	{ 0x82ea, 0x82ea, PDF_CMAP_SINGLE, 7156 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 7145 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7148 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 7161 },
	{ 0x82ee, 0x82ee, PDF_CMAP_SINGLE, 16408 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 2045 },
	{ 0x82f0, 0x82f0, PDF_CMAP_SINGLE, 7155 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 2037 },
	{ 0x82f2, 0x82f2, PDF_CMAP_SINGLE, 7150 },
	{ 0x82f3, 0x82f3, PDF_CMAP_SINGLE, 7160 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 7147 },
	{ 0x82f5, 0x82f5, PDF_CMAP_SINGLE, 7151 },
	{ 0x82f6, 0x82f6, PDF_CMAP_SINGLE, 7154 },
	{ 0x82f7, 0x82f7, PDF_CMAP_SINGLE, 15991 },
	{ 0x82f8, 0x82f8, PDF_CMAP_SINGLE, 18930 },
	{ 0x82f9, 0x82f9, PDF_CMAP_SINGLE, 7139 },
	{ 0x82fa, 0x82fa, PDF_CMAP_SINGLE, 7159 },
	{ 0x82fb, 0x82fb, PDF_CMAP_SINGLE, 7153 },
	{ 0x82fc, 0x82fc, PDF_CMAP_SINGLE, 16685 },
	{ 0x82fd, 0x82fd, PDF_CMAP_SINGLE, 15762 },
	{ 0x82fe, 0x82fe, PDF_CMAP_SINGLE, 7138 },
	{ 0x82ff, 0x82ff, PDF_CMAP_SINGLE, 17374 },
	{ 0x8300, 0x8300, PDF_CMAP_SINGLE, 7142 },
	{ 0x8301, 0x8301, PDF_CMAP_SINGLE, 2038 },
	{ 0x8302, 0x8302, PDF_CMAP_SINGLE, 2033 },
	{ 0x8303, 0x8303, PDF_CMAP_SINGLE, 2026 },
	{ 0x8304, 0x8304, PDF_CMAP_SINGLE, 2031 },
	{ 0x8305, 0x8305, PDF_CMAP_SINGLE, 2027 },
	{ 0x8306, 0x8306, PDF_CMAP_SINGLE, 2046 },
	{ 0x8307, 0x8307, PDF_CMAP_SINGLE, 7140 },
	{ 0x8308, 0x8308, PDF_CMAP_SINGLE, 7632 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 2034 },
	{ 0x830b, 0x830b, PDF_CMAP_SINGLE, 16054 },
	{ 0x830c, 0x830c, PDF_CMAP_SINGLE, 7152 },
	{ 0x830d, 0x830d, PDF_CMAP_SINGLE, 6781 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 7635 },
	{ 0x8317, 0x8317, PDF_CMAP_SINGLE, 2474 },
	{ 0x8318, 0x8318, PDF_CMAP_SINGLE, 14282 },
	{ 0x8319, 0x8319, PDF_CMAP_SINGLE, 7619 },
	{ 0x831a, 0x831a, PDF_CMAP_SINGLE, 14274 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 7630 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 7626 },
	{ 0x831d, 0x831d, PDF_CMAP_SINGLE, 14289 },
	{ 0x831e, 0x831e, PDF_CMAP_SINGLE, 7645 },
	{ 0x8320, 0x8320, PDF_CMAP_SINGLE, 7637 },
	{ 0x8322, 0x8322, PDF_CMAP_SINGLE, 7627 },
	{ 0x8324, 0x8324, PDF_CMAP_SINGLE, 7636 },
	{ 0x8325, 0x8325, PDF_CMAP_SINGLE, 7621 },
	{ 0x8326, 0x8326, PDF_CMAP_SINGLE, 7625 },
	{ 0x8327, 0x8327, PDF_CMAP_SINGLE, 7648 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 2477 },
	{ 0x8329, 0x8329, PDF_CMAP_SINGLE, 7640 },
	{ 0x832a, 0x832a, PDF_CMAP_SINGLE, 7631 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 2461 },
	{ 0x832c, 0x832c, PDF_CMAP_SINGLE, 7646 },
	{ 0x832d, 0x832d, PDF_CMAP_SINGLE, 7617 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 7639 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 2476 },
	{ 0x8332, 0x8332, PDF_CMAP_SINGLE, 2471 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 7616 },
	{ 0x8334, 0x8334, PDF_CMAP_SINGLE, 2469 },
	{ 0x8335, 0x8335, PDF_CMAP_SINGLE, 2468 },
	{ 0x8336, 0x8336, PDF_CMAP_SINGLE, 2473 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 7638 },
	{ 0x8338, 0x8338, PDF_CMAP_SINGLE, 2465 },
	{ 0x8339, 0x8339, PDF_CMAP_SINGLE, 2472 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 7144 },
	{ 0x833b, 0x833b, PDF_CMAP_SINGLE, 8848 },
	{ 0x833c, 0x833c, PDF_CMAP_SINGLE, 7633 },
	{ 0x833d, 0x833d, PDF_CMAP_SINGLE, 15658 },
	{ 0x833f, 0x833f, PDF_CMAP_SINGLE, 7623 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 2475 },
	{ 0x8341, 0x8341, PDF_CMAP_SINGLE, 7624 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7628 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 2478 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7618 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 7642 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 7641 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7649 },
	{ 0x8349, 0x8349, PDF_CMAP_SINGLE, 2467 },
	{ 0x834a, 0x834a, PDF_CMAP_SINGLE, 2464 },
	{ 0x834b, 0x834b, PDF_CMAP_SINGLE, 7647 },
	{ 0x834c, 0x834c, PDF_CMAP_SINGLE, 7643 },
	{ 0x834d, 0x834d, PDF_CMAP_SINGLE, 7634 },
	{ 0x834e, 0x834e, PDF_CMAP_SINGLE, 7629 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 2470 },
	{ 0x8350, 0x8350, PDF_CMAP_SINGLE, 2466 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 7620 },
	{ 0x8352, 0x8352, PDF_CMAP_SINGLE, 2462 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 7644 },
	{ 0x8354, 0x8354, PDF_CMAP_SINGLE, 2463 },
	{ 0x8356, 0x8356, PDF_CMAP_SINGLE, 7622 },
	{ 0x8357, 0x8357, PDF_CMAP_SINGLE, 18269 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 14262 },
	{ 0x8363, 0x8363, PDF_CMAP_SINGLE, 16875 },
	{ 0x8366, 0x8366, PDF_CMAP_SINGLE, 15902 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 17765 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 8223 },
	{ 0x8374, 0x8374, PDF_CMAP_SINGLE, 8225 },
	{ 0x8375, 0x8375, PDF_CMAP_SINGLE, 8230 },
	{ 0x8376, 0x8376, PDF_CMAP_SINGLE, 8250 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 2947 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 2937 },
	{ 0x837a, 0x837a, PDF_CMAP_SINGLE, 8222 },
	{ 0x837b, 0x837c, PDF_CMAP_RANGE, 2948 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 8233 },
	{ 0x837e, 0x837e, PDF_CMAP_SINGLE, 8240 },
	{ 0x837f, 0x837f, PDF_CMAP_SINGLE, 8246 },
	{ 0x8381, 0x8381, PDF_CMAP_SINGLE, 8227 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 8234 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 14291 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 2950 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 8248 },
	{ 0x8388, 0x8388, PDF_CMAP_SINGLE, 8243 },
	{ 0x8389, 0x8389, PDF_CMAP_SINGLE, 2945 },
	{ 0x838a, 0x838a, PDF_CMAP_SINGLE, 2943 },
	{ 0x838b, 0x838b, PDF_CMAP_SINGLE, 8239 },
	{ 0x838c, 0x838c, PDF_CMAP_SINGLE, 8235 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 8221 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 2934 },
	{ 0x838f, 0x838f, PDF_CMAP_SINGLE, 8226 },
	{ 0x8390, 0x8390, PDF_CMAP_SINGLE, 8218 },
	{ 0x8391, 0x8391, PDF_CMAP_SINGLE, 14316 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 2942 },
	{ 0x8393, 0x8393, PDF_CMAP_SINGLE, 2944 },
	{ 0x8394, 0x8394, PDF_CMAP_SINGLE, 8231 },
	{ 0x8395, 0x8395, PDF_CMAP_SINGLE, 8228 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 2939 },
	{ 0x8397, 0x8397, PDF_CMAP_SINGLE, 8244 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 2936 },
	{ 0x8399, 0x8399, PDF_CMAP_SINGLE, 8229 },
	{ 0x839a, 0x839a, PDF_CMAP_SINGLE, 8889 },
	{ 0x839b, 0x839b, PDF_CMAP_SINGLE, 8237 },
	{ 0x839c, 0x839c, PDF_CMAP_SINGLE, 14157 },
	{ 0x839d, 0x839d, PDF_CMAP_SINGLE, 8236 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 2935 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 2946 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 2938 },
	{ 0x83a3, 0x83a3, PDF_CMAP_SINGLE, 8219 },
	{ 0x83a4, 0x83a4, PDF_CMAP_SINGLE, 8224 },
	{ 0x83a5, 0x83a5, PDF_CMAP_SINGLE, 8241 },
	{ 0x83a6, 0x83a6, PDF_CMAP_SINGLE, 8247 },
	{ 0x83a7, 0x83a7, PDF_CMAP_SINGLE, 2951 },
	{ 0x83a8, 0x83a8, PDF_CMAP_SINGLE, 8220 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 8232 },
	{ 0x83aa, 0x83aa, PDF_CMAP_SINGLE, 8238 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 2941 },
	{ 0x83ac, 0x83ac, PDF_CMAP_SINGLE, 14294 },
	{ 0x83ae, 0x83ae, PDF_CMAP_SINGLE, 8249 },
	{ 0x83af, 0x83af, PDF_CMAP_SINGLE, 8242 },
	{ 0x83b0, 0x83b0, PDF_CMAP_SINGLE, 8245 },
	{ 0x83b9, 0x83b9, PDF_CMAP_SINGLE, 17063 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 2940 },
	{ 0x83be, 0x83be, PDF_CMAP_SINGLE, 16217 },
	{ 0x83bf, 0x83bf, PDF_CMAP_SINGLE, 8864 },
	{ 0x83c0, 0x83c0, PDF_CMAP_SINGLE, 8852 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 3381 },
	{ 0x83c2, 0x83c2, PDF_CMAP_SINGLE, 8881 },
	{ 0x83c3, 0x83c3, PDF_CMAP_SINGLE, 8890 },
	{ 0x83c4, 0x83c4, PDF_CMAP_SINGLE, 8893 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 3379 },
	{ 0x83c6, 0x83c6, PDF_CMAP_SINGLE, 8860 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 8885 },
	{ 0x83c8, 0x83c8, PDF_CMAP_SINGLE, 8861 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 8875 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 3392 },
	{ 0x83cb, 0x83cb, PDF_CMAP_SINGLE, 8871 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 3389 },
	{ 0x83cd, 0x83cd, PDF_CMAP_SINGLE, 16469 },
	{ 0x83ce, 0x83ce, PDF_CMAP_SINGLE, 8872 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 8849 },
	{ 0x83d1, 0x83d1, PDF_CMAP_SINGLE, 8886 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 14296 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 3398 },
	{ 0x83d5, 0x83d5, PDF_CMAP_SINGLE, 8883 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 8873 },
	{ 0x83d7, 0x83d7, PDF_CMAP_SINGLE, 8895 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 8868 },
	{ 0x83d9, 0x83d9, PDF_CMAP_SINGLE, 9550 },
	{ 0x83db, 0x83db, PDF_CMAP_SINGLE, 8898 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 3396 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 8866 },
	{ 0x83de, 0x83de, PDF_CMAP_SINGLE, 8878 },
	{ 0x83df, 0x83df, PDF_CMAP_SINGLE, 3399 },
	{ 0x83e0, 0x83e0, PDF_CMAP_SINGLE, 3378 },
	{ 0x83e1, 0x83e1, PDF_CMAP_SINGLE, 8870 },
	{ 0x83e2, 0x83e2, PDF_CMAP_SINGLE, 8896 },
	{ 0x83e3, 0x83e3, PDF_CMAP_SINGLE, 8863 },
	{ 0x83e4, 0x83e4, PDF_CMAP_SINGLE, 8856 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 8867 },
	{ 0x83e7, 0x83e7, PDF_CMAP_SINGLE, 8855 },
	{ 0x83e8, 0x83e8, PDF_CMAP_SINGLE, 8853 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 3374 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 8887 },
	{ 0x83eb, 0x83eb, PDF_CMAP_SINGLE, 8862 },
	{ 0x83ec, 0x83ec, PDF_CMAP_SINGLE, 8891 },
	{ 0x83ed, 0x83ed, PDF_CMAP_SINGLE, 15660 },
	{ 0x83ee, 0x83ee, PDF_CMAP_SINGLE, 8892 },
	{ 0x83ef, 0x83ef, PDF_CMAP_SINGLE, 3382 },
	{ 0x83f0, 0x83f0, PDF_CMAP_SINGLE, 3387 },
	{ 0x83f1, 0x83f1, PDF_CMAP_SINGLE, 3383 },
	{ 0x83f2, 0x83f2, PDF_CMAP_SINGLE, 3391 },
	{ 0x83f3, 0x83f3, PDF_CMAP_SINGLE, 8882 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 3384 },
	{ 0x83f5, 0x83f5, PDF_CMAP_SINGLE, 8874 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 8858 },
	{ 0x83f8, 0x83f8, PDF_CMAP_SINGLE, 3376 },
	{ 0x83f9, 0x83f9, PDF_CMAP_SINGLE, 8850 },
	{ 0x83fa, 0x83fa, PDF_CMAP_SINGLE, 8884 },
	{ 0x83fb, 0x83fb, PDF_CMAP_SINGLE, 8894 },
	{ 0x83fc, 0x83fc, PDF_CMAP_SINGLE, 8857 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 3390 },
	{ 0x83fe, 0x83fe, PDF_CMAP_SINGLE, 8899 },
	{ 0x83ff, 0x83ff, PDF_CMAP_SINGLE, 8869 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 8865 },
	{ 0x8403, 0x8403, PDF_CMAP_SINGLE, 3375 },
	{ 0x8404, 0x8404, PDF_CMAP_SINGLE, 3395 },
	{ 0x8405, 0x8405, PDF_CMAP_SINGLE, 18271 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 8880 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 3397 },
	{ 0x8409, 0x8409, PDF_CMAP_SINGLE, 8876 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 3386 },
	{ 0x840b, 0x840b, PDF_CMAP_SINGLE, 3380 },
	{ 0x840c, 0x840c, PDF_CMAP_SINGLE, 3388 },
	{ 0x840d, 0x840d, PDF_CMAP_SINGLE, 3377 },
	{ 0x840e, 0x840e, PDF_CMAP_SINGLE, 3394 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 8877 },
	{ 0x8410, 0x8410, PDF_CMAP_SINGLE, 8859 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 8879 },
	{ 0x8412, 0x8412, PDF_CMAP_SINGLE, 8854 },
	{ 0x8413, 0x8413, PDF_CMAP_SINGLE, 8888 },
	{ 0x8414, 0x8414, PDF_CMAP_SINGLE, 15376 },
	{ 0x8416, 0x8416, PDF_CMAP_SINGLE, 16439 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 15994 },
	{ 0x841b, 0x841b, PDF_CMAP_SINGLE, 8897 },
	{ 0x841c, 0x841c, PDF_CMAP_SINGLE, 16051 },
	{ 0x8420, 0x8420, PDF_CMAP_SINGLE, 15445 },
	{ 0x8421, 0x8421, PDF_CMAP_SINGLE, 14161 },
	{ 0x8423, 0x8423, PDF_CMAP_SINGLE, 8851 },
	{ 0x8424, 0x8424, PDF_CMAP_SINGLE, 18769 },
	{ 0x8426, 0x8426, PDF_CMAP_SINGLE, 18136 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 9549 },
	{ 0x842b, 0x842b, PDF_CMAP_SINGLE, 9571 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 3756 },
	{ 0x842d, 0x842d, PDF_CMAP_SINGLE, 9554 },
	{ 0x842e, 0x842e, PDF_CMAP_SINGLE, 16941 },
	{ 0x842f, 0x842f, PDF_CMAP_SINGLE, 9552 },
	{ 0x8430, 0x8430, PDF_CMAP_SINGLE, 9531 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 3806 },
	{ 0x8432, 0x8432, PDF_CMAP_SINGLE, 9547 },
	{ 0x8433, 0x8433, PDF_CMAP_SINGLE, 9567 },
	{ 0x8434, 0x8434, PDF_CMAP_SINGLE, 9543 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 3814 },
	{ 0x8436, 0x8436, PDF_CMAP_SINGLE, 9566 },
	{ 0x8437, 0x8437, PDF_CMAP_SINGLE, 9541 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 3393 },
	{ 0x8439, 0x8439, PDF_CMAP_SINGLE, 9557 },
	{ 0x843a, 0x843a, PDF_CMAP_SINGLE, 9542 },
	{ 0x843b, 0x843b, PDF_CMAP_SINGLE, 9564 },
	{ 0x843c, 0x843c, PDF_CMAP_SINGLE, 3813 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 3805 },
	{ 0x843e, 0x843e, PDF_CMAP_SINGLE, 15649 },
	{ 0x843f, 0x843f, PDF_CMAP_SINGLE, 9520 },
	{ 0x8440, 0x8440, PDF_CMAP_SINGLE, 9528 },
	{ 0x8442, 0x8442, PDF_CMAP_SINGLE, 9553 },
	{ 0x8443, 0x8443, PDF_CMAP_SINGLE, 9545 },
	{ 0x8444, 0x8444, PDF_CMAP_SINGLE, 9570 },
	{ 0x8445, 0x8445, PDF_CMAP_SINGLE, 9548 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 3819 },
	{ 0x8447, 0x8447, PDF_CMAP_SINGLE, 9565 },
	{ 0x8448, 0x8448, PDF_CMAP_SINGLE, 18200 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 3810 },
	{ 0x844a, 0x844a, PDF_CMAP_SINGLE, 14308 },
	{ 0x844b, 0x844b, PDF_CMAP_SINGLE, 9551 },
	{ 0x844c, 0x844c, PDF_CMAP_SINGLE, 9559 },
	{ 0x844d, 0x844d, PDF_CMAP_SINGLE, 9532 },
	{ 0x844e, 0x844e, PDF_CMAP_SINGLE, 9558 },
	{ 0x8450, 0x8450, PDF_CMAP_SINGLE, 9575 },
	{ 0x8451, 0x8451, PDF_CMAP_SINGLE, 9527 },
	{ 0x8452, 0x8452, PDF_CMAP_SINGLE, 9560 },
	{ 0x8453, 0x8453, PDF_CMAP_SINGLE, 15229 },
	{ 0x8454, 0x8454, PDF_CMAP_SINGLE, 9573 },
	{ 0x8455, 0x8455, PDF_CMAP_SINGLE, 14731 },
	{ 0x8456, 0x8456, PDF_CMAP_SINGLE, 9521 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 3385 },
	{ 0x8458, 0x8458, PDF_CMAP_SINGLE, 14302 },
	{ 0x8459, 0x8459, PDF_CMAP_SINGLE, 9535 },
	{ 0x845a, 0x845a, PDF_CMAP_SINGLE, 9534 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 3812 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 16091 },
	{ 0x845d, 0x845d, PDF_CMAP_SINGLE, 9538 },
	{ 0x845e, 0x845e, PDF_CMAP_SINGLE, 9540 },
	{ 0x845f, 0x845f, PDF_CMAP_SINGLE, 9555 },
	{ 0x8460, 0x8460, PDF_CMAP_SINGLE, 9572 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 3815 },
	{ 0x8462, 0x8462, PDF_CMAP_SINGLE, 17114 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 3816 },
	{ 0x8464, 0x8464, PDF_CMAP_SINGLE, 18273 },
	{ 0x8465, 0x8465, PDF_CMAP_SINGLE, 9526 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 3808 },
	{ 0x8467, 0x8467, PDF_CMAP_SINGLE, 9530 },
	{ 0x8468, 0x8468, PDF_CMAP_SINGLE, 9568 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 3817 },
	{ 0x846b, 0x846b, PDF_CMAP_SINGLE, 3809 },
	{ 0x846c, 0x846c, PDF_CMAP_SINGLE, 3811 },
	{ 0x846d, 0x846d, PDF_CMAP_SINGLE, 3818 },
	{ 0x846e, 0x846e, PDF_CMAP_SINGLE, 9574 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 9561 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 9556 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 14304 },
	{ 0x8472, 0x8472, PDF_CMAP_SINGLE, 15507 },
	{ 0x8473, 0x8473, PDF_CMAP_SINGLE, 9537 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 9536 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 3807 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 9522 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 3804 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 9546 },
	{ 0x8479, 0x8479, PDF_CMAP_SINGLE, 9523 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 9544 },
	{ 0x847d, 0x847d, PDF_CMAP_SINGLE, 9533 },
	{ 0x847e, 0x847e, PDF_CMAP_SINGLE, 9569 },
	{ 0x847f, 0x847f, PDF_CMAP_SINGLE, 14670 },
	{ 0x8480, 0x8480, PDF_CMAP_SINGLE, 18121 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 3803 },
	{ 0x8484, 0x8484, PDF_CMAP_SINGLE, 19103 },
	{ 0x8486, 0x8486, PDF_CMAP_SINGLE, 9529 },
	{ 0x8488, 0x8488, PDF_CMAP_SINGLE, 18274 },
	{ 0x848d, 0x848d, PDF_CMAP_SINGLE, 9525 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 9563 },
	{ 0x848f, 0x848f, PDF_CMAP_SINGLE, 9524 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 4238 },
	{ 0x8491, 0x8491, PDF_CMAP_SINGLE, 10189 },
	{ 0x8492, 0x8492, PDF_CMAP_SINGLE, 14166 },
	{ 0x8493, 0x8493, PDF_CMAP_SINGLE, 14318 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 10172 },
	{ 0x8495, 0x8495, PDF_CMAP_SINGLE, 15214 },
	{ 0x8496, 0x8496, PDF_CMAP_SINGLE, 18118 },
	{ 0x8497, 0x8497, PDF_CMAP_SINGLE, 10150 },
	{ 0x8498, 0x8498, PDF_CMAP_SINGLE, 10180 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 4230 },
	{ 0x849a, 0x849a, PDF_CMAP_SINGLE, 10165 },
	{ 0x849b, 0x849b, PDF_CMAP_SINGLE, 10175 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 4233 },
	{ 0x849d, 0x849d, PDF_CMAP_SINGLE, 10168 },
	{ 0x849e, 0x849e, PDF_CMAP_SINGLE, 4231 },
	{ 0x849f, 0x849f, PDF_CMAP_SINGLE, 10153 },
	{ 0x84a0, 0x84a0, PDF_CMAP_SINGLE, 10183 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 10152 },
	{ 0x84a2, 0x84a2, PDF_CMAP_SINGLE, 10171 },
	{ 0x84a3, 0x84a3, PDF_CMAP_SINGLE, 15262 },
	{ 0x84a4, 0x84a4, PDF_CMAP_SINGLE, 10151 },
	{ 0x84a7, 0x84a7, PDF_CMAP_SINGLE, 10169 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 10178 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 10176 },
	{ 0x84aa, 0x84aa, PDF_CMAP_SINGLE, 10164 },
	{ 0x84ab, 0x84ab, PDF_CMAP_SINGLE, 10159 },
	{ 0x84ac, 0x84ac, PDF_CMAP_SINGLE, 10157 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 15756 },
	{ 0x84ae, 0x84ae, PDF_CMAP_SINGLE, 10158 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 10177 },
	{ 0x84b0, 0x84b0, PDF_CMAP_SINGLE, 10188 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 10166 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 4232 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 10161 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 10181 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 4235 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 10160 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 10154 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 10170 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 4239 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 15436 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 18276 },
	{ 0x84bf, 0x84bf, PDF_CMAP_SINGLE, 4227 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 4236 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 10162 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 10156 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 4229 },
	{ 0x84c5, 0x84c5, PDF_CMAP_SINGLE, 9562 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 4228 },
	{ 0x84c7, 0x84c7, PDF_CMAP_SINGLE, 10173 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 4226 },
	{ 0x84ca, 0x84ca, PDF_CMAP_SINGLE, 4241 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 4234 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 10174 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 10163 },
	{ 0x84ce, 0x84ce, PDF_CMAP_SINGLE, 10155 },
	{ 0x84cf, 0x84cf, PDF_CMAP_SINGLE, 10182 },
	{ 0x84d0, 0x84d0, PDF_CMAP_SINGLE, 10167 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 4240 },
	{ 0x84d2, 0x84d2, PDF_CMAP_SINGLE, 10186 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 4237 },
	{ 0x84d4, 0x84d4, PDF_CMAP_SINGLE, 10185 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 10179 },
	{ 0x84d7, 0x84d7, PDF_CMAP_SINGLE, 10184 },
	{ 0x84da, 0x84da, PDF_CMAP_SINGLE, 16373 },
	{ 0x84db, 0x84db, PDF_CMAP_SINGLE, 10187 },
	{ 0x84de, 0x84de, PDF_CMAP_SINGLE, 14314 },
	{ 0x84e1, 0x84e1, PDF_CMAP_SINGLE, 18277 },
	{ 0x84e2, 0x84e2, PDF_CMAP_SINGLE, 14261 },
	{ 0x84e4, 0x84e4, PDF_CMAP_SINGLE, 14319 },
	{ 0x84e5, 0x84e5, PDF_CMAP_SINGLE, 15312 },
	{ 0x84e7, 0x84e8, PDF_CMAP_RANGE, 10755 },
	{ 0x84e9, 0x84e9, PDF_CMAP_SINGLE, 10743 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 10742 },
	{ 0x84eb, 0x84eb, PDF_CMAP_SINGLE, 10738 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 4603 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4595 },
	{ 0x84ef, 0x84ef, PDF_CMAP_SINGLE, 10758 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 10757 },
	{ 0x84f1, 0x84f1, PDF_CMAP_SINGLE, 9519 },
	{ 0x84f2, 0x84f2, PDF_CMAP_SINGLE, 10735 },
	{ 0x84f3, 0x84f3, PDF_CMAP_SINGLE, 10739 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 10733 },
	{ 0x84f6, 0x84f6, PDF_CMAP_SINGLE, 10752 },
	{ 0x84f7, 0x84f7, PDF_CMAP_SINGLE, 10737 },
	{ 0x84f8, 0x84f8, PDF_CMAP_SINGLE, 18278 },
	{ 0x84f9, 0x84f9, PDF_CMAP_SINGLE, 10759 },
	{ 0x84fa, 0x84fa, PDF_CMAP_SINGLE, 10730 },
	{ 0x84fb, 0x84fb, PDF_CMAP_SINGLE, 10728 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 10740 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 10750 },
	{ 0x84fe, 0x84fe, PDF_CMAP_SINGLE, 10745 },
	{ 0x84ff, 0x84ff, PDF_CMAP_SINGLE, 4605 },
	{ 0x8500, 0x8500, PDF_CMAP_SINGLE, 10719 },
	{ 0x8502, 0x8502, PDF_CMAP_SINGLE, 10749 },
	{ 0x8503, 0x8503, PDF_CMAP_SINGLE, 14728 },
	{ 0x8504, 0x8504, PDF_CMAP_SINGLE, 19147 },
	{ 0x8505, 0x8505, PDF_CMAP_SINGLE, 16619 },
	{ 0x8506, 0x8506, PDF_CMAP_SINGLE, 4606 },
	{ 0x8507, 0x8507, PDF_CMAP_SINGLE, 9539 },
	{ 0x8508, 0x8508, PDF_CMAP_SINGLE, 10731 },
	{ 0x8509, 0x8509, PDF_CMAP_SINGLE, 10722 },
	{ 0x850a, 0x850a, PDF_CMAP_SINGLE, 10725 },
	{ 0x850b, 0x850b, PDF_CMAP_SINGLE, 10763 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 10732 },
	{ 0x850d, 0x850d, PDF_CMAP_SINGLE, 10723 },
	{ 0x850e, 0x850e, PDF_CMAP_SINGLE, 10721 },
	{ 0x850f, 0x850f, PDF_CMAP_SINGLE, 10718 },
	{ 0x8510, 0x8510, PDF_CMAP_SINGLE, 18279 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 4599 },
	{ 0x8512, 0x8512, PDF_CMAP_SINGLE, 10741 },
	{ 0x8513, 0x8513, PDF_CMAP_SINGLE, 4598 },
	{ 0x8514, 0x8514, PDF_CMAP_SINGLE, 4602 },
	{ 0x8515, 0x8515, PDF_CMAP_SINGLE, 10736 },
	{ 0x8516, 0x8516, PDF_CMAP_SINGLE, 10744 },
	{ 0x8517, 0x8517, PDF_CMAP_SINGLE, 4592 },
	{ 0x8518, 0x8518, PDF_CMAP_SINGLE, 10760 },
	{ 0x8519, 0x8519, PDF_CMAP_SINGLE, 10764 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 4594 },
	{ 0x851c, 0x851c, PDF_CMAP_SINGLE, 10727 },
	{ 0x851d, 0x851d, PDF_CMAP_SINGLE, 10747 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 10751 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 10724 },
	{ 0x8520, 0x8520, PDF_CMAP_SINGLE, 10761 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 4601 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 4600 },
	{ 0x8524, 0x8524, PDF_CMAP_SINGLE, 10716 },
	{ 0x8525, 0x8525, PDF_CMAP_SINGLE, 4604 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 10754 },
	{ 0x8527, 0x8527, PDF_CMAP_SINGLE, 10726 },
	{ 0x8528, 0x8528, PDF_CMAP_SINGLE, 10746 },
	{ 0x8529, 0x8529, PDF_CMAP_SINGLE, 10720 },
	{ 0x852a, 0x852a, PDF_CMAP_SINGLE, 10734 },
	{ 0x852b, 0x852b, PDF_CMAP_SINGLE, 10729 },
	{ 0x852c, 0x852d, PDF_CMAP_RANGE, 4596 },
	{ 0x852e, 0x852e, PDF_CMAP_SINGLE, 10748 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 10765 },
	{ 0x8530, 0x8530, PDF_CMAP_SINGLE, 10762 },
	{ 0x8531, 0x8531, PDF_CMAP_SINGLE, 10753 },
	{ 0x8533, 0x8533, PDF_CMAP_SINGLE, 16936 },
	{ 0x8534, 0x8534, PDF_CMAP_SINGLE, 14324 },
	{ 0x8538, 0x8538, PDF_CMAP_SINGLE, 18280 },
	{ 0x853b, 0x853b, PDF_CMAP_SINGLE, 10717 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 4593 },
	{ 0x853e, 0x853e, PDF_CMAP_SINGLE, 11343 },
	{ 0x8540, 0x8540, PDF_CMAP_SINGLE, 11334 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 11337 },
	{ 0x8542, 0x8542, PDF_CMAP_SINGLE, 16482 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 4923 },
	{ 0x8544, 0x8544, PDF_CMAP_SINGLE, 11339 },
	{ 0x8545, 0x8545, PDF_CMAP_SINGLE, 11328 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 11335 },
	{ 0x8547, 0x8547, PDF_CMAP_SINGLE, 11341 },
	{ 0x8548, 0x8548, PDF_CMAP_SINGLE, 4920 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 4924 },
	{ 0x854a, 0x854a, PDF_CMAP_SINGLE, 4918 },
	{ 0x854b, 0x854b, PDF_CMAP_SINGLE, 14272 },
	{ 0x854c, 0x854c, PDF_CMAP_SINGLE, 16399 },
	{ 0x854d, 0x854d, PDF_CMAP_SINGLE, 11330 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 11346 },
	{ 0x8551, 0x8551, PDF_CMAP_SINGLE, 11340 },
	{ 0x8552, 0x8552, PDF_CMAP_SINGLE, 18281 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 11331 },
	{ 0x8554, 0x8554, PDF_CMAP_SINGLE, 11355 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 11349 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 11327 },
	{ 0x8557, 0x8557, PDF_CMAP_SINGLE, 11811 },
	{ 0x8558, 0x8558, PDF_CMAP_SINGLE, 11333 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 4919 },
	{ 0x855a, 0x855a, PDF_CMAP_SINGLE, 14299 },
	{ 0x855b, 0x855b, PDF_CMAP_SINGLE, 11344 },
	{ 0x855d, 0x855d, PDF_CMAP_SINGLE, 11354 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 4927 },
	{ 0x855f, 0x855f, PDF_CMAP_SINGLE, 18938 },
	{ 0x8560, 0x8560, PDF_CMAP_SINGLE, 11351 },
	{ 0x8561, 0x8561, PDF_CMAP_SINGLE, 11332 },
	{ 0x8562, 0x8562, PDF_CMAP_SINGLE, 11338 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 11342 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 11336 },
	{ 0x8565, 0x8565, PDF_CMAP_SINGLE, 11356 },
	{ 0x8566, 0x8566, PDF_CMAP_SINGLE, 11353 },
	{ 0x8567, 0x8567, PDF_CMAP_SINGLE, 11350 },
	{ 0x8568, 0x8569, PDF_CMAP_RANGE, 4921 },
	{ 0x856a, 0x856a, PDF_CMAP_SINGLE, 4926 },
	{ 0x856b, 0x856b, PDF_CMAP_SINGLE, 11329 },
	{ 0x856c, 0x856c, PDF_CMAP_SINGLE, 11357 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 4925 },
	{ 0x856e, 0x856e, PDF_CMAP_SINGLE, 11347 },
	{ 0x856f, 0x8570, PDF_CMAP_RANGE, 18283 },
	{ 0x8571, 0x8571, PDF_CMAP_SINGLE, 11345 },
	{ 0x8573, 0x8573, PDF_CMAP_SINGLE, 14328 },
	{ 0x8575, 0x8575, PDF_CMAP_SINGLE, 11348 },
	{ 0x8576, 0x8576, PDF_CMAP_SINGLE, 11824 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 11805 },
	{ 0x8578, 0x8578, PDF_CMAP_SINGLE, 11810 },
	{ 0x8579, 0x8579, PDF_CMAP_SINGLE, 11823 },
	{ 0x857a, 0x857a, PDF_CMAP_SINGLE, 11809 },
	{ 0x857b, 0x857b, PDF_CMAP_SINGLE, 11801 },
	{ 0x857c, 0x857c, PDF_CMAP_SINGLE, 11806 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 5212 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 11794 },
	{ 0x8581, 0x8581, PDF_CMAP_SINGLE, 11818 },
	{ 0x8582, 0x8582, PDF_CMAP_SINGLE, 11820 },
	{ 0x8583, 0x8583, PDF_CMAP_SINGLE, 11793 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 5211 },
	{ 0x8585, 0x8585, PDF_CMAP_SINGLE, 11822 },
	{ 0x8586, 0x8586, PDF_CMAP_SINGLE, 11814 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 5218 },
	{ 0x8588, 0x8588, PDF_CMAP_SINGLE, 11821 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 11807 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 5220 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 11799 },
	{ 0x858c, 0x858c, PDF_CMAP_SINGLE, 11352 },
	{ 0x858d, 0x858d, PDF_CMAP_SINGLE, 11815 },
	{ 0x858e, 0x858e, PDF_CMAP_SINGLE, 11812 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 11795 },
	{ 0x8590, 0x8590, PDF_CMAP_SINGLE, 11826 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 5214 },
	{ 0x8593, 0x8593, PDF_CMAP_SINGLE, 19018 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 5215 },
	{ 0x8595, 0x8595, PDF_CMAP_SINGLE, 11797 },
	{ 0x8596, 0x8596, PDF_CMAP_SINGLE, 11813 },
	{ 0x8597, 0x8597, PDF_CMAP_SINGLE, 18931 },
	{ 0x8598, 0x8598, PDF_CMAP_SINGLE, 11825 },
	{ 0x8599, 0x8599, PDF_CMAP_SINGLE, 11816 },
	{ 0x859a, 0x859a, PDF_CMAP_SINGLE, 11803 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 5217 },
	{ 0x859c, 0x859c, PDF_CMAP_SINGLE, 5213 },
	{ 0x859d, 0x859d, PDF_CMAP_SINGLE, 11817 },
	{ 0x859e, 0x859e, PDF_CMAP_SINGLE, 11804 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 11827 },
	{ 0x85a0, 0x85a0, PDF_CMAP_SINGLE, 11798 },
	{ 0x85a1, 0x85a1, PDF_CMAP_SINGLE, 11808 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 11819 },
	{ 0x85a3, 0x85a3, PDF_CMAP_SINGLE, 11800 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 11802 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 5221 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 11796 },
	{ 0x85a8, 0x85a8, PDF_CMAP_SINGLE, 5219 },
	{ 0x85a9, 0x85a9, PDF_CMAP_SINGLE, 5404 },
	{ 0x85aa, 0x85aa, PDF_CMAP_SINGLE, 5210 },
	{ 0x85af, 0x85af, PDF_CMAP_SINGLE, 5216 },
	{ 0x85b0, 0x85b0, PDF_CMAP_SINGLE, 5408 },
	{ 0x85b1, 0x85b1, PDF_CMAP_SINGLE, 12198 },
	{ 0x85b3, 0x85b3, PDF_CMAP_SINGLE, 12188 },
	{ 0x85b4, 0x85b4, PDF_CMAP_SINGLE, 12183 },
	{ 0x85b5, 0x85b5, PDF_CMAP_SINGLE, 12189 },
	{ 0x85b6, 0x85b6, PDF_CMAP_SINGLE, 12199 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 12203 },
	{ 0x85b8, 0x85b8, PDF_CMAP_SINGLE, 12202 },
	{ 0x85b9, 0x85b9, PDF_CMAP_SINGLE, 5410 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 5409 },
	{ 0x85bd, 0x85bd, PDF_CMAP_SINGLE, 12190 },
	{ 0x85be, 0x85be, PDF_CMAP_SINGLE, 12204 },
	{ 0x85bf, 0x85bf, PDF_CMAP_SINGLE, 12193 },
	{ 0x85c0, 0x85c0, PDF_CMAP_SINGLE, 12185 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 14334 },
	{ 0x85c2, 0x85c2, PDF_CMAP_SINGLE, 12187 },
	{ 0x85c3, 0x85c3, PDF_CMAP_SINGLE, 12186 },
	{ 0x85c4, 0x85c4, PDF_CMAP_SINGLE, 12192 },
	{ 0x85c5, 0x85c5, PDF_CMAP_SINGLE, 12197 },
	{ 0x85c6, 0x85c6, PDF_CMAP_SINGLE, 12184 },
	{ 0x85c7, 0x85c7, PDF_CMAP_SINGLE, 12191 },
	{ 0x85c8, 0x85c8, PDF_CMAP_SINGLE, 12196 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 5407 },
	{ 0x85cb, 0x85cb, PDF_CMAP_SINGLE, 12194 },
	{ 0x85cd, 0x85cd, PDF_CMAP_SINGLE, 5405 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 12195 },
	{ 0x85cf, 0x85cf, PDF_CMAP_SINGLE, 5403 },
	{ 0x85d0, 0x85d0, PDF_CMAP_SINGLE, 5406 },
	{ 0x85d1, 0x85d1, PDF_CMAP_SINGLE, 12545 },
	{ 0x85d2, 0x85d2, PDF_CMAP_SINGLE, 12200 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 5559 },
	{ 0x85d6, 0x85d6, PDF_CMAP_SINGLE, 18951 },
	{ 0x85d7, 0x85d7, PDF_CMAP_SINGLE, 12537 },
	{ 0x85d8, 0x85d8, PDF_CMAP_SINGLE, 12541 },
	{ 0x85d9, 0x85d9, PDF_CMAP_SINGLE, 12533 },
	{ 0x85da, 0x85da, PDF_CMAP_SINGLE, 12536 },
	{ 0x85dc, 0x85dc, PDF_CMAP_SINGLE, 12544 },
	{ 0x85dd, 0x85dd, PDF_CMAP_SINGLE, 5557 },
	{ 0x85de, 0x85de, PDF_CMAP_SINGLE, 12549 },
	{ 0x85df, 0x85df, PDF_CMAP_SINGLE, 12542 },
	{ 0x85e0, 0x85e0, PDF_CMAP_SINGLE, 18285 },
	{ 0x85e1, 0x85e1, PDF_CMAP_SINGLE, 12534 },
	{ 0x85e2, 0x85e2, PDF_CMAP_SINGLE, 12550 },
	{ 0x85e3, 0x85e3, PDF_CMAP_SINGLE, 12543 },
	{ 0x85e4, 0x85e5, PDF_CMAP_RANGE, 5560 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 12547 },
	{ 0x85e8, 0x85e8, PDF_CMAP_SINGLE, 12535 },
	{ 0x85e9, 0x85e9, PDF_CMAP_SINGLE, 5556 },
	{ 0x85ea, 0x85ea, PDF_CMAP_SINGLE, 5558 },
	{ 0x85eb, 0x85eb, PDF_CMAP_SINGLE, 12530 },
	{ 0x85ec, 0x85ec, PDF_CMAP_SINGLE, 12538 },
	{ 0x85ed, 0x85ed, PDF_CMAP_SINGLE, 12532 },
	{ 0x85ee, 0x85ee, PDF_CMAP_SINGLE, 15446 },
	{ 0x85ef, 0x85ef, PDF_CMAP_SINGLE, 12548 },
	{ 0x85f0, 0x85f0, PDF_CMAP_SINGLE, 12546 },
	{ 0x85f1, 0x85f1, PDF_CMAP_SINGLE, 12531 },
	{ 0x85f2, 0x85f2, PDF_CMAP_SINGLE, 12539 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 12837 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 5562 },
	{ 0x85f8, 0x85f8, PDF_CMAP_SINGLE, 12540 },
	{ 0x85f9, 0x85f9, PDF_CMAP_SINGLE, 5688 },
	{ 0x85fa, 0x85fa, PDF_CMAP_SINGLE, 5690 },
	{ 0x85fb, 0x85fb, PDF_CMAP_SINGLE, 5687 },
	{ 0x85fc, 0x85fc, PDF_CMAP_SINGLE, 15440 },
	{ 0x85fd, 0x85fd, PDF_CMAP_SINGLE, 12842 },
	{ 0x85fe, 0x85fe, PDF_CMAP_SINGLE, 12834 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 12832 },
	{ 0x8600, 0x8600, PDF_CMAP_SINGLE, 12836 },
	{ 0x8601, 0x8601, PDF_CMAP_SINGLE, 12833 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 14336 },
	{ 0x8604, 0x8604, PDF_CMAP_SINGLE, 12838 },
	{ 0x8605, 0x8605, PDF_CMAP_SINGLE, 12840 },
	{ 0x8606, 0x8606, PDF_CMAP_SINGLE, 5691 },
	{ 0x8607, 0x8607, PDF_CMAP_SINGLE, 5693 },
	{ 0x8609, 0x8609, PDF_CMAP_SINGLE, 12839 },
	{ 0x860a, 0x860a, PDF_CMAP_SINGLE, 5694 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 5692 },
	{ 0x860c, 0x860c, PDF_CMAP_SINGLE, 12841 },
	{ 0x860d, 0x860d, PDF_CMAP_SINGLE, 16484 },
	{ 0x860f, 0x860f, PDF_CMAP_SINGLE, 18882 },
	{ 0x8610, 0x8610, PDF_CMAP_SINGLE, 15231 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 5689 },
	{ 0x8613, 0x8613, PDF_CMAP_SINGLE, 19107 },
	{ 0x8614, 0x8614, PDF_CMAP_SINGLE, 18110 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 14331 },
	{ 0x8617, 0x8617, PDF_CMAP_SINGLE, 5765 },
	{ 0x8618, 0x8618, PDF_CMAP_SINGLE, 13046 },
	{ 0x8619, 0x8619, PDF_CMAP_SINGLE, 13052 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 5767 },
	{ 0x861b, 0x861b, PDF_CMAP_SINGLE, 12835 },
	{ 0x861c, 0x861c, PDF_CMAP_SINGLE, 13051 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 13058 },
	{ 0x861f, 0x861f, PDF_CMAP_SINGLE, 13049 },
	{ 0x8620, 0x8620, PDF_CMAP_SINGLE, 13056 },
	{ 0x8621, 0x8621, PDF_CMAP_SINGLE, 13055 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 12831 },
	{ 0x8623, 0x8623, PDF_CMAP_SINGLE, 13050 },
	{ 0x8624, 0x8624, PDF_CMAP_SINGLE, 12201 },
	{ 0x8625, 0x8625, PDF_CMAP_SINGLE, 13059 },
	{ 0x8626, 0x8626, PDF_CMAP_SINGLE, 13048 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 13053 },
	{ 0x8628, 0x8628, PDF_CMAP_SINGLE, 14341 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 13057 },
	{ 0x862a, 0x862a, PDF_CMAP_SINGLE, 13047 },
	{ 0x862c, 0x862c, PDF_CMAP_SINGLE, 13226 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 5766 },
	{ 0x862e, 0x862e, PDF_CMAP_SINGLE, 13054 },
	{ 0x862f, 0x862f, PDF_CMAP_SINGLE, 16286 },
	{ 0x8631, 0x8631, PDF_CMAP_SINGLE, 13362 },
	{ 0x8632, 0x8632, PDF_CMAP_SINGLE, 13227 },
	{ 0x8633, 0x8633, PDF_CMAP_SINGLE, 13225 },
	{ 0x8634, 0x8635, PDF_CMAP_RANGE, 13223 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 13228 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 5895 },
	{ 0x8639, 0x8639, PDF_CMAP_SINGLE, 13360 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 13358 },
	{ 0x863b, 0x863b, PDF_CMAP_SINGLE, 13363 },
	{ 0x863c, 0x863c, PDF_CMAP_SINGLE, 13361 },
	{ 0x863e, 0x863e, PDF_CMAP_SINGLE, 13364 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 5896 },
	{ 0x8640, 0x8640, PDF_CMAP_SINGLE, 13359 },
	{ 0x8642, 0x8642, PDF_CMAP_SINGLE, 14529 },
	{ 0x8643, 0x8643, PDF_CMAP_SINGLE, 13463 },
	{ 0x8645, 0x8645, PDF_CMAP_SINGLE, 15206 },
	{ 0x8646, 0x8648, PDF_CMAP_RANGE, 13535 },
	{ 0x864b, 0x864b, PDF_CMAP_SINGLE, 13630 },
	{ 0x864c, 0x864c, PDF_CMAP_SINGLE, 13622 },
	{ 0x864d, 0x864d, PDF_CMAP_SINGLE, 6168 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 1673 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 2047 },
	{ 0x8652, 0x8652, PDF_CMAP_SINGLE, 7651 },
	{ 0x8653, 0x8653, PDF_CMAP_SINGLE, 7650 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 2479 },
	{ 0x8655, 0x8655, PDF_CMAP_SINGLE, 2952 },
	{ 0x8656, 0x8656, PDF_CMAP_SINGLE, 8252 },
	{ 0x8659, 0x8659, PDF_CMAP_SINGLE, 8251 },
	{ 0x865b, 0x865b, PDF_CMAP_SINGLE, 3400 },
	{ 0x865c, 0x865c, PDF_CMAP_SINGLE, 3821 },
	{ 0x865e, 0x865e, PDF_CMAP_SINGLE, 3820 },
	{ 0x865f, 0x865f, PDF_CMAP_SINGLE, 3822 },
	{ 0x8661, 0x8661, PDF_CMAP_SINGLE, 10190 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 10766 },
	{ 0x8663, 0x8663, PDF_CMAP_SINGLE, 11358 },
	{ 0x8664, 0x8664, PDF_CMAP_SINGLE, 11360 },
	{ 0x8665, 0x8665, PDF_CMAP_SINGLE, 11359 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 5222 },
	{ 0x8668, 0x8668, PDF_CMAP_SINGLE, 11828 },
	{ 0x8669, 0x8669, PDF_CMAP_SINGLE, 12205 },
	{ 0x866a, 0x866a, PDF_CMAP_SINGLE, 13598 },
	{ 0x866b, 0x866b, PDF_CMAP_SINGLE, 1039 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 14355 },
	{ 0x866d, 0x866e, PDF_CMAP_RANGE, 6726 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 6725 },
	{ 0x8670, 0x8670, PDF_CMAP_SINGLE, 6724 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 1674 },
	{ 0x8672, 0x8672, PDF_CMAP_SINGLE, 18287 },
	{ 0x8673, 0x8673, PDF_CMAP_SINGLE, 7165 },
	{ 0x8674, 0x8674, PDF_CMAP_SINGLE, 7163 },
	{ 0x8677, 0x8677, PDF_CMAP_SINGLE, 7162 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 2048 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 2050 },
	{ 0x867b, 0x867b, PDF_CMAP_SINGLE, 2049 },
	{ 0x867c, 0x867c, PDF_CMAP_SINGLE, 7164 },
	{ 0x867e, 0x867e, PDF_CMAP_SINGLE, 14346 },
	{ 0x8685, 0x8685, PDF_CMAP_SINGLE, 7663 },
	{ 0x8686, 0x8686, PDF_CMAP_SINGLE, 7660 },
	{ 0x8687, 0x8687, PDF_CMAP_SINGLE, 7658 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 2480 },
	{ 0x868b, 0x868b, PDF_CMAP_SINGLE, 7661 },
	{ 0x868c, 0x868c, PDF_CMAP_SINGLE, 2485 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 7655 },
	{ 0x868e, 0x868e, PDF_CMAP_SINGLE, 7670 },
	{ 0x8690, 0x8690, PDF_CMAP_SINGLE, 7672 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 7656 },
	{ 0x8692, 0x8692, PDF_CMAP_SINGLE, 18288 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 2482 },
	{ 0x8694, 0x8694, PDF_CMAP_SINGLE, 7673 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 7668 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 7654 },
	{ 0x8697, 0x8697, PDF_CMAP_SINGLE, 7659 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 7669 },
	{ 0x8699, 0x8699, PDF_CMAP_SINGLE, 7665 },
	{ 0x869a, 0x869a, PDF_CMAP_SINGLE, 7662 },
	{ 0x869c, 0x869c, PDF_CMAP_SINGLE, 2487 },
	{ 0x869d, 0x869d, PDF_CMAP_SINGLE, 7671 },
	{ 0x869e, 0x869e, PDF_CMAP_SINGLE, 7657 },
	{ 0x86a0, 0x86a0, PDF_CMAP_SINGLE, 15240 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 7666 },
	{ 0x86a2, 0x86a2, PDF_CMAP_SINGLE, 7652 },
	{ 0x86a3, 0x86a3, PDF_CMAP_SINGLE, 2486 },
	{ 0x86a4, 0x86a4, PDF_CMAP_SINGLE, 2483 },
	{ 0x86a5, 0x86a5, PDF_CMAP_SINGLE, 7664 },
	{ 0x86a7, 0x86a7, PDF_CMAP_SINGLE, 7667 },
	{ 0x86a8, 0x86a8, PDF_CMAP_SINGLE, 7653 },
	{ 0x86a9, 0x86a9, PDF_CMAP_SINGLE, 2484 },
	{ 0x86aa, 0x86aa, PDF_CMAP_SINGLE, 2481 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 16134 },
	{ 0x86af, 0x86af, PDF_CMAP_SINGLE, 2962 },
	{ 0x86b0, 0x86b0, PDF_CMAP_SINGLE, 8259 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 2961 },
	{ 0x86b2, 0x86b2, PDF_CMAP_SINGLE, 17376 },
	{ 0x86b3, 0x86b3, PDF_CMAP_SINGLE, 8262 },
	{ 0x86b4, 0x86b4, PDF_CMAP_SINGLE, 8265 },
	{ 0x86b5, 0x86b5, PDF_CMAP_SINGLE, 2958 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 2956 },
	{ 0x86b7, 0x86b7, PDF_CMAP_SINGLE, 8254 },
	{ 0x86b8, 0x86b8, PDF_CMAP_SINGLE, 8263 },
	{ 0x86b9, 0x86b9, PDF_CMAP_SINGLE, 8261 },
	{ 0x86ba, 0x86ba, PDF_CMAP_SINGLE, 8258 },
	{ 0x86bb, 0x86bc, PDF_CMAP_RANGE, 8266 },
	{ 0x86bd, 0x86be, PDF_CMAP_RANGE, 8269 },
	{ 0x86bf, 0x86bf, PDF_CMAP_SINGLE, 8253 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 2955 },
	{ 0x86c1, 0x86c1, PDF_CMAP_SINGLE, 8256 },
	{ 0x86c2, 0x86c2, PDF_CMAP_SINGLE, 8255 },
	{ 0x86c3, 0x86c3, PDF_CMAP_SINGLE, 8268 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 2957 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 8257 },
	{ 0x86c6, 0x86c6, PDF_CMAP_SINGLE, 2959 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 2954 },
	{ 0x86c8, 0x86c8, PDF_CMAP_SINGLE, 8260 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 2963 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 2960 },
	{ 0x86cc, 0x86cc, PDF_CMAP_SINGLE, 8264 },
	{ 0x86d0, 0x86d0, PDF_CMAP_SINGLE, 3407 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 8914 },
	{ 0x86d3, 0x86d3, PDF_CMAP_SINGLE, 8903 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 3404 },
	{ 0x86d6, 0x86d6, PDF_CMAP_SINGLE, 9581 },
	{ 0x86d7, 0x86d7, PDF_CMAP_SINGLE, 8912 },
	{ 0x86d8, 0x86d8, PDF_CMAP_SINGLE, 8900 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 3402 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 8905 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 3405 },
	{ 0x86dc, 0x86dc, PDF_CMAP_SINGLE, 8909 },
	{ 0x86dd, 0x86dd, PDF_CMAP_SINGLE, 8907 },
	{ 0x86de, 0x86de, PDF_CMAP_SINGLE, 3408 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 3401 },
	{ 0x86e2, 0x86e2, PDF_CMAP_SINGLE, 8901 },
	{ 0x86e3, 0x86e3, PDF_CMAP_SINGLE, 8904 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 3406 },
	{ 0x86e6, 0x86e6, PDF_CMAP_SINGLE, 8902 },
	{ 0x86e8, 0x86e8, PDF_CMAP_SINGLE, 8913 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 8911 },
	{ 0x86ea, 0x86ea, PDF_CMAP_SINGLE, 8906 },
	{ 0x86eb, 0x86eb, PDF_CMAP_SINGLE, 8908 },
	{ 0x86ec, 0x86ec, PDF_CMAP_SINGLE, 8910 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 3403 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 18289 },
	{ 0x86f5, 0x86f5, PDF_CMAP_SINGLE, 9582 },
	{ 0x86f6, 0x86f6, PDF_CMAP_SINGLE, 9588 },
	{ 0x86f7, 0x86f7, PDF_CMAP_SINGLE, 9578 },
	{ 0x86f8, 0x86f8, PDF_CMAP_SINGLE, 9584 },
	{ 0x86f9, 0x86f9, PDF_CMAP_SINGLE, 3823 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 9580 },
	{ 0x86fb, 0x86fb, PDF_CMAP_SINGLE, 3829 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 3828 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 3827 },
	{ 0x8701, 0x8701, PDF_CMAP_SINGLE, 9587 },
	{ 0x8702, 0x8703, PDF_CMAP_RANGE, 3830 },
	{ 0x8704, 0x8704, PDF_CMAP_SINGLE, 9577 },
	{ 0x8705, 0x8705, PDF_CMAP_SINGLE, 9590 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 3832 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 3826 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 3825 },
	{ 0x8709, 0x8709, PDF_CMAP_SINGLE, 9586 },
	{ 0x870a, 0x870a, PDF_CMAP_SINGLE, 3833 },
	{ 0x870b, 0x870b, PDF_CMAP_SINGLE, 9576 },
	{ 0x870c, 0x870c, PDF_CMAP_SINGLE, 9579 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 9589 },
	{ 0x870e, 0x870e, PDF_CMAP_SINGLE, 9585 },
	{ 0x8711, 0x8711, PDF_CMAP_SINGLE, 10222 },
	{ 0x8712, 0x8712, PDF_CMAP_SINGLE, 10211 },
	{ 0x8713, 0x8713, PDF_CMAP_SINGLE, 3824 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 4248 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 10199 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 10220 },
	{ 0x871b, 0x871b, PDF_CMAP_SINGLE, 10200 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 4243 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 10197 },
	{ 0x8720, 0x8720, PDF_CMAP_SINGLE, 10206 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 10198 },
	{ 0x8722, 0x8722, PDF_CMAP_SINGLE, 4245 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 10192 },
	{ 0x8724, 0x8724, PDF_CMAP_SINGLE, 10219 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 4246 },
	{ 0x8726, 0x8727, PDF_CMAP_RANGE, 10216 },
	{ 0x8728, 0x8728, PDF_CMAP_SINGLE, 10193 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 4251 },
	{ 0x872a, 0x872a, PDF_CMAP_SINGLE, 10208 },
	{ 0x872c, 0x872c, PDF_CMAP_SINGLE, 10202 },
	{ 0x872d, 0x872d, PDF_CMAP_SINGLE, 10209 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 10196 },
	{ 0x8730, 0x8730, PDF_CMAP_SINGLE, 10221 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 10213 },
	{ 0x8732, 0x8732, PDF_CMAP_SINGLE, 10207 },
	{ 0x8733, 0x8733, PDF_CMAP_SINGLE, 10191 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 4247 },
	{ 0x8735, 0x8735, PDF_CMAP_SINGLE, 10214 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 4250 },
	{ 0x8738, 0x8738, PDF_CMAP_SINGLE, 10218 },
	{ 0x873a, 0x873a, PDF_CMAP_SINGLE, 10212 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 4244 },
	{ 0x873c, 0x873c, PDF_CMAP_SINGLE, 10210 },
	{ 0x873e, 0x873e, PDF_CMAP_SINGLE, 10204 },
	{ 0x873f, 0x873f, PDF_CMAP_SINGLE, 4242 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 10195 },
	{ 0x8741, 0x8741, PDF_CMAP_SINGLE, 10203 },
	{ 0x8742, 0x8742, PDF_CMAP_SINGLE, 10215 },
	{ 0x8743, 0x8743, PDF_CMAP_SINGLE, 10201 },
	{ 0x8746, 0x8746, PDF_CMAP_SINGLE, 10205 },
	{ 0x874c, 0x874c, PDF_CMAP_SINGLE, 4616 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 9583 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 10784 },
	{ 0x874f, 0x874f, PDF_CMAP_SINGLE, 10793 },
	{ 0x8750, 0x8750, PDF_CMAP_SINGLE, 10783 },
	{ 0x8751, 0x8751, PDF_CMAP_SINGLE, 10779 },
	{ 0x8752, 0x8752, PDF_CMAP_SINGLE, 10776 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 4617 },
	{ 0x8754, 0x8754, PDF_CMAP_SINGLE, 10774 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 4249 },
	{ 0x8756, 0x8756, PDF_CMAP_SINGLE, 10767 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 4615 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 10773 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 4614 },
	{ 0x875a, 0x875a, PDF_CMAP_SINGLE, 10778 },
	{ 0x875b, 0x875b, PDF_CMAP_SINGLE, 10775 },
	{ 0x875c, 0x875c, PDF_CMAP_SINGLE, 10791 },
	{ 0x875d, 0x875d, PDF_CMAP_SINGLE, 10786 },
	{ 0x875e, 0x875e, PDF_CMAP_SINGLE, 10780 },
	{ 0x875f, 0x875f, PDF_CMAP_SINGLE, 10785 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 4610 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 10777 },
	{ 0x8762, 0x8762, PDF_CMAP_SINGLE, 10796 },
	{ 0x8763, 0x8764, PDF_CMAP_RANGE, 10768 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 10792 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 4611 },
	{ 0x8767, 0x8767, PDF_CMAP_SINGLE, 10797 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 4613 },
	{ 0x8769, 0x8769, PDF_CMAP_SINGLE, 10798 },
	{ 0x876a, 0x876a, PDF_CMAP_SINGLE, 10782 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 10194 },
	{ 0x876c, 0x876c, PDF_CMAP_SINGLE, 10788 },
	{ 0x876d, 0x876d, PDF_CMAP_SINGLE, 10781 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 10790 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 10787 },
	{ 0x8770, 0x8770, PDF_CMAP_SINGLE, 15980 },
	{ 0x8771, 0x8771, PDF_CMAP_SINGLE, 18087 },
	{ 0x8773, 0x8773, PDF_CMAP_SINGLE, 10772 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 4608 },
	{ 0x8775, 0x8775, PDF_CMAP_SINGLE, 10795 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 4609 },
	{ 0x8777, 0x8777, PDF_CMAP_SINGLE, 10770 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 4612 },
	{ 0x8779, 0x8779, PDF_CMAP_SINGLE, 11371 },
	{ 0x877a, 0x877a, PDF_CMAP_SINGLE, 10789 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 10794 },
	{ 0x877d, 0x877d, PDF_CMAP_SINGLE, 16453 },
	{ 0x8781, 0x8781, PDF_CMAP_SINGLE, 11368 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 4607 },
	{ 0x8783, 0x8783, PDF_CMAP_SINGLE, 4928 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 11378 },
	{ 0x8785, 0x8785, PDF_CMAP_SINGLE, 11374 },
	{ 0x8786, 0x8786, PDF_CMAP_SINGLE, 18662 },
	{ 0x8787, 0x8787, PDF_CMAP_SINGLE, 11372 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 11367 },
	{ 0x8789, 0x8789, PDF_CMAP_SINGLE, 11382 },
	{ 0x878b, 0x878b, PDF_CMAP_SINGLE, 17378 },
	{ 0x878c, 0x878c, PDF_CMAP_SINGLE, 16367 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 4932 },
	{ 0x878f, 0x878f, PDF_CMAP_SINGLE, 11363 },
	{ 0x8790, 0x8791, PDF_CMAP_RANGE, 11375 },
	{ 0x8792, 0x8792, PDF_CMAP_SINGLE, 11366 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 11365 },
	{ 0x8794, 0x8794, PDF_CMAP_SINGLE, 11379 },
	{ 0x8796, 0x8796, PDF_CMAP_SINGLE, 11369 },
	{ 0x8797, 0x8797, PDF_CMAP_SINGLE, 11364 },
	{ 0x8798, 0x8798, PDF_CMAP_SINGLE, 11370 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 11381 },
	{ 0x879b, 0x879b, PDF_CMAP_SINGLE, 11362 },
	{ 0x879c, 0x879c, PDF_CMAP_SINGLE, 11380 },
	{ 0x879d, 0x879d, PDF_CMAP_SINGLE, 11377 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 4930 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 4929 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 4931 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 11373 },
	{ 0x87a4, 0x87a4, PDF_CMAP_SINGLE, 11361 },
	{ 0x87a5, 0x87a5, PDF_CMAP_SINGLE, 15752 },
	{ 0x87a9, 0x87a9, PDF_CMAP_SINGLE, 16364 },
	{ 0x87aa, 0x87aa, PDF_CMAP_SINGLE, 11830 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 5228 },
	{ 0x87ac, 0x87ac, PDF_CMAP_SINGLE, 11834 },
	{ 0x87ad, 0x87ad, PDF_CMAP_SINGLE, 11831 },
	{ 0x87ae, 0x87ae, PDF_CMAP_SINGLE, 11838 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 11844 },
	{ 0x87b0, 0x87b0, PDF_CMAP_SINGLE, 11833 },
	{ 0x87b1, 0x87b1, PDF_CMAP_SINGLE, 14351 },
	{ 0x87b2, 0x87b2, PDF_CMAP_SINGLE, 11853 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 5225 },
	{ 0x87b4, 0x87b4, PDF_CMAP_SINGLE, 11847 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 11836 },
	{ 0x87b6, 0x87b6, PDF_CMAP_SINGLE, 11848 },
	{ 0x87b7, 0x87b7, PDF_CMAP_SINGLE, 11843 },
	{ 0x87b8, 0x87b8, PDF_CMAP_SINGLE, 11850 },
	{ 0x87b9, 0x87b9, PDF_CMAP_SINGLE, 11835 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 5230 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 5229 },
	{ 0x87bc, 0x87bc, PDF_CMAP_SINGLE, 11837 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 11851 },
	{ 0x87be, 0x87be, PDF_CMAP_SINGLE, 11829 },
	{ 0x87bf, 0x87bf, PDF_CMAP_SINGLE, 11849 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 5223 },
	{ 0x87c1, 0x87c1, PDF_CMAP_SINGLE, 16380 },
	{ 0x87c2, 0x87c2, PDF_CMAP_SINGLE, 11841 },
	{ 0x87c3, 0x87c3, PDF_CMAP_SINGLE, 11840 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 11845 },
	{ 0x87c5, 0x87c5, PDF_CMAP_SINGLE, 11832 },
	{ 0x87c6, 0x87c6, PDF_CMAP_SINGLE, 5227 },
	{ 0x87c8, 0x87c8, PDF_CMAP_SINGLE, 5231 },
	{ 0x87c9, 0x87c9, PDF_CMAP_SINGLE, 11839 },
	{ 0x87ca, 0x87ca, PDF_CMAP_SINGLE, 11846 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 5232 },
	{ 0x87cc, 0x87cc, PDF_CMAP_SINGLE, 11842 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 16365 },
	{ 0x87d1, 0x87d1, PDF_CMAP_SINGLE, 5224 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 5226 },
	{ 0x87d3, 0x87d3, PDF_CMAP_SINGLE, 12218 },
	{ 0x87d4, 0x87d4, PDF_CMAP_SINGLE, 12216 },
	{ 0x87d6, 0x87d6, PDF_CMAP_SINGLE, 16274 },
	{ 0x87d7, 0x87d7, PDF_CMAP_SINGLE, 12222 },
	{ 0x87d8, 0x87d8, PDF_CMAP_SINGLE, 12220 },
	{ 0x87d9, 0x87d9, PDF_CMAP_SINGLE, 12223 },
	{ 0x87da, 0x87da, PDF_CMAP_SINGLE, 14352 },
	{ 0x87db, 0x87db, PDF_CMAP_SINGLE, 12209 },
	{ 0x87dc, 0x87dc, PDF_CMAP_SINGLE, 12217 },
	{ 0x87dd, 0x87dd, PDF_CMAP_SINGLE, 12227 },
	{ 0x87de, 0x87de, PDF_CMAP_SINGLE, 11852 },
	{ 0x87df, 0x87df, PDF_CMAP_SINGLE, 12213 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 5414 },
	{ 0x87e1, 0x87e1, PDF_CMAP_SINGLE, 10771 },
	{ 0x87e2, 0x87e2, PDF_CMAP_SINGLE, 12208 },
	{ 0x87e3, 0x87e3, PDF_CMAP_SINGLE, 12221 },
	{ 0x87e4, 0x87e4, PDF_CMAP_SINGLE, 12215 },
	{ 0x87e5, 0x87e5, PDF_CMAP_SINGLE, 12212 },
	{ 0x87e6, 0x87e6, PDF_CMAP_SINGLE, 12207 },
	{ 0x87e7, 0x87e7, PDF_CMAP_SINGLE, 12206 },
	{ 0x87e8, 0x87e8, PDF_CMAP_SINGLE, 12226 },
	{ 0x87ea, 0x87ea, PDF_CMAP_SINGLE, 12211 },
	{ 0x87eb, 0x87eb, PDF_CMAP_SINGLE, 12210 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 5412 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 12219 },
	{ 0x87ee, 0x87ee, PDF_CMAP_SINGLE, 14349 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 5411 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 5413 },
	{ 0x87f3, 0x87f3, PDF_CMAP_SINGLE, 12214 },
	{ 0x87f4, 0x87f4, PDF_CMAP_SINGLE, 12225 },
	{ 0x87f5, 0x87f5, PDF_CMAP_SINGLE, 16302 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 12554 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 5566 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 12552 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 5563 },
	{ 0x87fc, 0x87fc, PDF_CMAP_SINGLE, 12560 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 5567 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 12562 },
	{ 0x8800, 0x8800, PDF_CMAP_SINGLE, 12551 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 12224 },
	{ 0x8802, 0x8802, PDF_CMAP_SINGLE, 12564 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 12553 },
	{ 0x8804, 0x8804, PDF_CMAP_SINGLE, 16047 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 5564 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 12559 },
	{ 0x8808, 0x8808, PDF_CMAP_SINGLE, 12561 },
	{ 0x8809, 0x8809, PDF_CMAP_SINGLE, 12556 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 12563 },
	{ 0x880b, 0x880b, PDF_CMAP_SINGLE, 12558 },
	{ 0x880c, 0x880c, PDF_CMAP_SINGLE, 12557 },
	{ 0x880d, 0x880d, PDF_CMAP_SINGLE, 5565 },
	{ 0x880f, 0x880f, PDF_CMAP_SINGLE, 14353 },
	{ 0x8810, 0x8811, PDF_CMAP_RANGE, 12844 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 12847 },
	{ 0x8814, 0x8815, PDF_CMAP_RANGE, 5695 },
	{ 0x8816, 0x8816, PDF_CMAP_SINGLE, 12848 },
	{ 0x8817, 0x8817, PDF_CMAP_SINGLE, 12846 },
	{ 0x8818, 0x8818, PDF_CMAP_SINGLE, 16250 },
	{ 0x8819, 0x8819, PDF_CMAP_SINGLE, 12843 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 13062 },
	{ 0x881c, 0x881c, PDF_CMAP_SINGLE, 13065 },
	{ 0x881d, 0x881d, PDF_CMAP_SINGLE, 13061 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 5771 },
	{ 0x8820, 0x8820, PDF_CMAP_SINGLE, 13063 },
	{ 0x8821, 0x8821, PDF_CMAP_SINGLE, 5770 },
	{ 0x8822, 0x8822, PDF_CMAP_SINGLE, 5769 },
	{ 0x8823, 0x8823, PDF_CMAP_SINGLE, 5768 },
	{ 0x8824, 0x8824, PDF_CMAP_SINGLE, 13064 },
	{ 0x8825, 0x8825, PDF_CMAP_SINGLE, 13233 },
	{ 0x8826, 0x8826, PDF_CMAP_SINGLE, 13231 },
	{ 0x8827, 0x8827, PDF_CMAP_SINGLE, 15879 },
	{ 0x8828, 0x8828, PDF_CMAP_SINGLE, 13230 },
	{ 0x8829, 0x8829, PDF_CMAP_SINGLE, 13060 },
	{ 0x882a, 0x882a, PDF_CMAP_SINGLE, 13232 },
	{ 0x882b, 0x882b, PDF_CMAP_SINGLE, 13066 },
	{ 0x882c, 0x882c, PDF_CMAP_SINGLE, 13229 },
	{ 0x882d, 0x882d, PDF_CMAP_SINGLE, 15243 },
	{ 0x882e, 0x882e, PDF_CMAP_SINGLE, 13367 },
	{ 0x882f, 0x882f, PDF_CMAP_SINGLE, 13333 },
	{ 0x8830, 0x8830, PDF_CMAP_SINGLE, 13365 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 5897 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 13366 },
	{ 0x8833, 0x8833, PDF_CMAP_SINGLE, 13368 },
	{ 0x8835, 0x8835, PDF_CMAP_SINGLE, 13466 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 5928 },
	{ 0x8837, 0x8837, PDF_CMAP_SINGLE, 13465 },
	{ 0x8838, 0x8838, PDF_CMAP_SINGLE, 13464 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 5929 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 5959 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 13576 },
	{ 0x883d, 0x883d, PDF_CMAP_SINGLE, 13600 },
	{ 0x883e, 0x883e, PDF_CMAP_SINGLE, 13599 },
	{ 0x883f, 0x883f, PDF_CMAP_SINGLE, 13601 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 1040 },
	{ 0x8841, 0x8841, PDF_CMAP_SINGLE, 7166 },
	{ 0x8842, 0x8842, PDF_CMAP_SINGLE, 16218 },
	{ 0x8843, 0x8844, PDF_CMAP_RANGE, 7674 },
	{ 0x8845, 0x8846, PDF_CMAP_RANGE, 14358 },
	{ 0x8848, 0x8848, PDF_CMAP_SINGLE, 8915 },
	{ 0x884a, 0x884a, PDF_CMAP_SINGLE, 13067 },
	{ 0x884b, 0x884b, PDF_CMAP_SINGLE, 13467 },
	{ 0x884c, 0x884c, PDF_CMAP_SINGLE, 1041 },
	{ 0x884d, 0x884d, PDF_CMAP_SINGLE, 2051 },
	{ 0x884e, 0x884e, PDF_CMAP_SINGLE, 7167 },
	{ 0x884f, 0x884f, PDF_CMAP_SINGLE, 14404 },
	{ 0x8852, 0x8852, PDF_CMAP_SINGLE, 8271 },
	{ 0x8853, 0x8853, PDF_CMAP_SINGLE, 2964 },
	{ 0x8855, 0x8855, PDF_CMAP_SINGLE, 8917 },
	{ 0x8856, 0x8856, PDF_CMAP_SINGLE, 8916 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 3409 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 3834 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 10799 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 4618 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 4619 },
	{ 0x885e, 0x885e, PDF_CMAP_SINGLE, 14363 },
	{ 0x8860, 0x8860, PDF_CMAP_SINGLE, 14918 },
	{ 0x8861, 0x8861, PDF_CMAP_SINGLE, 4933 },
	{ 0x8862, 0x8862, PDF_CMAP_SINGLE, 5930 },
	{ 0x8863, 0x8863, PDF_CMAP_SINGLE, 1042 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 17666 },
	{ 0x8865, 0x8865, PDF_CMAP_SINGLE, 15868 },
	{ 0x8867, 0x8867, PDF_CMAP_SINGLE, 7168 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 1676 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 7170 },
	{ 0x886a, 0x886a, PDF_CMAP_SINGLE, 7169 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 2052 },
	{ 0x886d, 0x886d, PDF_CMAP_SINGLE, 7676 },
	{ 0x886e, 0x886e, PDF_CMAP_SINGLE, 15246 },
	{ 0x886f, 0x886f, PDF_CMAP_SINGLE, 7683 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 2488 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 7681 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 7679 },
	{ 0x8874, 0x8874, PDF_CMAP_SINGLE, 7686 },
	{ 0x8875, 0x8876, PDF_CMAP_RANGE, 7677 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2489 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 2493 },
	{ 0x887c, 0x887c, PDF_CMAP_SINGLE, 7687 },
	{ 0x887d, 0x887d, PDF_CMAP_SINGLE, 2492 },
	{ 0x887e, 0x887e, PDF_CMAP_SINGLE, 7685 },
	{ 0x887f, 0x887f, PDF_CMAP_SINGLE, 7682 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 7680 },
	{ 0x8881, 0x8882, PDF_CMAP_RANGE, 2490 },
	{ 0x8883, 0x8883, PDF_CMAP_SINGLE, 7684 },
	{ 0x8884, 0x8884, PDF_CMAP_SINGLE, 17143 },
	{ 0x8887, 0x8887, PDF_CMAP_SINGLE, 15248 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 2966 },
	{ 0x8889, 0x8889, PDF_CMAP_SINGLE, 8272 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 2971 },
	{ 0x888c, 0x888c, PDF_CMAP_SINGLE, 8288 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 2970 },
	{ 0x888e, 0x888e, PDF_CMAP_SINGLE, 8290 },
	{ 0x888f, 0x888f, PDF_CMAP_SINGLE, 18854 },
	{ 0x8890, 0x8890, PDF_CMAP_SINGLE, 18933 },
	{ 0x8891, 0x8891, PDF_CMAP_SINGLE, 8278 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 2968 },
	{ 0x8893, 0x8893, PDF_CMAP_SINGLE, 8289 },
	{ 0x8895, 0x8895, PDF_CMAP_SINGLE, 8273 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 2969 },
	{ 0x8897, 0x8897, PDF_CMAP_SINGLE, 8285 },
	{ 0x8898, 0x8898, PDF_CMAP_SINGLE, 8281 },
	{ 0x8899, 0x8899, PDF_CMAP_SINGLE, 8283 },
	{ 0x889a, 0x889a, PDF_CMAP_SINGLE, 8277 },
	{ 0x889b, 0x889b, PDF_CMAP_SINGLE, 8284 },
	{ 0x889c, 0x889c, PDF_CMAP_SINGLE, 14364 },
	{ 0x889d, 0x889d, PDF_CMAP_SINGLE, 19115 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 2965 },
	{ 0x889f, 0x889f, PDF_CMAP_SINGLE, 8280 },
	{ 0x88a0, 0x88a0, PDF_CMAP_SINGLE, 16288 },
	{ 0x88a1, 0x88a1, PDF_CMAP_SINGLE, 8279 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 8275 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 8286 },
	{ 0x88a7, 0x88a7, PDF_CMAP_SINGLE, 8282 },
	{ 0x88a8, 0x88a8, PDF_CMAP_SINGLE, 8274 },
	{ 0x88aa, 0x88aa, PDF_CMAP_SINGLE, 8276 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 2967 },
	{ 0x88ac, 0x88ac, PDF_CMAP_SINGLE, 8287 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 18293 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 3412 },
	{ 0x88b2, 0x88b2, PDF_CMAP_SINGLE, 8928 },
	{ 0x88b4, 0x88b5, PDF_CMAP_RANGE, 14366 },
	{ 0x88b6, 0x88b6, PDF_CMAP_SINGLE, 8924 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 8926 },
	{ 0x88b8, 0x88b8, PDF_CMAP_SINGLE, 8921 },
	{ 0x88b9, 0x88b9, PDF_CMAP_SINGLE, 8920 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 8918 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 8925 },
	{ 0x88bd, 0x88bd, PDF_CMAP_SINGLE, 8927 },
	{ 0x88be, 0x88be, PDF_CMAP_SINGLE, 8923 },
	{ 0x88bf, 0x88bf, PDF_CMAP_SINGLE, 15249 },
	{ 0x88c0, 0x88c0, PDF_CMAP_SINGLE, 8922 },
	{ 0x88c1, 0x88c2, PDF_CMAP_RANGE, 3410 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 14368 },
	{ 0x88c7, 0x88c7, PDF_CMAP_SINGLE, 16049 },
	{ 0x88c9, 0x88c9, PDF_CMAP_SINGLE, 8930 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 3842 },
	{ 0x88cb, 0x88cb, PDF_CMAP_SINGLE, 9592 },
	{ 0x88cc, 0x88cc, PDF_CMAP_SINGLE, 9598 },
	{ 0x88cd, 0x88ce, PDF_CMAP_RANGE, 9593 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 14058 },
	{ 0x88d0, 0x88d0, PDF_CMAP_SINGLE, 9599 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 3844 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 3836 },
	{ 0x88d5, 0x88d5, PDF_CMAP_SINGLE, 3843 },
	{ 0x88d6, 0x88d6, PDF_CMAP_SINGLE, 9591 },
	{ 0x88d7, 0x88d7, PDF_CMAP_SINGLE, 8919 },
	{ 0x88d8, 0x88d8, PDF_CMAP_SINGLE, 3839 },
	{ 0x88d9, 0x88d9, PDF_CMAP_SINGLE, 3837 },
	{ 0x88da, 0x88da, PDF_CMAP_SINGLE, 9597 },
	{ 0x88db, 0x88db, PDF_CMAP_SINGLE, 9596 },
	{ 0x88dc, 0x88dc, PDF_CMAP_SINGLE, 3838 },
	{ 0x88dd, 0x88dd, PDF_CMAP_SINGLE, 3840 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 9595 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 3835 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3841 },
	{ 0x88e6, 0x88e6, PDF_CMAP_SINGLE, 15250 },
	{ 0x88e7, 0x88e7, PDF_CMAP_SINGLE, 10224 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 4258 },
	{ 0x88eb, 0x88eb, PDF_CMAP_SINGLE, 10235 },
	{ 0x88ec, 0x88ec, PDF_CMAP_SINGLE, 10234 },
	{ 0x88ee, 0x88ee, PDF_CMAP_SINGLE, 10229 },
	{ 0x88ef, 0x88ef, PDF_CMAP_SINGLE, 4260 },
	{ 0x88f0, 0x88f0, PDF_CMAP_SINGLE, 10233 },
	{ 0x88f1, 0x88f2, PDF_CMAP_RANGE, 10225 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 4252 },
	{ 0x88f4, 0x88f4, PDF_CMAP_SINGLE, 4254 },
	{ 0x88f5, 0x88f5, PDF_CMAP_SINGLE, 15880 },
	{ 0x88f6, 0x88f6, PDF_CMAP_SINGLE, 10231 },
	{ 0x88f7, 0x88f7, PDF_CMAP_SINGLE, 10223 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 4256 },
	{ 0x88f9, 0x88f9, PDF_CMAP_SINGLE, 4255 },
	{ 0x88fa, 0x88fa, PDF_CMAP_SINGLE, 10227 },
	{ 0x88fb, 0x88fb, PDF_CMAP_SINGLE, 10232 },
	{ 0x88fc, 0x88fc, PDF_CMAP_SINGLE, 10230 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 4257 },
	{ 0x88fe, 0x88fe, PDF_CMAP_SINGLE, 10228 },
	{ 0x88ff, 0x88ff, PDF_CMAP_SINGLE, 18294 },
	{ 0x8900, 0x8900, PDF_CMAP_SINGLE, 17146 },
	{ 0x8901, 0x8901, PDF_CMAP_SINGLE, 8929 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 4253 },
	{ 0x8905, 0x8905, PDF_CMAP_SINGLE, 10800 },
	{ 0x8906, 0x8906, PDF_CMAP_SINGLE, 10807 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 4621 },
	{ 0x8909, 0x8909, PDF_CMAP_SINGLE, 10811 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 4625 },
	{ 0x890b, 0x890b, PDF_CMAP_SINGLE, 10803 },
	{ 0x890c, 0x890c, PDF_CMAP_SINGLE, 10801 },
	{ 0x890e, 0x890e, PDF_CMAP_SINGLE, 10810 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 4620 },
	{ 0x8911, 0x8911, PDF_CMAP_SINGLE, 10809 },
	{ 0x8912, 0x8913, PDF_CMAP_RANGE, 4622 },
	{ 0x8914, 0x8914, PDF_CMAP_SINGLE, 10802 },
	{ 0x8915, 0x8915, PDF_CMAP_SINGLE, 4624 },
	{ 0x8916, 0x8916, PDF_CMAP_SINGLE, 10808 },
	{ 0x8917, 0x8919, PDF_CMAP_RANGE, 10804 },
	{ 0x891a, 0x891a, PDF_CMAP_SINGLE, 4259 },
	{ 0x891e, 0x891e, PDF_CMAP_SINGLE, 11383 },
	{ 0x891f, 0x891f, PDF_CMAP_SINGLE, 11395 },
	{ 0x8921, 0x8921, PDF_CMAP_SINGLE, 4938 },
	{ 0x8922, 0x8922, PDF_CMAP_SINGLE, 11390 },
	{ 0x8923, 0x8923, PDF_CMAP_SINGLE, 11392 },
	{ 0x8924, 0x8924, PDF_CMAP_SINGLE, 18295 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 4936 },
	{ 0x8926, 0x8926, PDF_CMAP_SINGLE, 11384 },
	{ 0x8927, 0x8927, PDF_CMAP_SINGLE, 11388 },
	{ 0x8929, 0x8929, PDF_CMAP_SINGLE, 11391 },
	{ 0x892a, 0x892a, PDF_CMAP_SINGLE, 4934 },
	{ 0x892b, 0x892b, PDF_CMAP_SINGLE, 4937 },
	{ 0x892c, 0x892c, PDF_CMAP_SINGLE, 11394 },
	{ 0x892d, 0x892e, PDF_CMAP_RANGE, 11386 },
	{ 0x892f, 0x892f, PDF_CMAP_SINGLE, 11393 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 11385 },
	{ 0x8931, 0x8931, PDF_CMAP_SINGLE, 11389 },
	{ 0x8932, 0x8932, PDF_CMAP_SINGLE, 4935 },
	{ 0x8933, 0x8933, PDF_CMAP_SINGLE, 11855 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 11854 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 5234 },
	{ 0x8937, 0x8937, PDF_CMAP_SINGLE, 11860 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 5236 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 5233 },
	{ 0x893c, 0x893c, PDF_CMAP_SINGLE, 11856 },
	{ 0x893d, 0x893d, PDF_CMAP_SINGLE, 5237 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 11857 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 11858 },
	{ 0x8942, 0x8942, PDF_CMAP_SINGLE, 11861 },
	{ 0x8943, 0x8943, PDF_CMAP_SINGLE, 15894 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 5235 },
	{ 0x8946, 0x8946, PDF_CMAP_SINGLE, 12232 },
	{ 0x8947, 0x8947, PDF_CMAP_SINGLE, 17379 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 12235 },
	{ 0x894b, 0x894b, PDF_CMAP_SINGLE, 12229 },
	{ 0x894c, 0x894c, PDF_CMAP_SINGLE, 12231 },
	{ 0x894d, 0x894d, PDF_CMAP_SINGLE, 15252 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 12230 },
	{ 0x8950, 0x8951, PDF_CMAP_RANGE, 12233 },
	{ 0x8952, 0x8952, PDF_CMAP_SINGLE, 11859 },
	{ 0x8953, 0x8953, PDF_CMAP_SINGLE, 12228 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 15254 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 5570 },
	{ 0x8957, 0x8957, PDF_CMAP_SINGLE, 12568 },
	{ 0x8958, 0x8958, PDF_CMAP_SINGLE, 12571 },
	{ 0x8959, 0x8959, PDF_CMAP_SINGLE, 12573 },
	{ 0x895a, 0x895b, PDF_CMAP_RANGE, 12566 },
	{ 0x895c, 0x895c, PDF_CMAP_SINGLE, 12570 },
	{ 0x895d, 0x895d, PDF_CMAP_SINGLE, 12572 },
	{ 0x895e, 0x895e, PDF_CMAP_SINGLE, 5571 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 5569 },
	{ 0x8960, 0x8960, PDF_CMAP_SINGLE, 5568 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 12569 },
	{ 0x8962, 0x8962, PDF_CMAP_SINGLE, 12565 },
	{ 0x8963, 0x8963, PDF_CMAP_SINGLE, 12849 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 5697 },
	{ 0x8965, 0x8965, PDF_CMAP_SINGLE, 15251 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 12850 },
	{ 0x8969, 0x8969, PDF_CMAP_SINGLE, 13069 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 5772 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 13071 },
	{ 0x896c, 0x896c, PDF_CMAP_SINGLE, 5773 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 13068 },
	{ 0x896e, 0x896e, PDF_CMAP_SINGLE, 13070 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 5846 },
	{ 0x8971, 0x8971, PDF_CMAP_SINGLE, 13234 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 5845 },
	{ 0x8973, 0x8973, PDF_CMAP_SINGLE, 13371 },
	{ 0x8974, 0x8974, PDF_CMAP_SINGLE, 13370 },
	{ 0x8976, 0x8976, PDF_CMAP_SINGLE, 13369 },
	{ 0x8977, 0x8977, PDF_CMAP_SINGLE, 15850 },
	{ 0x8979, 0x897a, PDF_CMAP_RANGE, 13538 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 13541 },
	{ 0x897c, 0x897c, PDF_CMAP_SINGLE, 13540 },
	{ 0x897e, 0x897e, PDF_CMAP_SINGLE, 6169 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 1043 },
	{ 0x8980, 0x8980, PDF_CMAP_SINGLE, 18184 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 2053 },
	{ 0x8982, 0x8982, PDF_CMAP_SINGLE, 8291 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 3413 },
	{ 0x8985, 0x8985, PDF_CMAP_SINGLE, 9600 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 5415 },
	{ 0x8987, 0x8987, PDF_CMAP_SINGLE, 14370 },
	{ 0x8988, 0x8988, PDF_CMAP_SINGLE, 12574 },
	{ 0x8989, 0x8989, PDF_CMAP_SINGLE, 16361 },
	{ 0x898a, 0x898a, PDF_CMAP_SINGLE, 18090 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 1288 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 2973 },
	{ 0x8991, 0x8991, PDF_CMAP_SINGLE, 18296 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 2972 },
	{ 0x8994, 0x8994, PDF_CMAP_SINGLE, 16352 },
	{ 0x8995, 0x8995, PDF_CMAP_SINGLE, 8931 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 3414 },
	{ 0x8997, 0x8997, PDF_CMAP_SINGLE, 8933 },
	{ 0x8998, 0x8998, PDF_CMAP_SINGLE, 8932 },
	{ 0x899b, 0x899b, PDF_CMAP_SINGLE, 9601 },
	{ 0x899c, 0x899c, PDF_CMAP_SINGLE, 3845 },
	{ 0x899d, 0x899d, PDF_CMAP_SINGLE, 10236 },
	{ 0x899e, 0x899e, PDF_CMAP_SINGLE, 10239 },
	{ 0x899f, 0x899f, PDF_CMAP_SINGLE, 10238 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 10237 },
	{ 0x89a2, 0x89a2, PDF_CMAP_SINGLE, 10812 },
	{ 0x89a3, 0x89a3, PDF_CMAP_SINGLE, 10814 },
	{ 0x89a4, 0x89a4, PDF_CMAP_SINGLE, 10813 },
	{ 0x89a5, 0x89a5, PDF_CMAP_SINGLE, 15763 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 4940 },
	{ 0x89a7, 0x89a7, PDF_CMAP_SINGLE, 14374 },
	{ 0x89a9, 0x89a9, PDF_CMAP_SINGLE, 16377 },
	{ 0x89aa, 0x89aa, PDF_CMAP_SINGLE, 4939 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 5238 },
	{ 0x89ad, 0x89ad, PDF_CMAP_SINGLE, 11862 },
	{ 0x89ae, 0x89ae, PDF_CMAP_SINGLE, 11864 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 11863 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 5416 },
	{ 0x89b6, 0x89b6, PDF_CMAP_SINGLE, 12576 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 12575 },
	{ 0x89b9, 0x89b9, PDF_CMAP_SINGLE, 12851 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 5698 },
	{ 0x89bc, 0x89bc, PDF_CMAP_SINGLE, 14375 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 5774 },
	{ 0x89be, 0x89be, PDF_CMAP_SINGLE, 13236 },
	{ 0x89bf, 0x89bf, PDF_CMAP_SINGLE, 13235 },
	{ 0x89c0, 0x89c0, PDF_CMAP_SINGLE, 5960 },
	{ 0x89c1, 0x89c1, PDF_CMAP_SINGLE, 17667 },
	{ 0x89c6, 0x89c6, PDF_CMAP_SINGLE, 17766 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 1289 },
	{ 0x89d3, 0x89d3, PDF_CMAP_SINGLE, 7171 },
	{ 0x89d4, 0x89d4, PDF_CMAP_SINGLE, 2054 },
	{ 0x89d5, 0x89d5, PDF_CMAP_SINGLE, 8294 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 8292 },
	{ 0x89d9, 0x89d9, PDF_CMAP_SINGLE, 8293 },
	{ 0x89da, 0x89db, PDF_CMAP_RANGE, 8935 },
	{ 0x89dc, 0x89dc, PDF_CMAP_SINGLE, 9608 },
	{ 0x89dd, 0x89dd, PDF_CMAP_SINGLE, 8934 },
	{ 0x89df, 0x89df, PDF_CMAP_SINGLE, 9602 },
	{ 0x89e0, 0x89e0, PDF_CMAP_SINGLE, 9606 },
	{ 0x89e1, 0x89e1, PDF_CMAP_SINGLE, 9605 },
	{ 0x89e2, 0x89e2, PDF_CMAP_SINGLE, 9607 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 3846 },
	{ 0x89e4, 0x89e4, PDF_CMAP_SINGLE, 9604 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 9603 },
	{ 0x89e6, 0x89e6, PDF_CMAP_SINGLE, 9609 },
	{ 0x89e7, 0x89e7, PDF_CMAP_SINGLE, 14377 },
	{ 0x89e8, 0x89e8, PDF_CMAP_SINGLE, 10242 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 10240 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 10241 },
	{ 0x89ec, 0x89ec, PDF_CMAP_SINGLE, 10817 },
	{ 0x89ed, 0x89ed, PDF_CMAP_SINGLE, 10815 },
	{ 0x89f0, 0x89f0, PDF_CMAP_SINGLE, 10816 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 11396 },
	{ 0x89f2, 0x89f3, PDF_CMAP_RANGE, 11865 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 5417 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 12577 },
	{ 0x89f7, 0x89f7, PDF_CMAP_SINGLE, 12852 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 5699 },
	{ 0x89fa, 0x89fa, PDF_CMAP_SINGLE, 13072 },
	{ 0x89fb, 0x89fb, PDF_CMAP_SINGLE, 13237 },
	{ 0x89fc, 0x89fc, PDF_CMAP_SINGLE, 5847 },
	{ 0x89fe, 0x89fe, PDF_CMAP_SINGLE, 13372 },
	{ 0x89ff, 0x89ff, PDF_CMAP_SINGLE, 13542 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 1290 },
	{ 0x8a02, 0x8a03, PDF_CMAP_RANGE, 2056 },
	{ 0x8a04, 0x8a04, PDF_CMAP_SINGLE, 7172 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 7173 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 2055 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2499 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 2496 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 2503 },
	{ 0x8a10, 0x8a10, PDF_CMAP_SINGLE, 2495 },
	{ 0x8a11, 0x8a11, PDF_CMAP_SINGLE, 2504 },
	{ 0x8a12, 0x8a12, PDF_CMAP_SINGLE, 7688 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 2501 },
	{ 0x8a15, 0x8a15, PDF_CMAP_SINGLE, 2498 },
	{ 0x8a16, 0x8a16, PDF_CMAP_SINGLE, 2502 },
	{ 0x8a17, 0x8a17, PDF_CMAP_SINGLE, 2500 },
	{ 0x8a18, 0x8a18, PDF_CMAP_SINGLE, 2494 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 2981 },
	{ 0x8a1c, 0x8a1c, PDF_CMAP_SINGLE, 15267 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 2975 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 8298 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 2980 },
	{ 0x8a22, 0x8a22, PDF_CMAP_SINGLE, 2982 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 2976 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 2977 },
	{ 0x8a27, 0x8a27, PDF_CMAP_SINGLE, 8296 },
	{ 0x8a29, 0x8a29, PDF_CMAP_SINGLE, 17381 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 2974 },
	{ 0x8a2b, 0x8a2b, PDF_CMAP_SINGLE, 15313 },
	{ 0x8a2c, 0x8a2c, PDF_CMAP_SINGLE, 8297 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 2979 },
	{ 0x8a30, 0x8a30, PDF_CMAP_SINGLE, 8295 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 2978 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 3425 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 3427 },
	{ 0x8a38, 0x8a38, PDF_CMAP_SINGLE, 18298 },
	{ 0x8a39, 0x8a39, PDF_CMAP_SINGLE, 8939 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 3426 },
	{ 0x8a3b, 0x8a3b, PDF_CMAP_SINGLE, 3415 },
	{ 0x8a3c, 0x8a3c, PDF_CMAP_SINGLE, 3419 },
	{ 0x8a3d, 0x8a3d, PDF_CMAP_SINGLE, 15266 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 3865 },
	{ 0x8a3f, 0x8a3f, PDF_CMAP_SINGLE, 9614 },
	{ 0x8a40, 0x8a40, PDF_CMAP_SINGLE, 8941 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 3420 },
	{ 0x8a44, 0x8a45, PDF_CMAP_RANGE, 8944 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 3424 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 8947 },
	{ 0x8a49, 0x8a49, PDF_CMAP_SINGLE, 16368 },
	{ 0x8a4a, 0x8a4a, PDF_CMAP_SINGLE, 8949 },
	{ 0x8a4c, 0x8a4c, PDF_CMAP_SINGLE, 8950 },
	{ 0x8a4d, 0x8a4d, PDF_CMAP_SINGLE, 8938 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 8937 },
	{ 0x8a4f, 0x8a4f, PDF_CMAP_SINGLE, 8951 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 3423 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 8948 },
	{ 0x8a52, 0x8a52, PDF_CMAP_SINGLE, 8946 },
	{ 0x8a54, 0x8a54, PDF_CMAP_SINGLE, 3421 },
	{ 0x8a55, 0x8a55, PDF_CMAP_SINGLE, 3417 },
	{ 0x8a56, 0x8a56, PDF_CMAP_SINGLE, 3428 },
	{ 0x8a57, 0x8a58, PDF_CMAP_RANGE, 8942 },
	{ 0x8a59, 0x8a59, PDF_CMAP_SINGLE, 8940 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 3422 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 3418 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 3416 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 9613 },
	{ 0x8a62, 0x8a62, PDF_CMAP_SINGLE, 3860 },
	{ 0x8a63, 0x8a63, PDF_CMAP_SINGLE, 3855 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 3850 },
	{ 0x8a67, 0x8a67, PDF_CMAP_SINGLE, 16270 },
	{ 0x8a68, 0x8a68, PDF_CMAP_SINGLE, 3866 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 3851 },
	{ 0x8a6b, 0x8a6b, PDF_CMAP_SINGLE, 3847 },
	{ 0x8a6c, 0x8a6c, PDF_CMAP_SINGLE, 3862 },
	{ 0x8a6d, 0x8a6d, PDF_CMAP_SINGLE, 3859 },
	{ 0x8a6e, 0x8a6e, PDF_CMAP_SINGLE, 3861 },
	{ 0x8a70, 0x8a70, PDF_CMAP_SINGLE, 3852 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 3857 },
	{ 0x8a72, 0x8a73, PDF_CMAP_RANGE, 3848 },
	{ 0x8a74, 0x8a74, PDF_CMAP_SINGLE, 9621 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 9618 },
	{ 0x8a76, 0x8a76, PDF_CMAP_SINGLE, 9610 },
	{ 0x8a77, 0x8a77, PDF_CMAP_SINGLE, 9615 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 3863 },
	{ 0x8a7a, 0x8a7a, PDF_CMAP_SINGLE, 9622 },
	{ 0x8a7b, 0x8a7b, PDF_CMAP_SINGLE, 3864 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 3854 },
	{ 0x8a7e, 0x8a7e, PDF_CMAP_SINGLE, 16293 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 9612 },
	{ 0x8a81, 0x8a81, PDF_CMAP_SINGLE, 9620 },
	{ 0x8a82, 0x8a82, PDF_CMAP_SINGLE, 9616 },
	{ 0x8a83, 0x8a83, PDF_CMAP_SINGLE, 9619 },
	{ 0x8a84, 0x8a84, PDF_CMAP_SINGLE, 9617 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 3858 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 9611 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 3853 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 10245 },
	{ 0x8a8c, 0x8a8c, PDF_CMAP_SINGLE, 4262 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 4265 },
	{ 0x8a8f, 0x8a8f, PDF_CMAP_SINGLE, 10247 },
	{ 0x8a90, 0x8a90, PDF_CMAP_SINGLE, 14383 },
	{ 0x8a91, 0x8a91, PDF_CMAP_SINGLE, 4273 },
	{ 0x8a92, 0x8a92, PDF_CMAP_SINGLE, 10246 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 4267 },
	{ 0x8a94, 0x8a94, PDF_CMAP_SINGLE, 18299 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 4630 },
	{ 0x8a96, 0x8a96, PDF_CMAP_SINGLE, 10248 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 4272 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 10244 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 4274 },
	{ 0x8a9c, 0x8a9c, PDF_CMAP_SINGLE, 14380 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 4263 },
	{ 0x8aa0, 0x8aa0, PDF_CMAP_SINGLE, 3856 },
	{ 0x8aa1, 0x8aa1, PDF_CMAP_SINGLE, 4266 },
	{ 0x8aa3, 0x8aa3, PDF_CMAP_SINGLE, 4264 },
	{ 0x8aa4, 0x8aa4, PDF_CMAP_SINGLE, 4268 },
	{ 0x8aa5, 0x8aa5, PDF_CMAP_SINGLE, 4270 },
	{ 0x8aa6, 0x8aa6, PDF_CMAP_SINGLE, 4261 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 4275 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 4271 },
	{ 0x8aa9, 0x8aa9, PDF_CMAP_SINGLE, 14397 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 4269 },
	{ 0x8aab, 0x8aab, PDF_CMAP_SINGLE, 10243 },
	{ 0x8aaf, 0x8aaf, PDF_CMAP_SINGLE, 14401 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 4637 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 4633 },
	{ 0x8ab4, 0x8ab4, PDF_CMAP_SINGLE, 18300 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 4640 },
	{ 0x8ab8, 0x8ab8, PDF_CMAP_SINGLE, 10820 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 4641 },
	{ 0x8aba, 0x8aba, PDF_CMAP_SINGLE, 10832 },
	{ 0x8abb, 0x8abb, PDF_CMAP_SINGLE, 10825 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 4626 },
	{ 0x8abd, 0x8abd, PDF_CMAP_SINGLE, 10833 },
	{ 0x8abe, 0x8abe, PDF_CMAP_SINGLE, 10827 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 4636 },
	{ 0x8ac0, 0x8ac0, PDF_CMAP_SINGLE, 10828 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 4635 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 10831 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 4629 },
	{ 0x8ac5, 0x8ac5, PDF_CMAP_SINGLE, 10829 },
	{ 0x8ac6, 0x8ac6, PDF_CMAP_SINGLE, 10819 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 4628 },
	{ 0x8ac8, 0x8ac8, PDF_CMAP_SINGLE, 11407 },
	{ 0x8ac9, 0x8ac9, PDF_CMAP_SINGLE, 4634 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 4631 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 19049 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 4639 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 10818 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 10822 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 4627 },
	{ 0x8ad3, 0x8ad3, PDF_CMAP_SINGLE, 10821 },
	{ 0x8ad4, 0x8ad5, PDF_CMAP_RANGE, 10823 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4638 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 10826 },
	{ 0x8ad8, 0x8ad8, PDF_CMAP_SINGLE, 10830 },
	{ 0x8ad9, 0x8ad9, PDF_CMAP_SINGLE, 10834 },
	{ 0x8ada, 0x8ada, PDF_CMAP_SINGLE, 14786 },
	{ 0x8adb, 0x8adb, PDF_CMAP_SINGLE, 4642 },
	{ 0x8adc, 0x8adc, PDF_CMAP_SINGLE, 4946 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 11402 },
	{ 0x8ade, 0x8ade, PDF_CMAP_SINGLE, 11408 },
	{ 0x8adf, 0x8adf, PDF_CMAP_SINGLE, 11405 },
	{ 0x8ae0, 0x8ae0, PDF_CMAP_SINGLE, 11397 },
	{ 0x8ae1, 0x8ae1, PDF_CMAP_SINGLE, 11409 },
	{ 0x8ae2, 0x8ae2, PDF_CMAP_SINGLE, 11398 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 11404 },
	{ 0x8ae6, 0x8ae6, PDF_CMAP_SINGLE, 4941 },
	{ 0x8ae7, 0x8ae7, PDF_CMAP_SINGLE, 4947 },
	{ 0x8ae8, 0x8ae8, PDF_CMAP_SINGLE, 11410 },
	{ 0x8aea, 0x8aea, PDF_CMAP_SINGLE, 15833 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 4943 },
	{ 0x8aed, 0x8aed, PDF_CMAP_SINGLE, 4953 },
	{ 0x8aee, 0x8aee, PDF_CMAP_SINGLE, 4948 },
	{ 0x8aef, 0x8aef, PDF_CMAP_SINGLE, 11412 },
	{ 0x8af0, 0x8af0, PDF_CMAP_SINGLE, 11406 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 4944 },
	{ 0x8af2, 0x8af2, PDF_CMAP_SINGLE, 11399 },
	{ 0x8af3, 0x8af3, PDF_CMAP_SINGLE, 4954 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 11400 },
	{ 0x8af6, 0x8af6, PDF_CMAP_SINGLE, 4955 },
	{ 0x8af7, 0x8af7, PDF_CMAP_SINGLE, 4952 },
	{ 0x8af8, 0x8af8, PDF_CMAP_SINGLE, 4632 },
	{ 0x8af9, 0x8af9, PDF_CMAP_SINGLE, 19034 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 4942 },
	{ 0x8afb, 0x8afb, PDF_CMAP_SINGLE, 11413 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 4956 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 4949 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 11411 },
	{ 0x8b00, 0x8b00, PDF_CMAP_SINGLE, 4945 },
	{ 0x8b01, 0x8b02, PDF_CMAP_RANGE, 4950 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 5246 },
	{ 0x8b05, 0x8b05, PDF_CMAP_SINGLE, 11871 },
	{ 0x8b06, 0x8b06, PDF_CMAP_SINGLE, 11880 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 11877 },
	{ 0x8b08, 0x8b08, PDF_CMAP_SINGLE, 11879 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 5243 },
	{ 0x8b0b, 0x8b0b, PDF_CMAP_SINGLE, 11872 },
	{ 0x8b0c, 0x8b0c, PDF_CMAP_SINGLE, 14390 },
	{ 0x8b0d, 0x8b0d, PDF_CMAP_SINGLE, 11878 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 5239 },
	{ 0x8b0f, 0x8b0f, PDF_CMAP_SINGLE, 11874 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 5247 },
	{ 0x8b11, 0x8b11, PDF_CMAP_SINGLE, 11870 },
	{ 0x8b12, 0x8b12, PDF_CMAP_SINGLE, 11875 },
	{ 0x8b13, 0x8b13, PDF_CMAP_SINGLE, 11882 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 11403 },
	{ 0x8b15, 0x8b15, PDF_CMAP_SINGLE, 11876 },
	{ 0x8b16, 0x8b16, PDF_CMAP_SINGLE, 11869 },
	{ 0x8b17, 0x8b17, PDF_CMAP_SINGLE, 5240 },
	{ 0x8b18, 0x8b18, PDF_CMAP_SINGLE, 11868 },
	{ 0x8b19, 0x8b19, PDF_CMAP_SINGLE, 5241 },
	{ 0x8b1a, 0x8b1a, PDF_CMAP_SINGLE, 11883 },
	{ 0x8b1b, 0x8b1b, PDF_CMAP_SINGLE, 5242 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 11881 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 5245 },
	{ 0x8b1e, 0x8b1e, PDF_CMAP_SINGLE, 11867 },
	{ 0x8b1f, 0x8b1f, PDF_CMAP_SINGLE, 14391 },
	{ 0x8b20, 0x8b20, PDF_CMAP_SINGLE, 5244 },
	{ 0x8b22, 0x8b22, PDF_CMAP_SINGLE, 11873 },
	{ 0x8b23, 0x8b23, PDF_CMAP_SINGLE, 12238 },
	{ 0x8b24, 0x8b24, PDF_CMAP_SINGLE, 12252 },
	{ 0x8b25, 0x8b25, PDF_CMAP_SINGLE, 12247 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 12249 },
	{ 0x8b27, 0x8b27, PDF_CMAP_SINGLE, 12237 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 5418 },
	{ 0x8b2a, 0x8b2a, PDF_CMAP_SINGLE, 12236 },
	{ 0x8b2b, 0x8b2b, PDF_CMAP_SINGLE, 5421 },
	{ 0x8b2c, 0x8b2c, PDF_CMAP_SINGLE, 5420 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 16281 },
	{ 0x8b2e, 0x8b2e, PDF_CMAP_SINGLE, 12251 },
	{ 0x8b2f, 0x8b2f, PDF_CMAP_SINGLE, 12243 },
	{ 0x8b30, 0x8b30, PDF_CMAP_SINGLE, 12240 },
	{ 0x8b31, 0x8b31, PDF_CMAP_SINGLE, 12246 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 12239 },
	{ 0x8b35, 0x8b35, PDF_CMAP_SINGLE, 12241 },
	{ 0x8b36, 0x8b36, PDF_CMAP_SINGLE, 12250 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 12248 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 5419 },
	{ 0x8b3a, 0x8b3a, PDF_CMAP_SINGLE, 12255 },
	{ 0x8b3b, 0x8b3b, PDF_CMAP_SINGLE, 12253 },
	{ 0x8b3c, 0x8b3c, PDF_CMAP_SINGLE, 12244 },
	{ 0x8b3d, 0x8b3d, PDF_CMAP_SINGLE, 12254 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 12245 },
	{ 0x8b3f, 0x8b3f, PDF_CMAP_SINGLE, 14394 },
	{ 0x8b40, 0x8b40, PDF_CMAP_SINGLE, 12581 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 5572 },
	{ 0x8b42, 0x8b42, PDF_CMAP_SINGLE, 12588 },
	{ 0x8b43, 0x8b43, PDF_CMAP_SINGLE, 16370 },
	{ 0x8b45, 0x8b45, PDF_CMAP_SINGLE, 13075 },
	{ 0x8b46, 0x8b46, PDF_CMAP_SINGLE, 5579 },
	{ 0x8b47, 0x8b47, PDF_CMAP_SINGLE, 12242 },
	{ 0x8b48, 0x8b48, PDF_CMAP_SINGLE, 12579 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 5575 },
	{ 0x8b4a, 0x8b4a, PDF_CMAP_SINGLE, 12580 },
	{ 0x8b4b, 0x8b4b, PDF_CMAP_SINGLE, 12585 },
	{ 0x8b4c, 0x8b4c, PDF_CMAP_SINGLE, 18092 },
	{ 0x8b4d, 0x8b4d, PDF_CMAP_SINGLE, 14396 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_RANGE, 5577 },
	{ 0x8b50, 0x8b50, PDF_CMAP_SINGLE, 12578 },
	{ 0x8b51, 0x8b51, PDF_CMAP_SINGLE, 12587 },
	{ 0x8b52, 0x8b52, PDF_CMAP_SINGLE, 12589 },
	{ 0x8b53, 0x8b53, PDF_CMAP_SINGLE, 12582 },
	{ 0x8b54, 0x8b54, PDF_CMAP_SINGLE, 12584 },
	{ 0x8b55, 0x8b55, PDF_CMAP_SINGLE, 12586 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 12583 },
	{ 0x8b57, 0x8b57, PDF_CMAP_SINGLE, 12590 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 5574 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 5580 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 5576 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 5573 },
	{ 0x8b5d, 0x8b5d, PDF_CMAP_SINGLE, 12855 },
	{ 0x8b5e, 0x8b5e, PDF_CMAP_SINGLE, 15800 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 5704 },
	{ 0x8b60, 0x8b60, PDF_CMAP_SINGLE, 12853 },
	{ 0x8b62, 0x8b62, PDF_CMAP_SINGLE, 15828 },
	{ 0x8b63, 0x8b63, PDF_CMAP_SINGLE, 12857 },
	{ 0x8b65, 0x8b65, PDF_CMAP_SINGLE, 12858 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 5702 },
	{ 0x8b67, 0x8b67, PDF_CMAP_SINGLE, 12859 },
	{ 0x8b68, 0x8b68, PDF_CMAP_SINGLE, 12856 },
	{ 0x8b69, 0x8b69, PDF_CMAP_SINGLE, 15351 },
	{ 0x8b6a, 0x8b6a, PDF_CMAP_SINGLE, 12854 },
	{ 0x8b6b, 0x8b6b, PDF_CMAP_SINGLE, 5705 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 5701 },
	{ 0x8b6d, 0x8b6d, PDF_CMAP_SINGLE, 12860 },
	{ 0x8b6f, 0x8b6f, PDF_CMAP_SINGLE, 5703 },
	{ 0x8b70, 0x8b70, PDF_CMAP_SINGLE, 5700 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 5775 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 5776 },
	{ 0x8b78, 0x8b78, PDF_CMAP_SINGLE, 13074 },
	{ 0x8b79, 0x8b79, PDF_CMAP_SINGLE, 13073 },
	{ 0x8b7a, 0x8b7b, PDF_CMAP_RANGE, 13076 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 5777 },
	{ 0x8b7e, 0x8b7e, PDF_CMAP_SINGLE, 13238 },
	{ 0x8b7f, 0x8b7f, PDF_CMAP_SINGLE, 13243 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 5848 },
	{ 0x8b81, 0x8b81, PDF_CMAP_SINGLE, 15757 },
	{ 0x8b82, 0x8b82, PDF_CMAP_SINGLE, 13240 },
	{ 0x8b83, 0x8b83, PDF_CMAP_SINGLE, 18923 },
	{ 0x8b84, 0x8b84, PDF_CMAP_SINGLE, 13239 },
	{ 0x8b85, 0x8b85, PDF_CMAP_SINGLE, 13242 },
	{ 0x8b86, 0x8b86, PDF_CMAP_SINGLE, 13241 },
	{ 0x8b88, 0x8b88, PDF_CMAP_SINGLE, 13376 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5898 },
	{ 0x8b8b, 0x8b8b, PDF_CMAP_SINGLE, 13375 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 13373 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 13374 },
	{ 0x8b8f, 0x8b8f, PDF_CMAP_SINGLE, 19135 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 14399 },
	{ 0x8b92, 0x8b92, PDF_CMAP_SINGLE, 5932 },
	{ 0x8b93, 0x8b93, PDF_CMAP_SINGLE, 5931 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 13468 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 5933 },
	{ 0x8b98, 0x8b99, PDF_CMAP_RANGE, 13543 },
	{ 0x8b9a, 0x8b9a, PDF_CMAP_SINGLE, 5972 },
	{ 0x8b9b, 0x8b9b, PDF_CMAP_SINGLE, 14400 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 5978 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 13602 },
	{ 0x8b9f, 0x8b9f, PDF_CMAP_SINGLE, 13631 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 17669 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 17767 },
	{ 0x8be2, 0x8be2, PDF_CMAP_SINGLE, 17768 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1291 },
	{ 0x8c39, 0x8c39, PDF_CMAP_SINGLE, 8299 },
	{ 0x8c3b, 0x8c3b, PDF_CMAP_SINGLE, 8300 },
	{ 0x8c3c, 0x8c3c, PDF_CMAP_SINGLE, 9623 },
	{ 0x8c3d, 0x8c3d, PDF_CMAP_SINGLE, 10249 },
	{ 0x8c3e, 0x8c3e, PDF_CMAP_SINGLE, 10835 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 5249 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 5248 },
	{ 0x8c42, 0x8c42, PDF_CMAP_SINGLE, 12256 },
	{ 0x8c43, 0x8c43, PDF_CMAP_SINGLE, 12591 },
	{ 0x8c45, 0x8c45, PDF_CMAP_SINGLE, 13377 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 1292 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 7689 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 2505 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 2983 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 9625 },
	{ 0x8c4b, 0x8c4b, PDF_CMAP_SINGLE, 9624 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 4643 },
	{ 0x8c4d, 0x8c4d, PDF_CMAP_SINGLE, 10836 },
	{ 0x8c4e, 0x8c4e, PDF_CMAP_SINGLE, 4644 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 11884 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 5422 },
	{ 0x8c51, 0x8c51, PDF_CMAP_SINGLE, 18301 },
	{ 0x8c54, 0x8c54, PDF_CMAP_SINGLE, 5987 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 1293 },
	{ 0x8c56, 0x8c56, PDF_CMAP_SINGLE, 6728 },
	{ 0x8c57, 0x8c57, PDF_CMAP_SINGLE, 7690 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 2984 },
	{ 0x8c5c, 0x8c5d, PDF_CMAP_RANGE, 8301 },
	{ 0x8c5f, 0x8c5f, PDF_CMAP_SINGLE, 8952 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 3429 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 3867 },
	{ 0x8c64, 0x8c64, PDF_CMAP_SINGLE, 9627 },
	{ 0x8c65, 0x8c65, PDF_CMAP_SINGLE, 9626 },
	{ 0x8c66, 0x8c66, PDF_CMAP_SINGLE, 9628 },
	{ 0x8c68, 0x8c69, PDF_CMAP_RANGE, 10250 },
	{ 0x8c6a, 0x8c6a, PDF_CMAP_SINGLE, 4276 },
	{ 0x8c6b, 0x8c6b, PDF_CMAP_SINGLE, 4957 },
	{ 0x8c6c, 0x8c6c, PDF_CMAP_SINGLE, 4645 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 4958 },
	{ 0x8c6f, 0x8c6f, PDF_CMAP_SINGLE, 11888 },
	{ 0x8c70, 0x8c70, PDF_CMAP_SINGLE, 11885 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 11887 },
	{ 0x8c72, 0x8c72, PDF_CMAP_SINGLE, 11886 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 5250 },
	{ 0x8c75, 0x8c75, PDF_CMAP_SINGLE, 12257 },
	{ 0x8c76, 0x8c76, PDF_CMAP_SINGLE, 12593 },
	{ 0x8c77, 0x8c77, PDF_CMAP_SINGLE, 12592 },
	{ 0x8c78, 0x8c78, PDF_CMAP_SINGLE, 6375 },
	{ 0x8c79, 0x8c79, PDF_CMAP_SINGLE, 2507 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 2506 },
	{ 0x8c7b, 0x8c7b, PDF_CMAP_SINGLE, 7691 },
	{ 0x8c7d, 0x8c7d, PDF_CMAP_SINGLE, 8303 },
	{ 0x8c80, 0x8c80, PDF_CMAP_SINGLE, 8954 },
	{ 0x8c81, 0x8c81, PDF_CMAP_SINGLE, 8953 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 3430 },
	{ 0x8c84, 0x8c85, PDF_CMAP_RANGE, 9630 },
	{ 0x8c86, 0x8c86, PDF_CMAP_SINGLE, 9629 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 3869 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 3868 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 4278 },
	{ 0x8c8d, 0x8c8d, PDF_CMAP_SINGLE, 4277 },
	{ 0x8c8f, 0x8c8f, PDF_CMAP_SINGLE, 10837 },
	{ 0x8c90, 0x8c90, PDF_CMAP_SINGLE, 11416 },
	{ 0x8c91, 0x8c92, PDF_CMAP_RANGE, 11414 },
	{ 0x8c93, 0x8c93, PDF_CMAP_SINGLE, 4959 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 11890 },
	{ 0x8c95, 0x8c95, PDF_CMAP_SINGLE, 11889 },
	{ 0x8c97, 0x8c97, PDF_CMAP_SINGLE, 12260 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 12259 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 12258 },
	{ 0x8c9a, 0x8c9a, PDF_CMAP_SINGLE, 12594 },
	{ 0x8c9b, 0x8c9b, PDF_CMAP_SINGLE, 14405 },
	{ 0x8c9c, 0x8c9c, PDF_CMAP_SINGLE, 13603 },
	{ 0x8c9d, 0x8c9d, PDF_CMAP_SINGLE, 1294 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 2058 },
	{ 0x8c9f, 0x8c9f, PDF_CMAP_SINGLE, 15277 },
	{ 0x8ca0, 0x8ca0, PDF_CMAP_SINGLE, 2059 },
	{ 0x8ca1, 0x8ca2, PDF_CMAP_RANGE, 2508 },
	{ 0x8ca3, 0x8ca3, PDF_CMAP_SINGLE, 7693 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 7692 },
	{ 0x8ca5, 0x8ca5, PDF_CMAP_SINGLE, 8304 },
	{ 0x8ca7, 0x8ca7, PDF_CMAP_SINGLE, 2990 },
	{ 0x8ca8, 0x8ca8, PDF_CMAP_SINGLE, 2988 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 2985 },
	{ 0x8caa, 0x8caa, PDF_CMAP_SINGLE, 2989 },
	{ 0x8cab, 0x8cab, PDF_CMAP_SINGLE, 2987 },
	{ 0x8cac, 0x8cac, PDF_CMAP_SINGLE, 2986 },
	{ 0x8cad, 0x8cad, PDF_CMAP_SINGLE, 16369 },
	{ 0x8caf, 0x8caf, PDF_CMAP_SINGLE, 3431 },
	{ 0x8cb0, 0x8cb0, PDF_CMAP_SINGLE, 8957 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 3874 },
	{ 0x8cb3, 0x8cb3, PDF_CMAP_SINGLE, 3433 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 3438 },
	{ 0x8cb5, 0x8cb5, PDF_CMAP_SINGLE, 8959 },
	{ 0x8cb6, 0x8cb6, PDF_CMAP_SINGLE, 3440 },
	{ 0x8cb7, 0x8cb7, PDF_CMAP_SINGLE, 3439 },
	{ 0x8cb8, 0x8cb8, PDF_CMAP_SINGLE, 3442 },
	{ 0x8cb9, 0x8cb9, PDF_CMAP_SINGLE, 8958 },
	{ 0x8cba, 0x8cba, PDF_CMAP_SINGLE, 8955 },
	{ 0x8cbb, 0x8cbb, PDF_CMAP_SINGLE, 3436 },
	{ 0x8cbc, 0x8cbc, PDF_CMAP_SINGLE, 3432 },
	{ 0x8cbd, 0x8cbd, PDF_CMAP_SINGLE, 3434 },
	{ 0x8cbe, 0x8cbe, PDF_CMAP_SINGLE, 8956 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 3441 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 3437 },
	{ 0x8cc1, 0x8cc1, PDF_CMAP_SINGLE, 3435 },
	{ 0x8cc2, 0x8cc2, PDF_CMAP_SINGLE, 3876 },
	{ 0x8cc3, 0x8cc3, PDF_CMAP_SINGLE, 3875 },
	{ 0x8cc4, 0x8cc4, PDF_CMAP_SINGLE, 3873 },
	{ 0x8cc5, 0x8cc5, PDF_CMAP_SINGLE, 3877 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_RANGE, 3871 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 3870 },
	{ 0x8ccc, 0x8ccc, PDF_CMAP_SINGLE, 9632 },
	{ 0x8ccd, 0x8ccd, PDF_CMAP_SINGLE, 16025 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 10253 },
	{ 0x8cd1, 0x8cd2, PDF_CMAP_RANGE, 4280 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 4279 },
	{ 0x8cd4, 0x8cd4, PDF_CMAP_SINGLE, 18302 },
	{ 0x8cd5, 0x8cd5, PDF_CMAP_SINGLE, 10252 },
	{ 0x8cd6, 0x8cd6, PDF_CMAP_SINGLE, 14411 },
	{ 0x8cd7, 0x8cd7, PDF_CMAP_SINGLE, 10254 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 10840 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 10842 },
	{ 0x8cdb, 0x8cdb, PDF_CMAP_SINGLE, 14417 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 4654 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 10843 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 4647 },
	{ 0x8cdf, 0x8cdf, PDF_CMAP_SINGLE, 10839 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 4646 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 4656 },
	{ 0x8ce2, 0x8ce3, PDF_CMAP_RANGE, 4652 },
	{ 0x8ce4, 0x8ce4, PDF_CMAP_SINGLE, 4649 },
	{ 0x8ce5, 0x8ce5, PDF_CMAP_SINGLE, 10838 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 4648 },
	{ 0x8ce7, 0x8ce7, PDF_CMAP_SINGLE, 10844 },
	{ 0x8ce8, 0x8ce8, PDF_CMAP_SINGLE, 10841 },
	{ 0x8ce9, 0x8ce9, PDF_CMAP_SINGLE, 15278 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 4655 },
	{ 0x8ceb, 0x8ceb, PDF_CMAP_SINGLE, 16284 },
	{ 0x8cec, 0x8ced, PDF_CMAP_RANGE, 4650 },
	{ 0x8cee, 0x8cee, PDF_CMAP_SINGLE, 11418 },
	{ 0x8cf0, 0x8cf0, PDF_CMAP_SINGLE, 11420 },
	{ 0x8cf1, 0x8cf1, PDF_CMAP_SINGLE, 11419 },
	{ 0x8cf2, 0x8cf2, PDF_CMAP_SINGLE, 18303 },
	{ 0x8cf3, 0x8cf3, PDF_CMAP_SINGLE, 11421 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 4960 },
	{ 0x8cf5, 0x8cf5, PDF_CMAP_SINGLE, 11417 },
	{ 0x8cf7, 0x8cf7, PDF_CMAP_SINGLE, 16305 },
	{ 0x8cf8, 0x8cf8, PDF_CMAP_SINGLE, 5254 },
	{ 0x8cf9, 0x8cf9, PDF_CMAP_SINGLE, 11891 },
	{ 0x8cfa, 0x8cfa, PDF_CMAP_SINGLE, 5251 },
	{ 0x8cfb, 0x8cfb, PDF_CMAP_SINGLE, 5255 },
	{ 0x8cfc, 0x8cfc, PDF_CMAP_SINGLE, 5253 },
	{ 0x8cfd, 0x8cfd, PDF_CMAP_SINGLE, 5252 },
	{ 0x8cfe, 0x8cfe, PDF_CMAP_SINGLE, 12261 },
	{ 0x8d00, 0x8d00, PDF_CMAP_SINGLE, 12264 },
	{ 0x8d02, 0x8d02, PDF_CMAP_SINGLE, 12263 },
	{ 0x8d03, 0x8d03, PDF_CMAP_SINGLE, 14415 },
	{ 0x8d04, 0x8d04, PDF_CMAP_SINGLE, 12262 },
	{ 0x8d05, 0x8d05, PDF_CMAP_SINGLE, 5423 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 12595 },
	{ 0x8d08, 0x8d08, PDF_CMAP_SINGLE, 5581 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 12597 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 5582 },
	{ 0x8d0b, 0x8d0b, PDF_CMAP_SINGLE, 17779 },
	{ 0x8d0c, 0x8d0c, PDF_CMAP_SINGLE, 16278 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 5707 },
	{ 0x8d0f, 0x8d0f, PDF_CMAP_SINGLE, 5706 },
	{ 0x8d10, 0x8d10, PDF_CMAP_SINGLE, 13078 },
	{ 0x8d11, 0x8d11, PDF_CMAP_SINGLE, 14419 },
	{ 0x8d12, 0x8d12, PDF_CMAP_SINGLE, 14414 },
	{ 0x8d13, 0x8d13, PDF_CMAP_SINGLE, 5778 },
	{ 0x8d14, 0x8d14, PDF_CMAP_SINGLE, 13079 },
	{ 0x8d15, 0x8d15, PDF_CMAP_SINGLE, 13244 },
	{ 0x8d16, 0x8d17, PDF_CMAP_RANGE, 5849 },
	{ 0x8d18, 0x8d18, PDF_CMAP_SINGLE, 15983 },
	{ 0x8d19, 0x8d19, PDF_CMAP_SINGLE, 13378 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 5935 },
	{ 0x8d1c, 0x8d1c, PDF_CMAP_SINGLE, 18304 },
	{ 0x8d1d, 0x8d1d, PDF_CMAP_SINGLE, 17670 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 1295 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 2992 },
	{ 0x8d67, 0x8d67, PDF_CMAP_SINGLE, 2991 },
	{ 0x8d68, 0x8d69, PDF_CMAP_RANGE, 9633 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 4282 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 11422 },
	{ 0x8d6d, 0x8d6d, PDF_CMAP_SINGLE, 4657 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 11423 },
	{ 0x8d6f, 0x8d6f, PDF_CMAP_SINGLE, 11892 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 1296 },
	{ 0x8d72, 0x8d72, PDF_CMAP_SINGLE, 7174 },
	{ 0x8d73, 0x8d73, PDF_CMAP_SINGLE, 2061 },
	{ 0x8d74, 0x8d74, PDF_CMAP_SINGLE, 2060 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 7694 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 2510 },
	{ 0x8d78, 0x8d78, PDF_CMAP_SINGLE, 7695 },
	{ 0x8d79, 0x8d79, PDF_CMAP_SINGLE, 8307 },
	{ 0x8d7a, 0x8d7a, PDF_CMAP_SINGLE, 14817 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 8306 },
	{ 0x8d7d, 0x8d7d, PDF_CMAP_SINGLE, 8305 },
	{ 0x8d80, 0x8d80, PDF_CMAP_SINGLE, 8961 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 3445 },
	{ 0x8d82, 0x8d82, PDF_CMAP_SINGLE, 16233 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 8960 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 3444 },
	{ 0x8d89, 0x8d89, PDF_CMAP_SINGLE, 8962 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 3443 },
	{ 0x8d8c, 0x8d8c, PDF_CMAP_SINGLE, 9636 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 9639 },
	{ 0x8d8e, 0x8d8f, PDF_CMAP_RANGE, 9637 },
	{ 0x8d90, 0x8d90, PDF_CMAP_SINGLE, 9642 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 9635 },
	{ 0x8d92, 0x8d92, PDF_CMAP_SINGLE, 9643 },
	{ 0x8d93, 0x8d94, PDF_CMAP_RANGE, 9640 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 4284 },
	{ 0x8d96, 0x8d96, PDF_CMAP_SINGLE, 10255 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 4283 },
	{ 0x8d9b, 0x8d9b, PDF_CMAP_SINGLE, 10848 },
	{ 0x8d9c, 0x8d9c, PDF_CMAP_SINGLE, 10846 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 4658 },
	{ 0x8da0, 0x8da0, PDF_CMAP_SINGLE, 10845 },
	{ 0x8da1, 0x8da1, PDF_CMAP_SINGLE, 10847 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 4659 },
	{ 0x8da5, 0x8da5, PDF_CMAP_SINGLE, 11424 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 16375 },
	{ 0x8da7, 0x8da7, PDF_CMAP_SINGLE, 11425 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 5256 },
	{ 0x8da9, 0x8da9, PDF_CMAP_SINGLE, 14422 },
	{ 0x8daa, 0x8daa, PDF_CMAP_SINGLE, 12599 },
	{ 0x8dab, 0x8dab, PDF_CMAP_SINGLE, 12601 },
	{ 0x8dac, 0x8dac, PDF_CMAP_SINGLE, 12598 },
	{ 0x8dad, 0x8dad, PDF_CMAP_SINGLE, 12600 },
	{ 0x8dae, 0x8dae, PDF_CMAP_SINGLE, 12861 },
	{ 0x8daf, 0x8daf, PDF_CMAP_SINGLE, 13080 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 13577 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 1297 },
	{ 0x8db4, 0x8db4, PDF_CMAP_SINGLE, 2062 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 7696 },
	{ 0x8db6, 0x8db6, PDF_CMAP_SINGLE, 7698 },
	{ 0x8db7, 0x8db7, PDF_CMAP_SINGLE, 7697 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 8310 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 2994 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 8308 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 2993 },
	{ 0x8dbf, 0x8dbf, PDF_CMAP_SINGLE, 8311 },
	{ 0x8dc0, 0x8dc0, PDF_CMAP_SINGLE, 17258 },
	{ 0x8dc1, 0x8dc1, PDF_CMAP_SINGLE, 8312 },
	{ 0x8dc2, 0x8dc2, PDF_CMAP_SINGLE, 8309 },
	{ 0x8dc3, 0x8dc3, PDF_CMAP_SINGLE, 18306 },
	{ 0x8dc5, 0x8dc5, PDF_CMAP_SINGLE, 8974 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 3453 },
	{ 0x8dc7, 0x8dc7, PDF_CMAP_SINGLE, 8966 },
	{ 0x8dc8, 0x8dc8, PDF_CMAP_SINGLE, 8972 },
	{ 0x8dcb, 0x8dcb, PDF_CMAP_SINGLE, 3448 },
	{ 0x8dcc, 0x8dcc, PDF_CMAP_SINGLE, 3451 },
	{ 0x8dcd, 0x8dcd, PDF_CMAP_SINGLE, 8965 },
	{ 0x8dce, 0x8dce, PDF_CMAP_SINGLE, 3446 },
	{ 0x8dcf, 0x8dcf, PDF_CMAP_SINGLE, 8969 },
	{ 0x8dd0, 0x8dd0, PDF_CMAP_SINGLE, 9649 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 3450 },
	{ 0x8dd3, 0x8dd3, PDF_CMAP_SINGLE, 8964 },
	{ 0x8dd4, 0x8dd4, PDF_CMAP_SINGLE, 16258 },
	{ 0x8dd5, 0x8dd5, PDF_CMAP_SINGLE, 8970 },
	{ 0x8dd6, 0x8dd6, PDF_CMAP_SINGLE, 8967 },
	{ 0x8dd7, 0x8dd7, PDF_CMAP_SINGLE, 8973 },
	{ 0x8dd8, 0x8dd8, PDF_CMAP_SINGLE, 8963 },
	{ 0x8dd9, 0x8dd9, PDF_CMAP_SINGLE, 8971 },
	{ 0x8dda, 0x8dda, PDF_CMAP_SINGLE, 3449 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 3452 },
	{ 0x8ddc, 0x8ddc, PDF_CMAP_SINGLE, 8968 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 3447 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 3879 },
	{ 0x8de0, 0x8de0, PDF_CMAP_SINGLE, 9645 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 3878 },
	{ 0x8de2, 0x8de2, PDF_CMAP_SINGLE, 9652 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 9651 },
	{ 0x8de4, 0x8de4, PDF_CMAP_SINGLE, 3885 },
	{ 0x8de6, 0x8de6, PDF_CMAP_SINGLE, 3886 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 9653 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 3880 },
	{ 0x8de9, 0x8de9, PDF_CMAP_SINGLE, 9650 },
	{ 0x8dea, 0x8dea, PDF_CMAP_SINGLE, 3884 },
	{ 0x8deb, 0x8deb, PDF_CMAP_SINGLE, 9655 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 9646 },
	{ 0x8dee, 0x8dee, PDF_CMAP_SINGLE, 9648 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 3881 },
	{ 0x8df0, 0x8df0, PDF_CMAP_SINGLE, 9644 },
	{ 0x8df1, 0x8df1, PDF_CMAP_SINGLE, 9647 },
	{ 0x8df2, 0x8df2, PDF_CMAP_SINGLE, 9654 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3882 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 9656 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 3883 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 4285 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 10260 },
	{ 0x8dfe, 0x8dfe, PDF_CMAP_SINGLE, 10266 },
	{ 0x8dff, 0x8dff, PDF_CMAP_SINGLE, 10258 },
	{ 0x8e00, 0x8e00, PDF_CMAP_SINGLE, 10267 },
	{ 0x8e01, 0x8e01, PDF_CMAP_SINGLE, 16261 },
	{ 0x8e02, 0x8e02, PDF_CMAP_SINGLE, 10257 },
	{ 0x8e03, 0x8e03, PDF_CMAP_SINGLE, 10262 },
	{ 0x8e04, 0x8e04, PDF_CMAP_SINGLE, 10268 },
	{ 0x8e05, 0x8e05, PDF_CMAP_SINGLE, 10265 },
	{ 0x8e06, 0x8e06, PDF_CMAP_SINGLE, 10264 },
	{ 0x8e07, 0x8e07, PDF_CMAP_SINGLE, 10263 },
	{ 0x8e09, 0x8e09, PDF_CMAP_SINGLE, 10256 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 10261 },
	{ 0x8e0d, 0x8e0d, PDF_CMAP_SINGLE, 10259 },
	{ 0x8e0e, 0x8e0e, PDF_CMAP_SINGLE, 15966 },
	{ 0x8e0f, 0x8e0f, PDF_CMAP_SINGLE, 4664 },
	{ 0x8e10, 0x8e10, PDF_CMAP_SINGLE, 4661 },
	{ 0x8e11, 0x8e11, PDF_CMAP_SINGLE, 10857 },
	{ 0x8e12, 0x8e12, PDF_CMAP_SINGLE, 10862 },
	{ 0x8e13, 0x8e13, PDF_CMAP_SINGLE, 10864 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 10861 },
	{ 0x8e15, 0x8e15, PDF_CMAP_SINGLE, 10854 },
	{ 0x8e16, 0x8e16, PDF_CMAP_SINGLE, 10856 },
	{ 0x8e17, 0x8e17, PDF_CMAP_SINGLE, 10866 },
	{ 0x8e18, 0x8e18, PDF_CMAP_SINGLE, 10863 },
	{ 0x8e19, 0x8e19, PDF_CMAP_SINGLE, 10858 },
	{ 0x8e1a, 0x8e1a, PDF_CMAP_SINGLE, 10867 },
	{ 0x8e1b, 0x8e1b, PDF_CMAP_SINGLE, 10855 },
	{ 0x8e1c, 0x8e1c, PDF_CMAP_SINGLE, 10865 },
	{ 0x8e1d, 0x8e1d, PDF_CMAP_SINGLE, 4662 },
	{ 0x8e1e, 0x8e1e, PDF_CMAP_SINGLE, 4668 },
	{ 0x8e1f, 0x8e1f, PDF_CMAP_SINGLE, 4666 },
	{ 0x8e20, 0x8e20, PDF_CMAP_SINGLE, 10849 },
	{ 0x8e21, 0x8e21, PDF_CMAP_SINGLE, 4667 },
	{ 0x8e22, 0x8e22, PDF_CMAP_SINGLE, 4663 },
	{ 0x8e23, 0x8e23, PDF_CMAP_SINGLE, 10850 },
	{ 0x8e24, 0x8e24, PDF_CMAP_SINGLE, 10852 },
	{ 0x8e25, 0x8e25, PDF_CMAP_SINGLE, 10851 },
	{ 0x8e26, 0x8e27, PDF_CMAP_RANGE, 10859 },
	{ 0x8e28, 0x8e28, PDF_CMAP_SINGLE, 16264 },
	{ 0x8e29, 0x8e29, PDF_CMAP_SINGLE, 4665 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 15284 },
	{ 0x8e2b, 0x8e2b, PDF_CMAP_SINGLE, 4660 },
	{ 0x8e2d, 0x8e2d, PDF_CMAP_SINGLE, 16043 },
	{ 0x8e2e, 0x8e2e, PDF_CMAP_SINGLE, 10853 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 11435 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 4962 },
	{ 0x8e33, 0x8e33, PDF_CMAP_SINGLE, 11426 },
	{ 0x8e34, 0x8e34, PDF_CMAP_SINGLE, 4963 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 4966 },
	{ 0x8e36, 0x8e36, PDF_CMAP_SINGLE, 11431 },
	{ 0x8e38, 0x8e38, PDF_CMAP_SINGLE, 11428 },
	{ 0x8e39, 0x8e39, PDF_CMAP_SINGLE, 4965 },
	{ 0x8e3a, 0x8e3a, PDF_CMAP_SINGLE, 18308 },
	{ 0x8e3c, 0x8e3d, PDF_CMAP_RANGE, 11432 },
	{ 0x8e3e, 0x8e3e, PDF_CMAP_SINGLE, 11427 },
	{ 0x8e3f, 0x8e3f, PDF_CMAP_SINGLE, 11436 },
	{ 0x8e40, 0x8e40, PDF_CMAP_SINGLE, 11429 },
	{ 0x8e41, 0x8e41, PDF_CMAP_SINGLE, 11434 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 4964 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 4961 },
	{ 0x8e45, 0x8e45, PDF_CMAP_SINGLE, 11430 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 17170 },
	{ 0x8e47, 0x8e47, PDF_CMAP_SINGLE, 11898 },
	{ 0x8e48, 0x8e48, PDF_CMAP_SINGLE, 5259 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 5257 },
	{ 0x8e4a, 0x8e4a, PDF_CMAP_SINGLE, 5260 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 5258 },
	{ 0x8e4c, 0x8e4c, PDF_CMAP_SINGLE, 11897 },
	{ 0x8e4d, 0x8e4d, PDF_CMAP_SINGLE, 11894 },
	{ 0x8e4e, 0x8e4e, PDF_CMAP_SINGLE, 11893 },
	{ 0x8e4f, 0x8e4f, PDF_CMAP_SINGLE, 16259 },
	{ 0x8e50, 0x8e50, PDF_CMAP_SINGLE, 11896 },
	{ 0x8e53, 0x8e53, PDF_CMAP_SINGLE, 11895 },
	{ 0x8e54, 0x8e54, PDF_CMAP_SINGLE, 12278 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 5429 },
	{ 0x8e56, 0x8e56, PDF_CMAP_SINGLE, 12269 },
	{ 0x8e57, 0x8e57, PDF_CMAP_SINGLE, 12268 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 5424 },
	{ 0x8e5a, 0x8e5a, PDF_CMAP_SINGLE, 12274 },
	{ 0x8e5b, 0x8e5b, PDF_CMAP_SINGLE, 12273 },
	{ 0x8e5c, 0x8e5c, PDF_CMAP_SINGLE, 12265 },
	{ 0x8e5d, 0x8e5d, PDF_CMAP_SINGLE, 12276 },
	{ 0x8e5e, 0x8e5e, PDF_CMAP_SINGLE, 12270 },
	{ 0x8e5f, 0x8e5f, PDF_CMAP_SINGLE, 5428 },
	{ 0x8e60, 0x8e60, PDF_CMAP_SINGLE, 12267 },
	{ 0x8e61, 0x8e61, PDF_CMAP_SINGLE, 12275 },
	{ 0x8e62, 0x8e62, PDF_CMAP_SINGLE, 12266 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 5425 },
	{ 0x8e64, 0x8e64, PDF_CMAP_SINGLE, 5427 },
	{ 0x8e65, 0x8e65, PDF_CMAP_SINGLE, 12271 },
	{ 0x8e66, 0x8e66, PDF_CMAP_SINGLE, 5426 },
	{ 0x8e67, 0x8e67, PDF_CMAP_SINGLE, 12272 },
	{ 0x8e68, 0x8e68, PDF_CMAP_SINGLE, 17229 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 12277 },
	{ 0x8e6a, 0x8e6a, PDF_CMAP_SINGLE, 12605 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 5587 },
	{ 0x8e6d, 0x8e6d, PDF_CMAP_SINGLE, 12602 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 12606 },
	{ 0x8e71, 0x8e71, PDF_CMAP_SINGLE, 17383 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 5584 },
	{ 0x8e73, 0x8e73, PDF_CMAP_SINGLE, 12604 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 5589 },
	{ 0x8e75, 0x8e75, PDF_CMAP_SINGLE, 16265 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 5586 },
	{ 0x8e77, 0x8e77, PDF_CMAP_SINGLE, 16255 },
	{ 0x8e78, 0x8e78, PDF_CMAP_SINGLE, 12603 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 5588 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 12607 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 5583 },
	{ 0x8e7e, 0x8e7e, PDF_CMAP_SINGLE, 17188 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 18637 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 5709 },
	{ 0x8e82, 0x8e82, PDF_CMAP_SINGLE, 5711 },
	{ 0x8e84, 0x8e84, PDF_CMAP_SINGLE, 12864 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 5710 },
	{ 0x8e86, 0x8e86, PDF_CMAP_SINGLE, 12862 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 5585 },
	{ 0x8e88, 0x8e88, PDF_CMAP_SINGLE, 12863 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 5708 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 5779 },
	{ 0x8e8b, 0x8e8b, PDF_CMAP_SINGLE, 5781 },
	{ 0x8e8c, 0x8e8c, PDF_CMAP_SINGLE, 13082 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 5780 },
	{ 0x8e8e, 0x8e8e, PDF_CMAP_SINGLE, 13081 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 13249 },
	{ 0x8e91, 0x8e91, PDF_CMAP_SINGLE, 5851 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 13248 },
	{ 0x8e93, 0x8e93, PDF_CMAP_SINGLE, 5852 },
	{ 0x8e94, 0x8e94, PDF_CMAP_SINGLE, 13246 },
	{ 0x8e95, 0x8e95, PDF_CMAP_SINGLE, 13245 },
	{ 0x8e96, 0x8e97, PDF_CMAP_RANGE, 13250 },
	{ 0x8e98, 0x8e98, PDF_CMAP_SINGLE, 13379 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 13247 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 13473 },
	{ 0x8e9e, 0x8ea0, PDF_CMAP_RANGE, 13470 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 5961 },
	{ 0x8ea3, 0x8ea3, PDF_CMAP_SINGLE, 13547 },
	{ 0x8ea4, 0x8ea4, PDF_CMAP_SINGLE, 13546 },
	{ 0x8ea5, 0x8ea5, PDF_CMAP_SINGLE, 13545 },
	{ 0x8ea6, 0x8ea6, PDF_CMAP_SINGLE, 13578 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 15285 },
	{ 0x8ea8, 0x8ea8, PDF_CMAP_SINGLE, 13623 },
	{ 0x8ea9, 0x8ea9, PDF_CMAP_SINGLE, 13604 },
	{ 0x8eaa, 0x8eaa, PDF_CMAP_SINGLE, 5979 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 1298 },
	{ 0x8eac, 0x8eac, PDF_CMAP_SINGLE, 2511 },
	{ 0x8ead, 0x8ead, PDF_CMAP_SINGLE, 14430 },
	{ 0x8eb0, 0x8eb0, PDF_CMAP_SINGLE, 17385 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 3887 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14431 },
	{ 0x8eb9, 0x8eb9, PDF_CMAP_SINGLE, 18935 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 4669 },
	{ 0x8ebc, 0x8ebc, PDF_CMAP_SINGLE, 14496 },
	{ 0x8ebd, 0x8ebd, PDF_CMAP_SINGLE, 11437 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 5430 },
	{ 0x8ec2, 0x8ec2, PDF_CMAP_SINGLE, 12608 },
	{ 0x8ec3, 0x8ec3, PDF_CMAP_SINGLE, 14432 },
	{ 0x8ec9, 0x8ec9, PDF_CMAP_SINGLE, 13605 },
	{ 0x8eca, 0x8eca, PDF_CMAP_SINGLE, 1299 },
	{ 0x8ecb, 0x8ecb, PDF_CMAP_SINGLE, 1677 },
	{ 0x8ecc, 0x8ecc, PDF_CMAP_SINGLE, 2064 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 2063 },
	{ 0x8ece, 0x8ece, PDF_CMAP_SINGLE, 18312 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 2514 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 7699 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 2512 },
	{ 0x8ed3, 0x8ed3, PDF_CMAP_SINGLE, 7700 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 2513 },
	{ 0x8ed7, 0x8ed7, PDF_CMAP_SINGLE, 8317 },
	{ 0x8ed8, 0x8ed8, PDF_CMAP_SINGLE, 8313 },
	{ 0x8eda, 0x8eda, PDF_CMAP_SINGLE, 15806 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 2995 },
	{ 0x8edc, 0x8edc, PDF_CMAP_SINGLE, 8316 },
	{ 0x8edd, 0x8edd, PDF_CMAP_SINGLE, 8315 },
	{ 0x8ede, 0x8ede, PDF_CMAP_SINGLE, 8314 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 2996 },
	{ 0x8ee0, 0x8ee1, PDF_CMAP_RANGE, 8318 },
	{ 0x8ee2, 0x8ee2, PDF_CMAP_SINGLE, 18313 },
	{ 0x8ee4, 0x8ee4, PDF_CMAP_SINGLE, 18314 },
	{ 0x8ee5, 0x8ee5, PDF_CMAP_SINGLE, 8981 },
	{ 0x8ee6, 0x8ee6, PDF_CMAP_SINGLE, 8979 },
	{ 0x8ee7, 0x8ee8, PDF_CMAP_RANGE, 8983 },
	{ 0x8ee9, 0x8ee9, PDF_CMAP_SINGLE, 8990 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 8986 },
	{ 0x8eec, 0x8eec, PDF_CMAP_SINGLE, 8988 },
	{ 0x8eed, 0x8eed, PDF_CMAP_SINGLE, 18315 },
	{ 0x8eee, 0x8eee, PDF_CMAP_SINGLE, 8980 },
	{ 0x8eef, 0x8eef, PDF_CMAP_SINGLE, 8975 },
	{ 0x8ef1, 0x8ef1, PDF_CMAP_SINGLE, 8987 },
	{ 0x8ef2, 0x8ef2, PDF_CMAP_SINGLE, 18316 },
	{ 0x8ef4, 0x8ef4, PDF_CMAP_SINGLE, 8989 },
	{ 0x8ef5, 0x8ef5, PDF_CMAP_SINGLE, 8982 },
	{ 0x8ef6, 0x8ef6, PDF_CMAP_SINGLE, 8985 },
	{ 0x8ef7, 0x8ef7, PDF_CMAP_SINGLE, 8976 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 3455 },
	{ 0x8ef9, 0x8ef9, PDF_CMAP_SINGLE, 8978 },
	{ 0x8efa, 0x8efa, PDF_CMAP_SINGLE, 8977 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 3454 },
	{ 0x8efc, 0x8efc, PDF_CMAP_SINGLE, 3456 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 3890 },
	{ 0x8eff, 0x8eff, PDF_CMAP_SINGLE, 9658 },
	{ 0x8f00, 0x8f00, PDF_CMAP_SINGLE, 9660 },
	{ 0x8f01, 0x8f01, PDF_CMAP_SINGLE, 9659 },
	{ 0x8f02, 0x8f02, PDF_CMAP_SINGLE, 9664 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 3888 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 9661 },
	{ 0x8f06, 0x8f06, PDF_CMAP_SINGLE, 9657 },
	{ 0x8f07, 0x8f08, PDF_CMAP_RANGE, 9662 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 3889 },
	{ 0x8f0a, 0x8f0a, PDF_CMAP_SINGLE, 3891 },
	{ 0x8f0b, 0x8f0b, PDF_CMAP_SINGLE, 9665 },
	{ 0x8f0d, 0x8f0d, PDF_CMAP_SINGLE, 10272 },
	{ 0x8f0e, 0x8f0e, PDF_CMAP_SINGLE, 10271 },
	{ 0x8f10, 0x8f11, PDF_CMAP_RANGE, 10269 },
	{ 0x8f12, 0x8f12, PDF_CMAP_SINGLE, 4287 },
	{ 0x8f13, 0x8f13, PDF_CMAP_SINGLE, 4289 },
	{ 0x8f14, 0x8f14, PDF_CMAP_SINGLE, 4286 },
	{ 0x8f15, 0x8f15, PDF_CMAP_SINGLE, 4288 },
	{ 0x8f16, 0x8f17, PDF_CMAP_RANGE, 10874 },
	{ 0x8f18, 0x8f18, PDF_CMAP_SINGLE, 10870 },
	{ 0x8f19, 0x8f19, PDF_CMAP_SINGLE, 14434 },
	{ 0x8f1a, 0x8f1a, PDF_CMAP_SINGLE, 10871 },
	{ 0x8f1b, 0x8f1b, PDF_CMAP_SINGLE, 4671 },
	{ 0x8f1c, 0x8f1c, PDF_CMAP_SINGLE, 4676 },
	{ 0x8f1d, 0x8f1d, PDF_CMAP_SINGLE, 4670 },
	{ 0x8f1e, 0x8f1e, PDF_CMAP_SINGLE, 4677 },
	{ 0x8f1f, 0x8f1f, PDF_CMAP_SINGLE, 4672 },
	{ 0x8f20, 0x8f20, PDF_CMAP_SINGLE, 10872 },
	{ 0x8f23, 0x8f23, PDF_CMAP_SINGLE, 10873 },
	{ 0x8f24, 0x8f24, PDF_CMAP_SINGLE, 10869 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 4678 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 4674 },
	{ 0x8f29, 0x8f29, PDF_CMAP_SINGLE, 4673 },
	{ 0x8f2a, 0x8f2a, PDF_CMAP_SINGLE, 4675 },
	{ 0x8f2c, 0x8f2c, PDF_CMAP_SINGLE, 10868 },
	{ 0x8f2d, 0x8f2d, PDF_CMAP_SINGLE, 14435 },
	{ 0x8f2e, 0x8f2e, PDF_CMAP_SINGLE, 11439 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 4968 },
	{ 0x8f30, 0x8f30, PDF_CMAP_SINGLE, 15287 },
	{ 0x8f32, 0x8f32, PDF_CMAP_SINGLE, 11441 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 4970 },
	{ 0x8f34, 0x8f34, PDF_CMAP_SINGLE, 11444 },
	{ 0x8f35, 0x8f35, PDF_CMAP_SINGLE, 11440 },
	{ 0x8f36, 0x8f36, PDF_CMAP_SINGLE, 11438 },
	{ 0x8f37, 0x8f37, PDF_CMAP_SINGLE, 11443 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 4969 },
	{ 0x8f39, 0x8f39, PDF_CMAP_SINGLE, 11442 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 4967 },
	{ 0x8f3e, 0x8f3e, PDF_CMAP_SINGLE, 5262 },
	{ 0x8f3f, 0x8f3f, PDF_CMAP_SINGLE, 5265 },
	{ 0x8f40, 0x8f40, PDF_CMAP_SINGLE, 11900 },
	{ 0x8f41, 0x8f41, PDF_CMAP_SINGLE, 15623 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 5263 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 11899 },
	{ 0x8f44, 0x8f44, PDF_CMAP_SINGLE, 5261 },
	{ 0x8f45, 0x8f45, PDF_CMAP_SINGLE, 5264 },
	{ 0x8f46, 0x8f48, PDF_CMAP_RANGE, 12279 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 5431 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 15288 },
	{ 0x8f4b, 0x8f4b, PDF_CMAP_SINGLE, 12282 },
	{ 0x8f4d, 0x8f4d, PDF_CMAP_SINGLE, 5432 },
	{ 0x8f4e, 0x8f4e, PDF_CMAP_SINGLE, 5591 },
	{ 0x8f4f, 0x8f50, PDF_CMAP_RANGE, 12611 },
	{ 0x8f51, 0x8f51, PDF_CMAP_SINGLE, 12610 },
	{ 0x8f52, 0x8f52, PDF_CMAP_SINGLE, 12609 },
	{ 0x8f53, 0x8f53, PDF_CMAP_SINGLE, 12613 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 5590 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 12868 },
	{ 0x8f56, 0x8f57, PDF_CMAP_RANGE, 12866 },
	{ 0x8f58, 0x8f58, PDF_CMAP_SINGLE, 12869 },
	{ 0x8f59, 0x8f59, PDF_CMAP_SINGLE, 12865 },
	{ 0x8f5a, 0x8f5a, PDF_CMAP_SINGLE, 12870 },
	{ 0x8f5b, 0x8f5b, PDF_CMAP_SINGLE, 13084 },
	{ 0x8f5c, 0x8f5c, PDF_CMAP_SINGLE, 16177 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 13085 },
	{ 0x8f5e, 0x8f5e, PDF_CMAP_SINGLE, 13083 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 5782 },
	{ 0x8f60, 0x8f60, PDF_CMAP_SINGLE, 13252 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 5853 },
	{ 0x8f62, 0x8f62, PDF_CMAP_SINGLE, 13253 },
	{ 0x8f63, 0x8f63, PDF_CMAP_SINGLE, 13381 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 13380 },
	{ 0x8f66, 0x8f67, PDF_CMAP_RANGE, 17769 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 17771 },
	{ 0x8f93, 0x8f93, PDF_CMAP_SINGLE, 18226 },
	{ 0x8f9b, 0x8f9b, PDF_CMAP_SINGLE, 1300 },
	{ 0x8f9c, 0x8f9c, PDF_CMAP_SINGLE, 3457 },
	{ 0x8f9f, 0x8f9f, PDF_CMAP_SINGLE, 3892 },
	{ 0x8fa0, 0x8fa0, PDF_CMAP_SINGLE, 15764 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 4290 },
	{ 0x8fa5, 0x8fa5, PDF_CMAP_SINGLE, 14438 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 4972 },
	{ 0x8fa7, 0x8fa7, PDF_CMAP_SINGLE, 18864 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 4971 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 5592 },
	{ 0x8fae, 0x8fae, PDF_CMAP_SINGLE, 5679 },
	{ 0x8faf, 0x8faf, PDF_CMAP_SINGLE, 5783 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 1301 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 2515 },
	{ 0x8fb2, 0x8fb2, PDF_CMAP_SINGLE, 3893 },
	{ 0x8fb3, 0x8fb3, PDF_CMAP_SINGLE, 14442 },
	{ 0x8fb4, 0x8fb4, PDF_CMAP_SINGLE, 12614 },
	{ 0x8fb5, 0x8fb5, PDF_CMAP_SINGLE, 560 },
	{ 0x8fb6, 0x8fb6, PDF_CMAP_SINGLE, 18387 },
	{ 0x8fb7, 0x8fb7, PDF_CMAP_SINGLE, 17405 },
	{ 0x8fb8, 0x8fb8, PDF_CMAP_SINGLE, 17142 },
	{ 0x8fb9, 0x8fb9, PDF_CMAP_SINGLE, 18633 },
	{ 0x8fba, 0x8fba, PDF_CMAP_SINGLE, 18631 },
	{ 0x8fbb, 0x8fbc, PDF_CMAP_RANGE, 15723 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 18630 },
	{ 0x8fbf, 0x8fbf, PDF_CMAP_SINGLE, 6377 },
	{ 0x8fc1, 0x8fc1, PDF_CMAP_SINGLE, 18317 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 1302 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 1305 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 1304 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 1303 },
	{ 0x8fc9, 0x8fc9, PDF_CMAP_SINGLE, 6376 },
	{ 0x8fca, 0x8fca, PDF_CMAP_SINGLE, 18318 },
	{ 0x8fcb, 0x8fcb, PDF_CMAP_SINGLE, 6730 },
	{ 0x8fcc, 0x8fcc, PDF_CMAP_SINGLE, 18319 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 6732 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 1678 },
	{ 0x8fcf, 0x8fcf, PDF_CMAP_SINGLE, 18937 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 18606 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 1680 },
	{ 0x8fd2, 0x8fd2, PDF_CMAP_SINGLE, 6729 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 6731 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 1679 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 6734 },
	{ 0x8fd6, 0x8fd6, PDF_CMAP_SINGLE, 6733 },
	{ 0x8fd7, 0x8fd7, PDF_CMAP_SINGLE, 6735 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 16331 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 7178 },
	{ 0x8fe1, 0x8fe1, PDF_CMAP_SINGLE, 7176 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 2067 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 7175 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 2072 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 2069 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 2066 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 2073 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 2068 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 2071 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 2070 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 7177 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 2065 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 2521 },
	{ 0x8ff5, 0x8ff5, PDF_CMAP_SINGLE, 7702 },
	{ 0x8ff6, 0x8ff6, PDF_CMAP_SINGLE, 7708 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 2518 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 2525 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 14449 },
	{ 0x8ffa, 0x8ffa, PDF_CMAP_SINGLE, 2520 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 7705 },
	{ 0x8ffc, 0x8ffc, PDF_CMAP_SINGLE, 7707 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 2523 },
	{ 0x8ffe, 0x8ffe, PDF_CMAP_SINGLE, 7701 },
	{ 0x8fff, 0x8fff, PDF_CMAP_SINGLE, 7704 },
	{ 0x9000, 0x9000, PDF_CMAP_SINGLE, 2519 },
	{ 0x9001, 0x9001, PDF_CMAP_SINGLE, 2516 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 7703 },
	{ 0x9003, 0x9003, PDF_CMAP_SINGLE, 2522 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 7706 },
	{ 0x9005, 0x9005, PDF_CMAP_SINGLE, 2524 },
	{ 0x9006, 0x9006, PDF_CMAP_SINGLE, 2517 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 16096 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 8321 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 8324 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 2998 },
	{ 0x900f, 0x900f, PDF_CMAP_SINGLE, 3008 },
	{ 0x9010, 0x9010, PDF_CMAP_SINGLE, 3004 },
	{ 0x9011, 0x9011, PDF_CMAP_SINGLE, 8322 },
	{ 0x9012, 0x9012, PDF_CMAP_SINGLE, 15950 },
	{ 0x9014, 0x9014, PDF_CMAP_SINGLE, 3012 },
	{ 0x9015, 0x9015, PDF_CMAP_SINGLE, 3005 },
	{ 0x9016, 0x9016, PDF_CMAP_SINGLE, 3010 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 3000 },
	{ 0x9019, 0x9019, PDF_CMAP_SINGLE, 2997 },
	{ 0x901a, 0x901a, PDF_CMAP_SINGLE, 2999 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 3011 },
	{ 0x901c, 0x901c, PDF_CMAP_SINGLE, 8323 },
	{ 0x901d, 0x901d, PDF_CMAP_SINGLE, 3003 },
	{ 0x901e, 0x901e, PDF_CMAP_SINGLE, 3006 },
	{ 0x901f, 0x901f, PDF_CMAP_SINGLE, 3002 },
	{ 0x9020, 0x9020, PDF_CMAP_SINGLE, 3007 },
	{ 0x9021, 0x9021, PDF_CMAP_SINGLE, 8325 },
	{ 0x9022, 0x9022, PDF_CMAP_SINGLE, 3009 },
	{ 0x9023, 0x9023, PDF_CMAP_SINGLE, 3001 },
	{ 0x9024, 0x9024, PDF_CMAP_SINGLE, 8320 },
	{ 0x902d, 0x902d, PDF_CMAP_SINGLE, 8991 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 3458 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 8993 },
	{ 0x9031, 0x9031, PDF_CMAP_SINGLE, 3460 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 3462 },
	{ 0x9033, 0x9033, PDF_CMAP_SINGLE, 18320 },
	{ 0x9034, 0x9034, PDF_CMAP_SINGLE, 8992 },
	{ 0x9035, 0x9035, PDF_CMAP_SINGLE, 3459 },
	{ 0x9036, 0x9036, PDF_CMAP_SINGLE, 3463 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 14454 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 3461 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 3899 },
	{ 0x903d, 0x903d, PDF_CMAP_SINGLE, 9670 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 3907 },
	{ 0x903f, 0x903f, PDF_CMAP_SINGLE, 9667 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 3908 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 3897 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 9668 },
	{ 0x9046, 0x9046, PDF_CMAP_SINGLE, 19121 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 3902 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 9669 },
	{ 0x904a, 0x904a, PDF_CMAP_SINGLE, 3895 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 3894 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 16144 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 3905 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 3904 },
	{ 0x904f, 0x904f, PDF_CMAP_SINGLE, 3903 },
	{ 0x9050, 0x9050, PDF_CMAP_SINGLE, 3901 },
	{ 0x9051, 0x9051, PDF_CMAP_SINGLE, 3906 },
	{ 0x9052, 0x9052, PDF_CMAP_SINGLE, 9666 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 3896 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 3898 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 3900 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 15826 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 4292 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 4295 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 4299 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 4293 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 4298 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 4296 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 4291 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 14457 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 4297 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 4294 },
	{ 0x9064, 0x9064, PDF_CMAP_SINGLE, 15464 },
	{ 0x9067, 0x9067, PDF_CMAP_SINGLE, 10879 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 4681 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 4679 },
	{ 0x906b, 0x906b, PDF_CMAP_SINGLE, 10880 },
	{ 0x906c, 0x906c, PDF_CMAP_SINGLE, 15712 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 4682 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 4680 },
	{ 0x906f, 0x906f, PDF_CMAP_SINGLE, 10878 },
	{ 0x9070, 0x9070, PDF_CMAP_SINGLE, 10877 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 4976 },
	{ 0x9073, 0x9073, PDF_CMAP_SINGLE, 10876 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 4974 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 4973 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 11445 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 4683 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 4975 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 11446 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 4978 },
	{ 0x907b, 0x907b, PDF_CMAP_SINGLE, 11447 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 4977 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 5267 },
	{ 0x907e, 0x907e, PDF_CMAP_SINGLE, 11902 },
	{ 0x907f, 0x907f, PDF_CMAP_SINGLE, 5266 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 5271 },
	{ 0x9081, 0x9082, PDF_CMAP_RANGE, 5269 },
	{ 0x9083, 0x9083, PDF_CMAP_SINGLE, 5434 },
	{ 0x9084, 0x9084, PDF_CMAP_SINGLE, 5268 },
	{ 0x9085, 0x9085, PDF_CMAP_SINGLE, 11901 },
	{ 0x9086, 0x9086, PDF_CMAP_SINGLE, 11448 },
	{ 0x9087, 0x9087, PDF_CMAP_SINGLE, 5433 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 5435 },
	{ 0x908a, 0x908b, PDF_CMAP_RANGE, 5593 },
	{ 0x908d, 0x908d, PDF_CMAP_SINGLE, 12871 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 5900 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 5899 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 1307 },
	{ 0x9094, 0x9094, PDF_CMAP_SINGLE, 6174 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 2526 },
	{ 0x9097, 0x9098, PDF_CMAP_RANGE, 6171 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 6170 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 6173 },
	{ 0x909e, 0x909e, PDF_CMAP_SINGLE, 6381 },
	{ 0x909f, 0x909f, PDF_CMAP_SINGLE, 6378 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 6383 },
	{ 0x90a1, 0x90a1, PDF_CMAP_SINGLE, 6379 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 1308 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 1311 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 6380 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 1310 },
	{ 0x90a7, 0x90a7, PDF_CMAP_SINGLE, 6382 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 14460 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 1309 },
	{ 0x90ae, 0x90ae, PDF_CMAP_SINGLE, 14464 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 6738 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 6740 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 1683 },
	{ 0x90b2, 0x90b2, PDF_CMAP_SINGLE, 6736 },
	{ 0x90b3, 0x90b3, PDF_CMAP_SINGLE, 6739 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 6737 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 1681 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 1684 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 1682 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 15300 },
	{ 0x90bd, 0x90bd, PDF_CMAP_SINGLE, 7180 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 7184 },
	{ 0x90bf, 0x90bf, PDF_CMAP_SINGLE, 7181 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 2076 },
	{ 0x90c3, 0x90c3, PDF_CMAP_SINGLE, 2077 },
	{ 0x90c4, 0x90c4, PDF_CMAP_SINGLE, 14462 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 7183 },
	{ 0x90c7, 0x90c7, PDF_CMAP_SINGLE, 7185 },
	{ 0x90c8, 0x90c8, PDF_CMAP_SINGLE, 7187 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2074 },
	{ 0x90cb, 0x90cb, PDF_CMAP_SINGLE, 7186 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 2075 },
	{ 0x90d4, 0x90d4, PDF_CMAP_SINGLE, 8332 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 7182 },
	{ 0x90d6, 0x90d6, PDF_CMAP_SINGLE, 7709 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 7718 },
	{ 0x90d8, 0x90d8, PDF_CMAP_SINGLE, 7716 },
	{ 0x90d9, 0x90da, PDF_CMAP_RANGE, 7711 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 7717 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 7719 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 2528 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 7714 },
	{ 0x90e0, 0x90e0, PDF_CMAP_SINGLE, 7710 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 2527 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 2529 },
	{ 0x90e3, 0x90e3, PDF_CMAP_SINGLE, 7713 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 7720 },
	{ 0x90e5, 0x90e5, PDF_CMAP_SINGLE, 7715 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 3013 },
	{ 0x90e9, 0x90e9, PDF_CMAP_SINGLE, 8335 },
	{ 0x90ea, 0x90ea, PDF_CMAP_SINGLE, 8327 },
	{ 0x90eb, 0x90ec, PDF_CMAP_RANGE, 8333 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 3014 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 8326 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 8328 },
	{ 0x90f1, 0x90f1, PDF_CMAP_SINGLE, 7179 },
	{ 0x90f2, 0x90f3, PDF_CMAP_RANGE, 8330 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 8329 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 3465 },
	{ 0x90f9, 0x90f9, PDF_CMAP_SINGLE, 9000 },
	{ 0x90fa, 0x90fa, PDF_CMAP_SINGLE, 11449 },
	{ 0x90fb, 0x90fb, PDF_CMAP_SINGLE, 9001 },
	{ 0x90fc, 0x90fc, PDF_CMAP_SINGLE, 8998 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 3015 },
	{ 0x90fe, 0x90fe, PDF_CMAP_SINGLE, 3467 },
	{ 0x90ff, 0x90ff, PDF_CMAP_SINGLE, 8997 },
	{ 0x9100, 0x9100, PDF_CMAP_SINGLE, 9003 },
	{ 0x9101, 0x9101, PDF_CMAP_SINGLE, 9002 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 3464 },
	{ 0x9103, 0x9103, PDF_CMAP_SINGLE, 9006 },
	{ 0x9104, 0x9104, PDF_CMAP_SINGLE, 8996 },
	{ 0x9105, 0x9105, PDF_CMAP_SINGLE, 9005 },
	{ 0x9106, 0x9106, PDF_CMAP_SINGLE, 8994 },
	{ 0x9107, 0x9107, PDF_CMAP_SINGLE, 9004 },
	{ 0x9108, 0x9108, PDF_CMAP_SINGLE, 8999 },
	{ 0x9109, 0x9109, PDF_CMAP_SINGLE, 3466 },
	{ 0x910b, 0x910b, PDF_CMAP_SINGLE, 9677 },
	{ 0x910d, 0x910d, PDF_CMAP_SINGLE, 9672 },
	{ 0x910e, 0x910e, PDF_CMAP_SINGLE, 9678 },
	{ 0x910f, 0x910f, PDF_CMAP_SINGLE, 9673 },
	{ 0x9110, 0x9110, PDF_CMAP_SINGLE, 9671 },
	{ 0x9111, 0x9111, PDF_CMAP_SINGLE, 9674 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 3909 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 9676 },
	{ 0x9116, 0x9116, PDF_CMAP_SINGLE, 9675 },
	{ 0x9117, 0x9117, PDF_CMAP_SINGLE, 3910 },
	{ 0x9118, 0x9118, PDF_CMAP_SINGLE, 4301 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 4300 },
	{ 0x911a, 0x911a, PDF_CMAP_SINGLE, 10279 },
	{ 0x911b, 0x911b, PDF_CMAP_SINGLE, 10282 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 10274 },
	{ 0x911d, 0x911d, PDF_CMAP_SINGLE, 10278 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 4302 },
	{ 0x911f, 0x911f, PDF_CMAP_SINGLE, 10277 },
	{ 0x9120, 0x9120, PDF_CMAP_SINGLE, 10275 },
	{ 0x9121, 0x9121, PDF_CMAP_SINGLE, 10281 },
	{ 0x9122, 0x9122, PDF_CMAP_SINGLE, 10276 },
	{ 0x9123, 0x9123, PDF_CMAP_SINGLE, 10273 },
	{ 0x9124, 0x9124, PDF_CMAP_SINGLE, 10280 },
	{ 0x9126, 0x9126, PDF_CMAP_SINGLE, 10886 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 4686 },
	{ 0x9128, 0x9128, PDF_CMAP_SINGLE, 12283 },
	{ 0x9129, 0x912a, PDF_CMAP_RANGE, 10883 },
	{ 0x912b, 0x912b, PDF_CMAP_SINGLE, 10882 },
	{ 0x912c, 0x912c, PDF_CMAP_SINGLE, 8995 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 4685 },
	{ 0x912e, 0x912e, PDF_CMAP_SINGLE, 10887 },
	{ 0x912f, 0x912f, PDF_CMAP_SINGLE, 10881 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 4684 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 4687 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 10885 },
	{ 0x9133, 0x9133, PDF_CMAP_SINGLE, 11450 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 4979 },
	{ 0x9135, 0x9136, PDF_CMAP_RANGE, 11451 },
	{ 0x9138, 0x9138, PDF_CMAP_SINGLE, 11903 },
	{ 0x9139, 0x9139, PDF_CMAP_SINGLE, 5272 },
	{ 0x913a, 0x913b, PDF_CMAP_RANGE, 12284 },
	{ 0x913e, 0x913e, PDF_CMAP_SINGLE, 12286 },
	{ 0x913f, 0x913f, PDF_CMAP_SINGLE, 12616 },
	{ 0x9140, 0x9140, PDF_CMAP_SINGLE, 12615 },
	{ 0x9141, 0x9141, PDF_CMAP_SINGLE, 12873 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 12872 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 13087 },
	{ 0x9146, 0x9146, PDF_CMAP_SINGLE, 13086 },
	{ 0x9147, 0x9147, PDF_CMAP_SINGLE, 13254 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 5854 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 1312 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 2079 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 2078 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 2532 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 2531 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7722 },
	{ 0x9150, 0x9150, PDF_CMAP_SINGLE, 7721 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 16201 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2530 },
	{ 0x9153, 0x9153, PDF_CMAP_SINGLE, 8339 },
	{ 0x9155, 0x9155, PDF_CMAP_SINGLE, 8340 },
	{ 0x9156, 0x9156, PDF_CMAP_SINGLE, 8336 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 3016 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 8337 },
	{ 0x9159, 0x9159, PDF_CMAP_SINGLE, 16197 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 8338 },
	{ 0x915c, 0x915c, PDF_CMAP_SINGLE, 16199 },
	{ 0x915e, 0x915e, PDF_CMAP_SINGLE, 17387 },
	{ 0x915f, 0x915f, PDF_CMAP_SINGLE, 9009 },
	{ 0x9160, 0x9160, PDF_CMAP_SINGLE, 9011 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 9007 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 9010 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 3468 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 9008 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 3469 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 14466 },
	{ 0x9168, 0x9168, PDF_CMAP_SINGLE, 9186 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 3913 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 3912 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 3911 },
	{ 0x916e, 0x916f, PDF_CMAP_RANGE, 9679 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 16056 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 10284 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 10286 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 4306 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 4303 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 16027 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 4305 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 4304 },
	{ 0x9179, 0x9179, PDF_CMAP_SINGLE, 10285 },
	{ 0x917a, 0x917a, PDF_CMAP_SINGLE, 10283 },
	{ 0x917c, 0x917c, PDF_CMAP_SINGLE, 16239 },
	{ 0x9180, 0x9180, PDF_CMAP_SINGLE, 10894 },
	{ 0x9181, 0x9182, PDF_CMAP_RANGE, 10891 },
	{ 0x9183, 0x9183, PDF_CMAP_SINGLE, 4691 },
	{ 0x9184, 0x9184, PDF_CMAP_SINGLE, 10893 },
	{ 0x9185, 0x9186, PDF_CMAP_RANGE, 10888 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 4688 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 4689 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 10890 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 4690 },
	{ 0x918c, 0x918c, PDF_CMAP_SINGLE, 17388 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 11456 },
	{ 0x918e, 0x918e, PDF_CMAP_SINGLE, 16206 },
	{ 0x918f, 0x918f, PDF_CMAP_SINGLE, 11457 },
	{ 0x9190, 0x9191, PDF_CMAP_RANGE, 11454 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 4980 },
	{ 0x9193, 0x9193, PDF_CMAP_SINGLE, 11453 },
	{ 0x9199, 0x9199, PDF_CMAP_SINGLE, 11907 },
	{ 0x919a, 0x919a, PDF_CMAP_SINGLE, 11904 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 11906 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 5275 },
	{ 0x919d, 0x919d, PDF_CMAP_SINGLE, 11910 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 5274 },
	{ 0x919f, 0x919f, PDF_CMAP_SINGLE, 11908 },
	{ 0x91a0, 0x91a0, PDF_CMAP_SINGLE, 11911 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 11909 },
	{ 0x91a2, 0x91a2, PDF_CMAP_SINGLE, 11905 },
	{ 0x91a3, 0x91a3, PDF_CMAP_SINGLE, 5273 },
	{ 0x91a5, 0x91a5, PDF_CMAP_SINGLE, 12288 },
	{ 0x91a7, 0x91a7, PDF_CMAP_SINGLE, 12289 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 12287 },
	{ 0x91a9, 0x91a9, PDF_CMAP_SINGLE, 14467 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 12291 },
	{ 0x91ab, 0x91ac, PDF_CMAP_RANGE, 5436 },
	{ 0x91ad, 0x91ad, PDF_CMAP_SINGLE, 12618 },
	{ 0x91ae, 0x91ae, PDF_CMAP_SINGLE, 5596 },
	{ 0x91af, 0x91af, PDF_CMAP_SINGLE, 12290 },
	{ 0x91b0, 0x91b0, PDF_CMAP_SINGLE, 12617 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 5595 },
	{ 0x91b2, 0x91b3, PDF_CMAP_RANGE, 12876 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 5712 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 12875 },
	{ 0x91b6, 0x91b6, PDF_CMAP_SINGLE, 17903 },
	{ 0x91b7, 0x91b7, PDF_CMAP_SINGLE, 12874 },
	{ 0x91b9, 0x91b9, PDF_CMAP_SINGLE, 13089 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 5784 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 15304 },
	{ 0x91bc, 0x91bc, PDF_CMAP_SINGLE, 13382 },
	{ 0x91bd, 0x91bd, PDF_CMAP_SINGLE, 13475 },
	{ 0x91be, 0x91be, PDF_CMAP_SINGLE, 13474 },
	{ 0x91c0, 0x91c0, PDF_CMAP_SINGLE, 5936 },
	{ 0x91c1, 0x91c1, PDF_CMAP_SINGLE, 5962 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 13476 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 13579 },
	{ 0x91c4, 0x91c4, PDF_CMAP_SINGLE, 14468 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 5980 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 1313 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 1685 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 3914 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 5713 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 1314 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 2080 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 3017 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 3470 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 5438 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 1686 },
	{ 0x91d3, 0x91d3, PDF_CMAP_SINGLE, 7189 },
	{ 0x91d4, 0x91d4, PDF_CMAP_SINGLE, 7188 },
	{ 0x91d5, 0x91d5, PDF_CMAP_SINGLE, 7724 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 16734 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 2535 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 2533 },
	{ 0x91d9, 0x91d9, PDF_CMAP_SINGLE, 2537 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 7726 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 2536 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 2534 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 17722 },
	{ 0x91e2, 0x91e2, PDF_CMAP_SINGLE, 7725 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 3020 },
	{ 0x91e4, 0x91e4, PDF_CMAP_SINGLE, 8346 },
	{ 0x91e5, 0x91e5, PDF_CMAP_SINGLE, 15366 },
	{ 0x91e6, 0x91e6, PDF_CMAP_SINGLE, 3019 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 3021 },
	{ 0x91e8, 0x91e8, PDF_CMAP_SINGLE, 8351 },
	{ 0x91e9, 0x91e9, PDF_CMAP_SINGLE, 3023 },
	{ 0x91ea, 0x91eb, PDF_CMAP_RANGE, 8348 },
	{ 0x91ec, 0x91ec, PDF_CMAP_SINGLE, 8341 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 3022 },
	{ 0x91ee, 0x91ee, PDF_CMAP_SINGLE, 8352 },
	{ 0x91f1, 0x91f1, PDF_CMAP_SINGLE, 8343 },
	{ 0x91f3, 0x91f3, PDF_CMAP_SINGLE, 8344 },
	{ 0x91f4, 0x91f4, PDF_CMAP_SINGLE, 8342 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 3018 },
	{ 0x91f6, 0x91f6, PDF_CMAP_SINGLE, 18865 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 8350 },
	{ 0x91f8, 0x91f8, PDF_CMAP_SINGLE, 8345 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 8347 },
	{ 0x91fa, 0x91fa, PDF_CMAP_SINGLE, 15096 },
	{ 0x91fd, 0x91fd, PDF_CMAP_SINGLE, 9023 },
	{ 0x91fe, 0x91fe, PDF_CMAP_SINGLE, 14382 },
	{ 0x91ff, 0x91ff, PDF_CMAP_SINGLE, 9022 },
	{ 0x9200, 0x9200, PDF_CMAP_SINGLE, 9020 },
	{ 0x9201, 0x9201, PDF_CMAP_SINGLE, 9012 },
	{ 0x9202, 0x9202, PDF_CMAP_SINGLE, 9027 },
	{ 0x9203, 0x9203, PDF_CMAP_SINGLE, 9015 },
	{ 0x9204, 0x9204, PDF_CMAP_SINGLE, 9025 },
	{ 0x9205, 0x9205, PDF_CMAP_SINGLE, 9032 },
	{ 0x9206, 0x9206, PDF_CMAP_SINGLE, 9024 },
	{ 0x9207, 0x9207, PDF_CMAP_SINGLE, 3478 },
	{ 0x9208, 0x9208, PDF_CMAP_SINGLE, 16057 },
	{ 0x9209, 0x9209, PDF_CMAP_SINGLE, 3474 },
	{ 0x920a, 0x920a, PDF_CMAP_SINGLE, 9013 },
	{ 0x920c, 0x920c, PDF_CMAP_SINGLE, 9019 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 3476 },
	{ 0x920e, 0x920e, PDF_CMAP_SINGLE, 14472 },
	{ 0x920f, 0x920f, PDF_CMAP_SINGLE, 9018 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 3477 },
	{ 0x9211, 0x9211, PDF_CMAP_SINGLE, 3479 },
	{ 0x9212, 0x9212, PDF_CMAP_SINGLE, 9021 },
	{ 0x9213, 0x9213, PDF_CMAP_SINGLE, 18323 },
	{ 0x9214, 0x9215, PDF_CMAP_RANGE, 3471 },
	{ 0x9216, 0x9216, PDF_CMAP_SINGLE, 9033 },
	{ 0x9217, 0x9217, PDF_CMAP_SINGLE, 9031 },
	{ 0x9218, 0x9218, PDF_CMAP_SINGLE, 19110 },
	{ 0x9219, 0x9219, PDF_CMAP_SINGLE, 9030 },
	{ 0x921a, 0x921a, PDF_CMAP_SINGLE, 9016 },
	{ 0x921c, 0x921c, PDF_CMAP_SINGLE, 9028 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 3475 },
	{ 0x9221, 0x9221, PDF_CMAP_SINGLE, 18940 },
	{ 0x9223, 0x9223, PDF_CMAP_SINGLE, 3473 },
	{ 0x9224, 0x9224, PDF_CMAP_SINGLE, 9029 },
	{ 0x9225, 0x9225, PDF_CMAP_SINGLE, 9014 },
	{ 0x9226, 0x9226, PDF_CMAP_SINGLE, 9017 },
	{ 0x9227, 0x9227, PDF_CMAP_SINGLE, 9026 },
	{ 0x9228, 0x9228, PDF_CMAP_SINGLE, 18325 },
	{ 0x922a, 0x922a, PDF_CMAP_SINGLE, 15374 },
	{ 0x922b, 0x922b, PDF_CMAP_SINGLE, 15270 },
	{ 0x922d, 0x922d, PDF_CMAP_SINGLE, 10321 },
	{ 0x922e, 0x922e, PDF_CMAP_SINGLE, 9690 },
	{ 0x9230, 0x9230, PDF_CMAP_SINGLE, 9683 },
	{ 0x9231, 0x9231, PDF_CMAP_SINGLE, 9702 },
	{ 0x9232, 0x9232, PDF_CMAP_SINGLE, 9711 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 9686 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 3925 },
	{ 0x9235, 0x9235, PDF_CMAP_SINGLE, 15142 },
	{ 0x9236, 0x9236, PDF_CMAP_SINGLE, 9699 },
	{ 0x9237, 0x9237, PDF_CMAP_SINGLE, 3915 },
	{ 0x9238, 0x9238, PDF_CMAP_SINGLE, 3917 },
	{ 0x9239, 0x9239, PDF_CMAP_SINGLE, 3929 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 9684 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 16850 },
	{ 0x923d, 0x923d, PDF_CMAP_SINGLE, 3918 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 3920 },
	{ 0x923f, 0x923f, PDF_CMAP_SINGLE, 3930 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 3919 },
	{ 0x9241, 0x9241, PDF_CMAP_SINGLE, 14474 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 14490 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 3928 },
	{ 0x9246, 0x9246, PDF_CMAP_SINGLE, 9692 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 9681 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 3926 },
	{ 0x924a, 0x924a, PDF_CMAP_SINGLE, 9691 },
	{ 0x924b, 0x924b, PDF_CMAP_SINGLE, 3922 },
	{ 0x924c, 0x924c, PDF_CMAP_SINGLE, 9709 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 3927 },
	{ 0x924e, 0x924e, PDF_CMAP_SINGLE, 9707 },
	{ 0x924f, 0x924f, PDF_CMAP_SINGLE, 9695 },
	{ 0x9250, 0x9250, PDF_CMAP_SINGLE, 9705 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 3924 },
	{ 0x9252, 0x9252, PDF_CMAP_SINGLE, 9682 },
	{ 0x9253, 0x9253, PDF_CMAP_SINGLE, 9708 },
	{ 0x9254, 0x9254, PDF_CMAP_SINGLE, 9703 },
	{ 0x9255, 0x9255, PDF_CMAP_SINGLE, 15943 },
	{ 0x9256, 0x9256, PDF_CMAP_SINGLE, 9710 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 3916 },
	{ 0x9258, 0x9258, PDF_CMAP_SINGLE, 18326 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 3931 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 3921 },
	{ 0x925d, 0x925d, PDF_CMAP_SINGLE, 14851 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 9688 },
	{ 0x925f, 0x925f, PDF_CMAP_SINGLE, 15584 },
	{ 0x9260, 0x9260, PDF_CMAP_SINGLE, 9696 },
	{ 0x9261, 0x9261, PDF_CMAP_SINGLE, 9700 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 14475 },
	{ 0x9263, 0x9263, PDF_CMAP_SINGLE, 9704 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 3923 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 9687 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 9685 },
	{ 0x9267, 0x9267, PDF_CMAP_SINGLE, 9697 },
	{ 0x926b, 0x926b, PDF_CMAP_SINGLE, 18327 },
	{ 0x926c, 0x926c, PDF_CMAP_SINGLE, 9694 },
	{ 0x926d, 0x926d, PDF_CMAP_SINGLE, 9693 },
	{ 0x926e, 0x926e, PDF_CMAP_SINGLE, 15621 },
	{ 0x926f, 0x926f, PDF_CMAP_SINGLE, 9698 },
	{ 0x9270, 0x9270, PDF_CMAP_SINGLE, 9701 },
	{ 0x9272, 0x9272, PDF_CMAP_SINGLE, 9706 },
	{ 0x9276, 0x9276, PDF_CMAP_SINGLE, 10289 },
	{ 0x9277, 0x9277, PDF_CMAP_SINGLE, 15460 },
	{ 0x9278, 0x9278, PDF_CMAP_SINGLE, 4307 },
	{ 0x9279, 0x9279, PDF_CMAP_SINGLE, 10299 },
	{ 0x927a, 0x927a, PDF_CMAP_SINGLE, 10291 },
	{ 0x927b, 0x927b, PDF_CMAP_SINGLE, 4313 },
	{ 0x927c, 0x927c, PDF_CMAP_SINGLE, 4317 },
	{ 0x927d, 0x927d, PDF_CMAP_SINGLE, 10308 },
	{ 0x927e, 0x927e, PDF_CMAP_SINGLE, 10316 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 10301 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 4309 },
	{ 0x9281, 0x9281, PDF_CMAP_SINGLE, 15873 },
	{ 0x9282, 0x9282, PDF_CMAP_SINGLE, 10305 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 9689 },
	{ 0x9284, 0x9284, PDF_CMAP_SINGLE, 15545 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 4310 },
	{ 0x9286, 0x9286, PDF_CMAP_SINGLE, 10312 },
	{ 0x9287, 0x9287, PDF_CMAP_SINGLE, 10317 },
	{ 0x9288, 0x9288, PDF_CMAP_SINGLE, 10309 },
	{ 0x9289, 0x9289, PDF_CMAP_SINGLE, 15029 },
	{ 0x928a, 0x928a, PDF_CMAP_SINGLE, 10311 },
	{ 0x928b, 0x928b, PDF_CMAP_SINGLE, 10320 },
	{ 0x928c, 0x928c, PDF_CMAP_SINGLE, 10313 },
	{ 0x928d, 0x928d, PDF_CMAP_SINGLE, 10295 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 10304 },
	{ 0x928f, 0x928f, PDF_CMAP_SINGLE, 18109 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 4318 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 4314 },
	{ 0x9294, 0x9294, PDF_CMAP_SINGLE, 10293 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 10306 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 4312 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 10300 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 4311 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 10314 },
	{ 0x929a, 0x929a, PDF_CMAP_SINGLE, 10297 },
	{ 0x929b, 0x929b, PDF_CMAP_SINGLE, 10290 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 4315 },
	{ 0x929d, 0x929d, PDF_CMAP_SINGLE, 10319 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 10292 },
	{ 0x92a1, 0x92a1, PDF_CMAP_SINGLE, 10310 },
	{ 0x92a2, 0x92a2, PDF_CMAP_SINGLE, 10307 },
	{ 0x92a3, 0x92a3, PDF_CMAP_SINGLE, 10302 },
	{ 0x92a4, 0x92a4, PDF_CMAP_SINGLE, 10288 },
	{ 0x92a5, 0x92a5, PDF_CMAP_SINGLE, 10287 },
	{ 0x92a6, 0x92a6, PDF_CMAP_SINGLE, 10296 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 10315 },
	{ 0x92a8, 0x92a8, PDF_CMAP_SINGLE, 4316 },
	{ 0x92a9, 0x92a9, PDF_CMAP_SINGLE, 10318 },
	{ 0x92aa, 0x92aa, PDF_CMAP_SINGLE, 10294 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 10298 },
	{ 0x92ac, 0x92ac, PDF_CMAP_SINGLE, 4308 },
	{ 0x92ae, 0x92ae, PDF_CMAP_SINGLE, 18329 },
	{ 0x92b1, 0x92b1, PDF_CMAP_SINGLE, 18328 },
	{ 0x92b2, 0x92b2, PDF_CMAP_SINGLE, 4703 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 4698 },
	{ 0x92b4, 0x92b4, PDF_CMAP_SINGLE, 10928 },
	{ 0x92b5, 0x92b5, PDF_CMAP_SINGLE, 10924 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 10900 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 4694 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 14057 },
	{ 0x92ba, 0x92ba, PDF_CMAP_SINGLE, 15349 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 4693 },
	{ 0x92bc, 0x92bc, PDF_CMAP_SINGLE, 4699 },
	{ 0x92be, 0x92be, PDF_CMAP_SINGLE, 15456 },
	{ 0x92bf, 0x92bf, PDF_CMAP_SINGLE, 18330 },
	{ 0x92c0, 0x92c0, PDF_CMAP_SINGLE, 10898 },
	{ 0x92c1, 0x92c1, PDF_CMAP_SINGLE, 4697 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 10910 },
	{ 0x92c3, 0x92c4, PDF_CMAP_RANGE, 10896 },
	{ 0x92c5, 0x92c5, PDF_CMAP_SINGLE, 4692 },
	{ 0x92c6, 0x92c6, PDF_CMAP_SINGLE, 10927 },
	{ 0x92c7, 0x92c7, PDF_CMAP_SINGLE, 4701 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 10913 },
	{ 0x92c9, 0x92c9, PDF_CMAP_SINGLE, 10918 },
	{ 0x92ca, 0x92ca, PDF_CMAP_SINGLE, 10912 },
	{ 0x92cb, 0x92cb, PDF_CMAP_SINGLE, 11475 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 10908 },
	{ 0x92cd, 0x92cd, PDF_CMAP_SINGLE, 10916 },
	{ 0x92ce, 0x92ce, PDF_CMAP_SINGLE, 10914 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 10901 },
	{ 0x92d0, 0x92d0, PDF_CMAP_SINGLE, 10895 },
	{ 0x92d1, 0x92d1, PDF_CMAP_SINGLE, 10922 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 4700 },
	{ 0x92d3, 0x92d3, PDF_CMAP_SINGLE, 10923 },
	{ 0x92d4, 0x92d4, PDF_CMAP_SINGLE, 14433 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 10917 },
	{ 0x92d7, 0x92d7, PDF_CMAP_SINGLE, 10906 },
	{ 0x92d8, 0x92d8, PDF_CMAP_SINGLE, 10904 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 10899 },
	{ 0x92db, 0x92db, PDF_CMAP_SINGLE, 16396 },
	{ 0x92dd, 0x92dd, PDF_CMAP_SINGLE, 10907 },
	{ 0x92de, 0x92de, PDF_CMAP_SINGLE, 10920 },
	{ 0x92df, 0x92df, PDF_CMAP_SINGLE, 10903 },
	{ 0x92e0, 0x92e0, PDF_CMAP_SINGLE, 10919 },
	{ 0x92e1, 0x92e1, PDF_CMAP_SINGLE, 10925 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 18331 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 4696 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 15522 },
	{ 0x92e6, 0x92e6, PDF_CMAP_SINGLE, 10915 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 10921 },
	{ 0x92e8, 0x92e8, PDF_CMAP_SINGLE, 10911 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 10905 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 4695 },
	{ 0x92eb, 0x92eb, PDF_CMAP_SINGLE, 18332 },
	{ 0x92ec, 0x92ec, PDF_CMAP_SINGLE, 14692 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 10303 },
	{ 0x92ef, 0x92ef, PDF_CMAP_SINGLE, 10909 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 4702 },
	{ 0x92f1, 0x92f1, PDF_CMAP_SINGLE, 10902 },
	{ 0x92f2, 0x92f2, PDF_CMAP_SINGLE, 15026 },
	{ 0x92f3, 0x92f4, PDF_CMAP_RANGE, 18333 },
	{ 0x92f6, 0x92f6, PDF_CMAP_SINGLE, 15364 },
	{ 0x92f7, 0x92f7, PDF_CMAP_SINGLE, 11480 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 4983 },
	{ 0x92f9, 0x92f9, PDF_CMAP_SINGLE, 11479 },
	{ 0x92fa, 0x92fa, PDF_CMAP_SINGLE, 11477 },
	{ 0x92fb, 0x92fb, PDF_CMAP_SINGLE, 11495 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 4987 },
	{ 0x92fd, 0x92fd, PDF_CMAP_SINGLE, 18335 },
	{ 0x92fe, 0x92fe, PDF_CMAP_SINGLE, 11492 },
	{ 0x92ff, 0x92ff, PDF_CMAP_SINGLE, 11484 },
	{ 0x9300, 0x9300, PDF_CMAP_SINGLE, 11494 },
	{ 0x9301, 0x9301, PDF_CMAP_SINGLE, 11470 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 11482 },
	{ 0x9303, 0x9303, PDF_CMAP_SINGLE, 14439 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 4989 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 11462 },
	{ 0x9307, 0x9307, PDF_CMAP_SINGLE, 15121 },
	{ 0x9308, 0x9308, PDF_CMAP_SINGLE, 11460 },
	{ 0x9309, 0x9309, PDF_CMAP_SINGLE, 11493 },
	{ 0x930b, 0x930b, PDF_CMAP_SINGLE, 11491 },
	{ 0x930c, 0x930c, PDF_CMAP_SINGLE, 11490 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 11474 },
	{ 0x930e, 0x930e, PDF_CMAP_SINGLE, 11473 },
	{ 0x930f, 0x930f, PDF_CMAP_SINGLE, 11463 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 4991 },
	{ 0x9312, 0x9312, PDF_CMAP_SINGLE, 11469 },
	{ 0x9313, 0x9313, PDF_CMAP_SINGLE, 11478 },
	{ 0x9314, 0x9314, PDF_CMAP_SINGLE, 11489 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 4994 },
	{ 0x9316, 0x9316, PDF_CMAP_SINGLE, 11496 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 5283 },
	{ 0x9319, 0x9319, PDF_CMAP_SINGLE, 4996 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 4990 },
	{ 0x931b, 0x931b, PDF_CMAP_SINGLE, 11467 },
	{ 0x931d, 0x931d, PDF_CMAP_SINGLE, 11476 },
	{ 0x931e, 0x931e, PDF_CMAP_SINGLE, 11459 },
	{ 0x931f, 0x931f, PDF_CMAP_SINGLE, 11461 },
	{ 0x9320, 0x9320, PDF_CMAP_SINGLE, 4981 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 4993 },
	{ 0x9322, 0x9322, PDF_CMAP_SINGLE, 4986 },
	{ 0x9323, 0x9323, PDF_CMAP_SINGLE, 11468 },
	{ 0x9324, 0x9324, PDF_CMAP_SINGLE, 11483 },
	{ 0x9325, 0x9325, PDF_CMAP_SINGLE, 10926 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 4992 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 11458 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 5278 },
	{ 0x9329, 0x9329, PDF_CMAP_SINGLE, 11485 },
	{ 0x932a, 0x932a, PDF_CMAP_SINGLE, 11488 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 4988 },
	{ 0x932c, 0x932c, PDF_CMAP_SINGLE, 14482 },
	{ 0x932d, 0x932d, PDF_CMAP_SINGLE, 11472 },
	{ 0x932e, 0x932e, PDF_CMAP_SINGLE, 4995 },
	{ 0x932f, 0x932f, PDF_CMAP_SINGLE, 4985 },
	{ 0x9330, 0x9330, PDF_CMAP_SINGLE, 15875 },
	{ 0x9331, 0x9331, PDF_CMAP_SINGLE, 15549 },
	{ 0x9333, 0x9333, PDF_CMAP_SINGLE, 4984 },
	{ 0x9334, 0x9334, PDF_CMAP_SINGLE, 11481 },
	{ 0x9335, 0x9335, PDF_CMAP_SINGLE, 11487 },
	{ 0x9336, 0x9336, PDF_CMAP_SINGLE, 4982 },
	{ 0x9338, 0x9338, PDF_CMAP_SINGLE, 11465 },
	{ 0x9339, 0x9339, PDF_CMAP_SINGLE, 11486 },
	{ 0x933c, 0x933c, PDF_CMAP_SINGLE, 11466 },
	{ 0x9340, 0x9340, PDF_CMAP_SINGLE, 16725 },
	{ 0x9341, 0x9341, PDF_CMAP_SINGLE, 17015 },
	{ 0x9342, 0x9342, PDF_CMAP_SINGLE, 15098 },
	{ 0x9343, 0x9343, PDF_CMAP_SINGLE, 18336 },
	{ 0x9344, 0x9344, PDF_CMAP_SINGLE, 16406 },
	{ 0x9345, 0x9345, PDF_CMAP_SINGLE, 15988 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 11471 },
	{ 0x9347, 0x9347, PDF_CMAP_SINGLE, 11917 },
	{ 0x9348, 0x9348, PDF_CMAP_SINGLE, 15457 },
	{ 0x9349, 0x9349, PDF_CMAP_SINGLE, 11922 },
	{ 0x934a, 0x934a, PDF_CMAP_SINGLE, 5280 },
	{ 0x934b, 0x934b, PDF_CMAP_SINGLE, 5282 },
	{ 0x934c, 0x934c, PDF_CMAP_SINGLE, 11928 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 5276 },
	{ 0x934e, 0x934e, PDF_CMAP_SINGLE, 11942 },
	{ 0x934f, 0x934f, PDF_CMAP_SINGLE, 11934 },
	{ 0x9350, 0x9351, PDF_CMAP_RANGE, 11923 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 11933 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 5289 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 11932 },
	{ 0x9356, 0x9356, PDF_CMAP_SINGLE, 11916 },
	{ 0x9357, 0x9357, PDF_CMAP_SINGLE, 11931 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 11919 },
	{ 0x9359, 0x9359, PDF_CMAP_SINGLE, 11943 },
	{ 0x935a, 0x935a, PDF_CMAP_SINGLE, 5288 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 5286 },
	{ 0x935c, 0x935c, PDF_CMAP_SINGLE, 11920 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 11939 },
	{ 0x935f, 0x935f, PDF_CMAP_SINGLE, 15547 },
	{ 0x9360, 0x9360, PDF_CMAP_SINGLE, 11925 },
	{ 0x9361, 0x9361, PDF_CMAP_SINGLE, 11938 },
	{ 0x9362, 0x9362, PDF_CMAP_SINGLE, 17000 },
	{ 0x9363, 0x9363, PDF_CMAP_SINGLE, 11940 },
	{ 0x9364, 0x9364, PDF_CMAP_SINGLE, 11915 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 5281 },
	{ 0x9366, 0x9366, PDF_CMAP_SINGLE, 18108 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 11941 },
	{ 0x9368, 0x9368, PDF_CMAP_SINGLE, 15344 },
	{ 0x9369, 0x9369, PDF_CMAP_SINGLE, 14689 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 11929 },
	{ 0x936b, 0x936b, PDF_CMAP_SINGLE, 14483 },
	{ 0x936c, 0x936c, PDF_CMAP_SINGLE, 5285 },
	{ 0x936d, 0x936d, PDF_CMAP_SINGLE, 11926 },
	{ 0x936e, 0x936e, PDF_CMAP_SINGLE, 16300 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 5287 },
	{ 0x9371, 0x9371, PDF_CMAP_SINGLE, 11935 },
	{ 0x9373, 0x9373, PDF_CMAP_SINGLE, 14494 },
	{ 0x9374, 0x9374, PDF_CMAP_SINGLE, 15588 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 5279 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 11921 },
	{ 0x9377, 0x9377, PDF_CMAP_SINGLE, 11936 },
	{ 0x9378, 0x9378, PDF_CMAP_SINGLE, 14585 },
	{ 0x9379, 0x9379, PDF_CMAP_SINGLE, 11930 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 11464 },
	{ 0x937b, 0x937b, PDF_CMAP_SINGLE, 11937 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 11918 },
	{ 0x937d, 0x937d, PDF_CMAP_SINGLE, 15397 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 5284 },
	{ 0x9380, 0x9380, PDF_CMAP_SINGLE, 12308 },
	{ 0x9381, 0x9381, PDF_CMAP_SINGLE, 14693 },
	{ 0x9382, 0x9382, PDF_CMAP_SINGLE, 5277 },
	{ 0x9383, 0x9383, PDF_CMAP_SINGLE, 11913 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 18337 },
	{ 0x9385, 0x9385, PDF_CMAP_SINGLE, 17085 },
	{ 0x9386, 0x9386, PDF_CMAP_SINGLE, 15552 },
	{ 0x9387, 0x9387, PDF_CMAP_SINGLE, 15072 },
	{ 0x9388, 0x9388, PDF_CMAP_SINGLE, 12305 },
	{ 0x9389, 0x9389, PDF_CMAP_SINGLE, 12298 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 5440 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 12293 },
	{ 0x938d, 0x938d, PDF_CMAP_SINGLE, 12309 },
	{ 0x938e, 0x938e, PDF_CMAP_SINGLE, 12300 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 11927 },
	{ 0x9390, 0x9390, PDF_CMAP_SINGLE, 15676 },
	{ 0x9391, 0x9391, PDF_CMAP_SINGLE, 12311 },
	{ 0x9392, 0x9392, PDF_CMAP_SINGLE, 12294 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 5439 },
	{ 0x9395, 0x9395, PDF_CMAP_SINGLE, 12304 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 5441 },
	{ 0x9397, 0x9397, PDF_CMAP_SINGLE, 5449 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 5447 },
	{ 0x9399, 0x9399, PDF_CMAP_SINGLE, 12306 },
	{ 0x939a, 0x939a, PDF_CMAP_SINGLE, 5448 },
	{ 0x939b, 0x939b, PDF_CMAP_SINGLE, 12296 },
	{ 0x939c, 0x939c, PDF_CMAP_SINGLE, 14764 },
	{ 0x939d, 0x939d, PDF_CMAP_SINGLE, 12297 },
	{ 0x939e, 0x939e, PDF_CMAP_SINGLE, 12302 },
	{ 0x939f, 0x939f, PDF_CMAP_SINGLE, 12307 },
	{ 0x93a0, 0x93a0, PDF_CMAP_SINGLE, 15025 },
	{ 0x93a1, 0x93a1, PDF_CMAP_SINGLE, 11912 },
	{ 0x93a2, 0x93a2, PDF_CMAP_SINGLE, 5442 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 12316 },
	{ 0x93a4, 0x93a4, PDF_CMAP_SINGLE, 12313 },
	{ 0x93a5, 0x93a5, PDF_CMAP_SINGLE, 12640 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 12303 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 12299 },
	{ 0x93a8, 0x93a8, PDF_CMAP_SINGLE, 12314 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 12629 },
	{ 0x93aa, 0x93aa, PDF_CMAP_SINGLE, 12301 },
	{ 0x93ac, 0x93ac, PDF_CMAP_SINGLE, 5445 },
	{ 0x93ad, 0x93ad, PDF_CMAP_SINGLE, 18338 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 5444 },
	{ 0x93af, 0x93af, PDF_CMAP_SINGLE, 11914 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 5446 },
	{ 0x93b1, 0x93b1, PDF_CMAP_SINGLE, 12310 },
	{ 0x93b2, 0x93b2, PDF_CMAP_SINGLE, 12312 },
	{ 0x93b3, 0x93b3, PDF_CMAP_SINGLE, 5443 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 12315 },
	{ 0x93b5, 0x93b5, PDF_CMAP_SINGLE, 12292 },
	{ 0x93b7, 0x93b7, PDF_CMAP_SINGLE, 12295 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 15504 },
	{ 0x93ba, 0x93ba, PDF_CMAP_SINGLE, 17399 },
	{ 0x93bb, 0x93bb, PDF_CMAP_SINGLE, 15503 },
	{ 0x93bd, 0x93bd, PDF_CMAP_SINGLE, 14887 },
	{ 0x93bf, 0x93bf, PDF_CMAP_SINGLE, 15917 },
	{ 0x93c0, 0x93c0, PDF_CMAP_SINGLE, 12638 },
	{ 0x93c2, 0x93c2, PDF_CMAP_SINGLE, 12622 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 5600 },
	{ 0x93c4, 0x93c4, PDF_CMAP_SINGLE, 12636 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 18106 },
	{ 0x93c7, 0x93c7, PDF_CMAP_SINGLE, 12620 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 5601 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 12631 },
	{ 0x93cb, 0x93cb, PDF_CMAP_SINGLE, 14694 },
	{ 0x93cc, 0x93cc, PDF_CMAP_SINGLE, 12627 },
	{ 0x93cd, 0x93cd, PDF_CMAP_SINGLE, 5606 },
	{ 0x93ce, 0x93ce, PDF_CMAP_SINGLE, 12637 },
	{ 0x93cf, 0x93cf, PDF_CMAP_SINGLE, 12621 },
	{ 0x93d0, 0x93d0, PDF_CMAP_SINGLE, 12624 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 5598 },
	{ 0x93d2, 0x93d2, PDF_CMAP_SINGLE, 12639 },
	{ 0x93d3, 0x93d3, PDF_CMAP_SINGLE, 16609 },
	{ 0x93d4, 0x93d4, PDF_CMAP_SINGLE, 12632 },
	{ 0x93d5, 0x93d5, PDF_CMAP_SINGLE, 12635 },
	{ 0x93d6, 0x93d6, PDF_CMAP_SINGLE, 5604 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 5609 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 5607 },
	{ 0x93d9, 0x93d9, PDF_CMAP_SINGLE, 12628 },
	{ 0x93da, 0x93da, PDF_CMAP_SINGLE, 12623 },
	{ 0x93db, 0x93db, PDF_CMAP_SINGLE, 15455 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 5602 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 12619 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 5599 },
	{ 0x93e0, 0x93e0, PDF_CMAP_SINGLE, 14880 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 5597 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 5605 },
	{ 0x93e3, 0x93e3, PDF_CMAP_SINGLE, 12634 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 5608 },
	{ 0x93e6, 0x93e6, PDF_CMAP_SINGLE, 12630 },
	{ 0x93e7, 0x93e7, PDF_CMAP_SINGLE, 12641 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 5610 },
	{ 0x93ec, 0x93ec, PDF_CMAP_SINGLE, 12626 },
	{ 0x93ee, 0x93ee, PDF_CMAP_SINGLE, 12633 },
	{ 0x93f0, 0x93f0, PDF_CMAP_SINGLE, 16959 },
	{ 0x93f1, 0x93f1, PDF_CMAP_SINGLE, 17066 },
	{ 0x93f3, 0x93f3, PDF_CMAP_SINGLE, 14690 },
	{ 0x93f4, 0x93f4, PDF_CMAP_SINGLE, 18890 },
	{ 0x93f5, 0x93f5, PDF_CMAP_SINGLE, 12890 },
	{ 0x93f6, 0x93f6, PDF_CMAP_SINGLE, 12905 },
	{ 0x93f7, 0x93f7, PDF_CMAP_SINGLE, 12892 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 12899 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 12625 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 12897 },
	{ 0x93fb, 0x93fb, PDF_CMAP_SINGLE, 12880 },
	{ 0x93fc, 0x93fc, PDF_CMAP_SINGLE, 12903 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 5716 },
	{ 0x93fe, 0x93fe, PDF_CMAP_SINGLE, 12884 },
	{ 0x93ff, 0x93ff, PDF_CMAP_SINGLE, 12902 },
	{ 0x9400, 0x9400, PDF_CMAP_SINGLE, 12891 },
	{ 0x9401, 0x9401, PDF_CMAP_SINGLE, 15572 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 5715 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 15305 },
	{ 0x9406, 0x9406, PDF_CMAP_SINGLE, 12907 },
	{ 0x9407, 0x9407, PDF_CMAP_SINGLE, 12893 },
	{ 0x9408, 0x9408, PDF_CMAP_SINGLE, 14885 },
	{ 0x9409, 0x9409, PDF_CMAP_SINGLE, 12898 },
	{ 0x940a, 0x940a, PDF_CMAP_SINGLE, 12901 },
	{ 0x940b, 0x940b, PDF_CMAP_SINGLE, 12878 },
	{ 0x940c, 0x940c, PDF_CMAP_SINGLE, 12904 },
	{ 0x940d, 0x940d, PDF_CMAP_SINGLE, 12889 },
	{ 0x940e, 0x940e, PDF_CMAP_SINGLE, 12894 },
	{ 0x940f, 0x940f, PDF_CMAP_SINGLE, 12882 },
	{ 0x9410, 0x9410, PDF_CMAP_SINGLE, 12886 },
	{ 0x9411, 0x9411, PDF_CMAP_SINGLE, 12906 },
	{ 0x9412, 0x9412, PDF_CMAP_SINGLE, 12896 },
	{ 0x9413, 0x9413, PDF_CMAP_SINGLE, 12879 },
	{ 0x9414, 0x9414, PDF_CMAP_SINGLE, 12883 },
	{ 0x9415, 0x9415, PDF_CMAP_SINGLE, 12885 },
	{ 0x9416, 0x9416, PDF_CMAP_SINGLE, 12895 },
	{ 0x9417, 0x9417, PDF_CMAP_SINGLE, 18341 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 5714 },
	{ 0x9419, 0x9419, PDF_CMAP_SINGLE, 12888 },
	{ 0x941b, 0x941b, PDF_CMAP_SINGLE, 16712 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 18342 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 12881 },
	{ 0x9424, 0x9424, PDF_CMAP_SINGLE, 14659 },
	{ 0x9425, 0x9425, PDF_CMAP_SINGLE, 14544 },
	{ 0x9426, 0x9426, PDF_CMAP_SINGLE, 14499 },
	{ 0x9427, 0x9427, PDF_CMAP_SINGLE, 15309 },
	{ 0x9428, 0x9428, PDF_CMAP_SINGLE, 12887 },
	{ 0x9429, 0x9429, PDF_CMAP_SINGLE, 13093 },
	{ 0x942a, 0x942a, PDF_CMAP_SINGLE, 13097 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 5791 },
	{ 0x942c, 0x942c, PDF_CMAP_SINGLE, 13099 },
	{ 0x942d, 0x942d, PDF_CMAP_SINGLE, 18343 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 5785 },
	{ 0x942f, 0x942f, PDF_CMAP_SINGLE, 19105 },
	{ 0x9430, 0x9430, PDF_CMAP_SINGLE, 13095 },
	{ 0x9431, 0x9431, PDF_CMAP_SINGLE, 13101 },
	{ 0x9432, 0x9432, PDF_CMAP_SINGLE, 5790 },
	{ 0x9433, 0x9433, PDF_CMAP_SINGLE, 5786 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 5787 },
	{ 0x9436, 0x9436, PDF_CMAP_SINGLE, 13092 },
	{ 0x9437, 0x9437, PDF_CMAP_SINGLE, 13098 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 5789 },
	{ 0x9439, 0x9439, PDF_CMAP_SINGLE, 13096 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 5788 },
	{ 0x943b, 0x943b, PDF_CMAP_SINGLE, 13091 },
	{ 0x943c, 0x943c, PDF_CMAP_SINGLE, 12900 },
	{ 0x943d, 0x943d, PDF_CMAP_SINGLE, 13094 },
	{ 0x943e, 0x943e, PDF_CMAP_SINGLE, 18344 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 13090 },
	{ 0x9440, 0x9440, PDF_CMAP_SINGLE, 13100 },
	{ 0x9442, 0x9442, PDF_CMAP_SINGLE, 17010 },
	{ 0x9443, 0x9443, PDF_CMAP_SINGLE, 17013 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 5855 },
	{ 0x9445, 0x9445, PDF_CMAP_SINGLE, 13261 },
	{ 0x9446, 0x9446, PDF_CMAP_SINGLE, 13264 },
	{ 0x9447, 0x9447, PDF_CMAP_SINGLE, 13260 },
	{ 0x9448, 0x9449, PDF_CMAP_RANGE, 13262 },
	{ 0x944a, 0x944b, PDF_CMAP_RANGE, 13257 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 13255 },
	{ 0x944d, 0x944d, PDF_CMAP_SINGLE, 15461 },
	{ 0x944f, 0x944f, PDF_CMAP_SINGLE, 13259 },
	{ 0x9450, 0x9450, PDF_CMAP_SINGLE, 13256 },
	{ 0x9451, 0x9452, PDF_CMAP_RANGE, 5856 },
	{ 0x9454, 0x9454, PDF_CMAP_SINGLE, 18346 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 13384 },
	{ 0x9457, 0x9457, PDF_CMAP_SINGLE, 13386 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 15408 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 14495 },
	{ 0x945d, 0x945d, PDF_CMAP_SINGLE, 13385 },
	{ 0x945e, 0x945e, PDF_CMAP_SINGLE, 13387 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 5902 },
	{ 0x9462, 0x9462, PDF_CMAP_SINGLE, 13383 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 5901 },
	{ 0x9464, 0x9464, PDF_CMAP_SINGLE, 5903 },
	{ 0x9465, 0x9465, PDF_CMAP_SINGLE, 15838 },
	{ 0x9467, 0x9467, PDF_CMAP_SINGLE, 15032 },
	{ 0x9468, 0x9469, PDF_CMAP_RANGE, 13478 },
	{ 0x946a, 0x946a, PDF_CMAP_SINGLE, 5937 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 13477 },
	{ 0x946c, 0x946c, PDF_CMAP_SINGLE, 14206 },
	{ 0x946d, 0x946d, PDF_CMAP_SINGLE, 13549 },
	{ 0x946e, 0x946e, PDF_CMAP_SINGLE, 13548 },
	{ 0x946f, 0x946f, PDF_CMAP_SINGLE, 13550 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 5964 },
	{ 0x9471, 0x9471, PDF_CMAP_SINGLE, 13551 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 5963 },
	{ 0x9473, 0x9473, PDF_CMAP_SINGLE, 13552 },
	{ 0x9474, 0x9474, PDF_CMAP_SINGLE, 13580 },
	{ 0x9475, 0x9475, PDF_CMAP_SINGLE, 13583 },
	{ 0x9476, 0x9476, PDF_CMAP_SINGLE, 13582 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 5973 },
	{ 0x9478, 0x9478, PDF_CMAP_SINGLE, 13581 },
	{ 0x9479, 0x9479, PDF_CMAP_SINGLE, 18347 },
	{ 0x947b, 0x947b, PDF_CMAP_SINGLE, 16795 },
	{ 0x947c, 0x947c, PDF_CMAP_SINGLE, 5983 },
	{ 0x947d, 0x947e, PDF_CMAP_RANGE, 5981 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 5988 },
	{ 0x9480, 0x9481, PDF_CMAP_RANGE, 13625 },
	{ 0x9482, 0x9482, PDF_CMAP_SINGLE, 13624 },
	{ 0x9483, 0x9483, PDF_CMAP_SINGLE, 13632 },
	{ 0x9485, 0x9485, PDF_CMAP_SINGLE, 17671 },
	{ 0x949f, 0x949f, PDF_CMAP_SINGLE, 18565 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 17691 },
	{ 0x94c1, 0x94c1, PDF_CMAP_SINGLE, 18401 },
	{ 0x94c3, 0x94c3, PDF_CMAP_SINGLE, 18399 },
	{ 0x94dc, 0x94dc, PDF_CMAP_SINGLE, 18391 },
	{ 0x94f6, 0x94f6, PDF_CMAP_SINGLE, 18386 },
	{ 0x952d, 0x952d, PDF_CMAP_SINGLE, 18348 },
	{ 0x9547, 0x9547, PDF_CMAP_SINGLE, 18623 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 1687 },
	{ 0x9578, 0x9578, PDF_CMAP_SINGLE, 17672 },
	{ 0x957a, 0x957a, PDF_CMAP_SINGLE, 8353 },
	{ 0x957b, 0x957b, PDF_CMAP_SINGLE, 9034 },
	{ 0x957c, 0x957c, PDF_CMAP_SINGLE, 10929 },
	{ 0x957d, 0x957d, PDF_CMAP_SINGLE, 12642 },
	{ 0x957f, 0x957f, PDF_CMAP_SINGLE, 17673 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 1688 },
	{ 0x9582, 0x9582, PDF_CMAP_SINGLE, 2081 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 2538 },
	{ 0x9585, 0x9585, PDF_CMAP_SINGLE, 14497 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 8354 },
	{ 0x9588, 0x9588, PDF_CMAP_SINGLE, 8355 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 3024 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 3482 },
	{ 0x958c, 0x958c, PDF_CMAP_SINGLE, 9036 },
	{ 0x958d, 0x958d, PDF_CMAP_SINGLE, 9035 },
	{ 0x958e, 0x958e, PDF_CMAP_SINGLE, 3486 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 3481 },
	{ 0x9590, 0x9590, PDF_CMAP_SINGLE, 9037 },
	{ 0x9591, 0x9591, PDF_CMAP_SINGLE, 3483 },
	{ 0x9592, 0x9592, PDF_CMAP_SINGLE, 3485 },
	{ 0x9593, 0x9593, PDF_CMAP_SINGLE, 3484 },
	{ 0x9594, 0x9594, PDF_CMAP_SINGLE, 3480 },
	{ 0x9596, 0x9596, PDF_CMAP_SINGLE, 15317 },
	{ 0x9597, 0x9597, PDF_CMAP_SINGLE, 15316 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 3932 },
	{ 0x9599, 0x9599, PDF_CMAP_SINGLE, 15314 },
	{ 0x959b, 0x959b, PDF_CMAP_SINGLE, 9715 },
	{ 0x959c, 0x959c, PDF_CMAP_SINGLE, 9713 },
	{ 0x959e, 0x959e, PDF_CMAP_SINGLE, 9714 },
	{ 0x959f, 0x959f, PDF_CMAP_SINGLE, 9712 },
	{ 0x95a0, 0x95a0, PDF_CMAP_SINGLE, 14500 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 4319 },
	{ 0x95a2, 0x95a2, PDF_CMAP_SINGLE, 18349 },
	{ 0x95a3, 0x95a3, PDF_CMAP_SINGLE, 4322 },
	{ 0x95a4, 0x95a4, PDF_CMAP_SINGLE, 4324 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 4323 },
	{ 0x95a6, 0x95a6, PDF_CMAP_SINGLE, 14498 },
	{ 0x95a7, 0x95a7, PDF_CMAP_SINGLE, 15315 },
	{ 0x95a8, 0x95a9, PDF_CMAP_RANGE, 4320 },
	{ 0x95aa, 0x95aa, PDF_CMAP_SINGLE, 17301 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 10931 },
	{ 0x95ac, 0x95ac, PDF_CMAP_SINGLE, 10930 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 4704 },
	{ 0x95ae, 0x95ae, PDF_CMAP_SINGLE, 10932 },
	{ 0x95b0, 0x95b0, PDF_CMAP_SINGLE, 10933 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 4705 },
	{ 0x95b5, 0x95b5, PDF_CMAP_SINGLE, 11504 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 11502 },
	{ 0x95b7, 0x95b7, PDF_CMAP_SINGLE, 11949 },
	{ 0x95b9, 0x95ba, PDF_CMAP_RANGE, 11500 },
	{ 0x95bb, 0x95bb, PDF_CMAP_SINGLE, 4997 },
	{ 0x95bc, 0x95bc, PDF_CMAP_SINGLE, 11497 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 11505 },
	{ 0x95be, 0x95be, PDF_CMAP_SINGLE, 11499 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 11503 },
	{ 0x95c0, 0x95c0, PDF_CMAP_SINGLE, 11945 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 11947 },
	{ 0x95c5, 0x95c5, PDF_CMAP_SINGLE, 11948 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 5294 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 11944 },
	{ 0x95c8, 0x95c8, PDF_CMAP_SINGLE, 5293 },
	{ 0x95c9, 0x95c9, PDF_CMAP_SINGLE, 11946 },
	{ 0x95ca, 0x95cc, PDF_CMAP_RANGE, 5290 },
	{ 0x95cd, 0x95cd, PDF_CMAP_SINGLE, 11498 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 5452 },
	{ 0x95d1, 0x95d1, PDF_CMAP_SINGLE, 12319 },
	{ 0x95d2, 0x95d3, PDF_CMAP_RANGE, 12317 },
	{ 0x95d4, 0x95d4, PDF_CMAP_SINGLE, 5450 },
	{ 0x95d5, 0x95d5, PDF_CMAP_SINGLE, 5453 },
	{ 0x95d6, 0x95d6, PDF_CMAP_SINGLE, 5451 },
	{ 0x95da, 0x95db, PDF_CMAP_RANGE, 12643 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 5611 },
	{ 0x95de, 0x95de, PDF_CMAP_SINGLE, 12908 },
	{ 0x95df, 0x95df, PDF_CMAP_SINGLE, 12910 },
	{ 0x95e0, 0x95e0, PDF_CMAP_SINGLE, 12909 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 5717 },
	{ 0x95e2, 0x95e2, PDF_CMAP_SINGLE, 5792 },
	{ 0x95e3, 0x95e3, PDF_CMAP_SINGLE, 13104 },
	{ 0x95e4, 0x95e4, PDF_CMAP_SINGLE, 13103 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 13102 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 17674 },
	{ 0x95f4, 0x95f4, PDF_CMAP_SINGLE, 18351 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 1689 },
	{ 0x961d, 0x961d, PDF_CMAP_SINGLE, 17689 },
	{ 0x961e, 0x961e, PDF_CMAP_SINGLE, 6068 },
	{ 0x9620, 0x9620, PDF_CMAP_SINGLE, 6177 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 1044 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 6175 },
	{ 0x9623, 0x9623, PDF_CMAP_SINGLE, 6178 },
	{ 0x9624, 0x9624, PDF_CMAP_SINGLE, 6176 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 6385 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 1318 },
	{ 0x962c, 0x962c, PDF_CMAP_SINGLE, 1319 },
	{ 0x962d, 0x962d, PDF_CMAP_SINGLE, 6387 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 1316 },
	{ 0x962f, 0x962f, PDF_CMAP_SINGLE, 6386 },
	{ 0x9630, 0x9630, PDF_CMAP_SINGLE, 6384 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 1317 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 1315 },
	{ 0x9633, 0x9633, PDF_CMAP_SINGLE, 18352 },
	{ 0x9638, 0x9638, PDF_CMAP_SINGLE, 16337 },
	{ 0x9639, 0x9639, PDF_CMAP_SINGLE, 6741 },
	{ 0x963a, 0x963a, PDF_CMAP_SINGLE, 6744 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 1692 },
	{ 0x963c, 0x963c, PDF_CMAP_SINGLE, 6743 },
	{ 0x963d, 0x963d, PDF_CMAP_SINGLE, 6742 },
	{ 0x963f, 0x963f, PDF_CMAP_SINGLE, 1691 },
	{ 0x9640, 0x9640, PDF_CMAP_SINGLE, 1690 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 16189 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 1694 },
	{ 0x9643, 0x9643, PDF_CMAP_SINGLE, 6745 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 1693 },
	{ 0x9645, 0x9645, PDF_CMAP_SINGLE, 18290 },
	{ 0x964a, 0x964a, PDF_CMAP_SINGLE, 7194 },
	{ 0x964b, 0x964d, PDF_CMAP_RANGE, 2083 },
	{ 0x964e, 0x964e, PDF_CMAP_SINGLE, 7195 },
	{ 0x964f, 0x964f, PDF_CMAP_SINGLE, 7191 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 2082 },
	{ 0x9651, 0x9651, PDF_CMAP_SINGLE, 7192 },
	{ 0x9653, 0x9653, PDF_CMAP_SINGLE, 7193 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 7190 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 15847 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 2545 },
	{ 0x965b, 0x965b, PDF_CMAP_SINGLE, 2542 },
	{ 0x965c, 0x965c, PDF_CMAP_SINGLE, 7727 },
	{ 0x965d, 0x965d, PDF_CMAP_SINGLE, 2543 },
	{ 0x965e, 0x965e, PDF_CMAP_SINGLE, 2546 },
	{ 0x965f, 0x965f, PDF_CMAP_SINGLE, 7728 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 2541 },
	{ 0x9662, 0x9663, PDF_CMAP_RANGE, 2539 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 2544 },
	{ 0x9669, 0x9669, PDF_CMAP_SINGLE, 18270 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 3025 },
	{ 0x966b, 0x966b, PDF_CMAP_SINGLE, 8358 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 3033 },
	{ 0x966d, 0x966d, PDF_CMAP_SINGLE, 8357 },
	{ 0x966f, 0x966f, PDF_CMAP_SINGLE, 8360 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 3029 },
	{ 0x9671, 0x9671, PDF_CMAP_SINGLE, 8359 },
	{ 0x9672, 0x9672, PDF_CMAP_SINGLE, 3494 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 3027 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 3030 },
	{ 0x9675, 0x9675, PDF_CMAP_SINGLE, 3026 },
	{ 0x9676, 0x9677, PDF_CMAP_RANGE, 3031 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 3028 },
	{ 0x967b, 0x967b, PDF_CMAP_SINGLE, 14508 },
	{ 0x967c, 0x967c, PDF_CMAP_SINGLE, 8356 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 3490 },
	{ 0x967e, 0x967e, PDF_CMAP_SINGLE, 9039 },
	{ 0x9680, 0x9680, PDF_CMAP_SINGLE, 9043 },
	{ 0x9681, 0x9681, PDF_CMAP_SINGLE, 16198 },
	{ 0x9683, 0x9683, PDF_CMAP_SINGLE, 9042 },
	{ 0x9684, 0x9684, PDF_CMAP_SINGLE, 3495 },
	{ 0x9685, 0x9686, PDF_CMAP_RANGE, 3491 },
	{ 0x9687, 0x9687, PDF_CMAP_SINGLE, 9038 },
	{ 0x9688, 0x9689, PDF_CMAP_RANGE, 9040 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 3487 },
	{ 0x968b, 0x968b, PDF_CMAP_SINGLE, 3489 },
	{ 0x968d, 0x968d, PDF_CMAP_SINGLE, 3493 },
	{ 0x968e, 0x968e, PDF_CMAP_SINGLE, 3488 },
	{ 0x968f, 0x968f, PDF_CMAP_SINGLE, 15326 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 9718 },
	{ 0x9692, 0x9693, PDF_CMAP_RANGE, 9716 },
	{ 0x9694, 0x9695, PDF_CMAP_RANGE, 3934 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 14509 },
	{ 0x9697, 0x9697, PDF_CMAP_SINGLE, 9719 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 3933 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 4325 },
	{ 0x969b, 0x969b, PDF_CMAP_SINGLE, 4327 },
	{ 0x969c, 0x969c, PDF_CMAP_SINGLE, 4326 },
	{ 0x969e, 0x969e, PDF_CMAP_SINGLE, 10322 },
	{ 0x96a1, 0x96a1, PDF_CMAP_SINGLE, 10323 },
	{ 0x96a2, 0x96a2, PDF_CMAP_SINGLE, 10935 },
	{ 0x96a3, 0x96a3, PDF_CMAP_SINGLE, 14511 },
	{ 0x96a4, 0x96a4, PDF_CMAP_SINGLE, 10934 },
	{ 0x96a5, 0x96a5, PDF_CMAP_SINGLE, 17135 },
	{ 0x96a7, 0x96a8, PDF_CMAP_RANGE, 4998 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 11506 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 5000 },
	{ 0x96ac, 0x96ac, PDF_CMAP_SINGLE, 11952 },
	{ 0x96ae, 0x96ae, PDF_CMAP_SINGLE, 11950 },
	{ 0x96b0, 0x96b0, PDF_CMAP_SINGLE, 11951 },
	{ 0x96b1, 0x96b1, PDF_CMAP_SINGLE, 5295 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 12320 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 5612 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 561 },
	{ 0x96b8, 0x96b8, PDF_CMAP_SINGLE, 5296 },
	{ 0x96b9, 0x96b9, PDF_CMAP_SINGLE, 1695 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 2547 },
	{ 0x96bc, 0x96bc, PDF_CMAP_SINGLE, 7729 },
	{ 0x96bd, 0x96bd, PDF_CMAP_SINGLE, 14517 },
	{ 0x96bf, 0x96bf, PDF_CMAP_SINGLE, 8361 },
	{ 0x96c0, 0x96c0, PDF_CMAP_SINGLE, 3034 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 3496 },
	{ 0x96c2, 0x96c2, PDF_CMAP_SINGLE, 9044 },
	{ 0x96c3, 0x96c3, PDF_CMAP_SINGLE, 9046 },
	{ 0x96c4, 0x96c4, PDF_CMAP_SINGLE, 3498 },
	{ 0x96c5, 0x96c5, PDF_CMAP_SINGLE, 3497 },
	{ 0x96c6, 0x96c7, PDF_CMAP_RANGE, 3499 },
	{ 0x96c8, 0x96c8, PDF_CMAP_SINGLE, 9045 },
	{ 0x96c9, 0x96ca, PDF_CMAP_RANGE, 3938 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 3937 },
	{ 0x96cc, 0x96cc, PDF_CMAP_SINGLE, 4328 },
	{ 0x96cd, 0x96cd, PDF_CMAP_SINGLE, 3936 },
	{ 0x96ce, 0x96ce, PDF_CMAP_SINGLE, 9720 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 4329 },
	{ 0x96d3, 0x96d3, PDF_CMAP_SINGLE, 10936 },
	{ 0x96d4, 0x96d4, PDF_CMAP_SINGLE, 11507 },
	{ 0x96d5, 0x96d5, PDF_CMAP_SINGLE, 5001 },
	{ 0x96d6, 0x96d6, PDF_CMAP_SINGLE, 5297 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 12321 },
	{ 0x96d8, 0x96d8, PDF_CMAP_SINGLE, 12325 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 5456 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 12322 },
	{ 0x96db, 0x96db, PDF_CMAP_SINGLE, 5457 },
	{ 0x96dc, 0x96dc, PDF_CMAP_SINGLE, 5455 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 12326 },
	{ 0x96de, 0x96de, PDF_CMAP_SINGLE, 5458 },
	{ 0x96df, 0x96df, PDF_CMAP_SINGLE, 12324 },
	{ 0x96e1, 0x96e1, PDF_CMAP_SINGLE, 12645 },
	{ 0x96e2, 0x96e2, PDF_CMAP_SINGLE, 5454 },
	{ 0x96e3, 0x96e3, PDF_CMAP_SINGLE, 5613 },
	{ 0x96e5, 0x96e5, PDF_CMAP_SINGLE, 13480 },
	{ 0x96e8, 0x96e8, PDF_CMAP_SINGLE, 1696 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 3036 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 3035 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 3501 },
	{ 0x96f0, 0x96f0, PDF_CMAP_SINGLE, 9048 },
	{ 0x96f1, 0x96f1, PDF_CMAP_SINGLE, 9047 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 3502 },
	{ 0x96f4, 0x96f4, PDF_CMAP_SINGLE, 14852 },
	{ 0x96f5, 0x96f5, PDF_CMAP_SINGLE, 9724 },
	{ 0x96f6, 0x96f6, PDF_CMAP_SINGLE, 3943 },
	{ 0x96f7, 0x96f7, PDF_CMAP_SINGLE, 3940 },
	{ 0x96f8, 0x96f8, PDF_CMAP_SINGLE, 9723 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 3942 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 9721 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 3941 },
	{ 0x96fd, 0x96fd, PDF_CMAP_SINGLE, 9722 },
	{ 0x96ff, 0x96ff, PDF_CMAP_SINGLE, 10324 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 4330 },
	{ 0x9702, 0x9702, PDF_CMAP_SINGLE, 10939 },
	{ 0x9703, 0x9703, PDF_CMAP_SINGLE, 16141 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 4706 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 10937 },
	{ 0x9706, 0x9707, PDF_CMAP_RANGE, 4707 },
	{ 0x9708, 0x9708, PDF_CMAP_SINGLE, 10938 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 4709 },
	{ 0x970b, 0x970b, PDF_CMAP_SINGLE, 11508 },
	{ 0x970d, 0x970d, PDF_CMAP_SINGLE, 5005 },
	{ 0x970e, 0x970e, PDF_CMAP_SINGLE, 5002 },
	{ 0x970f, 0x970f, PDF_CMAP_SINGLE, 5007 },
	{ 0x9710, 0x9710, PDF_CMAP_SINGLE, 11510 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 5003 },
	{ 0x9712, 0x9712, PDF_CMAP_SINGLE, 11509 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 5006 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 5004 },
	{ 0x9718, 0x9718, PDF_CMAP_SINGLE, 11955 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 11957 },
	{ 0x971b, 0x971b, PDF_CMAP_SINGLE, 14537 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 5298 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 11956 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 5299 },
	{ 0x971f, 0x971f, PDF_CMAP_SINGLE, 11954 },
	{ 0x9720, 0x9720, PDF_CMAP_SINGLE, 11953 },
	{ 0x9721, 0x9721, PDF_CMAP_SINGLE, 16307 },
	{ 0x9722, 0x9722, PDF_CMAP_SINGLE, 12328 },
	{ 0x9723, 0x9723, PDF_CMAP_SINGLE, 12327 },
	{ 0x9724, 0x9724, PDF_CMAP_SINGLE, 5459 },
	{ 0x9725, 0x9725, PDF_CMAP_SINGLE, 12329 },
	{ 0x9726, 0x9726, PDF_CMAP_SINGLE, 12650 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 5615 },
	{ 0x9728, 0x9728, PDF_CMAP_SINGLE, 12649 },
	{ 0x9729, 0x9729, PDF_CMAP_SINGLE, 12646 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 5614 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 12647 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 12911 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 5718 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 14528 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 5795 },
	{ 0x9734, 0x9734, PDF_CMAP_SINGLE, 19007 },
	{ 0x9735, 0x9735, PDF_CMAP_SINGLE, 13105 },
	{ 0x9736, 0x9736, PDF_CMAP_SINGLE, 14530 },
	{ 0x9738, 0x9739, PDF_CMAP_RANGE, 5793 },
	{ 0x973a, 0x973a, PDF_CMAP_SINGLE, 13106 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 5858 },
	{ 0x973f, 0x973f, PDF_CMAP_SINGLE, 13265 },
	{ 0x9740, 0x9740, PDF_CMAP_SINGLE, 18355 },
	{ 0x9741, 0x9741, PDF_CMAP_SINGLE, 15337 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 5938 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 13482 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 5940 },
	{ 0x9746, 0x9746, PDF_CMAP_SINGLE, 13481 },
	{ 0x9747, 0x9747, PDF_CMAP_SINGLE, 13483 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 5939 },
	{ 0x9749, 0x9749, PDF_CMAP_SINGLE, 13553 },
	{ 0x974a, 0x974a, PDF_CMAP_SINGLE, 19109 },
	{ 0x974b, 0x974b, PDF_CMAP_SINGLE, 13606 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 17635 },
	{ 0x9752, 0x9752, PDF_CMAP_SINGLE, 1697 },
	{ 0x9755, 0x9755, PDF_CMAP_SINGLE, 18886 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 3944 },
	{ 0x9757, 0x9757, PDF_CMAP_SINGLE, 14540 },
	{ 0x9758, 0x9758, PDF_CMAP_SINGLE, 10325 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 10940 },
	{ 0x975b, 0x975c, PDF_CMAP_RANGE, 5008 },
	{ 0x975d, 0x975d, PDF_CMAP_SINGLE, 15919 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 1698 },
	{ 0x975f, 0x975f, PDF_CMAP_SINGLE, 14543 },
	{ 0x9760, 0x9760, PDF_CMAP_SINGLE, 4710 },
	{ 0x9761, 0x9761, PDF_CMAP_SINGLE, 5616 },
	{ 0x9762, 0x9762, PDF_CMAP_SINGLE, 2086 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 5010 },
	{ 0x9768, 0x9768, PDF_CMAP_SINGLE, 5904 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 2087 },
	{ 0x976a, 0x976a, PDF_CMAP_SINGLE, 8362 },
	{ 0x976c, 0x976c, PDF_CMAP_SINGLE, 9049 },
	{ 0x976d, 0x976d, PDF_CMAP_SINGLE, 16023 },
	{ 0x976e, 0x976e, PDF_CMAP_SINGLE, 9051 },
	{ 0x9770, 0x9770, PDF_CMAP_SINGLE, 9050 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 18123 },
	{ 0x9772, 0x9772, PDF_CMAP_SINGLE, 9728 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 9725 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 3945 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 3946 },
	{ 0x9777, 0x9778, PDF_CMAP_RANGE, 9726 },
	{ 0x977a, 0x977a, PDF_CMAP_SINGLE, 10327 },
	{ 0x977b, 0x977b, PDF_CMAP_SINGLE, 10332 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 4331 },
	{ 0x977d, 0x977d, PDF_CMAP_SINGLE, 10326 },
	{ 0x977e, 0x977e, PDF_CMAP_SINGLE, 10328 },
	{ 0x977f, 0x977f, PDF_CMAP_SINGLE, 10335 },
	{ 0x9780, 0x9780, PDF_CMAP_SINGLE, 10330 },
	{ 0x9781, 0x9781, PDF_CMAP_SINGLE, 10334 },
	{ 0x9782, 0x9782, PDF_CMAP_SINGLE, 10331 },
	{ 0x9783, 0x9783, PDF_CMAP_SINGLE, 10329 },
	{ 0x9784, 0x9784, PDF_CMAP_SINGLE, 10333 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 4332 },
	{ 0x9787, 0x9787, PDF_CMAP_SINGLE, 16576 },
	{ 0x9788, 0x9788, PDF_CMAP_SINGLE, 10943 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 14548 },
	{ 0x978a, 0x978a, PDF_CMAP_SINGLE, 10941 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 4712 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 4711 },
	{ 0x978e, 0x978e, PDF_CMAP_SINGLE, 10942 },
	{ 0x978f, 0x978f, PDF_CMAP_SINGLE, 4713 },
	{ 0x9794, 0x9794, PDF_CMAP_SINGLE, 11513 },
	{ 0x9797, 0x9797, PDF_CMAP_SINGLE, 11512 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 5011 },
	{ 0x9799, 0x9799, PDF_CMAP_SINGLE, 11511 },
	{ 0x979a, 0x979a, PDF_CMAP_SINGLE, 11958 },
	{ 0x979b, 0x979b, PDF_CMAP_SINGLE, 15355 },
	{ 0x979c, 0x979c, PDF_CMAP_SINGLE, 11960 },
	{ 0x979d, 0x979d, PDF_CMAP_SINGLE, 11962 },
	{ 0x979e, 0x979e, PDF_CMAP_SINGLE, 11961 },
	{ 0x979f, 0x979f, PDF_CMAP_SINGLE, 14549 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 5300 },
	{ 0x97a1, 0x97a1, PDF_CMAP_SINGLE, 11959 },
	{ 0x97a2, 0x97a2, PDF_CMAP_SINGLE, 12336 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 5460 },
	{ 0x97a4, 0x97a4, PDF_CMAP_SINGLE, 12334 },
	{ 0x97a5, 0x97a5, PDF_CMAP_SINGLE, 12337 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 5461 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 12332 },
	{ 0x97aa, 0x97aa, PDF_CMAP_SINGLE, 12335 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 12333 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 12330 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 5462 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 12331 },
	{ 0x97b1, 0x97b1, PDF_CMAP_SINGLE, 14550 },
	{ 0x97b2, 0x97b2, PDF_CMAP_SINGLE, 18357 },
	{ 0x97b3, 0x97b3, PDF_CMAP_SINGLE, 12651 },
	{ 0x97b4, 0x97b4, PDF_CMAP_SINGLE, 16179 },
	{ 0x97b6, 0x97b6, PDF_CMAP_SINGLE, 12653 },
	{ 0x97b7, 0x97b7, PDF_CMAP_SINGLE, 12652 },
	{ 0x97b8, 0x97b8, PDF_CMAP_SINGLE, 15765 },
	{ 0x97b9, 0x97b9, PDF_CMAP_SINGLE, 12913 },
	{ 0x97ba, 0x97ba, PDF_CMAP_SINGLE, 16181 },
	{ 0x97bb, 0x97bb, PDF_CMAP_SINGLE, 12914 },
	{ 0x97bd, 0x97bd, PDF_CMAP_SINGLE, 18766 },
	{ 0x97be, 0x97be, PDF_CMAP_SINGLE, 14551 },
	{ 0x97bf, 0x97bf, PDF_CMAP_SINGLE, 13107 },
	{ 0x97c0, 0x97c0, PDF_CMAP_SINGLE, 14552 },
	{ 0x97c1, 0x97c1, PDF_CMAP_SINGLE, 5861 },
	{ 0x97c2, 0x97c2, PDF_CMAP_SINGLE, 18358 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 5860 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 13388 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 5941 },
	{ 0x97c7, 0x97c7, PDF_CMAP_SINGLE, 13484 },
	{ 0x97c8, 0x97c8, PDF_CMAP_SINGLE, 16175 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 5974 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 2088 },
	{ 0x97cc, 0x97cc, PDF_CMAP_SINGLE, 3503 },
	{ 0x97cd, 0x97cd, PDF_CMAP_SINGLE, 10337 },
	{ 0x97ce, 0x97ce, PDF_CMAP_SINGLE, 10336 },
	{ 0x97cf, 0x97cf, PDF_CMAP_SINGLE, 10945 },
	{ 0x97d0, 0x97d0, PDF_CMAP_SINGLE, 10944 },
	{ 0x97d2, 0x97d2, PDF_CMAP_SINGLE, 14553 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 5301 },
	{ 0x97d4, 0x97d4, PDF_CMAP_SINGLE, 11964 },
	{ 0x97d5, 0x97d5, PDF_CMAP_SINGLE, 11963 },
	{ 0x97d6, 0x97d6, PDF_CMAP_SINGLE, 12340 },
	{ 0x97d7, 0x97d7, PDF_CMAP_SINGLE, 12338 },
	{ 0x97d8, 0x97d8, PDF_CMAP_SINGLE, 12341 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 12339 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 5617 },
	{ 0x97dd, 0x97df, PDF_CMAP_RANGE, 12654 },
	{ 0x97e0, 0x97e0, PDF_CMAP_SINGLE, 14554 },
	{ 0x97e1, 0x97e1, PDF_CMAP_SINGLE, 13108 },
	{ 0x97e3, 0x97e3, PDF_CMAP_SINGLE, 13266 },
	{ 0x97e5, 0x97e5, PDF_CMAP_SINGLE, 13485 },
	{ 0x97e6, 0x97e6, PDF_CMAP_SINGLE, 17676 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 2089 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 14556 },
	{ 0x97f0, 0x97f0, PDF_CMAP_SINGLE, 11514 },
	{ 0x97f1, 0x97f1, PDF_CMAP_SINGLE, 11965 },
	{ 0x97f2, 0x97f2, PDF_CMAP_SINGLE, 14669 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 2090 },
	{ 0x97f5, 0x97f5, PDF_CMAP_SINGLE, 14560 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 4333 },
	{ 0x97f8, 0x97f8, PDF_CMAP_SINGLE, 11515 },
	{ 0x97f9, 0x97f9, PDF_CMAP_SINGLE, 5463 },
	{ 0x97fa, 0x97fa, PDF_CMAP_SINGLE, 12342 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 5618 },
	{ 0x97fd, 0x97fe, PDF_CMAP_RANGE, 12915 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 5796 },
	{ 0x9800, 0x9800, PDF_CMAP_SINGLE, 13390 },
	{ 0x9801, 0x9801, PDF_CMAP_SINGLE, 2091 },
	{ 0x9802, 0x9803, PDF_CMAP_RANGE, 3039 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 8363 },
	{ 0x9805, 0x9806, PDF_CMAP_RANGE, 3504 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 9052 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 3506 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 3950 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 3952 },
	{ 0x980d, 0x980e, PDF_CMAP_RANGE, 9730 },
	{ 0x980f, 0x980f, PDF_CMAP_SINGLE, 9729 },
	{ 0x9810, 0x9811, PDF_CMAP_RANGE, 3947 },
	{ 0x9812, 0x9812, PDF_CMAP_SINGLE, 3951 },
	{ 0x9813, 0x9813, PDF_CMAP_SINGLE, 3949 },
	{ 0x9814, 0x9814, PDF_CMAP_SINGLE, 18770 },
	{ 0x9815, 0x9815, PDF_CMAP_SINGLE, 17212 },
	{ 0x9816, 0x9816, PDF_CMAP_SINGLE, 10338 },
	{ 0x9817, 0x9818, PDF_CMAP_RANGE, 4334 },
	{ 0x981b, 0x981b, PDF_CMAP_SINGLE, 10952 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 4716 },
	{ 0x981d, 0x981d, PDF_CMAP_SINGLE, 10947 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 10946 },
	{ 0x981f, 0x981f, PDF_CMAP_SINGLE, 15362 },
	{ 0x9820, 0x9820, PDF_CMAP_SINGLE, 10951 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 4714 },
	{ 0x9823, 0x9823, PDF_CMAP_SINGLE, 15927 },
	{ 0x9824, 0x9824, PDF_CMAP_SINGLE, 5018 },
	{ 0x9826, 0x9826, PDF_CMAP_SINGLE, 10948 },
	{ 0x9827, 0x9827, PDF_CMAP_SINGLE, 10953 },
	{ 0x9828, 0x9828, PDF_CMAP_SINGLE, 10950 },
	{ 0x9829, 0x9829, PDF_CMAP_SINGLE, 10949 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 4715 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 5016 },
	{ 0x982e, 0x982e, PDF_CMAP_SINGLE, 16242 },
	{ 0x982f, 0x982f, PDF_CMAP_SINGLE, 11517 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 5012 },
	{ 0x9832, 0x9832, PDF_CMAP_SINGLE, 11518 },
	{ 0x9833, 0x9833, PDF_CMAP_SINGLE, 14565 },
	{ 0x9834, 0x9834, PDF_CMAP_SINGLE, 14564 },
	{ 0x9835, 0x9835, PDF_CMAP_SINGLE, 11516 },
	{ 0x9837, 0x9837, PDF_CMAP_SINGLE, 5015 },
	{ 0x9838, 0x9838, PDF_CMAP_SINGLE, 5013 },
	{ 0x9839, 0x9839, PDF_CMAP_SINGLE, 5017 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 5014 },
	{ 0x9841, 0x9841, PDF_CMAP_SINGLE, 11966 },
	{ 0x9843, 0x9843, PDF_CMAP_SINGLE, 11971 },
	{ 0x9844, 0x9844, PDF_CMAP_SINGLE, 11967 },
	{ 0x9845, 0x9845, PDF_CMAP_SINGLE, 11970 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 5302 },
	{ 0x9847, 0x9847, PDF_CMAP_SINGLE, 15766 },
	{ 0x9848, 0x9848, PDF_CMAP_SINGLE, 11762 },
	{ 0x9849, 0x9849, PDF_CMAP_SINGLE, 11969 },
	{ 0x984a, 0x984a, PDF_CMAP_SINGLE, 11968 },
	{ 0x984b, 0x984b, PDF_CMAP_SINGLE, 14566 },
	{ 0x984c, 0x984c, PDF_CMAP_SINGLE, 5466 },
	{ 0x984d, 0x984d, PDF_CMAP_SINGLE, 5464 },
	{ 0x984e, 0x984e, PDF_CMAP_SINGLE, 5467 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 5465 },
	{ 0x9850, 0x9852, PDF_CMAP_RANGE, 12343 },
	{ 0x9853, 0x9853, PDF_CMAP_SINGLE, 5468 },
	{ 0x9856, 0x9856, PDF_CMAP_SINGLE, 19054 },
	{ 0x9857, 0x9857, PDF_CMAP_SINGLE, 12660 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 5620 },
	{ 0x9859, 0x9859, PDF_CMAP_SINGLE, 12658 },
	{ 0x985b, 0x985b, PDF_CMAP_SINGLE, 5621 },
	{ 0x985c, 0x985c, PDF_CMAP_SINGLE, 12657 },
	{ 0x985d, 0x985d, PDF_CMAP_SINGLE, 12659 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 5619 },
	{ 0x985f, 0x985f, PDF_CMAP_SINGLE, 12920 },
	{ 0x9860, 0x9860, PDF_CMAP_SINGLE, 12917 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 12918 },
	{ 0x9864, 0x9864, PDF_CMAP_SINGLE, 13109 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 5798 },
	{ 0x9866, 0x9866, PDF_CMAP_SINGLE, 14567 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 5797 },
	{ 0x9868, 0x9868, PDF_CMAP_SINGLE, 18947 },
	{ 0x9869, 0x9869, PDF_CMAP_SINGLE, 13268 },
	{ 0x986a, 0x986a, PDF_CMAP_SINGLE, 13267 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 5862 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 16437 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 5905 },
	{ 0x9870, 0x9870, PDF_CMAP_SINGLE, 5942 },
	{ 0x9871, 0x9871, PDF_CMAP_SINGLE, 5965 },
	{ 0x9872, 0x9872, PDF_CMAP_SINGLE, 13554 },
	{ 0x9873, 0x9874, PDF_CMAP_RANGE, 13607 },
	{ 0x9875, 0x9875, PDF_CMAP_SINGLE, 17677 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 2092 },
	{ 0x98a9, 0x98a9, PDF_CMAP_SINGLE, 9053 },
	{ 0x98ac, 0x98ac, PDF_CMAP_SINGLE, 9732 },
	{ 0x98ad, 0x98ae, PDF_CMAP_RANGE, 10339 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 4336 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 4337 },
	{ 0x98b2, 0x98b2, PDF_CMAP_SINGLE, 10954 },
	{ 0x98b3, 0x98b3, PDF_CMAP_SINGLE, 4717 },
	{ 0x98b4, 0x98b4, PDF_CMAP_SINGLE, 16243 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 5303 },
	{ 0x98b7, 0x98b7, PDF_CMAP_SINGLE, 14575 },
	{ 0x98b8, 0x98b8, PDF_CMAP_SINGLE, 12346 },
	{ 0x98b9, 0x98b9, PDF_CMAP_SINGLE, 18360 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 5469 },
	{ 0x98bb, 0x98bb, PDF_CMAP_SINGLE, 12663 },
	{ 0x98bc, 0x98bc, PDF_CMAP_SINGLE, 5622 },
	{ 0x98bd, 0x98bd, PDF_CMAP_SINGLE, 12662 },
	{ 0x98be, 0x98be, PDF_CMAP_SINGLE, 12664 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 12661 },
	{ 0x98c0, 0x98c0, PDF_CMAP_SINGLE, 13112 },
	{ 0x98c1, 0x98c2, PDF_CMAP_RANGE, 12921 },
	{ 0x98c3, 0x98c3, PDF_CMAP_SINGLE, 15224 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 5719 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 13111 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 14577 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 14576 },
	{ 0x98c9, 0x98c9, PDF_CMAP_SINGLE, 13110 },
	{ 0x98ca, 0x98ca, PDF_CMAP_SINGLE, 14574 },
	{ 0x98cb, 0x98cb, PDF_CMAP_SINGLE, 13269 },
	{ 0x98cc, 0x98cc, PDF_CMAP_SINGLE, 13609 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 17678 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 2093 },
	{ 0x98dc, 0x98dc, PDF_CMAP_SINGLE, 15371 },
	{ 0x98de, 0x98de, PDF_CMAP_SINGLE, 17679 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 2094 },
	{ 0x98e0, 0x98e0, PDF_CMAP_SINGLE, 18322 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 14582 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 2548 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 7730 },
	{ 0x98e5, 0x98e5, PDF_CMAP_SINGLE, 8364 },
	{ 0x98e6, 0x98e6, PDF_CMAP_SINGLE, 14583 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 3507 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 3510 },
	{ 0x98ea, 0x98ea, PDF_CMAP_SINGLE, 3508 },
	{ 0x98eb, 0x98eb, PDF_CMAP_SINGLE, 9054 },
	{ 0x98ec, 0x98ec, PDF_CMAP_SINGLE, 14584 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 3512 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 3509 },
	{ 0x98f1, 0x98f1, PDF_CMAP_SINGLE, 18362 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 3511 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 3954 },
	{ 0x98f5, 0x98f5, PDF_CMAP_SINGLE, 17253 },
	{ 0x98f6, 0x98f6, PDF_CMAP_SINGLE, 9733 },
	{ 0x98f9, 0x98f9, PDF_CMAP_SINGLE, 9734 },
	{ 0x98fa, 0x98fa, PDF_CMAP_SINGLE, 10956 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 3953 },
	{ 0x98fd, 0x98fe, PDF_CMAP_RANGE, 3955 },
	{ 0x9900, 0x9900, PDF_CMAP_SINGLE, 10342 },
	{ 0x9902, 0x9902, PDF_CMAP_SINGLE, 10341 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 4338 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 4339 },
	{ 0x9907, 0x9907, PDF_CMAP_SINGLE, 10343 },
	{ 0x9908, 0x9908, PDF_CMAP_SINGLE, 10955 },
	{ 0x9909, 0x9909, PDF_CMAP_SINGLE, 4341 },
	{ 0x990a, 0x990a, PDF_CMAP_SINGLE, 4718 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 4340 },
	{ 0x990e, 0x990e, PDF_CMAP_SINGLE, 17389 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 5019 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 10957 },
	{ 0x9912, 0x9912, PDF_CMAP_SINGLE, 4720 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 4719 },
	{ 0x9914, 0x9914, PDF_CMAP_SINGLE, 10958 },
	{ 0x9915, 0x9915, PDF_CMAP_SINGLE, 10961 },
	{ 0x9916, 0x9917, PDF_CMAP_RANGE, 10959 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 4721 },
	{ 0x9919, 0x9919, PDF_CMAP_SINGLE, 18364 },
	{ 0x991a, 0x991a, PDF_CMAP_SINGLE, 5024 },
	{ 0x991b, 0x991b, PDF_CMAP_SINGLE, 5022 },
	{ 0x991c, 0x991c, PDF_CMAP_SINGLE, 17390 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 5021 },
	{ 0x991f, 0x991f, PDF_CMAP_SINGLE, 11520 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 5023 },
	{ 0x9924, 0x9924, PDF_CMAP_SINGLE, 11519 },
	{ 0x9925, 0x9925, PDF_CMAP_SINGLE, 11972 },
	{ 0x9927, 0x9927, PDF_CMAP_SINGLE, 11521 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 5020 },
	{ 0x9929, 0x9929, PDF_CMAP_SINGLE, 11522 },
	{ 0x992a, 0x992a, PDF_CMAP_SINGLE, 11975 },
	{ 0x992b, 0x992c, PDF_CMAP_RANGE, 11973 },
	{ 0x992d, 0x992d, PDF_CMAP_SINGLE, 11979 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 5473 },
	{ 0x992f, 0x992f, PDF_CMAP_SINGLE, 11978 },
	{ 0x9930, 0x9930, PDF_CMAP_SINGLE, 11981 },
	{ 0x9931, 0x9931, PDF_CMAP_SINGLE, 11980 },
	{ 0x9932, 0x9932, PDF_CMAP_SINGLE, 11977 },
	{ 0x9933, 0x9933, PDF_CMAP_SINGLE, 11976 },
	{ 0x9935, 0x9935, PDF_CMAP_SINGLE, 5304 },
	{ 0x9937, 0x9937, PDF_CMAP_SINGLE, 18365 },
	{ 0x9938, 0x9938, PDF_CMAP_SINGLE, 15357 },
	{ 0x9939, 0x9939, PDF_CMAP_SINGLE, 14586 },
	{ 0x993a, 0x993a, PDF_CMAP_SINGLE, 12349 },
	{ 0x993b, 0x993b, PDF_CMAP_SINGLE, 15377 },
	{ 0x993c, 0x993c, PDF_CMAP_SINGLE, 12348 },
	{ 0x993d, 0x993d, PDF_CMAP_SINGLE, 5472 },
	{ 0x993e, 0x993f, PDF_CMAP_RANGE, 5470 },
	{ 0x9940, 0x9940, PDF_CMAP_SINGLE, 16180 },
	{ 0x9941, 0x9941, PDF_CMAP_SINGLE, 12347 },
	{ 0x9942, 0x9942, PDF_CMAP_SINGLE, 17407 },
	{ 0x9943, 0x9943, PDF_CMAP_SINGLE, 12667 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 5623 },
	{ 0x9947, 0x9947, PDF_CMAP_SINGLE, 12666 },
	{ 0x9948, 0x9948, PDF_CMAP_SINGLE, 12665 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 5624 },
	{ 0x994a, 0x994a, PDF_CMAP_SINGLE, 16313 },
	{ 0x994b, 0x994b, PDF_CMAP_SINGLE, 12927 },
	{ 0x994c, 0x994c, PDF_CMAP_SINGLE, 12926 },
	{ 0x994d, 0x994d, PDF_CMAP_SINGLE, 15378 },
	{ 0x994e, 0x994e, PDF_CMAP_SINGLE, 12924 },
	{ 0x9950, 0x9950, PDF_CMAP_SINGLE, 12923 },
	{ 0x9951, 0x9951, PDF_CMAP_SINGLE, 5721 },
	{ 0x9952, 0x9952, PDF_CMAP_SINGLE, 5720 },
	{ 0x9953, 0x9953, PDF_CMAP_SINGLE, 12928 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 13270 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 5863 },
	{ 0x9956, 0x9956, PDF_CMAP_SINGLE, 13114 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 5799 },
	{ 0x9958, 0x9958, PDF_CMAP_SINGLE, 13113 },
	{ 0x9959, 0x9959, PDF_CMAP_SINGLE, 12925 },
	{ 0x995b, 0x995b, PDF_CMAP_SINGLE, 13271 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 5906 },
	{ 0x995d, 0x995d, PDF_CMAP_SINGLE, 18366 },
	{ 0x995e, 0x995e, PDF_CMAP_SINGLE, 5966 },
	{ 0x995f, 0x995f, PDF_CMAP_SINGLE, 13555 },
	{ 0x9961, 0x9961, PDF_CMAP_SINGLE, 13610 },
	{ 0x9962, 0x9962, PDF_CMAP_SINGLE, 18367 },
	{ 0x9963, 0x9963, PDF_CMAP_SINGLE, 17680 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 2095 },
	{ 0x9997, 0x9997, PDF_CMAP_SINGLE, 8365 },
	{ 0x9998, 0x9998, PDF_CMAP_SINGLE, 11982 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 2096 },
	{ 0x999b, 0x999b, PDF_CMAP_SINGLE, 18124 },
	{ 0x999c, 0x999c, PDF_CMAP_SINGLE, 10345 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 10344 },
	{ 0x999e, 0x999e, PDF_CMAP_SINGLE, 11523 },
	{ 0x99a1, 0x99a1, PDF_CMAP_SINGLE, 11984 },
	{ 0x99a3, 0x99a3, PDF_CMAP_SINGLE, 11983 },
	{ 0x99a4, 0x99a4, PDF_CMAP_SINGLE, 16835 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 5474 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 12668 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5722 },
	{ 0x99aa, 0x99aa, PDF_CMAP_SINGLE, 15383 },
	{ 0x99ab, 0x99ab, PDF_CMAP_SINGLE, 13611 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 2549 },
	{ 0x99ad, 0x99ad, PDF_CMAP_SINGLE, 3514 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 3513 },
	{ 0x99af, 0x99af, PDF_CMAP_SINGLE, 9735 },
	{ 0x99b0, 0x99b0, PDF_CMAP_SINGLE, 9737 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 3958 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 9736 },
	{ 0x99b3, 0x99b3, PDF_CMAP_SINGLE, 3957 },
	{ 0x99b4, 0x99b4, PDF_CMAP_SINGLE, 3959 },
	{ 0x99b5, 0x99b5, PDF_CMAP_SINGLE, 9738 },
	{ 0x99b8, 0x99b8, PDF_CMAP_SINGLE, 14667 },
	{ 0x99b9, 0x99b9, PDF_CMAP_SINGLE, 10347 },
	{ 0x99ba, 0x99ba, PDF_CMAP_SINGLE, 10349 },
	{ 0x99bb, 0x99bb, PDF_CMAP_SINGLE, 10348 },
	{ 0x99bc, 0x99bc, PDF_CMAP_SINGLE, 15394 },
	{ 0x99bd, 0x99bd, PDF_CMAP_SINGLE, 10351 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 4342 },
	{ 0x99c2, 0x99c2, PDF_CMAP_SINGLE, 10350 },
	{ 0x99c3, 0x99c3, PDF_CMAP_SINGLE, 10346 },
	{ 0x99c4, 0x99c4, PDF_CMAP_SINGLE, 15735 },
	{ 0x99c5, 0x99c5, PDF_CMAP_SINGLE, 18369 },
	{ 0x99c7, 0x99c7, PDF_CMAP_SINGLE, 10352 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 10968 },
	{ 0x99cb, 0x99cb, PDF_CMAP_SINGLE, 10971 },
	{ 0x99cc, 0x99cc, PDF_CMAP_SINGLE, 10973 },
	{ 0x99cd, 0x99cd, PDF_CMAP_SINGLE, 10963 },
	{ 0x99ce, 0x99ce, PDF_CMAP_SINGLE, 10967 },
	{ 0x99cf, 0x99cf, PDF_CMAP_SINGLE, 10964 },
	{ 0x99d0, 0x99d0, PDF_CMAP_SINGLE, 4723 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 4726 },
	{ 0x99d2, 0x99d2, PDF_CMAP_SINGLE, 4728 },
	{ 0x99d3, 0x99d4, PDF_CMAP_RANGE, 10965 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 4727 },
	{ 0x99d6, 0x99d6, PDF_CMAP_SINGLE, 10969 },
	{ 0x99d7, 0x99d7, PDF_CMAP_SINGLE, 10972 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 10970 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 4729 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 16194 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 4725 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 10962 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 4722 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 4724 },
	{ 0x99e0, 0x99e0, PDF_CMAP_SINGLE, 18939 },
	{ 0x99e1, 0x99e1, PDF_CMAP_SINGLE, 14213 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 5026 },
	{ 0x99e3, 0x99e3, PDF_CMAP_SINGLE, 11529 },
	{ 0x99e4, 0x99e4, PDF_CMAP_SINGLE, 11527 },
	{ 0x99e5, 0x99e5, PDF_CMAP_SINGLE, 11526 },
	{ 0x99e6, 0x99e6, PDF_CMAP_SINGLE, 15205 },
	{ 0x99e7, 0x99e7, PDF_CMAP_SINGLE, 11532 },
	{ 0x99e9, 0x99e9, PDF_CMAP_SINGLE, 11531 },
	{ 0x99ea, 0x99ea, PDF_CMAP_SINGLE, 11530 },
	{ 0x99ec, 0x99ec, PDF_CMAP_SINGLE, 11525 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 5025 },
	{ 0x99ee, 0x99ee, PDF_CMAP_SINGLE, 11524 },
	{ 0x99f0, 0x99f0, PDF_CMAP_SINGLE, 11528 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 5027 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 11987 },
	{ 0x99f5, 0x99f5, PDF_CMAP_SINGLE, 14591 },
	{ 0x99f6, 0x99f6, PDF_CMAP_SINGLE, 11991 },
	{ 0x99f7, 0x99f7, PDF_CMAP_SINGLE, 11988 },
	{ 0x99f8, 0x99f8, PDF_CMAP_SINGLE, 11990 },
	{ 0x99f9, 0x99f9, PDF_CMAP_SINGLE, 11989 },
	{ 0x99fa, 0x99fa, PDF_CMAP_SINGLE, 11986 },
	{ 0x99fb, 0x99fb, PDF_CMAP_SINGLE, 11992 },
	{ 0x99fc, 0x99fc, PDF_CMAP_SINGLE, 11995 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 11993 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 5306 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 5305 },
	{ 0x9a02, 0x9a02, PDF_CMAP_SINGLE, 11985 },
	{ 0x9a03, 0x9a03, PDF_CMAP_SINGLE, 11996 },
	{ 0x9a04, 0x9a04, PDF_CMAP_SINGLE, 12354 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 12357 },
	{ 0x9a06, 0x9a06, PDF_CMAP_SINGLE, 12359 },
	{ 0x9a07, 0x9a07, PDF_CMAP_SINGLE, 12358 },
	{ 0x9a09, 0x9a09, PDF_CMAP_SINGLE, 12352 },
	{ 0x9a0a, 0x9a0a, PDF_CMAP_SINGLE, 12356 },
	{ 0x9a0b, 0x9a0b, PDF_CMAP_SINGLE, 12351 },
	{ 0x9a0c, 0x9a0c, PDF_CMAP_SINGLE, 14592 },
	{ 0x9a0d, 0x9a0d, PDF_CMAP_SINGLE, 12353 },
	{ 0x9a0e, 0x9a0e, PDF_CMAP_SINGLE, 5475 },
	{ 0x9a0f, 0x9a0f, PDF_CMAP_SINGLE, 12350 },
	{ 0x9a10, 0x9a10, PDF_CMAP_SINGLE, 14594 },
	{ 0x9a11, 0x9a11, PDF_CMAP_SINGLE, 12355 },
	{ 0x9a14, 0x9a14, PDF_CMAP_SINGLE, 12682 },
	{ 0x9a15, 0x9a15, PDF_CMAP_SINGLE, 12671 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 5625 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 5626 },
	{ 0x9a1a, 0x9a1a, PDF_CMAP_SINGLE, 12670 },
	{ 0x9a1b, 0x9a1b, PDF_CMAP_SINGLE, 12675 },
	{ 0x9a1c, 0x9a1c, PDF_CMAP_SINGLE, 12681 },
	{ 0x9a1d, 0x9a1d, PDF_CMAP_SINGLE, 12673 },
	{ 0x9a1e, 0x9a1e, PDF_CMAP_SINGLE, 12680 },
	{ 0x9a1f, 0x9a1f, PDF_CMAP_SINGLE, 15212 },
	{ 0x9a20, 0x9a20, PDF_CMAP_SINGLE, 12677 },
	{ 0x9a21, 0x9a21, PDF_CMAP_SINGLE, 15388 },
	{ 0x9a22, 0x9a22, PDF_CMAP_SINGLE, 12676 },
	{ 0x9a23, 0x9a23, PDF_CMAP_SINGLE, 12679 },
	{ 0x9a24, 0x9a24, PDF_CMAP_SINGLE, 12674 },
	{ 0x9a25, 0x9a25, PDF_CMAP_SINGLE, 12672 },
	{ 0x9a26, 0x9a26, PDF_CMAP_SINGLE, 15858 },
	{ 0x9a27, 0x9a27, PDF_CMAP_SINGLE, 12678 },
	{ 0x9a29, 0x9a29, PDF_CMAP_SINGLE, 12935 },
	{ 0x9a2a, 0x9a2a, PDF_CMAP_SINGLE, 12933 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 5723 },
	{ 0x9a2c, 0x9a2c, PDF_CMAP_SINGLE, 12932 },
	{ 0x9a2d, 0x9a2d, PDF_CMAP_SINGLE, 12938 },
	{ 0x9a2e, 0x9a2e, PDF_CMAP_SINGLE, 12936 },
	{ 0x9a2f, 0x9a2f, PDF_CMAP_SINGLE, 15390 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 5724 },
	{ 0x9a31, 0x9a31, PDF_CMAP_SINGLE, 12931 },
	{ 0x9a32, 0x9a32, PDF_CMAP_SINGLE, 12929 },
	{ 0x9a34, 0x9a34, PDF_CMAP_SINGLE, 12930 },
	{ 0x9a35, 0x9a35, PDF_CMAP_SINGLE, 5726 },
	{ 0x9a36, 0x9a36, PDF_CMAP_SINGLE, 12934 },
	{ 0x9a37, 0x9a37, PDF_CMAP_SINGLE, 5725 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 12937 },
	{ 0x9a39, 0x9a39, PDF_CMAP_SINGLE, 13115 },
	{ 0x9a3a, 0x9a3a, PDF_CMAP_SINGLE, 13121 },
	{ 0x9a3b, 0x9a3b, PDF_CMAP_SINGLE, 14593 },
	{ 0x9a3c, 0x9a3c, PDF_CMAP_SINGLE, 18371 },
	{ 0x9a3d, 0x9a3d, PDF_CMAP_SINGLE, 13116 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 5803 },
	{ 0x9a3f, 0x9a3f, PDF_CMAP_SINGLE, 13122 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 5802 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 13120 },
	{ 0x9a42, 0x9a42, PDF_CMAP_SINGLE, 13119 },
	{ 0x9a43, 0x9a43, PDF_CMAP_SINGLE, 5801 },
	{ 0x9a44, 0x9a44, PDF_CMAP_SINGLE, 13118 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 5800 },
	{ 0x9a46, 0x9a46, PDF_CMAP_SINGLE, 13117 },
	{ 0x9a48, 0x9a48, PDF_CMAP_SINGLE, 13277 },
	{ 0x9a49, 0x9a49, PDF_CMAP_SINGLE, 13279 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 13278 },
	{ 0x9a4c, 0x9a4c, PDF_CMAP_SINGLE, 13275 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 5865 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 13272 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 13276 },
	{ 0x9a50, 0x9a50, PDF_CMAP_SINGLE, 13281 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 13280 },
	{ 0x9a53, 0x9a54, PDF_CMAP_RANGE, 13273 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 5864 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 13391 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 5909 },
	{ 0x9a58, 0x9a58, PDF_CMAP_SINGLE, 14595 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 13392 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_RANGE, 5907 },
	{ 0x9a5c, 0x9a5c, PDF_CMAP_SINGLE, 15384 },
	{ 0x9a5e, 0x9a5e, PDF_CMAP_SINGLE, 13486 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 5943 },
	{ 0x9a60, 0x9a60, PDF_CMAP_SINGLE, 13584 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 5975 },
	{ 0x9a63, 0x9a63, PDF_CMAP_SINGLE, 15207 },
	{ 0x9a64, 0x9a64, PDF_CMAP_SINGLE, 13612 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 5976 },
	{ 0x9a66, 0x9a67, PDF_CMAP_RANGE, 13613 },
	{ 0x9a68, 0x9a68, PDF_CMAP_SINGLE, 13628 },
	{ 0x9a69, 0x9a69, PDF_CMAP_SINGLE, 13627 },
	{ 0x9a6a, 0x9a6a, PDF_CMAP_SINGLE, 5991 },
	{ 0x9a6b, 0x9a6b, PDF_CMAP_SINGLE, 13636 },
	{ 0x9a6c, 0x9a6c, PDF_CMAP_SINGLE, 17797 },
	{ 0x9a8f, 0x9a8f, PDF_CMAP_SINGLE, 17798 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2550 },
	{ 0x9aab, 0x9aab, PDF_CMAP_SINGLE, 9740 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 9739 },
	{ 0x9aaf, 0x9ab0, PDF_CMAP_RANGE, 4343 },
	{ 0x9ab1, 0x9ab1, PDF_CMAP_SINGLE, 10353 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 17202 },
	{ 0x9ab3, 0x9ab3, PDF_CMAP_SINGLE, 10974 },
	{ 0x9ab4, 0x9ab4, PDF_CMAP_SINGLE, 11535 },
	{ 0x9ab6, 0x9ab6, PDF_CMAP_SINGLE, 17895 },
	{ 0x9ab7, 0x9ab7, PDF_CMAP_SINGLE, 4730 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 5028 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 11533 },
	{ 0x9aba, 0x9aba, PDF_CMAP_SINGLE, 16244 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 11536 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 5029 },
	{ 0x9abd, 0x9abd, PDF_CMAP_SINGLE, 15767 },
	{ 0x9abe, 0x9abe, PDF_CMAP_SINGLE, 11997 },
	{ 0x9abf, 0x9abf, PDF_CMAP_SINGLE, 11534 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 12360 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 5476 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 12683 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 12941 },
	{ 0x9ac7, 0x9ac7, PDF_CMAP_SINGLE, 12939 },
	{ 0x9aca, 0x9aca, PDF_CMAP_SINGLE, 12940 },
	{ 0x9acd, 0x9acd, PDF_CMAP_SINGLE, 13123 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 5804 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 13282 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 5912 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 5866 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 5910 },
	{ 0x9ad5, 0x9ad5, PDF_CMAP_SINGLE, 13487 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 5967 },
	{ 0x9ad7, 0x9ad7, PDF_CMAP_SINGLE, 16298 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 2551 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 12361 },
	{ 0x9adf, 0x9adf, PDF_CMAP_SINGLE, 7731 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 14600 },
	{ 0x9ae1, 0x9ae1, PDF_CMAP_SINGLE, 3960 },
	{ 0x9ae2, 0x9ae2, PDF_CMAP_SINGLE, 14601 },
	{ 0x9ae3, 0x9ae3, PDF_CMAP_SINGLE, 10354 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 4345 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 10355 },
	{ 0x9aeb, 0x9aeb, PDF_CMAP_SINGLE, 10976 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 10975 },
	{ 0x9aed, 0x9aed, PDF_CMAP_SINGLE, 5031 },
	{ 0x9aee, 0x9aef, PDF_CMAP_RANGE, 4731 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 10979 },
	{ 0x9af2, 0x9af2, PDF_CMAP_SINGLE, 10978 },
	{ 0x9af3, 0x9af3, PDF_CMAP_SINGLE, 10977 },
	{ 0x9af4, 0x9af4, PDF_CMAP_SINGLE, 14603 },
	{ 0x9af6, 0x9af6, PDF_CMAP_SINGLE, 11537 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 11540 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 11539 },
	{ 0x9afa, 0x9afa, PDF_CMAP_SINGLE, 11538 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 5030 },
	{ 0x9afc, 0x9afc, PDF_CMAP_SINGLE, 12001 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 11999 },
	{ 0x9afe, 0x9afe, PDF_CMAP_SINGLE, 11998 },
	{ 0x9aff, 0x9aff, PDF_CMAP_SINGLE, 16151 },
	{ 0x9b01, 0x9b01, PDF_CMAP_SINGLE, 12000 },
	{ 0x9b02, 0x9b02, PDF_CMAP_SINGLE, 16146 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 5477 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 12363 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 5478 },
	{ 0x9b08, 0x9b08, PDF_CMAP_SINGLE, 12362 },
	{ 0x9b09, 0x9b09, PDF_CMAP_SINGLE, 16160 },
	{ 0x9b0a, 0x9b0a, PDF_CMAP_SINGLE, 12685 },
	{ 0x9b0b, 0x9b0b, PDF_CMAP_SINGLE, 12684 },
	{ 0x9b0c, 0x9b0c, PDF_CMAP_SINGLE, 12687 },
	{ 0x9b0d, 0x9b0d, PDF_CMAP_SINGLE, 5627 },
	{ 0x9b0e, 0x9b0e, PDF_CMAP_SINGLE, 12686 },
	{ 0x9b0f, 0x9b0f, PDF_CMAP_SINGLE, 18372 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 12942 },
	{ 0x9b11, 0x9b11, PDF_CMAP_SINGLE, 12944 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 12943 },
	{ 0x9b14, 0x9b14, PDF_CMAP_SINGLE, 14605 },
	{ 0x9b15, 0x9b15, PDF_CMAP_SINGLE, 13124 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 13127 },
	{ 0x9b17, 0x9b18, PDF_CMAP_RANGE, 13125 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 13283 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 5867 },
	{ 0x9b1e, 0x9b20, PDF_CMAP_RANGE, 13393 },
	{ 0x9b22, 0x9b22, PDF_CMAP_SINGLE, 5944 },
	{ 0x9b23, 0x9b23, PDF_CMAP_SINGLE, 5968 },
	{ 0x9b24, 0x9b24, PDF_CMAP_SINGLE, 13615 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 2552 },
	{ 0x9b27, 0x9b27, PDF_CMAP_SINGLE, 4733 },
	{ 0x9b28, 0x9b28, PDF_CMAP_SINGLE, 5032 },
	{ 0x9b29, 0x9b29, PDF_CMAP_SINGLE, 12365 },
	{ 0x9b2a, 0x9b2a, PDF_CMAP_SINGLE, 16153 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 13284 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 14606 },
	{ 0x9b2e, 0x9b2e, PDF_CMAP_SINGLE, 13585 },
	{ 0x9b2f, 0x9b2f, PDF_CMAP_SINGLE, 7732 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 5992 },
	{ 0x9b32, 0x9b32, PDF_CMAP_SINGLE, 2553 },
	{ 0x9b33, 0x9b33, PDF_CMAP_SINGLE, 11541 },
	{ 0x9b34, 0x9b34, PDF_CMAP_SINGLE, 14609 },
	{ 0x9b35, 0x9b35, PDF_CMAP_SINGLE, 12366 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 12688 },
	{ 0x9b39, 0x9b39, PDF_CMAP_SINGLE, 16128 },
	{ 0x9b3a, 0x9b3a, PDF_CMAP_SINGLE, 13128 },
	{ 0x9b3b, 0x9b3b, PDF_CMAP_SINGLE, 13285 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 2554 },
	{ 0x9b3e, 0x9b3f, PDF_CMAP_RANGE, 10356 },
	{ 0x9b40, 0x9b40, PDF_CMAP_SINGLE, 14613 },
	{ 0x9b41, 0x9b42, PDF_CMAP_RANGE, 4346 },
	{ 0x9b43, 0x9b43, PDF_CMAP_SINGLE, 10981 },
	{ 0x9b44, 0x9b44, PDF_CMAP_SINGLE, 4735 },
	{ 0x9b45, 0x9b45, PDF_CMAP_SINGLE, 4734 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 10980 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 12002 },
	{ 0x9b4a, 0x9b4a, PDF_CMAP_SINGLE, 12367 },
	{ 0x9b4b, 0x9b4b, PDF_CMAP_SINGLE, 12369 },
	{ 0x9b4c, 0x9b4c, PDF_CMAP_SINGLE, 12368 },
	{ 0x9b4d, 0x9b4d, PDF_CMAP_SINGLE, 5481 },
	{ 0x9b4e, 0x9b4e, PDF_CMAP_SINGLE, 5480 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 5479 },
	{ 0x9b50, 0x9b50, PDF_CMAP_SINGLE, 14612 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 5806 },
	{ 0x9b52, 0x9b52, PDF_CMAP_SINGLE, 13129 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 5805 },
	{ 0x9b55, 0x9b55, PDF_CMAP_SINGLE, 13287 },
	{ 0x9b56, 0x9b56, PDF_CMAP_SINGLE, 13286 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 5945 },
	{ 0x9b59, 0x9b59, PDF_CMAP_SINGLE, 13488 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 3041 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 9741 },
	{ 0x9b5f, 0x9b5f, PDF_CMAP_SINGLE, 10360 },
	{ 0x9b60, 0x9b61, PDF_CMAP_RANGE, 10358 },
	{ 0x9b64, 0x9b64, PDF_CMAP_SINGLE, 10990 },
	{ 0x9b66, 0x9b66, PDF_CMAP_SINGLE, 10985 },
	{ 0x9b67, 0x9b67, PDF_CMAP_SINGLE, 10982 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 10989 },
	{ 0x9b69, 0x9b69, PDF_CMAP_SINGLE, 18373 },
	{ 0x9b6c, 0x9b6c, PDF_CMAP_SINGLE, 10991 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4737 },
	{ 0x9b70, 0x9b70, PDF_CMAP_SINGLE, 10988 },
	{ 0x9b71, 0x9b71, PDF_CMAP_SINGLE, 10984 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 10983 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 10987 },
	{ 0x9b76, 0x9b76, PDF_CMAP_SINGLE, 10986 },
	{ 0x9b77, 0x9b77, PDF_CMAP_SINGLE, 4736 },
	{ 0x9b7a, 0x9b7a, PDF_CMAP_SINGLE, 11552 },
	{ 0x9b7b, 0x9b7b, PDF_CMAP_SINGLE, 11547 },
	{ 0x9b7c, 0x9b7c, PDF_CMAP_SINGLE, 11545 },
	{ 0x9b7d, 0x9b7d, PDF_CMAP_SINGLE, 11554 },
	{ 0x9b7e, 0x9b7e, PDF_CMAP_SINGLE, 11546 },
	{ 0x9b7f, 0x9b7f, PDF_CMAP_SINGLE, 15419 },
	{ 0x9b80, 0x9b80, PDF_CMAP_SINGLE, 11542 },
	{ 0x9b81, 0x9b81, PDF_CMAP_SINGLE, 17393 },
	{ 0x9b82, 0x9b82, PDF_CMAP_SINGLE, 11548 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 16921 },
	{ 0x9b85, 0x9b85, PDF_CMAP_SINGLE, 11543 },
	{ 0x9b86, 0x9b86, PDF_CMAP_SINGLE, 12011 },
	{ 0x9b87, 0x9b87, PDF_CMAP_SINGLE, 11544 },
	{ 0x9b88, 0x9b88, PDF_CMAP_SINGLE, 11555 },
	{ 0x9b8b, 0x9b8b, PDF_CMAP_SINGLE, 16110 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 17955 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 14617 },
	{ 0x9b8f, 0x9b8f, PDF_CMAP_SINGLE, 16301 },
	{ 0x9b90, 0x9b90, PDF_CMAP_SINGLE, 11551 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 5033 },
	{ 0x9b92, 0x9b92, PDF_CMAP_SINGLE, 11550 },
	{ 0x9b93, 0x9b93, PDF_CMAP_SINGLE, 11549 },
	{ 0x9b95, 0x9b95, PDF_CMAP_SINGLE, 11553 },
	{ 0x9b97, 0x9b97, PDF_CMAP_SINGLE, 16241 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 12003 },
	{ 0x9b9b, 0x9b9b, PDF_CMAP_SINGLE, 12006 },
	{ 0x9b9d, 0x9b9d, PDF_CMAP_SINGLE, 16214 },
	{ 0x9b9e, 0x9b9e, PDF_CMAP_SINGLE, 12005 },
	{ 0x9b9f, 0x9b9f, PDF_CMAP_SINGLE, 15422 },
	{ 0x9ba0, 0x9ba0, PDF_CMAP_SINGLE, 12013 },
	{ 0x9ba1, 0x9ba1, PDF_CMAP_SINGLE, 12008 },
	{ 0x9ba2, 0x9ba2, PDF_CMAP_SINGLE, 12012 },
	{ 0x9ba4, 0x9ba4, PDF_CMAP_SINGLE, 12010 },
	{ 0x9ba5, 0x9ba5, PDF_CMAP_SINGLE, 12009 },
	{ 0x9ba6, 0x9ba6, PDF_CMAP_SINGLE, 12007 },
	{ 0x9ba8, 0x9ba8, PDF_CMAP_SINGLE, 12004 },
	{ 0x9baa, 0x9baa, PDF_CMAP_SINGLE, 5309 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 5308 },
	{ 0x9bad, 0x9bad, PDF_CMAP_SINGLE, 5310 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 5307 },
	{ 0x9baf, 0x9baf, PDF_CMAP_SINGLE, 12014 },
	{ 0x9bb0, 0x9bb0, PDF_CMAP_SINGLE, 16308 },
	{ 0x9bb5, 0x9bb5, PDF_CMAP_SINGLE, 12375 },
	{ 0x9bb6, 0x9bb6, PDF_CMAP_SINGLE, 12378 },
	{ 0x9bb8, 0x9bb8, PDF_CMAP_SINGLE, 12376 },
	{ 0x9bb9, 0x9bb9, PDF_CMAP_SINGLE, 12380 },
	{ 0x9bbd, 0x9bbd, PDF_CMAP_SINGLE, 12381 },
	{ 0x9bbf, 0x9bbf, PDF_CMAP_SINGLE, 12373 },
	{ 0x9bc0, 0x9bc0, PDF_CMAP_SINGLE, 5486 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 12374 },
	{ 0x9bc3, 0x9bc3, PDF_CMAP_SINGLE, 12372 },
	{ 0x9bc4, 0x9bc4, PDF_CMAP_SINGLE, 12379 },
	{ 0x9bc6, 0x9bc6, PDF_CMAP_SINGLE, 12371 },
	{ 0x9bc7, 0x9bc7, PDF_CMAP_SINGLE, 12370 },
	{ 0x9bc8, 0x9bc8, PDF_CMAP_SINGLE, 5485 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 5483 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 5482 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 15420 },
	{ 0x9bd3, 0x9bd3, PDF_CMAP_SINGLE, 12377 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 12697 },
	{ 0x9bd5, 0x9bd5, PDF_CMAP_SINGLE, 12703 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 5630 },
	{ 0x9bd7, 0x9bd7, PDF_CMAP_SINGLE, 12698 },
	{ 0x9bd9, 0x9bd9, PDF_CMAP_SINGLE, 12701 },
	{ 0x9bda, 0x9bda, PDF_CMAP_SINGLE, 12705 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 5631 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 12700 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 18374 },
	{ 0x9bde, 0x9bde, PDF_CMAP_SINGLE, 12692 },
	{ 0x9be0, 0x9be0, PDF_CMAP_SINGLE, 12691 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 12704 },
	{ 0x9be2, 0x9be2, PDF_CMAP_SINGLE, 12695 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 12693 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 12702 },
	{ 0x9be6, 0x9be6, PDF_CMAP_SINGLE, 12694 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 5629 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 5628 },
	{ 0x9be9, 0x9be9, PDF_CMAP_SINGLE, 15297 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 12689 },
	{ 0x9bec, 0x9bec, PDF_CMAP_SINGLE, 12699 },
	{ 0x9bed, 0x9bed, PDF_CMAP_SINGLE, 16087 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 12696 },
	{ 0x9bf1, 0x9bf1, PDF_CMAP_SINGLE, 18375 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 18376 },
	{ 0x9bf7, 0x9bf7, PDF_CMAP_SINGLE, 12947 },
	{ 0x9bf8, 0x9bf8, PDF_CMAP_SINGLE, 12950 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5484 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 14619 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 14618 },
	{ 0x9c05, 0x9c05, PDF_CMAP_SINGLE, 12948 },
	{ 0x9c06, 0x9c06, PDF_CMAP_SINGLE, 12954 },
	{ 0x9c07, 0x9c07, PDF_CMAP_SINGLE, 12952 },
	{ 0x9c08, 0x9c08, PDF_CMAP_SINGLE, 12946 },
	{ 0x9c09, 0x9c09, PDF_CMAP_SINGLE, 12957 },
	{ 0x9c0a, 0x9c0a, PDF_CMAP_SINGLE, 16186 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 12945 },
	{ 0x9c0c, 0x9c0c, PDF_CMAP_SINGLE, 14620 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 5728 },
	{ 0x9c0e, 0x9c0e, PDF_CMAP_SINGLE, 12953 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 15418 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 12949 },
	{ 0x9c13, 0x9c13, PDF_CMAP_SINGLE, 5727 },
	{ 0x9c14, 0x9c14, PDF_CMAP_SINGLE, 12956 },
	{ 0x9c15, 0x9c15, PDF_CMAP_SINGLE, 16158 },
	{ 0x9c17, 0x9c17, PDF_CMAP_SINGLE, 12955 },
	{ 0x9c1b, 0x9c1b, PDF_CMAP_SINGLE, 18715 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 13132 },
	{ 0x9c1d, 0x9c1d, PDF_CMAP_SINGLE, 13131 },
	{ 0x9c1f, 0x9c1f, PDF_CMAP_SINGLE, 17954 },
	{ 0x9c20, 0x9c20, PDF_CMAP_SINGLE, 18378 },
	{ 0x9c21, 0x9c21, PDF_CMAP_SINGLE, 13138 },
	{ 0x9c23, 0x9c23, PDF_CMAP_SINGLE, 13134 },
	{ 0x9c24, 0x9c24, PDF_CMAP_SINGLE, 13137 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 5808 },
	{ 0x9c26, 0x9c26, PDF_CMAP_SINGLE, 17894 },
	{ 0x9c28, 0x9c29, PDF_CMAP_RANGE, 13135 },
	{ 0x9c2b, 0x9c2b, PDF_CMAP_SINGLE, 13130 },
	{ 0x9c2c, 0x9c2c, PDF_CMAP_SINGLE, 13133 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 5807 },
	{ 0x9c2e, 0x9c2e, PDF_CMAP_SINGLE, 16162 },
	{ 0x9c2f, 0x9c2f, PDF_CMAP_SINGLE, 15751 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 5869 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 13298 },
	{ 0x9c33, 0x9c33, PDF_CMAP_SINGLE, 13293 },
	{ 0x9c34, 0x9c34, PDF_CMAP_SINGLE, 13297 },
	{ 0x9c35, 0x9c35, PDF_CMAP_SINGLE, 15417 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 13300 },
	{ 0x9c37, 0x9c37, PDF_CMAP_SINGLE, 13296 },
	{ 0x9c39, 0x9c39, PDF_CMAP_SINGLE, 13292 },
	{ 0x9c3a, 0x9c3a, PDF_CMAP_SINGLE, 15742 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 5871 },
	{ 0x9c3c, 0x9c3c, PDF_CMAP_SINGLE, 13295 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 13299 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 5870 },
	{ 0x9c3f, 0x9c3f, PDF_CMAP_SINGLE, 13290 },
	{ 0x9c40, 0x9c40, PDF_CMAP_SINGLE, 12951 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 13294 },
	{ 0x9c44, 0x9c44, PDF_CMAP_SINGLE, 13291 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 15985 },
	{ 0x9c46, 0x9c46, PDF_CMAP_SINGLE, 13288 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 13289 },
	{ 0x9c49, 0x9c49, PDF_CMAP_SINGLE, 5868 },
	{ 0x9c4a, 0x9c4a, PDF_CMAP_SINGLE, 13399 },
	{ 0x9c4b, 0x9c4b, PDF_CMAP_SINGLE, 13401 },
	{ 0x9c4c, 0x9c4c, PDF_CMAP_SINGLE, 13404 },
	{ 0x9c4d, 0x9c4d, PDF_CMAP_SINGLE, 13400 },
	{ 0x9c4e, 0x9c4e, PDF_CMAP_SINGLE, 13405 },
	{ 0x9c4f, 0x9c4f, PDF_CMAP_SINGLE, 15414 },
	{ 0x9c50, 0x9c50, PDF_CMAP_SINGLE, 13398 },
	{ 0x9c52, 0x9c52, PDF_CMAP_SINGLE, 13396 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 15415 },
	{ 0x9c54, 0x9c54, PDF_CMAP_SINGLE, 5913 },
	{ 0x9c55, 0x9c55, PDF_CMAP_SINGLE, 13402 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 5915 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 5914 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 13397 },
	{ 0x9c59, 0x9c59, PDF_CMAP_SINGLE, 13403 },
	{ 0x9c5d, 0x9c5d, PDF_CMAP_SINGLE, 16063 },
	{ 0x9c5e, 0x9c5e, PDF_CMAP_SINGLE, 13493 },
	{ 0x9c5f, 0x9c5f, PDF_CMAP_SINGLE, 5946 },
	{ 0x9c60, 0x9c60, PDF_CMAP_SINGLE, 13494 },
	{ 0x9c62, 0x9c62, PDF_CMAP_SINGLE, 13492 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 13489 },
	{ 0x9c66, 0x9c66, PDF_CMAP_SINGLE, 13491 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 13490 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 13556 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 13558 },
	{ 0x9c6e, 0x9c6e, PDF_CMAP_SINGLE, 13557 },
	{ 0x9c71, 0x9c71, PDF_CMAP_SINGLE, 13588 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 16033 },
	{ 0x9c73, 0x9c73, PDF_CMAP_SINGLE, 13587 },
	{ 0x9c74, 0x9c74, PDF_CMAP_SINGLE, 13586 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 13589 },
	{ 0x9c77, 0x9c78, PDF_CMAP_RANGE, 5984 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 13633 },
	{ 0x9c7a, 0x9c7a, PDF_CMAP_SINGLE, 13637 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 15416 },
	{ 0x9c7c, 0x9c7c, PDF_CMAP_SINGLE, 17682 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 3042 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 9055 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 9744 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3961 },
	{ 0x9cea, 0x9cea, PDF_CMAP_SINGLE, 9742 },
	{ 0x9ced, 0x9ced, PDF_CMAP_SINGLE, 9743 },
	{ 0x9cf1, 0x9cf2, PDF_CMAP_RANGE, 10361 },
	{ 0x9cf3, 0x9cf3, PDF_CMAP_SINGLE, 4350 },
	{ 0x9cf4, 0x9cf4, PDF_CMAP_SINGLE, 4348 },
	{ 0x9cf5, 0x9cf5, PDF_CMAP_SINGLE, 10363 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 4349 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 10996 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 10999 },
	{ 0x9cfa, 0x9cfa, PDF_CMAP_SINGLE, 10993 },
	{ 0x9cfb, 0x9cfb, PDF_CMAP_SINGLE, 11000 },
	{ 0x9cfc, 0x9cfc, PDF_CMAP_SINGLE, 10992 },
	{ 0x9cfd, 0x9cfd, PDF_CMAP_SINGLE, 10994 },
	{ 0x9cff, 0x9cff, PDF_CMAP_SINGLE, 10995 },
	{ 0x9d00, 0x9d00, PDF_CMAP_SINGLE, 10998 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 16187 },
	{ 0x9d03, 0x9d03, PDF_CMAP_SINGLE, 4740 },
	{ 0x9d04, 0x9d04, PDF_CMAP_SINGLE, 11003 },
	{ 0x9d05, 0x9d05, PDF_CMAP_SINGLE, 11002 },
	{ 0x9d06, 0x9d06, PDF_CMAP_SINGLE, 4738 },
	{ 0x9d07, 0x9d07, PDF_CMAP_SINGLE, 10997 },
	{ 0x9d08, 0x9d08, PDF_CMAP_SINGLE, 11001 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 4739 },
	{ 0x9d0c, 0x9d0c, PDF_CMAP_SINGLE, 15430 },
	{ 0x9d10, 0x9d10, PDF_CMAP_SINGLE, 11565 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 5038 },
	{ 0x9d14, 0x9d14, PDF_CMAP_SINGLE, 11560 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 5034 },
	{ 0x9d16, 0x9d16, PDF_CMAP_SINGLE, 15484 },
	{ 0x9d17, 0x9d17, PDF_CMAP_SINGLE, 11557 },
	{ 0x9d18, 0x9d18, PDF_CMAP_SINGLE, 11563 },
	{ 0x9d19, 0x9d19, PDF_CMAP_SINGLE, 11566 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 5039 },
	{ 0x9d1d, 0x9d1d, PDF_CMAP_SINGLE, 11562 },
	{ 0x9d1e, 0x9d1e, PDF_CMAP_SINGLE, 11559 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 11567 },
	{ 0x9d20, 0x9d20, PDF_CMAP_SINGLE, 11558 },
	{ 0x9d21, 0x9d21, PDF_CMAP_SINGLE, 15425 },
	{ 0x9d22, 0x9d22, PDF_CMAP_SINGLE, 11564 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 5035 },
	{ 0x9d25, 0x9d25, PDF_CMAP_SINGLE, 11556 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 5036 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 5037 },
	{ 0x9d29, 0x9d29, PDF_CMAP_SINGLE, 11561 },
	{ 0x9d2d, 0x9d2d, PDF_CMAP_SINGLE, 12032 },
	{ 0x9d2e, 0x9d2f, PDF_CMAP_RANGE, 12019 },
	{ 0x9d30, 0x9d30, PDF_CMAP_SINGLE, 12023 },
	{ 0x9d31, 0x9d31, PDF_CMAP_SINGLE, 12021 },
	{ 0x9d33, 0x9d33, PDF_CMAP_SINGLE, 12015 },
	{ 0x9d34, 0x9d34, PDF_CMAP_SINGLE, 16458 },
	{ 0x9d36, 0x9d36, PDF_CMAP_SINGLE, 12018 },
	{ 0x9d37, 0x9d37, PDF_CMAP_SINGLE, 12028 },
	{ 0x9d38, 0x9d38, PDF_CMAP_SINGLE, 12022 },
	{ 0x9d39, 0x9d39, PDF_CMAP_SINGLE, 14638 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 5311 },
	{ 0x9d3d, 0x9d3d, PDF_CMAP_SINGLE, 12030 },
	{ 0x9d3e, 0x9d3e, PDF_CMAP_SINGLE, 12027 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 5312 },
	{ 0x9d40, 0x9d40, PDF_CMAP_SINGLE, 12029 },
	{ 0x9d41, 0x9d41, PDF_CMAP_SINGLE, 12016 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 12025 },
	{ 0x9d44, 0x9d44, PDF_CMAP_SINGLE, 16299 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 12024 },
	{ 0x9d49, 0x9d49, PDF_CMAP_SINGLE, 18381 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 12385 },
	{ 0x9d4b, 0x9d4b, PDF_CMAP_SINGLE, 12387 },
	{ 0x9d4c, 0x9d4c, PDF_CMAP_SINGLE, 12390 },
	{ 0x9d4e, 0x9d4e, PDF_CMAP_SINGLE, 17721 },
	{ 0x9d4f, 0x9d4f, PDF_CMAP_SINGLE, 12384 },
	{ 0x9d50, 0x9d50, PDF_CMAP_SINGLE, 16074 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 5487 },
	{ 0x9d52, 0x9d52, PDF_CMAP_SINGLE, 12392 },
	{ 0x9d53, 0x9d53, PDF_CMAP_SINGLE, 12383 },
	{ 0x9d54, 0x9d54, PDF_CMAP_SINGLE, 12393 },
	{ 0x9d56, 0x9d56, PDF_CMAP_SINGLE, 12389 },
	{ 0x9d57, 0x9d57, PDF_CMAP_SINGLE, 12391 },
	{ 0x9d58, 0x9d58, PDF_CMAP_SINGLE, 12395 },
	{ 0x9d59, 0x9d59, PDF_CMAP_SINGLE, 12388 },
	{ 0x9d5a, 0x9d5a, PDF_CMAP_SINGLE, 12396 },
	{ 0x9d5b, 0x9d5b, PDF_CMAP_SINGLE, 12386 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 12382 },
	{ 0x9d5d, 0x9d5d, PDF_CMAP_SINGLE, 5488 },
	{ 0x9d5e, 0x9d5e, PDF_CMAP_SINGLE, 15982 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 12394 },
	{ 0x9d60, 0x9d60, PDF_CMAP_SINGLE, 5489 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 5633 },
	{ 0x9d67, 0x9d67, PDF_CMAP_SINGLE, 12017 },
	{ 0x9d68, 0x9d68, PDF_CMAP_SINGLE, 12731 },
	{ 0x9d69, 0x9d69, PDF_CMAP_SINGLE, 12722 },
	{ 0x9d6a, 0x9d6a, PDF_CMAP_SINGLE, 5635 },
	{ 0x9d6b, 0x9d6b, PDF_CMAP_SINGLE, 12718 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 5636 },
	{ 0x9d6d, 0x9d6d, PDF_CMAP_SINGLE, 15276 },
	{ 0x9d6e, 0x9d6e, PDF_CMAP_SINGLE, 17211 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 12727 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 12721 },
	{ 0x9d71, 0x9d71, PDF_CMAP_SINGLE, 12711 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 5634 },
	{ 0x9d73, 0x9d73, PDF_CMAP_SINGLE, 12724 },
	{ 0x9d74, 0x9d75, PDF_CMAP_RANGE, 12719 },
	{ 0x9d77, 0x9d77, PDF_CMAP_SINGLE, 12706 },
	{ 0x9d78, 0x9d78, PDF_CMAP_SINGLE, 12713 },
	{ 0x9d79, 0x9d79, PDF_CMAP_SINGLE, 12728 },
	{ 0x9d7b, 0x9d7b, PDF_CMAP_SINGLE, 12725 },
	{ 0x9d7c, 0x9d7c, PDF_CMAP_SINGLE, 16126 },
	{ 0x9d7d, 0x9d7d, PDF_CMAP_SINGLE, 12717 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 14629 },
	{ 0x9d7f, 0x9d7f, PDF_CMAP_SINGLE, 12729 },
	{ 0x9d80, 0x9d80, PDF_CMAP_SINGLE, 12712 },
	{ 0x9d81, 0x9d81, PDF_CMAP_SINGLE, 12707 },
	{ 0x9d82, 0x9d82, PDF_CMAP_SINGLE, 12726 },
	{ 0x9d83, 0x9d83, PDF_CMAP_SINGLE, 14630 },
	{ 0x9d84, 0x9d84, PDF_CMAP_SINGLE, 12709 },
	{ 0x9d85, 0x9d85, PDF_CMAP_SINGLE, 12723 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 12714 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 12730 },
	{ 0x9d88, 0x9d88, PDF_CMAP_SINGLE, 12710 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 5632 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 12708 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 12715 },
	{ 0x9d90, 0x9d90, PDF_CMAP_SINGLE, 12964 },
	{ 0x9d92, 0x9d92, PDF_CMAP_SINGLE, 12962 },
	{ 0x9d93, 0x9d93, PDF_CMAP_SINGLE, 17395 },
	{ 0x9d94, 0x9d94, PDF_CMAP_SINGLE, 12967 },
	{ 0x9d96, 0x9d96, PDF_CMAP_SINGLE, 12979 },
	{ 0x9d97, 0x9d97, PDF_CMAP_SINGLE, 12970 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 12963 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 12959 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 12972 },
	{ 0x9d9b, 0x9d9b, PDF_CMAP_SINGLE, 12965 },
	{ 0x9d9c, 0x9d9c, PDF_CMAP_SINGLE, 12968 },
	{ 0x9d9d, 0x9d9d, PDF_CMAP_SINGLE, 12961 },
	{ 0x9d9e, 0x9d9e, PDF_CMAP_SINGLE, 12975 },
	{ 0x9d9f, 0x9d9f, PDF_CMAP_SINGLE, 12958 },
	{ 0x9da0, 0x9da0, PDF_CMAP_SINGLE, 12966 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 12971 },
	{ 0x9da2, 0x9da2, PDF_CMAP_SINGLE, 12973 },
	{ 0x9da3, 0x9da3, PDF_CMAP_SINGLE, 12976 },
	{ 0x9da4, 0x9da4, PDF_CMAP_SINGLE, 12960 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 15435 },
	{ 0x9da6, 0x9da7, PDF_CMAP_RANGE, 12980 },
	{ 0x9da8, 0x9da8, PDF_CMAP_SINGLE, 12974 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 12978 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 12969 },
	{ 0x9dab, 0x9dab, PDF_CMAP_SINGLE, 16176 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 13154 },
	{ 0x9dad, 0x9dad, PDF_CMAP_SINGLE, 13157 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 5809 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 13156 },
	{ 0x9db2, 0x9db2, PDF_CMAP_SINGLE, 13161 },
	{ 0x9db3, 0x9db3, PDF_CMAP_SINGLE, 13159 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 5810 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 13150 },
	{ 0x9db6, 0x9db6, PDF_CMAP_SINGLE, 13140 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 13139 },
	{ 0x9db8, 0x9db8, PDF_CMAP_SINGLE, 5812 },
	{ 0x9db9, 0x9dba, PDF_CMAP_RANGE, 13152 },
	{ 0x9dbb, 0x9dbb, PDF_CMAP_SINGLE, 13149 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 13141 },
	{ 0x9dbd, 0x9dbd, PDF_CMAP_SINGLE, 18384 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 13146 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 12977 },
	{ 0x9dc0, 0x9dc0, PDF_CMAP_SINGLE, 17394 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 13142 },
	{ 0x9dc2, 0x9dc2, PDF_CMAP_SINGLE, 5811 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 13148 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 14633 },
	{ 0x9dc5, 0x9dc5, PDF_CMAP_SINGLE, 13147 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 13143 },
	{ 0x9dc8, 0x9dc8, PDF_CMAP_SINGLE, 13155 },
	{ 0x9dc9, 0x9dc9, PDF_CMAP_SINGLE, 17785 },
	{ 0x9dca, 0x9dca, PDF_CMAP_SINGLE, 13144 },
	{ 0x9dcb, 0x9dcb, PDF_CMAP_SINGLE, 13305 },
	{ 0x9dcc, 0x9dcc, PDF_CMAP_SINGLE, 13158 },
	{ 0x9dcd, 0x9dcd, PDF_CMAP_SINGLE, 13160 },
	{ 0x9dce, 0x9dce, PDF_CMAP_SINGLE, 13151 },
	{ 0x9dcf, 0x9dcf, PDF_CMAP_SINGLE, 13145 },
	{ 0x9dd0, 0x9dd0, PDF_CMAP_SINGLE, 13306 },
	{ 0x9dd1, 0x9dd1, PDF_CMAP_SINGLE, 13308 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 13302 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 5872 },
	{ 0x9dd4, 0x9dd4, PDF_CMAP_SINGLE, 14622 },
	{ 0x9dd5, 0x9dd5, PDF_CMAP_SINGLE, 13315 },
	{ 0x9dd6, 0x9dd6, PDF_CMAP_SINGLE, 13313 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 5873 },
	{ 0x9dd8, 0x9dd8, PDF_CMAP_SINGLE, 13312 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 13311 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 13304 },
	{ 0x9ddb, 0x9ddb, PDF_CMAP_SINGLE, 13301 },
	{ 0x9ddc, 0x9ddc, PDF_CMAP_SINGLE, 13307 },
	{ 0x9ddd, 0x9ddd, PDF_CMAP_SINGLE, 13316 },
	{ 0x9dde, 0x9dde, PDF_CMAP_SINGLE, 13303 },
	{ 0x9ddf, 0x9ddf, PDF_CMAP_SINGLE, 13309 },
	{ 0x9de1, 0x9de1, PDF_CMAP_SINGLE, 13414 },
	{ 0x9de2, 0x9de2, PDF_CMAP_SINGLE, 13419 },
	{ 0x9de3, 0x9de3, PDF_CMAP_SINGLE, 13409 },
	{ 0x9de4, 0x9de4, PDF_CMAP_SINGLE, 13412 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 5916 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 13416 },
	{ 0x9de8, 0x9de8, PDF_CMAP_SINGLE, 13423 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 13310 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 13410 },
	{ 0x9dec, 0x9dec, PDF_CMAP_SINGLE, 13420 },
	{ 0x9ded, 0x9ded, PDF_CMAP_SINGLE, 13424 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 13415 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 13408 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 13418 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 13417 },
	{ 0x9df3, 0x9df3, PDF_CMAP_SINGLE, 13422 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 13421 },
	{ 0x9df5, 0x9df5, PDF_CMAP_SINGLE, 13314 },
	{ 0x9df6, 0x9df6, PDF_CMAP_SINGLE, 13413 },
	{ 0x9df7, 0x9df7, PDF_CMAP_SINGLE, 13407 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 13411 },
	{ 0x9df9, 0x9dfa, PDF_CMAP_RANGE, 5947 },
	{ 0x9dfb, 0x9dfb, PDF_CMAP_SINGLE, 13406 },
	{ 0x9dfc, 0x9dfc, PDF_CMAP_SINGLE, 18385 },
	{ 0x9dfd, 0x9dfd, PDF_CMAP_SINGLE, 13505 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 13496 },
	{ 0x9dff, 0x9dff, PDF_CMAP_SINGLE, 13504 },
	{ 0x9e00, 0x9e01, PDF_CMAP_RANGE, 13501 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 13495 },
	{ 0x9e03, 0x9e03, PDF_CMAP_SINGLE, 13498 },
	{ 0x9e04, 0x9e04, PDF_CMAP_SINGLE, 13506 },
	{ 0x9e05, 0x9e05, PDF_CMAP_SINGLE, 13500 },
	{ 0x9e06, 0x9e06, PDF_CMAP_SINGLE, 13499 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 13497 },
	{ 0x9e09, 0x9e09, PDF_CMAP_SINGLE, 13503 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 17790 },
	{ 0x9e0b, 0x9e0b, PDF_CMAP_SINGLE, 13559 },
	{ 0x9e0c, 0x9e0c, PDF_CMAP_SINGLE, 17786 },
	{ 0x9e0d, 0x9e0d, PDF_CMAP_SINGLE, 13560 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 14632 },
	{ 0x9e0f, 0x9e0f, PDF_CMAP_SINGLE, 13562 },
	{ 0x9e10, 0x9e10, PDF_CMAP_SINGLE, 13561 },
	{ 0x9e11, 0x9e11, PDF_CMAP_SINGLE, 13564 },
	{ 0x9e12, 0x9e12, PDF_CMAP_SINGLE, 13563 },
	{ 0x9e13, 0x9e13, PDF_CMAP_SINGLE, 13591 },
	{ 0x9e14, 0x9e14, PDF_CMAP_SINGLE, 13590 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 13616 },
	{ 0x9e17, 0x9e17, PDF_CMAP_SINGLE, 13617 },
	{ 0x9e18, 0x9e18, PDF_CMAP_SINGLE, 15428 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 13629 },
	{ 0x9e1a, 0x9e1a, PDF_CMAP_SINGLE, 5989 },
	{ 0x9e1b, 0x9e1b, PDF_CMAP_SINGLE, 5993 },
	{ 0x9e1c, 0x9e1c, PDF_CMAP_SINGLE, 16260 },
	{ 0x9e1d, 0x9e1d, PDF_CMAP_SINGLE, 13638 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 5994 },
	{ 0x9e1f, 0x9e1f, PDF_CMAP_SINGLE, 17683 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 3043 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 5729 },
	{ 0x9e7a, 0x9e7a, PDF_CMAP_SINGLE, 13162 },
	{ 0x9e7b, 0x9e7b, PDF_CMAP_SINGLE, 17400 },
	{ 0x9e7c, 0x9e7d, PDF_CMAP_RANGE, 5949 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 3044 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 9745 },
	{ 0x9e81, 0x9e81, PDF_CMAP_SINGLE, 16143 },
	{ 0x9e82, 0x9e82, PDF_CMAP_SINGLE, 3962 },
	{ 0x9e83, 0x9e83, PDF_CMAP_SINGLE, 11004 },
	{ 0x9e84, 0x9e84, PDF_CMAP_SINGLE, 16246 },
	{ 0x9e85, 0x9e85, PDF_CMAP_SINGLE, 16114 },
	{ 0x9e86, 0x9e87, PDF_CMAP_RANGE, 11569 },
	{ 0x9e88, 0x9e88, PDF_CMAP_SINGLE, 11568 },
	{ 0x9e89, 0x9e89, PDF_CMAP_SINGLE, 12034 },
	{ 0x9e8a, 0x9e8a, PDF_CMAP_SINGLE, 12033 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 5313 },
	{ 0x9e8c, 0x9e8c, PDF_CMAP_SINGLE, 12398 },
	{ 0x9e8d, 0x9e8d, PDF_CMAP_SINGLE, 12035 },
	{ 0x9e8e, 0x9e8e, PDF_CMAP_SINGLE, 12397 },
	{ 0x9e90, 0x9e90, PDF_CMAP_SINGLE, 14641 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 12733 },
	{ 0x9e92, 0x9e92, PDF_CMAP_SINGLE, 5637 },
	{ 0x9e93, 0x9e93, PDF_CMAP_SINGLE, 5639 },
	{ 0x9e94, 0x9e94, PDF_CMAP_SINGLE, 12732 },
	{ 0x9e95, 0x9e95, PDF_CMAP_SINGLE, 14642 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 16316 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 5638 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 16167 },
	{ 0x9e99, 0x9e99, PDF_CMAP_SINGLE, 12982 },
	{ 0x9e9a, 0x9e9a, PDF_CMAP_SINGLE, 12984 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 12983 },
	{ 0x9e9c, 0x9e9c, PDF_CMAP_SINGLE, 13163 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 5813 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 14643 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 5917 },
	{ 0x9ea0, 0x9ea0, PDF_CMAP_SINGLE, 13507 },
	{ 0x9ea1, 0x9ea1, PDF_CMAP_SINGLE, 13565 },
	{ 0x9ea2, 0x9ea2, PDF_CMAP_SINGLE, 14644 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 13642 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 3045 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 18756 },
	{ 0x9ea7, 0x9ea7, PDF_CMAP_SINGLE, 10364 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 16245 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 4741 },
	{ 0x9eaa, 0x9eaa, PDF_CMAP_SINGLE, 14646 },
	{ 0x9eab, 0x9eab, PDF_CMAP_SINGLE, 16018 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 17396 },
	{ 0x9ead, 0x9ead, PDF_CMAP_SINGLE, 11572 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 11571 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 14647 },
	{ 0x9eb0, 0x9eb0, PDF_CMAP_SINGLE, 12036 },
	{ 0x9eb1, 0x9eb1, PDF_CMAP_SINGLE, 18388 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 5640 },
	{ 0x9eb5, 0x9eb5, PDF_CMAP_SINGLE, 5730 },
	{ 0x9eb6, 0x9eb6, PDF_CMAP_SINGLE, 13317 },
	{ 0x9eb7, 0x9eb7, PDF_CMAP_SINGLE, 13634 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 3046 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 4351 },
	{ 0x9ebd, 0x9ebd, PDF_CMAP_SINGLE, 18389 },
	{ 0x9ebe, 0x9ebe, PDF_CMAP_SINGLE, 4742 },
	{ 0x9ebf, 0x9ebf, PDF_CMAP_SINGLE, 15736 },
	{ 0x9ec0, 0x9ec0, PDF_CMAP_SINGLE, 12734 },
	{ 0x9ec1, 0x9ec1, PDF_CMAP_SINGLE, 14649 },
	{ 0x9ec2, 0x9ec2, PDF_CMAP_SINGLE, 13425 },
	{ 0x9ec3, 0x9ec3, PDF_CMAP_SINGLE, 3515 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 17684 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 18390 },
	{ 0x9ec7, 0x9ec7, PDF_CMAP_SINGLE, 17783 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 12037 },
	{ 0x9ecc, 0x9ecc, PDF_CMAP_SINGLE, 5969 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 3516 },
	{ 0x9ece, 0x9ece, PDF_CMAP_SINGLE, 4743 },
	{ 0x9ecf, 0x9ecf, PDF_CMAP_SINGLE, 5314 },
	{ 0x9ed0, 0x9ed0, PDF_CMAP_SINGLE, 13426 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 3517 },
	{ 0x9ed3, 0x9ed3, PDF_CMAP_SINGLE, 11005 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 5041 },
	{ 0x9ed5, 0x9ed6, PDF_CMAP_RANGE, 11573 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 5040 },
	{ 0x9eda, 0x9eda, PDF_CMAP_SINGLE, 12038 },
	{ 0x9edb, 0x9edb, PDF_CMAP_SINGLE, 5318 },
	{ 0x9edc, 0x9edd, PDF_CMAP_RANGE, 5316 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 5315 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 12399 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 5490 },
	{ 0x9ee2, 0x9ee2, PDF_CMAP_SINGLE, 18392 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 12986 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 12985 },
	{ 0x9ee6, 0x9ee6, PDF_CMAP_SINGLE, 12988 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 12987 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 5731 },
	{ 0x9eeb, 0x9eeb, PDF_CMAP_SINGLE, 13164 },
	{ 0x9eed, 0x9eed, PDF_CMAP_SINGLE, 13166 },
	{ 0x9eee, 0x9eee, PDF_CMAP_SINGLE, 13165 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 5814 },
	{ 0x9ef0, 0x9ef0, PDF_CMAP_SINGLE, 13318 },
	{ 0x9ef1, 0x9ef1, PDF_CMAP_SINGLE, 18393 },
	{ 0x9ef2, 0x9ef3, PDF_CMAP_RANGE, 13427 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 5918 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 13566 },
	{ 0x9ef6, 0x9ef6, PDF_CMAP_SINGLE, 13592 },
	{ 0x9ef7, 0x9ef7, PDF_CMAP_SINGLE, 5986 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 18394 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 9056 },
	{ 0x9efa, 0x9efa, PDF_CMAP_SINGLE, 11575 },
	{ 0x9efb, 0x9efb, PDF_CMAP_SINGLE, 12039 },
	{ 0x9efc, 0x9efc, PDF_CMAP_SINGLE, 12735 },
	{ 0x9efd, 0x9efd, PDF_CMAP_SINGLE, 9746 },
	{ 0x9efe, 0x9efe, PDF_CMAP_SINGLE, 18382 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 12040 },
	{ 0x9f00, 0x9f00, PDF_CMAP_SINGLE, 12401 },
	{ 0x9f01, 0x9f01, PDF_CMAP_SINGLE, 12400 },
	{ 0x9f02, 0x9f02, PDF_CMAP_SINGLE, 14656 },
	{ 0x9f06, 0x9f06, PDF_CMAP_SINGLE, 13429 },
	{ 0x9f07, 0x9f07, PDF_CMAP_SINGLE, 5951 },
	{ 0x9f08, 0x9f08, PDF_CMAP_SINGLE, 14657 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 13567 },
	{ 0x9f0a, 0x9f0a, PDF_CMAP_SINGLE, 13593 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3963 },
	{ 0x9f0f, 0x9f10, PDF_CMAP_RANGE, 11006 },
	{ 0x9f12, 0x9f12, PDF_CMAP_SINGLE, 11576 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 3964 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 5491 },
	{ 0x9f16, 0x9f16, PDF_CMAP_SINGLE, 12402 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 14661 },
	{ 0x9f18, 0x9f18, PDF_CMAP_SINGLE, 13168 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 5815 },
	{ 0x9f1a, 0x9f1a, PDF_CMAP_SINGLE, 13169 },
	{ 0x9f1b, 0x9f1b, PDF_CMAP_SINGLE, 13167 },
	{ 0x9f1c, 0x9f1c, PDF_CMAP_SINGLE, 13430 },
	{ 0x9f1e, 0x9f1e, PDF_CMAP_SINGLE, 13508 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 3965 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 12043 },
	{ 0x9f23, 0x9f23, PDF_CMAP_SINGLE, 12042 },
	{ 0x9f24, 0x9f24, PDF_CMAP_SINGLE, 12041 },
	{ 0x9f25, 0x9f25, PDF_CMAP_SINGLE, 12403 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 16272 },
	{ 0x9f27, 0x9f27, PDF_CMAP_SINGLE, 17952 },
	{ 0x9f28, 0x9f28, PDF_CMAP_SINGLE, 12407 },
	{ 0x9f29, 0x9f29, PDF_CMAP_SINGLE, 12406 },
	{ 0x9f2a, 0x9f2a, PDF_CMAP_SINGLE, 12405 },
	{ 0x9f2b, 0x9f2b, PDF_CMAP_SINGLE, 12404 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 5492 },
	{ 0x9f2d, 0x9f2d, PDF_CMAP_SINGLE, 12736 },
	{ 0x9f2e, 0x9f2e, PDF_CMAP_SINGLE, 12990 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 5732 },
	{ 0x9f30, 0x9f30, PDF_CMAP_SINGLE, 12989 },
	{ 0x9f31, 0x9f31, PDF_CMAP_SINGLE, 13170 },
	{ 0x9f32, 0x9f32, PDF_CMAP_SINGLE, 13321 },
	{ 0x9f33, 0x9f33, PDF_CMAP_SINGLE, 13320 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 5874 },
	{ 0x9f35, 0x9f35, PDF_CMAP_SINGLE, 13319 },
	{ 0x9f36, 0x9f36, PDF_CMAP_SINGLE, 13433 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 13432 },
	{ 0x9f38, 0x9f38, PDF_CMAP_SINGLE, 13431 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 14663 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 4352 },
	{ 0x9f3d, 0x9f3d, PDF_CMAP_SINGLE, 11577 },
	{ 0x9f3e, 0x9f3e, PDF_CMAP_SINGLE, 5319 },
	{ 0x9f40, 0x9f41, PDF_CMAP_RANGE, 12737 },
	{ 0x9f42, 0x9f42, PDF_CMAP_SINGLE, 13322 },
	{ 0x9f43, 0x9f43, PDF_CMAP_SINGLE, 13434 },
	{ 0x9f44, 0x9f44, PDF_CMAP_SINGLE, 18395 },
	{ 0x9f45, 0x9f45, PDF_CMAP_SINGLE, 14666 },
	{ 0x9f46, 0x9f46, PDF_CMAP_SINGLE, 13509 },
	{ 0x9f47, 0x9f47, PDF_CMAP_SINGLE, 13568 },
	{ 0x9f48, 0x9f48, PDF_CMAP_SINGLE, 13618 },
	{ 0x9f49, 0x9f49, PDF_CMAP_SINGLE, 13644 },
	{ 0x9f4a, 0x9f4a, PDF_CMAP_SINGLE, 4353 },
	{ 0x9f4b, 0x9f4b, PDF_CMAP_SINGLE, 5320 },
	{ 0x9f4c, 0x9f4c, PDF_CMAP_SINGLE, 12408 },
	{ 0x9f4d, 0x9f4d, PDF_CMAP_SINGLE, 12739 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 13171 },
	{ 0x9f4f, 0x9f4f, PDF_CMAP_SINGLE, 13435 },
	{ 0x9f50, 0x9f50, PDF_CMAP_SINGLE, 18755 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 4745 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 16256 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 12044 },
	{ 0x9f55, 0x9f55, PDF_CMAP_SINGLE, 12409 },
	{ 0x9f56, 0x9f58, PDF_CMAP_RANGE, 12740 },
	{ 0x9f59, 0x9f59, PDF_CMAP_SINGLE, 12995 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 16163 },
	{ 0x9f5b, 0x9f5b, PDF_CMAP_SINGLE, 12991 },
	{ 0x9f5c, 0x9f5c, PDF_CMAP_SINGLE, 5816 },
	{ 0x9f5d, 0x9f5d, PDF_CMAP_SINGLE, 12994 },
	{ 0x9f5e, 0x9f5e, PDF_CMAP_SINGLE, 12993 },
	{ 0x9f5f, 0x9f5f, PDF_CMAP_SINGLE, 5733 },
	{ 0x9f60, 0x9f60, PDF_CMAP_SINGLE, 12992 },
	{ 0x9f61, 0x9f61, PDF_CMAP_SINGLE, 5735 },
	{ 0x9f62, 0x9f62, PDF_CMAP_SINGLE, 18105 },
	{ 0x9f63, 0x9f63, PDF_CMAP_SINGLE, 5734 },
	{ 0x9f64, 0x9f64, PDF_CMAP_SINGLE, 13173 },
	{ 0x9f65, 0x9f65, PDF_CMAP_SINGLE, 13172 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 5817 },
	{ 0x9f69, 0x9f69, PDF_CMAP_SINGLE, 14672 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 5876 },
	{ 0x9f6b, 0x9f6b, PDF_CMAP_SINGLE, 13323 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 5875 },
	{ 0x9f6e, 0x9f6f, PDF_CMAP_RANGE, 13438 },
	{ 0x9f70, 0x9f70, PDF_CMAP_SINGLE, 13437 },
	{ 0x9f71, 0x9f71, PDF_CMAP_SINGLE, 13436 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 5953 },
	{ 0x9f74, 0x9f76, PDF_CMAP_RANGE, 13510 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 5952 },
	{ 0x9f78, 0x9f78, PDF_CMAP_SINGLE, 13569 },
	{ 0x9f79, 0x9f79, PDF_CMAP_SINGLE, 13572 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 13571 },
	{ 0x9f7b, 0x9f7b, PDF_CMAP_SINGLE, 13570 },
	{ 0x9f7e, 0x9f7e, PDF_CMAP_SINGLE, 13643 },
	{ 0x9f7f, 0x9f7f, PDF_CMAP_SINGLE, 18048 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 5042 },
	{ 0x9f8e, 0x9f8e, PDF_CMAP_SINGLE, 14674 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 5500 },
	{ 0x9f91, 0x9f91, PDF_CMAP_SINGLE, 12996 },
	{ 0x9f92, 0x9f92, PDF_CMAP_SINGLE, 13174 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 5877 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 13324 },
	{ 0x9f96, 0x9f96, PDF_CMAP_SINGLE, 19104 },
	{ 0x9f97, 0x9f97, PDF_CMAP_SINGLE, 19096 },
	{ 0x9f98, 0x9f98, PDF_CMAP_SINGLE, 13645 },
	{ 0x9f99, 0x9f99, PDF_CMAP_SINGLE, 17799 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 5043 },
	{ 0x9f9f, 0x9f9f, PDF_CMAP_SINGLE, 17990 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 12045 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 13325 },
	{ 0x9fa4, 0x9fa4, PDF_CMAP_SINGLE, 13594 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 16240 },
	{ 0x9fa6, 0x9fa6, PDF_CMAP_SINGLE, 18852 },
	{ 0x9fa7, 0x9fa7, PDF_CMAP_SINGLE, 18893 },
	{ 0x9fa8, 0x9fa8, PDF_CMAP_SINGLE, 18900 },
	{ 0x9fa9, 0x9fa9, PDF_CMAP_SINGLE, 18932 },
	{ 0x9faa, 0x9faa, PDF_CMAP_SINGLE, 18934 },
	{ 0x9fab, 0x9fab, PDF_CMAP_SINGLE, 18936 },
	{ 0x9fac, 0x9fac, PDF_CMAP_SINGLE, 18941 },
	{ 0x9fad, 0x9fae, PDF_CMAP_RANGE, 18960 },
	{ 0x9faf, 0x9faf, PDF_CMAP_SINGLE, 18963 },
	{ 0x9fb0, 0x9fb0, PDF_CMAP_SINGLE, 14005 },
	{ 0x9fb1, 0x9fb1, PDF_CMAP_SINGLE, 14007 },
	{ 0x9fb2, 0x9fb2, PDF_CMAP_SINGLE, 19039 },
	{ 0x9fb3, 0x9fb3, PDF_CMAP_SINGLE, 19064 },
	{ 0x9fc7, 0x9fc7, PDF_CMAP_SINGLE, 19126 },
	{ 0x9fc8, 0x9fc8, PDF_CMAP_SINGLE, 19142 },
	{ 0x9fc9, 0x9fc9, PDF_CMAP_SINGLE, 19146 },
	{ 0x9fca, 0x9fca, PDF_CMAP_SINGLE, 19150 },
	{ 0x9fcb, 0x9fcb, PDF_CMAP_SINGLE, 19155 },
	{ 0xdc00, 0xdc00, PDF_CMAP_SINGLE, 16816 },
	{ 0xdc01, 0xdc01, PDF_CMAP_SINGLE, 14149 },
	{ 0xdc02, 0xdc02, PDF_CMAP_SINGLE, 18536 },
	{ 0xdc02, 0xdc02, PDF_CMAP_SINGLE, 14423 },
	{ 0xdc03, 0xdc03, PDF_CMAP_SINGLE, 17862 },
	{ 0xdc03, 0xdc03, PDF_CMAP_SINGLE, 17075 },
	{ 0xdc04, 0xdc04, PDF_CMAP_SINGLE, 18505 },
	{ 0xdc04, 0xdc04, PDF_CMAP_SINGLE, 17033 },
	{ 0xdc06, 0xdc06, PDF_CMAP_SINGLE, 18531 },
	{ 0xdc06, 0xdc06, PDF_CMAP_SINGLE, 15843 },
	{ 0xdc09, 0xdc09, PDF_CMAP_SINGLE, 15286 },
	{ 0xdc09, 0xdc09, PDF_CMAP_SINGLE, 18576 },
	{ 0xdc09, 0xdc09, PDF_CMAP_SINGLE, 18598 },
	{ 0xdc0b, 0xdc0b, PDF_CMAP_SINGLE, 17848 },
	{ 0xdc0d, 0xdc0d, PDF_CMAP_SINGLE, 15154 },
	{ 0xdc0e, 0xdc0e, PDF_CMAP_SINGLE, 17789 },
	{ 0xdc10, 0xdc10, PDF_CMAP_SINGLE, 14522 },
	{ 0xdc10, 0xdc10, PDF_CMAP_SINGLE, 17681 },
	{ 0xdc11, 0xdc11, PDF_CMAP_SINGLE, 16381 },
	{ 0xdc11, 0xdc11, PDF_CMAP_SINGLE, 15851 },
	{ 0xdc12, 0xdc12, PDF_CMAP_SINGLE, 18510 },
	{ 0xdc12, 0xdc12, PDF_CMAP_SINGLE, 15196 },
	{ 0xdc12, 0xdc12, PDF_CMAP_SINGLE, 15825 },
	{ 0xdc12, 0xdc12, PDF_CMAP_SINGLE, 14437 },
	{ 0xdc13, 0xdc13, PDF_CMAP_SINGLE, 14879 },
	{ 0xdc13, 0xdc13, PDF_CMAP_SINGLE, 15509 },
	{ 0xdc14, 0xdc14, PDF_CMAP_SINGLE, 14780 },
	{ 0xdc14, 0xdc14, PDF_CMAP_SINGLE, 14424 },
	{ 0xdc14, 0xdc14, PDF_CMAP_SINGLE, 18095 },
	{ 0xdc16, 0xdc16, PDF_CMAP_SINGLE, 16592 },
	{ 0xdc19, 0xdc19, PDF_CMAP_SINGLE, 18519 },
	{ 0xdc1c, 0xdc1c, PDF_CMAP_SINGLE, 17860 },
	{ 0xdc1c, 0xdc1c, PDF_CMAP_SINGLE, 15624 },
	{ 0xdc1d, 0xdc1d, PDF_CMAP_SINGLE, 15310 },
	{ 0xdc1d, 0xdc1d, PDF_CMAP_SINGLE, 14561 },
	{ 0xdc1d, 0xdc1e, PDF_CMAP_RANGE, 17924 },
	{ 0xdc1e, 0xdc1e, PDF_CMAP_SINGLE, 15030 },
	{ 0xdc1f, 0xdc1f, PDF_CMAP_SINGLE, 16432 },
	{ 0xdc20, 0xdc20, PDF_CMAP_SINGLE, 14828 },
	{ 0xdc20, 0xdc20, PDF_CMAP_SINGLE, 15886 },
	{ 0xdc20, 0xdc20, PDF_CMAP_SINGLE, 18151 },
	{ 0xdc21, 0xdc21, PDF_CMAP_SINGLE, 15861 },
	{ 0xdc21, 0xdc21, PDF_CMAP_SINGLE, 16211 },
	{ 0xdc21, 0xdc21, PDF_CMAP_SINGLE, 19091 },
	{ 0xdc21, 0xdc21, PDF_CMAP_SINGLE, 14754 },
	{ 0xdc22, 0xdc22, PDF_CMAP_SINGLE, 15244 },
	{ 0xdc23, 0xdc23, PDF_CMAP_SINGLE, 17030 },
	{ 0xdc23, 0xdc23, PDF_CMAP_SINGLE, 18577 },
	{ 0xdc23, 0xdc23, PDF_CMAP_SINGLE, 15066 },
	{ 0xdc24, 0xdc24, PDF_CMAP_SINGLE, 15539 },
	{ 0xdc24, 0xdc24, PDF_CMAP_SINGLE, 15960 },
	{ 0xdc25, 0xdc25, PDF_CMAP_SINGLE, 14976 },
	{ 0xdc25, 0xdc25, PDF_CMAP_SINGLE, 15704 },
	{ 0xdc26, 0xdc26, PDF_CMAP_SINGLE, 15511 },
	{ 0xdc26, 0xdc26, PDF_CMAP_SINGLE, 15346 },
	{ 0xdc27, 0xdc27, PDF_CMAP_SINGLE, 14350 },
	{ 0xdc28, 0xdc28, PDF_CMAP_SINGLE, 16630 },
	{ 0xdc28, 0xdc28, PDF_CMAP_SINGLE, 17316 },
	{ 0xdc29, 0xdc29, PDF_CMAP_SINGLE, 16812 },
	{ 0xdc29, 0xdc29, PDF_CMAP_SINGLE, 14193 },
	{ 0xdc29, 0xdc29, PDF_CMAP_SINGLE, 16486 },
	{ 0xdc2a, 0xdc2a, PDF_CMAP_SINGLE, 14979 },
	{ 0xdc2a, 0xdc2a, PDF_CMAP_SINGLE, 18455 },
	{ 0xdc2b, 0xdc2b, PDF_CMAP_SINGLE, 17031 },
	{ 0xdc2b, 0xdc2b, PDF_CMAP_SINGLE, 14655 },
	{ 0xdc2c, 0xdc2c, PDF_CMAP_SINGLE, 15200 },
	{ 0xdc2c, 0xdc2c, PDF_CMAP_SINGLE, 18516 },
	{ 0xdc2d, 0xdc2d, PDF_CMAP_SINGLE, 18451 },
	{ 0xdc2f, 0xdc2f, PDF_CMAP_SINGLE, 14711 },
	{ 0xdc30, 0xdc30, PDF_CMAP_SINGLE, 14392 },
	{ 0xdc30, 0xdc30, PDF_CMAP_SINGLE, 15925 },
	{ 0xdc32, 0xdc32, PDF_CMAP_SINGLE, 16510 },
	{ 0xdc33, 0xdc33, PDF_CMAP_SINGLE, 18488 },
	{ 0xdc33, 0xdc33, PDF_CMAP_SINGLE, 14558 },
	{ 0xdc34, 0xdc34, PDF_CMAP_SINGLE, 17291 },
	{ 0xdc34, 0xdc34, PDF_CMAP_SINGLE, 16113 },
	{ 0xdc35, 0xdc35, PDF_CMAP_SINGLE, 16564 },
	{ 0xdc36, 0xdc36, PDF_CMAP_SINGLE, 14768 },
	{ 0xdc38, 0xdc38, PDF_CMAP_SINGLE, 17176 },
	{ 0xdc39, 0xdc39, PDF_CMAP_SINGLE, 14715 },
	{ 0xdc39, 0xdc39, PDF_CMAP_SINGLE, 16971 },
	{ 0xdc39, 0xdc39, PDF_CMAP_SINGLE, 14492 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 17038 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 17941 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 14998 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 16562 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 18415 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 18728 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 15786 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 15311 },
	{ 0xdc3b, 0xdc3b, PDF_CMAP_SINGLE, 18411 },
	{ 0xdc3c, 0xdc3c, PDF_CMAP_SINGLE, 15039 },
	{ 0xdc3d, 0xdc3d, PDF_CMAP_SINGLE, 16957 },
	{ 0xdc3e, 0xdc3e, PDF_CMAP_SINGLE, 14929 },
	{ 0xdc3f, 0xdc3f, PDF_CMAP_SINGLE, 18943 },
	{ 0xdc3f, 0xdc3f, PDF_CMAP_SINGLE, 15209 },
	{ 0xdc3f, 0xdc3f, PDF_CMAP_SINGLE, 14966 },
	{ 0xdc40, 0xdc40, PDF_CMAP_SINGLE, 15235 },
	{ 0xdc40, 0xdc40, PDF_CMAP_SINGLE, 15490 },
	{ 0xdc40, 0xdc40, PDF_CMAP_SINGLE, 18727 },
	{ 0xdc41, 0xdc41, PDF_CMAP_SINGLE, 16012 },
	{ 0xdc41, 0xdc41, PDF_CMAP_SINGLE, 19048 },
	{ 0xdc42, 0xdc42, PDF_CMAP_SINGLE, 17163 },
	{ 0xdc43, 0xdc43, PDF_CMAP_SINGLE, 17228 },
	{ 0xdc44, 0xdc44, PDF_CMAP_SINGLE, 17024 },
	{ 0xdc45, 0xdc45, PDF_CMAP_SINGLE, 15275 },
	{ 0xdc45, 0xdc45, PDF_CMAP_SINGLE, 18462 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 14930 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 18520 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 18750 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 19069 },
	{ 0xdc48, 0xdc48, PDF_CMAP_SINGLE, 16071 },
	{ 0xdc48, 0xdc48, PDF_CMAP_SINGLE, 18578 },
	{ 0xdc48, 0xdc48, PDF_CMAP_SINGLE, 15268 },
	{ 0xdc49, 0xdc49, PDF_CMAP_SINGLE, 16688 },
	{ 0xdc4a, 0xdc4a, PDF_CMAP_SINGLE, 18652 },
	{ 0xdc4a, 0xdc4a, PDF_CMAP_SINGLE, 18537 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 16950 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 18559 },
	{ 0xdc4c, 0xdc4c, PDF_CMAP_SINGLE, 18477 },
	{ 0xdc4e, 0xdc4e, PDF_CMAP_SINGLE, 15193 },
	{ 0xdc4f, 0xdc4f, PDF_CMAP_SINGLE, 18434 },
	{ 0xdc50, 0xdc50, PDF_CMAP_SINGLE, 14944 },
	{ 0xdc50, 0xdc50, PDF_CMAP_SINGLE, 18566 },
	{ 0xdc51, 0xdc51, PDF_CMAP_SINGLE, 17294 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 15629 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 15570 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 17227 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 14737 },
	{ 0xdc53, 0xdc53, PDF_CMAP_SINGLE, 18416 },
	{ 0xdc53, 0xdc53, PDF_CMAP_SINGLE, 15662 },
	{ 0xdc54, 0xdc54, PDF_CMAP_SINGLE, 16594 },
	{ 0xdc55, 0xdc55, PDF_CMAP_SINGLE, 17919 },
	{ 0xdc55, 0xdc55, PDF_CMAP_SINGLE, 17886 },
	{ 0xdc57, 0xdc57, PDF_CMAP_SINGLE, 14192 },
	{ 0xdc57, 0xdc57, PDF_CMAP_SINGLE, 17149 },
	{ 0xdc57, 0xdc57, PDF_CMAP_SINGLE, 14589 },
	{ 0xdc58, 0xdc58, PDF_CMAP_SINGLE, 18569 },
	{ 0xdc5a, 0xdc5a, PDF_CMAP_SINGLE, 17003 },
	{ 0xdc5b, 0xdc5b, PDF_CMAP_SINGLE, 15685 },
	{ 0xdc5b, 0xdc5b, PDF_CMAP_SINGLE, 18111 },
	{ 0xdc5c, 0xdc5c, PDF_CMAP_SINGLE, 18433 },
	{ 0xdc5d, 0xdc5d, PDF_CMAP_SINGLE, 18527 },
	{ 0xdc5e, 0xdc5e, PDF_CMAP_SINGLE, 14251 },
	{ 0xdc5e, 0xdc5e, PDF_CMAP_SINGLE, 18548 },
	{ 0xdc5f, 0xdc5f, PDF_CMAP_SINGLE, 16765 },
	{ 0xdc60, 0xdc60, PDF_CMAP_SINGLE, 14920 },
	{ 0xdc61, 0xdc61, PDF_CMAP_SINGLE, 14870 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 15587 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 16016 },
	{ 0xdc63, 0xdc63, PDF_CMAP_SINGLE, 14616 },
	{ 0xdc63, 0xdc63, PDF_CMAP_SINGLE, 18914 },
	{ 0xdc64, 0xdc64, PDF_CMAP_SINGLE, 15100 },
	{ 0xdc64, 0xdc64, PDF_CMAP_SINGLE, 15449 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 14770 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 18417 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 18696 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 18074 },
	{ 0xdc66, 0xdc66, PDF_CMAP_SINGLE, 18489 },
	{ 0xdc68, 0xdc68, PDF_CMAP_SINGLE, 14218 },
	{ 0xdc68, 0xdc68, PDF_CMAP_SINGLE, 18589 },
	{ 0xdc69, 0xdc69, PDF_CMAP_SINGLE, 15860 },
	{ 0xdc6c, 0xdc6c, PDF_CMAP_SINGLE, 14555 },
	{ 0xdc6c, 0xdc6c, PDF_CMAP_SINGLE, 18991 },
	{ 0xdc6c, 0xdc6c, PDF_CMAP_SINGLE, 17801 },
	{ 0xdc6e, 0xdc6e, PDF_CMAP_SINGLE, 18521 },
	{ 0xdc6e, 0xdc6e, PDF_CMAP_SINGLE, 18547 },
	{ 0xdc6f, 0xdc6f, PDF_CMAP_SINGLE, 17853 },
	{ 0xdc6f, 0xdc6f, PDF_CMAP_SINGLE, 16855 },
	{ 0xdc70, 0xdc70, PDF_CMAP_SINGLE, 18456 },
	{ 0xdc70, 0xdc70, PDF_CMAP_SINGLE, 14990 },
	{ 0xdc71, 0xdc71, PDF_CMAP_SINGLE, 16721 },
	{ 0xdc71, 0xdc71, PDF_CMAP_SINGLE, 18469 },
	{ 0xdc72, 0xdc72, PDF_CMAP_SINGLE, 14908 },
	{ 0xdc72, 0xdc72, PDF_CMAP_SINGLE, 14996 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 14822 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 14702 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 15882 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 18851 },
	{ 0xdc75, 0xdc75, PDF_CMAP_SINGLE, 16021 },
	{ 0xdc76, 0xdc76, PDF_CMAP_SINGLE, 18435 },
	{ 0xdc77, 0xdc77, PDF_CMAP_SINGLE, 16535 },
	{ 0xdc77, 0xdc77, PDF_CMAP_SINGLE, 17852 },
	{ 0xdc77, 0xdc77, PDF_CMAP_SINGLE, 18767 },
	{ 0xdc78, 0xdc78, PDF_CMAP_SINGLE, 15804 },
	{ 0xdc78, 0xdc78, PDF_CMAP_SINGLE, 17916 },
	{ 0xdc78, 0xdc78, PDF_CMAP_SINGLE, 17634 },
	{ 0xdc79, 0xdc79, PDF_CMAP_SINGLE, 19062 },
	{ 0xdc7b, 0xdc7b, PDF_CMAP_SINGLE, 17328 },
	{ 0xdc7b, 0xdc7b, PDF_CMAP_SINGLE, 18651 },
	{ 0xdc7c, 0xdc7c, PDF_CMAP_SINGLE, 18418 },
	{ 0xdc7d, 0xdc7d, PDF_CMAP_SINGLE, 14611 },
	{ 0xdc7e, 0xdc7e, PDF_CMAP_SINGLE, 18889 },
	{ 0xdc7e, 0xdc7e, PDF_CMAP_SINGLE, 18772 },
	{ 0xdc7e, 0xdc7e, PDF_CMAP_SINGLE, 16590 },
	{ 0xdc7f, 0xdc7f, PDF_CMAP_SINGLE, 14159 },
	{ 0xdc80, 0xdc80, PDF_CMAP_SINGLE, 15661 },
	{ 0xdc81, 0xdc81, PDF_CMAP_SINGLE, 16622 },
	{ 0xdc82, 0xdc82, PDF_CMAP_SINGLE, 14173 },
	{ 0xdc82, 0xdc82, PDF_CMAP_SINGLE, 15654 },
	{ 0xdc82, 0xdc82, PDF_CMAP_SINGLE, 16706 },
	{ 0xdc83, 0xdc83, PDF_CMAP_SINGLE, 15578 },
	{ 0xdc83, 0xdc83, PDF_CMAP_SINGLE, 18784 },
	{ 0xdc83, 0xdc83, PDF_CMAP_SINGLE, 18579 },
	{ 0xdc84, 0xdc84, PDF_CMAP_SINGLE, 14808 },
	{ 0xdc84, 0xdc84, PDF_CMAP_SINGLE, 18538 },
	{ 0xdc84, 0xdc84, PDF_CMAP_SINGLE, 15579 },
	{ 0xdc85, 0xdc85, PDF_CMAP_SINGLE, 14710 },
	{ 0xdc85, 0xdc85, PDF_CMAP_SINGLE, 16934 },
	{ 0xdc86, 0xdc86, PDF_CMAP_SINGLE, 17636 },
	{ 0xdc86, 0xdc86, PDF_CMAP_SINGLE, 16356 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 17688 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 16441 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 15011 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 14229 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 18436 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 18478 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 15499 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 18539 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 16719 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 14335 },
	{ 0xdc88, 0xdc88, PDF_CMAP_SINGLE, 14926 },
	{ 0xdc89, 0xdc89, PDF_CMAP_SINGLE, 14223 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 14002 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 14249 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 16044 },
	{ 0xdc8b, 0xdc8b, PDF_CMAP_SINGLE, 16905 },
	{ 0xdc8b, 0xdc8b, PDF_CMAP_SINGLE, 14752 },
	{ 0xdc8b, 0xdc8b, PDF_CMAP_SINGLE, 14668 },
	{ 0xdc8c, 0xdc8c, PDF_CMAP_SINGLE, 18771 },
	{ 0xdc8d, 0xdc8d, PDF_CMAP_SINGLE, 18419 },
	{ 0xdc8d, 0xdc8d, PDF_CMAP_SINGLE, 14841 },
	{ 0xdc8d, 0xdc8d, PDF_CMAP_SINGLE, 14142 },
	{ 0xdc8e, 0xdc8e, PDF_CMAP_SINGLE, 16101 },
	{ 0xdc8e, 0xdc8e, PDF_CMAP_SINGLE, 14959 },
	{ 0xdc90, 0xdc90, PDF_CMAP_SINGLE, 18580 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 14816 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 16923 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 16969 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 14320 },
	{ 0xdc92, 0xdc92, PDF_CMAP_SINGLE, 14815 },
	{ 0xdc93, 0xdc93, PDF_CMAP_SINGLE, 18666 },
	{ 0xdc93, 0xdc93, PDF_CMAP_SINGLE, 15341 },
	{ 0xdc94, 0xdc94, PDF_CMAP_SINGLE, 18396 },
	{ 0xdc94, 0xdc94, PDF_CMAP_SINGLE, 16715 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 16013 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 18437 },
	{ 0xdc97, 0xdc97, PDF_CMAP_SINGLE, 16956 },
	{ 0xdc97, 0xdc97, PDF_CMAP_SINGLE, 17242 },
	{ 0xdc98, 0xdc98, PDF_CMAP_SINGLE, 14247 },
	{ 0xdc99, 0xdc99, PDF_CMAP_SINGLE, 16455 },
	{ 0xdc9a, 0xdc9a, PDF_CMAP_SINGLE, 15782 },
	{ 0xdc9a, 0xdc9a, PDF_CMAP_SINGLE, 15024 },
	{ 0xdc9a, 0xdc9a, PDF_CMAP_SINGLE, 19075 },
	{ 0xdc9b, 0xdc9b, PDF_CMAP_SINGLE, 14935 },
	{ 0xdc9b, 0xdc9b, PDF_CMAP_SINGLE, 14478 },
	{ 0xdc9c, 0xdc9c, PDF_CMAP_SINGLE, 18629 },
	{ 0xdc9d, 0xdc9d, PDF_CMAP_SINGLE, 17251 },
	{ 0xdc9d, 0xdc9d, PDF_CMAP_SINGLE, 17115 },
	{ 0xdc9d, 0xdc9d, PDF_CMAP_SINGLE, 15957 },
	{ 0xdc9e, 0xdc9e, PDF_CMAP_SINGLE, 17076 },
	{ 0xdc9e, 0xdc9e, PDF_CMAP_SINGLE, 18649 },
	{ 0xdc9f, 0xdc9f, PDF_CMAP_SINGLE, 15527 },
	{ 0xdc9f, 0xdc9f, PDF_CMAP_SINGLE, 15645 },
	{ 0xdca0, 0xdca0, PDF_CMAP_SINGLE, 14901 },
	{ 0xdca1, 0xdca1, PDF_CMAP_SINGLE, 15237 },
	{ 0xdca1, 0xdca1, PDF_CMAP_SINGLE, 14954 },
	{ 0xdca2, 0xdca2, PDF_CMAP_SINGLE, 14343 },
	{ 0xdca2, 0xdca2, PDF_CMAP_SINGLE, 18677 },
	{ 0xdca3, 0xdca3, PDF_CMAP_SINGLE, 14814 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 18924 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 15518 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 18557 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 15893 },
	{ 0xdca5, 0xdca5, PDF_CMAP_SINGLE, 16669 },
	{ 0xdca5, 0xdca5, PDF_CMAP_SINGLE, 17850 },
	{ 0xdca6, 0xdca6, PDF_CMAP_SINGLE, 16757 },
	{ 0xdca9, 0xdca9, PDF_CMAP_SINGLE, 17192 },
	{ 0xdcac, 0xdcac, PDF_CMAP_SINGLE, 18457 },
	{ 0xdcad, 0xdcad, PDF_CMAP_SINGLE, 17057 },
	{ 0xdcad, 0xdcad, PDF_CMAP_SINGLE, 18745 },
	{ 0xdcad, 0xdcad, PDF_CMAP_SINGLE, 15892 },
	{ 0xdcae, 0xdcae, PDF_CMAP_SINGLE, 18710 },
	{ 0xdcaf, 0xdcaf, PDF_CMAP_SINGLE, 15906 },
	{ 0xdcaf, 0xdcaf, PDF_CMAP_SINGLE, 14523 },
	{ 0xdcb0, 0xdcb0, PDF_CMAP_SINGLE, 18632 },
	{ 0xdcb0, 0xdcb0, PDF_CMAP_SINGLE, 15338 },
	{ 0xdcb1, 0xdcb1, PDF_CMAP_SINGLE, 18973 },
	{ 0xdcb1, 0xdcb1, PDF_CMAP_SINGLE, 14598 },
	{ 0xdcb2, 0xdcb2, PDF_CMAP_SINGLE, 17278 },
	{ 0xdcb2, 0xdcb2, PDF_CMAP_SINGLE, 17668 },
	{ 0xdcb4, 0xdcb4, PDF_CMAP_SINGLE, 16566 },
	{ 0xdcb5, 0xdcb5, PDF_CMAP_SINGLE, 18420 },
	{ 0xdcb5, 0xdcb5, PDF_CMAP_SINGLE, 18085 },
	{ 0xdcb6, 0xdcb6, PDF_CMAP_SINGLE, 14727 },
	{ 0xdcb7, 0xdcb7, PDF_CMAP_SINGLE, 18479 },
	{ 0xdcb7, 0xdcb7, PDF_CMAP_SINGLE, 14546 },
	{ 0xdcb7, 0xdcb7, PDF_CMAP_SINGLE, 15963 },
	{ 0xdcb7, 0xdcb7, PDF_CMAP_SINGLE, 15648 },
	{ 0xdcb8, 0xdcb8, PDF_CMAP_SINGLE, 15563 },
	{ 0xdcb8, 0xdcb8, PDF_CMAP_SINGLE, 14520 },
	{ 0xdcb9, 0xdcb9, PDF_CMAP_SINGLE, 14696 },
	{ 0xdcb9, 0xdcb9, PDF_CMAP_SINGLE, 14639 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 19141 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 14547 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 16697 },
	{ 0xdcbd, 0xdcbd, PDF_CMAP_SINGLE, 15849 },
	{ 0xdcbd, 0xdcbd, PDF_CMAP_SINGLE, 17902 },
	{ 0xdcbe, 0xdcbe, PDF_CMAP_SINGLE, 14707 },
	{ 0xdcbe, 0xdcbe, PDF_CMAP_SINGLE, 17174 },
	{ 0xdcbf, 0xdcbf, PDF_CMAP_SINGLE, 18439 },
	{ 0xdcbf, 0xdcbf, PDF_CMAP_SINGLE, 15577 },
	{ 0xdcc0, 0xdcc0, PDF_CMAP_SINGLE, 17877 },
	{ 0xdcc0, 0xdcc0, PDF_CMAP_SINGLE, 14322 },
	{ 0xdcc0, 0xdcc0, PDF_CMAP_SINGLE, 15508 },
	{ 0xdcc0, 0xdcc0, PDF_CMAP_SINGLE, 15854 },
	{ 0xdcc1, 0xdcc1, PDF_CMAP_SINGLE, 18663 },
	{ 0xdcc1, 0xdcc1, PDF_CMAP_SINGLE, 19143 },
	{ 0xdcc1, 0xdcc1, PDF_CMAP_SINGLE, 16772 },
	{ 0xdcc1, 0xdcc1, PDF_CMAP_SINGLE, 14170 },
	{ 0xdcc2, 0xdcc2, PDF_CMAP_SINGLE, 17201 },
	{ 0xdcc3, 0xdcc3, PDF_CMAP_SINGLE, 14230 },
	{ 0xdcc5, 0xdcc5, PDF_CMAP_SINGLE, 18975 },
	{ 0xdcc6, 0xdcc6, PDF_CMAP_SINGLE, 17167 },
	{ 0xdcc7, 0xdcc7, PDF_CMAP_SINGLE, 17296 },
	{ 0xdcc7, 0xdcc7, PDF_CMAP_SINGLE, 17275 },
	{ 0xdcc7, 0xdcc7, PDF_CMAP_SINGLE, 18738 },
	{ 0xdcc7, 0xdcc7, PDF_CMAP_SINGLE, 18549 },
	{ 0xdcc8, 0xdcc8, PDF_CMAP_SINGLE, 17268 },
	{ 0xdcc8, 0xdcc8, PDF_CMAP_SINGLE, 16409 },
	{ 0xdcc8, 0xdcc8, PDF_CMAP_SINGLE, 15386 },
	{ 0xdcc8, 0xdcc8, PDF_CMAP_SINGLE, 17849 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 17913 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 19144 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 17857 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 16917 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 14845 },
	{ 0xdcc9, 0xdcc9, PDF_CMAP_SINGLE, 18093 },
	{ 0xdcca, 0xdcca, PDF_CMAP_SINGLE, 17623 },
	{ 0xdcca, 0xdcca, PDF_CMAP_SINGLE, 17300 },
	{ 0xdccb, 0xdccb, PDF_CMAP_SINGLE, 17620 },
	{ 0xdccc, 0xdccc, PDF_CMAP_SINGLE, 14000 },
	{ 0xdccc, 0xdccc, PDF_CMAP_SINGLE, 19145 },
	{ 0xdccd, 0xdccd, PDF_CMAP_SINGLE, 17617 },
	{ 0xdccd, 0xdccd, PDF_CMAP_SINGLE, 14338 },
	{ 0xdccd, 0xdccd, PDF_CMAP_SINGLE, 17289 },
	{ 0xdccd, 0xdccd, PDF_CMAP_SINGLE, 15842 },
	{ 0xdcce, 0xdcce, PDF_CMAP_SINGLE, 15693 },
	{ 0xdccf, 0xdccf, PDF_CMAP_SINGLE, 18228 },
	{ 0xdccf, 0xdccf, PDF_CMAP_SINGLE, 17861 },
	{ 0xdcd0, 0xdcd0, PDF_CMAP_SINGLE, 16132 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 17616 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 14718 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 15216 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 18611 },
	{ 0xdcd2, 0xdcd2, PDF_CMAP_SINGLE, 18985 },
	{ 0xdcd2, 0xdcd2, PDF_CMAP_SINGLE, 17298 },
	{ 0xdcd3, 0xdcd3, PDF_CMAP_SINGLE, 16698 },
	{ 0xdcd3, 0xdcd3, PDF_CMAP_SINGLE, 17330 },
	{ 0xdcd3, 0xdcd3, PDF_CMAP_SINGLE, 17194 },
	{ 0xdcd4, 0xdcd4, PDF_CMAP_SINGLE, 16216 },
	{ 0xdcd5, 0xdcd5, PDF_CMAP_SINGLE, 18141 },
	{ 0xdcd5, 0xdcd5, PDF_CMAP_SINGLE, 17185 },
	{ 0xdcd6, 0xdcd6, PDF_CMAP_SINGLE, 17910 },
	{ 0xdcd6, 0xdcd6, PDF_CMAP_SINGLE, 15505 },
	{ 0xdcd6, 0xdcd6, PDF_CMAP_SINGLE, 18506 },
	{ 0xdcd7, 0xdcd7, PDF_CMAP_SINGLE, 18400 },
	{ 0xdcd7, 0xdcd7, PDF_CMAP_SINGLE, 16561 },
	{ 0xdcd8, 0xdcd8, PDF_CMAP_SINGLE, 15239 },
	{ 0xdcd9, 0xdcd9, PDF_CMAP_SINGLE, 14965 },
	{ 0xdcd9, 0xdcd9, PDF_CMAP_SINGLE, 15079 },
	{ 0xdcd9, 0xdcd9, PDF_CMAP_SINGLE, 14133 },
	{ 0xdcda, 0xdcda, PDF_CMAP_SINGLE, 18706 },
	{ 0xdcda, 0xdcda, PDF_CMAP_SINGLE, 15363 },
	{ 0xdcdb, 0xdcdb, PDF_CMAP_SINGLE, 16841 },
	{ 0xdcdd, 0xdcdd, PDF_CMAP_SINGLE, 18421 },
	{ 0xdcde, 0xdcde, PDF_CMAP_SINGLE, 19093 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14751 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14132 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14626 },
	{ 0xdce4, 0xdce4, PDF_CMAP_SINGLE, 14830 },
	{ 0xdce4, 0xdce4, PDF_CMAP_SINGLE, 15601 },
	{ 0xdce5, 0xdce5, PDF_CMAP_SINGLE, 15639 },
	{ 0xdce5, 0xdce5, PDF_CMAP_SINGLE, 14524 },
	{ 0xdce5, 0xdce5, PDF_CMAP_SINGLE, 17172 },
	{ 0xdce7, 0xdce7, PDF_CMAP_SINGLE, 18607 },
	{ 0xdce8, 0xdce8, PDF_CMAP_SINGLE, 15402 },
	{ 0xdce8, 0xdce8, PDF_CMAP_SINGLE, 15899 },
	{ 0xdce9, 0xdce9, PDF_CMAP_SINGLE, 17653 },
	{ 0xdce9, 0xdce9, PDF_CMAP_SINGLE, 17198 },
	{ 0xdcea, 0xdcea, PDF_CMAP_SINGLE, 14945 },
	{ 0xdcec, 0xdcec, PDF_CMAP_SINGLE, 19042 },
	{ 0xdcec, 0xdcec, PDF_CMAP_SINGLE, 18600 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 18422 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 16419 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 16845 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 15432 },
	{ 0xdcee, 0xdcee, PDF_CMAP_SINGLE, 18749 },
	{ 0xdcf0, 0xdcf0, PDF_CMAP_SINGLE, 15112 },
	{ 0xdcf1, 0xdcf1, PDF_CMAP_SINGLE, 16906 },
	{ 0xdcf1, 0xdcf1, PDF_CMAP_SINGLE, 15391 },
	{ 0xdcf2, 0xdcf2, PDF_CMAP_SINGLE, 18159 },
	{ 0xdcf3, 0xdcf3, PDF_CMAP_SINGLE, 15095 },
	{ 0xdcf3, 0xdcf3, PDF_CMAP_SINGLE, 15433 },
	{ 0xdcf4, 0xdcf4, PDF_CMAP_SINGLE, 17847 },
	{ 0xdcf4, 0xdcf4, PDF_CMAP_SINGLE, 18581 },
	{ 0xdcf5, 0xdcf6, PDF_CMAP_RANGE, 17926 },
	{ 0xdcf7, 0xdcf7, PDF_CMAP_SINGLE, 16834 },
	{ 0xdcf8, 0xdcf8, PDF_CMAP_SINGLE, 16732 },
	{ 0xdcf8, 0xdcf8, PDF_CMAP_SINGLE, 14348 },
	{ 0xdcfa, 0xdcfa, PDF_CMAP_SINGLE, 14685 },
	{ 0xdcfa, 0xdcfa, PDF_CMAP_SINGLE, 19055 },
	{ 0xdcfb, 0xdcfb, PDF_CMAP_SINGLE, 18511 },
	{ 0xdcfc, 0xdcfc, PDF_CMAP_SINGLE, 17241 },
	{ 0xdcfc, 0xdcfc, PDF_CMAP_SINGLE, 16446 },
	{ 0xdcfc, 0xdcfc, PDF_CMAP_SINGLE, 18674 },
	{ 0xdcfd, 0xdcfd, PDF_CMAP_SINGLE, 14260 },
	{ 0xdcfd, 0xdcfd, PDF_CMAP_SINGLE, 18720 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 15548 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 18697 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 14154 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 15424 },
	{ 0xdcff, 0xdcff, PDF_CMAP_SINGLE, 15559 },
	{ 0xdcff, 0xdcff, PDF_CMAP_SINGLE, 15794 },
	{ 0xdcff, 0xdcff, PDF_CMAP_SINGLE, 15321 },
	{ 0xdd00, 0xdd00, PDF_CMAP_SINGLE, 16916 },
	{ 0xdd01, 0xdd01, PDF_CMAP_SINGLE, 16687 },
	{ 0xdd01, 0xdd01, PDF_CMAP_SINGLE, 18097 },
	{ 0xdd02, 0xdd02, PDF_CMAP_SINGLE, 18534 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 14978 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 18528 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 18743 },
	{ 0xdd03, 0xdd04, PDF_CMAP_RANGE, 14898 },
	{ 0xdd04, 0xdd04, PDF_CMAP_SINGLE, 15427 },
	{ 0xdd05, 0xdd05, PDF_CMAP_SINGLE, 15669 },
	{ 0xdd05, 0xdd05, PDF_CMAP_SINGLE, 18917 },
	{ 0xdd05, 0xdd05, PDF_CMAP_SINGLE, 14590 },
	{ 0xdd06, 0xdd06, PDF_CMAP_SINGLE, 15080 },
	{ 0xdd07, 0xdd07, PDF_CMAP_SINGLE, 14936 },
	{ 0xdd07, 0xdd07, PDF_CMAP_SINGLE, 18870 },
	{ 0xdd08, 0xdd08, PDF_CMAP_SINGLE, 18481 },
	{ 0xdd08, 0xdd08, PDF_CMAP_SINGLE, 16943 },
	{ 0xdd09, 0xdd09, PDF_CMAP_SINGLE, 18127 },
	{ 0xdd0c, 0xdd0c, PDF_CMAP_SINGLE, 17614 },
	{ 0xdd0c, 0xdd0c, PDF_CMAP_SINGLE, 16915 },
	{ 0xdd0c, 0xdd0c, PDF_CMAP_SINGLE, 14428 },
	{ 0xdd0d, 0xdd0d, PDF_CMAP_SINGLE, 18954 },
	{ 0xdd0d, 0xdd0d, PDF_CMAP_SINGLE, 19098 },
	{ 0xdd0d, 0xdd0d, PDF_CMAP_SINGLE, 17158 },
	{ 0xdd0e, 0xdd0e, PDF_CMAP_SINGLE, 17628 },
	{ 0xdd0e, 0xdd0e, PDF_CMAP_SINGLE, 17108 },
	{ 0xdd0f, 0xdd0f, PDF_CMAP_SINGLE, 14753 },
	{ 0xdd10, 0xdd10, PDF_CMAP_SINGLE, 14416 },
	{ 0xdd10, 0xdd10, PDF_CMAP_SINGLE, 18601 },
	{ 0xdd11, 0xdd11, PDF_CMAP_SINGLE, 16645 },
	{ 0xdd11, 0xdd11, PDF_CMAP_SINGLE, 17107 },
	{ 0xdd12, 0xdd12, PDF_CMAP_SINGLE, 15016 },
	{ 0xdd12, 0xdd12, PDF_CMAP_SINGLE, 18482 },
	{ 0xdd12, 0xdd12, PDF_CMAP_SINGLE, 18540 },
	{ 0xdd13, 0xdd13, PDF_CMAP_SINGLE, 16425 },
	{ 0xdd13, 0xdd13, PDF_CMAP_SINGLE, 17092 },
	{ 0xdd15, 0xdd15, PDF_CMAP_SINGLE, 17164 },
	{ 0xdd15, 0xdd15, PDF_CMAP_SINGLE, 14526 },
	{ 0xdd16, 0xdd16, PDF_CMAP_SINGLE, 15191 },
	{ 0xdd16, 0xdd16, PDF_CMAP_SINGLE, 15542 },
	{ 0xdd17, 0xdd17, PDF_CMAP_SINGLE, 15061 },
	{ 0xdd18, 0xdd18, PDF_CMAP_SINGLE, 16777 },
	{ 0xdd19, 0xdd19, PDF_CMAP_SINGLE, 15864 },
	{ 0xdd19, 0xdd19, PDF_CMAP_SINGLE, 18500 },
	{ 0xdd19, 0xdd19, PDF_CMAP_SINGLE, 18721 },
	{ 0xdd1b, 0xdd1b, PDF_CMAP_SINGLE, 16294 },
	{ 0xdd1c, 0xdd1c, PDF_CMAP_SINGLE, 15832 },
	{ 0xdd20, 0xdd20, PDF_CMAP_SINGLE, 14164 },
	{ 0xdd21, 0xdd21, PDF_CMAP_SINGLE, 15844 },
	{ 0xdd21, 0xdd21, PDF_CMAP_SINGLE, 14201 },
	{ 0xdd22, 0xdd22, PDF_CMAP_SINGLE, 14747 },
	{ 0xdd22, 0xdd22, PDF_CMAP_SINGLE, 18713 },
	{ 0xdd22, 0xdd22, PDF_CMAP_SINGLE, 16392 },
	{ 0xdd23, 0xdd23, PDF_CMAP_SINGLE, 16889 },
	{ 0xdd23, 0xdd23, PDF_CMAP_SINGLE, 18718 },
	{ 0xdd23, 0xdd23, PDF_CMAP_SINGLE, 14627 },
	{ 0xdd24, 0xdd24, PDF_CMAP_SINGLE, 14986 },
	{ 0xdd24, 0xdd24, PDF_CMAP_SINGLE, 14378 },
	{ 0xdd25, 0xdd25, PDF_CMAP_SINGLE, 16926 },
	{ 0xdd26, 0xdd26, PDF_CMAP_SINGLE, 15881 },
	{ 0xdd26, 0xdd26, PDF_CMAP_SINGLE, 18550 },
	{ 0xdd26, 0xdd26, PDF_CMAP_SINGLE, 15652 },
	{ 0xdd26, 0xdd26, PDF_CMAP_SINGLE, 17155 },
	{ 0xdd27, 0xdd27, PDF_CMAP_SINGLE, 14912 },
	{ 0xdd27, 0xdd27, PDF_CMAP_SINGLE, 14579 },
	{ 0xdd27, 0xdd27, PDF_CMAP_SINGLE, 15215 },
	{ 0xdd28, 0xdd28, PDF_CMAP_SINGLE, 16135 },
	{ 0xdd28, 0xdd28, PDF_CMAP_SINGLE, 15691 },
	{ 0xdd29, 0xdd29, PDF_CMAP_SINGLE, 16421 },
	{ 0xdd2a, 0xdd2a, PDF_CMAP_SINGLE, 16879 },
	{ 0xdd2a, 0xdd2a, PDF_CMAP_SINGLE, 16623 },
	{ 0xdd2b, 0xdd2b, PDF_CMAP_SINGLE, 17165 },
	{ 0xdd2e, 0xdd2e, PDF_CMAP_SINGLE, 18582 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 18440 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 15299 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 18668 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 15434 },
	{ 0xdd2f, 0xdd2f, PDF_CMAP_SINGLE, 17056 },
	{ 0xdd30, 0xdd30, PDF_CMAP_SINGLE, 16448 },
	{ 0xdd30, 0xdd30, PDF_CMAP_SINGLE, 14265 },
	{ 0xdd30, 0xdd30, PDF_CMAP_SINGLE, 18861 },
	{ 0xdd31, 0xdd31, PDF_CMAP_SINGLE, 14481 },
	{ 0xdd31, 0xdd31, PDF_CMAP_SINGLE, 18741 },
	{ 0xdd32, 0xdd32, PDF_CMAP_SINGLE, 16920 },
	{ 0xdd33, 0xdd33, PDF_CMAP_SINGLE, 16413 },
	{ 0xdd33, 0xdd33, PDF_CMAP_SINGLE, 14470 },
	{ 0xdd33, 0xdd33, PDF_CMAP_SINGLE, 17791 },
	{ 0xdd34, 0xdd34, PDF_CMAP_SINGLE, 15846 },
	{ 0xdd34, 0xdd34, PDF_CMAP_SINGLE, 15318 },
	{ 0xdd34, 0xdd34, PDF_CMAP_SINGLE, 14631 },
	{ 0xdd35, 0xdd35, PDF_CMAP_SINGLE, 18201 },
	{ 0xdd39, 0xdd39, PDF_CMAP_SINGLE, 17863 },
	{ 0xdd39, 0xdd39, PDF_CMAP_SINGLE, 18551 },
	{ 0xdd3b, 0xdd3b, PDF_CMAP_SINGLE, 18441 },
	{ 0xdd3b, 0xdd3b, PDF_CMAP_SINGLE, 14636 },
	{ 0xdd3c, 0xdd3c, PDF_CMAP_SINGLE, 14455 },
	{ 0xdd3c, 0xdd3c, PDF_CMAP_SINGLE, 18602 },
	{ 0xdd3d, 0xdd3d, PDF_CMAP_SINGLE, 17878 },
	{ 0xdd3d, 0xdd3d, PDF_CMAP_SINGLE, 17957 },
	{ 0xdd3d, 0xdd3d, PDF_CMAP_SINGLE, 14445 },
	{ 0xdd3e, 0xdd3e, PDF_CMAP_SINGLE, 18716 },
	{ 0xdd3e, 0xdd3e, PDF_CMAP_SINGLE, 14614 },
	{ 0xdd3f, 0xdd3f, PDF_CMAP_SINGLE, 16974 },
	{ 0xdd3f, 0xdd3f, PDF_CMAP_SINGLE, 15816 },
	{ 0xdd3f, 0xdd3f, PDF_CMAP_SINGLE, 18522 },
	{ 0xdd40, 0xdd40, PDF_CMAP_SINGLE, 15979 },
	{ 0xdd40, 0xdd40, PDF_CMAP_SINGLE, 16978 },
	{ 0xdd40, 0xdd40, PDF_CMAP_SINGLE, 16808 },
	{ 0xdd41, 0xdd41, PDF_CMAP_SINGLE, 15472 },
	{ 0xdd42, 0xdd42, PDF_CMAP_SINGLE, 16404 },
	{ 0xdd43, 0xdd43, PDF_CMAP_SINGLE, 15558 },
	{ 0xdd43, 0xdd43, PDF_CMAP_SINGLE, 15202 },
	{ 0xdd44, 0xdd44, PDF_CMAP_SINGLE, 18742 },
	{ 0xdd44, 0xdd44, PDF_CMAP_SINGLE, 17210 },
	{ 0xdd44, 0xdd44, PDF_CMAP_SINGLE, 16967 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 17717 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 19019 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 17196 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 18732 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 14868 },
	{ 0xdd47, 0xdd47, PDF_CMAP_SINGLE, 16395 },
	{ 0xdd47, 0xdd47, PDF_CMAP_SINGLE, 17871 },
	{ 0xdd47, 0xdd47, PDF_CMAP_SINGLE, 18884 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 17876 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 17917 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 18660 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 15822 },
	{ 0xdd48, 0xdd48, PDF_CMAP_SINGLE, 15393 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 18658 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 15928 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 18612 },
	{ 0xdd4c, 0xdd4c, PDF_CMAP_SINGLE, 15144 },
	{ 0xdd4c, 0xdd4c, PDF_CMAP_SINGLE, 17851 },
	{ 0xdd4d, 0xdd4d, PDF_CMAP_SINGLE, 16551 },
	{ 0xdd4d, 0xdd4d, PDF_CMAP_SINGLE, 14938 },
	{ 0xdd4e, 0xdd4e, PDF_CMAP_SINGLE, 16621 },
	{ 0xdd4e, 0xdd4e, PDF_CMAP_SINGLE, 18855 },
	{ 0xdd4f, 0xdd4f, PDF_CMAP_SINGLE, 17872 },
	{ 0xdd4f, 0xdd4f, PDF_CMAP_SINGLE, 18470 },
	{ 0xdd4f, 0xdd4f, PDF_CMAP_SINGLE, 18583 },
	{ 0xdd50, 0xdd50, PDF_CMAP_SINGLE, 16607 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 19037 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 17664 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 15653 },
	{ 0xdd53, 0xdd53, PDF_CMAP_SINGLE, 19089 },
	{ 0xdd53, 0xdd53, PDF_CMAP_SINGLE, 14410 },
	{ 0xdd54, 0xdd54, PDF_CMAP_SINGLE, 16491 },
	{ 0xdd54, 0xdd54, PDF_CMAP_SINGLE, 16644 },
	{ 0xdd54, 0xdd54, PDF_CMAP_SINGLE, 14406 },
	{ 0xdd55, 0xdd55, PDF_CMAP_SINGLE, 16970 },
	{ 0xdd56, 0xdd56, PDF_CMAP_SINGLE, 18644 },
	{ 0xdd56, 0xdd56, PDF_CMAP_SINGLE, 18592 },
	{ 0xdd56, 0xdd57, PDF_CMAP_RANGE, 16976 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 16897 },
	{ 0xdd59, 0xdd59, PDF_CMAP_SINGLE, 18729 },
	{ 0xdd5a, 0xdd5a, PDF_CMAP_SINGLE, 14279 },
	{ 0xdd5a, 0xdd5a, PDF_CMAP_SINGLE, 14202 },
	{ 0xdd5a, 0xdd5a, PDF_CMAP_SINGLE, 18617 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 18104 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 15582 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 14975 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 15517 },
	{ 0xdd5b, 0xdd5b, PDF_CMAP_SINGLE, 18147 },
	{ 0xdd5c, 0xdd5c, PDF_CMAP_SINGLE, 14572 },
	{ 0xdd5c, 0xdd5c, PDF_CMAP_SINGLE, 14735 },
	{ 0xdd5c, 0xdd5c, PDF_CMAP_SINGLE, 15185 },
	{ 0xdd5d, 0xdd5d, PDF_CMAP_SINGLE, 15465 },
	{ 0xdd5d, 0xdd5d, PDF_CMAP_SINGLE, 17179 },
	{ 0xdd5d, 0xdd5d, PDF_CMAP_SINGLE, 16768 },
	{ 0xdd5e, 0xdd5e, PDF_CMAP_SINGLE, 19010 },
	{ 0xdd5e, 0xdd5e, PDF_CMAP_SINGLE, 18523 },
	{ 0xdd5e, 0xdd5e, PDF_CMAP_SINGLE, 18604 },
	{ 0xdd5f, 0xdd5f, PDF_CMAP_SINGLE, 17071 },
	{ 0xdd61, 0xdd61, PDF_CMAP_SINGLE, 19097 },
	{ 0xdd62, 0xdd62, PDF_CMAP_SINGLE, 18119 },
	{ 0xdd62, 0xdd62, PDF_CMAP_SINGLE, 18524 },
	{ 0xdd63, 0xdd63, PDF_CMAP_SINGLE, 18709 },
	{ 0xdd64, 0xdd64, PDF_CMAP_SINGLE, 18712 },
	{ 0xdd64, 0xdd64, PDF_CMAP_SINGLE, 17126 },
	{ 0xdd65, 0xdd65, PDF_CMAP_SINGLE, 14393 },
	{ 0xdd65, 0xdd65, PDF_CMAP_SINGLE, 14733 },
	{ 0xdd66, 0xdd66, PDF_CMAP_SINGLE, 18525 },
	{ 0xdd66, 0xdd66, PDF_CMAP_SINGLE, 18575 },
	{ 0xdd67, 0xdd67, PDF_CMAP_SINGLE, 18471 },
	{ 0xdd67, 0xdd67, PDF_CMAP_SINGLE, 14952 },
	{ 0xdd67, 0xdd67, PDF_CMAP_SINGLE, 18492 },
	{ 0xdd67, 0xdd67, PDF_CMAP_SINGLE, 18297 },
	{ 0xdd68, 0xdd68, PDF_CMAP_SINGLE, 17127 },
	{ 0xdd6a, 0xdd6a, PDF_CMAP_SINGLE, 18452 },
	{ 0xdd6a, 0xdd6a, PDF_CMAP_SINGLE, 18613 },
	{ 0xdd6b, 0xdd6b, PDF_CMAP_SINGLE, 15412 },
	{ 0xdd6c, 0xdd6c, PDF_CMAP_SINGLE, 14458 },
	{ 0xdd6c, 0xdd6c, PDF_CMAP_SINGLE, 14695 },
	{ 0xdd6d, 0xdd6d, PDF_CMAP_SINGLE, 19059 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 18423 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 15920 },
	{ 0xdd70, 0xdd70, PDF_CMAP_SINGLE, 16964 },
	{ 0xdd71, 0xdd71, PDF_CMAP_SINGLE, 16136 },
	{ 0xdd72, 0xdd72, PDF_CMAP_SINGLE, 17899 },
	{ 0xdd73, 0xdd73, PDF_CMAP_SINGLE, 16689 },
	{ 0xdd73, 0xdd73, PDF_CMAP_SINGLE, 18670 },
	{ 0xdd74, 0xdd74, PDF_CMAP_SINGLE, 16543 },
	{ 0xdd74, 0xdd74, PDF_CMAP_SINGLE, 17019 },
	{ 0xdd74, 0xdd74, PDF_CMAP_SINGLE, 18956 },
	{ 0xdd74, 0xdd74, PDF_CMAP_SINGLE, 17285 },
	{ 0xdd75, 0xdd75, PDF_CMAP_SINGLE, 14677 },
	{ 0xdd75, 0xdd75, PDF_CMAP_SINGLE, 14569 },
	{ 0xdd76, 0xdd76, PDF_CMAP_SINGLE, 15544 },
	{ 0xdd77, 0xdd77, PDF_CMAP_SINGLE, 18446 },
	{ 0xdd77, 0xdd77, PDF_CMAP_SINGLE, 17942 },
	{ 0xdd78, 0xdd78, PDF_CMAP_SINGLE, 14175 },
	{ 0xdd79, 0xdd79, PDF_CMAP_SINGLE, 18695 },
	{ 0xdd7a, 0xdd7a, PDF_CMAP_SINGLE, 15874 },
	{ 0xdd7a, 0xdd7a, PDF_CMAP_SINGLE, 15908 },
	{ 0xdd7a, 0xdd7a, PDF_CMAP_SINGLE, 14562 },
	{ 0xdd7b, 0xdd7b, PDF_CMAP_SINGLE, 18952 },
	{ 0xdd7c, 0xdd7c, PDF_CMAP_SINGLE, 17183 },
	{ 0xdd7c, 0xdd7c, PDF_CMAP_SINGLE, 14894 },
	{ 0xdd7c, 0xdd7c, PDF_CMAP_SINGLE, 16067 },
	{ 0xdd7d, 0xdd7d, PDF_CMAP_SINGLE, 14278 },
	{ 0xdd7d, 0xdd7d, PDF_CMAP_SINGLE, 17116 },
	{ 0xdd7e, 0xdd7e, PDF_CMAP_SINGLE, 14258 },
	{ 0xdd7e, 0xdd7e, PDF_CMAP_SINGLE, 17934 },
	{ 0xdd7e, 0xdd7e, PDF_CMAP_SINGLE, 14687 },
	{ 0xdd7e, 0xdd7f, PDF_CMAP_RANGE, 17884 },
	{ 0xdd7f, 0xdd7f, PDF_CMAP_SINGLE, 16933 },
	{ 0xdd80, 0xdd80, PDF_CMAP_SINGLE, 17937 },
	{ 0xdd80, 0xdd80, PDF_CMAP_SINGLE, 18872 },
	{ 0xdd80, 0xdd80, PDF_CMAP_SINGLE, 16771 },
	{ 0xdd81, 0xdd81, PDF_CMAP_SINGLE, 15122 },
	{ 0xdd82, 0xdd82, PDF_CMAP_SINGLE, 14337 },
	{ 0xdd82, 0xdd82, PDF_CMAP_SINGLE, 14983 },
	{ 0xdd82, 0xdd82, PDF_CMAP_SINGLE, 16749 },
	{ 0xdd83, 0xdd83, PDF_CMAP_SINGLE, 16451 },
	{ 0xdd84, 0xdd84, PDF_CMAP_SINGLE, 14971 },
	{ 0xdd84, 0xdd84, PDF_CMAP_SINGLE, 17845 },
	{ 0xdd87, 0xdd87, PDF_CMAP_SINGLE, 18684 },
	{ 0xdd88, 0xdd88, PDF_CMAP_SINGLE, 17078 },
	{ 0xdd88, 0xdd88, PDF_CMAP_SINGLE, 15930 },
	{ 0xdd89, 0xdd89, PDF_CMAP_SINGLE, 14961 },
	{ 0xdd89, 0xdd89, PDF_CMAP_SINGLE, 17384 },
	{ 0xdd89, 0xdd89, PDF_CMAP_SINGLE, 15345 },
	{ 0xdd8a, 0xdd8a, PDF_CMAP_SINGLE, 14425 },
	{ 0xdd8b, 0xdd8b, PDF_CMAP_SINGLE, 15520 },
	{ 0xdd8c, 0xdd8c, PDF_CMAP_SINGLE, 15145 },
	{ 0xdd8d, 0xdd8d, PDF_CMAP_SINGLE, 15871 },
	{ 0xdd8d, 0xdd8d, PDF_CMAP_SINGLE, 17238 },
	{ 0xdd8d, 0xdd8d, PDF_CMAP_SINGLE, 17048 },
	{ 0xdd8e, 0xdd8e, PDF_CMAP_SINGLE, 16449 },
	{ 0xdd8e, 0xdd8e, PDF_CMAP_SINGLE, 15089 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 15242 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 16750 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 14973 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 14407 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 14263 },
	{ 0xdd90, 0xdd90, PDF_CMAP_SINGLE, 15823 },
	{ 0xdd92, 0xdd92, PDF_CMAP_SINGLE, 14624 },
	{ 0xdd93, 0xdd93, PDF_CMAP_SINGLE, 18472 },
	{ 0xdd93, 0xdd93, PDF_CMAP_SINGLE, 14634 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 14732 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 14270 },
	{ 0xdd95, 0xdd95, PDF_CMAP_SINGLE, 18484 },
	{ 0xdd95, 0xdd95, PDF_CMAP_SINGLE, 14804 },
	{ 0xdd96, 0xdd96, PDF_CMAP_SINGLE, 17855 },
	{ 0xdd96, 0xdd96, PDF_CMAP_SINGLE, 16511 },
	{ 0xdd96, 0xdd96, PDF_CMAP_SINGLE, 14893 },
	{ 0xdd98, 0xdd98, PDF_CMAP_SINGLE, 14412 },
	{ 0xdd98, 0xdd98, PDF_CMAP_SINGLE, 15973 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 14708 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 19139 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 17299 },
	{ 0xdd9b, 0xdd9b, PDF_CMAP_SINGLE, 18618 },
	{ 0xdd9c, 0xdd9c, PDF_CMAP_SINGLE, 17254 },
	{ 0xdd9c, 0xdd9c, PDF_CMAP_SINGLE, 15451 },
	{ 0xdd9c, 0xdd9c, PDF_CMAP_SINGLE, 16886 },
	{ 0xdd9d, 0xdd9d, PDF_CMAP_SINGLE, 14740 },
	{ 0xdd9e, 0xdd9e, PDF_CMAP_SINGLE, 18700 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 18680 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 15213 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 16853 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 17225 },
	{ 0xdda1, 0xdda1, PDF_CMAP_SINGLE, 17854 },
	{ 0xdda1, 0xdda1, PDF_CMAP_SINGLE, 16927 },
	{ 0xdda2, 0xdda2, PDF_CMAP_SINGLE, 14660 },
	{ 0xdda3, 0xdda3, PDF_CMAP_SINGLE, 16533 },
	{ 0xdda3, 0xdda3, PDF_CMAP_SINGLE, 18669 },
	{ 0xdda3, 0xdda3, PDF_CMAP_SINGLE, 14284 },
	{ 0xdda3, 0xdda3, PDF_CMAP_SINGLE, 16069 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 19047 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 17922 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 16914 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 16440 },
	{ 0xdda4, 0xdda4, PDF_CMAP_SINGLE, 15442 },
	{ 0xdda4, 0xdda5, PDF_CMAP_RANGE, 16821 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 16660 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 16585 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 14293 },
	{ 0xdda6, 0xdda6, PDF_CMAP_SINGLE, 16628 },
	{ 0xdda7, 0xdda7, PDF_CMAP_SINGLE, 16037 },
	{ 0xdda7, 0xdda7, PDF_CMAP_SINGLE, 14264 },
	{ 0xdda7, 0xdda7, PDF_CMAP_SINGLE, 15230 },
	{ 0xdda8, 0xdda8, PDF_CMAP_SINGLE, 18101 },
	{ 0xdda8, 0xdda8, PDF_CMAP_SINGLE, 14909 },
	{ 0xdda8, 0xdda8, PDF_CMAP_SINGLE, 15903 },
	{ 0xdda9, 0xdda9, PDF_CMAP_SINGLE, 17235 },
	{ 0xdda9, 0xdda9, PDF_CMAP_SINGLE, 16637 },
	{ 0xddaa, 0xddaa, PDF_CMAP_SINGLE, 16414 },
	{ 0xddab, 0xddab, PDF_CMAP_SINGLE, 14776 },
	{ 0xddab, 0xddab, PDF_CMAP_SINGLE, 15088 },
	{ 0xddab, 0xddab, PDF_CMAP_SINGLE, 15907 },
	{ 0xddab, 0xddab, PDF_CMAP_SINGLE, 14625 },
	{ 0xddac, 0xddac, PDF_CMAP_SINGLE, 15488 },
	{ 0xddac, 0xddac, PDF_CMAP_SINGLE, 14829 },
	{ 0xddac, 0xddac, PDF_CMAP_SINGLE, 14136 },
	{ 0xddad, 0xddad, PDF_CMAP_SINGLE, 15575 },
	{ 0xddad, 0xddad, PDF_CMAP_SINGLE, 18746 },
	{ 0xddad, 0xddad, PDF_CMAP_SINGLE, 15211 },
	{ 0xddae, 0xddae, PDF_CMAP_SINGLE, 15120 },
	{ 0xddae, 0xddae, PDF_CMAP_SINGLE, 18558 },
	{ 0xddaf, 0xddaf, PDF_CMAP_SINGLE, 14924 },
	{ 0xddaf, 0xddaf, PDF_CMAP_SINGLE, 18584 },
	{ 0xddb0, 0xddb0, PDF_CMAP_SINGLE, 18608 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 18424 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 18535 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 14339 },
	{ 0xddb3, 0xddb3, PDF_CMAP_SINGLE, 16454 },
	{ 0xddb3, 0xddb3, PDF_CMAP_SINGLE, 14969 },
	{ 0xddb4, 0xddb4, PDF_CMAP_SINGLE, 15431 },
	{ 0xddb5, 0xddb5, PDF_CMAP_SINGLE, 15480 },
	{ 0xddb5, 0xddb5, PDF_CMAP_SINGLE, 15921 },
	{ 0xddb5, 0xddb5, PDF_CMAP_SINGLE, 19065 },
	{ 0xddb6, 0xddb6, PDF_CMAP_SINGLE, 18905 },
	{ 0xddb6, 0xddb6, PDF_CMAP_SINGLE, 18701 },
	{ 0xddb7, 0xddb7, PDF_CMAP_SINGLE, 18483 },
	{ 0xddb7, 0xddb7, PDF_CMAP_SINGLE, 15514 },
	{ 0xddb7, 0xddb7, PDF_CMAP_SINGLE, 17047 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 15405 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 15401 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 17171 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 18593 },
	{ 0xddb8, 0xddb8, PDF_CMAP_SINGLE, 18609 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 14361 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 15519 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 14476 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 19124 },
	{ 0xddb9, 0xddb9, PDF_CMAP_SINGLE, 18942 },
	{ 0xddba, 0xddba, PDF_CMAP_SINGLE, 14907 },
	{ 0xddba, 0xddba, PDF_CMAP_SINGLE, 15147 },
	{ 0xddba, 0xddba, PDF_CMAP_SINGLE, 15773 },
	{ 0xddbb, 0xddbb, PDF_CMAP_SINGLE, 19148 },
	{ 0xddbb, 0xddbb, PDF_CMAP_SINGLE, 14446 },
	{ 0xddbb, 0xddbb, PDF_CMAP_SINGLE, 14678 },
	{ 0xddbc, 0xddbc, PDF_CMAP_SINGLE, 16911 },
	{ 0xddbc, 0xddbc, PDF_CMAP_SINGLE, 14427 },
	{ 0xddbc, 0xddbc, PDF_CMAP_SINGLE, 16552 },
	{ 0xddbc, 0xddbc, PDF_CMAP_SINGLE, 15241 },
	{ 0xddbd, 0xddbd, PDF_CMAP_SINGLE, 16913 },
	{ 0xddbd, 0xddbd, PDF_CMAP_SINGLE, 14413 },
	{ 0xddbe, 0xddbe, PDF_CMAP_SINGLE, 14925 },
	{ 0xddbf, 0xddbf, PDF_CMAP_SINGLE, 18541 },
	{ 0xddc0, 0xddc0, PDF_CMAP_SINGLE, 18044 },
	{ 0xddc1, 0xddc1, PDF_CMAP_SINGLE, 16714 },
	{ 0xddc1, 0xddc1, PDF_CMAP_SINGLE, 19058 },
	{ 0xddc2, 0xddc2, PDF_CMAP_SINGLE, 17269 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 18779 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 15799 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 18614 },
	{ 0xddc5, 0xddc5, PDF_CMAP_SINGLE, 15094 },
	{ 0xddc6, 0xddc6, PDF_CMAP_SINGLE, 15084 },
	{ 0xddc8, 0xddc8, PDF_CMAP_SINGLE, 18425 },
	{ 0xddc8, 0xddc8, PDF_CMAP_SINGLE, 16831 },
	{ 0xddc8, 0xddc8, PDF_CMAP_SINGLE, 17240 },
	{ 0xddc9, 0xddc9, PDF_CMAP_SINGLE, 15471 },
	{ 0xddc9, 0xddc9, PDF_CMAP_SINGLE, 15797 },
	{ 0xddca, 0xddca, PDF_CMAP_SINGLE, 15788 },
	{ 0xddca, 0xddca, PDF_CMAP_SINGLE, 17802 },
	{ 0xddcb, 0xddcb, PDF_CMAP_SINGLE, 18627 },
	{ 0xddcc, 0xddcc, PDF_CMAP_SINGLE, 16659 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 16861 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 16844 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 18859 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 18560 },
	{ 0xddce, 0xddce, PDF_CMAP_SINGLE, 14287 },
	{ 0xddcf, 0xddcf, PDF_CMAP_SINGLE, 15194 },
	{ 0xddd0, 0xddd0, PDF_CMAP_SINGLE, 17803 },
	{ 0xddd0, 0xddd0, PDF_CMAP_SINGLE, 15876 },
	{ 0xddd1, 0xddd1, PDF_CMAP_SINGLE, 16142 },
	{ 0xddd4, 0xddd4, PDF_CMAP_SINGLE, 15153 },
	{ 0xddd4, 0xddd4, PDF_CMAP_SINGLE, 14408 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 18402 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 15554 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 14539 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 15365 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 14421 },
	{ 0xddd9, 0xddd9, PDF_CMAP_SINGLE, 18673 },
	{ 0xddda, 0xddda, PDF_CMAP_SINGLE, 17940 },
	{ 0xdddb, 0xdddb, PDF_CMAP_SINGLE, 16407 },
	{ 0xdddb, 0xdddb, PDF_CMAP_SINGLE, 14480 },
	{ 0xdddc, 0xdddc, PDF_CMAP_SINGLE, 18744 },
	{ 0xdddc, 0xdddc, PDF_CMAP_SINGLE, 15279 },
	{ 0xdddc, 0xdddc, PDF_CMAP_SINGLE, 15031 },
	{ 0xdddd, 0xdddd, PDF_CMAP_SINGLE, 16979 },
	{ 0xdddd, 0xdddd, PDF_CMAP_SINGLE, 18571 },
	{ 0xddde, 0xddde, PDF_CMAP_SINGLE, 15453 },
	{ 0xddde, 0xddde, PDF_CMAP_SINGLE, 14444 },
	{ 0xdddf, 0xdddf, PDF_CMAP_SINGLE, 16397 },
	{ 0xdddf, 0xdddf, PDF_CMAP_SINGLE, 16783 },
	{ 0xdddf, 0xdddf, PDF_CMAP_SINGLE, 14628 },
	{ 0xdde0, 0xdde0, PDF_CMAP_SINGLE, 14812 },
	{ 0xdde0, 0xdde0, PDF_CMAP_SINGLE, 17137 },
	{ 0xdde0, 0xdde0, PDF_CMAP_SINGLE, 14360 },
	{ 0xdde1, 0xdde1, PDF_CMAP_SINGLE, 15689 },
	{ 0xdde2, 0xdde2, PDF_CMAP_SINGLE, 17042 },
	{ 0xdde3, 0xdde3, PDF_CMAP_SINGLE, 14237 },
	{ 0xdde3, 0xdde3, PDF_CMAP_SINGLE, 14951 },
	{ 0xdde3, 0xdde3, PDF_CMAP_SINGLE, 17865 },
	{ 0xdde3, 0xdde3, PDF_CMAP_SINGLE, 18892 },
	{ 0xdde4, 0xdde4, PDF_CMAP_SINGLE, 14362 },
	{ 0xdde4, 0xdde4, PDF_CMAP_SINGLE, 16405 },
	{ 0xdde5, 0xdde5, PDF_CMAP_SINGLE, 15540 },
	{ 0xdde6, 0xdde6, PDF_CMAP_SINGLE, 16826 },
	{ 0xdde7, 0xdde7, PDF_CMAP_SINGLE, 17781 },
	{ 0xdde7, 0xdde7, PDF_CMAP_SINGLE, 15103 },
	{ 0xdde7, 0xdde8, PDF_CMAP_RANGE, 18594 },
	{ 0xdde8, 0xdde8, PDF_CMAP_SINGLE, 15796 },
	{ 0xdde9, 0xdde9, PDF_CMAP_SINGLE, 15949 },
	{ 0xddea, 0xddea, PDF_CMAP_SINGLE, 19099 },
	{ 0xddea, 0xddea, PDF_CMAP_SINGLE, 17181 },
	{ 0xddea, 0xddea, PDF_CMAP_SINGLE, 15075 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 15056 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 14991 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 17908 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 16928 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 14542 },
	{ 0xddec, 0xddec, PDF_CMAP_SINGLE, 18116 },
	{ 0xdded, 0xdded, PDF_CMAP_SINGLE, 15331 },
	{ 0xddee, 0xddee, PDF_CMAP_SINGLE, 17896 },
	{ 0xddf0, 0xddf0, PDF_CMAP_SINGLE, 15444 },
	{ 0xddf2, 0xddf2, PDF_CMAP_SINGLE, 16228 },
	{ 0xddf2, 0xddf2, PDF_CMAP_SINGLE, 17180 },
	{ 0xddf3, 0xddf3, PDF_CMAP_SINGLE, 14869 },
	{ 0xddf3, 0xddf3, PDF_CMAP_SINGLE, 18493 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 16830 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 17204 },
	{ 0xddf5, 0xddf5, PDF_CMAP_SINGLE, 15423 },
	{ 0xddf6, 0xddf6, PDF_CMAP_SINGLE, 15834 },
	{ 0xddf6, 0xddf6, PDF_CMAP_SINGLE, 17788 },
	{ 0xddf7, 0xddf7, PDF_CMAP_SINGLE, 15210 },
	{ 0xddf7, 0xddf8, PDF_CMAP_RANGE, 16818 },
	{ 0xddf8, 0xddf8, PDF_CMAP_SINGLE, 14267 },
	{ 0xddf8, 0xddf8, PDF_CMAP_SINGLE, 14680 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 16677 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 16922 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 15174 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 15260 },
	{ 0xddfa, 0xddfa, PDF_CMAP_SINGLE, 18552 },
	{ 0xddfa, 0xddfa, PDF_CMAP_SINGLE, 15521 },
	{ 0xddfb, 0xddfb, PDF_CMAP_SINGLE, 17049 },
	{ 0xddfb, 0xddfb, PDF_CMAP_SINGLE, 14886 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 15670 },
	{ 0xddfc, 0xddfc, PDF_CMAP_SINGLE, 15502 },
	{ 0xddfd, 0xddfd, PDF_CMAP_SINGLE, 14917 },
	{ 0xddfd, 0xddfd, PDF_CMAP_SINGLE, 18572 },
	{ 0xddfe, 0xddfe, PDF_CMAP_SINGLE, 15781 },
	{ 0xddff, 0xddff, PDF_CMAP_SINGLE, 19152 },
	{ 0xde00, 0xde00, PDF_CMAP_SINGLE, 16423 },
	{ 0xde00, 0xde00, PDF_CMAP_SINGLE, 15447 },
	{ 0xde00, 0xde00, PDF_CMAP_SINGLE, 14607 },
	{ 0xde01, 0xde01, PDF_CMAP_SINGLE, 17263 },
	{ 0xde04, 0xde04, PDF_CMAP_SINGLE, 16656 },
	{ 0xde04, 0xde04, PDF_CMAP_SINGLE, 18426 },
	{ 0xde05, 0xde05, PDF_CMAP_SINGLE, 17016 },
	{ 0xde05, 0xde05, PDF_CMAP_SINGLE, 16291 },
	{ 0xde06, 0xde06, PDF_CMAP_SINGLE, 18999 },
	{ 0xde06, 0xde06, PDF_CMAP_SINGLE, 16122 },
	{ 0xde07, 0xde07, PDF_CMAP_SINGLE, 15916 },
	{ 0xde07, 0xde07, PDF_CMAP_SINGLE, 14008 },
	{ 0xde07, 0xde07, PDF_CMAP_SINGLE, 17918 },
	{ 0xde08, 0xde08, PDF_CMAP_SINGLE, 17287 },
	{ 0xde09, 0xde09, PDF_CMAP_SINGLE, 17264 },
	{ 0xde0a, 0xde0a, PDF_CMAP_SINGLE, 18682 },
	{ 0xde0a, 0xde0a, PDF_CMAP_SINGLE, 15896 },
	{ 0xde0a, 0xde0a, PDF_CMAP_SINGLE, 18573 },
	{ 0xde0b, 0xde0b, PDF_CMAP_SINGLE, 14459 },
	{ 0xde0c, 0xde0c, PDF_CMAP_SINGLE, 14787 },
	{ 0xde0c, 0xde0c, PDF_CMAP_SINGLE, 17141 },
	{ 0xde0d, 0xde0d, PDF_CMAP_SINGLE, 14761 },
	{ 0xde0d, 0xde0d, PDF_CMAP_SINGLE, 16781 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 18427 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 14793 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 15204 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 16653 },
	{ 0xde0f, 0xde0f, PDF_CMAP_SINGLE, 17905 },
	{ 0xde0f, 0xde0f, PDF_CMAP_SINGLE, 14982 },
	{ 0xde0f, 0xde0f, PDF_CMAP_SINGLE, 18747 },
	{ 0xde0f, 0xde0f, PDF_CMAP_SINGLE, 17675 },
	{ 0xde10, 0xde10, PDF_CMAP_SINGLE, 17914 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 15790 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 15855 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 17929 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 18498 },
	{ 0xde12, 0xde12, PDF_CMAP_SINGLE, 18782 },
	{ 0xde12, 0xde12, PDF_CMAP_SINGLE, 17663 },
	{ 0xde12, 0xde12, PDF_CMAP_SINGLE, 15437 },
	{ 0xde13, 0xde13, PDF_CMAP_SINGLE, 16512 },
	{ 0xde13, 0xde13, PDF_CMAP_SINGLE, 18920 },
	{ 0xde14, 0xde14, PDF_CMAP_SINGLE, 18139 },
	{ 0xde15, 0xde15, PDF_CMAP_SINGLE, 18403 },
	{ 0xde15, 0xde15, PDF_CMAP_SINGLE, 18512 },
	{ 0xde16, 0xde16, PDF_CMAP_SINGLE, 14275 },
	{ 0xde16, 0xde16, PDF_CMAP_SINGLE, 15551 },
	{ 0xde17, 0xde17, PDF_CMAP_SINGLE, 18982 },
	{ 0xde18, 0xde18, PDF_CMAP_SINGLE, 16079 },
	{ 0xde18, 0xde18, PDF_CMAP_SINGLE, 15599 },
	{ 0xde19, 0xde19, PDF_CMAP_SINGLE, 18142 },
	{ 0xde19, 0xde19, PDF_CMAP_SINGLE, 18447 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 15561 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 18494 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 18585 },
	{ 0xde1b, 0xde1b, PDF_CMAP_SINGLE, 14931 },
	{ 0xde1b, 0xde1b, PDF_CMAP_SINGLE, 15354 },
	{ 0xde1b, 0xde1b, PDF_CMAP_SINGLE, 15232 },
	{ 0xde1b, 0xde1b, PDF_CMAP_SINGLE, 14662 },
	{ 0xde1c, 0xde1c, PDF_CMAP_SINGLE, 17117 },
	{ 0xde1c, 0xde1c, PDF_CMAP_SINGLE, 17144 },
	{ 0xde1d, 0xde1d, PDF_CMAP_SINGLE, 14832 },
	{ 0xde20, 0xde20, PDF_CMAP_SINGLE, 17122 },
	{ 0xde20, 0xde20, PDF_CMAP_SINGLE, 14635 },
	{ 0xde21, 0xde21, PDF_CMAP_SINGLE, 16838 },
	{ 0xde21, 0xde21, PDF_CMAP_SINGLE, 15102 },
	{ 0xde22, 0xde22, PDF_CMAP_SINGLE, 17005 },
	{ 0xde23, 0xde23, PDF_CMAP_SINGLE, 19043 },
	{ 0xde23, 0xde23, PDF_CMAP_SINGLE, 16802 },
	{ 0xde24, 0xde24, PDF_CMAP_SINGLE, 17044 },
	{ 0xde25, 0xde25, PDF_CMAP_SINGLE, 15325 },
	{ 0xde25, 0xde25, PDF_CMAP_SINGLE, 14723 },
	{ 0xde25, 0xde25, PDF_CMAP_SINGLE, 14332 },
	{ 0xde25, 0xde25, PDF_CMAP_SINGLE, 14376 },
	{ 0xde26, 0xde26, PDF_CMAP_SINGLE, 17053 },
	{ 0xde26, 0xde26, PDF_CMAP_SINGLE, 14236 },
	{ 0xde27, 0xde27, PDF_CMAP_SINGLE, 17004 },
	{ 0xde28, 0xde28, PDF_CMAP_SINGLE, 16801 },
	{ 0xde28, 0xde28, PDF_CMAP_SINGLE, 18615 },
	{ 0xde29, 0xde29, PDF_CMAP_SINGLE, 14587 },
	{ 0xde29, 0xde29, PDF_CMAP_SINGLE, 16736 },
	{ 0xde2a, 0xde2a, PDF_CMAP_SINGLE, 15005 },
	{ 0xde2a, 0xde2a, PDF_CMAP_SINGLE, 15885 },
	{ 0xde2b, 0xde2b, PDF_CMAP_SINGLE, 14921 },
	{ 0xde2c, 0xde2c, PDF_CMAP_SINGLE, 18705 },
	{ 0xde2d, 0xde2d, PDF_CMAP_SINGLE, 18453 },
	{ 0xde2d, 0xde2d, PDF_CMAP_SINGLE, 15609 },
	{ 0xde2d, 0xde2d, PDF_CMAP_SINGLE, 18553 },
	{ 0xde2d, 0xde2d, PDF_CMAP_SINGLE, 15421 },
	{ 0xde2e, 0xde2e, PDF_CMAP_SINGLE, 14505 },
	{ 0xde2e, 0xde2e, PDF_CMAP_SINGLE, 17039 },
	{ 0xde2f, 0xde2f, PDF_CMAP_SINGLE, 15923 },
	{ 0xde2f, 0xde2f, PDF_CMAP_SINGLE, 16944 },
	{ 0xde30, 0xde30, PDF_CMAP_SINGLE, 14571 },
	{ 0xde30, 0xde30, PDF_CMAP_SINGLE, 16954 },
	{ 0xde31, 0xde31, PDF_CMAP_SINGLE, 16829 },
	{ 0xde31, 0xde31, PDF_CMAP_SINGLE, 14734 },
	{ 0xde32, 0xde32, PDF_CMAP_SINGLE, 15105 },
	{ 0xde32, 0xde32, PDF_CMAP_SINGLE, 15814 },
	{ 0xde32, 0xde32, PDF_CMAP_SINGLE, 18863 },
	{ 0xde32, 0xde32, PDF_CMAP_SINGLE, 17297 },
	{ 0xde33, 0xde33, PDF_CMAP_SINGLE, 16450 },
	{ 0xde33, 0xde33, PDF_CMAP_SINGLE, 14637 },
	{ 0xde34, 0xde34, PDF_CMAP_SINGLE, 16658 },
	{ 0xde34, 0xde34, PDF_CMAP_SINGLE, 14980 },
	{ 0xde34, 0xde34, PDF_CMAP_SINGLE, 18554 },
	{ 0xde35, 0xde35, PDF_CMAP_SINGLE, 18643 },
	{ 0xde36, 0xde36, PDF_CMAP_SINGLE, 16790 },
	{ 0xde36, 0xde36, PDF_CMAP_SINGLE, 16726 },
	{ 0xde36, 0xde36, PDF_CMAP_SINGLE, 14688 },
	{ 0xde37, 0xde37, PDF_CMAP_SINGLE, 16678 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 19051 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 14805 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 17292 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 16332 },
	{ 0xde3b, 0xde3b, PDF_CMAP_SINGLE, 17282 },
	{ 0xde3b, 0xde3b, PDF_CMAP_SINGLE, 14333 },
	{ 0xde3c, 0xde3c, PDF_CMAP_SINGLE, 16603 },
	{ 0xde3c, 0xde3c, PDF_CMAP_SINGLE, 16865 },
	{ 0xde3d, 0xde3d, PDF_CMAP_SINGLE, 18773 },
	{ 0xde3e, 0xde3e, PDF_CMAP_SINGLE, 16615 },
	{ 0xde3e, 0xde3e, PDF_CMAP_SINGLE, 15534 },
	{ 0xde3e, 0xde3e, PDF_CMAP_SINGLE, 15961 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 15452 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 18485 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 15322 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 15225 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 14301 },
	{ 0xde43, 0xde43, PDF_CMAP_SINGLE, 14313 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 14184 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 14691 },
	{ 0xde45, 0xde45, PDF_CMAP_SINGLE, 18454 },
	{ 0xde45, 0xde45, PDF_CMAP_SINGLE, 15787 },
	{ 0xde45, 0xde45, PDF_CMAP_SINGLE, 16633 },
	{ 0xde45, 0xde45, PDF_CMAP_SINGLE, 15372 },
	{ 0xde46, 0xde46, PDF_CMAP_SINGLE, 14859 },
	{ 0xde47, 0xde47, PDF_CMAP_SINGLE, 14906 },
	{ 0xde48, 0xde48, PDF_CMAP_SINGLE, 14241 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 17274 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 15494 },
	{ 0xde4a, 0xde4a, PDF_CMAP_SINGLE, 15060 },
	{ 0xde4a, 0xde4a, PDF_CMAP_SINGLE, 16869 },
	{ 0xde4a, 0xde4a, PDF_CMAP_SINGLE, 16254 },
	{ 0xde4b, 0xde4b, PDF_CMAP_SINGLE, 17082 },
	{ 0xde4b, 0xde4b, PDF_CMAP_SINGLE, 18102 },
	{ 0xde4b, 0xde4b, PDF_CMAP_SINGLE, 15306 },
	{ 0xde4c, 0xde4c, PDF_CMAP_SINGLE, 17873 },
	{ 0xde4d, 0xde4d, PDF_CMAP_SINGLE, 15694 },
	{ 0xde4d, 0xde4d, PDF_CMAP_SINGLE, 17197 },
	{ 0xde4e, 0xde4e, PDF_CMAP_SINGLE, 15092 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 16583 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 18129 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 15604 },
	{ 0xde50, 0xde50, PDF_CMAP_SINGLE, 14441 },
	{ 0xde50, 0xde50, PDF_CMAP_SINGLE, 15569 },
	{ 0xde50, 0xde50, PDF_CMAP_SINGLE, 17921 },
	{ 0xde50, 0xde50, PDF_CMAP_SINGLE, 16730 },
	{ 0xde51, 0xde51, PDF_CMAP_SINGLE, 14682 },
	{ 0xde51, 0xde51, PDF_CMAP_SINGLE, 14774 },
	{ 0xde51, 0xde51, PDF_CMAP_SINGLE, 15228 },
	{ 0xde52, 0xde52, PDF_CMAP_SINGLE, 14257 },
	{ 0xde53, 0xde53, PDF_CMAP_SINGLE, 16751 },
	{ 0xde54, 0xde54, PDF_CMAP_SINGLE, 18997 },
	{ 0xde55, 0xde55, PDF_CMAP_SINGLE, 18642 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 16761 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 19028 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 15697 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 18740 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 17193 },
	{ 0xde57, 0xde57, PDF_CMAP_SINGLE, 15779 },
	{ 0xde58, 0xde58, PDF_CMAP_SINGLE, 15964 },
	{ 0xde59, 0xde59, PDF_CMAP_SINGLE, 16878 },
	{ 0xde59, 0xde59, PDF_CMAP_SINGLE, 15688 },
	{ 0xde59, 0xde59, PDF_CMAP_SINGLE, 14384 },
	{ 0xde59, 0xde59, PDF_CMAP_SINGLE, 18150 },
	{ 0xde5a, 0xde5a, PDF_CMAP_SINGLE, 18726 },
	{ 0xde5b, 0xde5b, PDF_CMAP_SINGLE, 15161 },
	{ 0xde5b, 0xde5b, PDF_CMAP_SINGLE, 17288 },
	{ 0xde5d, 0xde5d, PDF_CMAP_SINGLE, 18133 },
	{ 0xde5e, 0xde5e, PDF_CMAP_SINGLE, 19101 },
	{ 0xde5f, 0xde5f, PDF_CMAP_SINGLE, 15059 },
	{ 0xde61, 0xde61, PDF_CMAP_SINGLE, 14217 },
	{ 0xde61, 0xde61, PDF_CMAP_SINGLE, 15184 },
	{ 0xde62, 0xde62, PDF_CMAP_SINGLE, 14448 },
	{ 0xde63, 0xde63, PDF_CMAP_SINGLE, 18998 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 14940 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 14253 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 15329 },
	{ 0xde66, 0xde66, PDF_CMAP_SINGLE, 17250 },
	{ 0xde66, 0xde66, PDF_CMAP_SINGLE, 15537 },
	{ 0xde66, 0xde66, PDF_CMAP_SINGLE, 15328 },
	{ 0xde67, 0xde67, PDF_CMAP_SINGLE, 15369 },
	{ 0xde67, 0xde67, PDF_CMAP_SINGLE, 18970 },
	{ 0xde68, 0xde68, PDF_CMAP_SINGLE, 16733 },
	{ 0xde68, 0xde68, PDF_CMAP_SINGLE, 14621 },
	{ 0xde6a, 0xde6a, PDF_CMAP_SINGLE, 17658 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 18757 },
	{ 0xde6d, 0xde6d, PDF_CMAP_SINGLE, 18647 },
	{ 0xde6e, 0xde6e, PDF_CMAP_SINGLE, 16723 },
	{ 0xde6f, 0xde6f, PDF_CMAP_SINGLE, 19022 },
	{ 0xde71, 0xde71, PDF_CMAP_SINGLE, 15632 },
	{ 0xde72, 0xde72, PDF_CMAP_SINGLE, 16935 },
	{ 0xde73, 0xde73, PDF_CMAP_SINGLE, 18431 },
	{ 0xde74, 0xde74, PDF_CMAP_SINGLE, 14521 },
	{ 0xde74, 0xde74, PDF_CMAP_SINGLE, 18508 },
	{ 0xde75, 0xde75, PDF_CMAP_SINGLE, 14785 },
	{ 0xde75, 0xde75, PDF_CMAP_SINGLE, 16824 },
	{ 0xde76, 0xde76, PDF_CMAP_SINGLE, 18404 },
	{ 0xde76, 0xde76, PDF_CMAP_SINGLE, 16015 },
	{ 0xde76, 0xde76, PDF_CMAP_SINGLE, 15047 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 17906 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 18748 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 16992 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 18725 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 14312 },
	{ 0xde78, 0xde78, PDF_CMAP_SINGLE, 17915 },
	{ 0xde78, 0xde78, PDF_CMAP_SINGLE, 14946 },
	{ 0xde78, 0xde78, PDF_CMAP_SINGLE, 16608 },
	{ 0xde78, 0xde78, PDF_CMAP_SINGLE, 15332 },
	{ 0xde79, 0xde79, PDF_CMAP_SINGLE, 17920 },
	{ 0xde79, 0xde79, PDF_CMAP_SINGLE, 17072 },
	{ 0xde79, 0xde79, PDF_CMAP_SINGLE, 15771 },
	{ 0xde7a, 0xde7a, PDF_CMAP_SINGLE, 17290 },
	{ 0xde7a, 0xde7a, PDF_CMAP_SINGLE, 18100 },
	{ 0xde7a, 0xde7a, PDF_CMAP_SINGLE, 14398 },
	{ 0xde7b, 0xde7b, PDF_CMAP_SINGLE, 17933 },
	{ 0xde7c, 0xde7c, PDF_CMAP_SINGLE, 14504 },
	{ 0xde7c, 0xde7c, PDF_CMAP_SINGLE, 18465 },
	{ 0xde7c, 0xde7c, PDF_CMAP_SINGLE, 17943 },
	{ 0xde80, 0xde80, PDF_CMAP_SINGLE, 18561 },
	{ 0xde81, 0xde81, PDF_CMAP_SINGLE, 18869 },
	{ 0xde81, 0xde81, PDF_CMAP_SINGLE, 18646 },
	{ 0xde81, 0xde81, PDF_CMAP_SINGLE, 15586 },
	{ 0xde82, 0xde82, PDF_CMAP_SINGLE, 15188 },
	{ 0xde82, 0xde82, PDF_CMAP_SINGLE, 15556 },
	{ 0xde83, 0xde83, PDF_CMAP_SINGLE, 15253 },
	{ 0xde83, 0xde83, PDF_CMAP_SINGLE, 14876 },
	{ 0xde83, 0xde83, PDF_CMAP_SINGLE, 15642 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 15271 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 15443 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 16422 },
	{ 0xde85, 0xde85, PDF_CMAP_SINGLE, 18562 },
	{ 0xde88, 0xde88, PDF_CMAP_SINGLE, 18648 },
	{ 0xde88, 0xde88, PDF_CMAP_SINGLE, 19137 },
	{ 0xde89, 0xde89, PDF_CMAP_SINGLE, 14471 },
	{ 0xde89, 0xde89, PDF_CMAP_SINGLE, 14968 },
	{ 0xde89, 0xde89, PDF_CMAP_SINGLE, 14809 },
	{ 0xde8a, 0xde8a, PDF_CMAP_SINGLE, 19100 },
	{ 0xde8a, 0xde8a, PDF_CMAP_SINGLE, 16569 },
	{ 0xde8a, 0xde8a, PDF_CMAP_SINGLE, 14794 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 17286 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 14283 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 18984 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 18563 },
	{ 0xde8c, 0xde8c, PDF_CMAP_SINGLE, 17866 },
	{ 0xde8c, 0xde8c, PDF_CMAP_SINGLE, 17070 },
	{ 0xde8e, 0xde8e, PDF_CMAP_SINGLE, 19131 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 16618 },
	{ 0xde93, 0xde93, PDF_CMAP_SINGLE, 15681 },
	{ 0xde93, 0xde93, PDF_CMAP_SINGLE, 18634 },
	{ 0xde93, 0xde93, PDF_CMAP_SINGLE, 18624 },
	{ 0xde94, 0xde94, PDF_CMAP_SINGLE, 15535 },
	{ 0xde94, 0xde94, PDF_CMAP_SINGLE, 17930 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 14871 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 14254 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 18925 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 18694 },
	{ 0xde96, 0xde96, PDF_CMAP_SINGLE, 17189 },
	{ 0xde96, 0xde96, PDF_CMAP_SINGLE, 17880 },
	{ 0xde96, 0xde96, PDF_CMAP_SINGLE, 15119 },
	{ 0xde97, 0xde97, PDF_CMAP_SINGLE, 15109 },
	{ 0xde97, 0xde97, PDF_CMAP_SINGLE, 16202 },
	{ 0xde98, 0xde98, PDF_CMAP_SINGLE, 17887 },
	{ 0xde98, 0xde98, PDF_CMAP_SINGLE, 16553 },
	{ 0xde99, 0xde99, PDF_CMAP_SINGLE, 15146 },
	{ 0xde99, 0xde99, PDF_CMAP_SINGLE, 18518 },
	{ 0xde99, 0xde99, PDF_CMAP_SINGLE, 18086 },
	{ 0xde9a, 0xde9a, PDF_CMAP_SINGLE, 15785 },
	{ 0xde9b, 0xde9b, PDF_CMAP_SINGLE, 17272 },
	{ 0xde9b, 0xde9b, PDF_CMAP_SINGLE, 16515 },
	{ 0xde9c, 0xde9c, PDF_CMAP_SINGLE, 15159 },
	{ 0xde9c, 0xde9c, PDF_CMAP_SINGLE, 14818 },
	{ 0xde9d, 0xde9d, PDF_CMAP_SINGLE, 16000 },
	{ 0xde9e, 0xde9e, PDF_CMAP_SINGLE, 16735 },
	{ 0xde9e, 0xde9e, PDF_CMAP_SINGLE, 19090 },
	{ 0xde9f, 0xde9f, PDF_CMAP_SINGLE, 14440 },
	{ 0xdea0, 0xdea0, PDF_CMAP_SINGLE, 16415 },
	{ 0xdea2, 0xdea2, PDF_CMAP_SINGLE, 16017 },
	{ 0xdea4, 0xdea4, PDF_CMAP_SINGLE, 16682 },
	{ 0xdea4, 0xdea4, PDF_CMAP_SINGLE, 15055 },
	{ 0xdea5, 0xdea5, PDF_CMAP_SINGLE, 15678 },
	{ 0xdea5, 0xdea5, PDF_CMAP_SINGLE, 15934 },
	{ 0xdea5, 0xdea5, PDF_CMAP_SINGLE, 17360 },
	{ 0xdea6, 0xdea6, PDF_CMAP_SINGLE, 14729 },
	{ 0xdea6, 0xdea6, PDF_CMAP_SINGLE, 16464 },
	{ 0xdea6, 0xdea6, PDF_CMAP_SINGLE, 15172 },
	{ 0xdea7, 0xdea7, PDF_CMAP_SINGLE, 18497 },
	{ 0xdea7, 0xdea7, PDF_CMAP_SINGLE, 15810 },
	{ 0xdea7, 0xdea7, PDF_CMAP_SINGLE, 17200 },
	{ 0xdea8, 0xdea8, PDF_CMAP_SINGLE, 14725 },
	{ 0xdea8, 0xdea8, PDF_CMAP_SINGLE, 15792 },
	{ 0xdea9, 0xdea9, PDF_CMAP_SINGLE, 18707 },
	{ 0xdea9, 0xdea9, PDF_CMAP_SINGLE, 18675 },
	{ 0xdeaa, 0xdeaa, PDF_CMAP_SINGLE, 17868 },
	{ 0xdeaa, 0xdeaa, PDF_CMAP_SINGLE, 18590 },
	{ 0xdeab, 0xdeab, PDF_CMAP_SINGLE, 17874 },
	{ 0xdeab, 0xdeab, PDF_CMAP_SINGLE, 14962 },
	{ 0xdeab, 0xdeab, PDF_CMAP_SINGLE, 16508 },
	{ 0xdeac, 0xdeac, PDF_CMAP_SINGLE, 18676 },
	{ 0xdeac, 0xdeac, PDF_CMAP_SINGLE, 17124 },
	{ 0xdeac, 0xdeac, PDF_CMAP_SINGLE, 17084 },
	{ 0xdeac, 0xdeac, PDF_CMAP_SINGLE, 16085 },
	{ 0xdead, 0xdead, PDF_CMAP_SINGLE, 14960 },
	{ 0xdead, 0xdead, PDF_CMAP_SINGLE, 15647 },
	{ 0xdeae, 0xdeae, PDF_CMAP_SINGLE, 18529 },
	{ 0xdeaf, 0xdeaf, PDF_CMAP_SINGLE, 15158 },
	{ 0xdeaf, 0xdeaf, PDF_CMAP_SINGLE, 18641 },
	{ 0xdeb0, 0xdeb0, PDF_CMAP_SINGLE, 15634 },
	{ 0xdeb0, 0xdeb0, PDF_CMAP_SINGLE, 19013 },
	{ 0xdeb1, 0xdeb1, PDF_CMAP_SINGLE, 16424 },
	{ 0xdeb1, 0xdeb1, PDF_CMAP_SINGLE, 14757 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 15361 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 16092 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 19071 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 17059 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 16276 },
	{ 0xdeb3, 0xdeb3, PDF_CMAP_SINGLE, 16034 },
	{ 0xdeb3, 0xdeb3, PDF_CMAP_SINGLE, 15529 },
	{ 0xdeb3, 0xdeb3, PDF_CMAP_SINGLE, 14784 },
	{ 0xdeb4, 0xdeb4, PDF_CMAP_SINGLE, 15590 },
	{ 0xdeb4, 0xdeb4, PDF_CMAP_SINGLE, 14640 },
	{ 0xdeb5, 0xdeb5, PDF_CMAP_SINGLE, 18542 },
	{ 0xdeb6, 0xdeb6, PDF_CMAP_SINGLE, 16104 },
	{ 0xdeb6, 0xdeb6, PDF_CMAP_SINGLE, 18672 },
	{ 0xdeb6, 0xdeb6, PDF_CMAP_SINGLE, 18145 },
	{ 0xdeb7, 0xdeb7, PDF_CMAP_SINGLE, 18397 },
	{ 0xdeb8, 0xdeb8, PDF_CMAP_SINGLE, 16647 },
	{ 0xdeb9, 0xdeb9, PDF_CMAP_SINGLE, 18499 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 18405 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 18659 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 15116 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 18880 },
	{ 0xdebb, 0xdebb, PDF_CMAP_SINGLE, 16285 },
	{ 0xdebc, 0xdebc, PDF_CMAP_SINGLE, 14840 },
	{ 0xdebc, 0xdebc, PDF_CMAP_SINGLE, 15171 },
	{ 0xdebc, 0xdebc, PDF_CMAP_SINGLE, 18096 },
	{ 0xdebd, 0xdebd, PDF_CMAP_SINGLE, 14379 },
	{ 0xdebd, 0xdebd, PDF_CMAP_SINGLE, 17939 },
	{ 0xdebe, 0xdebe, PDF_CMAP_SINGLE, 14456 },
	{ 0xdebf, 0xdebf, PDF_CMAP_SINGLE, 15458 },
	{ 0xdebf, 0xdebf, PDF_CMAP_SINGLE, 14806 },
	{ 0xdebf, 0xdebf, PDF_CMAP_SINGLE, 18981 },
	{ 0xdec0, 0xdec0, PDF_CMAP_SINGLE, 14783 },
	{ 0xdec0, 0xdec0, PDF_CMAP_SINGLE, 14721 },
	{ 0xdec0, 0xdec0, PDF_CMAP_SINGLE, 18513 },
	{ 0xdec0, 0xdec0, PDF_CMAP_SINGLE, 15574 },
	{ 0xdec1, 0xdec1, PDF_CMAP_SINGLE, 17128 },
	{ 0xdec1, 0xdec1, PDF_CMAP_SINGLE, 17001 },
	{ 0xdec2, 0xdec2, PDF_CMAP_SINGLE, 18408 },
	{ 0xdec2, 0xdec2, PDF_CMAP_SINGLE, 15568 },
	{ 0xdec3, 0xdec3, PDF_CMAP_SINGLE, 17893 },
	{ 0xdec6, 0xdec6, PDF_CMAP_SINGLE, 14477 },
	{ 0xdec7, 0xdec7, PDF_CMAP_SINGLE, 15117 },
	{ 0xdec7, 0xdec7, PDF_CMAP_SINGLE, 17207 },
	{ 0xdec8, 0xdec8, PDF_CMAP_SINGLE, 18458 },
	{ 0xdec9, 0xdec9, PDF_CMAP_SINGLE, 14984 },
	{ 0xdeca, 0xdeca, PDF_CMAP_SINGLE, 15686 },
	{ 0xdeca, 0xdeca, PDF_CMAP_SINGLE, 18605 },
	{ 0xdecb, 0xdecb, PDF_CMAP_SINGLE, 15028 },
	{ 0xdecc, 0xdecc, PDF_CMAP_SINGLE, 15087 },
	{ 0xdecd, 0xdecd, PDF_CMAP_SINGLE, 18409 },
	{ 0xdecd, 0xdecd, PDF_CMAP_SINGLE, 17931 },
	{ 0xdece, 0xdece, PDF_CMAP_SINGLE, 15550 },
	{ 0xdecf, 0xdecf, PDF_CMAP_SINGLE, 17265 },
	{ 0xded0, 0xded0, PDF_CMAP_SINGLE, 14212 },
	{ 0xded0, 0xded0, PDF_CMAP_SINGLE, 18671 },
	{ 0xded1, 0xded1, PDF_CMAP_SINGLE, 14525 },
	{ 0xded1, 0xded1, PDF_CMAP_SINGLE, 18733 },
	{ 0xded2, 0xded2, PDF_CMAP_SINGLE, 14993 },
	{ 0xded2, 0xded2, PDF_CMAP_SINGLE, 16758 },
	{ 0xded3, 0xded3, PDF_CMAP_SINGLE, 14580 },
	{ 0xded3, 0xded3, PDF_CMAP_SINGLE, 14125 },
	{ 0xded4, 0xded4, PDF_CMAP_SINGLE, 14937 },
	{ 0xded4, 0xded4, PDF_CMAP_SINGLE, 14297 },
	{ 0xded5, 0xded5, PDF_CMAP_SINGLE, 16631 },
	{ 0xded5, 0xded5, PDF_CMAP_SINGLE, 18459 },
	{ 0xded5, 0xded5, PDF_CMAP_SINGLE, 18473 },
	{ 0xded5, 0xded5, PDF_CMAP_SINGLE, 14512 },
	{ 0xded6, 0xded6, PDF_CMAP_SINGLE, 14186 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 18428 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 14123 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 18739 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 14453 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 18071 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 17938 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 18655 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 14503 },
	{ 0xded9, 0xded9, PDF_CMAP_SINGLE, 17081 },
	{ 0xdeda, 0xdeda, PDF_CMAP_SINGLE, 17223 },
	{ 0xdedb, 0xdedb, PDF_CMAP_SINGLE, 16895 },
	{ 0xdedd, 0xdedd, PDF_CMAP_SINGLE, 16549 },
	{ 0xdede, 0xdede, PDF_CMAP_SINGLE, 16487 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 14402 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 15641 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 15065 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 16858 },
	{ 0xdedf, 0xdedf, PDF_CMAP_SINGLE, 15912 },
	{ 0xdee0, 0xdee0, PDF_CMAP_SINGLE, 14964 },
	{ 0xdee0, 0xdee0, PDF_CMAP_SINGLE, 14516 },
	{ 0xdee1, 0xdee1, PDF_CMAP_SINGLE, 14994 },
	{ 0xdee1, 0xdee1, PDF_CMAP_SINGLE, 14484 },
	{ 0xdee2, 0xdee2, PDF_CMAP_SINGLE, 18714 },
	{ 0xdee2, 0xdee2, PDF_CMAP_SINGLE, 16005 },
	{ 0xdee2, 0xdee2, PDF_CMAP_SINGLE, 14487 },
	{ 0xdee3, 0xdee3, PDF_CMAP_SINGLE, 18443 },
	{ 0xdee3, 0xdee3, PDF_CMAP_SINGLE, 15130 },
	{ 0xdee3, 0xdee3, PDF_CMAP_SINGLE, 15099 },
	{ 0xdee5, 0xdee5, PDF_CMAP_SINGLE, 16447 },
	{ 0xdee5, 0xdee5, PDF_CMAP_SINGLE, 14488 },
	{ 0xdee6, 0xdee6, PDF_CMAP_SINGLE, 15140 },
	{ 0xdee6, 0xdee6, PDF_CMAP_SINGLE, 17804 },
	{ 0xdee6, 0xdee6, PDF_CMAP_SINGLE, 18564 },
	{ 0xdee6, 0xdee6, PDF_CMAP_SINGLE, 14463 },
	{ 0xdee7, 0xdee7, PDF_CMAP_SINGLE, 17043 },
	{ 0xdee7, 0xdee7, PDF_CMAP_SINGLE, 15327 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 16503 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 18474 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 14238 },
	{ 0xdee9, 0xdee9, PDF_CMAP_SINGLE, 14534 },
	{ 0xdee9, 0xdee9, PDF_CMAP_SINGLE, 15201 },
	{ 0xdeea, 0xdeea, PDF_CMAP_SINGLE, 18885 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 14739 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 14895 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 18501 },
	{ 0xdeef, 0xdeef, PDF_CMAP_SINGLE, 18084 },
	{ 0xdef4, 0xdef4, PDF_CMAP_SINGLE, 18667 },
	{ 0xdef4, 0xdef4, PDF_CMAP_SINGLE, 15709 },
	{ 0xdef5, 0xdef5, PDF_CMAP_SINGLE, 15157 },
	{ 0xdef6, 0xdef6, PDF_CMAP_SINGLE, 14245 },
	{ 0xdef6, 0xdef6, PDF_CMAP_SINGLE, 15710 },
	{ 0xdef7, 0xdef7, PDF_CMAP_SINGLE, 18704 },
	{ 0xdef8, 0xdef8, PDF_CMAP_SINGLE, 15878 },
	{ 0xdef8, 0xdef8, PDF_CMAP_SINGLE, 15058 },
	{ 0xdef8, 0xdef8, PDF_CMAP_SINGLE, 18620 },
	{ 0xdef9, 0xdef9, PDF_CMAP_SINGLE, 16058 },
	{ 0xdef9, 0xdef9, PDF_CMAP_SINGLE, 15683 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 17904 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 14860 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 18138 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 15136 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 15510 },
	{ 0xdefb, 0xdefb, PDF_CMAP_SINGLE, 17907 },
	{ 0xdefb, 0xdefb, PDF_CMAP_SINGLE, 14714 },
	{ 0xdefb, 0xdefb, PDF_CMAP_SINGLE, 14736 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 14974 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 16902 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 18544 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 15308 },
	{ 0xdefe, 0xdefe, PDF_CMAP_SINGLE, 17332 },
	{ 0xdefe, 0xdefe, PDF_CMAP_SINGLE, 14903 },
	{ 0xdeff, 0xdeff, PDF_CMAP_SINGLE, 18626 },
	{ 0xdf00, 0xdf00, PDF_CMAP_SINGLE, 15707 },
	{ 0xdf02, 0xdf02, PDF_CMAP_SINGLE, 15399 },
	{ 0xdf03, 0xdf03, PDF_CMAP_SINGLE, 14882 },
	{ 0xdf03, 0xdf03, PDF_CMAP_SINGLE, 16881 },
	{ 0xdf04, 0xdf04, PDF_CMAP_SINGLE, 15698 },
	{ 0xdf05, 0xdf05, PDF_CMAP_SINGLE, 16417 },
	{ 0xdf05, 0xdf05, PDF_CMAP_SINGLE, 14285 },
	{ 0xdf05, 0xdf05, PDF_CMAP_SINGLE, 14602 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 18664 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 14573 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 15379 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 18586 },
	{ 0xdf0a, 0xdf0a, PDF_CMAP_SINGLE, 14779 },
	{ 0xdf0a, 0xdf0a, PDF_CMAP_SINGLE, 14527 },
	{ 0xdf0a, 0xdf0a, PDF_CMAP_SINGLE, 18082 },
	{ 0xdf0b, 0xdf0b, PDF_CMAP_SINGLE, 18574 },
	{ 0xdf0c, 0xdf0c, PDF_CMAP_SINGLE, 14479 },
	{ 0xdf0d, 0xdf0d, PDF_CMAP_SINGLE, 18081 },
	{ 0xdf0d, 0xdf0d, PDF_CMAP_SINGLE, 16548 },
	{ 0xdf0d, 0xdf0d, PDF_CMAP_SINGLE, 18502 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 14819 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18475 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18751 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 14153 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18616 },
	{ 0xdf0f, 0xdf0f, PDF_CMAP_SINGLE, 16139 },
	{ 0xdf0f, 0xdf0f, PDF_CMAP_SINGLE, 19083 },
	{ 0xdf0f, 0xdf0f, PDF_CMAP_SINGLE, 14369 },
	{ 0xdf0f, 0xdf0f, PDF_CMAP_SINGLE, 18877 },
	{ 0xdf10, 0xdf10, PDF_CMAP_SINGLE, 14862 },
	{ 0xdf11, 0xdf11, PDF_CMAP_SINGLE, 18653 },
	{ 0xdf12, 0xdf12, PDF_CMAP_SINGLE, 18909 },
	{ 0xdf13, 0xdf13, PDF_CMAP_SINGLE, 16832 },
	{ 0xdf13, 0xdf13, PDF_CMAP_SINGLE, 16485 },
	{ 0xdf13, 0xdf13, PDF_CMAP_SINGLE, 14461 },
	{ 0xdf13, 0xdf13, PDF_CMAP_SINGLE, 14763 },
	{ 0xdf14, 0xdf14, PDF_CMAP_SINGLE, 14234 },
	{ 0xdf15, 0xdf15, PDF_CMAP_SINGLE, 18460 },
	{ 0xdf15, 0xdf15, PDF_CMAP_SINGLE, 18958 },
	{ 0xdf16, 0xdf16, PDF_CMAP_SINGLE, 18495 },
	{ 0xdf16, 0xdf16, PDF_CMAP_SINGLE, 17224 },
	{ 0xdf18, 0xdf18, PDF_CMAP_SINGLE, 15711 },
	{ 0xdf18, 0xdf18, PDF_CMAP_SINGLE, 15914 },
	{ 0xdf1a, 0xdf1a, PDF_CMAP_SINGLE, 19033 },
	{ 0xdf1a, 0xdf1a, PDF_CMAP_SINGLE, 15046 },
	{ 0xdf1a, 0xdf1a, PDF_CMAP_SINGLE, 16344 },
	{ 0xdf1b, 0xdf1b, PDF_CMAP_SINGLE, 15837 },
	{ 0xdf1d, 0xdf1d, PDF_CMAP_SINGLE, 15506 },
	{ 0xdf1d, 0xdf1d, PDF_CMAP_SINGLE, 16792 },
	{ 0xdf1f, 0xdf1f, PDF_CMAP_SINGLE, 15404 },
	{ 0xdf1f, 0xdf1f, PDF_CMAP_SINGLE, 18490 },
	{ 0xdf20, 0xdf20, PDF_CMAP_SINGLE, 18703 },
	{ 0xdf20, 0xdf20, PDF_CMAP_SINGLE, 17279 },
	{ 0xdf21, 0xdf21, PDF_CMAP_SINGLE, 15292 },
	{ 0xdf21, 0xdf21, PDF_CMAP_SINGLE, 16942 },
	{ 0xdf22, 0xdf22, PDF_CMAP_SINGLE, 14943 },
	{ 0xdf22, 0xdf22, PDF_CMAP_SINGLE, 14972 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18995 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18621 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 18068 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 14451 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 15053 },
	{ 0xdf25, 0xdf25, PDF_CMAP_SINGLE, 14596 },
	{ 0xdf26, 0xdf26, PDF_CMAP_SINGLE, 15358 },
	{ 0xdf26, 0xdf26, PDF_CMAP_SINGLE, 16497 },
	{ 0xdf26, 0xdf26, PDF_CMAP_SINGLE, 14741 },
	{ 0xdf28, 0xdf28, PDF_CMAP_SINGLE, 16670 },
	{ 0xdf2a, 0xdf2a, PDF_CMAP_SINGLE, 18468 },
	{ 0xdf2b, 0xdf2b, PDF_CMAP_SINGLE, 18783 },
	{ 0xdf2c, 0xdf2c, PDF_CMAP_SINGLE, 18636 },
	{ 0xdf2d, 0xdf2d, PDF_CMAP_SINGLE, 18430 },
	{ 0xdf2d, 0xdf2d, PDF_CMAP_SINGLE, 16471 },
	{ 0xdf2d, 0xdf2d, PDF_CMAP_SINGLE, 18735 },
	{ 0xdf2e, 0xdf2e, PDF_CMAP_SINGLE, 17888 },
	{ 0xdf2e, 0xdf2e, PDF_CMAP_SINGLE, 16106 },
	{ 0xdf2f, 0xdf2f, PDF_CMAP_SINGLE, 15898 },
	{ 0xdf2f, 0xdf2f, PDF_CMAP_SINGLE, 18509 },
	{ 0xdf2f, 0xdf2f, PDF_CMAP_SINGLE, 18587 },
	{ 0xdf2f, 0xdf2f, PDF_CMAP_SINGLE, 15347 },
	{ 0xdf30, 0xdf30, PDF_CMAP_SINGLE, 17870 },
	{ 0xdf30, 0xdf30, PDF_CMAP_SINGLE, 18698 },
	{ 0xdf31, 0xdf31, PDF_CMAP_SINGLE, 17232 },
	{ 0xdf31, 0xdf31, PDF_CMAP_SINGLE, 17859 },
	{ 0xdf32, 0xdf32, PDF_CMAP_SINGLE, 18610 },
	{ 0xdf34, 0xdf34, PDF_CMAP_SINGLE, 18503 },
	{ 0xdf35, 0xdf35, PDF_CMAP_SINGLE, 15067 },
	{ 0xdf35, 0xdf35, PDF_CMAP_SINGLE, 16133 },
	{ 0xdf35, 0xdf35, PDF_CMAP_SINGLE, 17723 },
	{ 0xdf36, 0xdf36, PDF_CMAP_SINGLE, 18445 },
	{ 0xdf36, 0xdf36, PDF_CMAP_SINGLE, 15684 },
	{ 0xdf38, 0xdf38, PDF_CMAP_SINGLE, 14388 },
	{ 0xdf39, 0xdf39, PDF_CMAP_SINGLE, 17151 },
	{ 0xdf3a, 0xdf3a, PDF_CMAP_SINGLE, 16717 },
	{ 0xdf3a, 0xdf3a, PDF_CMAP_SINGLE, 15659 },
	{ 0xdf3a, 0xdf3a, PDF_CMAP_SINGLE, 14386 },
	{ 0xdf3a, 0xdf3a, PDF_CMAP_SINGLE, 14221 },
	{ 0xdf3b, 0xdf3b, PDF_CMAP_SINGLE, 16338 },
	{ 0xdf3b, 0xdf3b, PDF_CMAP_SINGLE, 14915 },
	{ 0xdf3c, 0xdf3c, PDF_CMAP_SINGLE, 15673 },
	{ 0xdf3d, 0xdf3d, PDF_CMAP_SINGLE, 17947 },
	{ 0xdf3f, 0xdf3f, PDF_CMAP_SINGLE, 18476 },
	{ 0xdf3f, 0xdf3f, PDF_CMAP_SINGLE, 14750 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 17169 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 18054 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 15515 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 15129 },
	{ 0xdf43, 0xdf43, PDF_CMAP_SINGLE, 16032 },
	{ 0xdf44, 0xdf44, PDF_CMAP_SINGLE, 16665 },
	{ 0xdf45, 0xdf45, PDF_CMAP_SINGLE, 14847 },
	{ 0xdf46, 0xdf46, PDF_CMAP_SINGLE, 15776 },
	{ 0xdf46, 0xdf46, PDF_CMAP_SINGLE, 18596 },
	{ 0xdf47, 0xdf47, PDF_CMAP_SINGLE, 16430 },
	{ 0xdf48, 0xdf48, PDF_CMAP_SINGLE, 14681 },
	{ 0xdf48, 0xdf48, PDF_CMAP_SINGLE, 15186 },
	{ 0xdf4a, 0xdf4a, PDF_CMAP_SINGLE, 15809 },
	{ 0xdf4b, 0xdf4b, PDF_CMAP_SINGLE, 14719 },
	{ 0xdf4b, 0xdf4b, PDF_CMAP_SINGLE, 17660 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 17243 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 18754 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 17231 },
	{ 0xdf4e, 0xdf4e, PDF_CMAP_SINGLE, 15801 },
	{ 0xdf50, 0xdf50, PDF_CMAP_SINGLE, 17889 },
	{ 0xdf50, 0xdf50, PDF_CMAP_SINGLE, 15368 },
	{ 0xdf51, 0xdf51, PDF_CMAP_SINGLE, 14215 },
	{ 0xdf51, 0xdf51, PDF_CMAP_SINGLE, 16907 },
	{ 0xdf51, 0xdf51, PDF_CMAP_SINGLE, 18987 },
	{ 0xdf52, 0xdf52, PDF_CMAP_SINGLE, 15812 },
	{ 0xdf52, 0xdf52, PDF_CMAP_SINGLE, 18983 },
	{ 0xdf53, 0xdf53, PDF_CMAP_SINGLE, 16542 },
	{ 0xdf57, 0xdf57, PDF_CMAP_SINGLE, 15470 },
	{ 0xdf5a, 0xdf5a, PDF_CMAP_SINGLE, 16900 },
	{ 0xdf5b, 0xdf5b, PDF_CMAP_SINGLE, 18555 },
	{ 0xdf5c, 0xdf5c, PDF_CMAP_SINGLE, 17364 },
	{ 0xdf5e, 0xdf5e, PDF_CMAP_SINGLE, 17303 },
	{ 0xdf61, 0xdf61, PDF_CMAP_SINGLE, 19000 },
	{ 0xdf62, 0xdf62, PDF_CMAP_SINGLE, 15631 },
	{ 0xdf63, 0xdf63, PDF_CMAP_SINGLE, 16595 },
	{ 0xdf63, 0xdf63, PDF_CMAP_SINGLE, 15526 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 17191 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 14648 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 15370 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 16998 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 19134 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 14436 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 15385 },
	{ 0xdf66, 0xdf66, PDF_CMAP_SINGLE, 14220 },
	{ 0xdf66, 0xdf66, PDF_CMAP_SINGLE, 16581 },
	{ 0xdf6a, 0xdf6a, PDF_CMAP_SINGLE, 16731 },
	{ 0xdf6a, 0xdf6a, PDF_CMAP_SINGLE, 14941 },
	{ 0xdf6c, 0xdf6c, PDF_CMAP_SINGLE, 18711 },
	{ 0xdf6c, 0xdf6c, PDF_CMAP_SINGLE, 18639 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 16193 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 14199 },
	{ 0xdf6e, 0xdf6e, PDF_CMAP_SINGLE, 15148 },
	{ 0xdf6e, 0xdf6e, PDF_CMAP_SINGLE, 18692 },
	{ 0xdf6f, 0xdf6f, PDF_CMAP_SINGLE, 14861 },
	{ 0xdf70, 0xdf70, PDF_CMAP_SINGLE, 16779 },
	{ 0xdf71, 0xdf71, PDF_CMAP_SINGLE, 16703 },
	{ 0xdf72, 0xdf72, PDF_CMAP_SINGLE, 16520 },
	{ 0xdf73, 0xdf73, PDF_CMAP_SINGLE, 14266 },
	{ 0xdf73, 0xdf73, PDF_CMAP_SINGLE, 14927 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 14717 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 18486 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 17956 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 16937 },
	{ 0xdf75, 0xdf75, PDF_CMAP_SINGLE, 18444 },
	{ 0xdf75, 0xdf75, PDF_CMAP_SINGLE, 17257 },
	{ 0xdf75, 0xdf75, PDF_CMAP_SINGLE, 15190 },
	{ 0xdf76, 0xdf76, PDF_CMAP_SINGLE, 18781 },
	{ 0xdf79, 0xdf79, PDF_CMAP_SINGLE, 15805 },
	{ 0xdf7d, 0xdf7d, PDF_CMAP_SINGLE, 16263 },
	{ 0xdf7e, 0xdf7e, PDF_CMAP_SINGLE, 16427 },
	{ 0xdf7f, 0xdf7f, PDF_CMAP_SINGLE, 18107 },
	{ 0xdf7f, 0xdf7f, PDF_CMAP_SINGLE, 15036 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 16429 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 16490 },
	{ 0xdf81, 0xdf81, PDF_CMAP_SINGLE, 15411 },
	{ 0xdf81, 0xdf81, PDF_CMAP_SINGLE, 15110 },
	{ 0xdf81, 0xdf81, PDF_CMAP_SINGLE, 15935 },
	{ 0xdf82, 0xdf82, PDF_CMAP_SINGLE, 16946 },
	{ 0xdf82, 0xdf82, PDF_CMAP_SINGLE, 17090 },
	{ 0xdf82, 0xdf82, PDF_CMAP_SINGLE, 15439 },
	{ 0xdf84, 0xdf84, PDF_CMAP_SINGLE, 15256 },
	{ 0xdf85, 0xdf85, PDF_CMAP_SINGLE, 15255 },
	{ 0xdf86, 0xdf86, PDF_CMAP_SINGLE, 18515 },
	{ 0xdf89, 0xdf89, PDF_CMAP_SINGLE, 18530 },
	{ 0xdf8a, 0xdf8a, PDF_CMAP_SINGLE, 18588 },
	{ 0xdf8c, 0xdf8c, PDF_CMAP_SINGLE, 14827 },
	{ 0xdf8d, 0xdf8d, PDF_CMAP_SINGLE, 17867 },
	{ 0xdf8e, 0xdf8e, PDF_CMAP_SINGLE, 16787 },
	{ 0xdf8f, 0xdf8f, PDF_CMAP_SINGLE, 16530 },
	{ 0xdf8f, 0xdf8f, PDF_CMAP_SINGLE, 18091 },
	{ 0xdf8f, 0xdf8f, PDF_CMAP_SINGLE, 15052 },
	{ 0xdf8f, 0xdf8f, PDF_CMAP_SINGLE, 17073 },
	{ 0xdf90, 0xdf90, PDF_CMAP_SINGLE, 18429 },
	{ 0xdf96, 0xdf96, PDF_CMAP_SINGLE, 19029 },
	{ 0xdf96, 0xdf96, PDF_CMAP_SINGLE, 18504 },
	{ 0xdf96, 0xdf96, PDF_CMAP_SINGLE, 14311 },
	{ 0xdf97, 0xdf97, PDF_CMAP_SINGLE, 15491 },
	{ 0xdf97, 0xdf97, PDF_CMAP_SINGLE, 14286 },
	{ 0xdf98, 0xdf98, PDF_CMAP_SINGLE, 15638 },
	{ 0xdf99, 0xdf99, PDF_CMAP_SINGLE, 18546 },
	{ 0xdf99, 0xdf99, PDF_CMAP_SINGLE, 18640 },
	{ 0xdf9a, 0xdf9a, PDF_CMAP_SINGLE, 15827 },
	{ 0xdf9a, 0xdf9a, PDF_CMAP_SINGLE, 14345 },
	{ 0xdf9c, 0xdf9c, PDF_CMAP_SINGLE, 15596 },
	{ 0xdf9c, 0xdf9c, PDF_CMAP_SINGLE, 16837 },
	{ 0xdf9c, 0xdf9c, PDF_CMAP_SINGLE, 15259 },
	{ 0xdf9d, 0xdf9d, PDF_CMAP_SINGLE, 18556 },
	{ 0xdf9d, 0xdf9d, PDF_CMAP_SINGLE, 16767 },
	{ 0xdf9e, 0xdf9e, PDF_CMAP_SINGLE, 18144 },
	{ 0xdf9f, 0xdf9f, PDF_CMAP_SINGLE, 19057 },
	{ 0xdf9f, 0xdf9f, PDF_CMAP_SINGLE, 17113 },
	{ 0xdfa0, 0xdfa0, PDF_CMAP_SINGLE, 18398 },
	{ 0xdfa1, 0xdfa1, PDF_CMAP_SINGLE, 15938 },
	{ 0xdfa1, 0xdfa1, PDF_CMAP_SINGLE, 16643 },
	{ 0xdfa2, 0xdfa2, PDF_CMAP_SINGLE, 15001 },
	{ 0xdfa7, 0xdfa7, PDF_CMAP_SINGLE, 15538 },
	{ 0xdfa8, 0xdfa8, PDF_CMAP_SINGLE, 14760 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 17875 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 18699 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 17716 },
	{ 0xdfab, 0xdfab, PDF_CMAP_SINGLE, 14913 },
	{ 0xdfad, 0xdfad, PDF_CMAP_SINGLE, 17882 },
	{ 0xdfb0, 0xdfb0, PDF_CMAP_SINGLE, 15175 },
	{ 0xdfb1, 0xdfb1, PDF_CMAP_SINGLE, 15883 },
	{ 0xdfb2, 0xdfb2, PDF_CMAP_SINGLE, 16627 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 15666 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 15170 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 18702 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 16416 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 17184 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 14276 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 16743 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 16443 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 16973 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 14158 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 15223 },
	{ 0xdfb5, 0xdfb5, PDF_CMAP_SINGLE, 18776 },
	{ 0xdfb5, 0xdfb5, PDF_CMAP_SINGLE, 17932 },
	{ 0xdfb5, 0xdfb5, PDF_CMAP_SINGLE, 15493 },
	{ 0xdfb5, 0xdfb5, PDF_CMAP_SINGLE, 17912 },
	{ 0xdfb6, 0xdfb6, PDF_CMAP_SINGLE, 17936 },
	{ 0xdfb7, 0xdfb7, PDF_CMAP_SINGLE, 16764 },
	{ 0xdfb7, 0xdfb7, PDF_CMAP_SINGLE, 14623 },
	{ 0xdfb8, 0xdfb8, PDF_CMAP_SINGLE, 18080 },
	{ 0xdfb9, 0xdfb9, PDF_CMAP_SINGLE, 19005 },
	{ 0xdfbc, 0xdfbc, PDF_CMAP_SINGLE, 18432 },
	{ 0xdfbc, 0xdfbc, PDF_CMAP_SINGLE, 14999 },
	{ 0xdfbd, 0xdfbd, PDF_CMAP_SINGLE, 15635 },
	{ 0xdfbd, 0xdfbd, PDF_CMAP_SINGLE, 15695 },
	{ 0xdfbe, 0xdfbe, PDF_CMAP_SINGLE, 15348 },
	{ 0xdfbe, 0xdfbe, PDF_CMAP_SINGLE, 18959 },
	{ 0xdfbf, 0xdfbf, PDF_CMAP_SINGLE, 18410 },
	{ 0xdfc0, 0xdfc0, PDF_CMAP_SINGLE, 15023 },
	{ 0xdfc0, 0xdfc0, PDF_CMAP_SINGLE, 15625 },
	{ 0xdfc1, 0xdfc1, PDF_CMAP_SINGLE, 17633 },
	{ 0xdfc1, 0xdfc1, PDF_CMAP_SINGLE, 15512 },
	{ 0xdfc2, 0xdfc2, PDF_CMAP_SINGLE, 18380 },
	{ 0xdfc2, 0xdfc2, PDF_CMAP_SINGLE, 16840 },
	{ 0xdfc2, 0xdfc2, PDF_CMAP_SINGLE, 18678 },
	{ 0xdfc2, 0xdfc2, PDF_CMAP_SINGLE, 18731 },
	{ 0xdfc3, 0xdfc3, PDF_CMAP_SINGLE, 18448 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 15900 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 16398 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 15360 },
	{ 0xdfc5, 0xdfc5, PDF_CMAP_SINGLE, 18599 },
	{ 0xdfc6, 0xdfc6, PDF_CMAP_SINGLE, 14849 },
	{ 0xdfc6, 0xdfc6, PDF_CMAP_SINGLE, 14726 },
	{ 0xdfc6, 0xdfc6, PDF_CMAP_SINGLE, 15164 },
	{ 0xdfc7, 0xdfc7, PDF_CMAP_SINGLE, 18449 },
	{ 0xdfc7, 0xdfc7, PDF_CMAP_SINGLE, 18526 },
	{ 0xdfc9, 0xdfc9, PDF_CMAP_SINGLE, 16874 },
	{ 0xdfca, 0xdfca, PDF_CMAP_SINGLE, 17219 },
	{ 0xdfcb, 0xdfcb, PDF_CMAP_SINGLE, 14777 },
	{ 0xdfcb, 0xdfcb, PDF_CMAP_SINGLE, 18412 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 18487 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 15819 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 16362 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 18568 },
	{ 0xdfcd, 0xdfcd, PDF_CMAP_SINGLE, 16893 },
	{ 0xdfce, 0xdfce, PDF_CMAP_SINGLE, 17293 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 16720 },
	{ 0xdfd4, 0xdfd4, PDF_CMAP_SINGLE, 18597 },
	{ 0xdfd4, 0xdfd4, PDF_CMAP_SINGLE, 17208 },
	{ 0xdfd5, 0xdfd5, PDF_CMAP_SINGLE, 15010 },
	{ 0xdfd5, 0xdfd5, PDF_CMAP_SINGLE, 14599 },
	{ 0xdfd6, 0xdfd6, PDF_CMAP_SINGLE, 15643 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 18466 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 15448 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 14878 },
	{ 0xdfd9, 0xdfd9, PDF_CMAP_SINGLE, 15536 },
	{ 0xdfd9, 0xdfd9, PDF_CMAP_SINGLE, 15258 },
	{ 0xdfda, 0xdfda, PDF_CMAP_SINGLE, 15296 },
	{ 0xdfda, 0xdfda, PDF_CMAP_SINGLE, 14684 },
	{ 0xdfdb, 0xdfdb, PDF_CMAP_SINGLE, 15663 },
	{ 0xdfdc, 0xdfdc, PDF_CMAP_SINGLE, 16624 },
	{ 0xdfde, 0xdfde, PDF_CMAP_SINGLE, 18491 },
	{ 0xdfde, 0xdfde, PDF_CMAP_SINGLE, 15655 },
	{ 0xdfde, 0xdfde, PDF_CMAP_SINGLE, 16090 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 14759 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 18134 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 15342 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 15438 },
	{ 0xdfe0, 0xdfe0, PDF_CMAP_SINGLE, 16066 },
	{ 0xdfe0, 0xdfe0, PDF_CMAP_SINGLE, 14127 },
	{ 0xdfe1, 0xdfe1, PDF_CMAP_SINGLE, 15839 },
	{ 0xdfe1, 0xdfe1, PDF_CMAP_SINGLE, 15062 },
	{ 0xdfe2, 0xdfe2, PDF_CMAP_SINGLE, 16495 },
	{ 0xdfe2, 0xdfe2, PDF_CMAP_SINGLE, 15780 },
	{ 0xdfe3, 0xdfe3, PDF_CMAP_SINGLE, 15633 },
	{ 0xdfe4, 0xdfe4, PDF_CMAP_SINGLE, 14303 },
	{ 0xdfe6, 0xdfe6, PDF_CMAP_SINGLE, 16852 },
	{ 0xdfe7, 0xdfe7, PDF_CMAP_SINGLE, 15553 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 17621 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 18480 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 17007 },
	{ 0xdfe9, 0xdfe9, PDF_CMAP_SINGLE, 15701 },
	{ 0xdfe9, 0xdfe9, PDF_CMAP_SINGLE, 14129 },
	{ 0xdfea, 0xdfea, PDF_CMAP_SINGLE, 18657 },
	{ 0xdfea, 0xdfea, PDF_CMAP_SINGLE, 15323 },
	{ 0xdfea, 0xdfea, PDF_CMAP_SINGLE, 15027 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 15562 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 18679 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 16532 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 14485 },
	{ 0xdfec, 0xdfec, PDF_CMAP_SINGLE, 18681 },
	{ 0xdfec, 0xdfec, PDF_CMAP_SINGLE, 16492 },
	{ 0xdfec, 0xdfec, PDF_CMAP_SINGLE, 14491 },
	{ 0xdfed, 0xdfed, PDF_CMAP_SINGLE, 18683 },
	{ 0xdfed, 0xdfed, PDF_CMAP_SINGLE, 15922 },
	{ 0xdfed, 0xdfed, PDF_CMAP_SINGLE, 15608 },
	{ 0xdfee, 0xdfee, PDF_CMAP_SINGLE, 15333 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 14705 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 15835 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 17355 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 19008 },
	{ 0xdff0, 0xdff0, PDF_CMAP_SINGLE, 16972 },
	{ 0xdff2, 0xdff2, PDF_CMAP_SINGLE, 16119 },
	{ 0xdff4, 0xdff4, PDF_CMAP_SINGLE, 17883 },
	{ 0xdff4, 0xdff4, PDF_CMAP_SINGLE, 15640 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 15613 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 18638 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 18730 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 16597 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 17103 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 15257 },
	{ 0xdff6, 0xdff6, PDF_CMAP_SINGLE, 18650 },
	{ 0xdff7, 0xdff7, PDF_CMAP_SINGLE, 15830 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 14712 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 14703 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 15197 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 14197 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 18911 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 16061 },
	{ 0xdffa, 0xdffa, PDF_CMAP_SINGLE, 15566 },
	{ 0xdffa, 0xdffa, PDF_CMAP_SINGLE, 18467 },
	{ 0xdffa, 0xdffa, PDF_CMAP_SINGLE, 15644 },
	{ 0xdffb, 0xdffb, PDF_CMAP_SINGLE, 18414 },
	{ 0xdffb, 0xdffb, PDF_CMAP_SINGLE, 16626 },
	{ 0xdffc, 0xdffc, PDF_CMAP_SINGLE, 18719 },
	{ 0xdffc, 0xdffc, PDF_CMAP_SINGLE, 16788 },
	{ 0xdffe, 0xdffe, PDF_CMAP_SINGLE, 14724 },
	{ 0xdffe, 0xdffe, PDF_CMAP_SINGLE, 15002 },
	{ 0xdfff, 0xdfff, PDF_CMAP_SINGLE, 16445 },
	{ 0xdfff, 0xdfff, PDF_CMAP_SINGLE, 19072 },
	{ 0xdfff, 0xdfff, PDF_CMAP_SINGLE, 14493 },
	{ 0xe000, 0xe01e, PDF_CMAP_RANGE, 16395 },
	{ 0xe01f, 0xe01f, PDF_CMAP_SINGLE, 2106 },
	{ 0xe020, 0xe025, PDF_CMAP_RANGE, 16427 },
	{ 0xe026, 0xe026, PDF_CMAP_SINGLE, 2557 },
	{ 0xe027, 0xe046, PDF_CMAP_RANGE, 16434 },
	{ 0xe047, 0xe048, PDF_CMAP_RANGE, 18758 },
	{ 0xe049, 0xe05a, PDF_CMAP_RANGE, 16468 },
	{ 0xe05b, 0xe05b, PDF_CMAP_SINGLE, 781 },
	{ 0xe05c, 0xe062, PDF_CMAP_RANGE, 16487 },
	{ 0xe063, 0xe063, PDF_CMAP_SINGLE, 363 },
	{ 0xe064, 0xe072, PDF_CMAP_RANGE, 16495 },
	{ 0xe073, 0xe073, PDF_CMAP_SINGLE, 2144 },
	{ 0xe074, 0xe0a4, PDF_CMAP_RANGE, 16511 },
	{ 0xe0a5, 0xe0a5, PDF_CMAP_SINGLE, 16002 },
	{ 0xe0a6, 0xe0af, PDF_CMAP_RANGE, 16561 },
	{ 0xe0b0, 0xe0b0, PDF_CMAP_SINGLE, 18760 },
	{ 0xe0b1, 0xe0ca, PDF_CMAP_RANGE, 16572 },
	{ 0xe0cb, 0xe0cb, PDF_CMAP_SINGLE, 18761 },
	{ 0xe0cc, 0xe0dd, PDF_CMAP_RANGE, 16599 },
	{ 0xe0de, 0xe0de, PDF_CMAP_SINGLE, 18762 },
	{ 0xe0df, 0xe0f2, PDF_CMAP_RANGE, 16618 },
	{ 0xe0f3, 0xe0f3, PDF_CMAP_SINGLE, 3107 },
	{ 0xe0f4, 0xe0f9, PDF_CMAP_RANGE, 16639 },
	{ 0xe0fa, 0xe0fa, PDF_CMAP_SINGLE, 18763 },
	{ 0xe0fb, 0xe0ff, PDF_CMAP_RANGE, 16645 },
	{ 0xe100, 0xe107, PDF_CMAP_RANGE, 16650 },
	{ 0xe108, 0xe108, PDF_CMAP_SINGLE, 18764 },
	{ 0xe109, 0xe12d, PDF_CMAP_RANGE, 16658 },
	{ 0xe12e, 0xe12e, PDF_CMAP_SINGLE, 6116 },
	{ 0xe12f, 0xe133, PDF_CMAP_RANGE, 16696 },
	{ 0xe134, 0xe134, PDF_CMAP_SINGLE, 16014 },
	{ 0xe135, 0xe143, PDF_CMAP_RANGE, 16702 },
	{ 0xe144, 0xe144, PDF_CMAP_SINGLE, 18765 },
	{ 0xe145, 0xe148, PDF_CMAP_RANGE, 16717 },
	{ 0xe149, 0xe149, PDF_CMAP_SINGLE, 8495 },
	{ 0xe14a, 0xe14b, PDF_CMAP_RANGE, 16721 },
	{ 0xe14c, 0xe14c, PDF_CMAP_SINGLE, 18766 },
	{ 0xe14d, 0xe15c, PDF_CMAP_RANGE, 16723 },
	{ 0xe15d, 0xe15d, PDF_CMAP_SINGLE, 18767 },
	{ 0xe15e, 0xe165, PDF_CMAP_RANGE, 16739 },
	{ 0xe166, 0xe166, PDF_CMAP_SINGLE, 16385 },
	{ 0xe167, 0xe167, PDF_CMAP_SINGLE, 18768 },
	{ 0xe168, 0xe16e, PDF_CMAP_RANGE, 16749 },
	{ 0xe16f, 0xe16f, PDF_CMAP_SINGLE, 18769 },
	{ 0xe170, 0xe190, PDF_CMAP_RANGE, 16756 },
	{ 0xe191, 0xe191, PDF_CMAP_SINGLE, 4447 },
	{ 0xe192, 0xe193, PDF_CMAP_RANGE, 16789 },
	{ 0xe194, 0xe195, PDF_CMAP_RANGE, 18770 },
	{ 0xe196, 0xe1a3, PDF_CMAP_RANGE, 16792 },
	{ 0xe1a4, 0xe1a4, PDF_CMAP_SINGLE, 18772 },
	{ 0xe1a5, 0xe1b9, PDF_CMAP_RANGE, 16807 },
	{ 0xe1ba, 0xe1ba, PDF_CMAP_SINGLE, 3193 },
	{ 0xe1bb, 0xe1bb, PDF_CMAP_SINGLE, 18773 },
	{ 0xe1bc, 0xe1c5, PDF_CMAP_RANGE, 16829 },
	{ 0xe1c6, 0xe1c6, PDF_CMAP_SINGLE, 18774 },
	{ 0xe1c7, 0xe1c8, PDF_CMAP_RANGE, 16839 },
	{ 0xe1c9, 0xe1c9, PDF_CMAP_SINGLE, 1219 },
	{ 0xe1ca, 0xe1df, PDF_CMAP_RANGE, 16842 },
	{ 0xe1e0, 0xe1e0, PDF_CMAP_SINGLE, 18775 },
	{ 0xe1e1, 0xe1ff, PDF_CMAP_RANGE, 16864 },
	{ 0xe200, 0xe200, PDF_CMAP_SINGLE, 16895 },
	{ 0xe201, 0xe201, PDF_CMAP_SINGLE, 18776 },
	{ 0xe202, 0xe22b, PDF_CMAP_RANGE, 16897 },
	{ 0xe22c, 0xe22c, PDF_CMAP_SINGLE, 6333 },
	{ 0xe22d, 0xe22d, PDF_CMAP_SINGLE, 3261 },
	{ 0xe22e, 0xe22f, PDF_CMAP_RANGE, 16941 },
	{ 0xe230, 0xe230, PDF_CMAP_SINGLE, 3237 },
	{ 0xe231, 0xe257, PDF_CMAP_RANGE, 16944 },
	{ 0xe258, 0xe258, PDF_CMAP_SINGLE, 18777 },
	{ 0xe259, 0xe265, PDF_CMAP_RANGE, 16984 },
	{ 0xe266, 0xe266, PDF_CMAP_SINGLE, 3278 },
	{ 0xe267, 0xe267, PDF_CMAP_SINGLE, 18778 },
	{ 0xe268, 0xe285, PDF_CMAP_RANGE, 16998 },
	{ 0xe286, 0xe286, PDF_CMAP_SINGLE, 15728 },
	{ 0xe287, 0xe2a0, PDF_CMAP_RANGE, 17029 },
	{ 0xe2a1, 0xe2a1, PDF_CMAP_SINGLE, 18779 },
	{ 0xe2a2, 0xe2a2, PDF_CMAP_SINGLE, 17055 },
	{ 0xe2a3, 0xe2a3, PDF_CMAP_SINGLE, 3716 },
	{ 0xe2a4, 0xe2ab, PDF_CMAP_RANGE, 17057 },
	{ 0xe2ac, 0xe2ac, PDF_CMAP_SINGLE, 18780 },
	{ 0xe2ad, 0xe2bb, PDF_CMAP_RANGE, 17065 },
	{ 0xe2bc, 0xe2bc, PDF_CMAP_SINGLE, 288 },
	{ 0xe2bd, 0xe2ee, PDF_CMAP_RANGE, 17080 },
	{ 0xe2ef, 0xe2ef, PDF_CMAP_SINGLE, 7080 },
	{ 0xe2f0, 0xe2f1, PDF_CMAP_RANGE, 18781 },
	{ 0xe2f2, 0xe2fe, PDF_CMAP_RANGE, 17131 },
	{ 0xe2ff, 0xe2ff, PDF_CMAP_SINGLE, 18783 },
	{ 0xe300, 0xe300, PDF_CMAP_SINGLE, 18784 },
	{ 0xe301, 0xe310, PDF_CMAP_RANGE, 17144 },
	{ 0xe311, 0xe315, PDF_CMAP_RANGE, 14123 },
	{ 0xe316, 0xe316, PDF_CMAP_SINGLE, 18072 },
	{ 0xe317, 0xe339, PDF_CMAP_RANGE, 14128 },
	{ 0xe33a, 0xe33a, PDF_CMAP_SINGLE, 4181 },
	{ 0xe33b, 0xe33b, PDF_CMAP_SINGLE, 14164 },
	{ 0xe33c, 0xe33d, PDF_CMAP_RANGE, 18073 },
	{ 0xe33e, 0xe33f, PDF_CMAP_RANGE, 14166 },
	{ 0xe340, 0xe340, PDF_CMAP_SINGLE, 11752 },
	{ 0xe341, 0xe346, PDF_CMAP_RANGE, 14169 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 18075 },
	{ 0xe348, 0xe34b, PDF_CMAP_RANGE, 14175 },
	{ 0xe34c, 0xe34c, PDF_CMAP_SINGLE, 18076 },
	{ 0xe34d, 0xe34e, PDF_CMAP_RANGE, 14180 },
	{ 0xe34f, 0xe34f, PDF_CMAP_SINGLE, 289 },
	{ 0xe350, 0xe354, PDF_CMAP_RANGE, 14182 },
	{ 0xe355, 0xe355, PDF_CMAP_SINGLE, 18077 },
	{ 0xe356, 0xe359, PDF_CMAP_RANGE, 14187 },
	{ 0xe35a, 0xe35a, PDF_CMAP_SINGLE, 4203 },
	{ 0xe35b, 0xe362, PDF_CMAP_RANGE, 14192 },
	{ 0xe363, 0xe363, PDF_CMAP_SINGLE, 4902 },
	{ 0xe364, 0xe366, PDF_CMAP_RANGE, 14201 },
	{ 0xe367, 0xe367, PDF_CMAP_SINGLE, 18078 },
	{ 0xe368, 0xe377, PDF_CMAP_RANGE, 14205 },
	{ 0xe378, 0xe378, PDF_CMAP_SINGLE, 18079 },
	{ 0xe379, 0xe37b, PDF_CMAP_RANGE, 14221 },
	{ 0xe37c, 0xe37c, PDF_CMAP_SINGLE, 1643 },
	{ 0xe37d, 0xe37e, PDF_CMAP_RANGE, 14225 },
	{ 0xe37f, 0xe37f, PDF_CMAP_SINGLE, 4910 },
	{ 0xe380, 0xe393, PDF_CMAP_RANGE, 14228 },
	{ 0xe394, 0xe394, PDF_CMAP_SINGLE, 18080 },
	{ 0xe395, 0xe39d, PDF_CMAP_RANGE, 14248 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 18081 },
	{ 0xe39f, 0xe3a4, PDF_CMAP_RANGE, 14257 },
	{ 0xe3a5, 0xe3a5, PDF_CMAP_SINGLE, 18082 },
	{ 0xe3a6, 0xe3c4, PDF_CMAP_RANGE, 14264 },
	{ 0xe3c5, 0xe3c5, PDF_CMAP_SINGLE, 3381 },
	{ 0xe3c6, 0xe3c6, PDF_CMAP_SINGLE, 14296 },
	{ 0xe3c7, 0xe3c7, PDF_CMAP_SINGLE, 18083 },
	{ 0xe3c8, 0xe3cc, PDF_CMAP_RANGE, 14297 },
	{ 0xe3cd, 0xe3cd, PDF_CMAP_SINGLE, 18084 },
	{ 0xe3ce, 0xe3d4, PDF_CMAP_RANGE, 14302 },
	{ 0xe3d5, 0xe3d5, PDF_CMAP_SINGLE, 18085 },
	{ 0xe3d6, 0xe3d6, PDF_CMAP_SINGLE, 14309 },
	{ 0xe3d7, 0xe3d7, PDF_CMAP_SINGLE, 10178 },
	{ 0xe3d8, 0xe3db, PDF_CMAP_RANGE, 14311 },
	{ 0xe3dc, 0xe3dc, PDF_CMAP_SINGLE, 8877 },
	{ 0xe3dd, 0xe3e6, PDF_CMAP_RANGE, 14316 },
	{ 0xe3e7, 0xe3e7, PDF_CMAP_SINGLE, 18086 },
	{ 0xe3e8, 0xe3fb, PDF_CMAP_RANGE, 14327 },
	{ 0xe3fc, 0xe3fc, PDF_CMAP_SINGLE, 18087 },
	{ 0xe3fd, 0xe3ff, PDF_CMAP_RANGE, 14348 },
	{ 0xe400, 0xe410, PDF_CMAP_RANGE, 14351 },
	{ 0xe411, 0xe411, PDF_CMAP_SINGLE, 18088 },
	{ 0xe412, 0xe412, PDF_CMAP_SINGLE, 14368 },
	{ 0xe413, 0xe413, PDF_CMAP_SINGLE, 18089 },
	{ 0xe414, 0xe415, PDF_CMAP_RANGE, 14369 },
	{ 0xe416, 0xe416, PDF_CMAP_SINGLE, 18090 },
	{ 0xe417, 0xe417, PDF_CMAP_SINGLE, 4940 },
	{ 0xe418, 0xe418, PDF_CMAP_SINGLE, 16377 },
	{ 0xe419, 0xe425, PDF_CMAP_RANGE, 14374 },
	{ 0xe426, 0xe426, PDF_CMAP_SINGLE, 18091 },
	{ 0xe427, 0xe42e, PDF_CMAP_RANGE, 14387 },
	{ 0xe42f, 0xe42f, PDF_CMAP_SINGLE, 18092 },
	{ 0xe430, 0xe447, PDF_CMAP_RANGE, 14396 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 18093 },
	{ 0xe449, 0xe449, PDF_CMAP_SINGLE, 14420 },
	{ 0xe44a, 0xe44a, PDF_CMAP_SINGLE, 2510 },
	{ 0xe44b, 0xe45f, PDF_CMAP_RANGE, 14422 },
	{ 0xe460, 0xe460, PDF_CMAP_SINGLE, 18094 },
	{ 0xe461, 0xe466, PDF_CMAP_RANGE, 14444 },
	{ 0xe467, 0xe469, PDF_CMAP_RANGE, 18095 },
	{ 0xe46a, 0xe477, PDF_CMAP_RANGE, 14451 },
	{ 0xe478, 0xe478, PDF_CMAP_SINGLE, 3015 },
	{ 0xe479, 0xe479, PDF_CMAP_SINGLE, 14466 },
	{ 0xe47a, 0xe47a, PDF_CMAP_SINGLE, 18098 },
	{ 0xe47b, 0xe484, PDF_CMAP_RANGE, 14467 },
	{ 0xe485, 0xe485, PDF_CMAP_SINGLE, 14057 },
	{ 0xe486, 0xe48e, PDF_CMAP_RANGE, 14477 },
	{ 0xe48f, 0xe48f, PDF_CMAP_SINGLE, 18099 },
	{ 0xe490, 0xe4a0, PDF_CMAP_RANGE, 14486 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 18100 },
	{ 0xe4a2, 0xe4c4, PDF_CMAP_RANGE, 14503 },
	{ 0xe4c5, 0xe4c5, PDF_CMAP_SINGLE, 5009 },
	{ 0xe4c6, 0xe4d9, PDF_CMAP_RANGE, 14539 },
	{ 0xe4da, 0xe4da, PDF_CMAP_SINGLE, 5796 },
	{ 0xe4db, 0xe4ff, PDF_CMAP_RANGE, 14560 },
	{ 0xe500, 0xe50c, PDF_CMAP_RANGE, 14597 },
	{ 0xe50d, 0xe50d, PDF_CMAP_SINGLE, 18101 },
	{ 0xe50e, 0xe515, PDF_CMAP_RANGE, 14610 },
	{ 0xe516, 0xe516, PDF_CMAP_SINGLE, 18102 },
	{ 0xe517, 0xe525, PDF_CMAP_RANGE, 14618 },
	{ 0xe526, 0xe526, PDF_CMAP_SINGLE, 18103 },
	{ 0xe527, 0xe527, PDF_CMAP_SINGLE, 14633 },
	{ 0xe528, 0xe528, PDF_CMAP_SINGLE, 18104 },
	{ 0xe529, 0xe544, PDF_CMAP_RANGE, 14634 },
	{ 0xe545, 0xe545, PDF_CMAP_SINGLE, 12402 },
	{ 0xe546, 0xe54d, PDF_CMAP_RANGE, 14663 },
	{ 0xe54e, 0xe54e, PDF_CMAP_SINGLE, 18105 },
	{ 0xe54f, 0xe588, PDF_CMAP_RANGE, 14672 },
	{ 0xe589, 0xe589, PDF_CMAP_SINGLE, 14728 },
	{ 0xe58a, 0xe5a8, PDF_CMAP_RANGE, 14731 },
	{ 0xe5a9, 0xe5a9, PDF_CMAP_SINGLE, 18106 },
	{ 0xe5aa, 0xe5b1, PDF_CMAP_RANGE, 14763 },
	{ 0xe5b2, 0xe5b2, PDF_CMAP_SINGLE, 18107 },
	{ 0xe5b3, 0xe5d1, PDF_CMAP_RANGE, 14772 },
	{ 0xe5d2, 0xe5d3, PDF_CMAP_RANGE, 281 },
	{ 0xe5d4, 0xe5d4, PDF_CMAP_SINGLE, 284 },
	{ 0xe5d5, 0xe5d5, PDF_CMAP_SINGLE, 283 },
	{ 0xe5d6, 0xe5ea, PDF_CMAP_RANGE, 14803 },
	{ 0xe5eb, 0xe5eb, PDF_CMAP_SINGLE, 16300 },
	{ 0xe5ec, 0xe5f3, PDF_CMAP_RANGE, 14825 },
	{ 0xe5f4, 0xe5f4, PDF_CMAP_SINGLE, 10620 },
	{ 0xe5f5, 0xe5ff, PDF_CMAP_RANGE, 14834 },
	{ 0xe600, 0xe607, PDF_CMAP_RANGE, 14845 },
	{ 0xe608, 0xe608, PDF_CMAP_SINGLE, 18108 },
	{ 0xe609, 0xe614, PDF_CMAP_RANGE, 14853 },
	{ 0xe615, 0xe615, PDF_CMAP_SINGLE, 18109 },
	{ 0xe616, 0xe649, PDF_CMAP_RANGE, 14865 },
	{ 0xe64a, 0xe64a, PDF_CMAP_SINGLE, 18110 },
	{ 0xe64b, 0xe669, PDF_CMAP_RANGE, 14917 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 18111 },
	{ 0xe66b, 0xe681, PDF_CMAP_RANGE, 14948 },
	{ 0xe682, 0xe682, PDF_CMAP_SINGLE, 18112 },
	{ 0xe683, 0xe6a4, PDF_CMAP_RANGE, 14971 },
	{ 0xe6a5, 0xe6a5, PDF_CMAP_SINGLE, 18113 },
	{ 0xe6a6, 0xe6a7, PDF_CMAP_RANGE, 15005 },
	{ 0xe6a8, 0xe6a8, PDF_CMAP_SINGLE, 18114 },
	{ 0xe6a9, 0xe6aa, PDF_CMAP_RANGE, 15008 },
	{ 0xe6ab, 0xe6ab, PDF_CMAP_SINGLE, 18115 },
	{ 0xe6ac, 0xe6c4, PDF_CMAP_RANGE, 15010 },
	{ 0xe6c5, 0xe6c5, PDF_CMAP_SINGLE, 18116 },
	{ 0xe6c6, 0xe6c6, PDF_CMAP_SINGLE, 7430 },
	{ 0xe6c7, 0xe6f7, PDF_CMAP_RANGE, 15036 },
	{ 0xe6f8, 0xe6f8, PDF_CMAP_SINGLE, 18117 },
	{ 0xe6f9, 0xe6ff, PDF_CMAP_RANGE, 15086 },
	{ 0xe700, 0xe726, PDF_CMAP_RANGE, 15093 },
	{ 0xe727, 0xe727, PDF_CMAP_SINGLE, 10657 },
	{ 0xe728, 0xe728, PDF_CMAP_SINGLE, 18118 },
	{ 0xe729, 0xe768, PDF_CMAP_RANGE, 15134 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 18119 },
	{ 0xe76a, 0xe775, PDF_CMAP_RANGE, 15198 },
	{ 0xe776, 0xe776, PDF_CMAP_SINGLE, 18120 },
	{ 0xe777, 0xe77a, PDF_CMAP_RANGE, 15210 },
	{ 0xe77b, 0xe77b, PDF_CMAP_SINGLE, 18121 },
	{ 0xe77c, 0xe7bf, PDF_CMAP_RANGE, 15215 },
	{ 0xe7c0, 0xe7c0, PDF_CMAP_SINGLE, 18122 },
	{ 0xe7c1, 0xe7d2, PDF_CMAP_RANGE, 15283 },
	{ 0xe7d3, 0xe7d3, PDF_CMAP_SINGLE, 6171 },
	{ 0xe7d4, 0xe7fc, PDF_CMAP_RANGE, 15302 },
	{ 0xe7fd, 0xe7fd, PDF_CMAP_SINGLE, 15919 },
	{ 0xe7fe, 0xe7ff, PDF_CMAP_RANGE, 15344 },
	{ 0xe800, 0xe809, PDF_CMAP_RANGE, 15346 },
	{ 0xe80a, 0xe80a, PDF_CMAP_SINGLE, 18123 },
	{ 0xe80b, 0xe822, PDF_CMAP_RANGE, 15357 },
	{ 0xe823, 0xe823, PDF_CMAP_SINGLE, 18124 },
	{ 0xe824, 0xe86a, PDF_CMAP_RANGE, 15382 },
	{ 0xe86b, 0xe86b, PDF_CMAP_SINGLE, 18125 },
	{ 0xe86c, 0xe883, PDF_CMAP_RANGE, 15453 },
	{ 0xe884, 0xe884, PDF_CMAP_SINGLE, 15478 },
	{ 0xe885, 0xe892, PDF_CMAP_RANGE, 15478 },
	{ 0xe893, 0xe893, PDF_CMAP_SINGLE, 11044 },
	{ 0xe894, 0xe8ff, PDF_CMAP_RANGE, 15493 },
	{ 0xe900, 0xe936, PDF_CMAP_RANGE, 15601 },
	{ 0xe937, 0xe938, PDF_CMAP_RANGE, 18126 },
	{ 0xe939, 0xe961, PDF_CMAP_RANGE, 15658 },
	{ 0xe962, 0xe962, PDF_CMAP_SINGLE, 18128 },
	{ 0xe963, 0xe967, PDF_CMAP_RANGE, 15700 },
	{ 0xe968, 0xe969, PDF_CMAP_RANGE, 18129 },
	{ 0xe96a, 0xe96a, PDF_CMAP_SINGLE, 15705 },
	{ 0xe96b, 0xe96c, PDF_CMAP_RANGE, 18131 },
	{ 0xe96d, 0xe96d, PDF_CMAP_SINGLE, 15706 },
	{ 0xe96e, 0xe973, PDF_CMAP_RANGE, 18133 },
	{ 0xe974, 0xe974, PDF_CMAP_SINGLE, 15707 },
	{ 0xe975, 0xe97f, PDF_CMAP_RANGE, 18139 },
	{ 0xe980, 0xe980, PDF_CMAP_SINGLE, 15708 },
	{ 0xe981, 0xe984, PDF_CMAP_RANGE, 18150 },
	{ 0xe985, 0xe985, PDF_CMAP_SINGLE, 17354 },
	{ 0xe986, 0xe986, PDF_CMAP_SINGLE, 18154 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 15709 },
	{ 0xe988, 0xe988, PDF_CMAP_SINGLE, 18155 },
	{ 0xe989, 0xe989, PDF_CMAP_SINGLE, 17356 },
	{ 0xe98a, 0xe98a, PDF_CMAP_SINGLE, 15710 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 18156 },
	{ 0xe98c, 0xe98c, PDF_CMAP_SINGLE, 17402 },
	{ 0xe98d, 0xe98d, PDF_CMAP_SINGLE, 18157 },
	{ 0xe98e, 0xe98f, PDF_CMAP_RANGE, 15711 },
	{ 0xe990, 0xe992, PDF_CMAP_RANGE, 18158 },
	{ 0xe993, 0xe993, PDF_CMAP_SINGLE, 15713 },
	{ 0xe994, 0xe994, PDF_CMAP_SINGLE, 18161 },
	{ 0xe995, 0xe995, PDF_CMAP_SINGLE, 15714 },
	{ 0xe996, 0xe996, PDF_CMAP_SINGLE, 18162 },
	{ 0xe997, 0xe998, PDF_CMAP_RANGE, 15715 },
	{ 0xe999, 0xe9a2, PDF_CMAP_RANGE, 18163 },
	{ 0xe9a3, 0xe9a3, PDF_CMAP_SINGLE, 17357 },
	{ 0xe9a4, 0xe9a8, PDF_CMAP_RANGE, 18173 },
	{ 0xe9a9, 0xe9aa, PDF_CMAP_RANGE, 17358 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 18178 },
	{ 0xe9ac, 0xe9ac, PDF_CMAP_SINGLE, 17361 },
	{ 0xe9ad, 0xe9af, PDF_CMAP_RANGE, 18179 },
	{ 0xe9b0, 0xe9b0, PDF_CMAP_SINGLE, 17364 },
	{ 0xe9b1, 0xe9b3, PDF_CMAP_RANGE, 18182 },
	{ 0xe9b4, 0xe9b4, PDF_CMAP_SINGLE, 15717 },
	{ 0xe9b5, 0xe9b7, PDF_CMAP_RANGE, 18185 },
	{ 0xe9b8, 0xe9b8, PDF_CMAP_SINGLE, 15718 },
	{ 0xe9b9, 0xe9bd, PDF_CMAP_RANGE, 18188 },
	{ 0xe9be, 0xe9be, PDF_CMAP_SINGLE, 15719 },
	{ 0xe9bf, 0xe9c0, PDF_CMAP_RANGE, 18193 },
	{ 0xe9c1, 0xe9c1, PDF_CMAP_SINGLE, 15720 },
	{ 0xe9c2, 0xe9c2, PDF_CMAP_SINGLE, 18195 },
	{ 0xe9c3, 0xe9c3, PDF_CMAP_SINGLE, 15721 },
	{ 0xe9c4, 0xe9c4, PDF_CMAP_SINGLE, 18196 },
	{ 0xe9c5, 0xe9c6, PDF_CMAP_RANGE, 17365 },
	{ 0xe9c7, 0xe9cc, PDF_CMAP_RANGE, 18197 },
	{ 0xe9cd, 0xe9cd, PDF_CMAP_SINGLE, 15722 },
	{ 0xe9ce, 0xe9ce, PDF_CMAP_SINGLE, 18203 },
	{ 0xe9cf, 0xe9cf, PDF_CMAP_SINGLE, 17367 },
	{ 0xe9d0, 0xe9d1, PDF_CMAP_RANGE, 18204 },
	{ 0xe9d2, 0xe9d2, PDF_CMAP_SINGLE, 17404 },
	{ 0xe9d3, 0xe9d3, PDF_CMAP_SINGLE, 15723 },
	{ 0xe9d4, 0xe9d4, PDF_CMAP_SINGLE, 18206 },
	{ 0xe9d5, 0xe9d5, PDF_CMAP_SINGLE, 15724 },
	{ 0xe9d6, 0xe9d6, PDF_CMAP_SINGLE, 18207 },
	{ 0xe9d7, 0xe9d7, PDF_CMAP_SINGLE, 17368 },
	{ 0xe9d8, 0xe9de, PDF_CMAP_RANGE, 18208 },
	{ 0xe9df, 0xe9df, PDF_CMAP_SINGLE, 15725 },
	{ 0xe9e0, 0xe9e3, PDF_CMAP_RANGE, 18215 },
	{ 0xe9e4, 0xe9e4, PDF_CMAP_SINGLE, 17369 },
	{ 0xe9e5, 0xe9eb, PDF_CMAP_RANGE, 18219 },
	{ 0xe9ec, 0xe9ec, PDF_CMAP_SINGLE, 17370 },
	{ 0xe9ed, 0xe9f3, PDF_CMAP_RANGE, 18226 },
	{ 0xe9f4, 0xe9f4, PDF_CMAP_SINGLE, 17372 },
	{ 0xe9f5, 0xe9f9, PDF_CMAP_RANGE, 18233 },
	{ 0xe9fa, 0xe9fa, PDF_CMAP_SINGLE, 15726 },
	{ 0xe9fb, 0xe9fd, PDF_CMAP_RANGE, 18238 },
	{ 0xe9fe, 0xe9fe, PDF_CMAP_SINGLE, 15727 },
	{ 0xe9ff, 0xe9ff, PDF_CMAP_SINGLE, 18241 },
	{ 0xea00, 0xea04, PDF_CMAP_RANGE, 18242 },
	{ 0xea05, 0xea05, PDF_CMAP_SINGLE, 15728 },
	{ 0xea06, 0xea07, PDF_CMAP_RANGE, 18247 },
	{ 0xea08, 0xea08, PDF_CMAP_SINGLE, 15729 },
	{ 0xea09, 0xea0f, PDF_CMAP_RANGE, 18249 },
	{ 0xea10, 0xea10, PDF_CMAP_SINGLE, 15730 },
	{ 0xea11, 0xea11, PDF_CMAP_SINGLE, 18256 },
	{ 0xea12, 0xea12, PDF_CMAP_SINGLE, 17349 },
	{ 0xea13, 0xea13, PDF_CMAP_SINGLE, 18257 },
	{ 0xea14, 0xea14, PDF_CMAP_SINGLE, 17350 },
	{ 0xea15, 0xea1b, PDF_CMAP_RANGE, 18258 },
	{ 0xea1c, 0xea1c, PDF_CMAP_SINGLE, 15731 },
	{ 0xea1d, 0xea1f, PDF_CMAP_RANGE, 18265 },
	{ 0xea20, 0xea20, PDF_CMAP_SINGLE, 17374 },
	{ 0xea21, 0xea23, PDF_CMAP_RANGE, 18268 },
	{ 0xea24, 0xea24, PDF_CMAP_SINGLE, 15732 },
	{ 0xea25, 0xea27, PDF_CMAP_RANGE, 18271 },
	{ 0xea28, 0xea28, PDF_CMAP_SINGLE, 15733 },
	{ 0xea29, 0xea37, PDF_CMAP_RANGE, 18274 },
	{ 0xea38, 0xea38, PDF_CMAP_SINGLE, 17376 },
	{ 0xea39, 0xea3a, PDF_CMAP_RANGE, 18289 },
	{ 0xea3b, 0xea3b, PDF_CMAP_SINGLE, 17378 },
	{ 0xea3c, 0xea40, PDF_CMAP_RANGE, 18291 },
	{ 0xea41, 0xea41, PDF_CMAP_SINGLE, 17379 },
	{ 0xea42, 0xea43, PDF_CMAP_RANGE, 18296 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 17381 },
	{ 0xea45, 0xea4c, PDF_CMAP_RANGE, 18298 },
	{ 0xea4d, 0xea4d, PDF_CMAP_SINGLE, 17375 },
	{ 0xea4e, 0xea4f, PDF_CMAP_RANGE, 18306 },
	{ 0xea50, 0xea50, PDF_CMAP_SINGLE, 15734 },
	{ 0xea51, 0xea51, PDF_CMAP_SINGLE, 18308 },
	{ 0xea52, 0xea52, PDF_CMAP_SINGLE, 17323 },
	{ 0xea53, 0xea53, PDF_CMAP_SINGLE, 18309 },
	{ 0xea54, 0xea54, PDF_CMAP_SINGLE, 17383 },
	{ 0xea55, 0xea55, PDF_CMAP_SINGLE, 18310 },
	{ 0xea56, 0xea56, PDF_CMAP_SINGLE, 17385 },
	{ 0xea57, 0xea5c, PDF_CMAP_RANGE, 18311 },
	{ 0xea5d, 0xea5d, PDF_CMAP_SINGLE, 17405 },
	{ 0xea5e, 0xea61, PDF_CMAP_RANGE, 18317 },
	{ 0xea62, 0xea62, PDF_CMAP_SINGLE, 15735 },
	{ 0xea63, 0xea76, PDF_CMAP_RANGE, 18321 },
	{ 0xea77, 0xea77, PDF_CMAP_SINGLE, 15736 },
	{ 0xea78, 0xea78, PDF_CMAP_SINGLE, 18341 },
	{ 0xea79, 0xea79, PDF_CMAP_SINGLE, 15737 },
	{ 0xea7a, 0xea85, PDF_CMAP_RANGE, 18342 },
	{ 0xea86, 0xea86, PDF_CMAP_SINGLE, 15738 },
	{ 0xea87, 0xea8b, PDF_CMAP_RANGE, 18354 },
	{ 0xea8c, 0xea8c, PDF_CMAP_SINGLE, 17327 },
	{ 0xea8d, 0xea8d, PDF_CMAP_SINGLE, 18359 },
	{ 0xea8e, 0xea8e, PDF_CMAP_SINGLE, 15739 },
	{ 0xea8f, 0xea92, PDF_CMAP_RANGE, 18360 },
	{ 0xea93, 0xea93, PDF_CMAP_SINGLE, 17389 },
	{ 0xea94, 0xea94, PDF_CMAP_SINGLE, 18364 },
	{ 0xea95, 0xea95, PDF_CMAP_SINGLE, 15740 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 17390 },
	{ 0xea97, 0xea97, PDF_CMAP_SINGLE, 18365 },
	{ 0xea98, 0xea98, PDF_CMAP_SINGLE, 17407 },
	{ 0xea99, 0xea9f, PDF_CMAP_RANGE, 18366 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 15741 },
	{ 0xeaa1, 0xeaa1, PDF_CMAP_SINGLE, 18373 },
	{ 0xeaa2, 0xeaa2, PDF_CMAP_SINGLE, 17393 },
	{ 0xeaa3, 0xeaaa, PDF_CMAP_RANGE, 18374 },
	{ 0xeaab, 0xeaab, PDF_CMAP_SINGLE, 15742 },
	{ 0xeaac, 0xeaad, PDF_CMAP_RANGE, 18382 },
	{ 0xeaae, 0xeaae, PDF_CMAP_SINGLE, 17395 },
	{ 0xeaaf, 0xeaaf, PDF_CMAP_SINGLE, 18384 },
	{ 0xeab0, 0xeab0, PDF_CMAP_SINGLE, 17394 },
	{ 0xeab1, 0xeab3, PDF_CMAP_RANGE, 18385 },
	{ 0xeab4, 0xeab4, PDF_CMAP_SINGLE, 17400 },
	{ 0xeab5, 0xeab5, PDF_CMAP_SINGLE, 17396 },
	{ 0xeab6, 0xeabc, PDF_CMAP_RANGE, 18388 },
	{ 0xeabd, 0xeabd, PDF_CMAP_SINGLE, 15743 },
	{ 0xeabe, 0xeac1, PDF_CMAP_RANGE, 18395 },
	{ 0xeac2, 0xeac2, PDF_CMAP_SINGLE, 15744 },
	{ 0xeac3, 0xeac3, PDF_CMAP_SINGLE, 18399 },
	{ 0xeac4, 0xeac4, PDF_CMAP_SINGLE, 15745 },
	{ 0xeac5, 0xeac5, PDF_CMAP_SINGLE, 18400 },
	{ 0xeac6, 0xeac6, PDF_CMAP_SINGLE, 15746 },
	{ 0xeac7, 0xeac7, PDF_CMAP_SINGLE, 18401 },
	{ 0xeac8, 0xeac8, PDF_CMAP_SINGLE, 17335 },
	{ 0xeac9, 0xead1, PDF_CMAP_RANGE, 18402 },
	{ 0xead2, 0xead2, PDF_CMAP_SINGLE, 15747 },
	{ 0xead3, 0xeada, PDF_CMAP_RANGE, 18411 },
	{ 0xeadb, 0xeadb, PDF_CMAP_SINGLE, 15748 },
	{ 0xeadc, 0xeadc, PDF_CMAP_SINGLE, 18419 },
	{ 0xeadd, 0xeadd, PDF_CMAP_SINGLE, 15749 },
	{ 0xeade, 0xeae3, PDF_CMAP_RANGE, 18420 },
	{ 0xeae4, 0xeae6, PDF_CMAP_RANGE, 15750 },
	{ 0xeae7, 0xeaec, PDF_CMAP_RANGE, 18426 },
	{ 0xeaed, 0xeaed, PDF_CMAP_SINGLE, 15753 },
	{ 0xeaee, 0xeaee, PDF_CMAP_SINGLE, 18432 },
	{ 0xeaef, 0xeaef, PDF_CMAP_SINGLE, 15754 },
	{ 0xeaf0, 0xeaf2, PDF_CMAP_RANGE, 18433 },
	{ 0xeaf3, 0xeaf3, PDF_CMAP_SINGLE, 15755 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 17328 },
	{ 0xeaf5, 0xeaf8, PDF_CMAP_RANGE, 18436 },
	{ 0xeaf9, 0xeaf9, PDF_CMAP_SINGLE, 17330 },
	{ 0xeafa, 0xeafc, PDF_CMAP_RANGE, 18440 },
	{ 0xeafd, 0xeafd, PDF_CMAP_SINGLE, 15756 },
	{ 0xeafe, 0xeaff, PDF_CMAP_RANGE, 18443 },
	{ 0xeb00, 0xeb00, PDF_CMAP_SINGLE, 18445 },
	{ 0xeb01, 0xeb01, PDF_CMAP_SINGLE, 15757 },
	{ 0xeb02, 0xeb05, PDF_CMAP_RANGE, 18446 },
	{ 0xeb06, 0xeb06, PDF_CMAP_SINGLE, 15758 },
	{ 0xeb07, 0xeb0f, PDF_CMAP_RANGE, 18450 },
	{ 0xeb10, 0xeb10, PDF_CMAP_SINGLE, 15759 },
	{ 0xeb11, 0xeb13, PDF_CMAP_RANGE, 18459 },
	{ 0xeb14, 0xeb14, PDF_CMAP_SINGLE, 15760 },
	{ 0xeb15, 0xeb1d, PDF_CMAP_RANGE, 18462 },
	{ 0xeb1e, 0xeb1e, PDF_CMAP_SINGLE, 15762 },
	{ 0xeb1f, 0xeb21, PDF_CMAP_RANGE, 18471 },
	{ 0xeb22, 0xeb22, PDF_CMAP_SINGLE, 15763 },
	{ 0xeb23, 0xeb23, PDF_CMAP_SINGLE, 18474 },
	{ 0xeb24, 0xeb24, PDF_CMAP_SINGLE, 15764 },
	{ 0xeb25, 0xeb25, PDF_CMAP_SINGLE, 18475 },
	{ 0xeb26, 0xeb26, PDF_CMAP_SINGLE, 15765 },
	{ 0xeb27, 0xeb27, PDF_CMAP_SINGLE, 18476 },
	{ 0xeb28, 0xeb29, PDF_CMAP_RANGE, 15766 },
	{ 0xeb2a, 0xeb2a, PDF_CMAP_SINGLE, 18477 },
	{ 0xeb2c, 0xeb39, PDF_CMAP_RANGE, 18478 },
	{ 0xeb3a, 0xeb3d, PDF_CMAP_RANGE, 15768 },
	{ 0xeb3e, 0xeb3f, PDF_CMAP_RANGE, 18492 },
	{ 0xeb40, 0xeb40, PDF_CMAP_SINGLE, 12112 },
	{ 0xeb41, 0xeb41, PDF_CMAP_SINGLE, 15773 },
	{ 0xeb42, 0xeb42, PDF_CMAP_SINGLE, 5925 },
	{ 0xeb43, 0xeb44, PDF_CMAP_RANGE, 18494 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 13037 },
	{ 0xeb46, 0xeb46, PDF_CMAP_SINGLE, 15776 },
	{ 0xeb47, 0xeb47, PDF_CMAP_SINGLE, 18496 },
	{ 0xeb48, 0xeb49, PDF_CMAP_RANGE, 15777 },
	{ 0xeb4a, 0xeb4a, PDF_CMAP_SINGLE, 18497 },
	{ 0xeb4b, 0xeb4c, PDF_CMAP_RANGE, 15779 },
	{ 0xeb4d, 0xeb4e, PDF_CMAP_RANGE, 18498 },
	{ 0xeb4f, 0xeb52, PDF_CMAP_RANGE, 15781 },
	{ 0xeb53, 0xeb53, PDF_CMAP_SINGLE, 18500 },
	{ 0xeb54, 0xeb54, PDF_CMAP_SINGLE, 15785 },
	{ 0xeb55, 0xeb56, PDF_CMAP_RANGE, 18501 },
	{ 0xeb57, 0xeb57, PDF_CMAP_SINGLE, 15786 },
	{ 0xeb58, 0xeb59, PDF_CMAP_RANGE, 18503 },
	{ 0xeb5a, 0xeb61, PDF_CMAP_RANGE, 15787 },
	{ 0xeb62, 0xeb65, PDF_CMAP_RANGE, 18505 },
	{ 0xeb66, 0xeb66, PDF_CMAP_SINGLE, 15795 },
	{ 0xeb67, 0xeb67, PDF_CMAP_SINGLE, 18509 },
	{ 0xeb68, 0xeb6d, PDF_CMAP_RANGE, 15796 },
	{ 0xeb6e, 0xeb6e, PDF_CMAP_SINGLE, 10969 },
	{ 0xeb6f, 0xeb71, PDF_CMAP_RANGE, 18510 },
	{ 0xeb72, 0xeb72, PDF_CMAP_SINGLE, 15803 },
	{ 0xeb73, 0xeb73, PDF_CMAP_SINGLE, 18513 },
	{ 0xeb74, 0xeb74, PDF_CMAP_SINGLE, 15804 },
	{ 0xeb75, 0xeb75, PDF_CMAP_SINGLE, 18514 },
	{ 0xeb76, 0xeb76, PDF_CMAP_SINGLE, 17360 },
	{ 0xeb77, 0xeb77, PDF_CMAP_SINGLE, 18515 },
	{ 0xeb78, 0xeb79, PDF_CMAP_RANGE, 15805 },
	{ 0xeb7a, 0xeb7a, PDF_CMAP_SINGLE, 18516 },
	{ 0xeb7b, 0xeb7b, PDF_CMAP_SINGLE, 15807 },
	{ 0xeb7c, 0xeb7c, PDF_CMAP_SINGLE, 18517 },
	{ 0xeb7d, 0xeb7d, PDF_CMAP_SINGLE, 15808 },
	{ 0xeb7e, 0xeb7f, PDF_CMAP_RANGE, 18518 },
	{ 0xeb80, 0xeb82, PDF_CMAP_RANGE, 15809 },
	{ 0xeb83, 0xeb84, PDF_CMAP_RANGE, 18520 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 15812 },
	{ 0xeb86, 0xeb86, PDF_CMAP_SINGLE, 7188 },
	{ 0xeb87, 0xeb87, PDF_CMAP_SINGLE, 15813 },
	{ 0xeb88, 0xeb88, PDF_CMAP_SINGLE, 18522 },
	{ 0xeb89, 0xeb89, PDF_CMAP_SINGLE, 15814 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 18523 },
	{ 0xeb8b, 0xeb8b, PDF_CMAP_SINGLE, 15815 },
	{ 0xeb8c, 0xeb8e, PDF_CMAP_RANGE, 18524 },
	{ 0xeb8f, 0xeb8f, PDF_CMAP_SINGLE, 15816 },
	{ 0xeb90, 0xeb90, PDF_CMAP_SINGLE, 18527 },
	{ 0xeb91, 0xeb93, PDF_CMAP_RANGE, 15817 },
	{ 0xeb94, 0xeb94, PDF_CMAP_SINGLE, 16890 },
	{ 0xeb95, 0xeb95, PDF_CMAP_SINGLE, 18528 },
	{ 0xeb96, 0xeb97, PDF_CMAP_RANGE, 15821 },
	{ 0xeb98, 0xeb9a, PDF_CMAP_RANGE, 18529 },
	{ 0xeb9b, 0xeb9b, PDF_CMAP_SINGLE, 15823 },
	{ 0xeb9c, 0xeb9d, PDF_CMAP_RANGE, 18532 },
	{ 0xeb9e, 0xeb9e, PDF_CMAP_SINGLE, 4841 },
	{ 0xeb9f, 0xeb9f, PDF_CMAP_SINGLE, 18534 },
	{ 0xeba0, 0xeba1, PDF_CMAP_RANGE, 15825 },
	{ 0xeba2, 0xeba2, PDF_CMAP_SINGLE, 18535 },
	{ 0xeba3, 0xeba4, PDF_CMAP_RANGE, 15827 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 18536 },
	{ 0xeba7, 0xeba8, PDF_CMAP_RANGE, 15829 },
	{ 0xeba9, 0xeba9, PDF_CMAP_SINGLE, 11438 },
	{ 0xebaa, 0xebaa, PDF_CMAP_SINGLE, 18538 },
	{ 0xebab, 0xebad, PDF_CMAP_RANGE, 15832 },
	{ 0xebae, 0xebae, PDF_CMAP_SINGLE, 18539 },
	{ 0xebaf, 0xebaf, PDF_CMAP_SINGLE, 15835 },
	{ 0xebb0, 0xebb0, PDF_CMAP_SINGLE, 18540 },
	{ 0xebb1, 0xebb1, PDF_CMAP_SINGLE, 15836 },
	{ 0xebb2, 0xebb3, PDF_CMAP_RANGE, 18541 },
	{ 0xebb4, 0xebb9, PDF_CMAP_RANGE, 15837 },
	{ 0xebba, 0xebba, PDF_CMAP_SINGLE, 6756 },
	{ 0xebbb, 0xebbb, PDF_CMAP_SINGLE, 18543 },
	{ 0xebbc, 0xebbc, PDF_CMAP_SINGLE, 15844 },
	{ 0xebbd, 0xebbd, PDF_CMAP_SINGLE, 18544 },
	{ 0xebbe, 0xebc2, PDF_CMAP_RANGE, 15845 },
	{ 0xebc3, 0xebc3, PDF_CMAP_SINGLE, 18545 },
	{ 0xebc4, 0xebc4, PDF_CMAP_SINGLE, 15850 },
	{ 0xebc5, 0xebc6, PDF_CMAP_RANGE, 18546 },
	{ 0xebc7, 0xebc7, PDF_CMAP_SINGLE, 15851 },
	{ 0xebc8, 0xebc8, PDF_CMAP_SINGLE, 18548 },
	{ 0xebc9, 0xebc9, PDF_CMAP_SINGLE, 5124 },
	{ 0xebca, 0xebca, PDF_CMAP_SINGLE, 18549 },
	{ 0xebcb, 0xebcd, PDF_CMAP_RANGE, 15853 },
	{ 0xebce, 0xebce, PDF_CMAP_SINGLE, 18550 },
	{ 0xebcf, 0xebcf, PDF_CMAP_SINGLE, 8996 },
	{ 0xebd0, 0xebd0, PDF_CMAP_SINGLE, 18551 },
	{ 0xebd1, 0xebd1, PDF_CMAP_SINGLE, 15857 },
	{ 0xebd2, 0xebd2, PDF_CMAP_SINGLE, 13418 },
	{ 0xebd3, 0xebd3, PDF_CMAP_SINGLE, 18552 },
	{ 0xebd4, 0xebd4, PDF_CMAP_SINGLE, 15858 },
	{ 0xebd5, 0xebd5, PDF_CMAP_SINGLE, 18553 },
	{ 0xebd6, 0xebd9, PDF_CMAP_RANGE, 15859 },
	{ 0xebda, 0xebdb, PDF_CMAP_RANGE, 18554 },
	{ 0xebdc, 0xebdd, PDF_CMAP_RANGE, 15863 },
	{ 0xebde, 0xebde, PDF_CMAP_SINGLE, 8849 },
	{ 0xebdf, 0xebdf, PDF_CMAP_SINGLE, 18556 },
	{ 0xebe0, 0xebe0, PDF_CMAP_SINGLE, 15866 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 18557 },
	{ 0xebe2, 0xebe3, PDF_CMAP_RANGE, 15867 },
	{ 0xebe4, 0xebe4, PDF_CMAP_SINGLE, 18558 },
	{ 0xebe5, 0xebe7, PDF_CMAP_RANGE, 15869 },
	{ 0xebe8, 0xebe9, PDF_CMAP_RANGE, 18559 },
	{ 0xebea, 0xebea, PDF_CMAP_SINGLE, 15872 },
	{ 0xebeb, 0xebec, PDF_CMAP_RANGE, 18561 },
	{ 0xebed, 0xebee, PDF_CMAP_RANGE, 15873 },
	{ 0xebef, 0xebef, PDF_CMAP_SINGLE, 18563 },
	{ 0xebf0, 0xebf0, PDF_CMAP_SINGLE, 15875 },
	{ 0xebf1, 0xebf1, PDF_CMAP_SINGLE, 18564 },
	{ 0xebf2, 0xebf3, PDF_CMAP_RANGE, 15876 },
	{ 0xebf4, 0xebf5, PDF_CMAP_RANGE, 18565 },
	{ 0xebf6, 0xebff, PDF_CMAP_RANGE, 15878 },
	{ 0xec00, 0xec00, PDF_CMAP_SINGLE, 18567 },
	{ 0xec01, 0xec01, PDF_CMAP_SINGLE, 6023 },
	{ 0xec02, 0xec02, PDF_CMAP_SINGLE, 2399 },
	{ 0xec03, 0xec07, PDF_CMAP_RANGE, 15890 },
	{ 0xec08, 0xec0a, PDF_CMAP_RANGE, 18568 },
	{ 0xec0b, 0xec12, PDF_CMAP_RANGE, 15895 },
	{ 0xec13, 0xec13, PDF_CMAP_SINGLE, 18571 },
	{ 0xec14, 0xec14, PDF_CMAP_SINGLE, 15903 },
	{ 0xec15, 0xec15, PDF_CMAP_SINGLE, 7833 },
	{ 0xec16, 0xec18, PDF_CMAP_RANGE, 15905 },
	{ 0xec19, 0xec1c, PDF_CMAP_RANGE, 18572 },
	{ 0xec1d, 0xec1f, PDF_CMAP_RANGE, 15908 },
	{ 0xec20, 0xec20, PDF_CMAP_SINGLE, 18576 },
	{ 0xec21, 0xec2a, PDF_CMAP_RANGE, 15911 },
	{ 0xec2b, 0xec2b, PDF_CMAP_SINGLE, 18577 },
	{ 0xec2c, 0xec2e, PDF_CMAP_RANGE, 15921 },
	{ 0xec2f, 0xec2f, PDF_CMAP_SINGLE, 18578 },
	{ 0xec30, 0xec31, PDF_CMAP_RANGE, 15924 },
	{ 0xec32, 0xec32, PDF_CMAP_SINGLE, 18579 },
	{ 0xec33, 0xec3e, PDF_CMAP_RANGE, 15926 },
	{ 0xec3f, 0xec41, PDF_CMAP_RANGE, 18580 },
	{ 0xec42, 0xec42, PDF_CMAP_SINGLE, 15938 },
	{ 0xec43, 0xec43, PDF_CMAP_SINGLE, 18583 },
	{ 0xec44, 0xec44, PDF_CMAP_SINGLE, 17384 },
	{ 0xec45, 0xec49, PDF_CMAP_RANGE, 18584 },
	{ 0xec4a, 0xec4a, PDF_CMAP_SINGLE, 15939 },
	{ 0xec4b, 0xec4c, PDF_CMAP_RANGE, 18589 },
	{ 0xec4d, 0xec4d, PDF_CMAP_SINGLE, 15940 },
	{ 0xec4e, 0xec4f, PDF_CMAP_RANGE, 18591 },
	{ 0xec50, 0xec51, PDF_CMAP_RANGE, 15942 },
	{ 0xec52, 0xec52, PDF_CMAP_SINGLE, 18593 },
	{ 0xec53, 0xec53, PDF_CMAP_SINGLE, 15944 },
	{ 0xec54, 0xec54, PDF_CMAP_SINGLE, 18594 },
	{ 0xec55, 0xec55, PDF_CMAP_SINGLE, 15945 },
	{ 0xec56, 0xec58, PDF_CMAP_RANGE, 18595 },
	{ 0xec59, 0xec59, PDF_CMAP_SINGLE, 15946 },
	{ 0xec5a, 0xec5a, PDF_CMAP_SINGLE, 18598 },
	{ 0xec5b, 0xec5b, PDF_CMAP_SINGLE, 9665 },
	{ 0xec5c, 0xec5d, PDF_CMAP_RANGE, 18599 },
	{ 0xec5e, 0xec5e, PDF_CMAP_SINGLE, 3340 },
	{ 0xec5f, 0xec64, PDF_CMAP_RANGE, 18601 },
	{ 0xec65, 0xec65, PDF_CMAP_SINGLE, 17353 },
	{ 0xec66, 0xec66, PDF_CMAP_SINGLE, 15948 },
	{ 0xec67, 0xec67, PDF_CMAP_SINGLE, 18607 },
	{ 0xec68, 0xec68, PDF_CMAP_SINGLE, 15949 },
	{ 0xec69, 0xec7b, PDF_CMAP_RANGE, 18608 },
	{ 0xec7c, 0xec7c, PDF_CMAP_SINGLE, 17397 },
	{ 0xec7d, 0xec7d, PDF_CMAP_SINGLE, 15950 },
	{ 0xec7e, 0xec81, PDF_CMAP_RANGE, 18627 },
	{ 0xec82, 0xec82, PDF_CMAP_SINGLE, 15951 },
	{ 0xec83, 0xec86, PDF_CMAP_RANGE, 18631 },
	{ 0xec87, 0xec8a, PDF_CMAP_RANGE, 15952 },
	{ 0xec8b, 0xec8b, PDF_CMAP_SINGLE, 18635 },
	{ 0xec8c, 0xec8c, PDF_CMAP_SINGLE, 15956 },
	{ 0xec8d, 0xec8d, PDF_CMAP_SINGLE, 18636 },
	{ 0xec8e, 0xec8f, PDF_CMAP_RANGE, 15957 },
	{ 0xec90, 0xec91, PDF_CMAP_RANGE, 18637 },
	{ 0xec92, 0xec92, PDF_CMAP_SINGLE, 15960 },
	{ 0xec93, 0xec94, PDF_CMAP_RANGE, 18639 },
	{ 0xec95, 0xec95, PDF_CMAP_SINGLE, 15961 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 18641 },
	{ 0xec97, 0xec97, PDF_CMAP_SINGLE, 15962 },
	{ 0xec98, 0xec98, PDF_CMAP_SINGLE, 18642 },
	{ 0xec99, 0xec99, PDF_CMAP_SINGLE, 15963 },
	{ 0xec9a, 0xec9d, PDF_CMAP_RANGE, 18643 },
	{ 0xec9e, 0xec9f, PDF_CMAP_RANGE, 15964 },
	{ 0xeca0, 0xeca0, PDF_CMAP_SINGLE, 18647 },
	{ 0xeca1, 0xeca2, PDF_CMAP_RANGE, 15966 },
	{ 0xeca3, 0xeca4, PDF_CMAP_RANGE, 18648 },
	{ 0xeca5, 0xeca5, PDF_CMAP_SINGLE, 15968 },
	{ 0xeca6, 0xeca6, PDF_CMAP_SINGLE, 1510 },
	{ 0xeca7, 0xeca8, PDF_CMAP_RANGE, 18650 },
	{ 0xeca9, 0xecaa, PDF_CMAP_RANGE, 15970 },
	{ 0xecab, 0xecab, PDF_CMAP_SINGLE, 18652 },
	{ 0xecac, 0xecac, PDF_CMAP_SINGLE, 15972 },
	{ 0xecad, 0xecae, PDF_CMAP_RANGE, 18653 },
	{ 0xecaf, 0xecb3, PDF_CMAP_RANGE, 15973 },
	{ 0xecb4, 0xecb4, PDF_CMAP_SINGLE, 18655 },
	{ 0xecb5, 0xecb7, PDF_CMAP_RANGE, 15978 },
	{ 0xecb8, 0xecbb, PDF_CMAP_RANGE, 18656 },
	{ 0xecbc, 0xecdd, PDF_CMAP_RANGE, 15981 },
	{ 0xecde, 0xecde, PDF_CMAP_SINGLE, 18660 },
	{ 0xecdf, 0xece3, PDF_CMAP_RANGE, 16015 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 18661 },
	{ 0xece5, 0xecff, PDF_CMAP_RANGE, 16021 },
	{ 0xed00, 0xed00, PDF_CMAP_SINGLE, 18662 },
	{ 0xed01, 0xed03, PDF_CMAP_RANGE, 16049 },
	{ 0xed04, 0xed04, PDF_CMAP_SINGLE, 18663 },
	{ 0xed05, 0xed06, PDF_CMAP_RANGE, 16052 },
	{ 0xed07, 0xed08, PDF_CMAP_RANGE, 18664 },
	{ 0xed09, 0xed09, PDF_CMAP_SINGLE, 16054 },
	{ 0xed0a, 0xed0a, PDF_CMAP_SINGLE, 18666 },
	{ 0xed0b, 0xed0b, PDF_CMAP_SINGLE, 16055 },
	{ 0xed0c, 0xed11, PDF_CMAP_RANGE, 18667 },
	{ 0xed12, 0xed12, PDF_CMAP_SINGLE, 16056 },
	{ 0xed13, 0xed13, PDF_CMAP_SINGLE, 18673 },
	{ 0xed14, 0xed14, PDF_CMAP_SINGLE, 16057 },
	{ 0xed15, 0xed17, PDF_CMAP_RANGE, 18674 },
	{ 0xed18, 0xed19, PDF_CMAP_RANGE, 16058 },
	{ 0xed1a, 0xed1a, PDF_CMAP_SINGLE, 18677 },
	{ 0xed1b, 0xed1b, PDF_CMAP_SINGLE, 16060 },
	{ 0xed1c, 0xed1c, PDF_CMAP_SINGLE, 18678 },
	{ 0xed1d, 0xed1d, PDF_CMAP_SINGLE, 16061 },
	{ 0xed1e, 0xed1e, PDF_CMAP_SINGLE, 18679 },
	{ 0xed1f, 0xed21, PDF_CMAP_RANGE, 16062 },
	{ 0xed22, 0xed22, PDF_CMAP_SINGLE, 18680 },
	{ 0xed23, 0xed25, PDF_CMAP_RANGE, 16065 },
	{ 0xed26, 0xed27, PDF_CMAP_RANGE, 18681 },
	{ 0xed28, 0xed28, PDF_CMAP_SINGLE, 1832 },
	{ 0xed29, 0xed29, PDF_CMAP_SINGLE, 16069 },
	{ 0xed2a, 0xed2a, PDF_CMAP_SINGLE, 18683 },
	{ 0xed2b, 0xed2c, PDF_CMAP_RANGE, 16070 },
	{ 0xed2d, 0xed2d, PDF_CMAP_SINGLE, 18684 },
	{ 0xed2e, 0xed30, PDF_CMAP_RANGE, 16072 },
	{ 0xed31, 0xed32, PDF_CMAP_RANGE, 18685 },
	{ 0xed33, 0xed33, PDF_CMAP_SINGLE, 16075 },
	{ 0xed34, 0xed34, PDF_CMAP_SINGLE, 18687 },
	{ 0xed35, 0xed36, PDF_CMAP_RANGE, 16076 },
	{ 0xed37, 0xed37, PDF_CMAP_SINGLE, 18688 },
	{ 0xed38, 0xed38, PDF_CMAP_SINGLE, 16078 },
	{ 0xed39, 0xed3a, PDF_CMAP_RANGE, 18689 },
	{ 0xed3b, 0xed3b, PDF_CMAP_SINGLE, 16079 },
	{ 0xed3c, 0xed3d, PDF_CMAP_RANGE, 18691 },
	{ 0xed3e, 0xed3f, PDF_CMAP_RANGE, 16080 },
	{ 0xed40, 0xed42, PDF_CMAP_RANGE, 18693 },
	{ 0xed43, 0xed44, PDF_CMAP_RANGE, 16082 },
	{ 0xed45, 0xed45, PDF_CMAP_SINGLE, 18696 },
	{ 0xed46, 0xed49, PDF_CMAP_RANGE, 16084 },
	{ 0xed4a, 0xed4b, PDF_CMAP_RANGE, 18697 },
	{ 0xed4c, 0xed4c, PDF_CMAP_SINGLE, 16089 },
	{ 0xed4d, 0xed4e, PDF_CMAP_RANGE, 18699 },
	{ 0xed4f, 0xed50, PDF_CMAP_RANGE, 16090 },
	{ 0xed51, 0xed51, PDF_CMAP_SINGLE, 18701 },
	{ 0xed52, 0xed52, PDF_CMAP_SINGLE, 16092 },
	{ 0xed53, 0xed54, PDF_CMAP_RANGE, 18702 },
	{ 0xed55, 0xed56, PDF_CMAP_RANGE, 16093 },
	{ 0xed57, 0xed58, PDF_CMAP_RANGE, 18704 },
	{ 0xed59, 0xed6d, PDF_CMAP_RANGE, 16095 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 9398 },
	{ 0xed6f, 0xed70, PDF_CMAP_RANGE, 16117 },
	{ 0xed71, 0xed71, PDF_CMAP_SINGLE, 18706 },
	{ 0xed72, 0xed74, PDF_CMAP_RANGE, 16119 },
	{ 0xed75, 0xed77, PDF_CMAP_RANGE, 18707 },
	{ 0xed78, 0xed79, PDF_CMAP_RANGE, 16122 },
	{ 0xed7a, 0xed7a, PDF_CMAP_SINGLE, 18710 },
	{ 0xed7b, 0xed7b, PDF_CMAP_SINGLE, 16124 },
	{ 0xed7c, 0xed7c, PDF_CMAP_SINGLE, 6150 },
	{ 0xed7d, 0xed80, PDF_CMAP_RANGE, 16126 },
	{ 0xed81, 0xed81, PDF_CMAP_SINGLE, 18711 },
	{ 0xed82, 0xed85, PDF_CMAP_RANGE, 16130 },
	{ 0xed86, 0xed86, PDF_CMAP_SINGLE, 18712 },
	{ 0xed87, 0xed88, PDF_CMAP_RANGE, 16134 },
	{ 0xed89, 0xed8a, PDF_CMAP_RANGE, 18713 },
	{ 0xed8b, 0xed9d, PDF_CMAP_RANGE, 16136 },
	{ 0xed9e, 0xed9e, PDF_CMAP_SINGLE, 13585 },
	{ 0xed9f, 0xeda3, PDF_CMAP_RANGE, 16156 },
	{ 0xeda4, 0xeda4, PDF_CMAP_SINGLE, 14661 },
	{ 0xeda5, 0xeda6, PDF_CMAP_RANGE, 18715 },
	{ 0xeda7, 0xedad, PDF_CMAP_RANGE, 16163 },
	{ 0xedae, 0xedae, PDF_CMAP_SINGLE, 18717 },
	{ 0xedaf, 0xedd0, PDF_CMAP_RANGE, 16170 },
	{ 0xedd1, 0xedd1, PDF_CMAP_SINGLE, 18718 },
	{ 0xedd2, 0xedd6, PDF_CMAP_RANGE, 16204 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 18719 },
	{ 0xedd8, 0xedda, PDF_CMAP_RANGE, 16209 },
	{ 0xeddb, 0xeddb, PDF_CMAP_SINGLE, 18720 },
	{ 0xeddc, 0xeddc, PDF_CMAP_SINGLE, 16212 },
	{ 0xeddd, 0xeddd, PDF_CMAP_SINGLE, 18721 },
	{ 0xedde, 0xede6, PDF_CMAP_RANGE, 16213 },
	{ 0xede7, 0xede7, PDF_CMAP_SINGLE, 3970 },
	{ 0xede8, 0xede8, PDF_CMAP_SINGLE, 18722 },
	{ 0xede9, 0xedef, PDF_CMAP_RANGE, 16224 },
	{ 0xedf0, 0xedf0, PDF_CMAP_SINGLE, 18723 },
	{ 0xedf1, 0xedf3, PDF_CMAP_RANGE, 16232 },
	{ 0xedf4, 0xedf4, PDF_CMAP_SINGLE, 14910 },
	{ 0xedf5, 0xedff, PDF_CMAP_RANGE, 16236 },
	{ 0xee00, 0xee00, PDF_CMAP_SINGLE, 18724 },
	{ 0xee01, 0xee14, PDF_CMAP_RANGE, 16247 },
	{ 0xee15, 0xee15, PDF_CMAP_SINGLE, 18725 },
	{ 0xee16, 0xee1a, PDF_CMAP_RANGE, 16267 },
	{ 0xee1b, 0xee1b, PDF_CMAP_SINGLE, 18726 },
	{ 0xee1c, 0xee21, PDF_CMAP_RANGE, 16272 },
	{ 0xee22, 0xee22, PDF_CMAP_SINGLE, 18727 },
	{ 0xee23, 0xee2f, PDF_CMAP_RANGE, 16278 },
	{ 0xee30, 0xee30, PDF_CMAP_SINGLE, 18728 },
	{ 0xee31, 0xee3d, PDF_CMAP_RANGE, 16291 },
	{ 0xee3e, 0xee3e, PDF_CMAP_SINGLE, 14353 },
	{ 0xee3f, 0xee47, PDF_CMAP_RANGE, 16305 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 18729 },
	{ 0xee49, 0xee51, PDF_CMAP_RANGE, 16314 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 1522 },
	{ 0xee53, 0xee55, PDF_CMAP_RANGE, 16324 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 18730 },
	{ 0xee57, 0xee5a, PDF_CMAP_RANGE, 16327 },
	{ 0xee5b, 0xee5b, PDF_CMAP_SINGLE, 18731 },
	{ 0xee5c, 0xee5f, PDF_CMAP_RANGE, 16331 },
	{ 0xee60, 0xee60, PDF_CMAP_SINGLE, 18732 },
	{ 0xee61, 0xee7d, PDF_CMAP_RANGE, 16335 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 18733 },
	{ 0xee7f, 0xee88, PDF_CMAP_RANGE, 16364 },
	{ 0xee89, 0xee89, PDF_CMAP_SINGLE, 18734 },
	{ 0xee8a, 0xee8d, PDF_CMAP_RANGE, 16374 },
	{ 0xee8e, 0xee8e, PDF_CMAP_SINGLE, 15086 },
	{ 0xee8f, 0xee97, PDF_CMAP_RANGE, 16379 },
	{ 0xee98, 0xee98, PDF_CMAP_SINGLE, 9341 },
	{ 0xee99, 0xee9b, PDF_CMAP_RANGE, 16389 },
	{ 0xee9c, 0xee9c, PDF_CMAP_SINGLE, 18735 },
	{ 0xee9d, 0xee9d, PDF_CMAP_SINGLE, 15049 },
	{ 0xee9e, 0xee9f, PDF_CMAP_RANGE, 16393 },
	{ 0xeea0, 0xeea6, PDF_CMAP_RANGE, 18736 },
	{ 0xeea7, 0xeea7, PDF_CMAP_SINGLE, 17332 },
	{ 0xeea8, 0xeeaa, PDF_CMAP_RANGE, 18743 },
	{ 0xeeab, 0xeeab, PDF_CMAP_SINGLE, 17336 },
	{ 0xeeac, 0xeeb7, PDF_CMAP_RANGE, 18746 },
	{ 0xf303, 0xf318, PDF_CMAP_RANGE, 17609 },
	{ 0xf319, 0xf349, PDF_CMAP_RANGE, 18785 },
	{ 0xf34a, 0xf34b, PDF_CMAP_RANGE, 18844 },
	{ 0xf3a0, 0xf3a1, PDF_CMAP_RANGE, 17716 },
	{ 0xf3a3, 0xf3a3, PDF_CMAP_SINGLE, 17718 },
	{ 0xf3a6, 0xf3a9, PDF_CMAP_RANGE, 17719 },
	{ 0xf3ac, 0xf3ac, PDF_CMAP_SINGLE, 17723 },
	{ 0xf3ad, 0xf3ad, PDF_CMAP_SINGLE, 17347 },
	{ 0xf3ae, 0xf3b0, PDF_CMAP_RANGE, 17724 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 17306 },
	{ 0xf3b2, 0xf3e3, PDF_CMAP_RANGE, 17727 },
	{ 0xf3e4, 0xf3e4, PDF_CMAP_SINGLE, 17314 },
	{ 0xf3e9, 0xf3e9, PDF_CMAP_SINGLE, 17388 },
	{ 0xf3ea, 0xf3ea, PDF_CMAP_SINGLE, 17777 },
	{ 0xf3eb, 0xf3eb, PDF_CMAP_SINGLE, 17387 },
	{ 0xf3ec, 0xf3ec, PDF_CMAP_SINGLE, 17778 },
	{ 0xf3ee, 0xf3f0, PDF_CMAP_RANGE, 17779 },
	{ 0xf3f3, 0xf3fd, PDF_CMAP_RANGE, 17782 },
	{ 0xf3ff, 0xf3ff, PDF_CMAP_SINGLE, 17793 },
	{ 0xf400, 0xf401, PDF_CMAP_RANGE, 17794 },
	{ 0xf403, 0xf40c, PDF_CMAP_RANGE, 17796 },
	{ 0xf40d, 0xf40d, PDF_CMAP_SINGLE, 17340 },
	{ 0xf40e, 0xf416, PDF_CMAP_RANGE, 17806 },
	{ 0xf417, 0xf417, PDF_CMAP_SINGLE, 17308 },
	{ 0xf418, 0xf418, PDF_CMAP_SINGLE, 17815 },
	{ 0xf419, 0xf419, PDF_CMAP_SINGLE, 17310 },
	{ 0xf41a, 0xf41a, PDF_CMAP_SINGLE, 17816 },
	{ 0xf41b, 0xf41b, PDF_CMAP_SINGLE, 17311 },
	{ 0xf41c, 0xf41e, PDF_CMAP_RANGE, 17817 },
	{ 0xf41f, 0xf41f, PDF_CMAP_SINGLE, 17313 },
	{ 0xf420, 0xf420, PDF_CMAP_SINGLE, 17820 },
	{ 0xf421, 0xf421, PDF_CMAP_SINGLE, 17315 },
	{ 0xf422, 0xf427, PDF_CMAP_RANGE, 17821 },
	{ 0xf428, 0xf429, PDF_CMAP_RANGE, 17317 },
	{ 0xf42a, 0xf437, PDF_CMAP_RANGE, 17827 },
	{ 0xf438, 0xf438, PDF_CMAP_SINGLE, 17321 },
	{ 0xf439, 0xf43d, PDF_CMAP_RANGE, 17841 },
	{ 0xf43e, 0xf43e, PDF_CMAP_SINGLE, 17161 },
	{ 0xf440, 0xf449, PDF_CMAP_RANGE, 17163 },
	{ 0xf44a, 0xf44a, PDF_CMAP_SINGLE, 17846 },
	{ 0xf44b, 0xf456, PDF_CMAP_RANGE, 17174 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 17847 },
	{ 0xf458, 0xf45a, PDF_CMAP_RANGE, 17187 },
	{ 0xf45b, 0xf45b, PDF_CMAP_SINGLE, 17848 },
	{ 0xf45c, 0xf45f, PDF_CMAP_RANGE, 17191 },
	{ 0xf461, 0xf46d, PDF_CMAP_RANGE, 17196 },
	{ 0xf46e, 0xf46e, PDF_CMAP_SINGLE, 17849 },
	{ 0xf46f, 0xf471, PDF_CMAP_RANGE, 17210 },
	{ 0xf473, 0xf473, PDF_CMAP_SINGLE, 17214 },
	{ 0xf474, 0xf474, PDF_CMAP_SINGLE, 17850 },
	{ 0xf475, 0xf476, PDF_CMAP_RANGE, 17216 },
	{ 0xf477, 0xf477, PDF_CMAP_SINGLE, 17851 },
	{ 0xf478, 0xf478, PDF_CMAP_SINGLE, 17219 },
	{ 0xf479, 0xf479, PDF_CMAP_SINGLE, 17852 },
	{ 0xf47a, 0xf47a, PDF_CMAP_SINGLE, 17221 },
	{ 0xf47b, 0xf47b, PDF_CMAP_SINGLE, 17853 },
	{ 0xf47c, 0xf482, PDF_CMAP_RANGE, 17223 },
	{ 0xf483, 0xf483, PDF_CMAP_SINGLE, 17854 },
	{ 0xf484, 0xf485, PDF_CMAP_RANGE, 17231 },
	{ 0xf487, 0xf48b, PDF_CMAP_RANGE, 17234 },
	{ 0xf48d, 0xf490, PDF_CMAP_RANGE, 17240 },
	{ 0xf491, 0xf491, PDF_CMAP_SINGLE, 17855 },
	{ 0xf492, 0xf492, PDF_CMAP_SINGLE, 17245 },
	{ 0xf493, 0xf493, PDF_CMAP_SINGLE, 17856 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 17247 },
	{ 0xf496, 0xf4a2, PDF_CMAP_RANGE, 17249 },
	{ 0xf4a4, 0xf4a6, PDF_CMAP_RANGE, 17263 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 17857 },
	{ 0xf4a9, 0xf4b0, PDF_CMAP_RANGE, 17268 },
	{ 0xf4b1, 0xf4b2, PDF_CMAP_RANGE, 17858 },
	{ 0xf4b3, 0xf4b7, PDF_CMAP_RANGE, 17278 },
	{ 0xf4ba, 0xf4c0, PDF_CMAP_RANGE, 17285 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 17860 },
	{ 0xf4c2, 0xf4c2, PDF_CMAP_SINGLE, 17371 },
	{ 0xf4c3, 0xf4cf, PDF_CMAP_RANGE, 17861 },
	{ 0xf4d1, 0xf4da, PDF_CMAP_RANGE, 17874 },
	{ 0xf4db, 0xf4dc, PDF_CMAP_RANGE, 17292 },
	{ 0xf4dd, 0xf4de, PDF_CMAP_RANGE, 17884 },
	{ 0xf4df, 0xf4df, PDF_CMAP_SINGLE, 17294 },
	{ 0xf4e0, 0xf4e0, PDF_CMAP_SINGLE, 17886 },
	{ 0xf4e1, 0xf4e1, PDF_CMAP_SINGLE, 17295 },
	{ 0xf4e2, 0xf4e2, PDF_CMAP_SINGLE, 17887 },
	{ 0xf4e3, 0xf4e3, PDF_CMAP_SINGLE, 17296 },
	{ 0xf4e4, 0xf4e4, PDF_CMAP_SINGLE, 17888 },
	{ 0xf4e5, 0xf4e5, PDF_CMAP_SINGLE, 17297 },
	{ 0xf4e6, 0xf4e6, PDF_CMAP_SINGLE, 17889 },
	{ 0xf4e7, 0xf4ea, PDF_CMAP_RANGE, 17298 },
	{ 0xf4eb, 0xf4ed, PDF_CMAP_RANGE, 17890 },
	{ 0xf4ef, 0xf4f1, PDF_CMAP_RANGE, 17893 },
	{ 0xf4f2, 0xf4f2, PDF_CMAP_SINGLE, 17303 },
	{ 0xf4f3, 0xf4f3, PDF_CMAP_SINGLE, 17896 },
	{ 0xf4f4, 0xf4f4, PDF_CMAP_SINGLE, 17304 },
	{ 0xf4f5, 0xf4f5, PDF_CMAP_SINGLE, 17348 },
	{ 0xf4f6, 0xf4fa, PDF_CMAP_RANGE, 17897 },
	{ 0xf4fb, 0xf4fb, PDF_CMAP_SINGLE, 17319 },
	{ 0xf4fc, 0xf4ff, PDF_CMAP_RANGE, 17902 },
	{ 0xf500, 0xf501, PDF_CMAP_RANGE, 17906 },
	{ 0xf502, 0xf502, PDF_CMAP_SINGLE, 17324 },
	{ 0xf503, 0xf537, PDF_CMAP_RANGE, 17908 },
	{ 0xf538, 0xf554, PDF_CMAP_RANGE, 17631 },
	{ 0xf556, 0xf575, PDF_CMAP_RANGE, 17660 },
	{ 0xf579, 0xf57b, PDF_CMAP_RANGE, 18851 },
	{ 0xf57d, 0xf57d, PDF_CMAP_SINGLE, 18855 },
	{ 0xf580, 0xf583, PDF_CMAP_RANGE, 18858 },
	{ 0xf585, 0xf585, PDF_CMAP_SINGLE, 18863 },
	{ 0xf58b, 0xf58f, PDF_CMAP_RANGE, 18869 },
	{ 0xf593, 0xf594, PDF_CMAP_RANGE, 18877 },
	{ 0xf596, 0xf597, PDF_CMAP_RANGE, 18880 },
	{ 0xf59b, 0xf59c, PDF_CMAP_RANGE, 18884 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 18889 },
	{ 0xf5a3, 0xf5a4, PDF_CMAP_RANGE, 18892 },
	{ 0xf5ab, 0xf5ab, PDF_CMAP_SINGLE, 18900 },
	{ 0xf5b0, 0xf5b0, PDF_CMAP_SINGLE, 18905 },
	{ 0xf5b4, 0xf5b4, PDF_CMAP_SINGLE, 18909 },
	{ 0xf5b6, 0xf5b6, PDF_CMAP_SINGLE, 18911 },
	{ 0xf5b9, 0xf5b9, PDF_CMAP_SINGLE, 18914 },
	{ 0xf5bd, 0xf5bd, PDF_CMAP_SINGLE, 18917 },
	{ 0xf5bf, 0xf5c0, PDF_CMAP_RANGE, 18919 },
	{ 0xf5c4, 0xf5c5, PDF_CMAP_RANGE, 18924 },
	{ 0xf5cc, 0xf5cc, PDF_CMAP_SINGLE, 18932 },
	{ 0xf5ce, 0xf5ce, PDF_CMAP_SINGLE, 18934 },
	{ 0xf5d0, 0xf5d0, PDF_CMAP_SINGLE, 18936 },
	{ 0xf5d5, 0xf5d9, PDF_CMAP_RANGE, 18941 },
	{ 0xf5e0, 0xf5e0, PDF_CMAP_SINGLE, 18949 },
	{ 0xf5e4, 0xf5e4, PDF_CMAP_SINGLE, 18952 },
	{ 0xf5e6, 0xf5e8, PDF_CMAP_RANGE, 18954 },
	{ 0xf5ea, 0xf5ed, PDF_CMAP_RANGE, 18958 },
	{ 0xf5ef, 0xf5ef, PDF_CMAP_SINGLE, 18963 },
	{ 0xf634, 0xf635, PDF_CMAP_RANGE, 17961 },
	{ 0xf636, 0xf636, PDF_CMAP_SINGLE, 17338 },
	{ 0xf637, 0xf63b, PDF_CMAP_RANGE, 17963 },
	{ 0xf63c, 0xf63c, PDF_CMAP_SINGLE, 17339 },
	{ 0xf63d, 0xf63d, PDF_CMAP_SINGLE, 17312 },
	{ 0xf63e, 0xf63e, PDF_CMAP_SINGLE, 17341 },
	{ 0xf63f, 0xf641, PDF_CMAP_RANGE, 17968 },
	{ 0xf642, 0xf642, PDF_CMAP_SINGLE, 17342 },
	{ 0xf643, 0xf649, PDF_CMAP_RANGE, 17971 },
	{ 0xf64a, 0xf64a, PDF_CMAP_SINGLE, 17343 },
	{ 0xf64b, 0xf64d, PDF_CMAP_RANGE, 17978 },
	{ 0xf64e, 0xf64e, PDF_CMAP_SINGLE, 17344 },
	{ 0xf64f, 0xf64f, PDF_CMAP_SINGLE, 17981 },
	{ 0xf650, 0xf650, PDF_CMAP_SINGLE, 17345 },
	{ 0xf651, 0xf656, PDF_CMAP_RANGE, 17982 },
	{ 0xf657, 0xf657, PDF_CMAP_SINGLE, 17346 },
	{ 0xf658, 0xf659, PDF_CMAP_RANGE, 17988 },
	{ 0xf65a, 0xf65a, PDF_CMAP_SINGLE, 17337 },
	{ 0xf65b, 0xf65b, PDF_CMAP_SINGLE, 17325 },
	{ 0xf65c, 0xf667, PDF_CMAP_RANGE, 17990 },
	{ 0xf668, 0xf668, PDF_CMAP_SINGLE, 17351 },
	{ 0xf669, 0xf674, PDF_CMAP_RANGE, 18002 },
	{ 0xf675, 0xf675, PDF_CMAP_SINGLE, 17352 },
	{ 0xf676, 0xf6ab, PDF_CMAP_RANGE, 18014 },
	{ 0xf6ac, 0xf6ac, PDF_CMAP_SINGLE, 17401 },
	{ 0xf6ad, 0xf6b0, PDF_CMAP_RANGE, 18068 },
	{ 0xf6b1, 0xf6ce, PDF_CMAP_RANGE, 506 },
	{ 0xf6cf, 0xf6de, PDF_CMAP_RANGE, 537 },
	{ 0xf6e0, 0xf6e2, PDF_CMAP_RANGE, 554 },
	{ 0xf6e4, 0xf6e4, PDF_CMAP_SINGLE, 558 },
	{ 0xf6e6, 0xf6e6, PDF_CMAP_SINGLE, 560 },
	{ 0xf6e8, 0xf6ed, PDF_CMAP_RANGE, 13747 },
	{ 0xf6f0, 0xf6ff, PDF_CMAP_RANGE, 13754 },
	{ 0xf700, 0xf7ee, PDF_CMAP_RANGE, 13770 },
	{ 0xf817, 0xf81d, PDF_CMAP_RANGE, 14049 },
	{ 0xf81e, 0xf820, PDF_CMAP_RANGE, 17606 },
	{ 0xf821, 0xf829, PDF_CMAP_RANGE, 17692 },
	{ 0xf82a, 0xf82a, PDF_CMAP_SINGLE, 18846 },
	{ 0xf82b, 0xf832, PDF_CMAP_RANGE, 17701 },
	{ 0xf833, 0xf833, PDF_CMAP_SINGLE, 18847 },
	{ 0xf834, 0xf83a, PDF_CMAP_RANGE, 17709 },
	{ 0xf83b, 0xf83b, PDF_CMAP_SINGLE, 18848 },
	{ 0xf83f, 0xf848, PDF_CMAP_RANGE, 18834 },
	{ 0xf907, 0xf907, PDF_CMAP_SINGLE, 17686 },
	{ 0xfa0c, 0xfa0c, PDF_CMAP_SINGLE, 628 },
	{ 0xfa0d, 0xfa0d, PDF_CMAP_SINGLE, 9089 },
	{ 0xfa12, 0xfa12, PDF_CMAP_SINGLE, 16828 },
	{ 0xfa26, 0xfa26, PDF_CMAP_SINGLE, 14465 },
	{ 0xfa48, 0xfa48, PDF_CMAP_SINGLE, 16997 },
	{ 0xfa5b, 0xfa5b, PDF_CMAP_SINGLE, 14224 },
	{ 0xfe10, 0xfe12, PDF_CMAP_RANGE, 14099 },
	{ 0xfe13, 0xfe13, PDF_CMAP_SINGLE, 14104 },
	{ 0xfe14, 0xfe14, PDF_CMAP_SINGLE, 14103 },
	{ 0xfe15, 0xfe15, PDF_CMAP_SINGLE, 14106 },
	{ 0xfe16, 0xfe16, PDF_CMAP_SINGLE, 14105 },
	{ 0xfe19, 0xfe19, PDF_CMAP_SINGLE, 13646 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 109 },
	{ 0xfe31, 0xfe31, PDF_CMAP_SINGLE, 122 },
	{ 0xfe33, 0xfe33, PDF_CMAP_SINGLE, 13743 },
	{ 0xfe34, 0xfe34, PDF_CMAP_SINGLE, 13745 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 130 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 134 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 138 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 142 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 146 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 150 },
	{ 0xfe41, 0xfe42, PDF_CMAP_RANGE, 154 },
	{ 0xfe43, 0xfe44, PDF_CMAP_RANGE, 158 },
	{ 0xfe49, 0xfe4a, PDF_CMAP_RANGE, 199 },
	{ 0xfe4b, 0xfe4c, PDF_CMAP_RANGE, 203 },
	{ 0xfe4d, 0xfe4e, PDF_CMAP_RANGE, 201 },
	{ 0xfe4f, 0xfe4f, PDF_CMAP_SINGLE, 13746 },
	{ 0xfe50, 0xfe52, PDF_CMAP_RANGE, 112 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 116 },
	{ 0xfe59, 0xfe5e, PDF_CMAP_RANGE, 160 },
	{ 0xfe5f, 0xfe61, PDF_CMAP_RANGE, 205 },
	{ 0xfe62, 0xfe66, PDF_CMAP_RANGE, 223 },
	{ 0xfe68, 0xfe68, PDF_CMAP_SINGLE, 258 },
	{ 0xfe69, 0xfe6b, PDF_CMAP_RANGE, 268 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 108 },
	{ 0xff02, 0xff02, PDF_CMAP_SINGLE, 14052 },
	{ 0xff03, 0xff03, PDF_CMAP_SINGLE, 174 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 259 },
	{ 0xff05, 0xff05, PDF_CMAP_SINGLE, 264 },
	{ 0xff06, 0xff06, PDF_CMAP_SINGLE, 175 },
	{ 0xff07, 0xff07, PDF_CMAP_SINGLE, 14051 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 128 },
	{ 0xff0a, 0xff0a, PDF_CMAP_SINGLE, 176 },
	{ 0xff0b, 0xff0b, PDF_CMAP_SINGLE, 208 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 100 },
	{ 0xff0d, 0xff0d, PDF_CMAP_SINGLE, 209 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 103 },
	{ 0xff0f, 0xff0f, PDF_CMAP_SINGLE, 257 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 333 },
	{ 0xff1a, 0xff1a, PDF_CMAP_SINGLE, 106 },
	{ 0xff1b, 0xff1b, PDF_CMAP_SINGLE, 105 },
	{ 0xff1c, 0xff1c, PDF_CMAP_SINGLE, 214 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 216 },
	{ 0xff1e, 0xff1e, PDF_CMAP_SINGLE, 215 },
	{ 0xff1f, 0xff1f, PDF_CMAP_SINGLE, 107 },
	{ 0xff20, 0xff20, PDF_CMAP_SINGLE, 265 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 365 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 13758 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 258 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 13759 },
	{ 0xff3e, 0xff3e, PDF_CMAP_SINGLE, 13748 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 197 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 391 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 132 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 120 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 133 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 228 },
	{ 0xff64, 0xff64, PDF_CMAP_SINGLE, 113 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 262 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 14049 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 196 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 14050 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 260 },
	{ 0xffed, 0xffed, PDF_CMAP_SINGLE, 14096 },
};

static const unsigned short pdf_cmap_UniCNS_UTF16_H_table[15173] =
{
	178,13747,195,280,212,18788,18786,18792,
	18790,18801,18796,18794,18805,18803,18810,18808,
	18832,18814,18812,18818,18816,211,18840,18822,
	18820,18785,18802,18789,18807,18791,18809,18841,
	18793,18815,18787,18804,18813,18795,18817,18835,
	18806,13748,504,503,505,18798,18829,18800,
	18831,121,123,111,110,104,248,245,
	247,246,252,251,220,233,232,240,
	239,219,222,322,14090,14072,14081,14063,
	14074,14083,14065,14078,14087,14069,14080,14089,
	14071,323,14084,14066,325,14086,14068,14073,
	14082,14064,14079,14088,14070,324,14085,14067,
	321,320,304,303,302,301,300,299,
	298,14096,310,313,190,189,183,182,
	192,191,188,187,184,181,329,328,
	186,185,241,243,242,17700,18846,596,
	539,602,540,607,5998,608,610,544,
	611,5999,545,546,6000,547,614,633,
	6005,548,548,6006,646,550,725,556,
	862,866,6163,6168,560,1307,17635,1698,
	7731,2552,7732,9056,9746,236,235,17815,
	18083,16075,17273,15149,17840,18070,17959,16236,
	18438,14342,16403,15583,16589,16613,14597,17046,
	15674,18464,14904,15283,14716,15581,16918,16636,
	14713,14720,15500,15671,16646,14743,17976,15592,
	16784,17295,14942,16095,16796,17203,18006,18005,
	18007,14426,18976,15004,18022,16883,14958,19036,
	14195,15594,18057,18654,16740,15727,15091,14745,
	16989,18132,19006,17963,15380,17156,18157,16501,
	14420,18162,15533,17055,15618,18919,15528,18166,
	14698,19092,15069,18168,15093,15373,15063,19063,
	15081,16521,17879,17236,19060,17367,16988,19130,
	15141,15598,16870,18250,15219,15074,14317,14813,
	19085,15650,16847,15217,15602,16052,18079,18275,
	15076,16563,19076,18752,18286,18969,16100,19081,
	15264,15866,16060,18988,18321,18765,19128,19106,
	15382,18368,14658,15396,15395,14676,16115,18974,
	19080,18971,595,597,598,18686,6001,617,
	615,618,616,6002,660,6010,6008,659,
	658,754,753,752,755,751,18685,16215,
	619,17651,661,6011,18898,662,6030,1045,
	6388,537,17687,620,663,756,6031,17719,
	13999,5996,599,6003,623,16526,757,759,
	758,2097,596,14001,17637,5997,600,625,
	624,882,15845,2555,7733,17806,17340,3518,
	539,601,17807,666,17820,1323,602,6004,
	626,667,669,6012,670,668,14788,883,
	18645,1324,1325,1699,540,627,671,18895,
	884,886,885,9059,16700,13175,603,17638,
	14769,17808,673,672,6013,674,680,6015,
	6014,677,16226,761,768,6037,6033,6036,
	769,785,6032,6034,18635,17809,899,6074,
	896,900,6084,6072,897,6076,14765,901,
	17307,887,6075,902,6079,14792,6083,6080,
	6077,888,890,903,892,6069,895,6085,
	6082,6073,891,18899,6070,889,17724,17725,
	14782,6071,6191,1068,1055,14653,6192,1052,
	1070,17810,1060,16229,1059,6180,1063,1058,
	6185,6193,6189,14795,1061,15247,1064,1049,
	1073,6182,14772,6400,1047,1069,1048,6187,
	6183,1062,1054,6179,1050,6190,1071,1074,
	1053,1066,1072,1051,6188,1067,6194,6181,
	1065,6184,1341,6402,6414,1333,6415,1328,
	1338,15474,1331,6396,1339,6395,6399,6406,
	6404,1347,1342,6390,6392,1344,1332,6393,
	6405,6412,1337,6398,6391,1336,6394,1340,
	6397,14773,1335,1345,6407,1346,6411,6409,
	6413,1343,6401,6389,6403,1334,6408,1329,
	6410,1716,1704,15690,6749,6757,19050,1703,
	1711,1723,6763,6761,6756,1705,6762,6752,
	1719,1710,1718,6747,6759,18152,6750,1714,
	6751,14799,6746,1721,1708,1717,1707,6748,
	6753,6760,1715,1712,6755,1720,6758,6754,
	1709,1722,1713,1706,1702,15462,14800,2124,
	2102,18896,2118,16168,2123,7204,7203,2105,
	14259,2114,16166,2127,7197,2108,7198,16420,
	2129,7217,2572,2113,2112,7199,2116,7749,
	2107,7210,2122,2121,2111,7202,7211,7196,
	2110,7212,2119,7200,2101,2104,2103,7213,
	2117,2106,2126,2128,7207,2125,14796,7215,
	7201,7216,7205,14608,7214,18849,2109,7757,
	7745,7750,7756,2559,7743,2563,7747,7740,
	2561,7744,2566,2571,2567,16172,7760,14801,
	2562,7746,2558,7741,7737,7751,7738,7736,
	7748,2564,15817,8376,7752,7735,7753,14797,
	2574,7758,2573,7734,7742,7759,2569,2568,
	2565,2570,3052,14798,8379,8372,8380,17308,
	8370,8373,3048,8374,16876,3051,8378,8368,
	8367,3053,3054,3050,3055,8366,8377,8375,
	8369,3047,8371,3525,3519,9063,3528,9069,
	9072,9066,3520,9075,3526,9076,9071,3527,
	18897,9060,3524,9061,9070,9068,9064,3523,
	9062,3529,9067,9074,9065,9073,14789,3976,
	3973,14545,3974,9756,9748,3972,3969,9749,
	16334,3971,9752,9754,16433,15101,9761,16753,
	9760,9755,3968,9747,3966,9750,3977,9753,
	9757,3970,3967,9759,9758,3975,9751,15593,
	4357,10371,10368,4358,4356,14802,10375,10372,
	10365,4355,16434,4359,10366,4354,4362,14790,
	16436,4749,11012,4746,11009,4748,4750,11010,
	4747,5045,11582,5046,11580,11581,11579,11578,
	11583,5044,14766,10367,12046,5047,5493,12998,
	12997,5820,5819,13326,604,628,682,681,
	771,770,906,905,907,904,1076,1075,
	1077,1348,1350,17306,1724,281,7218,282,
	2575,284,283,8381,16442,285,3978,286,
	683,908,1351,16444,606,686,17727,1078,
	1353,1352,1354,15054,17311,6009,14006,1726,
	1725,7220,7219,2576,542,687,6016,16038,
	14810,4752,543,774,911,6086,16456,16465,
	15740,16402,16457,6196,6417,1355,6418,2136,
	16938,7224,16459,7221,2137,2135,2134,7225,
	16460,7762,16461,15789,16463,8382,9077,9762,
	17818,4363,4753,11013,16975,607,621,18758,
	2577,3057,3979,16760,5998,688,17819,1356,
	16138,14003,629,6039,778,16473,6038,916,
	912,6197,1675,6198,16224,18736,6199,1082,
	1081,16474,17314,1085,1361,6423,16123,6419,
	1363,1359,1358,1360,1357,16475,1729,6765,
	6422,1734,6766,1730,1733,1732,1731,1728,
	19120,2140,7231,2138,16477,14820,7227,2141,
	2139,2142,16169,7226,7232,17945,3061,2578,
	7763,7765,7764,7766,2579,3058,16195,3531,
	9079,17315,9078,3981,3980,10102,15807,17821,
	4754,4755,16478,12412,13000,12999,13327,16232,
	610,780,779,17730,917,14821,1086,1089,
	1366,6424,6767,1738,6768,1737,16097,7233,
	16649,16170,2580,7768,2583,7767,2767,2582,
	2581,3063,3062,3533,16483,14823,10380,4369,
	10379,4756,11584,5048,12413,5641,544,630,
	692,6017,918,6202,1739,7234,2585,2584,
	8384,8383,611,695,783,2586,5999,6040,
	784,6091,920,919,6090,1090,18888,16182,
	7769,3538,9767,3982,16336,16493,11014,12743,
	545,696,17732,6427,6769,2589,612,18759,
	631,363,699,698,697,787,786,6041,
	6092,17733,1370,1367,1369,1368,613,17665,
	700,17319,789,788,6203,18442,1371,16077,
	6018,16506,791,790,6204,1091,16507,1092,
	1741,6770,18689,16509,2144,6428,16514,6429,
	6772,6771,6774,6773,1742,7770,2146,7235,
	2145,16516,8385,3066,16699,8386,16702,17824,
	16326,16517,9768,3983,16518,16523,17318,2590,
	614,632,703,702,14518,704,14842,17734,
	15933,1375,1377,17735,16528,16185,1743,2147,
	16531,11015,5321,633,794,809,804,799,
	798,805,803,796,810,797,793,808,
	807,806,795,802,16534,14344,801,811,
	800,934,930,936,6094,933,923,926,
	929,925,932,935,924,927,931,937,
	16536,15479,6215,6212,15972,1105,6213,1093,
	1095,1119,1114,16899,17827,6220,6214,1097,
	1099,6219,1106,6209,1118,1120,1094,1111,
	16556,6207,1117,15989,1102,18901,6208,1110,
	1108,1109,1115,6216,1096,1116,6218,1104,
	1101,6211,1100,6221,1103,1107,16081,19129,
	16039,1098,6217,6446,6445,1395,6448,6450,
	6439,6442,6449,1396,16178,15747,6436,6440,
	16541,6444,6789,1391,6793,1378,6441,1379,
	1392,1385,1381,6437,1384,1389,1398,6438,
	6431,1383,6432,6434,1386,18137,1388,6430,
	6435,1397,1393,6443,1399,16545,1390,6433,
	1387,17831,16024,1382,1380,16010,6447,6788,
	6776,6791,17832,16546,6778,1750,1764,1746,
	1763,1755,1760,1744,6777,6783,1759,6795,
	1761,14148,1751,17833,6785,6782,1749,17834,
	6775,1762,6790,1754,6792,1765,1745,1756,
	1753,6780,1757,1752,1758,1748,14755,17890,
	14848,1747,6779,17835,7255,2160,7237,17217,
	7242,2153,2164,7240,2148,2158,2163,7247,
	2159,2162,17836,7250,2154,7241,2156,7252,
	2152,2167,2150,15564,7257,7245,7249,2155,
	2166,7246,2161,7251,7258,7785,14963,7256,
	2168,2149,7248,7239,16041,2157,7238,17837,
	17161,7236,2165,17839,7777,16538,2609,7791,
	2607,2604,7782,2600,7786,2611,7776,7781,
	7780,2151,2606,7789,7792,7773,3084,17321,
	7793,2612,2598,2595,7784,2592,16554,7790,
	16488,2599,7794,7774,7788,2602,7775,7778,
	7783,18762,2603,2601,2613,2608,16544,2596,
	2748,2597,7779,2610,2605,7787,2594,16031,
	2593,15150,15302,16011,16164,3067,3070,8402,
	3090,8405,3068,8398,3074,3080,3355,16558,
	3078,8395,3091,3071,3079,8403,9095,8408,
	8396,17166,8387,8400,8394,3077,8406,3073,
	3093,3086,3075,3072,8407,8393,8399,8401,
	8389,8404,3069,8388,16559,3076,3092,3088,
	8390,3082,14581,3089,3085,3081,14846,8397,
	17892,16009,3087,15811,9093,9089,15951,9104,
	9082,9091,3545,9100,3555,9087,9086,9084,
	9102,3543,9096,9085,3546,9094,3541,9090,
	9097,9099,17843,17323,9103,3550,9083,3544,
	9088,18661,3539,3551,9098,3554,3542,287,
	3540,9092,15996,3549,16008,16003,9101,18310,
	3999,3994,9783,9779,17958,9773,3988,3984,
	9782,3985,9776,9778,9781,16002,3990,3997,
	3991,9774,3998,16205,9772,9777,3989,9770,
	3995,3987,3986,9775,9780,15976,3996,16253,
	17325,15975,18183,4377,10397,10394,15971,4371,
	4374,10392,4376,10384,4382,16567,10396,4373,
	10398,4372,10393,10395,4375,10391,10385,10382,
	15998,8392,10388,10381,10387,10386,10383,16572,
	4379,16573,18383,17858,4378,17327,4380,10389,
	4757,10390,16121,11024,11017,14354,4769,11021,
	4761,4765,11020,4764,4760,4763,4758,4768,
	11022,11018,4767,11016,4766,11023,11019,4381,
	4770,11025,4762,4759,15749,16055,16109,11591,
	11593,5050,11594,11592,11590,11589,5052,11588,
	5053,17960,14665,15948,5049,5054,5051,14758,
	11585,12051,12049,12054,12052,12050,12053,14756,
	14664,15560,5494,12419,12415,12420,12416,5323,
	17911,17983,12744,12746,5644,12745,5643,5642,
	15965,5739,5738,5740,13001,13177,13176,5821,
	5823,5822,13178,5878,13441,13440,13513,18570,
	6005,813,812,940,939,6096,938,6095,
	17736,1123,6224,1121,1124,16580,6222,16579,
	1122,16575,6452,1400,19002,1766,16578,7796,
	2614,2616,7795,2615,8409,3094,9105,15926,
	4001,10399,11027,11026,14850,13573,634,6020,
	943,947,6098,945,944,6097,946,942,
	15097,941,6099,18950,1131,18517,1127,6225,
	16588,16606,6226,16584,1129,6228,1125,6229,
	6227,1128,1130,1133,1132,1126,6230,1405,
	6465,1406,16171,6466,1404,1403,6456,6453,
	6458,6457,6454,6461,6459,1402,18722,6462,
	1408,6467,18140,14855,6460,1767,1401,15483,
	1768,6800,14854,6806,16742,1774,6804,6810,
	6801,6805,6807,6809,6803,17928,6802,1769,
	18532,1771,1770,6799,6808,19127,18241,6796,
	7270,7269,7274,7265,15481,7268,7267,14124,
	7271,7275,2171,2174,15195,7266,7272,15627,
	2173,7261,1772,7810,7273,2172,7263,7801,
	14853,16129,2617,2621,7817,7799,7813,2622,
	7815,16591,7821,7816,7804,7812,18260,7826,
	7824,7814,7809,7802,8417,7800,2626,7807,
	2627,2623,7797,7819,7805,16527,7803,7822,
	2624,16593,7828,2618,2620,7811,7806,2175,
	2619,7808,7823,7827,7818,19132,8412,8421,
	8420,3102,8413,16598,3103,3101,15400,8415,
	3098,8419,16596,8414,8416,8410,3096,8429,
	8428,8425,3095,7262,8422,3097,2625,8424,
	8411,8427,8426,15730,8430,8423,15746,15042,
	8418,9111,3567,3570,3565,9110,9114,9109,
	3559,3569,9106,3563,9113,3561,3560,9116,
	3562,9118,18761,9115,3558,17375,3568,9120,
	9108,9117,9107,15717,3564,14746,3566,9121,
	15398,17237,9791,4002,9797,4007,9793,9800,
	9785,4009,4003,9790,4385,9789,9798,4004,
	4008,9788,9794,9799,9784,4006,9792,9796,
	9802,9786,9795,18903,4005,9801,9787,17944,
	18691,4389,10401,4387,4386,10403,10408,10407,
	4392,16107,4744,4391,16599,10400,10406,4390,
	10405,14244,10402,18496,15869,11032,14059,11034,
	11029,4772,11031,4771,11033,4774,11035,4773,
	11028,11030,5058,11597,18904,5057,11598,5056,
	11596,5055,11595,5325,5324,12421,12423,16602,
	12422,5498,12747,5646,13329,13328,5920,635,
	705,1135,17737,15720,15482,6811,16604,9122,
	4010,9803,10409,548,6021,17738,17817,6231,
	17739,6468,6812,7276,2176,17012,12424,5741,
	636,814,6044,1409,19004,16611,2628,4013,
	4011,706,708,707,6022,709,815,6045,
	816,17740,1413,6469,1410,7277,1778,1777,
	1779,1776,6813,1414,1775,3106,8431,2629,
	4015,4014,9804,4393,4775,12055,13179,13330,
	638,817,15978,818,6106,953,956,6104,
	6102,955,6105,6107,6101,958,957,954,
	952,6103,15664,1147,1144,6237,6240,6239,
	1138,1146,15513,1143,6236,6234,1142,15280,
	1137,1140,15791,6235,6242,6238,1141,1145,
	1148,6233,6241,1139,1419,1427,15576,6481,
	6474,1428,6485,6470,6477,16173,14864,1418,
	6471,1416,6478,6482,1415,19052,6483,6476,
	6479,15620,1430,1421,6486,6484,16620,1426,
	1424,6475,1423,6473,6472,1422,1420,1429,
	1425,1417,1781,16625,1788,6827,1780,6819,
	6815,6831,6814,1783,6824,1786,1789,1784,
	6828,1787,15705,2186,6834,6816,15672,15282,
	6818,6825,6829,6833,6830,6823,6826,15946,
	16632,6820,1791,6822,6821,6832,1782,6817,
	1790,15245,1785,2192,7285,15497,2191,15160,
	2180,14867,2185,7282,7279,7284,2181,7278,
	18946,2184,2182,7286,2183,2187,14875,2188,
	16616,2190,2189,15163,15680,14863,7836,2630,
	8445,7835,7854,2636,15040,7852,2635,2631,
	7861,7845,7848,14615,2639,7858,7850,2632,
	2640,7855,7853,7838,7859,7847,7842,17923,
	7832,7860,7844,7831,15156,2638,7849,7862,
	7846,7834,7839,7830,15667,2637,16635,7843,
	7840,2633,7833,14775,7857,2634,14914,7841,
	7837,15679,7856,15668,3107,8438,15585,8435,
	8452,8440,8453,14865,3109,14877,15665,8449,
	8446,8447,8450,8454,7851,8456,9142,16949,
	3110,8457,8433,8443,16640,8448,14858,3108,
	3111,8455,8458,8437,8434,16754,15580,8436,
	19011,8441,8439,3112,16634,8442,8444,8432,
	9130,9128,3580,3578,9138,9129,9139,9135,
	9127,9141,9126,9133,3577,3576,3575,9131,
	9136,3572,3579,14486,9124,9134,9123,9132,
	3573,9137,9125,3574,9140,15496,14866,9808,
	4020,4019,4021,9820,9810,9816,9805,9819,
	9818,9822,9815,4016,9814,4022,15238,9807,
	4017,9821,4018,9809,9812,9817,9811,9806,
	14325,9813,10411,4397,10414,10413,10416,10418,
	14062,4395,10412,15389,10410,10419,10417,10415,
	10420,4399,4394,4396,4398,18687,10422,11042,
	15350,11041,11040,18921,11043,11036,11037,11045,
	11038,4776,11047,18753,11046,11039,11602,5061,
	11600,11605,11604,11044,5060,15236,11603,11599,
	11607,11606,5059,5647,14536,5648,14916,12748,
	17074,13005,13004,13331,13442,14897,641,710,
	819,6108,1150,1149,6487,1433,1432,6488,
	17742,17949,1792,2193,7288,15967,7864,2641,
	3114,7863,3113,14130,4023,16650,9823,4778,
	5062,11608,16652,5649,5824,549,6046,16452,
	820,6047,963,1154,1153,6243,1155,18665,
	6244,6489,6490,18962,1434,1436,1438,1435,
	1439,1437,16662,1796,1793,1795,1797,1794,
	7289,6835,7291,7290,2199,2196,2198,2200,
	2197,19015,2202,2201,2646,7866,7865,2645,
	16165,2644,2643,2647,2642,8464,8461,3116,
	8460,8465,3118,8463,3115,3117,8462,16657,
	9143,17083,4024,4032,9824,4026,4030,9825,
	4031,4025,4029,4402,8459,4403,4401,4400,
	11048,4779,5499,5650,7292,2203,16664,2650,
	2649,2648,8466,4033,4780,643,6023,711,
	6048,16666,16668,6109,966,14889,9146,15753,
	644,17640,712,6110,8467,3121,5063,645,
	665,713,6049,821,1160,1159,6492,1441,
	16671,16220,1802,6836,1801,2206,2204,16130,
	7294,2205,7293,2194,7867,4034,9826,10423,
	5064,5742,13443,6006,714,6051,6050,19035,
	967,16673,6254,6249,6251,6253,6250,1164,
	6247,6252,6255,1163,6256,18060,17745,18778,
	6506,14890,6500,6495,1445,6503,6501,6494,
	6507,6510,6505,6498,1447,6504,1448,6499,
	6502,6493,6497,6509,1449,6496,6508,1444,
	1446,17336,15692,6853,6846,6852,6845,6847,
	6841,6839,6854,6851,6837,16147,1804,19032,
	6848,6840,6838,1803,6844,6842,6843,6855,
	2211,16674,2210,7295,2207,7297,16675,2212,
	7298,2215,18857,2213,7299,6856,7301,2209,
	19016,2208,7296,7300,2214,2655,2654,7869,
	7873,7875,2656,7878,16679,2660,7880,2662,
	17337,2658,2666,16680,2663,7871,2657,7870,
	7868,7882,7872,2659,7881,2664,7877,7876,
	2665,7874,2661,7883,18131,7879,8482,8489,
	8478,3124,8474,8490,8468,8487,8485,8479,
	8488,8481,17961,8473,8491,8471,8484,8469,
	8492,16681,3125,17962,8486,9150,8472,3122,
	3123,8475,8480,8477,17338,16330,9157,9149,
	9156,9155,3581,8470,9153,3582,18148,9148,
	9152,9833,19038,9840,9831,18957,9838,9829,
	9837,9841,9828,9832,9827,4035,15924,9830,
	4036,9836,9835,9834,16676,9839,16140,10427,
	10429,4407,10430,10424,10434,10432,4406,10435,
	10426,10431,10433,10428,11052,11057,11051,11056,
	15929,11049,11059,11055,11060,11054,11050,11058,
	11061,11053,16728,11611,5068,17966,5066,5065,
	5067,12059,15359,12323,12428,13183,5826,5825,
	13182,5879,16684,13332,647,968,6115,6257,
	1306,2667,648,824,823,822,715,16225,
	3126,652,6024,16704,6258,15754,1166,1456,
	16174,6512,1454,1453,6511,1451,6513,1452,
	1455,1808,6861,6862,1807,7303,7302,2217,
	16693,2670,7885,2669,2671,2668,3128,7884,
	3129,8494,17312,3130,8493,3127,9161,9163,
	3583,9162,9160,9159,9844,4041,4039,4037,
	10439,10437,14891,4409,10438,4410,4408,4038,
	10436,11613,5069,11612,11065,12429,653,827,
	971,6116,970,1457,3584,551,716,828,
	1809,3131,552,6053,6054,16648,6117,17746,
	1168,6263,6261,1169,6260,6262,6264,18907,
	1458,6866,1460,6865,6867,6864,6868,1811,
	2221,7304,7306,2219,7307,2220,7305,6863,
	7886,7887,2675,2674,17341,2676,3135,9164,
	9166,9168,3586,3585,3132,9167,9165,14896,
	9848,16014,9846,9852,4042,9853,9850,4043,
	9847,9845,9851,4412,10441,9849,4414,10442,
	4413,4416,10443,4411,4415,11070,11068,11067,
	11069,11066,13008,13184,5954,553,1463,1170,
	14447,16707,1812,16708,16710,654,717,829,
	6118,1171,6265,6869,1813,4044,655,14910,
	16713,14911,972,3587,656,831,830,6119,
	973,6514,1466,1814,6870,7309,2222,2677,
	7891,2678,7892,16718,19056,3136,9855,9854,
	4417,10444,4781,11071,5070,15269,5827,13334,
	554,17641,6871,2679,8495,3588,15706,5327,
	555,1174,16722,1173,1815,2681,2953,2682,
	2680,3137,9856,4046,4418,16724,6007,6120,
	6268,1175,6267,1176,16472,1470,6521,6520,
	1469,6519,17969,1817,6872,1820,1816,1822,
	2225,2224,2223,7894,2683,2686,2684,7893,
	2689,2685,7895,2688,2687,16428,8497,8496,
	16727,3140,8498,3589,9171,3590,9170,15768,
	10445,4420,9857,4419,4047,16287,11073,11072,
	5071,11614,12430,18908,720,17642,832,1178,
	1180,6123,6271,6270,6269,6122,6121,975,
	1179,1177,974,1471,6522,16815,1472,6276,
	6278,6277,6523,1184,1182,6272,6274,6280,
	6275,6284,1474,6281,1183,6279,6282,1475,
	6283,6543,6530,6528,6542,6537,1827,1476,
	6538,6541,1823,6540,1477,1482,1480,6531,
	6526,6533,1487,6544,1824,6534,1825,1483,
	6536,16773,6887,1826,6525,1484,1828,1485,
	1481,1486,6535,6524,6539,1478,16737,6527,
	6532,6529,1479,6884,7314,6885,1834,6888,
	6878,1833,6880,6881,16500,6883,1829,14060,
	6879,6875,2230,6889,2226,7312,6882,7311,
	6877,6886,1832,2227,1838,2228,6890,7313,
	1831,2232,1837,1836,1835,2231,6891,2233,
	1830,14919,2690,7317,7319,7321,2234,2240,
	7898,7325,7316,2692,7896,2239,2237,7327,
	2238,7322,2241,7324,2236,7323,7326,7320,
	16746,2235,2693,7315,2691,16385,15068,2694,
	7900,7909,3143,18768,2696,2701,3144,7911,
	2708,8500,7901,2700,2703,2698,7899,7913,
	7916,8515,7914,8504,2699,2706,2710,7908,
	8499,7912,2695,8501,7915,8503,7904,17973,
	2704,7906,2709,7910,2702,7907,16890,2707,
	3145,3142,8502,15733,7905,2697,15748,15739,
	16393,16748,3150,3154,8506,3595,3152,8510,
	3156,9172,8512,3597,3148,3151,8513,16296,
	8514,8520,3158,3598,8516,8521,8509,3608,
	15413,3599,3157,8507,8522,15495,3607,3155,
	3592,8519,3159,8511,8505,3149,8508,8517,
	16118,3591,3596,3146,8518,18862,3147,3606,
	9859,9183,9175,9858,9179,9181,16183,9177,
	3609,17974,3604,4049,9184,9860,16756,3602,
	9176,9180,4048,3593,4050,3601,3603,3600,
	9182,15410,9870,4425,9877,4056,16314,4055,
	9872,16774,4424,9861,4054,18764,9876,16755,
	9873,10462,4422,3153,9875,4429,9867,4423,
	16320,4428,9862,9866,9863,9869,4057,4421,
	4051,10448,9871,4427,16317,4430,9868,16759,
	4426,10447,10452,4785,10446,11085,10461,4434,
	4432,4783,10458,10454,4438,15840,11076,14933,
	14932,4436,10453,16386,11074,10460,10451,4431,
	11075,4784,10459,4433,4435,10456,4439,10455,
	10450,10449,4782,10463,11078,11616,4787,17977,
	11084,17343,11082,4788,11083,14928,11080,5073,
	11615,11077,11079,5074,4790,5072,4789,5076,
	11081,4786,17978,16769,11623,12063,11619,16325,
	5328,11621,11620,5075,11618,11622,12071,12067,
	12069,12068,12066,5502,5505,5504,5503,5652,
	12755,5653,12431,5744,13009,5745,5828,5880,
	13335,13619,721,6056,833,979,1186,1185,
	6545,1489,1488,3160,9185,9187,9189,9188,
	9878,4058,9880,17980,10464,4440,5077,5329,
	5078,722,17690,16322,6285,6546,1491,1490,
	1492,1839,7329,2242,2713,3161,8523,723,
	17644,657,724,6057,837,836,982,6130,
	6132,981,6124,6131,6127,6129,980,6126,
	6133,6128,1193,1202,1200,6295,6292,6300,
	1198,6301,1191,6298,1197,6294,6293,1195,
	6299,1196,1493,1190,6296,16262,6286,1187,
	16208,1206,6291,6297,1192,1194,17344,1205,
	1199,1204,6290,1203,1189,1188,1201,1507,
	6558,6551,1502,1521,6547,6554,6556,6559,
	1516,6555,6548,1514,6552,6560,1499,6557,
	1509,1508,6550,1497,19153,1498,15759,1496,
	16778,1520,1512,1506,1494,6553,1505,1495,
	1513,1522,6894,1510,6549,1500,1517,1511,
	1515,1501,1840,6900,1852,1844,1846,1851,
	6912,1849,7330,2243,1854,6897,6905,1850,
	6904,6901,6909,6911,1843,1847,1853,2245,
	6906,1845,1856,6899,1848,2244,1842,6896,
	6903,6895,6898,6902,7331,1855,6910,1841,
	2259,7343,7333,7349,16093,2248,14949,7917,
	7351,7345,16150,7336,14947,7354,7339,2254,
	7940,7342,2256,2247,7356,7344,2250,7337,
	2251,7350,7357,2253,7341,7340,2261,2260,
	2246,2252,2255,7353,7347,2249,7332,7355,
	7346,7348,7352,16327,7918,2721,2741,2740,
	2729,16782,7936,7932,7929,7947,2724,2716,
	7934,3177,2720,7942,17255,2742,2739,7938,
	7921,16203,2738,7944,7920,2727,2731,7943,
	7930,2732,2726,7919,16776,3163,7946,7928,
	2737,7931,7945,2736,2717,7926,2722,2733,
	2728,7935,7925,7923,7941,2714,2734,2718,
	3162,7939,2719,2715,2730,2725,2723,7927,
	2735,7924,7937,7933,8526,8525,7922,8542,
	3165,18088,8548,8531,8545,9190,3168,8549,
	8552,3167,8534,8551,8550,3169,8527,3164,
	3172,14950,3170,8547,8537,3174,8554,8546,
	8555,3182,3180,8544,8543,15968,8541,8535,
	3173,17985,3171,7338,8528,18591,9220,8529,
	3166,3179,9192,8530,8553,9191,8538,8533,
	3178,8539,8536,16786,3183,17986,16028,3626,
	17987,9195,9214,9219,9207,3622,9224,3619,
	9193,9194,3612,3621,9201,9215,9221,3620,
	3614,9200,9196,9223,9197,9217,9208,9218,
	9210,3615,9881,3618,3616,9222,9213,9209,
	17881,9198,4068,9212,3623,9204,16785,9203,
	16797,14955,3617,3613,9902,9216,9211,9882,
	9893,9900,4066,3181,9898,4061,4060,9905,
	17346,9883,9896,9884,9895,4064,9899,8540,
	9906,9891,4067,10478,4441,9904,9903,4069,
	10465,4442,10466,14939,9885,9890,9886,9907,
	4063,4443,4065,4070,17260,9889,4059,4446,
	11092,10474,10473,10475,17187,10472,4447,17347,
	4453,4450,4451,10467,10470,10480,4459,4457,
	10471,8532,4444,10477,10468,4460,10476,4062,
	4449,9892,4456,4458,4455,4454,17121,4448,
	10479,4445,4461,17216,11091,4805,4802,17989,
	4793,4800,11093,4798,4792,11090,5080,4794,
	4801,5079,11089,11088,4803,16131,4804,11087,
	11086,5081,11096,4797,11094,5082,16793,5087,
	11626,11628,16794,5084,14248,11629,11625,17226,
	11627,5085,5088,11624,5083,5086,5331,11095,
	5330,5336,12075,12078,12074,5332,12072,5506,
	12076,12077,12073,5333,12432,12435,17718,12436,
	12434,12437,5507,12433,15017,12759,5655,12758,
	12757,12756,5654,5656,13010,5747,5746,16775,
	16064,13186,13185,13188,5881,5829,13339,13187,
	13338,5883,5882,5921,13444,13514,725,15970,
	8556,11097,556,17645,983,6134,1209,1207,
	6561,1523,1857,1858,7359,7948,2752,2751,
	2744,2747,2750,2746,8560,2743,3184,16805,
	8559,8562,3185,8557,8561,8558,14956,3627,
	16804,4071,9908,4806,4462,10481,10483,10482,
	10484,11099,11098,11631,11630,12438,16803,17992,
	726,16651,17748,18910,8563,3189,3188,9226,
	12079,5748,13340,727,2754,2753,3628,9909,
	4072,11100,728,838,1524,6562,6915,1859,
	2755,8566,3190,3629,16807,11632,5337,13515,
	729,6563,1525,1860,15931,6916,2265,7363,
	2266,7360,7949,2759,2756,8567,16810,8568,
	9227,15272,4074,4073,11633,12080,12440,12439,
	12760,557,6025,1861,16814,730,839,985,
	984,6136,6135,6302,1210,6305,6304,6303,
	6306,1526,6569,6565,6574,17994,1533,1531,
	6570,6566,6573,1530,1537,6568,6564,1536,
	6571,1529,6572,1532,1534,6576,6575,6567,
	1528,1527,1535,6921,6927,14995,1867,1864,
	6918,6923,17045,1870,1862,6922,1865,1868,
	1863,6930,1866,14989,1869,6919,6924,6928,
	6920,6917,6929,6926,2274,2267,2270,16820,
	2273,7366,15769,2268,7364,16823,2272,19001,
	7367,2271,17995,7953,2761,7952,7954,2760,
	2765,7365,19113,7951,7955,14992,2762,7950,
	2764,16825,2763,8575,14987,8570,3191,3195,
	3192,8573,8576,16827,3193,3194,3199,9236,
	8574,3197,8571,8569,18075,3198,3635,16738,
	9228,3631,9233,9232,9229,3637,14985,9231,
	3196,17997,9235,9230,3634,3630,3636,9234,
	15677,9912,9911,9910,4075,4076,10489,10492,
	4466,18760,4465,10493,10488,4468,10490,16565,
	4467,10487,15891,10491,4812,4809,11109,11108,
	11107,11102,15770,11103,4807,4811,11101,4808,
	11110,16568,14967,11106,12083,5091,5338,5509,
	12441,5657,19023,12761,5749,13341,5884,731,
	16004,6577,1871,2275,2766,3056,8577,16836,
	3638,9914,9913,732,990,6578,1539,16354,
	1538,15777,17999,2278,7956,18867,3203,15755,
	3202,9237,9506,9915,11111,14239,5339,5658,
	733,841,844,840,6144,992,994,15937,
	6139,6138,6141,6140,6143,991,6137,6142,
	16411,6307,1221,6308,6312,1220,6316,6311,
	6314,6310,1218,1224,6309,6315,1217,6313,
	1219,1212,1222,15718,16843,1560,18948,6580,
	1540,19017,1543,1561,1545,6588,1556,6583,
	1546,6591,6584,1559,6596,1554,1552,6597,
	19118,6595,6587,6579,6586,1547,1541,6590,
	6589,6581,18113,1555,6593,6582,1558,6600,
	1542,1549,6585,6594,1557,1544,1548,6592,
	15738,6964,1879,1892,16884,6959,6949,1885,
	6947,1878,6942,1888,6955,6936,6938,14330,
	6941,6933,6954,6970,1883,6943,6961,6972,
	6963,6948,6956,1889,1876,1884,1896,1873,
	1875,15192,16846,6940,1893,1886,6966,6937,
	1895,1890,6946,1894,6953,6945,1887,6965,
	6957,1881,6971,6944,1877,6962,6952,1882,
	6958,1874,6934,1891,2293,1880,6951,6950,
	6939,14742,6967,17749,1872,17351,16851,16328,
	15722,17750,7389,2300,7390,7378,2289,2301,
	7371,16498,7380,7388,18114,7370,7397,2279,
	16848,7396,7391,7392,2287,2298,7382,7383,
	7393,7384,7387,7381,15918,16925,2280,2284,
	7395,7375,2296,2292,16711,2295,2302,2285,
	2297,7386,2299,2282,15339,2281,7372,18003,
	7376,2290,7394,7385,7377,2294,2291,2283,
	2286,15760,14356,7981,7978,7971,7970,2775,
	7974,7989,7957,7990,7976,7994,7982,2774,
	7980,2769,2790,2780,18004,2784,2783,7958,
	18103,7987,7993,7985,7992,7973,7959,7991,
	7975,2772,2785,7984,2778,15018,7983,7960,
	2786,2788,7986,2789,2771,7967,7988,16612,
	2777,2787,7969,7979,7964,7968,2782,7963,
	2770,2779,2776,7972,2288,8600,2773,15006,
	16860,2781,15406,8603,8625,8614,3223,17131,
	3218,8597,3219,8613,8606,8619,3215,8579,
	8621,3205,8587,3208,3207,3210,3206,8595,
	3217,8590,8617,3213,8609,8622,8584,8589,
	8583,3212,8602,9259,8585,3216,8594,8618,
	3211,8591,8588,8605,3214,3204,3226,8608,
	8607,16863,16862,17147,17052,8580,3209,8620,
	8596,8598,18008,8611,8610,8578,3220,3222,
	8586,8593,8601,3221,8592,8604,8623,8612,
	15778,18912,15744,14269,9279,3646,9277,9256,
	9261,9254,16871,9273,9240,9247,9271,9281,
	9257,16470,9244,9263,9268,9248,18009,9265,
	9245,17775,3648,9272,9270,9241,9275,9276,
	9278,3652,3644,15750,7965,9249,9267,9250,
	3641,3657,9274,3655,3651,9239,3643,16868,
	9242,3656,18011,9258,9238,15637,3649,9262,
	9246,3650,9255,3640,3225,9264,8599,9243,
	14957,16859,9269,3645,9266,3642,9260,3653,
	9251,9280,3647,9391,18013,3654,9929,9927,
	9949,9925,9943,3639,4080,9921,9948,16872,
	9926,9940,4085,4078,9946,18012,9919,9945,
	14128,4096,9936,9952,4094,9928,4079,9930,
	9944,4088,9923,4092,4082,9932,9922,9916,
	16374,9942,4089,9951,9917,4086,4087,9924,
	9935,9931,9933,16221,4081,9950,4095,9934,
	9918,9939,4084,4093,4091,9920,9941,4090,
	14831,4083,9937,15932,18014,16856,10505,10501,
	10497,10517,10514,4471,18463,10521,10504,4482,
	10511,10500,10510,4479,10530,10527,10513,10498,
	16877,16839,10518,10520,4475,10508,10503,10494,
	4472,4480,15913,10523,4481,10495,10529,10516,
	10522,4478,16857,15696,10519,4477,10512,10499,
	10531,10496,10524,4474,15646,10506,15208,10507,
	4473,4470,10502,4476,10525,10528,10509,11124,
	19114,15682,17067,15135,11116,15012,11112,4825,
	4814,4819,4815,11126,11137,4813,11118,11127,
	10515,11128,18913,4820,16880,11143,4824,4827,
	11114,4823,15019,11132,11131,11123,15013,14252,
	11133,11140,11139,14298,4818,4816,11125,11122,
	11119,11138,4826,11136,4822,4821,18879,11130,
	18016,11113,11115,11117,11135,11129,4817,5100,
	11652,5093,11637,18019,11648,5095,11655,11639,
	16694,11656,11635,11646,15387,5102,11651,11657,
	11645,5094,11647,11636,5101,18021,11654,11641,
	5097,16882,11643,11640,5103,11642,5096,5099,
	11650,11638,11653,14307,11649,18023,5341,12094,
	15955,15179,5340,12093,18024,5344,12085,12089,
	5343,12086,15009,5345,5342,12084,12088,12092,
	16885,15014,16204,12447,12444,12453,12452,12450,
	12445,5513,18025,15530,12446,5512,5098,12449,
	5511,12454,12448,12443,12087,12442,12767,12766,
	12451,5659,12765,12769,13014,12768,5752,5750,
	13012,13011,13013,5751,15015,5830,13189,13344,
	5885,5955,13518,16888,13620,734,995,7398,
	7400,7401,2791,7996,7402,16891,7995,8626,
	3228,8627,8629,3229,3227,8628,9287,9285,
	9284,9283,9282,3658,9286,4097,9954,9956,
	4098,9955,4485,4484,10532,16238,11145,11144,
	11146,15022,11658,5104,12095,5348,12455,5831,
	735,845,996,1225,10658,4829,5349,736,
	17646,997,6603,1565,6602,18027,6974,1898,
	6975,7405,2304,2303,8630,8633,3231,8631,
	3230,8632,10535,10534,4486,10533,10536,11147,
	11659,5105,5350,12456,5753,6026,1900,6976,
	2305,3232,8634,3661,3660,18028,11151,11150,
	12096,13345,737,6027,846,1226,6317,1901,
	3662,738,6977,1902,6978,5106,739,16903,
	6979,16901,7410,7409,7407,7406,7411,7408,
	3233,8635,9291,16249,3663,9962,10539,10538,
	10537,10540,11153,11152,4830,11154,5107,11660,
	12097,12457,13192,740,848,847,1566,6028,
	6058,998,1567,14507,6605,2306,2310,7412,
	2309,8004,2795,3236,8638,16227,4099,741,
	17647,6059,16904,849,15877,17648,6062,17281,
	852,6061,851,850,1227,6060,16525,6150,
	6149,1004,6151,1005,1000,15290,1001,16908,
	1007,6148,999,1228,6336,6334,6321,1240,
	6328,1235,6322,6330,16586,6327,1238,6326,
	1245,6335,1247,18030,1249,6320,16910,1236,
	6337,1243,1246,14246,6628,1230,1253,1244,
	6323,1233,1248,6331,6626,6324,1239,1250,
	6338,6325,1237,1242,6606,1251,6332,1241,
	1252,1229,6329,1234,6333,6627,18031,1579,
	1595,6613,1586,15606,6634,1571,16076,1574,
	6625,6611,6615,1582,1584,6618,1591,1577,
	1590,6630,1585,6617,6619,1583,1588,6620,
	6633,1904,1594,16496,1572,6632,16273,6636,
	6616,6638,6623,1581,6612,1580,1598,1587,
	6610,6990,1593,1597,6624,6629,15703,1599,
	1592,1578,1568,6614,1569,6637,16269,6608,
	6607,6621,6609,1596,2311,1589,6622,1570,
	18915,6997,7000,18033,7003,6992,14873,18034,
	7006,7009,7011,6989,1905,1910,7417,1926,
	7004,7012,6996,6988,7444,1913,7001,6993,
	6995,1918,6998,1912,6985,7007,15043,1909,
	18035,1921,6982,1923,1907,1927,7008,6984,
	1924,7438,6631,1911,1906,6991,6983,1925,
	1917,7002,1922,1920,6994,1914,6986,6987,
	7005,1908,6999,15603,7420,2319,2324,16540,
	16930,7429,7435,7431,7418,7421,2331,2316,
	7430,7439,2326,2312,2321,7422,2323,7423,
	7433,16629,14807,2325,7448,7416,2318,2317,
	2329,7413,7446,7434,7447,7441,7427,7442,
	18036,7436,7445,7443,2330,7428,2315,2314,
	2322,2328,7440,2327,7425,2796,15848,7449,
	7424,7419,2320,2332,2313,16929,7432,7437,
	18038,15033,2830,8006,8010,2811,2809,18039,
	2800,8008,8007,2820,8013,2814,16379,8031,
	8043,2797,8028,8021,2832,8005,2829,2817,
	2828,2807,8017,8035,8027,2808,2802,8041,
	8034,8036,2810,2818,8038,8015,8032,8020,
	8040,2823,2799,2821,8025,2812,8019,8018,
	2801,8012,8042,2803,8022,2833,2827,8011,
	2824,2822,2831,8029,2826,7426,8030,2825,
	8037,2798,8026,2816,8014,2815,2813,2805,
	2804,8639,8039,18850,15610,8660,15038,2806,
	18041,3265,2819,3247,8654,3261,8642,3244,
	3240,8672,3246,3250,3245,3254,8676,8667,
	18042,17154,8649,3259,3253,8661,3237,8673,
	8666,3241,8655,3256,3238,8671,3258,8659,
	3262,8650,8657,8651,3260,3268,8647,8643,
	3243,8656,3257,3266,16948,16849,8658,8682,
	8674,3239,8680,3251,19024,3249,16614,8686,
	3248,8664,8652,8662,3271,8668,8665,8648,
	3267,8678,8675,8683,3242,8663,3270,3252,
	3255,8669,3269,8653,8009,14803,8679,8677,
	8681,8644,3264,9324,9323,9297,3669,3679,
	9331,3674,9321,17793,9294,3680,3670,9309,
	9299,9302,9326,3664,9333,9325,3673,9314,
	3684,3683,3677,9332,3665,9312,9303,9316,
	9328,3685,16894,3667,9311,15884,9304,3676,
	18045,3675,9307,9317,9327,9298,9308,3668,
	9318,3681,3672,9305,3671,9296,9310,9330,
	3263,4127,9313,9963,9315,9329,3678,9306,
	3666,3682,4506,9295,9320,16489,18047,16431,
	9319,17720,9991,4124,9980,9985,4113,9965,
	4126,4104,9964,9997,4128,9968,9984,16955,
	9970,10000,9996,4102,4112,10566,4125,4110,
	9966,16932,9995,4114,9992,10005,9972,9981,
	15034,15611,10003,9993,4109,9979,4103,4101,
	4119,16800,18126,9977,10004,10002,4107,10006,
	4111,4118,18051,9967,10542,9976,4105,4123,
	4120,4108,9986,9971,4121,9988,4115,4117,
	4100,18052,9990,9982,9969,4116,9998,10001,
	9978,9989,17026,4106,4489,9987,10541,9994,
	16960,9973,10551,10561,4492,10560,10572,4494,
	10558,15954,10562,4505,10553,4497,16961,10565,
	11162,4509,4508,10567,10550,10577,4503,4493,
	10570,10574,10579,10568,10556,4496,4499,4507,
	4502,10559,9983,16963,10555,10578,4498,15294,
	4501,10580,4490,10569,10543,10575,5116,18055,
	10557,4491,10552,4495,10544,4122,10548,16789,
	10573,10549,10547,4500,10546,10564,10563,10576,
	10554,4504,16953,11165,16966,4843,4833,11177,
	11157,4835,11169,4842,4837,11167,11662,11173,
	11179,11172,11155,11170,11181,11183,4832,11182,
	4838,4844,15911,4841,11176,4840,11171,15291,
	11159,11164,18056,11175,4839,4836,4831,4834,
	11163,10545,11158,11161,11178,11166,11156,11661,
	11180,11160,11174,16968,11664,5120,5108,15476,
	15475,5112,11666,11672,11670,5118,11673,11665,
	5113,11669,11668,11674,5119,5114,5117,11667,
	5121,5115,5122,5109,11671,14501,12110,16951,
	5355,12107,12109,5353,12105,12106,12101,18146,
	11168,12102,5354,12098,5351,12111,5352,12104,
	5357,12103,5519,12461,12467,5520,12458,5517,
	5514,12469,5518,15478,5515,12460,12108,12459,
	5516,12466,12783,12462,18062,12774,5661,12777,
	5662,12786,12776,12773,12775,12779,12782,12781,
	12785,12772,5660,12780,12784,12787,12778,13018,
	16980,5754,18887,18064,5832,13196,13347,13346,
	14418,13450,5922,13448,13451,13449,13520,5956,
	5970,13596,13574,15605,13595,742,17649,19154,
	18066,1010,6154,6339,14535,1254,18856,1257,
	16537,6340,16981,18067,18777,6647,6649,6644,
	6641,6643,17401,1603,16587,1601,18099,6645,
	1602,6642,6639,1600,6646,6640,1604,6648,
	7014,18242,7018,14706,1936,15543,18128,16982,
	1928,1931,1933,1935,1932,7017,7016,7013,
	1934,15721,1929,18450,17753,17008,7015,2333,
	7454,16984,7462,7467,2338,14701,7452,7458,
	7451,7461,7457,2334,2336,7466,7450,7453,
	7459,16987,7460,7468,7456,2838,8050,15048,
	8052,8048,18272,8060,8046,8059,2834,8044,
	8690,8054,2837,15630,8055,8058,8051,14883,
	8056,8061,8049,8045,8063,8700,3279,15687,
	8688,8697,8687,3276,8695,8693,8694,3274,
	8698,15273,16993,17002,8691,8689,3275,8692,
	8696,15700,15486,14189,3277,8699,15045,17304,
	9339,9346,9348,9354,16994,3698,9334,3690,
	16996,9349,3695,9355,3686,16999,9352,18533,
	9336,9353,9335,3700,19040,9347,3687,9356,
	3692,9340,3697,9351,9338,9345,9341,9337,
	3689,3696,3694,3691,3699,3688,9344,15557,
	3693,9350,15808,9342,18407,15597,16384,14679,
	9343,18724,10014,14389,4131,14704,10010,10016,
	10012,4133,10011,10015,10007,18182,10009,4132,
	15675,10013,10008,18866,4134,16991,10017,18461,
	4130,10586,10583,10595,10589,10591,4510,10585,
	10593,18625,16072,10592,10590,10596,4513,10587,
	10594,4511,15901,10582,10584,4512,10581,10597,
	16990,10588,18059,11187,4851,17006,11198,11196,
	4845,11186,11184,11199,11194,4849,4846,11193,
	11191,4852,11195,4847,14141,4848,11192,4850,
	11188,11197,4853,4854,16504,5124,5131,11675,
	11681,11679,5127,5126,5123,11677,15489,5129,
	5128,5125,11680,11676,11678,5130,18158,16026,
	5361,12114,12116,5360,12113,18916,12112,12473,
	12115,12474,5521,12472,15803,12471,12470,5522,
	17009,14844,5663,17011,14675,14697,13025,13024,
	5755,13348,13452,17014,18996,13521,13621,5990,
	13641,743,17650,5132,744,1607,2339,3701,
	745,2839,4135,6029,17018,7022,7469,5133,
	746,1608,8701,3280,9357,15049,3702,10019,
	17021,4514,17022,5523,747,8702,748,18010,
	1012,6155,1011,1260,1259,1258,6341,16059,
	1609,1610,6650,7028,1939,7026,1938,7027,
	1940,7472,7471,7470,2340,2840,8064,8067,
	3282,2841,17027,3281,8706,8705,15050,8707,
	9359,9358,9362,17353,9361,9360,4136,10022,
	15051,10021,4137,10020,10598,10599,4515,12479,
	5524,12478,12477,5756,12790,13199,13522,749,
	17652,6063,853,6064,6342,1611,6346,1262,
	6344,1261,6348,6345,6654,7029,6652,6659,
	17354,1612,1615,6660,6656,6651,1614,6653,
	1613,6658,16279,7036,7030,7031,1941,7034,
	7032,7478,7475,7477,2345,2344,2342,7474,
	7479,2341,2343,7476,8075,7473,7480,16282,
	8070,8074,8073,8709,8720,8076,18154,8071,
	8708,2845,2844,8069,8072,2846,2843,2842,
	8068,8077,8711,8718,3284,8717,8714,17034,
	3286,15728,17029,16306,8716,17035,8710,8712,
	8715,8713,3285,8719,3283,3703,18155,17356,
	9366,9365,9363,3706,3705,9367,18156,9364,
	10023,4138,3704,17402,9369,9368,10026,10024,
	4516,16209,4139,10025,10601,4517,10600,10609,
	10608,10606,10607,10610,4857,11204,11210,11209,
	11207,5134,16280,5135,11682,16084,5363,12117,
	5362,5525,5526,5664,12792,12791,13026,13200,
	5834,13350,13349,13351,854,7037,2346,2847,
	8078,855,6065,750,18918,6158,18149,6349,
	1263,6350,14838,16522,6668,14683,1618,6666,
	6662,6665,6661,1620,6664,16519,1617,1616,
	17037,1619,6667,6663,1947,1950,7045,7044,
	7043,1944,7052,7042,18868,1946,7485,7049,
	7047,1949,7039,7050,19123,7041,7051,7048,
	7040,17040,1945,7053,7038,1948,16667,17036,
	15127,7490,7481,7492,16499,7484,7482,7494,
	7491,7489,7493,2352,2350,18160,15523,7487,
	17061,7483,15546,7486,7497,7488,2351,7046,
	2347,2349,16401,15531,8092,8081,8079,17050,
	8080,15353,8087,15532,8088,8084,2852,8089,
	8086,8083,18875,2850,8082,2848,2851,8085,
	8093,2348,2849,8091,8090,2853,17772,15334,
	14836,17051,8725,16557,18163,8726,3296,14557,
	8732,8734,8727,3290,8731,8730,3291,3297,
	3298,8733,3288,8724,8722,8728,8721,3295,
	8723,8729,8735,3289,3294,15073,15113,17054,
	3287,3713,9376,3712,9380,14874,9370,15319,
	9381,17060,15541,17041,19116,9382,9379,9374,
	9373,16945,9383,3709,14837,9375,3714,3708,
	17094,3711,3710,17058,19020,10027,4141,4140,
	16641,10032,15086,4518,4142,15335,17064,4144,
	10033,3707,4143,10029,10031,10028,17065,10030,
	17062,10620,15057,10619,10616,4521,11211,4522,
	10615,17068,4520,10617,10614,10611,10618,4519,
	14881,18077,11214,11216,11685,4860,14835,11212,
	15114,4858,19021,4862,4861,11213,11217,4859,
	16752,11692,5138,5364,5139,5136,11689,11687,
	11693,5137,11691,11686,15525,12120,12123,12482,
	12118,18922,12793,12124,5527,12122,5365,12119,
	12121,12125,12483,12481,19125,5528,12480,15070,
	5665,15003,18170,5758,13202,5757,13204,13201,
	13203,5886,13453,856,6669,2854,9384,4863,
	5529,5835,13454,857,16382,6670,288,16358,
	16283,16351,9387,9386,9385,10034,4145,16350,
	10622,10621,17079,11697,12127,12126,5366,12484,
	13027,858,17795,1951,2857,9388,17080,859,
	7502,2858,6159,1265,1264,1952,7056,8737,
	16007,17754,6355,6354,15867,1621,6673,17087,
	1956,1954,17086,1955,1953,16363,2353,15082,
	2357,2356,7503,2355,2354,7504,15729,2861,
	8095,8094,3302,3301,8739,18173,17089,8738,
	2862,3717,9390,3718,9389,16207,10037,10623,
	4523,11219,15077,11698,5531,5530,5836,866,
	6674,18175,17093,2863,10038,4146,558,1622,
	17095,7060,1957,7505,16045,17096,2866,7511,
	2367,2365,7512,7507,2364,2363,2358,7510,
	7509,7506,2366,16275,7508,2359,2360,18177,
	2868,8096,8098,8102,8101,8097,8747,3307,
	3306,8741,3304,17358,8744,3308,8745,3309,
	8742,3303,3305,8746,8743,8740,9392,3720,
	3723,3722,3727,3726,9404,9402,9395,9399,
	9398,3724,9401,9397,9405,9396,3725,3719,
	3721,17097,9394,16271,10040,4150,10044,4149,
	10041,4148,9393,9400,10043,4151,10045,10042,
	10039,10626,10630,10632,10628,10627,10625,4526,
	4524,10629,4527,10624,4528,4147,10631,4525,
	17098,11221,11220,11228,11222,11227,11224,4866,
	11226,4868,15984,11225,11223,5141,11699,5140,
	11702,5142,18178,15986,17100,12131,17099,5369,
	12132,16112,5367,12133,5368,12130,12134,5532,
	12485,5533,5666,12129,5667,16042,16297,5759,
	13028,5838,13205,5837,17101,13352,15083,18179,
	13635,559,1962,7061,18181,867,1013,6160,
	17363,6358,1268,8104,7513,2368,17364,2869,
	8103,17104,8749,3313,8748,3312,9406,4531,
	10636,11703,17110,12135,12795,12794,13206,868,
	6676,2369,3314,9407,17111,10046,4532,11229,
	12136,12796,869,1626,1968,7062,1969,1967,
	1966,7514,2370,2871,8105,2870,18891,8750,
	2872,3315,9408,4153,4533,4870,11230,4869,
	11704,5143,12137,12797,870,1269,6679,6678,
	1627,6677,1628,6680,7068,1972,1971,7070,
	7069,1976,1977,7063,1973,7067,1974,15714,
	7517,7521,7519,7518,7515,17118,7525,7520,
	16360,8114,17119,7526,2376,2373,8112,8111,
	8116,2876,2873,2877,8106,2878,8115,2875,
	8113,2874,18186,8753,8755,8754,8757,3316,
	9417,9409,9413,17123,9418,9414,3730,3738,
	4157,3741,3734,3739,3732,9416,3740,9415,
	3736,3731,3737,9419,10051,10053,18089,15104,
	3735,17175,15108,10048,4155,10054,4156,10052,
	10047,10050,10055,4154,10049,4535,10640,10639,
	4538,4536,10637,4534,10638,11240,18190,11239,
	11231,11235,11234,11232,4871,11233,5148,11712,
	5145,11706,5147,5149,17125,5146,11711,11707,
	5144,11710,11705,11709,11708,15987,12139,5370,
	18193,5371,12138,15281,12488,16833,18194,12798,
	12801,5760,13353,13455,5971,871,1978,18894,
	8760,872,1270,1629,16513,8761,3317,3742,
	5150,11713,12489,12802,873,17129,6686,6681,
	6683,6685,6684,6682,1630,16341,1979,7081,
	7076,7073,15942,7080,7079,7078,7077,7082,
	1980,18196,2381,15123,2387,2386,7535,7529,
	7527,2384,8123,2379,7532,7536,7538,7534,
	7528,2385,7533,7531,2378,7539,2388,7537,
	2382,7530,2383,2380,17366,16410,15466,15044,
	2880,15403,8124,18197,8117,8119,8121,15469,
	8122,8120,2881,8125,18198,8118,16277,15467,
	8765,3318,8773,8772,8771,8768,2879,3319,
	8766,8769,3320,8770,8767,9431,16105,9435,
	3749,10061,3752,9433,14056,9430,9425,9421,
	15125,3748,3747,15124,3743,9424,3751,9423,
	9426,9434,9432,9422,10063,4159,10065,9429,
	10067,4163,10068,10064,4160,10060,4162,10057,
	10062,10066,10059,17132,3744,17140,10056,4161,
	10058,17777,15459,4541,10642,4545,17133,10648,
	10651,4542,4539,10644,10641,10646,10643,4546,
	10645,15126,17134,10647,4544,17138,16268,4876,
	11247,11241,11245,11246,18874,11243,4878,4875,
	11242,11244,4877,11250,14977,5154,15162,16574,
	11720,11714,5153,14515,11722,5152,11715,11723,
	11719,5155,11716,11724,11721,16335,12144,12146,
	12145,12491,12493,12490,12495,12804,12807,12803,
	5668,12808,5669,5671,5670,13030,17136,13032,
	13031,18203,18202,17139,13207,14211,13456,13525,
	874,17655,17152,6359,1631,6688,7087,7086,
	1983,1986,1985,1984,7083,7085,7084,7542,
	2390,7546,7545,7544,7540,2389,2393,2396,
	14309,15128,2397,7541,7543,2395,2394,2392,
	2391,8132,15997,8130,8126,2882,8127,2883,
	8778,16744,8777,8776,8775,9439,3753,9436,
	9438,15910,3754,17148,3755,9437,15862,17145,
	17800,10077,10079,18205,10078,10075,10074,10073,
	10072,10076,10069,17404,10652,10657,10655,17150,
	14686,10654,10653,10656,11253,15134,4879,5156,
	11726,17153,5157,11725,12148,12147,5375,12496,
	5536,12809,5839,13208,13458,13457,6066,3757,
	875,1273,1272,1271,15565,6689,17157,1635,
	1634,19025,1991,16947,7089,7092,7090,19111,
	17159,7088,7091,7553,7555,7554,2401,7549,
	15450,2399,2398,2402,2400,2403,7552,7551,
	7550,8134,2885,8779,8781,8783,3324,18206,
	8780,3323,8782,3321,9449,9440,9443,9447,
	3761,9445,3759,9448,3758,3763,3762,3760,
	18883,10080,19119,16346,4167,10083,10082,4168,
	19112,10659,15137,4552,4548,4550,16267,4547,
	4549,14700,11254,15857,4882,11256,11255,14131,
	10081,4884,4883,11729,5158,11730,11732,11728,
	11727,12149,5378,876,6161,1274,6690,1637,
	1636,6691,14135,7094,1992,7093,1993,15741,
	2405,7558,7556,7562,2406,7557,7561,8138,
	8140,17368,2886,18926,8139,2887,3328,3327,
	3326,8784,9451,9450,10086,4170,4169,4554,
	4553,14138,11260,10660,10662,10661,11259,11257,
	11261,11258,4885,14137,18209,11733,5159,5672,
	15743,17776,5887,877,7563,2407,14144,18213,
	14673,14145,3038,3037,15155,3330,8786,3329,
	8785,14429,14385,14146,9453,4171,10089,4172,
	5673,12813,1014,1638,6692,19095,1995,15165,
	1994,7565,7567,2408,7569,7572,7570,14150,
	15203,7571,7568,7564,2409,7573,7566,16247,
	8149,2893,8158,2890,8151,2894,16145,2888,
	8146,8159,8147,8144,2892,15199,2889,14152,
	8150,8153,2891,8154,2895,8155,8145,8152,
	8156,8148,8143,8141,8157,15726,15853,15909,
	8789,8795,3333,8788,8790,3331,8787,3338,
	8791,3337,8792,3339,3334,3340,3335,3336,
	14151,3332,3768,9464,18215,9467,9455,9460,
	9454,3770,18216,9456,3769,14156,9463,9462,
	9459,9461,9457,4176,9466,3766,9465,4183,
	10100,4182,10090,10092,4175,10097,10096,10099,
	4180,10094,10093,14162,4179,4174,10095,4177,
	10101,10103,10098,10091,4178,4562,4173,14155,
	16148,10104,10666,4555,19140,10668,4556,17948,
	4558,10671,10663,4181,10669,3767,4561,4557,
	19117,4888,10670,10664,4563,10667,16152,14160,
	11271,11266,11276,4886,11268,4890,11274,11265,
	11262,11277,5165,4891,11273,11263,4889,11267,
	4893,11264,11269,4892,11272,11275,18927,11738,
	11745,11743,5163,11750,11270,14167,11741,16159,
	11751,5162,11740,11739,11749,11746,11748,11736,
	11752,5160,11747,11756,11744,5164,5161,11742,
	11737,11755,4887,18218,17369,12150,12156,5384,
	12154,12153,5386,12159,5385,12160,12157,5382,
	12158,12151,5383,5381,12502,12155,5543,5541,
	12501,12503,12500,5542,16156,5675,14171,12818,
	12817,12815,12814,12816,5674,5676,5761,16312,
	13034,13033,16127,13211,13213,13215,13214,13212,
	15167,5841,5840,5889,5888,5890,13355,13354,
	13526,13459,13527,5957,15168,5958,13575,16210,
	5995,1015,18076,6693,7098,18221,15177,1996,
	16154,14172,7574,7580,16157,7578,2410,7576,
	7577,7579,15176,7575,2896,8160,2898,8162,
	2897,8161,8797,3341,8799,8796,8163,3342,
	14174,14061,8798,15178,9470,3771,9468,3772,
	9469,3773,18224,4184,10109,10106,10108,16343,
	18928,14177,10673,14176,4564,10675,10674,15173,
	289,11278,15708,11279,5171,16345,5167,5172,
	5168,5170,5166,14178,5387,11757,12504,17370,
	12819,13217,13035,14180,13216,14181,13528,13597,
	1016,17656,1275,17773,6694,1639,1999,7106,
	1997,7104,2002,1998,2003,2001,7105,2000,
	2414,2413,7591,2422,7584,7590,2418,7583,
	7589,7587,2417,2419,7586,2412,7585,2423,
	2420,2424,2421,7582,7581,7588,2415,2411,
	2416,8178,8171,3347,8170,2902,2909,2906,
	2912,2911,2907,8164,8167,8166,2903,8168,
	8177,2904,8165,2905,8172,2910,2900,2908,
	8169,2899,8173,16349,8816,8807,3344,8814,
	8811,8803,8812,3346,8804,16019,3779,8809,
	14185,3343,8815,3350,3352,8802,3351,8805,
	3345,8813,8806,8810,8808,3348,8801,3353,
	2901,3349,3354,3775,9477,9479,9481,9485,
	9475,9473,3777,9480,9484,9476,9472,9471,
	14187,18902,9482,9474,9478,3778,3776,9486,
	3774,9483,19003,10126,15187,4189,10118,4577,
	10128,4192,10132,4198,10113,18135,10129,10110,
	15516,10131,10114,14188,4205,4202,10130,10124,
	4188,4196,4195,14190,4194,4200,10111,4201,
	10125,4197,4187,10127,4199,10116,10115,10112,
	10120,10117,10121,4204,10133,4193,17774,10119,
	18229,18231,4203,15182,18230,14191,10680,4569,
	4578,4574,10677,15180,4571,4575,10689,4565,
	10681,4573,14194,14198,10684,10679,4572,4576,
	10678,10688,4567,10687,10686,4579,10440,4566,
	14196,10685,10676,4580,10683,4568,18232,15567,
	15181,4898,4903,4896,11295,11286,11298,11290,
	11296,4904,4897,11283,11289,11299,11292,11297,
	11285,11293,4899,11291,4902,4901,11284,11294,
	4900,11301,11300,11768,11764,5179,11758,5173,
	5189,11770,5181,5177,11761,5184,5187,11772,
	5176,11763,5185,11774,11771,11759,5180,5174,
	5188,11766,5183,11760,5178,11773,5182,5175,
	11767,5186,11765,12163,12169,5393,12172,12164,
	12171,12166,5394,5391,18964,12162,5390,12168,
	12170,5392,12167,12165,14182,12515,5545,18929,
	5546,14203,12511,12509,12513,5550,12514,12507,
	12506,12510,12508,5547,12512,12820,5681,5680,
	12821,15953,12823,12822,5682,13036,13039,5763,
	13041,16073,5762,13218,18078,5891,5893,13356,
	5892,13460,13530,13532,13529,13531,5977,1017,
	17762,2004,7455,2425,11302,5190,11775,14210,
	5395,15977,12517,12516,5683,13042,15183,13219,
	5926,6162,17654,17657,1640,1276,7597,7592,
	7596,7594,2426,7595,7593,9488,9491,9487,
	3780,4206,16391,3783,10134,4581,10690,4582,
	16251,4905,11305,11778,11777,11776,5551,12519,
	13220,5927,13461,1018,1642,1641,7109,2005,
	16924,7108,7599,17371,2427,8180,7598,14214,
	2915,8183,8821,14216,3786,9493,9492,9494,
	3785,3784,10691,10693,17372,4583,10692,11306,
	4906,12174,18233,12173,5552,12520,5554,5553,
	12824,13043,5764,1019,7110,2006,7602,2429,
	7601,7600,2916,8186,2917,8190,15198,10138,
	4209,10137,10698,18153,10699,4584,10696,10695,
	10697,10694,4909,11307,11780,5191,11779,12031,
	5396,14222,5397,5192,12825,5684,1020,17661,
	1021,2431,1643,2430,7111,14225,3358,1022,
	2008,2007,2009,1023,7114,2433,7603,2435,
	2432,2434,8194,2919,14226,8193,8192,18236,
	9497,10139,18237,4585,4910,11309,11308,18238,
	14228,13044,18239,1024,6360,6695,2010,7115,
	2436,7604,2437,2921,8195,8197,8825,3359,
	4211,16357,10141,10140,4210,14233,16355,16348,
	10700,16347,10701,5196,5195,12177,5399,5398,
	12523,12826,5843,5842,1025,17662,3790,3360,
	3789,4212,1026,6067,6696,6363,6364,1278,
	6361,1277,1280,6362,1282,1281,6365,1279,
	1648,1646,6698,1650,1652,1649,6701,6697,
	1653,1647,1283,1651,6700,16046,18243,6699,
	17778,15995,7123,15220,7117,7120,7129,7127,
	2016,7133,2019,7130,7122,7119,8829,7128,
	2011,7131,7116,7125,2013,2018,7126,2022,
	2020,7118,2017,7124,2021,2012,7132,17897,
	2442,2451,2440,2438,7607,2446,2443,7609,
	17780,2445,7608,7606,7611,2450,2448,8828,
	7612,7610,2439,16359,2441,2444,14242,2447,
	15221,2449,18249,8207,2923,15981,8198,8200,
	14243,8201,8209,8205,8203,8206,8210,2924,
	2928,8199,2926,14905,2925,8204,8202,2922,
	2927,3366,8837,8835,3368,15222,8830,9511,
	3367,8834,15990,8831,3363,3369,8836,3365,
	8833,4213,3364,8832,3370,3362,3361,18252,
	9503,9501,9499,9512,9504,9498,3794,3800,
	9509,3797,19053,17348,3795,9510,3792,3791,
	9505,3796,3371,9508,9500,3793,4218,4214,
	4219,10144,17349,10143,10145,4216,10142,4217,
	10148,10147,10146,10708,4591,10707,4590,10704,
	10703,4589,10706,10702,18259,11312,11322,4913,
	4912,17252,11316,11318,11317,11311,11320,4911,
	11319,11315,16050,11321,11314,5200,11782,11788,
	5204,5206,5203,5202,18261,5201,5199,11783,
	11786,5198,11787,16366,5205,11785,11784,5400,
	5401,12180,12524,5555,12828,5685,12827,13045,
	13221,5844,5894,1029,18262,1654,8838,4220,
	5207,11789,1030,2453,2452,8839,17105,11323,
	1031,2023,4221,4914,1032,1655,7134,2454,
	7613,2929,8843,3801,4222,4915,5208,5402,
	12526,1033,1656,2455,8211,3372,4223,10149,
	10709,16353,1034,3373,9513,4224,1035,6702,
	7135,2024,7615,16909,2458,14256,2459,16080,
	8216,8213,8215,2930,2932,2931,8212,2933,
	8214,17794,8846,9516,9515,9518,10714,10710,
	10713,10715,10712,10711,18263,11791,11790,11792,
	12182,12181,12528,15226,12529,12527,15227,5686,
	13222,13357,13462,1036,1284,5209,1037,8217,
	8847,18265,5934,6163,18210,17763,14280,6164,
	6166,1038,6167,6165,6372,1286,1287,6369,
	6367,6366,6370,1285,6371,7136,6707,1659,
	6706,6708,1658,6713,1662,6703,6717,6723,
	1669,6720,1666,15731,6710,6716,6718,14268,
	6705,1665,1660,6711,1667,1670,1664,1657,
	6715,6709,6722,1672,1668,1663,6714,2460,
	6712,1661,1671,18953,6704,6719,6721,17891,
	17764,14273,2041,2035,2043,2040,7143,7146,
	2036,18267,7137,2029,2039,2042,2044,7158,
	7149,14281,2028,7157,2032,2030,2025,7141,
	7156,7145,7148,7161,16408,2045,7155,2037,
	7150,7160,7147,7151,7154,15991,18930,7139,
	7159,7153,16685,15762,7138,17374,7142,2038,
	2033,2026,2031,2027,2046,7140,7632,2034,
	16054,7152,6781,7635,2474,14282,7619,14274,
	7630,7626,14289,7645,7636,7621,7625,7648,
	2477,7640,7631,2461,7646,7617,2476,2471,
	7616,2469,2468,2473,7638,2465,2472,7144,
	8848,7633,15658,7623,2475,7624,7628,2478,
	7618,7642,7641,7649,2467,2464,7647,7643,
	7634,7629,2470,2466,7620,2462,7644,2463,
	7622,18269,14262,16875,8223,8225,8230,8250,
	2947,2937,8233,8240,8246,14291,2950,8248,
	8243,2945,2943,8239,8235,8221,2934,8226,
	8218,14316,2942,2944,8231,8228,2939,8244,
	2936,8229,8889,8237,14157,8236,2935,2938,
	8219,8224,8241,8247,2951,8220,8232,8238,
	2941,14294,8249,8242,8245,2940,16217,8864,
	8852,3381,8881,8890,8893,3379,8860,8885,
	8861,8875,3392,8871,3389,16469,8872,8849,
	14296,3398,8883,8873,8895,8868,9550,8898,
	3396,8866,8878,3399,3378,8870,8896,8863,
	8856,8867,8855,8853,3374,8887,8862,8891,
	15660,8892,3382,3387,3383,3391,8882,3384,
	8874,8858,3376,8850,8884,8894,8857,3390,
	8899,8869,3375,3395,18271,8880,3397,8876,
	3386,3380,3388,3377,3394,8877,8859,8879,
	8854,8888,15376,8897,16051,15445,14161,8851,
	18769,9571,3756,9554,16941,9552,9531,3806,
	9547,9567,9543,3814,9566,9541,3393,9557,
	9542,9564,3813,3805,15649,9520,9528,9553,
	9545,9570,9548,3819,9565,18200,3810,14308,
	9551,9559,9532,9558,9575,9527,9560,15229,
	9573,14731,9521,3385,14302,9535,9534,3812,
	16091,9538,9540,9555,9572,3815,17114,3816,
	18273,9526,3808,9530,9568,3817,3809,3811,
	3818,9574,9561,9556,14304,15507,9537,9536,
	3807,9522,3804,9546,9523,9544,9533,9569,
	14670,18121,9525,9563,9524,4238,10189,14166,
	14318,10172,15214,18118,10150,10180,4230,10165,
	10175,4233,10168,4231,10153,10183,10152,10171,
	15262,10151,10169,10178,10176,10164,10159,10157,
	15756,10158,10177,10188,10166,4232,4235,10160,
	10154,10170,4239,15436,18276,4227,4236,10162,
	10156,4229,9562,4228,10173,4226,4241,4234,
	10174,10163,10155,10182,10167,4240,10186,4237,
	10185,10179,10184,16373,10187,18277,14261,14319,
	15312,10743,10742,10738,4603,4595,10758,10757,
	9519,10735,10739,10733,10752,10737,18278,10759,
	10730,10728,10740,10750,10745,4605,10719,10749,
	14728,19147,16619,4606,9539,10731,10722,10725,
	10763,10732,10723,10721,10718,18279,4599,10741,
	4598,4602,10736,10744,4592,10760,10764,4594,
	10727,10747,10751,10724,10761,4601,4600,10716,
	4604,10754,10726,10746,10720,10734,10729,10748,
	10765,10762,10753,16936,14324,4593,11343,11334,
	11337,16482,4923,11339,11328,11335,11341,4920,
	4924,4918,14272,16399,11330,11346,11340,18281,
	11331,11355,11349,11327,11811,11333,4919,14299,
	11344,11354,4927,18938,11351,11332,11338,11342,
	11336,11356,11353,11350,4926,11329,11357,4925,
	11347,11348,11824,11805,11810,11823,11809,11801,
	11806,11794,11818,11820,11793,5211,11822,11814,
	5218,11821,11807,5220,11799,11352,11815,11812,
	11795,11826,5214,19018,5215,11797,11813,18931,
	11825,11816,11803,5217,5213,11817,11804,11827,
	11798,11808,11819,11800,11802,5221,11796,5219,
	5404,5210,5216,5408,12198,12188,12183,12189,
	12199,12203,12202,5410,5409,12190,12204,12193,
	12185,14334,12187,12186,12192,12197,12184,12191,
	12196,5407,5405,12195,5403,5406,12545,12200,
	5559,18951,12537,12541,12533,12536,12544,5557,
	12549,12542,18285,12534,12550,12543,12535,5556,
	5558,12530,12538,12532,15446,12548,12546,12531,
	12539,12837,5562,12540,5688,5690,5687,15440,
	12842,12834,12832,12836,12833,14336,12838,12840,
	5691,5693,12839,5694,5692,12841,16484,18882,
	15231,5689,19107,18110,14331,5765,13046,13052,
	5767,12835,13051,13058,13049,13056,13055,12831,
	13050,12201,13059,13048,13053,14341,13057,13047,
	13226,5766,13054,16286,13362,13227,13225,5895,
	13360,13358,13363,13361,13364,5896,13359,14529,
	13463,13630,13622,6168,1673,7651,7650,2479,
	2952,8252,3400,3821,3820,3822,10190,10766,
	11358,11360,11359,5222,11828,12205,13598,1039,
	14355,6725,6724,1674,18287,7165,7163,2048,
	2050,2049,7164,7663,7660,7658,2480,7661,
	2485,7655,7670,7672,7656,18288,2482,7673,
	7668,7654,7659,7669,7665,7662,2487,7671,
	7657,15240,7666,7652,2486,2483,7664,7667,
	7653,2484,2481,2962,8259,2961,17376,8262,
	8265,2958,2956,8254,8263,8261,8258,8253,
	2955,8256,8255,8268,2957,8257,2959,2954,
	8260,2963,2960,8264,3407,8914,8903,3404,
	9581,8912,8900,3402,8905,3405,8909,8907,
	3408,3401,8901,8904,3406,8913,8911,8906,
	8908,8910,3403,9582,9588,9578,9584,3823,
	9580,3829,3827,9587,9577,9590,3832,3826,
	3825,9586,3833,9576,9579,9589,9585,10222,
	10211,3824,4248,10199,10220,10200,4243,10206,
	10198,4245,10192,10219,4246,10193,4251,10208,
	10202,10209,10196,10221,10213,10207,10191,4247,
	10214,4250,10218,10212,4244,10210,10204,4242,
	10195,10203,10215,10201,4616,9583,10784,10793,
	10783,10779,10776,4617,10774,4249,10767,4615,
	10773,4614,10778,10775,10791,10786,10780,10785,
	4610,10777,10796,10792,4611,10797,4613,10798,
	10782,10194,10788,10781,10790,10787,15980,18087,
	10772,4608,10795,4609,10770,4612,11371,10789,
	10794,11368,4607,4928,11378,11374,18662,11372,
	11367,11382,17378,16367,4932,11366,11365,11379,
	11369,11364,11370,11381,11362,11380,11377,4930,
	4929,4931,11373,11361,15752,16364,11830,5228,
	11834,11831,11838,11844,11833,14351,11853,5225,
	11847,11836,11848,11843,11850,11835,5230,5229,
	11837,11851,11829,11849,5223,16380,11841,11840,
	11845,11832,5227,5231,11839,11846,5232,11842,
	5224,5226,12218,12216,16274,12222,12220,12223,
	14352,12209,12217,12227,11852,12213,5414,10771,
	12208,12221,12215,12212,12207,12206,12226,12211,
	12210,5412,12219,14349,5411,5413,12214,12225,
	16302,5566,12552,5563,12560,5567,12562,12551,
	12224,12564,12553,16047,5564,12559,12561,12556,
	12563,12558,12557,5565,12848,12846,16250,12843,
	13062,13065,13061,5771,13063,5770,5769,5768,
	13064,13233,13231,15879,13230,13060,13232,13066,
	13229,15243,13367,13333,13365,5897,13366,13368,
	13466,5928,13465,13464,5929,5959,13576,13600,
	13599,13601,1040,7166,16218,13067,13467,1041,
	2051,7167,14404,8271,2964,8917,8916,3409,
	3834,10799,4618,4619,14363,14918,4933,5930,
	1042,17666,15868,7168,1676,7170,7169,2052,
	7676,15246,7683,2488,7681,7679,7687,2492,
	7685,7682,7680,7684,17143,15248,2966,8272,
	2971,8288,2970,8290,18854,18933,8278,2968,
	8289,8273,2969,8285,8281,8283,8277,8284,
	14364,19115,2965,8280,16288,8279,8275,8282,
	8274,8276,2967,8287,3412,8928,8924,8926,
	8921,8920,8918,8925,8927,8923,15249,8922,
	8930,3842,9592,9598,14058,9599,3836,3843,
	9591,8919,3839,3837,9597,9596,3838,3840,
	9595,3835,15250,10224,4258,10235,10234,10229,
	4260,10233,4252,4254,15880,10231,10223,4256,
	4255,10227,10232,10230,4257,10228,18294,17146,
	8929,4253,10800,10807,4621,10811,4625,10803,
	10801,4620,10809,10802,4624,10808,11383,11395,
	4938,11390,11392,18295,4936,11384,11388,11391,
	4934,4937,11394,11393,11385,11389,4935,11855,
	11854,5234,11860,5236,5233,11856,5237,11857,
	11858,11861,15894,5235,12232,17379,12229,12231,
	15252,11859,12228,15254,5570,12568,12571,12573,
	12570,12572,5571,5569,5568,12569,12565,12849,
	5697,15251,12850,13069,5772,13071,5773,13068,
	13070,5846,13234,5845,13371,13370,13369,15850,
	13541,13540,6169,1043,18184,2053,8291,3413,
	9600,5415,14370,12574,16361,18090,1288,2972,
	16352,8931,3414,8933,8932,9601,3845,10236,
	10239,10238,10237,10812,10814,10813,15763,4940,
	14374,16377,4939,5238,11862,11864,11863,12576,
	12575,12851,5698,14375,5774,13236,13235,5960,
	17667,1289,7171,2054,8294,8292,9608,8934,
	9602,9606,9605,9607,3846,9604,9603,9609,
	14377,10242,10240,10241,10817,10815,10816,11396,
	12577,12852,5699,13072,13237,5847,13372,13542,
	1290,7173,2055,2496,2503,2495,2504,7688,
	2501,2498,2502,2500,2494,2981,15267,2975,
	8298,2980,2982,2976,17381,2974,15313,8297,
	2979,8295,2978,18298,8939,3426,3415,3419,
	15266,3865,9614,8941,3420,8947,16368,8949,
	8950,8938,8937,8951,3423,8948,8946,3421,
	3417,3428,3416,9613,3860,3855,3850,16270,
	3866,3851,3847,3862,3859,3861,3852,3857,
	9621,9618,9610,9615,3863,9622,3864,3854,
	16293,9612,9620,9616,9619,9617,3858,9611,
	3853,10245,4262,4265,10247,14383,4273,10246,
	4267,18299,4630,10248,4272,10244,4274,3856,
	4266,4264,4268,4270,4261,4275,4271,14397,
	4269,10243,14401,4637,10820,4641,10832,10825,
	4626,10833,10827,4636,10828,4635,10831,4629,
	10829,10819,4628,11407,4634,4631,19049,4639,
	10822,4627,10821,4638,10826,10830,10834,14786,
	4642,4946,11402,11408,11405,11397,11409,11398,
	4941,4947,11410,15833,4943,4953,4948,11412,
	11406,4944,11399,4954,4955,4952,4632,19034,
	4942,11413,4956,4949,11411,4945,5246,11871,
	11880,11877,11879,5243,11872,14390,11878,5239,
	11874,5247,11870,11875,11882,11403,11876,11869,
	5240,11868,5241,11883,5242,11881,5245,11867,
	14391,5244,11873,12238,12252,12247,12249,12237,
	5418,12236,5421,5420,16281,12251,12243,12240,
	12246,12241,12250,12248,5419,12255,12253,12244,
	12254,12245,14394,12581,5572,12588,16370,13075,
	5579,12242,12579,5575,12580,12585,18092,14396,
	12578,12587,12589,12582,12584,12586,12583,12590,
	5574,5580,5576,5573,12855,15800,5704,12853,
	15828,12857,12858,5702,12859,12856,15351,12854,
	5705,5701,12860,5703,5700,5776,13074,13073,
	5777,13238,13243,5848,15757,13240,18923,13239,
	13242,13241,5898,13375,13373,13374,19135,14399,
	5932,5931,5972,14400,5978,13602,13631,17669,
	8300,9623,10249,10835,5249,5248,12256,12591,
	13377,1292,7689,2505,2983,9625,9624,4643,
	10836,4644,11884,5422,18301,5987,1293,6728,
	7690,3429,3867,9627,9626,9628,4276,4957,
	4645,4958,11888,11885,11887,11886,5250,12257,
	12593,12592,6375,2507,2506,7691,8954,8953,
	3430,3869,3868,4278,4277,10837,11416,4959,
	11890,11889,12260,12259,12258,12594,14405,13603,
	1294,2058,15277,2059,7693,7692,8304,2990,
	2988,2985,2989,2987,2986,16369,3431,8957,
	3874,3433,3438,8959,3440,3439,3442,8958,
	8955,3436,3432,3434,8956,3441,3437,3435,
	3876,3875,3873,3877,9632,16025,4279,18302,
	10252,14411,10254,10840,10842,14417,4654,10843,
	4647,10839,4646,4656,4649,10838,4648,10844,
	10841,15278,4655,16284,11420,11419,18303,11421,
	4960,11417,16305,5254,11891,5251,5255,5253,
	5252,12261,12263,14415,12262,5423,5581,12597,
	5582,17779,16278,5707,5706,13078,14419,14414,
	5778,13079,13244,15983,13378,5935,18304,17670,
	2992,2991,4282,11422,4657,11423,11892,1296,
	7174,2061,2060,7694,2510,7695,8307,14817,
	8306,8961,3445,16233,8960,3444,8962,3443,
	9636,9639,9642,9635,9643,4284,10255,10848,
	10846,4658,10845,10847,11424,16375,11425,5256,
	14422,12599,12601,12598,12600,12861,13080,13577,
	1297,2062,7696,7698,7697,8310,2994,2993,
	8311,17258,8312,8309,18306,8974,3453,8966,
	8972,3448,3451,8965,3446,8969,9649,3450,
	8964,16258,8970,8967,8973,8963,8971,3449,
	3452,8968,3447,3879,9645,3878,9652,9651,
	3885,3886,9653,3880,9650,3884,9655,9646,
	9648,3881,9644,9647,9654,3882,9656,4285,
	10260,10266,10258,10267,16261,10257,10262,10268,
	10265,10264,10263,10256,10261,10259,15966,4664,
	4661,10857,10862,10864,10861,10854,10856,10866,
	10863,10858,10867,10855,10865,4662,4668,4666,
	10849,4667,4663,10850,10852,10851,16264,4665,
	15284,4660,16043,10853,11435,4962,11426,4963,
	4966,11431,11428,4965,18308,11427,11436,11429,
	11434,4964,4961,11430,17170,11898,5259,5257,
	5260,5258,11897,11894,11893,16259,11896,11895,
	12278,5429,12269,12268,5424,12274,12273,12265,
	12276,12270,5428,12267,12275,12266,5425,5427,
	12271,5426,12272,17229,12277,12605,5587,12602,
	17383,5584,12604,5589,16265,5586,16255,12603,
	5588,12607,5583,18637,5709,5711,12864,5710,
	12862,5585,12863,5708,5779,5781,13082,5780,
	13081,13249,5851,13248,5852,13246,13245,13547,
	13546,13545,13578,15285,13623,13604,5979,1298,
	2511,14430,18935,4669,14496,11437,12608,14432,
	13605,1299,1677,2064,2063,18312,2514,7699,
	2512,7700,2513,8317,8313,15806,2995,8316,
	8315,8314,2996,18314,8981,8979,8986,8988,
	18315,8980,8975,8987,18316,8989,8982,8985,
	8976,3455,8978,8977,3454,3456,3890,9658,
	9660,9659,9664,3888,9661,9657,3889,3891,
	9665,10272,10271,4287,4289,4286,4288,10870,
	14434,10871,4671,4676,4670,4677,4672,10872,
	10873,10869,4678,4674,4673,4675,10868,14435,
	11439,4968,15287,11441,4970,11444,11440,11438,
	11443,4969,11442,5262,5265,11900,15623,5263,
	11899,5261,5264,5431,15288,12282,5432,5591,
	12610,12609,12613,5590,12868,12869,12865,12870,
	13084,16177,13085,13083,5782,13252,5853,13253,
	13381,13380,1300,3457,3892,15764,14438,4972,
	18864,4971,5592,5679,5783,1301,2515,3893,
	14442,12614,560,18387,17405,17142,18633,18631,
	18630,6377,18317,1302,1305,1304,1303,6376,
	18318,6730,18319,6732,1678,18937,18606,1680,
	6729,6731,1679,6734,6733,6735,7178,7176,
	2067,7175,2072,2069,2066,2068,2071,2070,
	7177,2521,7702,7708,2518,2525,14449,2520,
	7705,7707,2523,7701,7704,2519,2516,7703,
	2522,7706,2524,2517,8321,8324,2998,3008,
	3004,8322,15950,3012,3005,3010,3000,2997,
	2999,3011,8323,3003,3006,3002,3007,8325,
	3009,3001,8320,8991,3458,8993,3460,3462,
	18320,8992,3459,3463,14454,3461,3899,9670,
	3907,9667,3908,3897,19121,3902,9669,3895,
	3894,16144,3905,3904,3903,3901,3906,9666,
	3896,3898,3900,15826,4292,4295,4299,4293,
	4298,4296,4291,14457,4297,4294,15464,10879,
	4681,4679,10880,15712,4682,4680,10878,10877,
	4976,10876,4974,4973,11445,4683,4975,11446,
	4978,11447,4977,5267,11902,5266,5271,5434,
	5268,11901,11448,5433,5435,5900,5899,1307,
	6174,2526,6381,6378,6383,6379,1308,1311,
	6380,1310,6382,14460,14464,6738,6740,1683,
	6736,6739,6737,1681,1684,7180,7184,7181,
	2077,14462,7183,7185,7187,2074,7186,8332,
	7182,7709,7718,7716,7717,7719,2528,7714,
	7710,2527,2529,7713,7720,7715,3013,8335,
	8327,8326,8328,7179,8329,3465,9000,11449,
	9001,8998,3015,3467,8997,9003,9002,3464,
	9006,8996,9005,8994,9004,8999,3466,9672,
	9678,9673,9671,9674,3909,9675,3910,4301,
	4300,10279,10282,10274,10278,4302,10277,10275,
	10281,10276,10273,10280,10886,4686,12283,10882,
	8995,4685,10887,10881,4684,4687,10885,11450,
	4979,11903,5272,12286,12616,12615,12873,13086,
	13254,5854,1312,2079,2078,2532,2531,7721,
	16201,2530,8339,8340,8336,3016,8337,16197,
	8338,17387,9009,9011,9007,9010,3468,9008,
	3469,14466,9186,3913,3912,10284,10286,4306,
	4303,16027,4305,4304,10285,10283,4691,10893,
	4689,10890,4690,17388,11456,16206,11457,4980,
	11453,11907,11904,11906,5275,11910,5274,11908,
	11911,11909,11905,5273,12289,12287,14467,12291,
	12618,5596,12290,12617,5595,5712,12875,17903,
	12874,13089,5784,15304,13382,13475,13474,5936,
	5962,13476,13579,14468,5980,1313,1685,5713,
	1314,2080,3017,3470,5438,1686,7189,7188,
	7724,16734,2535,2533,2537,7726,2536,2534,
	7725,3020,8346,15366,3019,3021,8351,3023,
	8341,3022,8352,8344,8342,3018,18865,8350,
	8345,8347,15096,9023,14382,9022,9020,9012,
	9027,9015,9025,9032,9024,3478,16057,3474,
	9013,9019,3476,14472,9018,3477,3479,9021,
	18323,9033,9031,19110,9030,9016,3473,9029,
	9014,9017,9026,18325,15374,15270,10321,9690,
	9683,9702,9711,9686,3925,15142,9699,3915,
	3917,3929,9684,16850,3918,3920,3930,3919,
	14474,14490,3928,9692,9681,3926,9691,3922,
	9709,3927,9707,9695,9705,3924,9682,9708,
	9703,15943,9710,3916,18326,3931,3921,14851,
	9688,15584,9696,9700,14475,9704,3923,9687,
	9685,9697,18327,9694,9693,15621,9698,9701,
	10289,15460,4307,10299,10291,4313,4317,10308,
	10316,10301,4309,15873,10305,9689,15545,4310,
	10312,10317,10309,15029,10311,10320,10313,10295,
	10304,18109,4314,10293,10306,4312,10300,4311,
	10314,10297,10290,4315,10319,10292,10310,10307,
	10302,10288,10287,10296,10315,4316,10318,10294,
	10298,4308,18328,4703,4698,10928,10924,10900,
	4694,14057,15349,4693,4699,15456,18330,10898,
	4697,10910,4692,10927,4701,10913,10918,10912,
	11475,10908,10916,10914,10901,10895,10922,4700,
	10923,14433,10917,10906,10904,10899,10907,10920,
	10903,10919,10925,18331,4696,15522,10915,10921,
	10911,10905,4695,18332,14692,10303,10909,4702,
	10902,15026,15364,11480,4983,11479,11477,11495,
	4987,18335,11492,11484,11494,11470,11482,14439,
	4989,11462,15121,11460,11493,11491,11490,11474,
	11473,11463,4991,11469,11478,11489,4994,11496,
	5283,4996,4990,11467,11476,11459,11461,4981,
	4993,4986,11468,11483,10926,4992,11458,5278,
	11485,11488,4988,14482,11472,4995,4985,15875,
	15549,4984,11481,11487,4982,11465,11486,16725,
	17015,15098,18336,16406,15988,11471,11917,15457,
	11922,5280,5282,11928,5276,11942,11934,5289,
	11932,11916,11931,11919,11943,5288,5286,11920,
	11939,15547,11925,11938,17000,11940,11915,5281,
	18108,11941,15344,14689,11929,14483,5285,11926,
	16300,5287,11935,14494,15588,5279,11921,11936,
	14585,11930,11464,11937,11918,15397,5284,12308,
	14693,5277,11913,18337,17085,15552,15072,12305,
	12298,5440,12293,12309,12300,11927,15676,12311,
	12294,5439,12304,5441,5449,5447,12306,5448,
	12296,14764,12297,12302,12307,15025,11912,5442,
	12316,12313,12640,12303,12299,12314,12629,12301,
	5445,18338,5444,11914,5446,12310,12312,5443,
	12315,12292,12295,15504,17399,15503,15917,12638,
	12622,5600,12636,18106,12620,5601,12631,14694,
	12627,5606,12637,12621,12624,5598,12639,16609,
	12632,12635,5604,5609,5607,12628,12623,15455,
	12619,5599,14880,5597,5605,12634,5608,12630,
	12641,5610,16959,17066,14690,18890,12890,12905,
	12892,12899,12625,12897,12880,12903,5716,12884,
	12902,12891,15572,5715,15305,12907,12893,14885,
	12898,12901,12878,12904,12889,12894,12882,12886,
	12906,12896,12879,12883,12885,12895,18341,5714,
	12888,14659,14544,14499,15309,12887,13093,13097,
	5791,13099,18343,5785,19105,13095,13101,5790,
	5786,5787,13092,13098,5789,13096,5788,13091,
	12900,13094,18344,13090,13100,17010,17013,5855,
	13261,13264,13260,13255,15461,13259,13256,18346,
	13384,13386,15408,13385,13387,13383,5901,5903,
	15838,5937,13477,14206,13549,13548,13550,5964,
	13551,5963,13552,13580,13583,13582,5973,13581,
	18347,16795,5983,13624,13632,1687,17672,8353,
	9034,10929,12642,17673,1688,2081,2538,14497,
	8354,8355,3024,3482,9036,9035,3486,3481,
	9037,3483,3485,3484,3480,15317,15316,3932,
	15314,9715,9713,9714,9712,14500,4319,18349,
	4322,4324,4323,14498,15315,17301,10931,10930,
	4704,10932,10933,4705,11504,11502,11949,4997,
	11497,11505,11499,11503,11945,11948,5294,11944,
	5293,11946,5452,12319,5450,5453,5451,12908,
	12910,12909,5717,5792,13104,13103,13102,1689,
	17689,6068,6177,1044,6175,6178,6176,1319,
	6387,1316,6386,6384,1317,1315,18352,16337,
	6741,6744,1692,6743,6742,1691,1690,16189,
	1694,6745,1693,18290,7195,7191,2082,7192,
	7193,7190,2542,7727,2543,2546,7728,18270,
	3025,8358,3033,8357,8360,3029,8359,3494,
	3027,3030,3026,14508,8356,3490,9039,9043,
	16198,9042,3495,3487,3489,3493,3488,15326,
	14509,9719,3933,4325,4327,4326,10323,10935,
	14511,10934,17135,11506,5000,11951,5295,12320,
	5612,5296,1695,2547,7729,14517,8361,3034,
	3496,9044,9046,3498,3497,3937,4328,3936,
	9720,4329,10936,11507,5001,5297,12321,12325,
	5456,12322,5457,5455,12326,5458,12324,12645,
	5454,5613,1696,3036,3035,3501,9048,9047,
	3502,14852,9724,3943,3940,9723,3942,9721,
	3941,10324,4330,10939,16141,4706,10937,10938,
	4709,5005,5002,5007,11510,5003,11509,5006,
	11955,11957,14537,5298,11956,5299,11954,11953,
	16307,12328,12327,5459,12329,12650,5615,12649,
	12646,5614,5718,14528,5795,19007,13105,14530,
	13265,18355,15337,5938,13482,5940,13481,13483,
	5939,13553,19109,13606,17635,1697,18886,3944,
	14540,10325,15919,1698,14543,4710,5616,2086,
	5904,2087,8362,9049,16023,9051,9050,18123,
	9728,9725,3945,10327,10332,4331,10326,10328,
	10335,10330,10334,10331,10329,10333,4332,16576,
	10943,14548,10941,4712,4711,10942,4713,11512,
	5011,11511,11958,15355,11960,11962,11961,14549,
	5300,11959,12336,5460,12334,12337,5461,12335,
	12333,12330,5462,12331,14550,18357,12651,16179,
	12653,12652,15765,12913,16181,12914,18766,14551,
	13107,14552,5861,18358,5860,5941,13484,16175,
	5974,2088,3503,10337,10336,10945,10944,14553,
	5301,11964,11963,12340,12338,12341,12339,14554,
	13108,13485,17676,2089,14556,11514,11965,14669,
	2090,14560,4333,11515,5463,12342,5618,5796,
	13390,2091,9052,3506,3951,3949,18770,17212,
	10338,10952,4716,10947,10946,15362,10951,4714,
	15927,5018,10948,10953,10950,10949,5016,16242,
	11517,5012,11518,14565,14564,11516,5015,5013,
	5017,11971,11967,11970,5302,15766,11762,11969,
	11968,14566,5466,5464,5467,5465,19054,12660,
	5620,12658,5621,12657,12659,5619,12920,12917,
	13109,5798,14567,5797,18947,13268,13267,5862,
	16437,5905,5942,5965,13554,2092,9053,4337,
	10954,4717,16243,5303,14575,12346,18360,5469,
	12663,5622,12662,12664,12661,13112,15224,5719,
	13111,14577,14576,13110,14574,13269,13609,2093,
	15371,17679,2094,18322,14582,2548,7730,8364,
	14583,3507,3510,3508,9054,14584,3512,18362,
	3511,3954,17253,9733,9734,10956,10341,4338,
	10343,10955,4341,4718,5019,10957,4720,4719,
	10958,10961,4721,18364,5024,5022,17390,5021,
	11520,11519,11972,11521,5020,11522,11975,11979,
	5473,11978,11981,11980,11977,11976,18365,15357,
	14586,12349,15377,12348,5472,16180,12347,17407,
	12667,12666,12665,5624,16313,12927,12926,15378,
	12924,12923,5721,5720,12928,13270,5863,13114,
	5799,13113,12925,13271,5906,18366,5966,13555,
	13610,18367,17680,2095,8365,11982,2096,18124,
	10345,10344,11523,11983,16835,5474,15383,13611,
	2549,3514,3513,9735,9737,3958,9736,3957,
	3959,9738,14667,10347,10349,10348,15394,10351,
	4342,10350,10346,15735,18369,10971,10973,10963,
	10967,10964,4723,4726,4728,4727,10969,10972,
	10970,4729,16194,4725,10962,4722,4724,18939,
	14213,5026,11529,11527,11526,15205,11532,11531,
	11530,11525,5025,11524,11528,5027,11987,14591,
	11991,11988,11990,11989,11986,11992,11995,5305,
	11985,11996,12354,12357,12359,12358,12352,12356,
	12351,14592,12353,5475,12350,14594,12355,12682,
	12671,5625,5626,12670,12675,12681,12673,12680,
	15212,12677,15388,12676,12679,12674,12672,15858,
	12678,12935,12933,5723,12932,12938,12936,15390,
	5724,12931,12929,12930,5726,12934,5725,12937,
	13115,13121,14593,18371,13116,5803,13122,5802,
	13120,13119,5801,13118,5800,13117,13277,13279,
	13278,13275,5865,13272,13276,13281,5864,13391,
	5909,14595,13392,13486,5943,13584,5975,15207,
	13612,5976,13628,13627,5991,13636,17797,10353,
	17202,10974,11535,17895,4730,5028,11533,16244,
	11536,5029,15767,11997,11534,12360,5476,12683,
	12941,12939,5804,13282,5912,5866,13487,5967,
	16298,2551,7731,14600,3960,14601,10354,4345,
	10355,10976,10975,5031,10979,10978,10977,14603,
	11537,11540,11539,11538,5030,12001,11999,11998,
	16151,12000,16146,5477,12362,16160,12685,12684,
	12687,5627,12686,18372,12942,12944,12943,14605,
	13124,13127,13283,5867,5944,5968,13615,2552,
	4733,5032,12365,16153,13284,14606,13585,7732,
	5992,2553,11541,14609,12366,16128,13128,13285,
	2554,10981,4735,4734,10980,12367,12369,12368,
	5481,5480,5479,14612,5806,13129,5805,13287,
	13286,5945,13488,3041,9741,10985,10982,10989,
	18373,4737,10988,10984,10983,10987,10986,4736,
	11552,11547,11545,11554,11546,15419,11542,17393,
	11548,16921,11543,12011,11544,11555,17955,14617,
	16301,11551,5033,11550,11549,12003,12006,16214,
	12005,15422,12013,12008,12012,12010,12009,12007,
	5309,5308,5310,5307,12014,16308,12375,12378,
	12376,12380,12373,5486,12374,12372,12379,12371,
	12370,5485,5483,5482,12377,12697,12703,5630,
	12698,12701,12705,5631,12700,18374,12692,12691,
	12704,12695,12693,12702,12694,5629,5628,15297,
	12699,16087,12696,18375,12947,12950,12948,12954,
	12952,12946,12957,16186,12945,14620,5728,12953,
	12949,5727,12956,16158,18715,13132,13131,17954,
	18378,13138,13134,13137,5808,17894,13130,13133,
	5807,16162,15751,5869,13298,13293,13297,15417,
	13300,13296,13292,15742,5871,13295,13299,5870,
	13290,12951,13294,13291,15985,13288,13289,5868,
	13399,13401,13404,13400,13405,15414,13398,13396,
	15415,5913,13402,5915,5914,13397,13403,16063,
	13493,5946,13494,13492,13489,13491,13490,13556,
	13558,13557,13588,16033,13587,13586,13589,13633,
	13637,15416,17682,3042,9055,9744,3961,9742,
	4350,4348,10363,4349,10996,10999,10993,11000,
	10992,10994,10995,10998,16187,4740,11003,11002,
	4738,10997,11001,4739,11560,5034,15484,11557,
	11563,11566,11562,11559,11567,11558,15425,11564,
	5035,11556,5036,5037,11561,12023,12021,12015,
	16458,12018,12028,12022,14638,12030,12027,5312,
	12029,12016,16299,12024,18381,12385,12387,12390,
	17721,12384,16074,5487,12392,12383,12393,12389,
	12391,12395,12388,12396,12386,12382,5488,15982,
	12394,5489,5633,12017,12731,12722,5635,12718,
	5636,15276,17211,12727,12721,12711,5634,12724,
	12706,12713,12728,12725,16126,12717,14629,12729,
	12712,12707,12726,14630,12709,12723,12714,12730,
	12710,5632,12708,12962,17395,12967,12979,12970,
	12963,12959,12972,12965,12968,12961,12975,12958,
	12966,12971,12973,12976,12960,15435,12974,12978,
	12969,16176,13154,13157,13156,13161,13159,5810,
	13150,13140,13139,5812,13149,13141,18384,13146,
	12977,17394,13142,5811,13148,14633,13147,13143,
	13155,17785,13144,13305,13158,13160,13151,13145,
	13306,13308,13302,5872,14622,13315,13313,5873,
	13312,13311,13304,13301,13307,13316,13303,13309,
	13414,13419,13409,13412,5916,13416,13423,13310,
	13410,13420,13424,13415,13408,13418,13417,13422,
	13421,13314,13413,13407,13411,13406,18385,13505,
	13496,13504,13495,13498,13506,13500,13499,13497,
	13503,17790,13559,17786,13560,14632,13562,13561,
	13564,13563,13591,13590,13616,13617,15428,13629,
	5989,5993,16260,13638,5994,17683,5729,13162,
	17400,3044,9745,16143,3962,11004,16246,16114,
	11568,12034,12033,5313,12398,12035,12397,14641,
	12733,5637,5639,12732,14642,16316,5638,16167,
	12982,12984,12983,13163,5813,14643,5917,13507,
	13565,14644,13642,3045,18756,10364,16245,4741,
	14646,16018,17396,11572,11571,14647,12036,18388,
	5640,5730,13317,13634,3046,4351,18389,4742,
	15736,12734,14649,13425,3515,17684,18390,17783,
	12037,5969,3516,4743,5314,13426,3517,11005,
	5041,12038,5318,5315,12399,5490,12986,12985,
	12988,12987,5731,13166,13165,5814,13318,18393,
	5918,13566,13592,5986,18394,9056,11575,12039,
	12735,9746,18382,12040,12401,12400,14656,13429,
	5951,14657,13567,13593,11576,3964,5491,12402,
	14661,13168,5815,13169,13167,13430,12043,12042,
	12041,12403,16272,17952,12407,12406,12405,12404,
	5492,12736,12990,5732,12989,13170,13321,13320,
	5874,13319,13433,13432,13431,14663,11577,5319,
	13322,13434,18395,14666,13509,13568,13618,13644,
	4353,5320,12408,12739,13171,13435,18755,4745,
	16256,12044,12409,12995,16163,12991,5816,12994,
	12993,5733,12992,5735,18105,5734,13173,13172,
	14672,5876,13323,5875,13437,13436,5953,5952,
	13569,13572,13571,13570,13643,18048,5042,14674,
	5500,12996,13174,5877,13324,19104,19096,13645,
	17799,17990,12045,13594,16240,18852,18893,18900,
	18932,18934,18936,18941,18963,14005,14007,19039,
	19064,19126,19142,19146,19150,19155,16816,14149,
	18536,14423,17862,17075,18505,15154,17789,17681,
	16381,15851,18510,14437,14879,15509,14780,15624,
	15310,15030,16432,14828,18151,15861,14754,15244,
	17030,15066,15539,15960,14976,15704,15511,15346,
	14350,16630,17316,16812,16486,14979,18455,17031,
	14655,15200,18516,18451,14711,14392,16510,18488,
	14558,17291,16113,16564,14768,17176,14715,14492,
	17038,16562,18415,18411,15039,16957,14929,18943,
	14966,15235,18727,16012,19048,17163,17228,17024,
	15275,18462,14930,15268,16688,18652,18537,16950,
	18559,18477,15193,18434,14944,18566,17294,15629,
	14737,18416,15662,16594,17919,14589,18569,17003,
	15685,18111,18433,18527,14251,18548,16765,14920,
	14870,15587,16016,14616,18914,15100,15449,14770,
	18074,18489,18589,15860,18547,17853,16855,18456,
	14990,16721,18469,14908,14996,14822,16021,18435,
	16535,18767,15804,17634,19062,18651,18418,14611,
	18889,16590,14159,15661,16622,14173,16706,15578,
	18579,14808,15579,14710,16934,17636,16356,17688,
	14229,18436,14926,14223,14002,16044,16905,14668,
	18771,18419,14142,16101,18580,14816,14320,14815,
	18666,15341,18396,18437,16956,17242,14247,16455,
	15782,19075,14935,14478,18629,17251,15957,17076,
	18649,15527,15645,14901,15237,14954,14343,18677,
	14814,18924,15893,16669,17850,16757,18457,17057,
	15892,18710,15906,14523,18632,15338,18973,14598,
	17278,16566,18420,18085,14727,18479,15648,15563,
	14520,14696,16697,15849,17902,14707,17174,18439,
	15577,17877,15854,18663,14170,17201,14230,18975,
	17167,17296,18549,17268,17849,17913,18093,17623,
	17300,17620,14000,19145,17617,15842,15693,18228,
	17861,16132,17616,18611,18985,17298,16698,17194,
	16216,18141,17185,17910,18506,18400,16561,15239,
	14965,14133,18706,15363,16841,18421,19093,15601,
	15639,18607,15402,15899,17653,17198,14945,18600,
	18422,15432,18749,15112,16906,15391,18159,15095,
	15433,17847,16834,16732,19055,18511,17241,18674,
	14260,18720,15548,15424,15559,15321,16916,16687,
	18097,18534,14978,15427,15669,14590,15080,14936,
	18870,18481,16943,18127,14428,18954,17158,17628,
	17108,14753,14416,18601,16645,17107,15016,18540,
	16425,14526,15191,15542,15061,16777,15864,16294,
	15832,14164,15844,14201,14747,16392,16889,14627,
	14986,14378,16926,15881,17155,14912,15215,16135,
	15691,16421,16879,16623,17165,18582,18440,17056,
	16448,18861,14481,18741,16920,16413,17791,15846,
	14631,18201,14636,14455,18602,17878,14445,18716,
	14614,16974,18522,15979,16808,15472,16404,15558,
	15202,18742,16967,17717,19019,17196,14868,16395,
	18884,17876,15393,18658,17851,16551,14938,16621,
	18855,17872,18583,16607,19037,14410,16491,14406,
	16970,18644,16897,18729,14279,18617,18104,18147,
	14572,15185,15465,16768,19010,18604,17071,19097,
	18119,18524,18709,18712,17126,14393,14733,18525,
	18575,18471,18297,17127,18613,15412,14458,14695,
	19059,15920,16964,16136,17899,16689,18670,16543,
	17285,14677,14569,15544,18446,17942,14175,18695,
	15874,14562,18952,17183,16067,14278,17116,14258,
	16933,17937,16771,15122,14337,16749,16451,14971,
	18684,17078,15930,14961,15345,14425,15520,15145,
	15871,17048,16449,15089,15242,14263,15823,14624,
	18472,14634,14732,14270,18484,14804,17855,15973,
	14708,18618,17254,16886,14740,18700,17225,17854,
	16927,14660,16533,16069,19047,14293,16628,16037,
	15230,18101,15903,17235,16637,16414,14776,14625,
	15488,14136,15575,15211,15120,18558,14924,18584,
	18608,14339,16454,14969,15431,15480,19065,18905,
	18701,18483,17047,15405,18609,14361,18942,14907,
	15773,19148,14678,16911,15241,16913,14413,14925,
	18541,18044,16714,19058,17269,18779,15094,15084,
	17240,15471,15797,15788,17802,18627,16659,16861,
	18560,14287,15194,17803,15876,16142,14408,18402,
	18673,17940,16407,14480,18744,15031,16979,18571,
	15453,14444,16397,14628,14812,14360,15689,17042,
	14237,18892,14362,16405,15540,16826,17781,15796,
	15949,19099,15075,15056,14542,18116,15331,17896,
	17180,14869,18493,16830,17204,15423,15834,17788,
	15210,14680,16677,15260,18552,15521,17049,14886,
	15670,15502,14917,18572,15781,19152,16423,14607,
	17263,18426,17016,16291,18999,16122,15916,17918,
	17287,17264,18682,18573,14459,14787,17141,14761,
	16781,18427,16653,17905,17675,17914,15790,18498,
	18782,15437,16512,18920,18139,18403,18512,14275,
	15551,18982,16079,15599,18142,18447,15561,18585,
	14931,14662,17117,17144,14832,14635,16838,15102,
	17005,19043,16802,17044,15325,14376,17053,14236,
	17004,16801,18615,14587,16736,15005,15885,14921,
	18705,18453,15421,14505,17039,15923,16944,14571,
	16954,16829,14734,15105,17297,16450,14637,16658,
	18554,18643,16790,14688,16678,14333,16603,16865,
	18773,16615,14301,14313,14184,14691,18454,15372,
	14859,14906,14241,17274,15494,15060,16254,17082,
	15306,17873,15694,17197,15092,16583,15604,14441,
	16730,14682,15228,14257,16751,18997,18642,16761,
	17193,15779,15964,16878,18150,18726,15161,18133,
	19101,15059,15184,14448,18998,15329,17250,15328,
	15369,18970,16733,17658,18757,18647,16723,19022,
	15632,16935,18431,14521,18508,14785,16824,18404,
	15047,17906,14312,17915,15332,17920,15771,17290,
	14398,17933,14504,18561,18869,15586,15188,15556,
	15253,15642,15271,16422,18562,19137,14471,14809,
	19100,14794,17286,18563,17866,16618,15681,18624,
	15535,17930,14871,18694,17189,15119,15109,16202,
	17887,16553,15146,18086,15785,17272,16515,15159,
	14818,16000,16735,19090,14440,16415,15055,15678,
	17360,14729,15172,18497,17200,14725,15792,18707,
	18675,17868,18590,17874,16508,18676,16085,14960,
	15647,18529,15158,18641,15634,19013,16424,14757,
	15361,16276,16034,14784,15590,14640,18542,16104,
	18145,18397,16647,18499,18405,18880,16285,14840,
	18096,14379,17939,14456,15458,18981,14783,15574,
	17128,17001,18408,15568,17893,14477,15117,17207,
	18458,14984,15686,18605,15028,15087,18409,17931,
	15550,17265,14212,18671,14525,18733,14993,16758,
	14580,14125,14937,14297,16631,14512,14186,18428,
	14453,18071,14503,17081,17223,16895,16549,16487,
	14402,15912,14964,14516,14994,14484,18714,14487,
	18443,14488,15140,14463,17043,15327,16503,14238,
	14534,15201,18885,18501,18084,15709,15157,14245,
	15710,18704,15878,18620,16058,15683,17904,15510,
	17907,14736,14974,14903,18626,15707,15399,14882,
	16881,15698,16417,14602,18664,18082,18574,14479,
	18081,18502,14819,18616,16139,18877,14862,18653,
	18909,16832,14763,14234,18460,18958,18495,16344,
	15837,18490,18703,17279,15292,16942,14943,14972,
	18995,14596,15358,18468,18783,18636,18430,18735,
	17888,16106,15898,15347,17870,18698,17232,17859,
	18610,18503,15067,17723,18445,14388,17151,16717,
	14221,16338,14915,15673,17947,16032,16665,14847,
	15776,18596,16430,14681,15809,14719,17660,17243,
	15368,14215,18987,15812,18983,16542,16900,18555,
	17364,19000,15631,16595,15526,17191,15370,16998,
	15385,14220,18639,16193,14199,15148,18692,14861,
	16779,16703,16520,14266,14927,14717,16937,18444,
	15190,18781,16263,16427,18107,15036,16429,16490,
	15411,15935,16946,15256,15255,18515,18530,18588,
	14827,17867,16787,16530,17073,18429,14311,15491,
	14286,15638,18546,18640,15827,15259,18556,16767,
	18144,19057,17113,18398,15938,16643,15001,15538,
	14760,17875,15175,15883,16627,15666,16416,17184,
	15223,18776,17912,17936,16764,14623,18080,19005,
	14999,15635,15695,15348,18959,18410,15023,15625,
	17633,15512,18380,18731,18448,18599,14849,15164,
	18449,16874,17219,14777,18412,18487,18568,16893,
	17293,17208,15010,14599,15643,18466,15258,15296,
	14684,15663,16624,16090,14759,15438,16066,14127,
	15839,15062,16495,15780,15633,14303,16852,15553,
	17621,17007,15701,14129,18657,15027,15562,14485,
	18681,14491,18683,15608,15333,14705,19008,16972,
	15640,15613,15257,18650,15830,14712,15197,14197,
	16061,15566,15644,18414,16626,18719,15002,16445,
	16385,18768,3193,18773,6333,3261,3278,18778,
	18779,17055,3716,4181,14164,3381,14296,18083,
	18085,14309,10178,18088,14368,18089,18090,4940,
	16377,18093,14420,2510,3015,14466,18098,18103,
	14633,18104,284,283,18116,7430,10657,18118,
	17354,18154,15709,18155,17356,15710,18156,17402,
	18157,15713,18161,15714,18162,18178,17361,15720,
	18195,15721,18196,15722,18203,17367,17404,15723,
	18206,15724,18207,17368,15727,18241,15730,18256,
	17349,18257,17350,15734,18308,17323,18309,17383,
	18310,17385,15736,18341,15737,17327,18359,15739,
	17389,18364,15740,17390,18365,17407,15741,18373,
	17393,17395,18384,17394,17400,17396,15744,18399,
	15745,18400,15746,18401,17335,15748,18419,15749,
	15753,18432,15754,15755,17328,15763,18474,15764,
	18475,15765,18476,12112,15773,5925,13037,15776,
	18496,18500,15785,15795,18509,15803,18513,15804,
	18514,17360,18515,18516,15807,18517,15808,15812,
	7188,15813,18522,15814,18523,15815,15816,18527,
	16890,18528,4841,18534,11438,18538,18539,15835,
	18540,15836,6756,18543,15844,18544,18545,15850,
	15851,18548,5124,18549,18550,8996,18551,15857,
	13418,18552,15858,18553,8849,18556,15866,18557,
	18563,15875,18564,18567,6023,2399,18571,15903,
	7833,15938,18583,17384,18593,15944,18594,15945,
	15946,18598,9665,17353,15948,18607,15949,17397,
	15950,18635,15956,18636,15961,18641,15962,18642,
	15963,15968,1510,18652,15972,16054,18666,16055,
	16056,18673,16057,18677,16060,18678,16061,18679,
	1832,16069,18683,16075,18687,18688,16078,18701,
	16092,18710,16124,6150,18720,16212,18721,3970,
	18722,18735,15049,17723,17347,17388,17777,17387,
	17778,17308,17815,17310,17816,17311,17313,17820,
	17315,17214,17850,17851,17219,17852,17221,17853,
	17855,17245,17856,17247,17860,17371,17294,17886,
	17295,17887,17296,17888,17297,17889,17303,17896,
	17304,17348,17339,17312,17341,17344,17981,17345,
	17337,17325,628,9089,14104,14103,14106,14105,
	109,122,13743,13745,108,14052,174,259,
	264,175,14051,176,208,100,209,103,
	257,106,105,214,216,215,107,265,
	13758,258,13759,13748,197,132,120,133,
	228,14049,196,14050,260,
};

pdf_cmap pdf_cmap_UniCNS_UTF16_H =
{
	-1, "UniCNS-UTF16-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 4, 0xdc00, 0xdfff },
		{ 2, 0xe000, 0xffff },
	},
	18469, 18469, (pdf_range*) pdf_cmap_UniCNS_UTF16_H_ranges,
	15173, 15173, (unsigned short*) pdf_cmap_UniCNS_UTF16_H_table,
};

/* UniCNS-UTF16-V */

static const pdf_range pdf_cmap_UniCNS_UTF16_V_ranges[] =
{
	{ 0x2013, 0x2014, PDF_CMAP_TABLE, 0 },
	{ 0x2025, 0x2025, PDF_CMAP_SINGLE, 109 },
	{ 0x3008, 0x3009, PDF_CMAP_RANGE, 150 },
	{ 0x300a, 0x300b, PDF_CMAP_RANGE, 146 },
	{ 0x300c, 0x300d, PDF_CMAP_RANGE, 154 },
	{ 0x300e, 0x300f, PDF_CMAP_RANGE, 158 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 142 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 138 },
	{ 0xfe4f, 0xfe4f, PDF_CMAP_SINGLE, 13745 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 130 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 134 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 135 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 135 },
};

static const unsigned short pdf_cmap_UniCNS_UTF16_V_table[2] =
{
	120,122,
};

pdf_cmap pdf_cmap_UniCNS_UTF16_V =
{
	-1, "UniCNS-UTF16-V", "UniCNS-UTF16-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	13, 13, (pdf_range*) pdf_cmap_UniCNS_UTF16_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_UniCNS_UTF16_V_table,
};

