#include "fitz.h"
#include "mupdf.h"

/* Adobe-GB1-0 */

static const pdf_range pdf_cmap_Adobe_GB1_0_ranges[] =
{
	{ 0x0000, 0x1e24, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1e24, PDF_CMAP_RANGE, 7680 },
};

static const unsigned short pdf_cmap_Adobe_GB1_0_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_GB1_0 =
{
	-1, "Adobe-GB1-0", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x1eff },
	},
	31, 31, (pdf_range*) pdf_cmap_Adobe_GB1_0_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_GB1_0_table,
};

/* Adobe-GB1-1 */

static const pdf_range pdf_cmap_Adobe_GB1_1_ranges[] =
{
	{ 0x0000, 0x26a8, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26a8, PDF_CMAP_RANGE, 9728 },
};

static const unsigned short pdf_cmap_Adobe_GB1_1_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_GB1_1 =
{
	-1, "Adobe-GB1-1", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x26ff },
	},
	39, 39, (pdf_range*) pdf_cmap_Adobe_GB1_1_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_GB1_1_table,
};

/* Adobe-GB1-2 */

static const pdf_range pdf_cmap_Adobe_GB1_2_ranges[] =
{
	{ 0x0000, 0x566e, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4aff, PDF_CMAP_RANGE, 18944 },
	{ 0x4b00, 0x4bff, PDF_CMAP_RANGE, 19200 },
	{ 0x4c00, 0x4cff, PDF_CMAP_RANGE, 19456 },
	{ 0x4d00, 0x4dff, PDF_CMAP_RANGE, 19712 },
	{ 0x4e00, 0x4eff, PDF_CMAP_RANGE, 19968 },
	{ 0x4f00, 0x4fff, PDF_CMAP_RANGE, 20224 },
	{ 0x5000, 0x50ff, PDF_CMAP_RANGE, 20480 },
	{ 0x5100, 0x51ff, PDF_CMAP_RANGE, 20736 },
	{ 0x5200, 0x52ff, PDF_CMAP_RANGE, 20992 },
	{ 0x5300, 0x53ff, PDF_CMAP_RANGE, 21248 },
	{ 0x5400, 0x54ff, PDF_CMAP_RANGE, 21504 },
	{ 0x5500, 0x55ff, PDF_CMAP_RANGE, 21760 },
	{ 0x5600, 0x566e, PDF_CMAP_RANGE, 22016 },
};

static const unsigned short pdf_cmap_Adobe_GB1_2_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_GB1_2 =
{
	-1, "Adobe-GB1-2", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x56ff },
	},
	87, 87, (pdf_range*) pdf_cmap_Adobe_GB1_2_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_GB1_2_table,
};

/* Adobe-GB1-3 */

static const pdf_range pdf_cmap_Adobe_GB1_3_ranges[] =
{
	{ 0x0000, 0x5750, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4aff, PDF_CMAP_RANGE, 18944 },
	{ 0x4b00, 0x4bff, PDF_CMAP_RANGE, 19200 },
	{ 0x4c00, 0x4cff, PDF_CMAP_RANGE, 19456 },
	{ 0x4d00, 0x4dff, PDF_CMAP_RANGE, 19712 },
	{ 0x4e00, 0x4eff, PDF_CMAP_RANGE, 19968 },
	{ 0x4f00, 0x4fff, PDF_CMAP_RANGE, 20224 },
	{ 0x5000, 0x50ff, PDF_CMAP_RANGE, 20480 },
	{ 0x5100, 0x51ff, PDF_CMAP_RANGE, 20736 },
	{ 0x5200, 0x52ff, PDF_CMAP_RANGE, 20992 },
	{ 0x5300, 0x53ff, PDF_CMAP_RANGE, 21248 },
	{ 0x5400, 0x54ff, PDF_CMAP_RANGE, 21504 },
	{ 0x5500, 0x55ff, PDF_CMAP_RANGE, 21760 },
	{ 0x5600, 0x56ff, PDF_CMAP_RANGE, 22016 },
	{ 0x5700, 0x5750, PDF_CMAP_RANGE, 22272 },
};

static const unsigned short pdf_cmap_Adobe_GB1_3_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_GB1_3 =
{
	-1, "Adobe-GB1-3", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x57ff },
	},
	88, 88, (pdf_range*) pdf_cmap_Adobe_GB1_3_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_GB1_3_table,
};

/* Adobe-GB1-4 */

static const pdf_range pdf_cmap_Adobe_GB1_4_ranges[] =
{
	{ 0x0000, 0x7187, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4aff, PDF_CMAP_RANGE, 18944 },
	{ 0x4b00, 0x4bff, PDF_CMAP_RANGE, 19200 },
	{ 0x4c00, 0x4cff, PDF_CMAP_RANGE, 19456 },
	{ 0x4d00, 0x4dff, PDF_CMAP_RANGE, 19712 },
	{ 0x4e00, 0x4eff, PDF_CMAP_RANGE, 19968 },
	{ 0x4f00, 0x4fff, PDF_CMAP_RANGE, 20224 },
	{ 0x5000, 0x50ff, PDF_CMAP_RANGE, 20480 },
	{ 0x5100, 0x51ff, PDF_CMAP_RANGE, 20736 },
	{ 0x5200, 0x52ff, PDF_CMAP_RANGE, 20992 },
	{ 0x5300, 0x53ff, PDF_CMAP_RANGE, 21248 },
	{ 0x5400, 0x54ff, PDF_CMAP_RANGE, 21504 },
	{ 0x5500, 0x55ff, PDF_CMAP_RANGE, 21760 },
	{ 0x5600, 0x56ff, PDF_CMAP_RANGE, 22016 },
	{ 0x5700, 0x57ff, PDF_CMAP_RANGE, 22272 },
	{ 0x5800, 0x58ff, PDF_CMAP_RANGE, 22528 },
	{ 0x5900, 0x59ff, PDF_CMAP_RANGE, 22784 },
	{ 0x5a00, 0x5aff, PDF_CMAP_RANGE, 23040 },
	{ 0x5b00, 0x5bff, PDF_CMAP_RANGE, 23296 },
	{ 0x5c00, 0x5cff, PDF_CMAP_RANGE, 23552 },
	{ 0x5d00, 0x5dff, PDF_CMAP_RANGE, 23808 },
	{ 0x5e00, 0x5eff, PDF_CMAP_RANGE, 24064 },
	{ 0x5f00, 0x5fff, PDF_CMAP_RANGE, 24320 },
	{ 0x6000, 0x60ff, PDF_CMAP_RANGE, 24576 },
	{ 0x6100, 0x61ff, PDF_CMAP_RANGE, 24832 },
	{ 0x6200, 0x62ff, PDF_CMAP_RANGE, 25088 },
	{ 0x6300, 0x63ff, PDF_CMAP_RANGE, 25344 },
	{ 0x6400, 0x64ff, PDF_CMAP_RANGE, 25600 },
	{ 0x6500, 0x65ff, PDF_CMAP_RANGE, 25856 },
	{ 0x6600, 0x66ff, PDF_CMAP_RANGE, 26112 },
	{ 0x6700, 0x67ff, PDF_CMAP_RANGE, 26368 },
	{ 0x6800, 0x68ff, PDF_CMAP_RANGE, 26624 },
	{ 0x6900, 0x69ff, PDF_CMAP_RANGE, 26880 },
	{ 0x6a00, 0x6aff, PDF_CMAP_RANGE, 27136 },
	{ 0x6b00, 0x6bff, PDF_CMAP_RANGE, 27392 },
	{ 0x6c00, 0x6cff, PDF_CMAP_RANGE, 27648 },
	{ 0x6d00, 0x6dff, PDF_CMAP_RANGE, 27904 },
	{ 0x6e00, 0x6eff, PDF_CMAP_RANGE, 28160 },
	{ 0x6f00, 0x6fff, PDF_CMAP_RANGE, 28416 },
	{ 0x7000, 0x70ff, PDF_CMAP_RANGE, 28672 },
	{ 0x7100, 0x7187, PDF_CMAP_RANGE, 28928 },
};

static const unsigned short pdf_cmap_Adobe_GB1_4_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_GB1_4 =
{
	-1, "Adobe-GB1-4", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x71ff },
	},
	114, 114, (pdf_range*) pdf_cmap_Adobe_GB1_4_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_GB1_4_table,
};

/* Adobe-GB1-5 */

static const pdf_range pdf_cmap_Adobe_GB1_5_ranges[] =
{
	{ 0x0000, 0x764b, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4aff, PDF_CMAP_RANGE, 18944 },
	{ 0x4b00, 0x4bff, PDF_CMAP_RANGE, 19200 },
	{ 0x4c00, 0x4cff, PDF_CMAP_RANGE, 19456 },
	{ 0x4d00, 0x4dff, PDF_CMAP_RANGE, 19712 },
	{ 0x4e00, 0x4eff, PDF_CMAP_RANGE, 19968 },
	{ 0x4f00, 0x4fff, PDF_CMAP_RANGE, 20224 },
	{ 0x5000, 0x50ff, PDF_CMAP_RANGE, 20480 },
	{ 0x5100, 0x51ff, PDF_CMAP_RANGE, 20736 },
	{ 0x5200, 0x52ff, PDF_CMAP_RANGE, 20992 },
	{ 0x5300, 0x53ff, PDF_CMAP_RANGE, 21248 },
	{ 0x5400, 0x54ff, PDF_CMAP_RANGE, 21504 },
	{ 0x5500, 0x55ff, PDF_CMAP_RANGE, 21760 },
	{ 0x5600, 0x56ff, PDF_CMAP_RANGE, 22016 },
	{ 0x5700, 0x57ff, PDF_CMAP_RANGE, 22272 },
	{ 0x5800, 0x58ff, PDF_CMAP_RANGE, 22528 },
	{ 0x5900, 0x59ff, PDF_CMAP_RANGE, 22784 },
	{ 0x5a00, 0x5aff, PDF_CMAP_RANGE, 23040 },
	{ 0x5b00, 0x5bff, PDF_CMAP_RANGE, 23296 },
	{ 0x5c00, 0x5cff, PDF_CMAP_RANGE, 23552 },
	{ 0x5d00, 0x5dff, PDF_CMAP_RANGE, 23808 },
	{ 0x5e00, 0x5eff, PDF_CMAP_RANGE, 24064 },
	{ 0x5f00, 0x5fff, PDF_CMAP_RANGE, 24320 },
	{ 0x6000, 0x60ff, PDF_CMAP_RANGE, 24576 },
	{ 0x6100, 0x61ff, PDF_CMAP_RANGE, 24832 },
	{ 0x6200, 0x62ff, PDF_CMAP_RANGE, 25088 },
	{ 0x6300, 0x63ff, PDF_CMAP_RANGE, 25344 },
	{ 0x6400, 0x64ff, PDF_CMAP_RANGE, 25600 },
	{ 0x6500, 0x65ff, PDF_CMAP_RANGE, 25856 },
	{ 0x6600, 0x66ff, PDF_CMAP_RANGE, 26112 },
	{ 0x6700, 0x67ff, PDF_CMAP_RANGE, 26368 },
	{ 0x6800, 0x68ff, PDF_CMAP_RANGE, 26624 },
	{ 0x6900, 0x69ff, PDF_CMAP_RANGE, 26880 },
	{ 0x6a00, 0x6aff, PDF_CMAP_RANGE, 27136 },
	{ 0x6b00, 0x6bff, PDF_CMAP_RANGE, 27392 },
	{ 0x6c00, 0x6cff, PDF_CMAP_RANGE, 27648 },
	{ 0x6d00, 0x6dff, PDF_CMAP_RANGE, 27904 },
	{ 0x6e00, 0x6eff, PDF_CMAP_RANGE, 28160 },
	{ 0x6f00, 0x6fff, PDF_CMAP_RANGE, 28416 },
	{ 0x7000, 0x70ff, PDF_CMAP_RANGE, 28672 },
	{ 0x7100, 0x71ff, PDF_CMAP_RANGE, 28928 },
	{ 0x7200, 0x72ff, PDF_CMAP_RANGE, 29184 },
	{ 0x7300, 0x73ff, PDF_CMAP_RANGE, 29440 },
	{ 0x7400, 0x74ff, PDF_CMAP_RANGE, 29696 },
	{ 0x7500, 0x75ff, PDF_CMAP_RANGE, 29952 },
	{ 0x7600, 0x764b, PDF_CMAP_RANGE, 30208 },
};

static const unsigned short pdf_cmap_Adobe_GB1_5_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_GB1_5 =
{
	-1, "Adobe-GB1-5", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x76ff },
	},
	119, 119, (pdf_range*) pdf_cmap_Adobe_GB1_5_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_GB1_5_table,
};

/* GB-EUC-H */

static const pdf_range pdf_cmap_GB_EUC_H_ranges[] =
{
	{ 0x0020, 0x0020, PDF_CMAP_SINGLE, 7716 },
	{ 0x0021, 0x007e, PDF_CMAP_RANGE, 814 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 940 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1034 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1128 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1222 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1316 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1410 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1504 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1598 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1692 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1786 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 1880 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 1974 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2068 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2162 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2256 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2350 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2444 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2538 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2632 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2726 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2820 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 2914 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3008 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3102 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3196 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 3290 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3384 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 3478 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3572 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 3666 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3760 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 3854 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 3948 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 4042 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4136 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4230 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 4324 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 4418 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd7a1, 0xd7f9, PDF_CMAP_RANGE, 4606 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4695 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4789 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4883 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 4977 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5071 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
};

static const unsigned short pdf_cmap_GB_EUC_H_table[1] = { 0 };

pdf_cmap pdf_cmap_GB_EUC_H =
{
	-1, "GB-EUC-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa1a1, 0xfefe },
	},
	90, 90, (pdf_range*) pdf_cmap_GB_EUC_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_GB_EUC_H_table,
};

/* GB-EUC-V */

static const pdf_range pdf_cmap_GB_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa3fd, 0xa3fd, PDF_CMAP_SINGLE, 597 },
	{ 0xa3fe, 0xa3fe, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GB_EUC_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GB_EUC_V =
{
	-1, "GB-EUC-V", "GB-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GB_EUC_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GB_EUC_V_table,
};

/* GB-H */

static const pdf_range pdf_cmap_GB_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 96 },
	{ 0x2231, 0x2262, PDF_CMAP_RANGE, 190 },
	{ 0x2265, 0x226e, PDF_CMAP_RANGE, 240 },
	{ 0x2271, 0x227c, PDF_CMAP_RANGE, 250 },
	{ 0x2321, 0x237e, PDF_CMAP_RANGE, 262 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 356 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 439 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 525 },
	{ 0x2641, 0x2675, PDF_CMAP_RANGE, 549 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 602 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 635 },
	{ 0x2821, 0x2840, PDF_CMAP_RANGE, 668 },
	{ 0x2845, 0x286a, PDF_CMAP_RANGE, 700 },
	{ 0x2924, 0x296f, PDF_CMAP_RANGE, 738 },
	{ 0x2a21, 0x2a7e, PDF_CMAP_RANGE, 814 },
	{ 0x2b21, 0x2b40, PDF_CMAP_RANGE, 908 },
	{ 0x3021, 0x307e, PDF_CMAP_RANGE, 940 },
	{ 0x3121, 0x317e, PDF_CMAP_RANGE, 1034 },
	{ 0x3221, 0x327e, PDF_CMAP_RANGE, 1128 },
	{ 0x3321, 0x337e, PDF_CMAP_RANGE, 1222 },
	{ 0x3421, 0x347e, PDF_CMAP_RANGE, 1316 },
	{ 0x3521, 0x357e, PDF_CMAP_RANGE, 1410 },
	{ 0x3621, 0x367e, PDF_CMAP_RANGE, 1504 },
	{ 0x3721, 0x377e, PDF_CMAP_RANGE, 1598 },
	{ 0x3821, 0x387e, PDF_CMAP_RANGE, 1692 },
	{ 0x3921, 0x397e, PDF_CMAP_RANGE, 1786 },
	{ 0x3a21, 0x3a7e, PDF_CMAP_RANGE, 1880 },
	{ 0x3b21, 0x3b7e, PDF_CMAP_RANGE, 1974 },
	{ 0x3c21, 0x3c7e, PDF_CMAP_RANGE, 2068 },
	{ 0x3d21, 0x3d7e, PDF_CMAP_RANGE, 2162 },
	{ 0x3e21, 0x3e7e, PDF_CMAP_RANGE, 2256 },
	{ 0x3f21, 0x3f7e, PDF_CMAP_RANGE, 2350 },
	{ 0x4021, 0x407e, PDF_CMAP_RANGE, 2444 },
	{ 0x4121, 0x417e, PDF_CMAP_RANGE, 2538 },
	{ 0x4221, 0x427e, PDF_CMAP_RANGE, 2632 },
	{ 0x4321, 0x437e, PDF_CMAP_RANGE, 2726 },
	{ 0x4421, 0x447e, PDF_CMAP_RANGE, 2820 },
	{ 0x4521, 0x457e, PDF_CMAP_RANGE, 2914 },
	{ 0x4621, 0x467e, PDF_CMAP_RANGE, 3008 },
	{ 0x4721, 0x477e, PDF_CMAP_RANGE, 3102 },
	{ 0x4821, 0x487e, PDF_CMAP_RANGE, 3196 },
	{ 0x4921, 0x497e, PDF_CMAP_RANGE, 3290 },
	{ 0x4a21, 0x4a7e, PDF_CMAP_RANGE, 3384 },
	{ 0x4b21, 0x4b7e, PDF_CMAP_RANGE, 3478 },
	{ 0x4c21, 0x4c7e, PDF_CMAP_RANGE, 3572 },
	{ 0x4d21, 0x4d7e, PDF_CMAP_RANGE, 3666 },
	{ 0x4e21, 0x4e7e, PDF_CMAP_RANGE, 3760 },
	{ 0x4f21, 0x4f7e, PDF_CMAP_RANGE, 3854 },
	{ 0x5021, 0x507e, PDF_CMAP_RANGE, 3948 },
	{ 0x5121, 0x517e, PDF_CMAP_RANGE, 4042 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 4136 },
	{ 0x5321, 0x537e, PDF_CMAP_RANGE, 4230 },
	{ 0x5421, 0x547e, PDF_CMAP_RANGE, 4324 },
	{ 0x5521, 0x557e, PDF_CMAP_RANGE, 4418 },
	{ 0x5621, 0x567e, PDF_CMAP_RANGE, 4512 },
	{ 0x5721, 0x5779, PDF_CMAP_RANGE, 4606 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 4695 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 4789 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 4883 },
	{ 0x5b21, 0x5b7e, PDF_CMAP_RANGE, 4977 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5071 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 5165 },
	{ 0x5e21, 0x5e7e, PDF_CMAP_RANGE, 5259 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 5353 },
	{ 0x6021, 0x607e, PDF_CMAP_RANGE, 5447 },
	{ 0x6121, 0x617e, PDF_CMAP_RANGE, 5541 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 5635 },
	{ 0x6321, 0x637e, PDF_CMAP_RANGE, 5729 },
	{ 0x6421, 0x647e, PDF_CMAP_RANGE, 5823 },
	{ 0x6521, 0x657e, PDF_CMAP_RANGE, 5917 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 6011 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 6105 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 6199 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 6293 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 6387 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 6481 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 6575 },
	{ 0x6d21, 0x6d7e, PDF_CMAP_RANGE, 6669 },
	{ 0x6e21, 0x6e7e, PDF_CMAP_RANGE, 6763 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 6857 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 6951 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7045 },
	{ 0x7221, 0x727e, PDF_CMAP_RANGE, 7139 },
	{ 0x7321, 0x737e, PDF_CMAP_RANGE, 7233 },
	{ 0x7421, 0x747e, PDF_CMAP_RANGE, 7327 },
	{ 0x7521, 0x757e, PDF_CMAP_RANGE, 7421 },
	{ 0x7621, 0x767e, PDF_CMAP_RANGE, 7515 },
	{ 0x7721, 0x777e, PDF_CMAP_RANGE, 7609 },
};

static const unsigned short pdf_cmap_GB_H_table[1] = { 0 };

pdf_cmap pdf_cmap_GB_H =
{
	-1, "GB-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	88, 88, (pdf_range*) pdf_cmap_GB_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_GB_H_table,
};

/* GB-V */

static const pdf_range pdf_cmap_GB_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_TABLE, 0 },
	{ 0x212a, 0x212a, PDF_CMAP_SINGLE, 598 },
	{ 0x212b, 0x212c, PDF_CMAP_RANGE, 7704 },
	{ 0x212d, 0x212d, PDF_CMAP_SINGLE, 599 },
	{ 0x2132, 0x213f, PDF_CMAP_RANGE, 582 },
	{ 0x217e, 0x217e, PDF_CMAP_SINGLE, 7706 },
	{ 0x2321, 0x2321, PDF_CMAP_SINGLE, 578 },
	{ 0x2328, 0x2329, PDF_CMAP_RANGE, 580 },
	{ 0x232c, 0x232c, PDF_CMAP_SINGLE, 573 },
	{ 0x232e, 0x232e, PDF_CMAP_SINGLE, 7707 },
	{ 0x233a, 0x233b, PDF_CMAP_RANGE, 576 },
	{ 0x233d, 0x233d, PDF_CMAP_SINGLE, 7708 },
	{ 0x233f, 0x233f, PDF_CMAP_SINGLE, 579 },
	{ 0x235b, 0x235b, PDF_CMAP_SINGLE, 7709 },
	{ 0x235d, 0x235d, PDF_CMAP_SINGLE, 7710 },
	{ 0x235f, 0x235f, PDF_CMAP_SINGLE, 600 },
	{ 0x237b, 0x237b, PDF_CMAP_SINGLE, 596 },
	{ 0x237d, 0x237e, PDF_CMAP_TABLE, 2 },
	{ 0x237d, 0x237d, PDF_CMAP_SINGLE, 597 },
	{ 0x237e, 0x237e, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GB_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GB_V =
{
	-1, "GB-V", "GB-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GB_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GB_V_table,
};

/* GBK-EUC-H */

static const pdf_range pdf_cmap_GBK_EUC_H_ranges[] =
{
	{ 0x0020, 0x0020, PDF_CMAP_SINGLE, 7716 },
	{ 0x0021, 0x007e, PDF_CMAP_RANGE, 814 },
	{ 0x8140, 0x8178, PDF_CMAP_RANGE, 10072 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 8281 },
	{ 0x817a, 0x817e, PDF_CMAP_RANGE, 10129 },
	{ 0x8180, 0x8185, PDF_CMAP_RANGE, 10134 },
	{ 0x8186, 0x8186, PDF_CMAP_SINGLE, 8689 },
	{ 0x8187, 0x81ec, PDF_CMAP_RANGE, 10140 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 8178 },
	{ 0x81ee, 0x81f5, PDF_CMAP_RANGE, 10242 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 8285 },
	{ 0x81f7, 0x81fe, PDF_CMAP_RANGE, 10250 },
	{ 0x8240, 0x8252, PDF_CMAP_RANGE, 10258 },
	{ 0x8253, 0x8253, PDF_CMAP_SINGLE, 9884 },
	{ 0x8254, 0x8261, PDF_CMAP_RANGE, 10277 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 8629 },
	{ 0x8263, 0x8273, PDF_CMAP_RANGE, 10291 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 8908 },
	{ 0x8275, 0x8279, PDF_CMAP_RANGE, 10308 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 8214 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 10313 },
	{ 0x827d, 0x827e, PDF_CMAP_TABLE, 0 },
	{ 0x8280, 0x8280, PDF_CMAP_SINGLE, 7968 },
	{ 0x8281, 0x8282, PDF_CMAP_RANGE, 10316 },
	{ 0x8283, 0x8283, PDF_CMAP_SINGLE, 8319 },
	{ 0x8284, 0x828f, PDF_CMAP_RANGE, 10318 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 8284 },
	{ 0x8291, 0x82a4, PDF_CMAP_RANGE, 10330 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 8596 },
	{ 0x82a6, 0x82c7, PDF_CMAP_RANGE, 10350 },
	{ 0x82c8, 0x82c9, PDF_CMAP_TABLE, 2 },
	{ 0x82ca, 0x82e0, PDF_CMAP_RANGE, 10384 },
	{ 0x82e1, 0x82e4, PDF_CMAP_TABLE, 4 },
	{ 0x82e5, 0x82ec, PDF_CMAP_RANGE, 10408 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 9855 },
	{ 0x82ee, 0x82f1, PDF_CMAP_RANGE, 10416 },
	{ 0x82f2, 0x82f2, PDF_CMAP_SINGLE, 8750 },
	{ 0x82f3, 0x82f6, PDF_CMAP_RANGE, 10420 },
	{ 0x82f7, 0x82fb, PDF_CMAP_TABLE, 8 },
	{ 0x82fc, 0x82fe, PDF_CMAP_RANGE, 10425 },
	{ 0x8340, 0x8341, PDF_CMAP_TABLE, 13 },
	{ 0x8342, 0x8344, PDF_CMAP_RANGE, 10429 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 8916 },
	{ 0x8346, 0x8347, PDF_CMAP_RANGE, 10432 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 8121 },
	{ 0x8349, 0x834b, PDF_CMAP_RANGE, 10434 },
	{ 0x834c, 0x834c, PDF_CMAP_SINGLE, 8920 },
	{ 0x834d, 0x8352, PDF_CMAP_RANGE, 10437 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 8415 },
	{ 0x8354, 0x8356, PDF_CMAP_RANGE, 10443 },
	{ 0x8357, 0x8357, PDF_CMAP_SINGLE, 8386 },
	{ 0x8358, 0x835d, PDF_CMAP_RANGE, 10446 },
	{ 0x835e, 0x835e, PDF_CMAP_SINGLE, 8597 },
	{ 0x835f, 0x8364, PDF_CMAP_RANGE, 10452 },
	{ 0x8365, 0x8366, PDF_CMAP_TABLE, 15 },
	{ 0x8367, 0x8371, PDF_CMAP_RANGE, 10458 },
	{ 0x8372, 0x8372, PDF_CMAP_SINGLE, 8065 },
	{ 0x8373, 0x8377, PDF_CMAP_RANGE, 10469 },
	{ 0x8378, 0x837e, PDF_CMAP_TABLE, 17 },
	{ 0x8380, 0x8380, PDF_CMAP_SINGLE, 8080 },
	{ 0x8381, 0x8385, PDF_CMAP_RANGE, 10477 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 8918 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 10482 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 24 },
	{ 0x838b, 0x838c, PDF_CMAP_RANGE, 10484 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 9857 },
	{ 0x838e, 0x8393, PDF_CMAP_RANGE, 10486 },
	{ 0x8394, 0x8394, PDF_CMAP_SINGLE, 7798 },
	{ 0x8395, 0x839d, PDF_CMAP_RANGE, 10492 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 8753 },
	{ 0x839f, 0x83a5, PDF_CMAP_RANGE, 10501 },
	{ 0x83a6, 0x83a6, PDF_CMAP_SINGLE, 7827 },
	{ 0x83a7, 0x83aa, PDF_CMAP_RANGE, 10508 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 8914 },
	{ 0x83ac, 0x83ad, PDF_CMAP_RANGE, 10512 },
	{ 0x83ae, 0x83b0, PDF_CMAP_TABLE, 26 },
	{ 0x83b1, 0x83b9, PDF_CMAP_RANGE, 10514 },
	{ 0x83ba, 0x83ba, PDF_CMAP_SINGLE, 7909 },
	{ 0x83bb, 0x83c8, PDF_CMAP_RANGE, 10523 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 8229 },
	{ 0x83ca, 0x83f5, PDF_CMAP_RANGE, 10537 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 7887 },
	{ 0x83f7, 0x83fe, PDF_CMAP_RANGE, 10581 },
	{ 0x8440, 0x844f, PDF_CMAP_RANGE, 10589 },
	{ 0x8450, 0x8450, PDF_CMAP_SINGLE, 8153 },
	{ 0x8451, 0x8470, PDF_CMAP_RANGE, 10605 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 8904 },
	{ 0x8472, 0x8473, PDF_CMAP_RANGE, 10637 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 8803 },
	{ 0x8475, 0x8476, PDF_CMAP_RANGE, 10639 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 9859 },
	{ 0x8478, 0x847e, PDF_CMAP_RANGE, 10641 },
	{ 0x8480, 0x8481, PDF_CMAP_RANGE, 10648 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 7959 },
	{ 0x8483, 0x848d, PDF_CMAP_RANGE, 10650 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 7979 },
	{ 0x848f, 0x8491, PDF_CMAP_RANGE, 10661 },
	{ 0x8492, 0x8493, PDF_CMAP_TABLE, 29 },
	{ 0x8494, 0x849c, PDF_CMAP_RANGE, 10664 },
	{ 0x849d, 0x849d, PDF_CMAP_SINGLE, 8015 },
	{ 0x849e, 0x84a0, PDF_CMAP_RANGE, 10673 },
	{ 0x84a1, 0x84a6, PDF_CMAP_TABLE, 31 },
	{ 0x84a7, 0x84a8, PDF_CMAP_RANGE, 10677 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 8053 },
	{ 0x84aa, 0x84c4, PDF_CMAP_RANGE, 10679 },
	{ 0x84c5, 0x84c5, PDF_CMAP_SINGLE, 8126 },
	{ 0x84c6, 0x84d2, PDF_CMAP_RANGE, 10706 },
	{ 0x84d3, 0x84da, PDF_CMAP_TABLE, 37 },
	{ 0x84db, 0x84dc, PDF_CMAP_RANGE, 10722 },
	{ 0x84dd, 0x84dd, PDF_CMAP_SINGLE, 8498 },
	{ 0x84de, 0x84e9, PDF_CMAP_RANGE, 10724 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 8995 },
	{ 0x84eb, 0x84ed, PDF_CMAP_RANGE, 10736 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 8209 },
	{ 0x84ef, 0x84f0, PDF_CMAP_RANGE, 10739 },
	{ 0x84f1, 0x84f1, PDF_CMAP_SINGLE, 8437 },
	{ 0x84f2, 0x84fe, PDF_CMAP_RANGE, 10741 },
	{ 0x8540, 0x8550, PDF_CMAP_RANGE, 10754 },
	{ 0x8551, 0x8554, PDF_CMAP_TABLE, 45 },
	{ 0x8555, 0x855d, PDF_CMAP_RANGE, 10772 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 8431 },
	{ 0x855f, 0x8565, PDF_CMAP_RANGE, 10781 },
	{ 0x8566, 0x8566, PDF_CMAP_SINGLE, 8656 },
	{ 0x8567, 0x857e, PDF_CMAP_RANGE, 10788 },
	{ 0x8580, 0x8586, PDF_CMAP_RANGE, 10812 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 8897 },
	{ 0x8588, 0x858a, PDF_CMAP_RANGE, 10819 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 7780 },
	{ 0x858c, 0x8591, PDF_CMAP_RANGE, 10822 },
	{ 0x8592, 0x8592, PDF_CMAP_SINGLE, 8697 },
	{ 0x8593, 0x8595, PDF_CMAP_RANGE, 10828 },
	{ 0x8596, 0x8598, PDF_CMAP_TABLE, 49 },
	{ 0x8599, 0x85a1, PDF_CMAP_RANGE, 10832 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 7770 },
	{ 0x85a3, 0x85b1, PDF_CMAP_RANGE, 10841 },
	{ 0x85b2, 0x85b2, PDF_CMAP_SINGLE, 7842 },
	{ 0x85b3, 0x85fe, PDF_CMAP_RANGE, 10856 },
	{ 0x8640, 0x8649, PDF_CMAP_RANGE, 10932 },
	{ 0x864a, 0x864a, PDF_CMAP_SINGLE, 9069 },
	{ 0x864b, 0x8653, PDF_CMAP_RANGE, 10942 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 8775 },
	{ 0x8655, 0x8667, PDF_CMAP_RANGE, 10951 },
	{ 0x8668, 0x8668, PDF_CMAP_SINGLE, 9068 },
	{ 0x8669, 0x867e, PDF_CMAP_RANGE, 10970 },
	{ 0x8680, 0x8695, PDF_CMAP_RANGE, 10992 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 8604 },
	{ 0x8697, 0x8698, PDF_CMAP_RANGE, 11014 },
	{ 0x8699, 0x8699, PDF_CMAP_SINGLE, 8393 },
	{ 0x869a, 0x86a0, PDF_CMAP_RANGE, 11016 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 8688 },
	{ 0x86a2, 0x86c9, PDF_CMAP_RANGE, 11023 },
	{ 0x86ca, 0x86ce, PDF_CMAP_TABLE, 52 },
	{ 0x86cf, 0x86d0, PDF_CMAP_RANGE, 11065 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 8748 },
	{ 0x86d2, 0x86db, PDF_CMAP_RANGE, 11067 },
	{ 0x86dc, 0x86dd, PDF_CMAP_TABLE, 57 },
	{ 0x86de, 0x86e0, PDF_CMAP_RANGE, 11077 },
	{ 0x86e1, 0x86e1, PDF_CMAP_SINGLE, 8303 },
	{ 0x86e2, 0x86e7, PDF_CMAP_RANGE, 11080 },
	{ 0x86e8, 0x86e8, PDF_CMAP_SINGLE, 8609 },
	{ 0x86e9, 0x86ed, PDF_CMAP_RANGE, 11086 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 9081 },
	{ 0x86ef, 0x86f3, PDF_CMAP_RANGE, 11091 },
	{ 0x86f4, 0x86f4, PDF_CMAP_SINGLE, 9074 },
	{ 0x86f5, 0x86fe, PDF_CMAP_RANGE, 11096 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 8554 },
	{ 0x8741, 0x8743, PDF_CMAP_RANGE, 11106 },
	{ 0x8744, 0x8744, PDF_CMAP_SINGLE, 9084 },
	{ 0x8745, 0x8748, PDF_CMAP_RANGE, 11109 },
	{ 0x8749, 0x874c, PDF_CMAP_TABLE, 59 },
	{ 0x874d, 0x874e, PDF_CMAP_RANGE, 11114 },
	{ 0x874f, 0x874f, PDF_CMAP_SINGLE, 9079 },
	{ 0x8750, 0x8756, PDF_CMAP_RANGE, 11116 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 8012 },
	{ 0x8758, 0x8759, PDF_CMAP_RANGE, 11123 },
	{ 0x875a, 0x8760, PDF_CMAP_TABLE, 63 },
	{ 0x8761, 0x8765, PDF_CMAP_RANGE, 11127 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 9844 },
	{ 0x8767, 0x8779, PDF_CMAP_RANGE, 11132 },
	{ 0x877a, 0x877a, PDF_CMAP_SINGLE, 9071 },
	{ 0x877b, 0x877c, PDF_CMAP_RANGE, 11151 },
	{ 0x877d, 0x877e, PDF_CMAP_TABLE, 70 },
	{ 0x8780, 0x8782, PDF_CMAP_TABLE, 72 },
	{ 0x8783, 0x8785, PDF_CMAP_RANGE, 11155 },
	{ 0x8786, 0x878a, PDF_CMAP_TABLE, 75 },
	{ 0x878b, 0x878c, PDF_CMAP_RANGE, 11160 },
	{ 0x878d, 0x878e, PDF_CMAP_TABLE, 80 },
	{ 0x878f, 0x8792, PDF_CMAP_RANGE, 11162 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 9070 },
	{ 0x8794, 0x8797, PDF_CMAP_RANGE, 11166 },
	{ 0x8798, 0x8798, PDF_CMAP_SINGLE, 8631 },
	{ 0x8799, 0x879c, PDF_CMAP_RANGE, 11170 },
	{ 0x879d, 0x879d, PDF_CMAP_SINGLE, 9077 },
	{ 0x879e, 0x87a2, PDF_CMAP_RANGE, 11174 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 9090 },
	{ 0x87a4, 0x87a6, PDF_CMAP_RANGE, 11179 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 8350 },
	{ 0x87a8, 0x87b2, PDF_CMAP_RANGE, 11182 },
	{ 0x87b3, 0x87b5, PDF_CMAP_TABLE, 82 },
	{ 0x87b6, 0x87ba, PDF_CMAP_RANGE, 11194 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 9887 },
	{ 0x87bc, 0x87be, PDF_CMAP_RANGE, 11199 },
	{ 0x87bf, 0x87c2, PDF_CMAP_TABLE, 85 },
	{ 0x87c3, 0x87c9, PDF_CMAP_RANGE, 11203 },
	{ 0x87ca, 0x87cc, PDF_CMAP_TABLE, 89 },
	{ 0x87cd, 0x87ce, PDF_CMAP_RANGE, 11210 },
	{ 0x87cf, 0x87cf, PDF_CMAP_SINGLE, 8923 },
	{ 0x87d0, 0x87d1, PDF_CMAP_RANGE, 11212 },
	{ 0x87d2, 0x87d5, PDF_CMAP_TABLE, 92 },
	{ 0x87d6, 0x87d9, PDF_CMAP_RANGE, 11215 },
	{ 0x87da, 0x87da, PDF_CMAP_SINGLE, 8865 },
	{ 0x87db, 0x87f6, PDF_CMAP_RANGE, 11219 },
	{ 0x87f7, 0x87fa, PDF_CMAP_TABLE, 96 },
	{ 0x87fb, 0x87fe, PDF_CMAP_RANGE, 11248 },
	{ 0x8840, 0x8841, PDF_CMAP_TABLE, 100 },
	{ 0x8842, 0x8843, PDF_CMAP_RANGE, 11252 },
	{ 0x8844, 0x8846, PDF_CMAP_TABLE, 102 },
	{ 0x8847, 0x887e, PDF_CMAP_RANGE, 11255 },
	{ 0x8880, 0x88b9, PDF_CMAP_RANGE, 11311 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 9002 },
	{ 0x88bb, 0x88cb, PDF_CMAP_RANGE, 11369 },
	{ 0x88cc, 0x88cc, PDF_CMAP_SINGLE, 8843 },
	{ 0x88cd, 0x88d3, PDF_CMAP_RANGE, 11386 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 8069 },
	{ 0x88d5, 0x88d6, PDF_CMAP_RANGE, 11393 },
	{ 0x88d7, 0x88d7, PDF_CMAP_SINGLE, 8997 },
	{ 0x88d8, 0x88de, PDF_CMAP_RANGE, 11395 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 9004 },
	{ 0x88e0, 0x88e4, PDF_CMAP_RANGE, 11402 },
	{ 0x88e5, 0x88e5, PDF_CMAP_SINGLE, 9006 },
	{ 0x88e6, 0x88f1, PDF_CMAP_RANGE, 11407 },
	{ 0x88f2, 0x88f3, PDF_CMAP_TABLE, 105 },
	{ 0x88f4, 0x88f5, PDF_CMAP_RANGE, 11419 },
	{ 0x88f6, 0x88f6, PDF_CMAP_SINGLE, 7795 },
	{ 0x88f7, 0x88fe, PDF_CMAP_RANGE, 11421 },
	{ 0x8940, 0x894a, PDF_CMAP_RANGE, 11429 },
	{ 0x894b, 0x8950, PDF_CMAP_TABLE, 107 },
	{ 0x8951, 0x8953, PDF_CMAP_RANGE, 11442 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 8575 },
	{ 0x8955, 0x895c, PDF_CMAP_RANGE, 11445 },
	{ 0x895d, 0x895f, PDF_CMAP_TABLE, 113 },
	{ 0x8960, 0x896c, PDF_CMAP_RANGE, 11454 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 7805 },
	{ 0x896e, 0x8970, PDF_CMAP_RANGE, 11467 },
	{ 0x8971, 0x8971, PDF_CMAP_SINGLE, 8406 },
	{ 0x8972, 0x897b, PDF_CMAP_RANGE, 11470 },
	{ 0x897c, 0x897c, PDF_CMAP_SINGLE, 7874 },
	{ 0x897d, 0x897e, PDF_CMAP_RANGE, 11480 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 11482 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8882 },
	{ 0x898c, 0x8998, PDF_CMAP_RANGE, 11493 },
	{ 0x8999, 0x8999, PDF_CMAP_SINGLE, 7903 },
	{ 0x899a, 0x899d, PDF_CMAP_RANGE, 11506 },
	{ 0x899e, 0x899e, PDF_CMAP_SINGLE, 7929 },
	{ 0x899f, 0x89a5, PDF_CMAP_RANGE, 11510 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 116 },
	{ 0x89a9, 0x89ae, PDF_CMAP_RANGE, 11518 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 8551 },
	{ 0x89b0, 0x89b9, PDF_CMAP_RANGE, 11524 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8685 },
	{ 0x89bb, 0x89bd, PDF_CMAP_RANGE, 11534 },
	{ 0x89be, 0x89c0, PDF_CMAP_TABLE, 119 },
	{ 0x89c1, 0x89c3, PDF_CMAP_RANGE, 11537 },
	{ 0x89c4, 0x89c8, PDF_CMAP_TABLE, 122 },
	{ 0x89c9, 0x89cd, PDF_CMAP_RANGE, 11541 },
	{ 0x89ce, 0x89ce, PDF_CMAP_SINGLE, 7723 },
	{ 0x89cf, 0x89d0, PDF_CMAP_RANGE, 11546 },
	{ 0x89d1, 0x89d1, PDF_CMAP_SINGLE, 8878 },
	{ 0x89d2, 0x89d7, PDF_CMAP_RANGE, 11548 },
	{ 0x89d8, 0x89d8, PDF_CMAP_SINGLE, 8009 },
	{ 0x89d9, 0x89da, PDF_CMAP_RANGE, 11554 },
	{ 0x89db, 0x89db, PDF_CMAP_SINGLE, 8504 },
	{ 0x89dc, 0x89f3, PDF_CMAP_RANGE, 11556 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 8321 },
	{ 0x89f5, 0x89fe, PDF_CMAP_RANGE, 11580 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 127 },
	{ 0x8a42, 0x8a58, PDF_CMAP_RANGE, 11591 },
	{ 0x8a59, 0x8a5e, PDF_CMAP_TABLE, 129 },
	{ 0x8a5f, 0x8a78, PDF_CMAP_RANGE, 11616 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8877 },
	{ 0x8a7a, 0x8a7e, PDF_CMAP_RANGE, 11642 },
	{ 0x8a80, 0x8ae3, PDF_CMAP_RANGE, 11647 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 8255 },
	{ 0x8ae5, 0x8afe, PDF_CMAP_RANGE, 11747 },
	{ 0x8b40, 0x8b43, PDF_CMAP_RANGE, 11773 },
	{ 0x8b44, 0x8b44, PDF_CMAP_SINGLE, 7950 },
	{ 0x8b45, 0x8b48, PDF_CMAP_RANGE, 11777 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 9209 },
	{ 0x8b4a, 0x8b79, PDF_CMAP_RANGE, 11781 },
	{ 0x8b7a, 0x8b7a, PDF_CMAP_SINGLE, 9212 },
	{ 0x8b7b, 0x8b7e, PDF_CMAP_RANGE, 11829 },
	{ 0x8b80, 0x8b8b, PDF_CMAP_RANGE, 11833 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 8297 },
	{ 0x8b8d, 0x8b9d, PDF_CMAP_RANGE, 11845 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 9207 },
	{ 0x8b9f, 0x8bb2, PDF_CMAP_RANGE, 11862 },
	{ 0x8bb3, 0x8bb3, PDF_CMAP_SINGLE, 9206 },
	{ 0x8bb4, 0x8bb8, PDF_CMAP_RANGE, 11882 },
	{ 0x8bb9, 0x8bb9, PDF_CMAP_SINGLE, 9213 },
	{ 0x8bba, 0x8bbd, PDF_CMAP_RANGE, 11887 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 9208 },
	{ 0x8bbf, 0x8bc5, PDF_CMAP_RANGE, 11891 },
	{ 0x8bc6, 0x8bc9, PDF_CMAP_TABLE, 135 },
	{ 0x8bca, 0x8bd3, PDF_CMAP_RANGE, 11899 },
	{ 0x8bd4, 0x8bd4, PDF_CMAP_SINGLE, 9217 },
	{ 0x8bd5, 0x8bdb, PDF_CMAP_RANGE, 11909 },
	{ 0x8bdc, 0x8bdc, PDF_CMAP_SINGLE, 9215 },
	{ 0x8bdd, 0x8be4, PDF_CMAP_RANGE, 11916 },
	{ 0x8be5, 0x8be5, PDF_CMAP_SINGLE, 9216 },
	{ 0x8be6, 0x8bea, PDF_CMAP_RANGE, 11924 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 8737 },
	{ 0x8bec, 0x8bef, PDF_CMAP_RANGE, 11929 },
	{ 0x8bf0, 0x8bf0, PDF_CMAP_SINGLE, 8482 },
	{ 0x8bf1, 0x8bfe, PDF_CMAP_RANGE, 11933 },
	{ 0x8c40, 0x8c43, PDF_CMAP_RANGE, 11947 },
	{ 0x8c44, 0x8c44, PDF_CMAP_SINGLE, 9211 },
	{ 0x8c45, 0x8c4e, PDF_CMAP_RANGE, 11951 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 8538 },
	{ 0x8c50, 0x8c56, PDF_CMAP_RANGE, 11961 },
	{ 0x8c57, 0x8c57, PDF_CMAP_SINGLE, 8677 },
	{ 0x8c58, 0x8c5b, PDF_CMAP_RANGE, 11968 },
	{ 0x8c5c, 0x8c5c, PDF_CMAP_SINGLE, 8279 },
	{ 0x8c5d, 0x8c7e, PDF_CMAP_RANGE, 11972 },
	{ 0x8c80, 0x8c8a, PDF_CMAP_RANGE, 12006 },
	{ 0x8c8b, 0x8c92, PDF_CMAP_TABLE, 139 },
	{ 0x8c93, 0x8c98, PDF_CMAP_RANGE, 12019 },
	{ 0x8c99, 0x8c9a, PDF_CMAP_TABLE, 147 },
	{ 0x8c9b, 0x8ca1, PDF_CMAP_RANGE, 12025 },
	{ 0x8ca2, 0x8ca7, PDF_CMAP_TABLE, 149 },
	{ 0x8ca8, 0x8cbf, PDF_CMAP_RANGE, 12033 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 9052 },
	{ 0x8cc1, 0x8cd1, PDF_CMAP_RANGE, 12057 },
	{ 0x8cd2, 0x8cd5, PDF_CMAP_TABLE, 155 },
	{ 0x8cd6, 0x8cd8, PDF_CMAP_RANGE, 12075 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 8510 },
	{ 0x8cda, 0x8cf8, PDF_CMAP_RANGE, 12078 },
	{ 0x8cf9, 0x8cf9, PDF_CMAP_SINGLE, 7958 },
	{ 0x8cfa, 0x8cfe, PDF_CMAP_RANGE, 12109 },
	{ 0x8d40, 0x8d72, PDF_CMAP_RANGE, 12114 },
	{ 0x8d73, 0x8d75, PDF_CMAP_TABLE, 159 },
	{ 0x8d76, 0x8d7a, PDF_CMAP_RANGE, 12166 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 8628 },
	{ 0x8d7c, 0x8d7e, PDF_CMAP_RANGE, 12171 },
	{ 0x8d80, 0x8d87, PDF_CMAP_RANGE, 12174 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 9103 },
	{ 0x8d89, 0x8d8e, PDF_CMAP_RANGE, 12182 },
	{ 0x8d8f, 0x8d8f, PDF_CMAP_SINGLE, 7962 },
	{ 0x8d90, 0x8d9d, PDF_CMAP_RANGE, 12188 },
	{ 0x8d9e, 0x8d9e, PDF_CMAP_SINGLE, 9099 },
	{ 0x8d9f, 0x8db8, PDF_CMAP_RANGE, 12202 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 9098 },
	{ 0x8dba, 0x8de1, PDF_CMAP_RANGE, 12228 },
	{ 0x8de2, 0x8de4, PDF_CMAP_TABLE, 162 },
	{ 0x8de5, 0x8de6, PDF_CMAP_RANGE, 12269 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 9096 },
	{ 0x8de8, 0x8df6, PDF_CMAP_RANGE, 12271 },
	{ 0x8df7, 0x8df7, PDF_CMAP_SINGLE, 9102 },
	{ 0x8df8, 0x8dfd, PDF_CMAP_RANGE, 12286 },
	{ 0x8dfe, 0x8dfe, PDF_CMAP_SINGLE, 9101 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 12292 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 9100 },
	{ 0x8e47, 0x8e55, PDF_CMAP_RANGE, 12298 },
	{ 0x8e56, 0x8e5a, PDF_CMAP_TABLE, 165 },
	{ 0x8e5b, 0x8e67, PDF_CMAP_RANGE, 12315 },
	{ 0x8e68, 0x8e68, PDF_CMAP_SINGLE, 8169 },
	{ 0x8e69, 0x8e6d, PDF_CMAP_RANGE, 12328 },
	{ 0x8e6e, 0x8e70, PDF_CMAP_TABLE, 170 },
	{ 0x8e71, 0x8e7e, PDF_CMAP_RANGE, 12334 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 8996 },
	{ 0x8e81, 0x8e9a, PDF_CMAP_RANGE, 12348 },
	{ 0x8e9b, 0x8e9b, PDF_CMAP_SINGLE, 8515 },
	{ 0x8e9c, 0x8e9e, PDF_CMAP_RANGE, 12374 },
	{ 0x8e9f, 0x8e9f, PDF_CMAP_SINGLE, 8489 },
	{ 0x8ea0, 0x8ea3, PDF_CMAP_RANGE, 12377 },
	{ 0x8ea4, 0x8ea4, PDF_CMAP_SINGLE, 8823 },
	{ 0x8ea5, 0x8ea6, PDF_CMAP_RANGE, 12381 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 7847 },
	{ 0x8ea8, 0x8eab, PDF_CMAP_RANGE, 12383 },
	{ 0x8eac, 0x8eae, PDF_CMAP_TABLE, 173 },
	{ 0x8eaf, 0x8ebc, PDF_CMAP_RANGE, 12388 },
	{ 0x8ebd, 0x8ebe, PDF_CMAP_TABLE, 176 },
	{ 0x8ebf, 0x8ec2, PDF_CMAP_RANGE, 12402 },
	{ 0x8ec3, 0x8ec5, PDF_CMAP_TABLE, 178 },
	{ 0x8ec6, 0x8ecc, PDF_CMAP_RANGE, 12407 },
	{ 0x8ecd, 0x8ece, PDF_CMAP_TABLE, 181 },
	{ 0x8ecf, 0x8ed5, PDF_CMAP_RANGE, 12414 },
	{ 0x8ed6, 0x8ed7, PDF_CMAP_TABLE, 183 },
	{ 0x8ed8, 0x8eeb, PDF_CMAP_RANGE, 12421 },
	{ 0x8eec, 0x8eec, PDF_CMAP_SINGLE, 8160 },
	{ 0x8eed, 0x8efe, PDF_CMAP_RANGE, 12441 },
	{ 0x8f40, 0x8f51, PDF_CMAP_RANGE, 12459 },
	{ 0x8f52, 0x8f56, PDF_CMAP_TABLE, 185 },
	{ 0x8f57, 0x8f5c, PDF_CMAP_RANGE, 12477 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 8261 },
	{ 0x8f5e, 0x8f63, PDF_CMAP_RANGE, 12483 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 8568 },
	{ 0x8f65, 0x8f7e, PDF_CMAP_RANGE, 12489 },
	{ 0x8f80, 0x8f85, PDF_CMAP_RANGE, 12515 },
	{ 0x8f86, 0x8f88, PDF_CMAP_TABLE, 190 },
	{ 0x8f89, 0x8f94, PDF_CMAP_RANGE, 12522 },
	{ 0x8f95, 0x8f97, PDF_CMAP_TABLE, 193 },
	{ 0x8f98, 0x8f9a, PDF_CMAP_RANGE, 12535 },
	{ 0x8f9b, 0x8f9d, PDF_CMAP_TABLE, 196 },
	{ 0x8f9e, 0x8fa0, PDF_CMAP_RANGE, 12539 },
	{ 0x8fa1, 0x8fa1, PDF_CMAP_SINGLE, 9852 },
	{ 0x8fa2, 0x8fbc, PDF_CMAP_RANGE, 12542 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 8133 },
	{ 0x8fbe, 0x8fc3, PDF_CMAP_RANGE, 12569 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 199 },
	{ 0x8fc7, 0x8fcc, PDF_CMAP_RANGE, 12576 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 7947 },
	{ 0x8fce, 0x8fd7, PDF_CMAP_RANGE, 12582 },
	{ 0x8fd8, 0x8fd8, PDF_CMAP_SINGLE, 7804 },
	{ 0x8fd9, 0x8ffe, PDF_CMAP_RANGE, 12592 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 12630 },
	{ 0x9080, 0x909c, PDF_CMAP_RANGE, 12693 },
	{ 0x909d, 0x909e, PDF_CMAP_TABLE, 202 },
	{ 0x909f, 0x90b9, PDF_CMAP_RANGE, 12722 },
	{ 0x90ba, 0x90ba, PDF_CMAP_SINGLE, 7907 },
	{ 0x90bb, 0x90bf, PDF_CMAP_RANGE, 12749 },
	{ 0x90c0, 0x90c1, PDF_CMAP_TABLE, 204 },
	{ 0x90c2, 0x90c4, PDF_CMAP_RANGE, 12754 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 9145 },
	{ 0x90c6, 0x90da, PDF_CMAP_RANGE, 12757 },
	{ 0x90db, 0x90dc, PDF_CMAP_TABLE, 206 },
	{ 0x90dd, 0x90ec, PDF_CMAP_RANGE, 12778 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 9141 },
	{ 0x90ee, 0x90ef, PDF_CMAP_RANGE, 12794 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 9146 },
	{ 0x90f1, 0x90f6, PDF_CMAP_RANGE, 12796 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 9139 },
	{ 0x90f8, 0x90fe, PDF_CMAP_RANGE, 12802 },
	{ 0x9140, 0x9141, PDF_CMAP_RANGE, 12809 },
	{ 0x9142, 0x9142, PDF_CMAP_SINGLE, 8546 },
	{ 0x9143, 0x914a, PDF_CMAP_RANGE, 12811 },
	{ 0x914b, 0x914d, PDF_CMAP_TABLE, 208 },
	{ 0x914e, 0x9150, PDF_CMAP_RANGE, 12820 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 9143 },
	{ 0x9152, 0x9153, PDF_CMAP_RANGE, 12823 },
	{ 0x9154, 0x9155, PDF_CMAP_TABLE, 211 },
	{ 0x9156, 0x9158, PDF_CMAP_RANGE, 12825 },
	{ 0x9159, 0x915a, PDF_CMAP_TABLE, 213 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 12828 },
	{ 0x915d, 0x915d, PDF_CMAP_SINGLE, 8274 },
	{ 0x915e, 0x9160, PDF_CMAP_RANGE, 12830 },
	{ 0x9161, 0x9163, PDF_CMAP_TABLE, 215 },
	{ 0x9164, 0x916d, PDF_CMAP_RANGE, 12834 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 8754 },
	{ 0x916f, 0x9175, PDF_CMAP_RANGE, 12844 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 7743 },
	{ 0x9177, 0x9179, PDF_CMAP_RANGE, 12851 },
	{ 0x917a, 0x917c, PDF_CMAP_TABLE, 218 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 12854 },
	{ 0x9180, 0x9183, PDF_CMAP_RANGE, 12856 },
	{ 0x9184, 0x9184, PDF_CMAP_SINGLE, 7854 },
	{ 0x9185, 0x918c, PDF_CMAP_RANGE, 12860 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 7931 },
	{ 0x918e, 0x9190, PDF_CMAP_RANGE, 12868 },
	{ 0x9191, 0x9193, PDF_CMAP_TABLE, 221 },
	{ 0x9194, 0x9196, PDF_CMAP_RANGE, 12872 },
	{ 0x9197, 0x9197, PDF_CMAP_SINGLE, 8644 },
	{ 0x9198, 0x919a, PDF_CMAP_RANGE, 12875 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 8724 },
	{ 0x919c, 0x91a8, PDF_CMAP_RANGE, 12878 },
	{ 0x91a9, 0x91ab, PDF_CMAP_TABLE, 224 },
	{ 0x91ac, 0x91b9, PDF_CMAP_RANGE, 12891 },
	{ 0x91ba, 0x91bb, PDF_CMAP_TABLE, 227 },
	{ 0x91bc, 0x91be, PDF_CMAP_RANGE, 12905 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 9440 },
	{ 0x91c0, 0x91c2, PDF_CMAP_RANGE, 12908 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 9144 },
	{ 0x91c4, 0x91cc, PDF_CMAP_RANGE, 12911 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 7809 },
	{ 0x91ce, 0x91cf, PDF_CMAP_RANGE, 12920 },
	{ 0x91d0, 0x91d9, PDF_CMAP_TABLE, 229 },
	{ 0x91da, 0x91de, PDF_CMAP_RANGE, 12925 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 9441 },
	{ 0x91e0, 0x91e1, PDF_CMAP_RANGE, 12930 },
	{ 0x91e2, 0x91e2, PDF_CMAP_SINGLE, 9379 },
	{ 0x91e3, 0x91e9, PDF_CMAP_RANGE, 12932 },
	{ 0x91ea, 0x91ea, PDF_CMAP_SINGLE, 9380 },
	{ 0x91eb, 0x91ef, PDF_CMAP_RANGE, 12939 },
	{ 0x91f0, 0x91f2, PDF_CMAP_TABLE, 239 },
	{ 0x91f3, 0x91fe, PDF_CMAP_RANGE, 12945 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 12957 },
	{ 0x9280, 0x92b5, PDF_CMAP_RANGE, 13020 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 8657 },
	{ 0x92b7, 0x92cd, PDF_CMAP_RANGE, 13074 },
	{ 0x92ce, 0x92d0, PDF_CMAP_TABLE, 242 },
	{ 0x92d1, 0x92d3, PDF_CMAP_RANGE, 13098 },
	{ 0x92d4, 0x92d4, PDF_CMAP_SINGLE, 9858 },
	{ 0x92d5, 0x92de, PDF_CMAP_RANGE, 13101 },
	{ 0x92df, 0x92e0, PDF_CMAP_TABLE, 245 },
	{ 0x92e1, 0x92fd, PDF_CMAP_RANGE, 13111 },
	{ 0x92fe, 0x92fe, PDF_CMAP_SINGLE, 8077 },
	{ 0x9340, 0x934f, PDF_CMAP_RANGE, 13140 },
	{ 0x9350, 0x9350, PDF_CMAP_SINGLE, 8703 },
	{ 0x9351, 0x935c, PDF_CMAP_RANGE, 13156 },
	{ 0x935d, 0x935d, PDF_CMAP_SINGLE, 8024 },
	{ 0x935e, 0x936f, PDF_CMAP_RANGE, 13168 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 8539 },
	{ 0x9371, 0x9375, PDF_CMAP_RANGE, 13186 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 7862 },
	{ 0x9377, 0x937e, PDF_CMAP_RANGE, 13191 },
	{ 0x9380, 0x938b, PDF_CMAP_RANGE, 13199 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 8411 },
	{ 0x938d, 0x939c, PDF_CMAP_RANGE, 13211 },
	{ 0x939d, 0x939d, PDF_CMAP_SINGLE, 9056 },
	{ 0x939e, 0x93a4, PDF_CMAP_RANGE, 13227 },
	{ 0x93a5, 0x93a7, PDF_CMAP_TABLE, 247 },
	{ 0x93a8, 0x93b3, PDF_CMAP_RANGE, 13235 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 8846 },
	{ 0x93b5, 0x93b7, PDF_CMAP_RANGE, 13247 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 8159 },
	{ 0x93b9, 0x93ba, PDF_CMAP_RANGE, 13250 },
	{ 0x93bb, 0x93bd, PDF_CMAP_TABLE, 250 },
	{ 0x93be, 0x93c5, PDF_CMAP_RANGE, 13253 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 8194 },
	{ 0x93c7, 0x93ce, PDF_CMAP_RANGE, 13261 },
	{ 0x93cf, 0x93cf, PDF_CMAP_SINGLE, 8339 },
	{ 0x93d0, 0x93d6, PDF_CMAP_RANGE, 13269 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 9055 },
	{ 0x93d8, 0x93da, PDF_CMAP_RANGE, 13276 },
	{ 0x93db, 0x93dc, PDF_CMAP_TABLE, 253 },
	{ 0x93dd, 0x93e0, PDF_CMAP_RANGE, 13279 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 7944 },
	{ 0x93e2, 0x93e3, PDF_CMAP_RANGE, 13283 },
	{ 0x93e4, 0x93e5, PDF_CMAP_TABLE, 255 },
	{ 0x93e6, 0x93e8, PDF_CMAP_RANGE, 13285 },
	{ 0x93e9, 0x93f1, PDF_CMAP_TABLE, 257 },
	{ 0x93f2, 0x93f3, PDF_CMAP_RANGE, 13291 },
	{ 0x93f4, 0x93f5, PDF_CMAP_TABLE, 266 },
	{ 0x93f6, 0x93f9, PDF_CMAP_RANGE, 13293 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 7849 },
	{ 0x93fb, 0x93fd, PDF_CMAP_RANGE, 13297 },
	{ 0x93fe, 0x93fe, PDF_CMAP_SINGLE, 8140 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 13300 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8050 },
	{ 0x9445, 0x944c, PDF_CMAP_RANGE, 13304 },
	{ 0x944d, 0x944d, PDF_CMAP_SINGLE, 8344 },
	{ 0x944e, 0x944f, PDF_CMAP_RANGE, 13312 },
	{ 0x9450, 0x9455, PDF_CMAP_TABLE, 268 },
	{ 0x9456, 0x9457, PDF_CMAP_RANGE, 13315 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 9061 },
	{ 0x9459, 0x945a, PDF_CMAP_RANGE, 13317 },
	{ 0x945b, 0x945f, PDF_CMAP_TABLE, 274 },
	{ 0x9460, 0x9463, PDF_CMAP_RANGE, 13320 },
	{ 0x9464, 0x9466, PDF_CMAP_TABLE, 279 },
	{ 0x9467, 0x946d, PDF_CMAP_RANGE, 13325 },
	{ 0x946e, 0x946e, PDF_CMAP_SINGLE, 8252 },
	{ 0x946f, 0x9471, PDF_CMAP_RANGE, 13332 },
	{ 0x9472, 0x947a, PDF_CMAP_TABLE, 282 },
	{ 0x947b, 0x947e, PDF_CMAP_RANGE, 13339 },
	{ 0x9480, 0x9482, PDF_CMAP_TABLE, 291 },
	{ 0x9483, 0x9486, PDF_CMAP_RANGE, 13343 },
	{ 0x9487, 0x9488, PDF_CMAP_TABLE, 294 },
	{ 0x9489, 0x94a0, PDF_CMAP_RANGE, 13347 },
	{ 0x94a1, 0x94a1, PDF_CMAP_SINGLE, 7726 },
	{ 0x94a2, 0x94b2, PDF_CMAP_RANGE, 13371 },
	{ 0x94b3, 0x94b5, PDF_CMAP_TABLE, 296 },
	{ 0x94b6, 0x94be, PDF_CMAP_RANGE, 13389 },
	{ 0x94bf, 0x94c0, PDF_CMAP_TABLE, 299 },
	{ 0x94c1, 0x94cb, PDF_CMAP_RANGE, 13398 },
	{ 0x94cc, 0x94cc, PDF_CMAP_SINGLE, 9428 },
	{ 0x94cd, 0x94d7, PDF_CMAP_RANGE, 13409 },
	{ 0x94d8, 0x94d8, PDF_CMAP_SINGLE, 8815 },
	{ 0x94d9, 0x94df, PDF_CMAP_RANGE, 13420 },
	{ 0x94e0, 0x94e0, PDF_CMAP_SINGLE, 7895 },
	{ 0x94e1, 0x94fe, PDF_CMAP_RANGE, 13427 },
	{ 0x9540, 0x9571, PDF_CMAP_RANGE, 13457 },
	{ 0x9572, 0x9572, PDF_CMAP_SINGLE, 8493 },
	{ 0x9573, 0x957e, PDF_CMAP_RANGE, 13507 },
	{ 0x9580, 0x9582, PDF_CMAP_RANGE, 13519 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 8859 },
	{ 0x9584, 0x959d, PDF_CMAP_RANGE, 13522 },
	{ 0x959e, 0x959f, PDF_CMAP_TABLE, 301 },
	{ 0x95a0, 0x95b2, PDF_CMAP_RANGE, 13548 },
	{ 0x95b3, 0x95b3, PDF_CMAP_SINGLE, 7801 },
	{ 0x95b4, 0x95b9, PDF_CMAP_RANGE, 13567 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 8794 },
	{ 0x95bb, 0x95ce, PDF_CMAP_RANGE, 13573 },
	{ 0x95cf, 0x95d4, PDF_CMAP_TABLE, 303 },
	{ 0x95d5, 0x95e0, PDF_CMAP_RANGE, 13595 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 9385 },
	{ 0x95e2, 0x95e6, PDF_CMAP_RANGE, 13607 },
	{ 0x95e7, 0x95e7, PDF_CMAP_SINGLE, 8167 },
	{ 0x95e8, 0x95f0, PDF_CMAP_RANGE, 13612 },
	{ 0x95f1, 0x95f1, PDF_CMAP_SINGLE, 8466 },
	{ 0x95f2, 0x95f7, PDF_CMAP_RANGE, 13621 },
	{ 0x95f8, 0x95f8, PDF_CMAP_SINGLE, 8508 },
	{ 0x95f9, 0x95fd, PDF_CMAP_RANGE, 13627 },
	{ 0x95fe, 0x95fe, PDF_CMAP_SINGLE, 8028 },
	{ 0x9640, 0x9655, PDF_CMAP_RANGE, 13632 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 9412 },
	{ 0x9657, 0x967b, PDF_CMAP_RANGE, 13654 },
	{ 0x967c, 0x967c, PDF_CMAP_SINGLE, 7884 },
	{ 0x967d, 0x967e, PDF_CMAP_RANGE, 13691 },
	{ 0x9680, 0x96fe, PDF_CMAP_RANGE, 13693 },
	{ 0x9740, 0x9766, PDF_CMAP_RANGE, 13820 },
	{ 0x9767, 0x9767, PDF_CMAP_SINGLE, 9322 },
	{ 0x9768, 0x976b, PDF_CMAP_RANGE, 13859 },
	{ 0x976c, 0x976e, PDF_CMAP_TABLE, 309 },
	{ 0x976f, 0x977e, PDF_CMAP_RANGE, 13864 },
	{ 0x9780, 0x9795, PDF_CMAP_RANGE, 13880 },
	{ 0x9796, 0x9797, PDF_CMAP_TABLE, 312 },
	{ 0x9798, 0x979c, PDF_CMAP_RANGE, 13902 },
	{ 0x979d, 0x979d, PDF_CMAP_SINGLE, 7886 },
	{ 0x979e, 0x97a2, PDF_CMAP_RANGE, 13907 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 8818 },
	{ 0x97a4, 0x97be, PDF_CMAP_RANGE, 13912 },
	{ 0x97bf, 0x97bf, PDF_CMAP_SINGLE, 9331 },
	{ 0x97c0, 0x97ed, PDF_CMAP_RANGE, 13939 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 8702 },
	{ 0x97ef, 0x97f6, PDF_CMAP_RANGE, 13985 },
	{ 0x97f7, 0x97f7, PDF_CMAP_SINGLE, 7934 },
	{ 0x97f8, 0x97fe, PDF_CMAP_RANGE, 13993 },
	{ 0x9840, 0x9844, PDF_CMAP_RANGE, 14000 },
	{ 0x9845, 0x9845, PDF_CMAP_SINGLE, 9333 },
	{ 0x9846, 0x9848, PDF_CMAP_RANGE, 14005 },
	{ 0x9849, 0x9849, PDF_CMAP_SINGLE, 8714 },
	{ 0x984a, 0x984e, PDF_CMAP_RANGE, 14008 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 8047 },
	{ 0x9850, 0x9870, PDF_CMAP_RANGE, 14013 },
	{ 0x9871, 0x9873, PDF_CMAP_TABLE, 314 },
	{ 0x9874, 0x987e, PDF_CMAP_RANGE, 14047 },
	{ 0x9880, 0x9881, PDF_CMAP_TABLE, 317 },
	{ 0x9882, 0x988a, PDF_CMAP_RANGE, 14059 },
	{ 0x988b, 0x988c, PDF_CMAP_TABLE, 319 },
	{ 0x988d, 0x989f, PDF_CMAP_RANGE, 14068 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 9340 },
	{ 0x98a1, 0x98a9, PDF_CMAP_RANGE, 14087 },
	{ 0x98aa, 0x98aa, PDF_CMAP_SINGLE, 8097 },
	{ 0x98ab, 0x98b5, PDF_CMAP_RANGE, 14096 },
	{ 0x98b6, 0x98b7, PDF_CMAP_TABLE, 321 },
	{ 0x98b8, 0x98b9, PDF_CMAP_RANGE, 14107 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 9324 },
	{ 0x98bb, 0x98c6, PDF_CMAP_RANGE, 14109 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 8254 },
	{ 0x98c8, 0x98ca, PDF_CMAP_RANGE, 14121 },
	{ 0x98cb, 0x98cb, PDF_CMAP_SINGLE, 7756 },
	{ 0x98cc, 0x98cf, PDF_CMAP_RANGE, 14124 },
	{ 0x98d0, 0x98d0, PDF_CMAP_SINGLE, 8506 },
	{ 0x98d1, 0x98d2, PDF_CMAP_RANGE, 14128 },
	{ 0x98d3, 0x98d3, PDF_CMAP_SINGLE, 8708 },
	{ 0x98d4, 0x98e2, PDF_CMAP_RANGE, 14130 },
	{ 0x98e3, 0x98e5, PDF_CMAP_TABLE, 323 },
	{ 0x98e6, 0x98ee, PDF_CMAP_RANGE, 14145 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 9332 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 14154 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 8413 },
	{ 0x98f3, 0x98fe, PDF_CMAP_RANGE, 14156 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 14168 },
	{ 0x9943, 0x9945, PDF_CMAP_TABLE, 326 },
	{ 0x9946, 0x9965, PDF_CMAP_RANGE, 14172 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 9330 },
	{ 0x9967, 0x996d, PDF_CMAP_RANGE, 14204 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 7861 },
	{ 0x996f, 0x9974, PDF_CMAP_RANGE, 14211 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 9336 },
	{ 0x9976, 0x9979, PDF_CMAP_RANGE, 14217 },
	{ 0x997a, 0x997b, PDF_CMAP_TABLE, 329 },
	{ 0x997c, 0x997e, PDF_CMAP_RANGE, 14221 },
	{ 0x9980, 0x9984, PDF_CMAP_RANGE, 14224 },
	{ 0x9985, 0x9985, PDF_CMAP_SINGLE, 9881 },
	{ 0x9986, 0x9988, PDF_CMAP_RANGE, 14229 },
	{ 0x9989, 0x9989, PDF_CMAP_SINGLE, 9346 },
	{ 0x998a, 0x998d, PDF_CMAP_RANGE, 14232 },
	{ 0x998e, 0x998e, PDF_CMAP_SINGLE, 8353 },
	{ 0x998f, 0x9990, PDF_CMAP_RANGE, 14236 },
	{ 0x9991, 0x9991, PDF_CMAP_SINGLE, 8082 },
	{ 0x9992, 0x9998, PDF_CMAP_RANGE, 14238 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 7992 },
	{ 0x999a, 0x99a8, PDF_CMAP_RANGE, 14245 },
	{ 0x99a9, 0x99a9, PDF_CMAP_SINGLE, 9349 },
	{ 0x99aa, 0x99af, PDF_CMAP_RANGE, 14260 },
	{ 0x99b0, 0x99b5, PDF_CMAP_TABLE, 331 },
	{ 0x99b6, 0x99bc, PDF_CMAP_RANGE, 14267 },
	{ 0x99bd, 0x99c2, PDF_CMAP_TABLE, 337 },
	{ 0x99c3, 0x99c8, PDF_CMAP_RANGE, 14276 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 9327 },
	{ 0x99ca, 0x99cd, PDF_CMAP_RANGE, 14282 },
	{ 0x99ce, 0x99ce, PDF_CMAP_SINGLE, 9345 },
	{ 0x99cf, 0x99d0, PDF_CMAP_RANGE, 14286 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 8736 },
	{ 0x99d2, 0x99d9, PDF_CMAP_RANGE, 14288 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 8181 },
	{ 0x99db, 0x99df, PDF_CMAP_RANGE, 14296 },
	{ 0x99e0, 0x99e0, PDF_CMAP_SINGLE, 8436 },
	{ 0x99e1, 0x99e4, PDF_CMAP_RANGE, 14301 },
	{ 0x99e5, 0x99e5, PDF_CMAP_SINGLE, 9341 },
	{ 0x99e6, 0x99e7, PDF_CMAP_RANGE, 14305 },
	{ 0x99e8, 0x99e8, PDF_CMAP_SINGLE, 9337 },
	{ 0x99e9, 0x99eb, PDF_CMAP_RANGE, 14307 },
	{ 0x99ec, 0x99ec, PDF_CMAP_SINGLE, 9342 },
	{ 0x99ed, 0x99f3, PDF_CMAP_RANGE, 14310 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 9338 },
	{ 0x99f5, 0x99fe, PDF_CMAP_RANGE, 14317 },
	{ 0x9a40, 0x9a49, PDF_CMAP_RANGE, 14327 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 8419 },
	{ 0x9a4b, 0x9a56, PDF_CMAP_RANGE, 14337 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 8365 },
	{ 0x9a58, 0x9a64, PDF_CMAP_RANGE, 14349 },
	{ 0x9a65, 0x9a67, PDF_CMAP_TABLE, 343 },
	{ 0x9a68, 0x9a70, PDF_CMAP_RANGE, 14363 },
	{ 0x9a71, 0x9a71, PDF_CMAP_SINGLE, 8537 },
	{ 0x9a72, 0x9a75, PDF_CMAP_RANGE, 14372 },
	{ 0x9a76, 0x9a77, PDF_CMAP_TABLE, 346 },
	{ 0x9a78, 0x9a7e, PDF_CMAP_RANGE, 14376 },
	{ 0x9a80, 0x9a87, PDF_CMAP_RANGE, 14383 },
	{ 0x9a88, 0x9a88, PDF_CMAP_SINGLE, 7772 },
	{ 0x9a89, 0x9a8b, PDF_CMAP_RANGE, 14391 },
	{ 0x9a8c, 0x9a8c, PDF_CMAP_SINGLE, 9352 },
	{ 0x9a8d, 0x9a90, PDF_CMAP_RANGE, 14394 },
	{ 0x9a91, 0x9a91, PDF_CMAP_SINGLE, 9351 },
	{ 0x9a92, 0x9a96, PDF_CMAP_RANGE, 14398 },
	{ 0x9a97, 0x9a97, PDF_CMAP_SINGLE, 9354 },
	{ 0x9a98, 0x9a99, PDF_CMAP_RANGE, 14403 },
	{ 0x9a9a, 0x9a9b, PDF_CMAP_TABLE, 348 },
	{ 0x9a9c, 0x9a9d, PDF_CMAP_RANGE, 14405 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 8067 },
	{ 0x9a9f, 0x9aa1, PDF_CMAP_RANGE, 14407 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_TABLE, 350 },
	{ 0x9aa4, 0x9aa9, PDF_CMAP_RANGE, 14410 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 8367 },
	{ 0x9aab, 0x9acf, PDF_CMAP_RANGE, 14416 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 9408 },
	{ 0x9ad1, 0x9ad5, PDF_CMAP_RANGE, 14453 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 8813 },
	{ 0x9ad7, 0x9ad9, PDF_CMAP_RANGE, 14458 },
	{ 0x9ada, 0x9ada, PDF_CMAP_SINGLE, 9409 },
	{ 0x9adb, 0x9ae1, PDF_CMAP_RANGE, 14461 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_TABLE, 352 },
	{ 0x9ae6, 0x9afe, PDF_CMAP_RANGE, 14469 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 14494 },
	{ 0x9b80, 0x9bd0, PDF_CMAP_RANGE, 14557 },
	{ 0x9bd1, 0x9bd1, PDF_CMAP_SINGLE, 9180 },
	{ 0x9bd2, 0x9bdb, PDF_CMAP_RANGE, 14638 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 9179 },
	{ 0x9bdd, 0x9bfe, PDF_CMAP_RANGE, 14648 },
	{ 0x9c40, 0x9c52, PDF_CMAP_RANGE, 14682 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 8286 },
	{ 0x9c54, 0x9c58, PDF_CMAP_RANGE, 14701 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_TABLE, 356 },
	{ 0x9c5d, 0x9c74, PDF_CMAP_RANGE, 14707 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 8607 },
	{ 0x9c76, 0x9c78, PDF_CMAP_RANGE, 14731 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 7782 },
	{ 0x9c7a, 0x9c7e, PDF_CMAP_RANGE, 14734 },
	{ 0x9c80, 0x9c85, PDF_CMAP_RANGE, 14739 },
	{ 0x9c86, 0x9c86, PDF_CMAP_SINGLE, 8035 },
	{ 0x9c87, 0x9c9c, PDF_CMAP_RANGE, 14745 },
	{ 0x9c9d, 0x9c9d, PDF_CMAP_SINGLE, 9181 },
	{ 0x9c9e, 0x9caa, PDF_CMAP_RANGE, 14767 },
	{ 0x9cab, 0x9cab, PDF_CMAP_SINGLE, 8555 },
	{ 0x9cac, 0x9cc9, PDF_CMAP_RANGE, 14780 },
	{ 0x9cca, 0x9cca, PDF_CMAP_SINGLE, 8885 },
	{ 0x9ccb, 0x9cce, PDF_CMAP_RANGE, 14810 },
	{ 0x9ccf, 0x9ccf, PDF_CMAP_SINGLE, 7974 },
	{ 0x9cd0, 0x9ce5, PDF_CMAP_RANGE, 14814 },
	{ 0x9ce6, 0x9ce7, PDF_CMAP_TABLE, 360 },
	{ 0x9ce8, 0x9ceb, PDF_CMAP_RANGE, 14836 },
	{ 0x9cec, 0x9cee, PDF_CMAP_TABLE, 362 },
	{ 0x9cef, 0x9cfa, PDF_CMAP_RANGE, 14841 },
	{ 0x9cfb, 0x9cfb, PDF_CMAP_SINGLE, 8011 },
	{ 0x9cfc, 0x9cfd, PDF_CMAP_RANGE, 14853 },
	{ 0x9cfe, 0x9cfe, PDF_CMAP_SINGLE, 8850 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 14855 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 8484 },
	{ 0x9d43, 0x9d45, PDF_CMAP_RANGE, 14857 },
	{ 0x9d46, 0x9d47, PDF_CMAP_TABLE, 365 },
	{ 0x9d48, 0x9d4c, PDF_CMAP_RANGE, 14860 },
	{ 0x9d4d, 0x9d4f, PDF_CMAP_TABLE, 367 },
	{ 0x9d50, 0x9d60, PDF_CMAP_RANGE, 14866 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 8369 },
	{ 0x9d62, 0x9d67, PDF_CMAP_RANGE, 14883 },
	{ 0x9d68, 0x9d69, PDF_CMAP_TABLE, 370 },
	{ 0x9d6a, 0x9d6d, PDF_CMAP_RANGE, 14889 },
	{ 0x9d6e, 0x9d6e, PDF_CMAP_SINGLE, 8888 },
	{ 0x9d6f, 0x9d70, PDF_CMAP_RANGE, 14893 },
	{ 0x9d71, 0x9d71, PDF_CMAP_SINGLE, 8822 },
	{ 0x9d72, 0x9d74, PDF_CMAP_RANGE, 14895 },
	{ 0x9d75, 0x9d75, PDF_CMAP_SINGLE, 8091 },
	{ 0x9d76, 0x9d7a, PDF_CMAP_RANGE, 14898 },
	{ 0x9d7b, 0x9d7e, PDF_CMAP_TABLE, 372 },
	{ 0x9d80, 0x9d89, PDF_CMAP_RANGE, 14905 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 8382 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 14915 },
	{ 0x9d8d, 0x9d8d, PDF_CMAP_SINGLE, 8116 },
	{ 0x9d8e, 0x9d90, PDF_CMAP_RANGE, 14917 },
	{ 0x9d91, 0x9d91, PDF_CMAP_SINGLE, 9175 },
	{ 0x9d92, 0x9d98, PDF_CMAP_RANGE, 14920 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 8453 },
	{ 0x9d9a, 0x9da0, PDF_CMAP_RANGE, 14927 },
	{ 0x9da1, 0x9da2, PDF_CMAP_TABLE, 376 },
	{ 0x9da3, 0x9da6, PDF_CMAP_RANGE, 14934 },
	{ 0x9da7, 0x9da7, PDF_CMAP_SINGLE, 9194 },
	{ 0x9da8, 0x9dab, PDF_CMAP_RANGE, 14938 },
	{ 0x9dac, 0x9dad, PDF_CMAP_TABLE, 378 },
	{ 0x9dae, 0x9db1, PDF_CMAP_RANGE, 14942 },
	{ 0x9db2, 0x9db3, PDF_CMAP_TABLE, 380 },
	{ 0x9db4, 0x9dbd, PDF_CMAP_RANGE, 14946 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 8093 },
	{ 0x9dbf, 0x9dc5, PDF_CMAP_RANGE, 14956 },
	{ 0x9dc6, 0x9dc6, PDF_CMAP_SINGLE, 9189 },
	{ 0x9dc7, 0x9dc8, PDF_CMAP_RANGE, 14963 },
	{ 0x9dc9, 0x9dc9, PDF_CMAP_SINGLE, 8804 },
	{ 0x9dca, 0x9dcc, PDF_CMAP_RANGE, 14965 },
	{ 0x9dcd, 0x9dcd, PDF_CMAP_SINGLE, 9442 },
	{ 0x9dce, 0x9dd1, PDF_CMAP_RANGE, 14968 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 9182 },
	{ 0x9dd3, 0x9dd4, PDF_CMAP_RANGE, 14972 },
	{ 0x9dd5, 0x9dd5, PDF_CMAP_SINGLE, 7876 },
	{ 0x9dd6, 0x9de0, PDF_CMAP_RANGE, 14974 },
	{ 0x9de1, 0x9de2, PDF_CMAP_TABLE, 382 },
	{ 0x9de3, 0x9df0, PDF_CMAP_RANGE, 14985 },
	{ 0x9df1, 0x9df1, PDF_CMAP_SINGLE, 8491 },
	{ 0x9df2, 0x9df3, PDF_CMAP_RANGE, 14999 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 8357 },
	{ 0x9df5, 0x9df6, PDF_CMAP_RANGE, 15001 },
	{ 0x9df7, 0x9df7, PDF_CMAP_SINGLE, 9867 },
	{ 0x9df8, 0x9df9, PDF_CMAP_RANGE, 15003 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 8054 },
	{ 0x9dfb, 0x9dfc, PDF_CMAP_RANGE, 15005 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_TABLE, 384 },
	{ 0x9e40, 0x9e44, PDF_CMAP_RANGE, 15008 },
	{ 0x9e45, 0x9e45, PDF_CMAP_SINGLE, 8193 },
	{ 0x9e46, 0x9e47, PDF_CMAP_RANGE, 15013 },
	{ 0x9e48, 0x9e49, PDF_CMAP_TABLE, 386 },
	{ 0x9e4a, 0x9e51, PDF_CMAP_RANGE, 15015 },
	{ 0x9e52, 0x9e56, PDF_CMAP_TABLE, 388 },
	{ 0x9e57, 0x9e5c, PDF_CMAP_RANGE, 15025 },
	{ 0x9e5d, 0x9e5e, PDF_CMAP_TABLE, 393 },
	{ 0x9e5f, 0x9e60, PDF_CMAP_RANGE, 15031 },
	{ 0x9e61, 0x9e63, PDF_CMAP_TABLE, 395 },
	{ 0x9e64, 0x9e66, PDF_CMAP_RANGE, 15034 },
	{ 0x9e67, 0x9e67, PDF_CMAP_SINGLE, 9183 },
	{ 0x9e68, 0x9e6b, PDF_CMAP_RANGE, 15037 },
	{ 0x9e6c, 0x9e6c, PDF_CMAP_SINGLE, 7759 },
	{ 0x9e6d, 0x9e6e, PDF_CMAP_RANGE, 15041 },
	{ 0x9e6f, 0x9e6f, PDF_CMAP_SINGLE, 9177 },
	{ 0x9e70, 0x9e71, PDF_CMAP_RANGE, 15043 },
	{ 0x9e72, 0x9e75, PDF_CMAP_TABLE, 398 },
	{ 0x9e76, 0x9e7a, PDF_CMAP_RANGE, 15046 },
	{ 0x9e7b, 0x9e7c, PDF_CMAP_TABLE, 402 },
	{ 0x9e7d, 0x9e7e, PDF_CMAP_RANGE, 15051 },
	{ 0x9e80, 0x9e84, PDF_CMAP_RANGE, 15053 },
	{ 0x9e85, 0x9e87, PDF_CMAP_TABLE, 404 },
	{ 0x9e88, 0x9e90, PDF_CMAP_RANGE, 15059 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 8186 },
	{ 0x9e92, 0x9e95, PDF_CMAP_RANGE, 15068 },
	{ 0x9e96, 0x9e97, PDF_CMAP_TABLE, 407 },
	{ 0x9e98, 0x9ea1, PDF_CMAP_RANGE, 15072 },
	{ 0x9ea2, 0x9ea2, PDF_CMAP_SINGLE, 8454 },
	{ 0x9ea3, 0x9ea5, PDF_CMAP_RANGE, 15082 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 8203 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 15085 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 8550 },
	{ 0x9eaa, 0x9ead, PDF_CMAP_RANGE, 15087 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 9199 },
	{ 0x9eaf, 0x9eb2, PDF_CMAP_RANGE, 15091 },
	{ 0x9eb3, 0x9eb4, PDF_CMAP_TABLE, 409 },
	{ 0x9eb5, 0x9eb6, PDF_CMAP_RANGE, 15095 },
	{ 0x9eb7, 0x9eb7, PDF_CMAP_SINGLE, 9191 },
	{ 0x9eb8, 0x9ef4, PDF_CMAP_RANGE, 15097 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 8611 },
	{ 0x9ef6, 0x9efe, PDF_CMAP_RANGE, 15158 },
	{ 0x9f40, 0x9f4d, PDF_CMAP_RANGE, 15167 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 8570 },
	{ 0x9f4f, 0x9f6e, PDF_CMAP_RANGE, 15181 },
	{ 0x9f6f, 0x9f6f, PDF_CMAP_SINGLE, 8613 },
	{ 0x9f70, 0x9f7e, PDF_CMAP_RANGE, 15213 },
	{ 0x9f80, 0x9f91, PDF_CMAP_RANGE, 15228 },
	{ 0x9f92, 0x9f92, PDF_CMAP_SINGLE, 8226 },
	{ 0x9f93, 0x9f97, PDF_CMAP_RANGE, 15246 },
	{ 0x9f98, 0x9f98, PDF_CMAP_SINGLE, 9430 },
	{ 0x9f99, 0x9fa5, PDF_CMAP_RANGE, 15251 },
	{ 0x9fa6, 0x9fa6, PDF_CMAP_SINGLE, 9018 },
	{ 0x9fa7, 0x9fa8, PDF_CMAP_RANGE, 15264 },
	{ 0x9fa9, 0x9fa9, PDF_CMAP_SINGLE, 7918 },
	{ 0x9faa, 0x9fab, PDF_CMAP_RANGE, 15266 },
	{ 0x9fac, 0x9fac, PDF_CMAP_SINGLE, 9429 },
	{ 0x9fad, 0x9fc8, PDF_CMAP_RANGE, 15268 },
	{ 0x9fc9, 0x9fc9, PDF_CMAP_SINGLE, 8744 },
	{ 0x9fca, 0x9fcc, PDF_CMAP_RANGE, 15296 },
	{ 0x9fcd, 0x9fcd, PDF_CMAP_SINGLE, 9431 },
	{ 0x9fce, 0x9fe0, PDF_CMAP_RANGE, 15299 },
	{ 0x9fe1, 0x9fe1, PDF_CMAP_SINGLE, 8444 },
	{ 0x9fe2, 0x9fea, PDF_CMAP_RANGE, 15318 },
	{ 0x9feb, 0x9feb, PDF_CMAP_SINGLE, 7815 },
	{ 0x9fec, 0x9fed, PDF_CMAP_RANGE, 15327 },
	{ 0x9fee, 0x9fee, PDF_CMAP_SINGLE, 9432 },
	{ 0x9fef, 0x9ff3, PDF_CMAP_RANGE, 15329 },
	{ 0x9ff4, 0x9ff4, PDF_CMAP_SINGLE, 7866 },
	{ 0x9ff5, 0x9ffc, PDF_CMAP_RANGE, 15334 },
	{ 0x9ffd, 0x9ffe, PDF_CMAP_TABLE, 411 },
	{ 0xa040, 0xa042, PDF_CMAP_RANGE, 15343 },
	{ 0xa043, 0xa043, PDF_CMAP_SINGLE, 8556 },
	{ 0xa044, 0xa045, PDF_CMAP_RANGE, 15346 },
	{ 0xa046, 0xa046, PDF_CMAP_SINGLE, 9433 },
	{ 0xa047, 0xa048, PDF_CMAP_RANGE, 15348 },
	{ 0xa049, 0xa049, PDF_CMAP_SINGLE, 8743 },
	{ 0xa04a, 0xa04d, PDF_CMAP_RANGE, 15350 },
	{ 0xa04e, 0xa04e, PDF_CMAP_SINGLE, 7775 },
	{ 0xa04f, 0xa053, PDF_CMAP_RANGE, 15354 },
	{ 0xa054, 0xa054, PDF_CMAP_SINGLE, 8863 },
	{ 0xa055, 0xa059, PDF_CMAP_RANGE, 15359 },
	{ 0xa05a, 0xa05a, PDF_CMAP_SINGLE, 8029 },
	{ 0xa05b, 0xa060, PDF_CMAP_RANGE, 15364 },
	{ 0xa061, 0xa063, PDF_CMAP_TABLE, 413 },
	{ 0xa064, 0xa070, PDF_CMAP_RANGE, 15371 },
	{ 0xa071, 0xa071, PDF_CMAP_SINGLE, 8521 },
	{ 0xa072, 0xa073, PDF_CMAP_RANGE, 15384 },
	{ 0xa074, 0xa074, PDF_CMAP_SINGLE, 8262 },
	{ 0xa075, 0xa07e, PDF_CMAP_RANGE, 15386 },
	{ 0xa080, 0xa080, PDF_CMAP_SINGLE, 8192 },
	{ 0xa081, 0xa090, PDF_CMAP_RANGE, 15396 },
	{ 0xa091, 0xa091, PDF_CMAP_SINGLE, 8592 },
	{ 0xa092, 0xa093, PDF_CMAP_RANGE, 15412 },
	{ 0xa094, 0xa096, PDF_CMAP_TABLE, 416 },
	{ 0xa097, 0xa0a8, PDF_CMAP_RANGE, 15415 },
	{ 0xa0a9, 0xa0a9, PDF_CMAP_SINGLE, 9411 },
	{ 0xa0aa, 0xa0be, PDF_CMAP_RANGE, 15433 },
	{ 0xa0bf, 0xa0bf, PDF_CMAP_SINGLE, 8396 },
	{ 0xa0c0, 0xa0cd, PDF_CMAP_RANGE, 15454 },
	{ 0xa0ce, 0xa0ce, PDF_CMAP_SINGLE, 9024 },
	{ 0xa0cf, 0xa0d8, PDF_CMAP_RANGE, 15468 },
	{ 0xa0d9, 0xa0d9, PDF_CMAP_SINGLE, 7889 },
	{ 0xa0da, 0xa0dd, PDF_CMAP_RANGE, 15478 },
	{ 0xa0de, 0xa0de, PDF_CMAP_SINGLE, 8620 },
	{ 0xa0df, 0xa0ed, PDF_CMAP_RANGE, 15482 },
	{ 0xa0ee, 0xa0ee, PDF_CMAP_SINGLE, 8879 },
	{ 0xa0ef, 0xa0fe, PDF_CMAP_RANGE, 15497 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa2a1, 0xa2aa, PDF_CMAP_RANGE, 9897 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 9907 },
	{ 0xa880, 0xa895, PDF_CMAP_RANGE, 9970 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa940, 0xa957, PDF_CMAP_RANGE, 9992 },
	{ 0xa959, 0xa95a, PDF_CMAP_RANGE, 10016 },
	{ 0xa95c, 0xa95c, PDF_CMAP_SINGLE, 10018 },
	{ 0xa960, 0xa97e, PDF_CMAP_RANGE, 10019 },
	{ 0xa980, 0xa995, PDF_CMAP_RANGE, 10050 },
	{ 0xa996, 0xa996, PDF_CMAP_SINGLE, 7703 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaa40, 0xaa4c, PDF_CMAP_RANGE, 15513 },
	{ 0xaa4d, 0xaa4e, PDF_CMAP_TABLE, 419 },
	{ 0xaa4f, 0xaa70, PDF_CMAP_RANGE, 15526 },
	{ 0xaa71, 0xaa73, PDF_CMAP_TABLE, 421 },
	{ 0xaa74, 0xaa76, PDF_CMAP_RANGE, 15561 },
	{ 0xaa77, 0xaa77, PDF_CMAP_SINGLE, 9109 },
	{ 0xaa78, 0xaa79, PDF_CMAP_RANGE, 15564 },
	{ 0xaa7a, 0xaa7b, PDF_CMAP_TABLE, 424 },
	{ 0xaa7c, 0xaa7e, PDF_CMAP_RANGE, 15566 },
	{ 0xaa80, 0xaa99, PDF_CMAP_RANGE, 15569 },
	{ 0xaa9a, 0xaaa0, PDF_CMAP_TABLE, 426 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xab40, 0xab40, PDF_CMAP_SINGLE, 8036 },
	{ 0xab41, 0xab42, PDF_CMAP_RANGE, 15598 },
	{ 0xab43, 0xab4a, PDF_CMAP_TABLE, 433 },
	{ 0xab4b, 0xab4c, PDF_CMAP_RANGE, 15602 },
	{ 0xab4d, 0xab4d, PDF_CMAP_SINGLE, 9113 },
	{ 0xab4e, 0xab7e, PDF_CMAP_RANGE, 15604 },
	{ 0xab80, 0xaba0, PDF_CMAP_RANGE, 15653 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xac40, 0xac45, PDF_CMAP_RANGE, 15686 },
	{ 0xac46, 0xac46, PDF_CMAP_SINGLE, 8640 },
	{ 0xac47, 0xac70, PDF_CMAP_RANGE, 15692 },
	{ 0xac71, 0xac71, PDF_CMAP_SINGLE, 9312 },
	{ 0xac72, 0xac7b, PDF_CMAP_RANGE, 15734 },
	{ 0xac7c, 0xac7c, PDF_CMAP_SINGLE, 9308 },
	{ 0xac7d, 0xac7e, PDF_CMAP_RANGE, 15744 },
	{ 0xac80, 0xac8c, PDF_CMAP_RANGE, 15746 },
	{ 0xac8d, 0xac8d, PDF_CMAP_SINGLE, 8541 },
	{ 0xac8e, 0xac92, PDF_CMAP_RANGE, 15759 },
	{ 0xac93, 0xac94, PDF_CMAP_TABLE, 441 },
	{ 0xac95, 0xaca0, PDF_CMAP_RANGE, 15764 },
	{ 0xad40, 0xad48, PDF_CMAP_RANGE, 15776 },
	{ 0xad49, 0xad49, PDF_CMAP_SINGLE, 9313 },
	{ 0xad4a, 0xad5d, PDF_CMAP_RANGE, 15785 },
	{ 0xad5e, 0xad5e, PDF_CMAP_SINGLE, 9307 },
	{ 0xad5f, 0xad60, PDF_CMAP_RANGE, 15805 },
	{ 0xad61, 0xad61, PDF_CMAP_SINGLE, 9314 },
	{ 0xad62, 0xad67, PDF_CMAP_RANGE, 15807 },
	{ 0xad68, 0xad68, PDF_CMAP_SINGLE, 8020 },
	{ 0xad69, 0xad73, PDF_CMAP_RANGE, 15813 },
	{ 0xad74, 0xad74, PDF_CMAP_SINGLE, 9311 },
	{ 0xad75, 0xad7e, PDF_CMAP_RANGE, 15824 },
	{ 0xad80, 0xad81, PDF_CMAP_RANGE, 15834 },
	{ 0xad82, 0xad82, PDF_CMAP_SINGLE, 8428 },
	{ 0xad83, 0xad86, PDF_CMAP_RANGE, 15836 },
	{ 0xad87, 0xad87, PDF_CMAP_SINGLE, 9309 },
	{ 0xad88, 0xad8a, PDF_CMAP_RANGE, 15840 },
	{ 0xad8b, 0xad8b, PDF_CMAP_SINGLE, 9315 },
	{ 0xad8c, 0xad90, PDF_CMAP_RANGE, 15843 },
	{ 0xad91, 0xad91, PDF_CMAP_SINGLE, 9316 },
	{ 0xad92, 0xada0, PDF_CMAP_RANGE, 15848 },
	{ 0xae40, 0xae53, PDF_CMAP_RANGE, 15863 },
	{ 0xae54, 0xae54, PDF_CMAP_SINGLE, 9381 },
	{ 0xae55, 0xae61, PDF_CMAP_RANGE, 15883 },
	{ 0xae62, 0xae62, PDF_CMAP_SINGLE, 7792 },
	{ 0xae63, 0xae7e, PDF_CMAP_RANGE, 15896 },
	{ 0xae80, 0xae80, PDF_CMAP_SINGLE, 8335 },
	{ 0xae81, 0xae84, PDF_CMAP_RANGE, 15924 },
	{ 0xae85, 0xae85, PDF_CMAP_SINGLE, 7746 },
	{ 0xae86, 0xae8a, PDF_CMAP_RANGE, 15928 },
	{ 0xae8b, 0xae8b, PDF_CMAP_SINGLE, 8014 },
	{ 0xae8c, 0xae93, PDF_CMAP_RANGE, 15933 },
	{ 0xae94, 0xae94, PDF_CMAP_SINGLE, 7857 },
	{ 0xae95, 0xae9f, PDF_CMAP_RANGE, 15941 },
	{ 0xaea0, 0xaea0, PDF_CMAP_SINGLE, 7819 },
	{ 0xaf40, 0xaf63, PDF_CMAP_RANGE, 15952 },
	{ 0xaf64, 0xaf64, PDF_CMAP_SINGLE, 8134 },
	{ 0xaf65, 0xaf7a, PDF_CMAP_RANGE, 15988 },
	{ 0xaf7b, 0xaf7b, PDF_CMAP_SINGLE, 9646 },
	{ 0xaf7c, 0xaf7e, PDF_CMAP_RANGE, 16010 },
	{ 0xaf80, 0xaf81, PDF_CMAP_RANGE, 16013 },
	{ 0xaf82, 0xaf83, PDF_CMAP_TABLE, 443 },
	{ 0xaf84, 0xaf8d, PDF_CMAP_RANGE, 16015 },
	{ 0xaf8e, 0xaf91, PDF_CMAP_TABLE, 445 },
	{ 0xaf92, 0xaf9b, PDF_CMAP_RANGE, 16026 },
	{ 0xaf9c, 0xaf9c, PDF_CMAP_SINGLE, 9651 },
	{ 0xaf9d, 0xaf9e, PDF_CMAP_RANGE, 16036 },
	{ 0xaf9f, 0xafa0, PDF_CMAP_TABLE, 449 },
	{ 0xb040, 0xb040, PDF_CMAP_SINGLE, 16039 },
	{ 0xb041, 0xb042, PDF_CMAP_RANGE, 9647 },
	{ 0xb043, 0xb044, PDF_CMAP_TABLE, 451 },
	{ 0xb045, 0xb04e, PDF_CMAP_RANGE, 16041 },
	{ 0xb04f, 0xb04f, PDF_CMAP_SINGLE, 9644 },
	{ 0xb050, 0xb053, PDF_CMAP_RANGE, 16051 },
	{ 0xb054, 0xb054, PDF_CMAP_SINGLE, 7758 },
	{ 0xb055, 0xb056, PDF_CMAP_RANGE, 16055 },
	{ 0xb057, 0xb05f, PDF_CMAP_TABLE, 453 },
	{ 0xb060, 0xb061, PDF_CMAP_RANGE, 9652 },
	{ 0xb062, 0xb064, PDF_CMAP_TABLE, 462 },
	{ 0xb065, 0xb06b, PDF_CMAP_RANGE, 16060 },
	{ 0xb06c, 0xb06c, PDF_CMAP_SINGLE, 7913 },
	{ 0xb06d, 0xb07c, PDF_CMAP_RANGE, 16067 },
	{ 0xb07d, 0xb07e, PDF_CMAP_TABLE, 465 },
	{ 0xb080, 0xb096, PDF_CMAP_RANGE, 16084 },
	{ 0xb097, 0xb099, PDF_CMAP_TABLE, 467 },
	{ 0xb09a, 0xb0a0, PDF_CMAP_RANGE, 16108 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 940 },
	{ 0xb140, 0xb14a, PDF_CMAP_RANGE, 16115 },
	{ 0xb14b, 0xb152, PDF_CMAP_TABLE, 470 },
	{ 0xb153, 0xb17e, PDF_CMAP_RANGE, 16129 },
	{ 0xb180, 0xb196, PDF_CMAP_RANGE, 16173 },
	{ 0xb197, 0xb197, PDF_CMAP_SINGLE, 9860 },
	{ 0xb198, 0xb1a0, PDF_CMAP_RANGE, 16196 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1034 },
	{ 0xb240, 0xb241, PDF_CMAP_TABLE, 478 },
	{ 0xb242, 0xb266, PDF_CMAP_RANGE, 16206 },
	{ 0xb267, 0xb267, PDF_CMAP_SINGLE, 9454 },
	{ 0xb268, 0xb26c, PDF_CMAP_RANGE, 16243 },
	{ 0xb26d, 0xb26d, PDF_CMAP_SINGLE, 8308 },
	{ 0xb26e, 0xb273, PDF_CMAP_RANGE, 16248 },
	{ 0xb274, 0xb274, PDF_CMAP_SINGLE, 9863 },
	{ 0xb275, 0xb27e, PDF_CMAP_RANGE, 16254 },
	{ 0xb280, 0xb280, PDF_CMAP_SINGLE, 9456 },
	{ 0xb281, 0xb288, PDF_CMAP_RANGE, 16264 },
	{ 0xb289, 0xb289, PDF_CMAP_SINGLE, 9869 },
	{ 0xb28a, 0xb299, PDF_CMAP_RANGE, 16272 },
	{ 0xb29a, 0xb29a, PDF_CMAP_SINGLE, 8864 },
	{ 0xb29b, 0xb2a0, PDF_CMAP_RANGE, 16288 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1128 },
	{ 0xb340, 0xb342, PDF_CMAP_RANGE, 16294 },
	{ 0xb343, 0xb343, PDF_CMAP_SINGLE, 8107 },
	{ 0xb344, 0xb36f, PDF_CMAP_RANGE, 16297 },
	{ 0xb370, 0xb370, PDF_CMAP_SINGLE, 9896 },
	{ 0xb371, 0xb37e, PDF_CMAP_RANGE, 16341 },
	{ 0xb380, 0xb387, PDF_CMAP_RANGE, 16355 },
	{ 0xb388, 0xb388, PDF_CMAP_SINGLE, 9449 },
	{ 0xb389, 0xb38b, PDF_CMAP_RANGE, 16363 },
	{ 0xb38c, 0xb38e, PDF_CMAP_TABLE, 480 },
	{ 0xb38f, 0xb3a0, PDF_CMAP_RANGE, 16367 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1222 },
	{ 0xb440, 0xb453, PDF_CMAP_RANGE, 16385 },
	{ 0xb454, 0xb454, PDF_CMAP_SINGLE, 8520 },
	{ 0xb455, 0xb457, PDF_CMAP_RANGE, 16405 },
	{ 0xb458, 0xb458, PDF_CMAP_SINGLE, 9444 },
	{ 0xb459, 0xb45d, PDF_CMAP_RANGE, 16408 },
	{ 0xb45e, 0xb461, PDF_CMAP_TABLE, 483 },
	{ 0xb462, 0xb474, PDF_CMAP_RANGE, 16414 },
	{ 0xb475, 0xb475, PDF_CMAP_SINGLE, 8871 },
	{ 0xb476, 0xb47d, PDF_CMAP_RANGE, 16433 },
	{ 0xb47e, 0xb47e, PDF_CMAP_SINGLE, 9452 },
	{ 0xb480, 0xb482, PDF_CMAP_RANGE, 16441 },
	{ 0xb483, 0xb483, PDF_CMAP_SINGLE, 9451 },
	{ 0xb484, 0xb488, PDF_CMAP_RANGE, 16444 },
	{ 0xb489, 0xb489, PDF_CMAP_SINGLE, 9443 },
	{ 0xb48a, 0xb492, PDF_CMAP_RANGE, 16449 },
	{ 0xb493, 0xb493, PDF_CMAP_SINGLE, 9450 },
	{ 0xb494, 0xb4a0, PDF_CMAP_RANGE, 16458 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1316 },
	{ 0xb540, 0xb541, PDF_CMAP_TABLE, 487 },
	{ 0xb542, 0xb54a, PDF_CMAP_RANGE, 16472 },
	{ 0xb54b, 0xb54b, PDF_CMAP_SINGLE, 7719 },
	{ 0xb54c, 0xb555, PDF_CMAP_RANGE, 16481 },
	{ 0xb556, 0xb556, PDF_CMAP_SINGLE, 8166 },
	{ 0xb557, 0xb559, PDF_CMAP_RANGE, 16491 },
	{ 0xb55a, 0xb55c, PDF_CMAP_TABLE, 489 },
	{ 0xb55d, 0xb560, PDF_CMAP_RANGE, 16494 },
	{ 0xb561, 0xb561, PDF_CMAP_SINGLE, 9448 },
	{ 0xb562, 0xb57e, PDF_CMAP_RANGE, 16498 },
	{ 0xb580, 0xb59b, PDF_CMAP_RANGE, 16527 },
	{ 0xb59c, 0xb59d, PDF_CMAP_TABLE, 492 },
	{ 0xb59e, 0xb5a0, PDF_CMAP_RANGE, 16555 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1410 },
	{ 0xb640, 0xb651, PDF_CMAP_RANGE, 16558 },
	{ 0xb652, 0xb652, PDF_CMAP_SINGLE, 9890 },
	{ 0xb653, 0xb654, PDF_CMAP_RANGE, 16576 },
	{ 0xb655, 0xb655, PDF_CMAP_SINGLE, 9437 },
	{ 0xb656, 0xb658, PDF_CMAP_RANGE, 16578 },
	{ 0xb659, 0xb65c, PDF_CMAP_TABLE, 494 },
	{ 0xb65d, 0xb67e, PDF_CMAP_RANGE, 16582 },
	{ 0xb680, 0xb6a0, PDF_CMAP_RANGE, 16616 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1504 },
	{ 0xb740, 0xb74d, PDF_CMAP_RANGE, 16649 },
	{ 0xb74e, 0xb74e, PDF_CMAP_SINGLE, 8853 },
	{ 0xb74f, 0xb750, PDF_CMAP_RANGE, 16663 },
	{ 0xb751, 0xb751, PDF_CMAP_SINGLE, 7808 },
	{ 0xb752, 0xb758, PDF_CMAP_RANGE, 16665 },
	{ 0xb759, 0xb759, PDF_CMAP_SINGLE, 9847 },
	{ 0xb75a, 0xb763, PDF_CMAP_RANGE, 16672 },
	{ 0xb764, 0xb766, PDF_CMAP_TABLE, 498 },
	{ 0xb767, 0xb776, PDF_CMAP_RANGE, 16682 },
	{ 0xb777, 0xb778, PDF_CMAP_TABLE, 501 },
	{ 0xb779, 0xb77e, PDF_CMAP_RANGE, 16698 },
	{ 0xb780, 0xb782, PDF_CMAP_TABLE, 503 },
	{ 0xb783, 0xb7a0, PDF_CMAP_RANGE, 16705 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1598 },
	{ 0xb840, 0xb842, PDF_CMAP_RANGE, 16735 },
	{ 0xb843, 0xb846, PDF_CMAP_TABLE, 506 },
	{ 0xb847, 0xb84c, PDF_CMAP_RANGE, 16739 },
	{ 0xb84d, 0xb84d, PDF_CMAP_SINGLE, 9657 },
	{ 0xb84e, 0xb850, PDF_CMAP_RANGE, 16745 },
	{ 0xb851, 0xb851, PDF_CMAP_SINGLE, 8170 },
	{ 0xb852, 0xb859, PDF_CMAP_RANGE, 16748 },
	{ 0xb85a, 0xb860, PDF_CMAP_TABLE, 510 },
	{ 0xb861, 0xb876, PDF_CMAP_RANGE, 16758 },
	{ 0xb877, 0xb877, PDF_CMAP_SINGLE, 8513 },
	{ 0xb878, 0xb87e, PDF_CMAP_RANGE, 16780 },
	{ 0xb880, 0xb881, PDF_CMAP_RANGE, 16787 },
	{ 0xb882, 0xb882, PDF_CMAP_SINGLE, 8135 },
	{ 0xb883, 0xb8a0, PDF_CMAP_RANGE, 16789 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1692 },
	{ 0xb940, 0xb94f, PDF_CMAP_RANGE, 16819 },
	{ 0xb950, 0xb950, PDF_CMAP_SINGLE, 7745 },
	{ 0xb951, 0xb960, PDF_CMAP_RANGE, 16835 },
	{ 0xb961, 0xb961, PDF_CMAP_SINGLE, 9700 },
	{ 0xb962, 0xb97a, PDF_CMAP_RANGE, 16851 },
	{ 0xb97b, 0xb97b, PDF_CMAP_SINGLE, 8070 },
	{ 0xb97c, 0xb97e, PDF_CMAP_RANGE, 16876 },
	{ 0xb980, 0xb99c, PDF_CMAP_RANGE, 16879 },
	{ 0xb99d, 0xb99d, PDF_CMAP_SINGLE, 8115 },
	{ 0xb99e, 0xb99f, PDF_CMAP_RANGE, 16908 },
	{ 0xb9a0, 0xb9a0, PDF_CMAP_SINGLE, 7919 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1786 },
	{ 0xba40, 0xba41, PDF_CMAP_RANGE, 16910 },
	{ 0xba42, 0xba44, PDF_CMAP_TABLE, 517 },
	{ 0xba45, 0xba55, PDF_CMAP_RANGE, 16913 },
	{ 0xba56, 0xba56, PDF_CMAP_SINGLE, 9699 },
	{ 0xba57, 0xba58, PDF_CMAP_RANGE, 16930 },
	{ 0xba59, 0xba59, PDF_CMAP_SINGLE, 8465 },
	{ 0xba5a, 0xba5f, PDF_CMAP_RANGE, 16932 },
	{ 0xba60, 0xba60, PDF_CMAP_SINGLE, 9702 },
	{ 0xba61, 0xba69, PDF_CMAP_RANGE, 16938 },
	{ 0xba6a, 0xba6a, PDF_CMAP_SINGLE, 9703 },
	{ 0xba6b, 0xba73, PDF_CMAP_RANGE, 16947 },
	{ 0xba74, 0xba74, PDF_CMAP_SINGLE, 8257 },
	{ 0xba75, 0xba7e, PDF_CMAP_RANGE, 16956 },
	{ 0xba80, 0xba83, PDF_CMAP_RANGE, 16966 },
	{ 0xba84, 0xba88, PDF_CMAP_TABLE, 520 },
	{ 0xba89, 0xba8c, PDF_CMAP_RANGE, 16972 },
	{ 0xba8d, 0xba8d, PDF_CMAP_SINGLE, 9707 },
	{ 0xba8e, 0xba9d, PDF_CMAP_RANGE, 16976 },
	{ 0xba9e, 0xbaa0, PDF_CMAP_TABLE, 525 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 1880 },
	{ 0xbb40, 0xbb40, PDF_CMAP_SINGLE, 8183 },
	{ 0xbb41, 0xbb48, PDF_CMAP_RANGE, 16993 },
	{ 0xbb49, 0xbb49, PDF_CMAP_SINGLE, 7821 },
	{ 0xbb4a, 0xbb57, PDF_CMAP_RANGE, 17001 },
	{ 0xbb58, 0xbb58, PDF_CMAP_SINGLE, 9705 },
	{ 0xbb59, 0xbb5a, PDF_CMAP_RANGE, 17015 },
	{ 0xbb5b, 0xbb5c, PDF_CMAP_TABLE, 528 },
	{ 0xbb5d, 0xbb5f, PDF_CMAP_RANGE, 17017 },
	{ 0xbb60, 0xbb60, PDF_CMAP_SINGLE, 9875 },
	{ 0xbb61, 0xbb64, PDF_CMAP_RANGE, 17020 },
	{ 0xbb65, 0xbb6a, PDF_CMAP_TABLE, 530 },
	{ 0xbb6b, 0xbb6d, PDF_CMAP_RANGE, 17026 },
	{ 0xbb6e, 0xbb6e, PDF_CMAP_SINGLE, 9891 },
	{ 0xbb6f, 0xbb7e, PDF_CMAP_RANGE, 17029 },
	{ 0xbb80, 0xbba0, PDF_CMAP_RANGE, 17045 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 1974 },
	{ 0xbc40, 0xbc51, PDF_CMAP_RANGE, 17078 },
	{ 0xbc52, 0xbc53, PDF_CMAP_TABLE, 536 },
	{ 0xbc54, 0xbc59, PDF_CMAP_RANGE, 17096 },
	{ 0xbc5a, 0xbc5a, PDF_CMAP_SINGLE, 8228 },
	{ 0xbc5b, 0xbc60, PDF_CMAP_RANGE, 17102 },
	{ 0xbc61, 0xbc69, PDF_CMAP_TABLE, 538 },
	{ 0xbc6a, 0xbc6c, PDF_CMAP_RANGE, 17112 },
	{ 0xbc6d, 0xbc75, PDF_CMAP_TABLE, 547 },
	{ 0xbc76, 0xbc77, PDF_CMAP_RANGE, 9246 },
	{ 0xbc78, 0xbc7b, PDF_CMAP_TABLE, 556 },
	{ 0xbc7c, 0xbc7d, PDF_CMAP_RANGE, 17119 },
	{ 0xbc7e, 0xbc7e, PDF_CMAP_SINGLE, 8359 },
	{ 0xbc80, 0xbc81, PDF_CMAP_RANGE, 17121 },
	{ 0xbc82, 0xbc8b, PDF_CMAP_TABLE, 560 },
	{ 0xbc8c, 0xbc8e, PDF_CMAP_RANGE, 17125 },
	{ 0xbc8f, 0xbc8f, PDF_CMAP_SINGLE, 7923 },
	{ 0xbc90, 0xbc99, PDF_CMAP_RANGE, 17128 },
	{ 0xbc9a, 0xbc9d, PDF_CMAP_TABLE, 570 },
	{ 0xbc9e, 0xbca0, PDF_CMAP_RANGE, 17138 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2068 },
	{ 0xbd40, 0xbd41, PDF_CMAP_RANGE, 17141 },
	{ 0xbd42, 0xbd45, PDF_CMAP_TABLE, 574 },
	{ 0xbd46, 0xbd47, PDF_CMAP_RANGE, 17144 },
	{ 0xbd48, 0xbd4f, PDF_CMAP_TABLE, 578 },
	{ 0xbd50, 0xbd56, PDF_CMAP_RANGE, 17149 },
	{ 0xbd57, 0xbd59, PDF_CMAP_TABLE, 586 },
	{ 0xbd5a, 0xbd65, PDF_CMAP_RANGE, 17157 },
	{ 0xbd66, 0xbd67, PDF_CMAP_TABLE, 589 },
	{ 0xbd68, 0xbd69, PDF_CMAP_RANGE, 17169 },
	{ 0xbd6a, 0xbd6b, PDF_CMAP_TABLE, 591 },
	{ 0xbd6c, 0xbd6e, PDF_CMAP_RANGE, 17171 },
	{ 0xbd6f, 0xbd71, PDF_CMAP_TABLE, 593 },
	{ 0xbd72, 0xbd78, PDF_CMAP_RANGE, 17175 },
	{ 0xbd79, 0xbd7b, PDF_CMAP_TABLE, 596 },
	{ 0xbd7c, 0xbd7d, PDF_CMAP_RANGE, 17182 },
	{ 0xbd7e, 0xbd7e, PDF_CMAP_SINGLE, 8148 },
	{ 0xbd80, 0xbd81, PDF_CMAP_TABLE, 599 },
	{ 0xbd82, 0xbd88, PDF_CMAP_RANGE, 17185 },
	{ 0xbd89, 0xbd8b, PDF_CMAP_TABLE, 601 },
	{ 0xbd8c, 0xbd8d, PDF_CMAP_RANGE, 17193 },
	{ 0xbd8e, 0xbd91, PDF_CMAP_TABLE, 604 },
	{ 0xbd92, 0xbd96, PDF_CMAP_RANGE, 17196 },
	{ 0xbd97, 0xbd97, PDF_CMAP_SINGLE, 8536 },
	{ 0xbd98, 0xbd9a, PDF_CMAP_RANGE, 17201 },
	{ 0xbd9b, 0xbd9b, PDF_CMAP_SINGLE, 8130 },
	{ 0xbd9c, 0xbda0, PDF_CMAP_RANGE, 17204 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2162 },
	{ 0xbe40, 0xbe42, PDF_CMAP_RANGE, 17209 },
	{ 0xbe43, 0xbe45, PDF_CMAP_TABLE, 608 },
	{ 0xbe46, 0xbe48, PDF_CMAP_RANGE, 17213 },
	{ 0xbe49, 0xbe4a, PDF_CMAP_TABLE, 611 },
	{ 0xbe4b, 0xbe50, PDF_CMAP_RANGE, 17216 },
	{ 0xbe51, 0xbe59, PDF_CMAP_TABLE, 613 },
	{ 0xbe5a, 0xbe5c, PDF_CMAP_RANGE, 17223 },
	{ 0xbe5d, 0xbe64, PDF_CMAP_TABLE, 622 },
	{ 0xbe65, 0xbe68, PDF_CMAP_RANGE, 17227 },
	{ 0xbe69, 0xbe69, PDF_CMAP_SINGLE, 9269 },
	{ 0xbe6a, 0xbe6b, PDF_CMAP_RANGE, 17231 },
	{ 0xbe6c, 0xbe6c, PDF_CMAP_SINGLE, 9275 },
	{ 0xbe6d, 0xbe6e, PDF_CMAP_RANGE, 17233 },
	{ 0xbe6f, 0xbe70, PDF_CMAP_TABLE, 630 },
	{ 0xbe71, 0xbe75, PDF_CMAP_RANGE, 17235 },
	{ 0xbe76, 0xbe79, PDF_CMAP_TABLE, 632 },
	{ 0xbe7a, 0xbe7b, PDF_CMAP_RANGE, 17241 },
	{ 0xbe7c, 0xbe7e, PDF_CMAP_TABLE, 636 },
	{ 0xbe80, 0xbe82, PDF_CMAP_RANGE, 17243 },
	{ 0xbe83, 0xbe89, PDF_CMAP_TABLE, 639 },
	{ 0xbe8a, 0xbe8b, PDF_CMAP_RANGE, 17248 },
	{ 0xbe8c, 0xbe8f, PDF_CMAP_TABLE, 646 },
	{ 0xbe90, 0xbe91, PDF_CMAP_RANGE, 17251 },
	{ 0xbe92, 0xbe92, PDF_CMAP_SINGLE, 8326 },
	{ 0xbe93, 0xbe94, PDF_CMAP_RANGE, 17253 },
	{ 0xbe95, 0xbe9c, PDF_CMAP_TABLE, 650 },
	{ 0xbe9d, 0xbe9e, PDF_CMAP_RANGE, 17258 },
	{ 0xbe9f, 0xbea0, PDF_CMAP_TABLE, 658 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2256 },
	{ 0xbf40, 0xbf40, PDF_CMAP_SINGLE, 9894 },
	{ 0xbf41, 0xbf4c, PDF_CMAP_RANGE, 17261 },
	{ 0xbf4d, 0xbf50, PDF_CMAP_TABLE, 660 },
	{ 0xbf51, 0xbf54, PDF_CMAP_RANGE, 17273 },
	{ 0xbf55, 0xbf56, PDF_CMAP_TABLE, 664 },
	{ 0xbf57, 0xbf5f, PDF_CMAP_RANGE, 17277 },
	{ 0xbf60, 0xbf64, PDF_CMAP_TABLE, 666 },
	{ 0xbf65, 0xbf67, PDF_CMAP_RANGE, 17287 },
	{ 0xbf68, 0xbf68, PDF_CMAP_SINGLE, 8642 },
	{ 0xbf69, 0xbf6b, PDF_CMAP_RANGE, 17290 },
	{ 0xbf6c, 0xbf6c, PDF_CMAP_SINGLE, 8558 },
	{ 0xbf6d, 0xbf6f, PDF_CMAP_RANGE, 17293 },
	{ 0xbf70, 0xbf73, PDF_CMAP_TABLE, 671 },
	{ 0xbf74, 0xbf75, PDF_CMAP_RANGE, 17297 },
	{ 0xbf76, 0xbf7e, PDF_CMAP_TABLE, 675 },
	{ 0xbf80, 0xbf81, PDF_CMAP_RANGE, 17301 },
	{ 0xbf82, 0xbf83, PDF_CMAP_TABLE, 684 },
	{ 0xbf84, 0xbf88, PDF_CMAP_RANGE, 17303 },
	{ 0xbf89, 0xbf8a, PDF_CMAP_TABLE, 686 },
	{ 0xbf8b, 0xbf94, PDF_CMAP_RANGE, 17308 },
	{ 0xbf95, 0xbf98, PDF_CMAP_TABLE, 688 },
	{ 0xbf99, 0xbf9c, PDF_CMAP_RANGE, 17319 },
	{ 0xbf9d, 0xbf9d, PDF_CMAP_SINGLE, 9300 },
	{ 0xbf9e, 0xbfa0, PDF_CMAP_RANGE, 17323 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2350 },
	{ 0xc040, 0xc040, PDF_CMAP_SINGLE, 8443 },
	{ 0xc041, 0xc043, PDF_CMAP_RANGE, 17326 },
	{ 0xc044, 0xc044, PDF_CMAP_SINGLE, 9280 },
	{ 0xc045, 0xc04a, PDF_CMAP_RANGE, 17329 },
	{ 0xc04b, 0xc052, PDF_CMAP_TABLE, 692 },
	{ 0xc053, 0xc054, PDF_CMAP_RANGE, 17336 },
	{ 0xc055, 0xc055, PDF_CMAP_SINGLE, 8110 },
	{ 0xc056, 0xc05a, PDF_CMAP_RANGE, 17338 },
	{ 0xc05b, 0xc05b, PDF_CMAP_SINGLE, 8730 },
	{ 0xc05c, 0xc05d, PDF_CMAP_RANGE, 17343 },
	{ 0xc05e, 0xc060, PDF_CMAP_TABLE, 700 },
	{ 0xc061, 0xc068, PDF_CMAP_RANGE, 17345 },
	{ 0xc069, 0xc070, PDF_CMAP_TABLE, 703 },
	{ 0xc071, 0xc073, PDF_CMAP_RANGE, 17356 },
	{ 0xc074, 0xc079, PDF_CMAP_TABLE, 711 },
	{ 0xc07a, 0xc07b, PDF_CMAP_RANGE, 17361 },
	{ 0xc07c, 0xc07c, PDF_CMAP_SINGLE, 8191 },
	{ 0xc07d, 0xc07e, PDF_CMAP_RANGE, 17363 },
	{ 0xc080, 0xc09a, PDF_CMAP_RANGE, 17365 },
	{ 0xc09b, 0xc09d, PDF_CMAP_TABLE, 717 },
	{ 0xc09e, 0xc0a0, PDF_CMAP_RANGE, 17393 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2444 },
	{ 0xc140, 0xc14f, PDF_CMAP_RANGE, 17396 },
	{ 0xc150, 0xc150, PDF_CMAP_SINGLE, 7914 },
	{ 0xc151, 0xc153, PDF_CMAP_RANGE, 17412 },
	{ 0xc154, 0xc154, PDF_CMAP_SINGLE, 7724 },
	{ 0xc155, 0xc15e, PDF_CMAP_RANGE, 17415 },
	{ 0xc15f, 0xc162, PDF_CMAP_TABLE, 720 },
	{ 0xc163, 0xc174, PDF_CMAP_RANGE, 17426 },
	{ 0xc175, 0xc175, PDF_CMAP_SINGLE, 9714 },
	{ 0xc176, 0xc177, PDF_CMAP_RANGE, 17444 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 8725 },
	{ 0xc179, 0xc17e, PDF_CMAP_RANGE, 17446 },
	{ 0xc180, 0xc194, PDF_CMAP_RANGE, 17452 },
	{ 0xc195, 0xc195, PDF_CMAP_SINGLE, 8622 },
	{ 0xc196, 0xc1a0, PDF_CMAP_RANGE, 17473 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2538 },
	{ 0xc240, 0xc24d, PDF_CMAP_RANGE, 17484 },
	{ 0xc24e, 0xc24e, PDF_CMAP_SINGLE, 8416 },
	{ 0xc24f, 0xc264, PDF_CMAP_RANGE, 17498 },
	{ 0xc265, 0xc267, PDF_CMAP_TABLE, 724 },
	{ 0xc268, 0xc27c, PDF_CMAP_RANGE, 17521 },
	{ 0xc27d, 0xc27e, PDF_CMAP_TABLE, 727 },
	{ 0xc280, 0xc283, PDF_CMAP_RANGE, 17543 },
	{ 0xc284, 0xc284, PDF_CMAP_SINGLE, 8601 },
	{ 0xc285, 0xc292, PDF_CMAP_RANGE, 17547 },
	{ 0xc293, 0xc29c, PDF_CMAP_TABLE, 729 },
	{ 0xc29d, 0xc29f, PDF_CMAP_RANGE, 17563 },
	{ 0xc2a0, 0xc2a0, PDF_CMAP_SINGLE, 8569 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2632 },
	{ 0xc340, 0xc340, PDF_CMAP_SINGLE, 8248 },
	{ 0xc341, 0xc342, PDF_CMAP_RANGE, 17566 },
	{ 0xc343, 0xc343, PDF_CMAP_SINGLE, 8533 },
	{ 0xc344, 0xc37a, PDF_CMAP_RANGE, 17568 },
	{ 0xc37b, 0xc37b, PDF_CMAP_SINGLE, 8658 },
	{ 0xc37c, 0xc37e, PDF_CMAP_RANGE, 17623 },
	{ 0xc380, 0xc383, PDF_CMAP_RANGE, 17626 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 9415 },
	{ 0xc385, 0xc39a, PDF_CMAP_RANGE, 17630 },
	{ 0xc39b, 0xc39b, PDF_CMAP_SINGLE, 8825 },
	{ 0xc39c, 0xc3a0, PDF_CMAP_RANGE, 17652 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2726 },
	{ 0xc440, 0xc448, PDF_CMAP_RANGE, 17657 },
	{ 0xc449, 0xc449, PDF_CMAP_SINGLE, 8483 },
	{ 0xc44a, 0xc44b, PDF_CMAP_RANGE, 17666 },
	{ 0xc44c, 0xc44c, PDF_CMAP_SINGLE, 9413 },
	{ 0xc44d, 0xc453, PDF_CMAP_RANGE, 17668 },
	{ 0xc454, 0xc454, PDF_CMAP_SINGLE, 9417 },
	{ 0xc455, 0xc457, PDF_CMAP_RANGE, 17675 },
	{ 0xc458, 0xc458, PDF_CMAP_SINGLE, 8340 },
	{ 0xc459, 0xc45a, PDF_CMAP_RANGE, 17678 },
	{ 0xc45b, 0xc45b, PDF_CMAP_SINGLE, 8854 },
	{ 0xc45c, 0xc462, PDF_CMAP_RANGE, 17680 },
	{ 0xc463, 0xc463, PDF_CMAP_SINGLE, 7799 },
	{ 0xc464, 0xc476, PDF_CMAP_RANGE, 17687 },
	{ 0xc477, 0xc477, PDF_CMAP_SINGLE, 7942 },
	{ 0xc478, 0xc479, PDF_CMAP_RANGE, 17706 },
	{ 0xc47a, 0xc47a, PDF_CMAP_SINGLE, 8101 },
	{ 0xc47b, 0xc47e, PDF_CMAP_RANGE, 17708 },
	{ 0xc480, 0xc481, PDF_CMAP_TABLE, 739 },
	{ 0xc482, 0xc490, PDF_CMAP_RANGE, 17713 },
	{ 0xc491, 0xc493, PDF_CMAP_TABLE, 741 },
	{ 0xc494, 0xc497, PDF_CMAP_RANGE, 17728 },
	{ 0xc498, 0xc49c, PDF_CMAP_TABLE, 744 },
	{ 0xc49d, 0xc4a0, PDF_CMAP_RANGE, 17734 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2820 },
	{ 0xc540, 0xc543, PDF_CMAP_RANGE, 17738 },
	{ 0xc544, 0xc546, PDF_CMAP_TABLE, 749 },
	{ 0xc547, 0xc54a, PDF_CMAP_RANGE, 17743 },
	{ 0xc54b, 0xc54c, PDF_CMAP_TABLE, 752 },
	{ 0xc54d, 0xc551, PDF_CMAP_RANGE, 17747 },
	{ 0xc552, 0xc552, PDF_CMAP_SINGLE, 8236 },
	{ 0xc553, 0xc55e, PDF_CMAP_RANGE, 17752 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 8545 },
	{ 0xc560, 0xc562, PDF_CMAP_RANGE, 17764 },
	{ 0xc563, 0xc566, PDF_CMAP_TABLE, 754 },
	{ 0xc567, 0xc57e, PDF_CMAP_RANGE, 17767 },
	{ 0xc580, 0xc592, PDF_CMAP_RANGE, 17791 },
	{ 0xc593, 0xc593, PDF_CMAP_SINGLE, 7777 },
	{ 0xc594, 0xc59b, PDF_CMAP_RANGE, 17810 },
	{ 0xc59c, 0xc59e, PDF_CMAP_TABLE, 758 },
	{ 0xc59f, 0xc5a0, PDF_CMAP_RANGE, 17819 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 2914 },
	{ 0xc640, 0xc641, PDF_CMAP_TABLE, 761 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 17822 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 8072 },
	{ 0xc645, 0xc646, PDF_CMAP_RANGE, 17824 },
	{ 0xc647, 0xc647, PDF_CMAP_SINGLE, 8696 },
	{ 0xc648, 0xc662, PDF_CMAP_RANGE, 17826 },
	{ 0xc663, 0xc663, PDF_CMAP_SINGLE, 8994 },
	{ 0xc664, 0xc671, PDF_CMAP_RANGE, 17853 },
	{ 0xc672, 0xc672, PDF_CMAP_SINGLE, 9014 },
	{ 0xc673, 0xc67e, PDF_CMAP_RANGE, 17867 },
	{ 0xc680, 0xc6a0, PDF_CMAP_RANGE, 17879 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3008 },
	{ 0xc740, 0xc765, PDF_CMAP_RANGE, 17912 },
	{ 0xc766, 0xc766, PDF_CMAP_SINGLE, 8875 },
	{ 0xc767, 0xc76e, PDF_CMAP_RANGE, 17950 },
	{ 0xc76f, 0xc76f, PDF_CMAP_SINGLE, 8127 },
	{ 0xc770, 0xc775, PDF_CMAP_RANGE, 17958 },
	{ 0xc776, 0xc776, PDF_CMAP_SINGLE, 8061 },
	{ 0xc777, 0xc77a, PDF_CMAP_RANGE, 17964 },
	{ 0xc77b, 0xc77b, PDF_CMAP_SINGLE, 9011 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 17968 },
	{ 0xc780, 0xc7a0, PDF_CMAP_RANGE, 17971 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3102 },
	{ 0xc840, 0xc841, PDF_CMAP_TABLE, 763 },
	{ 0xc842, 0xc84e, PDF_CMAP_RANGE, 18005 },
	{ 0xc84f, 0xc84f, PDF_CMAP_SINGLE, 9012 },
	{ 0xc850, 0xc851, PDF_CMAP_RANGE, 18018 },
	{ 0xc852, 0xc852, PDF_CMAP_SINGLE, 8177 },
	{ 0xc853, 0xc865, PDF_CMAP_RANGE, 18020 },
	{ 0xc866, 0xc866, PDF_CMAP_SINGLE, 8587 },
	{ 0xc867, 0xc86d, PDF_CMAP_RANGE, 18039 },
	{ 0xc86e, 0xc86e, PDF_CMAP_SINGLE, 9033 },
	{ 0xc86f, 0xc87d, PDF_CMAP_RANGE, 18046 },
	{ 0xc87e, 0xc87e, PDF_CMAP_SINGLE, 8715 },
	{ 0xc880, 0xc886, PDF_CMAP_RANGE, 18061 },
	{ 0xc887, 0xc887, PDF_CMAP_SINGLE, 9030 },
	{ 0xc888, 0xc891, PDF_CMAP_RANGE, 18068 },
	{ 0xc892, 0xc894, PDF_CMAP_TABLE, 765 },
	{ 0xc895, 0xc898, PDF_CMAP_RANGE, 18079 },
	{ 0xc899, 0xc899, PDF_CMAP_SINGLE, 9889 },
	{ 0xc89a, 0xc89c, PDF_CMAP_RANGE, 18083 },
	{ 0xc89d, 0xc89d, PDF_CMAP_SINGLE, 8034 },
	{ 0xc89e, 0xc8a0, PDF_CMAP_RANGE, 18086 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3196 },
	{ 0xc940, 0xc94e, PDF_CMAP_RANGE, 18089 },
	{ 0xc94f, 0xc950, PDF_CMAP_TABLE, 768 },
	{ 0xc951, 0xc96d, PDF_CMAP_RANGE, 18104 },
	{ 0xc96e, 0xc970, PDF_CMAP_TABLE, 770 },
	{ 0xc971, 0xc976, PDF_CMAP_RANGE, 18134 },
	{ 0xc977, 0xc977, PDF_CMAP_SINGLE, 7954 },
	{ 0xc978, 0xc97e, PDF_CMAP_RANGE, 18140 },
	{ 0xc980, 0xc98e, PDF_CMAP_RANGE, 18147 },
	{ 0xc98f, 0xc990, PDF_CMAP_TABLE, 773 },
	{ 0xc991, 0xc99b, PDF_CMAP_RANGE, 18162 },
	{ 0xc99c, 0xc99c, PDF_CMAP_SINGLE, 9020 },
	{ 0xc99d, 0xc9a0, PDF_CMAP_RANGE, 18173 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 3290 },
	{ 0xca40, 0xca4d, PDF_CMAP_RANGE, 18177 },
	{ 0xca4e, 0xca4e, PDF_CMAP_SINGLE, 9839 },
	{ 0xca4f, 0xca55, PDF_CMAP_RANGE, 18191 },
	{ 0xca56, 0xca56, PDF_CMAP_SINGLE, 9041 },
	{ 0xca57, 0xca58, PDF_CMAP_RANGE, 18198 },
	{ 0xca59, 0xca59, PDF_CMAP_SINGLE, 8096 },
	{ 0xca5a, 0xca5b, PDF_CMAP_RANGE, 18200 },
	{ 0xca5c, 0xca5c, PDF_CMAP_SINGLE, 9016 },
	{ 0xca5d, 0xca60, PDF_CMAP_RANGE, 18202 },
	{ 0xca61, 0xca61, PDF_CMAP_SINGLE, 8731 },
	{ 0xca62, 0xca6d, PDF_CMAP_RANGE, 18206 },
	{ 0xca6e, 0xca6e, PDF_CMAP_SINGLE, 9026 },
	{ 0xca6f, 0xca71, PDF_CMAP_RANGE, 18218 },
	{ 0xca72, 0xca72, PDF_CMAP_SINGLE, 9039 },
	{ 0xca73, 0xca76, PDF_CMAP_RANGE, 18221 },
	{ 0xca77, 0xca77, PDF_CMAP_SINGLE, 9021 },
	{ 0xca78, 0xca7a, PDF_CMAP_RANGE, 18225 },
	{ 0xca7b, 0xca7e, PDF_CMAP_TABLE, 775 },
	{ 0xca80, 0xca81, PDF_CMAP_TABLE, 779 },
	{ 0xca82, 0xca88, PDF_CMAP_RANGE, 18230 },
	{ 0xca89, 0xca89, PDF_CMAP_SINGLE, 9040 },
	{ 0xca8a, 0xca8d, PDF_CMAP_RANGE, 18237 },
	{ 0xca8e, 0xca8f, PDF_CMAP_TABLE, 781 },
	{ 0xca90, 0xca91, PDF_CMAP_RANGE, 18241 },
	{ 0xca92, 0xca92, PDF_CMAP_SINGLE, 8651 },
	{ 0xca93, 0xca99, PDF_CMAP_RANGE, 18243 },
	{ 0xca9a, 0xca9a, PDF_CMAP_SINGLE, 9045 },
	{ 0xca9b, 0xcaa0, PDF_CMAP_RANGE, 18250 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3384 },
	{ 0xcb40, 0xcb42, PDF_CMAP_RANGE, 18256 },
	{ 0xcb43, 0xcb47, PDF_CMAP_TABLE, 783 },
	{ 0xcb48, 0xcb4a, PDF_CMAP_RANGE, 18261 },
	{ 0xcb4b, 0xcb4b, PDF_CMAP_SINGLE, 9856 },
	{ 0xcb4c, 0xcb4d, PDF_CMAP_RANGE, 18264 },
	{ 0xcb4e, 0xcb4e, PDF_CMAP_SINGLE, 8410 },
	{ 0xcb4f, 0xcb56, PDF_CMAP_RANGE, 18266 },
	{ 0xcb57, 0xcb57, PDF_CMAP_SINGLE, 9034 },
	{ 0xcb58, 0xcb5c, PDF_CMAP_RANGE, 18274 },
	{ 0xcb5d, 0xcb5f, PDF_CMAP_TABLE, 788 },
	{ 0xcb60, 0xcb69, PDF_CMAP_RANGE, 18280 },
	{ 0xcb6a, 0xcb6a, PDF_CMAP_SINGLE, 9023 },
	{ 0xcb6b, 0xcb7a, PDF_CMAP_RANGE, 18290 },
	{ 0xcb7b, 0xcb7c, PDF_CMAP_TABLE, 791 },
	{ 0xcb7d, 0xcb7e, PDF_CMAP_RANGE, 18306 },
	{ 0xcb80, 0xcb86, PDF_CMAP_RANGE, 18308 },
	{ 0xcb87, 0xcb87, PDF_CMAP_SINGLE, 8722 },
	{ 0xcb88, 0xcb8d, PDF_CMAP_RANGE, 18315 },
	{ 0xcb8e, 0xcb8e, PDF_CMAP_SINGLE, 8711 },
	{ 0xcb8f, 0xcb91, PDF_CMAP_RANGE, 18321 },
	{ 0xcb92, 0xcb92, PDF_CMAP_SINGLE, 9049 },
	{ 0xcb93, 0xcb9b, PDF_CMAP_RANGE, 18324 },
	{ 0xcb9c, 0xcb9e, PDF_CMAP_TABLE, 793 },
	{ 0xcb9f, 0xcba0, PDF_CMAP_RANGE, 18334 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 3478 },
	{ 0xcc40, 0xcc41, PDF_CMAP_TABLE, 796 },
	{ 0xcc42, 0xcc48, PDF_CMAP_RANGE, 18336 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_TABLE, 798 },
	{ 0xcc4c, 0xcc4e, PDF_CMAP_RANGE, 18343 },
	{ 0xcc4f, 0xcc4f, PDF_CMAP_SINGLE, 8379 },
	{ 0xcc50, 0xcc5b, PDF_CMAP_RANGE, 18346 },
	{ 0xcc5c, 0xcc5c, PDF_CMAP_SINGLE, 9050 },
	{ 0xcc5d, 0xcc5f, PDF_CMAP_RANGE, 18358 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 9046 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 18361 },
	{ 0xcc64, 0xcc64, PDF_CMAP_SINGLE, 9015 },
	{ 0xcc65, 0xcc6c, PDF_CMAP_RANGE, 18364 },
	{ 0xcc6d, 0xcc6d, PDF_CMAP_SINGLE, 8185 },
	{ 0xcc6e, 0xcc78, PDF_CMAP_RANGE, 18372 },
	{ 0xcc79, 0xcc79, PDF_CMAP_SINGLE, 9043 },
	{ 0xcc7a, 0xcc7c, PDF_CMAP_RANGE, 18383 },
	{ 0xcc7d, 0xcc7e, PDF_CMAP_TABLE, 801 },
	{ 0xcc80, 0xcc8d, PDF_CMAP_RANGE, 18387 },
	{ 0xcc8e, 0xcc8e, PDF_CMAP_SINGLE, 7829 },
	{ 0xcc8f, 0xcc93, PDF_CMAP_RANGE, 18401 },
	{ 0xcc94, 0xcc96, PDF_CMAP_TABLE, 803 },
	{ 0xcc97, 0xcc9c, PDF_CMAP_RANGE, 18407 },
	{ 0xcc9d, 0xcc9d, PDF_CMAP_SINGLE, 8168 },
	{ 0xcc9e, 0xcca0, PDF_CMAP_RANGE, 18413 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3572 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 18416 },
	{ 0xcd80, 0xcd8f, PDF_CMAP_RANGE, 18479 },
	{ 0xcd90, 0xcd90, PDF_CMAP_SINGLE, 9690 },
	{ 0xcd91, 0xcd97, PDF_CMAP_RANGE, 18495 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 9687 },
	{ 0xcd99, 0xcda0, PDF_CMAP_RANGE, 18502 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 3666 },
	{ 0xce40, 0xce66, PDF_CMAP_RANGE, 18510 },
	{ 0xce67, 0xce67, PDF_CMAP_SINGLE, 8494 },
	{ 0xce68, 0xce71, PDF_CMAP_RANGE, 18549 },
	{ 0xce72, 0xce72, PDF_CMAP_SINGLE, 8626 },
	{ 0xce73, 0xce7e, PDF_CMAP_RANGE, 18559 },
	{ 0xce80, 0xce81, PDF_CMAP_TABLE, 806 },
	{ 0xce82, 0xce86, PDF_CMAP_RANGE, 18572 },
	{ 0xce87, 0xce87, PDF_CMAP_SINGLE, 9692 },
	{ 0xce88, 0xce9a, PDF_CMAP_RANGE, 18577 },
	{ 0xce9b, 0xce9b, PDF_CMAP_SINGLE, 8300 },
	{ 0xce9c, 0xce9d, PDF_CMAP_RANGE, 18596 },
	{ 0xce9e, 0xce9e, PDF_CMAP_SINGLE, 8742 },
	{ 0xce9f, 0xcea0, PDF_CMAP_RANGE, 18598 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3760 },
	{ 0xcf40, 0xcf4d, PDF_CMAP_RANGE, 18600 },
	{ 0xcf4e, 0xcf4e, PDF_CMAP_SINGLE, 9696 },
	{ 0xcf4f, 0xcf54, PDF_CMAP_RANGE, 18614 },
	{ 0xcf55, 0xcf55, PDF_CMAP_SINGLE, 8827 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 18620 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 9694 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 18622 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 9697 },
	{ 0xcf5d, 0xcf6b, PDF_CMAP_RANGE, 18625 },
	{ 0xcf6c, 0xcf6c, PDF_CMAP_SINGLE, 9685 },
	{ 0xcf6d, 0xcf72, PDF_CMAP_RANGE, 18640 },
	{ 0xcf73, 0xcf75, PDF_CMAP_TABLE, 808 },
	{ 0xcf76, 0xcf77, PDF_CMAP_RANGE, 18647 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 7817 },
	{ 0xcf79, 0xcf7b, PDF_CMAP_RANGE, 18649 },
	{ 0xcf7c, 0xcf7c, PDF_CMAP_SINGLE, 9689 },
	{ 0xcf7d, 0xcf7e, PDF_CMAP_RANGE, 18652 },
	{ 0xcf80, 0xcf81, PDF_CMAP_TABLE, 811 },
	{ 0xcf82, 0xcf88, PDF_CMAP_RANGE, 18655 },
	{ 0xcf89, 0xcf8a, PDF_CMAP_TABLE, 813 },
	{ 0xcf8b, 0xcf92, PDF_CMAP_RANGE, 18662 },
	{ 0xcf93, 0xcf94, PDF_CMAP_TABLE, 815 },
	{ 0xcf95, 0xcf9d, PDF_CMAP_RANGE, 18670 },
	{ 0xcf9e, 0xcfa0, PDF_CMAP_TABLE, 817 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 3854 },
	{ 0xd040, 0xd04c, PDF_CMAP_RANGE, 18680 },
	{ 0xd04d, 0xd04d, PDF_CMAP_SINGLE, 7977 },
	{ 0xd04e, 0xd050, PDF_CMAP_RANGE, 18693 },
	{ 0xd051, 0xd051, PDF_CMAP_SINGLE, 7771 },
	{ 0xd052, 0xd054, PDF_CMAP_RANGE, 18696 },
	{ 0xd055, 0xd055, PDF_CMAP_SINGLE, 8310 },
	{ 0xd056, 0xd05b, PDF_CMAP_RANGE, 18699 },
	{ 0xd05c, 0xd05c, PDF_CMAP_SINGLE, 8855 },
	{ 0xd05d, 0xd05f, PDF_CMAP_RANGE, 18705 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 9872 },
	{ 0xd061, 0xd066, PDF_CMAP_RANGE, 18708 },
	{ 0xd067, 0xd067, PDF_CMAP_SINGLE, 8511 },
	{ 0xd068, 0xd06b, PDF_CMAP_RANGE, 18714 },
	{ 0xd06c, 0xd06e, PDF_CMAP_TABLE, 820 },
	{ 0xd06f, 0xd07c, PDF_CMAP_RANGE, 18719 },
	{ 0xd07d, 0xd07e, PDF_CMAP_TABLE, 823 },
	{ 0xd080, 0xd0a0, PDF_CMAP_RANGE, 18734 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 3948 },
	{ 0xd140, 0xd154, PDF_CMAP_RANGE, 18767 },
	{ 0xd155, 0xd155, PDF_CMAP_SINGLE, 9713 },
	{ 0xd156, 0xd158, PDF_CMAP_RANGE, 18788 },
	{ 0xd159, 0xd159, PDF_CMAP_SINGLE, 8204 },
	{ 0xd15a, 0xd160, PDF_CMAP_RANGE, 18791 },
	{ 0xd161, 0xd162, PDF_CMAP_TABLE, 825 },
	{ 0xd163, 0xd174, PDF_CMAP_RANGE, 18798 },
	{ 0xd175, 0xd175, PDF_CMAP_SINGLE, 9895 },
	{ 0xd176, 0xd17c, PDF_CMAP_RANGE, 18816 },
	{ 0xd17d, 0xd17e, PDF_CMAP_TABLE, 827 },
	{ 0xd180, 0xd19c, PDF_CMAP_RANGE, 18824 },
	{ 0xd19d, 0xd19e, PDF_CMAP_TABLE, 829 },
	{ 0xd19f, 0xd1a0, PDF_CMAP_RANGE, 18853 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 4042 },
	{ 0xd240, 0xd240, PDF_CMAP_SINGLE, 9662 },
	{ 0xd241, 0xd242, PDF_CMAP_RANGE, 18855 },
	{ 0xd243, 0xd243, PDF_CMAP_SINGLE, 8925 },
	{ 0xd244, 0xd24c, PDF_CMAP_RANGE, 18857 },
	{ 0xd24d, 0xd24d, PDF_CMAP_SINGLE, 9661 },
	{ 0xd24e, 0xd25b, PDF_CMAP_RANGE, 18866 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 7722 },
	{ 0xd25d, 0xd262, PDF_CMAP_RANGE, 18880 },
	{ 0xd263, 0xd264, PDF_CMAP_TABLE, 831 },
	{ 0xd265, 0xd267, PDF_CMAP_RANGE, 18886 },
	{ 0xd268, 0xd268, PDF_CMAP_SINGLE, 9663 },
	{ 0xd269, 0xd26c, PDF_CMAP_RANGE, 18889 },
	{ 0xd26d, 0xd26f, PDF_CMAP_TABLE, 833 },
	{ 0xd270, 0xd271, PDF_CMAP_RANGE, 18894 },
	{ 0xd272, 0xd272, PDF_CMAP_SINGLE, 7807 },
	{ 0xd273, 0xd274, PDF_CMAP_RANGE, 18896 },
	{ 0xd275, 0xd275, PDF_CMAP_SINGLE, 8621 },
	{ 0xd276, 0xd27e, PDF_CMAP_RANGE, 18898 },
	{ 0xd280, 0xd289, PDF_CMAP_RANGE, 18907 },
	{ 0xd28a, 0xd28a, PDF_CMAP_SINGLE, 8086 },
	{ 0xd28b, 0xd28d, PDF_CMAP_RANGE, 18917 },
	{ 0xd28e, 0xd28e, PDF_CMAP_SINGLE, 7986 },
	{ 0xd28f, 0xd291, PDF_CMAP_RANGE, 18920 },
	{ 0xd292, 0xd292, PDF_CMAP_SINGLE, 8324 },
	{ 0xd293, 0xd294, PDF_CMAP_RANGE, 18923 },
	{ 0xd295, 0xd297, PDF_CMAP_TABLE, 836 },
	{ 0xd298, 0xd29f, PDF_CMAP_RANGE, 18926 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 9402 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4136 },
	{ 0xd340, 0xd343, PDF_CMAP_RANGE, 18934 },
	{ 0xd344, 0xd344, PDF_CMAP_SINGLE, 9404 },
	{ 0xd345, 0xd347, PDF_CMAP_RANGE, 18938 },
	{ 0xd348, 0xd34a, PDF_CMAP_TABLE, 839 },
	{ 0xd34b, 0xd34c, PDF_CMAP_RANGE, 18942 },
	{ 0xd34d, 0xd34d, PDF_CMAP_SINGLE, 9405 },
	{ 0xd34e, 0xd34f, PDF_CMAP_RANGE, 18944 },
	{ 0xd350, 0xd350, PDF_CMAP_SINGLE, 9406 },
	{ 0xd351, 0xd354, PDF_CMAP_RANGE, 18946 },
	{ 0xd355, 0xd355, PDF_CMAP_SINGLE, 9407 },
	{ 0xd356, 0xd357, PDF_CMAP_RANGE, 18950 },
	{ 0xd358, 0xd358, PDF_CMAP_SINGLE, 8146 },
	{ 0xd359, 0xd35a, PDF_CMAP_RANGE, 18952 },
	{ 0xd35b, 0xd35e, PDF_CMAP_TABLE, 842 },
	{ 0xd35f, 0xd377, PDF_CMAP_RANGE, 18955 },
	{ 0xd378, 0xd37c, PDF_CMAP_TABLE, 846 },
	{ 0xd37d, 0xd37e, PDF_CMAP_RANGE, 18982 },
	{ 0xd380, 0xd384, PDF_CMAP_RANGE, 18984 },
	{ 0xd385, 0xd387, PDF_CMAP_TABLE, 851 },
	{ 0xd388, 0xd38a, PDF_CMAP_RANGE, 18989 },
	{ 0xd38b, 0xd393, PDF_CMAP_TABLE, 854 },
	{ 0xd394, 0xd395, PDF_CMAP_RANGE, 18996 },
	{ 0xd396, 0xd39b, PDF_CMAP_TABLE, 863 },
	{ 0xd39c, 0xd39d, PDF_CMAP_RANGE, 19000 },
	{ 0xd39e, 0xd3a0, PDF_CMAP_TABLE, 869 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4230 },
	{ 0xd440, 0xd441, PDF_CMAP_TABLE, 872 },
	{ 0xd442, 0xd444, PDF_CMAP_RANGE, 19004 },
	{ 0xd445, 0xd447, PDF_CMAP_TABLE, 874 },
	{ 0xd448, 0xd44b, PDF_CMAP_RANGE, 19008 },
	{ 0xd44c, 0xd44c, PDF_CMAP_SINGLE, 7922 },
	{ 0xd44d, 0xd44e, PDF_CMAP_RANGE, 19012 },
	{ 0xd44f, 0xd44f, PDF_CMAP_SINGLE, 8479 },
	{ 0xd450, 0xd452, PDF_CMAP_RANGE, 19014 },
	{ 0xd453, 0xd453, PDF_CMAP_SINGLE, 8669 },
	{ 0xd454, 0xd455, PDF_CMAP_RANGE, 19017 },
	{ 0xd456, 0xd458, PDF_CMAP_TABLE, 877 },
	{ 0xd459, 0xd45b, PDF_CMAP_RANGE, 19020 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 8834 },
	{ 0xd45d, 0xd461, PDF_CMAP_RANGE, 19023 },
	{ 0xd462, 0xd462, PDF_CMAP_SINGLE, 8934 },
	{ 0xd463, 0xd466, PDF_CMAP_RANGE, 19028 },
	{ 0xd467, 0xd467, PDF_CMAP_SINGLE, 8936 },
	{ 0xd468, 0xd46d, PDF_CMAP_RANGE, 19032 },
	{ 0xd46e, 0xd475, PDF_CMAP_TABLE, 880 },
	{ 0xd476, 0xd477, PDF_CMAP_RANGE, 19041 },
	{ 0xd478, 0xd478, PDF_CMAP_SINGLE, 8938 },
	{ 0xd479, 0xd47a, PDF_CMAP_RANGE, 19043 },
	{ 0xd47b, 0xd47b, PDF_CMAP_SINGLE, 8893 },
	{ 0xd47c, 0xd47d, PDF_CMAP_RANGE, 19045 },
	{ 0xd47e, 0xd47e, PDF_CMAP_SINGLE, 7838 },
	{ 0xd480, 0xd481, PDF_CMAP_RANGE, 19047 },
	{ 0xd482, 0xd484, PDF_CMAP_TABLE, 888 },
	{ 0xd485, 0xd486, PDF_CMAP_RANGE, 19049 },
	{ 0xd487, 0xd487, PDF_CMAP_SINGLE, 8503 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 19051 },
	{ 0xd48a, 0xd496, PDF_CMAP_TABLE, 891 },
	{ 0xd497, 0xd49b, PDF_CMAP_RANGE, 19056 },
	{ 0xd49c, 0xd49c, PDF_CMAP_SINGLE, 8944 },
	{ 0xd49d, 0xd49e, PDF_CMAP_RANGE, 19061 },
	{ 0xd49f, 0xd4a0, PDF_CMAP_TABLE, 904 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 4324 },
	{ 0xd540, 0xd542, PDF_CMAP_RANGE, 19064 },
	{ 0xd543, 0xd546, PDF_CMAP_TABLE, 906 },
	{ 0xd547, 0xd549, PDF_CMAP_RANGE, 19067 },
	{ 0xd54a, 0xd54a, PDF_CMAP_SINGLE, 8446 },
	{ 0xd54b, 0xd54d, PDF_CMAP_RANGE, 19070 },
	{ 0xd54e, 0xd54f, PDF_CMAP_RANGE, 8953 },
	{ 0xd550, 0xd551, PDF_CMAP_TABLE, 910 },
	{ 0xd552, 0xd553, PDF_CMAP_RANGE, 19074 },
	{ 0xd554, 0xd556, PDF_CMAP_TABLE, 912 },
	{ 0xd557, 0xd559, PDF_CMAP_RANGE, 19077 },
	{ 0xd55a, 0xd564, PDF_CMAP_TABLE, 915 },
	{ 0xd565, 0xd567, PDF_CMAP_RANGE, 19083 },
	{ 0xd568, 0xd568, PDF_CMAP_SINGLE, 8519 },
	{ 0xd569, 0xd56b, PDF_CMAP_RANGE, 19086 },
	{ 0xd56c, 0xd56e, PDF_CMAP_TABLE, 926 },
	{ 0xd56f, 0xd571, PDF_CMAP_RANGE, 19090 },
	{ 0xd572, 0xd572, PDF_CMAP_SINGLE, 8961 },
	{ 0xd573, 0xd574, PDF_CMAP_RANGE, 19093 },
	{ 0xd575, 0xd575, PDF_CMAP_SINGLE, 7925 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 19095 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 8728 },
	{ 0xd579, 0xd57a, PDF_CMAP_RANGE, 19097 },
	{ 0xd57b, 0xd57b, PDF_CMAP_SINGLE, 7878 },
	{ 0xd57c, 0xd57d, PDF_CMAP_RANGE, 19099 },
	{ 0xd57e, 0xd57e, PDF_CMAP_SINGLE, 8960 },
	{ 0xd580, 0xd581, PDF_CMAP_TABLE, 929 },
	{ 0xd582, 0xd583, PDF_CMAP_RANGE, 19102 },
	{ 0xd584, 0xd58f, PDF_CMAP_TABLE, 931 },
	{ 0xd590, 0xd592, PDF_CMAP_RANGE, 19109 },
	{ 0xd593, 0xd594, PDF_CMAP_TABLE, 943 },
	{ 0xd595, 0xd597, PDF_CMAP_RANGE, 19112 },
	{ 0xd598, 0xd59b, PDF_CMAP_TABLE, 945 },
	{ 0xd59c, 0xd59e, PDF_CMAP_RANGE, 19116 },
	{ 0xd59f, 0xd5a0, PDF_CMAP_TABLE, 949 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 4418 },
	{ 0xd640, 0xd643, PDF_CMAP_TABLE, 951 },
	{ 0xd644, 0xd646, PDF_CMAP_RANGE, 19121 },
	{ 0xd647, 0xd64a, PDF_CMAP_TABLE, 955 },
	{ 0xd64b, 0xd64c, PDF_CMAP_RANGE, 19125 },
	{ 0xd64d, 0xd64f, PDF_CMAP_TABLE, 959 },
	{ 0xd650, 0xd651, PDF_CMAP_RANGE, 19128 },
	{ 0xd652, 0xd661, PDF_CMAP_TABLE, 962 },
	{ 0xd662, 0xd664, PDF_CMAP_RANGE, 19135 },
	{ 0xd665, 0xd665, PDF_CMAP_SINGLE, 8023 },
	{ 0xd666, 0xd668, PDF_CMAP_RANGE, 19138 },
	{ 0xd669, 0xd66b, PDF_CMAP_TABLE, 978 },
	{ 0xd66c, 0xd66e, PDF_CMAP_RANGE, 19142 },
	{ 0xd66f, 0xd678, PDF_CMAP_TABLE, 981 },
	{ 0xd679, 0xd67b, PDF_CMAP_RANGE, 19148 },
	{ 0xd67c, 0xd67c, PDF_CMAP_SINGLE, 8710 },
	{ 0xd67d, 0xd67e, PDF_CMAP_RANGE, 19151 },
	{ 0xd680, 0xd682, PDF_CMAP_RANGE, 19153 },
	{ 0xd683, 0xd683, PDF_CMAP_SINGLE, 8973 },
	{ 0xd684, 0xd685, PDF_CMAP_RANGE, 19156 },
	{ 0xd686, 0xd688, PDF_CMAP_TABLE, 991 },
	{ 0xd689, 0xd68d, PDF_CMAP_RANGE, 19158 },
	{ 0xd68e, 0xd68e, PDF_CMAP_SINGLE, 8931 },
	{ 0xd68f, 0xd693, PDF_CMAP_RANGE, 19163 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 8122 },
	{ 0xd695, 0xd698, PDF_CMAP_RANGE, 19168 },
	{ 0xd699, 0xd699, PDF_CMAP_SINGLE, 8312 },
	{ 0xd69a, 0xd6a0, PDF_CMAP_RANGE, 19172 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd740, 0xd742, PDF_CMAP_RANGE, 19179 },
	{ 0xd743, 0xd743, PDF_CMAP_SINGLE, 8840 },
	{ 0xd744, 0xd747, PDF_CMAP_RANGE, 19182 },
	{ 0xd748, 0xd749, PDF_CMAP_TABLE, 994 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 19186 },
	{ 0xd750, 0xd756, PDF_CMAP_TABLE, 996 },
	{ 0xd757, 0xd763, PDF_CMAP_RANGE, 19194 },
	{ 0xd764, 0xd764, PDF_CMAP_SINGLE, 8984 },
	{ 0xd765, 0xd766, PDF_CMAP_RANGE, 19207 },
	{ 0xd767, 0xd768, PDF_CMAP_TABLE, 1003 },
	{ 0xd769, 0xd76b, PDF_CMAP_RANGE, 19209 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 8405 },
	{ 0xd76d, 0xd76e, PDF_CMAP_RANGE, 19212 },
	{ 0xd76f, 0xd76f, PDF_CMAP_SINGLE, 8010 },
	{ 0xd770, 0xd774, PDF_CMAP_RANGE, 19214 },
	{ 0xd775, 0xd775, PDF_CMAP_SINGLE, 8768 },
	{ 0xd776, 0xd777, PDF_CMAP_RANGE, 19219 },
	{ 0xd778, 0xd778, PDF_CMAP_SINGLE, 7891 },
	{ 0xd779, 0xd77e, PDF_CMAP_RANGE, 19221 },
	{ 0xd780, 0xd782, PDF_CMAP_RANGE, 19227 },
	{ 0xd783, 0xd783, PDF_CMAP_SINGLE, 7753 },
	{ 0xd784, 0xd786, PDF_CMAP_RANGE, 19230 },
	{ 0xd787, 0xd787, PDF_CMAP_SINGLE, 9755 },
	{ 0xd788, 0xd78a, PDF_CMAP_RANGE, 19233 },
	{ 0xd78b, 0xd78f, PDF_CMAP_TABLE, 1005 },
	{ 0xd790, 0xd794, PDF_CMAP_RANGE, 19237 },
	{ 0xd795, 0xd797, PDF_CMAP_TABLE, 1010 },
	{ 0xd798, 0xd7a0, PDF_CMAP_RANGE, 19243 },
	{ 0xd7a1, 0xd7f9, PDF_CMAP_RANGE, 4606 },
	{ 0xd840, 0xd84c, PDF_CMAP_RANGE, 19252 },
	{ 0xd84d, 0xd84d, PDF_CMAP_SINGLE, 8392 },
	{ 0xd84e, 0xd852, PDF_CMAP_RANGE, 19265 },
	{ 0xd853, 0xd853, PDF_CMAP_SINGLE, 7933 },
	{ 0xd854, 0xd87e, PDF_CMAP_RANGE, 19270 },
	{ 0xd880, 0xd88f, PDF_CMAP_RANGE, 19313 },
	{ 0xd890, 0xd895, PDF_CMAP_TABLE, 1013 },
	{ 0xd896, 0xd899, PDF_CMAP_RANGE, 19330 },
	{ 0xd89a, 0xd8a0, PDF_CMAP_TABLE, 1019 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4695 },
	{ 0xd940, 0xd95c, PDF_CMAP_TABLE, 1026 },
	{ 0xd95d, 0xd962, PDF_CMAP_RANGE, 19342 },
	{ 0xd963, 0xd967, PDF_CMAP_TABLE, 1055 },
	{ 0xd968, 0xd96b, PDF_CMAP_RANGE, 19349 },
	{ 0xd96c, 0xd979, PDF_CMAP_TABLE, 1060 },
	{ 0xd97a, 0xd97b, PDF_CMAP_RANGE, 19357 },
	{ 0xd97c, 0xd97e, PDF_CMAP_TABLE, 1074 },
	{ 0xd980, 0xd980, PDF_CMAP_SINGLE, 7892 },
	{ 0xd981, 0xd986, PDF_CMAP_RANGE, 19359 },
	{ 0xd987, 0xd987, PDF_CMAP_SINGLE, 8179 },
	{ 0xd988, 0xd98c, PDF_CMAP_RANGE, 19365 },
	{ 0xd98d, 0xd991, PDF_CMAP_TABLE, 1077 },
	{ 0xd992, 0xd996, PDF_CMAP_RANGE, 19370 },
	{ 0xd997, 0xd998, PDF_CMAP_TABLE, 1082 },
	{ 0xd999, 0xd99a, PDF_CMAP_RANGE, 19375 },
	{ 0xd99b, 0xd9a0, PDF_CMAP_TABLE, 1084 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4789 },
	{ 0xda40, 0xda42, PDF_CMAP_TABLE, 1090 },
	{ 0xda43, 0xda47, PDF_CMAP_RANGE, 19380 },
	{ 0xda48, 0xda48, PDF_CMAP_SINGLE, 8509 },
	{ 0xda49, 0xda4c, PDF_CMAP_RANGE, 19385 },
	{ 0xda4d, 0xda4e, PDF_CMAP_TABLE, 1093 },
	{ 0xda4f, 0xda72, PDF_CMAP_RANGE, 19389 },
	{ 0xda73, 0xda73, PDF_CMAP_SINGLE, 7956 },
	{ 0xda74, 0xda76, PDF_CMAP_RANGE, 19425 },
	{ 0xda77, 0xda77, PDF_CMAP_SINGLE, 8826 },
	{ 0xda78, 0xda7e, PDF_CMAP_RANGE, 19428 },
	{ 0xda80, 0xda84, PDF_CMAP_RANGE, 19435 },
	{ 0xda85, 0xda85, PDF_CMAP_SINGLE, 8430 },
	{ 0xda86, 0xda8d, PDF_CMAP_RANGE, 19440 },
	{ 0xda8e, 0xda8e, PDF_CMAP_SINGLE, 9720 },
	{ 0xda8f, 0xdaa0, PDF_CMAP_RANGE, 19448 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4883 },
	{ 0xdb40, 0xdb5f, PDF_CMAP_RANGE, 19466 },
	{ 0xdb60, 0xdb60, PDF_CMAP_SINGLE, 8084 },
	{ 0xdb61, 0xdb77, PDF_CMAP_RANGE, 19498 },
	{ 0xdb78, 0xdb78, PDF_CMAP_SINGLE, 8752 },
	{ 0xdb79, 0xdb7e, PDF_CMAP_RANGE, 19521 },
	{ 0xdb80, 0xdb83, PDF_CMAP_RANGE, 19527 },
	{ 0xdb84, 0xdb84, PDF_CMAP_SINGLE, 9725 },
	{ 0xdb85, 0xdb8a, PDF_CMAP_RANGE, 19531 },
	{ 0xdb8b, 0xdb8b, PDF_CMAP_SINGLE, 9728 },
	{ 0xdb8c, 0xdb97, PDF_CMAP_RANGE, 19537 },
	{ 0xdb98, 0xdb98, PDF_CMAP_SINGLE, 9734 },
	{ 0xdb99, 0xdba0, PDF_CMAP_RANGE, 19549 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 4977 },
	{ 0xdc40, 0xdc44, PDF_CMAP_RANGE, 19557 },
	{ 0xdc45, 0xdc45, PDF_CMAP_SINGLE, 9727 },
	{ 0xdc46, 0xdc4e, PDF_CMAP_RANGE, 19562 },
	{ 0xdc4f, 0xdc57, PDF_CMAP_TABLE, 1095 },
	{ 0xdc58, 0xdc5c, PDF_CMAP_RANGE, 19573 },
	{ 0xdc5d, 0xdc5d, PDF_CMAP_SINGLE, 9729 },
	{ 0xdc5e, 0xdc61, PDF_CMAP_RANGE, 19578 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 9733 },
	{ 0xdc63, 0xdc65, PDF_CMAP_RANGE, 19582 },
	{ 0xdc66, 0xdc67, PDF_CMAP_TABLE, 1104 },
	{ 0xdc68, 0xdc6a, PDF_CMAP_RANGE, 19585 },
	{ 0xdc6b, 0xdc6b, PDF_CMAP_SINGLE, 9735 },
	{ 0xdc6c, 0xdc7b, PDF_CMAP_RANGE, 19588 },
	{ 0xdc7c, 0xdc7c, PDF_CMAP_SINGLE, 8432 },
	{ 0xdc7d, 0xdc7e, PDF_CMAP_RANGE, 19604 },
	{ 0xdc80, 0xdc86, PDF_CMAP_RANGE, 19606 },
	{ 0xdc87, 0xdc8a, PDF_CMAP_TABLE, 1106 },
	{ 0xdc8b, 0xdc8d, PDF_CMAP_RANGE, 19613 },
	{ 0xdc8e, 0xdc90, PDF_CMAP_TABLE, 1110 },
	{ 0xdc91, 0xdc96, PDF_CMAP_RANGE, 19617 },
	{ 0xdc97, 0xdc97, PDF_CMAP_SINGLE, 9357 },
	{ 0xdc98, 0xdc9a, PDF_CMAP_RANGE, 19623 },
	{ 0xdc9b, 0xdc9b, PDF_CMAP_SINGLE, 8450 },
	{ 0xdc9c, 0xdc9f, PDF_CMAP_RANGE, 19626 },
	{ 0xdca0, 0xdca0, PDF_CMAP_SINGLE, 9364 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5071 },
	{ 0xdd40, 0xdd45, PDF_CMAP_RANGE, 19630 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 9363 },
	{ 0xdd47, 0xdd4c, PDF_CMAP_RANGE, 19636 },
	{ 0xdd4d, 0xdd4d, PDF_CMAP_SINGLE, 9358 },
	{ 0xdd4e, 0xdd52, PDF_CMAP_RANGE, 19642 },
	{ 0xdd53, 0xdd59, PDF_CMAP_TABLE, 1113 },
	{ 0xdd5a, 0xdd5d, PDF_CMAP_RANGE, 19648 },
	{ 0xdd5e, 0xdd65, PDF_CMAP_TABLE, 1120 },
	{ 0xdd66, 0xdd6c, PDF_CMAP_RANGE, 19655 },
	{ 0xdd6d, 0xdd70, PDF_CMAP_TABLE, 1128 },
	{ 0xdd71, 0xdd75, PDF_CMAP_RANGE, 19663 },
	{ 0xdd76, 0xdd78, PDF_CMAP_TABLE, 1132 },
	{ 0xdd79, 0xdd7a, PDF_CMAP_RANGE, 9373 },
	{ 0xdd7b, 0xdd7e, PDF_CMAP_RANGE, 19668 },
	{ 0xdd80, 0xdd82, PDF_CMAP_TABLE, 1135 },
	{ 0xdd83, 0xdd84, PDF_CMAP_RANGE, 19673 },
	{ 0xdd85, 0xdd86, PDF_CMAP_TABLE, 1138 },
	{ 0xdd87, 0xdd8a, PDF_CMAP_RANGE, 19675 },
	{ 0xdd8b, 0xdd8b, PDF_CMAP_SINGLE, 8048 },
	{ 0xdd8c, 0xdd8e, PDF_CMAP_RANGE, 19679 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 9376 },
	{ 0xdd90, 0xdd93, PDF_CMAP_RANGE, 19682 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 8507 },
	{ 0xdd95, 0xdd96, PDF_CMAP_RANGE, 19686 },
	{ 0xdd97, 0xdd97, PDF_CMAP_SINGLE, 7943 },
	{ 0xdd98, 0xdd99, PDF_CMAP_RANGE, 19688 },
	{ 0xdd9a, 0xdd9b, PDF_CMAP_TABLE, 1140 },
	{ 0xdd9c, 0xdd9d, PDF_CMAP_RANGE, 19690 },
	{ 0xdd9e, 0xdda0, PDF_CMAP_TABLE, 1142 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xde40, 0xde41, PDF_CMAP_TABLE, 1145 },
	{ 0xde42, 0xde43, PDF_CMAP_RANGE, 19693 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 8872 },
	{ 0xde45, 0xde47, PDF_CMAP_RANGE, 19695 },
	{ 0xde48, 0xde49, PDF_CMAP_TABLE, 1147 },
	{ 0xde4a, 0xde4e, PDF_CMAP_RANGE, 19698 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 9378 },
	{ 0xde50, 0xde59, PDF_CMAP_RANGE, 19703 },
	{ 0xde5a, 0xde5f, PDF_CMAP_TABLE, 1149 },
	{ 0xde60, 0xde6a, PDF_CMAP_RANGE, 19715 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 7728 },
	{ 0xde6c, 0xde6e, PDF_CMAP_RANGE, 19726 },
	{ 0xde6f, 0xde72, PDF_CMAP_TABLE, 1155 },
	{ 0xde73, 0xde7e, PDF_CMAP_RANGE, 19729 },
	{ 0xde80, 0xde91, PDF_CMAP_RANGE, 19741 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 9851 },
	{ 0xde93, 0xde9e, PDF_CMAP_RANGE, 19759 },
	{ 0xde9f, 0xdea0, PDF_CMAP_TABLE, 1159 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdf40, 0xdf42, PDF_CMAP_TABLE, 1161 },
	{ 0xdf43, 0xdf4c, PDF_CMAP_RANGE, 19773 },
	{ 0xdf4d, 0xdf4d, PDF_CMAP_SINGLE, 8123 },
	{ 0xdf4e, 0xdf5b, PDF_CMAP_RANGE, 19783 },
	{ 0xdf5c, 0xdf60, PDF_CMAP_TABLE, 1164 },
	{ 0xdf61, 0xdf63, PDF_CMAP_RANGE, 19798 },
	{ 0xdf64, 0xdf68, PDF_CMAP_TABLE, 1169 },
	{ 0xdf69, 0xdf6c, PDF_CMAP_RANGE, 19803 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 8499 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_RANGE, 19807 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 7812 },
	{ 0xdf75, 0xdf76, PDF_CMAP_RANGE, 19813 },
	{ 0xdf77, 0xdf7e, PDF_CMAP_TABLE, 1174 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 8021 },
	{ 0xdf81, 0xdf82, PDF_CMAP_RANGE, 19818 },
	{ 0xdf83, 0xdf85, PDF_CMAP_TABLE, 1182 },
	{ 0xdf86, 0xdf88, PDF_CMAP_RANGE, 19821 },
	{ 0xdf89, 0xdf8a, PDF_CMAP_TABLE, 1185 },
	{ 0xdf8b, 0xdfa0, PDF_CMAP_RANGE, 19824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe040, 0xe04f, PDF_CMAP_RANGE, 19846 },
	{ 0xe050, 0xe050, PDF_CMAP_SINGLE, 8990 },
	{ 0xe051, 0xe05c, PDF_CMAP_RANGE, 19862 },
	{ 0xe05d, 0xe05d, PDF_CMAP_SINGLE, 8755 },
	{ 0xe05e, 0xe068, PDF_CMAP_RANGE, 19874 },
	{ 0xe069, 0xe069, PDF_CMAP_SINGLE, 8992 },
	{ 0xe06a, 0xe06b, PDF_CMAP_RANGE, 19885 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 8647 },
	{ 0xe06d, 0xe074, PDF_CMAP_RANGE, 19887 },
	{ 0xe075, 0xe079, PDF_CMAP_TABLE, 1187 },
	{ 0xe07a, 0xe07e, PDF_CMAP_RANGE, 19897 },
	{ 0xe080, 0xe086, PDF_CMAP_RANGE, 19902 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 7867 },
	{ 0xe088, 0xe08c, PDF_CMAP_RANGE, 19909 },
	{ 0xe08d, 0xe094, PDF_CMAP_TABLE, 1192 },
	{ 0xe095, 0xe096, PDF_CMAP_RANGE, 19917 },
	{ 0xe097, 0xe097, PDF_CMAP_SINGLE, 8987 },
	{ 0xe098, 0xe0a0, PDF_CMAP_RANGE, 19919 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe140, 0xe141, PDF_CMAP_RANGE, 19928 },
	{ 0xe142, 0xe142, PDF_CMAP_SINGLE, 8993 },
	{ 0xe143, 0xe163, PDF_CMAP_RANGE, 19930 },
	{ 0xe164, 0xe164, PDF_CMAP_SINGLE, 8789 },
	{ 0xe165, 0xe167, PDF_CMAP_RANGE, 19963 },
	{ 0xe168, 0xe168, PDF_CMAP_SINGLE, 7823 },
	{ 0xe169, 0xe173, PDF_CMAP_RANGE, 19966 },
	{ 0xe174, 0xe175, PDF_CMAP_TABLE, 1200 },
	{ 0xe176, 0xe17e, PDF_CMAP_RANGE, 19977 },
	{ 0xe180, 0xe183, PDF_CMAP_RANGE, 19986 },
	{ 0xe184, 0xe189, PDF_CMAP_TABLE, 1202 },
	{ 0xe18a, 0xe18b, PDF_CMAP_RANGE, 19992 },
	{ 0xe18c, 0xe18d, PDF_CMAP_TABLE, 1208 },
	{ 0xe18e, 0xe190, PDF_CMAP_RANGE, 9459 },
	{ 0xe191, 0xe195, PDF_CMAP_TABLE, 1210 },
	{ 0xe196, 0xe197, PDF_CMAP_RANGE, 19996 },
	{ 0xe198, 0xe198, PDF_CMAP_SINGLE, 8832 },
	{ 0xe199, 0xe19d, PDF_CMAP_RANGE, 19998 },
	{ 0xe19e, 0xe1a0, PDF_CMAP_TABLE, 1215 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe240, 0xe243, PDF_CMAP_TABLE, 1218 },
	{ 0xe244, 0xe24e, PDF_CMAP_RANGE, 20006 },
	{ 0xe24f, 0xe254, PDF_CMAP_TABLE, 1222 },
	{ 0xe255, 0xe259, PDF_CMAP_RANGE, 20019 },
	{ 0xe25a, 0xe25b, PDF_CMAP_TABLE, 1228 },
	{ 0xe25c, 0xe25d, PDF_CMAP_RANGE, 20024 },
	{ 0xe25e, 0xe25e, PDF_CMAP_SINGLE, 9478 },
	{ 0xe25f, 0xe261, PDF_CMAP_RANGE, 20026 },
	{ 0xe262, 0xe263, PDF_CMAP_TABLE, 1230 },
	{ 0xe264, 0xe266, PDF_CMAP_RANGE, 20029 },
	{ 0xe267, 0xe26b, PDF_CMAP_TABLE, 1232 },
	{ 0xe26c, 0xe26d, PDF_CMAP_RANGE, 20033 },
	{ 0xe26e, 0xe26f, PDF_CMAP_TABLE, 1237 },
	{ 0xe270, 0xe277, PDF_CMAP_RANGE, 20035 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 8149 },
	{ 0xe279, 0xe27c, PDF_CMAP_RANGE, 20043 },
	{ 0xe27d, 0xe27e, PDF_CMAP_TABLE, 1239 },
	{ 0xe280, 0xe282, PDF_CMAP_TABLE, 1241 },
	{ 0xe283, 0xe288, PDF_CMAP_RANGE, 20048 },
	{ 0xe289, 0xe28b, PDF_CMAP_TABLE, 1244 },
	{ 0xe28c, 0xe28d, PDF_CMAP_RANGE, 20055 },
	{ 0xe28e, 0xe28f, PDF_CMAP_TABLE, 1247 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 20057 },
	{ 0xe292, 0xe295, PDF_CMAP_TABLE, 1249 },
	{ 0xe296, 0xe297, PDF_CMAP_RANGE, 20059 },
	{ 0xe298, 0xe29b, PDF_CMAP_TABLE, 1253 },
	{ 0xe29c, 0xe29f, PDF_CMAP_RANGE, 20061 },
	{ 0xe2a0, 0xe2a0, PDF_CMAP_SINGLE, 9473 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe340, 0xe341, PDF_CMAP_RANGE, 20065 },
	{ 0xe342, 0xe343, PDF_CMAP_TABLE, 1257 },
	{ 0xe344, 0xe346, PDF_CMAP_RANGE, 20067 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 9496 },
	{ 0xe348, 0xe34a, PDF_CMAP_RANGE, 20070 },
	{ 0xe34b, 0xe34b, PDF_CMAP_SINGLE, 7766 },
	{ 0xe34c, 0xe34e, PDF_CMAP_RANGE, 20073 },
	{ 0xe34f, 0xe351, PDF_CMAP_TABLE, 1259 },
	{ 0xe352, 0xe353, PDF_CMAP_RANGE, 20077 },
	{ 0xe354, 0xe355, PDF_CMAP_TABLE, 1262 },
	{ 0xe356, 0xe357, PDF_CMAP_RANGE, 20079 },
	{ 0xe358, 0xe358, PDF_CMAP_SINGLE, 9488 },
	{ 0xe359, 0xe35b, PDF_CMAP_RANGE, 20081 },
	{ 0xe35c, 0xe35c, PDF_CMAP_SINGLE, 7765 },
	{ 0xe35d, 0xe35f, PDF_CMAP_RANGE, 20084 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 9482 },
	{ 0xe361, 0xe365, PDF_CMAP_RANGE, 20087 },
	{ 0xe366, 0xe367, PDF_CMAP_RANGE, 9489 },
	{ 0xe368, 0xe370, PDF_CMAP_RANGE, 20092 },
	{ 0xe371, 0xe374, PDF_CMAP_TABLE, 1264 },
	{ 0xe375, 0xe377, PDF_CMAP_RANGE, 20102 },
	{ 0xe378, 0xe379, PDF_CMAP_TABLE, 1268 },
	{ 0xe37a, 0xe37b, PDF_CMAP_RANGE, 20105 },
	{ 0xe37c, 0xe37e, PDF_CMAP_TABLE, 1270 },
	{ 0xe380, 0xe389, PDF_CMAP_RANGE, 20108 },
	{ 0xe38a, 0xe38c, PDF_CMAP_TABLE, 1273 },
	{ 0xe38d, 0xe38e, PDF_CMAP_RANGE, 20119 },
	{ 0xe38f, 0xe395, PDF_CMAP_TABLE, 1276 },
	{ 0xe396, 0xe398, PDF_CMAP_RANGE, 20124 },
	{ 0xe399, 0xe399, PDF_CMAP_SINGLE, 9501 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 20127 },
	{ 0xe39c, 0xe3a0, PDF_CMAP_TABLE, 1283 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe440, 0xe444, PDF_CMAP_TABLE, 1288 },
	{ 0xe445, 0xe447, PDF_CMAP_RANGE, 20131 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 9509 },
	{ 0xe449, 0xe44d, PDF_CMAP_RANGE, 20134 },
	{ 0xe44e, 0xe453, PDF_CMAP_TABLE, 1293 },
	{ 0xe454, 0xe457, PDF_CMAP_RANGE, 20141 },
	{ 0xe458, 0xe45e, PDF_CMAP_TABLE, 1299 },
	{ 0xe45f, 0xe461, PDF_CMAP_RANGE, 20148 },
	{ 0xe462, 0xe462, PDF_CMAP_SINGLE, 9513 },
	{ 0xe463, 0xe464, PDF_CMAP_RANGE, 20151 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 9505 },
	{ 0xe466, 0xe467, PDF_CMAP_RANGE, 20153 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 7935 },
	{ 0xe469, 0xe472, PDF_CMAP_RANGE, 20155 },
	{ 0xe473, 0xe475, PDF_CMAP_TABLE, 1306 },
	{ 0xe476, 0xe478, PDF_CMAP_RANGE, 20166 },
	{ 0xe479, 0xe47e, PDF_CMAP_TABLE, 1309 },
	{ 0xe480, 0xe481, PDF_CMAP_TABLE, 1315 },
	{ 0xe482, 0xe483, PDF_CMAP_RANGE, 20171 },
	{ 0xe484, 0xe488, PDF_CMAP_TABLE, 1317 },
	{ 0xe489, 0xe48c, PDF_CMAP_RANGE, 20173 },
	{ 0xe48d, 0xe48f, PDF_CMAP_TABLE, 1322 },
	{ 0xe490, 0xe492, PDF_CMAP_RANGE, 20178 },
	{ 0xe493, 0xe493, PDF_CMAP_SINGLE, 7960 },
	{ 0xe494, 0xe497, PDF_CMAP_RANGE, 20181 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 9547 },
	{ 0xe499, 0xe49c, PDF_CMAP_RANGE, 20185 },
	{ 0xe49d, 0xe49d, PDF_CMAP_SINGLE, 9543 },
	{ 0xe49e, 0xe49f, PDF_CMAP_RANGE, 9551 },
	{ 0xe4a0, 0xe4a0, PDF_CMAP_SINGLE, 20189 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe540, 0xe545, PDF_CMAP_RANGE, 20190 },
	{ 0xe546, 0xe548, PDF_CMAP_TABLE, 1325 },
	{ 0xe549, 0xe54a, PDF_CMAP_RANGE, 20197 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 9548 },
	{ 0xe54c, 0xe54d, PDF_CMAP_RANGE, 20199 },
	{ 0xe54e, 0xe551, PDF_CMAP_TABLE, 1328 },
	{ 0xe552, 0xe554, PDF_CMAP_RANGE, 20201 },
	{ 0xe555, 0xe558, PDF_CMAP_TABLE, 1332 },
	{ 0xe559, 0xe55b, PDF_CMAP_RANGE, 20205 },
	{ 0xe55c, 0xe55e, PDF_CMAP_TABLE, 1336 },
	{ 0xe55f, 0xe560, PDF_CMAP_RANGE, 20209 },
	{ 0xe561, 0xe561, PDF_CMAP_SINGLE, 8619 },
	{ 0xe562, 0xe563, PDF_CMAP_RANGE, 20211 },
	{ 0xe564, 0xe565, PDF_CMAP_TABLE, 1339 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 20213 },
	{ 0xe568, 0xe569, PDF_CMAP_TABLE, 1341 },
	{ 0xe56a, 0xe56b, PDF_CMAP_RANGE, 20215 },
	{ 0xe56c, 0xe56e, PDF_CMAP_TABLE, 1343 },
	{ 0xe56f, 0xe574, PDF_CMAP_RANGE, 20218 },
	{ 0xe575, 0xe578, PDF_CMAP_TABLE, 1346 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 20225 },
	{ 0xe57b, 0xe57c, PDF_CMAP_TABLE, 1350 },
	{ 0xe57d, 0xe57e, PDF_CMAP_RANGE, 20227 },
	{ 0xe580, 0xe583, PDF_CMAP_TABLE, 1352 },
	{ 0xe584, 0xe589, PDF_CMAP_RANGE, 20231 },
	{ 0xe58a, 0xe58a, PDF_CMAP_SINGLE, 9558 },
	{ 0xe58b, 0xe58d, PDF_CMAP_RANGE, 20237 },
	{ 0xe58e, 0xe58e, PDF_CMAP_SINGLE, 8808 },
	{ 0xe58f, 0xe590, PDF_CMAP_RANGE, 20240 },
	{ 0xe591, 0xe591, PDF_CMAP_SINGLE, 7894 },
	{ 0xe592, 0xe599, PDF_CMAP_RANGE, 20242 },
	{ 0xe59a, 0xe59b, PDF_CMAP_TABLE, 1356 },
	{ 0xe59c, 0xe59e, PDF_CMAP_RANGE, 20250 },
	{ 0xe59f, 0xe5a0, PDF_CMAP_TABLE, 1358 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe640, 0xe640, PDF_CMAP_SINGLE, 8412 },
	{ 0xe641, 0xe643, PDF_CMAP_RANGE, 20254 },
	{ 0xe644, 0xe644, PDF_CMAP_SINGLE, 9561 },
	{ 0xe645, 0xe648, PDF_CMAP_RANGE, 20257 },
	{ 0xe649, 0xe64a, PDF_CMAP_TABLE, 1360 },
	{ 0xe64b, 0xe64d, PDF_CMAP_RANGE, 20261 },
	{ 0xe64e, 0xe64e, PDF_CMAP_SINGLE, 8829 },
	{ 0xe64f, 0xe651, PDF_CMAP_RANGE, 20264 },
	{ 0xe652, 0xe652, PDF_CMAP_SINGLE, 9598 },
	{ 0xe653, 0xe655, PDF_CMAP_RANGE, 20267 },
	{ 0xe656, 0xe658, PDF_CMAP_TABLE, 1362 },
	{ 0xe659, 0xe65a, PDF_CMAP_RANGE, 20271 },
	{ 0xe65b, 0xe65b, PDF_CMAP_SINGLE, 9566 },
	{ 0xe65c, 0xe65d, PDF_CMAP_RANGE, 20273 },
	{ 0xe65e, 0xe65e, PDF_CMAP_SINGLE, 7732 },
	{ 0xe65f, 0xe668, PDF_CMAP_RANGE, 20275 },
	{ 0xe669, 0xe66b, PDF_CMAP_TABLE, 1365 },
	{ 0xe66c, 0xe674, PDF_CMAP_RANGE, 20286 },
	{ 0xe675, 0xe676, PDF_CMAP_TABLE, 1368 },
	{ 0xe677, 0xe678, PDF_CMAP_RANGE, 20295 },
	{ 0xe679, 0xe67e, PDF_CMAP_TABLE, 1370 },
	{ 0xe680, 0xe684, PDF_CMAP_TABLE, 1376 },
	{ 0xe685, 0xe686, PDF_CMAP_RANGE, 20301 },
	{ 0xe687, 0xe689, PDF_CMAP_TABLE, 1381 },
	{ 0xe68a, 0xe68b, PDF_CMAP_RANGE, 20304 },
	{ 0xe68c, 0xe68c, PDF_CMAP_SINGLE, 9569 },
	{ 0xe68d, 0xe692, PDF_CMAP_RANGE, 20306 },
	{ 0xe693, 0xe693, PDF_CMAP_SINGLE, 9570 },
	{ 0xe694, 0xe696, PDF_CMAP_RANGE, 20312 },
	{ 0xe697, 0xe697, PDF_CMAP_SINGLE, 9580 },
	{ 0xe698, 0xe69a, PDF_CMAP_RANGE, 20315 },
	{ 0xe69b, 0xe69c, PDF_CMAP_TABLE, 1384 },
	{ 0xe69d, 0xe69e, PDF_CMAP_RANGE, 20318 },
	{ 0xe69f, 0xe6a0, PDF_CMAP_TABLE, 1386 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 20320 },
	{ 0xe743, 0xe743, PDF_CMAP_SINGLE, 9582 },
	{ 0xe744, 0xe747, PDF_CMAP_RANGE, 20323 },
	{ 0xe748, 0xe749, PDF_CMAP_TABLE, 1388 },
	{ 0xe74a, 0xe74c, PDF_CMAP_RANGE, 20327 },
	{ 0xe74d, 0xe74e, PDF_CMAP_RANGE, 9576 },
	{ 0xe74f, 0xe755, PDF_CMAP_TABLE, 1390 },
	{ 0xe756, 0xe758, PDF_CMAP_RANGE, 20332 },
	{ 0xe759, 0xe759, PDF_CMAP_SINGLE, 9757 },
	{ 0xe75a, 0xe765, PDF_CMAP_RANGE, 20335 },
	{ 0xe766, 0xe76a, PDF_CMAP_TABLE, 1397 },
	{ 0xe76b, 0xe773, PDF_CMAP_RANGE, 20349 },
	{ 0xe774, 0xe774, PDF_CMAP_SINGLE, 9506 },
	{ 0xe775, 0xe77b, PDF_CMAP_RANGE, 20358 },
	{ 0xe77c, 0xe77c, PDF_CMAP_SINGLE, 9523 },
	{ 0xe77d, 0xe77e, PDF_CMAP_RANGE, 20365 },
	{ 0xe780, 0xe781, PDF_CMAP_RANGE, 20367 },
	{ 0xe782, 0xe786, PDF_CMAP_TABLE, 1402 },
	{ 0xe787, 0xe789, PDF_CMAP_RANGE, 20370 },
	{ 0xe78a, 0xe78b, PDF_CMAP_TABLE, 1407 },
	{ 0xe78c, 0xe78e, PDF_CMAP_RANGE, 20373 },
	{ 0xe78f, 0xe78f, PDF_CMAP_SINGLE, 9584 },
	{ 0xe790, 0xe791, PDF_CMAP_RANGE, 20376 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 9589 },
	{ 0xe793, 0xe797, PDF_CMAP_RANGE, 20378 },
	{ 0xe798, 0xe799, PDF_CMAP_RANGE, 9537 },
	{ 0xe79a, 0xe79a, PDF_CMAP_SINGLE, 9565 },
	{ 0xe79b, 0xe79f, PDF_CMAP_RANGE, 20383 },
	{ 0xe7a0, 0xe7a0, PDF_CMAP_SINGLE, 8218 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe840, 0xe842, PDF_CMAP_RANGE, 20388 },
	{ 0xe843, 0xe846, PDF_CMAP_TABLE, 1409 },
	{ 0xe847, 0xe848, PDF_CMAP_RANGE, 20392 },
	{ 0xe849, 0xe84b, PDF_CMAP_TABLE, 1413 },
	{ 0xe84c, 0xe84e, PDF_CMAP_RANGE, 20395 },
	{ 0xe84f, 0xe84f, PDF_CMAP_SINGLE, 9595 },
	{ 0xe850, 0xe853, PDF_CMAP_RANGE, 20398 },
	{ 0xe854, 0xe854, PDF_CMAP_SINGLE, 8867 },
	{ 0xe855, 0xe859, PDF_CMAP_RANGE, 20402 },
	{ 0xe85a, 0xe85c, PDF_CMAP_TABLE, 1416 },
	{ 0xe85d, 0xe861, PDF_CMAP_RANGE, 20408 },
	{ 0xe862, 0xe864, PDF_CMAP_TABLE, 1419 },
	{ 0xe865, 0xe86f, PDF_CMAP_RANGE, 20414 },
	{ 0xe870, 0xe870, PDF_CMAP_SINGLE, 9492 },
	{ 0xe871, 0xe872, PDF_CMAP_RANGE, 20425 },
	{ 0xe873, 0xe875, PDF_CMAP_TABLE, 1422 },
	{ 0xe876, 0xe87b, PDF_CMAP_RANGE, 20428 },
	{ 0xe87c, 0xe87c, PDF_CMAP_SINGLE, 9588 },
	{ 0xe87d, 0xe87e, PDF_CMAP_RANGE, 20434 },
	{ 0xe880, 0xe882, PDF_CMAP_TABLE, 1425 },
	{ 0xe883, 0xe886, PDF_CMAP_RANGE, 20437 },
	{ 0xe887, 0xe889, PDF_CMAP_TABLE, 1428 },
	{ 0xe88a, 0xe88b, PDF_CMAP_RANGE, 20442 },
	{ 0xe88c, 0xe88f, PDF_CMAP_TABLE, 1431 },
	{ 0xe890, 0xe8a0, PDF_CMAP_RANGE, 20444 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe940, 0xe94b, PDF_CMAP_RANGE, 20461 },
	{ 0xe94c, 0xe94c, PDF_CMAP_SINGLE, 7797 },
	{ 0xe94d, 0xe953, PDF_CMAP_RANGE, 20473 },
	{ 0xe954, 0xe957, PDF_CMAP_TABLE, 1435 },
	{ 0xe958, 0xe959, PDF_CMAP_RANGE, 20481 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 9152 },
	{ 0xe95b, 0xe95c, PDF_CMAP_RANGE, 20483 },
	{ 0xe95d, 0xe968, PDF_CMAP_TABLE, 1439 },
	{ 0xe969, 0xe96b, PDF_CMAP_RANGE, 20489 },
	{ 0xe96c, 0xe96c, PDF_CMAP_SINGLE, 8809 },
	{ 0xe96d, 0xe974, PDF_CMAP_RANGE, 20492 },
	{ 0xe975, 0xe979, PDF_CMAP_TABLE, 1451 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 20501 },
	{ 0xe97c, 0xe97e, PDF_CMAP_TABLE, 1456 },
	{ 0xe980, 0xe982, PDF_CMAP_TABLE, 1459 },
	{ 0xe983, 0xe986, PDF_CMAP_RANGE, 20504 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 8783 },
	{ 0xe988, 0xe98a, PDF_CMAP_RANGE, 20508 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 9163 },
	{ 0xe98c, 0xe98d, PDF_CMAP_RANGE, 20511 },
	{ 0xe98e, 0xe994, PDF_CMAP_TABLE, 1462 },
	{ 0xe995, 0xe997, PDF_CMAP_RANGE, 20514 },
	{ 0xe998, 0xe998, PDF_CMAP_SINGLE, 9168 },
	{ 0xe999, 0xe99a, PDF_CMAP_RANGE, 20517 },
	{ 0xe99b, 0xe9a0, PDF_CMAP_TABLE, 1469 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xea40, 0xea40, PDF_CMAP_SINGLE, 8184 },
	{ 0xea41, 0xea43, PDF_CMAP_RANGE, 20521 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 9171 },
	{ 0xea45, 0xea47, PDF_CMAP_RANGE, 20524 },
	{ 0xea48, 0xea4a, PDF_CMAP_TABLE, 1475 },
	{ 0xea4b, 0xea4f, PDF_CMAP_RANGE, 20527 },
	{ 0xea50, 0xea52, PDF_CMAP_TABLE, 1478 },
	{ 0xea53, 0xea54, PDF_CMAP_RANGE, 20533 },
	{ 0xea55, 0xea56, PDF_CMAP_TABLE, 1481 },
	{ 0xea57, 0xea58, PDF_CMAP_RANGE, 20535 },
	{ 0xea59, 0xea59, PDF_CMAP_SINGLE, 9157 },
	{ 0xea5a, 0xea7e, PDF_CMAP_RANGE, 20537 },
	{ 0xea80, 0xea80, PDF_CMAP_SINGLE, 8986 },
	{ 0xea81, 0xea83, PDF_CMAP_RANGE, 20574 },
	{ 0xea84, 0xea84, PDF_CMAP_SINGLE, 8468 },
	{ 0xea85, 0xea86, PDF_CMAP_RANGE, 20577 },
	{ 0xea87, 0xea87, PDF_CMAP_SINGLE, 8836 },
	{ 0xea88, 0xea8d, PDF_CMAP_RANGE, 20579 },
	{ 0xea8e, 0xea91, PDF_CMAP_TABLE, 1483 },
	{ 0xea92, 0xea95, PDF_CMAP_RANGE, 20586 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 8705 },
	{ 0xea97, 0xea9f, PDF_CMAP_RANGE, 20590 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 7897 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeb40, 0xeb41, PDF_CMAP_TABLE, 1487 },
	{ 0xeb42, 0xeb44, PDF_CMAP_RANGE, 20600 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 8786 },
	{ 0xeb46, 0xeb47, PDF_CMAP_RANGE, 20603 },
	{ 0xeb48, 0xeb48, PDF_CMAP_SINGLE, 8057 },
	{ 0xeb49, 0xeb52, PDF_CMAP_RANGE, 20605 },
	{ 0xeb53, 0xeb55, PDF_CMAP_TABLE, 1489 },
	{ 0xeb56, 0xeb5a, PDF_CMAP_RANGE, 20616 },
	{ 0xeb5b, 0xeb5d, PDF_CMAP_TABLE, 1492 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 20622 },
	{ 0xeb60, 0xeb62, PDF_CMAP_TABLE, 1495 },
	{ 0xeb63, 0xeb6c, PDF_CMAP_RANGE, 20625 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 8534 },
	{ 0xeb6e, 0xeb6f, PDF_CMAP_RANGE, 20635 },
	{ 0xeb70, 0xeb73, PDF_CMAP_TABLE, 1498 },
	{ 0xeb74, 0xeb77, PDF_CMAP_RANGE, 20638 },
	{ 0xeb78, 0xeb79, PDF_CMAP_TABLE, 1502 },
	{ 0xeb7a, 0xeb7e, PDF_CMAP_RANGE, 20642 },
	{ 0xeb80, 0xeb84, PDF_CMAP_RANGE, 20647 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 8784 },
	{ 0xeb86, 0xeb89, PDF_CMAP_RANGE, 20652 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 7875 },
	{ 0xeb8b, 0xeba0, PDF_CMAP_RANGE, 20656 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xec40, 0xec45, PDF_CMAP_RANGE, 20678 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8616 },
	{ 0xec47, 0xec55, PDF_CMAP_RANGE, 20684 },
	{ 0xec56, 0xec56, PDF_CMAP_SINGLE, 9741 },
	{ 0xec57, 0xec59, PDF_CMAP_RANGE, 20699 },
	{ 0xec5a, 0xec5c, PDF_CMAP_TABLE, 1504 },
	{ 0xec5d, 0xec5f, PDF_CMAP_RANGE, 20703 },
	{ 0xec60, 0xec60, PDF_CMAP_SINGLE, 8242 },
	{ 0xec61, 0xec6d, PDF_CMAP_RANGE, 20706 },
	{ 0xec6e, 0xec6e, PDF_CMAP_SINGLE, 9739 },
	{ 0xec6f, 0xec75, PDF_CMAP_RANGE, 20719 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8899 },
	{ 0xec77, 0xec7e, PDF_CMAP_RANGE, 20726 },
	{ 0xec80, 0xec95, PDF_CMAP_RANGE, 20734 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 7971 },
	{ 0xec97, 0xeca0, PDF_CMAP_RANGE, 20756 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xed40, 0xed45, PDF_CMAP_RANGE, 20766 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 9877 },
	{ 0xed47, 0xed57, PDF_CMAP_RANGE, 20772 },
	{ 0xed58, 0xed58, PDF_CMAP_SINGLE, 9822 },
	{ 0xed59, 0xed5d, PDF_CMAP_RANGE, 20789 },
	{ 0xed5e, 0xed5e, PDF_CMAP_SINGLE, 9821 },
	{ 0xed5f, 0xed60, PDF_CMAP_RANGE, 20794 },
	{ 0xed61, 0xed61, PDF_CMAP_SINGLE, 9874 },
	{ 0xed62, 0xed63, PDF_CMAP_RANGE, 20796 },
	{ 0xed64, 0xed67, PDF_CMAP_TABLE, 1507 },
	{ 0xed68, 0xed6d, PDF_CMAP_RANGE, 20799 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 8000 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 20805 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 9317 },
	{ 0xed75, 0xed76, PDF_CMAP_RANGE, 20810 },
	{ 0xed77, 0xed79, PDF_CMAP_TABLE, 1511 },
	{ 0xed7a, 0xed7e, PDF_CMAP_RANGE, 20813 },
	{ 0xed80, 0xed90, PDF_CMAP_RANGE, 20818 },
	{ 0xed91, 0xeda0, PDF_CMAP_TABLE, 1514 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xee40, 0xee44, PDF_CMAP_TABLE, 1530 },
	{ 0xee45, 0xee47, PDF_CMAP_RANGE, 20840 },
	{ 0xee48, 0xee49, PDF_CMAP_TABLE, 1535 },
	{ 0xee4a, 0xee4c, PDF_CMAP_RANGE, 20843 },
	{ 0xee4d, 0xee4d, PDF_CMAP_SINGLE, 9673 },
	{ 0xee4e, 0xee51, PDF_CMAP_RANGE, 20846 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 9672 },
	{ 0xee53, 0xee54, PDF_CMAP_RANGE, 20850 },
	{ 0xee55, 0xee57, PDF_CMAP_TABLE, 1537 },
	{ 0xee58, 0xee5d, PDF_CMAP_RANGE, 20853 },
	{ 0xee5e, 0xee5e, PDF_CMAP_SINGLE, 8573 },
	{ 0xee5f, 0xee60, PDF_CMAP_RANGE, 20859 },
	{ 0xee61, 0xee61, PDF_CMAP_SINGLE, 8062 },
	{ 0xee62, 0xee67, PDF_CMAP_RANGE, 20861 },
	{ 0xee68, 0xee69, PDF_CMAP_TABLE, 1540 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 20867 },
	{ 0xee6c, 0xee6e, PDF_CMAP_TABLE, 1542 },
	{ 0xee6f, 0xee76, PDF_CMAP_RANGE, 20870 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 8154 },
	{ 0xee78, 0xee7c, PDF_CMAP_RANGE, 20878 },
	{ 0xee7d, 0xee7e, PDF_CMAP_TABLE, 1545 },
	{ 0xee80, 0xee80, PDF_CMAP_SINGLE, 9677 },
	{ 0xee81, 0xee84, PDF_CMAP_RANGE, 20883 },
	{ 0xee85, 0xee86, PDF_CMAP_TABLE, 1547 },
	{ 0xee87, 0xee89, PDF_CMAP_RANGE, 20887 },
	{ 0xee8a, 0xee8d, PDF_CMAP_TABLE, 1549 },
	{ 0xee8e, 0xee8f, PDF_CMAP_RANGE, 20891 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 8200 },
	{ 0xee91, 0xee93, PDF_CMAP_RANGE, 20893 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 9680 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 20896 },
	{ 0xee97, 0xee99, PDF_CMAP_TABLE, 1553 },
	{ 0xee9a, 0xee9c, PDF_CMAP_RANGE, 20899 },
	{ 0xee9d, 0xee9e, PDF_CMAP_TABLE, 1556 },
	{ 0xee9f, 0xeea0, PDF_CMAP_RANGE, 20902 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xef40, 0xef45, PDF_CMAP_TABLE, 1558 },
	{ 0xef46, 0xef4b, PDF_CMAP_RANGE, 20905 },
	{ 0xef4c, 0xef4c, PDF_CMAP_SINGLE, 7936 },
	{ 0xef4d, 0xef51, PDF_CMAP_RANGE, 20911 },
	{ 0xef52, 0xef53, PDF_CMAP_RANGE, 9420 },
	{ 0xef54, 0xef57, PDF_CMAP_TABLE, 1564 },
	{ 0xef58, 0xef59, PDF_CMAP_RANGE, 20918 },
	{ 0xef5a, 0xef5a, PDF_CMAP_SINGLE, 9422 },
	{ 0xef5b, 0xef5f, PDF_CMAP_RANGE, 20920 },
	{ 0xef60, 0xef60, PDF_CMAP_SINGLE, 9423 },
	{ 0xef61, 0xef67, PDF_CMAP_RANGE, 20925 },
	{ 0xef68, 0xef6c, PDF_CMAP_TABLE, 1568 },
	{ 0xef6d, 0xef76, PDF_CMAP_RANGE, 20934 },
	{ 0xef77, 0xef77, PDF_CMAP_SINGLE, 7924 },
	{ 0xef78, 0xef79, PDF_CMAP_RANGE, 20944 },
	{ 0xef7a, 0xef7c, PDF_CMAP_TABLE, 1573 },
	{ 0xef7d, 0xef7e, PDF_CMAP_RANGE, 20947 },
	{ 0xef80, 0xef81, PDF_CMAP_RANGE, 20949 },
	{ 0xef82, 0xef82, PDF_CMAP_SINGLE, 9117 },
	{ 0xef83, 0xef84, PDF_CMAP_RANGE, 9119 },
	{ 0xef85, 0xef88, PDF_CMAP_TABLE, 1576 },
	{ 0xef89, 0xef8a, PDF_CMAP_RANGE, 20953 },
	{ 0xef8b, 0xef8d, PDF_CMAP_TABLE, 1580 },
	{ 0xef8e, 0xef94, PDF_CMAP_RANGE, 20956 },
	{ 0xef95, 0xef97, PDF_CMAP_TABLE, 1583 },
	{ 0xef98, 0xef9b, PDF_CMAP_RANGE, 20963 },
	{ 0xef9c, 0xef9e, PDF_CMAP_TABLE, 1586 },
	{ 0xef9f, 0xefa0, PDF_CMAP_RANGE, 20968 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf040, 0xf044, PDF_CMAP_TABLE, 1589 },
	{ 0xf045, 0xf046, PDF_CMAP_RANGE, 20972 },
	{ 0xf047, 0xf049, PDF_CMAP_TABLE, 1594 },
	{ 0xf04a, 0xf04d, PDF_CMAP_RANGE, 20974 },
	{ 0xf04e, 0xf04e, PDF_CMAP_SINGLE, 8760 },
	{ 0xf04f, 0xf050, PDF_CMAP_RANGE, 20978 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 9125 },
	{ 0xf052, 0xf053, PDF_CMAP_RANGE, 20980 },
	{ 0xf054, 0xf054, PDF_CMAP_SINGLE, 8090 },
	{ 0xf055, 0xf056, PDF_CMAP_RANGE, 20982 },
	{ 0xf057, 0xf057, PDF_CMAP_SINGLE, 8643 },
	{ 0xf058, 0xf05d, PDF_CMAP_RANGE, 20984 },
	{ 0xf05e, 0xf05e, PDF_CMAP_SINGLE, 7982 },
	{ 0xf05f, 0xf067, PDF_CMAP_RANGE, 20990 },
	{ 0xf068, 0xf068, PDF_CMAP_SINGLE, 9116 },
	{ 0xf069, 0xf06b, PDF_CMAP_RANGE, 20999 },
	{ 0xf06c, 0xf06c, PDF_CMAP_SINGLE, 9126 },
	{ 0xf06d, 0xf070, PDF_CMAP_RANGE, 21002 },
	{ 0xf071, 0xf074, PDF_CMAP_TABLE, 1597 },
	{ 0xf075, 0xf077, PDF_CMAP_RANGE, 21007 },
	{ 0xf078, 0xf07a, PDF_CMAP_TABLE, 1601 },
	{ 0xf07b, 0xf07c, PDF_CMAP_RANGE, 21011 },
	{ 0xf07d, 0xf07e, PDF_CMAP_RANGE, 9129 },
	{ 0xf080, 0xf082, PDF_CMAP_TABLE, 1604 },
	{ 0xf083, 0xf086, PDF_CMAP_RANGE, 21013 },
	{ 0xf087, 0xf088, PDF_CMAP_TABLE, 1607 },
	{ 0xf089, 0xf08a, PDF_CMAP_RANGE, 21017 },
	{ 0xf08b, 0xf08b, PDF_CMAP_SINGLE, 9830 },
	{ 0xf08c, 0xf08f, PDF_CMAP_RANGE, 21019 },
	{ 0xf090, 0xf092, PDF_CMAP_TABLE, 1609 },
	{ 0xf093, 0xf095, PDF_CMAP_RANGE, 21024 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 9133 },
	{ 0xf097, 0xf0a0, PDF_CMAP_RANGE, 21027 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf140, 0xf151, PDF_CMAP_RANGE, 21037 },
	{ 0xf152, 0xf154, PDF_CMAP_TABLE, 1612 },
	{ 0xf155, 0xf156, PDF_CMAP_RANGE, 21055 },
	{ 0xf157, 0xf15a, PDF_CMAP_TABLE, 1615 },
	{ 0xf15b, 0xf166, PDF_CMAP_RANGE, 21058 },
	{ 0xf167, 0xf167, PDF_CMAP_SINGLE, 7767 },
	{ 0xf168, 0xf175, PDF_CMAP_RANGE, 21070 },
	{ 0xf176, 0xf17b, PDF_CMAP_TABLE, 1619 },
	{ 0xf17c, 0xf17d, PDF_CMAP_RANGE, 21085 },
	{ 0xf17e, 0xf17e, PDF_CMAP_SINGLE, 9224 },
	{ 0xf180, 0xf189, PDF_CMAP_TABLE, 1625 },
	{ 0xf18a, 0xf193, PDF_CMAP_RANGE, 21091 },
	{ 0xf194, 0xf194, PDF_CMAP_SINGLE, 7999 },
	{ 0xf195, 0xf197, PDF_CMAP_RANGE, 21101 },
	{ 0xf198, 0xf198, PDF_CMAP_SINGLE, 8295 },
	{ 0xf199, 0xf1a0, PDF_CMAP_RANGE, 21104 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf240, 0xf244, PDF_CMAP_RANGE, 21112 },
	{ 0xf245, 0xf247, PDF_CMAP_TABLE, 1635 },
	{ 0xf248, 0xf24a, PDF_CMAP_RANGE, 21118 },
	{ 0xf24b, 0xf24b, PDF_CMAP_SINGLE, 9231 },
	{ 0xf24c, 0xf252, PDF_CMAP_RANGE, 21121 },
	{ 0xf253, 0xf255, PDF_CMAP_TABLE, 1638 },
	{ 0xf256, 0xf25b, PDF_CMAP_RANGE, 21128 },
	{ 0xf25c, 0xf25c, PDF_CMAP_SINGLE, 9234 },
	{ 0xf25d, 0xf25e, PDF_CMAP_RANGE, 21134 },
	{ 0xf25f, 0xf25f, PDF_CMAP_SINGLE, 8375 },
	{ 0xf260, 0xf270, PDF_CMAP_RANGE, 21136 },
	{ 0xf271, 0xf276, PDF_CMAP_TABLE, 1641 },
	{ 0xf277, 0xf27b, PDF_CMAP_RANGE, 21155 },
	{ 0xf27c, 0xf27e, PDF_CMAP_TABLE, 1647 },
	{ 0xf280, 0xf284, PDF_CMAP_RANGE, 21160 },
	{ 0xf285, 0xf289, PDF_CMAP_TABLE, 1650 },
	{ 0xf28a, 0xf28b, PDF_CMAP_RANGE, 9238 },
	{ 0xf28c, 0xf28c, PDF_CMAP_SINGLE, 8433 },
	{ 0xf28d, 0xf290, PDF_CMAP_RANGE, 21166 },
	{ 0xf291, 0xf291, PDF_CMAP_SINGLE, 9226 },
	{ 0xf292, 0xf293, PDF_CMAP_RANGE, 21170 },
	{ 0xf294, 0xf296, PDF_CMAP_TABLE, 1655 },
	{ 0xf297, 0xf29b, PDF_CMAP_RANGE, 21173 },
	{ 0xf29c, 0xf29e, PDF_CMAP_TABLE, 1658 },
	{ 0xf29f, 0xf2a0, PDF_CMAP_RANGE, 21179 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf340, 0xf341, PDF_CMAP_TABLE, 1661 },
	{ 0xf342, 0xf344, PDF_CMAP_RANGE, 21181 },
	{ 0xf345, 0xf345, PDF_CMAP_SINGLE, 8860 },
	{ 0xf346, 0xf347, PDF_CMAP_RANGE, 21184 },
	{ 0xf348, 0xf34b, PDF_CMAP_TABLE, 1663 },
	{ 0xf34c, 0xf34f, PDF_CMAP_RANGE, 21187 },
	{ 0xf350, 0xf350, PDF_CMAP_SINGLE, 9228 },
	{ 0xf351, 0xf360, PDF_CMAP_RANGE, 21191 },
	{ 0xf361, 0xf361, PDF_CMAP_SINGLE, 7721 },
	{ 0xf362, 0xf373, PDF_CMAP_RANGE, 21207 },
	{ 0xf374, 0xf379, PDF_CMAP_TABLE, 1667 },
	{ 0xf37a, 0xf37e, PDF_CMAP_RANGE, 21226 },
	{ 0xf380, 0xf38b, PDF_CMAP_RANGE, 21231 },
	{ 0xf38c, 0xf38c, PDF_CMAP_SINGLE, 9845 },
	{ 0xf38d, 0xf39f, PDF_CMAP_RANGE, 21243 },
	{ 0xf3a0, 0xf3a0, PDF_CMAP_SINGLE, 8524 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf440, 0xf444, PDF_CMAP_RANGE, 21262 },
	{ 0xf445, 0xf445, PDF_CMAP_SINGLE, 9850 },
	{ 0xf446, 0xf44f, PDF_CMAP_RANGE, 21267 },
	{ 0xf450, 0xf450, PDF_CMAP_SINGLE, 9888 },
	{ 0xf451, 0xf456, PDF_CMAP_RANGE, 21277 },
	{ 0xf457, 0xf45d, PDF_CMAP_TABLE, 1673 },
	{ 0xf45e, 0xf461, PDF_CMAP_RANGE, 21286 },
	{ 0xf462, 0xf464, PDF_CMAP_TABLE, 1680 },
	{ 0xf465, 0xf474, PDF_CMAP_RANGE, 21291 },
	{ 0xf475, 0xf475, PDF_CMAP_SINGLE, 9829 },
	{ 0xf476, 0xf47b, PDF_CMAP_RANGE, 21307 },
	{ 0xf47c, 0xf47e, PDF_CMAP_TABLE, 1683 },
	{ 0xf480, 0xf493, PDF_CMAP_RANGE, 21314 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 8266 },
	{ 0xf495, 0xf498, PDF_CMAP_RANGE, 21334 },
	{ 0xf499, 0xf499, PDF_CMAP_SINGLE, 9759 },
	{ 0xf49a, 0xf49b, PDF_CMAP_RANGE, 21338 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 9758 },
	{ 0xf49d, 0xf4a0, PDF_CMAP_RANGE, 21340 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf540, 0xf544, PDF_CMAP_RANGE, 21344 },
	{ 0xf545, 0xf547, PDF_CMAP_TABLE, 1686 },
	{ 0xf548, 0xf551, PDF_CMAP_RANGE, 21350 },
	{ 0xf552, 0xf556, PDF_CMAP_TABLE, 1689 },
	{ 0xf557, 0xf55d, PDF_CMAP_RANGE, 21361 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 9769 },
	{ 0xf55f, 0xf560, PDF_CMAP_RANGE, 21368 },
	{ 0xf561, 0xf562, PDF_CMAP_TABLE, 1694 },
	{ 0xf563, 0xf56d, PDF_CMAP_RANGE, 21370 },
	{ 0xf56e, 0xf572, PDF_CMAP_TABLE, 1696 },
	{ 0xf573, 0xf57e, PDF_CMAP_RANGE, 21382 },
	{ 0xf580, 0xf584, PDF_CMAP_RANGE, 21394 },
	{ 0xf585, 0xf586, PDF_CMAP_TABLE, 1701 },
	{ 0xf587, 0xf58b, PDF_CMAP_RANGE, 21399 },
	{ 0xf58c, 0xf58f, PDF_CMAP_TABLE, 1703 },
	{ 0xf590, 0xf598, PDF_CMAP_RANGE, 21405 },
	{ 0xf599, 0xf59b, PDF_CMAP_TABLE, 1707 },
	{ 0xf59c, 0xf59f, PDF_CMAP_RANGE, 21415 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 9795 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf640, 0xf641, PDF_CMAP_TABLE, 1710 },
	{ 0xf642, 0xf644, PDF_CMAP_RANGE, 21420 },
	{ 0xf645, 0xf648, PDF_CMAP_TABLE, 1712 },
	{ 0xf649, 0xf64a, PDF_CMAP_RANGE, 21424 },
	{ 0xf64b, 0xf64d, PDF_CMAP_TABLE, 1716 },
	{ 0xf64e, 0xf64f, PDF_CMAP_RANGE, 9787 },
	{ 0xf650, 0xf653, PDF_CMAP_RANGE, 21427 },
	{ 0xf654, 0xf654, PDF_CMAP_SINGLE, 9794 },
	{ 0xf655, 0xf657, PDF_CMAP_RANGE, 21431 },
	{ 0xf658, 0xf658, PDF_CMAP_SINGLE, 9796 },
	{ 0xf659, 0xf660, PDF_CMAP_RANGE, 21434 },
	{ 0xf661, 0xf663, PDF_CMAP_TABLE, 1719 },
	{ 0xf664, 0xf66b, PDF_CMAP_RANGE, 21443 },
	{ 0xf66c, 0xf66d, PDF_CMAP_TABLE, 1722 },
	{ 0xf66e, 0xf670, PDF_CMAP_RANGE, 21451 },
	{ 0xf671, 0xf671, PDF_CMAP_SINGLE, 9801 },
	{ 0xf672, 0xf673, PDF_CMAP_RANGE, 21454 },
	{ 0xf674, 0xf677, PDF_CMAP_TABLE, 1724 },
	{ 0xf678, 0xf67e, PDF_CMAP_RANGE, 21457 },
	{ 0xf680, 0xf684, PDF_CMAP_RANGE, 21464 },
	{ 0xf685, 0xf685, PDF_CMAP_SINGLE, 9805 },
	{ 0xf686, 0xf687, PDF_CMAP_RANGE, 21469 },
	{ 0xf688, 0xf68a, PDF_CMAP_TABLE, 1728 },
	{ 0xf68b, 0xf68c, PDF_CMAP_RANGE, 21472 },
	{ 0xf68d, 0xf68e, PDF_CMAP_TABLE, 1731 },
	{ 0xf68f, 0xf691, PDF_CMAP_RANGE, 21474 },
	{ 0xf692, 0xf692, PDF_CMAP_SINGLE, 9807 },
	{ 0xf693, 0xf695, PDF_CMAP_RANGE, 21477 },
	{ 0xf696, 0xf6a0, PDF_CMAP_TABLE, 1733 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf740, 0xf741, PDF_CMAP_RANGE, 21484 },
	{ 0xf742, 0xf742, PDF_CMAP_SINGLE, 9812 },
	{ 0xf743, 0xf748, PDF_CMAP_RANGE, 21486 },
	{ 0xf749, 0xf749, PDF_CMAP_SINGLE, 9816 },
	{ 0xf74a, 0xf74b, PDF_CMAP_RANGE, 21492 },
	{ 0xf74c, 0xf74d, PDF_CMAP_TABLE, 1744 },
	{ 0xf74e, 0xf755, PDF_CMAP_RANGE, 21494 },
	{ 0xf756, 0xf75c, PDF_CMAP_TABLE, 1746 },
	{ 0xf75d, 0xf760, PDF_CMAP_RANGE, 21504 },
	{ 0xf761, 0xf763, PDF_CMAP_TABLE, 1753 },
	{ 0xf764, 0xf76a, PDF_CMAP_RANGE, 21509 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 9820 },
	{ 0xf76c, 0xf770, PDF_CMAP_RANGE, 21516 },
	{ 0xf771, 0xf771, PDF_CMAP_SINGLE, 9772 },
	{ 0xf772, 0xf77b, PDF_CMAP_RANGE, 21521 },
	{ 0xf77c, 0xf77e, PDF_CMAP_TABLE, 1756 },
	{ 0xf780, 0xf7a0, PDF_CMAP_RANGE, 21532 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
	{ 0xf840, 0xf841, PDF_CMAP_RANGE, 21565 },
	{ 0xf842, 0xf842, PDF_CMAP_SINGLE, 8348 },
	{ 0xf843, 0xf845, PDF_CMAP_RANGE, 21567 },
	{ 0xf846, 0xf846, PDF_CMAP_SINGLE, 9600 },
	{ 0xf847, 0xf848, PDF_CMAP_RANGE, 21570 },
	{ 0xf849, 0xf849, PDF_CMAP_SINGLE, 8924 },
	{ 0xf84a, 0xf84f, PDF_CMAP_RANGE, 21572 },
	{ 0xf850, 0xf853, PDF_CMAP_TABLE, 1759 },
	{ 0xf854, 0xf862, PDF_CMAP_RANGE, 21579 },
	{ 0xf863, 0xf866, PDF_CMAP_TABLE, 1763 },
	{ 0xf867, 0xf871, PDF_CMAP_RANGE, 21595 },
	{ 0xf872, 0xf872, PDF_CMAP_SINGLE, 8578 },
	{ 0xf873, 0xf877, PDF_CMAP_RANGE, 21606 },
	{ 0xf878, 0xf87c, PDF_CMAP_TABLE, 1767 },
	{ 0xf87d, 0xf87e, PDF_CMAP_RANGE, 21613 },
	{ 0xf880, 0xf881, PDF_CMAP_TABLE, 1772 },
	{ 0xf882, 0xf883, PDF_CMAP_RANGE, 21616 },
	{ 0xf884, 0xf886, PDF_CMAP_TABLE, 1774 },
	{ 0xf887, 0xf88c, PDF_CMAP_RANGE, 21619 },
	{ 0xf88d, 0xf88e, PDF_CMAP_TABLE, 1777 },
	{ 0xf88f, 0xf898, PDF_CMAP_RANGE, 21625 },
	{ 0xf899, 0xf899, PDF_CMAP_SINGLE, 8007 },
	{ 0xf89a, 0xf89c, PDF_CMAP_RANGE, 21635 },
	{ 0xf89d, 0xf89d, PDF_CMAP_SINGLE, 7965 },
	{ 0xf89e, 0xf89f, PDF_CMAP_RANGE, 21638 },
	{ 0xf8a0, 0xf8a0, PDF_CMAP_SINGLE, 9613 },
	{ 0xf940, 0xf94d, PDF_CMAP_RANGE, 21640 },
	{ 0xf94e, 0xf950, PDF_CMAP_TABLE, 1779 },
	{ 0xf951, 0xf958, PDF_CMAP_RANGE, 21654 },
	{ 0xf959, 0xf95a, PDF_CMAP_TABLE, 1782 },
	{ 0xf95b, 0xf95c, PDF_CMAP_RANGE, 21662 },
	{ 0xf95d, 0xf95e, PDF_CMAP_TABLE, 1784 },
	{ 0xf95f, 0xf966, PDF_CMAP_RANGE, 21664 },
	{ 0xf967, 0xf969, PDF_CMAP_TABLE, 1786 },
	{ 0xf96a, 0xf96b, PDF_CMAP_RANGE, 21673 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 9624 },
	{ 0xf96d, 0xf96e, PDF_CMAP_RANGE, 21675 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 8438 },
	{ 0xf970, 0xf97e, PDF_CMAP_RANGE, 21677 },
	{ 0xf980, 0xf984, PDF_CMAP_RANGE, 21692 },
	{ 0xf985, 0xf987, PDF_CMAP_TABLE, 1789 },
	{ 0xf988, 0xf990, PDF_CMAP_RANGE, 21698 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 9622 },
	{ 0xf992, 0xf995, PDF_CMAP_RANGE, 21707 },
	{ 0xf996, 0xf998, PDF_CMAP_TABLE, 1792 },
	{ 0xf999, 0xf9a0, PDF_CMAP_RANGE, 21712 },
	{ 0xfa40, 0xfa41, PDF_CMAP_RANGE, 21720 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 9629 },
	{ 0xfa43, 0xfa45, PDF_CMAP_RANGE, 21722 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 9630 },
	{ 0xfa47, 0xfa4b, PDF_CMAP_RANGE, 21725 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 9036 },
	{ 0xfa4d, 0xfa50, PDF_CMAP_RANGE, 21730 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 8004 },
	{ 0xfa52, 0xfa57, PDF_CMAP_RANGE, 21734 },
	{ 0xfa58, 0xfa59, PDF_CMAP_TABLE, 1795 },
	{ 0xfa5a, 0xfa5c, PDF_CMAP_RANGE, 21740 },
	{ 0xfa5d, 0xfa61, PDF_CMAP_TABLE, 1797 },
	{ 0xfa62, 0xfa6f, PDF_CMAP_RANGE, 21745 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 9634 },
	{ 0xfa71, 0xfa73, PDF_CMAP_RANGE, 21759 },
	{ 0xfa74, 0xfa77, PDF_CMAP_TABLE, 1802 },
	{ 0xfa78, 0xfa7e, PDF_CMAP_RANGE, 21763 },
	{ 0xfa80, 0xfa82, PDF_CMAP_RANGE, 21770 },
	{ 0xfa83, 0xfa84, PDF_CMAP_TABLE, 1806 },
	{ 0xfa85, 0xfa8c, PDF_CMAP_RANGE, 21773 },
	{ 0xfa8d, 0xfa8d, PDF_CMAP_SINGLE, 9636 },
	{ 0xfa8e, 0xfa8f, PDF_CMAP_RANGE, 21781 },
	{ 0xfa90, 0xfa91, PDF_CMAP_TABLE, 1808 },
	{ 0xfa92, 0xfa95, PDF_CMAP_RANGE, 21783 },
	{ 0xfa96, 0xfa98, PDF_CMAP_TABLE, 1810 },
	{ 0xfa99, 0xfaa0, PDF_CMAP_RANGE, 21787 },
	{ 0xfb40, 0xfb48, PDF_CMAP_RANGE, 21795 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 9640 },
	{ 0xfb4a, 0xfb51, PDF_CMAP_RANGE, 21804 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 9606 },
	{ 0xfb53, 0xfb56, PDF_CMAP_RANGE, 21812 },
	{ 0xfb57, 0xfb5b, PDF_CMAP_TABLE, 1813 },
	{ 0xfb5c, 0xfb74, PDF_CMAP_RANGE, 21817 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 9864 },
	{ 0xfb76, 0xfb78, PDF_CMAP_RANGE, 21842 },
	{ 0xfb79, 0xfb7e, PDF_CMAP_TABLE, 1818 },
	{ 0xfb80, 0xfb8f, PDF_CMAP_RANGE, 21847 },
	{ 0xfb90, 0xfb90, PDF_CMAP_SINGLE, 8207 },
	{ 0xfb91, 0xfb9b, PDF_CMAP_RANGE, 21863 },
	{ 0xfb9c, 0xfb9c, PDF_CMAP_SINGLE, 8305 },
	{ 0xfb9d, 0xfb9e, PDF_CMAP_RANGE, 21874 },
	{ 0xfb9f, 0xfba0, PDF_CMAP_TABLE, 1824 },
	{ 0xfc40, 0xfc43, PDF_CMAP_RANGE, 21877 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 9878 },
	{ 0xfc45, 0xfc48, PDF_CMAP_RANGE, 21881 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 9871 },
	{ 0xfc4a, 0xfc59, PDF_CMAP_RANGE, 21885 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 8922 },
	{ 0xfc5b, 0xfc62, PDF_CMAP_RANGE, 21901 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 7873 },
	{ 0xfc64, 0xfc67, PDF_CMAP_RANGE, 21909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 7859 },
	{ 0xfc69, 0xfc6e, PDF_CMAP_RANGE, 21913 },
	{ 0xfc6f, 0xfc71, PDF_CMAP_TABLE, 1826 },
	{ 0xfc72, 0xfc73, PDF_CMAP_RANGE, 21920 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 9833 },
	{ 0xfc75, 0xfc76, PDF_CMAP_RANGE, 21922 },
	{ 0xfc77, 0xfc78, PDF_CMAP_RANGE, 9752 },
	{ 0xfc79, 0xfc7e, PDF_CMAP_RANGE, 21924 },
	{ 0xfc80, 0xfc82, PDF_CMAP_RANGE, 21930 },
	{ 0xfc83, 0xfc83, PDF_CMAP_SINGLE, 9754 },
	{ 0xfc84, 0xfc89, PDF_CMAP_RANGE, 21933 },
	{ 0xfc8a, 0xfc8a, PDF_CMAP_SINGLE, 9843 },
	{ 0xfc8b, 0xfca0, PDF_CMAP_RANGE, 21939 },
	{ 0xfd40, 0xfd51, PDF_CMAP_RANGE, 21961 },
	{ 0xfd52, 0xfd53, PDF_CMAP_TABLE, 1829 },
	{ 0xfd54, 0xfd56, PDF_CMAP_RANGE, 21979 },
	{ 0xfd57, 0xfd5a, PDF_CMAP_TABLE, 1831 },
	{ 0xfd5b, 0xfd5e, PDF_CMAP_RANGE, 21983 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 9745 },
	{ 0xfd60, 0xfd61, PDF_CMAP_RANGE, 21987 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 9747 },
	{ 0xfd63, 0xfd64, PDF_CMAP_RANGE, 21989 },
	{ 0xfd65, 0xfd69, PDF_CMAP_TABLE, 1835 },
	{ 0xfd6a, 0xfd6b, PDF_CMAP_RANGE, 21992 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 9748 },
	{ 0xfd6d, 0xfd6f, PDF_CMAP_RANGE, 21994 },
	{ 0xfd70, 0xfd72, PDF_CMAP_TABLE, 1840 },
	{ 0xfd73, 0xfd77, PDF_CMAP_RANGE, 21998 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 8434 },
	{ 0xfd79, 0xfd7c, PDF_CMAP_RANGE, 22003 },
	{ 0xfd7d, 0xfd7e, PDF_CMAP_TABLE, 1843 },
	{ 0xfd80, 0xfd87, PDF_CMAP_RANGE, 22008 },
	{ 0xfd88, 0xfd88, PDF_CMAP_SINGLE, 8247 },
	{ 0xfd89, 0xfd8a, PDF_CMAP_RANGE, 22016 },
	{ 0xfd8b, 0xfd8b, PDF_CMAP_SINGLE, 8371 },
	{ 0xfd8c, 0xfd8e, PDF_CMAP_RANGE, 22018 },
	{ 0xfd8f, 0xfd90, PDF_CMAP_TABLE, 1845 },
	{ 0xfd91, 0xfd93, PDF_CMAP_RANGE, 22021 },
	{ 0xfd94, 0xfd94, PDF_CMAP_SINGLE, 7988 },
	{ 0xfd95, 0xfd9c, PDF_CMAP_RANGE, 22024 },
	{ 0xfd9d, 0xfda0, PDF_CMAP_TABLE, 1847 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 4697 },
	{ 0xfe41, 0xfe7e, PDF_CMAP_RANGE, 22032 },
	{ 0xfe80, 0xfea0, PDF_CMAP_RANGE, 22094 },
	{ 0xdd53, 0xdd53, PDF_CMAP_SINGLE, 8857 },
	{ 0xdd54, 0xdd54, PDF_CMAP_SINGLE, 9361 },
	{ 0xdd55, 0xdd55, PDF_CMAP_SINGLE, 9366 },
	{ 0xdd56, 0xdd56, PDF_CMAP_SINGLE, 9359 },
	{ 0xdd57, 0xdd57, PDF_CMAP_SINGLE, 9362 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 19647 },
	{ 0xdd59, 0xdd59, PDF_CMAP_SINGLE, 9367 },
	{ 0xdd5a, 0xdd5d, PDF_CMAP_RANGE, 19648 },
	{ 0xdd5e, 0xdd5e, PDF_CMAP_SINGLE, 8113 },
	{ 0xdd5f, 0xdd5f, PDF_CMAP_SINGLE, 19652 },
	{ 0xdd60, 0xdd60, PDF_CMAP_SINGLE, 9370 },
	{ 0xdd61, 0xdd61, PDF_CMAP_SINGLE, 19653 },
	{ 0xdd62, 0xdd62, PDF_CMAP_SINGLE, 9369 },
	{ 0xdd63, 0xdd63, PDF_CMAP_SINGLE, 19654 },
	{ 0xdd64, 0xdd64, PDF_CMAP_SINGLE, 8792 },
	{ 0xdd65, 0xdd65, PDF_CMAP_SINGLE, 9368 },
	{ 0xdd66, 0xdd6c, PDF_CMAP_RANGE, 19655 },
	{ 0xdd6d, 0xdd6d, PDF_CMAP_SINGLE, 9371 },
	{ 0xdd6e, 0xdd6e, PDF_CMAP_SINGLE, 19662 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 7945 },
	{ 0xdd70, 0xdd70, PDF_CMAP_SINGLE, 8422 },
	{ 0xdd71, 0xdd75, PDF_CMAP_RANGE, 19663 },
	{ 0xdd76, 0xdd76, PDF_CMAP_SINGLE, 8230 },
	{ 0xdd77, 0xdd77, PDF_CMAP_SINGLE, 9375 },
	{ 0xdd78, 0xdd78, PDF_CMAP_SINGLE, 8025 },
	{ 0xdd79, 0xdd7a, PDF_CMAP_RANGE, 9373 },
	{ 0xdd7b, 0xdd7e, PDF_CMAP_RANGE, 19668 },
	{ 0xdd80, 0xdd80, PDF_CMAP_SINGLE, 19672 },
	{ 0xdd81, 0xdd81, PDF_CMAP_SINGLE, 7995 },
	{ 0xdd82, 0xdd82, PDF_CMAP_SINGLE, 9372 },
	{ 0xdd83, 0xdd84, PDF_CMAP_RANGE, 19673 },
	{ 0xdd85, 0xdd85, PDF_CMAP_SINGLE, 7738 },
	{ 0xdd86, 0xdd86, PDF_CMAP_SINGLE, 8283 },
	{ 0xdd87, 0xdd8a, PDF_CMAP_RANGE, 19675 },
	{ 0xdd8b, 0xdd8b, PDF_CMAP_SINGLE, 8048 },
	{ 0xdd8c, 0xdd8e, PDF_CMAP_RANGE, 19679 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 9376 },
	{ 0xdd90, 0xdd93, PDF_CMAP_RANGE, 19682 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 8507 },
	{ 0xdd95, 0xdd96, PDF_CMAP_RANGE, 19686 },
	{ 0xdd97, 0xdd97, PDF_CMAP_SINGLE, 7943 },
	{ 0xdd98, 0xdd99, PDF_CMAP_RANGE, 19688 },
	{ 0xdd9a, 0xdd9a, PDF_CMAP_SINGLE, 8816 },
	{ 0xdd9b, 0xdd9b, PDF_CMAP_SINGLE, 8759 },
	{ 0xdd9c, 0xdd9d, PDF_CMAP_RANGE, 19690 },
	{ 0xdd9e, 0xdd9e, PDF_CMAP_SINGLE, 9426 },
	{ 0xdd9f, 0xdd9f, PDF_CMAP_SINGLE, 19692 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 8627 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 8773 },
	{ 0xde41, 0xde41, PDF_CMAP_SINGLE, 9377 },
	{ 0xde42, 0xde43, PDF_CMAP_RANGE, 19693 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 8872 },
	{ 0xde45, 0xde47, PDF_CMAP_RANGE, 19695 },
	{ 0xde48, 0xde48, PDF_CMAP_SINGLE, 8828 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 8112 },
	{ 0xde4a, 0xde4e, PDF_CMAP_RANGE, 19698 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 9378 },
	{ 0xde50, 0xde59, PDF_CMAP_RANGE, 19703 },
	{ 0xde5a, 0xde5a, PDF_CMAP_SINGLE, 8006 },
	{ 0xde5b, 0xde5b, PDF_CMAP_SINGLE, 19713 },
	{ 0xde5c, 0xde5c, PDF_CMAP_SINGLE, 9088 },
	{ 0xde5d, 0xde5d, PDF_CMAP_SINGLE, 9365 },
	{ 0xde5e, 0xde5e, PDF_CMAP_SINGLE, 19714 },
	{ 0xde5f, 0xde5f, PDF_CMAP_SINGLE, 9360 },
	{ 0xde60, 0xde6a, PDF_CMAP_RANGE, 19715 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 7728 },
	{ 0xde6c, 0xde6e, PDF_CMAP_RANGE, 19726 },
	{ 0xde6f, 0xde6f, PDF_CMAP_SINGLE, 7837 },
	{ 0xde70, 0xde70, PDF_CMAP_SINGLE, 7755 },
	{ 0xde71, 0xde71, PDF_CMAP_SINGLE, 7754 },
	{ 0xde72, 0xde72, PDF_CMAP_SINGLE, 8362 },
	{ 0xde73, 0xde7e, PDF_CMAP_RANGE, 19729 },
	{ 0xde80, 0xde91, PDF_CMAP_RANGE, 19741 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 9851 },
	{ 0xde93, 0xde9e, PDF_CMAP_RANGE, 19759 },
	{ 0xde9f, 0xde9f, PDF_CMAP_SINGLE, 9202 },
	{ 0xdea0, 0xdea0, PDF_CMAP_SINGLE, 19771 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdf40, 0xdf40, PDF_CMAP_SINGLE, 8830 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 19772 },
	{ 0xdf42, 0xdf42, PDF_CMAP_SINGLE, 8217 },
	{ 0xdf43, 0xdf4c, PDF_CMAP_RANGE, 19773 },
	{ 0xdf4d, 0xdf4d, PDF_CMAP_SINGLE, 8123 },
	{ 0xdf4e, 0xdf5b, PDF_CMAP_RANGE, 19783 },
	{ 0xdf5c, 0xdf5c, PDF_CMAP_SINGLE, 8787 },
	{ 0xdf5d, 0xdf5d, PDF_CMAP_SINGLE, 19797 },
	{ 0xdf5e, 0xdf5e, PDF_CMAP_SINGLE, 7998 },
	{ 0xdf5f, 0xdf5f, PDF_CMAP_SINGLE, 7846 },
	{ 0xdf60, 0xdf60, PDF_CMAP_SINGLE, 8590 },
	{ 0xdf61, 0xdf63, PDF_CMAP_RANGE, 19798 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 8684 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 19801 },
	{ 0xdf66, 0xdf66, PDF_CMAP_SINGLE, 7870 },
	{ 0xdf67, 0xdf67, PDF_CMAP_SINGLE, 19802 },
	{ 0xdf68, 0xdf68, PDF_CMAP_SINGLE, 8778 },
	{ 0xdf69, 0xdf6c, PDF_CMAP_RANGE, 19803 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 8499 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_RANGE, 19807 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 7812 },
	{ 0xdf75, 0xdf76, PDF_CMAP_RANGE, 19813 },
	{ 0xdf77, 0xdf77, PDF_CMAP_SINGLE, 8399 },
	{ 0xdf78, 0xdf78, PDF_CMAP_SINGLE, 8674 },
	{ 0xdf79, 0xdf79, PDF_CMAP_SINGLE, 19815 },
	{ 0xdf7a, 0xdf7a, PDF_CMAP_SINGLE, 8719 },
	{ 0xdf7b, 0xdf7b, PDF_CMAP_SINGLE, 19816 },
	{ 0xdf7c, 0xdf7c, PDF_CMAP_SINGLE, 8233 },
	{ 0xdf7d, 0xdf7d, PDF_CMAP_SINGLE, 19817 },
	{ 0xdf7e, 0xdf7e, PDF_CMAP_SINGLE, 8307 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 8021 },
	{ 0xdf81, 0xdf82, PDF_CMAP_RANGE, 19818 },
	{ 0xdf83, 0xdf83, PDF_CMAP_SINGLE, 9201 },
	{ 0xdf84, 0xdf84, PDF_CMAP_SINGLE, 19820 },
	{ 0xdf85, 0xdf85, PDF_CMAP_SINGLE, 7750 },
	{ 0xdf86, 0xdf88, PDF_CMAP_RANGE, 19821 },
	{ 0xdf89, 0xdf89, PDF_CMAP_SINGLE, 8291 },
	{ 0xdf8a, 0xdf8a, PDF_CMAP_SINGLE, 9203 },
	{ 0xdf8b, 0xdfa0, PDF_CMAP_RANGE, 19824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe040, 0xe04f, PDF_CMAP_RANGE, 19846 },
	{ 0xe050, 0xe050, PDF_CMAP_SINGLE, 8990 },
	{ 0xe051, 0xe05c, PDF_CMAP_RANGE, 19862 },
	{ 0xe05d, 0xe05d, PDF_CMAP_SINGLE, 8755 },
	{ 0xe05e, 0xe068, PDF_CMAP_RANGE, 19874 },
	{ 0xe069, 0xe069, PDF_CMAP_SINGLE, 8992 },
	{ 0xe06a, 0xe06b, PDF_CMAP_RANGE, 19885 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 8647 },
	{ 0xe06d, 0xe074, PDF_CMAP_RANGE, 19887 },
	{ 0xe075, 0xe075, PDF_CMAP_SINGLE, 8892 },
	{ 0xe076, 0xe076, PDF_CMAP_SINGLE, 19895 },
	{ 0xe077, 0xe077, PDF_CMAP_SINGLE, 8988 },
	{ 0xe078, 0xe078, PDF_CMAP_SINGLE, 19896 },
	{ 0xe079, 0xe079, PDF_CMAP_SINGLE, 8785 },
	{ 0xe07a, 0xe07e, PDF_CMAP_RANGE, 19897 },
	{ 0xe080, 0xe086, PDF_CMAP_RANGE, 19902 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 7867 },
	{ 0xe088, 0xe08c, PDF_CMAP_RANGE, 19909 },
	{ 0xe08d, 0xe08d, PDF_CMAP_SINGLE, 8839 },
	{ 0xe08e, 0xe08e, PDF_CMAP_SINGLE, 19914 },
	{ 0xe08f, 0xe08f, PDF_CMAP_SINGLE, 8237 },
	{ 0xe090, 0xe090, PDF_CMAP_SINGLE, 7851 },
	{ 0xe091, 0xe091, PDF_CMAP_SINGLE, 19915 },
	{ 0xe092, 0xe092, PDF_CMAP_SINGLE, 8989 },
	{ 0xe093, 0xe093, PDF_CMAP_SINGLE, 19916 },
	{ 0xe094, 0xe094, PDF_CMAP_SINGLE, 8991 },
	{ 0xe095, 0xe096, PDF_CMAP_RANGE, 19917 },
	{ 0xe097, 0xe097, PDF_CMAP_SINGLE, 8987 },
	{ 0xe098, 0xe0a0, PDF_CMAP_RANGE, 19919 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe140, 0xe141, PDF_CMAP_RANGE, 19928 },
	{ 0xe142, 0xe142, PDF_CMAP_SINGLE, 8993 },
	{ 0xe143, 0xe163, PDF_CMAP_RANGE, 19930 },
	{ 0xe164, 0xe164, PDF_CMAP_SINGLE, 8789 },
	{ 0xe165, 0xe167, PDF_CMAP_RANGE, 19963 },
	{ 0xe168, 0xe168, PDF_CMAP_SINGLE, 7823 },
	{ 0xe169, 0xe173, PDF_CMAP_RANGE, 19966 },
	{ 0xe174, 0xe174, PDF_CMAP_SINGLE, 8716 },
	{ 0xe175, 0xe175, PDF_CMAP_SINGLE, 8100 },
	{ 0xe176, 0xe17e, PDF_CMAP_RANGE, 19977 },
	{ 0xe180, 0xe183, PDF_CMAP_RANGE, 19986 },
	{ 0xe184, 0xe184, PDF_CMAP_SINGLE, 8347 },
	{ 0xe185, 0xe185, PDF_CMAP_SINGLE, 8664 },
	{ 0xe186, 0xe186, PDF_CMAP_SINGLE, 19990 },
	{ 0xe187, 0xe187, PDF_CMAP_SINGLE, 9722 },
	{ 0xe188, 0xe188, PDF_CMAP_SINGLE, 19991 },
	{ 0xe189, 0xe189, PDF_CMAP_SINGLE, 9721 },
	{ 0xe18a, 0xe18b, PDF_CMAP_RANGE, 19992 },
	{ 0xe18c, 0xe18c, PDF_CMAP_SINGLE, 8500 },
	{ 0xe18d, 0xe18d, PDF_CMAP_SINGLE, 19994 },
	{ 0xe18e, 0xe190, PDF_CMAP_RANGE, 9459 },
	{ 0xe191, 0xe191, PDF_CMAP_SINGLE, 9464 },
	{ 0xe192, 0xe192, PDF_CMAP_SINGLE, 19995 },
	{ 0xe193, 0xe193, PDF_CMAP_SINGLE, 9463 },
	{ 0xe194, 0xe194, PDF_CMAP_SINGLE, 7880 },
	{ 0xe195, 0xe195, PDF_CMAP_SINGLE, 9462 },
	{ 0xe196, 0xe197, PDF_CMAP_RANGE, 19996 },
	{ 0xe198, 0xe198, PDF_CMAP_SINGLE, 8832 },
	{ 0xe199, 0xe19d, PDF_CMAP_RANGE, 19998 },
	{ 0xe19e, 0xe19e, PDF_CMAP_SINGLE, 7877 },
	{ 0xe19f, 0xe19f, PDF_CMAP_SINGLE, 9467 },
	{ 0xe1a0, 0xe1a0, PDF_CMAP_SINGLE, 20003 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe240, 0xe240, PDF_CMAP_SINGLE, 20004 },
	{ 0xe241, 0xe241, PDF_CMAP_SINGLE, 9466 },
	{ 0xe242, 0xe242, PDF_CMAP_SINGLE, 20005 },
	{ 0xe243, 0xe243, PDF_CMAP_SINGLE, 7917 },
	{ 0xe244, 0xe24e, PDF_CMAP_RANGE, 20006 },
	{ 0xe24f, 0xe24f, PDF_CMAP_SINGLE, 9469 },
	{ 0xe250, 0xe250, PDF_CMAP_SINGLE, 20017 },
	{ 0xe251, 0xe251, PDF_CMAP_SINGLE, 9465 },
	{ 0xe252, 0xe252, PDF_CMAP_SINGLE, 20018 },
	{ 0xe253, 0xe253, PDF_CMAP_SINGLE, 9470 },
	{ 0xe254, 0xe254, PDF_CMAP_SINGLE, 8397 },
	{ 0xe255, 0xe259, PDF_CMAP_RANGE, 20019 },
	{ 0xe25a, 0xe25a, PDF_CMAP_SINGLE, 9480 },
	{ 0xe25b, 0xe25b, PDF_CMAP_SINGLE, 9476 },
	{ 0xe25c, 0xe25d, PDF_CMAP_RANGE, 20024 },
	{ 0xe25e, 0xe25e, PDF_CMAP_SINGLE, 9478 },
	{ 0xe25f, 0xe261, PDF_CMAP_RANGE, 20026 },
	{ 0xe262, 0xe262, PDF_CMAP_SINGLE, 9471 },
	{ 0xe263, 0xe263, PDF_CMAP_SINGLE, 8336 },
	{ 0xe264, 0xe266, PDF_CMAP_RANGE, 20029 },
	{ 0xe267, 0xe267, PDF_CMAP_SINGLE, 7901 },
	{ 0xe268, 0xe268, PDF_CMAP_SINGLE, 7973 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 20032 },
	{ 0xe26a, 0xe26a, PDF_CMAP_SINGLE, 9475 },
	{ 0xe26b, 0xe26b, PDF_CMAP_SINGLE, 9474 },
	{ 0xe26c, 0xe26d, PDF_CMAP_RANGE, 20033 },
	{ 0xe26e, 0xe26e, PDF_CMAP_SINGLE, 7802 },
	{ 0xe26f, 0xe26f, PDF_CMAP_SINGLE, 8358 },
	{ 0xe270, 0xe277, PDF_CMAP_RANGE, 20035 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 8149 },
	{ 0xe279, 0xe27c, PDF_CMAP_RANGE, 20043 },
	{ 0xe27d, 0xe27d, PDF_CMAP_SINGLE, 7953 },
	{ 0xe27e, 0xe27e, PDF_CMAP_SINGLE, 20047 },
	{ 0xe280, 0xe280, PDF_CMAP_SINGLE, 9479 },
	{ 0xe281, 0xe281, PDF_CMAP_SINGLE, 9472 },
	{ 0xe282, 0xe282, PDF_CMAP_SINGLE, 9477 },
	{ 0xe283, 0xe288, PDF_CMAP_RANGE, 20048 },
	{ 0xe289, 0xe289, PDF_CMAP_SINGLE, 9497 },
	{ 0xe28a, 0xe28a, PDF_CMAP_SINGLE, 20054 },
	{ 0xe28b, 0xe28b, PDF_CMAP_SINGLE, 9493 },
	{ 0xe28c, 0xe28d, PDF_CMAP_RANGE, 20055 },
	{ 0xe28e, 0xe28e, PDF_CMAP_SINGLE, 9484 },
	{ 0xe28f, 0xe28f, PDF_CMAP_SINGLE, 8241 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 20057 },
	{ 0xe292, 0xe292, PDF_CMAP_SINGLE, 9483 },
	{ 0xe293, 0xe293, PDF_CMAP_SINGLE, 9487 },
	{ 0xe294, 0xe294, PDF_CMAP_SINGLE, 9498 },
	{ 0xe295, 0xe295, PDF_CMAP_SINGLE, 9481 },
	{ 0xe296, 0xe297, PDF_CMAP_RANGE, 20059 },
	{ 0xe298, 0xe298, PDF_CMAP_SINGLE, 9486 },
	{ 0xe299, 0xe299, PDF_CMAP_SINGLE, 8756 },
	{ 0xe29a, 0xe29a, PDF_CMAP_SINGLE, 9491 },
	{ 0xe29b, 0xe29b, PDF_CMAP_SINGLE, 8064 },
	{ 0xe29c, 0xe29f, PDF_CMAP_RANGE, 20061 },
	{ 0xe2a0, 0xe2a0, PDF_CMAP_SINGLE, 9473 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe340, 0xe341, PDF_CMAP_RANGE, 20065 },
	{ 0xe342, 0xe342, PDF_CMAP_SINGLE, 9495 },
	{ 0xe343, 0xe343, PDF_CMAP_SINGLE, 9494 },
	{ 0xe344, 0xe346, PDF_CMAP_RANGE, 20067 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 9496 },
	{ 0xe348, 0xe34a, PDF_CMAP_RANGE, 20070 },
	{ 0xe34b, 0xe34b, PDF_CMAP_SINGLE, 7766 },
	{ 0xe34c, 0xe34e, PDF_CMAP_RANGE, 20073 },
	{ 0xe34f, 0xe34f, PDF_CMAP_SINGLE, 9485 },
	{ 0xe350, 0xe350, PDF_CMAP_SINGLE, 20076 },
	{ 0xe351, 0xe351, PDF_CMAP_SINGLE, 8403 },
	{ 0xe352, 0xe353, PDF_CMAP_RANGE, 20077 },
	{ 0xe354, 0xe354, PDF_CMAP_SINGLE, 8314 },
	{ 0xe355, 0xe355, PDF_CMAP_SINGLE, 8398 },
	{ 0xe356, 0xe357, PDF_CMAP_RANGE, 20079 },
	{ 0xe358, 0xe358, PDF_CMAP_SINGLE, 9488 },
	{ 0xe359, 0xe35b, PDF_CMAP_RANGE, 20081 },
	{ 0xe35c, 0xe35c, PDF_CMAP_SINGLE, 7765 },
	{ 0xe35d, 0xe35f, PDF_CMAP_RANGE, 20084 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 9482 },
	{ 0xe361, 0xe365, PDF_CMAP_RANGE, 20087 },
	{ 0xe366, 0xe367, PDF_CMAP_RANGE, 9489 },
	{ 0xe368, 0xe370, PDF_CMAP_RANGE, 20092 },
	{ 0xe371, 0xe371, PDF_CMAP_SINGLE, 8106 },
	{ 0xe372, 0xe372, PDF_CMAP_SINGLE, 20101 },
	{ 0xe373, 0xe373, PDF_CMAP_SINGLE, 9502 },
	{ 0xe374, 0xe374, PDF_CMAP_SINGLE, 7967 },
	{ 0xe375, 0xe377, PDF_CMAP_RANGE, 20102 },
	{ 0xe378, 0xe378, PDF_CMAP_SINGLE, 9517 },
	{ 0xe379, 0xe379, PDF_CMAP_SINGLE, 8733 },
	{ 0xe37a, 0xe37b, PDF_CMAP_RANGE, 20105 },
	{ 0xe37c, 0xe37c, PDF_CMAP_SINGLE, 9522 },
	{ 0xe37d, 0xe37d, PDF_CMAP_SINGLE, 20107 },
	{ 0xe37e, 0xe37e, PDF_CMAP_SINGLE, 8571 },
	{ 0xe380, 0xe389, PDF_CMAP_RANGE, 20108 },
	{ 0xe38a, 0xe38a, PDF_CMAP_SINGLE, 8623 },
	{ 0xe38b, 0xe38b, PDF_CMAP_SINGLE, 20118 },
	{ 0xe38c, 0xe38c, PDF_CMAP_SINGLE, 9516 },
	{ 0xe38d, 0xe38e, PDF_CMAP_RANGE, 20119 },
	{ 0xe38f, 0xe38f, PDF_CMAP_SINGLE, 9512 },
	{ 0xe390, 0xe390, PDF_CMAP_SINGLE, 20121 },
	{ 0xe391, 0xe391, PDF_CMAP_SINGLE, 8332 },
	{ 0xe392, 0xe392, PDF_CMAP_SINGLE, 20122 },
	{ 0xe393, 0xe393, PDF_CMAP_SINGLE, 9519 },
	{ 0xe394, 0xe394, PDF_CMAP_SINGLE, 20123 },
	{ 0xe395, 0xe395, PDF_CMAP_SINGLE, 8636 },
	{ 0xe396, 0xe398, PDF_CMAP_RANGE, 20124 },
	{ 0xe399, 0xe399, PDF_CMAP_SINGLE, 9501 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 20127 },
	{ 0xe39c, 0xe39c, PDF_CMAP_SINGLE, 9525 },
	{ 0xe39d, 0xe39d, PDF_CMAP_SINGLE, 20129 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 8717 },
	{ 0xe39f, 0xe39f, PDF_CMAP_SINGLE, 9510 },
	{ 0xe3a0, 0xe3a0, PDF_CMAP_SINGLE, 20130 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe440, 0xe440, PDF_CMAP_SINGLE, 9524 },
	{ 0xe441, 0xe441, PDF_CMAP_SINGLE, 9514 },
	{ 0xe442, 0xe442, PDF_CMAP_SINGLE, 9503 },
	{ 0xe443, 0xe443, PDF_CMAP_SINGLE, 9521 },
	{ 0xe444, 0xe444, PDF_CMAP_SINGLE, 9500 },
	{ 0xe445, 0xe447, PDF_CMAP_RANGE, 20131 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 9509 },
	{ 0xe449, 0xe44d, PDF_CMAP_RANGE, 20134 },
	{ 0xe44e, 0xe44e, PDF_CMAP_SINGLE, 8653 },
	{ 0xe44f, 0xe44f, PDF_CMAP_SINGLE, 20139 },
	{ 0xe450, 0xe450, PDF_CMAP_SINGLE, 8666 },
	{ 0xe451, 0xe451, PDF_CMAP_SINGLE, 20140 },
	{ 0xe452, 0xe452, PDF_CMAP_SINGLE, 8562 },
	{ 0xe453, 0xe453, PDF_CMAP_SINGLE, 9534 },
	{ 0xe454, 0xe457, PDF_CMAP_RANGE, 20141 },
	{ 0xe458, 0xe458, PDF_CMAP_SINGLE, 8271 },
	{ 0xe459, 0xe459, PDF_CMAP_SINGLE, 20145 },
	{ 0xe45a, 0xe45a, PDF_CMAP_SINGLE, 9539 },
	{ 0xe45b, 0xe45b, PDF_CMAP_SINGLE, 20146 },
	{ 0xe45c, 0xe45c, PDF_CMAP_SINGLE, 8663 },
	{ 0xe45d, 0xe45d, PDF_CMAP_SINGLE, 20147 },
	{ 0xe45e, 0xe45e, PDF_CMAP_SINGLE, 7740 },
	{ 0xe45f, 0xe461, PDF_CMAP_RANGE, 20148 },
	{ 0xe462, 0xe462, PDF_CMAP_SINGLE, 9513 },
	{ 0xe463, 0xe464, PDF_CMAP_RANGE, 20151 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 9505 },
	{ 0xe466, 0xe467, PDF_CMAP_RANGE, 20153 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 7935 },
	{ 0xe469, 0xe472, PDF_CMAP_RANGE, 20155 },
	{ 0xe473, 0xe473, PDF_CMAP_SINGLE, 9535 },
	{ 0xe474, 0xe474, PDF_CMAP_SINGLE, 20165 },
	{ 0xe475, 0xe475, PDF_CMAP_SINGLE, 9540 },
	{ 0xe476, 0xe478, PDF_CMAP_RANGE, 20166 },
	{ 0xe479, 0xe479, PDF_CMAP_SINGLE, 9507 },
	{ 0xe47a, 0xe47a, PDF_CMAP_SINGLE, 7824 },
	{ 0xe47b, 0xe47b, PDF_CMAP_SINGLE, 9530 },
	{ 0xe47c, 0xe47c, PDF_CMAP_SINGLE, 9541 },
	{ 0xe47d, 0xe47d, PDF_CMAP_SINGLE, 20169 },
	{ 0xe47e, 0xe47e, PDF_CMAP_SINGLE, 9533 },
	{ 0xe480, 0xe480, PDF_CMAP_SINGLE, 20170 },
	{ 0xe481, 0xe481, PDF_CMAP_SINGLE, 8385 },
	{ 0xe482, 0xe483, PDF_CMAP_RANGE, 20171 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 8451 },
	{ 0xe485, 0xe485, PDF_CMAP_SINGLE, 9504 },
	{ 0xe486, 0xe486, PDF_CMAP_SINGLE, 9532 },
	{ 0xe487, 0xe487, PDF_CMAP_SINGLE, 9531 },
	{ 0xe488, 0xe488, PDF_CMAP_SINGLE, 9528 },
	{ 0xe489, 0xe48c, PDF_CMAP_RANGE, 20173 },
	{ 0xe48d, 0xe48d, PDF_CMAP_SINGLE, 9536 },
	{ 0xe48e, 0xe48e, PDF_CMAP_SINGLE, 20177 },
	{ 0xe48f, 0xe48f, PDF_CMAP_SINGLE, 8141 },
	{ 0xe490, 0xe492, PDF_CMAP_RANGE, 20178 },
	{ 0xe493, 0xe493, PDF_CMAP_SINGLE, 7960 },
	{ 0xe494, 0xe497, PDF_CMAP_RANGE, 20181 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 9547 },
	{ 0xe499, 0xe49c, PDF_CMAP_RANGE, 20185 },
	{ 0xe49d, 0xe49d, PDF_CMAP_SINGLE, 9543 },
	{ 0xe49e, 0xe49f, PDF_CMAP_RANGE, 9551 },
	{ 0xe4a0, 0xe4a0, PDF_CMAP_SINGLE, 20189 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe540, 0xe545, PDF_CMAP_RANGE, 20190 },
	{ 0xe546, 0xe546, PDF_CMAP_SINGLE, 8880 },
	{ 0xe547, 0xe547, PDF_CMAP_SINGLE, 20196 },
	{ 0xe548, 0xe548, PDF_CMAP_SINGLE, 9542 },
	{ 0xe549, 0xe54a, PDF_CMAP_RANGE, 20197 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 9548 },
	{ 0xe54c, 0xe54d, PDF_CMAP_RANGE, 20199 },
	{ 0xe54e, 0xe54e, PDF_CMAP_SINGLE, 7834 },
	{ 0xe54f, 0xe54f, PDF_CMAP_SINGLE, 9554 },
	{ 0xe550, 0xe550, PDF_CMAP_SINGLE, 9520 },
	{ 0xe551, 0xe551, PDF_CMAP_SINGLE, 9545 },
	{ 0xe552, 0xe554, PDF_CMAP_RANGE, 20201 },
	{ 0xe555, 0xe555, PDF_CMAP_SINGLE, 9553 },
	{ 0xe556, 0xe556, PDF_CMAP_SINGLE, 7882 },
	{ 0xe557, 0xe557, PDF_CMAP_SINGLE, 20204 },
	{ 0xe558, 0xe558, PDF_CMAP_SINGLE, 8402 },
	{ 0xe559, 0xe55b, PDF_CMAP_RANGE, 20205 },
	{ 0xe55c, 0xe55c, PDF_CMAP_SINGLE, 8120 },
	{ 0xe55d, 0xe55d, PDF_CMAP_SINGLE, 20208 },
	{ 0xe55e, 0xe55e, PDF_CMAP_SINGLE, 8313 },
	{ 0xe55f, 0xe560, PDF_CMAP_RANGE, 20209 },
	{ 0xe561, 0xe561, PDF_CMAP_SINGLE, 8619 },
	{ 0xe562, 0xe563, PDF_CMAP_RANGE, 20211 },
	{ 0xe564, 0xe564, PDF_CMAP_SINGLE, 9549 },
	{ 0xe565, 0xe565, PDF_CMAP_SINGLE, 7845 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 20213 },
	{ 0xe568, 0xe568, PDF_CMAP_SINGLE, 8268 },
	{ 0xe569, 0xe569, PDF_CMAP_SINGLE, 8320 },
	{ 0xe56a, 0xe56b, PDF_CMAP_RANGE, 20215 },
	{ 0xe56c, 0xe56c, PDF_CMAP_SINGLE, 9837 },
	{ 0xe56d, 0xe56d, PDF_CMAP_SINGLE, 20217 },
	{ 0xe56e, 0xe56e, PDF_CMAP_SINGLE, 9527 },
	{ 0xe56f, 0xe574, PDF_CMAP_RANGE, 20218 },
	{ 0xe575, 0xe575, PDF_CMAP_SINGLE, 9546 },
	{ 0xe576, 0xe576, PDF_CMAP_SINGLE, 8632 },
	{ 0xe577, 0xe577, PDF_CMAP_SINGLE, 20224 },
	{ 0xe578, 0xe578, PDF_CMAP_SINGLE, 9550 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 20225 },
	{ 0xe57b, 0xe57b, PDF_CMAP_SINGLE, 9468 },
	{ 0xe57c, 0xe57c, PDF_CMAP_SINGLE, 9556 },
	{ 0xe57d, 0xe57e, PDF_CMAP_RANGE, 20227 },
	{ 0xe580, 0xe580, PDF_CMAP_SINGLE, 20229 },
	{ 0xe581, 0xe581, PDF_CMAP_SINGLE, 7996 },
	{ 0xe582, 0xe582, PDF_CMAP_SINGLE, 20230 },
	{ 0xe583, 0xe583, PDF_CMAP_SINGLE, 7893 },
	{ 0xe584, 0xe589, PDF_CMAP_RANGE, 20231 },
	{ 0xe58a, 0xe58a, PDF_CMAP_SINGLE, 9558 },
	{ 0xe58b, 0xe58d, PDF_CMAP_RANGE, 20237 },
	{ 0xe58e, 0xe58e, PDF_CMAP_SINGLE, 8808 },
	{ 0xe58f, 0xe590, PDF_CMAP_RANGE, 20240 },
	{ 0xe591, 0xe591, PDF_CMAP_SINGLE, 7894 },
	{ 0xe592, 0xe599, PDF_CMAP_RANGE, 20242 },
	{ 0xe59a, 0xe59a, PDF_CMAP_SINGLE, 9559 },
	{ 0xe59b, 0xe59b, PDF_CMAP_SINGLE, 9555 },
	{ 0xe59c, 0xe59e, PDF_CMAP_RANGE, 20250 },
	{ 0xe59f, 0xe59f, PDF_CMAP_SINGLE, 9544 },
	{ 0xe5a0, 0xe5a0, PDF_CMAP_SINGLE, 20253 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe640, 0xe640, PDF_CMAP_SINGLE, 8412 },
	{ 0xe641, 0xe643, PDF_CMAP_RANGE, 20254 },
	{ 0xe644, 0xe644, PDF_CMAP_SINGLE, 9561 },
	{ 0xe645, 0xe648, PDF_CMAP_RANGE, 20257 },
	{ 0xe649, 0xe649, PDF_CMAP_SINGLE, 8087 },
	{ 0xe64a, 0xe64a, PDF_CMAP_SINGLE, 9557 },
	{ 0xe64b, 0xe64d, PDF_CMAP_RANGE, 20261 },
	{ 0xe64e, 0xe64e, PDF_CMAP_SINGLE, 8829 },
	{ 0xe64f, 0xe651, PDF_CMAP_RANGE, 20264 },
	{ 0xe652, 0xe652, PDF_CMAP_SINGLE, 9598 },
	{ 0xe653, 0xe655, PDF_CMAP_RANGE, 20267 },
	{ 0xe656, 0xe656, PDF_CMAP_SINGLE, 8316 },
	{ 0xe657, 0xe657, PDF_CMAP_SINGLE, 20270 },
	{ 0xe658, 0xe658, PDF_CMAP_SINGLE, 9562 },
	{ 0xe659, 0xe65a, PDF_CMAP_RANGE, 20271 },
	{ 0xe65b, 0xe65b, PDF_CMAP_SINGLE, 9566 },
	{ 0xe65c, 0xe65d, PDF_CMAP_RANGE, 20273 },
	{ 0xe65e, 0xe65e, PDF_CMAP_SINGLE, 7732 },
	{ 0xe65f, 0xe668, PDF_CMAP_RANGE, 20275 },
	{ 0xe669, 0xe669, PDF_CMAP_SINGLE, 8542 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 20285 },
	{ 0xe66b, 0xe66b, PDF_CMAP_SINGLE, 9568 },
	{ 0xe66c, 0xe674, PDF_CMAP_RANGE, 20286 },
	{ 0xe675, 0xe675, PDF_CMAP_SINGLE, 8610 },
	{ 0xe676, 0xe676, PDF_CMAP_SINGLE, 9044 },
	{ 0xe677, 0xe678, PDF_CMAP_RANGE, 20295 },
	{ 0xe679, 0xe679, PDF_CMAP_SINGLE, 9571 },
	{ 0xe67a, 0xe67a, PDF_CMAP_SINGLE, 9511 },
	{ 0xe67b, 0xe67b, PDF_CMAP_SINGLE, 20297 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 9518 },
	{ 0xe67d, 0xe67d, PDF_CMAP_SINGLE, 9560 },
	{ 0xe67e, 0xe67e, PDF_CMAP_SINGLE, 20298 },
	{ 0xe680, 0xe680, PDF_CMAP_SINGLE, 7963 },
	{ 0xe681, 0xe681, PDF_CMAP_SINGLE, 20299 },
	{ 0xe682, 0xe682, PDF_CMAP_SINGLE, 8835 },
	{ 0xe683, 0xe683, PDF_CMAP_SINGLE, 20300 },
	{ 0xe684, 0xe684, PDF_CMAP_SINGLE, 9572 },
	{ 0xe685, 0xe686, PDF_CMAP_RANGE, 20301 },
	{ 0xe687, 0xe687, PDF_CMAP_SINGLE, 8352 },
	{ 0xe688, 0xe688, PDF_CMAP_SINGLE, 20303 },
	{ 0xe689, 0xe689, PDF_CMAP_SINGLE, 9573 },
	{ 0xe68a, 0xe68b, PDF_CMAP_RANGE, 20304 },
	{ 0xe68c, 0xe68c, PDF_CMAP_SINGLE, 9569 },
	{ 0xe68d, 0xe692, PDF_CMAP_RANGE, 20306 },
	{ 0xe693, 0xe693, PDF_CMAP_SINGLE, 9570 },
	{ 0xe694, 0xe696, PDF_CMAP_RANGE, 20312 },
	{ 0xe697, 0xe697, PDF_CMAP_SINGLE, 9580 },
	{ 0xe698, 0xe69a, PDF_CMAP_RANGE, 20315 },
	{ 0xe69b, 0xe69b, PDF_CMAP_SINGLE, 9581 },
	{ 0xe69c, 0xe69c, PDF_CMAP_SINGLE, 8224 },
	{ 0xe69d, 0xe69e, PDF_CMAP_RANGE, 20318 },
	{ 0xe69f, 0xe69f, PDF_CMAP_SINGLE, 9567 },
	{ 0xe6a0, 0xe6a0, PDF_CMAP_SINGLE, 9578 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 20320 },
	{ 0xe743, 0xe743, PDF_CMAP_SINGLE, 9582 },
	{ 0xe744, 0xe747, PDF_CMAP_RANGE, 20323 },
	{ 0xe748, 0xe748, PDF_CMAP_SINGLE, 9529 },
	{ 0xe749, 0xe749, PDF_CMAP_SINGLE, 9564 },
	{ 0xe74a, 0xe74c, PDF_CMAP_RANGE, 20327 },
	{ 0xe74d, 0xe74e, PDF_CMAP_RANGE, 9576 },
	{ 0xe74f, 0xe74f, PDF_CMAP_SINGLE, 9579 },
	{ 0xe750, 0xe750, PDF_CMAP_SINGLE, 7791 },
	{ 0xe751, 0xe751, PDF_CMAP_SINGLE, 20330 },
	{ 0xe752, 0xe752, PDF_CMAP_SINGLE, 8132 },
	{ 0xe753, 0xe753, PDF_CMAP_SINGLE, 9575 },
	{ 0xe754, 0xe754, PDF_CMAP_SINGLE, 20331 },
	{ 0xe755, 0xe755, PDF_CMAP_SINGLE, 9563 },
	{ 0xe756, 0xe758, PDF_CMAP_RANGE, 20332 },
	{ 0xe759, 0xe759, PDF_CMAP_SINGLE, 9757 },
	{ 0xe75a, 0xe765, PDF_CMAP_RANGE, 20335 },
	{ 0xe766, 0xe766, PDF_CMAP_SINGLE, 9515 },
	{ 0xe767, 0xe767, PDF_CMAP_SINGLE, 20347 },
	{ 0xe768, 0xe768, PDF_CMAP_SINGLE, 9585 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 20348 },
	{ 0xe76a, 0xe76a, PDF_CMAP_SINGLE, 9591 },
	{ 0xe76b, 0xe773, PDF_CMAP_RANGE, 20349 },
	{ 0xe774, 0xe774, PDF_CMAP_SINGLE, 9506 },
	{ 0xe775, 0xe77b, PDF_CMAP_RANGE, 20358 },
	{ 0xe77c, 0xe77c, PDF_CMAP_SINGLE, 9523 },
	{ 0xe77d, 0xe77e, PDF_CMAP_RANGE, 20365 },
	{ 0xe780, 0xe781, PDF_CMAP_RANGE, 20367 },
	{ 0xe782, 0xe782, PDF_CMAP_SINGLE, 8234 },
	{ 0xe783, 0xe783, PDF_CMAP_SINGLE, 20369 },
	{ 0xe784, 0xe784, PDF_CMAP_SINGLE, 9526 },
	{ 0xe785, 0xe785, PDF_CMAP_SINGLE, 9587 },
	{ 0xe786, 0xe786, PDF_CMAP_SINGLE, 9583 },
	{ 0xe787, 0xe789, PDF_CMAP_RANGE, 20370 },
	{ 0xe78a, 0xe78a, PDF_CMAP_SINGLE, 8851 },
	{ 0xe78b, 0xe78b, PDF_CMAP_SINGLE, 9592 },
	{ 0xe78c, 0xe78e, PDF_CMAP_RANGE, 20373 },
	{ 0xe78f, 0xe78f, PDF_CMAP_SINGLE, 9584 },
	{ 0xe790, 0xe791, PDF_CMAP_RANGE, 20376 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 9589 },
	{ 0xe793, 0xe797, PDF_CMAP_RANGE, 20378 },
	{ 0xe798, 0xe799, PDF_CMAP_RANGE, 9537 },
	{ 0xe79a, 0xe79a, PDF_CMAP_SINGLE, 9565 },
	{ 0xe79b, 0xe79f, PDF_CMAP_RANGE, 20383 },
	{ 0xe7a0, 0xe7a0, PDF_CMAP_SINGLE, 8218 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe840, 0xe842, PDF_CMAP_RANGE, 20388 },
	{ 0xe843, 0xe843, PDF_CMAP_SINGLE, 9594 },
	{ 0xe844, 0xe844, PDF_CMAP_SINGLE, 8198 },
	{ 0xe845, 0xe845, PDF_CMAP_SINGLE, 20391 },
	{ 0xe846, 0xe846, PDF_CMAP_SINGLE, 8567 },
	{ 0xe847, 0xe848, PDF_CMAP_RANGE, 20392 },
	{ 0xe849, 0xe849, PDF_CMAP_SINGLE, 9499 },
	{ 0xe84a, 0xe84a, PDF_CMAP_SINGLE, 20394 },
	{ 0xe84b, 0xe84b, PDF_CMAP_SINGLE, 9508 },
	{ 0xe84c, 0xe84e, PDF_CMAP_RANGE, 20395 },
	{ 0xe84f, 0xe84f, PDF_CMAP_SINGLE, 9595 },
	{ 0xe850, 0xe853, PDF_CMAP_RANGE, 20398 },
	{ 0xe854, 0xe854, PDF_CMAP_SINGLE, 8867 },
	{ 0xe855, 0xe859, PDF_CMAP_RANGE, 20402 },
	{ 0xe85a, 0xe85a, PDF_CMAP_SINGLE, 9593 },
	{ 0xe85b, 0xe85b, PDF_CMAP_SINGLE, 20407 },
	{ 0xe85c, 0xe85c, PDF_CMAP_SINGLE, 9574 },
	{ 0xe85d, 0xe861, PDF_CMAP_RANGE, 20408 },
	{ 0xe862, 0xe862, PDF_CMAP_SINGLE, 8083 },
	{ 0xe863, 0xe863, PDF_CMAP_SINGLE, 20413 },
	{ 0xe864, 0xe864, PDF_CMAP_SINGLE, 9596 },
	{ 0xe865, 0xe86f, PDF_CMAP_RANGE, 20414 },
	{ 0xe870, 0xe870, PDF_CMAP_SINGLE, 9492 },
	{ 0xe871, 0xe872, PDF_CMAP_RANGE, 20425 },
	{ 0xe873, 0xe873, PDF_CMAP_SINGLE, 9597 },
	{ 0xe874, 0xe874, PDF_CMAP_SINGLE, 20427 },
	{ 0xe875, 0xe875, PDF_CMAP_SINGLE, 9586 },
	{ 0xe876, 0xe87b, PDF_CMAP_RANGE, 20428 },
	{ 0xe87c, 0xe87c, PDF_CMAP_SINGLE, 9588 },
	{ 0xe87d, 0xe87e, PDF_CMAP_RANGE, 20434 },
	{ 0xe880, 0xe880, PDF_CMAP_SINGLE, 8782 },
	{ 0xe881, 0xe881, PDF_CMAP_SINGLE, 20436 },
	{ 0xe882, 0xe882, PDF_CMAP_SINGLE, 8646 },
	{ 0xe883, 0xe886, PDF_CMAP_RANGE, 20437 },
	{ 0xe887, 0xe887, PDF_CMAP_SINGLE, 8351 },
	{ 0xe888, 0xe888, PDF_CMAP_SINGLE, 20441 },
	{ 0xe889, 0xe889, PDF_CMAP_SINGLE, 9590 },
	{ 0xe88a, 0xe88b, PDF_CMAP_RANGE, 20442 },
	{ 0xe88c, 0xe88c, PDF_CMAP_SINGLE, 8292 },
	{ 0xe88d, 0xe88d, PDF_CMAP_SINGLE, 8895 },
	{ 0xe88e, 0xe88e, PDF_CMAP_SINGLE, 9756 },
	{ 0xe88f, 0xe88f, PDF_CMAP_SINGLE, 8798 },
	{ 0xe890, 0xe8a0, PDF_CMAP_RANGE, 20444 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe940, 0xe94b, PDF_CMAP_RANGE, 20461 },
	{ 0xe94c, 0xe94c, PDF_CMAP_SINGLE, 7797 },
	{ 0xe94d, 0xe953, PDF_CMAP_RANGE, 20473 },
	{ 0xe954, 0xe954, PDF_CMAP_SINGLE, 8317 },
	{ 0xe955, 0xe955, PDF_CMAP_SINGLE, 20480 },
	{ 0xe956, 0xe956, PDF_CMAP_SINGLE, 9151 },
	{ 0xe957, 0xe957, PDF_CMAP_SINGLE, 8467 },
	{ 0xe958, 0xe959, PDF_CMAP_RANGE, 20481 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 9152 },
	{ 0xe95b, 0xe95c, PDF_CMAP_RANGE, 20483 },
	{ 0xe95d, 0xe95d, PDF_CMAP_SINGLE, 7749 },
	{ 0xe95e, 0xe95e, PDF_CMAP_SINGLE, 20485 },
	{ 0xe95f, 0xe95f, PDF_CMAP_SINGLE, 8152 },
	{ 0xe960, 0xe960, PDF_CMAP_SINGLE, 9156 },
	{ 0xe961, 0xe961, PDF_CMAP_SINGLE, 20486 },
	{ 0xe962, 0xe962, PDF_CMAP_SINGLE, 9154 },
	{ 0xe963, 0xe963, PDF_CMAP_SINGLE, 8452 },
	{ 0xe964, 0xe964, PDF_CMAP_SINGLE, 20487 },
	{ 0xe965, 0xe965, PDF_CMAP_SINGLE, 8637 },
	{ 0xe966, 0xe966, PDF_CMAP_SINGLE, 20488 },
	{ 0xe967, 0xe967, PDF_CMAP_SINGLE, 8071 },
	{ 0xe968, 0xe968, PDF_CMAP_SINGLE, 9155 },
	{ 0xe969, 0xe96b, PDF_CMAP_RANGE, 20489 },
	{ 0xe96c, 0xe96c, PDF_CMAP_SINGLE, 8809 },
	{ 0xe96d, 0xe974, PDF_CMAP_RANGE, 20492 },
	{ 0xe975, 0xe975, PDF_CMAP_SINGLE, 8003 },
	{ 0xe976, 0xe976, PDF_CMAP_SINGLE, 20500 },
	{ 0xe977, 0xe977, PDF_CMAP_SINGLE, 7966 },
	{ 0xe978, 0xe978, PDF_CMAP_SINGLE, 9849 },
	{ 0xe979, 0xe979, PDF_CMAP_SINGLE, 7915 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 20501 },
	{ 0xe97c, 0xe97c, PDF_CMAP_SINGLE, 7989 },
	{ 0xe97d, 0xe97d, PDF_CMAP_SINGLE, 8330 },
	{ 0xe97e, 0xe97e, PDF_CMAP_SINGLE, 20503 },
	{ 0xe980, 0xe980, PDF_CMAP_SINGLE, 9159 },
	{ 0xe981, 0xe981, PDF_CMAP_SINGLE, 9161 },
	{ 0xe982, 0xe982, PDF_CMAP_SINGLE, 9158 },
	{ 0xe983, 0xe986, PDF_CMAP_RANGE, 20504 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 8783 },
	{ 0xe988, 0xe98a, PDF_CMAP_RANGE, 20508 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 9163 },
	{ 0xe98c, 0xe98d, PDF_CMAP_RANGE, 20511 },
	{ 0xe98e, 0xe98e, PDF_CMAP_SINGLE, 8691 },
	{ 0xe98f, 0xe98f, PDF_CMAP_SINGLE, 20513 },
	{ 0xe990, 0xe990, PDF_CMAP_SINGLE, 8695 },
	{ 0xe991, 0xe991, PDF_CMAP_SINGLE, 9167 },
	{ 0xe992, 0xe992, PDF_CMAP_SINGLE, 9166 },
	{ 0xe993, 0xe993, PDF_CMAP_SINGLE, 9162 },
	{ 0xe994, 0xe994, PDF_CMAP_SINGLE, 9165 },
	{ 0xe995, 0xe997, PDF_CMAP_RANGE, 20514 },
	{ 0xe998, 0xe998, PDF_CMAP_SINGLE, 9168 },
	{ 0xe999, 0xe99a, PDF_CMAP_RANGE, 20517 },
	{ 0xe99b, 0xe99b, PDF_CMAP_SINGLE, 9836 },
	{ 0xe99c, 0xe99c, PDF_CMAP_SINGLE, 20519 },
	{ 0xe99d, 0xe99d, PDF_CMAP_SINGLE, 9153 },
	{ 0xe99e, 0xe99e, PDF_CMAP_SINGLE, 20520 },
	{ 0xe99f, 0xe99f, PDF_CMAP_SINGLE, 8174 },
	{ 0xe9a0, 0xe9a0, PDF_CMAP_SINGLE, 9169 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xea40, 0xea40, PDF_CMAP_SINGLE, 8184 },
	{ 0xea41, 0xea43, PDF_CMAP_RANGE, 20521 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 9171 },
	{ 0xea45, 0xea47, PDF_CMAP_RANGE, 20524 },
	{ 0xea48, 0xea48, PDF_CMAP_SINGLE, 9170 },
	{ 0xea49, 0xea49, PDF_CMAP_SINGLE, 9172 },
	{ 0xea4a, 0xea4a, PDF_CMAP_SINGLE, 7832 },
	{ 0xea4b, 0xea4f, PDF_CMAP_RANGE, 20527 },
	{ 0xea50, 0xea50, PDF_CMAP_SINGLE, 7980 },
	{ 0xea51, 0xea51, PDF_CMAP_SINGLE, 20532 },
	{ 0xea52, 0xea52, PDF_CMAP_SINGLE, 9173 },
	{ 0xea53, 0xea54, PDF_CMAP_RANGE, 20533 },
	{ 0xea55, 0xea55, PDF_CMAP_SINGLE, 7793 },
	{ 0xea56, 0xea56, PDF_CMAP_SINGLE, 9873 },
	{ 0xea57, 0xea58, PDF_CMAP_RANGE, 20535 },
	{ 0xea59, 0xea59, PDF_CMAP_SINGLE, 9157 },
	{ 0xea5a, 0xea7e, PDF_CMAP_RANGE, 20537 },
	{ 0xea80, 0xea80, PDF_CMAP_SINGLE, 8986 },
	{ 0xea81, 0xea83, PDF_CMAP_RANGE, 20574 },
	{ 0xea84, 0xea84, PDF_CMAP_SINGLE, 8468 },
	{ 0xea85, 0xea86, PDF_CMAP_RANGE, 20577 },
	{ 0xea87, 0xea87, PDF_CMAP_SINGLE, 8836 },
	{ 0xea88, 0xea8d, PDF_CMAP_RANGE, 20579 },
	{ 0xea8e, 0xea8e, PDF_CMAP_SINGLE, 8732 },
	{ 0xea8f, 0xea8f, PDF_CMAP_SINGLE, 20585 },
	{ 0xea90, 0xea90, PDF_CMAP_SINGLE, 7806 },
	{ 0xea91, 0xea91, PDF_CMAP_SINGLE, 8269 },
	{ 0xea92, 0xea95, PDF_CMAP_RANGE, 20586 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 8705 },
	{ 0xea97, 0xea9f, PDF_CMAP_RANGE, 20590 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 7897 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeb40, 0xeb40, PDF_CMAP_SINGLE, 20599 },
	{ 0xeb41, 0xeb41, PDF_CMAP_SINGLE, 8114 },
	{ 0xeb42, 0xeb44, PDF_CMAP_RANGE, 20600 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 8786 },
	{ 0xeb46, 0xeb47, PDF_CMAP_RANGE, 20603 },
	{ 0xeb48, 0xeb48, PDF_CMAP_SINGLE, 8057 },
	{ 0xeb49, 0xeb52, PDF_CMAP_RANGE, 20605 },
	{ 0xeb53, 0xeb53, PDF_CMAP_SINGLE, 8535 },
	{ 0xeb54, 0xeb54, PDF_CMAP_SINGLE, 20615 },
	{ 0xeb55, 0xeb55, PDF_CMAP_SINGLE, 8639 },
	{ 0xeb56, 0xeb5a, PDF_CMAP_RANGE, 20616 },
	{ 0xeb5b, 0xeb5b, PDF_CMAP_SINGLE, 8735 },
	{ 0xeb5c, 0xeb5c, PDF_CMAP_SINGLE, 20621 },
	{ 0xeb5d, 0xeb5d, PDF_CMAP_SINGLE, 8253 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 20622 },
	{ 0xeb60, 0xeb60, PDF_CMAP_SINGLE, 8213 },
	{ 0xeb61, 0xeb61, PDF_CMAP_SINGLE, 20624 },
	{ 0xeb62, 0xeb62, PDF_CMAP_SINGLE, 9893 },
	{ 0xeb63, 0xeb6c, PDF_CMAP_RANGE, 20625 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 8534 },
	{ 0xeb6e, 0xeb6f, PDF_CMAP_RANGE, 20635 },
	{ 0xeb70, 0xeb70, PDF_CMAP_SINGLE, 8516 },
	{ 0xeb71, 0xeb71, PDF_CMAP_SINGLE, 20637 },
	{ 0xeb72, 0xeb72, PDF_CMAP_SINGLE, 7825 },
	{ 0xeb73, 0xeb73, PDF_CMAP_SINGLE, 8791 },
	{ 0xeb74, 0xeb77, PDF_CMAP_RANGE, 20638 },
	{ 0xeb78, 0xeb78, PDF_CMAP_SINGLE, 8202 },
	{ 0xeb79, 0xeb79, PDF_CMAP_SINGLE, 8338 },
	{ 0xeb7a, 0xeb7e, PDF_CMAP_RANGE, 20642 },
	{ 0xeb80, 0xeb84, PDF_CMAP_RANGE, 20647 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 8784 },
	{ 0xeb86, 0xeb89, PDF_CMAP_RANGE, 20652 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 7875 },
	{ 0xeb8b, 0xeba0, PDF_CMAP_RANGE, 20656 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xec40, 0xec45, PDF_CMAP_RANGE, 20678 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8616 },
	{ 0xec47, 0xec55, PDF_CMAP_RANGE, 20684 },
	{ 0xec56, 0xec56, PDF_CMAP_SINGLE, 9741 },
	{ 0xec57, 0xec59, PDF_CMAP_RANGE, 20699 },
	{ 0xec5a, 0xec5a, PDF_CMAP_SINGLE, 9740 },
	{ 0xec5b, 0xec5b, PDF_CMAP_SINGLE, 20702 },
	{ 0xec5c, 0xec5c, PDF_CMAP_SINGLE, 9742 },
	{ 0xec5d, 0xec5f, PDF_CMAP_RANGE, 20703 },
	{ 0xec60, 0xec60, PDF_CMAP_SINGLE, 8242 },
	{ 0xec61, 0xec6d, PDF_CMAP_RANGE, 20706 },
	{ 0xec6e, 0xec6e, PDF_CMAP_SINGLE, 9739 },
	{ 0xec6f, 0xec75, PDF_CMAP_RANGE, 20719 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8899 },
	{ 0xec77, 0xec7e, PDF_CMAP_RANGE, 20726 },
	{ 0xec80, 0xec95, PDF_CMAP_RANGE, 20734 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 7971 },
	{ 0xec97, 0xeca0, PDF_CMAP_RANGE, 20756 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xed40, 0xed45, PDF_CMAP_RANGE, 20766 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 9877 },
	{ 0xed47, 0xed57, PDF_CMAP_RANGE, 20772 },
	{ 0xed58, 0xed58, PDF_CMAP_SINGLE, 9822 },
	{ 0xed59, 0xed5d, PDF_CMAP_RANGE, 20789 },
	{ 0xed5e, 0xed5e, PDF_CMAP_SINGLE, 9821 },
	{ 0xed5f, 0xed60, PDF_CMAP_RANGE, 20794 },
	{ 0xed61, 0xed61, PDF_CMAP_SINGLE, 9874 },
	{ 0xed62, 0xed63, PDF_CMAP_RANGE, 20796 },
	{ 0xed64, 0xed64, PDF_CMAP_SINGLE, 9823 },
	{ 0xed65, 0xed65, PDF_CMAP_SINGLE, 20798 },
	{ 0xed66, 0xed66, PDF_CMAP_SINGLE, 8589 },
	{ 0xed67, 0xed67, PDF_CMAP_SINGLE, 8445 },
	{ 0xed68, 0xed6d, PDF_CMAP_RANGE, 20799 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 8000 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 20805 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 9317 },
	{ 0xed75, 0xed76, PDF_CMAP_RANGE, 20810 },
	{ 0xed77, 0xed77, PDF_CMAP_SINGLE, 9319 },
	{ 0xed78, 0xed78, PDF_CMAP_SINGLE, 20812 },
	{ 0xed79, 0xed79, PDF_CMAP_SINGLE, 9318 },
	{ 0xed7a, 0xed7e, PDF_CMAP_RANGE, 20813 },
	{ 0xed80, 0xed90, PDF_CMAP_RANGE, 20818 },
	{ 0xed91, 0xed91, PDF_CMAP_SINGLE, 8649 },
	{ 0xed92, 0xed92, PDF_CMAP_SINGLE, 20835 },
	{ 0xed93, 0xed93, PDF_CMAP_SINGLE, 8713 },
	{ 0xed94, 0xed94, PDF_CMAP_SINGLE, 7881 },
	{ 0xed95, 0xed95, PDF_CMAP_SINGLE, 8425 },
	{ 0xed96, 0xed96, PDF_CMAP_SINGLE, 20836 },
	{ 0xed97, 0xed97, PDF_CMAP_SINGLE, 8650 },
	{ 0xed98, 0xed98, PDF_CMAP_SINGLE, 8518 },
	{ 0xed99, 0xed99, PDF_CMAP_SINGLE, 9669 },
	{ 0xed9a, 0xed9a, PDF_CMAP_SINGLE, 8668 },
	{ 0xed9b, 0xed9b, PDF_CMAP_SINGLE, 20837 },
	{ 0xed9c, 0xed9c, PDF_CMAP_SINGLE, 9310 },
	{ 0xed9d, 0xed9d, PDF_CMAP_SINGLE, 20838 },
	{ 0xed9e, 0xed9e, PDF_CMAP_SINGLE, 8527 },
	{ 0xed9f, 0xed9f, PDF_CMAP_SINGLE, 20839 },
	{ 0xeda0, 0xeda0, PDF_CMAP_SINGLE, 9670 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xee40, 0xee40, PDF_CMAP_SINGLE, 9671 },
	{ 0xee41, 0xee41, PDF_CMAP_SINGLE, 8769 },
	{ 0xee42, 0xee42, PDF_CMAP_SINGLE, 8586 },
	{ 0xee43, 0xee43, PDF_CMAP_SINGLE, 7727 },
	{ 0xee44, 0xee44, PDF_CMAP_SINGLE, 7900 },
	{ 0xee45, 0xee47, PDF_CMAP_RANGE, 20840 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 8383 },
	{ 0xee49, 0xee49, PDF_CMAP_SINGLE, 8244 },
	{ 0xee4a, 0xee4c, PDF_CMAP_RANGE, 20843 },
	{ 0xee4d, 0xee4d, PDF_CMAP_SINGLE, 9673 },
	{ 0xee4e, 0xee51, PDF_CMAP_RANGE, 20846 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 9672 },
	{ 0xee53, 0xee54, PDF_CMAP_RANGE, 20850 },
	{ 0xee55, 0xee55, PDF_CMAP_SINGLE, 8718 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 20852 },
	{ 0xee57, 0xee57, PDF_CMAP_SINGLE, 9675 },
	{ 0xee58, 0xee5d, PDF_CMAP_RANGE, 20853 },
	{ 0xee5e, 0xee5e, PDF_CMAP_SINGLE, 8573 },
	{ 0xee5f, 0xee60, PDF_CMAP_RANGE, 20859 },
	{ 0xee61, 0xee61, PDF_CMAP_SINGLE, 8062 },
	{ 0xee62, 0xee67, PDF_CMAP_RANGE, 20861 },
	{ 0xee68, 0xee68, PDF_CMAP_SINGLE, 9676 },
	{ 0xee69, 0xee69, PDF_CMAP_SINGLE, 8131 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 20867 },
	{ 0xee6c, 0xee6c, PDF_CMAP_SINGLE, 8377 },
	{ 0xee6d, 0xee6d, PDF_CMAP_SINGLE, 20869 },
	{ 0xee6e, 0xee6e, PDF_CMAP_SINGLE, 8577 },
	{ 0xee6f, 0xee76, PDF_CMAP_RANGE, 20870 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 8154 },
	{ 0xee78, 0xee7c, PDF_CMAP_RANGE, 20878 },
	{ 0xee7d, 0xee7d, PDF_CMAP_SINGLE, 8563 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 7905 },
	{ 0xee80, 0xee80, PDF_CMAP_SINGLE, 9677 },
	{ 0xee81, 0xee84, PDF_CMAP_RANGE, 20883 },
	{ 0xee85, 0xee85, PDF_CMAP_SINGLE, 9678 },
	{ 0xee86, 0xee86, PDF_CMAP_SINGLE, 8694 },
	{ 0xee87, 0xee89, PDF_CMAP_RANGE, 20887 },
	{ 0xee8a, 0xee8a, PDF_CMAP_SINGLE, 8779 },
	{ 0xee8b, 0xee8b, PDF_CMAP_SINGLE, 9681 },
	{ 0xee8c, 0xee8c, PDF_CMAP_SINGLE, 20890 },
	{ 0xee8d, 0xee8d, PDF_CMAP_SINGLE, 7872 },
	{ 0xee8e, 0xee8f, PDF_CMAP_RANGE, 20891 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 8200 },
	{ 0xee91, 0xee93, PDF_CMAP_RANGE, 20893 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 9680 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 20896 },
	{ 0xee97, 0xee97, PDF_CMAP_SINGLE, 9682 },
	{ 0xee98, 0xee98, PDF_CMAP_SINGLE, 20898 },
	{ 0xee99, 0xee99, PDF_CMAP_SINGLE, 7978 },
	{ 0xee9a, 0xee9c, PDF_CMAP_RANGE, 20899 },
	{ 0xee9d, 0xee9d, PDF_CMAP_SINGLE, 7794 },
	{ 0xee9e, 0xee9e, PDF_CMAP_SINGLE, 9683 },
	{ 0xee9f, 0xeea0, PDF_CMAP_RANGE, 20902 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xef40, 0xef40, PDF_CMAP_SINGLE, 8638 },
	{ 0xef41, 0xef41, PDF_CMAP_SINGLE, 9684 },
	{ 0xef42, 0xef42, PDF_CMAP_SINGLE, 8260 },
	{ 0xef43, 0xef43, PDF_CMAP_SINGLE, 20904 },
	{ 0xef44, 0xef44, PDF_CMAP_SINGLE, 9679 },
	{ 0xef45, 0xef45, PDF_CMAP_SINGLE, 8435 },
	{ 0xef46, 0xef4b, PDF_CMAP_RANGE, 20905 },
	{ 0xef4c, 0xef4c, PDF_CMAP_SINGLE, 7936 },
	{ 0xef4d, 0xef51, PDF_CMAP_RANGE, 20911 },
	{ 0xef52, 0xef53, PDF_CMAP_RANGE, 9420 },
	{ 0xef54, 0xef54, PDF_CMAP_SINGLE, 20916 },
	{ 0xef55, 0xef55, PDF_CMAP_SINGLE, 9880 },
	{ 0xef56, 0xef56, PDF_CMAP_SINGLE, 20917 },
	{ 0xef57, 0xef57, PDF_CMAP_SINGLE, 9848 },
	{ 0xef58, 0xef59, PDF_CMAP_RANGE, 20918 },
	{ 0xef5a, 0xef5a, PDF_CMAP_SINGLE, 9422 },
	{ 0xef5b, 0xef5f, PDF_CMAP_RANGE, 20920 },
	{ 0xef60, 0xef60, PDF_CMAP_SINGLE, 9423 },
	{ 0xef61, 0xef67, PDF_CMAP_RANGE, 20925 },
	{ 0xef68, 0xef68, PDF_CMAP_SINGLE, 8376 },
	{ 0xef69, 0xef69, PDF_CMAP_SINGLE, 20932 },
	{ 0xef6a, 0xef6a, PDF_CMAP_SINGLE, 9424 },
	{ 0xef6b, 0xef6b, PDF_CMAP_SINGLE, 20933 },
	{ 0xef6c, 0xef6c, PDF_CMAP_SINGLE, 9425 },
	{ 0xef6d, 0xef76, PDF_CMAP_RANGE, 20934 },
	{ 0xef77, 0xef77, PDF_CMAP_SINGLE, 7924 },
	{ 0xef78, 0xef79, PDF_CMAP_RANGE, 20944 },
	{ 0xef7a, 0xef7a, PDF_CMAP_SINGLE, 9115 },
	{ 0xef7b, 0xef7b, PDF_CMAP_SINGLE, 20946 },
	{ 0xef7c, 0xef7c, PDF_CMAP_SINGLE, 9854 },
	{ 0xef7d, 0xef7e, PDF_CMAP_RANGE, 20947 },
	{ 0xef80, 0xef81, PDF_CMAP_RANGE, 20949 },
	{ 0xef82, 0xef82, PDF_CMAP_SINGLE, 9117 },
	{ 0xef83, 0xef84, PDF_CMAP_RANGE, 9119 },
	{ 0xef85, 0xef85, PDF_CMAP_SINGLE, 20951 },
	{ 0xef86, 0xef86, PDF_CMAP_SINGLE, 9121 },
	{ 0xef87, 0xef87, PDF_CMAP_SINGLE, 20952 },
	{ 0xef88, 0xef88, PDF_CMAP_SINGLE, 7921 },
	{ 0xef89, 0xef8a, PDF_CMAP_RANGE, 20953 },
	{ 0xef8b, 0xef8b, PDF_CMAP_SINGLE, 8734 },
	{ 0xef8c, 0xef8c, PDF_CMAP_SINGLE, 20955 },
	{ 0xef8d, 0xef8d, PDF_CMAP_SINGLE, 9122 },
	{ 0xef8e, 0xef94, PDF_CMAP_RANGE, 20956 },
	{ 0xef95, 0xef95, PDF_CMAP_SINGLE, 8523 },
	{ 0xef96, 0xef96, PDF_CMAP_SINGLE, 7734 },
	{ 0xef97, 0xef97, PDF_CMAP_SINGLE, 8501 },
	{ 0xef98, 0xef9b, PDF_CMAP_RANGE, 20963 },
	{ 0xef9c, 0xef9c, PDF_CMAP_SINGLE, 8109 },
	{ 0xef9d, 0xef9d, PDF_CMAP_SINGLE, 20967 },
	{ 0xef9e, 0xef9e, PDF_CMAP_SINGLE, 7763 },
	{ 0xef9f, 0xefa0, PDF_CMAP_RANGE, 20968 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf040, 0xf040, PDF_CMAP_SINGLE, 20970 },
	{ 0xf041, 0xf041, PDF_CMAP_SINGLE, 9123 },
	{ 0xf042, 0xf042, PDF_CMAP_SINGLE, 8707 },
	{ 0xf043, 0xf043, PDF_CMAP_SINGLE, 20971 },
	{ 0xf044, 0xf044, PDF_CMAP_SINGLE, 7911 },
	{ 0xf045, 0xf046, PDF_CMAP_RANGE, 20972 },
	{ 0xf047, 0xf047, PDF_CMAP_SINGLE, 9124 },
	{ 0xf048, 0xf048, PDF_CMAP_SINGLE, 8343 },
	{ 0xf049, 0xf049, PDF_CMAP_SINGLE, 7908 },
	{ 0xf04a, 0xf04d, PDF_CMAP_RANGE, 20974 },
	{ 0xf04e, 0xf04e, PDF_CMAP_SINGLE, 8760 },
	{ 0xf04f, 0xf050, PDF_CMAP_RANGE, 20978 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 9125 },
	{ 0xf052, 0xf053, PDF_CMAP_RANGE, 20980 },
	{ 0xf054, 0xf054, PDF_CMAP_SINGLE, 8090 },
	{ 0xf055, 0xf056, PDF_CMAP_RANGE, 20982 },
	{ 0xf057, 0xf057, PDF_CMAP_SINGLE, 8643 },
	{ 0xf058, 0xf05d, PDF_CMAP_RANGE, 20984 },
	{ 0xf05e, 0xf05e, PDF_CMAP_SINGLE, 7982 },
	{ 0xf05f, 0xf067, PDF_CMAP_RANGE, 20990 },
	{ 0xf068, 0xf068, PDF_CMAP_SINGLE, 9116 },
	{ 0xf069, 0xf06b, PDF_CMAP_RANGE, 20999 },
	{ 0xf06c, 0xf06c, PDF_CMAP_SINGLE, 9126 },
	{ 0xf06d, 0xf070, PDF_CMAP_RANGE, 21002 },
	{ 0xf071, 0xf071, PDF_CMAP_SINGLE, 9118 },
	{ 0xf072, 0xf072, PDF_CMAP_SINGLE, 21006 },
	{ 0xf073, 0xf073, PDF_CMAP_SINGLE, 8245 },
	{ 0xf074, 0xf074, PDF_CMAP_SINGLE, 9127 },
	{ 0xf075, 0xf077, PDF_CMAP_RANGE, 21007 },
	{ 0xf078, 0xf078, PDF_CMAP_SINGLE, 9128 },
	{ 0xf079, 0xf079, PDF_CMAP_SINGLE, 21010 },
	{ 0xf07a, 0xf07a, PDF_CMAP_SINGLE, 8309 },
	{ 0xf07b, 0xf07c, PDF_CMAP_RANGE, 21011 },
	{ 0xf07d, 0xf07e, PDF_CMAP_RANGE, 9129 },
	{ 0xf080, 0xf080, PDF_CMAP_SINGLE, 9131 },
	{ 0xf081, 0xf081, PDF_CMAP_SINGLE, 8171 },
	{ 0xf082, 0xf082, PDF_CMAP_SINGLE, 9132 },
	{ 0xf083, 0xf086, PDF_CMAP_RANGE, 21013 },
	{ 0xf087, 0xf087, PDF_CMAP_SINGLE, 8042 },
	{ 0xf088, 0xf088, PDF_CMAP_SINGLE, 8441 },
	{ 0xf089, 0xf08a, PDF_CMAP_RANGE, 21017 },
	{ 0xf08b, 0xf08b, PDF_CMAP_SINGLE, 9830 },
	{ 0xf08c, 0xf08f, PDF_CMAP_RANGE, 21019 },
	{ 0xf090, 0xf090, PDF_CMAP_SINGLE, 9831 },
	{ 0xf091, 0xf091, PDF_CMAP_SINGLE, 21023 },
	{ 0xf092, 0xf092, PDF_CMAP_SINGLE, 7788 },
	{ 0xf093, 0xf095, PDF_CMAP_RANGE, 21024 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 9133 },
	{ 0xf097, 0xf0a0, PDF_CMAP_RANGE, 21027 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf140, 0xf151, PDF_CMAP_RANGE, 21037 },
	{ 0xf152, 0xf152, PDF_CMAP_SINGLE, 8301 },
	{ 0xf153, 0xf153, PDF_CMAP_SINGLE, 8770 },
	{ 0xf154, 0xf154, PDF_CMAP_SINGLE, 7938 },
	{ 0xf155, 0xf156, PDF_CMAP_RANGE, 21055 },
	{ 0xf157, 0xf157, PDF_CMAP_SINGLE, 8579 },
	{ 0xf158, 0xf158, PDF_CMAP_SINGLE, 21057 },
	{ 0xf159, 0xf159, PDF_CMAP_SINGLE, 7813 },
	{ 0xf15a, 0xf15a, PDF_CMAP_SINGLE, 8681 },
	{ 0xf15b, 0xf166, PDF_CMAP_RANGE, 21058 },
	{ 0xf167, 0xf167, PDF_CMAP_SINGLE, 7767 },
	{ 0xf168, 0xf175, PDF_CMAP_RANGE, 21070 },
	{ 0xf176, 0xf176, PDF_CMAP_SINGLE, 8869 },
	{ 0xf177, 0xf177, PDF_CMAP_SINGLE, 9223 },
	{ 0xf178, 0xf178, PDF_CMAP_SINGLE, 8138 },
	{ 0xf179, 0xf179, PDF_CMAP_SINGLE, 21084 },
	{ 0xf17a, 0xf17a, PDF_CMAP_SINGLE, 9218 },
	{ 0xf17b, 0xf17b, PDF_CMAP_SINGLE, 8066 },
	{ 0xf17c, 0xf17d, PDF_CMAP_RANGE, 21085 },
	{ 0xf17e, 0xf17e, PDF_CMAP_SINGLE, 9224 },
	{ 0xf180, 0xf180, PDF_CMAP_SINGLE, 9220 },
	{ 0xf181, 0xf181, PDF_CMAP_SINGLE, 21087 },
	{ 0xf182, 0xf182, PDF_CMAP_SINGLE, 8497 },
	{ 0xf183, 0xf183, PDF_CMAP_SINGLE, 21088 },
	{ 0xf184, 0xf184, PDF_CMAP_SINGLE, 8580 },
	{ 0xf185, 0xf185, PDF_CMAP_SINGLE, 21089 },
	{ 0xf186, 0xf186, PDF_CMAP_SINGLE, 9219 },
	{ 0xf187, 0xf187, PDF_CMAP_SINGLE, 21090 },
	{ 0xf188, 0xf188, PDF_CMAP_SINGLE, 8302 },
	{ 0xf189, 0xf189, PDF_CMAP_SINGLE, 9227 },
	{ 0xf18a, 0xf193, PDF_CMAP_RANGE, 21091 },
	{ 0xf194, 0xf194, PDF_CMAP_SINGLE, 7999 },
	{ 0xf195, 0xf197, PDF_CMAP_RANGE, 21101 },
	{ 0xf198, 0xf198, PDF_CMAP_SINGLE, 8295 },
	{ 0xf199, 0xf1a0, PDF_CMAP_RANGE, 21104 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf240, 0xf244, PDF_CMAP_RANGE, 21112 },
	{ 0xf245, 0xf245, PDF_CMAP_SINGLE, 8151 },
	{ 0xf246, 0xf246, PDF_CMAP_SINGLE, 21117 },
	{ 0xf247, 0xf247, PDF_CMAP_SINGLE, 7811 },
	{ 0xf248, 0xf24a, PDF_CMAP_RANGE, 21118 },
	{ 0xf24b, 0xf24b, PDF_CMAP_SINGLE, 9231 },
	{ 0xf24c, 0xf252, PDF_CMAP_RANGE, 21121 },
	{ 0xf253, 0xf253, PDF_CMAP_SINGLE, 9230 },
	{ 0xf254, 0xf254, PDF_CMAP_SINGLE, 8391 },
	{ 0xf255, 0xf255, PDF_CMAP_SINGLE, 9229 },
	{ 0xf256, 0xf25b, PDF_CMAP_RANGE, 21128 },
	{ 0xf25c, 0xf25c, PDF_CMAP_SINGLE, 9234 },
	{ 0xf25d, 0xf25e, PDF_CMAP_RANGE, 21134 },
	{ 0xf25f, 0xf25f, PDF_CMAP_SINGLE, 8375 },
	{ 0xf260, 0xf270, PDF_CMAP_RANGE, 21136 },
	{ 0xf271, 0xf271, PDF_CMAP_SINGLE, 9200 },
	{ 0xf272, 0xf272, PDF_CMAP_SINGLE, 21153 },
	{ 0xf273, 0xf273, PDF_CMAP_SINGLE, 9233 },
	{ 0xf274, 0xf274, PDF_CMAP_SINGLE, 9236 },
	{ 0xf275, 0xf275, PDF_CMAP_SINGLE, 21154 },
	{ 0xf276, 0xf276, PDF_CMAP_SINGLE, 8560 },
	{ 0xf277, 0xf27b, PDF_CMAP_RANGE, 21155 },
	{ 0xf27c, 0xf27c, PDF_CMAP_SINGLE, 9221 },
	{ 0xf27d, 0xf27d, PDF_CMAP_SINGLE, 8460 },
	{ 0xf27e, 0xf27e, PDF_CMAP_SINGLE, 9237 },
	{ 0xf280, 0xf284, PDF_CMAP_RANGE, 21160 },
	{ 0xf285, 0xf285, PDF_CMAP_SINGLE, 8294 },
	{ 0xf286, 0xf286, PDF_CMAP_SINGLE, 21165 },
	{ 0xf287, 0xf287, PDF_CMAP_SINGLE, 9042 },
	{ 0xf288, 0xf288, PDF_CMAP_SINGLE, 9235 },
	{ 0xf289, 0xf289, PDF_CMAP_SINGLE, 9232 },
	{ 0xf28a, 0xf28b, PDF_CMAP_RANGE, 9238 },
	{ 0xf28c, 0xf28c, PDF_CMAP_SINGLE, 8433 },
	{ 0xf28d, 0xf290, PDF_CMAP_RANGE, 21166 },
	{ 0xf291, 0xf291, PDF_CMAP_SINGLE, 9226 },
	{ 0xf292, 0xf293, PDF_CMAP_RANGE, 21170 },
	{ 0xf294, 0xf294, PDF_CMAP_SINGLE, 9225 },
	{ 0xf295, 0xf295, PDF_CMAP_SINGLE, 21172 },
	{ 0xf296, 0xf296, PDF_CMAP_SINGLE, 9240 },
	{ 0xf297, 0xf29b, PDF_CMAP_RANGE, 21173 },
	{ 0xf29c, 0xf29c, PDF_CMAP_SINGLE, 8103 },
	{ 0xf29d, 0xf29d, PDF_CMAP_SINGLE, 21178 },
	{ 0xf29e, 0xf29e, PDF_CMAP_SINGLE, 8700 },
	{ 0xf29f, 0xf2a0, PDF_CMAP_RANGE, 21179 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf340, 0xf340, PDF_CMAP_SINGLE, 8129 },
	{ 0xf341, 0xf341, PDF_CMAP_SINGLE, 9222 },
	{ 0xf342, 0xf344, PDF_CMAP_RANGE, 21181 },
	{ 0xf345, 0xf345, PDF_CMAP_SINGLE, 8860 },
	{ 0xf346, 0xf347, PDF_CMAP_RANGE, 21184 },
	{ 0xf348, 0xf348, PDF_CMAP_SINGLE, 8270 },
	{ 0xf349, 0xf349, PDF_CMAP_SINGLE, 21186 },
	{ 0xf34a, 0xf34a, PDF_CMAP_SINGLE, 9242 },
	{ 0xf34b, 0xf34b, PDF_CMAP_SINGLE, 9241 },
	{ 0xf34c, 0xf34f, PDF_CMAP_RANGE, 21187 },
	{ 0xf350, 0xf350, PDF_CMAP_SINGLE, 9228 },
	{ 0xf351, 0xf360, PDF_CMAP_RANGE, 21191 },
	{ 0xf361, 0xf361, PDF_CMAP_SINGLE, 7721 },
	{ 0xf362, 0xf373, PDF_CMAP_RANGE, 21207 },
	{ 0xf374, 0xf374, PDF_CMAP_SINGLE, 9825 },
	{ 0xf375, 0xf375, PDF_CMAP_SINGLE, 21225 },
	{ 0xf376, 0xf376, PDF_CMAP_SINGLE, 9892 },
	{ 0xf377, 0xf377, PDF_CMAP_SINGLE, 8564 },
	{ 0xf378, 0xf378, PDF_CMAP_SINGLE, 9827 },
	{ 0xf379, 0xf379, PDF_CMAP_SINGLE, 9826 },
	{ 0xf37a, 0xf37e, PDF_CMAP_RANGE, 21226 },
	{ 0xf380, 0xf38b, PDF_CMAP_RANGE, 21231 },
	{ 0xf38c, 0xf38c, PDF_CMAP_SINGLE, 9845 },
	{ 0xf38d, 0xf39f, PDF_CMAP_RANGE, 21243 },
	{ 0xf3a0, 0xf3a0, PDF_CMAP_SINGLE, 8524 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf440, 0xf444, PDF_CMAP_RANGE, 21262 },
	{ 0xf445, 0xf445, PDF_CMAP_SINGLE, 9850 },
	{ 0xf446, 0xf44f, PDF_CMAP_RANGE, 21267 },
	{ 0xf450, 0xf450, PDF_CMAP_SINGLE, 9888 },
	{ 0xf451, 0xf456, PDF_CMAP_RANGE, 21277 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 9832 },
	{ 0xf458, 0xf458, PDF_CMAP_SINGLE, 21283 },
	{ 0xf459, 0xf459, PDF_CMAP_SINGLE, 7888 },
	{ 0xf45a, 0xf45a, PDF_CMAP_SINGLE, 21284 },
	{ 0xf45b, 0xf45b, PDF_CMAP_SINGLE, 8342 },
	{ 0xf45c, 0xf45c, PDF_CMAP_SINGLE, 21285 },
	{ 0xf45d, 0xf45d, PDF_CMAP_SINGLE, 9164 },
	{ 0xf45e, 0xf461, PDF_CMAP_RANGE, 21286 },
	{ 0xf462, 0xf462, PDF_CMAP_SINGLE, 9160 },
	{ 0xf463, 0xf463, PDF_CMAP_SINGLE, 21290 },
	{ 0xf464, 0xf464, PDF_CMAP_SINGLE, 8766 },
	{ 0xf465, 0xf474, PDF_CMAP_RANGE, 21291 },
	{ 0xf475, 0xf475, PDF_CMAP_SINGLE, 9829 },
	{ 0xf476, 0xf47b, PDF_CMAP_RANGE, 21307 },
	{ 0xf47c, 0xf47c, PDF_CMAP_SINGLE, 9828 },
	{ 0xf47d, 0xf47d, PDF_CMAP_SINGLE, 21313 },
	{ 0xf47e, 0xf47e, PDF_CMAP_SINGLE, 8761 },
	{ 0xf480, 0xf493, PDF_CMAP_RANGE, 21314 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 8266 },
	{ 0xf495, 0xf498, PDF_CMAP_RANGE, 21334 },
	{ 0xf499, 0xf499, PDF_CMAP_SINGLE, 9759 },
	{ 0xf49a, 0xf49b, PDF_CMAP_RANGE, 21338 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 9758 },
	{ 0xf49d, 0xf4a0, PDF_CMAP_RANGE, 21340 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf540, 0xf544, PDF_CMAP_RANGE, 21344 },
	{ 0xf545, 0xf545, PDF_CMAP_SINGLE, 9760 },
	{ 0xf546, 0xf546, PDF_CMAP_SINGLE, 21349 },
	{ 0xf547, 0xf547, PDF_CMAP_SINGLE, 9761 },
	{ 0xf548, 0xf551, PDF_CMAP_RANGE, 21350 },
	{ 0xf552, 0xf552, PDF_CMAP_SINGLE, 9762 },
	{ 0xf553, 0xf553, PDF_CMAP_SINGLE, 21360 },
	{ 0xf554, 0xf554, PDF_CMAP_SINGLE, 9767 },
	{ 0xf555, 0xf555, PDF_CMAP_SINGLE, 7737 },
	{ 0xf556, 0xf556, PDF_CMAP_SINGLE, 9765 },
	{ 0xf557, 0xf55d, PDF_CMAP_RANGE, 21361 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 9769 },
	{ 0xf55f, 0xf560, PDF_CMAP_RANGE, 21368 },
	{ 0xf561, 0xf561, PDF_CMAP_SINGLE, 9774 },
	{ 0xf562, 0xf562, PDF_CMAP_SINGLE, 9771 },
	{ 0xf563, 0xf56d, PDF_CMAP_RANGE, 21370 },
	{ 0xf56e, 0xf56e, PDF_CMAP_SINGLE, 9770 },
	{ 0xf56f, 0xf56f, PDF_CMAP_SINGLE, 9773 },
	{ 0xf570, 0xf570, PDF_CMAP_SINGLE, 21381 },
	{ 0xf571, 0xf571, PDF_CMAP_SINGLE, 9768 },
	{ 0xf572, 0xf572, PDF_CMAP_SINGLE, 8633 },
	{ 0xf573, 0xf57e, PDF_CMAP_RANGE, 21382 },
	{ 0xf580, 0xf584, PDF_CMAP_RANGE, 21394 },
	{ 0xf585, 0xf585, PDF_CMAP_SINGLE, 9782 },
	{ 0xf586, 0xf586, PDF_CMAP_SINGLE, 9776 },
	{ 0xf587, 0xf58b, PDF_CMAP_RANGE, 21399 },
	{ 0xf58c, 0xf58c, PDF_CMAP_SINGLE, 9784 },
	{ 0xf58d, 0xf58d, PDF_CMAP_SINGLE, 21404 },
	{ 0xf58e, 0xf58e, PDF_CMAP_SINGLE, 8205 },
	{ 0xf58f, 0xf58f, PDF_CMAP_SINGLE, 9783 },
	{ 0xf590, 0xf598, PDF_CMAP_RANGE, 21405 },
	{ 0xf599, 0xf599, PDF_CMAP_SINGLE, 9797 },
	{ 0xf59a, 0xf59a, PDF_CMAP_SINGLE, 21414 },
	{ 0xf59b, 0xf59b, PDF_CMAP_SINGLE, 9786 },
	{ 0xf59c, 0xf59f, PDF_CMAP_RANGE, 21415 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 9795 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf640, 0xf640, PDF_CMAP_SINGLE, 21419 },
	{ 0xf641, 0xf641, PDF_CMAP_SINGLE, 9792 },
	{ 0xf642, 0xf644, PDF_CMAP_RANGE, 21420 },
	{ 0xf645, 0xf645, PDF_CMAP_SINGLE, 9789 },
	{ 0xf646, 0xf646, PDF_CMAP_SINGLE, 9793 },
	{ 0xf647, 0xf647, PDF_CMAP_SINGLE, 21423 },
	{ 0xf648, 0xf648, PDF_CMAP_SINGLE, 9790 },
	{ 0xf649, 0xf64a, PDF_CMAP_RANGE, 21424 },
	{ 0xf64b, 0xf64b, PDF_CMAP_SINGLE, 9791 },
	{ 0xf64c, 0xf64c, PDF_CMAP_SINGLE, 8128 },
	{ 0xf64d, 0xf64d, PDF_CMAP_SINGLE, 21426 },
	{ 0xf64e, 0xf64f, PDF_CMAP_RANGE, 9787 },
	{ 0xf650, 0xf653, PDF_CMAP_RANGE, 21427 },
	{ 0xf654, 0xf654, PDF_CMAP_SINGLE, 9794 },
	{ 0xf655, 0xf657, PDF_CMAP_RANGE, 21431 },
	{ 0xf658, 0xf658, PDF_CMAP_SINGLE, 9796 },
	{ 0xf659, 0xf660, PDF_CMAP_RANGE, 21434 },
	{ 0xf661, 0xf661, PDF_CMAP_SINGLE, 9785 },
	{ 0xf662, 0xf662, PDF_CMAP_SINGLE, 21442 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 9804 },
	{ 0xf664, 0xf66b, PDF_CMAP_RANGE, 21443 },
	{ 0xf66c, 0xf66c, PDF_CMAP_SINGLE, 9799 },
	{ 0xf66d, 0xf66d, PDF_CMAP_SINGLE, 9803 },
	{ 0xf66e, 0xf670, PDF_CMAP_RANGE, 21451 },
	{ 0xf671, 0xf671, PDF_CMAP_SINGLE, 9801 },
	{ 0xf672, 0xf673, PDF_CMAP_RANGE, 21454 },
	{ 0xf674, 0xf674, PDF_CMAP_SINGLE, 9800 },
	{ 0xf675, 0xf675, PDF_CMAP_SINGLE, 21456 },
	{ 0xf676, 0xf676, PDF_CMAP_SINGLE, 9802 },
	{ 0xf677, 0xf677, PDF_CMAP_SINGLE, 8456 },
	{ 0xf678, 0xf67e, PDF_CMAP_RANGE, 21457 },
	{ 0xf680, 0xf684, PDF_CMAP_RANGE, 21464 },
	{ 0xf685, 0xf685, PDF_CMAP_SINGLE, 9805 },
	{ 0xf686, 0xf687, PDF_CMAP_RANGE, 21469 },
	{ 0xf688, 0xf688, PDF_CMAP_SINGLE, 9780 },
	{ 0xf689, 0xf689, PDF_CMAP_SINGLE, 21471 },
	{ 0xf68a, 0xf68a, PDF_CMAP_SINGLE, 9809 },
	{ 0xf68b, 0xf68c, PDF_CMAP_RANGE, 21472 },
	{ 0xf68d, 0xf68d, PDF_CMAP_SINGLE, 9808 },
	{ 0xf68e, 0xf68e, PDF_CMAP_SINGLE, 9810 },
	{ 0xf68f, 0xf691, PDF_CMAP_RANGE, 21474 },
	{ 0xf692, 0xf692, PDF_CMAP_SINGLE, 9807 },
	{ 0xf693, 0xf695, PDF_CMAP_RANGE, 21477 },
	{ 0xf696, 0xf696, PDF_CMAP_SINGLE, 9778 },
	{ 0xf697, 0xf697, PDF_CMAP_SINGLE, 9806 },
	{ 0xf698, 0xf698, PDF_CMAP_SINGLE, 9811 },
	{ 0xf699, 0xf699, PDF_CMAP_SINGLE, 21480 },
	{ 0xf69a, 0xf69a, PDF_CMAP_SINGLE, 9815 },
	{ 0xf69b, 0xf69b, PDF_CMAP_SINGLE, 21481 },
	{ 0xf69c, 0xf69c, PDF_CMAP_SINGLE, 9781 },
	{ 0xf69d, 0xf69d, PDF_CMAP_SINGLE, 21482 },
	{ 0xf69e, 0xf69e, PDF_CMAP_SINGLE, 9779 },
	{ 0xf69f, 0xf69f, PDF_CMAP_SINGLE, 21483 },
	{ 0xf6a0, 0xf6a0, PDF_CMAP_SINGLE, 9814 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf740, 0xf741, PDF_CMAP_RANGE, 21484 },
	{ 0xf742, 0xf742, PDF_CMAP_SINGLE, 9812 },
	{ 0xf743, 0xf748, PDF_CMAP_RANGE, 21486 },
	{ 0xf749, 0xf749, PDF_CMAP_SINGLE, 9816 },
	{ 0xf74a, 0xf74b, PDF_CMAP_RANGE, 21492 },
	{ 0xf74c, 0xf74c, PDF_CMAP_SINGLE, 9813 },
	{ 0xf74d, 0xf74d, PDF_CMAP_SINGLE, 7757 },
	{ 0xf74e, 0xf755, PDF_CMAP_RANGE, 21494 },
	{ 0xf756, 0xf756, PDF_CMAP_SINGLE, 9819 },
	{ 0xf757, 0xf757, PDF_CMAP_SINGLE, 21502 },
	{ 0xf758, 0xf758, PDF_CMAP_SINGLE, 9818 },
	{ 0xf759, 0xf759, PDF_CMAP_SINGLE, 21503 },
	{ 0xf75a, 0xf75a, PDF_CMAP_SINGLE, 9817 },
	{ 0xf75b, 0xf75b, PDF_CMAP_SINGLE, 8238 },
	{ 0xf75c, 0xf75c, PDF_CMAP_SINGLE, 9775 },
	{ 0xf75d, 0xf760, PDF_CMAP_RANGE, 21504 },
	{ 0xf761, 0xf761, PDF_CMAP_SINGLE, 9798 },
	{ 0xf762, 0xf762, PDF_CMAP_SINGLE, 21508 },
	{ 0xf763, 0xf763, PDF_CMAP_SINGLE, 9766 },
	{ 0xf764, 0xf76a, PDF_CMAP_RANGE, 21509 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 9820 },
	{ 0xf76c, 0xf770, PDF_CMAP_RANGE, 21516 },
	{ 0xf771, 0xf771, PDF_CMAP_SINGLE, 9772 },
	{ 0xf772, 0xf77b, PDF_CMAP_RANGE, 21521 },
	{ 0xf77c, 0xf77c, PDF_CMAP_SINGLE, 9763 },
	{ 0xf77d, 0xf77d, PDF_CMAP_SINGLE, 21531 },
	{ 0xf77e, 0xf77e, PDF_CMAP_SINGLE, 9777 },
	{ 0xf780, 0xf7a0, PDF_CMAP_RANGE, 21532 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
	{ 0xf840, 0xf841, PDF_CMAP_RANGE, 21565 },
	{ 0xf842, 0xf842, PDF_CMAP_SINGLE, 8348 },
	{ 0xf843, 0xf845, PDF_CMAP_RANGE, 21567 },
	{ 0xf846, 0xf846, PDF_CMAP_SINGLE, 9600 },
	{ 0xf847, 0xf848, PDF_CMAP_RANGE, 21570 },
	{ 0xf849, 0xf849, PDF_CMAP_SINGLE, 8924 },
	{ 0xf84a, 0xf84f, PDF_CMAP_RANGE, 21572 },
	{ 0xf850, 0xf850, PDF_CMAP_SINGLE, 7941 },
	{ 0xf851, 0xf851, PDF_CMAP_SINGLE, 8331 },
	{ 0xf852, 0xf852, PDF_CMAP_SINGLE, 21578 },
	{ 0xf853, 0xf853, PDF_CMAP_SINGLE, 9601 },
	{ 0xf854, 0xf862, PDF_CMAP_RANGE, 21579 },
	{ 0xf863, 0xf863, PDF_CMAP_SINGLE, 9603 },
	{ 0xf864, 0xf864, PDF_CMAP_SINGLE, 9602 },
	{ 0xf865, 0xf865, PDF_CMAP_SINGLE, 21594 },
	{ 0xf866, 0xf866, PDF_CMAP_SINGLE, 8686 },
	{ 0xf867, 0xf871, PDF_CMAP_RANGE, 21595 },
	{ 0xf872, 0xf872, PDF_CMAP_SINGLE, 8578 },
	{ 0xf873, 0xf877, PDF_CMAP_RANGE, 21606 },
	{ 0xf878, 0xf878, PDF_CMAP_SINGLE, 8771 },
	{ 0xf879, 0xf879, PDF_CMAP_SINGLE, 21611 },
	{ 0xf87a, 0xf87a, PDF_CMAP_SINGLE, 9607 },
	{ 0xf87b, 0xf87b, PDF_CMAP_SINGLE, 21612 },
	{ 0xf87c, 0xf87c, PDF_CMAP_SINGLE, 9608 },
	{ 0xf87d, 0xf87e, PDF_CMAP_RANGE, 21613 },
	{ 0xf880, 0xf880, PDF_CMAP_SINGLE, 21615 },
	{ 0xf881, 0xf881, PDF_CMAP_SINGLE, 9604 },
	{ 0xf882, 0xf883, PDF_CMAP_RANGE, 21616 },
	{ 0xf884, 0xf884, PDF_CMAP_SINGLE, 8701 },
	{ 0xf885, 0xf885, PDF_CMAP_SINGLE, 21618 },
	{ 0xf886, 0xf886, PDF_CMAP_SINGLE, 8687 },
	{ 0xf887, 0xf88c, PDF_CMAP_RANGE, 21619 },
	{ 0xf88d, 0xf88d, PDF_CMAP_SINGLE, 9610 },
	{ 0xf88e, 0xf88e, PDF_CMAP_SINGLE, 9612 },
	{ 0xf88f, 0xf898, PDF_CMAP_RANGE, 21625 },
	{ 0xf899, 0xf899, PDF_CMAP_SINGLE, 8007 },
	{ 0xf89a, 0xf89c, PDF_CMAP_RANGE, 21635 },
	{ 0xf89d, 0xf89d, PDF_CMAP_SINGLE, 7965 },
	{ 0xf89e, 0xf89f, PDF_CMAP_RANGE, 21638 },
	{ 0xf8a0, 0xf8a0, PDF_CMAP_SINGLE, 9613 },
	{ 0xf940, 0xf94d, PDF_CMAP_RANGE, 21640 },
	{ 0xf94e, 0xf94e, PDF_CMAP_SINGLE, 8144 },
	{ 0xf94f, 0xf94f, PDF_CMAP_SINGLE, 9618 },
	{ 0xf950, 0xf950, PDF_CMAP_SINGLE, 9615 },
	{ 0xf951, 0xf958, PDF_CMAP_RANGE, 21654 },
	{ 0xf959, 0xf959, PDF_CMAP_SINGLE, 9620 },
	{ 0xf95a, 0xf95a, PDF_CMAP_SINGLE, 7904 },
	{ 0xf95b, 0xf95c, PDF_CMAP_RANGE, 21662 },
	{ 0xf95d, 0xf95d, PDF_CMAP_SINGLE, 9617 },
	{ 0xf95e, 0xf95e, PDF_CMAP_SINGLE, 9621 },
	{ 0xf95f, 0xf966, PDF_CMAP_RANGE, 21664 },
	{ 0xf967, 0xf967, PDF_CMAP_SINGLE, 9623 },
	{ 0xf968, 0xf968, PDF_CMAP_SINGLE, 21672 },
	{ 0xf969, 0xf969, PDF_CMAP_SINGLE, 8374 },
	{ 0xf96a, 0xf96b, PDF_CMAP_RANGE, 21673 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 9624 },
	{ 0xf96d, 0xf96e, PDF_CMAP_RANGE, 21675 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 8438 },
	{ 0xf970, 0xf97e, PDF_CMAP_RANGE, 21677 },
	{ 0xf980, 0xf984, PDF_CMAP_RANGE, 21692 },
	{ 0xf985, 0xf985, PDF_CMAP_SINGLE, 9605 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 21697 },
	{ 0xf987, 0xf987, PDF_CMAP_SINGLE, 9625 },
	{ 0xf988, 0xf990, PDF_CMAP_RANGE, 21698 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 9622 },
	{ 0xf992, 0xf995, PDF_CMAP_RANGE, 21707 },
	{ 0xf996, 0xf996, PDF_CMAP_SINGLE, 9626 },
	{ 0xf997, 0xf997, PDF_CMAP_SINGLE, 21711 },
	{ 0xf998, 0xf998, PDF_CMAP_SINGLE, 9627 },
	{ 0xf999, 0xf9a0, PDF_CMAP_RANGE, 21712 },
	{ 0xfa40, 0xfa41, PDF_CMAP_RANGE, 21720 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 9629 },
	{ 0xfa43, 0xfa45, PDF_CMAP_RANGE, 21722 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 9630 },
	{ 0xfa47, 0xfa4b, PDF_CMAP_RANGE, 21725 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 9036 },
	{ 0xfa4d, 0xfa50, PDF_CMAP_RANGE, 21730 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 8004 },
	{ 0xfa52, 0xfa57, PDF_CMAP_RANGE, 21734 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 9824 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 9632 },
	{ 0xfa5a, 0xfa5c, PDF_CMAP_RANGE, 21740 },
	{ 0xfa5d, 0xfa5d, PDF_CMAP_SINGLE, 9628 },
	{ 0xfa5e, 0xfa5e, PDF_CMAP_SINGLE, 21743 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 9631 },
	{ 0xfa60, 0xfa60, PDF_CMAP_SINGLE, 21744 },
	{ 0xfa61, 0xfa61, PDF_CMAP_SINGLE, 8044 },
	{ 0xfa62, 0xfa6f, PDF_CMAP_RANGE, 21745 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 9634 },
	{ 0xfa71, 0xfa73, PDF_CMAP_RANGE, 21759 },
	{ 0xfa74, 0xfa74, PDF_CMAP_SINGLE, 8366 },
	{ 0xfa75, 0xfa75, PDF_CMAP_SINGLE, 21762 },
	{ 0xfa76, 0xfa76, PDF_CMAP_SINGLE, 9611 },
	{ 0xfa77, 0xfa77, PDF_CMAP_SINGLE, 9635 },
	{ 0xfa78, 0xfa7e, PDF_CMAP_RANGE, 21763 },
	{ 0xfa80, 0xfa82, PDF_CMAP_RANGE, 21770 },
	{ 0xfa83, 0xfa83, PDF_CMAP_SINGLE, 9609 },
	{ 0xfa84, 0xfa84, PDF_CMAP_SINGLE, 9637 },
	{ 0xfa85, 0xfa8c, PDF_CMAP_RANGE, 21773 },
	{ 0xfa8d, 0xfa8d, PDF_CMAP_SINGLE, 9636 },
	{ 0xfa8e, 0xfa8f, PDF_CMAP_RANGE, 21781 },
	{ 0xfa90, 0xfa90, PDF_CMAP_SINGLE, 9638 },
	{ 0xfa91, 0xfa91, PDF_CMAP_SINGLE, 9619 },
	{ 0xfa92, 0xfa95, PDF_CMAP_RANGE, 21783 },
	{ 0xfa96, 0xfa96, PDF_CMAP_SINGLE, 9639 },
	{ 0xfa97, 0xfa97, PDF_CMAP_SINGLE, 8738 },
	{ 0xfa98, 0xfa98, PDF_CMAP_SINGLE, 9641 },
	{ 0xfa99, 0xfaa0, PDF_CMAP_RANGE, 21787 },
	{ 0xfb40, 0xfb48, PDF_CMAP_RANGE, 21795 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 9640 },
	{ 0xfb4a, 0xfb51, PDF_CMAP_RANGE, 21804 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 9606 },
	{ 0xfb53, 0xfb56, PDF_CMAP_RANGE, 21812 },
	{ 0xfb57, 0xfb57, PDF_CMAP_SINGLE, 9633 },
	{ 0xfb58, 0xfb58, PDF_CMAP_SINGLE, 9642 },
	{ 0xfb59, 0xfb59, PDF_CMAP_SINGLE, 21816 },
	{ 0xfb5a, 0xfb5a, PDF_CMAP_SINGLE, 9616 },
	{ 0xfb5b, 0xfb5b, PDF_CMAP_SINGLE, 9614 },
	{ 0xfb5c, 0xfb74, PDF_CMAP_RANGE, 21817 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 9864 },
	{ 0xfb76, 0xfb78, PDF_CMAP_RANGE, 21842 },
	{ 0xfb79, 0xfb79, PDF_CMAP_SINGLE, 9886 },
	{ 0xfb7a, 0xfb7a, PDF_CMAP_SINGLE, 9723 },
	{ 0xfb7b, 0xfb7b, PDF_CMAP_SINGLE, 21845 },
	{ 0xfb7c, 0xfb7c, PDF_CMAP_SINGLE, 8076 },
	{ 0xfb7d, 0xfb7d, PDF_CMAP_SINGLE, 8692 },
	{ 0xfb7e, 0xfb7e, PDF_CMAP_SINGLE, 21846 },
	{ 0xfb80, 0xfb8f, PDF_CMAP_RANGE, 21847 },
	{ 0xfb90, 0xfb90, PDF_CMAP_SINGLE, 8207 },
	{ 0xfb91, 0xfb9b, PDF_CMAP_RANGE, 21863 },
	{ 0xfb9c, 0xfb9c, PDF_CMAP_SINGLE, 8305 },
	{ 0xfb9d, 0xfb9e, PDF_CMAP_RANGE, 21874 },
	{ 0xfb9f, 0xfb9f, PDF_CMAP_SINGLE, 9719 },
	{ 0xfba0, 0xfba0, PDF_CMAP_SINGLE, 21876 },
	{ 0xfc40, 0xfc43, PDF_CMAP_RANGE, 21877 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 9878 },
	{ 0xfc45, 0xfc48, PDF_CMAP_RANGE, 21881 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 9871 },
	{ 0xfc4a, 0xfc59, PDF_CMAP_RANGE, 21885 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 8922 },
	{ 0xfc5b, 0xfc62, PDF_CMAP_RANGE, 21901 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 7873 },
	{ 0xfc64, 0xfc67, PDF_CMAP_RANGE, 21909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 7859 },
	{ 0xfc69, 0xfc6e, PDF_CMAP_RANGE, 21913 },
	{ 0xfc6f, 0xfc6f, PDF_CMAP_SINGLE, 9834 },
	{ 0xfc70, 0xfc70, PDF_CMAP_SINGLE, 21919 },
	{ 0xfc71, 0xfc71, PDF_CMAP_SINGLE, 9866 },
	{ 0xfc72, 0xfc73, PDF_CMAP_RANGE, 21920 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 9833 },
	{ 0xfc75, 0xfc76, PDF_CMAP_RANGE, 21922 },
	{ 0xfc77, 0xfc78, PDF_CMAP_RANGE, 9752 },
	{ 0xfc79, 0xfc7e, PDF_CMAP_RANGE, 21924 },
	{ 0xfc80, 0xfc82, PDF_CMAP_RANGE, 21930 },
	{ 0xfc83, 0xfc83, PDF_CMAP_SINGLE, 9754 },
	{ 0xfc84, 0xfc89, PDF_CMAP_RANGE, 21933 },
	{ 0xfc8a, 0xfc8a, PDF_CMAP_SINGLE, 9843 },
	{ 0xfc8b, 0xfca0, PDF_CMAP_RANGE, 21939 },
	{ 0xfd40, 0xfd51, PDF_CMAP_RANGE, 21961 },
	{ 0xfd52, 0xfd52, PDF_CMAP_SINGLE, 8390 },
	{ 0xfd53, 0xfd53, PDF_CMAP_SINGLE, 8811 },
	{ 0xfd54, 0xfd56, PDF_CMAP_RANGE, 21979 },
	{ 0xfd57, 0xfd57, PDF_CMAP_SINGLE, 9427 },
	{ 0xfd58, 0xfd58, PDF_CMAP_SINGLE, 7814 },
	{ 0xfd59, 0xfd59, PDF_CMAP_SINGLE, 21982 },
	{ 0xfd5a, 0xfd5a, PDF_CMAP_SINGLE, 9743 },
	{ 0xfd5b, 0xfd5e, PDF_CMAP_RANGE, 21983 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 9745 },
	{ 0xfd60, 0xfd61, PDF_CMAP_RANGE, 21987 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 9747 },
	{ 0xfd63, 0xfd64, PDF_CMAP_RANGE, 21989 },
	{ 0xfd65, 0xfd65, PDF_CMAP_SINGLE, 9744 },
	{ 0xfd66, 0xfd66, PDF_CMAP_SINGLE, 9746 },
	{ 0xfd67, 0xfd67, PDF_CMAP_SINGLE, 8240 },
	{ 0xfd68, 0xfd68, PDF_CMAP_SINGLE, 21991 },
	{ 0xfd69, 0xfd69, PDF_CMAP_SINGLE, 9841 },
	{ 0xfd6a, 0xfd6b, PDF_CMAP_RANGE, 21992 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 9748 },
	{ 0xfd6d, 0xfd6f, PDF_CMAP_RANGE, 21994 },
	{ 0xfd70, 0xfd70, PDF_CMAP_SINGLE, 9750 },
	{ 0xfd71, 0xfd71, PDF_CMAP_SINGLE, 21997 },
	{ 0xfd72, 0xfd72, PDF_CMAP_SINGLE, 9749 },
	{ 0xfd73, 0xfd77, PDF_CMAP_RANGE, 21998 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 8434 },
	{ 0xfd79, 0xfd7c, PDF_CMAP_RANGE, 22003 },
	{ 0xfd7d, 0xfd7d, PDF_CMAP_SINGLE, 9751 },
	{ 0xfd7e, 0xfd7e, PDF_CMAP_SINGLE, 22007 },
	{ 0xfd80, 0xfd87, PDF_CMAP_RANGE, 22008 },
	{ 0xfd88, 0xfd88, PDF_CMAP_SINGLE, 8247 },
	{ 0xfd89, 0xfd8a, PDF_CMAP_RANGE, 22016 },
	{ 0xfd8b, 0xfd8b, PDF_CMAP_SINGLE, 8371 },
	{ 0xfd8c, 0xfd8e, PDF_CMAP_RANGE, 22018 },
	{ 0xfd8f, 0xfd8f, PDF_CMAP_SINGLE, 7970 },
	{ 0xfd90, 0xfd90, PDF_CMAP_SINGLE, 9453 },
	{ 0xfd91, 0xfd93, PDF_CMAP_RANGE, 22021 },
	{ 0xfd94, 0xfd94, PDF_CMAP_SINGLE, 7988 },
	{ 0xfd95, 0xfd9c, PDF_CMAP_RANGE, 22024 },
	{ 0xfd9d, 0xfd9d, PDF_CMAP_SINGLE, 2562 },
	{ 0xfd9e, 0xfd9e, PDF_CMAP_SINGLE, 16595 },
	{ 0xfd9f, 0xfd9f, PDF_CMAP_SINGLE, 8204 },
	{ 0xfda0, 0xfda0, PDF_CMAP_SINGLE, 20611 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 4697 },
	{ 0xfe41, 0xfe7e, PDF_CMAP_RANGE, 22032 },
	{ 0xfe80, 0xfea0, PDF_CMAP_RANGE, 22094 },
};

static const unsigned short pdf_cmap_GBK_EUC_H_table[1851] =
{
	7778,10315,7781,8833,8909,10407,8458,7742,
	7830,8907,8812,10424,8471,10428,8424,8108,
	8915,8720,10474,8911,10475,8723,10476,8164,
	8912,8910,8919,8917,8913,8906,7833,8143,
	8246,7994,10676,8905,8089,7885,10719,8617,
	10720,8678,10721,8487,8195,8901,8030,10771,
	8902,8208,10831,8898,8459,11063,8414,11064,
	7850,8408,8896,8368,11113,9082,7796,9080,
	8655,9064,11125,9073,11126,9065,9072,11153,
	11154,9078,9075,9087,11158,9076,11159,8373,
	7899,9842,9067,11193,8249,9085,8693,11202,
	9089,9083,9086,8652,9066,9865,11214,9879,
	9091,7997,11247,8591,8774,8776,8574,11254,
	8576,8709,7736,8163,9017,11440,9003,11441,
	9005,8615,11453,9007,8409,11517,8157,8199,
	8998,9001,8018,8251,9000,11540,8999,11590,
	8060,9051,7902,11614,8098,11615,7930,9210,
	11898,9214,8104,8421,12017,8495,8355,8481,
	12018,8660,8165,7818,7735,8094,8870,8680,
	12032,7898,7865,8272,7783,12074,9204,9097,
	12165,7863,9105,12268,8817,9104,12313,8243,
	12314,8764,8277,12333,9106,8837,12387,9092,
	9095,9094,8848,12406,7748,7730,9093,7955,
	8051,8327,7800,9134,7926,7985,9205,12521,
	8821,9838,12534,7856,8323,12538,8584,7841,
	12575,9107,9140,8318,8341,9147,7720,9149,
	7774,12819,7773,7983,9439,9138,8526,9148,
	12833,8427,8219,8380,9150,8329,12871,9137,
	8158,8739,9142,9868,9438,8190,8017,8673,
	12922,9136,12923,8142,12924,8478,8225,8819,
	12944,8624,8476,13097,9053,8461,8282,9057,
	13234,8256,9054,13252,7786,7852,7764,8384,
	9058,8544,13288,8605,8078,8749,13289,8263,
	13290,8802,8039,7858,7762,8356,7964,8847,
	13314,8173,7725,8530,9062,13319,8442,9059,
	13324,8346,8182,13335,9060,13336,7785,13337,
	9063,13338,8477,8793,8278,8547,8105,8188,
	7868,13388,8514,8222,7747,8790,9384,9383,
	13593,9862,9382,13594,8654,8565,13863,9325,
	9323,8799,9320,14046,8448,14058,9334,7975,
	8407,8874,8197,8387,8512,9335,8040,14171,
	8581,8075,9348,9344,9326,14266,9339,9350,
	9329,9347,9328,14274,9321,14275,9343,9419,
	14362,8019,8211,7987,9353,9355,8463,8155,
	8394,14468,8423,9410,8772,9186,14706,8404,
	7779,8328,7869,14840,9025,8264,9184,8311,
	14865,8762,8001,8220,8095,14903,9674,14904,
	9185,8172,9187,8462,8102,8196,8886,8361,
	8557,15007,8593,7760,8092,15023,9178,15024,
	8275,9193,9188,8661,15033,9190,8212,15045,
	9196,9195,9176,9198,9870,15058,9197,9174,
	9192,8585,8280,8473,15342,8124,15370,9434,
	8712,15414,7910,8630,7741,8757,15560,9111,
	8767,8490,7890,15595,9110,9112,15596,8354,
	15597,8235,15600,9108,8505,15601,8543,8641,
	9114,8741,8298,7937,8704,9650,7831,16025,
	8363,8232,16038,16040,9649,8706,9643,8838,
	16057,9645,16058,9654,16059,8675,8751,8549,
	9655,7717,16083,9664,16107,8858,8814,16126,
	8125,16127,8068,8370,16128,8259,16205,9455,
	9445,16366,8698,9446,8439,16413,8299,16471,
	7826,9447,8210,7916,8038,9436,8206,16581,
	9435,7864,9764,8041,8747,9599,8027,8603,
	16704,9853,8608,8582,16738,8429,7844,8417,
	16756,9656,8800,16757,8418,8868,16912,9704,
	9706,16970,8079,16971,9708,8400,8221,16992,
	9710,8250,9701,9709,17024,8201,17025,8293,
	9717,7932,9883,17108,9715,17109,8921,17110,
	9716,17111,9243,8136,17115,8059,17116,9245,
	17117,8780,8008,9244,8447,8602,17118,8337,
	9251,7835,9250,17123,8464,17124,8845,8049,
	7928,9249,8625,9254,9253,8480,8474,9252,
	17143,9256,9258,9257,17146,8852,17147,8894,
	17148,7729,9260,17156,8117,9259,8111,8296,
	8676,7969,17174,8449,8572,8522,9261,17184,
	8145,7731,17192,9263,9262,17195,9264,8667,
	8889,17212,9270,7822,9273,8645,9271,8594,
	17222,9274,7961,8588,7744,8883,8287,9272,
	9266,8820,17226,7836,9265,8325,8119,9267,
	8276,8670,17240,9268,9277,8073,9276,8046,
	7896,17246,7871,9285,17247,8777,9281,17250,
	7751,8022,8598,17255,9283,9279,17256,8227,
	17257,9282,9278,17260,9038,9286,9291,9284,
	9255,9292,7951,17286,9287,9289,9288,7939,
	17296,9290,8540,8891,9296,17299,9876,9295,
	9718,8273,17300,9294,8890,8045,9298,9297,
	9301,17318,8841,8470,8486,8033,9885,17335,
	8074,9302,9305,9304,8058,9293,9303,9299,
	17353,9248,17354,8671,9861,17355,7790,8740,
	9840,17359,8634,17360,9306,9698,17392,9882,
	8290,9457,17425,9458,9666,17520,9665,8488,
	17542,8215,7840,8485,8525,17561,9668,8349,
	8842,17562,9667,17712,8345,7853,9416,8360,
	8223,17732,8389,17733,9418,8176,17742,9414,
	8797,8926,8763,8665,8139,8137,9711,17818,
	8088,17821,9712,18004,8013,9031,18078,8595,
	9037,9032,7776,18133,9029,8216,9013,9028,
	9009,18228,9035,18229,9019,7860,8614,9022,
	18259,8052,18260,9008,8081,18279,8455,8180,
	9027,8788,18333,9010,7718,9047,9048,8258,
	8531,8289,18386,8265,18406,8002,18571,8606,
	7787,18646,9691,18654,8721,8745,9686,9693,
	9695,8175,18679,9688,8600,18718,7816,8844,
	18733,7768,8876,9846,18823,8161,9659,9660,
	9658,8583,18893,9835,8502,18925,9400,8420,
	18941,9401,8189,18954,9403,7981,9737,18980,
	9738,18981,7828,8927,7883,7949,8055,18992,
	8683,18993,8929,18994,8559,18995,8928,8682,
	18998,8930,8395,18999,8056,7906,19002,8690,
	19003,8528,8147,19007,8933,8532,19019,8935,
	8932,19038,8810,19039,8939,19040,8937,8381,
	8950,8679,8726,8492,19053,7784,8946,7991,
	8947,19054,8943,8016,7952,8648,19055,8945,
	8942,19063,8941,8862,8940,8162,19073,7855,
	8758,19076,8951,8765,19080,7810,8118,19081,
	8612,8618,8952,8529,19082,8032,8517,19089,
	8156,19101,8884,8553,19104,8957,19105,8426,
	19106,8948,19107,8955,19108,8956,8231,8288,
	8959,8958,7879,19115,8972,8949,19119,8966,
	19120,8970,8659,8963,19124,8967,8971,8031,
	19127,8969,8962,7940,8861,19130,8699,19131,
	8968,19132,8364,19133,8334,8965,8599,19134,
	8561,8856,8322,19141,8977,8964,19145,8975,
	7733,19146,8401,8976,8099,19147,8662,8978,
	8333,8979,8982,8043,8980,19192,8496,8981,
	8552,19193,8388,8729,8727,7789,8440,19236,
	8187,8985,8974,19242,8983,7739,8831,19329,
	7948,7769,7972,8378,8037,7920,8548,7984,
	8801,19334,19335,8866,9387,19336,9391,7912,
	7993,19337,7752,8304,7848,19338,9388,7927,
	8566,9389,19339,8315,8005,9386,8267,8239,
	8026,9392,19340,8887,8063,19341,8805,9394,
	8475,7761,19348,9396,9395,19353,7839,19354,
	8472,19355,8372,9135,8635,8306,8085,19356,
	7946,9398,8849,9397,8824,8873,9399,7976,
	8457,8903,9390,8881,8806,19377,8795,8900,
	19378,8469,19379,8746,9393,7957,8796,9724,
	7820,9730,19571,8781,19572,9732,9726,9731,
	7843,9736,7803,8807,7990,8150,8672,19616,
	9356,8857,9361,9366,9359,9362,19647,9367,
	8113,19652,9370,19653,9369,19654,8792,9368,
	9371,19662,7945,8422,8230,9375,8025,19672,
	7995,9372,7738,8283,8816,8759,9426,19692,
	8627,8773,9377,8828,8112,8006,19713,9088,
	9365,19714,9360,7837,7755,7754,8362,9202,
	19771,8830,19772,8217,8787,19797,7998,7846,
	8590,8684,19801,7870,19802,8778,8399,8674,
	19815,8719,19816,8233,19817,8307,9201,19820,
	7750,8291,9203,8892,19895,8988,19896,8785,
	8839,19914,8237,7851,19915,8989,19916,8991,
	8716,8100,8347,8664,19990,9722,19991,9721,
	8500,19994,9464,19995,9463,7880,9462,7877,
	9467,20003,20004,9466,20005,7917,9469,20017,
	9465,20018,9470,8397,9480,9476,9471,8336,
	7901,7973,20032,9475,9474,7802,8358,7953,
	20047,9479,9472,9477,9497,20054,9493,9484,
	8241,9483,9487,9498,9481,9486,8756,9491,
	8064,9495,9494,9485,20076,8403,8314,8398,
	8106,20101,9502,7967,9517,8733,9522,20107,
	8571,8623,20118,9516,9512,20121,8332,20122,
	9519,20123,8636,9525,20129,8717,9510,20130,
	9524,9514,9503,9521,9500,8653,20139,8666,
	20140,8562,9534,8271,20145,9539,20146,8663,
	20147,7740,9535,20165,9540,9507,7824,9530,
	9541,20169,9533,20170,8385,8451,9504,9532,
	9531,9528,9536,20177,8141,8880,20196,9542,
	7834,9554,9520,9545,9553,7882,20204,8402,
	8120,20208,8313,9549,7845,8268,8320,9837,
	20217,9527,9546,8632,20224,9550,9468,9556,
	20229,7996,20230,7893,9559,9555,9544,20253,
	8087,9557,8316,20270,9562,8542,20285,9568,
	8610,9044,9571,9511,20297,9518,9560,20298,
	7963,20299,8835,20300,9572,8352,20303,9573,
	9581,8224,9567,9578,9529,9564,9579,7791,
	20330,8132,9575,20331,9563,9515,20347,9585,
	20348,9591,8234,20369,9526,9587,9583,8851,
	9592,9594,8198,20391,8567,9499,20394,9508,
	9593,20407,9574,8083,20413,9596,9597,20427,
	9586,8782,20436,8646,8351,20441,9590,8292,
	8895,9756,8798,8317,20480,9151,8467,7749,
	20485,8152,9156,20486,9154,8452,20487,8637,
	20488,8071,9155,8003,20500,7966,9849,7915,
	7989,8330,20503,9159,9161,9158,8691,20513,
	8695,9167,9166,9162,9165,9836,20519,9153,
	20520,8174,9169,9170,9172,7832,7980,20532,
	9173,7793,9873,8732,20585,7806,8269,20599,
	8114,8535,20615,8639,8735,20621,8253,8213,
	20624,9893,8516,20637,7825,8791,8202,8338,
	9740,20702,9742,9823,20798,8589,8445,9319,
	20812,9318,8649,20835,8713,7881,8425,20836,
	8650,8518,9669,8668,20837,9310,20838,8527,
	20839,9670,9671,8769,8586,7727,7900,8383,
	8244,8718,20852,9675,9676,8131,8377,20869,
	8577,8563,7905,9678,8694,8779,9681,20890,
	7872,9682,20898,7978,7794,9683,8638,9684,
	8260,20904,9679,8435,20916,9880,20917,9848,
	8376,20932,9424,20933,9425,9115,20946,9854,
	20951,9121,20952,7921,8734,20955,9122,8523,
	7734,8501,8109,20967,7763,20970,9123,8707,
	20971,7911,9124,8343,7908,9118,21006,8245,
	9127,9128,21010,8309,9131,8171,9132,8042,
	8441,9831,21023,7788,8301,8770,7938,8579,
	21057,7813,8681,8869,9223,8138,21084,9218,
	8066,9220,21087,8497,21088,8580,21089,9219,
	21090,8302,9227,8151,21117,7811,9230,8391,
	9229,9200,21153,9233,9236,21154,8560,9221,
	8460,9237,8294,21165,9042,9235,9232,9225,
	21172,9240,8103,21178,8700,8129,9222,8270,
	21186,9242,9241,9825,21225,9892,8564,9827,
	9826,9832,21283,7888,21284,8342,21285,9164,
	9160,21290,8766,9828,21313,8761,9760,21349,
	9761,9762,21360,9767,7737,9765,9774,9771,
	9770,9773,21381,9768,8633,9782,9776,9784,
	21404,8205,9783,9797,21414,9786,21419,9792,
	9789,9793,21423,9790,9791,8128,21426,9785,
	21442,9804,9799,9803,9800,21456,9802,8456,
	9780,21471,9809,9808,9810,9778,9806,9811,
	21480,9815,21481,9781,21482,9779,21483,9814,
	9813,7757,9819,21502,9818,21503,9817,8238,
	9775,9798,21508,9766,9763,21531,9777,7941,
	8331,21578,9601,9603,9602,21594,8686,8771,
	21611,9607,21612,9608,21615,9604,8701,21618,
	8687,9610,9612,8144,9618,9615,9620,7904,
	9617,9621,9623,21672,8374,9605,21697,9625,
	9626,21711,9627,9824,9632,9628,21743,9631,
	21744,8044,8366,21762,9611,9635,9609,9637,
	9638,9619,9639,8738,9641,9633,9642,21816,
	9616,9614,9886,9723,21845,8076,8692,21846,
	9719,21876,9834,21919,9866,8390,8811,9427,
	7814,21982,9743,9744,9746,8240,21991,9841,
	9750,21997,9749,9751,22007,7970,9453,2562,
	16595,8204,20611,
};

pdf_cmap pdf_cmap_GBK_EUC_H =
{
	-1, "GBK-EUC-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0xfefe },
	},
	4071, 4071, (pdf_range*) pdf_cmap_GBK_EUC_H_ranges,
	1851, 1851, (unsigned short*) pdf_cmap_GBK_EUC_H_table,
};

/* GBK-EUC-V */

static const pdf_range pdf_cmap_GBK_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa3fd, 0xa3fd, PDF_CMAP_SINGLE, 597 },
	{ 0xa3fe, 0xa3fe, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GBK_EUC_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBK_EUC_V =
{
	-1, "GBK-EUC-V", "GBK-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GBK_EUC_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBK_EUC_V_table,
};

/* GBK2K-H */

static const pdf_range pdf_cmap_GBK2K_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8130, 0x8137, PDF_CMAP_RANGE, 22440 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 22690 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 23950 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 25210 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 26470 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 27730 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 28990 },
	{ 0x8130, 0x8139, PDF_CMAP_RANGE, 30091 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 22448 },
	{ 0x8140, 0x8178, PDF_CMAP_RANGE, 10072 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 8281 },
	{ 0x817a, 0x817e, PDF_CMAP_RANGE, 10129 },
	{ 0x8180, 0x8185, PDF_CMAP_RANGE, 10134 },
	{ 0x8186, 0x8186, PDF_CMAP_SINGLE, 8689 },
	{ 0x8187, 0x81ec, PDF_CMAP_RANGE, 10140 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 8178 },
	{ 0x81ee, 0x81f5, PDF_CMAP_RANGE, 10242 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 8285 },
	{ 0x81f7, 0x81fe, PDF_CMAP_RANGE, 10250 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 22449 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 22700 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 23960 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 25220 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 26480 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 27740 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 29000 },
	{ 0x8230, 0x8239, PDF_CMAP_RANGE, 30101 },
	{ 0x8240, 0x8252, PDF_CMAP_RANGE, 10258 },
	{ 0x8253, 0x8253, PDF_CMAP_SINGLE, 9884 },
	{ 0x8254, 0x8261, PDF_CMAP_RANGE, 10277 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 8629 },
	{ 0x8263, 0x8273, PDF_CMAP_RANGE, 10291 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 8908 },
	{ 0x8275, 0x8279, PDF_CMAP_RANGE, 10308 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 8214 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 10313 },
	{ 0x827d, 0x827e, PDF_CMAP_TABLE, 0 },
	{ 0x8280, 0x8280, PDF_CMAP_SINGLE, 7968 },
	{ 0x8281, 0x8282, PDF_CMAP_RANGE, 10316 },
	{ 0x8283, 0x8283, PDF_CMAP_SINGLE, 8319 },
	{ 0x8284, 0x828f, PDF_CMAP_RANGE, 10318 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 8284 },
	{ 0x8291, 0x82a4, PDF_CMAP_RANGE, 10330 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 8596 },
	{ 0x82a6, 0x82c7, PDF_CMAP_RANGE, 10350 },
	{ 0x82c8, 0x82c9, PDF_CMAP_TABLE, 2 },
	{ 0x82ca, 0x82e0, PDF_CMAP_RANGE, 10384 },
	{ 0x82e1, 0x82e4, PDF_CMAP_TABLE, 4 },
	{ 0x82e5, 0x82ec, PDF_CMAP_RANGE, 10408 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 9855 },
	{ 0x82ee, 0x82f1, PDF_CMAP_RANGE, 10416 },
	{ 0x82f2, 0x82f2, PDF_CMAP_SINGLE, 8750 },
	{ 0x82f3, 0x82f6, PDF_CMAP_RANGE, 10420 },
	{ 0x82f7, 0x82fb, PDF_CMAP_TABLE, 8 },
	{ 0x82fc, 0x82fe, PDF_CMAP_RANGE, 10425 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 22459 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 22710 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 23970 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 25230 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 26490 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 27750 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 29010 },
	{ 0x8330, 0x8339, PDF_CMAP_RANGE, 30111 },
	{ 0x8340, 0x8341, PDF_CMAP_TABLE, 13 },
	{ 0x8342, 0x8344, PDF_CMAP_RANGE, 10429 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 8916 },
	{ 0x8346, 0x8347, PDF_CMAP_RANGE, 10432 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 8121 },
	{ 0x8349, 0x834b, PDF_CMAP_RANGE, 10434 },
	{ 0x834c, 0x834c, PDF_CMAP_SINGLE, 8920 },
	{ 0x834d, 0x8352, PDF_CMAP_RANGE, 10437 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 8415 },
	{ 0x8354, 0x8356, PDF_CMAP_RANGE, 10443 },
	{ 0x8357, 0x8357, PDF_CMAP_SINGLE, 8386 },
	{ 0x8358, 0x835d, PDF_CMAP_RANGE, 10446 },
	{ 0x835e, 0x835e, PDF_CMAP_SINGLE, 8597 },
	{ 0x835f, 0x8364, PDF_CMAP_RANGE, 10452 },
	{ 0x8365, 0x8366, PDF_CMAP_TABLE, 15 },
	{ 0x8367, 0x8371, PDF_CMAP_RANGE, 10458 },
	{ 0x8372, 0x8372, PDF_CMAP_SINGLE, 8065 },
	{ 0x8373, 0x8377, PDF_CMAP_RANGE, 10469 },
	{ 0x8378, 0x837e, PDF_CMAP_TABLE, 17 },
	{ 0x8380, 0x8380, PDF_CMAP_SINGLE, 8080 },
	{ 0x8381, 0x8385, PDF_CMAP_RANGE, 10477 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 8918 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 10482 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 24 },
	{ 0x838b, 0x838c, PDF_CMAP_RANGE, 10484 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 9857 },
	{ 0x838e, 0x8393, PDF_CMAP_RANGE, 10486 },
	{ 0x8394, 0x8394, PDF_CMAP_SINGLE, 7798 },
	{ 0x8395, 0x839d, PDF_CMAP_RANGE, 10492 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 8753 },
	{ 0x839f, 0x83a5, PDF_CMAP_RANGE, 10501 },
	{ 0x83a6, 0x83a6, PDF_CMAP_SINGLE, 7827 },
	{ 0x83a7, 0x83aa, PDF_CMAP_RANGE, 10508 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 8914 },
	{ 0x83ac, 0x83ad, PDF_CMAP_RANGE, 10512 },
	{ 0x83ae, 0x83b0, PDF_CMAP_TABLE, 26 },
	{ 0x83b1, 0x83b9, PDF_CMAP_RANGE, 10514 },
	{ 0x83ba, 0x83ba, PDF_CMAP_SINGLE, 7909 },
	{ 0x83bb, 0x83c8, PDF_CMAP_RANGE, 10523 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 8229 },
	{ 0x83ca, 0x83f5, PDF_CMAP_RANGE, 10537 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 7887 },
	{ 0x83f7, 0x83fe, PDF_CMAP_RANGE, 10581 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 22469 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 22720 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 23980 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 25240 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 26500 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 27760 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 29020 },
	{ 0x8430, 0x8439, PDF_CMAP_RANGE, 30121 },
	{ 0x8436, 0x8436, PDF_CMAP_SINGLE, 22354 },
	{ 0x8440, 0x844f, PDF_CMAP_RANGE, 10589 },
	{ 0x8450, 0x8450, PDF_CMAP_SINGLE, 8153 },
	{ 0x8451, 0x8470, PDF_CMAP_RANGE, 10605 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 8904 },
	{ 0x8472, 0x8473, PDF_CMAP_RANGE, 10637 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 8803 },
	{ 0x8475, 0x8476, PDF_CMAP_RANGE, 10639 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 9859 },
	{ 0x8478, 0x847e, PDF_CMAP_RANGE, 10641 },
	{ 0x8480, 0x8481, PDF_CMAP_RANGE, 10648 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 7959 },
	{ 0x8483, 0x848d, PDF_CMAP_RANGE, 10650 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 7979 },
	{ 0x848f, 0x8491, PDF_CMAP_RANGE, 10661 },
	{ 0x8492, 0x8493, PDF_CMAP_TABLE, 29 },
	{ 0x8494, 0x849c, PDF_CMAP_RANGE, 10664 },
	{ 0x849d, 0x849d, PDF_CMAP_SINGLE, 8015 },
	{ 0x849e, 0x84a0, PDF_CMAP_RANGE, 10673 },
	{ 0x84a1, 0x84a6, PDF_CMAP_TABLE, 31 },
	{ 0x84a7, 0x84a8, PDF_CMAP_RANGE, 10677 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 8053 },
	{ 0x84aa, 0x84c4, PDF_CMAP_RANGE, 10679 },
	{ 0x84c5, 0x84c5, PDF_CMAP_SINGLE, 8126 },
	{ 0x84c6, 0x84d2, PDF_CMAP_RANGE, 10706 },
	{ 0x84d3, 0x84da, PDF_CMAP_TABLE, 37 },
	{ 0x84db, 0x84dc, PDF_CMAP_RANGE, 10722 },
	{ 0x84dd, 0x84dd, PDF_CMAP_SINGLE, 8498 },
	{ 0x84de, 0x84e9, PDF_CMAP_RANGE, 10724 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 8995 },
	{ 0x84eb, 0x84ed, PDF_CMAP_RANGE, 10736 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 8209 },
	{ 0x84ef, 0x84f0, PDF_CMAP_RANGE, 10739 },
	{ 0x84f1, 0x84f1, PDF_CMAP_SINGLE, 8437 },
	{ 0x84f2, 0x84fe, PDF_CMAP_RANGE, 10741 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 22479 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 22730 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 23990 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 25250 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 26510 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 27770 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 29030 },
	{ 0x8530, 0x8539, PDF_CMAP_RANGE, 30131 },
	{ 0x8540, 0x8550, PDF_CMAP_RANGE, 10754 },
	{ 0x8551, 0x8554, PDF_CMAP_TABLE, 45 },
	{ 0x8555, 0x855d, PDF_CMAP_RANGE, 10772 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 8431 },
	{ 0x855f, 0x8565, PDF_CMAP_RANGE, 10781 },
	{ 0x8566, 0x8566, PDF_CMAP_SINGLE, 8656 },
	{ 0x8567, 0x857e, PDF_CMAP_RANGE, 10788 },
	{ 0x8580, 0x8586, PDF_CMAP_RANGE, 10812 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 8897 },
	{ 0x8588, 0x858a, PDF_CMAP_RANGE, 10819 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 7780 },
	{ 0x858c, 0x8591, PDF_CMAP_RANGE, 10822 },
	{ 0x8592, 0x8592, PDF_CMAP_SINGLE, 8697 },
	{ 0x8593, 0x8595, PDF_CMAP_RANGE, 10828 },
	{ 0x8596, 0x8598, PDF_CMAP_TABLE, 49 },
	{ 0x8599, 0x85a1, PDF_CMAP_RANGE, 10832 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 7770 },
	{ 0x85a3, 0x85b1, PDF_CMAP_RANGE, 10841 },
	{ 0x85b2, 0x85b2, PDF_CMAP_SINGLE, 7842 },
	{ 0x85b3, 0x85fe, PDF_CMAP_RANGE, 10856 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 22489 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 22740 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 24000 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 25260 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 26520 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 27780 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 29040 },
	{ 0x8630, 0x8639, PDF_CMAP_RANGE, 30141 },
	{ 0x8640, 0x8649, PDF_CMAP_RANGE, 10932 },
	{ 0x864a, 0x864a, PDF_CMAP_SINGLE, 9069 },
	{ 0x864b, 0x8653, PDF_CMAP_RANGE, 10942 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 8775 },
	{ 0x8655, 0x8667, PDF_CMAP_RANGE, 10951 },
	{ 0x8668, 0x8668, PDF_CMAP_SINGLE, 9068 },
	{ 0x8669, 0x867e, PDF_CMAP_RANGE, 10970 },
	{ 0x8680, 0x8695, PDF_CMAP_RANGE, 10992 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 8604 },
	{ 0x8697, 0x8698, PDF_CMAP_RANGE, 11014 },
	{ 0x8699, 0x8699, PDF_CMAP_SINGLE, 8393 },
	{ 0x869a, 0x86a0, PDF_CMAP_RANGE, 11016 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 8688 },
	{ 0x86a2, 0x86c9, PDF_CMAP_RANGE, 11023 },
	{ 0x86ca, 0x86ce, PDF_CMAP_TABLE, 52 },
	{ 0x86cf, 0x86d0, PDF_CMAP_RANGE, 11065 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 8748 },
	{ 0x86d2, 0x86db, PDF_CMAP_RANGE, 11067 },
	{ 0x86dc, 0x86dd, PDF_CMAP_TABLE, 57 },
	{ 0x86de, 0x86e0, PDF_CMAP_RANGE, 11077 },
	{ 0x86e1, 0x86e1, PDF_CMAP_SINGLE, 8303 },
	{ 0x86e2, 0x86e7, PDF_CMAP_RANGE, 11080 },
	{ 0x86e8, 0x86e8, PDF_CMAP_SINGLE, 8609 },
	{ 0x86e9, 0x86ed, PDF_CMAP_RANGE, 11086 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 9081 },
	{ 0x86ef, 0x86f3, PDF_CMAP_RANGE, 11091 },
	{ 0x86f4, 0x86f4, PDF_CMAP_SINGLE, 9074 },
	{ 0x86f5, 0x86fe, PDF_CMAP_RANGE, 11096 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 22499 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 22750 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 24010 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 25270 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 26530 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 27790 },
	{ 0x8730, 0x8738, PDF_CMAP_RANGE, 29050 },
	{ 0x8730, 0x8739, PDF_CMAP_RANGE, 30151 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 8554 },
	{ 0x8741, 0x8743, PDF_CMAP_RANGE, 11106 },
	{ 0x8744, 0x8744, PDF_CMAP_SINGLE, 9084 },
	{ 0x8745, 0x8748, PDF_CMAP_RANGE, 11109 },
	{ 0x8749, 0x874c, PDF_CMAP_TABLE, 59 },
	{ 0x874d, 0x874e, PDF_CMAP_RANGE, 11114 },
	{ 0x874f, 0x874f, PDF_CMAP_SINGLE, 9079 },
	{ 0x8750, 0x8756, PDF_CMAP_RANGE, 11116 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 8012 },
	{ 0x8758, 0x8759, PDF_CMAP_RANGE, 11123 },
	{ 0x875a, 0x8760, PDF_CMAP_TABLE, 63 },
	{ 0x8761, 0x8765, PDF_CMAP_RANGE, 11127 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 9844 },
	{ 0x8767, 0x8779, PDF_CMAP_RANGE, 11132 },
	{ 0x877a, 0x877a, PDF_CMAP_SINGLE, 9071 },
	{ 0x877b, 0x877c, PDF_CMAP_RANGE, 11151 },
	{ 0x877d, 0x877e, PDF_CMAP_TABLE, 70 },
	{ 0x8780, 0x8782, PDF_CMAP_TABLE, 72 },
	{ 0x8783, 0x8785, PDF_CMAP_RANGE, 11155 },
	{ 0x8786, 0x878a, PDF_CMAP_TABLE, 75 },
	{ 0x878b, 0x878c, PDF_CMAP_RANGE, 11160 },
	{ 0x878d, 0x878e, PDF_CMAP_TABLE, 80 },
	{ 0x878f, 0x8792, PDF_CMAP_RANGE, 11162 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 9070 },
	{ 0x8794, 0x8797, PDF_CMAP_RANGE, 11166 },
	{ 0x8798, 0x8798, PDF_CMAP_SINGLE, 8631 },
	{ 0x8799, 0x879c, PDF_CMAP_RANGE, 11170 },
	{ 0x879d, 0x879d, PDF_CMAP_SINGLE, 9077 },
	{ 0x879e, 0x87a2, PDF_CMAP_RANGE, 11174 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 9090 },
	{ 0x87a4, 0x87a6, PDF_CMAP_RANGE, 11179 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 8350 },
	{ 0x87a8, 0x87b2, PDF_CMAP_RANGE, 11182 },
	{ 0x87b3, 0x87b5, PDF_CMAP_TABLE, 82 },
	{ 0x87b6, 0x87ba, PDF_CMAP_RANGE, 11194 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 9887 },
	{ 0x87bc, 0x87be, PDF_CMAP_RANGE, 11199 },
	{ 0x87bf, 0x87c2, PDF_CMAP_TABLE, 85 },
	{ 0x87c3, 0x87c9, PDF_CMAP_RANGE, 11203 },
	{ 0x87ca, 0x87cc, PDF_CMAP_TABLE, 89 },
	{ 0x87cd, 0x87ce, PDF_CMAP_RANGE, 11210 },
	{ 0x87cf, 0x87cf, PDF_CMAP_SINGLE, 8923 },
	{ 0x87d0, 0x87d1, PDF_CMAP_RANGE, 11212 },
	{ 0x87d2, 0x87d5, PDF_CMAP_TABLE, 92 },
	{ 0x87d6, 0x87d9, PDF_CMAP_RANGE, 11215 },
	{ 0x87da, 0x87da, PDF_CMAP_SINGLE, 8865 },
	{ 0x87db, 0x87f6, PDF_CMAP_RANGE, 11219 },
	{ 0x87f7, 0x87fa, PDF_CMAP_TABLE, 96 },
	{ 0x87fb, 0x87fe, PDF_CMAP_RANGE, 11248 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 22509 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 22760 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 24020 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 25280 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 26540 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 27800 },
	{ 0x8830, 0x8839, PDF_CMAP_RANGE, 30161 },
	{ 0x8840, 0x8841, PDF_CMAP_TABLE, 100 },
	{ 0x8842, 0x8843, PDF_CMAP_RANGE, 11252 },
	{ 0x8844, 0x8846, PDF_CMAP_TABLE, 102 },
	{ 0x8847, 0x887e, PDF_CMAP_RANGE, 11255 },
	{ 0x8880, 0x88b9, PDF_CMAP_RANGE, 11311 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 9002 },
	{ 0x88bb, 0x88cb, PDF_CMAP_RANGE, 11369 },
	{ 0x88cc, 0x88cc, PDF_CMAP_SINGLE, 8843 },
	{ 0x88cd, 0x88d3, PDF_CMAP_RANGE, 11386 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 8069 },
	{ 0x88d5, 0x88d6, PDF_CMAP_RANGE, 11393 },
	{ 0x88d7, 0x88d7, PDF_CMAP_SINGLE, 8997 },
	{ 0x88d8, 0x88de, PDF_CMAP_RANGE, 11395 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 9004 },
	{ 0x88e0, 0x88e4, PDF_CMAP_RANGE, 11402 },
	{ 0x88e5, 0x88e5, PDF_CMAP_SINGLE, 9006 },
	{ 0x88e6, 0x88f1, PDF_CMAP_RANGE, 11407 },
	{ 0x88f2, 0x88f3, PDF_CMAP_TABLE, 105 },
	{ 0x88f4, 0x88f5, PDF_CMAP_RANGE, 11419 },
	{ 0x88f6, 0x88f6, PDF_CMAP_SINGLE, 7795 },
	{ 0x88f7, 0x88fe, PDF_CMAP_RANGE, 11421 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 22519 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 22770 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 24030 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 25290 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 26550 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 27810 },
	{ 0x8930, 0x8939, PDF_CMAP_RANGE, 30171 },
	{ 0x8940, 0x894a, PDF_CMAP_RANGE, 11429 },
	{ 0x894b, 0x8950, PDF_CMAP_TABLE, 107 },
	{ 0x8951, 0x8953, PDF_CMAP_RANGE, 11442 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 8575 },
	{ 0x8955, 0x895c, PDF_CMAP_RANGE, 11445 },
	{ 0x895d, 0x895f, PDF_CMAP_TABLE, 113 },
	{ 0x8960, 0x896c, PDF_CMAP_RANGE, 11454 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 7805 },
	{ 0x896e, 0x8970, PDF_CMAP_RANGE, 11467 },
	{ 0x8971, 0x8971, PDF_CMAP_SINGLE, 8406 },
	{ 0x8972, 0x897b, PDF_CMAP_RANGE, 11470 },
	{ 0x897c, 0x897c, PDF_CMAP_SINGLE, 7874 },
	{ 0x897d, 0x897e, PDF_CMAP_RANGE, 11480 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 11482 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8882 },
	{ 0x898c, 0x8998, PDF_CMAP_RANGE, 11493 },
	{ 0x8999, 0x8999, PDF_CMAP_SINGLE, 7903 },
	{ 0x899a, 0x899d, PDF_CMAP_RANGE, 11506 },
	{ 0x899e, 0x899e, PDF_CMAP_SINGLE, 7929 },
	{ 0x899f, 0x89a5, PDF_CMAP_RANGE, 11510 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 116 },
	{ 0x89a9, 0x89ae, PDF_CMAP_RANGE, 11518 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 8551 },
	{ 0x89b0, 0x89b9, PDF_CMAP_RANGE, 11524 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8685 },
	{ 0x89bb, 0x89bd, PDF_CMAP_RANGE, 11534 },
	{ 0x89be, 0x89c0, PDF_CMAP_TABLE, 119 },
	{ 0x89c1, 0x89c3, PDF_CMAP_RANGE, 11537 },
	{ 0x89c4, 0x89c8, PDF_CMAP_TABLE, 122 },
	{ 0x89c9, 0x89cd, PDF_CMAP_RANGE, 11541 },
	{ 0x89ce, 0x89ce, PDF_CMAP_SINGLE, 7723 },
	{ 0x89cf, 0x89d0, PDF_CMAP_RANGE, 11546 },
	{ 0x89d1, 0x89d1, PDF_CMAP_SINGLE, 8878 },
	{ 0x89d2, 0x89d7, PDF_CMAP_RANGE, 11548 },
	{ 0x89d8, 0x89d8, PDF_CMAP_SINGLE, 8009 },
	{ 0x89d9, 0x89da, PDF_CMAP_RANGE, 11554 },
	{ 0x89db, 0x89db, PDF_CMAP_SINGLE, 8504 },
	{ 0x89dc, 0x89f3, PDF_CMAP_RANGE, 11556 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 8321 },
	{ 0x89f5, 0x89fe, PDF_CMAP_RANGE, 11580 },
	{ 0x8a30, 0x8a39, PDF_CMAP_RANGE, 22780 },
	{ 0x8a30, 0x8a39, PDF_CMAP_RANGE, 24040 },
	{ 0x8a30, 0x8a39, PDF_CMAP_RANGE, 25300 },
	{ 0x8a30, 0x8a39, PDF_CMAP_RANGE, 26560 },
	{ 0x8a30, 0x8a39, PDF_CMAP_RANGE, 27820 },
	{ 0x8a30, 0x8a39, PDF_CMAP_RANGE, 30181 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 127 },
	{ 0x8a42, 0x8a58, PDF_CMAP_RANGE, 11591 },
	{ 0x8a59, 0x8a5e, PDF_CMAP_TABLE, 129 },
	{ 0x8a5f, 0x8a78, PDF_CMAP_RANGE, 11616 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8877 },
	{ 0x8a7a, 0x8a7e, PDF_CMAP_RANGE, 11642 },
	{ 0x8a80, 0x8ae3, PDF_CMAP_RANGE, 11647 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 8255 },
	{ 0x8ae5, 0x8afe, PDF_CMAP_RANGE, 11747 },
	{ 0x8b30, 0x8b39, PDF_CMAP_RANGE, 22790 },
	{ 0x8b30, 0x8b39, PDF_CMAP_RANGE, 24050 },
	{ 0x8b30, 0x8b39, PDF_CMAP_RANGE, 25310 },
	{ 0x8b30, 0x8b39, PDF_CMAP_RANGE, 26570 },
	{ 0x8b30, 0x8b39, PDF_CMAP_RANGE, 27830 },
	{ 0x8b30, 0x8b39, PDF_CMAP_RANGE, 30191 },
	{ 0x8b32, 0x8b39, PDF_CMAP_TABLE, 135 },
	{ 0x8b40, 0x8b43, PDF_CMAP_RANGE, 11773 },
	{ 0x8b44, 0x8b44, PDF_CMAP_SINGLE, 7950 },
	{ 0x8b45, 0x8b48, PDF_CMAP_RANGE, 11777 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 9209 },
	{ 0x8b4a, 0x8b79, PDF_CMAP_RANGE, 11781 },
	{ 0x8b7a, 0x8b7a, PDF_CMAP_SINGLE, 9212 },
	{ 0x8b7b, 0x8b7e, PDF_CMAP_RANGE, 11829 },
	{ 0x8b80, 0x8b8b, PDF_CMAP_RANGE, 11833 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 8297 },
	{ 0x8b8d, 0x8b9d, PDF_CMAP_RANGE, 11845 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 9207 },
	{ 0x8b9f, 0x8bb2, PDF_CMAP_RANGE, 11862 },
	{ 0x8bb3, 0x8bb3, PDF_CMAP_SINGLE, 9206 },
	{ 0x8bb4, 0x8bb8, PDF_CMAP_RANGE, 11882 },
	{ 0x8bb9, 0x8bb9, PDF_CMAP_SINGLE, 9213 },
	{ 0x8bba, 0x8bbd, PDF_CMAP_RANGE, 11887 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 9208 },
	{ 0x8bbf, 0x8bc5, PDF_CMAP_RANGE, 11891 },
	{ 0x8bc6, 0x8bc9, PDF_CMAP_TABLE, 143 },
	{ 0x8bca, 0x8bd3, PDF_CMAP_RANGE, 11899 },
	{ 0x8bd4, 0x8bd4, PDF_CMAP_SINGLE, 9217 },
	{ 0x8bd5, 0x8bdb, PDF_CMAP_RANGE, 11909 },
	{ 0x8bdc, 0x8bdc, PDF_CMAP_SINGLE, 9215 },
	{ 0x8bdd, 0x8be4, PDF_CMAP_RANGE, 11916 },
	{ 0x8be5, 0x8be5, PDF_CMAP_SINGLE, 9216 },
	{ 0x8be6, 0x8bea, PDF_CMAP_RANGE, 11924 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 8737 },
	{ 0x8bec, 0x8bef, PDF_CMAP_RANGE, 11929 },
	{ 0x8bf0, 0x8bf0, PDF_CMAP_SINGLE, 8482 },
	{ 0x8bf1, 0x8bfe, PDF_CMAP_RANGE, 11933 },
	{ 0x8c30, 0x8c30, PDF_CMAP_SINGLE, 3238 },
	{ 0x8c30, 0x8c39, PDF_CMAP_RANGE, 22800 },
	{ 0x8c30, 0x8c39, PDF_CMAP_RANGE, 24060 },
	{ 0x8c30, 0x8c39, PDF_CMAP_RANGE, 25320 },
	{ 0x8c30, 0x8c39, PDF_CMAP_RANGE, 26580 },
	{ 0x8c30, 0x8c39, PDF_CMAP_RANGE, 27840 },
	{ 0x8c30, 0x8c39, PDF_CMAP_RANGE, 30201 },
	{ 0x8c31, 0x8c39, PDF_CMAP_TABLE, 147 },
	{ 0x8c40, 0x8c43, PDF_CMAP_RANGE, 11947 },
	{ 0x8c44, 0x8c44, PDF_CMAP_SINGLE, 9211 },
	{ 0x8c45, 0x8c4e, PDF_CMAP_RANGE, 11951 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 8538 },
	{ 0x8c50, 0x8c56, PDF_CMAP_RANGE, 11961 },
	{ 0x8c57, 0x8c57, PDF_CMAP_SINGLE, 8677 },
	{ 0x8c58, 0x8c5b, PDF_CMAP_RANGE, 11968 },
	{ 0x8c5c, 0x8c5c, PDF_CMAP_SINGLE, 8279 },
	{ 0x8c5d, 0x8c7e, PDF_CMAP_RANGE, 11972 },
	{ 0x8c80, 0x8c8a, PDF_CMAP_RANGE, 12006 },
	{ 0x8c8b, 0x8c92, PDF_CMAP_TABLE, 156 },
	{ 0x8c93, 0x8c98, PDF_CMAP_RANGE, 12019 },
	{ 0x8c99, 0x8c9a, PDF_CMAP_TABLE, 164 },
	{ 0x8c9b, 0x8ca1, PDF_CMAP_RANGE, 12025 },
	{ 0x8ca2, 0x8ca7, PDF_CMAP_TABLE, 166 },
	{ 0x8ca8, 0x8cbf, PDF_CMAP_RANGE, 12033 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 9052 },
	{ 0x8cc1, 0x8cd1, PDF_CMAP_RANGE, 12057 },
	{ 0x8cd2, 0x8cd5, PDF_CMAP_TABLE, 172 },
	{ 0x8cd6, 0x8cd8, PDF_CMAP_RANGE, 12075 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 8510 },
	{ 0x8cda, 0x8cf8, PDF_CMAP_RANGE, 12078 },
	{ 0x8cf9, 0x8cf9, PDF_CMAP_SINGLE, 7958 },
	{ 0x8cfa, 0x8cfe, PDF_CMAP_RANGE, 12109 },
	{ 0x8d30, 0x8d30, PDF_CMAP_SINGLE, 2543 },
	{ 0x8d30, 0x8d39, PDF_CMAP_RANGE, 22810 },
	{ 0x8d30, 0x8d39, PDF_CMAP_RANGE, 24070 },
	{ 0x8d30, 0x8d39, PDF_CMAP_RANGE, 25330 },
	{ 0x8d30, 0x8d39, PDF_CMAP_RANGE, 26590 },
	{ 0x8d30, 0x8d39, PDF_CMAP_RANGE, 27850 },
	{ 0x8d30, 0x8d39, PDF_CMAP_RANGE, 30211 },
	{ 0x8d31, 0x8d39, PDF_CMAP_TABLE, 176 },
	{ 0x8d40, 0x8d72, PDF_CMAP_RANGE, 12114 },
	{ 0x8d73, 0x8d75, PDF_CMAP_TABLE, 185 },
	{ 0x8d76, 0x8d7a, PDF_CMAP_RANGE, 12166 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 8628 },
	{ 0x8d7c, 0x8d7e, PDF_CMAP_RANGE, 12171 },
	{ 0x8d80, 0x8d87, PDF_CMAP_RANGE, 12174 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 9103 },
	{ 0x8d89, 0x8d8e, PDF_CMAP_RANGE, 12182 },
	{ 0x8d8f, 0x8d8f, PDF_CMAP_SINGLE, 7962 },
	{ 0x8d90, 0x8d9d, PDF_CMAP_RANGE, 12188 },
	{ 0x8d9e, 0x8d9e, PDF_CMAP_SINGLE, 9099 },
	{ 0x8d9f, 0x8db8, PDF_CMAP_RANGE, 12202 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 9098 },
	{ 0x8dba, 0x8de1, PDF_CMAP_RANGE, 12228 },
	{ 0x8de2, 0x8de4, PDF_CMAP_TABLE, 188 },
	{ 0x8de5, 0x8de6, PDF_CMAP_RANGE, 12269 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 9096 },
	{ 0x8de8, 0x8df6, PDF_CMAP_RANGE, 12271 },
	{ 0x8df7, 0x8df7, PDF_CMAP_SINGLE, 9102 },
	{ 0x8df8, 0x8dfd, PDF_CMAP_RANGE, 12286 },
	{ 0x8dfe, 0x8dfe, PDF_CMAP_SINGLE, 9101 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 4283 },
	{ 0x8e30, 0x8e39, PDF_CMAP_RANGE, 22820 },
	{ 0x8e30, 0x8e39, PDF_CMAP_RANGE, 24080 },
	{ 0x8e30, 0x8e39, PDF_CMAP_RANGE, 25340 },
	{ 0x8e30, 0x8e39, PDF_CMAP_RANGE, 26600 },
	{ 0x8e30, 0x8e39, PDF_CMAP_RANGE, 27860 },
	{ 0x8e30, 0x8e37, PDF_CMAP_RANGE, 30221 },
	{ 0x8e31, 0x8e39, PDF_CMAP_TABLE, 191 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 12292 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 9100 },
	{ 0x8e47, 0x8e55, PDF_CMAP_RANGE, 12298 },
	{ 0x8e56, 0x8e5a, PDF_CMAP_TABLE, 200 },
	{ 0x8e5b, 0x8e67, PDF_CMAP_RANGE, 12315 },
	{ 0x8e68, 0x8e68, PDF_CMAP_SINGLE, 8169 },
	{ 0x8e69, 0x8e6d, PDF_CMAP_RANGE, 12328 },
	{ 0x8e6e, 0x8e70, PDF_CMAP_TABLE, 205 },
	{ 0x8e71, 0x8e7e, PDF_CMAP_RANGE, 12334 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 8996 },
	{ 0x8e81, 0x8e9a, PDF_CMAP_RANGE, 12348 },
	{ 0x8e9b, 0x8e9b, PDF_CMAP_SINGLE, 8515 },
	{ 0x8e9c, 0x8e9e, PDF_CMAP_RANGE, 12374 },
	{ 0x8e9f, 0x8e9f, PDF_CMAP_SINGLE, 8489 },
	{ 0x8ea0, 0x8ea3, PDF_CMAP_RANGE, 12377 },
	{ 0x8ea4, 0x8ea4, PDF_CMAP_SINGLE, 8823 },
	{ 0x8ea5, 0x8ea6, PDF_CMAP_RANGE, 12381 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 7847 },
	{ 0x8ea8, 0x8eab, PDF_CMAP_RANGE, 12383 },
	{ 0x8eac, 0x8eae, PDF_CMAP_TABLE, 208 },
	{ 0x8eaf, 0x8ebc, PDF_CMAP_RANGE, 12388 },
	{ 0x8ebd, 0x8ebe, PDF_CMAP_TABLE, 211 },
	{ 0x8ebf, 0x8ec2, PDF_CMAP_RANGE, 12402 },
	{ 0x8ec3, 0x8ec5, PDF_CMAP_TABLE, 213 },
	{ 0x8ec6, 0x8ecc, PDF_CMAP_RANGE, 12407 },
	{ 0x8ecd, 0x8ece, PDF_CMAP_TABLE, 216 },
	{ 0x8ecf, 0x8ed5, PDF_CMAP_RANGE, 12414 },
	{ 0x8ed6, 0x8ed7, PDF_CMAP_TABLE, 218 },
	{ 0x8ed8, 0x8eeb, PDF_CMAP_RANGE, 12421 },
	{ 0x8eec, 0x8eec, PDF_CMAP_SINGLE, 8160 },
	{ 0x8eed, 0x8efe, PDF_CMAP_RANGE, 12441 },
	{ 0x8f30, 0x8f30, PDF_CMAP_SINGLE, 4656 },
	{ 0x8f30, 0x8f39, PDF_CMAP_RANGE, 22830 },
	{ 0x8f30, 0x8f39, PDF_CMAP_RANGE, 24090 },
	{ 0x8f30, 0x8f39, PDF_CMAP_RANGE, 25350 },
	{ 0x8f30, 0x8f39, PDF_CMAP_RANGE, 26610 },
	{ 0x8f30, 0x8f39, PDF_CMAP_RANGE, 27870 },
	{ 0x8f31, 0x8f31, PDF_CMAP_SINGLE, 5934 },
	{ 0x8f31, 0x8f39, PDF_CMAP_RANGE, 30229 },
	{ 0x8f32, 0x8f39, PDF_CMAP_TABLE, 220 },
	{ 0x8f40, 0x8f51, PDF_CMAP_RANGE, 12459 },
	{ 0x8f52, 0x8f56, PDF_CMAP_TABLE, 228 },
	{ 0x8f57, 0x8f5c, PDF_CMAP_RANGE, 12477 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 8261 },
	{ 0x8f5e, 0x8f63, PDF_CMAP_RANGE, 12483 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 8568 },
	{ 0x8f65, 0x8f7e, PDF_CMAP_RANGE, 12489 },
	{ 0x8f80, 0x8f85, PDF_CMAP_RANGE, 12515 },
	{ 0x8f86, 0x8f88, PDF_CMAP_TABLE, 233 },
	{ 0x8f89, 0x8f94, PDF_CMAP_RANGE, 12522 },
	{ 0x8f95, 0x8f97, PDF_CMAP_TABLE, 236 },
	{ 0x8f98, 0x8f9a, PDF_CMAP_RANGE, 12535 },
	{ 0x8f9b, 0x8f9d, PDF_CMAP_TABLE, 239 },
	{ 0x8f9e, 0x8fa0, PDF_CMAP_RANGE, 12539 },
	{ 0x8fa1, 0x8fa1, PDF_CMAP_SINGLE, 9852 },
	{ 0x8fa2, 0x8fbc, PDF_CMAP_RANGE, 12542 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 8133 },
	{ 0x8fbe, 0x8fc3, PDF_CMAP_RANGE, 12569 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 242 },
	{ 0x8fc7, 0x8fcc, PDF_CMAP_RANGE, 12576 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 7947 },
	{ 0x8fce, 0x8fd7, PDF_CMAP_RANGE, 12582 },
	{ 0x8fd8, 0x8fd8, PDF_CMAP_SINGLE, 7804 },
	{ 0x8fd9, 0x8ffe, PDF_CMAP_RANGE, 12592 },
	{ 0x9030, 0x9030, PDF_CMAP_SINGLE, 2093 },
	{ 0x9030, 0x9039, PDF_CMAP_RANGE, 22840 },
	{ 0x9030, 0x9039, PDF_CMAP_RANGE, 24100 },
	{ 0x9030, 0x9039, PDF_CMAP_RANGE, 25360 },
	{ 0x9030, 0x9039, PDF_CMAP_RANGE, 26620 },
	{ 0x9030, 0x9039, PDF_CMAP_RANGE, 27880 },
	{ 0x9030, 0x9039, PDF_CMAP_RANGE, 30238 },
	{ 0x9031, 0x9039, PDF_CMAP_TABLE, 245 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 12630 },
	{ 0x9080, 0x909c, PDF_CMAP_RANGE, 12693 },
	{ 0x909d, 0x909e, PDF_CMAP_TABLE, 254 },
	{ 0x909f, 0x90b9, PDF_CMAP_RANGE, 12722 },
	{ 0x90ba, 0x90ba, PDF_CMAP_SINGLE, 7907 },
	{ 0x90bb, 0x90bf, PDF_CMAP_RANGE, 12749 },
	{ 0x90c0, 0x90c1, PDF_CMAP_TABLE, 256 },
	{ 0x90c2, 0x90c4, PDF_CMAP_RANGE, 12754 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 9145 },
	{ 0x90c6, 0x90da, PDF_CMAP_RANGE, 12757 },
	{ 0x90db, 0x90dc, PDF_CMAP_TABLE, 258 },
	{ 0x90dd, 0x90ec, PDF_CMAP_RANGE, 12778 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 9141 },
	{ 0x90ee, 0x90ef, PDF_CMAP_RANGE, 12794 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 9146 },
	{ 0x90f1, 0x90f6, PDF_CMAP_RANGE, 12796 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 9139 },
	{ 0x90f8, 0x90fe, PDF_CMAP_RANGE, 12802 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 5614 },
	{ 0x9130, 0x9139, PDF_CMAP_RANGE, 22850 },
	{ 0x9130, 0x9139, PDF_CMAP_RANGE, 24110 },
	{ 0x9130, 0x9139, PDF_CMAP_RANGE, 25370 },
	{ 0x9130, 0x9139, PDF_CMAP_RANGE, 26630 },
	{ 0x9130, 0x9139, PDF_CMAP_RANGE, 27890 },
	{ 0x9130, 0x9139, PDF_CMAP_RANGE, 30248 },
	{ 0x9131, 0x9139, PDF_CMAP_TABLE, 260 },
	{ 0x9140, 0x9141, PDF_CMAP_RANGE, 12809 },
	{ 0x9142, 0x9142, PDF_CMAP_SINGLE, 8546 },
	{ 0x9143, 0x914a, PDF_CMAP_RANGE, 12811 },
	{ 0x914b, 0x914d, PDF_CMAP_TABLE, 269 },
	{ 0x914e, 0x9150, PDF_CMAP_RANGE, 12820 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 9143 },
	{ 0x9152, 0x9153, PDF_CMAP_RANGE, 12823 },
	{ 0x9154, 0x9155, PDF_CMAP_TABLE, 272 },
	{ 0x9156, 0x9158, PDF_CMAP_RANGE, 12825 },
	{ 0x9159, 0x915a, PDF_CMAP_TABLE, 274 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 12828 },
	{ 0x915d, 0x915d, PDF_CMAP_SINGLE, 8274 },
	{ 0x915e, 0x9160, PDF_CMAP_RANGE, 12830 },
	{ 0x9161, 0x9163, PDF_CMAP_TABLE, 276 },
	{ 0x9164, 0x916d, PDF_CMAP_RANGE, 12834 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 8754 },
	{ 0x916f, 0x9175, PDF_CMAP_RANGE, 12844 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 7743 },
	{ 0x9177, 0x9179, PDF_CMAP_RANGE, 12851 },
	{ 0x917a, 0x917c, PDF_CMAP_TABLE, 279 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 12854 },
	{ 0x9180, 0x9183, PDF_CMAP_RANGE, 12856 },
	{ 0x9184, 0x9184, PDF_CMAP_SINGLE, 7854 },
	{ 0x9185, 0x918c, PDF_CMAP_RANGE, 12860 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 7931 },
	{ 0x918e, 0x9190, PDF_CMAP_RANGE, 12868 },
	{ 0x9191, 0x9193, PDF_CMAP_TABLE, 282 },
	{ 0x9194, 0x9196, PDF_CMAP_RANGE, 12872 },
	{ 0x9197, 0x9197, PDF_CMAP_SINGLE, 8644 },
	{ 0x9198, 0x919a, PDF_CMAP_RANGE, 12875 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 8724 },
	{ 0x919c, 0x91a8, PDF_CMAP_RANGE, 12878 },
	{ 0x91a9, 0x91ab, PDF_CMAP_TABLE, 285 },
	{ 0x91ac, 0x91b9, PDF_CMAP_RANGE, 12891 },
	{ 0x91ba, 0x91bb, PDF_CMAP_TABLE, 288 },
	{ 0x91bc, 0x91be, PDF_CMAP_RANGE, 12905 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 9440 },
	{ 0x91c0, 0x91c2, PDF_CMAP_RANGE, 12908 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 9144 },
	{ 0x91c4, 0x91cc, PDF_CMAP_RANGE, 12911 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 7809 },
	{ 0x91ce, 0x91cf, PDF_CMAP_RANGE, 12920 },
	{ 0x91d0, 0x91d9, PDF_CMAP_TABLE, 290 },
	{ 0x91da, 0x91de, PDF_CMAP_RANGE, 12925 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 9441 },
	{ 0x91e0, 0x91e1, PDF_CMAP_RANGE, 12930 },
	{ 0x91e2, 0x91e2, PDF_CMAP_SINGLE, 9379 },
	{ 0x91e3, 0x91e9, PDF_CMAP_RANGE, 12932 },
	{ 0x91ea, 0x91ea, PDF_CMAP_SINGLE, 9380 },
	{ 0x91eb, 0x91ef, PDF_CMAP_RANGE, 12939 },
	{ 0x91f0, 0x91f2, PDF_CMAP_TABLE, 300 },
	{ 0x91f3, 0x91fe, PDF_CMAP_RANGE, 12945 },
	{ 0x9230, 0x9230, PDF_CMAP_SINGLE, 2240 },
	{ 0x9230, 0x9239, PDF_CMAP_RANGE, 22860 },
	{ 0x9230, 0x9239, PDF_CMAP_RANGE, 24120 },
	{ 0x9230, 0x9239, PDF_CMAP_RANGE, 25380 },
	{ 0x9230, 0x9239, PDF_CMAP_RANGE, 26640 },
	{ 0x9230, 0x9239, PDF_CMAP_RANGE, 27900 },
	{ 0x9230, 0x9239, PDF_CMAP_RANGE, 30258 },
	{ 0x9231, 0x9239, PDF_CMAP_TABLE, 303 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 12957 },
	{ 0x9280, 0x92b5, PDF_CMAP_RANGE, 13020 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 8657 },
	{ 0x92b7, 0x92cd, PDF_CMAP_RANGE, 13074 },
	{ 0x92ce, 0x92d0, PDF_CMAP_TABLE, 312 },
	{ 0x92d1, 0x92d3, PDF_CMAP_RANGE, 13098 },
	{ 0x92d4, 0x92d4, PDF_CMAP_SINGLE, 9858 },
	{ 0x92d5, 0x92de, PDF_CMAP_RANGE, 13101 },
	{ 0x92df, 0x92e0, PDF_CMAP_TABLE, 315 },
	{ 0x92e1, 0x92fd, PDF_CMAP_RANGE, 13111 },
	{ 0x92fe, 0x92fe, PDF_CMAP_SINGLE, 8077 },
	{ 0x9330, 0x9330, PDF_CMAP_SINGLE, 6589 },
	{ 0x9330, 0x9339, PDF_CMAP_RANGE, 22870 },
	{ 0x9330, 0x9339, PDF_CMAP_RANGE, 24130 },
	{ 0x9330, 0x9339, PDF_CMAP_RANGE, 25390 },
	{ 0x9330, 0x9339, PDF_CMAP_RANGE, 26650 },
	{ 0x9330, 0x9339, PDF_CMAP_RANGE, 27910 },
	{ 0x9330, 0x9339, PDF_CMAP_RANGE, 30268 },
	{ 0x9331, 0x9339, PDF_CMAP_TABLE, 317 },
	{ 0x9340, 0x934f, PDF_CMAP_RANGE, 13140 },
	{ 0x9350, 0x9350, PDF_CMAP_SINGLE, 8703 },
	{ 0x9351, 0x935c, PDF_CMAP_RANGE, 13156 },
	{ 0x935d, 0x935d, PDF_CMAP_SINGLE, 8024 },
	{ 0x935e, 0x936f, PDF_CMAP_RANGE, 13168 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 8539 },
	{ 0x9371, 0x9375, PDF_CMAP_RANGE, 13186 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 7862 },
	{ 0x9377, 0x937e, PDF_CMAP_RANGE, 13191 },
	{ 0x9380, 0x938b, PDF_CMAP_RANGE, 13199 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 8411 },
	{ 0x938d, 0x939c, PDF_CMAP_RANGE, 13211 },
	{ 0x939d, 0x939d, PDF_CMAP_SINGLE, 9056 },
	{ 0x939e, 0x93a4, PDF_CMAP_RANGE, 13227 },
	{ 0x93a5, 0x93a7, PDF_CMAP_TABLE, 326 },
	{ 0x93a8, 0x93b3, PDF_CMAP_RANGE, 13235 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 8846 },
	{ 0x93b5, 0x93b7, PDF_CMAP_RANGE, 13247 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 8159 },
	{ 0x93b9, 0x93ba, PDF_CMAP_RANGE, 13250 },
	{ 0x93bb, 0x93bd, PDF_CMAP_TABLE, 329 },
	{ 0x93be, 0x93c5, PDF_CMAP_RANGE, 13253 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 8194 },
	{ 0x93c7, 0x93ce, PDF_CMAP_RANGE, 13261 },
	{ 0x93cf, 0x93cf, PDF_CMAP_SINGLE, 8339 },
	{ 0x93d0, 0x93d6, PDF_CMAP_RANGE, 13269 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 9055 },
	{ 0x93d8, 0x93da, PDF_CMAP_RANGE, 13276 },
	{ 0x93db, 0x93dc, PDF_CMAP_TABLE, 332 },
	{ 0x93dd, 0x93e0, PDF_CMAP_RANGE, 13279 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 7944 },
	{ 0x93e2, 0x93e3, PDF_CMAP_RANGE, 13283 },
	{ 0x93e4, 0x93e5, PDF_CMAP_TABLE, 334 },
	{ 0x93e6, 0x93e8, PDF_CMAP_RANGE, 13285 },
	{ 0x93e9, 0x93f1, PDF_CMAP_TABLE, 336 },
	{ 0x93f2, 0x93f3, PDF_CMAP_RANGE, 13291 },
	{ 0x93f4, 0x93f5, PDF_CMAP_TABLE, 345 },
	{ 0x93f6, 0x93f9, PDF_CMAP_RANGE, 13293 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 7849 },
	{ 0x93fb, 0x93fd, PDF_CMAP_RANGE, 13297 },
	{ 0x93fe, 0x93fe, PDF_CMAP_SINGLE, 8140 },
	{ 0x9430, 0x9430, PDF_CMAP_SINGLE, 4713 },
	{ 0x9430, 0x9439, PDF_CMAP_RANGE, 22880 },
	{ 0x9430, 0x9439, PDF_CMAP_RANGE, 24140 },
	{ 0x9430, 0x9439, PDF_CMAP_RANGE, 25400 },
	{ 0x9430, 0x9439, PDF_CMAP_RANGE, 26660 },
	{ 0x9430, 0x9439, PDF_CMAP_RANGE, 27920 },
	{ 0x9430, 0x9435, PDF_CMAP_RANGE, 30278 },
	{ 0x9431, 0x9439, PDF_CMAP_TABLE, 347 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 13300 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8050 },
	{ 0x9445, 0x944c, PDF_CMAP_RANGE, 13304 },
	{ 0x944d, 0x944d, PDF_CMAP_SINGLE, 8344 },
	{ 0x944e, 0x944f, PDF_CMAP_RANGE, 13312 },
	{ 0x9450, 0x9455, PDF_CMAP_TABLE, 356 },
	{ 0x9456, 0x9457, PDF_CMAP_RANGE, 13315 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 9061 },
	{ 0x9459, 0x945a, PDF_CMAP_RANGE, 13317 },
	{ 0x945b, 0x945f, PDF_CMAP_TABLE, 362 },
	{ 0x9460, 0x9463, PDF_CMAP_RANGE, 13320 },
	{ 0x9464, 0x9466, PDF_CMAP_TABLE, 367 },
	{ 0x9467, 0x946d, PDF_CMAP_RANGE, 13325 },
	{ 0x946e, 0x946e, PDF_CMAP_SINGLE, 8252 },
	{ 0x946f, 0x9471, PDF_CMAP_RANGE, 13332 },
	{ 0x9472, 0x947a, PDF_CMAP_TABLE, 370 },
	{ 0x947b, 0x947e, PDF_CMAP_RANGE, 13339 },
	{ 0x9480, 0x9482, PDF_CMAP_TABLE, 379 },
	{ 0x9483, 0x9486, PDF_CMAP_RANGE, 13343 },
	{ 0x9487, 0x9488, PDF_CMAP_TABLE, 382 },
	{ 0x9489, 0x94a0, PDF_CMAP_RANGE, 13347 },
	{ 0x94a1, 0x94a1, PDF_CMAP_SINGLE, 7726 },
	{ 0x94a2, 0x94b2, PDF_CMAP_RANGE, 13371 },
	{ 0x94b3, 0x94b5, PDF_CMAP_TABLE, 384 },
	{ 0x94b6, 0x94be, PDF_CMAP_RANGE, 13389 },
	{ 0x94bf, 0x94c0, PDF_CMAP_TABLE, 387 },
	{ 0x94c1, 0x94cb, PDF_CMAP_RANGE, 13398 },
	{ 0x94cc, 0x94cc, PDF_CMAP_SINGLE, 9428 },
	{ 0x94cd, 0x94d7, PDF_CMAP_RANGE, 13409 },
	{ 0x94d8, 0x94d8, PDF_CMAP_SINGLE, 8815 },
	{ 0x94d9, 0x94df, PDF_CMAP_RANGE, 13420 },
	{ 0x94e0, 0x94e0, PDF_CMAP_SINGLE, 7895 },
	{ 0x94e1, 0x94fe, PDF_CMAP_RANGE, 13427 },
	{ 0x9530, 0x9530, PDF_CMAP_SINGLE, 1733 },
	{ 0x9530, 0x9539, PDF_CMAP_RANGE, 22890 },
	{ 0x9530, 0x9539, PDF_CMAP_RANGE, 24150 },
	{ 0x9530, 0x9539, PDF_CMAP_RANGE, 25410 },
	{ 0x9530, 0x9539, PDF_CMAP_RANGE, 26670 },
	{ 0x9530, 0x9539, PDF_CMAP_RANGE, 27930 },
	{ 0x9531, 0x9539, PDF_CMAP_TABLE, 389 },
	{ 0x9540, 0x9571, PDF_CMAP_RANGE, 13457 },
	{ 0x9572, 0x9572, PDF_CMAP_SINGLE, 8493 },
	{ 0x9573, 0x957e, PDF_CMAP_RANGE, 13507 },
	{ 0x9580, 0x9582, PDF_CMAP_RANGE, 13519 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 8859 },
	{ 0x9584, 0x959d, PDF_CMAP_RANGE, 13522 },
	{ 0x959e, 0x959f, PDF_CMAP_TABLE, 398 },
	{ 0x95a0, 0x95b2, PDF_CMAP_RANGE, 13548 },
	{ 0x95b3, 0x95b3, PDF_CMAP_SINGLE, 7801 },
	{ 0x95b4, 0x95b9, PDF_CMAP_RANGE, 13567 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 8794 },
	{ 0x95bb, 0x95ce, PDF_CMAP_RANGE, 13573 },
	{ 0x95cf, 0x95d4, PDF_CMAP_TABLE, 400 },
	{ 0x95d5, 0x95e0, PDF_CMAP_RANGE, 13595 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 9385 },
	{ 0x95e2, 0x95e6, PDF_CMAP_RANGE, 13607 },
	{ 0x95e7, 0x95e7, PDF_CMAP_SINGLE, 8167 },
	{ 0x95e8, 0x95f0, PDF_CMAP_RANGE, 13612 },
	{ 0x95f1, 0x95f1, PDF_CMAP_SINGLE, 8466 },
	{ 0x95f2, 0x95f7, PDF_CMAP_RANGE, 13621 },
	{ 0x95f8, 0x95f8, PDF_CMAP_SINGLE, 8508 },
	{ 0x95f9, 0x95fd, PDF_CMAP_RANGE, 13627 },
	{ 0x95fe, 0x95fe, PDF_CMAP_SINGLE, 8028 },
	{ 0x9630, 0x9630, PDF_CMAP_SINGLE, 2850 },
	{ 0x9630, 0x9639, PDF_CMAP_RANGE, 22900 },
	{ 0x9630, 0x9639, PDF_CMAP_RANGE, 24160 },
	{ 0x9630, 0x9639, PDF_CMAP_RANGE, 25420 },
	{ 0x9630, 0x9639, PDF_CMAP_RANGE, 26680 },
	{ 0x9630, 0x9639, PDF_CMAP_RANGE, 27940 },
	{ 0x9631, 0x9639, PDF_CMAP_TABLE, 406 },
	{ 0x9640, 0x9655, PDF_CMAP_RANGE, 13632 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 9412 },
	{ 0x9657, 0x967b, PDF_CMAP_RANGE, 13654 },
	{ 0x967c, 0x967c, PDF_CMAP_SINGLE, 7884 },
	{ 0x967d, 0x967e, PDF_CMAP_RANGE, 13691 },
	{ 0x9680, 0x96fe, PDF_CMAP_RANGE, 13693 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 2780 },
	{ 0x9730, 0x9739, PDF_CMAP_RANGE, 22910 },
	{ 0x9730, 0x9739, PDF_CMAP_RANGE, 24170 },
	{ 0x9730, 0x9739, PDF_CMAP_RANGE, 25430 },
	{ 0x9730, 0x9739, PDF_CMAP_RANGE, 26690 },
	{ 0x9730, 0x9739, PDF_CMAP_RANGE, 27950 },
	{ 0x9731, 0x9739, PDF_CMAP_TABLE, 415 },
	{ 0x9740, 0x9766, PDF_CMAP_RANGE, 13820 },
	{ 0x9767, 0x9767, PDF_CMAP_SINGLE, 9322 },
	{ 0x9768, 0x976b, PDF_CMAP_RANGE, 13859 },
	{ 0x976c, 0x976e, PDF_CMAP_TABLE, 424 },
	{ 0x976f, 0x977e, PDF_CMAP_RANGE, 13864 },
	{ 0x9780, 0x9795, PDF_CMAP_RANGE, 13880 },
	{ 0x9796, 0x9797, PDF_CMAP_TABLE, 427 },
	{ 0x9798, 0x979c, PDF_CMAP_RANGE, 13902 },
	{ 0x979d, 0x979d, PDF_CMAP_SINGLE, 7886 },
	{ 0x979e, 0x97a2, PDF_CMAP_RANGE, 13907 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 8818 },
	{ 0x97a4, 0x97be, PDF_CMAP_RANGE, 13912 },
	{ 0x97bf, 0x97bf, PDF_CMAP_SINGLE, 9331 },
	{ 0x97c0, 0x97ed, PDF_CMAP_RANGE, 13939 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 8702 },
	{ 0x97ef, 0x97f6, PDF_CMAP_RANGE, 13985 },
	{ 0x97f7, 0x97f7, PDF_CMAP_SINGLE, 7934 },
	{ 0x97f8, 0x97fe, PDF_CMAP_RANGE, 13993 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 6686 },
	{ 0x9830, 0x9839, PDF_CMAP_RANGE, 22920 },
	{ 0x9830, 0x9839, PDF_CMAP_RANGE, 24180 },
	{ 0x9830, 0x9839, PDF_CMAP_RANGE, 25440 },
	{ 0x9830, 0x9839, PDF_CMAP_RANGE, 26700 },
	{ 0x9830, 0x9839, PDF_CMAP_RANGE, 27960 },
	{ 0x9831, 0x9833, PDF_CMAP_TABLE, 429 },
	{ 0x9833, 0x9839, PDF_CMAP_RANGE, 29064 },
	{ 0x9834, 0x9839, PDF_CMAP_TABLE, 432 },
	{ 0x9840, 0x9844, PDF_CMAP_RANGE, 14000 },
	{ 0x9845, 0x9845, PDF_CMAP_SINGLE, 9333 },
	{ 0x9846, 0x9848, PDF_CMAP_RANGE, 14005 },
	{ 0x9849, 0x9849, PDF_CMAP_SINGLE, 8714 },
	{ 0x984a, 0x984e, PDF_CMAP_RANGE, 14008 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 8047 },
	{ 0x9850, 0x9870, PDF_CMAP_RANGE, 14013 },
	{ 0x9871, 0x9873, PDF_CMAP_TABLE, 438 },
	{ 0x9874, 0x987e, PDF_CMAP_RANGE, 14047 },
	{ 0x9880, 0x9881, PDF_CMAP_TABLE, 441 },
	{ 0x9882, 0x988a, PDF_CMAP_RANGE, 14059 },
	{ 0x988b, 0x988c, PDF_CMAP_TABLE, 443 },
	{ 0x988d, 0x989f, PDF_CMAP_RANGE, 14068 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 9340 },
	{ 0x98a1, 0x98a9, PDF_CMAP_RANGE, 14087 },
	{ 0x98aa, 0x98aa, PDF_CMAP_SINGLE, 8097 },
	{ 0x98ab, 0x98b5, PDF_CMAP_RANGE, 14096 },
	{ 0x98b6, 0x98b7, PDF_CMAP_TABLE, 445 },
	{ 0x98b8, 0x98b9, PDF_CMAP_RANGE, 14107 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 9324 },
	{ 0x98bb, 0x98c6, PDF_CMAP_RANGE, 14109 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 8254 },
	{ 0x98c8, 0x98ca, PDF_CMAP_RANGE, 14121 },
	{ 0x98cb, 0x98cb, PDF_CMAP_SINGLE, 7756 },
	{ 0x98cc, 0x98cf, PDF_CMAP_RANGE, 14124 },
	{ 0x98d0, 0x98d0, PDF_CMAP_SINGLE, 8506 },
	{ 0x98d1, 0x98d2, PDF_CMAP_RANGE, 14128 },
	{ 0x98d3, 0x98d3, PDF_CMAP_SINGLE, 8708 },
	{ 0x98d4, 0x98e2, PDF_CMAP_RANGE, 14130 },
	{ 0x98e3, 0x98e5, PDF_CMAP_TABLE, 447 },
	{ 0x98e6, 0x98ee, PDF_CMAP_RANGE, 14145 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 9332 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 14154 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 8413 },
	{ 0x98f3, 0x98fe, PDF_CMAP_RANGE, 14156 },
	{ 0x9930, 0x9930, PDF_CMAP_SINGLE, 3300 },
	{ 0x9930, 0x9939, PDF_CMAP_RANGE, 22930 },
	{ 0x9930, 0x9939, PDF_CMAP_RANGE, 24190 },
	{ 0x9930, 0x9939, PDF_CMAP_RANGE, 25450 },
	{ 0x9930, 0x9939, PDF_CMAP_RANGE, 26710 },
	{ 0x9930, 0x9939, PDF_CMAP_RANGE, 27970 },
	{ 0x9930, 0x9939, PDF_CMAP_RANGE, 29071 },
	{ 0x9931, 0x9939, PDF_CMAP_TABLE, 450 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 14168 },
	{ 0x9943, 0x9945, PDF_CMAP_TABLE, 459 },
	{ 0x9946, 0x9965, PDF_CMAP_RANGE, 14172 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 9330 },
	{ 0x9967, 0x996d, PDF_CMAP_RANGE, 14204 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 7861 },
	{ 0x996f, 0x9974, PDF_CMAP_RANGE, 14211 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 9336 },
	{ 0x9976, 0x9979, PDF_CMAP_RANGE, 14217 },
	{ 0x997a, 0x997b, PDF_CMAP_TABLE, 462 },
	{ 0x997c, 0x997e, PDF_CMAP_RANGE, 14221 },
	{ 0x9980, 0x9984, PDF_CMAP_RANGE, 14224 },
	{ 0x9985, 0x9985, PDF_CMAP_SINGLE, 9881 },
	{ 0x9986, 0x9988, PDF_CMAP_RANGE, 14229 },
	{ 0x9989, 0x9989, PDF_CMAP_SINGLE, 9346 },
	{ 0x998a, 0x998d, PDF_CMAP_RANGE, 14232 },
	{ 0x998e, 0x998e, PDF_CMAP_SINGLE, 8353 },
	{ 0x998f, 0x9990, PDF_CMAP_RANGE, 14236 },
	{ 0x9991, 0x9991, PDF_CMAP_SINGLE, 8082 },
	{ 0x9992, 0x9998, PDF_CMAP_RANGE, 14238 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 7992 },
	{ 0x999a, 0x99a8, PDF_CMAP_RANGE, 14245 },
	{ 0x99a9, 0x99a9, PDF_CMAP_SINGLE, 9349 },
	{ 0x99aa, 0x99af, PDF_CMAP_RANGE, 14260 },
	{ 0x99b0, 0x99b5, PDF_CMAP_TABLE, 464 },
	{ 0x99b6, 0x99bc, PDF_CMAP_RANGE, 14267 },
	{ 0x99bd, 0x99c2, PDF_CMAP_TABLE, 470 },
	{ 0x99c3, 0x99c8, PDF_CMAP_RANGE, 14276 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 9327 },
	{ 0x99ca, 0x99cd, PDF_CMAP_RANGE, 14282 },
	{ 0x99ce, 0x99ce, PDF_CMAP_SINGLE, 9345 },
	{ 0x99cf, 0x99d0, PDF_CMAP_RANGE, 14286 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 8736 },
	{ 0x99d2, 0x99d9, PDF_CMAP_RANGE, 14288 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 8181 },
	{ 0x99db, 0x99df, PDF_CMAP_RANGE, 14296 },
	{ 0x99e0, 0x99e0, PDF_CMAP_SINGLE, 8436 },
	{ 0x99e1, 0x99e4, PDF_CMAP_RANGE, 14301 },
	{ 0x99e5, 0x99e5, PDF_CMAP_SINGLE, 9341 },
	{ 0x99e6, 0x99e7, PDF_CMAP_RANGE, 14305 },
	{ 0x99e8, 0x99e8, PDF_CMAP_SINGLE, 9337 },
	{ 0x99e9, 0x99eb, PDF_CMAP_RANGE, 14307 },
	{ 0x99ec, 0x99ec, PDF_CMAP_SINGLE, 9342 },
	{ 0x99ed, 0x99f3, PDF_CMAP_RANGE, 14310 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 9338 },
	{ 0x99f5, 0x99fe, PDF_CMAP_RANGE, 14317 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 4093 },
	{ 0x9a30, 0x9a39, PDF_CMAP_RANGE, 22940 },
	{ 0x9a30, 0x9a39, PDF_CMAP_RANGE, 24200 },
	{ 0x9a30, 0x9a39, PDF_CMAP_RANGE, 25460 },
	{ 0x9a30, 0x9a39, PDF_CMAP_RANGE, 26720 },
	{ 0x9a30, 0x9a39, PDF_CMAP_RANGE, 27980 },
	{ 0x9a30, 0x9a39, PDF_CMAP_RANGE, 29081 },
	{ 0x9a31, 0x9a39, PDF_CMAP_TABLE, 476 },
	{ 0x9a40, 0x9a49, PDF_CMAP_RANGE, 14327 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 8419 },
	{ 0x9a4b, 0x9a56, PDF_CMAP_RANGE, 14337 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 8365 },
	{ 0x9a58, 0x9a64, PDF_CMAP_RANGE, 14349 },
	{ 0x9a65, 0x9a67, PDF_CMAP_TABLE, 485 },
	{ 0x9a68, 0x9a70, PDF_CMAP_RANGE, 14363 },
	{ 0x9a71, 0x9a71, PDF_CMAP_SINGLE, 8537 },
	{ 0x9a72, 0x9a75, PDF_CMAP_RANGE, 14372 },
	{ 0x9a76, 0x9a77, PDF_CMAP_TABLE, 488 },
	{ 0x9a78, 0x9a7e, PDF_CMAP_RANGE, 14376 },
	{ 0x9a80, 0x9a87, PDF_CMAP_RANGE, 14383 },
	{ 0x9a88, 0x9a88, PDF_CMAP_SINGLE, 7772 },
	{ 0x9a89, 0x9a8b, PDF_CMAP_RANGE, 14391 },
	{ 0x9a8c, 0x9a8c, PDF_CMAP_SINGLE, 9352 },
	{ 0x9a8d, 0x9a90, PDF_CMAP_RANGE, 14394 },
	{ 0x9a91, 0x9a91, PDF_CMAP_SINGLE, 9351 },
	{ 0x9a92, 0x9a96, PDF_CMAP_RANGE, 14398 },
	{ 0x9a97, 0x9a97, PDF_CMAP_SINGLE, 9354 },
	{ 0x9a98, 0x9a99, PDF_CMAP_RANGE, 14403 },
	{ 0x9a9a, 0x9a9b, PDF_CMAP_TABLE, 490 },
	{ 0x9a9c, 0x9a9d, PDF_CMAP_RANGE, 14405 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 8067 },
	{ 0x9a9f, 0x9aa1, PDF_CMAP_RANGE, 14407 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_TABLE, 492 },
	{ 0x9aa4, 0x9aa9, PDF_CMAP_RANGE, 14410 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 8367 },
	{ 0x9aab, 0x9acf, PDF_CMAP_RANGE, 14416 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 9408 },
	{ 0x9ad1, 0x9ad5, PDF_CMAP_RANGE, 14453 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 8813 },
	{ 0x9ad7, 0x9ad9, PDF_CMAP_RANGE, 14458 },
	{ 0x9ada, 0x9ada, PDF_CMAP_SINGLE, 9409 },
	{ 0x9adb, 0x9ae1, PDF_CMAP_RANGE, 14461 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_TABLE, 494 },
	{ 0x9ae6, 0x9afe, PDF_CMAP_RANGE, 14469 },
	{ 0x9b30, 0x9b30, PDF_CMAP_SINGLE, 7803 },
	{ 0x9b30, 0x9b39, PDF_CMAP_RANGE, 22950 },
	{ 0x9b30, 0x9b39, PDF_CMAP_RANGE, 24210 },
	{ 0x9b30, 0x9b39, PDF_CMAP_RANGE, 25470 },
	{ 0x9b30, 0x9b39, PDF_CMAP_RANGE, 26730 },
	{ 0x9b30, 0x9b39, PDF_CMAP_RANGE, 27990 },
	{ 0x9b30, 0x9b39, PDF_CMAP_RANGE, 29091 },
	{ 0x9b31, 0x9b39, PDF_CMAP_TABLE, 498 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 14494 },
	{ 0x9b80, 0x9bd0, PDF_CMAP_RANGE, 14557 },
	{ 0x9bd1, 0x9bd1, PDF_CMAP_SINGLE, 9180 },
	{ 0x9bd2, 0x9bdb, PDF_CMAP_RANGE, 14638 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 9179 },
	{ 0x9bdd, 0x9bfe, PDF_CMAP_RANGE, 14648 },
	{ 0x9c30, 0x9c30, PDF_CMAP_SINGLE, 8317 },
	{ 0x9c30, 0x9c39, PDF_CMAP_RANGE, 22960 },
	{ 0x9c30, 0x9c39, PDF_CMAP_RANGE, 24220 },
	{ 0x9c30, 0x9c39, PDF_CMAP_RANGE, 25480 },
	{ 0x9c30, 0x9c39, PDF_CMAP_RANGE, 26740 },
	{ 0x9c30, 0x9c39, PDF_CMAP_RANGE, 28000 },
	{ 0x9c30, 0x9c39, PDF_CMAP_RANGE, 29101 },
	{ 0x9c31, 0x9c39, PDF_CMAP_TABLE, 507 },
	{ 0x9c40, 0x9c52, PDF_CMAP_RANGE, 14682 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 8286 },
	{ 0x9c54, 0x9c58, PDF_CMAP_RANGE, 14701 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_TABLE, 516 },
	{ 0x9c5d, 0x9c74, PDF_CMAP_RANGE, 14707 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 8607 },
	{ 0x9c76, 0x9c78, PDF_CMAP_RANGE, 14731 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 7782 },
	{ 0x9c7a, 0x9c7e, PDF_CMAP_RANGE, 14734 },
	{ 0x9c80, 0x9c85, PDF_CMAP_RANGE, 14739 },
	{ 0x9c86, 0x9c86, PDF_CMAP_SINGLE, 8035 },
	{ 0x9c87, 0x9c9c, PDF_CMAP_RANGE, 14745 },
	{ 0x9c9d, 0x9c9d, PDF_CMAP_SINGLE, 9181 },
	{ 0x9c9e, 0x9caa, PDF_CMAP_RANGE, 14767 },
	{ 0x9cab, 0x9cab, PDF_CMAP_SINGLE, 8555 },
	{ 0x9cac, 0x9cc9, PDF_CMAP_RANGE, 14780 },
	{ 0x9cca, 0x9cca, PDF_CMAP_SINGLE, 8885 },
	{ 0x9ccb, 0x9cce, PDF_CMAP_RANGE, 14810 },
	{ 0x9ccf, 0x9ccf, PDF_CMAP_SINGLE, 7974 },
	{ 0x9cd0, 0x9ce5, PDF_CMAP_RANGE, 14814 },
	{ 0x9ce6, 0x9ce7, PDF_CMAP_TABLE, 520 },
	{ 0x9ce8, 0x9ceb, PDF_CMAP_RANGE, 14836 },
	{ 0x9cec, 0x9cee, PDF_CMAP_TABLE, 522 },
	{ 0x9cef, 0x9cfa, PDF_CMAP_RANGE, 14841 },
	{ 0x9cfb, 0x9cfb, PDF_CMAP_SINGLE, 8011 },
	{ 0x9cfc, 0x9cfd, PDF_CMAP_RANGE, 14853 },
	{ 0x9cfe, 0x9cfe, PDF_CMAP_SINGLE, 8850 },
	{ 0x9d30, 0x9d30, PDF_CMAP_SINGLE, 2289 },
	{ 0x9d30, 0x9d39, PDF_CMAP_RANGE, 22970 },
	{ 0x9d30, 0x9d39, PDF_CMAP_RANGE, 24230 },
	{ 0x9d30, 0x9d39, PDF_CMAP_RANGE, 25490 },
	{ 0x9d30, 0x9d39, PDF_CMAP_RANGE, 26750 },
	{ 0x9d30, 0x9d39, PDF_CMAP_RANGE, 28010 },
	{ 0x9d30, 0x9d39, PDF_CMAP_RANGE, 29111 },
	{ 0x9d31, 0x9d39, PDF_CMAP_TABLE, 525 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 14855 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 8484 },
	{ 0x9d43, 0x9d45, PDF_CMAP_RANGE, 14857 },
	{ 0x9d46, 0x9d47, PDF_CMAP_TABLE, 534 },
	{ 0x9d48, 0x9d4c, PDF_CMAP_RANGE, 14860 },
	{ 0x9d4d, 0x9d4f, PDF_CMAP_TABLE, 536 },
	{ 0x9d50, 0x9d60, PDF_CMAP_RANGE, 14866 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 8369 },
	{ 0x9d62, 0x9d67, PDF_CMAP_RANGE, 14883 },
	{ 0x9d68, 0x9d69, PDF_CMAP_TABLE, 539 },
	{ 0x9d6a, 0x9d6d, PDF_CMAP_RANGE, 14889 },
	{ 0x9d6e, 0x9d6e, PDF_CMAP_SINGLE, 8888 },
	{ 0x9d6f, 0x9d70, PDF_CMAP_RANGE, 14893 },
	{ 0x9d71, 0x9d71, PDF_CMAP_SINGLE, 8822 },
	{ 0x9d72, 0x9d74, PDF_CMAP_RANGE, 14895 },
	{ 0x9d75, 0x9d75, PDF_CMAP_SINGLE, 8091 },
	{ 0x9d76, 0x9d7a, PDF_CMAP_RANGE, 14898 },
	{ 0x9d7b, 0x9d7e, PDF_CMAP_TABLE, 541 },
	{ 0x9d80, 0x9d89, PDF_CMAP_RANGE, 14905 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 8382 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 14915 },
	{ 0x9d8d, 0x9d8d, PDF_CMAP_SINGLE, 8116 },
	{ 0x9d8e, 0x9d90, PDF_CMAP_RANGE, 14917 },
	{ 0x9d91, 0x9d91, PDF_CMAP_SINGLE, 9175 },
	{ 0x9d92, 0x9d98, PDF_CMAP_RANGE, 14920 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 8453 },
	{ 0x9d9a, 0x9da0, PDF_CMAP_RANGE, 14927 },
	{ 0x9da1, 0x9da2, PDF_CMAP_TABLE, 545 },
	{ 0x9da3, 0x9da6, PDF_CMAP_RANGE, 14934 },
	{ 0x9da7, 0x9da7, PDF_CMAP_SINGLE, 9194 },
	{ 0x9da8, 0x9dab, PDF_CMAP_RANGE, 14938 },
	{ 0x9dac, 0x9dad, PDF_CMAP_TABLE, 547 },
	{ 0x9dae, 0x9db1, PDF_CMAP_RANGE, 14942 },
	{ 0x9db2, 0x9db3, PDF_CMAP_TABLE, 549 },
	{ 0x9db4, 0x9dbd, PDF_CMAP_RANGE, 14946 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 8093 },
	{ 0x9dbf, 0x9dc5, PDF_CMAP_RANGE, 14956 },
	{ 0x9dc6, 0x9dc6, PDF_CMAP_SINGLE, 9189 },
	{ 0x9dc7, 0x9dc8, PDF_CMAP_RANGE, 14963 },
	{ 0x9dc9, 0x9dc9, PDF_CMAP_SINGLE, 8804 },
	{ 0x9dca, 0x9dcc, PDF_CMAP_RANGE, 14965 },
	{ 0x9dcd, 0x9dcd, PDF_CMAP_SINGLE, 9442 },
	{ 0x9dce, 0x9dd1, PDF_CMAP_RANGE, 14968 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 9182 },
	{ 0x9dd3, 0x9dd4, PDF_CMAP_RANGE, 14972 },
	{ 0x9dd5, 0x9dd5, PDF_CMAP_SINGLE, 7876 },
	{ 0x9dd6, 0x9de0, PDF_CMAP_RANGE, 14974 },
	{ 0x9de1, 0x9de2, PDF_CMAP_TABLE, 551 },
	{ 0x9de3, 0x9df0, PDF_CMAP_RANGE, 14985 },
	{ 0x9df1, 0x9df1, PDF_CMAP_SINGLE, 8491 },
	{ 0x9df2, 0x9df3, PDF_CMAP_RANGE, 14999 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 8357 },
	{ 0x9df5, 0x9df6, PDF_CMAP_RANGE, 15001 },
	{ 0x9df7, 0x9df7, PDF_CMAP_SINGLE, 9867 },
	{ 0x9df8, 0x9df9, PDF_CMAP_RANGE, 15003 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 8054 },
	{ 0x9dfb, 0x9dfc, PDF_CMAP_RANGE, 15005 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_TABLE, 553 },
	{ 0x9e30, 0x9e30, PDF_CMAP_SINGLE, 1754 },
	{ 0x9e30, 0x9e39, PDF_CMAP_RANGE, 22980 },
	{ 0x9e30, 0x9e39, PDF_CMAP_RANGE, 24240 },
	{ 0x9e30, 0x9e39, PDF_CMAP_RANGE, 25500 },
	{ 0x9e30, 0x9e39, PDF_CMAP_RANGE, 26760 },
	{ 0x9e30, 0x9e39, PDF_CMAP_RANGE, 28020 },
	{ 0x9e30, 0x9e39, PDF_CMAP_RANGE, 29121 },
	{ 0x9e31, 0x9e39, PDF_CMAP_TABLE, 555 },
	{ 0x9e40, 0x9e44, PDF_CMAP_RANGE, 15008 },
	{ 0x9e45, 0x9e45, PDF_CMAP_SINGLE, 8193 },
	{ 0x9e46, 0x9e47, PDF_CMAP_RANGE, 15013 },
	{ 0x9e48, 0x9e49, PDF_CMAP_TABLE, 564 },
	{ 0x9e4a, 0x9e51, PDF_CMAP_RANGE, 15015 },
	{ 0x9e52, 0x9e56, PDF_CMAP_TABLE, 566 },
	{ 0x9e57, 0x9e5c, PDF_CMAP_RANGE, 15025 },
	{ 0x9e5d, 0x9e5e, PDF_CMAP_TABLE, 571 },
	{ 0x9e5f, 0x9e60, PDF_CMAP_RANGE, 15031 },
	{ 0x9e61, 0x9e63, PDF_CMAP_TABLE, 573 },
	{ 0x9e64, 0x9e66, PDF_CMAP_RANGE, 15034 },
	{ 0x9e67, 0x9e67, PDF_CMAP_SINGLE, 9183 },
	{ 0x9e68, 0x9e6b, PDF_CMAP_RANGE, 15037 },
	{ 0x9e6c, 0x9e6c, PDF_CMAP_SINGLE, 7759 },
	{ 0x9e6d, 0x9e6e, PDF_CMAP_RANGE, 15041 },
	{ 0x9e6f, 0x9e6f, PDF_CMAP_SINGLE, 9177 },
	{ 0x9e70, 0x9e71, PDF_CMAP_RANGE, 15043 },
	{ 0x9e72, 0x9e75, PDF_CMAP_TABLE, 576 },
	{ 0x9e76, 0x9e7a, PDF_CMAP_RANGE, 15046 },
	{ 0x9e7b, 0x9e7c, PDF_CMAP_TABLE, 580 },
	{ 0x9e7d, 0x9e7e, PDF_CMAP_RANGE, 15051 },
	{ 0x9e80, 0x9e84, PDF_CMAP_RANGE, 15053 },
	{ 0x9e85, 0x9e87, PDF_CMAP_TABLE, 582 },
	{ 0x9e88, 0x9e90, PDF_CMAP_RANGE, 15059 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 8186 },
	{ 0x9e92, 0x9e95, PDF_CMAP_RANGE, 15068 },
	{ 0x9e96, 0x9e97, PDF_CMAP_TABLE, 585 },
	{ 0x9e98, 0x9ea1, PDF_CMAP_RANGE, 15072 },
	{ 0x9ea2, 0x9ea2, PDF_CMAP_SINGLE, 8454 },
	{ 0x9ea3, 0x9ea5, PDF_CMAP_RANGE, 15082 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 8203 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 15085 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 8550 },
	{ 0x9eaa, 0x9ead, PDF_CMAP_RANGE, 15087 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 9199 },
	{ 0x9eaf, 0x9eb2, PDF_CMAP_RANGE, 15091 },
	{ 0x9eb3, 0x9eb4, PDF_CMAP_TABLE, 587 },
	{ 0x9eb5, 0x9eb6, PDF_CMAP_RANGE, 15095 },
	{ 0x9eb7, 0x9eb7, PDF_CMAP_SINGLE, 9191 },
	{ 0x9eb8, 0x9ef4, PDF_CMAP_RANGE, 15097 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 8611 },
	{ 0x9ef6, 0x9efe, PDF_CMAP_RANGE, 15158 },
	{ 0x9f30, 0x9f30, PDF_CMAP_SINGLE, 8305 },
	{ 0x9f30, 0x9f39, PDF_CMAP_RANGE, 22990 },
	{ 0x9f30, 0x9f39, PDF_CMAP_RANGE, 24250 },
	{ 0x9f30, 0x9f39, PDF_CMAP_RANGE, 25510 },
	{ 0x9f30, 0x9f39, PDF_CMAP_RANGE, 26770 },
	{ 0x9f30, 0x9f39, PDF_CMAP_RANGE, 28030 },
	{ 0x9f30, 0x9f39, PDF_CMAP_RANGE, 29131 },
	{ 0x9f31, 0x9f39, PDF_CMAP_TABLE, 589 },
	{ 0x9f40, 0x9f4d, PDF_CMAP_RANGE, 15167 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 8570 },
	{ 0x9f4f, 0x9f6e, PDF_CMAP_RANGE, 15181 },
	{ 0x9f6f, 0x9f6f, PDF_CMAP_SINGLE, 8613 },
	{ 0x9f70, 0x9f7e, PDF_CMAP_RANGE, 15213 },
	{ 0x9f80, 0x9f91, PDF_CMAP_RANGE, 15228 },
	{ 0x9f92, 0x9f92, PDF_CMAP_SINGLE, 8226 },
	{ 0x9f93, 0x9f97, PDF_CMAP_RANGE, 15246 },
	{ 0x9f98, 0x9f98, PDF_CMAP_SINGLE, 9430 },
	{ 0x9f99, 0x9fa5, PDF_CMAP_RANGE, 15251 },
	{ 0x9fa6, 0x9fa6, PDF_CMAP_SINGLE, 9018 },
	{ 0x9fa7, 0x9fa8, PDF_CMAP_RANGE, 15264 },
	{ 0x9fa9, 0x9fa9, PDF_CMAP_SINGLE, 7918 },
	{ 0x9faa, 0x9fab, PDF_CMAP_RANGE, 15266 },
	{ 0x9fac, 0x9fac, PDF_CMAP_SINGLE, 9429 },
	{ 0x9fad, 0x9fc8, PDF_CMAP_RANGE, 15268 },
	{ 0x9fc9, 0x9fc9, PDF_CMAP_SINGLE, 8744 },
	{ 0x9fca, 0x9fcc, PDF_CMAP_RANGE, 15296 },
	{ 0x9fcd, 0x9fcd, PDF_CMAP_SINGLE, 9431 },
	{ 0x9fce, 0x9fe0, PDF_CMAP_RANGE, 15299 },
	{ 0x9fe1, 0x9fe1, PDF_CMAP_SINGLE, 8444 },
	{ 0x9fe2, 0x9fea, PDF_CMAP_RANGE, 15318 },
	{ 0x9feb, 0x9feb, PDF_CMAP_SINGLE, 7815 },
	{ 0x9fec, 0x9fed, PDF_CMAP_RANGE, 15327 },
	{ 0x9fee, 0x9fee, PDF_CMAP_SINGLE, 9432 },
	{ 0x9fef, 0x9ff3, PDF_CMAP_RANGE, 15329 },
	{ 0x9ff4, 0x9ff4, PDF_CMAP_SINGLE, 7866 },
	{ 0x9ff5, 0x9ffc, PDF_CMAP_RANGE, 15334 },
	{ 0x9ffd, 0x9ffe, PDF_CMAP_TABLE, 598 },
	{ 0xa030, 0xa030, PDF_CMAP_SINGLE, 1072 },
	{ 0xa030, 0xa039, PDF_CMAP_RANGE, 23000 },
	{ 0xa030, 0xa039, PDF_CMAP_RANGE, 24260 },
	{ 0xa030, 0xa039, PDF_CMAP_RANGE, 25520 },
	{ 0xa030, 0xa039, PDF_CMAP_RANGE, 26780 },
	{ 0xa030, 0xa039, PDF_CMAP_RANGE, 28040 },
	{ 0xa030, 0xa039, PDF_CMAP_RANGE, 29141 },
	{ 0xa031, 0xa035, PDF_CMAP_TABLE, 600 },
	{ 0xa040, 0xa042, PDF_CMAP_RANGE, 15343 },
	{ 0xa043, 0xa043, PDF_CMAP_SINGLE, 8556 },
	{ 0xa044, 0xa045, PDF_CMAP_RANGE, 15346 },
	{ 0xa046, 0xa046, PDF_CMAP_SINGLE, 9433 },
	{ 0xa047, 0xa048, PDF_CMAP_RANGE, 15348 },
	{ 0xa049, 0xa049, PDF_CMAP_SINGLE, 8743 },
	{ 0xa04a, 0xa04d, PDF_CMAP_RANGE, 15350 },
	{ 0xa04e, 0xa04e, PDF_CMAP_SINGLE, 7775 },
	{ 0xa04f, 0xa053, PDF_CMAP_RANGE, 15354 },
	{ 0xa054, 0xa054, PDF_CMAP_SINGLE, 8863 },
	{ 0xa055, 0xa059, PDF_CMAP_RANGE, 15359 },
	{ 0xa05a, 0xa05a, PDF_CMAP_SINGLE, 8029 },
	{ 0xa05b, 0xa060, PDF_CMAP_RANGE, 15364 },
	{ 0xa061, 0xa063, PDF_CMAP_TABLE, 605 },
	{ 0xa064, 0xa070, PDF_CMAP_RANGE, 15371 },
	{ 0xa071, 0xa071, PDF_CMAP_SINGLE, 8521 },
	{ 0xa072, 0xa073, PDF_CMAP_RANGE, 15384 },
	{ 0xa074, 0xa074, PDF_CMAP_SINGLE, 8262 },
	{ 0xa075, 0xa07e, PDF_CMAP_RANGE, 15386 },
	{ 0xa080, 0xa080, PDF_CMAP_SINGLE, 8192 },
	{ 0xa081, 0xa090, PDF_CMAP_RANGE, 15396 },
	{ 0xa091, 0xa091, PDF_CMAP_SINGLE, 8592 },
	{ 0xa092, 0xa093, PDF_CMAP_RANGE, 15412 },
	{ 0xa094, 0xa096, PDF_CMAP_TABLE, 608 },
	{ 0xa097, 0xa0a8, PDF_CMAP_RANGE, 15415 },
	{ 0xa0a9, 0xa0a9, PDF_CMAP_SINGLE, 9411 },
	{ 0xa0aa, 0xa0be, PDF_CMAP_RANGE, 15433 },
	{ 0xa0bf, 0xa0bf, PDF_CMAP_SINGLE, 8396 },
	{ 0xa0c0, 0xa0cd, PDF_CMAP_RANGE, 15454 },
	{ 0xa0ce, 0xa0ce, PDF_CMAP_SINGLE, 9024 },
	{ 0xa0cf, 0xa0d8, PDF_CMAP_RANGE, 15468 },
	{ 0xa0d9, 0xa0d9, PDF_CMAP_SINGLE, 7889 },
	{ 0xa0da, 0xa0dd, PDF_CMAP_RANGE, 15478 },
	{ 0xa0de, 0xa0de, PDF_CMAP_SINGLE, 8620 },
	{ 0xa0df, 0xa0ed, PDF_CMAP_RANGE, 15482 },
	{ 0xa0ee, 0xa0ee, PDF_CMAP_SINGLE, 8879 },
	{ 0xa0ef, 0xa0fe, PDF_CMAP_RANGE, 15497 },
	{ 0xa130, 0xa139, PDF_CMAP_RANGE, 23010 },
	{ 0xa130, 0xa139, PDF_CMAP_RANGE, 24270 },
	{ 0xa130, 0xa139, PDF_CMAP_RANGE, 25530 },
	{ 0xa130, 0xa139, PDF_CMAP_RANGE, 26790 },
	{ 0xa130, 0xa139, PDF_CMAP_RANGE, 28050 },
	{ 0xa130, 0xa139, PDF_CMAP_RANGE, 29151 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa230, 0xa239, PDF_CMAP_RANGE, 23020 },
	{ 0xa230, 0xa239, PDF_CMAP_RANGE, 24280 },
	{ 0xa230, 0xa239, PDF_CMAP_RANGE, 25540 },
	{ 0xa230, 0xa239, PDF_CMAP_RANGE, 26800 },
	{ 0xa230, 0xa239, PDF_CMAP_RANGE, 28060 },
	{ 0xa230, 0xa239, PDF_CMAP_RANGE, 29161 },
	{ 0xa2a1, 0xa2aa, PDF_CMAP_RANGE, 9897 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e3, 0xa2e3, PDF_CMAP_SINGLE, 22353 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa330, 0xa339, PDF_CMAP_RANGE, 23030 },
	{ 0xa330, 0xa339, PDF_CMAP_RANGE, 24290 },
	{ 0xa330, 0xa339, PDF_CMAP_RANGE, 25550 },
	{ 0xa330, 0xa339, PDF_CMAP_RANGE, 26810 },
	{ 0xa330, 0xa339, PDF_CMAP_RANGE, 28070 },
	{ 0xa330, 0xa339, PDF_CMAP_RANGE, 29171 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa430, 0xa439, PDF_CMAP_RANGE, 23040 },
	{ 0xa430, 0xa439, PDF_CMAP_RANGE, 24300 },
	{ 0xa430, 0xa439, PDF_CMAP_RANGE, 25560 },
	{ 0xa430, 0xa439, PDF_CMAP_RANGE, 26820 },
	{ 0xa430, 0xa439, PDF_CMAP_RANGE, 28080 },
	{ 0xa430, 0xa439, PDF_CMAP_RANGE, 29181 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa530, 0xa539, PDF_CMAP_RANGE, 23050 },
	{ 0xa530, 0xa539, PDF_CMAP_RANGE, 24310 },
	{ 0xa530, 0xa539, PDF_CMAP_RANGE, 25570 },
	{ 0xa530, 0xa539, PDF_CMAP_RANGE, 26830 },
	{ 0xa530, 0xa539, PDF_CMAP_RANGE, 28090 },
	{ 0xa530, 0xa539, PDF_CMAP_RANGE, 29191 },
	{ 0xa533, 0xa535, PDF_CMAP_RANGE, 22395 },
	{ 0xa538, 0xa539, PDF_CMAP_RANGE, 22398 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa630, 0xa630, PDF_CMAP_SINGLE, 22400 },
	{ 0xa630, 0xa639, PDF_CMAP_RANGE, 23060 },
	{ 0xa630, 0xa639, PDF_CMAP_RANGE, 24320 },
	{ 0xa630, 0xa639, PDF_CMAP_RANGE, 25580 },
	{ 0xa630, 0xa639, PDF_CMAP_RANGE, 26840 },
	{ 0xa630, 0xa639, PDF_CMAP_RANGE, 28100 },
	{ 0xa630, 0xa639, PDF_CMAP_RANGE, 29201 },
	{ 0xa634, 0xa634, PDF_CMAP_SINGLE, 22357 },
	{ 0xa636, 0xa636, PDF_CMAP_SINGLE, 22375 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa730, 0xa739, PDF_CMAP_RANGE, 23070 },
	{ 0xa730, 0xa739, PDF_CMAP_RANGE, 24330 },
	{ 0xa730, 0xa739, PDF_CMAP_RANGE, 25590 },
	{ 0xa730, 0xa739, PDF_CMAP_RANGE, 26850 },
	{ 0xa730, 0xa739, PDF_CMAP_RANGE, 28110 },
	{ 0xa730, 0xa739, PDF_CMAP_RANGE, 29211 },
	{ 0xa735, 0xa738, PDF_CMAP_TABLE, 611 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa830, 0xa839, PDF_CMAP_RANGE, 23080 },
	{ 0xa830, 0xa839, PDF_CMAP_RANGE, 24340 },
	{ 0xa830, 0xa839, PDF_CMAP_RANGE, 25600 },
	{ 0xa830, 0xa839, PDF_CMAP_RANGE, 26860 },
	{ 0xa830, 0xa839, PDF_CMAP_RANGE, 28120 },
	{ 0xa830, 0xa839, PDF_CMAP_RANGE, 29221 },
	{ 0xa836, 0xa838, PDF_CMAP_RANGE, 22401 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 9907 },
	{ 0xa880, 0xa895, PDF_CMAP_RANGE, 9970 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa930, 0xa939, PDF_CMAP_RANGE, 23090 },
	{ 0xa930, 0xa939, PDF_CMAP_RANGE, 24350 },
	{ 0xa930, 0xa939, PDF_CMAP_RANGE, 25610 },
	{ 0xa930, 0xa939, PDF_CMAP_RANGE, 26870 },
	{ 0xa930, 0xa939, PDF_CMAP_RANGE, 28130 },
	{ 0xa930, 0xa939, PDF_CMAP_RANGE, 29231 },
	{ 0xa940, 0xa957, PDF_CMAP_RANGE, 9992 },
	{ 0xa959, 0xa95a, PDF_CMAP_RANGE, 10016 },
	{ 0xa95c, 0xa95c, PDF_CMAP_SINGLE, 10018 },
	{ 0xa960, 0xa97e, PDF_CMAP_RANGE, 10019 },
	{ 0xa980, 0xa995, PDF_CMAP_RANGE, 10050 },
	{ 0xa996, 0xa996, PDF_CMAP_SINGLE, 7703 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaa30, 0xaa39, PDF_CMAP_RANGE, 23100 },
	{ 0xaa30, 0xaa39, PDF_CMAP_RANGE, 24360 },
	{ 0xaa30, 0xaa39, PDF_CMAP_RANGE, 25620 },
	{ 0xaa30, 0xaa39, PDF_CMAP_RANGE, 26880 },
	{ 0xaa30, 0xaa39, PDF_CMAP_RANGE, 28140 },
	{ 0xaa30, 0xaa39, PDF_CMAP_RANGE, 29241 },
	{ 0xaa40, 0xaa4c, PDF_CMAP_RANGE, 15513 },
	{ 0xaa4d, 0xaa4e, PDF_CMAP_TABLE, 615 },
	{ 0xaa4f, 0xaa70, PDF_CMAP_RANGE, 15526 },
	{ 0xaa71, 0xaa73, PDF_CMAP_TABLE, 617 },
	{ 0xaa74, 0xaa76, PDF_CMAP_RANGE, 15561 },
	{ 0xaa77, 0xaa77, PDF_CMAP_SINGLE, 9109 },
	{ 0xaa78, 0xaa79, PDF_CMAP_RANGE, 15564 },
	{ 0xaa7a, 0xaa7b, PDF_CMAP_TABLE, 620 },
	{ 0xaa7c, 0xaa7e, PDF_CMAP_RANGE, 15566 },
	{ 0xaa80, 0xaa99, PDF_CMAP_RANGE, 15569 },
	{ 0xaa9a, 0xaaa0, PDF_CMAP_TABLE, 622 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xab30, 0xab39, PDF_CMAP_RANGE, 23110 },
	{ 0xab30, 0xab39, PDF_CMAP_RANGE, 24370 },
	{ 0xab30, 0xab39, PDF_CMAP_RANGE, 25630 },
	{ 0xab30, 0xab39, PDF_CMAP_RANGE, 26890 },
	{ 0xab30, 0xab39, PDF_CMAP_RANGE, 28150 },
	{ 0xab30, 0xab39, PDF_CMAP_RANGE, 29251 },
	{ 0xab40, 0xab40, PDF_CMAP_SINGLE, 8036 },
	{ 0xab41, 0xab42, PDF_CMAP_RANGE, 15598 },
	{ 0xab43, 0xab4a, PDF_CMAP_TABLE, 629 },
	{ 0xab4b, 0xab4c, PDF_CMAP_RANGE, 15602 },
	{ 0xab4d, 0xab4d, PDF_CMAP_SINGLE, 9113 },
	{ 0xab4e, 0xab7e, PDF_CMAP_RANGE, 15604 },
	{ 0xab80, 0xaba0, PDF_CMAP_RANGE, 15653 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xac30, 0xac39, PDF_CMAP_RANGE, 23120 },
	{ 0xac30, 0xac39, PDF_CMAP_RANGE, 24380 },
	{ 0xac30, 0xac39, PDF_CMAP_RANGE, 25640 },
	{ 0xac30, 0xac39, PDF_CMAP_RANGE, 26900 },
	{ 0xac30, 0xac39, PDF_CMAP_RANGE, 28160 },
	{ 0xac30, 0xac39, PDF_CMAP_RANGE, 29261 },
	{ 0xac40, 0xac45, PDF_CMAP_RANGE, 15686 },
	{ 0xac46, 0xac46, PDF_CMAP_SINGLE, 8640 },
	{ 0xac47, 0xac70, PDF_CMAP_RANGE, 15692 },
	{ 0xac71, 0xac71, PDF_CMAP_SINGLE, 9312 },
	{ 0xac72, 0xac7b, PDF_CMAP_RANGE, 15734 },
	{ 0xac7c, 0xac7c, PDF_CMAP_SINGLE, 9308 },
	{ 0xac7d, 0xac7e, PDF_CMAP_RANGE, 15744 },
	{ 0xac80, 0xac8c, PDF_CMAP_RANGE, 15746 },
	{ 0xac8d, 0xac8d, PDF_CMAP_SINGLE, 8541 },
	{ 0xac8e, 0xac92, PDF_CMAP_RANGE, 15759 },
	{ 0xac93, 0xac94, PDF_CMAP_TABLE, 637 },
	{ 0xac95, 0xaca0, PDF_CMAP_RANGE, 15764 },
	{ 0xad30, 0xad39, PDF_CMAP_RANGE, 23130 },
	{ 0xad30, 0xad39, PDF_CMAP_RANGE, 24390 },
	{ 0xad30, 0xad39, PDF_CMAP_RANGE, 25650 },
	{ 0xad30, 0xad39, PDF_CMAP_RANGE, 26910 },
	{ 0xad30, 0xad39, PDF_CMAP_RANGE, 28170 },
	{ 0xad30, 0xad39, PDF_CMAP_RANGE, 29271 },
	{ 0xad40, 0xad48, PDF_CMAP_RANGE, 15776 },
	{ 0xad49, 0xad49, PDF_CMAP_SINGLE, 9313 },
	{ 0xad4a, 0xad5d, PDF_CMAP_RANGE, 15785 },
	{ 0xad5e, 0xad5e, PDF_CMAP_SINGLE, 9307 },
	{ 0xad5f, 0xad60, PDF_CMAP_RANGE, 15805 },
	{ 0xad61, 0xad61, PDF_CMAP_SINGLE, 9314 },
	{ 0xad62, 0xad67, PDF_CMAP_RANGE, 15807 },
	{ 0xad68, 0xad68, PDF_CMAP_SINGLE, 8020 },
	{ 0xad69, 0xad73, PDF_CMAP_RANGE, 15813 },
	{ 0xad74, 0xad74, PDF_CMAP_SINGLE, 9311 },
	{ 0xad75, 0xad7e, PDF_CMAP_RANGE, 15824 },
	{ 0xad80, 0xad81, PDF_CMAP_RANGE, 15834 },
	{ 0xad82, 0xad82, PDF_CMAP_SINGLE, 8428 },
	{ 0xad83, 0xad86, PDF_CMAP_RANGE, 15836 },
	{ 0xad87, 0xad87, PDF_CMAP_SINGLE, 9309 },
	{ 0xad88, 0xad8a, PDF_CMAP_RANGE, 15840 },
	{ 0xad8b, 0xad8b, PDF_CMAP_SINGLE, 9315 },
	{ 0xad8c, 0xad90, PDF_CMAP_RANGE, 15843 },
	{ 0xad91, 0xad91, PDF_CMAP_SINGLE, 9316 },
	{ 0xad92, 0xada0, PDF_CMAP_RANGE, 15848 },
	{ 0xae30, 0xae39, PDF_CMAP_RANGE, 23140 },
	{ 0xae30, 0xae39, PDF_CMAP_RANGE, 24400 },
	{ 0xae30, 0xae39, PDF_CMAP_RANGE, 25660 },
	{ 0xae30, 0xae39, PDF_CMAP_RANGE, 26920 },
	{ 0xae30, 0xae39, PDF_CMAP_RANGE, 28180 },
	{ 0xae30, 0xae39, PDF_CMAP_RANGE, 29281 },
	{ 0xae40, 0xae53, PDF_CMAP_RANGE, 15863 },
	{ 0xae54, 0xae54, PDF_CMAP_SINGLE, 9381 },
	{ 0xae55, 0xae61, PDF_CMAP_RANGE, 15883 },
	{ 0xae62, 0xae62, PDF_CMAP_SINGLE, 7792 },
	{ 0xae63, 0xae7e, PDF_CMAP_RANGE, 15896 },
	{ 0xae80, 0xae80, PDF_CMAP_SINGLE, 8335 },
	{ 0xae81, 0xae84, PDF_CMAP_RANGE, 15924 },
	{ 0xae85, 0xae85, PDF_CMAP_SINGLE, 7746 },
	{ 0xae86, 0xae8a, PDF_CMAP_RANGE, 15928 },
	{ 0xae8b, 0xae8b, PDF_CMAP_SINGLE, 8014 },
	{ 0xae8c, 0xae93, PDF_CMAP_RANGE, 15933 },
	{ 0xae94, 0xae94, PDF_CMAP_SINGLE, 7857 },
	{ 0xae95, 0xae9f, PDF_CMAP_RANGE, 15941 },
	{ 0xaea0, 0xaea0, PDF_CMAP_SINGLE, 7819 },
	{ 0xaf30, 0xaf39, PDF_CMAP_RANGE, 23150 },
	{ 0xaf30, 0xaf39, PDF_CMAP_RANGE, 24410 },
	{ 0xaf30, 0xaf39, PDF_CMAP_RANGE, 25670 },
	{ 0xaf30, 0xaf39, PDF_CMAP_RANGE, 26930 },
	{ 0xaf30, 0xaf39, PDF_CMAP_RANGE, 28190 },
	{ 0xaf30, 0xaf39, PDF_CMAP_RANGE, 29291 },
	{ 0xaf40, 0xaf63, PDF_CMAP_RANGE, 15952 },
	{ 0xaf64, 0xaf64, PDF_CMAP_SINGLE, 8134 },
	{ 0xaf65, 0xaf7a, PDF_CMAP_RANGE, 15988 },
	{ 0xaf7b, 0xaf7b, PDF_CMAP_SINGLE, 9646 },
	{ 0xaf7c, 0xaf7e, PDF_CMAP_RANGE, 16010 },
	{ 0xaf80, 0xaf81, PDF_CMAP_RANGE, 16013 },
	{ 0xaf82, 0xaf83, PDF_CMAP_TABLE, 639 },
	{ 0xaf84, 0xaf8d, PDF_CMAP_RANGE, 16015 },
	{ 0xaf8e, 0xaf91, PDF_CMAP_TABLE, 641 },
	{ 0xaf92, 0xaf9b, PDF_CMAP_RANGE, 16026 },
	{ 0xaf9c, 0xaf9c, PDF_CMAP_SINGLE, 9651 },
	{ 0xaf9d, 0xaf9e, PDF_CMAP_RANGE, 16036 },
	{ 0xaf9f, 0xafa0, PDF_CMAP_TABLE, 645 },
	{ 0xb030, 0xb039, PDF_CMAP_RANGE, 23160 },
	{ 0xb030, 0xb039, PDF_CMAP_RANGE, 24420 },
	{ 0xb030, 0xb039, PDF_CMAP_RANGE, 25680 },
	{ 0xb030, 0xb039, PDF_CMAP_RANGE, 26940 },
	{ 0xb030, 0xb039, PDF_CMAP_RANGE, 28200 },
	{ 0xb030, 0xb039, PDF_CMAP_RANGE, 29301 },
	{ 0xb040, 0xb040, PDF_CMAP_SINGLE, 16039 },
	{ 0xb041, 0xb042, PDF_CMAP_RANGE, 9647 },
	{ 0xb043, 0xb044, PDF_CMAP_TABLE, 647 },
	{ 0xb045, 0xb04e, PDF_CMAP_RANGE, 16041 },
	{ 0xb04f, 0xb04f, PDF_CMAP_SINGLE, 9644 },
	{ 0xb050, 0xb053, PDF_CMAP_RANGE, 16051 },
	{ 0xb054, 0xb054, PDF_CMAP_SINGLE, 7758 },
	{ 0xb055, 0xb056, PDF_CMAP_RANGE, 16055 },
	{ 0xb057, 0xb05f, PDF_CMAP_TABLE, 649 },
	{ 0xb060, 0xb061, PDF_CMAP_RANGE, 9652 },
	{ 0xb062, 0xb064, PDF_CMAP_TABLE, 658 },
	{ 0xb065, 0xb06b, PDF_CMAP_RANGE, 16060 },
	{ 0xb06c, 0xb06c, PDF_CMAP_SINGLE, 7913 },
	{ 0xb06d, 0xb07c, PDF_CMAP_RANGE, 16067 },
	{ 0xb07d, 0xb07e, PDF_CMAP_TABLE, 661 },
	{ 0xb080, 0xb096, PDF_CMAP_RANGE, 16084 },
	{ 0xb097, 0xb099, PDF_CMAP_TABLE, 663 },
	{ 0xb09a, 0xb0a0, PDF_CMAP_RANGE, 16108 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 940 },
	{ 0xb130, 0xb139, PDF_CMAP_RANGE, 23170 },
	{ 0xb130, 0xb139, PDF_CMAP_RANGE, 24430 },
	{ 0xb130, 0xb139, PDF_CMAP_RANGE, 25690 },
	{ 0xb130, 0xb139, PDF_CMAP_RANGE, 26950 },
	{ 0xb130, 0xb139, PDF_CMAP_RANGE, 28210 },
	{ 0xb130, 0xb139, PDF_CMAP_RANGE, 29311 },
	{ 0xb140, 0xb14a, PDF_CMAP_RANGE, 16115 },
	{ 0xb14b, 0xb152, PDF_CMAP_TABLE, 666 },
	{ 0xb153, 0xb17e, PDF_CMAP_RANGE, 16129 },
	{ 0xb180, 0xb196, PDF_CMAP_RANGE, 16173 },
	{ 0xb197, 0xb197, PDF_CMAP_SINGLE, 9860 },
	{ 0xb198, 0xb1a0, PDF_CMAP_RANGE, 16196 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1034 },
	{ 0xb230, 0xb239, PDF_CMAP_RANGE, 23180 },
	{ 0xb230, 0xb239, PDF_CMAP_RANGE, 24440 },
	{ 0xb230, 0xb239, PDF_CMAP_RANGE, 25700 },
	{ 0xb230, 0xb239, PDF_CMAP_RANGE, 26960 },
	{ 0xb230, 0xb239, PDF_CMAP_RANGE, 28220 },
	{ 0xb230, 0xb239, PDF_CMAP_RANGE, 29321 },
	{ 0xb240, 0xb241, PDF_CMAP_TABLE, 674 },
	{ 0xb242, 0xb266, PDF_CMAP_RANGE, 16206 },
	{ 0xb267, 0xb267, PDF_CMAP_SINGLE, 9454 },
	{ 0xb268, 0xb26c, PDF_CMAP_RANGE, 16243 },
	{ 0xb26d, 0xb26d, PDF_CMAP_SINGLE, 8308 },
	{ 0xb26e, 0xb273, PDF_CMAP_RANGE, 16248 },
	{ 0xb274, 0xb274, PDF_CMAP_SINGLE, 9863 },
	{ 0xb275, 0xb27e, PDF_CMAP_RANGE, 16254 },
	{ 0xb280, 0xb280, PDF_CMAP_SINGLE, 9456 },
	{ 0xb281, 0xb288, PDF_CMAP_RANGE, 16264 },
	{ 0xb289, 0xb289, PDF_CMAP_SINGLE, 9869 },
	{ 0xb28a, 0xb299, PDF_CMAP_RANGE, 16272 },
	{ 0xb29a, 0xb29a, PDF_CMAP_SINGLE, 8864 },
	{ 0xb29b, 0xb2a0, PDF_CMAP_RANGE, 16288 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1128 },
	{ 0xb330, 0xb339, PDF_CMAP_RANGE, 23190 },
	{ 0xb330, 0xb339, PDF_CMAP_RANGE, 24450 },
	{ 0xb330, 0xb339, PDF_CMAP_RANGE, 25710 },
	{ 0xb330, 0xb339, PDF_CMAP_RANGE, 26970 },
	{ 0xb330, 0xb339, PDF_CMAP_RANGE, 28230 },
	{ 0xb330, 0xb339, PDF_CMAP_RANGE, 29331 },
	{ 0xb340, 0xb342, PDF_CMAP_RANGE, 16294 },
	{ 0xb343, 0xb343, PDF_CMAP_SINGLE, 8107 },
	{ 0xb344, 0xb36f, PDF_CMAP_RANGE, 16297 },
	{ 0xb370, 0xb370, PDF_CMAP_SINGLE, 9896 },
	{ 0xb371, 0xb37e, PDF_CMAP_RANGE, 16341 },
	{ 0xb380, 0xb387, PDF_CMAP_RANGE, 16355 },
	{ 0xb388, 0xb388, PDF_CMAP_SINGLE, 9449 },
	{ 0xb389, 0xb38b, PDF_CMAP_RANGE, 16363 },
	{ 0xb38c, 0xb38e, PDF_CMAP_TABLE, 676 },
	{ 0xb38f, 0xb3a0, PDF_CMAP_RANGE, 16367 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1222 },
	{ 0xb430, 0xb439, PDF_CMAP_RANGE, 23200 },
	{ 0xb430, 0xb439, PDF_CMAP_RANGE, 24460 },
	{ 0xb430, 0xb439, PDF_CMAP_RANGE, 25720 },
	{ 0xb430, 0xb439, PDF_CMAP_RANGE, 26980 },
	{ 0xb430, 0xb439, PDF_CMAP_RANGE, 28240 },
	{ 0xb430, 0xb439, PDF_CMAP_RANGE, 29341 },
	{ 0xb434, 0xb439, PDF_CMAP_RANGE, 22404 },
	{ 0xb440, 0xb453, PDF_CMAP_RANGE, 16385 },
	{ 0xb454, 0xb454, PDF_CMAP_SINGLE, 8520 },
	{ 0xb455, 0xb457, PDF_CMAP_RANGE, 16405 },
	{ 0xb458, 0xb458, PDF_CMAP_SINGLE, 9444 },
	{ 0xb459, 0xb45d, PDF_CMAP_RANGE, 16408 },
	{ 0xb45e, 0xb461, PDF_CMAP_TABLE, 679 },
	{ 0xb462, 0xb474, PDF_CMAP_RANGE, 16414 },
	{ 0xb475, 0xb475, PDF_CMAP_SINGLE, 8871 },
	{ 0xb476, 0xb47d, PDF_CMAP_RANGE, 16433 },
	{ 0xb47e, 0xb47e, PDF_CMAP_SINGLE, 9452 },
	{ 0xb480, 0xb482, PDF_CMAP_RANGE, 16441 },
	{ 0xb483, 0xb483, PDF_CMAP_SINGLE, 9451 },
	{ 0xb484, 0xb488, PDF_CMAP_RANGE, 16444 },
	{ 0xb489, 0xb489, PDF_CMAP_SINGLE, 9443 },
	{ 0xb48a, 0xb492, PDF_CMAP_RANGE, 16449 },
	{ 0xb493, 0xb493, PDF_CMAP_SINGLE, 9450 },
	{ 0xb494, 0xb4a0, PDF_CMAP_RANGE, 16458 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1316 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 22410 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 23210 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 24470 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 25730 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 26990 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 28250 },
	{ 0xb530, 0xb539, PDF_CMAP_RANGE, 29351 },
	{ 0xb540, 0xb541, PDF_CMAP_TABLE, 683 },
	{ 0xb542, 0xb54a, PDF_CMAP_RANGE, 16472 },
	{ 0xb54b, 0xb54b, PDF_CMAP_SINGLE, 7719 },
	{ 0xb54c, 0xb555, PDF_CMAP_RANGE, 16481 },
	{ 0xb556, 0xb556, PDF_CMAP_SINGLE, 8166 },
	{ 0xb557, 0xb559, PDF_CMAP_RANGE, 16491 },
	{ 0xb55a, 0xb55c, PDF_CMAP_TABLE, 685 },
	{ 0xb55d, 0xb560, PDF_CMAP_RANGE, 16494 },
	{ 0xb561, 0xb561, PDF_CMAP_SINGLE, 9448 },
	{ 0xb562, 0xb57e, PDF_CMAP_RANGE, 16498 },
	{ 0xb580, 0xb59b, PDF_CMAP_RANGE, 16527 },
	{ 0xb59c, 0xb59d, PDF_CMAP_TABLE, 688 },
	{ 0xb59e, 0xb5a0, PDF_CMAP_RANGE, 16555 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1410 },
	{ 0xb630, 0xb637, PDF_CMAP_RANGE, 22420 },
	{ 0xb630, 0xb639, PDF_CMAP_RANGE, 23220 },
	{ 0xb630, 0xb639, PDF_CMAP_RANGE, 24480 },
	{ 0xb630, 0xb639, PDF_CMAP_RANGE, 25740 },
	{ 0xb630, 0xb639, PDF_CMAP_RANGE, 27000 },
	{ 0xb630, 0xb639, PDF_CMAP_RANGE, 28260 },
	{ 0xb630, 0xb639, PDF_CMAP_RANGE, 29361 },
	{ 0xb640, 0xb651, PDF_CMAP_RANGE, 16558 },
	{ 0xb652, 0xb652, PDF_CMAP_SINGLE, 9890 },
	{ 0xb653, 0xb654, PDF_CMAP_RANGE, 16576 },
	{ 0xb655, 0xb655, PDF_CMAP_SINGLE, 9437 },
	{ 0xb656, 0xb658, PDF_CMAP_RANGE, 16578 },
	{ 0xb659, 0xb65c, PDF_CMAP_TABLE, 690 },
	{ 0xb65d, 0xb67e, PDF_CMAP_RANGE, 16582 },
	{ 0xb680, 0xb6a0, PDF_CMAP_RANGE, 16616 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1504 },
	{ 0xb730, 0xb739, PDF_CMAP_RANGE, 23230 },
	{ 0xb730, 0xb739, PDF_CMAP_RANGE, 24490 },
	{ 0xb730, 0xb739, PDF_CMAP_RANGE, 25750 },
	{ 0xb730, 0xb739, PDF_CMAP_RANGE, 27010 },
	{ 0xb730, 0xb739, PDF_CMAP_RANGE, 28270 },
	{ 0xb730, 0xb739, PDF_CMAP_RANGE, 29371 },
	{ 0xb740, 0xb74d, PDF_CMAP_RANGE, 16649 },
	{ 0xb74e, 0xb74e, PDF_CMAP_SINGLE, 8853 },
	{ 0xb74f, 0xb750, PDF_CMAP_RANGE, 16663 },
	{ 0xb751, 0xb751, PDF_CMAP_SINGLE, 7808 },
	{ 0xb752, 0xb758, PDF_CMAP_RANGE, 16665 },
	{ 0xb759, 0xb759, PDF_CMAP_SINGLE, 9847 },
	{ 0xb75a, 0xb763, PDF_CMAP_RANGE, 16672 },
	{ 0xb764, 0xb766, PDF_CMAP_TABLE, 694 },
	{ 0xb767, 0xb776, PDF_CMAP_RANGE, 16682 },
	{ 0xb777, 0xb778, PDF_CMAP_TABLE, 697 },
	{ 0xb779, 0xb77e, PDF_CMAP_RANGE, 16698 },
	{ 0xb780, 0xb782, PDF_CMAP_TABLE, 699 },
	{ 0xb783, 0xb7a0, PDF_CMAP_RANGE, 16705 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1598 },
	{ 0xb830, 0xb839, PDF_CMAP_RANGE, 23240 },
	{ 0xb830, 0xb839, PDF_CMAP_RANGE, 24500 },
	{ 0xb830, 0xb839, PDF_CMAP_RANGE, 25760 },
	{ 0xb830, 0xb839, PDF_CMAP_RANGE, 27020 },
	{ 0xb830, 0xb839, PDF_CMAP_RANGE, 28280 },
	{ 0xb830, 0xb839, PDF_CMAP_RANGE, 29381 },
	{ 0xb840, 0xb842, PDF_CMAP_RANGE, 16735 },
	{ 0xb843, 0xb846, PDF_CMAP_TABLE, 702 },
	{ 0xb847, 0xb84c, PDF_CMAP_RANGE, 16739 },
	{ 0xb84d, 0xb84d, PDF_CMAP_SINGLE, 9657 },
	{ 0xb84e, 0xb850, PDF_CMAP_RANGE, 16745 },
	{ 0xb851, 0xb851, PDF_CMAP_SINGLE, 8170 },
	{ 0xb852, 0xb859, PDF_CMAP_RANGE, 16748 },
	{ 0xb85a, 0xb860, PDF_CMAP_TABLE, 706 },
	{ 0xb861, 0xb876, PDF_CMAP_RANGE, 16758 },
	{ 0xb877, 0xb877, PDF_CMAP_SINGLE, 8513 },
	{ 0xb878, 0xb87e, PDF_CMAP_RANGE, 16780 },
	{ 0xb880, 0xb881, PDF_CMAP_RANGE, 16787 },
	{ 0xb882, 0xb882, PDF_CMAP_SINGLE, 8135 },
	{ 0xb883, 0xb8a0, PDF_CMAP_RANGE, 16789 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1692 },
	{ 0xb930, 0xb939, PDF_CMAP_RANGE, 23250 },
	{ 0xb930, 0xb939, PDF_CMAP_RANGE, 24510 },
	{ 0xb930, 0xb939, PDF_CMAP_RANGE, 25770 },
	{ 0xb930, 0xb939, PDF_CMAP_RANGE, 27030 },
	{ 0xb930, 0xb939, PDF_CMAP_RANGE, 28290 },
	{ 0xb930, 0xb939, PDF_CMAP_RANGE, 29391 },
	{ 0xb940, 0xb94f, PDF_CMAP_RANGE, 16819 },
	{ 0xb950, 0xb950, PDF_CMAP_SINGLE, 7745 },
	{ 0xb951, 0xb960, PDF_CMAP_RANGE, 16835 },
	{ 0xb961, 0xb961, PDF_CMAP_SINGLE, 9700 },
	{ 0xb962, 0xb97a, PDF_CMAP_RANGE, 16851 },
	{ 0xb97b, 0xb97b, PDF_CMAP_SINGLE, 8070 },
	{ 0xb97c, 0xb97e, PDF_CMAP_RANGE, 16876 },
	{ 0xb980, 0xb99c, PDF_CMAP_RANGE, 16879 },
	{ 0xb99d, 0xb99d, PDF_CMAP_SINGLE, 8115 },
	{ 0xb99e, 0xb99f, PDF_CMAP_RANGE, 16908 },
	{ 0xb9a0, 0xb9a0, PDF_CMAP_SINGLE, 7919 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1786 },
	{ 0xba30, 0xba39, PDF_CMAP_RANGE, 23260 },
	{ 0xba30, 0xba39, PDF_CMAP_RANGE, 24520 },
	{ 0xba30, 0xba39, PDF_CMAP_RANGE, 25780 },
	{ 0xba30, 0xba39, PDF_CMAP_RANGE, 27040 },
	{ 0xba30, 0xba39, PDF_CMAP_RANGE, 28300 },
	{ 0xba30, 0xba39, PDF_CMAP_RANGE, 29401 },
	{ 0xba40, 0xba41, PDF_CMAP_RANGE, 16910 },
	{ 0xba42, 0xba44, PDF_CMAP_TABLE, 713 },
	{ 0xba45, 0xba55, PDF_CMAP_RANGE, 16913 },
	{ 0xba56, 0xba56, PDF_CMAP_SINGLE, 9699 },
	{ 0xba57, 0xba58, PDF_CMAP_RANGE, 16930 },
	{ 0xba59, 0xba59, PDF_CMAP_SINGLE, 8465 },
	{ 0xba5a, 0xba5f, PDF_CMAP_RANGE, 16932 },
	{ 0xba60, 0xba60, PDF_CMAP_SINGLE, 9702 },
	{ 0xba61, 0xba69, PDF_CMAP_RANGE, 16938 },
	{ 0xba6a, 0xba6a, PDF_CMAP_SINGLE, 9703 },
	{ 0xba6b, 0xba73, PDF_CMAP_RANGE, 16947 },
	{ 0xba74, 0xba74, PDF_CMAP_SINGLE, 8257 },
	{ 0xba75, 0xba7e, PDF_CMAP_RANGE, 16956 },
	{ 0xba80, 0xba83, PDF_CMAP_RANGE, 16966 },
	{ 0xba84, 0xba88, PDF_CMAP_TABLE, 716 },
	{ 0xba89, 0xba8c, PDF_CMAP_RANGE, 16972 },
	{ 0xba8d, 0xba8d, PDF_CMAP_SINGLE, 9707 },
	{ 0xba8e, 0xba9d, PDF_CMAP_RANGE, 16976 },
	{ 0xba9e, 0xbaa0, PDF_CMAP_TABLE, 721 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 1880 },
	{ 0xbb30, 0xbb39, PDF_CMAP_RANGE, 23270 },
	{ 0xbb30, 0xbb39, PDF_CMAP_RANGE, 24530 },
	{ 0xbb30, 0xbb39, PDF_CMAP_RANGE, 25790 },
	{ 0xbb30, 0xbb39, PDF_CMAP_RANGE, 27050 },
	{ 0xbb30, 0xbb39, PDF_CMAP_RANGE, 28310 },
	{ 0xbb30, 0xbb39, PDF_CMAP_RANGE, 29411 },
	{ 0xbb40, 0xbb40, PDF_CMAP_SINGLE, 8183 },
	{ 0xbb41, 0xbb48, PDF_CMAP_RANGE, 16993 },
	{ 0xbb49, 0xbb49, PDF_CMAP_SINGLE, 7821 },
	{ 0xbb4a, 0xbb57, PDF_CMAP_RANGE, 17001 },
	{ 0xbb58, 0xbb58, PDF_CMAP_SINGLE, 9705 },
	{ 0xbb59, 0xbb5a, PDF_CMAP_RANGE, 17015 },
	{ 0xbb5b, 0xbb5c, PDF_CMAP_TABLE, 724 },
	{ 0xbb5d, 0xbb5f, PDF_CMAP_RANGE, 17017 },
	{ 0xbb60, 0xbb60, PDF_CMAP_SINGLE, 9875 },
	{ 0xbb61, 0xbb64, PDF_CMAP_RANGE, 17020 },
	{ 0xbb65, 0xbb6a, PDF_CMAP_TABLE, 726 },
	{ 0xbb6b, 0xbb6d, PDF_CMAP_RANGE, 17026 },
	{ 0xbb6e, 0xbb6e, PDF_CMAP_SINGLE, 9891 },
	{ 0xbb6f, 0xbb7e, PDF_CMAP_RANGE, 17029 },
	{ 0xbb80, 0xbba0, PDF_CMAP_RANGE, 17045 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 1974 },
	{ 0xbc30, 0xbc39, PDF_CMAP_RANGE, 23280 },
	{ 0xbc30, 0xbc39, PDF_CMAP_RANGE, 24540 },
	{ 0xbc30, 0xbc39, PDF_CMAP_RANGE, 25800 },
	{ 0xbc30, 0xbc39, PDF_CMAP_RANGE, 27060 },
	{ 0xbc30, 0xbc39, PDF_CMAP_RANGE, 28320 },
	{ 0xbc30, 0xbc39, PDF_CMAP_RANGE, 29421 },
	{ 0xbc40, 0xbc51, PDF_CMAP_RANGE, 17078 },
	{ 0xbc52, 0xbc53, PDF_CMAP_TABLE, 732 },
	{ 0xbc54, 0xbc59, PDF_CMAP_RANGE, 17096 },
	{ 0xbc5a, 0xbc5a, PDF_CMAP_SINGLE, 8228 },
	{ 0xbc5b, 0xbc60, PDF_CMAP_RANGE, 17102 },
	{ 0xbc61, 0xbc69, PDF_CMAP_TABLE, 734 },
	{ 0xbc6a, 0xbc6c, PDF_CMAP_RANGE, 17112 },
	{ 0xbc6d, 0xbc75, PDF_CMAP_TABLE, 743 },
	{ 0xbc76, 0xbc77, PDF_CMAP_RANGE, 9246 },
	{ 0xbc78, 0xbc7b, PDF_CMAP_TABLE, 752 },
	{ 0xbc7c, 0xbc7d, PDF_CMAP_RANGE, 17119 },
	{ 0xbc7e, 0xbc7e, PDF_CMAP_SINGLE, 8359 },
	{ 0xbc80, 0xbc81, PDF_CMAP_RANGE, 17121 },
	{ 0xbc82, 0xbc8b, PDF_CMAP_TABLE, 756 },
	{ 0xbc8c, 0xbc8e, PDF_CMAP_RANGE, 17125 },
	{ 0xbc8f, 0xbc8f, PDF_CMAP_SINGLE, 7923 },
	{ 0xbc90, 0xbc99, PDF_CMAP_RANGE, 17128 },
	{ 0xbc9a, 0xbc9d, PDF_CMAP_TABLE, 766 },
	{ 0xbc9e, 0xbca0, PDF_CMAP_RANGE, 17138 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2068 },
	{ 0xbd30, 0xbd39, PDF_CMAP_RANGE, 23290 },
	{ 0xbd30, 0xbd39, PDF_CMAP_RANGE, 24550 },
	{ 0xbd30, 0xbd39, PDF_CMAP_RANGE, 25810 },
	{ 0xbd30, 0xbd39, PDF_CMAP_RANGE, 27070 },
	{ 0xbd30, 0xbd39, PDF_CMAP_RANGE, 28330 },
	{ 0xbd30, 0xbd39, PDF_CMAP_RANGE, 29431 },
	{ 0xbd40, 0xbd41, PDF_CMAP_RANGE, 17141 },
	{ 0xbd42, 0xbd45, PDF_CMAP_TABLE, 770 },
	{ 0xbd46, 0xbd47, PDF_CMAP_RANGE, 17144 },
	{ 0xbd48, 0xbd4f, PDF_CMAP_TABLE, 774 },
	{ 0xbd50, 0xbd56, PDF_CMAP_RANGE, 17149 },
	{ 0xbd57, 0xbd59, PDF_CMAP_TABLE, 782 },
	{ 0xbd5a, 0xbd65, PDF_CMAP_RANGE, 17157 },
	{ 0xbd66, 0xbd67, PDF_CMAP_TABLE, 785 },
	{ 0xbd68, 0xbd69, PDF_CMAP_RANGE, 17169 },
	{ 0xbd6a, 0xbd6b, PDF_CMAP_TABLE, 787 },
	{ 0xbd6c, 0xbd6e, PDF_CMAP_RANGE, 17171 },
	{ 0xbd6f, 0xbd71, PDF_CMAP_TABLE, 789 },
	{ 0xbd72, 0xbd78, PDF_CMAP_RANGE, 17175 },
	{ 0xbd79, 0xbd7b, PDF_CMAP_TABLE, 792 },
	{ 0xbd7c, 0xbd7d, PDF_CMAP_RANGE, 17182 },
	{ 0xbd7e, 0xbd7e, PDF_CMAP_SINGLE, 8148 },
	{ 0xbd80, 0xbd81, PDF_CMAP_TABLE, 795 },
	{ 0xbd82, 0xbd88, PDF_CMAP_RANGE, 17185 },
	{ 0xbd89, 0xbd8b, PDF_CMAP_TABLE, 797 },
	{ 0xbd8c, 0xbd8d, PDF_CMAP_RANGE, 17193 },
	{ 0xbd8e, 0xbd91, PDF_CMAP_TABLE, 800 },
	{ 0xbd92, 0xbd96, PDF_CMAP_RANGE, 17196 },
	{ 0xbd97, 0xbd97, PDF_CMAP_SINGLE, 8536 },
	{ 0xbd98, 0xbd9a, PDF_CMAP_RANGE, 17201 },
	{ 0xbd9b, 0xbd9b, PDF_CMAP_SINGLE, 8130 },
	{ 0xbd9c, 0xbda0, PDF_CMAP_RANGE, 17204 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2162 },
	{ 0xbe30, 0xbe39, PDF_CMAP_RANGE, 23300 },
	{ 0xbe30, 0xbe39, PDF_CMAP_RANGE, 24560 },
	{ 0xbe30, 0xbe39, PDF_CMAP_RANGE, 25820 },
	{ 0xbe30, 0xbe39, PDF_CMAP_RANGE, 27080 },
	{ 0xbe30, 0xbe39, PDF_CMAP_RANGE, 28340 },
	{ 0xbe30, 0xbe39, PDF_CMAP_RANGE, 29441 },
	{ 0xbe40, 0xbe42, PDF_CMAP_RANGE, 17209 },
	{ 0xbe43, 0xbe45, PDF_CMAP_TABLE, 804 },
	{ 0xbe46, 0xbe48, PDF_CMAP_RANGE, 17213 },
	{ 0xbe49, 0xbe4a, PDF_CMAP_TABLE, 807 },
	{ 0xbe4b, 0xbe50, PDF_CMAP_RANGE, 17216 },
	{ 0xbe51, 0xbe59, PDF_CMAP_TABLE, 809 },
	{ 0xbe5a, 0xbe5c, PDF_CMAP_RANGE, 17223 },
	{ 0xbe5d, 0xbe64, PDF_CMAP_TABLE, 818 },
	{ 0xbe65, 0xbe68, PDF_CMAP_RANGE, 17227 },
	{ 0xbe69, 0xbe69, PDF_CMAP_SINGLE, 9269 },
	{ 0xbe6a, 0xbe6b, PDF_CMAP_RANGE, 17231 },
	{ 0xbe6c, 0xbe6c, PDF_CMAP_SINGLE, 9275 },
	{ 0xbe6d, 0xbe6e, PDF_CMAP_RANGE, 17233 },
	{ 0xbe6f, 0xbe70, PDF_CMAP_TABLE, 826 },
	{ 0xbe71, 0xbe75, PDF_CMAP_RANGE, 17235 },
	{ 0xbe76, 0xbe79, PDF_CMAP_TABLE, 828 },
	{ 0xbe7a, 0xbe7b, PDF_CMAP_RANGE, 17241 },
	{ 0xbe7c, 0xbe7e, PDF_CMAP_TABLE, 832 },
	{ 0xbe80, 0xbe82, PDF_CMAP_RANGE, 17243 },
	{ 0xbe83, 0xbe89, PDF_CMAP_TABLE, 835 },
	{ 0xbe8a, 0xbe8b, PDF_CMAP_RANGE, 17248 },
	{ 0xbe8c, 0xbe8f, PDF_CMAP_TABLE, 842 },
	{ 0xbe90, 0xbe91, PDF_CMAP_RANGE, 17251 },
	{ 0xbe92, 0xbe92, PDF_CMAP_SINGLE, 8326 },
	{ 0xbe93, 0xbe94, PDF_CMAP_RANGE, 17253 },
	{ 0xbe95, 0xbe9c, PDF_CMAP_TABLE, 846 },
	{ 0xbe9d, 0xbe9e, PDF_CMAP_RANGE, 17258 },
	{ 0xbe9f, 0xbea0, PDF_CMAP_TABLE, 854 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2256 },
	{ 0xbf30, 0xbf39, PDF_CMAP_RANGE, 23310 },
	{ 0xbf30, 0xbf39, PDF_CMAP_RANGE, 24570 },
	{ 0xbf30, 0xbf39, PDF_CMAP_RANGE, 25830 },
	{ 0xbf30, 0xbf39, PDF_CMAP_RANGE, 27090 },
	{ 0xbf30, 0xbf39, PDF_CMAP_RANGE, 28350 },
	{ 0xbf30, 0xbf39, PDF_CMAP_RANGE, 29451 },
	{ 0xbf40, 0xbf40, PDF_CMAP_SINGLE, 9894 },
	{ 0xbf41, 0xbf4c, PDF_CMAP_RANGE, 17261 },
	{ 0xbf4d, 0xbf50, PDF_CMAP_TABLE, 856 },
	{ 0xbf51, 0xbf54, PDF_CMAP_RANGE, 17273 },
	{ 0xbf55, 0xbf56, PDF_CMAP_TABLE, 860 },
	{ 0xbf57, 0xbf5f, PDF_CMAP_RANGE, 17277 },
	{ 0xbf60, 0xbf64, PDF_CMAP_TABLE, 862 },
	{ 0xbf65, 0xbf67, PDF_CMAP_RANGE, 17287 },
	{ 0xbf68, 0xbf68, PDF_CMAP_SINGLE, 8642 },
	{ 0xbf69, 0xbf6b, PDF_CMAP_RANGE, 17290 },
	{ 0xbf6c, 0xbf6c, PDF_CMAP_SINGLE, 8558 },
	{ 0xbf6d, 0xbf6f, PDF_CMAP_RANGE, 17293 },
	{ 0xbf70, 0xbf73, PDF_CMAP_TABLE, 867 },
	{ 0xbf74, 0xbf75, PDF_CMAP_RANGE, 17297 },
	{ 0xbf76, 0xbf7e, PDF_CMAP_TABLE, 871 },
	{ 0xbf80, 0xbf81, PDF_CMAP_RANGE, 17301 },
	{ 0xbf82, 0xbf83, PDF_CMAP_TABLE, 880 },
	{ 0xbf84, 0xbf88, PDF_CMAP_RANGE, 17303 },
	{ 0xbf89, 0xbf8a, PDF_CMAP_TABLE, 882 },
	{ 0xbf8b, 0xbf94, PDF_CMAP_RANGE, 17308 },
	{ 0xbf95, 0xbf98, PDF_CMAP_TABLE, 884 },
	{ 0xbf99, 0xbf9c, PDF_CMAP_RANGE, 17319 },
	{ 0xbf9d, 0xbf9d, PDF_CMAP_SINGLE, 9300 },
	{ 0xbf9e, 0xbfa0, PDF_CMAP_RANGE, 17323 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2350 },
	{ 0xc030, 0xc039, PDF_CMAP_RANGE, 23320 },
	{ 0xc030, 0xc039, PDF_CMAP_RANGE, 24580 },
	{ 0xc030, 0xc039, PDF_CMAP_RANGE, 25840 },
	{ 0xc030, 0xc039, PDF_CMAP_RANGE, 27100 },
	{ 0xc030, 0xc039, PDF_CMAP_RANGE, 28360 },
	{ 0xc030, 0xc039, PDF_CMAP_RANGE, 29461 },
	{ 0xc040, 0xc040, PDF_CMAP_SINGLE, 8443 },
	{ 0xc041, 0xc043, PDF_CMAP_RANGE, 17326 },
	{ 0xc044, 0xc044, PDF_CMAP_SINGLE, 9280 },
	{ 0xc045, 0xc04a, PDF_CMAP_RANGE, 17329 },
	{ 0xc04b, 0xc052, PDF_CMAP_TABLE, 888 },
	{ 0xc053, 0xc054, PDF_CMAP_RANGE, 17336 },
	{ 0xc055, 0xc055, PDF_CMAP_SINGLE, 8110 },
	{ 0xc056, 0xc05a, PDF_CMAP_RANGE, 17338 },
	{ 0xc05b, 0xc05b, PDF_CMAP_SINGLE, 8730 },
	{ 0xc05c, 0xc05d, PDF_CMAP_RANGE, 17343 },
	{ 0xc05e, 0xc060, PDF_CMAP_TABLE, 896 },
	{ 0xc061, 0xc068, PDF_CMAP_RANGE, 17345 },
	{ 0xc069, 0xc070, PDF_CMAP_TABLE, 899 },
	{ 0xc071, 0xc073, PDF_CMAP_RANGE, 17356 },
	{ 0xc074, 0xc079, PDF_CMAP_TABLE, 907 },
	{ 0xc07a, 0xc07b, PDF_CMAP_RANGE, 17361 },
	{ 0xc07c, 0xc07c, PDF_CMAP_SINGLE, 8191 },
	{ 0xc07d, 0xc07e, PDF_CMAP_RANGE, 17363 },
	{ 0xc080, 0xc09a, PDF_CMAP_RANGE, 17365 },
	{ 0xc09b, 0xc09d, PDF_CMAP_TABLE, 913 },
	{ 0xc09e, 0xc0a0, PDF_CMAP_RANGE, 17393 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2444 },
	{ 0xc130, 0xc139, PDF_CMAP_RANGE, 23330 },
	{ 0xc130, 0xc139, PDF_CMAP_RANGE, 24590 },
	{ 0xc130, 0xc139, PDF_CMAP_RANGE, 25850 },
	{ 0xc130, 0xc139, PDF_CMAP_RANGE, 27110 },
	{ 0xc130, 0xc139, PDF_CMAP_RANGE, 28370 },
	{ 0xc130, 0xc139, PDF_CMAP_RANGE, 29471 },
	{ 0xc140, 0xc14f, PDF_CMAP_RANGE, 17396 },
	{ 0xc150, 0xc150, PDF_CMAP_SINGLE, 7914 },
	{ 0xc151, 0xc153, PDF_CMAP_RANGE, 17412 },
	{ 0xc154, 0xc154, PDF_CMAP_SINGLE, 7724 },
	{ 0xc155, 0xc15e, PDF_CMAP_RANGE, 17415 },
	{ 0xc15f, 0xc162, PDF_CMAP_TABLE, 916 },
	{ 0xc163, 0xc174, PDF_CMAP_RANGE, 17426 },
	{ 0xc175, 0xc175, PDF_CMAP_SINGLE, 9714 },
	{ 0xc176, 0xc177, PDF_CMAP_RANGE, 17444 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 8725 },
	{ 0xc179, 0xc17e, PDF_CMAP_RANGE, 17446 },
	{ 0xc180, 0xc194, PDF_CMAP_RANGE, 17452 },
	{ 0xc195, 0xc195, PDF_CMAP_SINGLE, 8622 },
	{ 0xc196, 0xc1a0, PDF_CMAP_RANGE, 17473 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2538 },
	{ 0xc230, 0xc239, PDF_CMAP_RANGE, 23340 },
	{ 0xc230, 0xc239, PDF_CMAP_RANGE, 24600 },
	{ 0xc230, 0xc239, PDF_CMAP_RANGE, 25860 },
	{ 0xc230, 0xc239, PDF_CMAP_RANGE, 27120 },
	{ 0xc230, 0xc239, PDF_CMAP_RANGE, 28380 },
	{ 0xc230, 0xc239, PDF_CMAP_RANGE, 29481 },
	{ 0xc240, 0xc24d, PDF_CMAP_RANGE, 17484 },
	{ 0xc24e, 0xc24e, PDF_CMAP_SINGLE, 8416 },
	{ 0xc24f, 0xc264, PDF_CMAP_RANGE, 17498 },
	{ 0xc265, 0xc267, PDF_CMAP_TABLE, 920 },
	{ 0xc268, 0xc27c, PDF_CMAP_RANGE, 17521 },
	{ 0xc27d, 0xc27e, PDF_CMAP_TABLE, 923 },
	{ 0xc280, 0xc283, PDF_CMAP_RANGE, 17543 },
	{ 0xc284, 0xc284, PDF_CMAP_SINGLE, 8601 },
	{ 0xc285, 0xc292, PDF_CMAP_RANGE, 17547 },
	{ 0xc293, 0xc29c, PDF_CMAP_TABLE, 925 },
	{ 0xc29d, 0xc29f, PDF_CMAP_RANGE, 17563 },
	{ 0xc2a0, 0xc2a0, PDF_CMAP_SINGLE, 8569 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2632 },
	{ 0xc330, 0xc339, PDF_CMAP_RANGE, 23350 },
	{ 0xc330, 0xc339, PDF_CMAP_RANGE, 24610 },
	{ 0xc330, 0xc339, PDF_CMAP_RANGE, 25870 },
	{ 0xc330, 0xc339, PDF_CMAP_RANGE, 27130 },
	{ 0xc330, 0xc339, PDF_CMAP_RANGE, 28390 },
	{ 0xc330, 0xc339, PDF_CMAP_RANGE, 29491 },
	{ 0xc340, 0xc340, PDF_CMAP_SINGLE, 8248 },
	{ 0xc341, 0xc342, PDF_CMAP_RANGE, 17566 },
	{ 0xc343, 0xc343, PDF_CMAP_SINGLE, 8533 },
	{ 0xc344, 0xc37a, PDF_CMAP_RANGE, 17568 },
	{ 0xc37b, 0xc37b, PDF_CMAP_SINGLE, 8658 },
	{ 0xc37c, 0xc37e, PDF_CMAP_RANGE, 17623 },
	{ 0xc380, 0xc383, PDF_CMAP_RANGE, 17626 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 9415 },
	{ 0xc385, 0xc39a, PDF_CMAP_RANGE, 17630 },
	{ 0xc39b, 0xc39b, PDF_CMAP_SINGLE, 8825 },
	{ 0xc39c, 0xc3a0, PDF_CMAP_RANGE, 17652 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2726 },
	{ 0xc430, 0xc439, PDF_CMAP_RANGE, 23360 },
	{ 0xc430, 0xc439, PDF_CMAP_RANGE, 24620 },
	{ 0xc430, 0xc439, PDF_CMAP_RANGE, 25880 },
	{ 0xc430, 0xc439, PDF_CMAP_RANGE, 27140 },
	{ 0xc430, 0xc439, PDF_CMAP_RANGE, 28400 },
	{ 0xc430, 0xc439, PDF_CMAP_RANGE, 29501 },
	{ 0xc440, 0xc448, PDF_CMAP_RANGE, 17657 },
	{ 0xc449, 0xc449, PDF_CMAP_SINGLE, 8483 },
	{ 0xc44a, 0xc44b, PDF_CMAP_RANGE, 17666 },
	{ 0xc44c, 0xc44c, PDF_CMAP_SINGLE, 9413 },
	{ 0xc44d, 0xc453, PDF_CMAP_RANGE, 17668 },
	{ 0xc454, 0xc454, PDF_CMAP_SINGLE, 9417 },
	{ 0xc455, 0xc457, PDF_CMAP_RANGE, 17675 },
	{ 0xc458, 0xc458, PDF_CMAP_SINGLE, 8340 },
	{ 0xc459, 0xc45a, PDF_CMAP_RANGE, 17678 },
	{ 0xc45b, 0xc45b, PDF_CMAP_SINGLE, 8854 },
	{ 0xc45c, 0xc462, PDF_CMAP_RANGE, 17680 },
	{ 0xc463, 0xc463, PDF_CMAP_SINGLE, 7799 },
	{ 0xc464, 0xc476, PDF_CMAP_RANGE, 17687 },
	{ 0xc477, 0xc477, PDF_CMAP_SINGLE, 7942 },
	{ 0xc478, 0xc479, PDF_CMAP_RANGE, 17706 },
	{ 0xc47a, 0xc47a, PDF_CMAP_SINGLE, 8101 },
	{ 0xc47b, 0xc47e, PDF_CMAP_RANGE, 17708 },
	{ 0xc480, 0xc481, PDF_CMAP_TABLE, 935 },
	{ 0xc482, 0xc490, PDF_CMAP_RANGE, 17713 },
	{ 0xc491, 0xc493, PDF_CMAP_TABLE, 937 },
	{ 0xc494, 0xc497, PDF_CMAP_RANGE, 17728 },
	{ 0xc498, 0xc49c, PDF_CMAP_TABLE, 940 },
	{ 0xc49d, 0xc4a0, PDF_CMAP_RANGE, 17734 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2820 },
	{ 0xc530, 0xc539, PDF_CMAP_RANGE, 23370 },
	{ 0xc530, 0xc539, PDF_CMAP_RANGE, 24630 },
	{ 0xc530, 0xc539, PDF_CMAP_RANGE, 25890 },
	{ 0xc530, 0xc539, PDF_CMAP_RANGE, 27150 },
	{ 0xc530, 0xc539, PDF_CMAP_RANGE, 28410 },
	{ 0xc530, 0xc539, PDF_CMAP_RANGE, 29511 },
	{ 0xc540, 0xc543, PDF_CMAP_RANGE, 17738 },
	{ 0xc544, 0xc546, PDF_CMAP_TABLE, 945 },
	{ 0xc547, 0xc54a, PDF_CMAP_RANGE, 17743 },
	{ 0xc54b, 0xc54c, PDF_CMAP_TABLE, 948 },
	{ 0xc54d, 0xc551, PDF_CMAP_RANGE, 17747 },
	{ 0xc552, 0xc552, PDF_CMAP_SINGLE, 8236 },
	{ 0xc553, 0xc55e, PDF_CMAP_RANGE, 17752 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 8545 },
	{ 0xc560, 0xc562, PDF_CMAP_RANGE, 17764 },
	{ 0xc563, 0xc566, PDF_CMAP_TABLE, 950 },
	{ 0xc567, 0xc57e, PDF_CMAP_RANGE, 17767 },
	{ 0xc580, 0xc592, PDF_CMAP_RANGE, 17791 },
	{ 0xc593, 0xc593, PDF_CMAP_SINGLE, 7777 },
	{ 0xc594, 0xc59b, PDF_CMAP_RANGE, 17810 },
	{ 0xc59c, 0xc59e, PDF_CMAP_TABLE, 954 },
	{ 0xc59f, 0xc5a0, PDF_CMAP_RANGE, 17819 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 2914 },
	{ 0xc630, 0xc639, PDF_CMAP_RANGE, 23380 },
	{ 0xc630, 0xc639, PDF_CMAP_RANGE, 24640 },
	{ 0xc630, 0xc639, PDF_CMAP_RANGE, 25900 },
	{ 0xc630, 0xc639, PDF_CMAP_RANGE, 27160 },
	{ 0xc630, 0xc639, PDF_CMAP_RANGE, 28420 },
	{ 0xc630, 0xc639, PDF_CMAP_RANGE, 29521 },
	{ 0xc640, 0xc641, PDF_CMAP_TABLE, 957 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 17822 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 8072 },
	{ 0xc645, 0xc646, PDF_CMAP_RANGE, 17824 },
	{ 0xc647, 0xc647, PDF_CMAP_SINGLE, 8696 },
	{ 0xc648, 0xc662, PDF_CMAP_RANGE, 17826 },
	{ 0xc663, 0xc663, PDF_CMAP_SINGLE, 8994 },
	{ 0xc664, 0xc671, PDF_CMAP_RANGE, 17853 },
	{ 0xc672, 0xc672, PDF_CMAP_SINGLE, 9014 },
	{ 0xc673, 0xc67e, PDF_CMAP_RANGE, 17867 },
	{ 0xc680, 0xc6a0, PDF_CMAP_RANGE, 17879 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3008 },
	{ 0xc730, 0xc739, PDF_CMAP_RANGE, 23390 },
	{ 0xc730, 0xc739, PDF_CMAP_RANGE, 24650 },
	{ 0xc730, 0xc739, PDF_CMAP_RANGE, 25910 },
	{ 0xc730, 0xc739, PDF_CMAP_RANGE, 27170 },
	{ 0xc730, 0xc739, PDF_CMAP_RANGE, 28430 },
	{ 0xc730, 0xc739, PDF_CMAP_RANGE, 29531 },
	{ 0xc740, 0xc765, PDF_CMAP_RANGE, 17912 },
	{ 0xc766, 0xc766, PDF_CMAP_SINGLE, 8875 },
	{ 0xc767, 0xc76e, PDF_CMAP_RANGE, 17950 },
	{ 0xc76f, 0xc76f, PDF_CMAP_SINGLE, 8127 },
	{ 0xc770, 0xc775, PDF_CMAP_RANGE, 17958 },
	{ 0xc776, 0xc776, PDF_CMAP_SINGLE, 8061 },
	{ 0xc777, 0xc77a, PDF_CMAP_RANGE, 17964 },
	{ 0xc77b, 0xc77b, PDF_CMAP_SINGLE, 9011 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 17968 },
	{ 0xc780, 0xc7a0, PDF_CMAP_RANGE, 17971 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3102 },
	{ 0xc830, 0xc839, PDF_CMAP_RANGE, 23400 },
	{ 0xc830, 0xc839, PDF_CMAP_RANGE, 24660 },
	{ 0xc830, 0xc839, PDF_CMAP_RANGE, 25920 },
	{ 0xc830, 0xc839, PDF_CMAP_RANGE, 27180 },
	{ 0xc830, 0xc839, PDF_CMAP_RANGE, 28440 },
	{ 0xc830, 0xc839, PDF_CMAP_RANGE, 29541 },
	{ 0xc840, 0xc841, PDF_CMAP_TABLE, 959 },
	{ 0xc842, 0xc84e, PDF_CMAP_RANGE, 18005 },
	{ 0xc84f, 0xc84f, PDF_CMAP_SINGLE, 9012 },
	{ 0xc850, 0xc851, PDF_CMAP_RANGE, 18018 },
	{ 0xc852, 0xc852, PDF_CMAP_SINGLE, 8177 },
	{ 0xc853, 0xc865, PDF_CMAP_RANGE, 18020 },
	{ 0xc866, 0xc866, PDF_CMAP_SINGLE, 8587 },
	{ 0xc867, 0xc86d, PDF_CMAP_RANGE, 18039 },
	{ 0xc86e, 0xc86e, PDF_CMAP_SINGLE, 9033 },
	{ 0xc86f, 0xc87d, PDF_CMAP_RANGE, 18046 },
	{ 0xc87e, 0xc87e, PDF_CMAP_SINGLE, 8715 },
	{ 0xc880, 0xc886, PDF_CMAP_RANGE, 18061 },
	{ 0xc887, 0xc887, PDF_CMAP_SINGLE, 9030 },
	{ 0xc888, 0xc891, PDF_CMAP_RANGE, 18068 },
	{ 0xc892, 0xc894, PDF_CMAP_TABLE, 961 },
	{ 0xc895, 0xc898, PDF_CMAP_RANGE, 18079 },
	{ 0xc899, 0xc899, PDF_CMAP_SINGLE, 9889 },
	{ 0xc89a, 0xc89c, PDF_CMAP_RANGE, 18083 },
	{ 0xc89d, 0xc89d, PDF_CMAP_SINGLE, 8034 },
	{ 0xc89e, 0xc8a0, PDF_CMAP_RANGE, 18086 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3196 },
	{ 0xc930, 0xc939, PDF_CMAP_RANGE, 23410 },
	{ 0xc930, 0xc939, PDF_CMAP_RANGE, 24670 },
	{ 0xc930, 0xc939, PDF_CMAP_RANGE, 25930 },
	{ 0xc930, 0xc939, PDF_CMAP_RANGE, 27190 },
	{ 0xc930, 0xc939, PDF_CMAP_RANGE, 28450 },
	{ 0xc930, 0xc939, PDF_CMAP_RANGE, 29551 },
	{ 0xc940, 0xc94e, PDF_CMAP_RANGE, 18089 },
	{ 0xc94f, 0xc950, PDF_CMAP_TABLE, 964 },
	{ 0xc951, 0xc96d, PDF_CMAP_RANGE, 18104 },
	{ 0xc96e, 0xc970, PDF_CMAP_TABLE, 966 },
	{ 0xc971, 0xc976, PDF_CMAP_RANGE, 18134 },
	{ 0xc977, 0xc977, PDF_CMAP_SINGLE, 7954 },
	{ 0xc978, 0xc97e, PDF_CMAP_RANGE, 18140 },
	{ 0xc980, 0xc98e, PDF_CMAP_RANGE, 18147 },
	{ 0xc98f, 0xc990, PDF_CMAP_TABLE, 969 },
	{ 0xc991, 0xc99b, PDF_CMAP_RANGE, 18162 },
	{ 0xc99c, 0xc99c, PDF_CMAP_SINGLE, 9020 },
	{ 0xc99d, 0xc9a0, PDF_CMAP_RANGE, 18173 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 3290 },
	{ 0xca30, 0xca39, PDF_CMAP_RANGE, 23420 },
	{ 0xca30, 0xca39, PDF_CMAP_RANGE, 24680 },
	{ 0xca30, 0xca39, PDF_CMAP_RANGE, 25940 },
	{ 0xca30, 0xca39, PDF_CMAP_RANGE, 27200 },
	{ 0xca30, 0xca39, PDF_CMAP_RANGE, 28460 },
	{ 0xca30, 0xca39, PDF_CMAP_RANGE, 29561 },
	{ 0xca40, 0xca4d, PDF_CMAP_RANGE, 18177 },
	{ 0xca4e, 0xca4e, PDF_CMAP_SINGLE, 9839 },
	{ 0xca4f, 0xca55, PDF_CMAP_RANGE, 18191 },
	{ 0xca56, 0xca56, PDF_CMAP_SINGLE, 9041 },
	{ 0xca57, 0xca58, PDF_CMAP_RANGE, 18198 },
	{ 0xca59, 0xca59, PDF_CMAP_SINGLE, 8096 },
	{ 0xca5a, 0xca5b, PDF_CMAP_RANGE, 18200 },
	{ 0xca5c, 0xca5c, PDF_CMAP_SINGLE, 9016 },
	{ 0xca5d, 0xca60, PDF_CMAP_RANGE, 18202 },
	{ 0xca61, 0xca61, PDF_CMAP_SINGLE, 8731 },
	{ 0xca62, 0xca6d, PDF_CMAP_RANGE, 18206 },
	{ 0xca6e, 0xca6e, PDF_CMAP_SINGLE, 9026 },
	{ 0xca6f, 0xca71, PDF_CMAP_RANGE, 18218 },
	{ 0xca72, 0xca72, PDF_CMAP_SINGLE, 9039 },
	{ 0xca73, 0xca76, PDF_CMAP_RANGE, 18221 },
	{ 0xca77, 0xca77, PDF_CMAP_SINGLE, 9021 },
	{ 0xca78, 0xca7a, PDF_CMAP_RANGE, 18225 },
	{ 0xca7b, 0xca7e, PDF_CMAP_TABLE, 971 },
	{ 0xca80, 0xca81, PDF_CMAP_TABLE, 975 },
	{ 0xca82, 0xca88, PDF_CMAP_RANGE, 18230 },
	{ 0xca89, 0xca89, PDF_CMAP_SINGLE, 9040 },
	{ 0xca8a, 0xca8d, PDF_CMAP_RANGE, 18237 },
	{ 0xca8e, 0xca8f, PDF_CMAP_TABLE, 977 },
	{ 0xca90, 0xca91, PDF_CMAP_RANGE, 18241 },
	{ 0xca92, 0xca92, PDF_CMAP_SINGLE, 8651 },
	{ 0xca93, 0xca99, PDF_CMAP_RANGE, 18243 },
	{ 0xca9a, 0xca9a, PDF_CMAP_SINGLE, 9045 },
	{ 0xca9b, 0xcaa0, PDF_CMAP_RANGE, 18250 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3384 },
	{ 0xcb30, 0xcb39, PDF_CMAP_RANGE, 23430 },
	{ 0xcb30, 0xcb39, PDF_CMAP_RANGE, 24690 },
	{ 0xcb30, 0xcb39, PDF_CMAP_RANGE, 25950 },
	{ 0xcb30, 0xcb39, PDF_CMAP_RANGE, 27210 },
	{ 0xcb30, 0xcb39, PDF_CMAP_RANGE, 28470 },
	{ 0xcb30, 0xcb39, PDF_CMAP_RANGE, 29571 },
	{ 0xcb40, 0xcb42, PDF_CMAP_RANGE, 18256 },
	{ 0xcb43, 0xcb47, PDF_CMAP_TABLE, 979 },
	{ 0xcb48, 0xcb4a, PDF_CMAP_RANGE, 18261 },
	{ 0xcb4b, 0xcb4b, PDF_CMAP_SINGLE, 9856 },
	{ 0xcb4c, 0xcb4d, PDF_CMAP_RANGE, 18264 },
	{ 0xcb4e, 0xcb4e, PDF_CMAP_SINGLE, 8410 },
	{ 0xcb4f, 0xcb56, PDF_CMAP_RANGE, 18266 },
	{ 0xcb57, 0xcb57, PDF_CMAP_SINGLE, 9034 },
	{ 0xcb58, 0xcb5c, PDF_CMAP_RANGE, 18274 },
	{ 0xcb5d, 0xcb5f, PDF_CMAP_TABLE, 984 },
	{ 0xcb60, 0xcb69, PDF_CMAP_RANGE, 18280 },
	{ 0xcb6a, 0xcb6a, PDF_CMAP_SINGLE, 9023 },
	{ 0xcb6b, 0xcb7a, PDF_CMAP_RANGE, 18290 },
	{ 0xcb7b, 0xcb7c, PDF_CMAP_TABLE, 987 },
	{ 0xcb7d, 0xcb7e, PDF_CMAP_RANGE, 18306 },
	{ 0xcb80, 0xcb86, PDF_CMAP_RANGE, 18308 },
	{ 0xcb87, 0xcb87, PDF_CMAP_SINGLE, 8722 },
	{ 0xcb88, 0xcb8d, PDF_CMAP_RANGE, 18315 },
	{ 0xcb8e, 0xcb8e, PDF_CMAP_SINGLE, 8711 },
	{ 0xcb8f, 0xcb91, PDF_CMAP_RANGE, 18321 },
	{ 0xcb92, 0xcb92, PDF_CMAP_SINGLE, 9049 },
	{ 0xcb93, 0xcb9b, PDF_CMAP_RANGE, 18324 },
	{ 0xcb9c, 0xcb9e, PDF_CMAP_TABLE, 989 },
	{ 0xcb9f, 0xcba0, PDF_CMAP_RANGE, 18334 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 3478 },
	{ 0xcc30, 0xcc39, PDF_CMAP_RANGE, 23440 },
	{ 0xcc30, 0xcc39, PDF_CMAP_RANGE, 24700 },
	{ 0xcc30, 0xcc39, PDF_CMAP_RANGE, 25960 },
	{ 0xcc30, 0xcc39, PDF_CMAP_RANGE, 27220 },
	{ 0xcc30, 0xcc39, PDF_CMAP_RANGE, 28480 },
	{ 0xcc30, 0xcc39, PDF_CMAP_RANGE, 29581 },
	{ 0xcc40, 0xcc41, PDF_CMAP_TABLE, 992 },
	{ 0xcc42, 0xcc48, PDF_CMAP_RANGE, 18336 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_TABLE, 994 },
	{ 0xcc4c, 0xcc4e, PDF_CMAP_RANGE, 18343 },
	{ 0xcc4f, 0xcc4f, PDF_CMAP_SINGLE, 8379 },
	{ 0xcc50, 0xcc5b, PDF_CMAP_RANGE, 18346 },
	{ 0xcc5c, 0xcc5c, PDF_CMAP_SINGLE, 9050 },
	{ 0xcc5d, 0xcc5f, PDF_CMAP_RANGE, 18358 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 9046 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 18361 },
	{ 0xcc64, 0xcc64, PDF_CMAP_SINGLE, 9015 },
	{ 0xcc65, 0xcc6c, PDF_CMAP_RANGE, 18364 },
	{ 0xcc6d, 0xcc6d, PDF_CMAP_SINGLE, 8185 },
	{ 0xcc6e, 0xcc78, PDF_CMAP_RANGE, 18372 },
	{ 0xcc79, 0xcc79, PDF_CMAP_SINGLE, 9043 },
	{ 0xcc7a, 0xcc7c, PDF_CMAP_RANGE, 18383 },
	{ 0xcc7d, 0xcc7e, PDF_CMAP_TABLE, 997 },
	{ 0xcc80, 0xcc8d, PDF_CMAP_RANGE, 18387 },
	{ 0xcc8e, 0xcc8e, PDF_CMAP_SINGLE, 7829 },
	{ 0xcc8f, 0xcc93, PDF_CMAP_RANGE, 18401 },
	{ 0xcc94, 0xcc96, PDF_CMAP_TABLE, 999 },
	{ 0xcc97, 0xcc9c, PDF_CMAP_RANGE, 18407 },
	{ 0xcc9d, 0xcc9d, PDF_CMAP_SINGLE, 8168 },
	{ 0xcc9e, 0xcca0, PDF_CMAP_RANGE, 18413 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3572 },
	{ 0xcd30, 0xcd39, PDF_CMAP_RANGE, 23450 },
	{ 0xcd30, 0xcd39, PDF_CMAP_RANGE, 24710 },
	{ 0xcd30, 0xcd39, PDF_CMAP_RANGE, 25970 },
	{ 0xcd30, 0xcd39, PDF_CMAP_RANGE, 27230 },
	{ 0xcd30, 0xcd39, PDF_CMAP_RANGE, 28490 },
	{ 0xcd30, 0xcd39, PDF_CMAP_RANGE, 29591 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 18416 },
	{ 0xcd80, 0xcd8f, PDF_CMAP_RANGE, 18479 },
	{ 0xcd90, 0xcd90, PDF_CMAP_SINGLE, 9690 },
	{ 0xcd91, 0xcd97, PDF_CMAP_RANGE, 18495 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 9687 },
	{ 0xcd99, 0xcda0, PDF_CMAP_RANGE, 18502 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 3666 },
	{ 0xce30, 0xce39, PDF_CMAP_RANGE, 23460 },
	{ 0xce30, 0xce39, PDF_CMAP_RANGE, 24720 },
	{ 0xce30, 0xce39, PDF_CMAP_RANGE, 25980 },
	{ 0xce30, 0xce39, PDF_CMAP_RANGE, 27240 },
	{ 0xce30, 0xce39, PDF_CMAP_RANGE, 28500 },
	{ 0xce30, 0xce39, PDF_CMAP_RANGE, 29601 },
	{ 0xce40, 0xce66, PDF_CMAP_RANGE, 18510 },
	{ 0xce67, 0xce67, PDF_CMAP_SINGLE, 8494 },
	{ 0xce68, 0xce71, PDF_CMAP_RANGE, 18549 },
	{ 0xce72, 0xce72, PDF_CMAP_SINGLE, 8626 },
	{ 0xce73, 0xce7e, PDF_CMAP_RANGE, 18559 },
	{ 0xce80, 0xce81, PDF_CMAP_TABLE, 1002 },
	{ 0xce82, 0xce86, PDF_CMAP_RANGE, 18572 },
	{ 0xce87, 0xce87, PDF_CMAP_SINGLE, 9692 },
	{ 0xce88, 0xce9a, PDF_CMAP_RANGE, 18577 },
	{ 0xce9b, 0xce9b, PDF_CMAP_SINGLE, 8300 },
	{ 0xce9c, 0xce9d, PDF_CMAP_RANGE, 18596 },
	{ 0xce9e, 0xce9e, PDF_CMAP_SINGLE, 8742 },
	{ 0xce9f, 0xcea0, PDF_CMAP_RANGE, 18598 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3760 },
	{ 0xcf30, 0xcf39, PDF_CMAP_RANGE, 23470 },
	{ 0xcf30, 0xcf39, PDF_CMAP_RANGE, 24730 },
	{ 0xcf30, 0xcf39, PDF_CMAP_RANGE, 25990 },
	{ 0xcf30, 0xcf39, PDF_CMAP_RANGE, 27250 },
	{ 0xcf30, 0xcf39, PDF_CMAP_RANGE, 28510 },
	{ 0xcf30, 0xcf39, PDF_CMAP_RANGE, 29611 },
	{ 0xcf40, 0xcf4d, PDF_CMAP_RANGE, 18600 },
	{ 0xcf4e, 0xcf4e, PDF_CMAP_SINGLE, 9696 },
	{ 0xcf4f, 0xcf54, PDF_CMAP_RANGE, 18614 },
	{ 0xcf55, 0xcf55, PDF_CMAP_SINGLE, 8827 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 18620 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 9694 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 18622 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 9697 },
	{ 0xcf5d, 0xcf6b, PDF_CMAP_RANGE, 18625 },
	{ 0xcf6c, 0xcf6c, PDF_CMAP_SINGLE, 9685 },
	{ 0xcf6d, 0xcf72, PDF_CMAP_RANGE, 18640 },
	{ 0xcf73, 0xcf75, PDF_CMAP_TABLE, 1004 },
	{ 0xcf76, 0xcf77, PDF_CMAP_RANGE, 18647 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 7817 },
	{ 0xcf79, 0xcf7b, PDF_CMAP_RANGE, 18649 },
	{ 0xcf7c, 0xcf7c, PDF_CMAP_SINGLE, 9689 },
	{ 0xcf7d, 0xcf7e, PDF_CMAP_RANGE, 18652 },
	{ 0xcf80, 0xcf81, PDF_CMAP_TABLE, 1007 },
	{ 0xcf82, 0xcf88, PDF_CMAP_RANGE, 18655 },
	{ 0xcf89, 0xcf8a, PDF_CMAP_TABLE, 1009 },
	{ 0xcf8b, 0xcf92, PDF_CMAP_RANGE, 18662 },
	{ 0xcf93, 0xcf94, PDF_CMAP_TABLE, 1011 },
	{ 0xcf95, 0xcf9d, PDF_CMAP_RANGE, 18670 },
	{ 0xcf9e, 0xcfa0, PDF_CMAP_TABLE, 1013 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 3854 },
	{ 0xd030, 0xd039, PDF_CMAP_RANGE, 23480 },
	{ 0xd030, 0xd039, PDF_CMAP_RANGE, 24740 },
	{ 0xd030, 0xd039, PDF_CMAP_RANGE, 26000 },
	{ 0xd030, 0xd039, PDF_CMAP_RANGE, 27260 },
	{ 0xd030, 0xd039, PDF_CMAP_RANGE, 28520 },
	{ 0xd030, 0xd039, PDF_CMAP_RANGE, 29621 },
	{ 0xd040, 0xd04c, PDF_CMAP_RANGE, 18680 },
	{ 0xd04d, 0xd04d, PDF_CMAP_SINGLE, 7977 },
	{ 0xd04e, 0xd050, PDF_CMAP_RANGE, 18693 },
	{ 0xd051, 0xd051, PDF_CMAP_SINGLE, 7771 },
	{ 0xd052, 0xd054, PDF_CMAP_RANGE, 18696 },
	{ 0xd055, 0xd055, PDF_CMAP_SINGLE, 8310 },
	{ 0xd056, 0xd05b, PDF_CMAP_RANGE, 18699 },
	{ 0xd05c, 0xd05c, PDF_CMAP_SINGLE, 8855 },
	{ 0xd05d, 0xd05f, PDF_CMAP_RANGE, 18705 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 9872 },
	{ 0xd061, 0xd066, PDF_CMAP_RANGE, 18708 },
	{ 0xd067, 0xd067, PDF_CMAP_SINGLE, 8511 },
	{ 0xd068, 0xd06b, PDF_CMAP_RANGE, 18714 },
	{ 0xd06c, 0xd06e, PDF_CMAP_TABLE, 1016 },
	{ 0xd06f, 0xd07c, PDF_CMAP_RANGE, 18719 },
	{ 0xd07d, 0xd07e, PDF_CMAP_TABLE, 1019 },
	{ 0xd080, 0xd0a0, PDF_CMAP_RANGE, 18734 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 3948 },
	{ 0xd130, 0xd139, PDF_CMAP_RANGE, 23490 },
	{ 0xd130, 0xd139, PDF_CMAP_RANGE, 24750 },
	{ 0xd130, 0xd139, PDF_CMAP_RANGE, 26010 },
	{ 0xd130, 0xd139, PDF_CMAP_RANGE, 27270 },
	{ 0xd130, 0xd139, PDF_CMAP_RANGE, 28530 },
	{ 0xd130, 0xd139, PDF_CMAP_RANGE, 29631 },
	{ 0xd140, 0xd154, PDF_CMAP_RANGE, 18767 },
	{ 0xd155, 0xd155, PDF_CMAP_SINGLE, 9713 },
	{ 0xd156, 0xd158, PDF_CMAP_RANGE, 18788 },
	{ 0xd159, 0xd159, PDF_CMAP_SINGLE, 8204 },
	{ 0xd15a, 0xd160, PDF_CMAP_RANGE, 18791 },
	{ 0xd161, 0xd162, PDF_CMAP_TABLE, 1021 },
	{ 0xd163, 0xd174, PDF_CMAP_RANGE, 18798 },
	{ 0xd175, 0xd175, PDF_CMAP_SINGLE, 9895 },
	{ 0xd176, 0xd17c, PDF_CMAP_RANGE, 18816 },
	{ 0xd17d, 0xd17e, PDF_CMAP_TABLE, 1023 },
	{ 0xd180, 0xd19c, PDF_CMAP_RANGE, 18824 },
	{ 0xd19d, 0xd19e, PDF_CMAP_TABLE, 1025 },
	{ 0xd19f, 0xd1a0, PDF_CMAP_RANGE, 18853 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 4042 },
	{ 0xd230, 0xd239, PDF_CMAP_RANGE, 23500 },
	{ 0xd230, 0xd239, PDF_CMAP_RANGE, 24760 },
	{ 0xd230, 0xd239, PDF_CMAP_RANGE, 26020 },
	{ 0xd230, 0xd239, PDF_CMAP_RANGE, 27280 },
	{ 0xd230, 0xd239, PDF_CMAP_RANGE, 28540 },
	{ 0xd230, 0xd239, PDF_CMAP_RANGE, 29641 },
	{ 0xd240, 0xd240, PDF_CMAP_SINGLE, 9662 },
	{ 0xd241, 0xd242, PDF_CMAP_RANGE, 18855 },
	{ 0xd243, 0xd243, PDF_CMAP_SINGLE, 8925 },
	{ 0xd244, 0xd24c, PDF_CMAP_RANGE, 18857 },
	{ 0xd24d, 0xd24d, PDF_CMAP_SINGLE, 9661 },
	{ 0xd24e, 0xd25b, PDF_CMAP_RANGE, 18866 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 7722 },
	{ 0xd25d, 0xd262, PDF_CMAP_RANGE, 18880 },
	{ 0xd263, 0xd264, PDF_CMAP_TABLE, 1027 },
	{ 0xd265, 0xd267, PDF_CMAP_RANGE, 18886 },
	{ 0xd268, 0xd268, PDF_CMAP_SINGLE, 9663 },
	{ 0xd269, 0xd26c, PDF_CMAP_RANGE, 18889 },
	{ 0xd26d, 0xd26f, PDF_CMAP_TABLE, 1029 },
	{ 0xd270, 0xd271, PDF_CMAP_RANGE, 18894 },
	{ 0xd272, 0xd272, PDF_CMAP_SINGLE, 7807 },
	{ 0xd273, 0xd274, PDF_CMAP_RANGE, 18896 },
	{ 0xd275, 0xd275, PDF_CMAP_SINGLE, 8621 },
	{ 0xd276, 0xd27e, PDF_CMAP_RANGE, 18898 },
	{ 0xd280, 0xd289, PDF_CMAP_RANGE, 18907 },
	{ 0xd28a, 0xd28a, PDF_CMAP_SINGLE, 8086 },
	{ 0xd28b, 0xd28d, PDF_CMAP_RANGE, 18917 },
	{ 0xd28e, 0xd28e, PDF_CMAP_SINGLE, 7986 },
	{ 0xd28f, 0xd291, PDF_CMAP_RANGE, 18920 },
	{ 0xd292, 0xd292, PDF_CMAP_SINGLE, 8324 },
	{ 0xd293, 0xd294, PDF_CMAP_RANGE, 18923 },
	{ 0xd295, 0xd297, PDF_CMAP_TABLE, 1032 },
	{ 0xd298, 0xd29f, PDF_CMAP_RANGE, 18926 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 9402 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4136 },
	{ 0xd330, 0xd339, PDF_CMAP_RANGE, 23510 },
	{ 0xd330, 0xd339, PDF_CMAP_RANGE, 24770 },
	{ 0xd330, 0xd339, PDF_CMAP_RANGE, 26030 },
	{ 0xd330, 0xd339, PDF_CMAP_RANGE, 27290 },
	{ 0xd330, 0xd339, PDF_CMAP_RANGE, 28550 },
	{ 0xd330, 0xd339, PDF_CMAP_RANGE, 29651 },
	{ 0xd340, 0xd343, PDF_CMAP_RANGE, 18934 },
	{ 0xd344, 0xd344, PDF_CMAP_SINGLE, 9404 },
	{ 0xd345, 0xd347, PDF_CMAP_RANGE, 18938 },
	{ 0xd348, 0xd34a, PDF_CMAP_TABLE, 1035 },
	{ 0xd34b, 0xd34c, PDF_CMAP_RANGE, 18942 },
	{ 0xd34d, 0xd34d, PDF_CMAP_SINGLE, 9405 },
	{ 0xd34e, 0xd34f, PDF_CMAP_RANGE, 18944 },
	{ 0xd350, 0xd350, PDF_CMAP_SINGLE, 9406 },
	{ 0xd351, 0xd354, PDF_CMAP_RANGE, 18946 },
	{ 0xd355, 0xd355, PDF_CMAP_SINGLE, 9407 },
	{ 0xd356, 0xd357, PDF_CMAP_RANGE, 18950 },
	{ 0xd358, 0xd358, PDF_CMAP_SINGLE, 8146 },
	{ 0xd359, 0xd35a, PDF_CMAP_RANGE, 18952 },
	{ 0xd35b, 0xd35e, PDF_CMAP_TABLE, 1038 },
	{ 0xd35f, 0xd377, PDF_CMAP_RANGE, 18955 },
	{ 0xd378, 0xd37c, PDF_CMAP_TABLE, 1042 },
	{ 0xd37d, 0xd37e, PDF_CMAP_RANGE, 18982 },
	{ 0xd380, 0xd384, PDF_CMAP_RANGE, 18984 },
	{ 0xd385, 0xd387, PDF_CMAP_TABLE, 1047 },
	{ 0xd388, 0xd38a, PDF_CMAP_RANGE, 18989 },
	{ 0xd38b, 0xd393, PDF_CMAP_TABLE, 1050 },
	{ 0xd394, 0xd395, PDF_CMAP_RANGE, 18996 },
	{ 0xd396, 0xd39b, PDF_CMAP_TABLE, 1059 },
	{ 0xd39c, 0xd39d, PDF_CMAP_RANGE, 19000 },
	{ 0xd39e, 0xd3a0, PDF_CMAP_TABLE, 1065 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4230 },
	{ 0xd430, 0xd439, PDF_CMAP_RANGE, 23520 },
	{ 0xd430, 0xd439, PDF_CMAP_RANGE, 24780 },
	{ 0xd430, 0xd439, PDF_CMAP_RANGE, 26040 },
	{ 0xd430, 0xd439, PDF_CMAP_RANGE, 27300 },
	{ 0xd430, 0xd439, PDF_CMAP_RANGE, 28560 },
	{ 0xd430, 0xd439, PDF_CMAP_RANGE, 29661 },
	{ 0xd440, 0xd441, PDF_CMAP_TABLE, 1068 },
	{ 0xd442, 0xd444, PDF_CMAP_RANGE, 19004 },
	{ 0xd445, 0xd447, PDF_CMAP_TABLE, 1070 },
	{ 0xd448, 0xd44b, PDF_CMAP_RANGE, 19008 },
	{ 0xd44c, 0xd44c, PDF_CMAP_SINGLE, 7922 },
	{ 0xd44d, 0xd44e, PDF_CMAP_RANGE, 19012 },
	{ 0xd44f, 0xd44f, PDF_CMAP_SINGLE, 8479 },
	{ 0xd450, 0xd452, PDF_CMAP_RANGE, 19014 },
	{ 0xd453, 0xd453, PDF_CMAP_SINGLE, 8669 },
	{ 0xd454, 0xd455, PDF_CMAP_RANGE, 19017 },
	{ 0xd456, 0xd458, PDF_CMAP_TABLE, 1073 },
	{ 0xd459, 0xd45b, PDF_CMAP_RANGE, 19020 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 8834 },
	{ 0xd45d, 0xd461, PDF_CMAP_RANGE, 19023 },
	{ 0xd462, 0xd462, PDF_CMAP_SINGLE, 8934 },
	{ 0xd463, 0xd466, PDF_CMAP_RANGE, 19028 },
	{ 0xd467, 0xd467, PDF_CMAP_SINGLE, 8936 },
	{ 0xd468, 0xd46d, PDF_CMAP_RANGE, 19032 },
	{ 0xd46e, 0xd475, PDF_CMAP_TABLE, 1076 },
	{ 0xd476, 0xd477, PDF_CMAP_RANGE, 19041 },
	{ 0xd478, 0xd478, PDF_CMAP_SINGLE, 8938 },
	{ 0xd479, 0xd47a, PDF_CMAP_RANGE, 19043 },
	{ 0xd47b, 0xd47b, PDF_CMAP_SINGLE, 8893 },
	{ 0xd47c, 0xd47d, PDF_CMAP_RANGE, 19045 },
	{ 0xd47e, 0xd47e, PDF_CMAP_SINGLE, 7838 },
	{ 0xd480, 0xd481, PDF_CMAP_RANGE, 19047 },
	{ 0xd482, 0xd484, PDF_CMAP_TABLE, 1084 },
	{ 0xd485, 0xd486, PDF_CMAP_RANGE, 19049 },
	{ 0xd487, 0xd487, PDF_CMAP_SINGLE, 8503 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 19051 },
	{ 0xd48a, 0xd496, PDF_CMAP_TABLE, 1087 },
	{ 0xd497, 0xd49b, PDF_CMAP_RANGE, 19056 },
	{ 0xd49c, 0xd49c, PDF_CMAP_SINGLE, 8944 },
	{ 0xd49d, 0xd49e, PDF_CMAP_RANGE, 19061 },
	{ 0xd49f, 0xd4a0, PDF_CMAP_TABLE, 1100 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 4324 },
	{ 0xd530, 0xd539, PDF_CMAP_RANGE, 23530 },
	{ 0xd530, 0xd539, PDF_CMAP_RANGE, 24790 },
	{ 0xd530, 0xd539, PDF_CMAP_RANGE, 26050 },
	{ 0xd530, 0xd539, PDF_CMAP_RANGE, 27310 },
	{ 0xd530, 0xd539, PDF_CMAP_RANGE, 28570 },
	{ 0xd530, 0xd539, PDF_CMAP_RANGE, 29671 },
	{ 0xd540, 0xd542, PDF_CMAP_RANGE, 19064 },
	{ 0xd543, 0xd546, PDF_CMAP_TABLE, 1102 },
	{ 0xd547, 0xd549, PDF_CMAP_RANGE, 19067 },
	{ 0xd54a, 0xd54a, PDF_CMAP_SINGLE, 8446 },
	{ 0xd54b, 0xd54d, PDF_CMAP_RANGE, 19070 },
	{ 0xd54e, 0xd54f, PDF_CMAP_RANGE, 8953 },
	{ 0xd550, 0xd551, PDF_CMAP_TABLE, 1106 },
	{ 0xd552, 0xd553, PDF_CMAP_RANGE, 19074 },
	{ 0xd554, 0xd556, PDF_CMAP_TABLE, 1108 },
	{ 0xd557, 0xd559, PDF_CMAP_RANGE, 19077 },
	{ 0xd55a, 0xd564, PDF_CMAP_TABLE, 1111 },
	{ 0xd565, 0xd567, PDF_CMAP_RANGE, 19083 },
	{ 0xd568, 0xd568, PDF_CMAP_SINGLE, 8519 },
	{ 0xd569, 0xd56b, PDF_CMAP_RANGE, 19086 },
	{ 0xd56c, 0xd56e, PDF_CMAP_TABLE, 1122 },
	{ 0xd56f, 0xd571, PDF_CMAP_RANGE, 19090 },
	{ 0xd572, 0xd572, PDF_CMAP_SINGLE, 8961 },
	{ 0xd573, 0xd574, PDF_CMAP_RANGE, 19093 },
	{ 0xd575, 0xd575, PDF_CMAP_SINGLE, 7925 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 19095 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 8728 },
	{ 0xd579, 0xd57a, PDF_CMAP_RANGE, 19097 },
	{ 0xd57b, 0xd57b, PDF_CMAP_SINGLE, 7878 },
	{ 0xd57c, 0xd57d, PDF_CMAP_RANGE, 19099 },
	{ 0xd57e, 0xd57e, PDF_CMAP_SINGLE, 8960 },
	{ 0xd580, 0xd581, PDF_CMAP_TABLE, 1125 },
	{ 0xd582, 0xd583, PDF_CMAP_RANGE, 19102 },
	{ 0xd584, 0xd58f, PDF_CMAP_TABLE, 1127 },
	{ 0xd590, 0xd592, PDF_CMAP_RANGE, 19109 },
	{ 0xd593, 0xd594, PDF_CMAP_TABLE, 1139 },
	{ 0xd595, 0xd597, PDF_CMAP_RANGE, 19112 },
	{ 0xd598, 0xd59b, PDF_CMAP_TABLE, 1141 },
	{ 0xd59c, 0xd59e, PDF_CMAP_RANGE, 19116 },
	{ 0xd59f, 0xd5a0, PDF_CMAP_TABLE, 1145 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 4418 },
	{ 0xd630, 0xd639, PDF_CMAP_RANGE, 23540 },
	{ 0xd630, 0xd639, PDF_CMAP_RANGE, 24800 },
	{ 0xd630, 0xd639, PDF_CMAP_RANGE, 26060 },
	{ 0xd630, 0xd639, PDF_CMAP_RANGE, 27320 },
	{ 0xd630, 0xd639, PDF_CMAP_RANGE, 28580 },
	{ 0xd630, 0xd639, PDF_CMAP_RANGE, 29681 },
	{ 0xd640, 0xd643, PDF_CMAP_TABLE, 1147 },
	{ 0xd644, 0xd646, PDF_CMAP_RANGE, 19121 },
	{ 0xd647, 0xd64a, PDF_CMAP_TABLE, 1151 },
	{ 0xd64b, 0xd64c, PDF_CMAP_RANGE, 19125 },
	{ 0xd64d, 0xd64f, PDF_CMAP_TABLE, 1155 },
	{ 0xd650, 0xd651, PDF_CMAP_RANGE, 19128 },
	{ 0xd652, 0xd661, PDF_CMAP_TABLE, 1158 },
	{ 0xd662, 0xd664, PDF_CMAP_RANGE, 19135 },
	{ 0xd665, 0xd665, PDF_CMAP_SINGLE, 8023 },
	{ 0xd666, 0xd668, PDF_CMAP_RANGE, 19138 },
	{ 0xd669, 0xd66b, PDF_CMAP_TABLE, 1174 },
	{ 0xd66c, 0xd66e, PDF_CMAP_RANGE, 19142 },
	{ 0xd66f, 0xd678, PDF_CMAP_TABLE, 1177 },
	{ 0xd679, 0xd67b, PDF_CMAP_RANGE, 19148 },
	{ 0xd67c, 0xd67c, PDF_CMAP_SINGLE, 8710 },
	{ 0xd67d, 0xd67e, PDF_CMAP_RANGE, 19151 },
	{ 0xd680, 0xd682, PDF_CMAP_RANGE, 19153 },
	{ 0xd683, 0xd683, PDF_CMAP_SINGLE, 8973 },
	{ 0xd684, 0xd685, PDF_CMAP_RANGE, 19156 },
	{ 0xd686, 0xd688, PDF_CMAP_TABLE, 1187 },
	{ 0xd689, 0xd68d, PDF_CMAP_RANGE, 19158 },
	{ 0xd68e, 0xd68e, PDF_CMAP_SINGLE, 8931 },
	{ 0xd68f, 0xd693, PDF_CMAP_RANGE, 19163 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 8122 },
	{ 0xd695, 0xd698, PDF_CMAP_RANGE, 19168 },
	{ 0xd699, 0xd699, PDF_CMAP_SINGLE, 8312 },
	{ 0xd69a, 0xd6a0, PDF_CMAP_RANGE, 19172 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd730, 0xd739, PDF_CMAP_RANGE, 23550 },
	{ 0xd730, 0xd739, PDF_CMAP_RANGE, 24810 },
	{ 0xd730, 0xd739, PDF_CMAP_RANGE, 26070 },
	{ 0xd730, 0xd739, PDF_CMAP_RANGE, 27330 },
	{ 0xd730, 0xd739, PDF_CMAP_RANGE, 28590 },
	{ 0xd730, 0xd739, PDF_CMAP_RANGE, 29691 },
	{ 0xd740, 0xd742, PDF_CMAP_RANGE, 19179 },
	{ 0xd743, 0xd743, PDF_CMAP_SINGLE, 8840 },
	{ 0xd744, 0xd747, PDF_CMAP_RANGE, 19182 },
	{ 0xd748, 0xd749, PDF_CMAP_TABLE, 1190 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 19186 },
	{ 0xd750, 0xd756, PDF_CMAP_TABLE, 1192 },
	{ 0xd757, 0xd763, PDF_CMAP_RANGE, 19194 },
	{ 0xd764, 0xd764, PDF_CMAP_SINGLE, 8984 },
	{ 0xd765, 0xd766, PDF_CMAP_RANGE, 19207 },
	{ 0xd767, 0xd768, PDF_CMAP_TABLE, 1199 },
	{ 0xd769, 0xd76b, PDF_CMAP_RANGE, 19209 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 8405 },
	{ 0xd76d, 0xd76e, PDF_CMAP_RANGE, 19212 },
	{ 0xd76f, 0xd76f, PDF_CMAP_SINGLE, 8010 },
	{ 0xd770, 0xd774, PDF_CMAP_RANGE, 19214 },
	{ 0xd775, 0xd775, PDF_CMAP_SINGLE, 8768 },
	{ 0xd776, 0xd777, PDF_CMAP_RANGE, 19219 },
	{ 0xd778, 0xd778, PDF_CMAP_SINGLE, 7891 },
	{ 0xd779, 0xd77e, PDF_CMAP_RANGE, 19221 },
	{ 0xd780, 0xd782, PDF_CMAP_RANGE, 19227 },
	{ 0xd783, 0xd783, PDF_CMAP_SINGLE, 7753 },
	{ 0xd784, 0xd786, PDF_CMAP_RANGE, 19230 },
	{ 0xd787, 0xd787, PDF_CMAP_SINGLE, 9755 },
	{ 0xd788, 0xd78a, PDF_CMAP_RANGE, 19233 },
	{ 0xd78b, 0xd78f, PDF_CMAP_TABLE, 1201 },
	{ 0xd790, 0xd794, PDF_CMAP_RANGE, 19237 },
	{ 0xd795, 0xd797, PDF_CMAP_TABLE, 1206 },
	{ 0xd798, 0xd7a0, PDF_CMAP_RANGE, 19243 },
	{ 0xd7a1, 0xd7f9, PDF_CMAP_RANGE, 4606 },
	{ 0xd830, 0xd839, PDF_CMAP_RANGE, 23560 },
	{ 0xd830, 0xd839, PDF_CMAP_RANGE, 24820 },
	{ 0xd830, 0xd839, PDF_CMAP_RANGE, 26080 },
	{ 0xd830, 0xd839, PDF_CMAP_RANGE, 27340 },
	{ 0xd830, 0xd839, PDF_CMAP_RANGE, 28600 },
	{ 0xd830, 0xd839, PDF_CMAP_RANGE, 29701 },
	{ 0xd840, 0xd84c, PDF_CMAP_RANGE, 19252 },
	{ 0xd84d, 0xd84d, PDF_CMAP_SINGLE, 8392 },
	{ 0xd84e, 0xd852, PDF_CMAP_RANGE, 19265 },
	{ 0xd853, 0xd853, PDF_CMAP_SINGLE, 7933 },
	{ 0xd854, 0xd87e, PDF_CMAP_RANGE, 19270 },
	{ 0xd880, 0xd88f, PDF_CMAP_RANGE, 19313 },
	{ 0xd890, 0xd895, PDF_CMAP_TABLE, 1209 },
	{ 0xd896, 0xd899, PDF_CMAP_RANGE, 19330 },
	{ 0xd89a, 0xd8a0, PDF_CMAP_TABLE, 1215 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4695 },
	{ 0xd930, 0xd939, PDF_CMAP_RANGE, 23570 },
	{ 0xd930, 0xd939, PDF_CMAP_RANGE, 24830 },
	{ 0xd930, 0xd939, PDF_CMAP_RANGE, 26090 },
	{ 0xd930, 0xd939, PDF_CMAP_RANGE, 27350 },
	{ 0xd930, 0xd939, PDF_CMAP_RANGE, 28610 },
	{ 0xd930, 0xd939, PDF_CMAP_RANGE, 29711 },
	{ 0xd940, 0xd95c, PDF_CMAP_TABLE, 1222 },
	{ 0xd95d, 0xd962, PDF_CMAP_RANGE, 19342 },
	{ 0xd963, 0xd967, PDF_CMAP_TABLE, 1251 },
	{ 0xd968, 0xd96b, PDF_CMAP_RANGE, 19349 },
	{ 0xd96c, 0xd979, PDF_CMAP_TABLE, 1256 },
	{ 0xd97a, 0xd97b, PDF_CMAP_RANGE, 19357 },
	{ 0xd97c, 0xd97e, PDF_CMAP_TABLE, 1270 },
	{ 0xd980, 0xd980, PDF_CMAP_SINGLE, 7892 },
	{ 0xd981, 0xd986, PDF_CMAP_RANGE, 19359 },
	{ 0xd987, 0xd987, PDF_CMAP_SINGLE, 8179 },
	{ 0xd988, 0xd98c, PDF_CMAP_RANGE, 19365 },
	{ 0xd98d, 0xd991, PDF_CMAP_TABLE, 1273 },
	{ 0xd992, 0xd996, PDF_CMAP_RANGE, 19370 },
	{ 0xd997, 0xd998, PDF_CMAP_TABLE, 1278 },
	{ 0xd999, 0xd99a, PDF_CMAP_RANGE, 19375 },
	{ 0xd99b, 0xd9a0, PDF_CMAP_TABLE, 1280 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4789 },
	{ 0xda30, 0xda39, PDF_CMAP_RANGE, 23580 },
	{ 0xda30, 0xda39, PDF_CMAP_RANGE, 24840 },
	{ 0xda30, 0xda39, PDF_CMAP_RANGE, 26100 },
	{ 0xda30, 0xda39, PDF_CMAP_RANGE, 27360 },
	{ 0xda30, 0xda39, PDF_CMAP_RANGE, 28620 },
	{ 0xda30, 0xda39, PDF_CMAP_RANGE, 29721 },
	{ 0xda40, 0xda42, PDF_CMAP_TABLE, 1286 },
	{ 0xda43, 0xda47, PDF_CMAP_RANGE, 19380 },
	{ 0xda48, 0xda48, PDF_CMAP_SINGLE, 8509 },
	{ 0xda49, 0xda4c, PDF_CMAP_RANGE, 19385 },
	{ 0xda4d, 0xda4e, PDF_CMAP_TABLE, 1289 },
	{ 0xda4f, 0xda72, PDF_CMAP_RANGE, 19389 },
	{ 0xda73, 0xda73, PDF_CMAP_SINGLE, 7956 },
	{ 0xda74, 0xda76, PDF_CMAP_RANGE, 19425 },
	{ 0xda77, 0xda77, PDF_CMAP_SINGLE, 8826 },
	{ 0xda78, 0xda7e, PDF_CMAP_RANGE, 19428 },
	{ 0xda80, 0xda84, PDF_CMAP_RANGE, 19435 },
	{ 0xda85, 0xda85, PDF_CMAP_SINGLE, 8430 },
	{ 0xda86, 0xda8d, PDF_CMAP_RANGE, 19440 },
	{ 0xda8e, 0xda8e, PDF_CMAP_SINGLE, 9720 },
	{ 0xda8f, 0xdaa0, PDF_CMAP_RANGE, 19448 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4883 },
	{ 0xdb30, 0xdb39, PDF_CMAP_RANGE, 23590 },
	{ 0xdb30, 0xdb39, PDF_CMAP_RANGE, 24850 },
	{ 0xdb30, 0xdb39, PDF_CMAP_RANGE, 26110 },
	{ 0xdb30, 0xdb39, PDF_CMAP_RANGE, 27370 },
	{ 0xdb30, 0xdb39, PDF_CMAP_RANGE, 28630 },
	{ 0xdb30, 0xdb39, PDF_CMAP_RANGE, 29731 },
	{ 0xdb40, 0xdb5f, PDF_CMAP_RANGE, 19466 },
	{ 0xdb60, 0xdb60, PDF_CMAP_SINGLE, 8084 },
	{ 0xdb61, 0xdb77, PDF_CMAP_RANGE, 19498 },
	{ 0xdb78, 0xdb78, PDF_CMAP_SINGLE, 8752 },
	{ 0xdb79, 0xdb7e, PDF_CMAP_RANGE, 19521 },
	{ 0xdb80, 0xdb83, PDF_CMAP_RANGE, 19527 },
	{ 0xdb84, 0xdb84, PDF_CMAP_SINGLE, 9725 },
	{ 0xdb85, 0xdb8a, PDF_CMAP_RANGE, 19531 },
	{ 0xdb8b, 0xdb8b, PDF_CMAP_SINGLE, 9728 },
	{ 0xdb8c, 0xdb97, PDF_CMAP_RANGE, 19537 },
	{ 0xdb98, 0xdb98, PDF_CMAP_SINGLE, 9734 },
	{ 0xdb99, 0xdba0, PDF_CMAP_RANGE, 19549 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 4977 },
	{ 0xdc30, 0xdc39, PDF_CMAP_RANGE, 23600 },
	{ 0xdc30, 0xdc39, PDF_CMAP_RANGE, 24860 },
	{ 0xdc30, 0xdc39, PDF_CMAP_RANGE, 26120 },
	{ 0xdc30, 0xdc39, PDF_CMAP_RANGE, 27380 },
	{ 0xdc30, 0xdc39, PDF_CMAP_RANGE, 28640 },
	{ 0xdc30, 0xdc39, PDF_CMAP_RANGE, 29741 },
	{ 0xdc40, 0xdc44, PDF_CMAP_RANGE, 19557 },
	{ 0xdc45, 0xdc45, PDF_CMAP_SINGLE, 9727 },
	{ 0xdc46, 0xdc4e, PDF_CMAP_RANGE, 19562 },
	{ 0xdc4f, 0xdc57, PDF_CMAP_TABLE, 1291 },
	{ 0xdc58, 0xdc5c, PDF_CMAP_RANGE, 19573 },
	{ 0xdc5d, 0xdc5d, PDF_CMAP_SINGLE, 9729 },
	{ 0xdc5e, 0xdc61, PDF_CMAP_RANGE, 19578 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 9733 },
	{ 0xdc63, 0xdc65, PDF_CMAP_RANGE, 19582 },
	{ 0xdc66, 0xdc67, PDF_CMAP_TABLE, 1300 },
	{ 0xdc68, 0xdc6a, PDF_CMAP_RANGE, 19585 },
	{ 0xdc6b, 0xdc6b, PDF_CMAP_SINGLE, 9735 },
	{ 0xdc6c, 0xdc7b, PDF_CMAP_RANGE, 19588 },
	{ 0xdc7c, 0xdc7c, PDF_CMAP_SINGLE, 8432 },
	{ 0xdc7d, 0xdc7e, PDF_CMAP_RANGE, 19604 },
	{ 0xdc80, 0xdc86, PDF_CMAP_RANGE, 19606 },
	{ 0xdc87, 0xdc8a, PDF_CMAP_TABLE, 1302 },
	{ 0xdc8b, 0xdc8d, PDF_CMAP_RANGE, 19613 },
	{ 0xdc8e, 0xdc90, PDF_CMAP_TABLE, 1306 },
	{ 0xdc91, 0xdc96, PDF_CMAP_RANGE, 19617 },
	{ 0xdc97, 0xdc97, PDF_CMAP_SINGLE, 9357 },
	{ 0xdc98, 0xdc9a, PDF_CMAP_RANGE, 19623 },
	{ 0xdc9b, 0xdc9b, PDF_CMAP_SINGLE, 8450 },
	{ 0xdc9c, 0xdc9f, PDF_CMAP_RANGE, 19626 },
	{ 0xdca0, 0xdca0, PDF_CMAP_SINGLE, 9364 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5071 },
	{ 0xdd30, 0xdd39, PDF_CMAP_RANGE, 23610 },
	{ 0xdd30, 0xdd39, PDF_CMAP_RANGE, 24870 },
	{ 0xdd30, 0xdd39, PDF_CMAP_RANGE, 26130 },
	{ 0xdd30, 0xdd39, PDF_CMAP_RANGE, 27390 },
	{ 0xdd30, 0xdd39, PDF_CMAP_RANGE, 28650 },
	{ 0xdd30, 0xdd39, PDF_CMAP_RANGE, 29751 },
	{ 0xdd40, 0xdd45, PDF_CMAP_RANGE, 19630 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 9363 },
	{ 0xdd47, 0xdd4c, PDF_CMAP_RANGE, 19636 },
	{ 0xdd4d, 0xdd4d, PDF_CMAP_SINGLE, 9358 },
	{ 0xdd4e, 0xdd52, PDF_CMAP_RANGE, 19642 },
	{ 0xdd53, 0xdd59, PDF_CMAP_TABLE, 1309 },
	{ 0xdd5a, 0xdd5d, PDF_CMAP_RANGE, 19648 },
	{ 0xdd5e, 0xdd65, PDF_CMAP_TABLE, 1316 },
	{ 0xdd66, 0xdd6c, PDF_CMAP_RANGE, 19655 },
	{ 0xdd6d, 0xdd70, PDF_CMAP_TABLE, 1324 },
	{ 0xdd71, 0xdd75, PDF_CMAP_RANGE, 19663 },
	{ 0xdd76, 0xdd78, PDF_CMAP_TABLE, 1328 },
	{ 0xdd79, 0xdd7a, PDF_CMAP_RANGE, 9373 },
	{ 0xdd7b, 0xdd7e, PDF_CMAP_RANGE, 19668 },
	{ 0xdd80, 0xdd82, PDF_CMAP_TABLE, 1331 },
	{ 0xdd83, 0xdd84, PDF_CMAP_RANGE, 19673 },
	{ 0xdd85, 0xdd86, PDF_CMAP_TABLE, 1334 },
	{ 0xdd87, 0xdd8a, PDF_CMAP_RANGE, 19675 },
	{ 0xdd8b, 0xdd8b, PDF_CMAP_SINGLE, 8048 },
	{ 0xdd8c, 0xdd8e, PDF_CMAP_RANGE, 19679 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 9376 },
	{ 0xdd90, 0xdd93, PDF_CMAP_RANGE, 19682 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 8507 },
	{ 0xdd95, 0xdd96, PDF_CMAP_RANGE, 19686 },
	{ 0xdd97, 0xdd97, PDF_CMAP_SINGLE, 7943 },
	{ 0xdd98, 0xdd99, PDF_CMAP_RANGE, 19688 },
	{ 0xdd9a, 0xdd9b, PDF_CMAP_TABLE, 1336 },
	{ 0xdd9c, 0xdd9d, PDF_CMAP_RANGE, 19690 },
	{ 0xdd9e, 0xdda0, PDF_CMAP_TABLE, 1338 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 23620 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 24880 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 26140 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 27400 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 28660 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 29761 },
	{ 0xde40, 0xde41, PDF_CMAP_TABLE, 1341 },
	{ 0xde42, 0xde43, PDF_CMAP_RANGE, 19693 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 8872 },
	{ 0xde45, 0xde47, PDF_CMAP_RANGE, 19695 },
	{ 0xde48, 0xde49, PDF_CMAP_TABLE, 1343 },
	{ 0xde4a, 0xde4e, PDF_CMAP_RANGE, 19698 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 9378 },
	{ 0xde50, 0xde59, PDF_CMAP_RANGE, 19703 },
	{ 0xde5a, 0xde5f, PDF_CMAP_TABLE, 1345 },
	{ 0xde60, 0xde6a, PDF_CMAP_RANGE, 19715 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 7728 },
	{ 0xde6c, 0xde6e, PDF_CMAP_RANGE, 19726 },
	{ 0xde6f, 0xde72, PDF_CMAP_TABLE, 1351 },
	{ 0xde73, 0xde7e, PDF_CMAP_RANGE, 19729 },
	{ 0xde80, 0xde91, PDF_CMAP_RANGE, 19741 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 9851 },
	{ 0xde93, 0xde9e, PDF_CMAP_RANGE, 19759 },
	{ 0xde9f, 0xdea0, PDF_CMAP_TABLE, 1355 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 23630 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 24890 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 26150 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 27410 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 28670 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 29771 },
	{ 0xdf40, 0xdf42, PDF_CMAP_TABLE, 1357 },
	{ 0xdf43, 0xdf4c, PDF_CMAP_RANGE, 19773 },
	{ 0xdf4d, 0xdf4d, PDF_CMAP_SINGLE, 8123 },
	{ 0xdf4e, 0xdf5b, PDF_CMAP_RANGE, 19783 },
	{ 0xdf5c, 0xdf60, PDF_CMAP_TABLE, 1360 },
	{ 0xdf61, 0xdf63, PDF_CMAP_RANGE, 19798 },
	{ 0xdf64, 0xdf68, PDF_CMAP_TABLE, 1365 },
	{ 0xdf69, 0xdf6c, PDF_CMAP_RANGE, 19803 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 8499 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_RANGE, 19807 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 7812 },
	{ 0xdf75, 0xdf76, PDF_CMAP_RANGE, 19813 },
	{ 0xdf77, 0xdf7e, PDF_CMAP_TABLE, 1370 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 8021 },
	{ 0xdf81, 0xdf82, PDF_CMAP_RANGE, 19818 },
	{ 0xdf83, 0xdf85, PDF_CMAP_TABLE, 1378 },
	{ 0xdf86, 0xdf88, PDF_CMAP_RANGE, 19821 },
	{ 0xdf89, 0xdf8a, PDF_CMAP_TABLE, 1381 },
	{ 0xdf8b, 0xdfa0, PDF_CMAP_RANGE, 19824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 23640 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 24900 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 26160 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 27420 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 28680 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 29781 },
	{ 0xe040, 0xe04f, PDF_CMAP_RANGE, 19846 },
	{ 0xe050, 0xe050, PDF_CMAP_SINGLE, 8990 },
	{ 0xe051, 0xe05c, PDF_CMAP_RANGE, 19862 },
	{ 0xe05d, 0xe05d, PDF_CMAP_SINGLE, 8755 },
	{ 0xe05e, 0xe068, PDF_CMAP_RANGE, 19874 },
	{ 0xe069, 0xe069, PDF_CMAP_SINGLE, 8992 },
	{ 0xe06a, 0xe06b, PDF_CMAP_RANGE, 19885 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 8647 },
	{ 0xe06d, 0xe074, PDF_CMAP_RANGE, 19887 },
	{ 0xe075, 0xe079, PDF_CMAP_TABLE, 1383 },
	{ 0xe07a, 0xe07e, PDF_CMAP_RANGE, 19897 },
	{ 0xe080, 0xe086, PDF_CMAP_RANGE, 19902 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 7867 },
	{ 0xe088, 0xe08c, PDF_CMAP_RANGE, 19909 },
	{ 0xe08d, 0xe094, PDF_CMAP_TABLE, 1388 },
	{ 0xe095, 0xe096, PDF_CMAP_RANGE, 19917 },
	{ 0xe097, 0xe097, PDF_CMAP_SINGLE, 8987 },
	{ 0xe098, 0xe0a0, PDF_CMAP_RANGE, 19919 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 23650 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 24910 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 26170 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 27430 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 28690 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 29791 },
	{ 0xe140, 0xe141, PDF_CMAP_RANGE, 19928 },
	{ 0xe142, 0xe142, PDF_CMAP_SINGLE, 8993 },
	{ 0xe143, 0xe163, PDF_CMAP_RANGE, 19930 },
	{ 0xe164, 0xe164, PDF_CMAP_SINGLE, 8789 },
	{ 0xe165, 0xe167, PDF_CMAP_RANGE, 19963 },
	{ 0xe168, 0xe168, PDF_CMAP_SINGLE, 7823 },
	{ 0xe169, 0xe173, PDF_CMAP_RANGE, 19966 },
	{ 0xe174, 0xe175, PDF_CMAP_TABLE, 1396 },
	{ 0xe176, 0xe17e, PDF_CMAP_RANGE, 19977 },
	{ 0xe180, 0xe183, PDF_CMAP_RANGE, 19986 },
	{ 0xe184, 0xe189, PDF_CMAP_TABLE, 1398 },
	{ 0xe18a, 0xe18b, PDF_CMAP_RANGE, 19992 },
	{ 0xe18c, 0xe18d, PDF_CMAP_TABLE, 1404 },
	{ 0xe18e, 0xe190, PDF_CMAP_RANGE, 9459 },
	{ 0xe191, 0xe195, PDF_CMAP_TABLE, 1406 },
	{ 0xe196, 0xe197, PDF_CMAP_RANGE, 19996 },
	{ 0xe198, 0xe198, PDF_CMAP_SINGLE, 8832 },
	{ 0xe199, 0xe19d, PDF_CMAP_RANGE, 19998 },
	{ 0xe19e, 0xe1a0, PDF_CMAP_TABLE, 1411 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 23660 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 24920 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 26180 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 27440 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 28700 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 29801 },
	{ 0xe240, 0xe243, PDF_CMAP_TABLE, 1414 },
	{ 0xe244, 0xe24e, PDF_CMAP_RANGE, 20006 },
	{ 0xe24f, 0xe254, PDF_CMAP_TABLE, 1418 },
	{ 0xe255, 0xe259, PDF_CMAP_RANGE, 20019 },
	{ 0xe25a, 0xe25b, PDF_CMAP_TABLE, 1424 },
	{ 0xe25c, 0xe25d, PDF_CMAP_RANGE, 20024 },
	{ 0xe25e, 0xe25e, PDF_CMAP_SINGLE, 9478 },
	{ 0xe25f, 0xe261, PDF_CMAP_RANGE, 20026 },
	{ 0xe262, 0xe263, PDF_CMAP_TABLE, 1426 },
	{ 0xe264, 0xe266, PDF_CMAP_RANGE, 20029 },
	{ 0xe267, 0xe26b, PDF_CMAP_TABLE, 1428 },
	{ 0xe26c, 0xe26d, PDF_CMAP_RANGE, 20033 },
	{ 0xe26e, 0xe26f, PDF_CMAP_TABLE, 1433 },
	{ 0xe270, 0xe277, PDF_CMAP_RANGE, 20035 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 8149 },
	{ 0xe279, 0xe27c, PDF_CMAP_RANGE, 20043 },
	{ 0xe27d, 0xe27e, PDF_CMAP_TABLE, 1435 },
	{ 0xe280, 0xe282, PDF_CMAP_TABLE, 1437 },
	{ 0xe283, 0xe288, PDF_CMAP_RANGE, 20048 },
	{ 0xe289, 0xe28b, PDF_CMAP_TABLE, 1440 },
	{ 0xe28c, 0xe28d, PDF_CMAP_RANGE, 20055 },
	{ 0xe28e, 0xe28f, PDF_CMAP_TABLE, 1443 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 20057 },
	{ 0xe292, 0xe295, PDF_CMAP_TABLE, 1445 },
	{ 0xe296, 0xe297, PDF_CMAP_RANGE, 20059 },
	{ 0xe298, 0xe29b, PDF_CMAP_TABLE, 1449 },
	{ 0xe29c, 0xe29f, PDF_CMAP_RANGE, 20061 },
	{ 0xe2a0, 0xe2a0, PDF_CMAP_SINGLE, 9473 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 23670 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 24930 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 26190 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 27450 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 28710 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 29811 },
	{ 0xe340, 0xe341, PDF_CMAP_RANGE, 20065 },
	{ 0xe342, 0xe343, PDF_CMAP_TABLE, 1453 },
	{ 0xe344, 0xe346, PDF_CMAP_RANGE, 20067 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 9496 },
	{ 0xe348, 0xe34a, PDF_CMAP_RANGE, 20070 },
	{ 0xe34b, 0xe34b, PDF_CMAP_SINGLE, 7766 },
	{ 0xe34c, 0xe34e, PDF_CMAP_RANGE, 20073 },
	{ 0xe34f, 0xe351, PDF_CMAP_TABLE, 1455 },
	{ 0xe352, 0xe353, PDF_CMAP_RANGE, 20077 },
	{ 0xe354, 0xe355, PDF_CMAP_TABLE, 1458 },
	{ 0xe356, 0xe357, PDF_CMAP_RANGE, 20079 },
	{ 0xe358, 0xe358, PDF_CMAP_SINGLE, 9488 },
	{ 0xe359, 0xe35b, PDF_CMAP_RANGE, 20081 },
	{ 0xe35c, 0xe35c, PDF_CMAP_SINGLE, 7765 },
	{ 0xe35d, 0xe35f, PDF_CMAP_RANGE, 20084 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 9482 },
	{ 0xe361, 0xe365, PDF_CMAP_RANGE, 20087 },
	{ 0xe366, 0xe367, PDF_CMAP_RANGE, 9489 },
	{ 0xe368, 0xe370, PDF_CMAP_RANGE, 20092 },
	{ 0xe371, 0xe374, PDF_CMAP_TABLE, 1460 },
	{ 0xe375, 0xe377, PDF_CMAP_RANGE, 20102 },
	{ 0xe378, 0xe379, PDF_CMAP_TABLE, 1464 },
	{ 0xe37a, 0xe37b, PDF_CMAP_RANGE, 20105 },
	{ 0xe37c, 0xe37e, PDF_CMAP_TABLE, 1466 },
	{ 0xe380, 0xe389, PDF_CMAP_RANGE, 20108 },
	{ 0xe38a, 0xe38c, PDF_CMAP_TABLE, 1469 },
	{ 0xe38d, 0xe38e, PDF_CMAP_RANGE, 20119 },
	{ 0xe38f, 0xe395, PDF_CMAP_TABLE, 1472 },
	{ 0xe396, 0xe398, PDF_CMAP_RANGE, 20124 },
	{ 0xe399, 0xe399, PDF_CMAP_SINGLE, 9501 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 20127 },
	{ 0xe39c, 0xe3a0, PDF_CMAP_TABLE, 1479 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 23680 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 24940 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 26200 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 27460 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 28720 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 29821 },
	{ 0xe440, 0xe444, PDF_CMAP_TABLE, 1484 },
	{ 0xe445, 0xe447, PDF_CMAP_RANGE, 20131 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 9509 },
	{ 0xe449, 0xe44d, PDF_CMAP_RANGE, 20134 },
	{ 0xe44e, 0xe453, PDF_CMAP_TABLE, 1489 },
	{ 0xe454, 0xe457, PDF_CMAP_RANGE, 20141 },
	{ 0xe458, 0xe45e, PDF_CMAP_TABLE, 1495 },
	{ 0xe45f, 0xe461, PDF_CMAP_RANGE, 20148 },
	{ 0xe462, 0xe462, PDF_CMAP_SINGLE, 9513 },
	{ 0xe463, 0xe464, PDF_CMAP_RANGE, 20151 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 9505 },
	{ 0xe466, 0xe467, PDF_CMAP_RANGE, 20153 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 7935 },
	{ 0xe469, 0xe472, PDF_CMAP_RANGE, 20155 },
	{ 0xe473, 0xe475, PDF_CMAP_TABLE, 1502 },
	{ 0xe476, 0xe478, PDF_CMAP_RANGE, 20166 },
	{ 0xe479, 0xe47e, PDF_CMAP_TABLE, 1505 },
	{ 0xe480, 0xe481, PDF_CMAP_TABLE, 1511 },
	{ 0xe482, 0xe483, PDF_CMAP_RANGE, 20171 },
	{ 0xe484, 0xe488, PDF_CMAP_TABLE, 1513 },
	{ 0xe489, 0xe48c, PDF_CMAP_RANGE, 20173 },
	{ 0xe48d, 0xe48f, PDF_CMAP_TABLE, 1518 },
	{ 0xe490, 0xe492, PDF_CMAP_RANGE, 20178 },
	{ 0xe493, 0xe493, PDF_CMAP_SINGLE, 7960 },
	{ 0xe494, 0xe497, PDF_CMAP_RANGE, 20181 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 9547 },
	{ 0xe499, 0xe49c, PDF_CMAP_RANGE, 20185 },
	{ 0xe49d, 0xe49d, PDF_CMAP_SINGLE, 9543 },
	{ 0xe49e, 0xe49f, PDF_CMAP_RANGE, 9551 },
	{ 0xe4a0, 0xe4a0, PDF_CMAP_SINGLE, 20189 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 23690 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 24950 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 26210 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 27470 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 28730 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 29831 },
	{ 0xe540, 0xe545, PDF_CMAP_RANGE, 20190 },
	{ 0xe546, 0xe548, PDF_CMAP_TABLE, 1521 },
	{ 0xe549, 0xe54a, PDF_CMAP_RANGE, 20197 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 9548 },
	{ 0xe54c, 0xe54d, PDF_CMAP_RANGE, 20199 },
	{ 0xe54e, 0xe551, PDF_CMAP_TABLE, 1524 },
	{ 0xe552, 0xe554, PDF_CMAP_RANGE, 20201 },
	{ 0xe555, 0xe558, PDF_CMAP_TABLE, 1528 },
	{ 0xe559, 0xe55b, PDF_CMAP_RANGE, 20205 },
	{ 0xe55c, 0xe55e, PDF_CMAP_TABLE, 1532 },
	{ 0xe55f, 0xe560, PDF_CMAP_RANGE, 20209 },
	{ 0xe561, 0xe561, PDF_CMAP_SINGLE, 8619 },
	{ 0xe562, 0xe563, PDF_CMAP_RANGE, 20211 },
	{ 0xe564, 0xe565, PDF_CMAP_TABLE, 1535 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 20213 },
	{ 0xe568, 0xe569, PDF_CMAP_TABLE, 1537 },
	{ 0xe56a, 0xe56b, PDF_CMAP_RANGE, 20215 },
	{ 0xe56c, 0xe56e, PDF_CMAP_TABLE, 1539 },
	{ 0xe56f, 0xe574, PDF_CMAP_RANGE, 20218 },
	{ 0xe575, 0xe578, PDF_CMAP_TABLE, 1542 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 20225 },
	{ 0xe57b, 0xe57c, PDF_CMAP_TABLE, 1546 },
	{ 0xe57d, 0xe57e, PDF_CMAP_RANGE, 20227 },
	{ 0xe580, 0xe583, PDF_CMAP_TABLE, 1548 },
	{ 0xe584, 0xe589, PDF_CMAP_RANGE, 20231 },
	{ 0xe58a, 0xe58a, PDF_CMAP_SINGLE, 9558 },
	{ 0xe58b, 0xe58d, PDF_CMAP_RANGE, 20237 },
	{ 0xe58e, 0xe58e, PDF_CMAP_SINGLE, 8808 },
	{ 0xe58f, 0xe590, PDF_CMAP_RANGE, 20240 },
	{ 0xe591, 0xe591, PDF_CMAP_SINGLE, 7894 },
	{ 0xe592, 0xe599, PDF_CMAP_RANGE, 20242 },
	{ 0xe59a, 0xe59b, PDF_CMAP_TABLE, 1552 },
	{ 0xe59c, 0xe59e, PDF_CMAP_RANGE, 20250 },
	{ 0xe59f, 0xe5a0, PDF_CMAP_TABLE, 1554 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 23700 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 24960 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 26220 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 27480 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 28740 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 29841 },
	{ 0xe640, 0xe640, PDF_CMAP_SINGLE, 8412 },
	{ 0xe641, 0xe643, PDF_CMAP_RANGE, 20254 },
	{ 0xe644, 0xe644, PDF_CMAP_SINGLE, 9561 },
	{ 0xe645, 0xe648, PDF_CMAP_RANGE, 20257 },
	{ 0xe649, 0xe64a, PDF_CMAP_TABLE, 1556 },
	{ 0xe64b, 0xe64d, PDF_CMAP_RANGE, 20261 },
	{ 0xe64e, 0xe64e, PDF_CMAP_SINGLE, 8829 },
	{ 0xe64f, 0xe651, PDF_CMAP_RANGE, 20264 },
	{ 0xe652, 0xe652, PDF_CMAP_SINGLE, 9598 },
	{ 0xe653, 0xe655, PDF_CMAP_RANGE, 20267 },
	{ 0xe656, 0xe658, PDF_CMAP_TABLE, 1558 },
	{ 0xe659, 0xe65a, PDF_CMAP_RANGE, 20271 },
	{ 0xe65b, 0xe65b, PDF_CMAP_SINGLE, 9566 },
	{ 0xe65c, 0xe65d, PDF_CMAP_RANGE, 20273 },
	{ 0xe65e, 0xe65e, PDF_CMAP_SINGLE, 7732 },
	{ 0xe65f, 0xe668, PDF_CMAP_RANGE, 20275 },
	{ 0xe669, 0xe66b, PDF_CMAP_TABLE, 1561 },
	{ 0xe66c, 0xe674, PDF_CMAP_RANGE, 20286 },
	{ 0xe675, 0xe676, PDF_CMAP_TABLE, 1564 },
	{ 0xe677, 0xe678, PDF_CMAP_RANGE, 20295 },
	{ 0xe679, 0xe67e, PDF_CMAP_TABLE, 1566 },
	{ 0xe680, 0xe684, PDF_CMAP_TABLE, 1572 },
	{ 0xe685, 0xe686, PDF_CMAP_RANGE, 20301 },
	{ 0xe687, 0xe689, PDF_CMAP_TABLE, 1577 },
	{ 0xe68a, 0xe68b, PDF_CMAP_RANGE, 20304 },
	{ 0xe68c, 0xe68c, PDF_CMAP_SINGLE, 9569 },
	{ 0xe68d, 0xe692, PDF_CMAP_RANGE, 20306 },
	{ 0xe693, 0xe693, PDF_CMAP_SINGLE, 9570 },
	{ 0xe694, 0xe696, PDF_CMAP_RANGE, 20312 },
	{ 0xe697, 0xe697, PDF_CMAP_SINGLE, 9580 },
	{ 0xe698, 0xe69a, PDF_CMAP_RANGE, 20315 },
	{ 0xe69b, 0xe69c, PDF_CMAP_TABLE, 1580 },
	{ 0xe69d, 0xe69e, PDF_CMAP_RANGE, 20318 },
	{ 0xe69f, 0xe6a0, PDF_CMAP_TABLE, 1582 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 23710 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 24970 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 26230 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 27490 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 28750 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 29851 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 20320 },
	{ 0xe743, 0xe743, PDF_CMAP_SINGLE, 9582 },
	{ 0xe744, 0xe747, PDF_CMAP_RANGE, 20323 },
	{ 0xe748, 0xe749, PDF_CMAP_TABLE, 1584 },
	{ 0xe74a, 0xe74c, PDF_CMAP_RANGE, 20327 },
	{ 0xe74d, 0xe74e, PDF_CMAP_RANGE, 9576 },
	{ 0xe74f, 0xe755, PDF_CMAP_TABLE, 1586 },
	{ 0xe756, 0xe758, PDF_CMAP_RANGE, 20332 },
	{ 0xe759, 0xe759, PDF_CMAP_SINGLE, 9757 },
	{ 0xe75a, 0xe765, PDF_CMAP_RANGE, 20335 },
	{ 0xe766, 0xe76a, PDF_CMAP_TABLE, 1593 },
	{ 0xe76b, 0xe773, PDF_CMAP_RANGE, 20349 },
	{ 0xe774, 0xe774, PDF_CMAP_SINGLE, 9506 },
	{ 0xe775, 0xe77b, PDF_CMAP_RANGE, 20358 },
	{ 0xe77c, 0xe77c, PDF_CMAP_SINGLE, 9523 },
	{ 0xe77d, 0xe77e, PDF_CMAP_RANGE, 20365 },
	{ 0xe780, 0xe781, PDF_CMAP_RANGE, 20367 },
	{ 0xe782, 0xe786, PDF_CMAP_TABLE, 1598 },
	{ 0xe787, 0xe789, PDF_CMAP_RANGE, 20370 },
	{ 0xe78a, 0xe78b, PDF_CMAP_TABLE, 1603 },
	{ 0xe78c, 0xe78e, PDF_CMAP_RANGE, 20373 },
	{ 0xe78f, 0xe78f, PDF_CMAP_SINGLE, 9584 },
	{ 0xe790, 0xe791, PDF_CMAP_RANGE, 20376 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 9589 },
	{ 0xe793, 0xe797, PDF_CMAP_RANGE, 20378 },
	{ 0xe798, 0xe799, PDF_CMAP_RANGE, 9537 },
	{ 0xe79a, 0xe79a, PDF_CMAP_SINGLE, 9565 },
	{ 0xe79b, 0xe79f, PDF_CMAP_RANGE, 20383 },
	{ 0xe7a0, 0xe7a0, PDF_CMAP_SINGLE, 8218 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 23720 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 24980 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 26240 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 27500 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 28760 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 29861 },
	{ 0xe840, 0xe842, PDF_CMAP_RANGE, 20388 },
	{ 0xe843, 0xe846, PDF_CMAP_TABLE, 1605 },
	{ 0xe847, 0xe848, PDF_CMAP_RANGE, 20392 },
	{ 0xe849, 0xe84b, PDF_CMAP_TABLE, 1609 },
	{ 0xe84c, 0xe84e, PDF_CMAP_RANGE, 20395 },
	{ 0xe84f, 0xe84f, PDF_CMAP_SINGLE, 9595 },
	{ 0xe850, 0xe853, PDF_CMAP_RANGE, 20398 },
	{ 0xe854, 0xe854, PDF_CMAP_SINGLE, 8867 },
	{ 0xe855, 0xe859, PDF_CMAP_RANGE, 20402 },
	{ 0xe85a, 0xe85c, PDF_CMAP_TABLE, 1612 },
	{ 0xe85d, 0xe861, PDF_CMAP_RANGE, 20408 },
	{ 0xe862, 0xe864, PDF_CMAP_TABLE, 1615 },
	{ 0xe865, 0xe86f, PDF_CMAP_RANGE, 20414 },
	{ 0xe870, 0xe870, PDF_CMAP_SINGLE, 9492 },
	{ 0xe871, 0xe872, PDF_CMAP_RANGE, 20425 },
	{ 0xe873, 0xe875, PDF_CMAP_TABLE, 1618 },
	{ 0xe876, 0xe87b, PDF_CMAP_RANGE, 20428 },
	{ 0xe87c, 0xe87c, PDF_CMAP_SINGLE, 9588 },
	{ 0xe87d, 0xe87e, PDF_CMAP_RANGE, 20434 },
	{ 0xe880, 0xe882, PDF_CMAP_TABLE, 1621 },
	{ 0xe883, 0xe886, PDF_CMAP_RANGE, 20437 },
	{ 0xe887, 0xe889, PDF_CMAP_TABLE, 1624 },
	{ 0xe88a, 0xe88b, PDF_CMAP_RANGE, 20442 },
	{ 0xe88c, 0xe88f, PDF_CMAP_TABLE, 1627 },
	{ 0xe890, 0xe8a0, PDF_CMAP_RANGE, 20444 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 23730 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 24990 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 26250 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 27510 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 28770 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 29871 },
	{ 0xe940, 0xe94b, PDF_CMAP_RANGE, 20461 },
	{ 0xe94c, 0xe94c, PDF_CMAP_SINGLE, 7797 },
	{ 0xe94d, 0xe953, PDF_CMAP_RANGE, 20473 },
	{ 0xe954, 0xe957, PDF_CMAP_TABLE, 1631 },
	{ 0xe958, 0xe959, PDF_CMAP_RANGE, 20481 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 9152 },
	{ 0xe95b, 0xe95c, PDF_CMAP_RANGE, 20483 },
	{ 0xe95d, 0xe968, PDF_CMAP_TABLE, 1635 },
	{ 0xe969, 0xe96b, PDF_CMAP_RANGE, 20489 },
	{ 0xe96c, 0xe96c, PDF_CMAP_SINGLE, 8809 },
	{ 0xe96d, 0xe974, PDF_CMAP_RANGE, 20492 },
	{ 0xe975, 0xe979, PDF_CMAP_TABLE, 1647 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 20501 },
	{ 0xe97c, 0xe97e, PDF_CMAP_TABLE, 1652 },
	{ 0xe980, 0xe982, PDF_CMAP_TABLE, 1655 },
	{ 0xe983, 0xe986, PDF_CMAP_RANGE, 20504 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 8783 },
	{ 0xe988, 0xe98a, PDF_CMAP_RANGE, 20508 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 9163 },
	{ 0xe98c, 0xe98d, PDF_CMAP_RANGE, 20511 },
	{ 0xe98e, 0xe994, PDF_CMAP_TABLE, 1658 },
	{ 0xe995, 0xe997, PDF_CMAP_RANGE, 20514 },
	{ 0xe998, 0xe998, PDF_CMAP_SINGLE, 9168 },
	{ 0xe999, 0xe99a, PDF_CMAP_RANGE, 20517 },
	{ 0xe99b, 0xe9a0, PDF_CMAP_TABLE, 1665 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 23740 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 25000 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 26260 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 27520 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 28780 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 29881 },
	{ 0xea40, 0xea40, PDF_CMAP_SINGLE, 8184 },
	{ 0xea41, 0xea43, PDF_CMAP_RANGE, 20521 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 9171 },
	{ 0xea45, 0xea47, PDF_CMAP_RANGE, 20524 },
	{ 0xea48, 0xea4a, PDF_CMAP_TABLE, 1671 },
	{ 0xea4b, 0xea4f, PDF_CMAP_RANGE, 20527 },
	{ 0xea50, 0xea52, PDF_CMAP_TABLE, 1674 },
	{ 0xea53, 0xea54, PDF_CMAP_RANGE, 20533 },
	{ 0xea55, 0xea56, PDF_CMAP_TABLE, 1677 },
	{ 0xea57, 0xea58, PDF_CMAP_RANGE, 20535 },
	{ 0xea59, 0xea59, PDF_CMAP_SINGLE, 9157 },
	{ 0xea5a, 0xea7e, PDF_CMAP_RANGE, 20537 },
	{ 0xea80, 0xea80, PDF_CMAP_SINGLE, 8986 },
	{ 0xea81, 0xea83, PDF_CMAP_RANGE, 20574 },
	{ 0xea84, 0xea84, PDF_CMAP_SINGLE, 8468 },
	{ 0xea85, 0xea86, PDF_CMAP_RANGE, 20577 },
	{ 0xea87, 0xea87, PDF_CMAP_SINGLE, 8836 },
	{ 0xea88, 0xea8d, PDF_CMAP_RANGE, 20579 },
	{ 0xea8e, 0xea91, PDF_CMAP_TABLE, 1679 },
	{ 0xea92, 0xea95, PDF_CMAP_RANGE, 20586 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 8705 },
	{ 0xea97, 0xea9f, PDF_CMAP_RANGE, 20590 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 7897 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 23750 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 25010 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 26270 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 27530 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 28790 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 29891 },
	{ 0xeb40, 0xeb41, PDF_CMAP_TABLE, 1683 },
	{ 0xeb42, 0xeb44, PDF_CMAP_RANGE, 20600 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 8786 },
	{ 0xeb46, 0xeb47, PDF_CMAP_RANGE, 20603 },
	{ 0xeb48, 0xeb48, PDF_CMAP_SINGLE, 8057 },
	{ 0xeb49, 0xeb52, PDF_CMAP_RANGE, 20605 },
	{ 0xeb53, 0xeb55, PDF_CMAP_TABLE, 1685 },
	{ 0xeb56, 0xeb5a, PDF_CMAP_RANGE, 20616 },
	{ 0xeb5b, 0xeb5d, PDF_CMAP_TABLE, 1688 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 20622 },
	{ 0xeb60, 0xeb62, PDF_CMAP_TABLE, 1691 },
	{ 0xeb63, 0xeb6c, PDF_CMAP_RANGE, 20625 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 8534 },
	{ 0xeb6e, 0xeb6f, PDF_CMAP_RANGE, 20635 },
	{ 0xeb70, 0xeb73, PDF_CMAP_TABLE, 1694 },
	{ 0xeb74, 0xeb77, PDF_CMAP_RANGE, 20638 },
	{ 0xeb78, 0xeb79, PDF_CMAP_TABLE, 1698 },
	{ 0xeb7a, 0xeb7e, PDF_CMAP_RANGE, 20642 },
	{ 0xeb80, 0xeb84, PDF_CMAP_RANGE, 20647 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 8784 },
	{ 0xeb86, 0xeb89, PDF_CMAP_RANGE, 20652 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 7875 },
	{ 0xeb8b, 0xeba0, PDF_CMAP_RANGE, 20656 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 23760 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 25020 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 26280 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 27540 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 28800 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 29901 },
	{ 0xec40, 0xec45, PDF_CMAP_RANGE, 20678 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8616 },
	{ 0xec47, 0xec55, PDF_CMAP_RANGE, 20684 },
	{ 0xec56, 0xec56, PDF_CMAP_SINGLE, 9741 },
	{ 0xec57, 0xec59, PDF_CMAP_RANGE, 20699 },
	{ 0xec5a, 0xec5c, PDF_CMAP_TABLE, 1700 },
	{ 0xec5d, 0xec5f, PDF_CMAP_RANGE, 20703 },
	{ 0xec60, 0xec60, PDF_CMAP_SINGLE, 8242 },
	{ 0xec61, 0xec6d, PDF_CMAP_RANGE, 20706 },
	{ 0xec6e, 0xec6e, PDF_CMAP_SINGLE, 9739 },
	{ 0xec6f, 0xec75, PDF_CMAP_RANGE, 20719 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8899 },
	{ 0xec77, 0xec7e, PDF_CMAP_RANGE, 20726 },
	{ 0xec80, 0xec95, PDF_CMAP_RANGE, 20734 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 7971 },
	{ 0xec97, 0xeca0, PDF_CMAP_RANGE, 20756 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 23770 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 25030 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 26290 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 27550 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 28810 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 29911 },
	{ 0xed40, 0xed45, PDF_CMAP_RANGE, 20766 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 9877 },
	{ 0xed47, 0xed57, PDF_CMAP_RANGE, 20772 },
	{ 0xed58, 0xed58, PDF_CMAP_SINGLE, 9822 },
	{ 0xed59, 0xed5d, PDF_CMAP_RANGE, 20789 },
	{ 0xed5e, 0xed5e, PDF_CMAP_SINGLE, 9821 },
	{ 0xed5f, 0xed60, PDF_CMAP_RANGE, 20794 },
	{ 0xed61, 0xed61, PDF_CMAP_SINGLE, 9874 },
	{ 0xed62, 0xed63, PDF_CMAP_RANGE, 20796 },
	{ 0xed64, 0xed67, PDF_CMAP_TABLE, 1703 },
	{ 0xed68, 0xed6d, PDF_CMAP_RANGE, 20799 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 8000 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 20805 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 9317 },
	{ 0xed75, 0xed76, PDF_CMAP_RANGE, 20810 },
	{ 0xed77, 0xed79, PDF_CMAP_TABLE, 1707 },
	{ 0xed7a, 0xed7e, PDF_CMAP_RANGE, 20813 },
	{ 0xed80, 0xed90, PDF_CMAP_RANGE, 20818 },
	{ 0xed91, 0xeda0, PDF_CMAP_TABLE, 1710 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 23780 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 25040 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 26300 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 27560 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 28820 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 29921 },
	{ 0xee39, 0xee39, PDF_CMAP_SINGLE, 22529 },
	{ 0xee40, 0xee44, PDF_CMAP_TABLE, 1726 },
	{ 0xee45, 0xee47, PDF_CMAP_RANGE, 20840 },
	{ 0xee48, 0xee49, PDF_CMAP_TABLE, 1731 },
	{ 0xee4a, 0xee4c, PDF_CMAP_RANGE, 20843 },
	{ 0xee4d, 0xee4d, PDF_CMAP_SINGLE, 9673 },
	{ 0xee4e, 0xee51, PDF_CMAP_RANGE, 20846 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 9672 },
	{ 0xee53, 0xee54, PDF_CMAP_RANGE, 20850 },
	{ 0xee55, 0xee57, PDF_CMAP_TABLE, 1733 },
	{ 0xee58, 0xee5d, PDF_CMAP_RANGE, 20853 },
	{ 0xee5e, 0xee5e, PDF_CMAP_SINGLE, 8573 },
	{ 0xee5f, 0xee60, PDF_CMAP_RANGE, 20859 },
	{ 0xee61, 0xee61, PDF_CMAP_SINGLE, 8062 },
	{ 0xee62, 0xee67, PDF_CMAP_RANGE, 20861 },
	{ 0xee68, 0xee69, PDF_CMAP_TABLE, 1736 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 20867 },
	{ 0xee6c, 0xee6e, PDF_CMAP_TABLE, 1738 },
	{ 0xee6f, 0xee76, PDF_CMAP_RANGE, 20870 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 8154 },
	{ 0xee78, 0xee7c, PDF_CMAP_RANGE, 20878 },
	{ 0xee7d, 0xee7e, PDF_CMAP_TABLE, 1741 },
	{ 0xee80, 0xee80, PDF_CMAP_SINGLE, 9677 },
	{ 0xee81, 0xee84, PDF_CMAP_RANGE, 20883 },
	{ 0xee85, 0xee86, PDF_CMAP_TABLE, 1743 },
	{ 0xee87, 0xee89, PDF_CMAP_RANGE, 20887 },
	{ 0xee8a, 0xee8d, PDF_CMAP_TABLE, 1745 },
	{ 0xee8e, 0xee8f, PDF_CMAP_RANGE, 20891 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 8200 },
	{ 0xee91, 0xee93, PDF_CMAP_RANGE, 20893 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 9680 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 20896 },
	{ 0xee97, 0xee99, PDF_CMAP_TABLE, 1749 },
	{ 0xee9a, 0xee9c, PDF_CMAP_RANGE, 20899 },
	{ 0xee9d, 0xee9e, PDF_CMAP_TABLE, 1752 },
	{ 0xee9f, 0xeea0, PDF_CMAP_RANGE, 20902 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 22530 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 23790 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 25050 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 26310 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 27570 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 28830 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 29931 },
	{ 0xef40, 0xef45, PDF_CMAP_TABLE, 1754 },
	{ 0xef46, 0xef4b, PDF_CMAP_RANGE, 20905 },
	{ 0xef4c, 0xef4c, PDF_CMAP_SINGLE, 7936 },
	{ 0xef4d, 0xef51, PDF_CMAP_RANGE, 20911 },
	{ 0xef52, 0xef53, PDF_CMAP_RANGE, 9420 },
	{ 0xef54, 0xef57, PDF_CMAP_TABLE, 1760 },
	{ 0xef58, 0xef59, PDF_CMAP_RANGE, 20918 },
	{ 0xef5a, 0xef5a, PDF_CMAP_SINGLE, 9422 },
	{ 0xef5b, 0xef5f, PDF_CMAP_RANGE, 20920 },
	{ 0xef60, 0xef60, PDF_CMAP_SINGLE, 9423 },
	{ 0xef61, 0xef67, PDF_CMAP_RANGE, 20925 },
	{ 0xef68, 0xef6c, PDF_CMAP_TABLE, 1764 },
	{ 0xef6d, 0xef76, PDF_CMAP_RANGE, 20934 },
	{ 0xef77, 0xef77, PDF_CMAP_SINGLE, 7924 },
	{ 0xef78, 0xef79, PDF_CMAP_RANGE, 20944 },
	{ 0xef7a, 0xef7c, PDF_CMAP_TABLE, 1769 },
	{ 0xef7d, 0xef7e, PDF_CMAP_RANGE, 20947 },
	{ 0xef80, 0xef81, PDF_CMAP_RANGE, 20949 },
	{ 0xef82, 0xef82, PDF_CMAP_SINGLE, 9117 },
	{ 0xef83, 0xef84, PDF_CMAP_RANGE, 9119 },
	{ 0xef85, 0xef88, PDF_CMAP_TABLE, 1772 },
	{ 0xef89, 0xef8a, PDF_CMAP_RANGE, 20953 },
	{ 0xef8b, 0xef8d, PDF_CMAP_TABLE, 1776 },
	{ 0xef8e, 0xef94, PDF_CMAP_RANGE, 20956 },
	{ 0xef95, 0xef97, PDF_CMAP_TABLE, 1779 },
	{ 0xef98, 0xef9b, PDF_CMAP_RANGE, 20963 },
	{ 0xef9c, 0xef9e, PDF_CMAP_TABLE, 1782 },
	{ 0xef9f, 0xefa0, PDF_CMAP_RANGE, 20968 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 22540 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 23800 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 25060 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 26320 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 27580 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 28840 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 29941 },
	{ 0xf040, 0xf044, PDF_CMAP_TABLE, 1785 },
	{ 0xf045, 0xf046, PDF_CMAP_RANGE, 20972 },
	{ 0xf047, 0xf049, PDF_CMAP_TABLE, 1790 },
	{ 0xf04a, 0xf04d, PDF_CMAP_RANGE, 20974 },
	{ 0xf04e, 0xf04e, PDF_CMAP_SINGLE, 8760 },
	{ 0xf04f, 0xf050, PDF_CMAP_RANGE, 20978 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 9125 },
	{ 0xf052, 0xf053, PDF_CMAP_RANGE, 20980 },
	{ 0xf054, 0xf054, PDF_CMAP_SINGLE, 8090 },
	{ 0xf055, 0xf056, PDF_CMAP_RANGE, 20982 },
	{ 0xf057, 0xf057, PDF_CMAP_SINGLE, 8643 },
	{ 0xf058, 0xf05d, PDF_CMAP_RANGE, 20984 },
	{ 0xf05e, 0xf05e, PDF_CMAP_SINGLE, 7982 },
	{ 0xf05f, 0xf067, PDF_CMAP_RANGE, 20990 },
	{ 0xf068, 0xf068, PDF_CMAP_SINGLE, 9116 },
	{ 0xf069, 0xf06b, PDF_CMAP_RANGE, 20999 },
	{ 0xf06c, 0xf06c, PDF_CMAP_SINGLE, 9126 },
	{ 0xf06d, 0xf070, PDF_CMAP_RANGE, 21002 },
	{ 0xf071, 0xf074, PDF_CMAP_TABLE, 1793 },
	{ 0xf075, 0xf077, PDF_CMAP_RANGE, 21007 },
	{ 0xf078, 0xf07a, PDF_CMAP_TABLE, 1797 },
	{ 0xf07b, 0xf07c, PDF_CMAP_RANGE, 21011 },
	{ 0xf07d, 0xf07e, PDF_CMAP_RANGE, 9129 },
	{ 0xf080, 0xf082, PDF_CMAP_TABLE, 1800 },
	{ 0xf083, 0xf086, PDF_CMAP_RANGE, 21013 },
	{ 0xf087, 0xf088, PDF_CMAP_TABLE, 1803 },
	{ 0xf089, 0xf08a, PDF_CMAP_RANGE, 21017 },
	{ 0xf08b, 0xf08b, PDF_CMAP_SINGLE, 9830 },
	{ 0xf08c, 0xf08f, PDF_CMAP_RANGE, 21019 },
	{ 0xf090, 0xf092, PDF_CMAP_TABLE, 1805 },
	{ 0xf093, 0xf095, PDF_CMAP_RANGE, 21024 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 9133 },
	{ 0xf097, 0xf0a0, PDF_CMAP_RANGE, 21027 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 22550 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 23810 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 25070 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 26330 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 27590 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 28850 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 29951 },
	{ 0xf140, 0xf151, PDF_CMAP_RANGE, 21037 },
	{ 0xf152, 0xf154, PDF_CMAP_TABLE, 1808 },
	{ 0xf155, 0xf156, PDF_CMAP_RANGE, 21055 },
	{ 0xf157, 0xf15a, PDF_CMAP_TABLE, 1811 },
	{ 0xf15b, 0xf166, PDF_CMAP_RANGE, 21058 },
	{ 0xf167, 0xf167, PDF_CMAP_SINGLE, 7767 },
	{ 0xf168, 0xf175, PDF_CMAP_RANGE, 21070 },
	{ 0xf176, 0xf17b, PDF_CMAP_TABLE, 1815 },
	{ 0xf17c, 0xf17d, PDF_CMAP_RANGE, 21085 },
	{ 0xf17e, 0xf17e, PDF_CMAP_SINGLE, 9224 },
	{ 0xf180, 0xf189, PDF_CMAP_TABLE, 1821 },
	{ 0xf18a, 0xf193, PDF_CMAP_RANGE, 21091 },
	{ 0xf194, 0xf194, PDF_CMAP_SINGLE, 7999 },
	{ 0xf195, 0xf197, PDF_CMAP_RANGE, 21101 },
	{ 0xf198, 0xf198, PDF_CMAP_SINGLE, 8295 },
	{ 0xf199, 0xf1a0, PDF_CMAP_RANGE, 21104 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 22560 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 23820 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 25080 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 26340 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 27600 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 28860 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 29961 },
	{ 0xf240, 0xf244, PDF_CMAP_RANGE, 21112 },
	{ 0xf245, 0xf247, PDF_CMAP_TABLE, 1831 },
	{ 0xf248, 0xf24a, PDF_CMAP_RANGE, 21118 },
	{ 0xf24b, 0xf24b, PDF_CMAP_SINGLE, 9231 },
	{ 0xf24c, 0xf252, PDF_CMAP_RANGE, 21121 },
	{ 0xf253, 0xf255, PDF_CMAP_TABLE, 1834 },
	{ 0xf256, 0xf25b, PDF_CMAP_RANGE, 21128 },
	{ 0xf25c, 0xf25c, PDF_CMAP_SINGLE, 9234 },
	{ 0xf25d, 0xf25e, PDF_CMAP_RANGE, 21134 },
	{ 0xf25f, 0xf25f, PDF_CMAP_SINGLE, 8375 },
	{ 0xf260, 0xf270, PDF_CMAP_RANGE, 21136 },
	{ 0xf271, 0xf276, PDF_CMAP_TABLE, 1837 },
	{ 0xf277, 0xf27b, PDF_CMAP_RANGE, 21155 },
	{ 0xf27c, 0xf27e, PDF_CMAP_TABLE, 1843 },
	{ 0xf280, 0xf284, PDF_CMAP_RANGE, 21160 },
	{ 0xf285, 0xf289, PDF_CMAP_TABLE, 1846 },
	{ 0xf28a, 0xf28b, PDF_CMAP_RANGE, 9238 },
	{ 0xf28c, 0xf28c, PDF_CMAP_SINGLE, 8433 },
	{ 0xf28d, 0xf290, PDF_CMAP_RANGE, 21166 },
	{ 0xf291, 0xf291, PDF_CMAP_SINGLE, 9226 },
	{ 0xf292, 0xf293, PDF_CMAP_RANGE, 21170 },
	{ 0xf294, 0xf296, PDF_CMAP_TABLE, 1851 },
	{ 0xf297, 0xf29b, PDF_CMAP_RANGE, 21173 },
	{ 0xf29c, 0xf29e, PDF_CMAP_TABLE, 1854 },
	{ 0xf29f, 0xf2a0, PDF_CMAP_RANGE, 21179 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 22570 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 23830 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 25090 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 26350 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 27610 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 28870 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 29971 },
	{ 0xf340, 0xf341, PDF_CMAP_TABLE, 1857 },
	{ 0xf342, 0xf344, PDF_CMAP_RANGE, 21181 },
	{ 0xf345, 0xf345, PDF_CMAP_SINGLE, 8860 },
	{ 0xf346, 0xf347, PDF_CMAP_RANGE, 21184 },
	{ 0xf348, 0xf34b, PDF_CMAP_TABLE, 1859 },
	{ 0xf34c, 0xf34f, PDF_CMAP_RANGE, 21187 },
	{ 0xf350, 0xf350, PDF_CMAP_SINGLE, 9228 },
	{ 0xf351, 0xf360, PDF_CMAP_RANGE, 21191 },
	{ 0xf361, 0xf361, PDF_CMAP_SINGLE, 7721 },
	{ 0xf362, 0xf373, PDF_CMAP_RANGE, 21207 },
	{ 0xf374, 0xf379, PDF_CMAP_TABLE, 1863 },
	{ 0xf37a, 0xf37e, PDF_CMAP_RANGE, 21226 },
	{ 0xf380, 0xf38b, PDF_CMAP_RANGE, 21231 },
	{ 0xf38c, 0xf38c, PDF_CMAP_SINGLE, 9845 },
	{ 0xf38d, 0xf39f, PDF_CMAP_RANGE, 21243 },
	{ 0xf3a0, 0xf3a0, PDF_CMAP_SINGLE, 8524 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 22580 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 23840 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 25100 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 26360 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 27620 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 28880 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 29981 },
	{ 0xf440, 0xf444, PDF_CMAP_RANGE, 21262 },
	{ 0xf445, 0xf445, PDF_CMAP_SINGLE, 9850 },
	{ 0xf446, 0xf44f, PDF_CMAP_RANGE, 21267 },
	{ 0xf450, 0xf450, PDF_CMAP_SINGLE, 9888 },
	{ 0xf451, 0xf456, PDF_CMAP_RANGE, 21277 },
	{ 0xf457, 0xf45d, PDF_CMAP_TABLE, 1869 },
	{ 0xf45e, 0xf461, PDF_CMAP_RANGE, 21286 },
	{ 0xf462, 0xf464, PDF_CMAP_TABLE, 1876 },
	{ 0xf465, 0xf474, PDF_CMAP_RANGE, 21291 },
	{ 0xf475, 0xf475, PDF_CMAP_SINGLE, 9829 },
	{ 0xf476, 0xf47b, PDF_CMAP_RANGE, 21307 },
	{ 0xf47c, 0xf47e, PDF_CMAP_TABLE, 1879 },
	{ 0xf480, 0xf493, PDF_CMAP_RANGE, 21314 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 8266 },
	{ 0xf495, 0xf498, PDF_CMAP_RANGE, 21334 },
	{ 0xf499, 0xf499, PDF_CMAP_SINGLE, 9759 },
	{ 0xf49a, 0xf49b, PDF_CMAP_RANGE, 21338 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 9758 },
	{ 0xf49d, 0xf4a0, PDF_CMAP_RANGE, 21340 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 22590 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 23850 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 25110 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 26370 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 27630 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 28890 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 29991 },
	{ 0xf540, 0xf544, PDF_CMAP_RANGE, 21344 },
	{ 0xf545, 0xf547, PDF_CMAP_TABLE, 1882 },
	{ 0xf548, 0xf551, PDF_CMAP_RANGE, 21350 },
	{ 0xf552, 0xf556, PDF_CMAP_TABLE, 1885 },
	{ 0xf557, 0xf55d, PDF_CMAP_RANGE, 21361 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 9769 },
	{ 0xf55f, 0xf560, PDF_CMAP_RANGE, 21368 },
	{ 0xf561, 0xf562, PDF_CMAP_TABLE, 1890 },
	{ 0xf563, 0xf56d, PDF_CMAP_RANGE, 21370 },
	{ 0xf56e, 0xf572, PDF_CMAP_TABLE, 1892 },
	{ 0xf573, 0xf57e, PDF_CMAP_RANGE, 21382 },
	{ 0xf580, 0xf584, PDF_CMAP_RANGE, 21394 },
	{ 0xf585, 0xf586, PDF_CMAP_TABLE, 1897 },
	{ 0xf587, 0xf58b, PDF_CMAP_RANGE, 21399 },
	{ 0xf58c, 0xf58f, PDF_CMAP_TABLE, 1899 },
	{ 0xf590, 0xf598, PDF_CMAP_RANGE, 21405 },
	{ 0xf599, 0xf59b, PDF_CMAP_TABLE, 1903 },
	{ 0xf59c, 0xf59f, PDF_CMAP_RANGE, 21415 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 9795 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 22600 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 23860 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 25120 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 26380 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 27640 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 28900 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 30001 },
	{ 0xf640, 0xf641, PDF_CMAP_TABLE, 1906 },
	{ 0xf642, 0xf644, PDF_CMAP_RANGE, 21420 },
	{ 0xf645, 0xf648, PDF_CMAP_TABLE, 1908 },
	{ 0xf649, 0xf64a, PDF_CMAP_RANGE, 21424 },
	{ 0xf64b, 0xf64d, PDF_CMAP_TABLE, 1912 },
	{ 0xf64e, 0xf64f, PDF_CMAP_RANGE, 9787 },
	{ 0xf650, 0xf653, PDF_CMAP_RANGE, 21427 },
	{ 0xf654, 0xf654, PDF_CMAP_SINGLE, 9794 },
	{ 0xf655, 0xf657, PDF_CMAP_RANGE, 21431 },
	{ 0xf658, 0xf658, PDF_CMAP_SINGLE, 9796 },
	{ 0xf659, 0xf660, PDF_CMAP_RANGE, 21434 },
	{ 0xf661, 0xf663, PDF_CMAP_TABLE, 1915 },
	{ 0xf664, 0xf66b, PDF_CMAP_RANGE, 21443 },
	{ 0xf66c, 0xf66d, PDF_CMAP_TABLE, 1918 },
	{ 0xf66e, 0xf670, PDF_CMAP_RANGE, 21451 },
	{ 0xf671, 0xf671, PDF_CMAP_SINGLE, 9801 },
	{ 0xf672, 0xf673, PDF_CMAP_RANGE, 21454 },
	{ 0xf674, 0xf677, PDF_CMAP_TABLE, 1920 },
	{ 0xf678, 0xf67e, PDF_CMAP_RANGE, 21457 },
	{ 0xf680, 0xf684, PDF_CMAP_RANGE, 21464 },
	{ 0xf685, 0xf685, PDF_CMAP_SINGLE, 9805 },
	{ 0xf686, 0xf687, PDF_CMAP_RANGE, 21469 },
	{ 0xf688, 0xf68a, PDF_CMAP_TABLE, 1924 },
	{ 0xf68b, 0xf68c, PDF_CMAP_RANGE, 21472 },
	{ 0xf68d, 0xf68e, PDF_CMAP_TABLE, 1927 },
	{ 0xf68f, 0xf691, PDF_CMAP_RANGE, 21474 },
	{ 0xf692, 0xf692, PDF_CMAP_SINGLE, 9807 },
	{ 0xf693, 0xf695, PDF_CMAP_RANGE, 21477 },
	{ 0xf696, 0xf6a0, PDF_CMAP_TABLE, 1929 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 22610 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 23870 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 25130 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 26390 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 27650 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 28910 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 30011 },
	{ 0xf740, 0xf741, PDF_CMAP_RANGE, 21484 },
	{ 0xf742, 0xf742, PDF_CMAP_SINGLE, 9812 },
	{ 0xf743, 0xf748, PDF_CMAP_RANGE, 21486 },
	{ 0xf749, 0xf749, PDF_CMAP_SINGLE, 9816 },
	{ 0xf74a, 0xf74b, PDF_CMAP_RANGE, 21492 },
	{ 0xf74c, 0xf74d, PDF_CMAP_TABLE, 1940 },
	{ 0xf74e, 0xf755, PDF_CMAP_RANGE, 21494 },
	{ 0xf756, 0xf75c, PDF_CMAP_TABLE, 1942 },
	{ 0xf75d, 0xf760, PDF_CMAP_RANGE, 21504 },
	{ 0xf761, 0xf763, PDF_CMAP_TABLE, 1949 },
	{ 0xf764, 0xf76a, PDF_CMAP_RANGE, 21509 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 9820 },
	{ 0xf76c, 0xf770, PDF_CMAP_RANGE, 21516 },
	{ 0xf771, 0xf771, PDF_CMAP_SINGLE, 9772 },
	{ 0xf772, 0xf77b, PDF_CMAP_RANGE, 21521 },
	{ 0xf77c, 0xf77e, PDF_CMAP_TABLE, 1952 },
	{ 0xf780, 0xf7a0, PDF_CMAP_RANGE, 21532 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 22620 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 23880 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 25140 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 26400 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 27660 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 28920 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 30021 },
	{ 0xf840, 0xf841, PDF_CMAP_RANGE, 21565 },
	{ 0xf842, 0xf842, PDF_CMAP_SINGLE, 8348 },
	{ 0xf843, 0xf845, PDF_CMAP_RANGE, 21567 },
	{ 0xf846, 0xf846, PDF_CMAP_SINGLE, 9600 },
	{ 0xf847, 0xf848, PDF_CMAP_RANGE, 21570 },
	{ 0xf849, 0xf849, PDF_CMAP_SINGLE, 8924 },
	{ 0xf84a, 0xf84f, PDF_CMAP_RANGE, 21572 },
	{ 0xf850, 0xf853, PDF_CMAP_TABLE, 1955 },
	{ 0xf854, 0xf862, PDF_CMAP_RANGE, 21579 },
	{ 0xf863, 0xf866, PDF_CMAP_TABLE, 1959 },
	{ 0xf867, 0xf871, PDF_CMAP_RANGE, 21595 },
	{ 0xf872, 0xf872, PDF_CMAP_SINGLE, 8578 },
	{ 0xf873, 0xf877, PDF_CMAP_RANGE, 21606 },
	{ 0xf878, 0xf87c, PDF_CMAP_TABLE, 1963 },
	{ 0xf87d, 0xf87e, PDF_CMAP_RANGE, 21613 },
	{ 0xf880, 0xf881, PDF_CMAP_TABLE, 1968 },
	{ 0xf882, 0xf883, PDF_CMAP_RANGE, 21616 },
	{ 0xf884, 0xf886, PDF_CMAP_TABLE, 1970 },
	{ 0xf887, 0xf88c, PDF_CMAP_RANGE, 21619 },
	{ 0xf88d, 0xf88e, PDF_CMAP_TABLE, 1973 },
	{ 0xf88f, 0xf898, PDF_CMAP_RANGE, 21625 },
	{ 0xf899, 0xf899, PDF_CMAP_SINGLE, 8007 },
	{ 0xf89a, 0xf89c, PDF_CMAP_RANGE, 21635 },
	{ 0xf89d, 0xf89d, PDF_CMAP_SINGLE, 7965 },
	{ 0xf89e, 0xf89f, PDF_CMAP_RANGE, 21638 },
	{ 0xf8a0, 0xf8a0, PDF_CMAP_SINGLE, 9613 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 22630 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 23890 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 25150 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 26410 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 27670 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 28930 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 30031 },
	{ 0xf940, 0xf94d, PDF_CMAP_RANGE, 21640 },
	{ 0xf94e, 0xf950, PDF_CMAP_TABLE, 1975 },
	{ 0xf951, 0xf958, PDF_CMAP_RANGE, 21654 },
	{ 0xf959, 0xf95a, PDF_CMAP_TABLE, 1978 },
	{ 0xf95b, 0xf95c, PDF_CMAP_RANGE, 21662 },
	{ 0xf95d, 0xf95e, PDF_CMAP_TABLE, 1980 },
	{ 0xf95f, 0xf966, PDF_CMAP_RANGE, 21664 },
	{ 0xf967, 0xf969, PDF_CMAP_TABLE, 1982 },
	{ 0xf96a, 0xf96b, PDF_CMAP_RANGE, 21673 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 9624 },
	{ 0xf96d, 0xf96e, PDF_CMAP_RANGE, 21675 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 8438 },
	{ 0xf970, 0xf97e, PDF_CMAP_RANGE, 21677 },
	{ 0xf980, 0xf984, PDF_CMAP_RANGE, 21692 },
	{ 0xf985, 0xf987, PDF_CMAP_TABLE, 1985 },
	{ 0xf988, 0xf990, PDF_CMAP_RANGE, 21698 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 9622 },
	{ 0xf992, 0xf995, PDF_CMAP_RANGE, 21707 },
	{ 0xf996, 0xf998, PDF_CMAP_TABLE, 1988 },
	{ 0xf999, 0xf9a0, PDF_CMAP_RANGE, 21712 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 22640 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 23900 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 25160 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 26420 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 27680 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 28940 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 30041 },
	{ 0xfa40, 0xfa41, PDF_CMAP_RANGE, 21720 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 9629 },
	{ 0xfa43, 0xfa45, PDF_CMAP_RANGE, 21722 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 9630 },
	{ 0xfa47, 0xfa4b, PDF_CMAP_RANGE, 21725 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 9036 },
	{ 0xfa4d, 0xfa50, PDF_CMAP_RANGE, 21730 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 8004 },
	{ 0xfa52, 0xfa57, PDF_CMAP_RANGE, 21734 },
	{ 0xfa58, 0xfa59, PDF_CMAP_TABLE, 1991 },
	{ 0xfa5a, 0xfa5c, PDF_CMAP_RANGE, 21740 },
	{ 0xfa5d, 0xfa61, PDF_CMAP_TABLE, 1993 },
	{ 0xfa62, 0xfa6f, PDF_CMAP_RANGE, 21745 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 9634 },
	{ 0xfa71, 0xfa73, PDF_CMAP_RANGE, 21759 },
	{ 0xfa74, 0xfa77, PDF_CMAP_TABLE, 1998 },
	{ 0xfa78, 0xfa7e, PDF_CMAP_RANGE, 21763 },
	{ 0xfa80, 0xfa82, PDF_CMAP_RANGE, 21770 },
	{ 0xfa83, 0xfa84, PDF_CMAP_TABLE, 2002 },
	{ 0xfa85, 0xfa8c, PDF_CMAP_RANGE, 21773 },
	{ 0xfa8d, 0xfa8d, PDF_CMAP_SINGLE, 9636 },
	{ 0xfa8e, 0xfa8f, PDF_CMAP_RANGE, 21781 },
	{ 0xfa90, 0xfa91, PDF_CMAP_TABLE, 2004 },
	{ 0xfa92, 0xfa95, PDF_CMAP_RANGE, 21783 },
	{ 0xfa96, 0xfa98, PDF_CMAP_TABLE, 2006 },
	{ 0xfa99, 0xfaa0, PDF_CMAP_RANGE, 21787 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 22650 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 23910 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 25170 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 26430 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 27690 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 28950 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 30051 },
	{ 0xfb40, 0xfb48, PDF_CMAP_RANGE, 21795 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 9640 },
	{ 0xfb4a, 0xfb51, PDF_CMAP_RANGE, 21804 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 9606 },
	{ 0xfb53, 0xfb56, PDF_CMAP_RANGE, 21812 },
	{ 0xfb57, 0xfb5b, PDF_CMAP_TABLE, 2009 },
	{ 0xfb5c, 0xfb74, PDF_CMAP_RANGE, 21817 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 9864 },
	{ 0xfb76, 0xfb78, PDF_CMAP_RANGE, 21842 },
	{ 0xfb79, 0xfb7e, PDF_CMAP_TABLE, 2014 },
	{ 0xfb80, 0xfb8f, PDF_CMAP_RANGE, 21847 },
	{ 0xfb90, 0xfb90, PDF_CMAP_SINGLE, 8207 },
	{ 0xfb91, 0xfb9b, PDF_CMAP_RANGE, 21863 },
	{ 0xfb9c, 0xfb9c, PDF_CMAP_SINGLE, 8305 },
	{ 0xfb9d, 0xfb9e, PDF_CMAP_RANGE, 21874 },
	{ 0xfb9f, 0xfba0, PDF_CMAP_TABLE, 2020 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 22660 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 23920 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 25180 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 26440 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 27700 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 28960 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 30061 },
	{ 0xfc40, 0xfc43, PDF_CMAP_RANGE, 21877 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 9878 },
	{ 0xfc45, 0xfc48, PDF_CMAP_RANGE, 21881 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 9871 },
	{ 0xfc4a, 0xfc59, PDF_CMAP_RANGE, 21885 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 8922 },
	{ 0xfc5b, 0xfc62, PDF_CMAP_RANGE, 21901 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 7873 },
	{ 0xfc64, 0xfc67, PDF_CMAP_RANGE, 21909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 7859 },
	{ 0xfc69, 0xfc6e, PDF_CMAP_RANGE, 21913 },
	{ 0xfc6f, 0xfc71, PDF_CMAP_TABLE, 2022 },
	{ 0xfc72, 0xfc73, PDF_CMAP_RANGE, 21920 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 9833 },
	{ 0xfc75, 0xfc76, PDF_CMAP_RANGE, 21922 },
	{ 0xfc77, 0xfc78, PDF_CMAP_RANGE, 9752 },
	{ 0xfc79, 0xfc7e, PDF_CMAP_RANGE, 21924 },
	{ 0xfc80, 0xfc82, PDF_CMAP_RANGE, 21930 },
	{ 0xfc83, 0xfc83, PDF_CMAP_SINGLE, 9754 },
	{ 0xfc84, 0xfc89, PDF_CMAP_RANGE, 21933 },
	{ 0xfc8a, 0xfc8a, PDF_CMAP_SINGLE, 9843 },
	{ 0xfc8b, 0xfca0, PDF_CMAP_RANGE, 21939 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 22670 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 23930 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 25190 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 26450 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 27710 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 28970 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 30071 },
	{ 0xfd38, 0xfd39, PDF_CMAP_RANGE, 22428 },
	{ 0xfd40, 0xfd51, PDF_CMAP_RANGE, 21961 },
	{ 0xfd52, 0xfd53, PDF_CMAP_TABLE, 2025 },
	{ 0xfd54, 0xfd56, PDF_CMAP_RANGE, 21979 },
	{ 0xfd57, 0xfd5a, PDF_CMAP_TABLE, 2027 },
	{ 0xfd5b, 0xfd5e, PDF_CMAP_RANGE, 21983 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 9745 },
	{ 0xfd60, 0xfd61, PDF_CMAP_RANGE, 21987 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 9747 },
	{ 0xfd63, 0xfd64, PDF_CMAP_RANGE, 21989 },
	{ 0xfd65, 0xfd69, PDF_CMAP_TABLE, 2031 },
	{ 0xfd6a, 0xfd6b, PDF_CMAP_RANGE, 21992 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 9748 },
	{ 0xfd6d, 0xfd6f, PDF_CMAP_RANGE, 21994 },
	{ 0xfd70, 0xfd72, PDF_CMAP_TABLE, 2036 },
	{ 0xfd73, 0xfd77, PDF_CMAP_RANGE, 21998 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 8434 },
	{ 0xfd79, 0xfd7c, PDF_CMAP_RANGE, 22003 },
	{ 0xfd7d, 0xfd7e, PDF_CMAP_TABLE, 2039 },
	{ 0xfd80, 0xfd87, PDF_CMAP_RANGE, 22008 },
	{ 0xfd88, 0xfd88, PDF_CMAP_SINGLE, 8247 },
	{ 0xfd89, 0xfd8a, PDF_CMAP_RANGE, 22016 },
	{ 0xfd8b, 0xfd8b, PDF_CMAP_SINGLE, 8371 },
	{ 0xfd8c, 0xfd8e, PDF_CMAP_RANGE, 22018 },
	{ 0xfd8f, 0xfd90, PDF_CMAP_TABLE, 2041 },
	{ 0xfd91, 0xfd93, PDF_CMAP_RANGE, 22021 },
	{ 0xfd94, 0xfd94, PDF_CMAP_SINGLE, 7988 },
	{ 0xfd95, 0xfd9c, PDF_CMAP_RANGE, 22024 },
	{ 0xfd9d, 0xfda0, PDF_CMAP_TABLE, 2043 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 22430 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 22680 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 23940 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 25200 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 26460 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 27720 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 28980 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 30081 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 4697 },
	{ 0xfe41, 0xfe7e, PDF_CMAP_RANGE, 22032 },
	{ 0xfe80, 0xfea0, PDF_CMAP_RANGE, 22094 },
	{ 0xdd9e, 0xdd9e, PDF_CMAP_SINGLE, 9426 },
	{ 0xdd9f, 0xdd9f, PDF_CMAP_SINGLE, 19692 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 8627 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 23620 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 24880 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 26140 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 27400 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 28660 },
	{ 0xde30, 0xde39, PDF_CMAP_RANGE, 29761 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 8773 },
	{ 0xde41, 0xde41, PDF_CMAP_SINGLE, 9377 },
	{ 0xde42, 0xde43, PDF_CMAP_RANGE, 19693 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 8872 },
	{ 0xde45, 0xde47, PDF_CMAP_RANGE, 19695 },
	{ 0xde48, 0xde48, PDF_CMAP_SINGLE, 8828 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 8112 },
	{ 0xde4a, 0xde4e, PDF_CMAP_RANGE, 19698 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 9378 },
	{ 0xde50, 0xde59, PDF_CMAP_RANGE, 19703 },
	{ 0xde5a, 0xde5a, PDF_CMAP_SINGLE, 8006 },
	{ 0xde5b, 0xde5b, PDF_CMAP_SINGLE, 19713 },
	{ 0xde5c, 0xde5c, PDF_CMAP_SINGLE, 9088 },
	{ 0xde5d, 0xde5d, PDF_CMAP_SINGLE, 9365 },
	{ 0xde5e, 0xde5e, PDF_CMAP_SINGLE, 19714 },
	{ 0xde5f, 0xde5f, PDF_CMAP_SINGLE, 9360 },
	{ 0xde60, 0xde6a, PDF_CMAP_RANGE, 19715 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 7728 },
	{ 0xde6c, 0xde6e, PDF_CMAP_RANGE, 19726 },
	{ 0xde6f, 0xde6f, PDF_CMAP_SINGLE, 7837 },
	{ 0xde70, 0xde70, PDF_CMAP_SINGLE, 7755 },
	{ 0xde71, 0xde71, PDF_CMAP_SINGLE, 7754 },
	{ 0xde72, 0xde72, PDF_CMAP_SINGLE, 8362 },
	{ 0xde73, 0xde7e, PDF_CMAP_RANGE, 19729 },
	{ 0xde80, 0xde91, PDF_CMAP_RANGE, 19741 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 9851 },
	{ 0xde93, 0xde9e, PDF_CMAP_RANGE, 19759 },
	{ 0xde9f, 0xde9f, PDF_CMAP_SINGLE, 9202 },
	{ 0xdea0, 0xdea0, PDF_CMAP_SINGLE, 19771 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 23630 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 24890 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 26150 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 27410 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 28670 },
	{ 0xdf30, 0xdf39, PDF_CMAP_RANGE, 29771 },
	{ 0xdf40, 0xdf40, PDF_CMAP_SINGLE, 8830 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 19772 },
	{ 0xdf42, 0xdf42, PDF_CMAP_SINGLE, 8217 },
	{ 0xdf43, 0xdf4c, PDF_CMAP_RANGE, 19773 },
	{ 0xdf4d, 0xdf4d, PDF_CMAP_SINGLE, 8123 },
	{ 0xdf4e, 0xdf5b, PDF_CMAP_RANGE, 19783 },
	{ 0xdf5c, 0xdf5c, PDF_CMAP_SINGLE, 8787 },
	{ 0xdf5d, 0xdf5d, PDF_CMAP_SINGLE, 19797 },
	{ 0xdf5e, 0xdf5e, PDF_CMAP_SINGLE, 7998 },
	{ 0xdf5f, 0xdf5f, PDF_CMAP_SINGLE, 7846 },
	{ 0xdf60, 0xdf60, PDF_CMAP_SINGLE, 8590 },
	{ 0xdf61, 0xdf63, PDF_CMAP_RANGE, 19798 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 8684 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 19801 },
	{ 0xdf66, 0xdf66, PDF_CMAP_SINGLE, 7870 },
	{ 0xdf67, 0xdf67, PDF_CMAP_SINGLE, 19802 },
	{ 0xdf68, 0xdf68, PDF_CMAP_SINGLE, 8778 },
	{ 0xdf69, 0xdf6c, PDF_CMAP_RANGE, 19803 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 8499 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_RANGE, 19807 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 7812 },
	{ 0xdf75, 0xdf76, PDF_CMAP_RANGE, 19813 },
	{ 0xdf77, 0xdf77, PDF_CMAP_SINGLE, 8399 },
	{ 0xdf78, 0xdf78, PDF_CMAP_SINGLE, 8674 },
	{ 0xdf79, 0xdf79, PDF_CMAP_SINGLE, 19815 },
	{ 0xdf7a, 0xdf7a, PDF_CMAP_SINGLE, 8719 },
	{ 0xdf7b, 0xdf7b, PDF_CMAP_SINGLE, 19816 },
	{ 0xdf7c, 0xdf7c, PDF_CMAP_SINGLE, 8233 },
	{ 0xdf7d, 0xdf7d, PDF_CMAP_SINGLE, 19817 },
	{ 0xdf7e, 0xdf7e, PDF_CMAP_SINGLE, 8307 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 8021 },
	{ 0xdf81, 0xdf82, PDF_CMAP_RANGE, 19818 },
	{ 0xdf83, 0xdf83, PDF_CMAP_SINGLE, 9201 },
	{ 0xdf84, 0xdf84, PDF_CMAP_SINGLE, 19820 },
	{ 0xdf85, 0xdf85, PDF_CMAP_SINGLE, 7750 },
	{ 0xdf86, 0xdf88, PDF_CMAP_RANGE, 19821 },
	{ 0xdf89, 0xdf89, PDF_CMAP_SINGLE, 8291 },
	{ 0xdf8a, 0xdf8a, PDF_CMAP_SINGLE, 9203 },
	{ 0xdf8b, 0xdfa0, PDF_CMAP_RANGE, 19824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 23640 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 24900 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 26160 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 27420 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 28680 },
	{ 0xe030, 0xe039, PDF_CMAP_RANGE, 29781 },
	{ 0xe040, 0xe04f, PDF_CMAP_RANGE, 19846 },
	{ 0xe050, 0xe050, PDF_CMAP_SINGLE, 8990 },
	{ 0xe051, 0xe05c, PDF_CMAP_RANGE, 19862 },
	{ 0xe05d, 0xe05d, PDF_CMAP_SINGLE, 8755 },
	{ 0xe05e, 0xe068, PDF_CMAP_RANGE, 19874 },
	{ 0xe069, 0xe069, PDF_CMAP_SINGLE, 8992 },
	{ 0xe06a, 0xe06b, PDF_CMAP_RANGE, 19885 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 8647 },
	{ 0xe06d, 0xe074, PDF_CMAP_RANGE, 19887 },
	{ 0xe075, 0xe075, PDF_CMAP_SINGLE, 8892 },
	{ 0xe076, 0xe076, PDF_CMAP_SINGLE, 19895 },
	{ 0xe077, 0xe077, PDF_CMAP_SINGLE, 8988 },
	{ 0xe078, 0xe078, PDF_CMAP_SINGLE, 19896 },
	{ 0xe079, 0xe079, PDF_CMAP_SINGLE, 8785 },
	{ 0xe07a, 0xe07e, PDF_CMAP_RANGE, 19897 },
	{ 0xe080, 0xe086, PDF_CMAP_RANGE, 19902 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 7867 },
	{ 0xe088, 0xe08c, PDF_CMAP_RANGE, 19909 },
	{ 0xe08d, 0xe08d, PDF_CMAP_SINGLE, 8839 },
	{ 0xe08e, 0xe08e, PDF_CMAP_SINGLE, 19914 },
	{ 0xe08f, 0xe08f, PDF_CMAP_SINGLE, 8237 },
	{ 0xe090, 0xe090, PDF_CMAP_SINGLE, 7851 },
	{ 0xe091, 0xe091, PDF_CMAP_SINGLE, 19915 },
	{ 0xe092, 0xe092, PDF_CMAP_SINGLE, 8989 },
	{ 0xe093, 0xe093, PDF_CMAP_SINGLE, 19916 },
	{ 0xe094, 0xe094, PDF_CMAP_SINGLE, 8991 },
	{ 0xe095, 0xe096, PDF_CMAP_RANGE, 19917 },
	{ 0xe097, 0xe097, PDF_CMAP_SINGLE, 8987 },
	{ 0xe098, 0xe0a0, PDF_CMAP_RANGE, 19919 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 23650 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 24910 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 26170 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 27430 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 28690 },
	{ 0xe130, 0xe139, PDF_CMAP_RANGE, 29791 },
	{ 0xe140, 0xe141, PDF_CMAP_RANGE, 19928 },
	{ 0xe142, 0xe142, PDF_CMAP_SINGLE, 8993 },
	{ 0xe143, 0xe163, PDF_CMAP_RANGE, 19930 },
	{ 0xe164, 0xe164, PDF_CMAP_SINGLE, 8789 },
	{ 0xe165, 0xe167, PDF_CMAP_RANGE, 19963 },
	{ 0xe168, 0xe168, PDF_CMAP_SINGLE, 7823 },
	{ 0xe169, 0xe173, PDF_CMAP_RANGE, 19966 },
	{ 0xe174, 0xe174, PDF_CMAP_SINGLE, 8716 },
	{ 0xe175, 0xe175, PDF_CMAP_SINGLE, 8100 },
	{ 0xe176, 0xe17e, PDF_CMAP_RANGE, 19977 },
	{ 0xe180, 0xe183, PDF_CMAP_RANGE, 19986 },
	{ 0xe184, 0xe184, PDF_CMAP_SINGLE, 8347 },
	{ 0xe185, 0xe185, PDF_CMAP_SINGLE, 8664 },
	{ 0xe186, 0xe186, PDF_CMAP_SINGLE, 19990 },
	{ 0xe187, 0xe187, PDF_CMAP_SINGLE, 9722 },
	{ 0xe188, 0xe188, PDF_CMAP_SINGLE, 19991 },
	{ 0xe189, 0xe189, PDF_CMAP_SINGLE, 9721 },
	{ 0xe18a, 0xe18b, PDF_CMAP_RANGE, 19992 },
	{ 0xe18c, 0xe18c, PDF_CMAP_SINGLE, 8500 },
	{ 0xe18d, 0xe18d, PDF_CMAP_SINGLE, 19994 },
	{ 0xe18e, 0xe190, PDF_CMAP_RANGE, 9459 },
	{ 0xe191, 0xe191, PDF_CMAP_SINGLE, 9464 },
	{ 0xe192, 0xe192, PDF_CMAP_SINGLE, 19995 },
	{ 0xe193, 0xe193, PDF_CMAP_SINGLE, 9463 },
	{ 0xe194, 0xe194, PDF_CMAP_SINGLE, 7880 },
	{ 0xe195, 0xe195, PDF_CMAP_SINGLE, 9462 },
	{ 0xe196, 0xe197, PDF_CMAP_RANGE, 19996 },
	{ 0xe198, 0xe198, PDF_CMAP_SINGLE, 8832 },
	{ 0xe199, 0xe19d, PDF_CMAP_RANGE, 19998 },
	{ 0xe19e, 0xe19e, PDF_CMAP_SINGLE, 7877 },
	{ 0xe19f, 0xe19f, PDF_CMAP_SINGLE, 9467 },
	{ 0xe1a0, 0xe1a0, PDF_CMAP_SINGLE, 20003 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 23660 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 24920 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 26180 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 27440 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 28700 },
	{ 0xe230, 0xe239, PDF_CMAP_RANGE, 29801 },
	{ 0xe240, 0xe240, PDF_CMAP_SINGLE, 20004 },
	{ 0xe241, 0xe241, PDF_CMAP_SINGLE, 9466 },
	{ 0xe242, 0xe242, PDF_CMAP_SINGLE, 20005 },
	{ 0xe243, 0xe243, PDF_CMAP_SINGLE, 7917 },
	{ 0xe244, 0xe24e, PDF_CMAP_RANGE, 20006 },
	{ 0xe24f, 0xe24f, PDF_CMAP_SINGLE, 9469 },
	{ 0xe250, 0xe250, PDF_CMAP_SINGLE, 20017 },
	{ 0xe251, 0xe251, PDF_CMAP_SINGLE, 9465 },
	{ 0xe252, 0xe252, PDF_CMAP_SINGLE, 20018 },
	{ 0xe253, 0xe253, PDF_CMAP_SINGLE, 9470 },
	{ 0xe254, 0xe254, PDF_CMAP_SINGLE, 8397 },
	{ 0xe255, 0xe259, PDF_CMAP_RANGE, 20019 },
	{ 0xe25a, 0xe25a, PDF_CMAP_SINGLE, 9480 },
	{ 0xe25b, 0xe25b, PDF_CMAP_SINGLE, 9476 },
	{ 0xe25c, 0xe25d, PDF_CMAP_RANGE, 20024 },
	{ 0xe25e, 0xe25e, PDF_CMAP_SINGLE, 9478 },
	{ 0xe25f, 0xe261, PDF_CMAP_RANGE, 20026 },
	{ 0xe262, 0xe262, PDF_CMAP_SINGLE, 9471 },
	{ 0xe263, 0xe263, PDF_CMAP_SINGLE, 8336 },
	{ 0xe264, 0xe266, PDF_CMAP_RANGE, 20029 },
	{ 0xe267, 0xe267, PDF_CMAP_SINGLE, 7901 },
	{ 0xe268, 0xe268, PDF_CMAP_SINGLE, 7973 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 20032 },
	{ 0xe26a, 0xe26a, PDF_CMAP_SINGLE, 9475 },
	{ 0xe26b, 0xe26b, PDF_CMAP_SINGLE, 9474 },
	{ 0xe26c, 0xe26d, PDF_CMAP_RANGE, 20033 },
	{ 0xe26e, 0xe26e, PDF_CMAP_SINGLE, 7802 },
	{ 0xe26f, 0xe26f, PDF_CMAP_SINGLE, 8358 },
	{ 0xe270, 0xe277, PDF_CMAP_RANGE, 20035 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 8149 },
	{ 0xe279, 0xe27c, PDF_CMAP_RANGE, 20043 },
	{ 0xe27d, 0xe27d, PDF_CMAP_SINGLE, 7953 },
	{ 0xe27e, 0xe27e, PDF_CMAP_SINGLE, 20047 },
	{ 0xe280, 0xe280, PDF_CMAP_SINGLE, 9479 },
	{ 0xe281, 0xe281, PDF_CMAP_SINGLE, 9472 },
	{ 0xe282, 0xe282, PDF_CMAP_SINGLE, 9477 },
	{ 0xe283, 0xe288, PDF_CMAP_RANGE, 20048 },
	{ 0xe289, 0xe289, PDF_CMAP_SINGLE, 9497 },
	{ 0xe28a, 0xe28a, PDF_CMAP_SINGLE, 20054 },
	{ 0xe28b, 0xe28b, PDF_CMAP_SINGLE, 9493 },
	{ 0xe28c, 0xe28d, PDF_CMAP_RANGE, 20055 },
	{ 0xe28e, 0xe28e, PDF_CMAP_SINGLE, 9484 },
	{ 0xe28f, 0xe28f, PDF_CMAP_SINGLE, 8241 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 20057 },
	{ 0xe292, 0xe292, PDF_CMAP_SINGLE, 9483 },
	{ 0xe293, 0xe293, PDF_CMAP_SINGLE, 9487 },
	{ 0xe294, 0xe294, PDF_CMAP_SINGLE, 9498 },
	{ 0xe295, 0xe295, PDF_CMAP_SINGLE, 9481 },
	{ 0xe296, 0xe297, PDF_CMAP_RANGE, 20059 },
	{ 0xe298, 0xe298, PDF_CMAP_SINGLE, 9486 },
	{ 0xe299, 0xe299, PDF_CMAP_SINGLE, 8756 },
	{ 0xe29a, 0xe29a, PDF_CMAP_SINGLE, 9491 },
	{ 0xe29b, 0xe29b, PDF_CMAP_SINGLE, 8064 },
	{ 0xe29c, 0xe29f, PDF_CMAP_RANGE, 20061 },
	{ 0xe2a0, 0xe2a0, PDF_CMAP_SINGLE, 9473 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 23670 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 24930 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 26190 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 27450 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 28710 },
	{ 0xe330, 0xe339, PDF_CMAP_RANGE, 29811 },
	{ 0xe340, 0xe341, PDF_CMAP_RANGE, 20065 },
	{ 0xe342, 0xe342, PDF_CMAP_SINGLE, 9495 },
	{ 0xe343, 0xe343, PDF_CMAP_SINGLE, 9494 },
	{ 0xe344, 0xe346, PDF_CMAP_RANGE, 20067 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 9496 },
	{ 0xe348, 0xe34a, PDF_CMAP_RANGE, 20070 },
	{ 0xe34b, 0xe34b, PDF_CMAP_SINGLE, 7766 },
	{ 0xe34c, 0xe34e, PDF_CMAP_RANGE, 20073 },
	{ 0xe34f, 0xe34f, PDF_CMAP_SINGLE, 9485 },
	{ 0xe350, 0xe350, PDF_CMAP_SINGLE, 20076 },
	{ 0xe351, 0xe351, PDF_CMAP_SINGLE, 8403 },
	{ 0xe352, 0xe353, PDF_CMAP_RANGE, 20077 },
	{ 0xe354, 0xe354, PDF_CMAP_SINGLE, 8314 },
	{ 0xe355, 0xe355, PDF_CMAP_SINGLE, 8398 },
	{ 0xe356, 0xe357, PDF_CMAP_RANGE, 20079 },
	{ 0xe358, 0xe358, PDF_CMAP_SINGLE, 9488 },
	{ 0xe359, 0xe35b, PDF_CMAP_RANGE, 20081 },
	{ 0xe35c, 0xe35c, PDF_CMAP_SINGLE, 7765 },
	{ 0xe35d, 0xe35f, PDF_CMAP_RANGE, 20084 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 9482 },
	{ 0xe361, 0xe365, PDF_CMAP_RANGE, 20087 },
	{ 0xe366, 0xe367, PDF_CMAP_RANGE, 9489 },
	{ 0xe368, 0xe370, PDF_CMAP_RANGE, 20092 },
	{ 0xe371, 0xe371, PDF_CMAP_SINGLE, 8106 },
	{ 0xe372, 0xe372, PDF_CMAP_SINGLE, 20101 },
	{ 0xe373, 0xe373, PDF_CMAP_SINGLE, 9502 },
	{ 0xe374, 0xe374, PDF_CMAP_SINGLE, 7967 },
	{ 0xe375, 0xe377, PDF_CMAP_RANGE, 20102 },
	{ 0xe378, 0xe378, PDF_CMAP_SINGLE, 9517 },
	{ 0xe379, 0xe379, PDF_CMAP_SINGLE, 8733 },
	{ 0xe37a, 0xe37b, PDF_CMAP_RANGE, 20105 },
	{ 0xe37c, 0xe37c, PDF_CMAP_SINGLE, 9522 },
	{ 0xe37d, 0xe37d, PDF_CMAP_SINGLE, 20107 },
	{ 0xe37e, 0xe37e, PDF_CMAP_SINGLE, 8571 },
	{ 0xe380, 0xe389, PDF_CMAP_RANGE, 20108 },
	{ 0xe38a, 0xe38a, PDF_CMAP_SINGLE, 8623 },
	{ 0xe38b, 0xe38b, PDF_CMAP_SINGLE, 20118 },
	{ 0xe38c, 0xe38c, PDF_CMAP_SINGLE, 9516 },
	{ 0xe38d, 0xe38e, PDF_CMAP_RANGE, 20119 },
	{ 0xe38f, 0xe38f, PDF_CMAP_SINGLE, 9512 },
	{ 0xe390, 0xe390, PDF_CMAP_SINGLE, 20121 },
	{ 0xe391, 0xe391, PDF_CMAP_SINGLE, 8332 },
	{ 0xe392, 0xe392, PDF_CMAP_SINGLE, 20122 },
	{ 0xe393, 0xe393, PDF_CMAP_SINGLE, 9519 },
	{ 0xe394, 0xe394, PDF_CMAP_SINGLE, 20123 },
	{ 0xe395, 0xe395, PDF_CMAP_SINGLE, 8636 },
	{ 0xe396, 0xe398, PDF_CMAP_RANGE, 20124 },
	{ 0xe399, 0xe399, PDF_CMAP_SINGLE, 9501 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 20127 },
	{ 0xe39c, 0xe39c, PDF_CMAP_SINGLE, 9525 },
	{ 0xe39d, 0xe39d, PDF_CMAP_SINGLE, 20129 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 8717 },
	{ 0xe39f, 0xe39f, PDF_CMAP_SINGLE, 9510 },
	{ 0xe3a0, 0xe3a0, PDF_CMAP_SINGLE, 20130 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 23680 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 24940 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 26200 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 27460 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 28720 },
	{ 0xe430, 0xe439, PDF_CMAP_RANGE, 29821 },
	{ 0xe440, 0xe440, PDF_CMAP_SINGLE, 9524 },
	{ 0xe441, 0xe441, PDF_CMAP_SINGLE, 9514 },
	{ 0xe442, 0xe442, PDF_CMAP_SINGLE, 9503 },
	{ 0xe443, 0xe443, PDF_CMAP_SINGLE, 9521 },
	{ 0xe444, 0xe444, PDF_CMAP_SINGLE, 9500 },
	{ 0xe445, 0xe447, PDF_CMAP_RANGE, 20131 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 9509 },
	{ 0xe449, 0xe44d, PDF_CMAP_RANGE, 20134 },
	{ 0xe44e, 0xe44e, PDF_CMAP_SINGLE, 8653 },
	{ 0xe44f, 0xe44f, PDF_CMAP_SINGLE, 20139 },
	{ 0xe450, 0xe450, PDF_CMAP_SINGLE, 8666 },
	{ 0xe451, 0xe451, PDF_CMAP_SINGLE, 20140 },
	{ 0xe452, 0xe452, PDF_CMAP_SINGLE, 8562 },
	{ 0xe453, 0xe453, PDF_CMAP_SINGLE, 9534 },
	{ 0xe454, 0xe457, PDF_CMAP_RANGE, 20141 },
	{ 0xe458, 0xe458, PDF_CMAP_SINGLE, 8271 },
	{ 0xe459, 0xe459, PDF_CMAP_SINGLE, 20145 },
	{ 0xe45a, 0xe45a, PDF_CMAP_SINGLE, 9539 },
	{ 0xe45b, 0xe45b, PDF_CMAP_SINGLE, 20146 },
	{ 0xe45c, 0xe45c, PDF_CMAP_SINGLE, 8663 },
	{ 0xe45d, 0xe45d, PDF_CMAP_SINGLE, 20147 },
	{ 0xe45e, 0xe45e, PDF_CMAP_SINGLE, 7740 },
	{ 0xe45f, 0xe461, PDF_CMAP_RANGE, 20148 },
	{ 0xe462, 0xe462, PDF_CMAP_SINGLE, 9513 },
	{ 0xe463, 0xe464, PDF_CMAP_RANGE, 20151 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 9505 },
	{ 0xe466, 0xe467, PDF_CMAP_RANGE, 20153 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 7935 },
	{ 0xe469, 0xe472, PDF_CMAP_RANGE, 20155 },
	{ 0xe473, 0xe473, PDF_CMAP_SINGLE, 9535 },
	{ 0xe474, 0xe474, PDF_CMAP_SINGLE, 20165 },
	{ 0xe475, 0xe475, PDF_CMAP_SINGLE, 9540 },
	{ 0xe476, 0xe478, PDF_CMAP_RANGE, 20166 },
	{ 0xe479, 0xe479, PDF_CMAP_SINGLE, 9507 },
	{ 0xe47a, 0xe47a, PDF_CMAP_SINGLE, 7824 },
	{ 0xe47b, 0xe47b, PDF_CMAP_SINGLE, 9530 },
	{ 0xe47c, 0xe47c, PDF_CMAP_SINGLE, 9541 },
	{ 0xe47d, 0xe47d, PDF_CMAP_SINGLE, 20169 },
	{ 0xe47e, 0xe47e, PDF_CMAP_SINGLE, 9533 },
	{ 0xe480, 0xe480, PDF_CMAP_SINGLE, 20170 },
	{ 0xe481, 0xe481, PDF_CMAP_SINGLE, 8385 },
	{ 0xe482, 0xe483, PDF_CMAP_RANGE, 20171 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 8451 },
	{ 0xe485, 0xe485, PDF_CMAP_SINGLE, 9504 },
	{ 0xe486, 0xe486, PDF_CMAP_SINGLE, 9532 },
	{ 0xe487, 0xe487, PDF_CMAP_SINGLE, 9531 },
	{ 0xe488, 0xe488, PDF_CMAP_SINGLE, 9528 },
	{ 0xe489, 0xe48c, PDF_CMAP_RANGE, 20173 },
	{ 0xe48d, 0xe48d, PDF_CMAP_SINGLE, 9536 },
	{ 0xe48e, 0xe48e, PDF_CMAP_SINGLE, 20177 },
	{ 0xe48f, 0xe48f, PDF_CMAP_SINGLE, 8141 },
	{ 0xe490, 0xe492, PDF_CMAP_RANGE, 20178 },
	{ 0xe493, 0xe493, PDF_CMAP_SINGLE, 7960 },
	{ 0xe494, 0xe497, PDF_CMAP_RANGE, 20181 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 9547 },
	{ 0xe499, 0xe49c, PDF_CMAP_RANGE, 20185 },
	{ 0xe49d, 0xe49d, PDF_CMAP_SINGLE, 9543 },
	{ 0xe49e, 0xe49f, PDF_CMAP_RANGE, 9551 },
	{ 0xe4a0, 0xe4a0, PDF_CMAP_SINGLE, 20189 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 23690 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 24950 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 26210 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 27470 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 28730 },
	{ 0xe530, 0xe539, PDF_CMAP_RANGE, 29831 },
	{ 0xe540, 0xe545, PDF_CMAP_RANGE, 20190 },
	{ 0xe546, 0xe546, PDF_CMAP_SINGLE, 8880 },
	{ 0xe547, 0xe547, PDF_CMAP_SINGLE, 20196 },
	{ 0xe548, 0xe548, PDF_CMAP_SINGLE, 9542 },
	{ 0xe549, 0xe54a, PDF_CMAP_RANGE, 20197 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 9548 },
	{ 0xe54c, 0xe54d, PDF_CMAP_RANGE, 20199 },
	{ 0xe54e, 0xe54e, PDF_CMAP_SINGLE, 7834 },
	{ 0xe54f, 0xe54f, PDF_CMAP_SINGLE, 9554 },
	{ 0xe550, 0xe550, PDF_CMAP_SINGLE, 9520 },
	{ 0xe551, 0xe551, PDF_CMAP_SINGLE, 9545 },
	{ 0xe552, 0xe554, PDF_CMAP_RANGE, 20201 },
	{ 0xe555, 0xe555, PDF_CMAP_SINGLE, 9553 },
	{ 0xe556, 0xe556, PDF_CMAP_SINGLE, 7882 },
	{ 0xe557, 0xe557, PDF_CMAP_SINGLE, 20204 },
	{ 0xe558, 0xe558, PDF_CMAP_SINGLE, 8402 },
	{ 0xe559, 0xe55b, PDF_CMAP_RANGE, 20205 },
	{ 0xe55c, 0xe55c, PDF_CMAP_SINGLE, 8120 },
	{ 0xe55d, 0xe55d, PDF_CMAP_SINGLE, 20208 },
	{ 0xe55e, 0xe55e, PDF_CMAP_SINGLE, 8313 },
	{ 0xe55f, 0xe560, PDF_CMAP_RANGE, 20209 },
	{ 0xe561, 0xe561, PDF_CMAP_SINGLE, 8619 },
	{ 0xe562, 0xe563, PDF_CMAP_RANGE, 20211 },
	{ 0xe564, 0xe564, PDF_CMAP_SINGLE, 9549 },
	{ 0xe565, 0xe565, PDF_CMAP_SINGLE, 7845 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 20213 },
	{ 0xe568, 0xe568, PDF_CMAP_SINGLE, 8268 },
	{ 0xe569, 0xe569, PDF_CMAP_SINGLE, 8320 },
	{ 0xe56a, 0xe56b, PDF_CMAP_RANGE, 20215 },
	{ 0xe56c, 0xe56c, PDF_CMAP_SINGLE, 9837 },
	{ 0xe56d, 0xe56d, PDF_CMAP_SINGLE, 20217 },
	{ 0xe56e, 0xe56e, PDF_CMAP_SINGLE, 9527 },
	{ 0xe56f, 0xe574, PDF_CMAP_RANGE, 20218 },
	{ 0xe575, 0xe575, PDF_CMAP_SINGLE, 9546 },
	{ 0xe576, 0xe576, PDF_CMAP_SINGLE, 8632 },
	{ 0xe577, 0xe577, PDF_CMAP_SINGLE, 20224 },
	{ 0xe578, 0xe578, PDF_CMAP_SINGLE, 9550 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 20225 },
	{ 0xe57b, 0xe57b, PDF_CMAP_SINGLE, 9468 },
	{ 0xe57c, 0xe57c, PDF_CMAP_SINGLE, 9556 },
	{ 0xe57d, 0xe57e, PDF_CMAP_RANGE, 20227 },
	{ 0xe580, 0xe580, PDF_CMAP_SINGLE, 20229 },
	{ 0xe581, 0xe581, PDF_CMAP_SINGLE, 7996 },
	{ 0xe582, 0xe582, PDF_CMAP_SINGLE, 20230 },
	{ 0xe583, 0xe583, PDF_CMAP_SINGLE, 7893 },
	{ 0xe584, 0xe589, PDF_CMAP_RANGE, 20231 },
	{ 0xe58a, 0xe58a, PDF_CMAP_SINGLE, 9558 },
	{ 0xe58b, 0xe58d, PDF_CMAP_RANGE, 20237 },
	{ 0xe58e, 0xe58e, PDF_CMAP_SINGLE, 8808 },
	{ 0xe58f, 0xe590, PDF_CMAP_RANGE, 20240 },
	{ 0xe591, 0xe591, PDF_CMAP_SINGLE, 7894 },
	{ 0xe592, 0xe599, PDF_CMAP_RANGE, 20242 },
	{ 0xe59a, 0xe59a, PDF_CMAP_SINGLE, 9559 },
	{ 0xe59b, 0xe59b, PDF_CMAP_SINGLE, 9555 },
	{ 0xe59c, 0xe59e, PDF_CMAP_RANGE, 20250 },
	{ 0xe59f, 0xe59f, PDF_CMAP_SINGLE, 9544 },
	{ 0xe5a0, 0xe5a0, PDF_CMAP_SINGLE, 20253 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 23700 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 24960 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 26220 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 27480 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 28740 },
	{ 0xe630, 0xe639, PDF_CMAP_RANGE, 29841 },
	{ 0xe640, 0xe640, PDF_CMAP_SINGLE, 8412 },
	{ 0xe641, 0xe643, PDF_CMAP_RANGE, 20254 },
	{ 0xe644, 0xe644, PDF_CMAP_SINGLE, 9561 },
	{ 0xe645, 0xe648, PDF_CMAP_RANGE, 20257 },
	{ 0xe649, 0xe649, PDF_CMAP_SINGLE, 8087 },
	{ 0xe64a, 0xe64a, PDF_CMAP_SINGLE, 9557 },
	{ 0xe64b, 0xe64d, PDF_CMAP_RANGE, 20261 },
	{ 0xe64e, 0xe64e, PDF_CMAP_SINGLE, 8829 },
	{ 0xe64f, 0xe651, PDF_CMAP_RANGE, 20264 },
	{ 0xe652, 0xe652, PDF_CMAP_SINGLE, 9598 },
	{ 0xe653, 0xe655, PDF_CMAP_RANGE, 20267 },
	{ 0xe656, 0xe656, PDF_CMAP_SINGLE, 8316 },
	{ 0xe657, 0xe657, PDF_CMAP_SINGLE, 20270 },
	{ 0xe658, 0xe658, PDF_CMAP_SINGLE, 9562 },
	{ 0xe659, 0xe65a, PDF_CMAP_RANGE, 20271 },
	{ 0xe65b, 0xe65b, PDF_CMAP_SINGLE, 9566 },
	{ 0xe65c, 0xe65d, PDF_CMAP_RANGE, 20273 },
	{ 0xe65e, 0xe65e, PDF_CMAP_SINGLE, 7732 },
	{ 0xe65f, 0xe668, PDF_CMAP_RANGE, 20275 },
	{ 0xe669, 0xe669, PDF_CMAP_SINGLE, 8542 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 20285 },
	{ 0xe66b, 0xe66b, PDF_CMAP_SINGLE, 9568 },
	{ 0xe66c, 0xe674, PDF_CMAP_RANGE, 20286 },
	{ 0xe675, 0xe675, PDF_CMAP_SINGLE, 8610 },
	{ 0xe676, 0xe676, PDF_CMAP_SINGLE, 9044 },
	{ 0xe677, 0xe678, PDF_CMAP_RANGE, 20295 },
	{ 0xe679, 0xe679, PDF_CMAP_SINGLE, 9571 },
	{ 0xe67a, 0xe67a, PDF_CMAP_SINGLE, 9511 },
	{ 0xe67b, 0xe67b, PDF_CMAP_SINGLE, 20297 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 9518 },
	{ 0xe67d, 0xe67d, PDF_CMAP_SINGLE, 9560 },
	{ 0xe67e, 0xe67e, PDF_CMAP_SINGLE, 20298 },
	{ 0xe680, 0xe680, PDF_CMAP_SINGLE, 7963 },
	{ 0xe681, 0xe681, PDF_CMAP_SINGLE, 20299 },
	{ 0xe682, 0xe682, PDF_CMAP_SINGLE, 8835 },
	{ 0xe683, 0xe683, PDF_CMAP_SINGLE, 20300 },
	{ 0xe684, 0xe684, PDF_CMAP_SINGLE, 9572 },
	{ 0xe685, 0xe686, PDF_CMAP_RANGE, 20301 },
	{ 0xe687, 0xe687, PDF_CMAP_SINGLE, 8352 },
	{ 0xe688, 0xe688, PDF_CMAP_SINGLE, 20303 },
	{ 0xe689, 0xe689, PDF_CMAP_SINGLE, 9573 },
	{ 0xe68a, 0xe68b, PDF_CMAP_RANGE, 20304 },
	{ 0xe68c, 0xe68c, PDF_CMAP_SINGLE, 9569 },
	{ 0xe68d, 0xe692, PDF_CMAP_RANGE, 20306 },
	{ 0xe693, 0xe693, PDF_CMAP_SINGLE, 9570 },
	{ 0xe694, 0xe696, PDF_CMAP_RANGE, 20312 },
	{ 0xe697, 0xe697, PDF_CMAP_SINGLE, 9580 },
	{ 0xe698, 0xe69a, PDF_CMAP_RANGE, 20315 },
	{ 0xe69b, 0xe69b, PDF_CMAP_SINGLE, 9581 },
	{ 0xe69c, 0xe69c, PDF_CMAP_SINGLE, 8224 },
	{ 0xe69d, 0xe69e, PDF_CMAP_RANGE, 20318 },
	{ 0xe69f, 0xe69f, PDF_CMAP_SINGLE, 9567 },
	{ 0xe6a0, 0xe6a0, PDF_CMAP_SINGLE, 9578 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 23710 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 24970 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 26230 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 27490 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 28750 },
	{ 0xe730, 0xe739, PDF_CMAP_RANGE, 29851 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 20320 },
	{ 0xe743, 0xe743, PDF_CMAP_SINGLE, 9582 },
	{ 0xe744, 0xe747, PDF_CMAP_RANGE, 20323 },
	{ 0xe748, 0xe748, PDF_CMAP_SINGLE, 9529 },
	{ 0xe749, 0xe749, PDF_CMAP_SINGLE, 9564 },
	{ 0xe74a, 0xe74c, PDF_CMAP_RANGE, 20327 },
	{ 0xe74d, 0xe74e, PDF_CMAP_RANGE, 9576 },
	{ 0xe74f, 0xe74f, PDF_CMAP_SINGLE, 9579 },
	{ 0xe750, 0xe750, PDF_CMAP_SINGLE, 7791 },
	{ 0xe751, 0xe751, PDF_CMAP_SINGLE, 20330 },
	{ 0xe752, 0xe752, PDF_CMAP_SINGLE, 8132 },
	{ 0xe753, 0xe753, PDF_CMAP_SINGLE, 9575 },
	{ 0xe754, 0xe754, PDF_CMAP_SINGLE, 20331 },
	{ 0xe755, 0xe755, PDF_CMAP_SINGLE, 9563 },
	{ 0xe756, 0xe758, PDF_CMAP_RANGE, 20332 },
	{ 0xe759, 0xe759, PDF_CMAP_SINGLE, 9757 },
	{ 0xe75a, 0xe765, PDF_CMAP_RANGE, 20335 },
	{ 0xe766, 0xe766, PDF_CMAP_SINGLE, 9515 },
	{ 0xe767, 0xe767, PDF_CMAP_SINGLE, 20347 },
	{ 0xe768, 0xe768, PDF_CMAP_SINGLE, 9585 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 20348 },
	{ 0xe76a, 0xe76a, PDF_CMAP_SINGLE, 9591 },
	{ 0xe76b, 0xe773, PDF_CMAP_RANGE, 20349 },
	{ 0xe774, 0xe774, PDF_CMAP_SINGLE, 9506 },
	{ 0xe775, 0xe77b, PDF_CMAP_RANGE, 20358 },
	{ 0xe77c, 0xe77c, PDF_CMAP_SINGLE, 9523 },
	{ 0xe77d, 0xe77e, PDF_CMAP_RANGE, 20365 },
	{ 0xe780, 0xe781, PDF_CMAP_RANGE, 20367 },
	{ 0xe782, 0xe782, PDF_CMAP_SINGLE, 8234 },
	{ 0xe783, 0xe783, PDF_CMAP_SINGLE, 20369 },
	{ 0xe784, 0xe784, PDF_CMAP_SINGLE, 9526 },
	{ 0xe785, 0xe785, PDF_CMAP_SINGLE, 9587 },
	{ 0xe786, 0xe786, PDF_CMAP_SINGLE, 9583 },
	{ 0xe787, 0xe789, PDF_CMAP_RANGE, 20370 },
	{ 0xe78a, 0xe78a, PDF_CMAP_SINGLE, 8851 },
	{ 0xe78b, 0xe78b, PDF_CMAP_SINGLE, 9592 },
	{ 0xe78c, 0xe78e, PDF_CMAP_RANGE, 20373 },
	{ 0xe78f, 0xe78f, PDF_CMAP_SINGLE, 9584 },
	{ 0xe790, 0xe791, PDF_CMAP_RANGE, 20376 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 9589 },
	{ 0xe793, 0xe797, PDF_CMAP_RANGE, 20378 },
	{ 0xe798, 0xe799, PDF_CMAP_RANGE, 9537 },
	{ 0xe79a, 0xe79a, PDF_CMAP_SINGLE, 9565 },
	{ 0xe79b, 0xe79f, PDF_CMAP_RANGE, 20383 },
	{ 0xe7a0, 0xe7a0, PDF_CMAP_SINGLE, 8218 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 23720 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 24980 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 26240 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 27500 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 28760 },
	{ 0xe830, 0xe839, PDF_CMAP_RANGE, 29861 },
	{ 0xe840, 0xe842, PDF_CMAP_RANGE, 20388 },
	{ 0xe843, 0xe843, PDF_CMAP_SINGLE, 9594 },
	{ 0xe844, 0xe844, PDF_CMAP_SINGLE, 8198 },
	{ 0xe845, 0xe845, PDF_CMAP_SINGLE, 20391 },
	{ 0xe846, 0xe846, PDF_CMAP_SINGLE, 8567 },
	{ 0xe847, 0xe848, PDF_CMAP_RANGE, 20392 },
	{ 0xe849, 0xe849, PDF_CMAP_SINGLE, 9499 },
	{ 0xe84a, 0xe84a, PDF_CMAP_SINGLE, 20394 },
	{ 0xe84b, 0xe84b, PDF_CMAP_SINGLE, 9508 },
	{ 0xe84c, 0xe84e, PDF_CMAP_RANGE, 20395 },
	{ 0xe84f, 0xe84f, PDF_CMAP_SINGLE, 9595 },
	{ 0xe850, 0xe853, PDF_CMAP_RANGE, 20398 },
	{ 0xe854, 0xe854, PDF_CMAP_SINGLE, 8867 },
	{ 0xe855, 0xe859, PDF_CMAP_RANGE, 20402 },
	{ 0xe85a, 0xe85a, PDF_CMAP_SINGLE, 9593 },
	{ 0xe85b, 0xe85b, PDF_CMAP_SINGLE, 20407 },
	{ 0xe85c, 0xe85c, PDF_CMAP_SINGLE, 9574 },
	{ 0xe85d, 0xe861, PDF_CMAP_RANGE, 20408 },
	{ 0xe862, 0xe862, PDF_CMAP_SINGLE, 8083 },
	{ 0xe863, 0xe863, PDF_CMAP_SINGLE, 20413 },
	{ 0xe864, 0xe864, PDF_CMAP_SINGLE, 9596 },
	{ 0xe865, 0xe86f, PDF_CMAP_RANGE, 20414 },
	{ 0xe870, 0xe870, PDF_CMAP_SINGLE, 9492 },
	{ 0xe871, 0xe872, PDF_CMAP_RANGE, 20425 },
	{ 0xe873, 0xe873, PDF_CMAP_SINGLE, 9597 },
	{ 0xe874, 0xe874, PDF_CMAP_SINGLE, 20427 },
	{ 0xe875, 0xe875, PDF_CMAP_SINGLE, 9586 },
	{ 0xe876, 0xe87b, PDF_CMAP_RANGE, 20428 },
	{ 0xe87c, 0xe87c, PDF_CMAP_SINGLE, 9588 },
	{ 0xe87d, 0xe87e, PDF_CMAP_RANGE, 20434 },
	{ 0xe880, 0xe880, PDF_CMAP_SINGLE, 8782 },
	{ 0xe881, 0xe881, PDF_CMAP_SINGLE, 20436 },
	{ 0xe882, 0xe882, PDF_CMAP_SINGLE, 8646 },
	{ 0xe883, 0xe886, PDF_CMAP_RANGE, 20437 },
	{ 0xe887, 0xe887, PDF_CMAP_SINGLE, 8351 },
	{ 0xe888, 0xe888, PDF_CMAP_SINGLE, 20441 },
	{ 0xe889, 0xe889, PDF_CMAP_SINGLE, 9590 },
	{ 0xe88a, 0xe88b, PDF_CMAP_RANGE, 20442 },
	{ 0xe88c, 0xe88c, PDF_CMAP_SINGLE, 8292 },
	{ 0xe88d, 0xe88d, PDF_CMAP_SINGLE, 8895 },
	{ 0xe88e, 0xe88e, PDF_CMAP_SINGLE, 9756 },
	{ 0xe88f, 0xe88f, PDF_CMAP_SINGLE, 8798 },
	{ 0xe890, 0xe8a0, PDF_CMAP_RANGE, 20444 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 23730 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 24990 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 26250 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 27510 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 28770 },
	{ 0xe930, 0xe939, PDF_CMAP_RANGE, 29871 },
	{ 0xe940, 0xe94b, PDF_CMAP_RANGE, 20461 },
	{ 0xe94c, 0xe94c, PDF_CMAP_SINGLE, 7797 },
	{ 0xe94d, 0xe953, PDF_CMAP_RANGE, 20473 },
	{ 0xe954, 0xe954, PDF_CMAP_SINGLE, 8317 },
	{ 0xe955, 0xe955, PDF_CMAP_SINGLE, 20480 },
	{ 0xe956, 0xe956, PDF_CMAP_SINGLE, 9151 },
	{ 0xe957, 0xe957, PDF_CMAP_SINGLE, 8467 },
	{ 0xe958, 0xe959, PDF_CMAP_RANGE, 20481 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 9152 },
	{ 0xe95b, 0xe95c, PDF_CMAP_RANGE, 20483 },
	{ 0xe95d, 0xe95d, PDF_CMAP_SINGLE, 7749 },
	{ 0xe95e, 0xe95e, PDF_CMAP_SINGLE, 20485 },
	{ 0xe95f, 0xe95f, PDF_CMAP_SINGLE, 8152 },
	{ 0xe960, 0xe960, PDF_CMAP_SINGLE, 9156 },
	{ 0xe961, 0xe961, PDF_CMAP_SINGLE, 20486 },
	{ 0xe962, 0xe962, PDF_CMAP_SINGLE, 9154 },
	{ 0xe963, 0xe963, PDF_CMAP_SINGLE, 8452 },
	{ 0xe964, 0xe964, PDF_CMAP_SINGLE, 20487 },
	{ 0xe965, 0xe965, PDF_CMAP_SINGLE, 8637 },
	{ 0xe966, 0xe966, PDF_CMAP_SINGLE, 20488 },
	{ 0xe967, 0xe967, PDF_CMAP_SINGLE, 8071 },
	{ 0xe968, 0xe968, PDF_CMAP_SINGLE, 9155 },
	{ 0xe969, 0xe96b, PDF_CMAP_RANGE, 20489 },
	{ 0xe96c, 0xe96c, PDF_CMAP_SINGLE, 8809 },
	{ 0xe96d, 0xe974, PDF_CMAP_RANGE, 20492 },
	{ 0xe975, 0xe975, PDF_CMAP_SINGLE, 8003 },
	{ 0xe976, 0xe976, PDF_CMAP_SINGLE, 20500 },
	{ 0xe977, 0xe977, PDF_CMAP_SINGLE, 7966 },
	{ 0xe978, 0xe978, PDF_CMAP_SINGLE, 9849 },
	{ 0xe979, 0xe979, PDF_CMAP_SINGLE, 7915 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 20501 },
	{ 0xe97c, 0xe97c, PDF_CMAP_SINGLE, 7989 },
	{ 0xe97d, 0xe97d, PDF_CMAP_SINGLE, 8330 },
	{ 0xe97e, 0xe97e, PDF_CMAP_SINGLE, 20503 },
	{ 0xe980, 0xe980, PDF_CMAP_SINGLE, 9159 },
	{ 0xe981, 0xe981, PDF_CMAP_SINGLE, 9161 },
	{ 0xe982, 0xe982, PDF_CMAP_SINGLE, 9158 },
	{ 0xe983, 0xe986, PDF_CMAP_RANGE, 20504 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 8783 },
	{ 0xe988, 0xe98a, PDF_CMAP_RANGE, 20508 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 9163 },
	{ 0xe98c, 0xe98d, PDF_CMAP_RANGE, 20511 },
	{ 0xe98e, 0xe98e, PDF_CMAP_SINGLE, 8691 },
	{ 0xe98f, 0xe98f, PDF_CMAP_SINGLE, 20513 },
	{ 0xe990, 0xe990, PDF_CMAP_SINGLE, 8695 },
	{ 0xe991, 0xe991, PDF_CMAP_SINGLE, 9167 },
	{ 0xe992, 0xe992, PDF_CMAP_SINGLE, 9166 },
	{ 0xe993, 0xe993, PDF_CMAP_SINGLE, 9162 },
	{ 0xe994, 0xe994, PDF_CMAP_SINGLE, 9165 },
	{ 0xe995, 0xe997, PDF_CMAP_RANGE, 20514 },
	{ 0xe998, 0xe998, PDF_CMAP_SINGLE, 9168 },
	{ 0xe999, 0xe99a, PDF_CMAP_RANGE, 20517 },
	{ 0xe99b, 0xe99b, PDF_CMAP_SINGLE, 9836 },
	{ 0xe99c, 0xe99c, PDF_CMAP_SINGLE, 20519 },
	{ 0xe99d, 0xe99d, PDF_CMAP_SINGLE, 9153 },
	{ 0xe99e, 0xe99e, PDF_CMAP_SINGLE, 20520 },
	{ 0xe99f, 0xe99f, PDF_CMAP_SINGLE, 8174 },
	{ 0xe9a0, 0xe9a0, PDF_CMAP_SINGLE, 9169 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 23740 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 25000 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 26260 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 27520 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 28780 },
	{ 0xea30, 0xea39, PDF_CMAP_RANGE, 29881 },
	{ 0xea40, 0xea40, PDF_CMAP_SINGLE, 8184 },
	{ 0xea41, 0xea43, PDF_CMAP_RANGE, 20521 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 9171 },
	{ 0xea45, 0xea47, PDF_CMAP_RANGE, 20524 },
	{ 0xea48, 0xea48, PDF_CMAP_SINGLE, 9170 },
	{ 0xea49, 0xea49, PDF_CMAP_SINGLE, 9172 },
	{ 0xea4a, 0xea4a, PDF_CMAP_SINGLE, 7832 },
	{ 0xea4b, 0xea4f, PDF_CMAP_RANGE, 20527 },
	{ 0xea50, 0xea50, PDF_CMAP_SINGLE, 7980 },
	{ 0xea51, 0xea51, PDF_CMAP_SINGLE, 20532 },
	{ 0xea52, 0xea52, PDF_CMAP_SINGLE, 9173 },
	{ 0xea53, 0xea54, PDF_CMAP_RANGE, 20533 },
	{ 0xea55, 0xea55, PDF_CMAP_SINGLE, 7793 },
	{ 0xea56, 0xea56, PDF_CMAP_SINGLE, 9873 },
	{ 0xea57, 0xea58, PDF_CMAP_RANGE, 20535 },
	{ 0xea59, 0xea59, PDF_CMAP_SINGLE, 9157 },
	{ 0xea5a, 0xea7e, PDF_CMAP_RANGE, 20537 },
	{ 0xea80, 0xea80, PDF_CMAP_SINGLE, 8986 },
	{ 0xea81, 0xea83, PDF_CMAP_RANGE, 20574 },
	{ 0xea84, 0xea84, PDF_CMAP_SINGLE, 8468 },
	{ 0xea85, 0xea86, PDF_CMAP_RANGE, 20577 },
	{ 0xea87, 0xea87, PDF_CMAP_SINGLE, 8836 },
	{ 0xea88, 0xea8d, PDF_CMAP_RANGE, 20579 },
	{ 0xea8e, 0xea8e, PDF_CMAP_SINGLE, 8732 },
	{ 0xea8f, 0xea8f, PDF_CMAP_SINGLE, 20585 },
	{ 0xea90, 0xea90, PDF_CMAP_SINGLE, 7806 },
	{ 0xea91, 0xea91, PDF_CMAP_SINGLE, 8269 },
	{ 0xea92, 0xea95, PDF_CMAP_RANGE, 20586 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 8705 },
	{ 0xea97, 0xea9f, PDF_CMAP_RANGE, 20590 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 7897 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 23750 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 25010 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 26270 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 27530 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 28790 },
	{ 0xeb30, 0xeb39, PDF_CMAP_RANGE, 29891 },
	{ 0xeb40, 0xeb40, PDF_CMAP_SINGLE, 20599 },
	{ 0xeb41, 0xeb41, PDF_CMAP_SINGLE, 8114 },
	{ 0xeb42, 0xeb44, PDF_CMAP_RANGE, 20600 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 8786 },
	{ 0xeb46, 0xeb47, PDF_CMAP_RANGE, 20603 },
	{ 0xeb48, 0xeb48, PDF_CMAP_SINGLE, 8057 },
	{ 0xeb49, 0xeb52, PDF_CMAP_RANGE, 20605 },
	{ 0xeb53, 0xeb53, PDF_CMAP_SINGLE, 8535 },
	{ 0xeb54, 0xeb54, PDF_CMAP_SINGLE, 20615 },
	{ 0xeb55, 0xeb55, PDF_CMAP_SINGLE, 8639 },
	{ 0xeb56, 0xeb5a, PDF_CMAP_RANGE, 20616 },
	{ 0xeb5b, 0xeb5b, PDF_CMAP_SINGLE, 8735 },
	{ 0xeb5c, 0xeb5c, PDF_CMAP_SINGLE, 20621 },
	{ 0xeb5d, 0xeb5d, PDF_CMAP_SINGLE, 8253 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 20622 },
	{ 0xeb60, 0xeb60, PDF_CMAP_SINGLE, 8213 },
	{ 0xeb61, 0xeb61, PDF_CMAP_SINGLE, 20624 },
	{ 0xeb62, 0xeb62, PDF_CMAP_SINGLE, 9893 },
	{ 0xeb63, 0xeb6c, PDF_CMAP_RANGE, 20625 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 8534 },
	{ 0xeb6e, 0xeb6f, PDF_CMAP_RANGE, 20635 },
	{ 0xeb70, 0xeb70, PDF_CMAP_SINGLE, 8516 },
	{ 0xeb71, 0xeb71, PDF_CMAP_SINGLE, 20637 },
	{ 0xeb72, 0xeb72, PDF_CMAP_SINGLE, 7825 },
	{ 0xeb73, 0xeb73, PDF_CMAP_SINGLE, 8791 },
	{ 0xeb74, 0xeb77, PDF_CMAP_RANGE, 20638 },
	{ 0xeb78, 0xeb78, PDF_CMAP_SINGLE, 8202 },
	{ 0xeb79, 0xeb79, PDF_CMAP_SINGLE, 8338 },
	{ 0xeb7a, 0xeb7e, PDF_CMAP_RANGE, 20642 },
	{ 0xeb80, 0xeb84, PDF_CMAP_RANGE, 20647 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 8784 },
	{ 0xeb86, 0xeb89, PDF_CMAP_RANGE, 20652 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 7875 },
	{ 0xeb8b, 0xeba0, PDF_CMAP_RANGE, 20656 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 23760 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 25020 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 26280 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 27540 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 28800 },
	{ 0xec30, 0xec39, PDF_CMAP_RANGE, 29901 },
	{ 0xec40, 0xec45, PDF_CMAP_RANGE, 20678 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8616 },
	{ 0xec47, 0xec55, PDF_CMAP_RANGE, 20684 },
	{ 0xec56, 0xec56, PDF_CMAP_SINGLE, 9741 },
	{ 0xec57, 0xec59, PDF_CMAP_RANGE, 20699 },
	{ 0xec5a, 0xec5a, PDF_CMAP_SINGLE, 9740 },
	{ 0xec5b, 0xec5b, PDF_CMAP_SINGLE, 20702 },
	{ 0xec5c, 0xec5c, PDF_CMAP_SINGLE, 9742 },
	{ 0xec5d, 0xec5f, PDF_CMAP_RANGE, 20703 },
	{ 0xec60, 0xec60, PDF_CMAP_SINGLE, 8242 },
	{ 0xec61, 0xec6d, PDF_CMAP_RANGE, 20706 },
	{ 0xec6e, 0xec6e, PDF_CMAP_SINGLE, 9739 },
	{ 0xec6f, 0xec75, PDF_CMAP_RANGE, 20719 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8899 },
	{ 0xec77, 0xec7e, PDF_CMAP_RANGE, 20726 },
	{ 0xec80, 0xec95, PDF_CMAP_RANGE, 20734 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 7971 },
	{ 0xec97, 0xeca0, PDF_CMAP_RANGE, 20756 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 23770 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 25030 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 26290 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 27550 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 28810 },
	{ 0xed30, 0xed39, PDF_CMAP_RANGE, 29911 },
	{ 0xed40, 0xed45, PDF_CMAP_RANGE, 20766 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 9877 },
	{ 0xed47, 0xed57, PDF_CMAP_RANGE, 20772 },
	{ 0xed58, 0xed58, PDF_CMAP_SINGLE, 9822 },
	{ 0xed59, 0xed5d, PDF_CMAP_RANGE, 20789 },
	{ 0xed5e, 0xed5e, PDF_CMAP_SINGLE, 9821 },
	{ 0xed5f, 0xed60, PDF_CMAP_RANGE, 20794 },
	{ 0xed61, 0xed61, PDF_CMAP_SINGLE, 9874 },
	{ 0xed62, 0xed63, PDF_CMAP_RANGE, 20796 },
	{ 0xed64, 0xed64, PDF_CMAP_SINGLE, 9823 },
	{ 0xed65, 0xed65, PDF_CMAP_SINGLE, 20798 },
	{ 0xed66, 0xed66, PDF_CMAP_SINGLE, 8589 },
	{ 0xed67, 0xed67, PDF_CMAP_SINGLE, 8445 },
	{ 0xed68, 0xed6d, PDF_CMAP_RANGE, 20799 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 8000 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 20805 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 9317 },
	{ 0xed75, 0xed76, PDF_CMAP_RANGE, 20810 },
	{ 0xed77, 0xed77, PDF_CMAP_SINGLE, 9319 },
	{ 0xed78, 0xed78, PDF_CMAP_SINGLE, 20812 },
	{ 0xed79, 0xed79, PDF_CMAP_SINGLE, 9318 },
	{ 0xed7a, 0xed7e, PDF_CMAP_RANGE, 20813 },
	{ 0xed80, 0xed90, PDF_CMAP_RANGE, 20818 },
	{ 0xed91, 0xed91, PDF_CMAP_SINGLE, 8649 },
	{ 0xed92, 0xed92, PDF_CMAP_SINGLE, 20835 },
	{ 0xed93, 0xed93, PDF_CMAP_SINGLE, 8713 },
	{ 0xed94, 0xed94, PDF_CMAP_SINGLE, 7881 },
	{ 0xed95, 0xed95, PDF_CMAP_SINGLE, 8425 },
	{ 0xed96, 0xed96, PDF_CMAP_SINGLE, 20836 },
	{ 0xed97, 0xed97, PDF_CMAP_SINGLE, 8650 },
	{ 0xed98, 0xed98, PDF_CMAP_SINGLE, 8518 },
	{ 0xed99, 0xed99, PDF_CMAP_SINGLE, 9669 },
	{ 0xed9a, 0xed9a, PDF_CMAP_SINGLE, 8668 },
	{ 0xed9b, 0xed9b, PDF_CMAP_SINGLE, 20837 },
	{ 0xed9c, 0xed9c, PDF_CMAP_SINGLE, 9310 },
	{ 0xed9d, 0xed9d, PDF_CMAP_SINGLE, 20838 },
	{ 0xed9e, 0xed9e, PDF_CMAP_SINGLE, 8527 },
	{ 0xed9f, 0xed9f, PDF_CMAP_SINGLE, 20839 },
	{ 0xeda0, 0xeda0, PDF_CMAP_SINGLE, 9670 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 23780 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 25040 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 26300 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 27560 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 28820 },
	{ 0xee30, 0xee39, PDF_CMAP_RANGE, 29921 },
	{ 0xee39, 0xee39, PDF_CMAP_SINGLE, 22529 },
	{ 0xee40, 0xee40, PDF_CMAP_SINGLE, 9671 },
	{ 0xee41, 0xee41, PDF_CMAP_SINGLE, 8769 },
	{ 0xee42, 0xee42, PDF_CMAP_SINGLE, 8586 },
	{ 0xee43, 0xee43, PDF_CMAP_SINGLE, 7727 },
	{ 0xee44, 0xee44, PDF_CMAP_SINGLE, 7900 },
	{ 0xee45, 0xee47, PDF_CMAP_RANGE, 20840 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 8383 },
	{ 0xee49, 0xee49, PDF_CMAP_SINGLE, 8244 },
	{ 0xee4a, 0xee4c, PDF_CMAP_RANGE, 20843 },
	{ 0xee4d, 0xee4d, PDF_CMAP_SINGLE, 9673 },
	{ 0xee4e, 0xee51, PDF_CMAP_RANGE, 20846 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 9672 },
	{ 0xee53, 0xee54, PDF_CMAP_RANGE, 20850 },
	{ 0xee55, 0xee55, PDF_CMAP_SINGLE, 8718 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 20852 },
	{ 0xee57, 0xee57, PDF_CMAP_SINGLE, 9675 },
	{ 0xee58, 0xee5d, PDF_CMAP_RANGE, 20853 },
	{ 0xee5e, 0xee5e, PDF_CMAP_SINGLE, 8573 },
	{ 0xee5f, 0xee60, PDF_CMAP_RANGE, 20859 },
	{ 0xee61, 0xee61, PDF_CMAP_SINGLE, 8062 },
	{ 0xee62, 0xee67, PDF_CMAP_RANGE, 20861 },
	{ 0xee68, 0xee68, PDF_CMAP_SINGLE, 9676 },
	{ 0xee69, 0xee69, PDF_CMAP_SINGLE, 8131 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 20867 },
	{ 0xee6c, 0xee6c, PDF_CMAP_SINGLE, 8377 },
	{ 0xee6d, 0xee6d, PDF_CMAP_SINGLE, 20869 },
	{ 0xee6e, 0xee6e, PDF_CMAP_SINGLE, 8577 },
	{ 0xee6f, 0xee76, PDF_CMAP_RANGE, 20870 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 8154 },
	{ 0xee78, 0xee7c, PDF_CMAP_RANGE, 20878 },
	{ 0xee7d, 0xee7d, PDF_CMAP_SINGLE, 8563 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 7905 },
	{ 0xee80, 0xee80, PDF_CMAP_SINGLE, 9677 },
	{ 0xee81, 0xee84, PDF_CMAP_RANGE, 20883 },
	{ 0xee85, 0xee85, PDF_CMAP_SINGLE, 9678 },
	{ 0xee86, 0xee86, PDF_CMAP_SINGLE, 8694 },
	{ 0xee87, 0xee89, PDF_CMAP_RANGE, 20887 },
	{ 0xee8a, 0xee8a, PDF_CMAP_SINGLE, 8779 },
	{ 0xee8b, 0xee8b, PDF_CMAP_SINGLE, 9681 },
	{ 0xee8c, 0xee8c, PDF_CMAP_SINGLE, 20890 },
	{ 0xee8d, 0xee8d, PDF_CMAP_SINGLE, 7872 },
	{ 0xee8e, 0xee8f, PDF_CMAP_RANGE, 20891 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 8200 },
	{ 0xee91, 0xee93, PDF_CMAP_RANGE, 20893 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 9680 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 20896 },
	{ 0xee97, 0xee97, PDF_CMAP_SINGLE, 9682 },
	{ 0xee98, 0xee98, PDF_CMAP_SINGLE, 20898 },
	{ 0xee99, 0xee99, PDF_CMAP_SINGLE, 7978 },
	{ 0xee9a, 0xee9c, PDF_CMAP_RANGE, 20899 },
	{ 0xee9d, 0xee9d, PDF_CMAP_SINGLE, 7794 },
	{ 0xee9e, 0xee9e, PDF_CMAP_SINGLE, 9683 },
	{ 0xee9f, 0xeea0, PDF_CMAP_RANGE, 20902 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 22530 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 23790 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 25050 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 26310 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 27570 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 28830 },
	{ 0xef30, 0xef39, PDF_CMAP_RANGE, 29931 },
	{ 0xef40, 0xef40, PDF_CMAP_SINGLE, 8638 },
	{ 0xef41, 0xef41, PDF_CMAP_SINGLE, 9684 },
	{ 0xef42, 0xef42, PDF_CMAP_SINGLE, 8260 },
	{ 0xef43, 0xef43, PDF_CMAP_SINGLE, 20904 },
	{ 0xef44, 0xef44, PDF_CMAP_SINGLE, 9679 },
	{ 0xef45, 0xef45, PDF_CMAP_SINGLE, 8435 },
	{ 0xef46, 0xef4b, PDF_CMAP_RANGE, 20905 },
	{ 0xef4c, 0xef4c, PDF_CMAP_SINGLE, 7936 },
	{ 0xef4d, 0xef51, PDF_CMAP_RANGE, 20911 },
	{ 0xef52, 0xef53, PDF_CMAP_RANGE, 9420 },
	{ 0xef54, 0xef54, PDF_CMAP_SINGLE, 20916 },
	{ 0xef55, 0xef55, PDF_CMAP_SINGLE, 9880 },
	{ 0xef56, 0xef56, PDF_CMAP_SINGLE, 20917 },
	{ 0xef57, 0xef57, PDF_CMAP_SINGLE, 9848 },
	{ 0xef58, 0xef59, PDF_CMAP_RANGE, 20918 },
	{ 0xef5a, 0xef5a, PDF_CMAP_SINGLE, 9422 },
	{ 0xef5b, 0xef5f, PDF_CMAP_RANGE, 20920 },
	{ 0xef60, 0xef60, PDF_CMAP_SINGLE, 9423 },
	{ 0xef61, 0xef67, PDF_CMAP_RANGE, 20925 },
	{ 0xef68, 0xef68, PDF_CMAP_SINGLE, 8376 },
	{ 0xef69, 0xef69, PDF_CMAP_SINGLE, 20932 },
	{ 0xef6a, 0xef6a, PDF_CMAP_SINGLE, 9424 },
	{ 0xef6b, 0xef6b, PDF_CMAP_SINGLE, 20933 },
	{ 0xef6c, 0xef6c, PDF_CMAP_SINGLE, 9425 },
	{ 0xef6d, 0xef76, PDF_CMAP_RANGE, 20934 },
	{ 0xef77, 0xef77, PDF_CMAP_SINGLE, 7924 },
	{ 0xef78, 0xef79, PDF_CMAP_RANGE, 20944 },
	{ 0xef7a, 0xef7a, PDF_CMAP_SINGLE, 9115 },
	{ 0xef7b, 0xef7b, PDF_CMAP_SINGLE, 20946 },
	{ 0xef7c, 0xef7c, PDF_CMAP_SINGLE, 9854 },
	{ 0xef7d, 0xef7e, PDF_CMAP_RANGE, 20947 },
	{ 0xef80, 0xef81, PDF_CMAP_RANGE, 20949 },
	{ 0xef82, 0xef82, PDF_CMAP_SINGLE, 9117 },
	{ 0xef83, 0xef84, PDF_CMAP_RANGE, 9119 },
	{ 0xef85, 0xef85, PDF_CMAP_SINGLE, 20951 },
	{ 0xef86, 0xef86, PDF_CMAP_SINGLE, 9121 },
	{ 0xef87, 0xef87, PDF_CMAP_SINGLE, 20952 },
	{ 0xef88, 0xef88, PDF_CMAP_SINGLE, 7921 },
	{ 0xef89, 0xef8a, PDF_CMAP_RANGE, 20953 },
	{ 0xef8b, 0xef8b, PDF_CMAP_SINGLE, 8734 },
	{ 0xef8c, 0xef8c, PDF_CMAP_SINGLE, 20955 },
	{ 0xef8d, 0xef8d, PDF_CMAP_SINGLE, 9122 },
	{ 0xef8e, 0xef94, PDF_CMAP_RANGE, 20956 },
	{ 0xef95, 0xef95, PDF_CMAP_SINGLE, 8523 },
	{ 0xef96, 0xef96, PDF_CMAP_SINGLE, 7734 },
	{ 0xef97, 0xef97, PDF_CMAP_SINGLE, 8501 },
	{ 0xef98, 0xef9b, PDF_CMAP_RANGE, 20963 },
	{ 0xef9c, 0xef9c, PDF_CMAP_SINGLE, 8109 },
	{ 0xef9d, 0xef9d, PDF_CMAP_SINGLE, 20967 },
	{ 0xef9e, 0xef9e, PDF_CMAP_SINGLE, 7763 },
	{ 0xef9f, 0xefa0, PDF_CMAP_RANGE, 20968 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 22540 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 23800 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 25060 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 26320 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 27580 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 28840 },
	{ 0xf030, 0xf039, PDF_CMAP_RANGE, 29941 },
	{ 0xf040, 0xf040, PDF_CMAP_SINGLE, 20970 },
	{ 0xf041, 0xf041, PDF_CMAP_SINGLE, 9123 },
	{ 0xf042, 0xf042, PDF_CMAP_SINGLE, 8707 },
	{ 0xf043, 0xf043, PDF_CMAP_SINGLE, 20971 },
	{ 0xf044, 0xf044, PDF_CMAP_SINGLE, 7911 },
	{ 0xf045, 0xf046, PDF_CMAP_RANGE, 20972 },
	{ 0xf047, 0xf047, PDF_CMAP_SINGLE, 9124 },
	{ 0xf048, 0xf048, PDF_CMAP_SINGLE, 8343 },
	{ 0xf049, 0xf049, PDF_CMAP_SINGLE, 7908 },
	{ 0xf04a, 0xf04d, PDF_CMAP_RANGE, 20974 },
	{ 0xf04e, 0xf04e, PDF_CMAP_SINGLE, 8760 },
	{ 0xf04f, 0xf050, PDF_CMAP_RANGE, 20978 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 9125 },
	{ 0xf052, 0xf053, PDF_CMAP_RANGE, 20980 },
	{ 0xf054, 0xf054, PDF_CMAP_SINGLE, 8090 },
	{ 0xf055, 0xf056, PDF_CMAP_RANGE, 20982 },
	{ 0xf057, 0xf057, PDF_CMAP_SINGLE, 8643 },
	{ 0xf058, 0xf05d, PDF_CMAP_RANGE, 20984 },
	{ 0xf05e, 0xf05e, PDF_CMAP_SINGLE, 7982 },
	{ 0xf05f, 0xf067, PDF_CMAP_RANGE, 20990 },
	{ 0xf068, 0xf068, PDF_CMAP_SINGLE, 9116 },
	{ 0xf069, 0xf06b, PDF_CMAP_RANGE, 20999 },
	{ 0xf06c, 0xf06c, PDF_CMAP_SINGLE, 9126 },
	{ 0xf06d, 0xf070, PDF_CMAP_RANGE, 21002 },
	{ 0xf071, 0xf071, PDF_CMAP_SINGLE, 9118 },
	{ 0xf072, 0xf072, PDF_CMAP_SINGLE, 21006 },
	{ 0xf073, 0xf073, PDF_CMAP_SINGLE, 8245 },
	{ 0xf074, 0xf074, PDF_CMAP_SINGLE, 9127 },
	{ 0xf075, 0xf077, PDF_CMAP_RANGE, 21007 },
	{ 0xf078, 0xf078, PDF_CMAP_SINGLE, 9128 },
	{ 0xf079, 0xf079, PDF_CMAP_SINGLE, 21010 },
	{ 0xf07a, 0xf07a, PDF_CMAP_SINGLE, 8309 },
	{ 0xf07b, 0xf07c, PDF_CMAP_RANGE, 21011 },
	{ 0xf07d, 0xf07e, PDF_CMAP_RANGE, 9129 },
	{ 0xf080, 0xf080, PDF_CMAP_SINGLE, 9131 },
	{ 0xf081, 0xf081, PDF_CMAP_SINGLE, 8171 },
	{ 0xf082, 0xf082, PDF_CMAP_SINGLE, 9132 },
	{ 0xf083, 0xf086, PDF_CMAP_RANGE, 21013 },
	{ 0xf087, 0xf087, PDF_CMAP_SINGLE, 8042 },
	{ 0xf088, 0xf088, PDF_CMAP_SINGLE, 8441 },
	{ 0xf089, 0xf08a, PDF_CMAP_RANGE, 21017 },
	{ 0xf08b, 0xf08b, PDF_CMAP_SINGLE, 9830 },
	{ 0xf08c, 0xf08f, PDF_CMAP_RANGE, 21019 },
	{ 0xf090, 0xf090, PDF_CMAP_SINGLE, 9831 },
	{ 0xf091, 0xf091, PDF_CMAP_SINGLE, 21023 },
	{ 0xf092, 0xf092, PDF_CMAP_SINGLE, 7788 },
	{ 0xf093, 0xf095, PDF_CMAP_RANGE, 21024 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 9133 },
	{ 0xf097, 0xf0a0, PDF_CMAP_RANGE, 21027 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 22550 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 23810 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 25070 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 26330 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 27590 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 28850 },
	{ 0xf130, 0xf139, PDF_CMAP_RANGE, 29951 },
	{ 0xf140, 0xf151, PDF_CMAP_RANGE, 21037 },
	{ 0xf152, 0xf152, PDF_CMAP_SINGLE, 8301 },
	{ 0xf153, 0xf153, PDF_CMAP_SINGLE, 8770 },
	{ 0xf154, 0xf154, PDF_CMAP_SINGLE, 7938 },
	{ 0xf155, 0xf156, PDF_CMAP_RANGE, 21055 },
	{ 0xf157, 0xf157, PDF_CMAP_SINGLE, 8579 },
	{ 0xf158, 0xf158, PDF_CMAP_SINGLE, 21057 },
	{ 0xf159, 0xf159, PDF_CMAP_SINGLE, 7813 },
	{ 0xf15a, 0xf15a, PDF_CMAP_SINGLE, 8681 },
	{ 0xf15b, 0xf166, PDF_CMAP_RANGE, 21058 },
	{ 0xf167, 0xf167, PDF_CMAP_SINGLE, 7767 },
	{ 0xf168, 0xf175, PDF_CMAP_RANGE, 21070 },
	{ 0xf176, 0xf176, PDF_CMAP_SINGLE, 8869 },
	{ 0xf177, 0xf177, PDF_CMAP_SINGLE, 9223 },
	{ 0xf178, 0xf178, PDF_CMAP_SINGLE, 8138 },
	{ 0xf179, 0xf179, PDF_CMAP_SINGLE, 21084 },
	{ 0xf17a, 0xf17a, PDF_CMAP_SINGLE, 9218 },
	{ 0xf17b, 0xf17b, PDF_CMAP_SINGLE, 8066 },
	{ 0xf17c, 0xf17d, PDF_CMAP_RANGE, 21085 },
	{ 0xf17e, 0xf17e, PDF_CMAP_SINGLE, 9224 },
	{ 0xf180, 0xf180, PDF_CMAP_SINGLE, 9220 },
	{ 0xf181, 0xf181, PDF_CMAP_SINGLE, 21087 },
	{ 0xf182, 0xf182, PDF_CMAP_SINGLE, 8497 },
	{ 0xf183, 0xf183, PDF_CMAP_SINGLE, 21088 },
	{ 0xf184, 0xf184, PDF_CMAP_SINGLE, 8580 },
	{ 0xf185, 0xf185, PDF_CMAP_SINGLE, 21089 },
	{ 0xf186, 0xf186, PDF_CMAP_SINGLE, 9219 },
	{ 0xf187, 0xf187, PDF_CMAP_SINGLE, 21090 },
	{ 0xf188, 0xf188, PDF_CMAP_SINGLE, 8302 },
	{ 0xf189, 0xf189, PDF_CMAP_SINGLE, 9227 },
	{ 0xf18a, 0xf193, PDF_CMAP_RANGE, 21091 },
	{ 0xf194, 0xf194, PDF_CMAP_SINGLE, 7999 },
	{ 0xf195, 0xf197, PDF_CMAP_RANGE, 21101 },
	{ 0xf198, 0xf198, PDF_CMAP_SINGLE, 8295 },
	{ 0xf199, 0xf1a0, PDF_CMAP_RANGE, 21104 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 22560 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 23820 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 25080 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 26340 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 27600 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 28860 },
	{ 0xf230, 0xf239, PDF_CMAP_RANGE, 29961 },
	{ 0xf240, 0xf244, PDF_CMAP_RANGE, 21112 },
	{ 0xf245, 0xf245, PDF_CMAP_SINGLE, 8151 },
	{ 0xf246, 0xf246, PDF_CMAP_SINGLE, 21117 },
	{ 0xf247, 0xf247, PDF_CMAP_SINGLE, 7811 },
	{ 0xf248, 0xf24a, PDF_CMAP_RANGE, 21118 },
	{ 0xf24b, 0xf24b, PDF_CMAP_SINGLE, 9231 },
	{ 0xf24c, 0xf252, PDF_CMAP_RANGE, 21121 },
	{ 0xf253, 0xf253, PDF_CMAP_SINGLE, 9230 },
	{ 0xf254, 0xf254, PDF_CMAP_SINGLE, 8391 },
	{ 0xf255, 0xf255, PDF_CMAP_SINGLE, 9229 },
	{ 0xf256, 0xf25b, PDF_CMAP_RANGE, 21128 },
	{ 0xf25c, 0xf25c, PDF_CMAP_SINGLE, 9234 },
	{ 0xf25d, 0xf25e, PDF_CMAP_RANGE, 21134 },
	{ 0xf25f, 0xf25f, PDF_CMAP_SINGLE, 8375 },
	{ 0xf260, 0xf270, PDF_CMAP_RANGE, 21136 },
	{ 0xf271, 0xf271, PDF_CMAP_SINGLE, 9200 },
	{ 0xf272, 0xf272, PDF_CMAP_SINGLE, 21153 },
	{ 0xf273, 0xf273, PDF_CMAP_SINGLE, 9233 },
	{ 0xf274, 0xf274, PDF_CMAP_SINGLE, 9236 },
	{ 0xf275, 0xf275, PDF_CMAP_SINGLE, 21154 },
	{ 0xf276, 0xf276, PDF_CMAP_SINGLE, 8560 },
	{ 0xf277, 0xf27b, PDF_CMAP_RANGE, 21155 },
	{ 0xf27c, 0xf27c, PDF_CMAP_SINGLE, 9221 },
	{ 0xf27d, 0xf27d, PDF_CMAP_SINGLE, 8460 },
	{ 0xf27e, 0xf27e, PDF_CMAP_SINGLE, 9237 },
	{ 0xf280, 0xf284, PDF_CMAP_RANGE, 21160 },
	{ 0xf285, 0xf285, PDF_CMAP_SINGLE, 8294 },
	{ 0xf286, 0xf286, PDF_CMAP_SINGLE, 21165 },
	{ 0xf287, 0xf287, PDF_CMAP_SINGLE, 9042 },
	{ 0xf288, 0xf288, PDF_CMAP_SINGLE, 9235 },
	{ 0xf289, 0xf289, PDF_CMAP_SINGLE, 9232 },
	{ 0xf28a, 0xf28b, PDF_CMAP_RANGE, 9238 },
	{ 0xf28c, 0xf28c, PDF_CMAP_SINGLE, 8433 },
	{ 0xf28d, 0xf290, PDF_CMAP_RANGE, 21166 },
	{ 0xf291, 0xf291, PDF_CMAP_SINGLE, 9226 },
	{ 0xf292, 0xf293, PDF_CMAP_RANGE, 21170 },
	{ 0xf294, 0xf294, PDF_CMAP_SINGLE, 9225 },
	{ 0xf295, 0xf295, PDF_CMAP_SINGLE, 21172 },
	{ 0xf296, 0xf296, PDF_CMAP_SINGLE, 9240 },
	{ 0xf297, 0xf29b, PDF_CMAP_RANGE, 21173 },
	{ 0xf29c, 0xf29c, PDF_CMAP_SINGLE, 8103 },
	{ 0xf29d, 0xf29d, PDF_CMAP_SINGLE, 21178 },
	{ 0xf29e, 0xf29e, PDF_CMAP_SINGLE, 8700 },
	{ 0xf29f, 0xf2a0, PDF_CMAP_RANGE, 21179 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 22570 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 23830 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 25090 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 26350 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 27610 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 28870 },
	{ 0xf330, 0xf339, PDF_CMAP_RANGE, 29971 },
	{ 0xf340, 0xf340, PDF_CMAP_SINGLE, 8129 },
	{ 0xf341, 0xf341, PDF_CMAP_SINGLE, 9222 },
	{ 0xf342, 0xf344, PDF_CMAP_RANGE, 21181 },
	{ 0xf345, 0xf345, PDF_CMAP_SINGLE, 8860 },
	{ 0xf346, 0xf347, PDF_CMAP_RANGE, 21184 },
	{ 0xf348, 0xf348, PDF_CMAP_SINGLE, 8270 },
	{ 0xf349, 0xf349, PDF_CMAP_SINGLE, 21186 },
	{ 0xf34a, 0xf34a, PDF_CMAP_SINGLE, 9242 },
	{ 0xf34b, 0xf34b, PDF_CMAP_SINGLE, 9241 },
	{ 0xf34c, 0xf34f, PDF_CMAP_RANGE, 21187 },
	{ 0xf350, 0xf350, PDF_CMAP_SINGLE, 9228 },
	{ 0xf351, 0xf360, PDF_CMAP_RANGE, 21191 },
	{ 0xf361, 0xf361, PDF_CMAP_SINGLE, 7721 },
	{ 0xf362, 0xf373, PDF_CMAP_RANGE, 21207 },
	{ 0xf374, 0xf374, PDF_CMAP_SINGLE, 9825 },
	{ 0xf375, 0xf375, PDF_CMAP_SINGLE, 21225 },
	{ 0xf376, 0xf376, PDF_CMAP_SINGLE, 9892 },
	{ 0xf377, 0xf377, PDF_CMAP_SINGLE, 8564 },
	{ 0xf378, 0xf378, PDF_CMAP_SINGLE, 9827 },
	{ 0xf379, 0xf379, PDF_CMAP_SINGLE, 9826 },
	{ 0xf37a, 0xf37e, PDF_CMAP_RANGE, 21226 },
	{ 0xf380, 0xf38b, PDF_CMAP_RANGE, 21231 },
	{ 0xf38c, 0xf38c, PDF_CMAP_SINGLE, 9845 },
	{ 0xf38d, 0xf39f, PDF_CMAP_RANGE, 21243 },
	{ 0xf3a0, 0xf3a0, PDF_CMAP_SINGLE, 8524 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 22580 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 23840 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 25100 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 26360 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 27620 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 28880 },
	{ 0xf430, 0xf439, PDF_CMAP_RANGE, 29981 },
	{ 0xf440, 0xf444, PDF_CMAP_RANGE, 21262 },
	{ 0xf445, 0xf445, PDF_CMAP_SINGLE, 9850 },
	{ 0xf446, 0xf44f, PDF_CMAP_RANGE, 21267 },
	{ 0xf450, 0xf450, PDF_CMAP_SINGLE, 9888 },
	{ 0xf451, 0xf456, PDF_CMAP_RANGE, 21277 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 9832 },
	{ 0xf458, 0xf458, PDF_CMAP_SINGLE, 21283 },
	{ 0xf459, 0xf459, PDF_CMAP_SINGLE, 7888 },
	{ 0xf45a, 0xf45a, PDF_CMAP_SINGLE, 21284 },
	{ 0xf45b, 0xf45b, PDF_CMAP_SINGLE, 8342 },
	{ 0xf45c, 0xf45c, PDF_CMAP_SINGLE, 21285 },
	{ 0xf45d, 0xf45d, PDF_CMAP_SINGLE, 9164 },
	{ 0xf45e, 0xf461, PDF_CMAP_RANGE, 21286 },
	{ 0xf462, 0xf462, PDF_CMAP_SINGLE, 9160 },
	{ 0xf463, 0xf463, PDF_CMAP_SINGLE, 21290 },
	{ 0xf464, 0xf464, PDF_CMAP_SINGLE, 8766 },
	{ 0xf465, 0xf474, PDF_CMAP_RANGE, 21291 },
	{ 0xf475, 0xf475, PDF_CMAP_SINGLE, 9829 },
	{ 0xf476, 0xf47b, PDF_CMAP_RANGE, 21307 },
	{ 0xf47c, 0xf47c, PDF_CMAP_SINGLE, 9828 },
	{ 0xf47d, 0xf47d, PDF_CMAP_SINGLE, 21313 },
	{ 0xf47e, 0xf47e, PDF_CMAP_SINGLE, 8761 },
	{ 0xf480, 0xf493, PDF_CMAP_RANGE, 21314 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 8266 },
	{ 0xf495, 0xf498, PDF_CMAP_RANGE, 21334 },
	{ 0xf499, 0xf499, PDF_CMAP_SINGLE, 9759 },
	{ 0xf49a, 0xf49b, PDF_CMAP_RANGE, 21338 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 9758 },
	{ 0xf49d, 0xf4a0, PDF_CMAP_RANGE, 21340 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 22590 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 23850 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 25110 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 26370 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 27630 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 28890 },
	{ 0xf530, 0xf539, PDF_CMAP_RANGE, 29991 },
	{ 0xf540, 0xf544, PDF_CMAP_RANGE, 21344 },
	{ 0xf545, 0xf545, PDF_CMAP_SINGLE, 9760 },
	{ 0xf546, 0xf546, PDF_CMAP_SINGLE, 21349 },
	{ 0xf547, 0xf547, PDF_CMAP_SINGLE, 9761 },
	{ 0xf548, 0xf551, PDF_CMAP_RANGE, 21350 },
	{ 0xf552, 0xf552, PDF_CMAP_SINGLE, 9762 },
	{ 0xf553, 0xf553, PDF_CMAP_SINGLE, 21360 },
	{ 0xf554, 0xf554, PDF_CMAP_SINGLE, 9767 },
	{ 0xf555, 0xf555, PDF_CMAP_SINGLE, 7737 },
	{ 0xf556, 0xf556, PDF_CMAP_SINGLE, 9765 },
	{ 0xf557, 0xf55d, PDF_CMAP_RANGE, 21361 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 9769 },
	{ 0xf55f, 0xf560, PDF_CMAP_RANGE, 21368 },
	{ 0xf561, 0xf561, PDF_CMAP_SINGLE, 9774 },
	{ 0xf562, 0xf562, PDF_CMAP_SINGLE, 9771 },
	{ 0xf563, 0xf56d, PDF_CMAP_RANGE, 21370 },
	{ 0xf56e, 0xf56e, PDF_CMAP_SINGLE, 9770 },
	{ 0xf56f, 0xf56f, PDF_CMAP_SINGLE, 9773 },
	{ 0xf570, 0xf570, PDF_CMAP_SINGLE, 21381 },
	{ 0xf571, 0xf571, PDF_CMAP_SINGLE, 9768 },
	{ 0xf572, 0xf572, PDF_CMAP_SINGLE, 8633 },
	{ 0xf573, 0xf57e, PDF_CMAP_RANGE, 21382 },
	{ 0xf580, 0xf584, PDF_CMAP_RANGE, 21394 },
	{ 0xf585, 0xf585, PDF_CMAP_SINGLE, 9782 },
	{ 0xf586, 0xf586, PDF_CMAP_SINGLE, 9776 },
	{ 0xf587, 0xf58b, PDF_CMAP_RANGE, 21399 },
	{ 0xf58c, 0xf58c, PDF_CMAP_SINGLE, 9784 },
	{ 0xf58d, 0xf58d, PDF_CMAP_SINGLE, 21404 },
	{ 0xf58e, 0xf58e, PDF_CMAP_SINGLE, 8205 },
	{ 0xf58f, 0xf58f, PDF_CMAP_SINGLE, 9783 },
	{ 0xf590, 0xf598, PDF_CMAP_RANGE, 21405 },
	{ 0xf599, 0xf599, PDF_CMAP_SINGLE, 9797 },
	{ 0xf59a, 0xf59a, PDF_CMAP_SINGLE, 21414 },
	{ 0xf59b, 0xf59b, PDF_CMAP_SINGLE, 9786 },
	{ 0xf59c, 0xf59f, PDF_CMAP_RANGE, 21415 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 9795 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 22600 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 23860 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 25120 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 26380 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 27640 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 28900 },
	{ 0xf630, 0xf639, PDF_CMAP_RANGE, 30001 },
	{ 0xf640, 0xf640, PDF_CMAP_SINGLE, 21419 },
	{ 0xf641, 0xf641, PDF_CMAP_SINGLE, 9792 },
	{ 0xf642, 0xf644, PDF_CMAP_RANGE, 21420 },
	{ 0xf645, 0xf645, PDF_CMAP_SINGLE, 9789 },
	{ 0xf646, 0xf646, PDF_CMAP_SINGLE, 9793 },
	{ 0xf647, 0xf647, PDF_CMAP_SINGLE, 21423 },
	{ 0xf648, 0xf648, PDF_CMAP_SINGLE, 9790 },
	{ 0xf649, 0xf64a, PDF_CMAP_RANGE, 21424 },
	{ 0xf64b, 0xf64b, PDF_CMAP_SINGLE, 9791 },
	{ 0xf64c, 0xf64c, PDF_CMAP_SINGLE, 8128 },
	{ 0xf64d, 0xf64d, PDF_CMAP_SINGLE, 21426 },
	{ 0xf64e, 0xf64f, PDF_CMAP_RANGE, 9787 },
	{ 0xf650, 0xf653, PDF_CMAP_RANGE, 21427 },
	{ 0xf654, 0xf654, PDF_CMAP_SINGLE, 9794 },
	{ 0xf655, 0xf657, PDF_CMAP_RANGE, 21431 },
	{ 0xf658, 0xf658, PDF_CMAP_SINGLE, 9796 },
	{ 0xf659, 0xf660, PDF_CMAP_RANGE, 21434 },
	{ 0xf661, 0xf661, PDF_CMAP_SINGLE, 9785 },
	{ 0xf662, 0xf662, PDF_CMAP_SINGLE, 21442 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 9804 },
	{ 0xf664, 0xf66b, PDF_CMAP_RANGE, 21443 },
	{ 0xf66c, 0xf66c, PDF_CMAP_SINGLE, 9799 },
	{ 0xf66d, 0xf66d, PDF_CMAP_SINGLE, 9803 },
	{ 0xf66e, 0xf670, PDF_CMAP_RANGE, 21451 },
	{ 0xf671, 0xf671, PDF_CMAP_SINGLE, 9801 },
	{ 0xf672, 0xf673, PDF_CMAP_RANGE, 21454 },
	{ 0xf674, 0xf674, PDF_CMAP_SINGLE, 9800 },
	{ 0xf675, 0xf675, PDF_CMAP_SINGLE, 21456 },
	{ 0xf676, 0xf676, PDF_CMAP_SINGLE, 9802 },
	{ 0xf677, 0xf677, PDF_CMAP_SINGLE, 8456 },
	{ 0xf678, 0xf67e, PDF_CMAP_RANGE, 21457 },
	{ 0xf680, 0xf684, PDF_CMAP_RANGE, 21464 },
	{ 0xf685, 0xf685, PDF_CMAP_SINGLE, 9805 },
	{ 0xf686, 0xf687, PDF_CMAP_RANGE, 21469 },
	{ 0xf688, 0xf688, PDF_CMAP_SINGLE, 9780 },
	{ 0xf689, 0xf689, PDF_CMAP_SINGLE, 21471 },
	{ 0xf68a, 0xf68a, PDF_CMAP_SINGLE, 9809 },
	{ 0xf68b, 0xf68c, PDF_CMAP_RANGE, 21472 },
	{ 0xf68d, 0xf68d, PDF_CMAP_SINGLE, 9808 },
	{ 0xf68e, 0xf68e, PDF_CMAP_SINGLE, 9810 },
	{ 0xf68f, 0xf691, PDF_CMAP_RANGE, 21474 },
	{ 0xf692, 0xf692, PDF_CMAP_SINGLE, 9807 },
	{ 0xf693, 0xf695, PDF_CMAP_RANGE, 21477 },
	{ 0xf696, 0xf696, PDF_CMAP_SINGLE, 9778 },
	{ 0xf697, 0xf697, PDF_CMAP_SINGLE, 9806 },
	{ 0xf698, 0xf698, PDF_CMAP_SINGLE, 9811 },
	{ 0xf699, 0xf699, PDF_CMAP_SINGLE, 21480 },
	{ 0xf69a, 0xf69a, PDF_CMAP_SINGLE, 9815 },
	{ 0xf69b, 0xf69b, PDF_CMAP_SINGLE, 21481 },
	{ 0xf69c, 0xf69c, PDF_CMAP_SINGLE, 9781 },
	{ 0xf69d, 0xf69d, PDF_CMAP_SINGLE, 21482 },
	{ 0xf69e, 0xf69e, PDF_CMAP_SINGLE, 9779 },
	{ 0xf69f, 0xf69f, PDF_CMAP_SINGLE, 21483 },
	{ 0xf6a0, 0xf6a0, PDF_CMAP_SINGLE, 9814 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 22610 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 23870 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 25130 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 26390 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 27650 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 28910 },
	{ 0xf730, 0xf739, PDF_CMAP_RANGE, 30011 },
	{ 0xf740, 0xf741, PDF_CMAP_RANGE, 21484 },
	{ 0xf742, 0xf742, PDF_CMAP_SINGLE, 9812 },
	{ 0xf743, 0xf748, PDF_CMAP_RANGE, 21486 },
	{ 0xf749, 0xf749, PDF_CMAP_SINGLE, 9816 },
	{ 0xf74a, 0xf74b, PDF_CMAP_RANGE, 21492 },
	{ 0xf74c, 0xf74c, PDF_CMAP_SINGLE, 9813 },
	{ 0xf74d, 0xf74d, PDF_CMAP_SINGLE, 7757 },
	{ 0xf74e, 0xf755, PDF_CMAP_RANGE, 21494 },
	{ 0xf756, 0xf756, PDF_CMAP_SINGLE, 9819 },
	{ 0xf757, 0xf757, PDF_CMAP_SINGLE, 21502 },
	{ 0xf758, 0xf758, PDF_CMAP_SINGLE, 9818 },
	{ 0xf759, 0xf759, PDF_CMAP_SINGLE, 21503 },
	{ 0xf75a, 0xf75a, PDF_CMAP_SINGLE, 9817 },
	{ 0xf75b, 0xf75b, PDF_CMAP_SINGLE, 8238 },
	{ 0xf75c, 0xf75c, PDF_CMAP_SINGLE, 9775 },
	{ 0xf75d, 0xf760, PDF_CMAP_RANGE, 21504 },
	{ 0xf761, 0xf761, PDF_CMAP_SINGLE, 9798 },
	{ 0xf762, 0xf762, PDF_CMAP_SINGLE, 21508 },
	{ 0xf763, 0xf763, PDF_CMAP_SINGLE, 9766 },
	{ 0xf764, 0xf76a, PDF_CMAP_RANGE, 21509 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 9820 },
	{ 0xf76c, 0xf770, PDF_CMAP_RANGE, 21516 },
	{ 0xf771, 0xf771, PDF_CMAP_SINGLE, 9772 },
	{ 0xf772, 0xf77b, PDF_CMAP_RANGE, 21521 },
	{ 0xf77c, 0xf77c, PDF_CMAP_SINGLE, 9763 },
	{ 0xf77d, 0xf77d, PDF_CMAP_SINGLE, 21531 },
	{ 0xf77e, 0xf77e, PDF_CMAP_SINGLE, 9777 },
	{ 0xf780, 0xf7a0, PDF_CMAP_RANGE, 21532 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 22620 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 23880 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 25140 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 26400 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 27660 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 28920 },
	{ 0xf830, 0xf839, PDF_CMAP_RANGE, 30021 },
	{ 0xf840, 0xf841, PDF_CMAP_RANGE, 21565 },
	{ 0xf842, 0xf842, PDF_CMAP_SINGLE, 8348 },
	{ 0xf843, 0xf845, PDF_CMAP_RANGE, 21567 },
	{ 0xf846, 0xf846, PDF_CMAP_SINGLE, 9600 },
	{ 0xf847, 0xf848, PDF_CMAP_RANGE, 21570 },
	{ 0xf849, 0xf849, PDF_CMAP_SINGLE, 8924 },
	{ 0xf84a, 0xf84f, PDF_CMAP_RANGE, 21572 },
	{ 0xf850, 0xf850, PDF_CMAP_SINGLE, 7941 },
	{ 0xf851, 0xf851, PDF_CMAP_SINGLE, 8331 },
	{ 0xf852, 0xf852, PDF_CMAP_SINGLE, 21578 },
	{ 0xf853, 0xf853, PDF_CMAP_SINGLE, 9601 },
	{ 0xf854, 0xf862, PDF_CMAP_RANGE, 21579 },
	{ 0xf863, 0xf863, PDF_CMAP_SINGLE, 9603 },
	{ 0xf864, 0xf864, PDF_CMAP_SINGLE, 9602 },
	{ 0xf865, 0xf865, PDF_CMAP_SINGLE, 21594 },
	{ 0xf866, 0xf866, PDF_CMAP_SINGLE, 8686 },
	{ 0xf867, 0xf871, PDF_CMAP_RANGE, 21595 },
	{ 0xf872, 0xf872, PDF_CMAP_SINGLE, 8578 },
	{ 0xf873, 0xf877, PDF_CMAP_RANGE, 21606 },
	{ 0xf878, 0xf878, PDF_CMAP_SINGLE, 8771 },
	{ 0xf879, 0xf879, PDF_CMAP_SINGLE, 21611 },
	{ 0xf87a, 0xf87a, PDF_CMAP_SINGLE, 9607 },
	{ 0xf87b, 0xf87b, PDF_CMAP_SINGLE, 21612 },
	{ 0xf87c, 0xf87c, PDF_CMAP_SINGLE, 9608 },
	{ 0xf87d, 0xf87e, PDF_CMAP_RANGE, 21613 },
	{ 0xf880, 0xf880, PDF_CMAP_SINGLE, 21615 },
	{ 0xf881, 0xf881, PDF_CMAP_SINGLE, 9604 },
	{ 0xf882, 0xf883, PDF_CMAP_RANGE, 21616 },
	{ 0xf884, 0xf884, PDF_CMAP_SINGLE, 8701 },
	{ 0xf885, 0xf885, PDF_CMAP_SINGLE, 21618 },
	{ 0xf886, 0xf886, PDF_CMAP_SINGLE, 8687 },
	{ 0xf887, 0xf88c, PDF_CMAP_RANGE, 21619 },
	{ 0xf88d, 0xf88d, PDF_CMAP_SINGLE, 9610 },
	{ 0xf88e, 0xf88e, PDF_CMAP_SINGLE, 9612 },
	{ 0xf88f, 0xf898, PDF_CMAP_RANGE, 21625 },
	{ 0xf899, 0xf899, PDF_CMAP_SINGLE, 8007 },
	{ 0xf89a, 0xf89c, PDF_CMAP_RANGE, 21635 },
	{ 0xf89d, 0xf89d, PDF_CMAP_SINGLE, 7965 },
	{ 0xf89e, 0xf89f, PDF_CMAP_RANGE, 21638 },
	{ 0xf8a0, 0xf8a0, PDF_CMAP_SINGLE, 9613 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 22630 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 23890 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 25150 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 26410 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 27670 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 28930 },
	{ 0xf930, 0xf939, PDF_CMAP_RANGE, 30031 },
	{ 0xf940, 0xf94d, PDF_CMAP_RANGE, 21640 },
	{ 0xf94e, 0xf94e, PDF_CMAP_SINGLE, 8144 },
	{ 0xf94f, 0xf94f, PDF_CMAP_SINGLE, 9618 },
	{ 0xf950, 0xf950, PDF_CMAP_SINGLE, 9615 },
	{ 0xf951, 0xf958, PDF_CMAP_RANGE, 21654 },
	{ 0xf959, 0xf959, PDF_CMAP_SINGLE, 9620 },
	{ 0xf95a, 0xf95a, PDF_CMAP_SINGLE, 7904 },
	{ 0xf95b, 0xf95c, PDF_CMAP_RANGE, 21662 },
	{ 0xf95d, 0xf95d, PDF_CMAP_SINGLE, 9617 },
	{ 0xf95e, 0xf95e, PDF_CMAP_SINGLE, 9621 },
	{ 0xf95f, 0xf966, PDF_CMAP_RANGE, 21664 },
	{ 0xf967, 0xf967, PDF_CMAP_SINGLE, 9623 },
	{ 0xf968, 0xf968, PDF_CMAP_SINGLE, 21672 },
	{ 0xf969, 0xf969, PDF_CMAP_SINGLE, 8374 },
	{ 0xf96a, 0xf96b, PDF_CMAP_RANGE, 21673 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 9624 },
	{ 0xf96d, 0xf96e, PDF_CMAP_RANGE, 21675 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 8438 },
	{ 0xf970, 0xf97e, PDF_CMAP_RANGE, 21677 },
	{ 0xf980, 0xf984, PDF_CMAP_RANGE, 21692 },
	{ 0xf985, 0xf985, PDF_CMAP_SINGLE, 9605 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 21697 },
	{ 0xf987, 0xf987, PDF_CMAP_SINGLE, 9625 },
	{ 0xf988, 0xf990, PDF_CMAP_RANGE, 21698 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 9622 },
	{ 0xf992, 0xf995, PDF_CMAP_RANGE, 21707 },
	{ 0xf996, 0xf996, PDF_CMAP_SINGLE, 9626 },
	{ 0xf997, 0xf997, PDF_CMAP_SINGLE, 21711 },
	{ 0xf998, 0xf998, PDF_CMAP_SINGLE, 9627 },
	{ 0xf999, 0xf9a0, PDF_CMAP_RANGE, 21712 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 22640 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 23900 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 25160 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 26420 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 27680 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 28940 },
	{ 0xfa30, 0xfa39, PDF_CMAP_RANGE, 30041 },
	{ 0xfa40, 0xfa41, PDF_CMAP_RANGE, 21720 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 9629 },
	{ 0xfa43, 0xfa45, PDF_CMAP_RANGE, 21722 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 9630 },
	{ 0xfa47, 0xfa4b, PDF_CMAP_RANGE, 21725 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 9036 },
	{ 0xfa4d, 0xfa50, PDF_CMAP_RANGE, 21730 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 8004 },
	{ 0xfa52, 0xfa57, PDF_CMAP_RANGE, 21734 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 9824 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 9632 },
	{ 0xfa5a, 0xfa5c, PDF_CMAP_RANGE, 21740 },
	{ 0xfa5d, 0xfa5d, PDF_CMAP_SINGLE, 9628 },
	{ 0xfa5e, 0xfa5e, PDF_CMAP_SINGLE, 21743 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 9631 },
	{ 0xfa60, 0xfa60, PDF_CMAP_SINGLE, 21744 },
	{ 0xfa61, 0xfa61, PDF_CMAP_SINGLE, 8044 },
	{ 0xfa62, 0xfa6f, PDF_CMAP_RANGE, 21745 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 9634 },
	{ 0xfa71, 0xfa73, PDF_CMAP_RANGE, 21759 },
	{ 0xfa74, 0xfa74, PDF_CMAP_SINGLE, 8366 },
	{ 0xfa75, 0xfa75, PDF_CMAP_SINGLE, 21762 },
	{ 0xfa76, 0xfa76, PDF_CMAP_SINGLE, 9611 },
	{ 0xfa77, 0xfa77, PDF_CMAP_SINGLE, 9635 },
	{ 0xfa78, 0xfa7e, PDF_CMAP_RANGE, 21763 },
	{ 0xfa80, 0xfa82, PDF_CMAP_RANGE, 21770 },
	{ 0xfa83, 0xfa83, PDF_CMAP_SINGLE, 9609 },
	{ 0xfa84, 0xfa84, PDF_CMAP_SINGLE, 9637 },
	{ 0xfa85, 0xfa8c, PDF_CMAP_RANGE, 21773 },
	{ 0xfa8d, 0xfa8d, PDF_CMAP_SINGLE, 9636 },
	{ 0xfa8e, 0xfa8f, PDF_CMAP_RANGE, 21781 },
	{ 0xfa90, 0xfa90, PDF_CMAP_SINGLE, 9638 },
	{ 0xfa91, 0xfa91, PDF_CMAP_SINGLE, 9619 },
	{ 0xfa92, 0xfa95, PDF_CMAP_RANGE, 21783 },
	{ 0xfa96, 0xfa96, PDF_CMAP_SINGLE, 9639 },
	{ 0xfa97, 0xfa97, PDF_CMAP_SINGLE, 8738 },
	{ 0xfa98, 0xfa98, PDF_CMAP_SINGLE, 9641 },
	{ 0xfa99, 0xfaa0, PDF_CMAP_RANGE, 21787 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 22650 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 23910 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 25170 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 26430 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 27690 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 28950 },
	{ 0xfb30, 0xfb39, PDF_CMAP_RANGE, 30051 },
	{ 0xfb40, 0xfb48, PDF_CMAP_RANGE, 21795 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 9640 },
	{ 0xfb4a, 0xfb51, PDF_CMAP_RANGE, 21804 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 9606 },
	{ 0xfb53, 0xfb56, PDF_CMAP_RANGE, 21812 },
	{ 0xfb57, 0xfb57, PDF_CMAP_SINGLE, 9633 },
	{ 0xfb58, 0xfb58, PDF_CMAP_SINGLE, 9642 },
	{ 0xfb59, 0xfb59, PDF_CMAP_SINGLE, 21816 },
	{ 0xfb5a, 0xfb5a, PDF_CMAP_SINGLE, 9616 },
	{ 0xfb5b, 0xfb5b, PDF_CMAP_SINGLE, 9614 },
	{ 0xfb5c, 0xfb74, PDF_CMAP_RANGE, 21817 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 9864 },
	{ 0xfb76, 0xfb78, PDF_CMAP_RANGE, 21842 },
	{ 0xfb79, 0xfb79, PDF_CMAP_SINGLE, 9886 },
	{ 0xfb7a, 0xfb7a, PDF_CMAP_SINGLE, 9723 },
	{ 0xfb7b, 0xfb7b, PDF_CMAP_SINGLE, 21845 },
	{ 0xfb7c, 0xfb7c, PDF_CMAP_SINGLE, 8076 },
	{ 0xfb7d, 0xfb7d, PDF_CMAP_SINGLE, 8692 },
	{ 0xfb7e, 0xfb7e, PDF_CMAP_SINGLE, 21846 },
	{ 0xfb80, 0xfb8f, PDF_CMAP_RANGE, 21847 },
	{ 0xfb90, 0xfb90, PDF_CMAP_SINGLE, 8207 },
	{ 0xfb91, 0xfb9b, PDF_CMAP_RANGE, 21863 },
	{ 0xfb9c, 0xfb9c, PDF_CMAP_SINGLE, 8305 },
	{ 0xfb9d, 0xfb9e, PDF_CMAP_RANGE, 21874 },
	{ 0xfb9f, 0xfb9f, PDF_CMAP_SINGLE, 9719 },
	{ 0xfba0, 0xfba0, PDF_CMAP_SINGLE, 21876 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 22660 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 23920 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 25180 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 26440 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 27700 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 28960 },
	{ 0xfc30, 0xfc39, PDF_CMAP_RANGE, 30061 },
	{ 0xfc40, 0xfc43, PDF_CMAP_RANGE, 21877 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 9878 },
	{ 0xfc45, 0xfc48, PDF_CMAP_RANGE, 21881 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 9871 },
	{ 0xfc4a, 0xfc59, PDF_CMAP_RANGE, 21885 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 8922 },
	{ 0xfc5b, 0xfc62, PDF_CMAP_RANGE, 21901 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 7873 },
	{ 0xfc64, 0xfc67, PDF_CMAP_RANGE, 21909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 7859 },
	{ 0xfc69, 0xfc6e, PDF_CMAP_RANGE, 21913 },
	{ 0xfc6f, 0xfc6f, PDF_CMAP_SINGLE, 9834 },
	{ 0xfc70, 0xfc70, PDF_CMAP_SINGLE, 21919 },
	{ 0xfc71, 0xfc71, PDF_CMAP_SINGLE, 9866 },
	{ 0xfc72, 0xfc73, PDF_CMAP_RANGE, 21920 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 9833 },
	{ 0xfc75, 0xfc76, PDF_CMAP_RANGE, 21922 },
	{ 0xfc77, 0xfc78, PDF_CMAP_RANGE, 9752 },
	{ 0xfc79, 0xfc7e, PDF_CMAP_RANGE, 21924 },
	{ 0xfc80, 0xfc82, PDF_CMAP_RANGE, 21930 },
	{ 0xfc83, 0xfc83, PDF_CMAP_SINGLE, 9754 },
	{ 0xfc84, 0xfc89, PDF_CMAP_RANGE, 21933 },
	{ 0xfc8a, 0xfc8a, PDF_CMAP_SINGLE, 9843 },
	{ 0xfc8b, 0xfca0, PDF_CMAP_RANGE, 21939 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 22670 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 23930 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 25190 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 26450 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 27710 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 28970 },
	{ 0xfd30, 0xfd39, PDF_CMAP_RANGE, 30071 },
	{ 0xfd38, 0xfd39, PDF_CMAP_RANGE, 22428 },
	{ 0xfd40, 0xfd51, PDF_CMAP_RANGE, 21961 },
	{ 0xfd52, 0xfd52, PDF_CMAP_SINGLE, 8390 },
	{ 0xfd53, 0xfd53, PDF_CMAP_SINGLE, 8811 },
	{ 0xfd54, 0xfd56, PDF_CMAP_RANGE, 21979 },
	{ 0xfd57, 0xfd57, PDF_CMAP_SINGLE, 9427 },
	{ 0xfd58, 0xfd58, PDF_CMAP_SINGLE, 7814 },
	{ 0xfd59, 0xfd59, PDF_CMAP_SINGLE, 21982 },
	{ 0xfd5a, 0xfd5a, PDF_CMAP_SINGLE, 9743 },
	{ 0xfd5b, 0xfd5e, PDF_CMAP_RANGE, 21983 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 9745 },
	{ 0xfd60, 0xfd61, PDF_CMAP_RANGE, 21987 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 9747 },
	{ 0xfd63, 0xfd64, PDF_CMAP_RANGE, 21989 },
	{ 0xfd65, 0xfd65, PDF_CMAP_SINGLE, 9744 },
	{ 0xfd66, 0xfd66, PDF_CMAP_SINGLE, 9746 },
	{ 0xfd67, 0xfd67, PDF_CMAP_SINGLE, 8240 },
	{ 0xfd68, 0xfd68, PDF_CMAP_SINGLE, 21991 },
	{ 0xfd69, 0xfd69, PDF_CMAP_SINGLE, 9841 },
	{ 0xfd6a, 0xfd6b, PDF_CMAP_RANGE, 21992 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 9748 },
	{ 0xfd6d, 0xfd6f, PDF_CMAP_RANGE, 21994 },
	{ 0xfd70, 0xfd70, PDF_CMAP_SINGLE, 9750 },
	{ 0xfd71, 0xfd71, PDF_CMAP_SINGLE, 21997 },
	{ 0xfd72, 0xfd72, PDF_CMAP_SINGLE, 9749 },
	{ 0xfd73, 0xfd77, PDF_CMAP_RANGE, 21998 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 8434 },
	{ 0xfd79, 0xfd7c, PDF_CMAP_RANGE, 22003 },
	{ 0xfd7d, 0xfd7d, PDF_CMAP_SINGLE, 9751 },
	{ 0xfd7e, 0xfd7e, PDF_CMAP_SINGLE, 22007 },
	{ 0xfd80, 0xfd87, PDF_CMAP_RANGE, 22008 },
	{ 0xfd88, 0xfd88, PDF_CMAP_SINGLE, 8247 },
	{ 0xfd89, 0xfd8a, PDF_CMAP_RANGE, 22016 },
	{ 0xfd8b, 0xfd8b, PDF_CMAP_SINGLE, 8371 },
	{ 0xfd8c, 0xfd8e, PDF_CMAP_RANGE, 22018 },
	{ 0xfd8f, 0xfd8f, PDF_CMAP_SINGLE, 7970 },
	{ 0xfd90, 0xfd90, PDF_CMAP_SINGLE, 9453 },
	{ 0xfd91, 0xfd93, PDF_CMAP_RANGE, 22021 },
	{ 0xfd94, 0xfd94, PDF_CMAP_SINGLE, 7988 },
	{ 0xfd95, 0xfd9c, PDF_CMAP_RANGE, 22024 },
	{ 0xfd9d, 0xfd9d, PDF_CMAP_SINGLE, 2562 },
	{ 0xfd9e, 0xfd9e, PDF_CMAP_SINGLE, 16595 },
	{ 0xfd9f, 0xfd9f, PDF_CMAP_SINGLE, 8204 },
	{ 0xfda0, 0xfda0, PDF_CMAP_SINGLE, 20611 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 22430 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 22680 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 23940 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 25200 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 26460 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 27720 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 28980 },
	{ 0xfe30, 0xfe39, PDF_CMAP_RANGE, 30081 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 4697 },
	{ 0xfe41, 0xfe7e, PDF_CMAP_RANGE, 22032 },
	{ 0xfe80, 0xfea0, PDF_CMAP_RANGE, 22094 },
};

static const unsigned short pdf_cmap_GBK2K_H_table[2047] =
{
	7778,10315,7781,8833,8909,10407,8458,7742,
	7830,8907,8812,10424,8471,10428,8424,8108,
	8915,8720,10474,8911,10475,8723,10476,8164,
	8912,8910,8919,8917,8913,8906,7833,8143,
	8246,7994,10676,8905,8089,7885,10719,8617,
	10720,8678,10721,8487,8195,8901,8030,10771,
	8902,8208,10831,8898,8459,11063,8414,11064,
	7850,8408,8896,8368,11113,9082,7796,9080,
	8655,9064,11125,9073,11126,9065,9072,11153,
	11154,9078,9075,9087,11158,9076,11159,8373,
	7899,9842,9067,11193,8249,9085,8693,11202,
	9089,9083,9086,8652,9066,9865,11214,9879,
	9091,7997,11247,8591,8774,8776,8574,11254,
	8576,8709,7736,8163,9017,11440,9003,11441,
	9005,8615,11453,9007,8409,11517,8157,8199,
	8998,9001,8018,8251,9000,11540,8999,11590,
	8060,9051,7902,11614,8098,11615,7930,4162,
	4707,4722,4709,4185,10131,1597,4867,9210,
	11898,9214,8104,1592,3270,982,4765,4884,
	4879,2091,5017,1431,8421,12017,8495,8355,
	8481,12018,8660,8165,7818,7735,8094,8870,
	8680,12032,7898,7865,8272,7783,12074,9204,
	4860,4710,4740,10778,3397,1150,4946,1228,
	5020,9097,12165,7863,9105,12268,8817,2407,
	5523,3698,3414,5660,11565,3859,1398,2927,
	9104,12313,8243,12314,8764,8277,12333,9106,
	8837,12387,9092,9095,9094,8848,12406,7748,
	7730,9093,7955,8051,1386,3948,5302,3395,
	6004,3318,6165,1789,8327,7800,9134,7926,
	7985,9205,12521,8821,9838,12534,7856,8323,
	12538,8584,7841,12575,9107,2238,1732,6163,
	1852,5016,5293,5366,1798,5986,9140,8318,
	8341,9147,7720,9149,5600,3983,1765,12946,
	3437,4518,6409,3795,1526,7774,12819,7773,
	7983,9439,9138,8526,9148,12833,8427,8219,
	8380,9150,8329,12871,9137,8158,8739,9142,
	9868,9438,8190,8017,8673,12922,9136,12923,
	8142,12924,8478,8225,8819,12944,8624,1626,
	3821,3248,4350,4357,2849,3124,4536,1400,
	8476,13097,9053,8461,8282,3826,1073,2736,
	3430,3095,3491,2053,4611,1715,9057,13234,
	8256,9054,13252,7786,7852,7764,8384,9058,
	8544,13288,8605,8078,8749,13289,8263,13290,
	8802,8039,7858,5789,3019,4073,2916,3209,
	4041,4310,1832,3728,7762,8356,7964,8847,
	13314,8173,7725,8530,9062,13319,8442,9059,
	13324,8346,8182,13335,9060,13336,7785,13337,
	9063,13338,8477,8793,8278,8547,8105,8188,
	7868,13388,8514,8222,7747,3379,4264,3650,
	7110,7008,16063,994,3011,2808,8790,9384,
	9383,13593,9862,9382,13594,8654,2737,3407,
	3398,3413,16587,1923,4049,2539,4592,7399,
	7262,3753,4123,4309,2490,1591,7115,1593,
	8565,13863,9325,9323,8799,3261,1249,4657,
	4544,2297,3353,5656,4570,7388,9320,14046,
	8448,14058,9334,7975,8407,8874,8197,8387,
	8512,9335,17826,7152,1291,4051,3995,4169,
	18908,8086,2200,8040,14171,8581,8075,9348,
	9344,9326,14266,9339,9350,9329,9347,9328,
	14274,9321,14275,9343,1825,1528,7445,7504,
	7739,1285,4668,4672,3366,9419,14362,8019,
	8211,7987,9353,9355,8463,8155,8394,14468,
	8423,9410,3980,1250,19731,4191,4276,19992,
	2522,2241,7797,1714,2542,7545,4303,20714,
	1636,2795,1770,8589,8772,9186,14706,8404,
	7779,8328,7869,14840,9025,4219,8713,7936,
	7924,3402,3438,3920,8301,1824,8264,9184,
	8311,14865,8762,8001,8220,8095,14903,9674,
	14904,9185,8172,9187,8462,8102,8196,8886,
	8361,8557,15007,7660,7888,5019,4704,1862,
	8761,8348,9864,2656,8593,7760,8092,15023,
	9178,15024,8275,9193,9188,8661,15033,9190,
	8212,15045,9196,9195,9176,9198,9870,15058,
	9197,9174,9192,8585,8280,2704,21894,3465,
	1937,6741,9752,1509,1821,3466,8473,15342,
	8390,7814,8247,7988,4851,8124,15370,9434,
	8712,15414,7910,22390,22392,22391,22393,8630,
	7741,8757,15560,9111,8767,8490,7890,15595,
	9110,9112,15596,8354,15597,8235,15600,9108,
	8505,15601,8543,8641,9114,8741,8298,7937,
	8704,9650,7831,16025,8363,8232,16038,16040,
	9649,8706,9643,8838,16057,9645,16058,9654,
	16059,8675,8751,8549,9655,7717,16083,9664,
	16107,8858,8814,16126,8125,16127,8068,8370,
	16128,8259,16205,9455,9445,16366,8698,9446,
	8439,16413,8299,16471,7826,9447,8210,7916,
	8038,9436,8206,16581,9435,7864,9764,8041,
	8747,9599,8027,8603,16704,9853,8608,8582,
	16738,8429,7844,8417,16756,9656,8800,16757,
	8418,8868,16912,9704,9706,16970,8079,16971,
	9708,8400,8221,16992,9710,8250,9701,9709,
	17024,8201,17025,8293,9717,7932,9883,17108,
	9715,17109,8921,17110,9716,17111,9243,8136,
	17115,8059,17116,9245,17117,8780,8008,9244,
	8447,8602,17118,8337,9251,7835,9250,17123,
	8464,17124,8845,8049,7928,9249,8625,9254,
	9253,8480,8474,9252,17143,9256,9258,9257,
	17146,8852,17147,8894,17148,7729,9260,17156,
	8117,9259,8111,8296,8676,7969,17174,8449,
	8572,8522,9261,17184,8145,7731,17192,9263,
	9262,17195,9264,8667,8889,17212,9270,7822,
	9273,8645,9271,8594,17222,9274,7961,8588,
	7744,8883,8287,9272,9266,8820,17226,7836,
	9265,8325,8119,9267,8276,8670,17240,9268,
	9277,8073,9276,8046,7896,17246,7871,9285,
	17247,8777,9281,17250,7751,8022,8598,17255,
	9283,9279,17256,8227,17257,9282,9278,17260,
	9038,9286,9291,9284,9255,9292,7951,17286,
	9287,9289,9288,7939,17296,9290,8540,8891,
	9296,17299,9876,9295,9718,8273,17300,9294,
	8890,8045,9298,9297,9301,17318,8841,8470,
	8486,8033,9885,17335,8074,9302,9305,9304,
	8058,9293,9303,9299,17353,9248,17354,8671,
	9861,17355,7790,8740,9840,17359,8634,17360,
	9306,9698,17392,9882,8290,9457,17425,9458,
	9666,17520,9665,8488,17542,8215,7840,8485,
	8525,17561,9668,8349,8842,17562,9667,17712,
	8345,7853,9416,8360,8223,17732,8389,17733,
	9418,8176,17742,9414,8797,8926,8763,8665,
	8139,8137,9711,17818,8088,17821,9712,18004,
	8013,9031,18078,8595,9037,9032,7776,18133,
	9029,8216,9013,9028,9009,18228,9035,18229,
	9019,7860,8614,9022,18259,8052,18260,9008,
	8081,18279,8455,8180,9027,8788,18333,9010,
	7718,9047,9048,8258,8531,8289,18386,8265,
	18406,8002,18571,8606,7787,18646,9691,18654,
	8721,8745,9686,9693,9695,8175,18679,9688,
	8600,18718,7816,8844,18733,7768,8876,9846,
	18823,8161,9659,9660,9658,8583,18893,9835,
	8502,18925,9400,8420,18941,9401,8189,18954,
	9403,7981,9737,18980,9738,18981,7828,8927,
	7883,7949,8055,18992,8683,18993,8929,18994,
	8559,18995,8928,8682,18998,8930,8395,18999,
	8056,7906,19002,8690,19003,8528,8147,19007,
	8933,8532,19019,8935,8932,19038,8810,19039,
	8939,19040,8937,8381,8950,8679,8726,8492,
	19053,7784,8946,7991,8947,19054,8943,8016,
	7952,8648,19055,8945,8942,19063,8941,8862,
	8940,8162,19073,7855,8758,19076,8951,8765,
	19080,7810,8118,19081,8612,8618,8952,8529,
	19082,8032,8517,19089,8156,19101,8884,8553,
	19104,8957,19105,8426,19106,8948,19107,8955,
	19108,8956,8231,8288,8959,8958,7879,19115,
	8972,8949,19119,8966,19120,8970,8659,8963,
	19124,8967,8971,8031,19127,8969,8962,7940,
	8861,19130,8699,19131,8968,19132,8364,19133,
	8334,8965,8599,19134,8561,8856,8322,19141,
	8977,8964,19145,8975,7733,19146,8401,8976,
	8099,19147,8662,8978,8333,8979,8982,8043,
	8980,19192,8496,8981,8552,19193,8388,8729,
	8727,7789,8440,19236,8187,8985,8974,19242,
	8983,7739,8831,19329,7948,7769,7972,8378,
	8037,7920,8548,7984,8801,19334,19335,8866,
	9387,19336,9391,7912,7993,19337,7752,8304,
	7848,19338,9388,7927,8566,9389,19339,8315,
	8005,9386,8267,8239,8026,9392,19340,8887,
	8063,19341,8805,9394,8475,7761,19348,9396,
	9395,19353,7839,19354,8472,19355,8372,9135,
	8635,8306,8085,19356,7946,9398,8849,9397,
	8824,8873,9399,7976,8457,8903,9390,8881,
	8806,19377,8795,8900,19378,8469,19379,8746,
	9393,7957,8796,9724,7820,9730,19571,8781,
	19572,9732,9726,9731,7843,9736,7803,8807,
	7990,8150,8672,19616,9356,8857,9361,9366,
	9359,9362,19647,9367,8113,19652,9370,19653,
	9369,19654,8792,9368,9371,19662,7945,8422,
	8230,9375,8025,19672,7995,9372,7738,8283,
	8816,8759,9426,19692,8627,8773,9377,8828,
	8112,8006,19713,9088,9365,19714,9360,7837,
	7755,7754,8362,9202,19771,8830,19772,8217,
	8787,19797,7998,7846,8590,8684,19801,7870,
	19802,8778,8399,8674,19815,8719,19816,8233,
	19817,8307,9201,19820,7750,8291,9203,8892,
	19895,8988,19896,8785,8839,19914,8237,7851,
	19915,8989,19916,8991,8716,8100,8347,8664,
	19990,9722,19991,9721,8500,19994,9464,19995,
	9463,7880,9462,7877,9467,20003,20004,9466,
	20005,7917,9469,20017,9465,20018,9470,8397,
	9480,9476,9471,8336,7901,7973,20032,9475,
	9474,7802,8358,7953,20047,9479,9472,9477,
	9497,20054,9493,9484,8241,9483,9487,9498,
	9481,9486,8756,9491,8064,9495,9494,9485,
	20076,8403,8314,8398,8106,20101,9502,7967,
	9517,8733,9522,20107,8571,8623,20118,9516,
	9512,20121,8332,20122,9519,20123,8636,9525,
	20129,8717,9510,20130,9524,9514,9503,9521,
	9500,8653,20139,8666,20140,8562,9534,8271,
	20145,9539,20146,8663,20147,7740,9535,20165,
	9540,9507,7824,9530,9541,20169,9533,20170,
	8385,8451,9504,9532,9531,9528,9536,20177,
	8141,8880,20196,9542,7834,9554,9520,9545,
	9553,7882,20204,8402,8120,20208,8313,9549,
	7845,8268,8320,9837,20217,9527,9546,8632,
	20224,9550,9468,9556,20229,7996,20230,7893,
	9559,9555,9544,20253,8087,9557,8316,20270,
	9562,8542,20285,9568,8610,9044,9571,9511,
	20297,9518,9560,20298,7963,20299,8835,20300,
	9572,8352,20303,9573,9581,8224,9567,9578,
	9529,9564,9579,7791,20330,8132,9575,20331,
	9563,9515,20347,9585,20348,9591,8234,20369,
	9526,9587,9583,8851,9592,9594,8198,20391,
	8567,9499,20394,9508,9593,20407,9574,8083,
	20413,9596,9597,20427,9586,8782,20436,8646,
	8351,20441,9590,8292,8895,9756,8798,8317,
	20480,9151,8467,7749,20485,8152,9156,20486,
	9154,8452,20487,8637,20488,8071,9155,8003,
	20500,7966,9849,7915,7989,8330,20503,9159,
	9161,9158,8691,20513,8695,9167,9166,9162,
	9165,9836,20519,9153,20520,8174,9169,9170,
	9172,7832,7980,20532,9173,7793,9873,8732,
	20585,7806,8269,20599,8114,8535,20615,8639,
	8735,20621,8253,8213,20624,9893,8516,20637,
	7825,8791,8202,8338,9740,20702,9742,9823,
	20798,8589,8445,9319,20812,9318,8649,20835,
	8713,7881,8425,20836,8650,8518,9669,8668,
	20837,9310,20838,8527,20839,9670,9671,8769,
	8586,7727,7900,8383,8244,8718,20852,9675,
	9676,8131,8377,20869,8577,8563,7905,9678,
	8694,8779,9681,20890,7872,9682,20898,7978,
	7794,9683,8638,9684,8260,20904,9679,8435,
	20916,9880,20917,9848,8376,20932,9424,20933,
	9425,9115,20946,9854,20951,9121,20952,7921,
	8734,20955,9122,8523,7734,8501,8109,20967,
	7763,20970,9123,8707,20971,7911,9124,8343,
	7908,9118,21006,8245,9127,9128,21010,8309,
	9131,8171,9132,8042,8441,9831,21023,7788,
	8301,8770,7938,8579,21057,7813,8681,8869,
	9223,8138,21084,9218,8066,9220,21087,8497,
	21088,8580,21089,9219,21090,8302,9227,8151,
	21117,7811,9230,8391,9229,9200,21153,9233,
	9236,21154,8560,9221,8460,9237,8294,21165,
	9042,9235,9232,9225,21172,9240,8103,21178,
	8700,8129,9222,8270,21186,9242,9241,9825,
	21225,9892,8564,9827,9826,9832,21283,7888,
	21284,8342,21285,9164,9160,21290,8766,9828,
	21313,8761,9760,21349,9761,9762,21360,9767,
	7737,9765,9774,9771,9770,9773,21381,9768,
	8633,9782,9776,9784,21404,8205,9783,9797,
	21414,9786,21419,9792,9789,9793,21423,9790,
	9791,8128,21426,9785,21442,9804,9799,9803,
	9800,21456,9802,8456,9780,21471,9809,9808,
	9810,9778,9806,9811,21480,9815,21481,9781,
	21482,9779,21483,9814,9813,7757,9819,21502,
	9818,21503,9817,8238,9775,9798,21508,9766,
	9763,21531,9777,7941,8331,21578,9601,9603,
	9602,21594,8686,8771,21611,9607,21612,9608,
	21615,9604,8701,21618,8687,9610,9612,8144,
	9618,9615,9620,7904,9617,9621,9623,21672,
	8374,9605,21697,9625,9626,21711,9627,9824,
	9632,9628,21743,9631,21744,8044,8366,21762,
	9611,9635,9609,9637,9638,9619,9639,8738,
	9641,9633,9642,21816,9616,9614,9886,9723,
	21845,8076,8692,21846,9719,21876,9834,21919,
	9866,8390,8811,9427,7814,21982,9743,9744,
	9746,8240,21991,9841,9750,21997,9749,9751,
	22007,7970,9453,2562,16595,8204,20611,
};

pdf_cmap pdf_cmap_GBK2K_H =
{
	-1, "GBK2K-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x007f },
		{ 4, 0x8130, 0xfe39 },
		{ 2, 0x8140, 0xfefe },
	},
	5088, 5088, (pdf_range*) pdf_cmap_GBK2K_H_ranges,
	2047, 2047, (unsigned short*) pdf_cmap_GBK2K_H_table,
};

/* GBK2K-V */

static const pdf_range pdf_cmap_GBK2K_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa4a1, 0xa4a1, PDF_CMAP_SINGLE, 22359 },
	{ 0xa4a3, 0xa4a3, PDF_CMAP_SINGLE, 22361 },
	{ 0xa4a5, 0xa4a5, PDF_CMAP_SINGLE, 22370 },
	{ 0xa4a7, 0xa4a7, PDF_CMAP_SINGLE, 22360 },
	{ 0xa4a9, 0xa4a9, PDF_CMAP_SINGLE, 22368 },
	{ 0xa4c3, 0xa4c3, PDF_CMAP_SINGLE, 22369 },
	{ 0xa4e3, 0xa4e3, PDF_CMAP_SINGLE, 22372 },
	{ 0xa4e5, 0xa4e5, PDF_CMAP_SINGLE, 22374 },
	{ 0xa4e7, 0xa4e7, PDF_CMAP_SINGLE, 22373 },
	{ 0xa4ee, 0xa4ee, PDF_CMAP_SINGLE, 22371 },
	{ 0xa5a1, 0xa5a1, PDF_CMAP_SINGLE, 22376 },
	{ 0xa5a3, 0xa5a3, PDF_CMAP_SINGLE, 22378 },
	{ 0xa5a5, 0xa5a5, PDF_CMAP_SINGLE, 22385 },
	{ 0xa5a7, 0xa5a7, PDF_CMAP_SINGLE, 22377 },
	{ 0xa5a9, 0xa5a9, PDF_CMAP_SINGLE, 22383 },
	{ 0xa5c3, 0xa5c3, PDF_CMAP_SINGLE, 22384 },
	{ 0xa5e3, 0xa5e3, PDF_CMAP_SINGLE, 22387 },
	{ 0xa5e5, 0xa5e5, PDF_CMAP_SINGLE, 22389 },
	{ 0xa5e7, 0xa5e7, PDF_CMAP_SINGLE, 22388 },
	{ 0xa5ee, 0xa5ee, PDF_CMAP_SINGLE, 22386 },
	{ 0xa960, 0xa960, PDF_CMAP_SINGLE, 22394 },
	{ 0xa5ee, 0xa5ee, PDF_CMAP_SINGLE, 22386 },
	{ 0xa960, 0xa960, PDF_CMAP_SINGLE, 22394 },
};

static const unsigned short pdf_cmap_GBK2K_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBK2K_V =
{
	-1, "GBK2K-V", "GBK2K-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	41, 41, (pdf_range*) pdf_cmap_GBK2K_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBK2K_V_table,
};

/* GBKp-EUC-H */

static const pdf_range pdf_cmap_GBKp_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8140, 0x8178, PDF_CMAP_RANGE, 10072 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 8281 },
	{ 0x817a, 0x817e, PDF_CMAP_RANGE, 10129 },
	{ 0x8180, 0x8185, PDF_CMAP_RANGE, 10134 },
	{ 0x8186, 0x8186, PDF_CMAP_SINGLE, 8689 },
	{ 0x8187, 0x81ec, PDF_CMAP_RANGE, 10140 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 8178 },
	{ 0x81ee, 0x81f5, PDF_CMAP_RANGE, 10242 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 8285 },
	{ 0x81f7, 0x81fe, PDF_CMAP_RANGE, 10250 },
	{ 0x8240, 0x8252, PDF_CMAP_RANGE, 10258 },
	{ 0x8253, 0x8253, PDF_CMAP_SINGLE, 9884 },
	{ 0x8254, 0x8261, PDF_CMAP_RANGE, 10277 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 8629 },
	{ 0x8263, 0x8273, PDF_CMAP_RANGE, 10291 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 8908 },
	{ 0x8275, 0x8279, PDF_CMAP_RANGE, 10308 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 8214 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 10313 },
	{ 0x827d, 0x827e, PDF_CMAP_TABLE, 0 },
	{ 0x8280, 0x8280, PDF_CMAP_SINGLE, 7968 },
	{ 0x8281, 0x8282, PDF_CMAP_RANGE, 10316 },
	{ 0x8283, 0x8283, PDF_CMAP_SINGLE, 8319 },
	{ 0x8284, 0x828f, PDF_CMAP_RANGE, 10318 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 8284 },
	{ 0x8291, 0x82a4, PDF_CMAP_RANGE, 10330 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 8596 },
	{ 0x82a6, 0x82c7, PDF_CMAP_RANGE, 10350 },
	{ 0x82c8, 0x82c9, PDF_CMAP_TABLE, 2 },
	{ 0x82ca, 0x82e0, PDF_CMAP_RANGE, 10384 },
	{ 0x82e1, 0x82e4, PDF_CMAP_TABLE, 4 },
	{ 0x82e5, 0x82ec, PDF_CMAP_RANGE, 10408 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 9855 },
	{ 0x82ee, 0x82f1, PDF_CMAP_RANGE, 10416 },
	{ 0x82f2, 0x82f2, PDF_CMAP_SINGLE, 8750 },
	{ 0x82f3, 0x82f6, PDF_CMAP_RANGE, 10420 },
	{ 0x82f7, 0x82fb, PDF_CMAP_TABLE, 8 },
	{ 0x82fc, 0x82fe, PDF_CMAP_RANGE, 10425 },
	{ 0x8340, 0x8341, PDF_CMAP_TABLE, 13 },
	{ 0x8342, 0x8344, PDF_CMAP_RANGE, 10429 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 8916 },
	{ 0x8346, 0x8347, PDF_CMAP_RANGE, 10432 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 8121 },
	{ 0x8349, 0x834b, PDF_CMAP_RANGE, 10434 },
	{ 0x834c, 0x834c, PDF_CMAP_SINGLE, 8920 },
	{ 0x834d, 0x8352, PDF_CMAP_RANGE, 10437 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 8415 },
	{ 0x8354, 0x8356, PDF_CMAP_RANGE, 10443 },
	{ 0x8357, 0x8357, PDF_CMAP_SINGLE, 8386 },
	{ 0x8358, 0x835d, PDF_CMAP_RANGE, 10446 },
	{ 0x835e, 0x835e, PDF_CMAP_SINGLE, 8597 },
	{ 0x835f, 0x8364, PDF_CMAP_RANGE, 10452 },
	{ 0x8365, 0x8366, PDF_CMAP_TABLE, 15 },
	{ 0x8367, 0x8371, PDF_CMAP_RANGE, 10458 },
	{ 0x8372, 0x8372, PDF_CMAP_SINGLE, 8065 },
	{ 0x8373, 0x8377, PDF_CMAP_RANGE, 10469 },
	{ 0x8378, 0x837e, PDF_CMAP_TABLE, 17 },
	{ 0x8380, 0x8380, PDF_CMAP_SINGLE, 8080 },
	{ 0x8381, 0x8385, PDF_CMAP_RANGE, 10477 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 8918 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 10482 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 24 },
	{ 0x838b, 0x838c, PDF_CMAP_RANGE, 10484 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 9857 },
	{ 0x838e, 0x8393, PDF_CMAP_RANGE, 10486 },
	{ 0x8394, 0x8394, PDF_CMAP_SINGLE, 7798 },
	{ 0x8395, 0x839d, PDF_CMAP_RANGE, 10492 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 8753 },
	{ 0x839f, 0x83a5, PDF_CMAP_RANGE, 10501 },
	{ 0x83a6, 0x83a6, PDF_CMAP_SINGLE, 7827 },
	{ 0x83a7, 0x83aa, PDF_CMAP_RANGE, 10508 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 8914 },
	{ 0x83ac, 0x83ad, PDF_CMAP_RANGE, 10512 },
	{ 0x83ae, 0x83b0, PDF_CMAP_TABLE, 26 },
	{ 0x83b1, 0x83b9, PDF_CMAP_RANGE, 10514 },
	{ 0x83ba, 0x83ba, PDF_CMAP_SINGLE, 7909 },
	{ 0x83bb, 0x83c8, PDF_CMAP_RANGE, 10523 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 8229 },
	{ 0x83ca, 0x83f5, PDF_CMAP_RANGE, 10537 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 7887 },
	{ 0x83f7, 0x83fe, PDF_CMAP_RANGE, 10581 },
	{ 0x8440, 0x844f, PDF_CMAP_RANGE, 10589 },
	{ 0x8450, 0x8450, PDF_CMAP_SINGLE, 8153 },
	{ 0x8451, 0x8470, PDF_CMAP_RANGE, 10605 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 8904 },
	{ 0x8472, 0x8473, PDF_CMAP_RANGE, 10637 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 8803 },
	{ 0x8475, 0x8476, PDF_CMAP_RANGE, 10639 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 9859 },
	{ 0x8478, 0x847e, PDF_CMAP_RANGE, 10641 },
	{ 0x8480, 0x8481, PDF_CMAP_RANGE, 10648 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 7959 },
	{ 0x8483, 0x848d, PDF_CMAP_RANGE, 10650 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 7979 },
	{ 0x848f, 0x8491, PDF_CMAP_RANGE, 10661 },
	{ 0x8492, 0x8493, PDF_CMAP_TABLE, 29 },
	{ 0x8494, 0x849c, PDF_CMAP_RANGE, 10664 },
	{ 0x849d, 0x849d, PDF_CMAP_SINGLE, 8015 },
	{ 0x849e, 0x84a0, PDF_CMAP_RANGE, 10673 },
	{ 0x84a1, 0x84a6, PDF_CMAP_TABLE, 31 },
	{ 0x84a7, 0x84a8, PDF_CMAP_RANGE, 10677 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 8053 },
	{ 0x84aa, 0x84c4, PDF_CMAP_RANGE, 10679 },
	{ 0x84c5, 0x84c5, PDF_CMAP_SINGLE, 8126 },
	{ 0x84c6, 0x84d2, PDF_CMAP_RANGE, 10706 },
	{ 0x84d3, 0x84da, PDF_CMAP_TABLE, 37 },
	{ 0x84db, 0x84dc, PDF_CMAP_RANGE, 10722 },
	{ 0x84dd, 0x84dd, PDF_CMAP_SINGLE, 8498 },
	{ 0x84de, 0x84e9, PDF_CMAP_RANGE, 10724 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 8995 },
	{ 0x84eb, 0x84ed, PDF_CMAP_RANGE, 10736 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 8209 },
	{ 0x84ef, 0x84f0, PDF_CMAP_RANGE, 10739 },
	{ 0x84f1, 0x84f1, PDF_CMAP_SINGLE, 8437 },
	{ 0x84f2, 0x84fe, PDF_CMAP_RANGE, 10741 },
	{ 0x8540, 0x8550, PDF_CMAP_RANGE, 10754 },
	{ 0x8551, 0x8554, PDF_CMAP_TABLE, 45 },
	{ 0x8555, 0x855d, PDF_CMAP_RANGE, 10772 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 8431 },
	{ 0x855f, 0x8565, PDF_CMAP_RANGE, 10781 },
	{ 0x8566, 0x8566, PDF_CMAP_SINGLE, 8656 },
	{ 0x8567, 0x857e, PDF_CMAP_RANGE, 10788 },
	{ 0x8580, 0x8586, PDF_CMAP_RANGE, 10812 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 8897 },
	{ 0x8588, 0x858a, PDF_CMAP_RANGE, 10819 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 7780 },
	{ 0x858c, 0x8591, PDF_CMAP_RANGE, 10822 },
	{ 0x8592, 0x8592, PDF_CMAP_SINGLE, 8697 },
	{ 0x8593, 0x8595, PDF_CMAP_RANGE, 10828 },
	{ 0x8596, 0x8598, PDF_CMAP_TABLE, 49 },
	{ 0x8599, 0x85a1, PDF_CMAP_RANGE, 10832 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 7770 },
	{ 0x85a3, 0x85b1, PDF_CMAP_RANGE, 10841 },
	{ 0x85b2, 0x85b2, PDF_CMAP_SINGLE, 7842 },
	{ 0x85b3, 0x85fe, PDF_CMAP_RANGE, 10856 },
	{ 0x8640, 0x8649, PDF_CMAP_RANGE, 10932 },
	{ 0x864a, 0x864a, PDF_CMAP_SINGLE, 9069 },
	{ 0x864b, 0x8653, PDF_CMAP_RANGE, 10942 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 8775 },
	{ 0x8655, 0x8667, PDF_CMAP_RANGE, 10951 },
	{ 0x8668, 0x8668, PDF_CMAP_SINGLE, 9068 },
	{ 0x8669, 0x867e, PDF_CMAP_RANGE, 10970 },
	{ 0x8680, 0x8695, PDF_CMAP_RANGE, 10992 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 8604 },
	{ 0x8697, 0x8698, PDF_CMAP_RANGE, 11014 },
	{ 0x8699, 0x8699, PDF_CMAP_SINGLE, 8393 },
	{ 0x869a, 0x86a0, PDF_CMAP_RANGE, 11016 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 8688 },
	{ 0x86a2, 0x86c9, PDF_CMAP_RANGE, 11023 },
	{ 0x86ca, 0x86ce, PDF_CMAP_TABLE, 52 },
	{ 0x86cf, 0x86d0, PDF_CMAP_RANGE, 11065 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 8748 },
	{ 0x86d2, 0x86db, PDF_CMAP_RANGE, 11067 },
	{ 0x86dc, 0x86dd, PDF_CMAP_TABLE, 57 },
	{ 0x86de, 0x86e0, PDF_CMAP_RANGE, 11077 },
	{ 0x86e1, 0x86e1, PDF_CMAP_SINGLE, 8303 },
	{ 0x86e2, 0x86e7, PDF_CMAP_RANGE, 11080 },
	{ 0x86e8, 0x86e8, PDF_CMAP_SINGLE, 8609 },
	{ 0x86e9, 0x86ed, PDF_CMAP_RANGE, 11086 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 9081 },
	{ 0x86ef, 0x86f3, PDF_CMAP_RANGE, 11091 },
	{ 0x86f4, 0x86f4, PDF_CMAP_SINGLE, 9074 },
	{ 0x86f5, 0x86fe, PDF_CMAP_RANGE, 11096 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 8554 },
	{ 0x8741, 0x8743, PDF_CMAP_RANGE, 11106 },
	{ 0x8744, 0x8744, PDF_CMAP_SINGLE, 9084 },
	{ 0x8745, 0x8748, PDF_CMAP_RANGE, 11109 },
	{ 0x8749, 0x874c, PDF_CMAP_TABLE, 59 },
	{ 0x874d, 0x874e, PDF_CMAP_RANGE, 11114 },
	{ 0x874f, 0x874f, PDF_CMAP_SINGLE, 9079 },
	{ 0x8750, 0x8756, PDF_CMAP_RANGE, 11116 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 8012 },
	{ 0x8758, 0x8759, PDF_CMAP_RANGE, 11123 },
	{ 0x875a, 0x8760, PDF_CMAP_TABLE, 63 },
	{ 0x8761, 0x8765, PDF_CMAP_RANGE, 11127 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 9844 },
	{ 0x8767, 0x8779, PDF_CMAP_RANGE, 11132 },
	{ 0x877a, 0x877a, PDF_CMAP_SINGLE, 9071 },
	{ 0x877b, 0x877c, PDF_CMAP_RANGE, 11151 },
	{ 0x877d, 0x877e, PDF_CMAP_TABLE, 70 },
	{ 0x8780, 0x8782, PDF_CMAP_TABLE, 72 },
	{ 0x8783, 0x8785, PDF_CMAP_RANGE, 11155 },
	{ 0x8786, 0x878a, PDF_CMAP_TABLE, 75 },
	{ 0x878b, 0x878c, PDF_CMAP_RANGE, 11160 },
	{ 0x878d, 0x878e, PDF_CMAP_TABLE, 80 },
	{ 0x878f, 0x8792, PDF_CMAP_RANGE, 11162 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 9070 },
	{ 0x8794, 0x8797, PDF_CMAP_RANGE, 11166 },
	{ 0x8798, 0x8798, PDF_CMAP_SINGLE, 8631 },
	{ 0x8799, 0x879c, PDF_CMAP_RANGE, 11170 },
	{ 0x879d, 0x879d, PDF_CMAP_SINGLE, 9077 },
	{ 0x879e, 0x87a2, PDF_CMAP_RANGE, 11174 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 9090 },
	{ 0x87a4, 0x87a6, PDF_CMAP_RANGE, 11179 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 8350 },
	{ 0x87a8, 0x87b2, PDF_CMAP_RANGE, 11182 },
	{ 0x87b3, 0x87b5, PDF_CMAP_TABLE, 82 },
	{ 0x87b6, 0x87ba, PDF_CMAP_RANGE, 11194 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 9887 },
	{ 0x87bc, 0x87be, PDF_CMAP_RANGE, 11199 },
	{ 0x87bf, 0x87c2, PDF_CMAP_TABLE, 85 },
	{ 0x87c3, 0x87c9, PDF_CMAP_RANGE, 11203 },
	{ 0x87ca, 0x87cc, PDF_CMAP_TABLE, 89 },
	{ 0x87cd, 0x87ce, PDF_CMAP_RANGE, 11210 },
	{ 0x87cf, 0x87cf, PDF_CMAP_SINGLE, 8923 },
	{ 0x87d0, 0x87d1, PDF_CMAP_RANGE, 11212 },
	{ 0x87d2, 0x87d5, PDF_CMAP_TABLE, 92 },
	{ 0x87d6, 0x87d9, PDF_CMAP_RANGE, 11215 },
	{ 0x87da, 0x87da, PDF_CMAP_SINGLE, 8865 },
	{ 0x87db, 0x87f6, PDF_CMAP_RANGE, 11219 },
	{ 0x87f7, 0x87fa, PDF_CMAP_TABLE, 96 },
	{ 0x87fb, 0x87fe, PDF_CMAP_RANGE, 11248 },
	{ 0x8840, 0x8841, PDF_CMAP_TABLE, 100 },
	{ 0x8842, 0x8843, PDF_CMAP_RANGE, 11252 },
	{ 0x8844, 0x8846, PDF_CMAP_TABLE, 102 },
	{ 0x8847, 0x887e, PDF_CMAP_RANGE, 11255 },
	{ 0x8880, 0x88b9, PDF_CMAP_RANGE, 11311 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 9002 },
	{ 0x88bb, 0x88cb, PDF_CMAP_RANGE, 11369 },
	{ 0x88cc, 0x88cc, PDF_CMAP_SINGLE, 8843 },
	{ 0x88cd, 0x88d3, PDF_CMAP_RANGE, 11386 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 8069 },
	{ 0x88d5, 0x88d6, PDF_CMAP_RANGE, 11393 },
	{ 0x88d7, 0x88d7, PDF_CMAP_SINGLE, 8997 },
	{ 0x88d8, 0x88de, PDF_CMAP_RANGE, 11395 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 9004 },
	{ 0x88e0, 0x88e4, PDF_CMAP_RANGE, 11402 },
	{ 0x88e5, 0x88e5, PDF_CMAP_SINGLE, 9006 },
	{ 0x88e6, 0x88f1, PDF_CMAP_RANGE, 11407 },
	{ 0x88f2, 0x88f3, PDF_CMAP_TABLE, 105 },
	{ 0x88f4, 0x88f5, PDF_CMAP_RANGE, 11419 },
	{ 0x88f6, 0x88f6, PDF_CMAP_SINGLE, 7795 },
	{ 0x88f7, 0x88fe, PDF_CMAP_RANGE, 11421 },
	{ 0x8940, 0x894a, PDF_CMAP_RANGE, 11429 },
	{ 0x894b, 0x8950, PDF_CMAP_TABLE, 107 },
	{ 0x8951, 0x8953, PDF_CMAP_RANGE, 11442 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 8575 },
	{ 0x8955, 0x895c, PDF_CMAP_RANGE, 11445 },
	{ 0x895d, 0x895f, PDF_CMAP_TABLE, 113 },
	{ 0x8960, 0x896c, PDF_CMAP_RANGE, 11454 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 7805 },
	{ 0x896e, 0x8970, PDF_CMAP_RANGE, 11467 },
	{ 0x8971, 0x8971, PDF_CMAP_SINGLE, 8406 },
	{ 0x8972, 0x897b, PDF_CMAP_RANGE, 11470 },
	{ 0x897c, 0x897c, PDF_CMAP_SINGLE, 7874 },
	{ 0x897d, 0x897e, PDF_CMAP_RANGE, 11480 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 11482 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8882 },
	{ 0x898c, 0x8998, PDF_CMAP_RANGE, 11493 },
	{ 0x8999, 0x8999, PDF_CMAP_SINGLE, 7903 },
	{ 0x899a, 0x899d, PDF_CMAP_RANGE, 11506 },
	{ 0x899e, 0x899e, PDF_CMAP_SINGLE, 7929 },
	{ 0x899f, 0x89a5, PDF_CMAP_RANGE, 11510 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 116 },
	{ 0x89a9, 0x89ae, PDF_CMAP_RANGE, 11518 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 8551 },
	{ 0x89b0, 0x89b9, PDF_CMAP_RANGE, 11524 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8685 },
	{ 0x89bb, 0x89bd, PDF_CMAP_RANGE, 11534 },
	{ 0x89be, 0x89c0, PDF_CMAP_TABLE, 119 },
	{ 0x89c1, 0x89c3, PDF_CMAP_RANGE, 11537 },
	{ 0x89c4, 0x89c8, PDF_CMAP_TABLE, 122 },
	{ 0x89c9, 0x89cd, PDF_CMAP_RANGE, 11541 },
	{ 0x89ce, 0x89ce, PDF_CMAP_SINGLE, 7723 },
	{ 0x89cf, 0x89d0, PDF_CMAP_RANGE, 11546 },
	{ 0x89d1, 0x89d1, PDF_CMAP_SINGLE, 8878 },
	{ 0x89d2, 0x89d7, PDF_CMAP_RANGE, 11548 },
	{ 0x89d8, 0x89d8, PDF_CMAP_SINGLE, 8009 },
	{ 0x89d9, 0x89da, PDF_CMAP_RANGE, 11554 },
	{ 0x89db, 0x89db, PDF_CMAP_SINGLE, 8504 },
	{ 0x89dc, 0x89f3, PDF_CMAP_RANGE, 11556 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 8321 },
	{ 0x89f5, 0x89fe, PDF_CMAP_RANGE, 11580 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 127 },
	{ 0x8a42, 0x8a58, PDF_CMAP_RANGE, 11591 },
	{ 0x8a59, 0x8a5e, PDF_CMAP_TABLE, 129 },
	{ 0x8a5f, 0x8a78, PDF_CMAP_RANGE, 11616 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8877 },
	{ 0x8a7a, 0x8a7e, PDF_CMAP_RANGE, 11642 },
	{ 0x8a80, 0x8ae3, PDF_CMAP_RANGE, 11647 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 8255 },
	{ 0x8ae5, 0x8afe, PDF_CMAP_RANGE, 11747 },
	{ 0x8b40, 0x8b43, PDF_CMAP_RANGE, 11773 },
	{ 0x8b44, 0x8b44, PDF_CMAP_SINGLE, 7950 },
	{ 0x8b45, 0x8b48, PDF_CMAP_RANGE, 11777 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 9209 },
	{ 0x8b4a, 0x8b79, PDF_CMAP_RANGE, 11781 },
	{ 0x8b7a, 0x8b7a, PDF_CMAP_SINGLE, 9212 },
	{ 0x8b7b, 0x8b7e, PDF_CMAP_RANGE, 11829 },
	{ 0x8b80, 0x8b8b, PDF_CMAP_RANGE, 11833 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 8297 },
	{ 0x8b8d, 0x8b9d, PDF_CMAP_RANGE, 11845 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 9207 },
	{ 0x8b9f, 0x8bb2, PDF_CMAP_RANGE, 11862 },
	{ 0x8bb3, 0x8bb3, PDF_CMAP_SINGLE, 9206 },
	{ 0x8bb4, 0x8bb8, PDF_CMAP_RANGE, 11882 },
	{ 0x8bb9, 0x8bb9, PDF_CMAP_SINGLE, 9213 },
	{ 0x8bba, 0x8bbd, PDF_CMAP_RANGE, 11887 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 9208 },
	{ 0x8bbf, 0x8bc5, PDF_CMAP_RANGE, 11891 },
	{ 0x8bc6, 0x8bc9, PDF_CMAP_TABLE, 135 },
	{ 0x8bca, 0x8bd3, PDF_CMAP_RANGE, 11899 },
	{ 0x8bd4, 0x8bd4, PDF_CMAP_SINGLE, 9217 },
	{ 0x8bd5, 0x8bdb, PDF_CMAP_RANGE, 11909 },
	{ 0x8bdc, 0x8bdc, PDF_CMAP_SINGLE, 9215 },
	{ 0x8bdd, 0x8be4, PDF_CMAP_RANGE, 11916 },
	{ 0x8be5, 0x8be5, PDF_CMAP_SINGLE, 9216 },
	{ 0x8be6, 0x8bea, PDF_CMAP_RANGE, 11924 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 8737 },
	{ 0x8bec, 0x8bef, PDF_CMAP_RANGE, 11929 },
	{ 0x8bf0, 0x8bf0, PDF_CMAP_SINGLE, 8482 },
	{ 0x8bf1, 0x8bfe, PDF_CMAP_RANGE, 11933 },
	{ 0x8c40, 0x8c43, PDF_CMAP_RANGE, 11947 },
	{ 0x8c44, 0x8c44, PDF_CMAP_SINGLE, 9211 },
	{ 0x8c45, 0x8c4e, PDF_CMAP_RANGE, 11951 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 8538 },
	{ 0x8c50, 0x8c56, PDF_CMAP_RANGE, 11961 },
	{ 0x8c57, 0x8c57, PDF_CMAP_SINGLE, 8677 },
	{ 0x8c58, 0x8c5b, PDF_CMAP_RANGE, 11968 },
	{ 0x8c5c, 0x8c5c, PDF_CMAP_SINGLE, 8279 },
	{ 0x8c5d, 0x8c7e, PDF_CMAP_RANGE, 11972 },
	{ 0x8c80, 0x8c8a, PDF_CMAP_RANGE, 12006 },
	{ 0x8c8b, 0x8c92, PDF_CMAP_TABLE, 139 },
	{ 0x8c93, 0x8c98, PDF_CMAP_RANGE, 12019 },
	{ 0x8c99, 0x8c9a, PDF_CMAP_TABLE, 147 },
	{ 0x8c9b, 0x8ca1, PDF_CMAP_RANGE, 12025 },
	{ 0x8ca2, 0x8ca7, PDF_CMAP_TABLE, 149 },
	{ 0x8ca8, 0x8cbf, PDF_CMAP_RANGE, 12033 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 9052 },
	{ 0x8cc1, 0x8cd1, PDF_CMAP_RANGE, 12057 },
	{ 0x8cd2, 0x8cd5, PDF_CMAP_TABLE, 155 },
	{ 0x8cd6, 0x8cd8, PDF_CMAP_RANGE, 12075 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 8510 },
	{ 0x8cda, 0x8cf8, PDF_CMAP_RANGE, 12078 },
	{ 0x8cf9, 0x8cf9, PDF_CMAP_SINGLE, 7958 },
	{ 0x8cfa, 0x8cfe, PDF_CMAP_RANGE, 12109 },
	{ 0x8d40, 0x8d72, PDF_CMAP_RANGE, 12114 },
	{ 0x8d73, 0x8d75, PDF_CMAP_TABLE, 159 },
	{ 0x8d76, 0x8d7a, PDF_CMAP_RANGE, 12166 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 8628 },
	{ 0x8d7c, 0x8d7e, PDF_CMAP_RANGE, 12171 },
	{ 0x8d80, 0x8d87, PDF_CMAP_RANGE, 12174 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 9103 },
	{ 0x8d89, 0x8d8e, PDF_CMAP_RANGE, 12182 },
	{ 0x8d8f, 0x8d8f, PDF_CMAP_SINGLE, 7962 },
	{ 0x8d90, 0x8d9d, PDF_CMAP_RANGE, 12188 },
	{ 0x8d9e, 0x8d9e, PDF_CMAP_SINGLE, 9099 },
	{ 0x8d9f, 0x8db8, PDF_CMAP_RANGE, 12202 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 9098 },
	{ 0x8dba, 0x8de1, PDF_CMAP_RANGE, 12228 },
	{ 0x8de2, 0x8de4, PDF_CMAP_TABLE, 162 },
	{ 0x8de5, 0x8de6, PDF_CMAP_RANGE, 12269 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 9096 },
	{ 0x8de8, 0x8df6, PDF_CMAP_RANGE, 12271 },
	{ 0x8df7, 0x8df7, PDF_CMAP_SINGLE, 9102 },
	{ 0x8df8, 0x8dfd, PDF_CMAP_RANGE, 12286 },
	{ 0x8dfe, 0x8dfe, PDF_CMAP_SINGLE, 9101 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 12292 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 9100 },
	{ 0x8e47, 0x8e55, PDF_CMAP_RANGE, 12298 },
	{ 0x8e56, 0x8e5a, PDF_CMAP_TABLE, 165 },
	{ 0x8e5b, 0x8e67, PDF_CMAP_RANGE, 12315 },
	{ 0x8e68, 0x8e68, PDF_CMAP_SINGLE, 8169 },
	{ 0x8e69, 0x8e6d, PDF_CMAP_RANGE, 12328 },
	{ 0x8e6e, 0x8e70, PDF_CMAP_TABLE, 170 },
	{ 0x8e71, 0x8e7e, PDF_CMAP_RANGE, 12334 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 8996 },
	{ 0x8e81, 0x8e9a, PDF_CMAP_RANGE, 12348 },
	{ 0x8e9b, 0x8e9b, PDF_CMAP_SINGLE, 8515 },
	{ 0x8e9c, 0x8e9e, PDF_CMAP_RANGE, 12374 },
	{ 0x8e9f, 0x8e9f, PDF_CMAP_SINGLE, 8489 },
	{ 0x8ea0, 0x8ea3, PDF_CMAP_RANGE, 12377 },
	{ 0x8ea4, 0x8ea4, PDF_CMAP_SINGLE, 8823 },
	{ 0x8ea5, 0x8ea6, PDF_CMAP_RANGE, 12381 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 7847 },
	{ 0x8ea8, 0x8eab, PDF_CMAP_RANGE, 12383 },
	{ 0x8eac, 0x8eae, PDF_CMAP_TABLE, 173 },
	{ 0x8eaf, 0x8ebc, PDF_CMAP_RANGE, 12388 },
	{ 0x8ebd, 0x8ebe, PDF_CMAP_TABLE, 176 },
	{ 0x8ebf, 0x8ec2, PDF_CMAP_RANGE, 12402 },
	{ 0x8ec3, 0x8ec5, PDF_CMAP_TABLE, 178 },
	{ 0x8ec6, 0x8ecc, PDF_CMAP_RANGE, 12407 },
	{ 0x8ecd, 0x8ece, PDF_CMAP_TABLE, 181 },
	{ 0x8ecf, 0x8ed5, PDF_CMAP_RANGE, 12414 },
	{ 0x8ed6, 0x8ed7, PDF_CMAP_TABLE, 183 },
	{ 0x8ed8, 0x8eeb, PDF_CMAP_RANGE, 12421 },
	{ 0x8eec, 0x8eec, PDF_CMAP_SINGLE, 8160 },
	{ 0x8eed, 0x8efe, PDF_CMAP_RANGE, 12441 },
	{ 0x8f40, 0x8f51, PDF_CMAP_RANGE, 12459 },
	{ 0x8f52, 0x8f56, PDF_CMAP_TABLE, 185 },
	{ 0x8f57, 0x8f5c, PDF_CMAP_RANGE, 12477 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 8261 },
	{ 0x8f5e, 0x8f63, PDF_CMAP_RANGE, 12483 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 8568 },
	{ 0x8f65, 0x8f7e, PDF_CMAP_RANGE, 12489 },
	{ 0x8f80, 0x8f85, PDF_CMAP_RANGE, 12515 },
	{ 0x8f86, 0x8f88, PDF_CMAP_TABLE, 190 },
	{ 0x8f89, 0x8f94, PDF_CMAP_RANGE, 12522 },
	{ 0x8f95, 0x8f97, PDF_CMAP_TABLE, 193 },
	{ 0x8f98, 0x8f9a, PDF_CMAP_RANGE, 12535 },
	{ 0x8f9b, 0x8f9d, PDF_CMAP_TABLE, 196 },
	{ 0x8f9e, 0x8fa0, PDF_CMAP_RANGE, 12539 },
	{ 0x8fa1, 0x8fa1, PDF_CMAP_SINGLE, 9852 },
	{ 0x8fa2, 0x8fbc, PDF_CMAP_RANGE, 12542 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 8133 },
	{ 0x8fbe, 0x8fc3, PDF_CMAP_RANGE, 12569 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 199 },
	{ 0x8fc7, 0x8fcc, PDF_CMAP_RANGE, 12576 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 7947 },
	{ 0x8fce, 0x8fd7, PDF_CMAP_RANGE, 12582 },
	{ 0x8fd8, 0x8fd8, PDF_CMAP_SINGLE, 7804 },
	{ 0x8fd9, 0x8ffe, PDF_CMAP_RANGE, 12592 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 12630 },
	{ 0x9080, 0x909c, PDF_CMAP_RANGE, 12693 },
	{ 0x909d, 0x909e, PDF_CMAP_TABLE, 202 },
	{ 0x909f, 0x90b9, PDF_CMAP_RANGE, 12722 },
	{ 0x90ba, 0x90ba, PDF_CMAP_SINGLE, 7907 },
	{ 0x90bb, 0x90bf, PDF_CMAP_RANGE, 12749 },
	{ 0x90c0, 0x90c1, PDF_CMAP_TABLE, 204 },
	{ 0x90c2, 0x90c4, PDF_CMAP_RANGE, 12754 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 9145 },
	{ 0x90c6, 0x90da, PDF_CMAP_RANGE, 12757 },
	{ 0x90db, 0x90dc, PDF_CMAP_TABLE, 206 },
	{ 0x90dd, 0x90ec, PDF_CMAP_RANGE, 12778 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 9141 },
	{ 0x90ee, 0x90ef, PDF_CMAP_RANGE, 12794 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 9146 },
	{ 0x90f1, 0x90f6, PDF_CMAP_RANGE, 12796 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 9139 },
	{ 0x90f8, 0x90fe, PDF_CMAP_RANGE, 12802 },
	{ 0x9140, 0x9141, PDF_CMAP_RANGE, 12809 },
	{ 0x9142, 0x9142, PDF_CMAP_SINGLE, 8546 },
	{ 0x9143, 0x914a, PDF_CMAP_RANGE, 12811 },
	{ 0x914b, 0x914d, PDF_CMAP_TABLE, 208 },
	{ 0x914e, 0x9150, PDF_CMAP_RANGE, 12820 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 9143 },
	{ 0x9152, 0x9153, PDF_CMAP_RANGE, 12823 },
	{ 0x9154, 0x9155, PDF_CMAP_TABLE, 211 },
	{ 0x9156, 0x9158, PDF_CMAP_RANGE, 12825 },
	{ 0x9159, 0x915a, PDF_CMAP_TABLE, 213 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 12828 },
	{ 0x915d, 0x915d, PDF_CMAP_SINGLE, 8274 },
	{ 0x915e, 0x9160, PDF_CMAP_RANGE, 12830 },
	{ 0x9161, 0x9163, PDF_CMAP_TABLE, 215 },
	{ 0x9164, 0x916d, PDF_CMAP_RANGE, 12834 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 8754 },
	{ 0x916f, 0x9175, PDF_CMAP_RANGE, 12844 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 7743 },
	{ 0x9177, 0x9179, PDF_CMAP_RANGE, 12851 },
	{ 0x917a, 0x917c, PDF_CMAP_TABLE, 218 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 12854 },
	{ 0x9180, 0x9183, PDF_CMAP_RANGE, 12856 },
	{ 0x9184, 0x9184, PDF_CMAP_SINGLE, 7854 },
	{ 0x9185, 0x918c, PDF_CMAP_RANGE, 12860 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 7931 },
	{ 0x918e, 0x9190, PDF_CMAP_RANGE, 12868 },
	{ 0x9191, 0x9193, PDF_CMAP_TABLE, 221 },
	{ 0x9194, 0x9196, PDF_CMAP_RANGE, 12872 },
	{ 0x9197, 0x9197, PDF_CMAP_SINGLE, 8644 },
	{ 0x9198, 0x919a, PDF_CMAP_RANGE, 12875 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 8724 },
	{ 0x919c, 0x91a8, PDF_CMAP_RANGE, 12878 },
	{ 0x91a9, 0x91ab, PDF_CMAP_TABLE, 224 },
	{ 0x91ac, 0x91b9, PDF_CMAP_RANGE, 12891 },
	{ 0x91ba, 0x91bb, PDF_CMAP_TABLE, 227 },
	{ 0x91bc, 0x91be, PDF_CMAP_RANGE, 12905 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 9440 },
	{ 0x91c0, 0x91c2, PDF_CMAP_RANGE, 12908 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 9144 },
	{ 0x91c4, 0x91cc, PDF_CMAP_RANGE, 12911 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 7809 },
	{ 0x91ce, 0x91cf, PDF_CMAP_RANGE, 12920 },
	{ 0x91d0, 0x91d9, PDF_CMAP_TABLE, 229 },
	{ 0x91da, 0x91de, PDF_CMAP_RANGE, 12925 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 9441 },
	{ 0x91e0, 0x91e1, PDF_CMAP_RANGE, 12930 },
	{ 0x91e2, 0x91e2, PDF_CMAP_SINGLE, 9379 },
	{ 0x91e3, 0x91e9, PDF_CMAP_RANGE, 12932 },
	{ 0x91ea, 0x91ea, PDF_CMAP_SINGLE, 9380 },
	{ 0x91eb, 0x91ef, PDF_CMAP_RANGE, 12939 },
	{ 0x91f0, 0x91f2, PDF_CMAP_TABLE, 239 },
	{ 0x91f3, 0x91fe, PDF_CMAP_RANGE, 12945 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 12957 },
	{ 0x9280, 0x92b5, PDF_CMAP_RANGE, 13020 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 8657 },
	{ 0x92b7, 0x92cd, PDF_CMAP_RANGE, 13074 },
	{ 0x92ce, 0x92d0, PDF_CMAP_TABLE, 242 },
	{ 0x92d1, 0x92d3, PDF_CMAP_RANGE, 13098 },
	{ 0x92d4, 0x92d4, PDF_CMAP_SINGLE, 9858 },
	{ 0x92d5, 0x92de, PDF_CMAP_RANGE, 13101 },
	{ 0x92df, 0x92e0, PDF_CMAP_TABLE, 245 },
	{ 0x92e1, 0x92fd, PDF_CMAP_RANGE, 13111 },
	{ 0x92fe, 0x92fe, PDF_CMAP_SINGLE, 8077 },
	{ 0x9340, 0x934f, PDF_CMAP_RANGE, 13140 },
	{ 0x9350, 0x9350, PDF_CMAP_SINGLE, 8703 },
	{ 0x9351, 0x935c, PDF_CMAP_RANGE, 13156 },
	{ 0x935d, 0x935d, PDF_CMAP_SINGLE, 8024 },
	{ 0x935e, 0x936f, PDF_CMAP_RANGE, 13168 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 8539 },
	{ 0x9371, 0x9375, PDF_CMAP_RANGE, 13186 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 7862 },
	{ 0x9377, 0x937e, PDF_CMAP_RANGE, 13191 },
	{ 0x9380, 0x938b, PDF_CMAP_RANGE, 13199 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 8411 },
	{ 0x938d, 0x939c, PDF_CMAP_RANGE, 13211 },
	{ 0x939d, 0x939d, PDF_CMAP_SINGLE, 9056 },
	{ 0x939e, 0x93a4, PDF_CMAP_RANGE, 13227 },
	{ 0x93a5, 0x93a7, PDF_CMAP_TABLE, 247 },
	{ 0x93a8, 0x93b3, PDF_CMAP_RANGE, 13235 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 8846 },
	{ 0x93b5, 0x93b7, PDF_CMAP_RANGE, 13247 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 8159 },
	{ 0x93b9, 0x93ba, PDF_CMAP_RANGE, 13250 },
	{ 0x93bb, 0x93bd, PDF_CMAP_TABLE, 250 },
	{ 0x93be, 0x93c5, PDF_CMAP_RANGE, 13253 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 8194 },
	{ 0x93c7, 0x93ce, PDF_CMAP_RANGE, 13261 },
	{ 0x93cf, 0x93cf, PDF_CMAP_SINGLE, 8339 },
	{ 0x93d0, 0x93d6, PDF_CMAP_RANGE, 13269 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 9055 },
	{ 0x93d8, 0x93da, PDF_CMAP_RANGE, 13276 },
	{ 0x93db, 0x93dc, PDF_CMAP_TABLE, 253 },
	{ 0x93dd, 0x93e0, PDF_CMAP_RANGE, 13279 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 7944 },
	{ 0x93e2, 0x93e3, PDF_CMAP_RANGE, 13283 },
	{ 0x93e4, 0x93e5, PDF_CMAP_TABLE, 255 },
	{ 0x93e6, 0x93e8, PDF_CMAP_RANGE, 13285 },
	{ 0x93e9, 0x93f1, PDF_CMAP_TABLE, 257 },
	{ 0x93f2, 0x93f3, PDF_CMAP_RANGE, 13291 },
	{ 0x93f4, 0x93f5, PDF_CMAP_TABLE, 266 },
	{ 0x93f6, 0x93f9, PDF_CMAP_RANGE, 13293 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 7849 },
	{ 0x93fb, 0x93fd, PDF_CMAP_RANGE, 13297 },
	{ 0x93fe, 0x93fe, PDF_CMAP_SINGLE, 8140 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 13300 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8050 },
	{ 0x9445, 0x944c, PDF_CMAP_RANGE, 13304 },
	{ 0x944d, 0x944d, PDF_CMAP_SINGLE, 8344 },
	{ 0x944e, 0x944f, PDF_CMAP_RANGE, 13312 },
	{ 0x9450, 0x9455, PDF_CMAP_TABLE, 268 },
	{ 0x9456, 0x9457, PDF_CMAP_RANGE, 13315 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 9061 },
	{ 0x9459, 0x945a, PDF_CMAP_RANGE, 13317 },
	{ 0x945b, 0x945f, PDF_CMAP_TABLE, 274 },
	{ 0x9460, 0x9463, PDF_CMAP_RANGE, 13320 },
	{ 0x9464, 0x9466, PDF_CMAP_TABLE, 279 },
	{ 0x9467, 0x946d, PDF_CMAP_RANGE, 13325 },
	{ 0x946e, 0x946e, PDF_CMAP_SINGLE, 8252 },
	{ 0x946f, 0x9471, PDF_CMAP_RANGE, 13332 },
	{ 0x9472, 0x947a, PDF_CMAP_TABLE, 282 },
	{ 0x947b, 0x947e, PDF_CMAP_RANGE, 13339 },
	{ 0x9480, 0x9482, PDF_CMAP_TABLE, 291 },
	{ 0x9483, 0x9486, PDF_CMAP_RANGE, 13343 },
	{ 0x9487, 0x9488, PDF_CMAP_TABLE, 294 },
	{ 0x9489, 0x94a0, PDF_CMAP_RANGE, 13347 },
	{ 0x94a1, 0x94a1, PDF_CMAP_SINGLE, 7726 },
	{ 0x94a2, 0x94b2, PDF_CMAP_RANGE, 13371 },
	{ 0x94b3, 0x94b5, PDF_CMAP_TABLE, 296 },
	{ 0x94b6, 0x94be, PDF_CMAP_RANGE, 13389 },
	{ 0x94bf, 0x94c0, PDF_CMAP_TABLE, 299 },
	{ 0x94c1, 0x94cb, PDF_CMAP_RANGE, 13398 },
	{ 0x94cc, 0x94cc, PDF_CMAP_SINGLE, 9428 },
	{ 0x94cd, 0x94d7, PDF_CMAP_RANGE, 13409 },
	{ 0x94d8, 0x94d8, PDF_CMAP_SINGLE, 8815 },
	{ 0x94d9, 0x94df, PDF_CMAP_RANGE, 13420 },
	{ 0x94e0, 0x94e0, PDF_CMAP_SINGLE, 7895 },
	{ 0x94e1, 0x94fe, PDF_CMAP_RANGE, 13427 },
	{ 0x9540, 0x9571, PDF_CMAP_RANGE, 13457 },
	{ 0x9572, 0x9572, PDF_CMAP_SINGLE, 8493 },
	{ 0x9573, 0x957e, PDF_CMAP_RANGE, 13507 },
	{ 0x9580, 0x9582, PDF_CMAP_RANGE, 13519 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 8859 },
	{ 0x9584, 0x959d, PDF_CMAP_RANGE, 13522 },
	{ 0x959e, 0x959f, PDF_CMAP_TABLE, 301 },
	{ 0x95a0, 0x95b2, PDF_CMAP_RANGE, 13548 },
	{ 0x95b3, 0x95b3, PDF_CMAP_SINGLE, 7801 },
	{ 0x95b4, 0x95b9, PDF_CMAP_RANGE, 13567 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 8794 },
	{ 0x95bb, 0x95ce, PDF_CMAP_RANGE, 13573 },
	{ 0x95cf, 0x95d4, PDF_CMAP_TABLE, 303 },
	{ 0x95d5, 0x95e0, PDF_CMAP_RANGE, 13595 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 9385 },
	{ 0x95e2, 0x95e6, PDF_CMAP_RANGE, 13607 },
	{ 0x95e7, 0x95e7, PDF_CMAP_SINGLE, 8167 },
	{ 0x95e8, 0x95f0, PDF_CMAP_RANGE, 13612 },
	{ 0x95f1, 0x95f1, PDF_CMAP_SINGLE, 8466 },
	{ 0x95f2, 0x95f7, PDF_CMAP_RANGE, 13621 },
	{ 0x95f8, 0x95f8, PDF_CMAP_SINGLE, 8508 },
	{ 0x95f9, 0x95fd, PDF_CMAP_RANGE, 13627 },
	{ 0x95fe, 0x95fe, PDF_CMAP_SINGLE, 8028 },
	{ 0x9640, 0x9655, PDF_CMAP_RANGE, 13632 },
	{ 0x9656, 0x9656, PDF_CMAP_SINGLE, 9412 },
	{ 0x9657, 0x967b, PDF_CMAP_RANGE, 13654 },
	{ 0x967c, 0x967c, PDF_CMAP_SINGLE, 7884 },
	{ 0x967d, 0x967e, PDF_CMAP_RANGE, 13691 },
	{ 0x9680, 0x96fe, PDF_CMAP_RANGE, 13693 },
	{ 0x9740, 0x9766, PDF_CMAP_RANGE, 13820 },
	{ 0x9767, 0x9767, PDF_CMAP_SINGLE, 9322 },
	{ 0x9768, 0x976b, PDF_CMAP_RANGE, 13859 },
	{ 0x976c, 0x976e, PDF_CMAP_TABLE, 309 },
	{ 0x976f, 0x977e, PDF_CMAP_RANGE, 13864 },
	{ 0x9780, 0x9795, PDF_CMAP_RANGE, 13880 },
	{ 0x9796, 0x9797, PDF_CMAP_TABLE, 312 },
	{ 0x9798, 0x979c, PDF_CMAP_RANGE, 13902 },
	{ 0x979d, 0x979d, PDF_CMAP_SINGLE, 7886 },
	{ 0x979e, 0x97a2, PDF_CMAP_RANGE, 13907 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 8818 },
	{ 0x97a4, 0x97be, PDF_CMAP_RANGE, 13912 },
	{ 0x97bf, 0x97bf, PDF_CMAP_SINGLE, 9331 },
	{ 0x97c0, 0x97ed, PDF_CMAP_RANGE, 13939 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 8702 },
	{ 0x97ef, 0x97f6, PDF_CMAP_RANGE, 13985 },
	{ 0x97f7, 0x97f7, PDF_CMAP_SINGLE, 7934 },
	{ 0x97f8, 0x97fe, PDF_CMAP_RANGE, 13993 },
	{ 0x9840, 0x9844, PDF_CMAP_RANGE, 14000 },
	{ 0x9845, 0x9845, PDF_CMAP_SINGLE, 9333 },
	{ 0x9846, 0x9848, PDF_CMAP_RANGE, 14005 },
	{ 0x9849, 0x9849, PDF_CMAP_SINGLE, 8714 },
	{ 0x984a, 0x984e, PDF_CMAP_RANGE, 14008 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 8047 },
	{ 0x9850, 0x9870, PDF_CMAP_RANGE, 14013 },
	{ 0x9871, 0x9873, PDF_CMAP_TABLE, 314 },
	{ 0x9874, 0x987e, PDF_CMAP_RANGE, 14047 },
	{ 0x9880, 0x9881, PDF_CMAP_TABLE, 317 },
	{ 0x9882, 0x988a, PDF_CMAP_RANGE, 14059 },
	{ 0x988b, 0x988c, PDF_CMAP_TABLE, 319 },
	{ 0x988d, 0x989f, PDF_CMAP_RANGE, 14068 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 9340 },
	{ 0x98a1, 0x98a9, PDF_CMAP_RANGE, 14087 },
	{ 0x98aa, 0x98aa, PDF_CMAP_SINGLE, 8097 },
	{ 0x98ab, 0x98b5, PDF_CMAP_RANGE, 14096 },
	{ 0x98b6, 0x98b7, PDF_CMAP_TABLE, 321 },
	{ 0x98b8, 0x98b9, PDF_CMAP_RANGE, 14107 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 9324 },
	{ 0x98bb, 0x98c6, PDF_CMAP_RANGE, 14109 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 8254 },
	{ 0x98c8, 0x98ca, PDF_CMAP_RANGE, 14121 },
	{ 0x98cb, 0x98cb, PDF_CMAP_SINGLE, 7756 },
	{ 0x98cc, 0x98cf, PDF_CMAP_RANGE, 14124 },
	{ 0x98d0, 0x98d0, PDF_CMAP_SINGLE, 8506 },
	{ 0x98d1, 0x98d2, PDF_CMAP_RANGE, 14128 },
	{ 0x98d3, 0x98d3, PDF_CMAP_SINGLE, 8708 },
	{ 0x98d4, 0x98e2, PDF_CMAP_RANGE, 14130 },
	{ 0x98e3, 0x98e5, PDF_CMAP_TABLE, 323 },
	{ 0x98e6, 0x98ee, PDF_CMAP_RANGE, 14145 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 9332 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 14154 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 8413 },
	{ 0x98f3, 0x98fe, PDF_CMAP_RANGE, 14156 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 14168 },
	{ 0x9943, 0x9945, PDF_CMAP_TABLE, 326 },
	{ 0x9946, 0x9965, PDF_CMAP_RANGE, 14172 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 9330 },
	{ 0x9967, 0x996d, PDF_CMAP_RANGE, 14204 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 7861 },
	{ 0x996f, 0x9974, PDF_CMAP_RANGE, 14211 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 9336 },
	{ 0x9976, 0x9979, PDF_CMAP_RANGE, 14217 },
	{ 0x997a, 0x997b, PDF_CMAP_TABLE, 329 },
	{ 0x997c, 0x997e, PDF_CMAP_RANGE, 14221 },
	{ 0x9980, 0x9984, PDF_CMAP_RANGE, 14224 },
	{ 0x9985, 0x9985, PDF_CMAP_SINGLE, 9881 },
	{ 0x9986, 0x9988, PDF_CMAP_RANGE, 14229 },
	{ 0x9989, 0x9989, PDF_CMAP_SINGLE, 9346 },
	{ 0x998a, 0x998d, PDF_CMAP_RANGE, 14232 },
	{ 0x998e, 0x998e, PDF_CMAP_SINGLE, 8353 },
	{ 0x998f, 0x9990, PDF_CMAP_RANGE, 14236 },
	{ 0x9991, 0x9991, PDF_CMAP_SINGLE, 8082 },
	{ 0x9992, 0x9998, PDF_CMAP_RANGE, 14238 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 7992 },
	{ 0x999a, 0x99a8, PDF_CMAP_RANGE, 14245 },
	{ 0x99a9, 0x99a9, PDF_CMAP_SINGLE, 9349 },
	{ 0x99aa, 0x99af, PDF_CMAP_RANGE, 14260 },
	{ 0x99b0, 0x99b5, PDF_CMAP_TABLE, 331 },
	{ 0x99b6, 0x99bc, PDF_CMAP_RANGE, 14267 },
	{ 0x99bd, 0x99c2, PDF_CMAP_TABLE, 337 },
	{ 0x99c3, 0x99c8, PDF_CMAP_RANGE, 14276 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 9327 },
	{ 0x99ca, 0x99cd, PDF_CMAP_RANGE, 14282 },
	{ 0x99ce, 0x99ce, PDF_CMAP_SINGLE, 9345 },
	{ 0x99cf, 0x99d0, PDF_CMAP_RANGE, 14286 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 8736 },
	{ 0x99d2, 0x99d9, PDF_CMAP_RANGE, 14288 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 8181 },
	{ 0x99db, 0x99df, PDF_CMAP_RANGE, 14296 },
	{ 0x99e0, 0x99e0, PDF_CMAP_SINGLE, 8436 },
	{ 0x99e1, 0x99e4, PDF_CMAP_RANGE, 14301 },
	{ 0x99e5, 0x99e5, PDF_CMAP_SINGLE, 9341 },
	{ 0x99e6, 0x99e7, PDF_CMAP_RANGE, 14305 },
	{ 0x99e8, 0x99e8, PDF_CMAP_SINGLE, 9337 },
	{ 0x99e9, 0x99eb, PDF_CMAP_RANGE, 14307 },
	{ 0x99ec, 0x99ec, PDF_CMAP_SINGLE, 9342 },
	{ 0x99ed, 0x99f3, PDF_CMAP_RANGE, 14310 },
	{ 0x99f4, 0x99f4, PDF_CMAP_SINGLE, 9338 },
	{ 0x99f5, 0x99fe, PDF_CMAP_RANGE, 14317 },
	{ 0x9a40, 0x9a49, PDF_CMAP_RANGE, 14327 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 8419 },
	{ 0x9a4b, 0x9a56, PDF_CMAP_RANGE, 14337 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 8365 },
	{ 0x9a58, 0x9a64, PDF_CMAP_RANGE, 14349 },
	{ 0x9a65, 0x9a67, PDF_CMAP_TABLE, 343 },
	{ 0x9a68, 0x9a70, PDF_CMAP_RANGE, 14363 },
	{ 0x9a71, 0x9a71, PDF_CMAP_SINGLE, 8537 },
	{ 0x9a72, 0x9a75, PDF_CMAP_RANGE, 14372 },
	{ 0x9a76, 0x9a77, PDF_CMAP_TABLE, 346 },
	{ 0x9a78, 0x9a7e, PDF_CMAP_RANGE, 14376 },
	{ 0x9a80, 0x9a87, PDF_CMAP_RANGE, 14383 },
	{ 0x9a88, 0x9a88, PDF_CMAP_SINGLE, 7772 },
	{ 0x9a89, 0x9a8b, PDF_CMAP_RANGE, 14391 },
	{ 0x9a8c, 0x9a8c, PDF_CMAP_SINGLE, 9352 },
	{ 0x9a8d, 0x9a90, PDF_CMAP_RANGE, 14394 },
	{ 0x9a91, 0x9a91, PDF_CMAP_SINGLE, 9351 },
	{ 0x9a92, 0x9a96, PDF_CMAP_RANGE, 14398 },
	{ 0x9a97, 0x9a97, PDF_CMAP_SINGLE, 9354 },
	{ 0x9a98, 0x9a99, PDF_CMAP_RANGE, 14403 },
	{ 0x9a9a, 0x9a9b, PDF_CMAP_TABLE, 348 },
	{ 0x9a9c, 0x9a9d, PDF_CMAP_RANGE, 14405 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 8067 },
	{ 0x9a9f, 0x9aa1, PDF_CMAP_RANGE, 14407 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_TABLE, 350 },
	{ 0x9aa4, 0x9aa9, PDF_CMAP_RANGE, 14410 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 8367 },
	{ 0x9aab, 0x9acf, PDF_CMAP_RANGE, 14416 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 9408 },
	{ 0x9ad1, 0x9ad5, PDF_CMAP_RANGE, 14453 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 8813 },
	{ 0x9ad7, 0x9ad9, PDF_CMAP_RANGE, 14458 },
	{ 0x9ada, 0x9ada, PDF_CMAP_SINGLE, 9409 },
	{ 0x9adb, 0x9ae1, PDF_CMAP_RANGE, 14461 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_TABLE, 352 },
	{ 0x9ae6, 0x9afe, PDF_CMAP_RANGE, 14469 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 14494 },
	{ 0x9b80, 0x9bd0, PDF_CMAP_RANGE, 14557 },
	{ 0x9bd1, 0x9bd1, PDF_CMAP_SINGLE, 9180 },
	{ 0x9bd2, 0x9bdb, PDF_CMAP_RANGE, 14638 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 9179 },
	{ 0x9bdd, 0x9bfe, PDF_CMAP_RANGE, 14648 },
	{ 0x9c40, 0x9c52, PDF_CMAP_RANGE, 14682 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 8286 },
	{ 0x9c54, 0x9c58, PDF_CMAP_RANGE, 14701 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_TABLE, 356 },
	{ 0x9c5d, 0x9c74, PDF_CMAP_RANGE, 14707 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 8607 },
	{ 0x9c76, 0x9c78, PDF_CMAP_RANGE, 14731 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 7782 },
	{ 0x9c7a, 0x9c7e, PDF_CMAP_RANGE, 14734 },
	{ 0x9c80, 0x9c85, PDF_CMAP_RANGE, 14739 },
	{ 0x9c86, 0x9c86, PDF_CMAP_SINGLE, 8035 },
	{ 0x9c87, 0x9c9c, PDF_CMAP_RANGE, 14745 },
	{ 0x9c9d, 0x9c9d, PDF_CMAP_SINGLE, 9181 },
	{ 0x9c9e, 0x9caa, PDF_CMAP_RANGE, 14767 },
	{ 0x9cab, 0x9cab, PDF_CMAP_SINGLE, 8555 },
	{ 0x9cac, 0x9cc9, PDF_CMAP_RANGE, 14780 },
	{ 0x9cca, 0x9cca, PDF_CMAP_SINGLE, 8885 },
	{ 0x9ccb, 0x9cce, PDF_CMAP_RANGE, 14810 },
	{ 0x9ccf, 0x9ccf, PDF_CMAP_SINGLE, 7974 },
	{ 0x9cd0, 0x9ce5, PDF_CMAP_RANGE, 14814 },
	{ 0x9ce6, 0x9ce7, PDF_CMAP_TABLE, 360 },
	{ 0x9ce8, 0x9ceb, PDF_CMAP_RANGE, 14836 },
	{ 0x9cec, 0x9cee, PDF_CMAP_TABLE, 362 },
	{ 0x9cef, 0x9cfa, PDF_CMAP_RANGE, 14841 },
	{ 0x9cfb, 0x9cfb, PDF_CMAP_SINGLE, 8011 },
	{ 0x9cfc, 0x9cfd, PDF_CMAP_RANGE, 14853 },
	{ 0x9cfe, 0x9cfe, PDF_CMAP_SINGLE, 8850 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 14855 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 8484 },
	{ 0x9d43, 0x9d45, PDF_CMAP_RANGE, 14857 },
	{ 0x9d46, 0x9d47, PDF_CMAP_TABLE, 365 },
	{ 0x9d48, 0x9d4c, PDF_CMAP_RANGE, 14860 },
	{ 0x9d4d, 0x9d4f, PDF_CMAP_TABLE, 367 },
	{ 0x9d50, 0x9d60, PDF_CMAP_RANGE, 14866 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 8369 },
	{ 0x9d62, 0x9d67, PDF_CMAP_RANGE, 14883 },
	{ 0x9d68, 0x9d69, PDF_CMAP_TABLE, 370 },
	{ 0x9d6a, 0x9d6d, PDF_CMAP_RANGE, 14889 },
	{ 0x9d6e, 0x9d6e, PDF_CMAP_SINGLE, 8888 },
	{ 0x9d6f, 0x9d70, PDF_CMAP_RANGE, 14893 },
	{ 0x9d71, 0x9d71, PDF_CMAP_SINGLE, 8822 },
	{ 0x9d72, 0x9d74, PDF_CMAP_RANGE, 14895 },
	{ 0x9d75, 0x9d75, PDF_CMAP_SINGLE, 8091 },
	{ 0x9d76, 0x9d7a, PDF_CMAP_RANGE, 14898 },
	{ 0x9d7b, 0x9d7e, PDF_CMAP_TABLE, 372 },
	{ 0x9d80, 0x9d89, PDF_CMAP_RANGE, 14905 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 8382 },
	{ 0x9d8b, 0x9d8c, PDF_CMAP_RANGE, 14915 },
	{ 0x9d8d, 0x9d8d, PDF_CMAP_SINGLE, 8116 },
	{ 0x9d8e, 0x9d90, PDF_CMAP_RANGE, 14917 },
	{ 0x9d91, 0x9d91, PDF_CMAP_SINGLE, 9175 },
	{ 0x9d92, 0x9d98, PDF_CMAP_RANGE, 14920 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 8453 },
	{ 0x9d9a, 0x9da0, PDF_CMAP_RANGE, 14927 },
	{ 0x9da1, 0x9da2, PDF_CMAP_TABLE, 376 },
	{ 0x9da3, 0x9da6, PDF_CMAP_RANGE, 14934 },
	{ 0x9da7, 0x9da7, PDF_CMAP_SINGLE, 9194 },
	{ 0x9da8, 0x9dab, PDF_CMAP_RANGE, 14938 },
	{ 0x9dac, 0x9dad, PDF_CMAP_TABLE, 378 },
	{ 0x9dae, 0x9db1, PDF_CMAP_RANGE, 14942 },
	{ 0x9db2, 0x9db3, PDF_CMAP_TABLE, 380 },
	{ 0x9db4, 0x9dbd, PDF_CMAP_RANGE, 14946 },
	{ 0x9dbe, 0x9dbe, PDF_CMAP_SINGLE, 8093 },
	{ 0x9dbf, 0x9dc5, PDF_CMAP_RANGE, 14956 },
	{ 0x9dc6, 0x9dc6, PDF_CMAP_SINGLE, 9189 },
	{ 0x9dc7, 0x9dc8, PDF_CMAP_RANGE, 14963 },
	{ 0x9dc9, 0x9dc9, PDF_CMAP_SINGLE, 8804 },
	{ 0x9dca, 0x9dcc, PDF_CMAP_RANGE, 14965 },
	{ 0x9dcd, 0x9dcd, PDF_CMAP_SINGLE, 9442 },
	{ 0x9dce, 0x9dd1, PDF_CMAP_RANGE, 14968 },
	{ 0x9dd2, 0x9dd2, PDF_CMAP_SINGLE, 9182 },
	{ 0x9dd3, 0x9dd4, PDF_CMAP_RANGE, 14972 },
	{ 0x9dd5, 0x9dd5, PDF_CMAP_SINGLE, 7876 },
	{ 0x9dd6, 0x9de0, PDF_CMAP_RANGE, 14974 },
	{ 0x9de1, 0x9de2, PDF_CMAP_TABLE, 382 },
	{ 0x9de3, 0x9df0, PDF_CMAP_RANGE, 14985 },
	{ 0x9df1, 0x9df1, PDF_CMAP_SINGLE, 8491 },
	{ 0x9df2, 0x9df3, PDF_CMAP_RANGE, 14999 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 8357 },
	{ 0x9df5, 0x9df6, PDF_CMAP_RANGE, 15001 },
	{ 0x9df7, 0x9df7, PDF_CMAP_SINGLE, 9867 },
	{ 0x9df8, 0x9df9, PDF_CMAP_RANGE, 15003 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 8054 },
	{ 0x9dfb, 0x9dfc, PDF_CMAP_RANGE, 15005 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_TABLE, 384 },
	{ 0x9e40, 0x9e44, PDF_CMAP_RANGE, 15008 },
	{ 0x9e45, 0x9e45, PDF_CMAP_SINGLE, 8193 },
	{ 0x9e46, 0x9e47, PDF_CMAP_RANGE, 15013 },
	{ 0x9e48, 0x9e49, PDF_CMAP_TABLE, 386 },
	{ 0x9e4a, 0x9e51, PDF_CMAP_RANGE, 15015 },
	{ 0x9e52, 0x9e56, PDF_CMAP_TABLE, 388 },
	{ 0x9e57, 0x9e5c, PDF_CMAP_RANGE, 15025 },
	{ 0x9e5d, 0x9e5e, PDF_CMAP_TABLE, 393 },
	{ 0x9e5f, 0x9e60, PDF_CMAP_RANGE, 15031 },
	{ 0x9e61, 0x9e63, PDF_CMAP_TABLE, 395 },
	{ 0x9e64, 0x9e66, PDF_CMAP_RANGE, 15034 },
	{ 0x9e67, 0x9e67, PDF_CMAP_SINGLE, 9183 },
	{ 0x9e68, 0x9e6b, PDF_CMAP_RANGE, 15037 },
	{ 0x9e6c, 0x9e6c, PDF_CMAP_SINGLE, 7759 },
	{ 0x9e6d, 0x9e6e, PDF_CMAP_RANGE, 15041 },
	{ 0x9e6f, 0x9e6f, PDF_CMAP_SINGLE, 9177 },
	{ 0x9e70, 0x9e71, PDF_CMAP_RANGE, 15043 },
	{ 0x9e72, 0x9e75, PDF_CMAP_TABLE, 398 },
	{ 0x9e76, 0x9e7a, PDF_CMAP_RANGE, 15046 },
	{ 0x9e7b, 0x9e7c, PDF_CMAP_TABLE, 402 },
	{ 0x9e7d, 0x9e7e, PDF_CMAP_RANGE, 15051 },
	{ 0x9e80, 0x9e84, PDF_CMAP_RANGE, 15053 },
	{ 0x9e85, 0x9e87, PDF_CMAP_TABLE, 404 },
	{ 0x9e88, 0x9e90, PDF_CMAP_RANGE, 15059 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 8186 },
	{ 0x9e92, 0x9e95, PDF_CMAP_RANGE, 15068 },
	{ 0x9e96, 0x9e97, PDF_CMAP_TABLE, 407 },
	{ 0x9e98, 0x9ea1, PDF_CMAP_RANGE, 15072 },
	{ 0x9ea2, 0x9ea2, PDF_CMAP_SINGLE, 8454 },
	{ 0x9ea3, 0x9ea5, PDF_CMAP_RANGE, 15082 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 8203 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 15085 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 8550 },
	{ 0x9eaa, 0x9ead, PDF_CMAP_RANGE, 15087 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 9199 },
	{ 0x9eaf, 0x9eb2, PDF_CMAP_RANGE, 15091 },
	{ 0x9eb3, 0x9eb4, PDF_CMAP_TABLE, 409 },
	{ 0x9eb5, 0x9eb6, PDF_CMAP_RANGE, 15095 },
	{ 0x9eb7, 0x9eb7, PDF_CMAP_SINGLE, 9191 },
	{ 0x9eb8, 0x9ef4, PDF_CMAP_RANGE, 15097 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 8611 },
	{ 0x9ef6, 0x9efe, PDF_CMAP_RANGE, 15158 },
	{ 0x9f40, 0x9f4d, PDF_CMAP_RANGE, 15167 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 8570 },
	{ 0x9f4f, 0x9f6e, PDF_CMAP_RANGE, 15181 },
	{ 0x9f6f, 0x9f6f, PDF_CMAP_SINGLE, 8613 },
	{ 0x9f70, 0x9f7e, PDF_CMAP_RANGE, 15213 },
	{ 0x9f80, 0x9f91, PDF_CMAP_RANGE, 15228 },
	{ 0x9f92, 0x9f92, PDF_CMAP_SINGLE, 8226 },
	{ 0x9f93, 0x9f97, PDF_CMAP_RANGE, 15246 },
	{ 0x9f98, 0x9f98, PDF_CMAP_SINGLE, 9430 },
	{ 0x9f99, 0x9fa5, PDF_CMAP_RANGE, 15251 },
	{ 0x9fa6, 0x9fa6, PDF_CMAP_SINGLE, 9018 },
	{ 0x9fa7, 0x9fa8, PDF_CMAP_RANGE, 15264 },
	{ 0x9fa9, 0x9fa9, PDF_CMAP_SINGLE, 7918 },
	{ 0x9faa, 0x9fab, PDF_CMAP_RANGE, 15266 },
	{ 0x9fac, 0x9fac, PDF_CMAP_SINGLE, 9429 },
	{ 0x9fad, 0x9fc8, PDF_CMAP_RANGE, 15268 },
	{ 0x9fc9, 0x9fc9, PDF_CMAP_SINGLE, 8744 },
	{ 0x9fca, 0x9fcc, PDF_CMAP_RANGE, 15296 },
	{ 0x9fcd, 0x9fcd, PDF_CMAP_SINGLE, 9431 },
	{ 0x9fce, 0x9fe0, PDF_CMAP_RANGE, 15299 },
	{ 0x9fe1, 0x9fe1, PDF_CMAP_SINGLE, 8444 },
	{ 0x9fe2, 0x9fea, PDF_CMAP_RANGE, 15318 },
	{ 0x9feb, 0x9feb, PDF_CMAP_SINGLE, 7815 },
	{ 0x9fec, 0x9fed, PDF_CMAP_RANGE, 15327 },
	{ 0x9fee, 0x9fee, PDF_CMAP_SINGLE, 9432 },
	{ 0x9fef, 0x9ff3, PDF_CMAP_RANGE, 15329 },
	{ 0x9ff4, 0x9ff4, PDF_CMAP_SINGLE, 7866 },
	{ 0x9ff5, 0x9ffc, PDF_CMAP_RANGE, 15334 },
	{ 0x9ffd, 0x9ffe, PDF_CMAP_TABLE, 411 },
	{ 0xa040, 0xa042, PDF_CMAP_RANGE, 15343 },
	{ 0xa043, 0xa043, PDF_CMAP_SINGLE, 8556 },
	{ 0xa044, 0xa045, PDF_CMAP_RANGE, 15346 },
	{ 0xa046, 0xa046, PDF_CMAP_SINGLE, 9433 },
	{ 0xa047, 0xa048, PDF_CMAP_RANGE, 15348 },
	{ 0xa049, 0xa049, PDF_CMAP_SINGLE, 8743 },
	{ 0xa04a, 0xa04d, PDF_CMAP_RANGE, 15350 },
	{ 0xa04e, 0xa04e, PDF_CMAP_SINGLE, 7775 },
	{ 0xa04f, 0xa053, PDF_CMAP_RANGE, 15354 },
	{ 0xa054, 0xa054, PDF_CMAP_SINGLE, 8863 },
	{ 0xa055, 0xa059, PDF_CMAP_RANGE, 15359 },
	{ 0xa05a, 0xa05a, PDF_CMAP_SINGLE, 8029 },
	{ 0xa05b, 0xa060, PDF_CMAP_RANGE, 15364 },
	{ 0xa061, 0xa063, PDF_CMAP_TABLE, 413 },
	{ 0xa064, 0xa070, PDF_CMAP_RANGE, 15371 },
	{ 0xa071, 0xa071, PDF_CMAP_SINGLE, 8521 },
	{ 0xa072, 0xa073, PDF_CMAP_RANGE, 15384 },
	{ 0xa074, 0xa074, PDF_CMAP_SINGLE, 8262 },
	{ 0xa075, 0xa07e, PDF_CMAP_RANGE, 15386 },
	{ 0xa080, 0xa080, PDF_CMAP_SINGLE, 8192 },
	{ 0xa081, 0xa090, PDF_CMAP_RANGE, 15396 },
	{ 0xa091, 0xa091, PDF_CMAP_SINGLE, 8592 },
	{ 0xa092, 0xa093, PDF_CMAP_RANGE, 15412 },
	{ 0xa094, 0xa096, PDF_CMAP_TABLE, 416 },
	{ 0xa097, 0xa0a8, PDF_CMAP_RANGE, 15415 },
	{ 0xa0a9, 0xa0a9, PDF_CMAP_SINGLE, 9411 },
	{ 0xa0aa, 0xa0be, PDF_CMAP_RANGE, 15433 },
	{ 0xa0bf, 0xa0bf, PDF_CMAP_SINGLE, 8396 },
	{ 0xa0c0, 0xa0cd, PDF_CMAP_RANGE, 15454 },
	{ 0xa0ce, 0xa0ce, PDF_CMAP_SINGLE, 9024 },
	{ 0xa0cf, 0xa0d8, PDF_CMAP_RANGE, 15468 },
	{ 0xa0d9, 0xa0d9, PDF_CMAP_SINGLE, 7889 },
	{ 0xa0da, 0xa0dd, PDF_CMAP_RANGE, 15478 },
	{ 0xa0de, 0xa0de, PDF_CMAP_SINGLE, 8620 },
	{ 0xa0df, 0xa0ed, PDF_CMAP_RANGE, 15482 },
	{ 0xa0ee, 0xa0ee, PDF_CMAP_SINGLE, 8879 },
	{ 0xa0ef, 0xa0fe, PDF_CMAP_RANGE, 15497 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa2a1, 0xa2aa, PDF_CMAP_RANGE, 9897 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa840, 0xa87e, PDF_CMAP_RANGE, 9907 },
	{ 0xa880, 0xa895, PDF_CMAP_RANGE, 9970 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa940, 0xa957, PDF_CMAP_RANGE, 9992 },
	{ 0xa959, 0xa95a, PDF_CMAP_RANGE, 10016 },
	{ 0xa95c, 0xa95c, PDF_CMAP_SINGLE, 10018 },
	{ 0xa960, 0xa97e, PDF_CMAP_RANGE, 10019 },
	{ 0xa980, 0xa995, PDF_CMAP_RANGE, 10050 },
	{ 0xa996, 0xa996, PDF_CMAP_SINGLE, 7703 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaa40, 0xaa4c, PDF_CMAP_RANGE, 15513 },
	{ 0xaa4d, 0xaa4e, PDF_CMAP_TABLE, 419 },
	{ 0xaa4f, 0xaa70, PDF_CMAP_RANGE, 15526 },
	{ 0xaa71, 0xaa73, PDF_CMAP_TABLE, 421 },
	{ 0xaa74, 0xaa76, PDF_CMAP_RANGE, 15561 },
	{ 0xaa77, 0xaa77, PDF_CMAP_SINGLE, 9109 },
	{ 0xaa78, 0xaa79, PDF_CMAP_RANGE, 15564 },
	{ 0xaa7a, 0xaa7b, PDF_CMAP_TABLE, 424 },
	{ 0xaa7c, 0xaa7e, PDF_CMAP_RANGE, 15566 },
	{ 0xaa80, 0xaa99, PDF_CMAP_RANGE, 15569 },
	{ 0xaa9a, 0xaaa0, PDF_CMAP_TABLE, 426 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xab40, 0xab40, PDF_CMAP_SINGLE, 8036 },
	{ 0xab41, 0xab42, PDF_CMAP_RANGE, 15598 },
	{ 0xab43, 0xab4a, PDF_CMAP_TABLE, 433 },
	{ 0xab4b, 0xab4c, PDF_CMAP_RANGE, 15602 },
	{ 0xab4d, 0xab4d, PDF_CMAP_SINGLE, 9113 },
	{ 0xab4e, 0xab7e, PDF_CMAP_RANGE, 15604 },
	{ 0xab80, 0xaba0, PDF_CMAP_RANGE, 15653 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xac40, 0xac45, PDF_CMAP_RANGE, 15686 },
	{ 0xac46, 0xac46, PDF_CMAP_SINGLE, 8640 },
	{ 0xac47, 0xac70, PDF_CMAP_RANGE, 15692 },
	{ 0xac71, 0xac71, PDF_CMAP_SINGLE, 9312 },
	{ 0xac72, 0xac7b, PDF_CMAP_RANGE, 15734 },
	{ 0xac7c, 0xac7c, PDF_CMAP_SINGLE, 9308 },
	{ 0xac7d, 0xac7e, PDF_CMAP_RANGE, 15744 },
	{ 0xac80, 0xac8c, PDF_CMAP_RANGE, 15746 },
	{ 0xac8d, 0xac8d, PDF_CMAP_SINGLE, 8541 },
	{ 0xac8e, 0xac92, PDF_CMAP_RANGE, 15759 },
	{ 0xac93, 0xac94, PDF_CMAP_TABLE, 441 },
	{ 0xac95, 0xaca0, PDF_CMAP_RANGE, 15764 },
	{ 0xad40, 0xad48, PDF_CMAP_RANGE, 15776 },
	{ 0xad49, 0xad49, PDF_CMAP_SINGLE, 9313 },
	{ 0xad4a, 0xad5d, PDF_CMAP_RANGE, 15785 },
	{ 0xad5e, 0xad5e, PDF_CMAP_SINGLE, 9307 },
	{ 0xad5f, 0xad60, PDF_CMAP_RANGE, 15805 },
	{ 0xad61, 0xad61, PDF_CMAP_SINGLE, 9314 },
	{ 0xad62, 0xad67, PDF_CMAP_RANGE, 15807 },
	{ 0xad68, 0xad68, PDF_CMAP_SINGLE, 8020 },
	{ 0xad69, 0xad73, PDF_CMAP_RANGE, 15813 },
	{ 0xad74, 0xad74, PDF_CMAP_SINGLE, 9311 },
	{ 0xad75, 0xad7e, PDF_CMAP_RANGE, 15824 },
	{ 0xad80, 0xad81, PDF_CMAP_RANGE, 15834 },
	{ 0xad82, 0xad82, PDF_CMAP_SINGLE, 8428 },
	{ 0xad83, 0xad86, PDF_CMAP_RANGE, 15836 },
	{ 0xad87, 0xad87, PDF_CMAP_SINGLE, 9309 },
	{ 0xad88, 0xad8a, PDF_CMAP_RANGE, 15840 },
	{ 0xad8b, 0xad8b, PDF_CMAP_SINGLE, 9315 },
	{ 0xad8c, 0xad90, PDF_CMAP_RANGE, 15843 },
	{ 0xad91, 0xad91, PDF_CMAP_SINGLE, 9316 },
	{ 0xad92, 0xada0, PDF_CMAP_RANGE, 15848 },
	{ 0xae40, 0xae53, PDF_CMAP_RANGE, 15863 },
	{ 0xae54, 0xae54, PDF_CMAP_SINGLE, 9381 },
	{ 0xae55, 0xae61, PDF_CMAP_RANGE, 15883 },
	{ 0xae62, 0xae62, PDF_CMAP_SINGLE, 7792 },
	{ 0xae63, 0xae7e, PDF_CMAP_RANGE, 15896 },
	{ 0xae80, 0xae80, PDF_CMAP_SINGLE, 8335 },
	{ 0xae81, 0xae84, PDF_CMAP_RANGE, 15924 },
	{ 0xae85, 0xae85, PDF_CMAP_SINGLE, 7746 },
	{ 0xae86, 0xae8a, PDF_CMAP_RANGE, 15928 },
	{ 0xae8b, 0xae8b, PDF_CMAP_SINGLE, 8014 },
	{ 0xae8c, 0xae93, PDF_CMAP_RANGE, 15933 },
	{ 0xae94, 0xae94, PDF_CMAP_SINGLE, 7857 },
	{ 0xae95, 0xae9f, PDF_CMAP_RANGE, 15941 },
	{ 0xaea0, 0xaea0, PDF_CMAP_SINGLE, 7819 },
	{ 0xaf40, 0xaf63, PDF_CMAP_RANGE, 15952 },
	{ 0xaf64, 0xaf64, PDF_CMAP_SINGLE, 8134 },
	{ 0xaf65, 0xaf7a, PDF_CMAP_RANGE, 15988 },
	{ 0xaf7b, 0xaf7b, PDF_CMAP_SINGLE, 9646 },
	{ 0xaf7c, 0xaf7e, PDF_CMAP_RANGE, 16010 },
	{ 0xaf80, 0xaf81, PDF_CMAP_RANGE, 16013 },
	{ 0xaf82, 0xaf83, PDF_CMAP_TABLE, 443 },
	{ 0xaf84, 0xaf8d, PDF_CMAP_RANGE, 16015 },
	{ 0xaf8e, 0xaf91, PDF_CMAP_TABLE, 445 },
	{ 0xaf92, 0xaf9b, PDF_CMAP_RANGE, 16026 },
	{ 0xaf9c, 0xaf9c, PDF_CMAP_SINGLE, 9651 },
	{ 0xaf9d, 0xaf9e, PDF_CMAP_RANGE, 16036 },
	{ 0xaf9f, 0xafa0, PDF_CMAP_TABLE, 449 },
	{ 0xb040, 0xb040, PDF_CMAP_SINGLE, 16039 },
	{ 0xb041, 0xb042, PDF_CMAP_RANGE, 9647 },
	{ 0xb043, 0xb044, PDF_CMAP_TABLE, 451 },
	{ 0xb045, 0xb04e, PDF_CMAP_RANGE, 16041 },
	{ 0xb04f, 0xb04f, PDF_CMAP_SINGLE, 9644 },
	{ 0xb050, 0xb053, PDF_CMAP_RANGE, 16051 },
	{ 0xb054, 0xb054, PDF_CMAP_SINGLE, 7758 },
	{ 0xb055, 0xb056, PDF_CMAP_RANGE, 16055 },
	{ 0xb057, 0xb05f, PDF_CMAP_TABLE, 453 },
	{ 0xb060, 0xb061, PDF_CMAP_RANGE, 9652 },
	{ 0xb062, 0xb064, PDF_CMAP_TABLE, 462 },
	{ 0xb065, 0xb06b, PDF_CMAP_RANGE, 16060 },
	{ 0xb06c, 0xb06c, PDF_CMAP_SINGLE, 7913 },
	{ 0xb06d, 0xb07c, PDF_CMAP_RANGE, 16067 },
	{ 0xb07d, 0xb07e, PDF_CMAP_TABLE, 465 },
	{ 0xb080, 0xb096, PDF_CMAP_RANGE, 16084 },
	{ 0xb097, 0xb099, PDF_CMAP_TABLE, 467 },
	{ 0xb09a, 0xb0a0, PDF_CMAP_RANGE, 16108 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 940 },
	{ 0xb140, 0xb14a, PDF_CMAP_RANGE, 16115 },
	{ 0xb14b, 0xb152, PDF_CMAP_TABLE, 470 },
	{ 0xb153, 0xb17e, PDF_CMAP_RANGE, 16129 },
	{ 0xb180, 0xb196, PDF_CMAP_RANGE, 16173 },
	{ 0xb197, 0xb197, PDF_CMAP_SINGLE, 9860 },
	{ 0xb198, 0xb1a0, PDF_CMAP_RANGE, 16196 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1034 },
	{ 0xb240, 0xb241, PDF_CMAP_TABLE, 478 },
	{ 0xb242, 0xb266, PDF_CMAP_RANGE, 16206 },
	{ 0xb267, 0xb267, PDF_CMAP_SINGLE, 9454 },
	{ 0xb268, 0xb26c, PDF_CMAP_RANGE, 16243 },
	{ 0xb26d, 0xb26d, PDF_CMAP_SINGLE, 8308 },
	{ 0xb26e, 0xb273, PDF_CMAP_RANGE, 16248 },
	{ 0xb274, 0xb274, PDF_CMAP_SINGLE, 9863 },
	{ 0xb275, 0xb27e, PDF_CMAP_RANGE, 16254 },
	{ 0xb280, 0xb280, PDF_CMAP_SINGLE, 9456 },
	{ 0xb281, 0xb288, PDF_CMAP_RANGE, 16264 },
	{ 0xb289, 0xb289, PDF_CMAP_SINGLE, 9869 },
	{ 0xb28a, 0xb299, PDF_CMAP_RANGE, 16272 },
	{ 0xb29a, 0xb29a, PDF_CMAP_SINGLE, 8864 },
	{ 0xb29b, 0xb2a0, PDF_CMAP_RANGE, 16288 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1128 },
	{ 0xb340, 0xb342, PDF_CMAP_RANGE, 16294 },
	{ 0xb343, 0xb343, PDF_CMAP_SINGLE, 8107 },
	{ 0xb344, 0xb36f, PDF_CMAP_RANGE, 16297 },
	{ 0xb370, 0xb370, PDF_CMAP_SINGLE, 9896 },
	{ 0xb371, 0xb37e, PDF_CMAP_RANGE, 16341 },
	{ 0xb380, 0xb387, PDF_CMAP_RANGE, 16355 },
	{ 0xb388, 0xb388, PDF_CMAP_SINGLE, 9449 },
	{ 0xb389, 0xb38b, PDF_CMAP_RANGE, 16363 },
	{ 0xb38c, 0xb38e, PDF_CMAP_TABLE, 480 },
	{ 0xb38f, 0xb3a0, PDF_CMAP_RANGE, 16367 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1222 },
	{ 0xb440, 0xb453, PDF_CMAP_RANGE, 16385 },
	{ 0xb454, 0xb454, PDF_CMAP_SINGLE, 8520 },
	{ 0xb455, 0xb457, PDF_CMAP_RANGE, 16405 },
	{ 0xb458, 0xb458, PDF_CMAP_SINGLE, 9444 },
	{ 0xb459, 0xb45d, PDF_CMAP_RANGE, 16408 },
	{ 0xb45e, 0xb461, PDF_CMAP_TABLE, 483 },
	{ 0xb462, 0xb474, PDF_CMAP_RANGE, 16414 },
	{ 0xb475, 0xb475, PDF_CMAP_SINGLE, 8871 },
	{ 0xb476, 0xb47d, PDF_CMAP_RANGE, 16433 },
	{ 0xb47e, 0xb47e, PDF_CMAP_SINGLE, 9452 },
	{ 0xb480, 0xb482, PDF_CMAP_RANGE, 16441 },
	{ 0xb483, 0xb483, PDF_CMAP_SINGLE, 9451 },
	{ 0xb484, 0xb488, PDF_CMAP_RANGE, 16444 },
	{ 0xb489, 0xb489, PDF_CMAP_SINGLE, 9443 },
	{ 0xb48a, 0xb492, PDF_CMAP_RANGE, 16449 },
	{ 0xb493, 0xb493, PDF_CMAP_SINGLE, 9450 },
	{ 0xb494, 0xb4a0, PDF_CMAP_RANGE, 16458 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1316 },
	{ 0xb540, 0xb541, PDF_CMAP_TABLE, 487 },
	{ 0xb542, 0xb54a, PDF_CMAP_RANGE, 16472 },
	{ 0xb54b, 0xb54b, PDF_CMAP_SINGLE, 7719 },
	{ 0xb54c, 0xb555, PDF_CMAP_RANGE, 16481 },
	{ 0xb556, 0xb556, PDF_CMAP_SINGLE, 8166 },
	{ 0xb557, 0xb559, PDF_CMAP_RANGE, 16491 },
	{ 0xb55a, 0xb55c, PDF_CMAP_TABLE, 489 },
	{ 0xb55d, 0xb560, PDF_CMAP_RANGE, 16494 },
	{ 0xb561, 0xb561, PDF_CMAP_SINGLE, 9448 },
	{ 0xb562, 0xb57e, PDF_CMAP_RANGE, 16498 },
	{ 0xb580, 0xb59b, PDF_CMAP_RANGE, 16527 },
	{ 0xb59c, 0xb59d, PDF_CMAP_TABLE, 492 },
	{ 0xb59e, 0xb5a0, PDF_CMAP_RANGE, 16555 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1410 },
	{ 0xb640, 0xb651, PDF_CMAP_RANGE, 16558 },
	{ 0xb652, 0xb652, PDF_CMAP_SINGLE, 9890 },
	{ 0xb653, 0xb654, PDF_CMAP_RANGE, 16576 },
	{ 0xb655, 0xb655, PDF_CMAP_SINGLE, 9437 },
	{ 0xb656, 0xb658, PDF_CMAP_RANGE, 16578 },
	{ 0xb659, 0xb65c, PDF_CMAP_TABLE, 494 },
	{ 0xb65d, 0xb67e, PDF_CMAP_RANGE, 16582 },
	{ 0xb680, 0xb6a0, PDF_CMAP_RANGE, 16616 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1504 },
	{ 0xb740, 0xb74d, PDF_CMAP_RANGE, 16649 },
	{ 0xb74e, 0xb74e, PDF_CMAP_SINGLE, 8853 },
	{ 0xb74f, 0xb750, PDF_CMAP_RANGE, 16663 },
	{ 0xb751, 0xb751, PDF_CMAP_SINGLE, 7808 },
	{ 0xb752, 0xb758, PDF_CMAP_RANGE, 16665 },
	{ 0xb759, 0xb759, PDF_CMAP_SINGLE, 9847 },
	{ 0xb75a, 0xb763, PDF_CMAP_RANGE, 16672 },
	{ 0xb764, 0xb766, PDF_CMAP_TABLE, 498 },
	{ 0xb767, 0xb776, PDF_CMAP_RANGE, 16682 },
	{ 0xb777, 0xb778, PDF_CMAP_TABLE, 501 },
	{ 0xb779, 0xb77e, PDF_CMAP_RANGE, 16698 },
	{ 0xb780, 0xb782, PDF_CMAP_TABLE, 503 },
	{ 0xb783, 0xb7a0, PDF_CMAP_RANGE, 16705 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1598 },
	{ 0xb840, 0xb842, PDF_CMAP_RANGE, 16735 },
	{ 0xb843, 0xb846, PDF_CMAP_TABLE, 506 },
	{ 0xb847, 0xb84c, PDF_CMAP_RANGE, 16739 },
	{ 0xb84d, 0xb84d, PDF_CMAP_SINGLE, 9657 },
	{ 0xb84e, 0xb850, PDF_CMAP_RANGE, 16745 },
	{ 0xb851, 0xb851, PDF_CMAP_SINGLE, 8170 },
	{ 0xb852, 0xb859, PDF_CMAP_RANGE, 16748 },
	{ 0xb85a, 0xb860, PDF_CMAP_TABLE, 510 },
	{ 0xb861, 0xb876, PDF_CMAP_RANGE, 16758 },
	{ 0xb877, 0xb877, PDF_CMAP_SINGLE, 8513 },
	{ 0xb878, 0xb87e, PDF_CMAP_RANGE, 16780 },
	{ 0xb880, 0xb881, PDF_CMAP_RANGE, 16787 },
	{ 0xb882, 0xb882, PDF_CMAP_SINGLE, 8135 },
	{ 0xb883, 0xb8a0, PDF_CMAP_RANGE, 16789 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1692 },
	{ 0xb940, 0xb94f, PDF_CMAP_RANGE, 16819 },
	{ 0xb950, 0xb950, PDF_CMAP_SINGLE, 7745 },
	{ 0xb951, 0xb960, PDF_CMAP_RANGE, 16835 },
	{ 0xb961, 0xb961, PDF_CMAP_SINGLE, 9700 },
	{ 0xb962, 0xb97a, PDF_CMAP_RANGE, 16851 },
	{ 0xb97b, 0xb97b, PDF_CMAP_SINGLE, 8070 },
	{ 0xb97c, 0xb97e, PDF_CMAP_RANGE, 16876 },
	{ 0xb980, 0xb99c, PDF_CMAP_RANGE, 16879 },
	{ 0xb99d, 0xb99d, PDF_CMAP_SINGLE, 8115 },
	{ 0xb99e, 0xb99f, PDF_CMAP_RANGE, 16908 },
	{ 0xb9a0, 0xb9a0, PDF_CMAP_SINGLE, 7919 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1786 },
	{ 0xba40, 0xba41, PDF_CMAP_RANGE, 16910 },
	{ 0xba42, 0xba44, PDF_CMAP_TABLE, 517 },
	{ 0xba45, 0xba55, PDF_CMAP_RANGE, 16913 },
	{ 0xba56, 0xba56, PDF_CMAP_SINGLE, 9699 },
	{ 0xba57, 0xba58, PDF_CMAP_RANGE, 16930 },
	{ 0xba59, 0xba59, PDF_CMAP_SINGLE, 8465 },
	{ 0xba5a, 0xba5f, PDF_CMAP_RANGE, 16932 },
	{ 0xba60, 0xba60, PDF_CMAP_SINGLE, 9702 },
	{ 0xba61, 0xba69, PDF_CMAP_RANGE, 16938 },
	{ 0xba6a, 0xba6a, PDF_CMAP_SINGLE, 9703 },
	{ 0xba6b, 0xba73, PDF_CMAP_RANGE, 16947 },
	{ 0xba74, 0xba74, PDF_CMAP_SINGLE, 8257 },
	{ 0xba75, 0xba7e, PDF_CMAP_RANGE, 16956 },
	{ 0xba80, 0xba83, PDF_CMAP_RANGE, 16966 },
	{ 0xba84, 0xba88, PDF_CMAP_TABLE, 520 },
	{ 0xba89, 0xba8c, PDF_CMAP_RANGE, 16972 },
	{ 0xba8d, 0xba8d, PDF_CMAP_SINGLE, 9707 },
	{ 0xba8e, 0xba9d, PDF_CMAP_RANGE, 16976 },
	{ 0xba9e, 0xbaa0, PDF_CMAP_TABLE, 525 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 1880 },
	{ 0xbb40, 0xbb40, PDF_CMAP_SINGLE, 8183 },
	{ 0xbb41, 0xbb48, PDF_CMAP_RANGE, 16993 },
	{ 0xbb49, 0xbb49, PDF_CMAP_SINGLE, 7821 },
	{ 0xbb4a, 0xbb57, PDF_CMAP_RANGE, 17001 },
	{ 0xbb58, 0xbb58, PDF_CMAP_SINGLE, 9705 },
	{ 0xbb59, 0xbb5a, PDF_CMAP_RANGE, 17015 },
	{ 0xbb5b, 0xbb5c, PDF_CMAP_TABLE, 528 },
	{ 0xbb5d, 0xbb5f, PDF_CMAP_RANGE, 17017 },
	{ 0xbb60, 0xbb60, PDF_CMAP_SINGLE, 9875 },
	{ 0xbb61, 0xbb64, PDF_CMAP_RANGE, 17020 },
	{ 0xbb65, 0xbb6a, PDF_CMAP_TABLE, 530 },
	{ 0xbb6b, 0xbb6d, PDF_CMAP_RANGE, 17026 },
	{ 0xbb6e, 0xbb6e, PDF_CMAP_SINGLE, 9891 },
	{ 0xbb6f, 0xbb7e, PDF_CMAP_RANGE, 17029 },
	{ 0xbb80, 0xbba0, PDF_CMAP_RANGE, 17045 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 1974 },
	{ 0xbc40, 0xbc51, PDF_CMAP_RANGE, 17078 },
	{ 0xbc52, 0xbc53, PDF_CMAP_TABLE, 536 },
	{ 0xbc54, 0xbc59, PDF_CMAP_RANGE, 17096 },
	{ 0xbc5a, 0xbc5a, PDF_CMAP_SINGLE, 8228 },
	{ 0xbc5b, 0xbc60, PDF_CMAP_RANGE, 17102 },
	{ 0xbc61, 0xbc69, PDF_CMAP_TABLE, 538 },
	{ 0xbc6a, 0xbc6c, PDF_CMAP_RANGE, 17112 },
	{ 0xbc6d, 0xbc75, PDF_CMAP_TABLE, 547 },
	{ 0xbc76, 0xbc77, PDF_CMAP_RANGE, 9246 },
	{ 0xbc78, 0xbc7b, PDF_CMAP_TABLE, 556 },
	{ 0xbc7c, 0xbc7d, PDF_CMAP_RANGE, 17119 },
	{ 0xbc7e, 0xbc7e, PDF_CMAP_SINGLE, 8359 },
	{ 0xbc80, 0xbc81, PDF_CMAP_RANGE, 17121 },
	{ 0xbc82, 0xbc8b, PDF_CMAP_TABLE, 560 },
	{ 0xbc8c, 0xbc8e, PDF_CMAP_RANGE, 17125 },
	{ 0xbc8f, 0xbc8f, PDF_CMAP_SINGLE, 7923 },
	{ 0xbc90, 0xbc99, PDF_CMAP_RANGE, 17128 },
	{ 0xbc9a, 0xbc9d, PDF_CMAP_TABLE, 570 },
	{ 0xbc9e, 0xbca0, PDF_CMAP_RANGE, 17138 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2068 },
	{ 0xbd40, 0xbd41, PDF_CMAP_RANGE, 17141 },
	{ 0xbd42, 0xbd45, PDF_CMAP_TABLE, 574 },
	{ 0xbd46, 0xbd47, PDF_CMAP_RANGE, 17144 },
	{ 0xbd48, 0xbd4f, PDF_CMAP_TABLE, 578 },
	{ 0xbd50, 0xbd56, PDF_CMAP_RANGE, 17149 },
	{ 0xbd57, 0xbd59, PDF_CMAP_TABLE, 586 },
	{ 0xbd5a, 0xbd65, PDF_CMAP_RANGE, 17157 },
	{ 0xbd66, 0xbd67, PDF_CMAP_TABLE, 589 },
	{ 0xbd68, 0xbd69, PDF_CMAP_RANGE, 17169 },
	{ 0xbd6a, 0xbd6b, PDF_CMAP_TABLE, 591 },
	{ 0xbd6c, 0xbd6e, PDF_CMAP_RANGE, 17171 },
	{ 0xbd6f, 0xbd71, PDF_CMAP_TABLE, 593 },
	{ 0xbd72, 0xbd78, PDF_CMAP_RANGE, 17175 },
	{ 0xbd79, 0xbd7b, PDF_CMAP_TABLE, 596 },
	{ 0xbd7c, 0xbd7d, PDF_CMAP_RANGE, 17182 },
	{ 0xbd7e, 0xbd7e, PDF_CMAP_SINGLE, 8148 },
	{ 0xbd80, 0xbd81, PDF_CMAP_TABLE, 599 },
	{ 0xbd82, 0xbd88, PDF_CMAP_RANGE, 17185 },
	{ 0xbd89, 0xbd8b, PDF_CMAP_TABLE, 601 },
	{ 0xbd8c, 0xbd8d, PDF_CMAP_RANGE, 17193 },
	{ 0xbd8e, 0xbd91, PDF_CMAP_TABLE, 604 },
	{ 0xbd92, 0xbd96, PDF_CMAP_RANGE, 17196 },
	{ 0xbd97, 0xbd97, PDF_CMAP_SINGLE, 8536 },
	{ 0xbd98, 0xbd9a, PDF_CMAP_RANGE, 17201 },
	{ 0xbd9b, 0xbd9b, PDF_CMAP_SINGLE, 8130 },
	{ 0xbd9c, 0xbda0, PDF_CMAP_RANGE, 17204 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2162 },
	{ 0xbe40, 0xbe42, PDF_CMAP_RANGE, 17209 },
	{ 0xbe43, 0xbe45, PDF_CMAP_TABLE, 608 },
	{ 0xbe46, 0xbe48, PDF_CMAP_RANGE, 17213 },
	{ 0xbe49, 0xbe4a, PDF_CMAP_TABLE, 611 },
	{ 0xbe4b, 0xbe50, PDF_CMAP_RANGE, 17216 },
	{ 0xbe51, 0xbe59, PDF_CMAP_TABLE, 613 },
	{ 0xbe5a, 0xbe5c, PDF_CMAP_RANGE, 17223 },
	{ 0xbe5d, 0xbe64, PDF_CMAP_TABLE, 622 },
	{ 0xbe65, 0xbe68, PDF_CMAP_RANGE, 17227 },
	{ 0xbe69, 0xbe69, PDF_CMAP_SINGLE, 9269 },
	{ 0xbe6a, 0xbe6b, PDF_CMAP_RANGE, 17231 },
	{ 0xbe6c, 0xbe6c, PDF_CMAP_SINGLE, 9275 },
	{ 0xbe6d, 0xbe6e, PDF_CMAP_RANGE, 17233 },
	{ 0xbe6f, 0xbe70, PDF_CMAP_TABLE, 630 },
	{ 0xbe71, 0xbe75, PDF_CMAP_RANGE, 17235 },
	{ 0xbe76, 0xbe79, PDF_CMAP_TABLE, 632 },
	{ 0xbe7a, 0xbe7b, PDF_CMAP_RANGE, 17241 },
	{ 0xbe7c, 0xbe7e, PDF_CMAP_TABLE, 636 },
	{ 0xbe80, 0xbe82, PDF_CMAP_RANGE, 17243 },
	{ 0xbe83, 0xbe89, PDF_CMAP_TABLE, 639 },
	{ 0xbe8a, 0xbe8b, PDF_CMAP_RANGE, 17248 },
	{ 0xbe8c, 0xbe8f, PDF_CMAP_TABLE, 646 },
	{ 0xbe90, 0xbe91, PDF_CMAP_RANGE, 17251 },
	{ 0xbe92, 0xbe92, PDF_CMAP_SINGLE, 8326 },
	{ 0xbe93, 0xbe94, PDF_CMAP_RANGE, 17253 },
	{ 0xbe95, 0xbe9c, PDF_CMAP_TABLE, 650 },
	{ 0xbe9d, 0xbe9e, PDF_CMAP_RANGE, 17258 },
	{ 0xbe9f, 0xbea0, PDF_CMAP_TABLE, 658 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2256 },
	{ 0xbf40, 0xbf40, PDF_CMAP_SINGLE, 9894 },
	{ 0xbf41, 0xbf4c, PDF_CMAP_RANGE, 17261 },
	{ 0xbf4d, 0xbf50, PDF_CMAP_TABLE, 660 },
	{ 0xbf51, 0xbf54, PDF_CMAP_RANGE, 17273 },
	{ 0xbf55, 0xbf56, PDF_CMAP_TABLE, 664 },
	{ 0xbf57, 0xbf5f, PDF_CMAP_RANGE, 17277 },
	{ 0xbf60, 0xbf64, PDF_CMAP_TABLE, 666 },
	{ 0xbf65, 0xbf67, PDF_CMAP_RANGE, 17287 },
	{ 0xbf68, 0xbf68, PDF_CMAP_SINGLE, 8642 },
	{ 0xbf69, 0xbf6b, PDF_CMAP_RANGE, 17290 },
	{ 0xbf6c, 0xbf6c, PDF_CMAP_SINGLE, 8558 },
	{ 0xbf6d, 0xbf6f, PDF_CMAP_RANGE, 17293 },
	{ 0xbf70, 0xbf73, PDF_CMAP_TABLE, 671 },
	{ 0xbf74, 0xbf75, PDF_CMAP_RANGE, 17297 },
	{ 0xbf76, 0xbf7e, PDF_CMAP_TABLE, 675 },
	{ 0xbf80, 0xbf81, PDF_CMAP_RANGE, 17301 },
	{ 0xbf82, 0xbf83, PDF_CMAP_TABLE, 684 },
	{ 0xbf84, 0xbf88, PDF_CMAP_RANGE, 17303 },
	{ 0xbf89, 0xbf8a, PDF_CMAP_TABLE, 686 },
	{ 0xbf8b, 0xbf94, PDF_CMAP_RANGE, 17308 },
	{ 0xbf95, 0xbf98, PDF_CMAP_TABLE, 688 },
	{ 0xbf99, 0xbf9c, PDF_CMAP_RANGE, 17319 },
	{ 0xbf9d, 0xbf9d, PDF_CMAP_SINGLE, 9300 },
	{ 0xbf9e, 0xbfa0, PDF_CMAP_RANGE, 17323 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2350 },
	{ 0xc040, 0xc040, PDF_CMAP_SINGLE, 8443 },
	{ 0xc041, 0xc043, PDF_CMAP_RANGE, 17326 },
	{ 0xc044, 0xc044, PDF_CMAP_SINGLE, 9280 },
	{ 0xc045, 0xc04a, PDF_CMAP_RANGE, 17329 },
	{ 0xc04b, 0xc052, PDF_CMAP_TABLE, 692 },
	{ 0xc053, 0xc054, PDF_CMAP_RANGE, 17336 },
	{ 0xc055, 0xc055, PDF_CMAP_SINGLE, 8110 },
	{ 0xc056, 0xc05a, PDF_CMAP_RANGE, 17338 },
	{ 0xc05b, 0xc05b, PDF_CMAP_SINGLE, 8730 },
	{ 0xc05c, 0xc05d, PDF_CMAP_RANGE, 17343 },
	{ 0xc05e, 0xc060, PDF_CMAP_TABLE, 700 },
	{ 0xc061, 0xc068, PDF_CMAP_RANGE, 17345 },
	{ 0xc069, 0xc070, PDF_CMAP_TABLE, 703 },
	{ 0xc071, 0xc073, PDF_CMAP_RANGE, 17356 },
	{ 0xc074, 0xc079, PDF_CMAP_TABLE, 711 },
	{ 0xc07a, 0xc07b, PDF_CMAP_RANGE, 17361 },
	{ 0xc07c, 0xc07c, PDF_CMAP_SINGLE, 8191 },
	{ 0xc07d, 0xc07e, PDF_CMAP_RANGE, 17363 },
	{ 0xc080, 0xc09a, PDF_CMAP_RANGE, 17365 },
	{ 0xc09b, 0xc09d, PDF_CMAP_TABLE, 717 },
	{ 0xc09e, 0xc0a0, PDF_CMAP_RANGE, 17393 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2444 },
	{ 0xc140, 0xc14f, PDF_CMAP_RANGE, 17396 },
	{ 0xc150, 0xc150, PDF_CMAP_SINGLE, 7914 },
	{ 0xc151, 0xc153, PDF_CMAP_RANGE, 17412 },
	{ 0xc154, 0xc154, PDF_CMAP_SINGLE, 7724 },
	{ 0xc155, 0xc15e, PDF_CMAP_RANGE, 17415 },
	{ 0xc15f, 0xc162, PDF_CMAP_TABLE, 720 },
	{ 0xc163, 0xc174, PDF_CMAP_RANGE, 17426 },
	{ 0xc175, 0xc175, PDF_CMAP_SINGLE, 9714 },
	{ 0xc176, 0xc177, PDF_CMAP_RANGE, 17444 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 8725 },
	{ 0xc179, 0xc17e, PDF_CMAP_RANGE, 17446 },
	{ 0xc180, 0xc194, PDF_CMAP_RANGE, 17452 },
	{ 0xc195, 0xc195, PDF_CMAP_SINGLE, 8622 },
	{ 0xc196, 0xc1a0, PDF_CMAP_RANGE, 17473 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2538 },
	{ 0xc240, 0xc24d, PDF_CMAP_RANGE, 17484 },
	{ 0xc24e, 0xc24e, PDF_CMAP_SINGLE, 8416 },
	{ 0xc24f, 0xc264, PDF_CMAP_RANGE, 17498 },
	{ 0xc265, 0xc267, PDF_CMAP_TABLE, 724 },
	{ 0xc268, 0xc27c, PDF_CMAP_RANGE, 17521 },
	{ 0xc27d, 0xc27e, PDF_CMAP_TABLE, 727 },
	{ 0xc280, 0xc283, PDF_CMAP_RANGE, 17543 },
	{ 0xc284, 0xc284, PDF_CMAP_SINGLE, 8601 },
	{ 0xc285, 0xc292, PDF_CMAP_RANGE, 17547 },
	{ 0xc293, 0xc29c, PDF_CMAP_TABLE, 729 },
	{ 0xc29d, 0xc29f, PDF_CMAP_RANGE, 17563 },
	{ 0xc2a0, 0xc2a0, PDF_CMAP_SINGLE, 8569 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2632 },
	{ 0xc340, 0xc340, PDF_CMAP_SINGLE, 8248 },
	{ 0xc341, 0xc342, PDF_CMAP_RANGE, 17566 },
	{ 0xc343, 0xc343, PDF_CMAP_SINGLE, 8533 },
	{ 0xc344, 0xc37a, PDF_CMAP_RANGE, 17568 },
	{ 0xc37b, 0xc37b, PDF_CMAP_SINGLE, 8658 },
	{ 0xc37c, 0xc37e, PDF_CMAP_RANGE, 17623 },
	{ 0xc380, 0xc383, PDF_CMAP_RANGE, 17626 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 9415 },
	{ 0xc385, 0xc39a, PDF_CMAP_RANGE, 17630 },
	{ 0xc39b, 0xc39b, PDF_CMAP_SINGLE, 8825 },
	{ 0xc39c, 0xc3a0, PDF_CMAP_RANGE, 17652 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2726 },
	{ 0xc440, 0xc448, PDF_CMAP_RANGE, 17657 },
	{ 0xc449, 0xc449, PDF_CMAP_SINGLE, 8483 },
	{ 0xc44a, 0xc44b, PDF_CMAP_RANGE, 17666 },
	{ 0xc44c, 0xc44c, PDF_CMAP_SINGLE, 9413 },
	{ 0xc44d, 0xc453, PDF_CMAP_RANGE, 17668 },
	{ 0xc454, 0xc454, PDF_CMAP_SINGLE, 9417 },
	{ 0xc455, 0xc457, PDF_CMAP_RANGE, 17675 },
	{ 0xc458, 0xc458, PDF_CMAP_SINGLE, 8340 },
	{ 0xc459, 0xc45a, PDF_CMAP_RANGE, 17678 },
	{ 0xc45b, 0xc45b, PDF_CMAP_SINGLE, 8854 },
	{ 0xc45c, 0xc462, PDF_CMAP_RANGE, 17680 },
	{ 0xc463, 0xc463, PDF_CMAP_SINGLE, 7799 },
	{ 0xc464, 0xc476, PDF_CMAP_RANGE, 17687 },
	{ 0xc477, 0xc477, PDF_CMAP_SINGLE, 7942 },
	{ 0xc478, 0xc479, PDF_CMAP_RANGE, 17706 },
	{ 0xc47a, 0xc47a, PDF_CMAP_SINGLE, 8101 },
	{ 0xc47b, 0xc47e, PDF_CMAP_RANGE, 17708 },
	{ 0xc480, 0xc481, PDF_CMAP_TABLE, 739 },
	{ 0xc482, 0xc490, PDF_CMAP_RANGE, 17713 },
	{ 0xc491, 0xc493, PDF_CMAP_TABLE, 741 },
	{ 0xc494, 0xc497, PDF_CMAP_RANGE, 17728 },
	{ 0xc498, 0xc49c, PDF_CMAP_TABLE, 744 },
	{ 0xc49d, 0xc4a0, PDF_CMAP_RANGE, 17734 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2820 },
	{ 0xc540, 0xc543, PDF_CMAP_RANGE, 17738 },
	{ 0xc544, 0xc546, PDF_CMAP_TABLE, 749 },
	{ 0xc547, 0xc54a, PDF_CMAP_RANGE, 17743 },
	{ 0xc54b, 0xc54c, PDF_CMAP_TABLE, 752 },
	{ 0xc54d, 0xc551, PDF_CMAP_RANGE, 17747 },
	{ 0xc552, 0xc552, PDF_CMAP_SINGLE, 8236 },
	{ 0xc553, 0xc55e, PDF_CMAP_RANGE, 17752 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 8545 },
	{ 0xc560, 0xc562, PDF_CMAP_RANGE, 17764 },
	{ 0xc563, 0xc566, PDF_CMAP_TABLE, 754 },
	{ 0xc567, 0xc57e, PDF_CMAP_RANGE, 17767 },
	{ 0xc580, 0xc592, PDF_CMAP_RANGE, 17791 },
	{ 0xc593, 0xc593, PDF_CMAP_SINGLE, 7777 },
	{ 0xc594, 0xc59b, PDF_CMAP_RANGE, 17810 },
	{ 0xc59c, 0xc59e, PDF_CMAP_TABLE, 758 },
	{ 0xc59f, 0xc5a0, PDF_CMAP_RANGE, 17819 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 2914 },
	{ 0xc640, 0xc641, PDF_CMAP_TABLE, 761 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 17822 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 8072 },
	{ 0xc645, 0xc646, PDF_CMAP_RANGE, 17824 },
	{ 0xc647, 0xc647, PDF_CMAP_SINGLE, 8696 },
	{ 0xc648, 0xc662, PDF_CMAP_RANGE, 17826 },
	{ 0xc663, 0xc663, PDF_CMAP_SINGLE, 8994 },
	{ 0xc664, 0xc671, PDF_CMAP_RANGE, 17853 },
	{ 0xc672, 0xc672, PDF_CMAP_SINGLE, 9014 },
	{ 0xc673, 0xc67e, PDF_CMAP_RANGE, 17867 },
	{ 0xc680, 0xc6a0, PDF_CMAP_RANGE, 17879 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3008 },
	{ 0xc740, 0xc765, PDF_CMAP_RANGE, 17912 },
	{ 0xc766, 0xc766, PDF_CMAP_SINGLE, 8875 },
	{ 0xc767, 0xc76e, PDF_CMAP_RANGE, 17950 },
	{ 0xc76f, 0xc76f, PDF_CMAP_SINGLE, 8127 },
	{ 0xc770, 0xc775, PDF_CMAP_RANGE, 17958 },
	{ 0xc776, 0xc776, PDF_CMAP_SINGLE, 8061 },
	{ 0xc777, 0xc77a, PDF_CMAP_RANGE, 17964 },
	{ 0xc77b, 0xc77b, PDF_CMAP_SINGLE, 9011 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 17968 },
	{ 0xc780, 0xc7a0, PDF_CMAP_RANGE, 17971 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3102 },
	{ 0xc840, 0xc841, PDF_CMAP_TABLE, 763 },
	{ 0xc842, 0xc84e, PDF_CMAP_RANGE, 18005 },
	{ 0xc84f, 0xc84f, PDF_CMAP_SINGLE, 9012 },
	{ 0xc850, 0xc851, PDF_CMAP_RANGE, 18018 },
	{ 0xc852, 0xc852, PDF_CMAP_SINGLE, 8177 },
	{ 0xc853, 0xc865, PDF_CMAP_RANGE, 18020 },
	{ 0xc866, 0xc866, PDF_CMAP_SINGLE, 8587 },
	{ 0xc867, 0xc86d, PDF_CMAP_RANGE, 18039 },
	{ 0xc86e, 0xc86e, PDF_CMAP_SINGLE, 9033 },
	{ 0xc86f, 0xc87d, PDF_CMAP_RANGE, 18046 },
	{ 0xc87e, 0xc87e, PDF_CMAP_SINGLE, 8715 },
	{ 0xc880, 0xc886, PDF_CMAP_RANGE, 18061 },
	{ 0xc887, 0xc887, PDF_CMAP_SINGLE, 9030 },
	{ 0xc888, 0xc891, PDF_CMAP_RANGE, 18068 },
	{ 0xc892, 0xc894, PDF_CMAP_TABLE, 765 },
	{ 0xc895, 0xc898, PDF_CMAP_RANGE, 18079 },
	{ 0xc899, 0xc899, PDF_CMAP_SINGLE, 9889 },
	{ 0xc89a, 0xc89c, PDF_CMAP_RANGE, 18083 },
	{ 0xc89d, 0xc89d, PDF_CMAP_SINGLE, 8034 },
	{ 0xc89e, 0xc8a0, PDF_CMAP_RANGE, 18086 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3196 },
	{ 0xc940, 0xc94e, PDF_CMAP_RANGE, 18089 },
	{ 0xc94f, 0xc950, PDF_CMAP_TABLE, 768 },
	{ 0xc951, 0xc96d, PDF_CMAP_RANGE, 18104 },
	{ 0xc96e, 0xc970, PDF_CMAP_TABLE, 770 },
	{ 0xc971, 0xc976, PDF_CMAP_RANGE, 18134 },
	{ 0xc977, 0xc977, PDF_CMAP_SINGLE, 7954 },
	{ 0xc978, 0xc97e, PDF_CMAP_RANGE, 18140 },
	{ 0xc980, 0xc98e, PDF_CMAP_RANGE, 18147 },
	{ 0xc98f, 0xc990, PDF_CMAP_TABLE, 773 },
	{ 0xc991, 0xc99b, PDF_CMAP_RANGE, 18162 },
	{ 0xc99c, 0xc99c, PDF_CMAP_SINGLE, 9020 },
	{ 0xc99d, 0xc9a0, PDF_CMAP_RANGE, 18173 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 3290 },
	{ 0xca40, 0xca4d, PDF_CMAP_RANGE, 18177 },
	{ 0xca4e, 0xca4e, PDF_CMAP_SINGLE, 9839 },
	{ 0xca4f, 0xca55, PDF_CMAP_RANGE, 18191 },
	{ 0xca56, 0xca56, PDF_CMAP_SINGLE, 9041 },
	{ 0xca57, 0xca58, PDF_CMAP_RANGE, 18198 },
	{ 0xca59, 0xca59, PDF_CMAP_SINGLE, 8096 },
	{ 0xca5a, 0xca5b, PDF_CMAP_RANGE, 18200 },
	{ 0xca5c, 0xca5c, PDF_CMAP_SINGLE, 9016 },
	{ 0xca5d, 0xca60, PDF_CMAP_RANGE, 18202 },
	{ 0xca61, 0xca61, PDF_CMAP_SINGLE, 8731 },
	{ 0xca62, 0xca6d, PDF_CMAP_RANGE, 18206 },
	{ 0xca6e, 0xca6e, PDF_CMAP_SINGLE, 9026 },
	{ 0xca6f, 0xca71, PDF_CMAP_RANGE, 18218 },
	{ 0xca72, 0xca72, PDF_CMAP_SINGLE, 9039 },
	{ 0xca73, 0xca76, PDF_CMAP_RANGE, 18221 },
	{ 0xca77, 0xca77, PDF_CMAP_SINGLE, 9021 },
	{ 0xca78, 0xca7a, PDF_CMAP_RANGE, 18225 },
	{ 0xca7b, 0xca7e, PDF_CMAP_TABLE, 775 },
	{ 0xca80, 0xca81, PDF_CMAP_TABLE, 779 },
	{ 0xca82, 0xca88, PDF_CMAP_RANGE, 18230 },
	{ 0xca89, 0xca89, PDF_CMAP_SINGLE, 9040 },
	{ 0xca8a, 0xca8d, PDF_CMAP_RANGE, 18237 },
	{ 0xca8e, 0xca8f, PDF_CMAP_TABLE, 781 },
	{ 0xca90, 0xca91, PDF_CMAP_RANGE, 18241 },
	{ 0xca92, 0xca92, PDF_CMAP_SINGLE, 8651 },
	{ 0xca93, 0xca99, PDF_CMAP_RANGE, 18243 },
	{ 0xca9a, 0xca9a, PDF_CMAP_SINGLE, 9045 },
	{ 0xca9b, 0xcaa0, PDF_CMAP_RANGE, 18250 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3384 },
	{ 0xcb40, 0xcb42, PDF_CMAP_RANGE, 18256 },
	{ 0xcb43, 0xcb47, PDF_CMAP_TABLE, 783 },
	{ 0xcb48, 0xcb4a, PDF_CMAP_RANGE, 18261 },
	{ 0xcb4b, 0xcb4b, PDF_CMAP_SINGLE, 9856 },
	{ 0xcb4c, 0xcb4d, PDF_CMAP_RANGE, 18264 },
	{ 0xcb4e, 0xcb4e, PDF_CMAP_SINGLE, 8410 },
	{ 0xcb4f, 0xcb56, PDF_CMAP_RANGE, 18266 },
	{ 0xcb57, 0xcb57, PDF_CMAP_SINGLE, 9034 },
	{ 0xcb58, 0xcb5c, PDF_CMAP_RANGE, 18274 },
	{ 0xcb5d, 0xcb5f, PDF_CMAP_TABLE, 788 },
	{ 0xcb60, 0xcb69, PDF_CMAP_RANGE, 18280 },
	{ 0xcb6a, 0xcb6a, PDF_CMAP_SINGLE, 9023 },
	{ 0xcb6b, 0xcb7a, PDF_CMAP_RANGE, 18290 },
	{ 0xcb7b, 0xcb7c, PDF_CMAP_TABLE, 791 },
	{ 0xcb7d, 0xcb7e, PDF_CMAP_RANGE, 18306 },
	{ 0xcb80, 0xcb86, PDF_CMAP_RANGE, 18308 },
	{ 0xcb87, 0xcb87, PDF_CMAP_SINGLE, 8722 },
	{ 0xcb88, 0xcb8d, PDF_CMAP_RANGE, 18315 },
	{ 0xcb8e, 0xcb8e, PDF_CMAP_SINGLE, 8711 },
	{ 0xcb8f, 0xcb91, PDF_CMAP_RANGE, 18321 },
	{ 0xcb92, 0xcb92, PDF_CMAP_SINGLE, 9049 },
	{ 0xcb93, 0xcb9b, PDF_CMAP_RANGE, 18324 },
	{ 0xcb9c, 0xcb9e, PDF_CMAP_TABLE, 793 },
	{ 0xcb9f, 0xcba0, PDF_CMAP_RANGE, 18334 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 3478 },
	{ 0xcc40, 0xcc41, PDF_CMAP_TABLE, 796 },
	{ 0xcc42, 0xcc48, PDF_CMAP_RANGE, 18336 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_TABLE, 798 },
	{ 0xcc4c, 0xcc4e, PDF_CMAP_RANGE, 18343 },
	{ 0xcc4f, 0xcc4f, PDF_CMAP_SINGLE, 8379 },
	{ 0xcc50, 0xcc5b, PDF_CMAP_RANGE, 18346 },
	{ 0xcc5c, 0xcc5c, PDF_CMAP_SINGLE, 9050 },
	{ 0xcc5d, 0xcc5f, PDF_CMAP_RANGE, 18358 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 9046 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 18361 },
	{ 0xcc64, 0xcc64, PDF_CMAP_SINGLE, 9015 },
	{ 0xcc65, 0xcc6c, PDF_CMAP_RANGE, 18364 },
	{ 0xcc6d, 0xcc6d, PDF_CMAP_SINGLE, 8185 },
	{ 0xcc6e, 0xcc78, PDF_CMAP_RANGE, 18372 },
	{ 0xcc79, 0xcc79, PDF_CMAP_SINGLE, 9043 },
	{ 0xcc7a, 0xcc7c, PDF_CMAP_RANGE, 18383 },
	{ 0xcc7d, 0xcc7e, PDF_CMAP_TABLE, 801 },
	{ 0xcc80, 0xcc8d, PDF_CMAP_RANGE, 18387 },
	{ 0xcc8e, 0xcc8e, PDF_CMAP_SINGLE, 7829 },
	{ 0xcc8f, 0xcc93, PDF_CMAP_RANGE, 18401 },
	{ 0xcc94, 0xcc96, PDF_CMAP_TABLE, 803 },
	{ 0xcc97, 0xcc9c, PDF_CMAP_RANGE, 18407 },
	{ 0xcc9d, 0xcc9d, PDF_CMAP_SINGLE, 8168 },
	{ 0xcc9e, 0xcca0, PDF_CMAP_RANGE, 18413 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3572 },
	{ 0xcd40, 0xcd7e, PDF_CMAP_RANGE, 18416 },
	{ 0xcd80, 0xcd8f, PDF_CMAP_RANGE, 18479 },
	{ 0xcd90, 0xcd90, PDF_CMAP_SINGLE, 9690 },
	{ 0xcd91, 0xcd97, PDF_CMAP_RANGE, 18495 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 9687 },
	{ 0xcd99, 0xcda0, PDF_CMAP_RANGE, 18502 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 3666 },
	{ 0xce40, 0xce66, PDF_CMAP_RANGE, 18510 },
	{ 0xce67, 0xce67, PDF_CMAP_SINGLE, 8494 },
	{ 0xce68, 0xce71, PDF_CMAP_RANGE, 18549 },
	{ 0xce72, 0xce72, PDF_CMAP_SINGLE, 8626 },
	{ 0xce73, 0xce7e, PDF_CMAP_RANGE, 18559 },
	{ 0xce80, 0xce81, PDF_CMAP_TABLE, 806 },
	{ 0xce82, 0xce86, PDF_CMAP_RANGE, 18572 },
	{ 0xce87, 0xce87, PDF_CMAP_SINGLE, 9692 },
	{ 0xce88, 0xce9a, PDF_CMAP_RANGE, 18577 },
	{ 0xce9b, 0xce9b, PDF_CMAP_SINGLE, 8300 },
	{ 0xce9c, 0xce9d, PDF_CMAP_RANGE, 18596 },
	{ 0xce9e, 0xce9e, PDF_CMAP_SINGLE, 8742 },
	{ 0xce9f, 0xcea0, PDF_CMAP_RANGE, 18598 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3760 },
	{ 0xcf40, 0xcf4d, PDF_CMAP_RANGE, 18600 },
	{ 0xcf4e, 0xcf4e, PDF_CMAP_SINGLE, 9696 },
	{ 0xcf4f, 0xcf54, PDF_CMAP_RANGE, 18614 },
	{ 0xcf55, 0xcf55, PDF_CMAP_SINGLE, 8827 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 18620 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 9694 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 18622 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 9697 },
	{ 0xcf5d, 0xcf6b, PDF_CMAP_RANGE, 18625 },
	{ 0xcf6c, 0xcf6c, PDF_CMAP_SINGLE, 9685 },
	{ 0xcf6d, 0xcf72, PDF_CMAP_RANGE, 18640 },
	{ 0xcf73, 0xcf75, PDF_CMAP_TABLE, 808 },
	{ 0xcf76, 0xcf77, PDF_CMAP_RANGE, 18647 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 7817 },
	{ 0xcf79, 0xcf7b, PDF_CMAP_RANGE, 18649 },
	{ 0xcf7c, 0xcf7c, PDF_CMAP_SINGLE, 9689 },
	{ 0xcf7d, 0xcf7e, PDF_CMAP_RANGE, 18652 },
	{ 0xcf80, 0xcf81, PDF_CMAP_TABLE, 811 },
	{ 0xcf82, 0xcf88, PDF_CMAP_RANGE, 18655 },
	{ 0xcf89, 0xcf8a, PDF_CMAP_TABLE, 813 },
	{ 0xcf8b, 0xcf92, PDF_CMAP_RANGE, 18662 },
	{ 0xcf93, 0xcf94, PDF_CMAP_TABLE, 815 },
	{ 0xcf95, 0xcf9d, PDF_CMAP_RANGE, 18670 },
	{ 0xcf9e, 0xcfa0, PDF_CMAP_TABLE, 817 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 3854 },
	{ 0xd040, 0xd04c, PDF_CMAP_RANGE, 18680 },
	{ 0xd04d, 0xd04d, PDF_CMAP_SINGLE, 7977 },
	{ 0xd04e, 0xd050, PDF_CMAP_RANGE, 18693 },
	{ 0xd051, 0xd051, PDF_CMAP_SINGLE, 7771 },
	{ 0xd052, 0xd054, PDF_CMAP_RANGE, 18696 },
	{ 0xd055, 0xd055, PDF_CMAP_SINGLE, 8310 },
	{ 0xd056, 0xd05b, PDF_CMAP_RANGE, 18699 },
	{ 0xd05c, 0xd05c, PDF_CMAP_SINGLE, 8855 },
	{ 0xd05d, 0xd05f, PDF_CMAP_RANGE, 18705 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 9872 },
	{ 0xd061, 0xd066, PDF_CMAP_RANGE, 18708 },
	{ 0xd067, 0xd067, PDF_CMAP_SINGLE, 8511 },
	{ 0xd068, 0xd06b, PDF_CMAP_RANGE, 18714 },
	{ 0xd06c, 0xd06e, PDF_CMAP_TABLE, 820 },
	{ 0xd06f, 0xd07c, PDF_CMAP_RANGE, 18719 },
	{ 0xd07d, 0xd07e, PDF_CMAP_TABLE, 823 },
	{ 0xd080, 0xd0a0, PDF_CMAP_RANGE, 18734 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 3948 },
	{ 0xd140, 0xd154, PDF_CMAP_RANGE, 18767 },
	{ 0xd155, 0xd155, PDF_CMAP_SINGLE, 9713 },
	{ 0xd156, 0xd158, PDF_CMAP_RANGE, 18788 },
	{ 0xd159, 0xd159, PDF_CMAP_SINGLE, 8204 },
	{ 0xd15a, 0xd160, PDF_CMAP_RANGE, 18791 },
	{ 0xd161, 0xd162, PDF_CMAP_TABLE, 825 },
	{ 0xd163, 0xd174, PDF_CMAP_RANGE, 18798 },
	{ 0xd175, 0xd175, PDF_CMAP_SINGLE, 9895 },
	{ 0xd176, 0xd17c, PDF_CMAP_RANGE, 18816 },
	{ 0xd17d, 0xd17e, PDF_CMAP_TABLE, 827 },
	{ 0xd180, 0xd19c, PDF_CMAP_RANGE, 18824 },
	{ 0xd19d, 0xd19e, PDF_CMAP_TABLE, 829 },
	{ 0xd19f, 0xd1a0, PDF_CMAP_RANGE, 18853 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 4042 },
	{ 0xd240, 0xd240, PDF_CMAP_SINGLE, 9662 },
	{ 0xd241, 0xd242, PDF_CMAP_RANGE, 18855 },
	{ 0xd243, 0xd243, PDF_CMAP_SINGLE, 8925 },
	{ 0xd244, 0xd24c, PDF_CMAP_RANGE, 18857 },
	{ 0xd24d, 0xd24d, PDF_CMAP_SINGLE, 9661 },
	{ 0xd24e, 0xd25b, PDF_CMAP_RANGE, 18866 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 7722 },
	{ 0xd25d, 0xd262, PDF_CMAP_RANGE, 18880 },
	{ 0xd263, 0xd264, PDF_CMAP_TABLE, 831 },
	{ 0xd265, 0xd267, PDF_CMAP_RANGE, 18886 },
	{ 0xd268, 0xd268, PDF_CMAP_SINGLE, 9663 },
	{ 0xd269, 0xd26c, PDF_CMAP_RANGE, 18889 },
	{ 0xd26d, 0xd26f, PDF_CMAP_TABLE, 833 },
	{ 0xd270, 0xd271, PDF_CMAP_RANGE, 18894 },
	{ 0xd272, 0xd272, PDF_CMAP_SINGLE, 7807 },
	{ 0xd273, 0xd274, PDF_CMAP_RANGE, 18896 },
	{ 0xd275, 0xd275, PDF_CMAP_SINGLE, 8621 },
	{ 0xd276, 0xd27e, PDF_CMAP_RANGE, 18898 },
	{ 0xd280, 0xd289, PDF_CMAP_RANGE, 18907 },
	{ 0xd28a, 0xd28a, PDF_CMAP_SINGLE, 8086 },
	{ 0xd28b, 0xd28d, PDF_CMAP_RANGE, 18917 },
	{ 0xd28e, 0xd28e, PDF_CMAP_SINGLE, 7986 },
	{ 0xd28f, 0xd291, PDF_CMAP_RANGE, 18920 },
	{ 0xd292, 0xd292, PDF_CMAP_SINGLE, 8324 },
	{ 0xd293, 0xd294, PDF_CMAP_RANGE, 18923 },
	{ 0xd295, 0xd297, PDF_CMAP_TABLE, 836 },
	{ 0xd298, 0xd29f, PDF_CMAP_RANGE, 18926 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 9402 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4136 },
	{ 0xd340, 0xd343, PDF_CMAP_RANGE, 18934 },
	{ 0xd344, 0xd344, PDF_CMAP_SINGLE, 9404 },
	{ 0xd345, 0xd347, PDF_CMAP_RANGE, 18938 },
	{ 0xd348, 0xd34a, PDF_CMAP_TABLE, 839 },
	{ 0xd34b, 0xd34c, PDF_CMAP_RANGE, 18942 },
	{ 0xd34d, 0xd34d, PDF_CMAP_SINGLE, 9405 },
	{ 0xd34e, 0xd34f, PDF_CMAP_RANGE, 18944 },
	{ 0xd350, 0xd350, PDF_CMAP_SINGLE, 9406 },
	{ 0xd351, 0xd354, PDF_CMAP_RANGE, 18946 },
	{ 0xd355, 0xd355, PDF_CMAP_SINGLE, 9407 },
	{ 0xd356, 0xd357, PDF_CMAP_RANGE, 18950 },
	{ 0xd358, 0xd358, PDF_CMAP_SINGLE, 8146 },
	{ 0xd359, 0xd35a, PDF_CMAP_RANGE, 18952 },
	{ 0xd35b, 0xd35e, PDF_CMAP_TABLE, 842 },
	{ 0xd35f, 0xd377, PDF_CMAP_RANGE, 18955 },
	{ 0xd378, 0xd37c, PDF_CMAP_TABLE, 846 },
	{ 0xd37d, 0xd37e, PDF_CMAP_RANGE, 18982 },
	{ 0xd380, 0xd384, PDF_CMAP_RANGE, 18984 },
	{ 0xd385, 0xd387, PDF_CMAP_TABLE, 851 },
	{ 0xd388, 0xd38a, PDF_CMAP_RANGE, 18989 },
	{ 0xd38b, 0xd393, PDF_CMAP_TABLE, 854 },
	{ 0xd394, 0xd395, PDF_CMAP_RANGE, 18996 },
	{ 0xd396, 0xd39b, PDF_CMAP_TABLE, 863 },
	{ 0xd39c, 0xd39d, PDF_CMAP_RANGE, 19000 },
	{ 0xd39e, 0xd3a0, PDF_CMAP_TABLE, 869 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4230 },
	{ 0xd440, 0xd441, PDF_CMAP_TABLE, 872 },
	{ 0xd442, 0xd444, PDF_CMAP_RANGE, 19004 },
	{ 0xd445, 0xd447, PDF_CMAP_TABLE, 874 },
	{ 0xd448, 0xd44b, PDF_CMAP_RANGE, 19008 },
	{ 0xd44c, 0xd44c, PDF_CMAP_SINGLE, 7922 },
	{ 0xd44d, 0xd44e, PDF_CMAP_RANGE, 19012 },
	{ 0xd44f, 0xd44f, PDF_CMAP_SINGLE, 8479 },
	{ 0xd450, 0xd452, PDF_CMAP_RANGE, 19014 },
	{ 0xd453, 0xd453, PDF_CMAP_SINGLE, 8669 },
	{ 0xd454, 0xd455, PDF_CMAP_RANGE, 19017 },
	{ 0xd456, 0xd458, PDF_CMAP_TABLE, 877 },
	{ 0xd459, 0xd45b, PDF_CMAP_RANGE, 19020 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 8834 },
	{ 0xd45d, 0xd461, PDF_CMAP_RANGE, 19023 },
	{ 0xd462, 0xd462, PDF_CMAP_SINGLE, 8934 },
	{ 0xd463, 0xd466, PDF_CMAP_RANGE, 19028 },
	{ 0xd467, 0xd467, PDF_CMAP_SINGLE, 8936 },
	{ 0xd468, 0xd46d, PDF_CMAP_RANGE, 19032 },
	{ 0xd46e, 0xd475, PDF_CMAP_TABLE, 880 },
	{ 0xd476, 0xd477, PDF_CMAP_RANGE, 19041 },
	{ 0xd478, 0xd478, PDF_CMAP_SINGLE, 8938 },
	{ 0xd479, 0xd47a, PDF_CMAP_RANGE, 19043 },
	{ 0xd47b, 0xd47b, PDF_CMAP_SINGLE, 8893 },
	{ 0xd47c, 0xd47d, PDF_CMAP_RANGE, 19045 },
	{ 0xd47e, 0xd47e, PDF_CMAP_SINGLE, 7838 },
	{ 0xd480, 0xd481, PDF_CMAP_RANGE, 19047 },
	{ 0xd482, 0xd484, PDF_CMAP_TABLE, 888 },
	{ 0xd485, 0xd486, PDF_CMAP_RANGE, 19049 },
	{ 0xd487, 0xd487, PDF_CMAP_SINGLE, 8503 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 19051 },
	{ 0xd48a, 0xd496, PDF_CMAP_TABLE, 891 },
	{ 0xd497, 0xd49b, PDF_CMAP_RANGE, 19056 },
	{ 0xd49c, 0xd49c, PDF_CMAP_SINGLE, 8944 },
	{ 0xd49d, 0xd49e, PDF_CMAP_RANGE, 19061 },
	{ 0xd49f, 0xd4a0, PDF_CMAP_TABLE, 904 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 4324 },
	{ 0xd540, 0xd542, PDF_CMAP_RANGE, 19064 },
	{ 0xd543, 0xd546, PDF_CMAP_TABLE, 906 },
	{ 0xd547, 0xd549, PDF_CMAP_RANGE, 19067 },
	{ 0xd54a, 0xd54a, PDF_CMAP_SINGLE, 8446 },
	{ 0xd54b, 0xd54d, PDF_CMAP_RANGE, 19070 },
	{ 0xd54e, 0xd54f, PDF_CMAP_RANGE, 8953 },
	{ 0xd550, 0xd551, PDF_CMAP_TABLE, 910 },
	{ 0xd552, 0xd553, PDF_CMAP_RANGE, 19074 },
	{ 0xd554, 0xd556, PDF_CMAP_TABLE, 912 },
	{ 0xd557, 0xd559, PDF_CMAP_RANGE, 19077 },
	{ 0xd55a, 0xd564, PDF_CMAP_TABLE, 915 },
	{ 0xd565, 0xd567, PDF_CMAP_RANGE, 19083 },
	{ 0xd568, 0xd568, PDF_CMAP_SINGLE, 8519 },
	{ 0xd569, 0xd56b, PDF_CMAP_RANGE, 19086 },
	{ 0xd56c, 0xd56e, PDF_CMAP_TABLE, 926 },
	{ 0xd56f, 0xd571, PDF_CMAP_RANGE, 19090 },
	{ 0xd572, 0xd572, PDF_CMAP_SINGLE, 8961 },
	{ 0xd573, 0xd574, PDF_CMAP_RANGE, 19093 },
	{ 0xd575, 0xd575, PDF_CMAP_SINGLE, 7925 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 19095 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 8728 },
	{ 0xd579, 0xd57a, PDF_CMAP_RANGE, 19097 },
	{ 0xd57b, 0xd57b, PDF_CMAP_SINGLE, 7878 },
	{ 0xd57c, 0xd57d, PDF_CMAP_RANGE, 19099 },
	{ 0xd57e, 0xd57e, PDF_CMAP_SINGLE, 8960 },
	{ 0xd580, 0xd581, PDF_CMAP_TABLE, 929 },
	{ 0xd582, 0xd583, PDF_CMAP_RANGE, 19102 },
	{ 0xd584, 0xd58f, PDF_CMAP_TABLE, 931 },
	{ 0xd590, 0xd592, PDF_CMAP_RANGE, 19109 },
	{ 0xd593, 0xd594, PDF_CMAP_TABLE, 943 },
	{ 0xd595, 0xd597, PDF_CMAP_RANGE, 19112 },
	{ 0xd598, 0xd59b, PDF_CMAP_TABLE, 945 },
	{ 0xd59c, 0xd59e, PDF_CMAP_RANGE, 19116 },
	{ 0xd59f, 0xd5a0, PDF_CMAP_TABLE, 949 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 4418 },
	{ 0xd640, 0xd643, PDF_CMAP_TABLE, 951 },
	{ 0xd644, 0xd646, PDF_CMAP_RANGE, 19121 },
	{ 0xd647, 0xd64a, PDF_CMAP_TABLE, 955 },
	{ 0xd64b, 0xd64c, PDF_CMAP_RANGE, 19125 },
	{ 0xd64d, 0xd64f, PDF_CMAP_TABLE, 959 },
	{ 0xd650, 0xd651, PDF_CMAP_RANGE, 19128 },
	{ 0xd652, 0xd661, PDF_CMAP_TABLE, 962 },
	{ 0xd662, 0xd664, PDF_CMAP_RANGE, 19135 },
	{ 0xd665, 0xd665, PDF_CMAP_SINGLE, 8023 },
	{ 0xd666, 0xd668, PDF_CMAP_RANGE, 19138 },
	{ 0xd669, 0xd66b, PDF_CMAP_TABLE, 978 },
	{ 0xd66c, 0xd66e, PDF_CMAP_RANGE, 19142 },
	{ 0xd66f, 0xd678, PDF_CMAP_TABLE, 981 },
	{ 0xd679, 0xd67b, PDF_CMAP_RANGE, 19148 },
	{ 0xd67c, 0xd67c, PDF_CMAP_SINGLE, 8710 },
	{ 0xd67d, 0xd67e, PDF_CMAP_RANGE, 19151 },
	{ 0xd680, 0xd682, PDF_CMAP_RANGE, 19153 },
	{ 0xd683, 0xd683, PDF_CMAP_SINGLE, 8973 },
	{ 0xd684, 0xd685, PDF_CMAP_RANGE, 19156 },
	{ 0xd686, 0xd688, PDF_CMAP_TABLE, 991 },
	{ 0xd689, 0xd68d, PDF_CMAP_RANGE, 19158 },
	{ 0xd68e, 0xd68e, PDF_CMAP_SINGLE, 8931 },
	{ 0xd68f, 0xd693, PDF_CMAP_RANGE, 19163 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 8122 },
	{ 0xd695, 0xd698, PDF_CMAP_RANGE, 19168 },
	{ 0xd699, 0xd699, PDF_CMAP_SINGLE, 8312 },
	{ 0xd69a, 0xd6a0, PDF_CMAP_RANGE, 19172 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd740, 0xd742, PDF_CMAP_RANGE, 19179 },
	{ 0xd743, 0xd743, PDF_CMAP_SINGLE, 8840 },
	{ 0xd744, 0xd747, PDF_CMAP_RANGE, 19182 },
	{ 0xd748, 0xd749, PDF_CMAP_TABLE, 994 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 19186 },
	{ 0xd750, 0xd756, PDF_CMAP_TABLE, 996 },
	{ 0xd757, 0xd763, PDF_CMAP_RANGE, 19194 },
	{ 0xd764, 0xd764, PDF_CMAP_SINGLE, 8984 },
	{ 0xd765, 0xd766, PDF_CMAP_RANGE, 19207 },
	{ 0xd767, 0xd768, PDF_CMAP_TABLE, 1003 },
	{ 0xd769, 0xd76b, PDF_CMAP_RANGE, 19209 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 8405 },
	{ 0xd76d, 0xd76e, PDF_CMAP_RANGE, 19212 },
	{ 0xd76f, 0xd76f, PDF_CMAP_SINGLE, 8010 },
	{ 0xd770, 0xd774, PDF_CMAP_RANGE, 19214 },
	{ 0xd775, 0xd775, PDF_CMAP_SINGLE, 8768 },
	{ 0xd776, 0xd777, PDF_CMAP_RANGE, 19219 },
	{ 0xd778, 0xd778, PDF_CMAP_SINGLE, 7891 },
	{ 0xd779, 0xd77e, PDF_CMAP_RANGE, 19221 },
	{ 0xd780, 0xd782, PDF_CMAP_RANGE, 19227 },
	{ 0xd783, 0xd783, PDF_CMAP_SINGLE, 7753 },
	{ 0xd784, 0xd786, PDF_CMAP_RANGE, 19230 },
	{ 0xd787, 0xd787, PDF_CMAP_SINGLE, 9755 },
	{ 0xd788, 0xd78a, PDF_CMAP_RANGE, 19233 },
	{ 0xd78b, 0xd78f, PDF_CMAP_TABLE, 1005 },
	{ 0xd790, 0xd794, PDF_CMAP_RANGE, 19237 },
	{ 0xd795, 0xd797, PDF_CMAP_TABLE, 1010 },
	{ 0xd798, 0xd7a0, PDF_CMAP_RANGE, 19243 },
	{ 0xd7a1, 0xd7f9, PDF_CMAP_RANGE, 4606 },
	{ 0xd840, 0xd84c, PDF_CMAP_RANGE, 19252 },
	{ 0xd84d, 0xd84d, PDF_CMAP_SINGLE, 8392 },
	{ 0xd84e, 0xd852, PDF_CMAP_RANGE, 19265 },
	{ 0xd853, 0xd853, PDF_CMAP_SINGLE, 7933 },
	{ 0xd854, 0xd87e, PDF_CMAP_RANGE, 19270 },
	{ 0xd880, 0xd88f, PDF_CMAP_RANGE, 19313 },
	{ 0xd890, 0xd895, PDF_CMAP_TABLE, 1013 },
	{ 0xd896, 0xd899, PDF_CMAP_RANGE, 19330 },
	{ 0xd89a, 0xd8a0, PDF_CMAP_TABLE, 1019 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4695 },
	{ 0xd940, 0xd95c, PDF_CMAP_TABLE, 1026 },
	{ 0xd95d, 0xd962, PDF_CMAP_RANGE, 19342 },
	{ 0xd963, 0xd967, PDF_CMAP_TABLE, 1055 },
	{ 0xd968, 0xd96b, PDF_CMAP_RANGE, 19349 },
	{ 0xd96c, 0xd979, PDF_CMAP_TABLE, 1060 },
	{ 0xd97a, 0xd97b, PDF_CMAP_RANGE, 19357 },
	{ 0xd97c, 0xd97e, PDF_CMAP_TABLE, 1074 },
	{ 0xd980, 0xd980, PDF_CMAP_SINGLE, 7892 },
	{ 0xd981, 0xd986, PDF_CMAP_RANGE, 19359 },
	{ 0xd987, 0xd987, PDF_CMAP_SINGLE, 8179 },
	{ 0xd988, 0xd98c, PDF_CMAP_RANGE, 19365 },
	{ 0xd98d, 0xd991, PDF_CMAP_TABLE, 1077 },
	{ 0xd992, 0xd996, PDF_CMAP_RANGE, 19370 },
	{ 0xd997, 0xd998, PDF_CMAP_TABLE, 1082 },
	{ 0xd999, 0xd99a, PDF_CMAP_RANGE, 19375 },
	{ 0xd99b, 0xd9a0, PDF_CMAP_TABLE, 1084 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4789 },
	{ 0xda40, 0xda42, PDF_CMAP_TABLE, 1090 },
	{ 0xda43, 0xda47, PDF_CMAP_RANGE, 19380 },
	{ 0xda48, 0xda48, PDF_CMAP_SINGLE, 8509 },
	{ 0xda49, 0xda4c, PDF_CMAP_RANGE, 19385 },
	{ 0xda4d, 0xda4e, PDF_CMAP_TABLE, 1093 },
	{ 0xda4f, 0xda72, PDF_CMAP_RANGE, 19389 },
	{ 0xda73, 0xda73, PDF_CMAP_SINGLE, 7956 },
	{ 0xda74, 0xda76, PDF_CMAP_RANGE, 19425 },
	{ 0xda77, 0xda77, PDF_CMAP_SINGLE, 8826 },
	{ 0xda78, 0xda7e, PDF_CMAP_RANGE, 19428 },
	{ 0xda80, 0xda84, PDF_CMAP_RANGE, 19435 },
	{ 0xda85, 0xda85, PDF_CMAP_SINGLE, 8430 },
	{ 0xda86, 0xda8d, PDF_CMAP_RANGE, 19440 },
	{ 0xda8e, 0xda8e, PDF_CMAP_SINGLE, 9720 },
	{ 0xda8f, 0xdaa0, PDF_CMAP_RANGE, 19448 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4883 },
	{ 0xdb40, 0xdb5f, PDF_CMAP_RANGE, 19466 },
	{ 0xdb60, 0xdb60, PDF_CMAP_SINGLE, 8084 },
	{ 0xdb61, 0xdb77, PDF_CMAP_RANGE, 19498 },
	{ 0xdb78, 0xdb78, PDF_CMAP_SINGLE, 8752 },
	{ 0xdb79, 0xdb7e, PDF_CMAP_RANGE, 19521 },
	{ 0xdb80, 0xdb83, PDF_CMAP_RANGE, 19527 },
	{ 0xdb84, 0xdb84, PDF_CMAP_SINGLE, 9725 },
	{ 0xdb85, 0xdb8a, PDF_CMAP_RANGE, 19531 },
	{ 0xdb8b, 0xdb8b, PDF_CMAP_SINGLE, 9728 },
	{ 0xdb8c, 0xdb97, PDF_CMAP_RANGE, 19537 },
	{ 0xdb98, 0xdb98, PDF_CMAP_SINGLE, 9734 },
	{ 0xdb99, 0xdba0, PDF_CMAP_RANGE, 19549 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 4977 },
	{ 0xdc40, 0xdc44, PDF_CMAP_RANGE, 19557 },
	{ 0xdc45, 0xdc45, PDF_CMAP_SINGLE, 9727 },
	{ 0xdc46, 0xdc4e, PDF_CMAP_RANGE, 19562 },
	{ 0xdc4f, 0xdc57, PDF_CMAP_TABLE, 1095 },
	{ 0xdc58, 0xdc5c, PDF_CMAP_RANGE, 19573 },
	{ 0xdc5d, 0xdc5d, PDF_CMAP_SINGLE, 9729 },
	{ 0xdc5e, 0xdc61, PDF_CMAP_RANGE, 19578 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 9733 },
	{ 0xdc63, 0xdc65, PDF_CMAP_RANGE, 19582 },
	{ 0xdc66, 0xdc67, PDF_CMAP_TABLE, 1104 },
	{ 0xdc68, 0xdc6a, PDF_CMAP_RANGE, 19585 },
	{ 0xdc6b, 0xdc6b, PDF_CMAP_SINGLE, 9735 },
	{ 0xdc6c, 0xdc7b, PDF_CMAP_RANGE, 19588 },
	{ 0xdc7c, 0xdc7c, PDF_CMAP_SINGLE, 8432 },
	{ 0xdc7d, 0xdc7e, PDF_CMAP_RANGE, 19604 },
	{ 0xdc80, 0xdc86, PDF_CMAP_RANGE, 19606 },
	{ 0xdc87, 0xdc8a, PDF_CMAP_TABLE, 1106 },
	{ 0xdc8b, 0xdc8d, PDF_CMAP_RANGE, 19613 },
	{ 0xdc8e, 0xdc90, PDF_CMAP_TABLE, 1110 },
	{ 0xdc91, 0xdc96, PDF_CMAP_RANGE, 19617 },
	{ 0xdc97, 0xdc97, PDF_CMAP_SINGLE, 9357 },
	{ 0xdc98, 0xdc9a, PDF_CMAP_RANGE, 19623 },
	{ 0xdc9b, 0xdc9b, PDF_CMAP_SINGLE, 8450 },
	{ 0xdc9c, 0xdc9f, PDF_CMAP_RANGE, 19626 },
	{ 0xdca0, 0xdca0, PDF_CMAP_SINGLE, 9364 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5071 },
	{ 0xdd40, 0xdd45, PDF_CMAP_RANGE, 19630 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 9363 },
	{ 0xdd47, 0xdd4c, PDF_CMAP_RANGE, 19636 },
	{ 0xdd4d, 0xdd4d, PDF_CMAP_SINGLE, 9358 },
	{ 0xdd4e, 0xdd52, PDF_CMAP_RANGE, 19642 },
	{ 0xdd53, 0xdd59, PDF_CMAP_TABLE, 1113 },
	{ 0xdd5a, 0xdd5d, PDF_CMAP_RANGE, 19648 },
	{ 0xdd5e, 0xdd65, PDF_CMAP_TABLE, 1120 },
	{ 0xdd66, 0xdd6c, PDF_CMAP_RANGE, 19655 },
	{ 0xdd6d, 0xdd70, PDF_CMAP_TABLE, 1128 },
	{ 0xdd71, 0xdd75, PDF_CMAP_RANGE, 19663 },
	{ 0xdd76, 0xdd78, PDF_CMAP_TABLE, 1132 },
	{ 0xdd79, 0xdd7a, PDF_CMAP_RANGE, 9373 },
	{ 0xdd7b, 0xdd7e, PDF_CMAP_RANGE, 19668 },
	{ 0xdd80, 0xdd82, PDF_CMAP_TABLE, 1135 },
	{ 0xdd83, 0xdd84, PDF_CMAP_RANGE, 19673 },
	{ 0xdd85, 0xdd86, PDF_CMAP_TABLE, 1138 },
	{ 0xdd87, 0xdd8a, PDF_CMAP_RANGE, 19675 },
	{ 0xdd8b, 0xdd8b, PDF_CMAP_SINGLE, 8048 },
	{ 0xdd8c, 0xdd8e, PDF_CMAP_RANGE, 19679 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 9376 },
	{ 0xdd90, 0xdd93, PDF_CMAP_RANGE, 19682 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 8507 },
	{ 0xdd95, 0xdd96, PDF_CMAP_RANGE, 19686 },
	{ 0xdd97, 0xdd97, PDF_CMAP_SINGLE, 7943 },
	{ 0xdd98, 0xdd99, PDF_CMAP_RANGE, 19688 },
	{ 0xdd9a, 0xdd9b, PDF_CMAP_TABLE, 1140 },
	{ 0xdd9c, 0xdd9d, PDF_CMAP_RANGE, 19690 },
	{ 0xdd9e, 0xdda0, PDF_CMAP_TABLE, 1142 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xde40, 0xde41, PDF_CMAP_TABLE, 1145 },
	{ 0xde42, 0xde43, PDF_CMAP_RANGE, 19693 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 8872 },
	{ 0xde45, 0xde47, PDF_CMAP_RANGE, 19695 },
	{ 0xde48, 0xde49, PDF_CMAP_TABLE, 1147 },
	{ 0xde4a, 0xde4e, PDF_CMAP_RANGE, 19698 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 9378 },
	{ 0xde50, 0xde59, PDF_CMAP_RANGE, 19703 },
	{ 0xde5a, 0xde5f, PDF_CMAP_TABLE, 1149 },
	{ 0xde60, 0xde6a, PDF_CMAP_RANGE, 19715 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 7728 },
	{ 0xde6c, 0xde6e, PDF_CMAP_RANGE, 19726 },
	{ 0xde6f, 0xde72, PDF_CMAP_TABLE, 1155 },
	{ 0xde73, 0xde7e, PDF_CMAP_RANGE, 19729 },
	{ 0xde80, 0xde91, PDF_CMAP_RANGE, 19741 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 9851 },
	{ 0xde93, 0xde9e, PDF_CMAP_RANGE, 19759 },
	{ 0xde9f, 0xdea0, PDF_CMAP_TABLE, 1159 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdf40, 0xdf42, PDF_CMAP_TABLE, 1161 },
	{ 0xdf43, 0xdf4c, PDF_CMAP_RANGE, 19773 },
	{ 0xdf4d, 0xdf4d, PDF_CMAP_SINGLE, 8123 },
	{ 0xdf4e, 0xdf5b, PDF_CMAP_RANGE, 19783 },
	{ 0xdf5c, 0xdf60, PDF_CMAP_TABLE, 1164 },
	{ 0xdf61, 0xdf63, PDF_CMAP_RANGE, 19798 },
	{ 0xdf64, 0xdf68, PDF_CMAP_TABLE, 1169 },
	{ 0xdf69, 0xdf6c, PDF_CMAP_RANGE, 19803 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 8499 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_RANGE, 19807 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 7812 },
	{ 0xdf75, 0xdf76, PDF_CMAP_RANGE, 19813 },
	{ 0xdf77, 0xdf7e, PDF_CMAP_TABLE, 1174 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 8021 },
	{ 0xdf81, 0xdf82, PDF_CMAP_RANGE, 19818 },
	{ 0xdf83, 0xdf85, PDF_CMAP_TABLE, 1182 },
	{ 0xdf86, 0xdf88, PDF_CMAP_RANGE, 19821 },
	{ 0xdf89, 0xdf8a, PDF_CMAP_TABLE, 1185 },
	{ 0xdf8b, 0xdfa0, PDF_CMAP_RANGE, 19824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe040, 0xe04f, PDF_CMAP_RANGE, 19846 },
	{ 0xe050, 0xe050, PDF_CMAP_SINGLE, 8990 },
	{ 0xe051, 0xe05c, PDF_CMAP_RANGE, 19862 },
	{ 0xe05d, 0xe05d, PDF_CMAP_SINGLE, 8755 },
	{ 0xe05e, 0xe068, PDF_CMAP_RANGE, 19874 },
	{ 0xe069, 0xe069, PDF_CMAP_SINGLE, 8992 },
	{ 0xe06a, 0xe06b, PDF_CMAP_RANGE, 19885 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 8647 },
	{ 0xe06d, 0xe074, PDF_CMAP_RANGE, 19887 },
	{ 0xe075, 0xe079, PDF_CMAP_TABLE, 1187 },
	{ 0xe07a, 0xe07e, PDF_CMAP_RANGE, 19897 },
	{ 0xe080, 0xe086, PDF_CMAP_RANGE, 19902 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 7867 },
	{ 0xe088, 0xe08c, PDF_CMAP_RANGE, 19909 },
	{ 0xe08d, 0xe094, PDF_CMAP_TABLE, 1192 },
	{ 0xe095, 0xe096, PDF_CMAP_RANGE, 19917 },
	{ 0xe097, 0xe097, PDF_CMAP_SINGLE, 8987 },
	{ 0xe098, 0xe0a0, PDF_CMAP_RANGE, 19919 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe140, 0xe141, PDF_CMAP_RANGE, 19928 },
	{ 0xe142, 0xe142, PDF_CMAP_SINGLE, 8993 },
	{ 0xe143, 0xe163, PDF_CMAP_RANGE, 19930 },
	{ 0xe164, 0xe164, PDF_CMAP_SINGLE, 8789 },
	{ 0xe165, 0xe167, PDF_CMAP_RANGE, 19963 },
	{ 0xe168, 0xe168, PDF_CMAP_SINGLE, 7823 },
	{ 0xe169, 0xe173, PDF_CMAP_RANGE, 19966 },
	{ 0xe174, 0xe175, PDF_CMAP_TABLE, 1200 },
	{ 0xe176, 0xe17e, PDF_CMAP_RANGE, 19977 },
	{ 0xe180, 0xe183, PDF_CMAP_RANGE, 19986 },
	{ 0xe184, 0xe189, PDF_CMAP_TABLE, 1202 },
	{ 0xe18a, 0xe18b, PDF_CMAP_RANGE, 19992 },
	{ 0xe18c, 0xe18d, PDF_CMAP_TABLE, 1208 },
	{ 0xe18e, 0xe190, PDF_CMAP_RANGE, 9459 },
	{ 0xe191, 0xe195, PDF_CMAP_TABLE, 1210 },
	{ 0xe196, 0xe197, PDF_CMAP_RANGE, 19996 },
	{ 0xe198, 0xe198, PDF_CMAP_SINGLE, 8832 },
	{ 0xe199, 0xe19d, PDF_CMAP_RANGE, 19998 },
	{ 0xe19e, 0xe1a0, PDF_CMAP_TABLE, 1215 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe240, 0xe243, PDF_CMAP_TABLE, 1218 },
	{ 0xe244, 0xe24e, PDF_CMAP_RANGE, 20006 },
	{ 0xe24f, 0xe254, PDF_CMAP_TABLE, 1222 },
	{ 0xe255, 0xe259, PDF_CMAP_RANGE, 20019 },
	{ 0xe25a, 0xe25b, PDF_CMAP_TABLE, 1228 },
	{ 0xe25c, 0xe25d, PDF_CMAP_RANGE, 20024 },
	{ 0xe25e, 0xe25e, PDF_CMAP_SINGLE, 9478 },
	{ 0xe25f, 0xe261, PDF_CMAP_RANGE, 20026 },
	{ 0xe262, 0xe263, PDF_CMAP_TABLE, 1230 },
	{ 0xe264, 0xe266, PDF_CMAP_RANGE, 20029 },
	{ 0xe267, 0xe26b, PDF_CMAP_TABLE, 1232 },
	{ 0xe26c, 0xe26d, PDF_CMAP_RANGE, 20033 },
	{ 0xe26e, 0xe26f, PDF_CMAP_TABLE, 1237 },
	{ 0xe270, 0xe277, PDF_CMAP_RANGE, 20035 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 8149 },
	{ 0xe279, 0xe27c, PDF_CMAP_RANGE, 20043 },
	{ 0xe27d, 0xe27e, PDF_CMAP_TABLE, 1239 },
	{ 0xe280, 0xe282, PDF_CMAP_TABLE, 1241 },
	{ 0xe283, 0xe288, PDF_CMAP_RANGE, 20048 },
	{ 0xe289, 0xe28b, PDF_CMAP_TABLE, 1244 },
	{ 0xe28c, 0xe28d, PDF_CMAP_RANGE, 20055 },
	{ 0xe28e, 0xe28f, PDF_CMAP_TABLE, 1247 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 20057 },
	{ 0xe292, 0xe295, PDF_CMAP_TABLE, 1249 },
	{ 0xe296, 0xe297, PDF_CMAP_RANGE, 20059 },
	{ 0xe298, 0xe29b, PDF_CMAP_TABLE, 1253 },
	{ 0xe29c, 0xe29f, PDF_CMAP_RANGE, 20061 },
	{ 0xe2a0, 0xe2a0, PDF_CMAP_SINGLE, 9473 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe340, 0xe341, PDF_CMAP_RANGE, 20065 },
	{ 0xe342, 0xe343, PDF_CMAP_TABLE, 1257 },
	{ 0xe344, 0xe346, PDF_CMAP_RANGE, 20067 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 9496 },
	{ 0xe348, 0xe34a, PDF_CMAP_RANGE, 20070 },
	{ 0xe34b, 0xe34b, PDF_CMAP_SINGLE, 7766 },
	{ 0xe34c, 0xe34e, PDF_CMAP_RANGE, 20073 },
	{ 0xe34f, 0xe351, PDF_CMAP_TABLE, 1259 },
	{ 0xe352, 0xe353, PDF_CMAP_RANGE, 20077 },
	{ 0xe354, 0xe355, PDF_CMAP_TABLE, 1262 },
	{ 0xe356, 0xe357, PDF_CMAP_RANGE, 20079 },
	{ 0xe358, 0xe358, PDF_CMAP_SINGLE, 9488 },
	{ 0xe359, 0xe35b, PDF_CMAP_RANGE, 20081 },
	{ 0xe35c, 0xe35c, PDF_CMAP_SINGLE, 7765 },
	{ 0xe35d, 0xe35f, PDF_CMAP_RANGE, 20084 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 9482 },
	{ 0xe361, 0xe365, PDF_CMAP_RANGE, 20087 },
	{ 0xe366, 0xe367, PDF_CMAP_RANGE, 9489 },
	{ 0xe368, 0xe370, PDF_CMAP_RANGE, 20092 },
	{ 0xe371, 0xe374, PDF_CMAP_TABLE, 1264 },
	{ 0xe375, 0xe377, PDF_CMAP_RANGE, 20102 },
	{ 0xe378, 0xe379, PDF_CMAP_TABLE, 1268 },
	{ 0xe37a, 0xe37b, PDF_CMAP_RANGE, 20105 },
	{ 0xe37c, 0xe37e, PDF_CMAP_TABLE, 1270 },
	{ 0xe380, 0xe389, PDF_CMAP_RANGE, 20108 },
	{ 0xe38a, 0xe38c, PDF_CMAP_TABLE, 1273 },
	{ 0xe38d, 0xe38e, PDF_CMAP_RANGE, 20119 },
	{ 0xe38f, 0xe395, PDF_CMAP_TABLE, 1276 },
	{ 0xe396, 0xe398, PDF_CMAP_RANGE, 20124 },
	{ 0xe399, 0xe399, PDF_CMAP_SINGLE, 9501 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 20127 },
	{ 0xe39c, 0xe3a0, PDF_CMAP_TABLE, 1283 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe440, 0xe444, PDF_CMAP_TABLE, 1288 },
	{ 0xe445, 0xe447, PDF_CMAP_RANGE, 20131 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 9509 },
	{ 0xe449, 0xe44d, PDF_CMAP_RANGE, 20134 },
	{ 0xe44e, 0xe453, PDF_CMAP_TABLE, 1293 },
	{ 0xe454, 0xe457, PDF_CMAP_RANGE, 20141 },
	{ 0xe458, 0xe45e, PDF_CMAP_TABLE, 1299 },
	{ 0xe45f, 0xe461, PDF_CMAP_RANGE, 20148 },
	{ 0xe462, 0xe462, PDF_CMAP_SINGLE, 9513 },
	{ 0xe463, 0xe464, PDF_CMAP_RANGE, 20151 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 9505 },
	{ 0xe466, 0xe467, PDF_CMAP_RANGE, 20153 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 7935 },
	{ 0xe469, 0xe472, PDF_CMAP_RANGE, 20155 },
	{ 0xe473, 0xe475, PDF_CMAP_TABLE, 1306 },
	{ 0xe476, 0xe478, PDF_CMAP_RANGE, 20166 },
	{ 0xe479, 0xe47e, PDF_CMAP_TABLE, 1309 },
	{ 0xe480, 0xe481, PDF_CMAP_TABLE, 1315 },
	{ 0xe482, 0xe483, PDF_CMAP_RANGE, 20171 },
	{ 0xe484, 0xe488, PDF_CMAP_TABLE, 1317 },
	{ 0xe489, 0xe48c, PDF_CMAP_RANGE, 20173 },
	{ 0xe48d, 0xe48f, PDF_CMAP_TABLE, 1322 },
	{ 0xe490, 0xe492, PDF_CMAP_RANGE, 20178 },
	{ 0xe493, 0xe493, PDF_CMAP_SINGLE, 7960 },
	{ 0xe494, 0xe497, PDF_CMAP_RANGE, 20181 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 9547 },
	{ 0xe499, 0xe49c, PDF_CMAP_RANGE, 20185 },
	{ 0xe49d, 0xe49d, PDF_CMAP_SINGLE, 9543 },
	{ 0xe49e, 0xe49f, PDF_CMAP_RANGE, 9551 },
	{ 0xe4a0, 0xe4a0, PDF_CMAP_SINGLE, 20189 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe540, 0xe545, PDF_CMAP_RANGE, 20190 },
	{ 0xe546, 0xe548, PDF_CMAP_TABLE, 1325 },
	{ 0xe549, 0xe54a, PDF_CMAP_RANGE, 20197 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 9548 },
	{ 0xe54c, 0xe54d, PDF_CMAP_RANGE, 20199 },
	{ 0xe54e, 0xe551, PDF_CMAP_TABLE, 1328 },
	{ 0xe552, 0xe554, PDF_CMAP_RANGE, 20201 },
	{ 0xe555, 0xe558, PDF_CMAP_TABLE, 1332 },
	{ 0xe559, 0xe55b, PDF_CMAP_RANGE, 20205 },
	{ 0xe55c, 0xe55e, PDF_CMAP_TABLE, 1336 },
	{ 0xe55f, 0xe560, PDF_CMAP_RANGE, 20209 },
	{ 0xe561, 0xe561, PDF_CMAP_SINGLE, 8619 },
	{ 0xe562, 0xe563, PDF_CMAP_RANGE, 20211 },
	{ 0xe564, 0xe565, PDF_CMAP_TABLE, 1339 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 20213 },
	{ 0xe568, 0xe569, PDF_CMAP_TABLE, 1341 },
	{ 0xe56a, 0xe56b, PDF_CMAP_RANGE, 20215 },
	{ 0xe56c, 0xe56e, PDF_CMAP_TABLE, 1343 },
	{ 0xe56f, 0xe574, PDF_CMAP_RANGE, 20218 },
	{ 0xe575, 0xe578, PDF_CMAP_TABLE, 1346 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 20225 },
	{ 0xe57b, 0xe57c, PDF_CMAP_TABLE, 1350 },
	{ 0xe57d, 0xe57e, PDF_CMAP_RANGE, 20227 },
	{ 0xe580, 0xe583, PDF_CMAP_TABLE, 1352 },
	{ 0xe584, 0xe589, PDF_CMAP_RANGE, 20231 },
	{ 0xe58a, 0xe58a, PDF_CMAP_SINGLE, 9558 },
	{ 0xe58b, 0xe58d, PDF_CMAP_RANGE, 20237 },
	{ 0xe58e, 0xe58e, PDF_CMAP_SINGLE, 8808 },
	{ 0xe58f, 0xe590, PDF_CMAP_RANGE, 20240 },
	{ 0xe591, 0xe591, PDF_CMAP_SINGLE, 7894 },
	{ 0xe592, 0xe599, PDF_CMAP_RANGE, 20242 },
	{ 0xe59a, 0xe59b, PDF_CMAP_TABLE, 1356 },
	{ 0xe59c, 0xe59e, PDF_CMAP_RANGE, 20250 },
	{ 0xe59f, 0xe5a0, PDF_CMAP_TABLE, 1358 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe640, 0xe640, PDF_CMAP_SINGLE, 8412 },
	{ 0xe641, 0xe643, PDF_CMAP_RANGE, 20254 },
	{ 0xe644, 0xe644, PDF_CMAP_SINGLE, 9561 },
	{ 0xe645, 0xe648, PDF_CMAP_RANGE, 20257 },
	{ 0xe649, 0xe64a, PDF_CMAP_TABLE, 1360 },
	{ 0xe64b, 0xe64d, PDF_CMAP_RANGE, 20261 },
	{ 0xe64e, 0xe64e, PDF_CMAP_SINGLE, 8829 },
	{ 0xe64f, 0xe651, PDF_CMAP_RANGE, 20264 },
	{ 0xe652, 0xe652, PDF_CMAP_SINGLE, 9598 },
	{ 0xe653, 0xe655, PDF_CMAP_RANGE, 20267 },
	{ 0xe656, 0xe658, PDF_CMAP_TABLE, 1362 },
	{ 0xe659, 0xe65a, PDF_CMAP_RANGE, 20271 },
	{ 0xe65b, 0xe65b, PDF_CMAP_SINGLE, 9566 },
	{ 0xe65c, 0xe65d, PDF_CMAP_RANGE, 20273 },
	{ 0xe65e, 0xe65e, PDF_CMAP_SINGLE, 7732 },
	{ 0xe65f, 0xe668, PDF_CMAP_RANGE, 20275 },
	{ 0xe669, 0xe66b, PDF_CMAP_TABLE, 1365 },
	{ 0xe66c, 0xe674, PDF_CMAP_RANGE, 20286 },
	{ 0xe675, 0xe676, PDF_CMAP_TABLE, 1368 },
	{ 0xe677, 0xe678, PDF_CMAP_RANGE, 20295 },
	{ 0xe679, 0xe67e, PDF_CMAP_TABLE, 1370 },
	{ 0xe680, 0xe684, PDF_CMAP_TABLE, 1376 },
	{ 0xe685, 0xe686, PDF_CMAP_RANGE, 20301 },
	{ 0xe687, 0xe689, PDF_CMAP_TABLE, 1381 },
	{ 0xe68a, 0xe68b, PDF_CMAP_RANGE, 20304 },
	{ 0xe68c, 0xe68c, PDF_CMAP_SINGLE, 9569 },
	{ 0xe68d, 0xe692, PDF_CMAP_RANGE, 20306 },
	{ 0xe693, 0xe693, PDF_CMAP_SINGLE, 9570 },
	{ 0xe694, 0xe696, PDF_CMAP_RANGE, 20312 },
	{ 0xe697, 0xe697, PDF_CMAP_SINGLE, 9580 },
	{ 0xe698, 0xe69a, PDF_CMAP_RANGE, 20315 },
	{ 0xe69b, 0xe69c, PDF_CMAP_TABLE, 1384 },
	{ 0xe69d, 0xe69e, PDF_CMAP_RANGE, 20318 },
	{ 0xe69f, 0xe6a0, PDF_CMAP_TABLE, 1386 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 20320 },
	{ 0xe743, 0xe743, PDF_CMAP_SINGLE, 9582 },
	{ 0xe744, 0xe747, PDF_CMAP_RANGE, 20323 },
	{ 0xe748, 0xe749, PDF_CMAP_TABLE, 1388 },
	{ 0xe74a, 0xe74c, PDF_CMAP_RANGE, 20327 },
	{ 0xe74d, 0xe74e, PDF_CMAP_RANGE, 9576 },
	{ 0xe74f, 0xe755, PDF_CMAP_TABLE, 1390 },
	{ 0xe756, 0xe758, PDF_CMAP_RANGE, 20332 },
	{ 0xe759, 0xe759, PDF_CMAP_SINGLE, 9757 },
	{ 0xe75a, 0xe765, PDF_CMAP_RANGE, 20335 },
	{ 0xe766, 0xe76a, PDF_CMAP_TABLE, 1397 },
	{ 0xe76b, 0xe773, PDF_CMAP_RANGE, 20349 },
	{ 0xe774, 0xe774, PDF_CMAP_SINGLE, 9506 },
	{ 0xe775, 0xe77b, PDF_CMAP_RANGE, 20358 },
	{ 0xe77c, 0xe77c, PDF_CMAP_SINGLE, 9523 },
	{ 0xe77d, 0xe77e, PDF_CMAP_RANGE, 20365 },
	{ 0xe780, 0xe781, PDF_CMAP_RANGE, 20367 },
	{ 0xe782, 0xe786, PDF_CMAP_TABLE, 1402 },
	{ 0xe787, 0xe789, PDF_CMAP_RANGE, 20370 },
	{ 0xe78a, 0xe78b, PDF_CMAP_TABLE, 1407 },
	{ 0xe78c, 0xe78e, PDF_CMAP_RANGE, 20373 },
	{ 0xe78f, 0xe78f, PDF_CMAP_SINGLE, 9584 },
	{ 0xe790, 0xe791, PDF_CMAP_RANGE, 20376 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 9589 },
	{ 0xe793, 0xe797, PDF_CMAP_RANGE, 20378 },
	{ 0xe798, 0xe799, PDF_CMAP_RANGE, 9537 },
	{ 0xe79a, 0xe79a, PDF_CMAP_SINGLE, 9565 },
	{ 0xe79b, 0xe79f, PDF_CMAP_RANGE, 20383 },
	{ 0xe7a0, 0xe7a0, PDF_CMAP_SINGLE, 8218 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe840, 0xe842, PDF_CMAP_RANGE, 20388 },
	{ 0xe843, 0xe846, PDF_CMAP_TABLE, 1409 },
	{ 0xe847, 0xe848, PDF_CMAP_RANGE, 20392 },
	{ 0xe849, 0xe84b, PDF_CMAP_TABLE, 1413 },
	{ 0xe84c, 0xe84e, PDF_CMAP_RANGE, 20395 },
	{ 0xe84f, 0xe84f, PDF_CMAP_SINGLE, 9595 },
	{ 0xe850, 0xe853, PDF_CMAP_RANGE, 20398 },
	{ 0xe854, 0xe854, PDF_CMAP_SINGLE, 8867 },
	{ 0xe855, 0xe859, PDF_CMAP_RANGE, 20402 },
	{ 0xe85a, 0xe85c, PDF_CMAP_TABLE, 1416 },
	{ 0xe85d, 0xe861, PDF_CMAP_RANGE, 20408 },
	{ 0xe862, 0xe864, PDF_CMAP_TABLE, 1419 },
	{ 0xe865, 0xe86f, PDF_CMAP_RANGE, 20414 },
	{ 0xe870, 0xe870, PDF_CMAP_SINGLE, 9492 },
	{ 0xe871, 0xe872, PDF_CMAP_RANGE, 20425 },
	{ 0xe873, 0xe875, PDF_CMAP_TABLE, 1422 },
	{ 0xe876, 0xe87b, PDF_CMAP_RANGE, 20428 },
	{ 0xe87c, 0xe87c, PDF_CMAP_SINGLE, 9588 },
	{ 0xe87d, 0xe87e, PDF_CMAP_RANGE, 20434 },
	{ 0xe880, 0xe882, PDF_CMAP_TABLE, 1425 },
	{ 0xe883, 0xe886, PDF_CMAP_RANGE, 20437 },
	{ 0xe887, 0xe889, PDF_CMAP_TABLE, 1428 },
	{ 0xe88a, 0xe88b, PDF_CMAP_RANGE, 20442 },
	{ 0xe88c, 0xe88f, PDF_CMAP_TABLE, 1431 },
	{ 0xe890, 0xe8a0, PDF_CMAP_RANGE, 20444 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe940, 0xe94b, PDF_CMAP_RANGE, 20461 },
	{ 0xe94c, 0xe94c, PDF_CMAP_SINGLE, 7797 },
	{ 0xe94d, 0xe953, PDF_CMAP_RANGE, 20473 },
	{ 0xe954, 0xe957, PDF_CMAP_TABLE, 1435 },
	{ 0xe958, 0xe959, PDF_CMAP_RANGE, 20481 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 9152 },
	{ 0xe95b, 0xe95c, PDF_CMAP_RANGE, 20483 },
	{ 0xe95d, 0xe968, PDF_CMAP_TABLE, 1439 },
	{ 0xe969, 0xe96b, PDF_CMAP_RANGE, 20489 },
	{ 0xe96c, 0xe96c, PDF_CMAP_SINGLE, 8809 },
	{ 0xe96d, 0xe974, PDF_CMAP_RANGE, 20492 },
	{ 0xe975, 0xe979, PDF_CMAP_TABLE, 1451 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 20501 },
	{ 0xe97c, 0xe97e, PDF_CMAP_TABLE, 1456 },
	{ 0xe980, 0xe982, PDF_CMAP_TABLE, 1459 },
	{ 0xe983, 0xe986, PDF_CMAP_RANGE, 20504 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 8783 },
	{ 0xe988, 0xe98a, PDF_CMAP_RANGE, 20508 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 9163 },
	{ 0xe98c, 0xe98d, PDF_CMAP_RANGE, 20511 },
	{ 0xe98e, 0xe994, PDF_CMAP_TABLE, 1462 },
	{ 0xe995, 0xe997, PDF_CMAP_RANGE, 20514 },
	{ 0xe998, 0xe998, PDF_CMAP_SINGLE, 9168 },
	{ 0xe999, 0xe99a, PDF_CMAP_RANGE, 20517 },
	{ 0xe99b, 0xe9a0, PDF_CMAP_TABLE, 1469 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xea40, 0xea40, PDF_CMAP_SINGLE, 8184 },
	{ 0xea41, 0xea43, PDF_CMAP_RANGE, 20521 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 9171 },
	{ 0xea45, 0xea47, PDF_CMAP_RANGE, 20524 },
	{ 0xea48, 0xea4a, PDF_CMAP_TABLE, 1475 },
	{ 0xea4b, 0xea4f, PDF_CMAP_RANGE, 20527 },
	{ 0xea50, 0xea52, PDF_CMAP_TABLE, 1478 },
	{ 0xea53, 0xea54, PDF_CMAP_RANGE, 20533 },
	{ 0xea55, 0xea56, PDF_CMAP_TABLE, 1481 },
	{ 0xea57, 0xea58, PDF_CMAP_RANGE, 20535 },
	{ 0xea59, 0xea59, PDF_CMAP_SINGLE, 9157 },
	{ 0xea5a, 0xea7e, PDF_CMAP_RANGE, 20537 },
	{ 0xea80, 0xea80, PDF_CMAP_SINGLE, 8986 },
	{ 0xea81, 0xea83, PDF_CMAP_RANGE, 20574 },
	{ 0xea84, 0xea84, PDF_CMAP_SINGLE, 8468 },
	{ 0xea85, 0xea86, PDF_CMAP_RANGE, 20577 },
	{ 0xea87, 0xea87, PDF_CMAP_SINGLE, 8836 },
	{ 0xea88, 0xea8d, PDF_CMAP_RANGE, 20579 },
	{ 0xea8e, 0xea91, PDF_CMAP_TABLE, 1483 },
	{ 0xea92, 0xea95, PDF_CMAP_RANGE, 20586 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 8705 },
	{ 0xea97, 0xea9f, PDF_CMAP_RANGE, 20590 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 7897 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeb40, 0xeb41, PDF_CMAP_TABLE, 1487 },
	{ 0xeb42, 0xeb44, PDF_CMAP_RANGE, 20600 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 8786 },
	{ 0xeb46, 0xeb47, PDF_CMAP_RANGE, 20603 },
	{ 0xeb48, 0xeb48, PDF_CMAP_SINGLE, 8057 },
	{ 0xeb49, 0xeb52, PDF_CMAP_RANGE, 20605 },
	{ 0xeb53, 0xeb55, PDF_CMAP_TABLE, 1489 },
	{ 0xeb56, 0xeb5a, PDF_CMAP_RANGE, 20616 },
	{ 0xeb5b, 0xeb5d, PDF_CMAP_TABLE, 1492 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 20622 },
	{ 0xeb60, 0xeb62, PDF_CMAP_TABLE, 1495 },
	{ 0xeb63, 0xeb6c, PDF_CMAP_RANGE, 20625 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 8534 },
	{ 0xeb6e, 0xeb6f, PDF_CMAP_RANGE, 20635 },
	{ 0xeb70, 0xeb73, PDF_CMAP_TABLE, 1498 },
	{ 0xeb74, 0xeb77, PDF_CMAP_RANGE, 20638 },
	{ 0xeb78, 0xeb79, PDF_CMAP_TABLE, 1502 },
	{ 0xeb7a, 0xeb7e, PDF_CMAP_RANGE, 20642 },
	{ 0xeb80, 0xeb84, PDF_CMAP_RANGE, 20647 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 8784 },
	{ 0xeb86, 0xeb89, PDF_CMAP_RANGE, 20652 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 7875 },
	{ 0xeb8b, 0xeba0, PDF_CMAP_RANGE, 20656 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xec40, 0xec45, PDF_CMAP_RANGE, 20678 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8616 },
	{ 0xec47, 0xec55, PDF_CMAP_RANGE, 20684 },
	{ 0xec56, 0xec56, PDF_CMAP_SINGLE, 9741 },
	{ 0xec57, 0xec59, PDF_CMAP_RANGE, 20699 },
	{ 0xec5a, 0xec5c, PDF_CMAP_TABLE, 1504 },
	{ 0xec5d, 0xec5f, PDF_CMAP_RANGE, 20703 },
	{ 0xec60, 0xec60, PDF_CMAP_SINGLE, 8242 },
	{ 0xec61, 0xec6d, PDF_CMAP_RANGE, 20706 },
	{ 0xec6e, 0xec6e, PDF_CMAP_SINGLE, 9739 },
	{ 0xec6f, 0xec75, PDF_CMAP_RANGE, 20719 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8899 },
	{ 0xec77, 0xec7e, PDF_CMAP_RANGE, 20726 },
	{ 0xec80, 0xec95, PDF_CMAP_RANGE, 20734 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 7971 },
	{ 0xec97, 0xeca0, PDF_CMAP_RANGE, 20756 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xed40, 0xed45, PDF_CMAP_RANGE, 20766 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 9877 },
	{ 0xed47, 0xed57, PDF_CMAP_RANGE, 20772 },
	{ 0xed58, 0xed58, PDF_CMAP_SINGLE, 9822 },
	{ 0xed59, 0xed5d, PDF_CMAP_RANGE, 20789 },
	{ 0xed5e, 0xed5e, PDF_CMAP_SINGLE, 9821 },
	{ 0xed5f, 0xed60, PDF_CMAP_RANGE, 20794 },
	{ 0xed61, 0xed61, PDF_CMAP_SINGLE, 9874 },
	{ 0xed62, 0xed63, PDF_CMAP_RANGE, 20796 },
	{ 0xed64, 0xed67, PDF_CMAP_TABLE, 1507 },
	{ 0xed68, 0xed6d, PDF_CMAP_RANGE, 20799 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 8000 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 20805 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 9317 },
	{ 0xed75, 0xed76, PDF_CMAP_RANGE, 20810 },
	{ 0xed77, 0xed79, PDF_CMAP_TABLE, 1511 },
	{ 0xed7a, 0xed7e, PDF_CMAP_RANGE, 20813 },
	{ 0xed80, 0xed90, PDF_CMAP_RANGE, 20818 },
	{ 0xed91, 0xeda0, PDF_CMAP_TABLE, 1514 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xee40, 0xee44, PDF_CMAP_TABLE, 1530 },
	{ 0xee45, 0xee47, PDF_CMAP_RANGE, 20840 },
	{ 0xee48, 0xee49, PDF_CMAP_TABLE, 1535 },
	{ 0xee4a, 0xee4c, PDF_CMAP_RANGE, 20843 },
	{ 0xee4d, 0xee4d, PDF_CMAP_SINGLE, 9673 },
	{ 0xee4e, 0xee51, PDF_CMAP_RANGE, 20846 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 9672 },
	{ 0xee53, 0xee54, PDF_CMAP_RANGE, 20850 },
	{ 0xee55, 0xee57, PDF_CMAP_TABLE, 1537 },
	{ 0xee58, 0xee5d, PDF_CMAP_RANGE, 20853 },
	{ 0xee5e, 0xee5e, PDF_CMAP_SINGLE, 8573 },
	{ 0xee5f, 0xee60, PDF_CMAP_RANGE, 20859 },
	{ 0xee61, 0xee61, PDF_CMAP_SINGLE, 8062 },
	{ 0xee62, 0xee67, PDF_CMAP_RANGE, 20861 },
	{ 0xee68, 0xee69, PDF_CMAP_TABLE, 1540 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 20867 },
	{ 0xee6c, 0xee6e, PDF_CMAP_TABLE, 1542 },
	{ 0xee6f, 0xee76, PDF_CMAP_RANGE, 20870 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 8154 },
	{ 0xee78, 0xee7c, PDF_CMAP_RANGE, 20878 },
	{ 0xee7d, 0xee7e, PDF_CMAP_TABLE, 1545 },
	{ 0xee80, 0xee80, PDF_CMAP_SINGLE, 9677 },
	{ 0xee81, 0xee84, PDF_CMAP_RANGE, 20883 },
	{ 0xee85, 0xee86, PDF_CMAP_TABLE, 1547 },
	{ 0xee87, 0xee89, PDF_CMAP_RANGE, 20887 },
	{ 0xee8a, 0xee8d, PDF_CMAP_TABLE, 1549 },
	{ 0xee8e, 0xee8f, PDF_CMAP_RANGE, 20891 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 8200 },
	{ 0xee91, 0xee93, PDF_CMAP_RANGE, 20893 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 9680 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 20896 },
	{ 0xee97, 0xee99, PDF_CMAP_TABLE, 1553 },
	{ 0xee9a, 0xee9c, PDF_CMAP_RANGE, 20899 },
	{ 0xee9d, 0xee9e, PDF_CMAP_TABLE, 1556 },
	{ 0xee9f, 0xeea0, PDF_CMAP_RANGE, 20902 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xef40, 0xef45, PDF_CMAP_TABLE, 1558 },
	{ 0xef46, 0xef4b, PDF_CMAP_RANGE, 20905 },
	{ 0xef4c, 0xef4c, PDF_CMAP_SINGLE, 7936 },
	{ 0xef4d, 0xef51, PDF_CMAP_RANGE, 20911 },
	{ 0xef52, 0xef53, PDF_CMAP_RANGE, 9420 },
	{ 0xef54, 0xef57, PDF_CMAP_TABLE, 1564 },
	{ 0xef58, 0xef59, PDF_CMAP_RANGE, 20918 },
	{ 0xef5a, 0xef5a, PDF_CMAP_SINGLE, 9422 },
	{ 0xef5b, 0xef5f, PDF_CMAP_RANGE, 20920 },
	{ 0xef60, 0xef60, PDF_CMAP_SINGLE, 9423 },
	{ 0xef61, 0xef67, PDF_CMAP_RANGE, 20925 },
	{ 0xef68, 0xef6c, PDF_CMAP_TABLE, 1568 },
	{ 0xef6d, 0xef76, PDF_CMAP_RANGE, 20934 },
	{ 0xef77, 0xef77, PDF_CMAP_SINGLE, 7924 },
	{ 0xef78, 0xef79, PDF_CMAP_RANGE, 20944 },
	{ 0xef7a, 0xef7c, PDF_CMAP_TABLE, 1573 },
	{ 0xef7d, 0xef7e, PDF_CMAP_RANGE, 20947 },
	{ 0xef80, 0xef81, PDF_CMAP_RANGE, 20949 },
	{ 0xef82, 0xef82, PDF_CMAP_SINGLE, 9117 },
	{ 0xef83, 0xef84, PDF_CMAP_RANGE, 9119 },
	{ 0xef85, 0xef88, PDF_CMAP_TABLE, 1576 },
	{ 0xef89, 0xef8a, PDF_CMAP_RANGE, 20953 },
	{ 0xef8b, 0xef8d, PDF_CMAP_TABLE, 1580 },
	{ 0xef8e, 0xef94, PDF_CMAP_RANGE, 20956 },
	{ 0xef95, 0xef97, PDF_CMAP_TABLE, 1583 },
	{ 0xef98, 0xef9b, PDF_CMAP_RANGE, 20963 },
	{ 0xef9c, 0xef9e, PDF_CMAP_TABLE, 1586 },
	{ 0xef9f, 0xefa0, PDF_CMAP_RANGE, 20968 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf040, 0xf044, PDF_CMAP_TABLE, 1589 },
	{ 0xf045, 0xf046, PDF_CMAP_RANGE, 20972 },
	{ 0xf047, 0xf049, PDF_CMAP_TABLE, 1594 },
	{ 0xf04a, 0xf04d, PDF_CMAP_RANGE, 20974 },
	{ 0xf04e, 0xf04e, PDF_CMAP_SINGLE, 8760 },
	{ 0xf04f, 0xf050, PDF_CMAP_RANGE, 20978 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 9125 },
	{ 0xf052, 0xf053, PDF_CMAP_RANGE, 20980 },
	{ 0xf054, 0xf054, PDF_CMAP_SINGLE, 8090 },
	{ 0xf055, 0xf056, PDF_CMAP_RANGE, 20982 },
	{ 0xf057, 0xf057, PDF_CMAP_SINGLE, 8643 },
	{ 0xf058, 0xf05d, PDF_CMAP_RANGE, 20984 },
	{ 0xf05e, 0xf05e, PDF_CMAP_SINGLE, 7982 },
	{ 0xf05f, 0xf067, PDF_CMAP_RANGE, 20990 },
	{ 0xf068, 0xf068, PDF_CMAP_SINGLE, 9116 },
	{ 0xf069, 0xf06b, PDF_CMAP_RANGE, 20999 },
	{ 0xf06c, 0xf06c, PDF_CMAP_SINGLE, 9126 },
	{ 0xf06d, 0xf070, PDF_CMAP_RANGE, 21002 },
	{ 0xf071, 0xf074, PDF_CMAP_TABLE, 1597 },
	{ 0xf075, 0xf077, PDF_CMAP_RANGE, 21007 },
	{ 0xf078, 0xf07a, PDF_CMAP_TABLE, 1601 },
	{ 0xf07b, 0xf07c, PDF_CMAP_RANGE, 21011 },
	{ 0xf07d, 0xf07e, PDF_CMAP_RANGE, 9129 },
	{ 0xf080, 0xf082, PDF_CMAP_TABLE, 1604 },
	{ 0xf083, 0xf086, PDF_CMAP_RANGE, 21013 },
	{ 0xf087, 0xf088, PDF_CMAP_TABLE, 1607 },
	{ 0xf089, 0xf08a, PDF_CMAP_RANGE, 21017 },
	{ 0xf08b, 0xf08b, PDF_CMAP_SINGLE, 9830 },
	{ 0xf08c, 0xf08f, PDF_CMAP_RANGE, 21019 },
	{ 0xf090, 0xf092, PDF_CMAP_TABLE, 1609 },
	{ 0xf093, 0xf095, PDF_CMAP_RANGE, 21024 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 9133 },
	{ 0xf097, 0xf0a0, PDF_CMAP_RANGE, 21027 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf140, 0xf151, PDF_CMAP_RANGE, 21037 },
	{ 0xf152, 0xf154, PDF_CMAP_TABLE, 1612 },
	{ 0xf155, 0xf156, PDF_CMAP_RANGE, 21055 },
	{ 0xf157, 0xf15a, PDF_CMAP_TABLE, 1615 },
	{ 0xf15b, 0xf166, PDF_CMAP_RANGE, 21058 },
	{ 0xf167, 0xf167, PDF_CMAP_SINGLE, 7767 },
	{ 0xf168, 0xf175, PDF_CMAP_RANGE, 21070 },
	{ 0xf176, 0xf17b, PDF_CMAP_TABLE, 1619 },
	{ 0xf17c, 0xf17d, PDF_CMAP_RANGE, 21085 },
	{ 0xf17e, 0xf17e, PDF_CMAP_SINGLE, 9224 },
	{ 0xf180, 0xf189, PDF_CMAP_TABLE, 1625 },
	{ 0xf18a, 0xf193, PDF_CMAP_RANGE, 21091 },
	{ 0xf194, 0xf194, PDF_CMAP_SINGLE, 7999 },
	{ 0xf195, 0xf197, PDF_CMAP_RANGE, 21101 },
	{ 0xf198, 0xf198, PDF_CMAP_SINGLE, 8295 },
	{ 0xf199, 0xf1a0, PDF_CMAP_RANGE, 21104 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf240, 0xf244, PDF_CMAP_RANGE, 21112 },
	{ 0xf245, 0xf247, PDF_CMAP_TABLE, 1635 },
	{ 0xf248, 0xf24a, PDF_CMAP_RANGE, 21118 },
	{ 0xf24b, 0xf24b, PDF_CMAP_SINGLE, 9231 },
	{ 0xf24c, 0xf252, PDF_CMAP_RANGE, 21121 },
	{ 0xf253, 0xf255, PDF_CMAP_TABLE, 1638 },
	{ 0xf256, 0xf25b, PDF_CMAP_RANGE, 21128 },
	{ 0xf25c, 0xf25c, PDF_CMAP_SINGLE, 9234 },
	{ 0xf25d, 0xf25e, PDF_CMAP_RANGE, 21134 },
	{ 0xf25f, 0xf25f, PDF_CMAP_SINGLE, 8375 },
	{ 0xf260, 0xf270, PDF_CMAP_RANGE, 21136 },
	{ 0xf271, 0xf276, PDF_CMAP_TABLE, 1641 },
	{ 0xf277, 0xf27b, PDF_CMAP_RANGE, 21155 },
	{ 0xf27c, 0xf27e, PDF_CMAP_TABLE, 1647 },
	{ 0xf280, 0xf284, PDF_CMAP_RANGE, 21160 },
	{ 0xf285, 0xf289, PDF_CMAP_TABLE, 1650 },
	{ 0xf28a, 0xf28b, PDF_CMAP_RANGE, 9238 },
	{ 0xf28c, 0xf28c, PDF_CMAP_SINGLE, 8433 },
	{ 0xf28d, 0xf290, PDF_CMAP_RANGE, 21166 },
	{ 0xf291, 0xf291, PDF_CMAP_SINGLE, 9226 },
	{ 0xf292, 0xf293, PDF_CMAP_RANGE, 21170 },
	{ 0xf294, 0xf296, PDF_CMAP_TABLE, 1655 },
	{ 0xf297, 0xf29b, PDF_CMAP_RANGE, 21173 },
	{ 0xf29c, 0xf29e, PDF_CMAP_TABLE, 1658 },
	{ 0xf29f, 0xf2a0, PDF_CMAP_RANGE, 21179 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf340, 0xf341, PDF_CMAP_TABLE, 1661 },
	{ 0xf342, 0xf344, PDF_CMAP_RANGE, 21181 },
	{ 0xf345, 0xf345, PDF_CMAP_SINGLE, 8860 },
	{ 0xf346, 0xf347, PDF_CMAP_RANGE, 21184 },
	{ 0xf348, 0xf34b, PDF_CMAP_TABLE, 1663 },
	{ 0xf34c, 0xf34f, PDF_CMAP_RANGE, 21187 },
	{ 0xf350, 0xf350, PDF_CMAP_SINGLE, 9228 },
	{ 0xf351, 0xf360, PDF_CMAP_RANGE, 21191 },
	{ 0xf361, 0xf361, PDF_CMAP_SINGLE, 7721 },
	{ 0xf362, 0xf373, PDF_CMAP_RANGE, 21207 },
	{ 0xf374, 0xf379, PDF_CMAP_TABLE, 1667 },
	{ 0xf37a, 0xf37e, PDF_CMAP_RANGE, 21226 },
	{ 0xf380, 0xf38b, PDF_CMAP_RANGE, 21231 },
	{ 0xf38c, 0xf38c, PDF_CMAP_SINGLE, 9845 },
	{ 0xf38d, 0xf39f, PDF_CMAP_RANGE, 21243 },
	{ 0xf3a0, 0xf3a0, PDF_CMAP_SINGLE, 8524 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf440, 0xf444, PDF_CMAP_RANGE, 21262 },
	{ 0xf445, 0xf445, PDF_CMAP_SINGLE, 9850 },
	{ 0xf446, 0xf44f, PDF_CMAP_RANGE, 21267 },
	{ 0xf450, 0xf450, PDF_CMAP_SINGLE, 9888 },
	{ 0xf451, 0xf456, PDF_CMAP_RANGE, 21277 },
	{ 0xf457, 0xf45d, PDF_CMAP_TABLE, 1673 },
	{ 0xf45e, 0xf461, PDF_CMAP_RANGE, 21286 },
	{ 0xf462, 0xf464, PDF_CMAP_TABLE, 1680 },
	{ 0xf465, 0xf474, PDF_CMAP_RANGE, 21291 },
	{ 0xf475, 0xf475, PDF_CMAP_SINGLE, 9829 },
	{ 0xf476, 0xf47b, PDF_CMAP_RANGE, 21307 },
	{ 0xf47c, 0xf47e, PDF_CMAP_TABLE, 1683 },
	{ 0xf480, 0xf493, PDF_CMAP_RANGE, 21314 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 8266 },
	{ 0xf495, 0xf498, PDF_CMAP_RANGE, 21334 },
	{ 0xf499, 0xf499, PDF_CMAP_SINGLE, 9759 },
	{ 0xf49a, 0xf49b, PDF_CMAP_RANGE, 21338 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 9758 },
	{ 0xf49d, 0xf4a0, PDF_CMAP_RANGE, 21340 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf540, 0xf544, PDF_CMAP_RANGE, 21344 },
	{ 0xf545, 0xf547, PDF_CMAP_TABLE, 1686 },
	{ 0xf548, 0xf551, PDF_CMAP_RANGE, 21350 },
	{ 0xf552, 0xf556, PDF_CMAP_TABLE, 1689 },
	{ 0xf557, 0xf55d, PDF_CMAP_RANGE, 21361 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 9769 },
	{ 0xf55f, 0xf560, PDF_CMAP_RANGE, 21368 },
	{ 0xf561, 0xf562, PDF_CMAP_TABLE, 1694 },
	{ 0xf563, 0xf56d, PDF_CMAP_RANGE, 21370 },
	{ 0xf56e, 0xf572, PDF_CMAP_TABLE, 1696 },
	{ 0xf573, 0xf57e, PDF_CMAP_RANGE, 21382 },
	{ 0xf580, 0xf584, PDF_CMAP_RANGE, 21394 },
	{ 0xf585, 0xf586, PDF_CMAP_TABLE, 1701 },
	{ 0xf587, 0xf58b, PDF_CMAP_RANGE, 21399 },
	{ 0xf58c, 0xf58f, PDF_CMAP_TABLE, 1703 },
	{ 0xf590, 0xf598, PDF_CMAP_RANGE, 21405 },
	{ 0xf599, 0xf59b, PDF_CMAP_TABLE, 1707 },
	{ 0xf59c, 0xf59f, PDF_CMAP_RANGE, 21415 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 9795 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf640, 0xf641, PDF_CMAP_TABLE, 1710 },
	{ 0xf642, 0xf644, PDF_CMAP_RANGE, 21420 },
	{ 0xf645, 0xf648, PDF_CMAP_TABLE, 1712 },
	{ 0xf649, 0xf64a, PDF_CMAP_RANGE, 21424 },
	{ 0xf64b, 0xf64d, PDF_CMAP_TABLE, 1716 },
	{ 0xf64e, 0xf64f, PDF_CMAP_RANGE, 9787 },
	{ 0xf650, 0xf653, PDF_CMAP_RANGE, 21427 },
	{ 0xf654, 0xf654, PDF_CMAP_SINGLE, 9794 },
	{ 0xf655, 0xf657, PDF_CMAP_RANGE, 21431 },
	{ 0xf658, 0xf658, PDF_CMAP_SINGLE, 9796 },
	{ 0xf659, 0xf660, PDF_CMAP_RANGE, 21434 },
	{ 0xf661, 0xf663, PDF_CMAP_TABLE, 1719 },
	{ 0xf664, 0xf66b, PDF_CMAP_RANGE, 21443 },
	{ 0xf66c, 0xf66d, PDF_CMAP_TABLE, 1722 },
	{ 0xf66e, 0xf670, PDF_CMAP_RANGE, 21451 },
	{ 0xf671, 0xf671, PDF_CMAP_SINGLE, 9801 },
	{ 0xf672, 0xf673, PDF_CMAP_RANGE, 21454 },
	{ 0xf674, 0xf677, PDF_CMAP_TABLE, 1724 },
	{ 0xf678, 0xf67e, PDF_CMAP_RANGE, 21457 },
	{ 0xf680, 0xf684, PDF_CMAP_RANGE, 21464 },
	{ 0xf685, 0xf685, PDF_CMAP_SINGLE, 9805 },
	{ 0xf686, 0xf687, PDF_CMAP_RANGE, 21469 },
	{ 0xf688, 0xf68a, PDF_CMAP_TABLE, 1728 },
	{ 0xf68b, 0xf68c, PDF_CMAP_RANGE, 21472 },
	{ 0xf68d, 0xf68e, PDF_CMAP_TABLE, 1731 },
	{ 0xf68f, 0xf691, PDF_CMAP_RANGE, 21474 },
	{ 0xf692, 0xf692, PDF_CMAP_SINGLE, 9807 },
	{ 0xf693, 0xf695, PDF_CMAP_RANGE, 21477 },
	{ 0xf696, 0xf6a0, PDF_CMAP_TABLE, 1733 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf740, 0xf741, PDF_CMAP_RANGE, 21484 },
	{ 0xf742, 0xf742, PDF_CMAP_SINGLE, 9812 },
	{ 0xf743, 0xf748, PDF_CMAP_RANGE, 21486 },
	{ 0xf749, 0xf749, PDF_CMAP_SINGLE, 9816 },
	{ 0xf74a, 0xf74b, PDF_CMAP_RANGE, 21492 },
	{ 0xf74c, 0xf74d, PDF_CMAP_TABLE, 1744 },
	{ 0xf74e, 0xf755, PDF_CMAP_RANGE, 21494 },
	{ 0xf756, 0xf75c, PDF_CMAP_TABLE, 1746 },
	{ 0xf75d, 0xf760, PDF_CMAP_RANGE, 21504 },
	{ 0xf761, 0xf763, PDF_CMAP_TABLE, 1753 },
	{ 0xf764, 0xf76a, PDF_CMAP_RANGE, 21509 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 9820 },
	{ 0xf76c, 0xf770, PDF_CMAP_RANGE, 21516 },
	{ 0xf771, 0xf771, PDF_CMAP_SINGLE, 9772 },
	{ 0xf772, 0xf77b, PDF_CMAP_RANGE, 21521 },
	{ 0xf77c, 0xf77e, PDF_CMAP_TABLE, 1756 },
	{ 0xf780, 0xf7a0, PDF_CMAP_RANGE, 21532 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
	{ 0xf840, 0xf841, PDF_CMAP_RANGE, 21565 },
	{ 0xf842, 0xf842, PDF_CMAP_SINGLE, 8348 },
	{ 0xf843, 0xf845, PDF_CMAP_RANGE, 21567 },
	{ 0xf846, 0xf846, PDF_CMAP_SINGLE, 9600 },
	{ 0xf847, 0xf848, PDF_CMAP_RANGE, 21570 },
	{ 0xf849, 0xf849, PDF_CMAP_SINGLE, 8924 },
	{ 0xf84a, 0xf84f, PDF_CMAP_RANGE, 21572 },
	{ 0xf850, 0xf853, PDF_CMAP_TABLE, 1759 },
	{ 0xf854, 0xf862, PDF_CMAP_RANGE, 21579 },
	{ 0xf863, 0xf866, PDF_CMAP_TABLE, 1763 },
	{ 0xf867, 0xf871, PDF_CMAP_RANGE, 21595 },
	{ 0xf872, 0xf872, PDF_CMAP_SINGLE, 8578 },
	{ 0xf873, 0xf877, PDF_CMAP_RANGE, 21606 },
	{ 0xf878, 0xf87c, PDF_CMAP_TABLE, 1767 },
	{ 0xf87d, 0xf87e, PDF_CMAP_RANGE, 21613 },
	{ 0xf880, 0xf881, PDF_CMAP_TABLE, 1772 },
	{ 0xf882, 0xf883, PDF_CMAP_RANGE, 21616 },
	{ 0xf884, 0xf886, PDF_CMAP_TABLE, 1774 },
	{ 0xf887, 0xf88c, PDF_CMAP_RANGE, 21619 },
	{ 0xf88d, 0xf88e, PDF_CMAP_TABLE, 1777 },
	{ 0xf88f, 0xf898, PDF_CMAP_RANGE, 21625 },
	{ 0xf899, 0xf899, PDF_CMAP_SINGLE, 8007 },
	{ 0xf89a, 0xf89c, PDF_CMAP_RANGE, 21635 },
	{ 0xf89d, 0xf89d, PDF_CMAP_SINGLE, 7965 },
	{ 0xf89e, 0xf89f, PDF_CMAP_RANGE, 21638 },
	{ 0xf8a0, 0xf8a0, PDF_CMAP_SINGLE, 9613 },
	{ 0xf940, 0xf94d, PDF_CMAP_RANGE, 21640 },
	{ 0xf94e, 0xf950, PDF_CMAP_TABLE, 1779 },
	{ 0xf951, 0xf958, PDF_CMAP_RANGE, 21654 },
	{ 0xf959, 0xf95a, PDF_CMAP_TABLE, 1782 },
	{ 0xf95b, 0xf95c, PDF_CMAP_RANGE, 21662 },
	{ 0xf95d, 0xf95e, PDF_CMAP_TABLE, 1784 },
	{ 0xf95f, 0xf966, PDF_CMAP_RANGE, 21664 },
	{ 0xf967, 0xf969, PDF_CMAP_TABLE, 1786 },
	{ 0xf96a, 0xf96b, PDF_CMAP_RANGE, 21673 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 9624 },
	{ 0xf96d, 0xf96e, PDF_CMAP_RANGE, 21675 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 8438 },
	{ 0xf970, 0xf97e, PDF_CMAP_RANGE, 21677 },
	{ 0xf980, 0xf984, PDF_CMAP_RANGE, 21692 },
	{ 0xf985, 0xf987, PDF_CMAP_TABLE, 1789 },
	{ 0xf988, 0xf990, PDF_CMAP_RANGE, 21698 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 9622 },
	{ 0xf992, 0xf995, PDF_CMAP_RANGE, 21707 },
	{ 0xf996, 0xf998, PDF_CMAP_TABLE, 1792 },
	{ 0xf999, 0xf9a0, PDF_CMAP_RANGE, 21712 },
	{ 0xfa40, 0xfa41, PDF_CMAP_RANGE, 21720 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 9629 },
	{ 0xfa43, 0xfa45, PDF_CMAP_RANGE, 21722 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 9630 },
	{ 0xfa47, 0xfa4b, PDF_CMAP_RANGE, 21725 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 9036 },
	{ 0xfa4d, 0xfa50, PDF_CMAP_RANGE, 21730 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 8004 },
	{ 0xfa52, 0xfa57, PDF_CMAP_RANGE, 21734 },
	{ 0xfa58, 0xfa59, PDF_CMAP_TABLE, 1795 },
	{ 0xfa5a, 0xfa5c, PDF_CMAP_RANGE, 21740 },
	{ 0xfa5d, 0xfa61, PDF_CMAP_TABLE, 1797 },
	{ 0xfa62, 0xfa6f, PDF_CMAP_RANGE, 21745 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 9634 },
	{ 0xfa71, 0xfa73, PDF_CMAP_RANGE, 21759 },
	{ 0xfa74, 0xfa77, PDF_CMAP_TABLE, 1802 },
	{ 0xfa78, 0xfa7e, PDF_CMAP_RANGE, 21763 },
	{ 0xfa80, 0xfa82, PDF_CMAP_RANGE, 21770 },
	{ 0xfa83, 0xfa84, PDF_CMAP_TABLE, 1806 },
	{ 0xfa85, 0xfa8c, PDF_CMAP_RANGE, 21773 },
	{ 0xfa8d, 0xfa8d, PDF_CMAP_SINGLE, 9636 },
	{ 0xfa8e, 0xfa8f, PDF_CMAP_RANGE, 21781 },
	{ 0xfa90, 0xfa91, PDF_CMAP_TABLE, 1808 },
	{ 0xfa92, 0xfa95, PDF_CMAP_RANGE, 21783 },
	{ 0xfa96, 0xfa98, PDF_CMAP_TABLE, 1810 },
	{ 0xfa99, 0xfaa0, PDF_CMAP_RANGE, 21787 },
	{ 0xfb40, 0xfb48, PDF_CMAP_RANGE, 21795 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 9640 },
	{ 0xfb4a, 0xfb51, PDF_CMAP_RANGE, 21804 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 9606 },
	{ 0xfb53, 0xfb56, PDF_CMAP_RANGE, 21812 },
	{ 0xfb57, 0xfb5b, PDF_CMAP_TABLE, 1813 },
	{ 0xfb5c, 0xfb74, PDF_CMAP_RANGE, 21817 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 9864 },
	{ 0xfb76, 0xfb78, PDF_CMAP_RANGE, 21842 },
	{ 0xfb79, 0xfb7e, PDF_CMAP_TABLE, 1818 },
	{ 0xfb80, 0xfb8f, PDF_CMAP_RANGE, 21847 },
	{ 0xfb90, 0xfb90, PDF_CMAP_SINGLE, 8207 },
	{ 0xfb91, 0xfb9b, PDF_CMAP_RANGE, 21863 },
	{ 0xfb9c, 0xfb9c, PDF_CMAP_SINGLE, 8305 },
	{ 0xfb9d, 0xfb9e, PDF_CMAP_RANGE, 21874 },
	{ 0xfb9f, 0xfba0, PDF_CMAP_TABLE, 1824 },
	{ 0xfc40, 0xfc43, PDF_CMAP_RANGE, 21877 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 9878 },
	{ 0xfc45, 0xfc48, PDF_CMAP_RANGE, 21881 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 9871 },
	{ 0xfc4a, 0xfc59, PDF_CMAP_RANGE, 21885 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 8922 },
	{ 0xfc5b, 0xfc62, PDF_CMAP_RANGE, 21901 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 7873 },
	{ 0xfc64, 0xfc67, PDF_CMAP_RANGE, 21909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 7859 },
	{ 0xfc69, 0xfc6e, PDF_CMAP_RANGE, 21913 },
	{ 0xfc6f, 0xfc71, PDF_CMAP_TABLE, 1826 },
	{ 0xfc72, 0xfc73, PDF_CMAP_RANGE, 21920 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 9833 },
	{ 0xfc75, 0xfc76, PDF_CMAP_RANGE, 21922 },
	{ 0xfc77, 0xfc78, PDF_CMAP_RANGE, 9752 },
	{ 0xfc79, 0xfc7e, PDF_CMAP_RANGE, 21924 },
	{ 0xfc80, 0xfc82, PDF_CMAP_RANGE, 21930 },
	{ 0xfc83, 0xfc83, PDF_CMAP_SINGLE, 9754 },
	{ 0xfc84, 0xfc89, PDF_CMAP_RANGE, 21933 },
	{ 0xfc8a, 0xfc8a, PDF_CMAP_SINGLE, 9843 },
	{ 0xfc8b, 0xfca0, PDF_CMAP_RANGE, 21939 },
	{ 0xfd40, 0xfd51, PDF_CMAP_RANGE, 21961 },
	{ 0xfd52, 0xfd53, PDF_CMAP_TABLE, 1829 },
	{ 0xfd54, 0xfd56, PDF_CMAP_RANGE, 21979 },
	{ 0xfd57, 0xfd5a, PDF_CMAP_TABLE, 1831 },
	{ 0xfd5b, 0xfd5e, PDF_CMAP_RANGE, 21983 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 9745 },
	{ 0xfd60, 0xfd61, PDF_CMAP_RANGE, 21987 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 9747 },
	{ 0xfd63, 0xfd64, PDF_CMAP_RANGE, 21989 },
	{ 0xfd65, 0xfd69, PDF_CMAP_TABLE, 1835 },
	{ 0xfd6a, 0xfd6b, PDF_CMAP_RANGE, 21992 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 9748 },
	{ 0xfd6d, 0xfd6f, PDF_CMAP_RANGE, 21994 },
	{ 0xfd70, 0xfd72, PDF_CMAP_TABLE, 1840 },
	{ 0xfd73, 0xfd77, PDF_CMAP_RANGE, 21998 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 8434 },
	{ 0xfd79, 0xfd7c, PDF_CMAP_RANGE, 22003 },
	{ 0xfd7d, 0xfd7e, PDF_CMAP_TABLE, 1843 },
	{ 0xfd80, 0xfd87, PDF_CMAP_RANGE, 22008 },
	{ 0xfd88, 0xfd88, PDF_CMAP_SINGLE, 8247 },
	{ 0xfd89, 0xfd8a, PDF_CMAP_RANGE, 22016 },
	{ 0xfd8b, 0xfd8b, PDF_CMAP_SINGLE, 8371 },
	{ 0xfd8c, 0xfd8e, PDF_CMAP_RANGE, 22018 },
	{ 0xfd8f, 0xfd90, PDF_CMAP_TABLE, 1845 },
	{ 0xfd91, 0xfd93, PDF_CMAP_RANGE, 22021 },
	{ 0xfd94, 0xfd94, PDF_CMAP_SINGLE, 7988 },
	{ 0xfd95, 0xfd9c, PDF_CMAP_RANGE, 22024 },
	{ 0xfd9d, 0xfda0, PDF_CMAP_TABLE, 1847 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 4697 },
	{ 0xfe41, 0xfe7e, PDF_CMAP_RANGE, 22032 },
	{ 0xfe80, 0xfea0, PDF_CMAP_RANGE, 22094 },
	{ 0xdd53, 0xdd53, PDF_CMAP_SINGLE, 8857 },
	{ 0xdd54, 0xdd54, PDF_CMAP_SINGLE, 9361 },
	{ 0xdd55, 0xdd55, PDF_CMAP_SINGLE, 9366 },
	{ 0xdd56, 0xdd56, PDF_CMAP_SINGLE, 9359 },
	{ 0xdd57, 0xdd57, PDF_CMAP_SINGLE, 9362 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 19647 },
	{ 0xdd59, 0xdd59, PDF_CMAP_SINGLE, 9367 },
	{ 0xdd5a, 0xdd5d, PDF_CMAP_RANGE, 19648 },
	{ 0xdd5e, 0xdd5e, PDF_CMAP_SINGLE, 8113 },
	{ 0xdd5f, 0xdd5f, PDF_CMAP_SINGLE, 19652 },
	{ 0xdd60, 0xdd60, PDF_CMAP_SINGLE, 9370 },
	{ 0xdd61, 0xdd61, PDF_CMAP_SINGLE, 19653 },
	{ 0xdd62, 0xdd62, PDF_CMAP_SINGLE, 9369 },
	{ 0xdd63, 0xdd63, PDF_CMAP_SINGLE, 19654 },
	{ 0xdd64, 0xdd64, PDF_CMAP_SINGLE, 8792 },
	{ 0xdd65, 0xdd65, PDF_CMAP_SINGLE, 9368 },
	{ 0xdd66, 0xdd6c, PDF_CMAP_RANGE, 19655 },
	{ 0xdd6d, 0xdd6d, PDF_CMAP_SINGLE, 9371 },
	{ 0xdd6e, 0xdd6e, PDF_CMAP_SINGLE, 19662 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 7945 },
	{ 0xdd70, 0xdd70, PDF_CMAP_SINGLE, 8422 },
	{ 0xdd71, 0xdd75, PDF_CMAP_RANGE, 19663 },
	{ 0xdd76, 0xdd76, PDF_CMAP_SINGLE, 8230 },
	{ 0xdd77, 0xdd77, PDF_CMAP_SINGLE, 9375 },
	{ 0xdd78, 0xdd78, PDF_CMAP_SINGLE, 8025 },
	{ 0xdd79, 0xdd7a, PDF_CMAP_RANGE, 9373 },
	{ 0xdd7b, 0xdd7e, PDF_CMAP_RANGE, 19668 },
	{ 0xdd80, 0xdd80, PDF_CMAP_SINGLE, 19672 },
	{ 0xdd81, 0xdd81, PDF_CMAP_SINGLE, 7995 },
	{ 0xdd82, 0xdd82, PDF_CMAP_SINGLE, 9372 },
	{ 0xdd83, 0xdd84, PDF_CMAP_RANGE, 19673 },
	{ 0xdd85, 0xdd85, PDF_CMAP_SINGLE, 7738 },
	{ 0xdd86, 0xdd86, PDF_CMAP_SINGLE, 8283 },
	{ 0xdd87, 0xdd8a, PDF_CMAP_RANGE, 19675 },
	{ 0xdd8b, 0xdd8b, PDF_CMAP_SINGLE, 8048 },
	{ 0xdd8c, 0xdd8e, PDF_CMAP_RANGE, 19679 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 9376 },
	{ 0xdd90, 0xdd93, PDF_CMAP_RANGE, 19682 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 8507 },
	{ 0xdd95, 0xdd96, PDF_CMAP_RANGE, 19686 },
	{ 0xdd97, 0xdd97, PDF_CMAP_SINGLE, 7943 },
	{ 0xdd98, 0xdd99, PDF_CMAP_RANGE, 19688 },
	{ 0xdd9a, 0xdd9a, PDF_CMAP_SINGLE, 8816 },
	{ 0xdd9b, 0xdd9b, PDF_CMAP_SINGLE, 8759 },
	{ 0xdd9c, 0xdd9d, PDF_CMAP_RANGE, 19690 },
	{ 0xdd9e, 0xdd9e, PDF_CMAP_SINGLE, 9426 },
	{ 0xdd9f, 0xdd9f, PDF_CMAP_SINGLE, 19692 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 8627 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 8773 },
	{ 0xde41, 0xde41, PDF_CMAP_SINGLE, 9377 },
	{ 0xde42, 0xde43, PDF_CMAP_RANGE, 19693 },
	{ 0xde44, 0xde44, PDF_CMAP_SINGLE, 8872 },
	{ 0xde45, 0xde47, PDF_CMAP_RANGE, 19695 },
	{ 0xde48, 0xde48, PDF_CMAP_SINGLE, 8828 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 8112 },
	{ 0xde4a, 0xde4e, PDF_CMAP_RANGE, 19698 },
	{ 0xde4f, 0xde4f, PDF_CMAP_SINGLE, 9378 },
	{ 0xde50, 0xde59, PDF_CMAP_RANGE, 19703 },
	{ 0xde5a, 0xde5a, PDF_CMAP_SINGLE, 8006 },
	{ 0xde5b, 0xde5b, PDF_CMAP_SINGLE, 19713 },
	{ 0xde5c, 0xde5c, PDF_CMAP_SINGLE, 9088 },
	{ 0xde5d, 0xde5d, PDF_CMAP_SINGLE, 9365 },
	{ 0xde5e, 0xde5e, PDF_CMAP_SINGLE, 19714 },
	{ 0xde5f, 0xde5f, PDF_CMAP_SINGLE, 9360 },
	{ 0xde60, 0xde6a, PDF_CMAP_RANGE, 19715 },
	{ 0xde6b, 0xde6b, PDF_CMAP_SINGLE, 7728 },
	{ 0xde6c, 0xde6e, PDF_CMAP_RANGE, 19726 },
	{ 0xde6f, 0xde6f, PDF_CMAP_SINGLE, 7837 },
	{ 0xde70, 0xde70, PDF_CMAP_SINGLE, 7755 },
	{ 0xde71, 0xde71, PDF_CMAP_SINGLE, 7754 },
	{ 0xde72, 0xde72, PDF_CMAP_SINGLE, 8362 },
	{ 0xde73, 0xde7e, PDF_CMAP_RANGE, 19729 },
	{ 0xde80, 0xde91, PDF_CMAP_RANGE, 19741 },
	{ 0xde92, 0xde92, PDF_CMAP_SINGLE, 9851 },
	{ 0xde93, 0xde9e, PDF_CMAP_RANGE, 19759 },
	{ 0xde9f, 0xde9f, PDF_CMAP_SINGLE, 9202 },
	{ 0xdea0, 0xdea0, PDF_CMAP_SINGLE, 19771 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdf40, 0xdf40, PDF_CMAP_SINGLE, 8830 },
	{ 0xdf41, 0xdf41, PDF_CMAP_SINGLE, 19772 },
	{ 0xdf42, 0xdf42, PDF_CMAP_SINGLE, 8217 },
	{ 0xdf43, 0xdf4c, PDF_CMAP_RANGE, 19773 },
	{ 0xdf4d, 0xdf4d, PDF_CMAP_SINGLE, 8123 },
	{ 0xdf4e, 0xdf5b, PDF_CMAP_RANGE, 19783 },
	{ 0xdf5c, 0xdf5c, PDF_CMAP_SINGLE, 8787 },
	{ 0xdf5d, 0xdf5d, PDF_CMAP_SINGLE, 19797 },
	{ 0xdf5e, 0xdf5e, PDF_CMAP_SINGLE, 7998 },
	{ 0xdf5f, 0xdf5f, PDF_CMAP_SINGLE, 7846 },
	{ 0xdf60, 0xdf60, PDF_CMAP_SINGLE, 8590 },
	{ 0xdf61, 0xdf63, PDF_CMAP_RANGE, 19798 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 8684 },
	{ 0xdf65, 0xdf65, PDF_CMAP_SINGLE, 19801 },
	{ 0xdf66, 0xdf66, PDF_CMAP_SINGLE, 7870 },
	{ 0xdf67, 0xdf67, PDF_CMAP_SINGLE, 19802 },
	{ 0xdf68, 0xdf68, PDF_CMAP_SINGLE, 8778 },
	{ 0xdf69, 0xdf6c, PDF_CMAP_RANGE, 19803 },
	{ 0xdf6d, 0xdf6d, PDF_CMAP_SINGLE, 8499 },
	{ 0xdf6e, 0xdf73, PDF_CMAP_RANGE, 19807 },
	{ 0xdf74, 0xdf74, PDF_CMAP_SINGLE, 7812 },
	{ 0xdf75, 0xdf76, PDF_CMAP_RANGE, 19813 },
	{ 0xdf77, 0xdf77, PDF_CMAP_SINGLE, 8399 },
	{ 0xdf78, 0xdf78, PDF_CMAP_SINGLE, 8674 },
	{ 0xdf79, 0xdf79, PDF_CMAP_SINGLE, 19815 },
	{ 0xdf7a, 0xdf7a, PDF_CMAP_SINGLE, 8719 },
	{ 0xdf7b, 0xdf7b, PDF_CMAP_SINGLE, 19816 },
	{ 0xdf7c, 0xdf7c, PDF_CMAP_SINGLE, 8233 },
	{ 0xdf7d, 0xdf7d, PDF_CMAP_SINGLE, 19817 },
	{ 0xdf7e, 0xdf7e, PDF_CMAP_SINGLE, 8307 },
	{ 0xdf80, 0xdf80, PDF_CMAP_SINGLE, 8021 },
	{ 0xdf81, 0xdf82, PDF_CMAP_RANGE, 19818 },
	{ 0xdf83, 0xdf83, PDF_CMAP_SINGLE, 9201 },
	{ 0xdf84, 0xdf84, PDF_CMAP_SINGLE, 19820 },
	{ 0xdf85, 0xdf85, PDF_CMAP_SINGLE, 7750 },
	{ 0xdf86, 0xdf88, PDF_CMAP_RANGE, 19821 },
	{ 0xdf89, 0xdf89, PDF_CMAP_SINGLE, 8291 },
	{ 0xdf8a, 0xdf8a, PDF_CMAP_SINGLE, 9203 },
	{ 0xdf8b, 0xdfa0, PDF_CMAP_RANGE, 19824 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe040, 0xe04f, PDF_CMAP_RANGE, 19846 },
	{ 0xe050, 0xe050, PDF_CMAP_SINGLE, 8990 },
	{ 0xe051, 0xe05c, PDF_CMAP_RANGE, 19862 },
	{ 0xe05d, 0xe05d, PDF_CMAP_SINGLE, 8755 },
	{ 0xe05e, 0xe068, PDF_CMAP_RANGE, 19874 },
	{ 0xe069, 0xe069, PDF_CMAP_SINGLE, 8992 },
	{ 0xe06a, 0xe06b, PDF_CMAP_RANGE, 19885 },
	{ 0xe06c, 0xe06c, PDF_CMAP_SINGLE, 8647 },
	{ 0xe06d, 0xe074, PDF_CMAP_RANGE, 19887 },
	{ 0xe075, 0xe075, PDF_CMAP_SINGLE, 8892 },
	{ 0xe076, 0xe076, PDF_CMAP_SINGLE, 19895 },
	{ 0xe077, 0xe077, PDF_CMAP_SINGLE, 8988 },
	{ 0xe078, 0xe078, PDF_CMAP_SINGLE, 19896 },
	{ 0xe079, 0xe079, PDF_CMAP_SINGLE, 8785 },
	{ 0xe07a, 0xe07e, PDF_CMAP_RANGE, 19897 },
	{ 0xe080, 0xe086, PDF_CMAP_RANGE, 19902 },
	{ 0xe087, 0xe087, PDF_CMAP_SINGLE, 7867 },
	{ 0xe088, 0xe08c, PDF_CMAP_RANGE, 19909 },
	{ 0xe08d, 0xe08d, PDF_CMAP_SINGLE, 8839 },
	{ 0xe08e, 0xe08e, PDF_CMAP_SINGLE, 19914 },
	{ 0xe08f, 0xe08f, PDF_CMAP_SINGLE, 8237 },
	{ 0xe090, 0xe090, PDF_CMAP_SINGLE, 7851 },
	{ 0xe091, 0xe091, PDF_CMAP_SINGLE, 19915 },
	{ 0xe092, 0xe092, PDF_CMAP_SINGLE, 8989 },
	{ 0xe093, 0xe093, PDF_CMAP_SINGLE, 19916 },
	{ 0xe094, 0xe094, PDF_CMAP_SINGLE, 8991 },
	{ 0xe095, 0xe096, PDF_CMAP_RANGE, 19917 },
	{ 0xe097, 0xe097, PDF_CMAP_SINGLE, 8987 },
	{ 0xe098, 0xe0a0, PDF_CMAP_RANGE, 19919 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe140, 0xe141, PDF_CMAP_RANGE, 19928 },
	{ 0xe142, 0xe142, PDF_CMAP_SINGLE, 8993 },
	{ 0xe143, 0xe163, PDF_CMAP_RANGE, 19930 },
	{ 0xe164, 0xe164, PDF_CMAP_SINGLE, 8789 },
	{ 0xe165, 0xe167, PDF_CMAP_RANGE, 19963 },
	{ 0xe168, 0xe168, PDF_CMAP_SINGLE, 7823 },
	{ 0xe169, 0xe173, PDF_CMAP_RANGE, 19966 },
	{ 0xe174, 0xe174, PDF_CMAP_SINGLE, 8716 },
	{ 0xe175, 0xe175, PDF_CMAP_SINGLE, 8100 },
	{ 0xe176, 0xe17e, PDF_CMAP_RANGE, 19977 },
	{ 0xe180, 0xe183, PDF_CMAP_RANGE, 19986 },
	{ 0xe184, 0xe184, PDF_CMAP_SINGLE, 8347 },
	{ 0xe185, 0xe185, PDF_CMAP_SINGLE, 8664 },
	{ 0xe186, 0xe186, PDF_CMAP_SINGLE, 19990 },
	{ 0xe187, 0xe187, PDF_CMAP_SINGLE, 9722 },
	{ 0xe188, 0xe188, PDF_CMAP_SINGLE, 19991 },
	{ 0xe189, 0xe189, PDF_CMAP_SINGLE, 9721 },
	{ 0xe18a, 0xe18b, PDF_CMAP_RANGE, 19992 },
	{ 0xe18c, 0xe18c, PDF_CMAP_SINGLE, 8500 },
	{ 0xe18d, 0xe18d, PDF_CMAP_SINGLE, 19994 },
	{ 0xe18e, 0xe190, PDF_CMAP_RANGE, 9459 },
	{ 0xe191, 0xe191, PDF_CMAP_SINGLE, 9464 },
	{ 0xe192, 0xe192, PDF_CMAP_SINGLE, 19995 },
	{ 0xe193, 0xe193, PDF_CMAP_SINGLE, 9463 },
	{ 0xe194, 0xe194, PDF_CMAP_SINGLE, 7880 },
	{ 0xe195, 0xe195, PDF_CMAP_SINGLE, 9462 },
	{ 0xe196, 0xe197, PDF_CMAP_RANGE, 19996 },
	{ 0xe198, 0xe198, PDF_CMAP_SINGLE, 8832 },
	{ 0xe199, 0xe19d, PDF_CMAP_RANGE, 19998 },
	{ 0xe19e, 0xe19e, PDF_CMAP_SINGLE, 7877 },
	{ 0xe19f, 0xe19f, PDF_CMAP_SINGLE, 9467 },
	{ 0xe1a0, 0xe1a0, PDF_CMAP_SINGLE, 20003 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe240, 0xe240, PDF_CMAP_SINGLE, 20004 },
	{ 0xe241, 0xe241, PDF_CMAP_SINGLE, 9466 },
	{ 0xe242, 0xe242, PDF_CMAP_SINGLE, 20005 },
	{ 0xe243, 0xe243, PDF_CMAP_SINGLE, 7917 },
	{ 0xe244, 0xe24e, PDF_CMAP_RANGE, 20006 },
	{ 0xe24f, 0xe24f, PDF_CMAP_SINGLE, 9469 },
	{ 0xe250, 0xe250, PDF_CMAP_SINGLE, 20017 },
	{ 0xe251, 0xe251, PDF_CMAP_SINGLE, 9465 },
	{ 0xe252, 0xe252, PDF_CMAP_SINGLE, 20018 },
	{ 0xe253, 0xe253, PDF_CMAP_SINGLE, 9470 },
	{ 0xe254, 0xe254, PDF_CMAP_SINGLE, 8397 },
	{ 0xe255, 0xe259, PDF_CMAP_RANGE, 20019 },
	{ 0xe25a, 0xe25a, PDF_CMAP_SINGLE, 9480 },
	{ 0xe25b, 0xe25b, PDF_CMAP_SINGLE, 9476 },
	{ 0xe25c, 0xe25d, PDF_CMAP_RANGE, 20024 },
	{ 0xe25e, 0xe25e, PDF_CMAP_SINGLE, 9478 },
	{ 0xe25f, 0xe261, PDF_CMAP_RANGE, 20026 },
	{ 0xe262, 0xe262, PDF_CMAP_SINGLE, 9471 },
	{ 0xe263, 0xe263, PDF_CMAP_SINGLE, 8336 },
	{ 0xe264, 0xe266, PDF_CMAP_RANGE, 20029 },
	{ 0xe267, 0xe267, PDF_CMAP_SINGLE, 7901 },
	{ 0xe268, 0xe268, PDF_CMAP_SINGLE, 7973 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 20032 },
	{ 0xe26a, 0xe26a, PDF_CMAP_SINGLE, 9475 },
	{ 0xe26b, 0xe26b, PDF_CMAP_SINGLE, 9474 },
	{ 0xe26c, 0xe26d, PDF_CMAP_RANGE, 20033 },
	{ 0xe26e, 0xe26e, PDF_CMAP_SINGLE, 7802 },
	{ 0xe26f, 0xe26f, PDF_CMAP_SINGLE, 8358 },
	{ 0xe270, 0xe277, PDF_CMAP_RANGE, 20035 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 8149 },
	{ 0xe279, 0xe27c, PDF_CMAP_RANGE, 20043 },
	{ 0xe27d, 0xe27d, PDF_CMAP_SINGLE, 7953 },
	{ 0xe27e, 0xe27e, PDF_CMAP_SINGLE, 20047 },
	{ 0xe280, 0xe280, PDF_CMAP_SINGLE, 9479 },
	{ 0xe281, 0xe281, PDF_CMAP_SINGLE, 9472 },
	{ 0xe282, 0xe282, PDF_CMAP_SINGLE, 9477 },
	{ 0xe283, 0xe288, PDF_CMAP_RANGE, 20048 },
	{ 0xe289, 0xe289, PDF_CMAP_SINGLE, 9497 },
	{ 0xe28a, 0xe28a, PDF_CMAP_SINGLE, 20054 },
	{ 0xe28b, 0xe28b, PDF_CMAP_SINGLE, 9493 },
	{ 0xe28c, 0xe28d, PDF_CMAP_RANGE, 20055 },
	{ 0xe28e, 0xe28e, PDF_CMAP_SINGLE, 9484 },
	{ 0xe28f, 0xe28f, PDF_CMAP_SINGLE, 8241 },
	{ 0xe290, 0xe291, PDF_CMAP_RANGE, 20057 },
	{ 0xe292, 0xe292, PDF_CMAP_SINGLE, 9483 },
	{ 0xe293, 0xe293, PDF_CMAP_SINGLE, 9487 },
	{ 0xe294, 0xe294, PDF_CMAP_SINGLE, 9498 },
	{ 0xe295, 0xe295, PDF_CMAP_SINGLE, 9481 },
	{ 0xe296, 0xe297, PDF_CMAP_RANGE, 20059 },
	{ 0xe298, 0xe298, PDF_CMAP_SINGLE, 9486 },
	{ 0xe299, 0xe299, PDF_CMAP_SINGLE, 8756 },
	{ 0xe29a, 0xe29a, PDF_CMAP_SINGLE, 9491 },
	{ 0xe29b, 0xe29b, PDF_CMAP_SINGLE, 8064 },
	{ 0xe29c, 0xe29f, PDF_CMAP_RANGE, 20061 },
	{ 0xe2a0, 0xe2a0, PDF_CMAP_SINGLE, 9473 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe340, 0xe341, PDF_CMAP_RANGE, 20065 },
	{ 0xe342, 0xe342, PDF_CMAP_SINGLE, 9495 },
	{ 0xe343, 0xe343, PDF_CMAP_SINGLE, 9494 },
	{ 0xe344, 0xe346, PDF_CMAP_RANGE, 20067 },
	{ 0xe347, 0xe347, PDF_CMAP_SINGLE, 9496 },
	{ 0xe348, 0xe34a, PDF_CMAP_RANGE, 20070 },
	{ 0xe34b, 0xe34b, PDF_CMAP_SINGLE, 7766 },
	{ 0xe34c, 0xe34e, PDF_CMAP_RANGE, 20073 },
	{ 0xe34f, 0xe34f, PDF_CMAP_SINGLE, 9485 },
	{ 0xe350, 0xe350, PDF_CMAP_SINGLE, 20076 },
	{ 0xe351, 0xe351, PDF_CMAP_SINGLE, 8403 },
	{ 0xe352, 0xe353, PDF_CMAP_RANGE, 20077 },
	{ 0xe354, 0xe354, PDF_CMAP_SINGLE, 8314 },
	{ 0xe355, 0xe355, PDF_CMAP_SINGLE, 8398 },
	{ 0xe356, 0xe357, PDF_CMAP_RANGE, 20079 },
	{ 0xe358, 0xe358, PDF_CMAP_SINGLE, 9488 },
	{ 0xe359, 0xe35b, PDF_CMAP_RANGE, 20081 },
	{ 0xe35c, 0xe35c, PDF_CMAP_SINGLE, 7765 },
	{ 0xe35d, 0xe35f, PDF_CMAP_RANGE, 20084 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 9482 },
	{ 0xe361, 0xe365, PDF_CMAP_RANGE, 20087 },
	{ 0xe366, 0xe367, PDF_CMAP_RANGE, 9489 },
	{ 0xe368, 0xe370, PDF_CMAP_RANGE, 20092 },
	{ 0xe371, 0xe371, PDF_CMAP_SINGLE, 8106 },
	{ 0xe372, 0xe372, PDF_CMAP_SINGLE, 20101 },
	{ 0xe373, 0xe373, PDF_CMAP_SINGLE, 9502 },
	{ 0xe374, 0xe374, PDF_CMAP_SINGLE, 7967 },
	{ 0xe375, 0xe377, PDF_CMAP_RANGE, 20102 },
	{ 0xe378, 0xe378, PDF_CMAP_SINGLE, 9517 },
	{ 0xe379, 0xe379, PDF_CMAP_SINGLE, 8733 },
	{ 0xe37a, 0xe37b, PDF_CMAP_RANGE, 20105 },
	{ 0xe37c, 0xe37c, PDF_CMAP_SINGLE, 9522 },
	{ 0xe37d, 0xe37d, PDF_CMAP_SINGLE, 20107 },
	{ 0xe37e, 0xe37e, PDF_CMAP_SINGLE, 8571 },
	{ 0xe380, 0xe389, PDF_CMAP_RANGE, 20108 },
	{ 0xe38a, 0xe38a, PDF_CMAP_SINGLE, 8623 },
	{ 0xe38b, 0xe38b, PDF_CMAP_SINGLE, 20118 },
	{ 0xe38c, 0xe38c, PDF_CMAP_SINGLE, 9516 },
	{ 0xe38d, 0xe38e, PDF_CMAP_RANGE, 20119 },
	{ 0xe38f, 0xe38f, PDF_CMAP_SINGLE, 9512 },
	{ 0xe390, 0xe390, PDF_CMAP_SINGLE, 20121 },
	{ 0xe391, 0xe391, PDF_CMAP_SINGLE, 8332 },
	{ 0xe392, 0xe392, PDF_CMAP_SINGLE, 20122 },
	{ 0xe393, 0xe393, PDF_CMAP_SINGLE, 9519 },
	{ 0xe394, 0xe394, PDF_CMAP_SINGLE, 20123 },
	{ 0xe395, 0xe395, PDF_CMAP_SINGLE, 8636 },
	{ 0xe396, 0xe398, PDF_CMAP_RANGE, 20124 },
	{ 0xe399, 0xe399, PDF_CMAP_SINGLE, 9501 },
	{ 0xe39a, 0xe39b, PDF_CMAP_RANGE, 20127 },
	{ 0xe39c, 0xe39c, PDF_CMAP_SINGLE, 9525 },
	{ 0xe39d, 0xe39d, PDF_CMAP_SINGLE, 20129 },
	{ 0xe39e, 0xe39e, PDF_CMAP_SINGLE, 8717 },
	{ 0xe39f, 0xe39f, PDF_CMAP_SINGLE, 9510 },
	{ 0xe3a0, 0xe3a0, PDF_CMAP_SINGLE, 20130 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe440, 0xe440, PDF_CMAP_SINGLE, 9524 },
	{ 0xe441, 0xe441, PDF_CMAP_SINGLE, 9514 },
	{ 0xe442, 0xe442, PDF_CMAP_SINGLE, 9503 },
	{ 0xe443, 0xe443, PDF_CMAP_SINGLE, 9521 },
	{ 0xe444, 0xe444, PDF_CMAP_SINGLE, 9500 },
	{ 0xe445, 0xe447, PDF_CMAP_RANGE, 20131 },
	{ 0xe448, 0xe448, PDF_CMAP_SINGLE, 9509 },
	{ 0xe449, 0xe44d, PDF_CMAP_RANGE, 20134 },
	{ 0xe44e, 0xe44e, PDF_CMAP_SINGLE, 8653 },
	{ 0xe44f, 0xe44f, PDF_CMAP_SINGLE, 20139 },
	{ 0xe450, 0xe450, PDF_CMAP_SINGLE, 8666 },
	{ 0xe451, 0xe451, PDF_CMAP_SINGLE, 20140 },
	{ 0xe452, 0xe452, PDF_CMAP_SINGLE, 8562 },
	{ 0xe453, 0xe453, PDF_CMAP_SINGLE, 9534 },
	{ 0xe454, 0xe457, PDF_CMAP_RANGE, 20141 },
	{ 0xe458, 0xe458, PDF_CMAP_SINGLE, 8271 },
	{ 0xe459, 0xe459, PDF_CMAP_SINGLE, 20145 },
	{ 0xe45a, 0xe45a, PDF_CMAP_SINGLE, 9539 },
	{ 0xe45b, 0xe45b, PDF_CMAP_SINGLE, 20146 },
	{ 0xe45c, 0xe45c, PDF_CMAP_SINGLE, 8663 },
	{ 0xe45d, 0xe45d, PDF_CMAP_SINGLE, 20147 },
	{ 0xe45e, 0xe45e, PDF_CMAP_SINGLE, 7740 },
	{ 0xe45f, 0xe461, PDF_CMAP_RANGE, 20148 },
	{ 0xe462, 0xe462, PDF_CMAP_SINGLE, 9513 },
	{ 0xe463, 0xe464, PDF_CMAP_RANGE, 20151 },
	{ 0xe465, 0xe465, PDF_CMAP_SINGLE, 9505 },
	{ 0xe466, 0xe467, PDF_CMAP_RANGE, 20153 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 7935 },
	{ 0xe469, 0xe472, PDF_CMAP_RANGE, 20155 },
	{ 0xe473, 0xe473, PDF_CMAP_SINGLE, 9535 },
	{ 0xe474, 0xe474, PDF_CMAP_SINGLE, 20165 },
	{ 0xe475, 0xe475, PDF_CMAP_SINGLE, 9540 },
	{ 0xe476, 0xe478, PDF_CMAP_RANGE, 20166 },
	{ 0xe479, 0xe479, PDF_CMAP_SINGLE, 9507 },
	{ 0xe47a, 0xe47a, PDF_CMAP_SINGLE, 7824 },
	{ 0xe47b, 0xe47b, PDF_CMAP_SINGLE, 9530 },
	{ 0xe47c, 0xe47c, PDF_CMAP_SINGLE, 9541 },
	{ 0xe47d, 0xe47d, PDF_CMAP_SINGLE, 20169 },
	{ 0xe47e, 0xe47e, PDF_CMAP_SINGLE, 9533 },
	{ 0xe480, 0xe480, PDF_CMAP_SINGLE, 20170 },
	{ 0xe481, 0xe481, PDF_CMAP_SINGLE, 8385 },
	{ 0xe482, 0xe483, PDF_CMAP_RANGE, 20171 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 8451 },
	{ 0xe485, 0xe485, PDF_CMAP_SINGLE, 9504 },
	{ 0xe486, 0xe486, PDF_CMAP_SINGLE, 9532 },
	{ 0xe487, 0xe487, PDF_CMAP_SINGLE, 9531 },
	{ 0xe488, 0xe488, PDF_CMAP_SINGLE, 9528 },
	{ 0xe489, 0xe48c, PDF_CMAP_RANGE, 20173 },
	{ 0xe48d, 0xe48d, PDF_CMAP_SINGLE, 9536 },
	{ 0xe48e, 0xe48e, PDF_CMAP_SINGLE, 20177 },
	{ 0xe48f, 0xe48f, PDF_CMAP_SINGLE, 8141 },
	{ 0xe490, 0xe492, PDF_CMAP_RANGE, 20178 },
	{ 0xe493, 0xe493, PDF_CMAP_SINGLE, 7960 },
	{ 0xe494, 0xe497, PDF_CMAP_RANGE, 20181 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 9547 },
	{ 0xe499, 0xe49c, PDF_CMAP_RANGE, 20185 },
	{ 0xe49d, 0xe49d, PDF_CMAP_SINGLE, 9543 },
	{ 0xe49e, 0xe49f, PDF_CMAP_RANGE, 9551 },
	{ 0xe4a0, 0xe4a0, PDF_CMAP_SINGLE, 20189 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe540, 0xe545, PDF_CMAP_RANGE, 20190 },
	{ 0xe546, 0xe546, PDF_CMAP_SINGLE, 8880 },
	{ 0xe547, 0xe547, PDF_CMAP_SINGLE, 20196 },
	{ 0xe548, 0xe548, PDF_CMAP_SINGLE, 9542 },
	{ 0xe549, 0xe54a, PDF_CMAP_RANGE, 20197 },
	{ 0xe54b, 0xe54b, PDF_CMAP_SINGLE, 9548 },
	{ 0xe54c, 0xe54d, PDF_CMAP_RANGE, 20199 },
	{ 0xe54e, 0xe54e, PDF_CMAP_SINGLE, 7834 },
	{ 0xe54f, 0xe54f, PDF_CMAP_SINGLE, 9554 },
	{ 0xe550, 0xe550, PDF_CMAP_SINGLE, 9520 },
	{ 0xe551, 0xe551, PDF_CMAP_SINGLE, 9545 },
	{ 0xe552, 0xe554, PDF_CMAP_RANGE, 20201 },
	{ 0xe555, 0xe555, PDF_CMAP_SINGLE, 9553 },
	{ 0xe556, 0xe556, PDF_CMAP_SINGLE, 7882 },
	{ 0xe557, 0xe557, PDF_CMAP_SINGLE, 20204 },
	{ 0xe558, 0xe558, PDF_CMAP_SINGLE, 8402 },
	{ 0xe559, 0xe55b, PDF_CMAP_RANGE, 20205 },
	{ 0xe55c, 0xe55c, PDF_CMAP_SINGLE, 8120 },
	{ 0xe55d, 0xe55d, PDF_CMAP_SINGLE, 20208 },
	{ 0xe55e, 0xe55e, PDF_CMAP_SINGLE, 8313 },
	{ 0xe55f, 0xe560, PDF_CMAP_RANGE, 20209 },
	{ 0xe561, 0xe561, PDF_CMAP_SINGLE, 8619 },
	{ 0xe562, 0xe563, PDF_CMAP_RANGE, 20211 },
	{ 0xe564, 0xe564, PDF_CMAP_SINGLE, 9549 },
	{ 0xe565, 0xe565, PDF_CMAP_SINGLE, 7845 },
	{ 0xe566, 0xe567, PDF_CMAP_RANGE, 20213 },
	{ 0xe568, 0xe568, PDF_CMAP_SINGLE, 8268 },
	{ 0xe569, 0xe569, PDF_CMAP_SINGLE, 8320 },
	{ 0xe56a, 0xe56b, PDF_CMAP_RANGE, 20215 },
	{ 0xe56c, 0xe56c, PDF_CMAP_SINGLE, 9837 },
	{ 0xe56d, 0xe56d, PDF_CMAP_SINGLE, 20217 },
	{ 0xe56e, 0xe56e, PDF_CMAP_SINGLE, 9527 },
	{ 0xe56f, 0xe574, PDF_CMAP_RANGE, 20218 },
	{ 0xe575, 0xe575, PDF_CMAP_SINGLE, 9546 },
	{ 0xe576, 0xe576, PDF_CMAP_SINGLE, 8632 },
	{ 0xe577, 0xe577, PDF_CMAP_SINGLE, 20224 },
	{ 0xe578, 0xe578, PDF_CMAP_SINGLE, 9550 },
	{ 0xe579, 0xe57a, PDF_CMAP_RANGE, 20225 },
	{ 0xe57b, 0xe57b, PDF_CMAP_SINGLE, 9468 },
	{ 0xe57c, 0xe57c, PDF_CMAP_SINGLE, 9556 },
	{ 0xe57d, 0xe57e, PDF_CMAP_RANGE, 20227 },
	{ 0xe580, 0xe580, PDF_CMAP_SINGLE, 20229 },
	{ 0xe581, 0xe581, PDF_CMAP_SINGLE, 7996 },
	{ 0xe582, 0xe582, PDF_CMAP_SINGLE, 20230 },
	{ 0xe583, 0xe583, PDF_CMAP_SINGLE, 7893 },
	{ 0xe584, 0xe589, PDF_CMAP_RANGE, 20231 },
	{ 0xe58a, 0xe58a, PDF_CMAP_SINGLE, 9558 },
	{ 0xe58b, 0xe58d, PDF_CMAP_RANGE, 20237 },
	{ 0xe58e, 0xe58e, PDF_CMAP_SINGLE, 8808 },
	{ 0xe58f, 0xe590, PDF_CMAP_RANGE, 20240 },
	{ 0xe591, 0xe591, PDF_CMAP_SINGLE, 7894 },
	{ 0xe592, 0xe599, PDF_CMAP_RANGE, 20242 },
	{ 0xe59a, 0xe59a, PDF_CMAP_SINGLE, 9559 },
	{ 0xe59b, 0xe59b, PDF_CMAP_SINGLE, 9555 },
	{ 0xe59c, 0xe59e, PDF_CMAP_RANGE, 20250 },
	{ 0xe59f, 0xe59f, PDF_CMAP_SINGLE, 9544 },
	{ 0xe5a0, 0xe5a0, PDF_CMAP_SINGLE, 20253 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe640, 0xe640, PDF_CMAP_SINGLE, 8412 },
	{ 0xe641, 0xe643, PDF_CMAP_RANGE, 20254 },
	{ 0xe644, 0xe644, PDF_CMAP_SINGLE, 9561 },
	{ 0xe645, 0xe648, PDF_CMAP_RANGE, 20257 },
	{ 0xe649, 0xe649, PDF_CMAP_SINGLE, 8087 },
	{ 0xe64a, 0xe64a, PDF_CMAP_SINGLE, 9557 },
	{ 0xe64b, 0xe64d, PDF_CMAP_RANGE, 20261 },
	{ 0xe64e, 0xe64e, PDF_CMAP_SINGLE, 8829 },
	{ 0xe64f, 0xe651, PDF_CMAP_RANGE, 20264 },
	{ 0xe652, 0xe652, PDF_CMAP_SINGLE, 9598 },
	{ 0xe653, 0xe655, PDF_CMAP_RANGE, 20267 },
	{ 0xe656, 0xe656, PDF_CMAP_SINGLE, 8316 },
	{ 0xe657, 0xe657, PDF_CMAP_SINGLE, 20270 },
	{ 0xe658, 0xe658, PDF_CMAP_SINGLE, 9562 },
	{ 0xe659, 0xe65a, PDF_CMAP_RANGE, 20271 },
	{ 0xe65b, 0xe65b, PDF_CMAP_SINGLE, 9566 },
	{ 0xe65c, 0xe65d, PDF_CMAP_RANGE, 20273 },
	{ 0xe65e, 0xe65e, PDF_CMAP_SINGLE, 7732 },
	{ 0xe65f, 0xe668, PDF_CMAP_RANGE, 20275 },
	{ 0xe669, 0xe669, PDF_CMAP_SINGLE, 8542 },
	{ 0xe66a, 0xe66a, PDF_CMAP_SINGLE, 20285 },
	{ 0xe66b, 0xe66b, PDF_CMAP_SINGLE, 9568 },
	{ 0xe66c, 0xe674, PDF_CMAP_RANGE, 20286 },
	{ 0xe675, 0xe675, PDF_CMAP_SINGLE, 8610 },
	{ 0xe676, 0xe676, PDF_CMAP_SINGLE, 9044 },
	{ 0xe677, 0xe678, PDF_CMAP_RANGE, 20295 },
	{ 0xe679, 0xe679, PDF_CMAP_SINGLE, 9571 },
	{ 0xe67a, 0xe67a, PDF_CMAP_SINGLE, 9511 },
	{ 0xe67b, 0xe67b, PDF_CMAP_SINGLE, 20297 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 9518 },
	{ 0xe67d, 0xe67d, PDF_CMAP_SINGLE, 9560 },
	{ 0xe67e, 0xe67e, PDF_CMAP_SINGLE, 20298 },
	{ 0xe680, 0xe680, PDF_CMAP_SINGLE, 7963 },
	{ 0xe681, 0xe681, PDF_CMAP_SINGLE, 20299 },
	{ 0xe682, 0xe682, PDF_CMAP_SINGLE, 8835 },
	{ 0xe683, 0xe683, PDF_CMAP_SINGLE, 20300 },
	{ 0xe684, 0xe684, PDF_CMAP_SINGLE, 9572 },
	{ 0xe685, 0xe686, PDF_CMAP_RANGE, 20301 },
	{ 0xe687, 0xe687, PDF_CMAP_SINGLE, 8352 },
	{ 0xe688, 0xe688, PDF_CMAP_SINGLE, 20303 },
	{ 0xe689, 0xe689, PDF_CMAP_SINGLE, 9573 },
	{ 0xe68a, 0xe68b, PDF_CMAP_RANGE, 20304 },
	{ 0xe68c, 0xe68c, PDF_CMAP_SINGLE, 9569 },
	{ 0xe68d, 0xe692, PDF_CMAP_RANGE, 20306 },
	{ 0xe693, 0xe693, PDF_CMAP_SINGLE, 9570 },
	{ 0xe694, 0xe696, PDF_CMAP_RANGE, 20312 },
	{ 0xe697, 0xe697, PDF_CMAP_SINGLE, 9580 },
	{ 0xe698, 0xe69a, PDF_CMAP_RANGE, 20315 },
	{ 0xe69b, 0xe69b, PDF_CMAP_SINGLE, 9581 },
	{ 0xe69c, 0xe69c, PDF_CMAP_SINGLE, 8224 },
	{ 0xe69d, 0xe69e, PDF_CMAP_RANGE, 20318 },
	{ 0xe69f, 0xe69f, PDF_CMAP_SINGLE, 9567 },
	{ 0xe6a0, 0xe6a0, PDF_CMAP_SINGLE, 9578 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe740, 0xe742, PDF_CMAP_RANGE, 20320 },
	{ 0xe743, 0xe743, PDF_CMAP_SINGLE, 9582 },
	{ 0xe744, 0xe747, PDF_CMAP_RANGE, 20323 },
	{ 0xe748, 0xe748, PDF_CMAP_SINGLE, 9529 },
	{ 0xe749, 0xe749, PDF_CMAP_SINGLE, 9564 },
	{ 0xe74a, 0xe74c, PDF_CMAP_RANGE, 20327 },
	{ 0xe74d, 0xe74e, PDF_CMAP_RANGE, 9576 },
	{ 0xe74f, 0xe74f, PDF_CMAP_SINGLE, 9579 },
	{ 0xe750, 0xe750, PDF_CMAP_SINGLE, 7791 },
	{ 0xe751, 0xe751, PDF_CMAP_SINGLE, 20330 },
	{ 0xe752, 0xe752, PDF_CMAP_SINGLE, 8132 },
	{ 0xe753, 0xe753, PDF_CMAP_SINGLE, 9575 },
	{ 0xe754, 0xe754, PDF_CMAP_SINGLE, 20331 },
	{ 0xe755, 0xe755, PDF_CMAP_SINGLE, 9563 },
	{ 0xe756, 0xe758, PDF_CMAP_RANGE, 20332 },
	{ 0xe759, 0xe759, PDF_CMAP_SINGLE, 9757 },
	{ 0xe75a, 0xe765, PDF_CMAP_RANGE, 20335 },
	{ 0xe766, 0xe766, PDF_CMAP_SINGLE, 9515 },
	{ 0xe767, 0xe767, PDF_CMAP_SINGLE, 20347 },
	{ 0xe768, 0xe768, PDF_CMAP_SINGLE, 9585 },
	{ 0xe769, 0xe769, PDF_CMAP_SINGLE, 20348 },
	{ 0xe76a, 0xe76a, PDF_CMAP_SINGLE, 9591 },
	{ 0xe76b, 0xe773, PDF_CMAP_RANGE, 20349 },
	{ 0xe774, 0xe774, PDF_CMAP_SINGLE, 9506 },
	{ 0xe775, 0xe77b, PDF_CMAP_RANGE, 20358 },
	{ 0xe77c, 0xe77c, PDF_CMAP_SINGLE, 9523 },
	{ 0xe77d, 0xe77e, PDF_CMAP_RANGE, 20365 },
	{ 0xe780, 0xe781, PDF_CMAP_RANGE, 20367 },
	{ 0xe782, 0xe782, PDF_CMAP_SINGLE, 8234 },
	{ 0xe783, 0xe783, PDF_CMAP_SINGLE, 20369 },
	{ 0xe784, 0xe784, PDF_CMAP_SINGLE, 9526 },
	{ 0xe785, 0xe785, PDF_CMAP_SINGLE, 9587 },
	{ 0xe786, 0xe786, PDF_CMAP_SINGLE, 9583 },
	{ 0xe787, 0xe789, PDF_CMAP_RANGE, 20370 },
	{ 0xe78a, 0xe78a, PDF_CMAP_SINGLE, 8851 },
	{ 0xe78b, 0xe78b, PDF_CMAP_SINGLE, 9592 },
	{ 0xe78c, 0xe78e, PDF_CMAP_RANGE, 20373 },
	{ 0xe78f, 0xe78f, PDF_CMAP_SINGLE, 9584 },
	{ 0xe790, 0xe791, PDF_CMAP_RANGE, 20376 },
	{ 0xe792, 0xe792, PDF_CMAP_SINGLE, 9589 },
	{ 0xe793, 0xe797, PDF_CMAP_RANGE, 20378 },
	{ 0xe798, 0xe799, PDF_CMAP_RANGE, 9537 },
	{ 0xe79a, 0xe79a, PDF_CMAP_SINGLE, 9565 },
	{ 0xe79b, 0xe79f, PDF_CMAP_RANGE, 20383 },
	{ 0xe7a0, 0xe7a0, PDF_CMAP_SINGLE, 8218 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe840, 0xe842, PDF_CMAP_RANGE, 20388 },
	{ 0xe843, 0xe843, PDF_CMAP_SINGLE, 9594 },
	{ 0xe844, 0xe844, PDF_CMAP_SINGLE, 8198 },
	{ 0xe845, 0xe845, PDF_CMAP_SINGLE, 20391 },
	{ 0xe846, 0xe846, PDF_CMAP_SINGLE, 8567 },
	{ 0xe847, 0xe848, PDF_CMAP_RANGE, 20392 },
	{ 0xe849, 0xe849, PDF_CMAP_SINGLE, 9499 },
	{ 0xe84a, 0xe84a, PDF_CMAP_SINGLE, 20394 },
	{ 0xe84b, 0xe84b, PDF_CMAP_SINGLE, 9508 },
	{ 0xe84c, 0xe84e, PDF_CMAP_RANGE, 20395 },
	{ 0xe84f, 0xe84f, PDF_CMAP_SINGLE, 9595 },
	{ 0xe850, 0xe853, PDF_CMAP_RANGE, 20398 },
	{ 0xe854, 0xe854, PDF_CMAP_SINGLE, 8867 },
	{ 0xe855, 0xe859, PDF_CMAP_RANGE, 20402 },
	{ 0xe85a, 0xe85a, PDF_CMAP_SINGLE, 9593 },
	{ 0xe85b, 0xe85b, PDF_CMAP_SINGLE, 20407 },
	{ 0xe85c, 0xe85c, PDF_CMAP_SINGLE, 9574 },
	{ 0xe85d, 0xe861, PDF_CMAP_RANGE, 20408 },
	{ 0xe862, 0xe862, PDF_CMAP_SINGLE, 8083 },
	{ 0xe863, 0xe863, PDF_CMAP_SINGLE, 20413 },
	{ 0xe864, 0xe864, PDF_CMAP_SINGLE, 9596 },
	{ 0xe865, 0xe86f, PDF_CMAP_RANGE, 20414 },
	{ 0xe870, 0xe870, PDF_CMAP_SINGLE, 9492 },
	{ 0xe871, 0xe872, PDF_CMAP_RANGE, 20425 },
	{ 0xe873, 0xe873, PDF_CMAP_SINGLE, 9597 },
	{ 0xe874, 0xe874, PDF_CMAP_SINGLE, 20427 },
	{ 0xe875, 0xe875, PDF_CMAP_SINGLE, 9586 },
	{ 0xe876, 0xe87b, PDF_CMAP_RANGE, 20428 },
	{ 0xe87c, 0xe87c, PDF_CMAP_SINGLE, 9588 },
	{ 0xe87d, 0xe87e, PDF_CMAP_RANGE, 20434 },
	{ 0xe880, 0xe880, PDF_CMAP_SINGLE, 8782 },
	{ 0xe881, 0xe881, PDF_CMAP_SINGLE, 20436 },
	{ 0xe882, 0xe882, PDF_CMAP_SINGLE, 8646 },
	{ 0xe883, 0xe886, PDF_CMAP_RANGE, 20437 },
	{ 0xe887, 0xe887, PDF_CMAP_SINGLE, 8351 },
	{ 0xe888, 0xe888, PDF_CMAP_SINGLE, 20441 },
	{ 0xe889, 0xe889, PDF_CMAP_SINGLE, 9590 },
	{ 0xe88a, 0xe88b, PDF_CMAP_RANGE, 20442 },
	{ 0xe88c, 0xe88c, PDF_CMAP_SINGLE, 8292 },
	{ 0xe88d, 0xe88d, PDF_CMAP_SINGLE, 8895 },
	{ 0xe88e, 0xe88e, PDF_CMAP_SINGLE, 9756 },
	{ 0xe88f, 0xe88f, PDF_CMAP_SINGLE, 8798 },
	{ 0xe890, 0xe8a0, PDF_CMAP_RANGE, 20444 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe940, 0xe94b, PDF_CMAP_RANGE, 20461 },
	{ 0xe94c, 0xe94c, PDF_CMAP_SINGLE, 7797 },
	{ 0xe94d, 0xe953, PDF_CMAP_RANGE, 20473 },
	{ 0xe954, 0xe954, PDF_CMAP_SINGLE, 8317 },
	{ 0xe955, 0xe955, PDF_CMAP_SINGLE, 20480 },
	{ 0xe956, 0xe956, PDF_CMAP_SINGLE, 9151 },
	{ 0xe957, 0xe957, PDF_CMAP_SINGLE, 8467 },
	{ 0xe958, 0xe959, PDF_CMAP_RANGE, 20481 },
	{ 0xe95a, 0xe95a, PDF_CMAP_SINGLE, 9152 },
	{ 0xe95b, 0xe95c, PDF_CMAP_RANGE, 20483 },
	{ 0xe95d, 0xe95d, PDF_CMAP_SINGLE, 7749 },
	{ 0xe95e, 0xe95e, PDF_CMAP_SINGLE, 20485 },
	{ 0xe95f, 0xe95f, PDF_CMAP_SINGLE, 8152 },
	{ 0xe960, 0xe960, PDF_CMAP_SINGLE, 9156 },
	{ 0xe961, 0xe961, PDF_CMAP_SINGLE, 20486 },
	{ 0xe962, 0xe962, PDF_CMAP_SINGLE, 9154 },
	{ 0xe963, 0xe963, PDF_CMAP_SINGLE, 8452 },
	{ 0xe964, 0xe964, PDF_CMAP_SINGLE, 20487 },
	{ 0xe965, 0xe965, PDF_CMAP_SINGLE, 8637 },
	{ 0xe966, 0xe966, PDF_CMAP_SINGLE, 20488 },
	{ 0xe967, 0xe967, PDF_CMAP_SINGLE, 8071 },
	{ 0xe968, 0xe968, PDF_CMAP_SINGLE, 9155 },
	{ 0xe969, 0xe96b, PDF_CMAP_RANGE, 20489 },
	{ 0xe96c, 0xe96c, PDF_CMAP_SINGLE, 8809 },
	{ 0xe96d, 0xe974, PDF_CMAP_RANGE, 20492 },
	{ 0xe975, 0xe975, PDF_CMAP_SINGLE, 8003 },
	{ 0xe976, 0xe976, PDF_CMAP_SINGLE, 20500 },
	{ 0xe977, 0xe977, PDF_CMAP_SINGLE, 7966 },
	{ 0xe978, 0xe978, PDF_CMAP_SINGLE, 9849 },
	{ 0xe979, 0xe979, PDF_CMAP_SINGLE, 7915 },
	{ 0xe97a, 0xe97b, PDF_CMAP_RANGE, 20501 },
	{ 0xe97c, 0xe97c, PDF_CMAP_SINGLE, 7989 },
	{ 0xe97d, 0xe97d, PDF_CMAP_SINGLE, 8330 },
	{ 0xe97e, 0xe97e, PDF_CMAP_SINGLE, 20503 },
	{ 0xe980, 0xe980, PDF_CMAP_SINGLE, 9159 },
	{ 0xe981, 0xe981, PDF_CMAP_SINGLE, 9161 },
	{ 0xe982, 0xe982, PDF_CMAP_SINGLE, 9158 },
	{ 0xe983, 0xe986, PDF_CMAP_RANGE, 20504 },
	{ 0xe987, 0xe987, PDF_CMAP_SINGLE, 8783 },
	{ 0xe988, 0xe98a, PDF_CMAP_RANGE, 20508 },
	{ 0xe98b, 0xe98b, PDF_CMAP_SINGLE, 9163 },
	{ 0xe98c, 0xe98d, PDF_CMAP_RANGE, 20511 },
	{ 0xe98e, 0xe98e, PDF_CMAP_SINGLE, 8691 },
	{ 0xe98f, 0xe98f, PDF_CMAP_SINGLE, 20513 },
	{ 0xe990, 0xe990, PDF_CMAP_SINGLE, 8695 },
	{ 0xe991, 0xe991, PDF_CMAP_SINGLE, 9167 },
	{ 0xe992, 0xe992, PDF_CMAP_SINGLE, 9166 },
	{ 0xe993, 0xe993, PDF_CMAP_SINGLE, 9162 },
	{ 0xe994, 0xe994, PDF_CMAP_SINGLE, 9165 },
	{ 0xe995, 0xe997, PDF_CMAP_RANGE, 20514 },
	{ 0xe998, 0xe998, PDF_CMAP_SINGLE, 9168 },
	{ 0xe999, 0xe99a, PDF_CMAP_RANGE, 20517 },
	{ 0xe99b, 0xe99b, PDF_CMAP_SINGLE, 9836 },
	{ 0xe99c, 0xe99c, PDF_CMAP_SINGLE, 20519 },
	{ 0xe99d, 0xe99d, PDF_CMAP_SINGLE, 9153 },
	{ 0xe99e, 0xe99e, PDF_CMAP_SINGLE, 20520 },
	{ 0xe99f, 0xe99f, PDF_CMAP_SINGLE, 8174 },
	{ 0xe9a0, 0xe9a0, PDF_CMAP_SINGLE, 9169 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xea40, 0xea40, PDF_CMAP_SINGLE, 8184 },
	{ 0xea41, 0xea43, PDF_CMAP_RANGE, 20521 },
	{ 0xea44, 0xea44, PDF_CMAP_SINGLE, 9171 },
	{ 0xea45, 0xea47, PDF_CMAP_RANGE, 20524 },
	{ 0xea48, 0xea48, PDF_CMAP_SINGLE, 9170 },
	{ 0xea49, 0xea49, PDF_CMAP_SINGLE, 9172 },
	{ 0xea4a, 0xea4a, PDF_CMAP_SINGLE, 7832 },
	{ 0xea4b, 0xea4f, PDF_CMAP_RANGE, 20527 },
	{ 0xea50, 0xea50, PDF_CMAP_SINGLE, 7980 },
	{ 0xea51, 0xea51, PDF_CMAP_SINGLE, 20532 },
	{ 0xea52, 0xea52, PDF_CMAP_SINGLE, 9173 },
	{ 0xea53, 0xea54, PDF_CMAP_RANGE, 20533 },
	{ 0xea55, 0xea55, PDF_CMAP_SINGLE, 7793 },
	{ 0xea56, 0xea56, PDF_CMAP_SINGLE, 9873 },
	{ 0xea57, 0xea58, PDF_CMAP_RANGE, 20535 },
	{ 0xea59, 0xea59, PDF_CMAP_SINGLE, 9157 },
	{ 0xea5a, 0xea7e, PDF_CMAP_RANGE, 20537 },
	{ 0xea80, 0xea80, PDF_CMAP_SINGLE, 8986 },
	{ 0xea81, 0xea83, PDF_CMAP_RANGE, 20574 },
	{ 0xea84, 0xea84, PDF_CMAP_SINGLE, 8468 },
	{ 0xea85, 0xea86, PDF_CMAP_RANGE, 20577 },
	{ 0xea87, 0xea87, PDF_CMAP_SINGLE, 8836 },
	{ 0xea88, 0xea8d, PDF_CMAP_RANGE, 20579 },
	{ 0xea8e, 0xea8e, PDF_CMAP_SINGLE, 8732 },
	{ 0xea8f, 0xea8f, PDF_CMAP_SINGLE, 20585 },
	{ 0xea90, 0xea90, PDF_CMAP_SINGLE, 7806 },
	{ 0xea91, 0xea91, PDF_CMAP_SINGLE, 8269 },
	{ 0xea92, 0xea95, PDF_CMAP_RANGE, 20586 },
	{ 0xea96, 0xea96, PDF_CMAP_SINGLE, 8705 },
	{ 0xea97, 0xea9f, PDF_CMAP_RANGE, 20590 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 7897 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeb40, 0xeb40, PDF_CMAP_SINGLE, 20599 },
	{ 0xeb41, 0xeb41, PDF_CMAP_SINGLE, 8114 },
	{ 0xeb42, 0xeb44, PDF_CMAP_RANGE, 20600 },
	{ 0xeb45, 0xeb45, PDF_CMAP_SINGLE, 8786 },
	{ 0xeb46, 0xeb47, PDF_CMAP_RANGE, 20603 },
	{ 0xeb48, 0xeb48, PDF_CMAP_SINGLE, 8057 },
	{ 0xeb49, 0xeb52, PDF_CMAP_RANGE, 20605 },
	{ 0xeb53, 0xeb53, PDF_CMAP_SINGLE, 8535 },
	{ 0xeb54, 0xeb54, PDF_CMAP_SINGLE, 20615 },
	{ 0xeb55, 0xeb55, PDF_CMAP_SINGLE, 8639 },
	{ 0xeb56, 0xeb5a, PDF_CMAP_RANGE, 20616 },
	{ 0xeb5b, 0xeb5b, PDF_CMAP_SINGLE, 8735 },
	{ 0xeb5c, 0xeb5c, PDF_CMAP_SINGLE, 20621 },
	{ 0xeb5d, 0xeb5d, PDF_CMAP_SINGLE, 8253 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 20622 },
	{ 0xeb60, 0xeb60, PDF_CMAP_SINGLE, 8213 },
	{ 0xeb61, 0xeb61, PDF_CMAP_SINGLE, 20624 },
	{ 0xeb62, 0xeb62, PDF_CMAP_SINGLE, 9893 },
	{ 0xeb63, 0xeb6c, PDF_CMAP_RANGE, 20625 },
	{ 0xeb6d, 0xeb6d, PDF_CMAP_SINGLE, 8534 },
	{ 0xeb6e, 0xeb6f, PDF_CMAP_RANGE, 20635 },
	{ 0xeb70, 0xeb70, PDF_CMAP_SINGLE, 8516 },
	{ 0xeb71, 0xeb71, PDF_CMAP_SINGLE, 20637 },
	{ 0xeb72, 0xeb72, PDF_CMAP_SINGLE, 7825 },
	{ 0xeb73, 0xeb73, PDF_CMAP_SINGLE, 8791 },
	{ 0xeb74, 0xeb77, PDF_CMAP_RANGE, 20638 },
	{ 0xeb78, 0xeb78, PDF_CMAP_SINGLE, 8202 },
	{ 0xeb79, 0xeb79, PDF_CMAP_SINGLE, 8338 },
	{ 0xeb7a, 0xeb7e, PDF_CMAP_RANGE, 20642 },
	{ 0xeb80, 0xeb84, PDF_CMAP_RANGE, 20647 },
	{ 0xeb85, 0xeb85, PDF_CMAP_SINGLE, 8784 },
	{ 0xeb86, 0xeb89, PDF_CMAP_RANGE, 20652 },
	{ 0xeb8a, 0xeb8a, PDF_CMAP_SINGLE, 7875 },
	{ 0xeb8b, 0xeba0, PDF_CMAP_RANGE, 20656 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xec40, 0xec45, PDF_CMAP_RANGE, 20678 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8616 },
	{ 0xec47, 0xec55, PDF_CMAP_RANGE, 20684 },
	{ 0xec56, 0xec56, PDF_CMAP_SINGLE, 9741 },
	{ 0xec57, 0xec59, PDF_CMAP_RANGE, 20699 },
	{ 0xec5a, 0xec5a, PDF_CMAP_SINGLE, 9740 },
	{ 0xec5b, 0xec5b, PDF_CMAP_SINGLE, 20702 },
	{ 0xec5c, 0xec5c, PDF_CMAP_SINGLE, 9742 },
	{ 0xec5d, 0xec5f, PDF_CMAP_RANGE, 20703 },
	{ 0xec60, 0xec60, PDF_CMAP_SINGLE, 8242 },
	{ 0xec61, 0xec6d, PDF_CMAP_RANGE, 20706 },
	{ 0xec6e, 0xec6e, PDF_CMAP_SINGLE, 9739 },
	{ 0xec6f, 0xec75, PDF_CMAP_RANGE, 20719 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8899 },
	{ 0xec77, 0xec7e, PDF_CMAP_RANGE, 20726 },
	{ 0xec80, 0xec95, PDF_CMAP_RANGE, 20734 },
	{ 0xec96, 0xec96, PDF_CMAP_SINGLE, 7971 },
	{ 0xec97, 0xeca0, PDF_CMAP_RANGE, 20756 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xed40, 0xed45, PDF_CMAP_RANGE, 20766 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 9877 },
	{ 0xed47, 0xed57, PDF_CMAP_RANGE, 20772 },
	{ 0xed58, 0xed58, PDF_CMAP_SINGLE, 9822 },
	{ 0xed59, 0xed5d, PDF_CMAP_RANGE, 20789 },
	{ 0xed5e, 0xed5e, PDF_CMAP_SINGLE, 9821 },
	{ 0xed5f, 0xed60, PDF_CMAP_RANGE, 20794 },
	{ 0xed61, 0xed61, PDF_CMAP_SINGLE, 9874 },
	{ 0xed62, 0xed63, PDF_CMAP_RANGE, 20796 },
	{ 0xed64, 0xed64, PDF_CMAP_SINGLE, 9823 },
	{ 0xed65, 0xed65, PDF_CMAP_SINGLE, 20798 },
	{ 0xed66, 0xed66, PDF_CMAP_SINGLE, 8589 },
	{ 0xed67, 0xed67, PDF_CMAP_SINGLE, 8445 },
	{ 0xed68, 0xed6d, PDF_CMAP_RANGE, 20799 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 8000 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 20805 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 9317 },
	{ 0xed75, 0xed76, PDF_CMAP_RANGE, 20810 },
	{ 0xed77, 0xed77, PDF_CMAP_SINGLE, 9319 },
	{ 0xed78, 0xed78, PDF_CMAP_SINGLE, 20812 },
	{ 0xed79, 0xed79, PDF_CMAP_SINGLE, 9318 },
	{ 0xed7a, 0xed7e, PDF_CMAP_RANGE, 20813 },
	{ 0xed80, 0xed90, PDF_CMAP_RANGE, 20818 },
	{ 0xed91, 0xed91, PDF_CMAP_SINGLE, 8649 },
	{ 0xed92, 0xed92, PDF_CMAP_SINGLE, 20835 },
	{ 0xed93, 0xed93, PDF_CMAP_SINGLE, 8713 },
	{ 0xed94, 0xed94, PDF_CMAP_SINGLE, 7881 },
	{ 0xed95, 0xed95, PDF_CMAP_SINGLE, 8425 },
	{ 0xed96, 0xed96, PDF_CMAP_SINGLE, 20836 },
	{ 0xed97, 0xed97, PDF_CMAP_SINGLE, 8650 },
	{ 0xed98, 0xed98, PDF_CMAP_SINGLE, 8518 },
	{ 0xed99, 0xed99, PDF_CMAP_SINGLE, 9669 },
	{ 0xed9a, 0xed9a, PDF_CMAP_SINGLE, 8668 },
	{ 0xed9b, 0xed9b, PDF_CMAP_SINGLE, 20837 },
	{ 0xed9c, 0xed9c, PDF_CMAP_SINGLE, 9310 },
	{ 0xed9d, 0xed9d, PDF_CMAP_SINGLE, 20838 },
	{ 0xed9e, 0xed9e, PDF_CMAP_SINGLE, 8527 },
	{ 0xed9f, 0xed9f, PDF_CMAP_SINGLE, 20839 },
	{ 0xeda0, 0xeda0, PDF_CMAP_SINGLE, 9670 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xee40, 0xee40, PDF_CMAP_SINGLE, 9671 },
	{ 0xee41, 0xee41, PDF_CMAP_SINGLE, 8769 },
	{ 0xee42, 0xee42, PDF_CMAP_SINGLE, 8586 },
	{ 0xee43, 0xee43, PDF_CMAP_SINGLE, 7727 },
	{ 0xee44, 0xee44, PDF_CMAP_SINGLE, 7900 },
	{ 0xee45, 0xee47, PDF_CMAP_RANGE, 20840 },
	{ 0xee48, 0xee48, PDF_CMAP_SINGLE, 8383 },
	{ 0xee49, 0xee49, PDF_CMAP_SINGLE, 8244 },
	{ 0xee4a, 0xee4c, PDF_CMAP_RANGE, 20843 },
	{ 0xee4d, 0xee4d, PDF_CMAP_SINGLE, 9673 },
	{ 0xee4e, 0xee51, PDF_CMAP_RANGE, 20846 },
	{ 0xee52, 0xee52, PDF_CMAP_SINGLE, 9672 },
	{ 0xee53, 0xee54, PDF_CMAP_RANGE, 20850 },
	{ 0xee55, 0xee55, PDF_CMAP_SINGLE, 8718 },
	{ 0xee56, 0xee56, PDF_CMAP_SINGLE, 20852 },
	{ 0xee57, 0xee57, PDF_CMAP_SINGLE, 9675 },
	{ 0xee58, 0xee5d, PDF_CMAP_RANGE, 20853 },
	{ 0xee5e, 0xee5e, PDF_CMAP_SINGLE, 8573 },
	{ 0xee5f, 0xee60, PDF_CMAP_RANGE, 20859 },
	{ 0xee61, 0xee61, PDF_CMAP_SINGLE, 8062 },
	{ 0xee62, 0xee67, PDF_CMAP_RANGE, 20861 },
	{ 0xee68, 0xee68, PDF_CMAP_SINGLE, 9676 },
	{ 0xee69, 0xee69, PDF_CMAP_SINGLE, 8131 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 20867 },
	{ 0xee6c, 0xee6c, PDF_CMAP_SINGLE, 8377 },
	{ 0xee6d, 0xee6d, PDF_CMAP_SINGLE, 20869 },
	{ 0xee6e, 0xee6e, PDF_CMAP_SINGLE, 8577 },
	{ 0xee6f, 0xee76, PDF_CMAP_RANGE, 20870 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 8154 },
	{ 0xee78, 0xee7c, PDF_CMAP_RANGE, 20878 },
	{ 0xee7d, 0xee7d, PDF_CMAP_SINGLE, 8563 },
	{ 0xee7e, 0xee7e, PDF_CMAP_SINGLE, 7905 },
	{ 0xee80, 0xee80, PDF_CMAP_SINGLE, 9677 },
	{ 0xee81, 0xee84, PDF_CMAP_RANGE, 20883 },
	{ 0xee85, 0xee85, PDF_CMAP_SINGLE, 9678 },
	{ 0xee86, 0xee86, PDF_CMAP_SINGLE, 8694 },
	{ 0xee87, 0xee89, PDF_CMAP_RANGE, 20887 },
	{ 0xee8a, 0xee8a, PDF_CMAP_SINGLE, 8779 },
	{ 0xee8b, 0xee8b, PDF_CMAP_SINGLE, 9681 },
	{ 0xee8c, 0xee8c, PDF_CMAP_SINGLE, 20890 },
	{ 0xee8d, 0xee8d, PDF_CMAP_SINGLE, 7872 },
	{ 0xee8e, 0xee8f, PDF_CMAP_RANGE, 20891 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 8200 },
	{ 0xee91, 0xee93, PDF_CMAP_RANGE, 20893 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 9680 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 20896 },
	{ 0xee97, 0xee97, PDF_CMAP_SINGLE, 9682 },
	{ 0xee98, 0xee98, PDF_CMAP_SINGLE, 20898 },
	{ 0xee99, 0xee99, PDF_CMAP_SINGLE, 7978 },
	{ 0xee9a, 0xee9c, PDF_CMAP_RANGE, 20899 },
	{ 0xee9d, 0xee9d, PDF_CMAP_SINGLE, 7794 },
	{ 0xee9e, 0xee9e, PDF_CMAP_SINGLE, 9683 },
	{ 0xee9f, 0xeea0, PDF_CMAP_RANGE, 20902 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xef40, 0xef40, PDF_CMAP_SINGLE, 8638 },
	{ 0xef41, 0xef41, PDF_CMAP_SINGLE, 9684 },
	{ 0xef42, 0xef42, PDF_CMAP_SINGLE, 8260 },
	{ 0xef43, 0xef43, PDF_CMAP_SINGLE, 20904 },
	{ 0xef44, 0xef44, PDF_CMAP_SINGLE, 9679 },
	{ 0xef45, 0xef45, PDF_CMAP_SINGLE, 8435 },
	{ 0xef46, 0xef4b, PDF_CMAP_RANGE, 20905 },
	{ 0xef4c, 0xef4c, PDF_CMAP_SINGLE, 7936 },
	{ 0xef4d, 0xef51, PDF_CMAP_RANGE, 20911 },
	{ 0xef52, 0xef53, PDF_CMAP_RANGE, 9420 },
	{ 0xef54, 0xef54, PDF_CMAP_SINGLE, 20916 },
	{ 0xef55, 0xef55, PDF_CMAP_SINGLE, 9880 },
	{ 0xef56, 0xef56, PDF_CMAP_SINGLE, 20917 },
	{ 0xef57, 0xef57, PDF_CMAP_SINGLE, 9848 },
	{ 0xef58, 0xef59, PDF_CMAP_RANGE, 20918 },
	{ 0xef5a, 0xef5a, PDF_CMAP_SINGLE, 9422 },
	{ 0xef5b, 0xef5f, PDF_CMAP_RANGE, 20920 },
	{ 0xef60, 0xef60, PDF_CMAP_SINGLE, 9423 },
	{ 0xef61, 0xef67, PDF_CMAP_RANGE, 20925 },
	{ 0xef68, 0xef68, PDF_CMAP_SINGLE, 8376 },
	{ 0xef69, 0xef69, PDF_CMAP_SINGLE, 20932 },
	{ 0xef6a, 0xef6a, PDF_CMAP_SINGLE, 9424 },
	{ 0xef6b, 0xef6b, PDF_CMAP_SINGLE, 20933 },
	{ 0xef6c, 0xef6c, PDF_CMAP_SINGLE, 9425 },
	{ 0xef6d, 0xef76, PDF_CMAP_RANGE, 20934 },
	{ 0xef77, 0xef77, PDF_CMAP_SINGLE, 7924 },
	{ 0xef78, 0xef79, PDF_CMAP_RANGE, 20944 },
	{ 0xef7a, 0xef7a, PDF_CMAP_SINGLE, 9115 },
	{ 0xef7b, 0xef7b, PDF_CMAP_SINGLE, 20946 },
	{ 0xef7c, 0xef7c, PDF_CMAP_SINGLE, 9854 },
	{ 0xef7d, 0xef7e, PDF_CMAP_RANGE, 20947 },
	{ 0xef80, 0xef81, PDF_CMAP_RANGE, 20949 },
	{ 0xef82, 0xef82, PDF_CMAP_SINGLE, 9117 },
	{ 0xef83, 0xef84, PDF_CMAP_RANGE, 9119 },
	{ 0xef85, 0xef85, PDF_CMAP_SINGLE, 20951 },
	{ 0xef86, 0xef86, PDF_CMAP_SINGLE, 9121 },
	{ 0xef87, 0xef87, PDF_CMAP_SINGLE, 20952 },
	{ 0xef88, 0xef88, PDF_CMAP_SINGLE, 7921 },
	{ 0xef89, 0xef8a, PDF_CMAP_RANGE, 20953 },
	{ 0xef8b, 0xef8b, PDF_CMAP_SINGLE, 8734 },
	{ 0xef8c, 0xef8c, PDF_CMAP_SINGLE, 20955 },
	{ 0xef8d, 0xef8d, PDF_CMAP_SINGLE, 9122 },
	{ 0xef8e, 0xef94, PDF_CMAP_RANGE, 20956 },
	{ 0xef95, 0xef95, PDF_CMAP_SINGLE, 8523 },
	{ 0xef96, 0xef96, PDF_CMAP_SINGLE, 7734 },
	{ 0xef97, 0xef97, PDF_CMAP_SINGLE, 8501 },
	{ 0xef98, 0xef9b, PDF_CMAP_RANGE, 20963 },
	{ 0xef9c, 0xef9c, PDF_CMAP_SINGLE, 8109 },
	{ 0xef9d, 0xef9d, PDF_CMAP_SINGLE, 20967 },
	{ 0xef9e, 0xef9e, PDF_CMAP_SINGLE, 7763 },
	{ 0xef9f, 0xefa0, PDF_CMAP_RANGE, 20968 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf040, 0xf040, PDF_CMAP_SINGLE, 20970 },
	{ 0xf041, 0xf041, PDF_CMAP_SINGLE, 9123 },
	{ 0xf042, 0xf042, PDF_CMAP_SINGLE, 8707 },
	{ 0xf043, 0xf043, PDF_CMAP_SINGLE, 20971 },
	{ 0xf044, 0xf044, PDF_CMAP_SINGLE, 7911 },
	{ 0xf045, 0xf046, PDF_CMAP_RANGE, 20972 },
	{ 0xf047, 0xf047, PDF_CMAP_SINGLE, 9124 },
	{ 0xf048, 0xf048, PDF_CMAP_SINGLE, 8343 },
	{ 0xf049, 0xf049, PDF_CMAP_SINGLE, 7908 },
	{ 0xf04a, 0xf04d, PDF_CMAP_RANGE, 20974 },
	{ 0xf04e, 0xf04e, PDF_CMAP_SINGLE, 8760 },
	{ 0xf04f, 0xf050, PDF_CMAP_RANGE, 20978 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 9125 },
	{ 0xf052, 0xf053, PDF_CMAP_RANGE, 20980 },
	{ 0xf054, 0xf054, PDF_CMAP_SINGLE, 8090 },
	{ 0xf055, 0xf056, PDF_CMAP_RANGE, 20982 },
	{ 0xf057, 0xf057, PDF_CMAP_SINGLE, 8643 },
	{ 0xf058, 0xf05d, PDF_CMAP_RANGE, 20984 },
	{ 0xf05e, 0xf05e, PDF_CMAP_SINGLE, 7982 },
	{ 0xf05f, 0xf067, PDF_CMAP_RANGE, 20990 },
	{ 0xf068, 0xf068, PDF_CMAP_SINGLE, 9116 },
	{ 0xf069, 0xf06b, PDF_CMAP_RANGE, 20999 },
	{ 0xf06c, 0xf06c, PDF_CMAP_SINGLE, 9126 },
	{ 0xf06d, 0xf070, PDF_CMAP_RANGE, 21002 },
	{ 0xf071, 0xf071, PDF_CMAP_SINGLE, 9118 },
	{ 0xf072, 0xf072, PDF_CMAP_SINGLE, 21006 },
	{ 0xf073, 0xf073, PDF_CMAP_SINGLE, 8245 },
	{ 0xf074, 0xf074, PDF_CMAP_SINGLE, 9127 },
	{ 0xf075, 0xf077, PDF_CMAP_RANGE, 21007 },
	{ 0xf078, 0xf078, PDF_CMAP_SINGLE, 9128 },
	{ 0xf079, 0xf079, PDF_CMAP_SINGLE, 21010 },
	{ 0xf07a, 0xf07a, PDF_CMAP_SINGLE, 8309 },
	{ 0xf07b, 0xf07c, PDF_CMAP_RANGE, 21011 },
	{ 0xf07d, 0xf07e, PDF_CMAP_RANGE, 9129 },
	{ 0xf080, 0xf080, PDF_CMAP_SINGLE, 9131 },
	{ 0xf081, 0xf081, PDF_CMAP_SINGLE, 8171 },
	{ 0xf082, 0xf082, PDF_CMAP_SINGLE, 9132 },
	{ 0xf083, 0xf086, PDF_CMAP_RANGE, 21013 },
	{ 0xf087, 0xf087, PDF_CMAP_SINGLE, 8042 },
	{ 0xf088, 0xf088, PDF_CMAP_SINGLE, 8441 },
	{ 0xf089, 0xf08a, PDF_CMAP_RANGE, 21017 },
	{ 0xf08b, 0xf08b, PDF_CMAP_SINGLE, 9830 },
	{ 0xf08c, 0xf08f, PDF_CMAP_RANGE, 21019 },
	{ 0xf090, 0xf090, PDF_CMAP_SINGLE, 9831 },
	{ 0xf091, 0xf091, PDF_CMAP_SINGLE, 21023 },
	{ 0xf092, 0xf092, PDF_CMAP_SINGLE, 7788 },
	{ 0xf093, 0xf095, PDF_CMAP_RANGE, 21024 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 9133 },
	{ 0xf097, 0xf0a0, PDF_CMAP_RANGE, 21027 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf140, 0xf151, PDF_CMAP_RANGE, 21037 },
	{ 0xf152, 0xf152, PDF_CMAP_SINGLE, 8301 },
	{ 0xf153, 0xf153, PDF_CMAP_SINGLE, 8770 },
	{ 0xf154, 0xf154, PDF_CMAP_SINGLE, 7938 },
	{ 0xf155, 0xf156, PDF_CMAP_RANGE, 21055 },
	{ 0xf157, 0xf157, PDF_CMAP_SINGLE, 8579 },
	{ 0xf158, 0xf158, PDF_CMAP_SINGLE, 21057 },
	{ 0xf159, 0xf159, PDF_CMAP_SINGLE, 7813 },
	{ 0xf15a, 0xf15a, PDF_CMAP_SINGLE, 8681 },
	{ 0xf15b, 0xf166, PDF_CMAP_RANGE, 21058 },
	{ 0xf167, 0xf167, PDF_CMAP_SINGLE, 7767 },
	{ 0xf168, 0xf175, PDF_CMAP_RANGE, 21070 },
	{ 0xf176, 0xf176, PDF_CMAP_SINGLE, 8869 },
	{ 0xf177, 0xf177, PDF_CMAP_SINGLE, 9223 },
	{ 0xf178, 0xf178, PDF_CMAP_SINGLE, 8138 },
	{ 0xf179, 0xf179, PDF_CMAP_SINGLE, 21084 },
	{ 0xf17a, 0xf17a, PDF_CMAP_SINGLE, 9218 },
	{ 0xf17b, 0xf17b, PDF_CMAP_SINGLE, 8066 },
	{ 0xf17c, 0xf17d, PDF_CMAP_RANGE, 21085 },
	{ 0xf17e, 0xf17e, PDF_CMAP_SINGLE, 9224 },
	{ 0xf180, 0xf180, PDF_CMAP_SINGLE, 9220 },
	{ 0xf181, 0xf181, PDF_CMAP_SINGLE, 21087 },
	{ 0xf182, 0xf182, PDF_CMAP_SINGLE, 8497 },
	{ 0xf183, 0xf183, PDF_CMAP_SINGLE, 21088 },
	{ 0xf184, 0xf184, PDF_CMAP_SINGLE, 8580 },
	{ 0xf185, 0xf185, PDF_CMAP_SINGLE, 21089 },
	{ 0xf186, 0xf186, PDF_CMAP_SINGLE, 9219 },
	{ 0xf187, 0xf187, PDF_CMAP_SINGLE, 21090 },
	{ 0xf188, 0xf188, PDF_CMAP_SINGLE, 8302 },
	{ 0xf189, 0xf189, PDF_CMAP_SINGLE, 9227 },
	{ 0xf18a, 0xf193, PDF_CMAP_RANGE, 21091 },
	{ 0xf194, 0xf194, PDF_CMAP_SINGLE, 7999 },
	{ 0xf195, 0xf197, PDF_CMAP_RANGE, 21101 },
	{ 0xf198, 0xf198, PDF_CMAP_SINGLE, 8295 },
	{ 0xf199, 0xf1a0, PDF_CMAP_RANGE, 21104 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf240, 0xf244, PDF_CMAP_RANGE, 21112 },
	{ 0xf245, 0xf245, PDF_CMAP_SINGLE, 8151 },
	{ 0xf246, 0xf246, PDF_CMAP_SINGLE, 21117 },
	{ 0xf247, 0xf247, PDF_CMAP_SINGLE, 7811 },
	{ 0xf248, 0xf24a, PDF_CMAP_RANGE, 21118 },
	{ 0xf24b, 0xf24b, PDF_CMAP_SINGLE, 9231 },
	{ 0xf24c, 0xf252, PDF_CMAP_RANGE, 21121 },
	{ 0xf253, 0xf253, PDF_CMAP_SINGLE, 9230 },
	{ 0xf254, 0xf254, PDF_CMAP_SINGLE, 8391 },
	{ 0xf255, 0xf255, PDF_CMAP_SINGLE, 9229 },
	{ 0xf256, 0xf25b, PDF_CMAP_RANGE, 21128 },
	{ 0xf25c, 0xf25c, PDF_CMAP_SINGLE, 9234 },
	{ 0xf25d, 0xf25e, PDF_CMAP_RANGE, 21134 },
	{ 0xf25f, 0xf25f, PDF_CMAP_SINGLE, 8375 },
	{ 0xf260, 0xf270, PDF_CMAP_RANGE, 21136 },
	{ 0xf271, 0xf271, PDF_CMAP_SINGLE, 9200 },
	{ 0xf272, 0xf272, PDF_CMAP_SINGLE, 21153 },
	{ 0xf273, 0xf273, PDF_CMAP_SINGLE, 9233 },
	{ 0xf274, 0xf274, PDF_CMAP_SINGLE, 9236 },
	{ 0xf275, 0xf275, PDF_CMAP_SINGLE, 21154 },
	{ 0xf276, 0xf276, PDF_CMAP_SINGLE, 8560 },
	{ 0xf277, 0xf27b, PDF_CMAP_RANGE, 21155 },
	{ 0xf27c, 0xf27c, PDF_CMAP_SINGLE, 9221 },
	{ 0xf27d, 0xf27d, PDF_CMAP_SINGLE, 8460 },
	{ 0xf27e, 0xf27e, PDF_CMAP_SINGLE, 9237 },
	{ 0xf280, 0xf284, PDF_CMAP_RANGE, 21160 },
	{ 0xf285, 0xf285, PDF_CMAP_SINGLE, 8294 },
	{ 0xf286, 0xf286, PDF_CMAP_SINGLE, 21165 },
	{ 0xf287, 0xf287, PDF_CMAP_SINGLE, 9042 },
	{ 0xf288, 0xf288, PDF_CMAP_SINGLE, 9235 },
	{ 0xf289, 0xf289, PDF_CMAP_SINGLE, 9232 },
	{ 0xf28a, 0xf28b, PDF_CMAP_RANGE, 9238 },
	{ 0xf28c, 0xf28c, PDF_CMAP_SINGLE, 8433 },
	{ 0xf28d, 0xf290, PDF_CMAP_RANGE, 21166 },
	{ 0xf291, 0xf291, PDF_CMAP_SINGLE, 9226 },
	{ 0xf292, 0xf293, PDF_CMAP_RANGE, 21170 },
	{ 0xf294, 0xf294, PDF_CMAP_SINGLE, 9225 },
	{ 0xf295, 0xf295, PDF_CMAP_SINGLE, 21172 },
	{ 0xf296, 0xf296, PDF_CMAP_SINGLE, 9240 },
	{ 0xf297, 0xf29b, PDF_CMAP_RANGE, 21173 },
	{ 0xf29c, 0xf29c, PDF_CMAP_SINGLE, 8103 },
	{ 0xf29d, 0xf29d, PDF_CMAP_SINGLE, 21178 },
	{ 0xf29e, 0xf29e, PDF_CMAP_SINGLE, 8700 },
	{ 0xf29f, 0xf2a0, PDF_CMAP_RANGE, 21179 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf340, 0xf340, PDF_CMAP_SINGLE, 8129 },
	{ 0xf341, 0xf341, PDF_CMAP_SINGLE, 9222 },
	{ 0xf342, 0xf344, PDF_CMAP_RANGE, 21181 },
	{ 0xf345, 0xf345, PDF_CMAP_SINGLE, 8860 },
	{ 0xf346, 0xf347, PDF_CMAP_RANGE, 21184 },
	{ 0xf348, 0xf348, PDF_CMAP_SINGLE, 8270 },
	{ 0xf349, 0xf349, PDF_CMAP_SINGLE, 21186 },
	{ 0xf34a, 0xf34a, PDF_CMAP_SINGLE, 9242 },
	{ 0xf34b, 0xf34b, PDF_CMAP_SINGLE, 9241 },
	{ 0xf34c, 0xf34f, PDF_CMAP_RANGE, 21187 },
	{ 0xf350, 0xf350, PDF_CMAP_SINGLE, 9228 },
	{ 0xf351, 0xf360, PDF_CMAP_RANGE, 21191 },
	{ 0xf361, 0xf361, PDF_CMAP_SINGLE, 7721 },
	{ 0xf362, 0xf373, PDF_CMAP_RANGE, 21207 },
	{ 0xf374, 0xf374, PDF_CMAP_SINGLE, 9825 },
	{ 0xf375, 0xf375, PDF_CMAP_SINGLE, 21225 },
	{ 0xf376, 0xf376, PDF_CMAP_SINGLE, 9892 },
	{ 0xf377, 0xf377, PDF_CMAP_SINGLE, 8564 },
	{ 0xf378, 0xf378, PDF_CMAP_SINGLE, 9827 },
	{ 0xf379, 0xf379, PDF_CMAP_SINGLE, 9826 },
	{ 0xf37a, 0xf37e, PDF_CMAP_RANGE, 21226 },
	{ 0xf380, 0xf38b, PDF_CMAP_RANGE, 21231 },
	{ 0xf38c, 0xf38c, PDF_CMAP_SINGLE, 9845 },
	{ 0xf38d, 0xf39f, PDF_CMAP_RANGE, 21243 },
	{ 0xf3a0, 0xf3a0, PDF_CMAP_SINGLE, 8524 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf440, 0xf444, PDF_CMAP_RANGE, 21262 },
	{ 0xf445, 0xf445, PDF_CMAP_SINGLE, 9850 },
	{ 0xf446, 0xf44f, PDF_CMAP_RANGE, 21267 },
	{ 0xf450, 0xf450, PDF_CMAP_SINGLE, 9888 },
	{ 0xf451, 0xf456, PDF_CMAP_RANGE, 21277 },
	{ 0xf457, 0xf457, PDF_CMAP_SINGLE, 9832 },
	{ 0xf458, 0xf458, PDF_CMAP_SINGLE, 21283 },
	{ 0xf459, 0xf459, PDF_CMAP_SINGLE, 7888 },
	{ 0xf45a, 0xf45a, PDF_CMAP_SINGLE, 21284 },
	{ 0xf45b, 0xf45b, PDF_CMAP_SINGLE, 8342 },
	{ 0xf45c, 0xf45c, PDF_CMAP_SINGLE, 21285 },
	{ 0xf45d, 0xf45d, PDF_CMAP_SINGLE, 9164 },
	{ 0xf45e, 0xf461, PDF_CMAP_RANGE, 21286 },
	{ 0xf462, 0xf462, PDF_CMAP_SINGLE, 9160 },
	{ 0xf463, 0xf463, PDF_CMAP_SINGLE, 21290 },
	{ 0xf464, 0xf464, PDF_CMAP_SINGLE, 8766 },
	{ 0xf465, 0xf474, PDF_CMAP_RANGE, 21291 },
	{ 0xf475, 0xf475, PDF_CMAP_SINGLE, 9829 },
	{ 0xf476, 0xf47b, PDF_CMAP_RANGE, 21307 },
	{ 0xf47c, 0xf47c, PDF_CMAP_SINGLE, 9828 },
	{ 0xf47d, 0xf47d, PDF_CMAP_SINGLE, 21313 },
	{ 0xf47e, 0xf47e, PDF_CMAP_SINGLE, 8761 },
	{ 0xf480, 0xf493, PDF_CMAP_RANGE, 21314 },
	{ 0xf494, 0xf494, PDF_CMAP_SINGLE, 8266 },
	{ 0xf495, 0xf498, PDF_CMAP_RANGE, 21334 },
	{ 0xf499, 0xf499, PDF_CMAP_SINGLE, 9759 },
	{ 0xf49a, 0xf49b, PDF_CMAP_RANGE, 21338 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 9758 },
	{ 0xf49d, 0xf4a0, PDF_CMAP_RANGE, 21340 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf540, 0xf544, PDF_CMAP_RANGE, 21344 },
	{ 0xf545, 0xf545, PDF_CMAP_SINGLE, 9760 },
	{ 0xf546, 0xf546, PDF_CMAP_SINGLE, 21349 },
	{ 0xf547, 0xf547, PDF_CMAP_SINGLE, 9761 },
	{ 0xf548, 0xf551, PDF_CMAP_RANGE, 21350 },
	{ 0xf552, 0xf552, PDF_CMAP_SINGLE, 9762 },
	{ 0xf553, 0xf553, PDF_CMAP_SINGLE, 21360 },
	{ 0xf554, 0xf554, PDF_CMAP_SINGLE, 9767 },
	{ 0xf555, 0xf555, PDF_CMAP_SINGLE, 7737 },
	{ 0xf556, 0xf556, PDF_CMAP_SINGLE, 9765 },
	{ 0xf557, 0xf55d, PDF_CMAP_RANGE, 21361 },
	{ 0xf55e, 0xf55e, PDF_CMAP_SINGLE, 9769 },
	{ 0xf55f, 0xf560, PDF_CMAP_RANGE, 21368 },
	{ 0xf561, 0xf561, PDF_CMAP_SINGLE, 9774 },
	{ 0xf562, 0xf562, PDF_CMAP_SINGLE, 9771 },
	{ 0xf563, 0xf56d, PDF_CMAP_RANGE, 21370 },
	{ 0xf56e, 0xf56e, PDF_CMAP_SINGLE, 9770 },
	{ 0xf56f, 0xf56f, PDF_CMAP_SINGLE, 9773 },
	{ 0xf570, 0xf570, PDF_CMAP_SINGLE, 21381 },
	{ 0xf571, 0xf571, PDF_CMAP_SINGLE, 9768 },
	{ 0xf572, 0xf572, PDF_CMAP_SINGLE, 8633 },
	{ 0xf573, 0xf57e, PDF_CMAP_RANGE, 21382 },
	{ 0xf580, 0xf584, PDF_CMAP_RANGE, 21394 },
	{ 0xf585, 0xf585, PDF_CMAP_SINGLE, 9782 },
	{ 0xf586, 0xf586, PDF_CMAP_SINGLE, 9776 },
	{ 0xf587, 0xf58b, PDF_CMAP_RANGE, 21399 },
	{ 0xf58c, 0xf58c, PDF_CMAP_SINGLE, 9784 },
	{ 0xf58d, 0xf58d, PDF_CMAP_SINGLE, 21404 },
	{ 0xf58e, 0xf58e, PDF_CMAP_SINGLE, 8205 },
	{ 0xf58f, 0xf58f, PDF_CMAP_SINGLE, 9783 },
	{ 0xf590, 0xf598, PDF_CMAP_RANGE, 21405 },
	{ 0xf599, 0xf599, PDF_CMAP_SINGLE, 9797 },
	{ 0xf59a, 0xf59a, PDF_CMAP_SINGLE, 21414 },
	{ 0xf59b, 0xf59b, PDF_CMAP_SINGLE, 9786 },
	{ 0xf59c, 0xf59f, PDF_CMAP_RANGE, 21415 },
	{ 0xf5a0, 0xf5a0, PDF_CMAP_SINGLE, 9795 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf640, 0xf640, PDF_CMAP_SINGLE, 21419 },
	{ 0xf641, 0xf641, PDF_CMAP_SINGLE, 9792 },
	{ 0xf642, 0xf644, PDF_CMAP_RANGE, 21420 },
	{ 0xf645, 0xf645, PDF_CMAP_SINGLE, 9789 },
	{ 0xf646, 0xf646, PDF_CMAP_SINGLE, 9793 },
	{ 0xf647, 0xf647, PDF_CMAP_SINGLE, 21423 },
	{ 0xf648, 0xf648, PDF_CMAP_SINGLE, 9790 },
	{ 0xf649, 0xf64a, PDF_CMAP_RANGE, 21424 },
	{ 0xf64b, 0xf64b, PDF_CMAP_SINGLE, 9791 },
	{ 0xf64c, 0xf64c, PDF_CMAP_SINGLE, 8128 },
	{ 0xf64d, 0xf64d, PDF_CMAP_SINGLE, 21426 },
	{ 0xf64e, 0xf64f, PDF_CMAP_RANGE, 9787 },
	{ 0xf650, 0xf653, PDF_CMAP_RANGE, 21427 },
	{ 0xf654, 0xf654, PDF_CMAP_SINGLE, 9794 },
	{ 0xf655, 0xf657, PDF_CMAP_RANGE, 21431 },
	{ 0xf658, 0xf658, PDF_CMAP_SINGLE, 9796 },
	{ 0xf659, 0xf660, PDF_CMAP_RANGE, 21434 },
	{ 0xf661, 0xf661, PDF_CMAP_SINGLE, 9785 },
	{ 0xf662, 0xf662, PDF_CMAP_SINGLE, 21442 },
	{ 0xf663, 0xf663, PDF_CMAP_SINGLE, 9804 },
	{ 0xf664, 0xf66b, PDF_CMAP_RANGE, 21443 },
	{ 0xf66c, 0xf66c, PDF_CMAP_SINGLE, 9799 },
	{ 0xf66d, 0xf66d, PDF_CMAP_SINGLE, 9803 },
	{ 0xf66e, 0xf670, PDF_CMAP_RANGE, 21451 },
	{ 0xf671, 0xf671, PDF_CMAP_SINGLE, 9801 },
	{ 0xf672, 0xf673, PDF_CMAP_RANGE, 21454 },
	{ 0xf674, 0xf674, PDF_CMAP_SINGLE, 9800 },
	{ 0xf675, 0xf675, PDF_CMAP_SINGLE, 21456 },
	{ 0xf676, 0xf676, PDF_CMAP_SINGLE, 9802 },
	{ 0xf677, 0xf677, PDF_CMAP_SINGLE, 8456 },
	{ 0xf678, 0xf67e, PDF_CMAP_RANGE, 21457 },
	{ 0xf680, 0xf684, PDF_CMAP_RANGE, 21464 },
	{ 0xf685, 0xf685, PDF_CMAP_SINGLE, 9805 },
	{ 0xf686, 0xf687, PDF_CMAP_RANGE, 21469 },
	{ 0xf688, 0xf688, PDF_CMAP_SINGLE, 9780 },
	{ 0xf689, 0xf689, PDF_CMAP_SINGLE, 21471 },
	{ 0xf68a, 0xf68a, PDF_CMAP_SINGLE, 9809 },
	{ 0xf68b, 0xf68c, PDF_CMAP_RANGE, 21472 },
	{ 0xf68d, 0xf68d, PDF_CMAP_SINGLE, 9808 },
	{ 0xf68e, 0xf68e, PDF_CMAP_SINGLE, 9810 },
	{ 0xf68f, 0xf691, PDF_CMAP_RANGE, 21474 },
	{ 0xf692, 0xf692, PDF_CMAP_SINGLE, 9807 },
	{ 0xf693, 0xf695, PDF_CMAP_RANGE, 21477 },
	{ 0xf696, 0xf696, PDF_CMAP_SINGLE, 9778 },
	{ 0xf697, 0xf697, PDF_CMAP_SINGLE, 9806 },
	{ 0xf698, 0xf698, PDF_CMAP_SINGLE, 9811 },
	{ 0xf699, 0xf699, PDF_CMAP_SINGLE, 21480 },
	{ 0xf69a, 0xf69a, PDF_CMAP_SINGLE, 9815 },
	{ 0xf69b, 0xf69b, PDF_CMAP_SINGLE, 21481 },
	{ 0xf69c, 0xf69c, PDF_CMAP_SINGLE, 9781 },
	{ 0xf69d, 0xf69d, PDF_CMAP_SINGLE, 21482 },
	{ 0xf69e, 0xf69e, PDF_CMAP_SINGLE, 9779 },
	{ 0xf69f, 0xf69f, PDF_CMAP_SINGLE, 21483 },
	{ 0xf6a0, 0xf6a0, PDF_CMAP_SINGLE, 9814 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf740, 0xf741, PDF_CMAP_RANGE, 21484 },
	{ 0xf742, 0xf742, PDF_CMAP_SINGLE, 9812 },
	{ 0xf743, 0xf748, PDF_CMAP_RANGE, 21486 },
	{ 0xf749, 0xf749, PDF_CMAP_SINGLE, 9816 },
	{ 0xf74a, 0xf74b, PDF_CMAP_RANGE, 21492 },
	{ 0xf74c, 0xf74c, PDF_CMAP_SINGLE, 9813 },
	{ 0xf74d, 0xf74d, PDF_CMAP_SINGLE, 7757 },
	{ 0xf74e, 0xf755, PDF_CMAP_RANGE, 21494 },
	{ 0xf756, 0xf756, PDF_CMAP_SINGLE, 9819 },
	{ 0xf757, 0xf757, PDF_CMAP_SINGLE, 21502 },
	{ 0xf758, 0xf758, PDF_CMAP_SINGLE, 9818 },
	{ 0xf759, 0xf759, PDF_CMAP_SINGLE, 21503 },
	{ 0xf75a, 0xf75a, PDF_CMAP_SINGLE, 9817 },
	{ 0xf75b, 0xf75b, PDF_CMAP_SINGLE, 8238 },
	{ 0xf75c, 0xf75c, PDF_CMAP_SINGLE, 9775 },
	{ 0xf75d, 0xf760, PDF_CMAP_RANGE, 21504 },
	{ 0xf761, 0xf761, PDF_CMAP_SINGLE, 9798 },
	{ 0xf762, 0xf762, PDF_CMAP_SINGLE, 21508 },
	{ 0xf763, 0xf763, PDF_CMAP_SINGLE, 9766 },
	{ 0xf764, 0xf76a, PDF_CMAP_RANGE, 21509 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 9820 },
	{ 0xf76c, 0xf770, PDF_CMAP_RANGE, 21516 },
	{ 0xf771, 0xf771, PDF_CMAP_SINGLE, 9772 },
	{ 0xf772, 0xf77b, PDF_CMAP_RANGE, 21521 },
	{ 0xf77c, 0xf77c, PDF_CMAP_SINGLE, 9763 },
	{ 0xf77d, 0xf77d, PDF_CMAP_SINGLE, 21531 },
	{ 0xf77e, 0xf77e, PDF_CMAP_SINGLE, 9777 },
	{ 0xf780, 0xf7a0, PDF_CMAP_RANGE, 21532 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
	{ 0xf840, 0xf841, PDF_CMAP_RANGE, 21565 },
	{ 0xf842, 0xf842, PDF_CMAP_SINGLE, 8348 },
	{ 0xf843, 0xf845, PDF_CMAP_RANGE, 21567 },
	{ 0xf846, 0xf846, PDF_CMAP_SINGLE, 9600 },
	{ 0xf847, 0xf848, PDF_CMAP_RANGE, 21570 },
	{ 0xf849, 0xf849, PDF_CMAP_SINGLE, 8924 },
	{ 0xf84a, 0xf84f, PDF_CMAP_RANGE, 21572 },
	{ 0xf850, 0xf850, PDF_CMAP_SINGLE, 7941 },
	{ 0xf851, 0xf851, PDF_CMAP_SINGLE, 8331 },
	{ 0xf852, 0xf852, PDF_CMAP_SINGLE, 21578 },
	{ 0xf853, 0xf853, PDF_CMAP_SINGLE, 9601 },
	{ 0xf854, 0xf862, PDF_CMAP_RANGE, 21579 },
	{ 0xf863, 0xf863, PDF_CMAP_SINGLE, 9603 },
	{ 0xf864, 0xf864, PDF_CMAP_SINGLE, 9602 },
	{ 0xf865, 0xf865, PDF_CMAP_SINGLE, 21594 },
	{ 0xf866, 0xf866, PDF_CMAP_SINGLE, 8686 },
	{ 0xf867, 0xf871, PDF_CMAP_RANGE, 21595 },
	{ 0xf872, 0xf872, PDF_CMAP_SINGLE, 8578 },
	{ 0xf873, 0xf877, PDF_CMAP_RANGE, 21606 },
	{ 0xf878, 0xf878, PDF_CMAP_SINGLE, 8771 },
	{ 0xf879, 0xf879, PDF_CMAP_SINGLE, 21611 },
	{ 0xf87a, 0xf87a, PDF_CMAP_SINGLE, 9607 },
	{ 0xf87b, 0xf87b, PDF_CMAP_SINGLE, 21612 },
	{ 0xf87c, 0xf87c, PDF_CMAP_SINGLE, 9608 },
	{ 0xf87d, 0xf87e, PDF_CMAP_RANGE, 21613 },
	{ 0xf880, 0xf880, PDF_CMAP_SINGLE, 21615 },
	{ 0xf881, 0xf881, PDF_CMAP_SINGLE, 9604 },
	{ 0xf882, 0xf883, PDF_CMAP_RANGE, 21616 },
	{ 0xf884, 0xf884, PDF_CMAP_SINGLE, 8701 },
	{ 0xf885, 0xf885, PDF_CMAP_SINGLE, 21618 },
	{ 0xf886, 0xf886, PDF_CMAP_SINGLE, 8687 },
	{ 0xf887, 0xf88c, PDF_CMAP_RANGE, 21619 },
	{ 0xf88d, 0xf88d, PDF_CMAP_SINGLE, 9610 },
	{ 0xf88e, 0xf88e, PDF_CMAP_SINGLE, 9612 },
	{ 0xf88f, 0xf898, PDF_CMAP_RANGE, 21625 },
	{ 0xf899, 0xf899, PDF_CMAP_SINGLE, 8007 },
	{ 0xf89a, 0xf89c, PDF_CMAP_RANGE, 21635 },
	{ 0xf89d, 0xf89d, PDF_CMAP_SINGLE, 7965 },
	{ 0xf89e, 0xf89f, PDF_CMAP_RANGE, 21638 },
	{ 0xf8a0, 0xf8a0, PDF_CMAP_SINGLE, 9613 },
	{ 0xf940, 0xf94d, PDF_CMAP_RANGE, 21640 },
	{ 0xf94e, 0xf94e, PDF_CMAP_SINGLE, 8144 },
	{ 0xf94f, 0xf94f, PDF_CMAP_SINGLE, 9618 },
	{ 0xf950, 0xf950, PDF_CMAP_SINGLE, 9615 },
	{ 0xf951, 0xf958, PDF_CMAP_RANGE, 21654 },
	{ 0xf959, 0xf959, PDF_CMAP_SINGLE, 9620 },
	{ 0xf95a, 0xf95a, PDF_CMAP_SINGLE, 7904 },
	{ 0xf95b, 0xf95c, PDF_CMAP_RANGE, 21662 },
	{ 0xf95d, 0xf95d, PDF_CMAP_SINGLE, 9617 },
	{ 0xf95e, 0xf95e, PDF_CMAP_SINGLE, 9621 },
	{ 0xf95f, 0xf966, PDF_CMAP_RANGE, 21664 },
	{ 0xf967, 0xf967, PDF_CMAP_SINGLE, 9623 },
	{ 0xf968, 0xf968, PDF_CMAP_SINGLE, 21672 },
	{ 0xf969, 0xf969, PDF_CMAP_SINGLE, 8374 },
	{ 0xf96a, 0xf96b, PDF_CMAP_RANGE, 21673 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 9624 },
	{ 0xf96d, 0xf96e, PDF_CMAP_RANGE, 21675 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 8438 },
	{ 0xf970, 0xf97e, PDF_CMAP_RANGE, 21677 },
	{ 0xf980, 0xf984, PDF_CMAP_RANGE, 21692 },
	{ 0xf985, 0xf985, PDF_CMAP_SINGLE, 9605 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 21697 },
	{ 0xf987, 0xf987, PDF_CMAP_SINGLE, 9625 },
	{ 0xf988, 0xf990, PDF_CMAP_RANGE, 21698 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 9622 },
	{ 0xf992, 0xf995, PDF_CMAP_RANGE, 21707 },
	{ 0xf996, 0xf996, PDF_CMAP_SINGLE, 9626 },
	{ 0xf997, 0xf997, PDF_CMAP_SINGLE, 21711 },
	{ 0xf998, 0xf998, PDF_CMAP_SINGLE, 9627 },
	{ 0xf999, 0xf9a0, PDF_CMAP_RANGE, 21712 },
	{ 0xfa40, 0xfa41, PDF_CMAP_RANGE, 21720 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 9629 },
	{ 0xfa43, 0xfa45, PDF_CMAP_RANGE, 21722 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 9630 },
	{ 0xfa47, 0xfa4b, PDF_CMAP_RANGE, 21725 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 9036 },
	{ 0xfa4d, 0xfa50, PDF_CMAP_RANGE, 21730 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 8004 },
	{ 0xfa52, 0xfa57, PDF_CMAP_RANGE, 21734 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 9824 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 9632 },
	{ 0xfa5a, 0xfa5c, PDF_CMAP_RANGE, 21740 },
	{ 0xfa5d, 0xfa5d, PDF_CMAP_SINGLE, 9628 },
	{ 0xfa5e, 0xfa5e, PDF_CMAP_SINGLE, 21743 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 9631 },
	{ 0xfa60, 0xfa60, PDF_CMAP_SINGLE, 21744 },
	{ 0xfa61, 0xfa61, PDF_CMAP_SINGLE, 8044 },
	{ 0xfa62, 0xfa6f, PDF_CMAP_RANGE, 21745 },
	{ 0xfa70, 0xfa70, PDF_CMAP_SINGLE, 9634 },
	{ 0xfa71, 0xfa73, PDF_CMAP_RANGE, 21759 },
	{ 0xfa74, 0xfa74, PDF_CMAP_SINGLE, 8366 },
	{ 0xfa75, 0xfa75, PDF_CMAP_SINGLE, 21762 },
	{ 0xfa76, 0xfa76, PDF_CMAP_SINGLE, 9611 },
	{ 0xfa77, 0xfa77, PDF_CMAP_SINGLE, 9635 },
	{ 0xfa78, 0xfa7e, PDF_CMAP_RANGE, 21763 },
	{ 0xfa80, 0xfa82, PDF_CMAP_RANGE, 21770 },
	{ 0xfa83, 0xfa83, PDF_CMAP_SINGLE, 9609 },
	{ 0xfa84, 0xfa84, PDF_CMAP_SINGLE, 9637 },
	{ 0xfa85, 0xfa8c, PDF_CMAP_RANGE, 21773 },
	{ 0xfa8d, 0xfa8d, PDF_CMAP_SINGLE, 9636 },
	{ 0xfa8e, 0xfa8f, PDF_CMAP_RANGE, 21781 },
	{ 0xfa90, 0xfa90, PDF_CMAP_SINGLE, 9638 },
	{ 0xfa91, 0xfa91, PDF_CMAP_SINGLE, 9619 },
	{ 0xfa92, 0xfa95, PDF_CMAP_RANGE, 21783 },
	{ 0xfa96, 0xfa96, PDF_CMAP_SINGLE, 9639 },
	{ 0xfa97, 0xfa97, PDF_CMAP_SINGLE, 8738 },
	{ 0xfa98, 0xfa98, PDF_CMAP_SINGLE, 9641 },
	{ 0xfa99, 0xfaa0, PDF_CMAP_RANGE, 21787 },
	{ 0xfb40, 0xfb48, PDF_CMAP_RANGE, 21795 },
	{ 0xfb49, 0xfb49, PDF_CMAP_SINGLE, 9640 },
	{ 0xfb4a, 0xfb51, PDF_CMAP_RANGE, 21804 },
	{ 0xfb52, 0xfb52, PDF_CMAP_SINGLE, 9606 },
	{ 0xfb53, 0xfb56, PDF_CMAP_RANGE, 21812 },
	{ 0xfb57, 0xfb57, PDF_CMAP_SINGLE, 9633 },
	{ 0xfb58, 0xfb58, PDF_CMAP_SINGLE, 9642 },
	{ 0xfb59, 0xfb59, PDF_CMAP_SINGLE, 21816 },
	{ 0xfb5a, 0xfb5a, PDF_CMAP_SINGLE, 9616 },
	{ 0xfb5b, 0xfb5b, PDF_CMAP_SINGLE, 9614 },
	{ 0xfb5c, 0xfb74, PDF_CMAP_RANGE, 21817 },
	{ 0xfb75, 0xfb75, PDF_CMAP_SINGLE, 9864 },
	{ 0xfb76, 0xfb78, PDF_CMAP_RANGE, 21842 },
	{ 0xfb79, 0xfb79, PDF_CMAP_SINGLE, 9886 },
	{ 0xfb7a, 0xfb7a, PDF_CMAP_SINGLE, 9723 },
	{ 0xfb7b, 0xfb7b, PDF_CMAP_SINGLE, 21845 },
	{ 0xfb7c, 0xfb7c, PDF_CMAP_SINGLE, 8076 },
	{ 0xfb7d, 0xfb7d, PDF_CMAP_SINGLE, 8692 },
	{ 0xfb7e, 0xfb7e, PDF_CMAP_SINGLE, 21846 },
	{ 0xfb80, 0xfb8f, PDF_CMAP_RANGE, 21847 },
	{ 0xfb90, 0xfb90, PDF_CMAP_SINGLE, 8207 },
	{ 0xfb91, 0xfb9b, PDF_CMAP_RANGE, 21863 },
	{ 0xfb9c, 0xfb9c, PDF_CMAP_SINGLE, 8305 },
	{ 0xfb9d, 0xfb9e, PDF_CMAP_RANGE, 21874 },
	{ 0xfb9f, 0xfb9f, PDF_CMAP_SINGLE, 9719 },
	{ 0xfba0, 0xfba0, PDF_CMAP_SINGLE, 21876 },
	{ 0xfc40, 0xfc43, PDF_CMAP_RANGE, 21877 },
	{ 0xfc44, 0xfc44, PDF_CMAP_SINGLE, 9878 },
	{ 0xfc45, 0xfc48, PDF_CMAP_RANGE, 21881 },
	{ 0xfc49, 0xfc49, PDF_CMAP_SINGLE, 9871 },
	{ 0xfc4a, 0xfc59, PDF_CMAP_RANGE, 21885 },
	{ 0xfc5a, 0xfc5a, PDF_CMAP_SINGLE, 8922 },
	{ 0xfc5b, 0xfc62, PDF_CMAP_RANGE, 21901 },
	{ 0xfc63, 0xfc63, PDF_CMAP_SINGLE, 7873 },
	{ 0xfc64, 0xfc67, PDF_CMAP_RANGE, 21909 },
	{ 0xfc68, 0xfc68, PDF_CMAP_SINGLE, 7859 },
	{ 0xfc69, 0xfc6e, PDF_CMAP_RANGE, 21913 },
	{ 0xfc6f, 0xfc6f, PDF_CMAP_SINGLE, 9834 },
	{ 0xfc70, 0xfc70, PDF_CMAP_SINGLE, 21919 },
	{ 0xfc71, 0xfc71, PDF_CMAP_SINGLE, 9866 },
	{ 0xfc72, 0xfc73, PDF_CMAP_RANGE, 21920 },
	{ 0xfc74, 0xfc74, PDF_CMAP_SINGLE, 9833 },
	{ 0xfc75, 0xfc76, PDF_CMAP_RANGE, 21922 },
	{ 0xfc77, 0xfc78, PDF_CMAP_RANGE, 9752 },
	{ 0xfc79, 0xfc7e, PDF_CMAP_RANGE, 21924 },
	{ 0xfc80, 0xfc82, PDF_CMAP_RANGE, 21930 },
	{ 0xfc83, 0xfc83, PDF_CMAP_SINGLE, 9754 },
	{ 0xfc84, 0xfc89, PDF_CMAP_RANGE, 21933 },
	{ 0xfc8a, 0xfc8a, PDF_CMAP_SINGLE, 9843 },
	{ 0xfc8b, 0xfca0, PDF_CMAP_RANGE, 21939 },
	{ 0xfd40, 0xfd51, PDF_CMAP_RANGE, 21961 },
	{ 0xfd52, 0xfd52, PDF_CMAP_SINGLE, 8390 },
	{ 0xfd53, 0xfd53, PDF_CMAP_SINGLE, 8811 },
	{ 0xfd54, 0xfd56, PDF_CMAP_RANGE, 21979 },
	{ 0xfd57, 0xfd57, PDF_CMAP_SINGLE, 9427 },
	{ 0xfd58, 0xfd58, PDF_CMAP_SINGLE, 7814 },
	{ 0xfd59, 0xfd59, PDF_CMAP_SINGLE, 21982 },
	{ 0xfd5a, 0xfd5a, PDF_CMAP_SINGLE, 9743 },
	{ 0xfd5b, 0xfd5e, PDF_CMAP_RANGE, 21983 },
	{ 0xfd5f, 0xfd5f, PDF_CMAP_SINGLE, 9745 },
	{ 0xfd60, 0xfd61, PDF_CMAP_RANGE, 21987 },
	{ 0xfd62, 0xfd62, PDF_CMAP_SINGLE, 9747 },
	{ 0xfd63, 0xfd64, PDF_CMAP_RANGE, 21989 },
	{ 0xfd65, 0xfd65, PDF_CMAP_SINGLE, 9744 },
	{ 0xfd66, 0xfd66, PDF_CMAP_SINGLE, 9746 },
	{ 0xfd67, 0xfd67, PDF_CMAP_SINGLE, 8240 },
	{ 0xfd68, 0xfd68, PDF_CMAP_SINGLE, 21991 },
	{ 0xfd69, 0xfd69, PDF_CMAP_SINGLE, 9841 },
	{ 0xfd6a, 0xfd6b, PDF_CMAP_RANGE, 21992 },
	{ 0xfd6c, 0xfd6c, PDF_CMAP_SINGLE, 9748 },
	{ 0xfd6d, 0xfd6f, PDF_CMAP_RANGE, 21994 },
	{ 0xfd70, 0xfd70, PDF_CMAP_SINGLE, 9750 },
	{ 0xfd71, 0xfd71, PDF_CMAP_SINGLE, 21997 },
	{ 0xfd72, 0xfd72, PDF_CMAP_SINGLE, 9749 },
	{ 0xfd73, 0xfd77, PDF_CMAP_RANGE, 21998 },
	{ 0xfd78, 0xfd78, PDF_CMAP_SINGLE, 8434 },
	{ 0xfd79, 0xfd7c, PDF_CMAP_RANGE, 22003 },
	{ 0xfd7d, 0xfd7d, PDF_CMAP_SINGLE, 9751 },
	{ 0xfd7e, 0xfd7e, PDF_CMAP_SINGLE, 22007 },
	{ 0xfd80, 0xfd87, PDF_CMAP_RANGE, 22008 },
	{ 0xfd88, 0xfd88, PDF_CMAP_SINGLE, 8247 },
	{ 0xfd89, 0xfd8a, PDF_CMAP_RANGE, 22016 },
	{ 0xfd8b, 0xfd8b, PDF_CMAP_SINGLE, 8371 },
	{ 0xfd8c, 0xfd8e, PDF_CMAP_RANGE, 22018 },
	{ 0xfd8f, 0xfd8f, PDF_CMAP_SINGLE, 7970 },
	{ 0xfd90, 0xfd90, PDF_CMAP_SINGLE, 9453 },
	{ 0xfd91, 0xfd93, PDF_CMAP_RANGE, 22021 },
	{ 0xfd94, 0xfd94, PDF_CMAP_SINGLE, 7988 },
	{ 0xfd95, 0xfd9c, PDF_CMAP_RANGE, 22024 },
	{ 0xfd9d, 0xfd9d, PDF_CMAP_SINGLE, 2562 },
	{ 0xfd9e, 0xfd9e, PDF_CMAP_SINGLE, 16595 },
	{ 0xfd9f, 0xfd9f, PDF_CMAP_SINGLE, 8204 },
	{ 0xfda0, 0xfda0, PDF_CMAP_SINGLE, 20611 },
	{ 0xfe40, 0xfe40, PDF_CMAP_SINGLE, 4697 },
	{ 0xfe41, 0xfe7e, PDF_CMAP_RANGE, 22032 },
	{ 0xfe80, 0xfea0, PDF_CMAP_RANGE, 22094 },
};

static const unsigned short pdf_cmap_GBKp_EUC_H_table[1851] =
{
	7778,10315,7781,8833,8909,10407,8458,7742,
	7830,8907,8812,10424,8471,10428,8424,8108,
	8915,8720,10474,8911,10475,8723,10476,8164,
	8912,8910,8919,8917,8913,8906,7833,8143,
	8246,7994,10676,8905,8089,7885,10719,8617,
	10720,8678,10721,8487,8195,8901,8030,10771,
	8902,8208,10831,8898,8459,11063,8414,11064,
	7850,8408,8896,8368,11113,9082,7796,9080,
	8655,9064,11125,9073,11126,9065,9072,11153,
	11154,9078,9075,9087,11158,9076,11159,8373,
	7899,9842,9067,11193,8249,9085,8693,11202,
	9089,9083,9086,8652,9066,9865,11214,9879,
	9091,7997,11247,8591,8774,8776,8574,11254,
	8576,8709,7736,8163,9017,11440,9003,11441,
	9005,8615,11453,9007,8409,11517,8157,8199,
	8998,9001,8018,8251,9000,11540,8999,11590,
	8060,9051,7902,11614,8098,11615,7930,9210,
	11898,9214,8104,8421,12017,8495,8355,8481,
	12018,8660,8165,7818,7735,8094,8870,8680,
	12032,7898,7865,8272,7783,12074,9204,9097,
	12165,7863,9105,12268,8817,9104,12313,8243,
	12314,8764,8277,12333,9106,8837,12387,9092,
	9095,9094,8848,12406,7748,7730,9093,7955,
	8051,8327,7800,9134,7926,7985,9205,12521,
	8821,9838,12534,7856,8323,12538,8584,7841,
	12575,9107,9140,8318,8341,9147,7720,9149,
	7774,12819,7773,7983,9439,9138,8526,9148,
	12833,8427,8219,8380,9150,8329,12871,9137,
	8158,8739,9142,9868,9438,8190,8017,8673,
	12922,9136,12923,8142,12924,8478,8225,8819,
	12944,8624,8476,13097,9053,8461,8282,9057,
	13234,8256,9054,13252,7786,7852,7764,8384,
	9058,8544,13288,8605,8078,8749,13289,8263,
	13290,8802,8039,7858,7762,8356,7964,8847,
	13314,8173,7725,8530,9062,13319,8442,9059,
	13324,8346,8182,13335,9060,13336,7785,13337,
	9063,13338,8477,8793,8278,8547,8105,8188,
	7868,13388,8514,8222,7747,8790,9384,9383,
	13593,9862,9382,13594,8654,8565,13863,9325,
	9323,8799,9320,14046,8448,14058,9334,7975,
	8407,8874,8197,8387,8512,9335,8040,14171,
	8581,8075,9348,9344,9326,14266,9339,9350,
	9329,9347,9328,14274,9321,14275,9343,9419,
	14362,8019,8211,7987,9353,9355,8463,8155,
	8394,14468,8423,9410,8772,9186,14706,8404,
	7779,8328,7869,14840,9025,8264,9184,8311,
	14865,8762,8001,8220,8095,14903,9674,14904,
	9185,8172,9187,8462,8102,8196,8886,8361,
	8557,15007,8593,7760,8092,15023,9178,15024,
	8275,9193,9188,8661,15033,9190,8212,15045,
	9196,9195,9176,9198,9870,15058,9197,9174,
	9192,8585,8280,8473,15342,8124,15370,9434,
	8712,15414,7910,8630,7741,8757,15560,9111,
	8767,8490,7890,15595,9110,9112,15596,8354,
	15597,8235,15600,9108,8505,15601,8543,8641,
	9114,8741,8298,7937,8704,9650,7831,16025,
	8363,8232,16038,16040,9649,8706,9643,8838,
	16057,9645,16058,9654,16059,8675,8751,8549,
	9655,7717,16083,9664,16107,8858,8814,16126,
	8125,16127,8068,8370,16128,8259,16205,9455,
	9445,16366,8698,9446,8439,16413,8299,16471,
	7826,9447,8210,7916,8038,9436,8206,16581,
	9435,7864,9764,8041,8747,9599,8027,8603,
	16704,9853,8608,8582,16738,8429,7844,8417,
	16756,9656,8800,16757,8418,8868,16912,9704,
	9706,16970,8079,16971,9708,8400,8221,16992,
	9710,8250,9701,9709,17024,8201,17025,8293,
	9717,7932,9883,17108,9715,17109,8921,17110,
	9716,17111,9243,8136,17115,8059,17116,9245,
	17117,8780,8008,9244,8447,8602,17118,8337,
	9251,7835,9250,17123,8464,17124,8845,8049,
	7928,9249,8625,9254,9253,8480,8474,9252,
	17143,9256,9258,9257,17146,8852,17147,8894,
	17148,7729,9260,17156,8117,9259,8111,8296,
	8676,7969,17174,8449,8572,8522,9261,17184,
	8145,7731,17192,9263,9262,17195,9264,8667,
	8889,17212,9270,7822,9273,8645,9271,8594,
	17222,9274,7961,8588,7744,8883,8287,9272,
	9266,8820,17226,7836,9265,8325,8119,9267,
	8276,8670,17240,9268,9277,8073,9276,8046,
	7896,17246,7871,9285,17247,8777,9281,17250,
	7751,8022,8598,17255,9283,9279,17256,8227,
	17257,9282,9278,17260,9038,9286,9291,9284,
	9255,9292,7951,17286,9287,9289,9288,7939,
	17296,9290,8540,8891,9296,17299,9876,9295,
	9718,8273,17300,9294,8890,8045,9298,9297,
	9301,17318,8841,8470,8486,8033,9885,17335,
	8074,9302,9305,9304,8058,9293,9303,9299,
	17353,9248,17354,8671,9861,17355,7790,8740,
	9840,17359,8634,17360,9306,9698,17392,9882,
	8290,9457,17425,9458,9666,17520,9665,8488,
	17542,8215,7840,8485,8525,17561,9668,8349,
	8842,17562,9667,17712,8345,7853,9416,8360,
	8223,17732,8389,17733,9418,8176,17742,9414,
	8797,8926,8763,8665,8139,8137,9711,17818,
	8088,17821,9712,18004,8013,9031,18078,8595,
	9037,9032,7776,18133,9029,8216,9013,9028,
	9009,18228,9035,18229,9019,7860,8614,9022,
	18259,8052,18260,9008,8081,18279,8455,8180,
	9027,8788,18333,9010,7718,9047,9048,8258,
	8531,8289,18386,8265,18406,8002,18571,8606,
	7787,18646,9691,18654,8721,8745,9686,9693,
	9695,8175,18679,9688,8600,18718,7816,8844,
	18733,7768,8876,9846,18823,8161,9659,9660,
	9658,8583,18893,9835,8502,18925,9400,8420,
	18941,9401,8189,18954,9403,7981,9737,18980,
	9738,18981,7828,8927,7883,7949,8055,18992,
	8683,18993,8929,18994,8559,18995,8928,8682,
	18998,8930,8395,18999,8056,7906,19002,8690,
	19003,8528,8147,19007,8933,8532,19019,8935,
	8932,19038,8810,19039,8939,19040,8937,8381,
	8950,8679,8726,8492,19053,7784,8946,7991,
	8947,19054,8943,8016,7952,8648,19055,8945,
	8942,19063,8941,8862,8940,8162,19073,7855,
	8758,19076,8951,8765,19080,7810,8118,19081,
	8612,8618,8952,8529,19082,8032,8517,19089,
	8156,19101,8884,8553,19104,8957,19105,8426,
	19106,8948,19107,8955,19108,8956,8231,8288,
	8959,8958,7879,19115,8972,8949,19119,8966,
	19120,8970,8659,8963,19124,8967,8971,8031,
	19127,8969,8962,7940,8861,19130,8699,19131,
	8968,19132,8364,19133,8334,8965,8599,19134,
	8561,8856,8322,19141,8977,8964,19145,8975,
	7733,19146,8401,8976,8099,19147,8662,8978,
	8333,8979,8982,8043,8980,19192,8496,8981,
	8552,19193,8388,8729,8727,7789,8440,19236,
	8187,8985,8974,19242,8983,7739,8831,19329,
	7948,7769,7972,8378,8037,7920,8548,7984,
	8801,19334,19335,8866,9387,19336,9391,7912,
	7993,19337,7752,8304,7848,19338,9388,7927,
	8566,9389,19339,8315,8005,9386,8267,8239,
	8026,9392,19340,8887,8063,19341,8805,9394,
	8475,7761,19348,9396,9395,19353,7839,19354,
	8472,19355,8372,9135,8635,8306,8085,19356,
	7946,9398,8849,9397,8824,8873,9399,7976,
	8457,8903,9390,8881,8806,19377,8795,8900,
	19378,8469,19379,8746,9393,7957,8796,9724,
	7820,9730,19571,8781,19572,9732,9726,9731,
	7843,9736,7803,8807,7990,8150,8672,19616,
	9356,8857,9361,9366,9359,9362,19647,9367,
	8113,19652,9370,19653,9369,19654,8792,9368,
	9371,19662,7945,8422,8230,9375,8025,19672,
	7995,9372,7738,8283,8816,8759,9426,19692,
	8627,8773,9377,8828,8112,8006,19713,9088,
	9365,19714,9360,7837,7755,7754,8362,9202,
	19771,8830,19772,8217,8787,19797,7998,7846,
	8590,8684,19801,7870,19802,8778,8399,8674,
	19815,8719,19816,8233,19817,8307,9201,19820,
	7750,8291,9203,8892,19895,8988,19896,8785,
	8839,19914,8237,7851,19915,8989,19916,8991,
	8716,8100,8347,8664,19990,9722,19991,9721,
	8500,19994,9464,19995,9463,7880,9462,7877,
	9467,20003,20004,9466,20005,7917,9469,20017,
	9465,20018,9470,8397,9480,9476,9471,8336,
	7901,7973,20032,9475,9474,7802,8358,7953,
	20047,9479,9472,9477,9497,20054,9493,9484,
	8241,9483,9487,9498,9481,9486,8756,9491,
	8064,9495,9494,9485,20076,8403,8314,8398,
	8106,20101,9502,7967,9517,8733,9522,20107,
	8571,8623,20118,9516,9512,20121,8332,20122,
	9519,20123,8636,9525,20129,8717,9510,20130,
	9524,9514,9503,9521,9500,8653,20139,8666,
	20140,8562,9534,8271,20145,9539,20146,8663,
	20147,7740,9535,20165,9540,9507,7824,9530,
	9541,20169,9533,20170,8385,8451,9504,9532,
	9531,9528,9536,20177,8141,8880,20196,9542,
	7834,9554,9520,9545,9553,7882,20204,8402,
	8120,20208,8313,9549,7845,8268,8320,9837,
	20217,9527,9546,8632,20224,9550,9468,9556,
	20229,7996,20230,7893,9559,9555,9544,20253,
	8087,9557,8316,20270,9562,8542,20285,9568,
	8610,9044,9571,9511,20297,9518,9560,20298,
	7963,20299,8835,20300,9572,8352,20303,9573,
	9581,8224,9567,9578,9529,9564,9579,7791,
	20330,8132,9575,20331,9563,9515,20347,9585,
	20348,9591,8234,20369,9526,9587,9583,8851,
	9592,9594,8198,20391,8567,9499,20394,9508,
	9593,20407,9574,8083,20413,9596,9597,20427,
	9586,8782,20436,8646,8351,20441,9590,8292,
	8895,9756,8798,8317,20480,9151,8467,7749,
	20485,8152,9156,20486,9154,8452,20487,8637,
	20488,8071,9155,8003,20500,7966,9849,7915,
	7989,8330,20503,9159,9161,9158,8691,20513,
	8695,9167,9166,9162,9165,9836,20519,9153,
	20520,8174,9169,9170,9172,7832,7980,20532,
	9173,7793,9873,8732,20585,7806,8269,20599,
	8114,8535,20615,8639,8735,20621,8253,8213,
	20624,9893,8516,20637,7825,8791,8202,8338,
	9740,20702,9742,9823,20798,8589,8445,9319,
	20812,9318,8649,20835,8713,7881,8425,20836,
	8650,8518,9669,8668,20837,9310,20838,8527,
	20839,9670,9671,8769,8586,7727,7900,8383,
	8244,8718,20852,9675,9676,8131,8377,20869,
	8577,8563,7905,9678,8694,8779,9681,20890,
	7872,9682,20898,7978,7794,9683,8638,9684,
	8260,20904,9679,8435,20916,9880,20917,9848,
	8376,20932,9424,20933,9425,9115,20946,9854,
	20951,9121,20952,7921,8734,20955,9122,8523,
	7734,8501,8109,20967,7763,20970,9123,8707,
	20971,7911,9124,8343,7908,9118,21006,8245,
	9127,9128,21010,8309,9131,8171,9132,8042,
	8441,9831,21023,7788,8301,8770,7938,8579,
	21057,7813,8681,8869,9223,8138,21084,9218,
	8066,9220,21087,8497,21088,8580,21089,9219,
	21090,8302,9227,8151,21117,7811,9230,8391,
	9229,9200,21153,9233,9236,21154,8560,9221,
	8460,9237,8294,21165,9042,9235,9232,9225,
	21172,9240,8103,21178,8700,8129,9222,8270,
	21186,9242,9241,9825,21225,9892,8564,9827,
	9826,9832,21283,7888,21284,8342,21285,9164,
	9160,21290,8766,9828,21313,8761,9760,21349,
	9761,9762,21360,9767,7737,9765,9774,9771,
	9770,9773,21381,9768,8633,9782,9776,9784,
	21404,8205,9783,9797,21414,9786,21419,9792,
	9789,9793,21423,9790,9791,8128,21426,9785,
	21442,9804,9799,9803,9800,21456,9802,8456,
	9780,21471,9809,9808,9810,9778,9806,9811,
	21480,9815,21481,9781,21482,9779,21483,9814,
	9813,7757,9819,21502,9818,21503,9817,8238,
	9775,9798,21508,9766,9763,21531,9777,7941,
	8331,21578,9601,9603,9602,21594,8686,8771,
	21611,9607,21612,9608,21615,9604,8701,21618,
	8687,9610,9612,8144,9618,9615,9620,7904,
	9617,9621,9623,21672,8374,9605,21697,9625,
	9626,21711,9627,9824,9632,9628,21743,9631,
	21744,8044,8366,21762,9611,9635,9609,9637,
	9638,9619,9639,8738,9641,9633,9642,21816,
	9616,9614,9886,9723,21845,8076,8692,21846,
	9719,21876,9834,21919,9866,8390,8811,9427,
	7814,21982,9743,9744,9746,8240,21991,9841,
	9750,21997,9749,9751,22007,7970,9453,2562,
	16595,8204,20611,
};

pdf_cmap pdf_cmap_GBKp_EUC_H =
{
	-1, "GBKp-EUC-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0xfefe },
	},
	4070, 4070, (pdf_range*) pdf_cmap_GBKp_EUC_H_ranges,
	1851, 1851, (unsigned short*) pdf_cmap_GBKp_EUC_H_table,
};

/* GBKp-EUC-V */

static const pdf_range pdf_cmap_GBKp_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa3fd, 0xa3fd, PDF_CMAP_SINGLE, 597 },
	{ 0xa3fe, 0xa3fe, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GBKp_EUC_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBKp_EUC_V =
{
	-1, "GBKp-EUC-V", "GBKp-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GBKp_EUC_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBKp_EUC_V_table,
};

/* GBpc-EUC-H */

static const pdf_range pdf_cmap_GBpc_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x0080, 0x0080, PDF_CMAP_SINGLE, 7712 },
	{ 0x00fd, 0x00ff, PDF_CMAP_RANGE, 7713 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 940 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1034 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1128 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1222 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1316 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1410 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1504 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1598 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1692 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1786 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 1880 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 1974 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2068 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2162 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2256 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2350 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2444 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2538 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2632 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2726 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2820 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 2914 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3008 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3102 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3196 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 3290 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3384 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 3478 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3572 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 3666 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3760 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 3854 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 3948 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 4042 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4136 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4230 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 4324 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 4418 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd7a1, 0xd7f9, PDF_CMAP_RANGE, 4606 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4695 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4789 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4883 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 4977 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5071 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5165 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5259 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5353 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5447 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5541 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5635 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5729 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5823 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5917 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6011 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6105 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6199 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6293 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6387 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6481 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6575 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6669 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6763 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6857 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6951 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7045 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7139 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7233 },
	{ 0xf4a1, 0xf4fe, PDF_CMAP_RANGE, 7327 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7421 },
	{ 0xf6a1, 0xf6fe, PDF_CMAP_RANGE, 7515 },
	{ 0xf7a1, 0xf7fe, PDF_CMAP_RANGE, 7609 },
};

static const unsigned short pdf_cmap_GBpc_EUC_H_table[1] = { 0 };

pdf_cmap pdf_cmap_GBpc_EUC_H =
{
	-1, "GBpc-EUC-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa1a1, 0xfcfe },
		{ 1, 0x00fd, 0x00ff },
	},
	91, 91, (pdf_range*) pdf_cmap_GBpc_EUC_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_GBpc_EUC_H_table,
};

/* GBpc-EUC-V */

static const pdf_range pdf_cmap_GBpc_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa3fd, 0xa3fd, PDF_CMAP_SINGLE, 597 },
	{ 0xa3fe, 0xa3fe, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GBpc_EUC_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBpc_EUC_V =
{
	-1, "GBpc-EUC-V", "GBpc-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GBpc_EUC_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBpc_EUC_V_table,
};

/* GBT-EUC-H */

static const pdf_range pdf_cmap_GBT_EUC_H_ranges[] =
{
	{ 0x0020, 0x0020, PDF_CMAP_SINGLE, 7716 },
	{ 0x0021, 0x007e, PDF_CMAP_RANGE, 814 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xb0a1, 0xb0a7, PDF_CMAP_RANGE, 940 },
	{ 0xb0a8, 0xb0aa, PDF_CMAP_TABLE, 0 },
	{ 0xb0ab, 0xb0ac, PDF_CMAP_RANGE, 950 },
	{ 0xb0ad, 0xb0ae, PDF_CMAP_RANGE, 7719 },
	{ 0xb0af, 0xb0b8, PDF_CMAP_RANGE, 954 },
	{ 0xb0b9, 0xb0b9, PDF_CMAP_SINGLE, 7721 },
	{ 0xb0ba, 0xb0bf, PDF_CMAP_RANGE, 965 },
	{ 0xb0c0, 0xb0c0, PDF_CMAP_SINGLE, 7722 },
	{ 0xb0c1, 0xb0d2, PDF_CMAP_RANGE, 972 },
	{ 0xb0d3, 0xb0d5, PDF_CMAP_TABLE, 3 },
	{ 0xb0d6, 0xb0d9, PDF_CMAP_RANGE, 993 },
	{ 0xb0da, 0xb0dc, PDF_CMAP_TABLE, 6 },
	{ 0xb0dd, 0xb0e3, PDF_CMAP_RANGE, 1000 },
	{ 0xb0e4, 0xb0e4, PDF_CMAP_SINGLE, 7727 },
	{ 0xb0e5, 0xb0eb, PDF_CMAP_RANGE, 1008 },
	{ 0xb0ec, 0xb0ed, PDF_CMAP_RANGE, 7728 },
	{ 0xb0ee, 0xb0ef, PDF_CMAP_TABLE, 9 },
	{ 0xb0f0, 0xb0f2, PDF_CMAP_RANGE, 1019 },
	{ 0xb0f3, 0xb0f3, PDF_CMAP_SINGLE, 7731 },
	{ 0xb0f4, 0xb0f6, PDF_CMAP_RANGE, 1023 },
	{ 0xb0f7, 0xb0f9, PDF_CMAP_TABLE, 11 },
	{ 0xb0fa, 0xb0fe, PDF_CMAP_RANGE, 1029 },
	{ 0xb1a1, 0xb1a4, PDF_CMAP_RANGE, 1034 },
	{ 0xb1a5, 0xb1a6, PDF_CMAP_RANGE, 7734 },
	{ 0xb1a7, 0xb1a8, PDF_CMAP_TABLE, 14 },
	{ 0xb1a9, 0xb1aa, PDF_CMAP_RANGE, 1042 },
	{ 0xb1ab, 0xb1ab, PDF_CMAP_SINGLE, 7737 },
	{ 0xb1ac, 0xb1b1, PDF_CMAP_RANGE, 1045 },
	{ 0xb1b2, 0xb1b3, PDF_CMAP_TABLE, 16 },
	{ 0xb1b4, 0xb1b5, PDF_CMAP_RANGE, 7739 },
	{ 0xb1b6, 0xb1b6, PDF_CMAP_SINGLE, 1055 },
	{ 0xb1b7, 0xb1b9, PDF_CMAP_RANGE, 7741 },
	{ 0xb1ba, 0xb1c0, PDF_CMAP_RANGE, 1059 },
	{ 0xb1c1, 0xb1c1, PDF_CMAP_SINGLE, 7744 },
	{ 0xb1c2, 0xb1c9, PDF_CMAP_RANGE, 1067 },
	{ 0xb1ca, 0xb1ca, PDF_CMAP_SINGLE, 7745 },
	{ 0xb1cb, 0xb1ce, PDF_CMAP_RANGE, 1076 },
	{ 0xb1cf, 0xb1d0, PDF_CMAP_RANGE, 7746 },
	{ 0xb1d1, 0xb1d2, PDF_CMAP_TABLE, 18 },
	{ 0xb1d3, 0xb1d4, PDF_CMAP_RANGE, 1084 },
	{ 0xb1d5, 0xb1d5, PDF_CMAP_SINGLE, 7749 },
	{ 0xb1d6, 0xb1de, PDF_CMAP_RANGE, 1087 },
	{ 0xb1df, 0xb1e1, PDF_CMAP_RANGE, 7750 },
	{ 0xb1e2, 0xb1e3, PDF_CMAP_RANGE, 1099 },
	{ 0xb1e4, 0xb1e4, PDF_CMAP_SINGLE, 7753 },
	{ 0xb1e5, 0xb1e6, PDF_CMAP_RANGE, 1102 },
	{ 0xb1e7, 0xb1e8, PDF_CMAP_RANGE, 7754 },
	{ 0xb1e9, 0xb1ea, PDF_CMAP_TABLE, 20 },
	{ 0xb1eb, 0xb1ed, PDF_CMAP_RANGE, 1108 },
	{ 0xb1ee, 0xb1ee, PDF_CMAP_SINGLE, 7757 },
	{ 0xb1ef, 0xb1f0, PDF_CMAP_RANGE, 1112 },
	{ 0xb1f1, 0xb1f1, PDF_CMAP_SINGLE, 7758 },
	{ 0xb1f2, 0xb1f3, PDF_CMAP_RANGE, 1115 },
	{ 0xb1f4, 0xb1f7, PDF_CMAP_RANGE, 7759 },
	{ 0xb1f8, 0xb1fc, PDF_CMAP_RANGE, 1121 },
	{ 0xb1fd, 0xb1fe, PDF_CMAP_TABLE, 22 },
	{ 0xb2a1, 0xb2a5, PDF_CMAP_RANGE, 1128 },
	{ 0xb2a6, 0xb2a7, PDF_CMAP_RANGE, 7764 },
	{ 0xb2a8, 0xb2ab, PDF_CMAP_RANGE, 1135 },
	{ 0xb2ac, 0xb2ac, PDF_CMAP_SINGLE, 7766 },
	{ 0xb2ad, 0xb2b4, PDF_CMAP_RANGE, 1140 },
	{ 0xb2b5, 0xb2b5, PDF_CMAP_SINGLE, 7767 },
	{ 0xb2b6, 0xb2b8, PDF_CMAP_RANGE, 1149 },
	{ 0xb2b9, 0xb2b9, PDF_CMAP_SINGLE, 7768 },
	{ 0xb2ba, 0xb2c5, PDF_CMAP_RANGE, 1153 },
	{ 0xb2c6, 0xb2c6, PDF_CMAP_SINGLE, 7769 },
	{ 0xb2c7, 0xb2cd, PDF_CMAP_RANGE, 1166 },
	{ 0xb2ce, 0xb2d7, PDF_CMAP_RANGE, 7770 },
	{ 0xb2d8, 0xb2dd, PDF_CMAP_RANGE, 1183 },
	{ 0xb2de, 0xb2e1, PDF_CMAP_TABLE, 24 },
	{ 0xb2e2, 0xb2e3, PDF_CMAP_RANGE, 7782 },
	{ 0xb2e4, 0xb2ee, PDF_CMAP_RANGE, 1195 },
	{ 0xb2ef, 0xb2ef, PDF_CMAP_SINGLE, 7784 },
	{ 0xb2f0, 0xb2f2, PDF_CMAP_RANGE, 1207 },
	{ 0xb2f3, 0xb2fc, PDF_CMAP_RANGE, 7785 },
	{ 0xb2fd, 0xb2fe, PDF_CMAP_RANGE, 1220 },
	{ 0xb3a1, 0xb3a2, PDF_CMAP_RANGE, 7795 },
	{ 0xb3a3, 0xb3a3, PDF_CMAP_SINGLE, 1224 },
	{ 0xb3a4, 0xb3a7, PDF_CMAP_RANGE, 7797 },
	{ 0xb3a8, 0xb3a9, PDF_CMAP_TABLE, 28 },
	{ 0xb3aa, 0xb3ad, PDF_CMAP_RANGE, 1231 },
	{ 0xb3ae, 0xb3ae, PDF_CMAP_SINGLE, 7802 },
	{ 0xb3af, 0xb3b4, PDF_CMAP_RANGE, 1236 },
	{ 0xb3b5, 0xb3b5, PDF_CMAP_SINGLE, 7803 },
	{ 0xb3b6, 0xb3b8, PDF_CMAP_RANGE, 1243 },
	{ 0xb3b9, 0xb3b9, PDF_CMAP_SINGLE, 7804 },
	{ 0xb3ba, 0xb3bd, PDF_CMAP_RANGE, 1247 },
	{ 0xb3be, 0xb3be, PDF_CMAP_SINGLE, 7805 },
	{ 0xb3bf, 0xb3c1, PDF_CMAP_RANGE, 1252 },
	{ 0xb3c2, 0xb3c6, PDF_CMAP_TABLE, 30 },
	{ 0xb3c7, 0xb3cc, PDF_CMAP_RANGE, 1260 },
	{ 0xb3cd, 0xb3cf, PDF_CMAP_TABLE, 35 },
	{ 0xb3d0, 0xb3d1, PDF_CMAP_RANGE, 1269 },
	{ 0xb3d2, 0xb3d2, PDF_CMAP_SINGLE, 7811 },
	{ 0xb3d3, 0xb3d8, PDF_CMAP_RANGE, 1272 },
	{ 0xb3d9, 0xb3dd, PDF_CMAP_TABLE, 38 },
	{ 0xb3de, 0xb3e2, PDF_CMAP_RANGE, 1283 },
	{ 0xb3e3, 0xb3e4, PDF_CMAP_TABLE, 43 },
	{ 0xb3e5, 0xb3e6, PDF_CMAP_RANGE, 7816 },
	{ 0xb3e7, 0xb3e8, PDF_CMAP_TABLE, 45 },
	{ 0xb3e9, 0xb3ea, PDF_CMAP_RANGE, 1294 },
	{ 0xb3eb, 0xb3ec, PDF_CMAP_RANGE, 7819 },
	{ 0xb3ed, 0xb3ee, PDF_CMAP_RANGE, 1298 },
	{ 0xb3ef, 0xb3f3, PDF_CMAP_TABLE, 47 },
	{ 0xb3f4, 0xb3f9, PDF_CMAP_RANGE, 1305 },
	{ 0xb3fa, 0xb3fb, PDF_CMAP_RANGE, 7824 },
	{ 0xb3fc, 0xb3fe, PDF_CMAP_RANGE, 1313 },
	{ 0xb4a1, 0xb4a2, PDF_CMAP_RANGE, 7826 },
	{ 0xb4a3, 0xb4a4, PDF_CMAP_RANGE, 1318 },
	{ 0xb4a5, 0xb4a6, PDF_CMAP_RANGE, 7828 },
	{ 0xb4a7, 0xb4aa, PDF_CMAP_RANGE, 1322 },
	{ 0xb4ab, 0xb4ab, PDF_CMAP_SINGLE, 7830 },
	{ 0xb4ac, 0xb4ae, PDF_CMAP_RANGE, 1327 },
	{ 0xb4af, 0xb4af, PDF_CMAP_SINGLE, 7831 },
	{ 0xb4b0, 0xb4b2, PDF_CMAP_RANGE, 1331 },
	{ 0xb4b3, 0xb4b4, PDF_CMAP_RANGE, 7832 },
	{ 0xb4b5, 0xb4b7, PDF_CMAP_RANGE, 1336 },
	{ 0xb4b8, 0xb4b8, PDF_CMAP_SINGLE, 7834 },
	{ 0xb4b9, 0xb4be, PDF_CMAP_RANGE, 1340 },
	{ 0xb4bf, 0xb4bf, PDF_CMAP_SINGLE, 7835 },
	{ 0xb4c0, 0xb4c1, PDF_CMAP_RANGE, 1347 },
	{ 0xb4c2, 0xb4c2, PDF_CMAP_SINGLE, 7836 },
	{ 0xb4c3, 0xb4c6, PDF_CMAP_RANGE, 1350 },
	{ 0xb4c7, 0xb4c7, PDF_CMAP_SINGLE, 7837 },
	{ 0xb4c8, 0xb4c9, PDF_CMAP_RANGE, 1355 },
	{ 0xb4ca, 0xb4ca, PDF_CMAP_SINGLE, 7838 },
	{ 0xb4cb, 0xb4cc, PDF_CMAP_RANGE, 1358 },
	{ 0xb4cd, 0xb4cf, PDF_CMAP_TABLE, 52 },
	{ 0xb4d0, 0xb4d2, PDF_CMAP_RANGE, 1363 },
	{ 0xb4d3, 0xb4d4, PDF_CMAP_RANGE, 7841 },
	{ 0xb4d5, 0xb4d9, PDF_CMAP_RANGE, 1368 },
	{ 0xb4da, 0xb4dc, PDF_CMAP_TABLE, 55 },
	{ 0xb4dd, 0xb4ec, PDF_CMAP_RANGE, 1376 },
	{ 0xb4ed, 0xb4ef, PDF_CMAP_TABLE, 58 },
	{ 0xb4f0, 0xb4f7, PDF_CMAP_RANGE, 1395 },
	{ 0xb4f8, 0xb4f8, PDF_CMAP_SINGLE, 7847 },
	{ 0xb4f9, 0xb4fa, PDF_CMAP_RANGE, 1404 },
	{ 0xb4fb, 0xb4fb, PDF_CMAP_SINGLE, 7848 },
	{ 0xb4fc, 0xb4fe, PDF_CMAP_RANGE, 1407 },
	{ 0xb5a1, 0xb5a2, PDF_CMAP_RANGE, 1410 },
	{ 0xb5a3, 0xb5a4, PDF_CMAP_TABLE, 61 },
	{ 0xb5a5, 0xb5a8, PDF_CMAP_RANGE, 7850 },
	{ 0xb5a9, 0xb5ab, PDF_CMAP_RANGE, 1418 },
	{ 0xb5ac, 0xb5ad, PDF_CMAP_TABLE, 63 },
	{ 0xb5ae, 0xb5af, PDF_CMAP_RANGE, 7855 },
	{ 0xb5b0, 0xb5b0, PDF_CMAP_SINGLE, 1425 },
	{ 0xb5b1, 0xb5b5, PDF_CMAP_RANGE, 7857 },
	{ 0xb5b6, 0xb5b7, PDF_CMAP_TABLE, 65 },
	{ 0xb5b8, 0xb5b9, PDF_CMAP_RANGE, 1433 },
	{ 0xb5ba, 0xb5bc, PDF_CMAP_RANGE, 7863 },
	{ 0xb5bd, 0xb5c5, PDF_CMAP_RANGE, 1438 },
	{ 0xb5c6, 0xb5c6, PDF_CMAP_SINGLE, 7866 },
	{ 0xb5c7, 0xb5ca, PDF_CMAP_RANGE, 1448 },
	{ 0xb5cb, 0xb5cb, PDF_CMAP_SINGLE, 7867 },
	{ 0xb5cc, 0xb5cf, PDF_CMAP_RANGE, 1453 },
	{ 0xb5d0, 0xb5d0, PDF_CMAP_SINGLE, 7868 },
	{ 0xb5d1, 0xb5d2, PDF_CMAP_RANGE, 1458 },
	{ 0xb5d3, 0xb5d3, PDF_CMAP_SINGLE, 7869 },
	{ 0xb5d4, 0xb5dc, PDF_CMAP_RANGE, 1461 },
	{ 0xb5dd, 0xb5df, PDF_CMAP_RANGE, 7870 },
	{ 0xb5e0, 0xb5e2, PDF_CMAP_RANGE, 1473 },
	{ 0xb5e3, 0xb5e3, PDF_CMAP_SINGLE, 7873 },
	{ 0xb5e4, 0xb5e5, PDF_CMAP_RANGE, 1477 },
	{ 0xb5e6, 0xb5e7, PDF_CMAP_RANGE, 7874 },
	{ 0xb5e8, 0xb5ec, PDF_CMAP_RANGE, 1481 },
	{ 0xb5ed, 0xb5ed, PDF_CMAP_SINGLE, 7876 },
	{ 0xb5ee, 0xb5f5, PDF_CMAP_RANGE, 1487 },
	{ 0xb5f6, 0xb5f7, PDF_CMAP_RANGE, 7877 },
	{ 0xb5f8, 0xb5fc, PDF_CMAP_RANGE, 1497 },
	{ 0xb5fd, 0xb5fe, PDF_CMAP_TABLE, 67 },
	{ 0xb6a1, 0xb6a3, PDF_CMAP_RANGE, 1504 },
	{ 0xb6a4, 0xb6a5, PDF_CMAP_RANGE, 7880 },
	{ 0xb6a6, 0xb6ab, PDF_CMAP_TABLE, 69 },
	{ 0xb6ac, 0xb6ae, PDF_CMAP_RANGE, 1515 },
	{ 0xb6af, 0xb6b0, PDF_CMAP_RANGE, 7885 },
	{ 0xb6b1, 0xb6b2, PDF_CMAP_RANGE, 1520 },
	{ 0xb6b3, 0xb6b3, PDF_CMAP_SINGLE, 7887 },
	{ 0xb6b4, 0xb6b6, PDF_CMAP_RANGE, 1523 },
	{ 0xb6b7, 0xb6b7, PDF_CMAP_SINGLE, 7888 },
	{ 0xb6b8, 0xb6be, PDF_CMAP_RANGE, 1527 },
	{ 0xb6bf, 0xb6c1, PDF_CMAP_RANGE, 7889 },
	{ 0xb6c2, 0xb6c3, PDF_CMAP_RANGE, 1537 },
	{ 0xb6c4, 0xb6c6, PDF_CMAP_TABLE, 75 },
	{ 0xb6c7, 0xb6cc, PDF_CMAP_RANGE, 1542 },
	{ 0xb6cd, 0xb6ce, PDF_CMAP_TABLE, 78 },
	{ 0xb6cf, 0xb6d0, PDF_CMAP_RANGE, 7895 },
	{ 0xb6d1, 0xb6d2, PDF_CMAP_RANGE, 1552 },
	{ 0xb6d3, 0xb6d4, PDF_CMAP_RANGE, 7897 },
	{ 0xb6d5, 0xb6d6, PDF_CMAP_TABLE, 80 },
	{ 0xb6d7, 0xb6d8, PDF_CMAP_RANGE, 1558 },
	{ 0xb6d9, 0xb6db, PDF_CMAP_TABLE, 82 },
	{ 0xb6dc, 0xb6e0, PDF_CMAP_RANGE, 1563 },
	{ 0xb6e1, 0xb6e1, PDF_CMAP_SINGLE, 7902 },
	{ 0xb6e2, 0xb6e8, PDF_CMAP_RANGE, 1569 },
	{ 0xb6e9, 0xb6e9, PDF_CMAP_SINGLE, 7903 },
	{ 0xb6ea, 0xb6eb, PDF_CMAP_RANGE, 1577 },
	{ 0xb6ec, 0xb6ed, PDF_CMAP_TABLE, 85 },
	{ 0xb6ee, 0xb6ef, PDF_CMAP_RANGE, 7905 },
	{ 0xb6f0, 0xb6f1, PDF_CMAP_TABLE, 87 },
	{ 0xb6f2, 0xb6f5, PDF_CMAP_RANGE, 1585 },
	{ 0xb6f6, 0xb6f6, PDF_CMAP_SINGLE, 7908 },
	{ 0xb6f7, 0xb6f8, PDF_CMAP_RANGE, 1590 },
	{ 0xb6f9, 0xb6fa, PDF_CMAP_TABLE, 89 },
	{ 0xb6fb, 0xb6fc, PDF_CMAP_RANGE, 7910 },
	{ 0xb6fd, 0xb6fe, PDF_CMAP_RANGE, 1596 },
	{ 0xb7a1, 0xb7a3, PDF_CMAP_RANGE, 7912 },
	{ 0xb7a4, 0xb7a6, PDF_CMAP_RANGE, 1601 },
	{ 0xb7a7, 0xb7a7, PDF_CMAP_SINGLE, 7915 },
	{ 0xb7a8, 0xb7ae, PDF_CMAP_RANGE, 1605 },
	{ 0xb7af, 0xb7b0, PDF_CMAP_RANGE, 7916 },
	{ 0xb7b1, 0xb7b2, PDF_CMAP_RANGE, 1614 },
	{ 0xb7b3, 0xb7b3, PDF_CMAP_SINGLE, 7918 },
	{ 0xb7b4, 0xb7b5, PDF_CMAP_RANGE, 1617 },
	{ 0xb7b6, 0xb7b7, PDF_CMAP_RANGE, 7919 },
	{ 0xb7b8, 0xb7b9, PDF_CMAP_TABLE, 91 },
	{ 0xb7ba, 0xb7c2, PDF_CMAP_RANGE, 1623 },
	{ 0xb7c3, 0xb7c4, PDF_CMAP_RANGE, 7922 },
	{ 0xb7c5, 0xb7c8, PDF_CMAP_RANGE, 1634 },
	{ 0xb7c9, 0xb7c9, PDF_CMAP_SINGLE, 7924 },
	{ 0xb7ca, 0xb7cb, PDF_CMAP_RANGE, 1639 },
	{ 0xb7cc, 0xb7cc, PDF_CMAP_SINGLE, 7925 },
	{ 0xb7cd, 0xb7ce, PDF_CMAP_RANGE, 1642 },
	{ 0xb7cf, 0xb7d1, PDF_CMAP_TABLE, 93 },
	{ 0xb7d2, 0xb7d6, PDF_CMAP_RANGE, 1647 },
	{ 0xb7d7, 0xb7d8, PDF_CMAP_RANGE, 7928 },
	{ 0xb7d9, 0xb7db, PDF_CMAP_RANGE, 1654 },
	{ 0xb7dc, 0xb7dc, PDF_CMAP_SINGLE, 7930 },
	{ 0xb7dd, 0xb7de, PDF_CMAP_RANGE, 1658 },
	{ 0xb7df, 0xb7e1, PDF_CMAP_RANGE, 7931 },
	{ 0xb7e2, 0xb7e3, PDF_CMAP_TABLE, 96 },
	{ 0xb7e4, 0xb7e5, PDF_CMAP_RANGE, 1665 },
	{ 0xb7e6, 0xb7e8, PDF_CMAP_RANGE, 7935 },
	{ 0xb7e9, 0xb7ea, PDF_CMAP_RANGE, 1670 },
	{ 0xb7eb, 0xb7ed, PDF_CMAP_RANGE, 7938 },
	{ 0xb7ee, 0xb7ef, PDF_CMAP_TABLE, 98 },
	{ 0xb7f0, 0xb7f3, PDF_CMAP_RANGE, 1677 },
	{ 0xb7f4, 0xb7f4, PDF_CMAP_SINGLE, 7942 },
	{ 0xb7f5, 0xb7f7, PDF_CMAP_RANGE, 1682 },
	{ 0xb7f8, 0xb7f8, PDF_CMAP_SINGLE, 7943 },
	{ 0xb7f9, 0xb7fe, PDF_CMAP_RANGE, 1686 },
	{ 0xb8a1, 0xb8a6, PDF_CMAP_RANGE, 1692 },
	{ 0xb8a7, 0xb8a8, PDF_CMAP_RANGE, 7944 },
	{ 0xb8a9, 0xb8b2, PDF_CMAP_RANGE, 1700 },
	{ 0xb8b3, 0xb8b4, PDF_CMAP_RANGE, 7946 },
	{ 0xb8b5, 0xb8b9, PDF_CMAP_RANGE, 1712 },
	{ 0xb8ba, 0xb8bd, PDF_CMAP_TABLE, 100 },
	{ 0xb8be, 0xb8bf, PDF_CMAP_RANGE, 7950 },
	{ 0xb8c0, 0xb8c2, PDF_CMAP_RANGE, 1723 },
	{ 0xb8c3, 0xb8c3, PDF_CMAP_SINGLE, 7952 },
	{ 0xb8c4, 0xb8c5, PDF_CMAP_RANGE, 1727 },
	{ 0xb8c6, 0xb8c7, PDF_CMAP_RANGE, 7953 },
	{ 0xb8c8, 0xb8c9, PDF_CMAP_TABLE, 104 },
	{ 0xb8ca, 0xb8ce, PDF_CMAP_RANGE, 1733 },
	{ 0xb8cf, 0xb8cf, PDF_CMAP_SINGLE, 7956 },
	{ 0xb8d0, 0xb8d2, PDF_CMAP_RANGE, 1739 },
	{ 0xb8d3, 0xb8d6, PDF_CMAP_RANGE, 7957 },
	{ 0xb8d7, 0xb8d8, PDF_CMAP_RANGE, 1746 },
	{ 0xb8d9, 0xb8da, PDF_CMAP_RANGE, 7961 },
	{ 0xb8db, 0xb8e3, PDF_CMAP_RANGE, 1750 },
	{ 0xb8e4, 0xb8e4, PDF_CMAP_SINGLE, 7963 },
	{ 0xb8e5, 0xb8e8, PDF_CMAP_RANGE, 1760 },
	{ 0xb8e9, 0xb8eb, PDF_CMAP_TABLE, 106 },
	{ 0xb8ec, 0xb8f2, PDF_CMAP_RANGE, 1767 },
	{ 0xb8f3, 0xb8f4, PDF_CMAP_TABLE, 109 },
	{ 0xb8f5, 0xb8f6, PDF_CMAP_RANGE, 7967 },
	{ 0xb8f7, 0xb8f8, PDF_CMAP_TABLE, 111 },
	{ 0xb8f9, 0xb8fe, PDF_CMAP_RANGE, 1780 },
	{ 0xb9a1, 0xb9a7, PDF_CMAP_RANGE, 1786 },
	{ 0xb9a8, 0xb9a8, PDF_CMAP_SINGLE, 7970 },
	{ 0xb9a9, 0xb9ad, PDF_CMAP_RANGE, 1794 },
	{ 0xb9ae, 0xb9ae, PDF_CMAP_SINGLE, 7971 },
	{ 0xb9af, 0xb9b0, PDF_CMAP_RANGE, 1800 },
	{ 0xb9b1, 0xb9b5, PDF_CMAP_TABLE, 113 },
	{ 0xb9b6, 0xb9b8, PDF_CMAP_RANGE, 1807 },
	{ 0xb9b9, 0xb9ba, PDF_CMAP_RANGE, 7975 },
	{ 0xb9bb, 0xb9c5, PDF_CMAP_RANGE, 1812 },
	{ 0xb9c6, 0xb9c6, PDF_CMAP_SINGLE, 7977 },
	{ 0xb9c7, 0xb9ca, PDF_CMAP_RANGE, 1824 },
	{ 0xb9cb, 0xb9cb, PDF_CMAP_SINGLE, 7978 },
	{ 0xb9cc, 0xb9cf, PDF_CMAP_RANGE, 1829 },
	{ 0xb9d0, 0xb9d0, PDF_CMAP_SINGLE, 7979 },
	{ 0xb9d1, 0xb9d7, PDF_CMAP_RANGE, 1834 },
	{ 0xb9d8, 0xb9d8, PDF_CMAP_SINGLE, 7980 },
	{ 0xb9d9, 0xb9da, PDF_CMAP_RANGE, 1842 },
	{ 0xb9db, 0xb9e3, PDF_CMAP_TABLE, 118 },
	{ 0xb9e4, 0xb9e5, PDF_CMAP_RANGE, 1853 },
	{ 0xb9e6, 0xb9e6, PDF_CMAP_SINGLE, 7986 },
	{ 0xb9e7, 0xb9e8, PDF_CMAP_RANGE, 1856 },
	{ 0xb9e9, 0xb9ec, PDF_CMAP_RANGE, 7987 },
	{ 0xb9ed, 0xb9ee, PDF_CMAP_TABLE, 127 },
	{ 0xb9ef, 0xb9f0, PDF_CMAP_RANGE, 1864 },
	{ 0xb9f1, 0xb9f2, PDF_CMAP_TABLE, 129 },
	{ 0xb9f3, 0xb9f5, PDF_CMAP_RANGE, 7993 },
	{ 0xb9f6, 0xb9f7, PDF_CMAP_RANGE, 1871 },
	{ 0xb9f8, 0xb9fa, PDF_CMAP_TABLE, 131 },
	{ 0xb9fb, 0xb9fc, PDF_CMAP_RANGE, 1876 },
	{ 0xb9fd, 0xb9fe, PDF_CMAP_TABLE, 134 },
	{ 0xbaa1, 0xbaa6, PDF_CMAP_RANGE, 1880 },
	{ 0xbaa7, 0xbaa7, PDF_CMAP_SINGLE, 7999 },
	{ 0xbaa8, 0xbaaa, PDF_CMAP_RANGE, 1887 },
	{ 0xbaab, 0xbaab, PDF_CMAP_SINGLE, 8000 },
	{ 0xbaac, 0xbab9, PDF_CMAP_RANGE, 1891 },
	{ 0xbaba, 0xbaba, PDF_CMAP_SINGLE, 8001 },
	{ 0xbabb, 0xbac4, PDF_CMAP_RANGE, 1906 },
	{ 0xbac5, 0xbac5, PDF_CMAP_SINGLE, 8002 },
	{ 0xbac6, 0xbad1, PDF_CMAP_RANGE, 1917 },
	{ 0xbad2, 0xbad2, PDF_CMAP_SINGLE, 8003 },
	{ 0xbad3, 0xbad6, PDF_CMAP_RANGE, 1930 },
	{ 0xbad7, 0xbad8, PDF_CMAP_RANGE, 8004 },
	{ 0xbad9, 0xbae3, PDF_CMAP_RANGE, 1936 },
	{ 0xbae4, 0xbae4, PDF_CMAP_SINGLE, 8006 },
	{ 0xbae5, 0xbae7, PDF_CMAP_RANGE, 1948 },
	{ 0xbae8, 0xbae8, PDF_CMAP_SINGLE, 8007 },
	{ 0xbae9, 0xbaeb, PDF_CMAP_RANGE, 1952 },
	{ 0xbaec, 0xbaec, PDF_CMAP_SINGLE, 8008 },
	{ 0xbaed, 0xbaf2, PDF_CMAP_RANGE, 1956 },
	{ 0xbaf3, 0xbaf3, PDF_CMAP_SINGLE, 5605 },
	{ 0xbaf4, 0xbaf7, PDF_CMAP_RANGE, 1963 },
	{ 0xbaf8, 0xbaf8, PDF_CMAP_SINGLE, 8009 },
	{ 0xbaf9, 0xbafe, PDF_CMAP_RANGE, 1968 },
	{ 0xbba1, 0xbba3, PDF_CMAP_RANGE, 1974 },
	{ 0xbba4, 0xbba6, PDF_CMAP_TABLE, 136 },
	{ 0xbba7, 0xbba8, PDF_CMAP_RANGE, 1980 },
	{ 0xbba9, 0xbbaa, PDF_CMAP_RANGE, 8012 },
	{ 0xbbab, 0xbbac, PDF_CMAP_RANGE, 1984 },
	{ 0xbbad, 0xbbae, PDF_CMAP_RANGE, 8014 },
	{ 0xbbaf, 0xbbb0, PDF_CMAP_TABLE, 139 },
	{ 0xbbb1, 0xbbb2, PDF_CMAP_RANGE, 1990 },
	{ 0xbbb3, 0xbbb4, PDF_CMAP_TABLE, 141 },
	{ 0xbbb5, 0xbbb7, PDF_CMAP_RANGE, 8018 },
	{ 0xbbb8, 0xbbb8, PDF_CMAP_SINGLE, 1997 },
	{ 0xbbb9, 0xbbba, PDF_CMAP_RANGE, 8021 },
	{ 0xbbbb, 0xbbd0, PDF_CMAP_RANGE, 2000 },
	{ 0xbbd1, 0xbbd2, PDF_CMAP_TABLE, 143 },
	{ 0xbbd3, 0xbbd4, PDF_CMAP_RANGE, 8024 },
	{ 0xbbd5, 0xbbde, PDF_CMAP_RANGE, 2026 },
	{ 0xbbdf, 0xbbe7, PDF_CMAP_RANGE, 8026 },
	{ 0xbbe8, 0xbbea, PDF_CMAP_RANGE, 2045 },
	{ 0xbbeb, 0xbbeb, PDF_CMAP_SINGLE, 8035 },
	{ 0xbbec, 0xbbf0, PDF_CMAP_RANGE, 2049 },
	{ 0xbbf1, 0xbbf1, PDF_CMAP_SINGLE, 8036 },
	{ 0xbbf2, 0xbbf4, PDF_CMAP_RANGE, 2055 },
	{ 0xbbf5, 0xbbf7, PDF_CMAP_RANGE, 8037 },
	{ 0xbbf8, 0xbbf9, PDF_CMAP_RANGE, 2061 },
	{ 0xbbfa, 0xbbfa, PDF_CMAP_SINGLE, 8040 },
	{ 0xbbfb, 0xbbfc, PDF_CMAP_RANGE, 2064 },
	{ 0xbbfd, 0xbbfe, PDF_CMAP_TABLE, 145 },
	{ 0xbca1, 0xbca2, PDF_CMAP_TABLE, 147 },
	{ 0xbca3, 0xbca4, PDF_CMAP_RANGE, 2070 },
	{ 0xbca5, 0xbca6, PDF_CMAP_RANGE, 8043 },
	{ 0xbca7, 0xbca7, PDF_CMAP_SINGLE, 2074 },
	{ 0xbca8, 0xbca9, PDF_CMAP_RANGE, 8045 },
	{ 0xbcaa, 0xbcad, PDF_CMAP_TABLE, 149 },
	{ 0xbcae, 0xbcb5, PDF_CMAP_RANGE, 2081 },
	{ 0xbcb6, 0xbcb8, PDF_CMAP_RANGE, 8049 },
	{ 0xbcb9, 0xbcba, PDF_CMAP_RANGE, 2092 },
	{ 0xbcbb, 0xbcbb, PDF_CMAP_SINGLE, 8052 },
	{ 0xbcbc, 0xbcc0, PDF_CMAP_RANGE, 2095 },
	{ 0xbcc1, 0xbcc3, PDF_CMAP_TABLE, 153 },
	{ 0xbcc4, 0xbcc5, PDF_CMAP_RANGE, 2103 },
	{ 0xbcc6, 0xbcc7, PDF_CMAP_RANGE, 8055 },
	{ 0xbcc8, 0xbcc9, PDF_CMAP_RANGE, 2107 },
	{ 0xbcca, 0xbccb, PDF_CMAP_TABLE, 156 },
	{ 0xbccc, 0xbccd, PDF_CMAP_RANGE, 8058 },
	{ 0xbcce, 0xbccf, PDF_CMAP_RANGE, 2113 },
	{ 0xbcd0, 0xbcd0, PDF_CMAP_SINGLE, 8060 },
	{ 0xbcd1, 0xbcd3, PDF_CMAP_RANGE, 2116 },
	{ 0xbcd4, 0xbcd6, PDF_CMAP_RANGE, 8061 },
	{ 0xbcd7, 0xbcd8, PDF_CMAP_TABLE, 158 },
	{ 0xbcd9, 0xbcda, PDF_CMAP_RANGE, 2124 },
	{ 0xbcdb, 0xbcde, PDF_CMAP_TABLE, 160 },
	{ 0xbcdf, 0xbce1, PDF_CMAP_RANGE, 8067 },
	{ 0xbce2, 0xbce2, PDF_CMAP_SINGLE, 2133 },
	{ 0xbce3, 0xbce4, PDF_CMAP_RANGE, 8070 },
	{ 0xbce5, 0xbce7, PDF_CMAP_RANGE, 2136 },
	{ 0xbce8, 0xbce9, PDF_CMAP_TABLE, 164 },
	{ 0xbcea, 0xbcec, PDF_CMAP_RANGE, 8073 },
	{ 0xbced, 0xbcee, PDF_CMAP_RANGE, 2144 },
	{ 0xbcef, 0xbcf3, PDF_CMAP_RANGE, 8076 },
	{ 0xbcf4, 0xbcf5, PDF_CMAP_RANGE, 2151 },
	{ 0xbcf6, 0xbcfc, PDF_CMAP_RANGE, 8081 },
	{ 0xbcfd, 0xbcfe, PDF_CMAP_RANGE, 2160 },
	{ 0xbda1, 0xbda1, PDF_CMAP_SINGLE, 2162 },
	{ 0xbda2, 0xbda7, PDF_CMAP_RANGE, 8088 },
	{ 0xbda8, 0xbdaa, PDF_CMAP_RANGE, 2169 },
	{ 0xbdab, 0xbdac, PDF_CMAP_RANGE, 8094 },
	{ 0xbdad, 0xbdae, PDF_CMAP_RANGE, 2174 },
	{ 0xbdaf, 0xbdb2, PDF_CMAP_RANGE, 8096 },
	{ 0xbdb3, 0xbdb4, PDF_CMAP_TABLE, 166 },
	{ 0xbdb5, 0xbdb9, PDF_CMAP_RANGE, 2182 },
	{ 0xbdba, 0xbdba, PDF_CMAP_SINGLE, 8101 },
	{ 0xbdbb, 0xbdbc, PDF_CMAP_RANGE, 2188 },
	{ 0xbdbd, 0xbdbf, PDF_CMAP_RANGE, 8102 },
	{ 0xbdc0, 0xbdc0, PDF_CMAP_SINGLE, 2193 },
	{ 0xbdc1, 0xbdc4, PDF_CMAP_RANGE, 8105 },
	{ 0xbdc5, 0xbdc7, PDF_CMAP_RANGE, 2198 },
	{ 0xbdc8, 0xbdca, PDF_CMAP_RANGE, 8109 },
	{ 0xbdcb, 0xbdcd, PDF_CMAP_RANGE, 2204 },
	{ 0xbdce, 0xbdcf, PDF_CMAP_RANGE, 8112 },
	{ 0xbdd0, 0xbdd6, PDF_CMAP_RANGE, 2209 },
	{ 0xbdd7, 0xbdd7, PDF_CMAP_SINGLE, 8114 },
	{ 0xbdd8, 0xbdd9, PDF_CMAP_RANGE, 2217 },
	{ 0xbdda, 0xbdda, PDF_CMAP_SINGLE, 8115 },
	{ 0xbddb, 0xbddf, PDF_CMAP_RANGE, 2220 },
	{ 0xbde0, 0xbde1, PDF_CMAP_RANGE, 8116 },
	{ 0xbde2, 0xbdea, PDF_CMAP_RANGE, 2227 },
	{ 0xbdeb, 0xbdeb, PDF_CMAP_SINGLE, 8118 },
	{ 0xbdec, 0xbdf3, PDF_CMAP_RANGE, 2237 },
	{ 0xbdf4, 0xbdf8, PDF_CMAP_RANGE, 8119 },
	{ 0xbdf9, 0xbdfc, PDF_CMAP_RANGE, 2250 },
	{ 0xbdfd, 0xbdfe, PDF_CMAP_TABLE, 168 },
	{ 0xbea1, 0xbea2, PDF_CMAP_RANGE, 8125 },
	{ 0xbea3, 0xbea4, PDF_CMAP_RANGE, 2258 },
	{ 0xbea5, 0xbea5, PDF_CMAP_SINGLE, 8127 },
	{ 0xbea6, 0xbea7, PDF_CMAP_RANGE, 2261 },
	{ 0xbea8, 0xbeaa, PDF_CMAP_TABLE, 170 },
	{ 0xbeab, 0xbeac, PDF_CMAP_RANGE, 2266 },
	{ 0xbead, 0xbead, PDF_CMAP_SINGLE, 8130 },
	{ 0xbeae, 0xbeb0, PDF_CMAP_RANGE, 2269 },
	{ 0xbeb1, 0xbeb1, PDF_CMAP_SINGLE, 8131 },
	{ 0xbeb2, 0xbeb4, PDF_CMAP_RANGE, 2273 },
	{ 0xbeb5, 0xbeb7, PDF_CMAP_RANGE, 8132 },
	{ 0xbeb8, 0xbeb9, PDF_CMAP_RANGE, 2279 },
	{ 0xbeba, 0xbeba, PDF_CMAP_SINGLE, 8135 },
	{ 0xbebb, 0xbebf, PDF_CMAP_RANGE, 2282 },
	{ 0xbec0, 0xbec0, PDF_CMAP_SINGLE, 8136 },
	{ 0xbec1, 0xbec8, PDF_CMAP_RANGE, 2288 },
	{ 0xbec9, 0xbec9, PDF_CMAP_SINGLE, 8137 },
	{ 0xbeca, 0xbed3, PDF_CMAP_RANGE, 2297 },
	{ 0xbed4, 0xbed4, PDF_CMAP_SINGLE, 8138 },
	{ 0xbed5, 0xbed8, PDF_CMAP_RANGE, 2308 },
	{ 0xbed9, 0xbed9, PDF_CMAP_SINGLE, 8139 },
	{ 0xbeda, 0xbedc, PDF_CMAP_RANGE, 2313 },
	{ 0xbedd, 0xbedd, PDF_CMAP_SINGLE, 8140 },
	{ 0xbede, 0xbee1, PDF_CMAP_RANGE, 2317 },
	{ 0xbee2, 0xbee2, PDF_CMAP_SINGLE, 8141 },
	{ 0xbee3, 0xbee4, PDF_CMAP_RANGE, 2322 },
	{ 0xbee5, 0xbee9, PDF_CMAP_TABLE, 173 },
	{ 0xbeea, 0xbeed, PDF_CMAP_RANGE, 2329 },
	{ 0xbeee, 0xbeee, PDF_CMAP_SINGLE, 8145 },
	{ 0xbeef, 0xbef4, PDF_CMAP_RANGE, 2334 },
	{ 0xbef5, 0xbef6, PDF_CMAP_TABLE, 178 },
	{ 0xbef7, 0xbef8, PDF_CMAP_RANGE, 8147 },
	{ 0xbef9, 0xbefa, PDF_CMAP_RANGE, 2344 },
	{ 0xbefb, 0xbefc, PDF_CMAP_RANGE, 8149 },
	{ 0xbefd, 0xbefe, PDF_CMAP_RANGE, 2348 },
	{ 0xbfa1, 0xbfa4, PDF_CMAP_RANGE, 2350 },
	{ 0xbfa5, 0xbfa5, PDF_CMAP_SINGLE, 8151 },
	{ 0xbfa6, 0xbfa9, PDF_CMAP_RANGE, 2355 },
	{ 0xbfaa, 0xbfaa, PDF_CMAP_SINGLE, 8152 },
	{ 0xbfab, 0xbfac, PDF_CMAP_RANGE, 2360 },
	{ 0xbfad, 0xbfad, PDF_CMAP_SINGLE, 8153 },
	{ 0xbfae, 0xbfc4, PDF_CMAP_RANGE, 2363 },
	{ 0xbfc5, 0xbfc7, PDF_CMAP_TABLE, 180 },
	{ 0xbfc8, 0xbfcd, PDF_CMAP_RANGE, 2389 },
	{ 0xbfce, 0xbfce, PDF_CMAP_SINGLE, 8156 },
	{ 0xbfcf, 0xbfd0, PDF_CMAP_RANGE, 2396 },
	{ 0xbfd1, 0xbfd2, PDF_CMAP_RANGE, 8157 },
	{ 0xbfd3, 0xbfd8, PDF_CMAP_RANGE, 2400 },
	{ 0xbfd9, 0xbfd9, PDF_CMAP_SINGLE, 8159 },
	{ 0xbfda, 0xbfe1, PDF_CMAP_RANGE, 2407 },
	{ 0xbfe2, 0xbfe4, PDF_CMAP_RANGE, 8160 },
	{ 0xbfe5, 0xbfe8, PDF_CMAP_RANGE, 2418 },
	{ 0xbfe9, 0xbfed, PDF_CMAP_TABLE, 183 },
	{ 0xbfee, 0xbff2, PDF_CMAP_RANGE, 2427 },
	{ 0xbff3, 0xbff8, PDF_CMAP_TABLE, 188 },
	{ 0xbff9, 0xbffa, PDF_CMAP_RANGE, 8169 },
	{ 0xbffb, 0xbffe, PDF_CMAP_RANGE, 2440 },
	{ 0xc0a1, 0xc0a3, PDF_CMAP_TABLE, 194 },
	{ 0xc0a4, 0xc0a8, PDF_CMAP_RANGE, 2447 },
	{ 0xc0a9, 0xc0ab, PDF_CMAP_TABLE, 197 },
	{ 0xc0ac, 0xc0ae, PDF_CMAP_RANGE, 2455 },
	{ 0xc0af, 0xc0b0, PDF_CMAP_RANGE, 8175 },
	{ 0xc0b1, 0xc0b2, PDF_CMAP_RANGE, 2460 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 8177 },
	{ 0xc0b7, 0xc0b7, PDF_CMAP_SINGLE, 2466 },
	{ 0xc0b8, 0xc0c4, PDF_CMAP_RANGE, 8181 },
	{ 0xc0c5, 0xc0cb, PDF_CMAP_RANGE, 2480 },
	{ 0xc0cc, 0xc0cd, PDF_CMAP_RANGE, 8194 },
	{ 0xc0ce, 0xc0d3, PDF_CMAP_RANGE, 2489 },
	{ 0xc0d4, 0xc0d8, PDF_CMAP_TABLE, 200 },
	{ 0xc0d9, 0xc0dc, PDF_CMAP_RANGE, 2500 },
	{ 0xc0dd, 0xc0dd, PDF_CMAP_SINGLE, 8199 },
	{ 0xc0de, 0xc0df, PDF_CMAP_RANGE, 2505 },
	{ 0xc0e0, 0xc0e0, PDF_CMAP_SINGLE, 8200 },
	{ 0xc0e1, 0xc0e8, PDF_CMAP_RANGE, 2508 },
	{ 0xc0e9, 0xc0ea, PDF_CMAP_TABLE, 205 },
	{ 0xc0eb, 0xc0ec, PDF_CMAP_RANGE, 8202 },
	{ 0xc0ed, 0xc0ee, PDF_CMAP_RANGE, 2520 },
	{ 0xc0ef, 0xc0f1, PDF_CMAP_RANGE, 8204 },
	{ 0xc0f2, 0xc0f5, PDF_CMAP_RANGE, 2525 },
	{ 0xc0f6, 0xc0fa, PDF_CMAP_RANGE, 8207 },
	{ 0xc0fb, 0xc0fe, PDF_CMAP_RANGE, 2534 },
	{ 0xc1a1, 0xc1a3, PDF_CMAP_RANGE, 2538 },
	{ 0xc1a4, 0xc1a5, PDF_CMAP_RANGE, 8212 },
	{ 0xc1a6, 0xc1a8, PDF_CMAP_RANGE, 2543 },
	{ 0xc1a9, 0xc1ad, PDF_CMAP_RANGE, 8214 },
	{ 0xc1ae, 0xc1ae, PDF_CMAP_SINGLE, 2551 },
	{ 0xc1af, 0xc1b8, PDF_CMAP_RANGE, 8219 },
	{ 0xc1b9, 0xc1bc, PDF_CMAP_RANGE, 2562 },
	{ 0xc1bd, 0xc1be, PDF_CMAP_RANGE, 8229 },
	{ 0xc1bf, 0xc1c1, PDF_CMAP_RANGE, 2568 },
	{ 0xc1c2, 0xc1c2, PDF_CMAP_SINGLE, 8231 },
	{ 0xc1c3, 0xc1c5, PDF_CMAP_RANGE, 2572 },
	{ 0xc1c6, 0xc1c6, PDF_CMAP_SINGLE, 8232 },
	{ 0xc1c7, 0xc1c8, PDF_CMAP_RANGE, 2576 },
	{ 0xc1c9, 0xc1c9, PDF_CMAP_SINGLE, 8233 },
	{ 0xc1ca, 0xc1cc, PDF_CMAP_RANGE, 2579 },
	{ 0xc1cd, 0xc1cd, PDF_CMAP_SINGLE, 8234 },
	{ 0xc1ce, 0xc1d3, PDF_CMAP_RANGE, 2583 },
	{ 0xc1d4, 0xc1d4, PDF_CMAP_SINGLE, 8235 },
	{ 0xc1d5, 0xc1d8, PDF_CMAP_RANGE, 2590 },
	{ 0xc1d9, 0xc1db, PDF_CMAP_RANGE, 8236 },
	{ 0xc1dc, 0xc1dd, PDF_CMAP_RANGE, 2597 },
	{ 0xc1de, 0xc1de, PDF_CMAP_SINGLE, 8239 },
	{ 0xc1df, 0xc1e3, PDF_CMAP_RANGE, 2600 },
	{ 0xc1e4, 0xc1e5, PDF_CMAP_RANGE, 8240 },
	{ 0xc1e6, 0xc1e8, PDF_CMAP_RANGE, 2607 },
	{ 0xc1e9, 0xc1ea, PDF_CMAP_TABLE, 207 },
	{ 0xc1eb, 0xc1ec, PDF_CMAP_RANGE, 8243 },
	{ 0xc1ed, 0xc1f2, PDF_CMAP_RANGE, 2614 },
	{ 0xc1f3, 0xc1f5, PDF_CMAP_TABLE, 209 },
	{ 0xc1f6, 0xc1f9, PDF_CMAP_RANGE, 2623 },
	{ 0xc1fa, 0xc1fd, PDF_CMAP_RANGE, 8247 },
	{ 0xc1fe, 0xc1fe, PDF_CMAP_SINGLE, 2631 },
	{ 0xc2a1, 0xc2a1, PDF_CMAP_SINGLE, 2632 },
	{ 0xc2a2, 0xc2a8, PDF_CMAP_RANGE, 8251 },
	{ 0xc2a9, 0xc2aa, PDF_CMAP_RANGE, 2640 },
	{ 0xc2ab, 0xc2b3, PDF_CMAP_RANGE, 8258 },
	{ 0xc2b4, 0xc2b7, PDF_CMAP_RANGE, 2651 },
	{ 0xc2b8, 0xc2b8, PDF_CMAP_SINGLE, 8267 },
	{ 0xc2b9, 0xc2bb, PDF_CMAP_RANGE, 2656 },
	{ 0xc2bc, 0xc2bd, PDF_CMAP_RANGE, 8268 },
	{ 0xc2be, 0xc2c1, PDF_CMAP_TABLE, 212 },
	{ 0xc2c2, 0xc2c4, PDF_CMAP_RANGE, 2665 },
	{ 0xc2c5, 0xc2c7, PDF_CMAP_RANGE, 8272 },
	{ 0xc2c8, 0xc2ca, PDF_CMAP_RANGE, 2671 },
	{ 0xc2cb, 0xc2d0, PDF_CMAP_RANGE, 8275 },
	{ 0xc2d1, 0xc2d2, PDF_CMAP_TABLE, 216 },
	{ 0xc2d3, 0xc2d4, PDF_CMAP_RANGE, 2682 },
	{ 0xc2d5, 0xc2dc, PDF_CMAP_RANGE, 8282 },
	{ 0xc2dd, 0xc2dd, PDF_CMAP_SINGLE, 2692 },
	{ 0xc2de, 0xc2e2, PDF_CMAP_RANGE, 8290 },
	{ 0xc2e3, 0xc2e5, PDF_CMAP_RANGE, 2698 },
	{ 0xc2e6, 0xc2e8, PDF_CMAP_RANGE, 8295 },
	{ 0xc2e9, 0xc2e9, PDF_CMAP_SINGLE, 2704 },
	{ 0xc2ea, 0xc2ee, PDF_CMAP_RANGE, 8298 },
	{ 0xc2ef, 0xc2f1, PDF_CMAP_TABLE, 218 },
	{ 0xc2f2, 0xc2f5, PDF_CMAP_RANGE, 8304 },
	{ 0xc2f6, 0xc2f6, PDF_CMAP_SINGLE, 2717 },
	{ 0xc2f7, 0xc2fa, PDF_CMAP_RANGE, 8308 },
	{ 0xc2fb, 0xc2fe, PDF_CMAP_RANGE, 2722 },
	{ 0xc3a1, 0xc3a1, PDF_CMAP_SINGLE, 8312 },
	{ 0xc3a2, 0xc3a9, PDF_CMAP_RANGE, 2727 },
	{ 0xc3aa, 0xc3aa, PDF_CMAP_SINGLE, 8313 },
	{ 0xc3ab, 0xc3ac, PDF_CMAP_RANGE, 2736 },
	{ 0xc3ad, 0xc3ad, PDF_CMAP_SINGLE, 8314 },
	{ 0xc3ae, 0xc3b2, PDF_CMAP_RANGE, 2739 },
	{ 0xc3b3, 0xc3b4, PDF_CMAP_TABLE, 221 },
	{ 0xc3b5, 0xc3bd, PDF_CMAP_RANGE, 2746 },
	{ 0xc3be, 0xc3be, PDF_CMAP_SINGLE, 8316 },
	{ 0xc3bf, 0xc3c4, PDF_CMAP_RANGE, 2756 },
	{ 0xc3c5, 0xc3c7, PDF_CMAP_RANGE, 8317 },
	{ 0xc3c8, 0xc3cb, PDF_CMAP_RANGE, 2765 },
	{ 0xc3cc, 0xc3ce, PDF_CMAP_TABLE, 223 },
	{ 0xc3cf, 0xc3d4, PDF_CMAP_RANGE, 2772 },
	{ 0xc3d5, 0xc3d6, PDF_CMAP_RANGE, 8322 },
	{ 0xc3d7, 0xc3d8, PDF_CMAP_RANGE, 2780 },
	{ 0xc3d9, 0xc3d9, PDF_CMAP_SINGLE, 8324 },
	{ 0xc3da, 0xc3df, PDF_CMAP_RANGE, 2783 },
	{ 0xc3e0, 0xc3e0, PDF_CMAP_SINGLE, 8325 },
	{ 0xc3e1, 0xc3e4, PDF_CMAP_RANGE, 2790 },
	{ 0xc3e5, 0xc3e5, PDF_CMAP_SINGLE, 8326 },
	{ 0xc3e6, 0xc3ec, PDF_CMAP_RANGE, 2795 },
	{ 0xc3ed, 0xc3ed, PDF_CMAP_SINGLE, 8327 },
	{ 0xc3ee, 0xc3ef, PDF_CMAP_RANGE, 2803 },
	{ 0xc3f0, 0xc3f0, PDF_CMAP_SINGLE, 8328 },
	{ 0xc3f1, 0xc3f4, PDF_CMAP_RANGE, 2806 },
	{ 0xc3f5, 0xc3f6, PDF_CMAP_RANGE, 8329 },
	{ 0xc3f7, 0xc3f8, PDF_CMAP_RANGE, 2812 },
	{ 0xc3f9, 0xc3fa, PDF_CMAP_RANGE, 8331 },
	{ 0xc3fb, 0xc3fc, PDF_CMAP_RANGE, 2816 },
	{ 0xc3fd, 0xc3fe, PDF_CMAP_TABLE, 226 },
	{ 0xc4a1, 0xc4b0, PDF_CMAP_RANGE, 2820 },
	{ 0xc4b1, 0xc4b1, PDF_CMAP_SINGLE, 8334 },
	{ 0xc4b2, 0xc4b5, PDF_CMAP_RANGE, 2837 },
	{ 0xc4b6, 0xc4b6, PDF_CMAP_SINGLE, 8335 },
	{ 0xc4b7, 0xc4c5, PDF_CMAP_RANGE, 2842 },
	{ 0xc4c6, 0xc4c6, PDF_CMAP_SINGLE, 8336 },
	{ 0xc4c7, 0xc4c8, PDF_CMAP_RANGE, 2858 },
	{ 0xc4c9, 0xc4c9, PDF_CMAP_SINGLE, 8337 },
	{ 0xc4ca, 0xc4d0, PDF_CMAP_RANGE, 2861 },
	{ 0xc4d1, 0xc4d2, PDF_CMAP_TABLE, 228 },
	{ 0xc4d3, 0xc4d6, PDF_CMAP_RANGE, 8339 },
	{ 0xc4d7, 0xc4d8, PDF_CMAP_RANGE, 2874 },
	{ 0xc4d9, 0xc4d9, PDF_CMAP_SINGLE, 8343 },
	{ 0xc4da, 0xc4e1, PDF_CMAP_RANGE, 2877 },
	{ 0xc4e2, 0xc4e2, PDF_CMAP_SINGLE, 8344 },
	{ 0xc4e3, 0xc4e4, PDF_CMAP_RANGE, 2886 },
	{ 0xc4e5, 0xc4e5, PDF_CMAP_SINGLE, 8345 },
	{ 0xc4e6, 0xc4eb, PDF_CMAP_RANGE, 2889 },
	{ 0xc4ec, 0xc4ec, PDF_CMAP_SINGLE, 8346 },
	{ 0xc4ed, 0xc4ef, PDF_CMAP_RANGE, 2896 },
	{ 0xc4f0, 0xc4f1, PDF_CMAP_RANGE, 8347 },
	{ 0xc4f2, 0xc4f3, PDF_CMAP_RANGE, 2901 },
	{ 0xc4f4, 0xc4f5, PDF_CMAP_TABLE, 230 },
	{ 0xc4f6, 0xc4f8, PDF_CMAP_RANGE, 8350 },
	{ 0xc4f9, 0xc4fa, PDF_CMAP_RANGE, 2908 },
	{ 0xc4fb, 0xc4fc, PDF_CMAP_RANGE, 8353 },
	{ 0xc4fd, 0xc4fe, PDF_CMAP_TABLE, 232 },
	{ 0xc5a1, 0xc5a2, PDF_CMAP_RANGE, 8356 },
	{ 0xc5a3, 0xc5a4, PDF_CMAP_RANGE, 2916 },
	{ 0xc5a5, 0xc5a9, PDF_CMAP_RANGE, 8358 },
	{ 0xc5aa, 0xc5b0, PDF_CMAP_RANGE, 2923 },
	{ 0xc5b1, 0xc5b1, PDF_CMAP_SINGLE, 8363 },
	{ 0xc5b2, 0xc5b4, PDF_CMAP_RANGE, 2931 },
	{ 0xc5b5, 0xc5b6, PDF_CMAP_TABLE, 234 },
	{ 0xc5b7, 0xc5b9, PDF_CMAP_RANGE, 8365 },
	{ 0xc5ba, 0xc5bd, PDF_CMAP_TABLE, 236 },
	{ 0xc5be, 0xc5cb, PDF_CMAP_RANGE, 2943 },
	{ 0xc5cc, 0xc5cc, PDF_CMAP_SINGLE, 8370 },
	{ 0xc5cd, 0xc5d2, PDF_CMAP_RANGE, 2958 },
	{ 0xc5d3, 0xc5d3, PDF_CMAP_SINGLE, 8371 },
	{ 0xc5d4, 0xc5e1, PDF_CMAP_RANGE, 2965 },
	{ 0xc5e2, 0xc5e2, PDF_CMAP_SINGLE, 8372 },
	{ 0xc5e3, 0xc5e6, PDF_CMAP_RANGE, 2980 },
	{ 0xc5e7, 0xc5e7, PDF_CMAP_SINGLE, 8373 },
	{ 0xc5e8, 0xc5f3, PDF_CMAP_RANGE, 2985 },
	{ 0xc5f4, 0xc5f4, PDF_CMAP_SINGLE, 8374 },
	{ 0xc5f5, 0xc5fe, PDF_CMAP_RANGE, 2998 },
	{ 0xc6a1, 0xc6ac, PDF_CMAP_RANGE, 3008 },
	{ 0xc6ad, 0xc6ae, PDF_CMAP_RANGE, 8375 },
	{ 0xc6af, 0xc6b4, PDF_CMAP_RANGE, 3022 },
	{ 0xc6b5, 0xc6b6, PDF_CMAP_RANGE, 8377 },
	{ 0xc6b7, 0xc6ba, PDF_CMAP_RANGE, 3030 },
	{ 0xc6bb, 0xc6bb, PDF_CMAP_SINGLE, 8379 },
	{ 0xc6bc, 0xc6bd, PDF_CMAP_RANGE, 3035 },
	{ 0xc6be, 0xc6c0, PDF_CMAP_TABLE, 240 },
	{ 0xc6c1, 0xc6c2, PDF_CMAP_RANGE, 3040 },
	{ 0xc6c3, 0xc6c4, PDF_CMAP_RANGE, 8382 },
	{ 0xc6c5, 0xc6ca, PDF_CMAP_RANGE, 3044 },
	{ 0xc6cb, 0xc6cd, PDF_CMAP_RANGE, 8384 },
	{ 0xc6ce, 0xc6d2, PDF_CMAP_RANGE, 3053 },
	{ 0xc6d3, 0xc6d3, PDF_CMAP_SINGLE, 8387 },
	{ 0xc6d4, 0xc6d6, PDF_CMAP_RANGE, 3059 },
	{ 0xc6d7, 0xc6d7, PDF_CMAP_SINGLE, 8388 },
	{ 0xc6d8, 0xc6e9, PDF_CMAP_RANGE, 3063 },
	{ 0xc6ea, 0xc6eb, PDF_CMAP_RANGE, 8389 },
	{ 0xc6ec, 0xc6ee, PDF_CMAP_RANGE, 3083 },
	{ 0xc6ef, 0xc6f1, PDF_CMAP_TABLE, 243 },
	{ 0xc6f2, 0xc6f3, PDF_CMAP_RANGE, 3089 },
	{ 0xc6f4, 0xc6f4, PDF_CMAP_SINGLE, 8393 },
	{ 0xc6f5, 0xc6f7, PDF_CMAP_RANGE, 3092 },
	{ 0xc6f8, 0xc6f8, PDF_CMAP_SINGLE, 8394 },
	{ 0xc6f9, 0xc6fc, PDF_CMAP_RANGE, 3096 },
	{ 0xc6fd, 0xc6fe, PDF_CMAP_TABLE, 246 },
	{ 0xc7a1, 0xc7a2, PDF_CMAP_RANGE, 3102 },
	{ 0xc7a3, 0xc7a4, PDF_CMAP_TABLE, 248 },
	{ 0xc7a5, 0xc7a6, PDF_CMAP_RANGE, 8397 },
	{ 0xc7a7, 0xc7a7, PDF_CMAP_SINGLE, 3108 },
	{ 0xc7a8, 0xc7a9, PDF_CMAP_RANGE, 8399 },
	{ 0xc7aa, 0xc7ab, PDF_CMAP_TABLE, 250 },
	{ 0xc7ac, 0xc7ad, PDF_CMAP_RANGE, 3113 },
	{ 0xc7ae, 0xc7af, PDF_CMAP_RANGE, 8402 },
	{ 0xc7b0, 0xc7b2, PDF_CMAP_RANGE, 3117 },
	{ 0xc7b3, 0xc7b5, PDF_CMAP_RANGE, 8404 },
	{ 0xc7b6, 0xc7b8, PDF_CMAP_RANGE, 3123 },
	{ 0xc7b9, 0xc7ba, PDF_CMAP_RANGE, 8407 },
	{ 0xc7bb, 0xc7bc, PDF_CMAP_RANGE, 3128 },
	{ 0xc7bd, 0xc7be, PDF_CMAP_RANGE, 8409 },
	{ 0xc7bf, 0xc7c2, PDF_CMAP_TABLE, 252 },
	{ 0xc7c3, 0xc7c4, PDF_CMAP_RANGE, 3136 },
	{ 0xc7c5, 0xc7c6, PDF_CMAP_TABLE, 256 },
	{ 0xc7c7, 0xc7c8, PDF_CMAP_RANGE, 8414 },
	{ 0xc7c9, 0xc7cb, PDF_CMAP_RANGE, 3142 },
	{ 0xc7cc, 0xc7cc, PDF_CMAP_SINGLE, 8416 },
	{ 0xc7cd, 0xc7ce, PDF_CMAP_RANGE, 3146 },
	{ 0xc7cf, 0xc7cf, PDF_CMAP_SINGLE, 8417 },
	{ 0xc7d0, 0xc7d3, PDF_CMAP_RANGE, 3149 },
	{ 0xc7d4, 0xc7d5, PDF_CMAP_RANGE, 8418 },
	{ 0xc7d6, 0xc7d7, PDF_CMAP_TABLE, 258 },
	{ 0xc7d8, 0xc7dd, PDF_CMAP_RANGE, 3157 },
	{ 0xc7de, 0xc7de, PDF_CMAP_SINGLE, 8421 },
	{ 0xc7df, 0xc7e0, PDF_CMAP_RANGE, 3164 },
	{ 0xc7e1, 0xc7e3, PDF_CMAP_RANGE, 8422 },
	{ 0xc7e4, 0xc7e9, PDF_CMAP_RANGE, 3169 },
	{ 0xc7ea, 0xc7ee, PDF_CMAP_RANGE, 8425 },
	{ 0xc7ef, 0xc7f6, PDF_CMAP_RANGE, 3180 },
	{ 0xc7f7, 0xc7f8, PDF_CMAP_RANGE, 8430 },
	{ 0xc7f9, 0xc7fa, PDF_CMAP_RANGE, 3190 },
	{ 0xc7fb, 0xc7fe, PDF_CMAP_TABLE, 260 },
	{ 0xc8a1, 0xc8a2, PDF_CMAP_RANGE, 3196 },
	{ 0xc8a3, 0xc8a3, PDF_CMAP_SINGLE, 8434 },
	{ 0xc8a4, 0xc8a6, PDF_CMAP_RANGE, 3199 },
	{ 0xc8a7, 0xc8a8, PDF_CMAP_RANGE, 8435 },
	{ 0xc8a9, 0xc8af, PDF_CMAP_RANGE, 3204 },
	{ 0xc8b0, 0xc8b0, PDF_CMAP_SINGLE, 8437 },
	{ 0xc8b1, 0xc8b4, PDF_CMAP_RANGE, 3212 },
	{ 0xc8b5, 0xc8b7, PDF_CMAP_TABLE, 264 },
	{ 0xc8b8, 0xc8c2, PDF_CMAP_RANGE, 3219 },
	{ 0xc8c3, 0xc8c6, PDF_CMAP_RANGE, 8440 },
	{ 0xc8c7, 0xc8c8, PDF_CMAP_TABLE, 267 },
	{ 0xc8c9, 0xc8cc, PDF_CMAP_RANGE, 3236 },
	{ 0xc8cd, 0xc8cf, PDF_CMAP_TABLE, 269 },
	{ 0xc8d0, 0xc8d1, PDF_CMAP_RANGE, 3243 },
	{ 0xc8d2, 0xc8d2, PDF_CMAP_SINGLE, 8447 },
	{ 0xc8d3, 0xc8d8, PDF_CMAP_RANGE, 3246 },
	{ 0xc8d9, 0xc8d9, PDF_CMAP_SINGLE, 8448 },
	{ 0xc8da, 0xc8dd, PDF_CMAP_RANGE, 3253 },
	{ 0xc8de, 0xc8de, PDF_CMAP_SINGLE, 8449 },
	{ 0xc8df, 0xc8ec, PDF_CMAP_RANGE, 3258 },
	{ 0xc8ed, 0xc8ed, PDF_CMAP_SINGLE, 8450 },
	{ 0xc8ee, 0xc8f0, PDF_CMAP_RANGE, 3273 },
	{ 0xc8f1, 0xc8f3, PDF_CMAP_RANGE, 8451 },
	{ 0xc8f4, 0xc8f6, PDF_CMAP_RANGE, 3279 },
	{ 0xc8f7, 0xc8f8, PDF_CMAP_RANGE, 8454 },
	{ 0xc8f9, 0xc8fc, PDF_CMAP_TABLE, 272 },
	{ 0xc8fd, 0xc8fe, PDF_CMAP_RANGE, 3288 },
	{ 0xc9a1, 0xc9a1, PDF_CMAP_SINGLE, 8458 },
	{ 0xc9a2, 0xc9a4, PDF_CMAP_RANGE, 3291 },
	{ 0xc9a5, 0xc9a6, PDF_CMAP_TABLE, 276 },
	{ 0xc9a7, 0xc9a8, PDF_CMAP_RANGE, 8460 },
	{ 0xc9a9, 0xc9ab, PDF_CMAP_RANGE, 3298 },
	{ 0xc9ac, 0xc9ac, PDF_CMAP_SINGLE, 8462 },
	{ 0xc9ad, 0xc9b0, PDF_CMAP_RANGE, 3302 },
	{ 0xc9b1, 0xc9b1, PDF_CMAP_SINGLE, 8463 },
	{ 0xc9b2, 0xc9b3, PDF_CMAP_RANGE, 3307 },
	{ 0xc9b4, 0xc9b4, PDF_CMAP_SINGLE, 8464 },
	{ 0xc9b5, 0xc9b7, PDF_CMAP_RANGE, 3310 },
	{ 0xc9b8, 0xc9b9, PDF_CMAP_RANGE, 8465 },
	{ 0xc9ba, 0xc9c0, PDF_CMAP_RANGE, 3315 },
	{ 0xc9c1, 0xc9c2, PDF_CMAP_RANGE, 8467 },
	{ 0xc9c3, 0xc9c4, PDF_CMAP_TABLE, 278 },
	{ 0xc9c5, 0xc9c8, PDF_CMAP_RANGE, 3326 },
	{ 0xc9c9, 0xc9cd, PDF_CMAP_TABLE, 280 },
	{ 0xc9ce, 0xc9d4, PDF_CMAP_RANGE, 3335 },
	{ 0xc9d5, 0xc9d5, PDF_CMAP_SINGLE, 8473 },
	{ 0xc9d6, 0xc9db, PDF_CMAP_RANGE, 3343 },
	{ 0xc9dc, 0xc9de, PDF_CMAP_TABLE, 285 },
	{ 0xc9df, 0xc9e0, PDF_CMAP_RANGE, 3352 },
	{ 0xc9e1, 0xc9e5, PDF_CMAP_TABLE, 288 },
	{ 0xc9e6, 0xc9e7, PDF_CMAP_RANGE, 3359 },
	{ 0xc9e8, 0xc9e8, PDF_CMAP_SINGLE, 8479 },
	{ 0xc9e9, 0xc9ef, PDF_CMAP_RANGE, 3362 },
	{ 0xc9f0, 0xc9f0, PDF_CMAP_SINGLE, 8480 },
	{ 0xc9f1, 0xc9f2, PDF_CMAP_RANGE, 3370 },
	{ 0xc9f3, 0xc9f4, PDF_CMAP_RANGE, 8481 },
	{ 0xc9f5, 0xc9f7, PDF_CMAP_TABLE, 293 },
	{ 0xc9f8, 0xc9f9, PDF_CMAP_RANGE, 8484 },
	{ 0xc9fa, 0xc9fd, PDF_CMAP_RANGE, 3379 },
	{ 0xc9fe, 0xc9fe, PDF_CMAP_SINGLE, 8486 },
	{ 0xcaa1, 0xcaa3, PDF_CMAP_RANGE, 3384 },
	{ 0xcaa4, 0xcaa6, PDF_CMAP_RANGE, 8487 },
	{ 0xcaa7, 0xcaa9, PDF_CMAP_TABLE, 296 },
	{ 0xcaaa, 0xcaab, PDF_CMAP_RANGE, 8491 },
	{ 0xcaac, 0xcab0, PDF_CMAP_RANGE, 3395 },
	{ 0xcab1, 0xcab1, PDF_CMAP_SINGLE, 8493 },
	{ 0xcab2, 0xcab3, PDF_CMAP_RANGE, 3401 },
	{ 0xcab4, 0xcab6, PDF_CMAP_RANGE, 8494 },
	{ 0xcab7, 0xcaba, PDF_CMAP_RANGE, 3406 },
	{ 0xcabb, 0xcabb, PDF_CMAP_SINGLE, 8497 },
	{ 0xcabc, 0xcac5, PDF_CMAP_RANGE, 3411 },
	{ 0xcac6, 0xcac6, PDF_CMAP_SINGLE, 8498 },
	{ 0xcac7, 0xcac9, PDF_CMAP_RANGE, 3422 },
	{ 0xcaca, 0xcaca, PDF_CMAP_SINGLE, 8499 },
	{ 0xcacb, 0xcacc, PDF_CMAP_RANGE, 3426 },
	{ 0xcacd, 0xcace, PDF_CMAP_RANGE, 8500 },
	{ 0xcacf, 0xcad2, PDF_CMAP_RANGE, 3430 },
	{ 0xcad3, 0xcad4, PDF_CMAP_RANGE, 8502 },
	{ 0xcad5, 0xcad8, PDF_CMAP_RANGE, 3436 },
	{ 0xcad9, 0xcad9, PDF_CMAP_SINGLE, 8504 },
	{ 0xcada, 0xcadd, PDF_CMAP_RANGE, 3441 },
	{ 0xcade, 0xcae0, PDF_CMAP_TABLE, 299 },
	{ 0xcae1, 0xcae3, PDF_CMAP_RANGE, 3448 },
	{ 0xcae4, 0xcae4, PDF_CMAP_SINGLE, 8507 },
	{ 0xcae5, 0xcae8, PDF_CMAP_RANGE, 3452 },
	{ 0xcae9, 0xcaea, PDF_CMAP_RANGE, 8508 },
	{ 0xcaeb, 0xcaf3, PDF_CMAP_RANGE, 3458 },
	{ 0xcaf4, 0xcaf5, PDF_CMAP_RANGE, 8510 },
	{ 0xcaf6, 0xcaf7, PDF_CMAP_TABLE, 302 },
	{ 0xcaf8, 0xcaf9, PDF_CMAP_RANGE, 3471 },
	{ 0xcafa, 0xcafa, PDF_CMAP_SINGLE, 8513 },
	{ 0xcafb, 0xcafc, PDF_CMAP_RANGE, 3474 },
	{ 0xcafd, 0xcafe, PDF_CMAP_TABLE, 304 },
	{ 0xcba1, 0xcba6, PDF_CMAP_RANGE, 3478 },
	{ 0xcba7, 0xcba7, PDF_CMAP_SINGLE, 8515 },
	{ 0xcba8, 0xcbaa, PDF_CMAP_RANGE, 3485 },
	{ 0xcbab, 0xcbad, PDF_CMAP_TABLE, 306 },
	{ 0xcbae, 0xcbb2, PDF_CMAP_RANGE, 3491 },
	{ 0xcbb3, 0xcbb4, PDF_CMAP_TABLE, 309 },
	{ 0xcbb5, 0xcbb6, PDF_CMAP_RANGE, 8519 },
	{ 0xcbb7, 0xcbb8, PDF_CMAP_TABLE, 311 },
	{ 0xcbb9, 0xcbbe, PDF_CMAP_RANGE, 3502 },
	{ 0xcbbf, 0xcbbf, PDF_CMAP_SINGLE, 8522 },
	{ 0xcbc0, 0xcbc6, PDF_CMAP_RANGE, 3509 },
	{ 0xcbc7, 0xcbc8, PDF_CMAP_TABLE, 313 },
	{ 0xcbc9, 0xcbcc, PDF_CMAP_RANGE, 8524 },
	{ 0xcbcd, 0xcbce, PDF_CMAP_RANGE, 3522 },
	{ 0xcbcf, 0xcbd0, PDF_CMAP_RANGE, 8528 },
	{ 0xcbd1, 0xcbd2, PDF_CMAP_RANGE, 3526 },
	{ 0xcbd3, 0xcbd5, PDF_CMAP_TABLE, 315 },
	{ 0xcbd6, 0xcbde, PDF_CMAP_RANGE, 3531 },
	{ 0xcbdf, 0xcbe0, PDF_CMAP_RANGE, 8532 },
	{ 0xcbe1, 0xcbe3, PDF_CMAP_RANGE, 3542 },
	{ 0xcbe4, 0xcbe5, PDF_CMAP_TABLE, 318 },
	{ 0xcbe6, 0xcbe7, PDF_CMAP_RANGE, 8535 },
	{ 0xcbe8, 0xcbe9, PDF_CMAP_RANGE, 3549 },
	{ 0xcbea, 0xcbea, PDF_CMAP_SINGLE, 8537 },
	{ 0xcbeb, 0xcbee, PDF_CMAP_RANGE, 3552 },
	{ 0xcbef, 0xcbf0, PDF_CMAP_RANGE, 8538 },
	{ 0xcbf1, 0xcbf4, PDF_CMAP_RANGE, 3558 },
	{ 0xcbf5, 0xcbf6, PDF_CMAP_RANGE, 8540 },
	{ 0xcbf7, 0xcbf8, PDF_CMAP_TABLE, 320 },
	{ 0xcbf9, 0xcbfe, PDF_CMAP_RANGE, 3566 },
	{ 0xcca1, 0xcca2, PDF_CMAP_RANGE, 8543 },
	{ 0xcca3, 0xcca7, PDF_CMAP_RANGE, 3574 },
	{ 0xcca8, 0xcca8, PDF_CMAP_SINGLE, 8545 },
	{ 0xcca9, 0xccab, PDF_CMAP_RANGE, 3580 },
	{ 0xccac, 0xccac, PDF_CMAP_SINGLE, 8546 },
	{ 0xccad, 0xccae, PDF_CMAP_RANGE, 3584 },
	{ 0xccaf, 0xccb3, PDF_CMAP_RANGE, 8547 },
	{ 0xccb4, 0xccb6, PDF_CMAP_RANGE, 3591 },
	{ 0xccb7, 0xccb8, PDF_CMAP_RANGE, 8552 },
	{ 0xccb9, 0xccbd, PDF_CMAP_RANGE, 3596 },
	{ 0xccbe, 0xccc0, PDF_CMAP_TABLE, 322 },
	{ 0xccc1, 0xcccb, PDF_CMAP_RANGE, 3604 },
	{ 0xcccc, 0xccd0, PDF_CMAP_TABLE, 325 },
	{ 0xccd1, 0xccd5, PDF_CMAP_RANGE, 3620 },
	{ 0xccd6, 0xccd6, PDF_CMAP_SINGLE, 8559 },
	{ 0xccd7, 0xccd9, PDF_CMAP_RANGE, 3626 },
	{ 0xccda, 0xccdc, PDF_CMAP_TABLE, 330 },
	{ 0xccdd, 0xccdf, PDF_CMAP_RANGE, 3632 },
	{ 0xcce0, 0xcce2, PDF_CMAP_TABLE, 333 },
	{ 0xcce3, 0xcce4, PDF_CMAP_RANGE, 3638 },
	{ 0xcce5, 0xcce5, PDF_CMAP_SINGLE, 8564 },
	{ 0xcce6, 0xccf4, PDF_CMAP_RANGE, 3641 },
	{ 0xccf5, 0xccf5, PDF_CMAP_SINGLE, 8565 },
	{ 0xccf6, 0xccf8, PDF_CMAP_RANGE, 3657 },
	{ 0xccf9, 0xccfa, PDF_CMAP_RANGE, 8566 },
	{ 0xccfb, 0xccfb, PDF_CMAP_SINGLE, 3662 },
	{ 0xccfc, 0xccfe, PDF_CMAP_RANGE, 8568 },
	{ 0xcda1, 0xcdac, PDF_CMAP_RANGE, 3666 },
	{ 0xcdad, 0xcdad, PDF_CMAP_SINGLE, 8571 },
	{ 0xcdae, 0xcdb2, PDF_CMAP_RANGE, 3679 },
	{ 0xcdb3, 0xcdb3, PDF_CMAP_SINGLE, 8572 },
	{ 0xcdb4, 0xcdb6, PDF_CMAP_RANGE, 3685 },
	{ 0xcdb7, 0xcdb7, PDF_CMAP_SINGLE, 8573 },
	{ 0xcdb8, 0xcdbb, PDF_CMAP_RANGE, 3689 },
	{ 0xcdbc, 0xcdbc, PDF_CMAP_SINGLE, 8574 },
	{ 0xcdbd, 0xcdbe, PDF_CMAP_RANGE, 3694 },
	{ 0xcdbf, 0xcdbf, PDF_CMAP_SINGLE, 8575 },
	{ 0xcdc0, 0xcdc4, PDF_CMAP_RANGE, 3697 },
	{ 0xcdc5, 0xcdc7, PDF_CMAP_TABLE, 336 },
	{ 0xcdc8, 0xcdd1, PDF_CMAP_RANGE, 3705 },
	{ 0xcdd2, 0xcdd3, PDF_CMAP_TABLE, 339 },
	{ 0xcdd4, 0xcdd6, PDF_CMAP_RANGE, 8579 },
	{ 0xcdd7, 0xcddc, PDF_CMAP_RANGE, 3720 },
	{ 0xcddd, 0xcddd, PDF_CMAP_SINGLE, 8582 },
	{ 0xcdde, 0xcddf, PDF_CMAP_RANGE, 3727 },
	{ 0xcde0, 0xcde0, PDF_CMAP_SINGLE, 8583 },
	{ 0xcde1, 0xcde3, PDF_CMAP_RANGE, 3730 },
	{ 0xcde4, 0xcde5, PDF_CMAP_RANGE, 8584 },
	{ 0xcde6, 0xcde7, PDF_CMAP_TABLE, 341 },
	{ 0xcde8, 0xcdf1, PDF_CMAP_RANGE, 3737 },
	{ 0xcdf2, 0xcdf2, PDF_CMAP_SINGLE, 8587 },
	{ 0xcdf3, 0xcdf7, PDF_CMAP_RANGE, 3748 },
	{ 0xcdf8, 0xcdf8, PDF_CMAP_SINGLE, 8588 },
	{ 0xcdf9, 0xcdfe, PDF_CMAP_RANGE, 3754 },
	{ 0xcea1, 0xcea3, PDF_CMAP_RANGE, 3760 },
	{ 0xcea4, 0xcea5, PDF_CMAP_RANGE, 8589 },
	{ 0xcea6, 0xcea7, PDF_CMAP_TABLE, 343 },
	{ 0xcea8, 0xcea9, PDF_CMAP_RANGE, 3767 },
	{ 0xceaa, 0xcead, PDF_CMAP_RANGE, 8592 },
	{ 0xceae, 0xceaf, PDF_CMAP_RANGE, 3773 },
	{ 0xceb0, 0xceb1, PDF_CMAP_RANGE, 8596 },
	{ 0xceb2, 0xceb3, PDF_CMAP_TABLE, 345 },
	{ 0xceb4, 0xcebc, PDF_CMAP_RANGE, 3779 },
	{ 0xcebd, 0xcebd, PDF_CMAP_SINGLE, 8599 },
	{ 0xcebe, 0xcebf, PDF_CMAP_RANGE, 3789 },
	{ 0xcec0, 0xcec0, PDF_CMAP_SINGLE, 8600 },
	{ 0xcec1, 0xcec4, PDF_CMAP_RANGE, 3792 },
	{ 0xcec5, 0xcec6, PDF_CMAP_RANGE, 8601 },
	{ 0xcec7, 0xceca, PDF_CMAP_TABLE, 347 },
	{ 0xcecb, 0xcecd, PDF_CMAP_RANGE, 3802 },
	{ 0xcece, 0xced1, PDF_CMAP_RANGE, 8605 },
	{ 0xced2, 0xced7, PDF_CMAP_RANGE, 3809 },
	{ 0xced8, 0xceda, PDF_CMAP_RANGE, 8609 },
	{ 0xcedb, 0xcedd, PDF_CMAP_TABLE, 351 },
	{ 0xcede, 0xcedf, PDF_CMAP_RANGE, 8613 },
	{ 0xcee0, 0xceea, PDF_CMAP_RANGE, 3823 },
	{ 0xceeb, 0xceed, PDF_CMAP_TABLE, 354 },
	{ 0xceee, 0xcef0, PDF_CMAP_RANGE, 3837 },
	{ 0xcef1, 0xcef3, PDF_CMAP_TABLE, 357 },
	{ 0xcef4, 0xcefc, PDF_CMAP_RANGE, 3843 },
	{ 0xcefd, 0xcefe, PDF_CMAP_RANGE, 8619 },
	{ 0xcfa1, 0xcfad, PDF_CMAP_RANGE, 3854 },
	{ 0xcfae, 0xcfb0, PDF_CMAP_TABLE, 360 },
	{ 0xcfb1, 0xcfb2, PDF_CMAP_RANGE, 3870 },
	{ 0xcfb3, 0xcfb3, PDF_CMAP_SINGLE, 8623 },
	{ 0xcfb4, 0xcfb6, PDF_CMAP_RANGE, 3873 },
	{ 0xcfb7, 0xcfb8, PDF_CMAP_RANGE, 8624 },
	{ 0xcfb9, 0xcfba, PDF_CMAP_TABLE, 363 },
	{ 0xcfbb, 0xcfbc, PDF_CMAP_RANGE, 3880 },
	{ 0xcfbd, 0xcfbe, PDF_CMAP_TABLE, 365 },
	{ 0xcfbf, 0xcfc1, PDF_CMAP_RANGE, 8628 },
	{ 0xcfc2, 0xcfc4, PDF_CMAP_RANGE, 3887 },
	{ 0xcfc5, 0xcfc7, PDF_CMAP_TABLE, 367 },
	{ 0xcfc8, 0xcfc9, PDF_CMAP_RANGE, 3893 },
	{ 0xcfca, 0xcfcb, PDF_CMAP_RANGE, 8633 },
	{ 0xcfcc, 0xcfcc, PDF_CMAP_SINGLE, 3897 },
	{ 0xcfcd, 0xcfce, PDF_CMAP_RANGE, 8635 },
	{ 0xcfcf, 0xcfd0, PDF_CMAP_TABLE, 370 },
	{ 0xcfd1, 0xcfd3, PDF_CMAP_RANGE, 3902 },
	{ 0xcfd4, 0xcfd8, PDF_CMAP_RANGE, 8638 },
	{ 0xcfd9, 0xcfdc, PDF_CMAP_TABLE, 372 },
	{ 0xcfdd, 0xcfde, PDF_CMAP_RANGE, 3914 },
	{ 0xcfdf, 0xcfdf, PDF_CMAP_SINGLE, 8645 },
	{ 0xcfe0, 0xcfe1, PDF_CMAP_RANGE, 3917 },
	{ 0xcfe2, 0xcfe2, PDF_CMAP_SINGLE, 8646 },
	{ 0xcfe3, 0xcfe6, PDF_CMAP_RANGE, 3920 },
	{ 0xcfe7, 0xcfe7, PDF_CMAP_SINGLE, 8647 },
	{ 0xcfe8, 0xcfe9, PDF_CMAP_RANGE, 3925 },
	{ 0xcfea, 0xcfee, PDF_CMAP_TABLE, 376 },
	{ 0xcfef, 0xcff3, PDF_CMAP_RANGE, 3932 },
	{ 0xcff4, 0xcff4, PDF_CMAP_SINGLE, 8651 },
	{ 0xcff5, 0xcff8, PDF_CMAP_RANGE, 3938 },
	{ 0xcff9, 0xcffa, PDF_CMAP_RANGE, 8652 },
	{ 0xcffb, 0xcffd, PDF_CMAP_RANGE, 3944 },
	{ 0xcffe, 0xcffe, PDF_CMAP_SINGLE, 8654 },
	{ 0xd0a1, 0xd0a4, PDF_CMAP_RANGE, 3948 },
	{ 0xd0a5, 0xd0a5, PDF_CMAP_SINGLE, 8655 },
	{ 0xd0a6, 0xd0ac, PDF_CMAP_RANGE, 3953 },
	{ 0xd0ad, 0xd0ae, PDF_CMAP_RANGE, 8656 },
	{ 0xd0af, 0xd0b1, PDF_CMAP_RANGE, 3962 },
	{ 0xd0b2, 0xd0b4, PDF_CMAP_RANGE, 8658 },
	{ 0xd0b5, 0xd0b9, PDF_CMAP_RANGE, 3968 },
	{ 0xd0ba, 0xd0bb, PDF_CMAP_RANGE, 8661 },
	{ 0xd0bc, 0xd0be, PDF_CMAP_RANGE, 3975 },
	{ 0xd0bf, 0xd0bf, PDF_CMAP_SINGLE, 8663 },
	{ 0xd0c0, 0xd0c5, PDF_CMAP_RANGE, 3979 },
	{ 0xd0c6, 0xd0c6, PDF_CMAP_SINGLE, 8664 },
	{ 0xd0c7, 0xd0ca, PDF_CMAP_RANGE, 3986 },
	{ 0xd0cb, 0xd0cb, PDF_CMAP_SINGLE, 8665 },
	{ 0xd0cc, 0xd0e1, PDF_CMAP_RANGE, 3991 },
	{ 0xd0e2, 0xd0e2, PDF_CMAP_SINGLE, 8666 },
	{ 0xd0e3, 0xd0e4, PDF_CMAP_RANGE, 4014 },
	{ 0xd0e5, 0xd0e5, PDF_CMAP_SINGLE, 8667 },
	{ 0xd0e6, 0xd0ea, PDF_CMAP_RANGE, 4017 },
	{ 0xd0eb, 0xd0ed, PDF_CMAP_TABLE, 381 },
	{ 0xd0ee, 0xd0f6, PDF_CMAP_RANGE, 4025 },
	{ 0xd0f7, 0xd0f9, PDF_CMAP_RANGE, 8670 },
	{ 0xd0fa, 0xd0fb, PDF_CMAP_RANGE, 4037 },
	{ 0xd0fc, 0xd0fc, PDF_CMAP_SINGLE, 8673 },
	{ 0xd0fd, 0xd0fe, PDF_CMAP_RANGE, 4040 },
	{ 0xd1a1, 0xd1a2, PDF_CMAP_RANGE, 8674 },
	{ 0xd1a3, 0xd1a4, PDF_CMAP_TABLE, 384 },
	{ 0xd1a5, 0xd1a6, PDF_CMAP_RANGE, 4046 },
	{ 0xd1a7, 0xd1a7, PDF_CMAP_SINGLE, 8677 },
	{ 0xd1a8, 0xd1aa, PDF_CMAP_RANGE, 4049 },
	{ 0xd1ab, 0xd1ab, PDF_CMAP_SINGLE, 8678 },
	{ 0xd1ac, 0xd1ae, PDF_CMAP_RANGE, 4053 },
	{ 0xd1af, 0xd1b1, PDF_CMAP_RANGE, 8679 },
	{ 0xd1b2, 0xd1b4, PDF_CMAP_RANGE, 4059 },
	{ 0xd1b5, 0xd1b7, PDF_CMAP_RANGE, 8682 },
	{ 0xd1b8, 0xd1ba, PDF_CMAP_TABLE, 386 },
	{ 0xd1bb, 0xd1bc, PDF_CMAP_RANGE, 8686 },
	{ 0xd1bd, 0xd1c5, PDF_CMAP_RANGE, 4070 },
	{ 0xd1c6, 0xd1c8, PDF_CMAP_RANGE, 8688 },
	{ 0xd1c9, 0xd1ca, PDF_CMAP_RANGE, 4082 },
	{ 0xd1cb, 0xd1cb, PDF_CMAP_SINGLE, 8691 },
	{ 0xd1cc, 0xd1cd, PDF_CMAP_RANGE, 4085 },
	{ 0xd1ce, 0xd1cf, PDF_CMAP_RANGE, 8692 },
	{ 0xd1d0, 0xd1d4, PDF_CMAP_RANGE, 4089 },
	{ 0xd1d5, 0xd1d6, PDF_CMAP_RANGE, 8694 },
	{ 0xd1d7, 0xd1dd, PDF_CMAP_RANGE, 4096 },
	{ 0xd1de, 0xd1de, PDF_CMAP_SINGLE, 8696 },
	{ 0xd1df, 0xd1e0, PDF_CMAP_RANGE, 4104 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_RANGE, 8697 },
	{ 0xd1e3, 0xd1e7, PDF_CMAP_RANGE, 4108 },
	{ 0xd1e8, 0xd1e9, PDF_CMAP_RANGE, 8699 },
	{ 0xd1ea, 0xd1eb, PDF_CMAP_RANGE, 4115 },
	{ 0xd1ec, 0xd1ed, PDF_CMAP_TABLE, 389 },
	{ 0xd1ee, 0xd1ef, PDF_CMAP_RANGE, 8702 },
	{ 0xd1f0, 0xd1f1, PDF_CMAP_TABLE, 391 },
	{ 0xd1f2, 0xd1f3, PDF_CMAP_RANGE, 4123 },
	{ 0xd1f4, 0xd1f4, PDF_CMAP_SINGLE, 8705 },
	{ 0xd1f5, 0xd1f6, PDF_CMAP_RANGE, 4126 },
	{ 0xd1f7, 0xd1f9, PDF_CMAP_RANGE, 8706 },
	{ 0xd1fa, 0xd1fe, PDF_CMAP_RANGE, 4131 },
	{ 0xd2a1, 0xd2a2, PDF_CMAP_TABLE, 393 },
	{ 0xd2a3, 0xd2a4, PDF_CMAP_RANGE, 4138 },
	{ 0xd2a5, 0xd2a5, PDF_CMAP_SINGLE, 8710 },
	{ 0xd2a6, 0xd2a8, PDF_CMAP_RANGE, 4141 },
	{ 0xd2a9, 0xd2a9, PDF_CMAP_SINGLE, 8711 },
	{ 0xd2aa, 0xd2ae, PDF_CMAP_RANGE, 4145 },
	{ 0xd2af, 0xd2af, PDF_CMAP_SINGLE, 8712 },
	{ 0xd2b0, 0xd2b2, PDF_CMAP_RANGE, 4151 },
	{ 0xd2b3, 0xd2b4, PDF_CMAP_TABLE, 395 },
	{ 0xd2b5, 0xd2b6, PDF_CMAP_RANGE, 8714 },
	{ 0xd2b7, 0xd2bc, PDF_CMAP_RANGE, 4158 },
	{ 0xd2bd, 0xd2bf, PDF_CMAP_TABLE, 397 },
	{ 0xd2c0, 0xd2c2, PDF_CMAP_RANGE, 4167 },
	{ 0xd2c3, 0xd2c7, PDF_CMAP_TABLE, 400 },
	{ 0xd2c8, 0xd2ce, PDF_CMAP_RANGE, 4175 },
	{ 0xd2cf, 0xd2cf, PDF_CMAP_SINGLE, 8721 },
	{ 0xd2d0, 0xd2d4, PDF_CMAP_RANGE, 4183 },
	{ 0xd2d5, 0xd2d5, PDF_CMAP_SINGLE, 8722 },
	{ 0xd2d6, 0xd2d9, PDF_CMAP_RANGE, 4189 },
	{ 0xd2da, 0xd2da, PDF_CMAP_SINGLE, 8723 },
	{ 0xd2db, 0xd2e3, PDF_CMAP_RANGE, 4194 },
	{ 0xd2e4, 0xd2e5, PDF_CMAP_RANGE, 8724 },
	{ 0xd2e6, 0xd2e7, PDF_CMAP_RANGE, 4205 },
	{ 0xd2e8, 0xd2eb, PDF_CMAP_RANGE, 8726 },
	{ 0xd2ec, 0xd2ee, PDF_CMAP_RANGE, 4211 },
	{ 0xd2ef, 0xd2f1, PDF_CMAP_TABLE, 405 },
	{ 0xd2f2, 0xd2f4, PDF_CMAP_RANGE, 4217 },
	{ 0xd2f5, 0xd2f5, PDF_CMAP_SINGLE, 8732 },
	{ 0xd2f6, 0xd2f7, PDF_CMAP_RANGE, 4221 },
	{ 0xd2f8, 0xd2f8, PDF_CMAP_SINGLE, 8733 },
	{ 0xd2f9, 0xd2fa, PDF_CMAP_RANGE, 4224 },
	{ 0xd2fb, 0xd2fb, PDF_CMAP_SINGLE, 8734 },
	{ 0xd2fc, 0xd2fd, PDF_CMAP_RANGE, 4227 },
	{ 0xd2fe, 0xd2fe, PDF_CMAP_SINGLE, 8735 },
	{ 0xd3a1, 0xd3a2, PDF_CMAP_RANGE, 4230 },
	{ 0xd3a3, 0xd3ac, PDF_CMAP_RANGE, 8736 },
	{ 0xd3ad, 0xd3ae, PDF_CMAP_TABLE, 408 },
	{ 0xd3af, 0xd3b0, PDF_CMAP_RANGE, 4244 },
	{ 0xd3b1, 0xd3b1, PDF_CMAP_SINGLE, 8747 },
	{ 0xd3b2, 0xd3b3, PDF_CMAP_RANGE, 4247 },
	{ 0xd3b4, 0xd3b6, PDF_CMAP_RANGE, 8748 },
	{ 0xd3b7, 0xd3b8, PDF_CMAP_TABLE, 410 },
	{ 0xd3b9, 0xd3ba, PDF_CMAP_RANGE, 4254 },
	{ 0xd3bb, 0xd3bb, PDF_CMAP_SINGLE, 8752 },
	{ 0xd3bc, 0xd3c4, PDF_CMAP_RANGE, 4257 },
	{ 0xd3c5, 0xd3c7, PDF_CMAP_TABLE, 412 },
	{ 0xd3c8, 0xd3c9, PDF_CMAP_RANGE, 4269 },
	{ 0xd3ca, 0xd3cc, PDF_CMAP_RANGE, 8755 },
	{ 0xd3cd, 0xd3d4, PDF_CMAP_RANGE, 4274 },
	{ 0xd3d5, 0xd3d5, PDF_CMAP_SINGLE, 8758 },
	{ 0xd3d6, 0xd3de, PDF_CMAP_RANGE, 4283 },
	{ 0xd3df, 0xd3e0, PDF_CMAP_RANGE, 8759 },
	{ 0xd3e1, 0xd3e2, PDF_CMAP_RANGE, 4294 },
	{ 0xd3e3, 0xd3e3, PDF_CMAP_SINGLE, 8761 },
	{ 0xd3e4, 0xd3e5, PDF_CMAP_RANGE, 4297 },
	{ 0xd3e6, 0xd3e6, PDF_CMAP_SINGLE, 8762 },
	{ 0xd3e7, 0xd3ea, PDF_CMAP_RANGE, 4300 },
	{ 0xd3eb, 0xd3ec, PDF_CMAP_RANGE, 8763 },
	{ 0xd3ed, 0xd3ee, PDF_CMAP_RANGE, 4306 },
	{ 0xd3ef, 0xd3ef, PDF_CMAP_SINGLE, 8765 },
	{ 0xd3f0, 0xd3f3, PDF_CMAP_RANGE, 4309 },
	{ 0xd3f4, 0xd3f4, PDF_CMAP_SINGLE, 8766 },
	{ 0xd3f5, 0xd3fb, PDF_CMAP_RANGE, 4314 },
	{ 0xd3fc, 0xd3fe, PDF_CMAP_TABLE, 415 },
	{ 0xd4a1, 0xd4a3, PDF_CMAP_RANGE, 4324 },
	{ 0xd4a4, 0xd4a5, PDF_CMAP_TABLE, 418 },
	{ 0xd4a6, 0xd4a8, PDF_CMAP_RANGE, 8770 },
	{ 0xd4a9, 0xd4ae, PDF_CMAP_RANGE, 4332 },
	{ 0xd4af, 0xd4b2, PDF_CMAP_RANGE, 8773 },
	{ 0xd4b3, 0xd4b4, PDF_CMAP_RANGE, 4342 },
	{ 0xd4b5, 0xd4b6, PDF_CMAP_RANGE, 8777 },
	{ 0xd4b7, 0xd4b8, PDF_CMAP_TABLE, 420 },
	{ 0xd4b9, 0xd4bb, PDF_CMAP_RANGE, 4348 },
	{ 0xd4bc, 0xd4bd, PDF_CMAP_TABLE, 422 },
	{ 0xd4be, 0xd4bf, PDF_CMAP_RANGE, 8781 },
	{ 0xd4c0, 0xd4c3, PDF_CMAP_RANGE, 4355 },
	{ 0xd4c4, 0xd4c5, PDF_CMAP_TABLE, 424 },
	{ 0xd4c6, 0xd4c7, PDF_CMAP_RANGE, 8784 },
	{ 0xd4c8, 0xd4ca, PDF_CMAP_TABLE, 426 },
	{ 0xd4cb, 0xd4ce, PDF_CMAP_RANGE, 8787 },
	{ 0xd4cf, 0xd4d2, PDF_CMAP_RANGE, 4370 },
	{ 0xd4d3, 0xd4d3, PDF_CMAP_SINGLE, 8791 },
	{ 0xd4d4, 0xd4d7, PDF_CMAP_RANGE, 4375 },
	{ 0xd4d8, 0xd4d8, PDF_CMAP_SINGLE, 8792 },
	{ 0xd4d9, 0xd4db, PDF_CMAP_RANGE, 4380 },
	{ 0xd4dc, 0xd4e0, PDF_CMAP_RANGE, 8793 },
	{ 0xd4e1, 0xd4e3, PDF_CMAP_RANGE, 4388 },
	{ 0xd4e4, 0xd4e6, PDF_CMAP_TABLE, 429 },
	{ 0xd4e7, 0xd4ed, PDF_CMAP_RANGE, 4394 },
	{ 0xd4ee, 0xd4ef, PDF_CMAP_TABLE, 432 },
	{ 0xd4f0, 0xd4f4, PDF_CMAP_RANGE, 8801 },
	{ 0xd4f5, 0xd4f8, PDF_CMAP_RANGE, 4408 },
	{ 0xd4f9, 0xd4f9, PDF_CMAP_SINGLE, 8806 },
	{ 0xd4fa, 0xd4fd, PDF_CMAP_RANGE, 4413 },
	{ 0xd4fe, 0xd4fe, PDF_CMAP_SINGLE, 8807 },
	{ 0xd5a1, 0xd5a2, PDF_CMAP_RANGE, 8808 },
	{ 0xd5a3, 0xd5a8, PDF_CMAP_RANGE, 4420 },
	{ 0xd5a9, 0xd5ab, PDF_CMAP_TABLE, 434 },
	{ 0xd5ac, 0xd5ad, PDF_CMAP_RANGE, 4429 },
	{ 0xd5ae, 0xd5ae, PDF_CMAP_SINGLE, 8812 },
	{ 0xd5af, 0xd5b0, PDF_CMAP_RANGE, 4432 },
	{ 0xd5b1, 0xd5b1, PDF_CMAP_SINGLE, 8813 },
	{ 0xd5b2, 0xd5b4, PDF_CMAP_RANGE, 4435 },
	{ 0xd5b5, 0xd5b8, PDF_CMAP_RANGE, 8814 },
	{ 0xd5b9, 0xd5ba, PDF_CMAP_RANGE, 4442 },
	{ 0xd5bb, 0xd5bd, PDF_CMAP_TABLE, 437 },
	{ 0xd5be, 0xd5bf, PDF_CMAP_RANGE, 4447 },
	{ 0xd5c0, 0xd5c0, PDF_CMAP_SINGLE, 8820 },
	{ 0xd5c1, 0xd5c4, PDF_CMAP_RANGE, 4450 },
	{ 0xd5c5, 0xd5c7, PDF_CMAP_TABLE, 440 },
	{ 0xd5c8, 0xd5c9, PDF_CMAP_RANGE, 4457 },
	{ 0xd5ca, 0xd5cb, PDF_CMAP_RANGE, 8823 },
	{ 0xd5cc, 0xd5cd, PDF_CMAP_TABLE, 443 },
	{ 0xd5ce, 0xd5d3, PDF_CMAP_RANGE, 4463 },
	{ 0xd5d4, 0xd5d4, PDF_CMAP_SINGLE, 8826 },
	{ 0xd5d5, 0xd5dc, PDF_CMAP_RANGE, 4470 },
	{ 0xd5dd, 0xd5de, PDF_CMAP_RANGE, 8827 },
	{ 0xd5df, 0xd5e2, PDF_CMAP_TABLE, 445 },
	{ 0xd5e3, 0xd5e9, PDF_CMAP_RANGE, 4484 },
	{ 0xd5ea, 0xd5ec, PDF_CMAP_RANGE, 8831 },
	{ 0xd5ed, 0xd5ee, PDF_CMAP_RANGE, 4494 },
	{ 0xd5ef, 0xd5ef, PDF_CMAP_SINGLE, 8834 },
	{ 0xd5f0, 0xd5f1, PDF_CMAP_RANGE, 4497 },
	{ 0xd5f2, 0xd5f3, PDF_CMAP_RANGE, 8835 },
	{ 0xd5f4, 0xd5f6, PDF_CMAP_RANGE, 4501 },
	{ 0xd5f7, 0xd5f7, PDF_CMAP_SINGLE, 5611 },
	{ 0xd5f8, 0xd5fe, PDF_CMAP_RANGE, 4505 },
	{ 0xd6a1, 0xd6a4, PDF_CMAP_RANGE, 8837 },
	{ 0xd6a5, 0xd6ae, PDF_CMAP_RANGE, 4516 },
	{ 0xd6af, 0xd6b0, PDF_CMAP_RANGE, 8841 },
	{ 0xd6b1, 0xd6b3, PDF_CMAP_RANGE, 4528 },
	{ 0xd6b4, 0xd6b4, PDF_CMAP_SINGLE, 8843 },
	{ 0xd6b5, 0xd6ba, PDF_CMAP_RANGE, 4532 },
	{ 0xd6bb, 0xd6be, PDF_CMAP_TABLE, 449 },
	{ 0xd6bf, 0xd6c0, PDF_CMAP_RANGE, 8846 },
	{ 0xd6c1, 0xd6c3, PDF_CMAP_RANGE, 4544 },
	{ 0xd6c4, 0xd6c4, PDF_CMAP_SINGLE, 8848 },
	{ 0xd6c5, 0xd6c9, PDF_CMAP_RANGE, 4548 },
	{ 0xd6ca, 0xd6ca, PDF_CMAP_SINGLE, 8849 },
	{ 0xd6cb, 0xd6cc, PDF_CMAP_RANGE, 4554 },
	{ 0xd6cd, 0xd6cd, PDF_CMAP_SINGLE, 8850 },
	{ 0xd6ce, 0xd6d2, PDF_CMAP_RANGE, 4557 },
	{ 0xd6d3, 0xd6d4, PDF_CMAP_TABLE, 453 },
	{ 0xd6d5, 0xd6d7, PDF_CMAP_RANGE, 8852 },
	{ 0xd6d8, 0xd6d9, PDF_CMAP_RANGE, 4567 },
	{ 0xd6da, 0xd6da, PDF_CMAP_SINGLE, 8855 },
	{ 0xd6db, 0xd6de, PDF_CMAP_RANGE, 4570 },
	{ 0xd6df, 0xd6e1, PDF_CMAP_TABLE, 455 },
	{ 0xd6e2, 0xd6e4, PDF_CMAP_RANGE, 4577 },
	{ 0xd6e5, 0xd6e6, PDF_CMAP_TABLE, 458 },
	{ 0xd6e7, 0xd6e8, PDF_CMAP_RANGE, 8859 },
	{ 0xd6e9, 0xd6ed, PDF_CMAP_RANGE, 4584 },
	{ 0xd6ee, 0xd6ef, PDF_CMAP_RANGE, 8861 },
	{ 0xd6f0, 0xd6f1, PDF_CMAP_RANGE, 4591 },
	{ 0xd6f2, 0xd6f2, PDF_CMAP_SINGLE, 8863 },
	{ 0xd6f3, 0xd6f4, PDF_CMAP_RANGE, 4594 },
	{ 0xd6f5, 0xd6f6, PDF_CMAP_RANGE, 8864 },
	{ 0xd6f7, 0xd6fb, PDF_CMAP_RANGE, 4598 },
	{ 0xd6fc, 0xd6fe, PDF_CMAP_RANGE, 8866 },
	{ 0xd7a1, 0xd7a3, PDF_CMAP_RANGE, 4606 },
	{ 0xd7a4, 0xd7a4, PDF_CMAP_SINGLE, 8869 },
	{ 0xd7a5, 0xd7a7, PDF_CMAP_RANGE, 4610 },
	{ 0xd7a8, 0xd7aa, PDF_CMAP_RANGE, 8870 },
	{ 0xd7ab, 0xd7ad, PDF_CMAP_TABLE, 460 },
	{ 0xd7ae, 0xd7b1, PDF_CMAP_RANGE, 8874 },
	{ 0xd7b2, 0xd7b2, PDF_CMAP_SINGLE, 4623 },
	{ 0xd7b3, 0xd7b4, PDF_CMAP_RANGE, 8878 },
	{ 0xd7b5, 0xd7b7, PDF_CMAP_TABLE, 463 },
	{ 0xd7b8, 0xd7bc, PDF_CMAP_RANGE, 8881 },
	{ 0xd7bd, 0xd7c6, PDF_CMAP_RANGE, 4634 },
	{ 0xd7c7, 0xd7c7, PDF_CMAP_SINGLE, 8886 },
	{ 0xd7c8, 0xd7c9, PDF_CMAP_RANGE, 4645 },
	{ 0xd7ca, 0xd7ca, PDF_CMAP_SINGLE, 8887 },
	{ 0xd7cb, 0xd7d4, PDF_CMAP_RANGE, 4648 },
	{ 0xd7d5, 0xd7d5, PDF_CMAP_SINGLE, 8888 },
	{ 0xd7d6, 0xd7da, PDF_CMAP_RANGE, 4659 },
	{ 0xd7db, 0xd7de, PDF_CMAP_RANGE, 8889 },
	{ 0xd7df, 0xd7e6, PDF_CMAP_RANGE, 4668 },
	{ 0xd7e7, 0xd7e8, PDF_CMAP_TABLE, 466 },
	{ 0xd7e9, 0xd7ea, PDF_CMAP_RANGE, 8894 },
	{ 0xd7eb, 0xd7f9, PDF_CMAP_RANGE, 4680 },
	{ 0xd8a1, 0xd8c3, PDF_CMAP_RANGE, 4695 },
	{ 0xd8c4, 0xd8c4, PDF_CMAP_SINGLE, 8896 },
	{ 0xd8c5, 0xd8c6, PDF_CMAP_RANGE, 4731 },
	{ 0xd8c7, 0xd8c9, PDF_CMAP_TABLE, 468 },
	{ 0xd8ca, 0xd8cb, PDF_CMAP_RANGE, 4736 },
	{ 0xd8cc, 0xd8cd, PDF_CMAP_RANGE, 8899 },
	{ 0xd8ce, 0xd8cf, PDF_CMAP_RANGE, 4740 },
	{ 0xd8d0, 0xd8d1, PDF_CMAP_RANGE, 8901 },
	{ 0xd8d2, 0xd8d3, PDF_CMAP_TABLE, 471 },
	{ 0xd8d4, 0xd8d8, PDF_CMAP_RANGE, 4746 },
	{ 0xd8d9, 0xd8da, PDF_CMAP_TABLE, 473 },
	{ 0xd8db, 0xd8dc, PDF_CMAP_RANGE, 8905 },
	{ 0xd8dd, 0xd8f0, PDF_CMAP_RANGE, 4755 },
	{ 0xd8f1, 0xd8f1, PDF_CMAP_SINGLE, 8907 },
	{ 0xd8f2, 0xd8f5, PDF_CMAP_RANGE, 4776 },
	{ 0xd8f6, 0xd8f7, PDF_CMAP_RANGE, 8908 },
	{ 0xd8f8, 0xd8fe, PDF_CMAP_RANGE, 4782 },
	{ 0xd9a1, 0xd9ac, PDF_CMAP_RANGE, 4789 },
	{ 0xd9ad, 0xd9b0, PDF_CMAP_TABLE, 475 },
	{ 0xd9b1, 0xd9b3, PDF_CMAP_RANGE, 8912 },
	{ 0xd9b4, 0xd9c6, PDF_CMAP_RANGE, 4808 },
	{ 0xd9c7, 0xd9c7, PDF_CMAP_SINGLE, 8915 },
	{ 0xd9c8, 0xd9cc, PDF_CMAP_RANGE, 4828 },
	{ 0xd9cd, 0xd9d0, PDF_CMAP_RANGE, 8916 },
	{ 0xd9d1, 0xd9dc, PDF_CMAP_RANGE, 4837 },
	{ 0xd9dd, 0xd9dd, PDF_CMAP_SINGLE, 8920 },
	{ 0xd9de, 0xd9e0, PDF_CMAP_RANGE, 4850 },
	{ 0xd9e1, 0xd9e1, PDF_CMAP_SINGLE, 8921 },
	{ 0xd9e2, 0xd9e3, PDF_CMAP_RANGE, 4854 },
	{ 0xd9e4, 0xd9e6, PDF_CMAP_TABLE, 479 },
	{ 0xd9e7, 0xd9eb, PDF_CMAP_RANGE, 4859 },
	{ 0xd9ec, 0xd9ec, PDF_CMAP_SINGLE, 8924 },
	{ 0xd9ed, 0xd9f3, PDF_CMAP_RANGE, 4865 },
	{ 0xd9f4, 0xd9f5, PDF_CMAP_RANGE, 8925 },
	{ 0xd9f6, 0xd9fe, PDF_CMAP_RANGE, 4874 },
	{ 0xdaa1, 0xdaa4, PDF_CMAP_RANGE, 4883 },
	{ 0xdaa5, 0xdadf, PDF_CMAP_RANGE, 8927 },
	{ 0xdae0, 0xdae9, PDF_CMAP_RANGE, 4946 },
	{ 0xdaea, 0xdaea, PDF_CMAP_SINGLE, 8986 },
	{ 0xdaeb, 0xdaf6, PDF_CMAP_RANGE, 4957 },
	{ 0xdaf7, 0xdaf9, PDF_CMAP_TABLE, 482 },
	{ 0xdafa, 0xdafd, PDF_CMAP_RANGE, 4972 },
	{ 0xdafe, 0xdafe, PDF_CMAP_SINGLE, 8989 },
	{ 0xdba1, 0xdba2, PDF_CMAP_RANGE, 4977 },
	{ 0xdba3, 0xdba3, PDF_CMAP_SINGLE, 8990 },
	{ 0xdba4, 0xdba5, PDF_CMAP_RANGE, 4980 },
	{ 0xdba6, 0xdba6, PDF_CMAP_SINGLE, 8991 },
	{ 0xdba7, 0xdba8, PDF_CMAP_RANGE, 4983 },
	{ 0xdba9, 0xdbaa, PDF_CMAP_RANGE, 8992 },
	{ 0xdbab, 0xdbba, PDF_CMAP_RANGE, 4987 },
	{ 0xdbbb, 0xdbbd, PDF_CMAP_TABLE, 485 },
	{ 0xdbbe, 0xdbce, PDF_CMAP_RANGE, 5006 },
	{ 0xdbcf, 0xdbd1, PDF_CMAP_TABLE, 488 },
	{ 0xdbd2, 0xdbda, PDF_CMAP_RANGE, 5026 },
	{ 0xdbdb, 0xdbdb, PDF_CMAP_SINGLE, 8998 },
	{ 0xdbdc, 0xdbdd, PDF_CMAP_RANGE, 5036 },
	{ 0xdbde, 0xdbde, PDF_CMAP_SINGLE, 8999 },
	{ 0xdbdf, 0xdbe1, PDF_CMAP_RANGE, 5039 },
	{ 0xdbe2, 0xdbe4, PDF_CMAP_TABLE, 491 },
	{ 0xdbe5, 0xdbea, PDF_CMAP_RANGE, 5045 },
	{ 0xdbeb, 0xdbeb, PDF_CMAP_SINGLE, 9002 },
	{ 0xdbec, 0xdbed, PDF_CMAP_RANGE, 5052 },
	{ 0xdbee, 0xdbee, PDF_CMAP_SINGLE, 9003 },
	{ 0xdbef, 0xdbf0, PDF_CMAP_RANGE, 5055 },
	{ 0xdbf1, 0xdbf1, PDF_CMAP_SINGLE, 9004 },
	{ 0xdbf2, 0xdbf4, PDF_CMAP_RANGE, 5058 },
	{ 0xdbf5, 0xdbf7, PDF_CMAP_RANGE, 9005 },
	{ 0xdbf8, 0xdbfe, PDF_CMAP_RANGE, 5064 },
	{ 0xdca1, 0xdcbb, PDF_CMAP_RANGE, 5071 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 9008 },
	{ 0xdcbd, 0xdcbe, PDF_CMAP_RANGE, 5099 },
	{ 0xdcbf, 0xdcbf, PDF_CMAP_SINGLE, 9009 },
	{ 0xdcc0, 0xdcc1, PDF_CMAP_RANGE, 5102 },
	{ 0xdcc2, 0xdcc2, PDF_CMAP_SINGLE, 9010 },
	{ 0xdcc3, 0xdcc7, PDF_CMAP_RANGE, 5105 },
	{ 0xdcc8, 0xdcca, PDF_CMAP_RANGE, 9011 },
	{ 0xdccb, 0xdcd0, PDF_CMAP_RANGE, 5113 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 9014 },
	{ 0xdcd2, 0xdcd6, PDF_CMAP_RANGE, 5120 },
	{ 0xdcd7, 0xdcd7, PDF_CMAP_SINGLE, 9015 },
	{ 0xdcd8, 0xdcdf, PDF_CMAP_RANGE, 5126 },
	{ 0xdce0, 0xdce0, PDF_CMAP_SINGLE, 9016 },
	{ 0xdce1, 0xdce2, PDF_CMAP_RANGE, 5135 },
	{ 0xdce3, 0xdce4, PDF_CMAP_RANGE, 9017 },
	{ 0xdce5, 0xdce8, PDF_CMAP_RANGE, 5139 },
	{ 0xdce9, 0xdcea, PDF_CMAP_RANGE, 9019 },
	{ 0xdceb, 0xdcf0, PDF_CMAP_RANGE, 5145 },
	{ 0xdcf1, 0xdcf1, PDF_CMAP_SINGLE, 9021 },
	{ 0xdcf2, 0xdcf5, PDF_CMAP_RANGE, 5152 },
	{ 0xdcf6, 0xdcf6, PDF_CMAP_SINGLE, 9022 },
	{ 0xdcf7, 0xdcf8, PDF_CMAP_RANGE, 5157 },
	{ 0xdcf9, 0xdcf9, PDF_CMAP_SINGLE, 9023 },
	{ 0xdcfa, 0xdcfc, PDF_CMAP_RANGE, 5160 },
	{ 0xdcfd, 0xdcfe, PDF_CMAP_RANGE, 9024 },
	{ 0xdda1, 0xdda2, PDF_CMAP_TABLE, 494 },
	{ 0xdda3, 0xdda7, PDF_CMAP_RANGE, 9027 },
	{ 0xdda8, 0xdda9, PDF_CMAP_RANGE, 5172 },
	{ 0xddaa, 0xddab, PDF_CMAP_RANGE, 9032 },
	{ 0xddac, 0xddb1, PDF_CMAP_RANGE, 5176 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 9034 },
	{ 0xddb3, 0xddb4, PDF_CMAP_RANGE, 5183 },
	{ 0xddb5, 0xddb5, PDF_CMAP_SINGLE, 9035 },
	{ 0xddb6, 0xddb9, PDF_CMAP_RANGE, 5186 },
	{ 0xddba, 0xddbb, PDF_CMAP_RANGE, 9036 },
	{ 0xddbc, 0xddd2, PDF_CMAP_RANGE, 5192 },
	{ 0xddd3, 0xddd3, PDF_CMAP_SINGLE, 9038 },
	{ 0xddd4, 0xddda, PDF_CMAP_RANGE, 5216 },
	{ 0xdddb, 0xdddb, PDF_CMAP_SINGLE, 9039 },
	{ 0xdddc, 0xdddd, PDF_CMAP_RANGE, 5224 },
	{ 0xddde, 0xddde, PDF_CMAP_SINGLE, 9040 },
	{ 0xdddf, 0xdde3, PDF_CMAP_RANGE, 5227 },
	{ 0xdde4, 0xdde4, PDF_CMAP_SINGLE, 9041 },
	{ 0xdde5, 0xddea, PDF_CMAP_RANGE, 5233 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 9042 },
	{ 0xddec, 0xddf0, PDF_CMAP_RANGE, 5240 },
	{ 0xddf1, 0xddf1, PDF_CMAP_SINGLE, 9043 },
	{ 0xddf2, 0xddf5, PDF_CMAP_RANGE, 5246 },
	{ 0xddf6, 0xddf7, PDF_CMAP_RANGE, 9044 },
	{ 0xddf8, 0xddfb, PDF_CMAP_RANGE, 5252 },
	{ 0xddfc, 0xddfe, PDF_CMAP_TABLE, 496 },
	{ 0xdea1, 0xdeac, PDF_CMAP_RANGE, 5259 },
	{ 0xdead, 0xdead, PDF_CMAP_SINGLE, 9048 },
	{ 0xdeae, 0xdeb3, PDF_CMAP_RANGE, 5272 },
	{ 0xdeb4, 0xdeb4, PDF_CMAP_SINGLE, 9049 },
	{ 0xdeb5, 0xdeb9, PDF_CMAP_RANGE, 5279 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 9050 },
	{ 0xdebb, 0xdec5, PDF_CMAP_RANGE, 5285 },
	{ 0xdec6, 0xdec6, PDF_CMAP_SINGLE, 9051 },
	{ 0xdec7, 0xdece, PDF_CMAP_RANGE, 5297 },
	{ 0xdecf, 0xded0, PDF_CMAP_TABLE, 499 },
	{ 0xded1, 0xded2, PDF_CMAP_RANGE, 9053 },
	{ 0xded3, 0xded7, PDF_CMAP_RANGE, 5309 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 9055 },
	{ 0xded9, 0xdee1, PDF_CMAP_RANGE, 5315 },
	{ 0xdee2, 0xdee2, PDF_CMAP_SINGLE, 9056 },
	{ 0xdee3, 0xdee7, PDF_CMAP_RANGE, 5325 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 9057 },
	{ 0xdee9, 0xdeeb, PDF_CMAP_RANGE, 5331 },
	{ 0xdeec, 0xdeec, PDF_CMAP_SINGLE, 9058 },
	{ 0xdeed, 0xdef2, PDF_CMAP_RANGE, 5335 },
	{ 0xdef3, 0xdef3, PDF_CMAP_SINGLE, 9059 },
	{ 0xdef4, 0xdefb, PDF_CMAP_RANGE, 5342 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 9060 },
	{ 0xdefd, 0xdefe, PDF_CMAP_RANGE, 5351 },
	{ 0xdfa1, 0xdfa1, PDF_CMAP_SINGLE, 5353 },
	{ 0xdfa2, 0xdfa3, PDF_CMAP_RANGE, 9061 },
	{ 0xdfa4, 0xdfa5, PDF_CMAP_TABLE, 501 },
	{ 0xdfa6, 0xdfb3, PDF_CMAP_RANGE, 5358 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 9064 },
	{ 0xdfb5, 0xdfbb, PDF_CMAP_RANGE, 5373 },
	{ 0xdfbc, 0xdfbd, PDF_CMAP_RANGE, 9065 },
	{ 0xdfbe, 0xdfbf, PDF_CMAP_TABLE, 503 },
	{ 0xdfc0, 0xdfc1, PDF_CMAP_RANGE, 5384 },
	{ 0xdfc2, 0xdfc3, PDF_CMAP_RANGE, 9068 },
	{ 0xdfc4, 0xdfcb, PDF_CMAP_RANGE, 5388 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 9070 },
	{ 0xdfcd, 0xdfcf, PDF_CMAP_RANGE, 5397 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 9071 },
	{ 0xdfd1, 0xdfd4, PDF_CMAP_RANGE, 5401 },
	{ 0xdfd5, 0xdfd5, PDF_CMAP_SINGLE, 9072 },
	{ 0xdfd6, 0xdfd7, PDF_CMAP_RANGE, 5406 },
	{ 0xdfd8, 0xdfd9, PDF_CMAP_RANGE, 9073 },
	{ 0xdfda, 0xdfdb, PDF_CMAP_RANGE, 5410 },
	{ 0xdfdc, 0xdfdc, PDF_CMAP_SINGLE, 9075 },
	{ 0xdfdd, 0xdfdf, PDF_CMAP_RANGE, 5413 },
	{ 0xdfe0, 0xdfe2, PDF_CMAP_TABLE, 505 },
	{ 0xdfe3, 0xdfe5, PDF_CMAP_RANGE, 5419 },
	{ 0xdfe6, 0xdfe6, PDF_CMAP_SINGLE, 9078 },
	{ 0xdfe7, 0xdfe8, PDF_CMAP_RANGE, 5423 },
	{ 0xdfe9, 0xdfeb, PDF_CMAP_TABLE, 508 },
	{ 0xdfec, 0xdfee, PDF_CMAP_RANGE, 5428 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 9081 },
	{ 0xdff0, 0xdff4, PDF_CMAP_RANGE, 5432 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 9082 },
	{ 0xdff6, 0xdff8, PDF_CMAP_RANGE, 5438 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 9083 },
	{ 0xdffa, 0xdffe, PDF_CMAP_RANGE, 5442 },
	{ 0xe0a1, 0xe0b5, PDF_CMAP_RANGE, 5447 },
	{ 0xe0b6, 0xe0b7, PDF_CMAP_RANGE, 9084 },
	{ 0xe0b8, 0xe0be, PDF_CMAP_RANGE, 5470 },
	{ 0xe0bf, 0xe0bf, PDF_CMAP_SINGLE, 9086 },
	{ 0xe0c0, 0xe0c7, PDF_CMAP_RANGE, 5478 },
	{ 0xe0c8, 0xe0c8, PDF_CMAP_SINGLE, 9087 },
	{ 0xe0c9, 0xe0cd, PDF_CMAP_RANGE, 5487 },
	{ 0xe0ce, 0xe0ce, PDF_CMAP_SINGLE, 9088 },
	{ 0xe0cf, 0xe0d2, PDF_CMAP_RANGE, 5493 },
	{ 0xe0d3, 0xe0d3, PDF_CMAP_SINGLE, 9089 },
	{ 0xe0d4, 0xe0df, PDF_CMAP_RANGE, 5498 },
	{ 0xe0e0, 0xe0e0, PDF_CMAP_SINGLE, 9090 },
	{ 0xe0e1, 0xe0ef, PDF_CMAP_RANGE, 5511 },
	{ 0xe0f0, 0xe0f0, PDF_CMAP_SINGLE, 9091 },
	{ 0xe0f1, 0xe0f7, PDF_CMAP_RANGE, 5527 },
	{ 0xe0f8, 0xe0f8, PDF_CMAP_SINGLE, 9092 },
	{ 0xe0f9, 0xe0fb, PDF_CMAP_RANGE, 5535 },
	{ 0xe0fc, 0xe0fe, PDF_CMAP_RANGE, 9093 },
	{ 0xe1a1, 0xe1aa, PDF_CMAP_RANGE, 5541 },
	{ 0xe1ab, 0xe1ad, PDF_CMAP_TABLE, 511 },
	{ 0xe1ae, 0xe1af, PDF_CMAP_RANGE, 5554 },
	{ 0xe1b0, 0xe1b0, PDF_CMAP_SINGLE, 9098 },
	{ 0xe1b1, 0xe1b3, PDF_CMAP_RANGE, 5557 },
	{ 0xe1b4, 0xe1b4, PDF_CMAP_SINGLE, 9099 },
	{ 0xe1b5, 0xe1ba, PDF_CMAP_RANGE, 5561 },
	{ 0xe1bb, 0xe1bd, PDF_CMAP_TABLE, 514 },
	{ 0xe1be, 0xe1bf, PDF_CMAP_RANGE, 5570 },
	{ 0xe1c0, 0xe1c1, PDF_CMAP_RANGE, 9102 },
	{ 0xe1c2, 0xe1c8, PDF_CMAP_RANGE, 5574 },
	{ 0xe1c9, 0xe1c9, PDF_CMAP_SINGLE, 9104 },
	{ 0xe1ca, 0xe1cf, PDF_CMAP_RANGE, 5582 },
	{ 0xe1d0, 0xe1d0, PDF_CMAP_SINGLE, 9105 },
	{ 0xe1d1, 0xe1da, PDF_CMAP_RANGE, 5589 },
	{ 0xe1db, 0xe1db, PDF_CMAP_SINGLE, 9106 },
	{ 0xe1dc, 0xe1e0, PDF_CMAP_RANGE, 5600 },
	{ 0xe1e1, 0xe1e2, PDF_CMAP_TABLE, 517 },
	{ 0xe1e3, 0xe1e6, PDF_CMAP_RANGE, 5607 },
	{ 0xe1e7, 0xe1e7, PDF_CMAP_SINGLE, 4504 },
	{ 0xe1e8, 0xe1ed, PDF_CMAP_RANGE, 5612 },
	{ 0xe1ee, 0xe1ef, PDF_CMAP_RANGE, 9108 },
	{ 0xe1f0, 0xe1f5, PDF_CMAP_RANGE, 5620 },
	{ 0xe1f6, 0xe1f8, PDF_CMAP_TABLE, 519 },
	{ 0xe1f9, 0xe1fc, PDF_CMAP_RANGE, 5629 },
	{ 0xe1fd, 0xe1fe, PDF_CMAP_TABLE, 522 },
	{ 0xe2a1, 0xe2a3, PDF_CMAP_RANGE, 5635 },
	{ 0xe2a4, 0xe2a4, PDF_CMAP_SINGLE, 9113 },
	{ 0xe2a5, 0xe2a7, PDF_CMAP_RANGE, 5639 },
	{ 0xe2a8, 0xe2a8, PDF_CMAP_SINGLE, 9114 },
	{ 0xe2a9, 0xe2ba, PDF_CMAP_RANGE, 5643 },
	{ 0xe2bb, 0xe2c4, PDF_CMAP_RANGE, 9115 },
	{ 0xe2c5, 0xe2c5, PDF_CMAP_SINGLE, 4293 },
	{ 0xe2c6, 0xe2ce, PDF_CMAP_RANGE, 9125 },
	{ 0xe2cf, 0xe2d0, PDF_CMAP_TABLE, 524 },
	{ 0xe2d1, 0xe2d8, PDF_CMAP_RANGE, 5683 },
	{ 0xe2d9, 0xe2d9, PDF_CMAP_SINGLE, 9135 },
	{ 0xe2da, 0xe2e2, PDF_CMAP_RANGE, 5692 },
	{ 0xe2e3, 0xe2e4, PDF_CMAP_RANGE, 9136 },
	{ 0xe2e5, 0xe2e6, PDF_CMAP_TABLE, 526 },
	{ 0xe2e7, 0xe2e8, PDF_CMAP_RANGE, 5705 },
	{ 0xe2e9, 0xe2eb, PDF_CMAP_RANGE, 9139 },
	{ 0xe2ec, 0xe2f7, PDF_CMAP_RANGE, 5710 },
	{ 0xe2f8, 0xe2f9, PDF_CMAP_TABLE, 528 },
	{ 0xe2fa, 0xe2fd, PDF_CMAP_RANGE, 9143 },
	{ 0xe2fe, 0xe2fe, PDF_CMAP_SINGLE, 5728 },
	{ 0xe3a1, 0xe3a2, PDF_CMAP_TABLE, 530 },
	{ 0xe3a3, 0xe3a4, PDF_CMAP_RANGE, 5731 },
	{ 0xe3a5, 0xe3a5, PDF_CMAP_SINGLE, 9148 },
	{ 0xe3a6, 0xe3aa, PDF_CMAP_RANGE, 5734 },
	{ 0xe3ab, 0xe3ab, PDF_CMAP_SINGLE, 9149 },
	{ 0xe3ac, 0xe3b3, PDF_CMAP_RANGE, 5740 },
	{ 0xe3b4, 0xe3b4, PDF_CMAP_SINGLE, 9150 },
	{ 0xe3b5, 0xe3c4, PDF_CMAP_RANGE, 5749 },
	{ 0xe3c5, 0xe3db, PDF_CMAP_RANGE, 9151 },
	{ 0xe3dc, 0xe3e2, PDF_CMAP_RANGE, 5788 },
	{ 0xe3e3, 0xe3e3, PDF_CMAP_SINGLE, 9174 },
	{ 0xe3e4, 0xe3ec, PDF_CMAP_RANGE, 5796 },
	{ 0xe3ed, 0xe3ed, PDF_CMAP_SINGLE, 9175 },
	{ 0xe3ee, 0xe3f0, PDF_CMAP_RANGE, 5806 },
	{ 0xe3f1, 0xe3f2, PDF_CMAP_RANGE, 9176 },
	{ 0xe3f3, 0xe3f7, PDF_CMAP_RANGE, 5811 },
	{ 0xe3f8, 0xe3f8, PDF_CMAP_SINGLE, 9178 },
	{ 0xe3f9, 0xe3fd, PDF_CMAP_RANGE, 5817 },
	{ 0xe3fe, 0xe3fe, PDF_CMAP_SINGLE, 9179 },
	{ 0xe4a1, 0xe4a3, PDF_CMAP_RANGE, 5823 },
	{ 0xe4a4, 0xe4a5, PDF_CMAP_RANGE, 9180 },
	{ 0xe4a6, 0xe4aa, PDF_CMAP_RANGE, 5828 },
	{ 0xe4ab, 0xe4ab, PDF_CMAP_SINGLE, 9182 },
	{ 0xe4ac, 0xe4ae, PDF_CMAP_RANGE, 5834 },
	{ 0xe4af, 0xe4b1, PDF_CMAP_RANGE, 9183 },
	{ 0xe4b2, 0xe4b4, PDF_CMAP_RANGE, 5840 },
	{ 0xe4b5, 0xe4b6, PDF_CMAP_RANGE, 9186 },
	{ 0xe4b7, 0xe4c1, PDF_CMAP_RANGE, 5845 },
	{ 0xe4c2, 0xe4c2, PDF_CMAP_SINGLE, 9188 },
	{ 0xe4c3, 0xe4c4, PDF_CMAP_RANGE, 5857 },
	{ 0xe4c5, 0xe4c5, PDF_CMAP_SINGLE, 9189 },
	{ 0xe4c6, 0xe4c8, PDF_CMAP_RANGE, 5860 },
	{ 0xe4c9, 0xe4c9, PDF_CMAP_SINGLE, 9190 },
	{ 0xe4ca, 0xe4d8, PDF_CMAP_RANGE, 5864 },
	{ 0xe4d9, 0xe4d9, PDF_CMAP_SINGLE, 9191 },
	{ 0xe4da, 0xe4db, PDF_CMAP_RANGE, 5880 },
	{ 0xe4dc, 0xe4de, PDF_CMAP_TABLE, 532 },
	{ 0xe4df, 0xe4e3, PDF_CMAP_RANGE, 5885 },
	{ 0xe4e4, 0xe4e4, PDF_CMAP_SINGLE, 9194 },
	{ 0xe4e5, 0xe4ea, PDF_CMAP_RANGE, 5891 },
	{ 0xe4eb, 0xe4ec, PDF_CMAP_RANGE, 9195 },
	{ 0xe4ed, 0xe4f1, PDF_CMAP_RANGE, 5899 },
	{ 0xe4f2, 0xe4f2, PDF_CMAP_SINGLE, 9197 },
	{ 0xe4f3, 0xe4fd, PDF_CMAP_RANGE, 5905 },
	{ 0xe4fe, 0xe4fe, PDF_CMAP_SINGLE, 9198 },
	{ 0xe5a1, 0xe5af, PDF_CMAP_RANGE, 5917 },
	{ 0xe5b0, 0xe5b0, PDF_CMAP_SINGLE, 9199 },
	{ 0xe5b1, 0xe5b8, PDF_CMAP_RANGE, 5933 },
	{ 0xe5b9, 0xe5b9, PDF_CMAP_SINGLE, 9200 },
	{ 0xe5ba, 0xe5c6, PDF_CMAP_RANGE, 5942 },
	{ 0xe5c7, 0xe5c9, PDF_CMAP_TABLE, 535 },
	{ 0xe5ca, 0xe5cd, PDF_CMAP_RANGE, 5958 },
	{ 0xe5ce, 0xe5ce, PDF_CMAP_SINGLE, 9203 },
	{ 0xe5cf, 0xe5ef, PDF_CMAP_RANGE, 5963 },
	{ 0xe5f0, 0xe5f2, PDF_CMAP_TABLE, 538 },
	{ 0xe5f3, 0xe5fb, PDF_CMAP_RANGE, 5999 },
	{ 0xe5fc, 0xe5fd, PDF_CMAP_RANGE, 9206 },
	{ 0xe5fe, 0xe5fe, PDF_CMAP_SINGLE, 6010 },
	{ 0xe6a1, 0xe6a2, PDF_CMAP_RANGE, 6011 },
	{ 0xe6a3, 0xe6a3, PDF_CMAP_SINGLE, 9208 },
	{ 0xe6a4, 0xe6aa, PDF_CMAP_RANGE, 6014 },
	{ 0xe6ab, 0xe6ac, PDF_CMAP_RANGE, 9209 },
	{ 0xe6ad, 0xe6ae, PDF_CMAP_TABLE, 541 },
	{ 0xe6af, 0xe6b3, PDF_CMAP_RANGE, 6025 },
	{ 0xe6b4, 0xe6b5, PDF_CMAP_RANGE, 9212 },
	{ 0xe6b6, 0xe6be, PDF_CMAP_RANGE, 6032 },
	{ 0xe6bf, 0xe6bf, PDF_CMAP_SINGLE, 9214 },
	{ 0xe6c0, 0xe6c7, PDF_CMAP_RANGE, 6042 },
	{ 0xe6c8, 0xe6c9, PDF_CMAP_RANGE, 9215 },
	{ 0xe6ca, 0xe6cc, PDF_CMAP_RANGE, 6052 },
	{ 0xe6cd, 0xe6cd, PDF_CMAP_SINGLE, 9217 },
	{ 0xe6ce, 0xe6df, PDF_CMAP_RANGE, 6056 },
	{ 0xe6e0, 0xe6fe, PDF_CMAP_RANGE, 9218 },
	{ 0xe7a1, 0xe7da, PDF_CMAP_RANGE, 9249 },
	{ 0xe7db, 0xe7e0, PDF_CMAP_RANGE, 6163 },
	{ 0xe7e1, 0xe7e2, PDF_CMAP_RANGE, 9307 },
	{ 0xe7e3, 0xe7e6, PDF_CMAP_RANGE, 6171 },
	{ 0xe7e7, 0xe7e7, PDF_CMAP_SINGLE, 9309 },
	{ 0xe7e8, 0xe7ee, PDF_CMAP_RANGE, 6176 },
	{ 0xe7ef, 0xe7ef, PDF_CMAP_SINGLE, 9310 },
	{ 0xe7f0, 0xe7f3, PDF_CMAP_RANGE, 6184 },
	{ 0xe7f4, 0xe7f6, PDF_CMAP_RANGE, 9311 },
	{ 0xe7f7, 0xe7fe, PDF_CMAP_RANGE, 6191 },
	{ 0xe8a1, 0xe8a7, PDF_CMAP_RANGE, 6199 },
	{ 0xe8a8, 0xe8a8, PDF_CMAP_SINGLE, 9314 },
	{ 0xe8a9, 0xe8ab, PDF_CMAP_RANGE, 6207 },
	{ 0xe8ac, 0xe8ac, PDF_CMAP_SINGLE, 9315 },
	{ 0xe8ad, 0xe8b5, PDF_CMAP_RANGE, 6211 },
	{ 0xe8b6, 0xe8b7, PDF_CMAP_TABLE, 543 },
	{ 0xe8b8, 0xe8ba, PDF_CMAP_RANGE, 9317 },
	{ 0xe8bb, 0xe8be, PDF_CMAP_RANGE, 6225 },
	{ 0xe8bf, 0xe8c0, PDF_CMAP_RANGE, 9320 },
	{ 0xe8c1, 0xe8c4, PDF_CMAP_RANGE, 6231 },
	{ 0xe8c5, 0xe8c6, PDF_CMAP_TABLE, 545 },
	{ 0xe8c7, 0xe8c9, PDF_CMAP_RANGE, 9323 },
	{ 0xe8ca, 0xe8cd, PDF_CMAP_RANGE, 6240 },
	{ 0xe8ce, 0xe8d0, PDF_CMAP_TABLE, 547 },
	{ 0xe8d1, 0xe8d2, PDF_CMAP_RANGE, 6247 },
	{ 0xe8d3, 0xe8d3, PDF_CMAP_SINGLE, 9328 },
	{ 0xe8d4, 0xe8dc, PDF_CMAP_RANGE, 6250 },
	{ 0xe8dd, 0xe8df, PDF_CMAP_TABLE, 550 },
	{ 0xe8e0, 0xe8e1, PDF_CMAP_RANGE, 6262 },
	{ 0xe8e2, 0xe8e3, PDF_CMAP_RANGE, 9331 },
	{ 0xe8e4, 0xe8e7, PDF_CMAP_TABLE, 553 },
	{ 0xe8e8, 0xe8ea, PDF_CMAP_RANGE, 6270 },
	{ 0xe8eb, 0xe8ef, PDF_CMAP_TABLE, 557 },
	{ 0xe8f0, 0xe8f8, PDF_CMAP_RANGE, 6278 },
	{ 0xe8f9, 0xe8f9, PDF_CMAP_SINGLE, 9338 },
	{ 0xe8fa, 0xe8fb, PDF_CMAP_RANGE, 6288 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 9339 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 6292 },
	{ 0xe9a1, 0xe9a1, PDF_CMAP_SINGLE, 9341 },
	{ 0xe9a2, 0xe9ac, PDF_CMAP_RANGE, 6294 },
	{ 0xe9ad, 0xe9ad, PDF_CMAP_SINGLE, 9342 },
	{ 0xe9ae, 0xe9b3, PDF_CMAP_RANGE, 6306 },
	{ 0xe9b4, 0xe9b5, PDF_CMAP_RANGE, 9343 },
	{ 0xe9b6, 0xe9b7, PDF_CMAP_TABLE, 562 },
	{ 0xe9b8, 0xe9c3, PDF_CMAP_RANGE, 6316 },
	{ 0xe9c4, 0xe9c6, PDF_CMAP_TABLE, 564 },
	{ 0xe9c7, 0xe9c8, PDF_CMAP_RANGE, 6331 },
	{ 0xe9c9, 0xe9c9, PDF_CMAP_SINGLE, 9348 },
	{ 0xe9ca, 0xe9d5, PDF_CMAP_RANGE, 6334 },
	{ 0xe9d6, 0xe9d6, PDF_CMAP_SINGLE, 9349 },
	{ 0xe9d7, 0xe9d9, PDF_CMAP_RANGE, 6347 },
	{ 0xe9da, 0xe9da, PDF_CMAP_SINGLE, 9350 },
	{ 0xe9db, 0xe9e3, PDF_CMAP_RANGE, 6351 },
	{ 0xe9e4, 0xe9e5, PDF_CMAP_TABLE, 567 },
	{ 0xe9e6, 0xe9e7, PDF_CMAP_RANGE, 9352 },
	{ 0xe9e8, 0xe9ec, PDF_CMAP_TABLE, 569 },
	{ 0xe9ed, 0xe9fe, PDF_CMAP_RANGE, 9356 },
	{ 0xeaa1, 0xeaa5, PDF_CMAP_RANGE, 9374 },
	{ 0xeaa6, 0xeaa6, PDF_CMAP_SINGLE, 6392 },
	{ 0xeaa7, 0xeaa8, PDF_CMAP_RANGE, 9379 },
	{ 0xeaa9, 0xeab0, PDF_CMAP_RANGE, 6395 },
	{ 0xeab1, 0xeab1, PDF_CMAP_SINGLE, 9381 },
	{ 0xeab2, 0xeabb, PDF_CMAP_RANGE, 6404 },
	{ 0xeabc, 0xeabc, PDF_CMAP_SINGLE, 9382 },
	{ 0xeabd, 0xeac9, PDF_CMAP_RANGE, 6415 },
	{ 0xeaca, 0xeaca, PDF_CMAP_SINGLE, 9383 },
	{ 0xeacb, 0xeacc, PDF_CMAP_RANGE, 6429 },
	{ 0xeacd, 0xeacd, PDF_CMAP_SINGLE, 9384 },
	{ 0xeace, 0xead2, PDF_CMAP_RANGE, 6432 },
	{ 0xead3, 0xead3, PDF_CMAP_SINGLE, 9385 },
	{ 0xead4, 0xead9, PDF_CMAP_RANGE, 6438 },
	{ 0xeada, 0xeaef, PDF_CMAP_RANGE, 9386 },
	{ 0xeaf0, 0xeafe, PDF_CMAP_RANGE, 6466 },
	{ 0xeba1, 0xeba6, PDF_CMAP_RANGE, 6481 },
	{ 0xeba7, 0xeba7, PDF_CMAP_SINGLE, 9408 },
	{ 0xeba8, 0xeba9, PDF_CMAP_RANGE, 6488 },
	{ 0xebaa, 0xebaa, PDF_CMAP_SINGLE, 9409 },
	{ 0xebab, 0xebb1, PDF_CMAP_RANGE, 6491 },
	{ 0xebb2, 0xebb2, PDF_CMAP_SINGLE, 9410 },
	{ 0xebb3, 0xebb8, PDF_CMAP_RANGE, 6499 },
	{ 0xebb9, 0xebb9, PDF_CMAP_SINGLE, 9411 },
	{ 0xebba, 0xebc9, PDF_CMAP_RANGE, 6506 },
	{ 0xebca, 0xebcb, PDF_CMAP_RANGE, 9412 },
	{ 0xebcc, 0xebcd, PDF_CMAP_TABLE, 574 },
	{ 0xebce, 0xebd5, PDF_CMAP_RANGE, 6526 },
	{ 0xebd6, 0xebd6, PDF_CMAP_SINGLE, 9415 },
	{ 0xebd7, 0xebd9, PDF_CMAP_RANGE, 6535 },
	{ 0xebda, 0xebda, PDF_CMAP_SINGLE, 9416 },
	{ 0xebdb, 0xebe0, PDF_CMAP_RANGE, 6539 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 9417 },
	{ 0xebe2, 0xebf6, PDF_CMAP_RANGE, 6546 },
	{ 0xebf7, 0xebf7, PDF_CMAP_SINGLE, 9418 },
	{ 0xebf8, 0xebfe, PDF_CMAP_RANGE, 6568 },
	{ 0xeca1, 0xeca2, PDF_CMAP_RANGE, 6575 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 9419 },
	{ 0xeca4, 0xeca8, PDF_CMAP_RANGE, 6578 },
	{ 0xeca9, 0xecae, PDF_CMAP_RANGE, 9420 },
	{ 0xecaf, 0xecb0, PDF_CMAP_RANGE, 6589 },
	{ 0xecb1, 0xecb1, PDF_CMAP_SINGLE, 9426 },
	{ 0xecb2, 0xecb3, PDF_CMAP_RANGE, 6592 },
	{ 0xecb4, 0xecb5, PDF_CMAP_RANGE, 9427 },
	{ 0xecb6, 0xecbd, PDF_CMAP_RANGE, 6596 },
	{ 0xecbe, 0xecbf, PDF_CMAP_RANGE, 9429 },
	{ 0xecc0, 0xecc1, PDF_CMAP_TABLE, 576 },
	{ 0xecc2, 0xecc6, PDF_CMAP_RANGE, 6608 },
	{ 0xecc7, 0xecc7, PDF_CMAP_SINGLE, 9432 },
	{ 0xecc8, 0xecca, PDF_CMAP_RANGE, 6614 },
	{ 0xeccb, 0xeccb, PDF_CMAP_SINGLE, 9433 },
	{ 0xeccc, 0xece1, PDF_CMAP_RANGE, 6618 },
	{ 0xece2, 0xece2, PDF_CMAP_SINGLE, 9434 },
	{ 0xece3, 0xecf1, PDF_CMAP_RANGE, 6641 },
	{ 0xecf2, 0xecf2, PDF_CMAP_SINGLE, 9435 },
	{ 0xecf3, 0xecf4, PDF_CMAP_RANGE, 6657 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 9436 },
	{ 0xecf6, 0xecf7, PDF_CMAP_RANGE, 6660 },
	{ 0xecf8, 0xecf8, PDF_CMAP_SINGLE, 9437 },
	{ 0xecf9, 0xecfe, PDF_CMAP_RANGE, 6663 },
	{ 0xeda1, 0xeda1, PDF_CMAP_SINGLE, 9438 },
	{ 0xeda2, 0xeda7, PDF_CMAP_RANGE, 6670 },
	{ 0xeda8, 0xeda8, PDF_CMAP_SINGLE, 9439 },
	{ 0xeda9, 0xedae, PDF_CMAP_RANGE, 6677 },
	{ 0xedaf, 0xedb0, PDF_CMAP_RANGE, 9440 },
	{ 0xedb1, 0xedb3, PDF_CMAP_RANGE, 6685 },
	{ 0xedb4, 0xedba, PDF_CMAP_TABLE, 578 },
	{ 0xedbb, 0xedbe, PDF_CMAP_RANGE, 6695 },
	{ 0xedbf, 0xedbf, PDF_CMAP_SINGLE, 9446 },
	{ 0xedc0, 0xedc1, PDF_CMAP_RANGE, 6700 },
	{ 0xedc2, 0xedc3, PDF_CMAP_RANGE, 9447 },
	{ 0xedc4, 0xedcb, PDF_CMAP_RANGE, 6704 },
	{ 0xedcc, 0xedcd, PDF_CMAP_RANGE, 9449 },
	{ 0xedce, 0xedd2, PDF_CMAP_RANGE, 6714 },
	{ 0xedd3, 0xedd3, PDF_CMAP_SINGLE, 9451 },
	{ 0xedd4, 0xedd6, PDF_CMAP_RANGE, 6720 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 9452 },
	{ 0xedd8, 0xede7, PDF_CMAP_RANGE, 6724 },
	{ 0xede8, 0xede8, PDF_CMAP_SINGLE, 9453 },
	{ 0xede9, 0xeded, PDF_CMAP_RANGE, 6741 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 9454 },
	{ 0xedef, 0xedf8, PDF_CMAP_RANGE, 6747 },
	{ 0xedf9, 0xedfa, PDF_CMAP_RANGE, 9455 },
	{ 0xedfb, 0xedfe, PDF_CMAP_RANGE, 6759 },
	{ 0xeea1, 0xeebb, PDF_CMAP_RANGE, 6763 },
	{ 0xeebc, 0xeebc, PDF_CMAP_SINGLE, 9457 },
	{ 0xeebd, 0xeebe, PDF_CMAP_RANGE, 6791 },
	{ 0xeebf, 0xeebf, PDF_CMAP_SINGLE, 9458 },
	{ 0xeec0, 0xeec3, PDF_CMAP_RANGE, 6794 },
	{ 0xeec4, 0xeefe, PDF_CMAP_RANGE, 9459 },
	{ 0xefa1, 0xeff1, PDF_CMAP_RANGE, 9518 },
	{ 0xeff2, 0xeffe, PDF_CMAP_RANGE, 6938 },
	{ 0xf0a1, 0xf0a2, PDF_CMAP_RANGE, 6951 },
	{ 0xf0a3, 0xf0a3, PDF_CMAP_SINGLE, 9599 },
	{ 0xf0a4, 0xf0ae, PDF_CMAP_RANGE, 6954 },
	{ 0xf0af, 0xf0d9, PDF_CMAP_RANGE, 9600 },
	{ 0xf0da, 0xf0db, PDF_CMAP_RANGE, 7008 },
	{ 0xf0dc, 0xf0dd, PDF_CMAP_RANGE, 9643 },
	{ 0xf0de, 0xf0df, PDF_CMAP_TABLE, 585 },
	{ 0xf0e0, 0xf0e8, PDF_CMAP_RANGE, 7014 },
	{ 0xf0e9, 0xf0e9, PDF_CMAP_SINGLE, 9646 },
	{ 0xf0ea, 0xf0eb, PDF_CMAP_RANGE, 7024 },
	{ 0xf0ec, 0xf0ec, PDF_CMAP_SINGLE, 9647 },
	{ 0xf0ed, 0xf0ee, PDF_CMAP_RANGE, 7027 },
	{ 0xf0ef, 0xf0ef, PDF_CMAP_SINGLE, 9648 },
	{ 0xf0f0, 0xf0f6, PDF_CMAP_RANGE, 7030 },
	{ 0xf0f7, 0xf0f9, PDF_CMAP_TABLE, 587 },
	{ 0xf0fa, 0xf0fb, PDF_CMAP_RANGE, 7040 },
	{ 0xf0fc, 0xf0fc, PDF_CMAP_SINGLE, 9651 },
	{ 0xf0fd, 0xf0fe, PDF_CMAP_RANGE, 7043 },
	{ 0xf1a1, 0xf1a7, PDF_CMAP_RANGE, 7045 },
	{ 0xf1a8, 0xf1a8, PDF_CMAP_SINGLE, 9652 },
	{ 0xf1a9, 0xf1aa, PDF_CMAP_RANGE, 7053 },
	{ 0xf1ab, 0xf1ab, PDF_CMAP_SINGLE, 9653 },
	{ 0xf1ac, 0xf1ad, PDF_CMAP_RANGE, 7056 },
	{ 0xf1ae, 0xf1ae, PDF_CMAP_SINGLE, 9654 },
	{ 0xf1af, 0xf1b1, PDF_CMAP_RANGE, 7059 },
	{ 0xf1b2, 0xf1b2, PDF_CMAP_SINGLE, 9655 },
	{ 0xf1b3, 0xf1bb, PDF_CMAP_RANGE, 7063 },
	{ 0xf1bc, 0xf1bc, PDF_CMAP_SINGLE, 9656 },
	{ 0xf1bd, 0xf1bf, PDF_CMAP_RANGE, 7073 },
	{ 0xf1c0, 0xf1c0, PDF_CMAP_SINGLE, 9657 },
	{ 0xf1c1, 0xf1c8, PDF_CMAP_RANGE, 7077 },
	{ 0xf1c9, 0xf1c9, PDF_CMAP_SINGLE, 9658 },
	{ 0xf1ca, 0xf1cc, PDF_CMAP_RANGE, 7086 },
	{ 0xf1cd, 0xf1ce, PDF_CMAP_TABLE, 590 },
	{ 0xf1cf, 0xf1d0, PDF_CMAP_RANGE, 9660 },
	{ 0xf1d1, 0xf1d9, PDF_CMAP_RANGE, 7093 },
	{ 0xf1da, 0xf1dc, PDF_CMAP_TABLE, 592 },
	{ 0xf1dd, 0xf1e3, PDF_CMAP_RANGE, 7105 },
	{ 0xf1e4, 0xf1e4, PDF_CMAP_SINGLE, 9664 },
	{ 0xf1e5, 0xf1eb, PDF_CMAP_RANGE, 7113 },
	{ 0xf1ec, 0xf1ec, PDF_CMAP_SINGLE, 9665 },
	{ 0xf1ed, 0xf1ee, PDF_CMAP_RANGE, 7121 },
	{ 0xf1ef, 0xf1ef, PDF_CMAP_SINGLE, 9666 },
	{ 0xf1f0, 0xf1f6, PDF_CMAP_RANGE, 7124 },
	{ 0xf1f7, 0xf1f9, PDF_CMAP_TABLE, 595 },
	{ 0xf1fa, 0xf1fb, PDF_CMAP_RANGE, 7134 },
	{ 0xf1fc, 0xf1fe, PDF_CMAP_RANGE, 9669 },
	{ 0xf2a1, 0xf2ad, PDF_CMAP_RANGE, 9672 },
	{ 0xf2ae, 0xf2b0, PDF_CMAP_RANGE, 7152 },
	{ 0xf2b1, 0xf2b2, PDF_CMAP_RANGE, 9685 },
	{ 0xf2b3, 0xf2b8, PDF_CMAP_RANGE, 7157 },
	{ 0xf2b9, 0xf2b9, PDF_CMAP_SINGLE, 9687 },
	{ 0xf2ba, 0xf2c2, PDF_CMAP_RANGE, 7164 },
	{ 0xf2c3, 0xf2c3, PDF_CMAP_SINGLE, 9688 },
	{ 0xf2c4, 0xf2c8, PDF_CMAP_RANGE, 7174 },
	{ 0xf2c9, 0xf2c9, PDF_CMAP_SINGLE, 9689 },
	{ 0xf2ca, 0xf2cb, PDF_CMAP_RANGE, 7180 },
	{ 0xf2cc, 0xf2cd, PDF_CMAP_RANGE, 9690 },
	{ 0xf2ce, 0xf2cf, PDF_CMAP_TABLE, 598 },
	{ 0xf2d0, 0xf2d2, PDF_CMAP_RANGE, 7186 },
	{ 0xf2d3, 0xf2d3, PDF_CMAP_SINGLE, 9693 },
	{ 0xf2d4, 0xf2e4, PDF_CMAP_RANGE, 7190 },
	{ 0xf2e5, 0xf2e5, PDF_CMAP_SINGLE, 9694 },
	{ 0xf2e6, 0xf2ed, PDF_CMAP_RANGE, 7208 },
	{ 0xf2ee, 0xf2ee, PDF_CMAP_SINGLE, 9695 },
	{ 0xf2ef, 0xf2f6, PDF_CMAP_RANGE, 7217 },
	{ 0xf2f7, 0xf2f7, PDF_CMAP_SINGLE, 9696 },
	{ 0xf2f8, 0xf2fc, PDF_CMAP_RANGE, 7226 },
	{ 0xf2fd, 0xf2fe, PDF_CMAP_TABLE, 600 },
	{ 0xf3a1, 0xf3be, PDF_CMAP_RANGE, 7233 },
	{ 0xf3bf, 0xf3bf, PDF_CMAP_SINGLE, 9698 },
	{ 0xf3c0, 0xf3c5, PDF_CMAP_RANGE, 7264 },
	{ 0xf3c6, 0xf3c8, PDF_CMAP_TABLE, 602 },
	{ 0xf3c9, 0xf3d5, PDF_CMAP_RANGE, 7273 },
	{ 0xf3d6, 0xf3d6, PDF_CMAP_SINGLE, 9701 },
	{ 0xf3d7, 0xf3d8, PDF_CMAP_RANGE, 7287 },
	{ 0xf3d9, 0xf3d9, PDF_CMAP_SINGLE, 9702 },
	{ 0xf3da, 0xf3e4, PDF_CMAP_RANGE, 7290 },
	{ 0xf3e5, 0xf3e6, PDF_CMAP_RANGE, 9703 },
	{ 0xf3e7, 0xf3e9, PDF_CMAP_RANGE, 7303 },
	{ 0xf3ea, 0xf3ec, PDF_CMAP_TABLE, 605 },
	{ 0xf3ed, 0xf3ee, PDF_CMAP_RANGE, 7309 },
	{ 0xf3ef, 0xf3f1, PDF_CMAP_TABLE, 608 },
	{ 0xf3f2, 0xf3fc, PDF_CMAP_RANGE, 7314 },
	{ 0xf3fd, 0xf3fe, PDF_CMAP_TABLE, 611 },
	{ 0xf4a1, 0xf4a4, PDF_CMAP_RANGE, 7327 },
	{ 0xf4a5, 0xf4a5, PDF_CMAP_SINGLE, 9710 },
	{ 0xf4a6, 0xf4ae, PDF_CMAP_RANGE, 7332 },
	{ 0xf4af, 0xf4af, PDF_CMAP_SINGLE, 9711 },
	{ 0xf4b0, 0xf4b4, PDF_CMAP_RANGE, 7342 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 9712 },
	{ 0xf4b6, 0xf4c0, PDF_CMAP_RANGE, 7348 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 9713 },
	{ 0xf4c2, 0xf4c6, PDF_CMAP_RANGE, 7360 },
	{ 0xf4c7, 0xf4c7, PDF_CMAP_SINGLE, 9714 },
	{ 0xf4c8, 0xf4ce, PDF_CMAP_RANGE, 7366 },
	{ 0xf4cf, 0xf4d0, PDF_CMAP_RANGE, 9715 },
	{ 0xf4d1, 0xf4d5, PDF_CMAP_RANGE, 7375 },
	{ 0xf4d6, 0xf4d6, PDF_CMAP_SINGLE, 9717 },
	{ 0xf4d7, 0xf4e9, PDF_CMAP_RANGE, 7381 },
	{ 0xf4ea, 0xf4ea, PDF_CMAP_SINGLE, 9718 },
	{ 0xf4eb, 0xf4ee, PDF_CMAP_RANGE, 7401 },
	{ 0xf4ef, 0xf4ef, PDF_CMAP_SINGLE, 9719 },
	{ 0xf4f0, 0xf4f4, PDF_CMAP_RANGE, 7406 },
	{ 0xf4f5, 0xf4f5, PDF_CMAP_SINGLE, 9720 },
	{ 0xf4f6, 0xf4fe, PDF_CMAP_RANGE, 7412 },
	{ 0xf5a1, 0xf5a5, PDF_CMAP_RANGE, 7421 },
	{ 0xf5a6, 0xf5a7, PDF_CMAP_RANGE, 9721 },
	{ 0xf5a8, 0xf5b9, PDF_CMAP_RANGE, 7428 },
	{ 0xf5ba, 0xf5bb, PDF_CMAP_RANGE, 9723 },
	{ 0xf5bc, 0xf5c3, PDF_CMAP_RANGE, 7448 },
	{ 0xf5c4, 0xf5c4, PDF_CMAP_SINGLE, 9725 },
	{ 0xf5c5, 0xf5c7, PDF_CMAP_RANGE, 7457 },
	{ 0xf5c8, 0xf5c8, PDF_CMAP_SINGLE, 9726 },
	{ 0xf5c9, 0xf5cd, PDF_CMAP_RANGE, 7461 },
	{ 0xf5ce, 0xf5cf, PDF_CMAP_RANGE, 9727 },
	{ 0xf5d0, 0xf5d0, PDF_CMAP_SINGLE, 7468 },
	{ 0xf5d1, 0xf5d2, PDF_CMAP_RANGE, 9729 },
	{ 0xf5d3, 0xf5d8, PDF_CMAP_RANGE, 7471 },
	{ 0xf5d9, 0xf5d9, PDF_CMAP_SINGLE, 9731 },
	{ 0xf5da, 0xf5db, PDF_CMAP_RANGE, 7478 },
	{ 0xf5dc, 0xf5dc, PDF_CMAP_SINGLE, 9732 },
	{ 0xf5dd, 0xf5e5, PDF_CMAP_RANGE, 7481 },
	{ 0xf5e6, 0xf5e7, PDF_CMAP_RANGE, 9733 },
	{ 0xf5e8, 0xf5ee, PDF_CMAP_RANGE, 7492 },
	{ 0xf5ef, 0xf5ef, PDF_CMAP_SINGLE, 9735 },
	{ 0xf5f0, 0xf5f1, PDF_CMAP_RANGE, 7500 },
	{ 0xf5f2, 0xf5f2, PDF_CMAP_SINGLE, 9736 },
	{ 0xf5f3, 0xf5fb, PDF_CMAP_RANGE, 7503 },
	{ 0xf5fc, 0xf5fc, PDF_CMAP_SINGLE, 9737 },
	{ 0xf5fd, 0xf5fe, PDF_CMAP_RANGE, 7513 },
	{ 0xf6a1, 0xf6a2, PDF_CMAP_RANGE, 7515 },
	{ 0xf6a3, 0xf6a3, PDF_CMAP_SINGLE, 9738 },
	{ 0xf6a4, 0xf6a5, PDF_CMAP_RANGE, 7518 },
	{ 0xf6a6, 0xf6a8, PDF_CMAP_TABLE, 613 },
	{ 0xf6a9, 0xf6aa, PDF_CMAP_RANGE, 7523 },
	{ 0xf6ab, 0xf6ab, PDF_CMAP_SINGLE, 9741 },
	{ 0xf6ac, 0xf6af, PDF_CMAP_RANGE, 7526 },
	{ 0xf6b0, 0xf6b0, PDF_CMAP_SINGLE, 9742 },
	{ 0xf6b1, 0xf6b2, PDF_CMAP_RANGE, 7531 },
	{ 0xf6b3, 0xf6be, PDF_CMAP_RANGE, 9743 },
	{ 0xf6bf, 0xf6c4, PDF_CMAP_RANGE, 7545 },
	{ 0xf6c5, 0xf6c9, PDF_CMAP_TABLE, 616 },
	{ 0xf6ca, 0xf6ce, PDF_CMAP_RANGE, 7556 },
	{ 0xf6cf, 0xf6fe, PDF_CMAP_RANGE, 9758 },
	{ 0xf7a1, 0xf7af, PDF_CMAP_RANGE, 9806 },
	{ 0xf7b0, 0xf7b1, PDF_CMAP_RANGE, 7624 },
	{ 0xf7b2, 0xf7b3, PDF_CMAP_RANGE, 9821 },
	{ 0xf7b4, 0xf7b5, PDF_CMAP_TABLE, 621 },
	{ 0xf7b6, 0xf7bc, PDF_CMAP_RANGE, 7630 },
	{ 0xf7bd, 0xf7bd, PDF_CMAP_SINGLE, 9824 },
	{ 0xf7be, 0xf7c2, PDF_CMAP_RANGE, 7638 },
	{ 0xf7c3, 0xf7c4, PDF_CMAP_TABLE, 623 },
	{ 0xf7c5, 0xf7c6, PDF_CMAP_RANGE, 9826 },
	{ 0xf7c7, 0xf7c9, PDF_CMAP_RANGE, 7647 },
	{ 0xf7ca, 0xf7cb, PDF_CMAP_RANGE, 9828 },
	{ 0xf7cc, 0xf7ce, PDF_CMAP_RANGE, 7652 },
	{ 0xf7cf, 0xf7d0, PDF_CMAP_RANGE, 9830 },
	{ 0xf7d1, 0xf7dd, PDF_CMAP_RANGE, 7657 },
	{ 0xf7de, 0xf7de, PDF_CMAP_SINGLE, 9832 },
	{ 0xf7df, 0xf7e0, PDF_CMAP_RANGE, 7671 },
	{ 0xf7e1, 0xf7e1, PDF_CMAP_SINGLE, 2745 },
	{ 0xf7e2, 0xf7f1, PDF_CMAP_RANGE, 7674 },
	{ 0xf7f2, 0xf7f2, PDF_CMAP_SINGLE, 9833 },
	{ 0xf7f3, 0xf7f4, PDF_CMAP_RANGE, 7691 },
	{ 0xf7f5, 0xf7f5, PDF_CMAP_SINGLE, 9834 },
	{ 0xf7f6, 0xf7fe, PDF_CMAP_RANGE, 7694 },
	{ 0xf8a1, 0xf8a6, PDF_CMAP_RANGE, 9835 },
	{ 0xf8a7, 0xf8a9, PDF_CMAP_TABLE, 625 },
	{ 0xf8aa, 0xf8ab, PDF_CMAP_RANGE, 9841 },
	{ 0xf8ac, 0xf8af, PDF_CMAP_TABLE, 628 },
	{ 0xf8b0, 0xf8b1, PDF_CMAP_RANGE, 9844 },
	{ 0xf8b2, 0xf8b5, PDF_CMAP_TABLE, 632 },
	{ 0xf8b6, 0xf8b7, PDF_CMAP_RANGE, 9847 },
	{ 0xf8b8, 0xf8b8, PDF_CMAP_SINGLE, 1852 },
	{ 0xf8b9, 0xf8ba, PDF_CMAP_RANGE, 9849 },
	{ 0xf8bb, 0xf8bb, PDF_CMAP_SINGLE, 1987 },
	{ 0xf8bc, 0xf8bf, PDF_CMAP_RANGE, 9851 },
	{ 0xf8c0, 0xf8c3, PDF_CMAP_TABLE, 636 },
	{ 0xf8c4, 0xf8c5, PDF_CMAP_RANGE, 9856 },
	{ 0xf8c6, 0xf8c6, PDF_CMAP_SINGLE, 2316 },
	{ 0xf8c7, 0xf8c8, PDF_CMAP_RANGE, 9858 },
	{ 0xf8c9, 0xf8ca, PDF_CMAP_TABLE, 640 },
	{ 0xf8cb, 0xf8cc, PDF_CMAP_RANGE, 2458 },
	{ 0xf8cd, 0xf8d0, PDF_CMAP_TABLE, 642 },
	{ 0xf8d1, 0xf8db, PDF_CMAP_RANGE, 9863 },
	{ 0xf8dc, 0xf8e0, PDF_CMAP_TABLE, 646 },
	{ 0xf8e1, 0xf8e5, PDF_CMAP_RANGE, 9874 },
	{ 0xf8e6, 0xf8eb, PDF_CMAP_TABLE, 651 },
	{ 0xf8ec, 0xf8ee, PDF_CMAP_RANGE, 9880 },
	{ 0xf8ef, 0xf8f1, PDF_CMAP_TABLE, 657 },
	{ 0xf8f2, 0xf8f7, PDF_CMAP_RANGE, 9884 },
	{ 0xf8f8, 0xf8f9, PDF_CMAP_TABLE, 660 },
	{ 0xf8fa, 0xf8fb, PDF_CMAP_RANGE, 9890 },
	{ 0xf8fc, 0xf8fe, PDF_CMAP_TABLE, 662 },
	{ 0xf9a1, 0xf9a3, PDF_CMAP_TABLE, 665 },
	{ 0xf9a4, 0xf9a5, PDF_CMAP_RANGE, 9894 },
	{ 0xf9a6, 0xf9a9, PDF_CMAP_TABLE, 668 },
	{ 0xe7db, 0xe7e0, PDF_CMAP_RANGE, 6163 },
	{ 0xe7e1, 0xe7e2, PDF_CMAP_RANGE, 9307 },
	{ 0xe7e3, 0xe7e6, PDF_CMAP_RANGE, 6171 },
	{ 0xe7e7, 0xe7e7, PDF_CMAP_SINGLE, 9309 },
	{ 0xe7e8, 0xe7ee, PDF_CMAP_RANGE, 6176 },
	{ 0xe7ef, 0xe7ef, PDF_CMAP_SINGLE, 9310 },
	{ 0xe7f0, 0xe7f3, PDF_CMAP_RANGE, 6184 },
	{ 0xe7f4, 0xe7f6, PDF_CMAP_RANGE, 9311 },
	{ 0xe7f7, 0xe7fe, PDF_CMAP_RANGE, 6191 },
	{ 0xe8a1, 0xe8a7, PDF_CMAP_RANGE, 6199 },
	{ 0xe8a8, 0xe8a8, PDF_CMAP_SINGLE, 9314 },
	{ 0xe8a9, 0xe8ab, PDF_CMAP_RANGE, 6207 },
	{ 0xe8ac, 0xe8ac, PDF_CMAP_SINGLE, 9315 },
	{ 0xe8ad, 0xe8b5, PDF_CMAP_RANGE, 6211 },
	{ 0xe8b6, 0xe8b6, PDF_CMAP_SINGLE, 9316 },
	{ 0xe8b7, 0xe8b7, PDF_CMAP_SINGLE, 6221 },
	{ 0xe8b8, 0xe8ba, PDF_CMAP_RANGE, 9317 },
	{ 0xe8bb, 0xe8be, PDF_CMAP_RANGE, 6225 },
	{ 0xe8bf, 0xe8c0, PDF_CMAP_RANGE, 9320 },
	{ 0xe8c1, 0xe8c4, PDF_CMAP_RANGE, 6231 },
	{ 0xe8c5, 0xe8c5, PDF_CMAP_SINGLE, 9322 },
	{ 0xe8c6, 0xe8c6, PDF_CMAP_SINGLE, 6236 },
	{ 0xe8c7, 0xe8c9, PDF_CMAP_RANGE, 9323 },
	{ 0xe8ca, 0xe8cd, PDF_CMAP_RANGE, 6240 },
	{ 0xe8ce, 0xe8ce, PDF_CMAP_SINGLE, 9326 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 6245 },
	{ 0xe8d0, 0xe8d0, PDF_CMAP_SINGLE, 9327 },
	{ 0xe8d1, 0xe8d2, PDF_CMAP_RANGE, 6247 },
	{ 0xe8d3, 0xe8d3, PDF_CMAP_SINGLE, 9328 },
	{ 0xe8d4, 0xe8dc, PDF_CMAP_RANGE, 6250 },
	{ 0xe8dd, 0xe8dd, PDF_CMAP_SINGLE, 9329 },
	{ 0xe8de, 0xe8de, PDF_CMAP_SINGLE, 6260 },
	{ 0xe8df, 0xe8df, PDF_CMAP_SINGLE, 9330 },
	{ 0xe8e0, 0xe8e1, PDF_CMAP_RANGE, 6262 },
	{ 0xe8e2, 0xe8e3, PDF_CMAP_RANGE, 9331 },
	{ 0xe8e4, 0xe8e4, PDF_CMAP_SINGLE, 6266 },
	{ 0xe8e5, 0xe8e5, PDF_CMAP_SINGLE, 9333 },
	{ 0xe8e6, 0xe8e6, PDF_CMAP_SINGLE, 6268 },
	{ 0xe8e7, 0xe8e7, PDF_CMAP_SINGLE, 9334 },
	{ 0xe8e8, 0xe8ea, PDF_CMAP_RANGE, 6270 },
	{ 0xe8eb, 0xe8eb, PDF_CMAP_SINGLE, 9335 },
	{ 0xe8ec, 0xe8ec, PDF_CMAP_SINGLE, 6274 },
	{ 0xe8ed, 0xe8ed, PDF_CMAP_SINGLE, 9336 },
	{ 0xe8ee, 0xe8ee, PDF_CMAP_SINGLE, 6276 },
	{ 0xe8ef, 0xe8ef, PDF_CMAP_SINGLE, 9337 },
	{ 0xe8f0, 0xe8f8, PDF_CMAP_RANGE, 6278 },
	{ 0xe8f9, 0xe8f9, PDF_CMAP_SINGLE, 9338 },
	{ 0xe8fa, 0xe8fb, PDF_CMAP_RANGE, 6288 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 9339 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 6292 },
	{ 0xe9a1, 0xe9a1, PDF_CMAP_SINGLE, 9341 },
	{ 0xe9a2, 0xe9ac, PDF_CMAP_RANGE, 6294 },
	{ 0xe9ad, 0xe9ad, PDF_CMAP_SINGLE, 9342 },
	{ 0xe9ae, 0xe9b3, PDF_CMAP_RANGE, 6306 },
	{ 0xe9b4, 0xe9b5, PDF_CMAP_RANGE, 9343 },
	{ 0xe9b6, 0xe9b6, PDF_CMAP_SINGLE, 6314 },
	{ 0xe9b7, 0xe9b7, PDF_CMAP_SINGLE, 9345 },
	{ 0xe9b8, 0xe9c3, PDF_CMAP_RANGE, 6316 },
	{ 0xe9c4, 0xe9c4, PDF_CMAP_SINGLE, 9346 },
	{ 0xe9c5, 0xe9c5, PDF_CMAP_SINGLE, 6329 },
	{ 0xe9c6, 0xe9c6, PDF_CMAP_SINGLE, 9347 },
	{ 0xe9c7, 0xe9c8, PDF_CMAP_RANGE, 6331 },
	{ 0xe9c9, 0xe9c9, PDF_CMAP_SINGLE, 9348 },
	{ 0xe9ca, 0xe9d5, PDF_CMAP_RANGE, 6334 },
	{ 0xe9d6, 0xe9d6, PDF_CMAP_SINGLE, 9349 },
	{ 0xe9d7, 0xe9d9, PDF_CMAP_RANGE, 6347 },
	{ 0xe9da, 0xe9da, PDF_CMAP_SINGLE, 9350 },
	{ 0xe9db, 0xe9e3, PDF_CMAP_RANGE, 6351 },
	{ 0xe9e4, 0xe9e4, PDF_CMAP_SINGLE, 9351 },
	{ 0xe9e5, 0xe9e5, PDF_CMAP_SINGLE, 6361 },
	{ 0xe9e6, 0xe9e7, PDF_CMAP_RANGE, 9352 },
	{ 0xe9e8, 0xe9e8, PDF_CMAP_SINGLE, 6364 },
	{ 0xe9e9, 0xe9e9, PDF_CMAP_SINGLE, 9354 },
	{ 0xe9ea, 0xe9ea, PDF_CMAP_SINGLE, 6366 },
	{ 0xe9eb, 0xe9eb, PDF_CMAP_SINGLE, 9355 },
	{ 0xe9ec, 0xe9ec, PDF_CMAP_SINGLE, 6368 },
	{ 0xe9ed, 0xe9fe, PDF_CMAP_RANGE, 9356 },
	{ 0xeaa1, 0xeaa5, PDF_CMAP_RANGE, 9374 },
	{ 0xeaa6, 0xeaa6, PDF_CMAP_SINGLE, 6392 },
	{ 0xeaa7, 0xeaa8, PDF_CMAP_RANGE, 9379 },
	{ 0xeaa9, 0xeab0, PDF_CMAP_RANGE, 6395 },
	{ 0xeab1, 0xeab1, PDF_CMAP_SINGLE, 9381 },
	{ 0xeab2, 0xeabb, PDF_CMAP_RANGE, 6404 },
	{ 0xeabc, 0xeabc, PDF_CMAP_SINGLE, 9382 },
	{ 0xeabd, 0xeac9, PDF_CMAP_RANGE, 6415 },
	{ 0xeaca, 0xeaca, PDF_CMAP_SINGLE, 9383 },
	{ 0xeacb, 0xeacc, PDF_CMAP_RANGE, 6429 },
	{ 0xeacd, 0xeacd, PDF_CMAP_SINGLE, 9384 },
	{ 0xeace, 0xead2, PDF_CMAP_RANGE, 6432 },
	{ 0xead3, 0xead3, PDF_CMAP_SINGLE, 9385 },
	{ 0xead4, 0xead9, PDF_CMAP_RANGE, 6438 },
	{ 0xeada, 0xeaef, PDF_CMAP_RANGE, 9386 },
	{ 0xeaf0, 0xeafe, PDF_CMAP_RANGE, 6466 },
	{ 0xeba1, 0xeba6, PDF_CMAP_RANGE, 6481 },
	{ 0xeba7, 0xeba7, PDF_CMAP_SINGLE, 9408 },
	{ 0xeba8, 0xeba9, PDF_CMAP_RANGE, 6488 },
	{ 0xebaa, 0xebaa, PDF_CMAP_SINGLE, 9409 },
	{ 0xebab, 0xebb1, PDF_CMAP_RANGE, 6491 },
	{ 0xebb2, 0xebb2, PDF_CMAP_SINGLE, 9410 },
	{ 0xebb3, 0xebb8, PDF_CMAP_RANGE, 6499 },
	{ 0xebb9, 0xebb9, PDF_CMAP_SINGLE, 9411 },
	{ 0xebba, 0xebc9, PDF_CMAP_RANGE, 6506 },
	{ 0xebca, 0xebcb, PDF_CMAP_RANGE, 9412 },
	{ 0xebcc, 0xebcc, PDF_CMAP_SINGLE, 6524 },
	{ 0xebcd, 0xebcd, PDF_CMAP_SINGLE, 9414 },
	{ 0xebce, 0xebd5, PDF_CMAP_RANGE, 6526 },
	{ 0xebd6, 0xebd6, PDF_CMAP_SINGLE, 9415 },
	{ 0xebd7, 0xebd9, PDF_CMAP_RANGE, 6535 },
	{ 0xebda, 0xebda, PDF_CMAP_SINGLE, 9416 },
	{ 0xebdb, 0xebe0, PDF_CMAP_RANGE, 6539 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 9417 },
	{ 0xebe2, 0xebf6, PDF_CMAP_RANGE, 6546 },
	{ 0xebf7, 0xebf7, PDF_CMAP_SINGLE, 9418 },
	{ 0xebf8, 0xebfe, PDF_CMAP_RANGE, 6568 },
	{ 0xeca1, 0xeca2, PDF_CMAP_RANGE, 6575 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 9419 },
	{ 0xeca4, 0xeca8, PDF_CMAP_RANGE, 6578 },
	{ 0xeca9, 0xecae, PDF_CMAP_RANGE, 9420 },
	{ 0xecaf, 0xecb0, PDF_CMAP_RANGE, 6589 },
	{ 0xecb1, 0xecb1, PDF_CMAP_SINGLE, 9426 },
	{ 0xecb2, 0xecb3, PDF_CMAP_RANGE, 6592 },
	{ 0xecb4, 0xecb5, PDF_CMAP_RANGE, 9427 },
	{ 0xecb6, 0xecbd, PDF_CMAP_RANGE, 6596 },
	{ 0xecbe, 0xecbf, PDF_CMAP_RANGE, 9429 },
	{ 0xecc0, 0xecc0, PDF_CMAP_SINGLE, 6606 },
	{ 0xecc1, 0xecc1, PDF_CMAP_SINGLE, 9431 },
	{ 0xecc2, 0xecc6, PDF_CMAP_RANGE, 6608 },
	{ 0xecc7, 0xecc7, PDF_CMAP_SINGLE, 9432 },
	{ 0xecc8, 0xecca, PDF_CMAP_RANGE, 6614 },
	{ 0xeccb, 0xeccb, PDF_CMAP_SINGLE, 9433 },
	{ 0xeccc, 0xece1, PDF_CMAP_RANGE, 6618 },
	{ 0xece2, 0xece2, PDF_CMAP_SINGLE, 9434 },
	{ 0xece3, 0xecf1, PDF_CMAP_RANGE, 6641 },
	{ 0xecf2, 0xecf2, PDF_CMAP_SINGLE, 9435 },
	{ 0xecf3, 0xecf4, PDF_CMAP_RANGE, 6657 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 9436 },
	{ 0xecf6, 0xecf7, PDF_CMAP_RANGE, 6660 },
	{ 0xecf8, 0xecf8, PDF_CMAP_SINGLE, 9437 },
	{ 0xecf9, 0xecfe, PDF_CMAP_RANGE, 6663 },
	{ 0xeda1, 0xeda1, PDF_CMAP_SINGLE, 9438 },
	{ 0xeda2, 0xeda7, PDF_CMAP_RANGE, 6670 },
	{ 0xeda8, 0xeda8, PDF_CMAP_SINGLE, 9439 },
	{ 0xeda9, 0xedae, PDF_CMAP_RANGE, 6677 },
	{ 0xedaf, 0xedb0, PDF_CMAP_RANGE, 9440 },
	{ 0xedb1, 0xedb3, PDF_CMAP_RANGE, 6685 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 9442 },
	{ 0xedb5, 0xedb5, PDF_CMAP_SINGLE, 6689 },
	{ 0xedb6, 0xedb6, PDF_CMAP_SINGLE, 9443 },
	{ 0xedb7, 0xedb7, PDF_CMAP_SINGLE, 6691 },
	{ 0xedb8, 0xedb8, PDF_CMAP_SINGLE, 9444 },
	{ 0xedb9, 0xedb9, PDF_CMAP_SINGLE, 6693 },
	{ 0xedba, 0xedba, PDF_CMAP_SINGLE, 9445 },
	{ 0xedbb, 0xedbe, PDF_CMAP_RANGE, 6695 },
	{ 0xedbf, 0xedbf, PDF_CMAP_SINGLE, 9446 },
	{ 0xedc0, 0xedc1, PDF_CMAP_RANGE, 6700 },
	{ 0xedc2, 0xedc3, PDF_CMAP_RANGE, 9447 },
	{ 0xedc4, 0xedcb, PDF_CMAP_RANGE, 6704 },
	{ 0xedcc, 0xedcd, PDF_CMAP_RANGE, 9449 },
	{ 0xedce, 0xedd2, PDF_CMAP_RANGE, 6714 },
	{ 0xedd3, 0xedd3, PDF_CMAP_SINGLE, 9451 },
	{ 0xedd4, 0xedd6, PDF_CMAP_RANGE, 6720 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 9452 },
	{ 0xedd8, 0xede7, PDF_CMAP_RANGE, 6724 },
	{ 0xede8, 0xede8, PDF_CMAP_SINGLE, 9453 },
	{ 0xede9, 0xeded, PDF_CMAP_RANGE, 6741 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 9454 },
	{ 0xedef, 0xedf8, PDF_CMAP_RANGE, 6747 },
	{ 0xedf9, 0xedfa, PDF_CMAP_RANGE, 9455 },
	{ 0xedfb, 0xedfe, PDF_CMAP_RANGE, 6759 },
	{ 0xeea1, 0xeebb, PDF_CMAP_RANGE, 6763 },
	{ 0xeebc, 0xeebc, PDF_CMAP_SINGLE, 9457 },
	{ 0xeebd, 0xeebe, PDF_CMAP_RANGE, 6791 },
	{ 0xeebf, 0xeebf, PDF_CMAP_SINGLE, 9458 },
	{ 0xeec0, 0xeec3, PDF_CMAP_RANGE, 6794 },
	{ 0xeec4, 0xeefe, PDF_CMAP_RANGE, 9459 },
	{ 0xefa1, 0xeff1, PDF_CMAP_RANGE, 9518 },
	{ 0xeff2, 0xeffe, PDF_CMAP_RANGE, 6938 },
	{ 0xf0a1, 0xf0a2, PDF_CMAP_RANGE, 6951 },
	{ 0xf0a3, 0xf0a3, PDF_CMAP_SINGLE, 9599 },
	{ 0xf0a4, 0xf0ae, PDF_CMAP_RANGE, 6954 },
	{ 0xf0af, 0xf0d9, PDF_CMAP_RANGE, 9600 },
	{ 0xf0da, 0xf0db, PDF_CMAP_RANGE, 7008 },
	{ 0xf0dc, 0xf0dd, PDF_CMAP_RANGE, 9643 },
	{ 0xf0de, 0xf0de, PDF_CMAP_SINGLE, 7012 },
	{ 0xf0df, 0xf0df, PDF_CMAP_SINGLE, 9645 },
	{ 0xf0e0, 0xf0e8, PDF_CMAP_RANGE, 7014 },
	{ 0xf0e9, 0xf0e9, PDF_CMAP_SINGLE, 9646 },
	{ 0xf0ea, 0xf0eb, PDF_CMAP_RANGE, 7024 },
	{ 0xf0ec, 0xf0ec, PDF_CMAP_SINGLE, 9647 },
	{ 0xf0ed, 0xf0ee, PDF_CMAP_RANGE, 7027 },
	{ 0xf0ef, 0xf0ef, PDF_CMAP_SINGLE, 9648 },
	{ 0xf0f0, 0xf0f6, PDF_CMAP_RANGE, 7030 },
	{ 0xf0f7, 0xf0f7, PDF_CMAP_SINGLE, 9649 },
	{ 0xf0f8, 0xf0f8, PDF_CMAP_SINGLE, 7038 },
	{ 0xf0f9, 0xf0f9, PDF_CMAP_SINGLE, 9650 },
	{ 0xf0fa, 0xf0fb, PDF_CMAP_RANGE, 7040 },
	{ 0xf0fc, 0xf0fc, PDF_CMAP_SINGLE, 9651 },
	{ 0xf0fd, 0xf0fe, PDF_CMAP_RANGE, 7043 },
	{ 0xf1a1, 0xf1a7, PDF_CMAP_RANGE, 7045 },
	{ 0xf1a8, 0xf1a8, PDF_CMAP_SINGLE, 9652 },
	{ 0xf1a9, 0xf1aa, PDF_CMAP_RANGE, 7053 },
	{ 0xf1ab, 0xf1ab, PDF_CMAP_SINGLE, 9653 },
	{ 0xf1ac, 0xf1ad, PDF_CMAP_RANGE, 7056 },
	{ 0xf1ae, 0xf1ae, PDF_CMAP_SINGLE, 9654 },
	{ 0xf1af, 0xf1b1, PDF_CMAP_RANGE, 7059 },
	{ 0xf1b2, 0xf1b2, PDF_CMAP_SINGLE, 9655 },
	{ 0xf1b3, 0xf1bb, PDF_CMAP_RANGE, 7063 },
	{ 0xf1bc, 0xf1bc, PDF_CMAP_SINGLE, 9656 },
	{ 0xf1bd, 0xf1bf, PDF_CMAP_RANGE, 7073 },
	{ 0xf1c0, 0xf1c0, PDF_CMAP_SINGLE, 9657 },
	{ 0xf1c1, 0xf1c8, PDF_CMAP_RANGE, 7077 },
	{ 0xf1c9, 0xf1c9, PDF_CMAP_SINGLE, 9658 },
	{ 0xf1ca, 0xf1cc, PDF_CMAP_RANGE, 7086 },
	{ 0xf1cd, 0xf1cd, PDF_CMAP_SINGLE, 9659 },
	{ 0xf1ce, 0xf1ce, PDF_CMAP_SINGLE, 7090 },
	{ 0xf1cf, 0xf1d0, PDF_CMAP_RANGE, 9660 },
	{ 0xf1d1, 0xf1d9, PDF_CMAP_RANGE, 7093 },
	{ 0xf1da, 0xf1da, PDF_CMAP_SINGLE, 9662 },
	{ 0xf1db, 0xf1db, PDF_CMAP_SINGLE, 7103 },
	{ 0xf1dc, 0xf1dc, PDF_CMAP_SINGLE, 9663 },
	{ 0xf1dd, 0xf1e3, PDF_CMAP_RANGE, 7105 },
	{ 0xf1e4, 0xf1e4, PDF_CMAP_SINGLE, 9664 },
	{ 0xf1e5, 0xf1eb, PDF_CMAP_RANGE, 7113 },
	{ 0xf1ec, 0xf1ec, PDF_CMAP_SINGLE, 9665 },
	{ 0xf1ed, 0xf1ee, PDF_CMAP_RANGE, 7121 },
	{ 0xf1ef, 0xf1ef, PDF_CMAP_SINGLE, 9666 },
	{ 0xf1f0, 0xf1f6, PDF_CMAP_RANGE, 7124 },
	{ 0xf1f7, 0xf1f7, PDF_CMAP_SINGLE, 9667 },
	{ 0xf1f8, 0xf1f8, PDF_CMAP_SINGLE, 7132 },
	{ 0xf1f9, 0xf1f9, PDF_CMAP_SINGLE, 9668 },
	{ 0xf1fa, 0xf1fb, PDF_CMAP_RANGE, 7134 },
	{ 0xf1fc, 0xf1fe, PDF_CMAP_RANGE, 9669 },
	{ 0xf2a1, 0xf2ad, PDF_CMAP_RANGE, 9672 },
	{ 0xf2ae, 0xf2b0, PDF_CMAP_RANGE, 7152 },
	{ 0xf2b1, 0xf2b2, PDF_CMAP_RANGE, 9685 },
	{ 0xf2b3, 0xf2b8, PDF_CMAP_RANGE, 7157 },
	{ 0xf2b9, 0xf2b9, PDF_CMAP_SINGLE, 9687 },
	{ 0xf2ba, 0xf2c2, PDF_CMAP_RANGE, 7164 },
	{ 0xf2c3, 0xf2c3, PDF_CMAP_SINGLE, 9688 },
	{ 0xf2c4, 0xf2c8, PDF_CMAP_RANGE, 7174 },
	{ 0xf2c9, 0xf2c9, PDF_CMAP_SINGLE, 9689 },
	{ 0xf2ca, 0xf2cb, PDF_CMAP_RANGE, 7180 },
	{ 0xf2cc, 0xf2cd, PDF_CMAP_RANGE, 9690 },
	{ 0xf2ce, 0xf2ce, PDF_CMAP_SINGLE, 7184 },
	{ 0xf2cf, 0xf2cf, PDF_CMAP_SINGLE, 9692 },
	{ 0xf2d0, 0xf2d2, PDF_CMAP_RANGE, 7186 },
	{ 0xf2d3, 0xf2d3, PDF_CMAP_SINGLE, 9693 },
	{ 0xf2d4, 0xf2e4, PDF_CMAP_RANGE, 7190 },
	{ 0xf2e5, 0xf2e5, PDF_CMAP_SINGLE, 9694 },
	{ 0xf2e6, 0xf2ed, PDF_CMAP_RANGE, 7208 },
	{ 0xf2ee, 0xf2ee, PDF_CMAP_SINGLE, 9695 },
	{ 0xf2ef, 0xf2f6, PDF_CMAP_RANGE, 7217 },
	{ 0xf2f7, 0xf2f7, PDF_CMAP_SINGLE, 9696 },
	{ 0xf2f8, 0xf2fc, PDF_CMAP_RANGE, 7226 },
	{ 0xf2fd, 0xf2fd, PDF_CMAP_SINGLE, 9697 },
	{ 0xf2fe, 0xf2fe, PDF_CMAP_SINGLE, 7232 },
	{ 0xf3a1, 0xf3be, PDF_CMAP_RANGE, 7233 },
	{ 0xf3bf, 0xf3bf, PDF_CMAP_SINGLE, 9698 },
	{ 0xf3c0, 0xf3c5, PDF_CMAP_RANGE, 7264 },
	{ 0xf3c6, 0xf3c6, PDF_CMAP_SINGLE, 9699 },
	{ 0xf3c7, 0xf3c7, PDF_CMAP_SINGLE, 7271 },
	{ 0xf3c8, 0xf3c8, PDF_CMAP_SINGLE, 9700 },
	{ 0xf3c9, 0xf3d5, PDF_CMAP_RANGE, 7273 },
	{ 0xf3d6, 0xf3d6, PDF_CMAP_SINGLE, 9701 },
	{ 0xf3d7, 0xf3d8, PDF_CMAP_RANGE, 7287 },
	{ 0xf3d9, 0xf3d9, PDF_CMAP_SINGLE, 9702 },
	{ 0xf3da, 0xf3e4, PDF_CMAP_RANGE, 7290 },
	{ 0xf3e5, 0xf3e6, PDF_CMAP_RANGE, 9703 },
	{ 0xf3e7, 0xf3e9, PDF_CMAP_RANGE, 7303 },
	{ 0xf3ea, 0xf3ea, PDF_CMAP_SINGLE, 9705 },
	{ 0xf3eb, 0xf3eb, PDF_CMAP_SINGLE, 7307 },
	{ 0xf3ec, 0xf3ec, PDF_CMAP_SINGLE, 9706 },
	{ 0xf3ed, 0xf3ee, PDF_CMAP_RANGE, 7309 },
	{ 0xf3ef, 0xf3ef, PDF_CMAP_SINGLE, 9707 },
	{ 0xf3f0, 0xf3f0, PDF_CMAP_SINGLE, 7312 },
	{ 0xf3f1, 0xf3f1, PDF_CMAP_SINGLE, 9708 },
	{ 0xf3f2, 0xf3fc, PDF_CMAP_RANGE, 7314 },
	{ 0xf3fd, 0xf3fd, PDF_CMAP_SINGLE, 9709 },
	{ 0xf3fe, 0xf3fe, PDF_CMAP_SINGLE, 7326 },
	{ 0xf4a1, 0xf4a4, PDF_CMAP_RANGE, 7327 },
	{ 0xf4a5, 0xf4a5, PDF_CMAP_SINGLE, 9710 },
	{ 0xf4a6, 0xf4ae, PDF_CMAP_RANGE, 7332 },
	{ 0xf4af, 0xf4af, PDF_CMAP_SINGLE, 9711 },
	{ 0xf4b0, 0xf4b4, PDF_CMAP_RANGE, 7342 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 9712 },
	{ 0xf4b6, 0xf4c0, PDF_CMAP_RANGE, 7348 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 9713 },
	{ 0xf4c2, 0xf4c6, PDF_CMAP_RANGE, 7360 },
	{ 0xf4c7, 0xf4c7, PDF_CMAP_SINGLE, 9714 },
	{ 0xf4c8, 0xf4ce, PDF_CMAP_RANGE, 7366 },
	{ 0xf4cf, 0xf4d0, PDF_CMAP_RANGE, 9715 },
	{ 0xf4d1, 0xf4d5, PDF_CMAP_RANGE, 7375 },
	{ 0xf4d6, 0xf4d6, PDF_CMAP_SINGLE, 9717 },
	{ 0xf4d7, 0xf4e9, PDF_CMAP_RANGE, 7381 },
	{ 0xf4ea, 0xf4ea, PDF_CMAP_SINGLE, 9718 },
	{ 0xf4eb, 0xf4ee, PDF_CMAP_RANGE, 7401 },
	{ 0xf4ef, 0xf4ef, PDF_CMAP_SINGLE, 9719 },
	{ 0xf4f0, 0xf4f4, PDF_CMAP_RANGE, 7406 },
	{ 0xf4f5, 0xf4f5, PDF_CMAP_SINGLE, 9720 },
	{ 0xf4f6, 0xf4fe, PDF_CMAP_RANGE, 7412 },
	{ 0xf5a1, 0xf5a5, PDF_CMAP_RANGE, 7421 },
	{ 0xf5a6, 0xf5a7, PDF_CMAP_RANGE, 9721 },
	{ 0xf5a8, 0xf5b9, PDF_CMAP_RANGE, 7428 },
	{ 0xf5ba, 0xf5bb, PDF_CMAP_RANGE, 9723 },
	{ 0xf5bc, 0xf5c3, PDF_CMAP_RANGE, 7448 },
	{ 0xf5c4, 0xf5c4, PDF_CMAP_SINGLE, 9725 },
	{ 0xf5c5, 0xf5c7, PDF_CMAP_RANGE, 7457 },
	{ 0xf5c8, 0xf5c8, PDF_CMAP_SINGLE, 9726 },
	{ 0xf5c9, 0xf5cd, PDF_CMAP_RANGE, 7461 },
	{ 0xf5ce, 0xf5cf, PDF_CMAP_RANGE, 9727 },
	{ 0xf5d0, 0xf5d0, PDF_CMAP_SINGLE, 7468 },
	{ 0xf5d1, 0xf5d2, PDF_CMAP_RANGE, 9729 },
	{ 0xf5d3, 0xf5d8, PDF_CMAP_RANGE, 7471 },
	{ 0xf5d9, 0xf5d9, PDF_CMAP_SINGLE, 9731 },
	{ 0xf5da, 0xf5db, PDF_CMAP_RANGE, 7478 },
	{ 0xf5dc, 0xf5dc, PDF_CMAP_SINGLE, 9732 },
	{ 0xf5dd, 0xf5e5, PDF_CMAP_RANGE, 7481 },
	{ 0xf5e6, 0xf5e7, PDF_CMAP_RANGE, 9733 },
	{ 0xf5e8, 0xf5ee, PDF_CMAP_RANGE, 7492 },
	{ 0xf5ef, 0xf5ef, PDF_CMAP_SINGLE, 9735 },
	{ 0xf5f0, 0xf5f1, PDF_CMAP_RANGE, 7500 },
	{ 0xf5f2, 0xf5f2, PDF_CMAP_SINGLE, 9736 },
	{ 0xf5f3, 0xf5fb, PDF_CMAP_RANGE, 7503 },
	{ 0xf5fc, 0xf5fc, PDF_CMAP_SINGLE, 9737 },
	{ 0xf5fd, 0xf5fe, PDF_CMAP_RANGE, 7513 },
	{ 0xf6a1, 0xf6a2, PDF_CMAP_RANGE, 7515 },
	{ 0xf6a3, 0xf6a3, PDF_CMAP_SINGLE, 9738 },
	{ 0xf6a4, 0xf6a5, PDF_CMAP_RANGE, 7518 },
	{ 0xf6a6, 0xf6a6, PDF_CMAP_SINGLE, 9739 },
	{ 0xf6a7, 0xf6a7, PDF_CMAP_SINGLE, 7521 },
	{ 0xf6a8, 0xf6a8, PDF_CMAP_SINGLE, 9740 },
	{ 0xf6a9, 0xf6aa, PDF_CMAP_RANGE, 7523 },
	{ 0xf6ab, 0xf6ab, PDF_CMAP_SINGLE, 9741 },
	{ 0xf6ac, 0xf6af, PDF_CMAP_RANGE, 7526 },
	{ 0xf6b0, 0xf6b0, PDF_CMAP_SINGLE, 9742 },
	{ 0xf6b1, 0xf6b2, PDF_CMAP_RANGE, 7531 },
	{ 0xf6b3, 0xf6be, PDF_CMAP_RANGE, 9743 },
	{ 0xf6bf, 0xf6c4, PDF_CMAP_RANGE, 7545 },
	{ 0xf6c5, 0xf6c5, PDF_CMAP_SINGLE, 9755 },
	{ 0xf6c6, 0xf6c6, PDF_CMAP_SINGLE, 7552 },
	{ 0xf6c7, 0xf6c7, PDF_CMAP_SINGLE, 9756 },
	{ 0xf6c8, 0xf6c8, PDF_CMAP_SINGLE, 7554 },
	{ 0xf6c9, 0xf6c9, PDF_CMAP_SINGLE, 9757 },
	{ 0xf6ca, 0xf6ce, PDF_CMAP_RANGE, 7556 },
	{ 0xf6cf, 0xf6fe, PDF_CMAP_RANGE, 9758 },
	{ 0xf7a1, 0xf7af, PDF_CMAP_RANGE, 9806 },
	{ 0xf7b0, 0xf7b1, PDF_CMAP_RANGE, 7624 },
	{ 0xf7b2, 0xf7b3, PDF_CMAP_RANGE, 9821 },
	{ 0xf7b4, 0xf7b4, PDF_CMAP_SINGLE, 7628 },
	{ 0xf7b5, 0xf7b5, PDF_CMAP_SINGLE, 9823 },
	{ 0xf7b6, 0xf7bc, PDF_CMAP_RANGE, 7630 },
	{ 0xf7bd, 0xf7bd, PDF_CMAP_SINGLE, 9824 },
	{ 0xf7be, 0xf7c2, PDF_CMAP_RANGE, 7638 },
	{ 0xf7c3, 0xf7c3, PDF_CMAP_SINGLE, 9825 },
	{ 0xf7c4, 0xf7c4, PDF_CMAP_SINGLE, 7644 },
	{ 0xf7c5, 0xf7c6, PDF_CMAP_RANGE, 9826 },
	{ 0xf7c7, 0xf7c9, PDF_CMAP_RANGE, 7647 },
	{ 0xf7ca, 0xf7cb, PDF_CMAP_RANGE, 9828 },
	{ 0xf7cc, 0xf7ce, PDF_CMAP_RANGE, 7652 },
	{ 0xf7cf, 0xf7d0, PDF_CMAP_RANGE, 9830 },
	{ 0xf7d1, 0xf7dd, PDF_CMAP_RANGE, 7657 },
	{ 0xf7de, 0xf7de, PDF_CMAP_SINGLE, 9832 },
	{ 0xf7df, 0xf7e0, PDF_CMAP_RANGE, 7671 },
	{ 0xf7e1, 0xf7e1, PDF_CMAP_SINGLE, 2745 },
	{ 0xf7e2, 0xf7f1, PDF_CMAP_RANGE, 7674 },
	{ 0xf7f2, 0xf7f2, PDF_CMAP_SINGLE, 9833 },
	{ 0xf7f3, 0xf7f4, PDF_CMAP_RANGE, 7691 },
	{ 0xf7f5, 0xf7f5, PDF_CMAP_SINGLE, 9834 },
	{ 0xf7f6, 0xf7fe, PDF_CMAP_RANGE, 7694 },
	{ 0xf8a1, 0xf8a6, PDF_CMAP_RANGE, 9835 },
	{ 0xf8a7, 0xf8a7, PDF_CMAP_SINGLE, 1228 },
	{ 0xf8a8, 0xf8a8, PDF_CMAP_SINGLE, 1290 },
	{ 0xf8a9, 0xf8a9, PDF_CMAP_SINGLE, 1304 },
	{ 0xf8aa, 0xf8ab, PDF_CMAP_RANGE, 9841 },
	{ 0xf8ac, 0xf8ac, PDF_CMAP_SINGLE, 1428 },
	{ 0xf8ad, 0xf8ad, PDF_CMAP_SINGLE, 1486 },
	{ 0xf8ae, 0xf8ae, PDF_CMAP_SINGLE, 9843 },
	{ 0xf8af, 0xf8af, PDF_CMAP_SINGLE, 1526 },
	{ 0xf8b0, 0xf8b1, PDF_CMAP_RANGE, 9844 },
	{ 0xf8b2, 0xf8b2, PDF_CMAP_SINGLE, 1619 },
	{ 0xf8b3, 0xf8b3, PDF_CMAP_SINGLE, 1662 },
	{ 0xf8b4, 0xf8b4, PDF_CMAP_SINGLE, 9846 },
	{ 0xf8b5, 0xf8b5, PDF_CMAP_SINGLE, 1732 },
	{ 0xf8b6, 0xf8b7, PDF_CMAP_RANGE, 9847 },
	{ 0xf8b8, 0xf8b8, PDF_CMAP_SINGLE, 1852 },
	{ 0xf8b9, 0xf8ba, PDF_CMAP_RANGE, 9849 },
	{ 0xf8bb, 0xf8bb, PDF_CMAP_SINGLE, 1987 },
	{ 0xf8bc, 0xf8bf, PDF_CMAP_RANGE, 9851 },
	{ 0xf8c0, 0xf8c0, PDF_CMAP_SINGLE, 2091 },
	{ 0xf8c1, 0xf8c1, PDF_CMAP_SINGLE, 9855 },
	{ 0xf8c2, 0xf8c2, PDF_CMAP_SINGLE, 2126 },
	{ 0xf8c3, 0xf8c3, PDF_CMAP_SINGLE, 2153 },
	{ 0xf8c4, 0xf8c5, PDF_CMAP_RANGE, 9856 },
	{ 0xf8c6, 0xf8c6, PDF_CMAP_SINGLE, 2316 },
	{ 0xf8c7, 0xf8c8, PDF_CMAP_RANGE, 9858 },
	{ 0xf8c9, 0xf8c9, PDF_CMAP_SINGLE, 2417 },
	{ 0xf8ca, 0xf8ca, PDF_CMAP_SINGLE, 9860 },
	{ 0xf8cb, 0xf8cc, PDF_CMAP_RANGE, 2458 },
	{ 0xf8cd, 0xf8cd, PDF_CMAP_SINGLE, 9861 },
	{ 0xf8ce, 0xf8ce, PDF_CMAP_SINGLE, 2522 },
	{ 0xf8cf, 0xf8cf, PDF_CMAP_SINGLE, 9862 },
	{ 0xf8d0, 0xf8d0, PDF_CMAP_SINGLE, 2554 },
	{ 0xf8d1, 0xf8db, PDF_CMAP_RANGE, 9863 },
	{ 0xf8dc, 0xf8dc, PDF_CMAP_SINGLE, 3034 },
	{ 0xf8dd, 0xf8dd, PDF_CMAP_SINGLE, 3037 },
	{ 0xf8de, 0xf8de, PDF_CMAP_SINGLE, 3050 },
	{ 0xf8df, 0xf8df, PDF_CMAP_SINGLE, 3052 },
	{ 0xf8e0, 0xf8e0, PDF_CMAP_SINGLE, 3058 },
	{ 0xf8e1, 0xf8e5, PDF_CMAP_RANGE, 9874 },
	{ 0xf8e6, 0xf8e6, PDF_CMAP_SINGLE, 3218 },
	{ 0xf8e7, 0xf8e7, PDF_CMAP_SINGLE, 3354 },
	{ 0xf8e8, 0xf8e8, PDF_CMAP_SINGLE, 3468 },
	{ 0xf8e9, 0xf8e9, PDF_CMAP_SINGLE, 3518 },
	{ 0xf8ea, 0xf8ea, PDF_CMAP_SINGLE, 9879 },
	{ 0xf8eb, 0xf8eb, PDF_CMAP_SINGLE, 3579 },
	{ 0xf8ec, 0xf8ee, PDF_CMAP_RANGE, 9880 },
	{ 0xf8ef, 0xf8ef, PDF_CMAP_SINGLE, 3696 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 9883 },
	{ 0xf8f1, 0xf8f1, PDF_CMAP_SINGLE, 3747 },
	{ 0xf8f2, 0xf8f7, PDF_CMAP_RANGE, 9884 },
	{ 0xf8f8, 0xf8f8, PDF_CMAP_SINGLE, 4157 },
	{ 0xf8f9, 0xf8f9, PDF_CMAP_SINGLE, 4313 },
	{ 0xf8fa, 0xf8fb, PDF_CMAP_RANGE, 9890 },
	{ 0xf8fc, 0xf8fc, PDF_CMAP_SINGLE, 4347 },
	{ 0xf8fd, 0xf8fd, PDF_CMAP_SINGLE, 4361 },
	{ 0xf8fe, 0xf8fe, PDF_CMAP_SINGLE, 9892 },
	{ 0xf9a1, 0xf9a1, PDF_CMAP_SINGLE, 4513 },
	{ 0xf9a2, 0xf9a2, PDF_CMAP_SINGLE, 9893 },
	{ 0xf9a3, 0xf9a3, PDF_CMAP_SINGLE, 4538 },
	{ 0xf9a4, 0xf9a5, PDF_CMAP_RANGE, 9894 },
	{ 0xf9a6, 0xf9a6, PDF_CMAP_SINGLE, 4565 },
	{ 0xf9a7, 0xf9a7, PDF_CMAP_SINGLE, 9896 },
	{ 0xf9a8, 0xf9a8, PDF_CMAP_SINGLE, 4605 },
	{ 0xf9a9, 0xf9a9, PDF_CMAP_SINGLE, 4633 },
};

static const unsigned short pdf_cmap_GBT_EUC_H_table[672] =
{
	7717,948,7718,7723,991,7724,7725,998,
	7726,1017,7730,7732,1027,7733,1040,7736,
	7738,1052,1082,7748,1106,7756,7763,1127,
	7780,1190,7781,1192,1229,7801,7806,1256,
	7807,1258,7808,7809,1267,7810,7812,1279,
	7813,1281,7814,7815,1289,1292,7818,7821,
	1301,7822,1303,7823,7839,1361,7840,7843,
	1374,7844,7845,1393,7846,7849,1413,7854,
	1422,1431,7862,7879,1503,1509,7882,1511,
	7883,1513,7884,7892,1540,7893,7894,1549,
	1556,7899,7900,1561,7901,7904,1580,1583,
	7907,7909,1593,1621,7921,7926,1645,7927,
	1663,7934,1675,7941,7948,1718,7949,1720,
	1731,7955,7964,1765,7965,7966,1775,1778,
	7969,7972,1803,7973,1805,7974,7981,1845,
	7982,1847,7983,1849,7984,1851,7985,1862,
	7991,7992,1867,7996,1874,7997,7998,1879,
	8010,1978,8011,1988,8016,8017,1993,8023,
	2023,8041,2067,2068,8042,2077,8047,2079,
	8048,8053,2101,8054,8057,2110,2122,8064,
	8065,2127,8066,2129,8072,2140,2180,8100,
	8124,2255,8128,2264,8129,8142,2325,8143,
	2327,8144,8146,2341,8154,2387,8155,8163,
	2423,8164,2425,8165,8166,2433,8167,2435,
	8168,2437,8171,2445,8172,8173,2453,8174,
	8196,2496,8197,2498,8198,8201,2517,8242,
	2611,8245,2621,8246,2661,8270,2663,8271,
	2680,8281,2710,8303,2712,8315,7673,8320,
	2770,8321,8333,2819,8338,2869,8349,2904,
	2912,8355,8364,2935,2939,8368,2941,8369,
	8380,3038,8381,8391,3087,8392,8395,3101,
	8396,3105,3111,8401,3132,8411,3134,8412,
	8413,3139,3155,8420,8432,3193,8433,3195,
	8438,3217,8439,3234,8444,8445,3241,8446,
	3284,8456,3286,8457,8459,3295,3324,8469,
	8470,3331,8471,3333,8472,8474,3350,8475,
	8476,3355,8477,3357,8478,3374,8483,3376,
	3390,8490,3392,8505,3446,8506,3469,8512,
	8514,3477,8516,3489,8517,8518,3497,3500,
	8521,8523,3517,8530,3529,8531,8534,3546,
	3564,8542,8554,3602,8555,8556,3616,8557,
	3618,8558,8560,3630,8561,8562,3636,8563,
	8576,3703,8577,8578,3716,3735,8586,3765,
	8591,3777,8598,3798,8603,3800,8604,3818,
	8612,3820,8615,3835,8616,8617,3841,8618,
	8621,3868,8622,3878,8626,8627,3883,8631,
	3891,8632,3900,8637,3910,8643,3912,8644,
	8648,3928,8649,3930,8650,8668,4023,8669,
	4044,8676,4065,8685,4067,8701,4118,4121,
	8704,4136,8709,8713,4155,8716,4165,8717,
	8718,4171,8719,4173,8720,8730,4215,8731,
	4242,8746,4252,8751,8753,4267,8754,8767,
	4322,8768,8769,4328,4346,8779,8780,4352,
	8783,4360,4363,8786,4365,8798,4392,8799,
	8800,4402,8810,4427,8811,8818,4445,8819,
	8821,4455,8822,4461,8825,4480,8829,4482,
	8830,8844,4539,8845,4541,8851,4563,8856,
	4575,8857,8858,4581,4616,8873,4618,4626,
	8880,4628,8893,4677,8897,4734,8898,4744,
	8903,8904,4752,8910,4802,8911,4804,8922,
	4857,8923,8987,4970,8988,8994,5004,8995,
	8996,5024,8997,9000,5043,9001,9026,5166,
	9046,5257,9047,9052,5306,5356,9063,5382,
	9067,9076,5417,9077,9079,5426,9080,9096,
	5552,9097,9100,5568,9101,1962,9107,9110,
	5627,9111,9112,5634,5681,9134,5703,9138,
	9142,5723,5729,9147,9192,5883,9193,9201,
	5956,9202,9204,5997,9205,6023,9211,9316,
	6221,9322,6236,9326,6245,9327,9329,6260,
	9330,6266,9333,6268,9334,9335,6274,9336,
	6276,9337,6314,9345,9346,6329,9347,9351,
	6361,6364,9354,6366,9355,6368,6524,9414,
	6606,9431,9442,6689,9443,6691,9444,6693,
	9445,7012,9645,9649,7038,9650,9659,7090,
	9662,7103,9663,9667,7132,9668,7184,9692,
	9697,7232,9699,7271,9700,9705,7307,9706,
	9707,7312,9708,9709,7326,9739,7521,9740,
	9755,7552,9756,7554,9757,7628,9823,9825,
	7644,1228,1290,1304,1428,1486,9843,1526,
	1619,1662,9846,1732,2091,9855,2126,2153,
	2417,9860,9861,2522,9862,2554,3034,3037,
	3050,3052,3058,3218,3354,3468,3518,9879,
	3579,3696,9883,3747,4157,4313,4347,4361,
	9892,4513,9893,4538,4565,9896,4605,4633,
};

pdf_cmap pdf_cmap_GBT_EUC_H =
{
	-1, "GBT-EUC-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa1a1, 0xfefe },
	},
	2283, 2283, (pdf_range*) pdf_cmap_GBT_EUC_H_ranges,
	672, 672, (unsigned short*) pdf_cmap_GBT_EUC_H_table,
};

/* GBT-EUC-V */

static const pdf_range pdf_cmap_GBT_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa3fd, 0xa3fd, PDF_CMAP_SINGLE, 597 },
	{ 0xa3fe, 0xa3fe, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GBT_EUC_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBT_EUC_V =
{
	-1, "GBT-EUC-V", "GBT-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GBT_EUC_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBT_EUC_V_table,
};

/* GBT-H */

static const pdf_range pdf_cmap_GBT_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 96 },
	{ 0x2231, 0x2262, PDF_CMAP_RANGE, 190 },
	{ 0x2265, 0x226e, PDF_CMAP_RANGE, 240 },
	{ 0x2271, 0x227c, PDF_CMAP_RANGE, 250 },
	{ 0x2321, 0x237e, PDF_CMAP_RANGE, 262 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 356 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 439 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 525 },
	{ 0x2641, 0x2675, PDF_CMAP_RANGE, 549 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 602 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 635 },
	{ 0x2821, 0x2840, PDF_CMAP_RANGE, 668 },
	{ 0x2845, 0x286a, PDF_CMAP_RANGE, 700 },
	{ 0x2924, 0x296f, PDF_CMAP_RANGE, 738 },
	{ 0x2a21, 0x2a7e, PDF_CMAP_RANGE, 814 },
	{ 0x2b21, 0x2b40, PDF_CMAP_RANGE, 908 },
	{ 0x3021, 0x3027, PDF_CMAP_RANGE, 940 },
	{ 0x3028, 0x302a, PDF_CMAP_TABLE, 0 },
	{ 0x302b, 0x302c, PDF_CMAP_RANGE, 950 },
	{ 0x302d, 0x302e, PDF_CMAP_RANGE, 7719 },
	{ 0x302f, 0x3038, PDF_CMAP_RANGE, 954 },
	{ 0x3039, 0x3039, PDF_CMAP_SINGLE, 7721 },
	{ 0x303a, 0x303f, PDF_CMAP_RANGE, 965 },
	{ 0x3040, 0x3040, PDF_CMAP_SINGLE, 7722 },
	{ 0x3041, 0x3052, PDF_CMAP_RANGE, 972 },
	{ 0x3053, 0x3055, PDF_CMAP_TABLE, 3 },
	{ 0x3056, 0x3059, PDF_CMAP_RANGE, 993 },
	{ 0x305a, 0x305c, PDF_CMAP_TABLE, 6 },
	{ 0x305d, 0x3063, PDF_CMAP_RANGE, 1000 },
	{ 0x3064, 0x3064, PDF_CMAP_SINGLE, 7727 },
	{ 0x3065, 0x306b, PDF_CMAP_RANGE, 1008 },
	{ 0x306c, 0x306d, PDF_CMAP_RANGE, 7728 },
	{ 0x306e, 0x306f, PDF_CMAP_TABLE, 9 },
	{ 0x3070, 0x3072, PDF_CMAP_RANGE, 1019 },
	{ 0x3073, 0x3073, PDF_CMAP_SINGLE, 7731 },
	{ 0x3074, 0x3076, PDF_CMAP_RANGE, 1023 },
	{ 0x3077, 0x3079, PDF_CMAP_TABLE, 11 },
	{ 0x307a, 0x307e, PDF_CMAP_RANGE, 1029 },
	{ 0x3121, 0x3124, PDF_CMAP_RANGE, 1034 },
	{ 0x3125, 0x3126, PDF_CMAP_RANGE, 7734 },
	{ 0x3127, 0x3128, PDF_CMAP_TABLE, 14 },
	{ 0x3129, 0x312a, PDF_CMAP_RANGE, 1042 },
	{ 0x312b, 0x312b, PDF_CMAP_SINGLE, 7737 },
	{ 0x312c, 0x3131, PDF_CMAP_RANGE, 1045 },
	{ 0x3132, 0x3133, PDF_CMAP_TABLE, 16 },
	{ 0x3134, 0x3135, PDF_CMAP_RANGE, 7739 },
	{ 0x3136, 0x3136, PDF_CMAP_SINGLE, 1055 },
	{ 0x3137, 0x3139, PDF_CMAP_RANGE, 7741 },
	{ 0x313a, 0x3140, PDF_CMAP_RANGE, 1059 },
	{ 0x3141, 0x3141, PDF_CMAP_SINGLE, 7744 },
	{ 0x3142, 0x3149, PDF_CMAP_RANGE, 1067 },
	{ 0x314a, 0x314a, PDF_CMAP_SINGLE, 7745 },
	{ 0x314b, 0x314e, PDF_CMAP_RANGE, 1076 },
	{ 0x314f, 0x3150, PDF_CMAP_RANGE, 7746 },
	{ 0x3151, 0x3152, PDF_CMAP_TABLE, 18 },
	{ 0x3153, 0x3154, PDF_CMAP_RANGE, 1084 },
	{ 0x3155, 0x3155, PDF_CMAP_SINGLE, 7749 },
	{ 0x3156, 0x315e, PDF_CMAP_RANGE, 1087 },
	{ 0x315f, 0x3161, PDF_CMAP_RANGE, 7750 },
	{ 0x3162, 0x3163, PDF_CMAP_RANGE, 1099 },
	{ 0x3164, 0x3164, PDF_CMAP_SINGLE, 7753 },
	{ 0x3165, 0x3166, PDF_CMAP_RANGE, 1102 },
	{ 0x3167, 0x3168, PDF_CMAP_RANGE, 7754 },
	{ 0x3169, 0x316a, PDF_CMAP_TABLE, 20 },
	{ 0x316b, 0x316d, PDF_CMAP_RANGE, 1108 },
	{ 0x316e, 0x316e, PDF_CMAP_SINGLE, 7757 },
	{ 0x316f, 0x3170, PDF_CMAP_RANGE, 1112 },
	{ 0x3171, 0x3171, PDF_CMAP_SINGLE, 7758 },
	{ 0x3172, 0x3173, PDF_CMAP_RANGE, 1115 },
	{ 0x3174, 0x3177, PDF_CMAP_RANGE, 7759 },
	{ 0x3178, 0x317c, PDF_CMAP_RANGE, 1121 },
	{ 0x317d, 0x317e, PDF_CMAP_TABLE, 22 },
	{ 0x3221, 0x3225, PDF_CMAP_RANGE, 1128 },
	{ 0x3226, 0x3227, PDF_CMAP_RANGE, 7764 },
	{ 0x3228, 0x322b, PDF_CMAP_RANGE, 1135 },
	{ 0x322c, 0x322c, PDF_CMAP_SINGLE, 7766 },
	{ 0x322d, 0x3234, PDF_CMAP_RANGE, 1140 },
	{ 0x3235, 0x3235, PDF_CMAP_SINGLE, 7767 },
	{ 0x3236, 0x3238, PDF_CMAP_RANGE, 1149 },
	{ 0x3239, 0x3239, PDF_CMAP_SINGLE, 7768 },
	{ 0x323a, 0x3245, PDF_CMAP_RANGE, 1153 },
	{ 0x3246, 0x3246, PDF_CMAP_SINGLE, 7769 },
	{ 0x3247, 0x324d, PDF_CMAP_RANGE, 1166 },
	{ 0x324e, 0x3257, PDF_CMAP_RANGE, 7770 },
	{ 0x3258, 0x325d, PDF_CMAP_RANGE, 1183 },
	{ 0x325e, 0x3261, PDF_CMAP_TABLE, 24 },
	{ 0x3262, 0x3263, PDF_CMAP_RANGE, 7782 },
	{ 0x3264, 0x326e, PDF_CMAP_RANGE, 1195 },
	{ 0x326f, 0x326f, PDF_CMAP_SINGLE, 7784 },
	{ 0x3270, 0x3272, PDF_CMAP_RANGE, 1207 },
	{ 0x3273, 0x327c, PDF_CMAP_RANGE, 7785 },
	{ 0x327d, 0x327e, PDF_CMAP_RANGE, 1220 },
	{ 0x3321, 0x3322, PDF_CMAP_RANGE, 7795 },
	{ 0x3323, 0x3323, PDF_CMAP_SINGLE, 1224 },
	{ 0x3324, 0x3327, PDF_CMAP_RANGE, 7797 },
	{ 0x3328, 0x3329, PDF_CMAP_TABLE, 28 },
	{ 0x332a, 0x332d, PDF_CMAP_RANGE, 1231 },
	{ 0x332e, 0x332e, PDF_CMAP_SINGLE, 7802 },
	{ 0x332f, 0x3334, PDF_CMAP_RANGE, 1236 },
	{ 0x3335, 0x3335, PDF_CMAP_SINGLE, 7803 },
	{ 0x3336, 0x3338, PDF_CMAP_RANGE, 1243 },
	{ 0x3339, 0x3339, PDF_CMAP_SINGLE, 7804 },
	{ 0x333a, 0x333d, PDF_CMAP_RANGE, 1247 },
	{ 0x333e, 0x333e, PDF_CMAP_SINGLE, 7805 },
	{ 0x333f, 0x3341, PDF_CMAP_RANGE, 1252 },
	{ 0x3342, 0x3346, PDF_CMAP_TABLE, 30 },
	{ 0x3347, 0x334c, PDF_CMAP_RANGE, 1260 },
	{ 0x334d, 0x334f, PDF_CMAP_TABLE, 35 },
	{ 0x3350, 0x3351, PDF_CMAP_RANGE, 1269 },
	{ 0x3352, 0x3352, PDF_CMAP_SINGLE, 7811 },
	{ 0x3353, 0x3358, PDF_CMAP_RANGE, 1272 },
	{ 0x3359, 0x335d, PDF_CMAP_TABLE, 38 },
	{ 0x335e, 0x3362, PDF_CMAP_RANGE, 1283 },
	{ 0x3363, 0x3364, PDF_CMAP_TABLE, 43 },
	{ 0x3365, 0x3366, PDF_CMAP_RANGE, 7816 },
	{ 0x3367, 0x3368, PDF_CMAP_TABLE, 45 },
	{ 0x3369, 0x336a, PDF_CMAP_RANGE, 1294 },
	{ 0x336b, 0x336c, PDF_CMAP_RANGE, 7819 },
	{ 0x336d, 0x336e, PDF_CMAP_RANGE, 1298 },
	{ 0x336f, 0x3373, PDF_CMAP_TABLE, 47 },
	{ 0x3374, 0x3379, PDF_CMAP_RANGE, 1305 },
	{ 0x337a, 0x337b, PDF_CMAP_RANGE, 7824 },
	{ 0x337c, 0x337e, PDF_CMAP_RANGE, 1313 },
	{ 0x3421, 0x3422, PDF_CMAP_RANGE, 7826 },
	{ 0x3423, 0x3424, PDF_CMAP_RANGE, 1318 },
	{ 0x3425, 0x3426, PDF_CMAP_RANGE, 7828 },
	{ 0x3427, 0x342a, PDF_CMAP_RANGE, 1322 },
	{ 0x342b, 0x342b, PDF_CMAP_SINGLE, 7830 },
	{ 0x342c, 0x342e, PDF_CMAP_RANGE, 1327 },
	{ 0x342f, 0x342f, PDF_CMAP_SINGLE, 7831 },
	{ 0x3430, 0x3432, PDF_CMAP_RANGE, 1331 },
	{ 0x3433, 0x3434, PDF_CMAP_RANGE, 7832 },
	{ 0x3435, 0x3437, PDF_CMAP_RANGE, 1336 },
	{ 0x3438, 0x3438, PDF_CMAP_SINGLE, 7834 },
	{ 0x3439, 0x343e, PDF_CMAP_RANGE, 1340 },
	{ 0x343f, 0x343f, PDF_CMAP_SINGLE, 7835 },
	{ 0x3440, 0x3441, PDF_CMAP_RANGE, 1347 },
	{ 0x3442, 0x3442, PDF_CMAP_SINGLE, 7836 },
	{ 0x3443, 0x3446, PDF_CMAP_RANGE, 1350 },
	{ 0x3447, 0x3447, PDF_CMAP_SINGLE, 7837 },
	{ 0x3448, 0x3449, PDF_CMAP_RANGE, 1355 },
	{ 0x344a, 0x344a, PDF_CMAP_SINGLE, 7838 },
	{ 0x344b, 0x344c, PDF_CMAP_RANGE, 1358 },
	{ 0x344d, 0x344f, PDF_CMAP_TABLE, 52 },
	{ 0x3450, 0x3452, PDF_CMAP_RANGE, 1363 },
	{ 0x3453, 0x3454, PDF_CMAP_RANGE, 7841 },
	{ 0x3455, 0x3459, PDF_CMAP_RANGE, 1368 },
	{ 0x345a, 0x345c, PDF_CMAP_TABLE, 55 },
	{ 0x345d, 0x346c, PDF_CMAP_RANGE, 1376 },
	{ 0x346d, 0x346f, PDF_CMAP_TABLE, 58 },
	{ 0x3470, 0x3477, PDF_CMAP_RANGE, 1395 },
	{ 0x3478, 0x3478, PDF_CMAP_SINGLE, 7847 },
	{ 0x3479, 0x347a, PDF_CMAP_RANGE, 1404 },
	{ 0x347b, 0x347b, PDF_CMAP_SINGLE, 7848 },
	{ 0x347c, 0x347e, PDF_CMAP_RANGE, 1407 },
	{ 0x3521, 0x3522, PDF_CMAP_RANGE, 1410 },
	{ 0x3523, 0x3524, PDF_CMAP_TABLE, 61 },
	{ 0x3525, 0x3528, PDF_CMAP_RANGE, 7850 },
	{ 0x3529, 0x352b, PDF_CMAP_RANGE, 1418 },
	{ 0x352c, 0x352d, PDF_CMAP_TABLE, 63 },
	{ 0x352e, 0x352f, PDF_CMAP_RANGE, 7855 },
	{ 0x3530, 0x3530, PDF_CMAP_SINGLE, 1425 },
	{ 0x3531, 0x3535, PDF_CMAP_RANGE, 7857 },
	{ 0x3536, 0x3537, PDF_CMAP_TABLE, 65 },
	{ 0x3538, 0x3539, PDF_CMAP_RANGE, 1433 },
	{ 0x353a, 0x353c, PDF_CMAP_RANGE, 7863 },
	{ 0x353d, 0x3545, PDF_CMAP_RANGE, 1438 },
	{ 0x3546, 0x3546, PDF_CMAP_SINGLE, 7866 },
	{ 0x3547, 0x354a, PDF_CMAP_RANGE, 1448 },
	{ 0x354b, 0x354b, PDF_CMAP_SINGLE, 7867 },
	{ 0x354c, 0x354f, PDF_CMAP_RANGE, 1453 },
	{ 0x3550, 0x3550, PDF_CMAP_SINGLE, 7868 },
	{ 0x3551, 0x3552, PDF_CMAP_RANGE, 1458 },
	{ 0x3553, 0x3553, PDF_CMAP_SINGLE, 7869 },
	{ 0x3554, 0x355c, PDF_CMAP_RANGE, 1461 },
	{ 0x355d, 0x355f, PDF_CMAP_RANGE, 7870 },
	{ 0x3560, 0x3562, PDF_CMAP_RANGE, 1473 },
	{ 0x3563, 0x3563, PDF_CMAP_SINGLE, 7873 },
	{ 0x3564, 0x3565, PDF_CMAP_RANGE, 1477 },
	{ 0x3566, 0x3567, PDF_CMAP_RANGE, 7874 },
	{ 0x3568, 0x356c, PDF_CMAP_RANGE, 1481 },
	{ 0x356d, 0x356d, PDF_CMAP_SINGLE, 7876 },
	{ 0x356e, 0x3575, PDF_CMAP_RANGE, 1487 },
	{ 0x3576, 0x3577, PDF_CMAP_RANGE, 7877 },
	{ 0x3578, 0x357c, PDF_CMAP_RANGE, 1497 },
	{ 0x357d, 0x357e, PDF_CMAP_TABLE, 67 },
	{ 0x3621, 0x3623, PDF_CMAP_RANGE, 1504 },
	{ 0x3624, 0x3625, PDF_CMAP_RANGE, 7880 },
	{ 0x3626, 0x362b, PDF_CMAP_TABLE, 69 },
	{ 0x362c, 0x362e, PDF_CMAP_RANGE, 1515 },
	{ 0x362f, 0x3630, PDF_CMAP_RANGE, 7885 },
	{ 0x3631, 0x3632, PDF_CMAP_RANGE, 1520 },
	{ 0x3633, 0x3633, PDF_CMAP_SINGLE, 7887 },
	{ 0x3634, 0x3636, PDF_CMAP_RANGE, 1523 },
	{ 0x3637, 0x3637, PDF_CMAP_SINGLE, 7888 },
	{ 0x3638, 0x363e, PDF_CMAP_RANGE, 1527 },
	{ 0x363f, 0x3641, PDF_CMAP_RANGE, 7889 },
	{ 0x3642, 0x3643, PDF_CMAP_RANGE, 1537 },
	{ 0x3644, 0x3646, PDF_CMAP_TABLE, 75 },
	{ 0x3647, 0x364c, PDF_CMAP_RANGE, 1542 },
	{ 0x364d, 0x364e, PDF_CMAP_TABLE, 78 },
	{ 0x364f, 0x3650, PDF_CMAP_RANGE, 7895 },
	{ 0x3651, 0x3652, PDF_CMAP_RANGE, 1552 },
	{ 0x3653, 0x3654, PDF_CMAP_RANGE, 7897 },
	{ 0x3655, 0x3656, PDF_CMAP_TABLE, 80 },
	{ 0x3657, 0x3658, PDF_CMAP_RANGE, 1558 },
	{ 0x3659, 0x365b, PDF_CMAP_TABLE, 82 },
	{ 0x365c, 0x3660, PDF_CMAP_RANGE, 1563 },
	{ 0x3661, 0x3661, PDF_CMAP_SINGLE, 7902 },
	{ 0x3662, 0x3668, PDF_CMAP_RANGE, 1569 },
	{ 0x3669, 0x3669, PDF_CMAP_SINGLE, 7903 },
	{ 0x366a, 0x366b, PDF_CMAP_RANGE, 1577 },
	{ 0x366c, 0x366d, PDF_CMAP_TABLE, 85 },
	{ 0x366e, 0x366f, PDF_CMAP_RANGE, 7905 },
	{ 0x3670, 0x3671, PDF_CMAP_TABLE, 87 },
	{ 0x3672, 0x3675, PDF_CMAP_RANGE, 1585 },
	{ 0x3676, 0x3676, PDF_CMAP_SINGLE, 7908 },
	{ 0x3677, 0x3678, PDF_CMAP_RANGE, 1590 },
	{ 0x3679, 0x367a, PDF_CMAP_TABLE, 89 },
	{ 0x367b, 0x367c, PDF_CMAP_RANGE, 7910 },
	{ 0x367d, 0x367e, PDF_CMAP_RANGE, 1596 },
	{ 0x3721, 0x3723, PDF_CMAP_RANGE, 7912 },
	{ 0x3724, 0x3726, PDF_CMAP_RANGE, 1601 },
	{ 0x3727, 0x3727, PDF_CMAP_SINGLE, 7915 },
	{ 0x3728, 0x372e, PDF_CMAP_RANGE, 1605 },
	{ 0x372f, 0x3730, PDF_CMAP_RANGE, 7916 },
	{ 0x3731, 0x3732, PDF_CMAP_RANGE, 1614 },
	{ 0x3733, 0x3733, PDF_CMAP_SINGLE, 7918 },
	{ 0x3734, 0x3735, PDF_CMAP_RANGE, 1617 },
	{ 0x3736, 0x3737, PDF_CMAP_RANGE, 7919 },
	{ 0x3738, 0x3739, PDF_CMAP_TABLE, 91 },
	{ 0x373a, 0x3742, PDF_CMAP_RANGE, 1623 },
	{ 0x3743, 0x3744, PDF_CMAP_RANGE, 7922 },
	{ 0x3745, 0x3748, PDF_CMAP_RANGE, 1634 },
	{ 0x3749, 0x3749, PDF_CMAP_SINGLE, 7924 },
	{ 0x374a, 0x374b, PDF_CMAP_RANGE, 1639 },
	{ 0x374c, 0x374c, PDF_CMAP_SINGLE, 7925 },
	{ 0x374d, 0x374e, PDF_CMAP_RANGE, 1642 },
	{ 0x374f, 0x3751, PDF_CMAP_TABLE, 93 },
	{ 0x3752, 0x3756, PDF_CMAP_RANGE, 1647 },
	{ 0x3757, 0x3758, PDF_CMAP_RANGE, 7928 },
	{ 0x3759, 0x375b, PDF_CMAP_RANGE, 1654 },
	{ 0x375c, 0x375c, PDF_CMAP_SINGLE, 7930 },
	{ 0x375d, 0x375e, PDF_CMAP_RANGE, 1658 },
	{ 0x375f, 0x3761, PDF_CMAP_RANGE, 7931 },
	{ 0x3762, 0x3763, PDF_CMAP_TABLE, 96 },
	{ 0x3764, 0x3765, PDF_CMAP_RANGE, 1665 },
	{ 0x3766, 0x3768, PDF_CMAP_RANGE, 7935 },
	{ 0x3769, 0x376a, PDF_CMAP_RANGE, 1670 },
	{ 0x376b, 0x376d, PDF_CMAP_RANGE, 7938 },
	{ 0x376e, 0x376f, PDF_CMAP_TABLE, 98 },
	{ 0x3770, 0x3773, PDF_CMAP_RANGE, 1677 },
	{ 0x3774, 0x3774, PDF_CMAP_SINGLE, 7942 },
	{ 0x3775, 0x3777, PDF_CMAP_RANGE, 1682 },
	{ 0x3778, 0x3778, PDF_CMAP_SINGLE, 7943 },
	{ 0x3779, 0x377e, PDF_CMAP_RANGE, 1686 },
	{ 0x3821, 0x3826, PDF_CMAP_RANGE, 1692 },
	{ 0x3827, 0x3828, PDF_CMAP_RANGE, 7944 },
	{ 0x3829, 0x3832, PDF_CMAP_RANGE, 1700 },
	{ 0x3833, 0x3834, PDF_CMAP_RANGE, 7946 },
	{ 0x3835, 0x3839, PDF_CMAP_RANGE, 1712 },
	{ 0x383a, 0x383d, PDF_CMAP_TABLE, 100 },
	{ 0x383e, 0x383f, PDF_CMAP_RANGE, 7950 },
	{ 0x3840, 0x3842, PDF_CMAP_RANGE, 1723 },
	{ 0x3843, 0x3843, PDF_CMAP_SINGLE, 7952 },
	{ 0x3844, 0x3845, PDF_CMAP_RANGE, 1727 },
	{ 0x3846, 0x3847, PDF_CMAP_RANGE, 7953 },
	{ 0x3848, 0x3849, PDF_CMAP_TABLE, 104 },
	{ 0x384a, 0x384e, PDF_CMAP_RANGE, 1733 },
	{ 0x384f, 0x384f, PDF_CMAP_SINGLE, 7956 },
	{ 0x3850, 0x3852, PDF_CMAP_RANGE, 1739 },
	{ 0x3853, 0x3856, PDF_CMAP_RANGE, 7957 },
	{ 0x3857, 0x3858, PDF_CMAP_RANGE, 1746 },
	{ 0x3859, 0x385a, PDF_CMAP_RANGE, 7961 },
	{ 0x385b, 0x3863, PDF_CMAP_RANGE, 1750 },
	{ 0x3864, 0x3864, PDF_CMAP_SINGLE, 7963 },
	{ 0x3865, 0x3868, PDF_CMAP_RANGE, 1760 },
	{ 0x3869, 0x386b, PDF_CMAP_TABLE, 106 },
	{ 0x386c, 0x3872, PDF_CMAP_RANGE, 1767 },
	{ 0x3873, 0x3874, PDF_CMAP_TABLE, 109 },
	{ 0x3875, 0x3876, PDF_CMAP_RANGE, 7967 },
	{ 0x3877, 0x3878, PDF_CMAP_TABLE, 111 },
	{ 0x3879, 0x387e, PDF_CMAP_RANGE, 1780 },
	{ 0x3921, 0x3927, PDF_CMAP_RANGE, 1786 },
	{ 0x3928, 0x3928, PDF_CMAP_SINGLE, 7970 },
	{ 0x3929, 0x392d, PDF_CMAP_RANGE, 1794 },
	{ 0x392e, 0x392e, PDF_CMAP_SINGLE, 7971 },
	{ 0x392f, 0x3930, PDF_CMAP_RANGE, 1800 },
	{ 0x3931, 0x3935, PDF_CMAP_TABLE, 113 },
	{ 0x3936, 0x3938, PDF_CMAP_RANGE, 1807 },
	{ 0x3939, 0x393a, PDF_CMAP_RANGE, 7975 },
	{ 0x393b, 0x3945, PDF_CMAP_RANGE, 1812 },
	{ 0x3946, 0x3946, PDF_CMAP_SINGLE, 7977 },
	{ 0x3947, 0x394a, PDF_CMAP_RANGE, 1824 },
	{ 0x394b, 0x394b, PDF_CMAP_SINGLE, 7978 },
	{ 0x394c, 0x394f, PDF_CMAP_RANGE, 1829 },
	{ 0x3950, 0x3950, PDF_CMAP_SINGLE, 7979 },
	{ 0x3951, 0x3957, PDF_CMAP_RANGE, 1834 },
	{ 0x3958, 0x3958, PDF_CMAP_SINGLE, 7980 },
	{ 0x3959, 0x395a, PDF_CMAP_RANGE, 1842 },
	{ 0x395b, 0x3963, PDF_CMAP_TABLE, 118 },
	{ 0x3964, 0x3965, PDF_CMAP_RANGE, 1853 },
	{ 0x3966, 0x3966, PDF_CMAP_SINGLE, 7986 },
	{ 0x3967, 0x3968, PDF_CMAP_RANGE, 1856 },
	{ 0x3969, 0x396c, PDF_CMAP_RANGE, 7987 },
	{ 0x396d, 0x396e, PDF_CMAP_TABLE, 127 },
	{ 0x396f, 0x3970, PDF_CMAP_RANGE, 1864 },
	{ 0x3971, 0x3972, PDF_CMAP_TABLE, 129 },
	{ 0x3973, 0x3975, PDF_CMAP_RANGE, 7993 },
	{ 0x3976, 0x3977, PDF_CMAP_RANGE, 1871 },
	{ 0x3978, 0x397a, PDF_CMAP_TABLE, 131 },
	{ 0x397b, 0x397c, PDF_CMAP_RANGE, 1876 },
	{ 0x397d, 0x397e, PDF_CMAP_TABLE, 134 },
	{ 0x3a21, 0x3a26, PDF_CMAP_RANGE, 1880 },
	{ 0x3a27, 0x3a27, PDF_CMAP_SINGLE, 7999 },
	{ 0x3a28, 0x3a2a, PDF_CMAP_RANGE, 1887 },
	{ 0x3a2b, 0x3a2b, PDF_CMAP_SINGLE, 8000 },
	{ 0x3a2c, 0x3a39, PDF_CMAP_RANGE, 1891 },
	{ 0x3a3a, 0x3a3a, PDF_CMAP_SINGLE, 8001 },
	{ 0x3a3b, 0x3a44, PDF_CMAP_RANGE, 1906 },
	{ 0x3a45, 0x3a45, PDF_CMAP_SINGLE, 8002 },
	{ 0x3a46, 0x3a51, PDF_CMAP_RANGE, 1917 },
	{ 0x3a52, 0x3a52, PDF_CMAP_SINGLE, 8003 },
	{ 0x3a53, 0x3a56, PDF_CMAP_RANGE, 1930 },
	{ 0x3a57, 0x3a58, PDF_CMAP_RANGE, 8004 },
	{ 0x3a59, 0x3a63, PDF_CMAP_RANGE, 1936 },
	{ 0x3a64, 0x3a64, PDF_CMAP_SINGLE, 8006 },
	{ 0x3a65, 0x3a67, PDF_CMAP_RANGE, 1948 },
	{ 0x3a68, 0x3a68, PDF_CMAP_SINGLE, 8007 },
	{ 0x3a69, 0x3a6b, PDF_CMAP_RANGE, 1952 },
	{ 0x3a6c, 0x3a6c, PDF_CMAP_SINGLE, 8008 },
	{ 0x3a6d, 0x3a72, PDF_CMAP_RANGE, 1956 },
	{ 0x3a73, 0x3a73, PDF_CMAP_SINGLE, 5605 },
	{ 0x3a74, 0x3a77, PDF_CMAP_RANGE, 1963 },
	{ 0x3a78, 0x3a78, PDF_CMAP_SINGLE, 8009 },
	{ 0x3a79, 0x3a7e, PDF_CMAP_RANGE, 1968 },
	{ 0x3b21, 0x3b23, PDF_CMAP_RANGE, 1974 },
	{ 0x3b24, 0x3b26, PDF_CMAP_TABLE, 136 },
	{ 0x3b27, 0x3b28, PDF_CMAP_RANGE, 1980 },
	{ 0x3b29, 0x3b2a, PDF_CMAP_RANGE, 8012 },
	{ 0x3b2b, 0x3b2c, PDF_CMAP_RANGE, 1984 },
	{ 0x3b2d, 0x3b2e, PDF_CMAP_RANGE, 8014 },
	{ 0x3b2f, 0x3b30, PDF_CMAP_TABLE, 139 },
	{ 0x3b31, 0x3b32, PDF_CMAP_RANGE, 1990 },
	{ 0x3b33, 0x3b34, PDF_CMAP_TABLE, 141 },
	{ 0x3b35, 0x3b37, PDF_CMAP_RANGE, 8018 },
	{ 0x3b38, 0x3b38, PDF_CMAP_SINGLE, 1997 },
	{ 0x3b39, 0x3b3a, PDF_CMAP_RANGE, 8021 },
	{ 0x3b3b, 0x3b50, PDF_CMAP_RANGE, 2000 },
	{ 0x3b51, 0x3b52, PDF_CMAP_TABLE, 143 },
	{ 0x3b53, 0x3b54, PDF_CMAP_RANGE, 8024 },
	{ 0x3b55, 0x3b5e, PDF_CMAP_RANGE, 2026 },
	{ 0x3b5f, 0x3b67, PDF_CMAP_RANGE, 8026 },
	{ 0x3b68, 0x3b6a, PDF_CMAP_RANGE, 2045 },
	{ 0x3b6b, 0x3b6b, PDF_CMAP_SINGLE, 8035 },
	{ 0x3b6c, 0x3b70, PDF_CMAP_RANGE, 2049 },
	{ 0x3b71, 0x3b71, PDF_CMAP_SINGLE, 8036 },
	{ 0x3b72, 0x3b74, PDF_CMAP_RANGE, 2055 },
	{ 0x3b75, 0x3b77, PDF_CMAP_RANGE, 8037 },
	{ 0x3b78, 0x3b79, PDF_CMAP_RANGE, 2061 },
	{ 0x3b7a, 0x3b7a, PDF_CMAP_SINGLE, 8040 },
	{ 0x3b7b, 0x3b7c, PDF_CMAP_RANGE, 2064 },
	{ 0x3b7d, 0x3b7e, PDF_CMAP_TABLE, 145 },
	{ 0x3c21, 0x3c22, PDF_CMAP_TABLE, 147 },
	{ 0x3c23, 0x3c24, PDF_CMAP_RANGE, 2070 },
	{ 0x3c25, 0x3c26, PDF_CMAP_RANGE, 8043 },
	{ 0x3c27, 0x3c27, PDF_CMAP_SINGLE, 2074 },
	{ 0x3c28, 0x3c29, PDF_CMAP_RANGE, 8045 },
	{ 0x3c2a, 0x3c2d, PDF_CMAP_TABLE, 149 },
	{ 0x3c2e, 0x3c35, PDF_CMAP_RANGE, 2081 },
	{ 0x3c36, 0x3c38, PDF_CMAP_RANGE, 8049 },
	{ 0x3c39, 0x3c3a, PDF_CMAP_RANGE, 2092 },
	{ 0x3c3b, 0x3c3b, PDF_CMAP_SINGLE, 8052 },
	{ 0x3c3c, 0x3c40, PDF_CMAP_RANGE, 2095 },
	{ 0x3c41, 0x3c43, PDF_CMAP_TABLE, 153 },
	{ 0x3c44, 0x3c45, PDF_CMAP_RANGE, 2103 },
	{ 0x3c46, 0x3c47, PDF_CMAP_RANGE, 8055 },
	{ 0x3c48, 0x3c49, PDF_CMAP_RANGE, 2107 },
	{ 0x3c4a, 0x3c4b, PDF_CMAP_TABLE, 156 },
	{ 0x3c4c, 0x3c4d, PDF_CMAP_RANGE, 8058 },
	{ 0x3c4e, 0x3c4f, PDF_CMAP_RANGE, 2113 },
	{ 0x3c50, 0x3c50, PDF_CMAP_SINGLE, 8060 },
	{ 0x3c51, 0x3c53, PDF_CMAP_RANGE, 2116 },
	{ 0x3c54, 0x3c56, PDF_CMAP_RANGE, 8061 },
	{ 0x3c57, 0x3c58, PDF_CMAP_TABLE, 158 },
	{ 0x3c59, 0x3c5a, PDF_CMAP_RANGE, 2124 },
	{ 0x3c5b, 0x3c5e, PDF_CMAP_TABLE, 160 },
	{ 0x3c5f, 0x3c61, PDF_CMAP_RANGE, 8067 },
	{ 0x3c62, 0x3c62, PDF_CMAP_SINGLE, 2133 },
	{ 0x3c63, 0x3c64, PDF_CMAP_RANGE, 8070 },
	{ 0x3c65, 0x3c67, PDF_CMAP_RANGE, 2136 },
	{ 0x3c68, 0x3c69, PDF_CMAP_TABLE, 164 },
	{ 0x3c6a, 0x3c6c, PDF_CMAP_RANGE, 8073 },
	{ 0x3c6d, 0x3c6e, PDF_CMAP_RANGE, 2144 },
	{ 0x3c6f, 0x3c73, PDF_CMAP_RANGE, 8076 },
	{ 0x3c74, 0x3c75, PDF_CMAP_RANGE, 2151 },
	{ 0x3c76, 0x3c7c, PDF_CMAP_RANGE, 8081 },
	{ 0x3c7d, 0x3c7e, PDF_CMAP_RANGE, 2160 },
	{ 0x3d21, 0x3d21, PDF_CMAP_SINGLE, 2162 },
	{ 0x3d22, 0x3d27, PDF_CMAP_RANGE, 8088 },
	{ 0x3d28, 0x3d2a, PDF_CMAP_RANGE, 2169 },
	{ 0x3d2b, 0x3d2c, PDF_CMAP_RANGE, 8094 },
	{ 0x3d2d, 0x3d2e, PDF_CMAP_RANGE, 2174 },
	{ 0x3d2f, 0x3d32, PDF_CMAP_RANGE, 8096 },
	{ 0x3d33, 0x3d34, PDF_CMAP_TABLE, 166 },
	{ 0x3d35, 0x3d39, PDF_CMAP_RANGE, 2182 },
	{ 0x3d3a, 0x3d3a, PDF_CMAP_SINGLE, 8101 },
	{ 0x3d3b, 0x3d3c, PDF_CMAP_RANGE, 2188 },
	{ 0x3d3d, 0x3d3f, PDF_CMAP_RANGE, 8102 },
	{ 0x3d40, 0x3d40, PDF_CMAP_SINGLE, 2193 },
	{ 0x3d41, 0x3d44, PDF_CMAP_RANGE, 8105 },
	{ 0x3d45, 0x3d47, PDF_CMAP_RANGE, 2198 },
	{ 0x3d48, 0x3d4a, PDF_CMAP_RANGE, 8109 },
	{ 0x3d4b, 0x3d4d, PDF_CMAP_RANGE, 2204 },
	{ 0x3d4e, 0x3d4f, PDF_CMAP_RANGE, 8112 },
	{ 0x3d50, 0x3d56, PDF_CMAP_RANGE, 2209 },
	{ 0x3d57, 0x3d57, PDF_CMAP_SINGLE, 8114 },
	{ 0x3d58, 0x3d59, PDF_CMAP_RANGE, 2217 },
	{ 0x3d5a, 0x3d5a, PDF_CMAP_SINGLE, 8115 },
	{ 0x3d5b, 0x3d5f, PDF_CMAP_RANGE, 2220 },
	{ 0x3d60, 0x3d61, PDF_CMAP_RANGE, 8116 },
	{ 0x3d62, 0x3d6a, PDF_CMAP_RANGE, 2227 },
	{ 0x3d6b, 0x3d6b, PDF_CMAP_SINGLE, 8118 },
	{ 0x3d6c, 0x3d73, PDF_CMAP_RANGE, 2237 },
	{ 0x3d74, 0x3d78, PDF_CMAP_RANGE, 8119 },
	{ 0x3d79, 0x3d7c, PDF_CMAP_RANGE, 2250 },
	{ 0x3d7d, 0x3d7e, PDF_CMAP_TABLE, 168 },
	{ 0x3e21, 0x3e22, PDF_CMAP_RANGE, 8125 },
	{ 0x3e23, 0x3e24, PDF_CMAP_RANGE, 2258 },
	{ 0x3e25, 0x3e25, PDF_CMAP_SINGLE, 8127 },
	{ 0x3e26, 0x3e27, PDF_CMAP_RANGE, 2261 },
	{ 0x3e28, 0x3e2a, PDF_CMAP_TABLE, 170 },
	{ 0x3e2b, 0x3e2c, PDF_CMAP_RANGE, 2266 },
	{ 0x3e2d, 0x3e2d, PDF_CMAP_SINGLE, 8130 },
	{ 0x3e2e, 0x3e30, PDF_CMAP_RANGE, 2269 },
	{ 0x3e31, 0x3e31, PDF_CMAP_SINGLE, 8131 },
	{ 0x3e32, 0x3e34, PDF_CMAP_RANGE, 2273 },
	{ 0x3e35, 0x3e37, PDF_CMAP_RANGE, 8132 },
	{ 0x3e38, 0x3e39, PDF_CMAP_RANGE, 2279 },
	{ 0x3e3a, 0x3e3a, PDF_CMAP_SINGLE, 8135 },
	{ 0x3e3b, 0x3e3f, PDF_CMAP_RANGE, 2282 },
	{ 0x3e40, 0x3e40, PDF_CMAP_SINGLE, 8136 },
	{ 0x3e41, 0x3e48, PDF_CMAP_RANGE, 2288 },
	{ 0x3e49, 0x3e49, PDF_CMAP_SINGLE, 8137 },
	{ 0x3e4a, 0x3e53, PDF_CMAP_RANGE, 2297 },
	{ 0x3e54, 0x3e54, PDF_CMAP_SINGLE, 8138 },
	{ 0x3e55, 0x3e58, PDF_CMAP_RANGE, 2308 },
	{ 0x3e59, 0x3e59, PDF_CMAP_SINGLE, 8139 },
	{ 0x3e5a, 0x3e5c, PDF_CMAP_RANGE, 2313 },
	{ 0x3e5d, 0x3e5d, PDF_CMAP_SINGLE, 8140 },
	{ 0x3e5e, 0x3e61, PDF_CMAP_RANGE, 2317 },
	{ 0x3e62, 0x3e62, PDF_CMAP_SINGLE, 8141 },
	{ 0x3e63, 0x3e64, PDF_CMAP_RANGE, 2322 },
	{ 0x3e65, 0x3e69, PDF_CMAP_TABLE, 173 },
	{ 0x3e6a, 0x3e6d, PDF_CMAP_RANGE, 2329 },
	{ 0x3e6e, 0x3e6e, PDF_CMAP_SINGLE, 8145 },
	{ 0x3e6f, 0x3e74, PDF_CMAP_RANGE, 2334 },
	{ 0x3e75, 0x3e76, PDF_CMAP_TABLE, 178 },
	{ 0x3e77, 0x3e78, PDF_CMAP_RANGE, 8147 },
	{ 0x3e79, 0x3e7a, PDF_CMAP_RANGE, 2344 },
	{ 0x3e7b, 0x3e7c, PDF_CMAP_RANGE, 8149 },
	{ 0x3e7d, 0x3e7e, PDF_CMAP_RANGE, 2348 },
	{ 0x3f21, 0x3f24, PDF_CMAP_RANGE, 2350 },
	{ 0x3f25, 0x3f25, PDF_CMAP_SINGLE, 8151 },
	{ 0x3f26, 0x3f29, PDF_CMAP_RANGE, 2355 },
	{ 0x3f2a, 0x3f2a, PDF_CMAP_SINGLE, 8152 },
	{ 0x3f2b, 0x3f2c, PDF_CMAP_RANGE, 2360 },
	{ 0x3f2d, 0x3f2d, PDF_CMAP_SINGLE, 8153 },
	{ 0x3f2e, 0x3f44, PDF_CMAP_RANGE, 2363 },
	{ 0x3f45, 0x3f47, PDF_CMAP_TABLE, 180 },
	{ 0x3f48, 0x3f4d, PDF_CMAP_RANGE, 2389 },
	{ 0x3f4e, 0x3f4e, PDF_CMAP_SINGLE, 8156 },
	{ 0x3f4f, 0x3f50, PDF_CMAP_RANGE, 2396 },
	{ 0x3f51, 0x3f52, PDF_CMAP_RANGE, 8157 },
	{ 0x3f53, 0x3f58, PDF_CMAP_RANGE, 2400 },
	{ 0x3f59, 0x3f59, PDF_CMAP_SINGLE, 8159 },
	{ 0x3f5a, 0x3f61, PDF_CMAP_RANGE, 2407 },
	{ 0x3f62, 0x3f64, PDF_CMAP_RANGE, 8160 },
	{ 0x3f65, 0x3f68, PDF_CMAP_RANGE, 2418 },
	{ 0x3f69, 0x3f6d, PDF_CMAP_TABLE, 183 },
	{ 0x3f6e, 0x3f72, PDF_CMAP_RANGE, 2427 },
	{ 0x3f73, 0x3f78, PDF_CMAP_TABLE, 188 },
	{ 0x3f79, 0x3f7a, PDF_CMAP_RANGE, 8169 },
	{ 0x3f7b, 0x3f7e, PDF_CMAP_RANGE, 2440 },
	{ 0x4021, 0x4023, PDF_CMAP_TABLE, 194 },
	{ 0x4024, 0x4028, PDF_CMAP_RANGE, 2447 },
	{ 0x4029, 0x402b, PDF_CMAP_TABLE, 197 },
	{ 0x402c, 0x402e, PDF_CMAP_RANGE, 2455 },
	{ 0x402f, 0x4030, PDF_CMAP_RANGE, 8175 },
	{ 0x4031, 0x4032, PDF_CMAP_RANGE, 2460 },
	{ 0x4033, 0x4036, PDF_CMAP_RANGE, 8177 },
	{ 0x4037, 0x4037, PDF_CMAP_SINGLE, 2466 },
	{ 0x4038, 0x4044, PDF_CMAP_RANGE, 8181 },
	{ 0x4045, 0x404b, PDF_CMAP_RANGE, 2480 },
	{ 0x404c, 0x404d, PDF_CMAP_RANGE, 8194 },
	{ 0x404e, 0x4053, PDF_CMAP_RANGE, 2489 },
	{ 0x4054, 0x4058, PDF_CMAP_TABLE, 200 },
	{ 0x4059, 0x405c, PDF_CMAP_RANGE, 2500 },
	{ 0x405d, 0x405d, PDF_CMAP_SINGLE, 8199 },
	{ 0x405e, 0x405f, PDF_CMAP_RANGE, 2505 },
	{ 0x4060, 0x4060, PDF_CMAP_SINGLE, 8200 },
	{ 0x4061, 0x4068, PDF_CMAP_RANGE, 2508 },
	{ 0x4069, 0x406a, PDF_CMAP_TABLE, 205 },
	{ 0x406b, 0x406c, PDF_CMAP_RANGE, 8202 },
	{ 0x406d, 0x406e, PDF_CMAP_RANGE, 2520 },
	{ 0x406f, 0x4071, PDF_CMAP_RANGE, 8204 },
	{ 0x4072, 0x4075, PDF_CMAP_RANGE, 2525 },
	{ 0x4076, 0x407a, PDF_CMAP_RANGE, 8207 },
	{ 0x407b, 0x407e, PDF_CMAP_RANGE, 2534 },
	{ 0x4121, 0x4123, PDF_CMAP_RANGE, 2538 },
	{ 0x4124, 0x4125, PDF_CMAP_RANGE, 8212 },
	{ 0x4126, 0x4128, PDF_CMAP_RANGE, 2543 },
	{ 0x4129, 0x412d, PDF_CMAP_RANGE, 8214 },
	{ 0x412e, 0x412e, PDF_CMAP_SINGLE, 2551 },
	{ 0x412f, 0x4138, PDF_CMAP_RANGE, 8219 },
	{ 0x4139, 0x413c, PDF_CMAP_RANGE, 2562 },
	{ 0x413d, 0x413e, PDF_CMAP_RANGE, 8229 },
	{ 0x413f, 0x4141, PDF_CMAP_RANGE, 2568 },
	{ 0x4142, 0x4142, PDF_CMAP_SINGLE, 8231 },
	{ 0x4143, 0x4145, PDF_CMAP_RANGE, 2572 },
	{ 0x4146, 0x4146, PDF_CMAP_SINGLE, 8232 },
	{ 0x4147, 0x4148, PDF_CMAP_RANGE, 2576 },
	{ 0x4149, 0x4149, PDF_CMAP_SINGLE, 8233 },
	{ 0x414a, 0x414c, PDF_CMAP_RANGE, 2579 },
	{ 0x414d, 0x414d, PDF_CMAP_SINGLE, 8234 },
	{ 0x414e, 0x4153, PDF_CMAP_RANGE, 2583 },
	{ 0x4154, 0x4154, PDF_CMAP_SINGLE, 8235 },
	{ 0x4155, 0x4158, PDF_CMAP_RANGE, 2590 },
	{ 0x4159, 0x415b, PDF_CMAP_RANGE, 8236 },
	{ 0x415c, 0x415d, PDF_CMAP_RANGE, 2597 },
	{ 0x415e, 0x415e, PDF_CMAP_SINGLE, 8239 },
	{ 0x415f, 0x4163, PDF_CMAP_RANGE, 2600 },
	{ 0x4164, 0x4165, PDF_CMAP_RANGE, 8240 },
	{ 0x4166, 0x4168, PDF_CMAP_RANGE, 2607 },
	{ 0x4169, 0x416a, PDF_CMAP_TABLE, 207 },
	{ 0x416b, 0x416c, PDF_CMAP_RANGE, 8243 },
	{ 0x416d, 0x4172, PDF_CMAP_RANGE, 2614 },
	{ 0x4173, 0x4175, PDF_CMAP_TABLE, 209 },
	{ 0x4176, 0x4179, PDF_CMAP_RANGE, 2623 },
	{ 0x417a, 0x417d, PDF_CMAP_RANGE, 8247 },
	{ 0x417e, 0x417e, PDF_CMAP_SINGLE, 2631 },
	{ 0x4221, 0x4221, PDF_CMAP_SINGLE, 2632 },
	{ 0x4222, 0x4228, PDF_CMAP_RANGE, 8251 },
	{ 0x4229, 0x422a, PDF_CMAP_RANGE, 2640 },
	{ 0x422b, 0x4233, PDF_CMAP_RANGE, 8258 },
	{ 0x4234, 0x4237, PDF_CMAP_RANGE, 2651 },
	{ 0x4238, 0x4238, PDF_CMAP_SINGLE, 8267 },
	{ 0x4239, 0x423b, PDF_CMAP_RANGE, 2656 },
	{ 0x423c, 0x423d, PDF_CMAP_RANGE, 8268 },
	{ 0x423e, 0x4241, PDF_CMAP_TABLE, 212 },
	{ 0x4242, 0x4244, PDF_CMAP_RANGE, 2665 },
	{ 0x4245, 0x4247, PDF_CMAP_RANGE, 8272 },
	{ 0x4248, 0x424a, PDF_CMAP_RANGE, 2671 },
	{ 0x424b, 0x4250, PDF_CMAP_RANGE, 8275 },
	{ 0x4251, 0x4252, PDF_CMAP_TABLE, 216 },
	{ 0x4253, 0x4254, PDF_CMAP_RANGE, 2682 },
	{ 0x4255, 0x425c, PDF_CMAP_RANGE, 8282 },
	{ 0x425d, 0x425d, PDF_CMAP_SINGLE, 2692 },
	{ 0x425e, 0x4262, PDF_CMAP_RANGE, 8290 },
	{ 0x4263, 0x4265, PDF_CMAP_RANGE, 2698 },
	{ 0x4266, 0x4268, PDF_CMAP_RANGE, 8295 },
	{ 0x4269, 0x4269, PDF_CMAP_SINGLE, 2704 },
	{ 0x426a, 0x426e, PDF_CMAP_RANGE, 8298 },
	{ 0x426f, 0x4271, PDF_CMAP_TABLE, 218 },
	{ 0x4272, 0x4275, PDF_CMAP_RANGE, 8304 },
	{ 0x4276, 0x4276, PDF_CMAP_SINGLE, 2717 },
	{ 0x4277, 0x427a, PDF_CMAP_RANGE, 8308 },
	{ 0x427b, 0x427e, PDF_CMAP_RANGE, 2722 },
	{ 0x4321, 0x4321, PDF_CMAP_SINGLE, 8312 },
	{ 0x4322, 0x4329, PDF_CMAP_RANGE, 2727 },
	{ 0x432a, 0x432a, PDF_CMAP_SINGLE, 8313 },
	{ 0x432b, 0x432c, PDF_CMAP_RANGE, 2736 },
	{ 0x432d, 0x432d, PDF_CMAP_SINGLE, 8314 },
	{ 0x432e, 0x4332, PDF_CMAP_RANGE, 2739 },
	{ 0x4333, 0x4334, PDF_CMAP_TABLE, 221 },
	{ 0x4335, 0x433d, PDF_CMAP_RANGE, 2746 },
	{ 0x433e, 0x433e, PDF_CMAP_SINGLE, 8316 },
	{ 0x433f, 0x4344, PDF_CMAP_RANGE, 2756 },
	{ 0x4345, 0x4347, PDF_CMAP_RANGE, 8317 },
	{ 0x4348, 0x434b, PDF_CMAP_RANGE, 2765 },
	{ 0x434c, 0x434e, PDF_CMAP_TABLE, 223 },
	{ 0x434f, 0x4354, PDF_CMAP_RANGE, 2772 },
	{ 0x4355, 0x4356, PDF_CMAP_RANGE, 8322 },
	{ 0x4357, 0x4358, PDF_CMAP_RANGE, 2780 },
	{ 0x4359, 0x4359, PDF_CMAP_SINGLE, 8324 },
	{ 0x435a, 0x435f, PDF_CMAP_RANGE, 2783 },
	{ 0x4360, 0x4360, PDF_CMAP_SINGLE, 8325 },
	{ 0x4361, 0x4364, PDF_CMAP_RANGE, 2790 },
	{ 0x4365, 0x4365, PDF_CMAP_SINGLE, 8326 },
	{ 0x4366, 0x436c, PDF_CMAP_RANGE, 2795 },
	{ 0x436d, 0x436d, PDF_CMAP_SINGLE, 8327 },
	{ 0x436e, 0x436f, PDF_CMAP_RANGE, 2803 },
	{ 0x4370, 0x4370, PDF_CMAP_SINGLE, 8328 },
	{ 0x4371, 0x4374, PDF_CMAP_RANGE, 2806 },
	{ 0x4375, 0x4376, PDF_CMAP_RANGE, 8329 },
	{ 0x4377, 0x4378, PDF_CMAP_RANGE, 2812 },
	{ 0x4379, 0x437a, PDF_CMAP_RANGE, 8331 },
	{ 0x437b, 0x437c, PDF_CMAP_RANGE, 2816 },
	{ 0x437d, 0x437e, PDF_CMAP_TABLE, 226 },
	{ 0x4421, 0x4430, PDF_CMAP_RANGE, 2820 },
	{ 0x4431, 0x4431, PDF_CMAP_SINGLE, 8334 },
	{ 0x4432, 0x4435, PDF_CMAP_RANGE, 2837 },
	{ 0x4436, 0x4436, PDF_CMAP_SINGLE, 8335 },
	{ 0x4437, 0x4445, PDF_CMAP_RANGE, 2842 },
	{ 0x4446, 0x4446, PDF_CMAP_SINGLE, 8336 },
	{ 0x4447, 0x4448, PDF_CMAP_RANGE, 2858 },
	{ 0x4449, 0x4449, PDF_CMAP_SINGLE, 8337 },
	{ 0x444a, 0x4450, PDF_CMAP_RANGE, 2861 },
	{ 0x4451, 0x4452, PDF_CMAP_TABLE, 228 },
	{ 0x4453, 0x4456, PDF_CMAP_RANGE, 8339 },
	{ 0x4457, 0x4458, PDF_CMAP_RANGE, 2874 },
	{ 0x4459, 0x4459, PDF_CMAP_SINGLE, 8343 },
	{ 0x445a, 0x4461, PDF_CMAP_RANGE, 2877 },
	{ 0x4462, 0x4462, PDF_CMAP_SINGLE, 8344 },
	{ 0x4463, 0x4464, PDF_CMAP_RANGE, 2886 },
	{ 0x4465, 0x4465, PDF_CMAP_SINGLE, 8345 },
	{ 0x4466, 0x446b, PDF_CMAP_RANGE, 2889 },
	{ 0x446c, 0x446c, PDF_CMAP_SINGLE, 8346 },
	{ 0x446d, 0x446f, PDF_CMAP_RANGE, 2896 },
	{ 0x4470, 0x4471, PDF_CMAP_RANGE, 8347 },
	{ 0x4472, 0x4473, PDF_CMAP_RANGE, 2901 },
	{ 0x4474, 0x4475, PDF_CMAP_TABLE, 230 },
	{ 0x4476, 0x4478, PDF_CMAP_RANGE, 8350 },
	{ 0x4479, 0x447a, PDF_CMAP_RANGE, 2908 },
	{ 0x447b, 0x447c, PDF_CMAP_RANGE, 8353 },
	{ 0x447d, 0x447e, PDF_CMAP_TABLE, 232 },
	{ 0x4521, 0x4522, PDF_CMAP_RANGE, 8356 },
	{ 0x4523, 0x4524, PDF_CMAP_RANGE, 2916 },
	{ 0x4525, 0x4529, PDF_CMAP_RANGE, 8358 },
	{ 0x452a, 0x4530, PDF_CMAP_RANGE, 2923 },
	{ 0x4531, 0x4531, PDF_CMAP_SINGLE, 8363 },
	{ 0x4532, 0x4534, PDF_CMAP_RANGE, 2931 },
	{ 0x4535, 0x4536, PDF_CMAP_TABLE, 234 },
	{ 0x4537, 0x4539, PDF_CMAP_RANGE, 8365 },
	{ 0x453a, 0x453d, PDF_CMAP_TABLE, 236 },
	{ 0x453e, 0x454b, PDF_CMAP_RANGE, 2943 },
	{ 0x454c, 0x454c, PDF_CMAP_SINGLE, 8370 },
	{ 0x454d, 0x4552, PDF_CMAP_RANGE, 2958 },
	{ 0x4553, 0x4553, PDF_CMAP_SINGLE, 8371 },
	{ 0x4554, 0x4561, PDF_CMAP_RANGE, 2965 },
	{ 0x4562, 0x4562, PDF_CMAP_SINGLE, 8372 },
	{ 0x4563, 0x4566, PDF_CMAP_RANGE, 2980 },
	{ 0x4567, 0x4567, PDF_CMAP_SINGLE, 8373 },
	{ 0x4568, 0x4573, PDF_CMAP_RANGE, 2985 },
	{ 0x4574, 0x4574, PDF_CMAP_SINGLE, 8374 },
	{ 0x4575, 0x457e, PDF_CMAP_RANGE, 2998 },
	{ 0x4621, 0x462c, PDF_CMAP_RANGE, 3008 },
	{ 0x462d, 0x462e, PDF_CMAP_RANGE, 8375 },
	{ 0x462f, 0x4634, PDF_CMAP_RANGE, 3022 },
	{ 0x4635, 0x4636, PDF_CMAP_RANGE, 8377 },
	{ 0x4637, 0x463a, PDF_CMAP_RANGE, 3030 },
	{ 0x463b, 0x463b, PDF_CMAP_SINGLE, 8379 },
	{ 0x463c, 0x463d, PDF_CMAP_RANGE, 3035 },
	{ 0x463e, 0x4640, PDF_CMAP_TABLE, 240 },
	{ 0x4641, 0x4642, PDF_CMAP_RANGE, 3040 },
	{ 0x4643, 0x4644, PDF_CMAP_RANGE, 8382 },
	{ 0x4645, 0x464a, PDF_CMAP_RANGE, 3044 },
	{ 0x464b, 0x464d, PDF_CMAP_RANGE, 8384 },
	{ 0x464e, 0x4652, PDF_CMAP_RANGE, 3053 },
	{ 0x4653, 0x4653, PDF_CMAP_SINGLE, 8387 },
	{ 0x4654, 0x4656, PDF_CMAP_RANGE, 3059 },
	{ 0x4657, 0x4657, PDF_CMAP_SINGLE, 8388 },
	{ 0x4658, 0x4669, PDF_CMAP_RANGE, 3063 },
	{ 0x466a, 0x466b, PDF_CMAP_RANGE, 8389 },
	{ 0x466c, 0x466e, PDF_CMAP_RANGE, 3083 },
	{ 0x466f, 0x4671, PDF_CMAP_TABLE, 243 },
	{ 0x4672, 0x4673, PDF_CMAP_RANGE, 3089 },
	{ 0x4674, 0x4674, PDF_CMAP_SINGLE, 8393 },
	{ 0x4675, 0x4677, PDF_CMAP_RANGE, 3092 },
	{ 0x4678, 0x4678, PDF_CMAP_SINGLE, 8394 },
	{ 0x4679, 0x467c, PDF_CMAP_RANGE, 3096 },
	{ 0x467d, 0x467e, PDF_CMAP_TABLE, 246 },
	{ 0x4721, 0x4722, PDF_CMAP_RANGE, 3102 },
	{ 0x4723, 0x4724, PDF_CMAP_TABLE, 248 },
	{ 0x4725, 0x4726, PDF_CMAP_RANGE, 8397 },
	{ 0x4727, 0x4727, PDF_CMAP_SINGLE, 3108 },
	{ 0x4728, 0x4729, PDF_CMAP_RANGE, 8399 },
	{ 0x472a, 0x472b, PDF_CMAP_TABLE, 250 },
	{ 0x472c, 0x472d, PDF_CMAP_RANGE, 3113 },
	{ 0x472e, 0x472f, PDF_CMAP_RANGE, 8402 },
	{ 0x4730, 0x4732, PDF_CMAP_RANGE, 3117 },
	{ 0x4733, 0x4735, PDF_CMAP_RANGE, 8404 },
	{ 0x4736, 0x4738, PDF_CMAP_RANGE, 3123 },
	{ 0x4739, 0x473a, PDF_CMAP_RANGE, 8407 },
	{ 0x473b, 0x473c, PDF_CMAP_RANGE, 3128 },
	{ 0x473d, 0x473e, PDF_CMAP_RANGE, 8409 },
	{ 0x473f, 0x4742, PDF_CMAP_TABLE, 252 },
	{ 0x4743, 0x4744, PDF_CMAP_RANGE, 3136 },
	{ 0x4745, 0x4746, PDF_CMAP_TABLE, 256 },
	{ 0x4747, 0x4748, PDF_CMAP_RANGE, 8414 },
	{ 0x4749, 0x474b, PDF_CMAP_RANGE, 3142 },
	{ 0x474c, 0x474c, PDF_CMAP_SINGLE, 8416 },
	{ 0x474d, 0x474e, PDF_CMAP_RANGE, 3146 },
	{ 0x474f, 0x474f, PDF_CMAP_SINGLE, 8417 },
	{ 0x4750, 0x4753, PDF_CMAP_RANGE, 3149 },
	{ 0x4754, 0x4755, PDF_CMAP_RANGE, 8418 },
	{ 0x4756, 0x4757, PDF_CMAP_TABLE, 258 },
	{ 0x4758, 0x475d, PDF_CMAP_RANGE, 3157 },
	{ 0x475e, 0x475e, PDF_CMAP_SINGLE, 8421 },
	{ 0x475f, 0x4760, PDF_CMAP_RANGE, 3164 },
	{ 0x4761, 0x4763, PDF_CMAP_RANGE, 8422 },
	{ 0x4764, 0x4769, PDF_CMAP_RANGE, 3169 },
	{ 0x476a, 0x476e, PDF_CMAP_RANGE, 8425 },
	{ 0x476f, 0x4776, PDF_CMAP_RANGE, 3180 },
	{ 0x4777, 0x4778, PDF_CMAP_RANGE, 8430 },
	{ 0x4779, 0x477a, PDF_CMAP_RANGE, 3190 },
	{ 0x477b, 0x477e, PDF_CMAP_TABLE, 260 },
	{ 0x4821, 0x4822, PDF_CMAP_RANGE, 3196 },
	{ 0x4823, 0x4823, PDF_CMAP_SINGLE, 8434 },
	{ 0x4824, 0x4826, PDF_CMAP_RANGE, 3199 },
	{ 0x4827, 0x4828, PDF_CMAP_RANGE, 8435 },
	{ 0x4829, 0x482f, PDF_CMAP_RANGE, 3204 },
	{ 0x4830, 0x4830, PDF_CMAP_SINGLE, 8437 },
	{ 0x4831, 0x4834, PDF_CMAP_RANGE, 3212 },
	{ 0x4835, 0x4837, PDF_CMAP_TABLE, 264 },
	{ 0x4838, 0x4842, PDF_CMAP_RANGE, 3219 },
	{ 0x4843, 0x4846, PDF_CMAP_RANGE, 8440 },
	{ 0x4847, 0x4848, PDF_CMAP_TABLE, 267 },
	{ 0x4849, 0x484c, PDF_CMAP_RANGE, 3236 },
	{ 0x484d, 0x484f, PDF_CMAP_TABLE, 269 },
	{ 0x4850, 0x4851, PDF_CMAP_RANGE, 3243 },
	{ 0x4852, 0x4852, PDF_CMAP_SINGLE, 8447 },
	{ 0x4853, 0x4858, PDF_CMAP_RANGE, 3246 },
	{ 0x4859, 0x4859, PDF_CMAP_SINGLE, 8448 },
	{ 0x485a, 0x485d, PDF_CMAP_RANGE, 3253 },
	{ 0x485e, 0x485e, PDF_CMAP_SINGLE, 8449 },
	{ 0x485f, 0x486c, PDF_CMAP_RANGE, 3258 },
	{ 0x486d, 0x486d, PDF_CMAP_SINGLE, 8450 },
	{ 0x486e, 0x4870, PDF_CMAP_RANGE, 3273 },
	{ 0x4871, 0x4873, PDF_CMAP_RANGE, 8451 },
	{ 0x4874, 0x4876, PDF_CMAP_RANGE, 3279 },
	{ 0x4877, 0x4878, PDF_CMAP_RANGE, 8454 },
	{ 0x4879, 0x487c, PDF_CMAP_TABLE, 272 },
	{ 0x487d, 0x487e, PDF_CMAP_RANGE, 3288 },
	{ 0x4921, 0x4921, PDF_CMAP_SINGLE, 8458 },
	{ 0x4922, 0x4924, PDF_CMAP_RANGE, 3291 },
	{ 0x4925, 0x4926, PDF_CMAP_TABLE, 276 },
	{ 0x4927, 0x4928, PDF_CMAP_RANGE, 8460 },
	{ 0x4929, 0x492b, PDF_CMAP_RANGE, 3298 },
	{ 0x492c, 0x492c, PDF_CMAP_SINGLE, 8462 },
	{ 0x492d, 0x4930, PDF_CMAP_RANGE, 3302 },
	{ 0x4931, 0x4931, PDF_CMAP_SINGLE, 8463 },
	{ 0x4932, 0x4933, PDF_CMAP_RANGE, 3307 },
	{ 0x4934, 0x4934, PDF_CMAP_SINGLE, 8464 },
	{ 0x4935, 0x4937, PDF_CMAP_RANGE, 3310 },
	{ 0x4938, 0x4939, PDF_CMAP_RANGE, 8465 },
	{ 0x493a, 0x4940, PDF_CMAP_RANGE, 3315 },
	{ 0x4941, 0x4942, PDF_CMAP_RANGE, 8467 },
	{ 0x4943, 0x4944, PDF_CMAP_TABLE, 278 },
	{ 0x4945, 0x4948, PDF_CMAP_RANGE, 3326 },
	{ 0x4949, 0x494d, PDF_CMAP_TABLE, 280 },
	{ 0x494e, 0x4954, PDF_CMAP_RANGE, 3335 },
	{ 0x4955, 0x4955, PDF_CMAP_SINGLE, 8473 },
	{ 0x4956, 0x495b, PDF_CMAP_RANGE, 3343 },
	{ 0x495c, 0x495e, PDF_CMAP_TABLE, 285 },
	{ 0x495f, 0x4960, PDF_CMAP_RANGE, 3352 },
	{ 0x4961, 0x4965, PDF_CMAP_TABLE, 288 },
	{ 0x4966, 0x4967, PDF_CMAP_RANGE, 3359 },
	{ 0x4968, 0x4968, PDF_CMAP_SINGLE, 8479 },
	{ 0x4969, 0x496f, PDF_CMAP_RANGE, 3362 },
	{ 0x4970, 0x4970, PDF_CMAP_SINGLE, 8480 },
	{ 0x4971, 0x4972, PDF_CMAP_RANGE, 3370 },
	{ 0x4973, 0x4974, PDF_CMAP_RANGE, 8481 },
	{ 0x4975, 0x4977, PDF_CMAP_TABLE, 293 },
	{ 0x4978, 0x4979, PDF_CMAP_RANGE, 8484 },
	{ 0x497a, 0x497d, PDF_CMAP_RANGE, 3379 },
	{ 0x497e, 0x497e, PDF_CMAP_SINGLE, 8486 },
	{ 0x4a21, 0x4a23, PDF_CMAP_RANGE, 3384 },
	{ 0x4a24, 0x4a26, PDF_CMAP_RANGE, 8487 },
	{ 0x4a27, 0x4a29, PDF_CMAP_TABLE, 296 },
	{ 0x4a2a, 0x4a2b, PDF_CMAP_RANGE, 8491 },
	{ 0x4a2c, 0x4a30, PDF_CMAP_RANGE, 3395 },
	{ 0x4a31, 0x4a31, PDF_CMAP_SINGLE, 8493 },
	{ 0x4a32, 0x4a33, PDF_CMAP_RANGE, 3401 },
	{ 0x4a34, 0x4a36, PDF_CMAP_RANGE, 8494 },
	{ 0x4a37, 0x4a3a, PDF_CMAP_RANGE, 3406 },
	{ 0x4a3b, 0x4a3b, PDF_CMAP_SINGLE, 8497 },
	{ 0x4a3c, 0x4a45, PDF_CMAP_RANGE, 3411 },
	{ 0x4a46, 0x4a46, PDF_CMAP_SINGLE, 8498 },
	{ 0x4a47, 0x4a49, PDF_CMAP_RANGE, 3422 },
	{ 0x4a4a, 0x4a4a, PDF_CMAP_SINGLE, 8499 },
	{ 0x4a4b, 0x4a4c, PDF_CMAP_RANGE, 3426 },
	{ 0x4a4d, 0x4a4e, PDF_CMAP_RANGE, 8500 },
	{ 0x4a4f, 0x4a52, PDF_CMAP_RANGE, 3430 },
	{ 0x4a53, 0x4a54, PDF_CMAP_RANGE, 8502 },
	{ 0x4a55, 0x4a58, PDF_CMAP_RANGE, 3436 },
	{ 0x4a59, 0x4a59, PDF_CMAP_SINGLE, 8504 },
	{ 0x4a5a, 0x4a5d, PDF_CMAP_RANGE, 3441 },
	{ 0x4a5e, 0x4a60, PDF_CMAP_TABLE, 299 },
	{ 0x4a61, 0x4a63, PDF_CMAP_RANGE, 3448 },
	{ 0x4a64, 0x4a64, PDF_CMAP_SINGLE, 8507 },
	{ 0x4a65, 0x4a68, PDF_CMAP_RANGE, 3452 },
	{ 0x4a69, 0x4a6a, PDF_CMAP_RANGE, 8508 },
	{ 0x4a6b, 0x4a73, PDF_CMAP_RANGE, 3458 },
	{ 0x4a74, 0x4a75, PDF_CMAP_RANGE, 8510 },
	{ 0x4a76, 0x4a77, PDF_CMAP_TABLE, 302 },
	{ 0x4a78, 0x4a79, PDF_CMAP_RANGE, 3471 },
	{ 0x4a7a, 0x4a7a, PDF_CMAP_SINGLE, 8513 },
	{ 0x4a7b, 0x4a7c, PDF_CMAP_RANGE, 3474 },
	{ 0x4a7d, 0x4a7e, PDF_CMAP_TABLE, 304 },
	{ 0x4b21, 0x4b26, PDF_CMAP_RANGE, 3478 },
	{ 0x4b27, 0x4b27, PDF_CMAP_SINGLE, 8515 },
	{ 0x4b28, 0x4b2a, PDF_CMAP_RANGE, 3485 },
	{ 0x4b2b, 0x4b2d, PDF_CMAP_TABLE, 306 },
	{ 0x4b2e, 0x4b32, PDF_CMAP_RANGE, 3491 },
	{ 0x4b33, 0x4b34, PDF_CMAP_TABLE, 309 },
	{ 0x4b35, 0x4b36, PDF_CMAP_RANGE, 8519 },
	{ 0x4b37, 0x4b38, PDF_CMAP_TABLE, 311 },
	{ 0x4b39, 0x4b3e, PDF_CMAP_RANGE, 3502 },
	{ 0x4b3f, 0x4b3f, PDF_CMAP_SINGLE, 8522 },
	{ 0x4b40, 0x4b46, PDF_CMAP_RANGE, 3509 },
	{ 0x4b47, 0x4b48, PDF_CMAP_TABLE, 313 },
	{ 0x4b49, 0x4b4c, PDF_CMAP_RANGE, 8524 },
	{ 0x4b4d, 0x4b4e, PDF_CMAP_RANGE, 3522 },
	{ 0x4b4f, 0x4b50, PDF_CMAP_RANGE, 8528 },
	{ 0x4b51, 0x4b52, PDF_CMAP_RANGE, 3526 },
	{ 0x4b53, 0x4b55, PDF_CMAP_TABLE, 315 },
	{ 0x4b56, 0x4b5e, PDF_CMAP_RANGE, 3531 },
	{ 0x4b5f, 0x4b60, PDF_CMAP_RANGE, 8532 },
	{ 0x4b61, 0x4b63, PDF_CMAP_RANGE, 3542 },
	{ 0x4b64, 0x4b65, PDF_CMAP_TABLE, 318 },
	{ 0x4b66, 0x4b67, PDF_CMAP_RANGE, 8535 },
	{ 0x4b68, 0x4b69, PDF_CMAP_RANGE, 3549 },
	{ 0x4b6a, 0x4b6a, PDF_CMAP_SINGLE, 8537 },
	{ 0x4b6b, 0x4b6e, PDF_CMAP_RANGE, 3552 },
	{ 0x4b6f, 0x4b70, PDF_CMAP_RANGE, 8538 },
	{ 0x4b71, 0x4b74, PDF_CMAP_RANGE, 3558 },
	{ 0x4b75, 0x4b76, PDF_CMAP_RANGE, 8540 },
	{ 0x4b77, 0x4b78, PDF_CMAP_TABLE, 320 },
	{ 0x4b79, 0x4b7e, PDF_CMAP_RANGE, 3566 },
	{ 0x4c21, 0x4c22, PDF_CMAP_RANGE, 8543 },
	{ 0x4c23, 0x4c27, PDF_CMAP_RANGE, 3574 },
	{ 0x4c28, 0x4c28, PDF_CMAP_SINGLE, 8545 },
	{ 0x4c29, 0x4c2b, PDF_CMAP_RANGE, 3580 },
	{ 0x4c2c, 0x4c2c, PDF_CMAP_SINGLE, 8546 },
	{ 0x4c2d, 0x4c2e, PDF_CMAP_RANGE, 3584 },
	{ 0x4c2f, 0x4c33, PDF_CMAP_RANGE, 8547 },
	{ 0x4c34, 0x4c36, PDF_CMAP_RANGE, 3591 },
	{ 0x4c37, 0x4c38, PDF_CMAP_RANGE, 8552 },
	{ 0x4c39, 0x4c3d, PDF_CMAP_RANGE, 3596 },
	{ 0x4c3e, 0x4c40, PDF_CMAP_TABLE, 322 },
	{ 0x4c41, 0x4c4b, PDF_CMAP_RANGE, 3604 },
	{ 0x4c4c, 0x4c50, PDF_CMAP_TABLE, 325 },
	{ 0x4c51, 0x4c55, PDF_CMAP_RANGE, 3620 },
	{ 0x4c56, 0x4c56, PDF_CMAP_SINGLE, 8559 },
	{ 0x4c57, 0x4c59, PDF_CMAP_RANGE, 3626 },
	{ 0x4c5a, 0x4c5c, PDF_CMAP_TABLE, 330 },
	{ 0x4c5d, 0x4c5f, PDF_CMAP_RANGE, 3632 },
	{ 0x4c60, 0x4c62, PDF_CMAP_TABLE, 333 },
	{ 0x4c63, 0x4c64, PDF_CMAP_RANGE, 3638 },
	{ 0x4c65, 0x4c65, PDF_CMAP_SINGLE, 8564 },
	{ 0x4c66, 0x4c74, PDF_CMAP_RANGE, 3641 },
	{ 0x4c75, 0x4c75, PDF_CMAP_SINGLE, 8565 },
	{ 0x4c76, 0x4c78, PDF_CMAP_RANGE, 3657 },
	{ 0x4c79, 0x4c7a, PDF_CMAP_RANGE, 8566 },
	{ 0x4c7b, 0x4c7b, PDF_CMAP_SINGLE, 3662 },
	{ 0x4c7c, 0x4c7e, PDF_CMAP_RANGE, 8568 },
	{ 0x4d21, 0x4d2c, PDF_CMAP_RANGE, 3666 },
	{ 0x4d2d, 0x4d2d, PDF_CMAP_SINGLE, 8571 },
	{ 0x4d2e, 0x4d32, PDF_CMAP_RANGE, 3679 },
	{ 0x4d33, 0x4d33, PDF_CMAP_SINGLE, 8572 },
	{ 0x4d34, 0x4d36, PDF_CMAP_RANGE, 3685 },
	{ 0x4d37, 0x4d37, PDF_CMAP_SINGLE, 8573 },
	{ 0x4d38, 0x4d3b, PDF_CMAP_RANGE, 3689 },
	{ 0x4d3c, 0x4d3c, PDF_CMAP_SINGLE, 8574 },
	{ 0x4d3d, 0x4d3e, PDF_CMAP_RANGE, 3694 },
	{ 0x4d3f, 0x4d3f, PDF_CMAP_SINGLE, 8575 },
	{ 0x4d40, 0x4d44, PDF_CMAP_RANGE, 3697 },
	{ 0x4d45, 0x4d47, PDF_CMAP_TABLE, 336 },
	{ 0x4d48, 0x4d51, PDF_CMAP_RANGE, 3705 },
	{ 0x4d52, 0x4d53, PDF_CMAP_TABLE, 339 },
	{ 0x4d54, 0x4d56, PDF_CMAP_RANGE, 8579 },
	{ 0x4d57, 0x4d5c, PDF_CMAP_RANGE, 3720 },
	{ 0x4d5d, 0x4d5d, PDF_CMAP_SINGLE, 8582 },
	{ 0x4d5e, 0x4d5f, PDF_CMAP_RANGE, 3727 },
	{ 0x4d60, 0x4d60, PDF_CMAP_SINGLE, 8583 },
	{ 0x4d61, 0x4d63, PDF_CMAP_RANGE, 3730 },
	{ 0x4d64, 0x4d65, PDF_CMAP_RANGE, 8584 },
	{ 0x4d66, 0x4d67, PDF_CMAP_TABLE, 341 },
	{ 0x4d68, 0x4d71, PDF_CMAP_RANGE, 3737 },
	{ 0x4d72, 0x4d72, PDF_CMAP_SINGLE, 8587 },
	{ 0x4d73, 0x4d77, PDF_CMAP_RANGE, 3748 },
	{ 0x4d78, 0x4d78, PDF_CMAP_SINGLE, 8588 },
	{ 0x4d79, 0x4d7e, PDF_CMAP_RANGE, 3754 },
	{ 0x4e21, 0x4e23, PDF_CMAP_RANGE, 3760 },
	{ 0x4e24, 0x4e25, PDF_CMAP_RANGE, 8589 },
	{ 0x4e26, 0x4e27, PDF_CMAP_TABLE, 343 },
	{ 0x4e28, 0x4e29, PDF_CMAP_RANGE, 3767 },
	{ 0x4e2a, 0x4e2d, PDF_CMAP_RANGE, 8592 },
	{ 0x4e2e, 0x4e2f, PDF_CMAP_RANGE, 3773 },
	{ 0x4e30, 0x4e31, PDF_CMAP_RANGE, 8596 },
	{ 0x4e32, 0x4e33, PDF_CMAP_TABLE, 345 },
	{ 0x4e34, 0x4e3c, PDF_CMAP_RANGE, 3779 },
	{ 0x4e3d, 0x4e3d, PDF_CMAP_SINGLE, 8599 },
	{ 0x4e3e, 0x4e3f, PDF_CMAP_RANGE, 3789 },
	{ 0x4e40, 0x4e40, PDF_CMAP_SINGLE, 8600 },
	{ 0x4e41, 0x4e44, PDF_CMAP_RANGE, 3792 },
	{ 0x4e45, 0x4e46, PDF_CMAP_RANGE, 8601 },
	{ 0x4e47, 0x4e4a, PDF_CMAP_TABLE, 347 },
	{ 0x4e4b, 0x4e4d, PDF_CMAP_RANGE, 3802 },
	{ 0x4e4e, 0x4e51, PDF_CMAP_RANGE, 8605 },
	{ 0x4e52, 0x4e57, PDF_CMAP_RANGE, 3809 },
	{ 0x4e58, 0x4e5a, PDF_CMAP_RANGE, 8609 },
	{ 0x4e5b, 0x4e5d, PDF_CMAP_TABLE, 351 },
	{ 0x4e5e, 0x4e5f, PDF_CMAP_RANGE, 8613 },
	{ 0x4e60, 0x4e6a, PDF_CMAP_RANGE, 3823 },
	{ 0x4e6b, 0x4e6d, PDF_CMAP_TABLE, 354 },
	{ 0x4e6e, 0x4e70, PDF_CMAP_RANGE, 3837 },
	{ 0x4e71, 0x4e73, PDF_CMAP_TABLE, 357 },
	{ 0x4e74, 0x4e7c, PDF_CMAP_RANGE, 3843 },
	{ 0x4e7d, 0x4e7e, PDF_CMAP_RANGE, 8619 },
	{ 0x4f21, 0x4f2d, PDF_CMAP_RANGE, 3854 },
	{ 0x4f2e, 0x4f30, PDF_CMAP_TABLE, 360 },
	{ 0x4f31, 0x4f32, PDF_CMAP_RANGE, 3870 },
	{ 0x4f33, 0x4f33, PDF_CMAP_SINGLE, 8623 },
	{ 0x4f34, 0x4f36, PDF_CMAP_RANGE, 3873 },
	{ 0x4f37, 0x4f38, PDF_CMAP_RANGE, 8624 },
	{ 0x4f39, 0x4f3a, PDF_CMAP_TABLE, 363 },
	{ 0x4f3b, 0x4f3c, PDF_CMAP_RANGE, 3880 },
	{ 0x4f3d, 0x4f3e, PDF_CMAP_TABLE, 365 },
	{ 0x4f3f, 0x4f41, PDF_CMAP_RANGE, 8628 },
	{ 0x4f42, 0x4f44, PDF_CMAP_RANGE, 3887 },
	{ 0x4f45, 0x4f47, PDF_CMAP_TABLE, 367 },
	{ 0x4f48, 0x4f49, PDF_CMAP_RANGE, 3893 },
	{ 0x4f4a, 0x4f4b, PDF_CMAP_RANGE, 8633 },
	{ 0x4f4c, 0x4f4c, PDF_CMAP_SINGLE, 3897 },
	{ 0x4f4d, 0x4f4e, PDF_CMAP_RANGE, 8635 },
	{ 0x4f4f, 0x4f50, PDF_CMAP_TABLE, 370 },
	{ 0x4f51, 0x4f53, PDF_CMAP_RANGE, 3902 },
	{ 0x4f54, 0x4f58, PDF_CMAP_RANGE, 8638 },
	{ 0x4f59, 0x4f5c, PDF_CMAP_TABLE, 372 },
	{ 0x4f5d, 0x4f5e, PDF_CMAP_RANGE, 3914 },
	{ 0x4f5f, 0x4f5f, PDF_CMAP_SINGLE, 8645 },
	{ 0x4f60, 0x4f61, PDF_CMAP_RANGE, 3917 },
	{ 0x4f62, 0x4f62, PDF_CMAP_SINGLE, 8646 },
	{ 0x4f63, 0x4f66, PDF_CMAP_RANGE, 3920 },
	{ 0x4f67, 0x4f67, PDF_CMAP_SINGLE, 8647 },
	{ 0x4f68, 0x4f69, PDF_CMAP_RANGE, 3925 },
	{ 0x4f6a, 0x4f6e, PDF_CMAP_TABLE, 376 },
	{ 0x4f6f, 0x4f73, PDF_CMAP_RANGE, 3932 },
	{ 0x4f74, 0x4f74, PDF_CMAP_SINGLE, 8651 },
	{ 0x4f75, 0x4f78, PDF_CMAP_RANGE, 3938 },
	{ 0x4f79, 0x4f7a, PDF_CMAP_RANGE, 8652 },
	{ 0x4f7b, 0x4f7d, PDF_CMAP_RANGE, 3944 },
	{ 0x4f7e, 0x4f7e, PDF_CMAP_SINGLE, 8654 },
	{ 0x5021, 0x5024, PDF_CMAP_RANGE, 3948 },
	{ 0x5025, 0x5025, PDF_CMAP_SINGLE, 8655 },
	{ 0x5026, 0x502c, PDF_CMAP_RANGE, 3953 },
	{ 0x502d, 0x502e, PDF_CMAP_RANGE, 8656 },
	{ 0x502f, 0x5031, PDF_CMAP_RANGE, 3962 },
	{ 0x5032, 0x5034, PDF_CMAP_RANGE, 8658 },
	{ 0x5035, 0x5039, PDF_CMAP_RANGE, 3968 },
	{ 0x503a, 0x503b, PDF_CMAP_RANGE, 8661 },
	{ 0x503c, 0x503e, PDF_CMAP_RANGE, 3975 },
	{ 0x503f, 0x503f, PDF_CMAP_SINGLE, 8663 },
	{ 0x5040, 0x5045, PDF_CMAP_RANGE, 3979 },
	{ 0x5046, 0x5046, PDF_CMAP_SINGLE, 8664 },
	{ 0x5047, 0x504a, PDF_CMAP_RANGE, 3986 },
	{ 0x504b, 0x504b, PDF_CMAP_SINGLE, 8665 },
	{ 0x504c, 0x5061, PDF_CMAP_RANGE, 3991 },
	{ 0x5062, 0x5062, PDF_CMAP_SINGLE, 8666 },
	{ 0x5063, 0x5064, PDF_CMAP_RANGE, 4014 },
	{ 0x5065, 0x5065, PDF_CMAP_SINGLE, 8667 },
	{ 0x5066, 0x506a, PDF_CMAP_RANGE, 4017 },
	{ 0x506b, 0x506d, PDF_CMAP_TABLE, 381 },
	{ 0x506e, 0x5076, PDF_CMAP_RANGE, 4025 },
	{ 0x5077, 0x5079, PDF_CMAP_RANGE, 8670 },
	{ 0x507a, 0x507b, PDF_CMAP_RANGE, 4037 },
	{ 0x507c, 0x507c, PDF_CMAP_SINGLE, 8673 },
	{ 0x507d, 0x507e, PDF_CMAP_RANGE, 4040 },
	{ 0x5121, 0x5122, PDF_CMAP_RANGE, 8674 },
	{ 0x5123, 0x5124, PDF_CMAP_TABLE, 384 },
	{ 0x5125, 0x5126, PDF_CMAP_RANGE, 4046 },
	{ 0x5127, 0x5127, PDF_CMAP_SINGLE, 8677 },
	{ 0x5128, 0x512a, PDF_CMAP_RANGE, 4049 },
	{ 0x512b, 0x512b, PDF_CMAP_SINGLE, 8678 },
	{ 0x512c, 0x512e, PDF_CMAP_RANGE, 4053 },
	{ 0x512f, 0x5131, PDF_CMAP_RANGE, 8679 },
	{ 0x5132, 0x5134, PDF_CMAP_RANGE, 4059 },
	{ 0x5135, 0x5137, PDF_CMAP_RANGE, 8682 },
	{ 0x5138, 0x513a, PDF_CMAP_TABLE, 386 },
	{ 0x513b, 0x513c, PDF_CMAP_RANGE, 8686 },
	{ 0x513d, 0x5145, PDF_CMAP_RANGE, 4070 },
	{ 0x5146, 0x5148, PDF_CMAP_RANGE, 8688 },
	{ 0x5149, 0x514a, PDF_CMAP_RANGE, 4082 },
	{ 0x514b, 0x514b, PDF_CMAP_SINGLE, 8691 },
	{ 0x514c, 0x514d, PDF_CMAP_RANGE, 4085 },
	{ 0x514e, 0x514f, PDF_CMAP_RANGE, 8692 },
	{ 0x5150, 0x5154, PDF_CMAP_RANGE, 4089 },
	{ 0x5155, 0x5156, PDF_CMAP_RANGE, 8694 },
	{ 0x5157, 0x515d, PDF_CMAP_RANGE, 4096 },
	{ 0x515e, 0x515e, PDF_CMAP_SINGLE, 8696 },
	{ 0x515f, 0x5160, PDF_CMAP_RANGE, 4104 },
	{ 0x5161, 0x5162, PDF_CMAP_RANGE, 8697 },
	{ 0x5163, 0x5167, PDF_CMAP_RANGE, 4108 },
	{ 0x5168, 0x5169, PDF_CMAP_RANGE, 8699 },
	{ 0x516a, 0x516b, PDF_CMAP_RANGE, 4115 },
	{ 0x516c, 0x516d, PDF_CMAP_TABLE, 389 },
	{ 0x516e, 0x516f, PDF_CMAP_RANGE, 8702 },
	{ 0x5170, 0x5171, PDF_CMAP_TABLE, 391 },
	{ 0x5172, 0x5173, PDF_CMAP_RANGE, 4123 },
	{ 0x5174, 0x5174, PDF_CMAP_SINGLE, 8705 },
	{ 0x5175, 0x5176, PDF_CMAP_RANGE, 4126 },
	{ 0x5177, 0x5179, PDF_CMAP_RANGE, 8706 },
	{ 0x517a, 0x517e, PDF_CMAP_RANGE, 4131 },
	{ 0x5221, 0x5222, PDF_CMAP_TABLE, 393 },
	{ 0x5223, 0x5224, PDF_CMAP_RANGE, 4138 },
	{ 0x5225, 0x5225, PDF_CMAP_SINGLE, 8710 },
	{ 0x5226, 0x5228, PDF_CMAP_RANGE, 4141 },
	{ 0x5229, 0x5229, PDF_CMAP_SINGLE, 8711 },
	{ 0x522a, 0x522e, PDF_CMAP_RANGE, 4145 },
	{ 0x522f, 0x522f, PDF_CMAP_SINGLE, 8712 },
	{ 0x5230, 0x5232, PDF_CMAP_RANGE, 4151 },
	{ 0x5233, 0x5234, PDF_CMAP_TABLE, 395 },
	{ 0x5235, 0x5236, PDF_CMAP_RANGE, 8714 },
	{ 0x5237, 0x523c, PDF_CMAP_RANGE, 4158 },
	{ 0x523d, 0x523f, PDF_CMAP_TABLE, 397 },
	{ 0x5240, 0x5242, PDF_CMAP_RANGE, 4167 },
	{ 0x5243, 0x5247, PDF_CMAP_TABLE, 400 },
	{ 0x5248, 0x524e, PDF_CMAP_RANGE, 4175 },
	{ 0x524f, 0x524f, PDF_CMAP_SINGLE, 8721 },
	{ 0x5250, 0x5254, PDF_CMAP_RANGE, 4183 },
	{ 0x5255, 0x5255, PDF_CMAP_SINGLE, 8722 },
	{ 0x5256, 0x5259, PDF_CMAP_RANGE, 4189 },
	{ 0x525a, 0x525a, PDF_CMAP_SINGLE, 8723 },
	{ 0x525b, 0x5263, PDF_CMAP_RANGE, 4194 },
	{ 0x5264, 0x5265, PDF_CMAP_RANGE, 8724 },
	{ 0x5266, 0x5267, PDF_CMAP_RANGE, 4205 },
	{ 0x5268, 0x526b, PDF_CMAP_RANGE, 8726 },
	{ 0x526c, 0x526e, PDF_CMAP_RANGE, 4211 },
	{ 0x526f, 0x5271, PDF_CMAP_TABLE, 405 },
	{ 0x5272, 0x5274, PDF_CMAP_RANGE, 4217 },
	{ 0x5275, 0x5275, PDF_CMAP_SINGLE, 8732 },
	{ 0x5276, 0x5277, PDF_CMAP_RANGE, 4221 },
	{ 0x5278, 0x5278, PDF_CMAP_SINGLE, 8733 },
	{ 0x5279, 0x527a, PDF_CMAP_RANGE, 4224 },
	{ 0x527b, 0x527b, PDF_CMAP_SINGLE, 8734 },
	{ 0x527c, 0x527d, PDF_CMAP_RANGE, 4227 },
	{ 0x527e, 0x527e, PDF_CMAP_SINGLE, 8735 },
	{ 0x5321, 0x5322, PDF_CMAP_RANGE, 4230 },
	{ 0x5323, 0x532c, PDF_CMAP_RANGE, 8736 },
	{ 0x532d, 0x532e, PDF_CMAP_TABLE, 408 },
	{ 0x532f, 0x5330, PDF_CMAP_RANGE, 4244 },
	{ 0x5331, 0x5331, PDF_CMAP_SINGLE, 8747 },
	{ 0x5332, 0x5333, PDF_CMAP_RANGE, 4247 },
	{ 0x5334, 0x5336, PDF_CMAP_RANGE, 8748 },
	{ 0x5337, 0x5338, PDF_CMAP_TABLE, 410 },
	{ 0x5339, 0x533a, PDF_CMAP_RANGE, 4254 },
	{ 0x533b, 0x533b, PDF_CMAP_SINGLE, 8752 },
	{ 0x533c, 0x5344, PDF_CMAP_RANGE, 4257 },
	{ 0x5345, 0x5347, PDF_CMAP_TABLE, 412 },
	{ 0x5348, 0x5349, PDF_CMAP_RANGE, 4269 },
	{ 0x534a, 0x534c, PDF_CMAP_RANGE, 8755 },
	{ 0x534d, 0x5354, PDF_CMAP_RANGE, 4274 },
	{ 0x5355, 0x5355, PDF_CMAP_SINGLE, 8758 },
	{ 0x5356, 0x535e, PDF_CMAP_RANGE, 4283 },
	{ 0x535f, 0x5360, PDF_CMAP_RANGE, 8759 },
	{ 0x5361, 0x5362, PDF_CMAP_RANGE, 4294 },
	{ 0x5363, 0x5363, PDF_CMAP_SINGLE, 8761 },
	{ 0x5364, 0x5365, PDF_CMAP_RANGE, 4297 },
	{ 0x5366, 0x5366, PDF_CMAP_SINGLE, 8762 },
	{ 0x5367, 0x536a, PDF_CMAP_RANGE, 4300 },
	{ 0x536b, 0x536c, PDF_CMAP_RANGE, 8763 },
	{ 0x536d, 0x536e, PDF_CMAP_RANGE, 4306 },
	{ 0x536f, 0x536f, PDF_CMAP_SINGLE, 8765 },
	{ 0x5370, 0x5373, PDF_CMAP_RANGE, 4309 },
	{ 0x5374, 0x5374, PDF_CMAP_SINGLE, 8766 },
	{ 0x5375, 0x537b, PDF_CMAP_RANGE, 4314 },
	{ 0x537c, 0x537e, PDF_CMAP_TABLE, 415 },
	{ 0x5421, 0x5423, PDF_CMAP_RANGE, 4324 },
	{ 0x5424, 0x5425, PDF_CMAP_TABLE, 418 },
	{ 0x5426, 0x5428, PDF_CMAP_RANGE, 8770 },
	{ 0x5429, 0x542e, PDF_CMAP_RANGE, 4332 },
	{ 0x542f, 0x5432, PDF_CMAP_RANGE, 8773 },
	{ 0x5433, 0x5434, PDF_CMAP_RANGE, 4342 },
	{ 0x5435, 0x5436, PDF_CMAP_RANGE, 8777 },
	{ 0x5437, 0x5438, PDF_CMAP_TABLE, 420 },
	{ 0x5439, 0x543b, PDF_CMAP_RANGE, 4348 },
	{ 0x543c, 0x543d, PDF_CMAP_TABLE, 422 },
	{ 0x543e, 0x543f, PDF_CMAP_RANGE, 8781 },
	{ 0x5440, 0x5443, PDF_CMAP_RANGE, 4355 },
	{ 0x5444, 0x5445, PDF_CMAP_TABLE, 424 },
	{ 0x5446, 0x5447, PDF_CMAP_RANGE, 8784 },
	{ 0x5448, 0x544a, PDF_CMAP_TABLE, 426 },
	{ 0x544b, 0x544e, PDF_CMAP_RANGE, 8787 },
	{ 0x544f, 0x5452, PDF_CMAP_RANGE, 4370 },
	{ 0x5453, 0x5453, PDF_CMAP_SINGLE, 8791 },
	{ 0x5454, 0x5457, PDF_CMAP_RANGE, 4375 },
	{ 0x5458, 0x5458, PDF_CMAP_SINGLE, 8792 },
	{ 0x5459, 0x545b, PDF_CMAP_RANGE, 4380 },
	{ 0x545c, 0x5460, PDF_CMAP_RANGE, 8793 },
	{ 0x5461, 0x5463, PDF_CMAP_RANGE, 4388 },
	{ 0x5464, 0x5466, PDF_CMAP_TABLE, 429 },
	{ 0x5467, 0x546d, PDF_CMAP_RANGE, 4394 },
	{ 0x546e, 0x546f, PDF_CMAP_TABLE, 432 },
	{ 0x5470, 0x5474, PDF_CMAP_RANGE, 8801 },
	{ 0x5475, 0x5478, PDF_CMAP_RANGE, 4408 },
	{ 0x5479, 0x5479, PDF_CMAP_SINGLE, 8806 },
	{ 0x547a, 0x547d, PDF_CMAP_RANGE, 4413 },
	{ 0x547e, 0x547e, PDF_CMAP_SINGLE, 8807 },
	{ 0x5521, 0x5522, PDF_CMAP_RANGE, 8808 },
	{ 0x5523, 0x5528, PDF_CMAP_RANGE, 4420 },
	{ 0x5529, 0x552b, PDF_CMAP_TABLE, 434 },
	{ 0x552c, 0x552d, PDF_CMAP_RANGE, 4429 },
	{ 0x552e, 0x552e, PDF_CMAP_SINGLE, 8812 },
	{ 0x552f, 0x5530, PDF_CMAP_RANGE, 4432 },
	{ 0x5531, 0x5531, PDF_CMAP_SINGLE, 8813 },
	{ 0x5532, 0x5534, PDF_CMAP_RANGE, 4435 },
	{ 0x5535, 0x5538, PDF_CMAP_RANGE, 8814 },
	{ 0x5539, 0x553a, PDF_CMAP_RANGE, 4442 },
	{ 0x553b, 0x553d, PDF_CMAP_TABLE, 437 },
	{ 0x553e, 0x553f, PDF_CMAP_RANGE, 4447 },
	{ 0x5540, 0x5540, PDF_CMAP_SINGLE, 8820 },
	{ 0x5541, 0x5544, PDF_CMAP_RANGE, 4450 },
	{ 0x5545, 0x5547, PDF_CMAP_TABLE, 440 },
	{ 0x5548, 0x5549, PDF_CMAP_RANGE, 4457 },
	{ 0x554a, 0x554b, PDF_CMAP_RANGE, 8823 },
	{ 0x554c, 0x554d, PDF_CMAP_TABLE, 443 },
	{ 0x554e, 0x5553, PDF_CMAP_RANGE, 4463 },
	{ 0x5554, 0x5554, PDF_CMAP_SINGLE, 8826 },
	{ 0x5555, 0x555c, PDF_CMAP_RANGE, 4470 },
	{ 0x555d, 0x555e, PDF_CMAP_RANGE, 8827 },
	{ 0x555f, 0x5562, PDF_CMAP_TABLE, 445 },
	{ 0x5563, 0x5569, PDF_CMAP_RANGE, 4484 },
	{ 0x556a, 0x556c, PDF_CMAP_RANGE, 8831 },
	{ 0x556d, 0x556e, PDF_CMAP_RANGE, 4494 },
	{ 0x556f, 0x556f, PDF_CMAP_SINGLE, 8834 },
	{ 0x5570, 0x5571, PDF_CMAP_RANGE, 4497 },
	{ 0x5572, 0x5573, PDF_CMAP_RANGE, 8835 },
	{ 0x5574, 0x5576, PDF_CMAP_RANGE, 4501 },
	{ 0x5577, 0x5577, PDF_CMAP_SINGLE, 5611 },
	{ 0x5578, 0x557e, PDF_CMAP_RANGE, 4505 },
	{ 0x5621, 0x5624, PDF_CMAP_RANGE, 8837 },
	{ 0x5625, 0x562e, PDF_CMAP_RANGE, 4516 },
	{ 0x562f, 0x5630, PDF_CMAP_RANGE, 8841 },
	{ 0x5631, 0x5633, PDF_CMAP_RANGE, 4528 },
	{ 0x5634, 0x5634, PDF_CMAP_SINGLE, 8843 },
	{ 0x5635, 0x563a, PDF_CMAP_RANGE, 4532 },
	{ 0x563b, 0x563e, PDF_CMAP_TABLE, 449 },
	{ 0x563f, 0x5640, PDF_CMAP_RANGE, 8846 },
	{ 0x5641, 0x5643, PDF_CMAP_RANGE, 4544 },
	{ 0x5644, 0x5644, PDF_CMAP_SINGLE, 8848 },
	{ 0x5645, 0x5649, PDF_CMAP_RANGE, 4548 },
	{ 0x564a, 0x564a, PDF_CMAP_SINGLE, 8849 },
	{ 0x564b, 0x564c, PDF_CMAP_RANGE, 4554 },
	{ 0x564d, 0x564d, PDF_CMAP_SINGLE, 8850 },
	{ 0x564e, 0x5652, PDF_CMAP_RANGE, 4557 },
	{ 0x5653, 0x5654, PDF_CMAP_TABLE, 453 },
	{ 0x5655, 0x5657, PDF_CMAP_RANGE, 8852 },
	{ 0x5658, 0x5659, PDF_CMAP_RANGE, 4567 },
	{ 0x565a, 0x565a, PDF_CMAP_SINGLE, 8855 },
	{ 0x565b, 0x565e, PDF_CMAP_RANGE, 4570 },
	{ 0x565f, 0x5661, PDF_CMAP_TABLE, 455 },
	{ 0x5662, 0x5664, PDF_CMAP_RANGE, 4577 },
	{ 0x5665, 0x5666, PDF_CMAP_TABLE, 458 },
	{ 0x5667, 0x5668, PDF_CMAP_RANGE, 8859 },
	{ 0x5669, 0x566d, PDF_CMAP_RANGE, 4584 },
	{ 0x566e, 0x566f, PDF_CMAP_RANGE, 8861 },
	{ 0x5670, 0x5671, PDF_CMAP_RANGE, 4591 },
	{ 0x5672, 0x5672, PDF_CMAP_SINGLE, 8863 },
	{ 0x5673, 0x5674, PDF_CMAP_RANGE, 4594 },
	{ 0x5675, 0x5676, PDF_CMAP_RANGE, 8864 },
	{ 0x5677, 0x567b, PDF_CMAP_RANGE, 4598 },
	{ 0x567c, 0x567e, PDF_CMAP_RANGE, 8866 },
	{ 0x5721, 0x5723, PDF_CMAP_RANGE, 4606 },
	{ 0x5724, 0x5724, PDF_CMAP_SINGLE, 8869 },
	{ 0x5725, 0x5727, PDF_CMAP_RANGE, 4610 },
	{ 0x5728, 0x572a, PDF_CMAP_RANGE, 8870 },
	{ 0x572b, 0x572d, PDF_CMAP_TABLE, 460 },
	{ 0x572e, 0x5731, PDF_CMAP_RANGE, 8874 },
	{ 0x5732, 0x5732, PDF_CMAP_SINGLE, 4623 },
	{ 0x5733, 0x5734, PDF_CMAP_RANGE, 8878 },
	{ 0x5735, 0x5737, PDF_CMAP_TABLE, 463 },
	{ 0x5738, 0x573c, PDF_CMAP_RANGE, 8881 },
	{ 0x573d, 0x5746, PDF_CMAP_RANGE, 4634 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 8886 },
	{ 0x5748, 0x5749, PDF_CMAP_RANGE, 4645 },
	{ 0x574a, 0x574a, PDF_CMAP_SINGLE, 8887 },
	{ 0x574b, 0x5754, PDF_CMAP_RANGE, 4648 },
	{ 0x5755, 0x5755, PDF_CMAP_SINGLE, 8888 },
	{ 0x5756, 0x575a, PDF_CMAP_RANGE, 4659 },
	{ 0x575b, 0x575e, PDF_CMAP_RANGE, 8889 },
	{ 0x575f, 0x5766, PDF_CMAP_RANGE, 4668 },
	{ 0x5767, 0x5768, PDF_CMAP_TABLE, 466 },
	{ 0x5769, 0x576a, PDF_CMAP_RANGE, 8894 },
	{ 0x576b, 0x5779, PDF_CMAP_RANGE, 4680 },
	{ 0x5821, 0x5843, PDF_CMAP_RANGE, 4695 },
	{ 0x5844, 0x5844, PDF_CMAP_SINGLE, 8896 },
	{ 0x5845, 0x5846, PDF_CMAP_RANGE, 4731 },
	{ 0x5847, 0x5849, PDF_CMAP_TABLE, 468 },
	{ 0x584a, 0x584b, PDF_CMAP_RANGE, 4736 },
	{ 0x584c, 0x584d, PDF_CMAP_RANGE, 8899 },
	{ 0x584e, 0x584f, PDF_CMAP_RANGE, 4740 },
	{ 0x5850, 0x5851, PDF_CMAP_RANGE, 8901 },
	{ 0x5852, 0x5853, PDF_CMAP_TABLE, 471 },
	{ 0x5854, 0x5858, PDF_CMAP_RANGE, 4746 },
	{ 0x5859, 0x585a, PDF_CMAP_TABLE, 473 },
	{ 0x585b, 0x585c, PDF_CMAP_RANGE, 8905 },
	{ 0x585d, 0x5870, PDF_CMAP_RANGE, 4755 },
	{ 0x5871, 0x5871, PDF_CMAP_SINGLE, 8907 },
	{ 0x5872, 0x5875, PDF_CMAP_RANGE, 4776 },
	{ 0x5876, 0x5877, PDF_CMAP_RANGE, 8908 },
	{ 0x5878, 0x587e, PDF_CMAP_RANGE, 4782 },
	{ 0x5921, 0x592c, PDF_CMAP_RANGE, 4789 },
	{ 0x592d, 0x5930, PDF_CMAP_TABLE, 475 },
	{ 0x5931, 0x5933, PDF_CMAP_RANGE, 8912 },
	{ 0x5934, 0x5946, PDF_CMAP_RANGE, 4808 },
	{ 0x5947, 0x5947, PDF_CMAP_SINGLE, 8915 },
	{ 0x5948, 0x594c, PDF_CMAP_RANGE, 4828 },
	{ 0x594d, 0x5950, PDF_CMAP_RANGE, 8916 },
	{ 0x5951, 0x595c, PDF_CMAP_RANGE, 4837 },
	{ 0x595d, 0x595d, PDF_CMAP_SINGLE, 8920 },
	{ 0x595e, 0x5960, PDF_CMAP_RANGE, 4850 },
	{ 0x5961, 0x5961, PDF_CMAP_SINGLE, 8921 },
	{ 0x5962, 0x5963, PDF_CMAP_RANGE, 4854 },
	{ 0x5964, 0x5966, PDF_CMAP_TABLE, 479 },
	{ 0x5967, 0x596b, PDF_CMAP_RANGE, 4859 },
	{ 0x596c, 0x596c, PDF_CMAP_SINGLE, 8924 },
	{ 0x596d, 0x5973, PDF_CMAP_RANGE, 4865 },
	{ 0x5974, 0x5975, PDF_CMAP_RANGE, 8925 },
	{ 0x5976, 0x597e, PDF_CMAP_RANGE, 4874 },
	{ 0x5a21, 0x5a24, PDF_CMAP_RANGE, 4883 },
	{ 0x5a25, 0x5a5f, PDF_CMAP_RANGE, 8927 },
	{ 0x5a60, 0x5a69, PDF_CMAP_RANGE, 4946 },
	{ 0x5a6a, 0x5a6a, PDF_CMAP_SINGLE, 8986 },
	{ 0x5a6b, 0x5a76, PDF_CMAP_RANGE, 4957 },
	{ 0x5a77, 0x5a79, PDF_CMAP_TABLE, 482 },
	{ 0x5a7a, 0x5a7d, PDF_CMAP_RANGE, 4972 },
	{ 0x5a7e, 0x5a7e, PDF_CMAP_SINGLE, 8989 },
	{ 0x5b21, 0x5b22, PDF_CMAP_RANGE, 4977 },
	{ 0x5b23, 0x5b23, PDF_CMAP_SINGLE, 8990 },
	{ 0x5b24, 0x5b25, PDF_CMAP_RANGE, 4980 },
	{ 0x5b26, 0x5b26, PDF_CMAP_SINGLE, 8991 },
	{ 0x5b27, 0x5b28, PDF_CMAP_RANGE, 4983 },
	{ 0x5b29, 0x5b2a, PDF_CMAP_RANGE, 8992 },
	{ 0x5b2b, 0x5b3a, PDF_CMAP_RANGE, 4987 },
	{ 0x5b3b, 0x5b3d, PDF_CMAP_TABLE, 485 },
	{ 0x5b3e, 0x5b4e, PDF_CMAP_RANGE, 5006 },
	{ 0x5b4f, 0x5b51, PDF_CMAP_TABLE, 488 },
	{ 0x5b52, 0x5b5a, PDF_CMAP_RANGE, 5026 },
	{ 0x5b5b, 0x5b5b, PDF_CMAP_SINGLE, 8998 },
	{ 0x5b5c, 0x5b5d, PDF_CMAP_RANGE, 5036 },
	{ 0x5b5e, 0x5b5e, PDF_CMAP_SINGLE, 8999 },
	{ 0x5b5f, 0x5b61, PDF_CMAP_RANGE, 5039 },
	{ 0x5b62, 0x5b64, PDF_CMAP_TABLE, 491 },
	{ 0x5b65, 0x5b6a, PDF_CMAP_RANGE, 5045 },
	{ 0x5b6b, 0x5b6b, PDF_CMAP_SINGLE, 9002 },
	{ 0x5b6c, 0x5b6d, PDF_CMAP_RANGE, 5052 },
	{ 0x5b6e, 0x5b6e, PDF_CMAP_SINGLE, 9003 },
	{ 0x5b6f, 0x5b70, PDF_CMAP_RANGE, 5055 },
	{ 0x5b71, 0x5b71, PDF_CMAP_SINGLE, 9004 },
	{ 0x5b72, 0x5b74, PDF_CMAP_RANGE, 5058 },
	{ 0x5b75, 0x5b77, PDF_CMAP_RANGE, 9005 },
	{ 0x5b78, 0x5b7e, PDF_CMAP_RANGE, 5064 },
	{ 0x5c21, 0x5c3b, PDF_CMAP_RANGE, 5071 },
	{ 0x5c3c, 0x5c3c, PDF_CMAP_SINGLE, 9008 },
	{ 0x5c3d, 0x5c3e, PDF_CMAP_RANGE, 5099 },
	{ 0x5c3f, 0x5c3f, PDF_CMAP_SINGLE, 9009 },
	{ 0x5c40, 0x5c41, PDF_CMAP_RANGE, 5102 },
	{ 0x5c42, 0x5c42, PDF_CMAP_SINGLE, 9010 },
	{ 0x5c43, 0x5c47, PDF_CMAP_RANGE, 5105 },
	{ 0x5c48, 0x5c4a, PDF_CMAP_RANGE, 9011 },
	{ 0x5c4b, 0x5c50, PDF_CMAP_RANGE, 5113 },
	{ 0x5c51, 0x5c51, PDF_CMAP_SINGLE, 9014 },
	{ 0x5c52, 0x5c56, PDF_CMAP_RANGE, 5120 },
	{ 0x5c57, 0x5c57, PDF_CMAP_SINGLE, 9015 },
	{ 0x5c58, 0x5c5f, PDF_CMAP_RANGE, 5126 },
	{ 0x5c60, 0x5c60, PDF_CMAP_SINGLE, 9016 },
	{ 0x5c61, 0x5c62, PDF_CMAP_RANGE, 5135 },
	{ 0x5c63, 0x5c64, PDF_CMAP_RANGE, 9017 },
	{ 0x5c65, 0x5c68, PDF_CMAP_RANGE, 5139 },
	{ 0x5c69, 0x5c6a, PDF_CMAP_RANGE, 9019 },
	{ 0x5c6b, 0x5c70, PDF_CMAP_RANGE, 5145 },
	{ 0x5c71, 0x5c71, PDF_CMAP_SINGLE, 9021 },
	{ 0x5c72, 0x5c75, PDF_CMAP_RANGE, 5152 },
	{ 0x5c76, 0x5c76, PDF_CMAP_SINGLE, 9022 },
	{ 0x5c77, 0x5c78, PDF_CMAP_RANGE, 5157 },
	{ 0x5c79, 0x5c79, PDF_CMAP_SINGLE, 9023 },
	{ 0x5c7a, 0x5c7c, PDF_CMAP_RANGE, 5160 },
	{ 0x5c7d, 0x5c7e, PDF_CMAP_RANGE, 9024 },
	{ 0x5d21, 0x5d22, PDF_CMAP_TABLE, 494 },
	{ 0x5d23, 0x5d27, PDF_CMAP_RANGE, 9027 },
	{ 0x5d28, 0x5d29, PDF_CMAP_RANGE, 5172 },
	{ 0x5d2a, 0x5d2b, PDF_CMAP_RANGE, 9032 },
	{ 0x5d2c, 0x5d31, PDF_CMAP_RANGE, 5176 },
	{ 0x5d32, 0x5d32, PDF_CMAP_SINGLE, 9034 },
	{ 0x5d33, 0x5d34, PDF_CMAP_RANGE, 5183 },
	{ 0x5d35, 0x5d35, PDF_CMAP_SINGLE, 9035 },
	{ 0x5d36, 0x5d39, PDF_CMAP_RANGE, 5186 },
	{ 0x5d3a, 0x5d3b, PDF_CMAP_RANGE, 9036 },
	{ 0x5d3c, 0x5d52, PDF_CMAP_RANGE, 5192 },
	{ 0x5d53, 0x5d53, PDF_CMAP_SINGLE, 9038 },
	{ 0x5d54, 0x5d5a, PDF_CMAP_RANGE, 5216 },
	{ 0x5d5b, 0x5d5b, PDF_CMAP_SINGLE, 9039 },
	{ 0x5d5c, 0x5d5d, PDF_CMAP_RANGE, 5224 },
	{ 0x5d5e, 0x5d5e, PDF_CMAP_SINGLE, 9040 },
	{ 0x5d5f, 0x5d63, PDF_CMAP_RANGE, 5227 },
	{ 0x5d64, 0x5d64, PDF_CMAP_SINGLE, 9041 },
	{ 0x5d65, 0x5d6a, PDF_CMAP_RANGE, 5233 },
	{ 0x5d6b, 0x5d6b, PDF_CMAP_SINGLE, 9042 },
	{ 0x5d6c, 0x5d70, PDF_CMAP_RANGE, 5240 },
	{ 0x5d71, 0x5d71, PDF_CMAP_SINGLE, 9043 },
	{ 0x5d72, 0x5d75, PDF_CMAP_RANGE, 5246 },
	{ 0x5d76, 0x5d77, PDF_CMAP_RANGE, 9044 },
	{ 0x5d78, 0x5d7b, PDF_CMAP_RANGE, 5252 },
	{ 0x5d7c, 0x5d7e, PDF_CMAP_TABLE, 496 },
	{ 0x5e21, 0x5e2c, PDF_CMAP_RANGE, 5259 },
	{ 0x5e2d, 0x5e2d, PDF_CMAP_SINGLE, 9048 },
	{ 0x5e2e, 0x5e33, PDF_CMAP_RANGE, 5272 },
	{ 0x5e34, 0x5e34, PDF_CMAP_SINGLE, 9049 },
	{ 0x5e35, 0x5e39, PDF_CMAP_RANGE, 5279 },
	{ 0x5e3a, 0x5e3a, PDF_CMAP_SINGLE, 9050 },
	{ 0x5e3b, 0x5e45, PDF_CMAP_RANGE, 5285 },
	{ 0x5e46, 0x5e46, PDF_CMAP_SINGLE, 9051 },
	{ 0x5e47, 0x5e4e, PDF_CMAP_RANGE, 5297 },
	{ 0x5e4f, 0x5e50, PDF_CMAP_TABLE, 499 },
	{ 0x5e51, 0x5e52, PDF_CMAP_RANGE, 9053 },
	{ 0x5e53, 0x5e57, PDF_CMAP_RANGE, 5309 },
	{ 0x5e58, 0x5e58, PDF_CMAP_SINGLE, 9055 },
	{ 0x5e59, 0x5e61, PDF_CMAP_RANGE, 5315 },
	{ 0x5e62, 0x5e62, PDF_CMAP_SINGLE, 9056 },
	{ 0x5e63, 0x5e67, PDF_CMAP_RANGE, 5325 },
	{ 0x5e68, 0x5e68, PDF_CMAP_SINGLE, 9057 },
	{ 0x5e69, 0x5e6b, PDF_CMAP_RANGE, 5331 },
	{ 0x5e6c, 0x5e6c, PDF_CMAP_SINGLE, 9058 },
	{ 0x5e6d, 0x5e72, PDF_CMAP_RANGE, 5335 },
	{ 0x5e73, 0x5e73, PDF_CMAP_SINGLE, 9059 },
	{ 0x5e74, 0x5e7b, PDF_CMAP_RANGE, 5342 },
	{ 0x5e7c, 0x5e7c, PDF_CMAP_SINGLE, 9060 },
	{ 0x5e7d, 0x5e7e, PDF_CMAP_RANGE, 5351 },
	{ 0x5f21, 0x5f21, PDF_CMAP_SINGLE, 5353 },
	{ 0x5f22, 0x5f23, PDF_CMAP_RANGE, 9061 },
	{ 0x5f24, 0x5f25, PDF_CMAP_TABLE, 501 },
	{ 0x5f26, 0x5f33, PDF_CMAP_RANGE, 5358 },
	{ 0x5f34, 0x5f34, PDF_CMAP_SINGLE, 9064 },
	{ 0x5f35, 0x5f3b, PDF_CMAP_RANGE, 5373 },
	{ 0x5f3c, 0x5f3d, PDF_CMAP_RANGE, 9065 },
	{ 0x5f3e, 0x5f3f, PDF_CMAP_TABLE, 503 },
	{ 0x5f40, 0x5f41, PDF_CMAP_RANGE, 5384 },
	{ 0x5f42, 0x5f43, PDF_CMAP_RANGE, 9068 },
	{ 0x5f44, 0x5f4b, PDF_CMAP_RANGE, 5388 },
	{ 0x5f4c, 0x5f4c, PDF_CMAP_SINGLE, 9070 },
	{ 0x5f4d, 0x5f4f, PDF_CMAP_RANGE, 5397 },
	{ 0x5f50, 0x5f50, PDF_CMAP_SINGLE, 9071 },
	{ 0x5f51, 0x5f54, PDF_CMAP_RANGE, 5401 },
	{ 0x5f55, 0x5f55, PDF_CMAP_SINGLE, 9072 },
	{ 0x5f56, 0x5f57, PDF_CMAP_RANGE, 5406 },
	{ 0x5f58, 0x5f59, PDF_CMAP_RANGE, 9073 },
	{ 0x5f5a, 0x5f5b, PDF_CMAP_RANGE, 5410 },
	{ 0x5f5c, 0x5f5c, PDF_CMAP_SINGLE, 9075 },
	{ 0x5f5d, 0x5f5f, PDF_CMAP_RANGE, 5413 },
	{ 0x5f60, 0x5f62, PDF_CMAP_TABLE, 505 },
	{ 0x5f63, 0x5f65, PDF_CMAP_RANGE, 5419 },
	{ 0x5f66, 0x5f66, PDF_CMAP_SINGLE, 9078 },
	{ 0x5f67, 0x5f68, PDF_CMAP_RANGE, 5423 },
	{ 0x5f69, 0x5f6b, PDF_CMAP_TABLE, 508 },
	{ 0x5f6c, 0x5f6e, PDF_CMAP_RANGE, 5428 },
	{ 0x5f6f, 0x5f6f, PDF_CMAP_SINGLE, 9081 },
	{ 0x5f70, 0x5f74, PDF_CMAP_RANGE, 5432 },
	{ 0x5f75, 0x5f75, PDF_CMAP_SINGLE, 9082 },
	{ 0x5f76, 0x5f78, PDF_CMAP_RANGE, 5438 },
	{ 0x5f79, 0x5f79, PDF_CMAP_SINGLE, 9083 },
	{ 0x5f7a, 0x5f7e, PDF_CMAP_RANGE, 5442 },
	{ 0x6021, 0x6035, PDF_CMAP_RANGE, 5447 },
	{ 0x6036, 0x6037, PDF_CMAP_RANGE, 9084 },
	{ 0x6038, 0x603e, PDF_CMAP_RANGE, 5470 },
	{ 0x603f, 0x603f, PDF_CMAP_SINGLE, 9086 },
	{ 0x6040, 0x6047, PDF_CMAP_RANGE, 5478 },
	{ 0x6048, 0x6048, PDF_CMAP_SINGLE, 9087 },
	{ 0x6049, 0x604d, PDF_CMAP_RANGE, 5487 },
	{ 0x604e, 0x604e, PDF_CMAP_SINGLE, 9088 },
	{ 0x604f, 0x6052, PDF_CMAP_RANGE, 5493 },
	{ 0x6053, 0x6053, PDF_CMAP_SINGLE, 9089 },
	{ 0x6054, 0x605f, PDF_CMAP_RANGE, 5498 },
	{ 0x6060, 0x6060, PDF_CMAP_SINGLE, 9090 },
	{ 0x6061, 0x606f, PDF_CMAP_RANGE, 5511 },
	{ 0x6070, 0x6070, PDF_CMAP_SINGLE, 9091 },
	{ 0x6071, 0x6077, PDF_CMAP_RANGE, 5527 },
	{ 0x6078, 0x6078, PDF_CMAP_SINGLE, 9092 },
	{ 0x6079, 0x607b, PDF_CMAP_RANGE, 5535 },
	{ 0x607c, 0x607e, PDF_CMAP_RANGE, 9093 },
	{ 0x6121, 0x612a, PDF_CMAP_RANGE, 5541 },
	{ 0x612b, 0x612d, PDF_CMAP_TABLE, 511 },
	{ 0x612e, 0x612f, PDF_CMAP_RANGE, 5554 },
	{ 0x6130, 0x6130, PDF_CMAP_SINGLE, 9098 },
	{ 0x6131, 0x6133, PDF_CMAP_RANGE, 5557 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 9099 },
	{ 0x6135, 0x613a, PDF_CMAP_RANGE, 5561 },
	{ 0x613b, 0x613d, PDF_CMAP_TABLE, 514 },
	{ 0x613e, 0x613f, PDF_CMAP_RANGE, 5570 },
	{ 0x6140, 0x6141, PDF_CMAP_RANGE, 9102 },
	{ 0x6142, 0x6148, PDF_CMAP_RANGE, 5574 },
	{ 0x6149, 0x6149, PDF_CMAP_SINGLE, 9104 },
	{ 0x614a, 0x614f, PDF_CMAP_RANGE, 5582 },
	{ 0x6150, 0x6150, PDF_CMAP_SINGLE, 9105 },
	{ 0x6151, 0x615a, PDF_CMAP_RANGE, 5589 },
	{ 0x615b, 0x615b, PDF_CMAP_SINGLE, 9106 },
	{ 0x615c, 0x6160, PDF_CMAP_RANGE, 5600 },
	{ 0x6161, 0x6162, PDF_CMAP_TABLE, 517 },
	{ 0x6163, 0x6166, PDF_CMAP_RANGE, 5607 },
	{ 0x6167, 0x6167, PDF_CMAP_SINGLE, 4504 },
	{ 0x6168, 0x616d, PDF_CMAP_RANGE, 5612 },
	{ 0x616e, 0x616f, PDF_CMAP_RANGE, 9108 },
	{ 0x6170, 0x6175, PDF_CMAP_RANGE, 5620 },
	{ 0x6176, 0x6178, PDF_CMAP_TABLE, 519 },
	{ 0x6179, 0x617c, PDF_CMAP_RANGE, 5629 },
	{ 0x617d, 0x617e, PDF_CMAP_TABLE, 522 },
	{ 0x6221, 0x6223, PDF_CMAP_RANGE, 5635 },
	{ 0x6224, 0x6224, PDF_CMAP_SINGLE, 9113 },
	{ 0x6225, 0x6227, PDF_CMAP_RANGE, 5639 },
	{ 0x6228, 0x6228, PDF_CMAP_SINGLE, 9114 },
	{ 0x6229, 0x623a, PDF_CMAP_RANGE, 5643 },
	{ 0x623b, 0x6244, PDF_CMAP_RANGE, 9115 },
	{ 0x6245, 0x6245, PDF_CMAP_SINGLE, 4293 },
	{ 0x6246, 0x624e, PDF_CMAP_RANGE, 9125 },
	{ 0x624f, 0x6250, PDF_CMAP_TABLE, 524 },
	{ 0x6251, 0x6258, PDF_CMAP_RANGE, 5683 },
	{ 0x6259, 0x6259, PDF_CMAP_SINGLE, 9135 },
	{ 0x625a, 0x6262, PDF_CMAP_RANGE, 5692 },
	{ 0x6263, 0x6264, PDF_CMAP_RANGE, 9136 },
	{ 0x6265, 0x6266, PDF_CMAP_TABLE, 526 },
	{ 0x6267, 0x6268, PDF_CMAP_RANGE, 5705 },
	{ 0x6269, 0x626b, PDF_CMAP_RANGE, 9139 },
	{ 0x626c, 0x6277, PDF_CMAP_RANGE, 5710 },
	{ 0x6278, 0x6279, PDF_CMAP_TABLE, 528 },
	{ 0x627a, 0x627d, PDF_CMAP_RANGE, 9143 },
	{ 0x627e, 0x627e, PDF_CMAP_SINGLE, 5728 },
	{ 0x6321, 0x6322, PDF_CMAP_TABLE, 530 },
	{ 0x6323, 0x6324, PDF_CMAP_RANGE, 5731 },
	{ 0x6325, 0x6325, PDF_CMAP_SINGLE, 9148 },
	{ 0x6326, 0x632a, PDF_CMAP_RANGE, 5734 },
	{ 0x632b, 0x632b, PDF_CMAP_SINGLE, 9149 },
	{ 0x632c, 0x6333, PDF_CMAP_RANGE, 5740 },
	{ 0x6334, 0x6334, PDF_CMAP_SINGLE, 9150 },
	{ 0x6335, 0x6344, PDF_CMAP_RANGE, 5749 },
	{ 0x6345, 0x635b, PDF_CMAP_RANGE, 9151 },
	{ 0x635c, 0x6362, PDF_CMAP_RANGE, 5788 },
	{ 0x6363, 0x6363, PDF_CMAP_SINGLE, 9174 },
	{ 0x6364, 0x636c, PDF_CMAP_RANGE, 5796 },
	{ 0x636d, 0x636d, PDF_CMAP_SINGLE, 9175 },
	{ 0x636e, 0x6370, PDF_CMAP_RANGE, 5806 },
	{ 0x6371, 0x6372, PDF_CMAP_RANGE, 9176 },
	{ 0x6373, 0x6377, PDF_CMAP_RANGE, 5811 },
	{ 0x6378, 0x6378, PDF_CMAP_SINGLE, 9178 },
	{ 0x6379, 0x637d, PDF_CMAP_RANGE, 5817 },
	{ 0x637e, 0x637e, PDF_CMAP_SINGLE, 9179 },
	{ 0x6421, 0x6423, PDF_CMAP_RANGE, 5823 },
	{ 0x6424, 0x6425, PDF_CMAP_RANGE, 9180 },
	{ 0x6426, 0x642a, PDF_CMAP_RANGE, 5828 },
	{ 0x642b, 0x642b, PDF_CMAP_SINGLE, 9182 },
	{ 0x642c, 0x642e, PDF_CMAP_RANGE, 5834 },
	{ 0x642f, 0x6431, PDF_CMAP_RANGE, 9183 },
	{ 0x6432, 0x6434, PDF_CMAP_RANGE, 5840 },
	{ 0x6435, 0x6436, PDF_CMAP_RANGE, 9186 },
	{ 0x6437, 0x6441, PDF_CMAP_RANGE, 5845 },
	{ 0x6442, 0x6442, PDF_CMAP_SINGLE, 9188 },
	{ 0x6443, 0x6444, PDF_CMAP_RANGE, 5857 },
	{ 0x6445, 0x6445, PDF_CMAP_SINGLE, 9189 },
	{ 0x6446, 0x6448, PDF_CMAP_RANGE, 5860 },
	{ 0x6449, 0x6449, PDF_CMAP_SINGLE, 9190 },
	{ 0x644a, 0x6458, PDF_CMAP_RANGE, 5864 },
	{ 0x6459, 0x6459, PDF_CMAP_SINGLE, 9191 },
	{ 0x645a, 0x645b, PDF_CMAP_RANGE, 5880 },
	{ 0x645c, 0x645e, PDF_CMAP_TABLE, 532 },
	{ 0x645f, 0x6463, PDF_CMAP_RANGE, 5885 },
	{ 0x6464, 0x6464, PDF_CMAP_SINGLE, 9194 },
	{ 0x6465, 0x646a, PDF_CMAP_RANGE, 5891 },
	{ 0x646b, 0x646c, PDF_CMAP_RANGE, 9195 },
	{ 0x646d, 0x6471, PDF_CMAP_RANGE, 5899 },
	{ 0x6472, 0x6472, PDF_CMAP_SINGLE, 9197 },
	{ 0x6473, 0x647d, PDF_CMAP_RANGE, 5905 },
	{ 0x647e, 0x647e, PDF_CMAP_SINGLE, 9198 },
	{ 0x6521, 0x652f, PDF_CMAP_RANGE, 5917 },
	{ 0x6530, 0x6530, PDF_CMAP_SINGLE, 9199 },
	{ 0x6531, 0x6538, PDF_CMAP_RANGE, 5933 },
	{ 0x6539, 0x6539, PDF_CMAP_SINGLE, 9200 },
	{ 0x653a, 0x6546, PDF_CMAP_RANGE, 5942 },
	{ 0x6547, 0x6549, PDF_CMAP_TABLE, 535 },
	{ 0x654a, 0x654d, PDF_CMAP_RANGE, 5958 },
	{ 0x654e, 0x654e, PDF_CMAP_SINGLE, 9203 },
	{ 0x654f, 0x656f, PDF_CMAP_RANGE, 5963 },
	{ 0x6570, 0x6572, PDF_CMAP_TABLE, 538 },
	{ 0x6573, 0x657b, PDF_CMAP_RANGE, 5999 },
	{ 0x657c, 0x657d, PDF_CMAP_RANGE, 9206 },
	{ 0x657e, 0x657e, PDF_CMAP_SINGLE, 6010 },
	{ 0x6621, 0x6622, PDF_CMAP_RANGE, 6011 },
	{ 0x6623, 0x6623, PDF_CMAP_SINGLE, 9208 },
	{ 0x6624, 0x662a, PDF_CMAP_RANGE, 6014 },
	{ 0x662b, 0x662c, PDF_CMAP_RANGE, 9209 },
	{ 0x662d, 0x662e, PDF_CMAP_TABLE, 541 },
	{ 0x662f, 0x6633, PDF_CMAP_RANGE, 6025 },
	{ 0x6634, 0x6635, PDF_CMAP_RANGE, 9212 },
	{ 0x6636, 0x663e, PDF_CMAP_RANGE, 6032 },
	{ 0x663f, 0x663f, PDF_CMAP_SINGLE, 9214 },
	{ 0x6640, 0x6647, PDF_CMAP_RANGE, 6042 },
	{ 0x6648, 0x6649, PDF_CMAP_RANGE, 9215 },
	{ 0x664a, 0x664c, PDF_CMAP_RANGE, 6052 },
	{ 0x664d, 0x664d, PDF_CMAP_SINGLE, 9217 },
	{ 0x664e, 0x665f, PDF_CMAP_RANGE, 6056 },
	{ 0x6660, 0x667e, PDF_CMAP_RANGE, 9218 },
	{ 0x6721, 0x675a, PDF_CMAP_RANGE, 9249 },
	{ 0x675b, 0x6760, PDF_CMAP_RANGE, 6163 },
	{ 0x6761, 0x6762, PDF_CMAP_RANGE, 9307 },
	{ 0x6763, 0x6766, PDF_CMAP_RANGE, 6171 },
	{ 0x6767, 0x6767, PDF_CMAP_SINGLE, 9309 },
	{ 0x6768, 0x676e, PDF_CMAP_RANGE, 6176 },
	{ 0x676f, 0x676f, PDF_CMAP_SINGLE, 9310 },
	{ 0x6770, 0x6773, PDF_CMAP_RANGE, 6184 },
	{ 0x6774, 0x6776, PDF_CMAP_RANGE, 9311 },
	{ 0x6777, 0x677e, PDF_CMAP_RANGE, 6191 },
	{ 0x6821, 0x6827, PDF_CMAP_RANGE, 6199 },
	{ 0x6828, 0x6828, PDF_CMAP_SINGLE, 9314 },
	{ 0x6829, 0x682b, PDF_CMAP_RANGE, 6207 },
	{ 0x682c, 0x682c, PDF_CMAP_SINGLE, 9315 },
	{ 0x682d, 0x6835, PDF_CMAP_RANGE, 6211 },
	{ 0x6836, 0x6837, PDF_CMAP_TABLE, 543 },
	{ 0x6838, 0x683a, PDF_CMAP_RANGE, 9317 },
	{ 0x683b, 0x683e, PDF_CMAP_RANGE, 6225 },
	{ 0x683f, 0x6840, PDF_CMAP_RANGE, 9320 },
	{ 0x6841, 0x6844, PDF_CMAP_RANGE, 6231 },
	{ 0x6845, 0x6846, PDF_CMAP_TABLE, 545 },
	{ 0x6847, 0x6849, PDF_CMAP_RANGE, 9323 },
	{ 0x684a, 0x684d, PDF_CMAP_RANGE, 6240 },
	{ 0x684e, 0x6850, PDF_CMAP_TABLE, 547 },
	{ 0x6851, 0x6852, PDF_CMAP_RANGE, 6247 },
	{ 0x6853, 0x6853, PDF_CMAP_SINGLE, 9328 },
	{ 0x6854, 0x685c, PDF_CMAP_RANGE, 6250 },
	{ 0x685d, 0x685f, PDF_CMAP_TABLE, 550 },
	{ 0x6860, 0x6861, PDF_CMAP_RANGE, 6262 },
	{ 0x6862, 0x6863, PDF_CMAP_RANGE, 9331 },
	{ 0x6864, 0x6867, PDF_CMAP_TABLE, 553 },
	{ 0x6868, 0x686a, PDF_CMAP_RANGE, 6270 },
	{ 0x686b, 0x686f, PDF_CMAP_TABLE, 557 },
	{ 0x6870, 0x6878, PDF_CMAP_RANGE, 6278 },
	{ 0x6879, 0x6879, PDF_CMAP_SINGLE, 9338 },
	{ 0x687a, 0x687b, PDF_CMAP_RANGE, 6288 },
	{ 0x687c, 0x687d, PDF_CMAP_RANGE, 9339 },
	{ 0x687e, 0x687e, PDF_CMAP_SINGLE, 6292 },
	{ 0x6921, 0x6921, PDF_CMAP_SINGLE, 9341 },
	{ 0x6922, 0x692c, PDF_CMAP_RANGE, 6294 },
	{ 0x692d, 0x692d, PDF_CMAP_SINGLE, 9342 },
	{ 0x692e, 0x6933, PDF_CMAP_RANGE, 6306 },
	{ 0x6934, 0x6935, PDF_CMAP_RANGE, 9343 },
	{ 0x6936, 0x6937, PDF_CMAP_TABLE, 562 },
	{ 0x6938, 0x6943, PDF_CMAP_RANGE, 6316 },
	{ 0x6944, 0x6946, PDF_CMAP_TABLE, 564 },
	{ 0x6947, 0x6948, PDF_CMAP_RANGE, 6331 },
	{ 0x6949, 0x6949, PDF_CMAP_SINGLE, 9348 },
	{ 0x694a, 0x6955, PDF_CMAP_RANGE, 6334 },
	{ 0x6956, 0x6956, PDF_CMAP_SINGLE, 9349 },
	{ 0x6957, 0x6959, PDF_CMAP_RANGE, 6347 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 9350 },
	{ 0x695b, 0x6963, PDF_CMAP_RANGE, 6351 },
	{ 0x6964, 0x6965, PDF_CMAP_TABLE, 567 },
	{ 0x6966, 0x6967, PDF_CMAP_RANGE, 9352 },
	{ 0x6968, 0x696c, PDF_CMAP_TABLE, 569 },
	{ 0x696d, 0x697e, PDF_CMAP_RANGE, 9356 },
	{ 0x6a21, 0x6a25, PDF_CMAP_RANGE, 9374 },
	{ 0x6a26, 0x6a26, PDF_CMAP_SINGLE, 6392 },
	{ 0x6a27, 0x6a28, PDF_CMAP_RANGE, 9379 },
	{ 0x6a29, 0x6a30, PDF_CMAP_RANGE, 6395 },
	{ 0x6a31, 0x6a31, PDF_CMAP_SINGLE, 9381 },
	{ 0x6a32, 0x6a3b, PDF_CMAP_RANGE, 6404 },
	{ 0x6a3c, 0x6a3c, PDF_CMAP_SINGLE, 9382 },
	{ 0x6a3d, 0x6a49, PDF_CMAP_RANGE, 6415 },
	{ 0x6a4a, 0x6a4a, PDF_CMAP_SINGLE, 9383 },
	{ 0x6a4b, 0x6a4c, PDF_CMAP_RANGE, 6429 },
	{ 0x6a4d, 0x6a4d, PDF_CMAP_SINGLE, 9384 },
	{ 0x6a4e, 0x6a52, PDF_CMAP_RANGE, 6432 },
	{ 0x6a53, 0x6a53, PDF_CMAP_SINGLE, 9385 },
	{ 0x6a54, 0x6a59, PDF_CMAP_RANGE, 6438 },
	{ 0x6a5a, 0x6a6f, PDF_CMAP_RANGE, 9386 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6466 },
	{ 0x6b21, 0x6b26, PDF_CMAP_RANGE, 6481 },
	{ 0x6b27, 0x6b27, PDF_CMAP_SINGLE, 9408 },
	{ 0x6b28, 0x6b29, PDF_CMAP_RANGE, 6488 },
	{ 0x6b2a, 0x6b2a, PDF_CMAP_SINGLE, 9409 },
	{ 0x6b2b, 0x6b31, PDF_CMAP_RANGE, 6491 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 9410 },
	{ 0x6b33, 0x6b38, PDF_CMAP_RANGE, 6499 },
	{ 0x6b39, 0x6b39, PDF_CMAP_SINGLE, 9411 },
	{ 0x6b3a, 0x6b49, PDF_CMAP_RANGE, 6506 },
	{ 0x6b4a, 0x6b4b, PDF_CMAP_RANGE, 9412 },
	{ 0x6b4c, 0x6b4d, PDF_CMAP_TABLE, 574 },
	{ 0x6b4e, 0x6b55, PDF_CMAP_RANGE, 6526 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 9415 },
	{ 0x6b57, 0x6b59, PDF_CMAP_RANGE, 6535 },
	{ 0x6b5a, 0x6b5a, PDF_CMAP_SINGLE, 9416 },
	{ 0x6b5b, 0x6b60, PDF_CMAP_RANGE, 6539 },
	{ 0x6b61, 0x6b61, PDF_CMAP_SINGLE, 9417 },
	{ 0x6b62, 0x6b76, PDF_CMAP_RANGE, 6546 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 9418 },
	{ 0x6b78, 0x6b7e, PDF_CMAP_RANGE, 6568 },
	{ 0x6c21, 0x6c22, PDF_CMAP_RANGE, 6575 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 9419 },
	{ 0x6c24, 0x6c28, PDF_CMAP_RANGE, 6578 },
	{ 0x6c29, 0x6c2e, PDF_CMAP_RANGE, 9420 },
	{ 0x6c2f, 0x6c30, PDF_CMAP_RANGE, 6589 },
	{ 0x6c31, 0x6c31, PDF_CMAP_SINGLE, 9426 },
	{ 0x6c32, 0x6c33, PDF_CMAP_RANGE, 6592 },
	{ 0x6c34, 0x6c35, PDF_CMAP_RANGE, 9427 },
	{ 0x6c36, 0x6c3d, PDF_CMAP_RANGE, 6596 },
	{ 0x6c3e, 0x6c3f, PDF_CMAP_RANGE, 9429 },
	{ 0x6c40, 0x6c41, PDF_CMAP_TABLE, 576 },
	{ 0x6c42, 0x6c46, PDF_CMAP_RANGE, 6608 },
	{ 0x6c47, 0x6c47, PDF_CMAP_SINGLE, 9432 },
	{ 0x6c48, 0x6c4a, PDF_CMAP_RANGE, 6614 },
	{ 0x6c4b, 0x6c4b, PDF_CMAP_SINGLE, 9433 },
	{ 0x6c4c, 0x6c61, PDF_CMAP_RANGE, 6618 },
	{ 0x6c62, 0x6c62, PDF_CMAP_SINGLE, 9434 },
	{ 0x6c63, 0x6c71, PDF_CMAP_RANGE, 6641 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 9435 },
	{ 0x6c73, 0x6c74, PDF_CMAP_RANGE, 6657 },
	{ 0x6c75, 0x6c75, PDF_CMAP_SINGLE, 9436 },
	{ 0x6c76, 0x6c77, PDF_CMAP_RANGE, 6660 },
	{ 0x6c78, 0x6c78, PDF_CMAP_SINGLE, 9437 },
	{ 0x6c79, 0x6c7e, PDF_CMAP_RANGE, 6663 },
	{ 0x6d21, 0x6d21, PDF_CMAP_SINGLE, 9438 },
	{ 0x6d22, 0x6d27, PDF_CMAP_RANGE, 6670 },
	{ 0x6d28, 0x6d28, PDF_CMAP_SINGLE, 9439 },
	{ 0x6d29, 0x6d2e, PDF_CMAP_RANGE, 6677 },
	{ 0x6d2f, 0x6d30, PDF_CMAP_RANGE, 9440 },
	{ 0x6d31, 0x6d33, PDF_CMAP_RANGE, 6685 },
	{ 0x6d34, 0x6d3a, PDF_CMAP_TABLE, 578 },
	{ 0x6d3b, 0x6d3e, PDF_CMAP_RANGE, 6695 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 9446 },
	{ 0x6d40, 0x6d41, PDF_CMAP_RANGE, 6700 },
	{ 0x6d42, 0x6d43, PDF_CMAP_RANGE, 9447 },
	{ 0x6d44, 0x6d4b, PDF_CMAP_RANGE, 6704 },
	{ 0x6d4c, 0x6d4d, PDF_CMAP_RANGE, 9449 },
	{ 0x6d4e, 0x6d52, PDF_CMAP_RANGE, 6714 },
	{ 0x6d53, 0x6d53, PDF_CMAP_SINGLE, 9451 },
	{ 0x6d54, 0x6d56, PDF_CMAP_RANGE, 6720 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 9452 },
	{ 0x6d58, 0x6d67, PDF_CMAP_RANGE, 6724 },
	{ 0x6d68, 0x6d68, PDF_CMAP_SINGLE, 9453 },
	{ 0x6d69, 0x6d6d, PDF_CMAP_RANGE, 6741 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 9454 },
	{ 0x6d6f, 0x6d78, PDF_CMAP_RANGE, 6747 },
	{ 0x6d79, 0x6d7a, PDF_CMAP_RANGE, 9455 },
	{ 0x6d7b, 0x6d7e, PDF_CMAP_RANGE, 6759 },
	{ 0x6e21, 0x6e3b, PDF_CMAP_RANGE, 6763 },
	{ 0x6e3c, 0x6e3c, PDF_CMAP_SINGLE, 9457 },
	{ 0x6e3d, 0x6e3e, PDF_CMAP_RANGE, 6791 },
	{ 0x6e3f, 0x6e3f, PDF_CMAP_SINGLE, 9458 },
	{ 0x6e40, 0x6e43, PDF_CMAP_RANGE, 6794 },
	{ 0x6e44, 0x6e7e, PDF_CMAP_RANGE, 9459 },
	{ 0x6f21, 0x6f71, PDF_CMAP_RANGE, 9518 },
	{ 0x6f72, 0x6f7e, PDF_CMAP_RANGE, 6938 },
	{ 0x7021, 0x7022, PDF_CMAP_RANGE, 6951 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 9599 },
	{ 0x7024, 0x702e, PDF_CMAP_RANGE, 6954 },
	{ 0x702f, 0x7059, PDF_CMAP_RANGE, 9600 },
	{ 0x705a, 0x705b, PDF_CMAP_RANGE, 7008 },
	{ 0x705c, 0x705d, PDF_CMAP_RANGE, 9643 },
	{ 0x705e, 0x705f, PDF_CMAP_TABLE, 585 },
	{ 0x7060, 0x7068, PDF_CMAP_RANGE, 7014 },
	{ 0x7069, 0x7069, PDF_CMAP_SINGLE, 9646 },
	{ 0x706a, 0x706b, PDF_CMAP_RANGE, 7024 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 9647 },
	{ 0x706d, 0x706e, PDF_CMAP_RANGE, 7027 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 9648 },
	{ 0x7070, 0x7076, PDF_CMAP_RANGE, 7030 },
	{ 0x7077, 0x7079, PDF_CMAP_TABLE, 587 },
	{ 0x707a, 0x707b, PDF_CMAP_RANGE, 7040 },
	{ 0x707c, 0x707c, PDF_CMAP_SINGLE, 9651 },
	{ 0x707d, 0x707e, PDF_CMAP_RANGE, 7043 },
	{ 0x7121, 0x7127, PDF_CMAP_RANGE, 7045 },
	{ 0x7128, 0x7128, PDF_CMAP_SINGLE, 9652 },
	{ 0x7129, 0x712a, PDF_CMAP_RANGE, 7053 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 9653 },
	{ 0x712c, 0x712d, PDF_CMAP_RANGE, 7056 },
	{ 0x712e, 0x712e, PDF_CMAP_SINGLE, 9654 },
	{ 0x712f, 0x7131, PDF_CMAP_RANGE, 7059 },
	{ 0x7132, 0x7132, PDF_CMAP_SINGLE, 9655 },
	{ 0x7133, 0x713b, PDF_CMAP_RANGE, 7063 },
	{ 0x713c, 0x713c, PDF_CMAP_SINGLE, 9656 },
	{ 0x713d, 0x713f, PDF_CMAP_RANGE, 7073 },
	{ 0x7140, 0x7140, PDF_CMAP_SINGLE, 9657 },
	{ 0x7141, 0x7148, PDF_CMAP_RANGE, 7077 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 9658 },
	{ 0x714a, 0x714c, PDF_CMAP_RANGE, 7086 },
	{ 0x714d, 0x714e, PDF_CMAP_TABLE, 590 },
	{ 0x714f, 0x7150, PDF_CMAP_RANGE, 9660 },
	{ 0x7151, 0x7159, PDF_CMAP_RANGE, 7093 },
	{ 0x715a, 0x715c, PDF_CMAP_TABLE, 592 },
	{ 0x715d, 0x7163, PDF_CMAP_RANGE, 7105 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 9664 },
	{ 0x7165, 0x716b, PDF_CMAP_RANGE, 7113 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 9665 },
	{ 0x716d, 0x716e, PDF_CMAP_RANGE, 7121 },
	{ 0x716f, 0x716f, PDF_CMAP_SINGLE, 9666 },
	{ 0x7170, 0x7176, PDF_CMAP_RANGE, 7124 },
	{ 0x7177, 0x7179, PDF_CMAP_TABLE, 595 },
	{ 0x717a, 0x717b, PDF_CMAP_RANGE, 7134 },
	{ 0x717c, 0x717e, PDF_CMAP_RANGE, 9669 },
	{ 0x7221, 0x722d, PDF_CMAP_RANGE, 9672 },
	{ 0x722e, 0x7230, PDF_CMAP_RANGE, 7152 },
	{ 0x7231, 0x7232, PDF_CMAP_RANGE, 9685 },
	{ 0x7233, 0x7238, PDF_CMAP_RANGE, 7157 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 9687 },
	{ 0x723a, 0x7242, PDF_CMAP_RANGE, 7164 },
	{ 0x7243, 0x7243, PDF_CMAP_SINGLE, 9688 },
	{ 0x7244, 0x7248, PDF_CMAP_RANGE, 7174 },
	{ 0x7249, 0x7249, PDF_CMAP_SINGLE, 9689 },
	{ 0x724a, 0x724b, PDF_CMAP_RANGE, 7180 },
	{ 0x724c, 0x724d, PDF_CMAP_RANGE, 9690 },
	{ 0x724e, 0x724f, PDF_CMAP_TABLE, 598 },
	{ 0x7250, 0x7252, PDF_CMAP_RANGE, 7186 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 9693 },
	{ 0x7254, 0x7264, PDF_CMAP_RANGE, 7190 },
	{ 0x7265, 0x7265, PDF_CMAP_SINGLE, 9694 },
	{ 0x7266, 0x726d, PDF_CMAP_RANGE, 7208 },
	{ 0x726e, 0x726e, PDF_CMAP_SINGLE, 9695 },
	{ 0x726f, 0x7276, PDF_CMAP_RANGE, 7217 },
	{ 0x7277, 0x7277, PDF_CMAP_SINGLE, 9696 },
	{ 0x7278, 0x727c, PDF_CMAP_RANGE, 7226 },
	{ 0x727d, 0x727e, PDF_CMAP_TABLE, 600 },
	{ 0x7321, 0x733e, PDF_CMAP_RANGE, 7233 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 9698 },
	{ 0x7340, 0x7345, PDF_CMAP_RANGE, 7264 },
	{ 0x7346, 0x7348, PDF_CMAP_TABLE, 602 },
	{ 0x7349, 0x7355, PDF_CMAP_RANGE, 7273 },
	{ 0x7356, 0x7356, PDF_CMAP_SINGLE, 9701 },
	{ 0x7357, 0x7358, PDF_CMAP_RANGE, 7287 },
	{ 0x7359, 0x7359, PDF_CMAP_SINGLE, 9702 },
	{ 0x735a, 0x7364, PDF_CMAP_RANGE, 7290 },
	{ 0x7365, 0x7366, PDF_CMAP_RANGE, 9703 },
	{ 0x7367, 0x7369, PDF_CMAP_RANGE, 7303 },
	{ 0x736a, 0x736c, PDF_CMAP_TABLE, 605 },
	{ 0x736d, 0x736e, PDF_CMAP_RANGE, 7309 },
	{ 0x736f, 0x7371, PDF_CMAP_TABLE, 608 },
	{ 0x7372, 0x737c, PDF_CMAP_RANGE, 7314 },
	{ 0x737d, 0x737e, PDF_CMAP_TABLE, 611 },
	{ 0x7421, 0x7424, PDF_CMAP_RANGE, 7327 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 9710 },
	{ 0x7426, 0x742e, PDF_CMAP_RANGE, 7332 },
	{ 0x742f, 0x742f, PDF_CMAP_SINGLE, 9711 },
	{ 0x7430, 0x7434, PDF_CMAP_RANGE, 7342 },
	{ 0x7435, 0x7435, PDF_CMAP_SINGLE, 9712 },
	{ 0x7436, 0x7440, PDF_CMAP_RANGE, 7348 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 9713 },
	{ 0x7442, 0x7446, PDF_CMAP_RANGE, 7360 },
	{ 0x7447, 0x7447, PDF_CMAP_SINGLE, 9714 },
	{ 0x7448, 0x744e, PDF_CMAP_RANGE, 7366 },
	{ 0x744f, 0x7450, PDF_CMAP_RANGE, 9715 },
	{ 0x7451, 0x7455, PDF_CMAP_RANGE, 7375 },
	{ 0x7456, 0x7456, PDF_CMAP_SINGLE, 9717 },
	{ 0x7457, 0x7469, PDF_CMAP_RANGE, 7381 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 9718 },
	{ 0x746b, 0x746e, PDF_CMAP_RANGE, 7401 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 9719 },
	{ 0x7470, 0x7474, PDF_CMAP_RANGE, 7406 },
	{ 0x7475, 0x7475, PDF_CMAP_SINGLE, 9720 },
	{ 0x7476, 0x747e, PDF_CMAP_RANGE, 7412 },
	{ 0x7521, 0x7525, PDF_CMAP_RANGE, 7421 },
	{ 0x7526, 0x7527, PDF_CMAP_RANGE, 9721 },
	{ 0x7528, 0x7539, PDF_CMAP_RANGE, 7428 },
	{ 0x753a, 0x753b, PDF_CMAP_RANGE, 9723 },
	{ 0x753c, 0x7543, PDF_CMAP_RANGE, 7448 },
	{ 0x7544, 0x7544, PDF_CMAP_SINGLE, 9725 },
	{ 0x7545, 0x7547, PDF_CMAP_RANGE, 7457 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 9726 },
	{ 0x7549, 0x754d, PDF_CMAP_RANGE, 7461 },
	{ 0x754e, 0x754f, PDF_CMAP_RANGE, 9727 },
	{ 0x7550, 0x7550, PDF_CMAP_SINGLE, 7468 },
	{ 0x7551, 0x7552, PDF_CMAP_RANGE, 9729 },
	{ 0x7553, 0x7558, PDF_CMAP_RANGE, 7471 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 9731 },
	{ 0x755a, 0x755b, PDF_CMAP_RANGE, 7478 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 9732 },
	{ 0x755d, 0x7565, PDF_CMAP_RANGE, 7481 },
	{ 0x7566, 0x7567, PDF_CMAP_RANGE, 9733 },
	{ 0x7568, 0x756e, PDF_CMAP_RANGE, 7492 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 9735 },
	{ 0x7570, 0x7571, PDF_CMAP_RANGE, 7500 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 9736 },
	{ 0x7573, 0x757b, PDF_CMAP_RANGE, 7503 },
	{ 0x757c, 0x757c, PDF_CMAP_SINGLE, 9737 },
	{ 0x757d, 0x757e, PDF_CMAP_RANGE, 7513 },
	{ 0x7621, 0x7622, PDF_CMAP_RANGE, 7515 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 9738 },
	{ 0x7624, 0x7625, PDF_CMAP_RANGE, 7518 },
	{ 0x7626, 0x7628, PDF_CMAP_TABLE, 613 },
	{ 0x7629, 0x762a, PDF_CMAP_RANGE, 7523 },
	{ 0x762b, 0x762b, PDF_CMAP_SINGLE, 9741 },
	{ 0x762c, 0x762f, PDF_CMAP_RANGE, 7526 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 9742 },
	{ 0x7631, 0x7632, PDF_CMAP_RANGE, 7531 },
	{ 0x7633, 0x763e, PDF_CMAP_RANGE, 9743 },
	{ 0x763f, 0x7644, PDF_CMAP_RANGE, 7545 },
	{ 0x7645, 0x7649, PDF_CMAP_TABLE, 616 },
	{ 0x764a, 0x764e, PDF_CMAP_RANGE, 7556 },
	{ 0x764f, 0x767e, PDF_CMAP_RANGE, 9758 },
	{ 0x7721, 0x772f, PDF_CMAP_RANGE, 9806 },
	{ 0x7730, 0x7731, PDF_CMAP_RANGE, 7624 },
	{ 0x7732, 0x7733, PDF_CMAP_RANGE, 9821 },
	{ 0x7734, 0x7735, PDF_CMAP_TABLE, 621 },
	{ 0x7736, 0x773c, PDF_CMAP_RANGE, 7630 },
	{ 0x773d, 0x773d, PDF_CMAP_SINGLE, 9824 },
	{ 0x773e, 0x7742, PDF_CMAP_RANGE, 7638 },
	{ 0x7743, 0x7744, PDF_CMAP_TABLE, 623 },
	{ 0x7745, 0x7746, PDF_CMAP_RANGE, 9826 },
	{ 0x7747, 0x7749, PDF_CMAP_RANGE, 7647 },
	{ 0x774a, 0x774b, PDF_CMAP_RANGE, 9828 },
	{ 0x774c, 0x774e, PDF_CMAP_RANGE, 7652 },
	{ 0x774f, 0x7750, PDF_CMAP_RANGE, 9830 },
	{ 0x7751, 0x775d, PDF_CMAP_RANGE, 7657 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 9832 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 7671 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 2745 },
	{ 0x7762, 0x7771, PDF_CMAP_RANGE, 7674 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 9833 },
	{ 0x7773, 0x7774, PDF_CMAP_RANGE, 7691 },
	{ 0x7775, 0x7775, PDF_CMAP_SINGLE, 9834 },
	{ 0x7776, 0x777e, PDF_CMAP_RANGE, 7694 },
	{ 0x7821, 0x7826, PDF_CMAP_RANGE, 9835 },
	{ 0x7827, 0x7829, PDF_CMAP_TABLE, 625 },
	{ 0x782a, 0x782b, PDF_CMAP_RANGE, 9841 },
	{ 0x782c, 0x782f, PDF_CMAP_TABLE, 628 },
	{ 0x7830, 0x7831, PDF_CMAP_RANGE, 9844 },
	{ 0x7832, 0x7835, PDF_CMAP_TABLE, 632 },
	{ 0x7836, 0x7837, PDF_CMAP_RANGE, 9847 },
	{ 0x7838, 0x7838, PDF_CMAP_SINGLE, 1852 },
	{ 0x7839, 0x783a, PDF_CMAP_RANGE, 9849 },
	{ 0x783b, 0x783b, PDF_CMAP_SINGLE, 1987 },
	{ 0x783c, 0x783f, PDF_CMAP_RANGE, 9851 },
	{ 0x7840, 0x7843, PDF_CMAP_TABLE, 636 },
	{ 0x7844, 0x7845, PDF_CMAP_RANGE, 9856 },
	{ 0x7846, 0x7846, PDF_CMAP_SINGLE, 2316 },
	{ 0x7847, 0x7848, PDF_CMAP_RANGE, 9858 },
	{ 0x7849, 0x784a, PDF_CMAP_TABLE, 640 },
	{ 0x784b, 0x784c, PDF_CMAP_RANGE, 2458 },
	{ 0x784d, 0x7850, PDF_CMAP_TABLE, 642 },
	{ 0x7851, 0x785b, PDF_CMAP_RANGE, 9863 },
	{ 0x785c, 0x7860, PDF_CMAP_TABLE, 646 },
	{ 0x7861, 0x7865, PDF_CMAP_RANGE, 9874 },
	{ 0x7866, 0x786b, PDF_CMAP_TABLE, 651 },
	{ 0x786c, 0x786e, PDF_CMAP_RANGE, 9880 },
	{ 0x786f, 0x7871, PDF_CMAP_TABLE, 657 },
	{ 0x7872, 0x7877, PDF_CMAP_RANGE, 9884 },
	{ 0x7878, 0x7879, PDF_CMAP_TABLE, 660 },
	{ 0x787a, 0x787b, PDF_CMAP_RANGE, 9890 },
	{ 0x787c, 0x787e, PDF_CMAP_TABLE, 662 },
	{ 0x7921, 0x7923, PDF_CMAP_TABLE, 665 },
	{ 0x7924, 0x7925, PDF_CMAP_RANGE, 9894 },
	{ 0x7926, 0x7929, PDF_CMAP_TABLE, 668 },
	{ 0x675b, 0x6760, PDF_CMAP_RANGE, 6163 },
	{ 0x6761, 0x6762, PDF_CMAP_RANGE, 9307 },
	{ 0x6763, 0x6766, PDF_CMAP_RANGE, 6171 },
	{ 0x6767, 0x6767, PDF_CMAP_SINGLE, 9309 },
	{ 0x6768, 0x676e, PDF_CMAP_RANGE, 6176 },
	{ 0x676f, 0x676f, PDF_CMAP_SINGLE, 9310 },
	{ 0x6770, 0x6773, PDF_CMAP_RANGE, 6184 },
	{ 0x6774, 0x6776, PDF_CMAP_RANGE, 9311 },
	{ 0x6777, 0x677e, PDF_CMAP_RANGE, 6191 },
	{ 0x6821, 0x6827, PDF_CMAP_RANGE, 6199 },
	{ 0x6828, 0x6828, PDF_CMAP_SINGLE, 9314 },
	{ 0x6829, 0x682b, PDF_CMAP_RANGE, 6207 },
	{ 0x682c, 0x682c, PDF_CMAP_SINGLE, 9315 },
	{ 0x682d, 0x6835, PDF_CMAP_RANGE, 6211 },
	{ 0x6836, 0x6836, PDF_CMAP_SINGLE, 9316 },
	{ 0x6837, 0x6837, PDF_CMAP_SINGLE, 6221 },
	{ 0x6838, 0x683a, PDF_CMAP_RANGE, 9317 },
	{ 0x683b, 0x683e, PDF_CMAP_RANGE, 6225 },
	{ 0x683f, 0x6840, PDF_CMAP_RANGE, 9320 },
	{ 0x6841, 0x6844, PDF_CMAP_RANGE, 6231 },
	{ 0x6845, 0x6845, PDF_CMAP_SINGLE, 9322 },
	{ 0x6846, 0x6846, PDF_CMAP_SINGLE, 6236 },
	{ 0x6847, 0x6849, PDF_CMAP_RANGE, 9323 },
	{ 0x684a, 0x684d, PDF_CMAP_RANGE, 6240 },
	{ 0x684e, 0x684e, PDF_CMAP_SINGLE, 9326 },
	{ 0x684f, 0x684f, PDF_CMAP_SINGLE, 6245 },
	{ 0x6850, 0x6850, PDF_CMAP_SINGLE, 9327 },
	{ 0x6851, 0x6852, PDF_CMAP_RANGE, 6247 },
	{ 0x6853, 0x6853, PDF_CMAP_SINGLE, 9328 },
	{ 0x6854, 0x685c, PDF_CMAP_RANGE, 6250 },
	{ 0x685d, 0x685d, PDF_CMAP_SINGLE, 9329 },
	{ 0x685e, 0x685e, PDF_CMAP_SINGLE, 6260 },
	{ 0x685f, 0x685f, PDF_CMAP_SINGLE, 9330 },
	{ 0x6860, 0x6861, PDF_CMAP_RANGE, 6262 },
	{ 0x6862, 0x6863, PDF_CMAP_RANGE, 9331 },
	{ 0x6864, 0x6864, PDF_CMAP_SINGLE, 6266 },
	{ 0x6865, 0x6865, PDF_CMAP_SINGLE, 9333 },
	{ 0x6866, 0x6866, PDF_CMAP_SINGLE, 6268 },
	{ 0x6867, 0x6867, PDF_CMAP_SINGLE, 9334 },
	{ 0x6868, 0x686a, PDF_CMAP_RANGE, 6270 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 9335 },
	{ 0x686c, 0x686c, PDF_CMAP_SINGLE, 6274 },
	{ 0x686d, 0x686d, PDF_CMAP_SINGLE, 9336 },
	{ 0x686e, 0x686e, PDF_CMAP_SINGLE, 6276 },
	{ 0x686f, 0x686f, PDF_CMAP_SINGLE, 9337 },
	{ 0x6870, 0x6878, PDF_CMAP_RANGE, 6278 },
	{ 0x6879, 0x6879, PDF_CMAP_SINGLE, 9338 },
	{ 0x687a, 0x687b, PDF_CMAP_RANGE, 6288 },
	{ 0x687c, 0x687d, PDF_CMAP_RANGE, 9339 },
	{ 0x687e, 0x687e, PDF_CMAP_SINGLE, 6292 },
	{ 0x6921, 0x6921, PDF_CMAP_SINGLE, 9341 },
	{ 0x6922, 0x692c, PDF_CMAP_RANGE, 6294 },
	{ 0x692d, 0x692d, PDF_CMAP_SINGLE, 9342 },
	{ 0x692e, 0x6933, PDF_CMAP_RANGE, 6306 },
	{ 0x6934, 0x6935, PDF_CMAP_RANGE, 9343 },
	{ 0x6936, 0x6936, PDF_CMAP_SINGLE, 6314 },
	{ 0x6937, 0x6937, PDF_CMAP_SINGLE, 9345 },
	{ 0x6938, 0x6943, PDF_CMAP_RANGE, 6316 },
	{ 0x6944, 0x6944, PDF_CMAP_SINGLE, 9346 },
	{ 0x6945, 0x6945, PDF_CMAP_SINGLE, 6329 },
	{ 0x6946, 0x6946, PDF_CMAP_SINGLE, 9347 },
	{ 0x6947, 0x6948, PDF_CMAP_RANGE, 6331 },
	{ 0x6949, 0x6949, PDF_CMAP_SINGLE, 9348 },
	{ 0x694a, 0x6955, PDF_CMAP_RANGE, 6334 },
	{ 0x6956, 0x6956, PDF_CMAP_SINGLE, 9349 },
	{ 0x6957, 0x6959, PDF_CMAP_RANGE, 6347 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 9350 },
	{ 0x695b, 0x6963, PDF_CMAP_RANGE, 6351 },
	{ 0x6964, 0x6964, PDF_CMAP_SINGLE, 9351 },
	{ 0x6965, 0x6965, PDF_CMAP_SINGLE, 6361 },
	{ 0x6966, 0x6967, PDF_CMAP_RANGE, 9352 },
	{ 0x6968, 0x6968, PDF_CMAP_SINGLE, 6364 },
	{ 0x6969, 0x6969, PDF_CMAP_SINGLE, 9354 },
	{ 0x696a, 0x696a, PDF_CMAP_SINGLE, 6366 },
	{ 0x696b, 0x696b, PDF_CMAP_SINGLE, 9355 },
	{ 0x696c, 0x696c, PDF_CMAP_SINGLE, 6368 },
	{ 0x696d, 0x697e, PDF_CMAP_RANGE, 9356 },
	{ 0x6a21, 0x6a25, PDF_CMAP_RANGE, 9374 },
	{ 0x6a26, 0x6a26, PDF_CMAP_SINGLE, 6392 },
	{ 0x6a27, 0x6a28, PDF_CMAP_RANGE, 9379 },
	{ 0x6a29, 0x6a30, PDF_CMAP_RANGE, 6395 },
	{ 0x6a31, 0x6a31, PDF_CMAP_SINGLE, 9381 },
	{ 0x6a32, 0x6a3b, PDF_CMAP_RANGE, 6404 },
	{ 0x6a3c, 0x6a3c, PDF_CMAP_SINGLE, 9382 },
	{ 0x6a3d, 0x6a49, PDF_CMAP_RANGE, 6415 },
	{ 0x6a4a, 0x6a4a, PDF_CMAP_SINGLE, 9383 },
	{ 0x6a4b, 0x6a4c, PDF_CMAP_RANGE, 6429 },
	{ 0x6a4d, 0x6a4d, PDF_CMAP_SINGLE, 9384 },
	{ 0x6a4e, 0x6a52, PDF_CMAP_RANGE, 6432 },
	{ 0x6a53, 0x6a53, PDF_CMAP_SINGLE, 9385 },
	{ 0x6a54, 0x6a59, PDF_CMAP_RANGE, 6438 },
	{ 0x6a5a, 0x6a6f, PDF_CMAP_RANGE, 9386 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6466 },
	{ 0x6b21, 0x6b26, PDF_CMAP_RANGE, 6481 },
	{ 0x6b27, 0x6b27, PDF_CMAP_SINGLE, 9408 },
	{ 0x6b28, 0x6b29, PDF_CMAP_RANGE, 6488 },
	{ 0x6b2a, 0x6b2a, PDF_CMAP_SINGLE, 9409 },
	{ 0x6b2b, 0x6b31, PDF_CMAP_RANGE, 6491 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 9410 },
	{ 0x6b33, 0x6b38, PDF_CMAP_RANGE, 6499 },
	{ 0x6b39, 0x6b39, PDF_CMAP_SINGLE, 9411 },
	{ 0x6b3a, 0x6b49, PDF_CMAP_RANGE, 6506 },
	{ 0x6b4a, 0x6b4b, PDF_CMAP_RANGE, 9412 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 6524 },
	{ 0x6b4d, 0x6b4d, PDF_CMAP_SINGLE, 9414 },
	{ 0x6b4e, 0x6b55, PDF_CMAP_RANGE, 6526 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 9415 },
	{ 0x6b57, 0x6b59, PDF_CMAP_RANGE, 6535 },
	{ 0x6b5a, 0x6b5a, PDF_CMAP_SINGLE, 9416 },
	{ 0x6b5b, 0x6b60, PDF_CMAP_RANGE, 6539 },
	{ 0x6b61, 0x6b61, PDF_CMAP_SINGLE, 9417 },
	{ 0x6b62, 0x6b76, PDF_CMAP_RANGE, 6546 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 9418 },
	{ 0x6b78, 0x6b7e, PDF_CMAP_RANGE, 6568 },
	{ 0x6c21, 0x6c22, PDF_CMAP_RANGE, 6575 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 9419 },
	{ 0x6c24, 0x6c28, PDF_CMAP_RANGE, 6578 },
	{ 0x6c29, 0x6c2e, PDF_CMAP_RANGE, 9420 },
	{ 0x6c2f, 0x6c30, PDF_CMAP_RANGE, 6589 },
	{ 0x6c31, 0x6c31, PDF_CMAP_SINGLE, 9426 },
	{ 0x6c32, 0x6c33, PDF_CMAP_RANGE, 6592 },
	{ 0x6c34, 0x6c35, PDF_CMAP_RANGE, 9427 },
	{ 0x6c36, 0x6c3d, PDF_CMAP_RANGE, 6596 },
	{ 0x6c3e, 0x6c3f, PDF_CMAP_RANGE, 9429 },
	{ 0x6c40, 0x6c40, PDF_CMAP_SINGLE, 6606 },
	{ 0x6c41, 0x6c41, PDF_CMAP_SINGLE, 9431 },
	{ 0x6c42, 0x6c46, PDF_CMAP_RANGE, 6608 },
	{ 0x6c47, 0x6c47, PDF_CMAP_SINGLE, 9432 },
	{ 0x6c48, 0x6c4a, PDF_CMAP_RANGE, 6614 },
	{ 0x6c4b, 0x6c4b, PDF_CMAP_SINGLE, 9433 },
	{ 0x6c4c, 0x6c61, PDF_CMAP_RANGE, 6618 },
	{ 0x6c62, 0x6c62, PDF_CMAP_SINGLE, 9434 },
	{ 0x6c63, 0x6c71, PDF_CMAP_RANGE, 6641 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 9435 },
	{ 0x6c73, 0x6c74, PDF_CMAP_RANGE, 6657 },
	{ 0x6c75, 0x6c75, PDF_CMAP_SINGLE, 9436 },
	{ 0x6c76, 0x6c77, PDF_CMAP_RANGE, 6660 },
	{ 0x6c78, 0x6c78, PDF_CMAP_SINGLE, 9437 },
	{ 0x6c79, 0x6c7e, PDF_CMAP_RANGE, 6663 },
	{ 0x6d21, 0x6d21, PDF_CMAP_SINGLE, 9438 },
	{ 0x6d22, 0x6d27, PDF_CMAP_RANGE, 6670 },
	{ 0x6d28, 0x6d28, PDF_CMAP_SINGLE, 9439 },
	{ 0x6d29, 0x6d2e, PDF_CMAP_RANGE, 6677 },
	{ 0x6d2f, 0x6d30, PDF_CMAP_RANGE, 9440 },
	{ 0x6d31, 0x6d33, PDF_CMAP_RANGE, 6685 },
	{ 0x6d34, 0x6d34, PDF_CMAP_SINGLE, 9442 },
	{ 0x6d35, 0x6d35, PDF_CMAP_SINGLE, 6689 },
	{ 0x6d36, 0x6d36, PDF_CMAP_SINGLE, 9443 },
	{ 0x6d37, 0x6d37, PDF_CMAP_SINGLE, 6691 },
	{ 0x6d38, 0x6d38, PDF_CMAP_SINGLE, 9444 },
	{ 0x6d39, 0x6d39, PDF_CMAP_SINGLE, 6693 },
	{ 0x6d3a, 0x6d3a, PDF_CMAP_SINGLE, 9445 },
	{ 0x6d3b, 0x6d3e, PDF_CMAP_RANGE, 6695 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 9446 },
	{ 0x6d40, 0x6d41, PDF_CMAP_RANGE, 6700 },
	{ 0x6d42, 0x6d43, PDF_CMAP_RANGE, 9447 },
	{ 0x6d44, 0x6d4b, PDF_CMAP_RANGE, 6704 },
	{ 0x6d4c, 0x6d4d, PDF_CMAP_RANGE, 9449 },
	{ 0x6d4e, 0x6d52, PDF_CMAP_RANGE, 6714 },
	{ 0x6d53, 0x6d53, PDF_CMAP_SINGLE, 9451 },
	{ 0x6d54, 0x6d56, PDF_CMAP_RANGE, 6720 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 9452 },
	{ 0x6d58, 0x6d67, PDF_CMAP_RANGE, 6724 },
	{ 0x6d68, 0x6d68, PDF_CMAP_SINGLE, 9453 },
	{ 0x6d69, 0x6d6d, PDF_CMAP_RANGE, 6741 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 9454 },
	{ 0x6d6f, 0x6d78, PDF_CMAP_RANGE, 6747 },
	{ 0x6d79, 0x6d7a, PDF_CMAP_RANGE, 9455 },
	{ 0x6d7b, 0x6d7e, PDF_CMAP_RANGE, 6759 },
	{ 0x6e21, 0x6e3b, PDF_CMAP_RANGE, 6763 },
	{ 0x6e3c, 0x6e3c, PDF_CMAP_SINGLE, 9457 },
	{ 0x6e3d, 0x6e3e, PDF_CMAP_RANGE, 6791 },
	{ 0x6e3f, 0x6e3f, PDF_CMAP_SINGLE, 9458 },
	{ 0x6e40, 0x6e43, PDF_CMAP_RANGE, 6794 },
	{ 0x6e44, 0x6e7e, PDF_CMAP_RANGE, 9459 },
	{ 0x6f21, 0x6f71, PDF_CMAP_RANGE, 9518 },
	{ 0x6f72, 0x6f7e, PDF_CMAP_RANGE, 6938 },
	{ 0x7021, 0x7022, PDF_CMAP_RANGE, 6951 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 9599 },
	{ 0x7024, 0x702e, PDF_CMAP_RANGE, 6954 },
	{ 0x702f, 0x7059, PDF_CMAP_RANGE, 9600 },
	{ 0x705a, 0x705b, PDF_CMAP_RANGE, 7008 },
	{ 0x705c, 0x705d, PDF_CMAP_RANGE, 9643 },
	{ 0x705e, 0x705e, PDF_CMAP_SINGLE, 7012 },
	{ 0x705f, 0x705f, PDF_CMAP_SINGLE, 9645 },
	{ 0x7060, 0x7068, PDF_CMAP_RANGE, 7014 },
	{ 0x7069, 0x7069, PDF_CMAP_SINGLE, 9646 },
	{ 0x706a, 0x706b, PDF_CMAP_RANGE, 7024 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 9647 },
	{ 0x706d, 0x706e, PDF_CMAP_RANGE, 7027 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 9648 },
	{ 0x7070, 0x7076, PDF_CMAP_RANGE, 7030 },
	{ 0x7077, 0x7077, PDF_CMAP_SINGLE, 9649 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 7038 },
	{ 0x7079, 0x7079, PDF_CMAP_SINGLE, 9650 },
	{ 0x707a, 0x707b, PDF_CMAP_RANGE, 7040 },
	{ 0x707c, 0x707c, PDF_CMAP_SINGLE, 9651 },
	{ 0x707d, 0x707e, PDF_CMAP_RANGE, 7043 },
	{ 0x7121, 0x7127, PDF_CMAP_RANGE, 7045 },
	{ 0x7128, 0x7128, PDF_CMAP_SINGLE, 9652 },
	{ 0x7129, 0x712a, PDF_CMAP_RANGE, 7053 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 9653 },
	{ 0x712c, 0x712d, PDF_CMAP_RANGE, 7056 },
	{ 0x712e, 0x712e, PDF_CMAP_SINGLE, 9654 },
	{ 0x712f, 0x7131, PDF_CMAP_RANGE, 7059 },
	{ 0x7132, 0x7132, PDF_CMAP_SINGLE, 9655 },
	{ 0x7133, 0x713b, PDF_CMAP_RANGE, 7063 },
	{ 0x713c, 0x713c, PDF_CMAP_SINGLE, 9656 },
	{ 0x713d, 0x713f, PDF_CMAP_RANGE, 7073 },
	{ 0x7140, 0x7140, PDF_CMAP_SINGLE, 9657 },
	{ 0x7141, 0x7148, PDF_CMAP_RANGE, 7077 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 9658 },
	{ 0x714a, 0x714c, PDF_CMAP_RANGE, 7086 },
	{ 0x714d, 0x714d, PDF_CMAP_SINGLE, 9659 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 7090 },
	{ 0x714f, 0x7150, PDF_CMAP_RANGE, 9660 },
	{ 0x7151, 0x7159, PDF_CMAP_RANGE, 7093 },
	{ 0x715a, 0x715a, PDF_CMAP_SINGLE, 9662 },
	{ 0x715b, 0x715b, PDF_CMAP_SINGLE, 7103 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 9663 },
	{ 0x715d, 0x7163, PDF_CMAP_RANGE, 7105 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 9664 },
	{ 0x7165, 0x716b, PDF_CMAP_RANGE, 7113 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 9665 },
	{ 0x716d, 0x716e, PDF_CMAP_RANGE, 7121 },
	{ 0x716f, 0x716f, PDF_CMAP_SINGLE, 9666 },
	{ 0x7170, 0x7176, PDF_CMAP_RANGE, 7124 },
	{ 0x7177, 0x7177, PDF_CMAP_SINGLE, 9667 },
	{ 0x7178, 0x7178, PDF_CMAP_SINGLE, 7132 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 9668 },
	{ 0x717a, 0x717b, PDF_CMAP_RANGE, 7134 },
	{ 0x717c, 0x717e, PDF_CMAP_RANGE, 9669 },
	{ 0x7221, 0x722d, PDF_CMAP_RANGE, 9672 },
	{ 0x722e, 0x7230, PDF_CMAP_RANGE, 7152 },
	{ 0x7231, 0x7232, PDF_CMAP_RANGE, 9685 },
	{ 0x7233, 0x7238, PDF_CMAP_RANGE, 7157 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 9687 },
	{ 0x723a, 0x7242, PDF_CMAP_RANGE, 7164 },
	{ 0x7243, 0x7243, PDF_CMAP_SINGLE, 9688 },
	{ 0x7244, 0x7248, PDF_CMAP_RANGE, 7174 },
	{ 0x7249, 0x7249, PDF_CMAP_SINGLE, 9689 },
	{ 0x724a, 0x724b, PDF_CMAP_RANGE, 7180 },
	{ 0x724c, 0x724d, PDF_CMAP_RANGE, 9690 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 7184 },
	{ 0x724f, 0x724f, PDF_CMAP_SINGLE, 9692 },
	{ 0x7250, 0x7252, PDF_CMAP_RANGE, 7186 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 9693 },
	{ 0x7254, 0x7264, PDF_CMAP_RANGE, 7190 },
	{ 0x7265, 0x7265, PDF_CMAP_SINGLE, 9694 },
	{ 0x7266, 0x726d, PDF_CMAP_RANGE, 7208 },
	{ 0x726e, 0x726e, PDF_CMAP_SINGLE, 9695 },
	{ 0x726f, 0x7276, PDF_CMAP_RANGE, 7217 },
	{ 0x7277, 0x7277, PDF_CMAP_SINGLE, 9696 },
	{ 0x7278, 0x727c, PDF_CMAP_RANGE, 7226 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 9697 },
	{ 0x727e, 0x727e, PDF_CMAP_SINGLE, 7232 },
	{ 0x7321, 0x733e, PDF_CMAP_RANGE, 7233 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 9698 },
	{ 0x7340, 0x7345, PDF_CMAP_RANGE, 7264 },
	{ 0x7346, 0x7346, PDF_CMAP_SINGLE, 9699 },
	{ 0x7347, 0x7347, PDF_CMAP_SINGLE, 7271 },
	{ 0x7348, 0x7348, PDF_CMAP_SINGLE, 9700 },
	{ 0x7349, 0x7355, PDF_CMAP_RANGE, 7273 },
	{ 0x7356, 0x7356, PDF_CMAP_SINGLE, 9701 },
	{ 0x7357, 0x7358, PDF_CMAP_RANGE, 7287 },
	{ 0x7359, 0x7359, PDF_CMAP_SINGLE, 9702 },
	{ 0x735a, 0x7364, PDF_CMAP_RANGE, 7290 },
	{ 0x7365, 0x7366, PDF_CMAP_RANGE, 9703 },
	{ 0x7367, 0x7369, PDF_CMAP_RANGE, 7303 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 9705 },
	{ 0x736b, 0x736b, PDF_CMAP_SINGLE, 7307 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 9706 },
	{ 0x736d, 0x736e, PDF_CMAP_RANGE, 7309 },
	{ 0x736f, 0x736f, PDF_CMAP_SINGLE, 9707 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 7312 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 9708 },
	{ 0x7372, 0x737c, PDF_CMAP_RANGE, 7314 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 9709 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 7326 },
	{ 0x7421, 0x7424, PDF_CMAP_RANGE, 7327 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 9710 },
	{ 0x7426, 0x742e, PDF_CMAP_RANGE, 7332 },
	{ 0x742f, 0x742f, PDF_CMAP_SINGLE, 9711 },
	{ 0x7430, 0x7434, PDF_CMAP_RANGE, 7342 },
	{ 0x7435, 0x7435, PDF_CMAP_SINGLE, 9712 },
	{ 0x7436, 0x7440, PDF_CMAP_RANGE, 7348 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 9713 },
	{ 0x7442, 0x7446, PDF_CMAP_RANGE, 7360 },
	{ 0x7447, 0x7447, PDF_CMAP_SINGLE, 9714 },
	{ 0x7448, 0x744e, PDF_CMAP_RANGE, 7366 },
	{ 0x744f, 0x7450, PDF_CMAP_RANGE, 9715 },
	{ 0x7451, 0x7455, PDF_CMAP_RANGE, 7375 },
	{ 0x7456, 0x7456, PDF_CMAP_SINGLE, 9717 },
	{ 0x7457, 0x7469, PDF_CMAP_RANGE, 7381 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 9718 },
	{ 0x746b, 0x746e, PDF_CMAP_RANGE, 7401 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 9719 },
	{ 0x7470, 0x7474, PDF_CMAP_RANGE, 7406 },
	{ 0x7475, 0x7475, PDF_CMAP_SINGLE, 9720 },
	{ 0x7476, 0x747e, PDF_CMAP_RANGE, 7412 },
	{ 0x7521, 0x7525, PDF_CMAP_RANGE, 7421 },
	{ 0x7526, 0x7527, PDF_CMAP_RANGE, 9721 },
	{ 0x7528, 0x7539, PDF_CMAP_RANGE, 7428 },
	{ 0x753a, 0x753b, PDF_CMAP_RANGE, 9723 },
	{ 0x753c, 0x7543, PDF_CMAP_RANGE, 7448 },
	{ 0x7544, 0x7544, PDF_CMAP_SINGLE, 9725 },
	{ 0x7545, 0x7547, PDF_CMAP_RANGE, 7457 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 9726 },
	{ 0x7549, 0x754d, PDF_CMAP_RANGE, 7461 },
	{ 0x754e, 0x754f, PDF_CMAP_RANGE, 9727 },
	{ 0x7550, 0x7550, PDF_CMAP_SINGLE, 7468 },
	{ 0x7551, 0x7552, PDF_CMAP_RANGE, 9729 },
	{ 0x7553, 0x7558, PDF_CMAP_RANGE, 7471 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 9731 },
	{ 0x755a, 0x755b, PDF_CMAP_RANGE, 7478 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 9732 },
	{ 0x755d, 0x7565, PDF_CMAP_RANGE, 7481 },
	{ 0x7566, 0x7567, PDF_CMAP_RANGE, 9733 },
	{ 0x7568, 0x756e, PDF_CMAP_RANGE, 7492 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 9735 },
	{ 0x7570, 0x7571, PDF_CMAP_RANGE, 7500 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 9736 },
	{ 0x7573, 0x757b, PDF_CMAP_RANGE, 7503 },
	{ 0x757c, 0x757c, PDF_CMAP_SINGLE, 9737 },
	{ 0x757d, 0x757e, PDF_CMAP_RANGE, 7513 },
	{ 0x7621, 0x7622, PDF_CMAP_RANGE, 7515 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 9738 },
	{ 0x7624, 0x7625, PDF_CMAP_RANGE, 7518 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 9739 },
	{ 0x7627, 0x7627, PDF_CMAP_SINGLE, 7521 },
	{ 0x7628, 0x7628, PDF_CMAP_SINGLE, 9740 },
	{ 0x7629, 0x762a, PDF_CMAP_RANGE, 7523 },
	{ 0x762b, 0x762b, PDF_CMAP_SINGLE, 9741 },
	{ 0x762c, 0x762f, PDF_CMAP_RANGE, 7526 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 9742 },
	{ 0x7631, 0x7632, PDF_CMAP_RANGE, 7531 },
	{ 0x7633, 0x763e, PDF_CMAP_RANGE, 9743 },
	{ 0x763f, 0x7644, PDF_CMAP_RANGE, 7545 },
	{ 0x7645, 0x7645, PDF_CMAP_SINGLE, 9755 },
	{ 0x7646, 0x7646, PDF_CMAP_SINGLE, 7552 },
	{ 0x7647, 0x7647, PDF_CMAP_SINGLE, 9756 },
	{ 0x7648, 0x7648, PDF_CMAP_SINGLE, 7554 },
	{ 0x7649, 0x7649, PDF_CMAP_SINGLE, 9757 },
	{ 0x764a, 0x764e, PDF_CMAP_RANGE, 7556 },
	{ 0x764f, 0x767e, PDF_CMAP_RANGE, 9758 },
	{ 0x7721, 0x772f, PDF_CMAP_RANGE, 9806 },
	{ 0x7730, 0x7731, PDF_CMAP_RANGE, 7624 },
	{ 0x7732, 0x7733, PDF_CMAP_RANGE, 9821 },
	{ 0x7734, 0x7734, PDF_CMAP_SINGLE, 7628 },
	{ 0x7735, 0x7735, PDF_CMAP_SINGLE, 9823 },
	{ 0x7736, 0x773c, PDF_CMAP_RANGE, 7630 },
	{ 0x773d, 0x773d, PDF_CMAP_SINGLE, 9824 },
	{ 0x773e, 0x7742, PDF_CMAP_RANGE, 7638 },
	{ 0x7743, 0x7743, PDF_CMAP_SINGLE, 9825 },
	{ 0x7744, 0x7744, PDF_CMAP_SINGLE, 7644 },
	{ 0x7745, 0x7746, PDF_CMAP_RANGE, 9826 },
	{ 0x7747, 0x7749, PDF_CMAP_RANGE, 7647 },
	{ 0x774a, 0x774b, PDF_CMAP_RANGE, 9828 },
	{ 0x774c, 0x774e, PDF_CMAP_RANGE, 7652 },
	{ 0x774f, 0x7750, PDF_CMAP_RANGE, 9830 },
	{ 0x7751, 0x775d, PDF_CMAP_RANGE, 7657 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 9832 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 7671 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 2745 },
	{ 0x7762, 0x7771, PDF_CMAP_RANGE, 7674 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 9833 },
	{ 0x7773, 0x7774, PDF_CMAP_RANGE, 7691 },
	{ 0x7775, 0x7775, PDF_CMAP_SINGLE, 9834 },
	{ 0x7776, 0x777e, PDF_CMAP_RANGE, 7694 },
	{ 0x7821, 0x7826, PDF_CMAP_RANGE, 9835 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 1228 },
	{ 0x7828, 0x7828, PDF_CMAP_SINGLE, 1290 },
	{ 0x7829, 0x7829, PDF_CMAP_SINGLE, 1304 },
	{ 0x782a, 0x782b, PDF_CMAP_RANGE, 9841 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 1428 },
	{ 0x782d, 0x782d, PDF_CMAP_SINGLE, 1486 },
	{ 0x782e, 0x782e, PDF_CMAP_SINGLE, 9843 },
	{ 0x782f, 0x782f, PDF_CMAP_SINGLE, 1526 },
	{ 0x7830, 0x7831, PDF_CMAP_RANGE, 9844 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 1619 },
	{ 0x7833, 0x7833, PDF_CMAP_SINGLE, 1662 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 9846 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 1732 },
	{ 0x7836, 0x7837, PDF_CMAP_RANGE, 9847 },
	{ 0x7838, 0x7838, PDF_CMAP_SINGLE, 1852 },
	{ 0x7839, 0x783a, PDF_CMAP_RANGE, 9849 },
	{ 0x783b, 0x783b, PDF_CMAP_SINGLE, 1987 },
	{ 0x783c, 0x783f, PDF_CMAP_RANGE, 9851 },
	{ 0x7840, 0x7840, PDF_CMAP_SINGLE, 2091 },
	{ 0x7841, 0x7841, PDF_CMAP_SINGLE, 9855 },
	{ 0x7842, 0x7842, PDF_CMAP_SINGLE, 2126 },
	{ 0x7843, 0x7843, PDF_CMAP_SINGLE, 2153 },
	{ 0x7844, 0x7845, PDF_CMAP_RANGE, 9856 },
	{ 0x7846, 0x7846, PDF_CMAP_SINGLE, 2316 },
	{ 0x7847, 0x7848, PDF_CMAP_RANGE, 9858 },
	{ 0x7849, 0x7849, PDF_CMAP_SINGLE, 2417 },
	{ 0x784a, 0x784a, PDF_CMAP_SINGLE, 9860 },
	{ 0x784b, 0x784c, PDF_CMAP_RANGE, 2458 },
	{ 0x784d, 0x784d, PDF_CMAP_SINGLE, 9861 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 2522 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 9862 },
	{ 0x7850, 0x7850, PDF_CMAP_SINGLE, 2554 },
	{ 0x7851, 0x785b, PDF_CMAP_RANGE, 9863 },
	{ 0x785c, 0x785c, PDF_CMAP_SINGLE, 3034 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3037 },
	{ 0x785e, 0x785e, PDF_CMAP_SINGLE, 3050 },
	{ 0x785f, 0x785f, PDF_CMAP_SINGLE, 3052 },
	{ 0x7860, 0x7860, PDF_CMAP_SINGLE, 3058 },
	{ 0x7861, 0x7865, PDF_CMAP_RANGE, 9874 },
	{ 0x7866, 0x7866, PDF_CMAP_SINGLE, 3218 },
	{ 0x7867, 0x7867, PDF_CMAP_SINGLE, 3354 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 3468 },
	{ 0x7869, 0x7869, PDF_CMAP_SINGLE, 3518 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 9879 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 3579 },
	{ 0x786c, 0x786e, PDF_CMAP_RANGE, 9880 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 3696 },
	{ 0x7870, 0x7870, PDF_CMAP_SINGLE, 9883 },
	{ 0x7871, 0x7871, PDF_CMAP_SINGLE, 3747 },
	{ 0x7872, 0x7877, PDF_CMAP_RANGE, 9884 },
	{ 0x7878, 0x7878, PDF_CMAP_SINGLE, 4157 },
	{ 0x7879, 0x7879, PDF_CMAP_SINGLE, 4313 },
	{ 0x787a, 0x787b, PDF_CMAP_RANGE, 9890 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 4347 },
	{ 0x787d, 0x787d, PDF_CMAP_SINGLE, 4361 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 9892 },
	{ 0x7921, 0x7921, PDF_CMAP_SINGLE, 4513 },
	{ 0x7922, 0x7922, PDF_CMAP_SINGLE, 9893 },
	{ 0x7923, 0x7923, PDF_CMAP_SINGLE, 4538 },
	{ 0x7924, 0x7925, PDF_CMAP_RANGE, 9894 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 4565 },
	{ 0x7927, 0x7927, PDF_CMAP_SINGLE, 9896 },
	{ 0x7928, 0x7928, PDF_CMAP_SINGLE, 4605 },
	{ 0x7929, 0x7929, PDF_CMAP_SINGLE, 4633 },
};

static const unsigned short pdf_cmap_GBT_H_table[672] =
{
	7717,948,7718,7723,991,7724,7725,998,
	7726,1017,7730,7732,1027,7733,1040,7736,
	7738,1052,1082,7748,1106,7756,7763,1127,
	7780,1190,7781,1192,1229,7801,7806,1256,
	7807,1258,7808,7809,1267,7810,7812,1279,
	7813,1281,7814,7815,1289,1292,7818,7821,
	1301,7822,1303,7823,7839,1361,7840,7843,
	1374,7844,7845,1393,7846,7849,1413,7854,
	1422,1431,7862,7879,1503,1509,7882,1511,
	7883,1513,7884,7892,1540,7893,7894,1549,
	1556,7899,7900,1561,7901,7904,1580,1583,
	7907,7909,1593,1621,7921,7926,1645,7927,
	1663,7934,1675,7941,7948,1718,7949,1720,
	1731,7955,7964,1765,7965,7966,1775,1778,
	7969,7972,1803,7973,1805,7974,7981,1845,
	7982,1847,7983,1849,7984,1851,7985,1862,
	7991,7992,1867,7996,1874,7997,7998,1879,
	8010,1978,8011,1988,8016,8017,1993,8023,
	2023,8041,2067,2068,8042,2077,8047,2079,
	8048,8053,2101,8054,8057,2110,2122,8064,
	8065,2127,8066,2129,8072,2140,2180,8100,
	8124,2255,8128,2264,8129,8142,2325,8143,
	2327,8144,8146,2341,8154,2387,8155,8163,
	2423,8164,2425,8165,8166,2433,8167,2435,
	8168,2437,8171,2445,8172,8173,2453,8174,
	8196,2496,8197,2498,8198,8201,2517,8242,
	2611,8245,2621,8246,2661,8270,2663,8271,
	2680,8281,2710,8303,2712,8315,7673,8320,
	2770,8321,8333,2819,8338,2869,8349,2904,
	2912,8355,8364,2935,2939,8368,2941,8369,
	8380,3038,8381,8391,3087,8392,8395,3101,
	8396,3105,3111,8401,3132,8411,3134,8412,
	8413,3139,3155,8420,8432,3193,8433,3195,
	8438,3217,8439,3234,8444,8445,3241,8446,
	3284,8456,3286,8457,8459,3295,3324,8469,
	8470,3331,8471,3333,8472,8474,3350,8475,
	8476,3355,8477,3357,8478,3374,8483,3376,
	3390,8490,3392,8505,3446,8506,3469,8512,
	8514,3477,8516,3489,8517,8518,3497,3500,
	8521,8523,3517,8530,3529,8531,8534,3546,
	3564,8542,8554,3602,8555,8556,3616,8557,
	3618,8558,8560,3630,8561,8562,3636,8563,
	8576,3703,8577,8578,3716,3735,8586,3765,
	8591,3777,8598,3798,8603,3800,8604,3818,
	8612,3820,8615,3835,8616,8617,3841,8618,
	8621,3868,8622,3878,8626,8627,3883,8631,
	3891,8632,3900,8637,3910,8643,3912,8644,
	8648,3928,8649,3930,8650,8668,4023,8669,
	4044,8676,4065,8685,4067,8701,4118,4121,
	8704,4136,8709,8713,4155,8716,4165,8717,
	8718,4171,8719,4173,8720,8730,4215,8731,
	4242,8746,4252,8751,8753,4267,8754,8767,
	4322,8768,8769,4328,4346,8779,8780,4352,
	8783,4360,4363,8786,4365,8798,4392,8799,
	8800,4402,8810,4427,8811,8818,4445,8819,
	8821,4455,8822,4461,8825,4480,8829,4482,
	8830,8844,4539,8845,4541,8851,4563,8856,
	4575,8857,8858,4581,4616,8873,4618,4626,
	8880,4628,8893,4677,8897,4734,8898,4744,
	8903,8904,4752,8910,4802,8911,4804,8922,
	4857,8923,8987,4970,8988,8994,5004,8995,
	8996,5024,8997,9000,5043,9001,9026,5166,
	9046,5257,9047,9052,5306,5356,9063,5382,
	9067,9076,5417,9077,9079,5426,9080,9096,
	5552,9097,9100,5568,9101,1962,9107,9110,
	5627,9111,9112,5634,5681,9134,5703,9138,
	9142,5723,5729,9147,9192,5883,9193,9201,
	5956,9202,9204,5997,9205,6023,9211,9316,
	6221,9322,6236,9326,6245,9327,9329,6260,
	9330,6266,9333,6268,9334,9335,6274,9336,
	6276,9337,6314,9345,9346,6329,9347,9351,
	6361,6364,9354,6366,9355,6368,6524,9414,
	6606,9431,9442,6689,9443,6691,9444,6693,
	9445,7012,9645,9649,7038,9650,9659,7090,
	9662,7103,9663,9667,7132,9668,7184,9692,
	9697,7232,9699,7271,9700,9705,7307,9706,
	9707,7312,9708,9709,7326,9739,7521,9740,
	9755,7552,9756,7554,9757,7628,9823,9825,
	7644,1228,1290,1304,1428,1486,9843,1526,
	1619,1662,9846,1732,2091,9855,2126,2153,
	2417,9860,9861,2522,9862,2554,3034,3037,
	3050,3052,3058,3218,3354,3468,3518,9879,
	3579,3696,9883,3747,4157,4313,4347,4361,
	9892,4513,9893,4538,4565,9896,4605,4633,
};

pdf_cmap pdf_cmap_GBT_H =
{
	-1, "GBT-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	2281, 2281, (pdf_range*) pdf_cmap_GBT_H_ranges,
	672, 672, (unsigned short*) pdf_cmap_GBT_H_table,
};

/* GBT-V */

static const pdf_range pdf_cmap_GBT_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_TABLE, 0 },
	{ 0x212a, 0x212a, PDF_CMAP_SINGLE, 598 },
	{ 0x212b, 0x212c, PDF_CMAP_RANGE, 7704 },
	{ 0x212d, 0x212d, PDF_CMAP_SINGLE, 599 },
	{ 0x2132, 0x213f, PDF_CMAP_RANGE, 582 },
	{ 0x217e, 0x217e, PDF_CMAP_SINGLE, 7706 },
	{ 0x2321, 0x2321, PDF_CMAP_SINGLE, 578 },
	{ 0x2328, 0x2329, PDF_CMAP_RANGE, 580 },
	{ 0x232c, 0x232c, PDF_CMAP_SINGLE, 573 },
	{ 0x232e, 0x232e, PDF_CMAP_SINGLE, 7707 },
	{ 0x233a, 0x233b, PDF_CMAP_RANGE, 576 },
	{ 0x233d, 0x233d, PDF_CMAP_SINGLE, 7708 },
	{ 0x233f, 0x233f, PDF_CMAP_SINGLE, 579 },
	{ 0x235b, 0x235b, PDF_CMAP_SINGLE, 7709 },
	{ 0x235d, 0x235d, PDF_CMAP_SINGLE, 7710 },
	{ 0x235f, 0x235f, PDF_CMAP_SINGLE, 600 },
	{ 0x237b, 0x237b, PDF_CMAP_SINGLE, 596 },
	{ 0x237d, 0x237e, PDF_CMAP_TABLE, 2 },
	{ 0x237d, 0x237d, PDF_CMAP_SINGLE, 597 },
	{ 0x237e, 0x237e, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GBT_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBT_V =
{
	-1, "GBT-V", "GBT-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GBT_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBT_V_table,
};

/* GBTpc-EUC-H */

static const pdf_range pdf_cmap_GBTpc_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x0080, 0x0080, PDF_CMAP_SINGLE, 7712 },
	{ 0x00fd, 0x00ff, PDF_CMAP_RANGE, 7713 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 96 },
	{ 0xa2b1, 0xa2e2, PDF_CMAP_RANGE, 190 },
	{ 0xa2e5, 0xa2ee, PDF_CMAP_RANGE, 240 },
	{ 0xa2f1, 0xa2fc, PDF_CMAP_RANGE, 250 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 262 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 356 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 439 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 525 },
	{ 0xa6c1, 0xa6f5, PDF_CMAP_RANGE, 549 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 602 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 635 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_RANGE, 668 },
	{ 0xa8c5, 0xa8ea, PDF_CMAP_RANGE, 700 },
	{ 0xa9a4, 0xa9ef, PDF_CMAP_RANGE, 738 },
	{ 0xaaa1, 0xaafe, PDF_CMAP_RANGE, 814 },
	{ 0xaba1, 0xabc0, PDF_CMAP_RANGE, 908 },
	{ 0xb0a1, 0xb0a7, PDF_CMAP_RANGE, 940 },
	{ 0xb0a8, 0xb0aa, PDF_CMAP_TABLE, 0 },
	{ 0xb0ab, 0xb0ac, PDF_CMAP_RANGE, 950 },
	{ 0xb0ad, 0xb0ae, PDF_CMAP_RANGE, 7719 },
	{ 0xb0af, 0xb0b8, PDF_CMAP_RANGE, 954 },
	{ 0xb0b9, 0xb0b9, PDF_CMAP_SINGLE, 7721 },
	{ 0xb0ba, 0xb0bf, PDF_CMAP_RANGE, 965 },
	{ 0xb0c0, 0xb0c0, PDF_CMAP_SINGLE, 7722 },
	{ 0xb0c1, 0xb0d2, PDF_CMAP_RANGE, 972 },
	{ 0xb0d3, 0xb0d5, PDF_CMAP_TABLE, 3 },
	{ 0xb0d6, 0xb0d9, PDF_CMAP_RANGE, 993 },
	{ 0xb0da, 0xb0dc, PDF_CMAP_TABLE, 6 },
	{ 0xb0dd, 0xb0e3, PDF_CMAP_RANGE, 1000 },
	{ 0xb0e4, 0xb0e4, PDF_CMAP_SINGLE, 7727 },
	{ 0xb0e5, 0xb0eb, PDF_CMAP_RANGE, 1008 },
	{ 0xb0ec, 0xb0ed, PDF_CMAP_RANGE, 7728 },
	{ 0xb0ee, 0xb0ef, PDF_CMAP_TABLE, 9 },
	{ 0xb0f0, 0xb0f2, PDF_CMAP_RANGE, 1019 },
	{ 0xb0f3, 0xb0f3, PDF_CMAP_SINGLE, 7731 },
	{ 0xb0f4, 0xb0f6, PDF_CMAP_RANGE, 1023 },
	{ 0xb0f7, 0xb0f9, PDF_CMAP_TABLE, 11 },
	{ 0xb0fa, 0xb0fe, PDF_CMAP_RANGE, 1029 },
	{ 0xb1a1, 0xb1a4, PDF_CMAP_RANGE, 1034 },
	{ 0xb1a5, 0xb1a6, PDF_CMAP_RANGE, 7734 },
	{ 0xb1a7, 0xb1a8, PDF_CMAP_TABLE, 14 },
	{ 0xb1a9, 0xb1aa, PDF_CMAP_RANGE, 1042 },
	{ 0xb1ab, 0xb1ab, PDF_CMAP_SINGLE, 7737 },
	{ 0xb1ac, 0xb1b1, PDF_CMAP_RANGE, 1045 },
	{ 0xb1b2, 0xb1b3, PDF_CMAP_TABLE, 16 },
	{ 0xb1b4, 0xb1b5, PDF_CMAP_RANGE, 7739 },
	{ 0xb1b6, 0xb1b6, PDF_CMAP_SINGLE, 1055 },
	{ 0xb1b7, 0xb1b9, PDF_CMAP_RANGE, 7741 },
	{ 0xb1ba, 0xb1c0, PDF_CMAP_RANGE, 1059 },
	{ 0xb1c1, 0xb1c1, PDF_CMAP_SINGLE, 7744 },
	{ 0xb1c2, 0xb1c9, PDF_CMAP_RANGE, 1067 },
	{ 0xb1ca, 0xb1ca, PDF_CMAP_SINGLE, 7745 },
	{ 0xb1cb, 0xb1ce, PDF_CMAP_RANGE, 1076 },
	{ 0xb1cf, 0xb1d0, PDF_CMAP_RANGE, 7746 },
	{ 0xb1d1, 0xb1d2, PDF_CMAP_TABLE, 18 },
	{ 0xb1d3, 0xb1d4, PDF_CMAP_RANGE, 1084 },
	{ 0xb1d5, 0xb1d5, PDF_CMAP_SINGLE, 7749 },
	{ 0xb1d6, 0xb1de, PDF_CMAP_RANGE, 1087 },
	{ 0xb1df, 0xb1e1, PDF_CMAP_RANGE, 7750 },
	{ 0xb1e2, 0xb1e3, PDF_CMAP_RANGE, 1099 },
	{ 0xb1e4, 0xb1e4, PDF_CMAP_SINGLE, 7753 },
	{ 0xb1e5, 0xb1e6, PDF_CMAP_RANGE, 1102 },
	{ 0xb1e7, 0xb1e8, PDF_CMAP_RANGE, 7754 },
	{ 0xb1e9, 0xb1ea, PDF_CMAP_TABLE, 20 },
	{ 0xb1eb, 0xb1ed, PDF_CMAP_RANGE, 1108 },
	{ 0xb1ee, 0xb1ee, PDF_CMAP_SINGLE, 7757 },
	{ 0xb1ef, 0xb1f0, PDF_CMAP_RANGE, 1112 },
	{ 0xb1f1, 0xb1f1, PDF_CMAP_SINGLE, 7758 },
	{ 0xb1f2, 0xb1f3, PDF_CMAP_RANGE, 1115 },
	{ 0xb1f4, 0xb1f7, PDF_CMAP_RANGE, 7759 },
	{ 0xb1f8, 0xb1fc, PDF_CMAP_RANGE, 1121 },
	{ 0xb1fd, 0xb1fe, PDF_CMAP_TABLE, 22 },
	{ 0xb2a1, 0xb2a5, PDF_CMAP_RANGE, 1128 },
	{ 0xb2a6, 0xb2a7, PDF_CMAP_RANGE, 7764 },
	{ 0xb2a8, 0xb2ab, PDF_CMAP_RANGE, 1135 },
	{ 0xb2ac, 0xb2ac, PDF_CMAP_SINGLE, 7766 },
	{ 0xb2ad, 0xb2b4, PDF_CMAP_RANGE, 1140 },
	{ 0xb2b5, 0xb2b5, PDF_CMAP_SINGLE, 7767 },
	{ 0xb2b6, 0xb2b8, PDF_CMAP_RANGE, 1149 },
	{ 0xb2b9, 0xb2b9, PDF_CMAP_SINGLE, 7768 },
	{ 0xb2ba, 0xb2c5, PDF_CMAP_RANGE, 1153 },
	{ 0xb2c6, 0xb2c6, PDF_CMAP_SINGLE, 7769 },
	{ 0xb2c7, 0xb2cd, PDF_CMAP_RANGE, 1166 },
	{ 0xb2ce, 0xb2d7, PDF_CMAP_RANGE, 7770 },
	{ 0xb2d8, 0xb2dd, PDF_CMAP_RANGE, 1183 },
	{ 0xb2de, 0xb2e1, PDF_CMAP_TABLE, 24 },
	{ 0xb2e2, 0xb2e3, PDF_CMAP_RANGE, 7782 },
	{ 0xb2e4, 0xb2ee, PDF_CMAP_RANGE, 1195 },
	{ 0xb2ef, 0xb2ef, PDF_CMAP_SINGLE, 7784 },
	{ 0xb2f0, 0xb2f2, PDF_CMAP_RANGE, 1207 },
	{ 0xb2f3, 0xb2fc, PDF_CMAP_RANGE, 7785 },
	{ 0xb2fd, 0xb2fe, PDF_CMAP_RANGE, 1220 },
	{ 0xb3a1, 0xb3a2, PDF_CMAP_RANGE, 7795 },
	{ 0xb3a3, 0xb3a3, PDF_CMAP_SINGLE, 1224 },
	{ 0xb3a4, 0xb3a7, PDF_CMAP_RANGE, 7797 },
	{ 0xb3a8, 0xb3a9, PDF_CMAP_TABLE, 28 },
	{ 0xb3aa, 0xb3ad, PDF_CMAP_RANGE, 1231 },
	{ 0xb3ae, 0xb3ae, PDF_CMAP_SINGLE, 7802 },
	{ 0xb3af, 0xb3b4, PDF_CMAP_RANGE, 1236 },
	{ 0xb3b5, 0xb3b5, PDF_CMAP_SINGLE, 7803 },
	{ 0xb3b6, 0xb3b8, PDF_CMAP_RANGE, 1243 },
	{ 0xb3b9, 0xb3b9, PDF_CMAP_SINGLE, 7804 },
	{ 0xb3ba, 0xb3bd, PDF_CMAP_RANGE, 1247 },
	{ 0xb3be, 0xb3be, PDF_CMAP_SINGLE, 7805 },
	{ 0xb3bf, 0xb3c1, PDF_CMAP_RANGE, 1252 },
	{ 0xb3c2, 0xb3c6, PDF_CMAP_TABLE, 30 },
	{ 0xb3c7, 0xb3cc, PDF_CMAP_RANGE, 1260 },
	{ 0xb3cd, 0xb3cf, PDF_CMAP_TABLE, 35 },
	{ 0xb3d0, 0xb3d1, PDF_CMAP_RANGE, 1269 },
	{ 0xb3d2, 0xb3d2, PDF_CMAP_SINGLE, 7811 },
	{ 0xb3d3, 0xb3d8, PDF_CMAP_RANGE, 1272 },
	{ 0xb3d9, 0xb3dd, PDF_CMAP_TABLE, 38 },
	{ 0xb3de, 0xb3e2, PDF_CMAP_RANGE, 1283 },
	{ 0xb3e3, 0xb3e4, PDF_CMAP_TABLE, 43 },
	{ 0xb3e5, 0xb3e6, PDF_CMAP_RANGE, 7816 },
	{ 0xb3e7, 0xb3e8, PDF_CMAP_TABLE, 45 },
	{ 0xb3e9, 0xb3ea, PDF_CMAP_RANGE, 1294 },
	{ 0xb3eb, 0xb3ec, PDF_CMAP_RANGE, 7819 },
	{ 0xb3ed, 0xb3ee, PDF_CMAP_RANGE, 1298 },
	{ 0xb3ef, 0xb3f3, PDF_CMAP_TABLE, 47 },
	{ 0xb3f4, 0xb3f9, PDF_CMAP_RANGE, 1305 },
	{ 0xb3fa, 0xb3fb, PDF_CMAP_RANGE, 7824 },
	{ 0xb3fc, 0xb3fe, PDF_CMAP_RANGE, 1313 },
	{ 0xb4a1, 0xb4a2, PDF_CMAP_RANGE, 7826 },
	{ 0xb4a3, 0xb4a4, PDF_CMAP_RANGE, 1318 },
	{ 0xb4a5, 0xb4a6, PDF_CMAP_RANGE, 7828 },
	{ 0xb4a7, 0xb4aa, PDF_CMAP_RANGE, 1322 },
	{ 0xb4ab, 0xb4ab, PDF_CMAP_SINGLE, 7830 },
	{ 0xb4ac, 0xb4ae, PDF_CMAP_RANGE, 1327 },
	{ 0xb4af, 0xb4af, PDF_CMAP_SINGLE, 7831 },
	{ 0xb4b0, 0xb4b2, PDF_CMAP_RANGE, 1331 },
	{ 0xb4b3, 0xb4b4, PDF_CMAP_RANGE, 7832 },
	{ 0xb4b5, 0xb4b7, PDF_CMAP_RANGE, 1336 },
	{ 0xb4b8, 0xb4b8, PDF_CMAP_SINGLE, 7834 },
	{ 0xb4b9, 0xb4be, PDF_CMAP_RANGE, 1340 },
	{ 0xb4bf, 0xb4bf, PDF_CMAP_SINGLE, 7835 },
	{ 0xb4c0, 0xb4c1, PDF_CMAP_RANGE, 1347 },
	{ 0xb4c2, 0xb4c2, PDF_CMAP_SINGLE, 7836 },
	{ 0xb4c3, 0xb4c6, PDF_CMAP_RANGE, 1350 },
	{ 0xb4c7, 0xb4c7, PDF_CMAP_SINGLE, 7837 },
	{ 0xb4c8, 0xb4c9, PDF_CMAP_RANGE, 1355 },
	{ 0xb4ca, 0xb4ca, PDF_CMAP_SINGLE, 7838 },
	{ 0xb4cb, 0xb4cc, PDF_CMAP_RANGE, 1358 },
	{ 0xb4cd, 0xb4cf, PDF_CMAP_TABLE, 52 },
	{ 0xb4d0, 0xb4d2, PDF_CMAP_RANGE, 1363 },
	{ 0xb4d3, 0xb4d4, PDF_CMAP_RANGE, 7841 },
	{ 0xb4d5, 0xb4d9, PDF_CMAP_RANGE, 1368 },
	{ 0xb4da, 0xb4dc, PDF_CMAP_TABLE, 55 },
	{ 0xb4dd, 0xb4ec, PDF_CMAP_RANGE, 1376 },
	{ 0xb4ed, 0xb4ef, PDF_CMAP_TABLE, 58 },
	{ 0xb4f0, 0xb4f7, PDF_CMAP_RANGE, 1395 },
	{ 0xb4f8, 0xb4f8, PDF_CMAP_SINGLE, 7847 },
	{ 0xb4f9, 0xb4fa, PDF_CMAP_RANGE, 1404 },
	{ 0xb4fb, 0xb4fb, PDF_CMAP_SINGLE, 7848 },
	{ 0xb4fc, 0xb4fe, PDF_CMAP_RANGE, 1407 },
	{ 0xb5a1, 0xb5a2, PDF_CMAP_RANGE, 1410 },
	{ 0xb5a3, 0xb5a4, PDF_CMAP_TABLE, 61 },
	{ 0xb5a5, 0xb5a8, PDF_CMAP_RANGE, 7850 },
	{ 0xb5a9, 0xb5ab, PDF_CMAP_RANGE, 1418 },
	{ 0xb5ac, 0xb5ad, PDF_CMAP_TABLE, 63 },
	{ 0xb5ae, 0xb5af, PDF_CMAP_RANGE, 7855 },
	{ 0xb5b0, 0xb5b0, PDF_CMAP_SINGLE, 1425 },
	{ 0xb5b1, 0xb5b5, PDF_CMAP_RANGE, 7857 },
	{ 0xb5b6, 0xb5b7, PDF_CMAP_TABLE, 65 },
	{ 0xb5b8, 0xb5b9, PDF_CMAP_RANGE, 1433 },
	{ 0xb5ba, 0xb5bc, PDF_CMAP_RANGE, 7863 },
	{ 0xb5bd, 0xb5c5, PDF_CMAP_RANGE, 1438 },
	{ 0xb5c6, 0xb5c6, PDF_CMAP_SINGLE, 7866 },
	{ 0xb5c7, 0xb5ca, PDF_CMAP_RANGE, 1448 },
	{ 0xb5cb, 0xb5cb, PDF_CMAP_SINGLE, 7867 },
	{ 0xb5cc, 0xb5cf, PDF_CMAP_RANGE, 1453 },
	{ 0xb5d0, 0xb5d0, PDF_CMAP_SINGLE, 7868 },
	{ 0xb5d1, 0xb5d2, PDF_CMAP_RANGE, 1458 },
	{ 0xb5d3, 0xb5d3, PDF_CMAP_SINGLE, 7869 },
	{ 0xb5d4, 0xb5dc, PDF_CMAP_RANGE, 1461 },
	{ 0xb5dd, 0xb5df, PDF_CMAP_RANGE, 7870 },
	{ 0xb5e0, 0xb5e2, PDF_CMAP_RANGE, 1473 },
	{ 0xb5e3, 0xb5e3, PDF_CMAP_SINGLE, 7873 },
	{ 0xb5e4, 0xb5e5, PDF_CMAP_RANGE, 1477 },
	{ 0xb5e6, 0xb5e7, PDF_CMAP_RANGE, 7874 },
	{ 0xb5e8, 0xb5ec, PDF_CMAP_RANGE, 1481 },
	{ 0xb5ed, 0xb5ed, PDF_CMAP_SINGLE, 7876 },
	{ 0xb5ee, 0xb5f5, PDF_CMAP_RANGE, 1487 },
	{ 0xb5f6, 0xb5f7, PDF_CMAP_RANGE, 7877 },
	{ 0xb5f8, 0xb5fc, PDF_CMAP_RANGE, 1497 },
	{ 0xb5fd, 0xb5fe, PDF_CMAP_TABLE, 67 },
	{ 0xb6a1, 0xb6a3, PDF_CMAP_RANGE, 1504 },
	{ 0xb6a4, 0xb6a5, PDF_CMAP_RANGE, 7880 },
	{ 0xb6a6, 0xb6ab, PDF_CMAP_TABLE, 69 },
	{ 0xb6ac, 0xb6ae, PDF_CMAP_RANGE, 1515 },
	{ 0xb6af, 0xb6b0, PDF_CMAP_RANGE, 7885 },
	{ 0xb6b1, 0xb6b2, PDF_CMAP_RANGE, 1520 },
	{ 0xb6b3, 0xb6b3, PDF_CMAP_SINGLE, 7887 },
	{ 0xb6b4, 0xb6b6, PDF_CMAP_RANGE, 1523 },
	{ 0xb6b7, 0xb6b7, PDF_CMAP_SINGLE, 7888 },
	{ 0xb6b8, 0xb6be, PDF_CMAP_RANGE, 1527 },
	{ 0xb6bf, 0xb6c1, PDF_CMAP_RANGE, 7889 },
	{ 0xb6c2, 0xb6c3, PDF_CMAP_RANGE, 1537 },
	{ 0xb6c4, 0xb6c6, PDF_CMAP_TABLE, 75 },
	{ 0xb6c7, 0xb6cc, PDF_CMAP_RANGE, 1542 },
	{ 0xb6cd, 0xb6ce, PDF_CMAP_TABLE, 78 },
	{ 0xb6cf, 0xb6d0, PDF_CMAP_RANGE, 7895 },
	{ 0xb6d1, 0xb6d2, PDF_CMAP_RANGE, 1552 },
	{ 0xb6d3, 0xb6d4, PDF_CMAP_RANGE, 7897 },
	{ 0xb6d5, 0xb6d6, PDF_CMAP_TABLE, 80 },
	{ 0xb6d7, 0xb6d8, PDF_CMAP_RANGE, 1558 },
	{ 0xb6d9, 0xb6db, PDF_CMAP_TABLE, 82 },
	{ 0xb6dc, 0xb6e0, PDF_CMAP_RANGE, 1563 },
	{ 0xb6e1, 0xb6e1, PDF_CMAP_SINGLE, 7902 },
	{ 0xb6e2, 0xb6e8, PDF_CMAP_RANGE, 1569 },
	{ 0xb6e9, 0xb6e9, PDF_CMAP_SINGLE, 7903 },
	{ 0xb6ea, 0xb6eb, PDF_CMAP_RANGE, 1577 },
	{ 0xb6ec, 0xb6ed, PDF_CMAP_TABLE, 85 },
	{ 0xb6ee, 0xb6ef, PDF_CMAP_RANGE, 7905 },
	{ 0xb6f0, 0xb6f1, PDF_CMAP_TABLE, 87 },
	{ 0xb6f2, 0xb6f5, PDF_CMAP_RANGE, 1585 },
	{ 0xb6f6, 0xb6f6, PDF_CMAP_SINGLE, 7908 },
	{ 0xb6f7, 0xb6f8, PDF_CMAP_RANGE, 1590 },
	{ 0xb6f9, 0xb6fa, PDF_CMAP_TABLE, 89 },
	{ 0xb6fb, 0xb6fc, PDF_CMAP_RANGE, 7910 },
	{ 0xb6fd, 0xb6fe, PDF_CMAP_RANGE, 1596 },
	{ 0xb7a1, 0xb7a3, PDF_CMAP_RANGE, 7912 },
	{ 0xb7a4, 0xb7a6, PDF_CMAP_RANGE, 1601 },
	{ 0xb7a7, 0xb7a7, PDF_CMAP_SINGLE, 7915 },
	{ 0xb7a8, 0xb7ae, PDF_CMAP_RANGE, 1605 },
	{ 0xb7af, 0xb7b0, PDF_CMAP_RANGE, 7916 },
	{ 0xb7b1, 0xb7b2, PDF_CMAP_RANGE, 1614 },
	{ 0xb7b3, 0xb7b3, PDF_CMAP_SINGLE, 7918 },
	{ 0xb7b4, 0xb7b5, PDF_CMAP_RANGE, 1617 },
	{ 0xb7b6, 0xb7b7, PDF_CMAP_RANGE, 7919 },
	{ 0xb7b8, 0xb7b9, PDF_CMAP_TABLE, 91 },
	{ 0xb7ba, 0xb7c2, PDF_CMAP_RANGE, 1623 },
	{ 0xb7c3, 0xb7c4, PDF_CMAP_RANGE, 7922 },
	{ 0xb7c5, 0xb7c8, PDF_CMAP_RANGE, 1634 },
	{ 0xb7c9, 0xb7c9, PDF_CMAP_SINGLE, 7924 },
	{ 0xb7ca, 0xb7cb, PDF_CMAP_RANGE, 1639 },
	{ 0xb7cc, 0xb7cc, PDF_CMAP_SINGLE, 7925 },
	{ 0xb7cd, 0xb7ce, PDF_CMAP_RANGE, 1642 },
	{ 0xb7cf, 0xb7d1, PDF_CMAP_TABLE, 93 },
	{ 0xb7d2, 0xb7d6, PDF_CMAP_RANGE, 1647 },
	{ 0xb7d7, 0xb7d8, PDF_CMAP_RANGE, 7928 },
	{ 0xb7d9, 0xb7db, PDF_CMAP_RANGE, 1654 },
	{ 0xb7dc, 0xb7dc, PDF_CMAP_SINGLE, 7930 },
	{ 0xb7dd, 0xb7de, PDF_CMAP_RANGE, 1658 },
	{ 0xb7df, 0xb7e1, PDF_CMAP_RANGE, 7931 },
	{ 0xb7e2, 0xb7e3, PDF_CMAP_TABLE, 96 },
	{ 0xb7e4, 0xb7e5, PDF_CMAP_RANGE, 1665 },
	{ 0xb7e6, 0xb7e8, PDF_CMAP_RANGE, 7935 },
	{ 0xb7e9, 0xb7ea, PDF_CMAP_RANGE, 1670 },
	{ 0xb7eb, 0xb7ed, PDF_CMAP_RANGE, 7938 },
	{ 0xb7ee, 0xb7ef, PDF_CMAP_TABLE, 98 },
	{ 0xb7f0, 0xb7f3, PDF_CMAP_RANGE, 1677 },
	{ 0xb7f4, 0xb7f4, PDF_CMAP_SINGLE, 7942 },
	{ 0xb7f5, 0xb7f7, PDF_CMAP_RANGE, 1682 },
	{ 0xb7f8, 0xb7f8, PDF_CMAP_SINGLE, 7943 },
	{ 0xb7f9, 0xb7fe, PDF_CMAP_RANGE, 1686 },
	{ 0xb8a1, 0xb8a6, PDF_CMAP_RANGE, 1692 },
	{ 0xb8a7, 0xb8a8, PDF_CMAP_RANGE, 7944 },
	{ 0xb8a9, 0xb8b2, PDF_CMAP_RANGE, 1700 },
	{ 0xb8b3, 0xb8b4, PDF_CMAP_RANGE, 7946 },
	{ 0xb8b5, 0xb8b9, PDF_CMAP_RANGE, 1712 },
	{ 0xb8ba, 0xb8bd, PDF_CMAP_TABLE, 100 },
	{ 0xb8be, 0xb8bf, PDF_CMAP_RANGE, 7950 },
	{ 0xb8c0, 0xb8c2, PDF_CMAP_RANGE, 1723 },
	{ 0xb8c3, 0xb8c3, PDF_CMAP_SINGLE, 7952 },
	{ 0xb8c4, 0xb8c5, PDF_CMAP_RANGE, 1727 },
	{ 0xb8c6, 0xb8c7, PDF_CMAP_RANGE, 7953 },
	{ 0xb8c8, 0xb8c9, PDF_CMAP_TABLE, 104 },
	{ 0xb8ca, 0xb8ce, PDF_CMAP_RANGE, 1733 },
	{ 0xb8cf, 0xb8cf, PDF_CMAP_SINGLE, 7956 },
	{ 0xb8d0, 0xb8d2, PDF_CMAP_RANGE, 1739 },
	{ 0xb8d3, 0xb8d6, PDF_CMAP_RANGE, 7957 },
	{ 0xb8d7, 0xb8d8, PDF_CMAP_RANGE, 1746 },
	{ 0xb8d9, 0xb8da, PDF_CMAP_RANGE, 7961 },
	{ 0xb8db, 0xb8e3, PDF_CMAP_RANGE, 1750 },
	{ 0xb8e4, 0xb8e4, PDF_CMAP_SINGLE, 7963 },
	{ 0xb8e5, 0xb8e8, PDF_CMAP_RANGE, 1760 },
	{ 0xb8e9, 0xb8eb, PDF_CMAP_TABLE, 106 },
	{ 0xb8ec, 0xb8f2, PDF_CMAP_RANGE, 1767 },
	{ 0xb8f3, 0xb8f4, PDF_CMAP_TABLE, 109 },
	{ 0xb8f5, 0xb8f6, PDF_CMAP_RANGE, 7967 },
	{ 0xb8f7, 0xb8f8, PDF_CMAP_TABLE, 111 },
	{ 0xb8f9, 0xb8fe, PDF_CMAP_RANGE, 1780 },
	{ 0xb9a1, 0xb9a7, PDF_CMAP_RANGE, 1786 },
	{ 0xb9a8, 0xb9a8, PDF_CMAP_SINGLE, 7970 },
	{ 0xb9a9, 0xb9ad, PDF_CMAP_RANGE, 1794 },
	{ 0xb9ae, 0xb9ae, PDF_CMAP_SINGLE, 7971 },
	{ 0xb9af, 0xb9b0, PDF_CMAP_RANGE, 1800 },
	{ 0xb9b1, 0xb9b5, PDF_CMAP_TABLE, 113 },
	{ 0xb9b6, 0xb9b8, PDF_CMAP_RANGE, 1807 },
	{ 0xb9b9, 0xb9ba, PDF_CMAP_RANGE, 7975 },
	{ 0xb9bb, 0xb9c5, PDF_CMAP_RANGE, 1812 },
	{ 0xb9c6, 0xb9c6, PDF_CMAP_SINGLE, 7977 },
	{ 0xb9c7, 0xb9ca, PDF_CMAP_RANGE, 1824 },
	{ 0xb9cb, 0xb9cb, PDF_CMAP_SINGLE, 7978 },
	{ 0xb9cc, 0xb9cf, PDF_CMAP_RANGE, 1829 },
	{ 0xb9d0, 0xb9d0, PDF_CMAP_SINGLE, 7979 },
	{ 0xb9d1, 0xb9d7, PDF_CMAP_RANGE, 1834 },
	{ 0xb9d8, 0xb9d8, PDF_CMAP_SINGLE, 7980 },
	{ 0xb9d9, 0xb9da, PDF_CMAP_RANGE, 1842 },
	{ 0xb9db, 0xb9e3, PDF_CMAP_TABLE, 118 },
	{ 0xb9e4, 0xb9e5, PDF_CMAP_RANGE, 1853 },
	{ 0xb9e6, 0xb9e6, PDF_CMAP_SINGLE, 7986 },
	{ 0xb9e7, 0xb9e8, PDF_CMAP_RANGE, 1856 },
	{ 0xb9e9, 0xb9ec, PDF_CMAP_RANGE, 7987 },
	{ 0xb9ed, 0xb9ee, PDF_CMAP_TABLE, 127 },
	{ 0xb9ef, 0xb9f0, PDF_CMAP_RANGE, 1864 },
	{ 0xb9f1, 0xb9f2, PDF_CMAP_TABLE, 129 },
	{ 0xb9f3, 0xb9f5, PDF_CMAP_RANGE, 7993 },
	{ 0xb9f6, 0xb9f7, PDF_CMAP_RANGE, 1871 },
	{ 0xb9f8, 0xb9fa, PDF_CMAP_TABLE, 131 },
	{ 0xb9fb, 0xb9fc, PDF_CMAP_RANGE, 1876 },
	{ 0xb9fd, 0xb9fe, PDF_CMAP_TABLE, 134 },
	{ 0xbaa1, 0xbaa6, PDF_CMAP_RANGE, 1880 },
	{ 0xbaa7, 0xbaa7, PDF_CMAP_SINGLE, 7999 },
	{ 0xbaa8, 0xbaaa, PDF_CMAP_RANGE, 1887 },
	{ 0xbaab, 0xbaab, PDF_CMAP_SINGLE, 8000 },
	{ 0xbaac, 0xbab9, PDF_CMAP_RANGE, 1891 },
	{ 0xbaba, 0xbaba, PDF_CMAP_SINGLE, 8001 },
	{ 0xbabb, 0xbac4, PDF_CMAP_RANGE, 1906 },
	{ 0xbac5, 0xbac5, PDF_CMAP_SINGLE, 8002 },
	{ 0xbac6, 0xbad1, PDF_CMAP_RANGE, 1917 },
	{ 0xbad2, 0xbad2, PDF_CMAP_SINGLE, 8003 },
	{ 0xbad3, 0xbad6, PDF_CMAP_RANGE, 1930 },
	{ 0xbad7, 0xbad8, PDF_CMAP_RANGE, 8004 },
	{ 0xbad9, 0xbae3, PDF_CMAP_RANGE, 1936 },
	{ 0xbae4, 0xbae4, PDF_CMAP_SINGLE, 8006 },
	{ 0xbae5, 0xbae7, PDF_CMAP_RANGE, 1948 },
	{ 0xbae8, 0xbae8, PDF_CMAP_SINGLE, 8007 },
	{ 0xbae9, 0xbaeb, PDF_CMAP_RANGE, 1952 },
	{ 0xbaec, 0xbaec, PDF_CMAP_SINGLE, 8008 },
	{ 0xbaed, 0xbaf2, PDF_CMAP_RANGE, 1956 },
	{ 0xbaf3, 0xbaf3, PDF_CMAP_SINGLE, 5605 },
	{ 0xbaf4, 0xbaf7, PDF_CMAP_RANGE, 1963 },
	{ 0xbaf8, 0xbaf8, PDF_CMAP_SINGLE, 8009 },
	{ 0xbaf9, 0xbafe, PDF_CMAP_RANGE, 1968 },
	{ 0xbba1, 0xbba3, PDF_CMAP_RANGE, 1974 },
	{ 0xbba4, 0xbba6, PDF_CMAP_TABLE, 136 },
	{ 0xbba7, 0xbba8, PDF_CMAP_RANGE, 1980 },
	{ 0xbba9, 0xbbaa, PDF_CMAP_RANGE, 8012 },
	{ 0xbbab, 0xbbac, PDF_CMAP_RANGE, 1984 },
	{ 0xbbad, 0xbbae, PDF_CMAP_RANGE, 8014 },
	{ 0xbbaf, 0xbbb0, PDF_CMAP_TABLE, 139 },
	{ 0xbbb1, 0xbbb2, PDF_CMAP_RANGE, 1990 },
	{ 0xbbb3, 0xbbb4, PDF_CMAP_TABLE, 141 },
	{ 0xbbb5, 0xbbb7, PDF_CMAP_RANGE, 8018 },
	{ 0xbbb8, 0xbbb8, PDF_CMAP_SINGLE, 1997 },
	{ 0xbbb9, 0xbbba, PDF_CMAP_RANGE, 8021 },
	{ 0xbbbb, 0xbbd0, PDF_CMAP_RANGE, 2000 },
	{ 0xbbd1, 0xbbd2, PDF_CMAP_TABLE, 143 },
	{ 0xbbd3, 0xbbd4, PDF_CMAP_RANGE, 8024 },
	{ 0xbbd5, 0xbbde, PDF_CMAP_RANGE, 2026 },
	{ 0xbbdf, 0xbbe7, PDF_CMAP_RANGE, 8026 },
	{ 0xbbe8, 0xbbea, PDF_CMAP_RANGE, 2045 },
	{ 0xbbeb, 0xbbeb, PDF_CMAP_SINGLE, 8035 },
	{ 0xbbec, 0xbbf0, PDF_CMAP_RANGE, 2049 },
	{ 0xbbf1, 0xbbf1, PDF_CMAP_SINGLE, 8036 },
	{ 0xbbf2, 0xbbf4, PDF_CMAP_RANGE, 2055 },
	{ 0xbbf5, 0xbbf7, PDF_CMAP_RANGE, 8037 },
	{ 0xbbf8, 0xbbf9, PDF_CMAP_RANGE, 2061 },
	{ 0xbbfa, 0xbbfa, PDF_CMAP_SINGLE, 8040 },
	{ 0xbbfb, 0xbbfc, PDF_CMAP_RANGE, 2064 },
	{ 0xbbfd, 0xbbfe, PDF_CMAP_TABLE, 145 },
	{ 0xbca1, 0xbca2, PDF_CMAP_TABLE, 147 },
	{ 0xbca3, 0xbca4, PDF_CMAP_RANGE, 2070 },
	{ 0xbca5, 0xbca6, PDF_CMAP_RANGE, 8043 },
	{ 0xbca7, 0xbca7, PDF_CMAP_SINGLE, 2074 },
	{ 0xbca8, 0xbca9, PDF_CMAP_RANGE, 8045 },
	{ 0xbcaa, 0xbcad, PDF_CMAP_TABLE, 149 },
	{ 0xbcae, 0xbcb5, PDF_CMAP_RANGE, 2081 },
	{ 0xbcb6, 0xbcb8, PDF_CMAP_RANGE, 8049 },
	{ 0xbcb9, 0xbcba, PDF_CMAP_RANGE, 2092 },
	{ 0xbcbb, 0xbcbb, PDF_CMAP_SINGLE, 8052 },
	{ 0xbcbc, 0xbcc0, PDF_CMAP_RANGE, 2095 },
	{ 0xbcc1, 0xbcc3, PDF_CMAP_TABLE, 153 },
	{ 0xbcc4, 0xbcc5, PDF_CMAP_RANGE, 2103 },
	{ 0xbcc6, 0xbcc7, PDF_CMAP_RANGE, 8055 },
	{ 0xbcc8, 0xbcc9, PDF_CMAP_RANGE, 2107 },
	{ 0xbcca, 0xbccb, PDF_CMAP_TABLE, 156 },
	{ 0xbccc, 0xbccd, PDF_CMAP_RANGE, 8058 },
	{ 0xbcce, 0xbccf, PDF_CMAP_RANGE, 2113 },
	{ 0xbcd0, 0xbcd0, PDF_CMAP_SINGLE, 8060 },
	{ 0xbcd1, 0xbcd3, PDF_CMAP_RANGE, 2116 },
	{ 0xbcd4, 0xbcd6, PDF_CMAP_RANGE, 8061 },
	{ 0xbcd7, 0xbcd8, PDF_CMAP_TABLE, 158 },
	{ 0xbcd9, 0xbcda, PDF_CMAP_RANGE, 2124 },
	{ 0xbcdb, 0xbcde, PDF_CMAP_TABLE, 160 },
	{ 0xbcdf, 0xbce1, PDF_CMAP_RANGE, 8067 },
	{ 0xbce2, 0xbce2, PDF_CMAP_SINGLE, 2133 },
	{ 0xbce3, 0xbce4, PDF_CMAP_RANGE, 8070 },
	{ 0xbce5, 0xbce7, PDF_CMAP_RANGE, 2136 },
	{ 0xbce8, 0xbce9, PDF_CMAP_TABLE, 164 },
	{ 0xbcea, 0xbcec, PDF_CMAP_RANGE, 8073 },
	{ 0xbced, 0xbcee, PDF_CMAP_RANGE, 2144 },
	{ 0xbcef, 0xbcf3, PDF_CMAP_RANGE, 8076 },
	{ 0xbcf4, 0xbcf5, PDF_CMAP_RANGE, 2151 },
	{ 0xbcf6, 0xbcfc, PDF_CMAP_RANGE, 8081 },
	{ 0xbcfd, 0xbcfe, PDF_CMAP_RANGE, 2160 },
	{ 0xbda1, 0xbda1, PDF_CMAP_SINGLE, 2162 },
	{ 0xbda2, 0xbda7, PDF_CMAP_RANGE, 8088 },
	{ 0xbda8, 0xbdaa, PDF_CMAP_RANGE, 2169 },
	{ 0xbdab, 0xbdac, PDF_CMAP_RANGE, 8094 },
	{ 0xbdad, 0xbdae, PDF_CMAP_RANGE, 2174 },
	{ 0xbdaf, 0xbdb2, PDF_CMAP_RANGE, 8096 },
	{ 0xbdb3, 0xbdb4, PDF_CMAP_TABLE, 166 },
	{ 0xbdb5, 0xbdb9, PDF_CMAP_RANGE, 2182 },
	{ 0xbdba, 0xbdba, PDF_CMAP_SINGLE, 8101 },
	{ 0xbdbb, 0xbdbc, PDF_CMAP_RANGE, 2188 },
	{ 0xbdbd, 0xbdbf, PDF_CMAP_RANGE, 8102 },
	{ 0xbdc0, 0xbdc0, PDF_CMAP_SINGLE, 2193 },
	{ 0xbdc1, 0xbdc4, PDF_CMAP_RANGE, 8105 },
	{ 0xbdc5, 0xbdc7, PDF_CMAP_RANGE, 2198 },
	{ 0xbdc8, 0xbdca, PDF_CMAP_RANGE, 8109 },
	{ 0xbdcb, 0xbdcd, PDF_CMAP_RANGE, 2204 },
	{ 0xbdce, 0xbdcf, PDF_CMAP_RANGE, 8112 },
	{ 0xbdd0, 0xbdd6, PDF_CMAP_RANGE, 2209 },
	{ 0xbdd7, 0xbdd7, PDF_CMAP_SINGLE, 8114 },
	{ 0xbdd8, 0xbdd9, PDF_CMAP_RANGE, 2217 },
	{ 0xbdda, 0xbdda, PDF_CMAP_SINGLE, 8115 },
	{ 0xbddb, 0xbddf, PDF_CMAP_RANGE, 2220 },
	{ 0xbde0, 0xbde1, PDF_CMAP_RANGE, 8116 },
	{ 0xbde2, 0xbdea, PDF_CMAP_RANGE, 2227 },
	{ 0xbdeb, 0xbdeb, PDF_CMAP_SINGLE, 8118 },
	{ 0xbdec, 0xbdf3, PDF_CMAP_RANGE, 2237 },
	{ 0xbdf4, 0xbdf8, PDF_CMAP_RANGE, 8119 },
	{ 0xbdf9, 0xbdfc, PDF_CMAP_RANGE, 2250 },
	{ 0xbdfd, 0xbdfe, PDF_CMAP_TABLE, 168 },
	{ 0xbea1, 0xbea2, PDF_CMAP_RANGE, 8125 },
	{ 0xbea3, 0xbea4, PDF_CMAP_RANGE, 2258 },
	{ 0xbea5, 0xbea5, PDF_CMAP_SINGLE, 8127 },
	{ 0xbea6, 0xbea7, PDF_CMAP_RANGE, 2261 },
	{ 0xbea8, 0xbeaa, PDF_CMAP_TABLE, 170 },
	{ 0xbeab, 0xbeac, PDF_CMAP_RANGE, 2266 },
	{ 0xbead, 0xbead, PDF_CMAP_SINGLE, 8130 },
	{ 0xbeae, 0xbeb0, PDF_CMAP_RANGE, 2269 },
	{ 0xbeb1, 0xbeb1, PDF_CMAP_SINGLE, 8131 },
	{ 0xbeb2, 0xbeb4, PDF_CMAP_RANGE, 2273 },
	{ 0xbeb5, 0xbeb7, PDF_CMAP_RANGE, 8132 },
	{ 0xbeb8, 0xbeb9, PDF_CMAP_RANGE, 2279 },
	{ 0xbeba, 0xbeba, PDF_CMAP_SINGLE, 8135 },
	{ 0xbebb, 0xbebf, PDF_CMAP_RANGE, 2282 },
	{ 0xbec0, 0xbec0, PDF_CMAP_SINGLE, 8136 },
	{ 0xbec1, 0xbec8, PDF_CMAP_RANGE, 2288 },
	{ 0xbec9, 0xbec9, PDF_CMAP_SINGLE, 8137 },
	{ 0xbeca, 0xbed3, PDF_CMAP_RANGE, 2297 },
	{ 0xbed4, 0xbed4, PDF_CMAP_SINGLE, 8138 },
	{ 0xbed5, 0xbed8, PDF_CMAP_RANGE, 2308 },
	{ 0xbed9, 0xbed9, PDF_CMAP_SINGLE, 8139 },
	{ 0xbeda, 0xbedc, PDF_CMAP_RANGE, 2313 },
	{ 0xbedd, 0xbedd, PDF_CMAP_SINGLE, 8140 },
	{ 0xbede, 0xbee1, PDF_CMAP_RANGE, 2317 },
	{ 0xbee2, 0xbee2, PDF_CMAP_SINGLE, 8141 },
	{ 0xbee3, 0xbee4, PDF_CMAP_RANGE, 2322 },
	{ 0xbee5, 0xbee9, PDF_CMAP_TABLE, 173 },
	{ 0xbeea, 0xbeed, PDF_CMAP_RANGE, 2329 },
	{ 0xbeee, 0xbeee, PDF_CMAP_SINGLE, 8145 },
	{ 0xbeef, 0xbef4, PDF_CMAP_RANGE, 2334 },
	{ 0xbef5, 0xbef6, PDF_CMAP_TABLE, 178 },
	{ 0xbef7, 0xbef8, PDF_CMAP_RANGE, 8147 },
	{ 0xbef9, 0xbefa, PDF_CMAP_RANGE, 2344 },
	{ 0xbefb, 0xbefc, PDF_CMAP_RANGE, 8149 },
	{ 0xbefd, 0xbefe, PDF_CMAP_RANGE, 2348 },
	{ 0xbfa1, 0xbfa4, PDF_CMAP_RANGE, 2350 },
	{ 0xbfa5, 0xbfa5, PDF_CMAP_SINGLE, 8151 },
	{ 0xbfa6, 0xbfa9, PDF_CMAP_RANGE, 2355 },
	{ 0xbfaa, 0xbfaa, PDF_CMAP_SINGLE, 8152 },
	{ 0xbfab, 0xbfac, PDF_CMAP_RANGE, 2360 },
	{ 0xbfad, 0xbfad, PDF_CMAP_SINGLE, 8153 },
	{ 0xbfae, 0xbfc4, PDF_CMAP_RANGE, 2363 },
	{ 0xbfc5, 0xbfc7, PDF_CMAP_TABLE, 180 },
	{ 0xbfc8, 0xbfcd, PDF_CMAP_RANGE, 2389 },
	{ 0xbfce, 0xbfce, PDF_CMAP_SINGLE, 8156 },
	{ 0xbfcf, 0xbfd0, PDF_CMAP_RANGE, 2396 },
	{ 0xbfd1, 0xbfd2, PDF_CMAP_RANGE, 8157 },
	{ 0xbfd3, 0xbfd8, PDF_CMAP_RANGE, 2400 },
	{ 0xbfd9, 0xbfd9, PDF_CMAP_SINGLE, 8159 },
	{ 0xbfda, 0xbfe1, PDF_CMAP_RANGE, 2407 },
	{ 0xbfe2, 0xbfe4, PDF_CMAP_RANGE, 8160 },
	{ 0xbfe5, 0xbfe8, PDF_CMAP_RANGE, 2418 },
	{ 0xbfe9, 0xbfed, PDF_CMAP_TABLE, 183 },
	{ 0xbfee, 0xbff2, PDF_CMAP_RANGE, 2427 },
	{ 0xbff3, 0xbff8, PDF_CMAP_TABLE, 188 },
	{ 0xbff9, 0xbffa, PDF_CMAP_RANGE, 8169 },
	{ 0xbffb, 0xbffe, PDF_CMAP_RANGE, 2440 },
	{ 0xc0a1, 0xc0a3, PDF_CMAP_TABLE, 194 },
	{ 0xc0a4, 0xc0a8, PDF_CMAP_RANGE, 2447 },
	{ 0xc0a9, 0xc0ab, PDF_CMAP_TABLE, 197 },
	{ 0xc0ac, 0xc0ae, PDF_CMAP_RANGE, 2455 },
	{ 0xc0af, 0xc0b0, PDF_CMAP_RANGE, 8175 },
	{ 0xc0b1, 0xc0b2, PDF_CMAP_RANGE, 2460 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 8177 },
	{ 0xc0b7, 0xc0b7, PDF_CMAP_SINGLE, 2466 },
	{ 0xc0b8, 0xc0c4, PDF_CMAP_RANGE, 8181 },
	{ 0xc0c5, 0xc0cb, PDF_CMAP_RANGE, 2480 },
	{ 0xc0cc, 0xc0cd, PDF_CMAP_RANGE, 8194 },
	{ 0xc0ce, 0xc0d3, PDF_CMAP_RANGE, 2489 },
	{ 0xc0d4, 0xc0d8, PDF_CMAP_TABLE, 200 },
	{ 0xc0d9, 0xc0dc, PDF_CMAP_RANGE, 2500 },
	{ 0xc0dd, 0xc0dd, PDF_CMAP_SINGLE, 8199 },
	{ 0xc0de, 0xc0df, PDF_CMAP_RANGE, 2505 },
	{ 0xc0e0, 0xc0e0, PDF_CMAP_SINGLE, 8200 },
	{ 0xc0e1, 0xc0e8, PDF_CMAP_RANGE, 2508 },
	{ 0xc0e9, 0xc0ea, PDF_CMAP_TABLE, 205 },
	{ 0xc0eb, 0xc0ec, PDF_CMAP_RANGE, 8202 },
	{ 0xc0ed, 0xc0ee, PDF_CMAP_RANGE, 2520 },
	{ 0xc0ef, 0xc0f1, PDF_CMAP_RANGE, 8204 },
	{ 0xc0f2, 0xc0f5, PDF_CMAP_RANGE, 2525 },
	{ 0xc0f6, 0xc0fa, PDF_CMAP_RANGE, 8207 },
	{ 0xc0fb, 0xc0fe, PDF_CMAP_RANGE, 2534 },
	{ 0xc1a1, 0xc1a3, PDF_CMAP_RANGE, 2538 },
	{ 0xc1a4, 0xc1a5, PDF_CMAP_RANGE, 8212 },
	{ 0xc1a6, 0xc1a8, PDF_CMAP_RANGE, 2543 },
	{ 0xc1a9, 0xc1ad, PDF_CMAP_RANGE, 8214 },
	{ 0xc1ae, 0xc1ae, PDF_CMAP_SINGLE, 2551 },
	{ 0xc1af, 0xc1b8, PDF_CMAP_RANGE, 8219 },
	{ 0xc1b9, 0xc1bc, PDF_CMAP_RANGE, 2562 },
	{ 0xc1bd, 0xc1be, PDF_CMAP_RANGE, 8229 },
	{ 0xc1bf, 0xc1c1, PDF_CMAP_RANGE, 2568 },
	{ 0xc1c2, 0xc1c2, PDF_CMAP_SINGLE, 8231 },
	{ 0xc1c3, 0xc1c5, PDF_CMAP_RANGE, 2572 },
	{ 0xc1c6, 0xc1c6, PDF_CMAP_SINGLE, 8232 },
	{ 0xc1c7, 0xc1c8, PDF_CMAP_RANGE, 2576 },
	{ 0xc1c9, 0xc1c9, PDF_CMAP_SINGLE, 8233 },
	{ 0xc1ca, 0xc1cc, PDF_CMAP_RANGE, 2579 },
	{ 0xc1cd, 0xc1cd, PDF_CMAP_SINGLE, 8234 },
	{ 0xc1ce, 0xc1d3, PDF_CMAP_RANGE, 2583 },
	{ 0xc1d4, 0xc1d4, PDF_CMAP_SINGLE, 8235 },
	{ 0xc1d5, 0xc1d8, PDF_CMAP_RANGE, 2590 },
	{ 0xc1d9, 0xc1db, PDF_CMAP_RANGE, 8236 },
	{ 0xc1dc, 0xc1dd, PDF_CMAP_RANGE, 2597 },
	{ 0xc1de, 0xc1de, PDF_CMAP_SINGLE, 8239 },
	{ 0xc1df, 0xc1e3, PDF_CMAP_RANGE, 2600 },
	{ 0xc1e4, 0xc1e5, PDF_CMAP_RANGE, 8240 },
	{ 0xc1e6, 0xc1e8, PDF_CMAP_RANGE, 2607 },
	{ 0xc1e9, 0xc1ea, PDF_CMAP_TABLE, 207 },
	{ 0xc1eb, 0xc1ec, PDF_CMAP_RANGE, 8243 },
	{ 0xc1ed, 0xc1f2, PDF_CMAP_RANGE, 2614 },
	{ 0xc1f3, 0xc1f5, PDF_CMAP_TABLE, 209 },
	{ 0xc1f6, 0xc1f9, PDF_CMAP_RANGE, 2623 },
	{ 0xc1fa, 0xc1fd, PDF_CMAP_RANGE, 8247 },
	{ 0xc1fe, 0xc1fe, PDF_CMAP_SINGLE, 2631 },
	{ 0xc2a1, 0xc2a1, PDF_CMAP_SINGLE, 2632 },
	{ 0xc2a2, 0xc2a8, PDF_CMAP_RANGE, 8251 },
	{ 0xc2a9, 0xc2aa, PDF_CMAP_RANGE, 2640 },
	{ 0xc2ab, 0xc2b3, PDF_CMAP_RANGE, 8258 },
	{ 0xc2b4, 0xc2b7, PDF_CMAP_RANGE, 2651 },
	{ 0xc2b8, 0xc2b8, PDF_CMAP_SINGLE, 8267 },
	{ 0xc2b9, 0xc2bb, PDF_CMAP_RANGE, 2656 },
	{ 0xc2bc, 0xc2bd, PDF_CMAP_RANGE, 8268 },
	{ 0xc2be, 0xc2c1, PDF_CMAP_TABLE, 212 },
	{ 0xc2c2, 0xc2c4, PDF_CMAP_RANGE, 2665 },
	{ 0xc2c5, 0xc2c7, PDF_CMAP_RANGE, 8272 },
	{ 0xc2c8, 0xc2ca, PDF_CMAP_RANGE, 2671 },
	{ 0xc2cb, 0xc2d0, PDF_CMAP_RANGE, 8275 },
	{ 0xc2d1, 0xc2d2, PDF_CMAP_TABLE, 216 },
	{ 0xc2d3, 0xc2d4, PDF_CMAP_RANGE, 2682 },
	{ 0xc2d5, 0xc2dc, PDF_CMAP_RANGE, 8282 },
	{ 0xc2dd, 0xc2dd, PDF_CMAP_SINGLE, 2692 },
	{ 0xc2de, 0xc2e2, PDF_CMAP_RANGE, 8290 },
	{ 0xc2e3, 0xc2e5, PDF_CMAP_RANGE, 2698 },
	{ 0xc2e6, 0xc2e8, PDF_CMAP_RANGE, 8295 },
	{ 0xc2e9, 0xc2e9, PDF_CMAP_SINGLE, 2704 },
	{ 0xc2ea, 0xc2ee, PDF_CMAP_RANGE, 8298 },
	{ 0xc2ef, 0xc2f1, PDF_CMAP_TABLE, 218 },
	{ 0xc2f2, 0xc2f5, PDF_CMAP_RANGE, 8304 },
	{ 0xc2f6, 0xc2f6, PDF_CMAP_SINGLE, 2717 },
	{ 0xc2f7, 0xc2fa, PDF_CMAP_RANGE, 8308 },
	{ 0xc2fb, 0xc2fe, PDF_CMAP_RANGE, 2722 },
	{ 0xc3a1, 0xc3a1, PDF_CMAP_SINGLE, 8312 },
	{ 0xc3a2, 0xc3a9, PDF_CMAP_RANGE, 2727 },
	{ 0xc3aa, 0xc3aa, PDF_CMAP_SINGLE, 8313 },
	{ 0xc3ab, 0xc3ac, PDF_CMAP_RANGE, 2736 },
	{ 0xc3ad, 0xc3ad, PDF_CMAP_SINGLE, 8314 },
	{ 0xc3ae, 0xc3b2, PDF_CMAP_RANGE, 2739 },
	{ 0xc3b3, 0xc3b4, PDF_CMAP_TABLE, 221 },
	{ 0xc3b5, 0xc3bd, PDF_CMAP_RANGE, 2746 },
	{ 0xc3be, 0xc3be, PDF_CMAP_SINGLE, 8316 },
	{ 0xc3bf, 0xc3c4, PDF_CMAP_RANGE, 2756 },
	{ 0xc3c5, 0xc3c7, PDF_CMAP_RANGE, 8317 },
	{ 0xc3c8, 0xc3cb, PDF_CMAP_RANGE, 2765 },
	{ 0xc3cc, 0xc3ce, PDF_CMAP_TABLE, 223 },
	{ 0xc3cf, 0xc3d4, PDF_CMAP_RANGE, 2772 },
	{ 0xc3d5, 0xc3d6, PDF_CMAP_RANGE, 8322 },
	{ 0xc3d7, 0xc3d8, PDF_CMAP_RANGE, 2780 },
	{ 0xc3d9, 0xc3d9, PDF_CMAP_SINGLE, 8324 },
	{ 0xc3da, 0xc3df, PDF_CMAP_RANGE, 2783 },
	{ 0xc3e0, 0xc3e0, PDF_CMAP_SINGLE, 8325 },
	{ 0xc3e1, 0xc3e4, PDF_CMAP_RANGE, 2790 },
	{ 0xc3e5, 0xc3e5, PDF_CMAP_SINGLE, 8326 },
	{ 0xc3e6, 0xc3ec, PDF_CMAP_RANGE, 2795 },
	{ 0xc3ed, 0xc3ed, PDF_CMAP_SINGLE, 8327 },
	{ 0xc3ee, 0xc3ef, PDF_CMAP_RANGE, 2803 },
	{ 0xc3f0, 0xc3f0, PDF_CMAP_SINGLE, 8328 },
	{ 0xc3f1, 0xc3f4, PDF_CMAP_RANGE, 2806 },
	{ 0xc3f5, 0xc3f6, PDF_CMAP_RANGE, 8329 },
	{ 0xc3f7, 0xc3f8, PDF_CMAP_RANGE, 2812 },
	{ 0xc3f9, 0xc3fa, PDF_CMAP_RANGE, 8331 },
	{ 0xc3fb, 0xc3fc, PDF_CMAP_RANGE, 2816 },
	{ 0xc3fd, 0xc3fe, PDF_CMAP_TABLE, 226 },
	{ 0xc4a1, 0xc4b0, PDF_CMAP_RANGE, 2820 },
	{ 0xc4b1, 0xc4b1, PDF_CMAP_SINGLE, 8334 },
	{ 0xc4b2, 0xc4b5, PDF_CMAP_RANGE, 2837 },
	{ 0xc4b6, 0xc4b6, PDF_CMAP_SINGLE, 8335 },
	{ 0xc4b7, 0xc4c5, PDF_CMAP_RANGE, 2842 },
	{ 0xc4c6, 0xc4c6, PDF_CMAP_SINGLE, 8336 },
	{ 0xc4c7, 0xc4c8, PDF_CMAP_RANGE, 2858 },
	{ 0xc4c9, 0xc4c9, PDF_CMAP_SINGLE, 8337 },
	{ 0xc4ca, 0xc4d0, PDF_CMAP_RANGE, 2861 },
	{ 0xc4d1, 0xc4d2, PDF_CMAP_TABLE, 228 },
	{ 0xc4d3, 0xc4d6, PDF_CMAP_RANGE, 8339 },
	{ 0xc4d7, 0xc4d8, PDF_CMAP_RANGE, 2874 },
	{ 0xc4d9, 0xc4d9, PDF_CMAP_SINGLE, 8343 },
	{ 0xc4da, 0xc4e1, PDF_CMAP_RANGE, 2877 },
	{ 0xc4e2, 0xc4e2, PDF_CMAP_SINGLE, 8344 },
	{ 0xc4e3, 0xc4e4, PDF_CMAP_RANGE, 2886 },
	{ 0xc4e5, 0xc4e5, PDF_CMAP_SINGLE, 8345 },
	{ 0xc4e6, 0xc4eb, PDF_CMAP_RANGE, 2889 },
	{ 0xc4ec, 0xc4ec, PDF_CMAP_SINGLE, 8346 },
	{ 0xc4ed, 0xc4ef, PDF_CMAP_RANGE, 2896 },
	{ 0xc4f0, 0xc4f1, PDF_CMAP_RANGE, 8347 },
	{ 0xc4f2, 0xc4f3, PDF_CMAP_RANGE, 2901 },
	{ 0xc4f4, 0xc4f5, PDF_CMAP_TABLE, 230 },
	{ 0xc4f6, 0xc4f8, PDF_CMAP_RANGE, 8350 },
	{ 0xc4f9, 0xc4fa, PDF_CMAP_RANGE, 2908 },
	{ 0xc4fb, 0xc4fc, PDF_CMAP_RANGE, 8353 },
	{ 0xc4fd, 0xc4fe, PDF_CMAP_TABLE, 232 },
	{ 0xc5a1, 0xc5a2, PDF_CMAP_RANGE, 8356 },
	{ 0xc5a3, 0xc5a4, PDF_CMAP_RANGE, 2916 },
	{ 0xc5a5, 0xc5a9, PDF_CMAP_RANGE, 8358 },
	{ 0xc5aa, 0xc5b0, PDF_CMAP_RANGE, 2923 },
	{ 0xc5b1, 0xc5b1, PDF_CMAP_SINGLE, 8363 },
	{ 0xc5b2, 0xc5b4, PDF_CMAP_RANGE, 2931 },
	{ 0xc5b5, 0xc5b6, PDF_CMAP_TABLE, 234 },
	{ 0xc5b7, 0xc5b9, PDF_CMAP_RANGE, 8365 },
	{ 0xc5ba, 0xc5bd, PDF_CMAP_TABLE, 236 },
	{ 0xc5be, 0xc5cb, PDF_CMAP_RANGE, 2943 },
	{ 0xc5cc, 0xc5cc, PDF_CMAP_SINGLE, 8370 },
	{ 0xc5cd, 0xc5d2, PDF_CMAP_RANGE, 2958 },
	{ 0xc5d3, 0xc5d3, PDF_CMAP_SINGLE, 8371 },
	{ 0xc5d4, 0xc5e1, PDF_CMAP_RANGE, 2965 },
	{ 0xc5e2, 0xc5e2, PDF_CMAP_SINGLE, 8372 },
	{ 0xc5e3, 0xc5e6, PDF_CMAP_RANGE, 2980 },
	{ 0xc5e7, 0xc5e7, PDF_CMAP_SINGLE, 8373 },
	{ 0xc5e8, 0xc5f3, PDF_CMAP_RANGE, 2985 },
	{ 0xc5f4, 0xc5f4, PDF_CMAP_SINGLE, 8374 },
	{ 0xc5f5, 0xc5fe, PDF_CMAP_RANGE, 2998 },
	{ 0xc6a1, 0xc6ac, PDF_CMAP_RANGE, 3008 },
	{ 0xc6ad, 0xc6ae, PDF_CMAP_RANGE, 8375 },
	{ 0xc6af, 0xc6b4, PDF_CMAP_RANGE, 3022 },
	{ 0xc6b5, 0xc6b6, PDF_CMAP_RANGE, 8377 },
	{ 0xc6b7, 0xc6ba, PDF_CMAP_RANGE, 3030 },
	{ 0xc6bb, 0xc6bb, PDF_CMAP_SINGLE, 8379 },
	{ 0xc6bc, 0xc6bd, PDF_CMAP_RANGE, 3035 },
	{ 0xc6be, 0xc6c0, PDF_CMAP_TABLE, 240 },
	{ 0xc6c1, 0xc6c2, PDF_CMAP_RANGE, 3040 },
	{ 0xc6c3, 0xc6c4, PDF_CMAP_RANGE, 8382 },
	{ 0xc6c5, 0xc6ca, PDF_CMAP_RANGE, 3044 },
	{ 0xc6cb, 0xc6cd, PDF_CMAP_RANGE, 8384 },
	{ 0xc6ce, 0xc6d2, PDF_CMAP_RANGE, 3053 },
	{ 0xc6d3, 0xc6d3, PDF_CMAP_SINGLE, 8387 },
	{ 0xc6d4, 0xc6d6, PDF_CMAP_RANGE, 3059 },
	{ 0xc6d7, 0xc6d7, PDF_CMAP_SINGLE, 8388 },
	{ 0xc6d8, 0xc6e9, PDF_CMAP_RANGE, 3063 },
	{ 0xc6ea, 0xc6eb, PDF_CMAP_RANGE, 8389 },
	{ 0xc6ec, 0xc6ee, PDF_CMAP_RANGE, 3083 },
	{ 0xc6ef, 0xc6f1, PDF_CMAP_TABLE, 243 },
	{ 0xc6f2, 0xc6f3, PDF_CMAP_RANGE, 3089 },
	{ 0xc6f4, 0xc6f4, PDF_CMAP_SINGLE, 8393 },
	{ 0xc6f5, 0xc6f7, PDF_CMAP_RANGE, 3092 },
	{ 0xc6f8, 0xc6f8, PDF_CMAP_SINGLE, 8394 },
	{ 0xc6f9, 0xc6fc, PDF_CMAP_RANGE, 3096 },
	{ 0xc6fd, 0xc6fe, PDF_CMAP_TABLE, 246 },
	{ 0xc7a1, 0xc7a2, PDF_CMAP_RANGE, 3102 },
	{ 0xc7a3, 0xc7a4, PDF_CMAP_TABLE, 248 },
	{ 0xc7a5, 0xc7a6, PDF_CMAP_RANGE, 8397 },
	{ 0xc7a7, 0xc7a7, PDF_CMAP_SINGLE, 3108 },
	{ 0xc7a8, 0xc7a9, PDF_CMAP_RANGE, 8399 },
	{ 0xc7aa, 0xc7ab, PDF_CMAP_TABLE, 250 },
	{ 0xc7ac, 0xc7ad, PDF_CMAP_RANGE, 3113 },
	{ 0xc7ae, 0xc7af, PDF_CMAP_RANGE, 8402 },
	{ 0xc7b0, 0xc7b2, PDF_CMAP_RANGE, 3117 },
	{ 0xc7b3, 0xc7b5, PDF_CMAP_RANGE, 8404 },
	{ 0xc7b6, 0xc7b8, PDF_CMAP_RANGE, 3123 },
	{ 0xc7b9, 0xc7ba, PDF_CMAP_RANGE, 8407 },
	{ 0xc7bb, 0xc7bc, PDF_CMAP_RANGE, 3128 },
	{ 0xc7bd, 0xc7be, PDF_CMAP_RANGE, 8409 },
	{ 0xc7bf, 0xc7c2, PDF_CMAP_TABLE, 252 },
	{ 0xc7c3, 0xc7c4, PDF_CMAP_RANGE, 3136 },
	{ 0xc7c5, 0xc7c6, PDF_CMAP_TABLE, 256 },
	{ 0xc7c7, 0xc7c8, PDF_CMAP_RANGE, 8414 },
	{ 0xc7c9, 0xc7cb, PDF_CMAP_RANGE, 3142 },
	{ 0xc7cc, 0xc7cc, PDF_CMAP_SINGLE, 8416 },
	{ 0xc7cd, 0xc7ce, PDF_CMAP_RANGE, 3146 },
	{ 0xc7cf, 0xc7cf, PDF_CMAP_SINGLE, 8417 },
	{ 0xc7d0, 0xc7d3, PDF_CMAP_RANGE, 3149 },
	{ 0xc7d4, 0xc7d5, PDF_CMAP_RANGE, 8418 },
	{ 0xc7d6, 0xc7d7, PDF_CMAP_TABLE, 258 },
	{ 0xc7d8, 0xc7dd, PDF_CMAP_RANGE, 3157 },
	{ 0xc7de, 0xc7de, PDF_CMAP_SINGLE, 8421 },
	{ 0xc7df, 0xc7e0, PDF_CMAP_RANGE, 3164 },
	{ 0xc7e1, 0xc7e3, PDF_CMAP_RANGE, 8422 },
	{ 0xc7e4, 0xc7e9, PDF_CMAP_RANGE, 3169 },
	{ 0xc7ea, 0xc7ee, PDF_CMAP_RANGE, 8425 },
	{ 0xc7ef, 0xc7f6, PDF_CMAP_RANGE, 3180 },
	{ 0xc7f7, 0xc7f8, PDF_CMAP_RANGE, 8430 },
	{ 0xc7f9, 0xc7fa, PDF_CMAP_RANGE, 3190 },
	{ 0xc7fb, 0xc7fe, PDF_CMAP_TABLE, 260 },
	{ 0xc8a1, 0xc8a2, PDF_CMAP_RANGE, 3196 },
	{ 0xc8a3, 0xc8a3, PDF_CMAP_SINGLE, 8434 },
	{ 0xc8a4, 0xc8a6, PDF_CMAP_RANGE, 3199 },
	{ 0xc8a7, 0xc8a8, PDF_CMAP_RANGE, 8435 },
	{ 0xc8a9, 0xc8af, PDF_CMAP_RANGE, 3204 },
	{ 0xc8b0, 0xc8b0, PDF_CMAP_SINGLE, 8437 },
	{ 0xc8b1, 0xc8b4, PDF_CMAP_RANGE, 3212 },
	{ 0xc8b5, 0xc8b7, PDF_CMAP_TABLE, 264 },
	{ 0xc8b8, 0xc8c2, PDF_CMAP_RANGE, 3219 },
	{ 0xc8c3, 0xc8c6, PDF_CMAP_RANGE, 8440 },
	{ 0xc8c7, 0xc8c8, PDF_CMAP_TABLE, 267 },
	{ 0xc8c9, 0xc8cc, PDF_CMAP_RANGE, 3236 },
	{ 0xc8cd, 0xc8cf, PDF_CMAP_TABLE, 269 },
	{ 0xc8d0, 0xc8d1, PDF_CMAP_RANGE, 3243 },
	{ 0xc8d2, 0xc8d2, PDF_CMAP_SINGLE, 8447 },
	{ 0xc8d3, 0xc8d8, PDF_CMAP_RANGE, 3246 },
	{ 0xc8d9, 0xc8d9, PDF_CMAP_SINGLE, 8448 },
	{ 0xc8da, 0xc8dd, PDF_CMAP_RANGE, 3253 },
	{ 0xc8de, 0xc8de, PDF_CMAP_SINGLE, 8449 },
	{ 0xc8df, 0xc8ec, PDF_CMAP_RANGE, 3258 },
	{ 0xc8ed, 0xc8ed, PDF_CMAP_SINGLE, 8450 },
	{ 0xc8ee, 0xc8f0, PDF_CMAP_RANGE, 3273 },
	{ 0xc8f1, 0xc8f3, PDF_CMAP_RANGE, 8451 },
	{ 0xc8f4, 0xc8f6, PDF_CMAP_RANGE, 3279 },
	{ 0xc8f7, 0xc8f8, PDF_CMAP_RANGE, 8454 },
	{ 0xc8f9, 0xc8fc, PDF_CMAP_TABLE, 272 },
	{ 0xc8fd, 0xc8fe, PDF_CMAP_RANGE, 3288 },
	{ 0xc9a1, 0xc9a1, PDF_CMAP_SINGLE, 8458 },
	{ 0xc9a2, 0xc9a4, PDF_CMAP_RANGE, 3291 },
	{ 0xc9a5, 0xc9a6, PDF_CMAP_TABLE, 276 },
	{ 0xc9a7, 0xc9a8, PDF_CMAP_RANGE, 8460 },
	{ 0xc9a9, 0xc9ab, PDF_CMAP_RANGE, 3298 },
	{ 0xc9ac, 0xc9ac, PDF_CMAP_SINGLE, 8462 },
	{ 0xc9ad, 0xc9b0, PDF_CMAP_RANGE, 3302 },
	{ 0xc9b1, 0xc9b1, PDF_CMAP_SINGLE, 8463 },
	{ 0xc9b2, 0xc9b3, PDF_CMAP_RANGE, 3307 },
	{ 0xc9b4, 0xc9b4, PDF_CMAP_SINGLE, 8464 },
	{ 0xc9b5, 0xc9b7, PDF_CMAP_RANGE, 3310 },
	{ 0xc9b8, 0xc9b9, PDF_CMAP_RANGE, 8465 },
	{ 0xc9ba, 0xc9c0, PDF_CMAP_RANGE, 3315 },
	{ 0xc9c1, 0xc9c2, PDF_CMAP_RANGE, 8467 },
	{ 0xc9c3, 0xc9c4, PDF_CMAP_TABLE, 278 },
	{ 0xc9c5, 0xc9c8, PDF_CMAP_RANGE, 3326 },
	{ 0xc9c9, 0xc9cd, PDF_CMAP_TABLE, 280 },
	{ 0xc9ce, 0xc9d4, PDF_CMAP_RANGE, 3335 },
	{ 0xc9d5, 0xc9d5, PDF_CMAP_SINGLE, 8473 },
	{ 0xc9d6, 0xc9db, PDF_CMAP_RANGE, 3343 },
	{ 0xc9dc, 0xc9de, PDF_CMAP_TABLE, 285 },
	{ 0xc9df, 0xc9e0, PDF_CMAP_RANGE, 3352 },
	{ 0xc9e1, 0xc9e5, PDF_CMAP_TABLE, 288 },
	{ 0xc9e6, 0xc9e7, PDF_CMAP_RANGE, 3359 },
	{ 0xc9e8, 0xc9e8, PDF_CMAP_SINGLE, 8479 },
	{ 0xc9e9, 0xc9ef, PDF_CMAP_RANGE, 3362 },
	{ 0xc9f0, 0xc9f0, PDF_CMAP_SINGLE, 8480 },
	{ 0xc9f1, 0xc9f2, PDF_CMAP_RANGE, 3370 },
	{ 0xc9f3, 0xc9f4, PDF_CMAP_RANGE, 8481 },
	{ 0xc9f5, 0xc9f7, PDF_CMAP_TABLE, 293 },
	{ 0xc9f8, 0xc9f9, PDF_CMAP_RANGE, 8484 },
	{ 0xc9fa, 0xc9fd, PDF_CMAP_RANGE, 3379 },
	{ 0xc9fe, 0xc9fe, PDF_CMAP_SINGLE, 8486 },
	{ 0xcaa1, 0xcaa3, PDF_CMAP_RANGE, 3384 },
	{ 0xcaa4, 0xcaa6, PDF_CMAP_RANGE, 8487 },
	{ 0xcaa7, 0xcaa9, PDF_CMAP_TABLE, 296 },
	{ 0xcaaa, 0xcaab, PDF_CMAP_RANGE, 8491 },
	{ 0xcaac, 0xcab0, PDF_CMAP_RANGE, 3395 },
	{ 0xcab1, 0xcab1, PDF_CMAP_SINGLE, 8493 },
	{ 0xcab2, 0xcab3, PDF_CMAP_RANGE, 3401 },
	{ 0xcab4, 0xcab6, PDF_CMAP_RANGE, 8494 },
	{ 0xcab7, 0xcaba, PDF_CMAP_RANGE, 3406 },
	{ 0xcabb, 0xcabb, PDF_CMAP_SINGLE, 8497 },
	{ 0xcabc, 0xcac5, PDF_CMAP_RANGE, 3411 },
	{ 0xcac6, 0xcac6, PDF_CMAP_SINGLE, 8498 },
	{ 0xcac7, 0xcac9, PDF_CMAP_RANGE, 3422 },
	{ 0xcaca, 0xcaca, PDF_CMAP_SINGLE, 8499 },
	{ 0xcacb, 0xcacc, PDF_CMAP_RANGE, 3426 },
	{ 0xcacd, 0xcace, PDF_CMAP_RANGE, 8500 },
	{ 0xcacf, 0xcad2, PDF_CMAP_RANGE, 3430 },
	{ 0xcad3, 0xcad4, PDF_CMAP_RANGE, 8502 },
	{ 0xcad5, 0xcad8, PDF_CMAP_RANGE, 3436 },
	{ 0xcad9, 0xcad9, PDF_CMAP_SINGLE, 8504 },
	{ 0xcada, 0xcadd, PDF_CMAP_RANGE, 3441 },
	{ 0xcade, 0xcae0, PDF_CMAP_TABLE, 299 },
	{ 0xcae1, 0xcae3, PDF_CMAP_RANGE, 3448 },
	{ 0xcae4, 0xcae4, PDF_CMAP_SINGLE, 8507 },
	{ 0xcae5, 0xcae8, PDF_CMAP_RANGE, 3452 },
	{ 0xcae9, 0xcaea, PDF_CMAP_RANGE, 8508 },
	{ 0xcaeb, 0xcaf3, PDF_CMAP_RANGE, 3458 },
	{ 0xcaf4, 0xcaf5, PDF_CMAP_RANGE, 8510 },
	{ 0xcaf6, 0xcaf7, PDF_CMAP_TABLE, 302 },
	{ 0xcaf8, 0xcaf9, PDF_CMAP_RANGE, 3471 },
	{ 0xcafa, 0xcafa, PDF_CMAP_SINGLE, 8513 },
	{ 0xcafb, 0xcafc, PDF_CMAP_RANGE, 3474 },
	{ 0xcafd, 0xcafe, PDF_CMAP_TABLE, 304 },
	{ 0xcba1, 0xcba6, PDF_CMAP_RANGE, 3478 },
	{ 0xcba7, 0xcba7, PDF_CMAP_SINGLE, 8515 },
	{ 0xcba8, 0xcbaa, PDF_CMAP_RANGE, 3485 },
	{ 0xcbab, 0xcbad, PDF_CMAP_TABLE, 306 },
	{ 0xcbae, 0xcbb2, PDF_CMAP_RANGE, 3491 },
	{ 0xcbb3, 0xcbb4, PDF_CMAP_TABLE, 309 },
	{ 0xcbb5, 0xcbb6, PDF_CMAP_RANGE, 8519 },
	{ 0xcbb7, 0xcbb8, PDF_CMAP_TABLE, 311 },
	{ 0xcbb9, 0xcbbe, PDF_CMAP_RANGE, 3502 },
	{ 0xcbbf, 0xcbbf, PDF_CMAP_SINGLE, 8522 },
	{ 0xcbc0, 0xcbc6, PDF_CMAP_RANGE, 3509 },
	{ 0xcbc7, 0xcbc8, PDF_CMAP_TABLE, 313 },
	{ 0xcbc9, 0xcbcc, PDF_CMAP_RANGE, 8524 },
	{ 0xcbcd, 0xcbce, PDF_CMAP_RANGE, 3522 },
	{ 0xcbcf, 0xcbd0, PDF_CMAP_RANGE, 8528 },
	{ 0xcbd1, 0xcbd2, PDF_CMAP_RANGE, 3526 },
	{ 0xcbd3, 0xcbd5, PDF_CMAP_TABLE, 315 },
	{ 0xcbd6, 0xcbde, PDF_CMAP_RANGE, 3531 },
	{ 0xcbdf, 0xcbe0, PDF_CMAP_RANGE, 8532 },
	{ 0xcbe1, 0xcbe3, PDF_CMAP_RANGE, 3542 },
	{ 0xcbe4, 0xcbe5, PDF_CMAP_TABLE, 318 },
	{ 0xcbe6, 0xcbe7, PDF_CMAP_RANGE, 8535 },
	{ 0xcbe8, 0xcbe9, PDF_CMAP_RANGE, 3549 },
	{ 0xcbea, 0xcbea, PDF_CMAP_SINGLE, 8537 },
	{ 0xcbeb, 0xcbee, PDF_CMAP_RANGE, 3552 },
	{ 0xcbef, 0xcbf0, PDF_CMAP_RANGE, 8538 },
	{ 0xcbf1, 0xcbf4, PDF_CMAP_RANGE, 3558 },
	{ 0xcbf5, 0xcbf6, PDF_CMAP_RANGE, 8540 },
	{ 0xcbf7, 0xcbf8, PDF_CMAP_TABLE, 320 },
	{ 0xcbf9, 0xcbfe, PDF_CMAP_RANGE, 3566 },
	{ 0xcca1, 0xcca2, PDF_CMAP_RANGE, 8543 },
	{ 0xcca3, 0xcca7, PDF_CMAP_RANGE, 3574 },
	{ 0xcca8, 0xcca8, PDF_CMAP_SINGLE, 8545 },
	{ 0xcca9, 0xccab, PDF_CMAP_RANGE, 3580 },
	{ 0xccac, 0xccac, PDF_CMAP_SINGLE, 8546 },
	{ 0xccad, 0xccae, PDF_CMAP_RANGE, 3584 },
	{ 0xccaf, 0xccb3, PDF_CMAP_RANGE, 8547 },
	{ 0xccb4, 0xccb6, PDF_CMAP_RANGE, 3591 },
	{ 0xccb7, 0xccb8, PDF_CMAP_RANGE, 8552 },
	{ 0xccb9, 0xccbd, PDF_CMAP_RANGE, 3596 },
	{ 0xccbe, 0xccc0, PDF_CMAP_TABLE, 322 },
	{ 0xccc1, 0xcccb, PDF_CMAP_RANGE, 3604 },
	{ 0xcccc, 0xccd0, PDF_CMAP_TABLE, 325 },
	{ 0xccd1, 0xccd5, PDF_CMAP_RANGE, 3620 },
	{ 0xccd6, 0xccd6, PDF_CMAP_SINGLE, 8559 },
	{ 0xccd7, 0xccd9, PDF_CMAP_RANGE, 3626 },
	{ 0xccda, 0xccdc, PDF_CMAP_TABLE, 330 },
	{ 0xccdd, 0xccdf, PDF_CMAP_RANGE, 3632 },
	{ 0xcce0, 0xcce2, PDF_CMAP_TABLE, 333 },
	{ 0xcce3, 0xcce4, PDF_CMAP_RANGE, 3638 },
	{ 0xcce5, 0xcce5, PDF_CMAP_SINGLE, 8564 },
	{ 0xcce6, 0xccf4, PDF_CMAP_RANGE, 3641 },
	{ 0xccf5, 0xccf5, PDF_CMAP_SINGLE, 8565 },
	{ 0xccf6, 0xccf8, PDF_CMAP_RANGE, 3657 },
	{ 0xccf9, 0xccfa, PDF_CMAP_RANGE, 8566 },
	{ 0xccfb, 0xccfb, PDF_CMAP_SINGLE, 3662 },
	{ 0xccfc, 0xccfe, PDF_CMAP_RANGE, 8568 },
	{ 0xcda1, 0xcdac, PDF_CMAP_RANGE, 3666 },
	{ 0xcdad, 0xcdad, PDF_CMAP_SINGLE, 8571 },
	{ 0xcdae, 0xcdb2, PDF_CMAP_RANGE, 3679 },
	{ 0xcdb3, 0xcdb3, PDF_CMAP_SINGLE, 8572 },
	{ 0xcdb4, 0xcdb6, PDF_CMAP_RANGE, 3685 },
	{ 0xcdb7, 0xcdb7, PDF_CMAP_SINGLE, 8573 },
	{ 0xcdb8, 0xcdbb, PDF_CMAP_RANGE, 3689 },
	{ 0xcdbc, 0xcdbc, PDF_CMAP_SINGLE, 8574 },
	{ 0xcdbd, 0xcdbe, PDF_CMAP_RANGE, 3694 },
	{ 0xcdbf, 0xcdbf, PDF_CMAP_SINGLE, 8575 },
	{ 0xcdc0, 0xcdc4, PDF_CMAP_RANGE, 3697 },
	{ 0xcdc5, 0xcdc7, PDF_CMAP_TABLE, 336 },
	{ 0xcdc8, 0xcdd1, PDF_CMAP_RANGE, 3705 },
	{ 0xcdd2, 0xcdd3, PDF_CMAP_TABLE, 339 },
	{ 0xcdd4, 0xcdd6, PDF_CMAP_RANGE, 8579 },
	{ 0xcdd7, 0xcddc, PDF_CMAP_RANGE, 3720 },
	{ 0xcddd, 0xcddd, PDF_CMAP_SINGLE, 8582 },
	{ 0xcdde, 0xcddf, PDF_CMAP_RANGE, 3727 },
	{ 0xcde0, 0xcde0, PDF_CMAP_SINGLE, 8583 },
	{ 0xcde1, 0xcde3, PDF_CMAP_RANGE, 3730 },
	{ 0xcde4, 0xcde5, PDF_CMAP_RANGE, 8584 },
	{ 0xcde6, 0xcde7, PDF_CMAP_TABLE, 341 },
	{ 0xcde8, 0xcdf1, PDF_CMAP_RANGE, 3737 },
	{ 0xcdf2, 0xcdf2, PDF_CMAP_SINGLE, 8587 },
	{ 0xcdf3, 0xcdf7, PDF_CMAP_RANGE, 3748 },
	{ 0xcdf8, 0xcdf8, PDF_CMAP_SINGLE, 8588 },
	{ 0xcdf9, 0xcdfe, PDF_CMAP_RANGE, 3754 },
	{ 0xcea1, 0xcea3, PDF_CMAP_RANGE, 3760 },
	{ 0xcea4, 0xcea5, PDF_CMAP_RANGE, 8589 },
	{ 0xcea6, 0xcea7, PDF_CMAP_TABLE, 343 },
	{ 0xcea8, 0xcea9, PDF_CMAP_RANGE, 3767 },
	{ 0xceaa, 0xcead, PDF_CMAP_RANGE, 8592 },
	{ 0xceae, 0xceaf, PDF_CMAP_RANGE, 3773 },
	{ 0xceb0, 0xceb1, PDF_CMAP_RANGE, 8596 },
	{ 0xceb2, 0xceb3, PDF_CMAP_TABLE, 345 },
	{ 0xceb4, 0xcebc, PDF_CMAP_RANGE, 3779 },
	{ 0xcebd, 0xcebd, PDF_CMAP_SINGLE, 8599 },
	{ 0xcebe, 0xcebf, PDF_CMAP_RANGE, 3789 },
	{ 0xcec0, 0xcec0, PDF_CMAP_SINGLE, 8600 },
	{ 0xcec1, 0xcec4, PDF_CMAP_RANGE, 3792 },
	{ 0xcec5, 0xcec6, PDF_CMAP_RANGE, 8601 },
	{ 0xcec7, 0xceca, PDF_CMAP_TABLE, 347 },
	{ 0xcecb, 0xcecd, PDF_CMAP_RANGE, 3802 },
	{ 0xcece, 0xced1, PDF_CMAP_RANGE, 8605 },
	{ 0xced2, 0xced7, PDF_CMAP_RANGE, 3809 },
	{ 0xced8, 0xceda, PDF_CMAP_RANGE, 8609 },
	{ 0xcedb, 0xcedd, PDF_CMAP_TABLE, 351 },
	{ 0xcede, 0xcedf, PDF_CMAP_RANGE, 8613 },
	{ 0xcee0, 0xceea, PDF_CMAP_RANGE, 3823 },
	{ 0xceeb, 0xceed, PDF_CMAP_TABLE, 354 },
	{ 0xceee, 0xcef0, PDF_CMAP_RANGE, 3837 },
	{ 0xcef1, 0xcef3, PDF_CMAP_TABLE, 357 },
	{ 0xcef4, 0xcefc, PDF_CMAP_RANGE, 3843 },
	{ 0xcefd, 0xcefe, PDF_CMAP_RANGE, 8619 },
	{ 0xcfa1, 0xcfad, PDF_CMAP_RANGE, 3854 },
	{ 0xcfae, 0xcfb0, PDF_CMAP_TABLE, 360 },
	{ 0xcfb1, 0xcfb2, PDF_CMAP_RANGE, 3870 },
	{ 0xcfb3, 0xcfb3, PDF_CMAP_SINGLE, 8623 },
	{ 0xcfb4, 0xcfb6, PDF_CMAP_RANGE, 3873 },
	{ 0xcfb7, 0xcfb8, PDF_CMAP_RANGE, 8624 },
	{ 0xcfb9, 0xcfba, PDF_CMAP_TABLE, 363 },
	{ 0xcfbb, 0xcfbc, PDF_CMAP_RANGE, 3880 },
	{ 0xcfbd, 0xcfbe, PDF_CMAP_TABLE, 365 },
	{ 0xcfbf, 0xcfc1, PDF_CMAP_RANGE, 8628 },
	{ 0xcfc2, 0xcfc4, PDF_CMAP_RANGE, 3887 },
	{ 0xcfc5, 0xcfc7, PDF_CMAP_TABLE, 367 },
	{ 0xcfc8, 0xcfc9, PDF_CMAP_RANGE, 3893 },
	{ 0xcfca, 0xcfcb, PDF_CMAP_RANGE, 8633 },
	{ 0xcfcc, 0xcfcc, PDF_CMAP_SINGLE, 3897 },
	{ 0xcfcd, 0xcfce, PDF_CMAP_RANGE, 8635 },
	{ 0xcfcf, 0xcfd0, PDF_CMAP_TABLE, 370 },
	{ 0xcfd1, 0xcfd3, PDF_CMAP_RANGE, 3902 },
	{ 0xcfd4, 0xcfd8, PDF_CMAP_RANGE, 8638 },
	{ 0xcfd9, 0xcfdc, PDF_CMAP_TABLE, 372 },
	{ 0xcfdd, 0xcfde, PDF_CMAP_RANGE, 3914 },
	{ 0xcfdf, 0xcfdf, PDF_CMAP_SINGLE, 8645 },
	{ 0xcfe0, 0xcfe1, PDF_CMAP_RANGE, 3917 },
	{ 0xcfe2, 0xcfe2, PDF_CMAP_SINGLE, 8646 },
	{ 0xcfe3, 0xcfe6, PDF_CMAP_RANGE, 3920 },
	{ 0xcfe7, 0xcfe7, PDF_CMAP_SINGLE, 8647 },
	{ 0xcfe8, 0xcfe9, PDF_CMAP_RANGE, 3925 },
	{ 0xcfea, 0xcfee, PDF_CMAP_TABLE, 376 },
	{ 0xcfef, 0xcff3, PDF_CMAP_RANGE, 3932 },
	{ 0xcff4, 0xcff4, PDF_CMAP_SINGLE, 8651 },
	{ 0xcff5, 0xcff8, PDF_CMAP_RANGE, 3938 },
	{ 0xcff9, 0xcffa, PDF_CMAP_RANGE, 8652 },
	{ 0xcffb, 0xcffd, PDF_CMAP_RANGE, 3944 },
	{ 0xcffe, 0xcffe, PDF_CMAP_SINGLE, 8654 },
	{ 0xd0a1, 0xd0a4, PDF_CMAP_RANGE, 3948 },
	{ 0xd0a5, 0xd0a5, PDF_CMAP_SINGLE, 8655 },
	{ 0xd0a6, 0xd0ac, PDF_CMAP_RANGE, 3953 },
	{ 0xd0ad, 0xd0ae, PDF_CMAP_RANGE, 8656 },
	{ 0xd0af, 0xd0b1, PDF_CMAP_RANGE, 3962 },
	{ 0xd0b2, 0xd0b4, PDF_CMAP_RANGE, 8658 },
	{ 0xd0b5, 0xd0b9, PDF_CMAP_RANGE, 3968 },
	{ 0xd0ba, 0xd0bb, PDF_CMAP_RANGE, 8661 },
	{ 0xd0bc, 0xd0be, PDF_CMAP_RANGE, 3975 },
	{ 0xd0bf, 0xd0bf, PDF_CMAP_SINGLE, 8663 },
	{ 0xd0c0, 0xd0c5, PDF_CMAP_RANGE, 3979 },
	{ 0xd0c6, 0xd0c6, PDF_CMAP_SINGLE, 8664 },
	{ 0xd0c7, 0xd0ca, PDF_CMAP_RANGE, 3986 },
	{ 0xd0cb, 0xd0cb, PDF_CMAP_SINGLE, 8665 },
	{ 0xd0cc, 0xd0e1, PDF_CMAP_RANGE, 3991 },
	{ 0xd0e2, 0xd0e2, PDF_CMAP_SINGLE, 8666 },
	{ 0xd0e3, 0xd0e4, PDF_CMAP_RANGE, 4014 },
	{ 0xd0e5, 0xd0e5, PDF_CMAP_SINGLE, 8667 },
	{ 0xd0e6, 0xd0ea, PDF_CMAP_RANGE, 4017 },
	{ 0xd0eb, 0xd0ed, PDF_CMAP_TABLE, 381 },
	{ 0xd0ee, 0xd0f6, PDF_CMAP_RANGE, 4025 },
	{ 0xd0f7, 0xd0f9, PDF_CMAP_RANGE, 8670 },
	{ 0xd0fa, 0xd0fb, PDF_CMAP_RANGE, 4037 },
	{ 0xd0fc, 0xd0fc, PDF_CMAP_SINGLE, 8673 },
	{ 0xd0fd, 0xd0fe, PDF_CMAP_RANGE, 4040 },
	{ 0xd1a1, 0xd1a2, PDF_CMAP_RANGE, 8674 },
	{ 0xd1a3, 0xd1a4, PDF_CMAP_TABLE, 384 },
	{ 0xd1a5, 0xd1a6, PDF_CMAP_RANGE, 4046 },
	{ 0xd1a7, 0xd1a7, PDF_CMAP_SINGLE, 8677 },
	{ 0xd1a8, 0xd1aa, PDF_CMAP_RANGE, 4049 },
	{ 0xd1ab, 0xd1ab, PDF_CMAP_SINGLE, 8678 },
	{ 0xd1ac, 0xd1ae, PDF_CMAP_RANGE, 4053 },
	{ 0xd1af, 0xd1b1, PDF_CMAP_RANGE, 8679 },
	{ 0xd1b2, 0xd1b4, PDF_CMAP_RANGE, 4059 },
	{ 0xd1b5, 0xd1b7, PDF_CMAP_RANGE, 8682 },
	{ 0xd1b8, 0xd1ba, PDF_CMAP_TABLE, 386 },
	{ 0xd1bb, 0xd1bc, PDF_CMAP_RANGE, 8686 },
	{ 0xd1bd, 0xd1c5, PDF_CMAP_RANGE, 4070 },
	{ 0xd1c6, 0xd1c8, PDF_CMAP_RANGE, 8688 },
	{ 0xd1c9, 0xd1ca, PDF_CMAP_RANGE, 4082 },
	{ 0xd1cb, 0xd1cb, PDF_CMAP_SINGLE, 8691 },
	{ 0xd1cc, 0xd1cd, PDF_CMAP_RANGE, 4085 },
	{ 0xd1ce, 0xd1cf, PDF_CMAP_RANGE, 8692 },
	{ 0xd1d0, 0xd1d4, PDF_CMAP_RANGE, 4089 },
	{ 0xd1d5, 0xd1d6, PDF_CMAP_RANGE, 8694 },
	{ 0xd1d7, 0xd1dd, PDF_CMAP_RANGE, 4096 },
	{ 0xd1de, 0xd1de, PDF_CMAP_SINGLE, 8696 },
	{ 0xd1df, 0xd1e0, PDF_CMAP_RANGE, 4104 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_RANGE, 8697 },
	{ 0xd1e3, 0xd1e7, PDF_CMAP_RANGE, 4108 },
	{ 0xd1e8, 0xd1e9, PDF_CMAP_RANGE, 8699 },
	{ 0xd1ea, 0xd1eb, PDF_CMAP_RANGE, 4115 },
	{ 0xd1ec, 0xd1ed, PDF_CMAP_TABLE, 389 },
	{ 0xd1ee, 0xd1ef, PDF_CMAP_RANGE, 8702 },
	{ 0xd1f0, 0xd1f1, PDF_CMAP_TABLE, 391 },
	{ 0xd1f2, 0xd1f3, PDF_CMAP_RANGE, 4123 },
	{ 0xd1f4, 0xd1f4, PDF_CMAP_SINGLE, 8705 },
	{ 0xd1f5, 0xd1f6, PDF_CMAP_RANGE, 4126 },
	{ 0xd1f7, 0xd1f9, PDF_CMAP_RANGE, 8706 },
	{ 0xd1fa, 0xd1fe, PDF_CMAP_RANGE, 4131 },
	{ 0xd2a1, 0xd2a2, PDF_CMAP_TABLE, 393 },
	{ 0xd2a3, 0xd2a4, PDF_CMAP_RANGE, 4138 },
	{ 0xd2a5, 0xd2a5, PDF_CMAP_SINGLE, 8710 },
	{ 0xd2a6, 0xd2a8, PDF_CMAP_RANGE, 4141 },
	{ 0xd2a9, 0xd2a9, PDF_CMAP_SINGLE, 8711 },
	{ 0xd2aa, 0xd2ae, PDF_CMAP_RANGE, 4145 },
	{ 0xd2af, 0xd2af, PDF_CMAP_SINGLE, 8712 },
	{ 0xd2b0, 0xd2b2, PDF_CMAP_RANGE, 4151 },
	{ 0xd2b3, 0xd2b4, PDF_CMAP_TABLE, 395 },
	{ 0xd2b5, 0xd2b6, PDF_CMAP_RANGE, 8714 },
	{ 0xd2b7, 0xd2bc, PDF_CMAP_RANGE, 4158 },
	{ 0xd2bd, 0xd2bf, PDF_CMAP_TABLE, 397 },
	{ 0xd2c0, 0xd2c2, PDF_CMAP_RANGE, 4167 },
	{ 0xd2c3, 0xd2c7, PDF_CMAP_TABLE, 400 },
	{ 0xd2c8, 0xd2ce, PDF_CMAP_RANGE, 4175 },
	{ 0xd2cf, 0xd2cf, PDF_CMAP_SINGLE, 8721 },
	{ 0xd2d0, 0xd2d4, PDF_CMAP_RANGE, 4183 },
	{ 0xd2d5, 0xd2d5, PDF_CMAP_SINGLE, 8722 },
	{ 0xd2d6, 0xd2d9, PDF_CMAP_RANGE, 4189 },
	{ 0xd2da, 0xd2da, PDF_CMAP_SINGLE, 8723 },
	{ 0xd2db, 0xd2e3, PDF_CMAP_RANGE, 4194 },
	{ 0xd2e4, 0xd2e5, PDF_CMAP_RANGE, 8724 },
	{ 0xd2e6, 0xd2e7, PDF_CMAP_RANGE, 4205 },
	{ 0xd2e8, 0xd2eb, PDF_CMAP_RANGE, 8726 },
	{ 0xd2ec, 0xd2ee, PDF_CMAP_RANGE, 4211 },
	{ 0xd2ef, 0xd2f1, PDF_CMAP_TABLE, 405 },
	{ 0xd2f2, 0xd2f4, PDF_CMAP_RANGE, 4217 },
	{ 0xd2f5, 0xd2f5, PDF_CMAP_SINGLE, 8732 },
	{ 0xd2f6, 0xd2f7, PDF_CMAP_RANGE, 4221 },
	{ 0xd2f8, 0xd2f8, PDF_CMAP_SINGLE, 8733 },
	{ 0xd2f9, 0xd2fa, PDF_CMAP_RANGE, 4224 },
	{ 0xd2fb, 0xd2fb, PDF_CMAP_SINGLE, 8734 },
	{ 0xd2fc, 0xd2fd, PDF_CMAP_RANGE, 4227 },
	{ 0xd2fe, 0xd2fe, PDF_CMAP_SINGLE, 8735 },
	{ 0xd3a1, 0xd3a2, PDF_CMAP_RANGE, 4230 },
	{ 0xd3a3, 0xd3ac, PDF_CMAP_RANGE, 8736 },
	{ 0xd3ad, 0xd3ae, PDF_CMAP_TABLE, 408 },
	{ 0xd3af, 0xd3b0, PDF_CMAP_RANGE, 4244 },
	{ 0xd3b1, 0xd3b1, PDF_CMAP_SINGLE, 8747 },
	{ 0xd3b2, 0xd3b3, PDF_CMAP_RANGE, 4247 },
	{ 0xd3b4, 0xd3b6, PDF_CMAP_RANGE, 8748 },
	{ 0xd3b7, 0xd3b8, PDF_CMAP_TABLE, 410 },
	{ 0xd3b9, 0xd3ba, PDF_CMAP_RANGE, 4254 },
	{ 0xd3bb, 0xd3bb, PDF_CMAP_SINGLE, 8752 },
	{ 0xd3bc, 0xd3c4, PDF_CMAP_RANGE, 4257 },
	{ 0xd3c5, 0xd3c7, PDF_CMAP_TABLE, 412 },
	{ 0xd3c8, 0xd3c9, PDF_CMAP_RANGE, 4269 },
	{ 0xd3ca, 0xd3cc, PDF_CMAP_RANGE, 8755 },
	{ 0xd3cd, 0xd3d4, PDF_CMAP_RANGE, 4274 },
	{ 0xd3d5, 0xd3d5, PDF_CMAP_SINGLE, 8758 },
	{ 0xd3d6, 0xd3de, PDF_CMAP_RANGE, 4283 },
	{ 0xd3df, 0xd3e0, PDF_CMAP_RANGE, 8759 },
	{ 0xd3e1, 0xd3e2, PDF_CMAP_RANGE, 4294 },
	{ 0xd3e3, 0xd3e3, PDF_CMAP_SINGLE, 8761 },
	{ 0xd3e4, 0xd3e5, PDF_CMAP_RANGE, 4297 },
	{ 0xd3e6, 0xd3e6, PDF_CMAP_SINGLE, 8762 },
	{ 0xd3e7, 0xd3ea, PDF_CMAP_RANGE, 4300 },
	{ 0xd3eb, 0xd3ec, PDF_CMAP_RANGE, 8763 },
	{ 0xd3ed, 0xd3ee, PDF_CMAP_RANGE, 4306 },
	{ 0xd3ef, 0xd3ef, PDF_CMAP_SINGLE, 8765 },
	{ 0xd3f0, 0xd3f3, PDF_CMAP_RANGE, 4309 },
	{ 0xd3f4, 0xd3f4, PDF_CMAP_SINGLE, 8766 },
	{ 0xd3f5, 0xd3fb, PDF_CMAP_RANGE, 4314 },
	{ 0xd3fc, 0xd3fe, PDF_CMAP_TABLE, 415 },
	{ 0xd4a1, 0xd4a3, PDF_CMAP_RANGE, 4324 },
	{ 0xd4a4, 0xd4a5, PDF_CMAP_TABLE, 418 },
	{ 0xd4a6, 0xd4a8, PDF_CMAP_RANGE, 8770 },
	{ 0xd4a9, 0xd4ae, PDF_CMAP_RANGE, 4332 },
	{ 0xd4af, 0xd4b2, PDF_CMAP_RANGE, 8773 },
	{ 0xd4b3, 0xd4b4, PDF_CMAP_RANGE, 4342 },
	{ 0xd4b5, 0xd4b6, PDF_CMAP_RANGE, 8777 },
	{ 0xd4b7, 0xd4b8, PDF_CMAP_TABLE, 420 },
	{ 0xd4b9, 0xd4bb, PDF_CMAP_RANGE, 4348 },
	{ 0xd4bc, 0xd4bd, PDF_CMAP_TABLE, 422 },
	{ 0xd4be, 0xd4bf, PDF_CMAP_RANGE, 8781 },
	{ 0xd4c0, 0xd4c3, PDF_CMAP_RANGE, 4355 },
	{ 0xd4c4, 0xd4c5, PDF_CMAP_TABLE, 424 },
	{ 0xd4c6, 0xd4c7, PDF_CMAP_RANGE, 8784 },
	{ 0xd4c8, 0xd4ca, PDF_CMAP_TABLE, 426 },
	{ 0xd4cb, 0xd4ce, PDF_CMAP_RANGE, 8787 },
	{ 0xd4cf, 0xd4d2, PDF_CMAP_RANGE, 4370 },
	{ 0xd4d3, 0xd4d3, PDF_CMAP_SINGLE, 8791 },
	{ 0xd4d4, 0xd4d7, PDF_CMAP_RANGE, 4375 },
	{ 0xd4d8, 0xd4d8, PDF_CMAP_SINGLE, 8792 },
	{ 0xd4d9, 0xd4db, PDF_CMAP_RANGE, 4380 },
	{ 0xd4dc, 0xd4e0, PDF_CMAP_RANGE, 8793 },
	{ 0xd4e1, 0xd4e3, PDF_CMAP_RANGE, 4388 },
	{ 0xd4e4, 0xd4e6, PDF_CMAP_TABLE, 429 },
	{ 0xd4e7, 0xd4ed, PDF_CMAP_RANGE, 4394 },
	{ 0xd4ee, 0xd4ef, PDF_CMAP_TABLE, 432 },
	{ 0xd4f0, 0xd4f4, PDF_CMAP_RANGE, 8801 },
	{ 0xd4f5, 0xd4f8, PDF_CMAP_RANGE, 4408 },
	{ 0xd4f9, 0xd4f9, PDF_CMAP_SINGLE, 8806 },
	{ 0xd4fa, 0xd4fd, PDF_CMAP_RANGE, 4413 },
	{ 0xd4fe, 0xd4fe, PDF_CMAP_SINGLE, 8807 },
	{ 0xd5a1, 0xd5a2, PDF_CMAP_RANGE, 8808 },
	{ 0xd5a3, 0xd5a8, PDF_CMAP_RANGE, 4420 },
	{ 0xd5a9, 0xd5ab, PDF_CMAP_TABLE, 434 },
	{ 0xd5ac, 0xd5ad, PDF_CMAP_RANGE, 4429 },
	{ 0xd5ae, 0xd5ae, PDF_CMAP_SINGLE, 8812 },
	{ 0xd5af, 0xd5b0, PDF_CMAP_RANGE, 4432 },
	{ 0xd5b1, 0xd5b1, PDF_CMAP_SINGLE, 8813 },
	{ 0xd5b2, 0xd5b4, PDF_CMAP_RANGE, 4435 },
	{ 0xd5b5, 0xd5b8, PDF_CMAP_RANGE, 8814 },
	{ 0xd5b9, 0xd5ba, PDF_CMAP_RANGE, 4442 },
	{ 0xd5bb, 0xd5bd, PDF_CMAP_TABLE, 437 },
	{ 0xd5be, 0xd5bf, PDF_CMAP_RANGE, 4447 },
	{ 0xd5c0, 0xd5c0, PDF_CMAP_SINGLE, 8820 },
	{ 0xd5c1, 0xd5c4, PDF_CMAP_RANGE, 4450 },
	{ 0xd5c5, 0xd5c7, PDF_CMAP_TABLE, 440 },
	{ 0xd5c8, 0xd5c9, PDF_CMAP_RANGE, 4457 },
	{ 0xd5ca, 0xd5cb, PDF_CMAP_RANGE, 8823 },
	{ 0xd5cc, 0xd5cd, PDF_CMAP_TABLE, 443 },
	{ 0xd5ce, 0xd5d3, PDF_CMAP_RANGE, 4463 },
	{ 0xd5d4, 0xd5d4, PDF_CMAP_SINGLE, 8826 },
	{ 0xd5d5, 0xd5dc, PDF_CMAP_RANGE, 4470 },
	{ 0xd5dd, 0xd5de, PDF_CMAP_RANGE, 8827 },
	{ 0xd5df, 0xd5e2, PDF_CMAP_TABLE, 445 },
	{ 0xd5e3, 0xd5e9, PDF_CMAP_RANGE, 4484 },
	{ 0xd5ea, 0xd5ec, PDF_CMAP_RANGE, 8831 },
	{ 0xd5ed, 0xd5ee, PDF_CMAP_RANGE, 4494 },
	{ 0xd5ef, 0xd5ef, PDF_CMAP_SINGLE, 8834 },
	{ 0xd5f0, 0xd5f1, PDF_CMAP_RANGE, 4497 },
	{ 0xd5f2, 0xd5f3, PDF_CMAP_RANGE, 8835 },
	{ 0xd5f4, 0xd5f6, PDF_CMAP_RANGE, 4501 },
	{ 0xd5f7, 0xd5f7, PDF_CMAP_SINGLE, 5611 },
	{ 0xd5f8, 0xd5fe, PDF_CMAP_RANGE, 4505 },
	{ 0xd6a1, 0xd6a4, PDF_CMAP_RANGE, 8837 },
	{ 0xd6a5, 0xd6ae, PDF_CMAP_RANGE, 4516 },
	{ 0xd6af, 0xd6b0, PDF_CMAP_RANGE, 8841 },
	{ 0xd6b1, 0xd6b3, PDF_CMAP_RANGE, 4528 },
	{ 0xd6b4, 0xd6b4, PDF_CMAP_SINGLE, 8843 },
	{ 0xd6b5, 0xd6ba, PDF_CMAP_RANGE, 4532 },
	{ 0xd6bb, 0xd6be, PDF_CMAP_TABLE, 449 },
	{ 0xd6bf, 0xd6c0, PDF_CMAP_RANGE, 8846 },
	{ 0xd6c1, 0xd6c3, PDF_CMAP_RANGE, 4544 },
	{ 0xd6c4, 0xd6c4, PDF_CMAP_SINGLE, 8848 },
	{ 0xd6c5, 0xd6c9, PDF_CMAP_RANGE, 4548 },
	{ 0xd6ca, 0xd6ca, PDF_CMAP_SINGLE, 8849 },
	{ 0xd6cb, 0xd6cc, PDF_CMAP_RANGE, 4554 },
	{ 0xd6cd, 0xd6cd, PDF_CMAP_SINGLE, 8850 },
	{ 0xd6ce, 0xd6d2, PDF_CMAP_RANGE, 4557 },
	{ 0xd6d3, 0xd6d4, PDF_CMAP_TABLE, 453 },
	{ 0xd6d5, 0xd6d7, PDF_CMAP_RANGE, 8852 },
	{ 0xd6d8, 0xd6d9, PDF_CMAP_RANGE, 4567 },
	{ 0xd6da, 0xd6da, PDF_CMAP_SINGLE, 8855 },
	{ 0xd6db, 0xd6de, PDF_CMAP_RANGE, 4570 },
	{ 0xd6df, 0xd6e1, PDF_CMAP_TABLE, 455 },
	{ 0xd6e2, 0xd6e4, PDF_CMAP_RANGE, 4577 },
	{ 0xd6e5, 0xd6e6, PDF_CMAP_TABLE, 458 },
	{ 0xd6e7, 0xd6e8, PDF_CMAP_RANGE, 8859 },
	{ 0xd6e9, 0xd6ed, PDF_CMAP_RANGE, 4584 },
	{ 0xd6ee, 0xd6ef, PDF_CMAP_RANGE, 8861 },
	{ 0xd6f0, 0xd6f1, PDF_CMAP_RANGE, 4591 },
	{ 0xd6f2, 0xd6f2, PDF_CMAP_SINGLE, 8863 },
	{ 0xd6f3, 0xd6f4, PDF_CMAP_RANGE, 4594 },
	{ 0xd6f5, 0xd6f6, PDF_CMAP_RANGE, 8864 },
	{ 0xd6f7, 0xd6fb, PDF_CMAP_RANGE, 4598 },
	{ 0xd6fc, 0xd6fe, PDF_CMAP_RANGE, 8866 },
	{ 0xd7a1, 0xd7a3, PDF_CMAP_RANGE, 4606 },
	{ 0xd7a4, 0xd7a4, PDF_CMAP_SINGLE, 8869 },
	{ 0xd7a5, 0xd7a7, PDF_CMAP_RANGE, 4610 },
	{ 0xd7a8, 0xd7aa, PDF_CMAP_RANGE, 8870 },
	{ 0xd7ab, 0xd7ad, PDF_CMAP_TABLE, 460 },
	{ 0xd7ae, 0xd7b1, PDF_CMAP_RANGE, 8874 },
	{ 0xd7b2, 0xd7b2, PDF_CMAP_SINGLE, 4623 },
	{ 0xd7b3, 0xd7b4, PDF_CMAP_RANGE, 8878 },
	{ 0xd7b5, 0xd7b7, PDF_CMAP_TABLE, 463 },
	{ 0xd7b8, 0xd7bc, PDF_CMAP_RANGE, 8881 },
	{ 0xd7bd, 0xd7c6, PDF_CMAP_RANGE, 4634 },
	{ 0xd7c7, 0xd7c7, PDF_CMAP_SINGLE, 8886 },
	{ 0xd7c8, 0xd7c9, PDF_CMAP_RANGE, 4645 },
	{ 0xd7ca, 0xd7ca, PDF_CMAP_SINGLE, 8887 },
	{ 0xd7cb, 0xd7d4, PDF_CMAP_RANGE, 4648 },
	{ 0xd7d5, 0xd7d5, PDF_CMAP_SINGLE, 8888 },
	{ 0xd7d6, 0xd7da, PDF_CMAP_RANGE, 4659 },
	{ 0xd7db, 0xd7de, PDF_CMAP_RANGE, 8889 },
	{ 0xd7df, 0xd7e6, PDF_CMAP_RANGE, 4668 },
	{ 0xd7e7, 0xd7e8, PDF_CMAP_TABLE, 466 },
	{ 0xd7e9, 0xd7ea, PDF_CMAP_RANGE, 8894 },
	{ 0xd7eb, 0xd7f9, PDF_CMAP_RANGE, 4680 },
	{ 0xd8a1, 0xd8c3, PDF_CMAP_RANGE, 4695 },
	{ 0xd8c4, 0xd8c4, PDF_CMAP_SINGLE, 8896 },
	{ 0xd8c5, 0xd8c6, PDF_CMAP_RANGE, 4731 },
	{ 0xd8c7, 0xd8c9, PDF_CMAP_TABLE, 468 },
	{ 0xd8ca, 0xd8cb, PDF_CMAP_RANGE, 4736 },
	{ 0xd8cc, 0xd8cd, PDF_CMAP_RANGE, 8899 },
	{ 0xd8ce, 0xd8cf, PDF_CMAP_RANGE, 4740 },
	{ 0xd8d0, 0xd8d1, PDF_CMAP_RANGE, 8901 },
	{ 0xd8d2, 0xd8d3, PDF_CMAP_TABLE, 471 },
	{ 0xd8d4, 0xd8d8, PDF_CMAP_RANGE, 4746 },
	{ 0xd8d9, 0xd8da, PDF_CMAP_TABLE, 473 },
	{ 0xd8db, 0xd8dc, PDF_CMAP_RANGE, 8905 },
	{ 0xd8dd, 0xd8f0, PDF_CMAP_RANGE, 4755 },
	{ 0xd8f1, 0xd8f1, PDF_CMAP_SINGLE, 8907 },
	{ 0xd8f2, 0xd8f5, PDF_CMAP_RANGE, 4776 },
	{ 0xd8f6, 0xd8f7, PDF_CMAP_RANGE, 8908 },
	{ 0xd8f8, 0xd8fe, PDF_CMAP_RANGE, 4782 },
	{ 0xd9a1, 0xd9ac, PDF_CMAP_RANGE, 4789 },
	{ 0xd9ad, 0xd9b0, PDF_CMAP_TABLE, 475 },
	{ 0xd9b1, 0xd9b3, PDF_CMAP_RANGE, 8912 },
	{ 0xd9b4, 0xd9c6, PDF_CMAP_RANGE, 4808 },
	{ 0xd9c7, 0xd9c7, PDF_CMAP_SINGLE, 8915 },
	{ 0xd9c8, 0xd9cc, PDF_CMAP_RANGE, 4828 },
	{ 0xd9cd, 0xd9d0, PDF_CMAP_RANGE, 8916 },
	{ 0xd9d1, 0xd9dc, PDF_CMAP_RANGE, 4837 },
	{ 0xd9dd, 0xd9dd, PDF_CMAP_SINGLE, 8920 },
	{ 0xd9de, 0xd9e0, PDF_CMAP_RANGE, 4850 },
	{ 0xd9e1, 0xd9e1, PDF_CMAP_SINGLE, 8921 },
	{ 0xd9e2, 0xd9e3, PDF_CMAP_RANGE, 4854 },
	{ 0xd9e4, 0xd9e6, PDF_CMAP_TABLE, 479 },
	{ 0xd9e7, 0xd9eb, PDF_CMAP_RANGE, 4859 },
	{ 0xd9ec, 0xd9ec, PDF_CMAP_SINGLE, 8924 },
	{ 0xd9ed, 0xd9f3, PDF_CMAP_RANGE, 4865 },
	{ 0xd9f4, 0xd9f5, PDF_CMAP_RANGE, 8925 },
	{ 0xd9f6, 0xd9fe, PDF_CMAP_RANGE, 4874 },
	{ 0xdaa1, 0xdaa4, PDF_CMAP_RANGE, 4883 },
	{ 0xdaa5, 0xdadf, PDF_CMAP_RANGE, 8927 },
	{ 0xdae0, 0xdae9, PDF_CMAP_RANGE, 4946 },
	{ 0xdaea, 0xdaea, PDF_CMAP_SINGLE, 8986 },
	{ 0xdaeb, 0xdaf6, PDF_CMAP_RANGE, 4957 },
	{ 0xdaf7, 0xdaf9, PDF_CMAP_TABLE, 482 },
	{ 0xdafa, 0xdafd, PDF_CMAP_RANGE, 4972 },
	{ 0xdafe, 0xdafe, PDF_CMAP_SINGLE, 8989 },
	{ 0xdba1, 0xdba2, PDF_CMAP_RANGE, 4977 },
	{ 0xdba3, 0xdba3, PDF_CMAP_SINGLE, 8990 },
	{ 0xdba4, 0xdba5, PDF_CMAP_RANGE, 4980 },
	{ 0xdba6, 0xdba6, PDF_CMAP_SINGLE, 8991 },
	{ 0xdba7, 0xdba8, PDF_CMAP_RANGE, 4983 },
	{ 0xdba9, 0xdbaa, PDF_CMAP_RANGE, 8992 },
	{ 0xdbab, 0xdbba, PDF_CMAP_RANGE, 4987 },
	{ 0xdbbb, 0xdbbd, PDF_CMAP_TABLE, 485 },
	{ 0xdbbe, 0xdbce, PDF_CMAP_RANGE, 5006 },
	{ 0xdbcf, 0xdbd1, PDF_CMAP_TABLE, 488 },
	{ 0xdbd2, 0xdbda, PDF_CMAP_RANGE, 5026 },
	{ 0xdbdb, 0xdbdb, PDF_CMAP_SINGLE, 8998 },
	{ 0xdbdc, 0xdbdd, PDF_CMAP_RANGE, 5036 },
	{ 0xdbde, 0xdbde, PDF_CMAP_SINGLE, 8999 },
	{ 0xdbdf, 0xdbe1, PDF_CMAP_RANGE, 5039 },
	{ 0xdbe2, 0xdbe4, PDF_CMAP_TABLE, 491 },
	{ 0xdbe5, 0xdbea, PDF_CMAP_RANGE, 5045 },
	{ 0xdbeb, 0xdbeb, PDF_CMAP_SINGLE, 9002 },
	{ 0xdbec, 0xdbed, PDF_CMAP_RANGE, 5052 },
	{ 0xdbee, 0xdbee, PDF_CMAP_SINGLE, 9003 },
	{ 0xdbef, 0xdbf0, PDF_CMAP_RANGE, 5055 },
	{ 0xdbf1, 0xdbf1, PDF_CMAP_SINGLE, 9004 },
	{ 0xdbf2, 0xdbf4, PDF_CMAP_RANGE, 5058 },
	{ 0xdbf5, 0xdbf7, PDF_CMAP_RANGE, 9005 },
	{ 0xdbf8, 0xdbfe, PDF_CMAP_RANGE, 5064 },
	{ 0xdca1, 0xdcbb, PDF_CMAP_RANGE, 5071 },
	{ 0xdcbc, 0xdcbc, PDF_CMAP_SINGLE, 9008 },
	{ 0xdcbd, 0xdcbe, PDF_CMAP_RANGE, 5099 },
	{ 0xdcbf, 0xdcbf, PDF_CMAP_SINGLE, 9009 },
	{ 0xdcc0, 0xdcc1, PDF_CMAP_RANGE, 5102 },
	{ 0xdcc2, 0xdcc2, PDF_CMAP_SINGLE, 9010 },
	{ 0xdcc3, 0xdcc7, PDF_CMAP_RANGE, 5105 },
	{ 0xdcc8, 0xdcca, PDF_CMAP_RANGE, 9011 },
	{ 0xdccb, 0xdcd0, PDF_CMAP_RANGE, 5113 },
	{ 0xdcd1, 0xdcd1, PDF_CMAP_SINGLE, 9014 },
	{ 0xdcd2, 0xdcd6, PDF_CMAP_RANGE, 5120 },
	{ 0xdcd7, 0xdcd7, PDF_CMAP_SINGLE, 9015 },
	{ 0xdcd8, 0xdcdf, PDF_CMAP_RANGE, 5126 },
	{ 0xdce0, 0xdce0, PDF_CMAP_SINGLE, 9016 },
	{ 0xdce1, 0xdce2, PDF_CMAP_RANGE, 5135 },
	{ 0xdce3, 0xdce4, PDF_CMAP_RANGE, 9017 },
	{ 0xdce5, 0xdce8, PDF_CMAP_RANGE, 5139 },
	{ 0xdce9, 0xdcea, PDF_CMAP_RANGE, 9019 },
	{ 0xdceb, 0xdcf0, PDF_CMAP_RANGE, 5145 },
	{ 0xdcf1, 0xdcf1, PDF_CMAP_SINGLE, 9021 },
	{ 0xdcf2, 0xdcf5, PDF_CMAP_RANGE, 5152 },
	{ 0xdcf6, 0xdcf6, PDF_CMAP_SINGLE, 9022 },
	{ 0xdcf7, 0xdcf8, PDF_CMAP_RANGE, 5157 },
	{ 0xdcf9, 0xdcf9, PDF_CMAP_SINGLE, 9023 },
	{ 0xdcfa, 0xdcfc, PDF_CMAP_RANGE, 5160 },
	{ 0xdcfd, 0xdcfe, PDF_CMAP_RANGE, 9024 },
	{ 0xdda1, 0xdda2, PDF_CMAP_TABLE, 494 },
	{ 0xdda3, 0xdda7, PDF_CMAP_RANGE, 9027 },
	{ 0xdda8, 0xdda9, PDF_CMAP_RANGE, 5172 },
	{ 0xddaa, 0xddab, PDF_CMAP_RANGE, 9032 },
	{ 0xddac, 0xddb1, PDF_CMAP_RANGE, 5176 },
	{ 0xddb2, 0xddb2, PDF_CMAP_SINGLE, 9034 },
	{ 0xddb3, 0xddb4, PDF_CMAP_RANGE, 5183 },
	{ 0xddb5, 0xddb5, PDF_CMAP_SINGLE, 9035 },
	{ 0xddb6, 0xddb9, PDF_CMAP_RANGE, 5186 },
	{ 0xddba, 0xddbb, PDF_CMAP_RANGE, 9036 },
	{ 0xddbc, 0xddd2, PDF_CMAP_RANGE, 5192 },
	{ 0xddd3, 0xddd3, PDF_CMAP_SINGLE, 9038 },
	{ 0xddd4, 0xddda, PDF_CMAP_RANGE, 5216 },
	{ 0xdddb, 0xdddb, PDF_CMAP_SINGLE, 9039 },
	{ 0xdddc, 0xdddd, PDF_CMAP_RANGE, 5224 },
	{ 0xddde, 0xddde, PDF_CMAP_SINGLE, 9040 },
	{ 0xdddf, 0xdde3, PDF_CMAP_RANGE, 5227 },
	{ 0xdde4, 0xdde4, PDF_CMAP_SINGLE, 9041 },
	{ 0xdde5, 0xddea, PDF_CMAP_RANGE, 5233 },
	{ 0xddeb, 0xddeb, PDF_CMAP_SINGLE, 9042 },
	{ 0xddec, 0xddf0, PDF_CMAP_RANGE, 5240 },
	{ 0xddf1, 0xddf1, PDF_CMAP_SINGLE, 9043 },
	{ 0xddf2, 0xddf5, PDF_CMAP_RANGE, 5246 },
	{ 0xddf6, 0xddf7, PDF_CMAP_RANGE, 9044 },
	{ 0xddf8, 0xddfb, PDF_CMAP_RANGE, 5252 },
	{ 0xddfc, 0xddfe, PDF_CMAP_TABLE, 496 },
	{ 0xdea1, 0xdeac, PDF_CMAP_RANGE, 5259 },
	{ 0xdead, 0xdead, PDF_CMAP_SINGLE, 9048 },
	{ 0xdeae, 0xdeb3, PDF_CMAP_RANGE, 5272 },
	{ 0xdeb4, 0xdeb4, PDF_CMAP_SINGLE, 9049 },
	{ 0xdeb5, 0xdeb9, PDF_CMAP_RANGE, 5279 },
	{ 0xdeba, 0xdeba, PDF_CMAP_SINGLE, 9050 },
	{ 0xdebb, 0xdec5, PDF_CMAP_RANGE, 5285 },
	{ 0xdec6, 0xdec6, PDF_CMAP_SINGLE, 9051 },
	{ 0xdec7, 0xdece, PDF_CMAP_RANGE, 5297 },
	{ 0xdecf, 0xded0, PDF_CMAP_TABLE, 499 },
	{ 0xded1, 0xded2, PDF_CMAP_RANGE, 9053 },
	{ 0xded3, 0xded7, PDF_CMAP_RANGE, 5309 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 9055 },
	{ 0xded9, 0xdee1, PDF_CMAP_RANGE, 5315 },
	{ 0xdee2, 0xdee2, PDF_CMAP_SINGLE, 9056 },
	{ 0xdee3, 0xdee7, PDF_CMAP_RANGE, 5325 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 9057 },
	{ 0xdee9, 0xdeeb, PDF_CMAP_RANGE, 5331 },
	{ 0xdeec, 0xdeec, PDF_CMAP_SINGLE, 9058 },
	{ 0xdeed, 0xdef2, PDF_CMAP_RANGE, 5335 },
	{ 0xdef3, 0xdef3, PDF_CMAP_SINGLE, 9059 },
	{ 0xdef4, 0xdefb, PDF_CMAP_RANGE, 5342 },
	{ 0xdefc, 0xdefc, PDF_CMAP_SINGLE, 9060 },
	{ 0xdefd, 0xdefe, PDF_CMAP_RANGE, 5351 },
	{ 0xdfa1, 0xdfa1, PDF_CMAP_SINGLE, 5353 },
	{ 0xdfa2, 0xdfa3, PDF_CMAP_RANGE, 9061 },
	{ 0xdfa4, 0xdfa5, PDF_CMAP_TABLE, 501 },
	{ 0xdfa6, 0xdfb3, PDF_CMAP_RANGE, 5358 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 9064 },
	{ 0xdfb5, 0xdfbb, PDF_CMAP_RANGE, 5373 },
	{ 0xdfbc, 0xdfbd, PDF_CMAP_RANGE, 9065 },
	{ 0xdfbe, 0xdfbf, PDF_CMAP_TABLE, 503 },
	{ 0xdfc0, 0xdfc1, PDF_CMAP_RANGE, 5384 },
	{ 0xdfc2, 0xdfc3, PDF_CMAP_RANGE, 9068 },
	{ 0xdfc4, 0xdfcb, PDF_CMAP_RANGE, 5388 },
	{ 0xdfcc, 0xdfcc, PDF_CMAP_SINGLE, 9070 },
	{ 0xdfcd, 0xdfcf, PDF_CMAP_RANGE, 5397 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 9071 },
	{ 0xdfd1, 0xdfd4, PDF_CMAP_RANGE, 5401 },
	{ 0xdfd5, 0xdfd5, PDF_CMAP_SINGLE, 9072 },
	{ 0xdfd6, 0xdfd7, PDF_CMAP_RANGE, 5406 },
	{ 0xdfd8, 0xdfd9, PDF_CMAP_RANGE, 9073 },
	{ 0xdfda, 0xdfdb, PDF_CMAP_RANGE, 5410 },
	{ 0xdfdc, 0xdfdc, PDF_CMAP_SINGLE, 9075 },
	{ 0xdfdd, 0xdfdf, PDF_CMAP_RANGE, 5413 },
	{ 0xdfe0, 0xdfe2, PDF_CMAP_TABLE, 505 },
	{ 0xdfe3, 0xdfe5, PDF_CMAP_RANGE, 5419 },
	{ 0xdfe6, 0xdfe6, PDF_CMAP_SINGLE, 9078 },
	{ 0xdfe7, 0xdfe8, PDF_CMAP_RANGE, 5423 },
	{ 0xdfe9, 0xdfeb, PDF_CMAP_TABLE, 508 },
	{ 0xdfec, 0xdfee, PDF_CMAP_RANGE, 5428 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 9081 },
	{ 0xdff0, 0xdff4, PDF_CMAP_RANGE, 5432 },
	{ 0xdff5, 0xdff5, PDF_CMAP_SINGLE, 9082 },
	{ 0xdff6, 0xdff8, PDF_CMAP_RANGE, 5438 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 9083 },
	{ 0xdffa, 0xdffe, PDF_CMAP_RANGE, 5442 },
	{ 0xe0a1, 0xe0b5, PDF_CMAP_RANGE, 5447 },
	{ 0xe0b6, 0xe0b7, PDF_CMAP_RANGE, 9084 },
	{ 0xe0b8, 0xe0be, PDF_CMAP_RANGE, 5470 },
	{ 0xe0bf, 0xe0bf, PDF_CMAP_SINGLE, 9086 },
	{ 0xe0c0, 0xe0c7, PDF_CMAP_RANGE, 5478 },
	{ 0xe0c8, 0xe0c8, PDF_CMAP_SINGLE, 9087 },
	{ 0xe0c9, 0xe0cd, PDF_CMAP_RANGE, 5487 },
	{ 0xe0ce, 0xe0ce, PDF_CMAP_SINGLE, 9088 },
	{ 0xe0cf, 0xe0d2, PDF_CMAP_RANGE, 5493 },
	{ 0xe0d3, 0xe0d3, PDF_CMAP_SINGLE, 9089 },
	{ 0xe0d4, 0xe0df, PDF_CMAP_RANGE, 5498 },
	{ 0xe0e0, 0xe0e0, PDF_CMAP_SINGLE, 9090 },
	{ 0xe0e1, 0xe0ef, PDF_CMAP_RANGE, 5511 },
	{ 0xe0f0, 0xe0f0, PDF_CMAP_SINGLE, 9091 },
	{ 0xe0f1, 0xe0f7, PDF_CMAP_RANGE, 5527 },
	{ 0xe0f8, 0xe0f8, PDF_CMAP_SINGLE, 9092 },
	{ 0xe0f9, 0xe0fb, PDF_CMAP_RANGE, 5535 },
	{ 0xe0fc, 0xe0fe, PDF_CMAP_RANGE, 9093 },
	{ 0xe1a1, 0xe1aa, PDF_CMAP_RANGE, 5541 },
	{ 0xe1ab, 0xe1ad, PDF_CMAP_TABLE, 511 },
	{ 0xe1ae, 0xe1af, PDF_CMAP_RANGE, 5554 },
	{ 0xe1b0, 0xe1b0, PDF_CMAP_SINGLE, 9098 },
	{ 0xe1b1, 0xe1b3, PDF_CMAP_RANGE, 5557 },
	{ 0xe1b4, 0xe1b4, PDF_CMAP_SINGLE, 9099 },
	{ 0xe1b5, 0xe1ba, PDF_CMAP_RANGE, 5561 },
	{ 0xe1bb, 0xe1bd, PDF_CMAP_TABLE, 514 },
	{ 0xe1be, 0xe1bf, PDF_CMAP_RANGE, 5570 },
	{ 0xe1c0, 0xe1c1, PDF_CMAP_RANGE, 9102 },
	{ 0xe1c2, 0xe1c8, PDF_CMAP_RANGE, 5574 },
	{ 0xe1c9, 0xe1c9, PDF_CMAP_SINGLE, 9104 },
	{ 0xe1ca, 0xe1cf, PDF_CMAP_RANGE, 5582 },
	{ 0xe1d0, 0xe1d0, PDF_CMAP_SINGLE, 9105 },
	{ 0xe1d1, 0xe1da, PDF_CMAP_RANGE, 5589 },
	{ 0xe1db, 0xe1db, PDF_CMAP_SINGLE, 9106 },
	{ 0xe1dc, 0xe1e0, PDF_CMAP_RANGE, 5600 },
	{ 0xe1e1, 0xe1e2, PDF_CMAP_TABLE, 517 },
	{ 0xe1e3, 0xe1e6, PDF_CMAP_RANGE, 5607 },
	{ 0xe1e7, 0xe1e7, PDF_CMAP_SINGLE, 4504 },
	{ 0xe1e8, 0xe1ed, PDF_CMAP_RANGE, 5612 },
	{ 0xe1ee, 0xe1ef, PDF_CMAP_RANGE, 9108 },
	{ 0xe1f0, 0xe1f5, PDF_CMAP_RANGE, 5620 },
	{ 0xe1f6, 0xe1f8, PDF_CMAP_TABLE, 519 },
	{ 0xe1f9, 0xe1fc, PDF_CMAP_RANGE, 5629 },
	{ 0xe1fd, 0xe1fe, PDF_CMAP_TABLE, 522 },
	{ 0xe2a1, 0xe2a3, PDF_CMAP_RANGE, 5635 },
	{ 0xe2a4, 0xe2a4, PDF_CMAP_SINGLE, 9113 },
	{ 0xe2a5, 0xe2a7, PDF_CMAP_RANGE, 5639 },
	{ 0xe2a8, 0xe2a8, PDF_CMAP_SINGLE, 9114 },
	{ 0xe2a9, 0xe2ba, PDF_CMAP_RANGE, 5643 },
	{ 0xe2bb, 0xe2c4, PDF_CMAP_RANGE, 9115 },
	{ 0xe2c5, 0xe2c5, PDF_CMAP_SINGLE, 4293 },
	{ 0xe2c6, 0xe2ce, PDF_CMAP_RANGE, 9125 },
	{ 0xe2cf, 0xe2d0, PDF_CMAP_TABLE, 524 },
	{ 0xe2d1, 0xe2d8, PDF_CMAP_RANGE, 5683 },
	{ 0xe2d9, 0xe2d9, PDF_CMAP_SINGLE, 9135 },
	{ 0xe2da, 0xe2e2, PDF_CMAP_RANGE, 5692 },
	{ 0xe2e3, 0xe2e4, PDF_CMAP_RANGE, 9136 },
	{ 0xe2e5, 0xe2e6, PDF_CMAP_TABLE, 526 },
	{ 0xe2e7, 0xe2e8, PDF_CMAP_RANGE, 5705 },
	{ 0xe2e9, 0xe2eb, PDF_CMAP_RANGE, 9139 },
	{ 0xe2ec, 0xe2f7, PDF_CMAP_RANGE, 5710 },
	{ 0xe2f8, 0xe2f9, PDF_CMAP_TABLE, 528 },
	{ 0xe2fa, 0xe2fd, PDF_CMAP_RANGE, 9143 },
	{ 0xe2fe, 0xe2fe, PDF_CMAP_SINGLE, 5728 },
	{ 0xe3a1, 0xe3a2, PDF_CMAP_TABLE, 530 },
	{ 0xe3a3, 0xe3a4, PDF_CMAP_RANGE, 5731 },
	{ 0xe3a5, 0xe3a5, PDF_CMAP_SINGLE, 9148 },
	{ 0xe3a6, 0xe3aa, PDF_CMAP_RANGE, 5734 },
	{ 0xe3ab, 0xe3ab, PDF_CMAP_SINGLE, 9149 },
	{ 0xe3ac, 0xe3b3, PDF_CMAP_RANGE, 5740 },
	{ 0xe3b4, 0xe3b4, PDF_CMAP_SINGLE, 9150 },
	{ 0xe3b5, 0xe3c4, PDF_CMAP_RANGE, 5749 },
	{ 0xe3c5, 0xe3db, PDF_CMAP_RANGE, 9151 },
	{ 0xe3dc, 0xe3e2, PDF_CMAP_RANGE, 5788 },
	{ 0xe3e3, 0xe3e3, PDF_CMAP_SINGLE, 9174 },
	{ 0xe3e4, 0xe3ec, PDF_CMAP_RANGE, 5796 },
	{ 0xe3ed, 0xe3ed, PDF_CMAP_SINGLE, 9175 },
	{ 0xe3ee, 0xe3f0, PDF_CMAP_RANGE, 5806 },
	{ 0xe3f1, 0xe3f2, PDF_CMAP_RANGE, 9176 },
	{ 0xe3f3, 0xe3f7, PDF_CMAP_RANGE, 5811 },
	{ 0xe3f8, 0xe3f8, PDF_CMAP_SINGLE, 9178 },
	{ 0xe3f9, 0xe3fd, PDF_CMAP_RANGE, 5817 },
	{ 0xe3fe, 0xe3fe, PDF_CMAP_SINGLE, 9179 },
	{ 0xe4a1, 0xe4a3, PDF_CMAP_RANGE, 5823 },
	{ 0xe4a4, 0xe4a5, PDF_CMAP_RANGE, 9180 },
	{ 0xe4a6, 0xe4aa, PDF_CMAP_RANGE, 5828 },
	{ 0xe4ab, 0xe4ab, PDF_CMAP_SINGLE, 9182 },
	{ 0xe4ac, 0xe4ae, PDF_CMAP_RANGE, 5834 },
	{ 0xe4af, 0xe4b1, PDF_CMAP_RANGE, 9183 },
	{ 0xe4b2, 0xe4b4, PDF_CMAP_RANGE, 5840 },
	{ 0xe4b5, 0xe4b6, PDF_CMAP_RANGE, 9186 },
	{ 0xe4b7, 0xe4c1, PDF_CMAP_RANGE, 5845 },
	{ 0xe4c2, 0xe4c2, PDF_CMAP_SINGLE, 9188 },
	{ 0xe4c3, 0xe4c4, PDF_CMAP_RANGE, 5857 },
	{ 0xe4c5, 0xe4c5, PDF_CMAP_SINGLE, 9189 },
	{ 0xe4c6, 0xe4c8, PDF_CMAP_RANGE, 5860 },
	{ 0xe4c9, 0xe4c9, PDF_CMAP_SINGLE, 9190 },
	{ 0xe4ca, 0xe4d8, PDF_CMAP_RANGE, 5864 },
	{ 0xe4d9, 0xe4d9, PDF_CMAP_SINGLE, 9191 },
	{ 0xe4da, 0xe4db, PDF_CMAP_RANGE, 5880 },
	{ 0xe4dc, 0xe4de, PDF_CMAP_TABLE, 532 },
	{ 0xe4df, 0xe4e3, PDF_CMAP_RANGE, 5885 },
	{ 0xe4e4, 0xe4e4, PDF_CMAP_SINGLE, 9194 },
	{ 0xe4e5, 0xe4ea, PDF_CMAP_RANGE, 5891 },
	{ 0xe4eb, 0xe4ec, PDF_CMAP_RANGE, 9195 },
	{ 0xe4ed, 0xe4f1, PDF_CMAP_RANGE, 5899 },
	{ 0xe4f2, 0xe4f2, PDF_CMAP_SINGLE, 9197 },
	{ 0xe4f3, 0xe4fd, PDF_CMAP_RANGE, 5905 },
	{ 0xe4fe, 0xe4fe, PDF_CMAP_SINGLE, 9198 },
	{ 0xe5a1, 0xe5af, PDF_CMAP_RANGE, 5917 },
	{ 0xe5b0, 0xe5b0, PDF_CMAP_SINGLE, 9199 },
	{ 0xe5b1, 0xe5b8, PDF_CMAP_RANGE, 5933 },
	{ 0xe5b9, 0xe5b9, PDF_CMAP_SINGLE, 9200 },
	{ 0xe5ba, 0xe5c6, PDF_CMAP_RANGE, 5942 },
	{ 0xe5c7, 0xe5c9, PDF_CMAP_TABLE, 535 },
	{ 0xe5ca, 0xe5cd, PDF_CMAP_RANGE, 5958 },
	{ 0xe5ce, 0xe5ce, PDF_CMAP_SINGLE, 9203 },
	{ 0xe5cf, 0xe5ef, PDF_CMAP_RANGE, 5963 },
	{ 0xe5f0, 0xe5f2, PDF_CMAP_TABLE, 538 },
	{ 0xe5f3, 0xe5fb, PDF_CMAP_RANGE, 5999 },
	{ 0xe5fc, 0xe5fd, PDF_CMAP_RANGE, 9206 },
	{ 0xe5fe, 0xe5fe, PDF_CMAP_SINGLE, 6010 },
	{ 0xe6a1, 0xe6a2, PDF_CMAP_RANGE, 6011 },
	{ 0xe6a3, 0xe6a3, PDF_CMAP_SINGLE, 9208 },
	{ 0xe6a4, 0xe6aa, PDF_CMAP_RANGE, 6014 },
	{ 0xe6ab, 0xe6ac, PDF_CMAP_RANGE, 9209 },
	{ 0xe6ad, 0xe6ae, PDF_CMAP_TABLE, 541 },
	{ 0xe6af, 0xe6b3, PDF_CMAP_RANGE, 6025 },
	{ 0xe6b4, 0xe6b5, PDF_CMAP_RANGE, 9212 },
	{ 0xe6b6, 0xe6be, PDF_CMAP_RANGE, 6032 },
	{ 0xe6bf, 0xe6bf, PDF_CMAP_SINGLE, 9214 },
	{ 0xe6c0, 0xe6c7, PDF_CMAP_RANGE, 6042 },
	{ 0xe6c8, 0xe6c9, PDF_CMAP_RANGE, 9215 },
	{ 0xe6ca, 0xe6cc, PDF_CMAP_RANGE, 6052 },
	{ 0xe6cd, 0xe6cd, PDF_CMAP_SINGLE, 9217 },
	{ 0xe6ce, 0xe6df, PDF_CMAP_RANGE, 6056 },
	{ 0xe6e0, 0xe6fe, PDF_CMAP_RANGE, 9218 },
	{ 0xe7a1, 0xe7da, PDF_CMAP_RANGE, 9249 },
	{ 0xe7db, 0xe7e0, PDF_CMAP_RANGE, 6163 },
	{ 0xe7e1, 0xe7e2, PDF_CMAP_RANGE, 9307 },
	{ 0xe7e3, 0xe7e6, PDF_CMAP_RANGE, 6171 },
	{ 0xe7e7, 0xe7e7, PDF_CMAP_SINGLE, 9309 },
	{ 0xe7e8, 0xe7ee, PDF_CMAP_RANGE, 6176 },
	{ 0xe7ef, 0xe7ef, PDF_CMAP_SINGLE, 9310 },
	{ 0xe7f0, 0xe7f3, PDF_CMAP_RANGE, 6184 },
	{ 0xe7f4, 0xe7f6, PDF_CMAP_RANGE, 9311 },
	{ 0xe7f7, 0xe7fe, PDF_CMAP_RANGE, 6191 },
	{ 0xe8a1, 0xe8a7, PDF_CMAP_RANGE, 6199 },
	{ 0xe8a8, 0xe8a8, PDF_CMAP_SINGLE, 9314 },
	{ 0xe8a9, 0xe8ab, PDF_CMAP_RANGE, 6207 },
	{ 0xe8ac, 0xe8ac, PDF_CMAP_SINGLE, 9315 },
	{ 0xe8ad, 0xe8b5, PDF_CMAP_RANGE, 6211 },
	{ 0xe8b6, 0xe8b7, PDF_CMAP_TABLE, 543 },
	{ 0xe8b8, 0xe8ba, PDF_CMAP_RANGE, 9317 },
	{ 0xe8bb, 0xe8be, PDF_CMAP_RANGE, 6225 },
	{ 0xe8bf, 0xe8c0, PDF_CMAP_RANGE, 9320 },
	{ 0xe8c1, 0xe8c4, PDF_CMAP_RANGE, 6231 },
	{ 0xe8c5, 0xe8c6, PDF_CMAP_TABLE, 545 },
	{ 0xe8c7, 0xe8c9, PDF_CMAP_RANGE, 9323 },
	{ 0xe8ca, 0xe8cd, PDF_CMAP_RANGE, 6240 },
	{ 0xe8ce, 0xe8d0, PDF_CMAP_TABLE, 547 },
	{ 0xe8d1, 0xe8d2, PDF_CMAP_RANGE, 6247 },
	{ 0xe8d3, 0xe8d3, PDF_CMAP_SINGLE, 9328 },
	{ 0xe8d4, 0xe8dc, PDF_CMAP_RANGE, 6250 },
	{ 0xe8dd, 0xe8df, PDF_CMAP_TABLE, 550 },
	{ 0xe8e0, 0xe8e1, PDF_CMAP_RANGE, 6262 },
	{ 0xe8e2, 0xe8e3, PDF_CMAP_RANGE, 9331 },
	{ 0xe8e4, 0xe8e7, PDF_CMAP_TABLE, 553 },
	{ 0xe8e8, 0xe8ea, PDF_CMAP_RANGE, 6270 },
	{ 0xe8eb, 0xe8ef, PDF_CMAP_TABLE, 557 },
	{ 0xe8f0, 0xe8f8, PDF_CMAP_RANGE, 6278 },
	{ 0xe8f9, 0xe8f9, PDF_CMAP_SINGLE, 9338 },
	{ 0xe8fa, 0xe8fb, PDF_CMAP_RANGE, 6288 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 9339 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 6292 },
	{ 0xe9a1, 0xe9a1, PDF_CMAP_SINGLE, 9341 },
	{ 0xe9a2, 0xe9ac, PDF_CMAP_RANGE, 6294 },
	{ 0xe9ad, 0xe9ad, PDF_CMAP_SINGLE, 9342 },
	{ 0xe9ae, 0xe9b3, PDF_CMAP_RANGE, 6306 },
	{ 0xe9b4, 0xe9b5, PDF_CMAP_RANGE, 9343 },
	{ 0xe9b6, 0xe9b7, PDF_CMAP_TABLE, 562 },
	{ 0xe9b8, 0xe9c3, PDF_CMAP_RANGE, 6316 },
	{ 0xe9c4, 0xe9c6, PDF_CMAP_TABLE, 564 },
	{ 0xe9c7, 0xe9c8, PDF_CMAP_RANGE, 6331 },
	{ 0xe9c9, 0xe9c9, PDF_CMAP_SINGLE, 9348 },
	{ 0xe9ca, 0xe9d5, PDF_CMAP_RANGE, 6334 },
	{ 0xe9d6, 0xe9d6, PDF_CMAP_SINGLE, 9349 },
	{ 0xe9d7, 0xe9d9, PDF_CMAP_RANGE, 6347 },
	{ 0xe9da, 0xe9da, PDF_CMAP_SINGLE, 9350 },
	{ 0xe9db, 0xe9e3, PDF_CMAP_RANGE, 6351 },
	{ 0xe9e4, 0xe9e5, PDF_CMAP_TABLE, 567 },
	{ 0xe9e6, 0xe9e7, PDF_CMAP_RANGE, 9352 },
	{ 0xe9e8, 0xe9ec, PDF_CMAP_TABLE, 569 },
	{ 0xe9ed, 0xe9fe, PDF_CMAP_RANGE, 9356 },
	{ 0xeaa1, 0xeaa5, PDF_CMAP_RANGE, 9374 },
	{ 0xeaa6, 0xeaa6, PDF_CMAP_SINGLE, 6392 },
	{ 0xeaa7, 0xeaa8, PDF_CMAP_RANGE, 9379 },
	{ 0xeaa9, 0xeab0, PDF_CMAP_RANGE, 6395 },
	{ 0xeab1, 0xeab1, PDF_CMAP_SINGLE, 9381 },
	{ 0xeab2, 0xeabb, PDF_CMAP_RANGE, 6404 },
	{ 0xeabc, 0xeabc, PDF_CMAP_SINGLE, 9382 },
	{ 0xeabd, 0xeac9, PDF_CMAP_RANGE, 6415 },
	{ 0xeaca, 0xeaca, PDF_CMAP_SINGLE, 9383 },
	{ 0xeacb, 0xeacc, PDF_CMAP_RANGE, 6429 },
	{ 0xeacd, 0xeacd, PDF_CMAP_SINGLE, 9384 },
	{ 0xeace, 0xead2, PDF_CMAP_RANGE, 6432 },
	{ 0xead3, 0xead3, PDF_CMAP_SINGLE, 9385 },
	{ 0xead4, 0xead9, PDF_CMAP_RANGE, 6438 },
	{ 0xeada, 0xeaef, PDF_CMAP_RANGE, 9386 },
	{ 0xeaf0, 0xeafe, PDF_CMAP_RANGE, 6466 },
	{ 0xeba1, 0xeba6, PDF_CMAP_RANGE, 6481 },
	{ 0xeba7, 0xeba7, PDF_CMAP_SINGLE, 9408 },
	{ 0xeba8, 0xeba9, PDF_CMAP_RANGE, 6488 },
	{ 0xebaa, 0xebaa, PDF_CMAP_SINGLE, 9409 },
	{ 0xebab, 0xebb1, PDF_CMAP_RANGE, 6491 },
	{ 0xebb2, 0xebb2, PDF_CMAP_SINGLE, 9410 },
	{ 0xebb3, 0xebb8, PDF_CMAP_RANGE, 6499 },
	{ 0xebb9, 0xebb9, PDF_CMAP_SINGLE, 9411 },
	{ 0xebba, 0xebc9, PDF_CMAP_RANGE, 6506 },
	{ 0xebca, 0xebcb, PDF_CMAP_RANGE, 9412 },
	{ 0xebcc, 0xebcd, PDF_CMAP_TABLE, 574 },
	{ 0xebce, 0xebd5, PDF_CMAP_RANGE, 6526 },
	{ 0xebd6, 0xebd6, PDF_CMAP_SINGLE, 9415 },
	{ 0xebd7, 0xebd9, PDF_CMAP_RANGE, 6535 },
	{ 0xebda, 0xebda, PDF_CMAP_SINGLE, 9416 },
	{ 0xebdb, 0xebe0, PDF_CMAP_RANGE, 6539 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 9417 },
	{ 0xebe2, 0xebf6, PDF_CMAP_RANGE, 6546 },
	{ 0xebf7, 0xebf7, PDF_CMAP_SINGLE, 9418 },
	{ 0xebf8, 0xebfe, PDF_CMAP_RANGE, 6568 },
	{ 0xeca1, 0xeca2, PDF_CMAP_RANGE, 6575 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 9419 },
	{ 0xeca4, 0xeca8, PDF_CMAP_RANGE, 6578 },
	{ 0xeca9, 0xecae, PDF_CMAP_RANGE, 9420 },
	{ 0xecaf, 0xecb0, PDF_CMAP_RANGE, 6589 },
	{ 0xecb1, 0xecb1, PDF_CMAP_SINGLE, 9426 },
	{ 0xecb2, 0xecb3, PDF_CMAP_RANGE, 6592 },
	{ 0xecb4, 0xecb5, PDF_CMAP_RANGE, 9427 },
	{ 0xecb6, 0xecbd, PDF_CMAP_RANGE, 6596 },
	{ 0xecbe, 0xecbf, PDF_CMAP_RANGE, 9429 },
	{ 0xecc0, 0xecc1, PDF_CMAP_TABLE, 576 },
	{ 0xecc2, 0xecc6, PDF_CMAP_RANGE, 6608 },
	{ 0xecc7, 0xecc7, PDF_CMAP_SINGLE, 9432 },
	{ 0xecc8, 0xecca, PDF_CMAP_RANGE, 6614 },
	{ 0xeccb, 0xeccb, PDF_CMAP_SINGLE, 9433 },
	{ 0xeccc, 0xece1, PDF_CMAP_RANGE, 6618 },
	{ 0xece2, 0xece2, PDF_CMAP_SINGLE, 9434 },
	{ 0xece3, 0xecf1, PDF_CMAP_RANGE, 6641 },
	{ 0xecf2, 0xecf2, PDF_CMAP_SINGLE, 9435 },
	{ 0xecf3, 0xecf4, PDF_CMAP_RANGE, 6657 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 9436 },
	{ 0xecf6, 0xecf7, PDF_CMAP_RANGE, 6660 },
	{ 0xecf8, 0xecf8, PDF_CMAP_SINGLE, 9437 },
	{ 0xecf9, 0xecfe, PDF_CMAP_RANGE, 6663 },
	{ 0xeda1, 0xeda1, PDF_CMAP_SINGLE, 9438 },
	{ 0xeda2, 0xeda7, PDF_CMAP_RANGE, 6670 },
	{ 0xeda8, 0xeda8, PDF_CMAP_SINGLE, 9439 },
	{ 0xeda9, 0xedae, PDF_CMAP_RANGE, 6677 },
	{ 0xedaf, 0xedb0, PDF_CMAP_RANGE, 9440 },
	{ 0xedb1, 0xedb3, PDF_CMAP_RANGE, 6685 },
	{ 0xedb4, 0xedba, PDF_CMAP_TABLE, 578 },
	{ 0xedbb, 0xedbe, PDF_CMAP_RANGE, 6695 },
	{ 0xedbf, 0xedbf, PDF_CMAP_SINGLE, 9446 },
	{ 0xedc0, 0xedc1, PDF_CMAP_RANGE, 6700 },
	{ 0xedc2, 0xedc3, PDF_CMAP_RANGE, 9447 },
	{ 0xedc4, 0xedcb, PDF_CMAP_RANGE, 6704 },
	{ 0xedcc, 0xedcd, PDF_CMAP_RANGE, 9449 },
	{ 0xedce, 0xedd2, PDF_CMAP_RANGE, 6714 },
	{ 0xedd3, 0xedd3, PDF_CMAP_SINGLE, 9451 },
	{ 0xedd4, 0xedd6, PDF_CMAP_RANGE, 6720 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 9452 },
	{ 0xedd8, 0xede7, PDF_CMAP_RANGE, 6724 },
	{ 0xede8, 0xede8, PDF_CMAP_SINGLE, 9453 },
	{ 0xede9, 0xeded, PDF_CMAP_RANGE, 6741 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 9454 },
	{ 0xedef, 0xedf8, PDF_CMAP_RANGE, 6747 },
	{ 0xedf9, 0xedfa, PDF_CMAP_RANGE, 9455 },
	{ 0xedfb, 0xedfe, PDF_CMAP_RANGE, 6759 },
	{ 0xeea1, 0xeebb, PDF_CMAP_RANGE, 6763 },
	{ 0xeebc, 0xeebc, PDF_CMAP_SINGLE, 9457 },
	{ 0xeebd, 0xeebe, PDF_CMAP_RANGE, 6791 },
	{ 0xeebf, 0xeebf, PDF_CMAP_SINGLE, 9458 },
	{ 0xeec0, 0xeec3, PDF_CMAP_RANGE, 6794 },
	{ 0xeec4, 0xeefe, PDF_CMAP_RANGE, 9459 },
	{ 0xefa1, 0xeff1, PDF_CMAP_RANGE, 9518 },
	{ 0xeff2, 0xeffe, PDF_CMAP_RANGE, 6938 },
	{ 0xf0a1, 0xf0a2, PDF_CMAP_RANGE, 6951 },
	{ 0xf0a3, 0xf0a3, PDF_CMAP_SINGLE, 9599 },
	{ 0xf0a4, 0xf0ae, PDF_CMAP_RANGE, 6954 },
	{ 0xf0af, 0xf0d9, PDF_CMAP_RANGE, 9600 },
	{ 0xf0da, 0xf0db, PDF_CMAP_RANGE, 7008 },
	{ 0xf0dc, 0xf0dd, PDF_CMAP_RANGE, 9643 },
	{ 0xf0de, 0xf0df, PDF_CMAP_TABLE, 585 },
	{ 0xf0e0, 0xf0e8, PDF_CMAP_RANGE, 7014 },
	{ 0xf0e9, 0xf0e9, PDF_CMAP_SINGLE, 9646 },
	{ 0xf0ea, 0xf0eb, PDF_CMAP_RANGE, 7024 },
	{ 0xf0ec, 0xf0ec, PDF_CMAP_SINGLE, 9647 },
	{ 0xf0ed, 0xf0ee, PDF_CMAP_RANGE, 7027 },
	{ 0xf0ef, 0xf0ef, PDF_CMAP_SINGLE, 9648 },
	{ 0xf0f0, 0xf0f6, PDF_CMAP_RANGE, 7030 },
	{ 0xf0f7, 0xf0f9, PDF_CMAP_TABLE, 587 },
	{ 0xf0fa, 0xf0fb, PDF_CMAP_RANGE, 7040 },
	{ 0xf0fc, 0xf0fc, PDF_CMAP_SINGLE, 9651 },
	{ 0xf0fd, 0xf0fe, PDF_CMAP_RANGE, 7043 },
	{ 0xf1a1, 0xf1a7, PDF_CMAP_RANGE, 7045 },
	{ 0xf1a8, 0xf1a8, PDF_CMAP_SINGLE, 9652 },
	{ 0xf1a9, 0xf1aa, PDF_CMAP_RANGE, 7053 },
	{ 0xf1ab, 0xf1ab, PDF_CMAP_SINGLE, 9653 },
	{ 0xf1ac, 0xf1ad, PDF_CMAP_RANGE, 7056 },
	{ 0xf1ae, 0xf1ae, PDF_CMAP_SINGLE, 9654 },
	{ 0xf1af, 0xf1b1, PDF_CMAP_RANGE, 7059 },
	{ 0xf1b2, 0xf1b2, PDF_CMAP_SINGLE, 9655 },
	{ 0xf1b3, 0xf1bb, PDF_CMAP_RANGE, 7063 },
	{ 0xf1bc, 0xf1bc, PDF_CMAP_SINGLE, 9656 },
	{ 0xf1bd, 0xf1bf, PDF_CMAP_RANGE, 7073 },
	{ 0xf1c0, 0xf1c0, PDF_CMAP_SINGLE, 9657 },
	{ 0xf1c1, 0xf1c8, PDF_CMAP_RANGE, 7077 },
	{ 0xf1c9, 0xf1c9, PDF_CMAP_SINGLE, 9658 },
	{ 0xf1ca, 0xf1cc, PDF_CMAP_RANGE, 7086 },
	{ 0xf1cd, 0xf1ce, PDF_CMAP_TABLE, 590 },
	{ 0xf1cf, 0xf1d0, PDF_CMAP_RANGE, 9660 },
	{ 0xf1d1, 0xf1d9, PDF_CMAP_RANGE, 7093 },
	{ 0xf1da, 0xf1dc, PDF_CMAP_TABLE, 592 },
	{ 0xf1dd, 0xf1e3, PDF_CMAP_RANGE, 7105 },
	{ 0xf1e4, 0xf1e4, PDF_CMAP_SINGLE, 9664 },
	{ 0xf1e5, 0xf1eb, PDF_CMAP_RANGE, 7113 },
	{ 0xf1ec, 0xf1ec, PDF_CMAP_SINGLE, 9665 },
	{ 0xf1ed, 0xf1ee, PDF_CMAP_RANGE, 7121 },
	{ 0xf1ef, 0xf1ef, PDF_CMAP_SINGLE, 9666 },
	{ 0xf1f0, 0xf1f6, PDF_CMAP_RANGE, 7124 },
	{ 0xf1f7, 0xf1f9, PDF_CMAP_TABLE, 595 },
	{ 0xf1fa, 0xf1fb, PDF_CMAP_RANGE, 7134 },
	{ 0xf1fc, 0xf1fe, PDF_CMAP_RANGE, 9669 },
	{ 0xf2a1, 0xf2ad, PDF_CMAP_RANGE, 9672 },
	{ 0xf2ae, 0xf2b0, PDF_CMAP_RANGE, 7152 },
	{ 0xf2b1, 0xf2b2, PDF_CMAP_RANGE, 9685 },
	{ 0xf2b3, 0xf2b8, PDF_CMAP_RANGE, 7157 },
	{ 0xf2b9, 0xf2b9, PDF_CMAP_SINGLE, 9687 },
	{ 0xf2ba, 0xf2c2, PDF_CMAP_RANGE, 7164 },
	{ 0xf2c3, 0xf2c3, PDF_CMAP_SINGLE, 9688 },
	{ 0xf2c4, 0xf2c8, PDF_CMAP_RANGE, 7174 },
	{ 0xf2c9, 0xf2c9, PDF_CMAP_SINGLE, 9689 },
	{ 0xf2ca, 0xf2cb, PDF_CMAP_RANGE, 7180 },
	{ 0xf2cc, 0xf2cd, PDF_CMAP_RANGE, 9690 },
	{ 0xf2ce, 0xf2cf, PDF_CMAP_TABLE, 598 },
	{ 0xf2d0, 0xf2d2, PDF_CMAP_RANGE, 7186 },
	{ 0xf2d3, 0xf2d3, PDF_CMAP_SINGLE, 9693 },
	{ 0xf2d4, 0xf2e4, PDF_CMAP_RANGE, 7190 },
	{ 0xf2e5, 0xf2e5, PDF_CMAP_SINGLE, 9694 },
	{ 0xf2e6, 0xf2ed, PDF_CMAP_RANGE, 7208 },
	{ 0xf2ee, 0xf2ee, PDF_CMAP_SINGLE, 9695 },
	{ 0xf2ef, 0xf2f6, PDF_CMAP_RANGE, 7217 },
	{ 0xf2f7, 0xf2f7, PDF_CMAP_SINGLE, 9696 },
	{ 0xf2f8, 0xf2fc, PDF_CMAP_RANGE, 7226 },
	{ 0xf2fd, 0xf2fe, PDF_CMAP_TABLE, 600 },
	{ 0xf3a1, 0xf3be, PDF_CMAP_RANGE, 7233 },
	{ 0xf3bf, 0xf3bf, PDF_CMAP_SINGLE, 9698 },
	{ 0xf3c0, 0xf3c5, PDF_CMAP_RANGE, 7264 },
	{ 0xf3c6, 0xf3c8, PDF_CMAP_TABLE, 602 },
	{ 0xf3c9, 0xf3d5, PDF_CMAP_RANGE, 7273 },
	{ 0xf3d6, 0xf3d6, PDF_CMAP_SINGLE, 9701 },
	{ 0xf3d7, 0xf3d8, PDF_CMAP_RANGE, 7287 },
	{ 0xf3d9, 0xf3d9, PDF_CMAP_SINGLE, 9702 },
	{ 0xf3da, 0xf3e4, PDF_CMAP_RANGE, 7290 },
	{ 0xf3e5, 0xf3e6, PDF_CMAP_RANGE, 9703 },
	{ 0xf3e7, 0xf3e9, PDF_CMAP_RANGE, 7303 },
	{ 0xf3ea, 0xf3ec, PDF_CMAP_TABLE, 605 },
	{ 0xf3ed, 0xf3ee, PDF_CMAP_RANGE, 7309 },
	{ 0xf3ef, 0xf3f1, PDF_CMAP_TABLE, 608 },
	{ 0xf3f2, 0xf3fc, PDF_CMAP_RANGE, 7314 },
	{ 0xf3fd, 0xf3fe, PDF_CMAP_TABLE, 611 },
	{ 0xf4a1, 0xf4a4, PDF_CMAP_RANGE, 7327 },
	{ 0xf4a5, 0xf4a5, PDF_CMAP_SINGLE, 9710 },
	{ 0xf4a6, 0xf4ae, PDF_CMAP_RANGE, 7332 },
	{ 0xf4af, 0xf4af, PDF_CMAP_SINGLE, 9711 },
	{ 0xf4b0, 0xf4b4, PDF_CMAP_RANGE, 7342 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 9712 },
	{ 0xf4b6, 0xf4c0, PDF_CMAP_RANGE, 7348 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 9713 },
	{ 0xf4c2, 0xf4c6, PDF_CMAP_RANGE, 7360 },
	{ 0xf4c7, 0xf4c7, PDF_CMAP_SINGLE, 9714 },
	{ 0xf4c8, 0xf4ce, PDF_CMAP_RANGE, 7366 },
	{ 0xf4cf, 0xf4d0, PDF_CMAP_RANGE, 9715 },
	{ 0xf4d1, 0xf4d5, PDF_CMAP_RANGE, 7375 },
	{ 0xf4d6, 0xf4d6, PDF_CMAP_SINGLE, 9717 },
	{ 0xf4d7, 0xf4e9, PDF_CMAP_RANGE, 7381 },
	{ 0xf4ea, 0xf4ea, PDF_CMAP_SINGLE, 9718 },
	{ 0xf4eb, 0xf4ee, PDF_CMAP_RANGE, 7401 },
	{ 0xf4ef, 0xf4ef, PDF_CMAP_SINGLE, 9719 },
	{ 0xf4f0, 0xf4f4, PDF_CMAP_RANGE, 7406 },
	{ 0xf4f5, 0xf4f5, PDF_CMAP_SINGLE, 9720 },
	{ 0xf4f6, 0xf4fe, PDF_CMAP_RANGE, 7412 },
	{ 0xf5a1, 0xf5a5, PDF_CMAP_RANGE, 7421 },
	{ 0xf5a6, 0xf5a7, PDF_CMAP_RANGE, 9721 },
	{ 0xf5a8, 0xf5b9, PDF_CMAP_RANGE, 7428 },
	{ 0xf5ba, 0xf5bb, PDF_CMAP_RANGE, 9723 },
	{ 0xf5bc, 0xf5c3, PDF_CMAP_RANGE, 7448 },
	{ 0xf5c4, 0xf5c4, PDF_CMAP_SINGLE, 9725 },
	{ 0xf5c5, 0xf5c7, PDF_CMAP_RANGE, 7457 },
	{ 0xf5c8, 0xf5c8, PDF_CMAP_SINGLE, 9726 },
	{ 0xf5c9, 0xf5cd, PDF_CMAP_RANGE, 7461 },
	{ 0xf5ce, 0xf5cf, PDF_CMAP_RANGE, 9727 },
	{ 0xf5d0, 0xf5d0, PDF_CMAP_SINGLE, 7468 },
	{ 0xf5d1, 0xf5d2, PDF_CMAP_RANGE, 9729 },
	{ 0xf5d3, 0xf5d8, PDF_CMAP_RANGE, 7471 },
	{ 0xf5d9, 0xf5d9, PDF_CMAP_SINGLE, 9731 },
	{ 0xf5da, 0xf5db, PDF_CMAP_RANGE, 7478 },
	{ 0xf5dc, 0xf5dc, PDF_CMAP_SINGLE, 9732 },
	{ 0xf5dd, 0xf5e5, PDF_CMAP_RANGE, 7481 },
	{ 0xf5e6, 0xf5e7, PDF_CMAP_RANGE, 9733 },
	{ 0xf5e8, 0xf5ee, PDF_CMAP_RANGE, 7492 },
	{ 0xf5ef, 0xf5ef, PDF_CMAP_SINGLE, 9735 },
	{ 0xf5f0, 0xf5f1, PDF_CMAP_RANGE, 7500 },
	{ 0xf5f2, 0xf5f2, PDF_CMAP_SINGLE, 9736 },
	{ 0xf5f3, 0xf5fb, PDF_CMAP_RANGE, 7503 },
	{ 0xf5fc, 0xf5fc, PDF_CMAP_SINGLE, 9737 },
	{ 0xf5fd, 0xf5fe, PDF_CMAP_RANGE, 7513 },
	{ 0xf6a1, 0xf6a2, PDF_CMAP_RANGE, 7515 },
	{ 0xf6a3, 0xf6a3, PDF_CMAP_SINGLE, 9738 },
	{ 0xf6a4, 0xf6a5, PDF_CMAP_RANGE, 7518 },
	{ 0xf6a6, 0xf6a8, PDF_CMAP_TABLE, 613 },
	{ 0xf6a9, 0xf6aa, PDF_CMAP_RANGE, 7523 },
	{ 0xf6ab, 0xf6ab, PDF_CMAP_SINGLE, 9741 },
	{ 0xf6ac, 0xf6af, PDF_CMAP_RANGE, 7526 },
	{ 0xf6b0, 0xf6b0, PDF_CMAP_SINGLE, 9742 },
	{ 0xf6b1, 0xf6b2, PDF_CMAP_RANGE, 7531 },
	{ 0xf6b3, 0xf6be, PDF_CMAP_RANGE, 9743 },
	{ 0xf6bf, 0xf6c4, PDF_CMAP_RANGE, 7545 },
	{ 0xf6c5, 0xf6c9, PDF_CMAP_TABLE, 616 },
	{ 0xf6ca, 0xf6ce, PDF_CMAP_RANGE, 7556 },
	{ 0xf6cf, 0xf6fe, PDF_CMAP_RANGE, 9758 },
	{ 0xf7a1, 0xf7af, PDF_CMAP_RANGE, 9806 },
	{ 0xf7b0, 0xf7b1, PDF_CMAP_RANGE, 7624 },
	{ 0xf7b2, 0xf7b3, PDF_CMAP_RANGE, 9821 },
	{ 0xf7b4, 0xf7b5, PDF_CMAP_TABLE, 621 },
	{ 0xf7b6, 0xf7bc, PDF_CMAP_RANGE, 7630 },
	{ 0xf7bd, 0xf7bd, PDF_CMAP_SINGLE, 9824 },
	{ 0xf7be, 0xf7c2, PDF_CMAP_RANGE, 7638 },
	{ 0xf7c3, 0xf7c4, PDF_CMAP_TABLE, 623 },
	{ 0xf7c5, 0xf7c6, PDF_CMAP_RANGE, 9826 },
	{ 0xf7c7, 0xf7c9, PDF_CMAP_RANGE, 7647 },
	{ 0xf7ca, 0xf7cb, PDF_CMAP_RANGE, 9828 },
	{ 0xf7cc, 0xf7ce, PDF_CMAP_RANGE, 7652 },
	{ 0xf7cf, 0xf7d0, PDF_CMAP_RANGE, 9830 },
	{ 0xf7d1, 0xf7dd, PDF_CMAP_RANGE, 7657 },
	{ 0xf7de, 0xf7de, PDF_CMAP_SINGLE, 9832 },
	{ 0xf7df, 0xf7e0, PDF_CMAP_RANGE, 7671 },
	{ 0xf7e1, 0xf7e1, PDF_CMAP_SINGLE, 2745 },
	{ 0xf7e2, 0xf7f1, PDF_CMAP_RANGE, 7674 },
	{ 0xf7f2, 0xf7f2, PDF_CMAP_SINGLE, 9833 },
	{ 0xf7f3, 0xf7f4, PDF_CMAP_RANGE, 7691 },
	{ 0xf7f5, 0xf7f5, PDF_CMAP_SINGLE, 9834 },
	{ 0xf7f6, 0xf7fe, PDF_CMAP_RANGE, 7694 },
	{ 0xf8a1, 0xf8a6, PDF_CMAP_RANGE, 9835 },
	{ 0xf8a7, 0xf8a9, PDF_CMAP_TABLE, 625 },
	{ 0xf8aa, 0xf8ab, PDF_CMAP_RANGE, 9841 },
	{ 0xf8ac, 0xf8af, PDF_CMAP_TABLE, 628 },
	{ 0xf8b0, 0xf8b1, PDF_CMAP_RANGE, 9844 },
	{ 0xf8b2, 0xf8b5, PDF_CMAP_TABLE, 632 },
	{ 0xf8b6, 0xf8b7, PDF_CMAP_RANGE, 9847 },
	{ 0xf8b8, 0xf8b8, PDF_CMAP_SINGLE, 1852 },
	{ 0xf8b9, 0xf8ba, PDF_CMAP_RANGE, 9849 },
	{ 0xf8bb, 0xf8bb, PDF_CMAP_SINGLE, 1987 },
	{ 0xf8bc, 0xf8bf, PDF_CMAP_RANGE, 9851 },
	{ 0xf8c0, 0xf8c3, PDF_CMAP_TABLE, 636 },
	{ 0xf8c4, 0xf8c5, PDF_CMAP_RANGE, 9856 },
	{ 0xf8c6, 0xf8c6, PDF_CMAP_SINGLE, 2316 },
	{ 0xf8c7, 0xf8c8, PDF_CMAP_RANGE, 9858 },
	{ 0xf8c9, 0xf8ca, PDF_CMAP_TABLE, 640 },
	{ 0xf8cb, 0xf8cc, PDF_CMAP_RANGE, 2458 },
	{ 0xf8cd, 0xf8d0, PDF_CMAP_TABLE, 642 },
	{ 0xf8d1, 0xf8db, PDF_CMAP_RANGE, 9863 },
	{ 0xf8dc, 0xf8e0, PDF_CMAP_TABLE, 646 },
	{ 0xf8e1, 0xf8e5, PDF_CMAP_RANGE, 9874 },
	{ 0xf8e6, 0xf8eb, PDF_CMAP_TABLE, 651 },
	{ 0xf8ec, 0xf8ee, PDF_CMAP_RANGE, 9880 },
	{ 0xf8ef, 0xf8f1, PDF_CMAP_TABLE, 657 },
	{ 0xf8f2, 0xf8f7, PDF_CMAP_RANGE, 9884 },
	{ 0xf8f8, 0xf8f9, PDF_CMAP_TABLE, 660 },
	{ 0xf8fa, 0xf8fb, PDF_CMAP_RANGE, 9890 },
	{ 0xf8fc, 0xf8fe, PDF_CMAP_TABLE, 662 },
	{ 0xf9a1, 0xf9a3, PDF_CMAP_TABLE, 665 },
	{ 0xf9a4, 0xf9a5, PDF_CMAP_RANGE, 9894 },
	{ 0xf9a6, 0xf9a9, PDF_CMAP_TABLE, 668 },
	{ 0xe7db, 0xe7e0, PDF_CMAP_RANGE, 6163 },
	{ 0xe7e1, 0xe7e2, PDF_CMAP_RANGE, 9307 },
	{ 0xe7e3, 0xe7e6, PDF_CMAP_RANGE, 6171 },
	{ 0xe7e7, 0xe7e7, PDF_CMAP_SINGLE, 9309 },
	{ 0xe7e8, 0xe7ee, PDF_CMAP_RANGE, 6176 },
	{ 0xe7ef, 0xe7ef, PDF_CMAP_SINGLE, 9310 },
	{ 0xe7f0, 0xe7f3, PDF_CMAP_RANGE, 6184 },
	{ 0xe7f4, 0xe7f6, PDF_CMAP_RANGE, 9311 },
	{ 0xe7f7, 0xe7fe, PDF_CMAP_RANGE, 6191 },
	{ 0xe8a1, 0xe8a7, PDF_CMAP_RANGE, 6199 },
	{ 0xe8a8, 0xe8a8, PDF_CMAP_SINGLE, 9314 },
	{ 0xe8a9, 0xe8ab, PDF_CMAP_RANGE, 6207 },
	{ 0xe8ac, 0xe8ac, PDF_CMAP_SINGLE, 9315 },
	{ 0xe8ad, 0xe8b5, PDF_CMAP_RANGE, 6211 },
	{ 0xe8b6, 0xe8b6, PDF_CMAP_SINGLE, 9316 },
	{ 0xe8b7, 0xe8b7, PDF_CMAP_SINGLE, 6221 },
	{ 0xe8b8, 0xe8ba, PDF_CMAP_RANGE, 9317 },
	{ 0xe8bb, 0xe8be, PDF_CMAP_RANGE, 6225 },
	{ 0xe8bf, 0xe8c0, PDF_CMAP_RANGE, 9320 },
	{ 0xe8c1, 0xe8c4, PDF_CMAP_RANGE, 6231 },
	{ 0xe8c5, 0xe8c5, PDF_CMAP_SINGLE, 9322 },
	{ 0xe8c6, 0xe8c6, PDF_CMAP_SINGLE, 6236 },
	{ 0xe8c7, 0xe8c9, PDF_CMAP_RANGE, 9323 },
	{ 0xe8ca, 0xe8cd, PDF_CMAP_RANGE, 6240 },
	{ 0xe8ce, 0xe8ce, PDF_CMAP_SINGLE, 9326 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 6245 },
	{ 0xe8d0, 0xe8d0, PDF_CMAP_SINGLE, 9327 },
	{ 0xe8d1, 0xe8d2, PDF_CMAP_RANGE, 6247 },
	{ 0xe8d3, 0xe8d3, PDF_CMAP_SINGLE, 9328 },
	{ 0xe8d4, 0xe8dc, PDF_CMAP_RANGE, 6250 },
	{ 0xe8dd, 0xe8dd, PDF_CMAP_SINGLE, 9329 },
	{ 0xe8de, 0xe8de, PDF_CMAP_SINGLE, 6260 },
	{ 0xe8df, 0xe8df, PDF_CMAP_SINGLE, 9330 },
	{ 0xe8e0, 0xe8e1, PDF_CMAP_RANGE, 6262 },
	{ 0xe8e2, 0xe8e3, PDF_CMAP_RANGE, 9331 },
	{ 0xe8e4, 0xe8e4, PDF_CMAP_SINGLE, 6266 },
	{ 0xe8e5, 0xe8e5, PDF_CMAP_SINGLE, 9333 },
	{ 0xe8e6, 0xe8e6, PDF_CMAP_SINGLE, 6268 },
	{ 0xe8e7, 0xe8e7, PDF_CMAP_SINGLE, 9334 },
	{ 0xe8e8, 0xe8ea, PDF_CMAP_RANGE, 6270 },
	{ 0xe8eb, 0xe8eb, PDF_CMAP_SINGLE, 9335 },
	{ 0xe8ec, 0xe8ec, PDF_CMAP_SINGLE, 6274 },
	{ 0xe8ed, 0xe8ed, PDF_CMAP_SINGLE, 9336 },
	{ 0xe8ee, 0xe8ee, PDF_CMAP_SINGLE, 6276 },
	{ 0xe8ef, 0xe8ef, PDF_CMAP_SINGLE, 9337 },
	{ 0xe8f0, 0xe8f8, PDF_CMAP_RANGE, 6278 },
	{ 0xe8f9, 0xe8f9, PDF_CMAP_SINGLE, 9338 },
	{ 0xe8fa, 0xe8fb, PDF_CMAP_RANGE, 6288 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 9339 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 6292 },
	{ 0xe9a1, 0xe9a1, PDF_CMAP_SINGLE, 9341 },
	{ 0xe9a2, 0xe9ac, PDF_CMAP_RANGE, 6294 },
	{ 0xe9ad, 0xe9ad, PDF_CMAP_SINGLE, 9342 },
	{ 0xe9ae, 0xe9b3, PDF_CMAP_RANGE, 6306 },
	{ 0xe9b4, 0xe9b5, PDF_CMAP_RANGE, 9343 },
	{ 0xe9b6, 0xe9b6, PDF_CMAP_SINGLE, 6314 },
	{ 0xe9b7, 0xe9b7, PDF_CMAP_SINGLE, 9345 },
	{ 0xe9b8, 0xe9c3, PDF_CMAP_RANGE, 6316 },
	{ 0xe9c4, 0xe9c4, PDF_CMAP_SINGLE, 9346 },
	{ 0xe9c5, 0xe9c5, PDF_CMAP_SINGLE, 6329 },
	{ 0xe9c6, 0xe9c6, PDF_CMAP_SINGLE, 9347 },
	{ 0xe9c7, 0xe9c8, PDF_CMAP_RANGE, 6331 },
	{ 0xe9c9, 0xe9c9, PDF_CMAP_SINGLE, 9348 },
	{ 0xe9ca, 0xe9d5, PDF_CMAP_RANGE, 6334 },
	{ 0xe9d6, 0xe9d6, PDF_CMAP_SINGLE, 9349 },
	{ 0xe9d7, 0xe9d9, PDF_CMAP_RANGE, 6347 },
	{ 0xe9da, 0xe9da, PDF_CMAP_SINGLE, 9350 },
	{ 0xe9db, 0xe9e3, PDF_CMAP_RANGE, 6351 },
	{ 0xe9e4, 0xe9e4, PDF_CMAP_SINGLE, 9351 },
	{ 0xe9e5, 0xe9e5, PDF_CMAP_SINGLE, 6361 },
	{ 0xe9e6, 0xe9e7, PDF_CMAP_RANGE, 9352 },
	{ 0xe9e8, 0xe9e8, PDF_CMAP_SINGLE, 6364 },
	{ 0xe9e9, 0xe9e9, PDF_CMAP_SINGLE, 9354 },
	{ 0xe9ea, 0xe9ea, PDF_CMAP_SINGLE, 6366 },
	{ 0xe9eb, 0xe9eb, PDF_CMAP_SINGLE, 9355 },
	{ 0xe9ec, 0xe9ec, PDF_CMAP_SINGLE, 6368 },
	{ 0xe9ed, 0xe9fe, PDF_CMAP_RANGE, 9356 },
	{ 0xeaa1, 0xeaa5, PDF_CMAP_RANGE, 9374 },
	{ 0xeaa6, 0xeaa6, PDF_CMAP_SINGLE, 6392 },
	{ 0xeaa7, 0xeaa8, PDF_CMAP_RANGE, 9379 },
	{ 0xeaa9, 0xeab0, PDF_CMAP_RANGE, 6395 },
	{ 0xeab1, 0xeab1, PDF_CMAP_SINGLE, 9381 },
	{ 0xeab2, 0xeabb, PDF_CMAP_RANGE, 6404 },
	{ 0xeabc, 0xeabc, PDF_CMAP_SINGLE, 9382 },
	{ 0xeabd, 0xeac9, PDF_CMAP_RANGE, 6415 },
	{ 0xeaca, 0xeaca, PDF_CMAP_SINGLE, 9383 },
	{ 0xeacb, 0xeacc, PDF_CMAP_RANGE, 6429 },
	{ 0xeacd, 0xeacd, PDF_CMAP_SINGLE, 9384 },
	{ 0xeace, 0xead2, PDF_CMAP_RANGE, 6432 },
	{ 0xead3, 0xead3, PDF_CMAP_SINGLE, 9385 },
	{ 0xead4, 0xead9, PDF_CMAP_RANGE, 6438 },
	{ 0xeada, 0xeaef, PDF_CMAP_RANGE, 9386 },
	{ 0xeaf0, 0xeafe, PDF_CMAP_RANGE, 6466 },
	{ 0xeba1, 0xeba6, PDF_CMAP_RANGE, 6481 },
	{ 0xeba7, 0xeba7, PDF_CMAP_SINGLE, 9408 },
	{ 0xeba8, 0xeba9, PDF_CMAP_RANGE, 6488 },
	{ 0xebaa, 0xebaa, PDF_CMAP_SINGLE, 9409 },
	{ 0xebab, 0xebb1, PDF_CMAP_RANGE, 6491 },
	{ 0xebb2, 0xebb2, PDF_CMAP_SINGLE, 9410 },
	{ 0xebb3, 0xebb8, PDF_CMAP_RANGE, 6499 },
	{ 0xebb9, 0xebb9, PDF_CMAP_SINGLE, 9411 },
	{ 0xebba, 0xebc9, PDF_CMAP_RANGE, 6506 },
	{ 0xebca, 0xebcb, PDF_CMAP_RANGE, 9412 },
	{ 0xebcc, 0xebcc, PDF_CMAP_SINGLE, 6524 },
	{ 0xebcd, 0xebcd, PDF_CMAP_SINGLE, 9414 },
	{ 0xebce, 0xebd5, PDF_CMAP_RANGE, 6526 },
	{ 0xebd6, 0xebd6, PDF_CMAP_SINGLE, 9415 },
	{ 0xebd7, 0xebd9, PDF_CMAP_RANGE, 6535 },
	{ 0xebda, 0xebda, PDF_CMAP_SINGLE, 9416 },
	{ 0xebdb, 0xebe0, PDF_CMAP_RANGE, 6539 },
	{ 0xebe1, 0xebe1, PDF_CMAP_SINGLE, 9417 },
	{ 0xebe2, 0xebf6, PDF_CMAP_RANGE, 6546 },
	{ 0xebf7, 0xebf7, PDF_CMAP_SINGLE, 9418 },
	{ 0xebf8, 0xebfe, PDF_CMAP_RANGE, 6568 },
	{ 0xeca1, 0xeca2, PDF_CMAP_RANGE, 6575 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 9419 },
	{ 0xeca4, 0xeca8, PDF_CMAP_RANGE, 6578 },
	{ 0xeca9, 0xecae, PDF_CMAP_RANGE, 9420 },
	{ 0xecaf, 0xecb0, PDF_CMAP_RANGE, 6589 },
	{ 0xecb1, 0xecb1, PDF_CMAP_SINGLE, 9426 },
	{ 0xecb2, 0xecb3, PDF_CMAP_RANGE, 6592 },
	{ 0xecb4, 0xecb5, PDF_CMAP_RANGE, 9427 },
	{ 0xecb6, 0xecbd, PDF_CMAP_RANGE, 6596 },
	{ 0xecbe, 0xecbf, PDF_CMAP_RANGE, 9429 },
	{ 0xecc0, 0xecc0, PDF_CMAP_SINGLE, 6606 },
	{ 0xecc1, 0xecc1, PDF_CMAP_SINGLE, 9431 },
	{ 0xecc2, 0xecc6, PDF_CMAP_RANGE, 6608 },
	{ 0xecc7, 0xecc7, PDF_CMAP_SINGLE, 9432 },
	{ 0xecc8, 0xecca, PDF_CMAP_RANGE, 6614 },
	{ 0xeccb, 0xeccb, PDF_CMAP_SINGLE, 9433 },
	{ 0xeccc, 0xece1, PDF_CMAP_RANGE, 6618 },
	{ 0xece2, 0xece2, PDF_CMAP_SINGLE, 9434 },
	{ 0xece3, 0xecf1, PDF_CMAP_RANGE, 6641 },
	{ 0xecf2, 0xecf2, PDF_CMAP_SINGLE, 9435 },
	{ 0xecf3, 0xecf4, PDF_CMAP_RANGE, 6657 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 9436 },
	{ 0xecf6, 0xecf7, PDF_CMAP_RANGE, 6660 },
	{ 0xecf8, 0xecf8, PDF_CMAP_SINGLE, 9437 },
	{ 0xecf9, 0xecfe, PDF_CMAP_RANGE, 6663 },
	{ 0xeda1, 0xeda1, PDF_CMAP_SINGLE, 9438 },
	{ 0xeda2, 0xeda7, PDF_CMAP_RANGE, 6670 },
	{ 0xeda8, 0xeda8, PDF_CMAP_SINGLE, 9439 },
	{ 0xeda9, 0xedae, PDF_CMAP_RANGE, 6677 },
	{ 0xedaf, 0xedb0, PDF_CMAP_RANGE, 9440 },
	{ 0xedb1, 0xedb3, PDF_CMAP_RANGE, 6685 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 9442 },
	{ 0xedb5, 0xedb5, PDF_CMAP_SINGLE, 6689 },
	{ 0xedb6, 0xedb6, PDF_CMAP_SINGLE, 9443 },
	{ 0xedb7, 0xedb7, PDF_CMAP_SINGLE, 6691 },
	{ 0xedb8, 0xedb8, PDF_CMAP_SINGLE, 9444 },
	{ 0xedb9, 0xedb9, PDF_CMAP_SINGLE, 6693 },
	{ 0xedba, 0xedba, PDF_CMAP_SINGLE, 9445 },
	{ 0xedbb, 0xedbe, PDF_CMAP_RANGE, 6695 },
	{ 0xedbf, 0xedbf, PDF_CMAP_SINGLE, 9446 },
	{ 0xedc0, 0xedc1, PDF_CMAP_RANGE, 6700 },
	{ 0xedc2, 0xedc3, PDF_CMAP_RANGE, 9447 },
	{ 0xedc4, 0xedcb, PDF_CMAP_RANGE, 6704 },
	{ 0xedcc, 0xedcd, PDF_CMAP_RANGE, 9449 },
	{ 0xedce, 0xedd2, PDF_CMAP_RANGE, 6714 },
	{ 0xedd3, 0xedd3, PDF_CMAP_SINGLE, 9451 },
	{ 0xedd4, 0xedd6, PDF_CMAP_RANGE, 6720 },
	{ 0xedd7, 0xedd7, PDF_CMAP_SINGLE, 9452 },
	{ 0xedd8, 0xede7, PDF_CMAP_RANGE, 6724 },
	{ 0xede8, 0xede8, PDF_CMAP_SINGLE, 9453 },
	{ 0xede9, 0xeded, PDF_CMAP_RANGE, 6741 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 9454 },
	{ 0xedef, 0xedf8, PDF_CMAP_RANGE, 6747 },
	{ 0xedf9, 0xedfa, PDF_CMAP_RANGE, 9455 },
	{ 0xedfb, 0xedfe, PDF_CMAP_RANGE, 6759 },
	{ 0xeea1, 0xeebb, PDF_CMAP_RANGE, 6763 },
	{ 0xeebc, 0xeebc, PDF_CMAP_SINGLE, 9457 },
	{ 0xeebd, 0xeebe, PDF_CMAP_RANGE, 6791 },
	{ 0xeebf, 0xeebf, PDF_CMAP_SINGLE, 9458 },
	{ 0xeec0, 0xeec3, PDF_CMAP_RANGE, 6794 },
	{ 0xeec4, 0xeefe, PDF_CMAP_RANGE, 9459 },
	{ 0xefa1, 0xeff1, PDF_CMAP_RANGE, 9518 },
	{ 0xeff2, 0xeffe, PDF_CMAP_RANGE, 6938 },
	{ 0xf0a1, 0xf0a2, PDF_CMAP_RANGE, 6951 },
	{ 0xf0a3, 0xf0a3, PDF_CMAP_SINGLE, 9599 },
	{ 0xf0a4, 0xf0ae, PDF_CMAP_RANGE, 6954 },
	{ 0xf0af, 0xf0d9, PDF_CMAP_RANGE, 9600 },
	{ 0xf0da, 0xf0db, PDF_CMAP_RANGE, 7008 },
	{ 0xf0dc, 0xf0dd, PDF_CMAP_RANGE, 9643 },
	{ 0xf0de, 0xf0de, PDF_CMAP_SINGLE, 7012 },
	{ 0xf0df, 0xf0df, PDF_CMAP_SINGLE, 9645 },
	{ 0xf0e0, 0xf0e8, PDF_CMAP_RANGE, 7014 },
	{ 0xf0e9, 0xf0e9, PDF_CMAP_SINGLE, 9646 },
	{ 0xf0ea, 0xf0eb, PDF_CMAP_RANGE, 7024 },
	{ 0xf0ec, 0xf0ec, PDF_CMAP_SINGLE, 9647 },
	{ 0xf0ed, 0xf0ee, PDF_CMAP_RANGE, 7027 },
	{ 0xf0ef, 0xf0ef, PDF_CMAP_SINGLE, 9648 },
	{ 0xf0f0, 0xf0f6, PDF_CMAP_RANGE, 7030 },
	{ 0xf0f7, 0xf0f7, PDF_CMAP_SINGLE, 9649 },
	{ 0xf0f8, 0xf0f8, PDF_CMAP_SINGLE, 7038 },
	{ 0xf0f9, 0xf0f9, PDF_CMAP_SINGLE, 9650 },
	{ 0xf0fa, 0xf0fb, PDF_CMAP_RANGE, 7040 },
	{ 0xf0fc, 0xf0fc, PDF_CMAP_SINGLE, 9651 },
	{ 0xf0fd, 0xf0fe, PDF_CMAP_RANGE, 7043 },
	{ 0xf1a1, 0xf1a7, PDF_CMAP_RANGE, 7045 },
	{ 0xf1a8, 0xf1a8, PDF_CMAP_SINGLE, 9652 },
	{ 0xf1a9, 0xf1aa, PDF_CMAP_RANGE, 7053 },
	{ 0xf1ab, 0xf1ab, PDF_CMAP_SINGLE, 9653 },
	{ 0xf1ac, 0xf1ad, PDF_CMAP_RANGE, 7056 },
	{ 0xf1ae, 0xf1ae, PDF_CMAP_SINGLE, 9654 },
	{ 0xf1af, 0xf1b1, PDF_CMAP_RANGE, 7059 },
	{ 0xf1b2, 0xf1b2, PDF_CMAP_SINGLE, 9655 },
	{ 0xf1b3, 0xf1bb, PDF_CMAP_RANGE, 7063 },
	{ 0xf1bc, 0xf1bc, PDF_CMAP_SINGLE, 9656 },
	{ 0xf1bd, 0xf1bf, PDF_CMAP_RANGE, 7073 },
	{ 0xf1c0, 0xf1c0, PDF_CMAP_SINGLE, 9657 },
	{ 0xf1c1, 0xf1c8, PDF_CMAP_RANGE, 7077 },
	{ 0xf1c9, 0xf1c9, PDF_CMAP_SINGLE, 9658 },
	{ 0xf1ca, 0xf1cc, PDF_CMAP_RANGE, 7086 },
	{ 0xf1cd, 0xf1cd, PDF_CMAP_SINGLE, 9659 },
	{ 0xf1ce, 0xf1ce, PDF_CMAP_SINGLE, 7090 },
	{ 0xf1cf, 0xf1d0, PDF_CMAP_RANGE, 9660 },
	{ 0xf1d1, 0xf1d9, PDF_CMAP_RANGE, 7093 },
	{ 0xf1da, 0xf1da, PDF_CMAP_SINGLE, 9662 },
	{ 0xf1db, 0xf1db, PDF_CMAP_SINGLE, 7103 },
	{ 0xf1dc, 0xf1dc, PDF_CMAP_SINGLE, 9663 },
	{ 0xf1dd, 0xf1e3, PDF_CMAP_RANGE, 7105 },
	{ 0xf1e4, 0xf1e4, PDF_CMAP_SINGLE, 9664 },
	{ 0xf1e5, 0xf1eb, PDF_CMAP_RANGE, 7113 },
	{ 0xf1ec, 0xf1ec, PDF_CMAP_SINGLE, 9665 },
	{ 0xf1ed, 0xf1ee, PDF_CMAP_RANGE, 7121 },
	{ 0xf1ef, 0xf1ef, PDF_CMAP_SINGLE, 9666 },
	{ 0xf1f0, 0xf1f6, PDF_CMAP_RANGE, 7124 },
	{ 0xf1f7, 0xf1f7, PDF_CMAP_SINGLE, 9667 },
	{ 0xf1f8, 0xf1f8, PDF_CMAP_SINGLE, 7132 },
	{ 0xf1f9, 0xf1f9, PDF_CMAP_SINGLE, 9668 },
	{ 0xf1fa, 0xf1fb, PDF_CMAP_RANGE, 7134 },
	{ 0xf1fc, 0xf1fe, PDF_CMAP_RANGE, 9669 },
	{ 0xf2a1, 0xf2ad, PDF_CMAP_RANGE, 9672 },
	{ 0xf2ae, 0xf2b0, PDF_CMAP_RANGE, 7152 },
	{ 0xf2b1, 0xf2b2, PDF_CMAP_RANGE, 9685 },
	{ 0xf2b3, 0xf2b8, PDF_CMAP_RANGE, 7157 },
	{ 0xf2b9, 0xf2b9, PDF_CMAP_SINGLE, 9687 },
	{ 0xf2ba, 0xf2c2, PDF_CMAP_RANGE, 7164 },
	{ 0xf2c3, 0xf2c3, PDF_CMAP_SINGLE, 9688 },
	{ 0xf2c4, 0xf2c8, PDF_CMAP_RANGE, 7174 },
	{ 0xf2c9, 0xf2c9, PDF_CMAP_SINGLE, 9689 },
	{ 0xf2ca, 0xf2cb, PDF_CMAP_RANGE, 7180 },
	{ 0xf2cc, 0xf2cd, PDF_CMAP_RANGE, 9690 },
	{ 0xf2ce, 0xf2ce, PDF_CMAP_SINGLE, 7184 },
	{ 0xf2cf, 0xf2cf, PDF_CMAP_SINGLE, 9692 },
	{ 0xf2d0, 0xf2d2, PDF_CMAP_RANGE, 7186 },
	{ 0xf2d3, 0xf2d3, PDF_CMAP_SINGLE, 9693 },
	{ 0xf2d4, 0xf2e4, PDF_CMAP_RANGE, 7190 },
	{ 0xf2e5, 0xf2e5, PDF_CMAP_SINGLE, 9694 },
	{ 0xf2e6, 0xf2ed, PDF_CMAP_RANGE, 7208 },
	{ 0xf2ee, 0xf2ee, PDF_CMAP_SINGLE, 9695 },
	{ 0xf2ef, 0xf2f6, PDF_CMAP_RANGE, 7217 },
	{ 0xf2f7, 0xf2f7, PDF_CMAP_SINGLE, 9696 },
	{ 0xf2f8, 0xf2fc, PDF_CMAP_RANGE, 7226 },
	{ 0xf2fd, 0xf2fd, PDF_CMAP_SINGLE, 9697 },
	{ 0xf2fe, 0xf2fe, PDF_CMAP_SINGLE, 7232 },
	{ 0xf3a1, 0xf3be, PDF_CMAP_RANGE, 7233 },
	{ 0xf3bf, 0xf3bf, PDF_CMAP_SINGLE, 9698 },
	{ 0xf3c0, 0xf3c5, PDF_CMAP_RANGE, 7264 },
	{ 0xf3c6, 0xf3c6, PDF_CMAP_SINGLE, 9699 },
	{ 0xf3c7, 0xf3c7, PDF_CMAP_SINGLE, 7271 },
	{ 0xf3c8, 0xf3c8, PDF_CMAP_SINGLE, 9700 },
	{ 0xf3c9, 0xf3d5, PDF_CMAP_RANGE, 7273 },
	{ 0xf3d6, 0xf3d6, PDF_CMAP_SINGLE, 9701 },
	{ 0xf3d7, 0xf3d8, PDF_CMAP_RANGE, 7287 },
	{ 0xf3d9, 0xf3d9, PDF_CMAP_SINGLE, 9702 },
	{ 0xf3da, 0xf3e4, PDF_CMAP_RANGE, 7290 },
	{ 0xf3e5, 0xf3e6, PDF_CMAP_RANGE, 9703 },
	{ 0xf3e7, 0xf3e9, PDF_CMAP_RANGE, 7303 },
	{ 0xf3ea, 0xf3ea, PDF_CMAP_SINGLE, 9705 },
	{ 0xf3eb, 0xf3eb, PDF_CMAP_SINGLE, 7307 },
	{ 0xf3ec, 0xf3ec, PDF_CMAP_SINGLE, 9706 },
	{ 0xf3ed, 0xf3ee, PDF_CMAP_RANGE, 7309 },
	{ 0xf3ef, 0xf3ef, PDF_CMAP_SINGLE, 9707 },
	{ 0xf3f0, 0xf3f0, PDF_CMAP_SINGLE, 7312 },
	{ 0xf3f1, 0xf3f1, PDF_CMAP_SINGLE, 9708 },
	{ 0xf3f2, 0xf3fc, PDF_CMAP_RANGE, 7314 },
	{ 0xf3fd, 0xf3fd, PDF_CMAP_SINGLE, 9709 },
	{ 0xf3fe, 0xf3fe, PDF_CMAP_SINGLE, 7326 },
	{ 0xf4a1, 0xf4a4, PDF_CMAP_RANGE, 7327 },
	{ 0xf4a5, 0xf4a5, PDF_CMAP_SINGLE, 9710 },
	{ 0xf4a6, 0xf4ae, PDF_CMAP_RANGE, 7332 },
	{ 0xf4af, 0xf4af, PDF_CMAP_SINGLE, 9711 },
	{ 0xf4b0, 0xf4b4, PDF_CMAP_RANGE, 7342 },
	{ 0xf4b5, 0xf4b5, PDF_CMAP_SINGLE, 9712 },
	{ 0xf4b6, 0xf4c0, PDF_CMAP_RANGE, 7348 },
	{ 0xf4c1, 0xf4c1, PDF_CMAP_SINGLE, 9713 },
	{ 0xf4c2, 0xf4c6, PDF_CMAP_RANGE, 7360 },
	{ 0xf4c7, 0xf4c7, PDF_CMAP_SINGLE, 9714 },
	{ 0xf4c8, 0xf4ce, PDF_CMAP_RANGE, 7366 },
	{ 0xf4cf, 0xf4d0, PDF_CMAP_RANGE, 9715 },
	{ 0xf4d1, 0xf4d5, PDF_CMAP_RANGE, 7375 },
	{ 0xf4d6, 0xf4d6, PDF_CMAP_SINGLE, 9717 },
	{ 0xf4d7, 0xf4e9, PDF_CMAP_RANGE, 7381 },
	{ 0xf4ea, 0xf4ea, PDF_CMAP_SINGLE, 9718 },
	{ 0xf4eb, 0xf4ee, PDF_CMAP_RANGE, 7401 },
	{ 0xf4ef, 0xf4ef, PDF_CMAP_SINGLE, 9719 },
	{ 0xf4f0, 0xf4f4, PDF_CMAP_RANGE, 7406 },
	{ 0xf4f5, 0xf4f5, PDF_CMAP_SINGLE, 9720 },
	{ 0xf4f6, 0xf4fe, PDF_CMAP_RANGE, 7412 },
	{ 0xf5a1, 0xf5a5, PDF_CMAP_RANGE, 7421 },
	{ 0xf5a6, 0xf5a7, PDF_CMAP_RANGE, 9721 },
	{ 0xf5a8, 0xf5b9, PDF_CMAP_RANGE, 7428 },
	{ 0xf5ba, 0xf5bb, PDF_CMAP_RANGE, 9723 },
	{ 0xf5bc, 0xf5c3, PDF_CMAP_RANGE, 7448 },
	{ 0xf5c4, 0xf5c4, PDF_CMAP_SINGLE, 9725 },
	{ 0xf5c5, 0xf5c7, PDF_CMAP_RANGE, 7457 },
	{ 0xf5c8, 0xf5c8, PDF_CMAP_SINGLE, 9726 },
	{ 0xf5c9, 0xf5cd, PDF_CMAP_RANGE, 7461 },
	{ 0xf5ce, 0xf5cf, PDF_CMAP_RANGE, 9727 },
	{ 0xf5d0, 0xf5d0, PDF_CMAP_SINGLE, 7468 },
	{ 0xf5d1, 0xf5d2, PDF_CMAP_RANGE, 9729 },
	{ 0xf5d3, 0xf5d8, PDF_CMAP_RANGE, 7471 },
	{ 0xf5d9, 0xf5d9, PDF_CMAP_SINGLE, 9731 },
	{ 0xf5da, 0xf5db, PDF_CMAP_RANGE, 7478 },
	{ 0xf5dc, 0xf5dc, PDF_CMAP_SINGLE, 9732 },
	{ 0xf5dd, 0xf5e5, PDF_CMAP_RANGE, 7481 },
	{ 0xf5e6, 0xf5e7, PDF_CMAP_RANGE, 9733 },
	{ 0xf5e8, 0xf5ee, PDF_CMAP_RANGE, 7492 },
	{ 0xf5ef, 0xf5ef, PDF_CMAP_SINGLE, 9735 },
	{ 0xf5f0, 0xf5f1, PDF_CMAP_RANGE, 7500 },
	{ 0xf5f2, 0xf5f2, PDF_CMAP_SINGLE, 9736 },
	{ 0xf5f3, 0xf5fb, PDF_CMAP_RANGE, 7503 },
	{ 0xf5fc, 0xf5fc, PDF_CMAP_SINGLE, 9737 },
	{ 0xf5fd, 0xf5fe, PDF_CMAP_RANGE, 7513 },
	{ 0xf6a1, 0xf6a2, PDF_CMAP_RANGE, 7515 },
	{ 0xf6a3, 0xf6a3, PDF_CMAP_SINGLE, 9738 },
	{ 0xf6a4, 0xf6a5, PDF_CMAP_RANGE, 7518 },
	{ 0xf6a6, 0xf6a6, PDF_CMAP_SINGLE, 9739 },
	{ 0xf6a7, 0xf6a7, PDF_CMAP_SINGLE, 7521 },
	{ 0xf6a8, 0xf6a8, PDF_CMAP_SINGLE, 9740 },
	{ 0xf6a9, 0xf6aa, PDF_CMAP_RANGE, 7523 },
	{ 0xf6ab, 0xf6ab, PDF_CMAP_SINGLE, 9741 },
	{ 0xf6ac, 0xf6af, PDF_CMAP_RANGE, 7526 },
	{ 0xf6b0, 0xf6b0, PDF_CMAP_SINGLE, 9742 },
	{ 0xf6b1, 0xf6b2, PDF_CMAP_RANGE, 7531 },
	{ 0xf6b3, 0xf6be, PDF_CMAP_RANGE, 9743 },
	{ 0xf6bf, 0xf6c4, PDF_CMAP_RANGE, 7545 },
	{ 0xf6c5, 0xf6c5, PDF_CMAP_SINGLE, 9755 },
	{ 0xf6c6, 0xf6c6, PDF_CMAP_SINGLE, 7552 },
	{ 0xf6c7, 0xf6c7, PDF_CMAP_SINGLE, 9756 },
	{ 0xf6c8, 0xf6c8, PDF_CMAP_SINGLE, 7554 },
	{ 0xf6c9, 0xf6c9, PDF_CMAP_SINGLE, 9757 },
	{ 0xf6ca, 0xf6ce, PDF_CMAP_RANGE, 7556 },
	{ 0xf6cf, 0xf6fe, PDF_CMAP_RANGE, 9758 },
	{ 0xf7a1, 0xf7af, PDF_CMAP_RANGE, 9806 },
	{ 0xf7b0, 0xf7b1, PDF_CMAP_RANGE, 7624 },
	{ 0xf7b2, 0xf7b3, PDF_CMAP_RANGE, 9821 },
	{ 0xf7b4, 0xf7b4, PDF_CMAP_SINGLE, 7628 },
	{ 0xf7b5, 0xf7b5, PDF_CMAP_SINGLE, 9823 },
	{ 0xf7b6, 0xf7bc, PDF_CMAP_RANGE, 7630 },
	{ 0xf7bd, 0xf7bd, PDF_CMAP_SINGLE, 9824 },
	{ 0xf7be, 0xf7c2, PDF_CMAP_RANGE, 7638 },
	{ 0xf7c3, 0xf7c3, PDF_CMAP_SINGLE, 9825 },
	{ 0xf7c4, 0xf7c4, PDF_CMAP_SINGLE, 7644 },
	{ 0xf7c5, 0xf7c6, PDF_CMAP_RANGE, 9826 },
	{ 0xf7c7, 0xf7c9, PDF_CMAP_RANGE, 7647 },
	{ 0xf7ca, 0xf7cb, PDF_CMAP_RANGE, 9828 },
	{ 0xf7cc, 0xf7ce, PDF_CMAP_RANGE, 7652 },
	{ 0xf7cf, 0xf7d0, PDF_CMAP_RANGE, 9830 },
	{ 0xf7d1, 0xf7dd, PDF_CMAP_RANGE, 7657 },
	{ 0xf7de, 0xf7de, PDF_CMAP_SINGLE, 9832 },
	{ 0xf7df, 0xf7e0, PDF_CMAP_RANGE, 7671 },
	{ 0xf7e1, 0xf7e1, PDF_CMAP_SINGLE, 2745 },
	{ 0xf7e2, 0xf7f1, PDF_CMAP_RANGE, 7674 },
	{ 0xf7f2, 0xf7f2, PDF_CMAP_SINGLE, 9833 },
	{ 0xf7f3, 0xf7f4, PDF_CMAP_RANGE, 7691 },
	{ 0xf7f5, 0xf7f5, PDF_CMAP_SINGLE, 9834 },
	{ 0xf7f6, 0xf7fe, PDF_CMAP_RANGE, 7694 },
	{ 0xf8a1, 0xf8a6, PDF_CMAP_RANGE, 9835 },
	{ 0xf8a7, 0xf8a7, PDF_CMAP_SINGLE, 1228 },
	{ 0xf8a8, 0xf8a8, PDF_CMAP_SINGLE, 1290 },
	{ 0xf8a9, 0xf8a9, PDF_CMAP_SINGLE, 1304 },
	{ 0xf8aa, 0xf8ab, PDF_CMAP_RANGE, 9841 },
	{ 0xf8ac, 0xf8ac, PDF_CMAP_SINGLE, 1428 },
	{ 0xf8ad, 0xf8ad, PDF_CMAP_SINGLE, 1486 },
	{ 0xf8ae, 0xf8ae, PDF_CMAP_SINGLE, 9843 },
	{ 0xf8af, 0xf8af, PDF_CMAP_SINGLE, 1526 },
	{ 0xf8b0, 0xf8b1, PDF_CMAP_RANGE, 9844 },
	{ 0xf8b2, 0xf8b2, PDF_CMAP_SINGLE, 1619 },
	{ 0xf8b3, 0xf8b3, PDF_CMAP_SINGLE, 1662 },
	{ 0xf8b4, 0xf8b4, PDF_CMAP_SINGLE, 9846 },
	{ 0xf8b5, 0xf8b5, PDF_CMAP_SINGLE, 1732 },
	{ 0xf8b6, 0xf8b7, PDF_CMAP_RANGE, 9847 },
	{ 0xf8b8, 0xf8b8, PDF_CMAP_SINGLE, 1852 },
	{ 0xf8b9, 0xf8ba, PDF_CMAP_RANGE, 9849 },
	{ 0xf8bb, 0xf8bb, PDF_CMAP_SINGLE, 1987 },
	{ 0xf8bc, 0xf8bf, PDF_CMAP_RANGE, 9851 },
	{ 0xf8c0, 0xf8c0, PDF_CMAP_SINGLE, 2091 },
	{ 0xf8c1, 0xf8c1, PDF_CMAP_SINGLE, 9855 },
	{ 0xf8c2, 0xf8c2, PDF_CMAP_SINGLE, 2126 },
	{ 0xf8c3, 0xf8c3, PDF_CMAP_SINGLE, 2153 },
	{ 0xf8c4, 0xf8c5, PDF_CMAP_RANGE, 9856 },
	{ 0xf8c6, 0xf8c6, PDF_CMAP_SINGLE, 2316 },
	{ 0xf8c7, 0xf8c8, PDF_CMAP_RANGE, 9858 },
	{ 0xf8c9, 0xf8c9, PDF_CMAP_SINGLE, 2417 },
	{ 0xf8ca, 0xf8ca, PDF_CMAP_SINGLE, 9860 },
	{ 0xf8cb, 0xf8cc, PDF_CMAP_RANGE, 2458 },
	{ 0xf8cd, 0xf8cd, PDF_CMAP_SINGLE, 9861 },
	{ 0xf8ce, 0xf8ce, PDF_CMAP_SINGLE, 2522 },
	{ 0xf8cf, 0xf8cf, PDF_CMAP_SINGLE, 9862 },
	{ 0xf8d0, 0xf8d0, PDF_CMAP_SINGLE, 2554 },
	{ 0xf8d1, 0xf8db, PDF_CMAP_RANGE, 9863 },
	{ 0xf8dc, 0xf8dc, PDF_CMAP_SINGLE, 3034 },
	{ 0xf8dd, 0xf8dd, PDF_CMAP_SINGLE, 3037 },
	{ 0xf8de, 0xf8de, PDF_CMAP_SINGLE, 3050 },
	{ 0xf8df, 0xf8df, PDF_CMAP_SINGLE, 3052 },
	{ 0xf8e0, 0xf8e0, PDF_CMAP_SINGLE, 3058 },
	{ 0xf8e1, 0xf8e5, PDF_CMAP_RANGE, 9874 },
	{ 0xf8e6, 0xf8e6, PDF_CMAP_SINGLE, 3218 },
	{ 0xf8e7, 0xf8e7, PDF_CMAP_SINGLE, 3354 },
	{ 0xf8e8, 0xf8e8, PDF_CMAP_SINGLE, 3468 },
	{ 0xf8e9, 0xf8e9, PDF_CMAP_SINGLE, 3518 },
	{ 0xf8ea, 0xf8ea, PDF_CMAP_SINGLE, 9879 },
	{ 0xf8eb, 0xf8eb, PDF_CMAP_SINGLE, 3579 },
	{ 0xf8ec, 0xf8ee, PDF_CMAP_RANGE, 9880 },
	{ 0xf8ef, 0xf8ef, PDF_CMAP_SINGLE, 3696 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 9883 },
	{ 0xf8f1, 0xf8f1, PDF_CMAP_SINGLE, 3747 },
	{ 0xf8f2, 0xf8f7, PDF_CMAP_RANGE, 9884 },
	{ 0xf8f8, 0xf8f8, PDF_CMAP_SINGLE, 4157 },
	{ 0xf8f9, 0xf8f9, PDF_CMAP_SINGLE, 4313 },
	{ 0xf8fa, 0xf8fb, PDF_CMAP_RANGE, 9890 },
	{ 0xf8fc, 0xf8fc, PDF_CMAP_SINGLE, 4347 },
	{ 0xf8fd, 0xf8fd, PDF_CMAP_SINGLE, 4361 },
	{ 0xf8fe, 0xf8fe, PDF_CMAP_SINGLE, 9892 },
	{ 0xf9a1, 0xf9a1, PDF_CMAP_SINGLE, 4513 },
	{ 0xf9a2, 0xf9a2, PDF_CMAP_SINGLE, 9893 },
	{ 0xf9a3, 0xf9a3, PDF_CMAP_SINGLE, 4538 },
	{ 0xf9a4, 0xf9a5, PDF_CMAP_RANGE, 9894 },
	{ 0xf9a6, 0xf9a6, PDF_CMAP_SINGLE, 4565 },
	{ 0xf9a7, 0xf9a7, PDF_CMAP_SINGLE, 9896 },
	{ 0xf9a8, 0xf9a8, PDF_CMAP_SINGLE, 4605 },
	{ 0xf9a9, 0xf9a9, PDF_CMAP_SINGLE, 4633 },
};

static const unsigned short pdf_cmap_GBTpc_EUC_H_table[672] =
{
	7717,948,7718,7723,991,7724,7725,998,
	7726,1017,7730,7732,1027,7733,1040,7736,
	7738,1052,1082,7748,1106,7756,7763,1127,
	7780,1190,7781,1192,1229,7801,7806,1256,
	7807,1258,7808,7809,1267,7810,7812,1279,
	7813,1281,7814,7815,1289,1292,7818,7821,
	1301,7822,1303,7823,7839,1361,7840,7843,
	1374,7844,7845,1393,7846,7849,1413,7854,
	1422,1431,7862,7879,1503,1509,7882,1511,
	7883,1513,7884,7892,1540,7893,7894,1549,
	1556,7899,7900,1561,7901,7904,1580,1583,
	7907,7909,1593,1621,7921,7926,1645,7927,
	1663,7934,1675,7941,7948,1718,7949,1720,
	1731,7955,7964,1765,7965,7966,1775,1778,
	7969,7972,1803,7973,1805,7974,7981,1845,
	7982,1847,7983,1849,7984,1851,7985,1862,
	7991,7992,1867,7996,1874,7997,7998,1879,
	8010,1978,8011,1988,8016,8017,1993,8023,
	2023,8041,2067,2068,8042,2077,8047,2079,
	8048,8053,2101,8054,8057,2110,2122,8064,
	8065,2127,8066,2129,8072,2140,2180,8100,
	8124,2255,8128,2264,8129,8142,2325,8143,
	2327,8144,8146,2341,8154,2387,8155,8163,
	2423,8164,2425,8165,8166,2433,8167,2435,
	8168,2437,8171,2445,8172,8173,2453,8174,
	8196,2496,8197,2498,8198,8201,2517,8242,
	2611,8245,2621,8246,2661,8270,2663,8271,
	2680,8281,2710,8303,2712,8315,7673,8320,
	2770,8321,8333,2819,8338,2869,8349,2904,
	2912,8355,8364,2935,2939,8368,2941,8369,
	8380,3038,8381,8391,3087,8392,8395,3101,
	8396,3105,3111,8401,3132,8411,3134,8412,
	8413,3139,3155,8420,8432,3193,8433,3195,
	8438,3217,8439,3234,8444,8445,3241,8446,
	3284,8456,3286,8457,8459,3295,3324,8469,
	8470,3331,8471,3333,8472,8474,3350,8475,
	8476,3355,8477,3357,8478,3374,8483,3376,
	3390,8490,3392,8505,3446,8506,3469,8512,
	8514,3477,8516,3489,8517,8518,3497,3500,
	8521,8523,3517,8530,3529,8531,8534,3546,
	3564,8542,8554,3602,8555,8556,3616,8557,
	3618,8558,8560,3630,8561,8562,3636,8563,
	8576,3703,8577,8578,3716,3735,8586,3765,
	8591,3777,8598,3798,8603,3800,8604,3818,
	8612,3820,8615,3835,8616,8617,3841,8618,
	8621,3868,8622,3878,8626,8627,3883,8631,
	3891,8632,3900,8637,3910,8643,3912,8644,
	8648,3928,8649,3930,8650,8668,4023,8669,
	4044,8676,4065,8685,4067,8701,4118,4121,
	8704,4136,8709,8713,4155,8716,4165,8717,
	8718,4171,8719,4173,8720,8730,4215,8731,
	4242,8746,4252,8751,8753,4267,8754,8767,
	4322,8768,8769,4328,4346,8779,8780,4352,
	8783,4360,4363,8786,4365,8798,4392,8799,
	8800,4402,8810,4427,8811,8818,4445,8819,
	8821,4455,8822,4461,8825,4480,8829,4482,
	8830,8844,4539,8845,4541,8851,4563,8856,
	4575,8857,8858,4581,4616,8873,4618,4626,
	8880,4628,8893,4677,8897,4734,8898,4744,
	8903,8904,4752,8910,4802,8911,4804,8922,
	4857,8923,8987,4970,8988,8994,5004,8995,
	8996,5024,8997,9000,5043,9001,9026,5166,
	9046,5257,9047,9052,5306,5356,9063,5382,
	9067,9076,5417,9077,9079,5426,9080,9096,
	5552,9097,9100,5568,9101,1962,9107,9110,
	5627,9111,9112,5634,5681,9134,5703,9138,
	9142,5723,5729,9147,9192,5883,9193,9201,
	5956,9202,9204,5997,9205,6023,9211,9316,
	6221,9322,6236,9326,6245,9327,9329,6260,
	9330,6266,9333,6268,9334,9335,6274,9336,
	6276,9337,6314,9345,9346,6329,9347,9351,
	6361,6364,9354,6366,9355,6368,6524,9414,
	6606,9431,9442,6689,9443,6691,9444,6693,
	9445,7012,9645,9649,7038,9650,9659,7090,
	9662,7103,9663,9667,7132,9668,7184,9692,
	9697,7232,9699,7271,9700,9705,7307,9706,
	9707,7312,9708,9709,7326,9739,7521,9740,
	9755,7552,9756,7554,9757,7628,9823,9825,
	7644,1228,1290,1304,1428,1486,9843,1526,
	1619,1662,9846,1732,2091,9855,2126,2153,
	2417,9860,9861,2522,9862,2554,3034,3037,
	3050,3052,3058,3218,3354,3468,3518,9879,
	3579,3696,9883,3747,4157,4313,4347,4361,
	9892,4513,9893,4538,4565,9896,4605,4633,
};

pdf_cmap pdf_cmap_GBTpc_EUC_H =
{
	-1, "GBTpc-EUC-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa1a1, 0xfcfe },
		{ 1, 0x00fd, 0x00ff },
	},
	2284, 2284, (pdf_range*) pdf_cmap_GBTpc_EUC_H_ranges,
	672, 672, (unsigned short*) pdf_cmap_GBTpc_EUC_H_table,
};

/* GBTpc-EUC-V */

static const pdf_range pdf_cmap_GBTpc_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_TABLE, 0 },
	{ 0xa1aa, 0xa1aa, PDF_CMAP_SINGLE, 598 },
	{ 0xa1ab, 0xa1ac, PDF_CMAP_RANGE, 7704 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 599 },
	{ 0xa1b2, 0xa1bf, PDF_CMAP_RANGE, 582 },
	{ 0xa1fe, 0xa1fe, PDF_CMAP_SINGLE, 7706 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 578 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 580 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 573 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 7707 },
	{ 0xa3ba, 0xa3bb, PDF_CMAP_RANGE, 576 },
	{ 0xa3bd, 0xa3bd, PDF_CMAP_SINGLE, 7708 },
	{ 0xa3bf, 0xa3bf, PDF_CMAP_SINGLE, 579 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 7709 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 7710 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 600 },
	{ 0xa3fb, 0xa3fb, PDF_CMAP_SINGLE, 596 },
	{ 0xa3fd, 0xa3fe, PDF_CMAP_TABLE, 2 },
	{ 0xa3fd, 0xa3fd, PDF_CMAP_SINGLE, 597 },
	{ 0xa3fe, 0xa3fe, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_GBTpc_EUC_V_table[4] =
{
	575,574,597,7711,
};

pdf_cmap pdf_cmap_GBTpc_EUC_V =
{
	-1, "GBTpc-EUC-V", "GBTpc-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	20, 20, (pdf_range*) pdf_cmap_GBTpc_EUC_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_GBTpc_EUC_V_table,
};

/* UniGB-UCS2-H */

static const pdf_range pdf_cmap_UniGB_UCS2_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a4, 0x00a5, PDF_CMAP_TABLE, 0 },
	{ 0x00a7, 0x00a8, PDF_CMAP_TABLE, 2 },
	{ 0x00b0, 0x00b1, PDF_CMAP_TABLE, 4 },
	{ 0x00d7, 0x00d7, PDF_CMAP_SINGLE, 128 },
	{ 0x00e0, 0x00e1, PDF_CMAP_TABLE, 6 },
	{ 0x00e8, 0x00ea, PDF_CMAP_TABLE, 8 },
	{ 0x00ec, 0x00ed, PDF_CMAP_TABLE, 11 },
	{ 0x00f2, 0x00f3, PDF_CMAP_TABLE, 13 },
	{ 0x00f7, 0x00f7, PDF_CMAP_SINGLE, 129 },
	{ 0x00f9, 0x00fa, PDF_CMAP_TABLE, 15 },
	{ 0x00fc, 0x00fc, PDF_CMAP_SINGLE, 692 },
	{ 0x0101, 0x0101, PDF_CMAP_SINGLE, 668 },
	{ 0x0113, 0x0113, PDF_CMAP_SINGLE, 672 },
	{ 0x011b, 0x011b, PDF_CMAP_SINGLE, 674 },
	{ 0x012b, 0x012b, PDF_CMAP_SINGLE, 676 },
	{ 0x014d, 0x014d, PDF_CMAP_SINGLE, 680 },
	{ 0x016b, 0x016b, PDF_CMAP_SINGLE, 684 },
	{ 0x01ce, 0x01ce, PDF_CMAP_SINGLE, 670 },
	{ 0x01d0, 0x01d0, PDF_CMAP_SINGLE, 678 },
	{ 0x01d2, 0x01d2, PDF_CMAP_SINGLE, 682 },
	{ 0x01d4, 0x01d4, PDF_CMAP_SINGLE, 686 },
	{ 0x01d6, 0x01d6, PDF_CMAP_SINGLE, 688 },
	{ 0x01d8, 0x01d8, PDF_CMAP_SINGLE, 689 },
	{ 0x01da, 0x01da, PDF_CMAP_SINGLE, 690 },
	{ 0x01dc, 0x01dc, PDF_CMAP_SINGLE, 691 },
	{ 0x01f9, 0x01f9, PDF_CMAP_SINGLE, 698 },
	{ 0x02c7, 0x02c7, PDF_CMAP_SINGLE, 101 },
	{ 0x02c9, 0x02c9, PDF_CMAP_SINGLE, 100 },
	{ 0x02ca, 0x02cb, PDF_CMAP_RANGE, 9907 },
	{ 0x02d9, 0x02d9, PDF_CMAP_SINGLE, 9909 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 525 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 542 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 549 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 566 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 608 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 602 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 609 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 642 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 641 },
	{ 0x1e3f, 0x1e3f, PDF_CMAP_SINGLE, 695 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 10018 },
	{ 0x2013, 0x2016, PDF_CMAP_TABLE, 17 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 109 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 111 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 21 },
	{ 0x2030, 0x2030, PDF_CMAP_SINGLE, 170 },
	{ 0x2032, 0x2033, PDF_CMAP_RANGE, 163 },
	{ 0x2035, 0x2035, PDF_CMAP_SINGLE, 9913 },
	{ 0x203b, 0x203b, PDF_CMAP_SINGLE, 184 },
	{ 0x20ac, 0x20ac, PDF_CMAP_SINGLE, 22353 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 165 },
	{ 0x2105, 0x2105, PDF_CMAP_SINGLE, 9914 },
	{ 0x2109, 0x2109, PDF_CMAP_SINGLE, 9915 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 172 },
	{ 0x2121, 0x2121, PDF_CMAP_SINGLE, 10016 },
	{ 0x2160, 0x216b, PDF_CMAP_RANGE, 250 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 9897 },
	{ 0x2190, 0x2191, PDF_CMAP_RANGE, 186 },
	{ 0x2192, 0x2193, PDF_CMAP_TABLE, 23 },
	{ 0x2196, 0x2199, PDF_CMAP_RANGE, 9916 },
	{ 0x2208, 0x2208, PDF_CMAP_SINGLE, 137 },
	{ 0x220f, 0x220f, PDF_CMAP_SINGLE, 134 },
	{ 0x2211, 0x2211, PDF_CMAP_SINGLE, 133 },
	{ 0x2215, 0x2215, PDF_CMAP_SINGLE, 9920 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 139 },
	{ 0x221d, 0x2220, PDF_CMAP_TABLE, 25 },
	{ 0x2223, 0x2223, PDF_CMAP_SINGLE, 9922 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 141 },
	{ 0x2227, 0x2228, PDF_CMAP_RANGE, 131 },
	{ 0x2229, 0x222b, PDF_CMAP_TABLE, 29 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 146 },
	{ 0x2234, 0x2237, PDF_CMAP_TABLE, 32 },
	{ 0x223d, 0x223d, PDF_CMAP_SINGLE, 150 },
	{ 0x2248, 0x2248, PDF_CMAP_SINGLE, 149 },
	{ 0x224c, 0x224c, PDF_CMAP_SINGLE, 148 },
	{ 0x2252, 0x2252, PDF_CMAP_SINGLE, 9923 },
	{ 0x2260, 0x2261, PDF_CMAP_TABLE, 36 },
	{ 0x2264, 0x2265, PDF_CMAP_RANGE, 155 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 9924 },
	{ 0x226e, 0x226f, PDF_CMAP_RANGE, 153 },
	{ 0x2295, 0x2295, PDF_CMAP_SINGLE, 9988 },
	{ 0x2299, 0x2299, PDF_CMAP_SINGLE, 144 },
	{ 0x22a5, 0x22a5, PDF_CMAP_SINGLE, 140 },
	{ 0x22bf, 0x22bf, PDF_CMAP_SINGLE, 9926 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 108 },
	{ 0x2312, 0x2312, PDF_CMAP_SINGLE, 143 },
	{ 0x2460, 0x2469, PDF_CMAP_RANGE, 230 },
	{ 0x2474, 0x2487, PDF_CMAP_RANGE, 210 },
	{ 0x2488, 0x249b, PDF_CMAP_RANGE, 190 },
	{ 0x2500, 0x254b, PDF_CMAP_RANGE, 738 },
	{ 0x2550, 0x2573, PDF_CMAP_RANGE, 9927 },
	{ 0x2581, 0x258f, PDF_CMAP_RANGE, 9963 },
	{ 0x2593, 0x2595, PDF_CMAP_RANGE, 9978 },
	{ 0x25a0, 0x25a1, PDF_CMAP_TABLE, 38 },
	{ 0x25b2, 0x25b3, PDF_CMAP_TABLE, 40 },
	{ 0x25bc, 0x25bd, PDF_CMAP_RANGE, 9981 },
	{ 0x25c6, 0x25c7, PDF_CMAP_TABLE, 42 },
	{ 0x25cb, 0x25cb, PDF_CMAP_SINGLE, 175 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 44 },
	{ 0x25e2, 0x25e5, PDF_CMAP_RANGE, 9983 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 46 },
	{ 0x2609, 0x2609, PDF_CMAP_SINGLE, 9987 },
	{ 0x2640, 0x2640, PDF_CMAP_SINGLE, 161 },
	{ 0x2642, 0x2642, PDF_CMAP_SINGLE, 160 },
	{ 0x2e80, 0x2e81, PDF_CMAP_TABLE, 48 },
	{ 0x2e82, 0x2e83, PDF_CMAP_RANGE, 22429 },
	{ 0x2e84, 0x2e84, PDF_CMAP_SINGLE, 22051 },
	{ 0x2e85, 0x2e87, PDF_CMAP_RANGE, 22431 },
	{ 0x2e88, 0x2e88, PDF_CMAP_SINGLE, 22054 },
	{ 0x2e89, 0x2e8a, PDF_CMAP_RANGE, 22434 },
	{ 0x2e8b, 0x2e8c, PDF_CMAP_TABLE, 50 },
	{ 0x2e8d, 0x2e96, PDF_CMAP_RANGE, 22436 },
	{ 0x2e97, 0x2e97, PDF_CMAP_SINGLE, 22061 },
	{ 0x2e98, 0x2e99, PDF_CMAP_RANGE, 22446 },
	{ 0x2e9b, 0x2ea6, PDF_CMAP_RANGE, 22448 },
	{ 0x2ea7, 0x2ea7, PDF_CMAP_SINGLE, 22074 },
	{ 0x2ea8, 0x2ea9, PDF_CMAP_RANGE, 22460 },
	{ 0x2eaa, 0x2eaa, PDF_CMAP_SINGLE, 22077 },
	{ 0x2eab, 0x2ead, PDF_CMAP_RANGE, 22462 },
	{ 0x2eae, 0x2eae, PDF_CMAP_SINGLE, 22080 },
	{ 0x2eaf, 0x2eb2, PDF_CMAP_RANGE, 22465 },
	{ 0x2eb3, 0x2eb3, PDF_CMAP_SINGLE, 22082 },
	{ 0x2eb4, 0x2eb5, PDF_CMAP_RANGE, 22469 },
	{ 0x2eb6, 0x2eb7, PDF_CMAP_RANGE, 22083 },
	{ 0x2eb8, 0x2eba, PDF_CMAP_RANGE, 22471 },
	{ 0x2ebb, 0x2ebb, PDF_CMAP_SINGLE, 22088 },
	{ 0x2ebc, 0x2ec9, PDF_CMAP_RANGE, 22474 },
	{ 0x2eca, 0x2eca, PDF_CMAP_SINGLE, 22098 },
	{ 0x2ecb, 0x2ef3, PDF_CMAP_RANGE, 22488 },
	{ 0x2f00, 0x2fd5, PDF_CMAP_TABLE, 52 },
	{ 0x2ff0, 0x2ffb, PDF_CMAP_RANGE, 10060 },
	{ 0x3000, 0x3002, PDF_CMAP_RANGE, 96 },
	{ 0x3003, 0x3003, PDF_CMAP_SINGLE, 103 },
	{ 0x3005, 0x3007, PDF_CMAP_TABLE, 266 },
	{ 0x3008, 0x300f, PDF_CMAP_RANGE, 115 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 125 },
	{ 0x3012, 0x3013, PDF_CMAP_TABLE, 269 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 113 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 123 },
	{ 0x301d, 0x301e, PDF_CMAP_RANGE, 9990 },
	{ 0x3021, 0x3029, PDF_CMAP_RANGE, 9992 },
	{ 0x3033, 0x3035, PDF_CMAP_RANGE, 22395 },
	{ 0x3038, 0x303a, PDF_CMAP_RANGE, 22398 },
	{ 0x303e, 0x303f, PDF_CMAP_TABLE, 271 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 356 },
	{ 0x3094, 0x3094, PDF_CMAP_SINGLE, 22375 },
	{ 0x309b, 0x309c, PDF_CMAP_RANGE, 10020 },
	{ 0x309d, 0x309e, PDF_CMAP_RANGE, 10025 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 439 },
	{ 0x30f7, 0x30fa, PDF_CMAP_RANGE, 22390 },
	{ 0x30fb, 0x30fc, PDF_CMAP_TABLE, 273 },
	{ 0x30fd, 0x30fe, PDF_CMAP_RANGE, 10022 },
	{ 0x3105, 0x3129, PDF_CMAP_RANGE, 700 },
	{ 0x312a, 0x312c, PDF_CMAP_RANGE, 22401 },
	{ 0x31a0, 0x31b7, PDF_CMAP_RANGE, 22404 },
	{ 0x3220, 0x3229, PDF_CMAP_RANGE, 240 },
	{ 0x3231, 0x3231, PDF_CMAP_SINGLE, 10017 },
	{ 0x32a3, 0x32a3, PDF_CMAP_SINGLE, 10001 },
	{ 0x338e, 0x338f, PDF_CMAP_RANGE, 10002 },
	{ 0x339c, 0x339e, PDF_CMAP_RANGE, 10004 },
	{ 0x33a1, 0x33a1, PDF_CMAP_SINGLE, 10007 },
	{ 0x33c4, 0x33c4, PDF_CMAP_SINGLE, 10008 },
	{ 0x33ce, 0x33ce, PDF_CMAP_SINGLE, 10009 },
	{ 0x33d1, 0x33d2, PDF_CMAP_RANGE, 10010 },
	{ 0x33d5, 0x33d5, PDF_CMAP_SINGLE, 10012 },
	{ 0x3400, 0x3446, PDF_CMAP_RANGE, 22529 },
	{ 0x3447, 0x3447, PDF_CMAP_SINGLE, 22053 },
	{ 0x3448, 0x3472, PDF_CMAP_RANGE, 22600 },
	{ 0x3473, 0x3473, PDF_CMAP_SINGLE, 22052 },
	{ 0x3474, 0x359d, PDF_CMAP_RANGE, 22643 },
	{ 0x359e, 0x359e, PDF_CMAP_SINGLE, 22057 },
	{ 0x359f, 0x360d, PDF_CMAP_RANGE, 22941 },
	{ 0x360e, 0x360e, PDF_CMAP_SINGLE, 22059 },
	{ 0x360f, 0x3619, PDF_CMAP_RANGE, 23052 },
	{ 0x361a, 0x361a, PDF_CMAP_SINGLE, 22058 },
	{ 0x361b, 0x3917, PDF_CMAP_RANGE, 23063 },
	{ 0x3918, 0x3918, PDF_CMAP_SINGLE, 22063 },
	{ 0x3919, 0x396d, PDF_CMAP_RANGE, 23828 },
	{ 0x396e, 0x396e, PDF_CMAP_SINGLE, 22062 },
	{ 0x396f, 0x39ce, PDF_CMAP_RANGE, 23913 },
	{ 0x39cf, 0x39d0, PDF_CMAP_TABLE, 275 },
	{ 0x39d1, 0x39de, PDF_CMAP_RANGE, 24009 },
	{ 0x39df, 0x39df, PDF_CMAP_SINGLE, 22066 },
	{ 0x39e0, 0x3a72, PDF_CMAP_RANGE, 24023 },
	{ 0x3a73, 0x3a73, PDF_CMAP_SINGLE, 22067 },
	{ 0x3a74, 0x3b4d, PDF_CMAP_RANGE, 24170 },
	{ 0x3b4e, 0x3b4e, PDF_CMAP_SINGLE, 22071 },
	{ 0x3b4f, 0x3c6d, PDF_CMAP_RANGE, 24388 },
	{ 0x3c6e, 0x3c6e, PDF_CMAP_SINGLE, 22072 },
	{ 0x3c6f, 0x3cdf, PDF_CMAP_RANGE, 24675 },
	{ 0x3ce0, 0x3ce0, PDF_CMAP_SINGLE, 22073 },
	{ 0x3ce1, 0x4055, PDF_CMAP_RANGE, 24788 },
	{ 0x4056, 0x4056, PDF_CMAP_SINGLE, 22078 },
	{ 0x4057, 0x415e, PDF_CMAP_RANGE, 25673 },
	{ 0x415f, 0x415f, PDF_CMAP_SINGLE, 22079 },
	{ 0x4160, 0x4336, PDF_CMAP_RANGE, 25937 },
	{ 0x4337, 0x4337, PDF_CMAP_SINGLE, 22081 },
	{ 0x4338, 0x43ab, PDF_CMAP_RANGE, 26408 },
	{ 0x43ac, 0x43ac, PDF_CMAP_SINGLE, 22087 },
	{ 0x43ad, 0x43b0, PDF_CMAP_RANGE, 26524 },
	{ 0x43b1, 0x43b1, PDF_CMAP_SINGLE, 22086 },
	{ 0x43b2, 0x43dc, PDF_CMAP_RANGE, 26528 },
	{ 0x43dd, 0x43dd, PDF_CMAP_SINGLE, 22089 },
	{ 0x43de, 0x44d5, PDF_CMAP_RANGE, 26571 },
	{ 0x44d6, 0x44d6, PDF_CMAP_SINGLE, 22090 },
	{ 0x44d7, 0x464b, PDF_CMAP_RANGE, 26819 },
	{ 0x464c, 0x464c, PDF_CMAP_SINGLE, 22092 },
	{ 0x464d, 0x4660, PDF_CMAP_RANGE, 27192 },
	{ 0x4661, 0x4661, PDF_CMAP_SINGLE, 22091 },
	{ 0x4662, 0x4722, PDF_CMAP_RANGE, 27212 },
	{ 0x4723, 0x4723, PDF_CMAP_SINGLE, 22094 },
	{ 0x4724, 0x4728, PDF_CMAP_RANGE, 27405 },
	{ 0x4729, 0x4729, PDF_CMAP_SINGLE, 22095 },
	{ 0x472a, 0x477b, PDF_CMAP_RANGE, 27410 },
	{ 0x477c, 0x477c, PDF_CMAP_SINGLE, 22096 },
	{ 0x477d, 0x478c, PDF_CMAP_RANGE, 27492 },
	{ 0x478d, 0x478d, PDF_CMAP_SINGLE, 22097 },
	{ 0x478e, 0x4946, PDF_CMAP_RANGE, 27508 },
	{ 0x4947, 0x4947, PDF_CMAP_SINGLE, 22099 },
	{ 0x4948, 0x4979, PDF_CMAP_RANGE, 27949 },
	{ 0x497a, 0x497a, PDF_CMAP_SINGLE, 22100 },
	{ 0x497b, 0x497c, PDF_CMAP_RANGE, 27999 },
	{ 0x497d, 0x497d, PDF_CMAP_SINGLE, 22101 },
	{ 0x497e, 0x4981, PDF_CMAP_RANGE, 28001 },
	{ 0x4982, 0x4983, PDF_CMAP_RANGE, 22102 },
	{ 0x4984, 0x4984, PDF_CMAP_SINGLE, 28005 },
	{ 0x4985, 0x4986, PDF_CMAP_RANGE, 22104 },
	{ 0x4987, 0x499a, PDF_CMAP_RANGE, 28006 },
	{ 0x499b, 0x499b, PDF_CMAP_SINGLE, 22107 },
	{ 0x499c, 0x499e, PDF_CMAP_RANGE, 28026 },
	{ 0x499f, 0x499f, PDF_CMAP_SINGLE, 22106 },
	{ 0x49a0, 0x49b5, PDF_CMAP_RANGE, 28029 },
	{ 0x49b6, 0x49b7, PDF_CMAP_TABLE, 277 },
	{ 0x49b8, 0x4c76, PDF_CMAP_RANGE, 28051 },
	{ 0x4c77, 0x4c77, PDF_CMAP_SINGLE, 22116 },
	{ 0x4c78, 0x4c9e, PDF_CMAP_RANGE, 28754 },
	{ 0x4c9f, 0x4ca1, PDF_CMAP_RANGE, 22113 },
	{ 0x4ca2, 0x4ca3, PDF_CMAP_TABLE, 279 },
	{ 0x4ca4, 0x4d12, PDF_CMAP_RANGE, 28793 },
	{ 0x4d13, 0x4d19, PDF_CMAP_RANGE, 22118 },
	{ 0x4d1a, 0x4dad, PDF_CMAP_RANGE, 28904 },
	{ 0x4dae, 0x4dae, PDF_CMAP_SINGLE, 22125 },
	{ 0x4daf, 0x4db5, PDF_CMAP_RANGE, 29052 },
	{ 0x4e00, 0x4e03, PDF_CMAP_TABLE, 281 },
	{ 0x4e04, 0x4e06, PDF_CMAP_RANGE, 10073 },
	{ 0x4e07, 0x4e1e, PDF_CMAP_TABLE, 285 },
	{ 0x4e1f, 0x4e21, PDF_CMAP_RANGE, 10079 },
	{ 0x4e22, 0x4e2d, PDF_CMAP_TABLE, 309 },
	{ 0x4e2e, 0x4e2f, PDF_CMAP_RANGE, 10085 },
	{ 0x4e30, 0x4e3f, PDF_CMAP_TABLE, 321 },
	{ 0x4e40, 0x4e42, PDF_CMAP_RANGE, 10092 },
	{ 0x4e43, 0x4e59, PDF_CMAP_TABLE, 337 },
	{ 0x4e5a, 0x4e5b, PDF_CMAP_RANGE, 10101 },
	{ 0x4e5c, 0x4e61, PDF_CMAP_TABLE, 360 },
	{ 0x4e62, 0x4e65, PDF_CMAP_RANGE, 10103 },
	{ 0x4e66, 0x4e66, PDF_CMAP_SINGLE, 3456 },
	{ 0x4e67, 0x4e68, PDF_CMAP_RANGE, 10107 },
	{ 0x4e69, 0x4e69, PDF_CMAP_SINGLE, 4726 },
	{ 0x4e6a, 0x4e6f, PDF_CMAP_RANGE, 10109 },
	{ 0x4e70, 0x4e73, PDF_CMAP_TABLE, 366 },
	{ 0x4e74, 0x4e7d, PDF_CMAP_RANGE, 10116 },
	{ 0x4e7e, 0x4e7e, PDF_CMAP_SINGLE, 3113 },
	{ 0x4e7f, 0x4e81, PDF_CMAP_RANGE, 10126 },
	{ 0x4e82, 0x4e82, PDF_CMAP_SINGLE, 8281 },
	{ 0x4e83, 0x4e85, PDF_CMAP_RANGE, 10129 },
	{ 0x4e86, 0x4e95, PDF_CMAP_TABLE, 370 },
	{ 0x4e96, 0x4e97, PDF_CMAP_RANGE, 10135 },
	{ 0x4e98, 0x4e9b, PDF_CMAP_TABLE, 386 },
	{ 0x4e9c, 0x4e9d, PDF_CMAP_RANGE, 10138 },
	{ 0x4e9e, 0x4eae, PDF_CMAP_TABLE, 390 },
	{ 0x4eaf, 0x4eb1, PDF_CMAP_RANGE, 10142 },
	{ 0x4eb2, 0x4eb5, PDF_CMAP_TABLE, 407 },
	{ 0x4eb6, 0x4eb9, PDF_CMAP_RANGE, 10146 },
	{ 0x4eba, 0x4ebb, PDF_CMAP_TABLE, 411 },
	{ 0x4ebc, 0x4ebe, PDF_CMAP_RANGE, 10150 },
	{ 0x4ebf, 0x4ece, PDF_CMAP_TABLE, 413 },
	{ 0x4ecf, 0x4ed0, PDF_CMAP_RANGE, 10155 },
	{ 0x4ed1, 0x4ed9, PDF_CMAP_TABLE, 429 },
	{ 0x4eda, 0x4edc, PDF_CMAP_RANGE, 10158 },
	{ 0x4edd, 0x4ee5, PDF_CMAP_TABLE, 438 },
	{ 0x4ee6, 0x4ee7, PDF_CMAP_RANGE, 10163 },
	{ 0x4ee8, 0x4eec, PDF_CMAP_TABLE, 447 },
	{ 0x4eed, 0x4eef, PDF_CMAP_RANGE, 10166 },
	{ 0x4ef0, 0x4ef7, PDF_CMAP_TABLE, 452 },
	{ 0x4ef8, 0x4efa, PDF_CMAP_RANGE, 10171 },
	{ 0x4efb, 0x4f01, PDF_CMAP_TABLE, 460 },
	{ 0x4f02, 0x4f08, PDF_CMAP_RANGE, 10177 },
	{ 0x4f09, 0x4f0a, PDF_CMAP_TABLE, 467 },
	{ 0x4f0b, 0x4f0c, PDF_CMAP_RANGE, 10184 },
	{ 0x4f0d, 0x4f11, PDF_CMAP_TABLE, 469 },
	{ 0x4f12, 0x4f16, PDF_CMAP_RANGE, 10186 },
	{ 0x4f17, 0x4f1b, PDF_CMAP_TABLE, 474 },
	{ 0x4f1c, 0x4f1d, PDF_CMAP_RANGE, 10191 },
	{ 0x4f1e, 0x4f27, PDF_CMAP_TABLE, 479 },
	{ 0x4f28, 0x4f29, PDF_CMAP_RANGE, 10195 },
	{ 0x4f2a, 0x4f2b, PDF_CMAP_TABLE, 489 },
	{ 0x4f2c, 0x4f2e, PDF_CMAP_RANGE, 10197 },
	{ 0x4f2f, 0x4f3d, PDF_CMAP_TABLE, 491 },
	{ 0x4f3e, 0x4f42, PDF_CMAP_RANGE, 10206 },
	{ 0x4f43, 0x4f43, PDF_CMAP_SINGLE, 1481 },
	{ 0x4f44, 0x4f45, PDF_CMAP_RANGE, 10211 },
	{ 0x4f46, 0x4f46, PDF_CMAP_SINGLE, 1420 },
	{ 0x4f47, 0x4f4c, PDF_CMAP_RANGE, 10213 },
	{ 0x4f4d, 0x4f60, PDF_CMAP_TABLE, 506 },
	{ 0x4f61, 0x4f62, PDF_CMAP_RANGE, 10222 },
	{ 0x4f63, 0x4f69, PDF_CMAP_TABLE, 526 },
	{ 0x4f6a, 0x4f6b, PDF_CMAP_RANGE, 10226 },
	{ 0x4f6c, 0x4f6c, PDF_CMAP_SINGLE, 2491 },
	{ 0x4f6d, 0x4f6e, PDF_CMAP_RANGE, 10228 },
	{ 0x4f6f, 0x4f70, PDF_CMAP_TABLE, 533 },
	{ 0x4f71, 0x4f72, PDF_CMAP_RANGE, 10230 },
	{ 0x4f73, 0x4f76, PDF_CMAP_TABLE, 535 },
	{ 0x4f77, 0x4f7a, PDF_CMAP_RANGE, 10233 },
	{ 0x4f7b, 0x4f7f, PDF_CMAP_TABLE, 539 },
	{ 0x4f80, 0x4f82, PDF_CMAP_RANGE, 10238 },
	{ 0x4f83, 0x4f91, PDF_CMAP_TABLE, 544 },
	{ 0x4f92, 0x4f93, PDF_CMAP_RANGE, 10247 },
	{ 0x4f94, 0x4f97, PDF_CMAP_TABLE, 559 },
	{ 0x4f98, 0x4f9a, PDF_CMAP_RANGE, 10250 },
	{ 0x4f9b, 0x4f9d, PDF_CMAP_TABLE, 563 },
	{ 0x4f9e, 0x4f9f, PDF_CMAP_RANGE, 10254 },
	{ 0x4fa0, 0x4fa0, PDF_CMAP_SINGLE, 3885 },
	{ 0x4fa1, 0x4fa2, PDF_CMAP_RANGE, 10256 },
	{ 0x4fa3, 0x4faf, PDF_CMAP_TABLE, 566 },
	{ 0x4fb0, 0x4fb4, PDF_CMAP_RANGE, 10261 },
	{ 0x4fb5, 0x4fb5, PDF_CMAP_SINGLE, 3155 },
	{ 0x4fb6, 0x4fbe, PDF_CMAP_RANGE, 10266 },
	{ 0x4fbf, 0x4fbf, PDF_CMAP_SINGLE, 1100 },
	{ 0x4fc0, 0x4fc1, PDF_CMAP_RANGE, 10275 },
	{ 0x4fc2, 0x4fc5, PDF_CMAP_TABLE, 579 },
	{ 0x4fc6, 0x4fc9, PDF_CMAP_RANGE, 10277 },
	{ 0x4fca, 0x4fca, PDF_CMAP_SINGLE, 2350 },
	{ 0x4fcb, 0x4fcd, PDF_CMAP_RANGE, 10281 },
	{ 0x4fce, 0x4fd1, PDF_CMAP_TABLE, 583 },
	{ 0x4fd2, 0x4fd6, PDF_CMAP_RANGE, 10284 },
	{ 0x4fd7, 0x4fe3, PDF_CMAP_TABLE, 587 },
	{ 0x4fe4, 0x4fe5, PDF_CMAP_RANGE, 10292 },
	{ 0x4fe6, 0x4fea, PDF_CMAP_TABLE, 600 },
	{ 0x4feb, 0x4fec, PDF_CMAP_RANGE, 10295 },
	{ 0x4fed, 0x4ff3, PDF_CMAP_TABLE, 605 },
	{ 0x4ff4, 0x4ff7, PDF_CMAP_RANGE, 10299 },
	{ 0x4ff8, 0x4ffa, PDF_CMAP_TABLE, 612 },
	{ 0x4ffb, 0x4ffd, PDF_CMAP_RANGE, 10304 },
	{ 0x4ffe, 0x5000, PDF_CMAP_TABLE, 615 },
	{ 0x5001, 0x5005, PDF_CMAP_RANGE, 10308 },
	{ 0x5006, 0x5006, PDF_CMAP_SINGLE, 8214 },
	{ 0x5007, 0x5008, PDF_CMAP_RANGE, 10313 },
	{ 0x5009, 0x5014, PDF_CMAP_TABLE, 618 },
	{ 0x5015, 0x5017, PDF_CMAP_RANGE, 10319 },
	{ 0x5018, 0x501c, PDF_CMAP_TABLE, 630 },
	{ 0x501d, 0x501e, PDF_CMAP_RANGE, 10323 },
	{ 0x501f, 0x5021, PDF_CMAP_TABLE, 635 },
	{ 0x5022, 0x5024, PDF_CMAP_RANGE, 10326 },
	{ 0x5025, 0x502e, PDF_CMAP_TABLE, 638 },
	{ 0x502f, 0x5039, PDF_CMAP_RANGE, 10330 },
	{ 0x503a, 0x503e, PDF_CMAP_TABLE, 648 },
	{ 0x503f, 0x5042, PDF_CMAP_RANGE, 10343 },
	{ 0x5043, 0x5043, PDF_CMAP_SINGLE, 4828 },
	{ 0x5044, 0x5046, PDF_CMAP_RANGE, 10347 },
	{ 0x5047, 0x5049, PDF_CMAP_TABLE, 653 },
	{ 0x504a, 0x504b, PDF_CMAP_RANGE, 10350 },
	{ 0x504c, 0x504f, PDF_CMAP_TABLE, 656 },
	{ 0x5050, 0x5054, PDF_CMAP_RANGE, 10353 },
	{ 0x5055, 0x5055, PDF_CMAP_SINGLE, 4829 },
	{ 0x5056, 0x5059, PDF_CMAP_RANGE, 10358 },
	{ 0x505a, 0x505c, PDF_CMAP_TABLE, 660 },
	{ 0x505d, 0x5064, PDF_CMAP_RANGE, 10363 },
	{ 0x5065, 0x5065, PDF_CMAP_SINGLE, 2162 },
	{ 0x5066, 0x506b, PDF_CMAP_RANGE, 10371 },
	{ 0x506c, 0x506c, PDF_CMAP_SINGLE, 4832 },
	{ 0x506d, 0x5073, PDF_CMAP_RANGE, 10377 },
	{ 0x5074, 0x5077, PDF_CMAP_TABLE, 663 },
	{ 0x5078, 0x507a, PDF_CMAP_RANGE, 10384 },
	{ 0x507b, 0x507b, PDF_CMAP_SINGLE, 4833 },
	{ 0x507c, 0x507d, PDF_CMAP_RANGE, 10387 },
	{ 0x507e, 0x5080, PDF_CMAP_TABLE, 667 },
	{ 0x5081, 0x5084, PDF_CMAP_RANGE, 10389 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 1712 },
	{ 0x5086, 0x5087, PDF_CMAP_RANGE, 10393 },
	{ 0x5088, 0x5088, PDF_CMAP_SINGLE, 2535 },
	{ 0x5089, 0x508c, PDF_CMAP_RANGE, 10395 },
	{ 0x508d, 0x508d, PDF_CMAP_SINGLE, 1027 },
	{ 0x508e, 0x5095, PDF_CMAP_RANGE, 10399 },
	{ 0x5096, 0x5099, PDF_CMAP_TABLE, 670 },
	{ 0x509a, 0x50a1, PDF_CMAP_RANGE, 10408 },
	{ 0x50a2, 0x50a9, PDF_CMAP_TABLE, 674 },
	{ 0x50aa, 0x50ab, PDF_CMAP_RANGE, 10418 },
	{ 0x50ac, 0x50ad, PDF_CMAP_TABLE, 682 },
	{ 0x50ae, 0x50b1, PDF_CMAP_RANGE, 10420 },
	{ 0x50b2, 0x50b7, PDF_CMAP_TABLE, 684 },
	{ 0x50b8, 0x50b9, PDF_CMAP_RANGE, 10425 },
	{ 0x50ba, 0x50bb, PDF_CMAP_TABLE, 690 },
	{ 0x50bc, 0x50bd, PDF_CMAP_RANGE, 10427 },
	{ 0x50be, 0x50be, PDF_CMAP_SINGLE, 8424 },
	{ 0x50bf, 0x50c1, PDF_CMAP_RANGE, 10429 },
	{ 0x50c2, 0x50c2, PDF_CMAP_SINGLE, 8916 },
	{ 0x50c3, 0x50c4, PDF_CMAP_RANGE, 10432 },
	{ 0x50c5, 0x50c5, PDF_CMAP_SINGLE, 8121 },
	{ 0x50c6, 0x50c8, PDF_CMAP_RANGE, 10434 },
	{ 0x50c9, 0x50c9, PDF_CMAP_SINGLE, 8920 },
	{ 0x50ca, 0x50ce, PDF_CMAP_RANGE, 10437 },
	{ 0x50cf, 0x50d1, PDF_CMAP_TABLE, 692 },
	{ 0x50d2, 0x50d4, PDF_CMAP_RANGE, 10443 },
	{ 0x50d5, 0x50d6, PDF_CMAP_TABLE, 695 },
	{ 0x50d7, 0x50d9, PDF_CMAP_RANGE, 10446 },
	{ 0x50da, 0x50da, PDF_CMAP_SINGLE, 2574 },
	{ 0x50db, 0x50dd, PDF_CMAP_RANGE, 10449 },
	{ 0x50de, 0x50de, PDF_CMAP_SINGLE, 8597 },
	{ 0x50df, 0x50e4, PDF_CMAP_RANGE, 10452 },
	{ 0x50e5, 0x50e8, PDF_CMAP_TABLE, 697 },
	{ 0x50e9, 0x50eb, PDF_CMAP_RANGE, 10458 },
	{ 0x50ec, 0x50ee, PDF_CMAP_TABLE, 701 },
	{ 0x50ef, 0x50f2, PDF_CMAP_RANGE, 10461 },
	{ 0x50f3, 0x50f5, PDF_CMAP_TABLE, 704 },
	{ 0x50f6, 0x50f8, PDF_CMAP_RANGE, 10466 },
	{ 0x50f9, 0x50fb, PDF_CMAP_TABLE, 707 },
	{ 0x50fc, 0x50ff, PDF_CMAP_RANGE, 10470 },
	{ 0x5100, 0x510b, PDF_CMAP_TABLE, 710 },
	{ 0x510c, 0x510f, PDF_CMAP_RANGE, 10478 },
	{ 0x5110, 0x5115, PDF_CMAP_TABLE, 722 },
	{ 0x5116, 0x5117, PDF_CMAP_RANGE, 10484 },
	{ 0x5118, 0x5118, PDF_CMAP_SINGLE, 9857 },
	{ 0x5119, 0x511e, PDF_CMAP_RANGE, 10486 },
	{ 0x511f, 0x5121, PDF_CMAP_TABLE, 728 },
	{ 0x5122, 0x5129, PDF_CMAP_RANGE, 10493 },
	{ 0x512a, 0x512a, PDF_CMAP_SINGLE, 8753 },
	{ 0x512b, 0x5131, PDF_CMAP_RANGE, 10501 },
	{ 0x5132, 0x5132, PDF_CMAP_SINGLE, 7827 },
	{ 0x5133, 0x5136, PDF_CMAP_RANGE, 10508 },
	{ 0x5137, 0x5137, PDF_CMAP_SINGLE, 8914 },
	{ 0x5138, 0x5139, PDF_CMAP_RANGE, 10512 },
	{ 0x513a, 0x513c, PDF_CMAP_TABLE, 731 },
	{ 0x513d, 0x513e, PDF_CMAP_RANGE, 10514 },
	{ 0x513f, 0x514d, PDF_CMAP_TABLE, 734 },
	{ 0x514e, 0x5150, PDF_CMAP_RANGE, 10520 },
	{ 0x5151, 0x5156, PDF_CMAP_TABLE, 749 },
	{ 0x5157, 0x5159, PDF_CMAP_RANGE, 10524 },
	{ 0x515a, 0x515c, PDF_CMAP_TABLE, 755 },
	{ 0x515d, 0x5161, PDF_CMAP_RANGE, 10528 },
	{ 0x5162, 0x5162, PDF_CMAP_SINGLE, 2259 },
	{ 0x5163, 0x5164, PDF_CMAP_RANGE, 10533 },
	{ 0x5165, 0x5165, PDF_CMAP_SINGLE, 3270 },
	{ 0x5166, 0x5167, PDF_CMAP_RANGE, 10535 },
	{ 0x5168, 0x517d, PDF_CMAP_TABLE, 758 },
	{ 0x517e, 0x517f, PDF_CMAP_RANGE, 10541 },
	{ 0x5180, 0x5182, PDF_CMAP_TABLE, 780 },
	{ 0x5183, 0x5184, PDF_CMAP_RANGE, 10543 },
	{ 0x5185, 0x5185, PDF_CMAP_SINGLE, 2877 },
	{ 0x5186, 0x5187, PDF_CMAP_RANGE, 10545 },
	{ 0x5188, 0x5189, PDF_CMAP_TABLE, 783 },
	{ 0x518a, 0x518b, PDF_CMAP_RANGE, 10547 },
	{ 0x518c, 0x518d, PDF_CMAP_TABLE, 785 },
	{ 0x518e, 0x5191, PDF_CMAP_RANGE, 10549 },
	{ 0x5192, 0x5192, PDF_CMAP_SINGLE, 2741 },
	{ 0x5193, 0x5194, PDF_CMAP_RANGE, 10553 },
	{ 0x5195, 0x519c, PDF_CMAP_TABLE, 787 },
	{ 0x519d, 0x519f, PDF_CMAP_RANGE, 10557 },
	{ 0x51a0, 0x51a5, PDF_CMAP_TABLE, 795 },
	{ 0x51a6, 0x51aa, PDF_CMAP_RANGE, 10562 },
	{ 0x51ab, 0x51ac, PDF_CMAP_TABLE, 801 },
	{ 0x51ad, 0x51ae, PDF_CMAP_RANGE, 10567 },
	{ 0x51af, 0x51b7, PDF_CMAP_TABLE, 803 },
	{ 0x51b8, 0x51ba, PDF_CMAP_RANGE, 10570 },
	{ 0x51bb, 0x51bd, PDF_CMAP_TABLE, 812 },
	{ 0x51be, 0x51bf, PDF_CMAP_RANGE, 10573 },
	{ 0x51c0, 0x51c0, PDF_CMAP_SINGLE, 2282 },
	{ 0x51c1, 0x51c3, PDF_CMAP_RANGE, 10575 },
	{ 0x51c4, 0x51d1, PDF_CMAP_TABLE, 815 },
	{ 0x51d2, 0x51da, PDF_CMAP_RANGE, 10583 },
	{ 0x51db, 0x51dd, PDF_CMAP_TABLE, 829 },
	{ 0x51de, 0x51df, PDF_CMAP_RANGE, 10593 },
	{ 0x51e0, 0x51e1, PDF_CMAP_TABLE, 832 },
	{ 0x51e2, 0x51e3, PDF_CMAP_RANGE, 10595 },
	{ 0x51e4, 0x51e4, PDF_CMAP_SINGLE, 1676 },
	{ 0x51e5, 0x51ea, PDF_CMAP_RANGE, 10597 },
	{ 0x51eb, 0x5203, PDF_CMAP_TABLE, 834 },
	{ 0x5204, 0x5205, PDF_CMAP_RANGE, 10609 },
	{ 0x5206, 0x520a, PDF_CMAP_TABLE, 859 },
	{ 0x520b, 0x520c, PDF_CMAP_RANGE, 10612 },
	{ 0x520d, 0x520e, PDF_CMAP_TABLE, 864 },
	{ 0x520f, 0x5210, PDF_CMAP_RANGE, 10614 },
	{ 0x5211, 0x5212, PDF_CMAP_TABLE, 866 },
	{ 0x5213, 0x5215, PDF_CMAP_RANGE, 10616 },
	{ 0x5216, 0x521d, PDF_CMAP_TABLE, 868 },
	{ 0x521e, 0x521f, PDF_CMAP_RANGE, 10620 },
	{ 0x5220, 0x5220, PDF_CMAP_SINGLE, 3319 },
	{ 0x5221, 0x5223, PDF_CMAP_RANGE, 10622 },
	{ 0x5224, 0x5224, PDF_CMAP_SINGLE, 2961 },
	{ 0x5225, 0x5227, PDF_CMAP_RANGE, 10625 },
	{ 0x5228, 0x5230, PDF_CMAP_TABLE, 876 },
	{ 0x5231, 0x5232, PDF_CMAP_RANGE, 10631 },
	{ 0x5233, 0x5233, PDF_CMAP_SINGLE, 4752 },
	{ 0x5234, 0x5235, PDF_CMAP_RANGE, 10633 },
	{ 0x5236, 0x523e, PDF_CMAP_TABLE, 885 },
	{ 0x523f, 0x5240, PDF_CMAP_RANGE, 4753 },
	{ 0x5241, 0x5244, PDF_CMAP_TABLE, 894 },
	{ 0x5245, 0x5246, PDF_CMAP_RANGE, 10637 },
	{ 0x5247, 0x5247, PDF_CMAP_SINGLE, 8803 },
	{ 0x5248, 0x5249, PDF_CMAP_RANGE, 10639 },
	{ 0x524a, 0x524d, PDF_CMAP_TABLE, 898 },
	{ 0x524e, 0x524f, PDF_CMAP_RANGE, 10641 },
	{ 0x5250, 0x5251, PDF_CMAP_TABLE, 902 },
	{ 0x5252, 0x5253, PDF_CMAP_RANGE, 10643 },
	{ 0x5254, 0x5256, PDF_CMAP_TABLE, 904 },
	{ 0x5257, 0x525a, PDF_CMAP_RANGE, 10646 },
	{ 0x525b, 0x525e, PDF_CMAP_TABLE, 907 },
	{ 0x525f, 0x5260, PDF_CMAP_RANGE, 10651 },
	{ 0x5261, 0x5261, PDF_CMAP_SINGLE, 4757 },
	{ 0x5262, 0x5264, PDF_CMAP_RANGE, 10653 },
	{ 0x5265, 0x526a, PDF_CMAP_TABLE, 911 },
	{ 0x526b, 0x526d, PDF_CMAP_RANGE, 10658 },
	{ 0x526e, 0x526f, PDF_CMAP_TABLE, 917 },
	{ 0x5270, 0x5271, PDF_CMAP_RANGE, 10661 },
	{ 0x5272, 0x5275, PDF_CMAP_TABLE, 919 },
	{ 0x5276, 0x527c, PDF_CMAP_RANGE, 10664 },
	{ 0x527d, 0x5283, PDF_CMAP_TABLE, 923 },
	{ 0x5284, 0x5286, PDF_CMAP_RANGE, 10673 },
	{ 0x5287, 0x528d, PDF_CMAP_TABLE, 930 },
	{ 0x528e, 0x528f, PDF_CMAP_RANGE, 10677 },
	{ 0x5290, 0x5293, PDF_CMAP_TABLE, 937 },
	{ 0x5294, 0x529a, PDF_CMAP_RANGE, 10680 },
	{ 0x529b, 0x52a3, PDF_CMAP_TABLE, 941 },
	{ 0x52a4, 0x52a7, PDF_CMAP_RANGE, 10688 },
	{ 0x52a8, 0x52ab, PDF_CMAP_TABLE, 950 },
	{ 0x52ac, 0x52ad, PDF_CMAP_RANGE, 5006 },
	{ 0x52ae, 0x52b0, PDF_CMAP_RANGE, 10692 },
	{ 0x52b1, 0x52b3, PDF_CMAP_TABLE, 954 },
	{ 0x52b4, 0x52bd, PDF_CMAP_RANGE, 10695 },
	{ 0x52be, 0x52c3, PDF_CMAP_TABLE, 957 },
	{ 0x52c4, 0x52c6, PDF_CMAP_RANGE, 10707 },
	{ 0x52c7, 0x52cb, PDF_CMAP_TABLE, 963 },
	{ 0x52cc, 0x52cf, PDF_CMAP_RANGE, 10712 },
	{ 0x52d0, 0x52d2, PDF_CMAP_TABLE, 968 },
	{ 0x52d3, 0x52d4, PDF_CMAP_RANGE, 10717 },
	{ 0x52d5, 0x52df, PDF_CMAP_TABLE, 971 },
	{ 0x52e0, 0x52e1, PDF_CMAP_RANGE, 10722 },
	{ 0x52e2, 0x52e4, PDF_CMAP_TABLE, 982 },
	{ 0x52e5, 0x52ef, PDF_CMAP_RANGE, 10725 },
	{ 0x52f0, 0x52f1, PDF_CMAP_TABLE, 985 },
	{ 0x52f2, 0x52f4, PDF_CMAP_RANGE, 10736 },
	{ 0x52f5, 0x52f5, PDF_CMAP_SINGLE, 8209 },
	{ 0x52f6, 0x52f7, PDF_CMAP_RANGE, 10739 },
	{ 0x52f8, 0x52fa, PDF_CMAP_TABLE, 987 },
	{ 0x52fb, 0x52fd, PDF_CMAP_RANGE, 10741 },
	{ 0x52fe, 0x5300, PDF_CMAP_TABLE, 990 },
	{ 0x5301, 0x5304, PDF_CMAP_RANGE, 10744 },
	{ 0x5305, 0x5308, PDF_CMAP_TABLE, 993 },
	{ 0x5309, 0x530c, PDF_CMAP_RANGE, 10749 },
	{ 0x530d, 0x5310, PDF_CMAP_TABLE, 997 },
	{ 0x5311, 0x5314, PDF_CMAP_RANGE, 10754 },
	{ 0x5315, 0x531a, PDF_CMAP_TABLE, 1001 },
	{ 0x531b, 0x531c, PDF_CMAP_RANGE, 10759 },
	{ 0x531d, 0x531d, PDF_CMAP_SINGLE, 4372 },
	{ 0x531e, 0x531f, PDF_CMAP_RANGE, 10761 },
	{ 0x5320, 0x5323, PDF_CMAP_TABLE, 1007 },
	{ 0x5324, 0x5325, PDF_CMAP_RANGE, 10764 },
	{ 0x5326, 0x5326, PDF_CMAP_SINGLE, 4742 },
	{ 0x5327, 0x5329, PDF_CMAP_RANGE, 10766 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 1640 },
	{ 0x532b, 0x532c, PDF_CMAP_RANGE, 10769 },
	{ 0x532d, 0x5331, PDF_CMAP_TABLE, 1011 },
	{ 0x5332, 0x5338, PDF_CMAP_RANGE, 10772 },
	{ 0x5339, 0x533b, PDF_CMAP_TABLE, 1016 },
	{ 0x533c, 0x533d, PDF_CMAP_RANGE, 10779 },
	{ 0x533e, 0x534a, PDF_CMAP_TABLE, 1019 },
	{ 0x534b, 0x534d, PDF_CMAP_RANGE, 10784 },
	{ 0x534e, 0x5357, PDF_CMAP_TABLE, 1032 },
	{ 0x5358, 0x5359, PDF_CMAP_RANGE, 10788 },
	{ 0x535a, 0x536b, PDF_CMAP_TABLE, 1042 },
	{ 0x536c, 0x536d, PDF_CMAP_RANGE, 10795 },
	{ 0x536e, 0x537a, PDF_CMAP_TABLE, 1060 },
	{ 0x537b, 0x537e, PDF_CMAP_RANGE, 10800 },
	{ 0x537f, 0x537f, PDF_CMAP_SINGLE, 3169 },
	{ 0x5380, 0x5381, PDF_CMAP_RANGE, 10804 },
	{ 0x5382, 0x5386, PDF_CMAP_TABLE, 1073 },
	{ 0x5387, 0x5388, PDF_CMAP_RANGE, 10807 },
	{ 0x5389, 0x538d, PDF_CMAP_TABLE, 1078 },
	{ 0x538e, 0x5394, PDF_CMAP_RANGE, 10810 },
	{ 0x5395, 0x5395, PDF_CMAP_SINGLE, 1189 },
	{ 0x5396, 0x5397, PDF_CMAP_RANGE, 10817 },
	{ 0x5398, 0x539a, PDF_CMAP_TABLE, 1083 },
	{ 0x539b, 0x539c, PDF_CMAP_RANGE, 10819 },
	{ 0x539d, 0x53a9, PDF_CMAP_TABLE, 1086 },
	{ 0x53aa, 0x53ac, PDF_CMAP_RANGE, 10825 },
	{ 0x53ad, 0x53ae, PDF_CMAP_TABLE, 1099 },
	{ 0x53af, 0x53b1, PDF_CMAP_RANGE, 10828 },
	{ 0x53b2, 0x53b6, PDF_CMAP_TABLE, 1101 },
	{ 0x53b7, 0x53ba, PDF_CMAP_RANGE, 10833 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 3200 },
	{ 0x53bc, 0x53be, PDF_CMAP_RANGE, 10837 },
	{ 0x53bf, 0x53c3, PDF_CMAP_TABLE, 1106 },
	{ 0x53c4, 0x53c7, PDF_CMAP_RANGE, 10841 },
	{ 0x53c8, 0x53cd, PDF_CMAP_TABLE, 1111 },
	{ 0x53ce, 0x53d0, PDF_CMAP_RANGE, 10845 },
	{ 0x53d1, 0x53d1, PDF_CMAP_SINGLE, 1599 },
	{ 0x53d2, 0x53d3, PDF_CMAP_RANGE, 10848 },
	{ 0x53d4, 0x53db, PDF_CMAP_TABLE, 1117 },
	{ 0x53dc, 0x53de, PDF_CMAP_RANGE, 10852 },
	{ 0x53df, 0x53fd, PDF_CMAP_TABLE, 1125 },
	{ 0x53fe, 0x5400, PDF_CMAP_RANGE, 10859 },
	{ 0x5401, 0x5417, PDF_CMAP_TABLE, 1156 },
	{ 0x5418, 0x541a, PDF_CMAP_RANGE, 10867 },
	{ 0x541b, 0x5423, PDF_CMAP_TABLE, 1179 },
	{ 0x5424, 0x5425, PDF_CMAP_RANGE, 10872 },
	{ 0x5426, 0x5435, PDF_CMAP_TABLE, 1188 },
	{ 0x5436, 0x5437, PDF_CMAP_RANGE, 10877 },
	{ 0x5438, 0x5440, PDF_CMAP_TABLE, 1204 },
	{ 0x5441, 0x5442, PDF_CMAP_RANGE, 10882 },
	{ 0x5443, 0x5443, PDF_CMAP_SINGLE, 5384 },
	{ 0x5444, 0x5445, PDF_CMAP_RANGE, 10884 },
	{ 0x5446, 0x544b, PDF_CMAP_TABLE, 1213 },
	{ 0x544c, 0x544f, PDF_CMAP_RANGE, 10888 },
	{ 0x5450, 0x5451, PDF_CMAP_TABLE, 1219 },
	{ 0x5452, 0x5454, PDF_CMAP_RANGE, 5380 },
	{ 0x5455, 0x545c, PDF_CMAP_TABLE, 1221 },
	{ 0x545d, 0x5461, PDF_CMAP_RANGE, 10894 },
	{ 0x5462, 0x5468, PDF_CMAP_TABLE, 1229 },
	{ 0x5469, 0x5470, PDF_CMAP_RANGE, 10902 },
	{ 0x5471, 0x5478, PDF_CMAP_TABLE, 1236 },
	{ 0x5479, 0x547a, PDF_CMAP_RANGE, 10911 },
	{ 0x547b, 0x547d, PDF_CMAP_TABLE, 1244 },
	{ 0x547e, 0x547f, PDF_CMAP_RANGE, 10913 },
	{ 0x5480, 0x5486, PDF_CMAP_TABLE, 1247 },
	{ 0x5487, 0x548a, PDF_CMAP_RANGE, 10918 },
	{ 0x548b, 0x5496, PDF_CMAP_TABLE, 1254 },
	{ 0x5497, 0x5498, PDF_CMAP_RANGE, 10925 },
	{ 0x5499, 0x5499, PDF_CMAP_SINGLE, 2629 },
	{ 0x549a, 0x549b, PDF_CMAP_RANGE, 5395 },
	{ 0x549c, 0x549d, PDF_CMAP_TABLE, 1266 },
	{ 0x549e, 0x54a2, PDF_CMAP_RANGE, 10928 },
	{ 0x54a3, 0x54a8, PDF_CMAP_TABLE, 1268 },
	{ 0x54a9, 0x54aa, PDF_CMAP_RANGE, 5419 },
	{ 0x54ab, 0x54b4, PDF_CMAP_TABLE, 1274 },
	{ 0x54b5, 0x54b7, PDF_CMAP_RANGE, 10937 },
	{ 0x54b8, 0x54b8, PDF_CMAP_SINGLE, 3897 },
	{ 0x54b9, 0x54ba, PDF_CMAP_RANGE, 10940 },
	{ 0x54bb, 0x54c9, PDF_CMAP_TABLE, 1284 },
	{ 0x54ca, 0x54cb, PDF_CMAP_RANGE, 10945 },
	{ 0x54cc, 0x54d2, PDF_CMAP_TABLE, 1299 },
	{ 0x54d3, 0x54d4, PDF_CMAP_RANGE, 5408 },
	{ 0x54d5, 0x54d8, PDF_CMAP_TABLE, 1306 },
	{ 0x54d9, 0x54da, PDF_CMAP_RANGE, 5416 },
	{ 0x54db, 0x54e1, PDF_CMAP_TABLE, 1310 },
	{ 0x54e2, 0x54e4, PDF_CMAP_RANGE, 10951 },
	{ 0x54e5, 0x54ea, PDF_CMAP_TABLE, 1317 },
	{ 0x54eb, 0x54ec, PDF_CMAP_RANGE, 10954 },
	{ 0x54ed, 0x54ee, PDF_CMAP_TABLE, 1323 },
	{ 0x54ef, 0x54f1, PDF_CMAP_RANGE, 10956 },
	{ 0x54f2, 0x54f3, PDF_CMAP_TABLE, 1325 },
	{ 0x54f4, 0x54f9, PDF_CMAP_RANGE, 10959 },
	{ 0x54fa, 0x5501, PDF_CMAP_TABLE, 1327 },
	{ 0x5502, 0x5503, PDF_CMAP_RANGE, 10968 },
	{ 0x5504, 0x5509, PDF_CMAP_TABLE, 1335 },
	{ 0x550a, 0x550e, PDF_CMAP_RANGE, 10972 },
	{ 0x550f, 0x5511, PDF_CMAP_TABLE, 1341 },
	{ 0x5512, 0x5513, PDF_CMAP_RANGE, 10977 },
	{ 0x5514, 0x5514, PDF_CMAP_SINGLE, 5429 },
	{ 0x5515, 0x551a, PDF_CMAP_RANGE, 10979 },
	{ 0x551b, 0x551b, PDF_CMAP_SINGLE, 5425 },
	{ 0x551c, 0x551f, PDF_CMAP_RANGE, 10985 },
	{ 0x5520, 0x5521, PDF_CMAP_TABLE, 1344 },
	{ 0x5522, 0x5523, PDF_CMAP_RANGE, 5431 },
	{ 0x5524, 0x5524, PDF_CMAP_SINGLE, 2002 },
	{ 0x5525, 0x5526, PDF_CMAP_RANGE, 10990 },
	{ 0x5527, 0x5527, PDF_CMAP_SINGLE, 5435 },
	{ 0x5528, 0x5529, PDF_CMAP_RANGE, 10992 },
	{ 0x552a, 0x5533, PDF_CMAP_TABLE, 1346 },
	{ 0x5534, 0x5536, PDF_CMAP_RANGE, 10997 },
	{ 0x5537, 0x5537, PDF_CMAP_SINGLE, 5447 },
	{ 0x5538, 0x553b, PDF_CMAP_RANGE, 11000 },
	{ 0x553c, 0x5546, PDF_CMAP_TABLE, 1356 },
	{ 0x5547, 0x5548, PDF_CMAP_RANGE, 11008 },
	{ 0x5549, 0x554a, PDF_CMAP_TABLE, 1367 },
	{ 0x554b, 0x554e, PDF_CMAP_RANGE, 11010 },
	{ 0x554f, 0x5550, PDF_CMAP_TABLE, 1369 },
	{ 0x5551, 0x5552, PDF_CMAP_RANGE, 11014 },
	{ 0x5553, 0x5556, PDF_CMAP_TABLE, 1371 },
	{ 0x5557, 0x555b, PDF_CMAP_RANGE, 11017 },
	{ 0x555c, 0x555e, PDF_CMAP_TABLE, 1375 },
	{ 0x555f, 0x5560, PDF_CMAP_RANGE, 11023 },
	{ 0x5561, 0x5561, PDF_CMAP_SINGLE, 1637 },
	{ 0x5562, 0x5563, PDF_CMAP_RANGE, 11025 },
	{ 0x5564, 0x5567, PDF_CMAP_TABLE, 1378 },
	{ 0x5568, 0x5569, PDF_CMAP_RANGE, 11027 },
	{ 0x556a, 0x556e, PDF_CMAP_TABLE, 1382 },
	{ 0x556f, 0x5574, PDF_CMAP_RANGE, 11030 },
	{ 0x5575, 0x5577, PDF_CMAP_RANGE, 5449 },
	{ 0x5578, 0x5578, PDF_CMAP_SINGLE, 3952 },
	{ 0x5579, 0x557a, PDF_CMAP_RANGE, 11036 },
	{ 0x557b, 0x5584, PDF_CMAP_TABLE, 1387 },
	{ 0x5585, 0x5586, PDF_CMAP_RANGE, 11040 },
	{ 0x5587, 0x558b, PDF_CMAP_TABLE, 1397 },
	{ 0x558c, 0x558e, PDF_CMAP_RANGE, 11042 },
	{ 0x558f, 0x5591, PDF_CMAP_TABLE, 1402 },
	{ 0x5592, 0x5593, PDF_CMAP_RANGE, 11046 },
	{ 0x5594, 0x5594, PDF_CMAP_SINGLE, 5470 },
	{ 0x5595, 0x5597, PDF_CMAP_RANGE, 11048 },
	{ 0x5598, 0x5599, PDF_CMAP_TABLE, 1405 },
	{ 0x559a, 0x559b, PDF_CMAP_RANGE, 11051 },
	{ 0x559c, 0x559f, PDF_CMAP_TABLE, 1407 },
	{ 0x55a0, 0x55a6, PDF_CMAP_RANGE, 11054 },
	{ 0x55a7, 0x55a7, PDF_CMAP_SINGLE, 4037 },
	{ 0x55a8, 0x55a9, PDF_CMAP_RANGE, 11061 },
	{ 0x55aa, 0x55ae, PDF_CMAP_TABLE, 1411 },
	{ 0x55af, 0x55b0, PDF_CMAP_RANGE, 11065 },
	{ 0x55b1, 0x55bc, PDF_CMAP_TABLE, 1416 },
	{ 0x55bd, 0x55be, PDF_CMAP_RANGE, 5468 },
	{ 0x55bf, 0x55c3, PDF_CMAP_RANGE, 11072 },
	{ 0x55c4, 0x55c9, PDF_CMAP_TABLE, 1428 },
	{ 0x55ca, 0x55cb, PDF_CMAP_RANGE, 11078 },
	{ 0x55cc, 0x55cd, PDF_CMAP_RANGE, 5487 },
	{ 0x55ce, 0x55ce, PDF_CMAP_SINGLE, 8303 },
	{ 0x55cf, 0x55d0, PDF_CMAP_RANGE, 11080 },
	{ 0x55d1, 0x55d6, PDF_CMAP_TABLE, 1434 },
	{ 0x55d7, 0x55d9, PDF_CMAP_RANGE, 11083 },
	{ 0x55da, 0x55ea, PDF_CMAP_TABLE, 1440 },
	{ 0x55eb, 0x55ec, PDF_CMAP_RANGE, 5477 },
	{ 0x55ed, 0x55ee, PDF_CMAP_RANGE, 11091 },
	{ 0x55ef, 0x55ef, PDF_CMAP_SINGLE, 5483 },
	{ 0x55f0, 0x55f1, PDF_CMAP_RANGE, 11093 },
	{ 0x55f2, 0x55f3, PDF_CMAP_RANGE, 5485 },
	{ 0x55f4, 0x55f7, PDF_CMAP_TABLE, 1457 },
	{ 0x55f8, 0x55fc, PDF_CMAP_RANGE, 11096 },
	{ 0x55fd, 0x5601, PDF_CMAP_TABLE, 1461 },
	{ 0x5602, 0x5605, PDF_CMAP_RANGE, 11102 },
	{ 0x5606, 0x5609, PDF_CMAP_TABLE, 1466 },
	{ 0x560a, 0x560b, PDF_CMAP_RANGE, 11107 },
	{ 0x560c, 0x560f, PDF_CMAP_TABLE, 1470 },
	{ 0x5610, 0x5613, PDF_CMAP_RANGE, 11109 },
	{ 0x5614, 0x5618, PDF_CMAP_TABLE, 1474 },
	{ 0x5619, 0x561a, PDF_CMAP_RANGE, 11114 },
	{ 0x561b, 0x561f, PDF_CMAP_TABLE, 1479 },
	{ 0x5620, 0x5622, PDF_CMAP_RANGE, 11117 },
	{ 0x5623, 0x5624, PDF_CMAP_TABLE, 1484 },
	{ 0x5625, 0x5626, PDF_CMAP_RANGE, 11120 },
	{ 0x5627, 0x5629, PDF_CMAP_TABLE, 1486 },
	{ 0x562a, 0x562b, PDF_CMAP_RANGE, 11123 },
	{ 0x562c, 0x563b, PDF_CMAP_TABLE, 1489 },
	{ 0x563c, 0x563e, PDF_CMAP_RANGE, 11128 },
	{ 0x563f, 0x5641, PDF_CMAP_TABLE, 1505 },
	{ 0x5642, 0x564b, PDF_CMAP_RANGE, 11132 },
	{ 0x564c, 0x564e, PDF_CMAP_TABLE, 1508 },
	{ 0x564f, 0x5653, PDF_CMAP_RANGE, 11142 },
	{ 0x5654, 0x5654, PDF_CMAP_SINGLE, 5512 },
	{ 0x5655, 0x5656, PDF_CMAP_RANGE, 11147 },
	{ 0x5657, 0x5659, PDF_CMAP_TABLE, 1511 },
	{ 0x565a, 0x565b, PDF_CMAP_RANGE, 11149 },
	{ 0x565c, 0x565d, PDF_CMAP_TABLE, 1514 },
	{ 0x565e, 0x565f, PDF_CMAP_RANGE, 11151 },
	{ 0x5660, 0x566c, PDF_CMAP_TABLE, 1516 },
	{ 0x566d, 0x566e, PDF_CMAP_RANGE, 11156 },
	{ 0x566f, 0x567a, PDF_CMAP_TABLE, 1529 },
	{ 0x567b, 0x567c, PDF_CMAP_RANGE, 5517 },
	{ 0x567d, 0x567f, PDF_CMAP_RANGE, 11163 },
	{ 0x5680, 0x5680, PDF_CMAP_SINGLE, 9070 },
	{ 0x5681, 0x5684, PDF_CMAP_RANGE, 11166 },
	{ 0x5685, 0x5687, PDF_CMAP_TABLE, 1541 },
	{ 0x5688, 0x568b, PDF_CMAP_RANGE, 11170 },
	{ 0x568c, 0x568f, PDF_CMAP_TABLE, 1544 },
	{ 0x5690, 0x5692, PDF_CMAP_RANGE, 11175 },
	{ 0x5693, 0x5695, PDF_CMAP_TABLE, 1548 },
	{ 0x5696, 0x5698, PDF_CMAP_RANGE, 11179 },
	{ 0x5699, 0x5699, PDF_CMAP_SINGLE, 8350 },
	{ 0x569a, 0x56a2, PDF_CMAP_RANGE, 11182 },
	{ 0x56a3, 0x56a3, PDF_CMAP_SINGLE, 3942 },
	{ 0x56a4, 0x56a5, PDF_CMAP_RANGE, 11191 },
	{ 0x56a6, 0x56a8, PDF_CMAP_TABLE, 1551 },
	{ 0x56a9, 0x56ad, PDF_CMAP_RANGE, 11194 },
	{ 0x56ae, 0x56af, PDF_CMAP_TABLE, 1554 },
	{ 0x56b0, 0x56b2, PDF_CMAP_RANGE, 11199 },
	{ 0x56b3, 0x56b7, PDF_CMAP_TABLE, 1556 },
	{ 0x56b8, 0x56bb, PDF_CMAP_RANGE, 11203 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 2193 },
	{ 0x56bd, 0x56bf, PDF_CMAP_RANGE, 11207 },
	{ 0x56c0, 0x56c2, PDF_CMAP_TABLE, 1561 },
	{ 0x56c3, 0x56c4, PDF_CMAP_RANGE, 11210 },
	{ 0x56c5, 0x56c5, PDF_CMAP_SINGLE, 8923 },
	{ 0x56c6, 0x56c7, PDF_CMAP_RANGE, 11212 },
	{ 0x56c8, 0x56cc, PDF_CMAP_TABLE, 1564 },
	{ 0x56cd, 0x56d0, PDF_CMAP_RANGE, 11215 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 8865 },
	{ 0x56d2, 0x56d3, PDF_CMAP_RANGE, 11219 },
	{ 0x56d4, 0x56d4, PDF_CMAP_SINGLE, 5522 },
	{ 0x56d5, 0x56d6, PDF_CMAP_RANGE, 11221 },
	{ 0x56d7, 0x56d7, PDF_CMAP_SINGLE, 5523 },
	{ 0x56d8, 0x56d9, PDF_CMAP_RANGE, 11223 },
	{ 0x56da, 0x56e4, PDF_CMAP_TABLE, 1569 },
	{ 0x56e5, 0x56ea, PDF_CMAP_RANGE, 11227 },
	{ 0x56eb, 0x56ed, PDF_CMAP_TABLE, 1580 },
	{ 0x56ee, 0x56ef, PDF_CMAP_RANGE, 11234 },
	{ 0x56f0, 0x56f1, PDF_CMAP_TABLE, 1583 },
	{ 0x56f2, 0x56f3, PDF_CMAP_RANGE, 11236 },
	{ 0x56f4, 0x56f5, PDF_CMAP_TABLE, 1585 },
	{ 0x56f6, 0x56f8, PDF_CMAP_RANGE, 11238 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 1587 },
	{ 0x56fb, 0x56fc, PDF_CMAP_RANGE, 11241 },
	{ 0x56fd, 0x56ff, PDF_CMAP_TABLE, 1589 },
	{ 0x5700, 0x5702, PDF_CMAP_RANGE, 11243 },
	{ 0x5703, 0x570d, PDF_CMAP_TABLE, 1592 },
	{ 0x570e, 0x5711, PDF_CMAP_RANGE, 11248 },
	{ 0x5712, 0x5713, PDF_CMAP_TABLE, 1603 },
	{ 0x5714, 0x5715, PDF_CMAP_RANGE, 11252 },
	{ 0x5716, 0x5718, PDF_CMAP_TABLE, 1605 },
	{ 0x5719, 0x571b, PDF_CMAP_RANGE, 11255 },
	{ 0x571c, 0x571c, PDF_CMAP_SINGLE, 5533 },
	{ 0x571d, 0x571e, PDF_CMAP_RANGE, 11258 },
	{ 0x571f, 0x571f, PDF_CMAP_SINGLE, 3698 },
	{ 0x5720, 0x5722, PDF_CMAP_RANGE, 11260 },
	{ 0x5723, 0x5723, PDF_CMAP_SINGLE, 3388 },
	{ 0x5724, 0x5727, PDF_CMAP_RANGE, 11263 },
	{ 0x5728, 0x572d, PDF_CMAP_TABLE, 1608 },
	{ 0x572e, 0x572f, PDF_CMAP_RANGE, 5036 },
	{ 0x5730, 0x5730, PDF_CMAP_SINGLE, 1465 },
	{ 0x5731, 0x5732, PDF_CMAP_RANGE, 11268 },
	{ 0x5733, 0x5733, PDF_CMAP_SINGLE, 5034 },
	{ 0x5734, 0x5738, PDF_CMAP_RANGE, 11270 },
	{ 0x5739, 0x573b, PDF_CMAP_TABLE, 1614 },
	{ 0x573c, 0x573d, PDF_CMAP_RANGE, 11275 },
	{ 0x573e, 0x5742, PDF_CMAP_TABLE, 1617 },
	{ 0x5743, 0x5746, PDF_CMAP_RANGE, 11279 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 2344 },
	{ 0x5748, 0x5749, PDF_CMAP_RANGE, 11283 },
	{ 0x574a, 0x5751, PDF_CMAP_TABLE, 1622 },
	{ 0x5752, 0x5756, PDF_CMAP_RANGE, 11286 },
	{ 0x5757, 0x5757, PDF_CMAP_SINGLE, 2422 },
	{ 0x5758, 0x5759, PDF_CMAP_RANGE, 11291 },
	{ 0x575a, 0x5761, PDF_CMAP_TABLE, 1630 },
	{ 0x5762, 0x5763, PDF_CMAP_RANGE, 11293 },
	{ 0x5764, 0x576f, PDF_CMAP_TABLE, 1638 },
	{ 0x5770, 0x5772, PDF_CMAP_RANGE, 11299 },
	{ 0x5773, 0x5773, PDF_CMAP_SINGLE, 5050 },
	{ 0x5774, 0x5775, PDF_CMAP_RANGE, 11302 },
	{ 0x5776, 0x5777, PDF_CMAP_TABLE, 1650 },
	{ 0x5778, 0x577a, PDF_CMAP_RANGE, 11304 },
	{ 0x577b, 0x577c, PDF_CMAP_TABLE, 1652 },
	{ 0x577d, 0x5781, PDF_CMAP_RANGE, 11307 },
	{ 0x5782, 0x5786, PDF_CMAP_TABLE, 1654 },
	{ 0x5787, 0x578a, PDF_CMAP_RANGE, 11312 },
	{ 0x578b, 0x578c, PDF_CMAP_TABLE, 1659 },
	{ 0x578d, 0x5791, PDF_CMAP_RANGE, 11316 },
	{ 0x5792, 0x5793, PDF_CMAP_TABLE, 1661 },
	{ 0x5794, 0x579a, PDF_CMAP_RANGE, 11321 },
	{ 0x579b, 0x579b, PDF_CMAP_SINGLE, 1569 },
	{ 0x579c, 0x579f, PDF_CMAP_RANGE, 11328 },
	{ 0x57a0, 0x57ae, PDF_CMAP_TABLE, 1663 },
	{ 0x57af, 0x57b1, PDF_CMAP_RANGE, 11336 },
	{ 0x57b2, 0x57b4, PDF_CMAP_TABLE, 1678 },
	{ 0x57b5, 0x57b7, PDF_CMAP_RANGE, 11340 },
	{ 0x57b8, 0x57b8, PDF_CMAP_SINGLE, 5065 },
	{ 0x57b9, 0x57c1, PDF_CMAP_RANGE, 11343 },
	{ 0x57c2, 0x57c3, PDF_CMAP_TABLE, 1681 },
	{ 0x57c4, 0x57ca, PDF_CMAP_RANGE, 11352 },
	{ 0x57cb, 0x57cb, PDF_CMAP_SINGLE, 2712 },
	{ 0x57cc, 0x57cd, PDF_CMAP_RANGE, 11359 },
	{ 0x57ce, 0x57cf, PDF_CMAP_TABLE, 1683 },
	{ 0x57d0, 0x57d1, PDF_CMAP_RANGE, 11361 },
	{ 0x57d2, 0x57d5, PDF_CMAP_TABLE, 1685 },
	{ 0x57d6, 0x57d7, PDF_CMAP_RANGE, 11364 },
	{ 0x57d8, 0x57da, PDF_CMAP_TABLE, 1689 },
	{ 0x57db, 0x57dc, PDF_CMAP_RANGE, 11366 },
	{ 0x57dd, 0x57e1, PDF_CMAP_TABLE, 1692 },
	{ 0x57e2, 0x57e3, PDF_CMAP_RANGE, 11369 },
	{ 0x57e4, 0x57e4, PDF_CMAP_SINGLE, 5069 },
	{ 0x57e5, 0x57ec, PDF_CMAP_RANGE, 11371 },
	{ 0x57ed, 0x57ef, PDF_CMAP_TABLE, 1697 },
	{ 0x57f0, 0x57f3, PDF_CMAP_RANGE, 11380 },
	{ 0x57f4, 0x57f4, PDF_CMAP_SINGLE, 5066 },
	{ 0x57f5, 0x57f6, PDF_CMAP_RANGE, 11384 },
	{ 0x57f7, 0x57fa, PDF_CMAP_TABLE, 1700 },
	{ 0x57fb, 0x57fc, PDF_CMAP_RANGE, 11386 },
	{ 0x57fd, 0x57fd, PDF_CMAP_SINGLE, 5073 },
	{ 0x57fe, 0x57ff, PDF_CMAP_RANGE, 11388 },
	{ 0x5800, 0x5802, PDF_CMAP_TABLE, 1704 },
	{ 0x5803, 0x5804, PDF_CMAP_RANGE, 11391 },
	{ 0x5805, 0x5807, PDF_CMAP_TABLE, 1707 },
	{ 0x5808, 0x5809, PDF_CMAP_RANGE, 11393 },
	{ 0x580a, 0x580d, PDF_CMAP_TABLE, 1710 },
	{ 0x580e, 0x5810, PDF_CMAP_RANGE, 11396 },
	{ 0x5811, 0x5811, PDF_CMAP_SINGLE, 3122 },
	{ 0x5812, 0x5814, PDF_CMAP_RANGE, 11399 },
	{ 0x5815, 0x5816, PDF_CMAP_TABLE, 1714 },
	{ 0x5817, 0x5818, PDF_CMAP_RANGE, 11402 },
	{ 0x5819, 0x5819, PDF_CMAP_SINGLE, 5077 },
	{ 0x581a, 0x581c, PDF_CMAP_RANGE, 11404 },
	{ 0x581d, 0x5821, PDF_CMAP_TABLE, 1716 },
	{ 0x5822, 0x5823, PDF_CMAP_RANGE, 11408 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 1453 },
	{ 0x5825, 0x5829, PDF_CMAP_RANGE, 11410 },
	{ 0x582a, 0x582a, PDF_CMAP_SINGLE, 2365 },
	{ 0x582b, 0x582e, PDF_CMAP_RANGE, 11415 },
	{ 0x582f, 0x5831, PDF_CMAP_TABLE, 1721 },
	{ 0x5832, 0x5833, PDF_CMAP_RANGE, 11419 },
	{ 0x5834, 0x5835, PDF_CMAP_TABLE, 1724 },
	{ 0x5836, 0x5843, PDF_CMAP_RANGE, 11421 },
	{ 0x5844, 0x5844, PDF_CMAP_SINGLE, 5078 },
	{ 0x5845, 0x5849, PDF_CMAP_RANGE, 11435 },
	{ 0x584a, 0x5854, PDF_CMAP_TABLE, 1726 },
	{ 0x5855, 0x5856, PDF_CMAP_RANGE, 11443 },
	{ 0x5857, 0x5858, PDF_CMAP_TABLE, 1737 },
	{ 0x5859, 0x585d, PDF_CMAP_RANGE, 11445 },
	{ 0x585e, 0x585e, PDF_CMAP_SINGLE, 3286 },
	{ 0x585f, 0x5861, PDF_CMAP_RANGE, 11450 },
	{ 0x5862, 0x5865, PDF_CMAP_TABLE, 1739 },
	{ 0x5866, 0x586a, PDF_CMAP_RANGE, 11454 },
	{ 0x586b, 0x586c, PDF_CMAP_TABLE, 1743 },
	{ 0x586d, 0x5874, PDF_CMAP_RANGE, 11459 },
	{ 0x5875, 0x5875, PDF_CMAP_SINGLE, 7805 },
	{ 0x5876, 0x5878, PDF_CMAP_RANGE, 11467 },
	{ 0x5879, 0x5879, PDF_CMAP_SINGLE, 8406 },
	{ 0x587a, 0x587d, PDF_CMAP_RANGE, 11470 },
	{ 0x587e, 0x5885, PDF_CMAP_TABLE, 1745 },
	{ 0x5886, 0x5888, PDF_CMAP_RANGE, 11477 },
	{ 0x5889, 0x588a, PDF_CMAP_TABLE, 1753 },
	{ 0x588b, 0x5891, PDF_CMAP_RANGE, 11480 },
	{ 0x5892, 0x5893, PDF_CMAP_TABLE, 1755 },
	{ 0x5894, 0x5898, PDF_CMAP_RANGE, 11487 },
	{ 0x5899, 0x589f, PDF_CMAP_TABLE, 1757 },
	{ 0x58a0, 0x58a7, PDF_CMAP_RANGE, 11494 },
	{ 0x58a8, 0x58a9, PDF_CMAP_TABLE, 1764 },
	{ 0x58aa, 0x58ad, PDF_CMAP_RANGE, 11502 },
	{ 0x58ae, 0x58ae, PDF_CMAP_SINGLE, 7903 },
	{ 0x58af, 0x58b2, PDF_CMAP_RANGE, 11506 },
	{ 0x58b3, 0x58b3, PDF_CMAP_SINGLE, 7929 },
	{ 0x58b4, 0x58ba, PDF_CMAP_RANGE, 11510 },
	{ 0x58bb, 0x58be, PDF_CMAP_TABLE, 1766 },
	{ 0x58bf, 0x58c0, PDF_CMAP_RANGE, 11518 },
	{ 0x58c1, 0x58c1, PDF_CMAP_SINGLE, 1091 },
	{ 0x58c2, 0x58c4, PDF_CMAP_RANGE, 11520 },
	{ 0x58c5, 0x58c7, PDF_CMAP_TABLE, 1770 },
	{ 0x58c8, 0x58d0, PDF_CMAP_RANGE, 11524 },
	{ 0x58d1, 0x58d5, PDF_CMAP_TABLE, 1773 },
	{ 0x58d6, 0x58d7, PDF_CMAP_RANGE, 11535 },
	{ 0x58d8, 0x58da, PDF_CMAP_TABLE, 1778 },
	{ 0x58db, 0x58dd, PDF_CMAP_RANGE, 11537 },
	{ 0x58de, 0x58e4, PDF_CMAP_TABLE, 1781 },
	{ 0x58e5, 0x58e8, PDF_CMAP_RANGE, 11542 },
	{ 0x58e9, 0x58f0, PDF_CMAP_TABLE, 1788 },
	{ 0x58f1, 0x58f2, PDF_CMAP_RANGE, 11548 },
	{ 0x58f3, 0x58f3, PDF_CMAP_SINGLE, 2388 },
	{ 0x58f4, 0x58f5, PDF_CMAP_RANGE, 11550 },
	{ 0x58f6, 0x58f6, PDF_CMAP_SINGLE, 1967 },
	{ 0x58f7, 0x58f8, PDF_CMAP_RANGE, 11552 },
	{ 0x58f9, 0x58fa, PDF_CMAP_TABLE, 1796 },
	{ 0x58fb, 0x58fc, PDF_CMAP_RANGE, 11554 },
	{ 0x58fd, 0x58fd, PDF_CMAP_SINGLE, 8504 },
	{ 0x58fe, 0x5901, PDF_CMAP_RANGE, 11556 },
	{ 0x5902, 0x5904, PDF_CMAP_TABLE, 1798 },
	{ 0x5905, 0x5906, PDF_CMAP_RANGE, 11561 },
	{ 0x5907, 0x5907, PDF_CMAP_SINGLE, 1057 },
	{ 0x5908, 0x590c, PDF_CMAP_RANGE, 11563 },
	{ 0x590d, 0x590f, PDF_CMAP_TABLE, 1801 },
	{ 0x5910, 0x5913, PDF_CMAP_RANGE, 11569 },
	{ 0x5914, 0x5916, PDF_CMAP_TABLE, 1804 },
	{ 0x5917, 0x5918, PDF_CMAP_RANGE, 11573 },
	{ 0x5919, 0x591c, PDF_CMAP_TABLE, 1807 },
	{ 0x591d, 0x591e, PDF_CMAP_RANGE, 11576 },
	{ 0x591f, 0x591f, PDF_CMAP_SINGLE, 1812 },
	{ 0x5920, 0x5921, PDF_CMAP_RANGE, 11578 },
	{ 0x5922, 0x5931, PDF_CMAP_TABLE, 1811 },
	{ 0x5932, 0x5933, PDF_CMAP_RANGE, 11585 },
	{ 0x5934, 0x5934, PDF_CMAP_SINGLE, 3688 },
	{ 0x5935, 0x5936, PDF_CMAP_RANGE, 11587 },
	{ 0x5937, 0x593e, PDF_CMAP_TABLE, 1827 },
	{ 0x593f, 0x5940, PDF_CMAP_RANGE, 11591 },
	{ 0x5941, 0x5944, PDF_CMAP_TABLE, 1835 },
	{ 0x5945, 0x5946, PDF_CMAP_RANGE, 11594 },
	{ 0x5947, 0x594b, PDF_CMAP_TABLE, 1839 },
	{ 0x594c, 0x594d, PDF_CMAP_RANGE, 11597 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 1844 },
	{ 0x5952, 0x5953, PDF_CMAP_RANGE, 11600 },
	{ 0x5954, 0x595a, PDF_CMAP_TABLE, 1848 },
	{ 0x595b, 0x595f, PDF_CMAP_RANGE, 11603 },
	{ 0x5960, 0x5962, PDF_CMAP_TABLE, 1855 },
	{ 0x5963, 0x5964, PDF_CMAP_RANGE, 11609 },
	{ 0x5965, 0x5965, PDF_CMAP_SINGLE, 973 },
	{ 0x5966, 0x5968, PDF_CMAP_RANGE, 11611 },
	{ 0x5969, 0x596e, PDF_CMAP_TABLE, 1858 },
	{ 0x596f, 0x5972, PDF_CMAP_RANGE, 11616 },
	{ 0x5973, 0x5979, PDF_CMAP_TABLE, 1864 },
	{ 0x597a, 0x597c, PDF_CMAP_RANGE, 11622 },
	{ 0x597d, 0x597d, PDF_CMAP_SINGLE, 1914 },
	{ 0x597e, 0x5980, PDF_CMAP_RANGE, 11625 },
	{ 0x5981, 0x598a, PDF_CMAP_TABLE, 1871 },
	{ 0x598b, 0x598c, PDF_CMAP_RANGE, 11630 },
	{ 0x598d, 0x598d, PDF_CMAP_SINGLE, 6007 },
	{ 0x598e, 0x5991, PDF_CMAP_RANGE, 11632 },
	{ 0x5992, 0x5993, PDF_CMAP_TABLE, 1881 },
	{ 0x5994, 0x5995, PDF_CMAP_RANGE, 11636 },
	{ 0x5996, 0x5999, PDF_CMAP_TABLE, 1883 },
	{ 0x599a, 0x599c, PDF_CMAP_RANGE, 11639 },
	{ 0x599d, 0x599e, PDF_CMAP_TABLE, 1887 },
	{ 0x599f, 0x59a2, PDF_CMAP_RANGE, 11642 },
	{ 0x59a3, 0x59a5, PDF_CMAP_TABLE, 1889 },
	{ 0x59a6, 0x59a7, PDF_CMAP_RANGE, 11646 },
	{ 0x59a8, 0x59a8, PDF_CMAP_SINGLE, 1630 },
	{ 0x59a9, 0x59aa, PDF_CMAP_RANGE, 6008 },
	{ 0x59ab, 0x59ab, PDF_CMAP_SINGLE, 6013 },
	{ 0x59ac, 0x59ad, PDF_CMAP_RANGE, 11648 },
	{ 0x59ae, 0x59af, PDF_CMAP_TABLE, 1892 },
	{ 0x59b0, 0x59b1, PDF_CMAP_RANGE, 11650 },
	{ 0x59b2, 0x59b2, PDF_CMAP_SINGLE, 6017 },
	{ 0x59b3, 0x59b8, PDF_CMAP_RANGE, 11652 },
	{ 0x59b9, 0x59bb, PDF_CMAP_TABLE, 1894 },
	{ 0x59bc, 0x59bd, PDF_CMAP_RANGE, 11659 },
	{ 0x59be, 0x59be, PDF_CMAP_SINGLE, 6020 },
	{ 0x59bf, 0x59c5, PDF_CMAP_RANGE, 11661 },
	{ 0x59c6, 0x59c6, PDF_CMAP_SINGLE, 2842 },
	{ 0x59c7, 0x59c9, PDF_CMAP_RANGE, 11668 },
	{ 0x59ca, 0x59cb, PDF_CMAP_TABLE, 1897 },
	{ 0x59cc, 0x59cf, PDF_CMAP_RANGE, 11671 },
	{ 0x59d0, 0x59d4, PDF_CMAP_TABLE, 1899 },
	{ 0x59d5, 0x59d6, PDF_CMAP_RANGE, 11675 },
	{ 0x59d7, 0x59dd, PDF_CMAP_TABLE, 1904 },
	{ 0x59de, 0x59e2, PDF_CMAP_RANGE, 11679 },
	{ 0x59e3, 0x59e5, PDF_CMAP_TABLE, 1911 },
	{ 0x59e6, 0x59e7, PDF_CMAP_RANGE, 11685 },
	{ 0x59e8, 0x59e8, PDF_CMAP_SINGLE, 4179 },
	{ 0x59e9, 0x59eb, PDF_CMAP_RANGE, 11687 },
	{ 0x59ec, 0x59ec, PDF_CMAP_SINGLE, 2074 },
	{ 0x59ed, 0x59f8, PDF_CMAP_RANGE, 11690 },
	{ 0x59f9, 0x59fb, PDF_CMAP_TABLE, 1914 },
	{ 0x59fc, 0x59fe, PDF_CMAP_RANGE, 11703 },
	{ 0x59ff, 0x5a04, PDF_CMAP_TABLE, 1917 },
	{ 0x5a05, 0x5a06, PDF_CMAP_RANGE, 6021 },
	{ 0x5a07, 0x5a09, PDF_CMAP_TABLE, 1923 },
	{ 0x5a0a, 0x5a0b, PDF_CMAP_RANGE, 11708 },
	{ 0x5a0c, 0x5a0c, PDF_CMAP_SINGLE, 6028 },
	{ 0x5a0d, 0x5a10, PDF_CMAP_RANGE, 11710 },
	{ 0x5a11, 0x5a13, PDF_CMAP_TABLE, 1926 },
	{ 0x5a14, 0x5a17, PDF_CMAP_RANGE, 11715 },
	{ 0x5a18, 0x5a18, PDF_CMAP_SINGLE, 2898 },
	{ 0x5a19, 0x5a1b, PDF_CMAP_RANGE, 11719 },
	{ 0x5a1c, 0x5a1c, PDF_CMAP_SINGLE, 2859 },
	{ 0x5a1d, 0x5a1e, PDF_CMAP_RANGE, 11722 },
	{ 0x5a1f, 0x5a20, PDF_CMAP_TABLE, 1929 },
	{ 0x5a21, 0x5a22, PDF_CMAP_RANGE, 11724 },
	{ 0x5a23, 0x5a25, PDF_CMAP_TABLE, 1931 },
	{ 0x5a26, 0x5a28, PDF_CMAP_RANGE, 11727 },
	{ 0x5a29, 0x5a29, PDF_CMAP_SINGLE, 2793 },
	{ 0x5a2a, 0x5a30, PDF_CMAP_RANGE, 11730 },
	{ 0x5a31, 0x5a36, PDF_CMAP_TABLE, 1934 },
	{ 0x5a37, 0x5a3b, PDF_CMAP_RANGE, 11739 },
	{ 0x5a3c, 0x5a3c, PDF_CMAP_SINGLE, 6039 },
	{ 0x5a3d, 0x5a3f, PDF_CMAP_RANGE, 11744 },
	{ 0x5a40, 0x5a41, PDF_CMAP_TABLE, 1940 },
	{ 0x5a42, 0x5a45, PDF_CMAP_RANGE, 11747 },
	{ 0x5a46, 0x5a46, PDF_CMAP_SINGLE, 3044 },
	{ 0x5a47, 0x5a48, PDF_CMAP_RANGE, 11751 },
	{ 0x5a49, 0x5a4a, PDF_CMAP_TABLE, 1942 },
	{ 0x5a4b, 0x5a54, PDF_CMAP_RANGE, 11753 },
	{ 0x5a55, 0x5a55, PDF_CMAP_SINGLE, 6038 },
	{ 0x5a56, 0x5a59, PDF_CMAP_RANGE, 11763 },
	{ 0x5a5a, 0x5a5a, PDF_CMAP_SINGLE, 2046 },
	{ 0x5a5b, 0x5a61, PDF_CMAP_RANGE, 11767 },
	{ 0x5a62, 0x5a62, PDF_CMAP_SINGLE, 6040 },
	{ 0x5a63, 0x5a65, PDF_CMAP_RANGE, 11774 },
	{ 0x5a66, 0x5a67, PDF_CMAP_TABLE, 1944 },
	{ 0x5a68, 0x5a69, PDF_CMAP_RANGE, 11777 },
	{ 0x5a6a, 0x5a6a, PDF_CMAP_SINGLE, 2466 },
	{ 0x5a6b, 0x5a6c, PDF_CMAP_RANGE, 11779 },
	{ 0x5a6d, 0x5a6d, PDF_CMAP_SINGLE, 9209 },
	{ 0x5a6e, 0x5a73, PDF_CMAP_RANGE, 11781 },
	{ 0x5a74, 0x5a77, PDF_CMAP_TABLE, 1946 },
	{ 0x5a78, 0x5a79, PDF_CMAP_RANGE, 11787 },
	{ 0x5a7a, 0x5a7a, PDF_CMAP_SINGLE, 6046 },
	{ 0x5a7b, 0x5a7e, PDF_CMAP_RANGE, 11789 },
	{ 0x5a7f, 0x5a7f, PDF_CMAP_SINGLE, 4033 },
	{ 0x5a80, 0x5a91, PDF_CMAP_RANGE, 11793 },
	{ 0x5a92, 0x5a92, PDF_CMAP_SINGLE, 2754 },
	{ 0x5a93, 0x5a99, PDF_CMAP_RANGE, 11811 },
	{ 0x5a9a, 0x5a9b, PDF_CMAP_TABLE, 1950 },
	{ 0x5a9c, 0x5aa6, PDF_CMAP_RANGE, 11818 },
	{ 0x5aa7, 0x5aa7, PDF_CMAP_SINGLE, 9212 },
	{ 0x5aa8, 0x5aa9, PDF_CMAP_RANGE, 11829 },
	{ 0x5aaa, 0x5aaa, PDF_CMAP_SINGLE, 6043 },
	{ 0x5aab, 0x5ab1, PDF_CMAP_RANGE, 11831 },
	{ 0x5ab2, 0x5ab5, PDF_CMAP_TABLE, 1952 },
	{ 0x5ab6, 0x5ab7, PDF_CMAP_RANGE, 11839 },
	{ 0x5ab8, 0x5ab8, PDF_CMAP_SINGLE, 6052 },
	{ 0x5ab9, 0x5abc, PDF_CMAP_RANGE, 11841 },
	{ 0x5abd, 0x5abe, PDF_CMAP_TABLE, 1956 },
	{ 0x5abf, 0x5ac0, PDF_CMAP_RANGE, 11845 },
	{ 0x5ac1, 0x5ac2, PDF_CMAP_TABLE, 1958 },
	{ 0x5ac3, 0x5ac8, PDF_CMAP_RANGE, 11847 },
	{ 0x5ac9, 0x5ac9, PDF_CMAP_SINGLE, 2088 },
	{ 0x5aca, 0x5acb, PDF_CMAP_RANGE, 11853 },
	{ 0x5acc, 0x5acc, PDF_CMAP_SINGLE, 3904 },
	{ 0x5acd, 0x5ad1, PDF_CMAP_RANGE, 11855 },
	{ 0x5ad2, 0x5ad8, PDF_CMAP_TABLE, 1960 },
	{ 0x5ad9, 0x5adb, PDF_CMAP_RANGE, 11862 },
	{ 0x5adc, 0x5adc, PDF_CMAP_SINGLE, 6059 },
	{ 0x5add, 0x5adf, PDF_CMAP_RANGE, 11865 },
	{ 0x5ae0, 0x5ae3, PDF_CMAP_TABLE, 1967 },
	{ 0x5ae4, 0x5ae5, PDF_CMAP_RANGE, 11869 },
	{ 0x5ae6, 0x5ae6, PDF_CMAP_SINGLE, 6057 },
	{ 0x5ae7, 0x5ae8, PDF_CMAP_RANGE, 11871 },
	{ 0x5ae9, 0x5aeb, PDF_CMAP_TABLE, 1971 },
	{ 0x5aec, 0x5af0, PDF_CMAP_RANGE, 11874 },
	{ 0x5af1, 0x5af1, PDF_CMAP_SINGLE, 6055 },
	{ 0x5af2, 0x5af4, PDF_CMAP_RANGE, 11879 },
	{ 0x5af5, 0x5af5, PDF_CMAP_SINGLE, 9206 },
	{ 0x5af6, 0x5afa, PDF_CMAP_RANGE, 11882 },
	{ 0x5afb, 0x5afb, PDF_CMAP_SINGLE, 9213 },
	{ 0x5afc, 0x5aff, PDF_CMAP_RANGE, 11887 },
	{ 0x5b00, 0x5b00, PDF_CMAP_SINGLE, 9208 },
	{ 0x5b01, 0x5b07, PDF_CMAP_RANGE, 11891 },
	{ 0x5b08, 0x5b0c, PDF_CMAP_TABLE, 1974 },
	{ 0x5b0d, 0x5b15, PDF_CMAP_RANGE, 11899 },
	{ 0x5b16, 0x5b19, PDF_CMAP_TABLE, 1979 },
	{ 0x5b1a, 0x5b20, PDF_CMAP_RANGE, 11909 },
	{ 0x5b21, 0x5b21, PDF_CMAP_SINGLE, 9215 },
	{ 0x5b22, 0x5b29, PDF_CMAP_RANGE, 11916 },
	{ 0x5b2a, 0x5b2a, PDF_CMAP_SINGLE, 9216 },
	{ 0x5b2b, 0x5b2f, PDF_CMAP_RANGE, 11924 },
	{ 0x5b30, 0x5b34, PDF_CMAP_TABLE, 1983 },
	{ 0x5b35, 0x5b36, PDF_CMAP_RANGE, 11931 },
	{ 0x5b37, 0x5b38, PDF_CMAP_TABLE, 1988 },
	{ 0x5b39, 0x5b3f, PDF_CMAP_RANGE, 11933 },
	{ 0x5b40, 0x5b40, PDF_CMAP_SINGLE, 6065 },
	{ 0x5b41, 0x5b4b, PDF_CMAP_RANGE, 11940 },
	{ 0x5b4c, 0x5b4c, PDF_CMAP_SINGLE, 9211 },
	{ 0x5b4d, 0x5b4f, PDF_CMAP_RANGE, 11951 },
	{ 0x5b50, 0x5b5f, PDF_CMAP_TABLE, 1990 },
	{ 0x5b60, 0x5b61, PDF_CMAP_RANGE, 11957 },
	{ 0x5b62, 0x5b66, PDF_CMAP_TABLE, 2006 },
	{ 0x5b67, 0x5b68, PDF_CMAP_RANGE, 11959 },
	{ 0x5b69, 0x5b6c, PDF_CMAP_TABLE, 2011 },
	{ 0x5b6d, 0x5b6f, PDF_CMAP_RANGE, 11961 },
	{ 0x5b70, 0x5b75, PDF_CMAP_TABLE, 2015 },
	{ 0x5b76, 0x5b77, PDF_CMAP_RANGE, 11966 },
	{ 0x5b78, 0x5b7a, PDF_CMAP_TABLE, 2021 },
	{ 0x5b7b, 0x5b7c, PDF_CMAP_RANGE, 11969 },
	{ 0x5b7d, 0x5b8c, PDF_CMAP_TABLE, 2024 },
	{ 0x5b8d, 0x5b8e, PDF_CMAP_RANGE, 11975 },
	{ 0x5b8f, 0x5b8f, PDF_CMAP_SINGLE, 1953 },
	{ 0x5b90, 0x5b92, PDF_CMAP_RANGE, 11977 },
	{ 0x5b93, 0x5ba6, PDF_CMAP_TABLE, 2040 },
	{ 0x5ba7, 0x5ba9, PDF_CMAP_RANGE, 11983 },
	{ 0x5baa, 0x5bab, PDF_CMAP_TABLE, 2060 },
	{ 0x5bac, 0x5baf, PDF_CMAP_RANGE, 11986 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 4378 },
	{ 0x5bb1, 0x5bb2, PDF_CMAP_RANGE, 11990 },
	{ 0x5bb3, 0x5bb9, PDF_CMAP_TABLE, 2062 },
	{ 0x5bba, 0x5bbc, PDF_CMAP_RANGE, 11993 },
	{ 0x5bbd, 0x5bbf, PDF_CMAP_TABLE, 2069 },
	{ 0x5bc0, 0x5bc1, PDF_CMAP_RANGE, 11996 },
	{ 0x5bc2, 0x5bc7, PDF_CMAP_TABLE, 2072 },
	{ 0x5bc8, 0x5bcb, PDF_CMAP_RANGE, 11999 },
	{ 0x5bcc, 0x5bcc, PDF_CMAP_SINGLE, 1718 },
	{ 0x5bcd, 0x5bcf, PDF_CMAP_RANGE, 12003 },
	{ 0x5bd0, 0x5bd3, PDF_CMAP_TABLE, 2078 },
	{ 0x5bd4, 0x5bdc, PDF_CMAP_RANGE, 12007 },
	{ 0x5bdd, 0x5bf0, PDF_CMAP_TABLE, 2082 },
	{ 0x5bf1, 0x5bf4, PDF_CMAP_RANGE, 12021 },
	{ 0x5bf5, 0x5bfc, PDF_CMAP_TABLE, 2102 },
	{ 0x5bfd, 0x5bfe, PDF_CMAP_RANGE, 12026 },
	{ 0x5bff, 0x5c01, PDF_CMAP_TABLE, 2110 },
	{ 0x5c02, 0x5c03, PDF_CMAP_RANGE, 12029 },
	{ 0x5c04, 0x5c11, PDF_CMAP_TABLE, 2113 },
	{ 0x5c12, 0x5c13, PDF_CMAP_RANGE, 12034 },
	{ 0x5c14, 0x5c1d, PDF_CMAP_TABLE, 2127 },
	{ 0x5c1e, 0x5c21, PDF_CMAP_RANGE, 12039 },
	{ 0x5c22, 0x5c27, PDF_CMAP_TABLE, 2137 },
	{ 0x5c28, 0x5c2b, PDF_CMAP_RANGE, 12045 },
	{ 0x5c2c, 0x5c2c, PDF_CMAP_SINGLE, 5304 },
	{ 0x5c2d, 0x5c30, PDF_CMAP_RANGE, 12049 },
	{ 0x5c31, 0x5c31, PDF_CMAP_SINGLE, 2300 },
	{ 0x5c32, 0x5c33, PDF_CMAP_RANGE, 12053 },
	{ 0x5c34, 0x5c34, PDF_CMAP_SINGLE, 5305 },
	{ 0x5c35, 0x5c36, PDF_CMAP_RANGE, 12055 },
	{ 0x5c37, 0x5c42, PDF_CMAP_TABLE, 2143 },
	{ 0x5c43, 0x5c44, PDF_CMAP_RANGE, 12057 },
	{ 0x5c45, 0x5c45, PDF_CMAP_SINGLE, 2306 },
	{ 0x5c46, 0x5c47, PDF_CMAP_RANGE, 12059 },
	{ 0x5c48, 0x5c4b, PDF_CMAP_TABLE, 2155 },
	{ 0x5c4c, 0x5c4d, PDF_CMAP_RANGE, 12061 },
	{ 0x5c4e, 0x5c51, PDF_CMAP_TABLE, 2159 },
	{ 0x5c52, 0x5c54, PDF_CMAP_RANGE, 12063 },
	{ 0x5c55, 0x5c55, PDF_CMAP_SINGLE, 4442 },
	{ 0x5c56, 0x5c58, PDF_CMAP_RANGE, 12066 },
	{ 0x5c59, 0x5c59, PDF_CMAP_SINGLE, 5993 },
	{ 0x5c5a, 0x5c5d, PDF_CMAP_RANGE, 12069 },
	{ 0x5c5e, 0x5c68, PDF_CMAP_TABLE, 2163 },
	{ 0x5c69, 0x5c6b, PDF_CMAP_RANGE, 12075 },
	{ 0x5c6c, 0x5c71, PDF_CMAP_TABLE, 2174 },
	{ 0x5c72, 0x5c78, PDF_CMAP_RANGE, 12080 },
	{ 0x5c79, 0x5c7a, PDF_CMAP_TABLE, 2180 },
	{ 0x5c7b, 0x5c7e, PDF_CMAP_RANGE, 12087 },
	{ 0x5c7f, 0x5c82, PDF_CMAP_TABLE, 2182 },
	{ 0x5c83, 0x5c87, PDF_CMAP_RANGE, 12092 },
	{ 0x5c88, 0x5c88, PDF_CMAP_SINGLE, 5552 },
	{ 0x5c89, 0x5c8b, PDF_CMAP_RANGE, 12097 },
	{ 0x5c8c, 0x5c8d, PDF_CMAP_TABLE, 2186 },
	{ 0x5c8e, 0x5c8f, PDF_CMAP_RANGE, 12100 },
	{ 0x5c90, 0x5c91, PDF_CMAP_TABLE, 2188 },
	{ 0x5c92, 0x5c93, PDF_CMAP_RANGE, 12102 },
	{ 0x5c94, 0x5c97, PDF_CMAP_TABLE, 2190 },
	{ 0x5c98, 0x5c99, PDF_CMAP_RANGE, 5553 },
	{ 0x5c9a, 0x5c9c, PDF_CMAP_TABLE, 2194 },
	{ 0x5c9d, 0x5ca0, PDF_CMAP_RANGE, 12105 },
	{ 0x5ca1, 0x5ca3, PDF_CMAP_TABLE, 2197 },
	{ 0x5ca4, 0x5ca8, PDF_CMAP_RANGE, 12109 },
	{ 0x5ca9, 0x5cad, PDF_CMAP_TABLE, 2200 },
	{ 0x5cae, 0x5cb0, PDF_CMAP_RANGE, 12115 },
	{ 0x5cb1, 0x5cb8, PDF_CMAP_TABLE, 2205 },
	{ 0x5cb9, 0x5cbc, PDF_CMAP_RANGE, 12121 },
	{ 0x5cbd, 0x5cc1, PDF_CMAP_TABLE, 2213 },
	{ 0x5cc2, 0x5cc3, PDF_CMAP_RANGE, 12127 },
	{ 0x5cc4, 0x5cc4, PDF_CMAP_SINGLE, 5567 },
	{ 0x5cc5, 0x5cca, PDF_CMAP_RANGE, 12129 },
	{ 0x5ccb, 0x5ccb, PDF_CMAP_SINGLE, 5570 },
	{ 0x5ccc, 0x5cd1, PDF_CMAP_RANGE, 12135 },
	{ 0x5cd2, 0x5cd2, PDF_CMAP_SINGLE, 5568 },
	{ 0x5cd3, 0x5cd8, PDF_CMAP_RANGE, 12141 },
	{ 0x5cd9, 0x5cd9, PDF_CMAP_SINGLE, 4548 },
	{ 0x5cda, 0x5ce0, PDF_CMAP_RANGE, 12147 },
	{ 0x5ce1, 0x5ce1, PDF_CMAP_SINGLE, 3884 },
	{ 0x5ce2, 0x5ce3, PDF_CMAP_RANGE, 12154 },
	{ 0x5ce4, 0x5cea, PDF_CMAP_TABLE, 2218 },
	{ 0x5ceb, 0x5cec, PDF_CMAP_RANGE, 12158 },
	{ 0x5ced, 0x5ced, PDF_CMAP_SINGLE, 3146 },
	{ 0x5cee, 0x5cef, PDF_CMAP_RANGE, 12160 },
	{ 0x5cf0, 0x5cf0, PDF_CMAP_SINGLE, 1666 },
	{ 0x5cf1, 0x5cf3, PDF_CMAP_RANGE, 12162 },
	{ 0x5cf4, 0x5cf6, PDF_CMAP_TABLE, 2225 },
	{ 0x5cf7, 0x5cfa, PDF_CMAP_RANGE, 12166 },
	{ 0x5cfb, 0x5cfd, PDF_CMAP_TABLE, 2228 },
	{ 0x5cfe, 0x5d01, PDF_CMAP_RANGE, 12171 },
	{ 0x5d02, 0x5d03, PDF_CMAP_RANGE, 5572 },
	{ 0x5d04, 0x5d05, PDF_CMAP_RANGE, 12175 },
	{ 0x5d06, 0x5d07, PDF_CMAP_TABLE, 2231 },
	{ 0x5d08, 0x5d0c, PDF_CMAP_RANGE, 12177 },
	{ 0x5d0d, 0x5d0e, PDF_CMAP_TABLE, 2233 },
	{ 0x5d0f, 0x5d13, PDF_CMAP_RANGE, 12182 },
	{ 0x5d14, 0x5d17, PDF_CMAP_TABLE, 2235 },
	{ 0x5d18, 0x5d1a, PDF_CMAP_RANGE, 12188 },
	{ 0x5d1b, 0x5d1b, PDF_CMAP_SINGLE, 5580 },
	{ 0x5d1c, 0x5d1d, PDF_CMAP_RANGE, 12191 },
	{ 0x5d1e, 0x5d1e, PDF_CMAP_SINGLE, 5578 },
	{ 0x5d1f, 0x5d23, PDF_CMAP_RANGE, 12193 },
	{ 0x5d24, 0x5d29, PDF_CMAP_TABLE, 2239 },
	{ 0x5d2a, 0x5d2b, PDF_CMAP_RANGE, 12200 },
	{ 0x5d2c, 0x5d2e, PDF_CMAP_TABLE, 2245 },
	{ 0x5d2f, 0x5d33, PDF_CMAP_RANGE, 12202 },
	{ 0x5d34, 0x5d34, PDF_CMAP_SINGLE, 5583 },
	{ 0x5d35, 0x5d3c, PDF_CMAP_RANGE, 12207 },
	{ 0x5d3d, 0x5d3e, PDF_CMAP_TABLE, 2248 },
	{ 0x5d3f, 0x5d46, PDF_CMAP_RANGE, 12215 },
	{ 0x5d47, 0x5d47, PDF_CMAP_SINGLE, 6946 },
	{ 0x5d48, 0x5d49, PDF_CMAP_RANGE, 12223 },
	{ 0x5d4a, 0x5d4c, PDF_CMAP_TABLE, 2250 },
	{ 0x5d4d, 0x5d4f, PDF_CMAP_RANGE, 12225 },
	{ 0x5d50, 0x5d50, PDF_CMAP_SINGLE, 9098 },
	{ 0x5d51, 0x5d57, PDF_CMAP_RANGE, 12228 },
	{ 0x5d58, 0x5d58, PDF_CMAP_SINGLE, 5581 },
	{ 0x5d59, 0x5d5a, PDF_CMAP_RANGE, 12235 },
	{ 0x5d5b, 0x5d5d, PDF_CMAP_TABLE, 2253 },
	{ 0x5d5e, 0x5d68, PDF_CMAP_RANGE, 12238 },
	{ 0x5d69, 0x5d6c, PDF_CMAP_TABLE, 2256 },
	{ 0x5d6d, 0x5d6e, PDF_CMAP_RANGE, 12250 },
	{ 0x5d6f, 0x5d6f, PDF_CMAP_SINGLE, 5587 },
	{ 0x5d70, 0x5d73, PDF_CMAP_RANGE, 12252 },
	{ 0x5d74, 0x5d74, PDF_CMAP_SINGLE, 5593 },
	{ 0x5d75, 0x5d80, PDF_CMAP_RANGE, 12256 },
	{ 0x5d81, 0x5d84, PDF_CMAP_TABLE, 2260 },
	{ 0x5d85, 0x5d86, PDF_CMAP_RANGE, 12269 },
	{ 0x5d87, 0x5d87, PDF_CMAP_SINGLE, 9096 },
	{ 0x5d88, 0x5d96, PDF_CMAP_RANGE, 12271 },
	{ 0x5d97, 0x5d99, PDF_CMAP_TABLE, 2264 },
	{ 0x5d9a, 0x5d9c, PDF_CMAP_RANGE, 12287 },
	{ 0x5d9d, 0x5d9d, PDF_CMAP_SINGLE, 5596 },
	{ 0x5d9e, 0x5d9f, PDF_CMAP_RANGE, 12290 },
	{ 0x5da0, 0x5da0, PDF_CMAP_SINGLE, 9101 },
	{ 0x5da1, 0x5da6, PDF_CMAP_RANGE, 12292 },
	{ 0x5da7, 0x5da7, PDF_CMAP_SINGLE, 9100 },
	{ 0x5da8, 0x5db6, PDF_CMAP_RANGE, 12298 },
	{ 0x5db7, 0x5dbc, PDF_CMAP_TABLE, 2267 },
	{ 0x5dbd, 0x5dc4, PDF_CMAP_RANGE, 12315 },
	{ 0x5dc5, 0x5dc5, PDF_CMAP_SINGLE, 5599 },
	{ 0x5dc6, 0x5dca, PDF_CMAP_RANGE, 12323 },
	{ 0x5dcb, 0x5dcd, PDF_CMAP_TABLE, 2273 },
	{ 0x5dce, 0x5dd1, PDF_CMAP_RANGE, 12329 },
	{ 0x5dd2, 0x5dd4, PDF_CMAP_TABLE, 2276 },
	{ 0x5dd5, 0x5dda, PDF_CMAP_RANGE, 12334 },
	{ 0x5ddb, 0x5dde, PDF_CMAP_TABLE, 2279 },
	{ 0x5ddf, 0x5de0, PDF_CMAP_RANGE, 12341 },
	{ 0x5de1, 0x5de2, PDF_CMAP_TABLE, 2283 },
	{ 0x5de3, 0x5de4, PDF_CMAP_RANGE, 12343 },
	{ 0x5de5, 0x5deb, PDF_CMAP_TABLE, 2285 },
	{ 0x5dec, 0x5ded, PDF_CMAP_RANGE, 12346 },
	{ 0x5dee, 0x5df4, PDF_CMAP_TABLE, 2292 },
	{ 0x5df5, 0x5df6, PDF_CMAP_RANGE, 12348 },
	{ 0x5df7, 0x5df7, PDF_CMAP_SINGLE, 3932 },
	{ 0x5df8, 0x5dfc, PDF_CMAP_RANGE, 12350 },
	{ 0x5dfd, 0x5e08, PDF_CMAP_TABLE, 2299 },
	{ 0x5e09, 0x5e0b, PDF_CMAP_RANGE, 12359 },
	{ 0x5e0c, 0x5e0c, PDF_CMAP_SINGLE, 3856 },
	{ 0x5e0d, 0x5e0e, PDF_CMAP_RANGE, 12362 },
	{ 0x5e0f, 0x5e11, PDF_CMAP_TABLE, 2311 },
	{ 0x5e12, 0x5e13, PDF_CMAP_RANGE, 12364 },
	{ 0x5e14, 0x5e1d, PDF_CMAP_TABLE, 2314 },
	{ 0x5e1e, 0x5e24, PDF_CMAP_RANGE, 12367 },
	{ 0x5e25, 0x5e27, PDF_CMAP_TABLE, 2324 },
	{ 0x5e28, 0x5e2a, PDF_CMAP_RANGE, 12374 },
	{ 0x5e2b, 0x5e2e, PDF_CMAP_TABLE, 2327 },
	{ 0x5e2f, 0x5e30, PDF_CMAP_RANGE, 12378 },
	{ 0x5e31, 0x5e33, PDF_CMAP_TABLE, 2331 },
	{ 0x5e34, 0x5e35, PDF_CMAP_RANGE, 12381 },
	{ 0x5e36, 0x5e38, PDF_CMAP_TABLE, 2334 },
	{ 0x5e39, 0x5e3a, PDF_CMAP_RANGE, 12383 },
	{ 0x5e3b, 0x5e3c, PDF_CMAP_RANGE, 5539 },
	{ 0x5e3d, 0x5e3d, PDF_CMAP_SINGLE, 2742 },
	{ 0x5e3e, 0x5e3f, PDF_CMAP_RANGE, 12385 },
	{ 0x5e40, 0x5e45, PDF_CMAP_TABLE, 2337 },
	{ 0x5e46, 0x5e4b, PDF_CMAP_RANGE, 12388 },
	{ 0x5e4c, 0x5e4c, PDF_CMAP_SINGLE, 2020 },
	{ 0x5e4d, 0x5e53, PDF_CMAP_RANGE, 12394 },
	{ 0x5e54, 0x5e58, PDF_CMAP_TABLE, 2343 },
	{ 0x5e59, 0x5e5a, PDF_CMAP_RANGE, 12402 },
	{ 0x5e5b, 0x5e5b, PDF_CMAP_SINGLE, 5544 },
	{ 0x5e5c, 0x5e5d, PDF_CMAP_RANGE, 12404 },
	{ 0x5e5e, 0x5e63, PDF_CMAP_TABLE, 2348 },
	{ 0x5e64, 0x5e6a, PDF_CMAP_RANGE, 12407 },
	{ 0x5e6b, 0x5e6c, PDF_CMAP_TABLE, 2354 },
	{ 0x5e6d, 0x5e71, PDF_CMAP_RANGE, 12414 },
	{ 0x5e72, 0x5e80, PDF_CMAP_TABLE, 2356 },
	{ 0x5e81, 0x5e83, PDF_CMAP_RANGE, 12421 },
	{ 0x5e84, 0x5e87, PDF_CMAP_TABLE, 2371 },
	{ 0x5e88, 0x5e89, PDF_CMAP_RANGE, 12425 },
	{ 0x5e8a, 0x5e8b, PDF_CMAP_TABLE, 2375 },
	{ 0x5e8c, 0x5e8e, PDF_CMAP_RANGE, 12427 },
	{ 0x5e8f, 0x5ea0, PDF_CMAP_TABLE, 2377 },
	{ 0x5ea1, 0x5ea4, PDF_CMAP_RANGE, 12434 },
	{ 0x5ea5, 0x5ea7, PDF_CMAP_TABLE, 2395 },
	{ 0x5ea8, 0x5eaa, PDF_CMAP_RANGE, 12438 },
	{ 0x5eab, 0x5ead, PDF_CMAP_TABLE, 2398 },
	{ 0x5eae, 0x5eb2, PDF_CMAP_RANGE, 12442 },
	{ 0x5eb3, 0x5eb9, PDF_CMAP_TABLE, 2401 },
	{ 0x5eba, 0x5ebd, PDF_CMAP_RANGE, 12448 },
	{ 0x5ebe, 0x5ebe, PDF_CMAP_SINGLE, 5689 },
	{ 0x5ebf, 0x5ec8, PDF_CMAP_RANGE, 12452 },
	{ 0x5ec9, 0x5eca, PDF_CMAP_TABLE, 2408 },
	{ 0x5ecb, 0x5ed0, PDF_CMAP_RANGE, 12462 },
	{ 0x5ed1, 0x5ed3, PDF_CMAP_TABLE, 2410 },
	{ 0x5ed4, 0x5ed5, PDF_CMAP_RANGE, 12468 },
	{ 0x5ed6, 0x5ed6, PDF_CMAP_SINGLE, 2583 },
	{ 0x5ed7, 0x5eda, PDF_CMAP_RANGE, 12470 },
	{ 0x5edb, 0x5edb, PDF_CMAP_SINGLE, 5694 },
	{ 0x5edc, 0x5ede, PDF_CMAP_RANGE, 12474 },
	{ 0x5edf, 0x5ee3, PDF_CMAP_TABLE, 2413 },
	{ 0x5ee4, 0x5ee7, PDF_CMAP_RANGE, 12477 },
	{ 0x5ee8, 0x5eec, PDF_CMAP_TABLE, 2418 },
	{ 0x5eed, 0x5ef2, PDF_CMAP_RANGE, 12483 },
	{ 0x5ef3, 0x5ef7, PDF_CMAP_TABLE, 2423 },
	{ 0x5ef8, 0x5ef9, PDF_CMAP_RANGE, 12490 },
	{ 0x5efa, 0x5efa, PDF_CMAP_SINGLE, 2169 },
	{ 0x5efb, 0x5efd, PDF_CMAP_RANGE, 12492 },
	{ 0x5efe, 0x5f04, PDF_CMAP_TABLE, 2428 },
	{ 0x5f05, 0x5f07, PDF_CMAP_RANGE, 12495 },
	{ 0x5f08, 0x5f0b, PDF_CMAP_TABLE, 2435 },
	{ 0x5f0c, 0x5f0e, PDF_CMAP_RANGE, 12499 },
	{ 0x5f0f, 0x5f18, PDF_CMAP_TABLE, 2439 },
	{ 0x5f19, 0x5f1a, PDF_CMAP_RANGE, 12506 },
	{ 0x5f1b, 0x5f1b, PDF_CMAP_SINGLE, 1279 },
	{ 0x5f1c, 0x5f1e, PDF_CMAP_RANGE, 12508 },
	{ 0x5f1f, 0x5f20, PDF_CMAP_TABLE, 2449 },
	{ 0x5f21, 0x5f24, PDF_CMAP_RANGE, 12511 },
	{ 0x5f25, 0x5f2a, PDF_CMAP_TABLE, 2451 },
	{ 0x5f2b, 0x5f2c, PDF_CMAP_RANGE, 12516 },
	{ 0x5f2d, 0x5f35, PDF_CMAP_TABLE, 2457 },
	{ 0x5f36, 0x5f38, PDF_CMAP_RANGE, 12522 },
	{ 0x5f39, 0x5f3c, PDF_CMAP_TABLE, 2466 },
	{ 0x5f3d, 0x5f3f, PDF_CMAP_RANGE, 12526 },
	{ 0x5f40, 0x5f40, PDF_CMAP_SINGLE, 6590 },
	{ 0x5f41, 0x5f45, PDF_CMAP_RANGE, 12529 },
	{ 0x5f46, 0x5f48, PDF_CMAP_TABLE, 2470 },
	{ 0x5f49, 0x5f4b, PDF_CMAP_RANGE, 12535 },
	{ 0x5f4c, 0x5f59, PDF_CMAP_TABLE, 2473 },
	{ 0x5f5a, 0x5f5c, PDF_CMAP_RANGE, 12542 },
	{ 0x5f5d, 0x5f5d, PDF_CMAP_SINGLE, 4180 },
	{ 0x5f5e, 0x5f60, PDF_CMAP_RANGE, 12545 },
	{ 0x5f61, 0x5f66, PDF_CMAP_TABLE, 2487 },
	{ 0x5f67, 0x5f68, PDF_CMAP_RANGE, 12550 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 2493 },
	{ 0x5f6e, 0x5f6f, PDF_CMAP_RANGE, 12553 },
	{ 0x5f70, 0x5f73, PDF_CMAP_TABLE, 2498 },
	{ 0x5f74, 0x5f76, PDF_CMAP_RANGE, 12556 },
	{ 0x5f77, 0x5f7c, PDF_CMAP_TABLE, 2502 },
	{ 0x5f7d, 0x5f7f, PDF_CMAP_RANGE, 12561 },
	{ 0x5f80, 0x5f8c, PDF_CMAP_TABLE, 2508 },
	{ 0x5f8d, 0x5f8f, PDF_CMAP_RANGE, 12566 },
	{ 0x5f90, 0x5f92, PDF_CMAP_TABLE, 2521 },
	{ 0x5f93, 0x5f94, PDF_CMAP_RANGE, 12569 },
	{ 0x5f95, 0x5f99, PDF_CMAP_TABLE, 2524 },
	{ 0x5f9a, 0x5f9b, PDF_CMAP_RANGE, 12572 },
	{ 0x5f9c, 0x5fa1, PDF_CMAP_TABLE, 2529 },
	{ 0x5fa2, 0x5fa7, PDF_CMAP_RANGE, 12576 },
	{ 0x5fa8, 0x5faa, PDF_CMAP_TABLE, 2535 },
	{ 0x5fab, 0x5fac, PDF_CMAP_RANGE, 12582 },
	{ 0x5fad, 0x5fae, PDF_CMAP_TABLE, 2538 },
	{ 0x5faf, 0x5fb4, PDF_CMAP_RANGE, 12584 },
	{ 0x5fb5, 0x5fb9, PDF_CMAP_TABLE, 2540 },
	{ 0x5fba, 0x5fbb, PDF_CMAP_RANGE, 12592 },
	{ 0x5fbc, 0x5fbd, PDF_CMAP_TABLE, 2545 },
	{ 0x5fbe, 0x5fc2, PDF_CMAP_RANGE, 12594 },
	{ 0x5fc3, 0x5fc6, PDF_CMAP_TABLE, 2547 },
	{ 0x5fc7, 0x5fc8, PDF_CMAP_RANGE, 12599 },
	{ 0x5fc9, 0x5fc9, PDF_CMAP_SINGLE, 5699 },
	{ 0x5fca, 0x5fcb, PDF_CMAP_RANGE, 12601 },
	{ 0x5fcc, 0x5fd2, PDF_CMAP_TABLE, 2551 },
	{ 0x5fd3, 0x5fd5, PDF_CMAP_RANGE, 12604 },
	{ 0x5fd6, 0x5fd9, PDF_CMAP_TABLE, 2558 },
	{ 0x5fda, 0x5fdc, PDF_CMAP_RANGE, 12607 },
	{ 0x5fdd, 0x5fdd, PDF_CMAP_SINGLE, 5763 },
	{ 0x5fde, 0x5fdf, PDF_CMAP_RANGE, 12610 },
	{ 0x5fe0, 0x5fe1, PDF_CMAP_TABLE, 2562 },
	{ 0x5fe2, 0x5fe3, PDF_CMAP_RANGE, 12612 },
	{ 0x5fe4, 0x5fe4, PDF_CMAP_SINGLE, 5706 },
	{ 0x5fe5, 0x5fe6, PDF_CMAP_RANGE, 12614 },
	{ 0x5fe7, 0x5fe7, PDF_CMAP_SINGLE, 4268 },
	{ 0x5fe8, 0x5fe9, PDF_CMAP_RANGE, 12616 },
	{ 0x5fea, 0x5fee, PDF_CMAP_TABLE, 2564 },
	{ 0x5fef, 0x5ff0, PDF_CMAP_RANGE, 12619 },
	{ 0x5ff1, 0x5ff1, PDF_CMAP_SINGLE, 1253 },
	{ 0x5ff2, 0x5ff4, PDF_CMAP_RANGE, 12621 },
	{ 0x5ff5, 0x5ff5, PDF_CMAP_SINGLE, 2897 },
	{ 0x5ff6, 0x5ff7, PDF_CMAP_RANGE, 12624 },
	{ 0x5ff8, 0x5ff8, PDF_CMAP_SINGLE, 5712 },
	{ 0x5ff9, 0x5ffa, PDF_CMAP_RANGE, 12626 },
	{ 0x5ffb, 0x6004, PDF_CMAP_TABLE, 2569 },
	{ 0x6005, 0x6006, PDF_CMAP_RANGE, 5708 },
	{ 0x6007, 0x6009, PDF_CMAP_RANGE, 12629 },
	{ 0x600a, 0x600a, PDF_CMAP_SINGLE, 5721 },
	{ 0x600b, 0x600c, PDF_CMAP_RANGE, 12632 },
	{ 0x600d, 0x600f, PDF_CMAP_TABLE, 2579 },
	{ 0x6010, 0x6011, PDF_CMAP_RANGE, 12634 },
	{ 0x6012, 0x6016, PDF_CMAP_TABLE, 2582 },
	{ 0x6017, 0x6018, PDF_CMAP_RANGE, 12637 },
	{ 0x6019, 0x601d, PDF_CMAP_TABLE, 2587 },
	{ 0x601e, 0x601f, PDF_CMAP_RANGE, 12640 },
	{ 0x6020, 0x6021, PDF_CMAP_TABLE, 2592 },
	{ 0x6022, 0x6024, PDF_CMAP_RANGE, 12642 },
	{ 0x6025, 0x602b, PDF_CMAP_TABLE, 2594 },
	{ 0x602c, 0x602e, PDF_CMAP_RANGE, 12645 },
	{ 0x602f, 0x602f, PDF_CMAP_SINGLE, 3152 },
	{ 0x6030, 0x6034, PDF_CMAP_RANGE, 12648 },
	{ 0x6035, 0x6035, PDF_CMAP_SINGLE, 5714 },
	{ 0x6036, 0x603a, PDF_CMAP_RANGE, 12653 },
	{ 0x603b, 0x603c, PDF_CMAP_TABLE, 2601 },
	{ 0x603d, 0x603e, PDF_CMAP_RANGE, 12658 },
	{ 0x603f, 0x6043, PDF_CMAP_TABLE, 2603 },
	{ 0x6044, 0x604a, PDF_CMAP_RANGE, 12661 },
	{ 0x604b, 0x604d, PDF_CMAP_TABLE, 2608 },
	{ 0x604e, 0x604f, PDF_CMAP_RANGE, 12669 },
	{ 0x6050, 0x6052, PDF_CMAP_TABLE, 2611 },
	{ 0x6053, 0x6054, PDF_CMAP_RANGE, 12672 },
	{ 0x6055, 0x6055, PDF_CMAP_SINGLE, 3478 },
	{ 0x6056, 0x6058, PDF_CMAP_RANGE, 12674 },
	{ 0x6059, 0x605a, PDF_CMAP_TABLE, 2614 },
	{ 0x605b, 0x605c, PDF_CMAP_RANGE, 12677 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 6670 },
	{ 0x605e, 0x6061, PDF_CMAP_RANGE, 12679 },
	{ 0x6062, 0x6064, PDF_CMAP_TABLE, 2616 },
	{ 0x6065, 0x6066, PDF_CMAP_RANGE, 12683 },
	{ 0x6067, 0x6070, PDF_CMAP_TABLE, 2619 },
	{ 0x6071, 0x6072, PDF_CMAP_RANGE, 12686 },
	{ 0x6073, 0x6073, PDF_CMAP_SINGLE, 2399 },
	{ 0x6074, 0x6075, PDF_CMAP_RANGE, 12688 },
	{ 0x6076, 0x6077, PDF_CMAP_TABLE, 2629 },
	{ 0x6078, 0x6079, PDF_CMAP_RANGE, 5724 },
	{ 0x607a, 0x607f, PDF_CMAP_TABLE, 2631 },
	{ 0x6080, 0x6082, PDF_CMAP_RANGE, 12692 },
	{ 0x6083, 0x6084, PDF_CMAP_TABLE, 2637 },
	{ 0x6085, 0x6088, PDF_CMAP_RANGE, 12695 },
	{ 0x6089, 0x6089, PDF_CMAP_SINGLE, 3857 },
	{ 0x608a, 0x608b, PDF_CMAP_RANGE, 12699 },
	{ 0x608c, 0x608d, PDF_CMAP_TABLE, 2639 },
	{ 0x608e, 0x6091, PDF_CMAP_RANGE, 12701 },
	{ 0x6092, 0x6096, PDF_CMAP_TABLE, 2641 },
	{ 0x6097, 0x6099, PDF_CMAP_RANGE, 12707 },
	{ 0x609a, 0x60a0, PDF_CMAP_TABLE, 2646 },
	{ 0x60a1, 0x60a2, PDF_CMAP_RANGE, 12712 },
	{ 0x60a3, 0x60a3, PDF_CMAP_SINGLE, 2001 },
	{ 0x60a4, 0x60a5, PDF_CMAP_RANGE, 12714 },
	{ 0x60a6, 0x60a8, PDF_CMAP_TABLE, 2653 },
	{ 0x60a9, 0x60aa, PDF_CMAP_RANGE, 12717 },
	{ 0x60ab, 0x60b8, PDF_CMAP_TABLE, 2656 },
	{ 0x60b9, 0x60ba, PDF_CMAP_RANGE, 12723 },
	{ 0x60bb, 0x60bc, PDF_CMAP_TABLE, 2670 },
	{ 0x60bd, 0x60c4, PDF_CMAP_RANGE, 12725 },
	{ 0x60c5, 0x60c6, PDF_CMAP_TABLE, 2672 },
	{ 0x60c7, 0x60c9, PDF_CMAP_RANGE, 12733 },
	{ 0x60ca, 0x60cb, PDF_CMAP_TABLE, 2674 },
	{ 0x60cc, 0x60d0, PDF_CMAP_RANGE, 12736 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 2056 },
	{ 0x60d2, 0x60d4, PDF_CMAP_RANGE, 12741 },
	{ 0x60d5, 0x60d5, PDF_CMAP_SINGLE, 3643 },
	{ 0x60d6, 0x60d7, PDF_CMAP_RANGE, 12744 },
	{ 0x60d8, 0x60e1, PDF_CMAP_TABLE, 2676 },
	{ 0x60e2, 0x60e5, PDF_CMAP_RANGE, 12749 },
	{ 0x60e6, 0x60f6, PDF_CMAP_TABLE, 2686 },
	{ 0x60f7, 0x60f8, PDF_CMAP_RANGE, 12755 },
	{ 0x60f9, 0x60fb, PDF_CMAP_TABLE, 2703 },
	{ 0x60fc, 0x60ff, PDF_CMAP_RANGE, 12757 },
	{ 0x6100, 0x6101, PDF_CMAP_TABLE, 2706 },
	{ 0x6102, 0x6105, PDF_CMAP_RANGE, 12761 },
	{ 0x6106, 0x6109, PDF_CMAP_TABLE, 2708 },
	{ 0x610a, 0x610c, PDF_CMAP_RANGE, 12766 },
	{ 0x610d, 0x610f, PDF_CMAP_TABLE, 2712 },
	{ 0x6110, 0x6114, PDF_CMAP_RANGE, 12769 },
	{ 0x6115, 0x6115, PDF_CMAP_SINGLE, 5749 },
	{ 0x6116, 0x6119, PDF_CMAP_RANGE, 12774 },
	{ 0x611a, 0x611c, PDF_CMAP_TABLE, 2715 },
	{ 0x611d, 0x611e, PDF_CMAP_RANGE, 12778 },
	{ 0x611f, 0x6120, PDF_CMAP_TABLE, 2718 },
	{ 0x6121, 0x6122, PDF_CMAP_RANGE, 12780 },
	{ 0x6123, 0x6127, PDF_CMAP_TABLE, 2720 },
	{ 0x6128, 0x612a, PDF_CMAP_RANGE, 12783 },
	{ 0x612b, 0x612b, PDF_CMAP_SINGLE, 5754 },
	{ 0x612c, 0x6133, PDF_CMAP_RANGE, 12786 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 9141 },
	{ 0x6135, 0x6136, PDF_CMAP_RANGE, 12794 },
	{ 0x6137, 0x6137, PDF_CMAP_SINGLE, 9146 },
	{ 0x6138, 0x613d, PDF_CMAP_RANGE, 12796 },
	{ 0x613e, 0x613f, PDF_CMAP_TABLE, 2725 },
	{ 0x6140, 0x6147, PDF_CMAP_RANGE, 12802 },
	{ 0x6148, 0x614e, PDF_CMAP_TABLE, 2727 },
	{ 0x614f, 0x6150, PDF_CMAP_RANGE, 12812 },
	{ 0x6151, 0x6151, PDF_CMAP_SINGLE, 3358 },
	{ 0x6152, 0x6154, PDF_CMAP_RANGE, 12814 },
	{ 0x6155, 0x6155, PDF_CMAP_SINGLE, 2848 },
	{ 0x6156, 0x6157, PDF_CMAP_RANGE, 12817 },
	{ 0x6158, 0x615a, PDF_CMAP_TABLE, 2734 },
	{ 0x615b, 0x615c, PDF_CMAP_RANGE, 12820 },
	{ 0x615d, 0x615f, PDF_CMAP_TABLE, 2737 },
	{ 0x6160, 0x6161, PDF_CMAP_RANGE, 12823 },
	{ 0x6162, 0x6164, PDF_CMAP_TABLE, 2740 },
	{ 0x6165, 0x6166, PDF_CMAP_RANGE, 12825 },
	{ 0x6167, 0x616b, PDF_CMAP_TABLE, 2743 },
	{ 0x616c, 0x616d, PDF_CMAP_RANGE, 12828 },
	{ 0x616e, 0x6170, PDF_CMAP_TABLE, 2748 },
	{ 0x6171, 0x6172, PDF_CMAP_RANGE, 12831 },
	{ 0x6173, 0x6177, PDF_CMAP_TABLE, 2751 },
	{ 0x6178, 0x6181, PDF_CMAP_RANGE, 12834 },
	{ 0x6182, 0x6182, PDF_CMAP_SINGLE, 8754 },
	{ 0x6183, 0x6189, PDF_CMAP_RANGE, 12844 },
	{ 0x618a, 0x618b, PDF_CMAP_TABLE, 2756 },
	{ 0x618c, 0x618d, PDF_CMAP_RANGE, 12851 },
	{ 0x618e, 0x6194, PDF_CMAP_TABLE, 2758 },
	{ 0x6195, 0x6199, PDF_CMAP_RANGE, 12855 },
	{ 0x619a, 0x619a, PDF_CMAP_SINGLE, 7854 },
	{ 0x619b, 0x619c, PDF_CMAP_RANGE, 12860 },
	{ 0x619d, 0x619d, PDF_CMAP_SINGLE, 6681 },
	{ 0x619e, 0x61a3, PDF_CMAP_RANGE, 12862 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 7931 },
	{ 0x61a5, 0x61a6, PDF_CMAP_RANGE, 12868 },
	{ 0x61a7, 0x61ae, PDF_CMAP_TABLE, 2765 },
	{ 0x61af, 0x61b1, PDF_CMAP_RANGE, 12872 },
	{ 0x61b2, 0x61b2, PDF_CMAP_SINGLE, 8644 },
	{ 0x61b3, 0x61b5, PDF_CMAP_RANGE, 12875 },
	{ 0x61b6, 0x61b7, PDF_CMAP_TABLE, 2773 },
	{ 0x61b8, 0x61bd, PDF_CMAP_RANGE, 12878 },
	{ 0x61be, 0x61be, PDF_CMAP_SINGLE, 1901 },
	{ 0x61bf, 0x61c1, PDF_CMAP_RANGE, 12884 },
	{ 0x61c2, 0x61c2, PDF_CMAP_SINGLE, 1517 },
	{ 0x61c3, 0x61c6, PDF_CMAP_RANGE, 12887 },
	{ 0x61c7, 0x61cc, PDF_CMAP_TABLE, 2775 },
	{ 0x61cd, 0x61d0, PDF_CMAP_RANGE, 12891 },
	{ 0x61d1, 0x61d4, PDF_CMAP_TABLE, 2781 },
	{ 0x61d5, 0x61dd, PDF_CMAP_RANGE, 12896 },
	{ 0x61de, 0x61df, PDF_CMAP_TABLE, 2785 },
	{ 0x61e0, 0x61e2, PDF_CMAP_RANGE, 12905 },
	{ 0x61e3, 0x61e3, PDF_CMAP_SINGLE, 9440 },
	{ 0x61e4, 0x61e5, PDF_CMAP_RANGE, 12908 },
	{ 0x61e6, 0x61e8, PDF_CMAP_TABLE, 2787 },
	{ 0x61e9, 0x61f1, PDF_CMAP_RANGE, 12911 },
	{ 0x61f2, 0x61f2, PDF_CMAP_SINGLE, 7809 },
	{ 0x61f3, 0x61f4, PDF_CMAP_RANGE, 12920 },
	{ 0x61f5, 0x6200, PDF_CMAP_TABLE, 2790 },
	{ 0x6201, 0x6205, PDF_CMAP_RANGE, 12925 },
	{ 0x6206, 0x621b, PDF_CMAP_TABLE, 2802 },
	{ 0x621c, 0x621e, PDF_CMAP_RANGE, 12933 },
	{ 0x621f, 0x6227, PDF_CMAP_TABLE, 2824 },
	{ 0x6228, 0x6229, PDF_CMAP_RANGE, 12939 },
	{ 0x622a, 0x6234, PDF_CMAP_TABLE, 2833 },
	{ 0x6235, 0x6236, PDF_CMAP_RANGE, 12945 },
	{ 0x6237, 0x6237, PDF_CMAP_SINGLE, 1980 },
	{ 0x6238, 0x623c, PDF_CMAP_RANGE, 12947 },
	{ 0x623d, 0x6243, PDF_CMAP_TABLE, 2844 },
	{ 0x6244, 0x6246, PDF_CMAP_RANGE, 12953 },
	{ 0x6247, 0x6247, PDF_CMAP_SINGLE, 3329 },
	{ 0x6248, 0x6249, PDF_CMAP_RANGE, 6646 },
	{ 0x624a, 0x624e, PDF_CMAP_TABLE, 2851 },
	{ 0x624f, 0x6250, PDF_CMAP_RANGE, 12957 },
	{ 0x6251, 0x6254, PDF_CMAP_TABLE, 2856 },
	{ 0x6255, 0x6257, PDF_CMAP_RANGE, 12959 },
	{ 0x6258, 0x6258, PDF_CMAP_SINGLE, 3713 },
	{ 0x6259, 0x625a, PDF_CMAP_RANGE, 12962 },
	{ 0x625b, 0x625b, PDF_CMAP_SINGLE, 2373 },
	{ 0x625c, 0x6262, PDF_CMAP_RANGE, 12964 },
	{ 0x6263, 0x6263, PDF_CMAP_SINGLE, 2408 },
	{ 0x6264, 0x6265, PDF_CMAP_RANGE, 12971 },
	{ 0x6266, 0x6270, PDF_CMAP_TABLE, 2860 },
	{ 0x6271, 0x6272, PDF_CMAP_RANGE, 12974 },
	{ 0x6273, 0x6273, PDF_CMAP_SINGLE, 1005 },
	{ 0x6274, 0x6275, PDF_CMAP_RANGE, 12976 },
	{ 0x6276, 0x6276, PDF_CMAP_SINGLE, 1683 },
	{ 0x6277, 0x6278, PDF_CMAP_RANGE, 12978 },
	{ 0x6279, 0x6279, PDF_CMAP_SINGLE, 3003 },
	{ 0x627a, 0x627b, PDF_CMAP_RANGE, 12980 },
	{ 0x627c, 0x6280, PDF_CMAP_TABLE, 2871 },
	{ 0x6281, 0x6283, PDF_CMAP_RANGE, 12983 },
	{ 0x6284, 0x6284, PDF_CMAP_SINGLE, 1234 },
	{ 0x6285, 0x6288, PDF_CMAP_RANGE, 12986 },
	{ 0x6289, 0x628a, PDF_CMAP_TABLE, 2876 },
	{ 0x628b, 0x6290, PDF_CMAP_RANGE, 12990 },
	{ 0x6291, 0x629b, PDF_CMAP_TABLE, 2878 },
	{ 0x629c, 0x629e, PDF_CMAP_RANGE, 12998 },
	{ 0x629f, 0x62a5, PDF_CMAP_TABLE, 2889 },
	{ 0x62a6, 0x62a7, PDF_CMAP_RANGE, 13002 },
	{ 0x62a8, 0x62a8, PDF_CMAP_SINGLE, 2987 },
	{ 0x62a9, 0x62aa, PDF_CMAP_RANGE, 13004 },
	{ 0x62ab, 0x62ac, PDF_CMAP_TABLE, 2896 },
	{ 0x62ad, 0x62b0, PDF_CMAP_RANGE, 13006 },
	{ 0x62b1, 0x62b1, PDF_CMAP_SINGLE, 1040 },
	{ 0x62b2, 0x62b4, PDF_CMAP_RANGE, 13010 },
	{ 0x62b5, 0x62b5, PDF_CMAP_SINGLE, 1463 },
	{ 0x62b6, 0x62b8, PDF_CMAP_RANGE, 13013 },
	{ 0x62b9, 0x62bf, PDF_CMAP_TABLE, 2898 },
	{ 0x62c0, 0x62c1, PDF_CMAP_RANGE, 13018 },
	{ 0x62c2, 0x62dc, PDF_CMAP_TABLE, 2905 },
	{ 0x62dd, 0x62de, PDF_CMAP_RANGE, 13025 },
	{ 0x62df, 0x62df, PDF_CMAP_SINGLE, 2885 },
	{ 0x62e0, 0x62e1, PDF_CMAP_RANGE, 13027 },
	{ 0x62e2, 0x62e9, PDF_CMAP_TABLE, 2932 },
	{ 0x62ea, 0x62eb, PDF_CMAP_RANGE, 13030 },
	{ 0x62ec, 0x62f7, PDF_CMAP_TABLE, 2940 },
	{ 0x62f8, 0x62fb, PDF_CMAP_RANGE, 13035 },
	{ 0x62fc, 0x6302, PDF_CMAP_TABLE, 2952 },
	{ 0x6303, 0x6306, PDF_CMAP_RANGE, 13040 },
	{ 0x6307, 0x6309, PDF_CMAP_TABLE, 2959 },
	{ 0x630a, 0x630d, PDF_CMAP_RANGE, 13044 },
	{ 0x630e, 0x630e, PDF_CMAP_SINGLE, 2419 },
	{ 0x630f, 0x6310, PDF_CMAP_RANGE, 13048 },
	{ 0x6311, 0x6311, PDF_CMAP_SINGLE, 3655 },
	{ 0x6312, 0x6315, PDF_CMAP_RANGE, 13050 },
	{ 0x6316, 0x6316, PDF_CMAP_SINGLE, 3723 },
	{ 0x6317, 0x6319, PDF_CMAP_RANGE, 13054 },
	{ 0x631a, 0x6325, PDF_CMAP_TABLE, 2962 },
	{ 0x6326, 0x6327, PDF_CMAP_RANGE, 13058 },
	{ 0x6328, 0x632b, PDF_CMAP_TABLE, 2974 },
	{ 0x632c, 0x632e, PDF_CMAP_RANGE, 13061 },
	{ 0x632f, 0x632f, PDF_CMAP_SINGLE, 4498 },
	{ 0x6330, 0x6331, PDF_CMAP_RANGE, 13064 },
	{ 0x6332, 0x6332, PDF_CMAP_SINGLE, 6479 },
	{ 0x6333, 0x6338, PDF_CMAP_RANGE, 13066 },
	{ 0x6339, 0x633a, PDF_CMAP_TABLE, 2978 },
	{ 0x633b, 0x633c, PDF_CMAP_RANGE, 13072 },
	{ 0x633d, 0x633e, PDF_CMAP_TABLE, 2980 },
	{ 0x633f, 0x6341, PDF_CMAP_RANGE, 13074 },
	{ 0x6342, 0x6346, PDF_CMAP_TABLE, 2982 },
	{ 0x6347, 0x6348, PDF_CMAP_RANGE, 13078 },
	{ 0x6349, 0x6350, PDF_CMAP_TABLE, 2987 },
	{ 0x6351, 0x6354, PDF_CMAP_RANGE, 13081 },
	{ 0x6355, 0x6355, PDF_CMAP_SINGLE, 1149 },
	{ 0x6356, 0x635d, PDF_CMAP_RANGE, 13085 },
	{ 0x635e, 0x6363, PDF_CMAP_TABLE, 2995 },
	{ 0x6364, 0x6366, PDF_CMAP_RANGE, 13094 },
	{ 0x6367, 0x636e, PDF_CMAP_TABLE, 3001 },
	{ 0x636f, 0x6370, PDF_CMAP_RANGE, 13099 },
	{ 0x6371, 0x6372, PDF_CMAP_TABLE, 3009 },
	{ 0x6373, 0x6375, PDF_CMAP_RANGE, 13101 },
	{ 0x6376, 0x6377, PDF_CMAP_TABLE, 3011 },
	{ 0x6378, 0x6379, PDF_CMAP_RANGE, 13104 },
	{ 0x637a, 0x637b, PDF_CMAP_TABLE, 3013 },
	{ 0x637c, 0x637f, PDF_CMAP_RANGE, 13106 },
	{ 0x6380, 0x6384, PDF_CMAP_TABLE, 3015 },
	{ 0x6385, 0x6386, PDF_CMAP_RANGE, 13111 },
	{ 0x6387, 0x6392, PDF_CMAP_TABLE, 3020 },
	{ 0x6393, 0x6395, PDF_CMAP_RANGE, 13116 },
	{ 0x6396, 0x6398, PDF_CMAP_TABLE, 3032 },
	{ 0x6399, 0x639f, PDF_CMAP_RANGE, 13120 },
	{ 0x63a0, 0x63b0, PDF_CMAP_TABLE, 3035 },
	{ 0x63b1, 0x63b2, PDF_CMAP_RANGE, 13132 },
	{ 0x63b3, 0x63b4, PDF_CMAP_TABLE, 3052 },
	{ 0x63b5, 0x63b6, PDF_CMAP_RANGE, 13134 },
	{ 0x63b7, 0x63c0, PDF_CMAP_TABLE, 3054 },
	{ 0x63c1, 0x63c3, PDF_CMAP_RANGE, 13140 },
	{ 0x63c4, 0x63c6, PDF_CMAP_TABLE, 3064 },
	{ 0x63c7, 0x63c8, PDF_CMAP_RANGE, 13144 },
	{ 0x63c9, 0x63c9, PDF_CMAP_SINGLE, 3259 },
	{ 0x63ca, 0x63cc, PDF_CMAP_RANGE, 13146 },
	{ 0x63cd, 0x63d2, PDF_CMAP_TABLE, 3067 },
	{ 0x63d3, 0x63d5, PDF_CMAP_RANGE, 13150 },
	{ 0x63d6, 0x63d6, PDF_CMAP_SINGLE, 4165 },
	{ 0x63d7, 0x63d9, PDF_CMAP_RANGE, 13153 },
	{ 0x63da, 0x63da, PDF_CMAP_SINGLE, 8703 },
	{ 0x63db, 0x63dd, PDF_CMAP_RANGE, 13156 },
	{ 0x63de, 0x63e3, PDF_CMAP_TABLE, 3073 },
	{ 0x63e4, 0x63e8, PDF_CMAP_RANGE, 13161 },
	{ 0x63e9, 0x63ea, PDF_CMAP_TABLE, 3079 },
	{ 0x63eb, 0x63ec, PDF_CMAP_RANGE, 13166 },
	{ 0x63ed, 0x63ee, PDF_CMAP_TABLE, 3081 },
	{ 0x63ef, 0x63f1, PDF_CMAP_RANGE, 13168 },
	{ 0x63f2, 0x63f8, PDF_CMAP_TABLE, 3083 },
	{ 0x63f9, 0x63fc, PDF_CMAP_RANGE, 13174 },
	{ 0x63fd, 0x6402, PDF_CMAP_TABLE, 3090 },
	{ 0x6403, 0x6404, PDF_CMAP_RANGE, 13179 },
	{ 0x6405, 0x6405, PDF_CMAP_SINGLE, 2194 },
	{ 0x6406, 0x640a, PDF_CMAP_RANGE, 13181 },
	{ 0x640b, 0x6410, PDF_CMAP_TABLE, 3096 },
	{ 0x6411, 0x6412, PDF_CMAP_RANGE, 13187 },
	{ 0x6413, 0x6414, PDF_CMAP_TABLE, 3102 },
	{ 0x6415, 0x6416, PDF_CMAP_RANGE, 13189 },
	{ 0x6417, 0x6417, PDF_CMAP_SINGLE, 7862 },
	{ 0x6418, 0x641a, PDF_CMAP_RANGE, 13191 },
	{ 0x641b, 0x6421, PDF_CMAP_TABLE, 3104 },
	{ 0x6422, 0x6425, PDF_CMAP_RANGE, 13196 },
	{ 0x6426, 0x6426, PDF_CMAP_SINGLE, 5347 },
	{ 0x6427, 0x6429, PDF_CMAP_RANGE, 13200 },
	{ 0x642a, 0x642d, PDF_CMAP_TABLE, 3111 },
	{ 0x642e, 0x6433, PDF_CMAP_RANGE, 13204 },
	{ 0x6434, 0x6436, PDF_CMAP_TABLE, 3115 },
	{ 0x6437, 0x6439, PDF_CMAP_RANGE, 13211 },
	{ 0x643a, 0x643a, PDF_CMAP_SINGLE, 3962 },
	{ 0x643b, 0x643c, PDF_CMAP_RANGE, 13214 },
	{ 0x643d, 0x6441, PDF_CMAP_TABLE, 3118 },
	{ 0x6442, 0x6443, PDF_CMAP_RANGE, 13218 },
	{ 0x6444, 0x644a, PDF_CMAP_TABLE, 3123 },
	{ 0x644b, 0x6450, PDF_CMAP_RANGE, 13221 },
	{ 0x6451, 0x6454, PDF_CMAP_TABLE, 3130 },
	{ 0x6455, 0x6457, PDF_CMAP_RANGE, 13228 },
	{ 0x6458, 0x6458, PDF_CMAP_SINGLE, 4427 },
	{ 0x6459, 0x645b, PDF_CMAP_RANGE, 13231 },
	{ 0x645c, 0x645f, PDF_CMAP_TABLE, 3134 },
	{ 0x6460, 0x6466, PDF_CMAP_RANGE, 13235 },
	{ 0x6467, 0x6469, PDF_CMAP_TABLE, 3138 },
	{ 0x646a, 0x646c, PDF_CMAP_RANGE, 13243 },
	{ 0x646d, 0x646f, PDF_CMAP_TABLE, 3141 },
	{ 0x6470, 0x6472, PDF_CMAP_RANGE, 13247 },
	{ 0x6473, 0x6473, PDF_CMAP_SINGLE, 8159 },
	{ 0x6474, 0x6475, PDF_CMAP_RANGE, 13250 },
	{ 0x6476, 0x6477, PDF_CMAP_TABLE, 3144 },
	{ 0x6478, 0x6479, PDF_CMAP_RANGE, 2819 },
	{ 0x647a, 0x647b, PDF_CMAP_TABLE, 3146 },
	{ 0x647c, 0x6481, PDF_CMAP_RANGE, 13253 },
	{ 0x6482, 0x6488, PDF_CMAP_TABLE, 3148 },
	{ 0x6489, 0x6490, PDF_CMAP_RANGE, 13261 },
	{ 0x6491, 0x6496, PDF_CMAP_TABLE, 3155 },
	{ 0x6497, 0x6498, PDF_CMAP_RANGE, 13270 },
	{ 0x6499, 0x6499, PDF_CMAP_SINGLE, 5356 },
	{ 0x649a, 0x649d, PDF_CMAP_RANGE, 13272 },
	{ 0x649e, 0x649f, PDF_CMAP_TABLE, 3161 },
	{ 0x64a0, 0x64a2, PDF_CMAP_RANGE, 13276 },
	{ 0x64a3, 0x64a5, PDF_CMAP_TABLE, 3163 },
	{ 0x64a6, 0x64a8, PDF_CMAP_RANGE, 13279 },
	{ 0x64a9, 0x64b6, PDF_CMAP_TABLE, 3166 },
	{ 0x64b7, 0x64b8, PDF_CMAP_RANGE, 5354 },
	{ 0x64b9, 0x64c7, PDF_CMAP_TABLE, 3180 },
	{ 0x64c8, 0x64c9, PDF_CMAP_RANGE, 13291 },
	{ 0x64ca, 0x64d4, PDF_CMAP_TABLE, 3195 },
	{ 0x64d5, 0x64d6, PDF_CMAP_RANGE, 13297 },
	{ 0x64d7, 0x64da, PDF_CMAP_TABLE, 3206 },
	{ 0x64db, 0x64dd, PDF_CMAP_RANGE, 13300 },
	{ 0x64de, 0x64e6, PDF_CMAP_TABLE, 3210 },
	{ 0x64e7, 0x64eb, PDF_CMAP_RANGE, 13307 },
	{ 0x64ec, 0x64ec, PDF_CMAP_SINGLE, 8344 },
	{ 0x64ed, 0x64ee, PDF_CMAP_RANGE, 13312 },
	{ 0x64ef, 0x64f4, PDF_CMAP_TABLE, 3219 },
	{ 0x64f5, 0x64f6, PDF_CMAP_RANGE, 13315 },
	{ 0x64f7, 0x64f7, PDF_CMAP_SINGLE, 9061 },
	{ 0x64f8, 0x64f9, PDF_CMAP_RANGE, 13317 },
	{ 0x64fa, 0x6500, PDF_CMAP_TABLE, 3225 },
	{ 0x6501, 0x6503, PDF_CMAP_RANGE, 13321 },
	{ 0x6504, 0x6506, PDF_CMAP_TABLE, 3232 },
	{ 0x6507, 0x6508, PDF_CMAP_RANGE, 13325 },
	{ 0x6509, 0x6509, PDF_CMAP_SINGLE, 5363 },
	{ 0x650a, 0x650e, PDF_CMAP_RANGE, 13327 },
	{ 0x650f, 0x650f, PDF_CMAP_SINGLE, 8252 },
	{ 0x6510, 0x6511, PDF_CMAP_RANGE, 13332 },
	{ 0x6512, 0x651d, PDF_CMAP_TABLE, 3235 },
	{ 0x651e, 0x6521, PDF_CMAP_RANGE, 13339 },
	{ 0x6522, 0x6525, PDF_CMAP_TABLE, 3247 },
	{ 0x6526, 0x6529, PDF_CMAP_RANGE, 13343 },
	{ 0x652a, 0x652f, PDF_CMAP_TABLE, 3251 },
	{ 0x6530, 0x6533, PDF_CMAP_RANGE, 13348 },
	{ 0x6534, 0x653b, PDF_CMAP_TABLE, 3257 },
	{ 0x653c, 0x653d, PDF_CMAP_RANGE, 13354 },
	{ 0x653e, 0x653f, PDF_CMAP_TABLE, 3265 },
	{ 0x6540, 0x6544, PDF_CMAP_RANGE, 13356 },
	{ 0x6545, 0x6545, PDF_CMAP_SINGLE, 1827 },
	{ 0x6546, 0x6547, PDF_CMAP_RANGE, 13361 },
	{ 0x6548, 0x6549, PDF_CMAP_TABLE, 3267 },
	{ 0x654a, 0x654b, PDF_CMAP_RANGE, 13363 },
	{ 0x654c, 0x654c, PDF_CMAP_SINGLE, 1457 },
	{ 0x654d, 0x654e, PDF_CMAP_RANGE, 13365 },
	{ 0x654f, 0x6551, PDF_CMAP_TABLE, 3269 },
	{ 0x6552, 0x6554, PDF_CMAP_RANGE, 13368 },
	{ 0x6555, 0x655e, PDF_CMAP_TABLE, 3272 },
	{ 0x655f, 0x6561, PDF_CMAP_RANGE, 13374 },
	{ 0x6562, 0x6563, PDF_CMAP_TABLE, 3282 },
	{ 0x6564, 0x6565, PDF_CMAP_RANGE, 13377 },
	{ 0x6566, 0x6566, PDF_CMAP_SINGLE, 1559 },
	{ 0x6567, 0x656a, PDF_CMAP_RANGE, 13379 },
	{ 0x656b, 0x656c, PDF_CMAP_TABLE, 3284 },
	{ 0x656d, 0x656f, PDF_CMAP_RANGE, 13383 },
	{ 0x6570, 0x6578, PDF_CMAP_TABLE, 3286 },
	{ 0x6579, 0x6581, PDF_CMAP_RANGE, 13389 },
	{ 0x6582, 0x6583, PDF_CMAP_TABLE, 3295 },
	{ 0x6584, 0x6586, PDF_CMAP_RANGE, 13398 },
	{ 0x6587, 0x6587, PDF_CMAP_SINGLE, 3795 },
	{ 0x6588, 0x658a, PDF_CMAP_RANGE, 13401 },
	{ 0x658b, 0x658c, PDF_CMAP_TABLE, 3297 },
	{ 0x658d, 0x658f, PDF_CMAP_RANGE, 13404 },
	{ 0x6590, 0x659c, PDF_CMAP_TABLE, 3299 },
	{ 0x659d, 0x659e, PDF_CMAP_RANGE, 13412 },
	{ 0x659f, 0x65a1, PDF_CMAP_TABLE, 3312 },
	{ 0x65a2, 0x65a3, PDF_CMAP_RANGE, 13415 },
	{ 0x65a4, 0x65b0, PDF_CMAP_TABLE, 3315 },
	{ 0x65b1, 0x65b6, PDF_CMAP_RANGE, 13421 },
	{ 0x65b7, 0x65b9, PDF_CMAP_TABLE, 3328 },
	{ 0x65ba, 0x65bb, PDF_CMAP_RANGE, 13428 },
	{ 0x65bc, 0x65bd, PDF_CMAP_TABLE, 3331 },
	{ 0x65be, 0x65c0, PDF_CMAP_RANGE, 13430 },
	{ 0x65c1, 0x65c6, PDF_CMAP_TABLE, 3333 },
	{ 0x65c7, 0x65ca, PDF_CMAP_RANGE, 13434 },
	{ 0x65cb, 0x65cf, PDF_CMAP_TABLE, 3339 },
	{ 0x65d0, 0x65d1, PDF_CMAP_RANGE, 13439 },
	{ 0x65d2, 0x65d2, PDF_CMAP_SINGLE, 6602 },
	{ 0x65d3, 0x65d5, PDF_CMAP_RANGE, 13441 },
	{ 0x65d6, 0x65d7, PDF_CMAP_TABLE, 3344 },
	{ 0x65d8, 0x65df, PDF_CMAP_RANGE, 13444 },
	{ 0x65e0, 0x65e2, PDF_CMAP_TABLE, 3346 },
	{ 0x65e3, 0x65e4, PDF_CMAP_RANGE, 13453 },
	{ 0x65e5, 0x65e9, PDF_CMAP_TABLE, 3349 },
	{ 0x65ea, 0x65eb, PDF_CMAP_RANGE, 13455 },
	{ 0x65ec, 0x65ed, PDF_CMAP_TABLE, 3354 },
	{ 0x65ee, 0x65f0, PDF_CMAP_RANGE, 6410 },
	{ 0x65f1, 0x65f1, PDF_CMAP_SINGLE, 1900 },
	{ 0x65f2, 0x65f5, PDF_CMAP_RANGE, 13457 },
	{ 0x65f6, 0x65f7, PDF_CMAP_TABLE, 3356 },
	{ 0x65f8, 0x65f9, PDF_CMAP_RANGE, 13461 },
	{ 0x65fa, 0x65fa, PDF_CMAP_SINGLE, 3755 },
	{ 0x65fb, 0x65ff, PDF_CMAP_RANGE, 13463 },
	{ 0x6600, 0x6603, PDF_CMAP_TABLE, 3358 },
	{ 0x6604, 0x6605, PDF_CMAP_RANGE, 13469 },
	{ 0x6606, 0x6606, PDF_CMAP_SINGLE, 2448 },
	{ 0x6607, 0x6609, PDF_CMAP_RANGE, 13471 },
	{ 0x660a, 0x660f, PDF_CMAP_TABLE, 3362 },
	{ 0x6610, 0x6612, PDF_CMAP_RANGE, 13476 },
	{ 0x6613, 0x6615, PDF_CMAP_TABLE, 3368 },
	{ 0x6616, 0x6618, PDF_CMAP_RANGE, 13479 },
	{ 0x6619, 0x6619, PDF_CMAP_SINGLE, 6414 },
	{ 0x661a, 0x661c, PDF_CMAP_RANGE, 13482 },
	{ 0x661d, 0x6620, PDF_CMAP_TABLE, 3371 },
	{ 0x6621, 0x6624, PDF_CMAP_RANGE, 13486 },
	{ 0x6625, 0x6628, PDF_CMAP_TABLE, 3375 },
	{ 0x6629, 0x662c, PDF_CMAP_RANGE, 13491 },
	{ 0x662d, 0x6631, PDF_CMAP_TABLE, 3379 },
	{ 0x6632, 0x6633, PDF_CMAP_RANGE, 13497 },
	{ 0x6634, 0x6636, PDF_CMAP_TABLE, 3384 },
	{ 0x6637, 0x663b, PDF_CMAP_RANGE, 13499 },
	{ 0x663c, 0x663e, PDF_CMAP_TABLE, 3387 },
	{ 0x663f, 0x6640, PDF_CMAP_RANGE, 13505 },
	{ 0x6641, 0x6643, PDF_CMAP_TABLE, 3390 },
	{ 0x6644, 0x664a, PDF_CMAP_RANGE, 13507 },
	{ 0x664b, 0x664c, PDF_CMAP_TABLE, 3393 },
	{ 0x664d, 0x664e, PDF_CMAP_RANGE, 13514 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 6430 },
	{ 0x6650, 0x6651, PDF_CMAP_RANGE, 13516 },
	{ 0x6652, 0x6657, PDF_CMAP_TABLE, 3395 },
	{ 0x6658, 0x6659, PDF_CMAP_RANGE, 13518 },
	{ 0x665a, 0x665a, PDF_CMAP_SINGLE, 3742 },
	{ 0x665b, 0x665c, PDF_CMAP_RANGE, 13520 },
	{ 0x665d, 0x6661, PDF_CMAP_TABLE, 3401 },
	{ 0x6662, 0x6663, PDF_CMAP_RANGE, 13524 },
	{ 0x6664, 0x6668, PDF_CMAP_TABLE, 3406 },
	{ 0x6669, 0x666d, PDF_CMAP_RANGE, 13528 },
	{ 0x666e, 0x6670, PDF_CMAP_TABLE, 3411 },
	{ 0x6671, 0x6673, PDF_CMAP_RANGE, 13533 },
	{ 0x6674, 0x6677, PDF_CMAP_TABLE, 3414 },
	{ 0x6678, 0x6679, PDF_CMAP_RANGE, 13537 },
	{ 0x667a, 0x667a, PDF_CMAP_SINGLE, 4550 },
	{ 0x667b, 0x667d, PDF_CMAP_RANGE, 13539 },
	{ 0x667e, 0x667e, PDF_CMAP_SINGLE, 2569 },
	{ 0x667f, 0x6681, PDF_CMAP_RANGE, 13542 },
	{ 0x6682, 0x6684, PDF_CMAP_TABLE, 3418 },
	{ 0x6685, 0x6686, PDF_CMAP_RANGE, 13546 },
	{ 0x6687, 0x6689, PDF_CMAP_TABLE, 3421 },
	{ 0x668a, 0x668b, PDF_CMAP_RANGE, 13548 },
	{ 0x668c, 0x668c, PDF_CMAP_SINGLE, 6436 },
	{ 0x668d, 0x6690, PDF_CMAP_RANGE, 13550 },
	{ 0x6691, 0x6691, PDF_CMAP_SINGLE, 3461 },
	{ 0x6692, 0x6695, PDF_CMAP_RANGE, 13554 },
	{ 0x6696, 0x6697, PDF_CMAP_TABLE, 3424 },
	{ 0x6698, 0x669c, PDF_CMAP_RANGE, 13558 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 6438 },
	{ 0x669e, 0x66a1, PDF_CMAP_RANGE, 13563 },
	{ 0x66a2, 0x66a2, PDF_CMAP_SINGLE, 7801 },
	{ 0x66a3, 0x66a6, PDF_CMAP_RANGE, 13567 },
	{ 0x66a7, 0x66a8, PDF_CMAP_TABLE, 3426 },
	{ 0x66a9, 0x66aa, PDF_CMAP_RANGE, 13571 },
	{ 0x66ab, 0x66ab, PDF_CMAP_SINGLE, 8794 },
	{ 0x66ac, 0x66ad, PDF_CMAP_RANGE, 13573 },
	{ 0x66ae, 0x66ae, PDF_CMAP_SINGLE, 2845 },
	{ 0x66af, 0x66b3, PDF_CMAP_RANGE, 13575 },
	{ 0x66b4, 0x66b4, PDF_CMAP_SINGLE, 1042 },
	{ 0x66b5, 0x66b8, PDF_CMAP_RANGE, 13580 },
	{ 0x66b9, 0x66b9, PDF_CMAP_SINGLE, 5979 },
	{ 0x66ba, 0x66bd, PDF_CMAP_RANGE, 13584 },
	{ 0x66be, 0x66be, PDF_CMAP_SINGLE, 6439 },
	{ 0x66bf, 0x66c3, PDF_CMAP_RANGE, 13588 },
	{ 0x66c4, 0x66c9, PDF_CMAP_TABLE, 3428 },
	{ 0x66ca, 0x66d5, PDF_CMAP_RANGE, 13595 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 9385 },
	{ 0x66d7, 0x66d8, PDF_CMAP_RANGE, 13607 },
	{ 0x66d9, 0x66da, PDF_CMAP_TABLE, 3434 },
	{ 0x66db, 0x66dc, PDF_CMAP_RANGE, 6440 },
	{ 0x66dd, 0x66dd, PDF_CMAP_SINGLE, 3063 },
	{ 0x66de, 0x66df, PDF_CMAP_RANGE, 13610 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 8167 },
	{ 0x66e1, 0x66e5, PDF_CMAP_RANGE, 13612 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 6442 },
	{ 0x66e7, 0x66e8, PDF_CMAP_RANGE, 13617 },
	{ 0x66e9, 0x66e9, PDF_CMAP_SINGLE, 6443 },
	{ 0x66ea, 0x66eb, PDF_CMAP_RANGE, 13619 },
	{ 0x66ec, 0x66ec, PDF_CMAP_SINGLE, 8466 },
	{ 0x66ed, 0x66ef, PDF_CMAP_RANGE, 13621 },
	{ 0x66f0, 0x66f4, PDF_CMAP_TABLE, 3436 },
	{ 0x66f5, 0x66f6, PDF_CMAP_RANGE, 13625 },
	{ 0x66f7, 0x66f9, PDF_CMAP_TABLE, 3441 },
	{ 0x66fa, 0x66fb, PDF_CMAP_RANGE, 13627 },
	{ 0x66fc, 0x6700, PDF_CMAP_TABLE, 3444 },
	{ 0x6701, 0x6702, PDF_CMAP_RANGE, 13630 },
	{ 0x6703, 0x6703, PDF_CMAP_SINGLE, 8028 },
	{ 0x6704, 0x6707, PDF_CMAP_RANGE, 13632 },
	{ 0x6708, 0x670d, PDF_CMAP_TABLE, 3449 },
	{ 0x670e, 0x670f, PDF_CMAP_RANGE, 13637 },
	{ 0x6710, 0x6710, PDF_CMAP_SINGLE, 6532 },
	{ 0x6711, 0x6713, PDF_CMAP_RANGE, 13639 },
	{ 0x6714, 0x6717, PDF_CMAP_TABLE, 3455 },
	{ 0x6718, 0x671a, PDF_CMAP_RANGE, 13643 },
	{ 0x671b, 0x671f, PDF_CMAP_TABLE, 3459 },
	{ 0x6720, 0x6725, PDF_CMAP_RANGE, 13648 },
	{ 0x6726, 0x6731, PDF_CMAP_TABLE, 3464 },
	{ 0x6732, 0x6733, PDF_CMAP_RANGE, 13657 },
	{ 0x6734, 0x6735, PDF_CMAP_TABLE, 3476 },
	{ 0x6736, 0x6739, PDF_CMAP_RANGE, 13659 },
	{ 0x673a, 0x673a, PDF_CMAP_SINGLE, 2063 },
	{ 0x673b, 0x673c, PDF_CMAP_RANGE, 13663 },
	{ 0x673d, 0x673d, PDF_CMAP_SINGLE, 4011 },
	{ 0x673e, 0x673f, PDF_CMAP_RANGE, 13665 },
	{ 0x6740, 0x6743, PDF_CMAP_TABLE, 3478 },
	{ 0x6744, 0x6745, PDF_CMAP_RANGE, 13668 },
	{ 0x6746, 0x6749, PDF_CMAP_TABLE, 3482 },
	{ 0x674a, 0x674b, PDF_CMAP_RANGE, 13671 },
	{ 0x674c, 0x6753, PDF_CMAP_TABLE, 3486 },
	{ 0x6754, 0x6755, PDF_CMAP_RANGE, 13675 },
	{ 0x6756, 0x6756, PDF_CMAP_SINGLE, 4457 },
	{ 0x6757, 0x675b, PDF_CMAP_RANGE, 13677 },
	{ 0x675c, 0x6761, PDF_CMAP_TABLE, 3494 },
	{ 0x6762, 0x6764, PDF_CMAP_RANGE, 13683 },
	{ 0x6765, 0x6765, PDF_CMAP_SINGLE, 2463 },
	{ 0x6766, 0x6767, PDF_CMAP_RANGE, 13686 },
	{ 0x6768, 0x676a, PDF_CMAP_TABLE, 3500 },
	{ 0x676b, 0x676c, PDF_CMAP_RANGE, 13688 },
	{ 0x676d, 0x6777, PDF_CMAP_TABLE, 3503 },
	{ 0x6778, 0x677b, PDF_CMAP_RANGE, 13693 },
	{ 0x677c, 0x6781, PDF_CMAP_TABLE, 3514 },
	{ 0x6782, 0x6783, PDF_CMAP_RANGE, 13699 },
	{ 0x6784, 0x6784, PDF_CMAP_SINGLE, 1810 },
	{ 0x6785, 0x6786, PDF_CMAP_RANGE, 13701 },
	{ 0x6787, 0x678b, PDF_CMAP_TABLE, 3520 },
	{ 0x678c, 0x678f, PDF_CMAP_RANGE, 13705 },
	{ 0x6790, 0x6790, PDF_CMAP_SINGLE, 3845 },
	{ 0x6791, 0x6794, PDF_CMAP_RANGE, 13709 },
	{ 0x6795, 0x679e, PDF_CMAP_TABLE, 3525 },
	{ 0x679f, 0x67a1, PDF_CMAP_RANGE, 13716 },
	{ 0x67a2, 0x67b0, PDF_CMAP_TABLE, 3535 },
	{ 0x67b1, 0x67b2, PDF_CMAP_RANGE, 13724 },
	{ 0x67b3, 0x67b8, PDF_CMAP_TABLE, 3550 },
	{ 0x67b9, 0x67c0, PDF_CMAP_RANGE, 13727 },
	{ 0x67c1, 0x67c4, PDF_CMAP_TABLE, 3556 },
	{ 0x67c5, 0x67ce, PDF_CMAP_RANGE, 13736 },
	{ 0x67cf, 0x67d4, PDF_CMAP_TABLE, 3560 },
	{ 0x67d5, 0x67d7, PDF_CMAP_RANGE, 13746 },
	{ 0x67d8, 0x67e2, PDF_CMAP_TABLE, 3566 },
	{ 0x67e3, 0x67e4, PDF_CMAP_RANGE, 13752 },
	{ 0x67e5, 0x67e5, PDF_CMAP_SINGLE, 1200 },
	{ 0x67e6, 0x67e8, PDF_CMAP_RANGE, 13754 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 6247 },
	{ 0x67ea, 0x67eb, PDF_CMAP_RANGE, 13757 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 2144 },
	{ 0x67ed, 0x67ee, PDF_CMAP_RANGE, 13759 },
	{ 0x67ef, 0x67f4, PDF_CMAP_TABLE, 3577 },
	{ 0x67f5, 0x67fc, PDF_CMAP_RANGE, 13762 },
	{ 0x67fd, 0x6800, PDF_CMAP_TABLE, 3583 },
	{ 0x6801, 0x6804, PDF_CMAP_RANGE, 13771 },
	{ 0x6805, 0x6813, PDF_CMAP_TABLE, 3587 },
	{ 0x6814, 0x6815, PDF_CMAP_RANGE, 13779 },
	{ 0x6816, 0x6817, PDF_CMAP_TABLE, 3602 },
	{ 0x6818, 0x681c, PDF_CMAP_RANGE, 13781 },
	{ 0x681d, 0x681d, PDF_CMAP_SINGLE, 6271 },
	{ 0x681e, 0x6820, PDF_CMAP_RANGE, 13786 },
	{ 0x6821, 0x6821, PDF_CMAP_SINGLE, 3950 },
	{ 0x6822, 0x6828, PDF_CMAP_RANGE, 13789 },
	{ 0x6829, 0x682a, PDF_CMAP_TABLE, 3604 },
	{ 0x682b, 0x6831, PDF_CMAP_RANGE, 13796 },
	{ 0x6832, 0x6833, PDF_CMAP_RANGE, 6262 },
	{ 0x6834, 0x6836, PDF_CMAP_RANGE, 13803 },
	{ 0x6837, 0x6839, PDF_CMAP_TABLE, 3606 },
	{ 0x683a, 0x683b, PDF_CMAP_RANGE, 13806 },
	{ 0x683c, 0x6855, PDF_CMAP_TABLE, 3609 },
	{ 0x6856, 0x685f, PDF_CMAP_RANGE, 13814 },
	{ 0x6860, 0x6861, PDF_CMAP_RANGE, 6264 },
	{ 0x6862, 0x686b, PDF_CMAP_TABLE, 3635 },
	{ 0x686c, 0x6873, PDF_CMAP_RANGE, 13825 },
	{ 0x6874, 0x6877, PDF_CMAP_TABLE, 3645 },
	{ 0x6878, 0x6880, PDF_CMAP_RANGE, 13834 },
	{ 0x6881, 0x6886, PDF_CMAP_TABLE, 3649 },
	{ 0x6887, 0x688e, PDF_CMAP_RANGE, 13845 },
	{ 0x688f, 0x688f, PDF_CMAP_SINGLE, 6282 },
	{ 0x6890, 0x6892, PDF_CMAP_RANGE, 13853 },
	{ 0x6893, 0x6893, PDF_CMAP_SINGLE, 6285 },
	{ 0x6894, 0x6896, PDF_CMAP_RANGE, 13856 },
	{ 0x6897, 0x6898, PDF_CMAP_TABLE, 3655 },
	{ 0x6899, 0x689c, PDF_CMAP_RANGE, 13859 },
	{ 0x689d, 0x689f, PDF_CMAP_TABLE, 3657 },
	{ 0x68a0, 0x68a1, PDF_CMAP_RANGE, 13864 },
	{ 0x68a2, 0x68a2, PDF_CMAP_SINGLE, 3339 },
	{ 0x68a3, 0x68a5, PDF_CMAP_RANGE, 13866 },
	{ 0x68a6, 0x68a8, PDF_CMAP_TABLE, 3660 },
	{ 0x68a9, 0x68ac, PDF_CMAP_RANGE, 13869 },
	{ 0x68ad, 0x68b0, PDF_CMAP_TABLE, 3663 },
	{ 0x68b1, 0x68b2, PDF_CMAP_RANGE, 13874 },
	{ 0x68b3, 0x68b5, PDF_CMAP_TABLE, 3667 },
	{ 0x68b6, 0x68bf, PDF_CMAP_RANGE, 13877 },
	{ 0x68c0, 0x68c2, PDF_CMAP_TABLE, 3670 },
	{ 0x68c3, 0x68c8, PDF_CMAP_RANGE, 13888 },
	{ 0x68c9, 0x68cd, PDF_CMAP_TABLE, 3673 },
	{ 0x68ce, 0x68d1, PDF_CMAP_RANGE, 13896 },
	{ 0x68d2, 0x68d2, PDF_CMAP_SINGLE, 1023 },
	{ 0x68d3, 0x68d4, PDF_CMAP_RANGE, 13900 },
	{ 0x68d5, 0x68da, PDF_CMAP_TABLE, 3678 },
	{ 0x68db, 0x68de, PDF_CMAP_RANGE, 13903 },
	{ 0x68df, 0x68e0, PDF_CMAP_TABLE, 3684 },
	{ 0x68e1, 0x68e2, PDF_CMAP_RANGE, 13907 },
	{ 0x68e3, 0x68e3, PDF_CMAP_SINGLE, 6298 },
	{ 0x68e4, 0x68e6, PDF_CMAP_RANGE, 13909 },
	{ 0x68e7, 0x68e7, PDF_CMAP_SINGLE, 8818 },
	{ 0x68e8, 0x68ed, PDF_CMAP_RANGE, 13912 },
	{ 0x68ee, 0x68f1, PDF_CMAP_TABLE, 3686 },
	{ 0x68f2, 0x68f4, PDF_CMAP_RANGE, 13919 },
	{ 0x68f5, 0x68f5, PDF_CMAP_SINGLE, 2384 },
	{ 0x68f6, 0x68f8, PDF_CMAP_RANGE, 13922 },
	{ 0x68f9, 0x68fc, PDF_CMAP_TABLE, 3690 },
	{ 0x68fd, 0x6900, PDF_CMAP_RANGE, 13926 },
	{ 0x6901, 0x6901, PDF_CMAP_SINGLE, 6296 },
	{ 0x6902, 0x6904, PDF_CMAP_RANGE, 13930 },
	{ 0x6905, 0x6905, PDF_CMAP_SINGLE, 4181 },
	{ 0x6906, 0x690a, PDF_CMAP_RANGE, 13933 },
	{ 0x690b, 0x6912, PDF_CMAP_TABLE, 3694 },
	{ 0x6913, 0x691e, PDF_CMAP_RANGE, 13940 },
	{ 0x691f, 0x6920, PDF_CMAP_RANGE, 6290 },
	{ 0x6921, 0x6923, PDF_CMAP_RANGE, 13952 },
	{ 0x6924, 0x6924, PDF_CMAP_SINGLE, 6293 },
	{ 0x6925, 0x692c, PDF_CMAP_RANGE, 13955 },
	{ 0x692d, 0x692d, PDF_CMAP_SINGLE, 3719 },
	{ 0x692e, 0x692f, PDF_CMAP_RANGE, 13963 },
	{ 0x6930, 0x6930, PDF_CMAP_SINGLE, 4147 },
	{ 0x6931, 0x6933, PDF_CMAP_RANGE, 13965 },
	{ 0x6934, 0x6934, PDF_CMAP_SINGLE, 6310 },
	{ 0x6935, 0x6938, PDF_CMAP_RANGE, 13968 },
	{ 0x6939, 0x6939, PDF_CMAP_SINGLE, 6301 },
	{ 0x693a, 0x693c, PDF_CMAP_RANGE, 13972 },
	{ 0x693d, 0x693f, PDF_CMAP_TABLE, 3702 },
	{ 0x6940, 0x6941, PDF_CMAP_RANGE, 13976 },
	{ 0x6942, 0x6942, PDF_CMAP_SINGLE, 6303 },
	{ 0x6943, 0x6949, PDF_CMAP_RANGE, 13978 },
	{ 0x694a, 0x694a, PDF_CMAP_SINGLE, 8702 },
	{ 0x694b, 0x6952, PDF_CMAP_RANGE, 13985 },
	{ 0x6953, 0x6954, PDF_CMAP_TABLE, 3705 },
	{ 0x6955, 0x6956, PDF_CMAP_RANGE, 13993 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 6297 },
	{ 0x6958, 0x6959, PDF_CMAP_RANGE, 13995 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 1315 },
	{ 0x695b, 0x695c, PDF_CMAP_RANGE, 13997 },
	{ 0x695d, 0x6960, PDF_CMAP_TABLE, 3707 },
	{ 0x6961, 0x6962, PDF_CMAP_RANGE, 14000 },
	{ 0x6963, 0x6963, PDF_CMAP_SINGLE, 6317 },
	{ 0x6964, 0x6965, PDF_CMAP_RANGE, 14002 },
	{ 0x6966, 0x6968, PDF_CMAP_TABLE, 3711 },
	{ 0x6969, 0x696a, PDF_CMAP_RANGE, 14005 },
	{ 0x696b, 0x696e, PDF_CMAP_TABLE, 3714 },
	{ 0x696f, 0x6970, PDF_CMAP_RANGE, 14008 },
	{ 0x6971, 0x6971, PDF_CMAP_SINGLE, 6300 },
	{ 0x6972, 0x6974, PDF_CMAP_RANGE, 14010 },
	{ 0x6975, 0x6979, PDF_CMAP_TABLE, 3718 },
	{ 0x697a, 0x697b, PDF_CMAP_RANGE, 14014 },
	{ 0x697c, 0x697c, PDF_CMAP_SINGLE, 2636 },
	{ 0x697d, 0x697f, PDF_CMAP_RANGE, 14016 },
	{ 0x6980, 0x6986, PDF_CMAP_TABLE, 3723 },
	{ 0x6987, 0x6988, PDF_CMAP_RANGE, 6312 },
	{ 0x6989, 0x6989, PDF_CMAP_SINGLE, 6315 },
	{ 0x698a, 0x698c, PDF_CMAP_RANGE, 14022 },
	{ 0x698d, 0x698d, PDF_CMAP_SINGLE, 6331 },
	{ 0x698e, 0x6993, PDF_CMAP_RANGE, 14025 },
	{ 0x6994, 0x6995, PDF_CMAP_TABLE, 3730 },
	{ 0x6996, 0x6997, PDF_CMAP_RANGE, 14031 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 6308 },
	{ 0x6999, 0x699a, PDF_CMAP_RANGE, 14033 },
	{ 0x699b, 0x699c, PDF_CMAP_TABLE, 3732 },
	{ 0x699d, 0x69a6, PDF_CMAP_RANGE, 14035 },
	{ 0x69a7, 0x69ae, PDF_CMAP_TABLE, 3734 },
	{ 0x69af, 0x69b0, PDF_CMAP_RANGE, 14047 },
	{ 0x69b1, 0x69b1, PDF_CMAP_SINGLE, 6325 },
	{ 0x69b2, 0x69b3, PDF_CMAP_RANGE, 14049 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 2618 },
	{ 0x69b5, 0x69b6, PDF_CMAP_RANGE, 14051 },
	{ 0x69b7, 0x69b7, PDF_CMAP_SINGLE, 3217 },
	{ 0x69b8, 0x69ba, PDF_CMAP_RANGE, 14053 },
	{ 0x69bb, 0x69bb, PDF_CMAP_SINGLE, 6321 },
	{ 0x69bc, 0x69be, PDF_CMAP_RANGE, 14056 },
	{ 0x69bf, 0x69c1, PDF_CMAP_TABLE, 3742 },
	{ 0x69c2, 0x69c9, PDF_CMAP_RANGE, 14060 },
	{ 0x69ca, 0x69d0, PDF_CMAP_TABLE, 3745 },
	{ 0x69d1, 0x69d3, PDF_CMAP_RANGE, 14069 },
	{ 0x69d4, 0x69d4, PDF_CMAP_SINGLE, 6324 },
	{ 0x69d5, 0x69da, PDF_CMAP_RANGE, 14072 },
	{ 0x69db, 0x69db, PDF_CMAP_SINGLE, 2154 },
	{ 0x69dc, 0x69de, PDF_CMAP_RANGE, 14078 },
	{ 0x69df, 0x69e0, PDF_CMAP_TABLE, 3752 },
	{ 0x69e1, 0x69e6, PDF_CMAP_RANGE, 14081 },
	{ 0x69e7, 0x69e7, PDF_CMAP_SINGLE, 9340 },
	{ 0x69e8, 0x69ec, PDF_CMAP_RANGE, 14087 },
	{ 0x69ed, 0x69ed, PDF_CMAP_SINGLE, 6334 },
	{ 0x69ee, 0x69f1, PDF_CMAP_RANGE, 14092 },
	{ 0x69f2, 0x69f3, PDF_CMAP_TABLE, 3754 },
	{ 0x69f4, 0x69fc, PDF_CMAP_RANGE, 14096 },
	{ 0x69fd, 0x6a02, PDF_CMAP_TABLE, 3756 },
	{ 0x6a03, 0x6a04, PDF_CMAP_RANGE, 14107 },
	{ 0x6a05, 0x6a05, PDF_CMAP_SINGLE, 9324 },
	{ 0x6a06, 0x6a09, PDF_CMAP_RANGE, 14109 },
	{ 0x6a0a, 0x6a0a, PDF_CMAP_SINGLE, 1611 },
	{ 0x6a0b, 0x6a12, PDF_CMAP_RANGE, 14113 },
	{ 0x6a13, 0x6a13, PDF_CMAP_SINGLE, 8254 },
	{ 0x6a14, 0x6a16, PDF_CMAP_RANGE, 14121 },
	{ 0x6a17, 0x6a18, PDF_CMAP_RANGE, 6335 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 7756 },
	{ 0x6a1a, 0x6a1d, PDF_CMAP_RANGE, 14124 },
	{ 0x6a1e, 0x6a23, PDF_CMAP_TABLE, 3762 },
	{ 0x6a24, 0x6a27, PDF_CMAP_RANGE, 14130 },
	{ 0x6a28, 0x6a2a, PDF_CMAP_TABLE, 3768 },
	{ 0x6a2b, 0x6a2e, PDF_CMAP_RANGE, 14135 },
	{ 0x6a2f, 0x6a31, PDF_CMAP_TABLE, 3771 },
	{ 0x6a32, 0x6a34, PDF_CMAP_RANGE, 14140 },
	{ 0x6a35, 0x6a35, PDF_CMAP_SINGLE, 6344 },
	{ 0x6a36, 0x6a37, PDF_CMAP_RANGE, 14143 },
	{ 0x6a38, 0x6a3a, PDF_CMAP_TABLE, 3774 },
	{ 0x6a3b, 0x6a3c, PDF_CMAP_RANGE, 14145 },
	{ 0x6a3d, 0x6a3e, PDF_CMAP_TABLE, 3777 },
	{ 0x6a3f, 0x6a43, PDF_CMAP_RANGE, 14147 },
	{ 0x6a44, 0x6a44, PDF_CMAP_SINGLE, 6339 },
	{ 0x6a45, 0x6a46, PDF_CMAP_RANGE, 14152 },
	{ 0x6a47, 0x6a48, PDF_CMAP_TABLE, 3779 },
	{ 0x6a49, 0x6a4a, PDF_CMAP_RANGE, 14154 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 8413 },
	{ 0x6a4c, 0x6a4f, PDF_CMAP_RANGE, 14156 },
	{ 0x6a50, 0x6a50, PDF_CMAP_SINGLE, 6342 },
	{ 0x6a51, 0x6a57, PDF_CMAP_RANGE, 14160 },
	{ 0x6a58, 0x6a5b, PDF_CMAP_TABLE, 3781 },
	{ 0x6a5c, 0x6a5e, PDF_CMAP_RANGE, 14168 },
	{ 0x6a5f, 0x6a62, PDF_CMAP_TABLE, 3785 },
	{ 0x6a63, 0x6a64, PDF_CMAP_RANGE, 14172 },
	{ 0x6a65, 0x6a65, PDF_CMAP_SINGLE, 6337 },
	{ 0x6a66, 0x6a70, PDF_CMAP_RANGE, 14174 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 1308 },
	{ 0x6a72, 0x6a78, PDF_CMAP_RANGE, 14185 },
	{ 0x6a79, 0x6a79, PDF_CMAP_SINGLE, 6346 },
	{ 0x6a7a, 0x6a7b, PDF_CMAP_RANGE, 14192 },
	{ 0x6a7c, 0x6a7c, PDF_CMAP_SINGLE, 6350 },
	{ 0x6a7d, 0x6a7f, PDF_CMAP_RANGE, 14194 },
	{ 0x6a80, 0x6a80, PDF_CMAP_SINGLE, 3591 },
	{ 0x6a81, 0x6a83, PDF_CMAP_RANGE, 14197 },
	{ 0x6a84, 0x6a84, PDF_CMAP_SINGLE, 3866 },
	{ 0x6a85, 0x6a88, PDF_CMAP_RANGE, 14200 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 9330 },
	{ 0x6a8a, 0x6a8d, PDF_CMAP_RANGE, 14204 },
	{ 0x6a8e, 0x6a91, PDF_CMAP_TABLE, 3789 },
	{ 0x6a92, 0x6a93, PDF_CMAP_RANGE, 14209 },
	{ 0x6a94, 0x6a94, PDF_CMAP_SINGLE, 7861 },
	{ 0x6a95, 0x6a96, PDF_CMAP_RANGE, 14211 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 6354 },
	{ 0x6a98, 0x6a9b, PDF_CMAP_RANGE, 14213 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 9336 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 14217 },
	{ 0x6aa0, 0x6aa3, PDF_CMAP_TABLE, 3793 },
	{ 0x6aa4, 0x6aa8, PDF_CMAP_RANGE, 14221 },
	{ 0x6aa9, 0x6aac, PDF_CMAP_TABLE, 3797 },
	{ 0x6aad, 0x6aae, PDF_CMAP_RANGE, 14227 },
	{ 0x6aaf, 0x6aaf, PDF_CMAP_SINGLE, 9881 },
	{ 0x6ab0, 0x6ab2, PDF_CMAP_RANGE, 14229 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 9346 },
	{ 0x6ab4, 0x6ab7, PDF_CMAP_RANGE, 14232 },
	{ 0x6ab8, 0x6ab8, PDF_CMAP_SINGLE, 8353 },
	{ 0x6ab9, 0x6aba, PDF_CMAP_RANGE, 14236 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 8082 },
	{ 0x6abc, 0x6ac2, PDF_CMAP_RANGE, 14238 },
	{ 0x6ac3, 0x6ac3, PDF_CMAP_SINGLE, 7992 },
	{ 0x6ac4, 0x6ad2, PDF_CMAP_RANGE, 14245 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 9349 },
	{ 0x6ad4, 0x6ad9, PDF_CMAP_RANGE, 14260 },
	{ 0x6ada, 0x6adf, PDF_CMAP_TABLE, 3801 },
	{ 0x6ae0, 0x6ae6, PDF_CMAP_RANGE, 14267 },
	{ 0x6ae7, 0x6aec, PDF_CMAP_TABLE, 3807 },
	{ 0x6aed, 0x6af2, PDF_CMAP_RANGE, 14276 },
	{ 0x6af3, 0x6af3, PDF_CMAP_SINGLE, 9327 },
	{ 0x6af4, 0x6af7, PDF_CMAP_RANGE, 14282 },
	{ 0x6af8, 0x6af8, PDF_CMAP_SINGLE, 9345 },
	{ 0x6af9, 0x6afa, PDF_CMAP_RANGE, 14286 },
	{ 0x6afb, 0x6afb, PDF_CMAP_SINGLE, 8736 },
	{ 0x6afc, 0x6b03, PDF_CMAP_RANGE, 14288 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 8181 },
	{ 0x6b05, 0x6b09, PDF_CMAP_RANGE, 14296 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 8436 },
	{ 0x6b0b, 0x6b0e, PDF_CMAP_RANGE, 14301 },
	{ 0x6b0f, 0x6b0f, PDF_CMAP_SINGLE, 9341 },
	{ 0x6b10, 0x6b11, PDF_CMAP_RANGE, 14305 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 9337 },
	{ 0x6b13, 0x6b15, PDF_CMAP_RANGE, 14307 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 9342 },
	{ 0x6b17, 0x6b1d, PDF_CMAP_RANGE, 14310 },
	{ 0x6b1e, 0x6b24, PDF_CMAP_TABLE, 3813 },
	{ 0x6b25, 0x6b26, PDF_CMAP_RANGE, 14318 },
	{ 0x6b27, 0x6b27, PDF_CMAP_SINGLE, 2936 },
	{ 0x6b28, 0x6b31, PDF_CMAP_RANGE, 14320 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 4320 },
	{ 0x6b33, 0x6b36, PDF_CMAP_RANGE, 14330 },
	{ 0x6b37, 0x6b3a, PDF_CMAP_TABLE, 3820 },
	{ 0x6b3b, 0x6b3c, PDF_CMAP_RANGE, 14335 },
	{ 0x6b3d, 0x6b3e, PDF_CMAP_TABLE, 3824 },
	{ 0x6b3f, 0x6b42, PDF_CMAP_RANGE, 14337 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 6580 },
	{ 0x6b44, 0x6b45, PDF_CMAP_RANGE, 14341 },
	{ 0x6b46, 0x6b49, PDF_CMAP_TABLE, 3826 },
	{ 0x6b4a, 0x6b4b, PDF_CMAP_RANGE, 14344 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 1763 },
	{ 0x6b4d, 0x6b4f, PDF_CMAP_RANGE, 14346 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 8365 },
	{ 0x6b51, 0x6b58, PDF_CMAP_RANGE, 14349 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 6582 },
	{ 0x6b5a, 0x6b5e, PDF_CMAP_RANGE, 14357 },
	{ 0x6b5f, 0x6b67, PDF_CMAP_TABLE, 3830 },
	{ 0x6b68, 0x6b69, PDF_CMAP_RANGE, 14363 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 3730 },
	{ 0x6b6b, 0x6b71, PDF_CMAP_RANGE, 14365 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 8537 },
	{ 0x6b73, 0x6b76, PDF_CMAP_RANGE, 14372 },
	{ 0x6b77, 0x6b7c, PDF_CMAP_TABLE, 3839 },
	{ 0x6b7d, 0x6b80, PDF_CMAP_RANGE, 14377 },
	{ 0x6b81, 0x6b82, PDF_CMAP_RANGE, 6358 },
	{ 0x6b83, 0x6b8d, PDF_CMAP_TABLE, 3845 },
	{ 0x6b8e, 0x6b91, PDF_CMAP_RANGE, 14384 },
	{ 0x6b92, 0x6b93, PDF_CMAP_RANGE, 6362 },
	{ 0x6b94, 0x6b95, PDF_CMAP_RANGE, 14388 },
	{ 0x6b96, 0x6b99, PDF_CMAP_TABLE, 3856 },
	{ 0x6b9a, 0x6b9b, PDF_CMAP_RANGE, 6365 },
	{ 0x6b9c, 0x6b9d, PDF_CMAP_RANGE, 14392 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 9352 },
	{ 0x6b9f, 0x6ba0, PDF_CMAP_RANGE, 14394 },
	{ 0x6ba1, 0x6ba1, PDF_CMAP_SINGLE, 6367 },
	{ 0x6ba2, 0x6ba3, PDF_CMAP_RANGE, 14396 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 9351 },
	{ 0x6ba5, 0x6ba9, PDF_CMAP_RANGE, 14398 },
	{ 0x6baa, 0x6bab, PDF_CMAP_TABLE, 3860 },
	{ 0x6bac, 0x6bad, PDF_CMAP_RANGE, 14403 },
	{ 0x6bae, 0x6baf, PDF_CMAP_TABLE, 3862 },
	{ 0x6bb0, 0x6bb1, PDF_CMAP_RANGE, 14405 },
	{ 0x6bb2, 0x6bb7, PDF_CMAP_TABLE, 3864 },
	{ 0x6bb8, 0x6bb9, PDF_CMAP_RANGE, 14408 },
	{ 0x6bba, 0x6bbb, PDF_CMAP_TABLE, 3870 },
	{ 0x6bbc, 0x6bbe, PDF_CMAP_RANGE, 14410 },
	{ 0x6bbf, 0x6bc2, PDF_CMAP_TABLE, 3872 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 14414 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_TABLE, 3876 },
	{ 0x6bc7, 0x6bca, PDF_CMAP_RANGE, 14416 },
	{ 0x6bcb, 0x6bcf, PDF_CMAP_TABLE, 3878 },
	{ 0x6bd0, 0x6bd1, PDF_CMAP_RANGE, 14422 },
	{ 0x6bd2, 0x6bdb, PDF_CMAP_TABLE, 3883 },
	{ 0x6bdc, 0x6be0, PDF_CMAP_RANGE, 14426 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 4434 },
	{ 0x6be2, 0x6be9, PDF_CMAP_RANGE, 14431 },
	{ 0x6bea, 0x6beb, PDF_CMAP_TABLE, 3893 },
	{ 0x6bec, 0x6bee, PDF_CMAP_RANGE, 14439 },
	{ 0x6bef, 0x6bef, PDF_CMAP_SINGLE, 3597 },
	{ 0x6bf0, 0x6bf2, PDF_CMAP_RANGE, 14442 },
	{ 0x6bf3, 0x6bf5, PDF_CMAP_TABLE, 3895 },
	{ 0x6bf6, 0x6bf8, PDF_CMAP_RANGE, 14446 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 6488 },
	{ 0x6bfa, 0x6bfc, PDF_CMAP_RANGE, 14449 },
	{ 0x6bfd, 0x6bff, PDF_CMAP_TABLE, 3898 },
	{ 0x6c00, 0x6c04, PDF_CMAP_RANGE, 14453 },
	{ 0x6c05, 0x6c08, PDF_CMAP_TABLE, 3901 },
	{ 0x6c09, 0x6c0b, PDF_CMAP_RANGE, 14458 },
	{ 0x6c0c, 0x6c17, PDF_CMAP_TABLE, 3905 },
	{ 0x6c18, 0x6c1a, PDF_CMAP_RANGE, 6494 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 1650 },
	{ 0x6c1c, 0x6c1e, PDF_CMAP_RANGE, 14464 },
	{ 0x6c1f, 0x6c35, PDF_CMAP_TABLE, 3917 },
	{ 0x6c36, 0x6c37, PDF_CMAP_RANGE, 14472 },
	{ 0x6c38, 0x6c38, PDF_CMAP_SINGLE, 4261 },
	{ 0x6c39, 0x6c3c, PDF_CMAP_RANGE, 14474 },
	{ 0x6c3d, 0x6c3d, PDF_CMAP_SINGLE, 4847 },
	{ 0x6c3e, 0x6c3f, PDF_CMAP_RANGE, 14478 },
	{ 0x6c40, 0x6c42, PDF_CMAP_TABLE, 3940 },
	{ 0x6c43, 0x6c45, PDF_CMAP_RANGE, 14480 },
	{ 0x6c46, 0x6c4a, PDF_CMAP_TABLE, 3943 },
	{ 0x6c4b, 0x6c4f, PDF_CMAP_RANGE, 14484 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 3864 },
	{ 0x6c51, 0x6c53, PDF_CMAP_RANGE, 14489 },
	{ 0x6c54, 0x6c57, PDF_CMAP_TABLE, 3948 },
	{ 0x6c58, 0x6c5a, PDF_CMAP_RANGE, 14493 },
	{ 0x6c5b, 0x6c61, PDF_CMAP_TABLE, 3952 },
	{ 0x6c62, 0x6c63, PDF_CMAP_RANGE, 14496 },
	{ 0x6c64, 0x6c64, PDF_CMAP_SINGLE, 3603 },
	{ 0x6c65, 0x6c67, PDF_CMAP_RANGE, 14498 },
	{ 0x6c68, 0x6c69, PDF_CMAP_RANGE, 5800 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 3749 },
	{ 0x6c6b, 0x6c6f, PDF_CMAP_RANGE, 14501 },
	{ 0x6c70, 0x6c76, PDF_CMAP_TABLE, 3959 },
	{ 0x6c77, 0x6c78, PDF_CMAP_RANGE, 14509 },
	{ 0x6c79, 0x6c79, PDF_CMAP_SINGLE, 4005 },
	{ 0x6c7a, 0x6c7c, PDF_CMAP_RANGE, 14511 },
	{ 0x6c7d, 0x6c7e, PDF_CMAP_TABLE, 3966 },
	{ 0x6c7f, 0x6c80, PDF_CMAP_RANGE, 14514 },
	{ 0x6c81, 0x6c89, PDF_CMAP_TABLE, 3968 },
	{ 0x6c8a, 0x6c8b, PDF_CMAP_RANGE, 14518 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 5799 },
	{ 0x6c8d, 0x6c8e, PDF_CMAP_RANGE, 14520 },
	{ 0x6c8f, 0x6c90, PDF_CMAP_TABLE, 3977 },
	{ 0x6c91, 0x6c92, PDF_CMAP_RANGE, 14522 },
	{ 0x6c93, 0x6c94, PDF_CMAP_TABLE, 3979 },
	{ 0x6c95, 0x6c98, PDF_CMAP_RANGE, 14524 },
	{ 0x6c99, 0x6c9b, PDF_CMAP_TABLE, 3981 },
	{ 0x6c9c, 0x6c9e, PDF_CMAP_RANGE, 14529 },
	{ 0x6c9f, 0x6cae, PDF_CMAP_TABLE, 3984 },
	{ 0x6caf, 0x6cb0, PDF_CMAP_RANGE, 14536 },
	{ 0x6cb1, 0x6cb3, PDF_CMAP_TABLE, 4000 },
	{ 0x6cb4, 0x6cb7, PDF_CMAP_RANGE, 14538 },
	{ 0x6cb8, 0x6cbf, PDF_CMAP_TABLE, 4003 },
	{ 0x6cc0, 0x6cc3, PDF_CMAP_RANGE, 14543 },
	{ 0x6cc4, 0x6cc5, PDF_CMAP_TABLE, 4011 },
	{ 0x6cc6, 0x6cc8, PDF_CMAP_RANGE, 14547 },
	{ 0x6cc9, 0x6ccc, PDF_CMAP_TABLE, 4013 },
	{ 0x6ccd, 0x6ccf, PDF_CMAP_RANGE, 14551 },
	{ 0x6cd0, 0x6cd0, PDF_CMAP_SINGLE, 5806 },
	{ 0x6cd1, 0x6cd2, PDF_CMAP_RANGE, 14554 },
	{ 0x6cd3, 0x6cd7, PDF_CMAP_TABLE, 4017 },
	{ 0x6cd8, 0x6cda, PDF_CMAP_RANGE, 14556 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 1623 },
	{ 0x6cdc, 0x6cdd, PDF_CMAP_RANGE, 14559 },
	{ 0x6cde, 0x6ce5, PDF_CMAP_TABLE, 4022 },
	{ 0x6ce6, 0x6ce7, PDF_CMAP_RANGE, 14563 },
	{ 0x6ce8, 0x6ceb, PDF_CMAP_TABLE, 4030 },
	{ 0x6cec, 0x6ced, PDF_CMAP_RANGE, 14566 },
	{ 0x6cee, 0x6cf6, PDF_CMAP_TABLE, 4034 },
	{ 0x6cf7, 0x6cf8, PDF_CMAP_RANGE, 5809 },
	{ 0x6cf9, 0x6d01, PDF_CMAP_TABLE, 4043 },
	{ 0x6d02, 0x6d03, PDF_CMAP_RANGE, 14573 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 5829 },
	{ 0x6d05, 0x6d06, PDF_CMAP_RANGE, 14575 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 5828 },
	{ 0x6d08, 0x6d0a, PDF_CMAP_RANGE, 14577 },
	{ 0x6d0b, 0x6d0e, PDF_CMAP_TABLE, 4052 },
	{ 0x6d0f, 0x6d11, PDF_CMAP_RANGE, 14581 },
	{ 0x6d12, 0x6d12, PDF_CMAP_SINGLE, 3282 },
	{ 0x6d13, 0x6d16, PDF_CMAP_RANGE, 14584 },
	{ 0x6d17, 0x6d1b, PDF_CMAP_TABLE, 4056 },
	{ 0x6d1c, 0x6d1d, PDF_CMAP_RANGE, 14589 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 1523 },
	{ 0x6d1f, 0x6d24, PDF_CMAP_RANGE, 14591 },
	{ 0x6d25, 0x6d27, PDF_CMAP_TABLE, 4061 },
	{ 0x6d28, 0x6d29, PDF_CMAP_RANGE, 14598 },
	{ 0x6d2a, 0x6d2b, PDF_CMAP_TABLE, 4064 },
	{ 0x6d2c, 0x6d2d, PDF_CMAP_RANGE, 14600 },
	{ 0x6d2e, 0x6d2e, PDF_CMAP_SINGLE, 5834 },
	{ 0x6d2f, 0x6d30, PDF_CMAP_RANGE, 14602 },
	{ 0x6d31, 0x6d35, PDF_CMAP_TABLE, 4066 },
	{ 0x6d36, 0x6d38, PDF_CMAP_RANGE, 14605 },
	{ 0x6d39, 0x6d3e, PDF_CMAP_TABLE, 4071 },
	{ 0x6d3f, 0x6d40, PDF_CMAP_RANGE, 14609 },
	{ 0x6d41, 0x6d54, PDF_CMAP_TABLE, 4077 },
	{ 0x6d55, 0x6d58, PDF_CMAP_RANGE, 14616 },
	{ 0x6d59, 0x6d60, PDF_CMAP_TABLE, 4097 },
	{ 0x6d61, 0x6d62, PDF_CMAP_RANGE, 14623 },
	{ 0x6d63, 0x6d63, PDF_CMAP_SINGLE, 5851 },
	{ 0x6d64, 0x6d65, PDF_CMAP_RANGE, 14625 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 3061 },
	{ 0x6d67, 0x6d68, PDF_CMAP_RANGE, 14627 },
	{ 0x6d69, 0x6d6a, PDF_CMAP_TABLE, 4105 },
	{ 0x6d6b, 0x6d6d, PDF_CMAP_RANGE, 14629 },
	{ 0x6d6e, 0x6d6f, PDF_CMAP_TABLE, 4107 },
	{ 0x6d70, 0x6d73, PDF_CMAP_RANGE, 14632 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 4324 },
	{ 0x6d75, 0x6d76, PDF_CMAP_RANGE, 14636 },
	{ 0x6d77, 0x6d79, PDF_CMAP_TABLE, 4109 },
	{ 0x6d7a, 0x6d7b, PDF_CMAP_RANGE, 14638 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 5850 },
	{ 0x6d7d, 0x6d81, PDF_CMAP_RANGE, 14640 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 3696 },
	{ 0x6d83, 0x6d84, PDF_CMAP_RANGE, 14645 },
	{ 0x6d85, 0x6d89, PDF_CMAP_TABLE, 4112 },
	{ 0x6d8a, 0x6d8b, PDF_CMAP_RANGE, 14648 },
	{ 0x6d8c, 0x6d8e, PDF_CMAP_TABLE, 4117 },
	{ 0x6d8f, 0x6d90, PDF_CMAP_RANGE, 14651 },
	{ 0x6d91, 0x6d92, PDF_CMAP_TABLE, 4120 },
	{ 0x6d93, 0x6d94, PDF_CMAP_RANGE, 5846 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 3644 },
	{ 0x6d96, 0x6d9a, PDF_CMAP_RANGE, 14654 },
	{ 0x6d9b, 0x6dab, PDF_CMAP_TABLE, 4122 },
	{ 0x6dac, 0x6dad, PDF_CMAP_RANGE, 14662 },
	{ 0x6dae, 0x6daf, PDF_CMAP_TABLE, 4139 },
	{ 0x6db0, 0x6db1, PDF_CMAP_RANGE, 14664 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 4161 },
	{ 0x6db3, 0x6db4, PDF_CMAP_RANGE, 14666 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 1892 },
	{ 0x6db6, 0x6db7, PDF_CMAP_RANGE, 14668 },
	{ 0x6db8, 0x6db8, PDF_CMAP_SINGLE, 1931 },
	{ 0x6db9, 0x6dbe, PDF_CMAP_RANGE, 14670 },
	{ 0x6dbf, 0x6dc0, PDF_CMAP_TABLE, 4141 },
	{ 0x6dc1, 0x6dc3, PDF_CMAP_RANGE, 14676 },
	{ 0x6dc4, 0x6dc7, PDF_CMAP_TABLE, 4143 },
	{ 0x6dc8, 0x6dca, PDF_CMAP_RANGE, 14679 },
	{ 0x6dcb, 0x6dcc, PDF_CMAP_TABLE, 4147 },
	{ 0x6dcd, 0x6dd0, PDF_CMAP_RANGE, 14682 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 3454 },
	{ 0x6dd2, 0x6dd5, PDF_CMAP_RANGE, 14686 },
	{ 0x6dd6, 0x6dd9, PDF_CMAP_TABLE, 4149 },
	{ 0x6dda, 0x6ddc, PDF_CMAP_RANGE, 14691 },
	{ 0x6ddd, 0x6de1, PDF_CMAP_TABLE, 4153 },
	{ 0x6de2, 0x6de3, PDF_CMAP_RANGE, 14695 },
	{ 0x6de4, 0x6de6, PDF_CMAP_TABLE, 4158 },
	{ 0x6de7, 0x6de9, PDF_CMAP_RANGE, 14698 },
	{ 0x6dea, 0x6dee, PDF_CMAP_TABLE, 4161 },
	{ 0x6def, 0x6df0, PDF_CMAP_RANGE, 14702 },
	{ 0x6df1, 0x6dfc, PDF_CMAP_TABLE, 4166 },
	{ 0x6dfd, 0x6e04, PDF_CMAP_RANGE, 14707 },
	{ 0x6e05, 0x6e05, PDF_CMAP_SINGLE, 3170 },
	{ 0x6e06, 0x6e09, PDF_CMAP_RANGE, 14715 },
	{ 0x6e0a, 0x6e11, PDF_CMAP_TABLE, 4178 },
	{ 0x6e12, 0x6e13, PDF_CMAP_RANGE, 14721 },
	{ 0x6e14, 0x6e17, PDF_CMAP_TABLE, 4186 },
	{ 0x6e18, 0x6e19, PDF_CMAP_RANGE, 14724 },
	{ 0x6e1a, 0x6e1a, PDF_CMAP_SINGLE, 5852 },
	{ 0x6e1b, 0x6e1c, PDF_CMAP_RANGE, 14726 },
	{ 0x6e1d, 0x6e1d, PDF_CMAP_SINGLE, 4298 },
	{ 0x6e1e, 0x6e1f, PDF_CMAP_RANGE, 14728 },
	{ 0x6e20, 0x6e26, PDF_CMAP_TABLE, 4190 },
	{ 0x6e27, 0x6e28, PDF_CMAP_RANGE, 14731 },
	{ 0x6e29, 0x6e2f, PDF_CMAP_TABLE, 4197 },
	{ 0x6e30, 0x6e31, PDF_CMAP_RANGE, 14735 },
	{ 0x6e32, 0x6e34, PDF_CMAP_TABLE, 4204 },
	{ 0x6e35, 0x6e37, PDF_CMAP_RANGE, 14738 },
	{ 0x6e38, 0x6e3a, PDF_CMAP_TABLE, 4207 },
	{ 0x6e3b, 0x6e3d, PDF_CMAP_RANGE, 14742 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 8035 },
	{ 0x6e3f, 0x6e42, PDF_CMAP_RANGE, 14745 },
	{ 0x6e43, 0x6e44, PDF_CMAP_TABLE, 4210 },
	{ 0x6e45, 0x6e4c, PDF_CMAP_RANGE, 14749 },
	{ 0x6e4d, 0x6e4e, PDF_CMAP_TABLE, 4212 },
	{ 0x6e4f, 0x6e52, PDF_CMAP_RANGE, 14757 },
	{ 0x6e53, 0x6e54, PDF_CMAP_RANGE, 5874 },
	{ 0x6e55, 0x6e58, PDF_CMAP_TABLE, 4214 },
	{ 0x6e59, 0x6e5a, PDF_CMAP_RANGE, 14763 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 4448 },
	{ 0x6e5c, 0x6e5d, PDF_CMAP_RANGE, 14765 },
	{ 0x6e5e, 0x6e5f, PDF_CMAP_TABLE, 4218 },
	{ 0x6e60, 0x6e6a, PDF_CMAP_RANGE, 14767 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 5870 },
	{ 0x6e6c, 0x6e6d, PDF_CMAP_RANGE, 14778 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 4220 },
	{ 0x6e70, 0x6e7d, PDF_CMAP_RANGE, 14780 },
	{ 0x6e7e, 0x6e7f, PDF_CMAP_TABLE, 4222 },
	{ 0x6e80, 0x6e82, PDF_CMAP_RANGE, 14794 },
	{ 0x6e83, 0x6e86, PDF_CMAP_TABLE, 4224 },
	{ 0x6e87, 0x6e88, PDF_CMAP_RANGE, 14798 },
	{ 0x6e89, 0x6e89, PDF_CMAP_SINGLE, 1731 },
	{ 0x6e8a, 0x6e8e, PDF_CMAP_RANGE, 14800 },
	{ 0x6e8f, 0x6e90, PDF_CMAP_TABLE, 4228 },
	{ 0x6e91, 0x6e95, PDF_CMAP_RANGE, 14805 },
	{ 0x6e96, 0x6e98, PDF_CMAP_TABLE, 4230 },
	{ 0x6e99, 0x6e9b, PDF_CMAP_RANGE, 14811 },
	{ 0x6e9c, 0x6e9f, PDF_CMAP_TABLE, 4233 },
	{ 0x6ea0, 0x6ea1, PDF_CMAP_RANGE, 14815 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 4206 },
	{ 0x6ea3, 0x6ea4, PDF_CMAP_RANGE, 14817 },
	{ 0x6ea5, 0x6ea7, PDF_CMAP_TABLE, 4237 },
	{ 0x6ea8, 0x6ea9, PDF_CMAP_RANGE, 14820 },
	{ 0x6eaa, 0x6eaa, PDF_CMAP_SINGLE, 3863 },
	{ 0x6eab, 0x6eae, PDF_CMAP_RANGE, 14822 },
	{ 0x6eaf, 0x6eb7, PDF_CMAP_TABLE, 4240 },
	{ 0x6eb8, 0x6eb9, PDF_CMAP_RANGE, 14829 },
	{ 0x6eba, 0x6ebd, PDF_CMAP_TABLE, 4249 },
	{ 0x6ebe, 0x6ec0, PDF_CMAP_RANGE, 14832 },
	{ 0x6ec1, 0x6ec7, PDF_CMAP_TABLE, 4253 },
	{ 0x6ec8, 0x6eca, PDF_CMAP_RANGE, 14837 },
	{ 0x6ecb, 0x6ed7, PDF_CMAP_TABLE, 4260 },
	{ 0x6ed8, 0x6ed9, PDF_CMAP_RANGE, 14844 },
	{ 0x6eda, 0x6eda, PDF_CMAP_SINGLE, 1871 },
	{ 0x6edb, 0x6edd, PDF_CMAP_RANGE, 14846 },
	{ 0x6ede, 0x6ee9, PDF_CMAP_TABLE, 4273 },
	{ 0x6eea, 0x6eeb, PDF_CMAP_RANGE, 14851 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 8011 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 14853 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 8850 },
	{ 0x6ef0, 0x6ef1, PDF_CMAP_RANGE, 14855 },
	{ 0x6ef2, 0x6ef4, PDF_CMAP_TABLE, 4285 },
	{ 0x6ef5, 0x6ef6, PDF_CMAP_RANGE, 14858 },
	{ 0x6ef7, 0x6ef9, PDF_CMAP_TABLE, 4288 },
	{ 0x6efa, 0x6efe, PDF_CMAP_RANGE, 14860 },
	{ 0x6eff, 0x6f02, PDF_CMAP_TABLE, 4291 },
	{ 0x6f03, 0x6f05, PDF_CMAP_RANGE, 14866 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 3072 },
	{ 0x6f07, 0x6f08, PDF_CMAP_RANGE, 14869 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 5907 },
	{ 0x6f0a, 0x6f0e, PDF_CMAP_RANGE, 14871 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 2640 },
	{ 0x6f10, 0x6f12, PDF_CMAP_RANGE, 14876 },
	{ 0x6f13, 0x6f15, PDF_CMAP_TABLE, 4295 },
	{ 0x6f16, 0x6f19, PDF_CMAP_RANGE, 14879 },
	{ 0x6f1a, 0x6f1a, PDF_CMAP_SINGLE, 8369 },
	{ 0x6f1b, 0x6f1f, PDF_CMAP_RANGE, 14883 },
	{ 0x6f20, 0x6f24, PDF_CMAP_TABLE, 4298 },
	{ 0x6f25, 0x6f28, PDF_CMAP_RANGE, 14889 },
	{ 0x6f29, 0x6f33, PDF_CMAP_TABLE, 4303 },
	{ 0x6f34, 0x6f35, PDF_CMAP_RANGE, 14895 },
	{ 0x6f36, 0x6f38, PDF_CMAP_TABLE, 4314 },
	{ 0x6f39, 0x6f3d, PDF_CMAP_RANGE, 14898 },
	{ 0x6f3e, 0x6f41, PDF_CMAP_TABLE, 4317 },
	{ 0x6f42, 0x6f45, PDF_CMAP_RANGE, 14904 },
	{ 0x6f46, 0x6f47, PDF_CMAP_RANGE, 5897 },
	{ 0x6f48, 0x6f4a, PDF_CMAP_RANGE, 14908 },
	{ 0x6f4b, 0x6f4d, PDF_CMAP_TABLE, 4321 },
	{ 0x6f4e, 0x6f50, PDF_CMAP_RANGE, 14912 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 8382 },
	{ 0x6f52, 0x6f53, PDF_CMAP_RANGE, 14915 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 8116 },
	{ 0x6f55, 0x6f57, PDF_CMAP_RANGE, 14917 },
	{ 0x6f58, 0x6f59, PDF_CMAP_TABLE, 4324 },
	{ 0x6f5a, 0x6f5b, PDF_CMAP_RANGE, 14920 },
	{ 0x6f5c, 0x6f5e, PDF_CMAP_TABLE, 4326 },
	{ 0x6f5f, 0x6f61, PDF_CMAP_RANGE, 14923 },
	{ 0x6f62, 0x6f66, PDF_CMAP_TABLE, 4329 },
	{ 0x6f67, 0x6f6c, PDF_CMAP_RANGE, 14928 },
	{ 0x6f6d, 0x6f74, PDF_CMAP_TABLE, 4334 },
	{ 0x6f75, 0x6f76, PDF_CMAP_RANGE, 14936 },
	{ 0x6f77, 0x6f7c, PDF_CMAP_TABLE, 4342 },
	{ 0x6f7d, 0x6f7e, PDF_CMAP_RANGE, 14940 },
	{ 0x6f7f, 0x6f80, PDF_CMAP_TABLE, 4348 },
	{ 0x6f81, 0x6f83, PDF_CMAP_RANGE, 14942 },
	{ 0x6f84, 0x6f89, PDF_CMAP_TABLE, 4350 },
	{ 0x6f8a, 0x6f8b, PDF_CMAP_RANGE, 14946 },
	{ 0x6f8c, 0x6f8e, PDF_CMAP_TABLE, 4356 },
	{ 0x6f8f, 0x6f96, PDF_CMAP_RANGE, 14948 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 8093 },
	{ 0x6f98, 0x6f9b, PDF_CMAP_RANGE, 14956 },
	{ 0x6f9c, 0x6f9c, PDF_CMAP_SINGLE, 2472 },
	{ 0x6f9d, 0x6f9f, PDF_CMAP_RANGE, 14960 },
	{ 0x6fa0, 0x6fa1, PDF_CMAP_TABLE, 4359 },
	{ 0x6fa2, 0x6fa3, PDF_CMAP_RANGE, 14963 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 8804 },
	{ 0x6fa5, 0x6fa6, PDF_CMAP_RANGE, 14965 },
	{ 0x6fa7, 0x6fa9, PDF_CMAP_TABLE, 4361 },
	{ 0x6faa, 0x6fad, PDF_CMAP_RANGE, 14968 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 9182 },
	{ 0x6faf, 0x6fb0, PDF_CMAP_RANGE, 14972 },
	{ 0x6fb1, 0x6fb3, PDF_CMAP_TABLE, 4364 },
	{ 0x6fb4, 0x6fb5, PDF_CMAP_RANGE, 14975 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 5920 },
	{ 0x6fb7, 0x6fb8, PDF_CMAP_RANGE, 14977 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5919 },
	{ 0x6fba, 0x6fbf, PDF_CMAP_RANGE, 14979 },
	{ 0x6fc0, 0x6fc3, PDF_CMAP_TABLE, 4367 },
	{ 0x6fc4, 0x6fc8, PDF_CMAP_RANGE, 14985 },
	{ 0x6fc9, 0x6fc9, PDF_CMAP_SINGLE, 5917 },
	{ 0x6fca, 0x6fd0, PDF_CMAP_RANGE, 14990 },
	{ 0x6fd1, 0x6fd2, PDF_CMAP_TABLE, 4371 },
	{ 0x6fd3, 0x6fd4, PDF_CMAP_RANGE, 14997 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 8491 },
	{ 0x6fd6, 0x6fd7, PDF_CMAP_RANGE, 14999 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 8357 },
	{ 0x6fd9, 0x6fda, PDF_CMAP_RANGE, 15001 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 9867 },
	{ 0x6fdc, 0x6fdd, PDF_CMAP_RANGE, 15003 },
	{ 0x6fde, 0x6fe1, PDF_CMAP_TABLE, 4373 },
	{ 0x6fe2, 0x6fe3, PDF_CMAP_RANGE, 15005 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 8557 },
	{ 0x6fe5, 0x6fea, PDF_CMAP_RANGE, 15007 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 8193 },
	{ 0x6fec, 0x6fed, PDF_CMAP_RANGE, 15013 },
	{ 0x6fee, 0x6ff1, PDF_CMAP_TABLE, 4377 },
	{ 0x6ff2, 0x6ff9, PDF_CMAP_RANGE, 15015 },
	{ 0x6ffa, 0x6fff, PDF_CMAP_TABLE, 4381 },
	{ 0x7000, 0x7004, PDF_CMAP_RANGE, 15026 },
	{ 0x7005, 0x7006, PDF_CMAP_TABLE, 4387 },
	{ 0x7007, 0x7008, PDF_CMAP_RANGE, 15031 },
	{ 0x7009, 0x700b, PDF_CMAP_TABLE, 4389 },
	{ 0x700c, 0x700e, PDF_CMAP_RANGE, 15034 },
	{ 0x700f, 0x7011, PDF_CMAP_TABLE, 4392 },
	{ 0x7012, 0x7014, PDF_CMAP_RANGE, 15038 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 7759 },
	{ 0x7016, 0x7017, PDF_CMAP_RANGE, 15041 },
	{ 0x7018, 0x7020, PDF_CMAP_TABLE, 4395 },
	{ 0x7021, 0x7022, PDF_CMAP_RANGE, 15046 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 5928 },
	{ 0x7024, 0x7026, PDF_CMAP_RANGE, 15048 },
	{ 0x7027, 0x7028, PDF_CMAP_TABLE, 4404 },
	{ 0x7029, 0x702f, PDF_CMAP_RANGE, 15051 },
	{ 0x7030, 0x7032, PDF_CMAP_TABLE, 4406 },
	{ 0x7033, 0x7034, PDF_CMAP_RANGE, 15059 },
	{ 0x7035, 0x7035, PDF_CMAP_SINGLE, 5931 },
	{ 0x7036, 0x7038, PDF_CMAP_RANGE, 15061 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 5930 },
	{ 0x703a, 0x703d, PDF_CMAP_RANGE, 15064 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 8186 },
	{ 0x703f, 0x7042, PDF_CMAP_RANGE, 15068 },
	{ 0x7043, 0x7044, PDF_CMAP_TABLE, 4409 },
	{ 0x7045, 0x704b, PDF_CMAP_RANGE, 15072 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 1849 },
	{ 0x704d, 0x704e, PDF_CMAP_RANGE, 15079 },
	{ 0x704f, 0x7051, PDF_CMAP_TABLE, 4411 },
	{ 0x7052, 0x7054, PDF_CMAP_RANGE, 15082 },
	{ 0x7055, 0x7055, PDF_CMAP_SINGLE, 8203 },
	{ 0x7056, 0x7057, PDF_CMAP_RANGE, 15085 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 8550 },
	{ 0x7059, 0x705c, PDF_CMAP_RANGE, 15087 },
	{ 0x705d, 0x705e, PDF_CMAP_TABLE, 4414 },
	{ 0x705f, 0x7062, PDF_CMAP_RANGE, 15091 },
	{ 0x7063, 0x7064, PDF_CMAP_TABLE, 4416 },
	{ 0x7065, 0x7066, PDF_CMAP_RANGE, 15095 },
	{ 0x7067, 0x7067, PDF_CMAP_SINGLE, 9191 },
	{ 0x7068, 0x706a, PDF_CMAP_RANGE, 15097 },
	{ 0x706b, 0x7070, PDF_CMAP_TABLE, 4418 },
	{ 0x7071, 0x7074, PDF_CMAP_RANGE, 15101 },
	{ 0x7075, 0x7078, PDF_CMAP_TABLE, 4424 },
	{ 0x7079, 0x707b, PDF_CMAP_RANGE, 15106 },
	{ 0x707c, 0x7080, PDF_CMAP_TABLE, 4428 },
	{ 0x7081, 0x7084, PDF_CMAP_RANGE, 15110 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 6419 },
	{ 0x7086, 0x7088, PDF_CMAP_RANGE, 15114 },
	{ 0x7089, 0x708a, PDF_CMAP_TABLE, 4433 },
	{ 0x708b, 0x708d, PDF_CMAP_RANGE, 15117 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 4096 },
	{ 0x708f, 0x7091, PDF_CMAP_RANGE, 15120 },
	{ 0x7092, 0x7096, PDF_CMAP_TABLE, 4435 },
	{ 0x7097, 0x7098, PDF_CMAP_RANGE, 15124 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 4554 },
	{ 0x709a, 0x709b, PDF_CMAP_RANGE, 15126 },
	{ 0x709c, 0x709d, PDF_CMAP_TABLE, 4440 },
	{ 0x709e, 0x70aa, PDF_CMAP_RANGE, 15128 },
	{ 0x70ab, 0x70b3, PDF_CMAP_TABLE, 4442 },
	{ 0x70b4, 0x70b6, PDF_CMAP_RANGE, 15143 },
	{ 0x70b7, 0x70bd, PDF_CMAP_TABLE, 4451 },
	{ 0x70be, 0x70bf, PDF_CMAP_RANGE, 15147 },
	{ 0x70c0, 0x70c3, PDF_CMAP_TABLE, 4458 },
	{ 0x70c4, 0x70c7, PDF_CMAP_RANGE, 15149 },
	{ 0x70c8, 0x70ca, PDF_CMAP_TABLE, 4462 },
	{ 0x70cb, 0x70ce, PDF_CMAP_RANGE, 15154 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 8611 },
	{ 0x70d0, 0x70d7, PDF_CMAP_RANGE, 15158 },
	{ 0x70d8, 0x70db, PDF_CMAP_TABLE, 4465 },
	{ 0x70dc, 0x70de, PDF_CMAP_RANGE, 15167 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 4085 },
	{ 0x70e0, 0x70e3, PDF_CMAP_RANGE, 15170 },
	{ 0x70e4, 0x70ef, PDF_CMAP_TABLE, 4469 },
	{ 0x70f0, 0x70f3, PDF_CMAP_RANGE, 15177 },
	{ 0x70f4, 0x70f4, PDF_CMAP_SINGLE, 8570 },
	{ 0x70f5, 0x70f6, PDF_CMAP_RANGE, 15181 },
	{ 0x70f7, 0x70f9, PDF_CMAP_TABLE, 4481 },
	{ 0x70fa, 0x70fc, PDF_CMAP_RANGE, 15184 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 1670 },
	{ 0x70fe, 0x7108, PDF_CMAP_RANGE, 15187 },
	{ 0x7109, 0x710a, PDF_CMAP_TABLE, 4484 },
	{ 0x710b, 0x710f, PDF_CMAP_RANGE, 15198 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 6615 },
	{ 0x7111, 0x7112, PDF_CMAP_RANGE, 15203 },
	{ 0x7113, 0x711a, PDF_CMAP_TABLE, 4486 },
	{ 0x711b, 0x7120, PDF_CMAP_RANGE, 15207 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 8613 },
	{ 0x7122, 0x7125, PDF_CMAP_RANGE, 15213 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2186 },
	{ 0x7127, 0x712e, PDF_CMAP_RANGE, 15217 },
	{ 0x712f, 0x7131, PDF_CMAP_TABLE, 4494 },
	{ 0x7132, 0x7135, PDF_CMAP_RANGE, 15225 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 3222 },
	{ 0x7137, 0x7144, PDF_CMAP_RANGE, 15229 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 6623 },
	{ 0x7146, 0x7148, PDF_CMAP_RANGE, 15243 },
	{ 0x7149, 0x714e, PDF_CMAP_TABLE, 4497 },
	{ 0x714f, 0x7151, PDF_CMAP_RANGE, 15248 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 9430 },
	{ 0x7153, 0x715b, PDF_CMAP_RANGE, 15251 },
	{ 0x715c, 0x715e, PDF_CMAP_TABLE, 4503 },
	{ 0x715f, 0x7161, PDF_CMAP_RANGE, 15261 },
	{ 0x7162, 0x7169, PDF_CMAP_TABLE, 4506 },
	{ 0x716a, 0x716b, PDF_CMAP_RANGE, 15266 },
	{ 0x716c, 0x716e, PDF_CMAP_TABLE, 4514 },
	{ 0x716f, 0x7171, PDF_CMAP_RANGE, 15269 },
	{ 0x7172, 0x7173, PDF_CMAP_TABLE, 4517 },
	{ 0x7174, 0x7177, PDF_CMAP_RANGE, 15272 },
	{ 0x7178, 0x717a, PDF_CMAP_TABLE, 4519 },
	{ 0x717b, 0x717c, PDF_CMAP_RANGE, 15277 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 3320 },
	{ 0x717e, 0x7183, PDF_CMAP_RANGE, 15279 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 3861 },
	{ 0x7185, 0x7189, PDF_CMAP_RANGE, 15285 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 4007 },
	{ 0x718b, 0x718e, PDF_CMAP_RANGE, 15290 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 4053 },
	{ 0x7190, 0x7191, PDF_CMAP_RANGE, 15294 },
	{ 0x7192, 0x7194, PDF_CMAP_TABLE, 4522 },
	{ 0x7195, 0x7196, PDF_CMAP_RANGE, 15297 },
	{ 0x7197, 0x7199, PDF_CMAP_TABLE, 4525 },
	{ 0x719a, 0x719e, PDF_CMAP_RANGE, 15299 },
	{ 0x719f, 0x71a0, PDF_CMAP_TABLE, 4528 },
	{ 0x71a1, 0x71a7, PDF_CMAP_RANGE, 15304 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 6631 },
	{ 0x71a9, 0x71ab, PDF_CMAP_RANGE, 15311 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 969 },
	{ 0x71ad, 0x71b0, PDF_CMAP_RANGE, 15314 },
	{ 0x71b1, 0x71b5, PDF_CMAP_TABLE, 4530 },
	{ 0x71b6, 0x71b8, PDF_CMAP_RANGE, 15320 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 6642 },
	{ 0x71ba, 0x71bd, PDF_CMAP_RANGE, 15323 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 7815 },
	{ 0x71bf, 0x71c0, PDF_CMAP_RANGE, 15327 },
	{ 0x71c1, 0x71c3, PDF_CMAP_TABLE, 4535 },
	{ 0x71c4, 0x71c7, PDF_CMAP_RANGE, 15330 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 7866 },
	{ 0x71c9, 0x71cd, PDF_CMAP_RANGE, 15334 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 2576 },
	{ 0x71cf, 0x71d1, PDF_CMAP_RANGE, 15339 },
	{ 0x71d2, 0x71d5, PDF_CMAP_TABLE, 4538 },
	{ 0x71d6, 0x71d8, PDF_CMAP_RANGE, 15343 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 8556 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 15346 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 9433 },
	{ 0x71dd, 0x71de, PDF_CMAP_RANGE, 15348 },
	{ 0x71df, 0x71e0, PDF_CMAP_TABLE, 4542 },
	{ 0x71e1, 0x71e4, PDF_CMAP_RANGE, 15350 },
	{ 0x71e5, 0x71e7, PDF_CMAP_TABLE, 4544 },
	{ 0x71e8, 0x71ec, PDF_CMAP_RANGE, 15354 },
	{ 0x71ed, 0x71ee, PDF_CMAP_TABLE, 4547 },
	{ 0x71ef, 0x71f3, PDF_CMAP_RANGE, 15359 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 8029 },
	{ 0x71f5, 0x71f8, PDF_CMAP_RANGE, 15364 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 6636 },
	{ 0x71fa, 0x71fb, PDF_CMAP_RANGE, 15368 },
	{ 0x71fc, 0x71ff, PDF_CMAP_TABLE, 4549 },
	{ 0x7200, 0x7205, PDF_CMAP_RANGE, 15372 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 1045 },
	{ 0x7207, 0x720c, PDF_CMAP_RANGE, 15378 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 8521 },
	{ 0x720e, 0x720f, PDF_CMAP_RANGE, 15384 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 8262 },
	{ 0x7211, 0x721a, PDF_CMAP_RANGE, 15386 },
	{ 0x721b, 0x721d, PDF_CMAP_TABLE, 4553 },
	{ 0x721e, 0x7227, PDF_CMAP_RANGE, 15397 },
	{ 0x7228, 0x722c, PDF_CMAP_TABLE, 4556 },
	{ 0x722d, 0x722f, PDF_CMAP_RANGE, 15409 },
	{ 0x7230, 0x7232, PDF_CMAP_TABLE, 4561 },
	{ 0x7233, 0x7234, PDF_CMAP_RANGE, 15412 },
	{ 0x7235, 0x723f, PDF_CMAP_TABLE, 4564 },
	{ 0x7240, 0x7246, PDF_CMAP_RANGE, 15415 },
	{ 0x7247, 0x7248, PDF_CMAP_TABLE, 4575 },
	{ 0x7249, 0x724b, PDF_CMAP_RANGE, 15422 },
	{ 0x724c, 0x724d, PDF_CMAP_TABLE, 4577 },
	{ 0x724e, 0x7251, PDF_CMAP_RANGE, 15425 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 6506 },
	{ 0x7253, 0x7255, PDF_CMAP_RANGE, 15429 },
	{ 0x7256, 0x7262, PDF_CMAP_TABLE, 4579 },
	{ 0x7263, 0x7265, PDF_CMAP_RANGE, 15437 },
	{ 0x7266, 0x7269, PDF_CMAP_TABLE, 4592 },
	{ 0x726a, 0x726d, PDF_CMAP_RANGE, 15441 },
	{ 0x726e, 0x726f, PDF_CMAP_TABLE, 4596 },
	{ 0x7270, 0x7271, PDF_CMAP_RANGE, 15445 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 3381 },
	{ 0x7273, 0x7274, PDF_CMAP_RANGE, 15447 },
	{ 0x7275, 0x7275, PDF_CMAP_SINGLE, 3104 },
	{ 0x7276, 0x7278, PDF_CMAP_RANGE, 15449 },
	{ 0x7279, 0x727a, PDF_CMAP_TABLE, 4598 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 15452 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 8396 },
	{ 0x727e, 0x727f, PDF_CMAP_RANGE, 6471 },
	{ 0x7280, 0x7281, PDF_CMAP_TABLE, 4600 },
	{ 0x7282, 0x7283, PDF_CMAP_RANGE, 15454 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 6473 },
	{ 0x7285, 0x7289, PDF_CMAP_RANGE, 15456 },
	{ 0x728a, 0x728f, PDF_CMAP_TABLE, 4602 },
	{ 0x7290, 0x7291, PDF_CMAP_RANGE, 15463 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 6477 },
	{ 0x7293, 0x7295, PDF_CMAP_RANGE, 15465 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 9024 },
	{ 0x7297, 0x729e, PDF_CMAP_RANGE, 15468 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 6467 },
	{ 0x72a0, 0x72a1, PDF_CMAP_RANGE, 15476 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 7889 },
	{ 0x72a3, 0x72a6, PDF_CMAP_RANGE, 15478 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 8620 },
	{ 0x72a8, 0x72ab, PDF_CMAP_RANGE, 15482 },
	{ 0x72ac, 0x72b0, PDF_CMAP_TABLE, 4608 },
	{ 0x72b1, 0x72b3, PDF_CMAP_RANGE, 15487 },
	{ 0x72b4, 0x72b6, PDF_CMAP_TABLE, 4613 },
	{ 0x72b7, 0x72b8, PDF_CMAP_RANGE, 5618 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 4273 },
	{ 0x72ba, 0x72bf, PDF_CMAP_RANGE, 15491 },
	{ 0x72c0, 0x72c4, PDF_CMAP_TABLE, 4616 },
	{ 0x72c5, 0x72c7, PDF_CMAP_RANGE, 15497 },
	{ 0x72c8, 0x72c8, PDF_CMAP_SINGLE, 1056 },
	{ 0x72c9, 0x72cc, PDF_CMAP_RANGE, 15500 },
	{ 0x72cd, 0x72d2, PDF_CMAP_TABLE, 4621 },
	{ 0x72d3, 0x72d6, PDF_CMAP_RANGE, 15506 },
	{ 0x72d7, 0x72d9, PDF_CMAP_TABLE, 4627 },
	{ 0x72da, 0x72dd, PDF_CMAP_RANGE, 15511 },
	{ 0x72de, 0x72e1, PDF_CMAP_TABLE, 4630 },
	{ 0x72e2, 0x72e7, PDF_CMAP_RANGE, 15516 },
	{ 0x72e8, 0x72e9, PDF_CMAP_TABLE, 4634 },
	{ 0x72ea, 0x72eb, PDF_CMAP_RANGE, 15522 },
	{ 0x72ec, 0x72f4, PDF_CMAP_TABLE, 4636 },
	{ 0x72f5, 0x72f6, PDF_CMAP_RANGE, 15524 },
	{ 0x72f7, 0x72f9, PDF_CMAP_TABLE, 4645 },
	{ 0x72fa, 0x72fb, PDF_CMAP_RANGE, 5634 },
	{ 0x72fc, 0x72fd, PDF_CMAP_TABLE, 4648 },
	{ 0x72fe, 0x7300, PDF_CMAP_RANGE, 15526 },
	{ 0x7301, 0x7303, PDF_CMAP_TABLE, 4650 },
	{ 0x7304, 0x7309, PDF_CMAP_RANGE, 15530 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 5639 },
	{ 0x730b, 0x730d, PDF_CMAP_RANGE, 15536 },
	{ 0x730e, 0x730e, PDF_CMAP_SINGLE, 2589 },
	{ 0x730f, 0x7312, PDF_CMAP_RANGE, 15539 },
	{ 0x7313, 0x7317, PDF_CMAP_TABLE, 4653 },
	{ 0x7318, 0x731a, PDF_CMAP_RANGE, 15544 },
	{ 0x731b, 0x731e, PDF_CMAP_TABLE, 4658 },
	{ 0x731f, 0x7320, PDF_CMAP_RANGE, 15547 },
	{ 0x7321, 0x7322, PDF_CMAP_TABLE, 4662 },
	{ 0x7323, 0x7324, PDF_CMAP_RANGE, 15549 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 5645 },
	{ 0x7326, 0x7328, PDF_CMAP_RANGE, 15551 },
	{ 0x7329, 0x732e, PDF_CMAP_TABLE, 4664 },
	{ 0x732f, 0x7330, PDF_CMAP_RANGE, 15555 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 5648 },
	{ 0x7332, 0x7333, PDF_CMAP_RANGE, 15557 },
	{ 0x7334, 0x733b, PDF_CMAP_TABLE, 4670 },
	{ 0x733c, 0x733d, PDF_CMAP_RANGE, 15561 },
	{ 0x733e, 0x7341, PDF_CMAP_TABLE, 4678 },
	{ 0x7342, 0x7343, PDF_CMAP_RANGE, 15564 },
	{ 0x7344, 0x7345, PDF_CMAP_TABLE, 4682 },
	{ 0x7346, 0x734c, PDF_CMAP_RANGE, 15566 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 5650 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 15573 },
	{ 0x7350, 0x7352, PDF_CMAP_TABLE, 4684 },
	{ 0x7353, 0x7356, PDF_CMAP_RANGE, 15576 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 5651 },
	{ 0x7358, 0x735f, PDF_CMAP_RANGE, 15580 },
	{ 0x7360, 0x7360, PDF_CMAP_SINGLE, 5652 },
	{ 0x7361, 0x7367, PDF_CMAP_RANGE, 15588 },
	{ 0x7368, 0x7372, PDF_CMAP_TABLE, 4687 },
	{ 0x7373, 0x7374, PDF_CMAP_RANGE, 15598 },
	{ 0x7375, 0x7380, PDF_CMAP_TABLE, 4698 },
	{ 0x7381, 0x7383, PDF_CMAP_RANGE, 15604 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 4041 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 15607 },
	{ 0x7387, 0x738b, PDF_CMAP_TABLE, 4710 },
	{ 0x738c, 0x738d, PDF_CMAP_RANGE, 15611 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6168 },
	{ 0x738f, 0x7390, PDF_CMAP_RANGE, 15613 },
	{ 0x7391, 0x7391, PDF_CMAP_SINGLE, 6169 },
	{ 0x7392, 0x7395, PDF_CMAP_RANGE, 15615 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 2288 },
	{ 0x7397, 0x739a, PDF_CMAP_RANGE, 15619 },
	{ 0x739b, 0x739b, PDF_CMAP_SINGLE, 2705 },
	{ 0x739c, 0x739e, PDF_CMAP_RANGE, 15623 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 6172 },
	{ 0x73a0, 0x73a1, PDF_CMAP_RANGE, 15626 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 6171 },
	{ 0x73a3, 0x73a8, PDF_CMAP_RANGE, 15628 },
	{ 0x73a9, 0x73ab, PDF_CMAP_TABLE, 4715 },
	{ 0x73ac, 0x73ad, PDF_CMAP_RANGE, 15635 },
	{ 0x73ae, 0x73b3, PDF_CMAP_TABLE, 4718 },
	{ 0x73b4, 0x73b6, PDF_CMAP_RANGE, 15638 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 6176 },
	{ 0x73b8, 0x73b9, PDF_CMAP_RANGE, 15641 },
	{ 0x73ba, 0x73bb, PDF_CMAP_TABLE, 4724 },
	{ 0x73bc, 0x73bf, PDF_CMAP_RANGE, 15643 },
	{ 0x73c0, 0x73c2, PDF_CMAP_TABLE, 4726 },
	{ 0x73c3, 0x73c7, PDF_CMAP_RANGE, 15648 },
	{ 0x73c8, 0x73ca, PDF_CMAP_TABLE, 4729 },
	{ 0x73cb, 0x73cc, PDF_CMAP_RANGE, 15653 },
	{ 0x73cd, 0x73d1, PDF_CMAP_TABLE, 4732 },
	{ 0x73d2, 0x73d8, PDF_CMAP_RANGE, 15656 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 6182 },
	{ 0x73da, 0x73dd, PDF_CMAP_RANGE, 15663 },
	{ 0x73de, 0x73e0, PDF_CMAP_TABLE, 4737 },
	{ 0x73e1, 0x73e4, PDF_CMAP_RANGE, 15668 },
	{ 0x73e5, 0x73e9, PDF_CMAP_TABLE, 4740 },
	{ 0x73ea, 0x73ec, PDF_CMAP_RANGE, 15674 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 1003 },
	{ 0x73ee, 0x73f1, PDF_CMAP_RANGE, 15677 },
	{ 0x73f2, 0x73f2, PDF_CMAP_SINGLE, 6189 },
	{ 0x73f3, 0x73fd, PDF_CMAP_RANGE, 15681 },
	{ 0x73fe, 0x73ff, PDF_CMAP_TABLE, 4745 },
	{ 0x7400, 0x7402, PDF_CMAP_RANGE, 15693 },
	{ 0x7403, 0x7406, PDF_CMAP_TABLE, 4747 },
	{ 0x7407, 0x7408, PDF_CMAP_RANGE, 15697 },
	{ 0x7409, 0x740a, PDF_CMAP_TABLE, 4751 },
	{ 0x740b, 0x740e, PDF_CMAP_RANGE, 15699 },
	{ 0x740f, 0x7410, PDF_CMAP_TABLE, 4753 },
	{ 0x7411, 0x7419, PDF_CMAP_RANGE, 15703 },
	{ 0x741a, 0x741b, PDF_CMAP_TABLE, 4755 },
	{ 0x741c, 0x7421, PDF_CMAP_RANGE, 15712 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 4638 },
	{ 0x7423, 0x7424, PDF_CMAP_RANGE, 15718 },
	{ 0x7425, 0x7430, PDF_CMAP_TABLE, 4757 },
	{ 0x7431, 0x7432, PDF_CMAP_RANGE, 15725 },
	{ 0x7433, 0x7436, PDF_CMAP_TABLE, 4769 },
	{ 0x7437, 0x743b, PDF_CMAP_RANGE, 15727 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 3178 },
	{ 0x743d, 0x743e, PDF_CMAP_RANGE, 15732 },
	{ 0x743f, 0x7441, PDF_CMAP_TABLE, 4773 },
	{ 0x7442, 0x744a, PDF_CMAP_RANGE, 15735 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 9308 },
	{ 0x744c, 0x7454, PDF_CMAP_RANGE, 15744 },
	{ 0x7455, 0x745f, PDF_CMAP_TABLE, 4776 },
	{ 0x7460, 0x7462, PDF_CMAP_RANGE, 15756 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 8541 },
	{ 0x7464, 0x7468, PDF_CMAP_RANGE, 15759 },
	{ 0x7469, 0x746a, PDF_CMAP_TABLE, 4787 },
	{ 0x746b, 0x746c, PDF_CMAP_RANGE, 15764 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 6207 },
	{ 0x746e, 0x746f, PDF_CMAP_RANGE, 15766 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 1854 },
	{ 0x7471, 0x7475, PDF_CMAP_RANGE, 15768 },
	{ 0x7476, 0x7477, PDF_CMAP_TABLE, 4789 },
	{ 0x7478, 0x747d, PDF_CMAP_RANGE, 15773 },
	{ 0x747e, 0x747f, PDF_CMAP_TABLE, 4791 },
	{ 0x7480, 0x7481, PDF_CMAP_RANGE, 6211 },
	{ 0x7482, 0x7483, PDF_CMAP_TABLE, 4793 },
	{ 0x7484, 0x7486, PDF_CMAP_RANGE, 15781 },
	{ 0x7487, 0x748b, PDF_CMAP_TABLE, 4795 },
	{ 0x748c, 0x748d, PDF_CMAP_RANGE, 15786 },
	{ 0x748e, 0x7490, PDF_CMAP_TABLE, 4800 },
	{ 0x7491, 0x749b, PDF_CMAP_RANGE, 15789 },
	{ 0x749c, 0x749e, PDF_CMAP_TABLE, 4803 },
	{ 0x749f, 0x74a2, PDF_CMAP_RANGE, 15801 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 9307 },
	{ 0x74a4, 0x74a5, PDF_CMAP_RANGE, 15805 },
	{ 0x74a6, 0x74a7, PDF_CMAP_TABLE, 4806 },
	{ 0x74a8, 0x74a9, PDF_CMAP_RANGE, 6216 },
	{ 0x74aa, 0x74af, PDF_CMAP_RANGE, 15807 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 8020 },
	{ 0x74b1, 0x74b9, PDF_CMAP_RANGE, 15813 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 6221 },
	{ 0x74bb, 0x74bc, PDF_CMAP_RANGE, 15822 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 9311 },
	{ 0x74be, 0x74c9, PDF_CMAP_RANGE, 15824 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 8428 },
	{ 0x74cb, 0x74ce, PDF_CMAP_RANGE, 15836 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 9309 },
	{ 0x74d0, 0x74d1, PDF_CMAP_RANGE, 15840 },
	{ 0x74d2, 0x74d4, PDF_CMAP_TABLE, 4808 },
	{ 0x74d5, 0x74d9, PDF_CMAP_RANGE, 15843 },
	{ 0x74da, 0x74e6, PDF_CMAP_TABLE, 4811 },
	{ 0x74e7, 0x74ed, PDF_CMAP_RANGE, 15853 },
	{ 0x74ee, 0x74ef, PDF_CMAP_TABLE, 4824 },
	{ 0x74f0, 0x74f3, PDF_CMAP_RANGE, 15860 },
	{ 0x74f4, 0x74f7, PDF_CMAP_TABLE, 4826 },
	{ 0x74f8, 0x74fe, PDF_CMAP_RANGE, 15865 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 6405 },
	{ 0x7500, 0x7503, PDF_CMAP_RANGE, 15872 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 4488 },
	{ 0x7505, 0x750b, PDF_CMAP_RANGE, 15876 },
	{ 0x750c, 0x7513, PDF_CMAP_TABLE, 4830 },
	{ 0x7514, 0x7517, PDF_CMAP_RANGE, 15886 },
	{ 0x7518, 0x751c, PDF_CMAP_TABLE, 4838 },
	{ 0x751d, 0x751e, PDF_CMAP_RANGE, 15891 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 3379 },
	{ 0x7520, 0x7522, PDF_CMAP_RANGE, 15893 },
	{ 0x7523, 0x7525, PDF_CMAP_TABLE, 4843 },
	{ 0x7526, 0x7527, PDF_CMAP_RANGE, 15897 },
	{ 0x7528, 0x753b, PDF_CMAP_TABLE, 4846 },
	{ 0x753c, 0x753d, PDF_CMAP_RANGE, 15904 },
	{ 0x753e, 0x7540, PDF_CMAP_TABLE, 4866 },
	{ 0x7541, 0x7544, PDF_CMAP_RANGE, 15907 },
	{ 0x7545, 0x7545, PDF_CMAP_SINGLE, 1230 },
	{ 0x7546, 0x7547, PDF_CMAP_RANGE, 15911 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 6780 },
	{ 0x7549, 0x754a, PDF_CMAP_RANGE, 15913 },
	{ 0x754b, 0x754f, PDF_CMAP_TABLE, 4869 },
	{ 0x7550, 0x7553, PDF_CMAP_RANGE, 15916 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 2960 },
	{ 0x7555, 0x7558, PDF_CMAP_RANGE, 15920 },
	{ 0x7559, 0x755d, PDF_CMAP_TABLE, 4874 },
	{ 0x755e, 0x7561, PDF_CMAP_RANGE, 15924 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 7746 },
	{ 0x7563, 0x7564, PDF_CMAP_RANGE, 15928 },
	{ 0x7565, 0x7566, PDF_CMAP_TABLE, 4879 },
	{ 0x7567, 0x7569, PDF_CMAP_RANGE, 15930 },
	{ 0x756a, 0x756b, PDF_CMAP_TABLE, 4881 },
	{ 0x756c, 0x7571, PDF_CMAP_RANGE, 15933 },
	{ 0x7572, 0x7579, PDF_CMAP_TABLE, 4883 },
	{ 0x757a, 0x757e, PDF_CMAP_RANGE, 15942 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 6164 },
	{ 0x7580, 0x7582, PDF_CMAP_RANGE, 15947 },
	{ 0x7583, 0x7583, PDF_CMAP_SINGLE, 6784 },
	{ 0x7584, 0x7585, PDF_CMAP_RANGE, 15950 },
	{ 0x7586, 0x7587, PDF_CMAP_TABLE, 4891 },
	{ 0x7588, 0x758a, PDF_CMAP_RANGE, 15952 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 7110 },
	{ 0x758c, 0x758e, PDF_CMAP_RANGE, 15955 },
	{ 0x758f, 0x759a, PDF_CMAP_TABLE, 4893 },
	{ 0x759b, 0x759c, PDF_CMAP_RANGE, 15962 },
	{ 0x759d, 0x75a5, PDF_CMAP_TABLE, 4905 },
	{ 0x75a6, 0x75aa, PDF_CMAP_RANGE, 15966 },
	{ 0x75ab, 0x75b2, PDF_CMAP_TABLE, 4914 },
	{ 0x75b3, 0x75b4, PDF_CMAP_RANGE, 7015 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 1350 },
	{ 0x75b6, 0x75b7, PDF_CMAP_RANGE, 15972 },
	{ 0x75b8, 0x75b9, PDF_CMAP_TABLE, 4922 },
	{ 0x75ba, 0x75bb, PDF_CMAP_RANGE, 15974 },
	{ 0x75bc, 0x75be, PDF_CMAP_TABLE, 4924 },
	{ 0x75bf, 0x75c1, PDF_CMAP_RANGE, 15976 },
	{ 0x75c2, 0x75ca, PDF_CMAP_TABLE, 4927 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 15980 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 7024 },
	{ 0x75ce, 0x75d1, PDF_CMAP_RANGE, 15982 },
	{ 0x75d2, 0x75db, PDF_CMAP_TABLE, 4936 },
	{ 0x75dc, 0x75dd, PDF_CMAP_RANGE, 15989 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 3013 },
	{ 0x75df, 0x75e1, PDF_CMAP_RANGE, 15991 },
	{ 0x75e2, 0x75eb, PDF_CMAP_TABLE, 4946 },
	{ 0x75ec, 0x75ef, PDF_CMAP_RANGE, 15996 },
	{ 0x75f0, 0x75f1, PDF_CMAP_TABLE, 4956 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 16000 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 1274 },
	{ 0x75f5, 0x75f8, PDF_CMAP_RANGE, 16002 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 1085 },
	{ 0x75fa, 0x75fb, PDF_CMAP_RANGE, 16006 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 7033 },
	{ 0x75fd, 0x75fe, PDF_CMAP_RANGE, 16008 },
	{ 0x75ff, 0x7605, PDF_CMAP_TABLE, 4958 },
	{ 0x7606, 0x7609, PDF_CMAP_RANGE, 16011 },
	{ 0x760a, 0x760d, PDF_CMAP_TABLE, 4965 },
	{ 0x760e, 0x760f, PDF_CMAP_RANGE, 16015 },
	{ 0x7610, 0x7610, PDF_CMAP_SINGLE, 7035 },
	{ 0x7611, 0x7614, PDF_CMAP_RANGE, 16017 },
	{ 0x7615, 0x761b, PDF_CMAP_TABLE, 4969 },
	{ 0x761c, 0x761d, PDF_CMAP_RANGE, 16023 },
	{ 0x761e, 0x762d, PDF_CMAP_TABLE, 4976 },
	{ 0x762e, 0x762f, PDF_CMAP_RANGE, 16028 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 7051 },
	{ 0x7631, 0x7632, PDF_CMAP_RANGE, 16030 },
	{ 0x7633, 0x7635, PDF_CMAP_TABLE, 4992 },
	{ 0x7636, 0x7637, PDF_CMAP_RANGE, 16032 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 3214 },
	{ 0x7639, 0x763a, PDF_CMAP_RANGE, 16034 },
	{ 0x763b, 0x7643, PDF_CMAP_TABLE, 4995 },
	{ 0x7644, 0x7645, PDF_CMAP_RANGE, 16038 },
	{ 0x7646, 0x7647, PDF_CMAP_RANGE, 9647 },
	{ 0x7648, 0x7649, PDF_CMAP_TABLE, 5004 },
	{ 0x764a, 0x764b, PDF_CMAP_RANGE, 16041 },
	{ 0x764c, 0x764d, PDF_CMAP_TABLE, 5006 },
	{ 0x764e, 0x7653, PDF_CMAP_RANGE, 16043 },
	{ 0x7654, 0x7658, PDF_CMAP_TABLE, 5008 },
	{ 0x7659, 0x765b, PDF_CMAP_RANGE, 16051 },
	{ 0x765c, 0x765f, PDF_CMAP_TABLE, 5013 },
	{ 0x7660, 0x7661, PDF_CMAP_RANGE, 16055 },
	{ 0x7662, 0x766c, PDF_CMAP_TABLE, 5017 },
	{ 0x766d, 0x766e, PDF_CMAP_RANGE, 9652 },
	{ 0x766f, 0x7672, PDF_CMAP_TABLE, 5028 },
	{ 0x7673, 0x7677, PDF_CMAP_RANGE, 16060 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 1864 },
	{ 0x7679, 0x767a, PDF_CMAP_RANGE, 16065 },
	{ 0x767b, 0x767e, PDF_CMAP_TABLE, 5032 },
	{ 0x767f, 0x7681, PDF_CMAP_RANGE, 16067 },
	{ 0x7682, 0x7688, PDF_CMAP_TABLE, 5036 },
	{ 0x7689, 0x768a, PDF_CMAP_RANGE, 16072 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 1753 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 16074 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 6958 },
	{ 0x768f, 0x7690, PDF_CMAP_RANGE, 16076 },
	{ 0x7691, 0x7693, PDF_CMAP_TABLE, 5043 },
	{ 0x7694, 0x7695, PDF_CMAP_RANGE, 16079 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 3743 },
	{ 0x7697, 0x7698, PDF_CMAP_RANGE, 16081 },
	{ 0x7699, 0x769a, PDF_CMAP_TABLE, 5046 },
	{ 0x769b, 0x76a3, PDF_CMAP_RANGE, 16083 },
	{ 0x76a4, 0x76a4, PDF_CMAP_SINGLE, 6961 },
	{ 0x76a5, 0x76ad, PDF_CMAP_RANGE, 16092 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3011 },
	{ 0x76af, 0x76b0, PDF_CMAP_RANGE, 16101 },
	{ 0x76b1, 0x76b4, PDF_CMAP_TABLE, 5048 },
	{ 0x76b5, 0x76b7, PDF_CMAP_RANGE, 16104 },
	{ 0x76b8, 0x76ba, PDF_CMAP_TABLE, 5052 },
	{ 0x76bb, 0x76be, PDF_CMAP_RANGE, 16108 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2808 },
	{ 0x76c0, 0x76c1, PDF_CMAP_RANGE, 16112 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 4288 },
	{ 0x76c3, 0x76c4, PDF_CMAP_RANGE, 16114 },
	{ 0x76c5, 0x76ca, PDF_CMAP_TABLE, 5055 },
	{ 0x76cb, 0x76cc, PDF_CMAP_RANGE, 16118 },
	{ 0x76cd, 0x76d8, PDF_CMAP_TABLE, 5061 },
	{ 0x76d9, 0x76da, PDF_CMAP_RANGE, 16122 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 3385 },
	{ 0x76dc, 0x76dd, PDF_CMAP_RANGE, 16124 },
	{ 0x76de, 0x76e7, PDF_CMAP_TABLE, 5073 },
	{ 0x76e8, 0x76ed, PDF_CMAP_RANGE, 16129 },
	{ 0x76ee, 0x76f4, PDF_CMAP_TABLE, 5083 },
	{ 0x76f5, 0x76f7, PDF_CMAP_RANGE, 16137 },
	{ 0x76f8, 0x76f9, PDF_CMAP_TABLE, 5090 },
	{ 0x76fa, 0x76fb, PDF_CMAP_RANGE, 16140 },
	{ 0x76fc, 0x7701, PDF_CMAP_TABLE, 5092 },
	{ 0x7702, 0x7703, PDF_CMAP_RANGE, 16145 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 6745 },
	{ 0x7705, 0x7706, PDF_CMAP_RANGE, 16147 },
	{ 0x7707, 0x7708, PDF_CMAP_RANGE, 6748 },
	{ 0x7709, 0x770d, PDF_CMAP_TABLE, 5098 },
	{ 0x770e, 0x7718, PDF_CMAP_RANGE, 16151 },
	{ 0x7719, 0x771a, PDF_CMAP_TABLE, 5103 },
	{ 0x771b, 0x771e, PDF_CMAP_RANGE, 16162 },
	{ 0x771f, 0x7722, PDF_CMAP_TABLE, 5105 },
	{ 0x7723, 0x7725, PDF_CMAP_RANGE, 16167 },
	{ 0x7726, 0x7729, PDF_CMAP_TABLE, 5109 },
	{ 0x772a, 0x772c, PDF_CMAP_RANGE, 16171 },
	{ 0x772d, 0x772f, PDF_CMAP_TABLE, 5113 },
	{ 0x7730, 0x7734, PDF_CMAP_RANGE, 16175 },
	{ 0x7735, 0x773c, PDF_CMAP_TABLE, 5116 },
	{ 0x773d, 0x773f, PDF_CMAP_RANGE, 16182 },
	{ 0x7740, 0x7743, PDF_CMAP_TABLE, 5124 },
	{ 0x7744, 0x7746, PDF_CMAP_RANGE, 16186 },
	{ 0x7747, 0x7747, PDF_CMAP_SINGLE, 6759 },
	{ 0x7748, 0x774e, PDF_CMAP_RANGE, 16189 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 9860 },
	{ 0x7750, 0x7751, PDF_CMAP_RANGE, 6757 },
	{ 0x7752, 0x7759, PDF_CMAP_RANGE, 16196 },
	{ 0x775a, 0x775b, PDF_CMAP_TABLE, 5128 },
	{ 0x775c, 0x775d, PDF_CMAP_RANGE, 16204 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 9455 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 16206 },
	{ 0x7761, 0x7768, PDF_CMAP_TABLE, 5130 },
	{ 0x7769, 0x776a, PDF_CMAP_RANGE, 16210 },
	{ 0x776b, 0x776c, PDF_CMAP_TABLE, 5138 },
	{ 0x776d, 0x7778, PDF_CMAP_RANGE, 16212 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 1538 },
	{ 0x777a, 0x777c, PDF_CMAP_RANGE, 16224 },
	{ 0x777d, 0x7780, PDF_CMAP_TABLE, 5140 },
	{ 0x7781, 0x7783, PDF_CMAP_RANGE, 16227 },
	{ 0x7784, 0x7785, PDF_CMAP_TABLE, 5144 },
	{ 0x7786, 0x778b, PDF_CMAP_RANGE, 16230 },
	{ 0x778c, 0x778e, PDF_CMAP_TABLE, 5146 },
	{ 0x778f, 0x7790, PDF_CMAP_RANGE, 16236 },
	{ 0x7791, 0x7792, PDF_CMAP_TABLE, 5149 },
	{ 0x7793, 0x7797, PDF_CMAP_RANGE, 16238 },
	{ 0x7798, 0x7798, PDF_CMAP_SINGLE, 9454 },
	{ 0x7799, 0x779d, PDF_CMAP_RANGE, 16243 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 8308 },
	{ 0x779f, 0x77a0, PDF_CMAP_RANGE, 6771 },
	{ 0x77a1, 0x77a2, PDF_CMAP_TABLE, 5151 },
	{ 0x77a3, 0x77a4, PDF_CMAP_RANGE, 16249 },
	{ 0x77a5, 0x77ad, PDF_CMAP_TABLE, 5153 },
	{ 0x77ae, 0x77af, PDF_CMAP_RANGE, 16254 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 6773 },
	{ 0x77b1, 0x77b2, PDF_CMAP_RANGE, 16256 },
	{ 0x77b3, 0x77b5, PDF_CMAP_TABLE, 5162 },
	{ 0x77b6, 0x77ba, PDF_CMAP_RANGE, 16259 },
	{ 0x77bb, 0x77bf, PDF_CMAP_TABLE, 5165 },
	{ 0x77c0, 0x77c6, PDF_CMAP_RANGE, 16265 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 9869 },
	{ 0x77c8, 0x77cc, PDF_CMAP_RANGE, 16272 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5015 },
	{ 0x77ce, 0x77d6, PDF_CMAP_RANGE, 16277 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 1318 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 16286 },
	{ 0x77da, 0x77dc, PDF_CMAP_TABLE, 5170 },
	{ 0x77dd, 0x77e1, PDF_CMAP_RANGE, 16288 },
	{ 0x77e2, 0x77ef, PDF_CMAP_TABLE, 5173 },
	{ 0x77f0, 0x77f2, PDF_CMAP_RANGE, 16297 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 3398 },
	{ 0x77f4, 0x77f5, PDF_CMAP_RANGE, 16300 },
	{ 0x77f6, 0x77f8, PDF_CMAP_TABLE, 5187 },
	{ 0x77f9, 0x77fc, PDF_CMAP_RANGE, 16303 },
	{ 0x77fd, 0x7802, PDF_CMAP_TABLE, 5190 },
	{ 0x7803, 0x7808, PDF_CMAP_RANGE, 16307 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 6693 },
	{ 0x780a, 0x780b, PDF_CMAP_RANGE, 16313 },
	{ 0x780c, 0x780d, PDF_CMAP_TABLE, 5196 },
	{ 0x780e, 0x7810, PDF_CMAP_RANGE, 16315 },
	{ 0x7811, 0x7816, PDF_CMAP_TABLE, 5198 },
	{ 0x7817, 0x7818, PDF_CMAP_RANGE, 6694 },
	{ 0x7819, 0x781b, PDF_CMAP_TABLE, 5204 },
	{ 0x781c, 0x781d, PDF_CMAP_RANGE, 6699 },
	{ 0x781e, 0x781f, PDF_CMAP_TABLE, 5207 },
	{ 0x7820, 0x7822, PDF_CMAP_RANGE, 16323 },
	{ 0x7823, 0x7829, PDF_CMAP_TABLE, 5209 },
	{ 0x782a, 0x782b, PDF_CMAP_RANGE, 16328 },
	{ 0x782c, 0x782d, PDF_CMAP_TABLE, 5216 },
	{ 0x782e, 0x782f, PDF_CMAP_RANGE, 16330 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 2986 },
	{ 0x7831, 0x7833, PDF_CMAP_RANGE, 16332 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 3045 },
	{ 0x7835, 0x7836, PDF_CMAP_RANGE, 16335 },
	{ 0x7837, 0x7838, PDF_CMAP_TABLE, 5218 },
	{ 0x7839, 0x783b, PDF_CMAP_RANGE, 6701 },
	{ 0x783c, 0x7840, PDF_CMAP_TABLE, 5220 },
	{ 0x7841, 0x7842, PDF_CMAP_RANGE, 16339 },
	{ 0x7843, 0x7847, PDF_CMAP_TABLE, 5225 },
	{ 0x7848, 0x784b, PDF_CMAP_RANGE, 16343 },
	{ 0x784c, 0x7852, PDF_CMAP_TABLE, 5230 },
	{ 0x7853, 0x7854, PDF_CMAP_RANGE, 16350 },
	{ 0x7855, 0x7855, PDF_CMAP_SINGLE, 3499 },
	{ 0x7856, 0x7857, PDF_CMAP_RANGE, 6712 },
	{ 0x7858, 0x785c, PDF_CMAP_RANGE, 16352 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3938 },
	{ 0x785e, 0x7863, PDF_CMAP_RANGE, 16357 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 9449 },
	{ 0x7865, 0x7867, PDF_CMAP_RANGE, 16363 },
	{ 0x7868, 0x786f, PDF_CMAP_TABLE, 5237 },
	{ 0x7870, 0x7876, PDF_CMAP_RANGE, 16367 },
	{ 0x7877, 0x7877, PDF_CMAP_SINGLE, 2146 },
	{ 0x7878, 0x787b, PDF_CMAP_RANGE, 16374 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 2993 },
	{ 0x787d, 0x7886, PDF_CMAP_RANGE, 16378 },
	{ 0x7887, 0x7889, PDF_CMAP_TABLE, 5245 },
	{ 0x788a, 0x788b, PDF_CMAP_RANGE, 16389 },
	{ 0x788c, 0x788e, PDF_CMAP_TABLE, 5248 },
	{ 0x788f, 0x7890, PDF_CMAP_RANGE, 16391 },
	{ 0x7891, 0x7893, PDF_CMAP_TABLE, 5251 },
	{ 0x7894, 0x7896, PDF_CMAP_RANGE, 16394 },
	{ 0x7897, 0x789c, PDF_CMAP_TABLE, 5254 },
	{ 0x789d, 0x789e, PDF_CMAP_RANGE, 16398 },
	{ 0x789f, 0x78a9, PDF_CMAP_TABLE, 5260 },
	{ 0x78aa, 0x78ac, PDF_CMAP_RANGE, 16405 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 9444 },
	{ 0x78ae, 0x78af, PDF_CMAP_RANGE, 16408 },
	{ 0x78b0, 0x78b4, PDF_CMAP_TABLE, 5271 },
	{ 0x78b5, 0x78b7, PDF_CMAP_RANGE, 16410 },
	{ 0x78b8, 0x78be, PDF_CMAP_TABLE, 5276 },
	{ 0x78bf, 0x78c0, PDF_CMAP_RANGE, 16415 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 1352 },
	{ 0x78c2, 0x78c4, PDF_CMAP_RANGE, 16417 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 1024 },
	{ 0x78c6, 0x78c8, PDF_CMAP_RANGE, 16420 },
	{ 0x78c9, 0x78cb, PDF_CMAP_TABLE, 5283 },
	{ 0x78cc, 0x78cf, PDF_CMAP_RANGE, 16423 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 2958 },
	{ 0x78d1, 0x78d3, PDF_CMAP_RANGE, 16427 },
	{ 0x78d4, 0x78d5, PDF_CMAP_TABLE, 5286 },
	{ 0x78d6, 0x78d8, PDF_CMAP_RANGE, 16430 },
	{ 0x78d9, 0x78da, PDF_CMAP_TABLE, 5288 },
	{ 0x78db, 0x78e2, PDF_CMAP_RANGE, 16433 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 9452 },
	{ 0x78e4, 0x78e6, PDF_CMAP_RANGE, 16441 },
	{ 0x78e7, 0x78e8, PDF_CMAP_TABLE, 5290 },
	{ 0x78e9, 0x78eb, PDF_CMAP_RANGE, 16444 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 6732 },
	{ 0x78ed, 0x78ee, PDF_CMAP_RANGE, 16447 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 9443 },
	{ 0x78f0, 0x78f1, PDF_CMAP_RANGE, 16449 },
	{ 0x78f2, 0x78f4, PDF_CMAP_TABLE, 5292 },
	{ 0x78f5, 0x78f6, PDF_CMAP_RANGE, 16452 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 2592 },
	{ 0x78f8, 0x78f9, PDF_CMAP_RANGE, 16454 },
	{ 0x78fa, 0x78fa, PDF_CMAP_SINGLE, 2012 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 16456 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 9450 },
	{ 0x78fe, 0x7900, PDF_CMAP_RANGE, 16458 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2185 },
	{ 0x7902, 0x7904, PDF_CMAP_RANGE, 16461 },
	{ 0x7905, 0x7905, PDF_CMAP_SINGLE, 6734 },
	{ 0x7906, 0x790d, PDF_CMAP_RANGE, 16464 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 7826 },
	{ 0x790f, 0x7912, PDF_CMAP_RANGE, 16472 },
	{ 0x7913, 0x7913, PDF_CMAP_SINGLE, 6736 },
	{ 0x7914, 0x7918, PDF_CMAP_RANGE, 16476 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 7719 },
	{ 0x791a, 0x791d, PDF_CMAP_RANGE, 16481 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 6738 },
	{ 0x791f, 0x7923, PDF_CMAP_RANGE, 16485 },
	{ 0x7924, 0x7926, PDF_CMAP_TABLE, 5295 },
	{ 0x7927, 0x7929, PDF_CMAP_RANGE, 16491 },
	{ 0x792a, 0x792c, PDF_CMAP_TABLE, 5298 },
	{ 0x792d, 0x7930, PDF_CMAP_RANGE, 16494 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 9448 },
	{ 0x7932, 0x7933, PDF_CMAP_RANGE, 16498 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 6739 },
	{ 0x7935, 0x7939, PDF_CMAP_RANGE, 16500 },
	{ 0x793a, 0x7941, PDF_CMAP_TABLE, 5301 },
	{ 0x7942, 0x7945, PDF_CMAP_RANGE, 16507 },
	{ 0x7946, 0x7949, PDF_CMAP_TABLE, 5309 },
	{ 0x794a, 0x7952, PDF_CMAP_RANGE, 16512 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 6654 },
	{ 0x7954, 0x7955, PDF_CMAP_RANGE, 16521 },
	{ 0x7956, 0x7957, PDF_CMAP_TABLE, 5313 },
	{ 0x7958, 0x7959, PDF_CMAP_RANGE, 16523 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 6655 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 6652 },
	{ 0x795d, 0x7962, PDF_CMAP_TABLE, 5315 },
	{ 0x7963, 0x7964, PDF_CMAP_RANGE, 16526 },
	{ 0x7965, 0x7968, PDF_CMAP_TABLE, 5321 },
	{ 0x7969, 0x796c, PDF_CMAP_RANGE, 16529 },
	{ 0x796d, 0x796f, PDF_CMAP_TABLE, 5325 },
	{ 0x7970, 0x7976, PDF_CMAP_RANGE, 16534 },
	{ 0x7977, 0x797a, PDF_CMAP_TABLE, 5328 },
	{ 0x797b, 0x797f, PDF_CMAP_RANGE, 16542 },
	{ 0x7980, 0x7981, PDF_CMAP_TABLE, 5332 },
	{ 0x7982, 0x7983, PDF_CMAP_RANGE, 16547 },
	{ 0x7984, 0x7985, PDF_CMAP_TABLE, 5334 },
	{ 0x7986, 0x7989, PDF_CMAP_RANGE, 16549 },
	{ 0x798a, 0x798a, PDF_CMAP_SINGLE, 6663 },
	{ 0x798b, 0x798c, PDF_CMAP_RANGE, 16553 },
	{ 0x798d, 0x798f, PDF_CMAP_TABLE, 5336 },
	{ 0x7990, 0x7999, PDF_CMAP_RANGE, 16555 },
	{ 0x799a, 0x799a, PDF_CMAP_SINGLE, 6664 },
	{ 0x799b, 0x79a5, PDF_CMAP_RANGE, 16565 },
	{ 0x79a6, 0x79a7, PDF_CMAP_TABLE, 5339 },
	{ 0x79a8, 0x79a9, PDF_CMAP_RANGE, 16576 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 9437 },
	{ 0x79ab, 0x79ad, PDF_CMAP_RANGE, 16578 },
	{ 0x79ae, 0x79b3, PDF_CMAP_TABLE, 5341 },
	{ 0x79b4, 0x79b8, PDF_CMAP_RANGE, 16583 },
	{ 0x79b9, 0x79c3, PDF_CMAP_TABLE, 5347 },
	{ 0x79c4, 0x79c5, PDF_CMAP_RANGE, 16591 },
	{ 0x79c6, 0x79c6, PDF_CMAP_SINGLE, 1740 },
	{ 0x79c7, 0x79c8, PDF_CMAP_RANGE, 16593 },
	{ 0x79c9, 0x79cd, PDF_CMAP_TABLE, 5358 },
	{ 0x79ce, 0x79d0, PDF_CMAP_RANGE, 16597 },
	{ 0x79d1, 0x79d2, PDF_CMAP_TABLE, 5363 },
	{ 0x79d3, 0x79d4, PDF_CMAP_RANGE, 16600 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 6941 },
	{ 0x79d6, 0x79d7, PDF_CMAP_RANGE, 16602 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2781 },
	{ 0x79d9, 0x79de, PDF_CMAP_RANGE, 16604 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 4671 },
	{ 0x79e0, 0x79e2, PDF_CMAP_RANGE, 16610 },
	{ 0x79e3, 0x79f0, PDF_CMAP_TABLE, 5365 },
	{ 0x79f1, 0x79f7, PDF_CMAP_RANGE, 16618 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 2214 },
	{ 0x79f9, 0x79fa, PDF_CMAP_RANGE, 16625 },
	{ 0x79fb, 0x79fd, PDF_CMAP_TABLE, 5379 },
	{ 0x79fe, 0x79ff, PDF_CMAP_RANGE, 16628 },
	{ 0x7a00, 0x7a03, PDF_CMAP_TABLE, 5382 },
	{ 0x7a04, 0x7a05, PDF_CMAP_RANGE, 16631 },
	{ 0x7a06, 0x7a06, PDF_CMAP_SINGLE, 6945 },
	{ 0x7a07, 0x7a0a, PDF_CMAP_RANGE, 16633 },
	{ 0x7a0b, 0x7a0e, PDF_CMAP_TABLE, 5386 },
	{ 0x7a0f, 0x7a13, PDF_CMAP_RANGE, 16638 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 6950 },
	{ 0x7a15, 0x7a16, PDF_CMAP_RANGE, 16643 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 1001 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 16645 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 4552 },
	{ 0x7a1b, 0x7a1d, PDF_CMAP_RANGE, 16647 },
	{ 0x7a1e, 0x7a20, PDF_CMAP_TABLE, 5390 },
	{ 0x7a21, 0x7a22, PDF_CMAP_RANGE, 16651 },
	{ 0x7a23, 0x7a23, PDF_CMAP_SINGLE, 7567 },
	{ 0x7a24, 0x7a2d, PDF_CMAP_RANGE, 16653 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 8853 },
	{ 0x7a2f, 0x7a30, PDF_CMAP_RANGE, 16663 },
	{ 0x7a31, 0x7a33, PDF_CMAP_TABLE, 5393 },
	{ 0x7a34, 0x7a36, PDF_CMAP_RANGE, 16666 },
	{ 0x7a37, 0x7a40, PDF_CMAP_TABLE, 5396 },
	{ 0x7a41, 0x7a45, PDF_CMAP_RANGE, 16672 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 2853 },
	{ 0x7a47, 0x7a4b, PDF_CMAP_RANGE, 16677 },
	{ 0x7a4c, 0x7a4e, PDF_CMAP_TABLE, 5406 },
	{ 0x7a4f, 0x7a50, PDF_CMAP_RANGE, 16682 },
	{ 0x7a51, 0x7a51, PDF_CMAP_SINGLE, 6953 },
	{ 0x7a52, 0x7a56, PDF_CMAP_RANGE, 16684 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 3552 },
	{ 0x7a58, 0x7a60, PDF_CMAP_RANGE, 16689 },
	{ 0x7a61, 0x7a62, PDF_CMAP_TABLE, 5409 },
	{ 0x7a63, 0x7a68, PDF_CMAP_RANGE, 16698 },
	{ 0x7a69, 0x7a6b, PDF_CMAP_TABLE, 5411 },
	{ 0x7a6c, 0x7a6f, PDF_CMAP_RANGE, 16705 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 6956 },
	{ 0x7a71, 0x7a73, PDF_CMAP_RANGE, 16709 },
	{ 0x7a74, 0x7a77, PDF_CMAP_TABLE, 5414 },
	{ 0x7a78, 0x7a79, PDF_CMAP_RANGE, 7066 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 2402 },
	{ 0x7a7b, 0x7a7e, PDF_CMAP_RANGE, 16713 },
	{ 0x7a7f, 0x7a88, PDF_CMAP_TABLE, 5418 },
	{ 0x7a89, 0x7a8c, PDF_CMAP_RANGE, 16720 },
	{ 0x7a8d, 0x7a8d, PDF_CMAP_SINGLE, 3148 },
	{ 0x7a8e, 0x7a90, PDF_CMAP_RANGE, 16724 },
	{ 0x7a91, 0x7a92, PDF_CMAP_TABLE, 5428 },
	{ 0x7a93, 0x7a94, PDF_CMAP_RANGE, 16727 },
	{ 0x7a95, 0x7a98, PDF_CMAP_TABLE, 5430 },
	{ 0x7a99, 0x7a9b, PDF_CMAP_RANGE, 16729 },
	{ 0x7a9c, 0x7aa0, PDF_CMAP_TABLE, 5434 },
	{ 0x7aa1, 0x7aa4, PDF_CMAP_RANGE, 16733 },
	{ 0x7aa5, 0x7aae, PDF_CMAP_TABLE, 5439 },
	{ 0x7aaf, 0x7ab2, PDF_CMAP_RANGE, 16739 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 7077 },
	{ 0x7ab4, 0x7ab5, PDF_CMAP_RANGE, 16743 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 9657 },
	{ 0x7ab7, 0x7ab9, PDF_CMAP_RANGE, 16745 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 8170 },
	{ 0x7abb, 0x7abe, PDF_CMAP_RANGE, 16748 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 2631 },
	{ 0x7ac0, 0x7ac3, PDF_CMAP_RANGE, 16752 },
	{ 0x7ac4, 0x7acb, PDF_CMAP_TABLE, 5449 },
	{ 0x7acc, 0x7ad5, PDF_CMAP_RANGE, 16758 },
	{ 0x7ad6, 0x7ad6, PDF_CMAP_SINGLE, 3473 },
	{ 0x7ad7, 0x7ad8, PDF_CMAP_RANGE, 16768 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 4447 },
	{ 0x7ada, 0x7add, PDF_CMAP_RANGE, 16770 },
	{ 0x7ade, 0x7ae0, PDF_CMAP_TABLE, 5457 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 16774 },
	{ 0x7ae3, 0x7ae6, PDF_CMAP_TABLE, 5460 },
	{ 0x7ae7, 0x7ae9, PDF_CMAP_RANGE, 16777 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 8513 },
	{ 0x7aeb, 0x7aec, PDF_CMAP_RANGE, 16780 },
	{ 0x7aed, 0x7aef, PDF_CMAP_TABLE, 5464 },
	{ 0x7af0, 0x7af5, PDF_CMAP_RANGE, 16783 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 8135 },
	{ 0x7af7, 0x7af8, PDF_CMAP_RANGE, 16789 },
	{ 0x7af9, 0x7afa, PDF_CMAP_TABLE, 5467 },
	{ 0x7afb, 0x7afc, PDF_CMAP_RANGE, 16791 },
	{ 0x7afd, 0x7aff, PDF_CMAP_TABLE, 5469 },
	{ 0x7b00, 0x7b02, PDF_CMAP_RANGE, 16794 },
	{ 0x7b03, 0x7b04, PDF_CMAP_RANGE, 7270 },
	{ 0x7b05, 0x7b0b, PDF_CMAP_TABLE, 5472 },
	{ 0x7b0c, 0x7b0e, PDF_CMAP_RANGE, 16800 },
	{ 0x7b0f, 0x7b11, PDF_CMAP_TABLE, 5479 },
	{ 0x7b12, 0x7b13, PDF_CMAP_RANGE, 16804 },
	{ 0x7b14, 0x7b15, PDF_CMAP_TABLE, 5482 },
	{ 0x7b16, 0x7b18, PDF_CMAP_RANGE, 16806 },
	{ 0x7b19, 0x7b1b, PDF_CMAP_TABLE, 5484 },
	{ 0x7b1c, 0x7b1d, PDF_CMAP_RANGE, 16810 },
	{ 0x7b1e, 0x7b20, PDF_CMAP_TABLE, 5487 },
	{ 0x7b21, 0x7b23, PDF_CMAP_RANGE, 16813 },
	{ 0x7b24, 0x7b2e, PDF_CMAP_TABLE, 5490 },
	{ 0x7b2f, 0x7b30, PDF_CMAP_RANGE, 16819 },
	{ 0x7b31, 0x7b33, PDF_CMAP_TABLE, 5501 },
	{ 0x7b34, 0x7b37, PDF_CMAP_RANGE, 16822 },
	{ 0x7b38, 0x7b3e, PDF_CMAP_TABLE, 5504 },
	{ 0x7b3f, 0x7b44, PDF_CMAP_RANGE, 16829 },
	{ 0x7b45, 0x7b4c, PDF_CMAP_TABLE, 5511 },
	{ 0x7b4d, 0x7b4e, PDF_CMAP_RANGE, 16837 },
	{ 0x7b4f, 0x7b5d, PDF_CMAP_TABLE, 5519 },
	{ 0x7b5e, 0x7b5f, PDF_CMAP_RANGE, 16844 },
	{ 0x7b60, 0x7b62, PDF_CMAP_TABLE, 5534 },
	{ 0x7b63, 0x7b66, PDF_CMAP_RANGE, 16847 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 9700 },
	{ 0x7b68, 0x7b6d, PDF_CMAP_RANGE, 16851 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 7295 },
	{ 0x7b6f, 0x7b70, PDF_CMAP_RANGE, 16857 },
	{ 0x7b71, 0x7b72, PDF_CMAP_TABLE, 5537 },
	{ 0x7b73, 0x7b74, PDF_CMAP_RANGE, 16859 },
	{ 0x7b75, 0x7b7b, PDF_CMAP_TABLE, 5539 },
	{ 0x7b7c, 0x7b7d, PDF_CMAP_RANGE, 16864 },
	{ 0x7b7e, 0x7b80, PDF_CMAP_TABLE, 5546 },
	{ 0x7b81, 0x7b84, PDF_CMAP_RANGE, 16867 },
	{ 0x7b85, 0x7b85, PDF_CMAP_SINGLE, 7307 },
	{ 0x7b86, 0x7b8a, PDF_CMAP_RANGE, 16871 },
	{ 0x7b8b, 0x7b8d, PDF_CMAP_TABLE, 5549 },
	{ 0x7b8e, 0x7b8f, PDF_CMAP_RANGE, 16877 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 7300 },
	{ 0x7b91, 0x7b93, PDF_CMAP_RANGE, 16879 },
	{ 0x7b94, 0x7b97, PDF_CMAP_TABLE, 5552 },
	{ 0x7b98, 0x7b9b, PDF_CMAP_RANGE, 16883 },
	{ 0x7b9c, 0x7b9d, PDF_CMAP_TABLE, 5556 },
	{ 0x7b9e, 0x7ba0, PDF_CMAP_RANGE, 16887 },
	{ 0x7ba1, 0x7ba2, PDF_CMAP_TABLE, 5558 },
	{ 0x7ba3, 0x7ba5, PDF_CMAP_RANGE, 16890 },
	{ 0x7ba6, 0x7ba7, PDF_CMAP_RANGE, 7301 },
	{ 0x7ba8, 0x7bad, PDF_CMAP_TABLE, 5560 },
	{ 0x7bae, 0x7bb0, PDF_CMAP_RANGE, 16893 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3921 },
	{ 0x7bb2, 0x7bb3, PDF_CMAP_RANGE, 16896 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 7312 },
	{ 0x7bb5, 0x7bb7, PDF_CMAP_RANGE, 16898 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 7303 },
	{ 0x7bb9, 0x7bbf, PDF_CMAP_RANGE, 16901 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 5566 },
	{ 0x7bc2, 0x7bc3, PDF_CMAP_RANGE, 16908 },
	{ 0x7bc4, 0x7bcc, PDF_CMAP_TABLE, 5568 },
	{ 0x7bcd, 0x7bd0, PDF_CMAP_RANGE, 16913 },
	{ 0x7bd1, 0x7bd3, PDF_CMAP_TABLE, 5577 },
	{ 0x7bd4, 0x7bd8, PDF_CMAP_RANGE, 16918 },
	{ 0x7bd9, 0x7bda, PDF_CMAP_TABLE, 5580 },
	{ 0x7bdb, 0x7bdc, PDF_CMAP_RANGE, 16923 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 7316 },
	{ 0x7bde, 0x7be0, PDF_CMAP_RANGE, 16925 },
	{ 0x7be1, 0x7be1, PDF_CMAP_SINGLE, 1374 },
	{ 0x7be2, 0x7be3, PDF_CMAP_RANGE, 16928 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 9699 },
	{ 0x7be5, 0x7be6, PDF_CMAP_RANGE, 7318 },
	{ 0x7be7, 0x7be8, PDF_CMAP_RANGE, 16930 },
	{ 0x7be9, 0x7bea, PDF_CMAP_TABLE, 5582 },
	{ 0x7beb, 0x7bed, PDF_CMAP_RANGE, 16932 },
	{ 0x7bee, 0x7bee, PDF_CMAP_SINGLE, 2469 },
	{ 0x7bef, 0x7bf0, PDF_CMAP_RANGE, 16935 },
	{ 0x7bf1, 0x7bf3, PDF_CMAP_TABLE, 5584 },
	{ 0x7bf4, 0x7bf6, PDF_CMAP_RANGE, 16938 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 2994 },
	{ 0x7bf8, 0x7bfb, PDF_CMAP_RANGE, 16941 },
	{ 0x7bfc, 0x7c00, PDF_CMAP_TABLE, 5587 },
	{ 0x7c01, 0x7c06, PDF_CMAP_RANGE, 16947 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 1371 },
	{ 0x7c08, 0x7c0a, PDF_CMAP_RANGE, 16953 },
	{ 0x7c0b, 0x7c0f, PDF_CMAP_TABLE, 5592 },
	{ 0x7c10, 0x7c15, PDF_CMAP_RANGE, 16957 },
	{ 0x7c16, 0x7c16, PDF_CMAP_SINGLE, 7325 },
	{ 0x7c17, 0x7c1d, PDF_CMAP_RANGE, 16963 },
	{ 0x7c1e, 0x7c23, PDF_CMAP_TABLE, 5597 },
	{ 0x7c24, 0x7c25, PDF_CMAP_RANGE, 16972 },
	{ 0x7c26, 0x7c27, PDF_CMAP_TABLE, 5603 },
	{ 0x7c28, 0x7c29, PDF_CMAP_RANGE, 16974 },
	{ 0x7c2a, 0x7c2b, PDF_CMAP_TABLE, 5605 },
	{ 0x7c2c, 0x7c37, PDF_CMAP_RANGE, 16976 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 7330 },
	{ 0x7c39, 0x7c3c, PDF_CMAP_RANGE, 16988 },
	{ 0x7c3d, 0x7c43, PDF_CMAP_TABLE, 5607 },
	{ 0x7c44, 0x7c4b, PDF_CMAP_RANGE, 16993 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 5614 },
	{ 0x7c4e, 0x7c5b, PDF_CMAP_RANGE, 17001 },
	{ 0x7c5c, 0x7c5c, PDF_CMAP_SINGLE, 9705 },
	{ 0x7c5d, 0x7c5e, PDF_CMAP_RANGE, 17015 },
	{ 0x7c5f, 0x7c60, PDF_CMAP_TABLE, 5616 },
	{ 0x7c61, 0x7c63, PDF_CMAP_RANGE, 17017 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 9875 },
	{ 0x7c65, 0x7c68, PDF_CMAP_RANGE, 17020 },
	{ 0x7c69, 0x7c6e, PDF_CMAP_TABLE, 5618 },
	{ 0x7c6f, 0x7c71, PDF_CMAP_RANGE, 17026 },
	{ 0x7c72, 0x7c74, PDF_CMAP_TABLE, 5624 },
	{ 0x7c75, 0x7c7a, PDF_CMAP_RANGE, 17029 },
	{ 0x7c7b, 0x7c7d, PDF_CMAP_TABLE, 5627 },
	{ 0x7c7e, 0x7c88, PDF_CMAP_RANGE, 17035 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 1656 },
	{ 0x7c8a, 0x7c90, PDF_CMAP_RANGE, 17046 },
	{ 0x7c91, 0x7c92, PDF_CMAP_TABLE, 5630 },
	{ 0x7c93, 0x7c94, PDF_CMAP_RANGE, 17053 },
	{ 0x7c95, 0x7c98, PDF_CMAP_TABLE, 5632 },
	{ 0x7c99, 0x7c9b, PDF_CMAP_RANGE, 17056 },
	{ 0x7c9c, 0x7c9f, PDF_CMAP_TABLE, 5636 },
	{ 0x7ca0, 0x7ca1, PDF_CMAP_RANGE, 17059 },
	{ 0x7ca2, 0x7ca5, PDF_CMAP_TABLE, 5640 },
	{ 0x7ca6, 0x7ca9, PDF_CMAP_RANGE, 17062 },
	{ 0x7caa, 0x7caa, PDF_CMAP_SINGLE, 1661 },
	{ 0x7cab, 0x7cad, PDF_CMAP_RANGE, 17066 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 2561 },
	{ 0x7caf, 0x7cb0, PDF_CMAP_RANGE, 17069 },
	{ 0x7cb1, 0x7cb3, PDF_CMAP_TABLE, 5644 },
	{ 0x7cb4, 0x7cb8, PDF_CMAP_RANGE, 17071 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 1381 },
	{ 0x7cba, 0x7cbb, PDF_CMAP_RANGE, 17076 },
	{ 0x7cbc, 0x7cbd, PDF_CMAP_RANGE, 7378 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 2266 },
	{ 0x7cbf, 0x7cc0, PDF_CMAP_RANGE, 17078 },
	{ 0x7cc1, 0x7cc1, PDF_CMAP_SINGLE, 7380 },
	{ 0x7cc2, 0x7cc4, PDF_CMAP_RANGE, 17080 },
	{ 0x7cc5, 0x7ccb, PDF_CMAP_TABLE, 5647 },
	{ 0x7ccc, 0x7ccd, PDF_CMAP_RANGE, 7382 },
	{ 0x7cce, 0x7cd4, PDF_CMAP_RANGE, 17086 },
	{ 0x7cd5, 0x7cd9, PDF_CMAP_TABLE, 5654 },
	{ 0x7cda, 0x7cdb, PDF_CMAP_RANGE, 17094 },
	{ 0x7cdc, 0x7ce0, PDF_CMAP_TABLE, 5659 },
	{ 0x7ce1, 0x7ce6, PDF_CMAP_RANGE, 17096 },
	{ 0x7ce7, 0x7ce8, PDF_CMAP_TABLE, 5664 },
	{ 0x7ce9, 0x7cee, PDF_CMAP_RANGE, 17102 },
	{ 0x7cef, 0x7cfb, PDF_CMAP_TABLE, 5666 },
	{ 0x7cfc, 0x7cfd, PDF_CMAP_RANGE, 17113 },
	{ 0x7cfe, 0x7d06, PDF_CMAP_TABLE, 5679 },
	{ 0x7d07, 0x7d08, PDF_CMAP_RANGE, 9246 },
	{ 0x7d09, 0x7d0d, PDF_CMAP_TABLE, 5688 },
	{ 0x7d0e, 0x7d0f, PDF_CMAP_RANGE, 17119 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 8359 },
	{ 0x7d11, 0x7d12, PDF_CMAP_RANGE, 17121 },
	{ 0x7d13, 0x7d1c, PDF_CMAP_TABLE, 5693 },
	{ 0x7d1d, 0x7d1f, PDF_CMAP_RANGE, 17125 },
	{ 0x7d20, 0x7d22, PDF_CMAP_TABLE, 5703 },
	{ 0x7d23, 0x7d26, PDF_CMAP_RANGE, 17128 },
	{ 0x7d27, 0x7d27, PDF_CMAP_SINGLE, 2245 },
	{ 0x7d28, 0x7d2a, PDF_CMAP_RANGE, 17132 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 4652 },
	{ 0x7d2c, 0x7d2e, PDF_CMAP_RANGE, 17135 },
	{ 0x7d2f, 0x7d33, PDF_CMAP_TABLE, 5706 },
	{ 0x7d34, 0x7d38, PDF_CMAP_RANGE, 17138 },
	{ 0x7d39, 0x7d3c, PDF_CMAP_TABLE, 5711 },
	{ 0x7d3d, 0x7d3e, PDF_CMAP_RANGE, 17144 },
	{ 0x7d3f, 0x7d46, PDF_CMAP_TABLE, 5715 },
	{ 0x7d47, 0x7d4d, PDF_CMAP_RANGE, 17149 },
	{ 0x7d4e, 0x7d50, PDF_CMAP_TABLE, 5723 },
	{ 0x7d51, 0x7d5c, PDF_CMAP_RANGE, 17157 },
	{ 0x7d5d, 0x7d5e, PDF_CMAP_TABLE, 5726 },
	{ 0x7d5f, 0x7d60, PDF_CMAP_RANGE, 17169 },
	{ 0x7d61, 0x7d62, PDF_CMAP_TABLE, 5728 },
	{ 0x7d63, 0x7d65, PDF_CMAP_RANGE, 17171 },
	{ 0x7d66, 0x7d68, PDF_CMAP_TABLE, 5730 },
	{ 0x7d69, 0x7d6d, PDF_CMAP_RANGE, 17175 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 4032 },
	{ 0x7d6f, 0x7d70, PDF_CMAP_RANGE, 17180 },
	{ 0x7d71, 0x7d73, PDF_CMAP_TABLE, 5733 },
	{ 0x7d74, 0x7d75, PDF_CMAP_RANGE, 17182 },
	{ 0x7d76, 0x7d79, PDF_CMAP_TABLE, 5736 },
	{ 0x7d7a, 0x7d80, PDF_CMAP_RANGE, 17185 },
	{ 0x7d81, 0x7d83, PDF_CMAP_TABLE, 5740 },
	{ 0x7d84, 0x7d85, PDF_CMAP_RANGE, 17193 },
	{ 0x7d86, 0x7d89, PDF_CMAP_TABLE, 5743 },
	{ 0x7d8a, 0x7d8e, PDF_CMAP_RANGE, 17196 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 8536 },
	{ 0x7d90, 0x7d92, PDF_CMAP_RANGE, 17201 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 8130 },
	{ 0x7d94, 0x7d9b, PDF_CMAP_RANGE, 17204 },
	{ 0x7d9c, 0x7d9e, PDF_CMAP_TABLE, 5747 },
	{ 0x7d9f, 0x7da1, PDF_CMAP_RANGE, 17213 },
	{ 0x7da2, 0x7da3, PDF_CMAP_TABLE, 5750 },
	{ 0x7da4, 0x7da5, PDF_CMAP_RANGE, 17216 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 7401 },
	{ 0x7da7, 0x7daa, PDF_CMAP_RANGE, 17218 },
	{ 0x7dab, 0x7db4, PDF_CMAP_TABLE, 5752 },
	{ 0x7db5, 0x7db7, PDF_CMAP_RANGE, 17223 },
	{ 0x7db8, 0x7dbf, PDF_CMAP_TABLE, 5762 },
	{ 0x7dc0, 0x7dc3, PDF_CMAP_RANGE, 17227 },
	{ 0x7dc4, 0x7dc4, PDF_CMAP_SINGLE, 9269 },
	{ 0x7dc5, 0x7dc6, PDF_CMAP_RANGE, 17231 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 9275 },
	{ 0x7dc8, 0x7dc9, PDF_CMAP_RANGE, 17233 },
	{ 0x7dca, 0x7dcb, PDF_CMAP_TABLE, 5770 },
	{ 0x7dcc, 0x7dd0, PDF_CMAP_RANGE, 17235 },
	{ 0x7dd1, 0x7dd4, PDF_CMAP_TABLE, 5772 },
	{ 0x7dd5, 0x7dd6, PDF_CMAP_RANGE, 17241 },
	{ 0x7dd7, 0x7dd9, PDF_CMAP_TABLE, 5776 },
	{ 0x7dda, 0x7ddc, PDF_CMAP_RANGE, 17243 },
	{ 0x7ddd, 0x7de3, PDF_CMAP_TABLE, 5779 },
	{ 0x7de4, 0x7de5, PDF_CMAP_RANGE, 17248 },
	{ 0x7de6, 0x7de9, PDF_CMAP_TABLE, 5786 },
	{ 0x7dea, 0x7deb, PDF_CMAP_RANGE, 17251 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 8326 },
	{ 0x7ded, 0x7dee, PDF_CMAP_RANGE, 17253 },
	{ 0x7def, 0x7df6, PDF_CMAP_TABLE, 5790 },
	{ 0x7df7, 0x7df8, PDF_CMAP_RANGE, 17258 },
	{ 0x7df9, 0x7dfb, PDF_CMAP_TABLE, 5798 },
	{ 0x7dfc, 0x7e07, PDF_CMAP_RANGE, 17261 },
	{ 0x7e08, 0x7e0b, PDF_CMAP_TABLE, 5801 },
	{ 0x7e0c, 0x7e0f, PDF_CMAP_RANGE, 17273 },
	{ 0x7e10, 0x7e11, PDF_CMAP_TABLE, 5805 },
	{ 0x7e12, 0x7e1a, PDF_CMAP_RANGE, 17277 },
	{ 0x7e1b, 0x7e1f, PDF_CMAP_TABLE, 5807 },
	{ 0x7e20, 0x7e22, PDF_CMAP_RANGE, 17287 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 8642 },
	{ 0x7e24, 0x7e26, PDF_CMAP_RANGE, 17290 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 8558 },
	{ 0x7e28, 0x7e2a, PDF_CMAP_RANGE, 17293 },
	{ 0x7e2b, 0x7e2e, PDF_CMAP_TABLE, 5812 },
	{ 0x7e2f, 0x7e30, PDF_CMAP_RANGE, 17297 },
	{ 0x7e31, 0x7e3e, PDF_CMAP_TABLE, 5816 },
	{ 0x7e3f, 0x7e40, PDF_CMAP_RANGE, 17303 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 1614 },
	{ 0x7e42, 0x7e44, PDF_CMAP_RANGE, 17305 },
	{ 0x7e45, 0x7e47, PDF_CMAP_TABLE, 5830 },
	{ 0x7e48, 0x7e51, PDF_CMAP_RANGE, 17308 },
	{ 0x7e52, 0x7e55, PDF_CMAP_TABLE, 5833 },
	{ 0x7e56, 0x7e59, PDF_CMAP_RANGE, 17319 },
	{ 0x7e5a, 0x7e5a, PDF_CMAP_SINGLE, 9300 },
	{ 0x7e5b, 0x7e5d, PDF_CMAP_RANGE, 17323 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 8443 },
	{ 0x7e5f, 0x7e61, PDF_CMAP_RANGE, 17326 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 9280 },
	{ 0x7e63, 0x7e68, PDF_CMAP_RANGE, 17329 },
	{ 0x7e69, 0x7e70, PDF_CMAP_TABLE, 5837 },
	{ 0x7e71, 0x7e72, PDF_CMAP_RANGE, 17336 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 8110 },
	{ 0x7e74, 0x7e78, PDF_CMAP_RANGE, 17338 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 8730 },
	{ 0x7e7a, 0x7e7b, PDF_CMAP_RANGE, 17343 },
	{ 0x7e7c, 0x7e7e, PDF_CMAP_TABLE, 5845 },
	{ 0x7e7f, 0x7e81, PDF_CMAP_RANGE, 17345 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 4680 },
	{ 0x7e83, 0x7e87, PDF_CMAP_RANGE, 17348 },
	{ 0x7e88, 0x7e8f, PDF_CMAP_TABLE, 5848 },
	{ 0x7e90, 0x7e92, PDF_CMAP_RANGE, 17356 },
	{ 0x7e93, 0x7e98, PDF_CMAP_TABLE, 5856 },
	{ 0x7e99, 0x7e9a, PDF_CMAP_RANGE, 17361 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_TABLE, 5862 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 17363 },
	{ 0x7e9f, 0x7ea7, PDF_CMAP_TABLE, 5864 },
	{ 0x7ea8, 0x7ea9, PDF_CMAP_RANGE, 6103 },
	{ 0x7eaa, 0x7eba, PDF_CMAP_TABLE, 5873 },
	{ 0x7ebb, 0x7ebc, PDF_CMAP_RANGE, 17367 },
	{ 0x7ebd, 0x7ebf, PDF_CMAP_TABLE, 5890 },
	{ 0x7ec0, 0x7ec2, PDF_CMAP_RANGE, 6108 },
	{ 0x7ec3, 0x7eca, PDF_CMAP_TABLE, 5893 },
	{ 0x7ecb, 0x7ecc, PDF_CMAP_RANGE, 6112 },
	{ 0x7ecd, 0x7edf, PDF_CMAP_TABLE, 5901 },
	{ 0x7ee0, 0x7ee1, PDF_CMAP_RANGE, 6118 },
	{ 0x7ee2, 0x7eed, PDF_CMAP_TABLE, 5920 },
	{ 0x7eee, 0x7eef, PDF_CMAP_RANGE, 6122 },
	{ 0x7ef0, 0x7ef0, PDF_CMAP_SINGLE, 1349 },
	{ 0x7ef1, 0x7ef2, PDF_CMAP_RANGE, 6124 },
	{ 0x7ef3, 0x7ef9, PDF_CMAP_TABLE, 5932 },
	{ 0x7efa, 0x7efb, PDF_CMAP_RANGE, 6128 },
	{ 0x7efc, 0x7f00, PDF_CMAP_TABLE, 5939 },
	{ 0x7f01, 0x7f03, PDF_CMAP_RANGE, 6131 },
	{ 0x7f04, 0x7f06, PDF_CMAP_TABLE, 5944 },
	{ 0x7f07, 0x7f08, PDF_CMAP_RANGE, 6134 },
	{ 0x7f09, 0x7f0a, PDF_CMAP_TABLE, 5947 },
	{ 0x7f0b, 0x7f0c, PDF_CMAP_RANGE, 6136 },
	{ 0x7f0d, 0x7f10, PDF_CMAP_TABLE, 5949 },
	{ 0x7f11, 0x7f12, PDF_CMAP_RANGE, 6139 },
	{ 0x7f13, 0x7f20, PDF_CMAP_TABLE, 5953 },
	{ 0x7f21, 0x7f27, PDF_CMAP_RANGE, 6146 },
	{ 0x7f28, 0x7f29, PDF_CMAP_TABLE, 5967 },
	{ 0x7f2a, 0x7f2d, PDF_CMAP_RANGE, 6153 },
	{ 0x7f2e, 0x7f2e, PDF_CMAP_SINGLE, 3330 },
	{ 0x7f2f, 0x7f33, PDF_CMAP_RANGE, 6157 },
	{ 0x7f34, 0x7f3a, PDF_CMAP_TABLE, 5969 },
	{ 0x7f3b, 0x7f41, PDF_CMAP_RANGE, 17378 },
	{ 0x7f42, 0x7f43, PDF_CMAP_TABLE, 5976 },
	{ 0x7f44, 0x7f45, PDF_CMAP_RANGE, 7264 },
	{ 0x7f46, 0x7f4b, PDF_CMAP_RANGE, 17386 },
	{ 0x7f4c, 0x7f51, PDF_CMAP_TABLE, 5978 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 17394 },
	{ 0x7f54, 0x7f5a, PDF_CMAP_TABLE, 5984 },
	{ 0x7f5b, 0x7f5e, PDF_CMAP_RANGE, 17398 },
	{ 0x7f5f, 0x7f62, PDF_CMAP_TABLE, 5991 },
	{ 0x7f63, 0x7f67, PDF_CMAP_RANGE, 17403 },
	{ 0x7f68, 0x7f6a, PDF_CMAP_TABLE, 5995 },
	{ 0x7f6b, 0x7f6d, PDF_CMAP_RANGE, 17408 },
	{ 0x7f6e, 0x7f74, PDF_CMAP_TABLE, 5998 },
	{ 0x7f75, 0x7f76, PDF_CMAP_RANGE, 17413 },
	{ 0x7f77, 0x7f79, PDF_CMAP_TABLE, 6005 },
	{ 0x7f7a, 0x7f7d, PDF_CMAP_RANGE, 17416 },
	{ 0x7f7e, 0x7f7e, PDF_CMAP_SINGLE, 6794 },
	{ 0x7f7f, 0x7f80, PDF_CMAP_RANGE, 17420 },
	{ 0x7f81, 0x7f81, PDF_CMAP_SINGLE, 6793 },
	{ 0x7f82, 0x7f84, PDF_CMAP_RANGE, 17422 },
	{ 0x7f85, 0x7f8e, PDF_CMAP_TABLE, 6008 },
	{ 0x7f8f, 0x7f93, PDF_CMAP_RANGE, 17429 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 1756 },
	{ 0x7f95, 0x7f99, PDF_CMAP_RANGE, 17434 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 2608 },
	{ 0x7f9b, 0x7f9c, PDF_CMAP_RANGE, 17439 },
	{ 0x7f9d, 0x7fa1, PDF_CMAP_TABLE, 6018 },
	{ 0x7fa2, 0x7fa3, PDF_CMAP_RANGE, 17442 },
	{ 0x7fa4, 0x7fa9, PDF_CMAP_TABLE, 6023 },
	{ 0x7faa, 0x7fae, PDF_CMAP_RANGE, 17446 },
	{ 0x7faf, 0x7fb0, PDF_CMAP_RANGE, 7367 },
	{ 0x7fb1, 0x7fb2, PDF_CMAP_TABLE, 6029 },
	{ 0x7fb3, 0x7fb7, PDF_CMAP_RANGE, 17452 },
	{ 0x7fb8, 0x7fb9, PDF_CMAP_TABLE, 6031 },
	{ 0x7fba, 0x7fbb, PDF_CMAP_RANGE, 17457 },
	{ 0x7fbc, 0x7fc1, PDF_CMAP_TABLE, 6033 },
	{ 0x7fc2, 0x7fc4, PDF_CMAP_RANGE, 17461 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 1286 },
	{ 0x7fc6, 0x7fc9, PDF_CMAP_RANGE, 17464 },
	{ 0x7fca, 0x7fce, PDF_CMAP_TABLE, 6039 },
	{ 0x7fcf, 0x7fd1, PDF_CMAP_RANGE, 17470 },
	{ 0x7fd2, 0x7fd5, PDF_CMAP_TABLE, 6044 },
	{ 0x7fd6, 0x7fd7, PDF_CMAP_RANGE, 17474 },
	{ 0x7fd8, 0x7fd8, PDF_CMAP_SINGLE, 3145 },
	{ 0x7fd9, 0x7fde, PDF_CMAP_RANGE, 17476 },
	{ 0x7fdf, 0x7fe1, PDF_CMAP_TABLE, 6048 },
	{ 0x7fe2, 0x7fe4, PDF_CMAP_RANGE, 17482 },
	{ 0x7fe5, 0x7fe6, PDF_CMAP_TABLE, 6051 },
	{ 0x7fe7, 0x7fe8, PDF_CMAP_RANGE, 17485 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 7396 },
	{ 0x7fea, 0x7fed, PDF_CMAP_RANGE, 17487 },
	{ 0x7fee, 0x7ff3, PDF_CMAP_TABLE, 6053 },
	{ 0x7ff4, 0x7ff8, PDF_CMAP_RANGE, 17493 },
	{ 0x7ff9, 0x7ffc, PDF_CMAP_TABLE, 6059 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 17499 },
	{ 0x8000, 0x8006, PDF_CMAP_TABLE, 6063 },
	{ 0x8007, 0x800a, PDF_CMAP_RANGE, 17503 },
	{ 0x800b, 0x800d, PDF_CMAP_TABLE, 6070 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 17507 },
	{ 0x8010, 0x8019, PDF_CMAP_TABLE, 6073 },
	{ 0x801a, 0x801b, PDF_CMAP_RANGE, 17511 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 7118 },
	{ 0x801d, 0x801f, PDF_CMAP_RANGE, 17513 },
	{ 0x8020, 0x8022, PDF_CMAP_TABLE, 6083 },
	{ 0x8023, 0x8024, PDF_CMAP_RANGE, 17517 },
	{ 0x8025, 0x8027, PDF_CMAP_RANGE, 7121 },
	{ 0x8028, 0x802e, PDF_CMAP_TABLE, 6086 },
	{ 0x802f, 0x8030, PDF_CMAP_RANGE, 17521 },
	{ 0x8031, 0x8038, PDF_CMAP_TABLE, 6093 },
	{ 0x8039, 0x803a, PDF_CMAP_RANGE, 17525 },
	{ 0x803b, 0x803f, PDF_CMAP_TABLE, 6101 },
	{ 0x8040, 0x8041, PDF_CMAP_RANGE, 17529 },
	{ 0x8042, 0x8043, PDF_CMAP_TABLE, 6106 },
	{ 0x8044, 0x8045, PDF_CMAP_RANGE, 17531 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 7130 },
	{ 0x8047, 0x8049, PDF_CMAP_RANGE, 17533 },
	{ 0x804a, 0x804d, PDF_CMAP_TABLE, 6108 },
	{ 0x804e, 0x8051, PDF_CMAP_RANGE, 17536 },
	{ 0x8052, 0x805a, PDF_CMAP_TABLE, 6112 },
	{ 0x805b, 0x805d, PDF_CMAP_RANGE, 17544 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 8601 },
	{ 0x805f, 0x8068, PDF_CMAP_RANGE, 17547 },
	{ 0x8069, 0x806a, PDF_CMAP_TABLE, 6121 },
	{ 0x806b, 0x806e, PDF_CMAP_RANGE, 17557 },
	{ 0x806f, 0x8079, PDF_CMAP_TABLE, 6123 },
	{ 0x807a, 0x807c, PDF_CMAP_RANGE, 17563 },
	{ 0x807d, 0x8080, PDF_CMAP_TABLE, 6134 },
	{ 0x8081, 0x8082, PDF_CMAP_RANGE, 17566 },
	{ 0x8083, 0x808c, PDF_CMAP_TABLE, 6138 },
	{ 0x808d, 0x8092, PDF_CMAP_RANGE, 17570 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6513 },
	{ 0x8094, 0x8095, PDF_CMAP_RANGE, 17576 },
	{ 0x8096, 0x80a5, PDF_CMAP_TABLE, 6148 },
	{ 0x80a6, 0x80a8, PDF_CMAP_RANGE, 17582 },
	{ 0x80a9, 0x80b4, PDF_CMAP_TABLE, 6164 },
	{ 0x80b5, 0x80b6, PDF_CMAP_RANGE, 17588 },
	{ 0x80b7, 0x80b7, PDF_CMAP_SINGLE, 6521 },
	{ 0x80b8, 0x80b9, PDF_CMAP_RANGE, 17590 },
	{ 0x80ba, 0x80c6, PDF_CMAP_TABLE, 6176 },
	{ 0x80c7, 0x80cb, PDF_CMAP_RANGE, 17594 },
	{ 0x80cc, 0x80ce, PDF_CMAP_TABLE, 6189 },
	{ 0x80cf, 0x80d5, PDF_CMAP_RANGE, 17599 },
	{ 0x80d6, 0x80de, PDF_CMAP_TABLE, 6192 },
	{ 0x80df, 0x80e0, PDF_CMAP_RANGE, 17607 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1969 },
	{ 0x80e2, 0x80e3, PDF_CMAP_RANGE, 17609 },
	{ 0x80e4, 0x80e6, PDF_CMAP_TABLE, 6201 },
	{ 0x80e7, 0x80ea, PDF_CMAP_RANGE, 6522 },
	{ 0x80eb, 0x80fd, PDF_CMAP_TABLE, 6204 },
	{ 0x80fe, 0x8101, PDF_CMAP_RANGE, 17617 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 4523 },
	{ 0x8103, 0x8104, PDF_CMAP_RANGE, 17621 },
	{ 0x8105, 0x8106, PDF_CMAP_TABLE, 6223 },
	{ 0x8107, 0x8108, PDF_CMAP_RANGE, 17623 },
	{ 0x8109, 0x810a, PDF_CMAP_TABLE, 6225 },
	{ 0x810b, 0x810c, PDF_CMAP_RANGE, 17625 },
	{ 0x810d, 0x810e, PDF_CMAP_RANGE, 6538 },
	{ 0x810f, 0x811b, PDF_CMAP_TABLE, 6227 },
	{ 0x811c, 0x811d, PDF_CMAP_RANGE, 17630 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 6546 },
	{ 0x811f, 0x812b, PDF_CMAP_RANGE, 17632 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 6547 },
	{ 0x812d, 0x812e, PDF_CMAP_RANGE, 17645 },
	{ 0x812f, 0x8132, PDF_CMAP_TABLE, 6240 },
	{ 0x8133, 0x8135, PDF_CMAP_RANGE, 17648 },
	{ 0x8136, 0x8139, PDF_CMAP_TABLE, 6244 },
	{ 0x813a, 0x813d, PDF_CMAP_RANGE, 17652 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 3009 },
	{ 0x813f, 0x8145, PDF_CMAP_RANGE, 17656 },
	{ 0x8146, 0x8156, PDF_CMAP_TABLE, 6248 },
	{ 0x8157, 0x8158, PDF_CMAP_RANGE, 17668 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 6554 },
	{ 0x815b, 0x815f, PDF_CMAP_RANGE, 17670 },
	{ 0x8160, 0x8161, PDF_CMAP_TABLE, 6265 },
	{ 0x8162, 0x8164, PDF_CMAP_RANGE, 17675 },
	{ 0x8165, 0x8171, PDF_CMAP_TABLE, 6267 },
	{ 0x8172, 0x8173, PDF_CMAP_RANGE, 17682 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6553 },
	{ 0x8175, 0x8177, PDF_CMAP_RANGE, 17684 },
	{ 0x8178, 0x817b, PDF_CMAP_TABLE, 6280 },
	{ 0x817c, 0x817d, PDF_CMAP_RANGE, 6559 },
	{ 0x817e, 0x8182, PDF_CMAP_TABLE, 6284 },
	{ 0x8183, 0x8187, PDF_CMAP_RANGE, 17688 },
	{ 0x8188, 0x818a, PDF_CMAP_TABLE, 6289 },
	{ 0x818b, 0x818e, PDF_CMAP_RANGE, 17694 },
	{ 0x818f, 0x8191, PDF_CMAP_TABLE, 6292 },
	{ 0x8192, 0x8197, PDF_CMAP_RANGE, 17699 },
	{ 0x8198, 0x819d, PDF_CMAP_TABLE, 6295 },
	{ 0x819e, 0x819f, PDF_CMAP_RANGE, 17706 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 8101 },
	{ 0x81a1, 0x81a2, PDF_CMAP_RANGE, 17708 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 6569 },
	{ 0x81a4, 0x81a5, PDF_CMAP_RANGE, 17710 },
	{ 0x81a6, 0x81aa, PDF_CMAP_TABLE, 6301 },
	{ 0x81ab, 0x81b2, PDF_CMAP_RANGE, 17713 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 3326 },
	{ 0x81b4, 0x81b9, PDF_CMAP_RANGE, 17721 },
	{ 0x81ba, 0x81c3, PDF_CMAP_TABLE, 6306 },
	{ 0x81c4, 0x81c5, PDF_CMAP_RANGE, 17728 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 4195 },
	{ 0x81c7, 0x81c8, PDF_CMAP_RANGE, 17730 },
	{ 0x81c9, 0x81cf, PDF_CMAP_TABLE, 6316 },
	{ 0x81d0, 0x81d7, PDF_CMAP_RANGE, 17734 },
	{ 0x81d8, 0x81da, PDF_CMAP_TABLE, 6323 },
	{ 0x81db, 0x81de, PDF_CMAP_RANGE, 17743 },
	{ 0x81df, 0x81e0, PDF_CMAP_TABLE, 6326 },
	{ 0x81e1, 0x81e2, PDF_CMAP_RANGE, 17747 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 1249 },
	{ 0x81e4, 0x81e6, PDF_CMAP_RANGE, 17749 },
	{ 0x81e7, 0x81ed, PDF_CMAP_TABLE, 6328 },
	{ 0x81ee, 0x81f2, PDF_CMAP_RANGE, 17754 },
	{ 0x81f3, 0x81f4, PDF_CMAP_RANGE, 4544 },
	{ 0x81f5, 0x81f9, PDF_CMAP_RANGE, 17759 },
	{ 0x81fa, 0x8200, PDF_CMAP_TABLE, 6335 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 7334 },
	{ 0x8203, 0x820b, PDF_CMAP_TABLE, 6342 },
	{ 0x820c, 0x820d, PDF_CMAP_RANGE, 3353 },
	{ 0x820e, 0x820f, PDF_CMAP_RANGE, 17768 },
	{ 0x8210, 0x8214, PDF_CMAP_TABLE, 6351 },
	{ 0x8215, 0x821a, PDF_CMAP_RANGE, 17772 },
	{ 0x821b, 0x8220, PDF_CMAP_TABLE, 6356 },
	{ 0x8221, 0x8223, PDF_CMAP_RANGE, 7339 },
	{ 0x8224, 0x8227, PDF_CMAP_RANGE, 17780 },
	{ 0x8228, 0x8232, PDF_CMAP_TABLE, 6362 },
	{ 0x8233, 0x8234, PDF_CMAP_RANGE, 7348 },
	{ 0x8235, 0x823b, PDF_CMAP_TABLE, 6373 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 17788 },
	{ 0x823e, 0x823e, PDF_CMAP_SINGLE, 7350 },
	{ 0x823f, 0x8243, PDF_CMAP_RANGE, 17790 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 7351 },
	{ 0x8245, 0x8246, PDF_CMAP_RANGE, 17795 },
	{ 0x8247, 0x824b, PDF_CMAP_TABLE, 6380 },
	{ 0x824c, 0x824e, PDF_CMAP_RANGE, 17799 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 7354 },
	{ 0x8250, 0x8257, PDF_CMAP_RANGE, 17802 },
	{ 0x8258, 0x825a, PDF_CMAP_TABLE, 6385 },
	{ 0x825b, 0x825e, PDF_CMAP_RANGE, 17810 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 7356 },
	{ 0x8260, 0x8263, PDF_CMAP_RANGE, 17814 },
	{ 0x8264, 0x8268, PDF_CMAP_TABLE, 6388 },
	{ 0x8269, 0x826a, PDF_CMAP_RANGE, 17820 },
	{ 0x826b, 0x826b, PDF_CMAP_SINGLE, 9712 },
	{ 0x826c, 0x826d, PDF_CMAP_RANGE, 17822 },
	{ 0x826e, 0x8274, PDF_CMAP_TABLE, 6393 },
	{ 0x8275, 0x8276, PDF_CMAP_RANGE, 17824 },
	{ 0x8277, 0x827a, PDF_CMAP_TABLE, 6400 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 17827 },
	{ 0x827d, 0x827f, PDF_CMAP_TABLE, 6404 },
	{ 0x8280, 0x8281, PDF_CMAP_RANGE, 17829 },
	{ 0x8282, 0x8284, PDF_CMAP_TABLE, 6407 },
	{ 0x8285, 0x8287, PDF_CMAP_RANGE, 17832 },
	{ 0x8288, 0x8292, PDF_CMAP_TABLE, 6410 },
	{ 0x8293, 0x8296, PDF_CMAP_RANGE, 17838 },
	{ 0x8297, 0x8299, PDF_CMAP_TABLE, 6421 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 17842 },
	{ 0x829c, 0x82a1, PDF_CMAP_TABLE, 6424 },
	{ 0x82a2, 0x82a3, PDF_CMAP_RANGE, 17846 },
	{ 0x82a4, 0x82b4, PDF_CMAP_TABLE, 6430 },
	{ 0x82b5, 0x82b6, PDF_CMAP_RANGE, 17850 },
	{ 0x82b7, 0x82be, PDF_CMAP_TABLE, 6447 },
	{ 0x82bf, 0x82c0, PDF_CMAP_RANGE, 17854 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 5112 },
	{ 0x82c2, 0x82c3, PDF_CMAP_RANGE, 17856 },
	{ 0x82c4, 0x82c4, PDF_CMAP_SINGLE, 5118 },
	{ 0x82c5, 0x82c6, PDF_CMAP_RANGE, 17858 },
	{ 0x82c7, 0x82ca, PDF_CMAP_TABLE, 6455 },
	{ 0x82cb, 0x82cc, PDF_CMAP_RANGE, 5110 },
	{ 0x82cd, 0x82d8, PDF_CMAP_TABLE, 6459 },
	{ 0x82d9, 0x82da, PDF_CMAP_RANGE, 17863 },
	{ 0x82db, 0x82e7, PDF_CMAP_TABLE, 6471 },
	{ 0x82e8, 0x82ea, PDF_CMAP_RANGE, 17867 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3316 },
	{ 0x82ec, 0x82ee, PDF_CMAP_RANGE, 17870 },
	{ 0x82ef, 0x82f1, PDF_CMAP_TABLE, 6484 },
	{ 0x82f2, 0x82f3, PDF_CMAP_RANGE, 17874 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 5128 },
	{ 0x82f5, 0x82f6, PDF_CMAP_RANGE, 17876 },
	{ 0x82f7, 0x82fb, PDF_CMAP_TABLE, 6487 },
	{ 0x82fc, 0x8300, PDF_CMAP_RANGE, 17880 },
	{ 0x8301, 0x8309, PDF_CMAP_TABLE, 6492 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 17885 },
	{ 0x830c, 0x8311, PDF_CMAP_TABLE, 6501 },
	{ 0x8312, 0x8313, PDF_CMAP_RANGE, 17889 },
	{ 0x8314, 0x8315, PDF_CMAP_RANGE, 5137 },
	{ 0x8316, 0x8317, PDF_CMAP_TABLE, 6507 },
	{ 0x8318, 0x8319, PDF_CMAP_RANGE, 17892 },
	{ 0x831a, 0x831c, PDF_CMAP_TABLE, 6509 },
	{ 0x831d, 0x8326, PDF_CMAP_RANGE, 17894 },
	{ 0x8327, 0x8328, PDF_CMAP_TABLE, 6512 },
	{ 0x8329, 0x832a, PDF_CMAP_RANGE, 17904 },
	{ 0x832b, 0x833c, PDF_CMAP_TABLE, 6514 },
	{ 0x833d, 0x833f, PDF_CMAP_RANGE, 17911 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 5157 },
	{ 0x8341, 0x8342, PDF_CMAP_RANGE, 17914 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 5155 },
	{ 0x8344, 0x8345, PDF_CMAP_RANGE, 17916 },
	{ 0x8346, 0x8349, PDF_CMAP_TABLE, 6532 },
	{ 0x834a, 0x834e, PDF_CMAP_RANGE, 17919 },
	{ 0x834f, 0x8354, PDF_CMAP_TABLE, 6536 },
	{ 0x8355, 0x8359, PDF_CMAP_RANGE, 17925 },
	{ 0x835a, 0x835a, PDF_CMAP_SINGLE, 2119 },
	{ 0x835b, 0x835c, PDF_CMAP_RANGE, 5143 },
	{ 0x835d, 0x836c, PDF_CMAP_TABLE, 6542 },
	{ 0x836d, 0x836e, PDF_CMAP_RANGE, 5170 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 4144 },
	{ 0x8370, 0x8376, PDF_CMAP_RANGE, 17932 },
	{ 0x8377, 0x8378, PDF_CMAP_TABLE, 6558 },
	{ 0x8379, 0x837a, PDF_CMAP_RANGE, 17939 },
	{ 0x837b, 0x837d, PDF_CMAP_TABLE, 6560 },
	{ 0x837e, 0x8384, PDF_CMAP_RANGE, 17941 },
	{ 0x8385, 0x8386, PDF_CMAP_TABLE, 6563 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 17948 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 6565 },
	{ 0x838b, 0x838d, PDF_CMAP_RANGE, 17950 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 3304 },
	{ 0x838f, 0x8391, PDF_CMAP_RANGE, 17953 },
	{ 0x8392, 0x8393, PDF_CMAP_TABLE, 6567 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 17956 },
	{ 0x8396, 0x8398, PDF_CMAP_TABLE, 6569 },
	{ 0x8399, 0x839a, PDF_CMAP_RANGE, 17959 },
	{ 0x839b, 0x83a2, PDF_CMAP_TABLE, 6572 },
	{ 0x83a3, 0x83a6, PDF_CMAP_RANGE, 17964 },
	{ 0x83a7, 0x83ab, PDF_CMAP_TABLE, 6580 },
	{ 0x83ac, 0x83af, PDF_CMAP_RANGE, 17968 },
	{ 0x83b0, 0x83b2, PDF_CMAP_TABLE, 6585 },
	{ 0x83b3, 0x83b4, PDF_CMAP_RANGE, 5174 },
	{ 0x83b5, 0x83bd, PDF_CMAP_TABLE, 6588 },
	{ 0x83be, 0x83bf, PDF_CMAP_RANGE, 17974 },
	{ 0x83c0, 0x83c1, PDF_CMAP_TABLE, 6597 },
	{ 0x83c2, 0x83c4, PDF_CMAP_RANGE, 17976 },
	{ 0x83c5, 0x83c7, PDF_CMAP_TABLE, 6599 },
	{ 0x83c8, 0x83c9, PDF_CMAP_RANGE, 17980 },
	{ 0x83ca, 0x83cc, PDF_CMAP_TABLE, 6602 },
	{ 0x83cd, 0x83ce, PDF_CMAP_RANGE, 17983 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 1921 },
	{ 0x83d0, 0x83d3, PDF_CMAP_RANGE, 17985 },
	{ 0x83d4, 0x83d8, PDF_CMAP_TABLE, 6605 },
	{ 0x83d9, 0x83db, PDF_CMAP_RANGE, 17991 },
	{ 0x83dc, 0x83e1, PDF_CMAP_TABLE, 6610 },
	{ 0x83e2, 0x83e4, PDF_CMAP_RANGE, 17995 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 5194 },
	{ 0x83e6, 0x83e8, PDF_CMAP_RANGE, 17998 },
	{ 0x83e9, 0x83ea, PDF_CMAP_TABLE, 6616 },
	{ 0x83eb, 0x83ee, PDF_CMAP_RANGE, 18001 },
	{ 0x83ef, 0x83f2, PDF_CMAP_TABLE, 6618 },
	{ 0x83f3, 0x83f7, PDF_CMAP_RANGE, 18005 },
	{ 0x83f8, 0x83f9, PDF_CMAP_RANGE, 5210 },
	{ 0x83fa, 0x83fc, PDF_CMAP_RANGE, 18010 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 5200 },
	{ 0x83fe, 0x8400, PDF_CMAP_RANGE, 18013 },
	{ 0x8401, 0x8407, PDF_CMAP_TABLE, 6622 },
	{ 0x8408, 0x8409, PDF_CMAP_RANGE, 18018 },
	{ 0x840a, 0x8411, PDF_CMAP_TABLE, 6629 },
	{ 0x8412, 0x8417, PDF_CMAP_RANGE, 18021 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 5197 },
	{ 0x8419, 0x841b, PDF_CMAP_RANGE, 18027 },
	{ 0x841c, 0x841d, PDF_CMAP_TABLE, 6637 },
	{ 0x841e, 0x8423, PDF_CMAP_RANGE, 18030 },
	{ 0x8424, 0x8425, PDF_CMAP_RANGE, 4238 },
	{ 0x8426, 0x8428, PDF_CMAP_TABLE, 6639 },
	{ 0x8429, 0x842b, PDF_CMAP_RANGE, 18036 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 8587 },
	{ 0x842d, 0x8430, PDF_CMAP_RANGE, 18039 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 5234 },
	{ 0x8432, 0x8434, PDF_CMAP_RANGE, 18043 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 9033 },
	{ 0x8436, 0x8437, PDF_CMAP_RANGE, 18046 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 5203 },
	{ 0x8439, 0x843b, PDF_CMAP_RANGE, 18048 },
	{ 0x843c, 0x843d, PDF_CMAP_TABLE, 6642 },
	{ 0x843e, 0x8445, PDF_CMAP_RANGE, 18051 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 5229 },
	{ 0x8447, 0x8448, PDF_CMAP_RANGE, 18059 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 8715 },
	{ 0x844a, 0x8450, PDF_CMAP_RANGE, 18061 },
	{ 0x8451, 0x8452, PDF_CMAP_TABLE, 6644 },
	{ 0x8453, 0x8456, PDF_CMAP_RANGE, 18068 },
	{ 0x8457, 0x845c, PDF_CMAP_TABLE, 6646 },
	{ 0x845d, 0x8460, PDF_CMAP_RANGE, 18073 },
	{ 0x8461, 0x8466, PDF_CMAP_TABLE, 6652 },
	{ 0x8467, 0x8468, PDF_CMAP_RANGE, 18079 },
	{ 0x8469, 0x847a, PDF_CMAP_TABLE, 6658 },
	{ 0x847b, 0x8481, PDF_CMAP_RANGE, 18087 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 1466 },
	{ 0x8483, 0x8486, PDF_CMAP_RANGE, 18094 },
	{ 0x8487, 0x8488, PDF_CMAP_RANGE, 5223 },
	{ 0x8489, 0x848e, PDF_CMAP_TABLE, 6676 },
	{ 0x848f, 0x8492, PDF_CMAP_RANGE, 18100 },
	{ 0x8493, 0x8494, PDF_CMAP_TABLE, 6682 },
	{ 0x8495, 0x8496, PDF_CMAP_RANGE, 18104 },
	{ 0x8497, 0x8499, PDF_CMAP_TABLE, 6684 },
	{ 0x849a, 0x849b, PDF_CMAP_RANGE, 18107 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3543 },
	{ 0x849d, 0x84a0, PDF_CMAP_RANGE, 18109 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 5246 },
	{ 0x84a2, 0x84ae, PDF_CMAP_RANGE, 18113 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 4759 },
	{ 0x84b0, 0x84b1, PDF_CMAP_RANGE, 18126 },
	{ 0x84b2, 0x84b4, PDF_CMAP_TABLE, 6687 },
	{ 0x84b5, 0x84b7, PDF_CMAP_RANGE, 18129 },
	{ 0x84b8, 0x84c1, PDF_CMAP_TABLE, 6690 },
	{ 0x84c2, 0x84c3, PDF_CMAP_RANGE, 18134 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 4025 },
	{ 0x84c5, 0x84c8, PDF_CMAP_RANGE, 18136 },
	{ 0x84c9, 0x84cd, PDF_CMAP_TABLE, 6700 },
	{ 0x84ce, 0x84cf, PDF_CMAP_RANGE, 18141 },
	{ 0x84d0, 0x84d3, PDF_CMAP_TABLE, 6705 },
	{ 0x84d4, 0x84d5, PDF_CMAP_RANGE, 18144 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 1078 },
	{ 0x84d7, 0x84dc, PDF_CMAP_RANGE, 18146 },
	{ 0x84dd, 0x84e0, PDF_CMAP_TABLE, 6709 },
	{ 0x84e1, 0x84e2, PDF_CMAP_RANGE, 18153 },
	{ 0x84e3, 0x84e6, PDF_CMAP_TABLE, 6713 },
	{ 0x84e7, 0x84eb, PDF_CMAP_RANGE, 18156 },
	{ 0x84ec, 0x84f0, PDF_CMAP_TABLE, 6717 },
	{ 0x84f1, 0x84fb, PDF_CMAP_RANGE, 18162 },
	{ 0x84fc, 0x84ff, PDF_CMAP_TABLE, 6722 },
	{ 0x8500, 0x850b, PDF_CMAP_RANGE, 18174 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 5252 },
	{ 0x850d, 0x8510, PDF_CMAP_RANGE, 18186 },
	{ 0x8511, 0x8514, PDF_CMAP_TABLE, 6726 },
	{ 0x8515, 0x8516, PDF_CMAP_RANGE, 18191 },
	{ 0x8517, 0x8517, PDF_CMAP_SINGLE, 4482 },
	{ 0x8518, 0x8519, PDF_CMAP_RANGE, 18193 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 3780 },
	{ 0x851b, 0x851d, PDF_CMAP_RANGE, 18195 },
	{ 0x851e, 0x8523, PDF_CMAP_TABLE, 6730 },
	{ 0x8524, 0x8525, PDF_CMAP_RANGE, 18200 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 9016 },
	{ 0x8527, 0x852a, PDF_CMAP_RANGE, 18202 },
	{ 0x852b, 0x852d, PDF_CMAP_TABLE, 6736 },
	{ 0x852e, 0x8536, PDF_CMAP_RANGE, 18206 },
	{ 0x8537, 0x853d, PDF_CMAP_TABLE, 6739 },
	{ 0x853e, 0x8540, PDF_CMAP_RANGE, 18215 },
	{ 0x8541, 0x8543, PDF_CMAP_TABLE, 6746 },
	{ 0x8544, 0x8545, PDF_CMAP_RANGE, 18219 },
	{ 0x8546, 0x854a, PDF_CMAP_TABLE, 6749 },
	{ 0x854b, 0x854d, PDF_CMAP_RANGE, 18222 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 9021 },
	{ 0x854f, 0x8551, PDF_CMAP_RANGE, 18225 },
	{ 0x8552, 0x8559, PDF_CMAP_TABLE, 6754 },
	{ 0x855a, 0x855d, PDF_CMAP_RANGE, 18230 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 5267 },
	{ 0x855f, 0x8561, PDF_CMAP_RANGE, 18234 },
	{ 0x8562, 0x8564, PDF_CMAP_TABLE, 6762 },
	{ 0x8565, 0x8567, PDF_CMAP_RANGE, 18238 },
	{ 0x8568, 0x856a, PDF_CMAP_TABLE, 6765 },
	{ 0x856b, 0x856c, PDF_CMAP_RANGE, 18241 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 8651 },
	{ 0x856e, 0x8571, PDF_CMAP_RANGE, 18243 },
	{ 0x8572, 0x8574, PDF_CMAP_TABLE, 6768 },
	{ 0x8575, 0x8576, PDF_CMAP_RANGE, 18248 },
	{ 0x8577, 0x857b, PDF_CMAP_TABLE, 6771 },
	{ 0x857c, 0x857d, PDF_CMAP_RANGE, 18251 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 2500 },
	{ 0x857f, 0x8583, PDF_CMAP_RANGE, 18253 },
	{ 0x8584, 0x858c, PDF_CMAP_TABLE, 6776 },
	{ 0x858d, 0x858e, PDF_CMAP_RANGE, 18261 },
	{ 0x858f, 0x8591, PDF_CMAP_TABLE, 6785 },
	{ 0x8592, 0x8593, PDF_CMAP_RANGE, 18264 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 8410 },
	{ 0x8595, 0x859a, PDF_CMAP_RANGE, 18266 },
	{ 0x859b, 0x859c, PDF_CMAP_TABLE, 6788 },
	{ 0x859d, 0x859e, PDF_CMAP_RANGE, 18272 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 9034 },
	{ 0x85a0, 0x85a3, PDF_CMAP_RANGE, 18274 },
	{ 0x85a4, 0x85aa, PDF_CMAP_TABLE, 6790 },
	{ 0x85ab, 0x85ad, PDF_CMAP_RANGE, 18280 },
	{ 0x85ae, 0x85b0, PDF_CMAP_TABLE, 6797 },
	{ 0x85b1, 0x85b6, PDF_CMAP_RANGE, 18283 },
	{ 0x85b7, 0x85ba, PDF_CMAP_TABLE, 6800 },
	{ 0x85bb, 0x85c0, PDF_CMAP_RANGE, 18290 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 5285 },
	{ 0x85c2, 0x85c8, PDF_CMAP_RANGE, 18296 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 2230 },
	{ 0x85ca, 0x85cc, PDF_CMAP_RANGE, 18303 },
	{ 0x85cd, 0x85d0, PDF_CMAP_TABLE, 6804 },
	{ 0x85d1, 0x85d2, PDF_CMAP_RANGE, 18306 },
	{ 0x85d3, 0x85d5, PDF_CMAP_TABLE, 6808 },
	{ 0x85d6, 0x85db, PDF_CMAP_RANGE, 18309 },
	{ 0x85dc, 0x85dd, PDF_CMAP_TABLE, 6811 },
	{ 0x85de, 0x85e3, PDF_CMAP_RANGE, 18315 },
	{ 0x85e4, 0x85e5, PDF_CMAP_TABLE, 6813 },
	{ 0x85e6, 0x85e8, PDF_CMAP_RANGE, 18321 },
	{ 0x85e9, 0x85ea, PDF_CMAP_TABLE, 6815 },
	{ 0x85eb, 0x85f3, PDF_CMAP_RANGE, 18324 },
	{ 0x85f4, 0x85f6, PDF_CMAP_TABLE, 6817 },
	{ 0x85f7, 0x85f8, PDF_CMAP_RANGE, 18334 },
	{ 0x85f9, 0x85fb, PDF_CMAP_TABLE, 6820 },
	{ 0x85fc, 0x85fe, PDF_CMAP_RANGE, 18336 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 5287 },
	{ 0x8600, 0x8603, PDF_CMAP_RANGE, 18339 },
	{ 0x8604, 0x8607, PDF_CMAP_TABLE, 6823 },
	{ 0x8608, 0x860a, PDF_CMAP_RANGE, 18343 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 8379 },
	{ 0x860c, 0x8610, PDF_CMAP_RANGE, 18346 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 2821 },
	{ 0x8612, 0x8615, PDF_CMAP_RANGE, 18351 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 5291 },
	{ 0x8617, 0x8619, PDF_CMAP_RANGE, 18355 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 9050 },
	{ 0x861b, 0x861d, PDF_CMAP_RANGE, 18358 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 9046 },
	{ 0x861f, 0x8621, PDF_CMAP_RANGE, 18361 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 9015 },
	{ 0x8623, 0x8626, PDF_CMAP_RANGE, 18364 },
	{ 0x8627, 0x8629, PDF_CMAP_TABLE, 6827 },
	{ 0x862a, 0x862c, PDF_CMAP_RANGE, 18369 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 8185 },
	{ 0x862e, 0x8637, PDF_CMAP_RANGE, 18372 },
	{ 0x8638, 0x863c, PDF_CMAP_TABLE, 6830 },
	{ 0x863d, 0x863e, PDF_CMAP_RANGE, 18384 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 8289 },
	{ 0x8640, 0x864c, PDF_CMAP_RANGE, 18386 },
	{ 0x864d, 0x8651, PDF_CMAP_TABLE, 6835 },
	{ 0x8652, 0x8653, PDF_CMAP_RANGE, 18399 },
	{ 0x8654, 0x8655, PDF_CMAP_TABLE, 6840 },
	{ 0x8656, 0x8659, PDF_CMAP_RANGE, 18401 },
	{ 0x865a, 0x865f, PDF_CMAP_TABLE, 6842 },
	{ 0x8660, 0x8661, PDF_CMAP_RANGE, 18407 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 6509 },
	{ 0x8663, 0x8666, PDF_CMAP_RANGE, 18409 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 8168 },
	{ 0x8668, 0x866a, PDF_CMAP_RANGE, 18413 },
	{ 0x866b, 0x866e, PDF_CMAP_TABLE, 6848 },
	{ 0x866f, 0x8670, PDF_CMAP_RANGE, 18417 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 3396 },
	{ 0x8672, 0x8678, PDF_CMAP_RANGE, 18419 },
	{ 0x8679, 0x8682, PDF_CMAP_TABLE, 6852 },
	{ 0x8683, 0x8689, PDF_CMAP_RANGE, 18426 },
	{ 0x868a, 0x868d, PDF_CMAP_TABLE, 6862 },
	{ 0x868e, 0x8692, PDF_CMAP_RANGE, 18433 },
	{ 0x8693, 0x8695, PDF_CMAP_TABLE, 6866 },
	{ 0x8696, 0x869b, PDF_CMAP_RANGE, 18439 },
	{ 0x869c, 0x869d, PDF_CMAP_TABLE, 6869 },
	{ 0x869e, 0x86a2, PDF_CMAP_RANGE, 18445 },
	{ 0x86a3, 0x86a4, PDF_CMAP_TABLE, 6871 },
	{ 0x86a5, 0x86a6, PDF_CMAP_RANGE, 18450 },
	{ 0x86a7, 0x86ac, PDF_CMAP_TABLE, 6873 },
	{ 0x86ad, 0x86ae, PDF_CMAP_RANGE, 18453 },
	{ 0x86af, 0x86b1, PDF_CMAP_TABLE, 6879 },
	{ 0x86b2, 0x86b3, PDF_CMAP_RANGE, 18455 },
	{ 0x86b4, 0x86b6, PDF_CMAP_TABLE, 6882 },
	{ 0x86b7, 0x86b9, PDF_CMAP_RANGE, 18457 },
	{ 0x86ba, 0x86ba, PDF_CMAP_SINGLE, 7175 },
	{ 0x86bb, 0x86bf, PDF_CMAP_RANGE, 18460 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 4602 },
	{ 0x86c1, 0x86c3, PDF_CMAP_RANGE, 18465 },
	{ 0x86c4, 0x86cb, PDF_CMAP_TABLE, 6885 },
	{ 0x86cc, 0x86cd, PDF_CMAP_RANGE, 18470 },
	{ 0x86ce, 0x86d1, PDF_CMAP_TABLE, 6893 },
	{ 0x86d2, 0x86d3, PDF_CMAP_RANGE, 18472 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 2028 },
	{ 0x86d5, 0x86d7, PDF_CMAP_RANGE, 18474 },
	{ 0x86d8, 0x86db, PDF_CMAP_TABLE, 6897 },
	{ 0x86dc, 0x86dd, PDF_CMAP_RANGE, 18478 },
	{ 0x86de, 0x86df, PDF_CMAP_TABLE, 6901 },
	{ 0x86e0, 0x86e3, PDF_CMAP_RANGE, 18480 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 1773 },
	{ 0x86e5, 0x86e8, PDF_CMAP_RANGE, 18484 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 7181 },
	{ 0x86ea, 0x86ec, PDF_CMAP_RANGE, 18488 },
	{ 0x86ed, 0x86f0, PDF_CMAP_TABLE, 6903 },
	{ 0x86f1, 0x86f2, PDF_CMAP_RANGE, 7182 },
	{ 0x86f3, 0x86f4, PDF_CMAP_TABLE, 6907 },
	{ 0x86f5, 0x86f7, PDF_CMAP_RANGE, 18492 },
	{ 0x86f8, 0x86fa, PDF_CMAP_TABLE, 6909 },
	{ 0x86fb, 0x86fd, PDF_CMAP_RANGE, 18495 },
	{ 0x86fe, 0x8703, PDF_CMAP_TABLE, 6912 },
	{ 0x8704, 0x8705, PDF_CMAP_RANGE, 18500 },
	{ 0x8706, 0x870a, PDF_CMAP_TABLE, 6918 },
	{ 0x870b, 0x870c, PDF_CMAP_RANGE, 18502 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 7198 },
	{ 0x870e, 0x8711, PDF_CMAP_RANGE, 18504 },
	{ 0x8712, 0x871e, PDF_CMAP_TABLE, 6923 },
	{ 0x871f, 0x8720, PDF_CMAP_RANGE, 18513 },
	{ 0x8721, 0x8725, PDF_CMAP_TABLE, 6936 },
	{ 0x8726, 0x8728, PDF_CMAP_RANGE, 18516 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 7210 },
	{ 0x872a, 0x872d, PDF_CMAP_RANGE, 18519 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 7204 },
	{ 0x872f, 0x8730, PDF_CMAP_RANGE, 18523 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 7209 },
	{ 0x8732, 0x8733, PDF_CMAP_RANGE, 18525 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 7208 },
	{ 0x8735, 0x8736, PDF_CMAP_RANGE, 18527 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 7211 },
	{ 0x8738, 0x873a, PDF_CMAP_RANGE, 18529 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 7201 },
	{ 0x873c, 0x873d, PDF_CMAP_RANGE, 18532 },
	{ 0x873e, 0x873f, PDF_CMAP_TABLE, 6941 },
	{ 0x8740, 0x8746, PDF_CMAP_RANGE, 18534 },
	{ 0x8747, 0x8749, PDF_CMAP_TABLE, 6943 },
	{ 0x874a, 0x874b, PDF_CMAP_RANGE, 18541 },
	{ 0x874c, 0x874e, PDF_CMAP_TABLE, 6946 },
	{ 0x874f, 0x8752, PDF_CMAP_RANGE, 18544 },
	{ 0x8753, 0x8759, PDF_CMAP_TABLE, 6949 },
	{ 0x875a, 0x875f, PDF_CMAP_RANGE, 18551 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 7218 },
	{ 0x8761, 0x8762, PDF_CMAP_RANGE, 18557 },
	{ 0x8763, 0x8766, PDF_CMAP_TABLE, 6956 },
	{ 0x8767, 0x876d, PDF_CMAP_RANGE, 18559 },
	{ 0x876e, 0x8770, PDF_CMAP_TABLE, 6960 },
	{ 0x8771, 0x8773, PDF_CMAP_RANGE, 18567 },
	{ 0x8774, 0x8778, PDF_CMAP_TABLE, 6963 },
	{ 0x8779, 0x877a, PDF_CMAP_RANGE, 18572 },
	{ 0x877b, 0x877c, PDF_CMAP_TABLE, 6968 },
	{ 0x877d, 0x877e, PDF_CMAP_RANGE, 7215 },
	{ 0x877f, 0x8781, PDF_CMAP_RANGE, 18574 },
	{ 0x8782, 0x8785, PDF_CMAP_TABLE, 6970 },
	{ 0x8786, 0x8787, PDF_CMAP_RANGE, 18577 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 7234 },
	{ 0x8789, 0x878a, PDF_CMAP_RANGE, 18579 },
	{ 0x878b, 0x878d, PDF_CMAP_TABLE, 6974 },
	{ 0x878e, 0x8792, PDF_CMAP_RANGE, 18582 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 7229 },
	{ 0x8794, 0x8796, PDF_CMAP_RANGE, 18587 },
	{ 0x8797, 0x8797, PDF_CMAP_SINGLE, 7237 },
	{ 0x8798, 0x879d, PDF_CMAP_RANGE, 18590 },
	{ 0x879e, 0x879f, PDF_CMAP_TABLE, 6977 },
	{ 0x87a0, 0x87a1, PDF_CMAP_RANGE, 18596 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 8742 },
	{ 0x87a3, 0x87a7, PDF_CMAP_RANGE, 18598 },
	{ 0x87a8, 0x87a8, PDF_CMAP_SINGLE, 7231 },
	{ 0x87a9, 0x87aa, PDF_CMAP_RANGE, 18603 },
	{ 0x87ab, 0x87af, PDF_CMAP_TABLE, 6979 },
	{ 0x87b0, 0x87b2, PDF_CMAP_RANGE, 18606 },
	{ 0x87b3, 0x87b5, PDF_CMAP_TABLE, 6984 },
	{ 0x87b6, 0x87b9, PDF_CMAP_RANGE, 18610 },
	{ 0x87ba, 0x87bd, PDF_CMAP_TABLE, 6987 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 18615 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 7248 },
	{ 0x87c1, 0x87c3, PDF_CMAP_RANGE, 18617 },
	{ 0x87c4, 0x87cb, PDF_CMAP_TABLE, 6991 },
	{ 0x87cc, 0x87cd, PDF_CMAP_RANGE, 18623 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 9697 },
	{ 0x87cf, 0x87d0, PDF_CMAP_RANGE, 18625 },
	{ 0x87d1, 0x87d3, PDF_CMAP_TABLE, 6999 },
	{ 0x87d4, 0x87da, PDF_CMAP_RANGE, 18627 },
	{ 0x87db, 0x87db, PDF_CMAP_SINGLE, 7250 },
	{ 0x87dc, 0x87df, PDF_CMAP_RANGE, 18634 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 7252 },
	{ 0x87e1, 0x87e2, PDF_CMAP_RANGE, 18638 },
	{ 0x87e3, 0x87e5, PDF_CMAP_TABLE, 7002 },
	{ 0x87e6, 0x87e9, PDF_CMAP_RANGE, 18641 },
	{ 0x87ea, 0x87ef, PDF_CMAP_TABLE, 7005 },
	{ 0x87f0, 0x87f1, PDF_CMAP_RANGE, 18647 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 7817 },
	{ 0x87f3, 0x87f5, PDF_CMAP_RANGE, 18649 },
	{ 0x87f6, 0x87f6, PDF_CMAP_SINGLE, 9689 },
	{ 0x87f7, 0x87f8, PDF_CMAP_RANGE, 18652 },
	{ 0x87f9, 0x87fb, PDF_CMAP_TABLE, 7011 },
	{ 0x87fc, 0x87fd, PDF_CMAP_RANGE, 18655 },
	{ 0x87fe, 0x87ff, PDF_CMAP_TABLE, 7014 },
	{ 0x8800, 0x8802, PDF_CMAP_RANGE, 18658 },
	{ 0x8803, 0x8806, PDF_CMAP_TABLE, 7016 },
	{ 0x8807, 0x8809, PDF_CMAP_RANGE, 18662 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 7257 },
	{ 0x880b, 0x880f, PDF_CMAP_RANGE, 18665 },
	{ 0x8810, 0x8816, PDF_CMAP_TABLE, 7020 },
	{ 0x8817, 0x881a, PDF_CMAP_RANGE, 18672 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 7258 },
	{ 0x881c, 0x881e, PDF_CMAP_RANGE, 18676 },
	{ 0x881f, 0x8823, PDF_CMAP_TABLE, 7027 },
	{ 0x8824, 0x8830, PDF_CMAP_RANGE, 18680 },
	{ 0x8831, 0x8832, PDF_CMAP_TABLE, 7032 },
	{ 0x8833, 0x8835, PDF_CMAP_RANGE, 18693 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 7771 },
	{ 0x8837, 0x8838, PDF_CMAP_RANGE, 18696 },
	{ 0x8839, 0x883c, PDF_CMAP_TABLE, 7034 },
	{ 0x883d, 0x883f, PDF_CMAP_RANGE, 18699 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 4051 },
	{ 0x8841, 0x8843, PDF_CMAP_RANGE, 18702 },
	{ 0x8844, 0x8846, PDF_CMAP_TABLE, 7038 },
	{ 0x8847, 0x8849, PDF_CMAP_RANGE, 18705 },
	{ 0x884a, 0x884d, PDF_CMAP_TABLE, 7041 },
	{ 0x884e, 0x8852, PDF_CMAP_RANGE, 18709 },
	{ 0x8853, 0x8854, PDF_CMAP_TABLE, 7045 },
	{ 0x8855, 0x8856, PDF_CMAP_RANGE, 18714 },
	{ 0x8857, 0x885d, PDF_CMAP_TABLE, 7047 },
	{ 0x885e, 0x8860, PDF_CMAP_RANGE, 18719 },
	{ 0x8861, 0x8865, PDF_CMAP_TABLE, 7054 },
	{ 0x8866, 0x8867, PDF_CMAP_RANGE, 18722 },
	{ 0x8868, 0x8872, PDF_CMAP_TABLE, 7059 },
	{ 0x8873, 0x8876, PDF_CMAP_RANGE, 18728 },
	{ 0x8877, 0x8879, PDF_CMAP_TABLE, 7070 },
	{ 0x887a, 0x887c, PDF_CMAP_RANGE, 18733 },
	{ 0x887d, 0x8885, PDF_CMAP_TABLE, 7073 },
	{ 0x8886, 0x8887, PDF_CMAP_RANGE, 18738 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 7360 },
	{ 0x8889, 0x888a, PDF_CMAP_RANGE, 18740 },
	{ 0x888b, 0x888d, PDF_CMAP_TABLE, 7082 },
	{ 0x888e, 0x8891, PDF_CMAP_RANGE, 18743 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 3598 },
	{ 0x8893, 0x8895, PDF_CMAP_RANGE, 18747 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 4015 },
	{ 0x8897, 0x889b, PDF_CMAP_RANGE, 18750 },
	{ 0x889c, 0x889c, PDF_CMAP_SINGLE, 3729 },
	{ 0x889d, 0x88a1, PDF_CMAP_RANGE, 18755 },
	{ 0x88a2, 0x88a4, PDF_CMAP_TABLE, 7085 },
	{ 0x88a5, 0x88aa, PDF_CMAP_RANGE, 18761 },
	{ 0x88ab, 0x88ad, PDF_CMAP_TABLE, 7088 },
	{ 0x88ae, 0x88b0, PDF_CMAP_RANGE, 18768 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 1695 },
	{ 0x88b2, 0x88b6, PDF_CMAP_RANGE, 18771 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 7086 },
	{ 0x88b8, 0x88bb, PDF_CMAP_RANGE, 18776 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 7087 },
	{ 0x88bd, 0x88c0, PDF_CMAP_RANGE, 18780 },
	{ 0x88c1, 0x88c2, PDF_CMAP_TABLE, 7091 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 18784 },
	{ 0x88c5, 0x88c6, PDF_CMAP_TABLE, 7093 },
	{ 0x88c7, 0x88c8, PDF_CMAP_RANGE, 18786 },
	{ 0x88c9, 0x88ca, PDF_CMAP_TABLE, 7095 },
	{ 0x88cb, 0x88cd, PDF_CMAP_RANGE, 18788 },
	{ 0x88ce, 0x88cf, PDF_CMAP_TABLE, 7097 },
	{ 0x88d0, 0x88d1, PDF_CMAP_RANGE, 18791 },
	{ 0x88d2, 0x88d5, PDF_CMAP_TABLE, 7099 },
	{ 0x88d6, 0x88d7, PDF_CMAP_RANGE, 18794 },
	{ 0x88d8, 0x88d9, PDF_CMAP_TABLE, 7103 },
	{ 0x88da, 0x88db, PDF_CMAP_RANGE, 18796 },
	{ 0x88dc, 0x88df, PDF_CMAP_TABLE, 7105 },
	{ 0x88e0, 0x88e1, PDF_CMAP_RANGE, 18799 },
	{ 0x88e2, 0x88e5, PDF_CMAP_TABLE, 7109 },
	{ 0x88e6, 0x88e7, PDF_CMAP_RANGE, 18801 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 7096 },
	{ 0x88e9, 0x88ef, PDF_CMAP_RANGE, 18803 },
	{ 0x88f0, 0x88f4, PDF_CMAP_TABLE, 7113 },
	{ 0x88f5, 0x88f7, PDF_CMAP_RANGE, 18811 },
	{ 0x88f8, 0x88f9, PDF_CMAP_TABLE, 7118 },
	{ 0x88fa, 0x88fb, PDF_CMAP_RANGE, 18814 },
	{ 0x88fc, 0x88ff, PDF_CMAP_TABLE, 7120 },
	{ 0x8900, 0x8901, PDF_CMAP_RANGE, 18817 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 1836 },
	{ 0x8903, 0x8906, PDF_CMAP_RANGE, 18819 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 9846 },
	{ 0x8908, 0x8909, PDF_CMAP_RANGE, 18823 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 7103 },
	{ 0x890b, 0x890f, PDF_CMAP_RANGE, 18825 },
	{ 0x8910, 0x8913, PDF_CMAP_TABLE, 7124 },
	{ 0x8914, 0x8918, PDF_CMAP_RANGE, 18831 },
	{ 0x8919, 0x891b, PDF_CMAP_TABLE, 7128 },
	{ 0x891c, 0x8920, PDF_CMAP_RANGE, 18836 },
	{ 0x8921, 0x8921, PDF_CMAP_SINGLE, 7099 },
	{ 0x8922, 0x8924, PDF_CMAP_RANGE, 18841 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 3271 },
	{ 0x8926, 0x8929, PDF_CMAP_RANGE, 18844 },
	{ 0x892a, 0x892b, PDF_CMAP_TABLE, 7131 },
	{ 0x892c, 0x892f, PDF_CMAP_RANGE, 18848 },
	{ 0x8930, 0x8938, PDF_CMAP_TABLE, 7133 },
	{ 0x8939, 0x893a, PDF_CMAP_RANGE, 18855 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 8925 },
	{ 0x893c, 0x8940, PDF_CMAP_RANGE, 18857 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 7107 },
	{ 0x8942, 0x8943, PDF_CMAP_RANGE, 18862 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 3922 },
	{ 0x8945, 0x8946, PDF_CMAP_RANGE, 18864 },
	{ 0x8947, 0x8947, PDF_CMAP_SINGLE, 9661 },
	{ 0x8948, 0x8955, PDF_CMAP_RANGE, 18866 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 7722 },
	{ 0x8957, 0x895c, PDF_CMAP_RANGE, 18880 },
	{ 0x895d, 0x8960, PDF_CMAP_TABLE, 7142 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 18886 },
	{ 0x8964, 0x8966, PDF_CMAP_TABLE, 7146 },
	{ 0x8967, 0x8969, PDF_CMAP_RANGE, 18890 },
	{ 0x896a, 0x896c, PDF_CMAP_TABLE, 7149 },
	{ 0x896d, 0x896e, PDF_CMAP_RANGE, 18894 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 7807 },
	{ 0x8970, 0x8971, PDF_CMAP_RANGE, 18896 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 8621 },
	{ 0x8973, 0x897a, PDF_CMAP_RANGE, 18898 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 7109 },
	{ 0x897c, 0x897e, PDF_CMAP_RANGE, 18906 },
	{ 0x897f, 0x8983, PDF_CMAP_TABLE, 7152 },
	{ 0x8984, 0x8985, PDF_CMAP_RANGE, 18911 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 1709 },
	{ 0x8987, 0x898a, PDF_CMAP_RANGE, 18913 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8086 },
	{ 0x898c, 0x898e, PDF_CMAP_RANGE, 18917 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 7986 },
	{ 0x8990, 0x8992, PDF_CMAP_RANGE, 18920 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 8324 },
	{ 0x8994, 0x8995, PDF_CMAP_RANGE, 18923 },
	{ 0x8996, 0x8998, PDF_CMAP_TABLE, 7157 },
	{ 0x8999, 0x89a0, PDF_CMAP_RANGE, 18926 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 9402 },
	{ 0x89a2, 0x89a5, PDF_CMAP_RANGE, 18934 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 9404 },
	{ 0x89a7, 0x89a9, PDF_CMAP_RANGE, 18938 },
	{ 0x89aa, 0x89ac, PDF_CMAP_TABLE, 7160 },
	{ 0x89ad, 0x89ae, PDF_CMAP_RANGE, 18942 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 9405 },
	{ 0x89b0, 0x89b1, PDF_CMAP_RANGE, 18944 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 9406 },
	{ 0x89b3, 0x89b6, PDF_CMAP_RANGE, 18946 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 9407 },
	{ 0x89b8, 0x89b9, PDF_CMAP_RANGE, 18950 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8146 },
	{ 0x89bb, 0x89bc, PDF_CMAP_RANGE, 18952 },
	{ 0x89bd, 0x89c9, PDF_CMAP_TABLE, 7163 },
	{ 0x89ca, 0x89cc, PDF_CMAP_RANGE, 6459 },
	{ 0x89cd, 0x89cd, PDF_CMAP_SINGLE, 18956 },
	{ 0x89ce, 0x89d1, PDF_CMAP_RANGE, 6462 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 2200 },
	{ 0x89d3, 0x89d5, PDF_CMAP_RANGE, 18957 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 7511 },
	{ 0x89d7, 0x89d9, PDF_CMAP_RANGE, 18960 },
	{ 0x89da, 0x89de, PDF_CMAP_TABLE, 7176 },
	{ 0x89df, 0x89e2, PDF_CMAP_RANGE, 18965 },
	{ 0x89e3, 0x89e6, PDF_CMAP_TABLE, 7181 },
	{ 0x89e7, 0x89ea, PDF_CMAP_RANGE, 18970 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 7516 },
	{ 0x89ec, 0x89ee, PDF_CMAP_RANGE, 18974 },
	{ 0x89ef, 0x89ef, PDF_CMAP_SINGLE, 7517 },
	{ 0x89f0, 0x89f2, PDF_CMAP_RANGE, 18977 },
	{ 0x89f3, 0x89f8, PDF_CMAP_TABLE, 7185 },
	{ 0x89f9, 0x89ff, PDF_CMAP_RANGE, 18982 },
	{ 0x8a00, 0x8a03, PDF_CMAP_TABLE, 7191 },
	{ 0x8a04, 0x8a06, PDF_CMAP_RANGE, 18989 },
	{ 0x8a07, 0x8a10, PDF_CMAP_TABLE, 7195 },
	{ 0x8a11, 0x8a12, PDF_CMAP_RANGE, 18996 },
	{ 0x8a13, 0x8a18, PDF_CMAP_TABLE, 7205 },
	{ 0x8a19, 0x8a1a, PDF_CMAP_RANGE, 19000 },
	{ 0x8a1b, 0x8a1f, PDF_CMAP_TABLE, 7211 },
	{ 0x8a20, 0x8a22, PDF_CMAP_RANGE, 19004 },
	{ 0x8a23, 0x8a25, PDF_CMAP_TABLE, 7216 },
	{ 0x8a26, 0x8a29, PDF_CMAP_RANGE, 19008 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 7922 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 19012 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 8479 },
	{ 0x8a2e, 0x8a30, PDF_CMAP_RANGE, 19014 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 8669 },
	{ 0x8a32, 0x8a33, PDF_CMAP_RANGE, 19017 },
	{ 0x8a34, 0x8a36, PDF_CMAP_TABLE, 7219 },
	{ 0x8a37, 0x8a39, PDF_CMAP_RANGE, 19020 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 8834 },
	{ 0x8a3b, 0x8a3d, PDF_CMAP_RANGE, 19023 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 7518 },
	{ 0x8a3f, 0x8a40, PDF_CMAP_RANGE, 19026 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 8934 },
	{ 0x8a42, 0x8a45, PDF_CMAP_RANGE, 19028 },
	{ 0x8a46, 0x8a48, PDF_CMAP_TABLE, 7222 },
	{ 0x8a49, 0x8a4d, PDF_CMAP_RANGE, 19033 },
	{ 0x8a4e, 0x8a55, PDF_CMAP_TABLE, 7225 },
	{ 0x8a56, 0x8a57, PDF_CMAP_RANGE, 19041 },
	{ 0x8a58, 0x8a58, PDF_CMAP_SINGLE, 8938 },
	{ 0x8a59, 0x8a5a, PDF_CMAP_RANGE, 19043 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 8893 },
	{ 0x8a5c, 0x8a5d, PDF_CMAP_RANGE, 19045 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 7838 },
	{ 0x8a5f, 0x8a60, PDF_CMAP_RANGE, 19047 },
	{ 0x8a61, 0x8a63, PDF_CMAP_TABLE, 7233 },
	{ 0x8a64, 0x8a65, PDF_CMAP_RANGE, 19049 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 8503 },
	{ 0x8a67, 0x8a68, PDF_CMAP_RANGE, 19051 },
	{ 0x8a69, 0x8a75, PDF_CMAP_TABLE, 7236 },
	{ 0x8a76, 0x8a78, PDF_CMAP_RANGE, 19056 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 4435 },
	{ 0x8a7a, 0x8a7b, PDF_CMAP_RANGE, 19059 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 8944 },
	{ 0x8a7d, 0x8a7e, PDF_CMAP_RANGE, 19061 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 8942 },
	{ 0x8a80, 0x8a83, PDF_CMAP_RANGE, 19063 },
	{ 0x8a84, 0x8a8a, PDF_CMAP_TABLE, 7249 },
	{ 0x8a8b, 0x8a8c, PDF_CMAP_RANGE, 19068 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 8446 },
	{ 0x8a8e, 0x8a90, PDF_CMAP_RANGE, 19070 },
	{ 0x8a91, 0x8a92, PDF_CMAP_RANGE, 8953 },
	{ 0x8a93, 0x8a95, PDF_CMAP_TABLE, 7256 },
	{ 0x8a96, 0x8a97, PDF_CMAP_RANGE, 19074 },
	{ 0x8a98, 0x8a9a, PDF_CMAP_TABLE, 7259 },
	{ 0x8a9b, 0x8a9d, PDF_CMAP_RANGE, 19077 },
	{ 0x8a9e, 0x8aa8, PDF_CMAP_TABLE, 7262 },
	{ 0x8aa9, 0x8aab, PDF_CMAP_RANGE, 19083 },
	{ 0x8aac, 0x8aac, PDF_CMAP_SINGLE, 8519 },
	{ 0x8aad, 0x8aaf, PDF_CMAP_RANGE, 19086 },
	{ 0x8ab0, 0x8ab2, PDF_CMAP_TABLE, 7273 },
	{ 0x8ab3, 0x8ab5, PDF_CMAP_RANGE, 19090 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 8961 },
	{ 0x8ab7, 0x8ab8, PDF_CMAP_RANGE, 19093 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 7925 },
	{ 0x8aba, 0x8abb, PDF_CMAP_RANGE, 19095 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 8728 },
	{ 0x8abd, 0x8abe, PDF_CMAP_RANGE, 19097 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 7878 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_RANGE, 19099 },
	{ 0x8ac2, 0x8ac4, PDF_CMAP_TABLE, 7276 },
	{ 0x8ac5, 0x8ac6, PDF_CMAP_RANGE, 19102 },
	{ 0x8ac7, 0x8ad2, PDF_CMAP_TABLE, 7279 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 19109 },
	{ 0x8ad6, 0x8ad7, PDF_CMAP_TABLE, 7291 },
	{ 0x8ad8, 0x8ada, PDF_CMAP_RANGE, 19112 },
	{ 0x8adb, 0x8ade, PDF_CMAP_TABLE, 7293 },
	{ 0x8adf, 0x8ae1, PDF_CMAP_RANGE, 19116 },
	{ 0x8ae2, 0x8ae7, PDF_CMAP_TABLE, 7297 },
	{ 0x8ae8, 0x8aea, PDF_CMAP_RANGE, 19121 },
	{ 0x8aeb, 0x8aee, PDF_CMAP_TABLE, 7303 },
	{ 0x8aef, 0x8af0, PDF_CMAP_RANGE, 19125 },
	{ 0x8af1, 0x8af3, PDF_CMAP_TABLE, 7307 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 19128 },
	{ 0x8af6, 0x8b07, PDF_CMAP_TABLE, 7310 },
	{ 0x8b08, 0x8b09, PDF_CMAP_RANGE, 19136 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 8023 },
	{ 0x8b0b, 0x8b0d, PDF_CMAP_RANGE, 19138 },
	{ 0x8b0e, 0x8b10, PDF_CMAP_TABLE, 7328 },
	{ 0x8b11, 0x8b13, PDF_CMAP_RANGE, 19142 },
	{ 0x8b14, 0x8b1d, PDF_CMAP_TABLE, 7331 },
	{ 0x8b1e, 0x8b20, PDF_CMAP_RANGE, 19148 },
	{ 0x8b21, 0x8b21, PDF_CMAP_SINGLE, 8710 },
	{ 0x8b22, 0x8b25, PDF_CMAP_RANGE, 19151 },
	{ 0x8b26, 0x8b28, PDF_CMAP_TABLE, 7341 },
	{ 0x8b29, 0x8b2a, PDF_CMAP_RANGE, 19156 },
	{ 0x8b2b, 0x8b2d, PDF_CMAP_TABLE, 7344 },
	{ 0x8b2e, 0x8b32, PDF_CMAP_RANGE, 19158 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 8931 },
	{ 0x8b34, 0x8b38, PDF_CMAP_RANGE, 19163 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 8122 },
	{ 0x8b3a, 0x8b3d, PDF_CMAP_RANGE, 19168 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 8312 },
	{ 0x8b3f, 0x8b48, PDF_CMAP_RANGE, 19172 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 8840 },
	{ 0x8b4a, 0x8b4d, PDF_CMAP_RANGE, 19182 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_TABLE, 7347 },
	{ 0x8b50, 0x8b55, PDF_CMAP_RANGE, 19186 },
	{ 0x8b56, 0x8b5c, PDF_CMAP_TABLE, 7349 },
	{ 0x8b5d, 0x8b65, PDF_CMAP_RANGE, 19194 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 2270 },
	{ 0x8b67, 0x8b6a, PDF_CMAP_RANGE, 19203 },
	{ 0x8b6b, 0x8b6c, PDF_CMAP_TABLE, 7356 },
	{ 0x8b6d, 0x8b6e, PDF_CMAP_RANGE, 19207 },
	{ 0x8b6f, 0x8b70, PDF_CMAP_TABLE, 7358 },
	{ 0x8b71, 0x8b73, PDF_CMAP_RANGE, 19209 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 8405 },
	{ 0x8b75, 0x8b76, PDF_CMAP_RANGE, 19212 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 8010 },
	{ 0x8b78, 0x8b7c, PDF_CMAP_RANGE, 19214 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 8768 },
	{ 0x8b7e, 0x8b7f, PDF_CMAP_RANGE, 19219 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 7891 },
	{ 0x8b81, 0x8b89, PDF_CMAP_RANGE, 19221 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 7753 },
	{ 0x8b8b, 0x8b8d, PDF_CMAP_RANGE, 19230 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 9755 },
	{ 0x8b8f, 0x8b91, PDF_CMAP_RANGE, 19233 },
	{ 0x8b92, 0x8b96, PDF_CMAP_TABLE, 7360 },
	{ 0x8b97, 0x8b9b, PDF_CMAP_RANGE, 19237 },
	{ 0x8b9c, 0x8ba5, PDF_CMAP_TABLE, 7365 },
	{ 0x8ba6, 0x8ba7, PDF_CMAP_RANGE, 4888 },
	{ 0x8ba8, 0x8bb3, PDF_CMAP_TABLE, 7375 },
	{ 0x8bb4, 0x8bb5, PDF_CMAP_RANGE, 4891 },
	{ 0x8bb6, 0x8bc1, PDF_CMAP_TABLE, 7387 },
	{ 0x8bc2, 0x8bc3, PDF_CMAP_RANGE, 4894 },
	{ 0x8bc4, 0x8bd1, PDF_CMAP_TABLE, 7399 },
	{ 0x8bd2, 0x8bd4, PDF_CMAP_RANGE, 4899 },
	{ 0x8bd5, 0x8bd7, PDF_CMAP_TABLE, 7413 },
	{ 0x8bd8, 0x8bd9, PDF_CMAP_RANGE, 4903 },
	{ 0x8bda, 0x8bde, PDF_CMAP_TABLE, 7416 },
	{ 0x8bdf, 0x8be0, PDF_CMAP_RANGE, 4906 },
	{ 0x8be1, 0x8be7, PDF_CMAP_TABLE, 7421 },
	{ 0x8be8, 0x8be9, PDF_CMAP_RANGE, 4909 },
	{ 0x8bea, 0x8c10, PDF_CMAP_TABLE, 7428 },
	{ 0x8c11, 0x8c12, PDF_CMAP_RANGE, 4924 },
	{ 0x8c13, 0x8c13, PDF_CMAP_SINGLE, 3788 },
	{ 0x8c14, 0x8c16, PDF_CMAP_RANGE, 4926 },
	{ 0x8c17, 0x8c1e, PDF_CMAP_TABLE, 7467 },
	{ 0x8c1f, 0x8c21, PDF_CMAP_RANGE, 4933 },
	{ 0x8c22, 0x8c29, PDF_CMAP_TABLE, 7475 },
	{ 0x8c2a, 0x8c2b, PDF_CMAP_RANGE, 4938 },
	{ 0x8c2c, 0x8c2d, PDF_CMAP_TABLE, 7483 },
	{ 0x8c2e, 0x8c2f, PDF_CMAP_RANGE, 4940 },
	{ 0x8c30, 0x8c31, PDF_CMAP_TABLE, 7485 },
	{ 0x8c32, 0x8c33, PDF_CMAP_RANGE, 4942 },
	{ 0x8c34, 0x8c34, PDF_CMAP_SINGLE, 3121 },
	{ 0x8c35, 0x8c36, PDF_CMAP_RANGE, 4944 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1825 },
	{ 0x8c38, 0x8c40, PDF_CMAP_RANGE, 19252 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 2050 },
	{ 0x8c42, 0x8c45, PDF_CMAP_RANGE, 19261 },
	{ 0x8c46, 0x8c49, PDF_CMAP_TABLE, 7487 },
	{ 0x8c4a, 0x8c4b, PDF_CMAP_RANGE, 19265 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 3732 },
	{ 0x8c4d, 0x8c4f, PDF_CMAP_RANGE, 19267 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 7933 },
	{ 0x8c51, 0x8c54, PDF_CMAP_RANGE, 19270 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 7445 },
	{ 0x8c56, 0x8c59, PDF_CMAP_RANGE, 19274 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 6544 },
	{ 0x8c5b, 0x8c60, PDF_CMAP_RANGE, 19278 },
	{ 0x8c61, 0x8c62, PDF_CMAP_TABLE, 7491 },
	{ 0x8c63, 0x8c69, PDF_CMAP_RANGE, 19284 },
	{ 0x8c6a, 0x8c6b, PDF_CMAP_TABLE, 7493 },
	{ 0x8c6c, 0x8c72, PDF_CMAP_RANGE, 19291 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 5597 },
	{ 0x8c74, 0x8c77, PDF_CMAP_RANGE, 19298 },
	{ 0x8c78, 0x8c7a, PDF_CMAP_TABLE, 7495 },
	{ 0x8c7b, 0x8c81, PDF_CMAP_RANGE, 19302 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 7505 },
	{ 0x8c83, 0x8c84, PDF_CMAP_RANGE, 19309 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 7507 },
	{ 0x8c86, 0x8c88, PDF_CMAP_RANGE, 19311 },
	{ 0x8c89, 0x8c8c, PDF_CMAP_TABLE, 7498 },
	{ 0x8c8d, 0x8c93, PDF_CMAP_RANGE, 19315 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 7509 },
	{ 0x8c95, 0x8c97, PDF_CMAP_RANGE, 19322 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 7508 },
	{ 0x8c99, 0x8c9c, PDF_CMAP_RANGE, 19325 },
	{ 0x8c9d, 0x8ca2, PDF_CMAP_TABLE, 7502 },
	{ 0x8ca3, 0x8ca6, PDF_CMAP_RANGE, 19330 },
	{ 0x8ca7, 0x8cac, PDF_CMAP_TABLE, 7508 },
	{ 0x8cad, 0x8cae, PDF_CMAP_RANGE, 19334 },
	{ 0x8caf, 0x8cca, PDF_CMAP_TABLE, 7514 },
	{ 0x8ccb, 0x8cd0, PDF_CMAP_RANGE, 19342 },
	{ 0x8cd1, 0x8cd5, PDF_CMAP_TABLE, 7542 },
	{ 0x8cd6, 0x8cd9, PDF_CMAP_RANGE, 19349 },
	{ 0x8cda, 0x8ce7, PDF_CMAP_TABLE, 7547 },
	{ 0x8ce8, 0x8ce9, PDF_CMAP_RANGE, 19357 },
	{ 0x8cea, 0x8ced, PDF_CMAP_TABLE, 7561 },
	{ 0x8cee, 0x8cf3, PDF_CMAP_RANGE, 19359 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8179 },
	{ 0x8cf5, 0x8cf9, PDF_CMAP_RANGE, 19365 },
	{ 0x8cfa, 0x8cff, PDF_CMAP_TABLE, 7565 },
	{ 0x8d00, 0x8d03, PDF_CMAP_RANGE, 19371 },
	{ 0x8d04, 0x8d05, PDF_CMAP_TABLE, 7571 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 19375 },
	{ 0x8d08, 0x8d10, PDF_CMAP_TABLE, 7573 },
	{ 0x8d11, 0x8d15, PDF_CMAP_RANGE, 19380 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 8509 },
	{ 0x8d17, 0x8d1a, PDF_CMAP_RANGE, 19385 },
	{ 0x8d1b, 0x8d31, PDF_CMAP_TABLE, 7582 },
	{ 0x8d32, 0x8d33, PDF_CMAP_RANGE, 6444 },
	{ 0x8d34, 0x8d44, PDF_CMAP_TABLE, 7605 },
	{ 0x8d45, 0x8d46, PDF_CMAP_RANGE, 6450 },
	{ 0x8d47, 0x8d47, PDF_CMAP_SINGLE, 6454 },
	{ 0x8d48, 0x8d49, PDF_CMAP_RANGE, 6452 },
	{ 0x8d4a, 0x8d50, PDF_CMAP_TABLE, 7622 },
	{ 0x8d51, 0x8d52, PDF_CMAP_RANGE, 19390 },
	{ 0x8d53, 0x8d67, PDF_CMAP_TABLE, 7629 },
	{ 0x8d68, 0x8d6a, PDF_CMAP_RANGE, 19395 },
	{ 0x8d6b, 0x8d6d, PDF_CMAP_TABLE, 7650 },
	{ 0x8d6e, 0x8d6f, PDF_CMAP_RANGE, 19399 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 4668 },
	{ 0x8d71, 0x8d72, PDF_CMAP_RANGE, 19401 },
	{ 0x8d73, 0x8d77, PDF_CMAP_TABLE, 7653 },
	{ 0x8d78, 0x8d80, PDF_CMAP_RANGE, 19403 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 1256 },
	{ 0x8d82, 0x8d83, PDF_CMAP_RANGE, 19412 },
	{ 0x8d84, 0x8d85, PDF_CMAP_TABLE, 7658 },
	{ 0x8d86, 0x8d89, PDF_CMAP_RANGE, 19414 },
	{ 0x8d8a, 0x8d8b, PDF_CMAP_TABLE, 7660 },
	{ 0x8d8c, 0x8d90, PDF_CMAP_RANGE, 19418 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 7410 },
	{ 0x8d92, 0x8d93, PDF_CMAP_RANGE, 19423 },
	{ 0x8d94, 0x8d95, PDF_CMAP_TABLE, 7662 },
	{ 0x8d96, 0x8d98, PDF_CMAP_RANGE, 19425 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 8826 },
	{ 0x8d9a, 0x8d9e, PDF_CMAP_RANGE, 19428 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 3614 },
	{ 0x8da0, 0x8da2, PDF_CMAP_RANGE, 19433 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 3199 },
	{ 0x8da4, 0x8da7, PDF_CMAP_RANGE, 19436 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 8430 },
	{ 0x8da9, 0x8db0, PDF_CMAP_RANGE, 19440 },
	{ 0x8db1, 0x8db5, PDF_CMAP_TABLE, 7664 },
	{ 0x8db6, 0x8db7, PDF_CMAP_RANGE, 19448 },
	{ 0x8db8, 0x8dbf, PDF_CMAP_TABLE, 7669 },
	{ 0x8dc0, 0x8dc2, PDF_CMAP_RANGE, 19453 },
	{ 0x8dc3, 0x8dc6, PDF_CMAP_TABLE, 7677 },
	{ 0x8dc7, 0x8dca, PDF_CMAP_RANGE, 19457 },
	{ 0x8dcb, 0x8dcd, PDF_CMAP_TABLE, 7681 },
	{ 0x8dce, 0x8dcf, PDF_CMAP_RANGE, 7461 },
	{ 0x8dd0, 0x8dd1, PDF_CMAP_TABLE, 7684 },
	{ 0x8dd2, 0x8dd5, PDF_CMAP_RANGE, 19463 },
	{ 0x8dd6, 0x8dd7, PDF_CMAP_RANGE, 7457 },
	{ 0x8dd8, 0x8dd9, PDF_CMAP_RANGE, 19467 },
	{ 0x8dda, 0x8ddf, PDF_CMAP_TABLE, 7686 },
	{ 0x8de0, 0x8de2, PDF_CMAP_RANGE, 19470 },
	{ 0x8de3, 0x8de4, PDF_CMAP_TABLE, 7692 },
	{ 0x8de5, 0x8de7, PDF_CMAP_RANGE, 19473 },
	{ 0x8de8, 0x8dec, PDF_CMAP_TABLE, 7694 },
	{ 0x8ded, 0x8dee, PDF_CMAP_RANGE, 19477 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 2654 },
	{ 0x8df0, 0x8df2, PDF_CMAP_RANGE, 19479 },
	{ 0x8df3, 0x8df6, PDF_CMAP_TABLE, 7699 },
	{ 0x8df7, 0x8df8, PDF_CMAP_RANGE, 7466 },
	{ 0x8df9, 0x8dfd, PDF_CMAP_TABLE, 7703 },
	{ 0x8dfe, 0x8e04, PDF_CMAP_RANGE, 19485 },
	{ 0x8e05, 0x8e05, PDF_CMAP_SINGLE, 7449 },
	{ 0x8e06, 0x8e08, PDF_CMAP_RANGE, 19492 },
	{ 0x8e09, 0x8e0c, PDF_CMAP_TABLE, 7708 },
	{ 0x8e0d, 0x8e0e, PDF_CMAP_RANGE, 19496 },
	{ 0x8e0f, 0x8e10, PDF_CMAP_TABLE, 7712 },
	{ 0x8e11, 0x8e13, PDF_CMAP_RANGE, 19498 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 7474 },
	{ 0x8e15, 0x8e1c, PDF_CMAP_RANGE, 19501 },
	{ 0x8e1d, 0x8e1f, PDF_CMAP_TABLE, 7714 },
	{ 0x8e20, 0x8e21, PDF_CMAP_RANGE, 19509 },
	{ 0x8e22, 0x8e23, PDF_CMAP_TABLE, 7717 },
	{ 0x8e24, 0x8e28, PDF_CMAP_RANGE, 19511 },
	{ 0x8e29, 0x8e31, PDF_CMAP_TABLE, 7719 },
	{ 0x8e32, 0x8e33, PDF_CMAP_RANGE, 19519 },
	{ 0x8e34, 0x8e35, PDF_CMAP_TABLE, 7728 },
	{ 0x8e36, 0x8e38, PDF_CMAP_RANGE, 19521 },
	{ 0x8e39, 0x8e3a, PDF_CMAP_TABLE, 7730 },
	{ 0x8e3b, 0x8e3c, PDF_CMAP_RANGE, 19524 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 7485 },
	{ 0x8e3e, 0x8e3f, PDF_CMAP_RANGE, 19526 },
	{ 0x8e40, 0x8e40, PDF_CMAP_SINGLE, 7482 },
	{ 0x8e41, 0x8e42, PDF_CMAP_RANGE, 7488 },
	{ 0x8e43, 0x8e44, PDF_CMAP_TABLE, 7732 },
	{ 0x8e45, 0x8e46, PDF_CMAP_RANGE, 19529 },
	{ 0x8e47, 0x8e4c, PDF_CMAP_TABLE, 7734 },
	{ 0x8e4d, 0x8e50, PDF_CMAP_RANGE, 19531 },
	{ 0x8e51, 0x8e52, PDF_CMAP_RANGE, 7490 },
	{ 0x8e53, 0x8e54, PDF_CMAP_RANGE, 19535 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 9728 },
	{ 0x8e56, 0x8e58, PDF_CMAP_RANGE, 19537 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 7450 },
	{ 0x8e5a, 0x8e62, PDF_CMAP_RANGE, 19540 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 9734 },
	{ 0x8e64, 0x8e65, PDF_CMAP_RANGE, 19549 },
	{ 0x8e66, 0x8e66, PDF_CMAP_SINGLE, 1069 },
	{ 0x8e67, 0x8e68, PDF_CMAP_RANGE, 19551 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 7451 },
	{ 0x8e6a, 0x8e6b, PDF_CMAP_RANGE, 19553 },
	{ 0x8e6c, 0x8e76, PDF_CMAP_TABLE, 7740 },
	{ 0x8e77, 0x8e79, PDF_CMAP_RANGE, 19559 },
	{ 0x8e7a, 0x8e7c, PDF_CMAP_TABLE, 7751 },
	{ 0x8e7d, 0x8e7e, PDF_CMAP_RANGE, 19563 },
	{ 0x8e7f, 0x8e81, PDF_CMAP_TABLE, 7754 },
	{ 0x8e82, 0x8e84, PDF_CMAP_RANGE, 19566 },
	{ 0x8e85, 0x8e94, PDF_CMAP_TABLE, 7757 },
	{ 0x8e95, 0x8e99, PDF_CMAP_RANGE, 19573 },
	{ 0x8e9a, 0x8e9e, PDF_CMAP_TABLE, 7773 },
	{ 0x8e9f, 0x8ea0, PDF_CMAP_RANGE, 19580 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 9733 },
	{ 0x8ea2, 0x8ea4, PDF_CMAP_RANGE, 19582 },
	{ 0x8ea5, 0x8ea6, PDF_CMAP_TABLE, 7778 },
	{ 0x8ea7, 0x8ea9, PDF_CMAP_RANGE, 19585 },
	{ 0x8eaa, 0x8eac, PDF_CMAP_TABLE, 7780 },
	{ 0x8ead, 0x8eae, PDF_CMAP_RANGE, 19588 },
	{ 0x8eaf, 0x8eaf, PDF_CMAP_SINGLE, 3192 },
	{ 0x8eb0, 0x8eb1, PDF_CMAP_RANGE, 19590 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 1570 },
	{ 0x8eb3, 0x8eb9, PDF_CMAP_RANGE, 19592 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 3612 },
	{ 0x8ebb, 0x8ebf, PDF_CMAP_RANGE, 19599 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 8432 },
	{ 0x8ec1, 0x8ec9, PDF_CMAP_RANGE, 19604 },
	{ 0x8eca, 0x8ece, PDF_CMAP_TABLE, 7783 },
	{ 0x8ecf, 0x8ed1, PDF_CMAP_RANGE, 19613 },
	{ 0x8ed2, 0x8ed4, PDF_CMAP_TABLE, 7788 },
	{ 0x8ed5, 0x8eda, PDF_CMAP_RANGE, 19617 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 9357 },
	{ 0x8edc, 0x8ede, PDF_CMAP_RANGE, 19623 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 8450 },
	{ 0x8ee0, 0x8ee3, PDF_CMAP_RANGE, 19626 },
	{ 0x8ee4, 0x8ee4, PDF_CMAP_SINGLE, 9364 },
	{ 0x8ee5, 0x8eea, PDF_CMAP_RANGE, 19630 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 9363 },
	{ 0x8eec, 0x8ef1, PDF_CMAP_RANGE, 19636 },
	{ 0x8ef2, 0x8ef2, PDF_CMAP_SINGLE, 9358 },
	{ 0x8ef3, 0x8ef7, PDF_CMAP_RANGE, 19642 },
	{ 0x8ef8, 0x8eff, PDF_CMAP_TABLE, 7791 },
	{ 0x8f00, 0x8f02, PDF_CMAP_RANGE, 19649 },
	{ 0x8f03, 0x8f0a, PDF_CMAP_TABLE, 7799 },
	{ 0x8f0b, 0x8f11, PDF_CMAP_RANGE, 19655 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 7807 },
	{ 0x8f16, 0x8f1a, PDF_CMAP_RANGE, 19663 },
	{ 0x8f1b, 0x8f1d, PDF_CMAP_TABLE, 7811 },
	{ 0x8f1e, 0x8f1f, PDF_CMAP_RANGE, 9373 },
	{ 0x8f20, 0x8f24, PDF_CMAP_RANGE, 19668 },
	{ 0x8f25, 0x8f26, PDF_CMAP_TABLE, 7814 },
	{ 0x8f27, 0x8f28, PDF_CMAP_RANGE, 19673 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 7816 },
	{ 0x8f2b, 0x8f2e, PDF_CMAP_RANGE, 19675 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 8048 },
	{ 0x8f30, 0x8f32, PDF_CMAP_RANGE, 19679 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 9376 },
	{ 0x8f34, 0x8f37, PDF_CMAP_RANGE, 19682 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 8507 },
	{ 0x8f39, 0x8f3a, PDF_CMAP_RANGE, 19686 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 7943 },
	{ 0x8f3c, 0x8f3d, PDF_CMAP_RANGE, 19688 },
	{ 0x8f3e, 0x8f3f, PDF_CMAP_TABLE, 7818 },
	{ 0x8f40, 0x8f41, PDF_CMAP_RANGE, 19690 },
	{ 0x8f42, 0x8f46, PDF_CMAP_TABLE, 7820 },
	{ 0x8f47, 0x8f48, PDF_CMAP_RANGE, 19693 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 8872 },
	{ 0x8f4a, 0x8f4c, PDF_CMAP_RANGE, 19695 },
	{ 0x8f4d, 0x8f4e, PDF_CMAP_TABLE, 7825 },
	{ 0x8f4f, 0x8f53, PDF_CMAP_RANGE, 19698 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 9378 },
	{ 0x8f55, 0x8f5e, PDF_CMAP_RANGE, 19703 },
	{ 0x8f5f, 0x8f70, PDF_CMAP_TABLE, 7827 },
	{ 0x8f71, 0x8f73, PDF_CMAP_RANGE, 6371 },
	{ 0x8f74, 0x8f74, PDF_CMAP_SINGLE, 4576 },
	{ 0x8f75, 0x8f76, PDF_CMAP_RANGE, 6374 },
	{ 0x8f77, 0x8f78, PDF_CMAP_TABLE, 7845 },
	{ 0x8f79, 0x8f7a, PDF_CMAP_RANGE, 6378 },
	{ 0x8f7b, 0x8f80, PDF_CMAP_TABLE, 7847 },
	{ 0x8f81, 0x8f82, PDF_CMAP_RANGE, 6382 },
	{ 0x8f83, 0x8f8c, PDF_CMAP_TABLE, 7853 },
	{ 0x8f8d, 0x8f8f, PDF_CMAP_RANGE, 6387 },
	{ 0x8f90, 0x8f9f, PDF_CMAP_TABLE, 7863 },
	{ 0x8fa0, 0x8fa2, PDF_CMAP_RANGE, 19721 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 2460 },
	{ 0x8fa4, 0x8fa5, PDF_CMAP_RANGE, 19724 },
	{ 0x8fa6, 0x8fa7, PDF_CMAP_TABLE, 7879 },
	{ 0x8fa8, 0x8fa9, PDF_CMAP_RANGE, 1103 },
	{ 0x8faa, 0x8fb2, PDF_CMAP_TABLE, 7881 },
	{ 0x8fb3, 0x8fb5, PDF_CMAP_RANGE, 19729 },
	{ 0x8fb6, 0x8fb6, PDF_CMAP_SINGLE, 5949 },
	{ 0x8fb7, 0x8fb8, PDF_CMAP_RANGE, 19732 },
	{ 0x8fb9, 0x8fb9, PDF_CMAP_SINGLE, 1096 },
	{ 0x8fba, 0x8fbc, PDF_CMAP_RANGE, 19734 },
	{ 0x8fbd, 0x8fbe, PDF_CMAP_TABLE, 7890 },
	{ 0x8fbf, 0x8fc0, PDF_CMAP_RANGE, 19737 },
	{ 0x8fc1, 0x8fc8, PDF_CMAP_TABLE, 7892 },
	{ 0x8fc9, 0x8fcd, PDF_CMAP_RANGE, 19741 },
	{ 0x8fce, 0x8fd5, PDF_CMAP_TABLE, 7900 },
	{ 0x8fd6, 0x8fd7, PDF_CMAP_RANGE, 19748 },
	{ 0x8fd8, 0x8fdf, PDF_CMAP_TABLE, 7908 },
	{ 0x8fe0, 0x8fe1, PDF_CMAP_RANGE, 19751 },
	{ 0x8fe2, 0x8ff0, PDF_CMAP_TABLE, 7916 },
	{ 0x8ff1, 0x8ff2, PDF_CMAP_RANGE, 19757 },
	{ 0x8ff3, 0x8ff4, PDF_CMAP_TABLE, 7931 },
	{ 0x8ff5, 0x8ff6, PDF_CMAP_RANGE, 19759 },
	{ 0x8ff7, 0x8ff9, PDF_CMAP_TABLE, 7933 },
	{ 0x8ffa, 0x8ffc, PDF_CMAP_RANGE, 19761 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 4628 },
	{ 0x8ffe, 0x8fff, PDF_CMAP_RANGE, 19764 },
	{ 0x9000, 0x9006, PDF_CMAP_TABLE, 7936 },
	{ 0x9007, 0x9008, PDF_CMAP_RANGE, 19766 },
	{ 0x9009, 0x9023, PDF_CMAP_TABLE, 7943 },
	{ 0x9024, 0x9025, PDF_CMAP_RANGE, 19773 },
	{ 0x9026, 0x9026, PDF_CMAP_SINGLE, 5962 },
	{ 0x9027, 0x902c, PDF_CMAP_RANGE, 19775 },
	{ 0x902d, 0x902f, PDF_CMAP_TABLE, 7970 },
	{ 0x9030, 0x9031, PDF_CMAP_RANGE, 19781 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 8123 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 19783 },
	{ 0x9035, 0x9036, PDF_CMAP_RANGE, 5967 },
	{ 0x9037, 0x9038, PDF_CMAP_TABLE, 7973 },
	{ 0x9039, 0x903a, PDF_CMAP_RANGE, 19786 },
	{ 0x903b, 0x903e, PDF_CMAP_TABLE, 7975 },
	{ 0x903f, 0x9040, PDF_CMAP_RANGE, 19789 },
	{ 0x9041, 0x9044, PDF_CMAP_TABLE, 7979 },
	{ 0x9045, 0x9046, PDF_CMAP_RANGE, 19792 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 4315 },
	{ 0x9048, 0x904a, PDF_CMAP_RANGE, 19794 },
	{ 0x904b, 0x9050, PDF_CMAP_TABLE, 7983 },
	{ 0x9051, 0x9052, PDF_CMAP_RANGE, 5972 },
	{ 0x9053, 0x9058, PDF_CMAP_TABLE, 7989 },
	{ 0x9059, 0x905a, PDF_CMAP_RANGE, 19799 },
	{ 0x905b, 0x9065, PDF_CMAP_TABLE, 7995 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 19805 },
	{ 0x9068, 0x9069, PDF_CMAP_TABLE, 8006 },
	{ 0x906a, 0x906c, PDF_CMAP_RANGE, 19807 },
	{ 0x906d, 0x906e, PDF_CMAP_TABLE, 8008 },
	{ 0x906f, 0x9071, PDF_CMAP_RANGE, 19810 },
	{ 0x9072, 0x9084, PDF_CMAP_TABLE, 8010 },
	{ 0x9085, 0x9086, PDF_CMAP_RANGE, 19818 },
	{ 0x9087, 0x908b, PDF_CMAP_TABLE, 8029 },
	{ 0x908c, 0x908e, PDF_CMAP_RANGE, 19821 },
	{ 0x908f, 0x909d, PDF_CMAP_TABLE, 8034 },
	{ 0x909e, 0x90a0, PDF_CMAP_RANGE, 19830 },
	{ 0x90a1, 0x90a3, PDF_CMAP_TABLE, 8049 },
	{ 0x90a4, 0x90a5, PDF_CMAP_RANGE, 19833 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 1017 },
	{ 0x90a7, 0x90a9, PDF_CMAP_RANGE, 19835 },
	{ 0x90aa, 0x90bb, PDF_CMAP_TABLE, 8052 },
	{ 0x90bc, 0x90bd, PDF_CMAP_RANGE, 19842 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 4981 },
	{ 0x90bf, 0x90c0, PDF_CMAP_RANGE, 19844 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 4313 },
	{ 0x90c2, 0x90c3, PDF_CMAP_RANGE, 19846 },
	{ 0x90c4, 0x90c7, PDF_CMAP_TABLE, 8070 },
	{ 0x90c8, 0x90c9, PDF_CMAP_RANGE, 19849 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2189 },
	{ 0x90cb, 0x90cd, PDF_CMAP_RANGE, 19851 },
	{ 0x90ce, 0x90d3, PDF_CMAP_TABLE, 8074 },
	{ 0x90d4, 0x90d6, PDF_CMAP_RANGE, 19855 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 4989 },
	{ 0x90d8, 0x90da, PDF_CMAP_RANGE, 19858 },
	{ 0x90db, 0x90e2, PDF_CMAP_TABLE, 8080 },
	{ 0x90e3, 0x90e5, PDF_CMAP_RANGE, 19863 },
	{ 0x90e6, 0x90e8, PDF_CMAP_TABLE, 8088 },
	{ 0x90e9, 0x90ea, PDF_CMAP_RANGE, 19866 },
	{ 0x90eb, 0x90ef, PDF_CMAP_TABLE, 8091 },
	{ 0x90f0, 0x90f3, PDF_CMAP_RANGE, 19870 },
	{ 0x90f4, 0x90f5, PDF_CMAP_TABLE, 8096 },
	{ 0x90f6, 0x90f7, PDF_CMAP_RANGE, 19874 },
	{ 0x90f8, 0x90f8, PDF_CMAP_SINGLE, 1415 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 19876 },
	{ 0x90fd, 0x90ff, PDF_CMAP_TABLE, 8098 },
	{ 0x9100, 0x9101, PDF_CMAP_RANGE, 19881 },
	{ 0x9102, 0x9106, PDF_CMAP_TABLE, 8101 },
	{ 0x9107, 0x9108, PDF_CMAP_RANGE, 19885 },
	{ 0x9109, 0x9109, PDF_CMAP_SINGLE, 8647 },
	{ 0x910a, 0x9111, PDF_CMAP_RANGE, 19887 },
	{ 0x9112, 0x9116, PDF_CMAP_TABLE, 8106 },
	{ 0x9117, 0x9118, PDF_CMAP_RANGE, 19897 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 1074 },
	{ 0x911a, 0x911d, PDF_CMAP_RANGE, 19899 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 4996 },
	{ 0x911f, 0x9121, PDF_CMAP_RANGE, 19903 },
	{ 0x9122, 0x9123, PDF_CMAP_TABLE, 8111 },
	{ 0x9124, 0x9126, PDF_CMAP_RANGE, 19906 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 7867 },
	{ 0x9128, 0x912c, PDF_CMAP_RANGE, 19909 },
	{ 0x912d, 0x9136, PDF_CMAP_TABLE, 8113 },
	{ 0x9137, 0x9138, PDF_CMAP_RANGE, 19917 },
	{ 0x9139, 0x913a, PDF_CMAP_TABLE, 8123 },
	{ 0x913b, 0x9142, PDF_CMAP_RANGE, 19919 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 5001 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 19927 },
	{ 0x9146, 0x914d, PDF_CMAP_TABLE, 8125 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7418 },
	{ 0x9150, 0x9152, PDF_CMAP_TABLE, 8133 },
	{ 0x9153, 0x9156, PDF_CMAP_RANGE, 19931 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 4026 },
	{ 0x9158, 0x9159, PDF_CMAP_RANGE, 19935 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 1648 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 19937 },
	{ 0x915d, 0x915e, PDF_CMAP_TABLE, 8136 },
	{ 0x915f, 0x9160, PDF_CMAP_RANGE, 19939 },
	{ 0x9161, 0x9165, PDF_CMAP_TABLE, 8138 },
	{ 0x9166, 0x9168, PDF_CMAP_RANGE, 19941 },
	{ 0x9169, 0x9179, PDF_CMAP_TABLE, 8143 },
	{ 0x917a, 0x917c, PDF_CMAP_RANGE, 19947 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 7426 },
	{ 0x917f, 0x917f, PDF_CMAP_SINGLE, 2899 },
	{ 0x9180, 0x9184, PDF_CMAP_RANGE, 19950 },
	{ 0x9185, 0x918d, PDF_CMAP_TABLE, 8160 },
	{ 0x918e, 0x918f, PDF_CMAP_RANGE, 19958 },
	{ 0x9190, 0x9192, PDF_CMAP_TABLE, 8169 },
	{ 0x9193, 0x9195, PDF_CMAP_RANGE, 19960 },
	{ 0x9196, 0x9196, PDF_CMAP_SINGLE, 8789 },
	{ 0x9197, 0x9199, PDF_CMAP_RANGE, 19963 },
	{ 0x919a, 0x919c, PDF_CMAP_TABLE, 8172 },
	{ 0x919d, 0x91a1, PDF_CMAP_RANGE, 19966 },
	{ 0x91a2, 0x91a3, PDF_CMAP_RANGE, 7436 },
	{ 0x91a4, 0x91a9, PDF_CMAP_RANGE, 19971 },
	{ 0x91aa, 0x91ac, PDF_CMAP_TABLE, 8175 },
	{ 0x91ad, 0x91af, PDF_CMAP_RANGE, 7439 },
	{ 0x91b0, 0x91b3, PDF_CMAP_RANGE, 19977 },
	{ 0x91b4, 0x91b5, PDF_CMAP_TABLE, 8178 },
	{ 0x91b6, 0x91b9, PDF_CMAP_RANGE, 19981 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 7444 },
	{ 0x91bb, 0x91bf, PDF_CMAP_RANGE, 19985 },
	{ 0x91c0, 0x91d1, PDF_CMAP_TABLE, 8180 },
	{ 0x91d2, 0x91d4, PDF_CMAP_RANGE, 9459 },
	{ 0x91d5, 0x91d9, PDF_CMAP_TABLE, 8198 },
	{ 0x91da, 0x91db, PDF_CMAP_RANGE, 19996 },
	{ 0x91dc, 0x91dd, PDF_CMAP_TABLE, 8203 },
	{ 0x91de, 0x91e2, PDF_CMAP_RANGE, 19998 },
	{ 0x91e3, 0x91e4, PDF_CMAP_TABLE, 8205 },
	{ 0x91e5, 0x91e6, PDF_CMAP_RANGE, 20003 },
	{ 0x91e7, 0x91e9, PDF_CMAP_TABLE, 8207 },
	{ 0x91ea, 0x91f4, PDF_CMAP_RANGE, 20006 },
	{ 0x91f5, 0x91fa, PDF_CMAP_TABLE, 8210 },
	{ 0x91fb, 0x91ff, PDF_CMAP_RANGE, 20019 },
	{ 0x9200, 0x9201, PDF_CMAP_TABLE, 8216 },
	{ 0x9202, 0x9203, PDF_CMAP_RANGE, 20024 },
	{ 0x9204, 0x9204, PDF_CMAP_SINGLE, 9478 },
	{ 0x9205, 0x9207, PDF_CMAP_RANGE, 20026 },
	{ 0x9208, 0x9209, PDF_CMAP_TABLE, 8218 },
	{ 0x920a, 0x920c, PDF_CMAP_RANGE, 20029 },
	{ 0x920d, 0x9211, PDF_CMAP_TABLE, 8220 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 20033 },
	{ 0x9214, 0x9215, PDF_CMAP_TABLE, 8225 },
	{ 0x9216, 0x921d, PDF_CMAP_RANGE, 20035 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 8149 },
	{ 0x921f, 0x9222, PDF_CMAP_RANGE, 20043 },
	{ 0x9223, 0x9227, PDF_CMAP_TABLE, 8227 },
	{ 0x9228, 0x922d, PDF_CMAP_RANGE, 20048 },
	{ 0x922e, 0x9230, PDF_CMAP_TABLE, 8232 },
	{ 0x9231, 0x9232, PDF_CMAP_RANGE, 20055 },
	{ 0x9233, 0x9234, PDF_CMAP_TABLE, 8235 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 20057 },
	{ 0x9237, 0x923a, PDF_CMAP_TABLE, 8237 },
	{ 0x923b, 0x923c, PDF_CMAP_RANGE, 20059 },
	{ 0x923d, 0x9240, PDF_CMAP_TABLE, 8241 },
	{ 0x9241, 0x9244, PDF_CMAP_RANGE, 20061 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 9473 },
	{ 0x9246, 0x9247, PDF_CMAP_RANGE, 20065 },
	{ 0x9248, 0x9249, PDF_CMAP_TABLE, 8245 },
	{ 0x924a, 0x924c, PDF_CMAP_RANGE, 20067 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 9496 },
	{ 0x924e, 0x9250, PDF_CMAP_RANGE, 20070 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 7766 },
	{ 0x9252, 0x9254, PDF_CMAP_RANGE, 20073 },
	{ 0x9255, 0x9257, PDF_CMAP_TABLE, 8247 },
	{ 0x9258, 0x9259, PDF_CMAP_RANGE, 20077 },
	{ 0x925a, 0x925b, PDF_CMAP_TABLE, 8250 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 20079 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 9488 },
	{ 0x925f, 0x9261, PDF_CMAP_RANGE, 20081 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 7765 },
	{ 0x9263, 0x9265, PDF_CMAP_RANGE, 20084 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 9482 },
	{ 0x9267, 0x926b, PDF_CMAP_RANGE, 20087 },
	{ 0x926c, 0x926d, PDF_CMAP_RANGE, 9489 },
	{ 0x926e, 0x9273, PDF_CMAP_RANGE, 20092 },
	{ 0x9274, 0x9274, PDF_CMAP_SINGLE, 2155 },
	{ 0x9275, 0x9277, PDF_CMAP_RANGE, 20098 },
	{ 0x9278, 0x927b, PDF_CMAP_TABLE, 8252 },
	{ 0x927c, 0x927e, PDF_CMAP_RANGE, 20102 },
	{ 0x927f, 0x9280, PDF_CMAP_TABLE, 8256 },
	{ 0x9281, 0x9282, PDF_CMAP_RANGE, 20105 },
	{ 0x9283, 0x9285, PDF_CMAP_TABLE, 8258 },
	{ 0x9286, 0x928d, PDF_CMAP_RANGE, 20108 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 7552 },
	{ 0x928f, 0x9290, PDF_CMAP_RANGE, 20116 },
	{ 0x9291, 0x9293, PDF_CMAP_TABLE, 8261 },
	{ 0x9294, 0x9295, PDF_CMAP_RANGE, 20119 },
	{ 0x9296, 0x929c, PDF_CMAP_TABLE, 8264 },
	{ 0x929d, 0x929f, PDF_CMAP_RANGE, 20124 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 9501 },
	{ 0x92a1, 0x92a2, PDF_CMAP_RANGE, 20127 },
	{ 0x92a3, 0x92ae, PDF_CMAP_TABLE, 8271 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 20132 },
	{ 0x92b1, 0x92b1, PDF_CMAP_SINGLE, 9509 },
	{ 0x92b2, 0x92b6, PDF_CMAP_RANGE, 20134 },
	{ 0x92b7, 0x92bc, PDF_CMAP_TABLE, 8283 },
	{ 0x92bd, 0x92c0, PDF_CMAP_RANGE, 20141 },
	{ 0x92c1, 0x92c8, PDF_CMAP_TABLE, 8289 },
	{ 0x92c9, 0x92cb, PDF_CMAP_RANGE, 20148 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 9513 },
	{ 0x92cd, 0x92ce, PDF_CMAP_RANGE, 20151 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 9505 },
	{ 0x92d0, 0x92d1, PDF_CMAP_RANGE, 20153 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 7935 },
	{ 0x92d3, 0x92dc, PDF_CMAP_RANGE, 20155 },
	{ 0x92dd, 0x92df, PDF_CMAP_TABLE, 8297 },
	{ 0x92e0, 0x92e2, PDF_CMAP_RANGE, 20166 },
	{ 0x92e3, 0x92ea, PDF_CMAP_TABLE, 8300 },
	{ 0x92eb, 0x92ec, PDF_CMAP_RANGE, 20171 },
	{ 0x92ed, 0x92f1, PDF_CMAP_TABLE, 8308 },
	{ 0x92f2, 0x92f5, PDF_CMAP_RANGE, 20173 },
	{ 0x92f6, 0x92f8, PDF_CMAP_TABLE, 8313 },
	{ 0x92f9, 0x92fb, PDF_CMAP_RANGE, 20178 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 7960 },
	{ 0x92fd, 0x9300, PDF_CMAP_RANGE, 20181 },
	{ 0x9301, 0x9301, PDF_CMAP_SINGLE, 9547 },
	{ 0x9302, 0x9305, PDF_CMAP_RANGE, 20185 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 9543 },
	{ 0x9307, 0x9308, PDF_CMAP_RANGE, 9551 },
	{ 0x9309, 0x930f, PDF_CMAP_RANGE, 20189 },
	{ 0x9310, 0x9312, PDF_CMAP_TABLE, 8316 },
	{ 0x9313, 0x9314, PDF_CMAP_RANGE, 20197 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 9548 },
	{ 0x9316, 0x9317, PDF_CMAP_RANGE, 20199 },
	{ 0x9318, 0x931b, PDF_CMAP_TABLE, 8319 },
	{ 0x931c, 0x931e, PDF_CMAP_RANGE, 20201 },
	{ 0x931f, 0x9322, PDF_CMAP_TABLE, 8323 },
	{ 0x9323, 0x9325, PDF_CMAP_RANGE, 20205 },
	{ 0x9326, 0x9328, PDF_CMAP_TABLE, 8327 },
	{ 0x9329, 0x932a, PDF_CMAP_RANGE, 20209 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 8619 },
	{ 0x932c, 0x932d, PDF_CMAP_RANGE, 20211 },
	{ 0x932e, 0x932f, PDF_CMAP_TABLE, 8330 },
	{ 0x9330, 0x9331, PDF_CMAP_RANGE, 20213 },
	{ 0x9332, 0x9333, PDF_CMAP_TABLE, 8332 },
	{ 0x9334, 0x9335, PDF_CMAP_RANGE, 20215 },
	{ 0x9336, 0x9338, PDF_CMAP_TABLE, 8334 },
	{ 0x9339, 0x933d, PDF_CMAP_RANGE, 20218 },
	{ 0x933e, 0x9343, PDF_CMAP_TABLE, 8337 },
	{ 0x9344, 0x9345, PDF_CMAP_RANGE, 20225 },
	{ 0x9346, 0x9347, PDF_CMAP_TABLE, 8343 },
	{ 0x9348, 0x934a, PDF_CMAP_RANGE, 20227 },
	{ 0x934b, 0x934d, PDF_CMAP_TABLE, 8345 },
	{ 0x934e, 0x9353, PDF_CMAP_RANGE, 20231 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 9558 },
	{ 0x9355, 0x9357, PDF_CMAP_RANGE, 20237 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 8808 },
	{ 0x9359, 0x935a, PDF_CMAP_RANGE, 20240 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 7894 },
	{ 0x935c, 0x9363, PDF_CMAP_RANGE, 20242 },
	{ 0x9364, 0x9365, PDF_CMAP_TABLE, 8348 },
	{ 0x9366, 0x9368, PDF_CMAP_RANGE, 20250 },
	{ 0x9369, 0x936c, PDF_CMAP_TABLE, 8350 },
	{ 0x936d, 0x936f, PDF_CMAP_RANGE, 20254 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 9561 },
	{ 0x9371, 0x9374, PDF_CMAP_RANGE, 20257 },
	{ 0x9375, 0x9376, PDF_CMAP_TABLE, 8354 },
	{ 0x9377, 0x9379, PDF_CMAP_RANGE, 20261 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 8829 },
	{ 0x937b, 0x937d, PDF_CMAP_RANGE, 20264 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 9598 },
	{ 0x937f, 0x9381, PDF_CMAP_RANGE, 20267 },
	{ 0x9382, 0x9384, PDF_CMAP_TABLE, 8356 },
	{ 0x9385, 0x9386, PDF_CMAP_RANGE, 20271 },
	{ 0x9387, 0x9387, PDF_CMAP_SINGLE, 9566 },
	{ 0x9388, 0x9389, PDF_CMAP_RANGE, 20273 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 7732 },
	{ 0x938b, 0x938e, PDF_CMAP_RANGE, 20275 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 7558 },
	{ 0x9390, 0x9395, PDF_CMAP_RANGE, 20279 },
	{ 0x9396, 0x9398, PDF_CMAP_TABLE, 8359 },
	{ 0x9399, 0x93a1, PDF_CMAP_RANGE, 20286 },
	{ 0x93a2, 0x93a3, PDF_CMAP_TABLE, 8362 },
	{ 0x93a4, 0x93a5, PDF_CMAP_RANGE, 20295 },
	{ 0x93a6, 0x93b0, PDF_CMAP_TABLE, 8364 },
	{ 0x93b1, 0x93b2, PDF_CMAP_RANGE, 20301 },
	{ 0x93b3, 0x93b5, PDF_CMAP_TABLE, 8375 },
	{ 0x93b6, 0x93b7, PDF_CMAP_RANGE, 20304 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 9569 },
	{ 0x93b9, 0x93be, PDF_CMAP_RANGE, 20306 },
	{ 0x93bf, 0x93bf, PDF_CMAP_SINGLE, 9570 },
	{ 0x93c0, 0x93c2, PDF_CMAP_RANGE, 20312 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 9580 },
	{ 0x93c4, 0x93c6, PDF_CMAP_RANGE, 20315 },
	{ 0x93c7, 0x93cd, PDF_CMAP_TABLE, 8378 },
	{ 0x93ce, 0x93d0, PDF_CMAP_RANGE, 20320 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 9582 },
	{ 0x93d2, 0x93d5, PDF_CMAP_RANGE, 20323 },
	{ 0x93d6, 0x93d8, PDF_CMAP_TABLE, 8385 },
	{ 0x93d9, 0x93db, PDF_CMAP_RANGE, 20327 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 9576 },
	{ 0x93de, 0x93e4, PDF_CMAP_TABLE, 8388 },
	{ 0x93e5, 0x93e7, PDF_CMAP_RANGE, 20332 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 9757 },
	{ 0x93e9, 0x93f4, PDF_CMAP_RANGE, 20335 },
	{ 0x93f5, 0x93f9, PDF_CMAP_TABLE, 8395 },
	{ 0x93fa, 0x9402, PDF_CMAP_RANGE, 20349 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 9506 },
	{ 0x9404, 0x940a, PDF_CMAP_RANGE, 20358 },
	{ 0x940b, 0x940b, PDF_CMAP_SINGLE, 9523 },
	{ 0x940c, 0x940f, PDF_CMAP_RANGE, 20365 },
	{ 0x9410, 0x9414, PDF_CMAP_TABLE, 8400 },
	{ 0x9415, 0x9417, PDF_CMAP_RANGE, 20370 },
	{ 0x9418, 0x9419, PDF_CMAP_TABLE, 8405 },
	{ 0x941a, 0x941c, PDF_CMAP_RANGE, 20373 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 9584 },
	{ 0x941e, 0x941f, PDF_CMAP_RANGE, 20376 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 9589 },
	{ 0x9421, 0x9425, PDF_CMAP_RANGE, 20378 },
	{ 0x9426, 0x9427, PDF_CMAP_RANGE, 9537 },
	{ 0x9428, 0x9428, PDF_CMAP_SINGLE, 9565 },
	{ 0x9429, 0x942d, PDF_CMAP_RANGE, 20383 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 8218 },
	{ 0x942f, 0x9431, PDF_CMAP_RANGE, 20388 },
	{ 0x9432, 0x9435, PDF_CMAP_TABLE, 8407 },
	{ 0x9436, 0x9437, PDF_CMAP_RANGE, 20392 },
	{ 0x9438, 0x943a, PDF_CMAP_TABLE, 8411 },
	{ 0x943b, 0x943d, PDF_CMAP_RANGE, 20395 },
	{ 0x943e, 0x943f, PDF_CMAP_TABLE, 8414 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 20398 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8867 },
	{ 0x9445, 0x9449, PDF_CMAP_RANGE, 20402 },
	{ 0x944a, 0x944c, PDF_CMAP_TABLE, 8416 },
	{ 0x944d, 0x9451, PDF_CMAP_RANGE, 20408 },
	{ 0x9452, 0x9454, PDF_CMAP_TABLE, 8419 },
	{ 0x9455, 0x945f, PDF_CMAP_RANGE, 20414 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 9492 },
	{ 0x9461, 0x9462, PDF_CMAP_RANGE, 20425 },
	{ 0x9463, 0x9465, PDF_CMAP_TABLE, 8422 },
	{ 0x9466, 0x946a, PDF_CMAP_RANGE, 20428 },
	{ 0x946b, 0x946d, PDF_CMAP_TABLE, 8425 },
	{ 0x946e, 0x946f, PDF_CMAP_RANGE, 20434 },
	{ 0x9470, 0x9472, PDF_CMAP_TABLE, 8428 },
	{ 0x9473, 0x9476, PDF_CMAP_RANGE, 20437 },
	{ 0x9477, 0x9479, PDF_CMAP_TABLE, 8431 },
	{ 0x947a, 0x947b, PDF_CMAP_RANGE, 20442 },
	{ 0x947c, 0x947f, PDF_CMAP_TABLE, 8434 },
	{ 0x9480, 0x9484, PDF_CMAP_RANGE, 20444 },
	{ 0x9485, 0x9487, PDF_CMAP_RANGE, 6798 },
	{ 0x9488, 0x948b, PDF_CMAP_TABLE, 8438 },
	{ 0x948c, 0x948d, PDF_CMAP_RANGE, 6803 },
	{ 0x948e, 0x948e, PDF_CMAP_SINGLE, 3106 },
	{ 0x948f, 0x9490, PDF_CMAP_RANGE, 6805 },
	{ 0x9491, 0x9499, PDF_CMAP_TABLE, 8442 },
	{ 0x949a, 0x949c, PDF_CMAP_RANGE, 6810 },
	{ 0x949d, 0x94a2, PDF_CMAP_TABLE, 8451 },
	{ 0x94a3, 0x94a4, PDF_CMAP_RANGE, 6813 },
	{ 0x94a5, 0x94ae, PDF_CMAP_TABLE, 8457 },
	{ 0x94af, 0x94b0, PDF_CMAP_RANGE, 6819 },
	{ 0x94b1, 0x94b5, PDF_CMAP_TABLE, 8467 },
	{ 0x94b6, 0x94ba, PDF_CMAP_RANGE, 6823 },
	{ 0x94bb, 0x94bb, PDF_CMAP_SINGLE, 4679 },
	{ 0x94bc, 0x94bd, PDF_CMAP_RANGE, 6828 },
	{ 0x94be, 0x94c7, PDF_CMAP_TABLE, 8472 },
	{ 0x94c8, 0x94ce, PDF_CMAP_RANGE, 6832 },
	{ 0x94cf, 0x94cf, PDF_CMAP_SINGLE, 20453 },
	{ 0x94d0, 0x94d2, PDF_CMAP_RANGE, 6839 },
	{ 0x94d3, 0x94d4, PDF_CMAP_RANGE, 20454 },
	{ 0x94d5, 0x94d7, PDF_CMAP_RANGE, 6842 },
	{ 0x94d8, 0x94dd, PDF_CMAP_TABLE, 8482 },
	{ 0x94de, 0x94e0, PDF_CMAP_RANGE, 6848 },
	{ 0x94e1, 0x94e3, PDF_CMAP_TABLE, 8488 },
	{ 0x94e4, 0x94e5, PDF_CMAP_RANGE, 6852 },
	{ 0x94e6, 0x94e6, PDF_CMAP_SINGLE, 20457 },
	{ 0x94e7, 0x94e8, PDF_CMAP_RANGE, 6854 },
	{ 0x94e9, 0x94ed, PDF_CMAP_TABLE, 8491 },
	{ 0x94ee, 0x94ef, PDF_CMAP_RANGE, 6859 },
	{ 0x94f0, 0x94f2, PDF_CMAP_TABLE, 8496 },
	{ 0x94f3, 0x94f5, PDF_CMAP_RANGE, 6861 },
	{ 0x94f6, 0x94fb, PDF_CMAP_TABLE, 8499 },
	{ 0x94fc, 0x94fd, PDF_CMAP_RANGE, 6866 },
	{ 0x94fe, 0x9505, PDF_CMAP_TABLE, 8505 },
	{ 0x9506, 0x9507, PDF_CMAP_RANGE, 6871 },
	{ 0x9508, 0x9508, PDF_CMAP_SINGLE, 4013 },
	{ 0x9509, 0x950a, PDF_CMAP_RANGE, 6873 },
	{ 0x950b, 0x950c, PDF_CMAP_TABLE, 8513 },
	{ 0x950d, 0x950f, PDF_CMAP_RANGE, 6875 },
	{ 0x9510, 0x9511, PDF_CMAP_TABLE, 8515 },
	{ 0x9512, 0x9516, PDF_CMAP_RANGE, 6878 },
	{ 0x9517, 0x951c, PDF_CMAP_TABLE, 8517 },
	{ 0x951d, 0x951f, PDF_CMAP_RANGE, 6885 },
	{ 0x9520, 0x9529, PDF_CMAP_TABLE, 8523 },
	{ 0x952a, 0x952b, PDF_CMAP_RANGE, 6889 },
	{ 0x952c, 0x9530, PDF_CMAP_TABLE, 8533 },
	{ 0x9531, 0x9532, PDF_CMAP_RANGE, 6893 },
	{ 0x9533, 0x9535, PDF_CMAP_TABLE, 8538 },
	{ 0x9536, 0x9538, PDF_CMAP_RANGE, 6896 },
	{ 0x9539, 0x953d, PDF_CMAP_TABLE, 8541 },
	{ 0x953e, 0x953f, PDF_CMAP_RANGE, 6900 },
	{ 0x9540, 0x9543, PDF_CMAP_TABLE, 8546 },
	{ 0x9544, 0x9546, PDF_CMAP_RANGE, 6904 },
	{ 0x9547, 0x954d, PDF_CMAP_TABLE, 8550 },
	{ 0x954e, 0x954f, PDF_CMAP_RANGE, 6909 },
	{ 0x9550, 0x9551, PDF_CMAP_TABLE, 8557 },
	{ 0x9552, 0x9554, PDF_CMAP_RANGE, 6911 },
	{ 0x9555, 0x9555, PDF_CMAP_SINGLE, 20467 },
	{ 0x9556, 0x9559, PDF_CMAP_RANGE, 6914 },
	{ 0x955a, 0x955d, PDF_CMAP_TABLE, 8559 },
	{ 0x955e, 0x955f, PDF_CMAP_RANGE, 6919 },
	{ 0x9560, 0x9560, PDF_CMAP_SINGLE, 20469 },
	{ 0x9561, 0x9562, PDF_CMAP_RANGE, 6922 },
	{ 0x9563, 0x9563, PDF_CMAP_SINGLE, 2582 },
	{ 0x9564, 0x956c, PDF_CMAP_RANGE, 6924 },
	{ 0x956d, 0x9570, PDF_CMAP_TABLE, 8563 },
	{ 0x9571, 0x9573, PDF_CMAP_RANGE, 6934 },
	{ 0x9574, 0x9575, PDF_CMAP_RANGE, 20471 },
	{ 0x9576, 0x9577, PDF_CMAP_TABLE, 8567 },
	{ 0x9578, 0x957e, PDF_CMAP_RANGE, 20473 },
	{ 0x957f, 0x9583, PDF_CMAP_TABLE, 8569 },
	{ 0x9584, 0x9585, PDF_CMAP_RANGE, 20481 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 9152 },
	{ 0x9587, 0x9588, PDF_CMAP_RANGE, 20483 },
	{ 0x9589, 0x9594, PDF_CMAP_TABLE, 8574 },
	{ 0x9595, 0x9597, PDF_CMAP_RANGE, 20489 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 8809 },
	{ 0x9599, 0x95a0, PDF_CMAP_RANGE, 20492 },
	{ 0x95a1, 0x95a5, PDF_CMAP_TABLE, 8586 },
	{ 0x95a6, 0x95a7, PDF_CMAP_RANGE, 20501 },
	{ 0x95a8, 0x95ad, PDF_CMAP_TABLE, 8591 },
	{ 0x95ae, 0x95b1, PDF_CMAP_RANGE, 20504 },
	{ 0x95b2, 0x95b2, PDF_CMAP_SINGLE, 8783 },
	{ 0x95b3, 0x95b5, PDF_CMAP_RANGE, 20508 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 9163 },
	{ 0x95b7, 0x95b8, PDF_CMAP_RANGE, 20511 },
	{ 0x95b9, 0x95bf, PDF_CMAP_TABLE, 8597 },
	{ 0x95c0, 0x95c2, PDF_CMAP_RANGE, 20514 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 9168 },
	{ 0x95c4, 0x95c5, PDF_CMAP_RANGE, 20517 },
	{ 0x95c6, 0x95cc, PDF_CMAP_TABLE, 8604 },
	{ 0x95cd, 0x95cf, PDF_CMAP_RANGE, 20521 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 9171 },
	{ 0x95d1, 0x95d3, PDF_CMAP_RANGE, 20524 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 8611 },
	{ 0x95d7, 0x95db, PDF_CMAP_RANGE, 20527 },
	{ 0x95dc, 0x95de, PDF_CMAP_TABLE, 8614 },
	{ 0x95df, 0x95e0, PDF_CMAP_RANGE, 20533 },
	{ 0x95e1, 0x95e2, PDF_CMAP_TABLE, 8617 },
	{ 0x95e3, 0x95e4, PDF_CMAP_RANGE, 20535 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 9157 },
	{ 0x95e6, 0x95e7, PDF_CMAP_RANGE, 20537 },
	{ 0x95e8, 0x95f4, PDF_CMAP_TABLE, 8619 },
	{ 0x95f5, 0x95f6, PDF_CMAP_RANGE, 5769 },
	{ 0x95f7, 0x9602, PDF_CMAP_TABLE, 8632 },
	{ 0x9603, 0x9604, PDF_CMAP_RANGE, 5773 },
	{ 0x9605, 0x9609, PDF_CMAP_TABLE, 8644 },
	{ 0x960a, 0x960d, PDF_CMAP_RANGE, 5777 },
	{ 0x960e, 0x9614, PDF_CMAP_TABLE, 8649 },
	{ 0x9615, 0x9617, PDF_CMAP_RANGE, 5783 },
	{ 0x9618, 0x9618, PDF_CMAP_SINGLE, 20543 },
	{ 0x9619, 0x961a, PDF_CMAP_RANGE, 5786 },
	{ 0x961b, 0x9622, PDF_CMAP_TABLE, 8656 },
	{ 0x9623, 0x9629, PDF_CMAP_RANGE, 20547 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 4952 },
	{ 0x962b, 0x962d, PDF_CMAP_RANGE, 20554 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 3273 },
	{ 0x962f, 0x9630, PDF_CMAP_RANGE, 20557 },
	{ 0x9631, 0x9636, PDF_CMAP_TABLE, 8664 },
	{ 0x9637, 0x963a, PDF_CMAP_RANGE, 20559 },
	{ 0x963b, 0x964d, PDF_CMAP_TABLE, 8670 },
	{ 0x964e, 0x964f, PDF_CMAP_RANGE, 20567 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 3915 },
	{ 0x9651, 0x9653, PDF_CMAP_RANGE, 20569 },
	{ 0x9654, 0x9655, PDF_CMAP_TABLE, 8689 },
	{ 0x9656, 0x9657, PDF_CMAP_RANGE, 20572 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 8986 },
	{ 0x9659, 0x965a, PDF_CMAP_RANGE, 20574 },
	{ 0x965b, 0x9664, PDF_CMAP_TABLE, 8691 },
	{ 0x9665, 0x9666, PDF_CMAP_RANGE, 20579 },
	{ 0x9667, 0x966c, PDF_CMAP_TABLE, 8701 },
	{ 0x966d, 0x966f, PDF_CMAP_RANGE, 20582 },
	{ 0x9670, 0x9678, PDF_CMAP_TABLE, 8707 },
	{ 0x9679, 0x967c, PDF_CMAP_RANGE, 20586 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 8705 },
	{ 0x967e, 0x9684, PDF_CMAP_RANGE, 20590 },
	{ 0x9685, 0x9690, PDF_CMAP_TABLE, 8716 },
	{ 0x9691, 0x9693, PDF_CMAP_RANGE, 20600 },
	{ 0x9694, 0x969c, PDF_CMAP_TABLE, 8728 },
	{ 0x969d, 0x96a6, PDF_CMAP_RANGE, 20605 },
	{ 0x96a7, 0x96aa, PDF_CMAP_TABLE, 8737 },
	{ 0x96ab, 0x96af, PDF_CMAP_RANGE, 20616 },
	{ 0x96b0, 0x96bb, PDF_CMAP_TABLE, 8741 },
	{ 0x96bc, 0x96bd, PDF_CMAP_RANGE, 7546 },
	{ 0x96be, 0x96c1, PDF_CMAP_TABLE, 8753 },
	{ 0x96c2, 0x96c3, PDF_CMAP_RANGE, 20626 },
	{ 0x96c4, 0x96c9, PDF_CMAP_TABLE, 8757 },
	{ 0x96ca, 0x96cb, PDF_CMAP_RANGE, 20629 },
	{ 0x96cc, 0x96cf, PDF_CMAP_TABLE, 8763 },
	{ 0x96d0, 0x96d1, PDF_CMAP_RANGE, 20631 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 7549 },
	{ 0x96d3, 0x96d4, PDF_CMAP_RANGE, 20633 },
	{ 0x96d5, 0x96d6, PDF_CMAP_TABLE, 8767 },
	{ 0x96d7, 0x96d8, PDF_CMAP_RANGE, 20635 },
	{ 0x96d9, 0x96dc, PDF_CMAP_TABLE, 8769 },
	{ 0x96dd, 0x96df, PDF_CMAP_RANGE, 20638 },
	{ 0x96e0, 0x96e3, PDF_CMAP_TABLE, 8773 },
	{ 0x96e4, 0x96e7, PDF_CMAP_RANGE, 20642 },
	{ 0x96e8, 0x96ea, PDF_CMAP_TABLE, 8777 },
	{ 0x96eb, 0x96ee, PDF_CMAP_RANGE, 20646 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 7523 },
	{ 0x96f0, 0x96f1, PDF_CMAP_RANGE, 20650 },
	{ 0x96f2, 0x96f3, PDF_CMAP_TABLE, 8780 },
	{ 0x96f4, 0x96f5, PDF_CMAP_RANGE, 20652 },
	{ 0x96f6, 0x96fb, PDF_CMAP_TABLE, 8782 },
	{ 0x96fc, 0x96fd, PDF_CMAP_RANGE, 20656 },
	{ 0x96fe, 0x9701, PDF_CMAP_TABLE, 8788 },
	{ 0x9702, 0x9703, PDF_CMAP_RANGE, 20659 },
	{ 0x9704, 0x9709, PDF_CMAP_TABLE, 8792 },
	{ 0x970a, 0x970c, PDF_CMAP_RANGE, 20662 },
	{ 0x970d, 0x970f, PDF_CMAP_TABLE, 8798 },
	{ 0x9710, 0x9712, PDF_CMAP_RANGE, 20665 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 2881 },
	{ 0x9714, 0x9715, PDF_CMAP_RANGE, 20668 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 2593 },
	{ 0x9717, 0x971b, PDF_CMAP_RANGE, 20670 },
	{ 0x971c, 0x971e, PDF_CMAP_TABLE, 8801 },
	{ 0x971f, 0x9726, PDF_CMAP_RANGE, 20676 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 8616 },
	{ 0x9728, 0x9729, PDF_CMAP_RANGE, 20684 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7529 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 20686 },
	{ 0x972d, 0x972d, PDF_CMAP_SINGLE, 7530 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 20688 },
	{ 0x9730, 0x9732, PDF_CMAP_TABLE, 8804 },
	{ 0x9733, 0x9737, PDF_CMAP_RANGE, 20691 },
	{ 0x9738, 0x9739, PDF_CMAP_TABLE, 8807 },
	{ 0x973a, 0x973c, PDF_CMAP_RANGE, 20696 },
	{ 0x973d, 0x973e, PDF_CMAP_TABLE, 8809 },
	{ 0x973f, 0x9741, PDF_CMAP_RANGE, 20699 },
	{ 0x9742, 0x9744, PDF_CMAP_TABLE, 8811 },
	{ 0x9745, 0x9747, PDF_CMAP_RANGE, 20703 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 8242 },
	{ 0x9749, 0x9751, PDF_CMAP_RANGE, 20706 },
	{ 0x9752, 0x9753, PDF_CMAP_TABLE, 8814 },
	{ 0x9754, 0x9755, PDF_CMAP_RANGE, 20715 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 2279 },
	{ 0x9757, 0x9758, PDF_CMAP_RANGE, 20717 },
	{ 0x9759, 0x975b, PDF_CMAP_TABLE, 8816 },
	{ 0x975c, 0x975d, PDF_CMAP_RANGE, 20719 },
	{ 0x975e, 0x9762, PDF_CMAP_TABLE, 8819 },
	{ 0x9763, 0x9764, PDF_CMAP_RANGE, 20722 },
	{ 0x9765, 0x9765, PDF_CMAP_SINGLE, 4738 },
	{ 0x9766, 0x9767, PDF_CMAP_RANGE, 20724 },
	{ 0x9768, 0x9769, PDF_CMAP_TABLE, 8824 },
	{ 0x976a, 0x9772, PDF_CMAP_RANGE, 20726 },
	{ 0x9773, 0x9776, PDF_CMAP_TABLE, 8826 },
	{ 0x9777, 0x977b, PDF_CMAP_RANGE, 20736 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7624 },
	{ 0x977d, 0x9784, PDF_CMAP_RANGE, 20741 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 7625 },
	{ 0x9786, 0x978a, PDF_CMAP_RANGE, 20749 },
	{ 0x978b, 0x9790, PDF_CMAP_TABLE, 8830 },
	{ 0x9791, 0x9792, PDF_CMAP_RANGE, 7626 },
	{ 0x9793, 0x9794, PDF_CMAP_TABLE, 8836 },
	{ 0x9795, 0x9797, PDF_CMAP_RANGE, 20758 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 3143 },
	{ 0x9799, 0x979f, PDF_CMAP_RANGE, 20761 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 2302 },
	{ 0x97a1, 0x97a2, PDF_CMAP_RANGE, 20768 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 7631 },
	{ 0x97a4, 0x97a5, PDF_CMAP_RANGE, 20770 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 9877 },
	{ 0x97a7, 0x97aa, PDF_CMAP_RANGE, 20772 },
	{ 0x97ab, 0x97af, PDF_CMAP_TABLE, 8838 },
	{ 0x97b0, 0x97b1, PDF_CMAP_RANGE, 20778 },
	{ 0x97b2, 0x97b4, PDF_CMAP_TABLE, 8843 },
	{ 0x97b5, 0x97bc, PDF_CMAP_RANGE, 20781 },
	{ 0x97bd, 0x97bd, PDF_CMAP_SINGLE, 9822 },
	{ 0x97be, 0x97c2, PDF_CMAP_RANGE, 20789 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 9821 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 20794 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 9874 },
	{ 0x97c7, 0x97c8, PDF_CMAP_RANGE, 20796 },
	{ 0x97c9, 0x97cc, PDF_CMAP_TABLE, 8846 },
	{ 0x97cd, 0x97d2, PDF_CMAP_RANGE, 20799 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 8000 },
	{ 0x97d4, 0x97d8, PDF_CMAP_RANGE, 20805 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 9317 },
	{ 0x97da, 0x97db, PDF_CMAP_RANGE, 20810 },
	{ 0x97dc, 0x97de, PDF_CMAP_TABLE, 8850 },
	{ 0x97df, 0x97e5, PDF_CMAP_RANGE, 20813 },
	{ 0x97e6, 0x97e9, PDF_CMAP_TABLE, 8853 },
	{ 0x97ea, 0x97ec, PDF_CMAP_RANGE, 6222 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 2289 },
	{ 0x97ee, 0x97f2, PDF_CMAP_RANGE, 20821 },
	{ 0x97f3, 0x97f6, PDF_CMAP_TABLE, 8857 },
	{ 0x97f7, 0x97fe, PDF_CMAP_RANGE, 20827 },
	{ 0x97ff, 0x980d, PDF_CMAP_TABLE, 8861 },
	{ 0x980e, 0x980f, PDF_CMAP_RANGE, 9670 },
	{ 0x9810, 0x9813, PDF_CMAP_TABLE, 8876 },
	{ 0x9814, 0x9816, PDF_CMAP_RANGE, 20840 },
	{ 0x9817, 0x9818, PDF_CMAP_TABLE, 8880 },
	{ 0x9819, 0x981b, PDF_CMAP_RANGE, 20843 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 9673 },
	{ 0x981d, 0x9820, PDF_CMAP_RANGE, 20846 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 9672 },
	{ 0x9822, 0x9823, PDF_CMAP_RANGE, 20850 },
	{ 0x9824, 0x9826, PDF_CMAP_TABLE, 8882 },
	{ 0x9827, 0x982c, PDF_CMAP_RANGE, 20853 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 8573 },
	{ 0x982e, 0x982f, PDF_CMAP_RANGE, 20859 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 8062 },
	{ 0x9831, 0x9836, PDF_CMAP_RANGE, 20861 },
	{ 0x9837, 0x9838, PDF_CMAP_TABLE, 8885 },
	{ 0x9839, 0x983a, PDF_CMAP_RANGE, 20867 },
	{ 0x983b, 0x983d, PDF_CMAP_TABLE, 8887 },
	{ 0x983e, 0x9845, PDF_CMAP_RANGE, 20870 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 8154 },
	{ 0x9847, 0x984b, PDF_CMAP_RANGE, 20878 },
	{ 0x984c, 0x984e, PDF_CMAP_TABLE, 8890 },
	{ 0x984f, 0x9852, PDF_CMAP_RANGE, 20883 },
	{ 0x9853, 0x9854, PDF_CMAP_TABLE, 8893 },
	{ 0x9855, 0x9857, PDF_CMAP_RANGE, 20887 },
	{ 0x9858, 0x985b, PDF_CMAP_TABLE, 8895 },
	{ 0x985c, 0x985d, PDF_CMAP_RANGE, 20891 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 8200 },
	{ 0x985f, 0x9861, PDF_CMAP_RANGE, 20893 },
	{ 0x9862, 0x9862, PDF_CMAP_SINGLE, 9680 },
	{ 0x9863, 0x9864, PDF_CMAP_RANGE, 20896 },
	{ 0x9865, 0x9867, PDF_CMAP_TABLE, 8899 },
	{ 0x9868, 0x986a, PDF_CMAP_RANGE, 20899 },
	{ 0x986b, 0x986c, PDF_CMAP_TABLE, 8902 },
	{ 0x986d, 0x986e, PDF_CMAP_RANGE, 20902 },
	{ 0x986f, 0x988b, PDF_CMAP_TABLE, 8904 },
	{ 0x988c, 0x988d, PDF_CMAP_RANGE, 7140 },
	{ 0x988e, 0x9899, PDF_CMAP_TABLE, 8933 },
	{ 0x989a, 0x989b, PDF_CMAP_RANGE, 7144 },
	{ 0x989c, 0x989d, PDF_CMAP_TABLE, 8945 },
	{ 0x989e, 0x989f, PDF_CMAP_RANGE, 7146 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 1472 },
	{ 0x98a1, 0x98a2, PDF_CMAP_RANGE, 7148 },
	{ 0x98a3, 0x98a4, PDF_CMAP_TABLE, 8947 },
	{ 0x98a5, 0x98a6, PDF_CMAP_RANGE, 7150 },
	{ 0x98a7, 0x98a8, PDF_CMAP_TABLE, 8949 },
	{ 0x98a9, 0x98ad, PDF_CMAP_RANGE, 20911 },
	{ 0x98ae, 0x98af, PDF_CMAP_RANGE, 9420 },
	{ 0x98b0, 0x98b3, PDF_CMAP_TABLE, 8951 },
	{ 0x98b4, 0x98b5, PDF_CMAP_RANGE, 20918 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 9422 },
	{ 0x98b7, 0x98bb, PDF_CMAP_RANGE, 20920 },
	{ 0x98bc, 0x98bc, PDF_CMAP_SINGLE, 9423 },
	{ 0x98bd, 0x98c3, PDF_CMAP_RANGE, 20925 },
	{ 0x98c4, 0x98c8, PDF_CMAP_TABLE, 8955 },
	{ 0x98c9, 0x98cd, PDF_CMAP_RANGE, 20934 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 1668 },
	{ 0x98cf, 0x98d0, PDF_CMAP_RANGE, 20939 },
	{ 0x98d1, 0x98d3, PDF_CMAP_RANGE, 6583 },
	{ 0x98d4, 0x98d5, PDF_CMAP_TABLE, 8960 },
	{ 0x98d6, 0x98d7, PDF_CMAP_RANGE, 20942 },
	{ 0x98d8, 0x98d8, PDF_CMAP_SINGLE, 3021 },
	{ 0x98d9, 0x98da, PDF_CMAP_RANGE, 6587 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 7924 },
	{ 0x98dc, 0x98dd, PDF_CMAP_RANGE, 20944 },
	{ 0x98de, 0x98e2, PDF_CMAP_TABLE, 8962 },
	{ 0x98e3, 0x98e6, PDF_CMAP_RANGE, 20947 },
	{ 0x98e7, 0x98e9, PDF_CMAP_TABLE, 8967 },
	{ 0x98ea, 0x98eb, PDF_CMAP_RANGE, 9119 },
	{ 0x98ec, 0x98ef, PDF_CMAP_TABLE, 8970 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 20953 },
	{ 0x98f2, 0x98f4, PDF_CMAP_TABLE, 8974 },
	{ 0x98f5, 0x98fb, PDF_CMAP_RANGE, 20956 },
	{ 0x98fc, 0x98ff, PDF_CMAP_TABLE, 8977 },
	{ 0x9900, 0x9902, PDF_CMAP_RANGE, 20964 },
	{ 0x9903, 0x9905, PDF_CMAP_TABLE, 8981 },
	{ 0x9906, 0x9908, PDF_CMAP_RANGE, 20968 },
	{ 0x9909, 0x990d, PDF_CMAP_TABLE, 8984 },
	{ 0x990e, 0x990f, PDF_CMAP_RANGE, 20972 },
	{ 0x9910, 0x9913, PDF_CMAP_TABLE, 8989 },
	{ 0x9914, 0x9917, PDF_CMAP_RANGE, 20974 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 8760 },
	{ 0x9919, 0x991a, PDF_CMAP_RANGE, 20978 },
	{ 0x991b, 0x991b, PDF_CMAP_SINGLE, 9125 },
	{ 0x991c, 0x991d, PDF_CMAP_RANGE, 20980 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 8090 },
	{ 0x991f, 0x9920, PDF_CMAP_RANGE, 20982 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 8643 },
	{ 0x9922, 0x9927, PDF_CMAP_RANGE, 20984 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 7982 },
	{ 0x9929, 0x992d, PDF_CMAP_RANGE, 20990 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7657 },
	{ 0x992f, 0x9932, PDF_CMAP_RANGE, 20995 },
	{ 0x9933, 0x9933, PDF_CMAP_SINGLE, 9116 },
	{ 0x9934, 0x9936, PDF_CMAP_RANGE, 20999 },
	{ 0x9937, 0x9937, PDF_CMAP_SINGLE, 9126 },
	{ 0x9938, 0x993b, PDF_CMAP_RANGE, 21002 },
	{ 0x993c, 0x993f, PDF_CMAP_TABLE, 8993 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 21007 },
	{ 0x9943, 0x9945, PDF_CMAP_TABLE, 8997 },
	{ 0x9946, 0x9947, PDF_CMAP_RANGE, 21011 },
	{ 0x9948, 0x994a, PDF_CMAP_RANGE, 9129 },
	{ 0x994b, 0x994c, PDF_CMAP_TABLE, 9000 },
	{ 0x994d, 0x9950, PDF_CMAP_RANGE, 21013 },
	{ 0x9951, 0x9957, PDF_CMAP_TABLE, 9002 },
	{ 0x9958, 0x995b, PDF_CMAP_RANGE, 21019 },
	{ 0x995c, 0x995e, PDF_CMAP_TABLE, 9009 },
	{ 0x995f, 0x9961, PDF_CMAP_RANGE, 21024 },
	{ 0x9962, 0x9966, PDF_CMAP_TABLE, 9012 },
	{ 0x9967, 0x996c, PDF_CMAP_RANGE, 5662 },
	{ 0x996d, 0x9977, PDF_CMAP_TABLE, 9017 },
	{ 0x9978, 0x9979, PDF_CMAP_RANGE, 21030 },
	{ 0x997a, 0x9981, PDF_CMAP_TABLE, 9028 },
	{ 0x9982, 0x9983, PDF_CMAP_RANGE, 21034 },
	{ 0x9984, 0x998f, PDF_CMAP_TABLE, 9036 },
	{ 0x9990, 0x9991, PDF_CMAP_RANGE, 5676 },
	{ 0x9992, 0x9992, PDF_CMAP_SINGLE, 2719 },
	{ 0x9993, 0x9995, PDF_CMAP_RANGE, 5678 },
	{ 0x9996, 0x9999, PDF_CMAP_TABLE, 9048 },
	{ 0x999a, 0x99a4, PDF_CMAP_RANGE, 21039 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 6955 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 21050 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5086 },
	{ 0x99a9, 0x99ab, PDF_CMAP_RANGE, 21052 },
	{ 0x99ac, 0x99ae, PDF_CMAP_TABLE, 9052 },
	{ 0x99af, 0x99b0, PDF_CMAP_RANGE, 21055 },
	{ 0x99b1, 0x99b4, PDF_CMAP_TABLE, 9055 },
	{ 0x99b5, 0x99c0, PDF_CMAP_RANGE, 21058 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 7767 },
	{ 0x99c2, 0x99cf, PDF_CMAP_RANGE, 21070 },
	{ 0x99d0, 0x99d5, PDF_CMAP_TABLE, 9059 },
	{ 0x99d6, 0x99d7, PDF_CMAP_RANGE, 21085 },
	{ 0x99d8, 0x99e2, PDF_CMAP_TABLE, 9065 },
	{ 0x99e3, 0x99ec, PDF_CMAP_RANGE, 21091 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 7999 },
	{ 0x99ee, 0x99f0, PDF_CMAP_RANGE, 21101 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 8295 },
	{ 0x99f2, 0x99fe, PDF_CMAP_RANGE, 21104 },
	{ 0x99ff, 0x9a01, PDF_CMAP_TABLE, 9076 },
	{ 0x9a02, 0x9a04, PDF_CMAP_RANGE, 21118 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 9231 },
	{ 0x9a06, 0x9a0c, PDF_CMAP_RANGE, 21121 },
	{ 0x9a0d, 0x9a0f, PDF_CMAP_TABLE, 9079 },
	{ 0x9a10, 0x9a15, PDF_CMAP_RANGE, 21128 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 9234 },
	{ 0x9a17, 0x9a18, PDF_CMAP_RANGE, 21134 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 8375 },
	{ 0x9a1a, 0x9a2a, PDF_CMAP_RANGE, 21136 },
	{ 0x9a2b, 0x9a30, PDF_CMAP_TABLE, 9082 },
	{ 0x9a31, 0x9a35, PDF_CMAP_RANGE, 21155 },
	{ 0x9a36, 0x9a38, PDF_CMAP_TABLE, 9088 },
	{ 0x9a39, 0x9a3d, PDF_CMAP_RANGE, 21160 },
	{ 0x9a3e, 0x9a42, PDF_CMAP_TABLE, 9091 },
	{ 0x9a43, 0x9a44, PDF_CMAP_RANGE, 9238 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 8433 },
	{ 0x9a46, 0x9a49, PDF_CMAP_RANGE, 21166 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 9226 },
	{ 0x9a4b, 0x9a4c, PDF_CMAP_RANGE, 21170 },
	{ 0x9a4d, 0x9a4f, PDF_CMAP_TABLE, 9096 },
	{ 0x9a50, 0x9a54, PDF_CMAP_RANGE, 21173 },
	{ 0x9a55, 0x9a57, PDF_CMAP_TABLE, 9099 },
	{ 0x9a58, 0x9a59, PDF_CMAP_RANGE, 21179 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_TABLE, 9102 },
	{ 0x9a5c, 0x9a5e, PDF_CMAP_RANGE, 21181 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 8860 },
	{ 0x9a60, 0x9a61, PDF_CMAP_RANGE, 21184 },
	{ 0x9a62, 0x9a65, PDF_CMAP_TABLE, 9104 },
	{ 0x9a66, 0x9a69, PDF_CMAP_RANGE, 21187 },
	{ 0x9a6a, 0x9a76, PDF_CMAP_TABLE, 9108 },
	{ 0x9a77, 0x9a78, PDF_CMAP_RANGE, 6075 },
	{ 0x9a79, 0x9a7f, PDF_CMAP_TABLE, 9121 },
	{ 0x9a80, 0x9a81, PDF_CMAP_RANGE, 6080 },
	{ 0x9a82, 0x9a8c, PDF_CMAP_TABLE, 9128 },
	{ 0x9a8d, 0x9a8e, PDF_CMAP_RANGE, 21195 },
	{ 0x9a8f, 0x9a91, PDF_CMAP_TABLE, 9139 },
	{ 0x9a92, 0x9a93, PDF_CMAP_RANGE, 6086 },
	{ 0x9a94, 0x9a95, PDF_CMAP_RANGE, 21197 },
	{ 0x9a96, 0x9a9a, PDF_CMAP_TABLE, 9142 },
	{ 0x9a9b, 0x9a9d, PDF_CMAP_RANGE, 6090 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 5941 },
	{ 0x9a9f, 0x9aa0, PDF_CMAP_RANGE, 6093 },
	{ 0x9aa1, 0x9aa1, PDF_CMAP_SINGLE, 2697 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_RANGE, 6095 },
	{ 0x9aa4, 0x9aa8, PDF_CMAP_TABLE, 9147 },
	{ 0x9aa9, 0x9aae, PDF_CMAP_RANGE, 21201 },
	{ 0x9aaf, 0x9ab1, PDF_CMAP_TABLE, 9152 },
	{ 0x9ab2, 0x9ab5, PDF_CMAP_RANGE, 21207 },
	{ 0x9ab6, 0x9abc, PDF_CMAP_TABLE, 9155 },
	{ 0x9abd, 0x9abf, PDF_CMAP_RANGE, 21213 },
	{ 0x9ac0, 0x9ac2, PDF_CMAP_TABLE, 9162 },
	{ 0x9ac3, 0x9ac4, PDF_CMAP_RANGE, 21216 },
	{ 0x9ac5, 0x9ac5, PDF_CMAP_SINGLE, 7643 },
	{ 0x9ac6, 0x9aca, PDF_CMAP_RANGE, 21218 },
	{ 0x9acb, 0x9acc, PDF_CMAP_RANGE, 7645 },
	{ 0x9acd, 0x9ace, PDF_CMAP_RANGE, 21223 },
	{ 0x9acf, 0x9ad8, PDF_CMAP_TABLE, 9165 },
	{ 0x9ad9, 0x9ade, PDF_CMAP_RANGE, 21227 },
	{ 0x9adf, 0x9ae1, PDF_CMAP_TABLE, 9175 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_RANGE, 21234 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7662 },
	{ 0x9ae7, 0x9aea, PDF_CMAP_RANGE, 21238 },
	{ 0x9aeb, 0x9aef, PDF_CMAP_TABLE, 9178 },
	{ 0x9af0, 0x9af8, PDF_CMAP_RANGE, 21243 },
	{ 0x9af9, 0x9afb, PDF_CMAP_TABLE, 9183 },
	{ 0x9afc, 0x9b02, PDF_CMAP_RANGE, 21253 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 4660 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 21260 },
	{ 0x9b06, 0x9b08, PDF_CMAP_TABLE, 9186 },
	{ 0x9b09, 0x9b0c, PDF_CMAP_RANGE, 21263 },
	{ 0x9b0d, 0x9b0f, PDF_CMAP_TABLE, 9189 },
	{ 0x9b10, 0x9b12, PDF_CMAP_RANGE, 21268 },
	{ 0x9b13, 0x9b13, PDF_CMAP_SINGLE, 7670 },
	{ 0x9b14, 0x9b19, PDF_CMAP_RANGE, 21271 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 9888 },
	{ 0x9b1b, 0x9b1e, PDF_CMAP_RANGE, 21277 },
	{ 0x9b1f, 0x9b1f, PDF_CMAP_SINGLE, 7671 },
	{ 0x9b20, 0x9b21, PDF_CMAP_RANGE, 21281 },
	{ 0x9b22, 0x9b29, PDF_CMAP_TABLE, 9192 },
	{ 0x9b2a, 0x9b2d, PDF_CMAP_RANGE, 21286 },
	{ 0x9b2e, 0x9b32, PDF_CMAP_TABLE, 9200 },
	{ 0x9b33, 0x9b3a, PDF_CMAP_RANGE, 21291 },
	{ 0x9b3b, 0x9b3c, PDF_CMAP_TABLE, 9205 },
	{ 0x9b3d, 0x9b40, PDF_CMAP_RANGE, 21299 },
	{ 0x9b41, 0x9b49, PDF_CMAP_TABLE, 9207 },
	{ 0x9b4a, 0x9b4c, PDF_CMAP_RANGE, 21304 },
	{ 0x9b4d, 0x9b51, PDF_CMAP_TABLE, 9216 },
	{ 0x9b52, 0x9b53, PDF_CMAP_RANGE, 21308 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 2826 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 21310 },
	{ 0x9b58, 0x9b5a, PDF_CMAP_TABLE, 9221 },
	{ 0x9b5b, 0x9b6e, PDF_CMAP_RANGE, 21314 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 8266 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 21334 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 9759 },
	{ 0x9b75, 0x9b76, PDF_CMAP_RANGE, 21338 },
	{ 0x9b77, 0x9b77, PDF_CMAP_SINGLE, 9758 },
	{ 0x9b78, 0x9b80, PDF_CMAP_RANGE, 21340 },
	{ 0x9b81, 0x9b83, PDF_CMAP_TABLE, 9224 },
	{ 0x9b84, 0x9b8d, PDF_CMAP_RANGE, 21350 },
	{ 0x9b8e, 0x9b92, PDF_CMAP_TABLE, 9227 },
	{ 0x9b93, 0x9b99, PDF_CMAP_RANGE, 21361 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 9769 },
	{ 0x9b9b, 0x9b9c, PDF_CMAP_RANGE, 21368 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_TABLE, 9232 },
	{ 0x9b9f, 0x9ba9, PDF_CMAP_RANGE, 21370 },
	{ 0x9baa, 0x9bae, PDF_CMAP_TABLE, 9234 },
	{ 0x9baf, 0x9bbf, PDF_CMAP_RANGE, 21382 },
	{ 0x9bc0, 0x9bc1, PDF_CMAP_TABLE, 9239 },
	{ 0x9bc2, 0x9bc6, PDF_CMAP_RANGE, 21399 },
	{ 0x9bc7, 0x9bca, PDF_CMAP_TABLE, 9241 },
	{ 0x9bcb, 0x9bd3, PDF_CMAP_RANGE, 21405 },
	{ 0x9bd4, 0x9bd6, PDF_CMAP_TABLE, 9245 },
	{ 0x9bd7, 0x9bda, PDF_CMAP_RANGE, 21415 },
	{ 0x9bdb, 0x9bdd, PDF_CMAP_TABLE, 9248 },
	{ 0x9bde, 0x9be0, PDF_CMAP_RANGE, 21420 },
	{ 0x9be1, 0x9be4, PDF_CMAP_TABLE, 9251 },
	{ 0x9be5, 0x9be6, PDF_CMAP_RANGE, 21424 },
	{ 0x9be7, 0x9be9, PDF_CMAP_TABLE, 9255 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 9787 },
	{ 0x9bec, 0x9bef, PDF_CMAP_RANGE, 21427 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 9794 },
	{ 0x9bf1, 0x9bf3, PDF_CMAP_RANGE, 21431 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 9796 },
	{ 0x9bf5, 0x9bfc, PDF_CMAP_RANGE, 21434 },
	{ 0x9bfd, 0x9bff, PDF_CMAP_TABLE, 9258 },
	{ 0x9c00, 0x9c07, PDF_CMAP_RANGE, 21443 },
	{ 0x9c08, 0x9c09, PDF_CMAP_TABLE, 9261 },
	{ 0x9c0a, 0x9c0c, PDF_CMAP_RANGE, 21451 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 9801 },
	{ 0x9c0e, 0x9c0f, PDF_CMAP_RANGE, 21454 },
	{ 0x9c10, 0x9c13, PDF_CMAP_TABLE, 9263 },
	{ 0x9c14, 0x9c1f, PDF_CMAP_RANGE, 21457 },
	{ 0x9c20, 0x9c20, PDF_CMAP_SINGLE, 9805 },
	{ 0x9c21, 0x9c22, PDF_CMAP_RANGE, 21469 },
	{ 0x9c23, 0x9c25, PDF_CMAP_TABLE, 9267 },
	{ 0x9c26, 0x9c27, PDF_CMAP_RANGE, 21472 },
	{ 0x9c28, 0x9c29, PDF_CMAP_TABLE, 9270 },
	{ 0x9c2a, 0x9c2c, PDF_CMAP_RANGE, 21474 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 9807 },
	{ 0x9c2e, 0x9c30, PDF_CMAP_RANGE, 21477 },
	{ 0x9c31, 0x9c3b, PDF_CMAP_TABLE, 9272 },
	{ 0x9c3c, 0x9c3d, PDF_CMAP_RANGE, 21484 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 9812 },
	{ 0x9c3f, 0x9c44, PDF_CMAP_RANGE, 21486 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 9816 },
	{ 0x9c46, 0x9c47, PDF_CMAP_RANGE, 21492 },
	{ 0x9c48, 0x9c49, PDF_CMAP_TABLE, 9283 },
	{ 0x9c4a, 0x9c51, PDF_CMAP_RANGE, 21494 },
	{ 0x9c52, 0x9c58, PDF_CMAP_TABLE, 9285 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_RANGE, 21504 },
	{ 0x9c5d, 0x9c5f, PDF_CMAP_TABLE, 9292 },
	{ 0x9c60, 0x9c66, PDF_CMAP_RANGE, 21509 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 9820 },
	{ 0x9c68, 0x9c6c, PDF_CMAP_RANGE, 21516 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 9772 },
	{ 0x9c6e, 0x9c77, PDF_CMAP_RANGE, 21521 },
	{ 0x9c78, 0x9c7c, PDF_CMAP_TABLE, 9295 },
	{ 0x9c7d, 0x9c7e, PDF_CMAP_RANGE, 21533 },
	{ 0x9c7f, 0x9c82, PDF_CMAP_TABLE, 9300 },
	{ 0x9c83, 0x9c84, PDF_CMAP_RANGE, 21536 },
	{ 0x9c85, 0x9c88, PDF_CMAP_RANGE, 7563 },
	{ 0x9c89, 0x9c8a, PDF_CMAP_RANGE, 21538 },
	{ 0x9c8b, 0x9c8f, PDF_CMAP_TABLE, 9304 },
	{ 0x9c90, 0x9c92, PDF_CMAP_RANGE, 7570 },
	{ 0x9c93, 0x9c93, PDF_CMAP_SINGLE, 21542 },
	{ 0x9c94, 0x9c95, PDF_CMAP_RANGE, 7573 },
	{ 0x9c96, 0x9c99, PDF_CMAP_RANGE, 21543 },
	{ 0x9c9a, 0x9c9b, PDF_CMAP_RANGE, 7575 },
	{ 0x9c9c, 0x9c9d, PDF_CMAP_TABLE, 9309 },
	{ 0x9c9e, 0x9ca3, PDF_CMAP_RANGE, 7577 },
	{ 0x9ca4, 0x9ca4, PDF_CMAP_SINGLE, 2523 },
	{ 0x9ca5, 0x9ca9, PDF_CMAP_RANGE, 7583 },
	{ 0x9caa, 0x9cac, PDF_CMAP_TABLE, 9311 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 7589 },
	{ 0x9caf, 0x9caf, PDF_CMAP_SINGLE, 21550 },
	{ 0x9cb0, 0x9cb7, PDF_CMAP_RANGE, 7591 },
	{ 0x9cb8, 0x9cb9, PDF_CMAP_TABLE, 9314 },
	{ 0x9cba, 0x9cbd, PDF_CMAP_RANGE, 7599 },
	{ 0x9cbe, 0x9cc2, PDF_CMAP_RANGE, 21552 },
	{ 0x9cc3, 0x9cc3, PDF_CMAP_SINGLE, 3285 },
	{ 0x9cc4, 0x9cc7, PDF_CMAP_RANGE, 7603 },
	{ 0x9cc8, 0x9cc9, PDF_CMAP_RANGE, 21557 },
	{ 0x9cca, 0x9cd0, PDF_CMAP_RANGE, 7607 },
	{ 0x9cd1, 0x9cd2, PDF_CMAP_RANGE, 21559 },
	{ 0x9cd3, 0x9cd5, PDF_CMAP_RANGE, 7614 },
	{ 0x9cd6, 0x9cd6, PDF_CMAP_SINGLE, 1111 },
	{ 0x9cd7, 0x9cd9, PDF_CMAP_RANGE, 7617 },
	{ 0x9cda, 0x9cdb, PDF_CMAP_RANGE, 21561 },
	{ 0x9cdc, 0x9cdd, PDF_CMAP_RANGE, 7620 },
	{ 0x9cde, 0x9cdf, PDF_CMAP_TABLE, 9316 },
	{ 0x9ce0, 0x9ce1, PDF_CMAP_RANGE, 21563 },
	{ 0x9ce2, 0x9ce2, PDF_CMAP_SINGLE, 7623 },
	{ 0x9ce3, 0x9ce4, PDF_CMAP_RANGE, 21565 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 8348 },
	{ 0x9ce6, 0x9ce8, PDF_CMAP_RANGE, 21567 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 9600 },
	{ 0x9cea, 0x9ceb, PDF_CMAP_RANGE, 21570 },
	{ 0x9cec, 0x9cec, PDF_CMAP_SINGLE, 8924 },
	{ 0x9ced, 0x9cf2, PDF_CMAP_RANGE, 21572 },
	{ 0x9cf3, 0x9cf6, PDF_CMAP_TABLE, 9318 },
	{ 0x9cf7, 0x9d05, PDF_CMAP_RANGE, 21579 },
	{ 0x9d06, 0x9d09, PDF_CMAP_TABLE, 9322 },
	{ 0x9d0a, 0x9d14, PDF_CMAP_RANGE, 21595 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 8578 },
	{ 0x9d16, 0x9d1a, PDF_CMAP_RANGE, 21606 },
	{ 0x9d1b, 0x9d1f, PDF_CMAP_TABLE, 9326 },
	{ 0x9d20, 0x9d22, PDF_CMAP_RANGE, 21613 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 9604 },
	{ 0x9d24, 0x9d25, PDF_CMAP_RANGE, 21616 },
	{ 0x9d26, 0x9d28, PDF_CMAP_TABLE, 9331 },
	{ 0x9d29, 0x9d2e, PDF_CMAP_RANGE, 21619 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_TABLE, 9334 },
	{ 0x9d31, 0x9d3a, PDF_CMAP_RANGE, 21625 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 8007 },
	{ 0x9d3c, 0x9d3e, PDF_CMAP_RANGE, 21635 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 7965 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 21638 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 9613 },
	{ 0x9d43, 0x9d50, PDF_CMAP_RANGE, 21640 },
	{ 0x9d51, 0x9d53, PDF_CMAP_TABLE, 9336 },
	{ 0x9d54, 0x9d5b, PDF_CMAP_RANGE, 21654 },
	{ 0x9d5c, 0x9d5d, PDF_CMAP_TABLE, 9339 },
	{ 0x9d5e, 0x9d5f, PDF_CMAP_RANGE, 21662 },
	{ 0x9d60, 0x9d61, PDF_CMAP_TABLE, 9341 },
	{ 0x9d62, 0x9d69, PDF_CMAP_RANGE, 21664 },
	{ 0x9d6a, 0x9d6c, PDF_CMAP_TABLE, 9343 },
	{ 0x9d6d, 0x9d6e, PDF_CMAP_RANGE, 21673 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 9624 },
	{ 0x9d70, 0x9d71, PDF_CMAP_RANGE, 21675 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 8438 },
	{ 0x9d73, 0x9d86, PDF_CMAP_RANGE, 21677 },
	{ 0x9d87, 0x9d89, PDF_CMAP_TABLE, 9346 },
	{ 0x9d8a, 0x9d92, PDF_CMAP_RANGE, 21698 },
	{ 0x9d93, 0x9d93, PDF_CMAP_SINGLE, 9622 },
	{ 0x9d94, 0x9d97, PDF_CMAP_RANGE, 21707 },
	{ 0x9d98, 0x9d9a, PDF_CMAP_TABLE, 9349 },
	{ 0x9d9b, 0x9da4, PDF_CMAP_RANGE, 21712 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 9629 },
	{ 0x9da6, 0x9da8, PDF_CMAP_RANGE, 21722 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 9630 },
	{ 0x9daa, 0x9dae, PDF_CMAP_RANGE, 21725 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 9036 },
	{ 0x9db0, 0x9db3, PDF_CMAP_RANGE, 21730 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 8004 },
	{ 0x9db5, 0x9dba, PDF_CMAP_RANGE, 21734 },
	{ 0x9dbb, 0x9dbc, PDF_CMAP_TABLE, 9352 },
	{ 0x9dbd, 0x9dbf, PDF_CMAP_RANGE, 21740 },
	{ 0x9dc0, 0x9dc4, PDF_CMAP_TABLE, 9354 },
	{ 0x9dc5, 0x9dd2, PDF_CMAP_RANGE, 21745 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 9634 },
	{ 0x9dd4, 0x9dd6, PDF_CMAP_RANGE, 21759 },
	{ 0x9dd7, 0x9dda, PDF_CMAP_TABLE, 9359 },
	{ 0x9ddb, 0x9de4, PDF_CMAP_RANGE, 21763 },
	{ 0x9de5, 0x9de6, PDF_CMAP_TABLE, 9363 },
	{ 0x9de7, 0x9dee, PDF_CMAP_RANGE, 21773 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 9636 },
	{ 0x9df0, 0x9df1, PDF_CMAP_RANGE, 21781 },
	{ 0x9df2, 0x9df3, PDF_CMAP_TABLE, 9365 },
	{ 0x9df4, 0x9df7, PDF_CMAP_RANGE, 21783 },
	{ 0x9df8, 0x9dfa, PDF_CMAP_TABLE, 9367 },
	{ 0x9dfb, 0x9e0b, PDF_CMAP_RANGE, 21787 },
	{ 0x9e0c, 0x9e0c, PDF_CMAP_SINGLE, 9640 },
	{ 0x9e0d, 0x9e14, PDF_CMAP_RANGE, 21804 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 9606 },
	{ 0x9e16, 0x9e19, PDF_CMAP_RANGE, 21812 },
	{ 0x9e1a, 0x9e27, PDF_CMAP_TABLE, 9370 },
	{ 0x9e28, 0x9e2c, PDF_CMAP_RANGE, 6967 },
	{ 0x9e2d, 0x9e38, PDF_CMAP_TABLE, 9384 },
	{ 0x9e39, 0x9e3a, PDF_CMAP_RANGE, 6977 },
	{ 0x9e3b, 0x9e3c, PDF_CMAP_RANGE, 21822 },
	{ 0x9e3d, 0x9e40, PDF_CMAP_TABLE, 9396 },
	{ 0x9e41, 0x9e42, PDF_CMAP_RANGE, 6980 },
	{ 0x9e43, 0x9e45, PDF_CMAP_TABLE, 9400 },
	{ 0x9e46, 0x9e49, PDF_CMAP_RANGE, 6983 },
	{ 0x9e4a, 0x9e4a, PDF_CMAP_SINGLE, 3216 },
	{ 0x9e4b, 0x9e4c, PDF_CMAP_RANGE, 6987 },
	{ 0x9e4d, 0x9e51, PDF_CMAP_TABLE, 9403 },
	{ 0x9e52, 0x9e54, PDF_CMAP_RANGE, 21827 },
	{ 0x9e55, 0x9e59, PDF_CMAP_TABLE, 9408 },
	{ 0x9e5a, 0x9e5c, PDF_CMAP_RANGE, 6993 },
	{ 0x9e5d, 0x9e5e, PDF_CMAP_TABLE, 9413 },
	{ 0x9e5f, 0x9e62, PDF_CMAP_RANGE, 21833 },
	{ 0x9e63, 0x9e65, PDF_CMAP_TABLE, 9415 },
	{ 0x9e66, 0x9e6c, PDF_CMAP_RANGE, 6998 },
	{ 0x9e6d, 0x9e6d, PDF_CMAP_SINGLE, 7006 },
	{ 0x9e6e, 0x9e6f, PDF_CMAP_RANGE, 21838 },
	{ 0x9e70, 0x9e75, PDF_CMAP_TABLE, 9418 },
	{ 0x9e76, 0x9e78, PDF_CMAP_RANGE, 21842 },
	{ 0x9e79, 0x9e7f, PDF_CMAP_TABLE, 9424 },
	{ 0x9e80, 0x9e81, PDF_CMAP_RANGE, 21846 },
	{ 0x9e82, 0x9e82, PDF_CMAP_SINGLE, 7676 },
	{ 0x9e83, 0x9e86, PDF_CMAP_RANGE, 21848 },
	{ 0x9e87, 0x9e88, PDF_CMAP_RANGE, 7677 },
	{ 0x9e89, 0x9e8a, PDF_CMAP_RANGE, 21852 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 7679 },
	{ 0x9e8c, 0x9e91, PDF_CMAP_RANGE, 21854 },
	{ 0x9e92, 0x9e93, PDF_CMAP_TABLE, 9431 },
	{ 0x9e94, 0x9e96, PDF_CMAP_RANGE, 21860 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 8207 },
	{ 0x9e98, 0x9e9c, PDF_CMAP_RANGE, 21863 },
	{ 0x9e9d, 0x9e9f, PDF_CMAP_TABLE, 9433 },
	{ 0x9ea0, 0x9ea4, PDF_CMAP_RANGE, 21869 },
	{ 0x9ea5, 0x9ea6, PDF_CMAP_TABLE, 9436 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 21874 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 9719 },
	{ 0x9eaa, 0x9eae, PDF_CMAP_RANGE, 21876 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 9878 },
	{ 0x9eb0, 0x9eb3, PDF_CMAP_RANGE, 21881 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 9438 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 21885 },
	{ 0x9eb8, 0x9eb8, PDF_CMAP_SINGLE, 7405 },
	{ 0x9eb9, 0x9eba, PDF_CMAP_RANGE, 21887 },
	{ 0x9ebb, 0x9ebc, PDF_CMAP_TABLE, 9440 },
	{ 0x9ebd, 0x9ebe, PDF_CMAP_RANGE, 7673 },
	{ 0x9ebf, 0x9ec3, PDF_CMAP_RANGE, 21890 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 2011 },
	{ 0x9ec5, 0x9ec8, PDF_CMAP_RANGE, 21895 },
	{ 0x9ec9, 0x9ec9, PDF_CMAP_SINGLE, 4856 },
	{ 0x9eca, 0x9ecb, PDF_CMAP_RANGE, 21899 },
	{ 0x9ecc, 0x9ed1, PDF_CMAP_TABLE, 9442 },
	{ 0x9ed2, 0x9ed3, PDF_CMAP_RANGE, 21902 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 3114 },
	{ 0x9ed5, 0x9ed7, PDF_CMAP_RANGE, 21904 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 2831 },
	{ 0x9ed9, 0x9eda, PDF_CMAP_RANGE, 21907 },
	{ 0x9edb, 0x9edd, PDF_CMAP_RANGE, 7684 },
	{ 0x9ede, 0x9ee2, PDF_CMAP_TABLE, 9448 },
	{ 0x9ee3, 0x9ee4, PDF_CMAP_RANGE, 21910 },
	{ 0x9ee5, 0x9eea, PDF_CMAP_TABLE, 9453 },
	{ 0x9eeb, 0x9eee, PDF_CMAP_RANGE, 21913 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 7694 },
	{ 0x9ef0, 0x9ef1, PDF_CMAP_RANGE, 21917 },
	{ 0x9ef2, 0x9ef4, PDF_CMAP_TABLE, 9459 },
	{ 0x9ef5, 0x9ef6, PDF_CMAP_RANGE, 21920 },
	{ 0x9ef7, 0x9efa, PDF_CMAP_TABLE, 9462 },
	{ 0x9efb, 0x9efc, PDF_CMAP_RANGE, 6742 },
	{ 0x9efd, 0x9eff, PDF_CMAP_TABLE, 9466 },
	{ 0x9f00, 0x9f08, PDF_CMAP_RANGE, 21924 },
	{ 0x9f09, 0x9f10, PDF_CMAP_TABLE, 9469 },
	{ 0x9f11, 0x9f12, PDF_CMAP_RANGE, 21936 },
	{ 0x9f13, 0x9f19, PDF_CMAP_TABLE, 9477 },
	{ 0x9f1a, 0x9f1f, PDF_CMAP_RANGE, 21941 },
	{ 0x9f20, 0x9f22, PDF_CMAP_TABLE, 9484 },
	{ 0x9f23, 0x9f2b, PDF_CMAP_RANGE, 21948 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7696 },
	{ 0x9f2d, 0x9f2e, PDF_CMAP_RANGE, 21957 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 7697 },
	{ 0x9f30, 0x9f36, PDF_CMAP_RANGE, 21959 },
	{ 0x9f37, 0x9f3c, PDF_CMAP_TABLE, 9487 },
	{ 0x9f3d, 0x9f3e, PDF_CMAP_RANGE, 7700 },
	{ 0x9f3f, 0x9f43, PDF_CMAP_RANGE, 21969 },
	{ 0x9f44, 0x9f44, PDF_CMAP_SINGLE, 7702 },
	{ 0x9f45, 0x9f49, PDF_CMAP_RANGE, 21974 },
	{ 0x9f4a, 0x9f4b, PDF_CMAP_TABLE, 9493 },
	{ 0x9f4c, 0x9f4e, PDF_CMAP_RANGE, 21979 },
	{ 0x9f4f, 0x9f54, PDF_CMAP_TABLE, 9495 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 21983 },
	{ 0x9f59, 0x9f59, PDF_CMAP_SINGLE, 9745 },
	{ 0x9f5a, 0x9f5b, PDF_CMAP_RANGE, 21987 },
	{ 0x9f5c, 0x9f5c, PDF_CMAP_SINGLE, 9747 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 21989 },
	{ 0x9f5f, 0x9f63, PDF_CMAP_TABLE, 9501 },
	{ 0x9f64, 0x9f65, PDF_CMAP_RANGE, 21992 },
	{ 0x9f66, 0x9f66, PDF_CMAP_SINGLE, 9748 },
	{ 0x9f67, 0x9f69, PDF_CMAP_RANGE, 21994 },
	{ 0x9f6a, 0x9f6c, PDF_CMAP_TABLE, 9506 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 21998 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 8434 },
	{ 0x9f73, 0x9f76, PDF_CMAP_RANGE, 22003 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 9751 },
	{ 0x9f78, 0x9f7e, PDF_CMAP_RANGE, 22007 },
	{ 0x9f7f, 0x9f80, PDF_CMAP_TABLE, 9509 },
	{ 0x9f81, 0x9f82, PDF_CMAP_RANGE, 22014 },
	{ 0x9f83, 0x9f84, PDF_CMAP_TABLE, 9511 },
	{ 0x9f85, 0x9f8a, PDF_CMAP_RANGE, 7535 },
	{ 0x9f8b, 0x9f8d, PDF_CMAP_TABLE, 9513 },
	{ 0x9f8e, 0x9f8f, PDF_CMAP_RANGE, 22016 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 8371 },
	{ 0x9f91, 0x9f93, PDF_CMAP_RANGE, 22018 },
	{ 0x9f94, 0x9f95, PDF_CMAP_TABLE, 9516 },
	{ 0x9f96, 0x9f98, PDF_CMAP_RANGE, 22021 },
	{ 0x9f99, 0x9f9c, PDF_CMAP_TABLE, 9518 },
	{ 0x9f9d, 0x9f9e, PDF_CMAP_RANGE, 22024 },
	{ 0x9f9f, 0x9fa0, PDF_CMAP_TABLE, 9522 },
	{ 0x9fa1, 0x9fa5, PDF_CMAP_RANGE, 22026 },
	{ 0xe816, 0xe818, PDF_CMAP_RANGE, 22048 },
	{ 0xe81e, 0xe81e, PDF_CMAP_SINGLE, 22056 },
	{ 0xe826, 0xe826, PDF_CMAP_SINGLE, 22064 },
	{ 0xe82b, 0xe82c, PDF_CMAP_RANGE, 22069 },
	{ 0xe831, 0xe832, PDF_CMAP_RANGE, 22075 },
	{ 0xe83b, 0xe83b, PDF_CMAP_SINGLE, 22085 },
	{ 0xe843, 0xe843, PDF_CMAP_SINGLE, 22093 },
	{ 0xe854, 0xe855, PDF_CMAP_RANGE, 22110 },
	{ 0xe864, 0xe864, PDF_CMAP_SINGLE, 22126 },
	{ 0xf92c, 0xf92c, PDF_CMAP_SINGLE, 22031 },
	{ 0xfa0d, 0xfa0f, PDF_CMAP_RANGE, 22032 },
	{ 0xfa11, 0xfa11, PDF_CMAP_SINGLE, 22035 },
	{ 0xfa13, 0xfa14, PDF_CMAP_RANGE, 22036 },
	{ 0xfa18, 0xfa18, PDF_CMAP_SINGLE, 22038 },
	{ 0xfa1f, 0xfa21, PDF_CMAP_RANGE, 22039 },
	{ 0xfa23, 0xfa24, PDF_CMAP_RANGE, 22042 },
	{ 0xfa27, 0xfa29, PDF_CMAP_RANGE, 22044 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 10013 },
	{ 0xfe49, 0xfe52, PDF_CMAP_RANGE, 10027 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 10037 },
	{ 0xfe59, 0xfe66, PDF_CMAP_RANGE, 10041 },
	{ 0xfe68, 0xfe6b, PDF_CMAP_RANGE, 10055 },
	{ 0xff01, 0xff03, PDF_CMAP_RANGE, 262 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 166 },
	{ 0xff05, 0xff5d, PDF_CMAP_RANGE, 266 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 106 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 168 },
	{ 0xffe2, 0xffe5, PDF_CMAP_TABLE, 9524 },
	{ 0x6ec6, 0x6ec6, PDF_CMAP_SINGLE, 14836 },
	{ 0x6ec7, 0x6ec7, PDF_CMAP_SINGLE, 1474 },
	{ 0x6ec8, 0x6eca, PDF_CMAP_RANGE, 14837 },
	{ 0x6ecb, 0x6ecb, PDF_CMAP_SINGLE, 4649 },
	{ 0x6ecc, 0x6ecc, PDF_CMAP_SINGLE, 7869 },
	{ 0x6ecd, 0x6ecd, PDF_CMAP_SINGLE, 14840 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 9025 },
	{ 0x6ecf, 0x6ecf, PDF_CMAP_SINGLE, 5892 },
	{ 0x6ed0, 0x6ed0, PDF_CMAP_SINGLE, 14841 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 1985 },
	{ 0x6ed2, 0x6ed2, PDF_CMAP_SINGLE, 14842 },
	{ 0x6ed3, 0x6ed3, PDF_CMAP_SINGLE, 4655 },
	{ 0x6ed4, 0x6ed4, PDF_CMAP_SINGLE, 3618 },
	{ 0x6ed5, 0x6ed5, PDF_CMAP_SINGLE, 6568 },
	{ 0x6ed6, 0x6ed6, PDF_CMAP_SINGLE, 14843 },
	{ 0x6ed7, 0x6ed7, PDF_CMAP_SINGLE, 5890 },
	{ 0x6ed8, 0x6ed9, PDF_CMAP_RANGE, 14844 },
	{ 0x6eda, 0x6eda, PDF_CMAP_SINGLE, 1871 },
	{ 0x6edb, 0x6edd, PDF_CMAP_RANGE, 14846 },
	{ 0x6ede, 0x6ede, PDF_CMAP_SINGLE, 4556 },
	{ 0x6edf, 0x6edf, PDF_CMAP_SINGLE, 5879 },
	{ 0x6ee0, 0x6ee0, PDF_CMAP_SINGLE, 5882 },
	{ 0x6ee1, 0x6ee1, PDF_CMAP_SINGLE, 2721 },
	{ 0x6ee2, 0x6ee2, PDF_CMAP_SINGLE, 5884 },
	{ 0x6ee3, 0x6ee3, PDF_CMAP_SINGLE, 14849 },
	{ 0x6ee4, 0x6ee4, PDF_CMAP_SINGLE, 2674 },
	{ 0x6ee5, 0x6ee5, PDF_CMAP_SINGLE, 2479 },
	{ 0x6ee6, 0x6ee6, PDF_CMAP_SINGLE, 2679 },
	{ 0x6ee7, 0x6ee7, PDF_CMAP_SINGLE, 14850 },
	{ 0x6ee8, 0x6ee8, PDF_CMAP_SINGLE, 1118 },
	{ 0x6ee9, 0x6ee9, PDF_CMAP_SINGLE, 3589 },
	{ 0x6eea, 0x6eeb, PDF_CMAP_RANGE, 14851 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 8011 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 14853 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 8850 },
	{ 0x6ef0, 0x6ef1, PDF_CMAP_RANGE, 14855 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 8484 },
	{ 0x6ef3, 0x6ef3, PDF_CMAP_SINGLE, 14857 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 1455 },
	{ 0x6ef5, 0x6ef6, PDF_CMAP_RANGE, 14858 },
	{ 0x6ef7, 0x6ef7, PDF_CMAP_SINGLE, 8264 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 9184 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 5901 },
	{ 0x6efa, 0x6efe, PDF_CMAP_RANGE, 14860 },
	{ 0x6eff, 0x6eff, PDF_CMAP_SINGLE, 8311 },
	{ 0x6f00, 0x6f00, PDF_CMAP_SINGLE, 14865 },
	{ 0x6f01, 0x6f01, PDF_CMAP_SINGLE, 8762 },
	{ 0x6f02, 0x6f02, PDF_CMAP_SINGLE, 3022 },
	{ 0x6f03, 0x6f05, PDF_CMAP_RANGE, 14866 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 3072 },
	{ 0x6f07, 0x6f08, PDF_CMAP_RANGE, 14869 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 5907 },
	{ 0x6f0a, 0x6f0e, PDF_CMAP_RANGE, 14871 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 2640 },
	{ 0x6f10, 0x6f12, PDF_CMAP_RANGE, 14876 },
	{ 0x6f13, 0x6f13, PDF_CMAP_SINGLE, 2519 },
	{ 0x6f14, 0x6f14, PDF_CMAP_SINGLE, 4102 },
	{ 0x6f15, 0x6f15, PDF_CMAP_SINGLE, 5900 },
	{ 0x6f16, 0x6f19, PDF_CMAP_RANGE, 14879 },
	{ 0x6f1a, 0x6f1a, PDF_CMAP_SINGLE, 8369 },
	{ 0x6f1b, 0x6f1f, PDF_CMAP_RANGE, 14883 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 2833 },
	{ 0x6f21, 0x6f21, PDF_CMAP_SINGLE, 14888 },
	{ 0x6f22, 0x6f22, PDF_CMAP_SINGLE, 8001 },
	{ 0x6f23, 0x6f23, PDF_CMAP_SINGLE, 8220 },
	{ 0x6f24, 0x6f24, PDF_CMAP_SINGLE, 5899 },
	{ 0x6f25, 0x6f28, PDF_CMAP_RANGE, 14889 },
	{ 0x6f29, 0x6f29, PDF_CMAP_SINGLE, 5908 },
	{ 0x6f2a, 0x6f2a, PDF_CMAP_SINGLE, 5906 },
	{ 0x6f2b, 0x6f2b, PDF_CMAP_SINGLE, 2725 },
	{ 0x6f2c, 0x6f2c, PDF_CMAP_SINGLE, 8888 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 5883 },
	{ 0x6f2e, 0x6f2e, PDF_CMAP_SINGLE, 14893 },
	{ 0x6f2f, 0x6f2f, PDF_CMAP_SINGLE, 5902 },
	{ 0x6f30, 0x6f30, PDF_CMAP_SINGLE, 14894 },
	{ 0x6f31, 0x6f31, PDF_CMAP_SINGLE, 3477 },
	{ 0x6f32, 0x6f32, PDF_CMAP_SINGLE, 8822 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 4453 },
	{ 0x6f34, 0x6f35, PDF_CMAP_RANGE, 14895 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 5903 },
	{ 0x6f37, 0x6f37, PDF_CMAP_SINGLE, 14897 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 8091 },
	{ 0x6f39, 0x6f3d, PDF_CMAP_RANGE, 14898 },
	{ 0x6f3e, 0x6f3e, PDF_CMAP_SINGLE, 4131 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 8095 },
	{ 0x6f40, 0x6f40, PDF_CMAP_SINGLE, 14903 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 9674 },
	{ 0x6f42, 0x6f45, PDF_CMAP_RANGE, 14904 },
	{ 0x6f46, 0x6f47, PDF_CMAP_RANGE, 5897 },
	{ 0x6f48, 0x6f4a, PDF_CMAP_RANGE, 14908 },
	{ 0x6f4b, 0x6f4b, PDF_CMAP_SINGLE, 5904 },
	{ 0x6f4c, 0x6f4c, PDF_CMAP_SINGLE, 14911 },
	{ 0x6f4d, 0x6f4d, PDF_CMAP_SINGLE, 3770 },
	{ 0x6f4e, 0x6f50, PDF_CMAP_RANGE, 14912 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 8382 },
	{ 0x6f52, 0x6f53, PDF_CMAP_RANGE, 14915 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 8116 },
	{ 0x6f55, 0x6f57, PDF_CMAP_RANGE, 14917 },
	{ 0x6f58, 0x6f58, PDF_CMAP_SINGLE, 2956 },
	{ 0x6f59, 0x6f59, PDF_CMAP_SINGLE, 9175 },
	{ 0x6f5a, 0x6f5b, PDF_CMAP_RANGE, 14920 },
	{ 0x6f5c, 0x6f5c, PDF_CMAP_SINGLE, 3118 },
	{ 0x6f5d, 0x6f5d, PDF_CMAP_SINGLE, 14922 },
	{ 0x6f5e, 0x6f5e, PDF_CMAP_SINGLE, 2657 },
	{ 0x6f5f, 0x6f61, PDF_CMAP_RANGE, 14923 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 5896 },
	{ 0x6f63, 0x6f63, PDF_CMAP_SINGLE, 14926 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 8453 },
	{ 0x6f65, 0x6f65, PDF_CMAP_SINGLE, 14927 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 2579 },
	{ 0x6f67, 0x6f6c, PDF_CMAP_RANGE, 14928 },
	{ 0x6f6d, 0x6f6d, PDF_CMAP_SINGLE, 3593 },
	{ 0x6f6e, 0x6f6e, PDF_CMAP_SINGLE, 1238 },
	{ 0x6f6f, 0x6f6f, PDF_CMAP_SINGLE, 9185 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 8172 },
	{ 0x6f71, 0x6f71, PDF_CMAP_SINGLE, 14934 },
	{ 0x6f72, 0x6f72, PDF_CMAP_SINGLE, 5913 },
	{ 0x6f73, 0x6f73, PDF_CMAP_SINGLE, 14935 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 5905 },
	{ 0x6f75, 0x6f76, PDF_CMAP_RANGE, 14936 },
	{ 0x6f77, 0x6f77, PDF_CMAP_SINGLE, 9194 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 5912 },
	{ 0x6f79, 0x6f79, PDF_CMAP_SINGLE, 14938 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 5915 },
	{ 0x6f7b, 0x6f7b, PDF_CMAP_SINGLE, 14939 },
	{ 0x6f7c, 0x6f7c, PDF_CMAP_SINGLE, 5914 },
	{ 0x6f7d, 0x6f7e, PDF_CMAP_RANGE, 14940 },
	{ 0x6f7f, 0x6f7f, PDF_CMAP_SINGLE, 9187 },
	{ 0x6f80, 0x6f80, PDF_CMAP_SINGLE, 8462 },
	{ 0x6f81, 0x6f83, PDF_CMAP_RANGE, 14942 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 1267 },
	{ 0x6f85, 0x6f85, PDF_CMAP_SINGLE, 14945 },
	{ 0x6f86, 0x6f86, PDF_CMAP_SINGLE, 8102 },
	{ 0x6f87, 0x6f87, PDF_CMAP_SINGLE, 8196 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 1247 },
	{ 0x6f89, 0x6f89, PDF_CMAP_SINGLE, 5909 },
	{ 0x6f8a, 0x6f8b, PDF_CMAP_RANGE, 14946 },
	{ 0x6f8c, 0x6f8c, PDF_CMAP_SINGLE, 5911 },
	{ 0x6f8d, 0x6f8d, PDF_CMAP_SINGLE, 5910 },
	{ 0x6f8e, 0x6f8e, PDF_CMAP_SINGLE, 2989 },
	{ 0x6f8f, 0x6f96, PDF_CMAP_RANGE, 14948 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 8093 },
	{ 0x6f98, 0x6f9b, PDF_CMAP_RANGE, 14956 },
	{ 0x6f9c, 0x6f9c, PDF_CMAP_SINGLE, 2472 },
	{ 0x6f9d, 0x6f9f, PDF_CMAP_RANGE, 14960 },
	{ 0x6fa0, 0x6fa0, PDF_CMAP_SINGLE, 9189 },
	{ 0x6fa1, 0x6fa1, PDF_CMAP_SINGLE, 4395 },
	{ 0x6fa2, 0x6fa3, PDF_CMAP_RANGE, 14963 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 8804 },
	{ 0x6fa5, 0x6fa6, PDF_CMAP_RANGE, 14965 },
	{ 0x6fa7, 0x6fa7, PDF_CMAP_SINGLE, 5918 },
	{ 0x6fa8, 0x6fa8, PDF_CMAP_SINGLE, 14967 },
	{ 0x6fa9, 0x6fa9, PDF_CMAP_SINGLE, 9442 },
	{ 0x6faa, 0x6fad, PDF_CMAP_RANGE, 14968 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 9182 },
	{ 0x6faf, 0x6fb0, PDF_CMAP_RANGE, 14972 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 7876 },
	{ 0x6fb2, 0x6fb2, PDF_CMAP_SINGLE, 14974 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 975 },
	{ 0x6fb4, 0x6fb5, PDF_CMAP_RANGE, 14975 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 5920 },
	{ 0x6fb7, 0x6fb8, PDF_CMAP_RANGE, 14977 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5919 },
	{ 0x6fba, 0x6fbf, PDF_CMAP_RANGE, 14979 },
	{ 0x6fc0, 0x6fc0, PDF_CMAP_SINGLE, 2071 },
	{ 0x6fc1, 0x6fc1, PDF_CMAP_SINGLE, 8886 },
	{ 0x6fc2, 0x6fc2, PDF_CMAP_SINGLE, 5921 },
	{ 0x6fc3, 0x6fc3, PDF_CMAP_SINGLE, 8361 },
	{ 0x6fc4, 0x6fc8, PDF_CMAP_RANGE, 14985 },
	{ 0x6fc9, 0x6fc9, PDF_CMAP_SINGLE, 5917 },
	{ 0x6fca, 0x6fd0, PDF_CMAP_RANGE, 14990 },
	{ 0x6fd1, 0x6fd1, PDF_CMAP_SINGLE, 5916 },
	{ 0x6fd2, 0x6fd2, PDF_CMAP_SINGLE, 1117 },
	{ 0x6fd3, 0x6fd4, PDF_CMAP_RANGE, 14997 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 8491 },
	{ 0x6fd6, 0x6fd7, PDF_CMAP_RANGE, 14999 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 8357 },
	{ 0x6fd9, 0x6fda, PDF_CMAP_RANGE, 15001 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 9867 },
	{ 0x6fdc, 0x6fdd, PDF_CMAP_RANGE, 15003 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 5924 },
	{ 0x6fdf, 0x6fdf, PDF_CMAP_SINGLE, 8054 },
	{ 0x6fe0, 0x6fe0, PDF_CMAP_SINGLE, 5925 },
	{ 0x6fe1, 0x6fe1, PDF_CMAP_SINGLE, 5922 },
	{ 0x6fe2, 0x6fe3, PDF_CMAP_RANGE, 15005 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 8557 },
	{ 0x6fe5, 0x6fea, PDF_CMAP_RANGE, 15007 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 8193 },
	{ 0x6fec, 0x6fed, PDF_CMAP_RANGE, 15013 },
	{ 0x6fee, 0x6fee, PDF_CMAP_SINGLE, 5923 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 5926 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 8593 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 7760 },
	{ 0x6ff2, 0x6ff9, PDF_CMAP_RANGE, 15015 },
	{ 0x6ffa, 0x6ffa, PDF_CMAP_SINGLE, 8092 },
	{ 0x6ffb, 0x6ffb, PDF_CMAP_SINGLE, 15023 },
	{ 0x6ffc, 0x6ffc, PDF_CMAP_SINGLE, 9178 },
	{ 0x6ffd, 0x6ffd, PDF_CMAP_SINGLE, 15024 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 8275 },
	{ 0x6fff, 0x6fff, PDF_CMAP_SINGLE, 15025 },
	{ 0x7000, 0x7004, PDF_CMAP_RANGE, 15026 },
	{ 0x7005, 0x7005, PDF_CMAP_SINGLE, 9193 },
	{ 0x7006, 0x7006, PDF_CMAP_SINGLE, 9188 },
	{ 0x7007, 0x7008, PDF_CMAP_RANGE, 15031 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 8661 },
	{ 0x700a, 0x700a, PDF_CMAP_SINGLE, 15033 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 9190 },
	{ 0x700c, 0x700e, PDF_CMAP_RANGE, 15034 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 9183 },
	{ 0x7010, 0x7010, PDF_CMAP_SINGLE, 15037 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 3064 },
	{ 0x7012, 0x7014, PDF_CMAP_RANGE, 15038 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 7759 },
	{ 0x7016, 0x7017, PDF_CMAP_RANGE, 15041 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 9177 },
	{ 0x7019, 0x7019, PDF_CMAP_SINGLE, 15043 },
	{ 0x701a, 0x701a, PDF_CMAP_SINGLE, 5927 },
	{ 0x701b, 0x701b, PDF_CMAP_SINGLE, 5929 },
	{ 0x701c, 0x701c, PDF_CMAP_SINGLE, 15044 },
	{ 0x701d, 0x701d, PDF_CMAP_SINGLE, 8212 },
	{ 0x701e, 0x701e, PDF_CMAP_SINGLE, 15045 },
	{ 0x701f, 0x701f, PDF_CMAP_SINGLE, 9196 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 9195 },
	{ 0x7021, 0x7022, PDF_CMAP_RANGE, 15046 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 5928 },
	{ 0x7024, 0x7026, PDF_CMAP_RANGE, 15048 },
	{ 0x7027, 0x7027, PDF_CMAP_SINGLE, 9176 },
	{ 0x7028, 0x7028, PDF_CMAP_SINGLE, 9198 },
	{ 0x7029, 0x702f, PDF_CMAP_RANGE, 15051 },
	{ 0x7030, 0x7030, PDF_CMAP_SINGLE, 9870 },
	{ 0x7031, 0x7031, PDF_CMAP_SINGLE, 15058 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 9197 },
	{ 0x7033, 0x7034, PDF_CMAP_RANGE, 15059 },
	{ 0x7035, 0x7035, PDF_CMAP_SINGLE, 5931 },
	{ 0x7036, 0x7038, PDF_CMAP_RANGE, 15061 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 5930 },
	{ 0x703a, 0x703d, PDF_CMAP_RANGE, 15064 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 8186 },
	{ 0x703f, 0x7042, PDF_CMAP_RANGE, 15068 },
	{ 0x7043, 0x7043, PDF_CMAP_SINGLE, 9174 },
	{ 0x7044, 0x7044, PDF_CMAP_SINGLE, 9192 },
	{ 0x7045, 0x704b, PDF_CMAP_RANGE, 15072 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 1849 },
	{ 0x704d, 0x704e, PDF_CMAP_RANGE, 15079 },
	{ 0x704f, 0x704f, PDF_CMAP_SINGLE, 5932 },
	{ 0x7050, 0x7050, PDF_CMAP_SINGLE, 15081 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 8454 },
	{ 0x7052, 0x7054, PDF_CMAP_RANGE, 15082 },
	{ 0x7055, 0x7055, PDF_CMAP_SINGLE, 8203 },
	{ 0x7056, 0x7057, PDF_CMAP_RANGE, 15085 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 8550 },
	{ 0x7059, 0x705c, PDF_CMAP_RANGE, 15087 },
	{ 0x705d, 0x705d, PDF_CMAP_SINGLE, 9199 },
	{ 0x705e, 0x705e, PDF_CMAP_SINGLE, 5933 },
	{ 0x705f, 0x7062, PDF_CMAP_RANGE, 15091 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 8585 },
	{ 0x7064, 0x7064, PDF_CMAP_SINGLE, 8280 },
	{ 0x7065, 0x7066, PDF_CMAP_RANGE, 15095 },
	{ 0x7067, 0x7067, PDF_CMAP_SINGLE, 9191 },
	{ 0x7068, 0x706a, PDF_CMAP_RANGE, 15097 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 2053 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 6639 },
	{ 0x706d, 0x706d, PDF_CMAP_SINGLE, 2805 },
	{ 0x706e, 0x706e, PDF_CMAP_SINGLE, 15100 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 1447 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 2023 },
	{ 0x7071, 0x7074, PDF_CMAP_RANGE, 15101 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 2610 },
	{ 0x7076, 0x7076, PDF_CMAP_SINGLE, 4401 },
	{ 0x7077, 0x7077, PDF_CMAP_SINGLE, 15105 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 2291 },
	{ 0x7079, 0x707b, PDF_CMAP_RANGE, 15106 },
	{ 0x707c, 0x707c, PDF_CMAP_SINGLE, 4643 },
	{ 0x707d, 0x707d, PDF_CMAP_SINGLE, 15109 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 4377 },
	{ 0x707f, 0x707f, PDF_CMAP_SINGLE, 1178 },
	{ 0x7080, 0x7080, PDF_CMAP_SINGLE, 6604 },
	{ 0x7081, 0x7084, PDF_CMAP_RANGE, 15110 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 6419 },
	{ 0x7086, 0x7088, PDF_CMAP_RANGE, 15114 },
	{ 0x7089, 0x7089, PDF_CMAP_SINGLE, 2646 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 1337 },
	{ 0x708b, 0x708d, PDF_CMAP_RANGE, 15117 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 4096 },
	{ 0x708f, 0x7091, PDF_CMAP_RANGE, 15120 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 1241 },
	{ 0x7093, 0x7093, PDF_CMAP_SINGLE, 15123 },
	{ 0x7094, 0x7094, PDF_CMAP_SINGLE, 3213 },
	{ 0x7095, 0x7095, PDF_CMAP_SINGLE, 2376 },
	{ 0x7096, 0x7096, PDF_CMAP_SINGLE, 6606 },
	{ 0x7097, 0x7098, PDF_CMAP_RANGE, 15124 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 4554 },
	{ 0x709a, 0x709b, PDF_CMAP_RANGE, 15126 },
	{ 0x709c, 0x709c, PDF_CMAP_SINGLE, 6605 },
	{ 0x709d, 0x709d, PDF_CMAP_SINGLE, 6607 },
	{ 0x709e, 0x70aa, PDF_CMAP_RANGE, 15128 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 6611 },
	{ 0x70ac, 0x70ac, PDF_CMAP_SINGLE, 2325 },
	{ 0x70ad, 0x70ad, PDF_CMAP_SINGLE, 3602 },
	{ 0x70ae, 0x70ae, PDF_CMAP_SINGLE, 2971 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 2283 },
	{ 0x70b0, 0x70b0, PDF_CMAP_SINGLE, 15141 },
	{ 0x70b1, 0x70b1, PDF_CMAP_SINGLE, 6612 },
	{ 0x70b2, 0x70b2, PDF_CMAP_SINGLE, 15142 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 1127 },
	{ 0x70b4, 0x70b6, PDF_CMAP_RANGE, 15143 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 6610 },
	{ 0x70b8, 0x70b8, PDF_CMAP_SINGLE, 4425 },
	{ 0x70b9, 0x70b9, PDF_CMAP_SINGLE, 1476 },
	{ 0x70ba, 0x70ba, PDF_CMAP_SINGLE, 15146 },
	{ 0x70bb, 0x70bb, PDF_CMAP_SINGLE, 6608 },
	{ 0x70bc, 0x70bc, PDF_CMAP_SINGLE, 2559 },
	{ 0x70bd, 0x70bd, PDF_CMAP_SINGLE, 1288 },
	{ 0x70be, 0x70bf, PDF_CMAP_RANGE, 15147 },
	{ 0x70c0, 0x70c0, PDF_CMAP_SINGLE, 6609 },
	{ 0x70c1, 0x70c1, PDF_CMAP_SINGLE, 3501 },
	{ 0x70c2, 0x70c2, PDF_CMAP_SINGLE, 2478 },
	{ 0x70c3, 0x70c3, PDF_CMAP_SINGLE, 3665 },
	{ 0x70c4, 0x70c7, PDF_CMAP_RANGE, 15149 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 2587 },
	{ 0x70c9, 0x70c9, PDF_CMAP_SINGLE, 15153 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 6614 },
	{ 0x70cb, 0x70ce, PDF_CMAP_RANGE, 15154 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 8611 },
	{ 0x70d0, 0x70d7, PDF_CMAP_RANGE, 15158 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 1949 },
	{ 0x70d9, 0x70d9, PDF_CMAP_SINGLE, 2494 },
	{ 0x70da, 0x70da, PDF_CMAP_SINGLE, 15166 },
	{ 0x70db, 0x70db, PDF_CMAP_SINGLE, 4593 },
	{ 0x70dc, 0x70de, PDF_CMAP_RANGE, 15167 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 4085 },
	{ 0x70e0, 0x70e3, PDF_CMAP_RANGE, 15170 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2379 },
	{ 0x70e5, 0x70e5, PDF_CMAP_SINGLE, 15174 },
	{ 0x70e6, 0x70e6, PDF_CMAP_SINGLE, 1616 },
	{ 0x70e7, 0x70e7, PDF_CMAP_SINGLE, 3342 },
	{ 0x70e8, 0x70e8, PDF_CMAP_SINGLE, 6613 },
	{ 0x70e9, 0x70e9, PDF_CMAP_SINGLE, 2039 },
	{ 0x70ea, 0x70ea, PDF_CMAP_SINGLE, 15175 },
	{ 0x70eb, 0x70eb, PDF_CMAP_SINGLE, 3615 },
	{ 0x70ec, 0x70ec, PDF_CMAP_SINGLE, 2254 },
	{ 0x70ed, 0x70ed, PDF_CMAP_SINGLE, 3235 },
	{ 0x70ee, 0x70ee, PDF_CMAP_SINGLE, 15176 },
	{ 0x70ef, 0x70ef, PDF_CMAP_SINGLE, 3862 },
	{ 0x70f0, 0x70f3, PDF_CMAP_RANGE, 15177 },
	{ 0x70f4, 0x70f4, PDF_CMAP_SINGLE, 8570 },
	{ 0x70f5, 0x70f6, PDF_CMAP_RANGE, 15181 },
	{ 0x70f7, 0x70f7, PDF_CMAP_SINGLE, 3738 },
	{ 0x70f8, 0x70f8, PDF_CMAP_SINGLE, 15183 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 2988 },
	{ 0x70fa, 0x70fc, PDF_CMAP_RANGE, 15184 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 1670 },
	{ 0x70fe, 0x70ff, PDF_CMAP_RANGE, 15187 },
	{ 0x7100, 0x7108, PDF_CMAP_RANGE, 15189 },
	{ 0x7109, 0x7109, PDF_CMAP_SINGLE, 4082 },
	{ 0x710a, 0x710a, PDF_CMAP_SINGLE, 1903 },
	{ 0x710b, 0x710f, PDF_CMAP_RANGE, 15198 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 6615 },
	{ 0x7111, 0x7112, PDF_CMAP_RANGE, 15203 },
	{ 0x7113, 0x7113, PDF_CMAP_SINGLE, 6616 },
	{ 0x7114, 0x7114, PDF_CMAP_SINGLE, 15205 },
	{ 0x7115, 0x7115, PDF_CMAP_SINGLE, 2005 },
	{ 0x7116, 0x7116, PDF_CMAP_SINGLE, 6617 },
	{ 0x7117, 0x7117, PDF_CMAP_SINGLE, 15206 },
	{ 0x7118, 0x7118, PDF_CMAP_SINGLE, 6640 },
	{ 0x7119, 0x7119, PDF_CMAP_SINGLE, 1059 },
	{ 0x711a, 0x711a, PDF_CMAP_SINGLE, 1654 },
	{ 0x711b, 0x7120, PDF_CMAP_RANGE, 15207 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 8613 },
	{ 0x7122, 0x7125, PDF_CMAP_RANGE, 15213 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2186 },
	{ 0x7127, 0x712e, PDF_CMAP_RANGE, 15217 },
	{ 0x712f, 0x712f, PDF_CMAP_SINGLE, 6618 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 4111 },
	{ 0x7131, 0x7131, PDF_CMAP_SINGLE, 6619 },
	{ 0x7132, 0x7135, PDF_CMAP_RANGE, 15225 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 3222 },
	{ 0x7137, 0x7144, PDF_CMAP_RANGE, 15229 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 6623 },
	{ 0x7146, 0x7148, PDF_CMAP_RANGE, 15243 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 8226 },
	{ 0x714a, 0x714a, PDF_CMAP_SINGLE, 6625 },
	{ 0x714b, 0x714b, PDF_CMAP_SINGLE, 15246 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 2018 },
	{ 0x714d, 0x714d, PDF_CMAP_SINGLE, 15247 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 2136 },
	{ 0x714f, 0x7151, PDF_CMAP_RANGE, 15248 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 9430 },
	{ 0x7153, 0x715b, PDF_CMAP_RANGE, 15251 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 6621 },
	{ 0x715d, 0x715d, PDF_CMAP_SINGLE, 15260 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 3312 },
	{ 0x715f, 0x7161, PDF_CMAP_RANGE, 15261 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 9018 },
	{ 0x7163, 0x7163, PDF_CMAP_SINGLE, 15264 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 2751 },
	{ 0x7165, 0x7165, PDF_CMAP_SINGLE, 15265 },
	{ 0x7166, 0x7166, PDF_CMAP_SINGLE, 6641 },
	{ 0x7167, 0x7167, PDF_CMAP_SINGLE, 4470 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 6622 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 7918 },
	{ 0x716a, 0x716b, PDF_CMAP_RANGE, 15266 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 9429 },
	{ 0x716d, 0x716d, PDF_CMAP_SINGLE, 15268 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 4594 },
	{ 0x716f, 0x7171, PDF_CMAP_RANGE, 15269 },
	{ 0x7172, 0x7172, PDF_CMAP_SINGLE, 6624 },
	{ 0x7173, 0x7173, PDF_CMAP_SINGLE, 6620 },
	{ 0x7174, 0x7177, PDF_CMAP_RANGE, 15272 },
	{ 0x7178, 0x7178, PDF_CMAP_SINGLE, 6626 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 15276 },
	{ 0x717a, 0x717a, PDF_CMAP_SINGLE, 6627 },
	{ 0x717b, 0x717c, PDF_CMAP_RANGE, 15277 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 3320 },
	{ 0x717e, 0x7183, PDF_CMAP_RANGE, 15279 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 3861 },
	{ 0x7185, 0x7189, PDF_CMAP_RANGE, 15285 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 4007 },
	{ 0x718b, 0x718e, PDF_CMAP_RANGE, 15290 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 4053 },
	{ 0x7190, 0x7191, PDF_CMAP_RANGE, 15294 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 8744 },
	{ 0x7193, 0x7193, PDF_CMAP_SINGLE, 15296 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 3254 },
	{ 0x7195, 0x7196, PDF_CMAP_RANGE, 15297 },
	{ 0x7197, 0x7197, PDF_CMAP_SINGLE, 9431 },
	{ 0x7198, 0x7198, PDF_CMAP_SINGLE, 6628 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 3844 },
	{ 0x719a, 0x719e, PDF_CMAP_RANGE, 15299 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 3459 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 6632 },
	{ 0x71a1, 0x71a7, PDF_CMAP_RANGE, 15304 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 6631 },
	{ 0x71a9, 0x71ab, PDF_CMAP_RANGE, 15311 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 969 },
	{ 0x71ad, 0x71b0, PDF_CMAP_RANGE, 15314 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 8444 },
	{ 0x71b2, 0x71b2, PDF_CMAP_SINGLE, 15318 },
	{ 0x71b3, 0x71b3, PDF_CMAP_SINGLE, 6629 },
	{ 0x71b4, 0x71b4, PDF_CMAP_SINGLE, 15319 },
	{ 0x71b5, 0x71b5, PDF_CMAP_SINGLE, 6630 },
	{ 0x71b6, 0x71b8, PDF_CMAP_RANGE, 15320 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 6642 },
	{ 0x71ba, 0x71bd, PDF_CMAP_RANGE, 15323 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 7815 },
	{ 0x71bf, 0x71c0, PDF_CMAP_RANGE, 15327 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 9432 },
	{ 0x71c2, 0x71c2, PDF_CMAP_SINGLE, 15329 },
	{ 0x71c3, 0x71c3, PDF_CMAP_SINGLE, 3223 },
	{ 0x71c4, 0x71c7, PDF_CMAP_RANGE, 15330 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 7866 },
	{ 0x71c9, 0x71cd, PDF_CMAP_RANGE, 15334 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 2576 },
	{ 0x71cf, 0x71d1, PDF_CMAP_RANGE, 15339 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 8473 },
	{ 0x71d3, 0x71d3, PDF_CMAP_SINGLE, 15342 },
	{ 0x71d4, 0x71d4, PDF_CMAP_SINGLE, 6634 },
	{ 0x71d5, 0x71d5, PDF_CMAP_SINGLE, 4105 },
	{ 0x71d6, 0x71d8, PDF_CMAP_RANGE, 15343 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 8556 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 15346 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 9433 },
	{ 0x71dd, 0x71de, PDF_CMAP_RANGE, 15348 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 8743 },
	{ 0x71e0, 0x71e0, PDF_CMAP_SINGLE, 6633 },
	{ 0x71e1, 0x71e4, PDF_CMAP_RANGE, 15350 },
	{ 0x71e5, 0x71e5, PDF_CMAP_SINGLE, 4402 },
	{ 0x71e6, 0x71e6, PDF_CMAP_SINGLE, 7775 },
	{ 0x71e7, 0x71e7, PDF_CMAP_SINGLE, 6635 },
	{ 0x71e8, 0x71ec, PDF_CMAP_RANGE, 15354 },
	{ 0x71ed, 0x71ed, PDF_CMAP_SINGLE, 8863 },
	{ 0x71ee, 0x71ee, PDF_CMAP_SINGLE, 5014 },
	{ 0x71ef, 0x71f3, PDF_CMAP_RANGE, 15359 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 8029 },
	{ 0x71f5, 0x71f8, PDF_CMAP_RANGE, 15364 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 6636 },
	{ 0x71fa, 0x71fb, PDF_CMAP_RANGE, 15368 },
	{ 0x71fc, 0x71fc, PDF_CMAP_SINGLE, 8124 },
	{ 0x71fd, 0x71fd, PDF_CMAP_SINGLE, 15370 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 9434 },
	{ 0x71ff, 0x71ff, PDF_CMAP_SINGLE, 15371 },
	{ 0x7200, 0x7205, PDF_CMAP_RANGE, 15372 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 1045 },
	{ 0x7207, 0x720c, PDF_CMAP_RANGE, 15378 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 8521 },
	{ 0x720e, 0x720f, PDF_CMAP_RANGE, 15384 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 8262 },
	{ 0x7211, 0x721a, PDF_CMAP_RANGE, 15386 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 8192 },
	{ 0x721c, 0x721c, PDF_CMAP_SINGLE, 15396 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 6637 },
	{ 0x721e, 0x7227, PDF_CMAP_RANGE, 15397 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 6638 },
	{ 0x7229, 0x7229, PDF_CMAP_SINGLE, 15407 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 4611 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 15408 },
	{ 0x722c, 0x722c, PDF_CMAP_SINGLE, 2945 },
	{ 0x722d, 0x722f, PDF_CMAP_RANGE, 15409 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 6508 },
	{ 0x7231, 0x7231, PDF_CMAP_SINGLE, 953 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 8592 },
	{ 0x7233, 0x7234, PDF_CMAP_RANGE, 15412 },
	{ 0x7235, 0x7235, PDF_CMAP_SINGLE, 2339 },
	{ 0x7236, 0x7236, PDF_CMAP_SINGLE, 1715 },
	{ 0x7237, 0x7237, PDF_CMAP_SINGLE, 4150 },
	{ 0x7238, 0x7238, PDF_CMAP_SINGLE, 993 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 1498 },
	{ 0x723a, 0x723a, PDF_CMAP_SINGLE, 8712 },
	{ 0x723b, 0x723b, PDF_CMAP_SINGLE, 4713 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 15414 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 3489 },
	{ 0x723e, 0x723e, PDF_CMAP_SINGLE, 7910 },
	{ 0x723f, 0x723f, PDF_CMAP_SINGLE, 5789 },
	{ 0x7240, 0x7246, PDF_CMAP_RANGE, 15415 },
	{ 0x7247, 0x7247, PDF_CMAP_SINGLE, 3019 },
	{ 0x7248, 0x7248, PDF_CMAP_SINGLE, 1009 },
	{ 0x7249, 0x724b, PDF_CMAP_RANGE, 15422 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 2951 },
	{ 0x724d, 0x724d, PDF_CMAP_SINGLE, 6505 },
	{ 0x724e, 0x7251, PDF_CMAP_RANGE, 15425 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 6506 },
	{ 0x7253, 0x7255, PDF_CMAP_RANGE, 15429 },
	{ 0x7256, 0x7256, PDF_CMAP_SINGLE, 6507 },
	{ 0x7257, 0x7257, PDF_CMAP_SINGLE, 15432 },
	{ 0x7258, 0x7258, PDF_CMAP_SINGLE, 9411 },
	{ 0x7259, 0x7259, PDF_CMAP_SINGLE, 4073 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 15433 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 2916 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 15434 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 6468 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 15435 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 2837 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 15436 },
	{ 0x7261, 0x7261, PDF_CMAP_SINGLE, 2840 },
	{ 0x7262, 0x7262, PDF_CMAP_SINGLE, 2489 },
	{ 0x7263, 0x7265, PDF_CMAP_RANGE, 15437 },
	{ 0x7266, 0x7266, PDF_CMAP_SINGLE, 6469 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 2852 },
	{ 0x7268, 0x7268, PDF_CMAP_SINGLE, 15440 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 3838 },
	{ 0x726a, 0x726d, PDF_CMAP_RANGE, 15441 },
	{ 0x726e, 0x726e, PDF_CMAP_SINGLE, 6466 },
	{ 0x726f, 0x726f, PDF_CMAP_SINGLE, 6470 },
	{ 0x7270, 0x7271, PDF_CMAP_RANGE, 15445 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 3381 },
	{ 0x7273, 0x7274, PDF_CMAP_RANGE, 15447 },
	{ 0x7275, 0x7275, PDF_CMAP_SINGLE, 3104 },
	{ 0x7276, 0x7278, PDF_CMAP_RANGE, 15449 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 3627 },
	{ 0x727a, 0x727a, PDF_CMAP_SINGLE, 3853 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 15452 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 8396 },
	{ 0x727e, 0x727f, PDF_CMAP_RANGE, 6471 },
	{ 0x7280, 0x7280, PDF_CMAP_SINGLE, 3865 },
	{ 0x7281, 0x7281, PDF_CMAP_SINGLE, 2514 },
	{ 0x7282, 0x7283, PDF_CMAP_RANGE, 15454 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 6473 },
	{ 0x7285, 0x7289, PDF_CMAP_RANGE, 15456 },
	{ 0x728a, 0x728a, PDF_CMAP_SINGLE, 1534 },
	{ 0x728b, 0x728b, PDF_CMAP_SINGLE, 6474 },
	{ 0x728c, 0x728c, PDF_CMAP_SINGLE, 15461 },
	{ 0x728d, 0x728d, PDF_CMAP_SINGLE, 6475 },
	{ 0x728e, 0x728e, PDF_CMAP_SINGLE, 15462 },
	{ 0x728f, 0x728f, PDF_CMAP_SINGLE, 6476 },
	{ 0x7290, 0x7291, PDF_CMAP_RANGE, 15463 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 6477 },
	{ 0x7293, 0x7295, PDF_CMAP_RANGE, 15465 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 9024 },
	{ 0x7297, 0x729e, PDF_CMAP_RANGE, 15468 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 6467 },
	{ 0x72a0, 0x72a1, PDF_CMAP_RANGE, 15476 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 7889 },
	{ 0x72a3, 0x72a6, PDF_CMAP_RANGE, 15478 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 8620 },
	{ 0x72a8, 0x72ab, PDF_CMAP_RANGE, 15482 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 3209 },
	{ 0x72ad, 0x72ad, PDF_CMAP_SINGLE, 5615 },
	{ 0x72ae, 0x72ae, PDF_CMAP_SINGLE, 15486 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 1621 },
	{ 0x72b0, 0x72b0, PDF_CMAP_SINGLE, 5616 },
	{ 0x72b1, 0x72b3, PDF_CMAP_RANGE, 15487 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 5617 },
	{ 0x72b5, 0x72b5, PDF_CMAP_SINGLE, 15490 },
	{ 0x72b6, 0x72b6, PDF_CMAP_SINGLE, 4625 },
	{ 0x72b7, 0x72b8, PDF_CMAP_RANGE, 5618 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 4273 },
	{ 0x72ba, 0x72bf, PDF_CMAP_RANGE, 15491 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 8879 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 5621 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 2430 },
	{ 0x72c3, 0x72c3, PDF_CMAP_SINGLE, 5620 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 1459 },
	{ 0x72c5, 0x72c7, PDF_CMAP_RANGE, 15497 },
	{ 0x72c8, 0x72c8, PDF_CMAP_SINGLE, 1056 },
	{ 0x72c9, 0x72cc, PDF_CMAP_RANGE, 15500 },
	{ 0x72cd, 0x72cd, PDF_CMAP_SINGLE, 5623 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5622 },
	{ 0x72cf, 0x72cf, PDF_CMAP_SINGLE, 15504 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1971 },
	{ 0x72d1, 0x72d1, PDF_CMAP_SINGLE, 15505 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 5624 },
	{ 0x72d3, 0x72d6, PDF_CMAP_RANGE, 15506 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 1808 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 15510 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 2304 },
	{ 0x72da, 0x72dd, PDF_CMAP_RANGE, 15511 },
	{ 0x72de, 0x72de, PDF_CMAP_SINGLE, 2911 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 15515 },
	{ 0x72e0, 0x72e0, PDF_CMAP_SINGLE, 1940 },
	{ 0x72e1, 0x72e1, PDF_CMAP_SINGLE, 2199 },
	{ 0x72e2, 0x72e7, PDF_CMAP_RANGE, 15516 },
	{ 0x72e8, 0x72e8, PDF_CMAP_SINGLE, 5625 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 5627 },
	{ 0x72ea, 0x72eb, PDF_CMAP_RANGE, 15522 },
	{ 0x72ec, 0x72ec, PDF_CMAP_SINGLE, 1535 },
	{ 0x72ed, 0x72ed, PDF_CMAP_SINGLE, 3886 },
	{ 0x72ee, 0x72ee, PDF_CMAP_SINGLE, 3391 },
	{ 0x72ef, 0x72ef, PDF_CMAP_SINGLE, 5626 },
	{ 0x72f0, 0x72f0, PDF_CMAP_SINGLE, 4505 },
	{ 0x72f1, 0x72f1, PDF_CMAP_SINGLE, 4321 },
	{ 0x72f2, 0x72f2, PDF_CMAP_SINGLE, 5628 },
	{ 0x72f3, 0x72f3, PDF_CMAP_SINGLE, 5632 },
	{ 0x72f4, 0x72f4, PDF_CMAP_SINGLE, 5629 },
	{ 0x72f5, 0x72f6, PDF_CMAP_RANGE, 15524 },
	{ 0x72f7, 0x72f7, PDF_CMAP_SINGLE, 5630 },
	{ 0x72f8, 0x72f8, PDF_CMAP_SINGLE, 2517 },
	{ 0x72f9, 0x72f9, PDF_CMAP_SINGLE, 8630 },
	{ 0x72fa, 0x72fb, PDF_CMAP_RANGE, 5634 },
	{ 0x72fc, 0x72fc, PDF_CMAP_SINGLE, 2482 },
	{ 0x72fd, 0x72fd, PDF_CMAP_SINGLE, 7741 },
	{ 0x72fe, 0x72ff, PDF_CMAP_RANGE, 15526 },
	{ 0x7300, 0x7300, PDF_CMAP_SINGLE, 15528 },
	{ 0x7301, 0x7301, PDF_CMAP_SINGLE, 5631 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 15529 },
	{ 0x7303, 0x7303, PDF_CMAP_SINGLE, 5633 },
	{ 0x7304, 0x7309, PDF_CMAP_RANGE, 15530 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 5639 },
	{ 0x730b, 0x730d, PDF_CMAP_RANGE, 15536 },
	{ 0x730e, 0x730e, PDF_CMAP_SINGLE, 2589 },
	{ 0x730f, 0x7312, PDF_CMAP_RANGE, 15539 },
	{ 0x7313, 0x7313, PDF_CMAP_SINGLE, 5637 },
	{ 0x7314, 0x7314, PDF_CMAP_SINGLE, 15543 },
	{ 0x7315, 0x7315, PDF_CMAP_SINGLE, 5642 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 1221 },
	{ 0x7317, 0x7317, PDF_CMAP_SINGLE, 5636 },
	{ 0x7318, 0x731a, PDF_CMAP_RANGE, 15544 },
	{ 0x731b, 0x731b, PDF_CMAP_SINGLE, 2770 },
	{ 0x731c, 0x731c, PDF_CMAP_SINGLE, 1161 },
	{ 0x731d, 0x731d, PDF_CMAP_SINGLE, 5641 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 5640 },
	{ 0x731f, 0x7320, PDF_CMAP_RANGE, 15547 },
	{ 0x7321, 0x7321, PDF_CMAP_SINGLE, 5638 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 5643 },
	{ 0x7323, 0x7324, PDF_CMAP_RANGE, 15549 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 5645 },
	{ 0x7326, 0x7328, PDF_CMAP_RANGE, 15551 },
	{ 0x7329, 0x7329, PDF_CMAP_SINGLE, 3988 },
	{ 0x732a, 0x732a, PDF_CMAP_SINGLE, 4588 },
	{ 0x732b, 0x732b, PDF_CMAP_SINGLE, 2733 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 5646 },
	{ 0x732d, 0x732d, PDF_CMAP_SINGLE, 15554 },
	{ 0x732e, 0x732e, PDF_CMAP_SINGLE, 3908 },
	{ 0x732f, 0x7330, PDF_CMAP_RANGE, 15555 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 5648 },
	{ 0x7332, 0x7333, PDF_CMAP_RANGE, 15557 },
	{ 0x7334, 0x7334, PDF_CMAP_SINGLE, 1958 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 15559 },
	{ 0x7336, 0x7336, PDF_CMAP_SINGLE, 8757 },
	{ 0x7337, 0x7337, PDF_CMAP_SINGLE, 6356 },
	{ 0x7338, 0x7338, PDF_CMAP_SINGLE, 5647 },
	{ 0x7339, 0x7339, PDF_CMAP_SINGLE, 5644 },
	{ 0x733a, 0x733a, PDF_CMAP_SINGLE, 15560 },
	{ 0x733b, 0x733b, PDF_CMAP_SINGLE, 9111 },
	{ 0x733c, 0x733d, PDF_CMAP_RANGE, 15561 },
	{ 0x733e, 0x733e, PDF_CMAP_SINGLE, 1984 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 4342 },
	{ 0x7340, 0x7340, PDF_CMAP_SINGLE, 15563 },
	{ 0x7341, 0x7341, PDF_CMAP_SINGLE, 9109 },
	{ 0x7342, 0x7343, PDF_CMAP_RANGE, 15564 },
	{ 0x7344, 0x7344, PDF_CMAP_SINGLE, 8767 },
	{ 0x7345, 0x7345, PDF_CMAP_SINGLE, 8490 },
	{ 0x7346, 0x734c, PDF_CMAP_RANGE, 15566 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 5650 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 15573 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 5649 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 15575 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 6357 },
	{ 0x7353, 0x7356, PDF_CMAP_RANGE, 15576 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 5651 },
	{ 0x7358, 0x735f, PDF_CMAP_RANGE, 15580 },
	{ 0x7360, 0x7360, PDF_CMAP_SINGLE, 5652 },
	{ 0x7361, 0x7367, PDF_CMAP_RANGE, 15588 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 7890 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 15595 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 9110 },
	{ 0x736b, 0x736b, PDF_CMAP_SINGLE, 9112 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 5653 },
	{ 0x736d, 0x736d, PDF_CMAP_SINGLE, 3572 },
	{ 0x736e, 0x736e, PDF_CMAP_SINGLE, 15596 },
	{ 0x736f, 0x736f, PDF_CMAP_SINGLE, 5654 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 8354 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 15597 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 8036 },
	{ 0x7373, 0x7374, PDF_CMAP_RANGE, 15598 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 8235 },
	{ 0x7376, 0x7376, PDF_CMAP_SINGLE, 15600 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 9108 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 8505 },
	{ 0x7379, 0x7379, PDF_CMAP_SINGLE, 15601 },
	{ 0x737a, 0x737a, PDF_CMAP_SINGLE, 8543 },
	{ 0x737b, 0x737b, PDF_CMAP_SINGLE, 8641 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 9114 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 15602 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 5655 },
	{ 0x737f, 0x737f, PDF_CMAP_SINGLE, 15603 },
	{ 0x7380, 0x7380, PDF_CMAP_SINGLE, 9113 },
	{ 0x7381, 0x7383, PDF_CMAP_RANGE, 15604 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 4041 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 15607 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 2673 },
	{ 0x7388, 0x7388, PDF_CMAP_SINGLE, 15609 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 4310 },
	{ 0x738a, 0x738a, PDF_CMAP_SINGLE, 15610 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 3750 },
	{ 0x738c, 0x738d, PDF_CMAP_RANGE, 15611 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6168 },
	{ 0x738f, 0x7390, PDF_CMAP_RANGE, 15613 },
	{ 0x7391, 0x7391, PDF_CMAP_SINGLE, 6169 },
	{ 0x7392, 0x7395, PDF_CMAP_RANGE, 15615 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 2288 },
	{ 0x7397, 0x739a, PDF_CMAP_RANGE, 15619 },
	{ 0x739b, 0x739b, PDF_CMAP_SINGLE, 2705 },
	{ 0x739c, 0x739e, PDF_CMAP_RANGE, 15623 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 6172 },
	{ 0x73a0, 0x73a1, PDF_CMAP_RANGE, 15626 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 6171 },
	{ 0x73a3, 0x73a8, PDF_CMAP_RANGE, 15628 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 3735 },
	{ 0x73aa, 0x73aa, PDF_CMAP_SINGLE, 15634 },
	{ 0x73ab, 0x73ab, PDF_CMAP_SINGLE, 2746 },
	{ 0x73ac, 0x73ad, PDF_CMAP_RANGE, 15635 },
	{ 0x73ae, 0x73ae, PDF_CMAP_SINGLE, 6170 },
	{ 0x73af, 0x73af, PDF_CMAP_SINGLE, 1996 },
	{ 0x73b0, 0x73b0, PDF_CMAP_SINGLE, 3907 },
	{ 0x73b1, 0x73b1, PDF_CMAP_SINGLE, 15637 },
	{ 0x73b2, 0x73b2, PDF_CMAP_SINGLE, 2602 },
	{ 0x73b3, 0x73b3, PDF_CMAP_SINGLE, 6177 },
	{ 0x73b4, 0x73b6, PDF_CMAP_RANGE, 15638 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 6176 },
	{ 0x73b8, 0x73b9, PDF_CMAP_RANGE, 15641 },
	{ 0x73ba, 0x73ba, PDF_CMAP_SINGLE, 6188 },
	{ 0x73bb, 0x73bb, PDF_CMAP_SINGLE, 1130 },
	{ 0x73bc, 0x73bf, PDF_CMAP_RANGE, 15643 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 6178 },
	{ 0x73c1, 0x73c1, PDF_CMAP_SINGLE, 15647 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 6174 },
	{ 0x73c3, 0x73c7, PDF_CMAP_RANGE, 15648 },
	{ 0x73c8, 0x73c8, PDF_CMAP_SINGLE, 6180 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 6179 },
	{ 0x73ca, 0x73ca, PDF_CMAP_SINGLE, 3315 },
	{ 0x73cb, 0x73cc, PDF_CMAP_RANGE, 15653 },
	{ 0x73cd, 0x73cd, PDF_CMAP_SINGLE, 4485 },
	{ 0x73ce, 0x73ce, PDF_CMAP_SINGLE, 15655 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 6173 },
	{ 0x73d0, 0x73d0, PDF_CMAP_SINGLE, 1606 },
	{ 0x73d1, 0x73d1, PDF_CMAP_SINGLE, 6175 },
	{ 0x73d2, 0x73d8, PDF_CMAP_RANGE, 15656 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 6182 },
	{ 0x73da, 0x73dd, PDF_CMAP_RANGE, 15663 },
	{ 0x73de, 0x73de, PDF_CMAP_SINGLE, 6187 },
	{ 0x73df, 0x73df, PDF_CMAP_SINGLE, 15667 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 4584 },
	{ 0x73e1, 0x73e4, PDF_CMAP_RANGE, 15668 },
	{ 0x73e5, 0x73e5, PDF_CMAP_SINGLE, 6181 },
	{ 0x73e6, 0x73e6, PDF_CMAP_SINGLE, 15672 },
	{ 0x73e7, 0x73e7, PDF_CMAP_SINGLE, 6186 },
	{ 0x73e8, 0x73e8, PDF_CMAP_SINGLE, 15673 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 6185 },
	{ 0x73ea, 0x73ec, PDF_CMAP_RANGE, 15674 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 1003 },
	{ 0x73ee, 0x73f1, PDF_CMAP_RANGE, 15677 },
	{ 0x73f2, 0x73f2, PDF_CMAP_SINGLE, 6189 },
	{ 0x73f3, 0x73fd, PDF_CMAP_RANGE, 15681 },
	{ 0x73fe, 0x73fe, PDF_CMAP_SINGLE, 8640 },
	{ 0x73ff, 0x73ff, PDF_CMAP_SINGLE, 15692 },
	{ 0x7400, 0x7402, PDF_CMAP_RANGE, 15693 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 3183 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 15696 },
	{ 0x7405, 0x7405, PDF_CMAP_SINGLE, 2480 },
	{ 0x7406, 0x7406, PDF_CMAP_SINGLE, 2520 },
	{ 0x7407, 0x7408, PDF_CMAP_RANGE, 15697 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 2617 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 6184 },
	{ 0x740b, 0x740e, PDF_CMAP_RANGE, 15699 },
	{ 0x740f, 0x740f, PDF_CMAP_SINGLE, 6190 },
	{ 0x7410, 0x7410, PDF_CMAP_SINGLE, 3563 },
	{ 0x7411, 0x7419, PDF_CMAP_RANGE, 15703 },
	{ 0x741a, 0x741a, PDF_CMAP_SINGLE, 6200 },
	{ 0x741b, 0x741b, PDF_CMAP_SINGLE, 6199 },
	{ 0x741c, 0x7421, PDF_CMAP_RANGE, 15712 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 4638 },
	{ 0x7423, 0x7424, PDF_CMAP_RANGE, 15718 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 6194 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 6193 },
	{ 0x7427, 0x7427, PDF_CMAP_SINGLE, 15720 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 6195 },
	{ 0x7429, 0x7429, PDF_CMAP_SINGLE, 15721 },
	{ 0x742a, 0x742a, PDF_CMAP_SINGLE, 6191 },
	{ 0x742b, 0x742b, PDF_CMAP_SINGLE, 15722 },
	{ 0x742c, 0x742c, PDF_CMAP_SINGLE, 6198 },
	{ 0x742d, 0x742d, PDF_CMAP_SINGLE, 15723 },
	{ 0x742e, 0x742e, PDF_CMAP_SINGLE, 6197 },
	{ 0x742f, 0x742f, PDF_CMAP_SINGLE, 15724 },
	{ 0x7430, 0x7430, PDF_CMAP_SINGLE, 6196 },
	{ 0x7431, 0x7432, PDF_CMAP_RANGE, 15725 },
	{ 0x7433, 0x7433, PDF_CMAP_SINGLE, 2590 },
	{ 0x7434, 0x7434, PDF_CMAP_SINGLE, 3158 },
	{ 0x7435, 0x7435, PDF_CMAP_SINGLE, 3006 },
	{ 0x7436, 0x7436, PDF_CMAP_SINGLE, 2948 },
	{ 0x7437, 0x743b, PDF_CMAP_RANGE, 15727 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 3178 },
	{ 0x743d, 0x743e, PDF_CMAP_RANGE, 15732 },
	{ 0x743f, 0x743f, PDF_CMAP_SINGLE, 9312 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 15734 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 6201 },
	{ 0x7442, 0x744a, PDF_CMAP_RANGE, 15735 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 9308 },
	{ 0x744c, 0x7454, PDF_CMAP_RANGE, 15744 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 6204 },
	{ 0x7456, 0x7456, PDF_CMAP_SINGLE, 15753 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 6203 },
	{ 0x7458, 0x7458, PDF_CMAP_SINGLE, 15754 },
	{ 0x7459, 0x7459, PDF_CMAP_SINGLE, 6205 },
	{ 0x745a, 0x745a, PDF_CMAP_SINGLE, 1966 },
	{ 0x745b, 0x745b, PDF_CMAP_SINGLE, 6192 },
	{ 0x745c, 0x745c, PDF_CMAP_SINGLE, 6202 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 15755 },
	{ 0x745e, 0x745e, PDF_CMAP_SINGLE, 3275 },
	{ 0x745f, 0x745f, PDF_CMAP_SINGLE, 3299 },
	{ 0x7460, 0x7462, PDF_CMAP_RANGE, 15756 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 8541 },
	{ 0x7464, 0x7468, PDF_CMAP_RANGE, 15759 },
	{ 0x7469, 0x7469, PDF_CMAP_SINGLE, 8741 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 8298 },
	{ 0x746b, 0x746c, PDF_CMAP_RANGE, 15764 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 6207 },
	{ 0x746e, 0x746f, PDF_CMAP_RANGE, 15766 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 1854 },
	{ 0x7471, 0x7475, PDF_CMAP_RANGE, 15768 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 4135 },
	{ 0x7477, 0x7477, PDF_CMAP_SINGLE, 6206 },
	{ 0x7478, 0x747d, PDF_CMAP_RANGE, 15773 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 6208 },
	{ 0x747f, 0x747f, PDF_CMAP_SINGLE, 15779 },
	{ 0x7480, 0x7481, PDF_CMAP_RANGE, 6211 },
	{ 0x7482, 0x7482, PDF_CMAP_SINGLE, 15780 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 2544 },
	{ 0x7484, 0x7486, PDF_CMAP_RANGE, 15781 },
	{ 0x7487, 0x7487, PDF_CMAP_SINGLE, 6213 },
	{ 0x7488, 0x7488, PDF_CMAP_SINGLE, 15784 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 9313 },
	{ 0x748a, 0x748a, PDF_CMAP_SINGLE, 15785 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 6214 },
	{ 0x748c, 0x748d, PDF_CMAP_RANGE, 15786 },
	{ 0x748e, 0x748e, PDF_CMAP_SINGLE, 6210 },
	{ 0x748f, 0x748f, PDF_CMAP_SINGLE, 15788 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 6218 },
	{ 0x7491, 0x749b, PDF_CMAP_RANGE, 15789 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 6209 },
	{ 0x749d, 0x749d, PDF_CMAP_SINGLE, 15800 },
	{ 0x749e, 0x749e, PDF_CMAP_SINGLE, 6215 },
	{ 0x749f, 0x74a2, PDF_CMAP_RANGE, 15801 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 9307 },
	{ 0x74a4, 0x74a5, PDF_CMAP_RANGE, 15805 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 9314 },
	{ 0x74a7, 0x74a7, PDF_CMAP_SINGLE, 6219 },
	{ 0x74a8, 0x74a9, PDF_CMAP_RANGE, 6216 },
	{ 0x74aa, 0x74af, PDF_CMAP_RANGE, 15807 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 8020 },
	{ 0x74b1, 0x74b9, PDF_CMAP_RANGE, 15813 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 6221 },
	{ 0x74bb, 0x74bc, PDF_CMAP_RANGE, 15822 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 9311 },
	{ 0x74be, 0x74c9, PDF_CMAP_RANGE, 15824 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 8428 },
	{ 0x74cb, 0x74ce, PDF_CMAP_RANGE, 15836 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 9309 },
	{ 0x74d0, 0x74d1, PDF_CMAP_RANGE, 15840 },
	{ 0x74d2, 0x74d2, PDF_CMAP_SINGLE, 6220 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 15842 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 9315 },
	{ 0x74d5, 0x74d9, PDF_CMAP_RANGE, 15843 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 9316 },
	{ 0x74db, 0x74db, PDF_CMAP_SINGLE, 15848 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 1832 },
	{ 0x74dd, 0x74dd, PDF_CMAP_SINGLE, 15849 },
	{ 0x74de, 0x74de, PDF_CMAP_SINGLE, 6962 },
	{ 0x74df, 0x74df, PDF_CMAP_SINGLE, 15850 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 6963 },
	{ 0x74e1, 0x74e1, PDF_CMAP_SINGLE, 15851 },
	{ 0x74e2, 0x74e2, PDF_CMAP_SINGLE, 3023 },
	{ 0x74e3, 0x74e3, PDF_CMAP_SINGLE, 1013 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 3226 },
	{ 0x74e5, 0x74e5, PDF_CMAP_SINGLE, 15852 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 3728 },
	{ 0x74e7, 0x74ed, PDF_CMAP_RANGE, 15853 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 3804 },
	{ 0x74ef, 0x74ef, PDF_CMAP_SINGLE, 6403 },
	{ 0x74f0, 0x74f3, PDF_CMAP_RANGE, 15860 },
	{ 0x74f4, 0x74f4, PDF_CMAP_SINGLE, 6404 },
	{ 0x74f5, 0x74f5, PDF_CMAP_SINGLE, 15864 },
	{ 0x74f6, 0x74f6, PDF_CMAP_SINGLE, 3038 },
	{ 0x74f7, 0x74f7, PDF_CMAP_SINGLE, 1356 },
	{ 0x74f8, 0x74fe, PDF_CMAP_RANGE, 15865 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 6405 },
	{ 0x7500, 0x7503, PDF_CMAP_RANGE, 15872 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 4488 },
	{ 0x7505, 0x750b, PDF_CMAP_RANGE, 15876 },
	{ 0x750c, 0x750c, PDF_CMAP_SINGLE, 9381 },
	{ 0x750d, 0x750d, PDF_CMAP_SINGLE, 5253 },
	{ 0x750e, 0x750e, PDF_CMAP_SINGLE, 15883 },
	{ 0x750f, 0x750f, PDF_CMAP_SINGLE, 6406 },
	{ 0x7510, 0x7510, PDF_CMAP_SINGLE, 15884 },
	{ 0x7511, 0x7511, PDF_CMAP_SINGLE, 6407 },
	{ 0x7512, 0x7512, PDF_CMAP_SINGLE, 15885 },
	{ 0x7513, 0x7513, PDF_CMAP_SINGLE, 6408 },
	{ 0x7514, 0x7517, PDF_CMAP_RANGE, 15886 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 1733 },
	{ 0x7519, 0x7519, PDF_CMAP_SINGLE, 5368 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 3374 },
	{ 0x751b, 0x751b, PDF_CMAP_SINGLE, 15890 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 3651 },
	{ 0x751d, 0x751e, PDF_CMAP_RANGE, 15891 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 3379 },
	{ 0x7520, 0x7522, PDF_CMAP_RANGE, 15893 },
	{ 0x7523, 0x7523, PDF_CMAP_SINGLE, 7792 },
	{ 0x7524, 0x7524, PDF_CMAP_SINGLE, 15896 },
	{ 0x7525, 0x7525, PDF_CMAP_SINGLE, 3380 },
	{ 0x7526, 0x7527, PDF_CMAP_RANGE, 15897 },
	{ 0x7528, 0x7528, PDF_CMAP_SINGLE, 4264 },
	{ 0x7529, 0x7529, PDF_CMAP_SINGLE, 3483 },
	{ 0x752a, 0x752a, PDF_CMAP_SINGLE, 15899 },
	{ 0x752b, 0x752b, PDF_CMAP_SINGLE, 1697 },
	{ 0x752c, 0x752c, PDF_CMAP_SINGLE, 6964 },
	{ 0x752d, 0x752d, PDF_CMAP_SINGLE, 1067 },
	{ 0x752e, 0x752e, PDF_CMAP_SINGLE, 15900 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 5940 },
	{ 0x7530, 0x7530, PDF_CMAP_SINGLE, 3650 },
	{ 0x7531, 0x7531, PDF_CMAP_SINGLE, 4270 },
	{ 0x7532, 0x7532, PDF_CMAP_SINGLE, 2122 },
	{ 0x7533, 0x7533, PDF_CMAP_SINGLE, 3363 },
	{ 0x7534, 0x7534, PDF_CMAP_SINGLE, 15901 },
	{ 0x7535, 0x7535, PDF_CMAP_SINGLE, 1480 },
	{ 0x7536, 0x7536, PDF_CMAP_SINGLE, 15902 },
	{ 0x7537, 0x7537, PDF_CMAP_SINGLE, 2867 },
	{ 0x7538, 0x7538, PDF_CMAP_SINGLE, 1482 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 15903 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 6776 },
	{ 0x753b, 0x753b, PDF_CMAP_SINGLE, 1986 },
	{ 0x753c, 0x753d, PDF_CMAP_RANGE, 15904 },
	{ 0x753e, 0x753e, PDF_CMAP_SINGLE, 6166 },
	{ 0x753f, 0x753f, PDF_CMAP_SINGLE, 15906 },
	{ 0x7540, 0x7540, PDF_CMAP_SINGLE, 6777 },
	{ 0x7541, 0x7544, PDF_CMAP_RANGE, 15907 },
	{ 0x7545, 0x7545, PDF_CMAP_SINGLE, 1230 },
	{ 0x7546, 0x7547, PDF_CMAP_RANGE, 15911 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 6780 },
	{ 0x7549, 0x754a, PDF_CMAP_RANGE, 15913 },
	{ 0x754b, 0x754b, PDF_CMAP_SINGLE, 6779 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 2232 },
	{ 0x754d, 0x754d, PDF_CMAP_SINGLE, 15915 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 6778 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 3782 },
	{ 0x7550, 0x7553, PDF_CMAP_RANGE, 15916 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 2960 },
	{ 0x7555, 0x7558, PDF_CMAP_RANGE, 15920 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 2621 },
	{ 0x755a, 0x755a, PDF_CMAP_SINGLE, 5022 },
	{ 0x755b, 0x755b, PDF_CMAP_SINGLE, 6781 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 4030 },
	{ 0x755d, 0x755d, PDF_CMAP_SINGLE, 8335 },
	{ 0x755e, 0x7561, PDF_CMAP_RANGE, 15924 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 7746 },
	{ 0x7563, 0x7564, PDF_CMAP_RANGE, 15928 },
	{ 0x7565, 0x7565, PDF_CMAP_SINGLE, 2683 },
	{ 0x7566, 0x7566, PDF_CMAP_SINGLE, 3079 },
	{ 0x7567, 0x7569, PDF_CMAP_RANGE, 15930 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 1609 },
	{ 0x756b, 0x756b, PDF_CMAP_SINGLE, 8014 },
	{ 0x756c, 0x7571, PDF_CMAP_RANGE, 15933 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 6782 },
	{ 0x7573, 0x7573, PDF_CMAP_SINGLE, 15939 },
	{ 0x7574, 0x7574, PDF_CMAP_SINGLE, 1296 },
	{ 0x7575, 0x7575, PDF_CMAP_SINGLE, 15940 },
	{ 0x7576, 0x7576, PDF_CMAP_SINGLE, 7857 },
	{ 0x7577, 0x7577, PDF_CMAP_SINGLE, 15941 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 2064 },
	{ 0x7579, 0x7579, PDF_CMAP_SINGLE, 6783 },
	{ 0x757a, 0x757e, PDF_CMAP_RANGE, 15942 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 6164 },
	{ 0x7580, 0x7582, PDF_CMAP_RANGE, 15947 },
	{ 0x7583, 0x7583, PDF_CMAP_SINGLE, 6784 },
	{ 0x7584, 0x7585, PDF_CMAP_RANGE, 15950 },
	{ 0x7586, 0x7586, PDF_CMAP_SINGLE, 2175 },
	{ 0x7587, 0x7587, PDF_CMAP_SINGLE, 7819 },
	{ 0x7588, 0x758a, PDF_CMAP_RANGE, 15952 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 7110 },
	{ 0x758c, 0x758e, PDF_CMAP_RANGE, 15955 },
	{ 0x758f, 0x758f, PDF_CMAP_SINGLE, 3455 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 15958 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 4176 },
	{ 0x7592, 0x7592, PDF_CMAP_SINGLE, 7008 },
	{ 0x7593, 0x7593, PDF_CMAP_SINGLE, 15959 },
	{ 0x7594, 0x7594, PDF_CMAP_SINGLE, 7009 },
	{ 0x7595, 0x7595, PDF_CMAP_SINGLE, 15960 },
	{ 0x7596, 0x7596, PDF_CMAP_SINGLE, 7010 },
	{ 0x7597, 0x7597, PDF_CMAP_SINGLE, 2575 },
	{ 0x7598, 0x7598, PDF_CMAP_SINGLE, 15961 },
	{ 0x7599, 0x7599, PDF_CMAP_SINGLE, 1768 },
	{ 0x759a, 0x759a, PDF_CMAP_SINGLE, 2301 },
	{ 0x759b, 0x759c, PDF_CMAP_RANGE, 15962 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 7012 },
	{ 0x759e, 0x759e, PDF_CMAP_SINGLE, 15964 },
	{ 0x759f, 0x759f, PDF_CMAP_SINGLE, 2930 },
	{ 0x75a0, 0x75a0, PDF_CMAP_SINGLE, 7011 },
	{ 0x75a1, 0x75a1, PDF_CMAP_SINGLE, 4122 },
	{ 0x75a2, 0x75a2, PDF_CMAP_SINGLE, 15965 },
	{ 0x75a3, 0x75a3, PDF_CMAP_SINGLE, 7014 },
	{ 0x75a4, 0x75a4, PDF_CMAP_SINGLE, 983 },
	{ 0x75a5, 0x75a5, PDF_CMAP_SINGLE, 2235 },
	{ 0x75a6, 0x75aa, PDF_CMAP_RANGE, 15966 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 4198 },
	{ 0x75ac, 0x75ac, PDF_CMAP_SINGLE, 7013 },
	{ 0x75ad, 0x75ad, PDF_CMAP_SINGLE, 15971 },
	{ 0x75ae, 0x75ae, PDF_CMAP_SINGLE, 1330 },
	{ 0x75af, 0x75af, PDF_CMAP_SINGLE, 1669 },
	{ 0x75b0, 0x75b0, PDF_CMAP_SINGLE, 7020 },
	{ 0x75b1, 0x75b1, PDF_CMAP_SINGLE, 7019 },
	{ 0x75b2, 0x75b2, PDF_CMAP_SINGLE, 3010 },
	{ 0x75b3, 0x75b4, PDF_CMAP_RANGE, 7015 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 1350 },
	{ 0x75b6, 0x75b7, PDF_CMAP_RANGE, 15972 },
	{ 0x75b8, 0x75b8, PDF_CMAP_SINGLE, 7017 },
	{ 0x75b9, 0x75b9, PDF_CMAP_SINGLE, 4495 },
	{ 0x75ba, 0x75bb, PDF_CMAP_RANGE, 15974 },
	{ 0x75bc, 0x75bc, PDF_CMAP_SINGLE, 3630 },
	{ 0x75bd, 0x75bd, PDF_CMAP_SINGLE, 2305 },
	{ 0x75be, 0x75be, PDF_CMAP_SINGLE, 2085 },
	{ 0x75bf, 0x75c1, PDF_CMAP_RANGE, 15976 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 7022 },
	{ 0x75c3, 0x75c3, PDF_CMAP_SINGLE, 7021 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 7018 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 1128 },
	{ 0x75c6, 0x75c6, PDF_CMAP_SINGLE, 15979 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 4513 },
	{ 0x75c8, 0x75c8, PDF_CMAP_SINGLE, 4253 },
	{ 0x75c9, 0x75c9, PDF_CMAP_SINGLE, 2278 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 3207 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 15980 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 7024 },
	{ 0x75ce, 0x75d1, PDF_CMAP_RANGE, 15982 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 4128 },
	{ 0x75d3, 0x75d3, PDF_CMAP_SINGLE, 15986 },
	{ 0x75d4, 0x75d4, PDF_CMAP_SINGLE, 4555 },
	{ 0x75d5, 0x75d5, PDF_CMAP_SINGLE, 1938 },
	{ 0x75d6, 0x75d6, PDF_CMAP_SINGLE, 7023 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 15987 },
	{ 0x75d8, 0x75d8, PDF_CMAP_SINGLE, 1530 },
	{ 0x75d9, 0x75d9, PDF_CMAP_SINGLE, 8134 },
	{ 0x75da, 0x75da, PDF_CMAP_SINGLE, 15988 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 3685 },
	{ 0x75dc, 0x75dd, PDF_CMAP_RANGE, 15989 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 3013 },
	{ 0x75df, 0x75e1, PDF_CMAP_RANGE, 15991 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 2538 },
	{ 0x75e3, 0x75e3, PDF_CMAP_SINGLE, 7025 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 7028 },
	{ 0x75e5, 0x75e5, PDF_CMAP_SINGLE, 15994 },
	{ 0x75e6, 0x75e6, PDF_CMAP_SINGLE, 7027 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 7030 },
	{ 0x75e8, 0x75e8, PDF_CMAP_SINGLE, 7026 },
	{ 0x75e9, 0x75e9, PDF_CMAP_SINGLE, 15995 },
	{ 0x75ea, 0x75ea, PDF_CMAP_SINGLE, 2003 },
	{ 0x75eb, 0x75eb, PDF_CMAP_SINGLE, 7029 },
	{ 0x75ec, 0x75ef, PDF_CMAP_RANGE, 15996 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 3592 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 7032 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 16000 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 1274 },
	{ 0x75f5, 0x75f8, PDF_CMAP_RANGE, 16002 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 1085 },
	{ 0x75fa, 0x75fb, PDF_CMAP_RANGE, 16006 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 7033 },
	{ 0x75fd, 0x75fe, PDF_CMAP_RANGE, 16008 },
	{ 0x75ff, 0x75ff, PDF_CMAP_SINGLE, 7034 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 7036 },
	{ 0x7601, 0x7601, PDF_CMAP_SINGLE, 1380 },
	{ 0x7602, 0x7602, PDF_CMAP_SINGLE, 9646 },
	{ 0x7603, 0x7603, PDF_CMAP_SINGLE, 7031 },
	{ 0x7604, 0x7604, PDF_CMAP_SINGLE, 16010 },
	{ 0x7605, 0x7605, PDF_CMAP_SINGLE, 7037 },
	{ 0x7606, 0x7609, PDF_CMAP_RANGE, 16011 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 7040 },
	{ 0x760b, 0x760b, PDF_CMAP_SINGLE, 7937 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 7038 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 8704 },
	{ 0x760e, 0x760f, PDF_CMAP_RANGE, 16015 },
	{ 0x7610, 0x7610, PDF_CMAP_SINGLE, 7035 },
	{ 0x7611, 0x7614, PDF_CMAP_RANGE, 16017 },
	{ 0x7615, 0x7615, PDF_CMAP_SINGLE, 7043 },
	{ 0x7616, 0x7616, PDF_CMAP_SINGLE, 16021 },
	{ 0x7617, 0x7617, PDF_CMAP_SINGLE, 7039 },
	{ 0x7618, 0x7618, PDF_CMAP_SINGLE, 7042 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 7044 },
	{ 0x761a, 0x761a, PDF_CMAP_SINGLE, 16022 },
	{ 0x761b, 0x761b, PDF_CMAP_SINGLE, 7045 },
	{ 0x761c, 0x761d, PDF_CMAP_RANGE, 16023 },
	{ 0x761e, 0x761e, PDF_CMAP_SINGLE, 9650 },
	{ 0x761f, 0x761f, PDF_CMAP_SINGLE, 3792 },
	{ 0x7620, 0x7620, PDF_CMAP_SINGLE, 7048 },
	{ 0x7621, 0x7621, PDF_CMAP_SINGLE, 7831 },
	{ 0x7622, 0x7622, PDF_CMAP_SINGLE, 7047 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 16025 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 2623 },
	{ 0x7625, 0x7625, PDF_CMAP_SINGLE, 7041 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 3444 },
	{ 0x7627, 0x7627, PDF_CMAP_SINGLE, 8363 },
	{ 0x7628, 0x7628, PDF_CMAP_SINGLE, 16026 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 1396 },
	{ 0x762a, 0x762a, PDF_CMAP_SINGLE, 1114 },
	{ 0x762b, 0x762b, PDF_CMAP_SINGLE, 3588 },
	{ 0x762c, 0x762c, PDF_CMAP_SINGLE, 16027 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 7050 },
	{ 0x762e, 0x762f, PDF_CMAP_RANGE, 16028 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 7051 },
	{ 0x7631, 0x7632, PDF_CMAP_RANGE, 16030 },
	{ 0x7633, 0x7633, PDF_CMAP_SINGLE, 7056 },
	{ 0x7634, 0x7634, PDF_CMAP_SINGLE, 4463 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 7053 },
	{ 0x7636, 0x7637, PDF_CMAP_RANGE, 16032 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 3214 },
	{ 0x7639, 0x763a, PDF_CMAP_RANGE, 16034 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 9651 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 7046 },
	{ 0x763d, 0x763d, PDF_CMAP_SINGLE, 16036 },
	{ 0x763e, 0x763e, PDF_CMAP_SINGLE, 7055 },
	{ 0x763f, 0x763f, PDF_CMAP_SINGLE, 7052 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 7049 },
	{ 0x7641, 0x7641, PDF_CMAP_SINGLE, 16037 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 8232 },
	{ 0x7643, 0x7643, PDF_CMAP_SINGLE, 7054 },
	{ 0x7644, 0x7645, PDF_CMAP_RANGE, 16038 },
	{ 0x7646, 0x7647, PDF_CMAP_RANGE, 9647 },
	{ 0x7648, 0x7648, PDF_CMAP_SINGLE, 16040 },
	{ 0x7649, 0x7649, PDF_CMAP_SINGLE, 9649 },
	{ 0x764a, 0x764b, PDF_CMAP_RANGE, 16041 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 948 },
	{ 0x764d, 0x764d, PDF_CMAP_SINGLE, 7057 },
	{ 0x764e, 0x7653, PDF_CMAP_RANGE, 16043 },
	{ 0x7654, 0x7654, PDF_CMAP_SINGLE, 7059 },
	{ 0x7655, 0x7655, PDF_CMAP_SINGLE, 16049 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 7061 },
	{ 0x7657, 0x7657, PDF_CMAP_SINGLE, 16050 },
	{ 0x7658, 0x7658, PDF_CMAP_SINGLE, 9644 },
	{ 0x7659, 0x765b, PDF_CMAP_RANGE, 16051 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 7060 },
	{ 0x765d, 0x765d, PDF_CMAP_SINGLE, 16054 },
	{ 0x765e, 0x765e, PDF_CMAP_SINGLE, 7058 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 7758 },
	{ 0x7660, 0x7661, PDF_CMAP_RANGE, 16055 },
	{ 0x7662, 0x7662, PDF_CMAP_SINGLE, 8706 },
	{ 0x7663, 0x7663, PDF_CMAP_SINGLE, 4043 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 9643 },
	{ 0x7665, 0x7665, PDF_CMAP_SINGLE, 8838 },
	{ 0x7666, 0x7666, PDF_CMAP_SINGLE, 16057 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 9645 },
	{ 0x7668, 0x7668, PDF_CMAP_SINGLE, 16058 },
	{ 0x7669, 0x7669, PDF_CMAP_SINGLE, 9654 },
	{ 0x766a, 0x766a, PDF_CMAP_SINGLE, 16059 },
	{ 0x766b, 0x766b, PDF_CMAP_SINGLE, 7062 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 8675 },
	{ 0x766d, 0x766e, PDF_CMAP_RANGE, 9652 },
	{ 0x766f, 0x766f, PDF_CMAP_SINGLE, 7063 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 8751 },
	{ 0x7671, 0x7671, PDF_CMAP_SINGLE, 8549 },
	{ 0x7672, 0x7672, PDF_CMAP_SINGLE, 9655 },
	{ 0x7673, 0x7677, PDF_CMAP_RANGE, 16060 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 1864 },
	{ 0x7679, 0x767a, PDF_CMAP_RANGE, 16065 },
	{ 0x767b, 0x767b, PDF_CMAP_SINGLE, 1448 },
	{ 0x767c, 0x767c, PDF_CMAP_SINGLE, 7913 },
	{ 0x767d, 0x767d, PDF_CMAP_SINGLE, 994 },
	{ 0x767e, 0x767e, PDF_CMAP_SINGLE, 996 },
	{ 0x767f, 0x7681, PDF_CMAP_RANGE, 16067 },
	{ 0x7682, 0x7682, PDF_CMAP_SINGLE, 4400 },
	{ 0x7683, 0x7683, PDF_CMAP_SINGLE, 16070 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 1445 },
	{ 0x7685, 0x7685, PDF_CMAP_SINGLE, 16071 },
	{ 0x7686, 0x7686, PDF_CMAP_SINGLE, 2213 },
	{ 0x7687, 0x7687, PDF_CMAP_SINGLE, 2015 },
	{ 0x7688, 0x7688, PDF_CMAP_SINGLE, 6957 },
	{ 0x7689, 0x768a, PDF_CMAP_RANGE, 16072 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 1753 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 16074 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 6958 },
	{ 0x768f, 0x7690, PDF_CMAP_RANGE, 16076 },
	{ 0x7691, 0x7691, PDF_CMAP_SINGLE, 947 },
	{ 0x7692, 0x7692, PDF_CMAP_SINGLE, 16078 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 6959 },
	{ 0x7694, 0x7695, PDF_CMAP_RANGE, 16079 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 3743 },
	{ 0x7697, 0x7698, PDF_CMAP_RANGE, 16081 },
	{ 0x7699, 0x7699, PDF_CMAP_SINGLE, 6960 },
	{ 0x769a, 0x769a, PDF_CMAP_SINGLE, 7717 },
	{ 0x769b, 0x76a3, PDF_CMAP_RANGE, 16083 },
	{ 0x76a4, 0x76a4, PDF_CMAP_SINGLE, 6961 },
	{ 0x76a5, 0x76ad, PDF_CMAP_RANGE, 16092 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3011 },
	{ 0x76af, 0x76b0, PDF_CMAP_RANGE, 16101 },
	{ 0x76b1, 0x76b1, PDF_CMAP_SINGLE, 4580 },
	{ 0x76b2, 0x76b2, PDF_CMAP_SINGLE, 7112 },
	{ 0x76b3, 0x76b3, PDF_CMAP_SINGLE, 16103 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 7113 },
	{ 0x76b5, 0x76b7, PDF_CMAP_RANGE, 16104 },
	{ 0x76b8, 0x76b8, PDF_CMAP_SINGLE, 9664 },
	{ 0x76b9, 0x76b9, PDF_CMAP_SINGLE, 16107 },
	{ 0x76ba, 0x76ba, PDF_CMAP_SINGLE, 8858 },
	{ 0x76bb, 0x76be, PDF_CMAP_RANGE, 16108 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2808 },
	{ 0x76c0, 0x76c1, PDF_CMAP_RANGE, 16112 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 4288 },
	{ 0x76c3, 0x76c4, PDF_CMAP_RANGE, 16114 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 4560 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 2985 },
	{ 0x76c7, 0x76c7, PDF_CMAP_SINGLE, 16116 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 4244 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 16117 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 4205 },
	{ 0x76cb, 0x76cc, PDF_CMAP_RANGE, 16118 },
	{ 0x76cd, 0x76cd, PDF_CMAP_SINGLE, 6795 },
	{ 0x76ce, 0x76ce, PDF_CMAP_SINGLE, 966 },
	{ 0x76cf, 0x76cf, PDF_CMAP_SINGLE, 4438 },
	{ 0x76d0, 0x76d0, PDF_CMAP_SINGLE, 4087 },
	{ 0x76d1, 0x76d1, PDF_CMAP_SINGLE, 2131 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 1927 },
	{ 0x76d3, 0x76d3, PDF_CMAP_SINGLE, 16120 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 2437 },
	{ 0x76d5, 0x76d5, PDF_CMAP_SINGLE, 16121 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 1730 },
	{ 0x76d7, 0x76d7, PDF_CMAP_SINGLE, 1442 },
	{ 0x76d8, 0x76d8, PDF_CMAP_SINGLE, 2957 },
	{ 0x76d9, 0x76da, PDF_CMAP_RANGE, 16122 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 3385 },
	{ 0x76dc, 0x76dd, PDF_CMAP_RANGE, 16124 },
	{ 0x76de, 0x76de, PDF_CMAP_SINGLE, 8814 },
	{ 0x76df, 0x76df, PDF_CMAP_SINGLE, 2768 },
	{ 0x76e0, 0x76e0, PDF_CMAP_SINGLE, 16126 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 8125 },
	{ 0x76e2, 0x76e2, PDF_CMAP_SINGLE, 16127 },
	{ 0x76e3, 0x76e3, PDF_CMAP_SINGLE, 8068 },
	{ 0x76e4, 0x76e4, PDF_CMAP_SINGLE, 8370 },
	{ 0x76e5, 0x76e5, PDF_CMAP_SINGLE, 6796 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 16128 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 8259 },
	{ 0x76e8, 0x76ed, PDF_CMAP_RANGE, 16129 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 2850 },
	{ 0x76ef, 0x76ef, PDF_CMAP_SINGLE, 1505 },
	{ 0x76f0, 0x76f0, PDF_CMAP_SINGLE, 16135 },
	{ 0x76f1, 0x76f1, PDF_CMAP_SINGLE, 6744 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 2729 },
	{ 0x76f3, 0x76f3, PDF_CMAP_SINGLE, 16136 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 4528 },
	{ 0x76f5, 0x76f7, PDF_CMAP_RANGE, 16137 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 3917 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 6747 },
	{ 0x76fa, 0x76fb, PDF_CMAP_RANGE, 16140 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 2959 },
	{ 0x76fd, 0x76fd, PDF_CMAP_SINGLE, 16142 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 1563 },
	{ 0x76ff, 0x76ff, PDF_CMAP_SINGLE, 16143 },
	{ 0x7700, 0x7700, PDF_CMAP_SINGLE, 16144 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 3384 },
	{ 0x7702, 0x7703, PDF_CMAP_RANGE, 16145 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 6745 },
	{ 0x7705, 0x7706, PDF_CMAP_RANGE, 16147 },
	{ 0x7707, 0x7708, PDF_CMAP_RANGE, 6748 },
	{ 0x7709, 0x7709, PDF_CMAP_SINGLE, 2753 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 16149 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 2369 },
	{ 0x770c, 0x770c, PDF_CMAP_SINGLE, 16150 },
	{ 0x770d, 0x770d, PDF_CMAP_SINGLE, 6746 },
	{ 0x770e, 0x7718, PDF_CMAP_RANGE, 16151 },
	{ 0x7719, 0x7719, PDF_CMAP_SINGLE, 6752 },
	{ 0x771a, 0x771a, PDF_CMAP_SINGLE, 6750 },
	{ 0x771b, 0x771e, PDF_CMAP_RANGE, 16162 },
	{ 0x771f, 0x771f, PDF_CMAP_SINGLE, 4487 },
	{ 0x7720, 0x7720, PDF_CMAP_SINGLE, 2788 },
	{ 0x7721, 0x7721, PDF_CMAP_SINGLE, 16166 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 6751 },
	{ 0x7723, 0x7725, PDF_CMAP_RANGE, 16167 },
	{ 0x7726, 0x7726, PDF_CMAP_SINGLE, 6754 },
	{ 0x7727, 0x7727, PDF_CMAP_SINGLE, 16170 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 4420 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 4044 },
	{ 0x772a, 0x772c, PDF_CMAP_RANGE, 16171 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 6753 },
	{ 0x772e, 0x772e, PDF_CMAP_SINGLE, 16174 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 2773 },
	{ 0x7730, 0x7734, PDF_CMAP_RANGE, 16175 },
	{ 0x7735, 0x7735, PDF_CMAP_SINGLE, 6755 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 2433 },
	{ 0x7737, 0x7737, PDF_CMAP_SINGLE, 2331 },
	{ 0x7738, 0x7738, PDF_CMAP_SINGLE, 6756 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 16180 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 3658 },
	{ 0x773b, 0x773b, PDF_CMAP_SINGLE, 16181 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 4100 },
	{ 0x773d, 0x773f, PDF_CMAP_RANGE, 16182 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 4642 },
	{ 0x7741, 0x7741, PDF_CMAP_SINGLE, 4503 },
	{ 0x7742, 0x7742, PDF_CMAP_SINGLE, 16185 },
	{ 0x7743, 0x7743, PDF_CMAP_SINGLE, 6760 },
	{ 0x7744, 0x7746, PDF_CMAP_RANGE, 16186 },
	{ 0x7747, 0x7747, PDF_CMAP_SINGLE, 6759 },
	{ 0x7748, 0x774e, PDF_CMAP_RANGE, 16189 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 9860 },
	{ 0x7750, 0x7751, PDF_CMAP_RANGE, 6757 },
	{ 0x7752, 0x7759, PDF_CMAP_RANGE, 16196 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 6761 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 2261 },
	{ 0x775c, 0x775d, PDF_CMAP_RANGE, 16204 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 9455 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 16206 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 3492 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 6763 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 1532 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 16208 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 6764 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 2851 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 16209 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 6762 },
	{ 0x7769, 0x776a, PDF_CMAP_RANGE, 16210 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 2223 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 1166 },
	{ 0x776d, 0x7778, PDF_CMAP_RANGE, 16212 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 1538 },
	{ 0x777a, 0x777c, PDF_CMAP_RANGE, 16224 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 6767 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 4720 },
	{ 0x777f, 0x777f, PDF_CMAP_SINGLE, 6765 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 6768 },
	{ 0x7781, 0x7783, PDF_CMAP_RANGE, 16227 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 2798 },
	{ 0x7785, 0x7785, PDF_CMAP_SINGLE, 1303 },
	{ 0x7786, 0x778b, PDF_CMAP_RANGE, 16230 },
	{ 0x778c, 0x778c, PDF_CMAP_SINGLE, 6769 },
	{ 0x778d, 0x778d, PDF_CMAP_SINGLE, 6766 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 3878 },
	{ 0x778f, 0x7790, PDF_CMAP_RANGE, 16236 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 6770 },
	{ 0x7792, 0x7792, PDF_CMAP_SINGLE, 2718 },
	{ 0x7793, 0x7797, PDF_CMAP_RANGE, 16238 },
	{ 0x7798, 0x7798, PDF_CMAP_SINGLE, 9454 },
	{ 0x7799, 0x779d, PDF_CMAP_RANGE, 16243 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 8308 },
	{ 0x779f, 0x77a0, PDF_CMAP_RANGE, 6771 },
	{ 0x77a1, 0x77a1, PDF_CMAP_SINGLE, 16248 },
	{ 0x77a2, 0x77a2, PDF_CMAP_SINGLE, 5269 },
	{ 0x77a3, 0x77a4, PDF_CMAP_RANGE, 16249 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 3026 },
	{ 0x77a6, 0x77a6, PDF_CMAP_SINGLE, 16251 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 3139 },
	{ 0x77a8, 0x77a8, PDF_CMAP_SINGLE, 16252 },
	{ 0x77a9, 0x77a9, PDF_CMAP_SINGLE, 4596 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 1450 },
	{ 0x77ab, 0x77ab, PDF_CMAP_SINGLE, 16253 },
	{ 0x77ac, 0x77ac, PDF_CMAP_SINGLE, 3495 },
	{ 0x77ad, 0x77ad, PDF_CMAP_SINGLE, 9863 },
	{ 0x77ae, 0x77af, PDF_CMAP_RANGE, 16254 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 6773 },
	{ 0x77b1, 0x77b2, PDF_CMAP_RANGE, 16256 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 3676 },
	{ 0x77b4, 0x77b4, PDF_CMAP_SINGLE, 16258 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 6774 },
	{ 0x77b6, 0x77ba, PDF_CMAP_RANGE, 16259 },
	{ 0x77bb, 0x77bb, PDF_CMAP_SINGLE, 4433 },
	{ 0x77bc, 0x77bc, PDF_CMAP_SINGLE, 9456 },
	{ 0x77bd, 0x77bd, PDF_CMAP_SINGLE, 6775 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 16264 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 7550 },
	{ 0x77c0, 0x77c6, PDF_CMAP_RANGE, 16265 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 9869 },
	{ 0x77c8, 0x77cc, PDF_CMAP_RANGE, 16272 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5015 },
	{ 0x77ce, 0x77d6, PDF_CMAP_RANGE, 16277 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 1318 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 16286 },
	{ 0x77da, 0x77da, PDF_CMAP_SINGLE, 8864 },
	{ 0x77db, 0x77db, PDF_CMAP_SINGLE, 2737 },
	{ 0x77dc, 0x77dc, PDF_CMAP_SINGLE, 7114 },
	{ 0x77dd, 0x77e1, PDF_CMAP_RANGE, 16288 },
	{ 0x77e2, 0x77e2, PDF_CMAP_SINGLE, 3407 },
	{ 0x77e3, 0x77e3, PDF_CMAP_SINGLE, 4186 },
	{ 0x77e4, 0x77e4, PDF_CMAP_SINGLE, 16293 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 4521 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 16294 },
	{ 0x77e7, 0x77e7, PDF_CMAP_SINGLE, 6938 },
	{ 0x77e8, 0x77e8, PDF_CMAP_SINGLE, 16295 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 2311 },
	{ 0x77ea, 0x77ea, PDF_CMAP_SINGLE, 16296 },
	{ 0x77eb, 0x77eb, PDF_CMAP_SINGLE, 2196 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 6939 },
	{ 0x77ed, 0x77ed, PDF_CMAP_SINGLE, 1547 },
	{ 0x77ee, 0x77ee, PDF_CMAP_SINGLE, 950 },
	{ 0x77ef, 0x77ef, PDF_CMAP_SINGLE, 8107 },
	{ 0x77f0, 0x77f2, PDF_CMAP_RANGE, 16297 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 3398 },
	{ 0x77f4, 0x77f5, PDF_CMAP_RANGE, 16300 },
	{ 0x77f6, 0x77f6, PDF_CMAP_SINGLE, 6690 },
	{ 0x77f7, 0x77f7, PDF_CMAP_SINGLE, 16302 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 6691 },
	{ 0x77f9, 0x77fc, PDF_CMAP_RANGE, 16303 },
	{ 0x77fd, 0x77fd, PDF_CMAP_SINGLE, 3848 },
	{ 0x77fe, 0x77fe, PDF_CMAP_SINGLE, 1612 },
	{ 0x77ff, 0x77ff, PDF_CMAP_SINGLE, 2432 },
	{ 0x7800, 0x7800, PDF_CMAP_SINGLE, 6692 },
	{ 0x7801, 0x7801, PDF_CMAP_SINGLE, 2706 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 3305 },
	{ 0x7803, 0x7808, PDF_CMAP_RANGE, 16307 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 6693 },
	{ 0x780a, 0x780b, PDF_CMAP_RANGE, 16313 },
	{ 0x780c, 0x780c, PDF_CMAP_SINGLE, 3093 },
	{ 0x780d, 0x780d, PDF_CMAP_SINGLE, 2368 },
	{ 0x780e, 0x7810, PDF_CMAP_RANGE, 16315 },
	{ 0x7811, 0x7811, PDF_CMAP_SINGLE, 6696 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 3001 },
	{ 0x7813, 0x7813, PDF_CMAP_SINGLE, 16318 },
	{ 0x7814, 0x7814, PDF_CMAP_SINGLE, 4089 },
	{ 0x7815, 0x7815, PDF_CMAP_SINGLE, 16319 },
	{ 0x7816, 0x7816, PDF_CMAP_SINGLE, 4614 },
	{ 0x7817, 0x7818, PDF_CMAP_RANGE, 6694 },
	{ 0x7819, 0x7819, PDF_CMAP_SINGLE, 16320 },
	{ 0x781a, 0x781a, PDF_CMAP_SINGLE, 4107 },
	{ 0x781b, 0x781b, PDF_CMAP_SINGLE, 16321 },
	{ 0x781c, 0x781d, PDF_CMAP_RANGE, 6699 },
	{ 0x781e, 0x781e, PDF_CMAP_SINGLE, 16322 },
	{ 0x781f, 0x781f, PDF_CMAP_SINGLE, 6704 },
	{ 0x7820, 0x7822, PDF_CMAP_RANGE, 16323 },
	{ 0x7823, 0x7823, PDF_CMAP_SINGLE, 6708 },
	{ 0x7824, 0x7824, PDF_CMAP_SINGLE, 16326 },
	{ 0x7825, 0x7825, PDF_CMAP_SINGLE, 6706 },
	{ 0x7826, 0x7826, PDF_CMAP_SINGLE, 6714 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 4489 },
	{ 0x7828, 0x7828, PDF_CMAP_SINGLE, 16327 },
	{ 0x7829, 0x7829, PDF_CMAP_SINGLE, 6709 },
	{ 0x782a, 0x782b, PDF_CMAP_RANGE, 16328 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 6707 },
	{ 0x782d, 0x782d, PDF_CMAP_SINGLE, 6698 },
	{ 0x782e, 0x782f, PDF_CMAP_RANGE, 16330 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 2986 },
	{ 0x7831, 0x7833, PDF_CMAP_RANGE, 16332 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 3045 },
	{ 0x7835, 0x7836, PDF_CMAP_RANGE, 16335 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 3362 },
	{ 0x7838, 0x7838, PDF_CMAP_SINGLE, 4373 },
	{ 0x7839, 0x783b, PDF_CMAP_RANGE, 6701 },
	{ 0x783c, 0x783c, PDF_CMAP_SINGLE, 6705 },
	{ 0x783d, 0x783d, PDF_CMAP_SINGLE, 16337 },
	{ 0x783e, 0x783e, PDF_CMAP_SINGLE, 2532 },
	{ 0x783f, 0x783f, PDF_CMAP_SINGLE, 16338 },
	{ 0x7840, 0x7840, PDF_CMAP_SINGLE, 1316 },
	{ 0x7841, 0x7842, PDF_CMAP_RANGE, 16339 },
	{ 0x7843, 0x7843, PDF_CMAP_SINGLE, 9896 },
	{ 0x7844, 0x7844, PDF_CMAP_SINGLE, 16341 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 1857 },
	{ 0x7846, 0x7846, PDF_CMAP_SINGLE, 16342 },
	{ 0x7847, 0x7847, PDF_CMAP_SINGLE, 6716 },
	{ 0x7848, 0x784b, PDF_CMAP_RANGE, 16343 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 6717 },
	{ 0x784d, 0x784d, PDF_CMAP_SINGLE, 16347 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 6710 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 16348 },
	{ 0x7850, 0x7850, PDF_CMAP_SINGLE, 6715 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 16349 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 3847 },
	{ 0x7853, 0x7854, PDF_CMAP_RANGE, 16350 },
	{ 0x7855, 0x7855, PDF_CMAP_SINGLE, 3499 },
	{ 0x7856, 0x7857, PDF_CMAP_RANGE, 6712 },
	{ 0x7858, 0x785c, PDF_CMAP_RANGE, 16352 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3938 },
	{ 0x785e, 0x7863, PDF_CMAP_RANGE, 16357 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 9449 },
	{ 0x7865, 0x7867, PDF_CMAP_RANGE, 16363 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 9445 },
	{ 0x7869, 0x7869, PDF_CMAP_SINGLE, 16366 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 6718 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 2619 },
	{ 0x786c, 0x786c, PDF_CMAP_SINGLE, 4247 },
	{ 0x786d, 0x786d, PDF_CMAP_SINGLE, 6711 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 3218 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 8698 },
	{ 0x7870, 0x7876, PDF_CMAP_RANGE, 16367 },
	{ 0x7877, 0x7877, PDF_CMAP_SINGLE, 2146 },
	{ 0x7878, 0x787b, PDF_CMAP_RANGE, 16374 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 2993 },
	{ 0x787d, 0x7886, PDF_CMAP_RANGE, 16378 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 6722 },
	{ 0x7888, 0x7888, PDF_CMAP_SINGLE, 16388 },
	{ 0x7889, 0x7889, PDF_CMAP_SINGLE, 1488 },
	{ 0x788a, 0x788b, PDF_CMAP_RANGE, 16389 },
	{ 0x788c, 0x788c, PDF_CMAP_SINGLE, 2652 },
	{ 0x788d, 0x788d, PDF_CMAP_SINGLE, 952 },
	{ 0x788e, 0x788e, PDF_CMAP_SINGLE, 3550 },
	{ 0x788f, 0x7890, PDF_CMAP_RANGE, 16391 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 1047 },
	{ 0x7892, 0x7892, PDF_CMAP_SINGLE, 16393 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 6720 },
	{ 0x7894, 0x7896, PDF_CMAP_RANGE, 16394 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 3740 },
	{ 0x7898, 0x7898, PDF_CMAP_SINGLE, 1475 },
	{ 0x7899, 0x7899, PDF_CMAP_SINGLE, 16397 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 6721 },
	{ 0x789b, 0x789b, PDF_CMAP_SINGLE, 6719 },
	{ 0x789c, 0x789c, PDF_CMAP_SINGLE, 6723 },
	{ 0x789d, 0x789e, PDF_CMAP_RANGE, 16398 },
	{ 0x789f, 0x789f, PDF_CMAP_SINGLE, 1499 },
	{ 0x78a0, 0x78a0, PDF_CMAP_SINGLE, 16400 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 6724 },
	{ 0x78a2, 0x78a2, PDF_CMAP_SINGLE, 16401 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 6725 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 16402 },
	{ 0x78a5, 0x78a5, PDF_CMAP_SINGLE, 6728 },
	{ 0x78a6, 0x78a6, PDF_CMAP_SINGLE, 16403 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 1077 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 16404 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 8520 },
	{ 0x78aa, 0x78ac, PDF_CMAP_RANGE, 16405 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 9444 },
	{ 0x78ae, 0x78af, PDF_CMAP_RANGE, 16408 },
	{ 0x78b0, 0x78b0, PDF_CMAP_SINGLE, 2999 },
	{ 0x78b1, 0x78b1, PDF_CMAP_SINGLE, 2145 },
	{ 0x78b2, 0x78b2, PDF_CMAP_SINGLE, 6726 },
	{ 0x78b3, 0x78b3, PDF_CMAP_SINGLE, 3599 },
	{ 0x78b4, 0x78b4, PDF_CMAP_SINGLE, 1201 },
	{ 0x78b5, 0x78b7, PDF_CMAP_RANGE, 16410 },
	{ 0x78b8, 0x78b8, PDF_CMAP_SINGLE, 9446 },
	{ 0x78b9, 0x78b9, PDF_CMAP_SINGLE, 6727 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 8439 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 16413 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 8299 },
	{ 0x78bd, 0x78bd, PDF_CMAP_SINGLE, 16414 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 2894 },
	{ 0x78bf, 0x78c0, PDF_CMAP_RANGE, 16415 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 1352 },
	{ 0x78c2, 0x78c4, PDF_CMAP_RANGE, 16417 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 1024 },
	{ 0x78c6, 0x78c8, PDF_CMAP_RANGE, 16420 },
	{ 0x78c9, 0x78c9, PDF_CMAP_SINGLE, 6731 },
	{ 0x78ca, 0x78ca, PDF_CMAP_SINGLE, 2501 },
	{ 0x78cb, 0x78cb, PDF_CMAP_SINGLE, 1387 },
	{ 0x78cc, 0x78cf, PDF_CMAP_RANGE, 16423 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 2958 },
	{ 0x78d1, 0x78d3, PDF_CMAP_RANGE, 16427 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 6729 },
	{ 0x78d5, 0x78d5, PDF_CMAP_SINGLE, 2385 },
	{ 0x78d6, 0x78d8, PDF_CMAP_RANGE, 16430 },
	{ 0x78d9, 0x78d9, PDF_CMAP_SINGLE, 6730 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 8871 },
	{ 0x78db, 0x78e2, PDF_CMAP_RANGE, 16433 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 9452 },
	{ 0x78e4, 0x78e6, PDF_CMAP_RANGE, 16441 },
	{ 0x78e7, 0x78e7, PDF_CMAP_SINGLE, 9451 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 2824 },
	{ 0x78e9, 0x78eb, PDF_CMAP_RANGE, 16444 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 6732 },
	{ 0x78ed, 0x78ee, PDF_CMAP_RANGE, 16447 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 9443 },
	{ 0x78f0, 0x78f1, PDF_CMAP_RANGE, 16449 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 6733 },
	{ 0x78f3, 0x78f3, PDF_CMAP_SINGLE, 16451 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 6735 },
	{ 0x78f5, 0x78f6, PDF_CMAP_RANGE, 16452 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 2592 },
	{ 0x78f8, 0x78f9, PDF_CMAP_RANGE, 16454 },
	{ 0x78fa, 0x78fa, PDF_CMAP_SINGLE, 2012 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 16456 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 9450 },
	{ 0x78fe, 0x78ff, PDF_CMAP_RANGE, 16458 },
	{ 0x7900, 0x7900, PDF_CMAP_SINGLE, 16460 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2185 },
	{ 0x7902, 0x7904, PDF_CMAP_RANGE, 16461 },
	{ 0x7905, 0x7905, PDF_CMAP_SINGLE, 6734 },
	{ 0x7906, 0x790d, PDF_CMAP_RANGE, 16464 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 7826 },
	{ 0x790f, 0x7912, PDF_CMAP_RANGE, 16472 },
	{ 0x7913, 0x7913, PDF_CMAP_SINGLE, 6736 },
	{ 0x7914, 0x7918, PDF_CMAP_RANGE, 16476 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 7719 },
	{ 0x791a, 0x791d, PDF_CMAP_RANGE, 16481 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 6738 },
	{ 0x791f, 0x7923, PDF_CMAP_RANGE, 16485 },
	{ 0x7924, 0x7924, PDF_CMAP_SINGLE, 6737 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 16490 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 8166 },
	{ 0x7927, 0x7929, PDF_CMAP_RANGE, 16491 },
	{ 0x792a, 0x792a, PDF_CMAP_SINGLE, 9447 },
	{ 0x792b, 0x792b, PDF_CMAP_SINGLE, 8210 },
	{ 0x792c, 0x792c, PDF_CMAP_SINGLE, 7916 },
	{ 0x792d, 0x7930, PDF_CMAP_RANGE, 16494 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 9448 },
	{ 0x7932, 0x7933, PDF_CMAP_RANGE, 16498 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 6739 },
	{ 0x7935, 0x7939, PDF_CMAP_RANGE, 16500 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 3413 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 6648 },
	{ 0x793c, 0x793c, PDF_CMAP_SINGLE, 2524 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 16505 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 3360 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 16506 },
	{ 0x7940, 0x7940, PDF_CMAP_SINGLE, 6649 },
	{ 0x7941, 0x7941, PDF_CMAP_SINGLE, 3085 },
	{ 0x7942, 0x7945, PDF_CMAP_RANGE, 16507 },
	{ 0x7946, 0x7946, PDF_CMAP_SINGLE, 6650 },
	{ 0x7947, 0x7947, PDF_CMAP_SINGLE, 16511 },
	{ 0x7948, 0x7948, PDF_CMAP_SINGLE, 3084 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 6651 },
	{ 0x794a, 0x7952, PDF_CMAP_RANGE, 16512 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 6654 },
	{ 0x7954, 0x7955, PDF_CMAP_RANGE, 16521 },
	{ 0x7956, 0x7956, PDF_CMAP_SINGLE, 4675 },
	{ 0x7957, 0x7957, PDF_CMAP_SINGLE, 6657 },
	{ 0x7958, 0x7959, PDF_CMAP_RANGE, 16523 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 6655 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 6652 },
	{ 0x795d, 0x795d, PDF_CMAP_SINGLE, 4608 },
	{ 0x795e, 0x795e, PDF_CMAP_SINGLE, 3370 },
	{ 0x795f, 0x795f, PDF_CMAP_SINGLE, 3555 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 6658 },
	{ 0x7961, 0x7961, PDF_CMAP_SINGLE, 16525 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 6656 },
	{ 0x7963, 0x7964, PDF_CMAP_RANGE, 16526 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 3926 },
	{ 0x7966, 0x7966, PDF_CMAP_SINGLE, 16528 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 6660 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 3024 },
	{ 0x7969, 0x796c, PDF_CMAP_RANGE, 16529 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2099 },
	{ 0x796e, 0x796e, PDF_CMAP_SINGLE, 16533 },
	{ 0x796f, 0x796f, PDF_CMAP_SINGLE, 6659 },
	{ 0x7970, 0x7976, PDF_CMAP_RANGE, 16534 },
	{ 0x7977, 0x7977, PDF_CMAP_SINGLE, 1436 },
	{ 0x7978, 0x7978, PDF_CMAP_SINGLE, 2059 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 16541 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 6661 },
	{ 0x797b, 0x797f, PDF_CMAP_RANGE, 16542 },
	{ 0x7980, 0x7980, PDF_CMAP_SINGLE, 4875 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 2252 },
	{ 0x7982, 0x7983, PDF_CMAP_RANGE, 16547 },
	{ 0x7984, 0x7984, PDF_CMAP_SINGLE, 2658 },
	{ 0x7985, 0x7985, PDF_CMAP_SINGLE, 6662 },
	{ 0x7986, 0x7989, PDF_CMAP_RANGE, 16549 },
	{ 0x798a, 0x798a, PDF_CMAP_SINGLE, 6663 },
	{ 0x798b, 0x798c, PDF_CMAP_RANGE, 16553 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 8038 },
	{ 0x798e, 0x798e, PDF_CMAP_SINGLE, 9436 },
	{ 0x798f, 0x798f, PDF_CMAP_SINGLE, 1694 },
	{ 0x7990, 0x7999, PDF_CMAP_RANGE, 16555 },
	{ 0x799a, 0x799a, PDF_CMAP_SINGLE, 6664 },
	{ 0x799b, 0x79a5, PDF_CMAP_RANGE, 16565 },
	{ 0x79a6, 0x79a6, PDF_CMAP_SINGLE, 9890 },
	{ 0x79a7, 0x79a7, PDF_CMAP_SINGLE, 6665 },
	{ 0x79a8, 0x79a9, PDF_CMAP_RANGE, 16576 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 9437 },
	{ 0x79ab, 0x79ad, PDF_CMAP_RANGE, 16578 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 8206 },
	{ 0x79af, 0x79af, PDF_CMAP_SINGLE, 16581 },
	{ 0x79b0, 0x79b0, PDF_CMAP_SINGLE, 9435 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 7864 },
	{ 0x79b2, 0x79b2, PDF_CMAP_SINGLE, 16582 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 6666 },
	{ 0x79b4, 0x79b8, PDF_CMAP_RANGE, 16583 },
	{ 0x79b9, 0x79b9, PDF_CMAP_SINGLE, 4306 },
	{ 0x79ba, 0x79ba, PDF_CMAP_SINGLE, 4708 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 2518 },
	{ 0x79bc, 0x79bc, PDF_CMAP_SINGLE, 16588 },
	{ 0x79bd, 0x79bd, PDF_CMAP_SINGLE, 3162 },
	{ 0x79be, 0x79be, PDF_CMAP_SINGLE, 1923 },
	{ 0x79bf, 0x79bf, PDF_CMAP_SINGLE, 16589 },
	{ 0x79c0, 0x79c0, PDF_CMAP_SINGLE, 4014 },
	{ 0x79c1, 0x79c1, PDF_CMAP_SINGLE, 3506 },
	{ 0x79c2, 0x79c2, PDF_CMAP_SINGLE, 16590 },
	{ 0x79c3, 0x79c3, PDF_CMAP_SINGLE, 3691 },
	{ 0x79c4, 0x79c5, PDF_CMAP_RANGE, 16591 },
	{ 0x79c6, 0x79c6, PDF_CMAP_SINGLE, 1740 },
	{ 0x79c7, 0x79c8, PDF_CMAP_RANGE, 16593 },
	{ 0x79c9, 0x79c9, PDF_CMAP_SINGLE, 1125 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 16595 },
	{ 0x79cb, 0x79cb, PDF_CMAP_SINGLE, 3180 },
	{ 0x79cc, 0x79cc, PDF_CMAP_SINGLE, 16596 },
	{ 0x79cd, 0x79cd, PDF_CMAP_SINGLE, 4565 },
	{ 0x79ce, 0x79d0, PDF_CMAP_RANGE, 16597 },
	{ 0x79d1, 0x79d1, PDF_CMAP_SINGLE, 2387 },
	{ 0x79d2, 0x79d2, PDF_CMAP_SINGLE, 2800 },
	{ 0x79d3, 0x79d4, PDF_CMAP_RANGE, 16600 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 6941 },
	{ 0x79d6, 0x79d7, PDF_CMAP_RANGE, 16602 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2781 },
	{ 0x79d9, 0x79de, PDF_CMAP_RANGE, 16604 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 4671 },
	{ 0x79e0, 0x79e2, PDF_CMAP_RANGE, 16610 },
	{ 0x79e3, 0x79e3, PDF_CMAP_SINGLE, 6943 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 1272 },
	{ 0x79e5, 0x79e5, PDF_CMAP_SINGLE, 16613 },
	{ 0x79e6, 0x79e6, PDF_CMAP_SINGLE, 3157 },
	{ 0x79e7, 0x79e7, PDF_CMAP_SINGLE, 4118 },
	{ 0x79e8, 0x79e8, PDF_CMAP_SINGLE, 16614 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 4551 },
	{ 0x79ea, 0x79ea, PDF_CMAP_SINGLE, 16615 },
	{ 0x79eb, 0x79eb, PDF_CMAP_SINGLE, 6944 },
	{ 0x79ec, 0x79ec, PDF_CMAP_SINGLE, 16616 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 6942 },
	{ 0x79ee, 0x79ee, PDF_CMAP_SINGLE, 16617 },
	{ 0x79ef, 0x79ef, PDF_CMAP_SINGLE, 2066 },
	{ 0x79f0, 0x79f0, PDF_CMAP_SINGLE, 1259 },
	{ 0x79f1, 0x79f7, PDF_CMAP_RANGE, 16618 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 2214 },
	{ 0x79f9, 0x79fa, PDF_CMAP_RANGE, 16625 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 4173 },
	{ 0x79fc, 0x79fc, PDF_CMAP_SINGLE, 16627 },
	{ 0x79fd, 0x79fd, PDF_CMAP_SINGLE, 2037 },
	{ 0x79fe, 0x79ff, PDF_CMAP_RANGE, 16628 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 3854 },
	{ 0x7a01, 0x7a01, PDF_CMAP_SINGLE, 16630 },
	{ 0x7a02, 0x7a02, PDF_CMAP_SINGLE, 6948 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 6947 },
	{ 0x7a04, 0x7a05, PDF_CMAP_RANGE, 16631 },
	{ 0x7a06, 0x7a06, PDF_CMAP_SINGLE, 6945 },
	{ 0x7a07, 0x7a0a, PDF_CMAP_RANGE, 16633 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 1265 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 16637 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 3341 },
	{ 0x7a0e, 0x7a0e, PDF_CMAP_SINGLE, 3493 },
	{ 0x7a0f, 0x7a13, PDF_CMAP_RANGE, 16638 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 6950 },
	{ 0x7a15, 0x7a16, PDF_CMAP_RANGE, 16643 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 1001 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 16645 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 4552 },
	{ 0x7a1b, 0x7a1d, PDF_CMAP_RANGE, 16647 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 6949 },
	{ 0x7a1f, 0x7a1f, PDF_CMAP_SINGLE, 16650 },
	{ 0x7a20, 0x7a20, PDF_CMAP_SINGLE, 1298 },
	{ 0x7a21, 0x7a22, PDF_CMAP_RANGE, 16651 },
	{ 0x7a23, 0x7a23, PDF_CMAP_SINGLE, 7567 },
	{ 0x7a24, 0x7a2d, PDF_CMAP_RANGE, 16653 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 8853 },
	{ 0x7a2f, 0x7a30, PDF_CMAP_RANGE, 16663 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 7808 },
	{ 0x7a32, 0x7a32, PDF_CMAP_SINGLE, 16665 },
	{ 0x7a33, 0x7a33, PDF_CMAP_SINGLE, 3799 },
	{ 0x7a34, 0x7a36, PDF_CMAP_RANGE, 16666 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 6952 },
	{ 0x7a38, 0x7a38, PDF_CMAP_SINGLE, 16669 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 6951 },
	{ 0x7a3a, 0x7a3a, PDF_CMAP_SINGLE, 16670 },
	{ 0x7a3b, 0x7a3b, PDF_CMAP_SINGLE, 1439 },
	{ 0x7a3c, 0x7a3c, PDF_CMAP_SINGLE, 2125 },
	{ 0x7a3d, 0x7a3d, PDF_CMAP_SINGLE, 2065 },
	{ 0x7a3e, 0x7a3e, PDF_CMAP_SINGLE, 16671 },
	{ 0x7a3f, 0x7a3f, PDF_CMAP_SINGLE, 1760 },
	{ 0x7a40, 0x7a40, PDF_CMAP_SINGLE, 9847 },
	{ 0x7a41, 0x7a45, PDF_CMAP_RANGE, 16672 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 2853 },
	{ 0x7a47, 0x7a4b, PDF_CMAP_RANGE, 16677 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 9764 },
	{ 0x7a4d, 0x7a4d, PDF_CMAP_SINGLE, 8041 },
	{ 0x7a4e, 0x7a4e, PDF_CMAP_SINGLE, 8747 },
	{ 0x7a4f, 0x7a50, PDF_CMAP_RANGE, 16682 },
	{ 0x7a51, 0x7a51, PDF_CMAP_SINGLE, 6953 },
	{ 0x7a52, 0x7a56, PDF_CMAP_RANGE, 16684 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 3552 },
	{ 0x7a58, 0x7a60, PDF_CMAP_RANGE, 16689 },
	{ 0x7a61, 0x7a61, PDF_CMAP_SINGLE, 9599 },
	{ 0x7a62, 0x7a62, PDF_CMAP_SINGLE, 8027 },
	{ 0x7a63, 0x7a68, PDF_CMAP_RANGE, 16698 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 8603 },
	{ 0x7a6a, 0x7a6a, PDF_CMAP_SINGLE, 16704 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 9853 },
	{ 0x7a6c, 0x7a6f, PDF_CMAP_RANGE, 16705 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 6956 },
	{ 0x7a71, 0x7a73, PDF_CMAP_RANGE, 16709 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 4049 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 16712 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 2286 },
	{ 0x7a77, 0x7a77, PDF_CMAP_SINGLE, 3179 },
	{ 0x7a78, 0x7a79, PDF_CMAP_RANGE, 7066 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 2402 },
	{ 0x7a7b, 0x7a7e, PDF_CMAP_RANGE, 16713 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 1324 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 7068 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 3692 },
	{ 0x7a82, 0x7a82, PDF_CMAP_SINGLE, 16717 },
	{ 0x7a83, 0x7a83, PDF_CMAP_SINGLE, 3153 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 4430 },
	{ 0x7a85, 0x7a85, PDF_CMAP_SINGLE, 16718 },
	{ 0x7a86, 0x7a86, PDF_CMAP_SINGLE, 7069 },
	{ 0x7a87, 0x7a87, PDF_CMAP_SINGLE, 16719 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 7070 },
	{ 0x7a89, 0x7a8c, PDF_CMAP_RANGE, 16720 },
	{ 0x7a8d, 0x7a8d, PDF_CMAP_SINGLE, 3148 },
	{ 0x7a8e, 0x7a90, PDF_CMAP_RANGE, 16724 },
	{ 0x7a91, 0x7a91, PDF_CMAP_SINGLE, 4139 },
	{ 0x7a92, 0x7a92, PDF_CMAP_SINGLE, 4558 },
	{ 0x7a93, 0x7a94, PDF_CMAP_RANGE, 16727 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 7071 },
	{ 0x7a96, 0x7a96, PDF_CMAP_SINGLE, 2210 },
	{ 0x7a97, 0x7a97, PDF_CMAP_SINGLE, 1331 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 2284 },
	{ 0x7a99, 0x7a9b, PDF_CMAP_RANGE, 16729 },
	{ 0x7a9c, 0x7a9c, PDF_CMAP_SINGLE, 1375 },
	{ 0x7a9d, 0x7a9d, PDF_CMAP_SINGLE, 3808 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 16732 },
	{ 0x7a9f, 0x7a9f, PDF_CMAP_SINGLE, 2412 },
	{ 0x7aa0, 0x7aa0, PDF_CMAP_SINGLE, 7073 },
	{ 0x7aa1, 0x7aa4, PDF_CMAP_RANGE, 16733 },
	{ 0x7aa5, 0x7aa5, PDF_CMAP_SINGLE, 2439 },
	{ 0x7aa6, 0x7aa6, PDF_CMAP_SINGLE, 7072 },
	{ 0x7aa7, 0x7aa7, PDF_CMAP_SINGLE, 16737 },
	{ 0x7aa8, 0x7aa8, PDF_CMAP_SINGLE, 7075 },
	{ 0x7aa9, 0x7aa9, PDF_CMAP_SINGLE, 8608 },
	{ 0x7aaa, 0x7aaa, PDF_CMAP_SINGLE, 8582 },
	{ 0x7aab, 0x7aab, PDF_CMAP_SINGLE, 16738 },
	{ 0x7aac, 0x7aac, PDF_CMAP_SINGLE, 7074 },
	{ 0x7aad, 0x7aad, PDF_CMAP_SINGLE, 7076 },
	{ 0x7aae, 0x7aae, PDF_CMAP_SINGLE, 8429 },
	{ 0x7aaf, 0x7ab2, PDF_CMAP_RANGE, 16739 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 7077 },
	{ 0x7ab4, 0x7ab5, PDF_CMAP_RANGE, 16743 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 9657 },
	{ 0x7ab7, 0x7ab9, PDF_CMAP_RANGE, 16745 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 8170 },
	{ 0x7abb, 0x7abe, PDF_CMAP_RANGE, 16748 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 2631 },
	{ 0x7ac0, 0x7ac3, PDF_CMAP_RANGE, 16752 },
	{ 0x7ac4, 0x7ac4, PDF_CMAP_SINGLE, 7844 },
	{ 0x7ac5, 0x7ac5, PDF_CMAP_SINGLE, 8417 },
	{ 0x7ac6, 0x7ac6, PDF_CMAP_SINGLE, 16756 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 9656 },
	{ 0x7ac8, 0x7ac8, PDF_CMAP_SINGLE, 8800 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 16757 },
	{ 0x7aca, 0x7aca, PDF_CMAP_SINGLE, 8418 },
	{ 0x7acb, 0x7acb, PDF_CMAP_SINGLE, 2539 },
	{ 0x7acc, 0x7ad5, PDF_CMAP_RANGE, 16758 },
	{ 0x7ad6, 0x7ad6, PDF_CMAP_SINGLE, 3473 },
	{ 0x7ad7, 0x7ad8, PDF_CMAP_RANGE, 16768 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 4447 },
	{ 0x7ada, 0x7add, PDF_CMAP_RANGE, 16770 },
	{ 0x7ade, 0x7ade, PDF_CMAP_SINGLE, 2281 },
	{ 0x7adf, 0x7adf, PDF_CMAP_SINGLE, 2280 },
	{ 0x7ae0, 0x7ae0, PDF_CMAP_SINGLE, 4451 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 16774 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 2351 },
	{ 0x7ae4, 0x7ae4, PDF_CMAP_SINGLE, 16776 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 3680 },
	{ 0x7ae6, 0x7ae6, PDF_CMAP_SINGLE, 7065 },
	{ 0x7ae7, 0x7ae9, PDF_CMAP_RANGE, 16777 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 8513 },
	{ 0x7aeb, 0x7aec, PDF_CMAP_RANGE, 16780 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 2224 },
	{ 0x7aee, 0x7aee, PDF_CMAP_SINGLE, 16782 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 1546 },
	{ 0x7af0, 0x7af5, PDF_CMAP_RANGE, 16783 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 8135 },
	{ 0x7af7, 0x7af8, PDF_CMAP_RANGE, 16789 },
	{ 0x7af9, 0x7af9, PDF_CMAP_SINGLE, 4592 },
	{ 0x7afa, 0x7afa, PDF_CMAP_SINGLE, 7267 },
	{ 0x7afb, 0x7afc, PDF_CMAP_RANGE, 16791 },
	{ 0x7afd, 0x7afd, PDF_CMAP_SINGLE, 7268 },
	{ 0x7afe, 0x7afe, PDF_CMAP_SINGLE, 16793 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1736 },
	{ 0x7b00, 0x7b02, PDF_CMAP_RANGE, 16794 },
	{ 0x7b03, 0x7b04, PDF_CMAP_RANGE, 7270 },
	{ 0x7b05, 0x7b05, PDF_CMAP_SINGLE, 16797 },
	{ 0x7b06, 0x7b06, PDF_CMAP_SINGLE, 981 },
	{ 0x7b07, 0x7b07, PDF_CMAP_SINGLE, 16798 },
	{ 0x7b08, 0x7b08, PDF_CMAP_SINGLE, 7269 },
	{ 0x7b09, 0x7b09, PDF_CMAP_SINGLE, 16799 },
	{ 0x7b0a, 0x7b0a, PDF_CMAP_SINGLE, 7273 },
	{ 0x7b0b, 0x7b0b, PDF_CMAP_SINGLE, 3558 },
	{ 0x7b0c, 0x7b0e, PDF_CMAP_RANGE, 16800 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 7275 },
	{ 0x7b10, 0x7b10, PDF_CMAP_SINGLE, 16803 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 3953 },
	{ 0x7b12, 0x7b13, PDF_CMAP_RANGE, 16804 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 1075 },
	{ 0x7b15, 0x7b15, PDF_CMAP_SINGLE, 7272 },
	{ 0x7b16, 0x7b18, PDF_CMAP_RANGE, 16806 },
	{ 0x7b19, 0x7b19, PDF_CMAP_SINGLE, 7279 },
	{ 0x7b1a, 0x7b1a, PDF_CMAP_SINGLE, 16809 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 1458 },
	{ 0x7b1c, 0x7b1d, PDF_CMAP_RANGE, 16810 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 7287 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 16812 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 7282 },
	{ 0x7b21, 0x7b23, PDF_CMAP_RANGE, 16813 },
	{ 0x7b24, 0x7b24, PDF_CMAP_SINGLE, 7284 },
	{ 0x7b25, 0x7b25, PDF_CMAP_SINGLE, 7283 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 1688 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 16816 },
	{ 0x7b28, 0x7b28, PDF_CMAP_SINGLE, 1064 },
	{ 0x7b29, 0x7b29, PDF_CMAP_SINGLE, 16817 },
	{ 0x7b2a, 0x7b2a, PDF_CMAP_SINGLE, 7278 },
	{ 0x7b2b, 0x7b2b, PDF_CMAP_SINGLE, 7274 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 1467 },
	{ 0x7b2d, 0x7b2d, PDF_CMAP_SINGLE, 16818 },
	{ 0x7b2e, 0x7b2e, PDF_CMAP_SINGLE, 7280 },
	{ 0x7b2f, 0x7b30, PDF_CMAP_RANGE, 16819 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 7281 },
	{ 0x7b32, 0x7b32, PDF_CMAP_SINGLE, 16821 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 7285 },
	{ 0x7b34, 0x7b37, PDF_CMAP_RANGE, 16822 },
	{ 0x7b38, 0x7b38, PDF_CMAP_SINGLE, 7277 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 16826 },
	{ 0x7b3a, 0x7b3a, PDF_CMAP_SINGLE, 2134 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 16827 },
	{ 0x7b3c, 0x7b3c, PDF_CMAP_SINGLE, 2630 },
	{ 0x7b3d, 0x7b3d, PDF_CMAP_SINGLE, 16828 },
	{ 0x7b3e, 0x7b3e, PDF_CMAP_SINGLE, 7286 },
	{ 0x7b3f, 0x7b44, PDF_CMAP_RANGE, 16829 },
	{ 0x7b45, 0x7b45, PDF_CMAP_SINGLE, 7290 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 7745 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 7276 },
	{ 0x7b48, 0x7b48, PDF_CMAP_SINGLE, 16835 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 1449 },
	{ 0x7b4a, 0x7b4a, PDF_CMAP_SINGLE, 16836 },
	{ 0x7b4b, 0x7b4b, PDF_CMAP_SINGLE, 2239 },
	{ 0x7b4c, 0x7b4c, PDF_CMAP_SINGLE, 7292 },
	{ 0x7b4d, 0x7b4e, PDF_CMAP_RANGE, 16837 },
	{ 0x7b4f, 0x7b4f, PDF_CMAP_SINGLE, 1601 },
	{ 0x7b50, 0x7b50, PDF_CMAP_SINGLE, 2429 },
	{ 0x7b51, 0x7b51, PDF_CMAP_SINGLE, 4605 },
	{ 0x7b52, 0x7b52, PDF_CMAP_SINGLE, 3683 },
	{ 0x7b53, 0x7b53, PDF_CMAP_SINGLE, 16839 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 1395 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 16840 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 1190 },
	{ 0x7b57, 0x7b57, PDF_CMAP_SINGLE, 16841 },
	{ 0x7b58, 0x7b58, PDF_CMAP_SINGLE, 7288 },
	{ 0x7b59, 0x7b59, PDF_CMAP_SINGLE, 16842 },
	{ 0x7b5a, 0x7b5a, PDF_CMAP_SINGLE, 7289 },
	{ 0x7b5b, 0x7b5b, PDF_CMAP_SINGLE, 3313 },
	{ 0x7b5c, 0x7b5c, PDF_CMAP_SINGLE, 16843 },
	{ 0x7b5d, 0x7b5d, PDF_CMAP_SINGLE, 7293 },
	{ 0x7b5e, 0x7b5f, PDF_CMAP_RANGE, 16844 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 7294 },
	{ 0x7b61, 0x7b61, PDF_CMAP_SINGLE, 16846 },
	{ 0x7b62, 0x7b62, PDF_CMAP_SINGLE, 7297 },
	{ 0x7b63, 0x7b66, PDF_CMAP_RANGE, 16847 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 9700 },
	{ 0x7b68, 0x7b6d, PDF_CMAP_RANGE, 16851 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 7295 },
	{ 0x7b6f, 0x7b70, PDF_CMAP_RANGE, 16857 },
	{ 0x7b71, 0x7b71, PDF_CMAP_SINGLE, 7299 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 7298 },
	{ 0x7b73, 0x7b74, PDF_CMAP_RANGE, 16859 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 7291 },
	{ 0x7b76, 0x7b76, PDF_CMAP_SINGLE, 16861 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 2423 },
	{ 0x7b78, 0x7b78, PDF_CMAP_SINGLE, 16862 },
	{ 0x7b79, 0x7b79, PDF_CMAP_SINGLE, 1300 },
	{ 0x7b7a, 0x7b7a, PDF_CMAP_SINGLE, 16863 },
	{ 0x7b7b, 0x7b7b, PDF_CMAP_SINGLE, 7296 },
	{ 0x7b7c, 0x7b7d, PDF_CMAP_RANGE, 16864 },
	{ 0x7b7e, 0x7b7e, PDF_CMAP_SINGLE, 3110 },
	{ 0x7b7f, 0x7b7f, PDF_CMAP_SINGLE, 16866 },
	{ 0x7b80, 0x7b80, PDF_CMAP_SINGLE, 2149 },
	{ 0x7b81, 0x7b84, PDF_CMAP_RANGE, 16867 },
	{ 0x7b85, 0x7b85, PDF_CMAP_SINGLE, 7307 },
	{ 0x7b86, 0x7b8a, PDF_CMAP_RANGE, 16871 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 8070 },
	{ 0x7b8c, 0x7b8c, PDF_CMAP_SINGLE, 16876 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 1816 },
	{ 0x7b8e, 0x7b8f, PDF_CMAP_RANGE, 16877 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 7300 },
	{ 0x7b91, 0x7b93, PDF_CMAP_RANGE, 16879 },
	{ 0x7b94, 0x7b94, PDF_CMAP_SINGLE, 1140 },
	{ 0x7b95, 0x7b95, PDF_CMAP_SINGLE, 2067 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 16882 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 3544 },
	{ 0x7b98, 0x7b9b, PDF_CMAP_RANGE, 16883 },
	{ 0x7b9c, 0x7b9c, PDF_CMAP_SINGLE, 7309 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 7305 },
	{ 0x7b9e, 0x7ba0, PDF_CMAP_RANGE, 16887 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 1845 },
	{ 0x7ba2, 0x7ba2, PDF_CMAP_SINGLE, 7310 },
	{ 0x7ba3, 0x7ba5, PDF_CMAP_RANGE, 16890 },
	{ 0x7ba6, 0x7ba7, PDF_CMAP_RANGE, 7301 },
	{ 0x7ba8, 0x7ba8, PDF_CMAP_SINGLE, 7306 },
	{ 0x7ba9, 0x7ba9, PDF_CMAP_SINGLE, 2696 },
	{ 0x7baa, 0x7baa, PDF_CMAP_SINGLE, 7308 },
	{ 0x7bab, 0x7bab, PDF_CMAP_SINGLE, 7311 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 7304 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 2160 },
	{ 0x7bae, 0x7bb0, PDF_CMAP_RANGE, 16893 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3921 },
	{ 0x7bb2, 0x7bb3, PDF_CMAP_RANGE, 16896 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 7312 },
	{ 0x7bb5, 0x7bb7, PDF_CMAP_RANGE, 16898 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 7303 },
	{ 0x7bb9, 0x7bbf, PDF_CMAP_RANGE, 16901 },
	{ 0x7bc0, 0x7bc0, PDF_CMAP_SINGLE, 8115 },
	{ 0x7bc1, 0x7bc1, PDF_CMAP_SINGLE, 7314 },
	{ 0x7bc2, 0x7bc3, PDF_CMAP_RANGE, 16908 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 7919 },
	{ 0x7bc5, 0x7bc5, PDF_CMAP_SINGLE, 16910 },
	{ 0x7bc6, 0x7bc6, PDF_CMAP_SINGLE, 4618 },
	{ 0x7bc7, 0x7bc7, PDF_CMAP_SINGLE, 3017 },
	{ 0x7bc8, 0x7bc8, PDF_CMAP_SINGLE, 16911 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 8868 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 16912 },
	{ 0x7bcb, 0x7bcb, PDF_CMAP_SINGLE, 9704 },
	{ 0x7bcc, 0x7bcc, PDF_CMAP_SINGLE, 7315 },
	{ 0x7bcd, 0x7bd0, PDF_CMAP_RANGE, 16913 },
	{ 0x7bd1, 0x7bd1, PDF_CMAP_SINGLE, 7313 },
	{ 0x7bd2, 0x7bd2, PDF_CMAP_SINGLE, 16917 },
	{ 0x7bd3, 0x7bd3, PDF_CMAP_SINGLE, 2639 },
	{ 0x7bd4, 0x7bd8, PDF_CMAP_RANGE, 16918 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 1752 },
	{ 0x7bda, 0x7bda, PDF_CMAP_SINGLE, 7317 },
	{ 0x7bdb, 0x7bdc, PDF_CMAP_RANGE, 16923 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 7316 },
	{ 0x7bde, 0x7be0, PDF_CMAP_RANGE, 16925 },
	{ 0x7be1, 0x7be1, PDF_CMAP_SINGLE, 1374 },
	{ 0x7be2, 0x7be3, PDF_CMAP_RANGE, 16928 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 9699 },
	{ 0x7be5, 0x7be6, PDF_CMAP_RANGE, 7318 },
	{ 0x7be7, 0x7be8, PDF_CMAP_RANGE, 16930 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 8465 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 7320 },
	{ 0x7beb, 0x7bed, PDF_CMAP_RANGE, 16932 },
	{ 0x7bee, 0x7bee, PDF_CMAP_SINGLE, 2469 },
	{ 0x7bef, 0x7bf0, PDF_CMAP_RANGE, 16935 },
	{ 0x7bf1, 0x7bf1, PDF_CMAP_SINGLE, 2516 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 16937 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 9702 },
	{ 0x7bf4, 0x7bf6, PDF_CMAP_RANGE, 16938 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 2994 },
	{ 0x7bf8, 0x7bfb, PDF_CMAP_RANGE, 16941 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 7323 },
	{ 0x7bfd, 0x7bfd, PDF_CMAP_SINGLE, 16945 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 7322 },
	{ 0x7bff, 0x7bff, PDF_CMAP_SINGLE, 16946 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 9703 },
	{ 0x7c01, 0x7c06, PDF_CMAP_RANGE, 16947 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 1371 },
	{ 0x7c08, 0x7c0a, PDF_CMAP_RANGE, 16953 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 7326 },
	{ 0x7c0c, 0x7c0c, PDF_CMAP_SINGLE, 7321 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 8257 },
	{ 0x7c0e, 0x7c0e, PDF_CMAP_SINGLE, 16956 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 7324 },
	{ 0x7c10, 0x7c15, PDF_CMAP_RANGE, 16957 },
	{ 0x7c16, 0x7c16, PDF_CMAP_SINGLE, 7325 },
	{ 0x7c17, 0x7c1d, PDF_CMAP_RANGE, 16963 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 9706 },
	{ 0x7c1f, 0x7c1f, PDF_CMAP_SINGLE, 7327 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 16970 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 8079 },
	{ 0x7c22, 0x7c22, PDF_CMAP_SINGLE, 16971 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 9708 },
	{ 0x7c24, 0x7c25, PDF_CMAP_RANGE, 16972 },
	{ 0x7c26, 0x7c26, PDF_CMAP_SINGLE, 7329 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 2014 },
	{ 0x7c28, 0x7c29, PDF_CMAP_RANGE, 16974 },
	{ 0x7c2a, 0x7c2a, PDF_CMAP_SINGLE, 7328 },
	{ 0x7c2b, 0x7c2b, PDF_CMAP_SINGLE, 9707 },
	{ 0x7c2c, 0x7c37, PDF_CMAP_RANGE, 16976 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 7330 },
	{ 0x7c39, 0x7c3c, PDF_CMAP_RANGE, 16988 },
	{ 0x7c3d, 0x7c3d, PDF_CMAP_SINGLE, 8400 },
	{ 0x7c3e, 0x7c3e, PDF_CMAP_SINGLE, 8221 },
	{ 0x7c3f, 0x7c3f, PDF_CMAP_SINGLE, 1157 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 7332 },
	{ 0x7c41, 0x7c41, PDF_CMAP_SINGLE, 7331 },
	{ 0x7c42, 0x7c42, PDF_CMAP_SINGLE, 16992 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 8183 },
	{ 0x7c44, 0x7c4b, PDF_CMAP_RANGE, 16993 },
	{ 0x7c4c, 0x7c4c, PDF_CMAP_SINGLE, 7821 },
	{ 0x7c4d, 0x7c4d, PDF_CMAP_SINGLE, 2081 },
	{ 0x7c4e, 0x7c5b, PDF_CMAP_RANGE, 17001 },
	{ 0x7c5c, 0x7c5c, PDF_CMAP_SINGLE, 9705 },
	{ 0x7c5d, 0x7c5e, PDF_CMAP_RANGE, 17015 },
	{ 0x7c5f, 0x7c5f, PDF_CMAP_SINGLE, 9710 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 8250 },
	{ 0x7c61, 0x7c63, PDF_CMAP_RANGE, 17017 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 9875 },
	{ 0x7c65, 0x7c68, PDF_CMAP_RANGE, 17020 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 9701 },
	{ 0x7c6a, 0x7c6a, PDF_CMAP_SINGLE, 9709 },
	{ 0x7c6b, 0x7c6b, PDF_CMAP_SINGLE, 17024 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 8201 },
	{ 0x7c6d, 0x7c6d, PDF_CMAP_SINGLE, 17025 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 8293 },
	{ 0x7c6f, 0x7c71, PDF_CMAP_RANGE, 17026 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 9891 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 2780 },
	{ 0x7c74, 0x7c74, PDF_CMAP_SINGLE, 4853 },
	{ 0x7c75, 0x7c7a, PDF_CMAP_RANGE, 17029 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 2507 },
	{ 0x7c7c, 0x7c7c, PDF_CMAP_SINGLE, 7370 },
	{ 0x7c7d, 0x7c7d, PDF_CMAP_SINGLE, 4654 },
	{ 0x7c7e, 0x7c88, PDF_CMAP_RANGE, 17035 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 1656 },
	{ 0x7c8a, 0x7c90, PDF_CMAP_RANGE, 17046 },
	{ 0x7c91, 0x7c91, PDF_CMAP_SINGLE, 7372 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 2540 },
	{ 0x7c93, 0x7c94, PDF_CMAP_RANGE, 17053 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 3048 },
	{ 0x7c96, 0x7c96, PDF_CMAP_SINGLE, 17055 },
	{ 0x7c97, 0x7c97, PDF_CMAP_SINGLE, 1369 },
	{ 0x7c98, 0x7c98, PDF_CMAP_SINGLE, 4436 },
	{ 0x7c99, 0x7c9b, PDF_CMAP_RANGE, 17056 },
	{ 0x7c9c, 0x7c9c, PDF_CMAP_SINGLE, 7374 },
	{ 0x7c9d, 0x7c9d, PDF_CMAP_SINGLE, 7373 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 7375 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 3535 },
	{ 0x7ca0, 0x7ca1, PDF_CMAP_RANGE, 17059 },
	{ 0x7ca2, 0x7ca2, PDF_CMAP_SINGLE, 7376 },
	{ 0x7ca3, 0x7ca3, PDF_CMAP_SINGLE, 17061 },
	{ 0x7ca4, 0x7ca4, PDF_CMAP_SINGLE, 4356 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 4575 },
	{ 0x7ca6, 0x7ca9, PDF_CMAP_RANGE, 17062 },
	{ 0x7caa, 0x7caa, PDF_CMAP_SINGLE, 1661 },
	{ 0x7cab, 0x7cad, PDF_CMAP_RANGE, 17066 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 2561 },
	{ 0x7caf, 0x7cb0, PDF_CMAP_RANGE, 17069 },
	{ 0x7cb1, 0x7cb1, PDF_CMAP_SINGLE, 2564 },
	{ 0x7cb2, 0x7cb2, PDF_CMAP_SINGLE, 7377 },
	{ 0x7cb3, 0x7cb3, PDF_CMAP_SINGLE, 2267 },
	{ 0x7cb4, 0x7cb8, PDF_CMAP_RANGE, 17071 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 1381 },
	{ 0x7cba, 0x7cbb, PDF_CMAP_RANGE, 17076 },
	{ 0x7cbc, 0x7cbd, PDF_CMAP_RANGE, 7378 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 2266 },
	{ 0x7cbf, 0x7cc0, PDF_CMAP_RANGE, 17078 },
	{ 0x7cc1, 0x7cc1, PDF_CMAP_SINGLE, 7380 },
	{ 0x7cc2, 0x7cc4, PDF_CMAP_RANGE, 17080 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 7385 },
	{ 0x7cc6, 0x7cc6, PDF_CMAP_SINGLE, 17083 },
	{ 0x7cc7, 0x7cc7, PDF_CMAP_SINGLE, 7381 },
	{ 0x7cc8, 0x7cc8, PDF_CMAP_SINGLE, 7384 },
	{ 0x7cc9, 0x7cc9, PDF_CMAP_SINGLE, 17084 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 1972 },
	{ 0x7ccb, 0x7ccb, PDF_CMAP_SINGLE, 17085 },
	{ 0x7ccc, 0x7ccd, PDF_CMAP_RANGE, 7382 },
	{ 0x7cce, 0x7cd4, PDF_CMAP_RANGE, 17086 },
	{ 0x7cd5, 0x7cd5, PDF_CMAP_SINGLE, 1757 },
	{ 0x7cd6, 0x7cd6, PDF_CMAP_SINGLE, 3610 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 7386 },
	{ 0x7cd8, 0x7cd8, PDF_CMAP_SINGLE, 17093 },
	{ 0x7cd9, 0x7cd9, PDF_CMAP_SINGLE, 1185 },
	{ 0x7cda, 0x7cdb, PDF_CMAP_RANGE, 17094 },
	{ 0x7cdc, 0x7cdc, PDF_CMAP_SINGLE, 2776 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 9717 },
	{ 0x7cde, 0x7cde, PDF_CMAP_SINGLE, 7932 },
	{ 0x7cdf, 0x7cdf, PDF_CMAP_SINGLE, 4390 },
	{ 0x7ce0, 0x7ce0, PDF_CMAP_SINGLE, 2372 },
	{ 0x7ce1, 0x7ce6, PDF_CMAP_RANGE, 17096 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 8228 },
	{ 0x7ce8, 0x7ce8, PDF_CMAP_SINGLE, 7387 },
	{ 0x7ce9, 0x7cee, PDF_CMAP_RANGE, 17102 },
	{ 0x7cef, 0x7cef, PDF_CMAP_SINGLE, 2933 },
	{ 0x7cf0, 0x7cf0, PDF_CMAP_SINGLE, 9883 },
	{ 0x7cf1, 0x7cf1, PDF_CMAP_SINGLE, 17108 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 9715 },
	{ 0x7cf3, 0x7cf3, PDF_CMAP_SINGLE, 17109 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 8921 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 17110 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 9716 },
	{ 0x7cf7, 0x7cf7, PDF_CMAP_SINGLE, 17111 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 7399 },
	{ 0x7cf9, 0x7cf9, PDF_CMAP_SINGLE, 9243 },
	{ 0x7cfa, 0x7cfa, PDF_CMAP_SINGLE, 17112 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 3874 },
	{ 0x7cfc, 0x7cfd, PDF_CMAP_RANGE, 17113 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 8136 },
	{ 0x7cff, 0x7cff, PDF_CMAP_SINGLE, 17115 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 8059 },
	{ 0x7d01, 0x7d01, PDF_CMAP_SINGLE, 17116 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 9245 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 17117 },
	{ 0x7d04, 0x7d04, PDF_CMAP_SINGLE, 8780 },
	{ 0x7d05, 0x7d05, PDF_CMAP_SINGLE, 8008 },
	{ 0x7d06, 0x7d06, PDF_CMAP_SINGLE, 9244 },
	{ 0x7d07, 0x7d08, PDF_CMAP_RANGE, 9246 },
	{ 0x7d09, 0x7d09, PDF_CMAP_SINGLE, 8447 },
	{ 0x7d0a, 0x7d0a, PDF_CMAP_SINGLE, 3800 },
	{ 0x7d0b, 0x7d0b, PDF_CMAP_SINGLE, 8602 },
	{ 0x7d0c, 0x7d0c, PDF_CMAP_SINGLE, 17118 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 8337 },
	{ 0x7d0e, 0x7d0f, PDF_CMAP_RANGE, 17119 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 8359 },
	{ 0x7d11, 0x7d12, PDF_CMAP_RANGE, 17121 },
	{ 0x7d13, 0x7d13, PDF_CMAP_SINGLE, 9251 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 7835 },
	{ 0x7d15, 0x7d15, PDF_CMAP_SINGLE, 9250 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 17123 },
	{ 0x7d17, 0x7d17, PDF_CMAP_SINGLE, 8464 },
	{ 0x7d18, 0x7d18, PDF_CMAP_SINGLE, 17124 },
	{ 0x7d19, 0x7d19, PDF_CMAP_SINGLE, 8845 },
	{ 0x7d1a, 0x7d1a, PDF_CMAP_SINGLE, 8049 },
	{ 0x7d1b, 0x7d1b, PDF_CMAP_SINGLE, 7928 },
	{ 0x7d1c, 0x7d1c, PDF_CMAP_SINGLE, 9249 },
	{ 0x7d1d, 0x7d1f, PDF_CMAP_RANGE, 17125 },
	{ 0x7d20, 0x7d20, PDF_CMAP_SINGLE, 3533 },
	{ 0x7d21, 0x7d21, PDF_CMAP_SINGLE, 7923 },
	{ 0x7d22, 0x7d22, PDF_CMAP_SINGLE, 3564 },
	{ 0x7d23, 0x7d26, PDF_CMAP_RANGE, 17128 },
	{ 0x7d27, 0x7d27, PDF_CMAP_SINGLE, 2245 },
	{ 0x7d28, 0x7d2a, PDF_CMAP_RANGE, 17132 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 4652 },
	{ 0x7d2c, 0x7d2e, PDF_CMAP_RANGE, 17135 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 2502 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 8625 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 9254 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 9253 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 8480 },
	{ 0x7d34, 0x7d38, PDF_CMAP_RANGE, 17138 },
	{ 0x7d39, 0x7d39, PDF_CMAP_SINGLE, 8474 },
	{ 0x7d3a, 0x7d3a, PDF_CMAP_SINGLE, 9252 },
	{ 0x7d3b, 0x7d3b, PDF_CMAP_SINGLE, 17143 },
	{ 0x7d3c, 0x7d3c, PDF_CMAP_SINGLE, 9256 },
	{ 0x7d3d, 0x7d3e, PDF_CMAP_RANGE, 17144 },
	{ 0x7d3f, 0x7d3f, PDF_CMAP_SINGLE, 9258 },
	{ 0x7d40, 0x7d40, PDF_CMAP_SINGLE, 9257 },
	{ 0x7d41, 0x7d41, PDF_CMAP_SINGLE, 17146 },
	{ 0x7d42, 0x7d42, PDF_CMAP_SINGLE, 8852 },
	{ 0x7d43, 0x7d43, PDF_CMAP_SINGLE, 17147 },
	{ 0x7d44, 0x7d44, PDF_CMAP_SINGLE, 8894 },
	{ 0x7d45, 0x7d45, PDF_CMAP_SINGLE, 17148 },
	{ 0x7d46, 0x7d46, PDF_CMAP_SINGLE, 7729 },
	{ 0x7d47, 0x7d4d, PDF_CMAP_RANGE, 17149 },
	{ 0x7d4e, 0x7d4e, PDF_CMAP_SINGLE, 9260 },
	{ 0x7d4f, 0x7d4f, PDF_CMAP_SINGLE, 17156 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 8117 },
	{ 0x7d51, 0x7d5c, PDF_CMAP_RANGE, 17157 },
	{ 0x7d5d, 0x7d5d, PDF_CMAP_SINGLE, 9259 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 8111 },
	{ 0x7d5f, 0x7d60, PDF_CMAP_RANGE, 17169 },
	{ 0x7d61, 0x7d61, PDF_CMAP_SINGLE, 8296 },
	{ 0x7d62, 0x7d62, PDF_CMAP_SINGLE, 8676 },
	{ 0x7d63, 0x7d65, PDF_CMAP_RANGE, 17171 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 7969 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 17174 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 8449 },
	{ 0x7d69, 0x7d6d, PDF_CMAP_RANGE, 17175 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 4032 },
	{ 0x7d6f, 0x7d70, PDF_CMAP_RANGE, 17180 },
	{ 0x7d71, 0x7d71, PDF_CMAP_SINGLE, 8572 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 8522 },
	{ 0x7d73, 0x7d73, PDF_CMAP_SINGLE, 9261 },
	{ 0x7d74, 0x7d75, PDF_CMAP_RANGE, 17182 },
	{ 0x7d76, 0x7d76, PDF_CMAP_SINGLE, 8148 },
	{ 0x7d77, 0x7d77, PDF_CMAP_SINGLE, 7400 },
	{ 0x7d78, 0x7d78, PDF_CMAP_SINGLE, 17184 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 8145 },
	{ 0x7d7a, 0x7d80, PDF_CMAP_RANGE, 17185 },
	{ 0x7d81, 0x7d81, PDF_CMAP_SINGLE, 7731 },
	{ 0x7d82, 0x7d82, PDF_CMAP_SINGLE, 17192 },
	{ 0x7d83, 0x7d83, PDF_CMAP_SINGLE, 9263 },
	{ 0x7d84, 0x7d85, PDF_CMAP_RANGE, 17193 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 9262 },
	{ 0x7d87, 0x7d87, PDF_CMAP_SINGLE, 17195 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 9264 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 8667 },
	{ 0x7d8a, 0x7d8e, PDF_CMAP_RANGE, 17196 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 8536 },
	{ 0x7d90, 0x7d92, PDF_CMAP_RANGE, 17201 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 8130 },
	{ 0x7d94, 0x7d9b, PDF_CMAP_RANGE, 17204 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 8889 },
	{ 0x7d9d, 0x7d9d, PDF_CMAP_SINGLE, 17212 },
	{ 0x7d9e, 0x7d9e, PDF_CMAP_SINGLE, 9270 },
	{ 0x7d9f, 0x7da1, PDF_CMAP_RANGE, 17213 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 7822 },
	{ 0x7da3, 0x7da3, PDF_CMAP_SINGLE, 9273 },
	{ 0x7da4, 0x7da5, PDF_CMAP_RANGE, 17216 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 7401 },
	{ 0x7da7, 0x7daa, PDF_CMAP_RANGE, 17218 },
	{ 0x7dab, 0x7dab, PDF_CMAP_SINGLE, 8645 },
	{ 0x7dac, 0x7dac, PDF_CMAP_SINGLE, 9271 },
	{ 0x7dad, 0x7dad, PDF_CMAP_SINGLE, 8594 },
	{ 0x7dae, 0x7dae, PDF_CMAP_SINGLE, 7402 },
	{ 0x7daf, 0x7daf, PDF_CMAP_SINGLE, 17222 },
	{ 0x7db0, 0x7db0, PDF_CMAP_SINGLE, 9274 },
	{ 0x7db1, 0x7db1, PDF_CMAP_SINGLE, 7961 },
	{ 0x7db2, 0x7db2, PDF_CMAP_SINGLE, 8588 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 7744 },
	{ 0x7db4, 0x7db4, PDF_CMAP_SINGLE, 8883 },
	{ 0x7db5, 0x7db7, PDF_CMAP_RANGE, 17223 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 8287 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 9272 },
	{ 0x7dba, 0x7dba, PDF_CMAP_SINGLE, 9266 },
	{ 0x7dbb, 0x7dbb, PDF_CMAP_SINGLE, 8820 },
	{ 0x7dbc, 0x7dbc, PDF_CMAP_SINGLE, 17226 },
	{ 0x7dbd, 0x7dbd, PDF_CMAP_SINGLE, 7836 },
	{ 0x7dbe, 0x7dbe, PDF_CMAP_SINGLE, 9265 },
	{ 0x7dbf, 0x7dbf, PDF_CMAP_SINGLE, 8325 },
	{ 0x7dc0, 0x7dc3, PDF_CMAP_RANGE, 17227 },
	{ 0x7dc4, 0x7dc4, PDF_CMAP_SINGLE, 9269 },
	{ 0x7dc5, 0x7dc6, PDF_CMAP_RANGE, 17231 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 9275 },
	{ 0x7dc8, 0x7dc9, PDF_CMAP_RANGE, 17233 },
	{ 0x7dca, 0x7dca, PDF_CMAP_SINGLE, 8119 },
	{ 0x7dcb, 0x7dcb, PDF_CMAP_SINGLE, 9267 },
	{ 0x7dcc, 0x7dd0, PDF_CMAP_RANGE, 17235 },
	{ 0x7dd1, 0x7dd1, PDF_CMAP_SINGLE, 8276 },
	{ 0x7dd2, 0x7dd2, PDF_CMAP_SINGLE, 8670 },
	{ 0x7dd3, 0x7dd3, PDF_CMAP_SINGLE, 17240 },
	{ 0x7dd4, 0x7dd4, PDF_CMAP_SINGLE, 9268 },
	{ 0x7dd5, 0x7dd6, PDF_CMAP_RANGE, 17241 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 9277 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 8073 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 9276 },
	{ 0x7dda, 0x7ddc, PDF_CMAP_RANGE, 17243 },
	{ 0x7ddd, 0x7ddd, PDF_CMAP_SINGLE, 8046 },
	{ 0x7dde, 0x7dde, PDF_CMAP_SINGLE, 7896 },
	{ 0x7ddf, 0x7ddf, PDF_CMAP_SINGLE, 17246 },
	{ 0x7de0, 0x7de0, PDF_CMAP_SINGLE, 7871 },
	{ 0x7de1, 0x7de1, PDF_CMAP_SINGLE, 9285 },
	{ 0x7de2, 0x7de2, PDF_CMAP_SINGLE, 17247 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 8777 },
	{ 0x7de4, 0x7de5, PDF_CMAP_RANGE, 17248 },
	{ 0x7de6, 0x7de6, PDF_CMAP_SINGLE, 9281 },
	{ 0x7de7, 0x7de7, PDF_CMAP_SINGLE, 17250 },
	{ 0x7de8, 0x7de8, PDF_CMAP_SINGLE, 7751 },
	{ 0x7de9, 0x7de9, PDF_CMAP_SINGLE, 8022 },
	{ 0x7dea, 0x7deb, PDF_CMAP_RANGE, 17251 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 8326 },
	{ 0x7ded, 0x7dee, PDF_CMAP_RANGE, 17253 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 8598 },
	{ 0x7df0, 0x7df0, PDF_CMAP_SINGLE, 17255 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 9283 },
	{ 0x7df2, 0x7df2, PDF_CMAP_SINGLE, 9279 },
	{ 0x7df3, 0x7df3, PDF_CMAP_SINGLE, 17256 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 8227 },
	{ 0x7df5, 0x7df5, PDF_CMAP_SINGLE, 17257 },
	{ 0x7df6, 0x7df6, PDF_CMAP_SINGLE, 9282 },
	{ 0x7df7, 0x7df8, PDF_CMAP_RANGE, 17258 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 9278 },
	{ 0x7dfa, 0x7dfa, PDF_CMAP_SINGLE, 17260 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 9894 },
	{ 0x7dfc, 0x7dff, PDF_CMAP_RANGE, 17261 },
	{ 0x7e00, 0x7e07, PDF_CMAP_RANGE, 17265 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 9038 },
	{ 0x7e09, 0x7e09, PDF_CMAP_SINGLE, 9286 },
	{ 0x7e0a, 0x7e0a, PDF_CMAP_SINGLE, 9291 },
	{ 0x7e0b, 0x7e0b, PDF_CMAP_SINGLE, 9284 },
	{ 0x7e0c, 0x7e0f, PDF_CMAP_RANGE, 17273 },
	{ 0x7e10, 0x7e10, PDF_CMAP_SINGLE, 9255 },
	{ 0x7e11, 0x7e11, PDF_CMAP_SINGLE, 9292 },
	{ 0x7e12, 0x7e1a, PDF_CMAP_RANGE, 17277 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 7951 },
	{ 0x7e1c, 0x7e1c, PDF_CMAP_SINGLE, 17286 },
	{ 0x7e1d, 0x7e1d, PDF_CMAP_SINGLE, 9287 },
	{ 0x7e1e, 0x7e1e, PDF_CMAP_SINGLE, 9289 },
	{ 0x7e1f, 0x7e1f, PDF_CMAP_SINGLE, 9288 },
	{ 0x7e20, 0x7e22, PDF_CMAP_RANGE, 17287 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 8642 },
	{ 0x7e24, 0x7e26, PDF_CMAP_RANGE, 17290 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 8558 },
	{ 0x7e28, 0x7e2a, PDF_CMAP_RANGE, 17293 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 7939 },
	{ 0x7e2c, 0x7e2c, PDF_CMAP_SINGLE, 17296 },
	{ 0x7e2d, 0x7e2d, PDF_CMAP_SINGLE, 9290 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 8540 },
	{ 0x7e2f, 0x7e30, PDF_CMAP_RANGE, 17297 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 8891 },
	{ 0x7e32, 0x7e32, PDF_CMAP_SINGLE, 9296 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 17299 },
	{ 0x7e34, 0x7e34, PDF_CMAP_SINGLE, 9876 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 9295 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 9718 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 8273 },
	{ 0x7e38, 0x7e38, PDF_CMAP_SINGLE, 17300 },
	{ 0x7e39, 0x7e39, PDF_CMAP_SINGLE, 9294 },
	{ 0x7e3a, 0x7e3a, PDF_CMAP_SINGLE, 17301 },
	{ 0x7e3b, 0x7e3b, PDF_CMAP_SINGLE, 7675 },
	{ 0x7e3c, 0x7e3c, PDF_CMAP_SINGLE, 17302 },
	{ 0x7e3d, 0x7e3d, PDF_CMAP_SINGLE, 8890 },
	{ 0x7e3e, 0x7e3e, PDF_CMAP_SINGLE, 8045 },
	{ 0x7e3f, 0x7e40, PDF_CMAP_RANGE, 17303 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 1614 },
	{ 0x7e42, 0x7e44, PDF_CMAP_RANGE, 17305 },
	{ 0x7e45, 0x7e45, PDF_CMAP_SINGLE, 9298 },
	{ 0x7e46, 0x7e46, PDF_CMAP_SINGLE, 9297 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 7403 },
	{ 0x7e48, 0x7e51, PDF_CMAP_RANGE, 17308 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 9301 },
	{ 0x7e53, 0x7e53, PDF_CMAP_SINGLE, 17318 },
	{ 0x7e54, 0x7e54, PDF_CMAP_SINGLE, 8841 },
	{ 0x7e55, 0x7e55, PDF_CMAP_SINGLE, 8470 },
	{ 0x7e56, 0x7e59, PDF_CMAP_RANGE, 17319 },
	{ 0x7e5a, 0x7e5a, PDF_CMAP_SINGLE, 9300 },
	{ 0x7e5b, 0x7e5d, PDF_CMAP_RANGE, 17323 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 8443 },
	{ 0x7e5f, 0x7e61, PDF_CMAP_RANGE, 17326 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 9280 },
	{ 0x7e63, 0x7e68, PDF_CMAP_RANGE, 17329 },
	{ 0x7e69, 0x7e69, PDF_CMAP_SINGLE, 8486 },
	{ 0x7e6a, 0x7e6a, PDF_CMAP_SINGLE, 8033 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 9885 },
	{ 0x7e6c, 0x7e6c, PDF_CMAP_SINGLE, 17335 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 8074 },
	{ 0x7e6e, 0x7e6e, PDF_CMAP_SINGLE, 9302 },
	{ 0x7e6f, 0x7e6f, PDF_CMAP_SINGLE, 9305 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 9304 },
	{ 0x7e71, 0x7e72, PDF_CMAP_RANGE, 17336 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 8110 },
	{ 0x7e74, 0x7e78, PDF_CMAP_RANGE, 17338 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 8730 },
	{ 0x7e7a, 0x7e7b, PDF_CMAP_RANGE, 17343 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 8058 },
	{ 0x7e7d, 0x7e7d, PDF_CMAP_SINGLE, 9293 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 9303 },
	{ 0x7e7f, 0x7e81, PDF_CMAP_RANGE, 17345 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 4680 },
	{ 0x7e83, 0x7e87, PDF_CMAP_RANGE, 17348 },
	{ 0x7e88, 0x7e88, PDF_CMAP_SINGLE, 9299 },
	{ 0x7e89, 0x7e89, PDF_CMAP_SINGLE, 17353 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 9248 },
	{ 0x7e8b, 0x7e8b, PDF_CMAP_SINGLE, 17354 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 8671 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 9861 },
	{ 0x7e8e, 0x7e8e, PDF_CMAP_SINGLE, 17355 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 7790 },
	{ 0x7e90, 0x7e92, PDF_CMAP_RANGE, 17356 },
	{ 0x7e93, 0x7e93, PDF_CMAP_SINGLE, 8740 },
	{ 0x7e94, 0x7e94, PDF_CMAP_SINGLE, 9840 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 17359 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 8634 },
	{ 0x7e97, 0x7e97, PDF_CMAP_SINGLE, 17360 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 9306 },
	{ 0x7e99, 0x7e9a, PDF_CMAP_RANGE, 17361 },
	{ 0x7e9b, 0x7e9b, PDF_CMAP_SINGLE, 7404 },
	{ 0x7e9c, 0x7e9c, PDF_CMAP_SINGLE, 8191 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 17363 },
	{ 0x7e9f, 0x7e9f, PDF_CMAP_SINGLE, 6099 },
	{ 0x7ea0, 0x7ea0, PDF_CMAP_SINGLE, 2287 },
	{ 0x7ea1, 0x7ea1, PDF_CMAP_SINGLE, 6100 },
	{ 0x7ea2, 0x7ea2, PDF_CMAP_SINGLE, 1955 },
	{ 0x7ea3, 0x7ea3, PDF_CMAP_SINGLE, 6101 },
	{ 0x7ea4, 0x7ea4, PDF_CMAP_SINGLE, 3896 },
	{ 0x7ea5, 0x7ea5, PDF_CMAP_SINGLE, 6102 },
	{ 0x7ea6, 0x7ea6, PDF_CMAP_SINGLE, 4351 },
	{ 0x7ea7, 0x7ea7, PDF_CMAP_SINGLE, 2089 },
	{ 0x7ea8, 0x7ea9, PDF_CMAP_RANGE, 6103 },
	{ 0x7eaa, 0x7eaa, PDF_CMAP_SINGLE, 2112 },
	{ 0x7eab, 0x7eab, PDF_CMAP_SINGLE, 3245 },
	{ 0x7eac, 0x7eac, PDF_CMAP_SINGLE, 3778 },
	{ 0x7ead, 0x7ead, PDF_CMAP_SINGLE, 6105 },
	{ 0x7eae, 0x7eae, PDF_CMAP_SINGLE, 17365 },
	{ 0x7eaf, 0x7eaf, PDF_CMAP_SINGLE, 1346 },
	{ 0x7eb0, 0x7eb0, PDF_CMAP_SINGLE, 6106 },
	{ 0x7eb1, 0x7eb1, PDF_CMAP_SINGLE, 3309 },
	{ 0x7eb2, 0x7eb2, PDF_CMAP_SINGLE, 1748 },
	{ 0x7eb3, 0x7eb3, PDF_CMAP_SINGLE, 2860 },
	{ 0x7eb4, 0x7eb4, PDF_CMAP_SINGLE, 17366 },
	{ 0x7eb5, 0x7eb5, PDF_CMAP_SINGLE, 4666 },
	{ 0x7eb6, 0x7eb6, PDF_CMAP_SINGLE, 2689 },
	{ 0x7eb7, 0x7eb7, PDF_CMAP_SINGLE, 1652 },
	{ 0x7eb8, 0x7eb8, PDF_CMAP_SINGLE, 4540 },
	{ 0x7eb9, 0x7eb9, PDF_CMAP_SINGLE, 3797 },
	{ 0x7eba, 0x7eba, PDF_CMAP_SINGLE, 1633 },
	{ 0x7ebb, 0x7ebc, PDF_CMAP_RANGE, 17367 },
	{ 0x7ebd, 0x7ebd, PDF_CMAP_SINGLE, 2919 },
	{ 0x7ebe, 0x7ebe, PDF_CMAP_SINGLE, 6107 },
	{ 0x7ebf, 0x7ebf, PDF_CMAP_SINGLE, 3916 },
	{ 0x7ec0, 0x7ec2, PDF_CMAP_RANGE, 6108 },
	{ 0x7ec3, 0x7ec3, PDF_CMAP_SINGLE, 2560 },
	{ 0x7ec4, 0x7ec4, PDF_CMAP_SINGLE, 4678 },
	{ 0x7ec5, 0x7ec5, PDF_CMAP_SINGLE, 3369 },
	{ 0x7ec6, 0x7ec6, PDF_CMAP_SINGLE, 3877 },
	{ 0x7ec7, 0x7ec7, PDF_CMAP_SINGLE, 4526 },
	{ 0x7ec8, 0x7ec8, PDF_CMAP_SINGLE, 4564 },
	{ 0x7ec9, 0x7ec9, PDF_CMAP_SINGLE, 6111 },
	{ 0x7eca, 0x7eca, PDF_CMAP_SINGLE, 1016 },
	{ 0x7ecb, 0x7ecc, PDF_CMAP_RANGE, 6112 },
	{ 0x7ecd, 0x7ecd, PDF_CMAP_SINGLE, 3349 },
	{ 0x7ece, 0x7ece, PDF_CMAP_SINGLE, 4214 },
	{ 0x7ecf, 0x7ecf, PDF_CMAP_SINGLE, 2268 },
	{ 0x7ed0, 0x7ed0, PDF_CMAP_SINGLE, 6114 },
	{ 0x7ed1, 0x7ed1, PDF_CMAP_SINGLE, 1022 },
	{ 0x7ed2, 0x7ed2, PDF_CMAP_SINGLE, 3257 },
	{ 0x7ed3, 0x7ed3, PDF_CMAP_SINGLE, 2226 },
	{ 0x7ed4, 0x7ed4, PDF_CMAP_SINGLE, 6115 },
	{ 0x7ed5, 0x7ed5, PDF_CMAP_SINGLE, 3233 },
	{ 0x7ed6, 0x7ed6, PDF_CMAP_SINGLE, 17369 },
	{ 0x7ed7, 0x7ed7, PDF_CMAP_SINGLE, 6116 },
	{ 0x7ed8, 0x7ed8, PDF_CMAP_SINGLE, 2043 },
	{ 0x7ed9, 0x7ed9, PDF_CMAP_SINGLE, 1779 },
	{ 0x7eda, 0x7eda, PDF_CMAP_SINGLE, 4045 },
	{ 0x7edb, 0x7edb, PDF_CMAP_SINGLE, 6117 },
	{ 0x7edc, 0x7edc, PDF_CMAP_SINGLE, 2702 },
	{ 0x7edd, 0x7edd, PDF_CMAP_SINGLE, 2343 },
	{ 0x7ede, 0x7ede, PDF_CMAP_SINGLE, 2203 },
	{ 0x7edf, 0x7edf, PDF_CMAP_SINGLE, 3684 },
	{ 0x7ee0, 0x7ee1, PDF_CMAP_RANGE, 6118 },
	{ 0x7ee2, 0x7ee2, PDF_CMAP_SINGLE, 2333 },
	{ 0x7ee3, 0x7ee3, PDF_CMAP_SINGLE, 4016 },
	{ 0x7ee4, 0x7ee4, PDF_CMAP_SINGLE, 17370 },
	{ 0x7ee5, 0x7ee5, PDF_CMAP_SINGLE, 3548 },
	{ 0x7ee6, 0x7ee6, PDF_CMAP_SINGLE, 3619 },
	{ 0x7ee7, 0x7ee7, PDF_CMAP_SINGLE, 2111 },
	{ 0x7ee8, 0x7ee8, PDF_CMAP_SINGLE, 6120 },
	{ 0x7ee9, 0x7ee9, PDF_CMAP_SINGLE, 2075 },
	{ 0x7eea, 0x7eea, PDF_CMAP_SINGLE, 4034 },
	{ 0x7eeb, 0x7eeb, PDF_CMAP_SINGLE, 6121 },
	{ 0x7eec, 0x7eec, PDF_CMAP_SINGLE, 17371 },
	{ 0x7eed, 0x7eed, PDF_CMAP_SINGLE, 4035 },
	{ 0x7eee, 0x7eef, PDF_CMAP_RANGE, 6122 },
	{ 0x7ef0, 0x7ef0, PDF_CMAP_SINGLE, 1349 },
	{ 0x7ef1, 0x7ef2, PDF_CMAP_RANGE, 6124 },
	{ 0x7ef3, 0x7ef3, PDF_CMAP_SINGLE, 3383 },
	{ 0x7ef4, 0x7ef4, PDF_CMAP_SINGLE, 3771 },
	{ 0x7ef5, 0x7ef5, PDF_CMAP_SINGLE, 2789 },
	{ 0x7ef6, 0x7ef6, PDF_CMAP_SINGLE, 6127 },
	{ 0x7ef7, 0x7ef7, PDF_CMAP_SINGLE, 1066 },
	{ 0x7ef8, 0x7ef8, PDF_CMAP_SINGLE, 1302 },
	{ 0x7ef9, 0x7ef9, PDF_CMAP_SINGLE, 17372 },
	{ 0x7efa, 0x7efb, PDF_CMAP_RANGE, 6128 },
	{ 0x7efc, 0x7efc, PDF_CMAP_SINGLE, 4664 },
	{ 0x7efd, 0x7efd, PDF_CMAP_SINGLE, 4449 },
	{ 0x7efe, 0x7efe, PDF_CMAP_SINGLE, 6130 },
	{ 0x7eff, 0x7eff, PDF_CMAP_SINGLE, 2675 },
	{ 0x7f00, 0x7f00, PDF_CMAP_SINGLE, 4631 },
	{ 0x7f01, 0x7f03, PDF_CMAP_RANGE, 6131 },
	{ 0x7f04, 0x7f04, PDF_CMAP_SINGLE, 2141 },
	{ 0x7f05, 0x7f05, PDF_CMAP_SINGLE, 2794 },
	{ 0x7f06, 0x7f06, PDF_CMAP_SINGLE, 2477 },
	{ 0x7f07, 0x7f08, PDF_CMAP_RANGE, 6134 },
	{ 0x7f09, 0x7f09, PDF_CMAP_SINGLE, 2076 },
	{ 0x7f0a, 0x7f0a, PDF_CMAP_SINGLE, 17373 },
	{ 0x7f0b, 0x7f0c, PDF_CMAP_RANGE, 6136 },
	{ 0x7f0d, 0x7f0d, PDF_CMAP_SINGLE, 6126 },
	{ 0x7f0e, 0x7f0e, PDF_CMAP_SINGLE, 1551 },
	{ 0x7f0f, 0x7f0f, PDF_CMAP_SINGLE, 6138 },
	{ 0x7f10, 0x7f10, PDF_CMAP_SINGLE, 17374 },
	{ 0x7f11, 0x7f12, PDF_CMAP_RANGE, 6139 },
	{ 0x7f13, 0x7f13, PDF_CMAP_SINGLE, 1999 },
	{ 0x7f14, 0x7f14, PDF_CMAP_SINGLE, 1471 },
	{ 0x7f15, 0x7f15, PDF_CMAP_SINGLE, 2669 },
	{ 0x7f16, 0x7f16, PDF_CMAP_SINGLE, 1097 },
	{ 0x7f17, 0x7f17, PDF_CMAP_SINGLE, 6141 },
	{ 0x7f18, 0x7f18, PDF_CMAP_SINGLE, 4344 },
	{ 0x7f19, 0x7f19, PDF_CMAP_SINGLE, 6142 },
	{ 0x7f1a, 0x7f1a, PDF_CMAP_SINGLE, 1722 },
	{ 0x7f1b, 0x7f1b, PDF_CMAP_SINGLE, 6144 },
	{ 0x7f1c, 0x7f1c, PDF_CMAP_SINGLE, 6143 },
	{ 0x7f1d, 0x7f1d, PDF_CMAP_SINGLE, 1673 },
	{ 0x7f1e, 0x7f1e, PDF_CMAP_SINGLE, 17375 },
	{ 0x7f1f, 0x7f1f, PDF_CMAP_SINGLE, 6145 },
	{ 0x7f20, 0x7f20, PDF_CMAP_SINGLE, 1215 },
	{ 0x7f21, 0x7f27, PDF_CMAP_RANGE, 6146 },
	{ 0x7f28, 0x7f28, PDF_CMAP_SINGLE, 4236 },
	{ 0x7f29, 0x7f29, PDF_CMAP_SINGLE, 3562 },
	{ 0x7f2a, 0x7f2d, PDF_CMAP_RANGE, 6153 },
	{ 0x7f2e, 0x7f2e, PDF_CMAP_SINGLE, 3330 },
	{ 0x7f2f, 0x7f33, PDF_CMAP_RANGE, 6157 },
	{ 0x7f34, 0x7f34, PDF_CMAP_SINGLE, 2202 },
	{ 0x7f35, 0x7f35, PDF_CMAP_SINGLE, 6162 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 7262 },
	{ 0x7f37, 0x7f37, PDF_CMAP_SINGLE, 17376 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 1746 },
	{ 0x7f39, 0x7f39, PDF_CMAP_SINGLE, 17377 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 3212 },
	{ 0x7f3b, 0x7f41, PDF_CMAP_RANGE, 17378 },
	{ 0x7f42, 0x7f42, PDF_CMAP_SINGLE, 7263 },
	{ 0x7f43, 0x7f43, PDF_CMAP_SINGLE, 17385 },
	{ 0x7f44, 0x7f45, PDF_CMAP_RANGE, 7264 },
	{ 0x7f46, 0x7f4b, PDF_CMAP_RANGE, 17386 },
	{ 0x7f4c, 0x7f4c, PDF_CMAP_SINGLE, 9698 },
	{ 0x7f4d, 0x7f4d, PDF_CMAP_SINGLE, 17392 },
	{ 0x7f4e, 0x7f4e, PDF_CMAP_SINGLE, 9882 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 17393 },
	{ 0x7f50, 0x7f50, PDF_CMAP_SINGLE, 1847 },
	{ 0x7f51, 0x7f51, PDF_CMAP_SINGLE, 3753 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 17394 },
	{ 0x7f54, 0x7f54, PDF_CMAP_SINGLE, 4766 },
	{ 0x7f55, 0x7f55, PDF_CMAP_SINGLE, 1896 },
	{ 0x7f56, 0x7f56, PDF_CMAP_SINGLE, 17396 },
	{ 0x7f57, 0x7f57, PDF_CMAP_SINGLE, 2693 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 6785 },
	{ 0x7f59, 0x7f59, PDF_CMAP_SINGLE, 17397 },
	{ 0x7f5a, 0x7f5a, PDF_CMAP_SINGLE, 1600 },
	{ 0x7f5b, 0x7f5e, PDF_CMAP_RANGE, 17398 },
	{ 0x7f5f, 0x7f5f, PDF_CMAP_SINGLE, 6787 },
	{ 0x7f60, 0x7f60, PDF_CMAP_SINGLE, 17402 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 6786 },
	{ 0x7f62, 0x7f62, PDF_CMAP_SINGLE, 992 },
	{ 0x7f63, 0x7f67, PDF_CMAP_RANGE, 17403 },
	{ 0x7f68, 0x7f68, PDF_CMAP_SINGLE, 6789 },
	{ 0x7f69, 0x7f69, PDF_CMAP_SINGLE, 4471 },
	{ 0x7f6a, 0x7f6a, PDF_CMAP_SINGLE, 4684 },
	{ 0x7f6b, 0x7f6d, PDF_CMAP_RANGE, 17408 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 4546 },
	{ 0x7f6f, 0x7f6f, PDF_CMAP_SINGLE, 17411 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 7914 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 6791 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 3463 },
	{ 0x7f73, 0x7f73, PDF_CMAP_SINGLE, 17412 },
	{ 0x7f74, 0x7f74, PDF_CMAP_SINGLE, 6790 },
	{ 0x7f75, 0x7f76, PDF_CMAP_RANGE, 17413 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 7724 },
	{ 0x7f78, 0x7f78, PDF_CMAP_SINGLE, 17415 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 6792 },
	{ 0x7f7a, 0x7f7d, PDF_CMAP_RANGE, 17416 },
	{ 0x7f7e, 0x7f7e, PDF_CMAP_SINGLE, 6794 },
	{ 0x7f7f, 0x7f80, PDF_CMAP_RANGE, 17420 },
	{ 0x7f81, 0x7f81, PDF_CMAP_SINGLE, 6793 },
	{ 0x7f82, 0x7f84, PDF_CMAP_RANGE, 17422 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 8290 },
	{ 0x7f86, 0x7f86, PDF_CMAP_SINGLE, 9457 },
	{ 0x7f87, 0x7f87, PDF_CMAP_SINGLE, 17425 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 9458 },
	{ 0x7f89, 0x7f89, PDF_CMAP_SINGLE, 17426 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 4123 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 17427 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 3129 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 17428 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 2757 },
	{ 0x7f8f, 0x7f93, PDF_CMAP_RANGE, 17429 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 1756 },
	{ 0x7f95, 0x7f99, PDF_CMAP_RANGE, 17434 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 2608 },
	{ 0x7f9b, 0x7f9c, PDF_CMAP_RANGE, 17439 },
	{ 0x7f9d, 0x7f9d, PDF_CMAP_SINGLE, 7364 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 4010 },
	{ 0x7f9f, 0x7f9f, PDF_CMAP_SINGLE, 7365 },
	{ 0x7fa0, 0x7fa0, PDF_CMAP_SINGLE, 17441 },
	{ 0x7fa1, 0x7fa1, PDF_CMAP_SINGLE, 3912 },
	{ 0x7fa2, 0x7fa3, PDF_CMAP_RANGE, 17442 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 3221 },
	{ 0x7fa5, 0x7fa5, PDF_CMAP_SINGLE, 9714 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 17444 },
	{ 0x7fa7, 0x7fa7, PDF_CMAP_SINGLE, 7366 },
	{ 0x7fa8, 0x7fa8, PDF_CMAP_SINGLE, 17445 },
	{ 0x7fa9, 0x7fa9, PDF_CMAP_SINGLE, 8725 },
	{ 0x7faa, 0x7fae, PDF_CMAP_RANGE, 17446 },
	{ 0x7faf, 0x7fb0, PDF_CMAP_RANGE, 7367 },
	{ 0x7fb1, 0x7fb1, PDF_CMAP_SINGLE, 17451 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 7369 },
	{ 0x7fb3, 0x7fb7, PDF_CMAP_RANGE, 17452 },
	{ 0x7fb8, 0x7fb8, PDF_CMAP_SINGLE, 4878 },
	{ 0x7fb9, 0x7fb9, PDF_CMAP_SINGLE, 1785 },
	{ 0x7fba, 0x7fbb, PDF_CMAP_RANGE, 17457 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 5997 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 4309 },
	{ 0x7fbe, 0x7fbe, PDF_CMAP_SINGLE, 17459 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 7390 },
	{ 0x7fc0, 0x7fc0, PDF_CMAP_SINGLE, 17460 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 3803 },
	{ 0x7fc2, 0x7fc4, PDF_CMAP_RANGE, 17461 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 1286 },
	{ 0x7fc6, 0x7fc9, PDF_CMAP_RANGE, 17464 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 7064 },
	{ 0x7fcb, 0x7fcb, PDF_CMAP_SINGLE, 17468 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 4213 },
	{ 0x7fcd, 0x7fcd, PDF_CMAP_SINGLE, 17469 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 7391 },
	{ 0x7fcf, 0x7fd1, PDF_CMAP_RANGE, 17470 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 8622 },
	{ 0x7fd3, 0x7fd3, PDF_CMAP_SINGLE, 17473 },
	{ 0x7fd4, 0x7fd4, PDF_CMAP_SINGLE, 3925 },
	{ 0x7fd5, 0x7fd5, PDF_CMAP_SINGLE, 7392 },
	{ 0x7fd6, 0x7fd7, PDF_CMAP_RANGE, 17474 },
	{ 0x7fd8, 0x7fd8, PDF_CMAP_SINGLE, 3145 },
	{ 0x7fd9, 0x7fde, PDF_CMAP_RANGE, 17476 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 1461 },
	{ 0x7fe0, 0x7fe0, PDF_CMAP_SINGLE, 1383 },
	{ 0x7fe1, 0x7fe1, PDF_CMAP_SINGLE, 7394 },
	{ 0x7fe2, 0x7fe4, PDF_CMAP_RANGE, 17482 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 7393 },
	{ 0x7fe6, 0x7fe6, PDF_CMAP_SINGLE, 7395 },
	{ 0x7fe7, 0x7fe8, PDF_CMAP_RANGE, 17485 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 7396 },
	{ 0x7fea, 0x7fed, PDF_CMAP_RANGE, 17487 },
	{ 0x7fee, 0x7fee, PDF_CMAP_SINGLE, 7397 },
	{ 0x7fef, 0x7fef, PDF_CMAP_SINGLE, 17491 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 1897 },
	{ 0x7ff1, 0x7ff1, PDF_CMAP_SINGLE, 970 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 17492 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 7398 },
	{ 0x7ff4, 0x7ff8, PDF_CMAP_RANGE, 17493 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 8416 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 17498 },
	{ 0x7ffb, 0x7ffb, PDF_CMAP_SINGLE, 1610 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 4212 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 17499 },
	{ 0x8000, 0x8000, PDF_CMAP_SINGLE, 4146 },
	{ 0x8001, 0x8001, PDF_CMAP_SINGLE, 2490 },
	{ 0x8002, 0x8002, PDF_CMAP_SINGLE, 17502 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 2377 },
	{ 0x8004, 0x8004, PDF_CMAP_SINGLE, 6483 },
	{ 0x8005, 0x8005, PDF_CMAP_SINGLE, 4480 },
	{ 0x8006, 0x8006, PDF_CMAP_SINGLE, 6426 },
	{ 0x8007, 0x800a, PDF_CMAP_RANGE, 17503 },
	{ 0x800b, 0x800b, PDF_CMAP_SINGLE, 7127 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 1591 },
	{ 0x800d, 0x800d, PDF_CMAP_SINGLE, 3480 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 17507 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 2864 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 17509 },
	{ 0x8012, 0x8012, PDF_CMAP_SINGLE, 7115 },
	{ 0x8013, 0x8013, PDF_CMAP_SINGLE, 17510 },
	{ 0x8014, 0x8014, PDF_CMAP_SINGLE, 7116 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 1782 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 7117 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 1915 },
	{ 0x8018, 0x8018, PDF_CMAP_SINGLE, 4360 },
	{ 0x8019, 0x8019, PDF_CMAP_SINGLE, 989 },
	{ 0x801a, 0x801b, PDF_CMAP_RANGE, 17511 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 7118 },
	{ 0x801d, 0x801f, PDF_CMAP_RANGE, 17513 },
	{ 0x8020, 0x8020, PDF_CMAP_SINGLE, 7119 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 17516 },
	{ 0x8022, 0x8022, PDF_CMAP_SINGLE, 7120 },
	{ 0x8023, 0x8024, PDF_CMAP_RANGE, 17517 },
	{ 0x8025, 0x8027, PDF_CMAP_RANGE, 7121 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 7125 },
	{ 0x8029, 0x8029, PDF_CMAP_SINGLE, 7124 },
	{ 0x802a, 0x802a, PDF_CMAP_SINGLE, 2966 },
	{ 0x802b, 0x802b, PDF_CMAP_SINGLE, 17519 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 9666 },
	{ 0x802d, 0x802d, PDF_CMAP_SINGLE, 17520 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 9665 },
	{ 0x802f, 0x8030, PDF_CMAP_RANGE, 17521 },
	{ 0x8031, 0x8031, PDF_CMAP_SINGLE, 7126 },
	{ 0x8032, 0x8032, PDF_CMAP_SINGLE, 17523 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 1593 },
	{ 0x8034, 0x8034, PDF_CMAP_SINGLE, 17524 },
	{ 0x8035, 0x8035, PDF_CMAP_SINGLE, 7128 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 4149 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 5297 },
	{ 0x8038, 0x8038, PDF_CMAP_SINGLE, 3519 },
	{ 0x8039, 0x803a, PDF_CMAP_RANGE, 17525 },
	{ 0x803b, 0x803b, PDF_CMAP_SINGLE, 1281 },
	{ 0x803c, 0x803c, PDF_CMAP_SINGLE, 17527 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 1411 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 17528 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 1787 },
	{ 0x8040, 0x8041, PDF_CMAP_RANGE, 17529 },
	{ 0x8042, 0x8042, PDF_CMAP_SINGLE, 2903 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 7129 },
	{ 0x8044, 0x8045, PDF_CMAP_RANGE, 17531 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 7130 },
	{ 0x8047, 0x8049, PDF_CMAP_RANGE, 17533 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 2573 },
	{ 0x804b, 0x804b, PDF_CMAP_SINGLE, 2628 },
	{ 0x804c, 0x804c, PDF_CMAP_SINGLE, 4527 },
	{ 0x804d, 0x804d, PDF_CMAP_SINGLE, 7131 },
	{ 0x804e, 0x8051, PDF_CMAP_RANGE, 17536 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 7132 },
	{ 0x8053, 0x8053, PDF_CMAP_SINGLE, 17540 },
	{ 0x8054, 0x8054, PDF_CMAP_SINGLE, 2547 },
	{ 0x8055, 0x8055, PDF_CMAP_SINGLE, 17541 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 8488 },
	{ 0x8057, 0x8057, PDF_CMAP_SINGLE, 17542 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 3031 },
	{ 0x8059, 0x8059, PDF_CMAP_SINGLE, 17543 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 2314 },
	{ 0x805b, 0x805d, PDF_CMAP_RANGE, 17544 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 8601 },
	{ 0x805f, 0x8068, PDF_CMAP_RANGE, 17547 },
	{ 0x8069, 0x8069, PDF_CMAP_SINGLE, 7133 },
	{ 0x806a, 0x806a, PDF_CMAP_SINGLE, 1362 },
	{ 0x806b, 0x806e, PDF_CMAP_RANGE, 17557 },
	{ 0x806f, 0x806f, PDF_CMAP_SINGLE, 8215 },
	{ 0x8070, 0x8070, PDF_CMAP_SINGLE, 7840 },
	{ 0x8071, 0x8071, PDF_CMAP_SINGLE, 7134 },
	{ 0x8072, 0x8072, PDF_CMAP_SINGLE, 8485 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 8525 },
	{ 0x8074, 0x8074, PDF_CMAP_SINGLE, 17561 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 9668 },
	{ 0x8076, 0x8076, PDF_CMAP_SINGLE, 8349 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 8842 },
	{ 0x8078, 0x8078, PDF_CMAP_SINGLE, 17562 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 9667 },
	{ 0x807a, 0x807c, PDF_CMAP_RANGE, 17563 },
	{ 0x807d, 0x807d, PDF_CMAP_SINGLE, 8569 },
	{ 0x807e, 0x807e, PDF_CMAP_SINGLE, 8248 },
	{ 0x807f, 0x807f, PDF_CMAP_SINGLE, 6686 },
	{ 0x8080, 0x8080, PDF_CMAP_SINGLE, 6685 },
	{ 0x8081, 0x8082, PDF_CMAP_RANGE, 17566 },
	{ 0x8083, 0x8083, PDF_CMAP_SINGLE, 3541 },
	{ 0x8084, 0x8084, PDF_CMAP_SINGLE, 4197 },
	{ 0x8085, 0x8085, PDF_CMAP_SINGLE, 8533 },
	{ 0x8086, 0x8086, PDF_CMAP_SINGLE, 3510 },
	{ 0x8087, 0x8087, PDF_CMAP_SINGLE, 4473 },
	{ 0x8088, 0x8088, PDF_CMAP_SINGLE, 17568 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 3261 },
	{ 0x808a, 0x808a, PDF_CMAP_SINGLE, 17569 },
	{ 0x808b, 0x808b, PDF_CMAP_SINGLE, 2506 },
	{ 0x808c, 0x808c, PDF_CMAP_SINGLE, 2068 },
	{ 0x808d, 0x8092, PDF_CMAP_RANGE, 17570 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6513 },
	{ 0x8094, 0x8095, PDF_CMAP_RANGE, 17576 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 3951 },
	{ 0x8097, 0x8097, PDF_CMAP_SINGLE, 17578 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 4577 },
	{ 0x8099, 0x8099, PDF_CMAP_SINGLE, 17579 },
	{ 0x809a, 0x809a, PDF_CMAP_SINGLE, 1542 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 1747 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 6512 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 1737 },
	{ 0x809e, 0x809e, PDF_CMAP_SINGLE, 17580 },
	{ 0x809f, 0x809f, PDF_CMAP_SINGLE, 6511 },
	{ 0x80a0, 0x80a0, PDF_CMAP_SINGLE, 1227 },
	{ 0x80a1, 0x80a1, PDF_CMAP_SINGLE, 1826 },
	{ 0x80a2, 0x80a2, PDF_CMAP_SINGLE, 4522 },
	{ 0x80a3, 0x80a3, PDF_CMAP_SINGLE, 17581 },
	{ 0x80a4, 0x80a4, PDF_CMAP_SINGLE, 1681 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 1639 },
	{ 0x80a6, 0x80a8, PDF_CMAP_RANGE, 17582 },
	{ 0x80a9, 0x80a9, PDF_CMAP_SINGLE, 2138 },
	{ 0x80aa, 0x80aa, PDF_CMAP_SINGLE, 1627 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 6518 },
	{ 0x80ac, 0x80ac, PDF_CMAP_SINGLE, 17585 },
	{ 0x80ad, 0x80ad, PDF_CMAP_SINGLE, 6519 },
	{ 0x80ae, 0x80ae, PDF_CMAP_SINGLE, 964 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 2396 },
	{ 0x80b0, 0x80b0, PDF_CMAP_SINGLE, 17586 },
	{ 0x80b1, 0x80b1, PDF_CMAP_SINGLE, 6517 },
	{ 0x80b2, 0x80b2, PDF_CMAP_SINGLE, 4322 },
	{ 0x80b3, 0x80b3, PDF_CMAP_SINGLE, 17587 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 6520 },
	{ 0x80b5, 0x80b6, PDF_CMAP_RANGE, 17588 },
	{ 0x80b7, 0x80b7, PDF_CMAP_SINGLE, 6521 },
	{ 0x80b8, 0x80b9, PDF_CMAP_RANGE, 17590 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 1643 },
	{ 0x80bb, 0x80bb, PDF_CMAP_SINGLE, 17592 },
	{ 0x80bc, 0x80bc, PDF_CMAP_SINGLE, 6514 },
	{ 0x80bd, 0x80bd, PDF_CMAP_SINGLE, 6516 },
	{ 0x80be, 0x80be, PDF_CMAP_SINGLE, 3375 },
	{ 0x80bf, 0x80bf, PDF_CMAP_SINGLE, 4566 },
	{ 0x80c0, 0x80c0, PDF_CMAP_SINGLE, 4462 },
	{ 0x80c1, 0x80c1, PDF_CMAP_SINGLE, 3965 },
	{ 0x80c2, 0x80c2, PDF_CMAP_SINGLE, 6527 },
	{ 0x80c3, 0x80c3, PDF_CMAP_SINGLE, 3783 },
	{ 0x80c4, 0x80c4, PDF_CMAP_SINGLE, 6528 },
	{ 0x80c5, 0x80c5, PDF_CMAP_SINGLE, 17593 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 1417 },
	{ 0x80c7, 0x80cb, PDF_CMAP_RANGE, 17594 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 1052 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 6530 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 3576 },
	{ 0x80cf, 0x80d5, PDF_CMAP_RANGE, 17599 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 2967 },
	{ 0x80d7, 0x80d7, PDF_CMAP_SINGLE, 6531 },
	{ 0x80d8, 0x80d8, PDF_CMAP_SINGLE, 17606 },
	{ 0x80d9, 0x80d9, PDF_CMAP_SINGLE, 6529 },
	{ 0x80da, 0x80da, PDF_CMAP_SINGLE, 2976 },
	{ 0x80db, 0x80db, PDF_CMAP_SINGLE, 6526 },
	{ 0x80dc, 0x80dc, PDF_CMAP_SINGLE, 3387 },
	{ 0x80dd, 0x80dd, PDF_CMAP_SINGLE, 6533 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 1030 },
	{ 0x80df, 0x80e0, PDF_CMAP_RANGE, 17607 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1969 },
	{ 0x80e2, 0x80e3, PDF_CMAP_RANGE, 17609 },
	{ 0x80e4, 0x80e4, PDF_CMAP_SINGLE, 4717 },
	{ 0x80e5, 0x80e5, PDF_CMAP_SINGLE, 7111 },
	{ 0x80e6, 0x80e6, PDF_CMAP_SINGLE, 17611 },
	{ 0x80e7, 0x80ea, PDF_CMAP_RANGE, 6522 },
	{ 0x80eb, 0x80eb, PDF_CMAP_SINGLE, 6534 },
	{ 0x80ec, 0x80ec, PDF_CMAP_SINGLE, 6042 },
	{ 0x80ed, 0x80ed, PDF_CMAP_SINGLE, 6537 },
	{ 0x80ee, 0x80ee, PDF_CMAP_SINGLE, 17612 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 2421 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 4175 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 6535 },
	{ 0x80f2, 0x80f2, PDF_CMAP_SINGLE, 6540 },
	{ 0x80f3, 0x80f3, PDF_CMAP_SINGLE, 1767 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 6536 },
	{ 0x80f5, 0x80f5, PDF_CMAP_SINGLE, 17613 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 2187 },
	{ 0x80f7, 0x80f7, PDF_CMAP_SINGLE, 17614 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 4003 },
	{ 0x80f9, 0x80f9, PDF_CMAP_SINGLE, 17615 },
	{ 0x80fa, 0x80fa, PDF_CMAP_SINGLE, 962 },
	{ 0x80fb, 0x80fb, PDF_CMAP_SINGLE, 17616 },
	{ 0x80fc, 0x80fc, PDF_CMAP_SINGLE, 6541 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 2879 },
	{ 0x80fe, 0x80ff, PDF_CMAP_RANGE, 17617 },
	{ 0x8100, 0x8101, PDF_CMAP_RANGE, 17619 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 4523 },
	{ 0x8103, 0x8104, PDF_CMAP_RANGE, 17621 },
	{ 0x8105, 0x8105, PDF_CMAP_SINGLE, 8658 },
	{ 0x8106, 0x8106, PDF_CMAP_SINGLE, 1379 },
	{ 0x8107, 0x8108, PDF_CMAP_RANGE, 17623 },
	{ 0x8109, 0x8109, PDF_CMAP_SINGLE, 2717 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 2092 },
	{ 0x810b, 0x810c, PDF_CMAP_RANGE, 17625 },
	{ 0x810d, 0x810e, PDF_CMAP_RANGE, 6538 },
	{ 0x810f, 0x810f, PDF_CMAP_SINGLE, 4387 },
	{ 0x8110, 0x8110, PDF_CMAP_SINGLE, 3081 },
	{ 0x8111, 0x8111, PDF_CMAP_SINGLE, 2871 },
	{ 0x8112, 0x8112, PDF_CMAP_SINGLE, 6543 },
	{ 0x8113, 0x8113, PDF_CMAP_SINGLE, 2920 },
	{ 0x8114, 0x8114, PDF_CMAP_SINGLE, 4873 },
	{ 0x8115, 0x8115, PDF_CMAP_SINGLE, 17627 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 1144 },
	{ 0x8117, 0x8117, PDF_CMAP_SINGLE, 17628 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 6548 },
	{ 0x8119, 0x8119, PDF_CMAP_SINGLE, 17629 },
	{ 0x811a, 0x811a, PDF_CMAP_SINGLE, 2198 },
	{ 0x811b, 0x811b, PDF_CMAP_SINGLE, 9415 },
	{ 0x811c, 0x811d, PDF_CMAP_RANGE, 17630 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 6546 },
	{ 0x811f, 0x812b, PDF_CMAP_RANGE, 17632 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 6547 },
	{ 0x812d, 0x812e, PDF_CMAP_RANGE, 17645 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 1703 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 17647 },
	{ 0x8131, 0x8131, PDF_CMAP_SINGLE, 3714 },
	{ 0x8132, 0x8132, PDF_CMAP_SINGLE, 6549 },
	{ 0x8133, 0x8135, PDF_CMAP_RANGE, 17648 },
	{ 0x8136, 0x8136, PDF_CMAP_SINGLE, 6545 },
	{ 0x8137, 0x8137, PDF_CMAP_SINGLE, 17651 },
	{ 0x8138, 0x8138, PDF_CMAP_SINGLE, 2556 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 8825 },
	{ 0x813a, 0x813d, PDF_CMAP_RANGE, 17652 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 3009 },
	{ 0x813f, 0x8145, PDF_CMAP_RANGE, 17656 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 3654 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 17663 },
	{ 0x8148, 0x8148, PDF_CMAP_SINGLE, 6550 },
	{ 0x8149, 0x8149, PDF_CMAP_SINGLE, 17664 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 2459 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 4159 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 6551 },
	{ 0x814d, 0x814d, PDF_CMAP_SINGLE, 17665 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 8483 },
	{ 0x814f, 0x814f, PDF_CMAP_SINGLE, 17666 },
	{ 0x8150, 0x8150, PDF_CMAP_SINGLE, 1706 },
	{ 0x8151, 0x8151, PDF_CMAP_SINGLE, 1704 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 17667 },
	{ 0x8153, 0x8153, PDF_CMAP_SINGLE, 6552 },
	{ 0x8154, 0x8154, PDF_CMAP_SINGLE, 3128 },
	{ 0x8155, 0x8155, PDF_CMAP_SINGLE, 3748 },
	{ 0x8156, 0x8156, PDF_CMAP_SINGLE, 9413 },
	{ 0x8157, 0x8158, PDF_CMAP_RANGE, 17668 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 6554 },
	{ 0x815b, 0x815f, PDF_CMAP_RANGE, 17670 },
	{ 0x8160, 0x8160, PDF_CMAP_SINGLE, 6557 },
	{ 0x8161, 0x8161, PDF_CMAP_SINGLE, 9417 },
	{ 0x8162, 0x8164, PDF_CMAP_RANGE, 17675 },
	{ 0x8165, 0x8165, PDF_CMAP_SINGLE, 3987 },
	{ 0x8166, 0x8166, PDF_CMAP_SINGLE, 8340 },
	{ 0x8167, 0x8167, PDF_CMAP_SINGLE, 6562 },
	{ 0x8168, 0x8168, PDF_CMAP_SINGLE, 17678 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 6558 },
	{ 0x816a, 0x816a, PDF_CMAP_SINGLE, 17679 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 8854 },
	{ 0x816c, 0x816c, PDF_CMAP_SINGLE, 17680 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 6561 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 3284 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 17681 },
	{ 0x8170, 0x8170, PDF_CMAP_SINGLE, 4133 },
	{ 0x8171, 0x8171, PDF_CMAP_SINGLE, 6556 },
	{ 0x8172, 0x8173, PDF_CMAP_RANGE, 17682 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6553 },
	{ 0x8175, 0x8177, PDF_CMAP_RANGE, 17684 },
	{ 0x8178, 0x8178, PDF_CMAP_SINGLE, 7799 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 1716 },
	{ 0x817a, 0x817a, PDF_CMAP_SINGLE, 3910 },
	{ 0x817b, 0x817b, PDF_CMAP_SINGLE, 2888 },
	{ 0x817c, 0x817d, PDF_CMAP_RANGE, 6559 },
	{ 0x817e, 0x817e, PDF_CMAP_SINGLE, 3629 },
	{ 0x817f, 0x817f, PDF_CMAP_SINGLE, 3705 },
	{ 0x8180, 0x8180, PDF_CMAP_SINGLE, 1021 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 17687 },
	{ 0x8182, 0x8182, PDF_CMAP_SINGLE, 6566 },
	{ 0x8183, 0x8187, PDF_CMAP_RANGE, 17688 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 6565 },
	{ 0x8189, 0x8189, PDF_CMAP_SINGLE, 17693 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 1145 },
	{ 0x818b, 0x818e, PDF_CMAP_RANGE, 17694 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 1755 },
	{ 0x8190, 0x8190, PDF_CMAP_SINGLE, 17698 },
	{ 0x8191, 0x8191, PDF_CMAP_SINGLE, 6567 },
	{ 0x8192, 0x8197, PDF_CMAP_RANGE, 17699 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 1109 },
	{ 0x8199, 0x8199, PDF_CMAP_SINGLE, 17705 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 7942 },
	{ 0x819b, 0x819b, PDF_CMAP_SINGLE, 3608 },
	{ 0x819c, 0x819c, PDF_CMAP_SINGLE, 2823 },
	{ 0x819d, 0x819d, PDF_CMAP_SINGLE, 3858 },
	{ 0x819e, 0x819f, PDF_CMAP_RANGE, 17706 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 8101 },
	{ 0x81a1, 0x81a2, PDF_CMAP_RANGE, 17708 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 6569 },
	{ 0x81a4, 0x81a5, PDF_CMAP_RANGE, 17710 },
	{ 0x81a6, 0x81a6, PDF_CMAP_SINGLE, 6576 },
	{ 0x81a7, 0x81a7, PDF_CMAP_SINGLE, 17712 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 2995 },
	{ 0x81a9, 0x81a9, PDF_CMAP_SINGLE, 8345 },
	{ 0x81aa, 0x81aa, PDF_CMAP_SINGLE, 6570 },
	{ 0x81ab, 0x81b2, PDF_CMAP_RANGE, 17713 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 3326 },
	{ 0x81b4, 0x81b9, PDF_CMAP_RANGE, 17721 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 5697 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 6574 },
	{ 0x81bc, 0x81bc, PDF_CMAP_SINGLE, 17727 },
	{ 0x81bd, 0x81bd, PDF_CMAP_SINGLE, 7853 },
	{ 0x81be, 0x81be, PDF_CMAP_SINGLE, 9416 },
	{ 0x81bf, 0x81bf, PDF_CMAP_SINGLE, 8360 },
	{ 0x81c0, 0x81c0, PDF_CMAP_SINGLE, 3711 },
	{ 0x81c1, 0x81c1, PDF_CMAP_SINGLE, 6575 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 1092 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 4252 },
	{ 0x81c4, 0x81c5, PDF_CMAP_RANGE, 17728 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 4195 },
	{ 0x81c7, 0x81c8, PDF_CMAP_RANGE, 17730 },
	{ 0x81c9, 0x81c9, PDF_CMAP_SINGLE, 8223 },
	{ 0x81ca, 0x81ca, PDF_CMAP_SINGLE, 6573 },
	{ 0x81cb, 0x81cb, PDF_CMAP_SINGLE, 17732 },
	{ 0x81cc, 0x81cc, PDF_CMAP_SINGLE, 6571 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 8389 },
	{ 0x81ce, 0x81ce, PDF_CMAP_SINGLE, 17733 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 9418 },
	{ 0x81d0, 0x81d7, PDF_CMAP_RANGE, 17734 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 8176 },
	{ 0x81d9, 0x81d9, PDF_CMAP_SINGLE, 17742 },
	{ 0x81da, 0x81da, PDF_CMAP_SINGLE, 9414 },
	{ 0x81db, 0x81de, PDF_CMAP_RANGE, 17743 },
	{ 0x81df, 0x81df, PDF_CMAP_SINGLE, 8797 },
	{ 0x81e0, 0x81e0, PDF_CMAP_SINGLE, 8926 },
	{ 0x81e1, 0x81e2, PDF_CMAP_RANGE, 17747 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 1249 },
	{ 0x81e4, 0x81e6, PDF_CMAP_RANGE, 17749 },
	{ 0x81e7, 0x81e7, PDF_CMAP_SINGLE, 6402 },
	{ 0x81e8, 0x81e8, PDF_CMAP_SINGLE, 8236 },
	{ 0x81e9, 0x81e9, PDF_CMAP_SINGLE, 17752 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 4657 },
	{ 0x81eb, 0x81eb, PDF_CMAP_SINGLE, 17753 },
	{ 0x81ec, 0x81ec, PDF_CMAP_SINGLE, 7337 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 1305 },
	{ 0x81ee, 0x81f2, PDF_CMAP_RANGE, 17754 },
	{ 0x81f3, 0x81f4, PDF_CMAP_RANGE, 4544 },
	{ 0x81f5, 0x81f9, PDF_CMAP_RANGE, 17759 },
	{ 0x81fa, 0x81fa, PDF_CMAP_SINGLE, 8545 },
	{ 0x81fb, 0x81fb, PDF_CMAP_SINGLE, 4490 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 2297 },
	{ 0x81fd, 0x81fd, PDF_CMAP_SINGLE, 17764 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 7333 },
	{ 0x81ff, 0x81ff, PDF_CMAP_SINGLE, 17765 },
	{ 0x8200, 0x8200, PDF_CMAP_SINGLE, 4143 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 7334 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 17766 },
	{ 0x8204, 0x8204, PDF_CMAP_SINGLE, 7336 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 2298 },
	{ 0x8206, 0x8206, PDF_CMAP_SINGLE, 4292 },
	{ 0x8207, 0x8207, PDF_CMAP_SINGLE, 8763 },
	{ 0x8208, 0x8208, PDF_CMAP_SINGLE, 8665 },
	{ 0x8209, 0x8209, PDF_CMAP_SINGLE, 8139 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 8137 },
	{ 0x820b, 0x820b, PDF_CMAP_SINGLE, 17767 },
	{ 0x820c, 0x820d, PDF_CMAP_RANGE, 3353 },
	{ 0x820e, 0x820f, PDF_CMAP_RANGE, 17768 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 7266 },
	{ 0x8211, 0x8211, PDF_CMAP_SINGLE, 17770 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 3453 },
	{ 0x8213, 0x8213, PDF_CMAP_SINGLE, 17771 },
	{ 0x8214, 0x8214, PDF_CMAP_SINGLE, 3653 },
	{ 0x8215, 0x821a, PDF_CMAP_RANGE, 17772 },
	{ 0x821b, 0x821b, PDF_CMAP_SINGLE, 5656 },
	{ 0x821c, 0x821c, PDF_CMAP_SINGLE, 3497 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 17778 },
	{ 0x821e, 0x821e, PDF_CMAP_SINGLE, 3831 },
	{ 0x821f, 0x821f, PDF_CMAP_SINGLE, 4570 },
	{ 0x8220, 0x8220, PDF_CMAP_SINGLE, 17779 },
	{ 0x8221, 0x8223, PDF_CMAP_RANGE, 7339 },
	{ 0x8224, 0x8227, PDF_CMAP_RANGE, 17780 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 7344 },
	{ 0x8229, 0x8229, PDF_CMAP_SINGLE, 17784 },
	{ 0x822a, 0x822a, PDF_CMAP_SINGLE, 1908 },
	{ 0x822b, 0x822b, PDF_CMAP_SINGLE, 7345 },
	{ 0x822c, 0x822c, PDF_CMAP_SINGLE, 1006 },
	{ 0x822d, 0x822d, PDF_CMAP_SINGLE, 7342 },
	{ 0x822e, 0x822e, PDF_CMAP_SINGLE, 17785 },
	{ 0x822f, 0x822f, PDF_CMAP_SINGLE, 7343 },
	{ 0x8230, 0x8230, PDF_CMAP_SINGLE, 2163 },
	{ 0x8231, 0x8231, PDF_CMAP_SINGLE, 1180 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 17786 },
	{ 0x8233, 0x8234, PDF_CMAP_RANGE, 7348 },
	{ 0x8235, 0x8235, PDF_CMAP_SINGLE, 1573 },
	{ 0x8236, 0x8236, PDF_CMAP_SINGLE, 1143 },
	{ 0x8237, 0x8237, PDF_CMAP_SINGLE, 3900 },
	{ 0x8238, 0x8238, PDF_CMAP_SINGLE, 7346 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 1327 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 17787 },
	{ 0x823b, 0x823b, PDF_CMAP_SINGLE, 7347 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 17788 },
	{ 0x823e, 0x823e, PDF_CMAP_SINGLE, 7350 },
	{ 0x823f, 0x8243, PDF_CMAP_RANGE, 17790 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 7351 },
	{ 0x8245, 0x8246, PDF_CMAP_RANGE, 17795 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3672 },
	{ 0x8248, 0x8248, PDF_CMAP_SINGLE, 17797 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 7352 },
	{ 0x824a, 0x824a, PDF_CMAP_SINGLE, 17798 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 7353 },
	{ 0x824c, 0x824e, PDF_CMAP_RANGE, 17799 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 7354 },
	{ 0x8250, 0x8257, PDF_CMAP_RANGE, 17802 },
	{ 0x8258, 0x8258, PDF_CMAP_SINGLE, 3527 },
	{ 0x8259, 0x8259, PDF_CMAP_SINGLE, 7777 },
	{ 0x825a, 0x825a, PDF_CMAP_SINGLE, 7355 },
	{ 0x825b, 0x825e, PDF_CMAP_RANGE, 17810 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 7356 },
	{ 0x8260, 0x8263, PDF_CMAP_RANGE, 17814 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 9711 },
	{ 0x8265, 0x8265, PDF_CMAP_SINGLE, 17818 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 8088 },
	{ 0x8267, 0x8267, PDF_CMAP_SINGLE, 17819 },
	{ 0x8268, 0x8268, PDF_CMAP_SINGLE, 7357 },
	{ 0x8269, 0x826a, PDF_CMAP_RANGE, 17820 },
	{ 0x826b, 0x826b, PDF_CMAP_SINGLE, 9712 },
	{ 0x826c, 0x826d, PDF_CMAP_RANGE, 17822 },
	{ 0x826e, 0x826e, PDF_CMAP_SINGLE, 7388 },
	{ 0x826f, 0x826f, PDF_CMAP_SINGLE, 2565 },
	{ 0x8270, 0x8270, PDF_CMAP_SINGLE, 2139 },
	{ 0x8271, 0x8271, PDF_CMAP_SINGLE, 8072 },
	{ 0x8272, 0x8272, PDF_CMAP_SINGLE, 3300 },
	{ 0x8273, 0x8273, PDF_CMAP_SINGLE, 4103 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 6001 },
	{ 0x8275, 0x8276, PDF_CMAP_RANGE, 17824 },
	{ 0x8277, 0x8277, PDF_CMAP_SINGLE, 8696 },
	{ 0x8278, 0x8278, PDF_CMAP_SINGLE, 17826 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 5089 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 4188 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 17827 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 5090 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 951 },
	{ 0x827f, 0x827f, PDF_CMAP_SINGLE, 5091 },
	{ 0x8280, 0x8281, PDF_CMAP_RANGE, 17829 },
	{ 0x8282, 0x8282, PDF_CMAP_SINGLE, 2219 },
	{ 0x8283, 0x8283, PDF_CMAP_SINGLE, 17831 },
	{ 0x8284, 0x8284, PDF_CMAP_SINGLE, 5095 },
	{ 0x8285, 0x8287, PDF_CMAP_RANGE, 17832 },
	{ 0x8288, 0x8288, PDF_CMAP_SINGLE, 4728 },
	{ 0x8289, 0x8289, PDF_CMAP_SINGLE, 17835 },
	{ 0x828a, 0x828a, PDF_CMAP_SINGLE, 5093 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 4312 },
	{ 0x828c, 0x828c, PDF_CMAP_SINGLE, 17836 },
	{ 0x828d, 0x828d, PDF_CMAP_SINGLE, 3343 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 5096 },
	{ 0x828f, 0x828f, PDF_CMAP_SINGLE, 5092 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 17837 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 5097 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 2727 },
	{ 0x8293, 0x8296, PDF_CMAP_RANGE, 17838 },
	{ 0x8297, 0x8297, PDF_CMAP_SINGLE, 5098 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 5107 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 5099 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 17842 },
	{ 0x829c, 0x829c, PDF_CMAP_SINGLE, 3822 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 4516 },
	{ 0x829e, 0x829e, PDF_CMAP_SINGLE, 17844 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 5117 },
	{ 0x82a0, 0x82a0, PDF_CMAP_SINGLE, 17845 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 5115 },
	{ 0x82a2, 0x82a3, PDF_CMAP_RANGE, 17846 },
	{ 0x82a4, 0x82a4, PDF_CMAP_SINGLE, 5120 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 2231 },
	{ 0x82a6, 0x82a6, PDF_CMAP_SINGLE, 2642 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 17848 },
	{ 0x82a8, 0x82a8, PDF_CMAP_SINGLE, 5094 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 5113 },
	{ 0x82aa, 0x82aa, PDF_CMAP_SINGLE, 5116 },
	{ 0x82ab, 0x82ab, PDF_CMAP_SINGLE, 5100 },
	{ 0x82ac, 0x82ac, PDF_CMAP_SINGLE, 1647 },
	{ 0x82ad, 0x82ad, PDF_CMAP_SINGLE, 976 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 5109 },
	{ 0x82af, 0x82af, PDF_CMAP_SINGLE, 3977 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 5103 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1981 },
	{ 0x82b2, 0x82b2, PDF_CMAP_SINGLE, 17849 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 1625 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 5114 },
	{ 0x82b5, 0x82b6, PDF_CMAP_RANGE, 17850 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 5108 },
	{ 0x82b8, 0x82b8, PDF_CMAP_SINGLE, 5101 },
	{ 0x82b9, 0x82b9, PDF_CMAP_SINGLE, 3160 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 17852 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 8994 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 17853 },
	{ 0x82bd, 0x82bd, PDF_CMAP_SINGLE, 4072 },
	{ 0x82be, 0x82be, PDF_CMAP_SINGLE, 5102 },
	{ 0x82bf, 0x82c0, PDF_CMAP_RANGE, 17854 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 5112 },
	{ 0x82c2, 0x82c3, PDF_CMAP_RANGE, 17856 },
	{ 0x82c4, 0x82c4, PDF_CMAP_SINGLE, 5118 },
	{ 0x82c5, 0x82c6, PDF_CMAP_RANGE, 17858 },
	{ 0x82c7, 0x82c7, PDF_CMAP_SINGLE, 3772 },
	{ 0x82c8, 0x82c8, PDF_CMAP_SINGLE, 5104 },
	{ 0x82c9, 0x82c9, PDF_CMAP_SINGLE, 17860 },
	{ 0x82ca, 0x82ca, PDF_CMAP_SINGLE, 5105 },
	{ 0x82cb, 0x82cc, PDF_CMAP_RANGE, 5110 },
	{ 0x82cd, 0x82cd, PDF_CMAP_SINGLE, 1179 },
	{ 0x82ce, 0x82ce, PDF_CMAP_SINGLE, 5119 },
	{ 0x82cf, 0x82cf, PDF_CMAP_SINGLE, 3530 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 17861 },
	{ 0x82d1, 0x82d1, PDF_CMAP_SINGLE, 4346 },
	{ 0x82d2, 0x82d2, PDF_CMAP_SINGLE, 5129 },
	{ 0x82d3, 0x82d3, PDF_CMAP_SINGLE, 5133 },
	{ 0x82d4, 0x82d4, PDF_CMAP_SINGLE, 3577 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 5140 },
	{ 0x82d6, 0x82d6, PDF_CMAP_SINGLE, 17862 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 2796 },
	{ 0x82d8, 0x82d8, PDF_CMAP_SINGLE, 5130 },
	{ 0x82d9, 0x82da, PDF_CMAP_RANGE, 17863 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 2382 },
	{ 0x82dc, 0x82dc, PDF_CMAP_SINGLE, 5127 },
	{ 0x82dd, 0x82dd, PDF_CMAP_SINGLE, 17865 },
	{ 0x82de, 0x82de, PDF_CMAP_SINGLE, 1029 },
	{ 0x82df, 0x82df, PDF_CMAP_SINGLE, 1807 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 5139 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 5121 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 17866 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 5106 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 5124 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 3279 },
	{ 0x82e6, 0x82e6, PDF_CMAP_SINGLE, 2413 },
	{ 0x82e7, 0x82e7, PDF_CMAP_SINGLE, 9014 },
	{ 0x82e8, 0x82ea, PDF_CMAP_RANGE, 17867 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3316 },
	{ 0x82ec, 0x82ee, PDF_CMAP_RANGE, 17870 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 1062 },
	{ 0x82f0, 0x82f0, PDF_CMAP_SINGLE, 17873 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 4231 },
	{ 0x82f2, 0x82f3, PDF_CMAP_RANGE, 17874 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 5128 },
	{ 0x82f5, 0x82f6, PDF_CMAP_RANGE, 17876 },
	{ 0x82f7, 0x82f7, PDF_CMAP_SINGLE, 5123 },
	{ 0x82f8, 0x82f8, PDF_CMAP_SINGLE, 17878 },
	{ 0x82f9, 0x82f9, PDF_CMAP_SINGLE, 3034 },
	{ 0x82fa, 0x82fa, PDF_CMAP_SINGLE, 17879 },
	{ 0x82fb, 0x82fb, PDF_CMAP_SINGLE, 5132 },
	{ 0x82fc, 0x82ff, PDF_CMAP_RANGE, 17880 },
	{ 0x8300, 0x8300, PDF_CMAP_SINGLE, 17884 },
	{ 0x8301, 0x8301, PDF_CMAP_SINGLE, 4639 },
	{ 0x8302, 0x8302, PDF_CMAP_SINGLE, 2740 },
	{ 0x8303, 0x8303, PDF_CMAP_SINGLE, 1619 },
	{ 0x8304, 0x8304, PDF_CMAP_SINGLE, 3150 },
	{ 0x8305, 0x8305, PDF_CMAP_SINGLE, 2734 },
	{ 0x8306, 0x8306, PDF_CMAP_SINGLE, 5136 },
	{ 0x8307, 0x8307, PDF_CMAP_SINGLE, 5126 },
	{ 0x8308, 0x8308, PDF_CMAP_SINGLE, 5145 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 5122 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 17885 },
	{ 0x830c, 0x830c, PDF_CMAP_SINGLE, 5131 },
	{ 0x830d, 0x830d, PDF_CMAP_SINGLE, 17887 },
	{ 0x830e, 0x830e, PDF_CMAP_SINGLE, 2260 },
	{ 0x830f, 0x830f, PDF_CMAP_SINGLE, 5125 },
	{ 0x8310, 0x8310, PDF_CMAP_SINGLE, 17888 },
	{ 0x8311, 0x8311, PDF_CMAP_SINGLE, 5134 },
	{ 0x8312, 0x8313, PDF_CMAP_RANGE, 17889 },
	{ 0x8314, 0x8315, PDF_CMAP_RANGE, 5137 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 17891 },
	{ 0x8317, 0x8317, PDF_CMAP_SINGLE, 5158 },
	{ 0x8318, 0x8319, PDF_CMAP_RANGE, 17892 },
	{ 0x831a, 0x831a, PDF_CMAP_SINGLE, 5135 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 5166 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 5141 },
	{ 0x831d, 0x8326, PDF_CMAP_RANGE, 17894 },
	{ 0x8327, 0x8327, PDF_CMAP_SINGLE, 2142 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 1351 },
	{ 0x8329, 0x832a, PDF_CMAP_RANGE, 17904 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 2728 },
	{ 0x832c, 0x832c, PDF_CMAP_SINGLE, 1198 },
	{ 0x832d, 0x832d, PDF_CMAP_SINGLE, 5160 },
	{ 0x832e, 0x832e, PDF_CMAP_SINGLE, 17906 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 5152 },
	{ 0x8330, 0x8330, PDF_CMAP_SINGLE, 17907 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 5149 },
	{ 0x8332, 0x8332, PDF_CMAP_SINGLE, 17908 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 5162 },
	{ 0x8334, 0x8334, PDF_CMAP_SINGLE, 5148 },
	{ 0x8335, 0x8335, PDF_CMAP_SINGLE, 4215 },
	{ 0x8336, 0x8336, PDF_CMAP_SINGLE, 1199 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 17909 },
	{ 0x8338, 0x8338, PDF_CMAP_SINGLE, 3250 },
	{ 0x8339, 0x8339, PDF_CMAP_SINGLE, 3262 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 5161 },
	{ 0x833b, 0x833b, PDF_CMAP_SINGLE, 17910 },
	{ 0x833c, 0x833c, PDF_CMAP_SINGLE, 5147 },
	{ 0x833d, 0x833f, PDF_CMAP_RANGE, 17911 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 5157 },
	{ 0x8341, 0x8342, PDF_CMAP_RANGE, 17914 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 5155 },
	{ 0x8344, 0x8345, PDF_CMAP_RANGE, 17916 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 2258 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 5154 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 17918 },
	{ 0x8349, 0x8349, PDF_CMAP_SINGLE, 1188 },
	{ 0x834a, 0x834e, PDF_CMAP_RANGE, 17919 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 5153 },
	{ 0x8350, 0x8350, PDF_CMAP_SINGLE, 2153 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 5142 },
	{ 0x8352, 0x8352, PDF_CMAP_SINGLE, 2009 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 17924 },
	{ 0x8354, 0x8354, PDF_CMAP_SINGLE, 2526 },
	{ 0x8355, 0x8359, PDF_CMAP_RANGE, 17925 },
	{ 0x835a, 0x835a, PDF_CMAP_SINGLE, 2119 },
	{ 0x835b, 0x835c, PDF_CMAP_RANGE, 5143 },
	{ 0x835d, 0x835d, PDF_CMAP_SINGLE, 17930 },
	{ 0x835e, 0x835e, PDF_CMAP_SINGLE, 5151 },
	{ 0x835f, 0x835f, PDF_CMAP_SINGLE, 5156 },
	{ 0x8360, 0x8360, PDF_CMAP_SINGLE, 5159 },
	{ 0x8361, 0x8361, PDF_CMAP_SINGLE, 1429 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 17931 },
	{ 0x8363, 0x8363, PDF_CMAP_SINGLE, 3252 },
	{ 0x8364, 0x8364, PDF_CMAP_SINGLE, 2044 },
	{ 0x8365, 0x8365, PDF_CMAP_SINGLE, 5164 },
	{ 0x8366, 0x8366, PDF_CMAP_SINGLE, 5163 },
	{ 0x8367, 0x8367, PDF_CMAP_SINGLE, 4240 },
	{ 0x8368, 0x8368, PDF_CMAP_SINGLE, 5165 },
	{ 0x8369, 0x8369, PDF_CMAP_SINGLE, 5167 },
	{ 0x836a, 0x836a, PDF_CMAP_SINGLE, 5169 },
	{ 0x836b, 0x836b, PDF_CMAP_SINGLE, 4216 },
	{ 0x836c, 0x836c, PDF_CMAP_SINGLE, 5168 },
	{ 0x836d, 0x836e, PDF_CMAP_RANGE, 5170 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 4144 },
	{ 0x8370, 0x8376, PDF_CMAP_RANGE, 17932 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 1920 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 5173 },
	{ 0x8379, 0x837a, PDF_CMAP_RANGE, 17939 },
	{ 0x837b, 0x837b, PDF_CMAP_SINGLE, 5186 },
	{ 0x837c, 0x837c, PDF_CMAP_SINGLE, 5181 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 5184 },
	{ 0x837e, 0x8384, PDF_CMAP_RANGE, 17941 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 5180 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 3053 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 17948 },
	{ 0x8389, 0x8389, PDF_CMAP_SINGLE, 2525 },
	{ 0x838a, 0x838a, PDF_CMAP_SINGLE, 8875 },
	{ 0x838b, 0x838d, PDF_CMAP_RANGE, 17950 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 3304 },
	{ 0x838f, 0x8391, PDF_CMAP_RANGE, 17953 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 5146 },
	{ 0x8393, 0x8393, PDF_CMAP_SINGLE, 5178 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 17956 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 8127 },
	{ 0x8397, 0x8397, PDF_CMAP_SINGLE, 17958 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 5187 },
	{ 0x8399, 0x839a, PDF_CMAP_RANGE, 17959 },
	{ 0x839b, 0x839b, PDF_CMAP_SINGLE, 5150 },
	{ 0x839c, 0x839c, PDF_CMAP_SINGLE, 5179 },
	{ 0x839d, 0x839d, PDF_CMAP_SINGLE, 17961 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 5188 },
	{ 0x839f, 0x839f, PDF_CMAP_SINGLE, 17962 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 5176 },
	{ 0x83a1, 0x83a1, PDF_CMAP_SINGLE, 17963 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 8061 },
	{ 0x83a3, 0x83a6, PDF_CMAP_RANGE, 17964 },
	{ 0x83a7, 0x83a7, PDF_CMAP_SINGLE, 9011 },
	{ 0x83a8, 0x83a8, PDF_CMAP_SINGLE, 5189 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 5183 },
	{ 0x83aa, 0x83aa, PDF_CMAP_SINGLE, 5177 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 2829 },
	{ 0x83ac, 0x83af, PDF_CMAP_RANGE, 17968 },
	{ 0x83b0, 0x83b0, PDF_CMAP_SINGLE, 5172 },
	{ 0x83b1, 0x83b1, PDF_CMAP_SINGLE, 2462 },
	{ 0x83b2, 0x83b2, PDF_CMAP_SINGLE, 2548 },
	{ 0x83b3, 0x83b4, PDF_CMAP_RANGE, 5174 },
	{ 0x83b5, 0x83b5, PDF_CMAP_SINGLE, 17972 },
	{ 0x83b6, 0x83b6, PDF_CMAP_SINGLE, 5182 },
	{ 0x83b7, 0x83b7, PDF_CMAP_SINGLE, 2054 },
	{ 0x83b8, 0x83b8, PDF_CMAP_SINGLE, 5185 },
	{ 0x83b9, 0x83b9, PDF_CMAP_SINGLE, 4237 },
	{ 0x83ba, 0x83ba, PDF_CMAP_SINGLE, 5190 },
	{ 0x83bb, 0x83bb, PDF_CMAP_SINGLE, 17973 },
	{ 0x83bc, 0x83bc, PDF_CMAP_SINGLE, 5191 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 2732 },
	{ 0x83be, 0x83bf, PDF_CMAP_RANGE, 17974 },
	{ 0x83c0, 0x83c0, PDF_CMAP_SINGLE, 5214 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 5192 },
	{ 0x83c2, 0x83c4, PDF_CMAP_RANGE, 17976 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 5213 },
	{ 0x83c6, 0x83c6, PDF_CMAP_SINGLE, 17979 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 1814 },
	{ 0x83c8, 0x83c9, PDF_CMAP_RANGE, 17980 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 2308 },
	{ 0x83cb, 0x83cb, PDF_CMAP_SINGLE, 17982 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 2345 },
	{ 0x83cd, 0x83ce, PDF_CMAP_RANGE, 17983 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 1921 },
	{ 0x83d0, 0x83d3, PDF_CMAP_RANGE, 17985 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 5206 },
	{ 0x83d5, 0x83d5, PDF_CMAP_SINGLE, 17989 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 5201 },
	{ 0x83d7, 0x83d7, PDF_CMAP_SINGLE, 17990 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 5195 },
	{ 0x83d9, 0x83db, PDF_CMAP_RANGE, 17991 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 1170 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 5199 },
	{ 0x83de, 0x83de, PDF_CMAP_SINGLE, 17994 },
	{ 0x83df, 0x83df, PDF_CMAP_SINGLE, 5207 },
	{ 0x83e0, 0x83e0, PDF_CMAP_SINGLE, 1131 },
	{ 0x83e1, 0x83e1, PDF_CMAP_SINGLE, 5217 },
	{ 0x83e2, 0x83e4, PDF_CMAP_RANGE, 17995 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 5194 },
	{ 0x83e6, 0x83e8, PDF_CMAP_RANGE, 17998 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 3055 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 5212 },
	{ 0x83eb, 0x83ee, PDF_CMAP_RANGE, 18001 },
	{ 0x83ef, 0x83ef, PDF_CMAP_SINGLE, 8013 },
	{ 0x83f0, 0x83f0, PDF_CMAP_SINGLE, 5216 },
	{ 0x83f1, 0x83f1, PDF_CMAP_SINGLE, 2603 },
	{ 0x83f2, 0x83f2, PDF_CMAP_SINGLE, 1635 },
	{ 0x83f3, 0x83f7, PDF_CMAP_RANGE, 18005 },
	{ 0x83f8, 0x83f9, PDF_CMAP_RANGE, 5210 },
	{ 0x83fa, 0x83fc, PDF_CMAP_RANGE, 18010 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 5200 },
	{ 0x83fe, 0x83ff, PDF_CMAP_RANGE, 18013 },
	{ 0x8400, 0x8400, PDF_CMAP_SINGLE, 18015 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 5193 },
	{ 0x8402, 0x8402, PDF_CMAP_SINGLE, 18016 },
	{ 0x8403, 0x8403, PDF_CMAP_SINGLE, 5209 },
	{ 0x8404, 0x8404, PDF_CMAP_SINGLE, 3620 },
	{ 0x8405, 0x8405, PDF_CMAP_SINGLE, 18017 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 5205 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 9012 },
	{ 0x8408, 0x8409, PDF_CMAP_RANGE, 18018 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 8177 },
	{ 0x840b, 0x840b, PDF_CMAP_SINGLE, 5198 },
	{ 0x840c, 0x840c, PDF_CMAP_SINGLE, 2765 },
	{ 0x840d, 0x840d, PDF_CMAP_SINGLE, 3035 },
	{ 0x840e, 0x840e, PDF_CMAP_SINGLE, 3773 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 5208 },
	{ 0x8410, 0x8410, PDF_CMAP_SINGLE, 18020 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 5204 },
	{ 0x8412, 0x8417, PDF_CMAP_RANGE, 18021 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 5197 },
	{ 0x8419, 0x841b, PDF_CMAP_RANGE, 18027 },
	{ 0x841c, 0x841c, PDF_CMAP_SINGLE, 5202 },
	{ 0x841d, 0x841d, PDF_CMAP_SINGLE, 2691 },
	{ 0x841e, 0x8423, PDF_CMAP_RANGE, 18030 },
	{ 0x8424, 0x8425, PDF_CMAP_RANGE, 4238 },
	{ 0x8426, 0x8426, PDF_CMAP_SINGLE, 5215 },
	{ 0x8427, 0x8427, PDF_CMAP_SINGLE, 3937 },
	{ 0x8428, 0x8428, PDF_CMAP_SINGLE, 3283 },
	{ 0x8429, 0x842b, PDF_CMAP_RANGE, 18036 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 8587 },
	{ 0x842d, 0x8430, PDF_CMAP_RANGE, 18039 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 5234 },
	{ 0x8432, 0x8434, PDF_CMAP_RANGE, 18043 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 9033 },
	{ 0x8436, 0x8437, PDF_CMAP_RANGE, 18046 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 5203 },
	{ 0x8439, 0x843b, PDF_CMAP_RANGE, 18048 },
	{ 0x843c, 0x843c, PDF_CMAP_SINGLE, 5228 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 2699 },
	{ 0x843e, 0x8445, PDF_CMAP_RANGE, 18051 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 5229 },
	{ 0x8447, 0x8448, PDF_CMAP_RANGE, 18059 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 8715 },
	{ 0x844a, 0x8450, PDF_CMAP_RANGE, 18061 },
	{ 0x8451, 0x8451, PDF_CMAP_SINGLE, 5219 },
	{ 0x8452, 0x8452, PDF_CMAP_SINGLE, 9030 },
	{ 0x8453, 0x8456, PDF_CMAP_RANGE, 18068 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 4599 },
	{ 0x8458, 0x8458, PDF_CMAP_SINGLE, 18072 },
	{ 0x8459, 0x8459, PDF_CMAP_SINGLE, 5221 },
	{ 0x845a, 0x845a, PDF_CMAP_SINGLE, 5220 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 1771 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 5218 },
	{ 0x845d, 0x8460, PDF_CMAP_RANGE, 18073 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 3054 },
	{ 0x8462, 0x8462, PDF_CMAP_SINGLE, 18077 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 1516 },
	{ 0x8464, 0x8464, PDF_CMAP_SINGLE, 9031 },
	{ 0x8465, 0x8465, PDF_CMAP_SINGLE, 18078 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 8595 },
	{ 0x8467, 0x8468, PDF_CMAP_RANGE, 18079 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 5230 },
	{ 0x846a, 0x846a, PDF_CMAP_SINGLE, 18081 },
	{ 0x846b, 0x846b, PDF_CMAP_SINGLE, 1968 },
	{ 0x846c, 0x846c, PDF_CMAP_SINGLE, 4388 },
	{ 0x846d, 0x846d, PDF_CMAP_SINGLE, 5235 },
	{ 0x846e, 0x846e, PDF_CMAP_SINGLE, 18082 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 9889 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 18083 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 1363 },
	{ 0x8472, 0x8472, PDF_CMAP_SINGLE, 18084 },
	{ 0x8473, 0x8473, PDF_CMAP_SINGLE, 5222 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 18085 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 2440 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 5231 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 8034 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 5227 },
	{ 0x8479, 0x8479, PDF_CMAP_SINGLE, 18086 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 5225 },
	{ 0x847b, 0x8481, PDF_CMAP_RANGE, 18087 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 1466 },
	{ 0x8483, 0x8486, PDF_CMAP_RANGE, 18094 },
	{ 0x8487, 0x8488, PDF_CMAP_RANGE, 5223 },
	{ 0x8489, 0x8489, PDF_CMAP_SINGLE, 5226 },
	{ 0x848a, 0x848a, PDF_CMAP_SINGLE, 18098 },
	{ 0x848b, 0x848b, PDF_CMAP_SINGLE, 2176 },
	{ 0x848c, 0x848c, PDF_CMAP_SINGLE, 5232 },
	{ 0x848d, 0x848d, PDF_CMAP_SINGLE, 18099 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 5233 },
	{ 0x848f, 0x8492, PDF_CMAP_RANGE, 18100 },
	{ 0x8493, 0x8493, PDF_CMAP_SINGLE, 9037 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 9032 },
	{ 0x8495, 0x8496, PDF_CMAP_RANGE, 18104 },
	{ 0x8497, 0x8497, PDF_CMAP_SINGLE, 5249 },
	{ 0x8498, 0x8498, PDF_CMAP_SINGLE, 18106 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 2766 },
	{ 0x849a, 0x849b, PDF_CMAP_RANGE, 18107 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3543 },
	{ 0x849d, 0x84a0, PDF_CMAP_RANGE, 18109 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 5246 },
	{ 0x84a2, 0x84ae, PDF_CMAP_RANGE, 18113 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 4759 },
	{ 0x84b0, 0x84b1, PDF_CMAP_RANGE, 18126 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 3056 },
	{ 0x84b3, 0x84b3, PDF_CMAP_SINGLE, 18128 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 5248 },
	{ 0x84b5, 0x84b7, PDF_CMAP_RANGE, 18129 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 4501 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 5247 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 5244 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 18132 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 7776 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 5240 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 18133 },
	{ 0x84bf, 0x84bf, PDF_CMAP_SINGLE, 5243 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 9029 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 5236 },
	{ 0x84c2, 0x84c3, PDF_CMAP_RANGE, 18134 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 4025 },
	{ 0x84c5, 0x84c8, PDF_CMAP_RANGE, 18136 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 3251 },
	{ 0x84ca, 0x84ca, PDF_CMAP_SINGLE, 5242 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 7954 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 18140 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 5237 },
	{ 0x84ce, 0x84cf, PDF_CMAP_RANGE, 18141 },
	{ 0x84d0, 0x84d0, PDF_CMAP_SINGLE, 5238 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 3559 },
	{ 0x84d2, 0x84d2, PDF_CMAP_SINGLE, 18143 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 5241 },
	{ 0x84d4, 0x84d5, PDF_CMAP_RANGE, 18144 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 1078 },
	{ 0x84d7, 0x84dc, PDF_CMAP_RANGE, 18146 },
	{ 0x84dd, 0x84dd, PDF_CMAP_SINGLE, 2465 },
	{ 0x84de, 0x84de, PDF_CMAP_SINGLE, 18152 },
	{ 0x84df, 0x84df, PDF_CMAP_SINGLE, 2094 },
	{ 0x84e0, 0x84e0, PDF_CMAP_SINGLE, 5245 },
	{ 0x84e1, 0x84e2, PDF_CMAP_RANGE, 18153 },
	{ 0x84e3, 0x84e3, PDF_CMAP_SINGLE, 5251 },
	{ 0x84e4, 0x84e4, PDF_CMAP_SINGLE, 18155 },
	{ 0x84e5, 0x84e5, PDF_CMAP_SINGLE, 5250 },
	{ 0x84e6, 0x84e6, PDF_CMAP_SINGLE, 5239 },
	{ 0x84e7, 0x84eb, PDF_CMAP_RANGE, 18156 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 2991 },
	{ 0x84ed, 0x84ed, PDF_CMAP_SINGLE, 18161 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 8216 },
	{ 0x84ef, 0x84ef, PDF_CMAP_SINGLE, 9013 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 5255 },
	{ 0x84f1, 0x84fb, PDF_CMAP_RANGE, 18162 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 5262 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 9020 },
	{ 0x84fe, 0x84fe, PDF_CMAP_SINGLE, 18173 },
	{ 0x84ff, 0x84ff, PDF_CMAP_SINGLE, 5261 },
	{ 0x8500, 0x850b, PDF_CMAP_RANGE, 18174 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 5252 },
	{ 0x850d, 0x8510, PDF_CMAP_RANGE, 18186 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 2804 },
	{ 0x8512, 0x8512, PDF_CMAP_SINGLE, 18190 },
	{ 0x8513, 0x8513, PDF_CMAP_SINGLE, 2722 },
	{ 0x8514, 0x8514, PDF_CMAP_SINGLE, 9839 },
	{ 0x8515, 0x8516, PDF_CMAP_RANGE, 18191 },
	{ 0x8517, 0x8517, PDF_CMAP_SINGLE, 4482 },
	{ 0x8518, 0x8519, PDF_CMAP_RANGE, 18193 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 3780 },
	{ 0x851b, 0x851d, PDF_CMAP_RANGE, 18195 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 9041 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 5257 },
	{ 0x8520, 0x8520, PDF_CMAP_SINGLE, 18198 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 1171 },
	{ 0x8522, 0x8522, PDF_CMAP_SINGLE, 18199 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 8096 },
	{ 0x8524, 0x8525, PDF_CMAP_RANGE, 18200 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 9016 },
	{ 0x8527, 0x852a, PDF_CMAP_RANGE, 18202 },
	{ 0x852b, 0x852b, PDF_CMAP_SINGLE, 2891 },
	{ 0x852c, 0x852c, PDF_CMAP_SINGLE, 3446 },
	{ 0x852d, 0x852d, PDF_CMAP_SINGLE, 8731 },
	{ 0x852e, 0x8536, PDF_CMAP_RANGE, 18206 },
	{ 0x8537, 0x8537, PDF_CMAP_SINGLE, 3131 },
	{ 0x8538, 0x8538, PDF_CMAP_SINGLE, 5254 },
	{ 0x8539, 0x8539, PDF_CMAP_SINGLE, 5256 },
	{ 0x853a, 0x853a, PDF_CMAP_SINGLE, 5258 },
	{ 0x853b, 0x853b, PDF_CMAP_SINGLE, 5260 },
	{ 0x853c, 0x853c, PDF_CMAP_SINGLE, 949 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 1079 },
	{ 0x853e, 0x8540, PDF_CMAP_RANGE, 18215 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 9026 },
	{ 0x8542, 0x8542, PDF_CMAP_SINGLE, 18218 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 5270 },
	{ 0x8544, 0x8545, PDF_CMAP_RANGE, 18219 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 9039 },
	{ 0x8547, 0x8547, PDF_CMAP_SINGLE, 18221 },
	{ 0x8548, 0x8548, PDF_CMAP_SINGLE, 5264 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 2183 },
	{ 0x854a, 0x854a, PDF_CMAP_SINGLE, 3274 },
	{ 0x854b, 0x854d, PDF_CMAP_RANGE, 18222 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 9021 },
	{ 0x854f, 0x8551, PDF_CMAP_RANGE, 18225 },
	{ 0x8552, 0x8552, PDF_CMAP_SINGLE, 9028 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 9009 },
	{ 0x8554, 0x8554, PDF_CMAP_SINGLE, 18228 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 9035 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 5259 },
	{ 0x8557, 0x8557, PDF_CMAP_SINGLE, 18229 },
	{ 0x8558, 0x8558, PDF_CMAP_SINGLE, 9019 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 5263 },
	{ 0x855a, 0x855d, PDF_CMAP_RANGE, 18230 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 5267 },
	{ 0x855f, 0x8561, PDF_CMAP_RANGE, 18234 },
	{ 0x8562, 0x8562, PDF_CMAP_SINGLE, 9040 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 18237 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 5266 },
	{ 0x8565, 0x8567, PDF_CMAP_RANGE, 18238 },
	{ 0x8568, 0x8568, PDF_CMAP_SINGLE, 5265 },
	{ 0x8569, 0x8569, PDF_CMAP_SINGLE, 7860 },
	{ 0x856a, 0x856a, PDF_CMAP_SINGLE, 8614 },
	{ 0x856b, 0x856c, PDF_CMAP_RANGE, 18241 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 8651 },
	{ 0x856e, 0x8571, PDF_CMAP_RANGE, 18243 },
	{ 0x8572, 0x8572, PDF_CMAP_SINGLE, 5271 },
	{ 0x8573, 0x8573, PDF_CMAP_SINGLE, 18247 },
	{ 0x8574, 0x8574, PDF_CMAP_SINGLE, 4367 },
	{ 0x8575, 0x8576, PDF_CMAP_RANGE, 18248 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 9045 },
	{ 0x8578, 0x8578, PDF_CMAP_SINGLE, 18250 },
	{ 0x8579, 0x8579, PDF_CMAP_SINGLE, 5277 },
	{ 0x857a, 0x857a, PDF_CMAP_SINGLE, 5268 },
	{ 0x857b, 0x857b, PDF_CMAP_SINGLE, 5272 },
	{ 0x857c, 0x857d, PDF_CMAP_RANGE, 18251 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 2500 },
	{ 0x857f, 0x8583, PDF_CMAP_RANGE, 18253 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 1034 },
	{ 0x8585, 0x8585, PDF_CMAP_SINGLE, 5280 },
	{ 0x8586, 0x8586, PDF_CMAP_SINGLE, 18258 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 5275 },
	{ 0x8588, 0x8588, PDF_CMAP_SINGLE, 9022 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 18259 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 8052 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 18260 },
	{ 0x858c, 0x858c, PDF_CMAP_SINGLE, 9008 },
	{ 0x858d, 0x858e, PDF_CMAP_RANGE, 18261 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 5276 },
	{ 0x8590, 0x8590, PDF_CMAP_SINGLE, 18263 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 9856 },
	{ 0x8592, 0x8593, PDF_CMAP_RANGE, 18264 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 8410 },
	{ 0x8595, 0x859a, PDF_CMAP_RANGE, 18266 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 4047 },
	{ 0x859c, 0x859c, PDF_CMAP_SINGLE, 5279 },
	{ 0x859d, 0x859e, PDF_CMAP_RANGE, 18272 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 9034 },
	{ 0x85a0, 0x85a3, PDF_CMAP_RANGE, 18274 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 5273 },
	{ 0x85a5, 0x85a5, PDF_CMAP_SINGLE, 18278 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 8081 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 18279 },
	{ 0x85a8, 0x85a8, PDF_CMAP_SINGLE, 5274 },
	{ 0x85a9, 0x85a9, PDF_CMAP_SINGLE, 8455 },
	{ 0x85aa, 0x85aa, PDF_CMAP_SINGLE, 3976 },
	{ 0x85ab, 0x85ad, PDF_CMAP_RANGE, 18280 },
	{ 0x85ae, 0x85ae, PDF_CMAP_SINGLE, 5278 },
	{ 0x85af, 0x85af, PDF_CMAP_SINGLE, 3460 },
	{ 0x85b0, 0x85b0, PDF_CMAP_SINGLE, 5283 },
	{ 0x85b1, 0x85b6, PDF_CMAP_RANGE, 18283 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 5282 },
	{ 0x85b8, 0x85b8, PDF_CMAP_SINGLE, 18289 },
	{ 0x85b9, 0x85b9, PDF_CMAP_SINGLE, 5281 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 9023 },
	{ 0x85bb, 0x85c0, PDF_CMAP_RANGE, 18290 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 5285 },
	{ 0x85c2, 0x85c8, PDF_CMAP_RANGE, 18296 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 2230 },
	{ 0x85ca, 0x85cc, PDF_CMAP_RANGE, 18303 },
	{ 0x85cd, 0x85cd, PDF_CMAP_SINGLE, 8180 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 9027 },
	{ 0x85cf, 0x85cf, PDF_CMAP_SINGLE, 1183 },
	{ 0x85d0, 0x85d0, PDF_CMAP_SINGLE, 2799 },
	{ 0x85d1, 0x85d2, PDF_CMAP_RANGE, 18306 },
	{ 0x85d3, 0x85d3, PDF_CMAP_SINGLE, 5284 },
	{ 0x85d4, 0x85d4, PDF_CMAP_SINGLE, 18308 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 2939 },
	{ 0x85d6, 0x85db, PDF_CMAP_RANGE, 18309 },
	{ 0x85dc, 0x85dc, PDF_CMAP_SINGLE, 5286 },
	{ 0x85dd, 0x85dd, PDF_CMAP_SINGLE, 8722 },
	{ 0x85de, 0x85e3, PDF_CMAP_RANGE, 18315 },
	{ 0x85e4, 0x85e4, PDF_CMAP_SINGLE, 3628 },
	{ 0x85e5, 0x85e5, PDF_CMAP_SINGLE, 8711 },
	{ 0x85e6, 0x85e8, PDF_CMAP_RANGE, 18321 },
	{ 0x85e9, 0x85e9, PDF_CMAP_SINGLE, 1607 },
	{ 0x85ea, 0x85ea, PDF_CMAP_SINGLE, 9049 },
	{ 0x85eb, 0x85f3, PDF_CMAP_RANGE, 18324 },
	{ 0x85f4, 0x85f4, PDF_CMAP_SINGLE, 8788 },
	{ 0x85f5, 0x85f5, PDF_CMAP_SINGLE, 18333 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 9010 },
	{ 0x85f7, 0x85f8, PDF_CMAP_RANGE, 18334 },
	{ 0x85f9, 0x85f9, PDF_CMAP_SINGLE, 7718 },
	{ 0x85fa, 0x85fa, PDF_CMAP_SINGLE, 9047 },
	{ 0x85fb, 0x85fb, PDF_CMAP_SINGLE, 4392 },
	{ 0x85fc, 0x85fe, PDF_CMAP_RANGE, 18336 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 5287 },
	{ 0x8600, 0x8603, PDF_CMAP_RANGE, 18339 },
	{ 0x8604, 0x8604, PDF_CMAP_SINGLE, 9048 },
	{ 0x8605, 0x8605, PDF_CMAP_SINGLE, 5289 },
	{ 0x8606, 0x8606, PDF_CMAP_SINGLE, 8258 },
	{ 0x8607, 0x8607, PDF_CMAP_SINGLE, 8531 },
	{ 0x8608, 0x860a, PDF_CMAP_RANGE, 18343 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 8379 },
	{ 0x860c, 0x8610, PDF_CMAP_RANGE, 18346 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 2821 },
	{ 0x8612, 0x8615, PDF_CMAP_RANGE, 18351 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 5291 },
	{ 0x8617, 0x8619, PDF_CMAP_RANGE, 18355 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 9050 },
	{ 0x861b, 0x861d, PDF_CMAP_RANGE, 18358 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 9046 },
	{ 0x861f, 0x8621, PDF_CMAP_RANGE, 18361 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 9015 },
	{ 0x8623, 0x8626, PDF_CMAP_RANGE, 18364 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 5288 },
	{ 0x8628, 0x8628, PDF_CMAP_SINGLE, 18368 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 5290 },
	{ 0x862a, 0x862c, PDF_CMAP_RANGE, 18369 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 8185 },
	{ 0x862e, 0x8637, PDF_CMAP_RANGE, 18372 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 4443 },
	{ 0x8639, 0x8639, PDF_CMAP_SINGLE, 18382 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 9043 },
	{ 0x863b, 0x863b, PDF_CMAP_SINGLE, 18383 },
	{ 0x863c, 0x863c, PDF_CMAP_SINGLE, 5292 },
	{ 0x863d, 0x863e, PDF_CMAP_RANGE, 18384 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 8289 },
	{ 0x8640, 0x864c, PDF_CMAP_RANGE, 18386 },
	{ 0x864d, 0x864d, PDF_CMAP_SINGLE, 7152 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 1975 },
	{ 0x864f, 0x864f, PDF_CMAP_SINGLE, 2649 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 2929 },
	{ 0x8651, 0x8651, PDF_CMAP_SINGLE, 2670 },
	{ 0x8652, 0x8653, PDF_CMAP_RANGE, 18399 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 7153 },
	{ 0x8655, 0x8655, PDF_CMAP_SINGLE, 7829 },
	{ 0x8656, 0x8659, PDF_CMAP_RANGE, 18401 },
	{ 0x865a, 0x865a, PDF_CMAP_SINGLE, 4020 },
	{ 0x865b, 0x865b, PDF_CMAP_SINGLE, 18405 },
	{ 0x865c, 0x865c, PDF_CMAP_SINGLE, 8265 },
	{ 0x865d, 0x865d, PDF_CMAP_SINGLE, 18406 },
	{ 0x865e, 0x865e, PDF_CMAP_SINGLE, 4290 },
	{ 0x865f, 0x865f, PDF_CMAP_SINGLE, 8002 },
	{ 0x8660, 0x8661, PDF_CMAP_RANGE, 18407 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 6509 },
	{ 0x8663, 0x8666, PDF_CMAP_RANGE, 18409 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 8168 },
	{ 0x8668, 0x866a, PDF_CMAP_RANGE, 18413 },
	{ 0x866b, 0x866b, PDF_CMAP_SINGLE, 1291 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 7154 },
	{ 0x866d, 0x866d, PDF_CMAP_SINGLE, 18416 },
	{ 0x866e, 0x866e, PDF_CMAP_SINGLE, 7155 },
	{ 0x866f, 0x8670, PDF_CMAP_RANGE, 18417 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 3396 },
	{ 0x8672, 0x8678, PDF_CMAP_RANGE, 18419 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 1950 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 7157 },
	{ 0x867b, 0x867b, PDF_CMAP_SINGLE, 7159 },
	{ 0x867c, 0x867c, PDF_CMAP_SINGLE, 7158 },
	{ 0x867d, 0x867d, PDF_CMAP_SINGLE, 3545 },
	{ 0x867e, 0x867e, PDF_CMAP_SINGLE, 3879 },
	{ 0x867f, 0x867f, PDF_CMAP_SINGLE, 7156 },
	{ 0x8680, 0x8680, PDF_CMAP_SINGLE, 3403 },
	{ 0x8681, 0x8681, PDF_CMAP_SINGLE, 4182 },
	{ 0x8682, 0x8682, PDF_CMAP_SINGLE, 2707 },
	{ 0x8683, 0x8689, PDF_CMAP_RANGE, 18426 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 3794 },
	{ 0x868b, 0x868b, PDF_CMAP_SINGLE, 7162 },
	{ 0x868c, 0x868c, PDF_CMAP_SINGLE, 1025 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 7161 },
	{ 0x868e, 0x8692, PDF_CMAP_RANGE, 18433 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 7168 },
	{ 0x8694, 0x8694, PDF_CMAP_SINGLE, 18438 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 1174 },
	{ 0x8696, 0x869b, PDF_CMAP_RANGE, 18439 },
	{ 0x869c, 0x869c, PDF_CMAP_SINGLE, 4074 },
	{ 0x869d, 0x869d, PDF_CMAP_SINGLE, 7164 },
	{ 0x869e, 0x86a2, PDF_CMAP_RANGE, 18445 },
	{ 0x86a3, 0x86a3, PDF_CMAP_SINGLE, 7166 },
	{ 0x86a4, 0x86a4, PDF_CMAP_SINGLE, 4396 },
	{ 0x86a5, 0x86a6, PDF_CMAP_RANGE, 18450 },
	{ 0x86a7, 0x86a7, PDF_CMAP_SINGLE, 7165 },
	{ 0x86a8, 0x86a8, PDF_CMAP_SINGLE, 7160 },
	{ 0x86a9, 0x86a9, PDF_CMAP_SINGLE, 7169 },
	{ 0x86aa, 0x86aa, PDF_CMAP_SINGLE, 7167 },
	{ 0x86ab, 0x86ab, PDF_CMAP_SINGLE, 18452 },
	{ 0x86ac, 0x86ac, PDF_CMAP_SINGLE, 7163 },
	{ 0x86ad, 0x86ae, PDF_CMAP_RANGE, 18453 },
	{ 0x86af, 0x86af, PDF_CMAP_SINGLE, 7177 },
	{ 0x86b0, 0x86b0, PDF_CMAP_SINGLE, 7174 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 7176 },
	{ 0x86b2, 0x86b3, PDF_CMAP_RANGE, 18455 },
	{ 0x86b4, 0x86b4, PDF_CMAP_SINGLE, 7180 },
	{ 0x86b5, 0x86b5, PDF_CMAP_SINGLE, 7172 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 7170 },
	{ 0x86b7, 0x86b9, PDF_CMAP_RANGE, 18457 },
	{ 0x86ba, 0x86ba, PDF_CMAP_SINGLE, 7175 },
	{ 0x86bb, 0x86bf, PDF_CMAP_RANGE, 18460 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 4602 },
	{ 0x86c1, 0x86c3, PDF_CMAP_RANGE, 18465 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 7171 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 18468 },
	{ 0x86c6, 0x86c6, PDF_CMAP_SINGLE, 3190 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 3352 },
	{ 0x86c8, 0x86c8, PDF_CMAP_SINGLE, 18469 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 7178 },
	{ 0x86ca, 0x86ca, PDF_CMAP_SINGLE, 1823 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 1425 },
	{ 0x86cc, 0x86cd, PDF_CMAP_RANGE, 18470 },
	{ 0x86ce, 0x86ce, PDF_CMAP_SINGLE, 7173 },
	{ 0x86cf, 0x86cf, PDF_CMAP_SINGLE, 7179 },
	{ 0x86d0, 0x86d0, PDF_CMAP_SINGLE, 7186 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 7192 },
	{ 0x86d2, 0x86d3, PDF_CMAP_RANGE, 18472 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 2028 },
	{ 0x86d5, 0x86d7, PDF_CMAP_RANGE, 18474 },
	{ 0x86d8, 0x86d8, PDF_CMAP_SINGLE, 7191 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 3725 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 18477 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 4586 },
	{ 0x86dc, 0x86dd, PDF_CMAP_RANGE, 18478 },
	{ 0x86de, 0x86de, PDF_CMAP_SINGLE, 7188 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 7190 },
	{ 0x86e0, 0x86e3, PDF_CMAP_RANGE, 18480 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 1773 },
	{ 0x86e5, 0x86e8, PDF_CMAP_RANGE, 18484 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 7181 },
	{ 0x86ea, 0x86ec, PDF_CMAP_RANGE, 18488 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 7184 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 2720 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 18491 },
	{ 0x86f0, 0x86f0, PDF_CMAP_SINGLE, 4478 },
	{ 0x86f1, 0x86f2, PDF_CMAP_RANGE, 7182 },
	{ 0x86f3, 0x86f3, PDF_CMAP_SINGLE, 7185 },
	{ 0x86f4, 0x86f4, PDF_CMAP_SINGLE, 7189 },
	{ 0x86f5, 0x86f7, PDF_CMAP_RANGE, 18492 },
	{ 0x86f8, 0x86f8, PDF_CMAP_SINGLE, 7195 },
	{ 0x86f9, 0x86f9, PDF_CMAP_SINGLE, 4257 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 9690 },
	{ 0x86fb, 0x86fd, PDF_CMAP_RANGE, 18495 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 1577 },
	{ 0x86ff, 0x86ff, PDF_CMAP_SINGLE, 18498 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 3464 },
	{ 0x8701, 0x8701, PDF_CMAP_SINGLE, 18499 },
	{ 0x8702, 0x8702, PDF_CMAP_SINGLE, 1665 },
	{ 0x8703, 0x8703, PDF_CMAP_SINGLE, 7193 },
	{ 0x8704, 0x8705, PDF_CMAP_RANGE, 18500 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 9687 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 7194 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 7196 },
	{ 0x8709, 0x8709, PDF_CMAP_SINGLE, 7199 },
	{ 0x870a, 0x870a, PDF_CMAP_SINGLE, 7197 },
	{ 0x870b, 0x870c, PDF_CMAP_RANGE, 18502 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 7198 },
	{ 0x870e, 0x8711, PDF_CMAP_RANGE, 18504 },
	{ 0x8712, 0x8712, PDF_CMAP_SINGLE, 4090 },
	{ 0x8713, 0x8713, PDF_CMAP_SINGLE, 7187 },
	{ 0x8714, 0x8714, PDF_CMAP_SINGLE, 18508 },
	{ 0x8715, 0x8715, PDF_CMAP_SINGLE, 3706 },
	{ 0x8716, 0x8716, PDF_CMAP_SINGLE, 18509 },
	{ 0x8717, 0x8717, PDF_CMAP_SINGLE, 3806 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 4520 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 18510 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 7205 },
	{ 0x871b, 0x871b, PDF_CMAP_SINGLE, 18511 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 2784 },
	{ 0x871d, 0x871d, PDF_CMAP_SINGLE, 18512 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 7202 },
	{ 0x871f, 0x8720, PDF_CMAP_RANGE, 18513 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 2458 },
	{ 0x8722, 0x8722, PDF_CMAP_SINGLE, 7214 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 7200 },
	{ 0x8724, 0x8724, PDF_CMAP_SINGLE, 18515 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 7203 },
	{ 0x8726, 0x8728, PDF_CMAP_RANGE, 18516 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 7210 },
	{ 0x872a, 0x872d, PDF_CMAP_RANGE, 18519 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 7204 },
	{ 0x872f, 0x8730, PDF_CMAP_RANGE, 18523 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 7209 },
	{ 0x8732, 0x8733, PDF_CMAP_RANGE, 18525 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 7208 },
	{ 0x8735, 0x8736, PDF_CMAP_RANGE, 18527 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 7211 },
	{ 0x8738, 0x873a, PDF_CMAP_RANGE, 18529 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 7201 },
	{ 0x873c, 0x873d, PDF_CMAP_RANGE, 18532 },
	{ 0x873e, 0x873e, PDF_CMAP_SINGLE, 7206 },
	{ 0x873f, 0x873f, PDF_CMAP_SINGLE, 7212 },
	{ 0x8740, 0x8746, PDF_CMAP_RANGE, 18534 },
	{ 0x8747, 0x8747, PDF_CMAP_SINGLE, 4241 },
	{ 0x8748, 0x8748, PDF_CMAP_SINGLE, 7207 },
	{ 0x8749, 0x8749, PDF_CMAP_SINGLE, 1212 },
	{ 0x874a, 0x874b, PDF_CMAP_RANGE, 18541 },
	{ 0x874c, 0x874c, PDF_CMAP_SINGLE, 7220 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 18543 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 3958 },
	{ 0x874f, 0x8752, PDF_CMAP_RANGE, 18544 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 7223 },
	{ 0x8754, 0x8754, PDF_CMAP_SINGLE, 18548 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 8494 },
	{ 0x8756, 0x8756, PDF_CMAP_SINGLE, 18549 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 2013 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 18550 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 7227 },
	{ 0x875a, 0x875f, PDF_CMAP_RANGE, 18551 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 7218 },
	{ 0x8761, 0x8762, PDF_CMAP_RANGE, 18557 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 7224 },
	{ 0x8764, 0x8764, PDF_CMAP_SINGLE, 7226 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 7228 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 8626 },
	{ 0x8767, 0x876d, PDF_CMAP_RANGE, 18559 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 7221 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 18566 },
	{ 0x8770, 0x8770, PDF_CMAP_SINGLE, 7219 },
	{ 0x8771, 0x8773, PDF_CMAP_RANGE, 18567 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 1970 },
	{ 0x8775, 0x8775, PDF_CMAP_SINGLE, 18570 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 1500 },
	{ 0x8777, 0x8777, PDF_CMAP_SINGLE, 18571 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 8606 },
	{ 0x8779, 0x877a, PDF_CMAP_RANGE, 18572 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 7217 },
	{ 0x877c, 0x877c, PDF_CMAP_SINGLE, 7225 },
	{ 0x877d, 0x877e, PDF_CMAP_RANGE, 7215 },
	{ 0x877f, 0x8781, PDF_CMAP_RANGE, 18574 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 7213 },
	{ 0x8783, 0x8783, PDF_CMAP_SINGLE, 7238 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 9692 },
	{ 0x8785, 0x8785, PDF_CMAP_SINGLE, 7235 },
	{ 0x8786, 0x8787, PDF_CMAP_RANGE, 18577 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 7234 },
	{ 0x8789, 0x878a, PDF_CMAP_RANGE, 18579 },
	{ 0x878b, 0x878b, PDF_CMAP_SINGLE, 7222 },
	{ 0x878c, 0x878c, PDF_CMAP_SINGLE, 18581 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 3253 },
	{ 0x878e, 0x8792, PDF_CMAP_RANGE, 18582 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 7229 },
	{ 0x8794, 0x8796, PDF_CMAP_RANGE, 18587 },
	{ 0x8797, 0x8797, PDF_CMAP_SINGLE, 7237 },
	{ 0x8798, 0x879d, PDF_CMAP_RANGE, 18590 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 8300 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 2813 },
	{ 0x87a0, 0x87a1, PDF_CMAP_RANGE, 18596 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 8742 },
	{ 0x87a3, 0x87a7, PDF_CMAP_RANGE, 18598 },
	{ 0x87a8, 0x87a8, PDF_CMAP_SINGLE, 7231 },
	{ 0x87a9, 0x87aa, PDF_CMAP_RANGE, 18603 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 7239 },
	{ 0x87ac, 0x87ac, PDF_CMAP_SINGLE, 7241 },
	{ 0x87ad, 0x87ad, PDF_CMAP_SINGLE, 7236 },
	{ 0x87ae, 0x87ae, PDF_CMAP_SINGLE, 18605 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 7230 },
	{ 0x87b0, 0x87b2, PDF_CMAP_RANGE, 18606 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 7243 },
	{ 0x87b4, 0x87b4, PDF_CMAP_SINGLE, 18609 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 7242 },
	{ 0x87b6, 0x87b9, PDF_CMAP_RANGE, 18610 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 2692 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 9696 },
	{ 0x87bc, 0x87bc, PDF_CMAP_SINGLE, 18614 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 7246 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 18615 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 7248 },
	{ 0x87c1, 0x87c3, PDF_CMAP_RANGE, 18617 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 8827 },
	{ 0x87c5, 0x87c5, PDF_CMAP_SINGLE, 18620 },
	{ 0x87c6, 0x87c6, PDF_CMAP_SINGLE, 7233 },
	{ 0x87c7, 0x87c7, PDF_CMAP_SINGLE, 18621 },
	{ 0x87c8, 0x87c8, PDF_CMAP_SINGLE, 9694 },
	{ 0x87c9, 0x87c9, PDF_CMAP_SINGLE, 18622 },
	{ 0x87ca, 0x87ca, PDF_CMAP_SINGLE, 7249 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 7244 },
	{ 0x87cc, 0x87cd, PDF_CMAP_RANGE, 18623 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 9697 },
	{ 0x87cf, 0x87d0, PDF_CMAP_RANGE, 18625 },
	{ 0x87d1, 0x87d1, PDF_CMAP_SINGLE, 7247 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 7232 },
	{ 0x87d3, 0x87d3, PDF_CMAP_SINGLE, 7245 },
	{ 0x87d4, 0x87da, PDF_CMAP_RANGE, 18627 },
	{ 0x87db, 0x87db, PDF_CMAP_SINGLE, 7250 },
	{ 0x87dc, 0x87df, PDF_CMAP_RANGE, 18634 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 7252 },
	{ 0x87e1, 0x87e2, PDF_CMAP_RANGE, 18638 },
	{ 0x87e3, 0x87e3, PDF_CMAP_SINGLE, 9685 },
	{ 0x87e4, 0x87e4, PDF_CMAP_SINGLE, 18640 },
	{ 0x87e5, 0x87e5, PDF_CMAP_SINGLE, 7240 },
	{ 0x87e6, 0x87e9, PDF_CMAP_RANGE, 18641 },
	{ 0x87ea, 0x87ea, PDF_CMAP_SINGLE, 7251 },
	{ 0x87eb, 0x87eb, PDF_CMAP_SINGLE, 18645 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 7787 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 18646 },
	{ 0x87ee, 0x87ee, PDF_CMAP_SINGLE, 7253 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 9691 },
	{ 0x87f0, 0x87f1, PDF_CMAP_RANGE, 18647 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 7817 },
	{ 0x87f3, 0x87f5, PDF_CMAP_RANGE, 18649 },
	{ 0x87f6, 0x87f6, PDF_CMAP_SINGLE, 9689 },
	{ 0x87f7, 0x87f8, PDF_CMAP_RANGE, 18652 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 3970 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 18654 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 8721 },
	{ 0x87fc, 0x87fd, PDF_CMAP_RANGE, 18655 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 7256 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 18657 },
	{ 0x8800, 0x8802, PDF_CMAP_RANGE, 18658 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 4877 },
	{ 0x8804, 0x8804, PDF_CMAP_SINGLE, 18661 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 8745 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 9686 },
	{ 0x8807, 0x8809, PDF_CMAP_RANGE, 18662 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 7257 },
	{ 0x880b, 0x880f, PDF_CMAP_RANGE, 18665 },
	{ 0x8810, 0x8810, PDF_CMAP_SINGLE, 9693 },
	{ 0x8811, 0x8811, PDF_CMAP_SINGLE, 9695 },
	{ 0x8812, 0x8812, PDF_CMAP_SINGLE, 18670 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 7255 },
	{ 0x8814, 0x8814, PDF_CMAP_SINGLE, 18671 },
	{ 0x8815, 0x8815, PDF_CMAP_SINGLE, 3263 },
	{ 0x8816, 0x8816, PDF_CMAP_SINGLE, 7254 },
	{ 0x8817, 0x881a, PDF_CMAP_RANGE, 18672 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 7258 },
	{ 0x881c, 0x881e, PDF_CMAP_RANGE, 18676 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 8175 },
	{ 0x8820, 0x8820, PDF_CMAP_SINGLE, 18679 },
	{ 0x8821, 0x8821, PDF_CMAP_SINGLE, 7259 },
	{ 0x8822, 0x8822, PDF_CMAP_SINGLE, 1347 },
	{ 0x8823, 0x8823, PDF_CMAP_SINGLE, 9688 },
	{ 0x8824, 0x8830, PDF_CMAP_RANGE, 18680 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 7977 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 6797 },
	{ 0x8833, 0x8835, PDF_CMAP_RANGE, 18693 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 7771 },
	{ 0x8837, 0x8838, PDF_CMAP_RANGE, 18696 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 7260 },
	{ 0x883a, 0x883a, PDF_CMAP_SINGLE, 18698 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 8310 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 7261 },
	{ 0x883d, 0x883f, PDF_CMAP_RANGE, 18699 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 4051 },
	{ 0x8841, 0x8843, PDF_CMAP_RANGE, 18702 },
	{ 0x8844, 0x8844, PDF_CMAP_SINGLE, 7338 },
	{ 0x8845, 0x8845, PDF_CMAP_SINGLE, 3985 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 8855 },
	{ 0x8847, 0x8849, PDF_CMAP_RANGE, 18705 },
	{ 0x884a, 0x884a, PDF_CMAP_SINGLE, 9872 },
	{ 0x884b, 0x884b, PDF_CMAP_SINGLE, 18708 },
	{ 0x884c, 0x884c, PDF_CMAP_SINGLE, 3995 },
	{ 0x884d, 0x884d, PDF_CMAP_SINGLE, 4101 },
	{ 0x884e, 0x8852, PDF_CMAP_RANGE, 18709 },
	{ 0x8853, 0x8853, PDF_CMAP_SINGLE, 8511 },
	{ 0x8854, 0x8854, PDF_CMAP_SINGLE, 3899 },
	{ 0x8855, 0x8856, PDF_CMAP_RANGE, 18714 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 2215 },
	{ 0x8858, 0x8858, PDF_CMAP_SINGLE, 18716 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 4076 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 18717 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 8600 },
	{ 0x885c, 0x885c, PDF_CMAP_SINGLE, 18718 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 7816 },
	{ 0x885e, 0x8860, PDF_CMAP_RANGE, 18719 },
	{ 0x8861, 0x8861, PDF_CMAP_SINGLE, 1945 },
	{ 0x8862, 0x8862, PDF_CMAP_SINGLE, 5613 },
	{ 0x8863, 0x8863, PDF_CMAP_SINGLE, 4169 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 7078 },
	{ 0x8865, 0x8865, PDF_CMAP_SINGLE, 1152 },
	{ 0x8866, 0x8867, PDF_CMAP_RANGE, 18722 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 1110 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 7079 },
	{ 0x886a, 0x886a, PDF_CMAP_SINGLE, 18724 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 3321 },
	{ 0x886c, 0x886c, PDF_CMAP_SINGLE, 1257 },
	{ 0x886d, 0x886d, PDF_CMAP_SINGLE, 18725 },
	{ 0x886e, 0x886e, PDF_CMAP_SINGLE, 4870 },
	{ 0x886f, 0x886f, PDF_CMAP_SINGLE, 18726 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 3482 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 18727 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 7080 },
	{ 0x8873, 0x8876, PDF_CMAP_RANGE, 18728 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 4563 },
	{ 0x8878, 0x8878, PDF_CMAP_SINGLE, 18732 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 8844 },
	{ 0x887a, 0x887c, PDF_CMAP_RANGE, 18733 },
	{ 0x887d, 0x887d, PDF_CMAP_SINGLE, 7081 },
	{ 0x887e, 0x887e, PDF_CMAP_SINGLE, 7358 },
	{ 0x887f, 0x887f, PDF_CMAP_SINGLE, 7082 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 18736 },
	{ 0x8881, 0x8881, PDF_CMAP_SINGLE, 4335 },
	{ 0x8882, 0x8882, PDF_CMAP_SINGLE, 7083 },
	{ 0x8883, 0x8883, PDF_CMAP_SINGLE, 18737 },
	{ 0x8884, 0x8884, PDF_CMAP_SINGLE, 971 },
	{ 0x8885, 0x8885, PDF_CMAP_SINGLE, 7359 },
	{ 0x8886, 0x8887, PDF_CMAP_RANGE, 18738 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 7360 },
	{ 0x8889, 0x888a, PDF_CMAP_RANGE, 18740 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 1407 },
	{ 0x888c, 0x888c, PDF_CMAP_SINGLE, 18742 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 2972 },
	{ 0x888e, 0x8891, PDF_CMAP_RANGE, 18743 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 3598 },
	{ 0x8893, 0x8895, PDF_CMAP_RANGE, 18747 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 4015 },
	{ 0x8897, 0x889b, PDF_CMAP_RANGE, 18750 },
	{ 0x889c, 0x889c, PDF_CMAP_SINGLE, 3729 },
	{ 0x889d, 0x88a1, PDF_CMAP_RANGE, 18755 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 7084 },
	{ 0x88a3, 0x88a3, PDF_CMAP_SINGLE, 18760 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 4871 },
	{ 0x88a5, 0x88aa, PDF_CMAP_RANGE, 18761 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 1060 },
	{ 0x88ac, 0x88ac, PDF_CMAP_SINGLE, 18767 },
	{ 0x88ad, 0x88ad, PDF_CMAP_SINGLE, 3867 },
	{ 0x88ae, 0x88b0, PDF_CMAP_RANGE, 18768 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 1695 },
	{ 0x88b2, 0x88b6, PDF_CMAP_RANGE, 18771 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 7086 },
	{ 0x88b8, 0x88bb, PDF_CMAP_RANGE, 18776 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 7087 },
	{ 0x88bd, 0x88c0, PDF_CMAP_RANGE, 18780 },
	{ 0x88c1, 0x88c1, PDF_CMAP_SINGLE, 1162 },
	{ 0x88c2, 0x88c2, PDF_CMAP_SINGLE, 2586 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 18784 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 4621 },
	{ 0x88c6, 0x88c6, PDF_CMAP_SINGLE, 7085 },
	{ 0x88c7, 0x88c8, PDF_CMAP_RANGE, 18786 },
	{ 0x88c9, 0x88c9, PDF_CMAP_SINGLE, 7088 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 9713 },
	{ 0x88cb, 0x88cd, PDF_CMAP_RANGE, 18788 },
	{ 0x88ce, 0x88ce, PDF_CMAP_SINGLE, 7090 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 8204 },
	{ 0x88d0, 0x88d1, PDF_CMAP_RANGE, 18791 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 4874 },
	{ 0x88d3, 0x88d3, PDF_CMAP_SINGLE, 18793 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 4200 },
	{ 0x88d5, 0x88d5, PDF_CMAP_SINGLE, 4326 },
	{ 0x88d6, 0x88d7, PDF_CMAP_RANGE, 18794 },
	{ 0x88d8, 0x88d8, PDF_CMAP_SINGLE, 7361 },
	{ 0x88d9, 0x88d9, PDF_CMAP_SINGLE, 3220 },
	{ 0x88da, 0x88db, PDF_CMAP_RANGE, 18796 },
	{ 0x88dc, 0x88dc, PDF_CMAP_SINGLE, 7768 },
	{ 0x88dd, 0x88dd, PDF_CMAP_SINGLE, 8876 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 18798 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 7362 },
	{ 0x88e0, 0x88e1, PDF_CMAP_RANGE, 18799 },
	{ 0x88e2, 0x88e2, PDF_CMAP_SINGLE, 7089 },
	{ 0x88e3, 0x88e3, PDF_CMAP_SINGLE, 7091 },
	{ 0x88e4, 0x88e4, PDF_CMAP_SINGLE, 2416 },
	{ 0x88e5, 0x88e5, PDF_CMAP_SINGLE, 7092 },
	{ 0x88e6, 0x88e7, PDF_CMAP_RANGE, 18801 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 7096 },
	{ 0x88e9, 0x88ef, PDF_CMAP_RANGE, 18803 },
	{ 0x88f0, 0x88f0, PDF_CMAP_SINGLE, 7098 },
	{ 0x88f1, 0x88f1, PDF_CMAP_SINGLE, 7093 },
	{ 0x88f2, 0x88f2, PDF_CMAP_SINGLE, 18810 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 3338 },
	{ 0x88f4, 0x88f4, PDF_CMAP_SINGLE, 2978 },
	{ 0x88f5, 0x88f7, PDF_CMAP_RANGE, 18811 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 2698 },
	{ 0x88f9, 0x88f9, PDF_CMAP_SINGLE, 1877 },
	{ 0x88fa, 0x88fb, PDF_CMAP_RANGE, 18814 },
	{ 0x88fc, 0x88fc, PDF_CMAP_SINGLE, 7095 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 9895 },
	{ 0x88fe, 0x88fe, PDF_CMAP_SINGLE, 7097 },
	{ 0x88ff, 0x88ff, PDF_CMAP_SINGLE, 18816 },
	{ 0x8900, 0x8901, PDF_CMAP_RANGE, 18817 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 1836 },
	{ 0x8903, 0x8906, PDF_CMAP_RANGE, 18819 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 9846 },
	{ 0x8908, 0x8909, PDF_CMAP_RANGE, 18823 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 7103 },
	{ 0x890b, 0x890f, PDF_CMAP_RANGE, 18825 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 1933 },
	{ 0x8911, 0x8911, PDF_CMAP_SINGLE, 18830 },
	{ 0x8912, 0x8912, PDF_CMAP_SINGLE, 1032 },
	{ 0x8913, 0x8913, PDF_CMAP_SINGLE, 7101 },
	{ 0x8914, 0x8918, PDF_CMAP_RANGE, 18831 },
	{ 0x8919, 0x8919, PDF_CMAP_SINGLE, 7100 },
	{ 0x891a, 0x891a, PDF_CMAP_SINGLE, 7094 },
	{ 0x891b, 0x891b, PDF_CMAP_SINGLE, 7102 },
	{ 0x891c, 0x8920, PDF_CMAP_RANGE, 18836 },
	{ 0x8921, 0x8921, PDF_CMAP_SINGLE, 7099 },
	{ 0x8922, 0x8924, PDF_CMAP_RANGE, 18841 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 3271 },
	{ 0x8926, 0x8929, PDF_CMAP_RANGE, 18844 },
	{ 0x892a, 0x892a, PDF_CMAP_SINGLE, 3707 },
	{ 0x892b, 0x892b, PDF_CMAP_SINGLE, 7105 },
	{ 0x892c, 0x892f, PDF_CMAP_RANGE, 18848 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 5945 },
	{ 0x8931, 0x8931, PDF_CMAP_SINGLE, 18852 },
	{ 0x8932, 0x8932, PDF_CMAP_SINGLE, 8161 },
	{ 0x8933, 0x8933, PDF_CMAP_SINGLE, 9659 },
	{ 0x8934, 0x8934, PDF_CMAP_SINGLE, 7104 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 18853 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 7106 },
	{ 0x8937, 0x8937, PDF_CMAP_SINGLE, 18854 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 9662 },
	{ 0x8939, 0x893a, PDF_CMAP_RANGE, 18855 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 8925 },
	{ 0x893c, 0x8940, PDF_CMAP_RANGE, 18857 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 7107 },
	{ 0x8942, 0x8943, PDF_CMAP_RANGE, 18862 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 3922 },
	{ 0x8945, 0x8946, PDF_CMAP_RANGE, 18864 },
	{ 0x8947, 0x8947, PDF_CMAP_SINGLE, 9661 },
	{ 0x8948, 0x8955, PDF_CMAP_RANGE, 18866 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 7722 },
	{ 0x8957, 0x895c, PDF_CMAP_RANGE, 18880 },
	{ 0x895d, 0x895d, PDF_CMAP_SINGLE, 9660 },
	{ 0x895e, 0x895e, PDF_CMAP_SINGLE, 7363 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 2244 },
	{ 0x8960, 0x8960, PDF_CMAP_SINGLE, 9658 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 18886 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 9663 },
	{ 0x8965, 0x8965, PDF_CMAP_SINGLE, 18889 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 7108 },
	{ 0x8967, 0x8969, PDF_CMAP_RANGE, 18890 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 8583 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 18893 },
	{ 0x896c, 0x896c, PDF_CMAP_SINGLE, 9835 },
	{ 0x896d, 0x896e, PDF_CMAP_RANGE, 18894 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 7807 },
	{ 0x8970, 0x8971, PDF_CMAP_RANGE, 18896 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 8621 },
	{ 0x8973, 0x897a, PDF_CMAP_RANGE, 18898 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 7109 },
	{ 0x897c, 0x897e, PDF_CMAP_RANGE, 18906 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 3846 },
	{ 0x8980, 0x8980, PDF_CMAP_SINGLE, 18909 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 4145 },
	{ 0x8982, 0x8982, PDF_CMAP_SINGLE, 18910 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 7135 },
	{ 0x8984, 0x8985, PDF_CMAP_RANGE, 18911 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 1709 },
	{ 0x8987, 0x898a, PDF_CMAP_RANGE, 18913 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8086 },
	{ 0x898c, 0x898e, PDF_CMAP_RANGE, 18917 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 7986 },
	{ 0x8990, 0x8992, PDF_CMAP_RANGE, 18920 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 8324 },
	{ 0x8994, 0x8995, PDF_CMAP_RANGE, 18923 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 8502 },
	{ 0x8997, 0x8997, PDF_CMAP_SINGLE, 18925 },
	{ 0x8998, 0x8998, PDF_CMAP_SINGLE, 9400 },
	{ 0x8999, 0x89a0, PDF_CMAP_RANGE, 18926 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 9402 },
	{ 0x89a2, 0x89a5, PDF_CMAP_RANGE, 18934 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 9404 },
	{ 0x89a7, 0x89a9, PDF_CMAP_RANGE, 18938 },
	{ 0x89aa, 0x89aa, PDF_CMAP_SINGLE, 8420 },
	{ 0x89ab, 0x89ab, PDF_CMAP_SINGLE, 18941 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 9401 },
	{ 0x89ad, 0x89ae, PDF_CMAP_RANGE, 18942 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 9405 },
	{ 0x89b0, 0x89b1, PDF_CMAP_RANGE, 18944 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 9406 },
	{ 0x89b3, 0x89b6, PDF_CMAP_RANGE, 18946 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 9407 },
	{ 0x89b8, 0x89b9, PDF_CMAP_RANGE, 18950 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8146 },
	{ 0x89bb, 0x89bc, PDF_CMAP_RANGE, 18952 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 8189 },
	{ 0x89be, 0x89be, PDF_CMAP_SINGLE, 18954 },
	{ 0x89bf, 0x89bf, PDF_CMAP_SINGLE, 9403 },
	{ 0x89c0, 0x89c0, PDF_CMAP_SINGLE, 7981 },
	{ 0x89c1, 0x89c1, PDF_CMAP_SINGLE, 2158 },
	{ 0x89c2, 0x89c2, PDF_CMAP_SINGLE, 1844 },
	{ 0x89c3, 0x89c3, PDF_CMAP_SINGLE, 18955 },
	{ 0x89c4, 0x89c4, PDF_CMAP_SINGLE, 1855 },
	{ 0x89c5, 0x89c5, PDF_CMAP_SINGLE, 2782 },
	{ 0x89c6, 0x89c6, PDF_CMAP_SINGLE, 3434 },
	{ 0x89c7, 0x89c7, PDF_CMAP_SINGLE, 6458 },
	{ 0x89c8, 0x89c8, PDF_CMAP_SINGLE, 2475 },
	{ 0x89c9, 0x89c9, PDF_CMAP_SINGLE, 2340 },
	{ 0x89ca, 0x89cc, PDF_CMAP_RANGE, 6459 },
	{ 0x89cd, 0x89cd, PDF_CMAP_SINGLE, 18956 },
	{ 0x89ce, 0x89d1, PDF_CMAP_RANGE, 6462 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 2200 },
	{ 0x89d3, 0x89d5, PDF_CMAP_RANGE, 18957 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 7511 },
	{ 0x89d7, 0x89d9, PDF_CMAP_RANGE, 18960 },
	{ 0x89da, 0x89da, PDF_CMAP_SINGLE, 7513 },
	{ 0x89db, 0x89db, PDF_CMAP_SINGLE, 18963 },
	{ 0x89dc, 0x89dc, PDF_CMAP_SINGLE, 7514 },
	{ 0x89dd, 0x89dd, PDF_CMAP_SINGLE, 18964 },
	{ 0x89de, 0x89de, PDF_CMAP_SINGLE, 7512 },
	{ 0x89df, 0x89e2, PDF_CMAP_RANGE, 18965 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 2227 },
	{ 0x89e4, 0x89e4, PDF_CMAP_SINGLE, 18969 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 7515 },
	{ 0x89e6, 0x89e6, PDF_CMAP_SINGLE, 1320 },
	{ 0x89e7, 0x89ea, PDF_CMAP_RANGE, 18970 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 7516 },
	{ 0x89ec, 0x89ee, PDF_CMAP_RANGE, 18974 },
	{ 0x89ef, 0x89ef, PDF_CMAP_SINGLE, 7517 },
	{ 0x89f0, 0x89f2, PDF_CMAP_RANGE, 18977 },
	{ 0x89f3, 0x89f3, PDF_CMAP_SINGLE, 6592 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 9737 },
	{ 0x89f5, 0x89f5, PDF_CMAP_SINGLE, 18980 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 9738 },
	{ 0x89f7, 0x89f7, PDF_CMAP_SINGLE, 18981 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7828 },
	{ 0x89f9, 0x89ff, PDF_CMAP_RANGE, 18982 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 4093 },
	{ 0x8a01, 0x8a01, PDF_CMAP_SINGLE, 8927 },
	{ 0x8a02, 0x8a02, PDF_CMAP_SINGLE, 7883 },
	{ 0x8a03, 0x8a03, PDF_CMAP_SINGLE, 7949 },
	{ 0x8a04, 0x8a06, PDF_CMAP_RANGE, 18989 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 4862 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 8055 },
	{ 0x8a09, 0x8a09, PDF_CMAP_SINGLE, 18992 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 8683 },
	{ 0x8a0b, 0x8a0b, PDF_CMAP_SINGLE, 18993 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 8929 },
	{ 0x8a0d, 0x8a0d, PDF_CMAP_SINGLE, 18994 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 8559 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 18995 },
	{ 0x8a10, 0x8a10, PDF_CMAP_SINGLE, 8928 },
	{ 0x8a11, 0x8a12, PDF_CMAP_RANGE, 18996 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 8682 },
	{ 0x8a14, 0x8a14, PDF_CMAP_SINGLE, 18998 },
	{ 0x8a15, 0x8a15, PDF_CMAP_SINGLE, 8930 },
	{ 0x8a16, 0x8a16, PDF_CMAP_SINGLE, 8395 },
	{ 0x8a17, 0x8a17, PDF_CMAP_SINGLE, 18999 },
	{ 0x8a18, 0x8a18, PDF_CMAP_SINGLE, 8056 },
	{ 0x8a19, 0x8a1a, PDF_CMAP_RANGE, 19000 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 7906 },
	{ 0x8a1c, 0x8a1c, PDF_CMAP_SINGLE, 19002 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 8690 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 19003 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 8528 },
	{ 0x8a20, 0x8a22, PDF_CMAP_RANGE, 19004 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 8147 },
	{ 0x8a24, 0x8a24, PDF_CMAP_SINGLE, 19007 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 8933 },
	{ 0x8a26, 0x8a29, PDF_CMAP_RANGE, 19008 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 7922 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 19012 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 8479 },
	{ 0x8a2e, 0x8a30, PDF_CMAP_RANGE, 19014 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 8669 },
	{ 0x8a32, 0x8a33, PDF_CMAP_RANGE, 19017 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 8532 },
	{ 0x8a35, 0x8a35, PDF_CMAP_SINGLE, 19019 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 8935 },
	{ 0x8a37, 0x8a39, PDF_CMAP_RANGE, 19020 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 8834 },
	{ 0x8a3b, 0x8a3d, PDF_CMAP_RANGE, 19023 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 7518 },
	{ 0x8a3f, 0x8a40, PDF_CMAP_RANGE, 19026 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 8934 },
	{ 0x8a42, 0x8a45, PDF_CMAP_RANGE, 19028 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 8936 },
	{ 0x8a47, 0x8a47, PDF_CMAP_SINGLE, 19032 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 6788 },
	{ 0x8a49, 0x8a4d, PDF_CMAP_RANGE, 19033 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 8932 },
	{ 0x8a4f, 0x8a4f, PDF_CMAP_SINGLE, 19038 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 8810 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 19039 },
	{ 0x8a52, 0x8a52, PDF_CMAP_SINGLE, 8939 },
	{ 0x8a53, 0x8a53, PDF_CMAP_SINGLE, 19040 },
	{ 0x8a54, 0x8a54, PDF_CMAP_SINGLE, 8937 },
	{ 0x8a55, 0x8a55, PDF_CMAP_SINGLE, 8381 },
	{ 0x8a56, 0x8a57, PDF_CMAP_RANGE, 19041 },
	{ 0x8a58, 0x8a58, PDF_CMAP_SINGLE, 8938 },
	{ 0x8a59, 0x8a5a, PDF_CMAP_RANGE, 19043 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 8893 },
	{ 0x8a5c, 0x8a5d, PDF_CMAP_RANGE, 19045 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 7838 },
	{ 0x8a5f, 0x8a60, PDF_CMAP_RANGE, 19047 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 8950 },
	{ 0x8a62, 0x8a62, PDF_CMAP_SINGLE, 8679 },
	{ 0x8a63, 0x8a63, PDF_CMAP_SINGLE, 8726 },
	{ 0x8a64, 0x8a65, PDF_CMAP_RANGE, 19049 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 8503 },
	{ 0x8a67, 0x8a68, PDF_CMAP_RANGE, 19051 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 8492 },
	{ 0x8a6a, 0x8a6a, PDF_CMAP_SINGLE, 19053 },
	{ 0x8a6b, 0x8a6b, PDF_CMAP_SINGLE, 7784 },
	{ 0x8a6c, 0x8a6c, PDF_CMAP_SINGLE, 8946 },
	{ 0x8a6d, 0x8a6d, PDF_CMAP_SINGLE, 7991 },
	{ 0x8a6e, 0x8a6e, PDF_CMAP_SINGLE, 8947 },
	{ 0x8a6f, 0x8a6f, PDF_CMAP_SINGLE, 19054 },
	{ 0x8a70, 0x8a70, PDF_CMAP_SINGLE, 8943 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 8016 },
	{ 0x8a72, 0x8a72, PDF_CMAP_SINGLE, 7952 },
	{ 0x8a73, 0x8a73, PDF_CMAP_SINGLE, 8648 },
	{ 0x8a74, 0x8a74, PDF_CMAP_SINGLE, 19055 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 8945 },
	{ 0x8a76, 0x8a78, PDF_CMAP_RANGE, 19056 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 4435 },
	{ 0x8a7a, 0x8a7b, PDF_CMAP_RANGE, 19059 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 8944 },
	{ 0x8a7d, 0x8a7e, PDF_CMAP_RANGE, 19061 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 8942 },
	{ 0x8a80, 0x8a83, PDF_CMAP_RANGE, 19063 },
	{ 0x8a84, 0x8a84, PDF_CMAP_SINGLE, 8941 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 8862 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 8940 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 8162 },
	{ 0x8a88, 0x8a88, PDF_CMAP_SINGLE, 19067 },
	{ 0x8a89, 0x8a89, PDF_CMAP_SINGLE, 4323 },
	{ 0x8a8a, 0x8a8a, PDF_CMAP_SINGLE, 3631 },
	{ 0x8a8b, 0x8a8c, PDF_CMAP_RANGE, 19068 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 8446 },
	{ 0x8a8e, 0x8a90, PDF_CMAP_RANGE, 19070 },
	{ 0x8a91, 0x8a92, PDF_CMAP_RANGE, 8953 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 3419 },
	{ 0x8a94, 0x8a94, PDF_CMAP_SINGLE, 19073 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 7855 },
	{ 0x8a96, 0x8a97, PDF_CMAP_RANGE, 19074 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 8758 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 19076 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 8951 },
	{ 0x8a9b, 0x8a9d, PDF_CMAP_RANGE, 19077 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 8765 },
	{ 0x8a9f, 0x8a9f, PDF_CMAP_SINGLE, 19080 },
	{ 0x8aa0, 0x8aa0, PDF_CMAP_SINGLE, 7810 },
	{ 0x8aa1, 0x8aa1, PDF_CMAP_SINGLE, 8118 },
	{ 0x8aa2, 0x8aa2, PDF_CMAP_SINGLE, 19081 },
	{ 0x8aa3, 0x8aa3, PDF_CMAP_SINGLE, 8612 },
	{ 0x8aa4, 0x8aa4, PDF_CMAP_SINGLE, 8618 },
	{ 0x8aa5, 0x8aa5, PDF_CMAP_SINGLE, 8952 },
	{ 0x8aa6, 0x8aa6, PDF_CMAP_SINGLE, 8529 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 19082 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 8032 },
	{ 0x8aa9, 0x8aab, PDF_CMAP_RANGE, 19083 },
	{ 0x8aac, 0x8aac, PDF_CMAP_SINGLE, 8519 },
	{ 0x8aad, 0x8aaf, PDF_CMAP_RANGE, 19086 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 8517 },
	{ 0x8ab1, 0x8ab1, PDF_CMAP_SINGLE, 19089 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 8156 },
	{ 0x8ab3, 0x8ab5, PDF_CMAP_RANGE, 19090 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 8961 },
	{ 0x8ab7, 0x8ab8, PDF_CMAP_RANGE, 19093 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 7925 },
	{ 0x8aba, 0x8abb, PDF_CMAP_RANGE, 19095 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 8728 },
	{ 0x8abd, 0x8abe, PDF_CMAP_RANGE, 19097 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 7878 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_RANGE, 19099 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 8960 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 19101 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 8884 },
	{ 0x8ac5, 0x8ac6, PDF_CMAP_RANGE, 19102 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 8553 },
	{ 0x8ac8, 0x8ac8, PDF_CMAP_SINGLE, 19104 },
	{ 0x8ac9, 0x8ac9, PDF_CMAP_SINGLE, 8957 },
	{ 0x8aca, 0x8aca, PDF_CMAP_SINGLE, 19105 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 8426 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 19106 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 8948 },
	{ 0x8ace, 0x8ace, PDF_CMAP_SINGLE, 19107 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 8955 },
	{ 0x8ad0, 0x8ad0, PDF_CMAP_SINGLE, 19108 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 8956 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 8231 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 19109 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 8288 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 8959 },
	{ 0x8ad8, 0x8ada, PDF_CMAP_RANGE, 19112 },
	{ 0x8adb, 0x8adb, PDF_CMAP_SINGLE, 8958 },
	{ 0x8adc, 0x8adc, PDF_CMAP_SINGLE, 7879 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 19115 },
	{ 0x8ade, 0x8ade, PDF_CMAP_SINGLE, 8972 },
	{ 0x8adf, 0x8ae1, PDF_CMAP_RANGE, 19116 },
	{ 0x8ae2, 0x8ae2, PDF_CMAP_SINGLE, 8949 },
	{ 0x8ae3, 0x8ae3, PDF_CMAP_SINGLE, 19119 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 8966 },
	{ 0x8ae5, 0x8ae5, PDF_CMAP_SINGLE, 19120 },
	{ 0x8ae6, 0x8ae6, PDF_CMAP_SINGLE, 8970 },
	{ 0x8ae7, 0x8ae7, PDF_CMAP_SINGLE, 8659 },
	{ 0x8ae8, 0x8aea, PDF_CMAP_RANGE, 19121 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 8963 },
	{ 0x8aec, 0x8aec, PDF_CMAP_SINGLE, 19124 },
	{ 0x8aed, 0x8aed, PDF_CMAP_SINGLE, 8967 },
	{ 0x8aee, 0x8aee, PDF_CMAP_SINGLE, 8971 },
	{ 0x8aef, 0x8af0, PDF_CMAP_RANGE, 19125 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 8031 },
	{ 0x8af2, 0x8af2, PDF_CMAP_SINGLE, 19127 },
	{ 0x8af3, 0x8af3, PDF_CMAP_SINGLE, 8969 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 19128 },
	{ 0x8af6, 0x8af6, PDF_CMAP_SINGLE, 8962 },
	{ 0x8af7, 0x8af7, PDF_CMAP_SINGLE, 7940 },
	{ 0x8af8, 0x8af8, PDF_CMAP_SINGLE, 8861 },
	{ 0x8af9, 0x8af9, PDF_CMAP_SINGLE, 19130 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 8699 },
	{ 0x8afb, 0x8afb, PDF_CMAP_SINGLE, 19131 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 8968 },
	{ 0x8afd, 0x8afd, PDF_CMAP_SINGLE, 19132 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 8364 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 19133 },
	{ 0x8b00, 0x8b00, PDF_CMAP_SINGLE, 8334 },
	{ 0x8b01, 0x8b01, PDF_CMAP_SINGLE, 8965 },
	{ 0x8b02, 0x8b02, PDF_CMAP_SINGLE, 8599 },
	{ 0x8b03, 0x8b03, PDF_CMAP_SINGLE, 19134 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 8561 },
	{ 0x8b05, 0x8b05, PDF_CMAP_SINGLE, 8856 },
	{ 0x8b06, 0x8b06, PDF_CMAP_SINGLE, 19135 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 5948 },
	{ 0x8b08, 0x8b09, PDF_CMAP_RANGE, 19136 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 8023 },
	{ 0x8b0b, 0x8b0d, PDF_CMAP_RANGE, 19138 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 8322 },
	{ 0x8b0f, 0x8b0f, PDF_CMAP_SINGLE, 19141 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 8977 },
	{ 0x8b11, 0x8b13, PDF_CMAP_RANGE, 19142 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 8964 },
	{ 0x8b15, 0x8b15, PDF_CMAP_SINGLE, 19145 },
	{ 0x8b16, 0x8b16, PDF_CMAP_SINGLE, 8975 },
	{ 0x8b17, 0x8b17, PDF_CMAP_SINGLE, 7733 },
	{ 0x8b18, 0x8b18, PDF_CMAP_SINGLE, 19146 },
	{ 0x8b19, 0x8b19, PDF_CMAP_SINGLE, 8401 },
	{ 0x8b1a, 0x8b1a, PDF_CMAP_SINGLE, 8976 },
	{ 0x8b1b, 0x8b1b, PDF_CMAP_SINGLE, 8099 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 19147 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 8662 },
	{ 0x8b1e, 0x8b20, PDF_CMAP_RANGE, 19148 },
	{ 0x8b21, 0x8b21, PDF_CMAP_SINGLE, 8710 },
	{ 0x8b22, 0x8b25, PDF_CMAP_RANGE, 19151 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 7519 },
	{ 0x8b27, 0x8b27, PDF_CMAP_SINGLE, 19155 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 8973 },
	{ 0x8b29, 0x8b2a, PDF_CMAP_RANGE, 19156 },
	{ 0x8b2b, 0x8b2b, PDF_CMAP_SINGLE, 8978 },
	{ 0x8b2c, 0x8b2c, PDF_CMAP_SINGLE, 8333 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 8979 },
	{ 0x8b2e, 0x8b32, PDF_CMAP_RANGE, 19158 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 8931 },
	{ 0x8b34, 0x8b38, PDF_CMAP_RANGE, 19163 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 8122 },
	{ 0x8b3a, 0x8b3d, PDF_CMAP_RANGE, 19168 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 8312 },
	{ 0x8b3f, 0x8b48, PDF_CMAP_RANGE, 19172 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 8840 },
	{ 0x8b4a, 0x8b4d, PDF_CMAP_RANGE, 19182 },
	{ 0x8b4e, 0x8b4e, PDF_CMAP_SINGLE, 8982 },
	{ 0x8b4f, 0x8b4f, PDF_CMAP_SINGLE, 8043 },
	{ 0x8b50, 0x8b55, PDF_CMAP_RANGE, 19186 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 8980 },
	{ 0x8b57, 0x8b57, PDF_CMAP_SINGLE, 19192 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 8496 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 8981 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 8552 },
	{ 0x8b5b, 0x8b5b, PDF_CMAP_SINGLE, 19193 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 8388 },
	{ 0x8b5d, 0x8b65, PDF_CMAP_RANGE, 19194 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 2270 },
	{ 0x8b67, 0x8b6a, PDF_CMAP_RANGE, 19203 },
	{ 0x8b6b, 0x8b6b, PDF_CMAP_SINGLE, 8984 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 3016 },
	{ 0x8b6d, 0x8b6e, PDF_CMAP_RANGE, 19207 },
	{ 0x8b6f, 0x8b6f, PDF_CMAP_SINGLE, 8729 },
	{ 0x8b70, 0x8b70, PDF_CMAP_SINGLE, 8727 },
	{ 0x8b71, 0x8b73, PDF_CMAP_RANGE, 19209 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 8405 },
	{ 0x8b75, 0x8b76, PDF_CMAP_RANGE, 19212 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 8010 },
	{ 0x8b78, 0x8b7c, PDF_CMAP_RANGE, 19214 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 8768 },
	{ 0x8b7e, 0x8b7f, PDF_CMAP_RANGE, 19219 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 7891 },
	{ 0x8b81, 0x8b89, PDF_CMAP_RANGE, 19221 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 7753 },
	{ 0x8b8b, 0x8b8d, PDF_CMAP_RANGE, 19230 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 9755 },
	{ 0x8b8f, 0x8b91, PDF_CMAP_RANGE, 19233 },
	{ 0x8b92, 0x8b92, PDF_CMAP_SINGLE, 7789 },
	{ 0x8b93, 0x8b93, PDF_CMAP_SINGLE, 8440 },
	{ 0x8b94, 0x8b94, PDF_CMAP_SINGLE, 19236 },
	{ 0x8b95, 0x8b95, PDF_CMAP_SINGLE, 8187 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 8985 },
	{ 0x8b97, 0x8b9b, PDF_CMAP_RANGE, 19237 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 8974 },
	{ 0x8b9d, 0x8b9d, PDF_CMAP_SINGLE, 19242 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 8983 },
	{ 0x8b9f, 0x8b9f, PDF_CMAP_SINGLE, 19243 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 4887 },
	{ 0x8ba1, 0x8ba1, PDF_CMAP_SINGLE, 2105 },
	{ 0x8ba2, 0x8ba2, PDF_CMAP_SINGLE, 1512 },
	{ 0x8ba3, 0x8ba3, PDF_CMAP_SINGLE, 1719 },
	{ 0x8ba4, 0x8ba4, PDF_CMAP_SINGLE, 3242 },
	{ 0x8ba5, 0x8ba5, PDF_CMAP_SINGLE, 2072 },
	{ 0x8ba6, 0x8ba7, PDF_CMAP_RANGE, 4888 },
	{ 0x8ba8, 0x8ba8, PDF_CMAP_SINGLE, 3625 },
	{ 0x8ba9, 0x8ba9, PDF_CMAP_SINGLE, 3230 },
	{ 0x8baa, 0x8baa, PDF_CMAP_SINGLE, 4890 },
	{ 0x8bab, 0x8bab, PDF_CMAP_SINGLE, 3100 },
	{ 0x8bac, 0x8bac, PDF_CMAP_SINGLE, 19244 },
	{ 0x8bad, 0x8bad, PDF_CMAP_SINGLE, 4062 },
	{ 0x8bae, 0x8bae, PDF_CMAP_SINGLE, 4208 },
	{ 0x8baf, 0x8baf, PDF_CMAP_SINGLE, 4063 },
	{ 0x8bb0, 0x8bb0, PDF_CMAP_SINGLE, 2106 },
	{ 0x8bb1, 0x8bb1, PDF_CMAP_SINGLE, 19245 },
	{ 0x8bb2, 0x8bb2, PDF_CMAP_SINGLE, 2179 },
	{ 0x8bb3, 0x8bb3, PDF_CMAP_SINGLE, 2041 },
	{ 0x8bb4, 0x8bb5, PDF_CMAP_RANGE, 4891 },
	{ 0x8bb6, 0x8bb6, PDF_CMAP_SINGLE, 4081 },
	{ 0x8bb7, 0x8bb7, PDF_CMAP_SINGLE, 4893 },
	{ 0x8bb8, 0x8bb8, PDF_CMAP_SINGLE, 4024 },
	{ 0x8bb9, 0x8bb9, PDF_CMAP_SINGLE, 1582 },
	{ 0x8bba, 0x8bba, PDF_CMAP_SINGLE, 2690 },
	{ 0x8bbb, 0x8bbb, PDF_CMAP_SINGLE, 19246 },
	{ 0x8bbc, 0x8bbc, PDF_CMAP_SINGLE, 3524 },
	{ 0x8bbd, 0x8bbd, PDF_CMAP_SINGLE, 1674 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 3361 },
	{ 0x8bbf, 0x8bbf, PDF_CMAP_SINGLE, 1632 },
	{ 0x8bc0, 0x8bc0, PDF_CMAP_SINGLE, 2342 },
	{ 0x8bc1, 0x8bc1, PDF_CMAP_SINGLE, 4515 },
	{ 0x8bc2, 0x8bc3, PDF_CMAP_RANGE, 4894 },
	{ 0x8bc4, 0x8bc4, PDF_CMAP_SINGLE, 3039 },
	{ 0x8bc5, 0x8bc5, PDF_CMAP_SINGLE, 4676 },
	{ 0x8bc6, 0x8bc6, PDF_CMAP_SINGLE, 3405 },
	{ 0x8bc7, 0x8bc7, PDF_CMAP_SINGLE, 19247 },
	{ 0x8bc8, 0x8bc8, PDF_CMAP_SINGLE, 4426 },
	{ 0x8bc9, 0x8bc9, PDF_CMAP_SINGLE, 3540 },
	{ 0x8bca, 0x8bca, PDF_CMAP_SINGLE, 4496 },
	{ 0x8bcb, 0x8bcb, PDF_CMAP_SINGLE, 4896 },
	{ 0x8bcc, 0x8bcc, PDF_CMAP_SINGLE, 4574 },
	{ 0x8bcd, 0x8bcd, PDF_CMAP_SINGLE, 1357 },
	{ 0x8bce, 0x8bce, PDF_CMAP_SINGLE, 4898 },
	{ 0x8bcf, 0x8bcf, PDF_CMAP_SINGLE, 4897 },
	{ 0x8bd0, 0x8bd0, PDF_CMAP_SINGLE, 19248 },
	{ 0x8bd1, 0x8bd1, PDF_CMAP_SINGLE, 4210 },
	{ 0x8bd2, 0x8bd4, PDF_CMAP_RANGE, 4899 },
	{ 0x8bd5, 0x8bd5, PDF_CMAP_SINGLE, 3435 },
	{ 0x8bd6, 0x8bd6, PDF_CMAP_SINGLE, 4902 },
	{ 0x8bd7, 0x8bd7, PDF_CMAP_SINGLE, 3394 },
	{ 0x8bd8, 0x8bd9, PDF_CMAP_RANGE, 4903 },
	{ 0x8bda, 0x8bda, PDF_CMAP_SINGLE, 1268 },
	{ 0x8bdb, 0x8bdb, PDF_CMAP_SINGLE, 4590 },
	{ 0x8bdc, 0x8bdc, PDF_CMAP_SINGLE, 4905 },
	{ 0x8bdd, 0x8bdd, PDF_CMAP_SINGLE, 1989 },
	{ 0x8bde, 0x8bde, PDF_CMAP_SINGLE, 1423 },
	{ 0x8bdf, 0x8be0, PDF_CMAP_RANGE, 4906 },
	{ 0x8be1, 0x8be1, PDF_CMAP_SINGLE, 1863 },
	{ 0x8be2, 0x8be2, PDF_CMAP_SINGLE, 4056 },
	{ 0x8be3, 0x8be3, PDF_CMAP_SINGLE, 4207 },
	{ 0x8be4, 0x8be4, PDF_CMAP_SINGLE, 4908 },
	{ 0x8be5, 0x8be5, PDF_CMAP_SINGLE, 1726 },
	{ 0x8be6, 0x8be6, PDF_CMAP_SINGLE, 3927 },
	{ 0x8be7, 0x8be7, PDF_CMAP_SINGLE, 1206 },
	{ 0x8be8, 0x8be9, PDF_CMAP_RANGE, 4909 },
	{ 0x8bea, 0x8bea, PDF_CMAP_SINGLE, 19249 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 2236 },
	{ 0x8bec, 0x8bec, PDF_CMAP_SINGLE, 3819 },
	{ 0x8bed, 0x8bed, PDF_CMAP_SINGLE, 4308 },
	{ 0x8bee, 0x8bee, PDF_CMAP_SINGLE, 4911 },
	{ 0x8bef, 0x8bef, PDF_CMAP_SINGLE, 3842 },
	{ 0x8bf0, 0x8bf0, PDF_CMAP_SINGLE, 4912 },
	{ 0x8bf1, 0x8bf1, PDF_CMAP_SINGLE, 4282 },
	{ 0x8bf2, 0x8bf2, PDF_CMAP_SINGLE, 2042 },
	{ 0x8bf3, 0x8bf3, PDF_CMAP_SINGLE, 4913 },
	{ 0x8bf4, 0x8bf4, PDF_CMAP_SINGLE, 3498 },
	{ 0x8bf5, 0x8bf5, PDF_CMAP_SINGLE, 3525 },
	{ 0x8bf6, 0x8bf6, PDF_CMAP_SINGLE, 4914 },
	{ 0x8bf7, 0x8bf7, PDF_CMAP_SINGLE, 3176 },
	{ 0x8bf8, 0x8bf8, PDF_CMAP_SINGLE, 4589 },
	{ 0x8bf9, 0x8bf9, PDF_CMAP_SINGLE, 4915 },
	{ 0x8bfa, 0x8bfa, PDF_CMAP_SINGLE, 2934 },
	{ 0x8bfb, 0x8bfb, PDF_CMAP_SINGLE, 1536 },
	{ 0x8bfc, 0x8bfc, PDF_CMAP_SINGLE, 4916 },
	{ 0x8bfd, 0x8bfd, PDF_CMAP_SINGLE, 1641 },
	{ 0x8bfe, 0x8bfe, PDF_CMAP_SINGLE, 2395 },
	{ 0x8bff, 0x8bff, PDF_CMAP_SINGLE, 4917 },
	{ 0x8c00, 0x8c00, PDF_CMAP_SINGLE, 4918 },
	{ 0x8c01, 0x8c01, PDF_CMAP_SINGLE, 3490 },
	{ 0x8c02, 0x8c02, PDF_CMAP_SINGLE, 4919 },
	{ 0x8c03, 0x8c03, PDF_CMAP_SINGLE, 1496 },
	{ 0x8c04, 0x8c04, PDF_CMAP_SINGLE, 4920 },
	{ 0x8c05, 0x8c05, PDF_CMAP_SINGLE, 2571 },
	{ 0x8c06, 0x8c06, PDF_CMAP_SINGLE, 4632 },
	{ 0x8c07, 0x8c07, PDF_CMAP_SINGLE, 4921 },
	{ 0x8c08, 0x8c08, PDF_CMAP_SINGLE, 3595 },
	{ 0x8c09, 0x8c09, PDF_CMAP_SINGLE, 19250 },
	{ 0x8c0a, 0x8c0a, PDF_CMAP_SINGLE, 4209 },
	{ 0x8c0b, 0x8c0b, PDF_CMAP_SINGLE, 2836 },
	{ 0x8c0c, 0x8c0c, PDF_CMAP_SINGLE, 4922 },
	{ 0x8c0d, 0x8c0d, PDF_CMAP_SINGLE, 1502 },
	{ 0x8c0e, 0x8c0e, PDF_CMAP_SINGLE, 2022 },
	{ 0x8c0f, 0x8c0f, PDF_CMAP_SINGLE, 4923 },
	{ 0x8c10, 0x8c10, PDF_CMAP_SINGLE, 3966 },
	{ 0x8c11, 0x8c12, PDF_CMAP_RANGE, 4924 },
	{ 0x8c13, 0x8c13, PDF_CMAP_SINGLE, 3788 },
	{ 0x8c14, 0x8c16, PDF_CMAP_RANGE, 4926 },
	{ 0x8c17, 0x8c17, PDF_CMAP_SINGLE, 1214 },
	{ 0x8c18, 0x8c18, PDF_CMAP_SINGLE, 4931 },
	{ 0x8c19, 0x8c19, PDF_CMAP_SINGLE, 4929 },
	{ 0x8c1a, 0x8c1a, PDF_CMAP_SINGLE, 4113 },
	{ 0x8c1b, 0x8c1b, PDF_CMAP_SINGLE, 4930 },
	{ 0x8c1c, 0x8c1c, PDF_CMAP_SINGLE, 2778 },
	{ 0x8c1d, 0x8c1d, PDF_CMAP_SINGLE, 4932 },
	{ 0x8c1e, 0x8c1e, PDF_CMAP_SINGLE, 19251 },
	{ 0x8c1f, 0x8c21, PDF_CMAP_RANGE, 4933 },
	{ 0x8c22, 0x8c22, PDF_CMAP_SINGLE, 3974 },
	{ 0x8c23, 0x8c23, PDF_CMAP_SINGLE, 4140 },
	{ 0x8c24, 0x8c24, PDF_CMAP_SINGLE, 1028 },
	{ 0x8c25, 0x8c25, PDF_CMAP_SINGLE, 4936 },
	{ 0x8c26, 0x8c26, PDF_CMAP_SINGLE, 3112 },
	{ 0x8c27, 0x8c27, PDF_CMAP_SINGLE, 4937 },
	{ 0x8c28, 0x8c28, PDF_CMAP_SINGLE, 2248 },
	{ 0x8c29, 0x8c29, PDF_CMAP_SINGLE, 2726 },
	{ 0x8c2a, 0x8c2b, PDF_CMAP_RANGE, 4938 },
	{ 0x8c2c, 0x8c2c, PDF_CMAP_SINGLE, 2818 },
	{ 0x8c2d, 0x8c2d, PDF_CMAP_SINGLE, 3594 },
	{ 0x8c2e, 0x8c2f, PDF_CMAP_RANGE, 4940 },
	{ 0x8c30, 0x8c30, PDF_CMAP_SINGLE, 2473 },
	{ 0x8c31, 0x8c31, PDF_CMAP_SINGLE, 3062 },
	{ 0x8c32, 0x8c33, PDF_CMAP_RANGE, 4942 },
	{ 0x8c34, 0x8c34, PDF_CMAP_SINGLE, 3121 },
	{ 0x8c35, 0x8c36, PDF_CMAP_RANGE, 4944 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1825 },
	{ 0x8c38, 0x8c40, PDF_CMAP_RANGE, 19252 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 2050 },
	{ 0x8c42, 0x8c45, PDF_CMAP_RANGE, 19261 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 1528 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 7414 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 8392 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 7415 },
	{ 0x8c4a, 0x8c4b, PDF_CMAP_RANGE, 19265 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 3732 },
	{ 0x8c4d, 0x8c4f, PDF_CMAP_RANGE, 19267 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 7933 },
	{ 0x8c51, 0x8c54, PDF_CMAP_RANGE, 19270 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 7445 },
	{ 0x8c56, 0x8c59, PDF_CMAP_RANGE, 19274 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 6544 },
	{ 0x8c5b, 0x8c60, PDF_CMAP_RANGE, 19278 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 3936 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 2004 },
	{ 0x8c63, 0x8c69, PDF_CMAP_RANGE, 19284 },
	{ 0x8c6a, 0x8c6a, PDF_CMAP_SINGLE, 1911 },
	{ 0x8c6b, 0x8c6b, PDF_CMAP_SINGLE, 4328 },
	{ 0x8c6c, 0x8c72, PDF_CMAP_RANGE, 19291 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 5597 },
	{ 0x8c74, 0x8c77, PDF_CMAP_RANGE, 19298 },
	{ 0x8c78, 0x8c78, PDF_CMAP_SINGLE, 7504 },
	{ 0x8c79, 0x8c79, PDF_CMAP_SINGLE, 1043 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 1209 },
	{ 0x8c7b, 0x8c81, PDF_CMAP_RANGE, 19302 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 7505 },
	{ 0x8c83, 0x8c84, PDF_CMAP_RANGE, 19309 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 7507 },
	{ 0x8c86, 0x8c88, PDF_CMAP_RANGE, 19311 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 1928 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 7506 },
	{ 0x8c8b, 0x8c8b, PDF_CMAP_SINGLE, 19314 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 2743 },
	{ 0x8c8d, 0x8c93, PDF_CMAP_RANGE, 19315 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 7509 },
	{ 0x8c95, 0x8c97, PDF_CMAP_RANGE, 19322 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 7508 },
	{ 0x8c99, 0x8c9c, PDF_CMAP_RANGE, 19325 },
	{ 0x8c9d, 0x8c9d, PDF_CMAP_SINGLE, 7739 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 8831 },
	{ 0x8c9f, 0x8c9f, PDF_CMAP_SINGLE, 19329 },
	{ 0x8ca0, 0x8ca0, PDF_CMAP_SINGLE, 7948 },
	{ 0x8ca1, 0x8ca1, PDF_CMAP_SINGLE, 7769 },
	{ 0x8ca2, 0x8ca2, PDF_CMAP_SINGLE, 7972 },
	{ 0x8ca3, 0x8ca6, PDF_CMAP_RANGE, 19330 },
	{ 0x8ca7, 0x8ca7, PDF_CMAP_SINGLE, 8378 },
	{ 0x8ca8, 0x8ca8, PDF_CMAP_SINGLE, 8037 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 7920 },
	{ 0x8caa, 0x8caa, PDF_CMAP_SINGLE, 8548 },
	{ 0x8cab, 0x8cab, PDF_CMAP_SINGLE, 7984 },
	{ 0x8cac, 0x8cac, PDF_CMAP_SINGLE, 8801 },
	{ 0x8cad, 0x8cae, PDF_CMAP_RANGE, 19334 },
	{ 0x8caf, 0x8caf, PDF_CMAP_SINGLE, 8866 },
	{ 0x8cb0, 0x8cb0, PDF_CMAP_SINGLE, 9387 },
	{ 0x8cb1, 0x8cb1, PDF_CMAP_SINGLE, 19336 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 9391 },
	{ 0x8cb3, 0x8cb3, PDF_CMAP_SINGLE, 7912 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 7993 },
	{ 0x8cb5, 0x8cb5, PDF_CMAP_SINGLE, 19337 },
	{ 0x8cb6, 0x8cb6, PDF_CMAP_SINGLE, 7752 },
	{ 0x8cb7, 0x8cb7, PDF_CMAP_SINGLE, 8304 },
	{ 0x8cb8, 0x8cb8, PDF_CMAP_SINGLE, 7848 },
	{ 0x8cb9, 0x8cb9, PDF_CMAP_SINGLE, 19338 },
	{ 0x8cba, 0x8cba, PDF_CMAP_SINGLE, 9388 },
	{ 0x8cbb, 0x8cbb, PDF_CMAP_SINGLE, 7927 },
	{ 0x8cbc, 0x8cbc, PDF_CMAP_SINGLE, 8566 },
	{ 0x8cbd, 0x8cbd, PDF_CMAP_SINGLE, 9389 },
	{ 0x8cbe, 0x8cbe, PDF_CMAP_SINGLE, 19339 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 8315 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 8005 },
	{ 0x8cc1, 0x8cc1, PDF_CMAP_SINGLE, 9386 },
	{ 0x8cc2, 0x8cc2, PDF_CMAP_SINGLE, 8267 },
	{ 0x8cc3, 0x8cc3, PDF_CMAP_SINGLE, 8239 },
	{ 0x8cc4, 0x8cc4, PDF_CMAP_SINGLE, 8026 },
	{ 0x8cc5, 0x8cc5, PDF_CMAP_SINGLE, 9392 },
	{ 0x8cc6, 0x8cc6, PDF_CMAP_SINGLE, 19340 },
	{ 0x8cc7, 0x8cc7, PDF_CMAP_SINGLE, 8887 },
	{ 0x8cc8, 0x8cc8, PDF_CMAP_SINGLE, 8063 },
	{ 0x8cc9, 0x8cc9, PDF_CMAP_SINGLE, 19341 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 8805 },
	{ 0x8ccb, 0x8cd0, PDF_CMAP_RANGE, 19342 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 9394 },
	{ 0x8cd2, 0x8cd2, PDF_CMAP_SINGLE, 8475 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 7761 },
	{ 0x8cd4, 0x8cd4, PDF_CMAP_SINGLE, 19348 },
	{ 0x8cd5, 0x8cd5, PDF_CMAP_SINGLE, 9396 },
	{ 0x8cd6, 0x8cd9, PDF_CMAP_RANGE, 19349 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 9395 },
	{ 0x8cdb, 0x8cdb, PDF_CMAP_SINGLE, 19353 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 7839 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 19354 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 8472 },
	{ 0x8cdf, 0x8cdf, PDF_CMAP_SINGLE, 19355 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 8372 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 9135 },
	{ 0x8ce2, 0x8ce2, PDF_CMAP_SINGLE, 8635 },
	{ 0x8ce3, 0x8ce3, PDF_CMAP_SINGLE, 8306 },
	{ 0x8ce4, 0x8ce4, PDF_CMAP_SINGLE, 8085 },
	{ 0x8ce5, 0x8ce5, PDF_CMAP_SINGLE, 19356 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 7946 },
	{ 0x8ce7, 0x8ce7, PDF_CMAP_SINGLE, 9398 },
	{ 0x8ce8, 0x8ce9, PDF_CMAP_RANGE, 19357 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 8849 },
	{ 0x8ceb, 0x8ceb, PDF_CMAP_SINGLE, 9397 },
	{ 0x8cec, 0x8cec, PDF_CMAP_SINGLE, 8824 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 7892 },
	{ 0x8cee, 0x8cf3, PDF_CMAP_RANGE, 19359 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8179 },
	{ 0x8cf5, 0x8cf9, PDF_CMAP_RANGE, 19365 },
	{ 0x8cfa, 0x8cfa, PDF_CMAP_SINGLE, 8873 },
	{ 0x8cfb, 0x8cfb, PDF_CMAP_SINGLE, 9399 },
	{ 0x8cfc, 0x8cfc, PDF_CMAP_SINGLE, 7976 },
	{ 0x8cfd, 0x8cfd, PDF_CMAP_SINGLE, 8457 },
	{ 0x8cfe, 0x8cfe, PDF_CMAP_SINGLE, 8903 },
	{ 0x8cff, 0x8cff, PDF_CMAP_SINGLE, 19370 },
	{ 0x8d00, 0x8d03, PDF_CMAP_RANGE, 19371 },
	{ 0x8d04, 0x8d04, PDF_CMAP_SINGLE, 9390 },
	{ 0x8d05, 0x8d05, PDF_CMAP_SINGLE, 8881 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 19375 },
	{ 0x8d08, 0x8d08, PDF_CMAP_SINGLE, 8806 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 19377 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 8795 },
	{ 0x8d0b, 0x8d0b, PDF_CMAP_SINGLE, 8900 },
	{ 0x8d0c, 0x8d0c, PDF_CMAP_SINGLE, 19378 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 8469 },
	{ 0x8d0e, 0x8d0e, PDF_CMAP_SINGLE, 19379 },
	{ 0x8d0f, 0x8d0f, PDF_CMAP_SINGLE, 8746 },
	{ 0x8d10, 0x8d10, PDF_CMAP_SINGLE, 9393 },
	{ 0x8d11, 0x8d15, PDF_CMAP_RANGE, 19380 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 8509 },
	{ 0x8d17, 0x8d1a, PDF_CMAP_RANGE, 19385 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 7957 },
	{ 0x8d1c, 0x8d1c, PDF_CMAP_SINGLE, 8796 },
	{ 0x8d1d, 0x8d1d, PDF_CMAP_SINGLE, 1053 },
	{ 0x8d1e, 0x8d1e, PDF_CMAP_SINGLE, 4491 },
	{ 0x8d1f, 0x8d1f, PDF_CMAP_SINGLE, 1717 },
	{ 0x8d20, 0x8d20, PDF_CMAP_SINGLE, 19389 },
	{ 0x8d21, 0x8d21, PDF_CMAP_SINGLE, 1802 },
	{ 0x8d22, 0x8d22, PDF_CMAP_SINGLE, 1165 },
	{ 0x8d23, 0x8d23, PDF_CMAP_SINGLE, 4403 },
	{ 0x8d24, 0x8d24, PDF_CMAP_SINGLE, 3898 },
	{ 0x8d25, 0x8d25, PDF_CMAP_SINGLE, 999 },
	{ 0x8d26, 0x8d26, PDF_CMAP_SINGLE, 4460 },
	{ 0x8d27, 0x8d27, PDF_CMAP_SINGLE, 2058 },
	{ 0x8d28, 0x8d28, PDF_CMAP_SINGLE, 4553 },
	{ 0x8d29, 0x8d29, PDF_CMAP_SINGLE, 1620 },
	{ 0x8d2a, 0x8d2a, PDF_CMAP_SINGLE, 3587 },
	{ 0x8d2b, 0x8d2b, PDF_CMAP_SINGLE, 3029 },
	{ 0x8d2c, 0x8d2c, PDF_CMAP_SINGLE, 1098 },
	{ 0x8d2d, 0x8d2d, PDF_CMAP_SINGLE, 1811 },
	{ 0x8d2e, 0x8d2e, PDF_CMAP_SINGLE, 4603 },
	{ 0x8d2f, 0x8d2f, PDF_CMAP_SINGLE, 1850 },
	{ 0x8d30, 0x8d30, PDF_CMAP_SINGLE, 1598 },
	{ 0x8d31, 0x8d31, PDF_CMAP_SINGLE, 2157 },
	{ 0x8d32, 0x8d33, PDF_CMAP_RANGE, 6444 },
	{ 0x8d34, 0x8d34, PDF_CMAP_SINGLE, 3660 },
	{ 0x8d35, 0x8d35, PDF_CMAP_SINGLE, 1868 },
	{ 0x8d36, 0x8d36, PDF_CMAP_SINGLE, 6446 },
	{ 0x8d37, 0x8d37, PDF_CMAP_SINGLE, 1406 },
	{ 0x8d38, 0x8d38, PDF_CMAP_SINGLE, 2744 },
	{ 0x8d39, 0x8d39, PDF_CMAP_SINGLE, 1646 },
	{ 0x8d3a, 0x8d3a, PDF_CMAP_SINGLE, 1935 },
	{ 0x8d3b, 0x8d3b, PDF_CMAP_SINGLE, 6447 },
	{ 0x8d3c, 0x8d3c, PDF_CMAP_SINGLE, 4407 },
	{ 0x8d3d, 0x8d3d, PDF_CMAP_SINGLE, 6448 },
	{ 0x8d3e, 0x8d3e, PDF_CMAP_SINGLE, 2121 },
	{ 0x8d3f, 0x8d3f, PDF_CMAP_SINGLE, 2036 },
	{ 0x8d40, 0x8d40, PDF_CMAP_SINGLE, 6449 },
	{ 0x8d41, 0x8d41, PDF_CMAP_SINGLE, 2599 },
	{ 0x8d42, 0x8d42, PDF_CMAP_SINGLE, 2655 },
	{ 0x8d43, 0x8d43, PDF_CMAP_SINGLE, 4386 },
	{ 0x8d44, 0x8d44, PDF_CMAP_SINGLE, 4647 },
	{ 0x8d45, 0x8d46, PDF_CMAP_RANGE, 6450 },
	{ 0x8d47, 0x8d47, PDF_CMAP_SINGLE, 6454 },
	{ 0x8d48, 0x8d49, PDF_CMAP_RANGE, 6452 },
	{ 0x8d4a, 0x8d4a, PDF_CMAP_SINGLE, 3351 },
	{ 0x8d4b, 0x8d4b, PDF_CMAP_SINGLE, 1710 },
	{ 0x8d4c, 0x8d4c, PDF_CMAP_SINGLE, 1539 },
	{ 0x8d4d, 0x8d4d, PDF_CMAP_SINGLE, 6455 },
	{ 0x8d4e, 0x8d4e, PDF_CMAP_SINGLE, 3457 },
	{ 0x8d4f, 0x8d4f, PDF_CMAP_SINGLE, 3334 },
	{ 0x8d50, 0x8d50, PDF_CMAP_SINGLE, 1360 },
	{ 0x8d51, 0x8d52, PDF_CMAP_RANGE, 19390 },
	{ 0x8d53, 0x8d53, PDF_CMAP_SINGLE, 5691 },
	{ 0x8d54, 0x8d54, PDF_CMAP_SINGLE, 2979 },
	{ 0x8d55, 0x8d55, PDF_CMAP_SINGLE, 6456 },
	{ 0x8d56, 0x8d56, PDF_CMAP_SINGLE, 2464 },
	{ 0x8d57, 0x8d57, PDF_CMAP_SINGLE, 19392 },
	{ 0x8d58, 0x8d58, PDF_CMAP_SINGLE, 4629 },
	{ 0x8d59, 0x8d59, PDF_CMAP_SINGLE, 6457 },
	{ 0x8d5a, 0x8d5a, PDF_CMAP_SINGLE, 4617 },
	{ 0x8d5b, 0x8d5b, PDF_CMAP_SINGLE, 3287 },
	{ 0x8d5c, 0x8d5c, PDF_CMAP_SINGLE, 4745 },
	{ 0x8d5d, 0x8d5d, PDF_CMAP_SINGLE, 4739 },
	{ 0x8d5e, 0x8d5e, PDF_CMAP_SINGLE, 4385 },
	{ 0x8d5f, 0x8d5f, PDF_CMAP_SINGLE, 19393 },
	{ 0x8d60, 0x8d60, PDF_CMAP_SINGLE, 4412 },
	{ 0x8d61, 0x8d61, PDF_CMAP_SINGLE, 3325 },
	{ 0x8d62, 0x8d62, PDF_CMAP_SINGLE, 4243 },
	{ 0x8d63, 0x8d63, PDF_CMAP_SINGLE, 1742 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 1285 },
	{ 0x8d65, 0x8d65, PDF_CMAP_SINGLE, 19394 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 3355 },
	{ 0x8d67, 0x8d67, PDF_CMAP_SINGLE, 7412 },
	{ 0x8d68, 0x8d6a, PDF_CMAP_RANGE, 19395 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 1932 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 19398 },
	{ 0x8d6d, 0x8d6d, PDF_CMAP_SINGLE, 7413 },
	{ 0x8d6e, 0x8d6f, PDF_CMAP_RANGE, 19399 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 4668 },
	{ 0x8d71, 0x8d72, PDF_CMAP_RANGE, 19401 },
	{ 0x8d73, 0x8d73, PDF_CMAP_SINGLE, 7407 },
	{ 0x8d74, 0x8d74, PDF_CMAP_SINGLE, 1707 },
	{ 0x8d75, 0x8d75, PDF_CMAP_SINGLE, 4469 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 1738 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 3087 },
	{ 0x8d78, 0x8d80, PDF_CMAP_RANGE, 19403 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 1256 },
	{ 0x8d82, 0x8d83, PDF_CMAP_RANGE, 19412 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 7408 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 1233 },
	{ 0x8d86, 0x8d89, PDF_CMAP_RANGE, 19414 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 4352 },
	{ 0x8d8b, 0x8d8b, PDF_CMAP_SINGLE, 3188 },
	{ 0x8d8c, 0x8d90, PDF_CMAP_RANGE, 19418 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 7410 },
	{ 0x8d92, 0x8d93, PDF_CMAP_RANGE, 19423 },
	{ 0x8d94, 0x8d94, PDF_CMAP_SINGLE, 7409 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 7956 },
	{ 0x8d96, 0x8d98, PDF_CMAP_RANGE, 19425 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 8826 },
	{ 0x8d9a, 0x8d9e, PDF_CMAP_RANGE, 19428 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 3614 },
	{ 0x8da0, 0x8da2, PDF_CMAP_RANGE, 19433 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 3199 },
	{ 0x8da4, 0x8da7, PDF_CMAP_RANGE, 19436 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 8430 },
	{ 0x8da9, 0x8db0, PDF_CMAP_RANGE, 19440 },
	{ 0x8db1, 0x8db1, PDF_CMAP_SINGLE, 7411 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 9720 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 4672 },
	{ 0x8db4, 0x8db4, PDF_CMAP_SINGLE, 2944 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 7452 },
	{ 0x8db6, 0x8db7, PDF_CMAP_RANGE, 19448 },
	{ 0x8db8, 0x8db8, PDF_CMAP_SINGLE, 7447 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 19450 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 7455 },
	{ 0x8dbb, 0x8dbb, PDF_CMAP_SINGLE, 19451 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 7454 },
	{ 0x8dbd, 0x8dbd, PDF_CMAP_SINGLE, 19452 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 4537 },
	{ 0x8dbf, 0x8dbf, PDF_CMAP_SINGLE, 7453 },
	{ 0x8dc0, 0x8dc2, PDF_CMAP_RANGE, 19453 },
	{ 0x8dc3, 0x8dc3, PDF_CMAP_SINGLE, 4353 },
	{ 0x8dc4, 0x8dc4, PDF_CMAP_SINGLE, 7456 },
	{ 0x8dc5, 0x8dc5, PDF_CMAP_SINGLE, 19456 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 7464 },
	{ 0x8dc7, 0x8dca, PDF_CMAP_RANGE, 19457 },
	{ 0x8dcb, 0x8dcb, PDF_CMAP_SINGLE, 986 },
	{ 0x8dcc, 0x8dcc, PDF_CMAP_SINGLE, 1497 },
	{ 0x8dcd, 0x8dcd, PDF_CMAP_SINGLE, 19461 },
	{ 0x8dce, 0x8dcf, PDF_CMAP_RANGE, 7461 },
	{ 0x8dd0, 0x8dd0, PDF_CMAP_SINGLE, 19462 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 2973 },
	{ 0x8dd2, 0x8dd5, PDF_CMAP_RANGE, 19463 },
	{ 0x8dd6, 0x8dd7, PDF_CMAP_RANGE, 7457 },
	{ 0x8dd8, 0x8dd9, PDF_CMAP_RANGE, 19467 },
	{ 0x8dda, 0x8dda, PDF_CMAP_SINGLE, 7459 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 7463 },
	{ 0x8ddc, 0x8ddc, PDF_CMAP_SINGLE, 19469 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 2319 },
	{ 0x8dde, 0x8dde, PDF_CMAP_SINGLE, 7460 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 1781 },
	{ 0x8de0, 0x8de2, PDF_CMAP_RANGE, 19470 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 7468 },
	{ 0x8de4, 0x8de4, PDF_CMAP_SINGLE, 7471 },
	{ 0x8de5, 0x8de7, PDF_CMAP_RANGE, 19473 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 2420 },
	{ 0x8de9, 0x8de9, PDF_CMAP_SINGLE, 19476 },
	{ 0x8dea, 0x8dea, PDF_CMAP_SINGLE, 1867 },
	{ 0x8deb, 0x8deb, PDF_CMAP_SINGLE, 7448 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 7465 },
	{ 0x8ded, 0x8dee, PDF_CMAP_RANGE, 19477 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 2654 },
	{ 0x8df0, 0x8df2, PDF_CMAP_RANGE, 19479 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3659 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 19482 },
	{ 0x8df5, 0x8df5, PDF_CMAP_SINGLE, 2156 },
	{ 0x8df6, 0x8df6, PDF_CMAP_SINGLE, 19483 },
	{ 0x8df7, 0x8df8, PDF_CMAP_RANGE, 7466 },
	{ 0x8df9, 0x8df9, PDF_CMAP_SINGLE, 7469 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 1572 },
	{ 0x8dfb, 0x8dfb, PDF_CMAP_SINGLE, 7470 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 19484 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 7473 },
	{ 0x8dfe, 0x8dff, PDF_CMAP_RANGE, 19485 },
	{ 0x8e00, 0x8e04, PDF_CMAP_RANGE, 19487 },
	{ 0x8e05, 0x8e05, PDF_CMAP_SINGLE, 7449 },
	{ 0x8e06, 0x8e08, PDF_CMAP_RANGE, 19492 },
	{ 0x8e09, 0x8e09, PDF_CMAP_SINGLE, 7472 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 4256 },
	{ 0x8e0b, 0x8e0b, PDF_CMAP_SINGLE, 19495 },
	{ 0x8e0c, 0x8e0c, PDF_CMAP_SINGLE, 1297 },
	{ 0x8e0d, 0x8e0e, PDF_CMAP_RANGE, 19496 },
	{ 0x8e0f, 0x8e0f, PDF_CMAP_SINGLE, 3575 },
	{ 0x8e10, 0x8e10, PDF_CMAP_SINGLE, 8084 },
	{ 0x8e11, 0x8e13, PDF_CMAP_RANGE, 19498 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 7474 },
	{ 0x8e15, 0x8e1c, PDF_CMAP_RANGE, 19501 },
	{ 0x8e1d, 0x8e1d, PDF_CMAP_SINGLE, 7475 },
	{ 0x8e1e, 0x8e1e, PDF_CMAP_SINGLE, 2320 },
	{ 0x8e1f, 0x8e1f, PDF_CMAP_SINGLE, 7476 },
	{ 0x8e20, 0x8e21, PDF_CMAP_RANGE, 19509 },
	{ 0x8e22, 0x8e22, PDF_CMAP_SINGLE, 3634 },
	{ 0x8e23, 0x8e23, PDF_CMAP_SINGLE, 7479 },
	{ 0x8e24, 0x8e28, PDF_CMAP_RANGE, 19511 },
	{ 0x8e29, 0x8e29, PDF_CMAP_SINGLE, 1167 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 4662 },
	{ 0x8e2b, 0x8e2b, PDF_CMAP_SINGLE, 19516 },
	{ 0x8e2c, 0x8e2c, PDF_CMAP_SINGLE, 7477 },
	{ 0x8e2d, 0x8e2d, PDF_CMAP_SINGLE, 19517 },
	{ 0x8e2e, 0x8e2e, PDF_CMAP_SINGLE, 7478 },
	{ 0x8e2f, 0x8e2f, PDF_CMAP_SINGLE, 7480 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 19518 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 7486 },
	{ 0x8e32, 0x8e33, PDF_CMAP_RANGE, 19519 },
	{ 0x8e34, 0x8e34, PDF_CMAP_SINGLE, 8752 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 7484 },
	{ 0x8e36, 0x8e38, PDF_CMAP_RANGE, 19521 },
	{ 0x8e39, 0x8e39, PDF_CMAP_SINGLE, 7483 },
	{ 0x8e3a, 0x8e3a, PDF_CMAP_SINGLE, 7481 },
	{ 0x8e3b, 0x8e3c, PDF_CMAP_RANGE, 19524 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 7485 },
	{ 0x8e3e, 0x8e3f, PDF_CMAP_RANGE, 19526 },
	{ 0x8e40, 0x8e40, PDF_CMAP_SINGLE, 7482 },
	{ 0x8e41, 0x8e42, PDF_CMAP_RANGE, 7488 },
	{ 0x8e43, 0x8e43, PDF_CMAP_SINGLE, 19528 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 3638 },
	{ 0x8e45, 0x8e46, PDF_CMAP_RANGE, 19529 },
	{ 0x8e47, 0x8e47, PDF_CMAP_SINGLE, 5947 },
	{ 0x8e48, 0x8e48, PDF_CMAP_SINGLE, 1433 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 7487 },
	{ 0x8e4a, 0x8e4a, PDF_CMAP_SINGLE, 7492 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 3574 },
	{ 0x8e4c, 0x8e4c, PDF_CMAP_SINGLE, 9725 },
	{ 0x8e4d, 0x8e50, PDF_CMAP_RANGE, 19531 },
	{ 0x8e51, 0x8e52, PDF_CMAP_RANGE, 7490 },
	{ 0x8e53, 0x8e54, PDF_CMAP_RANGE, 19535 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 9728 },
	{ 0x8e56, 0x8e58, PDF_CMAP_RANGE, 19537 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 7450 },
	{ 0x8e5a, 0x8e62, PDF_CMAP_RANGE, 19540 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 9734 },
	{ 0x8e64, 0x8e65, PDF_CMAP_RANGE, 19549 },
	{ 0x8e66, 0x8e66, PDF_CMAP_SINGLE, 1069 },
	{ 0x8e67, 0x8e68, PDF_CMAP_RANGE, 19551 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 7451 },
	{ 0x8e6a, 0x8e6b, PDF_CMAP_RANGE, 19553 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 1446 },
	{ 0x8e6d, 0x8e6d, PDF_CMAP_SINGLE, 1195 },
	{ 0x8e6e, 0x8e6e, PDF_CMAP_SINGLE, 19555 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 7496 },
	{ 0x8e70, 0x8e70, PDF_CMAP_SINGLE, 7493 },
	{ 0x8e71, 0x8e71, PDF_CMAP_SINGLE, 19556 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 1558 },
	{ 0x8e73, 0x8e73, PDF_CMAP_SINGLE, 19557 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 7497 },
	{ 0x8e75, 0x8e75, PDF_CMAP_SINGLE, 19558 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 7494 },
	{ 0x8e77, 0x8e79, PDF_CMAP_RANGE, 19559 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 9727 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 19562 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 7495 },
	{ 0x8e7d, 0x8e7e, PDF_CMAP_RANGE, 19563 },
	{ 0x8e7f, 0x8e7f, PDF_CMAP_SINGLE, 1373 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 19565 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 4397 },
	{ 0x8e82, 0x8e84, PDF_CMAP_RANGE, 19566 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 7498 },
	{ 0x8e86, 0x8e86, PDF_CMAP_SINGLE, 19569 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 1310 },
	{ 0x8e88, 0x8e88, PDF_CMAP_SINGLE, 19570 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 9724 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 7820 },
	{ 0x8e8b, 0x8e8b, PDF_CMAP_SINGLE, 9730 },
	{ 0x8e8c, 0x8e8c, PDF_CMAP_SINGLE, 19571 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 8781 },
	{ 0x8e8e, 0x8e8e, PDF_CMAP_SINGLE, 19572 },
	{ 0x8e8f, 0x8e8f, PDF_CMAP_SINGLE, 7499 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 7501 },
	{ 0x8e91, 0x8e91, PDF_CMAP_SINGLE, 9732 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 9726 },
	{ 0x8e93, 0x8e93, PDF_CMAP_SINGLE, 9731 },
	{ 0x8e94, 0x8e94, PDF_CMAP_SINGLE, 7500 },
	{ 0x8e95, 0x8e99, PDF_CMAP_RANGE, 19573 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 9729 },
	{ 0x8e9b, 0x8e9b, PDF_CMAP_SINGLE, 19578 },
	{ 0x8e9c, 0x8e9c, PDF_CMAP_SINGLE, 7502 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 19579 },
	{ 0x8e9e, 0x8e9e, PDF_CMAP_SINGLE, 7503 },
	{ 0x8e9f, 0x8ea0, PDF_CMAP_RANGE, 19580 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 9733 },
	{ 0x8ea2, 0x8ea4, PDF_CMAP_RANGE, 19582 },
	{ 0x8ea5, 0x8ea5, PDF_CMAP_SINGLE, 7843 },
	{ 0x8ea6, 0x8ea6, PDF_CMAP_SINGLE, 9736 },
	{ 0x8ea7, 0x8ea9, PDF_CMAP_RANGE, 19585 },
	{ 0x8eaa, 0x8eaa, PDF_CMAP_SINGLE, 9735 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 3366 },
	{ 0x8eac, 0x8eac, PDF_CMAP_SINGLE, 1795 },
	{ 0x8ead, 0x8eae, PDF_CMAP_RANGE, 19588 },
	{ 0x8eaf, 0x8eaf, PDF_CMAP_SINGLE, 3192 },
	{ 0x8eb0, 0x8eb1, PDF_CMAP_RANGE, 19590 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 1570 },
	{ 0x8eb3, 0x8eb9, PDF_CMAP_RANGE, 19592 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 3612 },
	{ 0x8ebb, 0x8ebf, PDF_CMAP_RANGE, 19599 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 8432 },
	{ 0x8ec1, 0x8ec9, PDF_CMAP_RANGE, 19604 },
	{ 0x8eca, 0x8eca, PDF_CMAP_SINGLE, 7803 },
	{ 0x8ecb, 0x8ecb, PDF_CMAP_SINGLE, 8807 },
	{ 0x8ecc, 0x8ecc, PDF_CMAP_SINGLE, 7990 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 8150 },
	{ 0x8ece, 0x8ece, PDF_CMAP_SINGLE, 6392 },
	{ 0x8ecf, 0x8ed1, PDF_CMAP_RANGE, 19613 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 8672 },
	{ 0x8ed3, 0x8ed3, PDF_CMAP_SINGLE, 19616 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 9356 },
	{ 0x8ed5, 0x8eda, PDF_CMAP_RANGE, 19617 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 9357 },
	{ 0x8edc, 0x8ede, PDF_CMAP_RANGE, 19623 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 8450 },
	{ 0x8ee0, 0x8ee3, PDF_CMAP_RANGE, 19626 },
	{ 0x8ee4, 0x8ee4, PDF_CMAP_SINGLE, 9364 },
	{ 0x8ee5, 0x8eea, PDF_CMAP_RANGE, 19630 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 9363 },
	{ 0x8eec, 0x8ef1, PDF_CMAP_RANGE, 19636 },
	{ 0x8ef2, 0x8ef2, PDF_CMAP_SINGLE, 9358 },
	{ 0x8ef3, 0x8ef7, PDF_CMAP_RANGE, 19642 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 8857 },
	{ 0x8ef9, 0x8ef9, PDF_CMAP_SINGLE, 9361 },
	{ 0x8efa, 0x8efa, PDF_CMAP_SINGLE, 9366 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 9359 },
	{ 0x8efc, 0x8efc, PDF_CMAP_SINGLE, 9362 },
	{ 0x8efd, 0x8efd, PDF_CMAP_SINGLE, 19647 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 9367 },
	{ 0x8eff, 0x8eff, PDF_CMAP_SINGLE, 19648 },
	{ 0x8f00, 0x8f02, PDF_CMAP_RANGE, 19649 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 8113 },
	{ 0x8f04, 0x8f04, PDF_CMAP_SINGLE, 19652 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 9370 },
	{ 0x8f06, 0x8f06, PDF_CMAP_SINGLE, 19653 },
	{ 0x8f07, 0x8f07, PDF_CMAP_SINGLE, 9369 },
	{ 0x8f08, 0x8f08, PDF_CMAP_SINGLE, 19654 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 8792 },
	{ 0x8f0a, 0x8f0a, PDF_CMAP_SINGLE, 9368 },
	{ 0x8f0b, 0x8f11, PDF_CMAP_RANGE, 19655 },
	{ 0x8f12, 0x8f12, PDF_CMAP_SINGLE, 9371 },
	{ 0x8f13, 0x8f13, PDF_CMAP_SINGLE, 19662 },
	{ 0x8f14, 0x8f14, PDF_CMAP_SINGLE, 7945 },
	{ 0x8f15, 0x8f15, PDF_CMAP_SINGLE, 8422 },
	{ 0x8f16, 0x8f1a, PDF_CMAP_RANGE, 19663 },
	{ 0x8f1b, 0x8f1b, PDF_CMAP_SINGLE, 8230 },
	{ 0x8f1c, 0x8f1c, PDF_CMAP_SINGLE, 9375 },
	{ 0x8f1d, 0x8f1d, PDF_CMAP_SINGLE, 8025 },
	{ 0x8f1e, 0x8f1f, PDF_CMAP_RANGE, 9373 },
	{ 0x8f20, 0x8f24, PDF_CMAP_RANGE, 19668 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 7995 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 9372 },
	{ 0x8f27, 0x8f28, PDF_CMAP_RANGE, 19673 },
	{ 0x8f29, 0x8f29, PDF_CMAP_SINGLE, 7738 },
	{ 0x8f2a, 0x8f2a, PDF_CMAP_SINGLE, 8283 },
	{ 0x8f2b, 0x8f2e, PDF_CMAP_RANGE, 19675 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 8048 },
	{ 0x8f30, 0x8f32, PDF_CMAP_RANGE, 19679 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 9376 },
	{ 0x8f34, 0x8f37, PDF_CMAP_RANGE, 19682 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 8507 },
	{ 0x8f39, 0x8f3a, PDF_CMAP_RANGE, 19686 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 7943 },
	{ 0x8f3c, 0x8f3d, PDF_CMAP_RANGE, 19688 },
	{ 0x8f3e, 0x8f3e, PDF_CMAP_SINGLE, 8816 },
	{ 0x8f3f, 0x8f3f, PDF_CMAP_SINGLE, 8759 },
	{ 0x8f40, 0x8f41, PDF_CMAP_RANGE, 19690 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 9426 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 19692 },
	{ 0x8f44, 0x8f44, PDF_CMAP_SINGLE, 8627 },
	{ 0x8f45, 0x8f45, PDF_CMAP_SINGLE, 8773 },
	{ 0x8f46, 0x8f46, PDF_CMAP_SINGLE, 9377 },
	{ 0x8f47, 0x8f48, PDF_CMAP_RANGE, 19693 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 8872 },
	{ 0x8f4a, 0x8f4c, PDF_CMAP_RANGE, 19695 },
	{ 0x8f4d, 0x8f4d, PDF_CMAP_SINGLE, 8828 },
	{ 0x8f4e, 0x8f4e, PDF_CMAP_SINGLE, 8112 },
	{ 0x8f4f, 0x8f53, PDF_CMAP_RANGE, 19698 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 9378 },
	{ 0x8f55, 0x8f5e, PDF_CMAP_RANGE, 19703 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 8006 },
	{ 0x8f60, 0x8f60, PDF_CMAP_SINGLE, 19713 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 9088 },
	{ 0x8f62, 0x8f62, PDF_CMAP_SINGLE, 9365 },
	{ 0x8f63, 0x8f63, PDF_CMAP_SINGLE, 19714 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 9360 },
	{ 0x8f65, 0x8f65, PDF_CMAP_SINGLE, 19715 },
	{ 0x8f66, 0x8f66, PDF_CMAP_SINGLE, 1242 },
	{ 0x8f67, 0x8f67, PDF_CMAP_SINGLE, 4417 },
	{ 0x8f68, 0x8f68, PDF_CMAP_SINGLE, 1861 },
	{ 0x8f69, 0x8f69, PDF_CMAP_SINGLE, 4036 },
	{ 0x8f6a, 0x8f6a, PDF_CMAP_SINGLE, 19716 },
	{ 0x8f6b, 0x8f6b, PDF_CMAP_SINGLE, 6369 },
	{ 0x8f6c, 0x8f6c, PDF_CMAP_SINGLE, 4615 },
	{ 0x8f6d, 0x8f6d, PDF_CMAP_SINGLE, 6370 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 2685 },
	{ 0x8f6f, 0x8f6f, PDF_CMAP_SINGLE, 3272 },
	{ 0x8f70, 0x8f70, PDF_CMAP_SINGLE, 1947 },
	{ 0x8f71, 0x8f73, PDF_CMAP_RANGE, 6371 },
	{ 0x8f74, 0x8f74, PDF_CMAP_SINGLE, 4576 },
	{ 0x8f75, 0x8f76, PDF_CMAP_RANGE, 6374 },
	{ 0x8f77, 0x8f77, PDF_CMAP_SINGLE, 6377 },
	{ 0x8f78, 0x8f78, PDF_CMAP_SINGLE, 6376 },
	{ 0x8f79, 0x8f7a, PDF_CMAP_RANGE, 6378 },
	{ 0x8f7b, 0x8f7b, PDF_CMAP_SINGLE, 3166 },
	{ 0x8f7c, 0x8f7c, PDF_CMAP_SINGLE, 6380 },
	{ 0x8f7d, 0x8f7d, PDF_CMAP_SINGLE, 4379 },
	{ 0x8f7e, 0x8f7e, PDF_CMAP_SINGLE, 6381 },
	{ 0x8f7f, 0x8f7f, PDF_CMAP_SINGLE, 2207 },
	{ 0x8f80, 0x8f80, PDF_CMAP_SINGLE, 19717 },
	{ 0x8f81, 0x8f82, PDF_CMAP_RANGE, 6382 },
	{ 0x8f83, 0x8f83, PDF_CMAP_SINGLE, 2208 },
	{ 0x8f84, 0x8f84, PDF_CMAP_SINGLE, 6384 },
	{ 0x8f85, 0x8f85, PDF_CMAP_SINGLE, 1699 },
	{ 0x8f86, 0x8f86, PDF_CMAP_SINGLE, 2567 },
	{ 0x8f87, 0x8f87, PDF_CMAP_SINGLE, 6385 },
	{ 0x8f88, 0x8f88, PDF_CMAP_SINGLE, 1051 },
	{ 0x8f89, 0x8f89, PDF_CMAP_SINGLE, 2025 },
	{ 0x8f8a, 0x8f8a, PDF_CMAP_SINGLE, 1870 },
	{ 0x8f8b, 0x8f8b, PDF_CMAP_SINGLE, 6386 },
	{ 0x8f8c, 0x8f8c, PDF_CMAP_SINGLE, 19718 },
	{ 0x8f8d, 0x8f8f, PDF_CMAP_RANGE, 6387 },
	{ 0x8f90, 0x8f90, PDF_CMAP_SINGLE, 1685 },
	{ 0x8f91, 0x8f91, PDF_CMAP_SINGLE, 2080 },
	{ 0x8f92, 0x8f92, PDF_CMAP_SINGLE, 19719 },
	{ 0x8f93, 0x8f93, PDF_CMAP_SINGLE, 3451 },
	{ 0x8f94, 0x8f94, PDF_CMAP_SINGLE, 5492 },
	{ 0x8f95, 0x8f95, PDF_CMAP_SINGLE, 4338 },
	{ 0x8f96, 0x8f96, PDF_CMAP_SINGLE, 3882 },
	{ 0x8f97, 0x8f97, PDF_CMAP_SINGLE, 4440 },
	{ 0x8f98, 0x8f98, PDF_CMAP_SINGLE, 6390 },
	{ 0x8f99, 0x8f99, PDF_CMAP_SINGLE, 4479 },
	{ 0x8f9a, 0x8f9a, PDF_CMAP_SINGLE, 6391 },
	{ 0x8f9b, 0x8f9b, PDF_CMAP_SINGLE, 3980 },
	{ 0x8f9c, 0x8f9c, PDF_CMAP_SINGLE, 1813 },
	{ 0x8f9d, 0x8f9d, PDF_CMAP_SINGLE, 19720 },
	{ 0x8f9e, 0x8f9e, PDF_CMAP_SINGLE, 1354 },
	{ 0x8f9f, 0x8f9f, PDF_CMAP_SINGLE, 1090 },
	{ 0x8fa0, 0x8fa2, PDF_CMAP_RANGE, 19721 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 2460 },
	{ 0x8fa4, 0x8fa5, PDF_CMAP_RANGE, 19724 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 7728 },
	{ 0x8fa7, 0x8fa7, PDF_CMAP_SINGLE, 19726 },
	{ 0x8fa8, 0x8fa9, PDF_CMAP_RANGE, 1103 },
	{ 0x8faa, 0x8faa, PDF_CMAP_SINGLE, 19727 },
	{ 0x8fab, 0x8fab, PDF_CMAP_SINGLE, 1105 },
	{ 0x8fac, 0x8fac, PDF_CMAP_SINGLE, 19728 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 7837 },
	{ 0x8fae, 0x8fae, PDF_CMAP_SINGLE, 7755 },
	{ 0x8faf, 0x8faf, PDF_CMAP_SINGLE, 7754 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 1250 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 3267 },
	{ 0x8fb2, 0x8fb2, PDF_CMAP_SINGLE, 8362 },
	{ 0x8fb3, 0x8fb5, PDF_CMAP_RANGE, 19729 },
	{ 0x8fb6, 0x8fb6, PDF_CMAP_SINGLE, 5949 },
	{ 0x8fb7, 0x8fb8, PDF_CMAP_RANGE, 19732 },
	{ 0x8fb9, 0x8fb9, PDF_CMAP_SINGLE, 1096 },
	{ 0x8fba, 0x8fbc, PDF_CMAP_RANGE, 19734 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 2578 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 1394 },
	{ 0x8fbf, 0x8fc0, PDF_CMAP_RANGE, 19737 },
	{ 0x8fc1, 0x8fc1, PDF_CMAP_SINGLE, 3109 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 4285 },
	{ 0x8fc3, 0x8fc3, PDF_CMAP_SINGLE, 19739 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 3096 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 4065 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 19740 },
	{ 0x8fc7, 0x8fc7, PDF_CMAP_SINGLE, 1878 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 2716 },
	{ 0x8fc9, 0x8fcd, PDF_CMAP_RANGE, 19741 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 4242 },
	{ 0x8fcf, 0x8fcf, PDF_CMAP_SINGLE, 19746 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 4366 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 2253 },
	{ 0x8fd2, 0x8fd2, PDF_CMAP_SINGLE, 19747 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 5950 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 1618 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 5951 },
	{ 0x8fd6, 0x8fd7, PDF_CMAP_RANGE, 19748 },
	{ 0x8fd8, 0x8fd8, PDF_CMAP_SINGLE, 1998 },
	{ 0x8fd9, 0x8fd9, PDF_CMAP_SINGLE, 4483 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 19750 },
	{ 0x8fdb, 0x8fdb, PDF_CMAP_SINGLE, 2249 },
	{ 0x8fdc, 0x8fdc, PDF_CMAP_SINGLE, 4345 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 3764 },
	{ 0x8fde, 0x8fde, PDF_CMAP_SINGLE, 2549 },
	{ 0x8fdf, 0x8fdf, PDF_CMAP_SINGLE, 1278 },
	{ 0x8fe0, 0x8fe1, PDF_CMAP_RANGE, 19751 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 3657 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 19753 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 5954 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 5952 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 5956 },
	{ 0x8fe7, 0x8fe7, PDF_CMAP_SINGLE, 19754 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 5958 },
	{ 0x8fe9, 0x8fe9, PDF_CMAP_SINGLE, 5955 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 1456 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 3047 },
	{ 0x8fec, 0x8fec, PDF_CMAP_SINGLE, 19755 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 1501 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 5953 },
	{ 0x8fef, 0x8fef, PDF_CMAP_SINGLE, 19756 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 3469 },
	{ 0x8ff1, 0x8ff2, PDF_CMAP_RANGE, 19757 },
	{ 0x8ff3, 0x8ff3, PDF_CMAP_SINGLE, 5957 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 9851 },
	{ 0x8ff5, 0x8ff6, PDF_CMAP_RANGE, 19759 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 2777 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 1070 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 2070 },
	{ 0x8ffa, 0x8ffc, PDF_CMAP_RANGE, 19761 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 4628 },
	{ 0x8ffe, 0x8fff, PDF_CMAP_RANGE, 19764 },
	{ 0x9000, 0x9000, PDF_CMAP_SINGLE, 3708 },
	{ 0x9001, 0x9001, PDF_CMAP_SINGLE, 3522 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 3425 },
	{ 0x9003, 0x9003, PDF_CMAP_SINGLE, 3622 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 5960 },
	{ 0x9005, 0x9005, PDF_CMAP_SINGLE, 5959 },
	{ 0x9006, 0x9006, PDF_CMAP_SINGLE, 2889 },
	{ 0x9007, 0x9008, PDF_CMAP_RANGE, 19766 },
	{ 0x9009, 0x9009, PDF_CMAP_SINGLE, 4042 },
	{ 0x900a, 0x900a, PDF_CMAP_SINGLE, 4064 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 5961 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 19768 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 5964 },
	{ 0x900e, 0x900e, PDF_CMAP_SINGLE, 19769 },
	{ 0x900f, 0x900f, PDF_CMAP_SINGLE, 3689 },
	{ 0x9010, 0x9010, PDF_CMAP_SINGLE, 4591 },
	{ 0x9011, 0x9011, PDF_CMAP_SINGLE, 5963 },
	{ 0x9012, 0x9012, PDF_CMAP_SINGLE, 1470 },
	{ 0x9013, 0x9013, PDF_CMAP_SINGLE, 19770 },
	{ 0x9014, 0x9014, PDF_CMAP_SINGLE, 3695 },
	{ 0x9015, 0x9015, PDF_CMAP_SINGLE, 9202 },
	{ 0x9016, 0x9016, PDF_CMAP_SINGLE, 5965 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 1529 },
	{ 0x9018, 0x9018, PDF_CMAP_SINGLE, 19771 },
	{ 0x9019, 0x9019, PDF_CMAP_SINGLE, 8830 },
	{ 0x901a, 0x901a, PDF_CMAP_SINGLE, 3673 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 1853 },
	{ 0x901c, 0x901c, PDF_CMAP_SINGLE, 19772 },
	{ 0x901d, 0x901d, PDF_CMAP_SINGLE, 3420 },
	{ 0x901e, 0x901e, PDF_CMAP_SINGLE, 1270 },
	{ 0x901f, 0x901f, PDF_CMAP_SINGLE, 3534 },
	{ 0x9020, 0x9020, PDF_CMAP_SINGLE, 4399 },
	{ 0x9021, 0x9021, PDF_CMAP_SINGLE, 5966 },
	{ 0x9022, 0x9022, PDF_CMAP_SINGLE, 1671 },
	{ 0x9023, 0x9023, PDF_CMAP_SINGLE, 8217 },
	{ 0x9024, 0x9025, PDF_CMAP_RANGE, 19773 },
	{ 0x9026, 0x9026, PDF_CMAP_SINGLE, 5962 },
	{ 0x9027, 0x902c, PDF_CMAP_RANGE, 19775 },
	{ 0x902d, 0x902d, PDF_CMAP_SINGLE, 5969 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 1409 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 5970 },
	{ 0x9030, 0x9031, PDF_CMAP_RANGE, 19781 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 8123 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 19783 },
	{ 0x9035, 0x9036, PDF_CMAP_RANGE, 5967 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 19785 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 4196 },
	{ 0x9039, 0x903a, PDF_CMAP_RANGE, 19786 },
	{ 0x903b, 0x903b, PDF_CMAP_SINGLE, 2694 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 1071 },
	{ 0x903d, 0x903d, PDF_CMAP_SINGLE, 19788 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 4295 },
	{ 0x903f, 0x9040, PDF_CMAP_RANGE, 19789 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 1564 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 3553 },
	{ 0x9043, 0x9043, PDF_CMAP_SINGLE, 19791 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 5971 },
	{ 0x9045, 0x9046, PDF_CMAP_RANGE, 19792 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 4315 },
	{ 0x9048, 0x904a, PDF_CMAP_RANGE, 19794 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 8787 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 19797 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 1106 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 7998 },
	{ 0x904f, 0x904f, PDF_CMAP_SINGLE, 1587 },
	{ 0x9050, 0x9050, PDF_CMAP_SINGLE, 5974 },
	{ 0x9051, 0x9052, PDF_CMAP_RANGE, 5972 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 1441 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 7846 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 8590 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 19798 },
	{ 0x9057, 0x9057, PDF_CMAP_SINGLE, 4172 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 5976 },
	{ 0x9059, 0x905a, PDF_CMAP_RANGE, 19799 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 5978 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 8684 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 19801 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 7870 },
	{ 0x905f, 0x905f, PDF_CMAP_SINGLE, 19802 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 8778 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 19803 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 5977 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 3119 },
	{ 0x9064, 0x9064, PDF_CMAP_SINGLE, 19804 },
	{ 0x9065, 0x9065, PDF_CMAP_SINGLE, 4138 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 19805 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 5975 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 8499 },
	{ 0x906a, 0x906c, PDF_CMAP_RANGE, 19807 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 4389 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 4475 },
	{ 0x906f, 0x9071, PDF_CMAP_RANGE, 19810 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 7812 },
	{ 0x9073, 0x9073, PDF_CMAP_SINGLE, 19813 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 5980 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 4686 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 19814 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 8399 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 8674 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 19815 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 8719 },
	{ 0x907b, 0x907b, PDF_CMAP_SINGLE, 19816 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 8233 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 5981 },
	{ 0x907e, 0x907e, PDF_CMAP_SINGLE, 19817 },
	{ 0x907f, 0x907f, PDF_CMAP_SINGLE, 1093 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 4132 },
	{ 0x9081, 0x9081, PDF_CMAP_SINGLE, 8307 },
	{ 0x9082, 0x9082, PDF_CMAP_SINGLE, 5982 },
	{ 0x9083, 0x9083, PDF_CMAP_SINGLE, 5984 },
	{ 0x9084, 0x9084, PDF_CMAP_SINGLE, 8021 },
	{ 0x9085, 0x9086, PDF_CMAP_RANGE, 19818 },
	{ 0x9087, 0x9087, PDF_CMAP_SINGLE, 9201 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 5983 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 19820 },
	{ 0x908a, 0x908a, PDF_CMAP_SINGLE, 7750 },
	{ 0x908b, 0x908b, PDF_CMAP_SINGLE, 5985 },
	{ 0x908c, 0x908e, PDF_CMAP_RANGE, 19821 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 8291 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 9203 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 4191 },
	{ 0x9092, 0x9092, PDF_CMAP_SINGLE, 19824 },
	{ 0x9093, 0x9093, PDF_CMAP_SINGLE, 1452 },
	{ 0x9094, 0x9094, PDF_CMAP_SINGLE, 19825 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 6167 },
	{ 0x9096, 0x9096, PDF_CMAP_SINGLE, 19826 },
	{ 0x9097, 0x9097, PDF_CMAP_SINGLE, 4967 },
	{ 0x9098, 0x9098, PDF_CMAP_SINGLE, 19827 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 4970 },
	{ 0x909a, 0x909a, PDF_CMAP_SINGLE, 19828 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 4968 },
	{ 0x909c, 0x909c, PDF_CMAP_SINGLE, 19829 },
	{ 0x909d, 0x909d, PDF_CMAP_SINGLE, 4969 },
	{ 0x909e, 0x90a0, PDF_CMAP_RANGE, 19830 },
	{ 0x90a1, 0x90a1, PDF_CMAP_SINGLE, 4972 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 3994 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 2858 },
	{ 0x90a4, 0x90a5, PDF_CMAP_RANGE, 19833 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 1017 },
	{ 0x90a7, 0x90a9, PDF_CMAP_RANGE, 19835 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 3963 },
	{ 0x90ab, 0x90ab, PDF_CMAP_SINGLE, 19838 },
	{ 0x90ac, 0x90ac, PDF_CMAP_SINGLE, 4971 },
	{ 0x90ad, 0x90ad, PDF_CMAP_SINGLE, 19839 },
	{ 0x90ae, 0x90ae, PDF_CMAP_SINGLE, 4271 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 1889 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 4978 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 3182 },
	{ 0x90b2, 0x90b2, PDF_CMAP_SINGLE, 19840 },
	{ 0x90b3, 0x90b3, PDF_CMAP_SINGLE, 4974 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 4973 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 3348 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 4975 },
	{ 0x90b7, 0x90b7, PDF_CMAP_SINGLE, 19841 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 4977 },
	{ 0x90b9, 0x90b9, PDF_CMAP_SINGLE, 4667 },
	{ 0x90ba, 0x90ba, PDF_CMAP_SINGLE, 4976 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 2595 },
	{ 0x90bc, 0x90bd, PDF_CMAP_RANGE, 19842 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 4981 },
	{ 0x90bf, 0x90c0, PDF_CMAP_RANGE, 19844 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 4313 },
	{ 0x90c2, 0x90c3, PDF_CMAP_RANGE, 19846 },
	{ 0x90c4, 0x90c4, PDF_CMAP_SINGLE, 4983 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 4980 },
	{ 0x90c6, 0x90c6, PDF_CMAP_SINGLE, 19848 },
	{ 0x90c7, 0x90c7, PDF_CMAP_SINGLE, 4984 },
	{ 0x90c8, 0x90c9, PDF_CMAP_RANGE, 19849 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2189 },
	{ 0x90cb, 0x90cd, PDF_CMAP_RANGE, 19851 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 2484 },
	{ 0x90cf, 0x90cf, PDF_CMAP_SINGLE, 4979 },
	{ 0x90d0, 0x90d0, PDF_CMAP_SINGLE, 4982 },
	{ 0x90d1, 0x90d1, PDF_CMAP_SINGLE, 4514 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 19854 },
	{ 0x90d3, 0x90d3, PDF_CMAP_SINGLE, 4985 },
	{ 0x90d4, 0x90d6, PDF_CMAP_RANGE, 19855 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 4989 },
	{ 0x90d8, 0x90da, PDF_CMAP_RANGE, 19858 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 4990 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 4988 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 1913 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 19861 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 8990 },
	{ 0x90e0, 0x90e0, PDF_CMAP_SINGLE, 19862 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 2353 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 4987 },
	{ 0x90e3, 0x90e5, PDF_CMAP_RANGE, 19863 },
	{ 0x90e6, 0x90e6, PDF_CMAP_SINGLE, 4986 },
	{ 0x90e7, 0x90e7, PDF_CMAP_SINGLE, 4362 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 1158 },
	{ 0x90e9, 0x90ea, PDF_CMAP_RANGE, 19866 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 4991 },
	{ 0x90ec, 0x90ec, PDF_CMAP_SINGLE, 19868 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 1874 },
	{ 0x90ee, 0x90ee, PDF_CMAP_SINGLE, 19869 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 4992 },
	{ 0x90f0, 0x90f3, PDF_CMAP_RANGE, 19870 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 1248 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 8755 },
	{ 0x90f6, 0x90f7, PDF_CMAP_RANGE, 19874 },
	{ 0x90f8, 0x90f8, PDF_CMAP_SINGLE, 1415 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 19876 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 1531 },
	{ 0x90fe, 0x90fe, PDF_CMAP_SINGLE, 4993 },
	{ 0x90ff, 0x90ff, PDF_CMAP_SINGLE, 19880 },
	{ 0x9100, 0x9101, PDF_CMAP_RANGE, 19881 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 1588 },
	{ 0x9103, 0x9103, PDF_CMAP_SINGLE, 19883 },
	{ 0x9104, 0x9104, PDF_CMAP_SINGLE, 4994 },
	{ 0x9105, 0x9105, PDF_CMAP_SINGLE, 19884 },
	{ 0x9106, 0x9106, PDF_CMAP_SINGLE, 8992 },
	{ 0x9107, 0x9108, PDF_CMAP_RANGE, 19885 },
	{ 0x9109, 0x9109, PDF_CMAP_SINGLE, 8647 },
	{ 0x910a, 0x9111, PDF_CMAP_RANGE, 19887 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 8892 },
	{ 0x9113, 0x9113, PDF_CMAP_SINGLE, 19895 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 8988 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 19896 },
	{ 0x9116, 0x9116, PDF_CMAP_SINGLE, 8785 },
	{ 0x9117, 0x9118, PDF_CMAP_RANGE, 19897 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 1074 },
	{ 0x911a, 0x911d, PDF_CMAP_RANGE, 19899 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 4996 },
	{ 0x911f, 0x9121, PDF_CMAP_RANGE, 19903 },
	{ 0x9122, 0x9122, PDF_CMAP_SINGLE, 4995 },
	{ 0x9123, 0x9123, PDF_CMAP_SINGLE, 4997 },
	{ 0x9124, 0x9126, PDF_CMAP_RANGE, 19906 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 7867 },
	{ 0x9128, 0x912c, PDF_CMAP_RANGE, 19909 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 8839 },
	{ 0x912e, 0x912e, PDF_CMAP_SINGLE, 19914 },
	{ 0x912f, 0x912f, PDF_CMAP_SINGLE, 4999 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 8237 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 4998 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 7851 },
	{ 0x9133, 0x9133, PDF_CMAP_SINGLE, 19915 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 8989 },
	{ 0x9135, 0x9135, PDF_CMAP_SINGLE, 19916 },
	{ 0x9136, 0x9136, PDF_CMAP_SINGLE, 8991 },
	{ 0x9137, 0x9138, PDF_CMAP_RANGE, 19917 },
	{ 0x9139, 0x9139, PDF_CMAP_SINGLE, 5000 },
	{ 0x913a, 0x913a, PDF_CMAP_SINGLE, 8987 },
	{ 0x913b, 0x9142, PDF_CMAP_RANGE, 19919 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 5001 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 19927 },
	{ 0x9146, 0x9146, PDF_CMAP_SINGLE, 5002 },
	{ 0x9147, 0x9147, PDF_CMAP_SINGLE, 19929 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 8993 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 4276 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 7416 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 3186 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 4640 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 2981 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7418 },
	{ 0x9150, 0x9150, PDF_CMAP_SINGLE, 7417 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 19930 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2293 },
	{ 0x9153, 0x9156, PDF_CMAP_RANGE, 19931 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 4026 },
	{ 0x9158, 0x9159, PDF_CMAP_RANGE, 19935 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 1648 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 19937 },
	{ 0x915d, 0x915d, PDF_CMAP_SINGLE, 4368 },
	{ 0x915e, 0x915e, PDF_CMAP_SINGLE, 3581 },
	{ 0x915f, 0x9160, PDF_CMAP_RANGE, 19939 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 7422 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 7421 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 1887 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 7420 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 3531 },
	{ 0x9166, 0x9168, PDF_CMAP_RANGE, 19941 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 7424 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 2493 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 19944 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 1295 },
	{ 0x916d, 0x916d, PDF_CMAP_SINGLE, 19945 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 3675 },
	{ 0x916f, 0x916f, PDF_CMAP_SINGLE, 7425 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 7423 },
	{ 0x9171, 0x9171, PDF_CMAP_SINGLE, 2181 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 7428 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 19946 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 7429 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 2206 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 2749 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 2414 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 3542 },
	{ 0x9179, 0x9179, PDF_CMAP_SINGLE, 7430 },
	{ 0x917a, 0x917c, PDF_CMAP_RANGE, 19947 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 7426 },
	{ 0x917f, 0x917f, PDF_CMAP_SINGLE, 2899 },
	{ 0x9180, 0x9184, PDF_CMAP_RANGE, 19950 },
	{ 0x9185, 0x9185, PDF_CMAP_SINGLE, 7432 },
	{ 0x9186, 0x9186, PDF_CMAP_SINGLE, 19955 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 1343 },
	{ 0x9188, 0x9188, PDF_CMAP_SINGLE, 19956 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 4682 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 19957 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 1370 },
	{ 0x918c, 0x918c, PDF_CMAP_SINGLE, 7431 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 7434 },
	{ 0x918e, 0x918f, PDF_CMAP_RANGE, 19958 },
	{ 0x9190, 0x9190, PDF_CMAP_SINGLE, 7433 },
	{ 0x9191, 0x9191, PDF_CMAP_SINGLE, 7435 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 3996 },
	{ 0x9193, 0x9195, PDF_CMAP_RANGE, 19960 },
	{ 0x9196, 0x9196, PDF_CMAP_SINGLE, 8789 },
	{ 0x9197, 0x9199, PDF_CMAP_RANGE, 19963 },
	{ 0x919a, 0x919a, PDF_CMAP_SINGLE, 2774 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 3204 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 7823 },
	{ 0x919d, 0x91a1, PDF_CMAP_RANGE, 19966 },
	{ 0x91a2, 0x91a3, PDF_CMAP_RANGE, 7436 },
	{ 0x91a4, 0x91a9, PDF_CMAP_RANGE, 19971 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 7438 },
	{ 0x91ab, 0x91ab, PDF_CMAP_SINGLE, 8716 },
	{ 0x91ac, 0x91ac, PDF_CMAP_SINGLE, 8100 },
	{ 0x91ad, 0x91af, PDF_CMAP_RANGE, 7439 },
	{ 0x91b0, 0x91b3, PDF_CMAP_RANGE, 19977 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 7443 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 7442 },
	{ 0x91b6, 0x91b9, PDF_CMAP_RANGE, 19981 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 7444 },
	{ 0x91bb, 0x91bf, PDF_CMAP_RANGE, 19985 },
	{ 0x91c0, 0x91c0, PDF_CMAP_SINGLE, 8347 },
	{ 0x91c1, 0x91c1, PDF_CMAP_SINGLE, 8664 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 19990 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 9722 },
	{ 0x91c4, 0x91c4, PDF_CMAP_SINGLE, 19991 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 9721 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 19992 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 1168 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 19993 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 4281 },
	{ 0x91ca, 0x91ca, PDF_CMAP_SINGLE, 3428 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 8500 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 2522 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 4567 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 4151 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 2568 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 19994 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 2241 },
	{ 0x91d2, 0x91d4, PDF_CMAP_RANGE, 9459 },
	{ 0x91d5, 0x91d5, PDF_CMAP_SINGLE, 9464 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 19995 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 9463 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 7880 },
	{ 0x91d9, 0x91d9, PDF_CMAP_SINGLE, 9462 },
	{ 0x91da, 0x91db, PDF_CMAP_RANGE, 19996 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 1701 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 8832 },
	{ 0x91de, 0x91e2, PDF_CMAP_RANGE, 19998 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 7877 },
	{ 0x91e4, 0x91e4, PDF_CMAP_SINGLE, 9467 },
	{ 0x91e5, 0x91e6, PDF_CMAP_RANGE, 20003 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 9466 },
	{ 0x91e8, 0x91e8, PDF_CMAP_SINGLE, 20005 },
	{ 0x91e9, 0x91e9, PDF_CMAP_SINGLE, 7917 },
	{ 0x91ea, 0x91f4, PDF_CMAP_RANGE, 20006 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 9469 },
	{ 0x91f6, 0x91f6, PDF_CMAP_SINGLE, 20017 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 9465 },
	{ 0x91f8, 0x91f8, PDF_CMAP_SINGLE, 20018 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 9470 },
	{ 0x91fa, 0x91fa, PDF_CMAP_SINGLE, 8397 },
	{ 0x91fb, 0x91ff, PDF_CMAP_RANGE, 20019 },
	{ 0x9200, 0x9200, PDF_CMAP_SINGLE, 9480 },
	{ 0x9201, 0x9201, PDF_CMAP_SINGLE, 9476 },
	{ 0x9202, 0x9203, PDF_CMAP_RANGE, 20024 },
	{ 0x9204, 0x9204, PDF_CMAP_SINGLE, 9478 },
	{ 0x9205, 0x9207, PDF_CMAP_RANGE, 20026 },
	{ 0x9208, 0x9208, PDF_CMAP_SINGLE, 9471 },
	{ 0x9209, 0x9209, PDF_CMAP_SINGLE, 8336 },
	{ 0x920a, 0x920c, PDF_CMAP_RANGE, 20029 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 7901 },
	{ 0x920e, 0x920e, PDF_CMAP_SINGLE, 7973 },
	{ 0x920f, 0x920f, PDF_CMAP_SINGLE, 20032 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 9475 },
	{ 0x9211, 0x9211, PDF_CMAP_SINGLE, 9474 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 20033 },
	{ 0x9214, 0x9214, PDF_CMAP_SINGLE, 7802 },
	{ 0x9215, 0x9215, PDF_CMAP_SINGLE, 8358 },
	{ 0x9216, 0x921d, PDF_CMAP_RANGE, 20035 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 8149 },
	{ 0x921f, 0x9222, PDF_CMAP_RANGE, 20043 },
	{ 0x9223, 0x9223, PDF_CMAP_SINGLE, 7953 },
	{ 0x9224, 0x9224, PDF_CMAP_SINGLE, 20047 },
	{ 0x9225, 0x9225, PDF_CMAP_SINGLE, 9479 },
	{ 0x9226, 0x9226, PDF_CMAP_SINGLE, 9472 },
	{ 0x9227, 0x9227, PDF_CMAP_SINGLE, 9477 },
	{ 0x9228, 0x922d, PDF_CMAP_RANGE, 20048 },
	{ 0x922e, 0x922e, PDF_CMAP_SINGLE, 9497 },
	{ 0x922f, 0x922f, PDF_CMAP_SINGLE, 20054 },
	{ 0x9230, 0x9230, PDF_CMAP_SINGLE, 9493 },
	{ 0x9231, 0x9232, PDF_CMAP_RANGE, 20055 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 9484 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 8241 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 20057 },
	{ 0x9237, 0x9237, PDF_CMAP_SINGLE, 9483 },
	{ 0x9238, 0x9238, PDF_CMAP_SINGLE, 9487 },
	{ 0x9239, 0x9239, PDF_CMAP_SINGLE, 9498 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 9481 },
	{ 0x923b, 0x923c, PDF_CMAP_RANGE, 20059 },
	{ 0x923d, 0x923d, PDF_CMAP_SINGLE, 9486 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 8756 },
	{ 0x923f, 0x923f, PDF_CMAP_SINGLE, 9491 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 8064 },
	{ 0x9241, 0x9244, PDF_CMAP_RANGE, 20061 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 9473 },
	{ 0x9246, 0x9247, PDF_CMAP_RANGE, 20065 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 9495 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 9494 },
	{ 0x924a, 0x924c, PDF_CMAP_RANGE, 20067 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 9496 },
	{ 0x924e, 0x9250, PDF_CMAP_RANGE, 20070 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 7766 },
	{ 0x9252, 0x9254, PDF_CMAP_RANGE, 20073 },
	{ 0x9255, 0x9255, PDF_CMAP_SINGLE, 9485 },
	{ 0x9256, 0x9256, PDF_CMAP_SINGLE, 20076 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 8403 },
	{ 0x9258, 0x9259, PDF_CMAP_RANGE, 20077 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 8314 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 8398 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 20079 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 9488 },
	{ 0x925f, 0x9261, PDF_CMAP_RANGE, 20081 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 7765 },
	{ 0x9263, 0x9265, PDF_CMAP_RANGE, 20084 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 9482 },
	{ 0x9267, 0x926b, PDF_CMAP_RANGE, 20087 },
	{ 0x926c, 0x926d, PDF_CMAP_RANGE, 9489 },
	{ 0x926e, 0x9273, PDF_CMAP_RANGE, 20092 },
	{ 0x9274, 0x9274, PDF_CMAP_SINGLE, 2155 },
	{ 0x9275, 0x9277, PDF_CMAP_RANGE, 20098 },
	{ 0x9278, 0x9278, PDF_CMAP_SINGLE, 8106 },
	{ 0x9279, 0x9279, PDF_CMAP_SINGLE, 20101 },
	{ 0x927a, 0x927a, PDF_CMAP_SINGLE, 9502 },
	{ 0x927b, 0x927b, PDF_CMAP_SINGLE, 7967 },
	{ 0x927c, 0x927e, PDF_CMAP_RANGE, 20102 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 9517 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 8733 },
	{ 0x9281, 0x9282, PDF_CMAP_RANGE, 20105 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 9522 },
	{ 0x9284, 0x9284, PDF_CMAP_SINGLE, 20107 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 8571 },
	{ 0x9286, 0x928d, PDF_CMAP_RANGE, 20108 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 7552 },
	{ 0x928f, 0x9290, PDF_CMAP_RANGE, 20116 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 8623 },
	{ 0x9292, 0x9292, PDF_CMAP_SINGLE, 20118 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 9516 },
	{ 0x9294, 0x9295, PDF_CMAP_RANGE, 20119 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 9512 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 20121 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 8332 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 20122 },
	{ 0x929a, 0x929a, PDF_CMAP_SINGLE, 9519 },
	{ 0x929b, 0x929b, PDF_CMAP_SINGLE, 20123 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 8636 },
	{ 0x929d, 0x929f, PDF_CMAP_RANGE, 20124 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 9501 },
	{ 0x92a1, 0x92a2, PDF_CMAP_RANGE, 20127 },
	{ 0x92a3, 0x92a3, PDF_CMAP_SINGLE, 9525 },
	{ 0x92a4, 0x92a4, PDF_CMAP_SINGLE, 20129 },
	{ 0x92a5, 0x92a5, PDF_CMAP_SINGLE, 8717 },
	{ 0x92a6, 0x92a6, PDF_CMAP_SINGLE, 9510 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 20130 },
	{ 0x92a8, 0x92a8, PDF_CMAP_SINGLE, 9524 },
	{ 0x92a9, 0x92a9, PDF_CMAP_SINGLE, 9514 },
	{ 0x92aa, 0x92aa, PDF_CMAP_SINGLE, 9503 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 9521 },
	{ 0x92ac, 0x92ac, PDF_CMAP_SINGLE, 9500 },
	{ 0x92ad, 0x92ad, PDF_CMAP_SINGLE, 20131 },
	{ 0x92ae, 0x92ae, PDF_CMAP_SINGLE, 7553 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 20132 },
	{ 0x92b1, 0x92b1, PDF_CMAP_SINGLE, 9509 },
	{ 0x92b2, 0x92b6, PDF_CMAP_RANGE, 20134 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 8653 },
	{ 0x92b8, 0x92b8, PDF_CMAP_SINGLE, 20139 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 8666 },
	{ 0x92ba, 0x92ba, PDF_CMAP_SINGLE, 20140 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 8562 },
	{ 0x92bc, 0x92bc, PDF_CMAP_SINGLE, 9534 },
	{ 0x92bd, 0x92c0, PDF_CMAP_RANGE, 20141 },
	{ 0x92c1, 0x92c1, PDF_CMAP_SINGLE, 8271 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 20145 },
	{ 0x92c3, 0x92c3, PDF_CMAP_SINGLE, 9539 },
	{ 0x92c4, 0x92c4, PDF_CMAP_SINGLE, 20146 },
	{ 0x92c5, 0x92c5, PDF_CMAP_SINGLE, 8663 },
	{ 0x92c6, 0x92c6, PDF_CMAP_SINGLE, 20147 },
	{ 0x92c7, 0x92c7, PDF_CMAP_SINGLE, 7740 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 7554 },
	{ 0x92c9, 0x92cb, PDF_CMAP_RANGE, 20148 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 9513 },
	{ 0x92cd, 0x92ce, PDF_CMAP_RANGE, 20151 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 9505 },
	{ 0x92d0, 0x92d1, PDF_CMAP_RANGE, 20153 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 7935 },
	{ 0x92d3, 0x92dc, PDF_CMAP_RANGE, 20155 },
	{ 0x92dd, 0x92dd, PDF_CMAP_SINGLE, 9535 },
	{ 0x92de, 0x92de, PDF_CMAP_SINGLE, 20165 },
	{ 0x92df, 0x92df, PDF_CMAP_SINGLE, 9540 },
	{ 0x92e0, 0x92e2, PDF_CMAP_RANGE, 20166 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 9507 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 7824 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 9530 },
	{ 0x92e6, 0x92e6, PDF_CMAP_SINGLE, 9541 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 20169 },
	{ 0x92e8, 0x92e8, PDF_CMAP_SINGLE, 9533 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 20170 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 8385 },
	{ 0x92eb, 0x92ec, PDF_CMAP_RANGE, 20171 },
	{ 0x92ed, 0x92ed, PDF_CMAP_SINGLE, 8451 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 9504 },
	{ 0x92ef, 0x92ef, PDF_CMAP_SINGLE, 9532 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 9531 },
	{ 0x92f1, 0x92f1, PDF_CMAP_SINGLE, 9528 },
	{ 0x92f2, 0x92f5, PDF_CMAP_RANGE, 20173 },
	{ 0x92f6, 0x92f6, PDF_CMAP_SINGLE, 9536 },
	{ 0x92f7, 0x92f7, PDF_CMAP_SINGLE, 20177 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 8141 },
	{ 0x92f9, 0x92fb, PDF_CMAP_RANGE, 20178 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 7960 },
	{ 0x92fd, 0x92ff, PDF_CMAP_RANGE, 20181 },
	{ 0x9300, 0x9300, PDF_CMAP_SINGLE, 20184 },
	{ 0x9301, 0x9301, PDF_CMAP_SINGLE, 9547 },
	{ 0x9302, 0x9305, PDF_CMAP_RANGE, 20185 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 9543 },
	{ 0x9307, 0x9308, PDF_CMAP_RANGE, 9551 },
	{ 0x9309, 0x930f, PDF_CMAP_RANGE, 20189 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 8880 },
	{ 0x9311, 0x9311, PDF_CMAP_SINGLE, 20196 },
	{ 0x9312, 0x9312, PDF_CMAP_SINGLE, 9542 },
	{ 0x9313, 0x9314, PDF_CMAP_RANGE, 20197 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 9548 },
	{ 0x9316, 0x9317, PDF_CMAP_RANGE, 20199 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 7834 },
	{ 0x9319, 0x9319, PDF_CMAP_SINGLE, 9554 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 9520 },
	{ 0x931b, 0x931b, PDF_CMAP_SINGLE, 9545 },
	{ 0x931c, 0x931e, PDF_CMAP_RANGE, 20201 },
	{ 0x931f, 0x931f, PDF_CMAP_SINGLE, 9553 },
	{ 0x9320, 0x9320, PDF_CMAP_SINGLE, 7882 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 20204 },
	{ 0x9322, 0x9322, PDF_CMAP_SINGLE, 8402 },
	{ 0x9323, 0x9325, PDF_CMAP_RANGE, 20205 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 8120 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 20208 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 8313 },
	{ 0x9329, 0x932a, PDF_CMAP_RANGE, 20209 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 8619 },
	{ 0x932c, 0x932d, PDF_CMAP_RANGE, 20211 },
	{ 0x932e, 0x932e, PDF_CMAP_SINGLE, 9549 },
	{ 0x932f, 0x932f, PDF_CMAP_SINGLE, 7845 },
	{ 0x9330, 0x9331, PDF_CMAP_RANGE, 20213 },
	{ 0x9332, 0x9332, PDF_CMAP_SINGLE, 8268 },
	{ 0x9333, 0x9333, PDF_CMAP_SINGLE, 8320 },
	{ 0x9334, 0x9335, PDF_CMAP_RANGE, 20215 },
	{ 0x9336, 0x9336, PDF_CMAP_SINGLE, 9837 },
	{ 0x9337, 0x9337, PDF_CMAP_SINGLE, 20217 },
	{ 0x9338, 0x9338, PDF_CMAP_SINGLE, 9527 },
	{ 0x9339, 0x933d, PDF_CMAP_RANGE, 20218 },
	{ 0x933e, 0x933e, PDF_CMAP_SINGLE, 7555 },
	{ 0x933f, 0x933f, PDF_CMAP_SINGLE, 20223 },
	{ 0x9340, 0x9340, PDF_CMAP_SINGLE, 9546 },
	{ 0x9341, 0x9341, PDF_CMAP_SINGLE, 8632 },
	{ 0x9342, 0x9342, PDF_CMAP_SINGLE, 20224 },
	{ 0x9343, 0x9343, PDF_CMAP_SINGLE, 9550 },
	{ 0x9344, 0x9345, PDF_CMAP_RANGE, 20225 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 9468 },
	{ 0x9347, 0x9347, PDF_CMAP_SINGLE, 9556 },
	{ 0x9348, 0x934a, PDF_CMAP_RANGE, 20227 },
	{ 0x934b, 0x934b, PDF_CMAP_SINGLE, 7996 },
	{ 0x934c, 0x934c, PDF_CMAP_SINGLE, 20230 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 7893 },
	{ 0x934e, 0x9353, PDF_CMAP_RANGE, 20231 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 9558 },
	{ 0x9355, 0x9357, PDF_CMAP_RANGE, 20237 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 8808 },
	{ 0x9359, 0x935a, PDF_CMAP_RANGE, 20240 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 7894 },
	{ 0x935c, 0x9363, PDF_CMAP_RANGE, 20242 },
	{ 0x9364, 0x9364, PDF_CMAP_SINGLE, 9559 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 9555 },
	{ 0x9366, 0x9368, PDF_CMAP_RANGE, 20250 },
	{ 0x9369, 0x9369, PDF_CMAP_SINGLE, 9544 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 7556 },
	{ 0x936b, 0x936b, PDF_CMAP_SINGLE, 20253 },
	{ 0x936c, 0x936c, PDF_CMAP_SINGLE, 8412 },
	{ 0x936d, 0x936f, PDF_CMAP_RANGE, 20254 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 9561 },
	{ 0x9371, 0x9374, PDF_CMAP_RANGE, 20257 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 8087 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 9557 },
	{ 0x9377, 0x9379, PDF_CMAP_RANGE, 20261 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 8829 },
	{ 0x937b, 0x937d, PDF_CMAP_RANGE, 20264 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 9598 },
	{ 0x937f, 0x9381, PDF_CMAP_RANGE, 20267 },
	{ 0x9382, 0x9382, PDF_CMAP_SINGLE, 8316 },
	{ 0x9383, 0x9383, PDF_CMAP_SINGLE, 20270 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 9562 },
	{ 0x9385, 0x9386, PDF_CMAP_RANGE, 20271 },
	{ 0x9387, 0x9387, PDF_CMAP_SINGLE, 9566 },
	{ 0x9388, 0x9389, PDF_CMAP_RANGE, 20273 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 7732 },
	{ 0x938b, 0x938e, PDF_CMAP_RANGE, 20275 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 7558 },
	{ 0x9390, 0x9395, PDF_CMAP_RANGE, 20279 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 8542 },
	{ 0x9397, 0x9397, PDF_CMAP_SINGLE, 20285 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 9568 },
	{ 0x9399, 0x93a1, PDF_CMAP_RANGE, 20286 },
	{ 0x93a2, 0x93a2, PDF_CMAP_SINGLE, 8610 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 9044 },
	{ 0x93a4, 0x93a5, PDF_CMAP_RANGE, 20295 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 9571 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 9511 },
	{ 0x93a8, 0x93a8, PDF_CMAP_SINGLE, 20297 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 9518 },
	{ 0x93aa, 0x93aa, PDF_CMAP_SINGLE, 9560 },
	{ 0x93ab, 0x93ab, PDF_CMAP_SINGLE, 20298 },
	{ 0x93ac, 0x93ac, PDF_CMAP_SINGLE, 7963 },
	{ 0x93ad, 0x93ad, PDF_CMAP_SINGLE, 20299 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 8835 },
	{ 0x93af, 0x93af, PDF_CMAP_SINGLE, 20300 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 9572 },
	{ 0x93b1, 0x93b2, PDF_CMAP_RANGE, 20301 },
	{ 0x93b3, 0x93b3, PDF_CMAP_SINGLE, 8352 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 20303 },
	{ 0x93b5, 0x93b5, PDF_CMAP_SINGLE, 9573 },
	{ 0x93b6, 0x93b7, PDF_CMAP_RANGE, 20304 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 9569 },
	{ 0x93b9, 0x93be, PDF_CMAP_RANGE, 20306 },
	{ 0x93bf, 0x93bf, PDF_CMAP_SINGLE, 9570 },
	{ 0x93c0, 0x93c2, PDF_CMAP_RANGE, 20312 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 9580 },
	{ 0x93c4, 0x93c6, PDF_CMAP_RANGE, 20315 },
	{ 0x93c7, 0x93c7, PDF_CMAP_SINGLE, 9581 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 8224 },
	{ 0x93c9, 0x93c9, PDF_CMAP_SINGLE, 20318 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 7557 },
	{ 0x93cb, 0x93cb, PDF_CMAP_SINGLE, 20319 },
	{ 0x93cc, 0x93cc, PDF_CMAP_SINGLE, 9567 },
	{ 0x93cd, 0x93cd, PDF_CMAP_SINGLE, 9578 },
	{ 0x93ce, 0x93d0, PDF_CMAP_RANGE, 20320 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 9582 },
	{ 0x93d2, 0x93d5, PDF_CMAP_RANGE, 20323 },
	{ 0x93d6, 0x93d6, PDF_CMAP_SINGLE, 7681 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 9529 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 9564 },
	{ 0x93d9, 0x93db, PDF_CMAP_RANGE, 20327 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 9576 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 9579 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 7791 },
	{ 0x93e0, 0x93e0, PDF_CMAP_SINGLE, 20330 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 8132 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 9575 },
	{ 0x93e3, 0x93e3, PDF_CMAP_SINGLE, 20331 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 9563 },
	{ 0x93e5, 0x93e7, PDF_CMAP_RANGE, 20332 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 9757 },
	{ 0x93e9, 0x93f4, PDF_CMAP_RANGE, 20335 },
	{ 0x93f5, 0x93f5, PDF_CMAP_SINGLE, 9515 },
	{ 0x93f6, 0x93f6, PDF_CMAP_SINGLE, 20347 },
	{ 0x93f7, 0x93f7, PDF_CMAP_SINGLE, 9585 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 20348 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 9591 },
	{ 0x93fa, 0x93ff, PDF_CMAP_RANGE, 20349 },
	{ 0x9400, 0x9402, PDF_CMAP_RANGE, 20355 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 9506 },
	{ 0x9404, 0x940a, PDF_CMAP_RANGE, 20358 },
	{ 0x940b, 0x940b, PDF_CMAP_SINGLE, 9523 },
	{ 0x940c, 0x940f, PDF_CMAP_RANGE, 20365 },
	{ 0x9410, 0x9410, PDF_CMAP_SINGLE, 8234 },
	{ 0x9411, 0x9411, PDF_CMAP_SINGLE, 20369 },
	{ 0x9412, 0x9412, PDF_CMAP_SINGLE, 9526 },
	{ 0x9413, 0x9413, PDF_CMAP_SINGLE, 9587 },
	{ 0x9414, 0x9414, PDF_CMAP_SINGLE, 9583 },
	{ 0x9415, 0x9417, PDF_CMAP_RANGE, 20370 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 8851 },
	{ 0x9419, 0x9419, PDF_CMAP_SINGLE, 9592 },
	{ 0x941a, 0x941c, PDF_CMAP_RANGE, 20373 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 9584 },
	{ 0x941e, 0x941f, PDF_CMAP_RANGE, 20376 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 9589 },
	{ 0x9421, 0x9425, PDF_CMAP_RANGE, 20378 },
	{ 0x9426, 0x9427, PDF_CMAP_RANGE, 9537 },
	{ 0x9428, 0x9428, PDF_CMAP_SINGLE, 9565 },
	{ 0x9429, 0x942d, PDF_CMAP_RANGE, 20383 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 8218 },
	{ 0x942f, 0x9431, PDF_CMAP_RANGE, 20388 },
	{ 0x9432, 0x9432, PDF_CMAP_SINGLE, 9594 },
	{ 0x9433, 0x9433, PDF_CMAP_SINGLE, 8198 },
	{ 0x9434, 0x9434, PDF_CMAP_SINGLE, 20391 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 8567 },
	{ 0x9436, 0x9437, PDF_CMAP_RANGE, 20392 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 9499 },
	{ 0x9439, 0x9439, PDF_CMAP_SINGLE, 20394 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 9508 },
	{ 0x943b, 0x943d, PDF_CMAP_RANGE, 20395 },
	{ 0x943e, 0x943e, PDF_CMAP_SINGLE, 7559 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 9595 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 20398 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8867 },
	{ 0x9445, 0x9449, PDF_CMAP_RANGE, 20402 },
	{ 0x944a, 0x944a, PDF_CMAP_SINGLE, 9593 },
	{ 0x944b, 0x944b, PDF_CMAP_SINGLE, 20407 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 9574 },
	{ 0x944d, 0x9451, PDF_CMAP_RANGE, 20408 },
	{ 0x9452, 0x9452, PDF_CMAP_SINGLE, 8083 },
	{ 0x9453, 0x9453, PDF_CMAP_SINGLE, 20413 },
	{ 0x9454, 0x9454, PDF_CMAP_SINGLE, 9596 },
	{ 0x9455, 0x945f, PDF_CMAP_RANGE, 20414 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 9492 },
	{ 0x9461, 0x9462, PDF_CMAP_RANGE, 20425 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 9597 },
	{ 0x9464, 0x9464, PDF_CMAP_SINGLE, 20427 },
	{ 0x9465, 0x9465, PDF_CMAP_SINGLE, 9586 },
	{ 0x9466, 0x946a, PDF_CMAP_RANGE, 20428 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 7560 },
	{ 0x946c, 0x946c, PDF_CMAP_SINGLE, 20433 },
	{ 0x946d, 0x946d, PDF_CMAP_SINGLE, 9588 },
	{ 0x946e, 0x946f, PDF_CMAP_RANGE, 20434 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 8782 },
	{ 0x9471, 0x9471, PDF_CMAP_SINGLE, 20436 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 8646 },
	{ 0x9473, 0x9476, PDF_CMAP_RANGE, 20437 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 8351 },
	{ 0x9478, 0x9478, PDF_CMAP_SINGLE, 20441 },
	{ 0x9479, 0x9479, PDF_CMAP_SINGLE, 9590 },
	{ 0x947a, 0x947b, PDF_CMAP_RANGE, 20442 },
	{ 0x947c, 0x947c, PDF_CMAP_SINGLE, 8292 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 8895 },
	{ 0x947e, 0x947e, PDF_CMAP_SINGLE, 9756 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 8798 },
	{ 0x9480, 0x9484, PDF_CMAP_RANGE, 20444 },
	{ 0x9485, 0x9487, PDF_CMAP_RANGE, 6798 },
	{ 0x9488, 0x9488, PDF_CMAP_SINGLE, 4492 },
	{ 0x9489, 0x9489, PDF_CMAP_SINGLE, 1507 },
	{ 0x948a, 0x948a, PDF_CMAP_SINGLE, 6802 },
	{ 0x948b, 0x948b, PDF_CMAP_SINGLE, 6801 },
	{ 0x948c, 0x948d, PDF_CMAP_RANGE, 6803 },
	{ 0x948e, 0x948e, PDF_CMAP_SINGLE, 3106 },
	{ 0x948f, 0x9490, PDF_CMAP_RANGE, 6805 },
	{ 0x9491, 0x9491, PDF_CMAP_SINGLE, 20449 },
	{ 0x9492, 0x9492, PDF_CMAP_SINGLE, 1613 },
	{ 0x9493, 0x9493, PDF_CMAP_SINGLE, 1495 },
	{ 0x9494, 0x9494, PDF_CMAP_SINGLE, 6807 },
	{ 0x9495, 0x9495, PDF_CMAP_SINGLE, 6809 },
	{ 0x9496, 0x9496, PDF_CMAP_SINGLE, 20450 },
	{ 0x9497, 0x9497, PDF_CMAP_SINGLE, 6808 },
	{ 0x9498, 0x9498, PDF_CMAP_SINGLE, 20451 },
	{ 0x9499, 0x9499, PDF_CMAP_SINGLE, 1729 },
	{ 0x949a, 0x949c, PDF_CMAP_RANGE, 6810 },
	{ 0x949d, 0x949d, PDF_CMAP_SINGLE, 1562 },
	{ 0x949e, 0x949e, PDF_CMAP_SINGLE, 1235 },
	{ 0x949f, 0x949f, PDF_CMAP_SINGLE, 4562 },
	{ 0x94a0, 0x94a0, PDF_CMAP_SINGLE, 2857 },
	{ 0x94a1, 0x94a1, PDF_CMAP_SINGLE, 1054 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 1745 },
	{ 0x94a3, 0x94a4, PDF_CMAP_RANGE, 6813 },
	{ 0x94a5, 0x94a5, PDF_CMAP_SINGLE, 4354 },
	{ 0x94a6, 0x94a6, PDF_CMAP_SINGLE, 3154 },
	{ 0x94a7, 0x94a7, PDF_CMAP_SINGLE, 2346 },
	{ 0x94a8, 0x94a8, PDF_CMAP_SINGLE, 3816 },
	{ 0x94a9, 0x94a9, PDF_CMAP_SINGLE, 1804 },
	{ 0x94aa, 0x94aa, PDF_CMAP_SINGLE, 6816 },
	{ 0x94ab, 0x94ab, PDF_CMAP_SINGLE, 6815 },
	{ 0x94ac, 0x94ac, PDF_CMAP_SINGLE, 6818 },
	{ 0x94ad, 0x94ad, PDF_CMAP_SINGLE, 6817 },
	{ 0x94ae, 0x94ae, PDF_CMAP_SINGLE, 2918 },
	{ 0x94af, 0x94b0, PDF_CMAP_RANGE, 6819 },
	{ 0x94b1, 0x94b1, PDF_CMAP_SINGLE, 3115 },
	{ 0x94b2, 0x94b2, PDF_CMAP_SINGLE, 6821 },
	{ 0x94b3, 0x94b3, PDF_CMAP_SINGLE, 3116 },
	{ 0x94b4, 0x94b4, PDF_CMAP_SINGLE, 6822 },
	{ 0x94b5, 0x94b5, PDF_CMAP_SINGLE, 1134 },
	{ 0x94b6, 0x94ba, PDF_CMAP_RANGE, 6823 },
	{ 0x94bb, 0x94bb, PDF_CMAP_SINGLE, 4679 },
	{ 0x94bc, 0x94bd, PDF_CMAP_RANGE, 6828 },
	{ 0x94be, 0x94be, PDF_CMAP_SINGLE, 2123 },
	{ 0x94bf, 0x94bf, PDF_CMAP_SINGLE, 6830 },
	{ 0x94c0, 0x94c0, PDF_CMAP_SINGLE, 4272 },
	{ 0x94c1, 0x94c1, PDF_CMAP_SINGLE, 3661 },
	{ 0x94c2, 0x94c2, PDF_CMAP_SINGLE, 1139 },
	{ 0x94c3, 0x94c3, PDF_CMAP_SINGLE, 2606 },
	{ 0x94c4, 0x94c4, PDF_CMAP_SINGLE, 6831 },
	{ 0x94c5, 0x94c5, PDF_CMAP_SINGLE, 3107 },
	{ 0x94c6, 0x94c6, PDF_CMAP_SINGLE, 2738 },
	{ 0x94c7, 0x94c7, PDF_CMAP_SINGLE, 20452 },
	{ 0x94c8, 0x94ce, PDF_CMAP_RANGE, 6832 },
	{ 0x94cf, 0x94cf, PDF_CMAP_SINGLE, 20453 },
	{ 0x94d0, 0x94d2, PDF_CMAP_RANGE, 6839 },
	{ 0x94d3, 0x94d4, PDF_CMAP_RANGE, 20454 },
	{ 0x94d5, 0x94d7, PDF_CMAP_RANGE, 6842 },
	{ 0x94d8, 0x94d8, PDF_CMAP_SINGLE, 6846 },
	{ 0x94d9, 0x94d9, PDF_CMAP_SINGLE, 6845 },
	{ 0x94da, 0x94da, PDF_CMAP_SINGLE, 20456 },
	{ 0x94db, 0x94db, PDF_CMAP_SINGLE, 6847 },
	{ 0x94dc, 0x94dc, PDF_CMAP_SINGLE, 3678 },
	{ 0x94dd, 0x94dd, PDF_CMAP_SINGLE, 2664 },
	{ 0x94de, 0x94e0, PDF_CMAP_RANGE, 6848 },
	{ 0x94e1, 0x94e1, PDF_CMAP_SINGLE, 4418 },
	{ 0x94e2, 0x94e2, PDF_CMAP_SINGLE, 6851 },
	{ 0x94e3, 0x94e3, PDF_CMAP_SINGLE, 3872 },
	{ 0x94e4, 0x94e5, PDF_CMAP_RANGE, 6852 },
	{ 0x94e6, 0x94e6, PDF_CMAP_SINGLE, 20457 },
	{ 0x94e7, 0x94e8, PDF_CMAP_RANGE, 6854 },
	{ 0x94e9, 0x94e9, PDF_CMAP_SINGLE, 6857 },
	{ 0x94ea, 0x94ea, PDF_CMAP_SINGLE, 6856 },
	{ 0x94eb, 0x94eb, PDF_CMAP_SINGLE, 6858 },
	{ 0x94ec, 0x94ec, PDF_CMAP_SINGLE, 1776 },
	{ 0x94ed, 0x94ed, PDF_CMAP_SINGLE, 2815 },
	{ 0x94ee, 0x94ef, PDF_CMAP_RANGE, 6859 },
	{ 0x94f0, 0x94f0, PDF_CMAP_SINGLE, 2195 },
	{ 0x94f1, 0x94f1, PDF_CMAP_SINGLE, 4166 },
	{ 0x94f2, 0x94f2, PDF_CMAP_SINGLE, 1216 },
	{ 0x94f3, 0x94f5, PDF_CMAP_RANGE, 6861 },
	{ 0x94f6, 0x94f6, PDF_CMAP_SINGLE, 4223 },
	{ 0x94f7, 0x94f7, PDF_CMAP_SINGLE, 6864 },
	{ 0x94f8, 0x94f8, PDF_CMAP_SINGLE, 4604 },
	{ 0x94f9, 0x94f9, PDF_CMAP_SINGLE, 6865 },
	{ 0x94fa, 0x94fa, PDF_CMAP_SINGLE, 3051 },
	{ 0x94fb, 0x94fb, PDF_CMAP_SINGLE, 20458 },
	{ 0x94fc, 0x94fd, PDF_CMAP_RANGE, 6866 },
	{ 0x94fe, 0x94fe, PDF_CMAP_SINGLE, 2557 },
	{ 0x94ff, 0x94ff, PDF_CMAP_SINGLE, 6868 },
	{ 0x9500, 0x9500, PDF_CMAP_SINGLE, 3943 },
	{ 0x9501, 0x9501, PDF_CMAP_SINGLE, 3565 },
	{ 0x9502, 0x9502, PDF_CMAP_SINGLE, 6870 },
	{ 0x9503, 0x9503, PDF_CMAP_SINGLE, 6869 },
	{ 0x9504, 0x9504, PDF_CMAP_SINGLE, 1311 },
	{ 0x9505, 0x9505, PDF_CMAP_SINGLE, 1873 },
	{ 0x9506, 0x9507, PDF_CMAP_RANGE, 6871 },
	{ 0x9508, 0x9508, PDF_CMAP_SINGLE, 4013 },
	{ 0x9509, 0x950a, PDF_CMAP_RANGE, 6873 },
	{ 0x950b, 0x950b, PDF_CMAP_SINGLE, 1667 },
	{ 0x950c, 0x950c, PDF_CMAP_SINGLE, 3978 },
	{ 0x950d, 0x950f, PDF_CMAP_RANGE, 6875 },
	{ 0x9510, 0x9510, PDF_CMAP_SINGLE, 3276 },
	{ 0x9511, 0x9511, PDF_CMAP_SINGLE, 3635 },
	{ 0x9512, 0x9516, PDF_CMAP_RANGE, 6878 },
	{ 0x9517, 0x9517, PDF_CMAP_SINGLE, 4481 },
	{ 0x9518, 0x9518, PDF_CMAP_SINGLE, 6883 },
	{ 0x9519, 0x9519, PDF_CMAP_SINGLE, 1392 },
	{ 0x951a, 0x951a, PDF_CMAP_SINGLE, 2735 },
	{ 0x951b, 0x951b, PDF_CMAP_SINGLE, 6884 },
	{ 0x951c, 0x951c, PDF_CMAP_SINGLE, 20459 },
	{ 0x951d, 0x951f, PDF_CMAP_RANGE, 6885 },
	{ 0x9520, 0x9520, PDF_CMAP_SINGLE, 20460 },
	{ 0x9521, 0x9521, PDF_CMAP_SINGLE, 3852 },
	{ 0x9522, 0x9522, PDF_CMAP_SINGLE, 6888 },
	{ 0x9523, 0x9523, PDF_CMAP_SINGLE, 2695 },
	{ 0x9524, 0x9524, PDF_CMAP_SINGLE, 1339 },
	{ 0x9525, 0x9525, PDF_CMAP_SINGLE, 4627 },
	{ 0x9526, 0x9526, PDF_CMAP_SINGLE, 2246 },
	{ 0x9527, 0x9527, PDF_CMAP_SINGLE, 20461 },
	{ 0x9528, 0x9528, PDF_CMAP_SINGLE, 3892 },
	{ 0x9529, 0x9529, PDF_CMAP_SINGLE, 6891 },
	{ 0x952a, 0x952b, PDF_CMAP_RANGE, 6889 },
	{ 0x952c, 0x952c, PDF_CMAP_SINGLE, 6892 },
	{ 0x952d, 0x952d, PDF_CMAP_SINGLE, 1510 },
	{ 0x952e, 0x952e, PDF_CMAP_SINGLE, 2159 },
	{ 0x952f, 0x952f, PDF_CMAP_SINGLE, 2321 },
	{ 0x9530, 0x9530, PDF_CMAP_SINGLE, 2769 },
	{ 0x9531, 0x9532, PDF_CMAP_RANGE, 6893 },
	{ 0x9533, 0x9533, PDF_CMAP_SINGLE, 20462 },
	{ 0x9534, 0x9534, PDF_CMAP_SINGLE, 6895 },
	{ 0x9535, 0x9535, PDF_CMAP_SINGLE, 6903 },
	{ 0x9536, 0x9538, PDF_CMAP_RANGE, 6896 },
	{ 0x9539, 0x9539, PDF_CMAP_SINGLE, 3135 },
	{ 0x953a, 0x953a, PDF_CMAP_SINGLE, 6937 },
	{ 0x953b, 0x953b, PDF_CMAP_SINGLE, 1548 },
	{ 0x953c, 0x953c, PDF_CMAP_SINGLE, 6899 },
	{ 0x953d, 0x953d, PDF_CMAP_SINGLE, 20463 },
	{ 0x953e, 0x953f, PDF_CMAP_RANGE, 6900 },
	{ 0x9540, 0x9540, PDF_CMAP_SINGLE, 1541 },
	{ 0x9541, 0x9541, PDF_CMAP_SINGLE, 2755 },
	{ 0x9542, 0x9542, PDF_CMAP_SINGLE, 6902 },
	{ 0x9543, 0x9543, PDF_CMAP_SINGLE, 20464 },
	{ 0x9544, 0x9546, PDF_CMAP_RANGE, 6904 },
	{ 0x9547, 0x9547, PDF_CMAP_SINGLE, 4499 },
	{ 0x9548, 0x9548, PDF_CMAP_SINGLE, 20465 },
	{ 0x9549, 0x9549, PDF_CMAP_SINGLE, 6907 },
	{ 0x954a, 0x954a, PDF_CMAP_SINGLE, 2906 },
	{ 0x954b, 0x954b, PDF_CMAP_SINGLE, 20466 },
	{ 0x954c, 0x954c, PDF_CMAP_SINGLE, 6908 },
	{ 0x954d, 0x954d, PDF_CMAP_SINGLE, 2907 },
	{ 0x954e, 0x954f, PDF_CMAP_RANGE, 6909 },
	{ 0x9550, 0x9550, PDF_CMAP_SINGLE, 1759 },
	{ 0x9551, 0x9551, PDF_CMAP_SINGLE, 1026 },
	{ 0x9552, 0x9554, PDF_CMAP_RANGE, 6911 },
	{ 0x9555, 0x9555, PDF_CMAP_SINGLE, 20467 },
	{ 0x9556, 0x9559, PDF_CMAP_RANGE, 6914 },
	{ 0x955a, 0x955a, PDF_CMAP_SINGLE, 20468 },
	{ 0x955b, 0x955b, PDF_CMAP_SINGLE, 6918 },
	{ 0x955c, 0x955c, PDF_CMAP_SINGLE, 2276 },
	{ 0x955d, 0x955d, PDF_CMAP_SINGLE, 6921 },
	{ 0x955e, 0x955f, PDF_CMAP_RANGE, 6919 },
	{ 0x9560, 0x9560, PDF_CMAP_SINGLE, 20469 },
	{ 0x9561, 0x9562, PDF_CMAP_RANGE, 6922 },
	{ 0x9563, 0x9563, PDF_CMAP_SINGLE, 2582 },
	{ 0x9564, 0x956c, PDF_CMAP_RANGE, 6924 },
	{ 0x956d, 0x956d, PDF_CMAP_SINGLE, 2499 },
	{ 0x956e, 0x956e, PDF_CMAP_SINGLE, 20470 },
	{ 0x956f, 0x956f, PDF_CMAP_SINGLE, 6933 },
	{ 0x9570, 0x9570, PDF_CMAP_SINGLE, 2550 },
	{ 0x9571, 0x9573, PDF_CMAP_RANGE, 6934 },
	{ 0x9574, 0x9575, PDF_CMAP_RANGE, 20471 },
	{ 0x9576, 0x9576, PDF_CMAP_SINGLE, 3919 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 7797 },
	{ 0x9578, 0x957e, PDF_CMAP_RANGE, 20473 },
	{ 0x957f, 0x957f, PDF_CMAP_SINGLE, 1225 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 8317 },
	{ 0x9581, 0x9581, PDF_CMAP_SINGLE, 20480 },
	{ 0x9582, 0x9582, PDF_CMAP_SINGLE, 9151 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 8467 },
	{ 0x9584, 0x9585, PDF_CMAP_RANGE, 20481 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 9152 },
	{ 0x9587, 0x9588, PDF_CMAP_RANGE, 20483 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 7749 },
	{ 0x958a, 0x958a, PDF_CMAP_SINGLE, 20485 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 8152 },
	{ 0x958c, 0x958c, PDF_CMAP_SINGLE, 9156 },
	{ 0x958d, 0x958d, PDF_CMAP_SINGLE, 20486 },
	{ 0x958e, 0x958e, PDF_CMAP_SINGLE, 9154 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 8452 },
	{ 0x9590, 0x9590, PDF_CMAP_SINGLE, 20487 },
	{ 0x9591, 0x9591, PDF_CMAP_SINGLE, 8637 },
	{ 0x9592, 0x9592, PDF_CMAP_SINGLE, 20488 },
	{ 0x9593, 0x9593, PDF_CMAP_SINGLE, 8071 },
	{ 0x9594, 0x9594, PDF_CMAP_SINGLE, 9155 },
	{ 0x9595, 0x9597, PDF_CMAP_RANGE, 20489 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 8809 },
	{ 0x9599, 0x95a0, PDF_CMAP_RANGE, 20492 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 8003 },
	{ 0x95a2, 0x95a2, PDF_CMAP_SINGLE, 20500 },
	{ 0x95a3, 0x95a3, PDF_CMAP_SINGLE, 7966 },
	{ 0x95a4, 0x95a4, PDF_CMAP_SINGLE, 9849 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 7915 },
	{ 0x95a6, 0x95a7, PDF_CMAP_RANGE, 20501 },
	{ 0x95a8, 0x95a8, PDF_CMAP_SINGLE, 7989 },
	{ 0x95a9, 0x95a9, PDF_CMAP_SINGLE, 8330 },
	{ 0x95aa, 0x95aa, PDF_CMAP_SINGLE, 20503 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 9159 },
	{ 0x95ac, 0x95ac, PDF_CMAP_SINGLE, 9161 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 9158 },
	{ 0x95ae, 0x95b1, PDF_CMAP_RANGE, 20504 },
	{ 0x95b2, 0x95b2, PDF_CMAP_SINGLE, 8783 },
	{ 0x95b3, 0x95b5, PDF_CMAP_RANGE, 20508 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 9163 },
	{ 0x95b7, 0x95b8, PDF_CMAP_RANGE, 20511 },
	{ 0x95b9, 0x95b9, PDF_CMAP_SINGLE, 8691 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 20513 },
	{ 0x95bb, 0x95bb, PDF_CMAP_SINGLE, 8695 },
	{ 0x95bc, 0x95bc, PDF_CMAP_SINGLE, 9167 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 9166 },
	{ 0x95be, 0x95be, PDF_CMAP_SINGLE, 9162 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 9165 },
	{ 0x95c0, 0x95c2, PDF_CMAP_RANGE, 20514 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 9168 },
	{ 0x95c4, 0x95c5, PDF_CMAP_RANGE, 20517 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 9836 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 20519 },
	{ 0x95c8, 0x95c8, PDF_CMAP_SINGLE, 9153 },
	{ 0x95c9, 0x95c9, PDF_CMAP_SINGLE, 20520 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 8174 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 9169 },
	{ 0x95cc, 0x95cc, PDF_CMAP_SINGLE, 8184 },
	{ 0x95cd, 0x95cf, PDF_CMAP_RANGE, 20521 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 9171 },
	{ 0x95d1, 0x95d3, PDF_CMAP_RANGE, 20524 },
	{ 0x95d4, 0x95d4, PDF_CMAP_SINGLE, 9170 },
	{ 0x95d5, 0x95d5, PDF_CMAP_SINGLE, 9172 },
	{ 0x95d6, 0x95d6, PDF_CMAP_SINGLE, 7832 },
	{ 0x95d7, 0x95db, PDF_CMAP_RANGE, 20527 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 7980 },
	{ 0x95dd, 0x95dd, PDF_CMAP_SINGLE, 20532 },
	{ 0x95de, 0x95de, PDF_CMAP_SINGLE, 9173 },
	{ 0x95df, 0x95e0, PDF_CMAP_RANGE, 20533 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 7793 },
	{ 0x95e2, 0x95e2, PDF_CMAP_SINGLE, 9873 },
	{ 0x95e3, 0x95e4, PDF_CMAP_RANGE, 20535 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 9157 },
	{ 0x95e6, 0x95e7, PDF_CMAP_RANGE, 20537 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 2762 },
	{ 0x95e9, 0x95e9, PDF_CMAP_SINGLE, 5765 },
	{ 0x95ea, 0x95ea, PDF_CMAP_SINGLE, 3322 },
	{ 0x95eb, 0x95eb, PDF_CMAP_SINGLE, 5766 },
	{ 0x95ec, 0x95ec, PDF_CMAP_SINGLE, 20539 },
	{ 0x95ed, 0x95ed, PDF_CMAP_SINGLE, 1086 },
	{ 0x95ee, 0x95ee, PDF_CMAP_SINGLE, 3801 },
	{ 0x95ef, 0x95ef, PDF_CMAP_SINGLE, 1334 },
	{ 0x95f0, 0x95f0, PDF_CMAP_SINGLE, 3277 },
	{ 0x95f1, 0x95f1, PDF_CMAP_SINGLE, 5767 },
	{ 0x95f2, 0x95f2, PDF_CMAP_SINGLE, 3901 },
	{ 0x95f3, 0x95f3, PDF_CMAP_SINGLE, 5768 },
	{ 0x95f4, 0x95f4, PDF_CMAP_SINGLE, 2135 },
	{ 0x95f5, 0x95f6, PDF_CMAP_RANGE, 5769 },
	{ 0x95f7, 0x95f7, PDF_CMAP_SINGLE, 2763 },
	{ 0x95f8, 0x95f8, PDF_CMAP_SINGLE, 4419 },
	{ 0x95f9, 0x95f9, PDF_CMAP_SINGLE, 2873 },
	{ 0x95fa, 0x95fa, PDF_CMAP_SINGLE, 1860 },
	{ 0x95fb, 0x95fb, PDF_CMAP_SINGLE, 3796 },
	{ 0x95fc, 0x95fc, PDF_CMAP_SINGLE, 5771 },
	{ 0x95fd, 0x95fd, PDF_CMAP_SINGLE, 2811 },
	{ 0x95fe, 0x95fe, PDF_CMAP_SINGLE, 5772 },
	{ 0x95ff, 0x95ff, PDF_CMAP_SINGLE, 20540 },
	{ 0x9600, 0x9600, PDF_CMAP_SINGLE, 1604 },
	{ 0x9601, 0x9601, PDF_CMAP_SINGLE, 1774 },
	{ 0x9602, 0x9602, PDF_CMAP_SINGLE, 1929 },
	{ 0x9603, 0x9604, PDF_CMAP_RANGE, 5773 },
	{ 0x9605, 0x9605, PDF_CMAP_SINGLE, 4359 },
	{ 0x9606, 0x9606, PDF_CMAP_SINGLE, 5775 },
	{ 0x9607, 0x9607, PDF_CMAP_SINGLE, 20541 },
	{ 0x9608, 0x9608, PDF_CMAP_SINGLE, 5776 },
	{ 0x9609, 0x9609, PDF_CMAP_SINGLE, 4084 },
	{ 0x960a, 0x960d, PDF_CMAP_RANGE, 5777 },
	{ 0x960e, 0x960e, PDF_CMAP_SINGLE, 4095 },
	{ 0x960f, 0x960f, PDF_CMAP_SINGLE, 5781 },
	{ 0x9610, 0x9610, PDF_CMAP_SINGLE, 1218 },
	{ 0x9611, 0x9611, PDF_CMAP_SINGLE, 2470 },
	{ 0x9612, 0x9612, PDF_CMAP_SINGLE, 5782 },
	{ 0x9613, 0x9613, PDF_CMAP_SINGLE, 20542 },
	{ 0x9614, 0x9614, PDF_CMAP_SINGLE, 2454 },
	{ 0x9615, 0x9617, PDF_CMAP_RANGE, 5783 },
	{ 0x9618, 0x9618, PDF_CMAP_SINGLE, 20543 },
	{ 0x9619, 0x961a, PDF_CMAP_RANGE, 5786 },
	{ 0x961b, 0x961b, PDF_CMAP_SINGLE, 20544 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 1714 },
	{ 0x961d, 0x961d, PDF_CMAP_SINGLE, 4948 },
	{ 0x961e, 0x961e, PDF_CMAP_SINGLE, 20545 },
	{ 0x961f, 0x961f, PDF_CMAP_SINGLE, 1554 },
	{ 0x9620, 0x9620, PDF_CMAP_SINGLE, 20546 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 4950 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 4949 },
	{ 0x9623, 0x9629, PDF_CMAP_RANGE, 20547 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 4952 },
	{ 0x962b, 0x962d, PDF_CMAP_RANGE, 20554 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 3273 },
	{ 0x962f, 0x9630, PDF_CMAP_RANGE, 20557 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 4951 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 1629 },
	{ 0x9633, 0x9633, PDF_CMAP_SINGLE, 4125 },
	{ 0x9634, 0x9634, PDF_CMAP_SINGLE, 4220 },
	{ 0x9635, 0x9635, PDF_CMAP_SINGLE, 4500 },
	{ 0x9636, 0x9636, PDF_CMAP_SINGLE, 2216 },
	{ 0x9637, 0x963a, PDF_CMAP_RANGE, 20559 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 4677 },
	{ 0x963c, 0x963c, PDF_CMAP_SINGLE, 4954 },
	{ 0x963d, 0x963d, PDF_CMAP_SINGLE, 4953 },
	{ 0x963e, 0x963e, PDF_CMAP_SINGLE, 20563 },
	{ 0x963f, 0x963f, PDF_CMAP_SINGLE, 941 },
	{ 0x9640, 0x9640, PDF_CMAP_SINGLE, 3716 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 20564 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 4955 },
	{ 0x9643, 0x9643, PDF_CMAP_SINGLE, 20565 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 1720 },
	{ 0x9645, 0x9645, PDF_CMAP_SINGLE, 2109 },
	{ 0x9646, 0x9646, PDF_CMAP_SINGLE, 2660 },
	{ 0x9647, 0x9647, PDF_CMAP_SINGLE, 2635 },
	{ 0x9648, 0x9648, PDF_CMAP_SINGLE, 1255 },
	{ 0x9649, 0x9649, PDF_CMAP_SINGLE, 4956 },
	{ 0x964a, 0x964a, PDF_CMAP_SINGLE, 20566 },
	{ 0x964b, 0x964b, PDF_CMAP_SINGLE, 2641 },
	{ 0x964c, 0x964c, PDF_CMAP_SINGLE, 2835 },
	{ 0x964d, 0x964d, PDF_CMAP_SINGLE, 2182 },
	{ 0x964e, 0x964f, PDF_CMAP_RANGE, 20567 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 3915 },
	{ 0x9651, 0x9653, PDF_CMAP_RANGE, 20569 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 4957 },
	{ 0x9655, 0x9655, PDF_CMAP_SINGLE, 3323 },
	{ 0x9656, 0x9657, PDF_CMAP_RANGE, 20572 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 8986 },
	{ 0x9659, 0x965a, PDF_CMAP_RANGE, 20574 },
	{ 0x965b, 0x965b, PDF_CMAP_SINGLE, 1094 },
	{ 0x965c, 0x965c, PDF_CMAP_SINGLE, 20576 },
	{ 0x965d, 0x965d, PDF_CMAP_SINGLE, 8468 },
	{ 0x965e, 0x965e, PDF_CMAP_SINGLE, 20577 },
	{ 0x965f, 0x965f, PDF_CMAP_SINGLE, 4958 },
	{ 0x9660, 0x9660, PDF_CMAP_SINGLE, 20578 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 1527 },
	{ 0x9662, 0x9662, PDF_CMAP_SINGLE, 4349 },
	{ 0x9663, 0x9663, PDF_CMAP_SINGLE, 8836 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 1314 },
	{ 0x9665, 0x9666, PDF_CMAP_RANGE, 20579 },
	{ 0x9667, 0x9667, PDF_CMAP_SINGLE, 4959 },
	{ 0x9668, 0x9668, PDF_CMAP_SINGLE, 4364 },
	{ 0x9669, 0x9669, PDF_CMAP_SINGLE, 3906 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 2980 },
	{ 0x966b, 0x966b, PDF_CMAP_SINGLE, 20581 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 4960 },
	{ 0x966d, 0x966f, PDF_CMAP_RANGE, 20582 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 8732 },
	{ 0x9671, 0x9671, PDF_CMAP_SINGLE, 20585 },
	{ 0x9672, 0x9672, PDF_CMAP_SINGLE, 4961 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 7806 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 4962 },
	{ 0x9675, 0x9675, PDF_CMAP_SINGLE, 2611 },
	{ 0x9676, 0x9676, PDF_CMAP_SINGLE, 3624 },
	{ 0x9677, 0x9677, PDF_CMAP_SINGLE, 3914 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 8269 },
	{ 0x9679, 0x967c, PDF_CMAP_RANGE, 20586 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 8705 },
	{ 0x967e, 0x9684, PDF_CMAP_RANGE, 20590 },
	{ 0x9685, 0x9685, PDF_CMAP_SINGLE, 4300 },
	{ 0x9686, 0x9686, PDF_CMAP_SINGLE, 2632 },
	{ 0x9687, 0x9687, PDF_CMAP_SINGLE, 20597 },
	{ 0x9688, 0x9688, PDF_CMAP_SINGLE, 4963 },
	{ 0x9689, 0x9689, PDF_CMAP_SINGLE, 20598 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 7897 },
	{ 0x968b, 0x968b, PDF_CMAP_SINGLE, 3546 },
	{ 0x968c, 0x968c, PDF_CMAP_SINGLE, 20599 },
	{ 0x968d, 0x968d, PDF_CMAP_SINGLE, 4964 },
	{ 0x968e, 0x968e, PDF_CMAP_SINGLE, 8114 },
	{ 0x968f, 0x968f, PDF_CMAP_SINGLE, 3547 },
	{ 0x9690, 0x9690, PDF_CMAP_SINGLE, 4229 },
	{ 0x9691, 0x9693, PDF_CMAP_RANGE, 20600 },
	{ 0x9694, 0x9694, PDF_CMAP_SINGLE, 1775 },
	{ 0x9695, 0x9695, PDF_CMAP_SINGLE, 8786 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 20603 },
	{ 0x9697, 0x9697, PDF_CMAP_SINGLE, 4965 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 954 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 3875 },
	{ 0x969a, 0x969a, PDF_CMAP_SINGLE, 20604 },
	{ 0x969b, 0x969b, PDF_CMAP_SINGLE, 8057 },
	{ 0x969c, 0x969c, PDF_CMAP_SINGLE, 4464 },
	{ 0x969d, 0x96a6, PDF_CMAP_RANGE, 20605 },
	{ 0x96a7, 0x96a7, PDF_CMAP_SINGLE, 3554 },
	{ 0x96a8, 0x96a8, PDF_CMAP_SINGLE, 8535 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 20615 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 8639 },
	{ 0x96ab, 0x96af, PDF_CMAP_RANGE, 20616 },
	{ 0x96b0, 0x96b0, PDF_CMAP_SINGLE, 4966 },
	{ 0x96b1, 0x96b1, PDF_CMAP_SINGLE, 8735 },
	{ 0x96b2, 0x96b2, PDF_CMAP_SINGLE, 20621 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 5764 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 8253 },
	{ 0x96b5, 0x96b5, PDF_CMAP_SINGLE, 20622 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 2542 },
	{ 0x96b7, 0x96b7, PDF_CMAP_SINGLE, 20623 },
	{ 0x96b8, 0x96b8, PDF_CMAP_SINGLE, 8213 },
	{ 0x96b9, 0x96b9, PDF_CMAP_SINGLE, 7545 },
	{ 0x96ba, 0x96ba, PDF_CMAP_SINGLE, 20624 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 9893 },
	{ 0x96bc, 0x96bd, PDF_CMAP_RANGE, 7546 },
	{ 0x96be, 0x96be, PDF_CMAP_SINGLE, 2868 },
	{ 0x96bf, 0x96bf, PDF_CMAP_SINGLE, 20625 },
	{ 0x96c0, 0x96c0, PDF_CMAP_SINGLE, 3219 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 4108 },
	{ 0x96c2, 0x96c3, PDF_CMAP_RANGE, 20626 },
	{ 0x96c4, 0x96c4, PDF_CMAP_SINGLE, 4006 },
	{ 0x96c5, 0x96c5, PDF_CMAP_SINGLE, 4078 },
	{ 0x96c6, 0x96c6, PDF_CMAP_SINGLE, 2082 },
	{ 0x96c7, 0x96c7, PDF_CMAP_SINGLE, 1830 },
	{ 0x96c8, 0x96c8, PDF_CMAP_SINGLE, 20628 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 6940 },
	{ 0x96ca, 0x96cb, PDF_CMAP_RANGE, 20629 },
	{ 0x96cc, 0x96cc, PDF_CMAP_SINGLE, 1353 },
	{ 0x96cd, 0x96cd, PDF_CMAP_SINGLE, 4255 },
	{ 0x96ce, 0x96ce, PDF_CMAP_SINGLE, 7548 },
	{ 0x96cf, 0x96cf, PDF_CMAP_SINGLE, 1312 },
	{ 0x96d0, 0x96d1, PDF_CMAP_RANGE, 20631 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 7549 },
	{ 0x96d3, 0x96d4, PDF_CMAP_RANGE, 20633 },
	{ 0x96d5, 0x96d5, PDF_CMAP_SINGLE, 1490 },
	{ 0x96d6, 0x96d6, PDF_CMAP_SINGLE, 8534 },
	{ 0x96d7, 0x96d8, PDF_CMAP_RANGE, 20635 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 8516 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 20637 },
	{ 0x96db, 0x96db, PDF_CMAP_SINGLE, 7825 },
	{ 0x96dc, 0x96dc, PDF_CMAP_SINGLE, 8791 },
	{ 0x96dd, 0x96df, PDF_CMAP_RANGE, 20638 },
	{ 0x96e0, 0x96e0, PDF_CMAP_SINGLE, 7551 },
	{ 0x96e1, 0x96e1, PDF_CMAP_SINGLE, 20641 },
	{ 0x96e2, 0x96e2, PDF_CMAP_SINGLE, 8202 },
	{ 0x96e3, 0x96e3, PDF_CMAP_SINGLE, 8338 },
	{ 0x96e4, 0x96e7, PDF_CMAP_RANGE, 20642 },
	{ 0x96e8, 0x96e8, PDF_CMAP_SINGLE, 4303 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 7521 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 4050 },
	{ 0x96eb, 0x96ee, PDF_CMAP_RANGE, 20646 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 7523 },
	{ 0x96f0, 0x96f1, PDF_CMAP_RANGE, 20650 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 8784 },
	{ 0x96f3, 0x96f3, PDF_CMAP_SINGLE, 7522 },
	{ 0x96f4, 0x96f5, PDF_CMAP_RANGE, 20652 },
	{ 0x96f6, 0x96f6, PDF_CMAP_SINGLE, 2604 },
	{ 0x96f7, 0x96f7, PDF_CMAP_SINGLE, 2498 },
	{ 0x96f8, 0x96f8, PDF_CMAP_SINGLE, 20654 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 1035 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 20655 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 7875 },
	{ 0x96fc, 0x96fd, PDF_CMAP_RANGE, 20656 },
	{ 0x96fe, 0x96fe, PDF_CMAP_SINGLE, 3836 },
	{ 0x96ff, 0x96ff, PDF_CMAP_SINGLE, 20658 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 4019 },
	{ 0x9701, 0x9701, PDF_CMAP_SINGLE, 7525 },
	{ 0x9702, 0x9703, PDF_CMAP_RANGE, 20659 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 3939 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 20661 },
	{ 0x9706, 0x9706, PDF_CMAP_SINGLE, 7524 },
	{ 0x9707, 0x9707, PDF_CMAP_SINGLE, 4497 },
	{ 0x9708, 0x9708, PDF_CMAP_SINGLE, 7526 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 2750 },
	{ 0x970a, 0x970c, PDF_CMAP_RANGE, 20662 },
	{ 0x970d, 0x970d, PDF_CMAP_SINGLE, 2057 },
	{ 0x970e, 0x970e, PDF_CMAP_SINGLE, 7528 },
	{ 0x970f, 0x970f, PDF_CMAP_SINGLE, 7527 },
	{ 0x9710, 0x9712, PDF_CMAP_RANGE, 20665 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 2881 },
	{ 0x9714, 0x9715, PDF_CMAP_RANGE, 20668 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 2593 },
	{ 0x9717, 0x971b, PDF_CMAP_RANGE, 20670 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 3487 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 20675 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 3881 },
	{ 0x971f, 0x9726, PDF_CMAP_RANGE, 20676 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 8616 },
	{ 0x9728, 0x9729, PDF_CMAP_RANGE, 20684 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7529 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 20686 },
	{ 0x972d, 0x972d, PDF_CMAP_SINGLE, 7530 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 20688 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 7531 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 20690 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 2653 },
	{ 0x9733, 0x9737, PDF_CMAP_RANGE, 20691 },
	{ 0x9738, 0x9738, PDF_CMAP_SINGLE, 991 },
	{ 0x9739, 0x9739, PDF_CMAP_SINGLE, 3002 },
	{ 0x973a, 0x973c, PDF_CMAP_RANGE, 20696 },
	{ 0x973d, 0x973d, PDF_CMAP_SINGLE, 9741 },
	{ 0x973e, 0x973e, PDF_CMAP_SINGLE, 7532 },
	{ 0x973f, 0x9741, PDF_CMAP_RANGE, 20699 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 9740 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 20702 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 9742 },
	{ 0x9745, 0x9747, PDF_CMAP_RANGE, 20703 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 8242 },
	{ 0x9749, 0x9751, PDF_CMAP_RANGE, 20706 },
	{ 0x9752, 0x9752, PDF_CMAP_SINGLE, 3165 },
	{ 0x9753, 0x9753, PDF_CMAP_SINGLE, 7520 },
	{ 0x9754, 0x9755, PDF_CMAP_RANGE, 20715 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 2279 },
	{ 0x9757, 0x9758, PDF_CMAP_RANGE, 20717 },
	{ 0x9759, 0x9759, PDF_CMAP_SINGLE, 2273 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 9739 },
	{ 0x975b, 0x975b, PDF_CMAP_SINGLE, 1478 },
	{ 0x975c, 0x975d, PDF_CMAP_RANGE, 20719 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 1636 },
	{ 0x975f, 0x975f, PDF_CMAP_SINGLE, 20721 },
	{ 0x9760, 0x9760, PDF_CMAP_SINGLE, 2380 },
	{ 0x9761, 0x9761, PDF_CMAP_SINGLE, 2775 },
	{ 0x9762, 0x9762, PDF_CMAP_SINGLE, 2795 },
	{ 0x9763, 0x9764, PDF_CMAP_RANGE, 20722 },
	{ 0x9765, 0x9765, PDF_CMAP_SINGLE, 4738 },
	{ 0x9766, 0x9767, PDF_CMAP_RANGE, 20724 },
	{ 0x9768, 0x9768, PDF_CMAP_SINGLE, 8899 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 1770 },
	{ 0x976a, 0x9772, PDF_CMAP_RANGE, 20726 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 2250 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 4046 },
	{ 0x9775, 0x9775, PDF_CMAP_SINGLE, 20735 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 987 },
	{ 0x9777, 0x977b, PDF_CMAP_RANGE, 20736 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7624 },
	{ 0x977d, 0x9784, PDF_CMAP_RANGE, 20741 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 7625 },
	{ 0x9786, 0x978a, PDF_CMAP_RANGE, 20749 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 3959 },
	{ 0x978c, 0x978c, PDF_CMAP_SINGLE, 20754 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 955 },
	{ 0x978e, 0x978e, PDF_CMAP_SINGLE, 20755 },
	{ 0x978f, 0x978f, PDF_CMAP_SINGLE, 7971 },
	{ 0x9790, 0x9790, PDF_CMAP_SINGLE, 20756 },
	{ 0x9791, 0x9792, PDF_CMAP_RANGE, 7626 },
	{ 0x9793, 0x9793, PDF_CMAP_SINGLE, 20757 },
	{ 0x9794, 0x9794, PDF_CMAP_SINGLE, 7628 },
	{ 0x9795, 0x9797, PDF_CMAP_RANGE, 20758 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 3143 },
	{ 0x9799, 0x979f, PDF_CMAP_RANGE, 20761 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 2302 },
	{ 0x97a1, 0x97a2, PDF_CMAP_RANGE, 20768 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 7631 },
	{ 0x97a4, 0x97a5, PDF_CMAP_RANGE, 20770 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 9877 },
	{ 0x97a7, 0x97aa, PDF_CMAP_RANGE, 20772 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 7630 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 20776 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 1095 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 20777 },
	{ 0x97af, 0x97af, PDF_CMAP_SINGLE, 7629 },
	{ 0x97b0, 0x97b1, PDF_CMAP_RANGE, 20778 },
	{ 0x97b2, 0x97b2, PDF_CMAP_SINGLE, 7632 },
	{ 0x97b3, 0x97b3, PDF_CMAP_SINGLE, 20780 },
	{ 0x97b4, 0x97b4, PDF_CMAP_SINGLE, 7633 },
	{ 0x97b5, 0x97bc, PDF_CMAP_RANGE, 20781 },
	{ 0x97bd, 0x97bd, PDF_CMAP_SINGLE, 9822 },
	{ 0x97be, 0x97c2, PDF_CMAP_RANGE, 20789 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 9821 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 20794 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 9874 },
	{ 0x97c7, 0x97c8, PDF_CMAP_RANGE, 20796 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 9823 },
	{ 0x97ca, 0x97ca, PDF_CMAP_SINGLE, 20798 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 8589 },
	{ 0x97cc, 0x97cc, PDF_CMAP_SINGLE, 8445 },
	{ 0x97cd, 0x97d2, PDF_CMAP_RANGE, 20799 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 8000 },
	{ 0x97d4, 0x97d8, PDF_CMAP_RANGE, 20805 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 9317 },
	{ 0x97da, 0x97db, PDF_CMAP_RANGE, 20810 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 9319 },
	{ 0x97dd, 0x97dd, PDF_CMAP_SINGLE, 20812 },
	{ 0x97de, 0x97de, PDF_CMAP_SINGLE, 9318 },
	{ 0x97df, 0x97e5, PDF_CMAP_RANGE, 20813 },
	{ 0x97e6, 0x97e6, PDF_CMAP_SINGLE, 3763 },
	{ 0x97e7, 0x97e7, PDF_CMAP_SINGLE, 3240 },
	{ 0x97e8, 0x97e8, PDF_CMAP_SINGLE, 20820 },
	{ 0x97e9, 0x97e9, PDF_CMAP_SINGLE, 1890 },
	{ 0x97ea, 0x97ec, PDF_CMAP_RANGE, 6222 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 2289 },
	{ 0x97ee, 0x97f2, PDF_CMAP_RANGE, 20821 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 4219 },
	{ 0x97f4, 0x97f4, PDF_CMAP_SINGLE, 20826 },
	{ 0x97f5, 0x97f5, PDF_CMAP_SINGLE, 4370 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 3345 },
	{ 0x97f7, 0x97fe, PDF_CMAP_RANGE, 20827 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 8649 },
	{ 0x9800, 0x9800, PDF_CMAP_SINGLE, 20835 },
	{ 0x9801, 0x9801, PDF_CMAP_SINGLE, 8713 },
	{ 0x9802, 0x9802, PDF_CMAP_SINGLE, 7881 },
	{ 0x9803, 0x9803, PDF_CMAP_SINGLE, 8425 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 20836 },
	{ 0x9805, 0x9805, PDF_CMAP_SINGLE, 8650 },
	{ 0x9806, 0x9806, PDF_CMAP_SINGLE, 8518 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 9669 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 8668 },
	{ 0x9809, 0x9809, PDF_CMAP_SINGLE, 20837 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 9310 },
	{ 0x980b, 0x980b, PDF_CMAP_SINGLE, 20838 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 8527 },
	{ 0x980d, 0x980d, PDF_CMAP_SINGLE, 20839 },
	{ 0x980e, 0x980f, PDF_CMAP_RANGE, 9670 },
	{ 0x9810, 0x9810, PDF_CMAP_SINGLE, 8769 },
	{ 0x9811, 0x9811, PDF_CMAP_SINGLE, 8586 },
	{ 0x9812, 0x9812, PDF_CMAP_SINGLE, 7727 },
	{ 0x9813, 0x9813, PDF_CMAP_SINGLE, 7900 },
	{ 0x9814, 0x9816, PDF_CMAP_RANGE, 20840 },
	{ 0x9817, 0x9817, PDF_CMAP_SINGLE, 8383 },
	{ 0x9818, 0x9818, PDF_CMAP_SINGLE, 8244 },
	{ 0x9819, 0x981b, PDF_CMAP_RANGE, 20843 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 9673 },
	{ 0x981d, 0x9820, PDF_CMAP_RANGE, 20846 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 9672 },
	{ 0x9822, 0x9823, PDF_CMAP_RANGE, 20850 },
	{ 0x9824, 0x9824, PDF_CMAP_SINGLE, 8718 },
	{ 0x9825, 0x9825, PDF_CMAP_SINGLE, 20852 },
	{ 0x9826, 0x9826, PDF_CMAP_SINGLE, 9675 },
	{ 0x9827, 0x982c, PDF_CMAP_RANGE, 20853 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 8573 },
	{ 0x982e, 0x982f, PDF_CMAP_RANGE, 20859 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 8062 },
	{ 0x9831, 0x9836, PDF_CMAP_RANGE, 20861 },
	{ 0x9837, 0x9837, PDF_CMAP_SINGLE, 9676 },
	{ 0x9838, 0x9838, PDF_CMAP_SINGLE, 8131 },
	{ 0x9839, 0x983a, PDF_CMAP_RANGE, 20867 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 8377 },
	{ 0x983c, 0x983c, PDF_CMAP_SINGLE, 20869 },
	{ 0x983d, 0x983d, PDF_CMAP_SINGLE, 8577 },
	{ 0x983e, 0x9845, PDF_CMAP_RANGE, 20870 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 8154 },
	{ 0x9847, 0x984b, PDF_CMAP_RANGE, 20878 },
	{ 0x984c, 0x984c, PDF_CMAP_SINGLE, 8563 },
	{ 0x984d, 0x984d, PDF_CMAP_SINGLE, 7905 },
	{ 0x984e, 0x984e, PDF_CMAP_SINGLE, 9677 },
	{ 0x984f, 0x9852, PDF_CMAP_RANGE, 20883 },
	{ 0x9853, 0x9853, PDF_CMAP_SINGLE, 9678 },
	{ 0x9854, 0x9854, PDF_CMAP_SINGLE, 8694 },
	{ 0x9855, 0x9857, PDF_CMAP_RANGE, 20887 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 8779 },
	{ 0x9859, 0x9859, PDF_CMAP_SINGLE, 9681 },
	{ 0x985a, 0x985a, PDF_CMAP_SINGLE, 20890 },
	{ 0x985b, 0x985b, PDF_CMAP_SINGLE, 7872 },
	{ 0x985c, 0x985d, PDF_CMAP_RANGE, 20891 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 8200 },
	{ 0x985f, 0x9861, PDF_CMAP_RANGE, 20893 },
	{ 0x9862, 0x9862, PDF_CMAP_SINGLE, 9680 },
	{ 0x9863, 0x9864, PDF_CMAP_RANGE, 20896 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 9682 },
	{ 0x9866, 0x9866, PDF_CMAP_SINGLE, 20898 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 7978 },
	{ 0x9868, 0x986a, PDF_CMAP_RANGE, 20899 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 7794 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 9683 },
	{ 0x986d, 0x986e, PDF_CMAP_RANGE, 20902 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 8638 },
	{ 0x9870, 0x9870, PDF_CMAP_SINGLE, 9684 },
	{ 0x9871, 0x9871, PDF_CMAP_SINGLE, 8260 },
	{ 0x9872, 0x9872, PDF_CMAP_SINGLE, 20904 },
	{ 0x9873, 0x9873, PDF_CMAP_SINGLE, 9679 },
	{ 0x9874, 0x9874, PDF_CMAP_SINGLE, 8435 },
	{ 0x9875, 0x9875, PDF_CMAP_SINGLE, 4154 },
	{ 0x9876, 0x9876, PDF_CMAP_SINGLE, 1508 },
	{ 0x9877, 0x9877, PDF_CMAP_SINGLE, 3175 },
	{ 0x9878, 0x9878, PDF_CMAP_SINGLE, 7136 },
	{ 0x9879, 0x9879, PDF_CMAP_SINGLE, 3931 },
	{ 0x987a, 0x987a, PDF_CMAP_SINGLE, 3496 },
	{ 0x987b, 0x987b, PDF_CMAP_SINGLE, 4022 },
	{ 0x987c, 0x987c, PDF_CMAP_SINGLE, 6183 },
	{ 0x987d, 0x987d, PDF_CMAP_SINGLE, 3736 },
	{ 0x987e, 0x987e, PDF_CMAP_SINGLE, 1828 },
	{ 0x987f, 0x987f, PDF_CMAP_SINGLE, 1560 },
	{ 0x9880, 0x9880, PDF_CMAP_SINGLE, 7137 },
	{ 0x9881, 0x9881, PDF_CMAP_SINGLE, 1007 },
	{ 0x9882, 0x9882, PDF_CMAP_SINGLE, 3521 },
	{ 0x9883, 0x9883, PDF_CMAP_SINGLE, 7138 },
	{ 0x9884, 0x9884, PDF_CMAP_SINGLE, 4327 },
	{ 0x9885, 0x9885, PDF_CMAP_SINGLE, 2644 },
	{ 0x9886, 0x9886, PDF_CMAP_SINGLE, 2613 },
	{ 0x9887, 0x9887, PDF_CMAP_SINGLE, 3043 },
	{ 0x9888, 0x9888, PDF_CMAP_SINGLE, 2272 },
	{ 0x9889, 0x9889, PDF_CMAP_SINGLE, 7139 },
	{ 0x988a, 0x988a, PDF_CMAP_SINGLE, 2120 },
	{ 0x988b, 0x988b, PDF_CMAP_SINGLE, 20905 },
	{ 0x988c, 0x988d, PDF_CMAP_RANGE, 7140 },
	{ 0x988e, 0x988e, PDF_CMAP_SINGLE, 20906 },
	{ 0x988f, 0x988f, PDF_CMAP_SINGLE, 7142 },
	{ 0x9890, 0x9890, PDF_CMAP_SINGLE, 4170 },
	{ 0x9891, 0x9891, PDF_CMAP_SINGLE, 3028 },
	{ 0x9892, 0x9892, PDF_CMAP_SINGLE, 20907 },
	{ 0x9893, 0x9893, PDF_CMAP_SINGLE, 3704 },
	{ 0x9894, 0x9894, PDF_CMAP_SINGLE, 7143 },
	{ 0x9895, 0x9895, PDF_CMAP_SINGLE, 20908 },
	{ 0x9896, 0x9896, PDF_CMAP_SINGLE, 4246 },
	{ 0x9897, 0x9897, PDF_CMAP_SINGLE, 2386 },
	{ 0x9898, 0x9898, PDF_CMAP_SINGLE, 3637 },
	{ 0x9899, 0x9899, PDF_CMAP_SINGLE, 20909 },
	{ 0x989a, 0x989b, PDF_CMAP_RANGE, 7144 },
	{ 0x989c, 0x989c, PDF_CMAP_SINGLE, 4094 },
	{ 0x989d, 0x989d, PDF_CMAP_SINGLE, 1581 },
	{ 0x989e, 0x989f, PDF_CMAP_RANGE, 7146 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 1472 },
	{ 0x98a1, 0x98a2, PDF_CMAP_RANGE, 7148 },
	{ 0x98a3, 0x98a3, PDF_CMAP_SINGLE, 20910 },
	{ 0x98a4, 0x98a4, PDF_CMAP_SINGLE, 1219 },
	{ 0x98a5, 0x98a6, PDF_CMAP_RANGE, 7150 },
	{ 0x98a7, 0x98a7, PDF_CMAP_SINGLE, 3202 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 7936 },
	{ 0x98a9, 0x98ad, PDF_CMAP_RANGE, 20911 },
	{ 0x98ae, 0x98af, PDF_CMAP_RANGE, 9420 },
	{ 0x98b0, 0x98b0, PDF_CMAP_SINGLE, 20916 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 9880 },
	{ 0x98b2, 0x98b2, PDF_CMAP_SINGLE, 20917 },
	{ 0x98b3, 0x98b3, PDF_CMAP_SINGLE, 9848 },
	{ 0x98b4, 0x98b5, PDF_CMAP_RANGE, 20918 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 9422 },
	{ 0x98b7, 0x98bb, PDF_CMAP_RANGE, 20920 },
	{ 0x98bc, 0x98bc, PDF_CMAP_SINGLE, 9423 },
	{ 0x98bd, 0x98c3, PDF_CMAP_RANGE, 20925 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 8376 },
	{ 0x98c5, 0x98c5, PDF_CMAP_SINGLE, 20932 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 9424 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 20933 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 9425 },
	{ 0x98c9, 0x98cd, PDF_CMAP_RANGE, 20934 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 1668 },
	{ 0x98cf, 0x98d0, PDF_CMAP_RANGE, 20939 },
	{ 0x98d1, 0x98d3, PDF_CMAP_RANGE, 6583 },
	{ 0x98d4, 0x98d4, PDF_CMAP_SINGLE, 20941 },
	{ 0x98d5, 0x98d5, PDF_CMAP_SINGLE, 6586 },
	{ 0x98d6, 0x98d7, PDF_CMAP_RANGE, 20942 },
	{ 0x98d8, 0x98d8, PDF_CMAP_SINGLE, 3021 },
	{ 0x98d9, 0x98da, PDF_CMAP_RANGE, 6587 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 7924 },
	{ 0x98dc, 0x98dd, PDF_CMAP_RANGE, 20944 },
	{ 0x98de, 0x98de, PDF_CMAP_SINGLE, 1638 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 3402 },
	{ 0x98e0, 0x98e0, PDF_CMAP_SINGLE, 9115 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 20946 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 9854 },
	{ 0x98e3, 0x98e6, PDF_CMAP_RANGE, 20947 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 5658 },
	{ 0x98e8, 0x98e8, PDF_CMAP_SINGLE, 7655 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 9117 },
	{ 0x98ea, 0x98eb, PDF_CMAP_RANGE, 9119 },
	{ 0x98ec, 0x98ec, PDF_CMAP_SINGLE, 20951 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 9121 },
	{ 0x98ee, 0x98ee, PDF_CMAP_SINGLE, 20952 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 7921 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 20953 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 8734 },
	{ 0x98f3, 0x98f3, PDF_CMAP_SINGLE, 20955 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 9122 },
	{ 0x98f5, 0x98fb, PDF_CMAP_RANGE, 20956 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 8523 },
	{ 0x98fd, 0x98fd, PDF_CMAP_SINGLE, 7734 },
	{ 0x98fe, 0x98fe, PDF_CMAP_SINGLE, 8501 },
	{ 0x98ff, 0x98ff, PDF_CMAP_SINGLE, 20963 },
	{ 0x9900, 0x9902, PDF_CMAP_RANGE, 20964 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 8109 },
	{ 0x9904, 0x9904, PDF_CMAP_SINGLE, 20967 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 7763 },
	{ 0x9906, 0x9908, PDF_CMAP_RANGE, 20968 },
	{ 0x9909, 0x9909, PDF_CMAP_SINGLE, 9123 },
	{ 0x990a, 0x990a, PDF_CMAP_SINGLE, 8707 },
	{ 0x990b, 0x990b, PDF_CMAP_SINGLE, 20971 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 7911 },
	{ 0x990d, 0x990d, PDF_CMAP_SINGLE, 7656 },
	{ 0x990e, 0x990f, PDF_CMAP_RANGE, 20972 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 1172 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 9124 },
	{ 0x9912, 0x9912, PDF_CMAP_SINGLE, 8343 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 7908 },
	{ 0x9914, 0x9917, PDF_CMAP_RANGE, 20974 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 8760 },
	{ 0x9919, 0x991a, PDF_CMAP_RANGE, 20978 },
	{ 0x991b, 0x991b, PDF_CMAP_SINGLE, 9125 },
	{ 0x991c, 0x991d, PDF_CMAP_RANGE, 20980 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 8090 },
	{ 0x991f, 0x9920, PDF_CMAP_RANGE, 20982 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 8643 },
	{ 0x9922, 0x9927, PDF_CMAP_RANGE, 20984 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 7982 },
	{ 0x9929, 0x992d, PDF_CMAP_RANGE, 20990 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7657 },
	{ 0x992f, 0x9932, PDF_CMAP_RANGE, 20995 },
	{ 0x9933, 0x9933, PDF_CMAP_SINGLE, 9116 },
	{ 0x9934, 0x9936, PDF_CMAP_RANGE, 20999 },
	{ 0x9937, 0x9937, PDF_CMAP_SINGLE, 9126 },
	{ 0x9938, 0x993b, PDF_CMAP_RANGE, 21002 },
	{ 0x993c, 0x993c, PDF_CMAP_SINGLE, 9118 },
	{ 0x993d, 0x993d, PDF_CMAP_SINGLE, 21006 },
	{ 0x993e, 0x993e, PDF_CMAP_SINGLE, 8245 },
	{ 0x993f, 0x993f, PDF_CMAP_SINGLE, 9127 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 21007 },
	{ 0x9943, 0x9943, PDF_CMAP_SINGLE, 9128 },
	{ 0x9944, 0x9944, PDF_CMAP_SINGLE, 21010 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 8309 },
	{ 0x9946, 0x9947, PDF_CMAP_RANGE, 21011 },
	{ 0x9948, 0x994a, PDF_CMAP_RANGE, 9129 },
	{ 0x994b, 0x994b, PDF_CMAP_SINGLE, 8171 },
	{ 0x994c, 0x994c, PDF_CMAP_SINGLE, 9132 },
	{ 0x994d, 0x9950, PDF_CMAP_RANGE, 21013 },
	{ 0x9951, 0x9951, PDF_CMAP_SINGLE, 8042 },
	{ 0x9952, 0x9952, PDF_CMAP_SINGLE, 8441 },
	{ 0x9953, 0x9953, PDF_CMAP_SINGLE, 21017 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 7659 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 7658 },
	{ 0x9956, 0x9956, PDF_CMAP_SINGLE, 21018 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 9830 },
	{ 0x9958, 0x995b, PDF_CMAP_RANGE, 21019 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 9831 },
	{ 0x995d, 0x995d, PDF_CMAP_SINGLE, 21023 },
	{ 0x995e, 0x995e, PDF_CMAP_SINGLE, 7788 },
	{ 0x995f, 0x9961, PDF_CMAP_RANGE, 21024 },
	{ 0x9962, 0x9962, PDF_CMAP_SINGLE, 9133 },
	{ 0x9963, 0x9963, PDF_CMAP_SINGLE, 5661 },
	{ 0x9964, 0x9964, PDF_CMAP_SINGLE, 21027 },
	{ 0x9965, 0x9965, PDF_CMAP_SINGLE, 2069 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 21028 },
	{ 0x9967, 0x996c, PDF_CMAP_RANGE, 5662 },
	{ 0x996d, 0x996d, PDF_CMAP_SINGLE, 1622 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 4226 },
	{ 0x996f, 0x996f, PDF_CMAP_SINGLE, 2165 },
	{ 0x9970, 0x9970, PDF_CMAP_SINGLE, 3429 },
	{ 0x9971, 0x9971, PDF_CMAP_SINGLE, 1038 },
	{ 0x9972, 0x9972, PDF_CMAP_SINGLE, 3516 },
	{ 0x9973, 0x9973, PDF_CMAP_SINGLE, 21029 },
	{ 0x9974, 0x9974, PDF_CMAP_SINGLE, 5668 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 1595 },
	{ 0x9976, 0x9976, PDF_CMAP_SINGLE, 3231 },
	{ 0x9977, 0x9977, PDF_CMAP_SINGLE, 5669 },
	{ 0x9978, 0x9979, PDF_CMAP_RANGE, 21030 },
	{ 0x997a, 0x997a, PDF_CMAP_SINGLE, 2201 },
	{ 0x997b, 0x997b, PDF_CMAP_SINGLE, 21032 },
	{ 0x997c, 0x997c, PDF_CMAP_SINGLE, 1126 },
	{ 0x997d, 0x997d, PDF_CMAP_SINGLE, 5670 },
	{ 0x997e, 0x997e, PDF_CMAP_SINGLE, 21033 },
	{ 0x997f, 0x997f, PDF_CMAP_SINGLE, 1589 },
	{ 0x9980, 0x9980, PDF_CMAP_SINGLE, 5671 },
	{ 0x9981, 0x9981, PDF_CMAP_SINGLE, 2876 },
	{ 0x9982, 0x9983, PDF_CMAP_RANGE, 21034 },
	{ 0x9984, 0x9984, PDF_CMAP_SINGLE, 5672 },
	{ 0x9985, 0x9985, PDF_CMAP_SINGLE, 3911 },
	{ 0x9986, 0x9986, PDF_CMAP_SINGLE, 1846 },
	{ 0x9987, 0x9987, PDF_CMAP_SINGLE, 5673 },
	{ 0x9988, 0x9988, PDF_CMAP_SINGLE, 2444 },
	{ 0x9989, 0x9989, PDF_CMAP_SINGLE, 21036 },
	{ 0x998a, 0x998a, PDF_CMAP_SINGLE, 5674 },
	{ 0x998b, 0x998b, PDF_CMAP_SINGLE, 1213 },
	{ 0x998c, 0x998c, PDF_CMAP_SINGLE, 21037 },
	{ 0x998d, 0x998d, PDF_CMAP_SINGLE, 5675 },
	{ 0x998e, 0x998e, PDF_CMAP_SINGLE, 21038 },
	{ 0x998f, 0x998f, PDF_CMAP_SINGLE, 2620 },
	{ 0x9990, 0x9991, PDF_CMAP_RANGE, 5676 },
	{ 0x9992, 0x9992, PDF_CMAP_SINGLE, 2719 },
	{ 0x9993, 0x9995, PDF_CMAP_RANGE, 5678 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 3438 },
	{ 0x9997, 0x9997, PDF_CMAP_SINGLE, 4718 },
	{ 0x9998, 0x9998, PDF_CMAP_SINGLE, 4857 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 3920 },
	{ 0x999a, 0x99a4, PDF_CMAP_RANGE, 21039 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 6955 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 21050 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5086 },
	{ 0x99a9, 0x99ab, PDF_CMAP_RANGE, 21052 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 8301 },
	{ 0x99ad, 0x99ad, PDF_CMAP_SINGLE, 8770 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 7938 },
	{ 0x99af, 0x99b0, PDF_CMAP_RANGE, 21055 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 8579 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 21057 },
	{ 0x99b3, 0x99b3, PDF_CMAP_SINGLE, 7813 },
	{ 0x99b4, 0x99b4, PDF_CMAP_SINGLE, 8681 },
	{ 0x99b5, 0x99c0, PDF_CMAP_RANGE, 21058 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 7767 },
	{ 0x99c2, 0x99cf, PDF_CMAP_RANGE, 21070 },
	{ 0x99d0, 0x99d0, PDF_CMAP_SINGLE, 8869 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 9223 },
	{ 0x99d2, 0x99d2, PDF_CMAP_SINGLE, 8138 },
	{ 0x99d3, 0x99d3, PDF_CMAP_SINGLE, 21084 },
	{ 0x99d4, 0x99d4, PDF_CMAP_SINGLE, 9218 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 8066 },
	{ 0x99d6, 0x99d7, PDF_CMAP_RANGE, 21085 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 9224 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 9220 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 21087 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 8497 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 21088 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 8580 },
	{ 0x99de, 0x99de, PDF_CMAP_SINGLE, 21089 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 9219 },
	{ 0x99e0, 0x99e0, PDF_CMAP_SINGLE, 21090 },
	{ 0x99e1, 0x99e1, PDF_CMAP_SINGLE, 8302 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 9227 },
	{ 0x99e3, 0x99ec, PDF_CMAP_RANGE, 21091 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 7999 },
	{ 0x99ee, 0x99f0, PDF_CMAP_RANGE, 21101 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 8295 },
	{ 0x99f2, 0x99fe, PDF_CMAP_RANGE, 21104 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 8151 },
	{ 0x9a00, 0x9a00, PDF_CMAP_SINGLE, 21117 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 7811 },
	{ 0x9a02, 0x9a04, PDF_CMAP_RANGE, 21118 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 9231 },
	{ 0x9a06, 0x9a0c, PDF_CMAP_RANGE, 21121 },
	{ 0x9a0d, 0x9a0d, PDF_CMAP_SINGLE, 9230 },
	{ 0x9a0e, 0x9a0e, PDF_CMAP_SINGLE, 8391 },
	{ 0x9a0f, 0x9a0f, PDF_CMAP_SINGLE, 9229 },
	{ 0x9a10, 0x9a15, PDF_CMAP_RANGE, 21128 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 9234 },
	{ 0x9a17, 0x9a18, PDF_CMAP_RANGE, 21134 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 8375 },
	{ 0x9a1a, 0x9a2a, PDF_CMAP_RANGE, 21136 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 9200 },
	{ 0x9a2c, 0x9a2c, PDF_CMAP_SINGLE, 21153 },
	{ 0x9a2d, 0x9a2d, PDF_CMAP_SINGLE, 9233 },
	{ 0x9a2e, 0x9a2e, PDF_CMAP_SINGLE, 9236 },
	{ 0x9a2f, 0x9a2f, PDF_CMAP_SINGLE, 21154 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 8560 },
	{ 0x9a31, 0x9a35, PDF_CMAP_RANGE, 21155 },
	{ 0x9a36, 0x9a36, PDF_CMAP_SINGLE, 9221 },
	{ 0x9a37, 0x9a37, PDF_CMAP_SINGLE, 8460 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 9237 },
	{ 0x9a39, 0x9a3d, PDF_CMAP_RANGE, 21160 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 8294 },
	{ 0x9a3f, 0x9a3f, PDF_CMAP_SINGLE, 21165 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 9042 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 9235 },
	{ 0x9a42, 0x9a42, PDF_CMAP_SINGLE, 9232 },
	{ 0x9a43, 0x9a44, PDF_CMAP_RANGE, 9238 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 8433 },
	{ 0x9a46, 0x9a49, PDF_CMAP_RANGE, 21166 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 9226 },
	{ 0x9a4b, 0x9a4c, PDF_CMAP_RANGE, 21170 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 9225 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 21172 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 9240 },
	{ 0x9a50, 0x9a54, PDF_CMAP_RANGE, 21173 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 8103 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 21178 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 8700 },
	{ 0x9a58, 0x9a59, PDF_CMAP_RANGE, 21179 },
	{ 0x9a5a, 0x9a5a, PDF_CMAP_SINGLE, 8129 },
	{ 0x9a5b, 0x9a5b, PDF_CMAP_SINGLE, 9222 },
	{ 0x9a5c, 0x9a5e, PDF_CMAP_RANGE, 21181 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 8860 },
	{ 0x9a60, 0x9a61, PDF_CMAP_RANGE, 21184 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 8270 },
	{ 0x9a63, 0x9a63, PDF_CMAP_SINGLE, 21186 },
	{ 0x9a64, 0x9a64, PDF_CMAP_SINGLE, 9242 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 9241 },
	{ 0x9a66, 0x9a69, PDF_CMAP_RANGE, 21187 },
	{ 0x9a6a, 0x9a6a, PDF_CMAP_SINGLE, 9228 },
	{ 0x9a6b, 0x9a6b, PDF_CMAP_SINGLE, 21191 },
	{ 0x9a6c, 0x9a6c, PDF_CMAP_SINGLE, 2708 },
	{ 0x9a6d, 0x9a6d, PDF_CMAP_SINGLE, 4329 },
	{ 0x9a6e, 0x9a6e, PDF_CMAP_SINGLE, 3717 },
	{ 0x9a6f, 0x9a6f, PDF_CMAP_SINGLE, 4058 },
	{ 0x9a70, 0x9a70, PDF_CMAP_SINGLE, 1280 },
	{ 0x9a71, 0x9a71, PDF_CMAP_SINGLE, 3194 },
	{ 0x9a72, 0x9a72, PDF_CMAP_SINGLE, 21192 },
	{ 0x9a73, 0x9a73, PDF_CMAP_SINGLE, 1148 },
	{ 0x9a74, 0x9a74, PDF_CMAP_SINGLE, 2662 },
	{ 0x9a75, 0x9a75, PDF_CMAP_SINGLE, 6074 },
	{ 0x9a76, 0x9a76, PDF_CMAP_SINGLE, 3410 },
	{ 0x9a77, 0x9a78, PDF_CMAP_RANGE, 6075 },
	{ 0x9a79, 0x9a79, PDF_CMAP_SINGLE, 2307 },
	{ 0x9a7a, 0x9a7a, PDF_CMAP_SINGLE, 6077 },
	{ 0x9a7b, 0x9a7b, PDF_CMAP_SINGLE, 4609 },
	{ 0x9a7c, 0x9a7c, PDF_CMAP_SINGLE, 3718 },
	{ 0x9a7d, 0x9a7d, PDF_CMAP_SINGLE, 6079 },
	{ 0x9a7e, 0x9a7e, PDF_CMAP_SINGLE, 2128 },
	{ 0x9a7f, 0x9a7f, PDF_CMAP_SINGLE, 6078 },
	{ 0x9a80, 0x9a81, PDF_CMAP_RANGE, 6080 },
	{ 0x9a82, 0x9a82, PDF_CMAP_SINGLE, 2709 },
	{ 0x9a83, 0x9a83, PDF_CMAP_SINGLE, 21193 },
	{ 0x9a84, 0x9a84, PDF_CMAP_SINGLE, 2191 },
	{ 0x9a85, 0x9a85, PDF_CMAP_SINGLE, 6082 },
	{ 0x9a86, 0x9a86, PDF_CMAP_SINGLE, 2701 },
	{ 0x9a87, 0x9a87, PDF_CMAP_SINGLE, 1886 },
	{ 0x9a88, 0x9a88, PDF_CMAP_SINGLE, 6083 },
	{ 0x9a89, 0x9a89, PDF_CMAP_SINGLE, 21194 },
	{ 0x9a8a, 0x9a8a, PDF_CMAP_SINGLE, 6084 },
	{ 0x9a8b, 0x9a8b, PDF_CMAP_SINGLE, 1271 },
	{ 0x9a8c, 0x9a8c, PDF_CMAP_SINGLE, 4114 },
	{ 0x9a8d, 0x9a8e, PDF_CMAP_RANGE, 21195 },
	{ 0x9a8f, 0x9a8f, PDF_CMAP_SINGLE, 2354 },
	{ 0x9a90, 0x9a90, PDF_CMAP_SINGLE, 6085 },
	{ 0x9a91, 0x9a91, PDF_CMAP_SINGLE, 3086 },
	{ 0x9a92, 0x9a93, PDF_CMAP_RANGE, 6086 },
	{ 0x9a94, 0x9a95, PDF_CMAP_RANGE, 21197 },
	{ 0x9a96, 0x9a96, PDF_CMAP_SINGLE, 6088 },
	{ 0x9a97, 0x9a97, PDF_CMAP_SINGLE, 3020 },
	{ 0x9a98, 0x9a98, PDF_CMAP_SINGLE, 6089 },
	{ 0x9a99, 0x9a99, PDF_CMAP_SINGLE, 21199 },
	{ 0x9a9a, 0x9a9a, PDF_CMAP_SINGLE, 3296 },
	{ 0x9a9b, 0x9a9d, PDF_CMAP_RANGE, 6090 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 5941 },
	{ 0x9a9f, 0x9aa0, PDF_CMAP_RANGE, 6093 },
	{ 0x9aa1, 0x9aa1, PDF_CMAP_SINGLE, 2697 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_RANGE, 6095 },
	{ 0x9aa4, 0x9aa4, PDF_CMAP_SINGLE, 4583 },
	{ 0x9aa5, 0x9aa5, PDF_CMAP_SINGLE, 6097 },
	{ 0x9aa6, 0x9aa6, PDF_CMAP_SINGLE, 21200 },
	{ 0x9aa7, 0x9aa7, PDF_CMAP_SINGLE, 6098 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 1824 },
	{ 0x9aa9, 0x9aae, PDF_CMAP_RANGE, 21201 },
	{ 0x9aaf, 0x9aaf, PDF_CMAP_SINGLE, 7721 },
	{ 0x9ab0, 0x9ab0, PDF_CMAP_SINGLE, 7635 },
	{ 0x9ab1, 0x9ab1, PDF_CMAP_SINGLE, 7634 },
	{ 0x9ab2, 0x9ab5, PDF_CMAP_RANGE, 21207 },
	{ 0x9ab6, 0x9ab6, PDF_CMAP_SINGLE, 7638 },
	{ 0x9ab7, 0x9ab7, PDF_CMAP_SINGLE, 7636 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 1880 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 21211 },
	{ 0x9aba, 0x9aba, PDF_CMAP_SINGLE, 7639 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 21212 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 7640 },
	{ 0x9abd, 0x9abf, PDF_CMAP_RANGE, 21213 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 7642 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 7641 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 7644 },
	{ 0x9ac3, 0x9ac4, PDF_CMAP_RANGE, 21216 },
	{ 0x9ac5, 0x9ac5, PDF_CMAP_SINGLE, 7643 },
	{ 0x9ac6, 0x9aca, PDF_CMAP_RANGE, 21218 },
	{ 0x9acb, 0x9acc, PDF_CMAP_RANGE, 7645 },
	{ 0x9acd, 0x9ace, PDF_CMAP_RANGE, 21223 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 9825 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 21225 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 7647 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 9892 },
	{ 0x9ad3, 0x9ad3, PDF_CMAP_SINGLE, 3549 },
	{ 0x9ad4, 0x9ad4, PDF_CMAP_SINGLE, 8564 },
	{ 0x9ad5, 0x9ad5, PDF_CMAP_SINGLE, 9827 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 9826 },
	{ 0x9ad7, 0x9ad7, PDF_CMAP_SINGLE, 21226 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 1754 },
	{ 0x9ad9, 0x9ade, PDF_CMAP_RANGE, 21227 },
	{ 0x9adf, 0x9adf, PDF_CMAP_SINGLE, 7660 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 21233 },
	{ 0x9ae1, 0x9ae1, PDF_CMAP_SINGLE, 7661 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_RANGE, 21234 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7662 },
	{ 0x9ae7, 0x9aea, PDF_CMAP_RANGE, 21238 },
	{ 0x9aeb, 0x9aeb, PDF_CMAP_SINGLE, 7664 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 21242 },
	{ 0x9aed, 0x9aed, PDF_CMAP_SINGLE, 7666 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 9845 },
	{ 0x9aef, 0x9aef, PDF_CMAP_SINGLE, 7663 },
	{ 0x9af0, 0x9af8, PDF_CMAP_RANGE, 21243 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 7667 },
	{ 0x9afa, 0x9afa, PDF_CMAP_SINGLE, 21252 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 7665 },
	{ 0x9afc, 0x9aff, PDF_CMAP_RANGE, 21253 },
	{ 0x9b00, 0x9b02, PDF_CMAP_RANGE, 21257 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 4660 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 21260 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 8524 },
	{ 0x9b07, 0x9b07, PDF_CMAP_SINGLE, 21262 },
	{ 0x9b08, 0x9b08, PDF_CMAP_SINGLE, 7668 },
	{ 0x9b09, 0x9b0c, PDF_CMAP_RANGE, 21263 },
	{ 0x9b0d, 0x9b0d, PDF_CMAP_SINGLE, 9850 },
	{ 0x9b0e, 0x9b0e, PDF_CMAP_SINGLE, 21267 },
	{ 0x9b0f, 0x9b0f, PDF_CMAP_SINGLE, 7669 },
	{ 0x9b10, 0x9b12, PDF_CMAP_RANGE, 21268 },
	{ 0x9b13, 0x9b13, PDF_CMAP_SINGLE, 7670 },
	{ 0x9b14, 0x9b19, PDF_CMAP_RANGE, 21271 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 9888 },
	{ 0x9b1b, 0x9b1e, PDF_CMAP_RANGE, 21277 },
	{ 0x9b1f, 0x9b1f, PDF_CMAP_SINGLE, 7671 },
	{ 0x9b20, 0x9b21, PDF_CMAP_RANGE, 21281 },
	{ 0x9b22, 0x9b22, PDF_CMAP_SINGLE, 9832 },
	{ 0x9b23, 0x9b23, PDF_CMAP_SINGLE, 7672 },
	{ 0x9b24, 0x9b24, PDF_CMAP_SINGLE, 21283 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 7888 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 21284 },
	{ 0x9b27, 0x9b27, PDF_CMAP_SINGLE, 8342 },
	{ 0x9b28, 0x9b28, PDF_CMAP_SINGLE, 21285 },
	{ 0x9b29, 0x9b29, PDF_CMAP_SINGLE, 9164 },
	{ 0x9b2a, 0x9b2d, PDF_CMAP_RANGE, 21286 },
	{ 0x9b2e, 0x9b2e, PDF_CMAP_SINGLE, 9160 },
	{ 0x9b2f, 0x9b2f, PDF_CMAP_SINGLE, 5019 },
	{ 0x9b30, 0x9b30, PDF_CMAP_SINGLE, 21290 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 8766 },
	{ 0x9b32, 0x9b32, PDF_CMAP_SINGLE, 4704 },
	{ 0x9b33, 0x9b3a, PDF_CMAP_RANGE, 21291 },
	{ 0x9b3b, 0x9b3b, PDF_CMAP_SINGLE, 6003 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 1862 },
	{ 0x9b3d, 0x9b40, PDF_CMAP_RANGE, 21299 },
	{ 0x9b41, 0x9b41, PDF_CMAP_SINGLE, 2442 },
	{ 0x9b42, 0x9b42, PDF_CMAP_SINGLE, 2047 },
	{ 0x9b43, 0x9b43, PDF_CMAP_SINGLE, 7649 },
	{ 0x9b44, 0x9b44, PDF_CMAP_SINGLE, 3046 },
	{ 0x9b45, 0x9b45, PDF_CMAP_SINGLE, 7648 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 21303 },
	{ 0x9b47, 0x9b47, PDF_CMAP_SINGLE, 7650 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 7652 },
	{ 0x9b49, 0x9b49, PDF_CMAP_SINGLE, 7651 },
	{ 0x9b4a, 0x9b4c, PDF_CMAP_RANGE, 21304 },
	{ 0x9b4d, 0x9b4d, PDF_CMAP_SINGLE, 7653 },
	{ 0x9b4e, 0x9b4e, PDF_CMAP_SINGLE, 9829 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 3785 },
	{ 0x9b50, 0x9b50, PDF_CMAP_SINGLE, 21307 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 7654 },
	{ 0x9b52, 0x9b53, PDF_CMAP_RANGE, 21308 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 2826 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 21310 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 9828 },
	{ 0x9b59, 0x9b59, PDF_CMAP_SINGLE, 21313 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 8761 },
	{ 0x9b5b, 0x9b6e, PDF_CMAP_RANGE, 21314 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 8266 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 21334 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 9759 },
	{ 0x9b75, 0x9b76, PDF_CMAP_RANGE, 21338 },
	{ 0x9b77, 0x9b77, PDF_CMAP_SINGLE, 9758 },
	{ 0x9b78, 0x9b80, PDF_CMAP_RANGE, 21340 },
	{ 0x9b81, 0x9b81, PDF_CMAP_SINGLE, 9760 },
	{ 0x9b82, 0x9b82, PDF_CMAP_SINGLE, 21349 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 9761 },
	{ 0x9b84, 0x9b8d, PDF_CMAP_RANGE, 21350 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 9762 },
	{ 0x9b8f, 0x9b8f, PDF_CMAP_SINGLE, 21360 },
	{ 0x9b90, 0x9b90, PDF_CMAP_SINGLE, 9767 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 7737 },
	{ 0x9b92, 0x9b92, PDF_CMAP_SINGLE, 9765 },
	{ 0x9b93, 0x9b99, PDF_CMAP_RANGE, 21361 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 9769 },
	{ 0x9b9b, 0x9b9c, PDF_CMAP_RANGE, 21368 },
	{ 0x9b9d, 0x9b9d, PDF_CMAP_SINGLE, 9774 },
	{ 0x9b9e, 0x9b9e, PDF_CMAP_SINGLE, 9771 },
	{ 0x9b9f, 0x9ba9, PDF_CMAP_RANGE, 21370 },
	{ 0x9baa, 0x9baa, PDF_CMAP_SINGLE, 9770 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 9773 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 21381 },
	{ 0x9bad, 0x9bad, PDF_CMAP_SINGLE, 9768 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 8633 },
	{ 0x9baf, 0x9bbf, PDF_CMAP_RANGE, 21382 },
	{ 0x9bc0, 0x9bc0, PDF_CMAP_SINGLE, 9782 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 9776 },
	{ 0x9bc2, 0x9bc6, PDF_CMAP_RANGE, 21399 },
	{ 0x9bc7, 0x9bc7, PDF_CMAP_SINGLE, 9784 },
	{ 0x9bc8, 0x9bc8, PDF_CMAP_SINGLE, 21404 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 8205 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 9783 },
	{ 0x9bcb, 0x9bd3, PDF_CMAP_RANGE, 21405 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 9797 },
	{ 0x9bd5, 0x9bd5, PDF_CMAP_SINGLE, 21414 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 9786 },
	{ 0x9bd7, 0x9bda, PDF_CMAP_RANGE, 21415 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 9795 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 21419 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 9792 },
	{ 0x9bde, 0x9be0, PDF_CMAP_RANGE, 21420 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 9789 },
	{ 0x9be2, 0x9be2, PDF_CMAP_SINGLE, 9793 },
	{ 0x9be3, 0x9be3, PDF_CMAP_SINGLE, 21423 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 9790 },
	{ 0x9be5, 0x9be6, PDF_CMAP_RANGE, 21424 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 9791 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 8128 },
	{ 0x9be9, 0x9be9, PDF_CMAP_SINGLE, 21426 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 9787 },
	{ 0x9bec, 0x9bef, PDF_CMAP_RANGE, 21427 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 9794 },
	{ 0x9bf1, 0x9bf3, PDF_CMAP_RANGE, 21431 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 9796 },
	{ 0x9bf5, 0x9bfc, PDF_CMAP_RANGE, 21434 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 9785 },
	{ 0x9bfe, 0x9bfe, PDF_CMAP_SINGLE, 21442 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 9804 },
	{ 0x9c00, 0x9c07, PDF_CMAP_RANGE, 21443 },
	{ 0x9c08, 0x9c08, PDF_CMAP_SINGLE, 9799 },
	{ 0x9c09, 0x9c09, PDF_CMAP_SINGLE, 9803 },
	{ 0x9c0a, 0x9c0c, PDF_CMAP_RANGE, 21451 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 9801 },
	{ 0x9c0e, 0x9c0f, PDF_CMAP_RANGE, 21454 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 9800 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 21456 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 9802 },
	{ 0x9c13, 0x9c13, PDF_CMAP_SINGLE, 8456 },
	{ 0x9c14, 0x9c1f, PDF_CMAP_RANGE, 21457 },
	{ 0x9c20, 0x9c20, PDF_CMAP_SINGLE, 9805 },
	{ 0x9c21, 0x9c22, PDF_CMAP_RANGE, 21469 },
	{ 0x9c23, 0x9c23, PDF_CMAP_SINGLE, 9780 },
	{ 0x9c24, 0x9c24, PDF_CMAP_SINGLE, 21471 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 9809 },
	{ 0x9c26, 0x9c27, PDF_CMAP_RANGE, 21472 },
	{ 0x9c28, 0x9c28, PDF_CMAP_SINGLE, 9808 },
	{ 0x9c29, 0x9c29, PDF_CMAP_SINGLE, 9810 },
	{ 0x9c2a, 0x9c2c, PDF_CMAP_RANGE, 21474 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 9807 },
	{ 0x9c2e, 0x9c30, PDF_CMAP_RANGE, 21477 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 9778 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 9806 },
	{ 0x9c33, 0x9c33, PDF_CMAP_SINGLE, 9811 },
	{ 0x9c34, 0x9c34, PDF_CMAP_SINGLE, 21480 },
	{ 0x9c35, 0x9c35, PDF_CMAP_SINGLE, 9815 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 21481 },
	{ 0x9c37, 0x9c37, PDF_CMAP_SINGLE, 9781 },
	{ 0x9c38, 0x9c38, PDF_CMAP_SINGLE, 21482 },
	{ 0x9c39, 0x9c39, PDF_CMAP_SINGLE, 9779 },
	{ 0x9c3a, 0x9c3a, PDF_CMAP_SINGLE, 21483 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 9814 },
	{ 0x9c3c, 0x9c3d, PDF_CMAP_RANGE, 21484 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 9812 },
	{ 0x9c3f, 0x9c44, PDF_CMAP_RANGE, 21486 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 9816 },
	{ 0x9c46, 0x9c47, PDF_CMAP_RANGE, 21492 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 9813 },
	{ 0x9c49, 0x9c49, PDF_CMAP_SINGLE, 7757 },
	{ 0x9c4a, 0x9c51, PDF_CMAP_RANGE, 21494 },
	{ 0x9c52, 0x9c52, PDF_CMAP_SINGLE, 9819 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 21502 },
	{ 0x9c54, 0x9c54, PDF_CMAP_SINGLE, 9818 },
	{ 0x9c55, 0x9c55, PDF_CMAP_SINGLE, 21503 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 9817 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 8238 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 9775 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_RANGE, 21504 },
	{ 0x9c5d, 0x9c5d, PDF_CMAP_SINGLE, 9798 },
	{ 0x9c5e, 0x9c5e, PDF_CMAP_SINGLE, 21508 },
	{ 0x9c5f, 0x9c5f, PDF_CMAP_SINGLE, 9766 },
	{ 0x9c60, 0x9c66, PDF_CMAP_RANGE, 21509 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 9820 },
	{ 0x9c68, 0x9c6c, PDF_CMAP_RANGE, 21516 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 9772 },
	{ 0x9c6e, 0x9c77, PDF_CMAP_RANGE, 21521 },
	{ 0x9c78, 0x9c78, PDF_CMAP_SINGLE, 9763 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 21531 },
	{ 0x9c7a, 0x9c7a, PDF_CMAP_SINGLE, 9777 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 21532 },
	{ 0x9c7c, 0x9c7c, PDF_CMAP_SINGLE, 4296 },
	{ 0x9c7d, 0x9c7e, PDF_CMAP_RANGE, 21533 },
	{ 0x9c7f, 0x9c7f, PDF_CMAP_SINGLE, 7561 },
	{ 0x9c80, 0x9c80, PDF_CMAP_SINGLE, 21535 },
	{ 0x9c81, 0x9c81, PDF_CMAP_SINGLE, 2650 },
	{ 0x9c82, 0x9c82, PDF_CMAP_SINGLE, 7562 },
	{ 0x9c83, 0x9c84, PDF_CMAP_RANGE, 21536 },
	{ 0x9c85, 0x9c88, PDF_CMAP_RANGE, 7563 },
	{ 0x9c89, 0x9c8a, PDF_CMAP_RANGE, 21538 },
	{ 0x9c8b, 0x9c8b, PDF_CMAP_SINGLE, 7568 },
	{ 0x9c8c, 0x9c8c, PDF_CMAP_SINGLE, 21540 },
	{ 0x9c8d, 0x9c8d, PDF_CMAP_SINGLE, 1044 },
	{ 0x9c8e, 0x9c8e, PDF_CMAP_SINGLE, 7569 },
	{ 0x9c8f, 0x9c8f, PDF_CMAP_SINGLE, 21541 },
	{ 0x9c90, 0x9c92, PDF_CMAP_RANGE, 7570 },
	{ 0x9c93, 0x9c93, PDF_CMAP_SINGLE, 21542 },
	{ 0x9c94, 0x9c95, PDF_CMAP_RANGE, 7573 },
	{ 0x9c96, 0x9c99, PDF_CMAP_RANGE, 21543 },
	{ 0x9c9a, 0x9c9b, PDF_CMAP_RANGE, 7575 },
	{ 0x9c9c, 0x9c9c, PDF_CMAP_SINGLE, 3895 },
	{ 0x9c9d, 0x9c9d, PDF_CMAP_SINGLE, 21547 },
	{ 0x9c9e, 0x9ca3, PDF_CMAP_RANGE, 7577 },
	{ 0x9ca4, 0x9ca4, PDF_CMAP_SINGLE, 2523 },
	{ 0x9ca5, 0x9ca9, PDF_CMAP_RANGE, 7583 },
	{ 0x9caa, 0x9caa, PDF_CMAP_SINGLE, 21548 },
	{ 0x9cab, 0x9cab, PDF_CMAP_SINGLE, 7588 },
	{ 0x9cac, 0x9cac, PDF_CMAP_SINGLE, 21549 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 7589 },
	{ 0x9caf, 0x9caf, PDF_CMAP_SINGLE, 21550 },
	{ 0x9cb0, 0x9cb7, PDF_CMAP_RANGE, 7591 },
	{ 0x9cb8, 0x9cb8, PDF_CMAP_SINGLE, 2263 },
	{ 0x9cb9, 0x9cb9, PDF_CMAP_SINGLE, 21551 },
	{ 0x9cba, 0x9cbd, PDF_CMAP_RANGE, 7599 },
	{ 0x9cbe, 0x9cc2, PDF_CMAP_RANGE, 21552 },
	{ 0x9cc3, 0x9cc3, PDF_CMAP_SINGLE, 3285 },
	{ 0x9cc4, 0x9cc7, PDF_CMAP_RANGE, 7603 },
	{ 0x9cc8, 0x9cc9, PDF_CMAP_RANGE, 21557 },
	{ 0x9cca, 0x9cd0, PDF_CMAP_RANGE, 7607 },
	{ 0x9cd1, 0x9cd2, PDF_CMAP_RANGE, 21559 },
	{ 0x9cd3, 0x9cd5, PDF_CMAP_RANGE, 7614 },
	{ 0x9cd6, 0x9cd6, PDF_CMAP_SINGLE, 1111 },
	{ 0x9cd7, 0x9cd9, PDF_CMAP_RANGE, 7617 },
	{ 0x9cda, 0x9cdb, PDF_CMAP_RANGE, 21561 },
	{ 0x9cdc, 0x9cdd, PDF_CMAP_RANGE, 7620 },
	{ 0x9cde, 0x9cde, PDF_CMAP_SINGLE, 2596 },
	{ 0x9cdf, 0x9cdf, PDF_CMAP_SINGLE, 7622 },
	{ 0x9ce0, 0x9ce1, PDF_CMAP_RANGE, 21563 },
	{ 0x9ce2, 0x9ce2, PDF_CMAP_SINGLE, 7623 },
	{ 0x9ce3, 0x9ce4, PDF_CMAP_RANGE, 21565 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 8348 },
	{ 0x9ce6, 0x9ce8, PDF_CMAP_RANGE, 21567 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 9600 },
	{ 0x9cea, 0x9ceb, PDF_CMAP_RANGE, 21570 },
	{ 0x9cec, 0x9cec, PDF_CMAP_SINGLE, 8924 },
	{ 0x9ced, 0x9cf2, PDF_CMAP_RANGE, 21572 },
	{ 0x9cf3, 0x9cf3, PDF_CMAP_SINGLE, 7941 },
	{ 0x9cf4, 0x9cf4, PDF_CMAP_SINGLE, 8331 },
	{ 0x9cf5, 0x9cf5, PDF_CMAP_SINGLE, 21578 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 9601 },
	{ 0x9cf7, 0x9cff, PDF_CMAP_RANGE, 21579 },
	{ 0x9d00, 0x9d05, PDF_CMAP_RANGE, 21588 },
	{ 0x9d06, 0x9d06, PDF_CMAP_SINGLE, 9603 },
	{ 0x9d07, 0x9d07, PDF_CMAP_SINGLE, 9602 },
	{ 0x9d08, 0x9d08, PDF_CMAP_SINGLE, 21594 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 8686 },
	{ 0x9d0a, 0x9d14, PDF_CMAP_RANGE, 21595 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 8578 },
	{ 0x9d16, 0x9d1a, PDF_CMAP_RANGE, 21606 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 8771 },
	{ 0x9d1c, 0x9d1c, PDF_CMAP_SINGLE, 21611 },
	{ 0x9d1d, 0x9d1d, PDF_CMAP_SINGLE, 9607 },
	{ 0x9d1e, 0x9d1e, PDF_CMAP_SINGLE, 21612 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 9608 },
	{ 0x9d20, 0x9d22, PDF_CMAP_RANGE, 21613 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 9604 },
	{ 0x9d24, 0x9d25, PDF_CMAP_RANGE, 21616 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 8701 },
	{ 0x9d27, 0x9d27, PDF_CMAP_SINGLE, 21618 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 8687 },
	{ 0x9d29, 0x9d2e, PDF_CMAP_RANGE, 21619 },
	{ 0x9d2f, 0x9d2f, PDF_CMAP_SINGLE, 9610 },
	{ 0x9d30, 0x9d30, PDF_CMAP_SINGLE, 9612 },
	{ 0x9d31, 0x9d3a, PDF_CMAP_RANGE, 21625 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 8007 },
	{ 0x9d3c, 0x9d3e, PDF_CMAP_RANGE, 21635 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 7965 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 21638 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 9613 },
	{ 0x9d43, 0x9d50, PDF_CMAP_RANGE, 21640 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 8144 },
	{ 0x9d52, 0x9d52, PDF_CMAP_SINGLE, 9618 },
	{ 0x9d53, 0x9d53, PDF_CMAP_SINGLE, 9615 },
	{ 0x9d54, 0x9d5b, PDF_CMAP_RANGE, 21654 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 9620 },
	{ 0x9d5d, 0x9d5d, PDF_CMAP_SINGLE, 7904 },
	{ 0x9d5e, 0x9d5f, PDF_CMAP_RANGE, 21662 },
	{ 0x9d60, 0x9d60, PDF_CMAP_SINGLE, 9617 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 9621 },
	{ 0x9d62, 0x9d69, PDF_CMAP_RANGE, 21664 },
	{ 0x9d6a, 0x9d6a, PDF_CMAP_SINGLE, 9623 },
	{ 0x9d6b, 0x9d6b, PDF_CMAP_SINGLE, 21672 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 8374 },
	{ 0x9d6d, 0x9d6e, PDF_CMAP_RANGE, 21673 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 9624 },
	{ 0x9d70, 0x9d71, PDF_CMAP_RANGE, 21675 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 8438 },
	{ 0x9d73, 0x9d86, PDF_CMAP_RANGE, 21677 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 9605 },
	{ 0x9d88, 0x9d88, PDF_CMAP_SINGLE, 21697 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 9625 },
	{ 0x9d8a, 0x9d92, PDF_CMAP_RANGE, 21698 },
	{ 0x9d93, 0x9d93, PDF_CMAP_SINGLE, 9622 },
	{ 0x9d94, 0x9d97, PDF_CMAP_RANGE, 21707 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 9626 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 21711 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 9627 },
	{ 0x9d9b, 0x9da4, PDF_CMAP_RANGE, 21712 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 9629 },
	{ 0x9da6, 0x9da8, PDF_CMAP_RANGE, 21722 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 9630 },
	{ 0x9daa, 0x9dae, PDF_CMAP_RANGE, 21725 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 9036 },
	{ 0x9db0, 0x9db3, PDF_CMAP_RANGE, 21730 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 8004 },
	{ 0x9db5, 0x9dba, PDF_CMAP_RANGE, 21734 },
	{ 0x9dbb, 0x9dbb, PDF_CMAP_SINGLE, 9824 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 9632 },
	{ 0x9dbd, 0x9dbf, PDF_CMAP_RANGE, 21740 },
	{ 0x9dc0, 0x9dc0, PDF_CMAP_SINGLE, 9628 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 21743 },
	{ 0x9dc2, 0x9dc2, PDF_CMAP_SINGLE, 9631 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 21744 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 8044 },
	{ 0x9dc5, 0x9dd2, PDF_CMAP_RANGE, 21745 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 9634 },
	{ 0x9dd4, 0x9dd6, PDF_CMAP_RANGE, 21759 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 8366 },
	{ 0x9dd8, 0x9dd8, PDF_CMAP_SINGLE, 21762 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 9611 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 9635 },
	{ 0x9ddb, 0x9de4, PDF_CMAP_RANGE, 21763 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 9609 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 9637 },
	{ 0x9de7, 0x9dee, PDF_CMAP_RANGE, 21773 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 9636 },
	{ 0x9df0, 0x9df1, PDF_CMAP_RANGE, 21781 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 9638 },
	{ 0x9df3, 0x9df3, PDF_CMAP_SINGLE, 9619 },
	{ 0x9df4, 0x9df7, PDF_CMAP_RANGE, 21783 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 9639 },
	{ 0x9df9, 0x9df9, PDF_CMAP_SINGLE, 8738 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 9641 },
	{ 0x9dfb, 0x9dff, PDF_CMAP_RANGE, 21787 },
	{ 0x9e00, 0x9e0b, PDF_CMAP_RANGE, 21792 },
	{ 0x9e0c, 0x9e0c, PDF_CMAP_SINGLE, 9640 },
	{ 0x9e0d, 0x9e14, PDF_CMAP_RANGE, 21804 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 9606 },
	{ 0x9e16, 0x9e19, PDF_CMAP_RANGE, 21812 },
	{ 0x9e1a, 0x9e1a, PDF_CMAP_SINGLE, 9633 },
	{ 0x9e1b, 0x9e1b, PDF_CMAP_SINGLE, 9642 },
	{ 0x9e1c, 0x9e1c, PDF_CMAP_SINGLE, 21816 },
	{ 0x9e1d, 0x9e1d, PDF_CMAP_SINGLE, 9616 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 9614 },
	{ 0x9e1f, 0x9e1f, PDF_CMAP_SINGLE, 2900 },
	{ 0x9e20, 0x9e20, PDF_CMAP_SINGLE, 6965 },
	{ 0x9e21, 0x9e21, PDF_CMAP_SINGLE, 2073 },
	{ 0x9e22, 0x9e22, PDF_CMAP_SINGLE, 6966 },
	{ 0x9e23, 0x9e23, PDF_CMAP_SINGLE, 2814 },
	{ 0x9e24, 0x9e24, PDF_CMAP_SINGLE, 21817 },
	{ 0x9e25, 0x9e25, PDF_CMAP_SINGLE, 2937 },
	{ 0x9e26, 0x9e26, PDF_CMAP_SINGLE, 4068 },
	{ 0x9e27, 0x9e27, PDF_CMAP_SINGLE, 21818 },
	{ 0x9e28, 0x9e2c, PDF_CMAP_RANGE, 6967 },
	{ 0x9e2d, 0x9e2d, PDF_CMAP_SINGLE, 4069 },
	{ 0x9e2e, 0x9e2e, PDF_CMAP_SINGLE, 21819 },
	{ 0x9e2f, 0x9e2f, PDF_CMAP_SINGLE, 4117 },
	{ 0x9e30, 0x9e30, PDF_CMAP_SINGLE, 21820 },
	{ 0x9e31, 0x9e31, PDF_CMAP_SINGLE, 6973 },
	{ 0x9e32, 0x9e32, PDF_CMAP_SINGLE, 6972 },
	{ 0x9e33, 0x9e33, PDF_CMAP_SINGLE, 4330 },
	{ 0x9e34, 0x9e34, PDF_CMAP_SINGLE, 21821 },
	{ 0x9e35, 0x9e35, PDF_CMAP_SINGLE, 3715 },
	{ 0x9e36, 0x9e36, PDF_CMAP_SINGLE, 6974 },
	{ 0x9e37, 0x9e37, PDF_CMAP_SINGLE, 6976 },
	{ 0x9e38, 0x9e38, PDF_CMAP_SINGLE, 6975 },
	{ 0x9e39, 0x9e3a, PDF_CMAP_RANGE, 6977 },
	{ 0x9e3b, 0x9e3c, PDF_CMAP_RANGE, 21822 },
	{ 0x9e3d, 0x9e3d, PDF_CMAP_SINGLE, 1766 },
	{ 0x9e3e, 0x9e3e, PDF_CMAP_SINGLE, 6979 },
	{ 0x9e3f, 0x9e3f, PDF_CMAP_SINGLE, 1951 },
	{ 0x9e40, 0x9e40, PDF_CMAP_SINGLE, 21824 },
	{ 0x9e41, 0x9e42, PDF_CMAP_RANGE, 6980 },
	{ 0x9e43, 0x9e43, PDF_CMAP_SINGLE, 2328 },
	{ 0x9e44, 0x9e44, PDF_CMAP_SINGLE, 6982 },
	{ 0x9e45, 0x9e45, PDF_CMAP_SINGLE, 1579 },
	{ 0x9e46, 0x9e49, PDF_CMAP_RANGE, 6983 },
	{ 0x9e4a, 0x9e4a, PDF_CMAP_SINGLE, 3216 },
	{ 0x9e4b, 0x9e4c, PDF_CMAP_RANGE, 6987 },
	{ 0x9e4d, 0x9e4d, PDF_CMAP_SINGLE, 21825 },
	{ 0x9e4e, 0x9e4e, PDF_CMAP_SINGLE, 6989 },
	{ 0x9e4f, 0x9e4f, PDF_CMAP_SINGLE, 2997 },
	{ 0x9e50, 0x9e50, PDF_CMAP_SINGLE, 21826 },
	{ 0x9e51, 0x9e51, PDF_CMAP_SINGLE, 6990 },
	{ 0x9e52, 0x9e54, PDF_CMAP_RANGE, 21827 },
	{ 0x9e55, 0x9e55, PDF_CMAP_SINGLE, 6991 },
	{ 0x9e56, 0x9e56, PDF_CMAP_SINGLE, 21830 },
	{ 0x9e57, 0x9e57, PDF_CMAP_SINGLE, 6992 },
	{ 0x9e58, 0x9e58, PDF_CMAP_SINGLE, 7637 },
	{ 0x9e59, 0x9e59, PDF_CMAP_SINGLE, 21831 },
	{ 0x9e5a, 0x9e5c, PDF_CMAP_RANGE, 6993 },
	{ 0x9e5d, 0x9e5d, PDF_CMAP_SINGLE, 21832 },
	{ 0x9e5e, 0x9e5e, PDF_CMAP_SINGLE, 6996 },
	{ 0x9e5f, 0x9e62, PDF_CMAP_RANGE, 21833 },
	{ 0x9e63, 0x9e63, PDF_CMAP_SINGLE, 6997 },
	{ 0x9e64, 0x9e64, PDF_CMAP_SINGLE, 1934 },
	{ 0x9e65, 0x9e65, PDF_CMAP_SINGLE, 21837 },
	{ 0x9e66, 0x9e6c, PDF_CMAP_RANGE, 6998 },
	{ 0x9e6d, 0x9e6d, PDF_CMAP_SINGLE, 7006 },
	{ 0x9e6e, 0x9e6f, PDF_CMAP_RANGE, 21838 },
	{ 0x9e70, 0x9e70, PDF_CMAP_SINGLE, 4234 },
	{ 0x9e71, 0x9e71, PDF_CMAP_SINGLE, 7005 },
	{ 0x9e72, 0x9e72, PDF_CMAP_SINGLE, 21840 },
	{ 0x9e73, 0x9e73, PDF_CMAP_SINGLE, 7007 },
	{ 0x9e74, 0x9e74, PDF_CMAP_SINGLE, 21841 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 9864 },
	{ 0x9e76, 0x9e78, PDF_CMAP_RANGE, 21842 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 9886 },
	{ 0x9e7a, 0x9e7a, PDF_CMAP_SINGLE, 9723 },
	{ 0x9e7b, 0x9e7b, PDF_CMAP_SINGLE, 21845 },
	{ 0x9e7c, 0x9e7c, PDF_CMAP_SINGLE, 8076 },
	{ 0x9e7d, 0x9e7d, PDF_CMAP_SINGLE, 8692 },
	{ 0x9e7e, 0x9e7e, PDF_CMAP_SINGLE, 7446 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 2656 },
	{ 0x9e80, 0x9e81, PDF_CMAP_RANGE, 21846 },
	{ 0x9e82, 0x9e82, PDF_CMAP_SINGLE, 7676 },
	{ 0x9e83, 0x9e86, PDF_CMAP_RANGE, 21848 },
	{ 0x9e87, 0x9e88, PDF_CMAP_RANGE, 7677 },
	{ 0x9e89, 0x9e8a, PDF_CMAP_RANGE, 21852 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 7679 },
	{ 0x9e8c, 0x9e91, PDF_CMAP_RANGE, 21854 },
	{ 0x9e92, 0x9e92, PDF_CMAP_SINGLE, 7680 },
	{ 0x9e93, 0x9e93, PDF_CMAP_SINGLE, 2651 },
	{ 0x9e94, 0x9e96, PDF_CMAP_RANGE, 21860 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 8207 },
	{ 0x9e98, 0x9e9c, PDF_CMAP_RANGE, 21863 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 7682 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 21868 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 7683 },
	{ 0x9ea0, 0x9ea4, PDF_CMAP_RANGE, 21869 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 8305 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 2714 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 21874 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 9719 },
	{ 0x9eaa, 0x9eae, PDF_CMAP_RANGE, 21876 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 9878 },
	{ 0x9eb0, 0x9eb3, PDF_CMAP_RANGE, 21881 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 7406 },
	{ 0x9eb5, 0x9eb5, PDF_CMAP_SINGLE, 9871 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 21885 },
	{ 0x9eb8, 0x9eb8, PDF_CMAP_SINGLE, 7405 },
	{ 0x9eb9, 0x9eba, PDF_CMAP_RANGE, 21887 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 2704 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 21889 },
	{ 0x9ebd, 0x9ebe, PDF_CMAP_RANGE, 7673 },
	{ 0x9ebf, 0x9ec3, PDF_CMAP_RANGE, 21890 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 2011 },
	{ 0x9ec5, 0x9ec8, PDF_CMAP_RANGE, 21895 },
	{ 0x9ec9, 0x9ec9, PDF_CMAP_SINGLE, 4856 },
	{ 0x9eca, 0x9ecb, PDF_CMAP_RANGE, 21899 },
	{ 0x9ecc, 0x9ecc, PDF_CMAP_SINGLE, 8922 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 3465 },
	{ 0x9ece, 0x9ece, PDF_CMAP_SINGLE, 2515 },
	{ 0x9ecf, 0x9ecf, PDF_CMAP_SINGLE, 6954 },
	{ 0x9ed0, 0x9ed0, PDF_CMAP_SINGLE, 21901 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 1937 },
	{ 0x9ed2, 0x9ed3, PDF_CMAP_RANGE, 21902 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 3114 },
	{ 0x9ed5, 0x9ed7, PDF_CMAP_RANGE, 21904 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 2831 },
	{ 0x9ed9, 0x9eda, PDF_CMAP_RANGE, 21907 },
	{ 0x9edb, 0x9edd, PDF_CMAP_RANGE, 7684 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 7873 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 7688 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 7687 },
	{ 0x9ee1, 0x9ee1, PDF_CMAP_SINGLE, 21909 },
	{ 0x9ee2, 0x9ee2, PDF_CMAP_SINGLE, 7689 },
	{ 0x9ee3, 0x9ee4, PDF_CMAP_RANGE, 21910 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 7692 },
	{ 0x9ee6, 0x9ee6, PDF_CMAP_SINGLE, 21912 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 7691 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 7859 },
	{ 0x9ee9, 0x9ee9, PDF_CMAP_SINGLE, 7690 },
	{ 0x9eea, 0x9eea, PDF_CMAP_SINGLE, 7693 },
	{ 0x9eeb, 0x9eee, PDF_CMAP_RANGE, 21913 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 7694 },
	{ 0x9ef0, 0x9ef1, PDF_CMAP_RANGE, 21917 },
	{ 0x9ef2, 0x9ef2, PDF_CMAP_SINGLE, 9834 },
	{ 0x9ef3, 0x9ef3, PDF_CMAP_SINGLE, 21919 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 9866 },
	{ 0x9ef5, 0x9ef6, PDF_CMAP_RANGE, 21920 },
	{ 0x9ef7, 0x9ef7, PDF_CMAP_SINGLE, 9833 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 21922 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 6741 },
	{ 0x9efa, 0x9efa, PDF_CMAP_SINGLE, 21923 },
	{ 0x9efb, 0x9efc, PDF_CMAP_RANGE, 6742 },
	{ 0x9efd, 0x9efd, PDF_CMAP_SINGLE, 9752 },
	{ 0x9efe, 0x9efe, PDF_CMAP_SINGLE, 7542 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 9753 },
	{ 0x9f00, 0x9f08, PDF_CMAP_RANGE, 21924 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 9754 },
	{ 0x9f0a, 0x9f0a, PDF_CMAP_SINGLE, 21933 },
	{ 0x9f0b, 0x9f0b, PDF_CMAP_SINGLE, 7543 },
	{ 0x9f0c, 0x9f0c, PDF_CMAP_SINGLE, 21934 },
	{ 0x9f0d, 0x9f0d, PDF_CMAP_SINGLE, 7544 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 1509 },
	{ 0x9f0f, 0x9f0f, PDF_CMAP_SINGLE, 21935 },
	{ 0x9f10, 0x9f10, PDF_CMAP_SINGLE, 4724 },
	{ 0x9f11, 0x9f12, PDF_CMAP_RANGE, 21936 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 1821 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 21938 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 9843 },
	{ 0x9f16, 0x9f16, PDF_CMAP_SINGLE, 21939 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 4721 },
	{ 0x9f18, 0x9f18, PDF_CMAP_SINGLE, 21940 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 5087 },
	{ 0x9f1a, 0x9f1f, PDF_CMAP_RANGE, 21941 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 3466 },
	{ 0x9f21, 0x9f21, PDF_CMAP_SINGLE, 21947 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 7695 },
	{ 0x9f23, 0x9f2b, PDF_CMAP_RANGE, 21948 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7696 },
	{ 0x9f2d, 0x9f2e, PDF_CMAP_RANGE, 21957 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 7697 },
	{ 0x9f30, 0x9f36, PDF_CMAP_RANGE, 21959 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 7699 },
	{ 0x9f38, 0x9f38, PDF_CMAP_SINGLE, 21966 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 7698 },
	{ 0x9f3a, 0x9f3a, PDF_CMAP_SINGLE, 21967 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 1072 },
	{ 0x9f3c, 0x9f3c, PDF_CMAP_SINGLE, 21968 },
	{ 0x9f3d, 0x9f3e, PDF_CMAP_RANGE, 7700 },
	{ 0x9f3f, 0x9f43, PDF_CMAP_RANGE, 21969 },
	{ 0x9f44, 0x9f44, PDF_CMAP_SINGLE, 7702 },
	{ 0x9f45, 0x9f49, PDF_CMAP_RANGE, 21974 },
	{ 0x9f4a, 0x9f4a, PDF_CMAP_SINGLE, 8390 },
	{ 0x9f4b, 0x9f4b, PDF_CMAP_SINGLE, 8811 },
	{ 0x9f4c, 0x9f4e, PDF_CMAP_RANGE, 21979 },
	{ 0x9f4f, 0x9f4f, PDF_CMAP_SINGLE, 9427 },
	{ 0x9f50, 0x9f50, PDF_CMAP_SINGLE, 3082 },
	{ 0x9f51, 0x9f51, PDF_CMAP_SINGLE, 6594 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7814 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 21982 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 9743 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 21983 },
	{ 0x9f59, 0x9f59, PDF_CMAP_SINGLE, 9745 },
	{ 0x9f5a, 0x9f5b, PDF_CMAP_RANGE, 21987 },
	{ 0x9f5c, 0x9f5c, PDF_CMAP_SINGLE, 9747 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 21989 },
	{ 0x9f5f, 0x9f5f, PDF_CMAP_SINGLE, 9744 },
	{ 0x9f60, 0x9f60, PDF_CMAP_SINGLE, 9746 },
	{ 0x9f61, 0x9f61, PDF_CMAP_SINGLE, 8240 },
	{ 0x9f62, 0x9f62, PDF_CMAP_SINGLE, 21991 },
	{ 0x9f63, 0x9f63, PDF_CMAP_SINGLE, 9841 },
	{ 0x9f64, 0x9f65, PDF_CMAP_RANGE, 21992 },
	{ 0x9f66, 0x9f66, PDF_CMAP_SINGLE, 9748 },
	{ 0x9f67, 0x9f69, PDF_CMAP_RANGE, 21994 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 9750 },
	{ 0x9f6b, 0x9f6b, PDF_CMAP_SINGLE, 21997 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 9749 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 21998 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 8434 },
	{ 0x9f73, 0x9f76, PDF_CMAP_RANGE, 22003 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 9751 },
	{ 0x9f78, 0x9f7e, PDF_CMAP_RANGE, 22007 },
	{ 0x9f7f, 0x9f7f, PDF_CMAP_SINGLE, 1282 },
	{ 0x9f80, 0x9f80, PDF_CMAP_SINGLE, 7533 },
	{ 0x9f81, 0x9f82, PDF_CMAP_RANGE, 22014 },
	{ 0x9f83, 0x9f83, PDF_CMAP_SINGLE, 7534 },
	{ 0x9f84, 0x9f84, PDF_CMAP_SINGLE, 2605 },
	{ 0x9f85, 0x9f8a, PDF_CMAP_RANGE, 7535 },
	{ 0x9f8b, 0x9f8b, PDF_CMAP_SINGLE, 3198 },
	{ 0x9f8c, 0x9f8c, PDF_CMAP_SINGLE, 7541 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 8247 },
	{ 0x9f8e, 0x9f8f, PDF_CMAP_RANGE, 22016 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 8371 },
	{ 0x9f91, 0x9f93, PDF_CMAP_RANGE, 22018 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 7970 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 9453 },
	{ 0x9f96, 0x9f98, PDF_CMAP_RANGE, 22021 },
	{ 0x9f99, 0x9f99, PDF_CMAP_SINGLE, 2627 },
	{ 0x9f9a, 0x9f9a, PDF_CMAP_SINGLE, 1793 },
	{ 0x9f9b, 0x9f9b, PDF_CMAP_SINGLE, 6740 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 7988 },
	{ 0x9f9d, 0x9f9e, PDF_CMAP_RANGE, 22024 },
	{ 0x9f9f, 0x9f9f, PDF_CMAP_SINGLE, 1859 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 4851 },
	{ 0x9fa1, 0x9fa5, PDF_CMAP_RANGE, 22026 },
	{ 0xe816, 0xe818, PDF_CMAP_RANGE, 22048 },
	{ 0xe81e, 0xe81e, PDF_CMAP_SINGLE, 22056 },
	{ 0xe826, 0xe826, PDF_CMAP_SINGLE, 22064 },
	{ 0xe82b, 0xe82c, PDF_CMAP_RANGE, 22069 },
	{ 0xe831, 0xe832, PDF_CMAP_RANGE, 22075 },
	{ 0xe83b, 0xe83b, PDF_CMAP_SINGLE, 22085 },
	{ 0xe843, 0xe843, PDF_CMAP_SINGLE, 22093 },
	{ 0xe854, 0xe855, PDF_CMAP_RANGE, 22110 },
	{ 0xe864, 0xe864, PDF_CMAP_SINGLE, 22126 },
	{ 0xf92c, 0xf92c, PDF_CMAP_SINGLE, 22031 },
	{ 0xfa0d, 0xfa0f, PDF_CMAP_RANGE, 22032 },
	{ 0xfa11, 0xfa11, PDF_CMAP_SINGLE, 22035 },
	{ 0xfa13, 0xfa14, PDF_CMAP_RANGE, 22036 },
	{ 0xfa18, 0xfa18, PDF_CMAP_SINGLE, 22038 },
	{ 0xfa1f, 0xfa21, PDF_CMAP_RANGE, 22039 },
	{ 0xfa23, 0xfa24, PDF_CMAP_RANGE, 22042 },
	{ 0xfa27, 0xfa29, PDF_CMAP_RANGE, 22044 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 10013 },
	{ 0xfe49, 0xfe52, PDF_CMAP_RANGE, 10027 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 10037 },
	{ 0xfe59, 0xfe66, PDF_CMAP_RANGE, 10041 },
	{ 0xfe68, 0xfe6b, PDF_CMAP_RANGE, 10055 },
	{ 0xff01, 0xff03, PDF_CMAP_RANGE, 262 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 166 },
	{ 0xff05, 0xff5d, PDF_CMAP_RANGE, 266 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 106 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 168 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 10014 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 355 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 10015 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 265 },
};

static const unsigned short pdf_cmap_UniGB_UCS2_H_table[9528] =
{
	167,22354,171,102,162,127,671,669,
	675,673,693,679,677,683,681,687,
	685,9910,105,9911,107,9912,108,185,
	188,151,157,9921,142,136,135,145,
	159,158,130,138,152,147,181,180,
	183,182,179,178,177,176,174,173,
	22428,22047,22055,22060,4162,4707,4722,4709,
	4185,10131,1597,4867,3238,1592,3270,982,
	4765,4884,4879,2091,5017,1431,2543,4860,
	4710,4740,10778,3397,1150,4946,1228,5020,
	4283,2407,5523,3698,3414,5660,11565,3859,
	1398,2927,4656,5934,1386,3948,5302,3395,
	6004,3318,6165,1789,2093,2238,1732,6163,
	1852,5016,5293,5366,1798,5986,5614,5600,
	3983,1765,12946,3437,4518,6409,3795,1526,
	2240,1626,3821,3248,4350,4357,2849,3124,
	4536,1400,6589,3826,1073,2736,3430,3095,
	3491,2053,4611,1715,4713,5789,3019,4073,
	2916,3209,4041,4310,1832,3728,1733,3379,
	4264,3650,7110,7008,16063,994,3011,2808,
	2850,2737,3407,3398,3413,16587,1923,4049,
	2539,4592,2780,7399,7262,3753,4123,4309,
	2490,1591,7115,1593,6686,3261,1249,4657,
	4544,2297,3353,5656,4570,7388,3300,17826,
	7152,1291,4051,3995,4169,18908,8086,2200,
	4093,1825,1528,7445,7504,7739,1285,4668,
	4672,3366,7803,3980,1250,19731,4191,4276,
	19992,2522,2241,7797,8317,1714,2542,7545,
	4303,20714,1636,2795,1770,8589,2289,4219,
	8713,7936,7924,3402,3438,3920,8301,1824,
	1754,7660,7888,5019,4704,1862,8761,8348,
	9864,2656,8305,2704,21894,3465,1937,6741,
	9752,1509,1821,3466,1072,8390,7814,8247,
	7988,4851,104,10024,7703,9989,189,10059,
	22357,99,10019,22065,22068,22109,22108,22117,
	22112,4162,1504,10072,3070,3747,4458,3288,
	3336,3887,4696,1154,4304,10076,4698,1304,
	10077,4613,3151,4701,3415,10078,3181,1124,
	4156,1367,1514,3508,4703,1513,10082,2566,
	4088,10083,3294,4707,10084,1777,4071,5788,
	4559,1662,10087,1329,10088,2594,10089,4722,
	10090,3737,1413,3769,4598,10091,2529,2312,
	4709,2862,10095,2290,10096,4711,2745,4204,
	10097,4525,3817,4424,1964,1603,2497,10098,
	3032,2963,3140,10099,1837,10100,1264,4185,
	4725,2292,3089,4153,3869,3924,2713,2681,
	10115,3268,2580,10132,4301,4506,10133,3417,
	1597,4695,4287,2436,10134,4361,1978,4727,
	3828,2269,4702,10137,4080,3956,8689,4723,
	4867,3751,2375,10140,2188,1884,4199,1217,
	1943,2841,10141,3930,2264,3669,2570,3156,
	4869,10145,4872,3238,4767,4193,3401,3237,
	4770,4768,4732,2247,3052,1301,10153,4769,
	2242,2234,10154,3247,1366,2687,10157,1181,
	4653,3426,3568,4461,1713,3894,4846,4774,
	3111,10161,4772,10162,1405,2615,4187,4771,
	10165,4174,4773,2764,4127,10169,4568,4776,
	10170,4779,2161,2126,3241,10174,1658,10175,
	1631,10176,3090,4782,4168,3832,2098,1689,
	1602,4008,4569,4266,2052,2038,4775,3290,
	3775,1326,10193,4777,10194,3332,4780,2686,
	4781,3776,4783,1141,1817,10200,4791,10201,
	1012,10202,2607,10203,3365,10204,3514,10205,
	3515,4792,3786,1454,4606,4689,4280,10219,
	3640,10220,1925,10221,4790,4848,4293,4787,
	1677,4692,4788,4784,4789,2886,4251,4778,
	4849,10224,4785,10225,2982,4121,998,2116,
	4794,10232,4793,4800,4802,10237,4799,3408,
	4797,4533,10241,8178,10242,1283,4796,10243,
	2536,10244,3427,10245,4798,10246,4795,4804,
	10249,8285,1520,1794,10253,4167,2665,10258,
	2197,4493,1191,3141,2424,4801,10259,4803,
	10260,3833,1957,9884,1372,1580,4808,4850,
	3147,2537,4812,3532,1690,10289,4809,10290,
	4811,1036,4294,4813,8629,3984,10291,4810,
	4805,10294,4806,2546,4807,2150,4009,1700,
	10297,2322,10298,4817,4814,10303,958,4822,
	10307,8908,7778,10315,7968,4824,1055,10316,
	4819,10317,8319,1434,10318,2338,3611,1961,
	4183,10322,4823,2233,10325,1232,4825,2330,
	10329,4826,4815,2882,8284,4818,4821,4820,
	4431,10341,4532,10342,3168,2124,4830,8596,
	4816,10352,4831,3018,4691,10362,3668,7781,
	8833,2941,3686,4827,1226,2443,8909,10407,
	8458,7742,9855,1401,10416,4834,10417,4835,
	1317,4836,1378,8750,972,7830,8907,8812,
	10424,8471,4837,3310,3934,10442,8415,8386,
	4838,8108,4842,3303,8915,4841,4840,4843,
	3536,10465,2170,8065,10469,3014,8720,10474,
	8911,10475,8723,10476,4839,4844,8164,8080,
	10477,4845,8918,10482,3264,10483,8912,8910,
	7798,10492,2503,8919,8917,8913,1592,4697,
	4365,10516,4333,4001,1289,4472,10517,3893,
	1851,10518,2392,10519,2791,1553,7909,10523,
	3700,4866,4868,1428,10527,1524,3206,8229,
	10537,982,1796,2626,4854,10538,2471,1803,
	10539,1841,3990,1121,3075,2318,1477,4645,
	10540,4129,2137,3445,2096,4858,4765,1743,
	3224,1192,4380,2790,4884,3258,10555,3967,
	10556,2347,2922,1843,10560,4885,10561,4332,
	4886,4879,1515,1672,1122,4880,1290,2341,
	10569,2435,4152,2511,1522,4882,4881,3071,
	10578,4633,4883,10579,2562,10580,1491,2609,
	7887,10581,2152,10582,1368,2598,10592,2912,
	2091,1615,4864,10603,3037,10604,2362,2016,
	8153,10605,1451,10606,5017,4002,10607,3690,
	967,1307,2060,5018,1894,10608,4391,1431,
	1492,4748,3243,1651,3149,4749,10611,2364,
	5003,4750,3991,1987,6510,2585,2622,4405,
	1744,1335,10619,1306,2970,2534,10628,1113,
	10629,4751,1831,10630,1438,4549,3479,3210,
	3307,1359,2393,10635,1869,10636,1574,2100,
	3645,8904,3940,9859,4755,3117,1833,2164,
	3633,10645,3049,7959,4758,10650,4756,1033,
	10656,2326,10657,3386,2151,7979,1708,1769,
	10663,8906,7833,4760,10671,2204,10672,4762,
	4761,8015,8143,3005,8246,7994,10676,8905,
	8089,4763,8053,10679,4764,2543,10687,3211,
	1015,1791,2118,3840,5005,2588,1518,4601,
	2925,2218,2531,2257,2488,5008,3421,10705,
	8126,10706,1137,4263,10710,2792,10711,4052,
	5010,10716,2496,7885,5011,10719,2366,8617,
	10720,8678,10721,8487,8195,2847,8498,10724,
	3159,5012,8995,8437,4860,3344,1805,3839,
	4363,1031,1365,10748,4004,4861,10753,5301,
	4863,4710,1988,1050,10758,1276,4740,2180,
	2428,10763,3880,8901,4743,8030,10771,8902,
	3012,3189,4164,4744,2887,8431,3397,10781,
	3108,10782,4700,10783,3382,3830,2033,1014,
	1983,3960,10787,1049,4673,4636,8656,1414,
	2715,2866,1136,10790,1150,10791,1102,5370,
	4445,2357,2643,4747,2648,10792,4746,3811,
	10793,4946,10794,3791,4714,2739,4230,3762,
	10797,2087,3215,2680,10798,2332,3969,10799,
	4947,1228,10806,1585,3663,2533,2530,10809,
	4066,4106,4733,2512,8897,1960,4734,10821,
	4336,7780,10822,3918,4735,10823,4736,3888,
	10824,1309,2294,8697,4737,8208,10831,8898,
	10832,5020,3909,10840,3289,1173,7770,4283,
	1197,2083,4278,3488,1617,3452,10850,3196,
	3443,1101,4027,10851,2962,5013,1503,10855,
	7842,2407,1822,2323,2614,10856,5374,5373,
	4538,2209,4474,979,1506,2390,3579,5371,
	3406,4279,10857,4741,4157,1916,3507,3601,
	10858,5375,1489,5372,4314,10862,1273,1778,
	10863,5378,10864,1926,2077,1494,10865,3677,
	2816,1962,2527,3699,3935,5376,3890,10866,
	2663,5377,2711,2348,10870,2600,3709,4222,
	1642,5385,10871,5388,1678,980,1557,1649,
	10874,1891,3664,2401,3494,3091,10875,4519,
	5389,10876,3825,1240,3851,1336,10879,3798,
	1959,10880,3824,10881,4070,1399,10886,1263,
	10887,1761,5379,2856,10892,2940,5383,5386,
	4340,5387,10893,3127,3815,2875,10899,5394,
	10900,5399,10901,4571,5393,5410,3781,10910,
	1918,5398,5392,2975,3364,1963,2817,2310,
	10915,5390,10916,5397,10917,2969,4423,1924,
	10922,2299,4258,1723,10923,4579,10924,5391,
	1815,2356,10927,5400,5411,5421,10933,5407,
	5406,4646,5991,4142,5402,10934,2358,10935,
	4382,10936,2389,5404,5413,9069,4083,10942,
	5414,946,3030,5403,10943,1948,10944,1566,
	3724,1879,4376,5415,3929,944,5423,5401,
	4079,5405,5412,10947,1982,10948,10949,5418,
	5422,5424,4249,10950,8775,1762,2935,5426,
	3347,2545,2855,2411,3941,4477,5430,1151,
	10965,1942,5428,10966,5009,10967,4109,9068,
	10970,3561,1344,10971,945,5433,3609,5434,
	5427,10989,5436,10994,1976,10995,3442,3767,
	5453,1231,10996,5452,5446,11004,3722,5444,
	11005,5442,11006,2397,4641,11007,3333,5440,
	940,8604,5445,8393,11016,5443,5448,5454,
	11022,8688,3008,3311,2461,5437,2943,11029,
	4730,5441,2905,5466,3639,11038,5463,11039,
	2355,5461,3784,5457,3327,2457,5460,1956,
	1895,5455,5438,11045,5465,1328,5471,3871,
	1919,11053,5462,8459,11063,8414,11064,7850,
	5458,8748,4414,11067,5439,11068,2984,11069,
	5459,11070,4316,11071,5482,4012,8408,8896,
	11077,5474,5476,5456,3293,5479,11082,5464,
	8609,11086,3423,5481,11087,5467,11088,3802,
	11089,3512,5491,5484,5480,11090,5489,9081,
	5472,11095,5490,9074,5473,3529,5499,11101,
	5500,5496,8554,11106,5494,2113,5495,9084,
	1725,4731,8368,11113,9082,7796,4021,2710,
	9079,11116,5493,5475,5498,5497,5501,11122,
	8012,5506,5502,9080,8655,9064,4597,1237,
	11125,4681,9073,3504,11126,9065,5504,11127,
	3850,1936,11131,9844,5511,5507,4148,5505,
	5503,5509,5510,9071,9072,11153,5508,11154,
	5514,9078,9075,11155,3094,4706,4398,5516,
	3424,9087,11158,5515,9076,11159,8373,11160,
	1724,11161,7899,9842,11162,5519,5513,8631,
	9077,11174,1910,3642,5520,11178,9090,9067,
	11193,8249,9887,5521,9085,8693,11202,9089,
	3229,9083,9086,8652,9066,9865,2869,11214,
	9879,3185,3513,11225,5524,2029,4716,4217,
	5525,3702,11226,1561,5527,11233,4339,2450,
	1364,3766,5526,5528,1829,1875,3693,5529,
	3059,5530,11246,4341,9091,3201,5532,5531,
	7997,11247,8591,8774,8776,8574,11254,8576,
	4381,5031,5033,11267,5032,1856,5035,1222,
	5039,2061,11277,4534,11278,5040,1624,11285,
	5024,3585,2367,1994,4693,2400,2132,3590,
	5038,990,3834,1653,4630,3041,2447,11295,
	3596,11296,5047,5041,3033,5043,11297,5048,
	11298,3000,5049,2381,5046,5045,1340,2455,
	2633,5042,5044,3992,5053,2504,5058,5059,
	5026,1809,4334,5052,11332,2398,5056,11333,
	5025,11334,1479,11335,5051,2418,5054,11339,
	5057,1786,942,1260,5055,5064,11363,3057,
	5060,5061,5063,5062,5070,11368,4311,1153,
	9002,5074,11379,5067,8843,5068,2977,2062,
	5075,11390,3606,8069,1552,5196,8997,5071,
	11395,5072,1576,9004,9006,5076,11407,5079,
	1037,8709,4104,7736,7795,1537,8163,9017,
	3567,6563,11440,9003,11441,3537,9005,11442,
	3571,8575,3604,8615,11453,9007,5080,3649,
	5081,5027,11474,5085,5082,11475,2274,11476,
	3474,5083,7874,3331,2844,3130,5084,11492,
	8882,11493,4409,4017,2830,1556,8409,5028,
	11517,8157,5029,11523,8551,5030,11533,8685,
	11534,1909,8199,8998,9001,8018,8251,9000,
	11540,8999,11541,3227,7723,11546,3414,3236,
	11547,4624,8878,3378,4163,8009,5660,11560,
	1321,1711,11568,3889,4859,3859,3731,4865,
	1567,11575,4160,8321,11580,5659,5657,11581,
	1398,11582,3647,3582,1679,11583,4712,4116,
	1906,11584,3390,4171,2417,2115,1568,11589,
	5295,11590,8060,5296,5004,11593,4098,3077,
	2865,1675,11596,1657,2441,4669,11599,3092,
	1061,5298,2178,3626,5300,11602,5299,1485,
	11608,3350,9051,7902,11614,8098,11615,7930,
	2927,2924,11620,2863,11621,2140,3570,6005,
	3266,6006,3758,11628,4622,1721,2703,11629,
	3244,1545,2110,4134,6011,11638,2803,8877,
	6014,6010,6015,3720,2880,6018,2760,11658,
	3069,6012,3411,2228,1820,6016,4000,3774,
	6019,6026,11677,4141,11678,2171,6023,6025,
	11684,2492,6027,11702,4221,4648,11706,3759,
	11707,3727,2637,2192,6024,6029,6032,11714,
	6034,2329,3368,6033,11726,1583,4302,6030,
	11737,6031,11738,3197,6035,8255,3746,6037,
	7950,6036,4233,6041,3373,6045,2761,6044,
	6049,3870,11838,6564,8297,6047,2129,3298,
	6050,11860,6051,11861,6056,9207,6058,6053,
	1462,11868,6054,2878,11873,6048,9210,6060,
	11898,9214,8104,6062,6061,11908,9217,8737,
	11929,6063,11930,4876,6064,8482,4656,6071,
	11954,6072,2404,4371,11955,4659,1385,3556,
	6068,4729,4651,3949,11956,2772,6073,2097,
	1819,6069,4048,1881,2678,8538,4705,3458,
	5994,11964,6070,11965,1682,8677,11968,3265,
	2904,11971,8279,5934,2913,11972,3569,5935,
	4429,11973,4307,3439,957,11974,3523,3739,
	5937,11980,5936,11981,4663,1842,4581,1511,
	3745,4178,1039,3404,11982,1293,3372,2394,
	4038,3433,5938,2007,3913,1797,1885,4112,
	3945,2117,11992,5939,3256,2426,1119,3539,
	2104,11998,2103,4225,2785,2409,2759,12006,
	1893,4325,3163,2834,1203,12016,1834,8421,
	12017,5943,2577,8495,8355,4432,8481,12018,
	8660,8165,12019,5944,12020,5946,7818,7735,
	12025,1386,1555,3511,4057,1437,3440,12028,
	1663,3357,12031,2172,8094,8870,3789,4685,
	8680,12032,7898,7865,3948,12033,3346,1594,
	6066,2133,12036,1251,12037,3337,12038,6067,
	1223,5302,12043,4269,5303,12044,4137,9052,
	3395,4227,1284,5990,2884,2256,3777,2901,
	2309,3015,1194,3193,3646,2237,3820,3409,
	3040,5992,3975,3467,12073,3697,2668,8272,
	5995,7783,2667,5996,12074,9204,8510,12078,
	6004,3710,12079,3318,4192,5548,4305,12091,
	3551,3088,5547,5549,5550,5555,1204,12104,
	5551,1749,5556,1435,5557,7958,5559,5564,
	4091,12114,5562,5561,2612,5563,12118,4355,
	12119,5558,12120,5566,961,5560,12125,2438,
	12126,5565,5569,5571,2676,12156,1578,12157,
	4317,9097,12165,7863,2349,12170,8628,5579,
	1292,9103,3080,1377,12187,4075,7962,5577,
	12198,5575,5574,12199,1065,9099,4441,5576,
	5584,5582,5591,5590,3123,5586,12237,5588,
	5592,12249,5589,5585,9105,5594,12268,8817,
	9102,12286,5595,5598,9104,12313,8243,12314,
	8764,8169,12328,3760,8277,12333,9106,6165,
	12340,1323,4572,4059,1239,1789,4688,3142,
	2317,1799,12345,3814,1205,5023,8996,2093,
	4184,3517,984,4855,2238,12355,12356,1083,
	3431,1155,12357,3484,1608,12358,3389,5534,
	4459,5537,5536,2946,3662,12366,2554,5535,
	4578,1142,4547,1468,8515,1403,4512,8489,
	12377,3868,1018,5538,12380,8823,7847,5541,
	1224,8837,12387,2786,9092,5542,1686,5543,
	2846,12401,9095,9094,5545,8848,12406,5546,
	1332,7748,7730,9093,1732,3036,2893,12419,
	1129,12420,3997,7955,6163,2008,4284,4265,
	8051,1852,5681,4620,12424,3177,1084,1333,
	5683,4029,2645,5682,12430,2415,4235,1464,
	5684,1483,12431,2802,1784,12432,1705,12433,
	2964,1644,5686,5685,1543,4694,8160,12441,
	3670,5690,12447,5688,3475,2370,4254,5687,
	2551,2483,5693,5692,2453,8327,7800,9134,
	7926,7985,5695,12481,5696,12482,8261,8568,
	5016,12489,4092,3667,5293,4699,2359,5021,
	4211,3097,2923,5294,12498,1088,5366,3412,
	12502,5369,12503,1798,12504,4228,12505,1696,
	1954,1469,4454,2779,3903,1974,12515,5999,
	5998,6000,12518,3733,12519,3280,12520,9205,
	12521,8821,1424,3132,12525,6002,9838,12534,
	7856,8323,12538,8584,12539,5986,12540,1858,
	1426,12541,2659,5988,5987,5989,9852,5614,
	3993,12548,3679,12549,4110,1169,1108,12552,
	1115,2990,4452,4245,12555,5600,5601,12559,
	4194,12560,1246,1076,3754,4504,5602,12564,
	2277,1408,12565,5603,1939,5604,1991,2672,
	5605,4023,8133,3694,5606,12571,1444,2952,
	5607,5608,12574,7841,12575,9107,4318,5609,
	7947,4054,5610,3761,5611,12590,1443,12591,
	7804,5612,2026,3983,5698,1089,4203,2108,
	3239,12603,5701,6668,6667,5367,5700,4541,
	3757,2731,4561,5705,5710,2425,12618,5711,
	5703,3982,12628,1965,5707,1659,1992,3583,
	3520,5702,5704,5718,4408,5717,2926,12636,
	4507,2947,1159,5713,12639,5716,2552,3505,
	1410,5723,2084,5715,3999,4348,5719,1839,
	5720,4665,6669,5722,12660,6673,5728,3432,
	2558,12668,2021,2403,12671,1946,6674,6671,
	2027,6675,4031,6672,1941,1590,5729,1521,
	3652,1792,12685,3855,3102,1584,12690,5727,
	5726,2872,5730,12691,4262,5735,3137,5737,
	1902,5736,12705,2031,12706,5731,5732,5738,
	12710,5734,12711,3841,4267,4358,12716,2909,
	6676,4039,5733,12719,2810,12720,5741,1048,
	12721,5746,9140,8318,12722,2101,5740,1440,
	3174,5744,2265,3744,5743,12746,5745,12747,
	3860,5742,12748,3768,2034,7907,1484,2324,
	1177,1266,12753,1058,5739,1176,1421,1848,
	1575,8341,9147,3928,5751,12754,2017,3234,
	3989,9145,5752,1299,6677,12765,4319,4297,
	6678,5753,4201,4291,7720,9149,1739,5747,
	5750,1660,12782,5748,2445,9139,4347,1355,
	12810,5755,8546,2010,12811,3376,7774,12819,
	7773,6679,12822,9143,2724,7983,9439,2032,
	2363,12827,9138,8526,8274,12830,3790,9148,
	12833,5756,8427,2371,7743,1112,4410,12853,
	8219,8380,9150,12854,5758,5759,1888,6680,
	12870,8329,5757,12871,9137,8724,5760,8158,
	3971,8739,974,6682,9142,6683,2476,12895,
	5761,9868,9438,2932,12910,9144,5762,8190,
	8017,8673,12922,9136,12923,8142,12924,8478,
	5088,8225,6684,9441,1765,12930,3835,6393,
	4018,3472,3249,3876,1262,3809,2229,12931,
	9379,5790,2055,6394,4446,12932,3068,6395,
	6396,12936,6398,6397,12937,6400,6399,12938,
	9380,2217,12941,6401,12942,2661,12943,8819,
	12944,8624,1348,1402,6644,6643,1628,3566,
	1099,12952,6645,12956,3437,5306,1164,4413,
	3050,978,1397,3246,3105,4531,12973,2452,
	5307,3297,4120,2917,1010,1243,3232,1586,
	12982,4467,1269,2095,2336,988,4189,3450,
	4610,12996,3687,1525,2374,4476,12997,1698,
	2968,5308,2406,2684,3133,13001,1977,1041,
	3004,3578,2827,13016,5309,4067,1294,13017,
	2807,1684,13020,4595,1412,1207,2839,2892,
	2456,5310,13021,1011,2949,2601,13022,1838,
	13023,2315,3721,985,13024,3712,5312,2303,
	4635,5311,4465,1000,2634,2147,13029,4250,
	2468,2914,1133,4404,2451,3418,5313,4509,
	13032,1801,13033,3208,3486,13034,5315,2378,
	3027,4612,3399,2854,13039,1275,1835,4535,
	6478,959,4542,2677,13057,3805,3573,3961,
	2870,1427,5314,4502,2090,2024,943,13060,
	2931,1391,5316,3671,3741,8657,3829,5318,
	13077,3682,2449,4634,13080,5317,977,1899,
	3340,2902,2327,2487,3557,13093,2148,2000,
	1432,2998,8476,5328,13097,9053,13098,5325,
	2316,5321,9858,1338,2222,5322,2896,3891,
	13110,1473,8461,8282,1565,3441,1493,5327,
	13113,4455,13114,5323,3616,3101,13115,2950,
	4155,13119,2337,2682,13127,3600,1245,13128,
	2212,13129,2405,3703,4099,1390,13130,5326,
	5319,5329,13131,6480,2647,5324,4543,1416,
	13136,1211,13137,5330,13138,5340,13139,8077,
	5335,13143,5339,4670,5337,2797,3636,13149,
	1196,5336,13159,5333,3812,13160,1322,2360,
	2285,2211,8024,5331,13171,4337,13172,5320,
	13173,5332,2474,13178,5334,1210,1764,2638,
	5343,5346,8539,13186,1138,1319,1389,3295,
	5344,3526,13194,1758,13195,5345,5348,3605,
	13203,1004,1393,5942,13210,8411,1202,13216,
	6481,13217,5342,3356,5341,997,4136,1120,
	13220,3586,9056,5338,13227,3481,9057,13234,
	5349,8256,1376,13242,2825,5351,13246,8846,
	9054,13252,5353,7786,2581,13259,5350,2334,
	13260,3025,8194,1258,3281,8339,13269,3503,
	5352,4623,9055,7852,1244,7764,2572,13282,
	7944,3144,1132,1388,13283,4616,13284,8384,
	9058,13285,2895,13286,13287,5357,8544,1898,
	13288,8605,8078,5358,8749,2505,13289,8263,
	3324,13290,8802,8039,7858,13293,1184,3171,
	13294,5359,13295,3161,13296,7849,5360,6482,
	13299,8140,3528,13303,8050,13304,5362,13305,
	5361,13306,1160,7762,8356,7964,8847,13314,
	8173,7725,8530,9062,13319,8442,13320,2955,
	9059,13324,8346,4383,13334,8182,13335,9060,
	13336,3228,7785,13337,9063,13338,8477,8793,
	8278,8547,5364,8105,2335,8188,13347,5365,
	4518,6409,6502,3436,13352,4786,1727,13353,
	1790,1634,4511,3954,7371,2809,13367,2295,
	6503,968,7726,13371,2205,13372,2555,13373,
	1087,1229,1741,3291,6504,2275,3476,13386,
	3136,13387,4508,7868,13388,1680,8514,8222,
	7747,4428,1116,6593,1002,13407,6595,13408,
	9428,13409,1526,13410,2584,13411,7510,3964,
	4486,13414,3810,2240,1287,13417,1702,13418,
	4439,13419,6697,8815,1550,13420,3502,3981,
	7895,13427,1626,6596,3392,2965,13433,6599,
	6598,2666,6597,4040,6600,13438,6601,4674,
	6603,3083,3821,13452,2107,3248,1418,2296,
	4539,4394,4055,4028,3400,2434,6418,13468,
	965,6416,6413,13474,1220,13475,2812,2045,
	4190,3843,6417,6421,13485,3986,4248,1341,
	13490,2758,4687,4466,13495,3422,13496,6423,
	6422,6425,6424,4582,13504,3905,6429,8493,
	2019,2251,3335,3314,3947,6428,4369,6431,
	6433,8859,13522,6427,13523,6432,3837,13526,
	2035,13527,1252,3060,2271,3849,3172,13536,
	2262,6434,4384,13545,6435,3883,8790,9384,
	2928,960,6437,7389,9383,13593,9862,9382,
	13594,8654,3462,13609,4350,13624,3191,4158,
	1783,6420,8508,1187,2723,13629,4411,3641,
	4683,4357,4277,6515,2996,13636,1691,3500,
	6542,13642,2485,3756,13646,1236,13647,3065,
	6572,9412,2849,13654,3779,2828,1063,4416,
	13655,3468,13656,4587,3058,1571,3306,13667,
	4374,3203,1734,13670,6228,3317,6225,13673,
	2521,3998,1163,1384,13674,6226,1540,13682,
	6227,3471,1751,3656,4119,6229,6232,1907,
	13690,1046,2221,7884,6415,6233,13691,6236,
	13692,6241,6242,13697,3518,1008,13698,2078,
	6231,13703,3752,13704,6240,4494,13713,2591,
	6234,13714,2747,13715,1876,4517,6238,3447,
	4393,13719,6230,13720,6235,6237,13721,3126,
	1664,13722,6239,13723,2410,6248,6253,13726,
	6251,2127,2114,6257,6260,13735,6256,1123,
	995,2838,1735,3073,3225,3260,6245,6250,
	6252,13749,1866,6254,4690,13750,2910,13751,
	6258,2383,6243,4600,13761,2625,1208,6261,
	13770,3416,6255,4421,13775,1107,4444,6244,
	6246,1519,6249,13776,6259,2467,13777,3470,
	13778,3485,3067,2528,6280,4585,4130,1922,
	1780,1772,4375,6277,13808,6276,6274,1865,
	3621,6268,3765,2431,13809,963,6279,6278,
	13810,4637,13811,6266,13812,3674,3292,13813,
	1997,2220,6272,6267,1430,6269,3138,6273,
	6275,2177,4619,13824,6286,6283,13833,3681,
	6284,2563,13843,6270,13844,2748,1019,1788,
	9322,8565,13863,9325,2771,3823,2513,3560,
	13873,3632,3968,3448,13876,6281,2143,13887,
	6287,2787,13894,3076,13895,1872,4661,9323,
	8799,2079,13902,2992,7886,3607,3302,13918,
	6294,2509,6292,1840,13925,6289,6295,13938,
	4529,4626,9331,6299,13939,2184,1325,13975,
	1342,7934,3955,6304,2510,13999,6302,6316,
	14004,9333,6306,14007,8714,6288,8047,14013,
	2361,6309,6318,6307,14019,1728,14020,6305,
	14021,4289,2481,6329,6319,1020,6320,4422,
	14045,9320,6322,14046,6323,8448,9334,14059,
	6326,6327,7975,6311,8407,6314,14068,1990,
	6328,6330,6338,8097,1186,14105,6332,14106,
	8874,8197,8506,4450,14128,2822,14129,8708,
	6348,14134,1944,6333,14139,4232,8387,8512,
	9335,6347,6340,3134,9332,6349,1261,14167,
	6343,8040,14171,3933,8581,6345,14208,6352,
	6351,6341,14220,8075,9348,6353,14226,6355,
	2767,9344,9326,14266,9339,9350,9329,9347,
	9328,14274,9321,14275,9343,9338,14317,3124,
	1361,1995,3979,6577,6578,14334,6579,3066,
	8419,2427,6581,3957,14343,3125,9419,14362,
	8019,4536,4510,1358,1156,3827,3078,8211,
	7987,1400,14376,3509,2130,4115,6361,14381,
	1404,6360,14382,4060,3449,1175,14383,6364,
	4530,14390,7772,14391,6368,9354,9353,9355,
	8067,6589,2938,1549,14407,4218,8463,8155,
	1487,14413,2030,6591,4202,8367,3826,14420,
	2843,14421,2756,1533,4719,1073,1080,1082,
	3007,14424,1081,14425,2736,6484,1912,6485,
	14445,6487,6486,14452,9408,6489,6491,6490,
	8813,9409,6492,14461,3430,4715,2806,14462,
	2730,3095,6493,2861,14463,1687,14467,6497,
	3167,8394,6499,14468,1883,4126,956,6498,
	6500,8423,9410,14469,1419,2671,3173,14470,
	6501,14471,3491,5791,3666,4524,3184,4852,
	2040,14483,1905,5794,5792,3328,14492,1904,
	4061,5793,3269,1800,2174,1277,3818,3584,
	14506,2086,14507,5802,14508,5803,3098,1655,
	3164,4177,3813,14516,5796,5804,14517,3371,
	1254,3074,5797,6687,5798,3308,14528,2983,
	1806,14532,2752,14533,5795,2942,2541,2688,
	1182,14534,5805,1979,2832,14535,5808,2313,
	5819,5813,1930,1645,4274,14542,4557,4468,
	1818,4437,4097,3972,3187,3205,1147,14550,
	2783,5820,5807,1605,5815,5812,2915,14561,
	5814,2974,1135,3099,14562,2883,4607,14565,
	2508,5817,5818,5821,3580,5811,14568,4259,
	14569,1068,6688,14570,5816,3973,3042,4406,
	5822,14571,14572,2225,4124,5825,14580,5831,
	3873,14588,5830,5836,2700,2243,14597,5824,
	1952,5832,1596,4573,5840,14604,5835,5823,
	14608,2051,3726,3103,2954,2624,14611,5826,
	14612,3120,2173,2190,5827,14613,4644,1193,
	14614,5833,2102,5837,14615,2048,5838,2921,
	5839,4484,2352,14620,5848,14621,5845,14622,
	5849,1917,2486,1692,5842,1882,2255,9180,
	2908,14647,9179,3944,3359,4260,14650,3902,
	5841,14653,3617,14659,2495,5843,2553,5844,
	3807,14660,2006,1460,14661,3278,2168,4456,
	3301,1693,5864,5866,4077,5857,1486,4650,
	5854,3946,5853,2597,3613,2874,14690,3623,
	5862,5861,5855,14694,5858,1422,4286,14697,
	5860,8286,4224,1382,14701,1993,3367,14704,
	1345,14705,8772,9186,2049,14706,4086,8404,
	3648,6689,4331,14719,5865,4658,5856,14720,
	2166,5859,4299,14723,5863,3377,3195,1544,
	14730,4415,1146,5877,8607,3793,14733,5867,
	7782,3787,14734,1750,5876,14737,2391,4275,
	14741,2801,2953,5878,3701,5869,14761,1973,
	14762,3923,9181,5872,5868,8555,3734,3393,
	2446,14797,2167,5873,5893,4343,8885,14810,
	5881,2616,7974,14814,5895,5885,14819,5886,
	3538,14826,5880,5871,14827,5891,14828,3255,
	5889,2890,5888,14831,5887,1313,5894,14835,
	7779,8328,14836,1474,4649,7869,14840,9025,
	5892,14841,1985,14842,4655,3618,6568,14843,
	5890,4556,5879,5882,2721,5884,14849,2674,
	2479,2679,14850,1118,3589,8484,14857,1455,
	8264,9184,5901,8311,14865,8762,3022,2519,
	4102,5900,2833,14888,8001,8220,5899,5908,
	5906,2725,8888,5883,14893,5902,14894,3477,
	8822,4453,5903,14897,8091,4131,8095,14903,
	9674,5904,14911,3770,2956,9175,3118,14922,
	2657,5896,14926,8453,14927,2579,3593,1238,
	9185,8172,14934,5913,14935,5905,9194,5912,
	14938,5915,14939,5914,9187,8462,1267,14945,
	8102,8196,1247,5909,5911,5910,2989,9189,
	4395,5918,14967,9442,7876,14974,975,2071,
	8886,5921,8361,5916,1117,5924,8054,5925,
	5922,5923,5926,8593,7760,8092,15023,9178,
	15024,8275,15025,9193,9188,8661,15033,9190,
	9183,15037,3064,9177,15043,5927,5929,15044,
	8212,15045,9196,9195,9176,9198,9870,15058,
	9197,9174,9192,5932,15081,8454,9199,5933,
	8585,8280,2053,6639,2805,15100,1447,2023,
	2610,4401,15105,2291,4643,15109,4377,1178,
	6604,2646,1337,1241,15123,3213,2376,6606,
	6605,6607,6611,2325,3602,2971,2283,15141,
	6612,15142,1127,6610,4425,1476,15146,6608,
	2559,1288,6609,3501,2478,3665,2587,15153,
	6614,1949,2494,15166,4593,2379,15174,1616,
	3342,6613,2039,15175,3615,2254,3235,15176,
	3862,3738,15183,2988,4082,1903,6616,15205,
	2005,6617,15206,6640,1059,1654,6618,4111,
	6619,8226,6625,15246,2018,15247,2136,6621,
	15260,3312,9018,15264,2751,15265,6641,4470,
	6622,7918,9429,15268,4594,6624,6620,6626,
	15276,6627,8744,15296,3254,9431,6628,3844,
	3459,6632,8444,15318,6629,15319,6630,9432,
	15329,3223,8473,15342,6634,4105,8743,6633,
	4402,7775,6635,8863,5014,8124,15370,9434,
	15371,8192,15396,6637,6638,15407,4611,15408,
	2945,6508,953,8592,2339,1715,4150,993,
	1498,8712,4713,15414,3489,7910,5789,3019,
	1009,2951,6505,6507,15432,9411,4073,15433,
	2916,15434,6468,15435,2837,15436,2840,2489,
	6469,2852,15440,3838,6466,6470,3627,3853,
	3865,2514,1534,6474,15461,6475,15462,6476,
	3209,5615,15486,1621,5616,5617,15490,4625,
	8879,5621,2430,5620,1459,5623,5622,15504,
	1971,15505,5624,1808,15510,2304,2911,15515,
	1940,2199,5625,5627,1535,3886,3391,5626,
	4505,4321,5628,5632,5629,5630,2517,8630,
	2482,7741,5631,15529,5633,5637,15543,5642,
	1221,5636,2770,1161,5641,5640,5638,5643,
	3988,4588,2733,5646,15554,3908,1958,15559,
	8757,6356,5647,5644,15560,9111,1984,4342,
	15563,9109,8767,8490,5649,15575,6357,7890,
	15595,9110,9112,5653,3572,15596,5654,8354,
	15597,8036,8235,15600,9108,8505,15601,8543,
	8641,9114,15602,5655,15603,9113,2673,15609,
	4310,15610,3750,3735,15634,2746,6170,1996,
	3907,15637,2602,6177,6188,1130,6178,15647,
	6174,6180,6179,3315,4485,15655,6173,1606,
	6175,6187,15667,4584,6181,15672,6186,15673,
	6185,8640,15692,3183,15696,2480,2520,2617,
	6184,6190,3563,6200,6199,6194,6193,15720,
	6195,15721,6191,15722,6198,15723,6197,15724,
	6196,2590,3158,3006,2948,9312,15734,6201,
	6204,15753,6203,15754,6205,1966,6192,6202,
	15755,3275,3299,8741,8298,4135,6206,6208,
	15779,15780,2544,6213,15784,9313,15785,6214,
	6210,15788,6218,6209,15800,6215,9314,6219,
	6220,15842,9315,9316,15848,1832,15849,6962,
	15850,6963,15851,3023,1013,3226,15852,3728,
	3804,6403,6404,15864,3038,1356,9381,5253,
	15883,6406,15884,6407,15885,6408,1733,5368,
	3374,15890,3651,7792,15896,3380,4264,3483,
	15899,1697,6964,1067,15900,5940,3650,4270,
	2122,3363,15901,1480,15902,2867,1482,15903,
	6776,1986,6166,15906,6777,6779,2232,15915,
	6778,3782,2621,5022,6781,4030,8335,2683,
	3079,1609,8014,6782,15939,1296,15940,7857,
	15941,2064,6783,2175,7819,3455,15958,4176,
	7008,15959,7009,15960,7010,2575,15961,1768,
	2301,7012,15964,2930,7011,4122,15965,7014,
	983,2235,4198,7013,15971,1330,1669,7020,
	7019,3010,7017,4495,3630,2305,2085,7022,
	7021,7018,1128,15979,4513,4253,2278,3207,
	4128,15986,4555,1938,7023,15987,1530,8134,
	15988,3685,2538,7025,7028,15994,7027,7030,
	7026,15995,2003,7029,3592,7032,7034,7036,
	1380,9646,7031,16010,7037,7040,7937,7038,
	8704,7043,16021,7039,7042,7044,16022,7045,
	9650,3792,7048,7831,7047,16025,2623,7041,
	3444,8363,16026,1396,1114,3588,16027,7050,
	7056,4463,7053,9651,7046,16036,7055,7052,
	7049,16037,8232,7054,16040,9649,948,7057,
	7059,16049,7061,16050,9644,7060,16054,7058,
	7758,8706,4043,9643,8838,16057,9645,16058,
	9654,16059,7062,8675,7063,8751,8549,9655,
	1448,7913,994,996,4400,16070,1445,16071,
	2213,2015,6957,947,16078,6959,6960,7717,
	4580,7112,16103,7113,9664,16107,8858,4560,
	2985,16116,4244,16117,4205,6795,966,4438,
	4087,2131,1927,16120,2437,16121,1730,1442,
	2957,8814,2768,16126,8125,16127,8068,8370,
	6796,16128,8259,2850,1505,16135,6744,2729,
	16136,4528,3917,6747,2959,16142,1563,16143,
	16144,3384,2753,16149,2369,16150,6746,6752,
	6750,4487,2788,16166,6751,6754,16170,4420,
	4044,6753,16174,2773,6755,2433,2331,6756,
	16180,3658,16181,4100,4642,4503,16185,6760,
	6761,2261,3492,6763,1532,16208,6764,2851,
	16209,6762,2223,1166,6767,4720,6765,6768,
	2798,1303,6769,6766,3878,6770,2718,16248,
	5269,3026,16251,3139,16252,4596,1450,16253,
	3495,9863,3676,16258,6774,4433,9456,6775,
	16264,7550,8864,2737,7114,3407,4186,16293,
	4521,16294,6938,16295,2311,16296,2196,6939,
	1547,950,8107,6690,16302,6691,3848,1612,
	2432,6692,2706,3305,3093,2368,6696,3001,
	16318,4089,16319,4614,16320,4107,16321,16322,
	6704,6708,16326,6706,6714,4489,16327,6709,
	6707,6698,3362,4373,6705,16337,2532,16338,
	1316,9896,16341,1857,16342,6716,6717,16347,
	6710,16348,6715,16349,3847,9445,16366,6718,
	2619,4247,6711,3218,8698,6722,16388,1488,
	2652,952,3550,1047,16393,6720,3740,1475,
	16397,6721,6719,6723,1499,16400,6724,16401,
	6725,16402,6728,16403,1077,16404,8520,2999,
	2145,6726,3599,1201,9446,6727,8439,16413,
	8299,16414,2894,6731,2501,1387,6729,2385,
	6730,8871,9451,2824,6733,16451,6735,6737,
	16490,8166,9447,8210,7916,3413,6648,2524,
	16505,3360,16506,6649,3085,6650,16511,3084,
	6651,4675,6657,4608,3370,3555,6658,16525,
	6656,3926,16528,6660,3024,2099,16533,6659,
	1436,2059,16541,6661,4875,2252,2658,6662,
	8038,9436,1694,9890,6665,8206,16581,9435,
	7864,16582,6666,4306,4708,2518,16588,3162,
	1923,16589,4014,3506,16590,3691,1125,16595,
	3180,16596,4565,2387,2800,6943,1272,16613,
	3157,4118,16614,4551,16615,6944,16616,6942,
	16617,2066,1259,4173,16627,2037,3854,16630,
	6948,6947,1265,16637,3341,3493,6949,16650,
	1298,7808,16665,3799,6952,16669,6951,16670,
	1439,2125,2065,16671,1760,9847,9764,8041,
	8747,9599,8027,8603,16704,9853,4049,16712,
	2286,3179,1324,7068,3692,16717,3153,4430,
	16718,7069,16719,7070,4139,4558,7071,2210,
	1331,2284,1375,3808,16732,2412,7073,2439,
	7072,16737,7075,8608,8582,16738,7074,7076,
	8429,7844,8417,16756,9656,8800,16757,8418,
	2539,2281,2280,4451,2351,16776,3680,7065,
	2224,16782,1546,4592,7267,7268,16793,1736,
	16797,981,16798,7269,16799,7273,3558,7275,
	16803,3953,1075,7272,7279,16809,1458,7287,
	16812,7282,7284,7283,1688,16816,1064,16817,
	7278,7274,1467,16818,7280,7281,16821,7285,
	7277,16826,2134,16827,2630,16828,7286,7290,
	7745,7276,16835,1449,16836,2239,7292,1601,
	2429,4605,3683,16839,1395,16840,1190,16841,
	7288,16842,7289,3313,16843,7293,7294,16846,
	7297,7299,7298,7291,16861,2423,16862,1300,
	16863,7296,3110,16866,2149,8070,16876,1816,
	1140,2067,16882,3544,7309,7305,1845,7310,
	7306,2696,7308,7311,7304,2160,8115,7314,
	7919,16910,4618,3017,16911,8868,16912,9704,
	7315,7313,16917,2639,1752,7317,8465,7320,
	2516,16937,9702,7323,16945,7322,16946,9703,
	7326,7321,8257,16956,7324,9706,7327,16970,
	8079,16971,9708,7329,2014,7328,9707,8400,
	8221,1157,7332,7331,16992,8183,7821,2081,
	9710,8250,9701,9709,17024,8201,17025,8293,
	9891,2780,4853,2507,7370,4654,7372,2540,
	3048,17055,1369,4436,7374,7373,7375,3535,
	7376,17061,4356,4575,2564,7377,2267,7385,
	17083,7381,7384,17084,1972,17085,1757,3610,
	7386,17093,1185,2776,9717,7932,4390,2372,
	8228,7387,2933,9883,17108,9715,17109,8921,
	17110,9716,17111,7399,9243,17112,3874,8136,
	17115,8059,17116,9245,17117,8780,8008,9244,
	8447,3800,8602,17118,8337,9251,7835,9250,
	17123,8464,17124,8845,8049,7928,9249,3533,
	7923,3564,2502,8625,9254,9253,8480,8474,
	9252,17143,9256,9258,9257,17146,8852,17147,
	8894,17148,7729,9260,17156,8117,9259,8111,
	8296,8676,7969,17174,8449,8572,8522,9261,
	8148,7400,17184,8145,7731,17192,9263,9262,
	17195,9264,8667,8889,17212,9270,7822,9273,
	8645,9271,8594,7402,17222,9274,7961,8588,
	7744,8883,8287,9272,9266,8820,17226,7836,
	9265,8325,8119,9267,8276,8670,17240,9268,
	9277,8073,9276,8046,7896,17246,7871,9285,
	17247,8777,9281,17250,7751,8022,8598,17255,
	9283,9279,17256,8227,17257,9282,9278,17260,
	9894,9038,9286,9291,9284,9255,9292,7951,
	17286,9287,9289,9288,7939,17296,9290,8540,
	8891,9296,17299,9876,9295,9718,8273,17300,
	9294,17301,7675,17302,8890,8045,9298,9297,
	7403,9301,17318,8841,8470,8486,8033,9885,
	17335,8074,9302,9305,9304,8058,9293,9303,
	9299,17353,9248,17354,8671,9861,17355,7790,
	8740,9840,17359,8634,17360,9306,7404,8191,
	6099,2287,6100,1955,6101,3896,6102,4351,
	2089,2112,3245,3778,6105,17365,1346,6106,
	3309,1748,2860,17366,4666,2689,1652,4540,
	3797,1633,2919,6107,3916,2560,4678,3369,
	3877,4526,4564,6111,1016,3349,4214,2268,
	6114,1022,3257,2226,6115,3233,17369,6116,
	2043,1779,4045,6117,2702,2343,2203,3684,
	2333,4016,17370,3548,3619,2111,6120,2075,
	4034,6121,17371,4035,3383,3771,2789,6127,
	1066,1302,17372,4664,4449,6130,2675,4631,
	2141,2794,2477,2076,17373,6126,1551,6138,
	17374,1999,1471,2669,1097,6141,4344,6142,
	1722,6144,6143,1673,17375,6145,1215,4236,
	3562,2202,6162,7262,17376,1746,17377,3212,
	7263,17385,9698,17392,9882,17393,1847,3753,
	4766,1896,17396,2693,6785,17397,1600,6787,
	17402,6786,992,6789,4471,4684,4546,17411,
	7914,6791,3463,17412,6790,7724,17415,6792,
	8290,9457,17425,9458,17426,4123,17427,3129,
	17428,2757,7364,4010,7365,17441,3912,3221,
	9714,17444,7366,17445,8725,17451,7369,4878,
	1785,5997,4309,17459,7390,17460,3803,7064,
	17468,4213,17469,7391,8622,17473,3925,7392,
	1461,1383,7394,7393,7395,7397,17491,1897,
	970,17492,7398,8416,17498,1610,4212,4146,
	2490,17502,2377,6483,4480,6426,7127,1591,
	3480,2864,17509,7115,17510,7116,1782,7117,
	1915,4360,989,7119,17516,7120,7125,7124,
	2966,17519,9666,17520,9665,7126,17523,1593,
	17524,7128,4149,5297,3519,1281,17527,1411,
	17528,1787,2903,7129,2573,2628,4527,7131,
	7132,17540,2547,17541,8488,17542,3031,17543,
	2314,7133,1362,8215,7840,7134,8485,8525,
	17561,9668,8349,8842,17562,9667,8569,8248,
	6686,6685,3541,4197,8533,3510,4473,17568,
	3261,17569,2506,2068,3951,17578,4577,17579,
	1542,1747,6512,1737,17580,6511,1227,1826,
	4522,17581,1681,1639,2138,1627,6518,17585,
	6519,964,2396,17586,6517,4322,17587,6520,
	1643,17592,6514,6516,3375,4566,4462,3965,
	6527,3783,6528,17593,1417,1052,6530,3576,
	2967,6531,17606,6529,2976,6526,3387,6533,
	1030,4717,7111,17611,6534,6042,6537,17612,
	2421,4175,6535,6540,1767,6536,17613,2187,
	17614,4003,17615,962,17616,6541,2879,8658,
	1379,2717,2092,4387,3081,2871,6543,2920,
	4873,17627,1144,17628,6548,17629,2198,9415,
	1703,17647,3714,6549,6545,17651,2556,8825,
	3654,17663,6550,17664,2459,4159,6551,17665,
	8483,17666,1706,1704,17667,6552,3128,3748,
	9413,6557,9417,3987,8340,6562,17678,6558,
	17679,8854,17680,6561,3284,17681,4133,6556,
	7799,1716,3910,2888,3629,3705,1021,17687,
	6566,6565,17693,1145,1755,17698,6567,1109,
	17705,7942,3608,2823,3858,6576,17712,2995,
	8345,6570,5697,6574,17727,7853,9416,8360,
	3711,6575,1092,4252,8223,6573,17732,6571,
	8389,17733,9418,8176,17742,9414,8797,8926,
	6402,8236,17752,4657,17753,7337,1305,8545,
	4490,2297,17764,7333,17765,4143,17766,7336,
	2298,4292,8763,8665,8139,8137,17767,7266,
	17770,3453,17771,3653,5656,3497,17778,3831,
	4570,17779,7344,17784,1908,7345,1006,7342,
	17785,7343,2163,1180,17786,1573,1143,3900,
	7346,1327,17787,7347,3672,17797,7352,17798,
	7353,3527,7777,7355,9711,17818,8088,17819,
	7357,7388,2565,2139,8072,3300,4103,6001,
	8696,17826,5089,4188,5090,951,5091,2219,
	17831,5095,4728,17835,5093,4312,17836,3343,
	5096,5092,17837,5097,2727,5098,5107,5099,
	3822,4516,17844,5117,17845,5115,5120,2231,
	2642,17848,5094,5113,5116,5100,1647,976,
	5109,3977,5103,1981,17849,1625,5114,5108,
	5101,3160,17852,8994,17853,4072,5102,3772,
	5104,17860,5105,1179,5119,3530,17861,4346,
	5129,5133,3577,5140,17862,2796,5130,2382,
	5127,17865,1029,1807,5139,5121,17866,5106,
	5124,3279,2413,9014,1062,17873,4231,5123,
	17878,3034,17879,5132,4639,2740,1619,3150,
	2734,5136,5126,5145,5122,5131,17887,2260,
	5125,17888,5134,17891,5158,5135,5166,5141,
	2142,1351,2728,1198,5160,17906,5152,17907,
	5149,17908,5162,5148,4215,1199,17909,3250,
	3262,5161,17910,5147,2258,5154,17918,1188,
	5153,2153,5142,2009,17924,2526,17930,5151,
	5156,5159,1429,17931,3252,2044,5164,5163,
	4240,5165,5167,5169,4216,5168,1920,5173,
	5186,5181,5184,5180,3053,2525,8875,5146,
	5178,8127,17958,5187,5150,5179,17961,5188,
	17962,5176,17963,8061,9011,5189,5183,5177,
	2829,5172,2462,2548,17972,5182,2054,5185,
	4237,5190,17973,5191,2732,5214,5192,5213,
	17979,1814,2308,17982,2345,5206,17989,5201,
	17990,5195,1170,5199,17994,5207,1131,5217,
	3055,5212,8013,5216,2603,1635,5193,18016,
	5209,3620,18017,5205,9012,8177,5198,2765,
	3035,3773,5208,18020,5204,5202,2691,5215,
	3937,3283,5228,2699,5219,9030,4599,18072,
	5221,5220,1771,5218,3054,18077,1516,9031,
	18078,8595,5230,18081,1968,4388,5235,18082,
	9889,18083,1363,18084,5222,18085,2440,5231,
	8034,5227,18086,5225,5226,18098,2176,5232,
	18099,5233,9037,9032,5249,18106,2766,3056,
	18128,5248,4501,5247,5244,18132,7776,5240,
	18133,5243,9029,5236,3251,5242,7954,18140,
	5237,5238,3559,18143,5241,2465,18152,2094,
	5245,5251,18155,5250,5239,2991,18161,8216,
	9013,5255,5262,9020,18173,5261,2804,18190,
	2722,9839,9041,5257,18198,1171,18199,8096,
	2891,3446,8731,3131,5254,5256,5258,5260,
	949,1079,9026,18218,5270,9039,18221,5264,
	2183,3274,9028,9009,18228,9035,5259,18229,
	9019,5263,9040,18237,5266,5265,7860,8614,
	5271,18247,4367,9045,18250,5277,5268,5272,
	1034,5280,18258,5275,9022,18259,8052,18260,
	9008,5276,18263,9856,4047,5279,5273,18278,
	8081,18279,5274,8455,3976,5278,3460,5283,
	5282,18289,5281,9023,8180,9027,1183,2799,
	5284,18308,2939,5286,8722,3628,8711,1607,
	9049,8788,18333,9010,7718,9047,4392,9048,
	5289,8258,8531,5288,18368,5290,4443,18382,
	9043,18383,5292,7152,1975,2649,2929,2670,
	7153,7829,4020,18405,8265,18406,4290,8002,
	1291,7154,18416,7155,1950,7157,7159,7158,
	3545,3879,7156,3403,4182,2707,3794,7162,
	1025,7161,7168,18438,1174,4074,7164,7166,
	4396,7165,7160,7169,7167,18452,7163,7177,
	7174,7176,7180,7172,7170,7171,18468,3190,
	3352,18469,7178,1823,1425,7173,7179,7186,
	7192,7191,3725,18477,4586,7188,7190,7184,
	2720,18491,4478,7185,7189,7195,4257,9690,
	1577,18498,3464,18499,1665,7193,9687,7194,
	7196,7199,7197,4090,7187,18508,3706,18509,
	3806,4520,18510,7205,18511,2784,18512,7202,
	2458,7214,7200,18515,7203,7206,7212,4241,
	7207,1212,7220,18543,3958,7223,18548,8494,
	18549,2013,18550,7227,7224,7226,7228,8626,
	7221,18566,7219,1970,18570,1500,18571,8606,
	7217,7225,7213,7238,9692,7235,7222,18581,
	3253,8300,2813,7239,7241,7236,18605,7230,
	7243,18609,7242,2692,9696,18614,7246,8827,
	18620,7233,18621,9694,18622,7249,7244,7247,
	7232,7245,9685,18640,7240,7251,18645,7787,
	18646,7253,9691,3970,18654,8721,7256,18657,
	4877,18661,8745,9686,9693,9695,18670,7255,
	18671,3263,7254,8175,18679,7259,1347,9688,
	7977,6797,7260,18698,8310,7261,7338,3985,
	8855,9872,18708,3995,4101,8511,3899,2215,
	18716,4076,18717,8600,18718,7816,1945,5613,
	4169,7078,1152,1110,7079,18724,3321,1257,
	18725,4870,18726,3482,18727,7080,4563,18732,
	8844,7081,7358,7082,18736,4335,7083,18737,
	971,7359,1407,18742,2972,7084,18760,4871,
	1060,18767,3867,1162,2586,4621,7085,7088,
	9713,7090,8204,4874,18793,4200,4326,7361,
	3220,7768,8876,18798,7362,7089,7091,2416,
	7092,7098,7093,18810,3338,2978,2698,1877,
	7095,9895,7097,18816,1933,18830,1032,7101,
	7100,7094,7102,3707,7105,5945,18852,8161,
	9659,7104,18853,7106,18854,9662,9660,7363,
	2244,9658,9663,18889,7108,8583,18893,9835,
	3846,18909,4145,18910,7135,8502,18925,9400,
	8420,18941,9401,8189,18954,9403,7981,2158,
	1844,18955,1855,2782,3434,6458,2475,2340,
	7513,18963,7514,18964,7512,2227,18969,7515,
	1320,6592,9737,18980,9738,18981,7828,4093,
	8927,7883,7949,4862,8055,18992,8683,18993,
	8929,18994,8559,18995,8928,8682,18998,8930,
	8395,18999,8056,7906,19002,8690,19003,8528,
	8147,19007,8933,8532,19019,8935,8936,19032,
	6788,8932,19038,8810,19039,8939,19040,8937,
	8381,8950,8679,8726,8492,19053,7784,8946,
	7991,8947,19054,8943,8016,7952,8648,19055,
	8945,8941,8862,8940,8162,19067,4323,3631,
	3419,19073,7855,8758,19076,8951,8765,19080,
	7810,8118,19081,8612,8618,8952,8529,19082,
	8032,8517,19089,8156,8960,19101,8884,8553,
	19104,8957,19105,8426,19106,8948,19107,8955,
	19108,8956,8231,8288,8959,8958,7879,19115,
	8972,8949,19119,8966,19120,8970,8659,8963,
	19124,8967,8971,8031,19127,8969,8962,7940,
	8861,19130,8699,19131,8968,19132,8364,19133,
	8334,8965,8599,19134,8561,8856,19135,5948,
	8322,19141,8977,8964,19145,8975,7733,19146,
	8401,8976,8099,19147,8662,7519,19155,8973,
	8978,8333,8979,8982,8043,8980,19192,8496,
	8981,8552,19193,8388,8984,3016,8729,8727,
	7789,8440,19236,8187,8985,8974,19242,8983,
	19243,4887,2105,1512,1719,3242,2072,3625,
	3230,4890,3100,19244,4062,4208,4063,2106,
	19245,2179,2041,4081,4893,4024,1582,2690,
	19246,3524,1674,3361,1632,2342,4515,3039,
	4676,3405,19247,4426,3540,4496,4896,4574,
	1357,4898,4897,19248,4210,3435,4902,3394,
	1268,4590,4905,1989,1423,1863,4056,4207,
	4908,1726,3927,1206,19249,2236,3819,4308,
	4911,3842,4912,4282,2042,4913,3498,3525,
	4914,3176,4589,4915,2934,1536,4916,1641,
	2395,4917,4918,3490,4919,1496,4920,2571,
	4632,4921,3595,19250,4209,2836,4922,1502,
	2022,4923,3966,1214,4931,4929,4113,4930,
	2778,4932,19251,3974,4140,1028,4936,3112,
	4937,2248,2726,2818,3594,2473,3062,1528,
	7414,8392,7415,3936,2004,1911,4328,7504,
	1043,1209,1928,7506,19314,2743,7739,8831,
	19329,7948,7769,7972,8378,8037,7920,8548,
	7984,8801,8866,9387,19336,9391,7912,7993,
	19337,7752,8304,7848,19338,9388,7927,8566,
	9389,19339,8315,8005,9386,8267,8239,8026,
	9392,19340,8887,8063,19341,8805,9394,8475,
	7761,19348,9396,9395,19353,7839,19354,8472,
	19355,8372,9135,8635,8306,8085,19356,7946,
	9398,8849,9397,8824,7892,8873,9399,7976,
	8457,8903,19370,9390,8881,8806,19377,8795,
	8900,19378,8469,19379,8746,9393,7957,8796,
	1053,4491,1717,19389,1802,1165,4403,3898,
	999,4460,2058,4553,1620,3587,3029,1098,
	1811,4603,1850,1598,2157,3660,1868,6446,
	1406,2744,1646,1935,6447,4407,6448,2121,
	2036,6449,2599,2655,4386,4647,3351,1710,
	1539,6455,3457,3334,1360,5691,2979,6456,
	2464,19392,4629,6457,4617,3287,4745,4739,
	4385,19393,4412,3325,4243,1742,1285,19394,
	3355,7412,1932,19398,7413,7407,1707,4469,
	1738,3087,7408,1233,4352,3188,7409,7956,
	7411,9720,4672,2944,7452,7447,19450,7455,
	19451,7454,19452,4537,7453,4353,7456,19456,
	7464,986,1497,19461,19462,2973,7459,7463,
	19469,2319,7460,1781,7468,7471,2420,19476,
	1867,7448,7465,3659,19482,2156,19483,7469,
	1572,7470,19484,7473,7472,4256,19495,1297,
	3575,8084,7475,2320,7476,3634,7479,1167,
	4662,19516,7477,19517,7478,7480,19518,7486,
	8752,7484,7483,7481,19528,3638,5947,1433,
	7487,7492,3574,9725,1446,1195,19555,7496,
	7493,19556,1558,19557,7497,19558,7494,9727,
	19562,7495,1373,19565,4397,7498,19569,1310,
	19570,9724,7820,9730,19571,8781,19572,7499,
	7501,9732,9726,9731,7500,9729,19578,7502,
	19579,7503,7843,9736,9735,3366,1795,7803,
	8807,7990,8150,6392,8672,19616,9356,8857,
	9361,9366,9359,9362,19647,9367,19648,8113,
	19652,9370,19653,9369,19654,8792,9368,9371,
	19662,7945,8422,8230,9375,8025,7995,9372,
	7738,8283,8816,8759,9426,19692,8627,8773,
	9377,8828,8112,8006,19713,9088,9365,19714,
	9360,19715,1242,4417,1861,4036,19716,6369,
	4615,6370,2685,3272,1947,6377,6376,3166,
	6380,4379,6381,2207,19717,2208,6384,1699,
	2567,6385,1051,2025,1870,6386,19718,1685,
	2080,19719,3451,5492,4338,3882,4440,6390,
	4479,6391,3980,1813,19720,1354,1090,7728,
	19726,19727,1105,19728,7837,7755,7754,1250,
	3267,8362,2578,1394,3109,4285,19739,3096,
	4065,19740,1878,2716,4242,19746,4366,2253,
	19747,5950,1618,5951,1998,4483,19750,2249,
	4345,3764,2549,1278,3657,19753,5954,5952,
	5956,19754,5958,5955,1456,3047,19755,1501,
	5953,19756,3469,5957,9851,2777,1070,2070,
	3708,3522,3425,3622,5960,5959,2889,4042,
	4064,5961,19768,5964,19769,3689,4591,5963,
	1470,19770,3695,9202,5965,1529,19771,8830,
	3673,1853,19772,3420,1270,3534,4399,5966,
	1671,8217,5969,1409,5970,19785,4196,2694,
	1071,19788,4295,1564,3553,19791,5971,8787,
	19797,1106,7998,1587,5974,1441,7846,8590,
	19798,4172,5976,5978,8684,19801,7870,19802,
	8778,19803,5977,3119,19804,4138,5975,8499,
	4389,4475,7812,19813,5980,4686,19814,8399,
	8674,19815,8719,19816,8233,5981,19817,1093,
	4132,8307,5982,5984,8021,9201,5983,19820,
	7750,5985,8291,9203,4191,19824,1452,19825,
	6167,19826,4967,19827,4970,19828,4968,19829,
	4969,4972,3994,2858,3963,19838,4971,19839,
	4271,1889,4978,3182,19840,4974,4973,3348,
	4975,19841,4977,4667,4976,2595,4983,4980,
	19848,4984,2484,4979,4982,4514,19854,4985,
	4990,4988,1913,19861,8990,19862,2353,4987,
	4986,4362,1158,4991,19868,1874,19869,4992,
	1248,8755,1531,4993,19880,1588,19883,4994,
	19884,8992,8892,19895,8988,19896,8785,4995,
	4997,8839,19914,4999,8237,4998,7851,19915,
	8989,19916,8991,5000,8987,5002,19929,8993,
	4276,7416,3186,4640,2981,7417,19930,2293,
	4368,3581,7422,7421,1887,7420,3531,7424,
	2493,19944,1295,19945,3675,7425,7423,2181,
	7428,19946,7429,2206,2749,2414,3542,7430,
	7432,19955,1343,19956,4682,19957,1370,7431,
	7434,7433,7435,3996,2774,3204,7823,7438,
	8716,8100,7443,7442,8347,8664,19990,9722,
	19991,9721,19992,1168,19993,4281,3428,8500,
	2522,4567,4151,2568,19994,2241,9464,19995,
	9463,7880,9462,1701,8832,7877,9467,9466,
	20005,7917,9469,20017,9465,20018,9470,8397,
	9480,9476,9471,8336,7901,7973,20032,9475,
	9474,7802,8358,7953,20047,9479,9472,9477,
	9497,20054,9493,9484,8241,9483,9487,9498,
	9481,9486,8756,9491,8064,9495,9494,9485,
	20076,8403,8314,8398,8106,20101,9502,7967,
	9517,8733,9522,20107,8571,8623,20118,9516,
	9512,20121,8332,20122,9519,20123,8636,9525,
	20129,8717,9510,20130,9524,9514,9503,9521,
	9500,20131,7553,8653,20139,8666,20140,8562,
	9534,8271,20145,9539,20146,8663,20147,7740,
	7554,9535,20165,9540,9507,7824,9530,9541,
	20169,9533,20170,8385,8451,9504,9532,9531,
	9528,9536,20177,8141,8880,20196,9542,7834,
	9554,9520,9545,9553,7882,20204,8402,8120,
	20208,8313,9549,7845,8268,8320,9837,20217,
	9527,7555,20223,9546,8632,20224,9550,9468,
	9556,7996,20230,7893,9559,9555,9544,7556,
	20253,8412,8087,9557,8316,20270,9562,8542,
	20285,9568,8610,9044,9571,9511,20297,9518,
	9560,20298,7963,20299,8835,20300,9572,8352,
	20303,9573,9581,8224,20318,7557,20319,9567,
	9578,7681,9529,9564,9579,7791,20330,8132,
	9575,20331,9563,9515,20347,9585,20348,9591,
	8234,20369,9526,9587,9583,8851,9592,9594,
	8198,20391,8567,9499,20394,9508,7559,9595,
	9593,20407,9574,8083,20413,9596,9597,20427,
	9586,7560,20433,9588,8782,20436,8646,8351,
	20441,9590,8292,8895,9756,8798,4492,1507,
	6802,6801,20449,1613,1495,6807,6809,20450,
	6808,20451,1729,1562,1235,4562,2857,1054,
	1745,4354,3154,2346,3816,1804,6816,6815,
	6818,6817,2918,3115,6821,3116,6822,1134,
	2123,6830,4272,3661,1139,2606,6831,3107,
	2738,20452,6846,6845,20456,6847,3678,2664,
	4418,6851,3872,6857,6856,6858,1776,2815,
	2195,4166,1216,4223,6864,4604,6865,3051,
	20458,2557,6868,3943,3565,6870,6869,1311,
	1873,1667,3978,3276,3635,4481,6883,1392,
	2735,6884,20459,20460,3852,6888,2695,1339,
	4627,2246,20461,3892,6891,6892,1510,2159,
	2321,2769,20462,6895,6903,3135,6937,1548,
	6899,20463,1541,2755,6902,20464,4499,20465,
	6907,2906,20466,6908,2907,1759,1026,20468,
	6918,2276,6921,2499,20470,6933,2550,3919,
	7797,1225,8317,20480,9151,8467,7749,20485,
	8152,9156,20486,9154,8452,20487,8637,20488,
	8071,9155,8003,20500,7966,9849,7915,7989,
	8330,20503,9159,9161,9158,8691,20513,8695,
	9167,9166,9162,9165,9836,20519,9153,20520,
	8174,9169,8184,9170,9172,7832,7980,20532,
	9173,7793,9873,2762,5765,3322,5766,20539,
	1086,3801,1334,3277,5767,3901,5768,2135,
	2763,4419,2873,1860,3796,5771,2811,5772,
	20540,1604,1774,1929,4359,5775,20541,5776,
	4084,4095,5781,1218,2470,5782,20542,2454,
	20544,1714,4948,20545,1554,20546,4950,4949,
	4951,1629,4125,4220,4500,2216,4677,4954,
	4953,20563,941,3716,20564,4955,20565,1720,
	2109,2660,2635,1255,4956,20566,2641,2835,
	2182,4957,3323,1094,20576,8468,20577,4958,
	20578,1527,4349,8836,1314,4959,4364,3906,
	2980,20581,4960,8732,20585,4961,7806,4962,
	2611,3624,3914,8269,4300,2632,20597,4963,
	20598,7897,3546,20599,4964,8114,3547,4229,
	1775,8786,20603,4965,954,3875,20604,8057,
	4464,3554,8535,20615,8639,4966,8735,20621,
	5764,8253,20622,2542,20623,8213,7545,20624,
	9893,2868,20625,3219,4108,4006,4078,2082,
	1830,20628,6940,1353,4255,7548,1312,1490,
	8534,8516,20637,7825,8791,7551,20641,8202,
	8338,4303,7521,4050,8784,7522,2604,2498,
	20654,1035,20655,7875,3836,20658,4019,7525,
	3939,20661,7524,4497,7526,2750,2057,7528,
	7527,3487,20675,3881,7531,20690,2653,991,
	3002,9741,7532,9740,20702,9742,3165,7520,
	2273,9739,1478,1636,20721,2380,2775,2795,
	8899,1770,2250,4046,20735,987,3959,20754,
	955,20755,7971,20756,20757,7628,7630,20776,
	1095,20777,7629,7632,20780,7633,9823,20798,
	8589,8445,9319,20812,9318,3763,3240,20820,
	1890,4219,20826,4370,3345,8649,20835,8713,
	7881,8425,20836,8650,8518,9669,8668,20837,
	9310,20838,8527,20839,8769,8586,7727,7900,
	8383,8244,8718,20852,9675,9676,8131,8377,
	20869,8577,8563,7905,9677,9678,8694,8779,
	9681,20890,7872,9682,20898,7978,7794,9683,
	8638,9684,8260,20904,9679,8435,4154,1508,
	3175,7136,3931,3496,4022,6183,3736,1828,
	1560,7137,1007,3521,7138,4327,2644,2613,
	3043,2272,7139,2120,20905,20906,7142,4170,
	3028,20907,3704,7143,20908,4246,2386,3637,
	20909,4094,1581,20910,1219,3202,7936,20916,
	9880,20917,9848,8376,20932,9424,20933,9425,
	20941,6586,1638,3402,9115,20946,9854,5658,
	7655,9117,20951,9121,20952,7921,8734,20955,
	9122,8523,7734,8501,20963,8109,20967,7763,
	9123,8707,20971,7911,7656,1172,9124,8343,
	7908,9118,21006,8245,9127,9128,21010,8309,
	8171,9132,8042,8441,21017,7659,7658,21018,
	9830,9831,21023,7788,9133,5661,21027,2069,
	21028,1622,4226,2165,3429,1038,3516,21029,
	5668,1595,3231,5669,2201,21032,1126,5670,
	21033,1589,5671,2876,5672,3911,1846,5673,
	2444,21036,5674,1213,21037,5675,21038,2620,
	3438,4718,4857,3920,8301,8770,7938,8579,
	21057,7813,8681,8869,9223,8138,21084,9218,
	8066,9224,9220,21087,8497,21088,8580,21089,
	9219,21090,8302,9227,8151,21117,7811,9230,
	8391,9229,9200,21153,9233,9236,21154,8560,
	9221,8460,9237,8294,21165,9042,9235,9232,
	9225,21172,9240,8103,21178,8700,8129,9222,
	8270,21186,9242,9241,9228,21191,2708,4329,
	3717,4058,1280,3194,21192,1148,2662,6074,
	3410,2307,6077,4609,3718,6079,2128,6078,
	2709,21193,2191,6082,2701,1886,6083,21194,
	6084,1271,4114,2354,6085,3086,6088,3020,
	6089,21199,3296,4583,6097,21200,6098,1824,
	7721,7635,7634,7638,7636,1880,21211,7639,
	21212,7640,7642,7641,7644,9825,21225,7647,
	9892,3549,8564,9827,9826,21226,1754,7660,
	21233,7661,7664,21242,7666,9845,7663,7667,
	21252,7665,8524,21262,7668,9850,21267,7669,
	9832,7672,21283,7888,21284,8342,21285,9164,
	9160,5019,21290,8766,4704,6003,1862,2442,
	2047,7649,3046,7648,21303,7650,7652,7651,
	7653,9829,3785,21307,7654,9828,21313,8761,
	9760,21349,9761,9762,21360,9767,7737,9765,
	9774,9771,9770,9773,21381,9768,8633,9782,
	9776,9784,21404,8205,9783,9797,21414,9786,
	9795,21419,9792,9789,9793,21423,9790,9791,
	8128,21426,9785,21442,9804,9799,9803,9800,
	21456,9802,8456,9780,21471,9809,9808,9810,
	9778,9806,9811,21480,9815,21481,9781,21482,
	9779,21483,9814,9813,7757,9819,21502,9818,
	21503,9817,8238,9775,9798,21508,9766,9763,
	21531,9777,21532,4296,7561,21535,2650,7562,
	7568,21540,1044,7569,21541,3895,21547,21548,
	7588,21549,2263,21551,2596,7622,7941,8331,
	21578,9601,9603,9602,21594,8686,8771,21611,
	9607,21612,9608,8701,21618,8687,9610,9612,
	8144,9618,9615,9620,7904,9617,9621,9623,
	21672,8374,9605,21697,9625,9626,21711,9627,
	9824,9632,9628,21743,9631,21744,8044,8366,
	21762,9611,9635,9609,9637,9638,9619,9639,
	8738,9641,9633,9642,21816,9616,9614,2900,
	6965,2073,6966,2814,21817,2937,4068,21818,
	4069,21819,4117,21820,6973,6972,4330,21821,
	3715,6974,6976,6975,1766,6979,1951,21824,
	2328,6982,1579,21825,6989,2997,21826,6990,
	6991,21830,6992,7637,21831,21832,6996,6997,
	1934,21837,4234,7005,21840,7007,21841,9864,
	9886,9723,21845,8076,8692,7446,2656,7680,
	2651,7682,21868,7683,8305,2714,7406,9871,
	2704,21889,8922,3465,2515,6954,21901,1937,
	7873,7688,7687,21909,7689,7692,21912,7691,
	7859,7690,7693,9834,21919,9866,9833,21922,
	6741,21923,9752,7542,9753,9754,21933,7543,
	21934,7544,1509,21935,4724,1821,21938,9843,
	21939,4721,21940,5087,3466,21947,7695,7699,
	21966,7698,21967,1072,21968,8390,8811,9427,
	3082,6594,7814,21982,9743,9744,9746,8240,
	21991,9841,9750,21997,9749,1282,7533,7534,
	2605,3198,7541,8247,7970,9453,2627,1793,
	6740,7988,1859,4851,10014,355,10015,265,
};

pdf_cmap pdf_cmap_UniGB_UCS2_H =
{
	-1, "UniGB-UCS2-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 2, 0xe000, 0xffff },
	},
	13825, 13825, (pdf_range*) pdf_cmap_UniGB_UCS2_H_ranges,
	9528, 9528, (unsigned short*) pdf_cmap_UniGB_UCS2_H_table,
};

/* UniGB-UCS2-V */

static const pdf_range pdf_cmap_UniGB_UCS2_V_ranges[] =
{
	{ 0x2014, 0x2014, PDF_CMAP_SINGLE, 598 },
	{ 0x2026, 0x2026, PDF_CMAP_SINGLE, 599 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 7708 },
	{ 0x3001, 0x3002, PDF_CMAP_TABLE, 0 },
	{ 0x3008, 0x300f, PDF_CMAP_RANGE, 584 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 594 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 7706 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 582 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 592 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 578 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 580 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 573 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 7707 },
	{ 0xff1a, 0xff1b, PDF_CMAP_RANGE, 576 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7708 },
	{ 0xff1f, 0xff1f, PDF_CMAP_SINGLE, 579 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7709 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7710 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 600 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 596 },
	{ 0xff5d, 0xff5e, PDF_CMAP_TABLE, 2 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7711 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7704 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_UniGB_UCS2_V_table[4] =
{
	575,574,597,7704,
};

pdf_cmap pdf_cmap_UniGB_UCS2_V =
{
	-1, "UniGB-UCS2-V", "UniGB-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	24, 24, (pdf_range*) pdf_cmap_UniGB_UCS2_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_UniGB_UCS2_V_table,
};

/* UniGB-UTF16-H */

static const pdf_range pdf_cmap_UniGB_UTF16_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a4, 0x00a5, PDF_CMAP_TABLE, 0 },
	{ 0x00a7, 0x00a9, PDF_CMAP_TABLE, 2 },
	{ 0x00b0, 0x00b1, PDF_CMAP_TABLE, 5 },
	{ 0x00b7, 0x00b7, PDF_CMAP_SINGLE, 99 },
	{ 0x00d7, 0x00d7, PDF_CMAP_SINGLE, 128 },
	{ 0x00e0, 0x00e1, PDF_CMAP_TABLE, 7 },
	{ 0x00e8, 0x00ea, PDF_CMAP_TABLE, 9 },
	{ 0x00ec, 0x00ed, PDF_CMAP_TABLE, 12 },
	{ 0x00f2, 0x00f3, PDF_CMAP_TABLE, 14 },
	{ 0x00f7, 0x00f7, PDF_CMAP_SINGLE, 129 },
	{ 0x00f9, 0x00fa, PDF_CMAP_TABLE, 16 },
	{ 0x00fc, 0x00fc, PDF_CMAP_SINGLE, 692 },
	{ 0x0101, 0x0101, PDF_CMAP_SINGLE, 668 },
	{ 0x0113, 0x0113, PDF_CMAP_SINGLE, 672 },
	{ 0x011b, 0x011b, PDF_CMAP_SINGLE, 674 },
	{ 0x012b, 0x012b, PDF_CMAP_SINGLE, 676 },
	{ 0x0144, 0x0144, PDF_CMAP_SINGLE, 696 },
	{ 0x0148, 0x0148, PDF_CMAP_SINGLE, 697 },
	{ 0x014d, 0x014d, PDF_CMAP_SINGLE, 680 },
	{ 0x016b, 0x016b, PDF_CMAP_SINGLE, 684 },
	{ 0x01ce, 0x01ce, PDF_CMAP_SINGLE, 670 },
	{ 0x01d0, 0x01d0, PDF_CMAP_SINGLE, 678 },
	{ 0x01d2, 0x01d2, PDF_CMAP_SINGLE, 682 },
	{ 0x01d4, 0x01d4, PDF_CMAP_SINGLE, 686 },
	{ 0x01d6, 0x01d6, PDF_CMAP_SINGLE, 688 },
	{ 0x01d8, 0x01d8, PDF_CMAP_SINGLE, 689 },
	{ 0x01da, 0x01da, PDF_CMAP_SINGLE, 690 },
	{ 0x01dc, 0x01dc, PDF_CMAP_SINGLE, 691 },
	{ 0x01f9, 0x01f9, PDF_CMAP_SINGLE, 698 },
	{ 0x0251, 0x0251, PDF_CMAP_SINGLE, 694 },
	{ 0x0261, 0x0261, PDF_CMAP_SINGLE, 699 },
	{ 0x02c7, 0x02c7, PDF_CMAP_SINGLE, 101 },
	{ 0x02c9, 0x02c9, PDF_CMAP_SINGLE, 100 },
	{ 0x02ca, 0x02cb, PDF_CMAP_RANGE, 9907 },
	{ 0x02d9, 0x02d9, PDF_CMAP_SINGLE, 9909 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 525 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 542 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 549 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 566 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 608 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 602 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 609 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 642 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 641 },
	{ 0x1e3f, 0x1e3f, PDF_CMAP_SINGLE, 695 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 10018 },
	{ 0x2013, 0x2016, PDF_CMAP_TABLE, 18 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 109 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 111 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 22 },
	{ 0x2030, 0x2030, PDF_CMAP_SINGLE, 170 },
	{ 0x2032, 0x2033, PDF_CMAP_RANGE, 163 },
	{ 0x2035, 0x2035, PDF_CMAP_SINGLE, 9913 },
	{ 0x203b, 0x203b, PDF_CMAP_SINGLE, 184 },
	{ 0x205d, 0x205d, PDF_CMAP_SINGLE, 599 },
	{ 0x20ac, 0x20ac, PDF_CMAP_SINGLE, 22353 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 165 },
	{ 0x2105, 0x2105, PDF_CMAP_SINGLE, 9914 },
	{ 0x2109, 0x2109, PDF_CMAP_SINGLE, 9915 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 172 },
	{ 0x2121, 0x2122, PDF_CMAP_TABLE, 24 },
	{ 0x2160, 0x216b, PDF_CMAP_RANGE, 250 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 9897 },
	{ 0x2190, 0x2191, PDF_CMAP_RANGE, 186 },
	{ 0x2192, 0x2193, PDF_CMAP_TABLE, 26 },
	{ 0x2196, 0x2199, PDF_CMAP_RANGE, 9916 },
	{ 0x2208, 0x2208, PDF_CMAP_SINGLE, 137 },
	{ 0x220f, 0x220f, PDF_CMAP_SINGLE, 134 },
	{ 0x2211, 0x2211, PDF_CMAP_SINGLE, 133 },
	{ 0x2215, 0x2215, PDF_CMAP_SINGLE, 9920 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 139 },
	{ 0x221d, 0x2220, PDF_CMAP_TABLE, 28 },
	{ 0x2223, 0x2223, PDF_CMAP_SINGLE, 9922 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 141 },
	{ 0x2227, 0x2228, PDF_CMAP_RANGE, 131 },
	{ 0x2229, 0x222b, PDF_CMAP_TABLE, 32 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 146 },
	{ 0x2234, 0x2237, PDF_CMAP_TABLE, 35 },
	{ 0x223d, 0x223d, PDF_CMAP_SINGLE, 150 },
	{ 0x2248, 0x2248, PDF_CMAP_SINGLE, 149 },
	{ 0x224c, 0x224c, PDF_CMAP_SINGLE, 148 },
	{ 0x2252, 0x2252, PDF_CMAP_SINGLE, 9923 },
	{ 0x2260, 0x2261, PDF_CMAP_TABLE, 39 },
	{ 0x2264, 0x2265, PDF_CMAP_RANGE, 155 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 9924 },
	{ 0x226e, 0x226f, PDF_CMAP_RANGE, 153 },
	{ 0x2295, 0x2295, PDF_CMAP_SINGLE, 9988 },
	{ 0x2299, 0x2299, PDF_CMAP_SINGLE, 144 },
	{ 0x22a5, 0x22a5, PDF_CMAP_SINGLE, 140 },
	{ 0x22bf, 0x22bf, PDF_CMAP_SINGLE, 9926 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 108 },
	{ 0x2312, 0x2312, PDF_CMAP_SINGLE, 143 },
	{ 0x2460, 0x2469, PDF_CMAP_RANGE, 230 },
	{ 0x2474, 0x2487, PDF_CMAP_RANGE, 210 },
	{ 0x2488, 0x249b, PDF_CMAP_RANGE, 190 },
	{ 0x2500, 0x254b, PDF_CMAP_RANGE, 738 },
	{ 0x2550, 0x2573, PDF_CMAP_RANGE, 9927 },
	{ 0x2581, 0x258f, PDF_CMAP_RANGE, 9963 },
	{ 0x2593, 0x2595, PDF_CMAP_RANGE, 9978 },
	{ 0x25a0, 0x25a1, PDF_CMAP_TABLE, 41 },
	{ 0x25b2, 0x25b3, PDF_CMAP_TABLE, 43 },
	{ 0x25bc, 0x25bd, PDF_CMAP_RANGE, 9981 },
	{ 0x25c6, 0x25c7, PDF_CMAP_TABLE, 45 },
	{ 0x25cb, 0x25cb, PDF_CMAP_SINGLE, 175 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 47 },
	{ 0x25e2, 0x25e5, PDF_CMAP_RANGE, 9983 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 49 },
	{ 0x2609, 0x2609, PDF_CMAP_SINGLE, 9987 },
	{ 0x2640, 0x2640, PDF_CMAP_SINGLE, 161 },
	{ 0x2642, 0x2642, PDF_CMAP_SINGLE, 160 },
	{ 0x2e80, 0x2e81, PDF_CMAP_TABLE, 51 },
	{ 0x2e82, 0x2e83, PDF_CMAP_RANGE, 22429 },
	{ 0x2e84, 0x2e84, PDF_CMAP_SINGLE, 22051 },
	{ 0x2e85, 0x2e87, PDF_CMAP_RANGE, 22431 },
	{ 0x2e88, 0x2e88, PDF_CMAP_SINGLE, 22054 },
	{ 0x2e89, 0x2e8a, PDF_CMAP_RANGE, 22434 },
	{ 0x2e8b, 0x2e8c, PDF_CMAP_TABLE, 53 },
	{ 0x2e8d, 0x2e96, PDF_CMAP_RANGE, 22436 },
	{ 0x2e97, 0x2e97, PDF_CMAP_SINGLE, 22061 },
	{ 0x2e98, 0x2e99, PDF_CMAP_RANGE, 22446 },
	{ 0x2e9b, 0x2ea6, PDF_CMAP_RANGE, 22448 },
	{ 0x2ea7, 0x2ea7, PDF_CMAP_SINGLE, 22074 },
	{ 0x2ea8, 0x2ea9, PDF_CMAP_RANGE, 22460 },
	{ 0x2eaa, 0x2eaa, PDF_CMAP_SINGLE, 22077 },
	{ 0x2eab, 0x2ead, PDF_CMAP_RANGE, 22462 },
	{ 0x2eae, 0x2eae, PDF_CMAP_SINGLE, 22080 },
	{ 0x2eaf, 0x2eb2, PDF_CMAP_RANGE, 22465 },
	{ 0x2eb3, 0x2eb3, PDF_CMAP_SINGLE, 22082 },
	{ 0x2eb4, 0x2eb5, PDF_CMAP_RANGE, 22469 },
	{ 0x2eb6, 0x2eb7, PDF_CMAP_RANGE, 22083 },
	{ 0x2eb8, 0x2eba, PDF_CMAP_RANGE, 22471 },
	{ 0x2ebb, 0x2ebb, PDF_CMAP_SINGLE, 22088 },
	{ 0x2ebc, 0x2ec9, PDF_CMAP_RANGE, 22474 },
	{ 0x2eca, 0x2eca, PDF_CMAP_SINGLE, 22098 },
	{ 0x2ecb, 0x2ef3, PDF_CMAP_RANGE, 22488 },
	{ 0x2f00, 0x2fd5, PDF_CMAP_TABLE, 55 },
	{ 0x2ff0, 0x2ffb, PDF_CMAP_RANGE, 10060 },
	{ 0x3000, 0x3002, PDF_CMAP_RANGE, 96 },
	{ 0x3003, 0x3003, PDF_CMAP_SINGLE, 103 },
	{ 0x3005, 0x3007, PDF_CMAP_TABLE, 269 },
	{ 0x3008, 0x300f, PDF_CMAP_RANGE, 115 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 125 },
	{ 0x3012, 0x3013, PDF_CMAP_TABLE, 272 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 113 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 123 },
	{ 0x301d, 0x301e, PDF_CMAP_RANGE, 9990 },
	{ 0x3021, 0x3029, PDF_CMAP_RANGE, 9992 },
	{ 0x3033, 0x3035, PDF_CMAP_RANGE, 22395 },
	{ 0x3038, 0x303a, PDF_CMAP_RANGE, 22398 },
	{ 0x303e, 0x303f, PDF_CMAP_TABLE, 274 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 356 },
	{ 0x3094, 0x3094, PDF_CMAP_SINGLE, 22375 },
	{ 0x309b, 0x309c, PDF_CMAP_RANGE, 10020 },
	{ 0x309d, 0x309e, PDF_CMAP_RANGE, 10025 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 439 },
	{ 0x30f7, 0x30fc, PDF_CMAP_TABLE, 276 },
	{ 0x30fd, 0x30fe, PDF_CMAP_RANGE, 10022 },
	{ 0x3105, 0x3129, PDF_CMAP_RANGE, 700 },
	{ 0x312a, 0x312c, PDF_CMAP_RANGE, 22401 },
	{ 0x31a0, 0x31b7, PDF_CMAP_RANGE, 22404 },
	{ 0x3220, 0x3229, PDF_CMAP_RANGE, 240 },
	{ 0x3231, 0x3231, PDF_CMAP_SINGLE, 10017 },
	{ 0x32a3, 0x32a3, PDF_CMAP_SINGLE, 10001 },
	{ 0x338e, 0x338f, PDF_CMAP_RANGE, 10002 },
	{ 0x339c, 0x339e, PDF_CMAP_RANGE, 10004 },
	{ 0x33a1, 0x33a1, PDF_CMAP_SINGLE, 10007 },
	{ 0x33c4, 0x33c4, PDF_CMAP_SINGLE, 10008 },
	{ 0x33ce, 0x33ce, PDF_CMAP_SINGLE, 10009 },
	{ 0x33d1, 0x33d2, PDF_CMAP_RANGE, 10010 },
	{ 0x33d5, 0x33d5, PDF_CMAP_SINGLE, 10012 },
	{ 0x3400, 0x3446, PDF_CMAP_RANGE, 22529 },
	{ 0x3447, 0x3447, PDF_CMAP_SINGLE, 22053 },
	{ 0x3448, 0x3472, PDF_CMAP_RANGE, 22600 },
	{ 0x3473, 0x3473, PDF_CMAP_SINGLE, 22052 },
	{ 0x3474, 0x359d, PDF_CMAP_RANGE, 22643 },
	{ 0x359e, 0x359e, PDF_CMAP_SINGLE, 22057 },
	{ 0x359f, 0x360d, PDF_CMAP_RANGE, 22941 },
	{ 0x360e, 0x360e, PDF_CMAP_SINGLE, 22059 },
	{ 0x360f, 0x3619, PDF_CMAP_RANGE, 23052 },
	{ 0x361a, 0x361a, PDF_CMAP_SINGLE, 22058 },
	{ 0x361b, 0x3917, PDF_CMAP_RANGE, 23063 },
	{ 0x3918, 0x3918, PDF_CMAP_SINGLE, 22063 },
	{ 0x3919, 0x396d, PDF_CMAP_RANGE, 23828 },
	{ 0x396e, 0x396e, PDF_CMAP_SINGLE, 22062 },
	{ 0x396f, 0x39ce, PDF_CMAP_RANGE, 23913 },
	{ 0x39cf, 0x39d0, PDF_CMAP_TABLE, 282 },
	{ 0x39d1, 0x39de, PDF_CMAP_RANGE, 24009 },
	{ 0x39df, 0x39df, PDF_CMAP_SINGLE, 22066 },
	{ 0x39e0, 0x3a72, PDF_CMAP_RANGE, 24023 },
	{ 0x3a73, 0x3a73, PDF_CMAP_SINGLE, 22067 },
	{ 0x3a74, 0x3b4d, PDF_CMAP_RANGE, 24170 },
	{ 0x3b4e, 0x3b4e, PDF_CMAP_SINGLE, 22071 },
	{ 0x3b4f, 0x3c6d, PDF_CMAP_RANGE, 24388 },
	{ 0x3c6e, 0x3c6e, PDF_CMAP_SINGLE, 22072 },
	{ 0x3c6f, 0x3cdf, PDF_CMAP_RANGE, 24675 },
	{ 0x3ce0, 0x3ce0, PDF_CMAP_SINGLE, 22073 },
	{ 0x3ce1, 0x4055, PDF_CMAP_RANGE, 24788 },
	{ 0x4056, 0x4056, PDF_CMAP_SINGLE, 22078 },
	{ 0x4057, 0x415e, PDF_CMAP_RANGE, 25673 },
	{ 0x415f, 0x415f, PDF_CMAP_SINGLE, 22079 },
	{ 0x4160, 0x4336, PDF_CMAP_RANGE, 25937 },
	{ 0x4337, 0x4337, PDF_CMAP_SINGLE, 22081 },
	{ 0x4338, 0x43ab, PDF_CMAP_RANGE, 26408 },
	{ 0x43ac, 0x43ac, PDF_CMAP_SINGLE, 22087 },
	{ 0x43ad, 0x43b0, PDF_CMAP_RANGE, 26524 },
	{ 0x43b1, 0x43b1, PDF_CMAP_SINGLE, 22086 },
	{ 0x43b2, 0x43dc, PDF_CMAP_RANGE, 26528 },
	{ 0x43dd, 0x43dd, PDF_CMAP_SINGLE, 22089 },
	{ 0x43de, 0x44d5, PDF_CMAP_RANGE, 26571 },
	{ 0x44d6, 0x44d6, PDF_CMAP_SINGLE, 22090 },
	{ 0x44d7, 0x464b, PDF_CMAP_RANGE, 26819 },
	{ 0x464c, 0x464c, PDF_CMAP_SINGLE, 22092 },
	{ 0x464d, 0x4660, PDF_CMAP_RANGE, 27192 },
	{ 0x4661, 0x4661, PDF_CMAP_SINGLE, 22091 },
	{ 0x4662, 0x4722, PDF_CMAP_RANGE, 27212 },
	{ 0x4723, 0x4723, PDF_CMAP_SINGLE, 22094 },
	{ 0x4724, 0x4728, PDF_CMAP_RANGE, 27405 },
	{ 0x4729, 0x4729, PDF_CMAP_SINGLE, 22095 },
	{ 0x472a, 0x477b, PDF_CMAP_RANGE, 27410 },
	{ 0x477c, 0x477c, PDF_CMAP_SINGLE, 22096 },
	{ 0x477d, 0x478c, PDF_CMAP_RANGE, 27492 },
	{ 0x478d, 0x478d, PDF_CMAP_SINGLE, 22097 },
	{ 0x478e, 0x4946, PDF_CMAP_RANGE, 27508 },
	{ 0x4947, 0x4947, PDF_CMAP_SINGLE, 22099 },
	{ 0x4948, 0x4979, PDF_CMAP_RANGE, 27949 },
	{ 0x497a, 0x497a, PDF_CMAP_SINGLE, 22100 },
	{ 0x497b, 0x497c, PDF_CMAP_RANGE, 27999 },
	{ 0x497d, 0x497d, PDF_CMAP_SINGLE, 22101 },
	{ 0x497e, 0x4981, PDF_CMAP_RANGE, 28001 },
	{ 0x4982, 0x4983, PDF_CMAP_RANGE, 22102 },
	{ 0x4984, 0x4984, PDF_CMAP_SINGLE, 28005 },
	{ 0x4985, 0x4986, PDF_CMAP_RANGE, 22104 },
	{ 0x4987, 0x499a, PDF_CMAP_RANGE, 28006 },
	{ 0x499b, 0x499b, PDF_CMAP_SINGLE, 22107 },
	{ 0x499c, 0x499e, PDF_CMAP_RANGE, 28026 },
	{ 0x499f, 0x499f, PDF_CMAP_SINGLE, 22106 },
	{ 0x49a0, 0x49b5, PDF_CMAP_RANGE, 28029 },
	{ 0x49b6, 0x49b7, PDF_CMAP_TABLE, 284 },
	{ 0x49b8, 0x4c76, PDF_CMAP_RANGE, 28051 },
	{ 0x4c77, 0x4c77, PDF_CMAP_SINGLE, 22116 },
	{ 0x4c78, 0x4c9e, PDF_CMAP_RANGE, 28754 },
	{ 0x4c9f, 0x4ca1, PDF_CMAP_RANGE, 22113 },
	{ 0x4ca2, 0x4ca3, PDF_CMAP_TABLE, 286 },
	{ 0x4ca4, 0x4d12, PDF_CMAP_RANGE, 28793 },
	{ 0x4d13, 0x4d19, PDF_CMAP_RANGE, 22118 },
	{ 0x4d1a, 0x4dad, PDF_CMAP_RANGE, 28904 },
	{ 0x4dae, 0x4dae, PDF_CMAP_SINGLE, 22125 },
	{ 0x4daf, 0x4db5, PDF_CMAP_RANGE, 29052 },
	{ 0x4e00, 0x4e03, PDF_CMAP_TABLE, 288 },
	{ 0x4e04, 0x4e06, PDF_CMAP_RANGE, 10073 },
	{ 0x4e07, 0x4e1e, PDF_CMAP_TABLE, 292 },
	{ 0x4e1f, 0x4e21, PDF_CMAP_RANGE, 10079 },
	{ 0x4e22, 0x4e2d, PDF_CMAP_TABLE, 316 },
	{ 0x4e2e, 0x4e2f, PDF_CMAP_RANGE, 10085 },
	{ 0x4e30, 0x4e3f, PDF_CMAP_TABLE, 328 },
	{ 0x4e40, 0x4e42, PDF_CMAP_RANGE, 10092 },
	{ 0x4e43, 0x4e59, PDF_CMAP_TABLE, 344 },
	{ 0x4e5a, 0x4e5b, PDF_CMAP_RANGE, 10101 },
	{ 0x4e5c, 0x4e61, PDF_CMAP_TABLE, 367 },
	{ 0x4e62, 0x4e65, PDF_CMAP_RANGE, 10103 },
	{ 0x4e66, 0x4e66, PDF_CMAP_SINGLE, 3456 },
	{ 0x4e67, 0x4e68, PDF_CMAP_RANGE, 10107 },
	{ 0x4e69, 0x4e69, PDF_CMAP_SINGLE, 4726 },
	{ 0x4e6a, 0x4e6f, PDF_CMAP_RANGE, 10109 },
	{ 0x4e70, 0x4e73, PDF_CMAP_TABLE, 373 },
	{ 0x4e74, 0x4e7d, PDF_CMAP_RANGE, 10116 },
	{ 0x4e7e, 0x4e7e, PDF_CMAP_SINGLE, 3113 },
	{ 0x4e7f, 0x4e81, PDF_CMAP_RANGE, 10126 },
	{ 0x4e82, 0x4e82, PDF_CMAP_SINGLE, 8281 },
	{ 0x4e83, 0x4e85, PDF_CMAP_RANGE, 10129 },
	{ 0x4e86, 0x4e95, PDF_CMAP_TABLE, 377 },
	{ 0x4e96, 0x4e97, PDF_CMAP_RANGE, 10135 },
	{ 0x4e98, 0x4e9b, PDF_CMAP_TABLE, 393 },
	{ 0x4e9c, 0x4e9d, PDF_CMAP_RANGE, 10138 },
	{ 0x4e9e, 0x4eae, PDF_CMAP_TABLE, 397 },
	{ 0x4eaf, 0x4eb1, PDF_CMAP_RANGE, 10142 },
	{ 0x4eb2, 0x4eb5, PDF_CMAP_TABLE, 414 },
	{ 0x4eb6, 0x4eb9, PDF_CMAP_RANGE, 10146 },
	{ 0x4eba, 0x4ebb, PDF_CMAP_TABLE, 418 },
	{ 0x4ebc, 0x4ebe, PDF_CMAP_RANGE, 10150 },
	{ 0x4ebf, 0x4ece, PDF_CMAP_TABLE, 420 },
	{ 0x4ecf, 0x4ed0, PDF_CMAP_RANGE, 10155 },
	{ 0x4ed1, 0x4ed9, PDF_CMAP_TABLE, 436 },
	{ 0x4eda, 0x4edc, PDF_CMAP_RANGE, 10158 },
	{ 0x4edd, 0x4ee5, PDF_CMAP_TABLE, 445 },
	{ 0x4ee6, 0x4ee7, PDF_CMAP_RANGE, 10163 },
	{ 0x4ee8, 0x4eec, PDF_CMAP_TABLE, 454 },
	{ 0x4eed, 0x4eef, PDF_CMAP_RANGE, 10166 },
	{ 0x4ef0, 0x4ef7, PDF_CMAP_TABLE, 459 },
	{ 0x4ef8, 0x4efa, PDF_CMAP_RANGE, 10171 },
	{ 0x4efb, 0x4f01, PDF_CMAP_TABLE, 467 },
	{ 0x4f02, 0x4f08, PDF_CMAP_RANGE, 10177 },
	{ 0x4f09, 0x4f0a, PDF_CMAP_TABLE, 474 },
	{ 0x4f0b, 0x4f0c, PDF_CMAP_RANGE, 10184 },
	{ 0x4f0d, 0x4f11, PDF_CMAP_TABLE, 476 },
	{ 0x4f12, 0x4f16, PDF_CMAP_RANGE, 10186 },
	{ 0x4f17, 0x4f1b, PDF_CMAP_TABLE, 481 },
	{ 0x4f1c, 0x4f1d, PDF_CMAP_RANGE, 10191 },
	{ 0x4f1e, 0x4f27, PDF_CMAP_TABLE, 486 },
	{ 0x4f28, 0x4f29, PDF_CMAP_RANGE, 10195 },
	{ 0x4f2a, 0x4f2b, PDF_CMAP_TABLE, 496 },
	{ 0x4f2c, 0x4f2e, PDF_CMAP_RANGE, 10197 },
	{ 0x4f2f, 0x4f3d, PDF_CMAP_TABLE, 498 },
	{ 0x4f3e, 0x4f42, PDF_CMAP_RANGE, 10206 },
	{ 0x4f43, 0x4f43, PDF_CMAP_SINGLE, 1481 },
	{ 0x4f44, 0x4f45, PDF_CMAP_RANGE, 10211 },
	{ 0x4f46, 0x4f46, PDF_CMAP_SINGLE, 1420 },
	{ 0x4f47, 0x4f4c, PDF_CMAP_RANGE, 10213 },
	{ 0x4f4d, 0x4f60, PDF_CMAP_TABLE, 513 },
	{ 0x4f61, 0x4f62, PDF_CMAP_RANGE, 10222 },
	{ 0x4f63, 0x4f69, PDF_CMAP_TABLE, 533 },
	{ 0x4f6a, 0x4f6b, PDF_CMAP_RANGE, 10226 },
	{ 0x4f6c, 0x4f6c, PDF_CMAP_SINGLE, 2491 },
	{ 0x4f6d, 0x4f6e, PDF_CMAP_RANGE, 10228 },
	{ 0x4f6f, 0x4f70, PDF_CMAP_TABLE, 540 },
	{ 0x4f71, 0x4f72, PDF_CMAP_RANGE, 10230 },
	{ 0x4f73, 0x4f76, PDF_CMAP_TABLE, 542 },
	{ 0x4f77, 0x4f7a, PDF_CMAP_RANGE, 10233 },
	{ 0x4f7b, 0x4f7f, PDF_CMAP_TABLE, 546 },
	{ 0x4f80, 0x4f82, PDF_CMAP_RANGE, 10238 },
	{ 0x4f83, 0x4f91, PDF_CMAP_TABLE, 551 },
	{ 0x4f92, 0x4f93, PDF_CMAP_RANGE, 10247 },
	{ 0x4f94, 0x4f97, PDF_CMAP_TABLE, 566 },
	{ 0x4f98, 0x4f9a, PDF_CMAP_RANGE, 10250 },
	{ 0x4f9b, 0x4f9d, PDF_CMAP_TABLE, 570 },
	{ 0x4f9e, 0x4f9f, PDF_CMAP_RANGE, 10254 },
	{ 0x4fa0, 0x4fa0, PDF_CMAP_SINGLE, 3885 },
	{ 0x4fa1, 0x4fa2, PDF_CMAP_RANGE, 10256 },
	{ 0x4fa3, 0x4faf, PDF_CMAP_TABLE, 573 },
	{ 0x4fb0, 0x4fb4, PDF_CMAP_RANGE, 10261 },
	{ 0x4fb5, 0x4fb5, PDF_CMAP_SINGLE, 3155 },
	{ 0x4fb6, 0x4fbe, PDF_CMAP_RANGE, 10266 },
	{ 0x4fbf, 0x4fbf, PDF_CMAP_SINGLE, 1100 },
	{ 0x4fc0, 0x4fc1, PDF_CMAP_RANGE, 10275 },
	{ 0x4fc2, 0x4fc5, PDF_CMAP_TABLE, 586 },
	{ 0x4fc6, 0x4fc9, PDF_CMAP_RANGE, 10277 },
	{ 0x4fca, 0x4fca, PDF_CMAP_SINGLE, 2350 },
	{ 0x4fcb, 0x4fcd, PDF_CMAP_RANGE, 10281 },
	{ 0x4fce, 0x4fd1, PDF_CMAP_TABLE, 590 },
	{ 0x4fd2, 0x4fd6, PDF_CMAP_RANGE, 10284 },
	{ 0x4fd7, 0x4fe3, PDF_CMAP_TABLE, 594 },
	{ 0x4fe4, 0x4fe5, PDF_CMAP_RANGE, 10292 },
	{ 0x4fe6, 0x4fea, PDF_CMAP_TABLE, 607 },
	{ 0x4feb, 0x4fec, PDF_CMAP_RANGE, 10295 },
	{ 0x4fed, 0x4ff3, PDF_CMAP_TABLE, 612 },
	{ 0x4ff4, 0x4ff7, PDF_CMAP_RANGE, 10299 },
	{ 0x4ff8, 0x4ffa, PDF_CMAP_TABLE, 619 },
	{ 0x4ffb, 0x4ffd, PDF_CMAP_RANGE, 10304 },
	{ 0x4ffe, 0x5000, PDF_CMAP_TABLE, 622 },
	{ 0x5001, 0x5005, PDF_CMAP_RANGE, 10308 },
	{ 0x5006, 0x5006, PDF_CMAP_SINGLE, 8214 },
	{ 0x5007, 0x5008, PDF_CMAP_RANGE, 10313 },
	{ 0x5009, 0x5014, PDF_CMAP_TABLE, 625 },
	{ 0x5015, 0x5017, PDF_CMAP_RANGE, 10319 },
	{ 0x5018, 0x501c, PDF_CMAP_TABLE, 637 },
	{ 0x501d, 0x501e, PDF_CMAP_RANGE, 10323 },
	{ 0x501f, 0x5021, PDF_CMAP_TABLE, 642 },
	{ 0x5022, 0x5024, PDF_CMAP_RANGE, 10326 },
	{ 0x5025, 0x502e, PDF_CMAP_TABLE, 645 },
	{ 0x502f, 0x5039, PDF_CMAP_RANGE, 10330 },
	{ 0x503a, 0x503e, PDF_CMAP_TABLE, 655 },
	{ 0x503f, 0x5042, PDF_CMAP_RANGE, 10343 },
	{ 0x5043, 0x5043, PDF_CMAP_SINGLE, 4828 },
	{ 0x5044, 0x5046, PDF_CMAP_RANGE, 10347 },
	{ 0x5047, 0x5049, PDF_CMAP_TABLE, 660 },
	{ 0x504a, 0x504b, PDF_CMAP_RANGE, 10350 },
	{ 0x504c, 0x504f, PDF_CMAP_TABLE, 663 },
	{ 0x5050, 0x5054, PDF_CMAP_RANGE, 10353 },
	{ 0x5055, 0x5055, PDF_CMAP_SINGLE, 4829 },
	{ 0x5056, 0x5059, PDF_CMAP_RANGE, 10358 },
	{ 0x505a, 0x505c, PDF_CMAP_TABLE, 667 },
	{ 0x505d, 0x5064, PDF_CMAP_RANGE, 10363 },
	{ 0x5065, 0x5065, PDF_CMAP_SINGLE, 2162 },
	{ 0x5066, 0x506b, PDF_CMAP_RANGE, 10371 },
	{ 0x506c, 0x506c, PDF_CMAP_SINGLE, 4832 },
	{ 0x506d, 0x5073, PDF_CMAP_RANGE, 10377 },
	{ 0x5074, 0x5077, PDF_CMAP_TABLE, 670 },
	{ 0x5078, 0x507a, PDF_CMAP_RANGE, 10384 },
	{ 0x507b, 0x507b, PDF_CMAP_SINGLE, 4833 },
	{ 0x507c, 0x507d, PDF_CMAP_RANGE, 10387 },
	{ 0x507e, 0x5080, PDF_CMAP_TABLE, 674 },
	{ 0x5081, 0x5084, PDF_CMAP_RANGE, 10389 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 1712 },
	{ 0x5086, 0x5087, PDF_CMAP_RANGE, 10393 },
	{ 0x5088, 0x5088, PDF_CMAP_SINGLE, 2535 },
	{ 0x5089, 0x508c, PDF_CMAP_RANGE, 10395 },
	{ 0x508d, 0x508d, PDF_CMAP_SINGLE, 1027 },
	{ 0x508e, 0x5095, PDF_CMAP_RANGE, 10399 },
	{ 0x5096, 0x5099, PDF_CMAP_TABLE, 677 },
	{ 0x509a, 0x50a1, PDF_CMAP_RANGE, 10408 },
	{ 0x50a2, 0x50a9, PDF_CMAP_TABLE, 681 },
	{ 0x50aa, 0x50ab, PDF_CMAP_RANGE, 10418 },
	{ 0x50ac, 0x50ad, PDF_CMAP_TABLE, 689 },
	{ 0x50ae, 0x50b1, PDF_CMAP_RANGE, 10420 },
	{ 0x50b2, 0x50b7, PDF_CMAP_TABLE, 691 },
	{ 0x50b8, 0x50b9, PDF_CMAP_RANGE, 10425 },
	{ 0x50ba, 0x50bb, PDF_CMAP_TABLE, 697 },
	{ 0x50bc, 0x50bd, PDF_CMAP_RANGE, 10427 },
	{ 0x50be, 0x50be, PDF_CMAP_SINGLE, 8424 },
	{ 0x50bf, 0x50c1, PDF_CMAP_RANGE, 10429 },
	{ 0x50c2, 0x50c2, PDF_CMAP_SINGLE, 8916 },
	{ 0x50c3, 0x50c4, PDF_CMAP_RANGE, 10432 },
	{ 0x50c5, 0x50c5, PDF_CMAP_SINGLE, 8121 },
	{ 0x50c6, 0x50c8, PDF_CMAP_RANGE, 10434 },
	{ 0x50c9, 0x50c9, PDF_CMAP_SINGLE, 8920 },
	{ 0x50ca, 0x50ce, PDF_CMAP_RANGE, 10437 },
	{ 0x50cf, 0x50d1, PDF_CMAP_TABLE, 699 },
	{ 0x50d2, 0x50d4, PDF_CMAP_RANGE, 10443 },
	{ 0x50d5, 0x50d6, PDF_CMAP_TABLE, 702 },
	{ 0x50d7, 0x50d9, PDF_CMAP_RANGE, 10446 },
	{ 0x50da, 0x50da, PDF_CMAP_SINGLE, 2574 },
	{ 0x50db, 0x50dd, PDF_CMAP_RANGE, 10449 },
	{ 0x50de, 0x50de, PDF_CMAP_SINGLE, 8597 },
	{ 0x50df, 0x50e4, PDF_CMAP_RANGE, 10452 },
	{ 0x50e5, 0x50e8, PDF_CMAP_TABLE, 704 },
	{ 0x50e9, 0x50eb, PDF_CMAP_RANGE, 10458 },
	{ 0x50ec, 0x50ee, PDF_CMAP_TABLE, 708 },
	{ 0x50ef, 0x50f2, PDF_CMAP_RANGE, 10461 },
	{ 0x50f3, 0x50f5, PDF_CMAP_TABLE, 711 },
	{ 0x50f6, 0x50f8, PDF_CMAP_RANGE, 10466 },
	{ 0x50f9, 0x50fb, PDF_CMAP_TABLE, 714 },
	{ 0x50fc, 0x50ff, PDF_CMAP_RANGE, 10470 },
	{ 0x5100, 0x510b, PDF_CMAP_TABLE, 717 },
	{ 0x510c, 0x510f, PDF_CMAP_RANGE, 10478 },
	{ 0x5110, 0x5115, PDF_CMAP_TABLE, 729 },
	{ 0x5116, 0x5117, PDF_CMAP_RANGE, 10484 },
	{ 0x5118, 0x5118, PDF_CMAP_SINGLE, 9857 },
	{ 0x5119, 0x511e, PDF_CMAP_RANGE, 10486 },
	{ 0x511f, 0x5121, PDF_CMAP_TABLE, 735 },
	{ 0x5122, 0x5129, PDF_CMAP_RANGE, 10493 },
	{ 0x512a, 0x512a, PDF_CMAP_SINGLE, 8753 },
	{ 0x512b, 0x5131, PDF_CMAP_RANGE, 10501 },
	{ 0x5132, 0x5132, PDF_CMAP_SINGLE, 7827 },
	{ 0x5133, 0x5136, PDF_CMAP_RANGE, 10508 },
	{ 0x5137, 0x5137, PDF_CMAP_SINGLE, 8914 },
	{ 0x5138, 0x5139, PDF_CMAP_RANGE, 10512 },
	{ 0x513a, 0x513c, PDF_CMAP_TABLE, 738 },
	{ 0x513d, 0x513e, PDF_CMAP_RANGE, 10514 },
	{ 0x513f, 0x514d, PDF_CMAP_TABLE, 741 },
	{ 0x514e, 0x5150, PDF_CMAP_RANGE, 10520 },
	{ 0x5151, 0x5156, PDF_CMAP_TABLE, 756 },
	{ 0x5157, 0x5159, PDF_CMAP_RANGE, 10524 },
	{ 0x515a, 0x515c, PDF_CMAP_TABLE, 762 },
	{ 0x515d, 0x5161, PDF_CMAP_RANGE, 10528 },
	{ 0x5162, 0x5162, PDF_CMAP_SINGLE, 2259 },
	{ 0x5163, 0x5164, PDF_CMAP_RANGE, 10533 },
	{ 0x5165, 0x5165, PDF_CMAP_SINGLE, 3270 },
	{ 0x5166, 0x5167, PDF_CMAP_RANGE, 10535 },
	{ 0x5168, 0x517d, PDF_CMAP_TABLE, 765 },
	{ 0x517e, 0x517f, PDF_CMAP_RANGE, 10541 },
	{ 0x5180, 0x5182, PDF_CMAP_TABLE, 787 },
	{ 0x5183, 0x5184, PDF_CMAP_RANGE, 10543 },
	{ 0x5185, 0x5185, PDF_CMAP_SINGLE, 2877 },
	{ 0x5186, 0x5187, PDF_CMAP_RANGE, 10545 },
	{ 0x5188, 0x5189, PDF_CMAP_TABLE, 790 },
	{ 0x518a, 0x518b, PDF_CMAP_RANGE, 10547 },
	{ 0x518c, 0x518d, PDF_CMAP_TABLE, 792 },
	{ 0x518e, 0x5191, PDF_CMAP_RANGE, 10549 },
	{ 0x5192, 0x5192, PDF_CMAP_SINGLE, 2741 },
	{ 0x5193, 0x5194, PDF_CMAP_RANGE, 10553 },
	{ 0x5195, 0x519c, PDF_CMAP_TABLE, 794 },
	{ 0x519d, 0x519f, PDF_CMAP_RANGE, 10557 },
	{ 0x51a0, 0x51a5, PDF_CMAP_TABLE, 802 },
	{ 0x51a6, 0x51aa, PDF_CMAP_RANGE, 10562 },
	{ 0x51ab, 0x51ac, PDF_CMAP_TABLE, 808 },
	{ 0x51ad, 0x51ae, PDF_CMAP_RANGE, 10567 },
	{ 0x51af, 0x51b7, PDF_CMAP_TABLE, 810 },
	{ 0x51b8, 0x51ba, PDF_CMAP_RANGE, 10570 },
	{ 0x51bb, 0x51bd, PDF_CMAP_TABLE, 819 },
	{ 0x51be, 0x51bf, PDF_CMAP_RANGE, 10573 },
	{ 0x51c0, 0x51c0, PDF_CMAP_SINGLE, 2282 },
	{ 0x51c1, 0x51c3, PDF_CMAP_RANGE, 10575 },
	{ 0x51c4, 0x51d1, PDF_CMAP_TABLE, 822 },
	{ 0x51d2, 0x51da, PDF_CMAP_RANGE, 10583 },
	{ 0x51db, 0x51dd, PDF_CMAP_TABLE, 836 },
	{ 0x51de, 0x51df, PDF_CMAP_RANGE, 10593 },
	{ 0x51e0, 0x51e1, PDF_CMAP_TABLE, 839 },
	{ 0x51e2, 0x51e3, PDF_CMAP_RANGE, 10595 },
	{ 0x51e4, 0x51e4, PDF_CMAP_SINGLE, 1676 },
	{ 0x51e5, 0x51ea, PDF_CMAP_RANGE, 10597 },
	{ 0x51eb, 0x5203, PDF_CMAP_TABLE, 841 },
	{ 0x5204, 0x5205, PDF_CMAP_RANGE, 10609 },
	{ 0x5206, 0x520a, PDF_CMAP_TABLE, 866 },
	{ 0x520b, 0x520c, PDF_CMAP_RANGE, 10612 },
	{ 0x520d, 0x520e, PDF_CMAP_TABLE, 871 },
	{ 0x520f, 0x5210, PDF_CMAP_RANGE, 10614 },
	{ 0x5211, 0x5212, PDF_CMAP_TABLE, 873 },
	{ 0x5213, 0x5215, PDF_CMAP_RANGE, 10616 },
	{ 0x5216, 0x521d, PDF_CMAP_TABLE, 875 },
	{ 0x521e, 0x521f, PDF_CMAP_RANGE, 10620 },
	{ 0x5220, 0x5220, PDF_CMAP_SINGLE, 3319 },
	{ 0x5221, 0x5223, PDF_CMAP_RANGE, 10622 },
	{ 0x5224, 0x5224, PDF_CMAP_SINGLE, 2961 },
	{ 0x5225, 0x5227, PDF_CMAP_RANGE, 10625 },
	{ 0x5228, 0x5230, PDF_CMAP_TABLE, 883 },
	{ 0x5231, 0x5232, PDF_CMAP_RANGE, 10631 },
	{ 0x5233, 0x5233, PDF_CMAP_SINGLE, 4752 },
	{ 0x5234, 0x5235, PDF_CMAP_RANGE, 10633 },
	{ 0x5236, 0x523e, PDF_CMAP_TABLE, 892 },
	{ 0x523f, 0x5240, PDF_CMAP_RANGE, 4753 },
	{ 0x5241, 0x5244, PDF_CMAP_TABLE, 901 },
	{ 0x5245, 0x5246, PDF_CMAP_RANGE, 10637 },
	{ 0x5247, 0x5247, PDF_CMAP_SINGLE, 8803 },
	{ 0x5248, 0x5249, PDF_CMAP_RANGE, 10639 },
	{ 0x524a, 0x524d, PDF_CMAP_TABLE, 905 },
	{ 0x524e, 0x524f, PDF_CMAP_RANGE, 10641 },
	{ 0x5250, 0x5251, PDF_CMAP_TABLE, 909 },
	{ 0x5252, 0x5253, PDF_CMAP_RANGE, 10643 },
	{ 0x5254, 0x5256, PDF_CMAP_TABLE, 911 },
	{ 0x5257, 0x525a, PDF_CMAP_RANGE, 10646 },
	{ 0x525b, 0x525e, PDF_CMAP_TABLE, 914 },
	{ 0x525f, 0x5260, PDF_CMAP_RANGE, 10651 },
	{ 0x5261, 0x5261, PDF_CMAP_SINGLE, 4757 },
	{ 0x5262, 0x5264, PDF_CMAP_RANGE, 10653 },
	{ 0x5265, 0x526a, PDF_CMAP_TABLE, 918 },
	{ 0x526b, 0x526d, PDF_CMAP_RANGE, 10658 },
	{ 0x526e, 0x526f, PDF_CMAP_TABLE, 924 },
	{ 0x5270, 0x5271, PDF_CMAP_RANGE, 10661 },
	{ 0x5272, 0x5275, PDF_CMAP_TABLE, 926 },
	{ 0x5276, 0x527c, PDF_CMAP_RANGE, 10664 },
	{ 0x527d, 0x5283, PDF_CMAP_TABLE, 930 },
	{ 0x5284, 0x5286, PDF_CMAP_RANGE, 10673 },
	{ 0x5287, 0x528d, PDF_CMAP_TABLE, 937 },
	{ 0x528e, 0x528f, PDF_CMAP_RANGE, 10677 },
	{ 0x5290, 0x5293, PDF_CMAP_TABLE, 944 },
	{ 0x5294, 0x529a, PDF_CMAP_RANGE, 10680 },
	{ 0x529b, 0x52a3, PDF_CMAP_TABLE, 948 },
	{ 0x52a4, 0x52a7, PDF_CMAP_RANGE, 10688 },
	{ 0x52a8, 0x52ab, PDF_CMAP_TABLE, 957 },
	{ 0x52ac, 0x52ad, PDF_CMAP_RANGE, 5006 },
	{ 0x52ae, 0x52b0, PDF_CMAP_RANGE, 10692 },
	{ 0x52b1, 0x52b3, PDF_CMAP_TABLE, 961 },
	{ 0x52b4, 0x52bd, PDF_CMAP_RANGE, 10695 },
	{ 0x52be, 0x52c3, PDF_CMAP_TABLE, 964 },
	{ 0x52c4, 0x52c6, PDF_CMAP_RANGE, 10707 },
	{ 0x52c7, 0x52cb, PDF_CMAP_TABLE, 970 },
	{ 0x52cc, 0x52cf, PDF_CMAP_RANGE, 10712 },
	{ 0x52d0, 0x52d2, PDF_CMAP_TABLE, 975 },
	{ 0x52d3, 0x52d4, PDF_CMAP_RANGE, 10717 },
	{ 0x52d5, 0x52df, PDF_CMAP_TABLE, 978 },
	{ 0x52e0, 0x52e1, PDF_CMAP_RANGE, 10722 },
	{ 0x52e2, 0x52e4, PDF_CMAP_TABLE, 989 },
	{ 0x52e5, 0x52ef, PDF_CMAP_RANGE, 10725 },
	{ 0x52f0, 0x52f1, PDF_CMAP_TABLE, 992 },
	{ 0x52f2, 0x52f4, PDF_CMAP_RANGE, 10736 },
	{ 0x52f5, 0x52f5, PDF_CMAP_SINGLE, 8209 },
	{ 0x52f6, 0x52f7, PDF_CMAP_RANGE, 10739 },
	{ 0x52f8, 0x52fa, PDF_CMAP_TABLE, 994 },
	{ 0x52fb, 0x52fd, PDF_CMAP_RANGE, 10741 },
	{ 0x52fe, 0x5300, PDF_CMAP_TABLE, 997 },
	{ 0x5301, 0x5304, PDF_CMAP_RANGE, 10744 },
	{ 0x5305, 0x5308, PDF_CMAP_TABLE, 1000 },
	{ 0x5309, 0x530c, PDF_CMAP_RANGE, 10749 },
	{ 0x530d, 0x5310, PDF_CMAP_TABLE, 1004 },
	{ 0x5311, 0x5314, PDF_CMAP_RANGE, 10754 },
	{ 0x5315, 0x531a, PDF_CMAP_TABLE, 1008 },
	{ 0x531b, 0x531c, PDF_CMAP_RANGE, 10759 },
	{ 0x531d, 0x531d, PDF_CMAP_SINGLE, 4372 },
	{ 0x531e, 0x531f, PDF_CMAP_RANGE, 10761 },
	{ 0x5320, 0x5323, PDF_CMAP_TABLE, 1014 },
	{ 0x5324, 0x5325, PDF_CMAP_RANGE, 10764 },
	{ 0x5326, 0x5326, PDF_CMAP_SINGLE, 4742 },
	{ 0x5327, 0x5329, PDF_CMAP_RANGE, 10766 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 1640 },
	{ 0x532b, 0x532c, PDF_CMAP_RANGE, 10769 },
	{ 0x532d, 0x5331, PDF_CMAP_TABLE, 1018 },
	{ 0x5332, 0x5338, PDF_CMAP_RANGE, 10772 },
	{ 0x5339, 0x533b, PDF_CMAP_TABLE, 1023 },
	{ 0x533c, 0x533d, PDF_CMAP_RANGE, 10779 },
	{ 0x533e, 0x534a, PDF_CMAP_TABLE, 1026 },
	{ 0x534b, 0x534d, PDF_CMAP_RANGE, 10784 },
	{ 0x534e, 0x5357, PDF_CMAP_TABLE, 1039 },
	{ 0x5358, 0x5359, PDF_CMAP_RANGE, 10788 },
	{ 0x535a, 0x536b, PDF_CMAP_TABLE, 1049 },
	{ 0x536c, 0x536d, PDF_CMAP_RANGE, 10795 },
	{ 0x536e, 0x537a, PDF_CMAP_TABLE, 1067 },
	{ 0x537b, 0x537e, PDF_CMAP_RANGE, 10800 },
	{ 0x537f, 0x537f, PDF_CMAP_SINGLE, 3169 },
	{ 0x5380, 0x5381, PDF_CMAP_RANGE, 10804 },
	{ 0x5382, 0x5386, PDF_CMAP_TABLE, 1080 },
	{ 0x5387, 0x5388, PDF_CMAP_RANGE, 10807 },
	{ 0x5389, 0x538d, PDF_CMAP_TABLE, 1085 },
	{ 0x538e, 0x5394, PDF_CMAP_RANGE, 10810 },
	{ 0x5395, 0x5395, PDF_CMAP_SINGLE, 1189 },
	{ 0x5396, 0x5397, PDF_CMAP_RANGE, 10817 },
	{ 0x5398, 0x539a, PDF_CMAP_TABLE, 1090 },
	{ 0x539b, 0x539c, PDF_CMAP_RANGE, 10819 },
	{ 0x539d, 0x53a9, PDF_CMAP_TABLE, 1093 },
	{ 0x53aa, 0x53ac, PDF_CMAP_RANGE, 10825 },
	{ 0x53ad, 0x53ae, PDF_CMAP_TABLE, 1106 },
	{ 0x53af, 0x53b1, PDF_CMAP_RANGE, 10828 },
	{ 0x53b2, 0x53b6, PDF_CMAP_TABLE, 1108 },
	{ 0x53b7, 0x53ba, PDF_CMAP_RANGE, 10833 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 3200 },
	{ 0x53bc, 0x53be, PDF_CMAP_RANGE, 10837 },
	{ 0x53bf, 0x53c3, PDF_CMAP_TABLE, 1113 },
	{ 0x53c4, 0x53c7, PDF_CMAP_RANGE, 10841 },
	{ 0x53c8, 0x53cd, PDF_CMAP_TABLE, 1118 },
	{ 0x53ce, 0x53d0, PDF_CMAP_RANGE, 10845 },
	{ 0x53d1, 0x53d1, PDF_CMAP_SINGLE, 1599 },
	{ 0x53d2, 0x53d3, PDF_CMAP_RANGE, 10848 },
	{ 0x53d4, 0x53db, PDF_CMAP_TABLE, 1124 },
	{ 0x53dc, 0x53de, PDF_CMAP_RANGE, 10852 },
	{ 0x53df, 0x53fd, PDF_CMAP_TABLE, 1132 },
	{ 0x53fe, 0x5400, PDF_CMAP_RANGE, 10859 },
	{ 0x5401, 0x5417, PDF_CMAP_TABLE, 1163 },
	{ 0x5418, 0x541a, PDF_CMAP_RANGE, 10867 },
	{ 0x541b, 0x5423, PDF_CMAP_TABLE, 1186 },
	{ 0x5424, 0x5425, PDF_CMAP_RANGE, 10872 },
	{ 0x5426, 0x5435, PDF_CMAP_TABLE, 1195 },
	{ 0x5436, 0x5437, PDF_CMAP_RANGE, 10877 },
	{ 0x5438, 0x5440, PDF_CMAP_TABLE, 1211 },
	{ 0x5441, 0x5442, PDF_CMAP_RANGE, 10882 },
	{ 0x5443, 0x5443, PDF_CMAP_SINGLE, 5384 },
	{ 0x5444, 0x5445, PDF_CMAP_RANGE, 10884 },
	{ 0x5446, 0x544b, PDF_CMAP_TABLE, 1220 },
	{ 0x544c, 0x544f, PDF_CMAP_RANGE, 10888 },
	{ 0x5450, 0x5451, PDF_CMAP_TABLE, 1226 },
	{ 0x5452, 0x5454, PDF_CMAP_RANGE, 5380 },
	{ 0x5455, 0x545c, PDF_CMAP_TABLE, 1228 },
	{ 0x545d, 0x5461, PDF_CMAP_RANGE, 10894 },
	{ 0x5462, 0x5468, PDF_CMAP_TABLE, 1236 },
	{ 0x5469, 0x5470, PDF_CMAP_RANGE, 10902 },
	{ 0x5471, 0x5478, PDF_CMAP_TABLE, 1243 },
	{ 0x5479, 0x547a, PDF_CMAP_RANGE, 10911 },
	{ 0x547b, 0x547d, PDF_CMAP_TABLE, 1251 },
	{ 0x547e, 0x547f, PDF_CMAP_RANGE, 10913 },
	{ 0x5480, 0x5486, PDF_CMAP_TABLE, 1254 },
	{ 0x5487, 0x548a, PDF_CMAP_RANGE, 10918 },
	{ 0x548b, 0x5496, PDF_CMAP_TABLE, 1261 },
	{ 0x5497, 0x5498, PDF_CMAP_RANGE, 10925 },
	{ 0x5499, 0x5499, PDF_CMAP_SINGLE, 2629 },
	{ 0x549a, 0x549b, PDF_CMAP_RANGE, 5395 },
	{ 0x549c, 0x549d, PDF_CMAP_TABLE, 1273 },
	{ 0x549e, 0x54a2, PDF_CMAP_RANGE, 10928 },
	{ 0x54a3, 0x54a8, PDF_CMAP_TABLE, 1275 },
	{ 0x54a9, 0x54aa, PDF_CMAP_RANGE, 5419 },
	{ 0x54ab, 0x54b4, PDF_CMAP_TABLE, 1281 },
	{ 0x54b5, 0x54b7, PDF_CMAP_RANGE, 10937 },
	{ 0x54b8, 0x54b8, PDF_CMAP_SINGLE, 3897 },
	{ 0x54b9, 0x54ba, PDF_CMAP_RANGE, 10940 },
	{ 0x54bb, 0x54c9, PDF_CMAP_TABLE, 1291 },
	{ 0x54ca, 0x54cb, PDF_CMAP_RANGE, 10945 },
	{ 0x54cc, 0x54d2, PDF_CMAP_TABLE, 1306 },
	{ 0x54d3, 0x54d4, PDF_CMAP_RANGE, 5408 },
	{ 0x54d5, 0x54d8, PDF_CMAP_TABLE, 1313 },
	{ 0x54d9, 0x54da, PDF_CMAP_RANGE, 5416 },
	{ 0x54db, 0x54e1, PDF_CMAP_TABLE, 1317 },
	{ 0x54e2, 0x54e4, PDF_CMAP_RANGE, 10951 },
	{ 0x54e5, 0x54ea, PDF_CMAP_TABLE, 1324 },
	{ 0x54eb, 0x54ec, PDF_CMAP_RANGE, 10954 },
	{ 0x54ed, 0x54ee, PDF_CMAP_TABLE, 1330 },
	{ 0x54ef, 0x54f1, PDF_CMAP_RANGE, 10956 },
	{ 0x54f2, 0x54f3, PDF_CMAP_TABLE, 1332 },
	{ 0x54f4, 0x54f9, PDF_CMAP_RANGE, 10959 },
	{ 0x54fa, 0x5501, PDF_CMAP_TABLE, 1334 },
	{ 0x5502, 0x5503, PDF_CMAP_RANGE, 10968 },
	{ 0x5504, 0x5509, PDF_CMAP_TABLE, 1342 },
	{ 0x550a, 0x550e, PDF_CMAP_RANGE, 10972 },
	{ 0x550f, 0x5511, PDF_CMAP_TABLE, 1348 },
	{ 0x5512, 0x5513, PDF_CMAP_RANGE, 10977 },
	{ 0x5514, 0x5514, PDF_CMAP_SINGLE, 5429 },
	{ 0x5515, 0x551a, PDF_CMAP_RANGE, 10979 },
	{ 0x551b, 0x551b, PDF_CMAP_SINGLE, 5425 },
	{ 0x551c, 0x551f, PDF_CMAP_RANGE, 10985 },
	{ 0x5520, 0x5521, PDF_CMAP_TABLE, 1351 },
	{ 0x5522, 0x5523, PDF_CMAP_RANGE, 5431 },
	{ 0x5524, 0x5524, PDF_CMAP_SINGLE, 2002 },
	{ 0x5525, 0x5526, PDF_CMAP_RANGE, 10990 },
	{ 0x5527, 0x5527, PDF_CMAP_SINGLE, 5435 },
	{ 0x5528, 0x5529, PDF_CMAP_RANGE, 10992 },
	{ 0x552a, 0x5533, PDF_CMAP_TABLE, 1353 },
	{ 0x5534, 0x5536, PDF_CMAP_RANGE, 10997 },
	{ 0x5537, 0x5537, PDF_CMAP_SINGLE, 5447 },
	{ 0x5538, 0x553b, PDF_CMAP_RANGE, 11000 },
	{ 0x553c, 0x5546, PDF_CMAP_TABLE, 1363 },
	{ 0x5547, 0x5548, PDF_CMAP_RANGE, 11008 },
	{ 0x5549, 0x554a, PDF_CMAP_TABLE, 1374 },
	{ 0x554b, 0x554e, PDF_CMAP_RANGE, 11010 },
	{ 0x554f, 0x5550, PDF_CMAP_TABLE, 1376 },
	{ 0x5551, 0x5552, PDF_CMAP_RANGE, 11014 },
	{ 0x5553, 0x5556, PDF_CMAP_TABLE, 1378 },
	{ 0x5557, 0x555b, PDF_CMAP_RANGE, 11017 },
	{ 0x555c, 0x555e, PDF_CMAP_TABLE, 1382 },
	{ 0x555f, 0x5560, PDF_CMAP_RANGE, 11023 },
	{ 0x5561, 0x5561, PDF_CMAP_SINGLE, 1637 },
	{ 0x5562, 0x5563, PDF_CMAP_RANGE, 11025 },
	{ 0x5564, 0x5567, PDF_CMAP_TABLE, 1385 },
	{ 0x5568, 0x5569, PDF_CMAP_RANGE, 11027 },
	{ 0x556a, 0x556e, PDF_CMAP_TABLE, 1389 },
	{ 0x556f, 0x5574, PDF_CMAP_RANGE, 11030 },
	{ 0x5575, 0x5577, PDF_CMAP_RANGE, 5449 },
	{ 0x5578, 0x5578, PDF_CMAP_SINGLE, 3952 },
	{ 0x5579, 0x557a, PDF_CMAP_RANGE, 11036 },
	{ 0x557b, 0x5584, PDF_CMAP_TABLE, 1394 },
	{ 0x5585, 0x5586, PDF_CMAP_RANGE, 11040 },
	{ 0x5587, 0x558b, PDF_CMAP_TABLE, 1404 },
	{ 0x558c, 0x558e, PDF_CMAP_RANGE, 11042 },
	{ 0x558f, 0x5591, PDF_CMAP_TABLE, 1409 },
	{ 0x5592, 0x5593, PDF_CMAP_RANGE, 11046 },
	{ 0x5594, 0x5594, PDF_CMAP_SINGLE, 5470 },
	{ 0x5595, 0x5597, PDF_CMAP_RANGE, 11048 },
	{ 0x5598, 0x5599, PDF_CMAP_TABLE, 1412 },
	{ 0x559a, 0x559b, PDF_CMAP_RANGE, 11051 },
	{ 0x559c, 0x559f, PDF_CMAP_TABLE, 1414 },
	{ 0x55a0, 0x55a6, PDF_CMAP_RANGE, 11054 },
	{ 0x55a7, 0x55a7, PDF_CMAP_SINGLE, 4037 },
	{ 0x55a8, 0x55a9, PDF_CMAP_RANGE, 11061 },
	{ 0x55aa, 0x55ae, PDF_CMAP_TABLE, 1418 },
	{ 0x55af, 0x55b0, PDF_CMAP_RANGE, 11065 },
	{ 0x55b1, 0x55bc, PDF_CMAP_TABLE, 1423 },
	{ 0x55bd, 0x55be, PDF_CMAP_RANGE, 5468 },
	{ 0x55bf, 0x55c3, PDF_CMAP_RANGE, 11072 },
	{ 0x55c4, 0x55c9, PDF_CMAP_TABLE, 1435 },
	{ 0x55ca, 0x55cb, PDF_CMAP_RANGE, 11078 },
	{ 0x55cc, 0x55cd, PDF_CMAP_RANGE, 5487 },
	{ 0x55ce, 0x55ce, PDF_CMAP_SINGLE, 8303 },
	{ 0x55cf, 0x55d0, PDF_CMAP_RANGE, 11080 },
	{ 0x55d1, 0x55d6, PDF_CMAP_TABLE, 1441 },
	{ 0x55d7, 0x55d9, PDF_CMAP_RANGE, 11083 },
	{ 0x55da, 0x55ea, PDF_CMAP_TABLE, 1447 },
	{ 0x55eb, 0x55ec, PDF_CMAP_RANGE, 5477 },
	{ 0x55ed, 0x55ee, PDF_CMAP_RANGE, 11091 },
	{ 0x55ef, 0x55ef, PDF_CMAP_SINGLE, 5483 },
	{ 0x55f0, 0x55f1, PDF_CMAP_RANGE, 11093 },
	{ 0x55f2, 0x55f3, PDF_CMAP_RANGE, 5485 },
	{ 0x55f4, 0x55f7, PDF_CMAP_TABLE, 1464 },
	{ 0x55f8, 0x55fc, PDF_CMAP_RANGE, 11096 },
	{ 0x55fd, 0x5601, PDF_CMAP_TABLE, 1468 },
	{ 0x5602, 0x5605, PDF_CMAP_RANGE, 11102 },
	{ 0x5606, 0x5609, PDF_CMAP_TABLE, 1473 },
	{ 0x560a, 0x560b, PDF_CMAP_RANGE, 11107 },
	{ 0x560c, 0x560f, PDF_CMAP_TABLE, 1477 },
	{ 0x5610, 0x5613, PDF_CMAP_RANGE, 11109 },
	{ 0x5614, 0x5618, PDF_CMAP_TABLE, 1481 },
	{ 0x5619, 0x561a, PDF_CMAP_RANGE, 11114 },
	{ 0x561b, 0x561f, PDF_CMAP_TABLE, 1486 },
	{ 0x5620, 0x5622, PDF_CMAP_RANGE, 11117 },
	{ 0x5623, 0x5624, PDF_CMAP_TABLE, 1491 },
	{ 0x5625, 0x5626, PDF_CMAP_RANGE, 11120 },
	{ 0x5627, 0x5629, PDF_CMAP_TABLE, 1493 },
	{ 0x562a, 0x562b, PDF_CMAP_RANGE, 11123 },
	{ 0x562c, 0x563b, PDF_CMAP_TABLE, 1496 },
	{ 0x563c, 0x563e, PDF_CMAP_RANGE, 11128 },
	{ 0x563f, 0x5641, PDF_CMAP_TABLE, 1512 },
	{ 0x5642, 0x564b, PDF_CMAP_RANGE, 11132 },
	{ 0x564c, 0x564e, PDF_CMAP_TABLE, 1515 },
	{ 0x564f, 0x5653, PDF_CMAP_RANGE, 11142 },
	{ 0x5654, 0x5654, PDF_CMAP_SINGLE, 5512 },
	{ 0x5655, 0x5656, PDF_CMAP_RANGE, 11147 },
	{ 0x5657, 0x5659, PDF_CMAP_TABLE, 1518 },
	{ 0x565a, 0x565b, PDF_CMAP_RANGE, 11149 },
	{ 0x565c, 0x565d, PDF_CMAP_TABLE, 1521 },
	{ 0x565e, 0x565f, PDF_CMAP_RANGE, 11151 },
	{ 0x5660, 0x566c, PDF_CMAP_TABLE, 1523 },
	{ 0x566d, 0x566e, PDF_CMAP_RANGE, 11156 },
	{ 0x566f, 0x567a, PDF_CMAP_TABLE, 1536 },
	{ 0x567b, 0x567c, PDF_CMAP_RANGE, 5517 },
	{ 0x567d, 0x567f, PDF_CMAP_RANGE, 11163 },
	{ 0x5680, 0x5680, PDF_CMAP_SINGLE, 9070 },
	{ 0x5681, 0x5684, PDF_CMAP_RANGE, 11166 },
	{ 0x5685, 0x5687, PDF_CMAP_TABLE, 1548 },
	{ 0x5688, 0x568b, PDF_CMAP_RANGE, 11170 },
	{ 0x568c, 0x568f, PDF_CMAP_TABLE, 1551 },
	{ 0x5690, 0x5692, PDF_CMAP_RANGE, 11175 },
	{ 0x5693, 0x5695, PDF_CMAP_TABLE, 1555 },
	{ 0x5696, 0x5698, PDF_CMAP_RANGE, 11179 },
	{ 0x5699, 0x5699, PDF_CMAP_SINGLE, 8350 },
	{ 0x569a, 0x56a2, PDF_CMAP_RANGE, 11182 },
	{ 0x56a3, 0x56a3, PDF_CMAP_SINGLE, 3942 },
	{ 0x56a4, 0x56a5, PDF_CMAP_RANGE, 11191 },
	{ 0x56a6, 0x56a8, PDF_CMAP_TABLE, 1558 },
	{ 0x56a9, 0x56ad, PDF_CMAP_RANGE, 11194 },
	{ 0x56ae, 0x56af, PDF_CMAP_TABLE, 1561 },
	{ 0x56b0, 0x56b2, PDF_CMAP_RANGE, 11199 },
	{ 0x56b3, 0x56b7, PDF_CMAP_TABLE, 1563 },
	{ 0x56b8, 0x56bb, PDF_CMAP_RANGE, 11203 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 2193 },
	{ 0x56bd, 0x56bf, PDF_CMAP_RANGE, 11207 },
	{ 0x56c0, 0x56c2, PDF_CMAP_TABLE, 1568 },
	{ 0x56c3, 0x56c4, PDF_CMAP_RANGE, 11210 },
	{ 0x56c5, 0x56c5, PDF_CMAP_SINGLE, 8923 },
	{ 0x56c6, 0x56c7, PDF_CMAP_RANGE, 11212 },
	{ 0x56c8, 0x56cc, PDF_CMAP_TABLE, 1571 },
	{ 0x56cd, 0x56d0, PDF_CMAP_RANGE, 11215 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 8865 },
	{ 0x56d2, 0x56d3, PDF_CMAP_RANGE, 11219 },
	{ 0x56d4, 0x56d4, PDF_CMAP_SINGLE, 5522 },
	{ 0x56d5, 0x56d6, PDF_CMAP_RANGE, 11221 },
	{ 0x56d7, 0x56d7, PDF_CMAP_SINGLE, 5523 },
	{ 0x56d8, 0x56d9, PDF_CMAP_RANGE, 11223 },
	{ 0x56da, 0x56e4, PDF_CMAP_TABLE, 1576 },
	{ 0x56e5, 0x56ea, PDF_CMAP_RANGE, 11227 },
	{ 0x56eb, 0x56ed, PDF_CMAP_TABLE, 1587 },
	{ 0x56ee, 0x56ef, PDF_CMAP_RANGE, 11234 },
	{ 0x56f0, 0x56f1, PDF_CMAP_TABLE, 1590 },
	{ 0x56f2, 0x56f3, PDF_CMAP_RANGE, 11236 },
	{ 0x56f4, 0x56f5, PDF_CMAP_TABLE, 1592 },
	{ 0x56f6, 0x56f8, PDF_CMAP_RANGE, 11238 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 1594 },
	{ 0x56fb, 0x56fc, PDF_CMAP_RANGE, 11241 },
	{ 0x56fd, 0x56ff, PDF_CMAP_TABLE, 1596 },
	{ 0x5700, 0x5702, PDF_CMAP_RANGE, 11243 },
	{ 0x5703, 0x570d, PDF_CMAP_TABLE, 1599 },
	{ 0x570e, 0x5711, PDF_CMAP_RANGE, 11248 },
	{ 0x5712, 0x5713, PDF_CMAP_TABLE, 1610 },
	{ 0x5714, 0x5715, PDF_CMAP_RANGE, 11252 },
	{ 0x5716, 0x5718, PDF_CMAP_TABLE, 1612 },
	{ 0x5719, 0x571b, PDF_CMAP_RANGE, 11255 },
	{ 0x571c, 0x571c, PDF_CMAP_SINGLE, 5533 },
	{ 0x571d, 0x571e, PDF_CMAP_RANGE, 11258 },
	{ 0x571f, 0x571f, PDF_CMAP_SINGLE, 3698 },
	{ 0x5720, 0x5722, PDF_CMAP_RANGE, 11260 },
	{ 0x5723, 0x5723, PDF_CMAP_SINGLE, 3388 },
	{ 0x5724, 0x5727, PDF_CMAP_RANGE, 11263 },
	{ 0x5728, 0x572d, PDF_CMAP_TABLE, 1615 },
	{ 0x572e, 0x572f, PDF_CMAP_RANGE, 5036 },
	{ 0x5730, 0x5730, PDF_CMAP_SINGLE, 1465 },
	{ 0x5731, 0x5732, PDF_CMAP_RANGE, 11268 },
	{ 0x5733, 0x5733, PDF_CMAP_SINGLE, 5034 },
	{ 0x5734, 0x5738, PDF_CMAP_RANGE, 11270 },
	{ 0x5739, 0x573b, PDF_CMAP_TABLE, 1621 },
	{ 0x573c, 0x573d, PDF_CMAP_RANGE, 11275 },
	{ 0x573e, 0x5742, PDF_CMAP_TABLE, 1624 },
	{ 0x5743, 0x5746, PDF_CMAP_RANGE, 11279 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 2344 },
	{ 0x5748, 0x5749, PDF_CMAP_RANGE, 11283 },
	{ 0x574a, 0x5751, PDF_CMAP_TABLE, 1629 },
	{ 0x5752, 0x5756, PDF_CMAP_RANGE, 11286 },
	{ 0x5757, 0x5757, PDF_CMAP_SINGLE, 2422 },
	{ 0x5758, 0x5759, PDF_CMAP_RANGE, 11291 },
	{ 0x575a, 0x5761, PDF_CMAP_TABLE, 1637 },
	{ 0x5762, 0x5763, PDF_CMAP_RANGE, 11293 },
	{ 0x5764, 0x576f, PDF_CMAP_TABLE, 1645 },
	{ 0x5770, 0x5772, PDF_CMAP_RANGE, 11299 },
	{ 0x5773, 0x5773, PDF_CMAP_SINGLE, 5050 },
	{ 0x5774, 0x5775, PDF_CMAP_RANGE, 11302 },
	{ 0x5776, 0x5777, PDF_CMAP_TABLE, 1657 },
	{ 0x5778, 0x577a, PDF_CMAP_RANGE, 11304 },
	{ 0x577b, 0x577c, PDF_CMAP_TABLE, 1659 },
	{ 0x577d, 0x5781, PDF_CMAP_RANGE, 11307 },
	{ 0x5782, 0x5786, PDF_CMAP_TABLE, 1661 },
	{ 0x5787, 0x578a, PDF_CMAP_RANGE, 11312 },
	{ 0x578b, 0x578c, PDF_CMAP_TABLE, 1666 },
	{ 0x578d, 0x5791, PDF_CMAP_RANGE, 11316 },
	{ 0x5792, 0x5793, PDF_CMAP_TABLE, 1668 },
	{ 0x5794, 0x579a, PDF_CMAP_RANGE, 11321 },
	{ 0x579b, 0x579b, PDF_CMAP_SINGLE, 1569 },
	{ 0x579c, 0x579f, PDF_CMAP_RANGE, 11328 },
	{ 0x57a0, 0x57ae, PDF_CMAP_TABLE, 1670 },
	{ 0x57af, 0x57b1, PDF_CMAP_RANGE, 11336 },
	{ 0x57b2, 0x57b4, PDF_CMAP_TABLE, 1685 },
	{ 0x57b5, 0x57b7, PDF_CMAP_RANGE, 11340 },
	{ 0x57b8, 0x57b8, PDF_CMAP_SINGLE, 5065 },
	{ 0x57b9, 0x57c1, PDF_CMAP_RANGE, 11343 },
	{ 0x57c2, 0x57c3, PDF_CMAP_TABLE, 1688 },
	{ 0x57c4, 0x57ca, PDF_CMAP_RANGE, 11352 },
	{ 0x57cb, 0x57cb, PDF_CMAP_SINGLE, 2712 },
	{ 0x57cc, 0x57cd, PDF_CMAP_RANGE, 11359 },
	{ 0x57ce, 0x57cf, PDF_CMAP_TABLE, 1690 },
	{ 0x57d0, 0x57d1, PDF_CMAP_RANGE, 11361 },
	{ 0x57d2, 0x57d5, PDF_CMAP_TABLE, 1692 },
	{ 0x57d6, 0x57d7, PDF_CMAP_RANGE, 11364 },
	{ 0x57d8, 0x57da, PDF_CMAP_TABLE, 1696 },
	{ 0x57db, 0x57dc, PDF_CMAP_RANGE, 11366 },
	{ 0x57dd, 0x57e1, PDF_CMAP_TABLE, 1699 },
	{ 0x57e2, 0x57e3, PDF_CMAP_RANGE, 11369 },
	{ 0x57e4, 0x57e4, PDF_CMAP_SINGLE, 5069 },
	{ 0x57e5, 0x57ec, PDF_CMAP_RANGE, 11371 },
	{ 0x57ed, 0x57ef, PDF_CMAP_TABLE, 1704 },
	{ 0x57f0, 0x57f3, PDF_CMAP_RANGE, 11380 },
	{ 0x57f4, 0x57f4, PDF_CMAP_SINGLE, 5066 },
	{ 0x57f5, 0x57f6, PDF_CMAP_RANGE, 11384 },
	{ 0x57f7, 0x57fa, PDF_CMAP_TABLE, 1707 },
	{ 0x57fb, 0x57fc, PDF_CMAP_RANGE, 11386 },
	{ 0x57fd, 0x57fd, PDF_CMAP_SINGLE, 5073 },
	{ 0x57fe, 0x57ff, PDF_CMAP_RANGE, 11388 },
	{ 0x5800, 0x5802, PDF_CMAP_TABLE, 1711 },
	{ 0x5803, 0x5804, PDF_CMAP_RANGE, 11391 },
	{ 0x5805, 0x5807, PDF_CMAP_TABLE, 1714 },
	{ 0x5808, 0x5809, PDF_CMAP_RANGE, 11393 },
	{ 0x580a, 0x580d, PDF_CMAP_TABLE, 1717 },
	{ 0x580e, 0x5810, PDF_CMAP_RANGE, 11396 },
	{ 0x5811, 0x5811, PDF_CMAP_SINGLE, 3122 },
	{ 0x5812, 0x5814, PDF_CMAP_RANGE, 11399 },
	{ 0x5815, 0x5816, PDF_CMAP_TABLE, 1721 },
	{ 0x5817, 0x5818, PDF_CMAP_RANGE, 11402 },
	{ 0x5819, 0x5819, PDF_CMAP_SINGLE, 5077 },
	{ 0x581a, 0x581c, PDF_CMAP_RANGE, 11404 },
	{ 0x581d, 0x5821, PDF_CMAP_TABLE, 1723 },
	{ 0x5822, 0x5823, PDF_CMAP_RANGE, 11408 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 1453 },
	{ 0x5825, 0x5829, PDF_CMAP_RANGE, 11410 },
	{ 0x582a, 0x582a, PDF_CMAP_SINGLE, 2365 },
	{ 0x582b, 0x582e, PDF_CMAP_RANGE, 11415 },
	{ 0x582f, 0x5831, PDF_CMAP_TABLE, 1728 },
	{ 0x5832, 0x5833, PDF_CMAP_RANGE, 11419 },
	{ 0x5834, 0x5835, PDF_CMAP_TABLE, 1731 },
	{ 0x5836, 0x5843, PDF_CMAP_RANGE, 11421 },
	{ 0x5844, 0x5844, PDF_CMAP_SINGLE, 5078 },
	{ 0x5845, 0x5849, PDF_CMAP_RANGE, 11435 },
	{ 0x584a, 0x5854, PDF_CMAP_TABLE, 1733 },
	{ 0x5855, 0x5856, PDF_CMAP_RANGE, 11443 },
	{ 0x5857, 0x5858, PDF_CMAP_TABLE, 1744 },
	{ 0x5859, 0x585d, PDF_CMAP_RANGE, 11445 },
	{ 0x585e, 0x585e, PDF_CMAP_SINGLE, 3286 },
	{ 0x585f, 0x5861, PDF_CMAP_RANGE, 11450 },
	{ 0x5862, 0x5865, PDF_CMAP_TABLE, 1746 },
	{ 0x5866, 0x586a, PDF_CMAP_RANGE, 11454 },
	{ 0x586b, 0x586c, PDF_CMAP_TABLE, 1750 },
	{ 0x586d, 0x5874, PDF_CMAP_RANGE, 11459 },
	{ 0x5875, 0x5875, PDF_CMAP_SINGLE, 7805 },
	{ 0x5876, 0x5878, PDF_CMAP_RANGE, 11467 },
	{ 0x5879, 0x5879, PDF_CMAP_SINGLE, 8406 },
	{ 0x587a, 0x587d, PDF_CMAP_RANGE, 11470 },
	{ 0x587e, 0x5885, PDF_CMAP_TABLE, 1752 },
	{ 0x5886, 0x5888, PDF_CMAP_RANGE, 11477 },
	{ 0x5889, 0x588a, PDF_CMAP_TABLE, 1760 },
	{ 0x588b, 0x5891, PDF_CMAP_RANGE, 11480 },
	{ 0x5892, 0x5893, PDF_CMAP_TABLE, 1762 },
	{ 0x5894, 0x5898, PDF_CMAP_RANGE, 11487 },
	{ 0x5899, 0x589f, PDF_CMAP_TABLE, 1764 },
	{ 0x58a0, 0x58a7, PDF_CMAP_RANGE, 11494 },
	{ 0x58a8, 0x58a9, PDF_CMAP_TABLE, 1771 },
	{ 0x58aa, 0x58ad, PDF_CMAP_RANGE, 11502 },
	{ 0x58ae, 0x58ae, PDF_CMAP_SINGLE, 7903 },
	{ 0x58af, 0x58b2, PDF_CMAP_RANGE, 11506 },
	{ 0x58b3, 0x58b3, PDF_CMAP_SINGLE, 7929 },
	{ 0x58b4, 0x58ba, PDF_CMAP_RANGE, 11510 },
	{ 0x58bb, 0x58be, PDF_CMAP_TABLE, 1773 },
	{ 0x58bf, 0x58c0, PDF_CMAP_RANGE, 11518 },
	{ 0x58c1, 0x58c1, PDF_CMAP_SINGLE, 1091 },
	{ 0x58c2, 0x58c4, PDF_CMAP_RANGE, 11520 },
	{ 0x58c5, 0x58c7, PDF_CMAP_TABLE, 1777 },
	{ 0x58c8, 0x58d0, PDF_CMAP_RANGE, 11524 },
	{ 0x58d1, 0x58d5, PDF_CMAP_TABLE, 1780 },
	{ 0x58d6, 0x58d7, PDF_CMAP_RANGE, 11535 },
	{ 0x58d8, 0x58da, PDF_CMAP_TABLE, 1785 },
	{ 0x58db, 0x58dd, PDF_CMAP_RANGE, 11537 },
	{ 0x58de, 0x58e4, PDF_CMAP_TABLE, 1788 },
	{ 0x58e5, 0x58e8, PDF_CMAP_RANGE, 11542 },
	{ 0x58e9, 0x58f0, PDF_CMAP_TABLE, 1795 },
	{ 0x58f1, 0x58f2, PDF_CMAP_RANGE, 11548 },
	{ 0x58f3, 0x58f3, PDF_CMAP_SINGLE, 2388 },
	{ 0x58f4, 0x58f5, PDF_CMAP_RANGE, 11550 },
	{ 0x58f6, 0x58f6, PDF_CMAP_SINGLE, 1967 },
	{ 0x58f7, 0x58f8, PDF_CMAP_RANGE, 11552 },
	{ 0x58f9, 0x58fa, PDF_CMAP_TABLE, 1803 },
	{ 0x58fb, 0x58fc, PDF_CMAP_RANGE, 11554 },
	{ 0x58fd, 0x58fd, PDF_CMAP_SINGLE, 8504 },
	{ 0x58fe, 0x5901, PDF_CMAP_RANGE, 11556 },
	{ 0x5902, 0x5904, PDF_CMAP_TABLE, 1805 },
	{ 0x5905, 0x5906, PDF_CMAP_RANGE, 11561 },
	{ 0x5907, 0x5907, PDF_CMAP_SINGLE, 1057 },
	{ 0x5908, 0x590c, PDF_CMAP_RANGE, 11563 },
	{ 0x590d, 0x590f, PDF_CMAP_TABLE, 1808 },
	{ 0x5910, 0x5913, PDF_CMAP_RANGE, 11569 },
	{ 0x5914, 0x5916, PDF_CMAP_TABLE, 1811 },
	{ 0x5917, 0x5918, PDF_CMAP_RANGE, 11573 },
	{ 0x5919, 0x591c, PDF_CMAP_TABLE, 1814 },
	{ 0x591d, 0x591e, PDF_CMAP_RANGE, 11576 },
	{ 0x591f, 0x591f, PDF_CMAP_SINGLE, 1812 },
	{ 0x5920, 0x5921, PDF_CMAP_RANGE, 11578 },
	{ 0x5922, 0x5931, PDF_CMAP_TABLE, 1818 },
	{ 0x5932, 0x5933, PDF_CMAP_RANGE, 11585 },
	{ 0x5934, 0x5934, PDF_CMAP_SINGLE, 3688 },
	{ 0x5935, 0x5936, PDF_CMAP_RANGE, 11587 },
	{ 0x5937, 0x593e, PDF_CMAP_TABLE, 1834 },
	{ 0x593f, 0x5940, PDF_CMAP_RANGE, 11591 },
	{ 0x5941, 0x5944, PDF_CMAP_TABLE, 1842 },
	{ 0x5945, 0x5946, PDF_CMAP_RANGE, 11594 },
	{ 0x5947, 0x594b, PDF_CMAP_TABLE, 1846 },
	{ 0x594c, 0x594d, PDF_CMAP_RANGE, 11597 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 1851 },
	{ 0x5952, 0x5953, PDF_CMAP_RANGE, 11600 },
	{ 0x5954, 0x595a, PDF_CMAP_TABLE, 1855 },
	{ 0x595b, 0x595f, PDF_CMAP_RANGE, 11603 },
	{ 0x5960, 0x5962, PDF_CMAP_TABLE, 1862 },
	{ 0x5963, 0x5964, PDF_CMAP_RANGE, 11609 },
	{ 0x5965, 0x5965, PDF_CMAP_SINGLE, 973 },
	{ 0x5966, 0x5968, PDF_CMAP_RANGE, 11611 },
	{ 0x5969, 0x596e, PDF_CMAP_TABLE, 1865 },
	{ 0x596f, 0x5972, PDF_CMAP_RANGE, 11616 },
	{ 0x5973, 0x5979, PDF_CMAP_TABLE, 1871 },
	{ 0x597a, 0x597c, PDF_CMAP_RANGE, 11622 },
	{ 0x597d, 0x597d, PDF_CMAP_SINGLE, 1914 },
	{ 0x597e, 0x5980, PDF_CMAP_RANGE, 11625 },
	{ 0x5981, 0x598a, PDF_CMAP_TABLE, 1878 },
	{ 0x598b, 0x598c, PDF_CMAP_RANGE, 11630 },
	{ 0x598d, 0x598d, PDF_CMAP_SINGLE, 6007 },
	{ 0x598e, 0x5991, PDF_CMAP_RANGE, 11632 },
	{ 0x5992, 0x5993, PDF_CMAP_TABLE, 1888 },
	{ 0x5994, 0x5995, PDF_CMAP_RANGE, 11636 },
	{ 0x5996, 0x5999, PDF_CMAP_TABLE, 1890 },
	{ 0x599a, 0x599c, PDF_CMAP_RANGE, 11639 },
	{ 0x599d, 0x599e, PDF_CMAP_TABLE, 1894 },
	{ 0x599f, 0x59a2, PDF_CMAP_RANGE, 11642 },
	{ 0x59a3, 0x59a5, PDF_CMAP_TABLE, 1896 },
	{ 0x59a6, 0x59a7, PDF_CMAP_RANGE, 11646 },
	{ 0x59a8, 0x59a8, PDF_CMAP_SINGLE, 1630 },
	{ 0x59a9, 0x59aa, PDF_CMAP_RANGE, 6008 },
	{ 0x59ab, 0x59ab, PDF_CMAP_SINGLE, 6013 },
	{ 0x59ac, 0x59ad, PDF_CMAP_RANGE, 11648 },
	{ 0x59ae, 0x59af, PDF_CMAP_TABLE, 1899 },
	{ 0x59b0, 0x59b1, PDF_CMAP_RANGE, 11650 },
	{ 0x59b2, 0x59b2, PDF_CMAP_SINGLE, 6017 },
	{ 0x59b3, 0x59b8, PDF_CMAP_RANGE, 11652 },
	{ 0x59b9, 0x59bb, PDF_CMAP_TABLE, 1901 },
	{ 0x59bc, 0x59bd, PDF_CMAP_RANGE, 11659 },
	{ 0x59be, 0x59be, PDF_CMAP_SINGLE, 6020 },
	{ 0x59bf, 0x59c5, PDF_CMAP_RANGE, 11661 },
	{ 0x59c6, 0x59c6, PDF_CMAP_SINGLE, 2842 },
	{ 0x59c7, 0x59c9, PDF_CMAP_RANGE, 11668 },
	{ 0x59ca, 0x59cb, PDF_CMAP_TABLE, 1904 },
	{ 0x59cc, 0x59cf, PDF_CMAP_RANGE, 11671 },
	{ 0x59d0, 0x59d4, PDF_CMAP_TABLE, 1906 },
	{ 0x59d5, 0x59d6, PDF_CMAP_RANGE, 11675 },
	{ 0x59d7, 0x59dd, PDF_CMAP_TABLE, 1911 },
	{ 0x59de, 0x59e2, PDF_CMAP_RANGE, 11679 },
	{ 0x59e3, 0x59e5, PDF_CMAP_TABLE, 1918 },
	{ 0x59e6, 0x59e7, PDF_CMAP_RANGE, 11685 },
	{ 0x59e8, 0x59e8, PDF_CMAP_SINGLE, 4179 },
	{ 0x59e9, 0x59eb, PDF_CMAP_RANGE, 11687 },
	{ 0x59ec, 0x59ec, PDF_CMAP_SINGLE, 2074 },
	{ 0x59ed, 0x59f8, PDF_CMAP_RANGE, 11690 },
	{ 0x59f9, 0x59fb, PDF_CMAP_TABLE, 1921 },
	{ 0x59fc, 0x59fe, PDF_CMAP_RANGE, 11703 },
	{ 0x59ff, 0x5a04, PDF_CMAP_TABLE, 1924 },
	{ 0x5a05, 0x5a06, PDF_CMAP_RANGE, 6021 },
	{ 0x5a07, 0x5a09, PDF_CMAP_TABLE, 1930 },
	{ 0x5a0a, 0x5a0b, PDF_CMAP_RANGE, 11708 },
	{ 0x5a0c, 0x5a0c, PDF_CMAP_SINGLE, 6028 },
	{ 0x5a0d, 0x5a10, PDF_CMAP_RANGE, 11710 },
	{ 0x5a11, 0x5a13, PDF_CMAP_TABLE, 1933 },
	{ 0x5a14, 0x5a17, PDF_CMAP_RANGE, 11715 },
	{ 0x5a18, 0x5a18, PDF_CMAP_SINGLE, 2898 },
	{ 0x5a19, 0x5a1b, PDF_CMAP_RANGE, 11719 },
	{ 0x5a1c, 0x5a1c, PDF_CMAP_SINGLE, 2859 },
	{ 0x5a1d, 0x5a1e, PDF_CMAP_RANGE, 11722 },
	{ 0x5a1f, 0x5a20, PDF_CMAP_TABLE, 1936 },
	{ 0x5a21, 0x5a22, PDF_CMAP_RANGE, 11724 },
	{ 0x5a23, 0x5a25, PDF_CMAP_TABLE, 1938 },
	{ 0x5a26, 0x5a28, PDF_CMAP_RANGE, 11727 },
	{ 0x5a29, 0x5a29, PDF_CMAP_SINGLE, 2793 },
	{ 0x5a2a, 0x5a30, PDF_CMAP_RANGE, 11730 },
	{ 0x5a31, 0x5a36, PDF_CMAP_TABLE, 1941 },
	{ 0x5a37, 0x5a3b, PDF_CMAP_RANGE, 11739 },
	{ 0x5a3c, 0x5a3c, PDF_CMAP_SINGLE, 6039 },
	{ 0x5a3d, 0x5a3f, PDF_CMAP_RANGE, 11744 },
	{ 0x5a40, 0x5a41, PDF_CMAP_TABLE, 1947 },
	{ 0x5a42, 0x5a45, PDF_CMAP_RANGE, 11747 },
	{ 0x5a46, 0x5a46, PDF_CMAP_SINGLE, 3044 },
	{ 0x5a47, 0x5a48, PDF_CMAP_RANGE, 11751 },
	{ 0x5a49, 0x5a4a, PDF_CMAP_TABLE, 1949 },
	{ 0x5a4b, 0x5a54, PDF_CMAP_RANGE, 11753 },
	{ 0x5a55, 0x5a55, PDF_CMAP_SINGLE, 6038 },
	{ 0x5a56, 0x5a59, PDF_CMAP_RANGE, 11763 },
	{ 0x5a5a, 0x5a5a, PDF_CMAP_SINGLE, 2046 },
	{ 0x5a5b, 0x5a61, PDF_CMAP_RANGE, 11767 },
	{ 0x5a62, 0x5a62, PDF_CMAP_SINGLE, 6040 },
	{ 0x5a63, 0x5a65, PDF_CMAP_RANGE, 11774 },
	{ 0x5a66, 0x5a67, PDF_CMAP_TABLE, 1951 },
	{ 0x5a68, 0x5a69, PDF_CMAP_RANGE, 11777 },
	{ 0x5a6a, 0x5a6a, PDF_CMAP_SINGLE, 2466 },
	{ 0x5a6b, 0x5a6c, PDF_CMAP_RANGE, 11779 },
	{ 0x5a6d, 0x5a6d, PDF_CMAP_SINGLE, 9209 },
	{ 0x5a6e, 0x5a73, PDF_CMAP_RANGE, 11781 },
	{ 0x5a74, 0x5a77, PDF_CMAP_TABLE, 1953 },
	{ 0x5a78, 0x5a79, PDF_CMAP_RANGE, 11787 },
	{ 0x5a7a, 0x5a7a, PDF_CMAP_SINGLE, 6046 },
	{ 0x5a7b, 0x5a7e, PDF_CMAP_RANGE, 11789 },
	{ 0x5a7f, 0x5a7f, PDF_CMAP_SINGLE, 4033 },
	{ 0x5a80, 0x5a91, PDF_CMAP_RANGE, 11793 },
	{ 0x5a92, 0x5a92, PDF_CMAP_SINGLE, 2754 },
	{ 0x5a93, 0x5a99, PDF_CMAP_RANGE, 11811 },
	{ 0x5a9a, 0x5a9b, PDF_CMAP_TABLE, 1957 },
	{ 0x5a9c, 0x5aa6, PDF_CMAP_RANGE, 11818 },
	{ 0x5aa7, 0x5aa7, PDF_CMAP_SINGLE, 9212 },
	{ 0x5aa8, 0x5aa9, PDF_CMAP_RANGE, 11829 },
	{ 0x5aaa, 0x5aaa, PDF_CMAP_SINGLE, 6043 },
	{ 0x5aab, 0x5ab1, PDF_CMAP_RANGE, 11831 },
	{ 0x5ab2, 0x5ab5, PDF_CMAP_TABLE, 1959 },
	{ 0x5ab6, 0x5ab7, PDF_CMAP_RANGE, 11839 },
	{ 0x5ab8, 0x5ab8, PDF_CMAP_SINGLE, 6052 },
	{ 0x5ab9, 0x5abc, PDF_CMAP_RANGE, 11841 },
	{ 0x5abd, 0x5abe, PDF_CMAP_TABLE, 1963 },
	{ 0x5abf, 0x5ac0, PDF_CMAP_RANGE, 11845 },
	{ 0x5ac1, 0x5ac2, PDF_CMAP_TABLE, 1965 },
	{ 0x5ac3, 0x5ac8, PDF_CMAP_RANGE, 11847 },
	{ 0x5ac9, 0x5ac9, PDF_CMAP_SINGLE, 2088 },
	{ 0x5aca, 0x5acb, PDF_CMAP_RANGE, 11853 },
	{ 0x5acc, 0x5acc, PDF_CMAP_SINGLE, 3904 },
	{ 0x5acd, 0x5ad1, PDF_CMAP_RANGE, 11855 },
	{ 0x5ad2, 0x5ad8, PDF_CMAP_TABLE, 1967 },
	{ 0x5ad9, 0x5adb, PDF_CMAP_RANGE, 11862 },
	{ 0x5adc, 0x5adc, PDF_CMAP_SINGLE, 6059 },
	{ 0x5add, 0x5adf, PDF_CMAP_RANGE, 11865 },
	{ 0x5ae0, 0x5ae3, PDF_CMAP_TABLE, 1974 },
	{ 0x5ae4, 0x5ae5, PDF_CMAP_RANGE, 11869 },
	{ 0x5ae6, 0x5ae6, PDF_CMAP_SINGLE, 6057 },
	{ 0x5ae7, 0x5ae8, PDF_CMAP_RANGE, 11871 },
	{ 0x5ae9, 0x5aeb, PDF_CMAP_TABLE, 1978 },
	{ 0x5aec, 0x5af0, PDF_CMAP_RANGE, 11874 },
	{ 0x5af1, 0x5af1, PDF_CMAP_SINGLE, 6055 },
	{ 0x5af2, 0x5af4, PDF_CMAP_RANGE, 11879 },
	{ 0x5af5, 0x5af5, PDF_CMAP_SINGLE, 9206 },
	{ 0x5af6, 0x5afa, PDF_CMAP_RANGE, 11882 },
	{ 0x5afb, 0x5afb, PDF_CMAP_SINGLE, 9213 },
	{ 0x5afc, 0x5aff, PDF_CMAP_RANGE, 11887 },
	{ 0x5b00, 0x5b00, PDF_CMAP_SINGLE, 9208 },
	{ 0x5b01, 0x5b07, PDF_CMAP_RANGE, 11891 },
	{ 0x5b08, 0x5b0c, PDF_CMAP_TABLE, 1981 },
	{ 0x5b0d, 0x5b15, PDF_CMAP_RANGE, 11899 },
	{ 0x5b16, 0x5b19, PDF_CMAP_TABLE, 1986 },
	{ 0x5b1a, 0x5b20, PDF_CMAP_RANGE, 11909 },
	{ 0x5b21, 0x5b21, PDF_CMAP_SINGLE, 9215 },
	{ 0x5b22, 0x5b29, PDF_CMAP_RANGE, 11916 },
	{ 0x5b2a, 0x5b2a, PDF_CMAP_SINGLE, 9216 },
	{ 0x5b2b, 0x5b2f, PDF_CMAP_RANGE, 11924 },
	{ 0x5b30, 0x5b34, PDF_CMAP_TABLE, 1990 },
	{ 0x5b35, 0x5b36, PDF_CMAP_RANGE, 11931 },
	{ 0x5b37, 0x5b38, PDF_CMAP_TABLE, 1995 },
	{ 0x5b39, 0x5b3f, PDF_CMAP_RANGE, 11933 },
	{ 0x5b40, 0x5b40, PDF_CMAP_SINGLE, 6065 },
	{ 0x5b41, 0x5b4b, PDF_CMAP_RANGE, 11940 },
	{ 0x5b4c, 0x5b4c, PDF_CMAP_SINGLE, 9211 },
	{ 0x5b4d, 0x5b4f, PDF_CMAP_RANGE, 11951 },
	{ 0x5b50, 0x5b5f, PDF_CMAP_TABLE, 1997 },
	{ 0x5b60, 0x5b61, PDF_CMAP_RANGE, 11957 },
	{ 0x5b62, 0x5b66, PDF_CMAP_TABLE, 2013 },
	{ 0x5b67, 0x5b68, PDF_CMAP_RANGE, 11959 },
	{ 0x5b69, 0x5b6c, PDF_CMAP_TABLE, 2018 },
	{ 0x5b6d, 0x5b6f, PDF_CMAP_RANGE, 11961 },
	{ 0x5b70, 0x5b75, PDF_CMAP_TABLE, 2022 },
	{ 0x5b76, 0x5b77, PDF_CMAP_RANGE, 11966 },
	{ 0x5b78, 0x5b7a, PDF_CMAP_TABLE, 2028 },
	{ 0x5b7b, 0x5b7c, PDF_CMAP_RANGE, 11969 },
	{ 0x5b7d, 0x5b8c, PDF_CMAP_TABLE, 2031 },
	{ 0x5b8d, 0x5b8e, PDF_CMAP_RANGE, 11975 },
	{ 0x5b8f, 0x5b8f, PDF_CMAP_SINGLE, 1953 },
	{ 0x5b90, 0x5b92, PDF_CMAP_RANGE, 11977 },
	{ 0x5b93, 0x5ba6, PDF_CMAP_TABLE, 2047 },
	{ 0x5ba7, 0x5ba9, PDF_CMAP_RANGE, 11983 },
	{ 0x5baa, 0x5bab, PDF_CMAP_TABLE, 2067 },
	{ 0x5bac, 0x5baf, PDF_CMAP_RANGE, 11986 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 4378 },
	{ 0x5bb1, 0x5bb2, PDF_CMAP_RANGE, 11990 },
	{ 0x5bb3, 0x5bb9, PDF_CMAP_TABLE, 2069 },
	{ 0x5bba, 0x5bbc, PDF_CMAP_RANGE, 11993 },
	{ 0x5bbd, 0x5bbf, PDF_CMAP_TABLE, 2076 },
	{ 0x5bc0, 0x5bc1, PDF_CMAP_RANGE, 11996 },
	{ 0x5bc2, 0x5bc7, PDF_CMAP_TABLE, 2079 },
	{ 0x5bc8, 0x5bcb, PDF_CMAP_RANGE, 11999 },
	{ 0x5bcc, 0x5bcc, PDF_CMAP_SINGLE, 1718 },
	{ 0x5bcd, 0x5bcf, PDF_CMAP_RANGE, 12003 },
	{ 0x5bd0, 0x5bd3, PDF_CMAP_TABLE, 2085 },
	{ 0x5bd4, 0x5bdc, PDF_CMAP_RANGE, 12007 },
	{ 0x5bdd, 0x5bf0, PDF_CMAP_TABLE, 2089 },
	{ 0x5bf1, 0x5bf4, PDF_CMAP_RANGE, 12021 },
	{ 0x5bf5, 0x5bfc, PDF_CMAP_TABLE, 2109 },
	{ 0x5bfd, 0x5bfe, PDF_CMAP_RANGE, 12026 },
	{ 0x5bff, 0x5c01, PDF_CMAP_TABLE, 2117 },
	{ 0x5c02, 0x5c03, PDF_CMAP_RANGE, 12029 },
	{ 0x5c04, 0x5c11, PDF_CMAP_TABLE, 2120 },
	{ 0x5c12, 0x5c13, PDF_CMAP_RANGE, 12034 },
	{ 0x5c14, 0x5c1d, PDF_CMAP_TABLE, 2134 },
	{ 0x5c1e, 0x5c21, PDF_CMAP_RANGE, 12039 },
	{ 0x5c22, 0x5c27, PDF_CMAP_TABLE, 2144 },
	{ 0x5c28, 0x5c2b, PDF_CMAP_RANGE, 12045 },
	{ 0x5c2c, 0x5c2c, PDF_CMAP_SINGLE, 5304 },
	{ 0x5c2d, 0x5c30, PDF_CMAP_RANGE, 12049 },
	{ 0x5c31, 0x5c31, PDF_CMAP_SINGLE, 2300 },
	{ 0x5c32, 0x5c33, PDF_CMAP_RANGE, 12053 },
	{ 0x5c34, 0x5c34, PDF_CMAP_SINGLE, 5305 },
	{ 0x5c35, 0x5c36, PDF_CMAP_RANGE, 12055 },
	{ 0x5c37, 0x5c42, PDF_CMAP_TABLE, 2150 },
	{ 0x5c43, 0x5c44, PDF_CMAP_RANGE, 12057 },
	{ 0x5c45, 0x5c45, PDF_CMAP_SINGLE, 2306 },
	{ 0x5c46, 0x5c47, PDF_CMAP_RANGE, 12059 },
	{ 0x5c48, 0x5c4b, PDF_CMAP_TABLE, 2162 },
	{ 0x5c4c, 0x5c4d, PDF_CMAP_RANGE, 12061 },
	{ 0x5c4e, 0x5c51, PDF_CMAP_TABLE, 2166 },
	{ 0x5c52, 0x5c54, PDF_CMAP_RANGE, 12063 },
	{ 0x5c55, 0x5c55, PDF_CMAP_SINGLE, 4442 },
	{ 0x5c56, 0x5c58, PDF_CMAP_RANGE, 12066 },
	{ 0x5c59, 0x5c59, PDF_CMAP_SINGLE, 5993 },
	{ 0x5c5a, 0x5c5d, PDF_CMAP_RANGE, 12069 },
	{ 0x5c5e, 0x5c68, PDF_CMAP_TABLE, 2170 },
	{ 0x5c69, 0x5c6b, PDF_CMAP_RANGE, 12075 },
	{ 0x5c6c, 0x5c71, PDF_CMAP_TABLE, 2181 },
	{ 0x5c72, 0x5c78, PDF_CMAP_RANGE, 12080 },
	{ 0x5c79, 0x5c7a, PDF_CMAP_TABLE, 2187 },
	{ 0x5c7b, 0x5c7e, PDF_CMAP_RANGE, 12087 },
	{ 0x5c7f, 0x5c82, PDF_CMAP_TABLE, 2189 },
	{ 0x5c83, 0x5c87, PDF_CMAP_RANGE, 12092 },
	{ 0x5c88, 0x5c88, PDF_CMAP_SINGLE, 5552 },
	{ 0x5c89, 0x5c8b, PDF_CMAP_RANGE, 12097 },
	{ 0x5c8c, 0x5c8d, PDF_CMAP_TABLE, 2193 },
	{ 0x5c8e, 0x5c8f, PDF_CMAP_RANGE, 12100 },
	{ 0x5c90, 0x5c91, PDF_CMAP_TABLE, 2195 },
	{ 0x5c92, 0x5c93, PDF_CMAP_RANGE, 12102 },
	{ 0x5c94, 0x5c97, PDF_CMAP_TABLE, 2197 },
	{ 0x5c98, 0x5c99, PDF_CMAP_RANGE, 5553 },
	{ 0x5c9a, 0x5c9c, PDF_CMAP_TABLE, 2201 },
	{ 0x5c9d, 0x5ca0, PDF_CMAP_RANGE, 12105 },
	{ 0x5ca1, 0x5ca3, PDF_CMAP_TABLE, 2204 },
	{ 0x5ca4, 0x5ca8, PDF_CMAP_RANGE, 12109 },
	{ 0x5ca9, 0x5cad, PDF_CMAP_TABLE, 2207 },
	{ 0x5cae, 0x5cb0, PDF_CMAP_RANGE, 12115 },
	{ 0x5cb1, 0x5cb8, PDF_CMAP_TABLE, 2212 },
	{ 0x5cb9, 0x5cbc, PDF_CMAP_RANGE, 12121 },
	{ 0x5cbd, 0x5cc1, PDF_CMAP_TABLE, 2220 },
	{ 0x5cc2, 0x5cc3, PDF_CMAP_RANGE, 12127 },
	{ 0x5cc4, 0x5cc4, PDF_CMAP_SINGLE, 5567 },
	{ 0x5cc5, 0x5cca, PDF_CMAP_RANGE, 12129 },
	{ 0x5ccb, 0x5ccb, PDF_CMAP_SINGLE, 5570 },
	{ 0x5ccc, 0x5cd1, PDF_CMAP_RANGE, 12135 },
	{ 0x5cd2, 0x5cd2, PDF_CMAP_SINGLE, 5568 },
	{ 0x5cd3, 0x5cd8, PDF_CMAP_RANGE, 12141 },
	{ 0x5cd9, 0x5cd9, PDF_CMAP_SINGLE, 4548 },
	{ 0x5cda, 0x5ce0, PDF_CMAP_RANGE, 12147 },
	{ 0x5ce1, 0x5ce1, PDF_CMAP_SINGLE, 3884 },
	{ 0x5ce2, 0x5ce3, PDF_CMAP_RANGE, 12154 },
	{ 0x5ce4, 0x5cea, PDF_CMAP_TABLE, 2225 },
	{ 0x5ceb, 0x5cec, PDF_CMAP_RANGE, 12158 },
	{ 0x5ced, 0x5ced, PDF_CMAP_SINGLE, 3146 },
	{ 0x5cee, 0x5cef, PDF_CMAP_RANGE, 12160 },
	{ 0x5cf0, 0x5cf0, PDF_CMAP_SINGLE, 1666 },
	{ 0x5cf1, 0x5cf3, PDF_CMAP_RANGE, 12162 },
	{ 0x5cf4, 0x5cf6, PDF_CMAP_TABLE, 2232 },
	{ 0x5cf7, 0x5cfa, PDF_CMAP_RANGE, 12166 },
	{ 0x5cfb, 0x5cfd, PDF_CMAP_TABLE, 2235 },
	{ 0x5cfe, 0x5d01, PDF_CMAP_RANGE, 12171 },
	{ 0x5d02, 0x5d03, PDF_CMAP_RANGE, 5572 },
	{ 0x5d04, 0x5d05, PDF_CMAP_RANGE, 12175 },
	{ 0x5d06, 0x5d07, PDF_CMAP_TABLE, 2238 },
	{ 0x5d08, 0x5d0c, PDF_CMAP_RANGE, 12177 },
	{ 0x5d0d, 0x5d0e, PDF_CMAP_TABLE, 2240 },
	{ 0x5d0f, 0x5d13, PDF_CMAP_RANGE, 12182 },
	{ 0x5d14, 0x5d17, PDF_CMAP_TABLE, 2242 },
	{ 0x5d18, 0x5d1a, PDF_CMAP_RANGE, 12188 },
	{ 0x5d1b, 0x5d1b, PDF_CMAP_SINGLE, 5580 },
	{ 0x5d1c, 0x5d1d, PDF_CMAP_RANGE, 12191 },
	{ 0x5d1e, 0x5d1e, PDF_CMAP_SINGLE, 5578 },
	{ 0x5d1f, 0x5d23, PDF_CMAP_RANGE, 12193 },
	{ 0x5d24, 0x5d29, PDF_CMAP_TABLE, 2246 },
	{ 0x5d2a, 0x5d2b, PDF_CMAP_RANGE, 12200 },
	{ 0x5d2c, 0x5d2e, PDF_CMAP_TABLE, 2252 },
	{ 0x5d2f, 0x5d33, PDF_CMAP_RANGE, 12202 },
	{ 0x5d34, 0x5d34, PDF_CMAP_SINGLE, 5583 },
	{ 0x5d35, 0x5d3c, PDF_CMAP_RANGE, 12207 },
	{ 0x5d3d, 0x5d3e, PDF_CMAP_TABLE, 2255 },
	{ 0x5d3f, 0x5d46, PDF_CMAP_RANGE, 12215 },
	{ 0x5d47, 0x5d47, PDF_CMAP_SINGLE, 6946 },
	{ 0x5d48, 0x5d49, PDF_CMAP_RANGE, 12223 },
	{ 0x5d4a, 0x5d4c, PDF_CMAP_TABLE, 2257 },
	{ 0x5d4d, 0x5d4f, PDF_CMAP_RANGE, 12225 },
	{ 0x5d50, 0x5d50, PDF_CMAP_SINGLE, 9098 },
	{ 0x5d51, 0x5d57, PDF_CMAP_RANGE, 12228 },
	{ 0x5d58, 0x5d58, PDF_CMAP_SINGLE, 5581 },
	{ 0x5d59, 0x5d5a, PDF_CMAP_RANGE, 12235 },
	{ 0x5d5b, 0x5d5d, PDF_CMAP_TABLE, 2260 },
	{ 0x5d5e, 0x5d68, PDF_CMAP_RANGE, 12238 },
	{ 0x5d69, 0x5d6c, PDF_CMAP_TABLE, 2263 },
	{ 0x5d6d, 0x5d6e, PDF_CMAP_RANGE, 12250 },
	{ 0x5d6f, 0x5d6f, PDF_CMAP_SINGLE, 5587 },
	{ 0x5d70, 0x5d73, PDF_CMAP_RANGE, 12252 },
	{ 0x5d74, 0x5d74, PDF_CMAP_SINGLE, 5593 },
	{ 0x5d75, 0x5d80, PDF_CMAP_RANGE, 12256 },
	{ 0x5d81, 0x5d84, PDF_CMAP_TABLE, 2267 },
	{ 0x5d85, 0x5d86, PDF_CMAP_RANGE, 12269 },
	{ 0x5d87, 0x5d87, PDF_CMAP_SINGLE, 9096 },
	{ 0x5d88, 0x5d96, PDF_CMAP_RANGE, 12271 },
	{ 0x5d97, 0x5d99, PDF_CMAP_TABLE, 2271 },
	{ 0x5d9a, 0x5d9c, PDF_CMAP_RANGE, 12287 },
	{ 0x5d9d, 0x5d9d, PDF_CMAP_SINGLE, 5596 },
	{ 0x5d9e, 0x5d9f, PDF_CMAP_RANGE, 12290 },
	{ 0x5da0, 0x5da0, PDF_CMAP_SINGLE, 9101 },
	{ 0x5da1, 0x5da6, PDF_CMAP_RANGE, 12292 },
	{ 0x5da7, 0x5da7, PDF_CMAP_SINGLE, 9100 },
	{ 0x5da8, 0x5db6, PDF_CMAP_RANGE, 12298 },
	{ 0x5db7, 0x5dbc, PDF_CMAP_TABLE, 2274 },
	{ 0x5dbd, 0x5dc4, PDF_CMAP_RANGE, 12315 },
	{ 0x5dc5, 0x5dc5, PDF_CMAP_SINGLE, 5599 },
	{ 0x5dc6, 0x5dca, PDF_CMAP_RANGE, 12323 },
	{ 0x5dcb, 0x5dcd, PDF_CMAP_TABLE, 2280 },
	{ 0x5dce, 0x5dd1, PDF_CMAP_RANGE, 12329 },
	{ 0x5dd2, 0x5dd4, PDF_CMAP_TABLE, 2283 },
	{ 0x5dd5, 0x5dda, PDF_CMAP_RANGE, 12334 },
	{ 0x5ddb, 0x5dde, PDF_CMAP_TABLE, 2286 },
	{ 0x5ddf, 0x5de0, PDF_CMAP_RANGE, 12341 },
	{ 0x5de1, 0x5de2, PDF_CMAP_TABLE, 2290 },
	{ 0x5de3, 0x5de4, PDF_CMAP_RANGE, 12343 },
	{ 0x5de5, 0x5deb, PDF_CMAP_TABLE, 2292 },
	{ 0x5dec, 0x5ded, PDF_CMAP_RANGE, 12346 },
	{ 0x5dee, 0x5df4, PDF_CMAP_TABLE, 2299 },
	{ 0x5df5, 0x5df6, PDF_CMAP_RANGE, 12348 },
	{ 0x5df7, 0x5df7, PDF_CMAP_SINGLE, 3932 },
	{ 0x5df8, 0x5dfc, PDF_CMAP_RANGE, 12350 },
	{ 0x5dfd, 0x5e08, PDF_CMAP_TABLE, 2306 },
	{ 0x5e09, 0x5e0b, PDF_CMAP_RANGE, 12359 },
	{ 0x5e0c, 0x5e0c, PDF_CMAP_SINGLE, 3856 },
	{ 0x5e0d, 0x5e0e, PDF_CMAP_RANGE, 12362 },
	{ 0x5e0f, 0x5e11, PDF_CMAP_TABLE, 2318 },
	{ 0x5e12, 0x5e13, PDF_CMAP_RANGE, 12364 },
	{ 0x5e14, 0x5e1d, PDF_CMAP_TABLE, 2321 },
	{ 0x5e1e, 0x5e24, PDF_CMAP_RANGE, 12367 },
	{ 0x5e25, 0x5e27, PDF_CMAP_TABLE, 2331 },
	{ 0x5e28, 0x5e2a, PDF_CMAP_RANGE, 12374 },
	{ 0x5e2b, 0x5e2e, PDF_CMAP_TABLE, 2334 },
	{ 0x5e2f, 0x5e30, PDF_CMAP_RANGE, 12378 },
	{ 0x5e31, 0x5e33, PDF_CMAP_TABLE, 2338 },
	{ 0x5e34, 0x5e35, PDF_CMAP_RANGE, 12381 },
	{ 0x5e36, 0x5e38, PDF_CMAP_TABLE, 2341 },
	{ 0x5e39, 0x5e3a, PDF_CMAP_RANGE, 12383 },
	{ 0x5e3b, 0x5e3c, PDF_CMAP_RANGE, 5539 },
	{ 0x5e3d, 0x5e3d, PDF_CMAP_SINGLE, 2742 },
	{ 0x5e3e, 0x5e3f, PDF_CMAP_RANGE, 12385 },
	{ 0x5e40, 0x5e45, PDF_CMAP_TABLE, 2344 },
	{ 0x5e46, 0x5e4b, PDF_CMAP_RANGE, 12388 },
	{ 0x5e4c, 0x5e4c, PDF_CMAP_SINGLE, 2020 },
	{ 0x5e4d, 0x5e53, PDF_CMAP_RANGE, 12394 },
	{ 0x5e54, 0x5e58, PDF_CMAP_TABLE, 2350 },
	{ 0x5e59, 0x5e5a, PDF_CMAP_RANGE, 12402 },
	{ 0x5e5b, 0x5e5b, PDF_CMAP_SINGLE, 5544 },
	{ 0x5e5c, 0x5e5d, PDF_CMAP_RANGE, 12404 },
	{ 0x5e5e, 0x5e63, PDF_CMAP_TABLE, 2355 },
	{ 0x5e64, 0x5e6a, PDF_CMAP_RANGE, 12407 },
	{ 0x5e6b, 0x5e6c, PDF_CMAP_TABLE, 2361 },
	{ 0x5e6d, 0x5e71, PDF_CMAP_RANGE, 12414 },
	{ 0x5e72, 0x5e80, PDF_CMAP_TABLE, 2363 },
	{ 0x5e81, 0x5e83, PDF_CMAP_RANGE, 12421 },
	{ 0x5e84, 0x5e87, PDF_CMAP_TABLE, 2378 },
	{ 0x5e88, 0x5e89, PDF_CMAP_RANGE, 12425 },
	{ 0x5e8a, 0x5e8b, PDF_CMAP_TABLE, 2382 },
	{ 0x5e8c, 0x5e8e, PDF_CMAP_RANGE, 12427 },
	{ 0x5e8f, 0x5ea0, PDF_CMAP_TABLE, 2384 },
	{ 0x5ea1, 0x5ea4, PDF_CMAP_RANGE, 12434 },
	{ 0x5ea5, 0x5ea7, PDF_CMAP_TABLE, 2402 },
	{ 0x5ea8, 0x5eaa, PDF_CMAP_RANGE, 12438 },
	{ 0x5eab, 0x5ead, PDF_CMAP_TABLE, 2405 },
	{ 0x5eae, 0x5eb2, PDF_CMAP_RANGE, 12442 },
	{ 0x5eb3, 0x5eb9, PDF_CMAP_TABLE, 2408 },
	{ 0x5eba, 0x5ebd, PDF_CMAP_RANGE, 12448 },
	{ 0x5ebe, 0x5ebe, PDF_CMAP_SINGLE, 5689 },
	{ 0x5ebf, 0x5ec8, PDF_CMAP_RANGE, 12452 },
	{ 0x5ec9, 0x5eca, PDF_CMAP_TABLE, 2415 },
	{ 0x5ecb, 0x5ed0, PDF_CMAP_RANGE, 12462 },
	{ 0x5ed1, 0x5ed3, PDF_CMAP_TABLE, 2417 },
	{ 0x5ed4, 0x5ed5, PDF_CMAP_RANGE, 12468 },
	{ 0x5ed6, 0x5ed6, PDF_CMAP_SINGLE, 2583 },
	{ 0x5ed7, 0x5eda, PDF_CMAP_RANGE, 12470 },
	{ 0x5edb, 0x5edb, PDF_CMAP_SINGLE, 5694 },
	{ 0x5edc, 0x5ede, PDF_CMAP_RANGE, 12474 },
	{ 0x5edf, 0x5ee3, PDF_CMAP_TABLE, 2420 },
	{ 0x5ee4, 0x5ee7, PDF_CMAP_RANGE, 12477 },
	{ 0x5ee8, 0x5eec, PDF_CMAP_TABLE, 2425 },
	{ 0x5eed, 0x5ef2, PDF_CMAP_RANGE, 12483 },
	{ 0x5ef3, 0x5ef7, PDF_CMAP_TABLE, 2430 },
	{ 0x5ef8, 0x5ef9, PDF_CMAP_RANGE, 12490 },
	{ 0x5efa, 0x5efa, PDF_CMAP_SINGLE, 2169 },
	{ 0x5efb, 0x5efd, PDF_CMAP_RANGE, 12492 },
	{ 0x5efe, 0x5f04, PDF_CMAP_TABLE, 2435 },
	{ 0x5f05, 0x5f07, PDF_CMAP_RANGE, 12495 },
	{ 0x5f08, 0x5f0b, PDF_CMAP_TABLE, 2442 },
	{ 0x5f0c, 0x5f0e, PDF_CMAP_RANGE, 12499 },
	{ 0x5f0f, 0x5f18, PDF_CMAP_TABLE, 2446 },
	{ 0x5f19, 0x5f1a, PDF_CMAP_RANGE, 12506 },
	{ 0x5f1b, 0x5f1b, PDF_CMAP_SINGLE, 1279 },
	{ 0x5f1c, 0x5f1e, PDF_CMAP_RANGE, 12508 },
	{ 0x5f1f, 0x5f20, PDF_CMAP_TABLE, 2456 },
	{ 0x5f21, 0x5f24, PDF_CMAP_RANGE, 12511 },
	{ 0x5f25, 0x5f2a, PDF_CMAP_TABLE, 2458 },
	{ 0x5f2b, 0x5f2c, PDF_CMAP_RANGE, 12516 },
	{ 0x5f2d, 0x5f35, PDF_CMAP_TABLE, 2464 },
	{ 0x5f36, 0x5f38, PDF_CMAP_RANGE, 12522 },
	{ 0x5f39, 0x5f3c, PDF_CMAP_TABLE, 2473 },
	{ 0x5f3d, 0x5f3f, PDF_CMAP_RANGE, 12526 },
	{ 0x5f40, 0x5f40, PDF_CMAP_SINGLE, 6590 },
	{ 0x5f41, 0x5f45, PDF_CMAP_RANGE, 12529 },
	{ 0x5f46, 0x5f48, PDF_CMAP_TABLE, 2477 },
	{ 0x5f49, 0x5f4b, PDF_CMAP_RANGE, 12535 },
	{ 0x5f4c, 0x5f59, PDF_CMAP_TABLE, 2480 },
	{ 0x5f5a, 0x5f5c, PDF_CMAP_RANGE, 12542 },
	{ 0x5f5d, 0x5f5d, PDF_CMAP_SINGLE, 4180 },
	{ 0x5f5e, 0x5f60, PDF_CMAP_RANGE, 12545 },
	{ 0x5f61, 0x5f66, PDF_CMAP_TABLE, 2494 },
	{ 0x5f67, 0x5f68, PDF_CMAP_RANGE, 12550 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 2500 },
	{ 0x5f6e, 0x5f6f, PDF_CMAP_RANGE, 12553 },
	{ 0x5f70, 0x5f73, PDF_CMAP_TABLE, 2505 },
	{ 0x5f74, 0x5f76, PDF_CMAP_RANGE, 12556 },
	{ 0x5f77, 0x5f7c, PDF_CMAP_TABLE, 2509 },
	{ 0x5f7d, 0x5f7f, PDF_CMAP_RANGE, 12561 },
	{ 0x5f80, 0x5f8c, PDF_CMAP_TABLE, 2515 },
	{ 0x5f8d, 0x5f8f, PDF_CMAP_RANGE, 12566 },
	{ 0x5f90, 0x5f92, PDF_CMAP_TABLE, 2528 },
	{ 0x5f93, 0x5f94, PDF_CMAP_RANGE, 12569 },
	{ 0x5f95, 0x5f99, PDF_CMAP_TABLE, 2531 },
	{ 0x5f9a, 0x5f9b, PDF_CMAP_RANGE, 12572 },
	{ 0x5f9c, 0x5fa1, PDF_CMAP_TABLE, 2536 },
	{ 0x5fa2, 0x5fa7, PDF_CMAP_RANGE, 12576 },
	{ 0x5fa8, 0x5faa, PDF_CMAP_TABLE, 2542 },
	{ 0x5fab, 0x5fac, PDF_CMAP_RANGE, 12582 },
	{ 0x5fad, 0x5fae, PDF_CMAP_TABLE, 2545 },
	{ 0x5faf, 0x5fb4, PDF_CMAP_RANGE, 12584 },
	{ 0x5fb5, 0x5fb9, PDF_CMAP_TABLE, 2547 },
	{ 0x5fba, 0x5fbb, PDF_CMAP_RANGE, 12592 },
	{ 0x5fbc, 0x5fbd, PDF_CMAP_TABLE, 2552 },
	{ 0x5fbe, 0x5fc2, PDF_CMAP_RANGE, 12594 },
	{ 0x5fc3, 0x5fc6, PDF_CMAP_TABLE, 2554 },
	{ 0x5fc7, 0x5fc8, PDF_CMAP_RANGE, 12599 },
	{ 0x5fc9, 0x5fc9, PDF_CMAP_SINGLE, 5699 },
	{ 0x5fca, 0x5fcb, PDF_CMAP_RANGE, 12601 },
	{ 0x5fcc, 0x5fd2, PDF_CMAP_TABLE, 2558 },
	{ 0x5fd3, 0x5fd5, PDF_CMAP_RANGE, 12604 },
	{ 0x5fd6, 0x5fd9, PDF_CMAP_TABLE, 2565 },
	{ 0x5fda, 0x5fdc, PDF_CMAP_RANGE, 12607 },
	{ 0x5fdd, 0x5fdd, PDF_CMAP_SINGLE, 5763 },
	{ 0x5fde, 0x5fdf, PDF_CMAP_RANGE, 12610 },
	{ 0x5fe0, 0x5fe1, PDF_CMAP_TABLE, 2569 },
	{ 0x5fe2, 0x5fe3, PDF_CMAP_RANGE, 12612 },
	{ 0x5fe4, 0x5fe4, PDF_CMAP_SINGLE, 5706 },
	{ 0x5fe5, 0x5fe6, PDF_CMAP_RANGE, 12614 },
	{ 0x5fe7, 0x5fe7, PDF_CMAP_SINGLE, 4268 },
	{ 0x5fe8, 0x5fe9, PDF_CMAP_RANGE, 12616 },
	{ 0x5fea, 0x5fee, PDF_CMAP_TABLE, 2571 },
	{ 0x5fef, 0x5ff0, PDF_CMAP_RANGE, 12619 },
	{ 0x5ff1, 0x5ff1, PDF_CMAP_SINGLE, 1253 },
	{ 0x5ff2, 0x5ff4, PDF_CMAP_RANGE, 12621 },
	{ 0x5ff5, 0x5ff5, PDF_CMAP_SINGLE, 2897 },
	{ 0x5ff6, 0x5ff7, PDF_CMAP_RANGE, 12624 },
	{ 0x5ff8, 0x5ff8, PDF_CMAP_SINGLE, 5712 },
	{ 0x5ff9, 0x5ffa, PDF_CMAP_RANGE, 12626 },
	{ 0x5ffb, 0x6004, PDF_CMAP_TABLE, 2576 },
	{ 0x6005, 0x6006, PDF_CMAP_RANGE, 5708 },
	{ 0x6007, 0x6009, PDF_CMAP_RANGE, 12629 },
	{ 0x600a, 0x600a, PDF_CMAP_SINGLE, 5721 },
	{ 0x600b, 0x600c, PDF_CMAP_RANGE, 12632 },
	{ 0x600d, 0x600f, PDF_CMAP_TABLE, 2586 },
	{ 0x6010, 0x6011, PDF_CMAP_RANGE, 12634 },
	{ 0x6012, 0x6016, PDF_CMAP_TABLE, 2589 },
	{ 0x6017, 0x6018, PDF_CMAP_RANGE, 12637 },
	{ 0x6019, 0x601d, PDF_CMAP_TABLE, 2594 },
	{ 0x601e, 0x601f, PDF_CMAP_RANGE, 12640 },
	{ 0x6020, 0x6021, PDF_CMAP_TABLE, 2599 },
	{ 0x6022, 0x6024, PDF_CMAP_RANGE, 12642 },
	{ 0x6025, 0x602b, PDF_CMAP_TABLE, 2601 },
	{ 0x602c, 0x602e, PDF_CMAP_RANGE, 12645 },
	{ 0x602f, 0x602f, PDF_CMAP_SINGLE, 3152 },
	{ 0x6030, 0x6034, PDF_CMAP_RANGE, 12648 },
	{ 0x6035, 0x6035, PDF_CMAP_SINGLE, 5714 },
	{ 0x6036, 0x603a, PDF_CMAP_RANGE, 12653 },
	{ 0x603b, 0x603c, PDF_CMAP_TABLE, 2608 },
	{ 0x603d, 0x603e, PDF_CMAP_RANGE, 12658 },
	{ 0x603f, 0x6043, PDF_CMAP_TABLE, 2610 },
	{ 0x6044, 0x604a, PDF_CMAP_RANGE, 12661 },
	{ 0x604b, 0x604d, PDF_CMAP_TABLE, 2615 },
	{ 0x604e, 0x604f, PDF_CMAP_RANGE, 12669 },
	{ 0x6050, 0x6052, PDF_CMAP_TABLE, 2618 },
	{ 0x6053, 0x6054, PDF_CMAP_RANGE, 12672 },
	{ 0x6055, 0x6055, PDF_CMAP_SINGLE, 3478 },
	{ 0x6056, 0x6058, PDF_CMAP_RANGE, 12674 },
	{ 0x6059, 0x605a, PDF_CMAP_TABLE, 2621 },
	{ 0x605b, 0x605c, PDF_CMAP_RANGE, 12677 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 6670 },
	{ 0x605e, 0x6061, PDF_CMAP_RANGE, 12679 },
	{ 0x6062, 0x6064, PDF_CMAP_TABLE, 2623 },
	{ 0x6065, 0x6066, PDF_CMAP_RANGE, 12683 },
	{ 0x6067, 0x6070, PDF_CMAP_TABLE, 2626 },
	{ 0x6071, 0x6072, PDF_CMAP_RANGE, 12686 },
	{ 0x6073, 0x6073, PDF_CMAP_SINGLE, 2399 },
	{ 0x6074, 0x6075, PDF_CMAP_RANGE, 12688 },
	{ 0x6076, 0x6077, PDF_CMAP_TABLE, 2636 },
	{ 0x6078, 0x6079, PDF_CMAP_RANGE, 5724 },
	{ 0x607a, 0x607f, PDF_CMAP_TABLE, 2638 },
	{ 0x6080, 0x6082, PDF_CMAP_RANGE, 12692 },
	{ 0x6083, 0x6084, PDF_CMAP_TABLE, 2644 },
	{ 0x6085, 0x6088, PDF_CMAP_RANGE, 12695 },
	{ 0x6089, 0x6089, PDF_CMAP_SINGLE, 3857 },
	{ 0x608a, 0x608b, PDF_CMAP_RANGE, 12699 },
	{ 0x608c, 0x608d, PDF_CMAP_TABLE, 2646 },
	{ 0x608e, 0x6091, PDF_CMAP_RANGE, 12701 },
	{ 0x6092, 0x6096, PDF_CMAP_TABLE, 2648 },
	{ 0x6097, 0x6099, PDF_CMAP_RANGE, 12707 },
	{ 0x609a, 0x60a0, PDF_CMAP_TABLE, 2653 },
	{ 0x60a1, 0x60a2, PDF_CMAP_RANGE, 12712 },
	{ 0x60a3, 0x60a3, PDF_CMAP_SINGLE, 2001 },
	{ 0x60a4, 0x60a5, PDF_CMAP_RANGE, 12714 },
	{ 0x60a6, 0x60a8, PDF_CMAP_TABLE, 2660 },
	{ 0x60a9, 0x60aa, PDF_CMAP_RANGE, 12717 },
	{ 0x60ab, 0x60b8, PDF_CMAP_TABLE, 2663 },
	{ 0x60b9, 0x60ba, PDF_CMAP_RANGE, 12723 },
	{ 0x60bb, 0x60bc, PDF_CMAP_TABLE, 2677 },
	{ 0x60bd, 0x60c4, PDF_CMAP_RANGE, 12725 },
	{ 0x60c5, 0x60c6, PDF_CMAP_TABLE, 2679 },
	{ 0x60c7, 0x60c9, PDF_CMAP_RANGE, 12733 },
	{ 0x60ca, 0x60cb, PDF_CMAP_TABLE, 2681 },
	{ 0x60cc, 0x60d0, PDF_CMAP_RANGE, 12736 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 2056 },
	{ 0x60d2, 0x60d4, PDF_CMAP_RANGE, 12741 },
	{ 0x60d5, 0x60d5, PDF_CMAP_SINGLE, 3643 },
	{ 0x60d6, 0x60d7, PDF_CMAP_RANGE, 12744 },
	{ 0x60d8, 0x60e1, PDF_CMAP_TABLE, 2683 },
	{ 0x60e2, 0x60e5, PDF_CMAP_RANGE, 12749 },
	{ 0x60e6, 0x60f6, PDF_CMAP_TABLE, 2693 },
	{ 0x60f7, 0x60f8, PDF_CMAP_RANGE, 12755 },
	{ 0x60f9, 0x60fb, PDF_CMAP_TABLE, 2710 },
	{ 0x60fc, 0x60ff, PDF_CMAP_RANGE, 12757 },
	{ 0x6100, 0x6101, PDF_CMAP_TABLE, 2713 },
	{ 0x6102, 0x6105, PDF_CMAP_RANGE, 12761 },
	{ 0x6106, 0x6109, PDF_CMAP_TABLE, 2715 },
	{ 0x610a, 0x610c, PDF_CMAP_RANGE, 12766 },
	{ 0x610d, 0x610f, PDF_CMAP_TABLE, 2719 },
	{ 0x6110, 0x6114, PDF_CMAP_RANGE, 12769 },
	{ 0x6115, 0x6115, PDF_CMAP_SINGLE, 5749 },
	{ 0x6116, 0x6119, PDF_CMAP_RANGE, 12774 },
	{ 0x611a, 0x611c, PDF_CMAP_TABLE, 2722 },
	{ 0x611d, 0x611e, PDF_CMAP_RANGE, 12778 },
	{ 0x611f, 0x6120, PDF_CMAP_TABLE, 2725 },
	{ 0x6121, 0x6122, PDF_CMAP_RANGE, 12780 },
	{ 0x6123, 0x6127, PDF_CMAP_TABLE, 2727 },
	{ 0x6128, 0x612a, PDF_CMAP_RANGE, 12783 },
	{ 0x612b, 0x612b, PDF_CMAP_SINGLE, 5754 },
	{ 0x612c, 0x6133, PDF_CMAP_RANGE, 12786 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 9141 },
	{ 0x6135, 0x6136, PDF_CMAP_RANGE, 12794 },
	{ 0x6137, 0x6137, PDF_CMAP_SINGLE, 9146 },
	{ 0x6138, 0x613d, PDF_CMAP_RANGE, 12796 },
	{ 0x613e, 0x613f, PDF_CMAP_TABLE, 2732 },
	{ 0x6140, 0x6147, PDF_CMAP_RANGE, 12802 },
	{ 0x6148, 0x614e, PDF_CMAP_TABLE, 2734 },
	{ 0x614f, 0x6150, PDF_CMAP_RANGE, 12812 },
	{ 0x6151, 0x6151, PDF_CMAP_SINGLE, 3358 },
	{ 0x6152, 0x6154, PDF_CMAP_RANGE, 12814 },
	{ 0x6155, 0x6155, PDF_CMAP_SINGLE, 2848 },
	{ 0x6156, 0x6157, PDF_CMAP_RANGE, 12817 },
	{ 0x6158, 0x615a, PDF_CMAP_TABLE, 2741 },
	{ 0x615b, 0x615c, PDF_CMAP_RANGE, 12820 },
	{ 0x615d, 0x615f, PDF_CMAP_TABLE, 2744 },
	{ 0x6160, 0x6161, PDF_CMAP_RANGE, 12823 },
	{ 0x6162, 0x6164, PDF_CMAP_TABLE, 2747 },
	{ 0x6165, 0x6166, PDF_CMAP_RANGE, 12825 },
	{ 0x6167, 0x616b, PDF_CMAP_TABLE, 2750 },
	{ 0x616c, 0x616d, PDF_CMAP_RANGE, 12828 },
	{ 0x616e, 0x6170, PDF_CMAP_TABLE, 2755 },
	{ 0x6171, 0x6172, PDF_CMAP_RANGE, 12831 },
	{ 0x6173, 0x6177, PDF_CMAP_TABLE, 2758 },
	{ 0x6178, 0x6181, PDF_CMAP_RANGE, 12834 },
	{ 0x6182, 0x6182, PDF_CMAP_SINGLE, 8754 },
	{ 0x6183, 0x6189, PDF_CMAP_RANGE, 12844 },
	{ 0x618a, 0x618b, PDF_CMAP_TABLE, 2763 },
	{ 0x618c, 0x618d, PDF_CMAP_RANGE, 12851 },
	{ 0x618e, 0x6194, PDF_CMAP_TABLE, 2765 },
	{ 0x6195, 0x6199, PDF_CMAP_RANGE, 12855 },
	{ 0x619a, 0x619a, PDF_CMAP_SINGLE, 7854 },
	{ 0x619b, 0x619c, PDF_CMAP_RANGE, 12860 },
	{ 0x619d, 0x619d, PDF_CMAP_SINGLE, 6681 },
	{ 0x619e, 0x61a3, PDF_CMAP_RANGE, 12862 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 7931 },
	{ 0x61a5, 0x61a6, PDF_CMAP_RANGE, 12868 },
	{ 0x61a7, 0x61ae, PDF_CMAP_TABLE, 2772 },
	{ 0x61af, 0x61b1, PDF_CMAP_RANGE, 12872 },
	{ 0x61b2, 0x61b2, PDF_CMAP_SINGLE, 8644 },
	{ 0x61b3, 0x61b5, PDF_CMAP_RANGE, 12875 },
	{ 0x61b6, 0x61b7, PDF_CMAP_TABLE, 2780 },
	{ 0x61b8, 0x61bd, PDF_CMAP_RANGE, 12878 },
	{ 0x61be, 0x61be, PDF_CMAP_SINGLE, 1901 },
	{ 0x61bf, 0x61c1, PDF_CMAP_RANGE, 12884 },
	{ 0x61c2, 0x61c2, PDF_CMAP_SINGLE, 1517 },
	{ 0x61c3, 0x61c6, PDF_CMAP_RANGE, 12887 },
	{ 0x61c7, 0x61cc, PDF_CMAP_TABLE, 2782 },
	{ 0x61cd, 0x61d0, PDF_CMAP_RANGE, 12891 },
	{ 0x61d1, 0x61d4, PDF_CMAP_TABLE, 2788 },
	{ 0x61d5, 0x61dd, PDF_CMAP_RANGE, 12896 },
	{ 0x61de, 0x61df, PDF_CMAP_TABLE, 2792 },
	{ 0x61e0, 0x61e2, PDF_CMAP_RANGE, 12905 },
	{ 0x61e3, 0x61e3, PDF_CMAP_SINGLE, 9440 },
	{ 0x61e4, 0x61e5, PDF_CMAP_RANGE, 12908 },
	{ 0x61e6, 0x61e8, PDF_CMAP_TABLE, 2794 },
	{ 0x61e9, 0x61f1, PDF_CMAP_RANGE, 12911 },
	{ 0x61f2, 0x61f2, PDF_CMAP_SINGLE, 7809 },
	{ 0x61f3, 0x61f4, PDF_CMAP_RANGE, 12920 },
	{ 0x61f5, 0x6200, PDF_CMAP_TABLE, 2797 },
	{ 0x6201, 0x6205, PDF_CMAP_RANGE, 12925 },
	{ 0x6206, 0x621b, PDF_CMAP_TABLE, 2809 },
	{ 0x621c, 0x621e, PDF_CMAP_RANGE, 12933 },
	{ 0x621f, 0x6227, PDF_CMAP_TABLE, 2831 },
	{ 0x6228, 0x6229, PDF_CMAP_RANGE, 12939 },
	{ 0x622a, 0x6234, PDF_CMAP_TABLE, 2840 },
	{ 0x6235, 0x6236, PDF_CMAP_RANGE, 12945 },
	{ 0x6237, 0x6237, PDF_CMAP_SINGLE, 1980 },
	{ 0x6238, 0x623c, PDF_CMAP_RANGE, 12947 },
	{ 0x623d, 0x6243, PDF_CMAP_TABLE, 2851 },
	{ 0x6244, 0x6246, PDF_CMAP_RANGE, 12953 },
	{ 0x6247, 0x6247, PDF_CMAP_SINGLE, 3329 },
	{ 0x6248, 0x6249, PDF_CMAP_RANGE, 6646 },
	{ 0x624a, 0x624e, PDF_CMAP_TABLE, 2858 },
	{ 0x624f, 0x6250, PDF_CMAP_RANGE, 12957 },
	{ 0x6251, 0x6254, PDF_CMAP_TABLE, 2863 },
	{ 0x6255, 0x6257, PDF_CMAP_RANGE, 12959 },
	{ 0x6258, 0x6258, PDF_CMAP_SINGLE, 3713 },
	{ 0x6259, 0x625a, PDF_CMAP_RANGE, 12962 },
	{ 0x625b, 0x625b, PDF_CMAP_SINGLE, 2373 },
	{ 0x625c, 0x6262, PDF_CMAP_RANGE, 12964 },
	{ 0x6263, 0x6263, PDF_CMAP_SINGLE, 2408 },
	{ 0x6264, 0x6265, PDF_CMAP_RANGE, 12971 },
	{ 0x6266, 0x6270, PDF_CMAP_TABLE, 2867 },
	{ 0x6271, 0x6272, PDF_CMAP_RANGE, 12974 },
	{ 0x6273, 0x6273, PDF_CMAP_SINGLE, 1005 },
	{ 0x6274, 0x6275, PDF_CMAP_RANGE, 12976 },
	{ 0x6276, 0x6276, PDF_CMAP_SINGLE, 1683 },
	{ 0x6277, 0x6278, PDF_CMAP_RANGE, 12978 },
	{ 0x6279, 0x6279, PDF_CMAP_SINGLE, 3003 },
	{ 0x627a, 0x627b, PDF_CMAP_RANGE, 12980 },
	{ 0x627c, 0x6280, PDF_CMAP_TABLE, 2878 },
	{ 0x6281, 0x6283, PDF_CMAP_RANGE, 12983 },
	{ 0x6284, 0x6284, PDF_CMAP_SINGLE, 1234 },
	{ 0x6285, 0x6288, PDF_CMAP_RANGE, 12986 },
	{ 0x6289, 0x628a, PDF_CMAP_TABLE, 2883 },
	{ 0x628b, 0x6290, PDF_CMAP_RANGE, 12990 },
	{ 0x6291, 0x629b, PDF_CMAP_TABLE, 2885 },
	{ 0x629c, 0x629e, PDF_CMAP_RANGE, 12998 },
	{ 0x629f, 0x62a5, PDF_CMAP_TABLE, 2896 },
	{ 0x62a6, 0x62a7, PDF_CMAP_RANGE, 13002 },
	{ 0x62a8, 0x62a8, PDF_CMAP_SINGLE, 2987 },
	{ 0x62a9, 0x62aa, PDF_CMAP_RANGE, 13004 },
	{ 0x62ab, 0x62ac, PDF_CMAP_TABLE, 2903 },
	{ 0x62ad, 0x62b0, PDF_CMAP_RANGE, 13006 },
	{ 0x62b1, 0x62b1, PDF_CMAP_SINGLE, 1040 },
	{ 0x62b2, 0x62b4, PDF_CMAP_RANGE, 13010 },
	{ 0x62b5, 0x62b5, PDF_CMAP_SINGLE, 1463 },
	{ 0x62b6, 0x62b8, PDF_CMAP_RANGE, 13013 },
	{ 0x62b9, 0x62bf, PDF_CMAP_TABLE, 2905 },
	{ 0x62c0, 0x62c1, PDF_CMAP_RANGE, 13018 },
	{ 0x62c2, 0x62dc, PDF_CMAP_TABLE, 2912 },
	{ 0x62dd, 0x62de, PDF_CMAP_RANGE, 13025 },
	{ 0x62df, 0x62df, PDF_CMAP_SINGLE, 2885 },
	{ 0x62e0, 0x62e1, PDF_CMAP_RANGE, 13027 },
	{ 0x62e2, 0x62e9, PDF_CMAP_TABLE, 2939 },
	{ 0x62ea, 0x62eb, PDF_CMAP_RANGE, 13030 },
	{ 0x62ec, 0x62f7, PDF_CMAP_TABLE, 2947 },
	{ 0x62f8, 0x62fb, PDF_CMAP_RANGE, 13035 },
	{ 0x62fc, 0x6302, PDF_CMAP_TABLE, 2959 },
	{ 0x6303, 0x6306, PDF_CMAP_RANGE, 13040 },
	{ 0x6307, 0x6309, PDF_CMAP_TABLE, 2966 },
	{ 0x630a, 0x630d, PDF_CMAP_RANGE, 13044 },
	{ 0x630e, 0x630e, PDF_CMAP_SINGLE, 2419 },
	{ 0x630f, 0x6310, PDF_CMAP_RANGE, 13048 },
	{ 0x6311, 0x6311, PDF_CMAP_SINGLE, 3655 },
	{ 0x6312, 0x6315, PDF_CMAP_RANGE, 13050 },
	{ 0x6316, 0x6316, PDF_CMAP_SINGLE, 3723 },
	{ 0x6317, 0x6319, PDF_CMAP_RANGE, 13054 },
	{ 0x631a, 0x6325, PDF_CMAP_TABLE, 2969 },
	{ 0x6326, 0x6327, PDF_CMAP_RANGE, 13058 },
	{ 0x6328, 0x632b, PDF_CMAP_TABLE, 2981 },
	{ 0x632c, 0x632e, PDF_CMAP_RANGE, 13061 },
	{ 0x632f, 0x632f, PDF_CMAP_SINGLE, 4498 },
	{ 0x6330, 0x6331, PDF_CMAP_RANGE, 13064 },
	{ 0x6332, 0x6332, PDF_CMAP_SINGLE, 6479 },
	{ 0x6333, 0x6338, PDF_CMAP_RANGE, 13066 },
	{ 0x6339, 0x633a, PDF_CMAP_TABLE, 2985 },
	{ 0x633b, 0x633c, PDF_CMAP_RANGE, 13072 },
	{ 0x633d, 0x633e, PDF_CMAP_TABLE, 2987 },
	{ 0x633f, 0x6341, PDF_CMAP_RANGE, 13074 },
	{ 0x6342, 0x6346, PDF_CMAP_TABLE, 2989 },
	{ 0x6347, 0x6348, PDF_CMAP_RANGE, 13078 },
	{ 0x6349, 0x6350, PDF_CMAP_TABLE, 2994 },
	{ 0x6351, 0x6354, PDF_CMAP_RANGE, 13081 },
	{ 0x6355, 0x6355, PDF_CMAP_SINGLE, 1149 },
	{ 0x6356, 0x635d, PDF_CMAP_RANGE, 13085 },
	{ 0x635e, 0x6363, PDF_CMAP_TABLE, 3002 },
	{ 0x6364, 0x6366, PDF_CMAP_RANGE, 13094 },
	{ 0x6367, 0x636e, PDF_CMAP_TABLE, 3008 },
	{ 0x636f, 0x6370, PDF_CMAP_RANGE, 13099 },
	{ 0x6371, 0x6372, PDF_CMAP_TABLE, 3016 },
	{ 0x6373, 0x6375, PDF_CMAP_RANGE, 13101 },
	{ 0x6376, 0x6377, PDF_CMAP_TABLE, 3018 },
	{ 0x6378, 0x6379, PDF_CMAP_RANGE, 13104 },
	{ 0x637a, 0x637b, PDF_CMAP_TABLE, 3020 },
	{ 0x637c, 0x637f, PDF_CMAP_RANGE, 13106 },
	{ 0x6380, 0x6384, PDF_CMAP_TABLE, 3022 },
	{ 0x6385, 0x6386, PDF_CMAP_RANGE, 13111 },
	{ 0x6387, 0x6392, PDF_CMAP_TABLE, 3027 },
	{ 0x6393, 0x6395, PDF_CMAP_RANGE, 13116 },
	{ 0x6396, 0x6398, PDF_CMAP_TABLE, 3039 },
	{ 0x6399, 0x639f, PDF_CMAP_RANGE, 13120 },
	{ 0x63a0, 0x63b0, PDF_CMAP_TABLE, 3042 },
	{ 0x63b1, 0x63b2, PDF_CMAP_RANGE, 13132 },
	{ 0x63b3, 0x63b4, PDF_CMAP_TABLE, 3059 },
	{ 0x63b5, 0x63b6, PDF_CMAP_RANGE, 13134 },
	{ 0x63b7, 0x63c0, PDF_CMAP_TABLE, 3061 },
	{ 0x63c1, 0x63c3, PDF_CMAP_RANGE, 13140 },
	{ 0x63c4, 0x63c6, PDF_CMAP_TABLE, 3071 },
	{ 0x63c7, 0x63c8, PDF_CMAP_RANGE, 13144 },
	{ 0x63c9, 0x63c9, PDF_CMAP_SINGLE, 3259 },
	{ 0x63ca, 0x63cc, PDF_CMAP_RANGE, 13146 },
	{ 0x63cd, 0x63d2, PDF_CMAP_TABLE, 3074 },
	{ 0x63d3, 0x63d5, PDF_CMAP_RANGE, 13150 },
	{ 0x63d6, 0x63d6, PDF_CMAP_SINGLE, 4165 },
	{ 0x63d7, 0x63d9, PDF_CMAP_RANGE, 13153 },
	{ 0x63da, 0x63da, PDF_CMAP_SINGLE, 8703 },
	{ 0x63db, 0x63dd, PDF_CMAP_RANGE, 13156 },
	{ 0x63de, 0x63e3, PDF_CMAP_TABLE, 3080 },
	{ 0x63e4, 0x63e8, PDF_CMAP_RANGE, 13161 },
	{ 0x63e9, 0x63ea, PDF_CMAP_TABLE, 3086 },
	{ 0x63eb, 0x63ec, PDF_CMAP_RANGE, 13166 },
	{ 0x63ed, 0x63ee, PDF_CMAP_TABLE, 3088 },
	{ 0x63ef, 0x63f1, PDF_CMAP_RANGE, 13168 },
	{ 0x63f2, 0x63f8, PDF_CMAP_TABLE, 3090 },
	{ 0x63f9, 0x63fc, PDF_CMAP_RANGE, 13174 },
	{ 0x63fd, 0x6402, PDF_CMAP_TABLE, 3097 },
	{ 0x6403, 0x6404, PDF_CMAP_RANGE, 13179 },
	{ 0x6405, 0x6405, PDF_CMAP_SINGLE, 2194 },
	{ 0x6406, 0x640a, PDF_CMAP_RANGE, 13181 },
	{ 0x640b, 0x6410, PDF_CMAP_TABLE, 3103 },
	{ 0x6411, 0x6412, PDF_CMAP_RANGE, 13187 },
	{ 0x6413, 0x6414, PDF_CMAP_TABLE, 3109 },
	{ 0x6415, 0x6416, PDF_CMAP_RANGE, 13189 },
	{ 0x6417, 0x6417, PDF_CMAP_SINGLE, 7862 },
	{ 0x6418, 0x641a, PDF_CMAP_RANGE, 13191 },
	{ 0x641b, 0x6421, PDF_CMAP_TABLE, 3111 },
	{ 0x6422, 0x6425, PDF_CMAP_RANGE, 13196 },
	{ 0x6426, 0x6426, PDF_CMAP_SINGLE, 5347 },
	{ 0x6427, 0x6429, PDF_CMAP_RANGE, 13200 },
	{ 0x642a, 0x642d, PDF_CMAP_TABLE, 3118 },
	{ 0x642e, 0x6433, PDF_CMAP_RANGE, 13204 },
	{ 0x6434, 0x6436, PDF_CMAP_TABLE, 3122 },
	{ 0x6437, 0x6439, PDF_CMAP_RANGE, 13211 },
	{ 0x643a, 0x643a, PDF_CMAP_SINGLE, 3962 },
	{ 0x643b, 0x643c, PDF_CMAP_RANGE, 13214 },
	{ 0x643d, 0x6441, PDF_CMAP_TABLE, 3125 },
	{ 0x6442, 0x6443, PDF_CMAP_RANGE, 13218 },
	{ 0x6444, 0x644a, PDF_CMAP_TABLE, 3130 },
	{ 0x644b, 0x6450, PDF_CMAP_RANGE, 13221 },
	{ 0x6451, 0x6454, PDF_CMAP_TABLE, 3137 },
	{ 0x6455, 0x6457, PDF_CMAP_RANGE, 13228 },
	{ 0x6458, 0x6458, PDF_CMAP_SINGLE, 4427 },
	{ 0x6459, 0x645b, PDF_CMAP_RANGE, 13231 },
	{ 0x645c, 0x645f, PDF_CMAP_TABLE, 3141 },
	{ 0x6460, 0x6466, PDF_CMAP_RANGE, 13235 },
	{ 0x6467, 0x6469, PDF_CMAP_TABLE, 3145 },
	{ 0x646a, 0x646c, PDF_CMAP_RANGE, 13243 },
	{ 0x646d, 0x646f, PDF_CMAP_TABLE, 3148 },
	{ 0x6470, 0x6472, PDF_CMAP_RANGE, 13247 },
	{ 0x6473, 0x6473, PDF_CMAP_SINGLE, 8159 },
	{ 0x6474, 0x6475, PDF_CMAP_RANGE, 13250 },
	{ 0x6476, 0x6477, PDF_CMAP_TABLE, 3151 },
	{ 0x6478, 0x6479, PDF_CMAP_RANGE, 2819 },
	{ 0x647a, 0x647b, PDF_CMAP_TABLE, 3153 },
	{ 0x647c, 0x6481, PDF_CMAP_RANGE, 13253 },
	{ 0x6482, 0x6488, PDF_CMAP_TABLE, 3155 },
	{ 0x6489, 0x6490, PDF_CMAP_RANGE, 13261 },
	{ 0x6491, 0x6496, PDF_CMAP_TABLE, 3162 },
	{ 0x6497, 0x6498, PDF_CMAP_RANGE, 13270 },
	{ 0x6499, 0x6499, PDF_CMAP_SINGLE, 5356 },
	{ 0x649a, 0x649d, PDF_CMAP_RANGE, 13272 },
	{ 0x649e, 0x649f, PDF_CMAP_TABLE, 3168 },
	{ 0x64a0, 0x64a2, PDF_CMAP_RANGE, 13276 },
	{ 0x64a3, 0x64a5, PDF_CMAP_TABLE, 3170 },
	{ 0x64a6, 0x64a8, PDF_CMAP_RANGE, 13279 },
	{ 0x64a9, 0x64b6, PDF_CMAP_TABLE, 3173 },
	{ 0x64b7, 0x64b8, PDF_CMAP_RANGE, 5354 },
	{ 0x64b9, 0x64c7, PDF_CMAP_TABLE, 3187 },
	{ 0x64c8, 0x64c9, PDF_CMAP_RANGE, 13291 },
	{ 0x64ca, 0x64d4, PDF_CMAP_TABLE, 3202 },
	{ 0x64d5, 0x64d6, PDF_CMAP_RANGE, 13297 },
	{ 0x64d7, 0x64da, PDF_CMAP_TABLE, 3213 },
	{ 0x64db, 0x64dd, PDF_CMAP_RANGE, 13300 },
	{ 0x64de, 0x64e6, PDF_CMAP_TABLE, 3217 },
	{ 0x64e7, 0x64eb, PDF_CMAP_RANGE, 13307 },
	{ 0x64ec, 0x64ec, PDF_CMAP_SINGLE, 8344 },
	{ 0x64ed, 0x64ee, PDF_CMAP_RANGE, 13312 },
	{ 0x64ef, 0x64f4, PDF_CMAP_TABLE, 3226 },
	{ 0x64f5, 0x64f6, PDF_CMAP_RANGE, 13315 },
	{ 0x64f7, 0x64f7, PDF_CMAP_SINGLE, 9061 },
	{ 0x64f8, 0x64f9, PDF_CMAP_RANGE, 13317 },
	{ 0x64fa, 0x6500, PDF_CMAP_TABLE, 3232 },
	{ 0x6501, 0x6503, PDF_CMAP_RANGE, 13321 },
	{ 0x6504, 0x6506, PDF_CMAP_TABLE, 3239 },
	{ 0x6507, 0x6508, PDF_CMAP_RANGE, 13325 },
	{ 0x6509, 0x6509, PDF_CMAP_SINGLE, 5363 },
	{ 0x650a, 0x650e, PDF_CMAP_RANGE, 13327 },
	{ 0x650f, 0x650f, PDF_CMAP_SINGLE, 8252 },
	{ 0x6510, 0x6511, PDF_CMAP_RANGE, 13332 },
	{ 0x6512, 0x651d, PDF_CMAP_TABLE, 3242 },
	{ 0x651e, 0x6521, PDF_CMAP_RANGE, 13339 },
	{ 0x6522, 0x6525, PDF_CMAP_TABLE, 3254 },
	{ 0x6526, 0x6529, PDF_CMAP_RANGE, 13343 },
	{ 0x652a, 0x652f, PDF_CMAP_TABLE, 3258 },
	{ 0x6530, 0x6533, PDF_CMAP_RANGE, 13348 },
	{ 0x6534, 0x653b, PDF_CMAP_TABLE, 3264 },
	{ 0x653c, 0x653d, PDF_CMAP_RANGE, 13354 },
	{ 0x653e, 0x653f, PDF_CMAP_TABLE, 3272 },
	{ 0x6540, 0x6544, PDF_CMAP_RANGE, 13356 },
	{ 0x6545, 0x6545, PDF_CMAP_SINGLE, 1827 },
	{ 0x6546, 0x6547, PDF_CMAP_RANGE, 13361 },
	{ 0x6548, 0x6549, PDF_CMAP_TABLE, 3274 },
	{ 0x654a, 0x654b, PDF_CMAP_RANGE, 13363 },
	{ 0x654c, 0x654c, PDF_CMAP_SINGLE, 1457 },
	{ 0x654d, 0x654e, PDF_CMAP_RANGE, 13365 },
	{ 0x654f, 0x6551, PDF_CMAP_TABLE, 3276 },
	{ 0x6552, 0x6554, PDF_CMAP_RANGE, 13368 },
	{ 0x6555, 0x655e, PDF_CMAP_TABLE, 3279 },
	{ 0x655f, 0x6561, PDF_CMAP_RANGE, 13374 },
	{ 0x6562, 0x6563, PDF_CMAP_TABLE, 3289 },
	{ 0x6564, 0x6565, PDF_CMAP_RANGE, 13377 },
	{ 0x6566, 0x6566, PDF_CMAP_SINGLE, 1559 },
	{ 0x6567, 0x656a, PDF_CMAP_RANGE, 13379 },
	{ 0x656b, 0x656c, PDF_CMAP_TABLE, 3291 },
	{ 0x656d, 0x656f, PDF_CMAP_RANGE, 13383 },
	{ 0x6570, 0x6578, PDF_CMAP_TABLE, 3293 },
	{ 0x6579, 0x6581, PDF_CMAP_RANGE, 13389 },
	{ 0x6582, 0x6583, PDF_CMAP_TABLE, 3302 },
	{ 0x6584, 0x6586, PDF_CMAP_RANGE, 13398 },
	{ 0x6587, 0x6587, PDF_CMAP_SINGLE, 3795 },
	{ 0x6588, 0x658a, PDF_CMAP_RANGE, 13401 },
	{ 0x658b, 0x658c, PDF_CMAP_TABLE, 3304 },
	{ 0x658d, 0x658f, PDF_CMAP_RANGE, 13404 },
	{ 0x6590, 0x659c, PDF_CMAP_TABLE, 3306 },
	{ 0x659d, 0x659e, PDF_CMAP_RANGE, 13412 },
	{ 0x659f, 0x65a1, PDF_CMAP_TABLE, 3319 },
	{ 0x65a2, 0x65a3, PDF_CMAP_RANGE, 13415 },
	{ 0x65a4, 0x65b0, PDF_CMAP_TABLE, 3322 },
	{ 0x65b1, 0x65b6, PDF_CMAP_RANGE, 13421 },
	{ 0x65b7, 0x65b9, PDF_CMAP_TABLE, 3335 },
	{ 0x65ba, 0x65bb, PDF_CMAP_RANGE, 13428 },
	{ 0x65bc, 0x65bd, PDF_CMAP_TABLE, 3338 },
	{ 0x65be, 0x65c0, PDF_CMAP_RANGE, 13430 },
	{ 0x65c1, 0x65c6, PDF_CMAP_TABLE, 3340 },
	{ 0x65c7, 0x65ca, PDF_CMAP_RANGE, 13434 },
	{ 0x65cb, 0x65cf, PDF_CMAP_TABLE, 3346 },
	{ 0x65d0, 0x65d1, PDF_CMAP_RANGE, 13439 },
	{ 0x65d2, 0x65d2, PDF_CMAP_SINGLE, 6602 },
	{ 0x65d3, 0x65d5, PDF_CMAP_RANGE, 13441 },
	{ 0x65d6, 0x65d7, PDF_CMAP_TABLE, 3351 },
	{ 0x65d8, 0x65df, PDF_CMAP_RANGE, 13444 },
	{ 0x65e0, 0x65e2, PDF_CMAP_TABLE, 3353 },
	{ 0x65e3, 0x65e4, PDF_CMAP_RANGE, 13453 },
	{ 0x65e5, 0x65e9, PDF_CMAP_TABLE, 3356 },
	{ 0x65ea, 0x65eb, PDF_CMAP_RANGE, 13455 },
	{ 0x65ec, 0x65ed, PDF_CMAP_TABLE, 3361 },
	{ 0x65ee, 0x65f0, PDF_CMAP_RANGE, 6410 },
	{ 0x65f1, 0x65f1, PDF_CMAP_SINGLE, 1900 },
	{ 0x65f2, 0x65f5, PDF_CMAP_RANGE, 13457 },
	{ 0x65f6, 0x65f7, PDF_CMAP_TABLE, 3363 },
	{ 0x65f8, 0x65f9, PDF_CMAP_RANGE, 13461 },
	{ 0x65fa, 0x65fa, PDF_CMAP_SINGLE, 3755 },
	{ 0x65fb, 0x65ff, PDF_CMAP_RANGE, 13463 },
	{ 0x6600, 0x6603, PDF_CMAP_TABLE, 3365 },
	{ 0x6604, 0x6605, PDF_CMAP_RANGE, 13469 },
	{ 0x6606, 0x6606, PDF_CMAP_SINGLE, 2448 },
	{ 0x6607, 0x6609, PDF_CMAP_RANGE, 13471 },
	{ 0x660a, 0x660f, PDF_CMAP_TABLE, 3369 },
	{ 0x6610, 0x6612, PDF_CMAP_RANGE, 13476 },
	{ 0x6613, 0x6615, PDF_CMAP_TABLE, 3375 },
	{ 0x6616, 0x6618, PDF_CMAP_RANGE, 13479 },
	{ 0x6619, 0x6619, PDF_CMAP_SINGLE, 6414 },
	{ 0x661a, 0x661c, PDF_CMAP_RANGE, 13482 },
	{ 0x661d, 0x6620, PDF_CMAP_TABLE, 3378 },
	{ 0x6621, 0x6624, PDF_CMAP_RANGE, 13486 },
	{ 0x6625, 0x6628, PDF_CMAP_TABLE, 3382 },
	{ 0x6629, 0x662c, PDF_CMAP_RANGE, 13491 },
	{ 0x662d, 0x6631, PDF_CMAP_TABLE, 3386 },
	{ 0x6632, 0x6633, PDF_CMAP_RANGE, 13497 },
	{ 0x6634, 0x6636, PDF_CMAP_TABLE, 3391 },
	{ 0x6637, 0x663b, PDF_CMAP_RANGE, 13499 },
	{ 0x663c, 0x663e, PDF_CMAP_TABLE, 3394 },
	{ 0x663f, 0x6640, PDF_CMAP_RANGE, 13505 },
	{ 0x6641, 0x6643, PDF_CMAP_TABLE, 3397 },
	{ 0x6644, 0x664a, PDF_CMAP_RANGE, 13507 },
	{ 0x664b, 0x664c, PDF_CMAP_TABLE, 3400 },
	{ 0x664d, 0x664e, PDF_CMAP_RANGE, 13514 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 6430 },
	{ 0x6650, 0x6651, PDF_CMAP_RANGE, 13516 },
	{ 0x6652, 0x6657, PDF_CMAP_TABLE, 3402 },
	{ 0x6658, 0x6659, PDF_CMAP_RANGE, 13518 },
	{ 0x665a, 0x665a, PDF_CMAP_SINGLE, 3742 },
	{ 0x665b, 0x665c, PDF_CMAP_RANGE, 13520 },
	{ 0x665d, 0x6661, PDF_CMAP_TABLE, 3408 },
	{ 0x6662, 0x6663, PDF_CMAP_RANGE, 13524 },
	{ 0x6664, 0x6668, PDF_CMAP_TABLE, 3413 },
	{ 0x6669, 0x666d, PDF_CMAP_RANGE, 13528 },
	{ 0x666e, 0x6670, PDF_CMAP_TABLE, 3418 },
	{ 0x6671, 0x6673, PDF_CMAP_RANGE, 13533 },
	{ 0x6674, 0x6677, PDF_CMAP_TABLE, 3421 },
	{ 0x6678, 0x6679, PDF_CMAP_RANGE, 13537 },
	{ 0x667a, 0x667a, PDF_CMAP_SINGLE, 4550 },
	{ 0x667b, 0x667d, PDF_CMAP_RANGE, 13539 },
	{ 0x667e, 0x667e, PDF_CMAP_SINGLE, 2569 },
	{ 0x667f, 0x6681, PDF_CMAP_RANGE, 13542 },
	{ 0x6682, 0x6684, PDF_CMAP_TABLE, 3425 },
	{ 0x6685, 0x6686, PDF_CMAP_RANGE, 13546 },
	{ 0x6687, 0x6689, PDF_CMAP_TABLE, 3428 },
	{ 0x668a, 0x668b, PDF_CMAP_RANGE, 13548 },
	{ 0x668c, 0x668c, PDF_CMAP_SINGLE, 6436 },
	{ 0x668d, 0x6690, PDF_CMAP_RANGE, 13550 },
	{ 0x6691, 0x6691, PDF_CMAP_SINGLE, 3461 },
	{ 0x6692, 0x6695, PDF_CMAP_RANGE, 13554 },
	{ 0x6696, 0x6697, PDF_CMAP_TABLE, 3431 },
	{ 0x6698, 0x669c, PDF_CMAP_RANGE, 13558 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 6438 },
	{ 0x669e, 0x66a1, PDF_CMAP_RANGE, 13563 },
	{ 0x66a2, 0x66a2, PDF_CMAP_SINGLE, 7801 },
	{ 0x66a3, 0x66a6, PDF_CMAP_RANGE, 13567 },
	{ 0x66a7, 0x66a8, PDF_CMAP_TABLE, 3433 },
	{ 0x66a9, 0x66aa, PDF_CMAP_RANGE, 13571 },
	{ 0x66ab, 0x66ab, PDF_CMAP_SINGLE, 8794 },
	{ 0x66ac, 0x66ad, PDF_CMAP_RANGE, 13573 },
	{ 0x66ae, 0x66ae, PDF_CMAP_SINGLE, 2845 },
	{ 0x66af, 0x66b3, PDF_CMAP_RANGE, 13575 },
	{ 0x66b4, 0x66b4, PDF_CMAP_SINGLE, 1042 },
	{ 0x66b5, 0x66b8, PDF_CMAP_RANGE, 13580 },
	{ 0x66b9, 0x66b9, PDF_CMAP_SINGLE, 5979 },
	{ 0x66ba, 0x66bd, PDF_CMAP_RANGE, 13584 },
	{ 0x66be, 0x66be, PDF_CMAP_SINGLE, 6439 },
	{ 0x66bf, 0x66c3, PDF_CMAP_RANGE, 13588 },
	{ 0x66c4, 0x66c9, PDF_CMAP_TABLE, 3435 },
	{ 0x66ca, 0x66d5, PDF_CMAP_RANGE, 13595 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 9385 },
	{ 0x66d7, 0x66d8, PDF_CMAP_RANGE, 13607 },
	{ 0x66d9, 0x66da, PDF_CMAP_TABLE, 3441 },
	{ 0x66db, 0x66dc, PDF_CMAP_RANGE, 6440 },
	{ 0x66dd, 0x66dd, PDF_CMAP_SINGLE, 3063 },
	{ 0x66de, 0x66df, PDF_CMAP_RANGE, 13610 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 8167 },
	{ 0x66e1, 0x66e5, PDF_CMAP_RANGE, 13612 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 6442 },
	{ 0x66e7, 0x66e8, PDF_CMAP_RANGE, 13617 },
	{ 0x66e9, 0x66e9, PDF_CMAP_SINGLE, 6443 },
	{ 0x66ea, 0x66eb, PDF_CMAP_RANGE, 13619 },
	{ 0x66ec, 0x66ec, PDF_CMAP_SINGLE, 8466 },
	{ 0x66ed, 0x66ef, PDF_CMAP_RANGE, 13621 },
	{ 0x66f0, 0x66f4, PDF_CMAP_TABLE, 3443 },
	{ 0x66f5, 0x66f6, PDF_CMAP_RANGE, 13625 },
	{ 0x66f7, 0x66f9, PDF_CMAP_TABLE, 3448 },
	{ 0x66fa, 0x66fb, PDF_CMAP_RANGE, 13627 },
	{ 0x66fc, 0x6700, PDF_CMAP_TABLE, 3451 },
	{ 0x6701, 0x6702, PDF_CMAP_RANGE, 13630 },
	{ 0x6703, 0x6703, PDF_CMAP_SINGLE, 8028 },
	{ 0x6704, 0x6707, PDF_CMAP_RANGE, 13632 },
	{ 0x6708, 0x670d, PDF_CMAP_TABLE, 3456 },
	{ 0x670e, 0x670f, PDF_CMAP_RANGE, 13637 },
	{ 0x6710, 0x6710, PDF_CMAP_SINGLE, 6532 },
	{ 0x6711, 0x6713, PDF_CMAP_RANGE, 13639 },
	{ 0x6714, 0x6717, PDF_CMAP_TABLE, 3462 },
	{ 0x6718, 0x671a, PDF_CMAP_RANGE, 13643 },
	{ 0x671b, 0x671f, PDF_CMAP_TABLE, 3466 },
	{ 0x6720, 0x6725, PDF_CMAP_RANGE, 13648 },
	{ 0x6726, 0x6731, PDF_CMAP_TABLE, 3471 },
	{ 0x6732, 0x6733, PDF_CMAP_RANGE, 13657 },
	{ 0x6734, 0x6735, PDF_CMAP_TABLE, 3483 },
	{ 0x6736, 0x6739, PDF_CMAP_RANGE, 13659 },
	{ 0x673a, 0x673a, PDF_CMAP_SINGLE, 2063 },
	{ 0x673b, 0x673c, PDF_CMAP_RANGE, 13663 },
	{ 0x673d, 0x673d, PDF_CMAP_SINGLE, 4011 },
	{ 0x673e, 0x673f, PDF_CMAP_RANGE, 13665 },
	{ 0x6740, 0x6743, PDF_CMAP_TABLE, 3485 },
	{ 0x6744, 0x6745, PDF_CMAP_RANGE, 13668 },
	{ 0x6746, 0x6749, PDF_CMAP_TABLE, 3489 },
	{ 0x674a, 0x674b, PDF_CMAP_RANGE, 13671 },
	{ 0x674c, 0x6753, PDF_CMAP_TABLE, 3493 },
	{ 0x6754, 0x6755, PDF_CMAP_RANGE, 13675 },
	{ 0x6756, 0x6756, PDF_CMAP_SINGLE, 4457 },
	{ 0x6757, 0x675b, PDF_CMAP_RANGE, 13677 },
	{ 0x675c, 0x6761, PDF_CMAP_TABLE, 3501 },
	{ 0x6762, 0x6764, PDF_CMAP_RANGE, 13683 },
	{ 0x6765, 0x6765, PDF_CMAP_SINGLE, 2463 },
	{ 0x6766, 0x6767, PDF_CMAP_RANGE, 13686 },
	{ 0x6768, 0x676a, PDF_CMAP_TABLE, 3507 },
	{ 0x676b, 0x676c, PDF_CMAP_RANGE, 13688 },
	{ 0x676d, 0x6777, PDF_CMAP_TABLE, 3510 },
	{ 0x6778, 0x677b, PDF_CMAP_RANGE, 13693 },
	{ 0x677c, 0x6781, PDF_CMAP_TABLE, 3521 },
	{ 0x6782, 0x6783, PDF_CMAP_RANGE, 13699 },
	{ 0x6784, 0x6784, PDF_CMAP_SINGLE, 1810 },
	{ 0x6785, 0x6786, PDF_CMAP_RANGE, 13701 },
	{ 0x6787, 0x678b, PDF_CMAP_TABLE, 3527 },
	{ 0x678c, 0x678f, PDF_CMAP_RANGE, 13705 },
	{ 0x6790, 0x6790, PDF_CMAP_SINGLE, 3845 },
	{ 0x6791, 0x6794, PDF_CMAP_RANGE, 13709 },
	{ 0x6795, 0x679e, PDF_CMAP_TABLE, 3532 },
	{ 0x679f, 0x67a1, PDF_CMAP_RANGE, 13716 },
	{ 0x67a2, 0x67b0, PDF_CMAP_TABLE, 3542 },
	{ 0x67b1, 0x67b2, PDF_CMAP_RANGE, 13724 },
	{ 0x67b3, 0x67b8, PDF_CMAP_TABLE, 3557 },
	{ 0x67b9, 0x67c0, PDF_CMAP_RANGE, 13727 },
	{ 0x67c1, 0x67c4, PDF_CMAP_TABLE, 3563 },
	{ 0x67c5, 0x67ce, PDF_CMAP_RANGE, 13736 },
	{ 0x67cf, 0x67d4, PDF_CMAP_TABLE, 3567 },
	{ 0x67d5, 0x67d7, PDF_CMAP_RANGE, 13746 },
	{ 0x67d8, 0x67e2, PDF_CMAP_TABLE, 3573 },
	{ 0x67e3, 0x67e4, PDF_CMAP_RANGE, 13752 },
	{ 0x67e5, 0x67e5, PDF_CMAP_SINGLE, 1200 },
	{ 0x67e6, 0x67e8, PDF_CMAP_RANGE, 13754 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 6247 },
	{ 0x67ea, 0x67eb, PDF_CMAP_RANGE, 13757 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 2144 },
	{ 0x67ed, 0x67ee, PDF_CMAP_RANGE, 13759 },
	{ 0x67ef, 0x67f4, PDF_CMAP_TABLE, 3584 },
	{ 0x67f5, 0x67fc, PDF_CMAP_RANGE, 13762 },
	{ 0x67fd, 0x6800, PDF_CMAP_TABLE, 3590 },
	{ 0x6801, 0x6804, PDF_CMAP_RANGE, 13771 },
	{ 0x6805, 0x6813, PDF_CMAP_TABLE, 3594 },
	{ 0x6814, 0x6815, PDF_CMAP_RANGE, 13779 },
	{ 0x6816, 0x6817, PDF_CMAP_TABLE, 3609 },
	{ 0x6818, 0x681c, PDF_CMAP_RANGE, 13781 },
	{ 0x681d, 0x681d, PDF_CMAP_SINGLE, 6271 },
	{ 0x681e, 0x6820, PDF_CMAP_RANGE, 13786 },
	{ 0x6821, 0x6821, PDF_CMAP_SINGLE, 3950 },
	{ 0x6822, 0x6828, PDF_CMAP_RANGE, 13789 },
	{ 0x6829, 0x682a, PDF_CMAP_TABLE, 3611 },
	{ 0x682b, 0x6831, PDF_CMAP_RANGE, 13796 },
	{ 0x6832, 0x6833, PDF_CMAP_RANGE, 6262 },
	{ 0x6834, 0x6836, PDF_CMAP_RANGE, 13803 },
	{ 0x6837, 0x6839, PDF_CMAP_TABLE, 3613 },
	{ 0x683a, 0x683b, PDF_CMAP_RANGE, 13806 },
	{ 0x683c, 0x6855, PDF_CMAP_TABLE, 3616 },
	{ 0x6856, 0x685f, PDF_CMAP_RANGE, 13814 },
	{ 0x6860, 0x6861, PDF_CMAP_RANGE, 6264 },
	{ 0x6862, 0x686b, PDF_CMAP_TABLE, 3642 },
	{ 0x686c, 0x6873, PDF_CMAP_RANGE, 13825 },
	{ 0x6874, 0x6877, PDF_CMAP_TABLE, 3652 },
	{ 0x6878, 0x6880, PDF_CMAP_RANGE, 13834 },
	{ 0x6881, 0x6886, PDF_CMAP_TABLE, 3656 },
	{ 0x6887, 0x688e, PDF_CMAP_RANGE, 13845 },
	{ 0x688f, 0x688f, PDF_CMAP_SINGLE, 6282 },
	{ 0x6890, 0x6892, PDF_CMAP_RANGE, 13853 },
	{ 0x6893, 0x6893, PDF_CMAP_SINGLE, 6285 },
	{ 0x6894, 0x6896, PDF_CMAP_RANGE, 13856 },
	{ 0x6897, 0x6898, PDF_CMAP_TABLE, 3662 },
	{ 0x6899, 0x689c, PDF_CMAP_RANGE, 13859 },
	{ 0x689d, 0x689f, PDF_CMAP_TABLE, 3664 },
	{ 0x68a0, 0x68a1, PDF_CMAP_RANGE, 13864 },
	{ 0x68a2, 0x68a2, PDF_CMAP_SINGLE, 3339 },
	{ 0x68a3, 0x68a5, PDF_CMAP_RANGE, 13866 },
	{ 0x68a6, 0x68a8, PDF_CMAP_TABLE, 3667 },
	{ 0x68a9, 0x68ac, PDF_CMAP_RANGE, 13869 },
	{ 0x68ad, 0x68b0, PDF_CMAP_TABLE, 3670 },
	{ 0x68b1, 0x68b2, PDF_CMAP_RANGE, 13874 },
	{ 0x68b3, 0x68b5, PDF_CMAP_TABLE, 3674 },
	{ 0x68b6, 0x68bf, PDF_CMAP_RANGE, 13877 },
	{ 0x68c0, 0x68c2, PDF_CMAP_TABLE, 3677 },
	{ 0x68c3, 0x68c8, PDF_CMAP_RANGE, 13888 },
	{ 0x68c9, 0x68cd, PDF_CMAP_TABLE, 3680 },
	{ 0x68ce, 0x68d1, PDF_CMAP_RANGE, 13896 },
	{ 0x68d2, 0x68d2, PDF_CMAP_SINGLE, 1023 },
	{ 0x68d3, 0x68d4, PDF_CMAP_RANGE, 13900 },
	{ 0x68d5, 0x68da, PDF_CMAP_TABLE, 3685 },
	{ 0x68db, 0x68de, PDF_CMAP_RANGE, 13903 },
	{ 0x68df, 0x68e0, PDF_CMAP_TABLE, 3691 },
	{ 0x68e1, 0x68e2, PDF_CMAP_RANGE, 13907 },
	{ 0x68e3, 0x68e3, PDF_CMAP_SINGLE, 6298 },
	{ 0x68e4, 0x68e6, PDF_CMAP_RANGE, 13909 },
	{ 0x68e7, 0x68e7, PDF_CMAP_SINGLE, 8818 },
	{ 0x68e8, 0x68ed, PDF_CMAP_RANGE, 13912 },
	{ 0x68ee, 0x68f1, PDF_CMAP_TABLE, 3693 },
	{ 0x68f2, 0x68f4, PDF_CMAP_RANGE, 13919 },
	{ 0x68f5, 0x68f5, PDF_CMAP_SINGLE, 2384 },
	{ 0x68f6, 0x68f8, PDF_CMAP_RANGE, 13922 },
	{ 0x68f9, 0x68fc, PDF_CMAP_TABLE, 3697 },
	{ 0x68fd, 0x6900, PDF_CMAP_RANGE, 13926 },
	{ 0x6901, 0x6901, PDF_CMAP_SINGLE, 6296 },
	{ 0x6902, 0x6904, PDF_CMAP_RANGE, 13930 },
	{ 0x6905, 0x6905, PDF_CMAP_SINGLE, 4181 },
	{ 0x6906, 0x690a, PDF_CMAP_RANGE, 13933 },
	{ 0x690b, 0x6912, PDF_CMAP_TABLE, 3701 },
	{ 0x6913, 0x691e, PDF_CMAP_RANGE, 13940 },
	{ 0x691f, 0x6920, PDF_CMAP_RANGE, 6290 },
	{ 0x6921, 0x6923, PDF_CMAP_RANGE, 13952 },
	{ 0x6924, 0x6924, PDF_CMAP_SINGLE, 6293 },
	{ 0x6925, 0x692c, PDF_CMAP_RANGE, 13955 },
	{ 0x692d, 0x692d, PDF_CMAP_SINGLE, 3719 },
	{ 0x692e, 0x692f, PDF_CMAP_RANGE, 13963 },
	{ 0x6930, 0x6930, PDF_CMAP_SINGLE, 4147 },
	{ 0x6931, 0x6933, PDF_CMAP_RANGE, 13965 },
	{ 0x6934, 0x6934, PDF_CMAP_SINGLE, 6310 },
	{ 0x6935, 0x6938, PDF_CMAP_RANGE, 13968 },
	{ 0x6939, 0x6939, PDF_CMAP_SINGLE, 6301 },
	{ 0x693a, 0x693c, PDF_CMAP_RANGE, 13972 },
	{ 0x693d, 0x693f, PDF_CMAP_TABLE, 3709 },
	{ 0x6940, 0x6941, PDF_CMAP_RANGE, 13976 },
	{ 0x6942, 0x6942, PDF_CMAP_SINGLE, 6303 },
	{ 0x6943, 0x6949, PDF_CMAP_RANGE, 13978 },
	{ 0x694a, 0x694a, PDF_CMAP_SINGLE, 8702 },
	{ 0x694b, 0x6952, PDF_CMAP_RANGE, 13985 },
	{ 0x6953, 0x6954, PDF_CMAP_TABLE, 3712 },
	{ 0x6955, 0x6956, PDF_CMAP_RANGE, 13993 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 6297 },
	{ 0x6958, 0x6959, PDF_CMAP_RANGE, 13995 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 1315 },
	{ 0x695b, 0x695c, PDF_CMAP_RANGE, 13997 },
	{ 0x695d, 0x6960, PDF_CMAP_TABLE, 3714 },
	{ 0x6961, 0x6962, PDF_CMAP_RANGE, 14000 },
	{ 0x6963, 0x6963, PDF_CMAP_SINGLE, 6317 },
	{ 0x6964, 0x6965, PDF_CMAP_RANGE, 14002 },
	{ 0x6966, 0x6968, PDF_CMAP_TABLE, 3718 },
	{ 0x6969, 0x696a, PDF_CMAP_RANGE, 14005 },
	{ 0x696b, 0x696e, PDF_CMAP_TABLE, 3721 },
	{ 0x696f, 0x6970, PDF_CMAP_RANGE, 14008 },
	{ 0x6971, 0x6971, PDF_CMAP_SINGLE, 6300 },
	{ 0x6972, 0x6974, PDF_CMAP_RANGE, 14010 },
	{ 0x6975, 0x6979, PDF_CMAP_TABLE, 3725 },
	{ 0x697a, 0x697b, PDF_CMAP_RANGE, 14014 },
	{ 0x697c, 0x697c, PDF_CMAP_SINGLE, 2636 },
	{ 0x697d, 0x697f, PDF_CMAP_RANGE, 14016 },
	{ 0x6980, 0x6986, PDF_CMAP_TABLE, 3730 },
	{ 0x6987, 0x6988, PDF_CMAP_RANGE, 6312 },
	{ 0x6989, 0x6989, PDF_CMAP_SINGLE, 6315 },
	{ 0x698a, 0x698c, PDF_CMAP_RANGE, 14022 },
	{ 0x698d, 0x698d, PDF_CMAP_SINGLE, 6331 },
	{ 0x698e, 0x6993, PDF_CMAP_RANGE, 14025 },
	{ 0x6994, 0x6995, PDF_CMAP_TABLE, 3737 },
	{ 0x6996, 0x6997, PDF_CMAP_RANGE, 14031 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 6308 },
	{ 0x6999, 0x699a, PDF_CMAP_RANGE, 14033 },
	{ 0x699b, 0x699c, PDF_CMAP_TABLE, 3739 },
	{ 0x699d, 0x69a6, PDF_CMAP_RANGE, 14035 },
	{ 0x69a7, 0x69ae, PDF_CMAP_TABLE, 3741 },
	{ 0x69af, 0x69b0, PDF_CMAP_RANGE, 14047 },
	{ 0x69b1, 0x69b1, PDF_CMAP_SINGLE, 6325 },
	{ 0x69b2, 0x69b3, PDF_CMAP_RANGE, 14049 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 2618 },
	{ 0x69b5, 0x69b6, PDF_CMAP_RANGE, 14051 },
	{ 0x69b7, 0x69b7, PDF_CMAP_SINGLE, 3217 },
	{ 0x69b8, 0x69ba, PDF_CMAP_RANGE, 14053 },
	{ 0x69bb, 0x69bb, PDF_CMAP_SINGLE, 6321 },
	{ 0x69bc, 0x69be, PDF_CMAP_RANGE, 14056 },
	{ 0x69bf, 0x69c1, PDF_CMAP_TABLE, 3749 },
	{ 0x69c2, 0x69c9, PDF_CMAP_RANGE, 14060 },
	{ 0x69ca, 0x69d0, PDF_CMAP_TABLE, 3752 },
	{ 0x69d1, 0x69d3, PDF_CMAP_RANGE, 14069 },
	{ 0x69d4, 0x69d4, PDF_CMAP_SINGLE, 6324 },
	{ 0x69d5, 0x69da, PDF_CMAP_RANGE, 14072 },
	{ 0x69db, 0x69db, PDF_CMAP_SINGLE, 2154 },
	{ 0x69dc, 0x69de, PDF_CMAP_RANGE, 14078 },
	{ 0x69df, 0x69e0, PDF_CMAP_TABLE, 3759 },
	{ 0x69e1, 0x69e6, PDF_CMAP_RANGE, 14081 },
	{ 0x69e7, 0x69e7, PDF_CMAP_SINGLE, 9340 },
	{ 0x69e8, 0x69ec, PDF_CMAP_RANGE, 14087 },
	{ 0x69ed, 0x69ed, PDF_CMAP_SINGLE, 6334 },
	{ 0x69ee, 0x69f1, PDF_CMAP_RANGE, 14092 },
	{ 0x69f2, 0x69f3, PDF_CMAP_TABLE, 3761 },
	{ 0x69f4, 0x69fc, PDF_CMAP_RANGE, 14096 },
	{ 0x69fd, 0x6a02, PDF_CMAP_TABLE, 3763 },
	{ 0x6a03, 0x6a04, PDF_CMAP_RANGE, 14107 },
	{ 0x6a05, 0x6a05, PDF_CMAP_SINGLE, 9324 },
	{ 0x6a06, 0x6a09, PDF_CMAP_RANGE, 14109 },
	{ 0x6a0a, 0x6a0a, PDF_CMAP_SINGLE, 1611 },
	{ 0x6a0b, 0x6a12, PDF_CMAP_RANGE, 14113 },
	{ 0x6a13, 0x6a13, PDF_CMAP_SINGLE, 8254 },
	{ 0x6a14, 0x6a16, PDF_CMAP_RANGE, 14121 },
	{ 0x6a17, 0x6a18, PDF_CMAP_RANGE, 6335 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 7756 },
	{ 0x6a1a, 0x6a1d, PDF_CMAP_RANGE, 14124 },
	{ 0x6a1e, 0x6a23, PDF_CMAP_TABLE, 3769 },
	{ 0x6a24, 0x6a27, PDF_CMAP_RANGE, 14130 },
	{ 0x6a28, 0x6a2a, PDF_CMAP_TABLE, 3775 },
	{ 0x6a2b, 0x6a2e, PDF_CMAP_RANGE, 14135 },
	{ 0x6a2f, 0x6a31, PDF_CMAP_TABLE, 3778 },
	{ 0x6a32, 0x6a34, PDF_CMAP_RANGE, 14140 },
	{ 0x6a35, 0x6a35, PDF_CMAP_SINGLE, 6344 },
	{ 0x6a36, 0x6a37, PDF_CMAP_RANGE, 14143 },
	{ 0x6a38, 0x6a3a, PDF_CMAP_TABLE, 3781 },
	{ 0x6a3b, 0x6a3c, PDF_CMAP_RANGE, 14145 },
	{ 0x6a3d, 0x6a3e, PDF_CMAP_TABLE, 3784 },
	{ 0x6a3f, 0x6a43, PDF_CMAP_RANGE, 14147 },
	{ 0x6a44, 0x6a44, PDF_CMAP_SINGLE, 6339 },
	{ 0x6a45, 0x6a46, PDF_CMAP_RANGE, 14152 },
	{ 0x6a47, 0x6a48, PDF_CMAP_TABLE, 3786 },
	{ 0x6a49, 0x6a4a, PDF_CMAP_RANGE, 14154 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 8413 },
	{ 0x6a4c, 0x6a4f, PDF_CMAP_RANGE, 14156 },
	{ 0x6a50, 0x6a50, PDF_CMAP_SINGLE, 6342 },
	{ 0x6a51, 0x6a57, PDF_CMAP_RANGE, 14160 },
	{ 0x6a58, 0x6a5b, PDF_CMAP_TABLE, 3788 },
	{ 0x6a5c, 0x6a5e, PDF_CMAP_RANGE, 14168 },
	{ 0x6a5f, 0x6a62, PDF_CMAP_TABLE, 3792 },
	{ 0x6a63, 0x6a64, PDF_CMAP_RANGE, 14172 },
	{ 0x6a65, 0x6a65, PDF_CMAP_SINGLE, 6337 },
	{ 0x6a66, 0x6a70, PDF_CMAP_RANGE, 14174 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 1308 },
	{ 0x6a72, 0x6a78, PDF_CMAP_RANGE, 14185 },
	{ 0x6a79, 0x6a79, PDF_CMAP_SINGLE, 6346 },
	{ 0x6a7a, 0x6a7b, PDF_CMAP_RANGE, 14192 },
	{ 0x6a7c, 0x6a7c, PDF_CMAP_SINGLE, 6350 },
	{ 0x6a7d, 0x6a7f, PDF_CMAP_RANGE, 14194 },
	{ 0x6a80, 0x6a80, PDF_CMAP_SINGLE, 3591 },
	{ 0x6a81, 0x6a83, PDF_CMAP_RANGE, 14197 },
	{ 0x6a84, 0x6a84, PDF_CMAP_SINGLE, 3866 },
	{ 0x6a85, 0x6a88, PDF_CMAP_RANGE, 14200 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 9330 },
	{ 0x6a8a, 0x6a8d, PDF_CMAP_RANGE, 14204 },
	{ 0x6a8e, 0x6a91, PDF_CMAP_TABLE, 3796 },
	{ 0x6a92, 0x6a93, PDF_CMAP_RANGE, 14209 },
	{ 0x6a94, 0x6a94, PDF_CMAP_SINGLE, 7861 },
	{ 0x6a95, 0x6a96, PDF_CMAP_RANGE, 14211 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 6354 },
	{ 0x6a98, 0x6a9b, PDF_CMAP_RANGE, 14213 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 9336 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 14217 },
	{ 0x6aa0, 0x6aa3, PDF_CMAP_TABLE, 3800 },
	{ 0x6aa4, 0x6aa8, PDF_CMAP_RANGE, 14221 },
	{ 0x6aa9, 0x6aac, PDF_CMAP_TABLE, 3804 },
	{ 0x6aad, 0x6aae, PDF_CMAP_RANGE, 14227 },
	{ 0x6aaf, 0x6aaf, PDF_CMAP_SINGLE, 9881 },
	{ 0x6ab0, 0x6ab2, PDF_CMAP_RANGE, 14229 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 9346 },
	{ 0x6ab4, 0x6ab7, PDF_CMAP_RANGE, 14232 },
	{ 0x6ab8, 0x6ab8, PDF_CMAP_SINGLE, 8353 },
	{ 0x6ab9, 0x6aba, PDF_CMAP_RANGE, 14236 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 8082 },
	{ 0x6abc, 0x6ac2, PDF_CMAP_RANGE, 14238 },
	{ 0x6ac3, 0x6ac3, PDF_CMAP_SINGLE, 7992 },
	{ 0x6ac4, 0x6ad2, PDF_CMAP_RANGE, 14245 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 9349 },
	{ 0x6ad4, 0x6ad9, PDF_CMAP_RANGE, 14260 },
	{ 0x6ada, 0x6adf, PDF_CMAP_TABLE, 3808 },
	{ 0x6ae0, 0x6ae6, PDF_CMAP_RANGE, 14267 },
	{ 0x6ae7, 0x6aec, PDF_CMAP_TABLE, 3814 },
	{ 0x6aed, 0x6af2, PDF_CMAP_RANGE, 14276 },
	{ 0x6af3, 0x6af3, PDF_CMAP_SINGLE, 9327 },
	{ 0x6af4, 0x6af7, PDF_CMAP_RANGE, 14282 },
	{ 0x6af8, 0x6af8, PDF_CMAP_SINGLE, 9345 },
	{ 0x6af9, 0x6afa, PDF_CMAP_RANGE, 14286 },
	{ 0x6afb, 0x6afb, PDF_CMAP_SINGLE, 8736 },
	{ 0x6afc, 0x6b03, PDF_CMAP_RANGE, 14288 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 8181 },
	{ 0x6b05, 0x6b09, PDF_CMAP_RANGE, 14296 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 8436 },
	{ 0x6b0b, 0x6b0e, PDF_CMAP_RANGE, 14301 },
	{ 0x6b0f, 0x6b0f, PDF_CMAP_SINGLE, 9341 },
	{ 0x6b10, 0x6b11, PDF_CMAP_RANGE, 14305 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 9337 },
	{ 0x6b13, 0x6b15, PDF_CMAP_RANGE, 14307 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 9342 },
	{ 0x6b17, 0x6b1d, PDF_CMAP_RANGE, 14310 },
	{ 0x6b1e, 0x6b24, PDF_CMAP_TABLE, 3820 },
	{ 0x6b25, 0x6b26, PDF_CMAP_RANGE, 14318 },
	{ 0x6b27, 0x6b27, PDF_CMAP_SINGLE, 2936 },
	{ 0x6b28, 0x6b31, PDF_CMAP_RANGE, 14320 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 4320 },
	{ 0x6b33, 0x6b36, PDF_CMAP_RANGE, 14330 },
	{ 0x6b37, 0x6b3a, PDF_CMAP_TABLE, 3827 },
	{ 0x6b3b, 0x6b3c, PDF_CMAP_RANGE, 14335 },
	{ 0x6b3d, 0x6b3e, PDF_CMAP_TABLE, 3831 },
	{ 0x6b3f, 0x6b42, PDF_CMAP_RANGE, 14337 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 6580 },
	{ 0x6b44, 0x6b45, PDF_CMAP_RANGE, 14341 },
	{ 0x6b46, 0x6b49, PDF_CMAP_TABLE, 3833 },
	{ 0x6b4a, 0x6b4b, PDF_CMAP_RANGE, 14344 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 1763 },
	{ 0x6b4d, 0x6b4f, PDF_CMAP_RANGE, 14346 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 8365 },
	{ 0x6b51, 0x6b58, PDF_CMAP_RANGE, 14349 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 6582 },
	{ 0x6b5a, 0x6b5e, PDF_CMAP_RANGE, 14357 },
	{ 0x6b5f, 0x6b67, PDF_CMAP_TABLE, 3837 },
	{ 0x6b68, 0x6b69, PDF_CMAP_RANGE, 14363 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 3730 },
	{ 0x6b6b, 0x6b71, PDF_CMAP_RANGE, 14365 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 8537 },
	{ 0x6b73, 0x6b76, PDF_CMAP_RANGE, 14372 },
	{ 0x6b77, 0x6b7c, PDF_CMAP_TABLE, 3846 },
	{ 0x6b7d, 0x6b80, PDF_CMAP_RANGE, 14377 },
	{ 0x6b81, 0x6b82, PDF_CMAP_RANGE, 6358 },
	{ 0x6b83, 0x6b8d, PDF_CMAP_TABLE, 3852 },
	{ 0x6b8e, 0x6b91, PDF_CMAP_RANGE, 14384 },
	{ 0x6b92, 0x6b93, PDF_CMAP_RANGE, 6362 },
	{ 0x6b94, 0x6b95, PDF_CMAP_RANGE, 14388 },
	{ 0x6b96, 0x6b99, PDF_CMAP_TABLE, 3863 },
	{ 0x6b9a, 0x6b9b, PDF_CMAP_RANGE, 6365 },
	{ 0x6b9c, 0x6b9d, PDF_CMAP_RANGE, 14392 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 9352 },
	{ 0x6b9f, 0x6ba0, PDF_CMAP_RANGE, 14394 },
	{ 0x6ba1, 0x6ba1, PDF_CMAP_SINGLE, 6367 },
	{ 0x6ba2, 0x6ba3, PDF_CMAP_RANGE, 14396 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 9351 },
	{ 0x6ba5, 0x6ba9, PDF_CMAP_RANGE, 14398 },
	{ 0x6baa, 0x6bab, PDF_CMAP_TABLE, 3867 },
	{ 0x6bac, 0x6bad, PDF_CMAP_RANGE, 14403 },
	{ 0x6bae, 0x6baf, PDF_CMAP_TABLE, 3869 },
	{ 0x6bb0, 0x6bb1, PDF_CMAP_RANGE, 14405 },
	{ 0x6bb2, 0x6bb7, PDF_CMAP_TABLE, 3871 },
	{ 0x6bb8, 0x6bb9, PDF_CMAP_RANGE, 14408 },
	{ 0x6bba, 0x6bbb, PDF_CMAP_TABLE, 3877 },
	{ 0x6bbc, 0x6bbe, PDF_CMAP_RANGE, 14410 },
	{ 0x6bbf, 0x6bc2, PDF_CMAP_TABLE, 3879 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 14414 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_TABLE, 3883 },
	{ 0x6bc7, 0x6bca, PDF_CMAP_RANGE, 14416 },
	{ 0x6bcb, 0x6bcf, PDF_CMAP_TABLE, 3885 },
	{ 0x6bd0, 0x6bd1, PDF_CMAP_RANGE, 14422 },
	{ 0x6bd2, 0x6bdb, PDF_CMAP_TABLE, 3890 },
	{ 0x6bdc, 0x6be0, PDF_CMAP_RANGE, 14426 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 4434 },
	{ 0x6be2, 0x6be9, PDF_CMAP_RANGE, 14431 },
	{ 0x6bea, 0x6beb, PDF_CMAP_TABLE, 3900 },
	{ 0x6bec, 0x6bee, PDF_CMAP_RANGE, 14439 },
	{ 0x6bef, 0x6bef, PDF_CMAP_SINGLE, 3597 },
	{ 0x6bf0, 0x6bf2, PDF_CMAP_RANGE, 14442 },
	{ 0x6bf3, 0x6bf5, PDF_CMAP_TABLE, 3902 },
	{ 0x6bf6, 0x6bf8, PDF_CMAP_RANGE, 14446 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 6488 },
	{ 0x6bfa, 0x6bfc, PDF_CMAP_RANGE, 14449 },
	{ 0x6bfd, 0x6bff, PDF_CMAP_TABLE, 3905 },
	{ 0x6c00, 0x6c04, PDF_CMAP_RANGE, 14453 },
	{ 0x6c05, 0x6c08, PDF_CMAP_TABLE, 3908 },
	{ 0x6c09, 0x6c0b, PDF_CMAP_RANGE, 14458 },
	{ 0x6c0c, 0x6c17, PDF_CMAP_TABLE, 3912 },
	{ 0x6c18, 0x6c1a, PDF_CMAP_RANGE, 6494 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 1650 },
	{ 0x6c1c, 0x6c1e, PDF_CMAP_RANGE, 14464 },
	{ 0x6c1f, 0x6c35, PDF_CMAP_TABLE, 3924 },
	{ 0x6c36, 0x6c37, PDF_CMAP_RANGE, 14472 },
	{ 0x6c38, 0x6c38, PDF_CMAP_SINGLE, 4261 },
	{ 0x6c39, 0x6c3c, PDF_CMAP_RANGE, 14474 },
	{ 0x6c3d, 0x6c3d, PDF_CMAP_SINGLE, 4847 },
	{ 0x6c3e, 0x6c3f, PDF_CMAP_RANGE, 14478 },
	{ 0x6c40, 0x6c42, PDF_CMAP_TABLE, 3947 },
	{ 0x6c43, 0x6c45, PDF_CMAP_RANGE, 14480 },
	{ 0x6c46, 0x6c4a, PDF_CMAP_TABLE, 3950 },
	{ 0x6c4b, 0x6c4f, PDF_CMAP_RANGE, 14484 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 3864 },
	{ 0x6c51, 0x6c53, PDF_CMAP_RANGE, 14489 },
	{ 0x6c54, 0x6c57, PDF_CMAP_TABLE, 3955 },
	{ 0x6c58, 0x6c5a, PDF_CMAP_RANGE, 14493 },
	{ 0x6c5b, 0x6c61, PDF_CMAP_TABLE, 3959 },
	{ 0x6c62, 0x6c63, PDF_CMAP_RANGE, 14496 },
	{ 0x6c64, 0x6c64, PDF_CMAP_SINGLE, 3603 },
	{ 0x6c65, 0x6c67, PDF_CMAP_RANGE, 14498 },
	{ 0x6c68, 0x6c69, PDF_CMAP_RANGE, 5800 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 3749 },
	{ 0x6c6b, 0x6c6f, PDF_CMAP_RANGE, 14501 },
	{ 0x6c70, 0x6c76, PDF_CMAP_TABLE, 3966 },
	{ 0x6c77, 0x6c78, PDF_CMAP_RANGE, 14509 },
	{ 0x6c79, 0x6c79, PDF_CMAP_SINGLE, 4005 },
	{ 0x6c7a, 0x6c7c, PDF_CMAP_RANGE, 14511 },
	{ 0x6c7d, 0x6c7e, PDF_CMAP_TABLE, 3973 },
	{ 0x6c7f, 0x6c80, PDF_CMAP_RANGE, 14514 },
	{ 0x6c81, 0x6c89, PDF_CMAP_TABLE, 3975 },
	{ 0x6c8a, 0x6c8b, PDF_CMAP_RANGE, 14518 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 5799 },
	{ 0x6c8d, 0x6c8e, PDF_CMAP_RANGE, 14520 },
	{ 0x6c8f, 0x6c90, PDF_CMAP_TABLE, 3984 },
	{ 0x6c91, 0x6c92, PDF_CMAP_RANGE, 14522 },
	{ 0x6c93, 0x6c94, PDF_CMAP_TABLE, 3986 },
	{ 0x6c95, 0x6c98, PDF_CMAP_RANGE, 14524 },
	{ 0x6c99, 0x6c9b, PDF_CMAP_TABLE, 3988 },
	{ 0x6c9c, 0x6c9e, PDF_CMAP_RANGE, 14529 },
	{ 0x6c9f, 0x6cae, PDF_CMAP_TABLE, 3991 },
	{ 0x6caf, 0x6cb0, PDF_CMAP_RANGE, 14536 },
	{ 0x6cb1, 0x6cb3, PDF_CMAP_TABLE, 4007 },
	{ 0x6cb4, 0x6cb7, PDF_CMAP_RANGE, 14538 },
	{ 0x6cb8, 0x6cbf, PDF_CMAP_TABLE, 4010 },
	{ 0x6cc0, 0x6cc3, PDF_CMAP_RANGE, 14543 },
	{ 0x6cc4, 0x6cc5, PDF_CMAP_TABLE, 4018 },
	{ 0x6cc6, 0x6cc8, PDF_CMAP_RANGE, 14547 },
	{ 0x6cc9, 0x6ccc, PDF_CMAP_TABLE, 4020 },
	{ 0x6ccd, 0x6ccf, PDF_CMAP_RANGE, 14551 },
	{ 0x6cd0, 0x6cd0, PDF_CMAP_SINGLE, 5806 },
	{ 0x6cd1, 0x6cd2, PDF_CMAP_RANGE, 14554 },
	{ 0x6cd3, 0x6cd7, PDF_CMAP_TABLE, 4024 },
	{ 0x6cd8, 0x6cda, PDF_CMAP_RANGE, 14556 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 1623 },
	{ 0x6cdc, 0x6cdd, PDF_CMAP_RANGE, 14559 },
	{ 0x6cde, 0x6ce5, PDF_CMAP_TABLE, 4029 },
	{ 0x6ce6, 0x6ce7, PDF_CMAP_RANGE, 14563 },
	{ 0x6ce8, 0x6ceb, PDF_CMAP_TABLE, 4037 },
	{ 0x6cec, 0x6ced, PDF_CMAP_RANGE, 14566 },
	{ 0x6cee, 0x6cf6, PDF_CMAP_TABLE, 4041 },
	{ 0x6cf7, 0x6cf8, PDF_CMAP_RANGE, 5809 },
	{ 0x6cf9, 0x6d01, PDF_CMAP_TABLE, 4050 },
	{ 0x6d02, 0x6d03, PDF_CMAP_RANGE, 14573 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 5829 },
	{ 0x6d05, 0x6d06, PDF_CMAP_RANGE, 14575 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 5828 },
	{ 0x6d08, 0x6d0a, PDF_CMAP_RANGE, 14577 },
	{ 0x6d0b, 0x6d0e, PDF_CMAP_TABLE, 4059 },
	{ 0x6d0f, 0x6d11, PDF_CMAP_RANGE, 14581 },
	{ 0x6d12, 0x6d12, PDF_CMAP_SINGLE, 3282 },
	{ 0x6d13, 0x6d16, PDF_CMAP_RANGE, 14584 },
	{ 0x6d17, 0x6d1b, PDF_CMAP_TABLE, 4063 },
	{ 0x6d1c, 0x6d1d, PDF_CMAP_RANGE, 14589 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 1523 },
	{ 0x6d1f, 0x6d24, PDF_CMAP_RANGE, 14591 },
	{ 0x6d25, 0x6d27, PDF_CMAP_TABLE, 4068 },
	{ 0x6d28, 0x6d29, PDF_CMAP_RANGE, 14598 },
	{ 0x6d2a, 0x6d2b, PDF_CMAP_TABLE, 4071 },
	{ 0x6d2c, 0x6d2d, PDF_CMAP_RANGE, 14600 },
	{ 0x6d2e, 0x6d2e, PDF_CMAP_SINGLE, 5834 },
	{ 0x6d2f, 0x6d30, PDF_CMAP_RANGE, 14602 },
	{ 0x6d31, 0x6d35, PDF_CMAP_TABLE, 4073 },
	{ 0x6d36, 0x6d38, PDF_CMAP_RANGE, 14605 },
	{ 0x6d39, 0x6d3e, PDF_CMAP_TABLE, 4078 },
	{ 0x6d3f, 0x6d40, PDF_CMAP_RANGE, 14609 },
	{ 0x6d41, 0x6d54, PDF_CMAP_TABLE, 4084 },
	{ 0x6d55, 0x6d58, PDF_CMAP_RANGE, 14616 },
	{ 0x6d59, 0x6d60, PDF_CMAP_TABLE, 4104 },
	{ 0x6d61, 0x6d62, PDF_CMAP_RANGE, 14623 },
	{ 0x6d63, 0x6d63, PDF_CMAP_SINGLE, 5851 },
	{ 0x6d64, 0x6d65, PDF_CMAP_RANGE, 14625 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 3061 },
	{ 0x6d67, 0x6d68, PDF_CMAP_RANGE, 14627 },
	{ 0x6d69, 0x6d6a, PDF_CMAP_TABLE, 4112 },
	{ 0x6d6b, 0x6d6d, PDF_CMAP_RANGE, 14629 },
	{ 0x6d6e, 0x6d6f, PDF_CMAP_TABLE, 4114 },
	{ 0x6d70, 0x6d73, PDF_CMAP_RANGE, 14632 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 4324 },
	{ 0x6d75, 0x6d76, PDF_CMAP_RANGE, 14636 },
	{ 0x6d77, 0x6d79, PDF_CMAP_TABLE, 4116 },
	{ 0x6d7a, 0x6d7b, PDF_CMAP_RANGE, 14638 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 5850 },
	{ 0x6d7d, 0x6d81, PDF_CMAP_RANGE, 14640 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 3696 },
	{ 0x6d83, 0x6d84, PDF_CMAP_RANGE, 14645 },
	{ 0x6d85, 0x6d89, PDF_CMAP_TABLE, 4119 },
	{ 0x6d8a, 0x6d8b, PDF_CMAP_RANGE, 14648 },
	{ 0x6d8c, 0x6d8e, PDF_CMAP_TABLE, 4124 },
	{ 0x6d8f, 0x6d90, PDF_CMAP_RANGE, 14651 },
	{ 0x6d91, 0x6d92, PDF_CMAP_TABLE, 4127 },
	{ 0x6d93, 0x6d94, PDF_CMAP_RANGE, 5846 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 3644 },
	{ 0x6d96, 0x6d9a, PDF_CMAP_RANGE, 14654 },
	{ 0x6d9b, 0x6dab, PDF_CMAP_TABLE, 4129 },
	{ 0x6dac, 0x6dad, PDF_CMAP_RANGE, 14662 },
	{ 0x6dae, 0x6daf, PDF_CMAP_TABLE, 4146 },
	{ 0x6db0, 0x6db1, PDF_CMAP_RANGE, 14664 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 4161 },
	{ 0x6db3, 0x6db4, PDF_CMAP_RANGE, 14666 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 1892 },
	{ 0x6db6, 0x6db7, PDF_CMAP_RANGE, 14668 },
	{ 0x6db8, 0x6db8, PDF_CMAP_SINGLE, 1931 },
	{ 0x6db9, 0x6dbe, PDF_CMAP_RANGE, 14670 },
	{ 0x6dbf, 0x6dc0, PDF_CMAP_TABLE, 4148 },
	{ 0x6dc1, 0x6dc3, PDF_CMAP_RANGE, 14676 },
	{ 0x6dc4, 0x6dc7, PDF_CMAP_TABLE, 4150 },
	{ 0x6dc8, 0x6dca, PDF_CMAP_RANGE, 14679 },
	{ 0x6dcb, 0x6dcc, PDF_CMAP_TABLE, 4154 },
	{ 0x6dcd, 0x6dd0, PDF_CMAP_RANGE, 14682 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 3454 },
	{ 0x6dd2, 0x6dd5, PDF_CMAP_RANGE, 14686 },
	{ 0x6dd6, 0x6dd9, PDF_CMAP_TABLE, 4156 },
	{ 0x6dda, 0x6ddc, PDF_CMAP_RANGE, 14691 },
	{ 0x6ddd, 0x6de1, PDF_CMAP_TABLE, 4160 },
	{ 0x6de2, 0x6de3, PDF_CMAP_RANGE, 14695 },
	{ 0x6de4, 0x6de6, PDF_CMAP_TABLE, 4165 },
	{ 0x6de7, 0x6de9, PDF_CMAP_RANGE, 14698 },
	{ 0x6dea, 0x6dee, PDF_CMAP_TABLE, 4168 },
	{ 0x6def, 0x6df0, PDF_CMAP_RANGE, 14702 },
	{ 0x6df1, 0x6dfc, PDF_CMAP_TABLE, 4173 },
	{ 0x6dfd, 0x6e04, PDF_CMAP_RANGE, 14707 },
	{ 0x6e05, 0x6e05, PDF_CMAP_SINGLE, 3170 },
	{ 0x6e06, 0x6e09, PDF_CMAP_RANGE, 14715 },
	{ 0x6e0a, 0x6e11, PDF_CMAP_TABLE, 4185 },
	{ 0x6e12, 0x6e13, PDF_CMAP_RANGE, 14721 },
	{ 0x6e14, 0x6e17, PDF_CMAP_TABLE, 4193 },
	{ 0x6e18, 0x6e19, PDF_CMAP_RANGE, 14724 },
	{ 0x6e1a, 0x6e1a, PDF_CMAP_SINGLE, 5852 },
	{ 0x6e1b, 0x6e1c, PDF_CMAP_RANGE, 14726 },
	{ 0x6e1d, 0x6e1d, PDF_CMAP_SINGLE, 4298 },
	{ 0x6e1e, 0x6e1f, PDF_CMAP_RANGE, 14728 },
	{ 0x6e20, 0x6e26, PDF_CMAP_TABLE, 4197 },
	{ 0x6e27, 0x6e28, PDF_CMAP_RANGE, 14731 },
	{ 0x6e29, 0x6e2f, PDF_CMAP_TABLE, 4204 },
	{ 0x6e30, 0x6e31, PDF_CMAP_RANGE, 14735 },
	{ 0x6e32, 0x6e34, PDF_CMAP_TABLE, 4211 },
	{ 0x6e35, 0x6e37, PDF_CMAP_RANGE, 14738 },
	{ 0x6e38, 0x6e3a, PDF_CMAP_TABLE, 4214 },
	{ 0x6e3b, 0x6e3d, PDF_CMAP_RANGE, 14742 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 8035 },
	{ 0x6e3f, 0x6e42, PDF_CMAP_RANGE, 14745 },
	{ 0x6e43, 0x6e44, PDF_CMAP_TABLE, 4217 },
	{ 0x6e45, 0x6e4c, PDF_CMAP_RANGE, 14749 },
	{ 0x6e4d, 0x6e4e, PDF_CMAP_TABLE, 4219 },
	{ 0x6e4f, 0x6e52, PDF_CMAP_RANGE, 14757 },
	{ 0x6e53, 0x6e54, PDF_CMAP_RANGE, 5874 },
	{ 0x6e55, 0x6e58, PDF_CMAP_TABLE, 4221 },
	{ 0x6e59, 0x6e5a, PDF_CMAP_RANGE, 14763 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 4448 },
	{ 0x6e5c, 0x6e5d, PDF_CMAP_RANGE, 14765 },
	{ 0x6e5e, 0x6e5f, PDF_CMAP_TABLE, 4225 },
	{ 0x6e60, 0x6e6a, PDF_CMAP_RANGE, 14767 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 5870 },
	{ 0x6e6c, 0x6e6d, PDF_CMAP_RANGE, 14778 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 4227 },
	{ 0x6e70, 0x6e7d, PDF_CMAP_RANGE, 14780 },
	{ 0x6e7e, 0x6e7f, PDF_CMAP_TABLE, 4229 },
	{ 0x6e80, 0x6e82, PDF_CMAP_RANGE, 14794 },
	{ 0x6e83, 0x6e86, PDF_CMAP_TABLE, 4231 },
	{ 0x6e87, 0x6e88, PDF_CMAP_RANGE, 14798 },
	{ 0x6e89, 0x6e89, PDF_CMAP_SINGLE, 1731 },
	{ 0x6e8a, 0x6e8e, PDF_CMAP_RANGE, 14800 },
	{ 0x6e8f, 0x6e90, PDF_CMAP_TABLE, 4235 },
	{ 0x6e91, 0x6e95, PDF_CMAP_RANGE, 14805 },
	{ 0x6e96, 0x6e98, PDF_CMAP_TABLE, 4237 },
	{ 0x6e99, 0x6e9b, PDF_CMAP_RANGE, 14811 },
	{ 0x6e9c, 0x6e9f, PDF_CMAP_TABLE, 4240 },
	{ 0x6ea0, 0x6ea1, PDF_CMAP_RANGE, 14815 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 4206 },
	{ 0x6ea3, 0x6ea4, PDF_CMAP_RANGE, 14817 },
	{ 0x6ea5, 0x6ea7, PDF_CMAP_TABLE, 4244 },
	{ 0x6ea8, 0x6ea9, PDF_CMAP_RANGE, 14820 },
	{ 0x6eaa, 0x6eaa, PDF_CMAP_SINGLE, 3863 },
	{ 0x6eab, 0x6eae, PDF_CMAP_RANGE, 14822 },
	{ 0x6eaf, 0x6eb7, PDF_CMAP_TABLE, 4247 },
	{ 0x6eb8, 0x6eb9, PDF_CMAP_RANGE, 14829 },
	{ 0x6eba, 0x6ebd, PDF_CMAP_TABLE, 4256 },
	{ 0x6ebe, 0x6ec0, PDF_CMAP_RANGE, 14832 },
	{ 0x6ec1, 0x6ec7, PDF_CMAP_TABLE, 4260 },
	{ 0x6ec8, 0x6eca, PDF_CMAP_RANGE, 14837 },
	{ 0x6ecb, 0x6ed7, PDF_CMAP_TABLE, 4267 },
	{ 0x6ed8, 0x6ed9, PDF_CMAP_RANGE, 14844 },
	{ 0x6eda, 0x6eda, PDF_CMAP_SINGLE, 1871 },
	{ 0x6edb, 0x6edd, PDF_CMAP_RANGE, 14846 },
	{ 0x6ede, 0x6ee9, PDF_CMAP_TABLE, 4280 },
	{ 0x6eea, 0x6eeb, PDF_CMAP_RANGE, 14851 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 8011 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 14853 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 8850 },
	{ 0x6ef0, 0x6ef1, PDF_CMAP_RANGE, 14855 },
	{ 0x6ef2, 0x6ef4, PDF_CMAP_TABLE, 4292 },
	{ 0x6ef5, 0x6ef6, PDF_CMAP_RANGE, 14858 },
	{ 0x6ef7, 0x6ef9, PDF_CMAP_TABLE, 4295 },
	{ 0x6efa, 0x6efe, PDF_CMAP_RANGE, 14860 },
	{ 0x6eff, 0x6f02, PDF_CMAP_TABLE, 4298 },
	{ 0x6f03, 0x6f05, PDF_CMAP_RANGE, 14866 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 3072 },
	{ 0x6f07, 0x6f08, PDF_CMAP_RANGE, 14869 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 5907 },
	{ 0x6f0a, 0x6f0e, PDF_CMAP_RANGE, 14871 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 2640 },
	{ 0x6f10, 0x6f12, PDF_CMAP_RANGE, 14876 },
	{ 0x6f13, 0x6f15, PDF_CMAP_TABLE, 4302 },
	{ 0x6f16, 0x6f19, PDF_CMAP_RANGE, 14879 },
	{ 0x6f1a, 0x6f1a, PDF_CMAP_SINGLE, 8369 },
	{ 0x6f1b, 0x6f1f, PDF_CMAP_RANGE, 14883 },
	{ 0x6f20, 0x6f24, PDF_CMAP_TABLE, 4305 },
	{ 0x6f25, 0x6f28, PDF_CMAP_RANGE, 14889 },
	{ 0x6f29, 0x6f33, PDF_CMAP_TABLE, 4310 },
	{ 0x6f34, 0x6f35, PDF_CMAP_RANGE, 14895 },
	{ 0x6f36, 0x6f38, PDF_CMAP_TABLE, 4321 },
	{ 0x6f39, 0x6f3d, PDF_CMAP_RANGE, 14898 },
	{ 0x6f3e, 0x6f41, PDF_CMAP_TABLE, 4324 },
	{ 0x6f42, 0x6f45, PDF_CMAP_RANGE, 14904 },
	{ 0x6f46, 0x6f47, PDF_CMAP_RANGE, 5897 },
	{ 0x6f48, 0x6f4a, PDF_CMAP_RANGE, 14908 },
	{ 0x6f4b, 0x6f4d, PDF_CMAP_TABLE, 4328 },
	{ 0x6f4e, 0x6f50, PDF_CMAP_RANGE, 14912 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 8382 },
	{ 0x6f52, 0x6f53, PDF_CMAP_RANGE, 14915 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 8116 },
	{ 0x6f55, 0x6f57, PDF_CMAP_RANGE, 14917 },
	{ 0x6f58, 0x6f59, PDF_CMAP_TABLE, 4331 },
	{ 0x6f5a, 0x6f5b, PDF_CMAP_RANGE, 14920 },
	{ 0x6f5c, 0x6f5e, PDF_CMAP_TABLE, 4333 },
	{ 0x6f5f, 0x6f61, PDF_CMAP_RANGE, 14923 },
	{ 0x6f62, 0x6f66, PDF_CMAP_TABLE, 4336 },
	{ 0x6f67, 0x6f6c, PDF_CMAP_RANGE, 14928 },
	{ 0x6f6d, 0x6f74, PDF_CMAP_TABLE, 4341 },
	{ 0x6f75, 0x6f76, PDF_CMAP_RANGE, 14936 },
	{ 0x6f77, 0x6f7c, PDF_CMAP_TABLE, 4349 },
	{ 0x6f7d, 0x6f7e, PDF_CMAP_RANGE, 14940 },
	{ 0x6f7f, 0x6f80, PDF_CMAP_TABLE, 4355 },
	{ 0x6f81, 0x6f83, PDF_CMAP_RANGE, 14942 },
	{ 0x6f84, 0x6f89, PDF_CMAP_TABLE, 4357 },
	{ 0x6f8a, 0x6f8b, PDF_CMAP_RANGE, 14946 },
	{ 0x6f8c, 0x6f8e, PDF_CMAP_TABLE, 4363 },
	{ 0x6f8f, 0x6f96, PDF_CMAP_RANGE, 14948 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 8093 },
	{ 0x6f98, 0x6f9b, PDF_CMAP_RANGE, 14956 },
	{ 0x6f9c, 0x6f9c, PDF_CMAP_SINGLE, 2472 },
	{ 0x6f9d, 0x6f9f, PDF_CMAP_RANGE, 14960 },
	{ 0x6fa0, 0x6fa1, PDF_CMAP_TABLE, 4366 },
	{ 0x6fa2, 0x6fa3, PDF_CMAP_RANGE, 14963 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 8804 },
	{ 0x6fa5, 0x6fa6, PDF_CMAP_RANGE, 14965 },
	{ 0x6fa7, 0x6fa9, PDF_CMAP_TABLE, 4368 },
	{ 0x6faa, 0x6fad, PDF_CMAP_RANGE, 14968 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 9182 },
	{ 0x6faf, 0x6fb0, PDF_CMAP_RANGE, 14972 },
	{ 0x6fb1, 0x6fb3, PDF_CMAP_TABLE, 4371 },
	{ 0x6fb4, 0x6fb5, PDF_CMAP_RANGE, 14975 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 5920 },
	{ 0x6fb7, 0x6fb8, PDF_CMAP_RANGE, 14977 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5919 },
	{ 0x6fba, 0x6fbf, PDF_CMAP_RANGE, 14979 },
	{ 0x6fc0, 0x6fc3, PDF_CMAP_TABLE, 4374 },
	{ 0x6fc4, 0x6fc8, PDF_CMAP_RANGE, 14985 },
	{ 0x6fc9, 0x6fc9, PDF_CMAP_SINGLE, 5917 },
	{ 0x6fca, 0x6fd0, PDF_CMAP_RANGE, 14990 },
	{ 0x6fd1, 0x6fd2, PDF_CMAP_TABLE, 4378 },
	{ 0x6fd3, 0x6fd4, PDF_CMAP_RANGE, 14997 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 8491 },
	{ 0x6fd6, 0x6fd7, PDF_CMAP_RANGE, 14999 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 8357 },
	{ 0x6fd9, 0x6fda, PDF_CMAP_RANGE, 15001 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 9867 },
	{ 0x6fdc, 0x6fdd, PDF_CMAP_RANGE, 15003 },
	{ 0x6fde, 0x6fe1, PDF_CMAP_TABLE, 4380 },
	{ 0x6fe2, 0x6fe3, PDF_CMAP_RANGE, 15005 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 8557 },
	{ 0x6fe5, 0x6fea, PDF_CMAP_RANGE, 15007 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 8193 },
	{ 0x6fec, 0x6fed, PDF_CMAP_RANGE, 15013 },
	{ 0x6fee, 0x6ff1, PDF_CMAP_TABLE, 4384 },
	{ 0x6ff2, 0x6ff9, PDF_CMAP_RANGE, 15015 },
	{ 0x6ffa, 0x6fff, PDF_CMAP_TABLE, 4388 },
	{ 0x7000, 0x7004, PDF_CMAP_RANGE, 15026 },
	{ 0x7005, 0x7006, PDF_CMAP_TABLE, 4394 },
	{ 0x7007, 0x7008, PDF_CMAP_RANGE, 15031 },
	{ 0x7009, 0x700b, PDF_CMAP_TABLE, 4396 },
	{ 0x700c, 0x700e, PDF_CMAP_RANGE, 15034 },
	{ 0x700f, 0x7011, PDF_CMAP_TABLE, 4399 },
	{ 0x7012, 0x7014, PDF_CMAP_RANGE, 15038 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 7759 },
	{ 0x7016, 0x7017, PDF_CMAP_RANGE, 15041 },
	{ 0x7018, 0x7020, PDF_CMAP_TABLE, 4402 },
	{ 0x7021, 0x7022, PDF_CMAP_RANGE, 15046 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 5928 },
	{ 0x7024, 0x7026, PDF_CMAP_RANGE, 15048 },
	{ 0x7027, 0x7028, PDF_CMAP_TABLE, 4411 },
	{ 0x7029, 0x702f, PDF_CMAP_RANGE, 15051 },
	{ 0x7030, 0x7032, PDF_CMAP_TABLE, 4413 },
	{ 0x7033, 0x7034, PDF_CMAP_RANGE, 15059 },
	{ 0x7035, 0x7035, PDF_CMAP_SINGLE, 5931 },
	{ 0x7036, 0x7038, PDF_CMAP_RANGE, 15061 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 5930 },
	{ 0x703a, 0x703d, PDF_CMAP_RANGE, 15064 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 8186 },
	{ 0x703f, 0x7042, PDF_CMAP_RANGE, 15068 },
	{ 0x7043, 0x7044, PDF_CMAP_TABLE, 4416 },
	{ 0x7045, 0x704b, PDF_CMAP_RANGE, 15072 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 1849 },
	{ 0x704d, 0x704e, PDF_CMAP_RANGE, 15079 },
	{ 0x704f, 0x7051, PDF_CMAP_TABLE, 4418 },
	{ 0x7052, 0x7054, PDF_CMAP_RANGE, 15082 },
	{ 0x7055, 0x7055, PDF_CMAP_SINGLE, 8203 },
	{ 0x7056, 0x7057, PDF_CMAP_RANGE, 15085 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 8550 },
	{ 0x7059, 0x705c, PDF_CMAP_RANGE, 15087 },
	{ 0x705d, 0x705e, PDF_CMAP_TABLE, 4421 },
	{ 0x705f, 0x7062, PDF_CMAP_RANGE, 15091 },
	{ 0x7063, 0x7064, PDF_CMAP_TABLE, 4423 },
	{ 0x7065, 0x7066, PDF_CMAP_RANGE, 15095 },
	{ 0x7067, 0x7067, PDF_CMAP_SINGLE, 9191 },
	{ 0x7068, 0x706a, PDF_CMAP_RANGE, 15097 },
	{ 0x706b, 0x7070, PDF_CMAP_TABLE, 4425 },
	{ 0x7071, 0x7074, PDF_CMAP_RANGE, 15101 },
	{ 0x7075, 0x7078, PDF_CMAP_TABLE, 4431 },
	{ 0x7079, 0x707b, PDF_CMAP_RANGE, 15106 },
	{ 0x707c, 0x7080, PDF_CMAP_TABLE, 4435 },
	{ 0x7081, 0x7084, PDF_CMAP_RANGE, 15110 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 6419 },
	{ 0x7086, 0x7088, PDF_CMAP_RANGE, 15114 },
	{ 0x7089, 0x708a, PDF_CMAP_TABLE, 4440 },
	{ 0x708b, 0x708d, PDF_CMAP_RANGE, 15117 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 4096 },
	{ 0x708f, 0x7091, PDF_CMAP_RANGE, 15120 },
	{ 0x7092, 0x7096, PDF_CMAP_TABLE, 4442 },
	{ 0x7097, 0x7098, PDF_CMAP_RANGE, 15124 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 4554 },
	{ 0x709a, 0x709b, PDF_CMAP_RANGE, 15126 },
	{ 0x709c, 0x709d, PDF_CMAP_TABLE, 4447 },
	{ 0x709e, 0x70aa, PDF_CMAP_RANGE, 15128 },
	{ 0x70ab, 0x70b3, PDF_CMAP_TABLE, 4449 },
	{ 0x70b4, 0x70b6, PDF_CMAP_RANGE, 15143 },
	{ 0x70b7, 0x70bd, PDF_CMAP_TABLE, 4458 },
	{ 0x70be, 0x70bf, PDF_CMAP_RANGE, 15147 },
	{ 0x70c0, 0x70c3, PDF_CMAP_TABLE, 4465 },
	{ 0x70c4, 0x70c7, PDF_CMAP_RANGE, 15149 },
	{ 0x70c8, 0x70ca, PDF_CMAP_TABLE, 4469 },
	{ 0x70cb, 0x70ce, PDF_CMAP_RANGE, 15154 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 8611 },
	{ 0x70d0, 0x70d7, PDF_CMAP_RANGE, 15158 },
	{ 0x70d8, 0x70db, PDF_CMAP_TABLE, 4472 },
	{ 0x70dc, 0x70de, PDF_CMAP_RANGE, 15167 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 4085 },
	{ 0x70e0, 0x70e3, PDF_CMAP_RANGE, 15170 },
	{ 0x70e4, 0x70ef, PDF_CMAP_TABLE, 4476 },
	{ 0x70f0, 0x70f3, PDF_CMAP_RANGE, 15177 },
	{ 0x70f4, 0x70f4, PDF_CMAP_SINGLE, 8570 },
	{ 0x70f5, 0x70f6, PDF_CMAP_RANGE, 15181 },
	{ 0x70f7, 0x70f9, PDF_CMAP_TABLE, 4488 },
	{ 0x70fa, 0x70fc, PDF_CMAP_RANGE, 15184 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 1670 },
	{ 0x70fe, 0x7108, PDF_CMAP_RANGE, 15187 },
	{ 0x7109, 0x710a, PDF_CMAP_TABLE, 4491 },
	{ 0x710b, 0x710f, PDF_CMAP_RANGE, 15198 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 6615 },
	{ 0x7111, 0x7112, PDF_CMAP_RANGE, 15203 },
	{ 0x7113, 0x711a, PDF_CMAP_TABLE, 4493 },
	{ 0x711b, 0x7120, PDF_CMAP_RANGE, 15207 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 8613 },
	{ 0x7122, 0x7125, PDF_CMAP_RANGE, 15213 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2186 },
	{ 0x7127, 0x712e, PDF_CMAP_RANGE, 15217 },
	{ 0x712f, 0x7131, PDF_CMAP_TABLE, 4501 },
	{ 0x7132, 0x7135, PDF_CMAP_RANGE, 15225 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 3222 },
	{ 0x7137, 0x7144, PDF_CMAP_RANGE, 15229 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 6623 },
	{ 0x7146, 0x7148, PDF_CMAP_RANGE, 15243 },
	{ 0x7149, 0x714e, PDF_CMAP_TABLE, 4504 },
	{ 0x714f, 0x7151, PDF_CMAP_RANGE, 15248 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 9430 },
	{ 0x7153, 0x715b, PDF_CMAP_RANGE, 15251 },
	{ 0x715c, 0x715e, PDF_CMAP_TABLE, 4510 },
	{ 0x715f, 0x7161, PDF_CMAP_RANGE, 15261 },
	{ 0x7162, 0x7169, PDF_CMAP_TABLE, 4513 },
	{ 0x716a, 0x716b, PDF_CMAP_RANGE, 15266 },
	{ 0x716c, 0x716e, PDF_CMAP_TABLE, 4521 },
	{ 0x716f, 0x7171, PDF_CMAP_RANGE, 15269 },
	{ 0x7172, 0x7173, PDF_CMAP_TABLE, 4524 },
	{ 0x7174, 0x7177, PDF_CMAP_RANGE, 15272 },
	{ 0x7178, 0x717a, PDF_CMAP_TABLE, 4526 },
	{ 0x717b, 0x717c, PDF_CMAP_RANGE, 15277 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 3320 },
	{ 0x717e, 0x7183, PDF_CMAP_RANGE, 15279 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 3861 },
	{ 0x7185, 0x7189, PDF_CMAP_RANGE, 15285 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 4007 },
	{ 0x718b, 0x718e, PDF_CMAP_RANGE, 15290 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 4053 },
	{ 0x7190, 0x7191, PDF_CMAP_RANGE, 15294 },
	{ 0x7192, 0x7194, PDF_CMAP_TABLE, 4529 },
	{ 0x7195, 0x7196, PDF_CMAP_RANGE, 15297 },
	{ 0x7197, 0x7199, PDF_CMAP_TABLE, 4532 },
	{ 0x719a, 0x719e, PDF_CMAP_RANGE, 15299 },
	{ 0x719f, 0x71a0, PDF_CMAP_TABLE, 4535 },
	{ 0x71a1, 0x71a7, PDF_CMAP_RANGE, 15304 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 6631 },
	{ 0x71a9, 0x71ab, PDF_CMAP_RANGE, 15311 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 969 },
	{ 0x71ad, 0x71b0, PDF_CMAP_RANGE, 15314 },
	{ 0x71b1, 0x71b5, PDF_CMAP_TABLE, 4537 },
	{ 0x71b6, 0x71b8, PDF_CMAP_RANGE, 15320 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 6642 },
	{ 0x71ba, 0x71bd, PDF_CMAP_RANGE, 15323 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 7815 },
	{ 0x71bf, 0x71c0, PDF_CMAP_RANGE, 15327 },
	{ 0x71c1, 0x71c3, PDF_CMAP_TABLE, 4542 },
	{ 0x71c4, 0x71c7, PDF_CMAP_RANGE, 15330 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 7866 },
	{ 0x71c9, 0x71cd, PDF_CMAP_RANGE, 15334 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 2576 },
	{ 0x71cf, 0x71d1, PDF_CMAP_RANGE, 15339 },
	{ 0x71d2, 0x71d5, PDF_CMAP_TABLE, 4545 },
	{ 0x71d6, 0x71d8, PDF_CMAP_RANGE, 15343 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 8556 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 15346 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 9433 },
	{ 0x71dd, 0x71de, PDF_CMAP_RANGE, 15348 },
	{ 0x71df, 0x71e0, PDF_CMAP_TABLE, 4549 },
	{ 0x71e1, 0x71e4, PDF_CMAP_RANGE, 15350 },
	{ 0x71e5, 0x71e7, PDF_CMAP_TABLE, 4551 },
	{ 0x71e8, 0x71ec, PDF_CMAP_RANGE, 15354 },
	{ 0x71ed, 0x71ee, PDF_CMAP_TABLE, 4554 },
	{ 0x71ef, 0x71f3, PDF_CMAP_RANGE, 15359 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 8029 },
	{ 0x71f5, 0x71f8, PDF_CMAP_RANGE, 15364 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 6636 },
	{ 0x71fa, 0x71fb, PDF_CMAP_RANGE, 15368 },
	{ 0x71fc, 0x71ff, PDF_CMAP_TABLE, 4556 },
	{ 0x7200, 0x7205, PDF_CMAP_RANGE, 15372 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 1045 },
	{ 0x7207, 0x720c, PDF_CMAP_RANGE, 15378 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 8521 },
	{ 0x720e, 0x720f, PDF_CMAP_RANGE, 15384 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 8262 },
	{ 0x7211, 0x721a, PDF_CMAP_RANGE, 15386 },
	{ 0x721b, 0x721d, PDF_CMAP_TABLE, 4560 },
	{ 0x721e, 0x7227, PDF_CMAP_RANGE, 15397 },
	{ 0x7228, 0x722c, PDF_CMAP_TABLE, 4563 },
	{ 0x722d, 0x722f, PDF_CMAP_RANGE, 15409 },
	{ 0x7230, 0x7232, PDF_CMAP_TABLE, 4568 },
	{ 0x7233, 0x7234, PDF_CMAP_RANGE, 15412 },
	{ 0x7235, 0x723f, PDF_CMAP_TABLE, 4571 },
	{ 0x7240, 0x7246, PDF_CMAP_RANGE, 15415 },
	{ 0x7247, 0x7248, PDF_CMAP_TABLE, 4582 },
	{ 0x7249, 0x724b, PDF_CMAP_RANGE, 15422 },
	{ 0x724c, 0x724d, PDF_CMAP_TABLE, 4584 },
	{ 0x724e, 0x7251, PDF_CMAP_RANGE, 15425 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 6506 },
	{ 0x7253, 0x7255, PDF_CMAP_RANGE, 15429 },
	{ 0x7256, 0x7262, PDF_CMAP_TABLE, 4586 },
	{ 0x7263, 0x7265, PDF_CMAP_RANGE, 15437 },
	{ 0x7266, 0x7269, PDF_CMAP_TABLE, 4599 },
	{ 0x726a, 0x726d, PDF_CMAP_RANGE, 15441 },
	{ 0x726e, 0x726f, PDF_CMAP_TABLE, 4603 },
	{ 0x7270, 0x7271, PDF_CMAP_RANGE, 15445 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 3381 },
	{ 0x7273, 0x7274, PDF_CMAP_RANGE, 15447 },
	{ 0x7275, 0x7275, PDF_CMAP_SINGLE, 3104 },
	{ 0x7276, 0x7278, PDF_CMAP_RANGE, 15449 },
	{ 0x7279, 0x727a, PDF_CMAP_TABLE, 4605 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 15452 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 8396 },
	{ 0x727e, 0x727f, PDF_CMAP_RANGE, 6471 },
	{ 0x7280, 0x7281, PDF_CMAP_TABLE, 4607 },
	{ 0x7282, 0x7283, PDF_CMAP_RANGE, 15454 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 6473 },
	{ 0x7285, 0x7289, PDF_CMAP_RANGE, 15456 },
	{ 0x728a, 0x728f, PDF_CMAP_TABLE, 4609 },
	{ 0x7290, 0x7291, PDF_CMAP_RANGE, 15463 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 6477 },
	{ 0x7293, 0x7295, PDF_CMAP_RANGE, 15465 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 9024 },
	{ 0x7297, 0x729e, PDF_CMAP_RANGE, 15468 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 6467 },
	{ 0x72a0, 0x72a1, PDF_CMAP_RANGE, 15476 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 7889 },
	{ 0x72a3, 0x72a6, PDF_CMAP_RANGE, 15478 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 8620 },
	{ 0x72a8, 0x72ab, PDF_CMAP_RANGE, 15482 },
	{ 0x72ac, 0x72b0, PDF_CMAP_TABLE, 4615 },
	{ 0x72b1, 0x72b3, PDF_CMAP_RANGE, 15487 },
	{ 0x72b4, 0x72b6, PDF_CMAP_TABLE, 4620 },
	{ 0x72b7, 0x72b8, PDF_CMAP_RANGE, 5618 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 4273 },
	{ 0x72ba, 0x72bf, PDF_CMAP_RANGE, 15491 },
	{ 0x72c0, 0x72c4, PDF_CMAP_TABLE, 4623 },
	{ 0x72c5, 0x72c7, PDF_CMAP_RANGE, 15497 },
	{ 0x72c8, 0x72c8, PDF_CMAP_SINGLE, 1056 },
	{ 0x72c9, 0x72cc, PDF_CMAP_RANGE, 15500 },
	{ 0x72cd, 0x72d2, PDF_CMAP_TABLE, 4628 },
	{ 0x72d3, 0x72d6, PDF_CMAP_RANGE, 15506 },
	{ 0x72d7, 0x72d9, PDF_CMAP_TABLE, 4634 },
	{ 0x72da, 0x72dd, PDF_CMAP_RANGE, 15511 },
	{ 0x72de, 0x72e1, PDF_CMAP_TABLE, 4637 },
	{ 0x72e2, 0x72e7, PDF_CMAP_RANGE, 15516 },
	{ 0x72e8, 0x72e9, PDF_CMAP_TABLE, 4641 },
	{ 0x72ea, 0x72eb, PDF_CMAP_RANGE, 15522 },
	{ 0x72ec, 0x72f4, PDF_CMAP_TABLE, 4643 },
	{ 0x72f5, 0x72f6, PDF_CMAP_RANGE, 15524 },
	{ 0x72f7, 0x72f9, PDF_CMAP_TABLE, 4652 },
	{ 0x72fa, 0x72fb, PDF_CMAP_RANGE, 5634 },
	{ 0x72fc, 0x72fd, PDF_CMAP_TABLE, 4655 },
	{ 0x72fe, 0x7300, PDF_CMAP_RANGE, 15526 },
	{ 0x7301, 0x7303, PDF_CMAP_TABLE, 4657 },
	{ 0x7304, 0x7309, PDF_CMAP_RANGE, 15530 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 5639 },
	{ 0x730b, 0x730d, PDF_CMAP_RANGE, 15536 },
	{ 0x730e, 0x730e, PDF_CMAP_SINGLE, 2589 },
	{ 0x730f, 0x7312, PDF_CMAP_RANGE, 15539 },
	{ 0x7313, 0x7317, PDF_CMAP_TABLE, 4660 },
	{ 0x7318, 0x731a, PDF_CMAP_RANGE, 15544 },
	{ 0x731b, 0x731e, PDF_CMAP_TABLE, 4665 },
	{ 0x731f, 0x7320, PDF_CMAP_RANGE, 15547 },
	{ 0x7321, 0x7322, PDF_CMAP_TABLE, 4669 },
	{ 0x7323, 0x7324, PDF_CMAP_RANGE, 15549 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 5645 },
	{ 0x7326, 0x7328, PDF_CMAP_RANGE, 15551 },
	{ 0x7329, 0x732e, PDF_CMAP_TABLE, 4671 },
	{ 0x732f, 0x7330, PDF_CMAP_RANGE, 15555 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 5648 },
	{ 0x7332, 0x7333, PDF_CMAP_RANGE, 15557 },
	{ 0x7334, 0x733b, PDF_CMAP_TABLE, 4677 },
	{ 0x733c, 0x733d, PDF_CMAP_RANGE, 15561 },
	{ 0x733e, 0x7341, PDF_CMAP_TABLE, 4685 },
	{ 0x7342, 0x7343, PDF_CMAP_RANGE, 15564 },
	{ 0x7344, 0x7345, PDF_CMAP_TABLE, 4689 },
	{ 0x7346, 0x734c, PDF_CMAP_RANGE, 15566 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 5650 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 15573 },
	{ 0x7350, 0x7352, PDF_CMAP_TABLE, 4691 },
	{ 0x7353, 0x7356, PDF_CMAP_RANGE, 15576 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 5651 },
	{ 0x7358, 0x735f, PDF_CMAP_RANGE, 15580 },
	{ 0x7360, 0x7360, PDF_CMAP_SINGLE, 5652 },
	{ 0x7361, 0x7367, PDF_CMAP_RANGE, 15588 },
	{ 0x7368, 0x7372, PDF_CMAP_TABLE, 4694 },
	{ 0x7373, 0x7374, PDF_CMAP_RANGE, 15598 },
	{ 0x7375, 0x7380, PDF_CMAP_TABLE, 4705 },
	{ 0x7381, 0x7383, PDF_CMAP_RANGE, 15604 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 4041 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 15607 },
	{ 0x7387, 0x738b, PDF_CMAP_TABLE, 4717 },
	{ 0x738c, 0x738d, PDF_CMAP_RANGE, 15611 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6168 },
	{ 0x738f, 0x7390, PDF_CMAP_RANGE, 15613 },
	{ 0x7391, 0x7391, PDF_CMAP_SINGLE, 6169 },
	{ 0x7392, 0x7395, PDF_CMAP_RANGE, 15615 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 2288 },
	{ 0x7397, 0x739a, PDF_CMAP_RANGE, 15619 },
	{ 0x739b, 0x739b, PDF_CMAP_SINGLE, 2705 },
	{ 0x739c, 0x739e, PDF_CMAP_RANGE, 15623 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 6172 },
	{ 0x73a0, 0x73a1, PDF_CMAP_RANGE, 15626 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 6171 },
	{ 0x73a3, 0x73a8, PDF_CMAP_RANGE, 15628 },
	{ 0x73a9, 0x73ab, PDF_CMAP_TABLE, 4722 },
	{ 0x73ac, 0x73ad, PDF_CMAP_RANGE, 15635 },
	{ 0x73ae, 0x73b3, PDF_CMAP_TABLE, 4725 },
	{ 0x73b4, 0x73b6, PDF_CMAP_RANGE, 15638 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 6176 },
	{ 0x73b8, 0x73b9, PDF_CMAP_RANGE, 15641 },
	{ 0x73ba, 0x73bb, PDF_CMAP_TABLE, 4731 },
	{ 0x73bc, 0x73bf, PDF_CMAP_RANGE, 15643 },
	{ 0x73c0, 0x73c2, PDF_CMAP_TABLE, 4733 },
	{ 0x73c3, 0x73c7, PDF_CMAP_RANGE, 15648 },
	{ 0x73c8, 0x73ca, PDF_CMAP_TABLE, 4736 },
	{ 0x73cb, 0x73cc, PDF_CMAP_RANGE, 15653 },
	{ 0x73cd, 0x73d1, PDF_CMAP_TABLE, 4739 },
	{ 0x73d2, 0x73d8, PDF_CMAP_RANGE, 15656 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 6182 },
	{ 0x73da, 0x73dd, PDF_CMAP_RANGE, 15663 },
	{ 0x73de, 0x73e0, PDF_CMAP_TABLE, 4744 },
	{ 0x73e1, 0x73e4, PDF_CMAP_RANGE, 15668 },
	{ 0x73e5, 0x73e9, PDF_CMAP_TABLE, 4747 },
	{ 0x73ea, 0x73ec, PDF_CMAP_RANGE, 15674 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 1003 },
	{ 0x73ee, 0x73f1, PDF_CMAP_RANGE, 15677 },
	{ 0x73f2, 0x73f2, PDF_CMAP_SINGLE, 6189 },
	{ 0x73f3, 0x73fd, PDF_CMAP_RANGE, 15681 },
	{ 0x73fe, 0x73ff, PDF_CMAP_TABLE, 4752 },
	{ 0x7400, 0x7402, PDF_CMAP_RANGE, 15693 },
	{ 0x7403, 0x7406, PDF_CMAP_TABLE, 4754 },
	{ 0x7407, 0x7408, PDF_CMAP_RANGE, 15697 },
	{ 0x7409, 0x740a, PDF_CMAP_TABLE, 4758 },
	{ 0x740b, 0x740e, PDF_CMAP_RANGE, 15699 },
	{ 0x740f, 0x7410, PDF_CMAP_TABLE, 4760 },
	{ 0x7411, 0x7419, PDF_CMAP_RANGE, 15703 },
	{ 0x741a, 0x741b, PDF_CMAP_TABLE, 4762 },
	{ 0x741c, 0x7421, PDF_CMAP_RANGE, 15712 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 4638 },
	{ 0x7423, 0x7424, PDF_CMAP_RANGE, 15718 },
	{ 0x7425, 0x7430, PDF_CMAP_TABLE, 4764 },
	{ 0x7431, 0x7432, PDF_CMAP_RANGE, 15725 },
	{ 0x7433, 0x7436, PDF_CMAP_TABLE, 4776 },
	{ 0x7437, 0x743b, PDF_CMAP_RANGE, 15727 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 3178 },
	{ 0x743d, 0x743e, PDF_CMAP_RANGE, 15732 },
	{ 0x743f, 0x7441, PDF_CMAP_TABLE, 4780 },
	{ 0x7442, 0x744a, PDF_CMAP_RANGE, 15735 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 9308 },
	{ 0x744c, 0x7454, PDF_CMAP_RANGE, 15744 },
	{ 0x7455, 0x745f, PDF_CMAP_TABLE, 4783 },
	{ 0x7460, 0x7462, PDF_CMAP_RANGE, 15756 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 8541 },
	{ 0x7464, 0x7468, PDF_CMAP_RANGE, 15759 },
	{ 0x7469, 0x746a, PDF_CMAP_TABLE, 4794 },
	{ 0x746b, 0x746c, PDF_CMAP_RANGE, 15764 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 6207 },
	{ 0x746e, 0x746f, PDF_CMAP_RANGE, 15766 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 1854 },
	{ 0x7471, 0x7475, PDF_CMAP_RANGE, 15768 },
	{ 0x7476, 0x7477, PDF_CMAP_TABLE, 4796 },
	{ 0x7478, 0x747d, PDF_CMAP_RANGE, 15773 },
	{ 0x747e, 0x747f, PDF_CMAP_TABLE, 4798 },
	{ 0x7480, 0x7481, PDF_CMAP_RANGE, 6211 },
	{ 0x7482, 0x7483, PDF_CMAP_TABLE, 4800 },
	{ 0x7484, 0x7486, PDF_CMAP_RANGE, 15781 },
	{ 0x7487, 0x748b, PDF_CMAP_TABLE, 4802 },
	{ 0x748c, 0x748d, PDF_CMAP_RANGE, 15786 },
	{ 0x748e, 0x7490, PDF_CMAP_TABLE, 4807 },
	{ 0x7491, 0x749b, PDF_CMAP_RANGE, 15789 },
	{ 0x749c, 0x749e, PDF_CMAP_TABLE, 4810 },
	{ 0x749f, 0x74a2, PDF_CMAP_RANGE, 15801 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 9307 },
	{ 0x74a4, 0x74a5, PDF_CMAP_RANGE, 15805 },
	{ 0x74a6, 0x74a7, PDF_CMAP_TABLE, 4813 },
	{ 0x74a8, 0x74a9, PDF_CMAP_RANGE, 6216 },
	{ 0x74aa, 0x74af, PDF_CMAP_RANGE, 15807 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 8020 },
	{ 0x74b1, 0x74b9, PDF_CMAP_RANGE, 15813 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 6221 },
	{ 0x74bb, 0x74bc, PDF_CMAP_RANGE, 15822 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 9311 },
	{ 0x74be, 0x74c9, PDF_CMAP_RANGE, 15824 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 8428 },
	{ 0x74cb, 0x74ce, PDF_CMAP_RANGE, 15836 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 9309 },
	{ 0x74d0, 0x74d1, PDF_CMAP_RANGE, 15840 },
	{ 0x74d2, 0x74d4, PDF_CMAP_TABLE, 4815 },
	{ 0x74d5, 0x74d9, PDF_CMAP_RANGE, 15843 },
	{ 0x74da, 0x74e6, PDF_CMAP_TABLE, 4818 },
	{ 0x74e7, 0x74ed, PDF_CMAP_RANGE, 15853 },
	{ 0x74ee, 0x74ef, PDF_CMAP_TABLE, 4831 },
	{ 0x74f0, 0x74f3, PDF_CMAP_RANGE, 15860 },
	{ 0x74f4, 0x74f7, PDF_CMAP_TABLE, 4833 },
	{ 0x74f8, 0x74fe, PDF_CMAP_RANGE, 15865 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 6405 },
	{ 0x7500, 0x7503, PDF_CMAP_RANGE, 15872 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 4488 },
	{ 0x7505, 0x750b, PDF_CMAP_RANGE, 15876 },
	{ 0x750c, 0x7513, PDF_CMAP_TABLE, 4837 },
	{ 0x7514, 0x7517, PDF_CMAP_RANGE, 15886 },
	{ 0x7518, 0x751c, PDF_CMAP_TABLE, 4845 },
	{ 0x751d, 0x751e, PDF_CMAP_RANGE, 15891 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 3379 },
	{ 0x7520, 0x7522, PDF_CMAP_RANGE, 15893 },
	{ 0x7523, 0x7525, PDF_CMAP_TABLE, 4850 },
	{ 0x7526, 0x7527, PDF_CMAP_RANGE, 15897 },
	{ 0x7528, 0x753b, PDF_CMAP_TABLE, 4853 },
	{ 0x753c, 0x753d, PDF_CMAP_RANGE, 15904 },
	{ 0x753e, 0x7540, PDF_CMAP_TABLE, 4873 },
	{ 0x7541, 0x7544, PDF_CMAP_RANGE, 15907 },
	{ 0x7545, 0x7545, PDF_CMAP_SINGLE, 1230 },
	{ 0x7546, 0x7547, PDF_CMAP_RANGE, 15911 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 6780 },
	{ 0x7549, 0x754a, PDF_CMAP_RANGE, 15913 },
	{ 0x754b, 0x754f, PDF_CMAP_TABLE, 4876 },
	{ 0x7550, 0x7553, PDF_CMAP_RANGE, 15916 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 2960 },
	{ 0x7555, 0x7558, PDF_CMAP_RANGE, 15920 },
	{ 0x7559, 0x755d, PDF_CMAP_TABLE, 4881 },
	{ 0x755e, 0x7561, PDF_CMAP_RANGE, 15924 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 7746 },
	{ 0x7563, 0x7564, PDF_CMAP_RANGE, 15928 },
	{ 0x7565, 0x7566, PDF_CMAP_TABLE, 4886 },
	{ 0x7567, 0x7569, PDF_CMAP_RANGE, 15930 },
	{ 0x756a, 0x756b, PDF_CMAP_TABLE, 4888 },
	{ 0x756c, 0x7571, PDF_CMAP_RANGE, 15933 },
	{ 0x7572, 0x7579, PDF_CMAP_TABLE, 4890 },
	{ 0x757a, 0x757e, PDF_CMAP_RANGE, 15942 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 6164 },
	{ 0x7580, 0x7582, PDF_CMAP_RANGE, 15947 },
	{ 0x7583, 0x7583, PDF_CMAP_SINGLE, 6784 },
	{ 0x7584, 0x7585, PDF_CMAP_RANGE, 15950 },
	{ 0x7586, 0x7587, PDF_CMAP_TABLE, 4898 },
	{ 0x7588, 0x758a, PDF_CMAP_RANGE, 15952 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 7110 },
	{ 0x758c, 0x758e, PDF_CMAP_RANGE, 15955 },
	{ 0x758f, 0x759a, PDF_CMAP_TABLE, 4900 },
	{ 0x759b, 0x759c, PDF_CMAP_RANGE, 15962 },
	{ 0x759d, 0x75a5, PDF_CMAP_TABLE, 4912 },
	{ 0x75a6, 0x75aa, PDF_CMAP_RANGE, 15966 },
	{ 0x75ab, 0x75b2, PDF_CMAP_TABLE, 4921 },
	{ 0x75b3, 0x75b4, PDF_CMAP_RANGE, 7015 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 1350 },
	{ 0x75b6, 0x75b7, PDF_CMAP_RANGE, 15972 },
	{ 0x75b8, 0x75b9, PDF_CMAP_TABLE, 4929 },
	{ 0x75ba, 0x75bb, PDF_CMAP_RANGE, 15974 },
	{ 0x75bc, 0x75be, PDF_CMAP_TABLE, 4931 },
	{ 0x75bf, 0x75c1, PDF_CMAP_RANGE, 15976 },
	{ 0x75c2, 0x75ca, PDF_CMAP_TABLE, 4934 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 15980 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 7024 },
	{ 0x75ce, 0x75d1, PDF_CMAP_RANGE, 15982 },
	{ 0x75d2, 0x75db, PDF_CMAP_TABLE, 4943 },
	{ 0x75dc, 0x75dd, PDF_CMAP_RANGE, 15989 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 3013 },
	{ 0x75df, 0x75e1, PDF_CMAP_RANGE, 15991 },
	{ 0x75e2, 0x75eb, PDF_CMAP_TABLE, 4953 },
	{ 0x75ec, 0x75ef, PDF_CMAP_RANGE, 15996 },
	{ 0x75f0, 0x75f1, PDF_CMAP_TABLE, 4963 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 16000 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 1274 },
	{ 0x75f5, 0x75f8, PDF_CMAP_RANGE, 16002 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 1085 },
	{ 0x75fa, 0x75fb, PDF_CMAP_RANGE, 16006 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 7033 },
	{ 0x75fd, 0x75fe, PDF_CMAP_RANGE, 16008 },
	{ 0x75ff, 0x7605, PDF_CMAP_TABLE, 4965 },
	{ 0x7606, 0x7609, PDF_CMAP_RANGE, 16011 },
	{ 0x760a, 0x760d, PDF_CMAP_TABLE, 4972 },
	{ 0x760e, 0x760f, PDF_CMAP_RANGE, 16015 },
	{ 0x7610, 0x7610, PDF_CMAP_SINGLE, 7035 },
	{ 0x7611, 0x7614, PDF_CMAP_RANGE, 16017 },
	{ 0x7615, 0x761b, PDF_CMAP_TABLE, 4976 },
	{ 0x761c, 0x761d, PDF_CMAP_RANGE, 16023 },
	{ 0x761e, 0x762d, PDF_CMAP_TABLE, 4983 },
	{ 0x762e, 0x762f, PDF_CMAP_RANGE, 16028 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 7051 },
	{ 0x7631, 0x7632, PDF_CMAP_RANGE, 16030 },
	{ 0x7633, 0x7635, PDF_CMAP_TABLE, 4999 },
	{ 0x7636, 0x7637, PDF_CMAP_RANGE, 16032 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 3214 },
	{ 0x7639, 0x763a, PDF_CMAP_RANGE, 16034 },
	{ 0x763b, 0x7643, PDF_CMAP_TABLE, 5002 },
	{ 0x7644, 0x7645, PDF_CMAP_RANGE, 16038 },
	{ 0x7646, 0x7647, PDF_CMAP_RANGE, 9647 },
	{ 0x7648, 0x7649, PDF_CMAP_TABLE, 5011 },
	{ 0x764a, 0x764b, PDF_CMAP_RANGE, 16041 },
	{ 0x764c, 0x764d, PDF_CMAP_TABLE, 5013 },
	{ 0x764e, 0x7653, PDF_CMAP_RANGE, 16043 },
	{ 0x7654, 0x7658, PDF_CMAP_TABLE, 5015 },
	{ 0x7659, 0x765b, PDF_CMAP_RANGE, 16051 },
	{ 0x765c, 0x765f, PDF_CMAP_TABLE, 5020 },
	{ 0x7660, 0x7661, PDF_CMAP_RANGE, 16055 },
	{ 0x7662, 0x766c, PDF_CMAP_TABLE, 5024 },
	{ 0x766d, 0x766e, PDF_CMAP_RANGE, 9652 },
	{ 0x766f, 0x7672, PDF_CMAP_TABLE, 5035 },
	{ 0x7673, 0x7677, PDF_CMAP_RANGE, 16060 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 1864 },
	{ 0x7679, 0x767a, PDF_CMAP_RANGE, 16065 },
	{ 0x767b, 0x767e, PDF_CMAP_TABLE, 5039 },
	{ 0x767f, 0x7681, PDF_CMAP_RANGE, 16067 },
	{ 0x7682, 0x7688, PDF_CMAP_TABLE, 5043 },
	{ 0x7689, 0x768a, PDF_CMAP_RANGE, 16072 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 1753 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 16074 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 6958 },
	{ 0x768f, 0x7690, PDF_CMAP_RANGE, 16076 },
	{ 0x7691, 0x7693, PDF_CMAP_TABLE, 5050 },
	{ 0x7694, 0x7695, PDF_CMAP_RANGE, 16079 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 3743 },
	{ 0x7697, 0x7698, PDF_CMAP_RANGE, 16081 },
	{ 0x7699, 0x769a, PDF_CMAP_TABLE, 5053 },
	{ 0x769b, 0x76a3, PDF_CMAP_RANGE, 16083 },
	{ 0x76a4, 0x76a4, PDF_CMAP_SINGLE, 6961 },
	{ 0x76a5, 0x76ad, PDF_CMAP_RANGE, 16092 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3011 },
	{ 0x76af, 0x76b0, PDF_CMAP_RANGE, 16101 },
	{ 0x76b1, 0x76b4, PDF_CMAP_TABLE, 5055 },
	{ 0x76b5, 0x76b7, PDF_CMAP_RANGE, 16104 },
	{ 0x76b8, 0x76ba, PDF_CMAP_TABLE, 5059 },
	{ 0x76bb, 0x76be, PDF_CMAP_RANGE, 16108 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2808 },
	{ 0x76c0, 0x76c1, PDF_CMAP_RANGE, 16112 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 4288 },
	{ 0x76c3, 0x76c4, PDF_CMAP_RANGE, 16114 },
	{ 0x76c5, 0x76ca, PDF_CMAP_TABLE, 5062 },
	{ 0x76cb, 0x76cc, PDF_CMAP_RANGE, 16118 },
	{ 0x76cd, 0x76d8, PDF_CMAP_TABLE, 5068 },
	{ 0x76d9, 0x76da, PDF_CMAP_RANGE, 16122 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 3385 },
	{ 0x76dc, 0x76dd, PDF_CMAP_RANGE, 16124 },
	{ 0x76de, 0x76e7, PDF_CMAP_TABLE, 5080 },
	{ 0x76e8, 0x76ed, PDF_CMAP_RANGE, 16129 },
	{ 0x76ee, 0x76f4, PDF_CMAP_TABLE, 5090 },
	{ 0x76f5, 0x76f7, PDF_CMAP_RANGE, 16137 },
	{ 0x76f8, 0x76f9, PDF_CMAP_TABLE, 5097 },
	{ 0x76fa, 0x76fb, PDF_CMAP_RANGE, 16140 },
	{ 0x76fc, 0x7701, PDF_CMAP_TABLE, 5099 },
	{ 0x7702, 0x7703, PDF_CMAP_RANGE, 16145 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 6745 },
	{ 0x7705, 0x7706, PDF_CMAP_RANGE, 16147 },
	{ 0x7707, 0x7708, PDF_CMAP_RANGE, 6748 },
	{ 0x7709, 0x770d, PDF_CMAP_TABLE, 5105 },
	{ 0x770e, 0x7718, PDF_CMAP_RANGE, 16151 },
	{ 0x7719, 0x771a, PDF_CMAP_TABLE, 5110 },
	{ 0x771b, 0x771e, PDF_CMAP_RANGE, 16162 },
	{ 0x771f, 0x7722, PDF_CMAP_TABLE, 5112 },
	{ 0x7723, 0x7725, PDF_CMAP_RANGE, 16167 },
	{ 0x7726, 0x7729, PDF_CMAP_TABLE, 5116 },
	{ 0x772a, 0x772c, PDF_CMAP_RANGE, 16171 },
	{ 0x772d, 0x772f, PDF_CMAP_TABLE, 5120 },
	{ 0x7730, 0x7734, PDF_CMAP_RANGE, 16175 },
	{ 0x7735, 0x773c, PDF_CMAP_TABLE, 5123 },
	{ 0x773d, 0x773f, PDF_CMAP_RANGE, 16182 },
	{ 0x7740, 0x7743, PDF_CMAP_TABLE, 5131 },
	{ 0x7744, 0x7746, PDF_CMAP_RANGE, 16186 },
	{ 0x7747, 0x7747, PDF_CMAP_SINGLE, 6759 },
	{ 0x7748, 0x774e, PDF_CMAP_RANGE, 16189 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 9860 },
	{ 0x7750, 0x7751, PDF_CMAP_RANGE, 6757 },
	{ 0x7752, 0x7759, PDF_CMAP_RANGE, 16196 },
	{ 0x775a, 0x775b, PDF_CMAP_TABLE, 5135 },
	{ 0x775c, 0x775d, PDF_CMAP_RANGE, 16204 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 9455 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 16206 },
	{ 0x7761, 0x7768, PDF_CMAP_TABLE, 5137 },
	{ 0x7769, 0x776a, PDF_CMAP_RANGE, 16210 },
	{ 0x776b, 0x776c, PDF_CMAP_TABLE, 5145 },
	{ 0x776d, 0x7778, PDF_CMAP_RANGE, 16212 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 1538 },
	{ 0x777a, 0x777c, PDF_CMAP_RANGE, 16224 },
	{ 0x777d, 0x7780, PDF_CMAP_TABLE, 5147 },
	{ 0x7781, 0x7783, PDF_CMAP_RANGE, 16227 },
	{ 0x7784, 0x7785, PDF_CMAP_TABLE, 5151 },
	{ 0x7786, 0x778b, PDF_CMAP_RANGE, 16230 },
	{ 0x778c, 0x778e, PDF_CMAP_TABLE, 5153 },
	{ 0x778f, 0x7790, PDF_CMAP_RANGE, 16236 },
	{ 0x7791, 0x7792, PDF_CMAP_TABLE, 5156 },
	{ 0x7793, 0x7797, PDF_CMAP_RANGE, 16238 },
	{ 0x7798, 0x7798, PDF_CMAP_SINGLE, 9454 },
	{ 0x7799, 0x779d, PDF_CMAP_RANGE, 16243 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 8308 },
	{ 0x779f, 0x77a0, PDF_CMAP_RANGE, 6771 },
	{ 0x77a1, 0x77a2, PDF_CMAP_TABLE, 5158 },
	{ 0x77a3, 0x77a4, PDF_CMAP_RANGE, 16249 },
	{ 0x77a5, 0x77ad, PDF_CMAP_TABLE, 5160 },
	{ 0x77ae, 0x77af, PDF_CMAP_RANGE, 16254 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 6773 },
	{ 0x77b1, 0x77b2, PDF_CMAP_RANGE, 16256 },
	{ 0x77b3, 0x77b5, PDF_CMAP_TABLE, 5169 },
	{ 0x77b6, 0x77ba, PDF_CMAP_RANGE, 16259 },
	{ 0x77bb, 0x77bf, PDF_CMAP_TABLE, 5172 },
	{ 0x77c0, 0x77c6, PDF_CMAP_RANGE, 16265 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 9869 },
	{ 0x77c8, 0x77cc, PDF_CMAP_RANGE, 16272 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5015 },
	{ 0x77ce, 0x77d6, PDF_CMAP_RANGE, 16277 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 1318 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 16286 },
	{ 0x77da, 0x77dc, PDF_CMAP_TABLE, 5177 },
	{ 0x77dd, 0x77e1, PDF_CMAP_RANGE, 16288 },
	{ 0x77e2, 0x77ef, PDF_CMAP_TABLE, 5180 },
	{ 0x77f0, 0x77f2, PDF_CMAP_RANGE, 16297 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 3398 },
	{ 0x77f4, 0x77f5, PDF_CMAP_RANGE, 16300 },
	{ 0x77f6, 0x77f8, PDF_CMAP_TABLE, 5194 },
	{ 0x77f9, 0x77fc, PDF_CMAP_RANGE, 16303 },
	{ 0x77fd, 0x7802, PDF_CMAP_TABLE, 5197 },
	{ 0x7803, 0x7808, PDF_CMAP_RANGE, 16307 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 6693 },
	{ 0x780a, 0x780b, PDF_CMAP_RANGE, 16313 },
	{ 0x780c, 0x780d, PDF_CMAP_TABLE, 5203 },
	{ 0x780e, 0x7810, PDF_CMAP_RANGE, 16315 },
	{ 0x7811, 0x7816, PDF_CMAP_TABLE, 5205 },
	{ 0x7817, 0x7818, PDF_CMAP_RANGE, 6694 },
	{ 0x7819, 0x781b, PDF_CMAP_TABLE, 5211 },
	{ 0x781c, 0x781d, PDF_CMAP_RANGE, 6699 },
	{ 0x781e, 0x781f, PDF_CMAP_TABLE, 5214 },
	{ 0x7820, 0x7822, PDF_CMAP_RANGE, 16323 },
	{ 0x7823, 0x7829, PDF_CMAP_TABLE, 5216 },
	{ 0x782a, 0x782b, PDF_CMAP_RANGE, 16328 },
	{ 0x782c, 0x782d, PDF_CMAP_TABLE, 5223 },
	{ 0x782e, 0x782f, PDF_CMAP_RANGE, 16330 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 2986 },
	{ 0x7831, 0x7833, PDF_CMAP_RANGE, 16332 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 3045 },
	{ 0x7835, 0x7836, PDF_CMAP_RANGE, 16335 },
	{ 0x7837, 0x7838, PDF_CMAP_TABLE, 5225 },
	{ 0x7839, 0x783b, PDF_CMAP_RANGE, 6701 },
	{ 0x783c, 0x7840, PDF_CMAP_TABLE, 5227 },
	{ 0x7841, 0x7842, PDF_CMAP_RANGE, 16339 },
	{ 0x7843, 0x7847, PDF_CMAP_TABLE, 5232 },
	{ 0x7848, 0x784b, PDF_CMAP_RANGE, 16343 },
	{ 0x784c, 0x7852, PDF_CMAP_TABLE, 5237 },
	{ 0x7853, 0x7854, PDF_CMAP_RANGE, 16350 },
	{ 0x7855, 0x7855, PDF_CMAP_SINGLE, 3499 },
	{ 0x7856, 0x7857, PDF_CMAP_RANGE, 6712 },
	{ 0x7858, 0x785c, PDF_CMAP_RANGE, 16352 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3938 },
	{ 0x785e, 0x7863, PDF_CMAP_RANGE, 16357 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 9449 },
	{ 0x7865, 0x7867, PDF_CMAP_RANGE, 16363 },
	{ 0x7868, 0x786f, PDF_CMAP_TABLE, 5244 },
	{ 0x7870, 0x7876, PDF_CMAP_RANGE, 16367 },
	{ 0x7877, 0x7877, PDF_CMAP_SINGLE, 2146 },
	{ 0x7878, 0x787b, PDF_CMAP_RANGE, 16374 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 2993 },
	{ 0x787d, 0x7886, PDF_CMAP_RANGE, 16378 },
	{ 0x7887, 0x7889, PDF_CMAP_TABLE, 5252 },
	{ 0x788a, 0x788b, PDF_CMAP_RANGE, 16389 },
	{ 0x788c, 0x788e, PDF_CMAP_TABLE, 5255 },
	{ 0x788f, 0x7890, PDF_CMAP_RANGE, 16391 },
	{ 0x7891, 0x7893, PDF_CMAP_TABLE, 5258 },
	{ 0x7894, 0x7896, PDF_CMAP_RANGE, 16394 },
	{ 0x7897, 0x789c, PDF_CMAP_TABLE, 5261 },
	{ 0x789d, 0x789e, PDF_CMAP_RANGE, 16398 },
	{ 0x789f, 0x78a9, PDF_CMAP_TABLE, 5267 },
	{ 0x78aa, 0x78ac, PDF_CMAP_RANGE, 16405 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 9444 },
	{ 0x78ae, 0x78af, PDF_CMAP_RANGE, 16408 },
	{ 0x78b0, 0x78b4, PDF_CMAP_TABLE, 5278 },
	{ 0x78b5, 0x78b7, PDF_CMAP_RANGE, 16410 },
	{ 0x78b8, 0x78be, PDF_CMAP_TABLE, 5283 },
	{ 0x78bf, 0x78c0, PDF_CMAP_RANGE, 16415 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 1352 },
	{ 0x78c2, 0x78c4, PDF_CMAP_RANGE, 16417 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 1024 },
	{ 0x78c6, 0x78c8, PDF_CMAP_RANGE, 16420 },
	{ 0x78c9, 0x78cb, PDF_CMAP_TABLE, 5290 },
	{ 0x78cc, 0x78cf, PDF_CMAP_RANGE, 16423 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 2958 },
	{ 0x78d1, 0x78d3, PDF_CMAP_RANGE, 16427 },
	{ 0x78d4, 0x78d5, PDF_CMAP_TABLE, 5293 },
	{ 0x78d6, 0x78d8, PDF_CMAP_RANGE, 16430 },
	{ 0x78d9, 0x78da, PDF_CMAP_TABLE, 5295 },
	{ 0x78db, 0x78e2, PDF_CMAP_RANGE, 16433 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 9452 },
	{ 0x78e4, 0x78e6, PDF_CMAP_RANGE, 16441 },
	{ 0x78e7, 0x78e8, PDF_CMAP_TABLE, 5297 },
	{ 0x78e9, 0x78eb, PDF_CMAP_RANGE, 16444 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 6732 },
	{ 0x78ed, 0x78ee, PDF_CMAP_RANGE, 16447 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 9443 },
	{ 0x78f0, 0x78f1, PDF_CMAP_RANGE, 16449 },
	{ 0x78f2, 0x78f4, PDF_CMAP_TABLE, 5299 },
	{ 0x78f5, 0x78f6, PDF_CMAP_RANGE, 16452 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 2592 },
	{ 0x78f8, 0x78f9, PDF_CMAP_RANGE, 16454 },
	{ 0x78fa, 0x78fa, PDF_CMAP_SINGLE, 2012 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 16456 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 9450 },
	{ 0x78fe, 0x7900, PDF_CMAP_RANGE, 16458 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2185 },
	{ 0x7902, 0x7904, PDF_CMAP_RANGE, 16461 },
	{ 0x7905, 0x7905, PDF_CMAP_SINGLE, 6734 },
	{ 0x7906, 0x790d, PDF_CMAP_RANGE, 16464 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 7826 },
	{ 0x790f, 0x7912, PDF_CMAP_RANGE, 16472 },
	{ 0x7913, 0x7913, PDF_CMAP_SINGLE, 6736 },
	{ 0x7914, 0x7918, PDF_CMAP_RANGE, 16476 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 7719 },
	{ 0x791a, 0x791d, PDF_CMAP_RANGE, 16481 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 6738 },
	{ 0x791f, 0x7923, PDF_CMAP_RANGE, 16485 },
	{ 0x7924, 0x7926, PDF_CMAP_TABLE, 5302 },
	{ 0x7927, 0x7929, PDF_CMAP_RANGE, 16491 },
	{ 0x792a, 0x792c, PDF_CMAP_TABLE, 5305 },
	{ 0x792d, 0x7930, PDF_CMAP_RANGE, 16494 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 9448 },
	{ 0x7932, 0x7933, PDF_CMAP_RANGE, 16498 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 6739 },
	{ 0x7935, 0x7939, PDF_CMAP_RANGE, 16500 },
	{ 0x793a, 0x7941, PDF_CMAP_TABLE, 5308 },
	{ 0x7942, 0x7945, PDF_CMAP_RANGE, 16507 },
	{ 0x7946, 0x7949, PDF_CMAP_TABLE, 5316 },
	{ 0x794a, 0x7952, PDF_CMAP_RANGE, 16512 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 6654 },
	{ 0x7954, 0x7955, PDF_CMAP_RANGE, 16521 },
	{ 0x7956, 0x7957, PDF_CMAP_TABLE, 5320 },
	{ 0x7958, 0x7959, PDF_CMAP_RANGE, 16523 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 6655 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 6652 },
	{ 0x795d, 0x7962, PDF_CMAP_TABLE, 5322 },
	{ 0x7963, 0x7964, PDF_CMAP_RANGE, 16526 },
	{ 0x7965, 0x7968, PDF_CMAP_TABLE, 5328 },
	{ 0x7969, 0x796c, PDF_CMAP_RANGE, 16529 },
	{ 0x796d, 0x796f, PDF_CMAP_TABLE, 5332 },
	{ 0x7970, 0x7976, PDF_CMAP_RANGE, 16534 },
	{ 0x7977, 0x797a, PDF_CMAP_TABLE, 5335 },
	{ 0x797b, 0x797f, PDF_CMAP_RANGE, 16542 },
	{ 0x7980, 0x7981, PDF_CMAP_TABLE, 5339 },
	{ 0x7982, 0x7983, PDF_CMAP_RANGE, 16547 },
	{ 0x7984, 0x7985, PDF_CMAP_TABLE, 5341 },
	{ 0x7986, 0x7989, PDF_CMAP_RANGE, 16549 },
	{ 0x798a, 0x798a, PDF_CMAP_SINGLE, 6663 },
	{ 0x798b, 0x798c, PDF_CMAP_RANGE, 16553 },
	{ 0x798d, 0x798f, PDF_CMAP_TABLE, 5343 },
	{ 0x7990, 0x7999, PDF_CMAP_RANGE, 16555 },
	{ 0x799a, 0x799a, PDF_CMAP_SINGLE, 6664 },
	{ 0x799b, 0x79a5, PDF_CMAP_RANGE, 16565 },
	{ 0x79a6, 0x79a7, PDF_CMAP_TABLE, 5346 },
	{ 0x79a8, 0x79a9, PDF_CMAP_RANGE, 16576 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 9437 },
	{ 0x79ab, 0x79ad, PDF_CMAP_RANGE, 16578 },
	{ 0x79ae, 0x79b3, PDF_CMAP_TABLE, 5348 },
	{ 0x79b4, 0x79b8, PDF_CMAP_RANGE, 16583 },
	{ 0x79b9, 0x79c3, PDF_CMAP_TABLE, 5354 },
	{ 0x79c4, 0x79c5, PDF_CMAP_RANGE, 16591 },
	{ 0x79c6, 0x79c6, PDF_CMAP_SINGLE, 1740 },
	{ 0x79c7, 0x79c8, PDF_CMAP_RANGE, 16593 },
	{ 0x79c9, 0x79cd, PDF_CMAP_TABLE, 5365 },
	{ 0x79ce, 0x79d0, PDF_CMAP_RANGE, 16597 },
	{ 0x79d1, 0x79d2, PDF_CMAP_TABLE, 5370 },
	{ 0x79d3, 0x79d4, PDF_CMAP_RANGE, 16600 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 6941 },
	{ 0x79d6, 0x79d7, PDF_CMAP_RANGE, 16602 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2781 },
	{ 0x79d9, 0x79de, PDF_CMAP_RANGE, 16604 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 4671 },
	{ 0x79e0, 0x79e2, PDF_CMAP_RANGE, 16610 },
	{ 0x79e3, 0x79f0, PDF_CMAP_TABLE, 5372 },
	{ 0x79f1, 0x79f7, PDF_CMAP_RANGE, 16618 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 2214 },
	{ 0x79f9, 0x79fa, PDF_CMAP_RANGE, 16625 },
	{ 0x79fb, 0x79fd, PDF_CMAP_TABLE, 5386 },
	{ 0x79fe, 0x79ff, PDF_CMAP_RANGE, 16628 },
	{ 0x7a00, 0x7a03, PDF_CMAP_TABLE, 5389 },
	{ 0x7a04, 0x7a05, PDF_CMAP_RANGE, 16631 },
	{ 0x7a06, 0x7a06, PDF_CMAP_SINGLE, 6945 },
	{ 0x7a07, 0x7a0a, PDF_CMAP_RANGE, 16633 },
	{ 0x7a0b, 0x7a0e, PDF_CMAP_TABLE, 5393 },
	{ 0x7a0f, 0x7a13, PDF_CMAP_RANGE, 16638 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 6950 },
	{ 0x7a15, 0x7a16, PDF_CMAP_RANGE, 16643 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 1001 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 16645 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 4552 },
	{ 0x7a1b, 0x7a1d, PDF_CMAP_RANGE, 16647 },
	{ 0x7a1e, 0x7a20, PDF_CMAP_TABLE, 5397 },
	{ 0x7a21, 0x7a22, PDF_CMAP_RANGE, 16651 },
	{ 0x7a23, 0x7a23, PDF_CMAP_SINGLE, 7567 },
	{ 0x7a24, 0x7a2d, PDF_CMAP_RANGE, 16653 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 8853 },
	{ 0x7a2f, 0x7a30, PDF_CMAP_RANGE, 16663 },
	{ 0x7a31, 0x7a33, PDF_CMAP_TABLE, 5400 },
	{ 0x7a34, 0x7a36, PDF_CMAP_RANGE, 16666 },
	{ 0x7a37, 0x7a40, PDF_CMAP_TABLE, 5403 },
	{ 0x7a41, 0x7a45, PDF_CMAP_RANGE, 16672 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 2853 },
	{ 0x7a47, 0x7a4b, PDF_CMAP_RANGE, 16677 },
	{ 0x7a4c, 0x7a4e, PDF_CMAP_TABLE, 5413 },
	{ 0x7a4f, 0x7a50, PDF_CMAP_RANGE, 16682 },
	{ 0x7a51, 0x7a51, PDF_CMAP_SINGLE, 6953 },
	{ 0x7a52, 0x7a56, PDF_CMAP_RANGE, 16684 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 3552 },
	{ 0x7a58, 0x7a60, PDF_CMAP_RANGE, 16689 },
	{ 0x7a61, 0x7a62, PDF_CMAP_TABLE, 5416 },
	{ 0x7a63, 0x7a68, PDF_CMAP_RANGE, 16698 },
	{ 0x7a69, 0x7a6b, PDF_CMAP_TABLE, 5418 },
	{ 0x7a6c, 0x7a6f, PDF_CMAP_RANGE, 16705 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 6956 },
	{ 0x7a71, 0x7a73, PDF_CMAP_RANGE, 16709 },
	{ 0x7a74, 0x7a77, PDF_CMAP_TABLE, 5421 },
	{ 0x7a78, 0x7a79, PDF_CMAP_RANGE, 7066 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 2402 },
	{ 0x7a7b, 0x7a7e, PDF_CMAP_RANGE, 16713 },
	{ 0x7a7f, 0x7a88, PDF_CMAP_TABLE, 5425 },
	{ 0x7a89, 0x7a8c, PDF_CMAP_RANGE, 16720 },
	{ 0x7a8d, 0x7a8d, PDF_CMAP_SINGLE, 3148 },
	{ 0x7a8e, 0x7a90, PDF_CMAP_RANGE, 16724 },
	{ 0x7a91, 0x7a92, PDF_CMAP_TABLE, 5435 },
	{ 0x7a93, 0x7a94, PDF_CMAP_RANGE, 16727 },
	{ 0x7a95, 0x7a98, PDF_CMAP_TABLE, 5437 },
	{ 0x7a99, 0x7a9b, PDF_CMAP_RANGE, 16729 },
	{ 0x7a9c, 0x7aa0, PDF_CMAP_TABLE, 5441 },
	{ 0x7aa1, 0x7aa4, PDF_CMAP_RANGE, 16733 },
	{ 0x7aa5, 0x7aae, PDF_CMAP_TABLE, 5446 },
	{ 0x7aaf, 0x7ab2, PDF_CMAP_RANGE, 16739 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 7077 },
	{ 0x7ab4, 0x7ab5, PDF_CMAP_RANGE, 16743 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 9657 },
	{ 0x7ab7, 0x7ab9, PDF_CMAP_RANGE, 16745 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 8170 },
	{ 0x7abb, 0x7abe, PDF_CMAP_RANGE, 16748 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 2631 },
	{ 0x7ac0, 0x7ac3, PDF_CMAP_RANGE, 16752 },
	{ 0x7ac4, 0x7acb, PDF_CMAP_TABLE, 5456 },
	{ 0x7acc, 0x7ad5, PDF_CMAP_RANGE, 16758 },
	{ 0x7ad6, 0x7ad6, PDF_CMAP_SINGLE, 3473 },
	{ 0x7ad7, 0x7ad8, PDF_CMAP_RANGE, 16768 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 4447 },
	{ 0x7ada, 0x7add, PDF_CMAP_RANGE, 16770 },
	{ 0x7ade, 0x7ae0, PDF_CMAP_TABLE, 5464 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 16774 },
	{ 0x7ae3, 0x7ae6, PDF_CMAP_TABLE, 5467 },
	{ 0x7ae7, 0x7ae9, PDF_CMAP_RANGE, 16777 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 8513 },
	{ 0x7aeb, 0x7aec, PDF_CMAP_RANGE, 16780 },
	{ 0x7aed, 0x7aef, PDF_CMAP_TABLE, 5471 },
	{ 0x7af0, 0x7af5, PDF_CMAP_RANGE, 16783 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 8135 },
	{ 0x7af7, 0x7af8, PDF_CMAP_RANGE, 16789 },
	{ 0x7af9, 0x7afa, PDF_CMAP_TABLE, 5474 },
	{ 0x7afb, 0x7afc, PDF_CMAP_RANGE, 16791 },
	{ 0x7afd, 0x7aff, PDF_CMAP_TABLE, 5476 },
	{ 0x7b00, 0x7b02, PDF_CMAP_RANGE, 16794 },
	{ 0x7b03, 0x7b04, PDF_CMAP_RANGE, 7270 },
	{ 0x7b05, 0x7b0b, PDF_CMAP_TABLE, 5479 },
	{ 0x7b0c, 0x7b0e, PDF_CMAP_RANGE, 16800 },
	{ 0x7b0f, 0x7b11, PDF_CMAP_TABLE, 5486 },
	{ 0x7b12, 0x7b13, PDF_CMAP_RANGE, 16804 },
	{ 0x7b14, 0x7b15, PDF_CMAP_TABLE, 5489 },
	{ 0x7b16, 0x7b18, PDF_CMAP_RANGE, 16806 },
	{ 0x7b19, 0x7b1b, PDF_CMAP_TABLE, 5491 },
	{ 0x7b1c, 0x7b1d, PDF_CMAP_RANGE, 16810 },
	{ 0x7b1e, 0x7b20, PDF_CMAP_TABLE, 5494 },
	{ 0x7b21, 0x7b23, PDF_CMAP_RANGE, 16813 },
	{ 0x7b24, 0x7b2e, PDF_CMAP_TABLE, 5497 },
	{ 0x7b2f, 0x7b30, PDF_CMAP_RANGE, 16819 },
	{ 0x7b31, 0x7b33, PDF_CMAP_TABLE, 5508 },
	{ 0x7b34, 0x7b37, PDF_CMAP_RANGE, 16822 },
	{ 0x7b38, 0x7b3e, PDF_CMAP_TABLE, 5511 },
	{ 0x7b3f, 0x7b44, PDF_CMAP_RANGE, 16829 },
	{ 0x7b45, 0x7b4c, PDF_CMAP_TABLE, 5518 },
	{ 0x7b4d, 0x7b4e, PDF_CMAP_RANGE, 16837 },
	{ 0x7b4f, 0x7b5d, PDF_CMAP_TABLE, 5526 },
	{ 0x7b5e, 0x7b5f, PDF_CMAP_RANGE, 16844 },
	{ 0x7b60, 0x7b62, PDF_CMAP_TABLE, 5541 },
	{ 0x7b63, 0x7b66, PDF_CMAP_RANGE, 16847 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 9700 },
	{ 0x7b68, 0x7b6d, PDF_CMAP_RANGE, 16851 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 7295 },
	{ 0x7b6f, 0x7b70, PDF_CMAP_RANGE, 16857 },
	{ 0x7b71, 0x7b72, PDF_CMAP_TABLE, 5544 },
	{ 0x7b73, 0x7b74, PDF_CMAP_RANGE, 16859 },
	{ 0x7b75, 0x7b7b, PDF_CMAP_TABLE, 5546 },
	{ 0x7b7c, 0x7b7d, PDF_CMAP_RANGE, 16864 },
	{ 0x7b7e, 0x7b80, PDF_CMAP_TABLE, 5553 },
	{ 0x7b81, 0x7b84, PDF_CMAP_RANGE, 16867 },
	{ 0x7b85, 0x7b85, PDF_CMAP_SINGLE, 7307 },
	{ 0x7b86, 0x7b8a, PDF_CMAP_RANGE, 16871 },
	{ 0x7b8b, 0x7b8d, PDF_CMAP_TABLE, 5556 },
	{ 0x7b8e, 0x7b8f, PDF_CMAP_RANGE, 16877 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 7300 },
	{ 0x7b91, 0x7b93, PDF_CMAP_RANGE, 16879 },
	{ 0x7b94, 0x7b97, PDF_CMAP_TABLE, 5559 },
	{ 0x7b98, 0x7b9b, PDF_CMAP_RANGE, 16883 },
	{ 0x7b9c, 0x7b9d, PDF_CMAP_TABLE, 5563 },
	{ 0x7b9e, 0x7ba0, PDF_CMAP_RANGE, 16887 },
	{ 0x7ba1, 0x7ba2, PDF_CMAP_TABLE, 5565 },
	{ 0x7ba3, 0x7ba5, PDF_CMAP_RANGE, 16890 },
	{ 0x7ba6, 0x7ba7, PDF_CMAP_RANGE, 7301 },
	{ 0x7ba8, 0x7bad, PDF_CMAP_TABLE, 5567 },
	{ 0x7bae, 0x7bb0, PDF_CMAP_RANGE, 16893 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3921 },
	{ 0x7bb2, 0x7bb3, PDF_CMAP_RANGE, 16896 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 7312 },
	{ 0x7bb5, 0x7bb7, PDF_CMAP_RANGE, 16898 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 7303 },
	{ 0x7bb9, 0x7bbf, PDF_CMAP_RANGE, 16901 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 5573 },
	{ 0x7bc2, 0x7bc3, PDF_CMAP_RANGE, 16908 },
	{ 0x7bc4, 0x7bcc, PDF_CMAP_TABLE, 5575 },
	{ 0x7bcd, 0x7bd0, PDF_CMAP_RANGE, 16913 },
	{ 0x7bd1, 0x7bd3, PDF_CMAP_TABLE, 5584 },
	{ 0x7bd4, 0x7bd8, PDF_CMAP_RANGE, 16918 },
	{ 0x7bd9, 0x7bda, PDF_CMAP_TABLE, 5587 },
	{ 0x7bdb, 0x7bdc, PDF_CMAP_RANGE, 16923 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 7316 },
	{ 0x7bde, 0x7be0, PDF_CMAP_RANGE, 16925 },
	{ 0x7be1, 0x7be1, PDF_CMAP_SINGLE, 1374 },
	{ 0x7be2, 0x7be3, PDF_CMAP_RANGE, 16928 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 9699 },
	{ 0x7be5, 0x7be6, PDF_CMAP_RANGE, 7318 },
	{ 0x7be7, 0x7be8, PDF_CMAP_RANGE, 16930 },
	{ 0x7be9, 0x7bea, PDF_CMAP_TABLE, 5589 },
	{ 0x7beb, 0x7bed, PDF_CMAP_RANGE, 16932 },
	{ 0x7bee, 0x7bee, PDF_CMAP_SINGLE, 2469 },
	{ 0x7bef, 0x7bf0, PDF_CMAP_RANGE, 16935 },
	{ 0x7bf1, 0x7bf3, PDF_CMAP_TABLE, 5591 },
	{ 0x7bf4, 0x7bf6, PDF_CMAP_RANGE, 16938 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 2994 },
	{ 0x7bf8, 0x7bfb, PDF_CMAP_RANGE, 16941 },
	{ 0x7bfc, 0x7c00, PDF_CMAP_TABLE, 5594 },
	{ 0x7c01, 0x7c06, PDF_CMAP_RANGE, 16947 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 1371 },
	{ 0x7c08, 0x7c0a, PDF_CMAP_RANGE, 16953 },
	{ 0x7c0b, 0x7c0f, PDF_CMAP_TABLE, 5599 },
	{ 0x7c10, 0x7c15, PDF_CMAP_RANGE, 16957 },
	{ 0x7c16, 0x7c16, PDF_CMAP_SINGLE, 7325 },
	{ 0x7c17, 0x7c1d, PDF_CMAP_RANGE, 16963 },
	{ 0x7c1e, 0x7c23, PDF_CMAP_TABLE, 5604 },
	{ 0x7c24, 0x7c25, PDF_CMAP_RANGE, 16972 },
	{ 0x7c26, 0x7c27, PDF_CMAP_TABLE, 5610 },
	{ 0x7c28, 0x7c29, PDF_CMAP_RANGE, 16974 },
	{ 0x7c2a, 0x7c2b, PDF_CMAP_TABLE, 5612 },
	{ 0x7c2c, 0x7c37, PDF_CMAP_RANGE, 16976 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 7330 },
	{ 0x7c39, 0x7c3c, PDF_CMAP_RANGE, 16988 },
	{ 0x7c3d, 0x7c43, PDF_CMAP_TABLE, 5614 },
	{ 0x7c44, 0x7c4b, PDF_CMAP_RANGE, 16993 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 5621 },
	{ 0x7c4e, 0x7c5b, PDF_CMAP_RANGE, 17001 },
	{ 0x7c5c, 0x7c5c, PDF_CMAP_SINGLE, 9705 },
	{ 0x7c5d, 0x7c5e, PDF_CMAP_RANGE, 17015 },
	{ 0x7c5f, 0x7c60, PDF_CMAP_TABLE, 5623 },
	{ 0x7c61, 0x7c63, PDF_CMAP_RANGE, 17017 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 9875 },
	{ 0x7c65, 0x7c68, PDF_CMAP_RANGE, 17020 },
	{ 0x7c69, 0x7c6e, PDF_CMAP_TABLE, 5625 },
	{ 0x7c6f, 0x7c71, PDF_CMAP_RANGE, 17026 },
	{ 0x7c72, 0x7c74, PDF_CMAP_TABLE, 5631 },
	{ 0x7c75, 0x7c7a, PDF_CMAP_RANGE, 17029 },
	{ 0x7c7b, 0x7c7d, PDF_CMAP_TABLE, 5634 },
	{ 0x7c7e, 0x7c88, PDF_CMAP_RANGE, 17035 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 1656 },
	{ 0x7c8a, 0x7c90, PDF_CMAP_RANGE, 17046 },
	{ 0x7c91, 0x7c92, PDF_CMAP_TABLE, 5637 },
	{ 0x7c93, 0x7c94, PDF_CMAP_RANGE, 17053 },
	{ 0x7c95, 0x7c98, PDF_CMAP_TABLE, 5639 },
	{ 0x7c99, 0x7c9b, PDF_CMAP_RANGE, 17056 },
	{ 0x7c9c, 0x7c9f, PDF_CMAP_TABLE, 5643 },
	{ 0x7ca0, 0x7ca1, PDF_CMAP_RANGE, 17059 },
	{ 0x7ca2, 0x7ca5, PDF_CMAP_TABLE, 5647 },
	{ 0x7ca6, 0x7ca9, PDF_CMAP_RANGE, 17062 },
	{ 0x7caa, 0x7caa, PDF_CMAP_SINGLE, 1661 },
	{ 0x7cab, 0x7cad, PDF_CMAP_RANGE, 17066 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 2561 },
	{ 0x7caf, 0x7cb0, PDF_CMAP_RANGE, 17069 },
	{ 0x7cb1, 0x7cb3, PDF_CMAP_TABLE, 5651 },
	{ 0x7cb4, 0x7cb8, PDF_CMAP_RANGE, 17071 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 1381 },
	{ 0x7cba, 0x7cbb, PDF_CMAP_RANGE, 17076 },
	{ 0x7cbc, 0x7cbd, PDF_CMAP_RANGE, 7378 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 2266 },
	{ 0x7cbf, 0x7cc0, PDF_CMAP_RANGE, 17078 },
	{ 0x7cc1, 0x7cc1, PDF_CMAP_SINGLE, 7380 },
	{ 0x7cc2, 0x7cc4, PDF_CMAP_RANGE, 17080 },
	{ 0x7cc5, 0x7ccb, PDF_CMAP_TABLE, 5654 },
	{ 0x7ccc, 0x7ccd, PDF_CMAP_RANGE, 7382 },
	{ 0x7cce, 0x7cd4, PDF_CMAP_RANGE, 17086 },
	{ 0x7cd5, 0x7cd9, PDF_CMAP_TABLE, 5661 },
	{ 0x7cda, 0x7cdb, PDF_CMAP_RANGE, 17094 },
	{ 0x7cdc, 0x7ce0, PDF_CMAP_TABLE, 5666 },
	{ 0x7ce1, 0x7ce6, PDF_CMAP_RANGE, 17096 },
	{ 0x7ce7, 0x7ce8, PDF_CMAP_TABLE, 5671 },
	{ 0x7ce9, 0x7cee, PDF_CMAP_RANGE, 17102 },
	{ 0x7cef, 0x7cfb, PDF_CMAP_TABLE, 5673 },
	{ 0x7cfc, 0x7cfd, PDF_CMAP_RANGE, 17113 },
	{ 0x7cfe, 0x7d06, PDF_CMAP_TABLE, 5686 },
	{ 0x7d07, 0x7d08, PDF_CMAP_RANGE, 9246 },
	{ 0x7d09, 0x7d0d, PDF_CMAP_TABLE, 5695 },
	{ 0x7d0e, 0x7d0f, PDF_CMAP_RANGE, 17119 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 8359 },
	{ 0x7d11, 0x7d12, PDF_CMAP_RANGE, 17121 },
	{ 0x7d13, 0x7d1c, PDF_CMAP_TABLE, 5700 },
	{ 0x7d1d, 0x7d1f, PDF_CMAP_RANGE, 17125 },
	{ 0x7d20, 0x7d22, PDF_CMAP_TABLE, 5710 },
	{ 0x7d23, 0x7d26, PDF_CMAP_RANGE, 17128 },
	{ 0x7d27, 0x7d27, PDF_CMAP_SINGLE, 2245 },
	{ 0x7d28, 0x7d2a, PDF_CMAP_RANGE, 17132 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 4652 },
	{ 0x7d2c, 0x7d2e, PDF_CMAP_RANGE, 17135 },
	{ 0x7d2f, 0x7d33, PDF_CMAP_TABLE, 5713 },
	{ 0x7d34, 0x7d38, PDF_CMAP_RANGE, 17138 },
	{ 0x7d39, 0x7d3c, PDF_CMAP_TABLE, 5718 },
	{ 0x7d3d, 0x7d3e, PDF_CMAP_RANGE, 17144 },
	{ 0x7d3f, 0x7d46, PDF_CMAP_TABLE, 5722 },
	{ 0x7d47, 0x7d4d, PDF_CMAP_RANGE, 17149 },
	{ 0x7d4e, 0x7d50, PDF_CMAP_TABLE, 5730 },
	{ 0x7d51, 0x7d5c, PDF_CMAP_RANGE, 17157 },
	{ 0x7d5d, 0x7d5e, PDF_CMAP_TABLE, 5733 },
	{ 0x7d5f, 0x7d60, PDF_CMAP_RANGE, 17169 },
	{ 0x7d61, 0x7d62, PDF_CMAP_TABLE, 5735 },
	{ 0x7d63, 0x7d65, PDF_CMAP_RANGE, 17171 },
	{ 0x7d66, 0x7d68, PDF_CMAP_TABLE, 5737 },
	{ 0x7d69, 0x7d6d, PDF_CMAP_RANGE, 17175 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 4032 },
	{ 0x7d6f, 0x7d70, PDF_CMAP_RANGE, 17180 },
	{ 0x7d71, 0x7d73, PDF_CMAP_TABLE, 5740 },
	{ 0x7d74, 0x7d75, PDF_CMAP_RANGE, 17182 },
	{ 0x7d76, 0x7d79, PDF_CMAP_TABLE, 5743 },
	{ 0x7d7a, 0x7d80, PDF_CMAP_RANGE, 17185 },
	{ 0x7d81, 0x7d83, PDF_CMAP_TABLE, 5747 },
	{ 0x7d84, 0x7d85, PDF_CMAP_RANGE, 17193 },
	{ 0x7d86, 0x7d89, PDF_CMAP_TABLE, 5750 },
	{ 0x7d8a, 0x7d8e, PDF_CMAP_RANGE, 17196 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 8536 },
	{ 0x7d90, 0x7d92, PDF_CMAP_RANGE, 17201 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 8130 },
	{ 0x7d94, 0x7d9b, PDF_CMAP_RANGE, 17204 },
	{ 0x7d9c, 0x7d9e, PDF_CMAP_TABLE, 5754 },
	{ 0x7d9f, 0x7da1, PDF_CMAP_RANGE, 17213 },
	{ 0x7da2, 0x7da3, PDF_CMAP_TABLE, 5757 },
	{ 0x7da4, 0x7da5, PDF_CMAP_RANGE, 17216 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 7401 },
	{ 0x7da7, 0x7daa, PDF_CMAP_RANGE, 17218 },
	{ 0x7dab, 0x7db4, PDF_CMAP_TABLE, 5759 },
	{ 0x7db5, 0x7db7, PDF_CMAP_RANGE, 17223 },
	{ 0x7db8, 0x7dbf, PDF_CMAP_TABLE, 5769 },
	{ 0x7dc0, 0x7dc3, PDF_CMAP_RANGE, 17227 },
	{ 0x7dc4, 0x7dc4, PDF_CMAP_SINGLE, 9269 },
	{ 0x7dc5, 0x7dc6, PDF_CMAP_RANGE, 17231 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 9275 },
	{ 0x7dc8, 0x7dc9, PDF_CMAP_RANGE, 17233 },
	{ 0x7dca, 0x7dcb, PDF_CMAP_TABLE, 5777 },
	{ 0x7dcc, 0x7dd0, PDF_CMAP_RANGE, 17235 },
	{ 0x7dd1, 0x7dd4, PDF_CMAP_TABLE, 5779 },
	{ 0x7dd5, 0x7dd6, PDF_CMAP_RANGE, 17241 },
	{ 0x7dd7, 0x7dd9, PDF_CMAP_TABLE, 5783 },
	{ 0x7dda, 0x7ddc, PDF_CMAP_RANGE, 17243 },
	{ 0x7ddd, 0x7de3, PDF_CMAP_TABLE, 5786 },
	{ 0x7de4, 0x7de5, PDF_CMAP_RANGE, 17248 },
	{ 0x7de6, 0x7de9, PDF_CMAP_TABLE, 5793 },
	{ 0x7dea, 0x7deb, PDF_CMAP_RANGE, 17251 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 8326 },
	{ 0x7ded, 0x7dee, PDF_CMAP_RANGE, 17253 },
	{ 0x7def, 0x7df6, PDF_CMAP_TABLE, 5797 },
	{ 0x7df7, 0x7df8, PDF_CMAP_RANGE, 17258 },
	{ 0x7df9, 0x7dfb, PDF_CMAP_TABLE, 5805 },
	{ 0x7dfc, 0x7e07, PDF_CMAP_RANGE, 17261 },
	{ 0x7e08, 0x7e0b, PDF_CMAP_TABLE, 5808 },
	{ 0x7e0c, 0x7e0f, PDF_CMAP_RANGE, 17273 },
	{ 0x7e10, 0x7e11, PDF_CMAP_TABLE, 5812 },
	{ 0x7e12, 0x7e1a, PDF_CMAP_RANGE, 17277 },
	{ 0x7e1b, 0x7e1f, PDF_CMAP_TABLE, 5814 },
	{ 0x7e20, 0x7e22, PDF_CMAP_RANGE, 17287 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 8642 },
	{ 0x7e24, 0x7e26, PDF_CMAP_RANGE, 17290 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 8558 },
	{ 0x7e28, 0x7e2a, PDF_CMAP_RANGE, 17293 },
	{ 0x7e2b, 0x7e2e, PDF_CMAP_TABLE, 5819 },
	{ 0x7e2f, 0x7e30, PDF_CMAP_RANGE, 17297 },
	{ 0x7e31, 0x7e3e, PDF_CMAP_TABLE, 5823 },
	{ 0x7e3f, 0x7e40, PDF_CMAP_RANGE, 17303 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 1614 },
	{ 0x7e42, 0x7e44, PDF_CMAP_RANGE, 17305 },
	{ 0x7e45, 0x7e47, PDF_CMAP_TABLE, 5837 },
	{ 0x7e48, 0x7e51, PDF_CMAP_RANGE, 17308 },
	{ 0x7e52, 0x7e55, PDF_CMAP_TABLE, 5840 },
	{ 0x7e56, 0x7e59, PDF_CMAP_RANGE, 17319 },
	{ 0x7e5a, 0x7e5a, PDF_CMAP_SINGLE, 9300 },
	{ 0x7e5b, 0x7e5d, PDF_CMAP_RANGE, 17323 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 8443 },
	{ 0x7e5f, 0x7e61, PDF_CMAP_RANGE, 17326 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 9280 },
	{ 0x7e63, 0x7e68, PDF_CMAP_RANGE, 17329 },
	{ 0x7e69, 0x7e70, PDF_CMAP_TABLE, 5844 },
	{ 0x7e71, 0x7e72, PDF_CMAP_RANGE, 17336 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 8110 },
	{ 0x7e74, 0x7e78, PDF_CMAP_RANGE, 17338 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 8730 },
	{ 0x7e7a, 0x7e7b, PDF_CMAP_RANGE, 17343 },
	{ 0x7e7c, 0x7e7e, PDF_CMAP_TABLE, 5852 },
	{ 0x7e7f, 0x7e81, PDF_CMAP_RANGE, 17345 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 4680 },
	{ 0x7e83, 0x7e87, PDF_CMAP_RANGE, 17348 },
	{ 0x7e88, 0x7e8f, PDF_CMAP_TABLE, 5855 },
	{ 0x7e90, 0x7e92, PDF_CMAP_RANGE, 17356 },
	{ 0x7e93, 0x7e98, PDF_CMAP_TABLE, 5863 },
	{ 0x7e99, 0x7e9a, PDF_CMAP_RANGE, 17361 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_TABLE, 5869 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 17363 },
	{ 0x7e9f, 0x7ea7, PDF_CMAP_TABLE, 5871 },
	{ 0x7ea8, 0x7ea9, PDF_CMAP_RANGE, 6103 },
	{ 0x7eaa, 0x7eba, PDF_CMAP_TABLE, 5880 },
	{ 0x7ebb, 0x7ebc, PDF_CMAP_RANGE, 17367 },
	{ 0x7ebd, 0x7ebf, PDF_CMAP_TABLE, 5897 },
	{ 0x7ec0, 0x7ec2, PDF_CMAP_RANGE, 6108 },
	{ 0x7ec3, 0x7eca, PDF_CMAP_TABLE, 5900 },
	{ 0x7ecb, 0x7ecc, PDF_CMAP_RANGE, 6112 },
	{ 0x7ecd, 0x7edf, PDF_CMAP_TABLE, 5908 },
	{ 0x7ee0, 0x7ee1, PDF_CMAP_RANGE, 6118 },
	{ 0x7ee2, 0x7eed, PDF_CMAP_TABLE, 5927 },
	{ 0x7eee, 0x7eef, PDF_CMAP_RANGE, 6122 },
	{ 0x7ef0, 0x7ef0, PDF_CMAP_SINGLE, 1349 },
	{ 0x7ef1, 0x7ef2, PDF_CMAP_RANGE, 6124 },
	{ 0x7ef3, 0x7ef9, PDF_CMAP_TABLE, 5939 },
	{ 0x7efa, 0x7efb, PDF_CMAP_RANGE, 6128 },
	{ 0x7efc, 0x7f00, PDF_CMAP_TABLE, 5946 },
	{ 0x7f01, 0x7f03, PDF_CMAP_RANGE, 6131 },
	{ 0x7f04, 0x7f06, PDF_CMAP_TABLE, 5951 },
	{ 0x7f07, 0x7f08, PDF_CMAP_RANGE, 6134 },
	{ 0x7f09, 0x7f0a, PDF_CMAP_TABLE, 5954 },
	{ 0x7f0b, 0x7f0c, PDF_CMAP_RANGE, 6136 },
	{ 0x7f0d, 0x7f10, PDF_CMAP_TABLE, 5956 },
	{ 0x7f11, 0x7f12, PDF_CMAP_RANGE, 6139 },
	{ 0x7f13, 0x7f20, PDF_CMAP_TABLE, 5960 },
	{ 0x7f21, 0x7f27, PDF_CMAP_RANGE, 6146 },
	{ 0x7f28, 0x7f29, PDF_CMAP_TABLE, 5974 },
	{ 0x7f2a, 0x7f2d, PDF_CMAP_RANGE, 6153 },
	{ 0x7f2e, 0x7f2e, PDF_CMAP_SINGLE, 3330 },
	{ 0x7f2f, 0x7f33, PDF_CMAP_RANGE, 6157 },
	{ 0x7f34, 0x7f3a, PDF_CMAP_TABLE, 5976 },
	{ 0x7f3b, 0x7f41, PDF_CMAP_RANGE, 17378 },
	{ 0x7f42, 0x7f43, PDF_CMAP_TABLE, 5983 },
	{ 0x7f44, 0x7f45, PDF_CMAP_RANGE, 7264 },
	{ 0x7f46, 0x7f4b, PDF_CMAP_RANGE, 17386 },
	{ 0x7f4c, 0x7f51, PDF_CMAP_TABLE, 5985 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 17394 },
	{ 0x7f54, 0x7f5a, PDF_CMAP_TABLE, 5991 },
	{ 0x7f5b, 0x7f5e, PDF_CMAP_RANGE, 17398 },
	{ 0x7f5f, 0x7f62, PDF_CMAP_TABLE, 5998 },
	{ 0x7f63, 0x7f67, PDF_CMAP_RANGE, 17403 },
	{ 0x7f68, 0x7f6a, PDF_CMAP_TABLE, 6002 },
	{ 0x7f6b, 0x7f6d, PDF_CMAP_RANGE, 17408 },
	{ 0x7f6e, 0x7f74, PDF_CMAP_TABLE, 6005 },
	{ 0x7f75, 0x7f76, PDF_CMAP_RANGE, 17413 },
	{ 0x7f77, 0x7f79, PDF_CMAP_TABLE, 6012 },
	{ 0x7f7a, 0x7f7d, PDF_CMAP_RANGE, 17416 },
	{ 0x7f7e, 0x7f7e, PDF_CMAP_SINGLE, 6794 },
	{ 0x7f7f, 0x7f80, PDF_CMAP_RANGE, 17420 },
	{ 0x7f81, 0x7f81, PDF_CMAP_SINGLE, 6793 },
	{ 0x7f82, 0x7f84, PDF_CMAP_RANGE, 17422 },
	{ 0x7f85, 0x7f8e, PDF_CMAP_TABLE, 6015 },
	{ 0x7f8f, 0x7f93, PDF_CMAP_RANGE, 17429 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 1756 },
	{ 0x7f95, 0x7f99, PDF_CMAP_RANGE, 17434 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 2608 },
	{ 0x7f9b, 0x7f9c, PDF_CMAP_RANGE, 17439 },
	{ 0x7f9d, 0x7fa1, PDF_CMAP_TABLE, 6025 },
	{ 0x7fa2, 0x7fa3, PDF_CMAP_RANGE, 17442 },
	{ 0x7fa4, 0x7fa9, PDF_CMAP_TABLE, 6030 },
	{ 0x7faa, 0x7fae, PDF_CMAP_RANGE, 17446 },
	{ 0x7faf, 0x7fb0, PDF_CMAP_RANGE, 7367 },
	{ 0x7fb1, 0x7fb2, PDF_CMAP_TABLE, 6036 },
	{ 0x7fb3, 0x7fb7, PDF_CMAP_RANGE, 17452 },
	{ 0x7fb8, 0x7fb9, PDF_CMAP_TABLE, 6038 },
	{ 0x7fba, 0x7fbb, PDF_CMAP_RANGE, 17457 },
	{ 0x7fbc, 0x7fc1, PDF_CMAP_TABLE, 6040 },
	{ 0x7fc2, 0x7fc4, PDF_CMAP_RANGE, 17461 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 1286 },
	{ 0x7fc6, 0x7fc9, PDF_CMAP_RANGE, 17464 },
	{ 0x7fca, 0x7fce, PDF_CMAP_TABLE, 6046 },
	{ 0x7fcf, 0x7fd1, PDF_CMAP_RANGE, 17470 },
	{ 0x7fd2, 0x7fd5, PDF_CMAP_TABLE, 6051 },
	{ 0x7fd6, 0x7fd7, PDF_CMAP_RANGE, 17474 },
	{ 0x7fd8, 0x7fd8, PDF_CMAP_SINGLE, 3145 },
	{ 0x7fd9, 0x7fde, PDF_CMAP_RANGE, 17476 },
	{ 0x7fdf, 0x7fe1, PDF_CMAP_TABLE, 6055 },
	{ 0x7fe2, 0x7fe4, PDF_CMAP_RANGE, 17482 },
	{ 0x7fe5, 0x7fe6, PDF_CMAP_TABLE, 6058 },
	{ 0x7fe7, 0x7fe8, PDF_CMAP_RANGE, 17485 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 7396 },
	{ 0x7fea, 0x7fed, PDF_CMAP_RANGE, 17487 },
	{ 0x7fee, 0x7ff3, PDF_CMAP_TABLE, 6060 },
	{ 0x7ff4, 0x7ff8, PDF_CMAP_RANGE, 17493 },
	{ 0x7ff9, 0x7ffc, PDF_CMAP_TABLE, 6066 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 17499 },
	{ 0x8000, 0x8006, PDF_CMAP_TABLE, 6070 },
	{ 0x8007, 0x800a, PDF_CMAP_RANGE, 17503 },
	{ 0x800b, 0x800d, PDF_CMAP_TABLE, 6077 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 17507 },
	{ 0x8010, 0x8019, PDF_CMAP_TABLE, 6080 },
	{ 0x801a, 0x801b, PDF_CMAP_RANGE, 17511 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 7118 },
	{ 0x801d, 0x801f, PDF_CMAP_RANGE, 17513 },
	{ 0x8020, 0x8022, PDF_CMAP_TABLE, 6090 },
	{ 0x8023, 0x8024, PDF_CMAP_RANGE, 17517 },
	{ 0x8025, 0x8027, PDF_CMAP_RANGE, 7121 },
	{ 0x8028, 0x802e, PDF_CMAP_TABLE, 6093 },
	{ 0x802f, 0x8030, PDF_CMAP_RANGE, 17521 },
	{ 0x8031, 0x8038, PDF_CMAP_TABLE, 6100 },
	{ 0x8039, 0x803a, PDF_CMAP_RANGE, 17525 },
	{ 0x803b, 0x803f, PDF_CMAP_TABLE, 6108 },
	{ 0x8040, 0x8041, PDF_CMAP_RANGE, 17529 },
	{ 0x8042, 0x8043, PDF_CMAP_TABLE, 6113 },
	{ 0x8044, 0x8045, PDF_CMAP_RANGE, 17531 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 7130 },
	{ 0x8047, 0x8049, PDF_CMAP_RANGE, 17533 },
	{ 0x804a, 0x804d, PDF_CMAP_TABLE, 6115 },
	{ 0x804e, 0x8051, PDF_CMAP_RANGE, 17536 },
	{ 0x8052, 0x805a, PDF_CMAP_TABLE, 6119 },
	{ 0x805b, 0x805d, PDF_CMAP_RANGE, 17544 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 8601 },
	{ 0x805f, 0x8068, PDF_CMAP_RANGE, 17547 },
	{ 0x8069, 0x806a, PDF_CMAP_TABLE, 6128 },
	{ 0x806b, 0x806e, PDF_CMAP_RANGE, 17557 },
	{ 0x806f, 0x8079, PDF_CMAP_TABLE, 6130 },
	{ 0x807a, 0x807c, PDF_CMAP_RANGE, 17563 },
	{ 0x807d, 0x8080, PDF_CMAP_TABLE, 6141 },
	{ 0x8081, 0x8082, PDF_CMAP_RANGE, 17566 },
	{ 0x8083, 0x808c, PDF_CMAP_TABLE, 6145 },
	{ 0x808d, 0x8092, PDF_CMAP_RANGE, 17570 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6513 },
	{ 0x8094, 0x8095, PDF_CMAP_RANGE, 17576 },
	{ 0x8096, 0x80a5, PDF_CMAP_TABLE, 6155 },
	{ 0x80a6, 0x80a8, PDF_CMAP_RANGE, 17582 },
	{ 0x80a9, 0x80b4, PDF_CMAP_TABLE, 6171 },
	{ 0x80b5, 0x80b6, PDF_CMAP_RANGE, 17588 },
	{ 0x80b7, 0x80b7, PDF_CMAP_SINGLE, 6521 },
	{ 0x80b8, 0x80b9, PDF_CMAP_RANGE, 17590 },
	{ 0x80ba, 0x80c6, PDF_CMAP_TABLE, 6183 },
	{ 0x80c7, 0x80cb, PDF_CMAP_RANGE, 17594 },
	{ 0x80cc, 0x80ce, PDF_CMAP_TABLE, 6196 },
	{ 0x80cf, 0x80d5, PDF_CMAP_RANGE, 17599 },
	{ 0x80d6, 0x80de, PDF_CMAP_TABLE, 6199 },
	{ 0x80df, 0x80e0, PDF_CMAP_RANGE, 17607 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1969 },
	{ 0x80e2, 0x80e3, PDF_CMAP_RANGE, 17609 },
	{ 0x80e4, 0x80e6, PDF_CMAP_TABLE, 6208 },
	{ 0x80e7, 0x80ea, PDF_CMAP_RANGE, 6522 },
	{ 0x80eb, 0x80fd, PDF_CMAP_TABLE, 6211 },
	{ 0x80fe, 0x8101, PDF_CMAP_RANGE, 17617 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 4523 },
	{ 0x8103, 0x8104, PDF_CMAP_RANGE, 17621 },
	{ 0x8105, 0x8106, PDF_CMAP_TABLE, 6230 },
	{ 0x8107, 0x8108, PDF_CMAP_RANGE, 17623 },
	{ 0x8109, 0x810a, PDF_CMAP_TABLE, 6232 },
	{ 0x810b, 0x810c, PDF_CMAP_RANGE, 17625 },
	{ 0x810d, 0x810e, PDF_CMAP_RANGE, 6538 },
	{ 0x810f, 0x811b, PDF_CMAP_TABLE, 6234 },
	{ 0x811c, 0x811d, PDF_CMAP_RANGE, 17630 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 6546 },
	{ 0x811f, 0x812b, PDF_CMAP_RANGE, 17632 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 6547 },
	{ 0x812d, 0x812e, PDF_CMAP_RANGE, 17645 },
	{ 0x812f, 0x8132, PDF_CMAP_TABLE, 6247 },
	{ 0x8133, 0x8135, PDF_CMAP_RANGE, 17648 },
	{ 0x8136, 0x8139, PDF_CMAP_TABLE, 6251 },
	{ 0x813a, 0x813d, PDF_CMAP_RANGE, 17652 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 3009 },
	{ 0x813f, 0x8145, PDF_CMAP_RANGE, 17656 },
	{ 0x8146, 0x8156, PDF_CMAP_TABLE, 6255 },
	{ 0x8157, 0x8158, PDF_CMAP_RANGE, 17668 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 6554 },
	{ 0x815b, 0x815f, PDF_CMAP_RANGE, 17670 },
	{ 0x8160, 0x8161, PDF_CMAP_TABLE, 6272 },
	{ 0x8162, 0x8164, PDF_CMAP_RANGE, 17675 },
	{ 0x8165, 0x8171, PDF_CMAP_TABLE, 6274 },
	{ 0x8172, 0x8173, PDF_CMAP_RANGE, 17682 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6553 },
	{ 0x8175, 0x8177, PDF_CMAP_RANGE, 17684 },
	{ 0x8178, 0x817b, PDF_CMAP_TABLE, 6287 },
	{ 0x817c, 0x817d, PDF_CMAP_RANGE, 6559 },
	{ 0x817e, 0x8182, PDF_CMAP_TABLE, 6291 },
	{ 0x8183, 0x8187, PDF_CMAP_RANGE, 17688 },
	{ 0x8188, 0x818a, PDF_CMAP_TABLE, 6296 },
	{ 0x818b, 0x818e, PDF_CMAP_RANGE, 17694 },
	{ 0x818f, 0x8191, PDF_CMAP_TABLE, 6299 },
	{ 0x8192, 0x8197, PDF_CMAP_RANGE, 17699 },
	{ 0x8198, 0x819d, PDF_CMAP_TABLE, 6302 },
	{ 0x819e, 0x819f, PDF_CMAP_RANGE, 17706 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 8101 },
	{ 0x81a1, 0x81a2, PDF_CMAP_RANGE, 17708 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 6569 },
	{ 0x81a4, 0x81a5, PDF_CMAP_RANGE, 17710 },
	{ 0x81a6, 0x81aa, PDF_CMAP_TABLE, 6308 },
	{ 0x81ab, 0x81b2, PDF_CMAP_RANGE, 17713 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 3326 },
	{ 0x81b4, 0x81b9, PDF_CMAP_RANGE, 17721 },
	{ 0x81ba, 0x81c3, PDF_CMAP_TABLE, 6313 },
	{ 0x81c4, 0x81c5, PDF_CMAP_RANGE, 17728 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 4195 },
	{ 0x81c7, 0x81c8, PDF_CMAP_RANGE, 17730 },
	{ 0x81c9, 0x81cf, PDF_CMAP_TABLE, 6323 },
	{ 0x81d0, 0x81d7, PDF_CMAP_RANGE, 17734 },
	{ 0x81d8, 0x81da, PDF_CMAP_TABLE, 6330 },
	{ 0x81db, 0x81de, PDF_CMAP_RANGE, 17743 },
	{ 0x81df, 0x81e0, PDF_CMAP_TABLE, 6333 },
	{ 0x81e1, 0x81e2, PDF_CMAP_RANGE, 17747 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 1249 },
	{ 0x81e4, 0x81e6, PDF_CMAP_RANGE, 17749 },
	{ 0x81e7, 0x81ed, PDF_CMAP_TABLE, 6335 },
	{ 0x81ee, 0x81f2, PDF_CMAP_RANGE, 17754 },
	{ 0x81f3, 0x81f4, PDF_CMAP_RANGE, 4544 },
	{ 0x81f5, 0x81f9, PDF_CMAP_RANGE, 17759 },
	{ 0x81fa, 0x8200, PDF_CMAP_TABLE, 6342 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 7334 },
	{ 0x8203, 0x820b, PDF_CMAP_TABLE, 6349 },
	{ 0x820c, 0x820d, PDF_CMAP_RANGE, 3353 },
	{ 0x820e, 0x820f, PDF_CMAP_RANGE, 17768 },
	{ 0x8210, 0x8214, PDF_CMAP_TABLE, 6358 },
	{ 0x8215, 0x821a, PDF_CMAP_RANGE, 17772 },
	{ 0x821b, 0x8220, PDF_CMAP_TABLE, 6363 },
	{ 0x8221, 0x8223, PDF_CMAP_RANGE, 7339 },
	{ 0x8224, 0x8227, PDF_CMAP_RANGE, 17780 },
	{ 0x8228, 0x8232, PDF_CMAP_TABLE, 6369 },
	{ 0x8233, 0x8234, PDF_CMAP_RANGE, 7348 },
	{ 0x8235, 0x823b, PDF_CMAP_TABLE, 6380 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 17788 },
	{ 0x823e, 0x823e, PDF_CMAP_SINGLE, 7350 },
	{ 0x823f, 0x8243, PDF_CMAP_RANGE, 17790 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 7351 },
	{ 0x8245, 0x8246, PDF_CMAP_RANGE, 17795 },
	{ 0x8247, 0x824b, PDF_CMAP_TABLE, 6387 },
	{ 0x824c, 0x824e, PDF_CMAP_RANGE, 17799 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 7354 },
	{ 0x8250, 0x8257, PDF_CMAP_RANGE, 17802 },
	{ 0x8258, 0x825a, PDF_CMAP_TABLE, 6392 },
	{ 0x825b, 0x825e, PDF_CMAP_RANGE, 17810 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 7356 },
	{ 0x8260, 0x8263, PDF_CMAP_RANGE, 17814 },
	{ 0x8264, 0x8268, PDF_CMAP_TABLE, 6395 },
	{ 0x8269, 0x826a, PDF_CMAP_RANGE, 17820 },
	{ 0x826b, 0x826b, PDF_CMAP_SINGLE, 9712 },
	{ 0x826c, 0x826d, PDF_CMAP_RANGE, 17822 },
	{ 0x826e, 0x8274, PDF_CMAP_TABLE, 6400 },
	{ 0x8275, 0x8276, PDF_CMAP_RANGE, 17824 },
	{ 0x8277, 0x827a, PDF_CMAP_TABLE, 6407 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 17827 },
	{ 0x827d, 0x827f, PDF_CMAP_TABLE, 6411 },
	{ 0x8280, 0x8281, PDF_CMAP_RANGE, 17829 },
	{ 0x8282, 0x8284, PDF_CMAP_TABLE, 6414 },
	{ 0x8285, 0x8287, PDF_CMAP_RANGE, 17832 },
	{ 0x8288, 0x8292, PDF_CMAP_TABLE, 6417 },
	{ 0x8293, 0x8296, PDF_CMAP_RANGE, 17838 },
	{ 0x8297, 0x8299, PDF_CMAP_TABLE, 6428 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 17842 },
	{ 0x829c, 0x82a1, PDF_CMAP_TABLE, 6431 },
	{ 0x82a2, 0x82a3, PDF_CMAP_RANGE, 17846 },
	{ 0x82a4, 0x82b4, PDF_CMAP_TABLE, 6437 },
	{ 0x82b5, 0x82b6, PDF_CMAP_RANGE, 17850 },
	{ 0x82b7, 0x82be, PDF_CMAP_TABLE, 6454 },
	{ 0x82bf, 0x82c0, PDF_CMAP_RANGE, 17854 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 5112 },
	{ 0x82c2, 0x82c3, PDF_CMAP_RANGE, 17856 },
	{ 0x82c4, 0x82c4, PDF_CMAP_SINGLE, 5118 },
	{ 0x82c5, 0x82c6, PDF_CMAP_RANGE, 17858 },
	{ 0x82c7, 0x82ca, PDF_CMAP_TABLE, 6462 },
	{ 0x82cb, 0x82cc, PDF_CMAP_RANGE, 5110 },
	{ 0x82cd, 0x82d8, PDF_CMAP_TABLE, 6466 },
	{ 0x82d9, 0x82da, PDF_CMAP_RANGE, 17863 },
	{ 0x82db, 0x82e7, PDF_CMAP_TABLE, 6478 },
	{ 0x82e8, 0x82ea, PDF_CMAP_RANGE, 17867 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3316 },
	{ 0x82ec, 0x82ee, PDF_CMAP_RANGE, 17870 },
	{ 0x82ef, 0x82f1, PDF_CMAP_TABLE, 6491 },
	{ 0x82f2, 0x82f3, PDF_CMAP_RANGE, 17874 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 5128 },
	{ 0x82f5, 0x82f6, PDF_CMAP_RANGE, 17876 },
	{ 0x82f7, 0x82fb, PDF_CMAP_TABLE, 6494 },
	{ 0x82fc, 0x8300, PDF_CMAP_RANGE, 17880 },
	{ 0x8301, 0x8309, PDF_CMAP_TABLE, 6499 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 17885 },
	{ 0x830c, 0x8311, PDF_CMAP_TABLE, 6508 },
	{ 0x8312, 0x8313, PDF_CMAP_RANGE, 17889 },
	{ 0x8314, 0x8315, PDF_CMAP_RANGE, 5137 },
	{ 0x8316, 0x8317, PDF_CMAP_TABLE, 6514 },
	{ 0x8318, 0x8319, PDF_CMAP_RANGE, 17892 },
	{ 0x831a, 0x831c, PDF_CMAP_TABLE, 6516 },
	{ 0x831d, 0x8326, PDF_CMAP_RANGE, 17894 },
	{ 0x8327, 0x8328, PDF_CMAP_TABLE, 6519 },
	{ 0x8329, 0x832a, PDF_CMAP_RANGE, 17904 },
	{ 0x832b, 0x833c, PDF_CMAP_TABLE, 6521 },
	{ 0x833d, 0x833f, PDF_CMAP_RANGE, 17911 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 5157 },
	{ 0x8341, 0x8342, PDF_CMAP_RANGE, 17914 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 5155 },
	{ 0x8344, 0x8345, PDF_CMAP_RANGE, 17916 },
	{ 0x8346, 0x8349, PDF_CMAP_TABLE, 6539 },
	{ 0x834a, 0x834e, PDF_CMAP_RANGE, 17919 },
	{ 0x834f, 0x8354, PDF_CMAP_TABLE, 6543 },
	{ 0x8355, 0x8359, PDF_CMAP_RANGE, 17925 },
	{ 0x835a, 0x835a, PDF_CMAP_SINGLE, 2119 },
	{ 0x835b, 0x835c, PDF_CMAP_RANGE, 5143 },
	{ 0x835d, 0x836c, PDF_CMAP_TABLE, 6549 },
	{ 0x836d, 0x836e, PDF_CMAP_RANGE, 5170 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 4144 },
	{ 0x8370, 0x8376, PDF_CMAP_RANGE, 17932 },
	{ 0x8377, 0x8378, PDF_CMAP_TABLE, 6565 },
	{ 0x8379, 0x837a, PDF_CMAP_RANGE, 17939 },
	{ 0x837b, 0x837d, PDF_CMAP_TABLE, 6567 },
	{ 0x837e, 0x8384, PDF_CMAP_RANGE, 17941 },
	{ 0x8385, 0x8386, PDF_CMAP_TABLE, 6570 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 17948 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 6572 },
	{ 0x838b, 0x838d, PDF_CMAP_RANGE, 17950 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 3304 },
	{ 0x838f, 0x8391, PDF_CMAP_RANGE, 17953 },
	{ 0x8392, 0x8393, PDF_CMAP_TABLE, 6574 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 17956 },
	{ 0x8396, 0x8398, PDF_CMAP_TABLE, 6576 },
	{ 0x8399, 0x839a, PDF_CMAP_RANGE, 17959 },
	{ 0x839b, 0x83a2, PDF_CMAP_TABLE, 6579 },
	{ 0x83a3, 0x83a6, PDF_CMAP_RANGE, 17964 },
	{ 0x83a7, 0x83ab, PDF_CMAP_TABLE, 6587 },
	{ 0x83ac, 0x83af, PDF_CMAP_RANGE, 17968 },
	{ 0x83b0, 0x83b2, PDF_CMAP_TABLE, 6592 },
	{ 0x83b3, 0x83b4, PDF_CMAP_RANGE, 5174 },
	{ 0x83b5, 0x83bd, PDF_CMAP_TABLE, 6595 },
	{ 0x83be, 0x83bf, PDF_CMAP_RANGE, 17974 },
	{ 0x83c0, 0x83c1, PDF_CMAP_TABLE, 6604 },
	{ 0x83c2, 0x83c4, PDF_CMAP_RANGE, 17976 },
	{ 0x83c5, 0x83c7, PDF_CMAP_TABLE, 6606 },
	{ 0x83c8, 0x83c9, PDF_CMAP_RANGE, 17980 },
	{ 0x83ca, 0x83cc, PDF_CMAP_TABLE, 6609 },
	{ 0x83cd, 0x83ce, PDF_CMAP_RANGE, 17983 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 1921 },
	{ 0x83d0, 0x83d3, PDF_CMAP_RANGE, 17985 },
	{ 0x83d4, 0x83d8, PDF_CMAP_TABLE, 6612 },
	{ 0x83d9, 0x83db, PDF_CMAP_RANGE, 17991 },
	{ 0x83dc, 0x83e1, PDF_CMAP_TABLE, 6617 },
	{ 0x83e2, 0x83e4, PDF_CMAP_RANGE, 17995 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 5194 },
	{ 0x83e6, 0x83e8, PDF_CMAP_RANGE, 17998 },
	{ 0x83e9, 0x83ea, PDF_CMAP_TABLE, 6623 },
	{ 0x83eb, 0x83ee, PDF_CMAP_RANGE, 18001 },
	{ 0x83ef, 0x83f2, PDF_CMAP_TABLE, 6625 },
	{ 0x83f3, 0x83f7, PDF_CMAP_RANGE, 18005 },
	{ 0x83f8, 0x83f9, PDF_CMAP_RANGE, 5210 },
	{ 0x83fa, 0x83fc, PDF_CMAP_RANGE, 18010 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 5200 },
	{ 0x83fe, 0x8400, PDF_CMAP_RANGE, 18013 },
	{ 0x8401, 0x8407, PDF_CMAP_TABLE, 6629 },
	{ 0x8408, 0x8409, PDF_CMAP_RANGE, 18018 },
	{ 0x840a, 0x8411, PDF_CMAP_TABLE, 6636 },
	{ 0x8412, 0x8417, PDF_CMAP_RANGE, 18021 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 5197 },
	{ 0x8419, 0x841b, PDF_CMAP_RANGE, 18027 },
	{ 0x841c, 0x841d, PDF_CMAP_TABLE, 6644 },
	{ 0x841e, 0x8423, PDF_CMAP_RANGE, 18030 },
	{ 0x8424, 0x8425, PDF_CMAP_RANGE, 4238 },
	{ 0x8426, 0x8428, PDF_CMAP_TABLE, 6646 },
	{ 0x8429, 0x842b, PDF_CMAP_RANGE, 18036 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 8587 },
	{ 0x842d, 0x8430, PDF_CMAP_RANGE, 18039 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 5234 },
	{ 0x8432, 0x8434, PDF_CMAP_RANGE, 18043 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 9033 },
	{ 0x8436, 0x8437, PDF_CMAP_RANGE, 18046 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 5203 },
	{ 0x8439, 0x843b, PDF_CMAP_RANGE, 18048 },
	{ 0x843c, 0x843d, PDF_CMAP_TABLE, 6649 },
	{ 0x843e, 0x8445, PDF_CMAP_RANGE, 18051 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 5229 },
	{ 0x8447, 0x8448, PDF_CMAP_RANGE, 18059 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 8715 },
	{ 0x844a, 0x8450, PDF_CMAP_RANGE, 18061 },
	{ 0x8451, 0x8452, PDF_CMAP_TABLE, 6651 },
	{ 0x8453, 0x8456, PDF_CMAP_RANGE, 18068 },
	{ 0x8457, 0x845c, PDF_CMAP_TABLE, 6653 },
	{ 0x845d, 0x8460, PDF_CMAP_RANGE, 18073 },
	{ 0x8461, 0x8466, PDF_CMAP_TABLE, 6659 },
	{ 0x8467, 0x8468, PDF_CMAP_RANGE, 18079 },
	{ 0x8469, 0x847a, PDF_CMAP_TABLE, 6665 },
	{ 0x847b, 0x8481, PDF_CMAP_RANGE, 18087 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 1466 },
	{ 0x8483, 0x8486, PDF_CMAP_RANGE, 18094 },
	{ 0x8487, 0x8488, PDF_CMAP_RANGE, 5223 },
	{ 0x8489, 0x848e, PDF_CMAP_TABLE, 6683 },
	{ 0x848f, 0x8492, PDF_CMAP_RANGE, 18100 },
	{ 0x8493, 0x8494, PDF_CMAP_TABLE, 6689 },
	{ 0x8495, 0x8496, PDF_CMAP_RANGE, 18104 },
	{ 0x8497, 0x8499, PDF_CMAP_TABLE, 6691 },
	{ 0x849a, 0x849b, PDF_CMAP_RANGE, 18107 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3543 },
	{ 0x849d, 0x84a0, PDF_CMAP_RANGE, 18109 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 5246 },
	{ 0x84a2, 0x84ae, PDF_CMAP_RANGE, 18113 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 4759 },
	{ 0x84b0, 0x84b1, PDF_CMAP_RANGE, 18126 },
	{ 0x84b2, 0x84b4, PDF_CMAP_TABLE, 6694 },
	{ 0x84b5, 0x84b7, PDF_CMAP_RANGE, 18129 },
	{ 0x84b8, 0x84c1, PDF_CMAP_TABLE, 6697 },
	{ 0x84c2, 0x84c3, PDF_CMAP_RANGE, 18134 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 4025 },
	{ 0x84c5, 0x84c8, PDF_CMAP_RANGE, 18136 },
	{ 0x84c9, 0x84cd, PDF_CMAP_TABLE, 6707 },
	{ 0x84ce, 0x84cf, PDF_CMAP_RANGE, 18141 },
	{ 0x84d0, 0x84d3, PDF_CMAP_TABLE, 6712 },
	{ 0x84d4, 0x84d5, PDF_CMAP_RANGE, 18144 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 1078 },
	{ 0x84d7, 0x84dc, PDF_CMAP_RANGE, 18146 },
	{ 0x84dd, 0x84e0, PDF_CMAP_TABLE, 6716 },
	{ 0x84e1, 0x84e2, PDF_CMAP_RANGE, 18153 },
	{ 0x84e3, 0x84e6, PDF_CMAP_TABLE, 6720 },
	{ 0x84e7, 0x84eb, PDF_CMAP_RANGE, 18156 },
	{ 0x84ec, 0x84f0, PDF_CMAP_TABLE, 6724 },
	{ 0x84f1, 0x84fb, PDF_CMAP_RANGE, 18162 },
	{ 0x84fc, 0x84ff, PDF_CMAP_TABLE, 6729 },
	{ 0x8500, 0x850b, PDF_CMAP_RANGE, 18174 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 5252 },
	{ 0x850d, 0x8510, PDF_CMAP_RANGE, 18186 },
	{ 0x8511, 0x8514, PDF_CMAP_TABLE, 6733 },
	{ 0x8515, 0x8516, PDF_CMAP_RANGE, 18191 },
	{ 0x8517, 0x8517, PDF_CMAP_SINGLE, 4482 },
	{ 0x8518, 0x8519, PDF_CMAP_RANGE, 18193 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 3780 },
	{ 0x851b, 0x851d, PDF_CMAP_RANGE, 18195 },
	{ 0x851e, 0x8523, PDF_CMAP_TABLE, 6737 },
	{ 0x8524, 0x8525, PDF_CMAP_RANGE, 18200 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 9016 },
	{ 0x8527, 0x852a, PDF_CMAP_RANGE, 18202 },
	{ 0x852b, 0x852d, PDF_CMAP_TABLE, 6743 },
	{ 0x852e, 0x8536, PDF_CMAP_RANGE, 18206 },
	{ 0x8537, 0x853d, PDF_CMAP_TABLE, 6746 },
	{ 0x853e, 0x8540, PDF_CMAP_RANGE, 18215 },
	{ 0x8541, 0x8543, PDF_CMAP_TABLE, 6753 },
	{ 0x8544, 0x8545, PDF_CMAP_RANGE, 18219 },
	{ 0x8546, 0x854a, PDF_CMAP_TABLE, 6756 },
	{ 0x854b, 0x854d, PDF_CMAP_RANGE, 18222 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 9021 },
	{ 0x854f, 0x8551, PDF_CMAP_RANGE, 18225 },
	{ 0x8552, 0x8559, PDF_CMAP_TABLE, 6761 },
	{ 0x855a, 0x855d, PDF_CMAP_RANGE, 18230 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 5267 },
	{ 0x855f, 0x8561, PDF_CMAP_RANGE, 18234 },
	{ 0x8562, 0x8564, PDF_CMAP_TABLE, 6769 },
	{ 0x8565, 0x8567, PDF_CMAP_RANGE, 18238 },
	{ 0x8568, 0x856a, PDF_CMAP_TABLE, 6772 },
	{ 0x856b, 0x856c, PDF_CMAP_RANGE, 18241 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 8651 },
	{ 0x856e, 0x8571, PDF_CMAP_RANGE, 18243 },
	{ 0x8572, 0x8574, PDF_CMAP_TABLE, 6775 },
	{ 0x8575, 0x8576, PDF_CMAP_RANGE, 18248 },
	{ 0x8577, 0x857b, PDF_CMAP_TABLE, 6778 },
	{ 0x857c, 0x857d, PDF_CMAP_RANGE, 18251 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 2500 },
	{ 0x857f, 0x8583, PDF_CMAP_RANGE, 18253 },
	{ 0x8584, 0x858c, PDF_CMAP_TABLE, 6783 },
	{ 0x858d, 0x858e, PDF_CMAP_RANGE, 18261 },
	{ 0x858f, 0x8591, PDF_CMAP_TABLE, 6792 },
	{ 0x8592, 0x8593, PDF_CMAP_RANGE, 18264 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 8410 },
	{ 0x8595, 0x859a, PDF_CMAP_RANGE, 18266 },
	{ 0x859b, 0x859c, PDF_CMAP_TABLE, 6795 },
	{ 0x859d, 0x859e, PDF_CMAP_RANGE, 18272 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 9034 },
	{ 0x85a0, 0x85a3, PDF_CMAP_RANGE, 18274 },
	{ 0x85a4, 0x85aa, PDF_CMAP_TABLE, 6797 },
	{ 0x85ab, 0x85ad, PDF_CMAP_RANGE, 18280 },
	{ 0x85ae, 0x85b0, PDF_CMAP_TABLE, 6804 },
	{ 0x85b1, 0x85b6, PDF_CMAP_RANGE, 18283 },
	{ 0x85b7, 0x85ba, PDF_CMAP_TABLE, 6807 },
	{ 0x85bb, 0x85c0, PDF_CMAP_RANGE, 18290 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 5285 },
	{ 0x85c2, 0x85c8, PDF_CMAP_RANGE, 18296 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 2230 },
	{ 0x85ca, 0x85cc, PDF_CMAP_RANGE, 18303 },
	{ 0x85cd, 0x85d0, PDF_CMAP_TABLE, 6811 },
	{ 0x85d1, 0x85d2, PDF_CMAP_RANGE, 18306 },
	{ 0x85d3, 0x85d5, PDF_CMAP_TABLE, 6815 },
	{ 0x85d6, 0x85db, PDF_CMAP_RANGE, 18309 },
	{ 0x85dc, 0x85dd, PDF_CMAP_TABLE, 6818 },
	{ 0x85de, 0x85e3, PDF_CMAP_RANGE, 18315 },
	{ 0x85e4, 0x85e5, PDF_CMAP_TABLE, 6820 },
	{ 0x85e6, 0x85e8, PDF_CMAP_RANGE, 18321 },
	{ 0x85e9, 0x85ea, PDF_CMAP_TABLE, 6822 },
	{ 0x85eb, 0x85f3, PDF_CMAP_RANGE, 18324 },
	{ 0x85f4, 0x85f6, PDF_CMAP_TABLE, 6824 },
	{ 0x85f7, 0x85f8, PDF_CMAP_RANGE, 18334 },
	{ 0x85f9, 0x85fb, PDF_CMAP_TABLE, 6827 },
	{ 0x85fc, 0x85fe, PDF_CMAP_RANGE, 18336 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 5287 },
	{ 0x8600, 0x8603, PDF_CMAP_RANGE, 18339 },
	{ 0x8604, 0x8607, PDF_CMAP_TABLE, 6830 },
	{ 0x8608, 0x860a, PDF_CMAP_RANGE, 18343 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 8379 },
	{ 0x860c, 0x8610, PDF_CMAP_RANGE, 18346 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 2821 },
	{ 0x8612, 0x8615, PDF_CMAP_RANGE, 18351 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 5291 },
	{ 0x8617, 0x8619, PDF_CMAP_RANGE, 18355 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 9050 },
	{ 0x861b, 0x861d, PDF_CMAP_RANGE, 18358 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 9046 },
	{ 0x861f, 0x8621, PDF_CMAP_RANGE, 18361 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 9015 },
	{ 0x8623, 0x8626, PDF_CMAP_RANGE, 18364 },
	{ 0x8627, 0x8629, PDF_CMAP_TABLE, 6834 },
	{ 0x862a, 0x862c, PDF_CMAP_RANGE, 18369 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 8185 },
	{ 0x862e, 0x8637, PDF_CMAP_RANGE, 18372 },
	{ 0x8638, 0x863c, PDF_CMAP_TABLE, 6837 },
	{ 0x863d, 0x863e, PDF_CMAP_RANGE, 18384 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 8289 },
	{ 0x8640, 0x864c, PDF_CMAP_RANGE, 18386 },
	{ 0x864d, 0x8651, PDF_CMAP_TABLE, 6842 },
	{ 0x8652, 0x8653, PDF_CMAP_RANGE, 18399 },
	{ 0x8654, 0x8655, PDF_CMAP_TABLE, 6847 },
	{ 0x8656, 0x8659, PDF_CMAP_RANGE, 18401 },
	{ 0x865a, 0x865f, PDF_CMAP_TABLE, 6849 },
	{ 0x8660, 0x8661, PDF_CMAP_RANGE, 18407 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 6509 },
	{ 0x8663, 0x8666, PDF_CMAP_RANGE, 18409 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 8168 },
	{ 0x8668, 0x866a, PDF_CMAP_RANGE, 18413 },
	{ 0x866b, 0x866e, PDF_CMAP_TABLE, 6855 },
	{ 0x866f, 0x8670, PDF_CMAP_RANGE, 18417 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 3396 },
	{ 0x8672, 0x8678, PDF_CMAP_RANGE, 18419 },
	{ 0x8679, 0x8682, PDF_CMAP_TABLE, 6859 },
	{ 0x8683, 0x8689, PDF_CMAP_RANGE, 18426 },
	{ 0x868a, 0x868d, PDF_CMAP_TABLE, 6869 },
	{ 0x868e, 0x8692, PDF_CMAP_RANGE, 18433 },
	{ 0x8693, 0x8695, PDF_CMAP_TABLE, 6873 },
	{ 0x8696, 0x869b, PDF_CMAP_RANGE, 18439 },
	{ 0x869c, 0x869d, PDF_CMAP_TABLE, 6876 },
	{ 0x869e, 0x86a2, PDF_CMAP_RANGE, 18445 },
	{ 0x86a3, 0x86a4, PDF_CMAP_TABLE, 6878 },
	{ 0x86a5, 0x86a6, PDF_CMAP_RANGE, 18450 },
	{ 0x86a7, 0x86ac, PDF_CMAP_TABLE, 6880 },
	{ 0x86ad, 0x86ae, PDF_CMAP_RANGE, 18453 },
	{ 0x86af, 0x86b1, PDF_CMAP_TABLE, 6886 },
	{ 0x86b2, 0x86b3, PDF_CMAP_RANGE, 18455 },
	{ 0x86b4, 0x86b6, PDF_CMAP_TABLE, 6889 },
	{ 0x86b7, 0x86b9, PDF_CMAP_RANGE, 18457 },
	{ 0x86ba, 0x86ba, PDF_CMAP_SINGLE, 7175 },
	{ 0x86bb, 0x86bf, PDF_CMAP_RANGE, 18460 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 4602 },
	{ 0x86c1, 0x86c3, PDF_CMAP_RANGE, 18465 },
	{ 0x86c4, 0x86cb, PDF_CMAP_TABLE, 6892 },
	{ 0x86cc, 0x86cd, PDF_CMAP_RANGE, 18470 },
	{ 0x86ce, 0x86d1, PDF_CMAP_TABLE, 6900 },
	{ 0x86d2, 0x86d3, PDF_CMAP_RANGE, 18472 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 2028 },
	{ 0x86d5, 0x86d7, PDF_CMAP_RANGE, 18474 },
	{ 0x86d8, 0x86db, PDF_CMAP_TABLE, 6904 },
	{ 0x86dc, 0x86dd, PDF_CMAP_RANGE, 18478 },
	{ 0x86de, 0x86df, PDF_CMAP_TABLE, 6908 },
	{ 0x86e0, 0x86e3, PDF_CMAP_RANGE, 18480 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 1773 },
	{ 0x86e5, 0x86e8, PDF_CMAP_RANGE, 18484 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 7181 },
	{ 0x86ea, 0x86ec, PDF_CMAP_RANGE, 18488 },
	{ 0x86ed, 0x86f0, PDF_CMAP_TABLE, 6910 },
	{ 0x86f1, 0x86f2, PDF_CMAP_RANGE, 7182 },
	{ 0x86f3, 0x86f4, PDF_CMAP_TABLE, 6914 },
	{ 0x86f5, 0x86f7, PDF_CMAP_RANGE, 18492 },
	{ 0x86f8, 0x86fa, PDF_CMAP_TABLE, 6916 },
	{ 0x86fb, 0x86fd, PDF_CMAP_RANGE, 18495 },
	{ 0x86fe, 0x8703, PDF_CMAP_TABLE, 6919 },
	{ 0x8704, 0x8705, PDF_CMAP_RANGE, 18500 },
	{ 0x8706, 0x870a, PDF_CMAP_TABLE, 6925 },
	{ 0x870b, 0x870c, PDF_CMAP_RANGE, 18502 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 7198 },
	{ 0x870e, 0x8711, PDF_CMAP_RANGE, 18504 },
	{ 0x8712, 0x871e, PDF_CMAP_TABLE, 6930 },
	{ 0x871f, 0x8720, PDF_CMAP_RANGE, 18513 },
	{ 0x8721, 0x8725, PDF_CMAP_TABLE, 6943 },
	{ 0x8726, 0x8728, PDF_CMAP_RANGE, 18516 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 7210 },
	{ 0x872a, 0x872d, PDF_CMAP_RANGE, 18519 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 7204 },
	{ 0x872f, 0x8730, PDF_CMAP_RANGE, 18523 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 7209 },
	{ 0x8732, 0x8733, PDF_CMAP_RANGE, 18525 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 7208 },
	{ 0x8735, 0x8736, PDF_CMAP_RANGE, 18527 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 7211 },
	{ 0x8738, 0x873a, PDF_CMAP_RANGE, 18529 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 7201 },
	{ 0x873c, 0x873d, PDF_CMAP_RANGE, 18532 },
	{ 0x873e, 0x873f, PDF_CMAP_TABLE, 6948 },
	{ 0x8740, 0x8746, PDF_CMAP_RANGE, 18534 },
	{ 0x8747, 0x8749, PDF_CMAP_TABLE, 6950 },
	{ 0x874a, 0x874b, PDF_CMAP_RANGE, 18541 },
	{ 0x874c, 0x874e, PDF_CMAP_TABLE, 6953 },
	{ 0x874f, 0x8752, PDF_CMAP_RANGE, 18544 },
	{ 0x8753, 0x8759, PDF_CMAP_TABLE, 6956 },
	{ 0x875a, 0x875f, PDF_CMAP_RANGE, 18551 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 7218 },
	{ 0x8761, 0x8762, PDF_CMAP_RANGE, 18557 },
	{ 0x8763, 0x8766, PDF_CMAP_TABLE, 6963 },
	{ 0x8767, 0x876d, PDF_CMAP_RANGE, 18559 },
	{ 0x876e, 0x8770, PDF_CMAP_TABLE, 6967 },
	{ 0x8771, 0x8773, PDF_CMAP_RANGE, 18567 },
	{ 0x8774, 0x8778, PDF_CMAP_TABLE, 6970 },
	{ 0x8779, 0x877a, PDF_CMAP_RANGE, 18572 },
	{ 0x877b, 0x877c, PDF_CMAP_TABLE, 6975 },
	{ 0x877d, 0x877e, PDF_CMAP_RANGE, 7215 },
	{ 0x877f, 0x8781, PDF_CMAP_RANGE, 18574 },
	{ 0x8782, 0x8785, PDF_CMAP_TABLE, 6977 },
	{ 0x8786, 0x8787, PDF_CMAP_RANGE, 18577 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 7234 },
	{ 0x8789, 0x878a, PDF_CMAP_RANGE, 18579 },
	{ 0x878b, 0x878d, PDF_CMAP_TABLE, 6981 },
	{ 0x878e, 0x8792, PDF_CMAP_RANGE, 18582 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 7229 },
	{ 0x8794, 0x8796, PDF_CMAP_RANGE, 18587 },
	{ 0x8797, 0x8797, PDF_CMAP_SINGLE, 7237 },
	{ 0x8798, 0x879d, PDF_CMAP_RANGE, 18590 },
	{ 0x879e, 0x879f, PDF_CMAP_TABLE, 6984 },
	{ 0x87a0, 0x87a1, PDF_CMAP_RANGE, 18596 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 8742 },
	{ 0x87a3, 0x87a7, PDF_CMAP_RANGE, 18598 },
	{ 0x87a8, 0x87a8, PDF_CMAP_SINGLE, 7231 },
	{ 0x87a9, 0x87aa, PDF_CMAP_RANGE, 18603 },
	{ 0x87ab, 0x87af, PDF_CMAP_TABLE, 6986 },
	{ 0x87b0, 0x87b2, PDF_CMAP_RANGE, 18606 },
	{ 0x87b3, 0x87b5, PDF_CMAP_TABLE, 6991 },
	{ 0x87b6, 0x87b9, PDF_CMAP_RANGE, 18610 },
	{ 0x87ba, 0x87bd, PDF_CMAP_TABLE, 6994 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 18615 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 7248 },
	{ 0x87c1, 0x87c3, PDF_CMAP_RANGE, 18617 },
	{ 0x87c4, 0x87cb, PDF_CMAP_TABLE, 6998 },
	{ 0x87cc, 0x87cd, PDF_CMAP_RANGE, 18623 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 9697 },
	{ 0x87cf, 0x87d0, PDF_CMAP_RANGE, 18625 },
	{ 0x87d1, 0x87d3, PDF_CMAP_TABLE, 7006 },
	{ 0x87d4, 0x87da, PDF_CMAP_RANGE, 18627 },
	{ 0x87db, 0x87db, PDF_CMAP_SINGLE, 7250 },
	{ 0x87dc, 0x87df, PDF_CMAP_RANGE, 18634 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 7252 },
	{ 0x87e1, 0x87e2, PDF_CMAP_RANGE, 18638 },
	{ 0x87e3, 0x87e5, PDF_CMAP_TABLE, 7009 },
	{ 0x87e6, 0x87e9, PDF_CMAP_RANGE, 18641 },
	{ 0x87ea, 0x87ef, PDF_CMAP_TABLE, 7012 },
	{ 0x87f0, 0x87f1, PDF_CMAP_RANGE, 18647 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 7817 },
	{ 0x87f3, 0x87f5, PDF_CMAP_RANGE, 18649 },
	{ 0x87f6, 0x87f6, PDF_CMAP_SINGLE, 9689 },
	{ 0x87f7, 0x87f8, PDF_CMAP_RANGE, 18652 },
	{ 0x87f9, 0x87fb, PDF_CMAP_TABLE, 7018 },
	{ 0x87fc, 0x87fd, PDF_CMAP_RANGE, 18655 },
	{ 0x87fe, 0x87ff, PDF_CMAP_TABLE, 7021 },
	{ 0x8800, 0x8802, PDF_CMAP_RANGE, 18658 },
	{ 0x8803, 0x8806, PDF_CMAP_TABLE, 7023 },
	{ 0x8807, 0x8809, PDF_CMAP_RANGE, 18662 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 7257 },
	{ 0x880b, 0x880f, PDF_CMAP_RANGE, 18665 },
	{ 0x8810, 0x8816, PDF_CMAP_TABLE, 7027 },
	{ 0x8817, 0x881a, PDF_CMAP_RANGE, 18672 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 7258 },
	{ 0x881c, 0x881e, PDF_CMAP_RANGE, 18676 },
	{ 0x881f, 0x8823, PDF_CMAP_TABLE, 7034 },
	{ 0x8824, 0x8830, PDF_CMAP_RANGE, 18680 },
	{ 0x8831, 0x8832, PDF_CMAP_TABLE, 7039 },
	{ 0x8833, 0x8835, PDF_CMAP_RANGE, 18693 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 7771 },
	{ 0x8837, 0x8838, PDF_CMAP_RANGE, 18696 },
	{ 0x8839, 0x883c, PDF_CMAP_TABLE, 7041 },
	{ 0x883d, 0x883f, PDF_CMAP_RANGE, 18699 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 4051 },
	{ 0x8841, 0x8843, PDF_CMAP_RANGE, 18702 },
	{ 0x8844, 0x8846, PDF_CMAP_TABLE, 7045 },
	{ 0x8847, 0x8849, PDF_CMAP_RANGE, 18705 },
	{ 0x884a, 0x884d, PDF_CMAP_TABLE, 7048 },
	{ 0x884e, 0x8852, PDF_CMAP_RANGE, 18709 },
	{ 0x8853, 0x8854, PDF_CMAP_TABLE, 7052 },
	{ 0x8855, 0x8856, PDF_CMAP_RANGE, 18714 },
	{ 0x8857, 0x885d, PDF_CMAP_TABLE, 7054 },
	{ 0x885e, 0x8860, PDF_CMAP_RANGE, 18719 },
	{ 0x8861, 0x8865, PDF_CMAP_TABLE, 7061 },
	{ 0x8866, 0x8867, PDF_CMAP_RANGE, 18722 },
	{ 0x8868, 0x8872, PDF_CMAP_TABLE, 7066 },
	{ 0x8873, 0x8876, PDF_CMAP_RANGE, 18728 },
	{ 0x8877, 0x8879, PDF_CMAP_TABLE, 7077 },
	{ 0x887a, 0x887c, PDF_CMAP_RANGE, 18733 },
	{ 0x887d, 0x8885, PDF_CMAP_TABLE, 7080 },
	{ 0x8886, 0x8887, PDF_CMAP_RANGE, 18738 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 7360 },
	{ 0x8889, 0x888a, PDF_CMAP_RANGE, 18740 },
	{ 0x888b, 0x888d, PDF_CMAP_TABLE, 7089 },
	{ 0x888e, 0x8891, PDF_CMAP_RANGE, 18743 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 3598 },
	{ 0x8893, 0x8895, PDF_CMAP_RANGE, 18747 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 4015 },
	{ 0x8897, 0x889b, PDF_CMAP_RANGE, 18750 },
	{ 0x889c, 0x889c, PDF_CMAP_SINGLE, 3729 },
	{ 0x889d, 0x88a1, PDF_CMAP_RANGE, 18755 },
	{ 0x88a2, 0x88a4, PDF_CMAP_TABLE, 7092 },
	{ 0x88a5, 0x88aa, PDF_CMAP_RANGE, 18761 },
	{ 0x88ab, 0x88ad, PDF_CMAP_TABLE, 7095 },
	{ 0x88ae, 0x88b0, PDF_CMAP_RANGE, 18768 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 1695 },
	{ 0x88b2, 0x88b6, PDF_CMAP_RANGE, 18771 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 7086 },
	{ 0x88b8, 0x88bb, PDF_CMAP_RANGE, 18776 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 7087 },
	{ 0x88bd, 0x88c0, PDF_CMAP_RANGE, 18780 },
	{ 0x88c1, 0x88c2, PDF_CMAP_TABLE, 7098 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 18784 },
	{ 0x88c5, 0x88c6, PDF_CMAP_TABLE, 7100 },
	{ 0x88c7, 0x88c8, PDF_CMAP_RANGE, 18786 },
	{ 0x88c9, 0x88ca, PDF_CMAP_TABLE, 7102 },
	{ 0x88cb, 0x88cd, PDF_CMAP_RANGE, 18788 },
	{ 0x88ce, 0x88cf, PDF_CMAP_TABLE, 7104 },
	{ 0x88d0, 0x88d1, PDF_CMAP_RANGE, 18791 },
	{ 0x88d2, 0x88d5, PDF_CMAP_TABLE, 7106 },
	{ 0x88d6, 0x88d7, PDF_CMAP_RANGE, 18794 },
	{ 0x88d8, 0x88d9, PDF_CMAP_TABLE, 7110 },
	{ 0x88da, 0x88db, PDF_CMAP_RANGE, 18796 },
	{ 0x88dc, 0x88df, PDF_CMAP_TABLE, 7112 },
	{ 0x88e0, 0x88e1, PDF_CMAP_RANGE, 18799 },
	{ 0x88e2, 0x88e5, PDF_CMAP_TABLE, 7116 },
	{ 0x88e6, 0x88e7, PDF_CMAP_RANGE, 18801 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 7096 },
	{ 0x88e9, 0x88ef, PDF_CMAP_RANGE, 18803 },
	{ 0x88f0, 0x88f4, PDF_CMAP_TABLE, 7120 },
	{ 0x88f5, 0x88f7, PDF_CMAP_RANGE, 18811 },
	{ 0x88f8, 0x88f9, PDF_CMAP_TABLE, 7125 },
	{ 0x88fa, 0x88fb, PDF_CMAP_RANGE, 18814 },
	{ 0x88fc, 0x88ff, PDF_CMAP_TABLE, 7127 },
	{ 0x8900, 0x8901, PDF_CMAP_RANGE, 18817 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 1836 },
	{ 0x8903, 0x8906, PDF_CMAP_RANGE, 18819 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 9846 },
	{ 0x8908, 0x8909, PDF_CMAP_RANGE, 18823 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 7103 },
	{ 0x890b, 0x890f, PDF_CMAP_RANGE, 18825 },
	{ 0x8910, 0x8913, PDF_CMAP_TABLE, 7131 },
	{ 0x8914, 0x8918, PDF_CMAP_RANGE, 18831 },
	{ 0x8919, 0x891b, PDF_CMAP_TABLE, 7135 },
	{ 0x891c, 0x8920, PDF_CMAP_RANGE, 18836 },
	{ 0x8921, 0x8921, PDF_CMAP_SINGLE, 7099 },
	{ 0x8922, 0x8924, PDF_CMAP_RANGE, 18841 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 3271 },
	{ 0x8926, 0x8929, PDF_CMAP_RANGE, 18844 },
	{ 0x892a, 0x892b, PDF_CMAP_TABLE, 7138 },
	{ 0x892c, 0x892f, PDF_CMAP_RANGE, 18848 },
	{ 0x8930, 0x8938, PDF_CMAP_TABLE, 7140 },
	{ 0x8939, 0x893a, PDF_CMAP_RANGE, 18855 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 8925 },
	{ 0x893c, 0x8940, PDF_CMAP_RANGE, 18857 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 7107 },
	{ 0x8942, 0x8943, PDF_CMAP_RANGE, 18862 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 3922 },
	{ 0x8945, 0x8946, PDF_CMAP_RANGE, 18864 },
	{ 0x8947, 0x8947, PDF_CMAP_SINGLE, 9661 },
	{ 0x8948, 0x8955, PDF_CMAP_RANGE, 18866 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 7722 },
	{ 0x8957, 0x895c, PDF_CMAP_RANGE, 18880 },
	{ 0x895d, 0x8960, PDF_CMAP_TABLE, 7149 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 18886 },
	{ 0x8964, 0x8966, PDF_CMAP_TABLE, 7153 },
	{ 0x8967, 0x8969, PDF_CMAP_RANGE, 18890 },
	{ 0x896a, 0x896c, PDF_CMAP_TABLE, 7156 },
	{ 0x896d, 0x896e, PDF_CMAP_RANGE, 18894 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 7807 },
	{ 0x8970, 0x8971, PDF_CMAP_RANGE, 18896 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 8621 },
	{ 0x8973, 0x897a, PDF_CMAP_RANGE, 18898 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 7109 },
	{ 0x897c, 0x897e, PDF_CMAP_RANGE, 18906 },
	{ 0x897f, 0x8983, PDF_CMAP_TABLE, 7159 },
	{ 0x8984, 0x8985, PDF_CMAP_RANGE, 18911 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 1709 },
	{ 0x8987, 0x898a, PDF_CMAP_RANGE, 18913 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8086 },
	{ 0x898c, 0x898e, PDF_CMAP_RANGE, 18917 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 7986 },
	{ 0x8990, 0x8992, PDF_CMAP_RANGE, 18920 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 8324 },
	{ 0x8994, 0x8995, PDF_CMAP_RANGE, 18923 },
	{ 0x8996, 0x8998, PDF_CMAP_TABLE, 7164 },
	{ 0x8999, 0x89a0, PDF_CMAP_RANGE, 18926 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 9402 },
	{ 0x89a2, 0x89a5, PDF_CMAP_RANGE, 18934 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 9404 },
	{ 0x89a7, 0x89a9, PDF_CMAP_RANGE, 18938 },
	{ 0x89aa, 0x89ac, PDF_CMAP_TABLE, 7167 },
	{ 0x89ad, 0x89ae, PDF_CMAP_RANGE, 18942 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 9405 },
	{ 0x89b0, 0x89b1, PDF_CMAP_RANGE, 18944 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 9406 },
	{ 0x89b3, 0x89b6, PDF_CMAP_RANGE, 18946 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 9407 },
	{ 0x89b8, 0x89b9, PDF_CMAP_RANGE, 18950 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8146 },
	{ 0x89bb, 0x89bc, PDF_CMAP_RANGE, 18952 },
	{ 0x89bd, 0x89c9, PDF_CMAP_TABLE, 7170 },
	{ 0x89ca, 0x89cc, PDF_CMAP_RANGE, 6459 },
	{ 0x89cd, 0x89cd, PDF_CMAP_SINGLE, 18956 },
	{ 0x89ce, 0x89d1, PDF_CMAP_RANGE, 6462 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 2200 },
	{ 0x89d3, 0x89d5, PDF_CMAP_RANGE, 18957 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 7511 },
	{ 0x89d7, 0x89d9, PDF_CMAP_RANGE, 18960 },
	{ 0x89da, 0x89de, PDF_CMAP_TABLE, 7183 },
	{ 0x89df, 0x89e2, PDF_CMAP_RANGE, 18965 },
	{ 0x89e3, 0x89e6, PDF_CMAP_TABLE, 7188 },
	{ 0x89e7, 0x89ea, PDF_CMAP_RANGE, 18970 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 7516 },
	{ 0x89ec, 0x89ee, PDF_CMAP_RANGE, 18974 },
	{ 0x89ef, 0x89ef, PDF_CMAP_SINGLE, 7517 },
	{ 0x89f0, 0x89f2, PDF_CMAP_RANGE, 18977 },
	{ 0x89f3, 0x89f8, PDF_CMAP_TABLE, 7192 },
	{ 0x89f9, 0x89ff, PDF_CMAP_RANGE, 18982 },
	{ 0x8a00, 0x8a03, PDF_CMAP_TABLE, 7198 },
	{ 0x8a04, 0x8a06, PDF_CMAP_RANGE, 18989 },
	{ 0x8a07, 0x8a10, PDF_CMAP_TABLE, 7202 },
	{ 0x8a11, 0x8a12, PDF_CMAP_RANGE, 18996 },
	{ 0x8a13, 0x8a18, PDF_CMAP_TABLE, 7212 },
	{ 0x8a19, 0x8a1a, PDF_CMAP_RANGE, 19000 },
	{ 0x8a1b, 0x8a1f, PDF_CMAP_TABLE, 7218 },
	{ 0x8a20, 0x8a22, PDF_CMAP_RANGE, 19004 },
	{ 0x8a23, 0x8a25, PDF_CMAP_TABLE, 7223 },
	{ 0x8a26, 0x8a29, PDF_CMAP_RANGE, 19008 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 7922 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 19012 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 8479 },
	{ 0x8a2e, 0x8a30, PDF_CMAP_RANGE, 19014 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 8669 },
	{ 0x8a32, 0x8a33, PDF_CMAP_RANGE, 19017 },
	{ 0x8a34, 0x8a36, PDF_CMAP_TABLE, 7226 },
	{ 0x8a37, 0x8a39, PDF_CMAP_RANGE, 19020 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 8834 },
	{ 0x8a3b, 0x8a3d, PDF_CMAP_RANGE, 19023 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 7518 },
	{ 0x8a3f, 0x8a40, PDF_CMAP_RANGE, 19026 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 8934 },
	{ 0x8a42, 0x8a45, PDF_CMAP_RANGE, 19028 },
	{ 0x8a46, 0x8a48, PDF_CMAP_TABLE, 7229 },
	{ 0x8a49, 0x8a4d, PDF_CMAP_RANGE, 19033 },
	{ 0x8a4e, 0x8a55, PDF_CMAP_TABLE, 7232 },
	{ 0x8a56, 0x8a57, PDF_CMAP_RANGE, 19041 },
	{ 0x8a58, 0x8a58, PDF_CMAP_SINGLE, 8938 },
	{ 0x8a59, 0x8a5a, PDF_CMAP_RANGE, 19043 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 8893 },
	{ 0x8a5c, 0x8a5d, PDF_CMAP_RANGE, 19045 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 7838 },
	{ 0x8a5f, 0x8a60, PDF_CMAP_RANGE, 19047 },
	{ 0x8a61, 0x8a63, PDF_CMAP_TABLE, 7240 },
	{ 0x8a64, 0x8a65, PDF_CMAP_RANGE, 19049 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 8503 },
	{ 0x8a67, 0x8a68, PDF_CMAP_RANGE, 19051 },
	{ 0x8a69, 0x8a75, PDF_CMAP_TABLE, 7243 },
	{ 0x8a76, 0x8a78, PDF_CMAP_RANGE, 19056 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 4435 },
	{ 0x8a7a, 0x8a7b, PDF_CMAP_RANGE, 19059 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 8944 },
	{ 0x8a7d, 0x8a7e, PDF_CMAP_RANGE, 19061 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 8942 },
	{ 0x8a80, 0x8a83, PDF_CMAP_RANGE, 19063 },
	{ 0x8a84, 0x8a8a, PDF_CMAP_TABLE, 7256 },
	{ 0x8a8b, 0x8a8c, PDF_CMAP_RANGE, 19068 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 8446 },
	{ 0x8a8e, 0x8a90, PDF_CMAP_RANGE, 19070 },
	{ 0x8a91, 0x8a92, PDF_CMAP_RANGE, 8953 },
	{ 0x8a93, 0x8a95, PDF_CMAP_TABLE, 7263 },
	{ 0x8a96, 0x8a97, PDF_CMAP_RANGE, 19074 },
	{ 0x8a98, 0x8a9a, PDF_CMAP_TABLE, 7266 },
	{ 0x8a9b, 0x8a9d, PDF_CMAP_RANGE, 19077 },
	{ 0x8a9e, 0x8aa8, PDF_CMAP_TABLE, 7269 },
	{ 0x8aa9, 0x8aab, PDF_CMAP_RANGE, 19083 },
	{ 0x8aac, 0x8aac, PDF_CMAP_SINGLE, 8519 },
	{ 0x8aad, 0x8aaf, PDF_CMAP_RANGE, 19086 },
	{ 0x8ab0, 0x8ab2, PDF_CMAP_TABLE, 7280 },
	{ 0x8ab3, 0x8ab5, PDF_CMAP_RANGE, 19090 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 8961 },
	{ 0x8ab7, 0x8ab8, PDF_CMAP_RANGE, 19093 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 7925 },
	{ 0x8aba, 0x8abb, PDF_CMAP_RANGE, 19095 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 8728 },
	{ 0x8abd, 0x8abe, PDF_CMAP_RANGE, 19097 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 7878 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_RANGE, 19099 },
	{ 0x8ac2, 0x8ac4, PDF_CMAP_TABLE, 7283 },
	{ 0x8ac5, 0x8ac6, PDF_CMAP_RANGE, 19102 },
	{ 0x8ac7, 0x8ad2, PDF_CMAP_TABLE, 7286 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 19109 },
	{ 0x8ad6, 0x8ad7, PDF_CMAP_TABLE, 7298 },
	{ 0x8ad8, 0x8ada, PDF_CMAP_RANGE, 19112 },
	{ 0x8adb, 0x8ade, PDF_CMAP_TABLE, 7300 },
	{ 0x8adf, 0x8ae1, PDF_CMAP_RANGE, 19116 },
	{ 0x8ae2, 0x8ae7, PDF_CMAP_TABLE, 7304 },
	{ 0x8ae8, 0x8aea, PDF_CMAP_RANGE, 19121 },
	{ 0x8aeb, 0x8aee, PDF_CMAP_TABLE, 7310 },
	{ 0x8aef, 0x8af0, PDF_CMAP_RANGE, 19125 },
	{ 0x8af1, 0x8af3, PDF_CMAP_TABLE, 7314 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 19128 },
	{ 0x8af6, 0x8b07, PDF_CMAP_TABLE, 7317 },
	{ 0x8b08, 0x8b09, PDF_CMAP_RANGE, 19136 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 8023 },
	{ 0x8b0b, 0x8b0d, PDF_CMAP_RANGE, 19138 },
	{ 0x8b0e, 0x8b10, PDF_CMAP_TABLE, 7335 },
	{ 0x8b11, 0x8b13, PDF_CMAP_RANGE, 19142 },
	{ 0x8b14, 0x8b1d, PDF_CMAP_TABLE, 7338 },
	{ 0x8b1e, 0x8b20, PDF_CMAP_RANGE, 19148 },
	{ 0x8b21, 0x8b21, PDF_CMAP_SINGLE, 8710 },
	{ 0x8b22, 0x8b25, PDF_CMAP_RANGE, 19151 },
	{ 0x8b26, 0x8b28, PDF_CMAP_TABLE, 7348 },
	{ 0x8b29, 0x8b2a, PDF_CMAP_RANGE, 19156 },
	{ 0x8b2b, 0x8b2d, PDF_CMAP_TABLE, 7351 },
	{ 0x8b2e, 0x8b32, PDF_CMAP_RANGE, 19158 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 8931 },
	{ 0x8b34, 0x8b38, PDF_CMAP_RANGE, 19163 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 8122 },
	{ 0x8b3a, 0x8b3d, PDF_CMAP_RANGE, 19168 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 8312 },
	{ 0x8b3f, 0x8b48, PDF_CMAP_RANGE, 19172 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 8840 },
	{ 0x8b4a, 0x8b4d, PDF_CMAP_RANGE, 19182 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_TABLE, 7354 },
	{ 0x8b50, 0x8b55, PDF_CMAP_RANGE, 19186 },
	{ 0x8b56, 0x8b5c, PDF_CMAP_TABLE, 7356 },
	{ 0x8b5d, 0x8b65, PDF_CMAP_RANGE, 19194 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 2270 },
	{ 0x8b67, 0x8b6a, PDF_CMAP_RANGE, 19203 },
	{ 0x8b6b, 0x8b6c, PDF_CMAP_TABLE, 7363 },
	{ 0x8b6d, 0x8b6e, PDF_CMAP_RANGE, 19207 },
	{ 0x8b6f, 0x8b70, PDF_CMAP_TABLE, 7365 },
	{ 0x8b71, 0x8b73, PDF_CMAP_RANGE, 19209 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 8405 },
	{ 0x8b75, 0x8b76, PDF_CMAP_RANGE, 19212 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 8010 },
	{ 0x8b78, 0x8b7c, PDF_CMAP_RANGE, 19214 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 8768 },
	{ 0x8b7e, 0x8b7f, PDF_CMAP_RANGE, 19219 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 7891 },
	{ 0x8b81, 0x8b89, PDF_CMAP_RANGE, 19221 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 7753 },
	{ 0x8b8b, 0x8b8d, PDF_CMAP_RANGE, 19230 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 9755 },
	{ 0x8b8f, 0x8b91, PDF_CMAP_RANGE, 19233 },
	{ 0x8b92, 0x8b96, PDF_CMAP_TABLE, 7367 },
	{ 0x8b97, 0x8b9b, PDF_CMAP_RANGE, 19237 },
	{ 0x8b9c, 0x8ba5, PDF_CMAP_TABLE, 7372 },
	{ 0x8ba6, 0x8ba7, PDF_CMAP_RANGE, 4888 },
	{ 0x8ba8, 0x8bb3, PDF_CMAP_TABLE, 7382 },
	{ 0x8bb4, 0x8bb5, PDF_CMAP_RANGE, 4891 },
	{ 0x8bb6, 0x8bc1, PDF_CMAP_TABLE, 7394 },
	{ 0x8bc2, 0x8bc3, PDF_CMAP_RANGE, 4894 },
	{ 0x8bc4, 0x8bd1, PDF_CMAP_TABLE, 7406 },
	{ 0x8bd2, 0x8bd4, PDF_CMAP_RANGE, 4899 },
	{ 0x8bd5, 0x8bd7, PDF_CMAP_TABLE, 7420 },
	{ 0x8bd8, 0x8bd9, PDF_CMAP_RANGE, 4903 },
	{ 0x8bda, 0x8bde, PDF_CMAP_TABLE, 7423 },
	{ 0x8bdf, 0x8be0, PDF_CMAP_RANGE, 4906 },
	{ 0x8be1, 0x8be7, PDF_CMAP_TABLE, 7428 },
	{ 0x8be8, 0x8be9, PDF_CMAP_RANGE, 4909 },
	{ 0x8bea, 0x8c10, PDF_CMAP_TABLE, 7435 },
	{ 0x8c11, 0x8c12, PDF_CMAP_RANGE, 4924 },
	{ 0x8c13, 0x8c13, PDF_CMAP_SINGLE, 3788 },
	{ 0x8c14, 0x8c16, PDF_CMAP_RANGE, 4926 },
	{ 0x8c17, 0x8c1e, PDF_CMAP_TABLE, 7474 },
	{ 0x8c1f, 0x8c21, PDF_CMAP_RANGE, 4933 },
	{ 0x8c22, 0x8c29, PDF_CMAP_TABLE, 7482 },
	{ 0x8c2a, 0x8c2b, PDF_CMAP_RANGE, 4938 },
	{ 0x8c2c, 0x8c2d, PDF_CMAP_TABLE, 7490 },
	{ 0x8c2e, 0x8c2f, PDF_CMAP_RANGE, 4940 },
	{ 0x8c30, 0x8c31, PDF_CMAP_TABLE, 7492 },
	{ 0x8c32, 0x8c33, PDF_CMAP_RANGE, 4942 },
	{ 0x8c34, 0x8c34, PDF_CMAP_SINGLE, 3121 },
	{ 0x8c35, 0x8c36, PDF_CMAP_RANGE, 4944 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1825 },
	{ 0x8c38, 0x8c40, PDF_CMAP_RANGE, 19252 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 2050 },
	{ 0x8c42, 0x8c45, PDF_CMAP_RANGE, 19261 },
	{ 0x8c46, 0x8c49, PDF_CMAP_TABLE, 7494 },
	{ 0x8c4a, 0x8c4b, PDF_CMAP_RANGE, 19265 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 3732 },
	{ 0x8c4d, 0x8c4f, PDF_CMAP_RANGE, 19267 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 7933 },
	{ 0x8c51, 0x8c54, PDF_CMAP_RANGE, 19270 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 7445 },
	{ 0x8c56, 0x8c59, PDF_CMAP_RANGE, 19274 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 6544 },
	{ 0x8c5b, 0x8c60, PDF_CMAP_RANGE, 19278 },
	{ 0x8c61, 0x8c62, PDF_CMAP_TABLE, 7498 },
	{ 0x8c63, 0x8c69, PDF_CMAP_RANGE, 19284 },
	{ 0x8c6a, 0x8c6b, PDF_CMAP_TABLE, 7500 },
	{ 0x8c6c, 0x8c72, PDF_CMAP_RANGE, 19291 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 5597 },
	{ 0x8c74, 0x8c77, PDF_CMAP_RANGE, 19298 },
	{ 0x8c78, 0x8c7a, PDF_CMAP_TABLE, 7502 },
	{ 0x8c7b, 0x8c81, PDF_CMAP_RANGE, 19302 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 7505 },
	{ 0x8c83, 0x8c84, PDF_CMAP_RANGE, 19309 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 7507 },
	{ 0x8c86, 0x8c88, PDF_CMAP_RANGE, 19311 },
	{ 0x8c89, 0x8c8c, PDF_CMAP_TABLE, 7505 },
	{ 0x8c8d, 0x8c93, PDF_CMAP_RANGE, 19315 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 7509 },
	{ 0x8c95, 0x8c97, PDF_CMAP_RANGE, 19322 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 7508 },
	{ 0x8c99, 0x8c9c, PDF_CMAP_RANGE, 19325 },
	{ 0x8c9d, 0x8ca2, PDF_CMAP_TABLE, 7509 },
	{ 0x8ca3, 0x8ca6, PDF_CMAP_RANGE, 19330 },
	{ 0x8ca7, 0x8cac, PDF_CMAP_TABLE, 7515 },
	{ 0x8cad, 0x8cae, PDF_CMAP_RANGE, 19334 },
	{ 0x8caf, 0x8cca, PDF_CMAP_TABLE, 7521 },
	{ 0x8ccb, 0x8cd0, PDF_CMAP_RANGE, 19342 },
	{ 0x8cd1, 0x8cd5, PDF_CMAP_TABLE, 7549 },
	{ 0x8cd6, 0x8cd9, PDF_CMAP_RANGE, 19349 },
	{ 0x8cda, 0x8ce7, PDF_CMAP_TABLE, 7554 },
	{ 0x8ce8, 0x8ce9, PDF_CMAP_RANGE, 19357 },
	{ 0x8cea, 0x8ced, PDF_CMAP_TABLE, 7568 },
	{ 0x8cee, 0x8cf3, PDF_CMAP_RANGE, 19359 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8179 },
	{ 0x8cf5, 0x8cf9, PDF_CMAP_RANGE, 19365 },
	{ 0x8cfa, 0x8cff, PDF_CMAP_TABLE, 7572 },
	{ 0x8d00, 0x8d03, PDF_CMAP_RANGE, 19371 },
	{ 0x8d04, 0x8d05, PDF_CMAP_TABLE, 7578 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 19375 },
	{ 0x8d08, 0x8d10, PDF_CMAP_TABLE, 7580 },
	{ 0x8d11, 0x8d15, PDF_CMAP_RANGE, 19380 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 8509 },
	{ 0x8d17, 0x8d1a, PDF_CMAP_RANGE, 19385 },
	{ 0x8d1b, 0x8d31, PDF_CMAP_TABLE, 7589 },
	{ 0x8d32, 0x8d33, PDF_CMAP_RANGE, 6444 },
	{ 0x8d34, 0x8d44, PDF_CMAP_TABLE, 7612 },
	{ 0x8d45, 0x8d46, PDF_CMAP_RANGE, 6450 },
	{ 0x8d47, 0x8d47, PDF_CMAP_SINGLE, 6454 },
	{ 0x8d48, 0x8d49, PDF_CMAP_RANGE, 6452 },
	{ 0x8d4a, 0x8d50, PDF_CMAP_TABLE, 7629 },
	{ 0x8d51, 0x8d52, PDF_CMAP_RANGE, 19390 },
	{ 0x8d53, 0x8d67, PDF_CMAP_TABLE, 7636 },
	{ 0x8d68, 0x8d6a, PDF_CMAP_RANGE, 19395 },
	{ 0x8d6b, 0x8d6d, PDF_CMAP_TABLE, 7657 },
	{ 0x8d6e, 0x8d6f, PDF_CMAP_RANGE, 19399 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 4668 },
	{ 0x8d71, 0x8d72, PDF_CMAP_RANGE, 19401 },
	{ 0x8d73, 0x8d77, PDF_CMAP_TABLE, 7660 },
	{ 0x8d78, 0x8d80, PDF_CMAP_RANGE, 19403 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 1256 },
	{ 0x8d82, 0x8d83, PDF_CMAP_RANGE, 19412 },
	{ 0x8d84, 0x8d85, PDF_CMAP_TABLE, 7665 },
	{ 0x8d86, 0x8d89, PDF_CMAP_RANGE, 19414 },
	{ 0x8d8a, 0x8d8b, PDF_CMAP_TABLE, 7667 },
	{ 0x8d8c, 0x8d90, PDF_CMAP_RANGE, 19418 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 7410 },
	{ 0x8d92, 0x8d93, PDF_CMAP_RANGE, 19423 },
	{ 0x8d94, 0x8d95, PDF_CMAP_TABLE, 7669 },
	{ 0x8d96, 0x8d98, PDF_CMAP_RANGE, 19425 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 8826 },
	{ 0x8d9a, 0x8d9e, PDF_CMAP_RANGE, 19428 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 3614 },
	{ 0x8da0, 0x8da2, PDF_CMAP_RANGE, 19433 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 3199 },
	{ 0x8da4, 0x8da7, PDF_CMAP_RANGE, 19436 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 8430 },
	{ 0x8da9, 0x8db0, PDF_CMAP_RANGE, 19440 },
	{ 0x8db1, 0x8db5, PDF_CMAP_TABLE, 7671 },
	{ 0x8db6, 0x8db7, PDF_CMAP_RANGE, 19448 },
	{ 0x8db8, 0x8dbf, PDF_CMAP_TABLE, 7676 },
	{ 0x8dc0, 0x8dc2, PDF_CMAP_RANGE, 19453 },
	{ 0x8dc3, 0x8dc6, PDF_CMAP_TABLE, 7684 },
	{ 0x8dc7, 0x8dca, PDF_CMAP_RANGE, 19457 },
	{ 0x8dcb, 0x8dcd, PDF_CMAP_TABLE, 7688 },
	{ 0x8dce, 0x8dcf, PDF_CMAP_RANGE, 7461 },
	{ 0x8dd0, 0x8dd1, PDF_CMAP_TABLE, 7691 },
	{ 0x8dd2, 0x8dd5, PDF_CMAP_RANGE, 19463 },
	{ 0x8dd6, 0x8dd7, PDF_CMAP_RANGE, 7457 },
	{ 0x8dd8, 0x8dd9, PDF_CMAP_RANGE, 19467 },
	{ 0x8dda, 0x8ddf, PDF_CMAP_TABLE, 7693 },
	{ 0x8de0, 0x8de2, PDF_CMAP_RANGE, 19470 },
	{ 0x8de3, 0x8de4, PDF_CMAP_TABLE, 7699 },
	{ 0x8de5, 0x8de7, PDF_CMAP_RANGE, 19473 },
	{ 0x8de8, 0x8dec, PDF_CMAP_TABLE, 7701 },
	{ 0x8ded, 0x8dee, PDF_CMAP_RANGE, 19477 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 2654 },
	{ 0x8df0, 0x8df2, PDF_CMAP_RANGE, 19479 },
	{ 0x8df3, 0x8df6, PDF_CMAP_TABLE, 7706 },
	{ 0x8df7, 0x8df8, PDF_CMAP_RANGE, 7466 },
	{ 0x8df9, 0x8dfd, PDF_CMAP_TABLE, 7710 },
	{ 0x8dfe, 0x8e04, PDF_CMAP_RANGE, 19485 },
	{ 0x8e05, 0x8e05, PDF_CMAP_SINGLE, 7449 },
	{ 0x8e06, 0x8e08, PDF_CMAP_RANGE, 19492 },
	{ 0x8e09, 0x8e0c, PDF_CMAP_TABLE, 7715 },
	{ 0x8e0d, 0x8e0e, PDF_CMAP_RANGE, 19496 },
	{ 0x8e0f, 0x8e10, PDF_CMAP_TABLE, 7719 },
	{ 0x8e11, 0x8e13, PDF_CMAP_RANGE, 19498 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 7474 },
	{ 0x8e15, 0x8e1c, PDF_CMAP_RANGE, 19501 },
	{ 0x8e1d, 0x8e1f, PDF_CMAP_TABLE, 7721 },
	{ 0x8e20, 0x8e21, PDF_CMAP_RANGE, 19509 },
	{ 0x8e22, 0x8e23, PDF_CMAP_TABLE, 7724 },
	{ 0x8e24, 0x8e28, PDF_CMAP_RANGE, 19511 },
	{ 0x8e29, 0x8e31, PDF_CMAP_TABLE, 7726 },
	{ 0x8e32, 0x8e33, PDF_CMAP_RANGE, 19519 },
	{ 0x8e34, 0x8e35, PDF_CMAP_TABLE, 7735 },
	{ 0x8e36, 0x8e38, PDF_CMAP_RANGE, 19521 },
	{ 0x8e39, 0x8e3a, PDF_CMAP_TABLE, 7737 },
	{ 0x8e3b, 0x8e3c, PDF_CMAP_RANGE, 19524 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 7485 },
	{ 0x8e3e, 0x8e3f, PDF_CMAP_RANGE, 19526 },
	{ 0x8e40, 0x8e40, PDF_CMAP_SINGLE, 7482 },
	{ 0x8e41, 0x8e42, PDF_CMAP_RANGE, 7488 },
	{ 0x8e43, 0x8e44, PDF_CMAP_TABLE, 7739 },
	{ 0x8e45, 0x8e46, PDF_CMAP_RANGE, 19529 },
	{ 0x8e47, 0x8e4c, PDF_CMAP_TABLE, 7741 },
	{ 0x8e4d, 0x8e50, PDF_CMAP_RANGE, 19531 },
	{ 0x8e51, 0x8e52, PDF_CMAP_RANGE, 7490 },
	{ 0x8e53, 0x8e54, PDF_CMAP_RANGE, 19535 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 9728 },
	{ 0x8e56, 0x8e58, PDF_CMAP_RANGE, 19537 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 7450 },
	{ 0x8e5a, 0x8e62, PDF_CMAP_RANGE, 19540 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 9734 },
	{ 0x8e64, 0x8e65, PDF_CMAP_RANGE, 19549 },
	{ 0x8e66, 0x8e66, PDF_CMAP_SINGLE, 1069 },
	{ 0x8e67, 0x8e68, PDF_CMAP_RANGE, 19551 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 7451 },
	{ 0x8e6a, 0x8e6b, PDF_CMAP_RANGE, 19553 },
	{ 0x8e6c, 0x8e76, PDF_CMAP_TABLE, 7747 },
	{ 0x8e77, 0x8e79, PDF_CMAP_RANGE, 19559 },
	{ 0x8e7a, 0x8e7c, PDF_CMAP_TABLE, 7758 },
	{ 0x8e7d, 0x8e7e, PDF_CMAP_RANGE, 19563 },
	{ 0x8e7f, 0x8e81, PDF_CMAP_TABLE, 7761 },
	{ 0x8e82, 0x8e84, PDF_CMAP_RANGE, 19566 },
	{ 0x8e85, 0x8e94, PDF_CMAP_TABLE, 7764 },
	{ 0x8e95, 0x8e99, PDF_CMAP_RANGE, 19573 },
	{ 0x8e9a, 0x8e9e, PDF_CMAP_TABLE, 7780 },
	{ 0x8e9f, 0x8ea0, PDF_CMAP_RANGE, 19580 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 9733 },
	{ 0x8ea2, 0x8ea4, PDF_CMAP_RANGE, 19582 },
	{ 0x8ea5, 0x8ea6, PDF_CMAP_TABLE, 7785 },
	{ 0x8ea7, 0x8ea9, PDF_CMAP_RANGE, 19585 },
	{ 0x8eaa, 0x8eac, PDF_CMAP_TABLE, 7787 },
	{ 0x8ead, 0x8eae, PDF_CMAP_RANGE, 19588 },
	{ 0x8eaf, 0x8eaf, PDF_CMAP_SINGLE, 3192 },
	{ 0x8eb0, 0x8eb1, PDF_CMAP_RANGE, 19590 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 1570 },
	{ 0x8eb3, 0x8eb9, PDF_CMAP_RANGE, 19592 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 3612 },
	{ 0x8ebb, 0x8ebf, PDF_CMAP_RANGE, 19599 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 8432 },
	{ 0x8ec1, 0x8ec9, PDF_CMAP_RANGE, 19604 },
	{ 0x8eca, 0x8ece, PDF_CMAP_TABLE, 7790 },
	{ 0x8ecf, 0x8ed1, PDF_CMAP_RANGE, 19613 },
	{ 0x8ed2, 0x8ed4, PDF_CMAP_TABLE, 7795 },
	{ 0x8ed5, 0x8eda, PDF_CMAP_RANGE, 19617 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 9357 },
	{ 0x8edc, 0x8ede, PDF_CMAP_RANGE, 19623 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 8450 },
	{ 0x8ee0, 0x8ee3, PDF_CMAP_RANGE, 19626 },
	{ 0x8ee4, 0x8ee4, PDF_CMAP_SINGLE, 9364 },
	{ 0x8ee5, 0x8eea, PDF_CMAP_RANGE, 19630 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 9363 },
	{ 0x8eec, 0x8ef1, PDF_CMAP_RANGE, 19636 },
	{ 0x8ef2, 0x8ef2, PDF_CMAP_SINGLE, 9358 },
	{ 0x8ef3, 0x8ef7, PDF_CMAP_RANGE, 19642 },
	{ 0x8ef8, 0x8eff, PDF_CMAP_TABLE, 7798 },
	{ 0x8f00, 0x8f02, PDF_CMAP_RANGE, 19649 },
	{ 0x8f03, 0x8f0a, PDF_CMAP_TABLE, 7806 },
	{ 0x8f0b, 0x8f11, PDF_CMAP_RANGE, 19655 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 7814 },
	{ 0x8f16, 0x8f1a, PDF_CMAP_RANGE, 19663 },
	{ 0x8f1b, 0x8f1d, PDF_CMAP_TABLE, 7818 },
	{ 0x8f1e, 0x8f1f, PDF_CMAP_RANGE, 9373 },
	{ 0x8f20, 0x8f24, PDF_CMAP_RANGE, 19668 },
	{ 0x8f25, 0x8f26, PDF_CMAP_TABLE, 7821 },
	{ 0x8f27, 0x8f28, PDF_CMAP_RANGE, 19673 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 7823 },
	{ 0x8f2b, 0x8f2e, PDF_CMAP_RANGE, 19675 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 8048 },
	{ 0x8f30, 0x8f32, PDF_CMAP_RANGE, 19679 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 9376 },
	{ 0x8f34, 0x8f37, PDF_CMAP_RANGE, 19682 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 8507 },
	{ 0x8f39, 0x8f3a, PDF_CMAP_RANGE, 19686 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 7943 },
	{ 0x8f3c, 0x8f3d, PDF_CMAP_RANGE, 19688 },
	{ 0x8f3e, 0x8f3f, PDF_CMAP_TABLE, 7825 },
	{ 0x8f40, 0x8f41, PDF_CMAP_RANGE, 19690 },
	{ 0x8f42, 0x8f46, PDF_CMAP_TABLE, 7827 },
	{ 0x8f47, 0x8f48, PDF_CMAP_RANGE, 19693 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 8872 },
	{ 0x8f4a, 0x8f4c, PDF_CMAP_RANGE, 19695 },
	{ 0x8f4d, 0x8f4e, PDF_CMAP_TABLE, 7832 },
	{ 0x8f4f, 0x8f53, PDF_CMAP_RANGE, 19698 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 9378 },
	{ 0x8f55, 0x8f5e, PDF_CMAP_RANGE, 19703 },
	{ 0x8f5f, 0x8f70, PDF_CMAP_TABLE, 7834 },
	{ 0x8f71, 0x8f73, PDF_CMAP_RANGE, 6371 },
	{ 0x8f74, 0x8f74, PDF_CMAP_SINGLE, 4576 },
	{ 0x8f75, 0x8f76, PDF_CMAP_RANGE, 6374 },
	{ 0x8f77, 0x8f78, PDF_CMAP_TABLE, 7852 },
	{ 0x8f79, 0x8f7a, PDF_CMAP_RANGE, 6378 },
	{ 0x8f7b, 0x8f80, PDF_CMAP_TABLE, 7854 },
	{ 0x8f81, 0x8f82, PDF_CMAP_RANGE, 6382 },
	{ 0x8f83, 0x8f8c, PDF_CMAP_TABLE, 7860 },
	{ 0x8f8d, 0x8f8f, PDF_CMAP_RANGE, 6387 },
	{ 0x8f90, 0x8f9f, PDF_CMAP_TABLE, 7870 },
	{ 0x8fa0, 0x8fa2, PDF_CMAP_RANGE, 19721 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 2460 },
	{ 0x8fa4, 0x8fa5, PDF_CMAP_RANGE, 19724 },
	{ 0x8fa6, 0x8fa7, PDF_CMAP_TABLE, 7886 },
	{ 0x8fa8, 0x8fa9, PDF_CMAP_RANGE, 1103 },
	{ 0x8faa, 0x8fb2, PDF_CMAP_TABLE, 7888 },
	{ 0x8fb3, 0x8fb5, PDF_CMAP_RANGE, 19729 },
	{ 0x8fb6, 0x8fb6, PDF_CMAP_SINGLE, 5949 },
	{ 0x8fb7, 0x8fb8, PDF_CMAP_RANGE, 19732 },
	{ 0x8fb9, 0x8fb9, PDF_CMAP_SINGLE, 1096 },
	{ 0x8fba, 0x8fbc, PDF_CMAP_RANGE, 19734 },
	{ 0x8fbd, 0x8fbe, PDF_CMAP_TABLE, 7897 },
	{ 0x8fbf, 0x8fc0, PDF_CMAP_RANGE, 19737 },
	{ 0x8fc1, 0x8fc8, PDF_CMAP_TABLE, 7899 },
	{ 0x8fc9, 0x8fcd, PDF_CMAP_RANGE, 19741 },
	{ 0x8fce, 0x8fd5, PDF_CMAP_TABLE, 7907 },
	{ 0x8fd6, 0x8fd7, PDF_CMAP_RANGE, 19748 },
	{ 0x8fd8, 0x8fdf, PDF_CMAP_TABLE, 7915 },
	{ 0x8fe0, 0x8fe1, PDF_CMAP_RANGE, 19751 },
	{ 0x8fe2, 0x8ff0, PDF_CMAP_TABLE, 7923 },
	{ 0x8ff1, 0x8ff2, PDF_CMAP_RANGE, 19757 },
	{ 0x8ff3, 0x8ff4, PDF_CMAP_TABLE, 7938 },
	{ 0x8ff5, 0x8ff6, PDF_CMAP_RANGE, 19759 },
	{ 0x8ff7, 0x8ff9, PDF_CMAP_TABLE, 7940 },
	{ 0x8ffa, 0x8ffc, PDF_CMAP_RANGE, 19761 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 4628 },
	{ 0x8ffe, 0x8fff, PDF_CMAP_RANGE, 19764 },
	{ 0x9000, 0x9006, PDF_CMAP_TABLE, 7943 },
	{ 0x9007, 0x9008, PDF_CMAP_RANGE, 19766 },
	{ 0x9009, 0x9023, PDF_CMAP_TABLE, 7950 },
	{ 0x9024, 0x9025, PDF_CMAP_RANGE, 19773 },
	{ 0x9026, 0x9026, PDF_CMAP_SINGLE, 5962 },
	{ 0x9027, 0x902c, PDF_CMAP_RANGE, 19775 },
	{ 0x902d, 0x902f, PDF_CMAP_TABLE, 7977 },
	{ 0x9030, 0x9031, PDF_CMAP_RANGE, 19781 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 8123 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 19783 },
	{ 0x9035, 0x9036, PDF_CMAP_RANGE, 5967 },
	{ 0x9037, 0x9038, PDF_CMAP_TABLE, 7980 },
	{ 0x9039, 0x903a, PDF_CMAP_RANGE, 19786 },
	{ 0x903b, 0x903e, PDF_CMAP_TABLE, 7982 },
	{ 0x903f, 0x9040, PDF_CMAP_RANGE, 19789 },
	{ 0x9041, 0x9044, PDF_CMAP_TABLE, 7986 },
	{ 0x9045, 0x9046, PDF_CMAP_RANGE, 19792 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 4315 },
	{ 0x9048, 0x904a, PDF_CMAP_RANGE, 19794 },
	{ 0x904b, 0x9050, PDF_CMAP_TABLE, 7990 },
	{ 0x9051, 0x9052, PDF_CMAP_RANGE, 5972 },
	{ 0x9053, 0x9058, PDF_CMAP_TABLE, 7996 },
	{ 0x9059, 0x905a, PDF_CMAP_RANGE, 19799 },
	{ 0x905b, 0x9065, PDF_CMAP_TABLE, 8002 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 19805 },
	{ 0x9068, 0x9069, PDF_CMAP_TABLE, 8013 },
	{ 0x906a, 0x906c, PDF_CMAP_RANGE, 19807 },
	{ 0x906d, 0x906e, PDF_CMAP_TABLE, 8015 },
	{ 0x906f, 0x9071, PDF_CMAP_RANGE, 19810 },
	{ 0x9072, 0x9084, PDF_CMAP_TABLE, 8017 },
	{ 0x9085, 0x9086, PDF_CMAP_RANGE, 19818 },
	{ 0x9087, 0x908b, PDF_CMAP_TABLE, 8036 },
	{ 0x908c, 0x908e, PDF_CMAP_RANGE, 19821 },
	{ 0x908f, 0x909d, PDF_CMAP_TABLE, 8041 },
	{ 0x909e, 0x90a0, PDF_CMAP_RANGE, 19830 },
	{ 0x90a1, 0x90a3, PDF_CMAP_TABLE, 8056 },
	{ 0x90a4, 0x90a5, PDF_CMAP_RANGE, 19833 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 1017 },
	{ 0x90a7, 0x90a9, PDF_CMAP_RANGE, 19835 },
	{ 0x90aa, 0x90bb, PDF_CMAP_TABLE, 8059 },
	{ 0x90bc, 0x90bd, PDF_CMAP_RANGE, 19842 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 4981 },
	{ 0x90bf, 0x90c0, PDF_CMAP_RANGE, 19844 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 4313 },
	{ 0x90c2, 0x90c3, PDF_CMAP_RANGE, 19846 },
	{ 0x90c4, 0x90c7, PDF_CMAP_TABLE, 8077 },
	{ 0x90c8, 0x90c9, PDF_CMAP_RANGE, 19849 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2189 },
	{ 0x90cb, 0x90cd, PDF_CMAP_RANGE, 19851 },
	{ 0x90ce, 0x90d3, PDF_CMAP_TABLE, 8081 },
	{ 0x90d4, 0x90d6, PDF_CMAP_RANGE, 19855 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 4989 },
	{ 0x90d8, 0x90da, PDF_CMAP_RANGE, 19858 },
	{ 0x90db, 0x90e2, PDF_CMAP_TABLE, 8087 },
	{ 0x90e3, 0x90e5, PDF_CMAP_RANGE, 19863 },
	{ 0x90e6, 0x90e8, PDF_CMAP_TABLE, 8095 },
	{ 0x90e9, 0x90ea, PDF_CMAP_RANGE, 19866 },
	{ 0x90eb, 0x90ef, PDF_CMAP_TABLE, 8098 },
	{ 0x90f0, 0x90f3, PDF_CMAP_RANGE, 19870 },
	{ 0x90f4, 0x90f5, PDF_CMAP_TABLE, 8103 },
	{ 0x90f6, 0x90f7, PDF_CMAP_RANGE, 19874 },
	{ 0x90f8, 0x90f8, PDF_CMAP_SINGLE, 1415 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 19876 },
	{ 0x90fd, 0x90ff, PDF_CMAP_TABLE, 8105 },
	{ 0x9100, 0x9101, PDF_CMAP_RANGE, 19881 },
	{ 0x9102, 0x9106, PDF_CMAP_TABLE, 8108 },
	{ 0x9107, 0x9108, PDF_CMAP_RANGE, 19885 },
	{ 0x9109, 0x9109, PDF_CMAP_SINGLE, 8647 },
	{ 0x910a, 0x9111, PDF_CMAP_RANGE, 19887 },
	{ 0x9112, 0x9116, PDF_CMAP_TABLE, 8113 },
	{ 0x9117, 0x9118, PDF_CMAP_RANGE, 19897 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 1074 },
	{ 0x911a, 0x911d, PDF_CMAP_RANGE, 19899 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 4996 },
	{ 0x911f, 0x9121, PDF_CMAP_RANGE, 19903 },
	{ 0x9122, 0x9123, PDF_CMAP_TABLE, 8118 },
	{ 0x9124, 0x9126, PDF_CMAP_RANGE, 19906 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 7867 },
	{ 0x9128, 0x912c, PDF_CMAP_RANGE, 19909 },
	{ 0x912d, 0x9136, PDF_CMAP_TABLE, 8120 },
	{ 0x9137, 0x9138, PDF_CMAP_RANGE, 19917 },
	{ 0x9139, 0x913a, PDF_CMAP_TABLE, 8130 },
	{ 0x913b, 0x9142, PDF_CMAP_RANGE, 19919 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 5001 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 19927 },
	{ 0x9146, 0x914d, PDF_CMAP_TABLE, 8132 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7418 },
	{ 0x9150, 0x9152, PDF_CMAP_TABLE, 8140 },
	{ 0x9153, 0x9156, PDF_CMAP_RANGE, 19931 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 4026 },
	{ 0x9158, 0x9159, PDF_CMAP_RANGE, 19935 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 1648 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 19937 },
	{ 0x915d, 0x915e, PDF_CMAP_TABLE, 8143 },
	{ 0x915f, 0x9160, PDF_CMAP_RANGE, 19939 },
	{ 0x9161, 0x9165, PDF_CMAP_TABLE, 8145 },
	{ 0x9166, 0x9168, PDF_CMAP_RANGE, 19941 },
	{ 0x9169, 0x9179, PDF_CMAP_TABLE, 8150 },
	{ 0x917a, 0x917c, PDF_CMAP_RANGE, 19947 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 7426 },
	{ 0x917f, 0x917f, PDF_CMAP_SINGLE, 2899 },
	{ 0x9180, 0x9184, PDF_CMAP_RANGE, 19950 },
	{ 0x9185, 0x918d, PDF_CMAP_TABLE, 8167 },
	{ 0x918e, 0x918f, PDF_CMAP_RANGE, 19958 },
	{ 0x9190, 0x9192, PDF_CMAP_TABLE, 8176 },
	{ 0x9193, 0x9195, PDF_CMAP_RANGE, 19960 },
	{ 0x9196, 0x9196, PDF_CMAP_SINGLE, 8789 },
	{ 0x9197, 0x9199, PDF_CMAP_RANGE, 19963 },
	{ 0x919a, 0x919c, PDF_CMAP_TABLE, 8179 },
	{ 0x919d, 0x91a1, PDF_CMAP_RANGE, 19966 },
	{ 0x91a2, 0x91a3, PDF_CMAP_RANGE, 7436 },
	{ 0x91a4, 0x91a9, PDF_CMAP_RANGE, 19971 },
	{ 0x91aa, 0x91ac, PDF_CMAP_TABLE, 8182 },
	{ 0x91ad, 0x91af, PDF_CMAP_RANGE, 7439 },
	{ 0x91b0, 0x91b3, PDF_CMAP_RANGE, 19977 },
	{ 0x91b4, 0x91b5, PDF_CMAP_TABLE, 8185 },
	{ 0x91b6, 0x91b9, PDF_CMAP_RANGE, 19981 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 7444 },
	{ 0x91bb, 0x91bf, PDF_CMAP_RANGE, 19985 },
	{ 0x91c0, 0x91d1, PDF_CMAP_TABLE, 8187 },
	{ 0x91d2, 0x91d4, PDF_CMAP_RANGE, 9459 },
	{ 0x91d5, 0x91d9, PDF_CMAP_TABLE, 8205 },
	{ 0x91da, 0x91db, PDF_CMAP_RANGE, 19996 },
	{ 0x91dc, 0x91dd, PDF_CMAP_TABLE, 8210 },
	{ 0x91de, 0x91e2, PDF_CMAP_RANGE, 19998 },
	{ 0x91e3, 0x91e4, PDF_CMAP_TABLE, 8212 },
	{ 0x91e5, 0x91e6, PDF_CMAP_RANGE, 20003 },
	{ 0x91e7, 0x91e9, PDF_CMAP_TABLE, 8214 },
	{ 0x91ea, 0x91f4, PDF_CMAP_RANGE, 20006 },
	{ 0x91f5, 0x91fa, PDF_CMAP_TABLE, 8217 },
	{ 0x91fb, 0x91ff, PDF_CMAP_RANGE, 20019 },
	{ 0x9200, 0x9201, PDF_CMAP_TABLE, 8223 },
	{ 0x9202, 0x9203, PDF_CMAP_RANGE, 20024 },
	{ 0x9204, 0x9204, PDF_CMAP_SINGLE, 9478 },
	{ 0x9205, 0x9207, PDF_CMAP_RANGE, 20026 },
	{ 0x9208, 0x9209, PDF_CMAP_TABLE, 8225 },
	{ 0x920a, 0x920c, PDF_CMAP_RANGE, 20029 },
	{ 0x920d, 0x9211, PDF_CMAP_TABLE, 8227 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 20033 },
	{ 0x9214, 0x9215, PDF_CMAP_TABLE, 8232 },
	{ 0x9216, 0x921d, PDF_CMAP_RANGE, 20035 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 8149 },
	{ 0x921f, 0x9222, PDF_CMAP_RANGE, 20043 },
	{ 0x9223, 0x9227, PDF_CMAP_TABLE, 8234 },
	{ 0x9228, 0x922d, PDF_CMAP_RANGE, 20048 },
	{ 0x922e, 0x9230, PDF_CMAP_TABLE, 8239 },
	{ 0x9231, 0x9232, PDF_CMAP_RANGE, 20055 },
	{ 0x9233, 0x9234, PDF_CMAP_TABLE, 8242 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 20057 },
	{ 0x9237, 0x923a, PDF_CMAP_TABLE, 8244 },
	{ 0x923b, 0x923c, PDF_CMAP_RANGE, 20059 },
	{ 0x923d, 0x9240, PDF_CMAP_TABLE, 8248 },
	{ 0x9241, 0x9244, PDF_CMAP_RANGE, 20061 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 9473 },
	{ 0x9246, 0x9247, PDF_CMAP_RANGE, 20065 },
	{ 0x9248, 0x9249, PDF_CMAP_TABLE, 8252 },
	{ 0x924a, 0x924c, PDF_CMAP_RANGE, 20067 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 9496 },
	{ 0x924e, 0x9250, PDF_CMAP_RANGE, 20070 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 7766 },
	{ 0x9252, 0x9254, PDF_CMAP_RANGE, 20073 },
	{ 0x9255, 0x9257, PDF_CMAP_TABLE, 8254 },
	{ 0x9258, 0x9259, PDF_CMAP_RANGE, 20077 },
	{ 0x925a, 0x925b, PDF_CMAP_TABLE, 8257 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 20079 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 9488 },
	{ 0x925f, 0x9261, PDF_CMAP_RANGE, 20081 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 7765 },
	{ 0x9263, 0x9265, PDF_CMAP_RANGE, 20084 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 9482 },
	{ 0x9267, 0x926b, PDF_CMAP_RANGE, 20087 },
	{ 0x926c, 0x926d, PDF_CMAP_RANGE, 9489 },
	{ 0x926e, 0x9273, PDF_CMAP_RANGE, 20092 },
	{ 0x9274, 0x9274, PDF_CMAP_SINGLE, 2155 },
	{ 0x9275, 0x9277, PDF_CMAP_RANGE, 20098 },
	{ 0x9278, 0x927b, PDF_CMAP_TABLE, 8259 },
	{ 0x927c, 0x927e, PDF_CMAP_RANGE, 20102 },
	{ 0x927f, 0x9280, PDF_CMAP_TABLE, 8263 },
	{ 0x9281, 0x9282, PDF_CMAP_RANGE, 20105 },
	{ 0x9283, 0x9285, PDF_CMAP_TABLE, 8265 },
	{ 0x9286, 0x928d, PDF_CMAP_RANGE, 20108 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 7552 },
	{ 0x928f, 0x9290, PDF_CMAP_RANGE, 20116 },
	{ 0x9291, 0x9293, PDF_CMAP_TABLE, 8268 },
	{ 0x9294, 0x9295, PDF_CMAP_RANGE, 20119 },
	{ 0x9296, 0x929c, PDF_CMAP_TABLE, 8271 },
	{ 0x929d, 0x929f, PDF_CMAP_RANGE, 20124 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 9501 },
	{ 0x92a1, 0x92a2, PDF_CMAP_RANGE, 20127 },
	{ 0x92a3, 0x92ae, PDF_CMAP_TABLE, 8278 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 20132 },
	{ 0x92b1, 0x92b1, PDF_CMAP_SINGLE, 9509 },
	{ 0x92b2, 0x92b6, PDF_CMAP_RANGE, 20134 },
	{ 0x92b7, 0x92bc, PDF_CMAP_TABLE, 8290 },
	{ 0x92bd, 0x92c0, PDF_CMAP_RANGE, 20141 },
	{ 0x92c1, 0x92c8, PDF_CMAP_TABLE, 8296 },
	{ 0x92c9, 0x92cb, PDF_CMAP_RANGE, 20148 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 9513 },
	{ 0x92cd, 0x92ce, PDF_CMAP_RANGE, 20151 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 9505 },
	{ 0x92d0, 0x92d1, PDF_CMAP_RANGE, 20153 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 7935 },
	{ 0x92d3, 0x92dc, PDF_CMAP_RANGE, 20155 },
	{ 0x92dd, 0x92df, PDF_CMAP_TABLE, 8304 },
	{ 0x92e0, 0x92e2, PDF_CMAP_RANGE, 20166 },
	{ 0x92e3, 0x92ea, PDF_CMAP_TABLE, 8307 },
	{ 0x92eb, 0x92ec, PDF_CMAP_RANGE, 20171 },
	{ 0x92ed, 0x92f1, PDF_CMAP_TABLE, 8315 },
	{ 0x92f2, 0x92f5, PDF_CMAP_RANGE, 20173 },
	{ 0x92f6, 0x92f8, PDF_CMAP_TABLE, 8320 },
	{ 0x92f9, 0x92fb, PDF_CMAP_RANGE, 20178 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 7960 },
	{ 0x92fd, 0x9300, PDF_CMAP_RANGE, 20181 },
	{ 0x9301, 0x9301, PDF_CMAP_SINGLE, 9547 },
	{ 0x9302, 0x9305, PDF_CMAP_RANGE, 20185 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 9543 },
	{ 0x9307, 0x9308, PDF_CMAP_RANGE, 9551 },
	{ 0x9309, 0x930f, PDF_CMAP_RANGE, 20189 },
	{ 0x9310, 0x9312, PDF_CMAP_TABLE, 8323 },
	{ 0x9313, 0x9314, PDF_CMAP_RANGE, 20197 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 9548 },
	{ 0x9316, 0x9317, PDF_CMAP_RANGE, 20199 },
	{ 0x9318, 0x931b, PDF_CMAP_TABLE, 8326 },
	{ 0x931c, 0x931e, PDF_CMAP_RANGE, 20201 },
	{ 0x931f, 0x9322, PDF_CMAP_TABLE, 8330 },
	{ 0x9323, 0x9325, PDF_CMAP_RANGE, 20205 },
	{ 0x9326, 0x9328, PDF_CMAP_TABLE, 8334 },
	{ 0x9329, 0x932a, PDF_CMAP_RANGE, 20209 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 8619 },
	{ 0x932c, 0x932d, PDF_CMAP_RANGE, 20211 },
	{ 0x932e, 0x932f, PDF_CMAP_TABLE, 8337 },
	{ 0x9330, 0x9331, PDF_CMAP_RANGE, 20213 },
	{ 0x9332, 0x9333, PDF_CMAP_TABLE, 8339 },
	{ 0x9334, 0x9335, PDF_CMAP_RANGE, 20215 },
	{ 0x9336, 0x9338, PDF_CMAP_TABLE, 8341 },
	{ 0x9339, 0x933d, PDF_CMAP_RANGE, 20218 },
	{ 0x933e, 0x9343, PDF_CMAP_TABLE, 8344 },
	{ 0x9344, 0x9345, PDF_CMAP_RANGE, 20225 },
	{ 0x9346, 0x9347, PDF_CMAP_TABLE, 8350 },
	{ 0x9348, 0x934a, PDF_CMAP_RANGE, 20227 },
	{ 0x934b, 0x934d, PDF_CMAP_TABLE, 8352 },
	{ 0x934e, 0x9353, PDF_CMAP_RANGE, 20231 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 9558 },
	{ 0x9355, 0x9357, PDF_CMAP_RANGE, 20237 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 8808 },
	{ 0x9359, 0x935a, PDF_CMAP_RANGE, 20240 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 7894 },
	{ 0x935c, 0x9363, PDF_CMAP_RANGE, 20242 },
	{ 0x9364, 0x9365, PDF_CMAP_TABLE, 8355 },
	{ 0x9366, 0x9368, PDF_CMAP_RANGE, 20250 },
	{ 0x9369, 0x936c, PDF_CMAP_TABLE, 8357 },
	{ 0x936d, 0x936f, PDF_CMAP_RANGE, 20254 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 9561 },
	{ 0x9371, 0x9374, PDF_CMAP_RANGE, 20257 },
	{ 0x9375, 0x9376, PDF_CMAP_TABLE, 8361 },
	{ 0x9377, 0x9379, PDF_CMAP_RANGE, 20261 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 8829 },
	{ 0x937b, 0x937d, PDF_CMAP_RANGE, 20264 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 9598 },
	{ 0x937f, 0x9381, PDF_CMAP_RANGE, 20267 },
	{ 0x9382, 0x9384, PDF_CMAP_TABLE, 8363 },
	{ 0x9385, 0x9386, PDF_CMAP_RANGE, 20271 },
	{ 0x9387, 0x9387, PDF_CMAP_SINGLE, 9566 },
	{ 0x9388, 0x9389, PDF_CMAP_RANGE, 20273 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 7732 },
	{ 0x938b, 0x938e, PDF_CMAP_RANGE, 20275 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 7558 },
	{ 0x9390, 0x9395, PDF_CMAP_RANGE, 20279 },
	{ 0x9396, 0x9398, PDF_CMAP_TABLE, 8366 },
	{ 0x9399, 0x93a1, PDF_CMAP_RANGE, 20286 },
	{ 0x93a2, 0x93a3, PDF_CMAP_TABLE, 8369 },
	{ 0x93a4, 0x93a5, PDF_CMAP_RANGE, 20295 },
	{ 0x93a6, 0x93b0, PDF_CMAP_TABLE, 8371 },
	{ 0x93b1, 0x93b2, PDF_CMAP_RANGE, 20301 },
	{ 0x93b3, 0x93b5, PDF_CMAP_TABLE, 8382 },
	{ 0x93b6, 0x93b7, PDF_CMAP_RANGE, 20304 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 9569 },
	{ 0x93b9, 0x93be, PDF_CMAP_RANGE, 20306 },
	{ 0x93bf, 0x93bf, PDF_CMAP_SINGLE, 9570 },
	{ 0x93c0, 0x93c2, PDF_CMAP_RANGE, 20312 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 9580 },
	{ 0x93c4, 0x93c6, PDF_CMAP_RANGE, 20315 },
	{ 0x93c7, 0x93cd, PDF_CMAP_TABLE, 8385 },
	{ 0x93ce, 0x93d0, PDF_CMAP_RANGE, 20320 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 9582 },
	{ 0x93d2, 0x93d5, PDF_CMAP_RANGE, 20323 },
	{ 0x93d6, 0x93d8, PDF_CMAP_TABLE, 8392 },
	{ 0x93d9, 0x93db, PDF_CMAP_RANGE, 20327 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 9576 },
	{ 0x93de, 0x93e4, PDF_CMAP_TABLE, 8395 },
	{ 0x93e5, 0x93e7, PDF_CMAP_RANGE, 20332 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 9757 },
	{ 0x93e9, 0x93f4, PDF_CMAP_RANGE, 20335 },
	{ 0x93f5, 0x93f9, PDF_CMAP_TABLE, 8402 },
	{ 0x93fa, 0x9402, PDF_CMAP_RANGE, 20349 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 9506 },
	{ 0x9404, 0x940a, PDF_CMAP_RANGE, 20358 },
	{ 0x940b, 0x940b, PDF_CMAP_SINGLE, 9523 },
	{ 0x940c, 0x940f, PDF_CMAP_RANGE, 20365 },
	{ 0x9410, 0x9414, PDF_CMAP_TABLE, 8407 },
	{ 0x9415, 0x9417, PDF_CMAP_RANGE, 20370 },
	{ 0x9418, 0x9419, PDF_CMAP_TABLE, 8412 },
	{ 0x941a, 0x941c, PDF_CMAP_RANGE, 20373 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 9584 },
	{ 0x941e, 0x941f, PDF_CMAP_RANGE, 20376 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 9589 },
	{ 0x9421, 0x9425, PDF_CMAP_RANGE, 20378 },
	{ 0x9426, 0x9427, PDF_CMAP_RANGE, 9537 },
	{ 0x9428, 0x9428, PDF_CMAP_SINGLE, 9565 },
	{ 0x9429, 0x942d, PDF_CMAP_RANGE, 20383 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 8218 },
	{ 0x942f, 0x9431, PDF_CMAP_RANGE, 20388 },
	{ 0x9432, 0x9435, PDF_CMAP_TABLE, 8414 },
	{ 0x9436, 0x9437, PDF_CMAP_RANGE, 20392 },
	{ 0x9438, 0x943a, PDF_CMAP_TABLE, 8418 },
	{ 0x943b, 0x943d, PDF_CMAP_RANGE, 20395 },
	{ 0x943e, 0x943f, PDF_CMAP_TABLE, 8421 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 20398 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8867 },
	{ 0x9445, 0x9449, PDF_CMAP_RANGE, 20402 },
	{ 0x944a, 0x944c, PDF_CMAP_TABLE, 8423 },
	{ 0x944d, 0x9451, PDF_CMAP_RANGE, 20408 },
	{ 0x9452, 0x9454, PDF_CMAP_TABLE, 8426 },
	{ 0x9455, 0x945f, PDF_CMAP_RANGE, 20414 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 9492 },
	{ 0x9461, 0x9462, PDF_CMAP_RANGE, 20425 },
	{ 0x9463, 0x9465, PDF_CMAP_TABLE, 8429 },
	{ 0x9466, 0x946a, PDF_CMAP_RANGE, 20428 },
	{ 0x946b, 0x946d, PDF_CMAP_TABLE, 8432 },
	{ 0x946e, 0x946f, PDF_CMAP_RANGE, 20434 },
	{ 0x9470, 0x9472, PDF_CMAP_TABLE, 8435 },
	{ 0x9473, 0x9476, PDF_CMAP_RANGE, 20437 },
	{ 0x9477, 0x9479, PDF_CMAP_TABLE, 8438 },
	{ 0x947a, 0x947b, PDF_CMAP_RANGE, 20442 },
	{ 0x947c, 0x947f, PDF_CMAP_TABLE, 8441 },
	{ 0x9480, 0x9484, PDF_CMAP_RANGE, 20444 },
	{ 0x9485, 0x9487, PDF_CMAP_RANGE, 6798 },
	{ 0x9488, 0x948b, PDF_CMAP_TABLE, 8445 },
	{ 0x948c, 0x948d, PDF_CMAP_RANGE, 6803 },
	{ 0x948e, 0x948e, PDF_CMAP_SINGLE, 3106 },
	{ 0x948f, 0x9490, PDF_CMAP_RANGE, 6805 },
	{ 0x9491, 0x9499, PDF_CMAP_TABLE, 8449 },
	{ 0x949a, 0x949c, PDF_CMAP_RANGE, 6810 },
	{ 0x949d, 0x94a2, PDF_CMAP_TABLE, 8458 },
	{ 0x94a3, 0x94a4, PDF_CMAP_RANGE, 6813 },
	{ 0x94a5, 0x94ae, PDF_CMAP_TABLE, 8464 },
	{ 0x94af, 0x94b0, PDF_CMAP_RANGE, 6819 },
	{ 0x94b1, 0x94b5, PDF_CMAP_TABLE, 8474 },
	{ 0x94b6, 0x94ba, PDF_CMAP_RANGE, 6823 },
	{ 0x94bb, 0x94bb, PDF_CMAP_SINGLE, 4679 },
	{ 0x94bc, 0x94bd, PDF_CMAP_RANGE, 6828 },
	{ 0x94be, 0x94c7, PDF_CMAP_TABLE, 8479 },
	{ 0x94c8, 0x94ce, PDF_CMAP_RANGE, 6832 },
	{ 0x94cf, 0x94cf, PDF_CMAP_SINGLE, 20453 },
	{ 0x94d0, 0x94d2, PDF_CMAP_RANGE, 6839 },
	{ 0x94d3, 0x94d4, PDF_CMAP_RANGE, 20454 },
	{ 0x94d5, 0x94d7, PDF_CMAP_RANGE, 6842 },
	{ 0x94d8, 0x94dd, PDF_CMAP_TABLE, 8489 },
	{ 0x94de, 0x94e0, PDF_CMAP_RANGE, 6848 },
	{ 0x94e1, 0x94e3, PDF_CMAP_TABLE, 8495 },
	{ 0x94e4, 0x94e5, PDF_CMAP_RANGE, 6852 },
	{ 0x94e6, 0x94e6, PDF_CMAP_SINGLE, 20457 },
	{ 0x94e7, 0x94e8, PDF_CMAP_RANGE, 6854 },
	{ 0x94e9, 0x94ed, PDF_CMAP_TABLE, 8498 },
	{ 0x94ee, 0x94ef, PDF_CMAP_RANGE, 6859 },
	{ 0x94f0, 0x94f2, PDF_CMAP_TABLE, 8503 },
	{ 0x94f3, 0x94f5, PDF_CMAP_RANGE, 6861 },
	{ 0x94f6, 0x94fb, PDF_CMAP_TABLE, 8506 },
	{ 0x94fc, 0x94fd, PDF_CMAP_RANGE, 6866 },
	{ 0x94fe, 0x9505, PDF_CMAP_TABLE, 8512 },
	{ 0x9506, 0x9507, PDF_CMAP_RANGE, 6871 },
	{ 0x9508, 0x9508, PDF_CMAP_SINGLE, 4013 },
	{ 0x9509, 0x950a, PDF_CMAP_RANGE, 6873 },
	{ 0x950b, 0x950c, PDF_CMAP_TABLE, 8520 },
	{ 0x950d, 0x950f, PDF_CMAP_RANGE, 6875 },
	{ 0x9510, 0x9511, PDF_CMAP_TABLE, 8522 },
	{ 0x9512, 0x9516, PDF_CMAP_RANGE, 6878 },
	{ 0x9517, 0x951c, PDF_CMAP_TABLE, 8524 },
	{ 0x951d, 0x951f, PDF_CMAP_RANGE, 6885 },
	{ 0x9520, 0x9529, PDF_CMAP_TABLE, 8530 },
	{ 0x952a, 0x952b, PDF_CMAP_RANGE, 6889 },
	{ 0x952c, 0x9530, PDF_CMAP_TABLE, 8540 },
	{ 0x9531, 0x9532, PDF_CMAP_RANGE, 6893 },
	{ 0x9533, 0x9535, PDF_CMAP_TABLE, 8545 },
	{ 0x9536, 0x9538, PDF_CMAP_RANGE, 6896 },
	{ 0x9539, 0x953d, PDF_CMAP_TABLE, 8548 },
	{ 0x953e, 0x953f, PDF_CMAP_RANGE, 6900 },
	{ 0x9540, 0x9543, PDF_CMAP_TABLE, 8553 },
	{ 0x9544, 0x9546, PDF_CMAP_RANGE, 6904 },
	{ 0x9547, 0x954d, PDF_CMAP_TABLE, 8557 },
	{ 0x954e, 0x954f, PDF_CMAP_RANGE, 6909 },
	{ 0x9550, 0x9551, PDF_CMAP_TABLE, 8564 },
	{ 0x9552, 0x9554, PDF_CMAP_RANGE, 6911 },
	{ 0x9555, 0x9555, PDF_CMAP_SINGLE, 20467 },
	{ 0x9556, 0x9559, PDF_CMAP_RANGE, 6914 },
	{ 0x955a, 0x955d, PDF_CMAP_TABLE, 8566 },
	{ 0x955e, 0x955f, PDF_CMAP_RANGE, 6919 },
	{ 0x9560, 0x9560, PDF_CMAP_SINGLE, 20469 },
	{ 0x9561, 0x9562, PDF_CMAP_RANGE, 6922 },
	{ 0x9563, 0x9563, PDF_CMAP_SINGLE, 2582 },
	{ 0x9564, 0x956c, PDF_CMAP_RANGE, 6924 },
	{ 0x956d, 0x9570, PDF_CMAP_TABLE, 8570 },
	{ 0x9571, 0x9573, PDF_CMAP_RANGE, 6934 },
	{ 0x9574, 0x9575, PDF_CMAP_RANGE, 20471 },
	{ 0x9576, 0x9577, PDF_CMAP_TABLE, 8574 },
	{ 0x9578, 0x957e, PDF_CMAP_RANGE, 20473 },
	{ 0x957f, 0x9583, PDF_CMAP_TABLE, 8576 },
	{ 0x9584, 0x9585, PDF_CMAP_RANGE, 20481 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 9152 },
	{ 0x9587, 0x9588, PDF_CMAP_RANGE, 20483 },
	{ 0x9589, 0x9594, PDF_CMAP_TABLE, 8581 },
	{ 0x9595, 0x9597, PDF_CMAP_RANGE, 20489 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 8809 },
	{ 0x9599, 0x95a0, PDF_CMAP_RANGE, 20492 },
	{ 0x95a1, 0x95a5, PDF_CMAP_TABLE, 8593 },
	{ 0x95a6, 0x95a7, PDF_CMAP_RANGE, 20501 },
	{ 0x95a8, 0x95ad, PDF_CMAP_TABLE, 8598 },
	{ 0x95ae, 0x95b1, PDF_CMAP_RANGE, 20504 },
	{ 0x95b2, 0x95b2, PDF_CMAP_SINGLE, 8783 },
	{ 0x95b3, 0x95b5, PDF_CMAP_RANGE, 20508 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 9163 },
	{ 0x95b7, 0x95b8, PDF_CMAP_RANGE, 20511 },
	{ 0x95b9, 0x95bf, PDF_CMAP_TABLE, 8604 },
	{ 0x95c0, 0x95c2, PDF_CMAP_RANGE, 20514 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 9168 },
	{ 0x95c4, 0x95c5, PDF_CMAP_RANGE, 20517 },
	{ 0x95c6, 0x95cc, PDF_CMAP_TABLE, 8611 },
	{ 0x95cd, 0x95cf, PDF_CMAP_RANGE, 20521 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 9171 },
	{ 0x95d1, 0x95d3, PDF_CMAP_RANGE, 20524 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 8618 },
	{ 0x95d7, 0x95db, PDF_CMAP_RANGE, 20527 },
	{ 0x95dc, 0x95de, PDF_CMAP_TABLE, 8621 },
	{ 0x95df, 0x95e0, PDF_CMAP_RANGE, 20533 },
	{ 0x95e1, 0x95e2, PDF_CMAP_TABLE, 8624 },
	{ 0x95e3, 0x95e4, PDF_CMAP_RANGE, 20535 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 9157 },
	{ 0x95e6, 0x95e7, PDF_CMAP_RANGE, 20537 },
	{ 0x95e8, 0x95f4, PDF_CMAP_TABLE, 8626 },
	{ 0x95f5, 0x95f6, PDF_CMAP_RANGE, 5769 },
	{ 0x95f7, 0x9602, PDF_CMAP_TABLE, 8639 },
	{ 0x9603, 0x9604, PDF_CMAP_RANGE, 5773 },
	{ 0x9605, 0x9609, PDF_CMAP_TABLE, 8651 },
	{ 0x960a, 0x960d, PDF_CMAP_RANGE, 5777 },
	{ 0x960e, 0x9614, PDF_CMAP_TABLE, 8656 },
	{ 0x9615, 0x9617, PDF_CMAP_RANGE, 5783 },
	{ 0x9618, 0x9618, PDF_CMAP_SINGLE, 20543 },
	{ 0x9619, 0x961a, PDF_CMAP_RANGE, 5786 },
	{ 0x961b, 0x9622, PDF_CMAP_TABLE, 8663 },
	{ 0x9623, 0x9629, PDF_CMAP_RANGE, 20547 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 4952 },
	{ 0x962b, 0x962d, PDF_CMAP_RANGE, 20554 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 3273 },
	{ 0x962f, 0x9630, PDF_CMAP_RANGE, 20557 },
	{ 0x9631, 0x9636, PDF_CMAP_TABLE, 8671 },
	{ 0x9637, 0x963a, PDF_CMAP_RANGE, 20559 },
	{ 0x963b, 0x964d, PDF_CMAP_TABLE, 8677 },
	{ 0x964e, 0x964f, PDF_CMAP_RANGE, 20567 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 3915 },
	{ 0x9651, 0x9653, PDF_CMAP_RANGE, 20569 },
	{ 0x9654, 0x9655, PDF_CMAP_TABLE, 8696 },
	{ 0x9656, 0x9657, PDF_CMAP_RANGE, 20572 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 8986 },
	{ 0x9659, 0x965a, PDF_CMAP_RANGE, 20574 },
	{ 0x965b, 0x9664, PDF_CMAP_TABLE, 8698 },
	{ 0x9665, 0x9666, PDF_CMAP_RANGE, 20579 },
	{ 0x9667, 0x966c, PDF_CMAP_TABLE, 8708 },
	{ 0x966d, 0x966f, PDF_CMAP_RANGE, 20582 },
	{ 0x9670, 0x9678, PDF_CMAP_TABLE, 8714 },
	{ 0x9679, 0x967c, PDF_CMAP_RANGE, 20586 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 8705 },
	{ 0x967e, 0x9684, PDF_CMAP_RANGE, 20590 },
	{ 0x9685, 0x9690, PDF_CMAP_TABLE, 8723 },
	{ 0x9691, 0x9693, PDF_CMAP_RANGE, 20600 },
	{ 0x9694, 0x969c, PDF_CMAP_TABLE, 8735 },
	{ 0x969d, 0x96a6, PDF_CMAP_RANGE, 20605 },
	{ 0x96a7, 0x96aa, PDF_CMAP_TABLE, 8744 },
	{ 0x96ab, 0x96af, PDF_CMAP_RANGE, 20616 },
	{ 0x96b0, 0x96bb, PDF_CMAP_TABLE, 8748 },
	{ 0x96bc, 0x96bd, PDF_CMAP_RANGE, 7546 },
	{ 0x96be, 0x96c1, PDF_CMAP_TABLE, 8760 },
	{ 0x96c2, 0x96c3, PDF_CMAP_RANGE, 20626 },
	{ 0x96c4, 0x96c9, PDF_CMAP_TABLE, 8764 },
	{ 0x96ca, 0x96cb, PDF_CMAP_RANGE, 20629 },
	{ 0x96cc, 0x96cf, PDF_CMAP_TABLE, 8770 },
	{ 0x96d0, 0x96d1, PDF_CMAP_RANGE, 20631 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 7549 },
	{ 0x96d3, 0x96d4, PDF_CMAP_RANGE, 20633 },
	{ 0x96d5, 0x96d6, PDF_CMAP_TABLE, 8774 },
	{ 0x96d7, 0x96d8, PDF_CMAP_RANGE, 20635 },
	{ 0x96d9, 0x96dc, PDF_CMAP_TABLE, 8776 },
	{ 0x96dd, 0x96df, PDF_CMAP_RANGE, 20638 },
	{ 0x96e0, 0x96e3, PDF_CMAP_TABLE, 8780 },
	{ 0x96e4, 0x96e7, PDF_CMAP_RANGE, 20642 },
	{ 0x96e8, 0x96ea, PDF_CMAP_TABLE, 8784 },
	{ 0x96eb, 0x96ee, PDF_CMAP_RANGE, 20646 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 7523 },
	{ 0x96f0, 0x96f1, PDF_CMAP_RANGE, 20650 },
	{ 0x96f2, 0x96f3, PDF_CMAP_TABLE, 8787 },
	{ 0x96f4, 0x96f5, PDF_CMAP_RANGE, 20652 },
	{ 0x96f6, 0x96fb, PDF_CMAP_TABLE, 8789 },
	{ 0x96fc, 0x96fd, PDF_CMAP_RANGE, 20656 },
	{ 0x96fe, 0x9701, PDF_CMAP_TABLE, 8795 },
	{ 0x9702, 0x9703, PDF_CMAP_RANGE, 20659 },
	{ 0x9704, 0x9709, PDF_CMAP_TABLE, 8799 },
	{ 0x970a, 0x970c, PDF_CMAP_RANGE, 20662 },
	{ 0x970d, 0x970f, PDF_CMAP_TABLE, 8805 },
	{ 0x9710, 0x9712, PDF_CMAP_RANGE, 20665 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 2881 },
	{ 0x9714, 0x9715, PDF_CMAP_RANGE, 20668 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 2593 },
	{ 0x9717, 0x971b, PDF_CMAP_RANGE, 20670 },
	{ 0x971c, 0x971e, PDF_CMAP_TABLE, 8808 },
	{ 0x971f, 0x9726, PDF_CMAP_RANGE, 20676 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 8616 },
	{ 0x9728, 0x9729, PDF_CMAP_RANGE, 20684 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7529 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 20686 },
	{ 0x972d, 0x972d, PDF_CMAP_SINGLE, 7530 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 20688 },
	{ 0x9730, 0x9732, PDF_CMAP_TABLE, 8811 },
	{ 0x9733, 0x9737, PDF_CMAP_RANGE, 20691 },
	{ 0x9738, 0x9739, PDF_CMAP_TABLE, 8814 },
	{ 0x973a, 0x973c, PDF_CMAP_RANGE, 20696 },
	{ 0x973d, 0x973e, PDF_CMAP_TABLE, 8816 },
	{ 0x973f, 0x9741, PDF_CMAP_RANGE, 20699 },
	{ 0x9742, 0x9744, PDF_CMAP_TABLE, 8818 },
	{ 0x9745, 0x9747, PDF_CMAP_RANGE, 20703 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 8242 },
	{ 0x9749, 0x9751, PDF_CMAP_RANGE, 20706 },
	{ 0x9752, 0x9753, PDF_CMAP_TABLE, 8821 },
	{ 0x9754, 0x9755, PDF_CMAP_RANGE, 20715 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 2279 },
	{ 0x9757, 0x9758, PDF_CMAP_RANGE, 20717 },
	{ 0x9759, 0x975b, PDF_CMAP_TABLE, 8823 },
	{ 0x975c, 0x975d, PDF_CMAP_RANGE, 20719 },
	{ 0x975e, 0x9762, PDF_CMAP_TABLE, 8826 },
	{ 0x9763, 0x9764, PDF_CMAP_RANGE, 20722 },
	{ 0x9765, 0x9765, PDF_CMAP_SINGLE, 4738 },
	{ 0x9766, 0x9767, PDF_CMAP_RANGE, 20724 },
	{ 0x9768, 0x9769, PDF_CMAP_TABLE, 8831 },
	{ 0x976a, 0x9772, PDF_CMAP_RANGE, 20726 },
	{ 0x9773, 0x9776, PDF_CMAP_TABLE, 8833 },
	{ 0x9777, 0x977b, PDF_CMAP_RANGE, 20736 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7624 },
	{ 0x977d, 0x9784, PDF_CMAP_RANGE, 20741 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 7625 },
	{ 0x9786, 0x978a, PDF_CMAP_RANGE, 20749 },
	{ 0x978b, 0x9790, PDF_CMAP_TABLE, 8837 },
	{ 0x9791, 0x9792, PDF_CMAP_RANGE, 7626 },
	{ 0x9793, 0x9794, PDF_CMAP_TABLE, 8843 },
	{ 0x9795, 0x9797, PDF_CMAP_RANGE, 20758 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 3143 },
	{ 0x9799, 0x979f, PDF_CMAP_RANGE, 20761 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 2302 },
	{ 0x97a1, 0x97a2, PDF_CMAP_RANGE, 20768 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 7631 },
	{ 0x97a4, 0x97a5, PDF_CMAP_RANGE, 20770 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 9877 },
	{ 0x97a7, 0x97aa, PDF_CMAP_RANGE, 20772 },
	{ 0x97ab, 0x97af, PDF_CMAP_TABLE, 8845 },
	{ 0x97b0, 0x97b1, PDF_CMAP_RANGE, 20778 },
	{ 0x97b2, 0x97b4, PDF_CMAP_TABLE, 8850 },
	{ 0x97b5, 0x97bc, PDF_CMAP_RANGE, 20781 },
	{ 0x97bd, 0x97bd, PDF_CMAP_SINGLE, 9822 },
	{ 0x97be, 0x97c2, PDF_CMAP_RANGE, 20789 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 9821 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 20794 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 9874 },
	{ 0x97c7, 0x97c8, PDF_CMAP_RANGE, 20796 },
	{ 0x97c9, 0x97cc, PDF_CMAP_TABLE, 8853 },
	{ 0x97cd, 0x97d2, PDF_CMAP_RANGE, 20799 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 8000 },
	{ 0x97d4, 0x97d8, PDF_CMAP_RANGE, 20805 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 9317 },
	{ 0x97da, 0x97db, PDF_CMAP_RANGE, 20810 },
	{ 0x97dc, 0x97de, PDF_CMAP_TABLE, 8857 },
	{ 0x97df, 0x97e5, PDF_CMAP_RANGE, 20813 },
	{ 0x97e6, 0x97e9, PDF_CMAP_TABLE, 8860 },
	{ 0x97ea, 0x97ec, PDF_CMAP_RANGE, 6222 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 2289 },
	{ 0x97ee, 0x97f2, PDF_CMAP_RANGE, 20821 },
	{ 0x97f3, 0x97f6, PDF_CMAP_TABLE, 8864 },
	{ 0x97f7, 0x97fe, PDF_CMAP_RANGE, 20827 },
	{ 0x97ff, 0x980d, PDF_CMAP_TABLE, 8868 },
	{ 0x980e, 0x980f, PDF_CMAP_RANGE, 9670 },
	{ 0x9810, 0x9813, PDF_CMAP_TABLE, 8883 },
	{ 0x9814, 0x9816, PDF_CMAP_RANGE, 20840 },
	{ 0x9817, 0x9818, PDF_CMAP_TABLE, 8887 },
	{ 0x9819, 0x981b, PDF_CMAP_RANGE, 20843 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 9673 },
	{ 0x981d, 0x9820, PDF_CMAP_RANGE, 20846 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 9672 },
	{ 0x9822, 0x9823, PDF_CMAP_RANGE, 20850 },
	{ 0x9824, 0x9826, PDF_CMAP_TABLE, 8889 },
	{ 0x9827, 0x982c, PDF_CMAP_RANGE, 20853 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 8573 },
	{ 0x982e, 0x982f, PDF_CMAP_RANGE, 20859 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 8062 },
	{ 0x9831, 0x9836, PDF_CMAP_RANGE, 20861 },
	{ 0x9837, 0x9838, PDF_CMAP_TABLE, 8892 },
	{ 0x9839, 0x983a, PDF_CMAP_RANGE, 20867 },
	{ 0x983b, 0x983d, PDF_CMAP_TABLE, 8894 },
	{ 0x983e, 0x9845, PDF_CMAP_RANGE, 20870 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 8154 },
	{ 0x9847, 0x984b, PDF_CMAP_RANGE, 20878 },
	{ 0x984c, 0x984e, PDF_CMAP_TABLE, 8897 },
	{ 0x984f, 0x9852, PDF_CMAP_RANGE, 20883 },
	{ 0x9853, 0x9854, PDF_CMAP_TABLE, 8900 },
	{ 0x9855, 0x9857, PDF_CMAP_RANGE, 20887 },
	{ 0x9858, 0x985b, PDF_CMAP_TABLE, 8902 },
	{ 0x985c, 0x985d, PDF_CMAP_RANGE, 20891 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 8200 },
	{ 0x985f, 0x9861, PDF_CMAP_RANGE, 20893 },
	{ 0x9862, 0x9862, PDF_CMAP_SINGLE, 9680 },
	{ 0x9863, 0x9864, PDF_CMAP_RANGE, 20896 },
	{ 0x9865, 0x9867, PDF_CMAP_TABLE, 8906 },
	{ 0x9868, 0x986a, PDF_CMAP_RANGE, 20899 },
	{ 0x986b, 0x986c, PDF_CMAP_TABLE, 8909 },
	{ 0x986d, 0x986e, PDF_CMAP_RANGE, 20902 },
	{ 0x986f, 0x988b, PDF_CMAP_TABLE, 8911 },
	{ 0x988c, 0x988d, PDF_CMAP_RANGE, 7140 },
	{ 0x988e, 0x9899, PDF_CMAP_TABLE, 8940 },
	{ 0x989a, 0x989b, PDF_CMAP_RANGE, 7144 },
	{ 0x989c, 0x989d, PDF_CMAP_TABLE, 8952 },
	{ 0x989e, 0x989f, PDF_CMAP_RANGE, 7146 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 1472 },
	{ 0x98a1, 0x98a2, PDF_CMAP_RANGE, 7148 },
	{ 0x98a3, 0x98a4, PDF_CMAP_TABLE, 8954 },
	{ 0x98a5, 0x98a6, PDF_CMAP_RANGE, 7150 },
	{ 0x98a7, 0x98a8, PDF_CMAP_TABLE, 8956 },
	{ 0x98a9, 0x98ad, PDF_CMAP_RANGE, 20911 },
	{ 0x98ae, 0x98af, PDF_CMAP_RANGE, 9420 },
	{ 0x98b0, 0x98b3, PDF_CMAP_TABLE, 8958 },
	{ 0x98b4, 0x98b5, PDF_CMAP_RANGE, 20918 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 9422 },
	{ 0x98b7, 0x98bb, PDF_CMAP_RANGE, 20920 },
	{ 0x98bc, 0x98bc, PDF_CMAP_SINGLE, 9423 },
	{ 0x98bd, 0x98c3, PDF_CMAP_RANGE, 20925 },
	{ 0x98c4, 0x98c8, PDF_CMAP_TABLE, 8962 },
	{ 0x98c9, 0x98cd, PDF_CMAP_RANGE, 20934 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 1668 },
	{ 0x98cf, 0x98d0, PDF_CMAP_RANGE, 20939 },
	{ 0x98d1, 0x98d3, PDF_CMAP_RANGE, 6583 },
	{ 0x98d4, 0x98d5, PDF_CMAP_TABLE, 8967 },
	{ 0x98d6, 0x98d7, PDF_CMAP_RANGE, 20942 },
	{ 0x98d8, 0x98d8, PDF_CMAP_SINGLE, 3021 },
	{ 0x98d9, 0x98da, PDF_CMAP_RANGE, 6587 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 7924 },
	{ 0x98dc, 0x98dd, PDF_CMAP_RANGE, 20944 },
	{ 0x98de, 0x98e2, PDF_CMAP_TABLE, 8969 },
	{ 0x98e3, 0x98e6, PDF_CMAP_RANGE, 20947 },
	{ 0x98e7, 0x98e9, PDF_CMAP_TABLE, 8974 },
	{ 0x98ea, 0x98eb, PDF_CMAP_RANGE, 9119 },
	{ 0x98ec, 0x98ef, PDF_CMAP_TABLE, 8977 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 20953 },
	{ 0x98f2, 0x98f4, PDF_CMAP_TABLE, 8981 },
	{ 0x98f5, 0x98fb, PDF_CMAP_RANGE, 20956 },
	{ 0x98fc, 0x98ff, PDF_CMAP_TABLE, 8984 },
	{ 0x9900, 0x9902, PDF_CMAP_RANGE, 20964 },
	{ 0x9903, 0x9905, PDF_CMAP_TABLE, 8988 },
	{ 0x9906, 0x9908, PDF_CMAP_RANGE, 20968 },
	{ 0x9909, 0x990d, PDF_CMAP_TABLE, 8991 },
	{ 0x990e, 0x990f, PDF_CMAP_RANGE, 20972 },
	{ 0x9910, 0x9913, PDF_CMAP_TABLE, 8996 },
	{ 0x9914, 0x9917, PDF_CMAP_RANGE, 20974 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 8760 },
	{ 0x9919, 0x991a, PDF_CMAP_RANGE, 20978 },
	{ 0x991b, 0x991b, PDF_CMAP_SINGLE, 9125 },
	{ 0x991c, 0x991d, PDF_CMAP_RANGE, 20980 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 8090 },
	{ 0x991f, 0x9920, PDF_CMAP_RANGE, 20982 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 8643 },
	{ 0x9922, 0x9927, PDF_CMAP_RANGE, 20984 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 7982 },
	{ 0x9929, 0x992d, PDF_CMAP_RANGE, 20990 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7657 },
	{ 0x992f, 0x9932, PDF_CMAP_RANGE, 20995 },
	{ 0x9933, 0x9933, PDF_CMAP_SINGLE, 9116 },
	{ 0x9934, 0x9936, PDF_CMAP_RANGE, 20999 },
	{ 0x9937, 0x9937, PDF_CMAP_SINGLE, 9126 },
	{ 0x9938, 0x993b, PDF_CMAP_RANGE, 21002 },
	{ 0x993c, 0x993f, PDF_CMAP_TABLE, 9000 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 21007 },
	{ 0x9943, 0x9945, PDF_CMAP_TABLE, 9004 },
	{ 0x9946, 0x9947, PDF_CMAP_RANGE, 21011 },
	{ 0x9948, 0x994a, PDF_CMAP_RANGE, 9129 },
	{ 0x994b, 0x994c, PDF_CMAP_TABLE, 9007 },
	{ 0x994d, 0x9950, PDF_CMAP_RANGE, 21013 },
	{ 0x9951, 0x9957, PDF_CMAP_TABLE, 9009 },
	{ 0x9958, 0x995b, PDF_CMAP_RANGE, 21019 },
	{ 0x995c, 0x995e, PDF_CMAP_TABLE, 9016 },
	{ 0x995f, 0x9961, PDF_CMAP_RANGE, 21024 },
	{ 0x9962, 0x9966, PDF_CMAP_TABLE, 9019 },
	{ 0x9967, 0x996c, PDF_CMAP_RANGE, 5662 },
	{ 0x996d, 0x9977, PDF_CMAP_TABLE, 9024 },
	{ 0x9978, 0x9979, PDF_CMAP_RANGE, 21030 },
	{ 0x997a, 0x9981, PDF_CMAP_TABLE, 9035 },
	{ 0x9982, 0x9983, PDF_CMAP_RANGE, 21034 },
	{ 0x9984, 0x998f, PDF_CMAP_TABLE, 9043 },
	{ 0x9990, 0x9991, PDF_CMAP_RANGE, 5676 },
	{ 0x9992, 0x9992, PDF_CMAP_SINGLE, 2719 },
	{ 0x9993, 0x9995, PDF_CMAP_RANGE, 5678 },
	{ 0x9996, 0x9999, PDF_CMAP_TABLE, 9055 },
	{ 0x999a, 0x99a4, PDF_CMAP_RANGE, 21039 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 6955 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 21050 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5086 },
	{ 0x99a9, 0x99ab, PDF_CMAP_RANGE, 21052 },
	{ 0x99ac, 0x99ae, PDF_CMAP_TABLE, 9059 },
	{ 0x99af, 0x99b0, PDF_CMAP_RANGE, 21055 },
	{ 0x99b1, 0x99b4, PDF_CMAP_TABLE, 9062 },
	{ 0x99b5, 0x99c0, PDF_CMAP_RANGE, 21058 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 7767 },
	{ 0x99c2, 0x99cf, PDF_CMAP_RANGE, 21070 },
	{ 0x99d0, 0x99d5, PDF_CMAP_TABLE, 9066 },
	{ 0x99d6, 0x99d7, PDF_CMAP_RANGE, 21085 },
	{ 0x99d8, 0x99e2, PDF_CMAP_TABLE, 9072 },
	{ 0x99e3, 0x99ec, PDF_CMAP_RANGE, 21091 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 7999 },
	{ 0x99ee, 0x99f0, PDF_CMAP_RANGE, 21101 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 8295 },
	{ 0x99f2, 0x99fe, PDF_CMAP_RANGE, 21104 },
	{ 0x99ff, 0x9a01, PDF_CMAP_TABLE, 9083 },
	{ 0x9a02, 0x9a04, PDF_CMAP_RANGE, 21118 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 9231 },
	{ 0x9a06, 0x9a0c, PDF_CMAP_RANGE, 21121 },
	{ 0x9a0d, 0x9a0f, PDF_CMAP_TABLE, 9086 },
	{ 0x9a10, 0x9a15, PDF_CMAP_RANGE, 21128 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 9234 },
	{ 0x9a17, 0x9a18, PDF_CMAP_RANGE, 21134 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 8375 },
	{ 0x9a1a, 0x9a2a, PDF_CMAP_RANGE, 21136 },
	{ 0x9a2b, 0x9a30, PDF_CMAP_TABLE, 9089 },
	{ 0x9a31, 0x9a35, PDF_CMAP_RANGE, 21155 },
	{ 0x9a36, 0x9a38, PDF_CMAP_TABLE, 9095 },
	{ 0x9a39, 0x9a3d, PDF_CMAP_RANGE, 21160 },
	{ 0x9a3e, 0x9a42, PDF_CMAP_TABLE, 9098 },
	{ 0x9a43, 0x9a44, PDF_CMAP_RANGE, 9238 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 8433 },
	{ 0x9a46, 0x9a49, PDF_CMAP_RANGE, 21166 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 9226 },
	{ 0x9a4b, 0x9a4c, PDF_CMAP_RANGE, 21170 },
	{ 0x9a4d, 0x9a4f, PDF_CMAP_TABLE, 9103 },
	{ 0x9a50, 0x9a54, PDF_CMAP_RANGE, 21173 },
	{ 0x9a55, 0x9a57, PDF_CMAP_TABLE, 9106 },
	{ 0x9a58, 0x9a59, PDF_CMAP_RANGE, 21179 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_TABLE, 9109 },
	{ 0x9a5c, 0x9a5e, PDF_CMAP_RANGE, 21181 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 8860 },
	{ 0x9a60, 0x9a61, PDF_CMAP_RANGE, 21184 },
	{ 0x9a62, 0x9a65, PDF_CMAP_TABLE, 9111 },
	{ 0x9a66, 0x9a69, PDF_CMAP_RANGE, 21187 },
	{ 0x9a6a, 0x9a76, PDF_CMAP_TABLE, 9115 },
	{ 0x9a77, 0x9a78, PDF_CMAP_RANGE, 6075 },
	{ 0x9a79, 0x9a7f, PDF_CMAP_TABLE, 9128 },
	{ 0x9a80, 0x9a81, PDF_CMAP_RANGE, 6080 },
	{ 0x9a82, 0x9a8c, PDF_CMAP_TABLE, 9135 },
	{ 0x9a8d, 0x9a8e, PDF_CMAP_RANGE, 21195 },
	{ 0x9a8f, 0x9a91, PDF_CMAP_TABLE, 9146 },
	{ 0x9a92, 0x9a93, PDF_CMAP_RANGE, 6086 },
	{ 0x9a94, 0x9a95, PDF_CMAP_RANGE, 21197 },
	{ 0x9a96, 0x9a9a, PDF_CMAP_TABLE, 9149 },
	{ 0x9a9b, 0x9a9d, PDF_CMAP_RANGE, 6090 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 5941 },
	{ 0x9a9f, 0x9aa0, PDF_CMAP_RANGE, 6093 },
	{ 0x9aa1, 0x9aa1, PDF_CMAP_SINGLE, 2697 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_RANGE, 6095 },
	{ 0x9aa4, 0x9aa8, PDF_CMAP_TABLE, 9154 },
	{ 0x9aa9, 0x9aae, PDF_CMAP_RANGE, 21201 },
	{ 0x9aaf, 0x9ab1, PDF_CMAP_TABLE, 9159 },
	{ 0x9ab2, 0x9ab5, PDF_CMAP_RANGE, 21207 },
	{ 0x9ab6, 0x9abc, PDF_CMAP_TABLE, 9162 },
	{ 0x9abd, 0x9abf, PDF_CMAP_RANGE, 21213 },
	{ 0x9ac0, 0x9ac2, PDF_CMAP_TABLE, 9169 },
	{ 0x9ac3, 0x9ac4, PDF_CMAP_RANGE, 21216 },
	{ 0x9ac5, 0x9ac5, PDF_CMAP_SINGLE, 7643 },
	{ 0x9ac6, 0x9aca, PDF_CMAP_RANGE, 21218 },
	{ 0x9acb, 0x9acc, PDF_CMAP_RANGE, 7645 },
	{ 0x9acd, 0x9ace, PDF_CMAP_RANGE, 21223 },
	{ 0x9acf, 0x9ad8, PDF_CMAP_TABLE, 9172 },
	{ 0x9ad9, 0x9ade, PDF_CMAP_RANGE, 21227 },
	{ 0x9adf, 0x9ae1, PDF_CMAP_TABLE, 9182 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_RANGE, 21234 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7662 },
	{ 0x9ae7, 0x9aea, PDF_CMAP_RANGE, 21238 },
	{ 0x9aeb, 0x9aef, PDF_CMAP_TABLE, 9185 },
	{ 0x9af0, 0x9af8, PDF_CMAP_RANGE, 21243 },
	{ 0x9af9, 0x9afb, PDF_CMAP_TABLE, 9190 },
	{ 0x9afc, 0x9b02, PDF_CMAP_RANGE, 21253 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 4660 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 21260 },
	{ 0x9b06, 0x9b08, PDF_CMAP_TABLE, 9193 },
	{ 0x9b09, 0x9b0c, PDF_CMAP_RANGE, 21263 },
	{ 0x9b0d, 0x9b0f, PDF_CMAP_TABLE, 9196 },
	{ 0x9b10, 0x9b12, PDF_CMAP_RANGE, 21268 },
	{ 0x9b13, 0x9b13, PDF_CMAP_SINGLE, 7670 },
	{ 0x9b14, 0x9b19, PDF_CMAP_RANGE, 21271 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 9888 },
	{ 0x9b1b, 0x9b1e, PDF_CMAP_RANGE, 21277 },
	{ 0x9b1f, 0x9b1f, PDF_CMAP_SINGLE, 7671 },
	{ 0x9b20, 0x9b21, PDF_CMAP_RANGE, 21281 },
	{ 0x9b22, 0x9b29, PDF_CMAP_TABLE, 9199 },
	{ 0x9b2a, 0x9b2d, PDF_CMAP_RANGE, 21286 },
	{ 0x9b2e, 0x9b32, PDF_CMAP_TABLE, 9207 },
	{ 0x9b33, 0x9b3a, PDF_CMAP_RANGE, 21291 },
	{ 0x9b3b, 0x9b3c, PDF_CMAP_TABLE, 9212 },
	{ 0x9b3d, 0x9b40, PDF_CMAP_RANGE, 21299 },
	{ 0x9b41, 0x9b49, PDF_CMAP_TABLE, 9214 },
	{ 0x9b4a, 0x9b4c, PDF_CMAP_RANGE, 21304 },
	{ 0x9b4d, 0x9b51, PDF_CMAP_TABLE, 9223 },
	{ 0x9b52, 0x9b53, PDF_CMAP_RANGE, 21308 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 2826 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 21310 },
	{ 0x9b58, 0x9b5a, PDF_CMAP_TABLE, 9228 },
	{ 0x9b5b, 0x9b6e, PDF_CMAP_RANGE, 21314 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 8266 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 21334 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 9759 },
	{ 0x9b75, 0x9b76, PDF_CMAP_RANGE, 21338 },
	{ 0x9b77, 0x9b77, PDF_CMAP_SINGLE, 9758 },
	{ 0x9b78, 0x9b80, PDF_CMAP_RANGE, 21340 },
	{ 0x9b81, 0x9b83, PDF_CMAP_TABLE, 9231 },
	{ 0x9b84, 0x9b8d, PDF_CMAP_RANGE, 21350 },
	{ 0x9b8e, 0x9b92, PDF_CMAP_TABLE, 9234 },
	{ 0x9b93, 0x9b99, PDF_CMAP_RANGE, 21361 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 9769 },
	{ 0x9b9b, 0x9b9c, PDF_CMAP_RANGE, 21368 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_TABLE, 9239 },
	{ 0x9b9f, 0x9ba9, PDF_CMAP_RANGE, 21370 },
	{ 0x9baa, 0x9bae, PDF_CMAP_TABLE, 9241 },
	{ 0x9baf, 0x9bbf, PDF_CMAP_RANGE, 21382 },
	{ 0x9bc0, 0x9bc1, PDF_CMAP_TABLE, 9246 },
	{ 0x9bc2, 0x9bc6, PDF_CMAP_RANGE, 21399 },
	{ 0x9bc7, 0x9bca, PDF_CMAP_TABLE, 9248 },
	{ 0x9bcb, 0x9bd3, PDF_CMAP_RANGE, 21405 },
	{ 0x9bd4, 0x9bd6, PDF_CMAP_TABLE, 9252 },
	{ 0x9bd7, 0x9bda, PDF_CMAP_RANGE, 21415 },
	{ 0x9bdb, 0x9bdd, PDF_CMAP_TABLE, 9255 },
	{ 0x9bde, 0x9be0, PDF_CMAP_RANGE, 21420 },
	{ 0x9be1, 0x9be4, PDF_CMAP_TABLE, 9258 },
	{ 0x9be5, 0x9be6, PDF_CMAP_RANGE, 21424 },
	{ 0x9be7, 0x9be9, PDF_CMAP_TABLE, 9262 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 9787 },
	{ 0x9bec, 0x9bef, PDF_CMAP_RANGE, 21427 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 9794 },
	{ 0x9bf1, 0x9bf3, PDF_CMAP_RANGE, 21431 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 9796 },
	{ 0x9bf5, 0x9bfc, PDF_CMAP_RANGE, 21434 },
	{ 0x9bfd, 0x9bff, PDF_CMAP_TABLE, 9265 },
	{ 0x9c00, 0x9c07, PDF_CMAP_RANGE, 21443 },
	{ 0x9c08, 0x9c09, PDF_CMAP_TABLE, 9268 },
	{ 0x9c0a, 0x9c0c, PDF_CMAP_RANGE, 21451 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 9801 },
	{ 0x9c0e, 0x9c0f, PDF_CMAP_RANGE, 21454 },
	{ 0x9c10, 0x9c13, PDF_CMAP_TABLE, 9270 },
	{ 0x9c14, 0x9c1f, PDF_CMAP_RANGE, 21457 },
	{ 0x9c20, 0x9c20, PDF_CMAP_SINGLE, 9805 },
	{ 0x9c21, 0x9c22, PDF_CMAP_RANGE, 21469 },
	{ 0x9c23, 0x9c25, PDF_CMAP_TABLE, 9274 },
	{ 0x9c26, 0x9c27, PDF_CMAP_RANGE, 21472 },
	{ 0x9c28, 0x9c29, PDF_CMAP_TABLE, 9277 },
	{ 0x9c2a, 0x9c2c, PDF_CMAP_RANGE, 21474 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 9807 },
	{ 0x9c2e, 0x9c30, PDF_CMAP_RANGE, 21477 },
	{ 0x9c31, 0x9c3b, PDF_CMAP_TABLE, 9279 },
	{ 0x9c3c, 0x9c3d, PDF_CMAP_RANGE, 21484 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 9812 },
	{ 0x9c3f, 0x9c44, PDF_CMAP_RANGE, 21486 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 9816 },
	{ 0x9c46, 0x9c47, PDF_CMAP_RANGE, 21492 },
	{ 0x9c48, 0x9c49, PDF_CMAP_TABLE, 9290 },
	{ 0x9c4a, 0x9c51, PDF_CMAP_RANGE, 21494 },
	{ 0x9c52, 0x9c58, PDF_CMAP_TABLE, 9292 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_RANGE, 21504 },
	{ 0x9c5d, 0x9c5f, PDF_CMAP_TABLE, 9299 },
	{ 0x9c60, 0x9c66, PDF_CMAP_RANGE, 21509 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 9820 },
	{ 0x9c68, 0x9c6c, PDF_CMAP_RANGE, 21516 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 9772 },
	{ 0x9c6e, 0x9c77, PDF_CMAP_RANGE, 21521 },
	{ 0x9c78, 0x9c7c, PDF_CMAP_TABLE, 9302 },
	{ 0x9c7d, 0x9c7e, PDF_CMAP_RANGE, 21533 },
	{ 0x9c7f, 0x9c82, PDF_CMAP_TABLE, 9307 },
	{ 0x9c83, 0x9c84, PDF_CMAP_RANGE, 21536 },
	{ 0x9c85, 0x9c88, PDF_CMAP_RANGE, 7563 },
	{ 0x9c89, 0x9c8a, PDF_CMAP_RANGE, 21538 },
	{ 0x9c8b, 0x9c8f, PDF_CMAP_TABLE, 9311 },
	{ 0x9c90, 0x9c92, PDF_CMAP_RANGE, 7570 },
	{ 0x9c93, 0x9c93, PDF_CMAP_SINGLE, 21542 },
	{ 0x9c94, 0x9c95, PDF_CMAP_RANGE, 7573 },
	{ 0x9c96, 0x9c99, PDF_CMAP_RANGE, 21543 },
	{ 0x9c9a, 0x9c9b, PDF_CMAP_RANGE, 7575 },
	{ 0x9c9c, 0x9c9d, PDF_CMAP_TABLE, 9316 },
	{ 0x9c9e, 0x9ca3, PDF_CMAP_RANGE, 7577 },
	{ 0x9ca4, 0x9ca4, PDF_CMAP_SINGLE, 2523 },
	{ 0x9ca5, 0x9ca9, PDF_CMAP_RANGE, 7583 },
	{ 0x9caa, 0x9cac, PDF_CMAP_TABLE, 9318 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 7589 },
	{ 0x9caf, 0x9caf, PDF_CMAP_SINGLE, 21550 },
	{ 0x9cb0, 0x9cb7, PDF_CMAP_RANGE, 7591 },
	{ 0x9cb8, 0x9cb9, PDF_CMAP_TABLE, 9321 },
	{ 0x9cba, 0x9cbd, PDF_CMAP_RANGE, 7599 },
	{ 0x9cbe, 0x9cc2, PDF_CMAP_RANGE, 21552 },
	{ 0x9cc3, 0x9cc3, PDF_CMAP_SINGLE, 3285 },
	{ 0x9cc4, 0x9cc7, PDF_CMAP_RANGE, 7603 },
	{ 0x9cc8, 0x9cc9, PDF_CMAP_RANGE, 21557 },
	{ 0x9cca, 0x9cd0, PDF_CMAP_RANGE, 7607 },
	{ 0x9cd1, 0x9cd2, PDF_CMAP_RANGE, 21559 },
	{ 0x9cd3, 0x9cd5, PDF_CMAP_RANGE, 7614 },
	{ 0x9cd6, 0x9cd6, PDF_CMAP_SINGLE, 1111 },
	{ 0x9cd7, 0x9cd9, PDF_CMAP_RANGE, 7617 },
	{ 0x9cda, 0x9cdb, PDF_CMAP_RANGE, 21561 },
	{ 0x9cdc, 0x9cdd, PDF_CMAP_RANGE, 7620 },
	{ 0x9cde, 0x9cdf, PDF_CMAP_TABLE, 9323 },
	{ 0x9ce0, 0x9ce1, PDF_CMAP_RANGE, 21563 },
	{ 0x9ce2, 0x9ce2, PDF_CMAP_SINGLE, 7623 },
	{ 0x9ce3, 0x9ce4, PDF_CMAP_RANGE, 21565 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 8348 },
	{ 0x9ce6, 0x9ce8, PDF_CMAP_RANGE, 21567 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 9600 },
	{ 0x9cea, 0x9ceb, PDF_CMAP_RANGE, 21570 },
	{ 0x9cec, 0x9cec, PDF_CMAP_SINGLE, 8924 },
	{ 0x9ced, 0x9cf2, PDF_CMAP_RANGE, 21572 },
	{ 0x9cf3, 0x9cf6, PDF_CMAP_TABLE, 9325 },
	{ 0x9cf7, 0x9d05, PDF_CMAP_RANGE, 21579 },
	{ 0x9d06, 0x9d09, PDF_CMAP_TABLE, 9329 },
	{ 0x9d0a, 0x9d14, PDF_CMAP_RANGE, 21595 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 8578 },
	{ 0x9d16, 0x9d1a, PDF_CMAP_RANGE, 21606 },
	{ 0x9d1b, 0x9d1f, PDF_CMAP_TABLE, 9333 },
	{ 0x9d20, 0x9d22, PDF_CMAP_RANGE, 21613 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 9604 },
	{ 0x9d24, 0x9d25, PDF_CMAP_RANGE, 21616 },
	{ 0x9d26, 0x9d28, PDF_CMAP_TABLE, 9338 },
	{ 0x9d29, 0x9d2e, PDF_CMAP_RANGE, 21619 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_TABLE, 9341 },
	{ 0x9d31, 0x9d3a, PDF_CMAP_RANGE, 21625 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 8007 },
	{ 0x9d3c, 0x9d3e, PDF_CMAP_RANGE, 21635 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 7965 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 21638 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 9613 },
	{ 0x9d43, 0x9d50, PDF_CMAP_RANGE, 21640 },
	{ 0x9d51, 0x9d53, PDF_CMAP_TABLE, 9343 },
	{ 0x9d54, 0x9d5b, PDF_CMAP_RANGE, 21654 },
	{ 0x9d5c, 0x9d5d, PDF_CMAP_TABLE, 9346 },
	{ 0x9d5e, 0x9d5f, PDF_CMAP_RANGE, 21662 },
	{ 0x9d60, 0x9d61, PDF_CMAP_TABLE, 9348 },
	{ 0x9d62, 0x9d69, PDF_CMAP_RANGE, 21664 },
	{ 0x9d6a, 0x9d6c, PDF_CMAP_TABLE, 9350 },
	{ 0x9d6d, 0x9d6e, PDF_CMAP_RANGE, 21673 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 9624 },
	{ 0x9d70, 0x9d71, PDF_CMAP_RANGE, 21675 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 8438 },
	{ 0x9d73, 0x9d86, PDF_CMAP_RANGE, 21677 },
	{ 0x9d87, 0x9d89, PDF_CMAP_TABLE, 9353 },
	{ 0x9d8a, 0x9d92, PDF_CMAP_RANGE, 21698 },
	{ 0x9d93, 0x9d93, PDF_CMAP_SINGLE, 9622 },
	{ 0x9d94, 0x9d97, PDF_CMAP_RANGE, 21707 },
	{ 0x9d98, 0x9d9a, PDF_CMAP_TABLE, 9356 },
	{ 0x9d9b, 0x9da4, PDF_CMAP_RANGE, 21712 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 9629 },
	{ 0x9da6, 0x9da8, PDF_CMAP_RANGE, 21722 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 9630 },
	{ 0x9daa, 0x9dae, PDF_CMAP_RANGE, 21725 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 9036 },
	{ 0x9db0, 0x9db3, PDF_CMAP_RANGE, 21730 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 8004 },
	{ 0x9db5, 0x9dba, PDF_CMAP_RANGE, 21734 },
	{ 0x9dbb, 0x9dbc, PDF_CMAP_TABLE, 9359 },
	{ 0x9dbd, 0x9dbf, PDF_CMAP_RANGE, 21740 },
	{ 0x9dc0, 0x9dc4, PDF_CMAP_TABLE, 9361 },
	{ 0x9dc5, 0x9dd2, PDF_CMAP_RANGE, 21745 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 9634 },
	{ 0x9dd4, 0x9dd6, PDF_CMAP_RANGE, 21759 },
	{ 0x9dd7, 0x9dda, PDF_CMAP_TABLE, 9366 },
	{ 0x9ddb, 0x9de4, PDF_CMAP_RANGE, 21763 },
	{ 0x9de5, 0x9de6, PDF_CMAP_TABLE, 9370 },
	{ 0x9de7, 0x9dee, PDF_CMAP_RANGE, 21773 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 9636 },
	{ 0x9df0, 0x9df1, PDF_CMAP_RANGE, 21781 },
	{ 0x9df2, 0x9df3, PDF_CMAP_TABLE, 9372 },
	{ 0x9df4, 0x9df7, PDF_CMAP_RANGE, 21783 },
	{ 0x9df8, 0x9dfa, PDF_CMAP_TABLE, 9374 },
	{ 0x9dfb, 0x9e0b, PDF_CMAP_RANGE, 21787 },
	{ 0x9e0c, 0x9e0c, PDF_CMAP_SINGLE, 9640 },
	{ 0x9e0d, 0x9e14, PDF_CMAP_RANGE, 21804 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 9606 },
	{ 0x9e16, 0x9e19, PDF_CMAP_RANGE, 21812 },
	{ 0x9e1a, 0x9e27, PDF_CMAP_TABLE, 9377 },
	{ 0x9e28, 0x9e2c, PDF_CMAP_RANGE, 6967 },
	{ 0x9e2d, 0x9e38, PDF_CMAP_TABLE, 9391 },
	{ 0x9e39, 0x9e3a, PDF_CMAP_RANGE, 6977 },
	{ 0x9e3b, 0x9e3c, PDF_CMAP_RANGE, 21822 },
	{ 0x9e3d, 0x9e40, PDF_CMAP_TABLE, 9403 },
	{ 0x9e41, 0x9e42, PDF_CMAP_RANGE, 6980 },
	{ 0x9e43, 0x9e45, PDF_CMAP_TABLE, 9407 },
	{ 0x9e46, 0x9e49, PDF_CMAP_RANGE, 6983 },
	{ 0x9e4a, 0x9e4a, PDF_CMAP_SINGLE, 3216 },
	{ 0x9e4b, 0x9e4c, PDF_CMAP_RANGE, 6987 },
	{ 0x9e4d, 0x9e51, PDF_CMAP_TABLE, 9410 },
	{ 0x9e52, 0x9e54, PDF_CMAP_RANGE, 21827 },
	{ 0x9e55, 0x9e59, PDF_CMAP_TABLE, 9415 },
	{ 0x9e5a, 0x9e5c, PDF_CMAP_RANGE, 6993 },
	{ 0x9e5d, 0x9e5e, PDF_CMAP_TABLE, 9420 },
	{ 0x9e5f, 0x9e62, PDF_CMAP_RANGE, 21833 },
	{ 0x9e63, 0x9e65, PDF_CMAP_TABLE, 9422 },
	{ 0x9e66, 0x9e6c, PDF_CMAP_RANGE, 6998 },
	{ 0x9e6d, 0x9e6d, PDF_CMAP_SINGLE, 7006 },
	{ 0x9e6e, 0x9e6f, PDF_CMAP_RANGE, 21838 },
	{ 0x9e70, 0x9e75, PDF_CMAP_TABLE, 9425 },
	{ 0x9e76, 0x9e78, PDF_CMAP_RANGE, 21842 },
	{ 0x9e79, 0x9e7f, PDF_CMAP_TABLE, 9431 },
	{ 0x9e80, 0x9e81, PDF_CMAP_RANGE, 21846 },
	{ 0x9e82, 0x9e82, PDF_CMAP_SINGLE, 7676 },
	{ 0x9e83, 0x9e86, PDF_CMAP_RANGE, 21848 },
	{ 0x9e87, 0x9e88, PDF_CMAP_RANGE, 7677 },
	{ 0x9e89, 0x9e8a, PDF_CMAP_RANGE, 21852 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 7679 },
	{ 0x9e8c, 0x9e91, PDF_CMAP_RANGE, 21854 },
	{ 0x9e92, 0x9e93, PDF_CMAP_TABLE, 9438 },
	{ 0x9e94, 0x9e96, PDF_CMAP_RANGE, 21860 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 8207 },
	{ 0x9e98, 0x9e9c, PDF_CMAP_RANGE, 21863 },
	{ 0x9e9d, 0x9e9f, PDF_CMAP_TABLE, 9440 },
	{ 0x9ea0, 0x9ea4, PDF_CMAP_RANGE, 21869 },
	{ 0x9ea5, 0x9ea6, PDF_CMAP_TABLE, 9443 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 21874 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 9719 },
	{ 0x9eaa, 0x9eae, PDF_CMAP_RANGE, 21876 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 9878 },
	{ 0x9eb0, 0x9eb3, PDF_CMAP_RANGE, 21881 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 9445 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 21885 },
	{ 0x9eb8, 0x9eb8, PDF_CMAP_SINGLE, 7405 },
	{ 0x9eb9, 0x9eba, PDF_CMAP_RANGE, 21887 },
	{ 0x9ebb, 0x9ebc, PDF_CMAP_TABLE, 9447 },
	{ 0x9ebd, 0x9ebe, PDF_CMAP_RANGE, 7673 },
	{ 0x9ebf, 0x9ec3, PDF_CMAP_RANGE, 21890 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 2011 },
	{ 0x9ec5, 0x9ec8, PDF_CMAP_RANGE, 21895 },
	{ 0x9ec9, 0x9ec9, PDF_CMAP_SINGLE, 4856 },
	{ 0x9eca, 0x9ecb, PDF_CMAP_RANGE, 21899 },
	{ 0x9ecc, 0x9ed1, PDF_CMAP_TABLE, 9449 },
	{ 0x9ed2, 0x9ed3, PDF_CMAP_RANGE, 21902 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 3114 },
	{ 0x9ed5, 0x9ed7, PDF_CMAP_RANGE, 21904 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 2831 },
	{ 0x9ed9, 0x9eda, PDF_CMAP_RANGE, 21907 },
	{ 0x9edb, 0x9edd, PDF_CMAP_RANGE, 7684 },
	{ 0x9ede, 0x9ee2, PDF_CMAP_TABLE, 9455 },
	{ 0x9ee3, 0x9ee4, PDF_CMAP_RANGE, 21910 },
	{ 0x9ee5, 0x9eea, PDF_CMAP_TABLE, 9460 },
	{ 0x9eeb, 0x9eee, PDF_CMAP_RANGE, 21913 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 7694 },
	{ 0x9ef0, 0x9ef1, PDF_CMAP_RANGE, 21917 },
	{ 0x9ef2, 0x9ef4, PDF_CMAP_TABLE, 9466 },
	{ 0x9ef5, 0x9ef6, PDF_CMAP_RANGE, 21920 },
	{ 0x9ef7, 0x9efa, PDF_CMAP_TABLE, 9469 },
	{ 0x9efb, 0x9efc, PDF_CMAP_RANGE, 6742 },
	{ 0x9efd, 0x9eff, PDF_CMAP_TABLE, 9473 },
	{ 0x9f00, 0x9f08, PDF_CMAP_RANGE, 21924 },
	{ 0x9f09, 0x9f10, PDF_CMAP_TABLE, 9476 },
	{ 0x9f11, 0x9f12, PDF_CMAP_RANGE, 21936 },
	{ 0x9f13, 0x9f19, PDF_CMAP_TABLE, 9484 },
	{ 0x9f1a, 0x9f1f, PDF_CMAP_RANGE, 21941 },
	{ 0x9f20, 0x9f22, PDF_CMAP_TABLE, 9491 },
	{ 0x9f23, 0x9f2b, PDF_CMAP_RANGE, 21948 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7696 },
	{ 0x9f2d, 0x9f2e, PDF_CMAP_RANGE, 21957 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 7697 },
	{ 0x9f30, 0x9f36, PDF_CMAP_RANGE, 21959 },
	{ 0x9f37, 0x9f3c, PDF_CMAP_TABLE, 9494 },
	{ 0x9f3d, 0x9f3e, PDF_CMAP_RANGE, 7700 },
	{ 0x9f3f, 0x9f43, PDF_CMAP_RANGE, 21969 },
	{ 0x9f44, 0x9f44, PDF_CMAP_SINGLE, 7702 },
	{ 0x9f45, 0x9f49, PDF_CMAP_RANGE, 21974 },
	{ 0x9f4a, 0x9f4b, PDF_CMAP_TABLE, 9500 },
	{ 0x9f4c, 0x9f4e, PDF_CMAP_RANGE, 21979 },
	{ 0x9f4f, 0x9f54, PDF_CMAP_TABLE, 9502 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 21983 },
	{ 0x9f59, 0x9f59, PDF_CMAP_SINGLE, 9745 },
	{ 0x9f5a, 0x9f5b, PDF_CMAP_RANGE, 21987 },
	{ 0x9f5c, 0x9f5c, PDF_CMAP_SINGLE, 9747 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 21989 },
	{ 0x9f5f, 0x9f63, PDF_CMAP_TABLE, 9508 },
	{ 0x9f64, 0x9f65, PDF_CMAP_RANGE, 21992 },
	{ 0x9f66, 0x9f66, PDF_CMAP_SINGLE, 9748 },
	{ 0x9f67, 0x9f69, PDF_CMAP_RANGE, 21994 },
	{ 0x9f6a, 0x9f6c, PDF_CMAP_TABLE, 9513 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 21998 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 8434 },
	{ 0x9f73, 0x9f76, PDF_CMAP_RANGE, 22003 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 9751 },
	{ 0x9f78, 0x9f7e, PDF_CMAP_RANGE, 22007 },
	{ 0x9f7f, 0x9f80, PDF_CMAP_TABLE, 9516 },
	{ 0x9f81, 0x9f82, PDF_CMAP_RANGE, 22014 },
	{ 0x9f83, 0x9f84, PDF_CMAP_TABLE, 9518 },
	{ 0x9f85, 0x9f8a, PDF_CMAP_RANGE, 7535 },
	{ 0x9f8b, 0x9f8d, PDF_CMAP_TABLE, 9520 },
	{ 0x9f8e, 0x9f8f, PDF_CMAP_RANGE, 22016 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 8371 },
	{ 0x9f91, 0x9f93, PDF_CMAP_RANGE, 22018 },
	{ 0x9f94, 0x9f95, PDF_CMAP_TABLE, 9523 },
	{ 0x9f96, 0x9f98, PDF_CMAP_RANGE, 22021 },
	{ 0x9f99, 0x9f9c, PDF_CMAP_TABLE, 9525 },
	{ 0x9f9d, 0x9f9e, PDF_CMAP_RANGE, 22024 },
	{ 0x9f9f, 0x9fa0, PDF_CMAP_TABLE, 9529 },
	{ 0x9fa1, 0x9fa5, PDF_CMAP_RANGE, 22026 },
	{ 0x9fb4, 0x9fb5, PDF_CMAP_TABLE, 9531 },
	{ 0x9fb6, 0x9fb7, PDF_CMAP_RANGE, 22069 },
	{ 0x9fb8, 0x9fbb, PDF_CMAP_TABLE, 9533 },
	{ 0xa000, 0xa48c, PDF_CMAP_RANGE, 29064 },
	{ 0xa490, 0xa4c6, PDF_CMAP_RANGE, 30229 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 22048 },
	{ 0xdc89, 0xdc89, PDF_CMAP_SINGLE, 22049 },
	{ 0xdccc, 0xdccc, PDF_CMAP_SINGLE, 22050 },
	{ 0xdd09, 0xdd09, PDF_CMAP_SINGLE, 22093 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 22085 },
	{ 0xdd9d, 0xdd9d, PDF_CMAP_SINGLE, 22110 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 22075 },
	{ 0xddfe, 0xddfe, PDF_CMAP_SINGLE, 22111 },
	{ 0xe76c, 0xe76c, PDF_CMAP_SINGLE, 22353 },
	{ 0xe78d, 0xe793, PDF_CMAP_RANGE, 573 },
	{ 0xe794, 0xe795, PDF_CMAP_RANGE, 592 },
	{ 0xe796, 0xe796, PDF_CMAP_SINGLE, 599 },
	{ 0xe7c7, 0xe7c8, PDF_CMAP_TABLE, 9537 },
	{ 0xe7e7, 0xe7f3, PDF_CMAP_RANGE, 10059 },
	{ 0xe815, 0xe864, PDF_CMAP_RANGE, 22047 },
	{ 0xf92c, 0xf92c, PDF_CMAP_SINGLE, 22031 },
	{ 0xf979, 0xf979, PDF_CMAP_SINGLE, 2562 },
	{ 0xf995, 0xf995, PDF_CMAP_SINGLE, 16595 },
	{ 0xf9e7, 0xf9e7, PDF_CMAP_SINGLE, 8204 },
	{ 0xf9f1, 0xf9f1, PDF_CMAP_SINGLE, 20611 },
	{ 0xfa0c, 0xfa0c, PDF_CMAP_SINGLE, 4697 },
	{ 0xfa0d, 0xfa0f, PDF_CMAP_RANGE, 22032 },
	{ 0xfa11, 0xfa11, PDF_CMAP_SINGLE, 22035 },
	{ 0xfa13, 0xfa14, PDF_CMAP_RANGE, 22036 },
	{ 0xfa18, 0xfa18, PDF_CMAP_SINGLE, 22038 },
	{ 0xfa1f, 0xfa21, PDF_CMAP_RANGE, 22039 },
	{ 0xfa23, 0xfa24, PDF_CMAP_RANGE, 22042 },
	{ 0xfa27, 0xfa29, PDF_CMAP_RANGE, 22044 },
	{ 0xfe10, 0xfe12, PDF_CMAP_TABLE, 9539 },
	{ 0xfe13, 0xfe16, PDF_CMAP_RANGE, 576 },
	{ 0xfe17, 0xfe18, PDF_CMAP_RANGE, 592 },
	{ 0xfe19, 0xfe19, PDF_CMAP_SINGLE, 599 },
	{ 0xfe30, 0xfe31, PDF_CMAP_TABLE, 9542 },
	{ 0xfe33, 0xfe34, PDF_CMAP_RANGE, 600 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 580 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 596 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 582 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 594 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 586 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 584 },
	{ 0xfe41, 0xfe44, PDF_CMAP_RANGE, 588 },
	{ 0xfe49, 0xfe52, PDF_CMAP_RANGE, 10027 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 10037 },
	{ 0xfe59, 0xfe66, PDF_CMAP_RANGE, 10041 },
	{ 0xfe68, 0xfe6b, PDF_CMAP_RANGE, 10055 },
	{ 0xff01, 0xff03, PDF_CMAP_RANGE, 262 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 166 },
	{ 0xff05, 0xff5d, PDF_CMAP_RANGE, 266 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 106 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 168 },
	{ 0xffe2, 0xffe5, PDF_CMAP_TABLE, 9544 },
	{ 0x6ee1, 0x6ee1, PDF_CMAP_SINGLE, 2721 },
	{ 0x6ee2, 0x6ee2, PDF_CMAP_SINGLE, 5884 },
	{ 0x6ee3, 0x6ee3, PDF_CMAP_SINGLE, 14849 },
	{ 0x6ee4, 0x6ee4, PDF_CMAP_SINGLE, 2674 },
	{ 0x6ee5, 0x6ee5, PDF_CMAP_SINGLE, 2479 },
	{ 0x6ee6, 0x6ee6, PDF_CMAP_SINGLE, 2679 },
	{ 0x6ee7, 0x6ee7, PDF_CMAP_SINGLE, 14850 },
	{ 0x6ee8, 0x6ee8, PDF_CMAP_SINGLE, 1118 },
	{ 0x6ee9, 0x6ee9, PDF_CMAP_SINGLE, 3589 },
	{ 0x6eea, 0x6eeb, PDF_CMAP_RANGE, 14851 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 8011 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 14853 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 8850 },
	{ 0x6ef0, 0x6ef1, PDF_CMAP_RANGE, 14855 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 8484 },
	{ 0x6ef3, 0x6ef3, PDF_CMAP_SINGLE, 14857 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 1455 },
	{ 0x6ef5, 0x6ef6, PDF_CMAP_RANGE, 14858 },
	{ 0x6ef7, 0x6ef7, PDF_CMAP_SINGLE, 8264 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 9184 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 5901 },
	{ 0x6efa, 0x6efe, PDF_CMAP_RANGE, 14860 },
	{ 0x6eff, 0x6eff, PDF_CMAP_SINGLE, 8311 },
	{ 0x6f00, 0x6f00, PDF_CMAP_SINGLE, 14865 },
	{ 0x6f01, 0x6f01, PDF_CMAP_SINGLE, 8762 },
	{ 0x6f02, 0x6f02, PDF_CMAP_SINGLE, 3022 },
	{ 0x6f03, 0x6f05, PDF_CMAP_RANGE, 14866 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 3072 },
	{ 0x6f07, 0x6f08, PDF_CMAP_RANGE, 14869 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 5907 },
	{ 0x6f0a, 0x6f0e, PDF_CMAP_RANGE, 14871 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 2640 },
	{ 0x6f10, 0x6f12, PDF_CMAP_RANGE, 14876 },
	{ 0x6f13, 0x6f13, PDF_CMAP_SINGLE, 2519 },
	{ 0x6f14, 0x6f14, PDF_CMAP_SINGLE, 4102 },
	{ 0x6f15, 0x6f15, PDF_CMAP_SINGLE, 5900 },
	{ 0x6f16, 0x6f19, PDF_CMAP_RANGE, 14879 },
	{ 0x6f1a, 0x6f1a, PDF_CMAP_SINGLE, 8369 },
	{ 0x6f1b, 0x6f1f, PDF_CMAP_RANGE, 14883 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 2833 },
	{ 0x6f21, 0x6f21, PDF_CMAP_SINGLE, 14888 },
	{ 0x6f22, 0x6f22, PDF_CMAP_SINGLE, 8001 },
	{ 0x6f23, 0x6f23, PDF_CMAP_SINGLE, 8220 },
	{ 0x6f24, 0x6f24, PDF_CMAP_SINGLE, 5899 },
	{ 0x6f25, 0x6f28, PDF_CMAP_RANGE, 14889 },
	{ 0x6f29, 0x6f29, PDF_CMAP_SINGLE, 5908 },
	{ 0x6f2a, 0x6f2a, PDF_CMAP_SINGLE, 5906 },
	{ 0x6f2b, 0x6f2b, PDF_CMAP_SINGLE, 2725 },
	{ 0x6f2c, 0x6f2c, PDF_CMAP_SINGLE, 8888 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 5883 },
	{ 0x6f2e, 0x6f2e, PDF_CMAP_SINGLE, 14893 },
	{ 0x6f2f, 0x6f2f, PDF_CMAP_SINGLE, 5902 },
	{ 0x6f30, 0x6f30, PDF_CMAP_SINGLE, 14894 },
	{ 0x6f31, 0x6f31, PDF_CMAP_SINGLE, 3477 },
	{ 0x6f32, 0x6f32, PDF_CMAP_SINGLE, 8822 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 4453 },
	{ 0x6f34, 0x6f35, PDF_CMAP_RANGE, 14895 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 5903 },
	{ 0x6f37, 0x6f37, PDF_CMAP_SINGLE, 14897 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 8091 },
	{ 0x6f39, 0x6f3d, PDF_CMAP_RANGE, 14898 },
	{ 0x6f3e, 0x6f3e, PDF_CMAP_SINGLE, 4131 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 8095 },
	{ 0x6f40, 0x6f40, PDF_CMAP_SINGLE, 14903 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 9674 },
	{ 0x6f42, 0x6f45, PDF_CMAP_RANGE, 14904 },
	{ 0x6f46, 0x6f47, PDF_CMAP_RANGE, 5897 },
	{ 0x6f48, 0x6f4a, PDF_CMAP_RANGE, 14908 },
	{ 0x6f4b, 0x6f4b, PDF_CMAP_SINGLE, 5904 },
	{ 0x6f4c, 0x6f4c, PDF_CMAP_SINGLE, 14911 },
	{ 0x6f4d, 0x6f4d, PDF_CMAP_SINGLE, 3770 },
	{ 0x6f4e, 0x6f50, PDF_CMAP_RANGE, 14912 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 8382 },
	{ 0x6f52, 0x6f53, PDF_CMAP_RANGE, 14915 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 8116 },
	{ 0x6f55, 0x6f57, PDF_CMAP_RANGE, 14917 },
	{ 0x6f58, 0x6f58, PDF_CMAP_SINGLE, 2956 },
	{ 0x6f59, 0x6f59, PDF_CMAP_SINGLE, 9175 },
	{ 0x6f5a, 0x6f5b, PDF_CMAP_RANGE, 14920 },
	{ 0x6f5c, 0x6f5c, PDF_CMAP_SINGLE, 3118 },
	{ 0x6f5d, 0x6f5d, PDF_CMAP_SINGLE, 14922 },
	{ 0x6f5e, 0x6f5e, PDF_CMAP_SINGLE, 2657 },
	{ 0x6f5f, 0x6f61, PDF_CMAP_RANGE, 14923 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 5896 },
	{ 0x6f63, 0x6f63, PDF_CMAP_SINGLE, 14926 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 8453 },
	{ 0x6f65, 0x6f65, PDF_CMAP_SINGLE, 14927 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 2579 },
	{ 0x6f67, 0x6f6c, PDF_CMAP_RANGE, 14928 },
	{ 0x6f6d, 0x6f6d, PDF_CMAP_SINGLE, 3593 },
	{ 0x6f6e, 0x6f6e, PDF_CMAP_SINGLE, 1238 },
	{ 0x6f6f, 0x6f6f, PDF_CMAP_SINGLE, 9185 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 8172 },
	{ 0x6f71, 0x6f71, PDF_CMAP_SINGLE, 14934 },
	{ 0x6f72, 0x6f72, PDF_CMAP_SINGLE, 5913 },
	{ 0x6f73, 0x6f73, PDF_CMAP_SINGLE, 14935 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 5905 },
	{ 0x6f75, 0x6f76, PDF_CMAP_RANGE, 14936 },
	{ 0x6f77, 0x6f77, PDF_CMAP_SINGLE, 9194 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 5912 },
	{ 0x6f79, 0x6f79, PDF_CMAP_SINGLE, 14938 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 5915 },
	{ 0x6f7b, 0x6f7b, PDF_CMAP_SINGLE, 14939 },
	{ 0x6f7c, 0x6f7c, PDF_CMAP_SINGLE, 5914 },
	{ 0x6f7d, 0x6f7e, PDF_CMAP_RANGE, 14940 },
	{ 0x6f7f, 0x6f7f, PDF_CMAP_SINGLE, 9187 },
	{ 0x6f80, 0x6f80, PDF_CMAP_SINGLE, 8462 },
	{ 0x6f81, 0x6f83, PDF_CMAP_RANGE, 14942 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 1267 },
	{ 0x6f85, 0x6f85, PDF_CMAP_SINGLE, 14945 },
	{ 0x6f86, 0x6f86, PDF_CMAP_SINGLE, 8102 },
	{ 0x6f87, 0x6f87, PDF_CMAP_SINGLE, 8196 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 1247 },
	{ 0x6f89, 0x6f89, PDF_CMAP_SINGLE, 5909 },
	{ 0x6f8a, 0x6f8b, PDF_CMAP_RANGE, 14946 },
	{ 0x6f8c, 0x6f8c, PDF_CMAP_SINGLE, 5911 },
	{ 0x6f8d, 0x6f8d, PDF_CMAP_SINGLE, 5910 },
	{ 0x6f8e, 0x6f8e, PDF_CMAP_SINGLE, 2989 },
	{ 0x6f8f, 0x6f96, PDF_CMAP_RANGE, 14948 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 8093 },
	{ 0x6f98, 0x6f9b, PDF_CMAP_RANGE, 14956 },
	{ 0x6f9c, 0x6f9c, PDF_CMAP_SINGLE, 2472 },
	{ 0x6f9d, 0x6f9f, PDF_CMAP_RANGE, 14960 },
	{ 0x6fa0, 0x6fa0, PDF_CMAP_SINGLE, 9189 },
	{ 0x6fa1, 0x6fa1, PDF_CMAP_SINGLE, 4395 },
	{ 0x6fa2, 0x6fa3, PDF_CMAP_RANGE, 14963 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 8804 },
	{ 0x6fa5, 0x6fa6, PDF_CMAP_RANGE, 14965 },
	{ 0x6fa7, 0x6fa7, PDF_CMAP_SINGLE, 5918 },
	{ 0x6fa8, 0x6fa8, PDF_CMAP_SINGLE, 14967 },
	{ 0x6fa9, 0x6fa9, PDF_CMAP_SINGLE, 9442 },
	{ 0x6faa, 0x6fad, PDF_CMAP_RANGE, 14968 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 9182 },
	{ 0x6faf, 0x6fb0, PDF_CMAP_RANGE, 14972 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 7876 },
	{ 0x6fb2, 0x6fb2, PDF_CMAP_SINGLE, 14974 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 975 },
	{ 0x6fb4, 0x6fb5, PDF_CMAP_RANGE, 14975 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 5920 },
	{ 0x6fb7, 0x6fb8, PDF_CMAP_RANGE, 14977 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5919 },
	{ 0x6fba, 0x6fbf, PDF_CMAP_RANGE, 14979 },
	{ 0x6fc0, 0x6fc0, PDF_CMAP_SINGLE, 2071 },
	{ 0x6fc1, 0x6fc1, PDF_CMAP_SINGLE, 8886 },
	{ 0x6fc2, 0x6fc2, PDF_CMAP_SINGLE, 5921 },
	{ 0x6fc3, 0x6fc3, PDF_CMAP_SINGLE, 8361 },
	{ 0x6fc4, 0x6fc8, PDF_CMAP_RANGE, 14985 },
	{ 0x6fc9, 0x6fc9, PDF_CMAP_SINGLE, 5917 },
	{ 0x6fca, 0x6fd0, PDF_CMAP_RANGE, 14990 },
	{ 0x6fd1, 0x6fd1, PDF_CMAP_SINGLE, 5916 },
	{ 0x6fd2, 0x6fd2, PDF_CMAP_SINGLE, 1117 },
	{ 0x6fd3, 0x6fd4, PDF_CMAP_RANGE, 14997 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 8491 },
	{ 0x6fd6, 0x6fd7, PDF_CMAP_RANGE, 14999 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 8357 },
	{ 0x6fd9, 0x6fda, PDF_CMAP_RANGE, 15001 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 9867 },
	{ 0x6fdc, 0x6fdd, PDF_CMAP_RANGE, 15003 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 5924 },
	{ 0x6fdf, 0x6fdf, PDF_CMAP_SINGLE, 8054 },
	{ 0x6fe0, 0x6fe0, PDF_CMAP_SINGLE, 5925 },
	{ 0x6fe1, 0x6fe1, PDF_CMAP_SINGLE, 5922 },
	{ 0x6fe2, 0x6fe3, PDF_CMAP_RANGE, 15005 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 8557 },
	{ 0x6fe5, 0x6fea, PDF_CMAP_RANGE, 15007 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 8193 },
	{ 0x6fec, 0x6fed, PDF_CMAP_RANGE, 15013 },
	{ 0x6fee, 0x6fee, PDF_CMAP_SINGLE, 5923 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 5926 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 8593 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 7760 },
	{ 0x6ff2, 0x6ff9, PDF_CMAP_RANGE, 15015 },
	{ 0x6ffa, 0x6ffa, PDF_CMAP_SINGLE, 8092 },
	{ 0x6ffb, 0x6ffb, PDF_CMAP_SINGLE, 15023 },
	{ 0x6ffc, 0x6ffc, PDF_CMAP_SINGLE, 9178 },
	{ 0x6ffd, 0x6ffd, PDF_CMAP_SINGLE, 15024 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 8275 },
	{ 0x6fff, 0x6fff, PDF_CMAP_SINGLE, 15025 },
	{ 0x7000, 0x7004, PDF_CMAP_RANGE, 15026 },
	{ 0x7005, 0x7005, PDF_CMAP_SINGLE, 9193 },
	{ 0x7006, 0x7006, PDF_CMAP_SINGLE, 9188 },
	{ 0x7007, 0x7008, PDF_CMAP_RANGE, 15031 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 8661 },
	{ 0x700a, 0x700a, PDF_CMAP_SINGLE, 15033 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 9190 },
	{ 0x700c, 0x700e, PDF_CMAP_RANGE, 15034 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 9183 },
	{ 0x7010, 0x7010, PDF_CMAP_SINGLE, 15037 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 3064 },
	{ 0x7012, 0x7014, PDF_CMAP_RANGE, 15038 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 7759 },
	{ 0x7016, 0x7017, PDF_CMAP_RANGE, 15041 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 9177 },
	{ 0x7019, 0x7019, PDF_CMAP_SINGLE, 15043 },
	{ 0x701a, 0x701a, PDF_CMAP_SINGLE, 5927 },
	{ 0x701b, 0x701b, PDF_CMAP_SINGLE, 5929 },
	{ 0x701c, 0x701c, PDF_CMAP_SINGLE, 15044 },
	{ 0x701d, 0x701d, PDF_CMAP_SINGLE, 8212 },
	{ 0x701e, 0x701e, PDF_CMAP_SINGLE, 15045 },
	{ 0x701f, 0x701f, PDF_CMAP_SINGLE, 9196 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 9195 },
	{ 0x7021, 0x7022, PDF_CMAP_RANGE, 15046 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 5928 },
	{ 0x7024, 0x7026, PDF_CMAP_RANGE, 15048 },
	{ 0x7027, 0x7027, PDF_CMAP_SINGLE, 9176 },
	{ 0x7028, 0x7028, PDF_CMAP_SINGLE, 9198 },
	{ 0x7029, 0x702f, PDF_CMAP_RANGE, 15051 },
	{ 0x7030, 0x7030, PDF_CMAP_SINGLE, 9870 },
	{ 0x7031, 0x7031, PDF_CMAP_SINGLE, 15058 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 9197 },
	{ 0x7033, 0x7034, PDF_CMAP_RANGE, 15059 },
	{ 0x7035, 0x7035, PDF_CMAP_SINGLE, 5931 },
	{ 0x7036, 0x7038, PDF_CMAP_RANGE, 15061 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 5930 },
	{ 0x703a, 0x703d, PDF_CMAP_RANGE, 15064 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 8186 },
	{ 0x703f, 0x7042, PDF_CMAP_RANGE, 15068 },
	{ 0x7043, 0x7043, PDF_CMAP_SINGLE, 9174 },
	{ 0x7044, 0x7044, PDF_CMAP_SINGLE, 9192 },
	{ 0x7045, 0x704b, PDF_CMAP_RANGE, 15072 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 1849 },
	{ 0x704d, 0x704e, PDF_CMAP_RANGE, 15079 },
	{ 0x704f, 0x704f, PDF_CMAP_SINGLE, 5932 },
	{ 0x7050, 0x7050, PDF_CMAP_SINGLE, 15081 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 8454 },
	{ 0x7052, 0x7054, PDF_CMAP_RANGE, 15082 },
	{ 0x7055, 0x7055, PDF_CMAP_SINGLE, 8203 },
	{ 0x7056, 0x7057, PDF_CMAP_RANGE, 15085 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 8550 },
	{ 0x7059, 0x705c, PDF_CMAP_RANGE, 15087 },
	{ 0x705d, 0x705d, PDF_CMAP_SINGLE, 9199 },
	{ 0x705e, 0x705e, PDF_CMAP_SINGLE, 5933 },
	{ 0x705f, 0x7062, PDF_CMAP_RANGE, 15091 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 8585 },
	{ 0x7064, 0x7064, PDF_CMAP_SINGLE, 8280 },
	{ 0x7065, 0x7066, PDF_CMAP_RANGE, 15095 },
	{ 0x7067, 0x7067, PDF_CMAP_SINGLE, 9191 },
	{ 0x7068, 0x706a, PDF_CMAP_RANGE, 15097 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 2053 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 6639 },
	{ 0x706d, 0x706d, PDF_CMAP_SINGLE, 2805 },
	{ 0x706e, 0x706e, PDF_CMAP_SINGLE, 15100 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 1447 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 2023 },
	{ 0x7071, 0x7074, PDF_CMAP_RANGE, 15101 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 2610 },
	{ 0x7076, 0x7076, PDF_CMAP_SINGLE, 4401 },
	{ 0x7077, 0x7077, PDF_CMAP_SINGLE, 15105 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 2291 },
	{ 0x7079, 0x707b, PDF_CMAP_RANGE, 15106 },
	{ 0x707c, 0x707c, PDF_CMAP_SINGLE, 4643 },
	{ 0x707d, 0x707d, PDF_CMAP_SINGLE, 15109 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 4377 },
	{ 0x707f, 0x707f, PDF_CMAP_SINGLE, 1178 },
	{ 0x7080, 0x7080, PDF_CMAP_SINGLE, 6604 },
	{ 0x7081, 0x7084, PDF_CMAP_RANGE, 15110 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 6419 },
	{ 0x7086, 0x7088, PDF_CMAP_RANGE, 15114 },
	{ 0x7089, 0x7089, PDF_CMAP_SINGLE, 2646 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 1337 },
	{ 0x708b, 0x708d, PDF_CMAP_RANGE, 15117 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 4096 },
	{ 0x708f, 0x7091, PDF_CMAP_RANGE, 15120 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 1241 },
	{ 0x7093, 0x7093, PDF_CMAP_SINGLE, 15123 },
	{ 0x7094, 0x7094, PDF_CMAP_SINGLE, 3213 },
	{ 0x7095, 0x7095, PDF_CMAP_SINGLE, 2376 },
	{ 0x7096, 0x7096, PDF_CMAP_SINGLE, 6606 },
	{ 0x7097, 0x7098, PDF_CMAP_RANGE, 15124 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 4554 },
	{ 0x709a, 0x709b, PDF_CMAP_RANGE, 15126 },
	{ 0x709c, 0x709c, PDF_CMAP_SINGLE, 6605 },
	{ 0x709d, 0x709d, PDF_CMAP_SINGLE, 6607 },
	{ 0x709e, 0x70aa, PDF_CMAP_RANGE, 15128 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 6611 },
	{ 0x70ac, 0x70ac, PDF_CMAP_SINGLE, 2325 },
	{ 0x70ad, 0x70ad, PDF_CMAP_SINGLE, 3602 },
	{ 0x70ae, 0x70ae, PDF_CMAP_SINGLE, 2971 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 2283 },
	{ 0x70b0, 0x70b0, PDF_CMAP_SINGLE, 15141 },
	{ 0x70b1, 0x70b1, PDF_CMAP_SINGLE, 6612 },
	{ 0x70b2, 0x70b2, PDF_CMAP_SINGLE, 15142 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 1127 },
	{ 0x70b4, 0x70b6, PDF_CMAP_RANGE, 15143 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 6610 },
	{ 0x70b8, 0x70b8, PDF_CMAP_SINGLE, 4425 },
	{ 0x70b9, 0x70b9, PDF_CMAP_SINGLE, 1476 },
	{ 0x70ba, 0x70ba, PDF_CMAP_SINGLE, 15146 },
	{ 0x70bb, 0x70bb, PDF_CMAP_SINGLE, 6608 },
	{ 0x70bc, 0x70bc, PDF_CMAP_SINGLE, 2559 },
	{ 0x70bd, 0x70bd, PDF_CMAP_SINGLE, 1288 },
	{ 0x70be, 0x70bf, PDF_CMAP_RANGE, 15147 },
	{ 0x70c0, 0x70c0, PDF_CMAP_SINGLE, 6609 },
	{ 0x70c1, 0x70c1, PDF_CMAP_SINGLE, 3501 },
	{ 0x70c2, 0x70c2, PDF_CMAP_SINGLE, 2478 },
	{ 0x70c3, 0x70c3, PDF_CMAP_SINGLE, 3665 },
	{ 0x70c4, 0x70c7, PDF_CMAP_RANGE, 15149 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 2587 },
	{ 0x70c9, 0x70c9, PDF_CMAP_SINGLE, 15153 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 6614 },
	{ 0x70cb, 0x70ce, PDF_CMAP_RANGE, 15154 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 8611 },
	{ 0x70d0, 0x70d7, PDF_CMAP_RANGE, 15158 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 1949 },
	{ 0x70d9, 0x70d9, PDF_CMAP_SINGLE, 2494 },
	{ 0x70da, 0x70da, PDF_CMAP_SINGLE, 15166 },
	{ 0x70db, 0x70db, PDF_CMAP_SINGLE, 4593 },
	{ 0x70dc, 0x70de, PDF_CMAP_RANGE, 15167 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 4085 },
	{ 0x70e0, 0x70e3, PDF_CMAP_RANGE, 15170 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2379 },
	{ 0x70e5, 0x70e5, PDF_CMAP_SINGLE, 15174 },
	{ 0x70e6, 0x70e6, PDF_CMAP_SINGLE, 1616 },
	{ 0x70e7, 0x70e7, PDF_CMAP_SINGLE, 3342 },
	{ 0x70e8, 0x70e8, PDF_CMAP_SINGLE, 6613 },
	{ 0x70e9, 0x70e9, PDF_CMAP_SINGLE, 2039 },
	{ 0x70ea, 0x70ea, PDF_CMAP_SINGLE, 15175 },
	{ 0x70eb, 0x70eb, PDF_CMAP_SINGLE, 3615 },
	{ 0x70ec, 0x70ec, PDF_CMAP_SINGLE, 2254 },
	{ 0x70ed, 0x70ed, PDF_CMAP_SINGLE, 3235 },
	{ 0x70ee, 0x70ee, PDF_CMAP_SINGLE, 15176 },
	{ 0x70ef, 0x70ef, PDF_CMAP_SINGLE, 3862 },
	{ 0x70f0, 0x70f3, PDF_CMAP_RANGE, 15177 },
	{ 0x70f4, 0x70f4, PDF_CMAP_SINGLE, 8570 },
	{ 0x70f5, 0x70f6, PDF_CMAP_RANGE, 15181 },
	{ 0x70f7, 0x70f7, PDF_CMAP_SINGLE, 3738 },
	{ 0x70f8, 0x70f8, PDF_CMAP_SINGLE, 15183 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 2988 },
	{ 0x70fa, 0x70fc, PDF_CMAP_RANGE, 15184 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 1670 },
	{ 0x70fe, 0x70ff, PDF_CMAP_RANGE, 15187 },
	{ 0x7100, 0x7108, PDF_CMAP_RANGE, 15189 },
	{ 0x7109, 0x7109, PDF_CMAP_SINGLE, 4082 },
	{ 0x710a, 0x710a, PDF_CMAP_SINGLE, 1903 },
	{ 0x710b, 0x710f, PDF_CMAP_RANGE, 15198 },
	{ 0x7110, 0x7110, PDF_CMAP_SINGLE, 6615 },
	{ 0x7111, 0x7112, PDF_CMAP_RANGE, 15203 },
	{ 0x7113, 0x7113, PDF_CMAP_SINGLE, 6616 },
	{ 0x7114, 0x7114, PDF_CMAP_SINGLE, 15205 },
	{ 0x7115, 0x7115, PDF_CMAP_SINGLE, 2005 },
	{ 0x7116, 0x7116, PDF_CMAP_SINGLE, 6617 },
	{ 0x7117, 0x7117, PDF_CMAP_SINGLE, 15206 },
	{ 0x7118, 0x7118, PDF_CMAP_SINGLE, 6640 },
	{ 0x7119, 0x7119, PDF_CMAP_SINGLE, 1059 },
	{ 0x711a, 0x711a, PDF_CMAP_SINGLE, 1654 },
	{ 0x711b, 0x7120, PDF_CMAP_RANGE, 15207 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 8613 },
	{ 0x7122, 0x7125, PDF_CMAP_RANGE, 15213 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2186 },
	{ 0x7127, 0x712e, PDF_CMAP_RANGE, 15217 },
	{ 0x712f, 0x712f, PDF_CMAP_SINGLE, 6618 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 4111 },
	{ 0x7131, 0x7131, PDF_CMAP_SINGLE, 6619 },
	{ 0x7132, 0x7135, PDF_CMAP_RANGE, 15225 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 3222 },
	{ 0x7137, 0x7144, PDF_CMAP_RANGE, 15229 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 6623 },
	{ 0x7146, 0x7148, PDF_CMAP_RANGE, 15243 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 8226 },
	{ 0x714a, 0x714a, PDF_CMAP_SINGLE, 6625 },
	{ 0x714b, 0x714b, PDF_CMAP_SINGLE, 15246 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 2018 },
	{ 0x714d, 0x714d, PDF_CMAP_SINGLE, 15247 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 2136 },
	{ 0x714f, 0x7151, PDF_CMAP_RANGE, 15248 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 9430 },
	{ 0x7153, 0x715b, PDF_CMAP_RANGE, 15251 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 6621 },
	{ 0x715d, 0x715d, PDF_CMAP_SINGLE, 15260 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 3312 },
	{ 0x715f, 0x7161, PDF_CMAP_RANGE, 15261 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 9018 },
	{ 0x7163, 0x7163, PDF_CMAP_SINGLE, 15264 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 2751 },
	{ 0x7165, 0x7165, PDF_CMAP_SINGLE, 15265 },
	{ 0x7166, 0x7166, PDF_CMAP_SINGLE, 6641 },
	{ 0x7167, 0x7167, PDF_CMAP_SINGLE, 4470 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 6622 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 7918 },
	{ 0x716a, 0x716b, PDF_CMAP_RANGE, 15266 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 9429 },
	{ 0x716d, 0x716d, PDF_CMAP_SINGLE, 15268 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 4594 },
	{ 0x716f, 0x7171, PDF_CMAP_RANGE, 15269 },
	{ 0x7172, 0x7172, PDF_CMAP_SINGLE, 6624 },
	{ 0x7173, 0x7173, PDF_CMAP_SINGLE, 6620 },
	{ 0x7174, 0x7177, PDF_CMAP_RANGE, 15272 },
	{ 0x7178, 0x7178, PDF_CMAP_SINGLE, 6626 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 15276 },
	{ 0x717a, 0x717a, PDF_CMAP_SINGLE, 6627 },
	{ 0x717b, 0x717c, PDF_CMAP_RANGE, 15277 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 3320 },
	{ 0x717e, 0x7183, PDF_CMAP_RANGE, 15279 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 3861 },
	{ 0x7185, 0x7189, PDF_CMAP_RANGE, 15285 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 4007 },
	{ 0x718b, 0x718e, PDF_CMAP_RANGE, 15290 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 4053 },
	{ 0x7190, 0x7191, PDF_CMAP_RANGE, 15294 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 8744 },
	{ 0x7193, 0x7193, PDF_CMAP_SINGLE, 15296 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 3254 },
	{ 0x7195, 0x7196, PDF_CMAP_RANGE, 15297 },
	{ 0x7197, 0x7197, PDF_CMAP_SINGLE, 9431 },
	{ 0x7198, 0x7198, PDF_CMAP_SINGLE, 6628 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 3844 },
	{ 0x719a, 0x719e, PDF_CMAP_RANGE, 15299 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 3459 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 6632 },
	{ 0x71a1, 0x71a7, PDF_CMAP_RANGE, 15304 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 6631 },
	{ 0x71a9, 0x71ab, PDF_CMAP_RANGE, 15311 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 969 },
	{ 0x71ad, 0x71b0, PDF_CMAP_RANGE, 15314 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 8444 },
	{ 0x71b2, 0x71b2, PDF_CMAP_SINGLE, 15318 },
	{ 0x71b3, 0x71b3, PDF_CMAP_SINGLE, 6629 },
	{ 0x71b4, 0x71b4, PDF_CMAP_SINGLE, 15319 },
	{ 0x71b5, 0x71b5, PDF_CMAP_SINGLE, 6630 },
	{ 0x71b6, 0x71b8, PDF_CMAP_RANGE, 15320 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 6642 },
	{ 0x71ba, 0x71bd, PDF_CMAP_RANGE, 15323 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 7815 },
	{ 0x71bf, 0x71c0, PDF_CMAP_RANGE, 15327 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 9432 },
	{ 0x71c2, 0x71c2, PDF_CMAP_SINGLE, 15329 },
	{ 0x71c3, 0x71c3, PDF_CMAP_SINGLE, 3223 },
	{ 0x71c4, 0x71c7, PDF_CMAP_RANGE, 15330 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 7866 },
	{ 0x71c9, 0x71cd, PDF_CMAP_RANGE, 15334 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 2576 },
	{ 0x71cf, 0x71d1, PDF_CMAP_RANGE, 15339 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 8473 },
	{ 0x71d3, 0x71d3, PDF_CMAP_SINGLE, 15342 },
	{ 0x71d4, 0x71d4, PDF_CMAP_SINGLE, 6634 },
	{ 0x71d5, 0x71d5, PDF_CMAP_SINGLE, 4105 },
	{ 0x71d6, 0x71d8, PDF_CMAP_RANGE, 15343 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 8556 },
	{ 0x71da, 0x71db, PDF_CMAP_RANGE, 15346 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 9433 },
	{ 0x71dd, 0x71de, PDF_CMAP_RANGE, 15348 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 8743 },
	{ 0x71e0, 0x71e0, PDF_CMAP_SINGLE, 6633 },
	{ 0x71e1, 0x71e4, PDF_CMAP_RANGE, 15350 },
	{ 0x71e5, 0x71e5, PDF_CMAP_SINGLE, 4402 },
	{ 0x71e6, 0x71e6, PDF_CMAP_SINGLE, 7775 },
	{ 0x71e7, 0x71e7, PDF_CMAP_SINGLE, 6635 },
	{ 0x71e8, 0x71ec, PDF_CMAP_RANGE, 15354 },
	{ 0x71ed, 0x71ed, PDF_CMAP_SINGLE, 8863 },
	{ 0x71ee, 0x71ee, PDF_CMAP_SINGLE, 5014 },
	{ 0x71ef, 0x71f3, PDF_CMAP_RANGE, 15359 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 8029 },
	{ 0x71f5, 0x71f8, PDF_CMAP_RANGE, 15364 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 6636 },
	{ 0x71fa, 0x71fb, PDF_CMAP_RANGE, 15368 },
	{ 0x71fc, 0x71fc, PDF_CMAP_SINGLE, 8124 },
	{ 0x71fd, 0x71fd, PDF_CMAP_SINGLE, 15370 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 9434 },
	{ 0x71ff, 0x71ff, PDF_CMAP_SINGLE, 15371 },
	{ 0x7200, 0x7205, PDF_CMAP_RANGE, 15372 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 1045 },
	{ 0x7207, 0x720c, PDF_CMAP_RANGE, 15378 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 8521 },
	{ 0x720e, 0x720f, PDF_CMAP_RANGE, 15384 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 8262 },
	{ 0x7211, 0x721a, PDF_CMAP_RANGE, 15386 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 8192 },
	{ 0x721c, 0x721c, PDF_CMAP_SINGLE, 15396 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 6637 },
	{ 0x721e, 0x7227, PDF_CMAP_RANGE, 15397 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 6638 },
	{ 0x7229, 0x7229, PDF_CMAP_SINGLE, 15407 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 4611 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 15408 },
	{ 0x722c, 0x722c, PDF_CMAP_SINGLE, 2945 },
	{ 0x722d, 0x722f, PDF_CMAP_RANGE, 15409 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 6508 },
	{ 0x7231, 0x7231, PDF_CMAP_SINGLE, 953 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 8592 },
	{ 0x7233, 0x7234, PDF_CMAP_RANGE, 15412 },
	{ 0x7235, 0x7235, PDF_CMAP_SINGLE, 2339 },
	{ 0x7236, 0x7236, PDF_CMAP_SINGLE, 1715 },
	{ 0x7237, 0x7237, PDF_CMAP_SINGLE, 4150 },
	{ 0x7238, 0x7238, PDF_CMAP_SINGLE, 993 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 1498 },
	{ 0x723a, 0x723a, PDF_CMAP_SINGLE, 8712 },
	{ 0x723b, 0x723b, PDF_CMAP_SINGLE, 4713 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 15414 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 3489 },
	{ 0x723e, 0x723e, PDF_CMAP_SINGLE, 7910 },
	{ 0x723f, 0x723f, PDF_CMAP_SINGLE, 5789 },
	{ 0x7240, 0x7246, PDF_CMAP_RANGE, 15415 },
	{ 0x7247, 0x7247, PDF_CMAP_SINGLE, 3019 },
	{ 0x7248, 0x7248, PDF_CMAP_SINGLE, 1009 },
	{ 0x7249, 0x724b, PDF_CMAP_RANGE, 15422 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 2951 },
	{ 0x724d, 0x724d, PDF_CMAP_SINGLE, 6505 },
	{ 0x724e, 0x7251, PDF_CMAP_RANGE, 15425 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 6506 },
	{ 0x7253, 0x7255, PDF_CMAP_RANGE, 15429 },
	{ 0x7256, 0x7256, PDF_CMAP_SINGLE, 6507 },
	{ 0x7257, 0x7257, PDF_CMAP_SINGLE, 15432 },
	{ 0x7258, 0x7258, PDF_CMAP_SINGLE, 9411 },
	{ 0x7259, 0x7259, PDF_CMAP_SINGLE, 4073 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 15433 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 2916 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 15434 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 6468 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 15435 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 2837 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 15436 },
	{ 0x7261, 0x7261, PDF_CMAP_SINGLE, 2840 },
	{ 0x7262, 0x7262, PDF_CMAP_SINGLE, 2489 },
	{ 0x7263, 0x7265, PDF_CMAP_RANGE, 15437 },
	{ 0x7266, 0x7266, PDF_CMAP_SINGLE, 6469 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 2852 },
	{ 0x7268, 0x7268, PDF_CMAP_SINGLE, 15440 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 3838 },
	{ 0x726a, 0x726d, PDF_CMAP_RANGE, 15441 },
	{ 0x726e, 0x726e, PDF_CMAP_SINGLE, 6466 },
	{ 0x726f, 0x726f, PDF_CMAP_SINGLE, 6470 },
	{ 0x7270, 0x7271, PDF_CMAP_RANGE, 15445 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 3381 },
	{ 0x7273, 0x7274, PDF_CMAP_RANGE, 15447 },
	{ 0x7275, 0x7275, PDF_CMAP_SINGLE, 3104 },
	{ 0x7276, 0x7278, PDF_CMAP_RANGE, 15449 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 3627 },
	{ 0x727a, 0x727a, PDF_CMAP_SINGLE, 3853 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 15452 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 8396 },
	{ 0x727e, 0x727f, PDF_CMAP_RANGE, 6471 },
	{ 0x7280, 0x7280, PDF_CMAP_SINGLE, 3865 },
	{ 0x7281, 0x7281, PDF_CMAP_SINGLE, 2514 },
	{ 0x7282, 0x7283, PDF_CMAP_RANGE, 15454 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 6473 },
	{ 0x7285, 0x7289, PDF_CMAP_RANGE, 15456 },
	{ 0x728a, 0x728a, PDF_CMAP_SINGLE, 1534 },
	{ 0x728b, 0x728b, PDF_CMAP_SINGLE, 6474 },
	{ 0x728c, 0x728c, PDF_CMAP_SINGLE, 15461 },
	{ 0x728d, 0x728d, PDF_CMAP_SINGLE, 6475 },
	{ 0x728e, 0x728e, PDF_CMAP_SINGLE, 15462 },
	{ 0x728f, 0x728f, PDF_CMAP_SINGLE, 6476 },
	{ 0x7290, 0x7291, PDF_CMAP_RANGE, 15463 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 6477 },
	{ 0x7293, 0x7295, PDF_CMAP_RANGE, 15465 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 9024 },
	{ 0x7297, 0x729e, PDF_CMAP_RANGE, 15468 },
	{ 0x729f, 0x729f, PDF_CMAP_SINGLE, 6467 },
	{ 0x72a0, 0x72a1, PDF_CMAP_RANGE, 15476 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 7889 },
	{ 0x72a3, 0x72a6, PDF_CMAP_RANGE, 15478 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 8620 },
	{ 0x72a8, 0x72ab, PDF_CMAP_RANGE, 15482 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 3209 },
	{ 0x72ad, 0x72ad, PDF_CMAP_SINGLE, 5615 },
	{ 0x72ae, 0x72ae, PDF_CMAP_SINGLE, 15486 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 1621 },
	{ 0x72b0, 0x72b0, PDF_CMAP_SINGLE, 5616 },
	{ 0x72b1, 0x72b3, PDF_CMAP_RANGE, 15487 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 5617 },
	{ 0x72b5, 0x72b5, PDF_CMAP_SINGLE, 15490 },
	{ 0x72b6, 0x72b6, PDF_CMAP_SINGLE, 4625 },
	{ 0x72b7, 0x72b8, PDF_CMAP_RANGE, 5618 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 4273 },
	{ 0x72ba, 0x72bf, PDF_CMAP_RANGE, 15491 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 8879 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 5621 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 2430 },
	{ 0x72c3, 0x72c3, PDF_CMAP_SINGLE, 5620 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 1459 },
	{ 0x72c5, 0x72c7, PDF_CMAP_RANGE, 15497 },
	{ 0x72c8, 0x72c8, PDF_CMAP_SINGLE, 1056 },
	{ 0x72c9, 0x72cc, PDF_CMAP_RANGE, 15500 },
	{ 0x72cd, 0x72cd, PDF_CMAP_SINGLE, 5623 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5622 },
	{ 0x72cf, 0x72cf, PDF_CMAP_SINGLE, 15504 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1971 },
	{ 0x72d1, 0x72d1, PDF_CMAP_SINGLE, 15505 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 5624 },
	{ 0x72d3, 0x72d6, PDF_CMAP_RANGE, 15506 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 1808 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 15510 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 2304 },
	{ 0x72da, 0x72dd, PDF_CMAP_RANGE, 15511 },
	{ 0x72de, 0x72de, PDF_CMAP_SINGLE, 2911 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 15515 },
	{ 0x72e0, 0x72e0, PDF_CMAP_SINGLE, 1940 },
	{ 0x72e1, 0x72e1, PDF_CMAP_SINGLE, 2199 },
	{ 0x72e2, 0x72e7, PDF_CMAP_RANGE, 15516 },
	{ 0x72e8, 0x72e8, PDF_CMAP_SINGLE, 5625 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 5627 },
	{ 0x72ea, 0x72eb, PDF_CMAP_RANGE, 15522 },
	{ 0x72ec, 0x72ec, PDF_CMAP_SINGLE, 1535 },
	{ 0x72ed, 0x72ed, PDF_CMAP_SINGLE, 3886 },
	{ 0x72ee, 0x72ee, PDF_CMAP_SINGLE, 3391 },
	{ 0x72ef, 0x72ef, PDF_CMAP_SINGLE, 5626 },
	{ 0x72f0, 0x72f0, PDF_CMAP_SINGLE, 4505 },
	{ 0x72f1, 0x72f1, PDF_CMAP_SINGLE, 4321 },
	{ 0x72f2, 0x72f2, PDF_CMAP_SINGLE, 5628 },
	{ 0x72f3, 0x72f3, PDF_CMAP_SINGLE, 5632 },
	{ 0x72f4, 0x72f4, PDF_CMAP_SINGLE, 5629 },
	{ 0x72f5, 0x72f6, PDF_CMAP_RANGE, 15524 },
	{ 0x72f7, 0x72f7, PDF_CMAP_SINGLE, 5630 },
	{ 0x72f8, 0x72f8, PDF_CMAP_SINGLE, 2517 },
	{ 0x72f9, 0x72f9, PDF_CMAP_SINGLE, 8630 },
	{ 0x72fa, 0x72fb, PDF_CMAP_RANGE, 5634 },
	{ 0x72fc, 0x72fc, PDF_CMAP_SINGLE, 2482 },
	{ 0x72fd, 0x72fd, PDF_CMAP_SINGLE, 7741 },
	{ 0x72fe, 0x72ff, PDF_CMAP_RANGE, 15526 },
	{ 0x7300, 0x7300, PDF_CMAP_SINGLE, 15528 },
	{ 0x7301, 0x7301, PDF_CMAP_SINGLE, 5631 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 15529 },
	{ 0x7303, 0x7303, PDF_CMAP_SINGLE, 5633 },
	{ 0x7304, 0x7309, PDF_CMAP_RANGE, 15530 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 5639 },
	{ 0x730b, 0x730d, PDF_CMAP_RANGE, 15536 },
	{ 0x730e, 0x730e, PDF_CMAP_SINGLE, 2589 },
	{ 0x730f, 0x7312, PDF_CMAP_RANGE, 15539 },
	{ 0x7313, 0x7313, PDF_CMAP_SINGLE, 5637 },
	{ 0x7314, 0x7314, PDF_CMAP_SINGLE, 15543 },
	{ 0x7315, 0x7315, PDF_CMAP_SINGLE, 5642 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 1221 },
	{ 0x7317, 0x7317, PDF_CMAP_SINGLE, 5636 },
	{ 0x7318, 0x731a, PDF_CMAP_RANGE, 15544 },
	{ 0x731b, 0x731b, PDF_CMAP_SINGLE, 2770 },
	{ 0x731c, 0x731c, PDF_CMAP_SINGLE, 1161 },
	{ 0x731d, 0x731d, PDF_CMAP_SINGLE, 5641 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 5640 },
	{ 0x731f, 0x7320, PDF_CMAP_RANGE, 15547 },
	{ 0x7321, 0x7321, PDF_CMAP_SINGLE, 5638 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 5643 },
	{ 0x7323, 0x7324, PDF_CMAP_RANGE, 15549 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 5645 },
	{ 0x7326, 0x7328, PDF_CMAP_RANGE, 15551 },
	{ 0x7329, 0x7329, PDF_CMAP_SINGLE, 3988 },
	{ 0x732a, 0x732a, PDF_CMAP_SINGLE, 4588 },
	{ 0x732b, 0x732b, PDF_CMAP_SINGLE, 2733 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 5646 },
	{ 0x732d, 0x732d, PDF_CMAP_SINGLE, 15554 },
	{ 0x732e, 0x732e, PDF_CMAP_SINGLE, 3908 },
	{ 0x732f, 0x7330, PDF_CMAP_RANGE, 15555 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 5648 },
	{ 0x7332, 0x7333, PDF_CMAP_RANGE, 15557 },
	{ 0x7334, 0x7334, PDF_CMAP_SINGLE, 1958 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 15559 },
	{ 0x7336, 0x7336, PDF_CMAP_SINGLE, 8757 },
	{ 0x7337, 0x7337, PDF_CMAP_SINGLE, 6356 },
	{ 0x7338, 0x7338, PDF_CMAP_SINGLE, 5647 },
	{ 0x7339, 0x7339, PDF_CMAP_SINGLE, 5644 },
	{ 0x733a, 0x733a, PDF_CMAP_SINGLE, 15560 },
	{ 0x733b, 0x733b, PDF_CMAP_SINGLE, 9111 },
	{ 0x733c, 0x733d, PDF_CMAP_RANGE, 15561 },
	{ 0x733e, 0x733e, PDF_CMAP_SINGLE, 1984 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 4342 },
	{ 0x7340, 0x7340, PDF_CMAP_SINGLE, 15563 },
	{ 0x7341, 0x7341, PDF_CMAP_SINGLE, 9109 },
	{ 0x7342, 0x7343, PDF_CMAP_RANGE, 15564 },
	{ 0x7344, 0x7344, PDF_CMAP_SINGLE, 8767 },
	{ 0x7345, 0x7345, PDF_CMAP_SINGLE, 8490 },
	{ 0x7346, 0x734c, PDF_CMAP_RANGE, 15566 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 5650 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 15573 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 5649 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 15575 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 6357 },
	{ 0x7353, 0x7356, PDF_CMAP_RANGE, 15576 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 5651 },
	{ 0x7358, 0x735f, PDF_CMAP_RANGE, 15580 },
	{ 0x7360, 0x7360, PDF_CMAP_SINGLE, 5652 },
	{ 0x7361, 0x7367, PDF_CMAP_RANGE, 15588 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 7890 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 15595 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 9110 },
	{ 0x736b, 0x736b, PDF_CMAP_SINGLE, 9112 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 5653 },
	{ 0x736d, 0x736d, PDF_CMAP_SINGLE, 3572 },
	{ 0x736e, 0x736e, PDF_CMAP_SINGLE, 15596 },
	{ 0x736f, 0x736f, PDF_CMAP_SINGLE, 5654 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 8354 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 15597 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 8036 },
	{ 0x7373, 0x7374, PDF_CMAP_RANGE, 15598 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 8235 },
	{ 0x7376, 0x7376, PDF_CMAP_SINGLE, 15600 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 9108 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 8505 },
	{ 0x7379, 0x7379, PDF_CMAP_SINGLE, 15601 },
	{ 0x737a, 0x737a, PDF_CMAP_SINGLE, 8543 },
	{ 0x737b, 0x737b, PDF_CMAP_SINGLE, 8641 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 9114 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 15602 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 5655 },
	{ 0x737f, 0x737f, PDF_CMAP_SINGLE, 15603 },
	{ 0x7380, 0x7380, PDF_CMAP_SINGLE, 9113 },
	{ 0x7381, 0x7383, PDF_CMAP_RANGE, 15604 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 4041 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 15607 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 2673 },
	{ 0x7388, 0x7388, PDF_CMAP_SINGLE, 15609 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 4310 },
	{ 0x738a, 0x738a, PDF_CMAP_SINGLE, 15610 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 3750 },
	{ 0x738c, 0x738d, PDF_CMAP_RANGE, 15611 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6168 },
	{ 0x738f, 0x7390, PDF_CMAP_RANGE, 15613 },
	{ 0x7391, 0x7391, PDF_CMAP_SINGLE, 6169 },
	{ 0x7392, 0x7395, PDF_CMAP_RANGE, 15615 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 2288 },
	{ 0x7397, 0x739a, PDF_CMAP_RANGE, 15619 },
	{ 0x739b, 0x739b, PDF_CMAP_SINGLE, 2705 },
	{ 0x739c, 0x739e, PDF_CMAP_RANGE, 15623 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 6172 },
	{ 0x73a0, 0x73a1, PDF_CMAP_RANGE, 15626 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 6171 },
	{ 0x73a3, 0x73a8, PDF_CMAP_RANGE, 15628 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 3735 },
	{ 0x73aa, 0x73aa, PDF_CMAP_SINGLE, 15634 },
	{ 0x73ab, 0x73ab, PDF_CMAP_SINGLE, 2746 },
	{ 0x73ac, 0x73ad, PDF_CMAP_RANGE, 15635 },
	{ 0x73ae, 0x73ae, PDF_CMAP_SINGLE, 6170 },
	{ 0x73af, 0x73af, PDF_CMAP_SINGLE, 1996 },
	{ 0x73b0, 0x73b0, PDF_CMAP_SINGLE, 3907 },
	{ 0x73b1, 0x73b1, PDF_CMAP_SINGLE, 15637 },
	{ 0x73b2, 0x73b2, PDF_CMAP_SINGLE, 2602 },
	{ 0x73b3, 0x73b3, PDF_CMAP_SINGLE, 6177 },
	{ 0x73b4, 0x73b6, PDF_CMAP_RANGE, 15638 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 6176 },
	{ 0x73b8, 0x73b9, PDF_CMAP_RANGE, 15641 },
	{ 0x73ba, 0x73ba, PDF_CMAP_SINGLE, 6188 },
	{ 0x73bb, 0x73bb, PDF_CMAP_SINGLE, 1130 },
	{ 0x73bc, 0x73bf, PDF_CMAP_RANGE, 15643 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 6178 },
	{ 0x73c1, 0x73c1, PDF_CMAP_SINGLE, 15647 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 6174 },
	{ 0x73c3, 0x73c7, PDF_CMAP_RANGE, 15648 },
	{ 0x73c8, 0x73c8, PDF_CMAP_SINGLE, 6180 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 6179 },
	{ 0x73ca, 0x73ca, PDF_CMAP_SINGLE, 3315 },
	{ 0x73cb, 0x73cc, PDF_CMAP_RANGE, 15653 },
	{ 0x73cd, 0x73cd, PDF_CMAP_SINGLE, 4485 },
	{ 0x73ce, 0x73ce, PDF_CMAP_SINGLE, 15655 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 6173 },
	{ 0x73d0, 0x73d0, PDF_CMAP_SINGLE, 1606 },
	{ 0x73d1, 0x73d1, PDF_CMAP_SINGLE, 6175 },
	{ 0x73d2, 0x73d8, PDF_CMAP_RANGE, 15656 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 6182 },
	{ 0x73da, 0x73dd, PDF_CMAP_RANGE, 15663 },
	{ 0x73de, 0x73de, PDF_CMAP_SINGLE, 6187 },
	{ 0x73df, 0x73df, PDF_CMAP_SINGLE, 15667 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 4584 },
	{ 0x73e1, 0x73e4, PDF_CMAP_RANGE, 15668 },
	{ 0x73e5, 0x73e5, PDF_CMAP_SINGLE, 6181 },
	{ 0x73e6, 0x73e6, PDF_CMAP_SINGLE, 15672 },
	{ 0x73e7, 0x73e7, PDF_CMAP_SINGLE, 6186 },
	{ 0x73e8, 0x73e8, PDF_CMAP_SINGLE, 15673 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 6185 },
	{ 0x73ea, 0x73ec, PDF_CMAP_RANGE, 15674 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 1003 },
	{ 0x73ee, 0x73f1, PDF_CMAP_RANGE, 15677 },
	{ 0x73f2, 0x73f2, PDF_CMAP_SINGLE, 6189 },
	{ 0x73f3, 0x73fd, PDF_CMAP_RANGE, 15681 },
	{ 0x73fe, 0x73fe, PDF_CMAP_SINGLE, 8640 },
	{ 0x73ff, 0x73ff, PDF_CMAP_SINGLE, 15692 },
	{ 0x7400, 0x7402, PDF_CMAP_RANGE, 15693 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 3183 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 15696 },
	{ 0x7405, 0x7405, PDF_CMAP_SINGLE, 2480 },
	{ 0x7406, 0x7406, PDF_CMAP_SINGLE, 2520 },
	{ 0x7407, 0x7408, PDF_CMAP_RANGE, 15697 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 2617 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 6184 },
	{ 0x740b, 0x740e, PDF_CMAP_RANGE, 15699 },
	{ 0x740f, 0x740f, PDF_CMAP_SINGLE, 6190 },
	{ 0x7410, 0x7410, PDF_CMAP_SINGLE, 3563 },
	{ 0x7411, 0x7419, PDF_CMAP_RANGE, 15703 },
	{ 0x741a, 0x741a, PDF_CMAP_SINGLE, 6200 },
	{ 0x741b, 0x741b, PDF_CMAP_SINGLE, 6199 },
	{ 0x741c, 0x7421, PDF_CMAP_RANGE, 15712 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 4638 },
	{ 0x7423, 0x7424, PDF_CMAP_RANGE, 15718 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 6194 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 6193 },
	{ 0x7427, 0x7427, PDF_CMAP_SINGLE, 15720 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 6195 },
	{ 0x7429, 0x7429, PDF_CMAP_SINGLE, 15721 },
	{ 0x742a, 0x742a, PDF_CMAP_SINGLE, 6191 },
	{ 0x742b, 0x742b, PDF_CMAP_SINGLE, 15722 },
	{ 0x742c, 0x742c, PDF_CMAP_SINGLE, 6198 },
	{ 0x742d, 0x742d, PDF_CMAP_SINGLE, 15723 },
	{ 0x742e, 0x742e, PDF_CMAP_SINGLE, 6197 },
	{ 0x742f, 0x742f, PDF_CMAP_SINGLE, 15724 },
	{ 0x7430, 0x7430, PDF_CMAP_SINGLE, 6196 },
	{ 0x7431, 0x7432, PDF_CMAP_RANGE, 15725 },
	{ 0x7433, 0x7433, PDF_CMAP_SINGLE, 2590 },
	{ 0x7434, 0x7434, PDF_CMAP_SINGLE, 3158 },
	{ 0x7435, 0x7435, PDF_CMAP_SINGLE, 3006 },
	{ 0x7436, 0x7436, PDF_CMAP_SINGLE, 2948 },
	{ 0x7437, 0x743b, PDF_CMAP_RANGE, 15727 },
	{ 0x743c, 0x743c, PDF_CMAP_SINGLE, 3178 },
	{ 0x743d, 0x743e, PDF_CMAP_RANGE, 15732 },
	{ 0x743f, 0x743f, PDF_CMAP_SINGLE, 9312 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 15734 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 6201 },
	{ 0x7442, 0x744a, PDF_CMAP_RANGE, 15735 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 9308 },
	{ 0x744c, 0x7454, PDF_CMAP_RANGE, 15744 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 6204 },
	{ 0x7456, 0x7456, PDF_CMAP_SINGLE, 15753 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 6203 },
	{ 0x7458, 0x7458, PDF_CMAP_SINGLE, 15754 },
	{ 0x7459, 0x7459, PDF_CMAP_SINGLE, 6205 },
	{ 0x745a, 0x745a, PDF_CMAP_SINGLE, 1966 },
	{ 0x745b, 0x745b, PDF_CMAP_SINGLE, 6192 },
	{ 0x745c, 0x745c, PDF_CMAP_SINGLE, 6202 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 15755 },
	{ 0x745e, 0x745e, PDF_CMAP_SINGLE, 3275 },
	{ 0x745f, 0x745f, PDF_CMAP_SINGLE, 3299 },
	{ 0x7460, 0x7462, PDF_CMAP_RANGE, 15756 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 8541 },
	{ 0x7464, 0x7468, PDF_CMAP_RANGE, 15759 },
	{ 0x7469, 0x7469, PDF_CMAP_SINGLE, 8741 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 8298 },
	{ 0x746b, 0x746c, PDF_CMAP_RANGE, 15764 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 6207 },
	{ 0x746e, 0x746f, PDF_CMAP_RANGE, 15766 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 1854 },
	{ 0x7471, 0x7475, PDF_CMAP_RANGE, 15768 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 4135 },
	{ 0x7477, 0x7477, PDF_CMAP_SINGLE, 6206 },
	{ 0x7478, 0x747d, PDF_CMAP_RANGE, 15773 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 6208 },
	{ 0x747f, 0x747f, PDF_CMAP_SINGLE, 15779 },
	{ 0x7480, 0x7481, PDF_CMAP_RANGE, 6211 },
	{ 0x7482, 0x7482, PDF_CMAP_SINGLE, 15780 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 2544 },
	{ 0x7484, 0x7486, PDF_CMAP_RANGE, 15781 },
	{ 0x7487, 0x7487, PDF_CMAP_SINGLE, 6213 },
	{ 0x7488, 0x7488, PDF_CMAP_SINGLE, 15784 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 9313 },
	{ 0x748a, 0x748a, PDF_CMAP_SINGLE, 15785 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 6214 },
	{ 0x748c, 0x748d, PDF_CMAP_RANGE, 15786 },
	{ 0x748e, 0x748e, PDF_CMAP_SINGLE, 6210 },
	{ 0x748f, 0x748f, PDF_CMAP_SINGLE, 15788 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 6218 },
	{ 0x7491, 0x749b, PDF_CMAP_RANGE, 15789 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 6209 },
	{ 0x749d, 0x749d, PDF_CMAP_SINGLE, 15800 },
	{ 0x749e, 0x749e, PDF_CMAP_SINGLE, 6215 },
	{ 0x749f, 0x74a2, PDF_CMAP_RANGE, 15801 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 9307 },
	{ 0x74a4, 0x74a5, PDF_CMAP_RANGE, 15805 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 9314 },
	{ 0x74a7, 0x74a7, PDF_CMAP_SINGLE, 6219 },
	{ 0x74a8, 0x74a9, PDF_CMAP_RANGE, 6216 },
	{ 0x74aa, 0x74af, PDF_CMAP_RANGE, 15807 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 8020 },
	{ 0x74b1, 0x74b9, PDF_CMAP_RANGE, 15813 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 6221 },
	{ 0x74bb, 0x74bc, PDF_CMAP_RANGE, 15822 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 9311 },
	{ 0x74be, 0x74c9, PDF_CMAP_RANGE, 15824 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 8428 },
	{ 0x74cb, 0x74ce, PDF_CMAP_RANGE, 15836 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 9309 },
	{ 0x74d0, 0x74d1, PDF_CMAP_RANGE, 15840 },
	{ 0x74d2, 0x74d2, PDF_CMAP_SINGLE, 6220 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 15842 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 9315 },
	{ 0x74d5, 0x74d9, PDF_CMAP_RANGE, 15843 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 9316 },
	{ 0x74db, 0x74db, PDF_CMAP_SINGLE, 15848 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 1832 },
	{ 0x74dd, 0x74dd, PDF_CMAP_SINGLE, 15849 },
	{ 0x74de, 0x74de, PDF_CMAP_SINGLE, 6962 },
	{ 0x74df, 0x74df, PDF_CMAP_SINGLE, 15850 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 6963 },
	{ 0x74e1, 0x74e1, PDF_CMAP_SINGLE, 15851 },
	{ 0x74e2, 0x74e2, PDF_CMAP_SINGLE, 3023 },
	{ 0x74e3, 0x74e3, PDF_CMAP_SINGLE, 1013 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 3226 },
	{ 0x74e5, 0x74e5, PDF_CMAP_SINGLE, 15852 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 3728 },
	{ 0x74e7, 0x74ed, PDF_CMAP_RANGE, 15853 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 3804 },
	{ 0x74ef, 0x74ef, PDF_CMAP_SINGLE, 6403 },
	{ 0x74f0, 0x74f3, PDF_CMAP_RANGE, 15860 },
	{ 0x74f4, 0x74f4, PDF_CMAP_SINGLE, 6404 },
	{ 0x74f5, 0x74f5, PDF_CMAP_SINGLE, 15864 },
	{ 0x74f6, 0x74f6, PDF_CMAP_SINGLE, 3038 },
	{ 0x74f7, 0x74f7, PDF_CMAP_SINGLE, 1356 },
	{ 0x74f8, 0x74fe, PDF_CMAP_RANGE, 15865 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 6405 },
	{ 0x7500, 0x7503, PDF_CMAP_RANGE, 15872 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 4488 },
	{ 0x7505, 0x750b, PDF_CMAP_RANGE, 15876 },
	{ 0x750c, 0x750c, PDF_CMAP_SINGLE, 9381 },
	{ 0x750d, 0x750d, PDF_CMAP_SINGLE, 5253 },
	{ 0x750e, 0x750e, PDF_CMAP_SINGLE, 15883 },
	{ 0x750f, 0x750f, PDF_CMAP_SINGLE, 6406 },
	{ 0x7510, 0x7510, PDF_CMAP_SINGLE, 15884 },
	{ 0x7511, 0x7511, PDF_CMAP_SINGLE, 6407 },
	{ 0x7512, 0x7512, PDF_CMAP_SINGLE, 15885 },
	{ 0x7513, 0x7513, PDF_CMAP_SINGLE, 6408 },
	{ 0x7514, 0x7517, PDF_CMAP_RANGE, 15886 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 1733 },
	{ 0x7519, 0x7519, PDF_CMAP_SINGLE, 5368 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 3374 },
	{ 0x751b, 0x751b, PDF_CMAP_SINGLE, 15890 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 3651 },
	{ 0x751d, 0x751e, PDF_CMAP_RANGE, 15891 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 3379 },
	{ 0x7520, 0x7522, PDF_CMAP_RANGE, 15893 },
	{ 0x7523, 0x7523, PDF_CMAP_SINGLE, 7792 },
	{ 0x7524, 0x7524, PDF_CMAP_SINGLE, 15896 },
	{ 0x7525, 0x7525, PDF_CMAP_SINGLE, 3380 },
	{ 0x7526, 0x7527, PDF_CMAP_RANGE, 15897 },
	{ 0x7528, 0x7528, PDF_CMAP_SINGLE, 4264 },
	{ 0x7529, 0x7529, PDF_CMAP_SINGLE, 3483 },
	{ 0x752a, 0x752a, PDF_CMAP_SINGLE, 15899 },
	{ 0x752b, 0x752b, PDF_CMAP_SINGLE, 1697 },
	{ 0x752c, 0x752c, PDF_CMAP_SINGLE, 6964 },
	{ 0x752d, 0x752d, PDF_CMAP_SINGLE, 1067 },
	{ 0x752e, 0x752e, PDF_CMAP_SINGLE, 15900 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 5940 },
	{ 0x7530, 0x7530, PDF_CMAP_SINGLE, 3650 },
	{ 0x7531, 0x7531, PDF_CMAP_SINGLE, 4270 },
	{ 0x7532, 0x7532, PDF_CMAP_SINGLE, 2122 },
	{ 0x7533, 0x7533, PDF_CMAP_SINGLE, 3363 },
	{ 0x7534, 0x7534, PDF_CMAP_SINGLE, 15901 },
	{ 0x7535, 0x7535, PDF_CMAP_SINGLE, 1480 },
	{ 0x7536, 0x7536, PDF_CMAP_SINGLE, 15902 },
	{ 0x7537, 0x7537, PDF_CMAP_SINGLE, 2867 },
	{ 0x7538, 0x7538, PDF_CMAP_SINGLE, 1482 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 15903 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 6776 },
	{ 0x753b, 0x753b, PDF_CMAP_SINGLE, 1986 },
	{ 0x753c, 0x753d, PDF_CMAP_RANGE, 15904 },
	{ 0x753e, 0x753e, PDF_CMAP_SINGLE, 6166 },
	{ 0x753f, 0x753f, PDF_CMAP_SINGLE, 15906 },
	{ 0x7540, 0x7540, PDF_CMAP_SINGLE, 6777 },
	{ 0x7541, 0x7544, PDF_CMAP_RANGE, 15907 },
	{ 0x7545, 0x7545, PDF_CMAP_SINGLE, 1230 },
	{ 0x7546, 0x7547, PDF_CMAP_RANGE, 15911 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 6780 },
	{ 0x7549, 0x754a, PDF_CMAP_RANGE, 15913 },
	{ 0x754b, 0x754b, PDF_CMAP_SINGLE, 6779 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 2232 },
	{ 0x754d, 0x754d, PDF_CMAP_SINGLE, 15915 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 6778 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 3782 },
	{ 0x7550, 0x7553, PDF_CMAP_RANGE, 15916 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 2960 },
	{ 0x7555, 0x7558, PDF_CMAP_RANGE, 15920 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 2621 },
	{ 0x755a, 0x755a, PDF_CMAP_SINGLE, 5022 },
	{ 0x755b, 0x755b, PDF_CMAP_SINGLE, 6781 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 4030 },
	{ 0x755d, 0x755d, PDF_CMAP_SINGLE, 8335 },
	{ 0x755e, 0x7561, PDF_CMAP_RANGE, 15924 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 7746 },
	{ 0x7563, 0x7564, PDF_CMAP_RANGE, 15928 },
	{ 0x7565, 0x7565, PDF_CMAP_SINGLE, 2683 },
	{ 0x7566, 0x7566, PDF_CMAP_SINGLE, 3079 },
	{ 0x7567, 0x7569, PDF_CMAP_RANGE, 15930 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 1609 },
	{ 0x756b, 0x756b, PDF_CMAP_SINGLE, 8014 },
	{ 0x756c, 0x7571, PDF_CMAP_RANGE, 15933 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 6782 },
	{ 0x7573, 0x7573, PDF_CMAP_SINGLE, 15939 },
	{ 0x7574, 0x7574, PDF_CMAP_SINGLE, 1296 },
	{ 0x7575, 0x7575, PDF_CMAP_SINGLE, 15940 },
	{ 0x7576, 0x7576, PDF_CMAP_SINGLE, 7857 },
	{ 0x7577, 0x7577, PDF_CMAP_SINGLE, 15941 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 2064 },
	{ 0x7579, 0x7579, PDF_CMAP_SINGLE, 6783 },
	{ 0x757a, 0x757e, PDF_CMAP_RANGE, 15942 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 6164 },
	{ 0x7580, 0x7582, PDF_CMAP_RANGE, 15947 },
	{ 0x7583, 0x7583, PDF_CMAP_SINGLE, 6784 },
	{ 0x7584, 0x7585, PDF_CMAP_RANGE, 15950 },
	{ 0x7586, 0x7586, PDF_CMAP_SINGLE, 2175 },
	{ 0x7587, 0x7587, PDF_CMAP_SINGLE, 7819 },
	{ 0x7588, 0x758a, PDF_CMAP_RANGE, 15952 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 7110 },
	{ 0x758c, 0x758e, PDF_CMAP_RANGE, 15955 },
	{ 0x758f, 0x758f, PDF_CMAP_SINGLE, 3455 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 15958 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 4176 },
	{ 0x7592, 0x7592, PDF_CMAP_SINGLE, 7008 },
	{ 0x7593, 0x7593, PDF_CMAP_SINGLE, 15959 },
	{ 0x7594, 0x7594, PDF_CMAP_SINGLE, 7009 },
	{ 0x7595, 0x7595, PDF_CMAP_SINGLE, 15960 },
	{ 0x7596, 0x7596, PDF_CMAP_SINGLE, 7010 },
	{ 0x7597, 0x7597, PDF_CMAP_SINGLE, 2575 },
	{ 0x7598, 0x7598, PDF_CMAP_SINGLE, 15961 },
	{ 0x7599, 0x7599, PDF_CMAP_SINGLE, 1768 },
	{ 0x759a, 0x759a, PDF_CMAP_SINGLE, 2301 },
	{ 0x759b, 0x759c, PDF_CMAP_RANGE, 15962 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 7012 },
	{ 0x759e, 0x759e, PDF_CMAP_SINGLE, 15964 },
	{ 0x759f, 0x759f, PDF_CMAP_SINGLE, 2930 },
	{ 0x75a0, 0x75a0, PDF_CMAP_SINGLE, 7011 },
	{ 0x75a1, 0x75a1, PDF_CMAP_SINGLE, 4122 },
	{ 0x75a2, 0x75a2, PDF_CMAP_SINGLE, 15965 },
	{ 0x75a3, 0x75a3, PDF_CMAP_SINGLE, 7014 },
	{ 0x75a4, 0x75a4, PDF_CMAP_SINGLE, 983 },
	{ 0x75a5, 0x75a5, PDF_CMAP_SINGLE, 2235 },
	{ 0x75a6, 0x75aa, PDF_CMAP_RANGE, 15966 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 4198 },
	{ 0x75ac, 0x75ac, PDF_CMAP_SINGLE, 7013 },
	{ 0x75ad, 0x75ad, PDF_CMAP_SINGLE, 15971 },
	{ 0x75ae, 0x75ae, PDF_CMAP_SINGLE, 1330 },
	{ 0x75af, 0x75af, PDF_CMAP_SINGLE, 1669 },
	{ 0x75b0, 0x75b0, PDF_CMAP_SINGLE, 7020 },
	{ 0x75b1, 0x75b1, PDF_CMAP_SINGLE, 7019 },
	{ 0x75b2, 0x75b2, PDF_CMAP_SINGLE, 3010 },
	{ 0x75b3, 0x75b4, PDF_CMAP_RANGE, 7015 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 1350 },
	{ 0x75b6, 0x75b7, PDF_CMAP_RANGE, 15972 },
	{ 0x75b8, 0x75b8, PDF_CMAP_SINGLE, 7017 },
	{ 0x75b9, 0x75b9, PDF_CMAP_SINGLE, 4495 },
	{ 0x75ba, 0x75bb, PDF_CMAP_RANGE, 15974 },
	{ 0x75bc, 0x75bc, PDF_CMAP_SINGLE, 3630 },
	{ 0x75bd, 0x75bd, PDF_CMAP_SINGLE, 2305 },
	{ 0x75be, 0x75be, PDF_CMAP_SINGLE, 2085 },
	{ 0x75bf, 0x75c1, PDF_CMAP_RANGE, 15976 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 7022 },
	{ 0x75c3, 0x75c3, PDF_CMAP_SINGLE, 7021 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 7018 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 1128 },
	{ 0x75c6, 0x75c6, PDF_CMAP_SINGLE, 15979 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 4513 },
	{ 0x75c8, 0x75c8, PDF_CMAP_SINGLE, 4253 },
	{ 0x75c9, 0x75c9, PDF_CMAP_SINGLE, 2278 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 3207 },
	{ 0x75cb, 0x75cc, PDF_CMAP_RANGE, 15980 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 7024 },
	{ 0x75ce, 0x75d1, PDF_CMAP_RANGE, 15982 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 4128 },
	{ 0x75d3, 0x75d3, PDF_CMAP_SINGLE, 15986 },
	{ 0x75d4, 0x75d4, PDF_CMAP_SINGLE, 4555 },
	{ 0x75d5, 0x75d5, PDF_CMAP_SINGLE, 1938 },
	{ 0x75d6, 0x75d6, PDF_CMAP_SINGLE, 7023 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 15987 },
	{ 0x75d8, 0x75d8, PDF_CMAP_SINGLE, 1530 },
	{ 0x75d9, 0x75d9, PDF_CMAP_SINGLE, 8134 },
	{ 0x75da, 0x75da, PDF_CMAP_SINGLE, 15988 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 3685 },
	{ 0x75dc, 0x75dd, PDF_CMAP_RANGE, 15989 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 3013 },
	{ 0x75df, 0x75e1, PDF_CMAP_RANGE, 15991 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 2538 },
	{ 0x75e3, 0x75e3, PDF_CMAP_SINGLE, 7025 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 7028 },
	{ 0x75e5, 0x75e5, PDF_CMAP_SINGLE, 15994 },
	{ 0x75e6, 0x75e6, PDF_CMAP_SINGLE, 7027 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 7030 },
	{ 0x75e8, 0x75e8, PDF_CMAP_SINGLE, 7026 },
	{ 0x75e9, 0x75e9, PDF_CMAP_SINGLE, 15995 },
	{ 0x75ea, 0x75ea, PDF_CMAP_SINGLE, 2003 },
	{ 0x75eb, 0x75eb, PDF_CMAP_SINGLE, 7029 },
	{ 0x75ec, 0x75ef, PDF_CMAP_RANGE, 15996 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 3592 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 7032 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 16000 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 1274 },
	{ 0x75f5, 0x75f8, PDF_CMAP_RANGE, 16002 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 1085 },
	{ 0x75fa, 0x75fb, PDF_CMAP_RANGE, 16006 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 7033 },
	{ 0x75fd, 0x75fe, PDF_CMAP_RANGE, 16008 },
	{ 0x75ff, 0x75ff, PDF_CMAP_SINGLE, 7034 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 7036 },
	{ 0x7601, 0x7601, PDF_CMAP_SINGLE, 1380 },
	{ 0x7602, 0x7602, PDF_CMAP_SINGLE, 9646 },
	{ 0x7603, 0x7603, PDF_CMAP_SINGLE, 7031 },
	{ 0x7604, 0x7604, PDF_CMAP_SINGLE, 16010 },
	{ 0x7605, 0x7605, PDF_CMAP_SINGLE, 7037 },
	{ 0x7606, 0x7609, PDF_CMAP_RANGE, 16011 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 7040 },
	{ 0x760b, 0x760b, PDF_CMAP_SINGLE, 7937 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 7038 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 8704 },
	{ 0x760e, 0x760f, PDF_CMAP_RANGE, 16015 },
	{ 0x7610, 0x7610, PDF_CMAP_SINGLE, 7035 },
	{ 0x7611, 0x7614, PDF_CMAP_RANGE, 16017 },
	{ 0x7615, 0x7615, PDF_CMAP_SINGLE, 7043 },
	{ 0x7616, 0x7616, PDF_CMAP_SINGLE, 16021 },
	{ 0x7617, 0x7617, PDF_CMAP_SINGLE, 7039 },
	{ 0x7618, 0x7618, PDF_CMAP_SINGLE, 7042 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 7044 },
	{ 0x761a, 0x761a, PDF_CMAP_SINGLE, 16022 },
	{ 0x761b, 0x761b, PDF_CMAP_SINGLE, 7045 },
	{ 0x761c, 0x761d, PDF_CMAP_RANGE, 16023 },
	{ 0x761e, 0x761e, PDF_CMAP_SINGLE, 9650 },
	{ 0x761f, 0x761f, PDF_CMAP_SINGLE, 3792 },
	{ 0x7620, 0x7620, PDF_CMAP_SINGLE, 7048 },
	{ 0x7621, 0x7621, PDF_CMAP_SINGLE, 7831 },
	{ 0x7622, 0x7622, PDF_CMAP_SINGLE, 7047 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 16025 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 2623 },
	{ 0x7625, 0x7625, PDF_CMAP_SINGLE, 7041 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 3444 },
	{ 0x7627, 0x7627, PDF_CMAP_SINGLE, 8363 },
	{ 0x7628, 0x7628, PDF_CMAP_SINGLE, 16026 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 1396 },
	{ 0x762a, 0x762a, PDF_CMAP_SINGLE, 1114 },
	{ 0x762b, 0x762b, PDF_CMAP_SINGLE, 3588 },
	{ 0x762c, 0x762c, PDF_CMAP_SINGLE, 16027 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 7050 },
	{ 0x762e, 0x762f, PDF_CMAP_RANGE, 16028 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 7051 },
	{ 0x7631, 0x7632, PDF_CMAP_RANGE, 16030 },
	{ 0x7633, 0x7633, PDF_CMAP_SINGLE, 7056 },
	{ 0x7634, 0x7634, PDF_CMAP_SINGLE, 4463 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 7053 },
	{ 0x7636, 0x7637, PDF_CMAP_RANGE, 16032 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 3214 },
	{ 0x7639, 0x763a, PDF_CMAP_RANGE, 16034 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 9651 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 7046 },
	{ 0x763d, 0x763d, PDF_CMAP_SINGLE, 16036 },
	{ 0x763e, 0x763e, PDF_CMAP_SINGLE, 7055 },
	{ 0x763f, 0x763f, PDF_CMAP_SINGLE, 7052 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 7049 },
	{ 0x7641, 0x7641, PDF_CMAP_SINGLE, 16037 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 8232 },
	{ 0x7643, 0x7643, PDF_CMAP_SINGLE, 7054 },
	{ 0x7644, 0x7645, PDF_CMAP_RANGE, 16038 },
	{ 0x7646, 0x7647, PDF_CMAP_RANGE, 9647 },
	{ 0x7648, 0x7648, PDF_CMAP_SINGLE, 16040 },
	{ 0x7649, 0x7649, PDF_CMAP_SINGLE, 9649 },
	{ 0x764a, 0x764b, PDF_CMAP_RANGE, 16041 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 948 },
	{ 0x764d, 0x764d, PDF_CMAP_SINGLE, 7057 },
	{ 0x764e, 0x7653, PDF_CMAP_RANGE, 16043 },
	{ 0x7654, 0x7654, PDF_CMAP_SINGLE, 7059 },
	{ 0x7655, 0x7655, PDF_CMAP_SINGLE, 16049 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 7061 },
	{ 0x7657, 0x7657, PDF_CMAP_SINGLE, 16050 },
	{ 0x7658, 0x7658, PDF_CMAP_SINGLE, 9644 },
	{ 0x7659, 0x765b, PDF_CMAP_RANGE, 16051 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 7060 },
	{ 0x765d, 0x765d, PDF_CMAP_SINGLE, 16054 },
	{ 0x765e, 0x765e, PDF_CMAP_SINGLE, 7058 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 7758 },
	{ 0x7660, 0x7661, PDF_CMAP_RANGE, 16055 },
	{ 0x7662, 0x7662, PDF_CMAP_SINGLE, 8706 },
	{ 0x7663, 0x7663, PDF_CMAP_SINGLE, 4043 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 9643 },
	{ 0x7665, 0x7665, PDF_CMAP_SINGLE, 8838 },
	{ 0x7666, 0x7666, PDF_CMAP_SINGLE, 16057 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 9645 },
	{ 0x7668, 0x7668, PDF_CMAP_SINGLE, 16058 },
	{ 0x7669, 0x7669, PDF_CMAP_SINGLE, 9654 },
	{ 0x766a, 0x766a, PDF_CMAP_SINGLE, 16059 },
	{ 0x766b, 0x766b, PDF_CMAP_SINGLE, 7062 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 8675 },
	{ 0x766d, 0x766e, PDF_CMAP_RANGE, 9652 },
	{ 0x766f, 0x766f, PDF_CMAP_SINGLE, 7063 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 8751 },
	{ 0x7671, 0x7671, PDF_CMAP_SINGLE, 8549 },
	{ 0x7672, 0x7672, PDF_CMAP_SINGLE, 9655 },
	{ 0x7673, 0x7677, PDF_CMAP_RANGE, 16060 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 1864 },
	{ 0x7679, 0x767a, PDF_CMAP_RANGE, 16065 },
	{ 0x767b, 0x767b, PDF_CMAP_SINGLE, 1448 },
	{ 0x767c, 0x767c, PDF_CMAP_SINGLE, 7913 },
	{ 0x767d, 0x767d, PDF_CMAP_SINGLE, 994 },
	{ 0x767e, 0x767e, PDF_CMAP_SINGLE, 996 },
	{ 0x767f, 0x7681, PDF_CMAP_RANGE, 16067 },
	{ 0x7682, 0x7682, PDF_CMAP_SINGLE, 4400 },
	{ 0x7683, 0x7683, PDF_CMAP_SINGLE, 16070 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 1445 },
	{ 0x7685, 0x7685, PDF_CMAP_SINGLE, 16071 },
	{ 0x7686, 0x7686, PDF_CMAP_SINGLE, 2213 },
	{ 0x7687, 0x7687, PDF_CMAP_SINGLE, 2015 },
	{ 0x7688, 0x7688, PDF_CMAP_SINGLE, 6957 },
	{ 0x7689, 0x768a, PDF_CMAP_RANGE, 16072 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 1753 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 16074 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 6958 },
	{ 0x768f, 0x7690, PDF_CMAP_RANGE, 16076 },
	{ 0x7691, 0x7691, PDF_CMAP_SINGLE, 947 },
	{ 0x7692, 0x7692, PDF_CMAP_SINGLE, 16078 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 6959 },
	{ 0x7694, 0x7695, PDF_CMAP_RANGE, 16079 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 3743 },
	{ 0x7697, 0x7698, PDF_CMAP_RANGE, 16081 },
	{ 0x7699, 0x7699, PDF_CMAP_SINGLE, 6960 },
	{ 0x769a, 0x769a, PDF_CMAP_SINGLE, 7717 },
	{ 0x769b, 0x76a3, PDF_CMAP_RANGE, 16083 },
	{ 0x76a4, 0x76a4, PDF_CMAP_SINGLE, 6961 },
	{ 0x76a5, 0x76ad, PDF_CMAP_RANGE, 16092 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3011 },
	{ 0x76af, 0x76b0, PDF_CMAP_RANGE, 16101 },
	{ 0x76b1, 0x76b1, PDF_CMAP_SINGLE, 4580 },
	{ 0x76b2, 0x76b2, PDF_CMAP_SINGLE, 7112 },
	{ 0x76b3, 0x76b3, PDF_CMAP_SINGLE, 16103 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 7113 },
	{ 0x76b5, 0x76b7, PDF_CMAP_RANGE, 16104 },
	{ 0x76b8, 0x76b8, PDF_CMAP_SINGLE, 9664 },
	{ 0x76b9, 0x76b9, PDF_CMAP_SINGLE, 16107 },
	{ 0x76ba, 0x76ba, PDF_CMAP_SINGLE, 8858 },
	{ 0x76bb, 0x76be, PDF_CMAP_RANGE, 16108 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2808 },
	{ 0x76c0, 0x76c1, PDF_CMAP_RANGE, 16112 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 4288 },
	{ 0x76c3, 0x76c4, PDF_CMAP_RANGE, 16114 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 4560 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 2985 },
	{ 0x76c7, 0x76c7, PDF_CMAP_SINGLE, 16116 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 4244 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 16117 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 4205 },
	{ 0x76cb, 0x76cc, PDF_CMAP_RANGE, 16118 },
	{ 0x76cd, 0x76cd, PDF_CMAP_SINGLE, 6795 },
	{ 0x76ce, 0x76ce, PDF_CMAP_SINGLE, 966 },
	{ 0x76cf, 0x76cf, PDF_CMAP_SINGLE, 4438 },
	{ 0x76d0, 0x76d0, PDF_CMAP_SINGLE, 4087 },
	{ 0x76d1, 0x76d1, PDF_CMAP_SINGLE, 2131 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 1927 },
	{ 0x76d3, 0x76d3, PDF_CMAP_SINGLE, 16120 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 2437 },
	{ 0x76d5, 0x76d5, PDF_CMAP_SINGLE, 16121 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 1730 },
	{ 0x76d7, 0x76d7, PDF_CMAP_SINGLE, 1442 },
	{ 0x76d8, 0x76d8, PDF_CMAP_SINGLE, 2957 },
	{ 0x76d9, 0x76da, PDF_CMAP_RANGE, 16122 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 3385 },
	{ 0x76dc, 0x76dd, PDF_CMAP_RANGE, 16124 },
	{ 0x76de, 0x76de, PDF_CMAP_SINGLE, 8814 },
	{ 0x76df, 0x76df, PDF_CMAP_SINGLE, 2768 },
	{ 0x76e0, 0x76e0, PDF_CMAP_SINGLE, 16126 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 8125 },
	{ 0x76e2, 0x76e2, PDF_CMAP_SINGLE, 16127 },
	{ 0x76e3, 0x76e3, PDF_CMAP_SINGLE, 8068 },
	{ 0x76e4, 0x76e4, PDF_CMAP_SINGLE, 8370 },
	{ 0x76e5, 0x76e5, PDF_CMAP_SINGLE, 6796 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 16128 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 8259 },
	{ 0x76e8, 0x76ed, PDF_CMAP_RANGE, 16129 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 2850 },
	{ 0x76ef, 0x76ef, PDF_CMAP_SINGLE, 1505 },
	{ 0x76f0, 0x76f0, PDF_CMAP_SINGLE, 16135 },
	{ 0x76f1, 0x76f1, PDF_CMAP_SINGLE, 6744 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 2729 },
	{ 0x76f3, 0x76f3, PDF_CMAP_SINGLE, 16136 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 4528 },
	{ 0x76f5, 0x76f7, PDF_CMAP_RANGE, 16137 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 3917 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 6747 },
	{ 0x76fa, 0x76fb, PDF_CMAP_RANGE, 16140 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 2959 },
	{ 0x76fd, 0x76fd, PDF_CMAP_SINGLE, 16142 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 1563 },
	{ 0x76ff, 0x76ff, PDF_CMAP_SINGLE, 16143 },
	{ 0x7700, 0x7700, PDF_CMAP_SINGLE, 16144 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 3384 },
	{ 0x7702, 0x7703, PDF_CMAP_RANGE, 16145 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 6745 },
	{ 0x7705, 0x7706, PDF_CMAP_RANGE, 16147 },
	{ 0x7707, 0x7708, PDF_CMAP_RANGE, 6748 },
	{ 0x7709, 0x7709, PDF_CMAP_SINGLE, 2753 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 16149 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 2369 },
	{ 0x770c, 0x770c, PDF_CMAP_SINGLE, 16150 },
	{ 0x770d, 0x770d, PDF_CMAP_SINGLE, 6746 },
	{ 0x770e, 0x7718, PDF_CMAP_RANGE, 16151 },
	{ 0x7719, 0x7719, PDF_CMAP_SINGLE, 6752 },
	{ 0x771a, 0x771a, PDF_CMAP_SINGLE, 6750 },
	{ 0x771b, 0x771e, PDF_CMAP_RANGE, 16162 },
	{ 0x771f, 0x771f, PDF_CMAP_SINGLE, 4487 },
	{ 0x7720, 0x7720, PDF_CMAP_SINGLE, 2788 },
	{ 0x7721, 0x7721, PDF_CMAP_SINGLE, 16166 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 6751 },
	{ 0x7723, 0x7725, PDF_CMAP_RANGE, 16167 },
	{ 0x7726, 0x7726, PDF_CMAP_SINGLE, 6754 },
	{ 0x7727, 0x7727, PDF_CMAP_SINGLE, 16170 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 4420 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 4044 },
	{ 0x772a, 0x772c, PDF_CMAP_RANGE, 16171 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 6753 },
	{ 0x772e, 0x772e, PDF_CMAP_SINGLE, 16174 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 2773 },
	{ 0x7730, 0x7734, PDF_CMAP_RANGE, 16175 },
	{ 0x7735, 0x7735, PDF_CMAP_SINGLE, 6755 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 2433 },
	{ 0x7737, 0x7737, PDF_CMAP_SINGLE, 2331 },
	{ 0x7738, 0x7738, PDF_CMAP_SINGLE, 6756 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 16180 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 3658 },
	{ 0x773b, 0x773b, PDF_CMAP_SINGLE, 16181 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 4100 },
	{ 0x773d, 0x773f, PDF_CMAP_RANGE, 16182 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 4642 },
	{ 0x7741, 0x7741, PDF_CMAP_SINGLE, 4503 },
	{ 0x7742, 0x7742, PDF_CMAP_SINGLE, 16185 },
	{ 0x7743, 0x7743, PDF_CMAP_SINGLE, 6760 },
	{ 0x7744, 0x7746, PDF_CMAP_RANGE, 16186 },
	{ 0x7747, 0x7747, PDF_CMAP_SINGLE, 6759 },
	{ 0x7748, 0x774e, PDF_CMAP_RANGE, 16189 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 9860 },
	{ 0x7750, 0x7751, PDF_CMAP_RANGE, 6757 },
	{ 0x7752, 0x7759, PDF_CMAP_RANGE, 16196 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 6761 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 2261 },
	{ 0x775c, 0x775d, PDF_CMAP_RANGE, 16204 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 9455 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 16206 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 3492 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 6763 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 1532 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 16208 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 6764 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 2851 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 16209 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 6762 },
	{ 0x7769, 0x776a, PDF_CMAP_RANGE, 16210 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 2223 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 1166 },
	{ 0x776d, 0x7778, PDF_CMAP_RANGE, 16212 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 1538 },
	{ 0x777a, 0x777c, PDF_CMAP_RANGE, 16224 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 6767 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 4720 },
	{ 0x777f, 0x777f, PDF_CMAP_SINGLE, 6765 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 6768 },
	{ 0x7781, 0x7783, PDF_CMAP_RANGE, 16227 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 2798 },
	{ 0x7785, 0x7785, PDF_CMAP_SINGLE, 1303 },
	{ 0x7786, 0x778b, PDF_CMAP_RANGE, 16230 },
	{ 0x778c, 0x778c, PDF_CMAP_SINGLE, 6769 },
	{ 0x778d, 0x778d, PDF_CMAP_SINGLE, 6766 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 3878 },
	{ 0x778f, 0x7790, PDF_CMAP_RANGE, 16236 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 6770 },
	{ 0x7792, 0x7792, PDF_CMAP_SINGLE, 2718 },
	{ 0x7793, 0x7797, PDF_CMAP_RANGE, 16238 },
	{ 0x7798, 0x7798, PDF_CMAP_SINGLE, 9454 },
	{ 0x7799, 0x779d, PDF_CMAP_RANGE, 16243 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 8308 },
	{ 0x779f, 0x77a0, PDF_CMAP_RANGE, 6771 },
	{ 0x77a1, 0x77a1, PDF_CMAP_SINGLE, 16248 },
	{ 0x77a2, 0x77a2, PDF_CMAP_SINGLE, 5269 },
	{ 0x77a3, 0x77a4, PDF_CMAP_RANGE, 16249 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 3026 },
	{ 0x77a6, 0x77a6, PDF_CMAP_SINGLE, 16251 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 3139 },
	{ 0x77a8, 0x77a8, PDF_CMAP_SINGLE, 16252 },
	{ 0x77a9, 0x77a9, PDF_CMAP_SINGLE, 4596 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 1450 },
	{ 0x77ab, 0x77ab, PDF_CMAP_SINGLE, 16253 },
	{ 0x77ac, 0x77ac, PDF_CMAP_SINGLE, 3495 },
	{ 0x77ad, 0x77ad, PDF_CMAP_SINGLE, 9863 },
	{ 0x77ae, 0x77af, PDF_CMAP_RANGE, 16254 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 6773 },
	{ 0x77b1, 0x77b2, PDF_CMAP_RANGE, 16256 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 3676 },
	{ 0x77b4, 0x77b4, PDF_CMAP_SINGLE, 16258 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 6774 },
	{ 0x77b6, 0x77ba, PDF_CMAP_RANGE, 16259 },
	{ 0x77bb, 0x77bb, PDF_CMAP_SINGLE, 4433 },
	{ 0x77bc, 0x77bc, PDF_CMAP_SINGLE, 9456 },
	{ 0x77bd, 0x77bd, PDF_CMAP_SINGLE, 6775 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 16264 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 7550 },
	{ 0x77c0, 0x77c6, PDF_CMAP_RANGE, 16265 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 9869 },
	{ 0x77c8, 0x77cc, PDF_CMAP_RANGE, 16272 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5015 },
	{ 0x77ce, 0x77d6, PDF_CMAP_RANGE, 16277 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 1318 },
	{ 0x77d8, 0x77d9, PDF_CMAP_RANGE, 16286 },
	{ 0x77da, 0x77da, PDF_CMAP_SINGLE, 8864 },
	{ 0x77db, 0x77db, PDF_CMAP_SINGLE, 2737 },
	{ 0x77dc, 0x77dc, PDF_CMAP_SINGLE, 7114 },
	{ 0x77dd, 0x77e1, PDF_CMAP_RANGE, 16288 },
	{ 0x77e2, 0x77e2, PDF_CMAP_SINGLE, 3407 },
	{ 0x77e3, 0x77e3, PDF_CMAP_SINGLE, 4186 },
	{ 0x77e4, 0x77e4, PDF_CMAP_SINGLE, 16293 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 4521 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 16294 },
	{ 0x77e7, 0x77e7, PDF_CMAP_SINGLE, 6938 },
	{ 0x77e8, 0x77e8, PDF_CMAP_SINGLE, 16295 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 2311 },
	{ 0x77ea, 0x77ea, PDF_CMAP_SINGLE, 16296 },
	{ 0x77eb, 0x77eb, PDF_CMAP_SINGLE, 2196 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 6939 },
	{ 0x77ed, 0x77ed, PDF_CMAP_SINGLE, 1547 },
	{ 0x77ee, 0x77ee, PDF_CMAP_SINGLE, 950 },
	{ 0x77ef, 0x77ef, PDF_CMAP_SINGLE, 8107 },
	{ 0x77f0, 0x77f2, PDF_CMAP_RANGE, 16297 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 3398 },
	{ 0x77f4, 0x77f5, PDF_CMAP_RANGE, 16300 },
	{ 0x77f6, 0x77f6, PDF_CMAP_SINGLE, 6690 },
	{ 0x77f7, 0x77f7, PDF_CMAP_SINGLE, 16302 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 6691 },
	{ 0x77f9, 0x77fc, PDF_CMAP_RANGE, 16303 },
	{ 0x77fd, 0x77fd, PDF_CMAP_SINGLE, 3848 },
	{ 0x77fe, 0x77fe, PDF_CMAP_SINGLE, 1612 },
	{ 0x77ff, 0x77ff, PDF_CMAP_SINGLE, 2432 },
	{ 0x7800, 0x7800, PDF_CMAP_SINGLE, 6692 },
	{ 0x7801, 0x7801, PDF_CMAP_SINGLE, 2706 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 3305 },
	{ 0x7803, 0x7808, PDF_CMAP_RANGE, 16307 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 6693 },
	{ 0x780a, 0x780b, PDF_CMAP_RANGE, 16313 },
	{ 0x780c, 0x780c, PDF_CMAP_SINGLE, 3093 },
	{ 0x780d, 0x780d, PDF_CMAP_SINGLE, 2368 },
	{ 0x780e, 0x7810, PDF_CMAP_RANGE, 16315 },
	{ 0x7811, 0x7811, PDF_CMAP_SINGLE, 6696 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 3001 },
	{ 0x7813, 0x7813, PDF_CMAP_SINGLE, 16318 },
	{ 0x7814, 0x7814, PDF_CMAP_SINGLE, 4089 },
	{ 0x7815, 0x7815, PDF_CMAP_SINGLE, 16319 },
	{ 0x7816, 0x7816, PDF_CMAP_SINGLE, 4614 },
	{ 0x7817, 0x7818, PDF_CMAP_RANGE, 6694 },
	{ 0x7819, 0x7819, PDF_CMAP_SINGLE, 16320 },
	{ 0x781a, 0x781a, PDF_CMAP_SINGLE, 4107 },
	{ 0x781b, 0x781b, PDF_CMAP_SINGLE, 16321 },
	{ 0x781c, 0x781d, PDF_CMAP_RANGE, 6699 },
	{ 0x781e, 0x781e, PDF_CMAP_SINGLE, 16322 },
	{ 0x781f, 0x781f, PDF_CMAP_SINGLE, 6704 },
	{ 0x7820, 0x7822, PDF_CMAP_RANGE, 16323 },
	{ 0x7823, 0x7823, PDF_CMAP_SINGLE, 6708 },
	{ 0x7824, 0x7824, PDF_CMAP_SINGLE, 16326 },
	{ 0x7825, 0x7825, PDF_CMAP_SINGLE, 6706 },
	{ 0x7826, 0x7826, PDF_CMAP_SINGLE, 6714 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 4489 },
	{ 0x7828, 0x7828, PDF_CMAP_SINGLE, 16327 },
	{ 0x7829, 0x7829, PDF_CMAP_SINGLE, 6709 },
	{ 0x782a, 0x782b, PDF_CMAP_RANGE, 16328 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 6707 },
	{ 0x782d, 0x782d, PDF_CMAP_SINGLE, 6698 },
	{ 0x782e, 0x782f, PDF_CMAP_RANGE, 16330 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 2986 },
	{ 0x7831, 0x7833, PDF_CMAP_RANGE, 16332 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 3045 },
	{ 0x7835, 0x7836, PDF_CMAP_RANGE, 16335 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 3362 },
	{ 0x7838, 0x7838, PDF_CMAP_SINGLE, 4373 },
	{ 0x7839, 0x783b, PDF_CMAP_RANGE, 6701 },
	{ 0x783c, 0x783c, PDF_CMAP_SINGLE, 6705 },
	{ 0x783d, 0x783d, PDF_CMAP_SINGLE, 16337 },
	{ 0x783e, 0x783e, PDF_CMAP_SINGLE, 2532 },
	{ 0x783f, 0x783f, PDF_CMAP_SINGLE, 16338 },
	{ 0x7840, 0x7840, PDF_CMAP_SINGLE, 1316 },
	{ 0x7841, 0x7842, PDF_CMAP_RANGE, 16339 },
	{ 0x7843, 0x7843, PDF_CMAP_SINGLE, 9896 },
	{ 0x7844, 0x7844, PDF_CMAP_SINGLE, 16341 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 1857 },
	{ 0x7846, 0x7846, PDF_CMAP_SINGLE, 16342 },
	{ 0x7847, 0x7847, PDF_CMAP_SINGLE, 6716 },
	{ 0x7848, 0x784b, PDF_CMAP_RANGE, 16343 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 6717 },
	{ 0x784d, 0x784d, PDF_CMAP_SINGLE, 16347 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 6710 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 16348 },
	{ 0x7850, 0x7850, PDF_CMAP_SINGLE, 6715 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 16349 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 3847 },
	{ 0x7853, 0x7854, PDF_CMAP_RANGE, 16350 },
	{ 0x7855, 0x7855, PDF_CMAP_SINGLE, 3499 },
	{ 0x7856, 0x7857, PDF_CMAP_RANGE, 6712 },
	{ 0x7858, 0x785c, PDF_CMAP_RANGE, 16352 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 3938 },
	{ 0x785e, 0x7863, PDF_CMAP_RANGE, 16357 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 9449 },
	{ 0x7865, 0x7867, PDF_CMAP_RANGE, 16363 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 9445 },
	{ 0x7869, 0x7869, PDF_CMAP_SINGLE, 16366 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 6718 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 2619 },
	{ 0x786c, 0x786c, PDF_CMAP_SINGLE, 4247 },
	{ 0x786d, 0x786d, PDF_CMAP_SINGLE, 6711 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 3218 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 8698 },
	{ 0x7870, 0x7876, PDF_CMAP_RANGE, 16367 },
	{ 0x7877, 0x7877, PDF_CMAP_SINGLE, 2146 },
	{ 0x7878, 0x787b, PDF_CMAP_RANGE, 16374 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 2993 },
	{ 0x787d, 0x7886, PDF_CMAP_RANGE, 16378 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 6722 },
	{ 0x7888, 0x7888, PDF_CMAP_SINGLE, 16388 },
	{ 0x7889, 0x7889, PDF_CMAP_SINGLE, 1488 },
	{ 0x788a, 0x788b, PDF_CMAP_RANGE, 16389 },
	{ 0x788c, 0x788c, PDF_CMAP_SINGLE, 2652 },
	{ 0x788d, 0x788d, PDF_CMAP_SINGLE, 952 },
	{ 0x788e, 0x788e, PDF_CMAP_SINGLE, 3550 },
	{ 0x788f, 0x7890, PDF_CMAP_RANGE, 16391 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 1047 },
	{ 0x7892, 0x7892, PDF_CMAP_SINGLE, 16393 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 6720 },
	{ 0x7894, 0x7896, PDF_CMAP_RANGE, 16394 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 3740 },
	{ 0x7898, 0x7898, PDF_CMAP_SINGLE, 1475 },
	{ 0x7899, 0x7899, PDF_CMAP_SINGLE, 16397 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 6721 },
	{ 0x789b, 0x789b, PDF_CMAP_SINGLE, 6719 },
	{ 0x789c, 0x789c, PDF_CMAP_SINGLE, 6723 },
	{ 0x789d, 0x789e, PDF_CMAP_RANGE, 16398 },
	{ 0x789f, 0x789f, PDF_CMAP_SINGLE, 1499 },
	{ 0x78a0, 0x78a0, PDF_CMAP_SINGLE, 16400 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 6724 },
	{ 0x78a2, 0x78a2, PDF_CMAP_SINGLE, 16401 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 6725 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 16402 },
	{ 0x78a5, 0x78a5, PDF_CMAP_SINGLE, 6728 },
	{ 0x78a6, 0x78a6, PDF_CMAP_SINGLE, 16403 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 1077 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 16404 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 8520 },
	{ 0x78aa, 0x78ac, PDF_CMAP_RANGE, 16405 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 9444 },
	{ 0x78ae, 0x78af, PDF_CMAP_RANGE, 16408 },
	{ 0x78b0, 0x78b0, PDF_CMAP_SINGLE, 2999 },
	{ 0x78b1, 0x78b1, PDF_CMAP_SINGLE, 2145 },
	{ 0x78b2, 0x78b2, PDF_CMAP_SINGLE, 6726 },
	{ 0x78b3, 0x78b3, PDF_CMAP_SINGLE, 3599 },
	{ 0x78b4, 0x78b4, PDF_CMAP_SINGLE, 1201 },
	{ 0x78b5, 0x78b7, PDF_CMAP_RANGE, 16410 },
	{ 0x78b8, 0x78b8, PDF_CMAP_SINGLE, 9446 },
	{ 0x78b9, 0x78b9, PDF_CMAP_SINGLE, 6727 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 8439 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 16413 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 8299 },
	{ 0x78bd, 0x78bd, PDF_CMAP_SINGLE, 16414 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 2894 },
	{ 0x78bf, 0x78c0, PDF_CMAP_RANGE, 16415 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 1352 },
	{ 0x78c2, 0x78c4, PDF_CMAP_RANGE, 16417 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 1024 },
	{ 0x78c6, 0x78c8, PDF_CMAP_RANGE, 16420 },
	{ 0x78c9, 0x78c9, PDF_CMAP_SINGLE, 6731 },
	{ 0x78ca, 0x78ca, PDF_CMAP_SINGLE, 2501 },
	{ 0x78cb, 0x78cb, PDF_CMAP_SINGLE, 1387 },
	{ 0x78cc, 0x78cf, PDF_CMAP_RANGE, 16423 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 2958 },
	{ 0x78d1, 0x78d3, PDF_CMAP_RANGE, 16427 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 6729 },
	{ 0x78d5, 0x78d5, PDF_CMAP_SINGLE, 2385 },
	{ 0x78d6, 0x78d8, PDF_CMAP_RANGE, 16430 },
	{ 0x78d9, 0x78d9, PDF_CMAP_SINGLE, 6730 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 8871 },
	{ 0x78db, 0x78e2, PDF_CMAP_RANGE, 16433 },
	{ 0x78e3, 0x78e3, PDF_CMAP_SINGLE, 9452 },
	{ 0x78e4, 0x78e6, PDF_CMAP_RANGE, 16441 },
	{ 0x78e7, 0x78e7, PDF_CMAP_SINGLE, 9451 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 2824 },
	{ 0x78e9, 0x78eb, PDF_CMAP_RANGE, 16444 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 6732 },
	{ 0x78ed, 0x78ee, PDF_CMAP_RANGE, 16447 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 9443 },
	{ 0x78f0, 0x78f1, PDF_CMAP_RANGE, 16449 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 6733 },
	{ 0x78f3, 0x78f3, PDF_CMAP_SINGLE, 16451 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 6735 },
	{ 0x78f5, 0x78f6, PDF_CMAP_RANGE, 16452 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 2592 },
	{ 0x78f8, 0x78f9, PDF_CMAP_RANGE, 16454 },
	{ 0x78fa, 0x78fa, PDF_CMAP_SINGLE, 2012 },
	{ 0x78fb, 0x78fc, PDF_CMAP_RANGE, 16456 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 9450 },
	{ 0x78fe, 0x78ff, PDF_CMAP_RANGE, 16458 },
	{ 0x7900, 0x7900, PDF_CMAP_SINGLE, 16460 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2185 },
	{ 0x7902, 0x7904, PDF_CMAP_RANGE, 16461 },
	{ 0x7905, 0x7905, PDF_CMAP_SINGLE, 6734 },
	{ 0x7906, 0x790d, PDF_CMAP_RANGE, 16464 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 7826 },
	{ 0x790f, 0x7912, PDF_CMAP_RANGE, 16472 },
	{ 0x7913, 0x7913, PDF_CMAP_SINGLE, 6736 },
	{ 0x7914, 0x7918, PDF_CMAP_RANGE, 16476 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 7719 },
	{ 0x791a, 0x791d, PDF_CMAP_RANGE, 16481 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 6738 },
	{ 0x791f, 0x7923, PDF_CMAP_RANGE, 16485 },
	{ 0x7924, 0x7924, PDF_CMAP_SINGLE, 6737 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 16490 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 8166 },
	{ 0x7927, 0x7929, PDF_CMAP_RANGE, 16491 },
	{ 0x792a, 0x792a, PDF_CMAP_SINGLE, 9447 },
	{ 0x792b, 0x792b, PDF_CMAP_SINGLE, 8210 },
	{ 0x792c, 0x792c, PDF_CMAP_SINGLE, 7916 },
	{ 0x792d, 0x7930, PDF_CMAP_RANGE, 16494 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 9448 },
	{ 0x7932, 0x7933, PDF_CMAP_RANGE, 16498 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 6739 },
	{ 0x7935, 0x7939, PDF_CMAP_RANGE, 16500 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 3413 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 6648 },
	{ 0x793c, 0x793c, PDF_CMAP_SINGLE, 2524 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 16505 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 3360 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 16506 },
	{ 0x7940, 0x7940, PDF_CMAP_SINGLE, 6649 },
	{ 0x7941, 0x7941, PDF_CMAP_SINGLE, 3085 },
	{ 0x7942, 0x7945, PDF_CMAP_RANGE, 16507 },
	{ 0x7946, 0x7946, PDF_CMAP_SINGLE, 6650 },
	{ 0x7947, 0x7947, PDF_CMAP_SINGLE, 16511 },
	{ 0x7948, 0x7948, PDF_CMAP_SINGLE, 3084 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 6651 },
	{ 0x794a, 0x7952, PDF_CMAP_RANGE, 16512 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 6654 },
	{ 0x7954, 0x7955, PDF_CMAP_RANGE, 16521 },
	{ 0x7956, 0x7956, PDF_CMAP_SINGLE, 4675 },
	{ 0x7957, 0x7957, PDF_CMAP_SINGLE, 6657 },
	{ 0x7958, 0x7959, PDF_CMAP_RANGE, 16523 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 6655 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 6652 },
	{ 0x795d, 0x795d, PDF_CMAP_SINGLE, 4608 },
	{ 0x795e, 0x795e, PDF_CMAP_SINGLE, 3370 },
	{ 0x795f, 0x795f, PDF_CMAP_SINGLE, 3555 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 6658 },
	{ 0x7961, 0x7961, PDF_CMAP_SINGLE, 16525 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 6656 },
	{ 0x7963, 0x7964, PDF_CMAP_RANGE, 16526 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 3926 },
	{ 0x7966, 0x7966, PDF_CMAP_SINGLE, 16528 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 6660 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 3024 },
	{ 0x7969, 0x796c, PDF_CMAP_RANGE, 16529 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2099 },
	{ 0x796e, 0x796e, PDF_CMAP_SINGLE, 16533 },
	{ 0x796f, 0x796f, PDF_CMAP_SINGLE, 6659 },
	{ 0x7970, 0x7976, PDF_CMAP_RANGE, 16534 },
	{ 0x7977, 0x7977, PDF_CMAP_SINGLE, 1436 },
	{ 0x7978, 0x7978, PDF_CMAP_SINGLE, 2059 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 16541 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 6661 },
	{ 0x797b, 0x797f, PDF_CMAP_RANGE, 16542 },
	{ 0x7980, 0x7980, PDF_CMAP_SINGLE, 4875 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 2252 },
	{ 0x7982, 0x7983, PDF_CMAP_RANGE, 16547 },
	{ 0x7984, 0x7984, PDF_CMAP_SINGLE, 2658 },
	{ 0x7985, 0x7985, PDF_CMAP_SINGLE, 6662 },
	{ 0x7986, 0x7989, PDF_CMAP_RANGE, 16549 },
	{ 0x798a, 0x798a, PDF_CMAP_SINGLE, 6663 },
	{ 0x798b, 0x798c, PDF_CMAP_RANGE, 16553 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 8038 },
	{ 0x798e, 0x798e, PDF_CMAP_SINGLE, 9436 },
	{ 0x798f, 0x798f, PDF_CMAP_SINGLE, 1694 },
	{ 0x7990, 0x7999, PDF_CMAP_RANGE, 16555 },
	{ 0x799a, 0x799a, PDF_CMAP_SINGLE, 6664 },
	{ 0x799b, 0x79a5, PDF_CMAP_RANGE, 16565 },
	{ 0x79a6, 0x79a6, PDF_CMAP_SINGLE, 9890 },
	{ 0x79a7, 0x79a7, PDF_CMAP_SINGLE, 6665 },
	{ 0x79a8, 0x79a9, PDF_CMAP_RANGE, 16576 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 9437 },
	{ 0x79ab, 0x79ad, PDF_CMAP_RANGE, 16578 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 8206 },
	{ 0x79af, 0x79af, PDF_CMAP_SINGLE, 16581 },
	{ 0x79b0, 0x79b0, PDF_CMAP_SINGLE, 9435 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 7864 },
	{ 0x79b2, 0x79b2, PDF_CMAP_SINGLE, 16582 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 6666 },
	{ 0x79b4, 0x79b8, PDF_CMAP_RANGE, 16583 },
	{ 0x79b9, 0x79b9, PDF_CMAP_SINGLE, 4306 },
	{ 0x79ba, 0x79ba, PDF_CMAP_SINGLE, 4708 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 2518 },
	{ 0x79bc, 0x79bc, PDF_CMAP_SINGLE, 16588 },
	{ 0x79bd, 0x79bd, PDF_CMAP_SINGLE, 3162 },
	{ 0x79be, 0x79be, PDF_CMAP_SINGLE, 1923 },
	{ 0x79bf, 0x79bf, PDF_CMAP_SINGLE, 16589 },
	{ 0x79c0, 0x79c0, PDF_CMAP_SINGLE, 4014 },
	{ 0x79c1, 0x79c1, PDF_CMAP_SINGLE, 3506 },
	{ 0x79c2, 0x79c2, PDF_CMAP_SINGLE, 16590 },
	{ 0x79c3, 0x79c3, PDF_CMAP_SINGLE, 3691 },
	{ 0x79c4, 0x79c5, PDF_CMAP_RANGE, 16591 },
	{ 0x79c6, 0x79c6, PDF_CMAP_SINGLE, 1740 },
	{ 0x79c7, 0x79c8, PDF_CMAP_RANGE, 16593 },
	{ 0x79c9, 0x79c9, PDF_CMAP_SINGLE, 1125 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 16595 },
	{ 0x79cb, 0x79cb, PDF_CMAP_SINGLE, 3180 },
	{ 0x79cc, 0x79cc, PDF_CMAP_SINGLE, 16596 },
	{ 0x79cd, 0x79cd, PDF_CMAP_SINGLE, 4565 },
	{ 0x79ce, 0x79d0, PDF_CMAP_RANGE, 16597 },
	{ 0x79d1, 0x79d1, PDF_CMAP_SINGLE, 2387 },
	{ 0x79d2, 0x79d2, PDF_CMAP_SINGLE, 2800 },
	{ 0x79d3, 0x79d4, PDF_CMAP_RANGE, 16600 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 6941 },
	{ 0x79d6, 0x79d7, PDF_CMAP_RANGE, 16602 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 2781 },
	{ 0x79d9, 0x79de, PDF_CMAP_RANGE, 16604 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 4671 },
	{ 0x79e0, 0x79e2, PDF_CMAP_RANGE, 16610 },
	{ 0x79e3, 0x79e3, PDF_CMAP_SINGLE, 6943 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 1272 },
	{ 0x79e5, 0x79e5, PDF_CMAP_SINGLE, 16613 },
	{ 0x79e6, 0x79e6, PDF_CMAP_SINGLE, 3157 },
	{ 0x79e7, 0x79e7, PDF_CMAP_SINGLE, 4118 },
	{ 0x79e8, 0x79e8, PDF_CMAP_SINGLE, 16614 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 4551 },
	{ 0x79ea, 0x79ea, PDF_CMAP_SINGLE, 16615 },
	{ 0x79eb, 0x79eb, PDF_CMAP_SINGLE, 6944 },
	{ 0x79ec, 0x79ec, PDF_CMAP_SINGLE, 16616 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 6942 },
	{ 0x79ee, 0x79ee, PDF_CMAP_SINGLE, 16617 },
	{ 0x79ef, 0x79ef, PDF_CMAP_SINGLE, 2066 },
	{ 0x79f0, 0x79f0, PDF_CMAP_SINGLE, 1259 },
	{ 0x79f1, 0x79f7, PDF_CMAP_RANGE, 16618 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 2214 },
	{ 0x79f9, 0x79fa, PDF_CMAP_RANGE, 16625 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 4173 },
	{ 0x79fc, 0x79fc, PDF_CMAP_SINGLE, 16627 },
	{ 0x79fd, 0x79fd, PDF_CMAP_SINGLE, 2037 },
	{ 0x79fe, 0x79ff, PDF_CMAP_RANGE, 16628 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 3854 },
	{ 0x7a01, 0x7a01, PDF_CMAP_SINGLE, 16630 },
	{ 0x7a02, 0x7a02, PDF_CMAP_SINGLE, 6948 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 6947 },
	{ 0x7a04, 0x7a05, PDF_CMAP_RANGE, 16631 },
	{ 0x7a06, 0x7a06, PDF_CMAP_SINGLE, 6945 },
	{ 0x7a07, 0x7a0a, PDF_CMAP_RANGE, 16633 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 1265 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 16637 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 3341 },
	{ 0x7a0e, 0x7a0e, PDF_CMAP_SINGLE, 3493 },
	{ 0x7a0f, 0x7a13, PDF_CMAP_RANGE, 16638 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 6950 },
	{ 0x7a15, 0x7a16, PDF_CMAP_RANGE, 16643 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 1001 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 16645 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 4552 },
	{ 0x7a1b, 0x7a1d, PDF_CMAP_RANGE, 16647 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 6949 },
	{ 0x7a1f, 0x7a1f, PDF_CMAP_SINGLE, 16650 },
	{ 0x7a20, 0x7a20, PDF_CMAP_SINGLE, 1298 },
	{ 0x7a21, 0x7a22, PDF_CMAP_RANGE, 16651 },
	{ 0x7a23, 0x7a23, PDF_CMAP_SINGLE, 7567 },
	{ 0x7a24, 0x7a2d, PDF_CMAP_RANGE, 16653 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 8853 },
	{ 0x7a2f, 0x7a30, PDF_CMAP_RANGE, 16663 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 7808 },
	{ 0x7a32, 0x7a32, PDF_CMAP_SINGLE, 16665 },
	{ 0x7a33, 0x7a33, PDF_CMAP_SINGLE, 3799 },
	{ 0x7a34, 0x7a36, PDF_CMAP_RANGE, 16666 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 6952 },
	{ 0x7a38, 0x7a38, PDF_CMAP_SINGLE, 16669 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 6951 },
	{ 0x7a3a, 0x7a3a, PDF_CMAP_SINGLE, 16670 },
	{ 0x7a3b, 0x7a3b, PDF_CMAP_SINGLE, 1439 },
	{ 0x7a3c, 0x7a3c, PDF_CMAP_SINGLE, 2125 },
	{ 0x7a3d, 0x7a3d, PDF_CMAP_SINGLE, 2065 },
	{ 0x7a3e, 0x7a3e, PDF_CMAP_SINGLE, 16671 },
	{ 0x7a3f, 0x7a3f, PDF_CMAP_SINGLE, 1760 },
	{ 0x7a40, 0x7a40, PDF_CMAP_SINGLE, 9847 },
	{ 0x7a41, 0x7a45, PDF_CMAP_RANGE, 16672 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 2853 },
	{ 0x7a47, 0x7a4b, PDF_CMAP_RANGE, 16677 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 9764 },
	{ 0x7a4d, 0x7a4d, PDF_CMAP_SINGLE, 8041 },
	{ 0x7a4e, 0x7a4e, PDF_CMAP_SINGLE, 8747 },
	{ 0x7a4f, 0x7a50, PDF_CMAP_RANGE, 16682 },
	{ 0x7a51, 0x7a51, PDF_CMAP_SINGLE, 6953 },
	{ 0x7a52, 0x7a56, PDF_CMAP_RANGE, 16684 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 3552 },
	{ 0x7a58, 0x7a60, PDF_CMAP_RANGE, 16689 },
	{ 0x7a61, 0x7a61, PDF_CMAP_SINGLE, 9599 },
	{ 0x7a62, 0x7a62, PDF_CMAP_SINGLE, 8027 },
	{ 0x7a63, 0x7a68, PDF_CMAP_RANGE, 16698 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 8603 },
	{ 0x7a6a, 0x7a6a, PDF_CMAP_SINGLE, 16704 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 9853 },
	{ 0x7a6c, 0x7a6f, PDF_CMAP_RANGE, 16705 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 6956 },
	{ 0x7a71, 0x7a73, PDF_CMAP_RANGE, 16709 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 4049 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 16712 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 2286 },
	{ 0x7a77, 0x7a77, PDF_CMAP_SINGLE, 3179 },
	{ 0x7a78, 0x7a79, PDF_CMAP_RANGE, 7066 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 2402 },
	{ 0x7a7b, 0x7a7e, PDF_CMAP_RANGE, 16713 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 1324 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 7068 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 3692 },
	{ 0x7a82, 0x7a82, PDF_CMAP_SINGLE, 16717 },
	{ 0x7a83, 0x7a83, PDF_CMAP_SINGLE, 3153 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 4430 },
	{ 0x7a85, 0x7a85, PDF_CMAP_SINGLE, 16718 },
	{ 0x7a86, 0x7a86, PDF_CMAP_SINGLE, 7069 },
	{ 0x7a87, 0x7a87, PDF_CMAP_SINGLE, 16719 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 7070 },
	{ 0x7a89, 0x7a8c, PDF_CMAP_RANGE, 16720 },
	{ 0x7a8d, 0x7a8d, PDF_CMAP_SINGLE, 3148 },
	{ 0x7a8e, 0x7a90, PDF_CMAP_RANGE, 16724 },
	{ 0x7a91, 0x7a91, PDF_CMAP_SINGLE, 4139 },
	{ 0x7a92, 0x7a92, PDF_CMAP_SINGLE, 4558 },
	{ 0x7a93, 0x7a94, PDF_CMAP_RANGE, 16727 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 7071 },
	{ 0x7a96, 0x7a96, PDF_CMAP_SINGLE, 2210 },
	{ 0x7a97, 0x7a97, PDF_CMAP_SINGLE, 1331 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 2284 },
	{ 0x7a99, 0x7a9b, PDF_CMAP_RANGE, 16729 },
	{ 0x7a9c, 0x7a9c, PDF_CMAP_SINGLE, 1375 },
	{ 0x7a9d, 0x7a9d, PDF_CMAP_SINGLE, 3808 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 16732 },
	{ 0x7a9f, 0x7a9f, PDF_CMAP_SINGLE, 2412 },
	{ 0x7aa0, 0x7aa0, PDF_CMAP_SINGLE, 7073 },
	{ 0x7aa1, 0x7aa4, PDF_CMAP_RANGE, 16733 },
	{ 0x7aa5, 0x7aa5, PDF_CMAP_SINGLE, 2439 },
	{ 0x7aa6, 0x7aa6, PDF_CMAP_SINGLE, 7072 },
	{ 0x7aa7, 0x7aa7, PDF_CMAP_SINGLE, 16737 },
	{ 0x7aa8, 0x7aa8, PDF_CMAP_SINGLE, 7075 },
	{ 0x7aa9, 0x7aa9, PDF_CMAP_SINGLE, 8608 },
	{ 0x7aaa, 0x7aaa, PDF_CMAP_SINGLE, 8582 },
	{ 0x7aab, 0x7aab, PDF_CMAP_SINGLE, 16738 },
	{ 0x7aac, 0x7aac, PDF_CMAP_SINGLE, 7074 },
	{ 0x7aad, 0x7aad, PDF_CMAP_SINGLE, 7076 },
	{ 0x7aae, 0x7aae, PDF_CMAP_SINGLE, 8429 },
	{ 0x7aaf, 0x7ab2, PDF_CMAP_RANGE, 16739 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 7077 },
	{ 0x7ab4, 0x7ab5, PDF_CMAP_RANGE, 16743 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 9657 },
	{ 0x7ab7, 0x7ab9, PDF_CMAP_RANGE, 16745 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 8170 },
	{ 0x7abb, 0x7abe, PDF_CMAP_RANGE, 16748 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 2631 },
	{ 0x7ac0, 0x7ac3, PDF_CMAP_RANGE, 16752 },
	{ 0x7ac4, 0x7ac4, PDF_CMAP_SINGLE, 7844 },
	{ 0x7ac5, 0x7ac5, PDF_CMAP_SINGLE, 8417 },
	{ 0x7ac6, 0x7ac6, PDF_CMAP_SINGLE, 16756 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 9656 },
	{ 0x7ac8, 0x7ac8, PDF_CMAP_SINGLE, 8800 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 16757 },
	{ 0x7aca, 0x7aca, PDF_CMAP_SINGLE, 8418 },
	{ 0x7acb, 0x7acb, PDF_CMAP_SINGLE, 2539 },
	{ 0x7acc, 0x7ad5, PDF_CMAP_RANGE, 16758 },
	{ 0x7ad6, 0x7ad6, PDF_CMAP_SINGLE, 3473 },
	{ 0x7ad7, 0x7ad8, PDF_CMAP_RANGE, 16768 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 4447 },
	{ 0x7ada, 0x7add, PDF_CMAP_RANGE, 16770 },
	{ 0x7ade, 0x7ade, PDF_CMAP_SINGLE, 2281 },
	{ 0x7adf, 0x7adf, PDF_CMAP_SINGLE, 2280 },
	{ 0x7ae0, 0x7ae0, PDF_CMAP_SINGLE, 4451 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 16774 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 2351 },
	{ 0x7ae4, 0x7ae4, PDF_CMAP_SINGLE, 16776 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 3680 },
	{ 0x7ae6, 0x7ae6, PDF_CMAP_SINGLE, 7065 },
	{ 0x7ae7, 0x7ae9, PDF_CMAP_RANGE, 16777 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 8513 },
	{ 0x7aeb, 0x7aec, PDF_CMAP_RANGE, 16780 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 2224 },
	{ 0x7aee, 0x7aee, PDF_CMAP_SINGLE, 16782 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 1546 },
	{ 0x7af0, 0x7af5, PDF_CMAP_RANGE, 16783 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 8135 },
	{ 0x7af7, 0x7af8, PDF_CMAP_RANGE, 16789 },
	{ 0x7af9, 0x7af9, PDF_CMAP_SINGLE, 4592 },
	{ 0x7afa, 0x7afa, PDF_CMAP_SINGLE, 7267 },
	{ 0x7afb, 0x7afc, PDF_CMAP_RANGE, 16791 },
	{ 0x7afd, 0x7afd, PDF_CMAP_SINGLE, 7268 },
	{ 0x7afe, 0x7afe, PDF_CMAP_SINGLE, 16793 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1736 },
	{ 0x7b00, 0x7b02, PDF_CMAP_RANGE, 16794 },
	{ 0x7b03, 0x7b04, PDF_CMAP_RANGE, 7270 },
	{ 0x7b05, 0x7b05, PDF_CMAP_SINGLE, 16797 },
	{ 0x7b06, 0x7b06, PDF_CMAP_SINGLE, 981 },
	{ 0x7b07, 0x7b07, PDF_CMAP_SINGLE, 16798 },
	{ 0x7b08, 0x7b08, PDF_CMAP_SINGLE, 7269 },
	{ 0x7b09, 0x7b09, PDF_CMAP_SINGLE, 16799 },
	{ 0x7b0a, 0x7b0a, PDF_CMAP_SINGLE, 7273 },
	{ 0x7b0b, 0x7b0b, PDF_CMAP_SINGLE, 3558 },
	{ 0x7b0c, 0x7b0e, PDF_CMAP_RANGE, 16800 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 7275 },
	{ 0x7b10, 0x7b10, PDF_CMAP_SINGLE, 16803 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 3953 },
	{ 0x7b12, 0x7b13, PDF_CMAP_RANGE, 16804 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 1075 },
	{ 0x7b15, 0x7b15, PDF_CMAP_SINGLE, 7272 },
	{ 0x7b16, 0x7b18, PDF_CMAP_RANGE, 16806 },
	{ 0x7b19, 0x7b19, PDF_CMAP_SINGLE, 7279 },
	{ 0x7b1a, 0x7b1a, PDF_CMAP_SINGLE, 16809 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 1458 },
	{ 0x7b1c, 0x7b1d, PDF_CMAP_RANGE, 16810 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 7287 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 16812 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 7282 },
	{ 0x7b21, 0x7b23, PDF_CMAP_RANGE, 16813 },
	{ 0x7b24, 0x7b24, PDF_CMAP_SINGLE, 7284 },
	{ 0x7b25, 0x7b25, PDF_CMAP_SINGLE, 7283 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 1688 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 16816 },
	{ 0x7b28, 0x7b28, PDF_CMAP_SINGLE, 1064 },
	{ 0x7b29, 0x7b29, PDF_CMAP_SINGLE, 16817 },
	{ 0x7b2a, 0x7b2a, PDF_CMAP_SINGLE, 7278 },
	{ 0x7b2b, 0x7b2b, PDF_CMAP_SINGLE, 7274 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 1467 },
	{ 0x7b2d, 0x7b2d, PDF_CMAP_SINGLE, 16818 },
	{ 0x7b2e, 0x7b2e, PDF_CMAP_SINGLE, 7280 },
	{ 0x7b2f, 0x7b30, PDF_CMAP_RANGE, 16819 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 7281 },
	{ 0x7b32, 0x7b32, PDF_CMAP_SINGLE, 16821 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 7285 },
	{ 0x7b34, 0x7b37, PDF_CMAP_RANGE, 16822 },
	{ 0x7b38, 0x7b38, PDF_CMAP_SINGLE, 7277 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 16826 },
	{ 0x7b3a, 0x7b3a, PDF_CMAP_SINGLE, 2134 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 16827 },
	{ 0x7b3c, 0x7b3c, PDF_CMAP_SINGLE, 2630 },
	{ 0x7b3d, 0x7b3d, PDF_CMAP_SINGLE, 16828 },
	{ 0x7b3e, 0x7b3e, PDF_CMAP_SINGLE, 7286 },
	{ 0x7b3f, 0x7b44, PDF_CMAP_RANGE, 16829 },
	{ 0x7b45, 0x7b45, PDF_CMAP_SINGLE, 7290 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 7745 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 7276 },
	{ 0x7b48, 0x7b48, PDF_CMAP_SINGLE, 16835 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 1449 },
	{ 0x7b4a, 0x7b4a, PDF_CMAP_SINGLE, 16836 },
	{ 0x7b4b, 0x7b4b, PDF_CMAP_SINGLE, 2239 },
	{ 0x7b4c, 0x7b4c, PDF_CMAP_SINGLE, 7292 },
	{ 0x7b4d, 0x7b4e, PDF_CMAP_RANGE, 16837 },
	{ 0x7b4f, 0x7b4f, PDF_CMAP_SINGLE, 1601 },
	{ 0x7b50, 0x7b50, PDF_CMAP_SINGLE, 2429 },
	{ 0x7b51, 0x7b51, PDF_CMAP_SINGLE, 4605 },
	{ 0x7b52, 0x7b52, PDF_CMAP_SINGLE, 3683 },
	{ 0x7b53, 0x7b53, PDF_CMAP_SINGLE, 16839 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 1395 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 16840 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 1190 },
	{ 0x7b57, 0x7b57, PDF_CMAP_SINGLE, 16841 },
	{ 0x7b58, 0x7b58, PDF_CMAP_SINGLE, 7288 },
	{ 0x7b59, 0x7b59, PDF_CMAP_SINGLE, 16842 },
	{ 0x7b5a, 0x7b5a, PDF_CMAP_SINGLE, 7289 },
	{ 0x7b5b, 0x7b5b, PDF_CMAP_SINGLE, 3313 },
	{ 0x7b5c, 0x7b5c, PDF_CMAP_SINGLE, 16843 },
	{ 0x7b5d, 0x7b5d, PDF_CMAP_SINGLE, 7293 },
	{ 0x7b5e, 0x7b5f, PDF_CMAP_RANGE, 16844 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 7294 },
	{ 0x7b61, 0x7b61, PDF_CMAP_SINGLE, 16846 },
	{ 0x7b62, 0x7b62, PDF_CMAP_SINGLE, 7297 },
	{ 0x7b63, 0x7b66, PDF_CMAP_RANGE, 16847 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 9700 },
	{ 0x7b68, 0x7b6d, PDF_CMAP_RANGE, 16851 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 7295 },
	{ 0x7b6f, 0x7b70, PDF_CMAP_RANGE, 16857 },
	{ 0x7b71, 0x7b71, PDF_CMAP_SINGLE, 7299 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 7298 },
	{ 0x7b73, 0x7b74, PDF_CMAP_RANGE, 16859 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 7291 },
	{ 0x7b76, 0x7b76, PDF_CMAP_SINGLE, 16861 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 2423 },
	{ 0x7b78, 0x7b78, PDF_CMAP_SINGLE, 16862 },
	{ 0x7b79, 0x7b79, PDF_CMAP_SINGLE, 1300 },
	{ 0x7b7a, 0x7b7a, PDF_CMAP_SINGLE, 16863 },
	{ 0x7b7b, 0x7b7b, PDF_CMAP_SINGLE, 7296 },
	{ 0x7b7c, 0x7b7d, PDF_CMAP_RANGE, 16864 },
	{ 0x7b7e, 0x7b7e, PDF_CMAP_SINGLE, 3110 },
	{ 0x7b7f, 0x7b7f, PDF_CMAP_SINGLE, 16866 },
	{ 0x7b80, 0x7b80, PDF_CMAP_SINGLE, 2149 },
	{ 0x7b81, 0x7b84, PDF_CMAP_RANGE, 16867 },
	{ 0x7b85, 0x7b85, PDF_CMAP_SINGLE, 7307 },
	{ 0x7b86, 0x7b8a, PDF_CMAP_RANGE, 16871 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 8070 },
	{ 0x7b8c, 0x7b8c, PDF_CMAP_SINGLE, 16876 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 1816 },
	{ 0x7b8e, 0x7b8f, PDF_CMAP_RANGE, 16877 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 7300 },
	{ 0x7b91, 0x7b93, PDF_CMAP_RANGE, 16879 },
	{ 0x7b94, 0x7b94, PDF_CMAP_SINGLE, 1140 },
	{ 0x7b95, 0x7b95, PDF_CMAP_SINGLE, 2067 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 16882 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 3544 },
	{ 0x7b98, 0x7b9b, PDF_CMAP_RANGE, 16883 },
	{ 0x7b9c, 0x7b9c, PDF_CMAP_SINGLE, 7309 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 7305 },
	{ 0x7b9e, 0x7ba0, PDF_CMAP_RANGE, 16887 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 1845 },
	{ 0x7ba2, 0x7ba2, PDF_CMAP_SINGLE, 7310 },
	{ 0x7ba3, 0x7ba5, PDF_CMAP_RANGE, 16890 },
	{ 0x7ba6, 0x7ba7, PDF_CMAP_RANGE, 7301 },
	{ 0x7ba8, 0x7ba8, PDF_CMAP_SINGLE, 7306 },
	{ 0x7ba9, 0x7ba9, PDF_CMAP_SINGLE, 2696 },
	{ 0x7baa, 0x7baa, PDF_CMAP_SINGLE, 7308 },
	{ 0x7bab, 0x7bab, PDF_CMAP_SINGLE, 7311 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 7304 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 2160 },
	{ 0x7bae, 0x7bb0, PDF_CMAP_RANGE, 16893 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3921 },
	{ 0x7bb2, 0x7bb3, PDF_CMAP_RANGE, 16896 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 7312 },
	{ 0x7bb5, 0x7bb7, PDF_CMAP_RANGE, 16898 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 7303 },
	{ 0x7bb9, 0x7bbf, PDF_CMAP_RANGE, 16901 },
	{ 0x7bc0, 0x7bc0, PDF_CMAP_SINGLE, 8115 },
	{ 0x7bc1, 0x7bc1, PDF_CMAP_SINGLE, 7314 },
	{ 0x7bc2, 0x7bc3, PDF_CMAP_RANGE, 16908 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 7919 },
	{ 0x7bc5, 0x7bc5, PDF_CMAP_SINGLE, 16910 },
	{ 0x7bc6, 0x7bc6, PDF_CMAP_SINGLE, 4618 },
	{ 0x7bc7, 0x7bc7, PDF_CMAP_SINGLE, 3017 },
	{ 0x7bc8, 0x7bc8, PDF_CMAP_SINGLE, 16911 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 8868 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 16912 },
	{ 0x7bcb, 0x7bcb, PDF_CMAP_SINGLE, 9704 },
	{ 0x7bcc, 0x7bcc, PDF_CMAP_SINGLE, 7315 },
	{ 0x7bcd, 0x7bd0, PDF_CMAP_RANGE, 16913 },
	{ 0x7bd1, 0x7bd1, PDF_CMAP_SINGLE, 7313 },
	{ 0x7bd2, 0x7bd2, PDF_CMAP_SINGLE, 16917 },
	{ 0x7bd3, 0x7bd3, PDF_CMAP_SINGLE, 2639 },
	{ 0x7bd4, 0x7bd8, PDF_CMAP_RANGE, 16918 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 1752 },
	{ 0x7bda, 0x7bda, PDF_CMAP_SINGLE, 7317 },
	{ 0x7bdb, 0x7bdc, PDF_CMAP_RANGE, 16923 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 7316 },
	{ 0x7bde, 0x7be0, PDF_CMAP_RANGE, 16925 },
	{ 0x7be1, 0x7be1, PDF_CMAP_SINGLE, 1374 },
	{ 0x7be2, 0x7be3, PDF_CMAP_RANGE, 16928 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 9699 },
	{ 0x7be5, 0x7be6, PDF_CMAP_RANGE, 7318 },
	{ 0x7be7, 0x7be8, PDF_CMAP_RANGE, 16930 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 8465 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 7320 },
	{ 0x7beb, 0x7bed, PDF_CMAP_RANGE, 16932 },
	{ 0x7bee, 0x7bee, PDF_CMAP_SINGLE, 2469 },
	{ 0x7bef, 0x7bf0, PDF_CMAP_RANGE, 16935 },
	{ 0x7bf1, 0x7bf1, PDF_CMAP_SINGLE, 2516 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 16937 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 9702 },
	{ 0x7bf4, 0x7bf6, PDF_CMAP_RANGE, 16938 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 2994 },
	{ 0x7bf8, 0x7bfb, PDF_CMAP_RANGE, 16941 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 7323 },
	{ 0x7bfd, 0x7bfd, PDF_CMAP_SINGLE, 16945 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 7322 },
	{ 0x7bff, 0x7bff, PDF_CMAP_SINGLE, 16946 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 9703 },
	{ 0x7c01, 0x7c06, PDF_CMAP_RANGE, 16947 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 1371 },
	{ 0x7c08, 0x7c0a, PDF_CMAP_RANGE, 16953 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 7326 },
	{ 0x7c0c, 0x7c0c, PDF_CMAP_SINGLE, 7321 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 8257 },
	{ 0x7c0e, 0x7c0e, PDF_CMAP_SINGLE, 16956 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 7324 },
	{ 0x7c10, 0x7c15, PDF_CMAP_RANGE, 16957 },
	{ 0x7c16, 0x7c16, PDF_CMAP_SINGLE, 7325 },
	{ 0x7c17, 0x7c1d, PDF_CMAP_RANGE, 16963 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 9706 },
	{ 0x7c1f, 0x7c1f, PDF_CMAP_SINGLE, 7327 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 16970 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 8079 },
	{ 0x7c22, 0x7c22, PDF_CMAP_SINGLE, 16971 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 9708 },
	{ 0x7c24, 0x7c25, PDF_CMAP_RANGE, 16972 },
	{ 0x7c26, 0x7c26, PDF_CMAP_SINGLE, 7329 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 2014 },
	{ 0x7c28, 0x7c29, PDF_CMAP_RANGE, 16974 },
	{ 0x7c2a, 0x7c2a, PDF_CMAP_SINGLE, 7328 },
	{ 0x7c2b, 0x7c2b, PDF_CMAP_SINGLE, 9707 },
	{ 0x7c2c, 0x7c37, PDF_CMAP_RANGE, 16976 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 7330 },
	{ 0x7c39, 0x7c3c, PDF_CMAP_RANGE, 16988 },
	{ 0x7c3d, 0x7c3d, PDF_CMAP_SINGLE, 8400 },
	{ 0x7c3e, 0x7c3e, PDF_CMAP_SINGLE, 8221 },
	{ 0x7c3f, 0x7c3f, PDF_CMAP_SINGLE, 1157 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 7332 },
	{ 0x7c41, 0x7c41, PDF_CMAP_SINGLE, 7331 },
	{ 0x7c42, 0x7c42, PDF_CMAP_SINGLE, 16992 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 8183 },
	{ 0x7c44, 0x7c4b, PDF_CMAP_RANGE, 16993 },
	{ 0x7c4c, 0x7c4c, PDF_CMAP_SINGLE, 7821 },
	{ 0x7c4d, 0x7c4d, PDF_CMAP_SINGLE, 2081 },
	{ 0x7c4e, 0x7c5b, PDF_CMAP_RANGE, 17001 },
	{ 0x7c5c, 0x7c5c, PDF_CMAP_SINGLE, 9705 },
	{ 0x7c5d, 0x7c5e, PDF_CMAP_RANGE, 17015 },
	{ 0x7c5f, 0x7c5f, PDF_CMAP_SINGLE, 9710 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 8250 },
	{ 0x7c61, 0x7c63, PDF_CMAP_RANGE, 17017 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 9875 },
	{ 0x7c65, 0x7c68, PDF_CMAP_RANGE, 17020 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 9701 },
	{ 0x7c6a, 0x7c6a, PDF_CMAP_SINGLE, 9709 },
	{ 0x7c6b, 0x7c6b, PDF_CMAP_SINGLE, 17024 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 8201 },
	{ 0x7c6d, 0x7c6d, PDF_CMAP_SINGLE, 17025 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 8293 },
	{ 0x7c6f, 0x7c71, PDF_CMAP_RANGE, 17026 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 9891 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 2780 },
	{ 0x7c74, 0x7c74, PDF_CMAP_SINGLE, 4853 },
	{ 0x7c75, 0x7c7a, PDF_CMAP_RANGE, 17029 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 2507 },
	{ 0x7c7c, 0x7c7c, PDF_CMAP_SINGLE, 7370 },
	{ 0x7c7d, 0x7c7d, PDF_CMAP_SINGLE, 4654 },
	{ 0x7c7e, 0x7c88, PDF_CMAP_RANGE, 17035 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 1656 },
	{ 0x7c8a, 0x7c90, PDF_CMAP_RANGE, 17046 },
	{ 0x7c91, 0x7c91, PDF_CMAP_SINGLE, 7372 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 2540 },
	{ 0x7c93, 0x7c94, PDF_CMAP_RANGE, 17053 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 3048 },
	{ 0x7c96, 0x7c96, PDF_CMAP_SINGLE, 17055 },
	{ 0x7c97, 0x7c97, PDF_CMAP_SINGLE, 1369 },
	{ 0x7c98, 0x7c98, PDF_CMAP_SINGLE, 4436 },
	{ 0x7c99, 0x7c9b, PDF_CMAP_RANGE, 17056 },
	{ 0x7c9c, 0x7c9c, PDF_CMAP_SINGLE, 7374 },
	{ 0x7c9d, 0x7c9d, PDF_CMAP_SINGLE, 7373 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 7375 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 3535 },
	{ 0x7ca0, 0x7ca1, PDF_CMAP_RANGE, 17059 },
	{ 0x7ca2, 0x7ca2, PDF_CMAP_SINGLE, 7376 },
	{ 0x7ca3, 0x7ca3, PDF_CMAP_SINGLE, 17061 },
	{ 0x7ca4, 0x7ca4, PDF_CMAP_SINGLE, 4356 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 4575 },
	{ 0x7ca6, 0x7ca9, PDF_CMAP_RANGE, 17062 },
	{ 0x7caa, 0x7caa, PDF_CMAP_SINGLE, 1661 },
	{ 0x7cab, 0x7cad, PDF_CMAP_RANGE, 17066 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 2561 },
	{ 0x7caf, 0x7cb0, PDF_CMAP_RANGE, 17069 },
	{ 0x7cb1, 0x7cb1, PDF_CMAP_SINGLE, 2564 },
	{ 0x7cb2, 0x7cb2, PDF_CMAP_SINGLE, 7377 },
	{ 0x7cb3, 0x7cb3, PDF_CMAP_SINGLE, 2267 },
	{ 0x7cb4, 0x7cb8, PDF_CMAP_RANGE, 17071 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 1381 },
	{ 0x7cba, 0x7cbb, PDF_CMAP_RANGE, 17076 },
	{ 0x7cbc, 0x7cbd, PDF_CMAP_RANGE, 7378 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 2266 },
	{ 0x7cbf, 0x7cc0, PDF_CMAP_RANGE, 17078 },
	{ 0x7cc1, 0x7cc1, PDF_CMAP_SINGLE, 7380 },
	{ 0x7cc2, 0x7cc4, PDF_CMAP_RANGE, 17080 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 7385 },
	{ 0x7cc6, 0x7cc6, PDF_CMAP_SINGLE, 17083 },
	{ 0x7cc7, 0x7cc7, PDF_CMAP_SINGLE, 7381 },
	{ 0x7cc8, 0x7cc8, PDF_CMAP_SINGLE, 7384 },
	{ 0x7cc9, 0x7cc9, PDF_CMAP_SINGLE, 17084 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 1972 },
	{ 0x7ccb, 0x7ccb, PDF_CMAP_SINGLE, 17085 },
	{ 0x7ccc, 0x7ccd, PDF_CMAP_RANGE, 7382 },
	{ 0x7cce, 0x7cd4, PDF_CMAP_RANGE, 17086 },
	{ 0x7cd5, 0x7cd5, PDF_CMAP_SINGLE, 1757 },
	{ 0x7cd6, 0x7cd6, PDF_CMAP_SINGLE, 3610 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 7386 },
	{ 0x7cd8, 0x7cd8, PDF_CMAP_SINGLE, 17093 },
	{ 0x7cd9, 0x7cd9, PDF_CMAP_SINGLE, 1185 },
	{ 0x7cda, 0x7cdb, PDF_CMAP_RANGE, 17094 },
	{ 0x7cdc, 0x7cdc, PDF_CMAP_SINGLE, 2776 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 9717 },
	{ 0x7cde, 0x7cde, PDF_CMAP_SINGLE, 7932 },
	{ 0x7cdf, 0x7cdf, PDF_CMAP_SINGLE, 4390 },
	{ 0x7ce0, 0x7ce0, PDF_CMAP_SINGLE, 2372 },
	{ 0x7ce1, 0x7ce6, PDF_CMAP_RANGE, 17096 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 8228 },
	{ 0x7ce8, 0x7ce8, PDF_CMAP_SINGLE, 7387 },
	{ 0x7ce9, 0x7cee, PDF_CMAP_RANGE, 17102 },
	{ 0x7cef, 0x7cef, PDF_CMAP_SINGLE, 2933 },
	{ 0x7cf0, 0x7cf0, PDF_CMAP_SINGLE, 9883 },
	{ 0x7cf1, 0x7cf1, PDF_CMAP_SINGLE, 17108 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 9715 },
	{ 0x7cf3, 0x7cf3, PDF_CMAP_SINGLE, 17109 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 8921 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 17110 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 9716 },
	{ 0x7cf7, 0x7cf7, PDF_CMAP_SINGLE, 17111 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 7399 },
	{ 0x7cf9, 0x7cf9, PDF_CMAP_SINGLE, 9243 },
	{ 0x7cfa, 0x7cfa, PDF_CMAP_SINGLE, 17112 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 3874 },
	{ 0x7cfc, 0x7cfd, PDF_CMAP_RANGE, 17113 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 8136 },
	{ 0x7cff, 0x7cff, PDF_CMAP_SINGLE, 17115 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 8059 },
	{ 0x7d01, 0x7d01, PDF_CMAP_SINGLE, 17116 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 9245 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 17117 },
	{ 0x7d04, 0x7d04, PDF_CMAP_SINGLE, 8780 },
	{ 0x7d05, 0x7d05, PDF_CMAP_SINGLE, 8008 },
	{ 0x7d06, 0x7d06, PDF_CMAP_SINGLE, 9244 },
	{ 0x7d07, 0x7d08, PDF_CMAP_RANGE, 9246 },
	{ 0x7d09, 0x7d09, PDF_CMAP_SINGLE, 8447 },
	{ 0x7d0a, 0x7d0a, PDF_CMAP_SINGLE, 3800 },
	{ 0x7d0b, 0x7d0b, PDF_CMAP_SINGLE, 8602 },
	{ 0x7d0c, 0x7d0c, PDF_CMAP_SINGLE, 17118 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 8337 },
	{ 0x7d0e, 0x7d0f, PDF_CMAP_RANGE, 17119 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 8359 },
	{ 0x7d11, 0x7d12, PDF_CMAP_RANGE, 17121 },
	{ 0x7d13, 0x7d13, PDF_CMAP_SINGLE, 9251 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 7835 },
	{ 0x7d15, 0x7d15, PDF_CMAP_SINGLE, 9250 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 17123 },
	{ 0x7d17, 0x7d17, PDF_CMAP_SINGLE, 8464 },
	{ 0x7d18, 0x7d18, PDF_CMAP_SINGLE, 17124 },
	{ 0x7d19, 0x7d19, PDF_CMAP_SINGLE, 8845 },
	{ 0x7d1a, 0x7d1a, PDF_CMAP_SINGLE, 8049 },
	{ 0x7d1b, 0x7d1b, PDF_CMAP_SINGLE, 7928 },
	{ 0x7d1c, 0x7d1c, PDF_CMAP_SINGLE, 9249 },
	{ 0x7d1d, 0x7d1f, PDF_CMAP_RANGE, 17125 },
	{ 0x7d20, 0x7d20, PDF_CMAP_SINGLE, 3533 },
	{ 0x7d21, 0x7d21, PDF_CMAP_SINGLE, 7923 },
	{ 0x7d22, 0x7d22, PDF_CMAP_SINGLE, 3564 },
	{ 0x7d23, 0x7d26, PDF_CMAP_RANGE, 17128 },
	{ 0x7d27, 0x7d27, PDF_CMAP_SINGLE, 2245 },
	{ 0x7d28, 0x7d2a, PDF_CMAP_RANGE, 17132 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 4652 },
	{ 0x7d2c, 0x7d2e, PDF_CMAP_RANGE, 17135 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 2502 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 8625 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 9254 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 9253 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 8480 },
	{ 0x7d34, 0x7d38, PDF_CMAP_RANGE, 17138 },
	{ 0x7d39, 0x7d39, PDF_CMAP_SINGLE, 8474 },
	{ 0x7d3a, 0x7d3a, PDF_CMAP_SINGLE, 9252 },
	{ 0x7d3b, 0x7d3b, PDF_CMAP_SINGLE, 17143 },
	{ 0x7d3c, 0x7d3c, PDF_CMAP_SINGLE, 9256 },
	{ 0x7d3d, 0x7d3e, PDF_CMAP_RANGE, 17144 },
	{ 0x7d3f, 0x7d3f, PDF_CMAP_SINGLE, 9258 },
	{ 0x7d40, 0x7d40, PDF_CMAP_SINGLE, 9257 },
	{ 0x7d41, 0x7d41, PDF_CMAP_SINGLE, 17146 },
	{ 0x7d42, 0x7d42, PDF_CMAP_SINGLE, 8852 },
	{ 0x7d43, 0x7d43, PDF_CMAP_SINGLE, 17147 },
	{ 0x7d44, 0x7d44, PDF_CMAP_SINGLE, 8894 },
	{ 0x7d45, 0x7d45, PDF_CMAP_SINGLE, 17148 },
	{ 0x7d46, 0x7d46, PDF_CMAP_SINGLE, 7729 },
	{ 0x7d47, 0x7d4d, PDF_CMAP_RANGE, 17149 },
	{ 0x7d4e, 0x7d4e, PDF_CMAP_SINGLE, 9260 },
	{ 0x7d4f, 0x7d4f, PDF_CMAP_SINGLE, 17156 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 8117 },
	{ 0x7d51, 0x7d5c, PDF_CMAP_RANGE, 17157 },
	{ 0x7d5d, 0x7d5d, PDF_CMAP_SINGLE, 9259 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 8111 },
	{ 0x7d5f, 0x7d60, PDF_CMAP_RANGE, 17169 },
	{ 0x7d61, 0x7d61, PDF_CMAP_SINGLE, 8296 },
	{ 0x7d62, 0x7d62, PDF_CMAP_SINGLE, 8676 },
	{ 0x7d63, 0x7d65, PDF_CMAP_RANGE, 17171 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 7969 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 17174 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 8449 },
	{ 0x7d69, 0x7d6d, PDF_CMAP_RANGE, 17175 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 4032 },
	{ 0x7d6f, 0x7d70, PDF_CMAP_RANGE, 17180 },
	{ 0x7d71, 0x7d71, PDF_CMAP_SINGLE, 8572 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 8522 },
	{ 0x7d73, 0x7d73, PDF_CMAP_SINGLE, 9261 },
	{ 0x7d74, 0x7d75, PDF_CMAP_RANGE, 17182 },
	{ 0x7d76, 0x7d76, PDF_CMAP_SINGLE, 8148 },
	{ 0x7d77, 0x7d77, PDF_CMAP_SINGLE, 7400 },
	{ 0x7d78, 0x7d78, PDF_CMAP_SINGLE, 17184 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 8145 },
	{ 0x7d7a, 0x7d80, PDF_CMAP_RANGE, 17185 },
	{ 0x7d81, 0x7d81, PDF_CMAP_SINGLE, 7731 },
	{ 0x7d82, 0x7d82, PDF_CMAP_SINGLE, 17192 },
	{ 0x7d83, 0x7d83, PDF_CMAP_SINGLE, 9263 },
	{ 0x7d84, 0x7d85, PDF_CMAP_RANGE, 17193 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 9262 },
	{ 0x7d87, 0x7d87, PDF_CMAP_SINGLE, 17195 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 9264 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 8667 },
	{ 0x7d8a, 0x7d8e, PDF_CMAP_RANGE, 17196 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 8536 },
	{ 0x7d90, 0x7d92, PDF_CMAP_RANGE, 17201 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 8130 },
	{ 0x7d94, 0x7d9b, PDF_CMAP_RANGE, 17204 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 8889 },
	{ 0x7d9d, 0x7d9d, PDF_CMAP_SINGLE, 17212 },
	{ 0x7d9e, 0x7d9e, PDF_CMAP_SINGLE, 9270 },
	{ 0x7d9f, 0x7da1, PDF_CMAP_RANGE, 17213 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 7822 },
	{ 0x7da3, 0x7da3, PDF_CMAP_SINGLE, 9273 },
	{ 0x7da4, 0x7da5, PDF_CMAP_RANGE, 17216 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 7401 },
	{ 0x7da7, 0x7daa, PDF_CMAP_RANGE, 17218 },
	{ 0x7dab, 0x7dab, PDF_CMAP_SINGLE, 8645 },
	{ 0x7dac, 0x7dac, PDF_CMAP_SINGLE, 9271 },
	{ 0x7dad, 0x7dad, PDF_CMAP_SINGLE, 8594 },
	{ 0x7dae, 0x7dae, PDF_CMAP_SINGLE, 7402 },
	{ 0x7daf, 0x7daf, PDF_CMAP_SINGLE, 17222 },
	{ 0x7db0, 0x7db0, PDF_CMAP_SINGLE, 9274 },
	{ 0x7db1, 0x7db1, PDF_CMAP_SINGLE, 7961 },
	{ 0x7db2, 0x7db2, PDF_CMAP_SINGLE, 8588 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 7744 },
	{ 0x7db4, 0x7db4, PDF_CMAP_SINGLE, 8883 },
	{ 0x7db5, 0x7db7, PDF_CMAP_RANGE, 17223 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 8287 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 9272 },
	{ 0x7dba, 0x7dba, PDF_CMAP_SINGLE, 9266 },
	{ 0x7dbb, 0x7dbb, PDF_CMAP_SINGLE, 8820 },
	{ 0x7dbc, 0x7dbc, PDF_CMAP_SINGLE, 17226 },
	{ 0x7dbd, 0x7dbd, PDF_CMAP_SINGLE, 7836 },
	{ 0x7dbe, 0x7dbe, PDF_CMAP_SINGLE, 9265 },
	{ 0x7dbf, 0x7dbf, PDF_CMAP_SINGLE, 8325 },
	{ 0x7dc0, 0x7dc3, PDF_CMAP_RANGE, 17227 },
	{ 0x7dc4, 0x7dc4, PDF_CMAP_SINGLE, 9269 },
	{ 0x7dc5, 0x7dc6, PDF_CMAP_RANGE, 17231 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 9275 },
	{ 0x7dc8, 0x7dc9, PDF_CMAP_RANGE, 17233 },
	{ 0x7dca, 0x7dca, PDF_CMAP_SINGLE, 8119 },
	{ 0x7dcb, 0x7dcb, PDF_CMAP_SINGLE, 9267 },
	{ 0x7dcc, 0x7dd0, PDF_CMAP_RANGE, 17235 },
	{ 0x7dd1, 0x7dd1, PDF_CMAP_SINGLE, 8276 },
	{ 0x7dd2, 0x7dd2, PDF_CMAP_SINGLE, 8670 },
	{ 0x7dd3, 0x7dd3, PDF_CMAP_SINGLE, 17240 },
	{ 0x7dd4, 0x7dd4, PDF_CMAP_SINGLE, 9268 },
	{ 0x7dd5, 0x7dd6, PDF_CMAP_RANGE, 17241 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 9277 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 8073 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 9276 },
	{ 0x7dda, 0x7ddc, PDF_CMAP_RANGE, 17243 },
	{ 0x7ddd, 0x7ddd, PDF_CMAP_SINGLE, 8046 },
	{ 0x7dde, 0x7dde, PDF_CMAP_SINGLE, 7896 },
	{ 0x7ddf, 0x7ddf, PDF_CMAP_SINGLE, 17246 },
	{ 0x7de0, 0x7de0, PDF_CMAP_SINGLE, 7871 },
	{ 0x7de1, 0x7de1, PDF_CMAP_SINGLE, 9285 },
	{ 0x7de2, 0x7de2, PDF_CMAP_SINGLE, 17247 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 8777 },
	{ 0x7de4, 0x7de5, PDF_CMAP_RANGE, 17248 },
	{ 0x7de6, 0x7de6, PDF_CMAP_SINGLE, 9281 },
	{ 0x7de7, 0x7de7, PDF_CMAP_SINGLE, 17250 },
	{ 0x7de8, 0x7de8, PDF_CMAP_SINGLE, 7751 },
	{ 0x7de9, 0x7de9, PDF_CMAP_SINGLE, 8022 },
	{ 0x7dea, 0x7deb, PDF_CMAP_RANGE, 17251 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 8326 },
	{ 0x7ded, 0x7dee, PDF_CMAP_RANGE, 17253 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 8598 },
	{ 0x7df0, 0x7df0, PDF_CMAP_SINGLE, 17255 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 9283 },
	{ 0x7df2, 0x7df2, PDF_CMAP_SINGLE, 9279 },
	{ 0x7df3, 0x7df3, PDF_CMAP_SINGLE, 17256 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 8227 },
	{ 0x7df5, 0x7df5, PDF_CMAP_SINGLE, 17257 },
	{ 0x7df6, 0x7df6, PDF_CMAP_SINGLE, 9282 },
	{ 0x7df7, 0x7df8, PDF_CMAP_RANGE, 17258 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 9278 },
	{ 0x7dfa, 0x7dfa, PDF_CMAP_SINGLE, 17260 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 9894 },
	{ 0x7dfc, 0x7dff, PDF_CMAP_RANGE, 17261 },
	{ 0x7e00, 0x7e07, PDF_CMAP_RANGE, 17265 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 9038 },
	{ 0x7e09, 0x7e09, PDF_CMAP_SINGLE, 9286 },
	{ 0x7e0a, 0x7e0a, PDF_CMAP_SINGLE, 9291 },
	{ 0x7e0b, 0x7e0b, PDF_CMAP_SINGLE, 9284 },
	{ 0x7e0c, 0x7e0f, PDF_CMAP_RANGE, 17273 },
	{ 0x7e10, 0x7e10, PDF_CMAP_SINGLE, 9255 },
	{ 0x7e11, 0x7e11, PDF_CMAP_SINGLE, 9292 },
	{ 0x7e12, 0x7e1a, PDF_CMAP_RANGE, 17277 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 7951 },
	{ 0x7e1c, 0x7e1c, PDF_CMAP_SINGLE, 17286 },
	{ 0x7e1d, 0x7e1d, PDF_CMAP_SINGLE, 9287 },
	{ 0x7e1e, 0x7e1e, PDF_CMAP_SINGLE, 9289 },
	{ 0x7e1f, 0x7e1f, PDF_CMAP_SINGLE, 9288 },
	{ 0x7e20, 0x7e22, PDF_CMAP_RANGE, 17287 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 8642 },
	{ 0x7e24, 0x7e26, PDF_CMAP_RANGE, 17290 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 8558 },
	{ 0x7e28, 0x7e2a, PDF_CMAP_RANGE, 17293 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 7939 },
	{ 0x7e2c, 0x7e2c, PDF_CMAP_SINGLE, 17296 },
	{ 0x7e2d, 0x7e2d, PDF_CMAP_SINGLE, 9290 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 8540 },
	{ 0x7e2f, 0x7e30, PDF_CMAP_RANGE, 17297 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 8891 },
	{ 0x7e32, 0x7e32, PDF_CMAP_SINGLE, 9296 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 17299 },
	{ 0x7e34, 0x7e34, PDF_CMAP_SINGLE, 9876 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 9295 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 9718 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 8273 },
	{ 0x7e38, 0x7e38, PDF_CMAP_SINGLE, 17300 },
	{ 0x7e39, 0x7e39, PDF_CMAP_SINGLE, 9294 },
	{ 0x7e3a, 0x7e3a, PDF_CMAP_SINGLE, 17301 },
	{ 0x7e3b, 0x7e3b, PDF_CMAP_SINGLE, 7675 },
	{ 0x7e3c, 0x7e3c, PDF_CMAP_SINGLE, 17302 },
	{ 0x7e3d, 0x7e3d, PDF_CMAP_SINGLE, 8890 },
	{ 0x7e3e, 0x7e3e, PDF_CMAP_SINGLE, 8045 },
	{ 0x7e3f, 0x7e40, PDF_CMAP_RANGE, 17303 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 1614 },
	{ 0x7e42, 0x7e44, PDF_CMAP_RANGE, 17305 },
	{ 0x7e45, 0x7e45, PDF_CMAP_SINGLE, 9298 },
	{ 0x7e46, 0x7e46, PDF_CMAP_SINGLE, 9297 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 7403 },
	{ 0x7e48, 0x7e51, PDF_CMAP_RANGE, 17308 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 9301 },
	{ 0x7e53, 0x7e53, PDF_CMAP_SINGLE, 17318 },
	{ 0x7e54, 0x7e54, PDF_CMAP_SINGLE, 8841 },
	{ 0x7e55, 0x7e55, PDF_CMAP_SINGLE, 8470 },
	{ 0x7e56, 0x7e59, PDF_CMAP_RANGE, 17319 },
	{ 0x7e5a, 0x7e5a, PDF_CMAP_SINGLE, 9300 },
	{ 0x7e5b, 0x7e5d, PDF_CMAP_RANGE, 17323 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 8443 },
	{ 0x7e5f, 0x7e61, PDF_CMAP_RANGE, 17326 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 9280 },
	{ 0x7e63, 0x7e68, PDF_CMAP_RANGE, 17329 },
	{ 0x7e69, 0x7e69, PDF_CMAP_SINGLE, 8486 },
	{ 0x7e6a, 0x7e6a, PDF_CMAP_SINGLE, 8033 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 9885 },
	{ 0x7e6c, 0x7e6c, PDF_CMAP_SINGLE, 17335 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 8074 },
	{ 0x7e6e, 0x7e6e, PDF_CMAP_SINGLE, 9302 },
	{ 0x7e6f, 0x7e6f, PDF_CMAP_SINGLE, 9305 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 9304 },
	{ 0x7e71, 0x7e72, PDF_CMAP_RANGE, 17336 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 8110 },
	{ 0x7e74, 0x7e78, PDF_CMAP_RANGE, 17338 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 8730 },
	{ 0x7e7a, 0x7e7b, PDF_CMAP_RANGE, 17343 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 8058 },
	{ 0x7e7d, 0x7e7d, PDF_CMAP_SINGLE, 9293 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 9303 },
	{ 0x7e7f, 0x7e81, PDF_CMAP_RANGE, 17345 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 4680 },
	{ 0x7e83, 0x7e87, PDF_CMAP_RANGE, 17348 },
	{ 0x7e88, 0x7e88, PDF_CMAP_SINGLE, 9299 },
	{ 0x7e89, 0x7e89, PDF_CMAP_SINGLE, 17353 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 9248 },
	{ 0x7e8b, 0x7e8b, PDF_CMAP_SINGLE, 17354 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 8671 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 9861 },
	{ 0x7e8e, 0x7e8e, PDF_CMAP_SINGLE, 17355 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 7790 },
	{ 0x7e90, 0x7e92, PDF_CMAP_RANGE, 17356 },
	{ 0x7e93, 0x7e93, PDF_CMAP_SINGLE, 8740 },
	{ 0x7e94, 0x7e94, PDF_CMAP_SINGLE, 9840 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 17359 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 8634 },
	{ 0x7e97, 0x7e97, PDF_CMAP_SINGLE, 17360 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 9306 },
	{ 0x7e99, 0x7e9a, PDF_CMAP_RANGE, 17361 },
	{ 0x7e9b, 0x7e9b, PDF_CMAP_SINGLE, 7404 },
	{ 0x7e9c, 0x7e9c, PDF_CMAP_SINGLE, 8191 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 17363 },
	{ 0x7e9f, 0x7e9f, PDF_CMAP_SINGLE, 6099 },
	{ 0x7ea0, 0x7ea0, PDF_CMAP_SINGLE, 2287 },
	{ 0x7ea1, 0x7ea1, PDF_CMAP_SINGLE, 6100 },
	{ 0x7ea2, 0x7ea2, PDF_CMAP_SINGLE, 1955 },
	{ 0x7ea3, 0x7ea3, PDF_CMAP_SINGLE, 6101 },
	{ 0x7ea4, 0x7ea4, PDF_CMAP_SINGLE, 3896 },
	{ 0x7ea5, 0x7ea5, PDF_CMAP_SINGLE, 6102 },
	{ 0x7ea6, 0x7ea6, PDF_CMAP_SINGLE, 4351 },
	{ 0x7ea7, 0x7ea7, PDF_CMAP_SINGLE, 2089 },
	{ 0x7ea8, 0x7ea9, PDF_CMAP_RANGE, 6103 },
	{ 0x7eaa, 0x7eaa, PDF_CMAP_SINGLE, 2112 },
	{ 0x7eab, 0x7eab, PDF_CMAP_SINGLE, 3245 },
	{ 0x7eac, 0x7eac, PDF_CMAP_SINGLE, 3778 },
	{ 0x7ead, 0x7ead, PDF_CMAP_SINGLE, 6105 },
	{ 0x7eae, 0x7eae, PDF_CMAP_SINGLE, 17365 },
	{ 0x7eaf, 0x7eaf, PDF_CMAP_SINGLE, 1346 },
	{ 0x7eb0, 0x7eb0, PDF_CMAP_SINGLE, 6106 },
	{ 0x7eb1, 0x7eb1, PDF_CMAP_SINGLE, 3309 },
	{ 0x7eb2, 0x7eb2, PDF_CMAP_SINGLE, 1748 },
	{ 0x7eb3, 0x7eb3, PDF_CMAP_SINGLE, 2860 },
	{ 0x7eb4, 0x7eb4, PDF_CMAP_SINGLE, 17366 },
	{ 0x7eb5, 0x7eb5, PDF_CMAP_SINGLE, 4666 },
	{ 0x7eb6, 0x7eb6, PDF_CMAP_SINGLE, 2689 },
	{ 0x7eb7, 0x7eb7, PDF_CMAP_SINGLE, 1652 },
	{ 0x7eb8, 0x7eb8, PDF_CMAP_SINGLE, 4540 },
	{ 0x7eb9, 0x7eb9, PDF_CMAP_SINGLE, 3797 },
	{ 0x7eba, 0x7eba, PDF_CMAP_SINGLE, 1633 },
	{ 0x7ebb, 0x7ebc, PDF_CMAP_RANGE, 17367 },
	{ 0x7ebd, 0x7ebd, PDF_CMAP_SINGLE, 2919 },
	{ 0x7ebe, 0x7ebe, PDF_CMAP_SINGLE, 6107 },
	{ 0x7ebf, 0x7ebf, PDF_CMAP_SINGLE, 3916 },
	{ 0x7ec0, 0x7ec2, PDF_CMAP_RANGE, 6108 },
	{ 0x7ec3, 0x7ec3, PDF_CMAP_SINGLE, 2560 },
	{ 0x7ec4, 0x7ec4, PDF_CMAP_SINGLE, 4678 },
	{ 0x7ec5, 0x7ec5, PDF_CMAP_SINGLE, 3369 },
	{ 0x7ec6, 0x7ec6, PDF_CMAP_SINGLE, 3877 },
	{ 0x7ec7, 0x7ec7, PDF_CMAP_SINGLE, 4526 },
	{ 0x7ec8, 0x7ec8, PDF_CMAP_SINGLE, 4564 },
	{ 0x7ec9, 0x7ec9, PDF_CMAP_SINGLE, 6111 },
	{ 0x7eca, 0x7eca, PDF_CMAP_SINGLE, 1016 },
	{ 0x7ecb, 0x7ecc, PDF_CMAP_RANGE, 6112 },
	{ 0x7ecd, 0x7ecd, PDF_CMAP_SINGLE, 3349 },
	{ 0x7ece, 0x7ece, PDF_CMAP_SINGLE, 4214 },
	{ 0x7ecf, 0x7ecf, PDF_CMAP_SINGLE, 2268 },
	{ 0x7ed0, 0x7ed0, PDF_CMAP_SINGLE, 6114 },
	{ 0x7ed1, 0x7ed1, PDF_CMAP_SINGLE, 1022 },
	{ 0x7ed2, 0x7ed2, PDF_CMAP_SINGLE, 3257 },
	{ 0x7ed3, 0x7ed3, PDF_CMAP_SINGLE, 2226 },
	{ 0x7ed4, 0x7ed4, PDF_CMAP_SINGLE, 6115 },
	{ 0x7ed5, 0x7ed5, PDF_CMAP_SINGLE, 3233 },
	{ 0x7ed6, 0x7ed6, PDF_CMAP_SINGLE, 17369 },
	{ 0x7ed7, 0x7ed7, PDF_CMAP_SINGLE, 6116 },
	{ 0x7ed8, 0x7ed8, PDF_CMAP_SINGLE, 2043 },
	{ 0x7ed9, 0x7ed9, PDF_CMAP_SINGLE, 1779 },
	{ 0x7eda, 0x7eda, PDF_CMAP_SINGLE, 4045 },
	{ 0x7edb, 0x7edb, PDF_CMAP_SINGLE, 6117 },
	{ 0x7edc, 0x7edc, PDF_CMAP_SINGLE, 2702 },
	{ 0x7edd, 0x7edd, PDF_CMAP_SINGLE, 2343 },
	{ 0x7ede, 0x7ede, PDF_CMAP_SINGLE, 2203 },
	{ 0x7edf, 0x7edf, PDF_CMAP_SINGLE, 3684 },
	{ 0x7ee0, 0x7ee1, PDF_CMAP_RANGE, 6118 },
	{ 0x7ee2, 0x7ee2, PDF_CMAP_SINGLE, 2333 },
	{ 0x7ee3, 0x7ee3, PDF_CMAP_SINGLE, 4016 },
	{ 0x7ee4, 0x7ee4, PDF_CMAP_SINGLE, 17370 },
	{ 0x7ee5, 0x7ee5, PDF_CMAP_SINGLE, 3548 },
	{ 0x7ee6, 0x7ee6, PDF_CMAP_SINGLE, 3619 },
	{ 0x7ee7, 0x7ee7, PDF_CMAP_SINGLE, 2111 },
	{ 0x7ee8, 0x7ee8, PDF_CMAP_SINGLE, 6120 },
	{ 0x7ee9, 0x7ee9, PDF_CMAP_SINGLE, 2075 },
	{ 0x7eea, 0x7eea, PDF_CMAP_SINGLE, 4034 },
	{ 0x7eeb, 0x7eeb, PDF_CMAP_SINGLE, 6121 },
	{ 0x7eec, 0x7eec, PDF_CMAP_SINGLE, 17371 },
	{ 0x7eed, 0x7eed, PDF_CMAP_SINGLE, 4035 },
	{ 0x7eee, 0x7eef, PDF_CMAP_RANGE, 6122 },
	{ 0x7ef0, 0x7ef0, PDF_CMAP_SINGLE, 1349 },
	{ 0x7ef1, 0x7ef2, PDF_CMAP_RANGE, 6124 },
	{ 0x7ef3, 0x7ef3, PDF_CMAP_SINGLE, 3383 },
	{ 0x7ef4, 0x7ef4, PDF_CMAP_SINGLE, 3771 },
	{ 0x7ef5, 0x7ef5, PDF_CMAP_SINGLE, 2789 },
	{ 0x7ef6, 0x7ef6, PDF_CMAP_SINGLE, 6127 },
	{ 0x7ef7, 0x7ef7, PDF_CMAP_SINGLE, 1066 },
	{ 0x7ef8, 0x7ef8, PDF_CMAP_SINGLE, 1302 },
	{ 0x7ef9, 0x7ef9, PDF_CMAP_SINGLE, 17372 },
	{ 0x7efa, 0x7efb, PDF_CMAP_RANGE, 6128 },
	{ 0x7efc, 0x7efc, PDF_CMAP_SINGLE, 4664 },
	{ 0x7efd, 0x7efd, PDF_CMAP_SINGLE, 4449 },
	{ 0x7efe, 0x7efe, PDF_CMAP_SINGLE, 6130 },
	{ 0x7eff, 0x7eff, PDF_CMAP_SINGLE, 2675 },
	{ 0x7f00, 0x7f00, PDF_CMAP_SINGLE, 4631 },
	{ 0x7f01, 0x7f03, PDF_CMAP_RANGE, 6131 },
	{ 0x7f04, 0x7f04, PDF_CMAP_SINGLE, 2141 },
	{ 0x7f05, 0x7f05, PDF_CMAP_SINGLE, 2794 },
	{ 0x7f06, 0x7f06, PDF_CMAP_SINGLE, 2477 },
	{ 0x7f07, 0x7f08, PDF_CMAP_RANGE, 6134 },
	{ 0x7f09, 0x7f09, PDF_CMAP_SINGLE, 2076 },
	{ 0x7f0a, 0x7f0a, PDF_CMAP_SINGLE, 17373 },
	{ 0x7f0b, 0x7f0c, PDF_CMAP_RANGE, 6136 },
	{ 0x7f0d, 0x7f0d, PDF_CMAP_SINGLE, 6126 },
	{ 0x7f0e, 0x7f0e, PDF_CMAP_SINGLE, 1551 },
	{ 0x7f0f, 0x7f0f, PDF_CMAP_SINGLE, 6138 },
	{ 0x7f10, 0x7f10, PDF_CMAP_SINGLE, 17374 },
	{ 0x7f11, 0x7f12, PDF_CMAP_RANGE, 6139 },
	{ 0x7f13, 0x7f13, PDF_CMAP_SINGLE, 1999 },
	{ 0x7f14, 0x7f14, PDF_CMAP_SINGLE, 1471 },
	{ 0x7f15, 0x7f15, PDF_CMAP_SINGLE, 2669 },
	{ 0x7f16, 0x7f16, PDF_CMAP_SINGLE, 1097 },
	{ 0x7f17, 0x7f17, PDF_CMAP_SINGLE, 6141 },
	{ 0x7f18, 0x7f18, PDF_CMAP_SINGLE, 4344 },
	{ 0x7f19, 0x7f19, PDF_CMAP_SINGLE, 6142 },
	{ 0x7f1a, 0x7f1a, PDF_CMAP_SINGLE, 1722 },
	{ 0x7f1b, 0x7f1b, PDF_CMAP_SINGLE, 6144 },
	{ 0x7f1c, 0x7f1c, PDF_CMAP_SINGLE, 6143 },
	{ 0x7f1d, 0x7f1d, PDF_CMAP_SINGLE, 1673 },
	{ 0x7f1e, 0x7f1e, PDF_CMAP_SINGLE, 17375 },
	{ 0x7f1f, 0x7f1f, PDF_CMAP_SINGLE, 6145 },
	{ 0x7f20, 0x7f20, PDF_CMAP_SINGLE, 1215 },
	{ 0x7f21, 0x7f27, PDF_CMAP_RANGE, 6146 },
	{ 0x7f28, 0x7f28, PDF_CMAP_SINGLE, 4236 },
	{ 0x7f29, 0x7f29, PDF_CMAP_SINGLE, 3562 },
	{ 0x7f2a, 0x7f2d, PDF_CMAP_RANGE, 6153 },
	{ 0x7f2e, 0x7f2e, PDF_CMAP_SINGLE, 3330 },
	{ 0x7f2f, 0x7f33, PDF_CMAP_RANGE, 6157 },
	{ 0x7f34, 0x7f34, PDF_CMAP_SINGLE, 2202 },
	{ 0x7f35, 0x7f35, PDF_CMAP_SINGLE, 6162 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 7262 },
	{ 0x7f37, 0x7f37, PDF_CMAP_SINGLE, 17376 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 1746 },
	{ 0x7f39, 0x7f39, PDF_CMAP_SINGLE, 17377 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 3212 },
	{ 0x7f3b, 0x7f41, PDF_CMAP_RANGE, 17378 },
	{ 0x7f42, 0x7f42, PDF_CMAP_SINGLE, 7263 },
	{ 0x7f43, 0x7f43, PDF_CMAP_SINGLE, 17385 },
	{ 0x7f44, 0x7f45, PDF_CMAP_RANGE, 7264 },
	{ 0x7f46, 0x7f4b, PDF_CMAP_RANGE, 17386 },
	{ 0x7f4c, 0x7f4c, PDF_CMAP_SINGLE, 9698 },
	{ 0x7f4d, 0x7f4d, PDF_CMAP_SINGLE, 17392 },
	{ 0x7f4e, 0x7f4e, PDF_CMAP_SINGLE, 9882 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 17393 },
	{ 0x7f50, 0x7f50, PDF_CMAP_SINGLE, 1847 },
	{ 0x7f51, 0x7f51, PDF_CMAP_SINGLE, 3753 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 17394 },
	{ 0x7f54, 0x7f54, PDF_CMAP_SINGLE, 4766 },
	{ 0x7f55, 0x7f55, PDF_CMAP_SINGLE, 1896 },
	{ 0x7f56, 0x7f56, PDF_CMAP_SINGLE, 17396 },
	{ 0x7f57, 0x7f57, PDF_CMAP_SINGLE, 2693 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 6785 },
	{ 0x7f59, 0x7f59, PDF_CMAP_SINGLE, 17397 },
	{ 0x7f5a, 0x7f5a, PDF_CMAP_SINGLE, 1600 },
	{ 0x7f5b, 0x7f5e, PDF_CMAP_RANGE, 17398 },
	{ 0x7f5f, 0x7f5f, PDF_CMAP_SINGLE, 6787 },
	{ 0x7f60, 0x7f60, PDF_CMAP_SINGLE, 17402 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 6786 },
	{ 0x7f62, 0x7f62, PDF_CMAP_SINGLE, 992 },
	{ 0x7f63, 0x7f67, PDF_CMAP_RANGE, 17403 },
	{ 0x7f68, 0x7f68, PDF_CMAP_SINGLE, 6789 },
	{ 0x7f69, 0x7f69, PDF_CMAP_SINGLE, 4471 },
	{ 0x7f6a, 0x7f6a, PDF_CMAP_SINGLE, 4684 },
	{ 0x7f6b, 0x7f6d, PDF_CMAP_RANGE, 17408 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 4546 },
	{ 0x7f6f, 0x7f6f, PDF_CMAP_SINGLE, 17411 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 7914 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 6791 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 3463 },
	{ 0x7f73, 0x7f73, PDF_CMAP_SINGLE, 17412 },
	{ 0x7f74, 0x7f74, PDF_CMAP_SINGLE, 6790 },
	{ 0x7f75, 0x7f76, PDF_CMAP_RANGE, 17413 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 7724 },
	{ 0x7f78, 0x7f78, PDF_CMAP_SINGLE, 17415 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 6792 },
	{ 0x7f7a, 0x7f7d, PDF_CMAP_RANGE, 17416 },
	{ 0x7f7e, 0x7f7e, PDF_CMAP_SINGLE, 6794 },
	{ 0x7f7f, 0x7f80, PDF_CMAP_RANGE, 17420 },
	{ 0x7f81, 0x7f81, PDF_CMAP_SINGLE, 6793 },
	{ 0x7f82, 0x7f84, PDF_CMAP_RANGE, 17422 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 8290 },
	{ 0x7f86, 0x7f86, PDF_CMAP_SINGLE, 9457 },
	{ 0x7f87, 0x7f87, PDF_CMAP_SINGLE, 17425 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 9458 },
	{ 0x7f89, 0x7f89, PDF_CMAP_SINGLE, 17426 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 4123 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 17427 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 3129 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 17428 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 2757 },
	{ 0x7f8f, 0x7f93, PDF_CMAP_RANGE, 17429 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 1756 },
	{ 0x7f95, 0x7f99, PDF_CMAP_RANGE, 17434 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 2608 },
	{ 0x7f9b, 0x7f9c, PDF_CMAP_RANGE, 17439 },
	{ 0x7f9d, 0x7f9d, PDF_CMAP_SINGLE, 7364 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 4010 },
	{ 0x7f9f, 0x7f9f, PDF_CMAP_SINGLE, 7365 },
	{ 0x7fa0, 0x7fa0, PDF_CMAP_SINGLE, 17441 },
	{ 0x7fa1, 0x7fa1, PDF_CMAP_SINGLE, 3912 },
	{ 0x7fa2, 0x7fa3, PDF_CMAP_RANGE, 17442 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 3221 },
	{ 0x7fa5, 0x7fa5, PDF_CMAP_SINGLE, 9714 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 17444 },
	{ 0x7fa7, 0x7fa7, PDF_CMAP_SINGLE, 7366 },
	{ 0x7fa8, 0x7fa8, PDF_CMAP_SINGLE, 17445 },
	{ 0x7fa9, 0x7fa9, PDF_CMAP_SINGLE, 8725 },
	{ 0x7faa, 0x7fae, PDF_CMAP_RANGE, 17446 },
	{ 0x7faf, 0x7fb0, PDF_CMAP_RANGE, 7367 },
	{ 0x7fb1, 0x7fb1, PDF_CMAP_SINGLE, 17451 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 7369 },
	{ 0x7fb3, 0x7fb7, PDF_CMAP_RANGE, 17452 },
	{ 0x7fb8, 0x7fb8, PDF_CMAP_SINGLE, 4878 },
	{ 0x7fb9, 0x7fb9, PDF_CMAP_SINGLE, 1785 },
	{ 0x7fba, 0x7fbb, PDF_CMAP_RANGE, 17457 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 5997 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 4309 },
	{ 0x7fbe, 0x7fbe, PDF_CMAP_SINGLE, 17459 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 7390 },
	{ 0x7fc0, 0x7fc0, PDF_CMAP_SINGLE, 17460 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 3803 },
	{ 0x7fc2, 0x7fc4, PDF_CMAP_RANGE, 17461 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 1286 },
	{ 0x7fc6, 0x7fc9, PDF_CMAP_RANGE, 17464 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 7064 },
	{ 0x7fcb, 0x7fcb, PDF_CMAP_SINGLE, 17468 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 4213 },
	{ 0x7fcd, 0x7fcd, PDF_CMAP_SINGLE, 17469 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 7391 },
	{ 0x7fcf, 0x7fd1, PDF_CMAP_RANGE, 17470 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 8622 },
	{ 0x7fd3, 0x7fd3, PDF_CMAP_SINGLE, 17473 },
	{ 0x7fd4, 0x7fd4, PDF_CMAP_SINGLE, 3925 },
	{ 0x7fd5, 0x7fd5, PDF_CMAP_SINGLE, 7392 },
	{ 0x7fd6, 0x7fd7, PDF_CMAP_RANGE, 17474 },
	{ 0x7fd8, 0x7fd8, PDF_CMAP_SINGLE, 3145 },
	{ 0x7fd9, 0x7fde, PDF_CMAP_RANGE, 17476 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 1461 },
	{ 0x7fe0, 0x7fe0, PDF_CMAP_SINGLE, 1383 },
	{ 0x7fe1, 0x7fe1, PDF_CMAP_SINGLE, 7394 },
	{ 0x7fe2, 0x7fe4, PDF_CMAP_RANGE, 17482 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 7393 },
	{ 0x7fe6, 0x7fe6, PDF_CMAP_SINGLE, 7395 },
	{ 0x7fe7, 0x7fe8, PDF_CMAP_RANGE, 17485 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 7396 },
	{ 0x7fea, 0x7fed, PDF_CMAP_RANGE, 17487 },
	{ 0x7fee, 0x7fee, PDF_CMAP_SINGLE, 7397 },
	{ 0x7fef, 0x7fef, PDF_CMAP_SINGLE, 17491 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 1897 },
	{ 0x7ff1, 0x7ff1, PDF_CMAP_SINGLE, 970 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 17492 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 7398 },
	{ 0x7ff4, 0x7ff8, PDF_CMAP_RANGE, 17493 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 8416 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 17498 },
	{ 0x7ffb, 0x7ffb, PDF_CMAP_SINGLE, 1610 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 4212 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 17499 },
	{ 0x8000, 0x8000, PDF_CMAP_SINGLE, 4146 },
	{ 0x8001, 0x8001, PDF_CMAP_SINGLE, 2490 },
	{ 0x8002, 0x8002, PDF_CMAP_SINGLE, 17502 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 2377 },
	{ 0x8004, 0x8004, PDF_CMAP_SINGLE, 6483 },
	{ 0x8005, 0x8005, PDF_CMAP_SINGLE, 4480 },
	{ 0x8006, 0x8006, PDF_CMAP_SINGLE, 6426 },
	{ 0x8007, 0x800a, PDF_CMAP_RANGE, 17503 },
	{ 0x800b, 0x800b, PDF_CMAP_SINGLE, 7127 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 1591 },
	{ 0x800d, 0x800d, PDF_CMAP_SINGLE, 3480 },
	{ 0x800e, 0x800f, PDF_CMAP_RANGE, 17507 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 2864 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 17509 },
	{ 0x8012, 0x8012, PDF_CMAP_SINGLE, 7115 },
	{ 0x8013, 0x8013, PDF_CMAP_SINGLE, 17510 },
	{ 0x8014, 0x8014, PDF_CMAP_SINGLE, 7116 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 1782 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 7117 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 1915 },
	{ 0x8018, 0x8018, PDF_CMAP_SINGLE, 4360 },
	{ 0x8019, 0x8019, PDF_CMAP_SINGLE, 989 },
	{ 0x801a, 0x801b, PDF_CMAP_RANGE, 17511 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 7118 },
	{ 0x801d, 0x801f, PDF_CMAP_RANGE, 17513 },
	{ 0x8020, 0x8020, PDF_CMAP_SINGLE, 7119 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 17516 },
	{ 0x8022, 0x8022, PDF_CMAP_SINGLE, 7120 },
	{ 0x8023, 0x8024, PDF_CMAP_RANGE, 17517 },
	{ 0x8025, 0x8027, PDF_CMAP_RANGE, 7121 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 7125 },
	{ 0x8029, 0x8029, PDF_CMAP_SINGLE, 7124 },
	{ 0x802a, 0x802a, PDF_CMAP_SINGLE, 2966 },
	{ 0x802b, 0x802b, PDF_CMAP_SINGLE, 17519 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 9666 },
	{ 0x802d, 0x802d, PDF_CMAP_SINGLE, 17520 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 9665 },
	{ 0x802f, 0x8030, PDF_CMAP_RANGE, 17521 },
	{ 0x8031, 0x8031, PDF_CMAP_SINGLE, 7126 },
	{ 0x8032, 0x8032, PDF_CMAP_SINGLE, 17523 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 1593 },
	{ 0x8034, 0x8034, PDF_CMAP_SINGLE, 17524 },
	{ 0x8035, 0x8035, PDF_CMAP_SINGLE, 7128 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 4149 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 5297 },
	{ 0x8038, 0x8038, PDF_CMAP_SINGLE, 3519 },
	{ 0x8039, 0x803a, PDF_CMAP_RANGE, 17525 },
	{ 0x803b, 0x803b, PDF_CMAP_SINGLE, 1281 },
	{ 0x803c, 0x803c, PDF_CMAP_SINGLE, 17527 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 1411 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 17528 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 1787 },
	{ 0x8040, 0x8041, PDF_CMAP_RANGE, 17529 },
	{ 0x8042, 0x8042, PDF_CMAP_SINGLE, 2903 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 7129 },
	{ 0x8044, 0x8045, PDF_CMAP_RANGE, 17531 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 7130 },
	{ 0x8047, 0x8049, PDF_CMAP_RANGE, 17533 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 2573 },
	{ 0x804b, 0x804b, PDF_CMAP_SINGLE, 2628 },
	{ 0x804c, 0x804c, PDF_CMAP_SINGLE, 4527 },
	{ 0x804d, 0x804d, PDF_CMAP_SINGLE, 7131 },
	{ 0x804e, 0x8051, PDF_CMAP_RANGE, 17536 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 7132 },
	{ 0x8053, 0x8053, PDF_CMAP_SINGLE, 17540 },
	{ 0x8054, 0x8054, PDF_CMAP_SINGLE, 2547 },
	{ 0x8055, 0x8055, PDF_CMAP_SINGLE, 17541 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 8488 },
	{ 0x8057, 0x8057, PDF_CMAP_SINGLE, 17542 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 3031 },
	{ 0x8059, 0x8059, PDF_CMAP_SINGLE, 17543 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 2314 },
	{ 0x805b, 0x805d, PDF_CMAP_RANGE, 17544 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 8601 },
	{ 0x805f, 0x8068, PDF_CMAP_RANGE, 17547 },
	{ 0x8069, 0x8069, PDF_CMAP_SINGLE, 7133 },
	{ 0x806a, 0x806a, PDF_CMAP_SINGLE, 1362 },
	{ 0x806b, 0x806e, PDF_CMAP_RANGE, 17557 },
	{ 0x806f, 0x806f, PDF_CMAP_SINGLE, 8215 },
	{ 0x8070, 0x8070, PDF_CMAP_SINGLE, 7840 },
	{ 0x8071, 0x8071, PDF_CMAP_SINGLE, 7134 },
	{ 0x8072, 0x8072, PDF_CMAP_SINGLE, 8485 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 8525 },
	{ 0x8074, 0x8074, PDF_CMAP_SINGLE, 17561 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 9668 },
	{ 0x8076, 0x8076, PDF_CMAP_SINGLE, 8349 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 8842 },
	{ 0x8078, 0x8078, PDF_CMAP_SINGLE, 17562 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 9667 },
	{ 0x807a, 0x807c, PDF_CMAP_RANGE, 17563 },
	{ 0x807d, 0x807d, PDF_CMAP_SINGLE, 8569 },
	{ 0x807e, 0x807e, PDF_CMAP_SINGLE, 8248 },
	{ 0x807f, 0x807f, PDF_CMAP_SINGLE, 6686 },
	{ 0x8080, 0x8080, PDF_CMAP_SINGLE, 6685 },
	{ 0x8081, 0x8082, PDF_CMAP_RANGE, 17566 },
	{ 0x8083, 0x8083, PDF_CMAP_SINGLE, 3541 },
	{ 0x8084, 0x8084, PDF_CMAP_SINGLE, 4197 },
	{ 0x8085, 0x8085, PDF_CMAP_SINGLE, 8533 },
	{ 0x8086, 0x8086, PDF_CMAP_SINGLE, 3510 },
	{ 0x8087, 0x8087, PDF_CMAP_SINGLE, 4473 },
	{ 0x8088, 0x8088, PDF_CMAP_SINGLE, 17568 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 3261 },
	{ 0x808a, 0x808a, PDF_CMAP_SINGLE, 17569 },
	{ 0x808b, 0x808b, PDF_CMAP_SINGLE, 2506 },
	{ 0x808c, 0x808c, PDF_CMAP_SINGLE, 2068 },
	{ 0x808d, 0x8092, PDF_CMAP_RANGE, 17570 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6513 },
	{ 0x8094, 0x8095, PDF_CMAP_RANGE, 17576 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 3951 },
	{ 0x8097, 0x8097, PDF_CMAP_SINGLE, 17578 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 4577 },
	{ 0x8099, 0x8099, PDF_CMAP_SINGLE, 17579 },
	{ 0x809a, 0x809a, PDF_CMAP_SINGLE, 1542 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 1747 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 6512 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 1737 },
	{ 0x809e, 0x809e, PDF_CMAP_SINGLE, 17580 },
	{ 0x809f, 0x809f, PDF_CMAP_SINGLE, 6511 },
	{ 0x80a0, 0x80a0, PDF_CMAP_SINGLE, 1227 },
	{ 0x80a1, 0x80a1, PDF_CMAP_SINGLE, 1826 },
	{ 0x80a2, 0x80a2, PDF_CMAP_SINGLE, 4522 },
	{ 0x80a3, 0x80a3, PDF_CMAP_SINGLE, 17581 },
	{ 0x80a4, 0x80a4, PDF_CMAP_SINGLE, 1681 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 1639 },
	{ 0x80a6, 0x80a8, PDF_CMAP_RANGE, 17582 },
	{ 0x80a9, 0x80a9, PDF_CMAP_SINGLE, 2138 },
	{ 0x80aa, 0x80aa, PDF_CMAP_SINGLE, 1627 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 6518 },
	{ 0x80ac, 0x80ac, PDF_CMAP_SINGLE, 17585 },
	{ 0x80ad, 0x80ad, PDF_CMAP_SINGLE, 6519 },
	{ 0x80ae, 0x80ae, PDF_CMAP_SINGLE, 964 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 2396 },
	{ 0x80b0, 0x80b0, PDF_CMAP_SINGLE, 17586 },
	{ 0x80b1, 0x80b1, PDF_CMAP_SINGLE, 6517 },
	{ 0x80b2, 0x80b2, PDF_CMAP_SINGLE, 4322 },
	{ 0x80b3, 0x80b3, PDF_CMAP_SINGLE, 17587 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 6520 },
	{ 0x80b5, 0x80b6, PDF_CMAP_RANGE, 17588 },
	{ 0x80b7, 0x80b7, PDF_CMAP_SINGLE, 6521 },
	{ 0x80b8, 0x80b9, PDF_CMAP_RANGE, 17590 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 1643 },
	{ 0x80bb, 0x80bb, PDF_CMAP_SINGLE, 17592 },
	{ 0x80bc, 0x80bc, PDF_CMAP_SINGLE, 6514 },
	{ 0x80bd, 0x80bd, PDF_CMAP_SINGLE, 6516 },
	{ 0x80be, 0x80be, PDF_CMAP_SINGLE, 3375 },
	{ 0x80bf, 0x80bf, PDF_CMAP_SINGLE, 4566 },
	{ 0x80c0, 0x80c0, PDF_CMAP_SINGLE, 4462 },
	{ 0x80c1, 0x80c1, PDF_CMAP_SINGLE, 3965 },
	{ 0x80c2, 0x80c2, PDF_CMAP_SINGLE, 6527 },
	{ 0x80c3, 0x80c3, PDF_CMAP_SINGLE, 3783 },
	{ 0x80c4, 0x80c4, PDF_CMAP_SINGLE, 6528 },
	{ 0x80c5, 0x80c5, PDF_CMAP_SINGLE, 17593 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 1417 },
	{ 0x80c7, 0x80cb, PDF_CMAP_RANGE, 17594 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 1052 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 6530 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 3576 },
	{ 0x80cf, 0x80d5, PDF_CMAP_RANGE, 17599 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 2967 },
	{ 0x80d7, 0x80d7, PDF_CMAP_SINGLE, 6531 },
	{ 0x80d8, 0x80d8, PDF_CMAP_SINGLE, 17606 },
	{ 0x80d9, 0x80d9, PDF_CMAP_SINGLE, 6529 },
	{ 0x80da, 0x80da, PDF_CMAP_SINGLE, 2976 },
	{ 0x80db, 0x80db, PDF_CMAP_SINGLE, 6526 },
	{ 0x80dc, 0x80dc, PDF_CMAP_SINGLE, 3387 },
	{ 0x80dd, 0x80dd, PDF_CMAP_SINGLE, 6533 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 1030 },
	{ 0x80df, 0x80e0, PDF_CMAP_RANGE, 17607 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1969 },
	{ 0x80e2, 0x80e3, PDF_CMAP_RANGE, 17609 },
	{ 0x80e4, 0x80e4, PDF_CMAP_SINGLE, 4717 },
	{ 0x80e5, 0x80e5, PDF_CMAP_SINGLE, 7111 },
	{ 0x80e6, 0x80e6, PDF_CMAP_SINGLE, 17611 },
	{ 0x80e7, 0x80ea, PDF_CMAP_RANGE, 6522 },
	{ 0x80eb, 0x80eb, PDF_CMAP_SINGLE, 6534 },
	{ 0x80ec, 0x80ec, PDF_CMAP_SINGLE, 6042 },
	{ 0x80ed, 0x80ed, PDF_CMAP_SINGLE, 6537 },
	{ 0x80ee, 0x80ee, PDF_CMAP_SINGLE, 17612 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 2421 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 4175 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 6535 },
	{ 0x80f2, 0x80f2, PDF_CMAP_SINGLE, 6540 },
	{ 0x80f3, 0x80f3, PDF_CMAP_SINGLE, 1767 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 6536 },
	{ 0x80f5, 0x80f5, PDF_CMAP_SINGLE, 17613 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 2187 },
	{ 0x80f7, 0x80f7, PDF_CMAP_SINGLE, 17614 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 4003 },
	{ 0x80f9, 0x80f9, PDF_CMAP_SINGLE, 17615 },
	{ 0x80fa, 0x80fa, PDF_CMAP_SINGLE, 962 },
	{ 0x80fb, 0x80fb, PDF_CMAP_SINGLE, 17616 },
	{ 0x80fc, 0x80fc, PDF_CMAP_SINGLE, 6541 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 2879 },
	{ 0x80fe, 0x80ff, PDF_CMAP_RANGE, 17617 },
	{ 0x8100, 0x8101, PDF_CMAP_RANGE, 17619 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 4523 },
	{ 0x8103, 0x8104, PDF_CMAP_RANGE, 17621 },
	{ 0x8105, 0x8105, PDF_CMAP_SINGLE, 8658 },
	{ 0x8106, 0x8106, PDF_CMAP_SINGLE, 1379 },
	{ 0x8107, 0x8108, PDF_CMAP_RANGE, 17623 },
	{ 0x8109, 0x8109, PDF_CMAP_SINGLE, 2717 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 2092 },
	{ 0x810b, 0x810c, PDF_CMAP_RANGE, 17625 },
	{ 0x810d, 0x810e, PDF_CMAP_RANGE, 6538 },
	{ 0x810f, 0x810f, PDF_CMAP_SINGLE, 4387 },
	{ 0x8110, 0x8110, PDF_CMAP_SINGLE, 3081 },
	{ 0x8111, 0x8111, PDF_CMAP_SINGLE, 2871 },
	{ 0x8112, 0x8112, PDF_CMAP_SINGLE, 6543 },
	{ 0x8113, 0x8113, PDF_CMAP_SINGLE, 2920 },
	{ 0x8114, 0x8114, PDF_CMAP_SINGLE, 4873 },
	{ 0x8115, 0x8115, PDF_CMAP_SINGLE, 17627 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 1144 },
	{ 0x8117, 0x8117, PDF_CMAP_SINGLE, 17628 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 6548 },
	{ 0x8119, 0x8119, PDF_CMAP_SINGLE, 17629 },
	{ 0x811a, 0x811a, PDF_CMAP_SINGLE, 2198 },
	{ 0x811b, 0x811b, PDF_CMAP_SINGLE, 9415 },
	{ 0x811c, 0x811d, PDF_CMAP_RANGE, 17630 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 6546 },
	{ 0x811f, 0x812b, PDF_CMAP_RANGE, 17632 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 6547 },
	{ 0x812d, 0x812e, PDF_CMAP_RANGE, 17645 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 1703 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 17647 },
	{ 0x8131, 0x8131, PDF_CMAP_SINGLE, 3714 },
	{ 0x8132, 0x8132, PDF_CMAP_SINGLE, 6549 },
	{ 0x8133, 0x8135, PDF_CMAP_RANGE, 17648 },
	{ 0x8136, 0x8136, PDF_CMAP_SINGLE, 6545 },
	{ 0x8137, 0x8137, PDF_CMAP_SINGLE, 17651 },
	{ 0x8138, 0x8138, PDF_CMAP_SINGLE, 2556 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 8825 },
	{ 0x813a, 0x813d, PDF_CMAP_RANGE, 17652 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 3009 },
	{ 0x813f, 0x8145, PDF_CMAP_RANGE, 17656 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 3654 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 17663 },
	{ 0x8148, 0x8148, PDF_CMAP_SINGLE, 6550 },
	{ 0x8149, 0x8149, PDF_CMAP_SINGLE, 17664 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 2459 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 4159 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 6551 },
	{ 0x814d, 0x814d, PDF_CMAP_SINGLE, 17665 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 8483 },
	{ 0x814f, 0x814f, PDF_CMAP_SINGLE, 17666 },
	{ 0x8150, 0x8150, PDF_CMAP_SINGLE, 1706 },
	{ 0x8151, 0x8151, PDF_CMAP_SINGLE, 1704 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 17667 },
	{ 0x8153, 0x8153, PDF_CMAP_SINGLE, 6552 },
	{ 0x8154, 0x8154, PDF_CMAP_SINGLE, 3128 },
	{ 0x8155, 0x8155, PDF_CMAP_SINGLE, 3748 },
	{ 0x8156, 0x8156, PDF_CMAP_SINGLE, 9413 },
	{ 0x8157, 0x8158, PDF_CMAP_RANGE, 17668 },
	{ 0x8159, 0x815a, PDF_CMAP_RANGE, 6554 },
	{ 0x815b, 0x815f, PDF_CMAP_RANGE, 17670 },
	{ 0x8160, 0x8160, PDF_CMAP_SINGLE, 6557 },
	{ 0x8161, 0x8161, PDF_CMAP_SINGLE, 9417 },
	{ 0x8162, 0x8164, PDF_CMAP_RANGE, 17675 },
	{ 0x8165, 0x8165, PDF_CMAP_SINGLE, 3987 },
	{ 0x8166, 0x8166, PDF_CMAP_SINGLE, 8340 },
	{ 0x8167, 0x8167, PDF_CMAP_SINGLE, 6562 },
	{ 0x8168, 0x8168, PDF_CMAP_SINGLE, 17678 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 6558 },
	{ 0x816a, 0x816a, PDF_CMAP_SINGLE, 17679 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 8854 },
	{ 0x816c, 0x816c, PDF_CMAP_SINGLE, 17680 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 6561 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 3284 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 17681 },
	{ 0x8170, 0x8170, PDF_CMAP_SINGLE, 4133 },
	{ 0x8171, 0x8171, PDF_CMAP_SINGLE, 6556 },
	{ 0x8172, 0x8173, PDF_CMAP_RANGE, 17682 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6553 },
	{ 0x8175, 0x8177, PDF_CMAP_RANGE, 17684 },
	{ 0x8178, 0x8178, PDF_CMAP_SINGLE, 7799 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 1716 },
	{ 0x817a, 0x817a, PDF_CMAP_SINGLE, 3910 },
	{ 0x817b, 0x817b, PDF_CMAP_SINGLE, 2888 },
	{ 0x817c, 0x817d, PDF_CMAP_RANGE, 6559 },
	{ 0x817e, 0x817e, PDF_CMAP_SINGLE, 3629 },
	{ 0x817f, 0x817f, PDF_CMAP_SINGLE, 3705 },
	{ 0x8180, 0x8180, PDF_CMAP_SINGLE, 1021 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 17687 },
	{ 0x8182, 0x8182, PDF_CMAP_SINGLE, 6566 },
	{ 0x8183, 0x8187, PDF_CMAP_RANGE, 17688 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 6565 },
	{ 0x8189, 0x8189, PDF_CMAP_SINGLE, 17693 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 1145 },
	{ 0x818b, 0x818e, PDF_CMAP_RANGE, 17694 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 1755 },
	{ 0x8190, 0x8190, PDF_CMAP_SINGLE, 17698 },
	{ 0x8191, 0x8191, PDF_CMAP_SINGLE, 6567 },
	{ 0x8192, 0x8197, PDF_CMAP_RANGE, 17699 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 1109 },
	{ 0x8199, 0x8199, PDF_CMAP_SINGLE, 17705 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 7942 },
	{ 0x819b, 0x819b, PDF_CMAP_SINGLE, 3608 },
	{ 0x819c, 0x819c, PDF_CMAP_SINGLE, 2823 },
	{ 0x819d, 0x819d, PDF_CMAP_SINGLE, 3858 },
	{ 0x819e, 0x819f, PDF_CMAP_RANGE, 17706 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 8101 },
	{ 0x81a1, 0x81a2, PDF_CMAP_RANGE, 17708 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 6569 },
	{ 0x81a4, 0x81a5, PDF_CMAP_RANGE, 17710 },
	{ 0x81a6, 0x81a6, PDF_CMAP_SINGLE, 6576 },
	{ 0x81a7, 0x81a7, PDF_CMAP_SINGLE, 17712 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 2995 },
	{ 0x81a9, 0x81a9, PDF_CMAP_SINGLE, 8345 },
	{ 0x81aa, 0x81aa, PDF_CMAP_SINGLE, 6570 },
	{ 0x81ab, 0x81b2, PDF_CMAP_RANGE, 17713 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 3326 },
	{ 0x81b4, 0x81b9, PDF_CMAP_RANGE, 17721 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 5697 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 6574 },
	{ 0x81bc, 0x81bc, PDF_CMAP_SINGLE, 17727 },
	{ 0x81bd, 0x81bd, PDF_CMAP_SINGLE, 7853 },
	{ 0x81be, 0x81be, PDF_CMAP_SINGLE, 9416 },
	{ 0x81bf, 0x81bf, PDF_CMAP_SINGLE, 8360 },
	{ 0x81c0, 0x81c0, PDF_CMAP_SINGLE, 3711 },
	{ 0x81c1, 0x81c1, PDF_CMAP_SINGLE, 6575 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 1092 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 4252 },
	{ 0x81c4, 0x81c5, PDF_CMAP_RANGE, 17728 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 4195 },
	{ 0x81c7, 0x81c8, PDF_CMAP_RANGE, 17730 },
	{ 0x81c9, 0x81c9, PDF_CMAP_SINGLE, 8223 },
	{ 0x81ca, 0x81ca, PDF_CMAP_SINGLE, 6573 },
	{ 0x81cb, 0x81cb, PDF_CMAP_SINGLE, 17732 },
	{ 0x81cc, 0x81cc, PDF_CMAP_SINGLE, 6571 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 8389 },
	{ 0x81ce, 0x81ce, PDF_CMAP_SINGLE, 17733 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 9418 },
	{ 0x81d0, 0x81d7, PDF_CMAP_RANGE, 17734 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 8176 },
	{ 0x81d9, 0x81d9, PDF_CMAP_SINGLE, 17742 },
	{ 0x81da, 0x81da, PDF_CMAP_SINGLE, 9414 },
	{ 0x81db, 0x81de, PDF_CMAP_RANGE, 17743 },
	{ 0x81df, 0x81df, PDF_CMAP_SINGLE, 8797 },
	{ 0x81e0, 0x81e0, PDF_CMAP_SINGLE, 8926 },
	{ 0x81e1, 0x81e2, PDF_CMAP_RANGE, 17747 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 1249 },
	{ 0x81e4, 0x81e6, PDF_CMAP_RANGE, 17749 },
	{ 0x81e7, 0x81e7, PDF_CMAP_SINGLE, 6402 },
	{ 0x81e8, 0x81e8, PDF_CMAP_SINGLE, 8236 },
	{ 0x81e9, 0x81e9, PDF_CMAP_SINGLE, 17752 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 4657 },
	{ 0x81eb, 0x81eb, PDF_CMAP_SINGLE, 17753 },
	{ 0x81ec, 0x81ec, PDF_CMAP_SINGLE, 7337 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 1305 },
	{ 0x81ee, 0x81f2, PDF_CMAP_RANGE, 17754 },
	{ 0x81f3, 0x81f4, PDF_CMAP_RANGE, 4544 },
	{ 0x81f5, 0x81f9, PDF_CMAP_RANGE, 17759 },
	{ 0x81fa, 0x81fa, PDF_CMAP_SINGLE, 8545 },
	{ 0x81fb, 0x81fb, PDF_CMAP_SINGLE, 4490 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 2297 },
	{ 0x81fd, 0x81fd, PDF_CMAP_SINGLE, 17764 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 7333 },
	{ 0x81ff, 0x81ff, PDF_CMAP_SINGLE, 17765 },
	{ 0x8200, 0x8200, PDF_CMAP_SINGLE, 4143 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 7334 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 17766 },
	{ 0x8204, 0x8204, PDF_CMAP_SINGLE, 7336 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 2298 },
	{ 0x8206, 0x8206, PDF_CMAP_SINGLE, 4292 },
	{ 0x8207, 0x8207, PDF_CMAP_SINGLE, 8763 },
	{ 0x8208, 0x8208, PDF_CMAP_SINGLE, 8665 },
	{ 0x8209, 0x8209, PDF_CMAP_SINGLE, 8139 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 8137 },
	{ 0x820b, 0x820b, PDF_CMAP_SINGLE, 17767 },
	{ 0x820c, 0x820d, PDF_CMAP_RANGE, 3353 },
	{ 0x820e, 0x820f, PDF_CMAP_RANGE, 17768 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 7266 },
	{ 0x8211, 0x8211, PDF_CMAP_SINGLE, 17770 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 3453 },
	{ 0x8213, 0x8213, PDF_CMAP_SINGLE, 17771 },
	{ 0x8214, 0x8214, PDF_CMAP_SINGLE, 3653 },
	{ 0x8215, 0x821a, PDF_CMAP_RANGE, 17772 },
	{ 0x821b, 0x821b, PDF_CMAP_SINGLE, 5656 },
	{ 0x821c, 0x821c, PDF_CMAP_SINGLE, 3497 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 17778 },
	{ 0x821e, 0x821e, PDF_CMAP_SINGLE, 3831 },
	{ 0x821f, 0x821f, PDF_CMAP_SINGLE, 4570 },
	{ 0x8220, 0x8220, PDF_CMAP_SINGLE, 17779 },
	{ 0x8221, 0x8223, PDF_CMAP_RANGE, 7339 },
	{ 0x8224, 0x8227, PDF_CMAP_RANGE, 17780 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 7344 },
	{ 0x8229, 0x8229, PDF_CMAP_SINGLE, 17784 },
	{ 0x822a, 0x822a, PDF_CMAP_SINGLE, 1908 },
	{ 0x822b, 0x822b, PDF_CMAP_SINGLE, 7345 },
	{ 0x822c, 0x822c, PDF_CMAP_SINGLE, 1006 },
	{ 0x822d, 0x822d, PDF_CMAP_SINGLE, 7342 },
	{ 0x822e, 0x822e, PDF_CMAP_SINGLE, 17785 },
	{ 0x822f, 0x822f, PDF_CMAP_SINGLE, 7343 },
	{ 0x8230, 0x8230, PDF_CMAP_SINGLE, 2163 },
	{ 0x8231, 0x8231, PDF_CMAP_SINGLE, 1180 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 17786 },
	{ 0x8233, 0x8234, PDF_CMAP_RANGE, 7348 },
	{ 0x8235, 0x8235, PDF_CMAP_SINGLE, 1573 },
	{ 0x8236, 0x8236, PDF_CMAP_SINGLE, 1143 },
	{ 0x8237, 0x8237, PDF_CMAP_SINGLE, 3900 },
	{ 0x8238, 0x8238, PDF_CMAP_SINGLE, 7346 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 1327 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 17787 },
	{ 0x823b, 0x823b, PDF_CMAP_SINGLE, 7347 },
	{ 0x823c, 0x823d, PDF_CMAP_RANGE, 17788 },
	{ 0x823e, 0x823e, PDF_CMAP_SINGLE, 7350 },
	{ 0x823f, 0x8243, PDF_CMAP_RANGE, 17790 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 7351 },
	{ 0x8245, 0x8246, PDF_CMAP_RANGE, 17795 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3672 },
	{ 0x8248, 0x8248, PDF_CMAP_SINGLE, 17797 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 7352 },
	{ 0x824a, 0x824a, PDF_CMAP_SINGLE, 17798 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 7353 },
	{ 0x824c, 0x824e, PDF_CMAP_RANGE, 17799 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 7354 },
	{ 0x8250, 0x8257, PDF_CMAP_RANGE, 17802 },
	{ 0x8258, 0x8258, PDF_CMAP_SINGLE, 3527 },
	{ 0x8259, 0x8259, PDF_CMAP_SINGLE, 7777 },
	{ 0x825a, 0x825a, PDF_CMAP_SINGLE, 7355 },
	{ 0x825b, 0x825e, PDF_CMAP_RANGE, 17810 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 7356 },
	{ 0x8260, 0x8263, PDF_CMAP_RANGE, 17814 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 9711 },
	{ 0x8265, 0x8265, PDF_CMAP_SINGLE, 17818 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 8088 },
	{ 0x8267, 0x8267, PDF_CMAP_SINGLE, 17819 },
	{ 0x8268, 0x8268, PDF_CMAP_SINGLE, 7357 },
	{ 0x8269, 0x826a, PDF_CMAP_RANGE, 17820 },
	{ 0x826b, 0x826b, PDF_CMAP_SINGLE, 9712 },
	{ 0x826c, 0x826d, PDF_CMAP_RANGE, 17822 },
	{ 0x826e, 0x826e, PDF_CMAP_SINGLE, 7388 },
	{ 0x826f, 0x826f, PDF_CMAP_SINGLE, 2565 },
	{ 0x8270, 0x8270, PDF_CMAP_SINGLE, 2139 },
	{ 0x8271, 0x8271, PDF_CMAP_SINGLE, 8072 },
	{ 0x8272, 0x8272, PDF_CMAP_SINGLE, 3300 },
	{ 0x8273, 0x8273, PDF_CMAP_SINGLE, 4103 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 6001 },
	{ 0x8275, 0x8276, PDF_CMAP_RANGE, 17824 },
	{ 0x8277, 0x8277, PDF_CMAP_SINGLE, 8696 },
	{ 0x8278, 0x8278, PDF_CMAP_SINGLE, 17826 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 5089 },
	{ 0x827a, 0x827a, PDF_CMAP_SINGLE, 4188 },
	{ 0x827b, 0x827c, PDF_CMAP_RANGE, 17827 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 5090 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 951 },
	{ 0x827f, 0x827f, PDF_CMAP_SINGLE, 5091 },
	{ 0x8280, 0x8281, PDF_CMAP_RANGE, 17829 },
	{ 0x8282, 0x8282, PDF_CMAP_SINGLE, 2219 },
	{ 0x8283, 0x8283, PDF_CMAP_SINGLE, 17831 },
	{ 0x8284, 0x8284, PDF_CMAP_SINGLE, 5095 },
	{ 0x8285, 0x8287, PDF_CMAP_RANGE, 17832 },
	{ 0x8288, 0x8288, PDF_CMAP_SINGLE, 4728 },
	{ 0x8289, 0x8289, PDF_CMAP_SINGLE, 17835 },
	{ 0x828a, 0x828a, PDF_CMAP_SINGLE, 5093 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 4312 },
	{ 0x828c, 0x828c, PDF_CMAP_SINGLE, 17836 },
	{ 0x828d, 0x828d, PDF_CMAP_SINGLE, 3343 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 5096 },
	{ 0x828f, 0x828f, PDF_CMAP_SINGLE, 5092 },
	{ 0x8290, 0x8290, PDF_CMAP_SINGLE, 17837 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 5097 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 2727 },
	{ 0x8293, 0x8296, PDF_CMAP_RANGE, 17838 },
	{ 0x8297, 0x8297, PDF_CMAP_SINGLE, 5098 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 5107 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 5099 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 17842 },
	{ 0x829c, 0x829c, PDF_CMAP_SINGLE, 3822 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 4516 },
	{ 0x829e, 0x829e, PDF_CMAP_SINGLE, 17844 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 5117 },
	{ 0x82a0, 0x82a0, PDF_CMAP_SINGLE, 17845 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 5115 },
	{ 0x82a2, 0x82a3, PDF_CMAP_RANGE, 17846 },
	{ 0x82a4, 0x82a4, PDF_CMAP_SINGLE, 5120 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 2231 },
	{ 0x82a6, 0x82a6, PDF_CMAP_SINGLE, 2642 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 17848 },
	{ 0x82a8, 0x82a8, PDF_CMAP_SINGLE, 5094 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 5113 },
	{ 0x82aa, 0x82aa, PDF_CMAP_SINGLE, 5116 },
	{ 0x82ab, 0x82ab, PDF_CMAP_SINGLE, 5100 },
	{ 0x82ac, 0x82ac, PDF_CMAP_SINGLE, 1647 },
	{ 0x82ad, 0x82ad, PDF_CMAP_SINGLE, 976 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 5109 },
	{ 0x82af, 0x82af, PDF_CMAP_SINGLE, 3977 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 5103 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1981 },
	{ 0x82b2, 0x82b2, PDF_CMAP_SINGLE, 17849 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 1625 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 5114 },
	{ 0x82b5, 0x82b6, PDF_CMAP_RANGE, 17850 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 5108 },
	{ 0x82b8, 0x82b8, PDF_CMAP_SINGLE, 5101 },
	{ 0x82b9, 0x82b9, PDF_CMAP_SINGLE, 3160 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 17852 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 8994 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 17853 },
	{ 0x82bd, 0x82bd, PDF_CMAP_SINGLE, 4072 },
	{ 0x82be, 0x82be, PDF_CMAP_SINGLE, 5102 },
	{ 0x82bf, 0x82c0, PDF_CMAP_RANGE, 17854 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 5112 },
	{ 0x82c2, 0x82c3, PDF_CMAP_RANGE, 17856 },
	{ 0x82c4, 0x82c4, PDF_CMAP_SINGLE, 5118 },
	{ 0x82c5, 0x82c6, PDF_CMAP_RANGE, 17858 },
	{ 0x82c7, 0x82c7, PDF_CMAP_SINGLE, 3772 },
	{ 0x82c8, 0x82c8, PDF_CMAP_SINGLE, 5104 },
	{ 0x82c9, 0x82c9, PDF_CMAP_SINGLE, 17860 },
	{ 0x82ca, 0x82ca, PDF_CMAP_SINGLE, 5105 },
	{ 0x82cb, 0x82cc, PDF_CMAP_RANGE, 5110 },
	{ 0x82cd, 0x82cd, PDF_CMAP_SINGLE, 1179 },
	{ 0x82ce, 0x82ce, PDF_CMAP_SINGLE, 5119 },
	{ 0x82cf, 0x82cf, PDF_CMAP_SINGLE, 3530 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 17861 },
	{ 0x82d1, 0x82d1, PDF_CMAP_SINGLE, 4346 },
	{ 0x82d2, 0x82d2, PDF_CMAP_SINGLE, 5129 },
	{ 0x82d3, 0x82d3, PDF_CMAP_SINGLE, 5133 },
	{ 0x82d4, 0x82d4, PDF_CMAP_SINGLE, 3577 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 5140 },
	{ 0x82d6, 0x82d6, PDF_CMAP_SINGLE, 17862 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 2796 },
	{ 0x82d8, 0x82d8, PDF_CMAP_SINGLE, 5130 },
	{ 0x82d9, 0x82da, PDF_CMAP_RANGE, 17863 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 2382 },
	{ 0x82dc, 0x82dc, PDF_CMAP_SINGLE, 5127 },
	{ 0x82dd, 0x82dd, PDF_CMAP_SINGLE, 17865 },
	{ 0x82de, 0x82de, PDF_CMAP_SINGLE, 1029 },
	{ 0x82df, 0x82df, PDF_CMAP_SINGLE, 1807 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 5139 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 5121 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 17866 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 5106 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 5124 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 3279 },
	{ 0x82e6, 0x82e6, PDF_CMAP_SINGLE, 2413 },
	{ 0x82e7, 0x82e7, PDF_CMAP_SINGLE, 9014 },
	{ 0x82e8, 0x82ea, PDF_CMAP_RANGE, 17867 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3316 },
	{ 0x82ec, 0x82ee, PDF_CMAP_RANGE, 17870 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 1062 },
	{ 0x82f0, 0x82f0, PDF_CMAP_SINGLE, 17873 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 4231 },
	{ 0x82f2, 0x82f3, PDF_CMAP_RANGE, 17874 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 5128 },
	{ 0x82f5, 0x82f6, PDF_CMAP_RANGE, 17876 },
	{ 0x82f7, 0x82f7, PDF_CMAP_SINGLE, 5123 },
	{ 0x82f8, 0x82f8, PDF_CMAP_SINGLE, 17878 },
	{ 0x82f9, 0x82f9, PDF_CMAP_SINGLE, 3034 },
	{ 0x82fa, 0x82fa, PDF_CMAP_SINGLE, 17879 },
	{ 0x82fb, 0x82fb, PDF_CMAP_SINGLE, 5132 },
	{ 0x82fc, 0x82ff, PDF_CMAP_RANGE, 17880 },
	{ 0x8300, 0x8300, PDF_CMAP_SINGLE, 17884 },
	{ 0x8301, 0x8301, PDF_CMAP_SINGLE, 4639 },
	{ 0x8302, 0x8302, PDF_CMAP_SINGLE, 2740 },
	{ 0x8303, 0x8303, PDF_CMAP_SINGLE, 1619 },
	{ 0x8304, 0x8304, PDF_CMAP_SINGLE, 3150 },
	{ 0x8305, 0x8305, PDF_CMAP_SINGLE, 2734 },
	{ 0x8306, 0x8306, PDF_CMAP_SINGLE, 5136 },
	{ 0x8307, 0x8307, PDF_CMAP_SINGLE, 5126 },
	{ 0x8308, 0x8308, PDF_CMAP_SINGLE, 5145 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 5122 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 17885 },
	{ 0x830c, 0x830c, PDF_CMAP_SINGLE, 5131 },
	{ 0x830d, 0x830d, PDF_CMAP_SINGLE, 17887 },
	{ 0x830e, 0x830e, PDF_CMAP_SINGLE, 2260 },
	{ 0x830f, 0x830f, PDF_CMAP_SINGLE, 5125 },
	{ 0x8310, 0x8310, PDF_CMAP_SINGLE, 17888 },
	{ 0x8311, 0x8311, PDF_CMAP_SINGLE, 5134 },
	{ 0x8312, 0x8313, PDF_CMAP_RANGE, 17889 },
	{ 0x8314, 0x8315, PDF_CMAP_RANGE, 5137 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 17891 },
	{ 0x8317, 0x8317, PDF_CMAP_SINGLE, 5158 },
	{ 0x8318, 0x8319, PDF_CMAP_RANGE, 17892 },
	{ 0x831a, 0x831a, PDF_CMAP_SINGLE, 5135 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 5166 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 5141 },
	{ 0x831d, 0x8326, PDF_CMAP_RANGE, 17894 },
	{ 0x8327, 0x8327, PDF_CMAP_SINGLE, 2142 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 1351 },
	{ 0x8329, 0x832a, PDF_CMAP_RANGE, 17904 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 2728 },
	{ 0x832c, 0x832c, PDF_CMAP_SINGLE, 1198 },
	{ 0x832d, 0x832d, PDF_CMAP_SINGLE, 5160 },
	{ 0x832e, 0x832e, PDF_CMAP_SINGLE, 17906 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 5152 },
	{ 0x8330, 0x8330, PDF_CMAP_SINGLE, 17907 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 5149 },
	{ 0x8332, 0x8332, PDF_CMAP_SINGLE, 17908 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 5162 },
	{ 0x8334, 0x8334, PDF_CMAP_SINGLE, 5148 },
	{ 0x8335, 0x8335, PDF_CMAP_SINGLE, 4215 },
	{ 0x8336, 0x8336, PDF_CMAP_SINGLE, 1199 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 17909 },
	{ 0x8338, 0x8338, PDF_CMAP_SINGLE, 3250 },
	{ 0x8339, 0x8339, PDF_CMAP_SINGLE, 3262 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 5161 },
	{ 0x833b, 0x833b, PDF_CMAP_SINGLE, 17910 },
	{ 0x833c, 0x833c, PDF_CMAP_SINGLE, 5147 },
	{ 0x833d, 0x833f, PDF_CMAP_RANGE, 17911 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 5157 },
	{ 0x8341, 0x8342, PDF_CMAP_RANGE, 17914 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 5155 },
	{ 0x8344, 0x8345, PDF_CMAP_RANGE, 17916 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 2258 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 5154 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 17918 },
	{ 0x8349, 0x8349, PDF_CMAP_SINGLE, 1188 },
	{ 0x834a, 0x834e, PDF_CMAP_RANGE, 17919 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 5153 },
	{ 0x8350, 0x8350, PDF_CMAP_SINGLE, 2153 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 5142 },
	{ 0x8352, 0x8352, PDF_CMAP_SINGLE, 2009 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 17924 },
	{ 0x8354, 0x8354, PDF_CMAP_SINGLE, 2526 },
	{ 0x8355, 0x8359, PDF_CMAP_RANGE, 17925 },
	{ 0x835a, 0x835a, PDF_CMAP_SINGLE, 2119 },
	{ 0x835b, 0x835c, PDF_CMAP_RANGE, 5143 },
	{ 0x835d, 0x835d, PDF_CMAP_SINGLE, 17930 },
	{ 0x835e, 0x835e, PDF_CMAP_SINGLE, 5151 },
	{ 0x835f, 0x835f, PDF_CMAP_SINGLE, 5156 },
	{ 0x8360, 0x8360, PDF_CMAP_SINGLE, 5159 },
	{ 0x8361, 0x8361, PDF_CMAP_SINGLE, 1429 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 17931 },
	{ 0x8363, 0x8363, PDF_CMAP_SINGLE, 3252 },
	{ 0x8364, 0x8364, PDF_CMAP_SINGLE, 2044 },
	{ 0x8365, 0x8365, PDF_CMAP_SINGLE, 5164 },
	{ 0x8366, 0x8366, PDF_CMAP_SINGLE, 5163 },
	{ 0x8367, 0x8367, PDF_CMAP_SINGLE, 4240 },
	{ 0x8368, 0x8368, PDF_CMAP_SINGLE, 5165 },
	{ 0x8369, 0x8369, PDF_CMAP_SINGLE, 5167 },
	{ 0x836a, 0x836a, PDF_CMAP_SINGLE, 5169 },
	{ 0x836b, 0x836b, PDF_CMAP_SINGLE, 4216 },
	{ 0x836c, 0x836c, PDF_CMAP_SINGLE, 5168 },
	{ 0x836d, 0x836e, PDF_CMAP_RANGE, 5170 },
	{ 0x836f, 0x836f, PDF_CMAP_SINGLE, 4144 },
	{ 0x8370, 0x8376, PDF_CMAP_RANGE, 17932 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 1920 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 5173 },
	{ 0x8379, 0x837a, PDF_CMAP_RANGE, 17939 },
	{ 0x837b, 0x837b, PDF_CMAP_SINGLE, 5186 },
	{ 0x837c, 0x837c, PDF_CMAP_SINGLE, 5181 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 5184 },
	{ 0x837e, 0x8384, PDF_CMAP_RANGE, 17941 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 5180 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 3053 },
	{ 0x8387, 0x8388, PDF_CMAP_RANGE, 17948 },
	{ 0x8389, 0x8389, PDF_CMAP_SINGLE, 2525 },
	{ 0x838a, 0x838a, PDF_CMAP_SINGLE, 8875 },
	{ 0x838b, 0x838d, PDF_CMAP_RANGE, 17950 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 3304 },
	{ 0x838f, 0x8391, PDF_CMAP_RANGE, 17953 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 5146 },
	{ 0x8393, 0x8393, PDF_CMAP_SINGLE, 5178 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 17956 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 8127 },
	{ 0x8397, 0x8397, PDF_CMAP_SINGLE, 17958 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 5187 },
	{ 0x8399, 0x839a, PDF_CMAP_RANGE, 17959 },
	{ 0x839b, 0x839b, PDF_CMAP_SINGLE, 5150 },
	{ 0x839c, 0x839c, PDF_CMAP_SINGLE, 5179 },
	{ 0x839d, 0x839d, PDF_CMAP_SINGLE, 17961 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 5188 },
	{ 0x839f, 0x839f, PDF_CMAP_SINGLE, 17962 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 5176 },
	{ 0x83a1, 0x83a1, PDF_CMAP_SINGLE, 17963 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 8061 },
	{ 0x83a3, 0x83a6, PDF_CMAP_RANGE, 17964 },
	{ 0x83a7, 0x83a7, PDF_CMAP_SINGLE, 9011 },
	{ 0x83a8, 0x83a8, PDF_CMAP_SINGLE, 5189 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 5183 },
	{ 0x83aa, 0x83aa, PDF_CMAP_SINGLE, 5177 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 2829 },
	{ 0x83ac, 0x83af, PDF_CMAP_RANGE, 17968 },
	{ 0x83b0, 0x83b0, PDF_CMAP_SINGLE, 5172 },
	{ 0x83b1, 0x83b1, PDF_CMAP_SINGLE, 2462 },
	{ 0x83b2, 0x83b2, PDF_CMAP_SINGLE, 2548 },
	{ 0x83b3, 0x83b4, PDF_CMAP_RANGE, 5174 },
	{ 0x83b5, 0x83b5, PDF_CMAP_SINGLE, 17972 },
	{ 0x83b6, 0x83b6, PDF_CMAP_SINGLE, 5182 },
	{ 0x83b7, 0x83b7, PDF_CMAP_SINGLE, 2054 },
	{ 0x83b8, 0x83b8, PDF_CMAP_SINGLE, 5185 },
	{ 0x83b9, 0x83b9, PDF_CMAP_SINGLE, 4237 },
	{ 0x83ba, 0x83ba, PDF_CMAP_SINGLE, 5190 },
	{ 0x83bb, 0x83bb, PDF_CMAP_SINGLE, 17973 },
	{ 0x83bc, 0x83bc, PDF_CMAP_SINGLE, 5191 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 2732 },
	{ 0x83be, 0x83bf, PDF_CMAP_RANGE, 17974 },
	{ 0x83c0, 0x83c0, PDF_CMAP_SINGLE, 5214 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 5192 },
	{ 0x83c2, 0x83c4, PDF_CMAP_RANGE, 17976 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 5213 },
	{ 0x83c6, 0x83c6, PDF_CMAP_SINGLE, 17979 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 1814 },
	{ 0x83c8, 0x83c9, PDF_CMAP_RANGE, 17980 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 2308 },
	{ 0x83cb, 0x83cb, PDF_CMAP_SINGLE, 17982 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 2345 },
	{ 0x83cd, 0x83ce, PDF_CMAP_RANGE, 17983 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 1921 },
	{ 0x83d0, 0x83d3, PDF_CMAP_RANGE, 17985 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 5206 },
	{ 0x83d5, 0x83d5, PDF_CMAP_SINGLE, 17989 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 5201 },
	{ 0x83d7, 0x83d7, PDF_CMAP_SINGLE, 17990 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 5195 },
	{ 0x83d9, 0x83db, PDF_CMAP_RANGE, 17991 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 1170 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 5199 },
	{ 0x83de, 0x83de, PDF_CMAP_SINGLE, 17994 },
	{ 0x83df, 0x83df, PDF_CMAP_SINGLE, 5207 },
	{ 0x83e0, 0x83e0, PDF_CMAP_SINGLE, 1131 },
	{ 0x83e1, 0x83e1, PDF_CMAP_SINGLE, 5217 },
	{ 0x83e2, 0x83e4, PDF_CMAP_RANGE, 17995 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 5194 },
	{ 0x83e6, 0x83e8, PDF_CMAP_RANGE, 17998 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 3055 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 5212 },
	{ 0x83eb, 0x83ee, PDF_CMAP_RANGE, 18001 },
	{ 0x83ef, 0x83ef, PDF_CMAP_SINGLE, 8013 },
	{ 0x83f0, 0x83f0, PDF_CMAP_SINGLE, 5216 },
	{ 0x83f1, 0x83f1, PDF_CMAP_SINGLE, 2603 },
	{ 0x83f2, 0x83f2, PDF_CMAP_SINGLE, 1635 },
	{ 0x83f3, 0x83f7, PDF_CMAP_RANGE, 18005 },
	{ 0x83f8, 0x83f9, PDF_CMAP_RANGE, 5210 },
	{ 0x83fa, 0x83fc, PDF_CMAP_RANGE, 18010 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 5200 },
	{ 0x83fe, 0x83ff, PDF_CMAP_RANGE, 18013 },
	{ 0x8400, 0x8400, PDF_CMAP_SINGLE, 18015 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 5193 },
	{ 0x8402, 0x8402, PDF_CMAP_SINGLE, 18016 },
	{ 0x8403, 0x8403, PDF_CMAP_SINGLE, 5209 },
	{ 0x8404, 0x8404, PDF_CMAP_SINGLE, 3620 },
	{ 0x8405, 0x8405, PDF_CMAP_SINGLE, 18017 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 5205 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 9012 },
	{ 0x8408, 0x8409, PDF_CMAP_RANGE, 18018 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 8177 },
	{ 0x840b, 0x840b, PDF_CMAP_SINGLE, 5198 },
	{ 0x840c, 0x840c, PDF_CMAP_SINGLE, 2765 },
	{ 0x840d, 0x840d, PDF_CMAP_SINGLE, 3035 },
	{ 0x840e, 0x840e, PDF_CMAP_SINGLE, 3773 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 5208 },
	{ 0x8410, 0x8410, PDF_CMAP_SINGLE, 18020 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 5204 },
	{ 0x8412, 0x8417, PDF_CMAP_RANGE, 18021 },
	{ 0x8418, 0x8418, PDF_CMAP_SINGLE, 5197 },
	{ 0x8419, 0x841b, PDF_CMAP_RANGE, 18027 },
	{ 0x841c, 0x841c, PDF_CMAP_SINGLE, 5202 },
	{ 0x841d, 0x841d, PDF_CMAP_SINGLE, 2691 },
	{ 0x841e, 0x8423, PDF_CMAP_RANGE, 18030 },
	{ 0x8424, 0x8425, PDF_CMAP_RANGE, 4238 },
	{ 0x8426, 0x8426, PDF_CMAP_SINGLE, 5215 },
	{ 0x8427, 0x8427, PDF_CMAP_SINGLE, 3937 },
	{ 0x8428, 0x8428, PDF_CMAP_SINGLE, 3283 },
	{ 0x8429, 0x842b, PDF_CMAP_RANGE, 18036 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 8587 },
	{ 0x842d, 0x8430, PDF_CMAP_RANGE, 18039 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 5234 },
	{ 0x8432, 0x8434, PDF_CMAP_RANGE, 18043 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 9033 },
	{ 0x8436, 0x8437, PDF_CMAP_RANGE, 18046 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 5203 },
	{ 0x8439, 0x843b, PDF_CMAP_RANGE, 18048 },
	{ 0x843c, 0x843c, PDF_CMAP_SINGLE, 5228 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 2699 },
	{ 0x843e, 0x8445, PDF_CMAP_RANGE, 18051 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 5229 },
	{ 0x8447, 0x8448, PDF_CMAP_RANGE, 18059 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 8715 },
	{ 0x844a, 0x8450, PDF_CMAP_RANGE, 18061 },
	{ 0x8451, 0x8451, PDF_CMAP_SINGLE, 5219 },
	{ 0x8452, 0x8452, PDF_CMAP_SINGLE, 9030 },
	{ 0x8453, 0x8456, PDF_CMAP_RANGE, 18068 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 4599 },
	{ 0x8458, 0x8458, PDF_CMAP_SINGLE, 18072 },
	{ 0x8459, 0x8459, PDF_CMAP_SINGLE, 5221 },
	{ 0x845a, 0x845a, PDF_CMAP_SINGLE, 5220 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 1771 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 5218 },
	{ 0x845d, 0x8460, PDF_CMAP_RANGE, 18073 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 3054 },
	{ 0x8462, 0x8462, PDF_CMAP_SINGLE, 18077 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 1516 },
	{ 0x8464, 0x8464, PDF_CMAP_SINGLE, 9031 },
	{ 0x8465, 0x8465, PDF_CMAP_SINGLE, 18078 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 8595 },
	{ 0x8467, 0x8468, PDF_CMAP_RANGE, 18079 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 5230 },
	{ 0x846a, 0x846a, PDF_CMAP_SINGLE, 18081 },
	{ 0x846b, 0x846b, PDF_CMAP_SINGLE, 1968 },
	{ 0x846c, 0x846c, PDF_CMAP_SINGLE, 4388 },
	{ 0x846d, 0x846d, PDF_CMAP_SINGLE, 5235 },
	{ 0x846e, 0x846e, PDF_CMAP_SINGLE, 18082 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 9889 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 18083 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 1363 },
	{ 0x8472, 0x8472, PDF_CMAP_SINGLE, 18084 },
	{ 0x8473, 0x8473, PDF_CMAP_SINGLE, 5222 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 18085 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 2440 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 5231 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 8034 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 5227 },
	{ 0x8479, 0x8479, PDF_CMAP_SINGLE, 18086 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 5225 },
	{ 0x847b, 0x8481, PDF_CMAP_RANGE, 18087 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 1466 },
	{ 0x8483, 0x8486, PDF_CMAP_RANGE, 18094 },
	{ 0x8487, 0x8488, PDF_CMAP_RANGE, 5223 },
	{ 0x8489, 0x8489, PDF_CMAP_SINGLE, 5226 },
	{ 0x848a, 0x848a, PDF_CMAP_SINGLE, 18098 },
	{ 0x848b, 0x848b, PDF_CMAP_SINGLE, 2176 },
	{ 0x848c, 0x848c, PDF_CMAP_SINGLE, 5232 },
	{ 0x848d, 0x848d, PDF_CMAP_SINGLE, 18099 },
	{ 0x848e, 0x848e, PDF_CMAP_SINGLE, 5233 },
	{ 0x848f, 0x8492, PDF_CMAP_RANGE, 18100 },
	{ 0x8493, 0x8493, PDF_CMAP_SINGLE, 9037 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 9032 },
	{ 0x8495, 0x8496, PDF_CMAP_RANGE, 18104 },
	{ 0x8497, 0x8497, PDF_CMAP_SINGLE, 5249 },
	{ 0x8498, 0x8498, PDF_CMAP_SINGLE, 18106 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 2766 },
	{ 0x849a, 0x849b, PDF_CMAP_RANGE, 18107 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3543 },
	{ 0x849d, 0x84a0, PDF_CMAP_RANGE, 18109 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 5246 },
	{ 0x84a2, 0x84ae, PDF_CMAP_RANGE, 18113 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 4759 },
	{ 0x84b0, 0x84b1, PDF_CMAP_RANGE, 18126 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 3056 },
	{ 0x84b3, 0x84b3, PDF_CMAP_SINGLE, 18128 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 5248 },
	{ 0x84b5, 0x84b7, PDF_CMAP_RANGE, 18129 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 4501 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 5247 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 5244 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 18132 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 7776 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 5240 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 18133 },
	{ 0x84bf, 0x84bf, PDF_CMAP_SINGLE, 5243 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 9029 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 5236 },
	{ 0x84c2, 0x84c3, PDF_CMAP_RANGE, 18134 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 4025 },
	{ 0x84c5, 0x84c8, PDF_CMAP_RANGE, 18136 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 3251 },
	{ 0x84ca, 0x84ca, PDF_CMAP_SINGLE, 5242 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 7954 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 18140 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 5237 },
	{ 0x84ce, 0x84cf, PDF_CMAP_RANGE, 18141 },
	{ 0x84d0, 0x84d0, PDF_CMAP_SINGLE, 5238 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 3559 },
	{ 0x84d2, 0x84d2, PDF_CMAP_SINGLE, 18143 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 5241 },
	{ 0x84d4, 0x84d5, PDF_CMAP_RANGE, 18144 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 1078 },
	{ 0x84d7, 0x84dc, PDF_CMAP_RANGE, 18146 },
	{ 0x84dd, 0x84dd, PDF_CMAP_SINGLE, 2465 },
	{ 0x84de, 0x84de, PDF_CMAP_SINGLE, 18152 },
	{ 0x84df, 0x84df, PDF_CMAP_SINGLE, 2094 },
	{ 0x84e0, 0x84e0, PDF_CMAP_SINGLE, 5245 },
	{ 0x84e1, 0x84e2, PDF_CMAP_RANGE, 18153 },
	{ 0x84e3, 0x84e3, PDF_CMAP_SINGLE, 5251 },
	{ 0x84e4, 0x84e4, PDF_CMAP_SINGLE, 18155 },
	{ 0x84e5, 0x84e5, PDF_CMAP_SINGLE, 5250 },
	{ 0x84e6, 0x84e6, PDF_CMAP_SINGLE, 5239 },
	{ 0x84e7, 0x84eb, PDF_CMAP_RANGE, 18156 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 2991 },
	{ 0x84ed, 0x84ed, PDF_CMAP_SINGLE, 18161 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 8216 },
	{ 0x84ef, 0x84ef, PDF_CMAP_SINGLE, 9013 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 5255 },
	{ 0x84f1, 0x84fb, PDF_CMAP_RANGE, 18162 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 5262 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 9020 },
	{ 0x84fe, 0x84fe, PDF_CMAP_SINGLE, 18173 },
	{ 0x84ff, 0x84ff, PDF_CMAP_SINGLE, 5261 },
	{ 0x8500, 0x850b, PDF_CMAP_RANGE, 18174 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 5252 },
	{ 0x850d, 0x8510, PDF_CMAP_RANGE, 18186 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 2804 },
	{ 0x8512, 0x8512, PDF_CMAP_SINGLE, 18190 },
	{ 0x8513, 0x8513, PDF_CMAP_SINGLE, 2722 },
	{ 0x8514, 0x8514, PDF_CMAP_SINGLE, 9839 },
	{ 0x8515, 0x8516, PDF_CMAP_RANGE, 18191 },
	{ 0x8517, 0x8517, PDF_CMAP_SINGLE, 4482 },
	{ 0x8518, 0x8519, PDF_CMAP_RANGE, 18193 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 3780 },
	{ 0x851b, 0x851d, PDF_CMAP_RANGE, 18195 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 9041 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 5257 },
	{ 0x8520, 0x8520, PDF_CMAP_SINGLE, 18198 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 1171 },
	{ 0x8522, 0x8522, PDF_CMAP_SINGLE, 18199 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 8096 },
	{ 0x8524, 0x8525, PDF_CMAP_RANGE, 18200 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 9016 },
	{ 0x8527, 0x852a, PDF_CMAP_RANGE, 18202 },
	{ 0x852b, 0x852b, PDF_CMAP_SINGLE, 2891 },
	{ 0x852c, 0x852c, PDF_CMAP_SINGLE, 3446 },
	{ 0x852d, 0x852d, PDF_CMAP_SINGLE, 8731 },
	{ 0x852e, 0x8536, PDF_CMAP_RANGE, 18206 },
	{ 0x8537, 0x8537, PDF_CMAP_SINGLE, 3131 },
	{ 0x8538, 0x8538, PDF_CMAP_SINGLE, 5254 },
	{ 0x8539, 0x8539, PDF_CMAP_SINGLE, 5256 },
	{ 0x853a, 0x853a, PDF_CMAP_SINGLE, 5258 },
	{ 0x853b, 0x853b, PDF_CMAP_SINGLE, 5260 },
	{ 0x853c, 0x853c, PDF_CMAP_SINGLE, 949 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 1079 },
	{ 0x853e, 0x8540, PDF_CMAP_RANGE, 18215 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 9026 },
	{ 0x8542, 0x8542, PDF_CMAP_SINGLE, 18218 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 5270 },
	{ 0x8544, 0x8545, PDF_CMAP_RANGE, 18219 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 9039 },
	{ 0x8547, 0x8547, PDF_CMAP_SINGLE, 18221 },
	{ 0x8548, 0x8548, PDF_CMAP_SINGLE, 5264 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 2183 },
	{ 0x854a, 0x854a, PDF_CMAP_SINGLE, 3274 },
	{ 0x854b, 0x854d, PDF_CMAP_RANGE, 18222 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 9021 },
	{ 0x854f, 0x8551, PDF_CMAP_RANGE, 18225 },
	{ 0x8552, 0x8552, PDF_CMAP_SINGLE, 9028 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 9009 },
	{ 0x8554, 0x8554, PDF_CMAP_SINGLE, 18228 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 9035 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 5259 },
	{ 0x8557, 0x8557, PDF_CMAP_SINGLE, 18229 },
	{ 0x8558, 0x8558, PDF_CMAP_SINGLE, 9019 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 5263 },
	{ 0x855a, 0x855d, PDF_CMAP_RANGE, 18230 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 5267 },
	{ 0x855f, 0x8561, PDF_CMAP_RANGE, 18234 },
	{ 0x8562, 0x8562, PDF_CMAP_SINGLE, 9040 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 18237 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 5266 },
	{ 0x8565, 0x8567, PDF_CMAP_RANGE, 18238 },
	{ 0x8568, 0x8568, PDF_CMAP_SINGLE, 5265 },
	{ 0x8569, 0x8569, PDF_CMAP_SINGLE, 7860 },
	{ 0x856a, 0x856a, PDF_CMAP_SINGLE, 8614 },
	{ 0x856b, 0x856c, PDF_CMAP_RANGE, 18241 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 8651 },
	{ 0x856e, 0x8571, PDF_CMAP_RANGE, 18243 },
	{ 0x8572, 0x8572, PDF_CMAP_SINGLE, 5271 },
	{ 0x8573, 0x8573, PDF_CMAP_SINGLE, 18247 },
	{ 0x8574, 0x8574, PDF_CMAP_SINGLE, 4367 },
	{ 0x8575, 0x8576, PDF_CMAP_RANGE, 18248 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 9045 },
	{ 0x8578, 0x8578, PDF_CMAP_SINGLE, 18250 },
	{ 0x8579, 0x8579, PDF_CMAP_SINGLE, 5277 },
	{ 0x857a, 0x857a, PDF_CMAP_SINGLE, 5268 },
	{ 0x857b, 0x857b, PDF_CMAP_SINGLE, 5272 },
	{ 0x857c, 0x857d, PDF_CMAP_RANGE, 18251 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 2500 },
	{ 0x857f, 0x8583, PDF_CMAP_RANGE, 18253 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 1034 },
	{ 0x8585, 0x8585, PDF_CMAP_SINGLE, 5280 },
	{ 0x8586, 0x8586, PDF_CMAP_SINGLE, 18258 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 5275 },
	{ 0x8588, 0x8588, PDF_CMAP_SINGLE, 9022 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 18259 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 8052 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 18260 },
	{ 0x858c, 0x858c, PDF_CMAP_SINGLE, 9008 },
	{ 0x858d, 0x858e, PDF_CMAP_RANGE, 18261 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 5276 },
	{ 0x8590, 0x8590, PDF_CMAP_SINGLE, 18263 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 9856 },
	{ 0x8592, 0x8593, PDF_CMAP_RANGE, 18264 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 8410 },
	{ 0x8595, 0x859a, PDF_CMAP_RANGE, 18266 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 4047 },
	{ 0x859c, 0x859c, PDF_CMAP_SINGLE, 5279 },
	{ 0x859d, 0x859e, PDF_CMAP_RANGE, 18272 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 9034 },
	{ 0x85a0, 0x85a3, PDF_CMAP_RANGE, 18274 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 5273 },
	{ 0x85a5, 0x85a5, PDF_CMAP_SINGLE, 18278 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 8081 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 18279 },
	{ 0x85a8, 0x85a8, PDF_CMAP_SINGLE, 5274 },
	{ 0x85a9, 0x85a9, PDF_CMAP_SINGLE, 8455 },
	{ 0x85aa, 0x85aa, PDF_CMAP_SINGLE, 3976 },
	{ 0x85ab, 0x85ad, PDF_CMAP_RANGE, 18280 },
	{ 0x85ae, 0x85ae, PDF_CMAP_SINGLE, 5278 },
	{ 0x85af, 0x85af, PDF_CMAP_SINGLE, 3460 },
	{ 0x85b0, 0x85b0, PDF_CMAP_SINGLE, 5283 },
	{ 0x85b1, 0x85b6, PDF_CMAP_RANGE, 18283 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 5282 },
	{ 0x85b8, 0x85b8, PDF_CMAP_SINGLE, 18289 },
	{ 0x85b9, 0x85b9, PDF_CMAP_SINGLE, 5281 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 9023 },
	{ 0x85bb, 0x85c0, PDF_CMAP_RANGE, 18290 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 5285 },
	{ 0x85c2, 0x85c8, PDF_CMAP_RANGE, 18296 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 2230 },
	{ 0x85ca, 0x85cc, PDF_CMAP_RANGE, 18303 },
	{ 0x85cd, 0x85cd, PDF_CMAP_SINGLE, 8180 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 9027 },
	{ 0x85cf, 0x85cf, PDF_CMAP_SINGLE, 1183 },
	{ 0x85d0, 0x85d0, PDF_CMAP_SINGLE, 2799 },
	{ 0x85d1, 0x85d2, PDF_CMAP_RANGE, 18306 },
	{ 0x85d3, 0x85d3, PDF_CMAP_SINGLE, 5284 },
	{ 0x85d4, 0x85d4, PDF_CMAP_SINGLE, 18308 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 2939 },
	{ 0x85d6, 0x85db, PDF_CMAP_RANGE, 18309 },
	{ 0x85dc, 0x85dc, PDF_CMAP_SINGLE, 5286 },
	{ 0x85dd, 0x85dd, PDF_CMAP_SINGLE, 8722 },
	{ 0x85de, 0x85e3, PDF_CMAP_RANGE, 18315 },
	{ 0x85e4, 0x85e4, PDF_CMAP_SINGLE, 3628 },
	{ 0x85e5, 0x85e5, PDF_CMAP_SINGLE, 8711 },
	{ 0x85e6, 0x85e8, PDF_CMAP_RANGE, 18321 },
	{ 0x85e9, 0x85e9, PDF_CMAP_SINGLE, 1607 },
	{ 0x85ea, 0x85ea, PDF_CMAP_SINGLE, 9049 },
	{ 0x85eb, 0x85f3, PDF_CMAP_RANGE, 18324 },
	{ 0x85f4, 0x85f4, PDF_CMAP_SINGLE, 8788 },
	{ 0x85f5, 0x85f5, PDF_CMAP_SINGLE, 18333 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 9010 },
	{ 0x85f7, 0x85f8, PDF_CMAP_RANGE, 18334 },
	{ 0x85f9, 0x85f9, PDF_CMAP_SINGLE, 7718 },
	{ 0x85fa, 0x85fa, PDF_CMAP_SINGLE, 9047 },
	{ 0x85fb, 0x85fb, PDF_CMAP_SINGLE, 4392 },
	{ 0x85fc, 0x85fe, PDF_CMAP_RANGE, 18336 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 5287 },
	{ 0x8600, 0x8603, PDF_CMAP_RANGE, 18339 },
	{ 0x8604, 0x8604, PDF_CMAP_SINGLE, 9048 },
	{ 0x8605, 0x8605, PDF_CMAP_SINGLE, 5289 },
	{ 0x8606, 0x8606, PDF_CMAP_SINGLE, 8258 },
	{ 0x8607, 0x8607, PDF_CMAP_SINGLE, 8531 },
	{ 0x8608, 0x860a, PDF_CMAP_RANGE, 18343 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 8379 },
	{ 0x860c, 0x8610, PDF_CMAP_RANGE, 18346 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 2821 },
	{ 0x8612, 0x8615, PDF_CMAP_RANGE, 18351 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 5291 },
	{ 0x8617, 0x8619, PDF_CMAP_RANGE, 18355 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 9050 },
	{ 0x861b, 0x861d, PDF_CMAP_RANGE, 18358 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 9046 },
	{ 0x861f, 0x8621, PDF_CMAP_RANGE, 18361 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 9015 },
	{ 0x8623, 0x8626, PDF_CMAP_RANGE, 18364 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 5288 },
	{ 0x8628, 0x8628, PDF_CMAP_SINGLE, 18368 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 5290 },
	{ 0x862a, 0x862c, PDF_CMAP_RANGE, 18369 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 8185 },
	{ 0x862e, 0x8637, PDF_CMAP_RANGE, 18372 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 4443 },
	{ 0x8639, 0x8639, PDF_CMAP_SINGLE, 18382 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 9043 },
	{ 0x863b, 0x863b, PDF_CMAP_SINGLE, 18383 },
	{ 0x863c, 0x863c, PDF_CMAP_SINGLE, 5292 },
	{ 0x863d, 0x863e, PDF_CMAP_RANGE, 18384 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 8289 },
	{ 0x8640, 0x864c, PDF_CMAP_RANGE, 18386 },
	{ 0x864d, 0x864d, PDF_CMAP_SINGLE, 7152 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 1975 },
	{ 0x864f, 0x864f, PDF_CMAP_SINGLE, 2649 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 2929 },
	{ 0x8651, 0x8651, PDF_CMAP_SINGLE, 2670 },
	{ 0x8652, 0x8653, PDF_CMAP_RANGE, 18399 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 7153 },
	{ 0x8655, 0x8655, PDF_CMAP_SINGLE, 7829 },
	{ 0x8656, 0x8659, PDF_CMAP_RANGE, 18401 },
	{ 0x865a, 0x865a, PDF_CMAP_SINGLE, 4020 },
	{ 0x865b, 0x865b, PDF_CMAP_SINGLE, 18405 },
	{ 0x865c, 0x865c, PDF_CMAP_SINGLE, 8265 },
	{ 0x865d, 0x865d, PDF_CMAP_SINGLE, 18406 },
	{ 0x865e, 0x865e, PDF_CMAP_SINGLE, 4290 },
	{ 0x865f, 0x865f, PDF_CMAP_SINGLE, 8002 },
	{ 0x8660, 0x8661, PDF_CMAP_RANGE, 18407 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 6509 },
	{ 0x8663, 0x8666, PDF_CMAP_RANGE, 18409 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 8168 },
	{ 0x8668, 0x866a, PDF_CMAP_RANGE, 18413 },
	{ 0x866b, 0x866b, PDF_CMAP_SINGLE, 1291 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 7154 },
	{ 0x866d, 0x866d, PDF_CMAP_SINGLE, 18416 },
	{ 0x866e, 0x866e, PDF_CMAP_SINGLE, 7155 },
	{ 0x866f, 0x8670, PDF_CMAP_RANGE, 18417 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 3396 },
	{ 0x8672, 0x8678, PDF_CMAP_RANGE, 18419 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 1950 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 7157 },
	{ 0x867b, 0x867b, PDF_CMAP_SINGLE, 7159 },
	{ 0x867c, 0x867c, PDF_CMAP_SINGLE, 7158 },
	{ 0x867d, 0x867d, PDF_CMAP_SINGLE, 3545 },
	{ 0x867e, 0x867e, PDF_CMAP_SINGLE, 3879 },
	{ 0x867f, 0x867f, PDF_CMAP_SINGLE, 7156 },
	{ 0x8680, 0x8680, PDF_CMAP_SINGLE, 3403 },
	{ 0x8681, 0x8681, PDF_CMAP_SINGLE, 4182 },
	{ 0x8682, 0x8682, PDF_CMAP_SINGLE, 2707 },
	{ 0x8683, 0x8689, PDF_CMAP_RANGE, 18426 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 3794 },
	{ 0x868b, 0x868b, PDF_CMAP_SINGLE, 7162 },
	{ 0x868c, 0x868c, PDF_CMAP_SINGLE, 1025 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 7161 },
	{ 0x868e, 0x8692, PDF_CMAP_RANGE, 18433 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 7168 },
	{ 0x8694, 0x8694, PDF_CMAP_SINGLE, 18438 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 1174 },
	{ 0x8696, 0x869b, PDF_CMAP_RANGE, 18439 },
	{ 0x869c, 0x869c, PDF_CMAP_SINGLE, 4074 },
	{ 0x869d, 0x869d, PDF_CMAP_SINGLE, 7164 },
	{ 0x869e, 0x86a2, PDF_CMAP_RANGE, 18445 },
	{ 0x86a3, 0x86a3, PDF_CMAP_SINGLE, 7166 },
	{ 0x86a4, 0x86a4, PDF_CMAP_SINGLE, 4396 },
	{ 0x86a5, 0x86a6, PDF_CMAP_RANGE, 18450 },
	{ 0x86a7, 0x86a7, PDF_CMAP_SINGLE, 7165 },
	{ 0x86a8, 0x86a8, PDF_CMAP_SINGLE, 7160 },
	{ 0x86a9, 0x86a9, PDF_CMAP_SINGLE, 7169 },
	{ 0x86aa, 0x86aa, PDF_CMAP_SINGLE, 7167 },
	{ 0x86ab, 0x86ab, PDF_CMAP_SINGLE, 18452 },
	{ 0x86ac, 0x86ac, PDF_CMAP_SINGLE, 7163 },
	{ 0x86ad, 0x86ae, PDF_CMAP_RANGE, 18453 },
	{ 0x86af, 0x86af, PDF_CMAP_SINGLE, 7177 },
	{ 0x86b0, 0x86b0, PDF_CMAP_SINGLE, 7174 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 7176 },
	{ 0x86b2, 0x86b3, PDF_CMAP_RANGE, 18455 },
	{ 0x86b4, 0x86b4, PDF_CMAP_SINGLE, 7180 },
	{ 0x86b5, 0x86b5, PDF_CMAP_SINGLE, 7172 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 7170 },
	{ 0x86b7, 0x86b9, PDF_CMAP_RANGE, 18457 },
	{ 0x86ba, 0x86ba, PDF_CMAP_SINGLE, 7175 },
	{ 0x86bb, 0x86bf, PDF_CMAP_RANGE, 18460 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 4602 },
	{ 0x86c1, 0x86c3, PDF_CMAP_RANGE, 18465 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 7171 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 18468 },
	{ 0x86c6, 0x86c6, PDF_CMAP_SINGLE, 3190 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 3352 },
	{ 0x86c8, 0x86c8, PDF_CMAP_SINGLE, 18469 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 7178 },
	{ 0x86ca, 0x86ca, PDF_CMAP_SINGLE, 1823 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 1425 },
	{ 0x86cc, 0x86cd, PDF_CMAP_RANGE, 18470 },
	{ 0x86ce, 0x86ce, PDF_CMAP_SINGLE, 7173 },
	{ 0x86cf, 0x86cf, PDF_CMAP_SINGLE, 7179 },
	{ 0x86d0, 0x86d0, PDF_CMAP_SINGLE, 7186 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 7192 },
	{ 0x86d2, 0x86d3, PDF_CMAP_RANGE, 18472 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 2028 },
	{ 0x86d5, 0x86d7, PDF_CMAP_RANGE, 18474 },
	{ 0x86d8, 0x86d8, PDF_CMAP_SINGLE, 7191 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 3725 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 18477 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 4586 },
	{ 0x86dc, 0x86dd, PDF_CMAP_RANGE, 18478 },
	{ 0x86de, 0x86de, PDF_CMAP_SINGLE, 7188 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 7190 },
	{ 0x86e0, 0x86e3, PDF_CMAP_RANGE, 18480 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 1773 },
	{ 0x86e5, 0x86e8, PDF_CMAP_RANGE, 18484 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 7181 },
	{ 0x86ea, 0x86ec, PDF_CMAP_RANGE, 18488 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 7184 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 2720 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 18491 },
	{ 0x86f0, 0x86f0, PDF_CMAP_SINGLE, 4478 },
	{ 0x86f1, 0x86f2, PDF_CMAP_RANGE, 7182 },
	{ 0x86f3, 0x86f3, PDF_CMAP_SINGLE, 7185 },
	{ 0x86f4, 0x86f4, PDF_CMAP_SINGLE, 7189 },
	{ 0x86f5, 0x86f7, PDF_CMAP_RANGE, 18492 },
	{ 0x86f8, 0x86f8, PDF_CMAP_SINGLE, 7195 },
	{ 0x86f9, 0x86f9, PDF_CMAP_SINGLE, 4257 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 9690 },
	{ 0x86fb, 0x86fd, PDF_CMAP_RANGE, 18495 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 1577 },
	{ 0x86ff, 0x86ff, PDF_CMAP_SINGLE, 18498 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 3464 },
	{ 0x8701, 0x8701, PDF_CMAP_SINGLE, 18499 },
	{ 0x8702, 0x8702, PDF_CMAP_SINGLE, 1665 },
	{ 0x8703, 0x8703, PDF_CMAP_SINGLE, 7193 },
	{ 0x8704, 0x8705, PDF_CMAP_RANGE, 18500 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 9687 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 7194 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 7196 },
	{ 0x8709, 0x8709, PDF_CMAP_SINGLE, 7199 },
	{ 0x870a, 0x870a, PDF_CMAP_SINGLE, 7197 },
	{ 0x870b, 0x870c, PDF_CMAP_RANGE, 18502 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 7198 },
	{ 0x870e, 0x8711, PDF_CMAP_RANGE, 18504 },
	{ 0x8712, 0x8712, PDF_CMAP_SINGLE, 4090 },
	{ 0x8713, 0x8713, PDF_CMAP_SINGLE, 7187 },
	{ 0x8714, 0x8714, PDF_CMAP_SINGLE, 18508 },
	{ 0x8715, 0x8715, PDF_CMAP_SINGLE, 3706 },
	{ 0x8716, 0x8716, PDF_CMAP_SINGLE, 18509 },
	{ 0x8717, 0x8717, PDF_CMAP_SINGLE, 3806 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 4520 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 18510 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 7205 },
	{ 0x871b, 0x871b, PDF_CMAP_SINGLE, 18511 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 2784 },
	{ 0x871d, 0x871d, PDF_CMAP_SINGLE, 18512 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 7202 },
	{ 0x871f, 0x8720, PDF_CMAP_RANGE, 18513 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 2458 },
	{ 0x8722, 0x8722, PDF_CMAP_SINGLE, 7214 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 7200 },
	{ 0x8724, 0x8724, PDF_CMAP_SINGLE, 18515 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 7203 },
	{ 0x8726, 0x8728, PDF_CMAP_RANGE, 18516 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 7210 },
	{ 0x872a, 0x872d, PDF_CMAP_RANGE, 18519 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 7204 },
	{ 0x872f, 0x8730, PDF_CMAP_RANGE, 18523 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 7209 },
	{ 0x8732, 0x8733, PDF_CMAP_RANGE, 18525 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 7208 },
	{ 0x8735, 0x8736, PDF_CMAP_RANGE, 18527 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 7211 },
	{ 0x8738, 0x873a, PDF_CMAP_RANGE, 18529 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 7201 },
	{ 0x873c, 0x873d, PDF_CMAP_RANGE, 18532 },
	{ 0x873e, 0x873e, PDF_CMAP_SINGLE, 7206 },
	{ 0x873f, 0x873f, PDF_CMAP_SINGLE, 7212 },
	{ 0x8740, 0x8746, PDF_CMAP_RANGE, 18534 },
	{ 0x8747, 0x8747, PDF_CMAP_SINGLE, 4241 },
	{ 0x8748, 0x8748, PDF_CMAP_SINGLE, 7207 },
	{ 0x8749, 0x8749, PDF_CMAP_SINGLE, 1212 },
	{ 0x874a, 0x874b, PDF_CMAP_RANGE, 18541 },
	{ 0x874c, 0x874c, PDF_CMAP_SINGLE, 7220 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 18543 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 3958 },
	{ 0x874f, 0x8752, PDF_CMAP_RANGE, 18544 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 7223 },
	{ 0x8754, 0x8754, PDF_CMAP_SINGLE, 18548 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 8494 },
	{ 0x8756, 0x8756, PDF_CMAP_SINGLE, 18549 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 2013 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 18550 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 7227 },
	{ 0x875a, 0x875f, PDF_CMAP_RANGE, 18551 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 7218 },
	{ 0x8761, 0x8762, PDF_CMAP_RANGE, 18557 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 7224 },
	{ 0x8764, 0x8764, PDF_CMAP_SINGLE, 7226 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 7228 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 8626 },
	{ 0x8767, 0x876d, PDF_CMAP_RANGE, 18559 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 7221 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 18566 },
	{ 0x8770, 0x8770, PDF_CMAP_SINGLE, 7219 },
	{ 0x8771, 0x8773, PDF_CMAP_RANGE, 18567 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 1970 },
	{ 0x8775, 0x8775, PDF_CMAP_SINGLE, 18570 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 1500 },
	{ 0x8777, 0x8777, PDF_CMAP_SINGLE, 18571 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 8606 },
	{ 0x8779, 0x877a, PDF_CMAP_RANGE, 18572 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 7217 },
	{ 0x877c, 0x877c, PDF_CMAP_SINGLE, 7225 },
	{ 0x877d, 0x877e, PDF_CMAP_RANGE, 7215 },
	{ 0x877f, 0x8781, PDF_CMAP_RANGE, 18574 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 7213 },
	{ 0x8783, 0x8783, PDF_CMAP_SINGLE, 7238 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 9692 },
	{ 0x8785, 0x8785, PDF_CMAP_SINGLE, 7235 },
	{ 0x8786, 0x8787, PDF_CMAP_RANGE, 18577 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 7234 },
	{ 0x8789, 0x878a, PDF_CMAP_RANGE, 18579 },
	{ 0x878b, 0x878b, PDF_CMAP_SINGLE, 7222 },
	{ 0x878c, 0x878c, PDF_CMAP_SINGLE, 18581 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 3253 },
	{ 0x878e, 0x8792, PDF_CMAP_RANGE, 18582 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 7229 },
	{ 0x8794, 0x8796, PDF_CMAP_RANGE, 18587 },
	{ 0x8797, 0x8797, PDF_CMAP_SINGLE, 7237 },
	{ 0x8798, 0x879d, PDF_CMAP_RANGE, 18590 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 8300 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 2813 },
	{ 0x87a0, 0x87a1, PDF_CMAP_RANGE, 18596 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 8742 },
	{ 0x87a3, 0x87a7, PDF_CMAP_RANGE, 18598 },
	{ 0x87a8, 0x87a8, PDF_CMAP_SINGLE, 7231 },
	{ 0x87a9, 0x87aa, PDF_CMAP_RANGE, 18603 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 7239 },
	{ 0x87ac, 0x87ac, PDF_CMAP_SINGLE, 7241 },
	{ 0x87ad, 0x87ad, PDF_CMAP_SINGLE, 7236 },
	{ 0x87ae, 0x87ae, PDF_CMAP_SINGLE, 18605 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 7230 },
	{ 0x87b0, 0x87b2, PDF_CMAP_RANGE, 18606 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 7243 },
	{ 0x87b4, 0x87b4, PDF_CMAP_SINGLE, 18609 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 7242 },
	{ 0x87b6, 0x87b9, PDF_CMAP_RANGE, 18610 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 2692 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 9696 },
	{ 0x87bc, 0x87bc, PDF_CMAP_SINGLE, 18614 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 7246 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 18615 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 7248 },
	{ 0x87c1, 0x87c3, PDF_CMAP_RANGE, 18617 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 8827 },
	{ 0x87c5, 0x87c5, PDF_CMAP_SINGLE, 18620 },
	{ 0x87c6, 0x87c6, PDF_CMAP_SINGLE, 7233 },
	{ 0x87c7, 0x87c7, PDF_CMAP_SINGLE, 18621 },
	{ 0x87c8, 0x87c8, PDF_CMAP_SINGLE, 9694 },
	{ 0x87c9, 0x87c9, PDF_CMAP_SINGLE, 18622 },
	{ 0x87ca, 0x87ca, PDF_CMAP_SINGLE, 7249 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 7244 },
	{ 0x87cc, 0x87cd, PDF_CMAP_RANGE, 18623 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 9697 },
	{ 0x87cf, 0x87d0, PDF_CMAP_RANGE, 18625 },
	{ 0x87d1, 0x87d1, PDF_CMAP_SINGLE, 7247 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 7232 },
	{ 0x87d3, 0x87d3, PDF_CMAP_SINGLE, 7245 },
	{ 0x87d4, 0x87da, PDF_CMAP_RANGE, 18627 },
	{ 0x87db, 0x87db, PDF_CMAP_SINGLE, 7250 },
	{ 0x87dc, 0x87df, PDF_CMAP_RANGE, 18634 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 7252 },
	{ 0x87e1, 0x87e2, PDF_CMAP_RANGE, 18638 },
	{ 0x87e3, 0x87e3, PDF_CMAP_SINGLE, 9685 },
	{ 0x87e4, 0x87e4, PDF_CMAP_SINGLE, 18640 },
	{ 0x87e5, 0x87e5, PDF_CMAP_SINGLE, 7240 },
	{ 0x87e6, 0x87e9, PDF_CMAP_RANGE, 18641 },
	{ 0x87ea, 0x87ea, PDF_CMAP_SINGLE, 7251 },
	{ 0x87eb, 0x87eb, PDF_CMAP_SINGLE, 18645 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 7787 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 18646 },
	{ 0x87ee, 0x87ee, PDF_CMAP_SINGLE, 7253 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 9691 },
	{ 0x87f0, 0x87f1, PDF_CMAP_RANGE, 18647 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 7817 },
	{ 0x87f3, 0x87f5, PDF_CMAP_RANGE, 18649 },
	{ 0x87f6, 0x87f6, PDF_CMAP_SINGLE, 9689 },
	{ 0x87f7, 0x87f8, PDF_CMAP_RANGE, 18652 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 3970 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 18654 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 8721 },
	{ 0x87fc, 0x87fd, PDF_CMAP_RANGE, 18655 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 7256 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 18657 },
	{ 0x8800, 0x8802, PDF_CMAP_RANGE, 18658 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 4877 },
	{ 0x8804, 0x8804, PDF_CMAP_SINGLE, 18661 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 8745 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 9686 },
	{ 0x8807, 0x8809, PDF_CMAP_RANGE, 18662 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 7257 },
	{ 0x880b, 0x880f, PDF_CMAP_RANGE, 18665 },
	{ 0x8810, 0x8810, PDF_CMAP_SINGLE, 9693 },
	{ 0x8811, 0x8811, PDF_CMAP_SINGLE, 9695 },
	{ 0x8812, 0x8812, PDF_CMAP_SINGLE, 18670 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 7255 },
	{ 0x8814, 0x8814, PDF_CMAP_SINGLE, 18671 },
	{ 0x8815, 0x8815, PDF_CMAP_SINGLE, 3263 },
	{ 0x8816, 0x8816, PDF_CMAP_SINGLE, 7254 },
	{ 0x8817, 0x881a, PDF_CMAP_RANGE, 18672 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 7258 },
	{ 0x881c, 0x881e, PDF_CMAP_RANGE, 18676 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 8175 },
	{ 0x8820, 0x8820, PDF_CMAP_SINGLE, 18679 },
	{ 0x8821, 0x8821, PDF_CMAP_SINGLE, 7259 },
	{ 0x8822, 0x8822, PDF_CMAP_SINGLE, 1347 },
	{ 0x8823, 0x8823, PDF_CMAP_SINGLE, 9688 },
	{ 0x8824, 0x8830, PDF_CMAP_RANGE, 18680 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 7977 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 6797 },
	{ 0x8833, 0x8835, PDF_CMAP_RANGE, 18693 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 7771 },
	{ 0x8837, 0x8838, PDF_CMAP_RANGE, 18696 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 7260 },
	{ 0x883a, 0x883a, PDF_CMAP_SINGLE, 18698 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 8310 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 7261 },
	{ 0x883d, 0x883f, PDF_CMAP_RANGE, 18699 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 4051 },
	{ 0x8841, 0x8843, PDF_CMAP_RANGE, 18702 },
	{ 0x8844, 0x8844, PDF_CMAP_SINGLE, 7338 },
	{ 0x8845, 0x8845, PDF_CMAP_SINGLE, 3985 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 8855 },
	{ 0x8847, 0x8849, PDF_CMAP_RANGE, 18705 },
	{ 0x884a, 0x884a, PDF_CMAP_SINGLE, 9872 },
	{ 0x884b, 0x884b, PDF_CMAP_SINGLE, 18708 },
	{ 0x884c, 0x884c, PDF_CMAP_SINGLE, 3995 },
	{ 0x884d, 0x884d, PDF_CMAP_SINGLE, 4101 },
	{ 0x884e, 0x8852, PDF_CMAP_RANGE, 18709 },
	{ 0x8853, 0x8853, PDF_CMAP_SINGLE, 8511 },
	{ 0x8854, 0x8854, PDF_CMAP_SINGLE, 3899 },
	{ 0x8855, 0x8856, PDF_CMAP_RANGE, 18714 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 2215 },
	{ 0x8858, 0x8858, PDF_CMAP_SINGLE, 18716 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 4076 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 18717 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 8600 },
	{ 0x885c, 0x885c, PDF_CMAP_SINGLE, 18718 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 7816 },
	{ 0x885e, 0x8860, PDF_CMAP_RANGE, 18719 },
	{ 0x8861, 0x8861, PDF_CMAP_SINGLE, 1945 },
	{ 0x8862, 0x8862, PDF_CMAP_SINGLE, 5613 },
	{ 0x8863, 0x8863, PDF_CMAP_SINGLE, 4169 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 7078 },
	{ 0x8865, 0x8865, PDF_CMAP_SINGLE, 1152 },
	{ 0x8866, 0x8867, PDF_CMAP_RANGE, 18722 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 1110 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 7079 },
	{ 0x886a, 0x886a, PDF_CMAP_SINGLE, 18724 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 3321 },
	{ 0x886c, 0x886c, PDF_CMAP_SINGLE, 1257 },
	{ 0x886d, 0x886d, PDF_CMAP_SINGLE, 18725 },
	{ 0x886e, 0x886e, PDF_CMAP_SINGLE, 4870 },
	{ 0x886f, 0x886f, PDF_CMAP_SINGLE, 18726 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 3482 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 18727 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 7080 },
	{ 0x8873, 0x8876, PDF_CMAP_RANGE, 18728 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 4563 },
	{ 0x8878, 0x8878, PDF_CMAP_SINGLE, 18732 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 8844 },
	{ 0x887a, 0x887c, PDF_CMAP_RANGE, 18733 },
	{ 0x887d, 0x887d, PDF_CMAP_SINGLE, 7081 },
	{ 0x887e, 0x887e, PDF_CMAP_SINGLE, 7358 },
	{ 0x887f, 0x887f, PDF_CMAP_SINGLE, 7082 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 18736 },
	{ 0x8881, 0x8881, PDF_CMAP_SINGLE, 4335 },
	{ 0x8882, 0x8882, PDF_CMAP_SINGLE, 7083 },
	{ 0x8883, 0x8883, PDF_CMAP_SINGLE, 18737 },
	{ 0x8884, 0x8884, PDF_CMAP_SINGLE, 971 },
	{ 0x8885, 0x8885, PDF_CMAP_SINGLE, 7359 },
	{ 0x8886, 0x8887, PDF_CMAP_RANGE, 18738 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 7360 },
	{ 0x8889, 0x888a, PDF_CMAP_RANGE, 18740 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 1407 },
	{ 0x888c, 0x888c, PDF_CMAP_SINGLE, 18742 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 2972 },
	{ 0x888e, 0x8891, PDF_CMAP_RANGE, 18743 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 3598 },
	{ 0x8893, 0x8895, PDF_CMAP_RANGE, 18747 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 4015 },
	{ 0x8897, 0x889b, PDF_CMAP_RANGE, 18750 },
	{ 0x889c, 0x889c, PDF_CMAP_SINGLE, 3729 },
	{ 0x889d, 0x88a1, PDF_CMAP_RANGE, 18755 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 7084 },
	{ 0x88a3, 0x88a3, PDF_CMAP_SINGLE, 18760 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 4871 },
	{ 0x88a5, 0x88aa, PDF_CMAP_RANGE, 18761 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 1060 },
	{ 0x88ac, 0x88ac, PDF_CMAP_SINGLE, 18767 },
	{ 0x88ad, 0x88ad, PDF_CMAP_SINGLE, 3867 },
	{ 0x88ae, 0x88b0, PDF_CMAP_RANGE, 18768 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 1695 },
	{ 0x88b2, 0x88b6, PDF_CMAP_RANGE, 18771 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 7086 },
	{ 0x88b8, 0x88bb, PDF_CMAP_RANGE, 18776 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 7087 },
	{ 0x88bd, 0x88c0, PDF_CMAP_RANGE, 18780 },
	{ 0x88c1, 0x88c1, PDF_CMAP_SINGLE, 1162 },
	{ 0x88c2, 0x88c2, PDF_CMAP_SINGLE, 2586 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 18784 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 4621 },
	{ 0x88c6, 0x88c6, PDF_CMAP_SINGLE, 7085 },
	{ 0x88c7, 0x88c8, PDF_CMAP_RANGE, 18786 },
	{ 0x88c9, 0x88c9, PDF_CMAP_SINGLE, 7088 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 9713 },
	{ 0x88cb, 0x88cd, PDF_CMAP_RANGE, 18788 },
	{ 0x88ce, 0x88ce, PDF_CMAP_SINGLE, 7090 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 8204 },
	{ 0x88d0, 0x88d1, PDF_CMAP_RANGE, 18791 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 4874 },
	{ 0x88d3, 0x88d3, PDF_CMAP_SINGLE, 18793 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 4200 },
	{ 0x88d5, 0x88d5, PDF_CMAP_SINGLE, 4326 },
	{ 0x88d6, 0x88d7, PDF_CMAP_RANGE, 18794 },
	{ 0x88d8, 0x88d8, PDF_CMAP_SINGLE, 7361 },
	{ 0x88d9, 0x88d9, PDF_CMAP_SINGLE, 3220 },
	{ 0x88da, 0x88db, PDF_CMAP_RANGE, 18796 },
	{ 0x88dc, 0x88dc, PDF_CMAP_SINGLE, 7768 },
	{ 0x88dd, 0x88dd, PDF_CMAP_SINGLE, 8876 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 18798 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 7362 },
	{ 0x88e0, 0x88e1, PDF_CMAP_RANGE, 18799 },
	{ 0x88e2, 0x88e2, PDF_CMAP_SINGLE, 7089 },
	{ 0x88e3, 0x88e3, PDF_CMAP_SINGLE, 7091 },
	{ 0x88e4, 0x88e4, PDF_CMAP_SINGLE, 2416 },
	{ 0x88e5, 0x88e5, PDF_CMAP_SINGLE, 7092 },
	{ 0x88e6, 0x88e7, PDF_CMAP_RANGE, 18801 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 7096 },
	{ 0x88e9, 0x88ef, PDF_CMAP_RANGE, 18803 },
	{ 0x88f0, 0x88f0, PDF_CMAP_SINGLE, 7098 },
	{ 0x88f1, 0x88f1, PDF_CMAP_SINGLE, 7093 },
	{ 0x88f2, 0x88f2, PDF_CMAP_SINGLE, 18810 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 3338 },
	{ 0x88f4, 0x88f4, PDF_CMAP_SINGLE, 2978 },
	{ 0x88f5, 0x88f7, PDF_CMAP_RANGE, 18811 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 2698 },
	{ 0x88f9, 0x88f9, PDF_CMAP_SINGLE, 1877 },
	{ 0x88fa, 0x88fb, PDF_CMAP_RANGE, 18814 },
	{ 0x88fc, 0x88fc, PDF_CMAP_SINGLE, 7095 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 9895 },
	{ 0x88fe, 0x88fe, PDF_CMAP_SINGLE, 7097 },
	{ 0x88ff, 0x88ff, PDF_CMAP_SINGLE, 18816 },
	{ 0x8900, 0x8901, PDF_CMAP_RANGE, 18817 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 1836 },
	{ 0x8903, 0x8906, PDF_CMAP_RANGE, 18819 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 9846 },
	{ 0x8908, 0x8909, PDF_CMAP_RANGE, 18823 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 7103 },
	{ 0x890b, 0x890f, PDF_CMAP_RANGE, 18825 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 1933 },
	{ 0x8911, 0x8911, PDF_CMAP_SINGLE, 18830 },
	{ 0x8912, 0x8912, PDF_CMAP_SINGLE, 1032 },
	{ 0x8913, 0x8913, PDF_CMAP_SINGLE, 7101 },
	{ 0x8914, 0x8918, PDF_CMAP_RANGE, 18831 },
	{ 0x8919, 0x8919, PDF_CMAP_SINGLE, 7100 },
	{ 0x891a, 0x891a, PDF_CMAP_SINGLE, 7094 },
	{ 0x891b, 0x891b, PDF_CMAP_SINGLE, 7102 },
	{ 0x891c, 0x8920, PDF_CMAP_RANGE, 18836 },
	{ 0x8921, 0x8921, PDF_CMAP_SINGLE, 7099 },
	{ 0x8922, 0x8924, PDF_CMAP_RANGE, 18841 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 3271 },
	{ 0x8926, 0x8929, PDF_CMAP_RANGE, 18844 },
	{ 0x892a, 0x892a, PDF_CMAP_SINGLE, 3707 },
	{ 0x892b, 0x892b, PDF_CMAP_SINGLE, 7105 },
	{ 0x892c, 0x892f, PDF_CMAP_RANGE, 18848 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 5945 },
	{ 0x8931, 0x8931, PDF_CMAP_SINGLE, 18852 },
	{ 0x8932, 0x8932, PDF_CMAP_SINGLE, 8161 },
	{ 0x8933, 0x8933, PDF_CMAP_SINGLE, 9659 },
	{ 0x8934, 0x8934, PDF_CMAP_SINGLE, 7104 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 18853 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 7106 },
	{ 0x8937, 0x8937, PDF_CMAP_SINGLE, 18854 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 9662 },
	{ 0x8939, 0x893a, PDF_CMAP_RANGE, 18855 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 8925 },
	{ 0x893c, 0x8940, PDF_CMAP_RANGE, 18857 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 7107 },
	{ 0x8942, 0x8943, PDF_CMAP_RANGE, 18862 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 3922 },
	{ 0x8945, 0x8946, PDF_CMAP_RANGE, 18864 },
	{ 0x8947, 0x8947, PDF_CMAP_SINGLE, 9661 },
	{ 0x8948, 0x8955, PDF_CMAP_RANGE, 18866 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 7722 },
	{ 0x8957, 0x895c, PDF_CMAP_RANGE, 18880 },
	{ 0x895d, 0x895d, PDF_CMAP_SINGLE, 9660 },
	{ 0x895e, 0x895e, PDF_CMAP_SINGLE, 7363 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 2244 },
	{ 0x8960, 0x8960, PDF_CMAP_SINGLE, 9658 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 18886 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 9663 },
	{ 0x8965, 0x8965, PDF_CMAP_SINGLE, 18889 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 7108 },
	{ 0x8967, 0x8969, PDF_CMAP_RANGE, 18890 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 8583 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 18893 },
	{ 0x896c, 0x896c, PDF_CMAP_SINGLE, 9835 },
	{ 0x896d, 0x896e, PDF_CMAP_RANGE, 18894 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 7807 },
	{ 0x8970, 0x8971, PDF_CMAP_RANGE, 18896 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 8621 },
	{ 0x8973, 0x897a, PDF_CMAP_RANGE, 18898 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 7109 },
	{ 0x897c, 0x897e, PDF_CMAP_RANGE, 18906 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 3846 },
	{ 0x8980, 0x8980, PDF_CMAP_SINGLE, 18909 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 4145 },
	{ 0x8982, 0x8982, PDF_CMAP_SINGLE, 18910 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 7135 },
	{ 0x8984, 0x8985, PDF_CMAP_RANGE, 18911 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 1709 },
	{ 0x8987, 0x898a, PDF_CMAP_RANGE, 18913 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 8086 },
	{ 0x898c, 0x898e, PDF_CMAP_RANGE, 18917 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 7986 },
	{ 0x8990, 0x8992, PDF_CMAP_RANGE, 18920 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 8324 },
	{ 0x8994, 0x8995, PDF_CMAP_RANGE, 18923 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 8502 },
	{ 0x8997, 0x8997, PDF_CMAP_SINGLE, 18925 },
	{ 0x8998, 0x8998, PDF_CMAP_SINGLE, 9400 },
	{ 0x8999, 0x89a0, PDF_CMAP_RANGE, 18926 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 9402 },
	{ 0x89a2, 0x89a5, PDF_CMAP_RANGE, 18934 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 9404 },
	{ 0x89a7, 0x89a9, PDF_CMAP_RANGE, 18938 },
	{ 0x89aa, 0x89aa, PDF_CMAP_SINGLE, 8420 },
	{ 0x89ab, 0x89ab, PDF_CMAP_SINGLE, 18941 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 9401 },
	{ 0x89ad, 0x89ae, PDF_CMAP_RANGE, 18942 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 9405 },
	{ 0x89b0, 0x89b1, PDF_CMAP_RANGE, 18944 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 9406 },
	{ 0x89b3, 0x89b6, PDF_CMAP_RANGE, 18946 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 9407 },
	{ 0x89b8, 0x89b9, PDF_CMAP_RANGE, 18950 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 8146 },
	{ 0x89bb, 0x89bc, PDF_CMAP_RANGE, 18952 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 8189 },
	{ 0x89be, 0x89be, PDF_CMAP_SINGLE, 18954 },
	{ 0x89bf, 0x89bf, PDF_CMAP_SINGLE, 9403 },
	{ 0x89c0, 0x89c0, PDF_CMAP_SINGLE, 7981 },
	{ 0x89c1, 0x89c1, PDF_CMAP_SINGLE, 2158 },
	{ 0x89c2, 0x89c2, PDF_CMAP_SINGLE, 1844 },
	{ 0x89c3, 0x89c3, PDF_CMAP_SINGLE, 18955 },
	{ 0x89c4, 0x89c4, PDF_CMAP_SINGLE, 1855 },
	{ 0x89c5, 0x89c5, PDF_CMAP_SINGLE, 2782 },
	{ 0x89c6, 0x89c6, PDF_CMAP_SINGLE, 3434 },
	{ 0x89c7, 0x89c7, PDF_CMAP_SINGLE, 6458 },
	{ 0x89c8, 0x89c8, PDF_CMAP_SINGLE, 2475 },
	{ 0x89c9, 0x89c9, PDF_CMAP_SINGLE, 2340 },
	{ 0x89ca, 0x89cc, PDF_CMAP_RANGE, 6459 },
	{ 0x89cd, 0x89cd, PDF_CMAP_SINGLE, 18956 },
	{ 0x89ce, 0x89d1, PDF_CMAP_RANGE, 6462 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 2200 },
	{ 0x89d3, 0x89d5, PDF_CMAP_RANGE, 18957 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 7511 },
	{ 0x89d7, 0x89d9, PDF_CMAP_RANGE, 18960 },
	{ 0x89da, 0x89da, PDF_CMAP_SINGLE, 7513 },
	{ 0x89db, 0x89db, PDF_CMAP_SINGLE, 18963 },
	{ 0x89dc, 0x89dc, PDF_CMAP_SINGLE, 7514 },
	{ 0x89dd, 0x89dd, PDF_CMAP_SINGLE, 18964 },
	{ 0x89de, 0x89de, PDF_CMAP_SINGLE, 7512 },
	{ 0x89df, 0x89e2, PDF_CMAP_RANGE, 18965 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 2227 },
	{ 0x89e4, 0x89e4, PDF_CMAP_SINGLE, 18969 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 7515 },
	{ 0x89e6, 0x89e6, PDF_CMAP_SINGLE, 1320 },
	{ 0x89e7, 0x89ea, PDF_CMAP_RANGE, 18970 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 7516 },
	{ 0x89ec, 0x89ee, PDF_CMAP_RANGE, 18974 },
	{ 0x89ef, 0x89ef, PDF_CMAP_SINGLE, 7517 },
	{ 0x89f0, 0x89f2, PDF_CMAP_RANGE, 18977 },
	{ 0x89f3, 0x89f3, PDF_CMAP_SINGLE, 6592 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 9737 },
	{ 0x89f5, 0x89f5, PDF_CMAP_SINGLE, 18980 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 9738 },
	{ 0x89f7, 0x89f7, PDF_CMAP_SINGLE, 18981 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7828 },
	{ 0x89f9, 0x89ff, PDF_CMAP_RANGE, 18982 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 4093 },
	{ 0x8a01, 0x8a01, PDF_CMAP_SINGLE, 8927 },
	{ 0x8a02, 0x8a02, PDF_CMAP_SINGLE, 7883 },
	{ 0x8a03, 0x8a03, PDF_CMAP_SINGLE, 7949 },
	{ 0x8a04, 0x8a06, PDF_CMAP_RANGE, 18989 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 4862 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 8055 },
	{ 0x8a09, 0x8a09, PDF_CMAP_SINGLE, 18992 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 8683 },
	{ 0x8a0b, 0x8a0b, PDF_CMAP_SINGLE, 18993 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 8929 },
	{ 0x8a0d, 0x8a0d, PDF_CMAP_SINGLE, 18994 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 8559 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 18995 },
	{ 0x8a10, 0x8a10, PDF_CMAP_SINGLE, 8928 },
	{ 0x8a11, 0x8a12, PDF_CMAP_RANGE, 18996 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 8682 },
	{ 0x8a14, 0x8a14, PDF_CMAP_SINGLE, 18998 },
	{ 0x8a15, 0x8a15, PDF_CMAP_SINGLE, 8930 },
	{ 0x8a16, 0x8a16, PDF_CMAP_SINGLE, 8395 },
	{ 0x8a17, 0x8a17, PDF_CMAP_SINGLE, 18999 },
	{ 0x8a18, 0x8a18, PDF_CMAP_SINGLE, 8056 },
	{ 0x8a19, 0x8a1a, PDF_CMAP_RANGE, 19000 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 7906 },
	{ 0x8a1c, 0x8a1c, PDF_CMAP_SINGLE, 19002 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 8690 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 19003 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 8528 },
	{ 0x8a20, 0x8a22, PDF_CMAP_RANGE, 19004 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 8147 },
	{ 0x8a24, 0x8a24, PDF_CMAP_SINGLE, 19007 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 8933 },
	{ 0x8a26, 0x8a29, PDF_CMAP_RANGE, 19008 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 7922 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 19012 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 8479 },
	{ 0x8a2e, 0x8a30, PDF_CMAP_RANGE, 19014 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 8669 },
	{ 0x8a32, 0x8a33, PDF_CMAP_RANGE, 19017 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 8532 },
	{ 0x8a35, 0x8a35, PDF_CMAP_SINGLE, 19019 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 8935 },
	{ 0x8a37, 0x8a39, PDF_CMAP_RANGE, 19020 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 8834 },
	{ 0x8a3b, 0x8a3d, PDF_CMAP_RANGE, 19023 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 7518 },
	{ 0x8a3f, 0x8a40, PDF_CMAP_RANGE, 19026 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 8934 },
	{ 0x8a42, 0x8a45, PDF_CMAP_RANGE, 19028 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 8936 },
	{ 0x8a47, 0x8a47, PDF_CMAP_SINGLE, 19032 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 6788 },
	{ 0x8a49, 0x8a4d, PDF_CMAP_RANGE, 19033 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 8932 },
	{ 0x8a4f, 0x8a4f, PDF_CMAP_SINGLE, 19038 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 8810 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 19039 },
	{ 0x8a52, 0x8a52, PDF_CMAP_SINGLE, 8939 },
	{ 0x8a53, 0x8a53, PDF_CMAP_SINGLE, 19040 },
	{ 0x8a54, 0x8a54, PDF_CMAP_SINGLE, 8937 },
	{ 0x8a55, 0x8a55, PDF_CMAP_SINGLE, 8381 },
	{ 0x8a56, 0x8a57, PDF_CMAP_RANGE, 19041 },
	{ 0x8a58, 0x8a58, PDF_CMAP_SINGLE, 8938 },
	{ 0x8a59, 0x8a5a, PDF_CMAP_RANGE, 19043 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 8893 },
	{ 0x8a5c, 0x8a5d, PDF_CMAP_RANGE, 19045 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 7838 },
	{ 0x8a5f, 0x8a60, PDF_CMAP_RANGE, 19047 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 8950 },
	{ 0x8a62, 0x8a62, PDF_CMAP_SINGLE, 8679 },
	{ 0x8a63, 0x8a63, PDF_CMAP_SINGLE, 8726 },
	{ 0x8a64, 0x8a65, PDF_CMAP_RANGE, 19049 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 8503 },
	{ 0x8a67, 0x8a68, PDF_CMAP_RANGE, 19051 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 8492 },
	{ 0x8a6a, 0x8a6a, PDF_CMAP_SINGLE, 19053 },
	{ 0x8a6b, 0x8a6b, PDF_CMAP_SINGLE, 7784 },
	{ 0x8a6c, 0x8a6c, PDF_CMAP_SINGLE, 8946 },
	{ 0x8a6d, 0x8a6d, PDF_CMAP_SINGLE, 7991 },
	{ 0x8a6e, 0x8a6e, PDF_CMAP_SINGLE, 8947 },
	{ 0x8a6f, 0x8a6f, PDF_CMAP_SINGLE, 19054 },
	{ 0x8a70, 0x8a70, PDF_CMAP_SINGLE, 8943 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 8016 },
	{ 0x8a72, 0x8a72, PDF_CMAP_SINGLE, 7952 },
	{ 0x8a73, 0x8a73, PDF_CMAP_SINGLE, 8648 },
	{ 0x8a74, 0x8a74, PDF_CMAP_SINGLE, 19055 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 8945 },
	{ 0x8a76, 0x8a78, PDF_CMAP_RANGE, 19056 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 4435 },
	{ 0x8a7a, 0x8a7b, PDF_CMAP_RANGE, 19059 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 8944 },
	{ 0x8a7d, 0x8a7e, PDF_CMAP_RANGE, 19061 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 8942 },
	{ 0x8a80, 0x8a83, PDF_CMAP_RANGE, 19063 },
	{ 0x8a84, 0x8a84, PDF_CMAP_SINGLE, 8941 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 8862 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 8940 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 8162 },
	{ 0x8a88, 0x8a88, PDF_CMAP_SINGLE, 19067 },
	{ 0x8a89, 0x8a89, PDF_CMAP_SINGLE, 4323 },
	{ 0x8a8a, 0x8a8a, PDF_CMAP_SINGLE, 3631 },
	{ 0x8a8b, 0x8a8c, PDF_CMAP_RANGE, 19068 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 8446 },
	{ 0x8a8e, 0x8a90, PDF_CMAP_RANGE, 19070 },
	{ 0x8a91, 0x8a92, PDF_CMAP_RANGE, 8953 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 3419 },
	{ 0x8a94, 0x8a94, PDF_CMAP_SINGLE, 19073 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 7855 },
	{ 0x8a96, 0x8a97, PDF_CMAP_RANGE, 19074 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 8758 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 19076 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 8951 },
	{ 0x8a9b, 0x8a9d, PDF_CMAP_RANGE, 19077 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 8765 },
	{ 0x8a9f, 0x8a9f, PDF_CMAP_SINGLE, 19080 },
	{ 0x8aa0, 0x8aa0, PDF_CMAP_SINGLE, 7810 },
	{ 0x8aa1, 0x8aa1, PDF_CMAP_SINGLE, 8118 },
	{ 0x8aa2, 0x8aa2, PDF_CMAP_SINGLE, 19081 },
	{ 0x8aa3, 0x8aa3, PDF_CMAP_SINGLE, 8612 },
	{ 0x8aa4, 0x8aa4, PDF_CMAP_SINGLE, 8618 },
	{ 0x8aa5, 0x8aa5, PDF_CMAP_SINGLE, 8952 },
	{ 0x8aa6, 0x8aa6, PDF_CMAP_SINGLE, 8529 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 19082 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 8032 },
	{ 0x8aa9, 0x8aab, PDF_CMAP_RANGE, 19083 },
	{ 0x8aac, 0x8aac, PDF_CMAP_SINGLE, 8519 },
	{ 0x8aad, 0x8aaf, PDF_CMAP_RANGE, 19086 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 8517 },
	{ 0x8ab1, 0x8ab1, PDF_CMAP_SINGLE, 19089 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 8156 },
	{ 0x8ab3, 0x8ab5, PDF_CMAP_RANGE, 19090 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 8961 },
	{ 0x8ab7, 0x8ab8, PDF_CMAP_RANGE, 19093 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 7925 },
	{ 0x8aba, 0x8abb, PDF_CMAP_RANGE, 19095 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 8728 },
	{ 0x8abd, 0x8abe, PDF_CMAP_RANGE, 19097 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 7878 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_RANGE, 19099 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 8960 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 19101 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 8884 },
	{ 0x8ac5, 0x8ac6, PDF_CMAP_RANGE, 19102 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 8553 },
	{ 0x8ac8, 0x8ac8, PDF_CMAP_SINGLE, 19104 },
	{ 0x8ac9, 0x8ac9, PDF_CMAP_SINGLE, 8957 },
	{ 0x8aca, 0x8aca, PDF_CMAP_SINGLE, 19105 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 8426 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 19106 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 8948 },
	{ 0x8ace, 0x8ace, PDF_CMAP_SINGLE, 19107 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 8955 },
	{ 0x8ad0, 0x8ad0, PDF_CMAP_SINGLE, 19108 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 8956 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 8231 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 19109 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 8288 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 8959 },
	{ 0x8ad8, 0x8ada, PDF_CMAP_RANGE, 19112 },
	{ 0x8adb, 0x8adb, PDF_CMAP_SINGLE, 8958 },
	{ 0x8adc, 0x8adc, PDF_CMAP_SINGLE, 7879 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 19115 },
	{ 0x8ade, 0x8ade, PDF_CMAP_SINGLE, 8972 },
	{ 0x8adf, 0x8ae1, PDF_CMAP_RANGE, 19116 },
	{ 0x8ae2, 0x8ae2, PDF_CMAP_SINGLE, 8949 },
	{ 0x8ae3, 0x8ae3, PDF_CMAP_SINGLE, 19119 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 8966 },
	{ 0x8ae5, 0x8ae5, PDF_CMAP_SINGLE, 19120 },
	{ 0x8ae6, 0x8ae6, PDF_CMAP_SINGLE, 8970 },
	{ 0x8ae7, 0x8ae7, PDF_CMAP_SINGLE, 8659 },
	{ 0x8ae8, 0x8aea, PDF_CMAP_RANGE, 19121 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 8963 },
	{ 0x8aec, 0x8aec, PDF_CMAP_SINGLE, 19124 },
	{ 0x8aed, 0x8aed, PDF_CMAP_SINGLE, 8967 },
	{ 0x8aee, 0x8aee, PDF_CMAP_SINGLE, 8971 },
	{ 0x8aef, 0x8af0, PDF_CMAP_RANGE, 19125 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 8031 },
	{ 0x8af2, 0x8af2, PDF_CMAP_SINGLE, 19127 },
	{ 0x8af3, 0x8af3, PDF_CMAP_SINGLE, 8969 },
	{ 0x8af4, 0x8af5, PDF_CMAP_RANGE, 19128 },
	{ 0x8af6, 0x8af6, PDF_CMAP_SINGLE, 8962 },
	{ 0x8af7, 0x8af7, PDF_CMAP_SINGLE, 7940 },
	{ 0x8af8, 0x8af8, PDF_CMAP_SINGLE, 8861 },
	{ 0x8af9, 0x8af9, PDF_CMAP_SINGLE, 19130 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 8699 },
	{ 0x8afb, 0x8afb, PDF_CMAP_SINGLE, 19131 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 8968 },
	{ 0x8afd, 0x8afd, PDF_CMAP_SINGLE, 19132 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 8364 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 19133 },
	{ 0x8b00, 0x8b00, PDF_CMAP_SINGLE, 8334 },
	{ 0x8b01, 0x8b01, PDF_CMAP_SINGLE, 8965 },
	{ 0x8b02, 0x8b02, PDF_CMAP_SINGLE, 8599 },
	{ 0x8b03, 0x8b03, PDF_CMAP_SINGLE, 19134 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 8561 },
	{ 0x8b05, 0x8b05, PDF_CMAP_SINGLE, 8856 },
	{ 0x8b06, 0x8b06, PDF_CMAP_SINGLE, 19135 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 5948 },
	{ 0x8b08, 0x8b09, PDF_CMAP_RANGE, 19136 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 8023 },
	{ 0x8b0b, 0x8b0d, PDF_CMAP_RANGE, 19138 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 8322 },
	{ 0x8b0f, 0x8b0f, PDF_CMAP_SINGLE, 19141 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 8977 },
	{ 0x8b11, 0x8b13, PDF_CMAP_RANGE, 19142 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 8964 },
	{ 0x8b15, 0x8b15, PDF_CMAP_SINGLE, 19145 },
	{ 0x8b16, 0x8b16, PDF_CMAP_SINGLE, 8975 },
	{ 0x8b17, 0x8b17, PDF_CMAP_SINGLE, 7733 },
	{ 0x8b18, 0x8b18, PDF_CMAP_SINGLE, 19146 },
	{ 0x8b19, 0x8b19, PDF_CMAP_SINGLE, 8401 },
	{ 0x8b1a, 0x8b1a, PDF_CMAP_SINGLE, 8976 },
	{ 0x8b1b, 0x8b1b, PDF_CMAP_SINGLE, 8099 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 19147 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 8662 },
	{ 0x8b1e, 0x8b20, PDF_CMAP_RANGE, 19148 },
	{ 0x8b21, 0x8b21, PDF_CMAP_SINGLE, 8710 },
	{ 0x8b22, 0x8b25, PDF_CMAP_RANGE, 19151 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 7519 },
	{ 0x8b27, 0x8b27, PDF_CMAP_SINGLE, 19155 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 8973 },
	{ 0x8b29, 0x8b2a, PDF_CMAP_RANGE, 19156 },
	{ 0x8b2b, 0x8b2b, PDF_CMAP_SINGLE, 8978 },
	{ 0x8b2c, 0x8b2c, PDF_CMAP_SINGLE, 8333 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 8979 },
	{ 0x8b2e, 0x8b32, PDF_CMAP_RANGE, 19158 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 8931 },
	{ 0x8b34, 0x8b38, PDF_CMAP_RANGE, 19163 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 8122 },
	{ 0x8b3a, 0x8b3d, PDF_CMAP_RANGE, 19168 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 8312 },
	{ 0x8b3f, 0x8b48, PDF_CMAP_RANGE, 19172 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 8840 },
	{ 0x8b4a, 0x8b4d, PDF_CMAP_RANGE, 19182 },
	{ 0x8b4e, 0x8b4e, PDF_CMAP_SINGLE, 8982 },
	{ 0x8b4f, 0x8b4f, PDF_CMAP_SINGLE, 8043 },
	{ 0x8b50, 0x8b55, PDF_CMAP_RANGE, 19186 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 8980 },
	{ 0x8b57, 0x8b57, PDF_CMAP_SINGLE, 19192 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 8496 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 8981 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 8552 },
	{ 0x8b5b, 0x8b5b, PDF_CMAP_SINGLE, 19193 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 8388 },
	{ 0x8b5d, 0x8b65, PDF_CMAP_RANGE, 19194 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 2270 },
	{ 0x8b67, 0x8b6a, PDF_CMAP_RANGE, 19203 },
	{ 0x8b6b, 0x8b6b, PDF_CMAP_SINGLE, 8984 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 3016 },
	{ 0x8b6d, 0x8b6e, PDF_CMAP_RANGE, 19207 },
	{ 0x8b6f, 0x8b6f, PDF_CMAP_SINGLE, 8729 },
	{ 0x8b70, 0x8b70, PDF_CMAP_SINGLE, 8727 },
	{ 0x8b71, 0x8b73, PDF_CMAP_RANGE, 19209 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 8405 },
	{ 0x8b75, 0x8b76, PDF_CMAP_RANGE, 19212 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 8010 },
	{ 0x8b78, 0x8b7c, PDF_CMAP_RANGE, 19214 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 8768 },
	{ 0x8b7e, 0x8b7f, PDF_CMAP_RANGE, 19219 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 7891 },
	{ 0x8b81, 0x8b89, PDF_CMAP_RANGE, 19221 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 7753 },
	{ 0x8b8b, 0x8b8d, PDF_CMAP_RANGE, 19230 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 9755 },
	{ 0x8b8f, 0x8b91, PDF_CMAP_RANGE, 19233 },
	{ 0x8b92, 0x8b92, PDF_CMAP_SINGLE, 7789 },
	{ 0x8b93, 0x8b93, PDF_CMAP_SINGLE, 8440 },
	{ 0x8b94, 0x8b94, PDF_CMAP_SINGLE, 19236 },
	{ 0x8b95, 0x8b95, PDF_CMAP_SINGLE, 8187 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 8985 },
	{ 0x8b97, 0x8b9b, PDF_CMAP_RANGE, 19237 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 8974 },
	{ 0x8b9d, 0x8b9d, PDF_CMAP_SINGLE, 19242 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 8983 },
	{ 0x8b9f, 0x8b9f, PDF_CMAP_SINGLE, 19243 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 4887 },
	{ 0x8ba1, 0x8ba1, PDF_CMAP_SINGLE, 2105 },
	{ 0x8ba2, 0x8ba2, PDF_CMAP_SINGLE, 1512 },
	{ 0x8ba3, 0x8ba3, PDF_CMAP_SINGLE, 1719 },
	{ 0x8ba4, 0x8ba4, PDF_CMAP_SINGLE, 3242 },
	{ 0x8ba5, 0x8ba5, PDF_CMAP_SINGLE, 2072 },
	{ 0x8ba6, 0x8ba7, PDF_CMAP_RANGE, 4888 },
	{ 0x8ba8, 0x8ba8, PDF_CMAP_SINGLE, 3625 },
	{ 0x8ba9, 0x8ba9, PDF_CMAP_SINGLE, 3230 },
	{ 0x8baa, 0x8baa, PDF_CMAP_SINGLE, 4890 },
	{ 0x8bab, 0x8bab, PDF_CMAP_SINGLE, 3100 },
	{ 0x8bac, 0x8bac, PDF_CMAP_SINGLE, 19244 },
	{ 0x8bad, 0x8bad, PDF_CMAP_SINGLE, 4062 },
	{ 0x8bae, 0x8bae, PDF_CMAP_SINGLE, 4208 },
	{ 0x8baf, 0x8baf, PDF_CMAP_SINGLE, 4063 },
	{ 0x8bb0, 0x8bb0, PDF_CMAP_SINGLE, 2106 },
	{ 0x8bb1, 0x8bb1, PDF_CMAP_SINGLE, 19245 },
	{ 0x8bb2, 0x8bb2, PDF_CMAP_SINGLE, 2179 },
	{ 0x8bb3, 0x8bb3, PDF_CMAP_SINGLE, 2041 },
	{ 0x8bb4, 0x8bb5, PDF_CMAP_RANGE, 4891 },
	{ 0x8bb6, 0x8bb6, PDF_CMAP_SINGLE, 4081 },
	{ 0x8bb7, 0x8bb7, PDF_CMAP_SINGLE, 4893 },
	{ 0x8bb8, 0x8bb8, PDF_CMAP_SINGLE, 4024 },
	{ 0x8bb9, 0x8bb9, PDF_CMAP_SINGLE, 1582 },
	{ 0x8bba, 0x8bba, PDF_CMAP_SINGLE, 2690 },
	{ 0x8bbb, 0x8bbb, PDF_CMAP_SINGLE, 19246 },
	{ 0x8bbc, 0x8bbc, PDF_CMAP_SINGLE, 3524 },
	{ 0x8bbd, 0x8bbd, PDF_CMAP_SINGLE, 1674 },
	{ 0x8bbe, 0x8bbe, PDF_CMAP_SINGLE, 3361 },
	{ 0x8bbf, 0x8bbf, PDF_CMAP_SINGLE, 1632 },
	{ 0x8bc0, 0x8bc0, PDF_CMAP_SINGLE, 2342 },
	{ 0x8bc1, 0x8bc1, PDF_CMAP_SINGLE, 4515 },
	{ 0x8bc2, 0x8bc3, PDF_CMAP_RANGE, 4894 },
	{ 0x8bc4, 0x8bc4, PDF_CMAP_SINGLE, 3039 },
	{ 0x8bc5, 0x8bc5, PDF_CMAP_SINGLE, 4676 },
	{ 0x8bc6, 0x8bc6, PDF_CMAP_SINGLE, 3405 },
	{ 0x8bc7, 0x8bc7, PDF_CMAP_SINGLE, 19247 },
	{ 0x8bc8, 0x8bc8, PDF_CMAP_SINGLE, 4426 },
	{ 0x8bc9, 0x8bc9, PDF_CMAP_SINGLE, 3540 },
	{ 0x8bca, 0x8bca, PDF_CMAP_SINGLE, 4496 },
	{ 0x8bcb, 0x8bcb, PDF_CMAP_SINGLE, 4896 },
	{ 0x8bcc, 0x8bcc, PDF_CMAP_SINGLE, 4574 },
	{ 0x8bcd, 0x8bcd, PDF_CMAP_SINGLE, 1357 },
	{ 0x8bce, 0x8bce, PDF_CMAP_SINGLE, 4898 },
	{ 0x8bcf, 0x8bcf, PDF_CMAP_SINGLE, 4897 },
	{ 0x8bd0, 0x8bd0, PDF_CMAP_SINGLE, 19248 },
	{ 0x8bd1, 0x8bd1, PDF_CMAP_SINGLE, 4210 },
	{ 0x8bd2, 0x8bd4, PDF_CMAP_RANGE, 4899 },
	{ 0x8bd5, 0x8bd5, PDF_CMAP_SINGLE, 3435 },
	{ 0x8bd6, 0x8bd6, PDF_CMAP_SINGLE, 4902 },
	{ 0x8bd7, 0x8bd7, PDF_CMAP_SINGLE, 3394 },
	{ 0x8bd8, 0x8bd9, PDF_CMAP_RANGE, 4903 },
	{ 0x8bda, 0x8bda, PDF_CMAP_SINGLE, 1268 },
	{ 0x8bdb, 0x8bdb, PDF_CMAP_SINGLE, 4590 },
	{ 0x8bdc, 0x8bdc, PDF_CMAP_SINGLE, 4905 },
	{ 0x8bdd, 0x8bdd, PDF_CMAP_SINGLE, 1989 },
	{ 0x8bde, 0x8bde, PDF_CMAP_SINGLE, 1423 },
	{ 0x8bdf, 0x8be0, PDF_CMAP_RANGE, 4906 },
	{ 0x8be1, 0x8be1, PDF_CMAP_SINGLE, 1863 },
	{ 0x8be2, 0x8be2, PDF_CMAP_SINGLE, 4056 },
	{ 0x8be3, 0x8be3, PDF_CMAP_SINGLE, 4207 },
	{ 0x8be4, 0x8be4, PDF_CMAP_SINGLE, 4908 },
	{ 0x8be5, 0x8be5, PDF_CMAP_SINGLE, 1726 },
	{ 0x8be6, 0x8be6, PDF_CMAP_SINGLE, 3927 },
	{ 0x8be7, 0x8be7, PDF_CMAP_SINGLE, 1206 },
	{ 0x8be8, 0x8be9, PDF_CMAP_RANGE, 4909 },
	{ 0x8bea, 0x8bea, PDF_CMAP_SINGLE, 19249 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 2236 },
	{ 0x8bec, 0x8bec, PDF_CMAP_SINGLE, 3819 },
	{ 0x8bed, 0x8bed, PDF_CMAP_SINGLE, 4308 },
	{ 0x8bee, 0x8bee, PDF_CMAP_SINGLE, 4911 },
	{ 0x8bef, 0x8bef, PDF_CMAP_SINGLE, 3842 },
	{ 0x8bf0, 0x8bf0, PDF_CMAP_SINGLE, 4912 },
	{ 0x8bf1, 0x8bf1, PDF_CMAP_SINGLE, 4282 },
	{ 0x8bf2, 0x8bf2, PDF_CMAP_SINGLE, 2042 },
	{ 0x8bf3, 0x8bf3, PDF_CMAP_SINGLE, 4913 },
	{ 0x8bf4, 0x8bf4, PDF_CMAP_SINGLE, 3498 },
	{ 0x8bf5, 0x8bf5, PDF_CMAP_SINGLE, 3525 },
	{ 0x8bf6, 0x8bf6, PDF_CMAP_SINGLE, 4914 },
	{ 0x8bf7, 0x8bf7, PDF_CMAP_SINGLE, 3176 },
	{ 0x8bf8, 0x8bf8, PDF_CMAP_SINGLE, 4589 },
	{ 0x8bf9, 0x8bf9, PDF_CMAP_SINGLE, 4915 },
	{ 0x8bfa, 0x8bfa, PDF_CMAP_SINGLE, 2934 },
	{ 0x8bfb, 0x8bfb, PDF_CMAP_SINGLE, 1536 },
	{ 0x8bfc, 0x8bfc, PDF_CMAP_SINGLE, 4916 },
	{ 0x8bfd, 0x8bfd, PDF_CMAP_SINGLE, 1641 },
	{ 0x8bfe, 0x8bfe, PDF_CMAP_SINGLE, 2395 },
	{ 0x8bff, 0x8bff, PDF_CMAP_SINGLE, 4917 },
	{ 0x8c00, 0x8c00, PDF_CMAP_SINGLE, 4918 },
	{ 0x8c01, 0x8c01, PDF_CMAP_SINGLE, 3490 },
	{ 0x8c02, 0x8c02, PDF_CMAP_SINGLE, 4919 },
	{ 0x8c03, 0x8c03, PDF_CMAP_SINGLE, 1496 },
	{ 0x8c04, 0x8c04, PDF_CMAP_SINGLE, 4920 },
	{ 0x8c05, 0x8c05, PDF_CMAP_SINGLE, 2571 },
	{ 0x8c06, 0x8c06, PDF_CMAP_SINGLE, 4632 },
	{ 0x8c07, 0x8c07, PDF_CMAP_SINGLE, 4921 },
	{ 0x8c08, 0x8c08, PDF_CMAP_SINGLE, 3595 },
	{ 0x8c09, 0x8c09, PDF_CMAP_SINGLE, 19250 },
	{ 0x8c0a, 0x8c0a, PDF_CMAP_SINGLE, 4209 },
	{ 0x8c0b, 0x8c0b, PDF_CMAP_SINGLE, 2836 },
	{ 0x8c0c, 0x8c0c, PDF_CMAP_SINGLE, 4922 },
	{ 0x8c0d, 0x8c0d, PDF_CMAP_SINGLE, 1502 },
	{ 0x8c0e, 0x8c0e, PDF_CMAP_SINGLE, 2022 },
	{ 0x8c0f, 0x8c0f, PDF_CMAP_SINGLE, 4923 },
	{ 0x8c10, 0x8c10, PDF_CMAP_SINGLE, 3966 },
	{ 0x8c11, 0x8c12, PDF_CMAP_RANGE, 4924 },
	{ 0x8c13, 0x8c13, PDF_CMAP_SINGLE, 3788 },
	{ 0x8c14, 0x8c16, PDF_CMAP_RANGE, 4926 },
	{ 0x8c17, 0x8c17, PDF_CMAP_SINGLE, 1214 },
	{ 0x8c18, 0x8c18, PDF_CMAP_SINGLE, 4931 },
	{ 0x8c19, 0x8c19, PDF_CMAP_SINGLE, 4929 },
	{ 0x8c1a, 0x8c1a, PDF_CMAP_SINGLE, 4113 },
	{ 0x8c1b, 0x8c1b, PDF_CMAP_SINGLE, 4930 },
	{ 0x8c1c, 0x8c1c, PDF_CMAP_SINGLE, 2778 },
	{ 0x8c1d, 0x8c1d, PDF_CMAP_SINGLE, 4932 },
	{ 0x8c1e, 0x8c1e, PDF_CMAP_SINGLE, 19251 },
	{ 0x8c1f, 0x8c21, PDF_CMAP_RANGE, 4933 },
	{ 0x8c22, 0x8c22, PDF_CMAP_SINGLE, 3974 },
	{ 0x8c23, 0x8c23, PDF_CMAP_SINGLE, 4140 },
	{ 0x8c24, 0x8c24, PDF_CMAP_SINGLE, 1028 },
	{ 0x8c25, 0x8c25, PDF_CMAP_SINGLE, 4936 },
	{ 0x8c26, 0x8c26, PDF_CMAP_SINGLE, 3112 },
	{ 0x8c27, 0x8c27, PDF_CMAP_SINGLE, 4937 },
	{ 0x8c28, 0x8c28, PDF_CMAP_SINGLE, 2248 },
	{ 0x8c29, 0x8c29, PDF_CMAP_SINGLE, 2726 },
	{ 0x8c2a, 0x8c2b, PDF_CMAP_RANGE, 4938 },
	{ 0x8c2c, 0x8c2c, PDF_CMAP_SINGLE, 2818 },
	{ 0x8c2d, 0x8c2d, PDF_CMAP_SINGLE, 3594 },
	{ 0x8c2e, 0x8c2f, PDF_CMAP_RANGE, 4940 },
	{ 0x8c30, 0x8c30, PDF_CMAP_SINGLE, 2473 },
	{ 0x8c31, 0x8c31, PDF_CMAP_SINGLE, 3062 },
	{ 0x8c32, 0x8c33, PDF_CMAP_RANGE, 4942 },
	{ 0x8c34, 0x8c34, PDF_CMAP_SINGLE, 3121 },
	{ 0x8c35, 0x8c36, PDF_CMAP_RANGE, 4944 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 1825 },
	{ 0x8c38, 0x8c40, PDF_CMAP_RANGE, 19252 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 2050 },
	{ 0x8c42, 0x8c45, PDF_CMAP_RANGE, 19261 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 1528 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 7414 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 8392 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 7415 },
	{ 0x8c4a, 0x8c4b, PDF_CMAP_RANGE, 19265 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 3732 },
	{ 0x8c4d, 0x8c4f, PDF_CMAP_RANGE, 19267 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 7933 },
	{ 0x8c51, 0x8c54, PDF_CMAP_RANGE, 19270 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 7445 },
	{ 0x8c56, 0x8c59, PDF_CMAP_RANGE, 19274 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 6544 },
	{ 0x8c5b, 0x8c60, PDF_CMAP_RANGE, 19278 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 3936 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 2004 },
	{ 0x8c63, 0x8c69, PDF_CMAP_RANGE, 19284 },
	{ 0x8c6a, 0x8c6a, PDF_CMAP_SINGLE, 1911 },
	{ 0x8c6b, 0x8c6b, PDF_CMAP_SINGLE, 4328 },
	{ 0x8c6c, 0x8c72, PDF_CMAP_RANGE, 19291 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 5597 },
	{ 0x8c74, 0x8c77, PDF_CMAP_RANGE, 19298 },
	{ 0x8c78, 0x8c78, PDF_CMAP_SINGLE, 7504 },
	{ 0x8c79, 0x8c79, PDF_CMAP_SINGLE, 1043 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 1209 },
	{ 0x8c7b, 0x8c81, PDF_CMAP_RANGE, 19302 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 7505 },
	{ 0x8c83, 0x8c84, PDF_CMAP_RANGE, 19309 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 7507 },
	{ 0x8c86, 0x8c88, PDF_CMAP_RANGE, 19311 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 1928 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 7506 },
	{ 0x8c8b, 0x8c8b, PDF_CMAP_SINGLE, 19314 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 2743 },
	{ 0x8c8d, 0x8c93, PDF_CMAP_RANGE, 19315 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 7509 },
	{ 0x8c95, 0x8c97, PDF_CMAP_RANGE, 19322 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 7508 },
	{ 0x8c99, 0x8c9c, PDF_CMAP_RANGE, 19325 },
	{ 0x8c9d, 0x8c9d, PDF_CMAP_SINGLE, 7739 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 8831 },
	{ 0x8c9f, 0x8c9f, PDF_CMAP_SINGLE, 19329 },
	{ 0x8ca0, 0x8ca0, PDF_CMAP_SINGLE, 7948 },
	{ 0x8ca1, 0x8ca1, PDF_CMAP_SINGLE, 7769 },
	{ 0x8ca2, 0x8ca2, PDF_CMAP_SINGLE, 7972 },
	{ 0x8ca3, 0x8ca6, PDF_CMAP_RANGE, 19330 },
	{ 0x8ca7, 0x8ca7, PDF_CMAP_SINGLE, 8378 },
	{ 0x8ca8, 0x8ca8, PDF_CMAP_SINGLE, 8037 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 7920 },
	{ 0x8caa, 0x8caa, PDF_CMAP_SINGLE, 8548 },
	{ 0x8cab, 0x8cab, PDF_CMAP_SINGLE, 7984 },
	{ 0x8cac, 0x8cac, PDF_CMAP_SINGLE, 8801 },
	{ 0x8cad, 0x8cae, PDF_CMAP_RANGE, 19334 },
	{ 0x8caf, 0x8caf, PDF_CMAP_SINGLE, 8866 },
	{ 0x8cb0, 0x8cb0, PDF_CMAP_SINGLE, 9387 },
	{ 0x8cb1, 0x8cb1, PDF_CMAP_SINGLE, 19336 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 9391 },
	{ 0x8cb3, 0x8cb3, PDF_CMAP_SINGLE, 7912 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 7993 },
	{ 0x8cb5, 0x8cb5, PDF_CMAP_SINGLE, 19337 },
	{ 0x8cb6, 0x8cb6, PDF_CMAP_SINGLE, 7752 },
	{ 0x8cb7, 0x8cb7, PDF_CMAP_SINGLE, 8304 },
	{ 0x8cb8, 0x8cb8, PDF_CMAP_SINGLE, 7848 },
	{ 0x8cb9, 0x8cb9, PDF_CMAP_SINGLE, 19338 },
	{ 0x8cba, 0x8cba, PDF_CMAP_SINGLE, 9388 },
	{ 0x8cbb, 0x8cbb, PDF_CMAP_SINGLE, 7927 },
	{ 0x8cbc, 0x8cbc, PDF_CMAP_SINGLE, 8566 },
	{ 0x8cbd, 0x8cbd, PDF_CMAP_SINGLE, 9389 },
	{ 0x8cbe, 0x8cbe, PDF_CMAP_SINGLE, 19339 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 8315 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 8005 },
	{ 0x8cc1, 0x8cc1, PDF_CMAP_SINGLE, 9386 },
	{ 0x8cc2, 0x8cc2, PDF_CMAP_SINGLE, 8267 },
	{ 0x8cc3, 0x8cc3, PDF_CMAP_SINGLE, 8239 },
	{ 0x8cc4, 0x8cc4, PDF_CMAP_SINGLE, 8026 },
	{ 0x8cc5, 0x8cc5, PDF_CMAP_SINGLE, 9392 },
	{ 0x8cc6, 0x8cc6, PDF_CMAP_SINGLE, 19340 },
	{ 0x8cc7, 0x8cc7, PDF_CMAP_SINGLE, 8887 },
	{ 0x8cc8, 0x8cc8, PDF_CMAP_SINGLE, 8063 },
	{ 0x8cc9, 0x8cc9, PDF_CMAP_SINGLE, 19341 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 8805 },
	{ 0x8ccb, 0x8cd0, PDF_CMAP_RANGE, 19342 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 9394 },
	{ 0x8cd2, 0x8cd2, PDF_CMAP_SINGLE, 8475 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 7761 },
	{ 0x8cd4, 0x8cd4, PDF_CMAP_SINGLE, 19348 },
	{ 0x8cd5, 0x8cd5, PDF_CMAP_SINGLE, 9396 },
	{ 0x8cd6, 0x8cd9, PDF_CMAP_RANGE, 19349 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 9395 },
	{ 0x8cdb, 0x8cdb, PDF_CMAP_SINGLE, 19353 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 7839 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 19354 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 8472 },
	{ 0x8cdf, 0x8cdf, PDF_CMAP_SINGLE, 19355 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 8372 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 9135 },
	{ 0x8ce2, 0x8ce2, PDF_CMAP_SINGLE, 8635 },
	{ 0x8ce3, 0x8ce3, PDF_CMAP_SINGLE, 8306 },
	{ 0x8ce4, 0x8ce4, PDF_CMAP_SINGLE, 8085 },
	{ 0x8ce5, 0x8ce5, PDF_CMAP_SINGLE, 19356 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 7946 },
	{ 0x8ce7, 0x8ce7, PDF_CMAP_SINGLE, 9398 },
	{ 0x8ce8, 0x8ce9, PDF_CMAP_RANGE, 19357 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 8849 },
	{ 0x8ceb, 0x8ceb, PDF_CMAP_SINGLE, 9397 },
	{ 0x8cec, 0x8cec, PDF_CMAP_SINGLE, 8824 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 7892 },
	{ 0x8cee, 0x8cf3, PDF_CMAP_RANGE, 19359 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8179 },
	{ 0x8cf5, 0x8cf9, PDF_CMAP_RANGE, 19365 },
	{ 0x8cfa, 0x8cfa, PDF_CMAP_SINGLE, 8873 },
	{ 0x8cfb, 0x8cfb, PDF_CMAP_SINGLE, 9399 },
	{ 0x8cfc, 0x8cfc, PDF_CMAP_SINGLE, 7976 },
	{ 0x8cfd, 0x8cfd, PDF_CMAP_SINGLE, 8457 },
	{ 0x8cfe, 0x8cfe, PDF_CMAP_SINGLE, 8903 },
	{ 0x8cff, 0x8cff, PDF_CMAP_SINGLE, 19370 },
	{ 0x8d00, 0x8d03, PDF_CMAP_RANGE, 19371 },
	{ 0x8d04, 0x8d04, PDF_CMAP_SINGLE, 9390 },
	{ 0x8d05, 0x8d05, PDF_CMAP_SINGLE, 8881 },
	{ 0x8d06, 0x8d07, PDF_CMAP_RANGE, 19375 },
	{ 0x8d08, 0x8d08, PDF_CMAP_SINGLE, 8806 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 19377 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 8795 },
	{ 0x8d0b, 0x8d0b, PDF_CMAP_SINGLE, 8900 },
	{ 0x8d0c, 0x8d0c, PDF_CMAP_SINGLE, 19378 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 8469 },
	{ 0x8d0e, 0x8d0e, PDF_CMAP_SINGLE, 19379 },
	{ 0x8d0f, 0x8d0f, PDF_CMAP_SINGLE, 8746 },
	{ 0x8d10, 0x8d10, PDF_CMAP_SINGLE, 9393 },
	{ 0x8d11, 0x8d15, PDF_CMAP_RANGE, 19380 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 8509 },
	{ 0x8d17, 0x8d1a, PDF_CMAP_RANGE, 19385 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 7957 },
	{ 0x8d1c, 0x8d1c, PDF_CMAP_SINGLE, 8796 },
	{ 0x8d1d, 0x8d1d, PDF_CMAP_SINGLE, 1053 },
	{ 0x8d1e, 0x8d1e, PDF_CMAP_SINGLE, 4491 },
	{ 0x8d1f, 0x8d1f, PDF_CMAP_SINGLE, 1717 },
	{ 0x8d20, 0x8d20, PDF_CMAP_SINGLE, 19389 },
	{ 0x8d21, 0x8d21, PDF_CMAP_SINGLE, 1802 },
	{ 0x8d22, 0x8d22, PDF_CMAP_SINGLE, 1165 },
	{ 0x8d23, 0x8d23, PDF_CMAP_SINGLE, 4403 },
	{ 0x8d24, 0x8d24, PDF_CMAP_SINGLE, 3898 },
	{ 0x8d25, 0x8d25, PDF_CMAP_SINGLE, 999 },
	{ 0x8d26, 0x8d26, PDF_CMAP_SINGLE, 4460 },
	{ 0x8d27, 0x8d27, PDF_CMAP_SINGLE, 2058 },
	{ 0x8d28, 0x8d28, PDF_CMAP_SINGLE, 4553 },
	{ 0x8d29, 0x8d29, PDF_CMAP_SINGLE, 1620 },
	{ 0x8d2a, 0x8d2a, PDF_CMAP_SINGLE, 3587 },
	{ 0x8d2b, 0x8d2b, PDF_CMAP_SINGLE, 3029 },
	{ 0x8d2c, 0x8d2c, PDF_CMAP_SINGLE, 1098 },
	{ 0x8d2d, 0x8d2d, PDF_CMAP_SINGLE, 1811 },
	{ 0x8d2e, 0x8d2e, PDF_CMAP_SINGLE, 4603 },
	{ 0x8d2f, 0x8d2f, PDF_CMAP_SINGLE, 1850 },
	{ 0x8d30, 0x8d30, PDF_CMAP_SINGLE, 1598 },
	{ 0x8d31, 0x8d31, PDF_CMAP_SINGLE, 2157 },
	{ 0x8d32, 0x8d33, PDF_CMAP_RANGE, 6444 },
	{ 0x8d34, 0x8d34, PDF_CMAP_SINGLE, 3660 },
	{ 0x8d35, 0x8d35, PDF_CMAP_SINGLE, 1868 },
	{ 0x8d36, 0x8d36, PDF_CMAP_SINGLE, 6446 },
	{ 0x8d37, 0x8d37, PDF_CMAP_SINGLE, 1406 },
	{ 0x8d38, 0x8d38, PDF_CMAP_SINGLE, 2744 },
	{ 0x8d39, 0x8d39, PDF_CMAP_SINGLE, 1646 },
	{ 0x8d3a, 0x8d3a, PDF_CMAP_SINGLE, 1935 },
	{ 0x8d3b, 0x8d3b, PDF_CMAP_SINGLE, 6447 },
	{ 0x8d3c, 0x8d3c, PDF_CMAP_SINGLE, 4407 },
	{ 0x8d3d, 0x8d3d, PDF_CMAP_SINGLE, 6448 },
	{ 0x8d3e, 0x8d3e, PDF_CMAP_SINGLE, 2121 },
	{ 0x8d3f, 0x8d3f, PDF_CMAP_SINGLE, 2036 },
	{ 0x8d40, 0x8d40, PDF_CMAP_SINGLE, 6449 },
	{ 0x8d41, 0x8d41, PDF_CMAP_SINGLE, 2599 },
	{ 0x8d42, 0x8d42, PDF_CMAP_SINGLE, 2655 },
	{ 0x8d43, 0x8d43, PDF_CMAP_SINGLE, 4386 },
	{ 0x8d44, 0x8d44, PDF_CMAP_SINGLE, 4647 },
	{ 0x8d45, 0x8d46, PDF_CMAP_RANGE, 6450 },
	{ 0x8d47, 0x8d47, PDF_CMAP_SINGLE, 6454 },
	{ 0x8d48, 0x8d49, PDF_CMAP_RANGE, 6452 },
	{ 0x8d4a, 0x8d4a, PDF_CMAP_SINGLE, 3351 },
	{ 0x8d4b, 0x8d4b, PDF_CMAP_SINGLE, 1710 },
	{ 0x8d4c, 0x8d4c, PDF_CMAP_SINGLE, 1539 },
	{ 0x8d4d, 0x8d4d, PDF_CMAP_SINGLE, 6455 },
	{ 0x8d4e, 0x8d4e, PDF_CMAP_SINGLE, 3457 },
	{ 0x8d4f, 0x8d4f, PDF_CMAP_SINGLE, 3334 },
	{ 0x8d50, 0x8d50, PDF_CMAP_SINGLE, 1360 },
	{ 0x8d51, 0x8d52, PDF_CMAP_RANGE, 19390 },
	{ 0x8d53, 0x8d53, PDF_CMAP_SINGLE, 5691 },
	{ 0x8d54, 0x8d54, PDF_CMAP_SINGLE, 2979 },
	{ 0x8d55, 0x8d55, PDF_CMAP_SINGLE, 6456 },
	{ 0x8d56, 0x8d56, PDF_CMAP_SINGLE, 2464 },
	{ 0x8d57, 0x8d57, PDF_CMAP_SINGLE, 19392 },
	{ 0x8d58, 0x8d58, PDF_CMAP_SINGLE, 4629 },
	{ 0x8d59, 0x8d59, PDF_CMAP_SINGLE, 6457 },
	{ 0x8d5a, 0x8d5a, PDF_CMAP_SINGLE, 4617 },
	{ 0x8d5b, 0x8d5b, PDF_CMAP_SINGLE, 3287 },
	{ 0x8d5c, 0x8d5c, PDF_CMAP_SINGLE, 4745 },
	{ 0x8d5d, 0x8d5d, PDF_CMAP_SINGLE, 4739 },
	{ 0x8d5e, 0x8d5e, PDF_CMAP_SINGLE, 4385 },
	{ 0x8d5f, 0x8d5f, PDF_CMAP_SINGLE, 19393 },
	{ 0x8d60, 0x8d60, PDF_CMAP_SINGLE, 4412 },
	{ 0x8d61, 0x8d61, PDF_CMAP_SINGLE, 3325 },
	{ 0x8d62, 0x8d62, PDF_CMAP_SINGLE, 4243 },
	{ 0x8d63, 0x8d63, PDF_CMAP_SINGLE, 1742 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 1285 },
	{ 0x8d65, 0x8d65, PDF_CMAP_SINGLE, 19394 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 3355 },
	{ 0x8d67, 0x8d67, PDF_CMAP_SINGLE, 7412 },
	{ 0x8d68, 0x8d6a, PDF_CMAP_RANGE, 19395 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 1932 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 19398 },
	{ 0x8d6d, 0x8d6d, PDF_CMAP_SINGLE, 7413 },
	{ 0x8d6e, 0x8d6f, PDF_CMAP_RANGE, 19399 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 4668 },
	{ 0x8d71, 0x8d72, PDF_CMAP_RANGE, 19401 },
	{ 0x8d73, 0x8d73, PDF_CMAP_SINGLE, 7407 },
	{ 0x8d74, 0x8d74, PDF_CMAP_SINGLE, 1707 },
	{ 0x8d75, 0x8d75, PDF_CMAP_SINGLE, 4469 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 1738 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 3087 },
	{ 0x8d78, 0x8d80, PDF_CMAP_RANGE, 19403 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 1256 },
	{ 0x8d82, 0x8d83, PDF_CMAP_RANGE, 19412 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 7408 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 1233 },
	{ 0x8d86, 0x8d89, PDF_CMAP_RANGE, 19414 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 4352 },
	{ 0x8d8b, 0x8d8b, PDF_CMAP_SINGLE, 3188 },
	{ 0x8d8c, 0x8d90, PDF_CMAP_RANGE, 19418 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 7410 },
	{ 0x8d92, 0x8d93, PDF_CMAP_RANGE, 19423 },
	{ 0x8d94, 0x8d94, PDF_CMAP_SINGLE, 7409 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 7956 },
	{ 0x8d96, 0x8d98, PDF_CMAP_RANGE, 19425 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 8826 },
	{ 0x8d9a, 0x8d9e, PDF_CMAP_RANGE, 19428 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 3614 },
	{ 0x8da0, 0x8da2, PDF_CMAP_RANGE, 19433 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 3199 },
	{ 0x8da4, 0x8da7, PDF_CMAP_RANGE, 19436 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 8430 },
	{ 0x8da9, 0x8db0, PDF_CMAP_RANGE, 19440 },
	{ 0x8db1, 0x8db1, PDF_CMAP_SINGLE, 7411 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 9720 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 4672 },
	{ 0x8db4, 0x8db4, PDF_CMAP_SINGLE, 2944 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 7452 },
	{ 0x8db6, 0x8db7, PDF_CMAP_RANGE, 19448 },
	{ 0x8db8, 0x8db8, PDF_CMAP_SINGLE, 7447 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 19450 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 7455 },
	{ 0x8dbb, 0x8dbb, PDF_CMAP_SINGLE, 19451 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 7454 },
	{ 0x8dbd, 0x8dbd, PDF_CMAP_SINGLE, 19452 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 4537 },
	{ 0x8dbf, 0x8dbf, PDF_CMAP_SINGLE, 7453 },
	{ 0x8dc0, 0x8dc2, PDF_CMAP_RANGE, 19453 },
	{ 0x8dc3, 0x8dc3, PDF_CMAP_SINGLE, 4353 },
	{ 0x8dc4, 0x8dc4, PDF_CMAP_SINGLE, 7456 },
	{ 0x8dc5, 0x8dc5, PDF_CMAP_SINGLE, 19456 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 7464 },
	{ 0x8dc7, 0x8dca, PDF_CMAP_RANGE, 19457 },
	{ 0x8dcb, 0x8dcb, PDF_CMAP_SINGLE, 986 },
	{ 0x8dcc, 0x8dcc, PDF_CMAP_SINGLE, 1497 },
	{ 0x8dcd, 0x8dcd, PDF_CMAP_SINGLE, 19461 },
	{ 0x8dce, 0x8dcf, PDF_CMAP_RANGE, 7461 },
	{ 0x8dd0, 0x8dd0, PDF_CMAP_SINGLE, 19462 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 2973 },
	{ 0x8dd2, 0x8dd5, PDF_CMAP_RANGE, 19463 },
	{ 0x8dd6, 0x8dd7, PDF_CMAP_RANGE, 7457 },
	{ 0x8dd8, 0x8dd9, PDF_CMAP_RANGE, 19467 },
	{ 0x8dda, 0x8dda, PDF_CMAP_SINGLE, 7459 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 7463 },
	{ 0x8ddc, 0x8ddc, PDF_CMAP_SINGLE, 19469 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 2319 },
	{ 0x8dde, 0x8dde, PDF_CMAP_SINGLE, 7460 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 1781 },
	{ 0x8de0, 0x8de2, PDF_CMAP_RANGE, 19470 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 7468 },
	{ 0x8de4, 0x8de4, PDF_CMAP_SINGLE, 7471 },
	{ 0x8de5, 0x8de7, PDF_CMAP_RANGE, 19473 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 2420 },
	{ 0x8de9, 0x8de9, PDF_CMAP_SINGLE, 19476 },
	{ 0x8dea, 0x8dea, PDF_CMAP_SINGLE, 1867 },
	{ 0x8deb, 0x8deb, PDF_CMAP_SINGLE, 7448 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 7465 },
	{ 0x8ded, 0x8dee, PDF_CMAP_RANGE, 19477 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 2654 },
	{ 0x8df0, 0x8df2, PDF_CMAP_RANGE, 19479 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3659 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 19482 },
	{ 0x8df5, 0x8df5, PDF_CMAP_SINGLE, 2156 },
	{ 0x8df6, 0x8df6, PDF_CMAP_SINGLE, 19483 },
	{ 0x8df7, 0x8df8, PDF_CMAP_RANGE, 7466 },
	{ 0x8df9, 0x8df9, PDF_CMAP_SINGLE, 7469 },
	{ 0x8dfa, 0x8dfa, PDF_CMAP_SINGLE, 1572 },
	{ 0x8dfb, 0x8dfb, PDF_CMAP_SINGLE, 7470 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 19484 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 7473 },
	{ 0x8dfe, 0x8dff, PDF_CMAP_RANGE, 19485 },
	{ 0x8e00, 0x8e04, PDF_CMAP_RANGE, 19487 },
	{ 0x8e05, 0x8e05, PDF_CMAP_SINGLE, 7449 },
	{ 0x8e06, 0x8e08, PDF_CMAP_RANGE, 19492 },
	{ 0x8e09, 0x8e09, PDF_CMAP_SINGLE, 7472 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 4256 },
	{ 0x8e0b, 0x8e0b, PDF_CMAP_SINGLE, 19495 },
	{ 0x8e0c, 0x8e0c, PDF_CMAP_SINGLE, 1297 },
	{ 0x8e0d, 0x8e0e, PDF_CMAP_RANGE, 19496 },
	{ 0x8e0f, 0x8e0f, PDF_CMAP_SINGLE, 3575 },
	{ 0x8e10, 0x8e10, PDF_CMAP_SINGLE, 8084 },
	{ 0x8e11, 0x8e13, PDF_CMAP_RANGE, 19498 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 7474 },
	{ 0x8e15, 0x8e1c, PDF_CMAP_RANGE, 19501 },
	{ 0x8e1d, 0x8e1d, PDF_CMAP_SINGLE, 7475 },
	{ 0x8e1e, 0x8e1e, PDF_CMAP_SINGLE, 2320 },
	{ 0x8e1f, 0x8e1f, PDF_CMAP_SINGLE, 7476 },
	{ 0x8e20, 0x8e21, PDF_CMAP_RANGE, 19509 },
	{ 0x8e22, 0x8e22, PDF_CMAP_SINGLE, 3634 },
	{ 0x8e23, 0x8e23, PDF_CMAP_SINGLE, 7479 },
	{ 0x8e24, 0x8e28, PDF_CMAP_RANGE, 19511 },
	{ 0x8e29, 0x8e29, PDF_CMAP_SINGLE, 1167 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 4662 },
	{ 0x8e2b, 0x8e2b, PDF_CMAP_SINGLE, 19516 },
	{ 0x8e2c, 0x8e2c, PDF_CMAP_SINGLE, 7477 },
	{ 0x8e2d, 0x8e2d, PDF_CMAP_SINGLE, 19517 },
	{ 0x8e2e, 0x8e2e, PDF_CMAP_SINGLE, 7478 },
	{ 0x8e2f, 0x8e2f, PDF_CMAP_SINGLE, 7480 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 19518 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 7486 },
	{ 0x8e32, 0x8e33, PDF_CMAP_RANGE, 19519 },
	{ 0x8e34, 0x8e34, PDF_CMAP_SINGLE, 8752 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 7484 },
	{ 0x8e36, 0x8e38, PDF_CMAP_RANGE, 19521 },
	{ 0x8e39, 0x8e39, PDF_CMAP_SINGLE, 7483 },
	{ 0x8e3a, 0x8e3a, PDF_CMAP_SINGLE, 7481 },
	{ 0x8e3b, 0x8e3c, PDF_CMAP_RANGE, 19524 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 7485 },
	{ 0x8e3e, 0x8e3f, PDF_CMAP_RANGE, 19526 },
	{ 0x8e40, 0x8e40, PDF_CMAP_SINGLE, 7482 },
	{ 0x8e41, 0x8e42, PDF_CMAP_RANGE, 7488 },
	{ 0x8e43, 0x8e43, PDF_CMAP_SINGLE, 19528 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 3638 },
	{ 0x8e45, 0x8e46, PDF_CMAP_RANGE, 19529 },
	{ 0x8e47, 0x8e47, PDF_CMAP_SINGLE, 5947 },
	{ 0x8e48, 0x8e48, PDF_CMAP_SINGLE, 1433 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 7487 },
	{ 0x8e4a, 0x8e4a, PDF_CMAP_SINGLE, 7492 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 3574 },
	{ 0x8e4c, 0x8e4c, PDF_CMAP_SINGLE, 9725 },
	{ 0x8e4d, 0x8e50, PDF_CMAP_RANGE, 19531 },
	{ 0x8e51, 0x8e52, PDF_CMAP_RANGE, 7490 },
	{ 0x8e53, 0x8e54, PDF_CMAP_RANGE, 19535 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 9728 },
	{ 0x8e56, 0x8e58, PDF_CMAP_RANGE, 19537 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 7450 },
	{ 0x8e5a, 0x8e62, PDF_CMAP_RANGE, 19540 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 9734 },
	{ 0x8e64, 0x8e65, PDF_CMAP_RANGE, 19549 },
	{ 0x8e66, 0x8e66, PDF_CMAP_SINGLE, 1069 },
	{ 0x8e67, 0x8e68, PDF_CMAP_RANGE, 19551 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 7451 },
	{ 0x8e6a, 0x8e6b, PDF_CMAP_RANGE, 19553 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 1446 },
	{ 0x8e6d, 0x8e6d, PDF_CMAP_SINGLE, 1195 },
	{ 0x8e6e, 0x8e6e, PDF_CMAP_SINGLE, 19555 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 7496 },
	{ 0x8e70, 0x8e70, PDF_CMAP_SINGLE, 7493 },
	{ 0x8e71, 0x8e71, PDF_CMAP_SINGLE, 19556 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 1558 },
	{ 0x8e73, 0x8e73, PDF_CMAP_SINGLE, 19557 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 7497 },
	{ 0x8e75, 0x8e75, PDF_CMAP_SINGLE, 19558 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 7494 },
	{ 0x8e77, 0x8e79, PDF_CMAP_RANGE, 19559 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 9727 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 19562 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 7495 },
	{ 0x8e7d, 0x8e7e, PDF_CMAP_RANGE, 19563 },
	{ 0x8e7f, 0x8e7f, PDF_CMAP_SINGLE, 1373 },
	{ 0x8e80, 0x8e80, PDF_CMAP_SINGLE, 19565 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 4397 },
	{ 0x8e82, 0x8e84, PDF_CMAP_RANGE, 19566 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 7498 },
	{ 0x8e86, 0x8e86, PDF_CMAP_SINGLE, 19569 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 1310 },
	{ 0x8e88, 0x8e88, PDF_CMAP_SINGLE, 19570 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 9724 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 7820 },
	{ 0x8e8b, 0x8e8b, PDF_CMAP_SINGLE, 9730 },
	{ 0x8e8c, 0x8e8c, PDF_CMAP_SINGLE, 19571 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 8781 },
	{ 0x8e8e, 0x8e8e, PDF_CMAP_SINGLE, 19572 },
	{ 0x8e8f, 0x8e8f, PDF_CMAP_SINGLE, 7499 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 7501 },
	{ 0x8e91, 0x8e91, PDF_CMAP_SINGLE, 9732 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 9726 },
	{ 0x8e93, 0x8e93, PDF_CMAP_SINGLE, 9731 },
	{ 0x8e94, 0x8e94, PDF_CMAP_SINGLE, 7500 },
	{ 0x8e95, 0x8e99, PDF_CMAP_RANGE, 19573 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 9729 },
	{ 0x8e9b, 0x8e9b, PDF_CMAP_SINGLE, 19578 },
	{ 0x8e9c, 0x8e9c, PDF_CMAP_SINGLE, 7502 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 19579 },
	{ 0x8e9e, 0x8e9e, PDF_CMAP_SINGLE, 7503 },
	{ 0x8e9f, 0x8ea0, PDF_CMAP_RANGE, 19580 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 9733 },
	{ 0x8ea2, 0x8ea4, PDF_CMAP_RANGE, 19582 },
	{ 0x8ea5, 0x8ea5, PDF_CMAP_SINGLE, 7843 },
	{ 0x8ea6, 0x8ea6, PDF_CMAP_SINGLE, 9736 },
	{ 0x8ea7, 0x8ea9, PDF_CMAP_RANGE, 19585 },
	{ 0x8eaa, 0x8eaa, PDF_CMAP_SINGLE, 9735 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 3366 },
	{ 0x8eac, 0x8eac, PDF_CMAP_SINGLE, 1795 },
	{ 0x8ead, 0x8eae, PDF_CMAP_RANGE, 19588 },
	{ 0x8eaf, 0x8eaf, PDF_CMAP_SINGLE, 3192 },
	{ 0x8eb0, 0x8eb1, PDF_CMAP_RANGE, 19590 },
	{ 0x8eb2, 0x8eb2, PDF_CMAP_SINGLE, 1570 },
	{ 0x8eb3, 0x8eb9, PDF_CMAP_RANGE, 19592 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 3612 },
	{ 0x8ebb, 0x8ebf, PDF_CMAP_RANGE, 19599 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 8432 },
	{ 0x8ec1, 0x8ec9, PDF_CMAP_RANGE, 19604 },
	{ 0x8eca, 0x8eca, PDF_CMAP_SINGLE, 7803 },
	{ 0x8ecb, 0x8ecb, PDF_CMAP_SINGLE, 8807 },
	{ 0x8ecc, 0x8ecc, PDF_CMAP_SINGLE, 7990 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 8150 },
	{ 0x8ece, 0x8ece, PDF_CMAP_SINGLE, 6392 },
	{ 0x8ecf, 0x8ed1, PDF_CMAP_RANGE, 19613 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 8672 },
	{ 0x8ed3, 0x8ed3, PDF_CMAP_SINGLE, 19616 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 9356 },
	{ 0x8ed5, 0x8eda, PDF_CMAP_RANGE, 19617 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 9357 },
	{ 0x8edc, 0x8ede, PDF_CMAP_RANGE, 19623 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 8450 },
	{ 0x8ee0, 0x8ee3, PDF_CMAP_RANGE, 19626 },
	{ 0x8ee4, 0x8ee4, PDF_CMAP_SINGLE, 9364 },
	{ 0x8ee5, 0x8eea, PDF_CMAP_RANGE, 19630 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 9363 },
	{ 0x8eec, 0x8ef1, PDF_CMAP_RANGE, 19636 },
	{ 0x8ef2, 0x8ef2, PDF_CMAP_SINGLE, 9358 },
	{ 0x8ef3, 0x8ef7, PDF_CMAP_RANGE, 19642 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 8857 },
	{ 0x8ef9, 0x8ef9, PDF_CMAP_SINGLE, 9361 },
	{ 0x8efa, 0x8efa, PDF_CMAP_SINGLE, 9366 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 9359 },
	{ 0x8efc, 0x8efc, PDF_CMAP_SINGLE, 9362 },
	{ 0x8efd, 0x8efd, PDF_CMAP_SINGLE, 19647 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 9367 },
	{ 0x8eff, 0x8eff, PDF_CMAP_SINGLE, 19648 },
	{ 0x8f00, 0x8f02, PDF_CMAP_RANGE, 19649 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 8113 },
	{ 0x8f04, 0x8f04, PDF_CMAP_SINGLE, 19652 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 9370 },
	{ 0x8f06, 0x8f06, PDF_CMAP_SINGLE, 19653 },
	{ 0x8f07, 0x8f07, PDF_CMAP_SINGLE, 9369 },
	{ 0x8f08, 0x8f08, PDF_CMAP_SINGLE, 19654 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 8792 },
	{ 0x8f0a, 0x8f0a, PDF_CMAP_SINGLE, 9368 },
	{ 0x8f0b, 0x8f11, PDF_CMAP_RANGE, 19655 },
	{ 0x8f12, 0x8f12, PDF_CMAP_SINGLE, 9371 },
	{ 0x8f13, 0x8f13, PDF_CMAP_SINGLE, 19662 },
	{ 0x8f14, 0x8f14, PDF_CMAP_SINGLE, 7945 },
	{ 0x8f15, 0x8f15, PDF_CMAP_SINGLE, 8422 },
	{ 0x8f16, 0x8f1a, PDF_CMAP_RANGE, 19663 },
	{ 0x8f1b, 0x8f1b, PDF_CMAP_SINGLE, 8230 },
	{ 0x8f1c, 0x8f1c, PDF_CMAP_SINGLE, 9375 },
	{ 0x8f1d, 0x8f1d, PDF_CMAP_SINGLE, 8025 },
	{ 0x8f1e, 0x8f1f, PDF_CMAP_RANGE, 9373 },
	{ 0x8f20, 0x8f24, PDF_CMAP_RANGE, 19668 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 7995 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 9372 },
	{ 0x8f27, 0x8f28, PDF_CMAP_RANGE, 19673 },
	{ 0x8f29, 0x8f29, PDF_CMAP_SINGLE, 7738 },
	{ 0x8f2a, 0x8f2a, PDF_CMAP_SINGLE, 8283 },
	{ 0x8f2b, 0x8f2e, PDF_CMAP_RANGE, 19675 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 8048 },
	{ 0x8f30, 0x8f32, PDF_CMAP_RANGE, 19679 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 9376 },
	{ 0x8f34, 0x8f37, PDF_CMAP_RANGE, 19682 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 8507 },
	{ 0x8f39, 0x8f3a, PDF_CMAP_RANGE, 19686 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 7943 },
	{ 0x8f3c, 0x8f3d, PDF_CMAP_RANGE, 19688 },
	{ 0x8f3e, 0x8f3e, PDF_CMAP_SINGLE, 8816 },
	{ 0x8f3f, 0x8f3f, PDF_CMAP_SINGLE, 8759 },
	{ 0x8f40, 0x8f41, PDF_CMAP_RANGE, 19690 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 9426 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 19692 },
	{ 0x8f44, 0x8f44, PDF_CMAP_SINGLE, 8627 },
	{ 0x8f45, 0x8f45, PDF_CMAP_SINGLE, 8773 },
	{ 0x8f46, 0x8f46, PDF_CMAP_SINGLE, 9377 },
	{ 0x8f47, 0x8f48, PDF_CMAP_RANGE, 19693 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 8872 },
	{ 0x8f4a, 0x8f4c, PDF_CMAP_RANGE, 19695 },
	{ 0x8f4d, 0x8f4d, PDF_CMAP_SINGLE, 8828 },
	{ 0x8f4e, 0x8f4e, PDF_CMAP_SINGLE, 8112 },
	{ 0x8f4f, 0x8f53, PDF_CMAP_RANGE, 19698 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 9378 },
	{ 0x8f55, 0x8f5e, PDF_CMAP_RANGE, 19703 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 8006 },
	{ 0x8f60, 0x8f60, PDF_CMAP_SINGLE, 19713 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 9088 },
	{ 0x8f62, 0x8f62, PDF_CMAP_SINGLE, 9365 },
	{ 0x8f63, 0x8f63, PDF_CMAP_SINGLE, 19714 },
	{ 0x8f64, 0x8f64, PDF_CMAP_SINGLE, 9360 },
	{ 0x8f65, 0x8f65, PDF_CMAP_SINGLE, 19715 },
	{ 0x8f66, 0x8f66, PDF_CMAP_SINGLE, 1242 },
	{ 0x8f67, 0x8f67, PDF_CMAP_SINGLE, 4417 },
	{ 0x8f68, 0x8f68, PDF_CMAP_SINGLE, 1861 },
	{ 0x8f69, 0x8f69, PDF_CMAP_SINGLE, 4036 },
	{ 0x8f6a, 0x8f6a, PDF_CMAP_SINGLE, 19716 },
	{ 0x8f6b, 0x8f6b, PDF_CMAP_SINGLE, 6369 },
	{ 0x8f6c, 0x8f6c, PDF_CMAP_SINGLE, 4615 },
	{ 0x8f6d, 0x8f6d, PDF_CMAP_SINGLE, 6370 },
	{ 0x8f6e, 0x8f6e, PDF_CMAP_SINGLE, 2685 },
	{ 0x8f6f, 0x8f6f, PDF_CMAP_SINGLE, 3272 },
	{ 0x8f70, 0x8f70, PDF_CMAP_SINGLE, 1947 },
	{ 0x8f71, 0x8f73, PDF_CMAP_RANGE, 6371 },
	{ 0x8f74, 0x8f74, PDF_CMAP_SINGLE, 4576 },
	{ 0x8f75, 0x8f76, PDF_CMAP_RANGE, 6374 },
	{ 0x8f77, 0x8f77, PDF_CMAP_SINGLE, 6377 },
	{ 0x8f78, 0x8f78, PDF_CMAP_SINGLE, 6376 },
	{ 0x8f79, 0x8f7a, PDF_CMAP_RANGE, 6378 },
	{ 0x8f7b, 0x8f7b, PDF_CMAP_SINGLE, 3166 },
	{ 0x8f7c, 0x8f7c, PDF_CMAP_SINGLE, 6380 },
	{ 0x8f7d, 0x8f7d, PDF_CMAP_SINGLE, 4379 },
	{ 0x8f7e, 0x8f7e, PDF_CMAP_SINGLE, 6381 },
	{ 0x8f7f, 0x8f7f, PDF_CMAP_SINGLE, 2207 },
	{ 0x8f80, 0x8f80, PDF_CMAP_SINGLE, 19717 },
	{ 0x8f81, 0x8f82, PDF_CMAP_RANGE, 6382 },
	{ 0x8f83, 0x8f83, PDF_CMAP_SINGLE, 2208 },
	{ 0x8f84, 0x8f84, PDF_CMAP_SINGLE, 6384 },
	{ 0x8f85, 0x8f85, PDF_CMAP_SINGLE, 1699 },
	{ 0x8f86, 0x8f86, PDF_CMAP_SINGLE, 2567 },
	{ 0x8f87, 0x8f87, PDF_CMAP_SINGLE, 6385 },
	{ 0x8f88, 0x8f88, PDF_CMAP_SINGLE, 1051 },
	{ 0x8f89, 0x8f89, PDF_CMAP_SINGLE, 2025 },
	{ 0x8f8a, 0x8f8a, PDF_CMAP_SINGLE, 1870 },
	{ 0x8f8b, 0x8f8b, PDF_CMAP_SINGLE, 6386 },
	{ 0x8f8c, 0x8f8c, PDF_CMAP_SINGLE, 19718 },
	{ 0x8f8d, 0x8f8f, PDF_CMAP_RANGE, 6387 },
	{ 0x8f90, 0x8f90, PDF_CMAP_SINGLE, 1685 },
	{ 0x8f91, 0x8f91, PDF_CMAP_SINGLE, 2080 },
	{ 0x8f92, 0x8f92, PDF_CMAP_SINGLE, 19719 },
	{ 0x8f93, 0x8f93, PDF_CMAP_SINGLE, 3451 },
	{ 0x8f94, 0x8f94, PDF_CMAP_SINGLE, 5492 },
	{ 0x8f95, 0x8f95, PDF_CMAP_SINGLE, 4338 },
	{ 0x8f96, 0x8f96, PDF_CMAP_SINGLE, 3882 },
	{ 0x8f97, 0x8f97, PDF_CMAP_SINGLE, 4440 },
	{ 0x8f98, 0x8f98, PDF_CMAP_SINGLE, 6390 },
	{ 0x8f99, 0x8f99, PDF_CMAP_SINGLE, 4479 },
	{ 0x8f9a, 0x8f9a, PDF_CMAP_SINGLE, 6391 },
	{ 0x8f9b, 0x8f9b, PDF_CMAP_SINGLE, 3980 },
	{ 0x8f9c, 0x8f9c, PDF_CMAP_SINGLE, 1813 },
	{ 0x8f9d, 0x8f9d, PDF_CMAP_SINGLE, 19720 },
	{ 0x8f9e, 0x8f9e, PDF_CMAP_SINGLE, 1354 },
	{ 0x8f9f, 0x8f9f, PDF_CMAP_SINGLE, 1090 },
	{ 0x8fa0, 0x8fa2, PDF_CMAP_RANGE, 19721 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 2460 },
	{ 0x8fa4, 0x8fa5, PDF_CMAP_RANGE, 19724 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 7728 },
	{ 0x8fa7, 0x8fa7, PDF_CMAP_SINGLE, 19726 },
	{ 0x8fa8, 0x8fa9, PDF_CMAP_RANGE, 1103 },
	{ 0x8faa, 0x8faa, PDF_CMAP_SINGLE, 19727 },
	{ 0x8fab, 0x8fab, PDF_CMAP_SINGLE, 1105 },
	{ 0x8fac, 0x8fac, PDF_CMAP_SINGLE, 19728 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 7837 },
	{ 0x8fae, 0x8fae, PDF_CMAP_SINGLE, 7755 },
	{ 0x8faf, 0x8faf, PDF_CMAP_SINGLE, 7754 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 1250 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 3267 },
	{ 0x8fb2, 0x8fb2, PDF_CMAP_SINGLE, 8362 },
	{ 0x8fb3, 0x8fb5, PDF_CMAP_RANGE, 19729 },
	{ 0x8fb6, 0x8fb6, PDF_CMAP_SINGLE, 5949 },
	{ 0x8fb7, 0x8fb8, PDF_CMAP_RANGE, 19732 },
	{ 0x8fb9, 0x8fb9, PDF_CMAP_SINGLE, 1096 },
	{ 0x8fba, 0x8fbc, PDF_CMAP_RANGE, 19734 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 2578 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 1394 },
	{ 0x8fbf, 0x8fc0, PDF_CMAP_RANGE, 19737 },
	{ 0x8fc1, 0x8fc1, PDF_CMAP_SINGLE, 3109 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 4285 },
	{ 0x8fc3, 0x8fc3, PDF_CMAP_SINGLE, 19739 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 3096 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 4065 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 19740 },
	{ 0x8fc7, 0x8fc7, PDF_CMAP_SINGLE, 1878 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 2716 },
	{ 0x8fc9, 0x8fcd, PDF_CMAP_RANGE, 19741 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 4242 },
	{ 0x8fcf, 0x8fcf, PDF_CMAP_SINGLE, 19746 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 4366 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 2253 },
	{ 0x8fd2, 0x8fd2, PDF_CMAP_SINGLE, 19747 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 5950 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 1618 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 5951 },
	{ 0x8fd6, 0x8fd7, PDF_CMAP_RANGE, 19748 },
	{ 0x8fd8, 0x8fd8, PDF_CMAP_SINGLE, 1998 },
	{ 0x8fd9, 0x8fd9, PDF_CMAP_SINGLE, 4483 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 19750 },
	{ 0x8fdb, 0x8fdb, PDF_CMAP_SINGLE, 2249 },
	{ 0x8fdc, 0x8fdc, PDF_CMAP_SINGLE, 4345 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 3764 },
	{ 0x8fde, 0x8fde, PDF_CMAP_SINGLE, 2549 },
	{ 0x8fdf, 0x8fdf, PDF_CMAP_SINGLE, 1278 },
	{ 0x8fe0, 0x8fe1, PDF_CMAP_RANGE, 19751 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 3657 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 19753 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 5954 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 5952 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 5956 },
	{ 0x8fe7, 0x8fe7, PDF_CMAP_SINGLE, 19754 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 5958 },
	{ 0x8fe9, 0x8fe9, PDF_CMAP_SINGLE, 5955 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 1456 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 3047 },
	{ 0x8fec, 0x8fec, PDF_CMAP_SINGLE, 19755 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 1501 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 5953 },
	{ 0x8fef, 0x8fef, PDF_CMAP_SINGLE, 19756 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 3469 },
	{ 0x8ff1, 0x8ff2, PDF_CMAP_RANGE, 19757 },
	{ 0x8ff3, 0x8ff3, PDF_CMAP_SINGLE, 5957 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 9851 },
	{ 0x8ff5, 0x8ff6, PDF_CMAP_RANGE, 19759 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 2777 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 1070 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 2070 },
	{ 0x8ffa, 0x8ffc, PDF_CMAP_RANGE, 19761 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 4628 },
	{ 0x8ffe, 0x8fff, PDF_CMAP_RANGE, 19764 },
	{ 0x9000, 0x9000, PDF_CMAP_SINGLE, 3708 },
	{ 0x9001, 0x9001, PDF_CMAP_SINGLE, 3522 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 3425 },
	{ 0x9003, 0x9003, PDF_CMAP_SINGLE, 3622 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 5960 },
	{ 0x9005, 0x9005, PDF_CMAP_SINGLE, 5959 },
	{ 0x9006, 0x9006, PDF_CMAP_SINGLE, 2889 },
	{ 0x9007, 0x9008, PDF_CMAP_RANGE, 19766 },
	{ 0x9009, 0x9009, PDF_CMAP_SINGLE, 4042 },
	{ 0x900a, 0x900a, PDF_CMAP_SINGLE, 4064 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 5961 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 19768 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 5964 },
	{ 0x900e, 0x900e, PDF_CMAP_SINGLE, 19769 },
	{ 0x900f, 0x900f, PDF_CMAP_SINGLE, 3689 },
	{ 0x9010, 0x9010, PDF_CMAP_SINGLE, 4591 },
	{ 0x9011, 0x9011, PDF_CMAP_SINGLE, 5963 },
	{ 0x9012, 0x9012, PDF_CMAP_SINGLE, 1470 },
	{ 0x9013, 0x9013, PDF_CMAP_SINGLE, 19770 },
	{ 0x9014, 0x9014, PDF_CMAP_SINGLE, 3695 },
	{ 0x9015, 0x9015, PDF_CMAP_SINGLE, 9202 },
	{ 0x9016, 0x9016, PDF_CMAP_SINGLE, 5965 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 1529 },
	{ 0x9018, 0x9018, PDF_CMAP_SINGLE, 19771 },
	{ 0x9019, 0x9019, PDF_CMAP_SINGLE, 8830 },
	{ 0x901a, 0x901a, PDF_CMAP_SINGLE, 3673 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 1853 },
	{ 0x901c, 0x901c, PDF_CMAP_SINGLE, 19772 },
	{ 0x901d, 0x901d, PDF_CMAP_SINGLE, 3420 },
	{ 0x901e, 0x901e, PDF_CMAP_SINGLE, 1270 },
	{ 0x901f, 0x901f, PDF_CMAP_SINGLE, 3534 },
	{ 0x9020, 0x9020, PDF_CMAP_SINGLE, 4399 },
	{ 0x9021, 0x9021, PDF_CMAP_SINGLE, 5966 },
	{ 0x9022, 0x9022, PDF_CMAP_SINGLE, 1671 },
	{ 0x9023, 0x9023, PDF_CMAP_SINGLE, 8217 },
	{ 0x9024, 0x9025, PDF_CMAP_RANGE, 19773 },
	{ 0x9026, 0x9026, PDF_CMAP_SINGLE, 5962 },
	{ 0x9027, 0x902c, PDF_CMAP_RANGE, 19775 },
	{ 0x902d, 0x902d, PDF_CMAP_SINGLE, 5969 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 1409 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 5970 },
	{ 0x9030, 0x9031, PDF_CMAP_RANGE, 19781 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 8123 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 19783 },
	{ 0x9035, 0x9036, PDF_CMAP_RANGE, 5967 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 19785 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 4196 },
	{ 0x9039, 0x903a, PDF_CMAP_RANGE, 19786 },
	{ 0x903b, 0x903b, PDF_CMAP_SINGLE, 2694 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 1071 },
	{ 0x903d, 0x903d, PDF_CMAP_SINGLE, 19788 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 4295 },
	{ 0x903f, 0x9040, PDF_CMAP_RANGE, 19789 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 1564 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 3553 },
	{ 0x9043, 0x9043, PDF_CMAP_SINGLE, 19791 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 5971 },
	{ 0x9045, 0x9046, PDF_CMAP_RANGE, 19792 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 4315 },
	{ 0x9048, 0x904a, PDF_CMAP_RANGE, 19794 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 8787 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 19797 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 1106 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 7998 },
	{ 0x904f, 0x904f, PDF_CMAP_SINGLE, 1587 },
	{ 0x9050, 0x9050, PDF_CMAP_SINGLE, 5974 },
	{ 0x9051, 0x9052, PDF_CMAP_RANGE, 5972 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 1441 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 7846 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 8590 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 19798 },
	{ 0x9057, 0x9057, PDF_CMAP_SINGLE, 4172 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 5976 },
	{ 0x9059, 0x905a, PDF_CMAP_RANGE, 19799 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 5978 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 8684 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 19801 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 7870 },
	{ 0x905f, 0x905f, PDF_CMAP_SINGLE, 19802 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 8778 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 19803 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 5977 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 3119 },
	{ 0x9064, 0x9064, PDF_CMAP_SINGLE, 19804 },
	{ 0x9065, 0x9065, PDF_CMAP_SINGLE, 4138 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 19805 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 5975 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 8499 },
	{ 0x906a, 0x906c, PDF_CMAP_RANGE, 19807 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 4389 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 4475 },
	{ 0x906f, 0x9071, PDF_CMAP_RANGE, 19810 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 7812 },
	{ 0x9073, 0x9073, PDF_CMAP_SINGLE, 19813 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 5980 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 4686 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 19814 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 8399 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 8674 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 19815 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 8719 },
	{ 0x907b, 0x907b, PDF_CMAP_SINGLE, 19816 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 8233 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 5981 },
	{ 0x907e, 0x907e, PDF_CMAP_SINGLE, 19817 },
	{ 0x907f, 0x907f, PDF_CMAP_SINGLE, 1093 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 4132 },
	{ 0x9081, 0x9081, PDF_CMAP_SINGLE, 8307 },
	{ 0x9082, 0x9082, PDF_CMAP_SINGLE, 5982 },
	{ 0x9083, 0x9083, PDF_CMAP_SINGLE, 5984 },
	{ 0x9084, 0x9084, PDF_CMAP_SINGLE, 8021 },
	{ 0x9085, 0x9086, PDF_CMAP_RANGE, 19818 },
	{ 0x9087, 0x9087, PDF_CMAP_SINGLE, 9201 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 5983 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 19820 },
	{ 0x908a, 0x908a, PDF_CMAP_SINGLE, 7750 },
	{ 0x908b, 0x908b, PDF_CMAP_SINGLE, 5985 },
	{ 0x908c, 0x908e, PDF_CMAP_RANGE, 19821 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 8291 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 9203 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 4191 },
	{ 0x9092, 0x9092, PDF_CMAP_SINGLE, 19824 },
	{ 0x9093, 0x9093, PDF_CMAP_SINGLE, 1452 },
	{ 0x9094, 0x9094, PDF_CMAP_SINGLE, 19825 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 6167 },
	{ 0x9096, 0x9096, PDF_CMAP_SINGLE, 19826 },
	{ 0x9097, 0x9097, PDF_CMAP_SINGLE, 4967 },
	{ 0x9098, 0x9098, PDF_CMAP_SINGLE, 19827 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 4970 },
	{ 0x909a, 0x909a, PDF_CMAP_SINGLE, 19828 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 4968 },
	{ 0x909c, 0x909c, PDF_CMAP_SINGLE, 19829 },
	{ 0x909d, 0x909d, PDF_CMAP_SINGLE, 4969 },
	{ 0x909e, 0x90a0, PDF_CMAP_RANGE, 19830 },
	{ 0x90a1, 0x90a1, PDF_CMAP_SINGLE, 4972 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 3994 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 2858 },
	{ 0x90a4, 0x90a5, PDF_CMAP_RANGE, 19833 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 1017 },
	{ 0x90a7, 0x90a9, PDF_CMAP_RANGE, 19835 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 3963 },
	{ 0x90ab, 0x90ab, PDF_CMAP_SINGLE, 19838 },
	{ 0x90ac, 0x90ac, PDF_CMAP_SINGLE, 4971 },
	{ 0x90ad, 0x90ad, PDF_CMAP_SINGLE, 19839 },
	{ 0x90ae, 0x90ae, PDF_CMAP_SINGLE, 4271 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 1889 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 4978 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 3182 },
	{ 0x90b2, 0x90b2, PDF_CMAP_SINGLE, 19840 },
	{ 0x90b3, 0x90b3, PDF_CMAP_SINGLE, 4974 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 4973 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 3348 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 4975 },
	{ 0x90b7, 0x90b7, PDF_CMAP_SINGLE, 19841 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 4977 },
	{ 0x90b9, 0x90b9, PDF_CMAP_SINGLE, 4667 },
	{ 0x90ba, 0x90ba, PDF_CMAP_SINGLE, 4976 },
	{ 0x90bb, 0x90bb, PDF_CMAP_SINGLE, 2595 },
	{ 0x90bc, 0x90bd, PDF_CMAP_RANGE, 19842 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 4981 },
	{ 0x90bf, 0x90c0, PDF_CMAP_RANGE, 19844 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 4313 },
	{ 0x90c2, 0x90c3, PDF_CMAP_RANGE, 19846 },
	{ 0x90c4, 0x90c4, PDF_CMAP_SINGLE, 4983 },
	{ 0x90c5, 0x90c5, PDF_CMAP_SINGLE, 4980 },
	{ 0x90c6, 0x90c6, PDF_CMAP_SINGLE, 19848 },
	{ 0x90c7, 0x90c7, PDF_CMAP_SINGLE, 4984 },
	{ 0x90c8, 0x90c9, PDF_CMAP_RANGE, 19849 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2189 },
	{ 0x90cb, 0x90cd, PDF_CMAP_RANGE, 19851 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 2484 },
	{ 0x90cf, 0x90cf, PDF_CMAP_SINGLE, 4979 },
	{ 0x90d0, 0x90d0, PDF_CMAP_SINGLE, 4982 },
	{ 0x90d1, 0x90d1, PDF_CMAP_SINGLE, 4514 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 19854 },
	{ 0x90d3, 0x90d3, PDF_CMAP_SINGLE, 4985 },
	{ 0x90d4, 0x90d6, PDF_CMAP_RANGE, 19855 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 4989 },
	{ 0x90d8, 0x90da, PDF_CMAP_RANGE, 19858 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 4990 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 4988 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 1913 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 19861 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 8990 },
	{ 0x90e0, 0x90e0, PDF_CMAP_SINGLE, 19862 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 2353 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 4987 },
	{ 0x90e3, 0x90e5, PDF_CMAP_RANGE, 19863 },
	{ 0x90e6, 0x90e6, PDF_CMAP_SINGLE, 4986 },
	{ 0x90e7, 0x90e7, PDF_CMAP_SINGLE, 4362 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 1158 },
	{ 0x90e9, 0x90ea, PDF_CMAP_RANGE, 19866 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 4991 },
	{ 0x90ec, 0x90ec, PDF_CMAP_SINGLE, 19868 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 1874 },
	{ 0x90ee, 0x90ee, PDF_CMAP_SINGLE, 19869 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 4992 },
	{ 0x90f0, 0x90f3, PDF_CMAP_RANGE, 19870 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 1248 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 8755 },
	{ 0x90f6, 0x90f7, PDF_CMAP_RANGE, 19874 },
	{ 0x90f8, 0x90f8, PDF_CMAP_SINGLE, 1415 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 19876 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 1531 },
	{ 0x90fe, 0x90fe, PDF_CMAP_SINGLE, 4993 },
	{ 0x90ff, 0x90ff, PDF_CMAP_SINGLE, 19880 },
	{ 0x9100, 0x9101, PDF_CMAP_RANGE, 19881 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 1588 },
	{ 0x9103, 0x9103, PDF_CMAP_SINGLE, 19883 },
	{ 0x9104, 0x9104, PDF_CMAP_SINGLE, 4994 },
	{ 0x9105, 0x9105, PDF_CMAP_SINGLE, 19884 },
	{ 0x9106, 0x9106, PDF_CMAP_SINGLE, 8992 },
	{ 0x9107, 0x9108, PDF_CMAP_RANGE, 19885 },
	{ 0x9109, 0x9109, PDF_CMAP_SINGLE, 8647 },
	{ 0x910a, 0x9111, PDF_CMAP_RANGE, 19887 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 8892 },
	{ 0x9113, 0x9113, PDF_CMAP_SINGLE, 19895 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 8988 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 19896 },
	{ 0x9116, 0x9116, PDF_CMAP_SINGLE, 8785 },
	{ 0x9117, 0x9118, PDF_CMAP_RANGE, 19897 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 1074 },
	{ 0x911a, 0x911d, PDF_CMAP_RANGE, 19899 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 4996 },
	{ 0x911f, 0x9121, PDF_CMAP_RANGE, 19903 },
	{ 0x9122, 0x9122, PDF_CMAP_SINGLE, 4995 },
	{ 0x9123, 0x9123, PDF_CMAP_SINGLE, 4997 },
	{ 0x9124, 0x9126, PDF_CMAP_RANGE, 19906 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 7867 },
	{ 0x9128, 0x912c, PDF_CMAP_RANGE, 19909 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 8839 },
	{ 0x912e, 0x912e, PDF_CMAP_SINGLE, 19914 },
	{ 0x912f, 0x912f, PDF_CMAP_SINGLE, 4999 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 8237 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 4998 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 7851 },
	{ 0x9133, 0x9133, PDF_CMAP_SINGLE, 19915 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 8989 },
	{ 0x9135, 0x9135, PDF_CMAP_SINGLE, 19916 },
	{ 0x9136, 0x9136, PDF_CMAP_SINGLE, 8991 },
	{ 0x9137, 0x9138, PDF_CMAP_RANGE, 19917 },
	{ 0x9139, 0x9139, PDF_CMAP_SINGLE, 5000 },
	{ 0x913a, 0x913a, PDF_CMAP_SINGLE, 8987 },
	{ 0x913b, 0x9142, PDF_CMAP_RANGE, 19919 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 5001 },
	{ 0x9144, 0x9145, PDF_CMAP_RANGE, 19927 },
	{ 0x9146, 0x9146, PDF_CMAP_SINGLE, 5002 },
	{ 0x9147, 0x9147, PDF_CMAP_SINGLE, 19929 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 8993 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 4276 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 7416 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 3186 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 4640 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 2981 },
	{ 0x914e, 0x914f, PDF_CMAP_RANGE, 7418 },
	{ 0x9150, 0x9150, PDF_CMAP_SINGLE, 7417 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 19930 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2293 },
	{ 0x9153, 0x9156, PDF_CMAP_RANGE, 19931 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 4026 },
	{ 0x9158, 0x9159, PDF_CMAP_RANGE, 19935 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 1648 },
	{ 0x915b, 0x915c, PDF_CMAP_RANGE, 19937 },
	{ 0x915d, 0x915d, PDF_CMAP_SINGLE, 4368 },
	{ 0x915e, 0x915e, PDF_CMAP_SINGLE, 3581 },
	{ 0x915f, 0x9160, PDF_CMAP_RANGE, 19939 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 7422 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 7421 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 1887 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 7420 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 3531 },
	{ 0x9166, 0x9168, PDF_CMAP_RANGE, 19941 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 7424 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 2493 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 19944 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 1295 },
	{ 0x916d, 0x916d, PDF_CMAP_SINGLE, 19945 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 3675 },
	{ 0x916f, 0x916f, PDF_CMAP_SINGLE, 7425 },
	{ 0x9170, 0x9170, PDF_CMAP_SINGLE, 7423 },
	{ 0x9171, 0x9171, PDF_CMAP_SINGLE, 2181 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 7428 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 19946 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 7429 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 2206 },
	{ 0x9176, 0x9176, PDF_CMAP_SINGLE, 2749 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 2414 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 3542 },
	{ 0x9179, 0x9179, PDF_CMAP_SINGLE, 7430 },
	{ 0x917a, 0x917c, PDF_CMAP_RANGE, 19947 },
	{ 0x917d, 0x917e, PDF_CMAP_RANGE, 7426 },
	{ 0x917f, 0x917f, PDF_CMAP_SINGLE, 2899 },
	{ 0x9180, 0x9184, PDF_CMAP_RANGE, 19950 },
	{ 0x9185, 0x9185, PDF_CMAP_SINGLE, 7432 },
	{ 0x9186, 0x9186, PDF_CMAP_SINGLE, 19955 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 1343 },
	{ 0x9188, 0x9188, PDF_CMAP_SINGLE, 19956 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 4682 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 19957 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 1370 },
	{ 0x918c, 0x918c, PDF_CMAP_SINGLE, 7431 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 7434 },
	{ 0x918e, 0x918f, PDF_CMAP_RANGE, 19958 },
	{ 0x9190, 0x9190, PDF_CMAP_SINGLE, 7433 },
	{ 0x9191, 0x9191, PDF_CMAP_SINGLE, 7435 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 3996 },
	{ 0x9193, 0x9195, PDF_CMAP_RANGE, 19960 },
	{ 0x9196, 0x9196, PDF_CMAP_SINGLE, 8789 },
	{ 0x9197, 0x9199, PDF_CMAP_RANGE, 19963 },
	{ 0x919a, 0x919a, PDF_CMAP_SINGLE, 2774 },
	{ 0x919b, 0x919b, PDF_CMAP_SINGLE, 3204 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 7823 },
	{ 0x919d, 0x91a1, PDF_CMAP_RANGE, 19966 },
	{ 0x91a2, 0x91a3, PDF_CMAP_RANGE, 7436 },
	{ 0x91a4, 0x91a9, PDF_CMAP_RANGE, 19971 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 7438 },
	{ 0x91ab, 0x91ab, PDF_CMAP_SINGLE, 8716 },
	{ 0x91ac, 0x91ac, PDF_CMAP_SINGLE, 8100 },
	{ 0x91ad, 0x91af, PDF_CMAP_RANGE, 7439 },
	{ 0x91b0, 0x91b3, PDF_CMAP_RANGE, 19977 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 7443 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 7442 },
	{ 0x91b6, 0x91b9, PDF_CMAP_RANGE, 19981 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 7444 },
	{ 0x91bb, 0x91bf, PDF_CMAP_RANGE, 19985 },
	{ 0x91c0, 0x91c0, PDF_CMAP_SINGLE, 8347 },
	{ 0x91c1, 0x91c1, PDF_CMAP_SINGLE, 8664 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 19990 },
	{ 0x91c3, 0x91c3, PDF_CMAP_SINGLE, 9722 },
	{ 0x91c4, 0x91c4, PDF_CMAP_SINGLE, 19991 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 9721 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 19992 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 1168 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 19993 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 4281 },
	{ 0x91ca, 0x91ca, PDF_CMAP_SINGLE, 3428 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 8500 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 2522 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 4567 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 4151 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 2568 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 19994 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 2241 },
	{ 0x91d2, 0x91d4, PDF_CMAP_RANGE, 9459 },
	{ 0x91d5, 0x91d5, PDF_CMAP_SINGLE, 9464 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 19995 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 9463 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 7880 },
	{ 0x91d9, 0x91d9, PDF_CMAP_SINGLE, 9462 },
	{ 0x91da, 0x91db, PDF_CMAP_RANGE, 19996 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 1701 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 8832 },
	{ 0x91de, 0x91e2, PDF_CMAP_RANGE, 19998 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 7877 },
	{ 0x91e4, 0x91e4, PDF_CMAP_SINGLE, 9467 },
	{ 0x91e5, 0x91e6, PDF_CMAP_RANGE, 20003 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 9466 },
	{ 0x91e8, 0x91e8, PDF_CMAP_SINGLE, 20005 },
	{ 0x91e9, 0x91e9, PDF_CMAP_SINGLE, 7917 },
	{ 0x91ea, 0x91f4, PDF_CMAP_RANGE, 20006 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 9469 },
	{ 0x91f6, 0x91f6, PDF_CMAP_SINGLE, 20017 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 9465 },
	{ 0x91f8, 0x91f8, PDF_CMAP_SINGLE, 20018 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 9470 },
	{ 0x91fa, 0x91fa, PDF_CMAP_SINGLE, 8397 },
	{ 0x91fb, 0x91ff, PDF_CMAP_RANGE, 20019 },
	{ 0x9200, 0x9200, PDF_CMAP_SINGLE, 9480 },
	{ 0x9201, 0x9201, PDF_CMAP_SINGLE, 9476 },
	{ 0x9202, 0x9203, PDF_CMAP_RANGE, 20024 },
	{ 0x9204, 0x9204, PDF_CMAP_SINGLE, 9478 },
	{ 0x9205, 0x9207, PDF_CMAP_RANGE, 20026 },
	{ 0x9208, 0x9208, PDF_CMAP_SINGLE, 9471 },
	{ 0x9209, 0x9209, PDF_CMAP_SINGLE, 8336 },
	{ 0x920a, 0x920c, PDF_CMAP_RANGE, 20029 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 7901 },
	{ 0x920e, 0x920e, PDF_CMAP_SINGLE, 7973 },
	{ 0x920f, 0x920f, PDF_CMAP_SINGLE, 20032 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 9475 },
	{ 0x9211, 0x9211, PDF_CMAP_SINGLE, 9474 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 20033 },
	{ 0x9214, 0x9214, PDF_CMAP_SINGLE, 7802 },
	{ 0x9215, 0x9215, PDF_CMAP_SINGLE, 8358 },
	{ 0x9216, 0x921d, PDF_CMAP_RANGE, 20035 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 8149 },
	{ 0x921f, 0x9222, PDF_CMAP_RANGE, 20043 },
	{ 0x9223, 0x9223, PDF_CMAP_SINGLE, 7953 },
	{ 0x9224, 0x9224, PDF_CMAP_SINGLE, 20047 },
	{ 0x9225, 0x9225, PDF_CMAP_SINGLE, 9479 },
	{ 0x9226, 0x9226, PDF_CMAP_SINGLE, 9472 },
	{ 0x9227, 0x9227, PDF_CMAP_SINGLE, 9477 },
	{ 0x9228, 0x922d, PDF_CMAP_RANGE, 20048 },
	{ 0x922e, 0x922e, PDF_CMAP_SINGLE, 9497 },
	{ 0x922f, 0x922f, PDF_CMAP_SINGLE, 20054 },
	{ 0x9230, 0x9230, PDF_CMAP_SINGLE, 9493 },
	{ 0x9231, 0x9232, PDF_CMAP_RANGE, 20055 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 9484 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 8241 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 20057 },
	{ 0x9237, 0x9237, PDF_CMAP_SINGLE, 9483 },
	{ 0x9238, 0x9238, PDF_CMAP_SINGLE, 9487 },
	{ 0x9239, 0x9239, PDF_CMAP_SINGLE, 9498 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 9481 },
	{ 0x923b, 0x923c, PDF_CMAP_RANGE, 20059 },
	{ 0x923d, 0x923d, PDF_CMAP_SINGLE, 9486 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 8756 },
	{ 0x923f, 0x923f, PDF_CMAP_SINGLE, 9491 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 8064 },
	{ 0x9241, 0x9244, PDF_CMAP_RANGE, 20061 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 9473 },
	{ 0x9246, 0x9247, PDF_CMAP_RANGE, 20065 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 9495 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 9494 },
	{ 0x924a, 0x924c, PDF_CMAP_RANGE, 20067 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 9496 },
	{ 0x924e, 0x9250, PDF_CMAP_RANGE, 20070 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 7766 },
	{ 0x9252, 0x9254, PDF_CMAP_RANGE, 20073 },
	{ 0x9255, 0x9255, PDF_CMAP_SINGLE, 9485 },
	{ 0x9256, 0x9256, PDF_CMAP_SINGLE, 20076 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 8403 },
	{ 0x9258, 0x9259, PDF_CMAP_RANGE, 20077 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 8314 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 8398 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 20079 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 9488 },
	{ 0x925f, 0x9261, PDF_CMAP_RANGE, 20081 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 7765 },
	{ 0x9263, 0x9265, PDF_CMAP_RANGE, 20084 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 9482 },
	{ 0x9267, 0x926b, PDF_CMAP_RANGE, 20087 },
	{ 0x926c, 0x926d, PDF_CMAP_RANGE, 9489 },
	{ 0x926e, 0x9273, PDF_CMAP_RANGE, 20092 },
	{ 0x9274, 0x9274, PDF_CMAP_SINGLE, 2155 },
	{ 0x9275, 0x9277, PDF_CMAP_RANGE, 20098 },
	{ 0x9278, 0x9278, PDF_CMAP_SINGLE, 8106 },
	{ 0x9279, 0x9279, PDF_CMAP_SINGLE, 20101 },
	{ 0x927a, 0x927a, PDF_CMAP_SINGLE, 9502 },
	{ 0x927b, 0x927b, PDF_CMAP_SINGLE, 7967 },
	{ 0x927c, 0x927e, PDF_CMAP_RANGE, 20102 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 9517 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 8733 },
	{ 0x9281, 0x9282, PDF_CMAP_RANGE, 20105 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 9522 },
	{ 0x9284, 0x9284, PDF_CMAP_SINGLE, 20107 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 8571 },
	{ 0x9286, 0x928d, PDF_CMAP_RANGE, 20108 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 7552 },
	{ 0x928f, 0x9290, PDF_CMAP_RANGE, 20116 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 8623 },
	{ 0x9292, 0x9292, PDF_CMAP_SINGLE, 20118 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 9516 },
	{ 0x9294, 0x9295, PDF_CMAP_RANGE, 20119 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 9512 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 20121 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 8332 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 20122 },
	{ 0x929a, 0x929a, PDF_CMAP_SINGLE, 9519 },
	{ 0x929b, 0x929b, PDF_CMAP_SINGLE, 20123 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 8636 },
	{ 0x929d, 0x929f, PDF_CMAP_RANGE, 20124 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 9501 },
	{ 0x92a1, 0x92a2, PDF_CMAP_RANGE, 20127 },
	{ 0x92a3, 0x92a3, PDF_CMAP_SINGLE, 9525 },
	{ 0x92a4, 0x92a4, PDF_CMAP_SINGLE, 20129 },
	{ 0x92a5, 0x92a5, PDF_CMAP_SINGLE, 8717 },
	{ 0x92a6, 0x92a6, PDF_CMAP_SINGLE, 9510 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 20130 },
	{ 0x92a8, 0x92a8, PDF_CMAP_SINGLE, 9524 },
	{ 0x92a9, 0x92a9, PDF_CMAP_SINGLE, 9514 },
	{ 0x92aa, 0x92aa, PDF_CMAP_SINGLE, 9503 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 9521 },
	{ 0x92ac, 0x92ac, PDF_CMAP_SINGLE, 9500 },
	{ 0x92ad, 0x92ad, PDF_CMAP_SINGLE, 20131 },
	{ 0x92ae, 0x92ae, PDF_CMAP_SINGLE, 7553 },
	{ 0x92af, 0x92b0, PDF_CMAP_RANGE, 20132 },
	{ 0x92b1, 0x92b1, PDF_CMAP_SINGLE, 9509 },
	{ 0x92b2, 0x92b6, PDF_CMAP_RANGE, 20134 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 8653 },
	{ 0x92b8, 0x92b8, PDF_CMAP_SINGLE, 20139 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 8666 },
	{ 0x92ba, 0x92ba, PDF_CMAP_SINGLE, 20140 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 8562 },
	{ 0x92bc, 0x92bc, PDF_CMAP_SINGLE, 9534 },
	{ 0x92bd, 0x92c0, PDF_CMAP_RANGE, 20141 },
	{ 0x92c1, 0x92c1, PDF_CMAP_SINGLE, 8271 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 20145 },
	{ 0x92c3, 0x92c3, PDF_CMAP_SINGLE, 9539 },
	{ 0x92c4, 0x92c4, PDF_CMAP_SINGLE, 20146 },
	{ 0x92c5, 0x92c5, PDF_CMAP_SINGLE, 8663 },
	{ 0x92c6, 0x92c6, PDF_CMAP_SINGLE, 20147 },
	{ 0x92c7, 0x92c7, PDF_CMAP_SINGLE, 7740 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 7554 },
	{ 0x92c9, 0x92cb, PDF_CMAP_RANGE, 20148 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 9513 },
	{ 0x92cd, 0x92ce, PDF_CMAP_RANGE, 20151 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 9505 },
	{ 0x92d0, 0x92d1, PDF_CMAP_RANGE, 20153 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 7935 },
	{ 0x92d3, 0x92dc, PDF_CMAP_RANGE, 20155 },
	{ 0x92dd, 0x92dd, PDF_CMAP_SINGLE, 9535 },
	{ 0x92de, 0x92de, PDF_CMAP_SINGLE, 20165 },
	{ 0x92df, 0x92df, PDF_CMAP_SINGLE, 9540 },
	{ 0x92e0, 0x92e2, PDF_CMAP_RANGE, 20166 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 9507 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 7824 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 9530 },
	{ 0x92e6, 0x92e6, PDF_CMAP_SINGLE, 9541 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 20169 },
	{ 0x92e8, 0x92e8, PDF_CMAP_SINGLE, 9533 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 20170 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 8385 },
	{ 0x92eb, 0x92ec, PDF_CMAP_RANGE, 20171 },
	{ 0x92ed, 0x92ed, PDF_CMAP_SINGLE, 8451 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 9504 },
	{ 0x92ef, 0x92ef, PDF_CMAP_SINGLE, 9532 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 9531 },
	{ 0x92f1, 0x92f1, PDF_CMAP_SINGLE, 9528 },
	{ 0x92f2, 0x92f5, PDF_CMAP_RANGE, 20173 },
	{ 0x92f6, 0x92f6, PDF_CMAP_SINGLE, 9536 },
	{ 0x92f7, 0x92f7, PDF_CMAP_SINGLE, 20177 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 8141 },
	{ 0x92f9, 0x92fb, PDF_CMAP_RANGE, 20178 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 7960 },
	{ 0x92fd, 0x92ff, PDF_CMAP_RANGE, 20181 },
	{ 0x9300, 0x9300, PDF_CMAP_SINGLE, 20184 },
	{ 0x9301, 0x9301, PDF_CMAP_SINGLE, 9547 },
	{ 0x9302, 0x9305, PDF_CMAP_RANGE, 20185 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 9543 },
	{ 0x9307, 0x9308, PDF_CMAP_RANGE, 9551 },
	{ 0x9309, 0x930f, PDF_CMAP_RANGE, 20189 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 8880 },
	{ 0x9311, 0x9311, PDF_CMAP_SINGLE, 20196 },
	{ 0x9312, 0x9312, PDF_CMAP_SINGLE, 9542 },
	{ 0x9313, 0x9314, PDF_CMAP_RANGE, 20197 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 9548 },
	{ 0x9316, 0x9317, PDF_CMAP_RANGE, 20199 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 7834 },
	{ 0x9319, 0x9319, PDF_CMAP_SINGLE, 9554 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 9520 },
	{ 0x931b, 0x931b, PDF_CMAP_SINGLE, 9545 },
	{ 0x931c, 0x931e, PDF_CMAP_RANGE, 20201 },
	{ 0x931f, 0x931f, PDF_CMAP_SINGLE, 9553 },
	{ 0x9320, 0x9320, PDF_CMAP_SINGLE, 7882 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 20204 },
	{ 0x9322, 0x9322, PDF_CMAP_SINGLE, 8402 },
	{ 0x9323, 0x9325, PDF_CMAP_RANGE, 20205 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 8120 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 20208 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 8313 },
	{ 0x9329, 0x932a, PDF_CMAP_RANGE, 20209 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 8619 },
	{ 0x932c, 0x932d, PDF_CMAP_RANGE, 20211 },
	{ 0x932e, 0x932e, PDF_CMAP_SINGLE, 9549 },
	{ 0x932f, 0x932f, PDF_CMAP_SINGLE, 7845 },
	{ 0x9330, 0x9331, PDF_CMAP_RANGE, 20213 },
	{ 0x9332, 0x9332, PDF_CMAP_SINGLE, 8268 },
	{ 0x9333, 0x9333, PDF_CMAP_SINGLE, 8320 },
	{ 0x9334, 0x9335, PDF_CMAP_RANGE, 20215 },
	{ 0x9336, 0x9336, PDF_CMAP_SINGLE, 9837 },
	{ 0x9337, 0x9337, PDF_CMAP_SINGLE, 20217 },
	{ 0x9338, 0x9338, PDF_CMAP_SINGLE, 9527 },
	{ 0x9339, 0x933d, PDF_CMAP_RANGE, 20218 },
	{ 0x933e, 0x933e, PDF_CMAP_SINGLE, 7555 },
	{ 0x933f, 0x933f, PDF_CMAP_SINGLE, 20223 },
	{ 0x9340, 0x9340, PDF_CMAP_SINGLE, 9546 },
	{ 0x9341, 0x9341, PDF_CMAP_SINGLE, 8632 },
	{ 0x9342, 0x9342, PDF_CMAP_SINGLE, 20224 },
	{ 0x9343, 0x9343, PDF_CMAP_SINGLE, 9550 },
	{ 0x9344, 0x9345, PDF_CMAP_RANGE, 20225 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 9468 },
	{ 0x9347, 0x9347, PDF_CMAP_SINGLE, 9556 },
	{ 0x9348, 0x934a, PDF_CMAP_RANGE, 20227 },
	{ 0x934b, 0x934b, PDF_CMAP_SINGLE, 7996 },
	{ 0x934c, 0x934c, PDF_CMAP_SINGLE, 20230 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 7893 },
	{ 0x934e, 0x9353, PDF_CMAP_RANGE, 20231 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 9558 },
	{ 0x9355, 0x9357, PDF_CMAP_RANGE, 20237 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 8808 },
	{ 0x9359, 0x935a, PDF_CMAP_RANGE, 20240 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 7894 },
	{ 0x935c, 0x9363, PDF_CMAP_RANGE, 20242 },
	{ 0x9364, 0x9364, PDF_CMAP_SINGLE, 9559 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 9555 },
	{ 0x9366, 0x9368, PDF_CMAP_RANGE, 20250 },
	{ 0x9369, 0x9369, PDF_CMAP_SINGLE, 9544 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 7556 },
	{ 0x936b, 0x936b, PDF_CMAP_SINGLE, 20253 },
	{ 0x936c, 0x936c, PDF_CMAP_SINGLE, 8412 },
	{ 0x936d, 0x936f, PDF_CMAP_RANGE, 20254 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 9561 },
	{ 0x9371, 0x9374, PDF_CMAP_RANGE, 20257 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 8087 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 9557 },
	{ 0x9377, 0x9379, PDF_CMAP_RANGE, 20261 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 8829 },
	{ 0x937b, 0x937d, PDF_CMAP_RANGE, 20264 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 9598 },
	{ 0x937f, 0x9381, PDF_CMAP_RANGE, 20267 },
	{ 0x9382, 0x9382, PDF_CMAP_SINGLE, 8316 },
	{ 0x9383, 0x9383, PDF_CMAP_SINGLE, 20270 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 9562 },
	{ 0x9385, 0x9386, PDF_CMAP_RANGE, 20271 },
	{ 0x9387, 0x9387, PDF_CMAP_SINGLE, 9566 },
	{ 0x9388, 0x9389, PDF_CMAP_RANGE, 20273 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 7732 },
	{ 0x938b, 0x938e, PDF_CMAP_RANGE, 20275 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 7558 },
	{ 0x9390, 0x9395, PDF_CMAP_RANGE, 20279 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 8542 },
	{ 0x9397, 0x9397, PDF_CMAP_SINGLE, 20285 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 9568 },
	{ 0x9399, 0x93a1, PDF_CMAP_RANGE, 20286 },
	{ 0x93a2, 0x93a2, PDF_CMAP_SINGLE, 8610 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 9044 },
	{ 0x93a4, 0x93a5, PDF_CMAP_RANGE, 20295 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 9571 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 9511 },
	{ 0x93a8, 0x93a8, PDF_CMAP_SINGLE, 20297 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 9518 },
	{ 0x93aa, 0x93aa, PDF_CMAP_SINGLE, 9560 },
	{ 0x93ab, 0x93ab, PDF_CMAP_SINGLE, 20298 },
	{ 0x93ac, 0x93ac, PDF_CMAP_SINGLE, 7963 },
	{ 0x93ad, 0x93ad, PDF_CMAP_SINGLE, 20299 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 8835 },
	{ 0x93af, 0x93af, PDF_CMAP_SINGLE, 20300 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 9572 },
	{ 0x93b1, 0x93b2, PDF_CMAP_RANGE, 20301 },
	{ 0x93b3, 0x93b3, PDF_CMAP_SINGLE, 8352 },
	{ 0x93b4, 0x93b4, PDF_CMAP_SINGLE, 20303 },
	{ 0x93b5, 0x93b5, PDF_CMAP_SINGLE, 9573 },
	{ 0x93b6, 0x93b7, PDF_CMAP_RANGE, 20304 },
	{ 0x93b8, 0x93b8, PDF_CMAP_SINGLE, 9569 },
	{ 0x93b9, 0x93be, PDF_CMAP_RANGE, 20306 },
	{ 0x93bf, 0x93bf, PDF_CMAP_SINGLE, 9570 },
	{ 0x93c0, 0x93c2, PDF_CMAP_RANGE, 20312 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 9580 },
	{ 0x93c4, 0x93c6, PDF_CMAP_RANGE, 20315 },
	{ 0x93c7, 0x93c7, PDF_CMAP_SINGLE, 9581 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 8224 },
	{ 0x93c9, 0x93c9, PDF_CMAP_SINGLE, 20318 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 7557 },
	{ 0x93cb, 0x93cb, PDF_CMAP_SINGLE, 20319 },
	{ 0x93cc, 0x93cc, PDF_CMAP_SINGLE, 9567 },
	{ 0x93cd, 0x93cd, PDF_CMAP_SINGLE, 9578 },
	{ 0x93ce, 0x93d0, PDF_CMAP_RANGE, 20320 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 9582 },
	{ 0x93d2, 0x93d5, PDF_CMAP_RANGE, 20323 },
	{ 0x93d6, 0x93d6, PDF_CMAP_SINGLE, 7681 },
	{ 0x93d7, 0x93d7, PDF_CMAP_SINGLE, 9529 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 9564 },
	{ 0x93d9, 0x93db, PDF_CMAP_RANGE, 20327 },
	{ 0x93dc, 0x93dd, PDF_CMAP_RANGE, 9576 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 9579 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 7791 },
	{ 0x93e0, 0x93e0, PDF_CMAP_SINGLE, 20330 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 8132 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 9575 },
	{ 0x93e3, 0x93e3, PDF_CMAP_SINGLE, 20331 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 9563 },
	{ 0x93e5, 0x93e7, PDF_CMAP_RANGE, 20332 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 9757 },
	{ 0x93e9, 0x93f4, PDF_CMAP_RANGE, 20335 },
	{ 0x93f5, 0x93f5, PDF_CMAP_SINGLE, 9515 },
	{ 0x93f6, 0x93f6, PDF_CMAP_SINGLE, 20347 },
	{ 0x93f7, 0x93f7, PDF_CMAP_SINGLE, 9585 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 20348 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 9591 },
	{ 0x93fa, 0x93ff, PDF_CMAP_RANGE, 20349 },
	{ 0x9400, 0x9402, PDF_CMAP_RANGE, 20355 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 9506 },
	{ 0x9404, 0x940a, PDF_CMAP_RANGE, 20358 },
	{ 0x940b, 0x940b, PDF_CMAP_SINGLE, 9523 },
	{ 0x940c, 0x940f, PDF_CMAP_RANGE, 20365 },
	{ 0x9410, 0x9410, PDF_CMAP_SINGLE, 8234 },
	{ 0x9411, 0x9411, PDF_CMAP_SINGLE, 20369 },
	{ 0x9412, 0x9412, PDF_CMAP_SINGLE, 9526 },
	{ 0x9413, 0x9413, PDF_CMAP_SINGLE, 9587 },
	{ 0x9414, 0x9414, PDF_CMAP_SINGLE, 9583 },
	{ 0x9415, 0x9417, PDF_CMAP_RANGE, 20370 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 8851 },
	{ 0x9419, 0x9419, PDF_CMAP_SINGLE, 9592 },
	{ 0x941a, 0x941c, PDF_CMAP_RANGE, 20373 },
	{ 0x941d, 0x941d, PDF_CMAP_SINGLE, 9584 },
	{ 0x941e, 0x941f, PDF_CMAP_RANGE, 20376 },
	{ 0x9420, 0x9420, PDF_CMAP_SINGLE, 9589 },
	{ 0x9421, 0x9425, PDF_CMAP_RANGE, 20378 },
	{ 0x9426, 0x9427, PDF_CMAP_RANGE, 9537 },
	{ 0x9428, 0x9428, PDF_CMAP_SINGLE, 9565 },
	{ 0x9429, 0x942d, PDF_CMAP_RANGE, 20383 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 8218 },
	{ 0x942f, 0x9431, PDF_CMAP_RANGE, 20388 },
	{ 0x9432, 0x9432, PDF_CMAP_SINGLE, 9594 },
	{ 0x9433, 0x9433, PDF_CMAP_SINGLE, 8198 },
	{ 0x9434, 0x9434, PDF_CMAP_SINGLE, 20391 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 8567 },
	{ 0x9436, 0x9437, PDF_CMAP_RANGE, 20392 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 9499 },
	{ 0x9439, 0x9439, PDF_CMAP_SINGLE, 20394 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 9508 },
	{ 0x943b, 0x943d, PDF_CMAP_RANGE, 20395 },
	{ 0x943e, 0x943e, PDF_CMAP_SINGLE, 7559 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 9595 },
	{ 0x9440, 0x9443, PDF_CMAP_RANGE, 20398 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 8867 },
	{ 0x9445, 0x9449, PDF_CMAP_RANGE, 20402 },
	{ 0x944a, 0x944a, PDF_CMAP_SINGLE, 9593 },
	{ 0x944b, 0x944b, PDF_CMAP_SINGLE, 20407 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 9574 },
	{ 0x944d, 0x9451, PDF_CMAP_RANGE, 20408 },
	{ 0x9452, 0x9452, PDF_CMAP_SINGLE, 8083 },
	{ 0x9453, 0x9453, PDF_CMAP_SINGLE, 20413 },
	{ 0x9454, 0x9454, PDF_CMAP_SINGLE, 9596 },
	{ 0x9455, 0x945f, PDF_CMAP_RANGE, 20414 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 9492 },
	{ 0x9461, 0x9462, PDF_CMAP_RANGE, 20425 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 9597 },
	{ 0x9464, 0x9464, PDF_CMAP_SINGLE, 20427 },
	{ 0x9465, 0x9465, PDF_CMAP_SINGLE, 9586 },
	{ 0x9466, 0x946a, PDF_CMAP_RANGE, 20428 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 7560 },
	{ 0x946c, 0x946c, PDF_CMAP_SINGLE, 20433 },
	{ 0x946d, 0x946d, PDF_CMAP_SINGLE, 9588 },
	{ 0x946e, 0x946f, PDF_CMAP_RANGE, 20434 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 8782 },
	{ 0x9471, 0x9471, PDF_CMAP_SINGLE, 20436 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 8646 },
	{ 0x9473, 0x9476, PDF_CMAP_RANGE, 20437 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 8351 },
	{ 0x9478, 0x9478, PDF_CMAP_SINGLE, 20441 },
	{ 0x9479, 0x9479, PDF_CMAP_SINGLE, 9590 },
	{ 0x947a, 0x947b, PDF_CMAP_RANGE, 20442 },
	{ 0x947c, 0x947c, PDF_CMAP_SINGLE, 8292 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 8895 },
	{ 0x947e, 0x947e, PDF_CMAP_SINGLE, 9756 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 8798 },
	{ 0x9480, 0x9484, PDF_CMAP_RANGE, 20444 },
	{ 0x9485, 0x9487, PDF_CMAP_RANGE, 6798 },
	{ 0x9488, 0x9488, PDF_CMAP_SINGLE, 4492 },
	{ 0x9489, 0x9489, PDF_CMAP_SINGLE, 1507 },
	{ 0x948a, 0x948a, PDF_CMAP_SINGLE, 6802 },
	{ 0x948b, 0x948b, PDF_CMAP_SINGLE, 6801 },
	{ 0x948c, 0x948d, PDF_CMAP_RANGE, 6803 },
	{ 0x948e, 0x948e, PDF_CMAP_SINGLE, 3106 },
	{ 0x948f, 0x9490, PDF_CMAP_RANGE, 6805 },
	{ 0x9491, 0x9491, PDF_CMAP_SINGLE, 20449 },
	{ 0x9492, 0x9492, PDF_CMAP_SINGLE, 1613 },
	{ 0x9493, 0x9493, PDF_CMAP_SINGLE, 1495 },
	{ 0x9494, 0x9494, PDF_CMAP_SINGLE, 6807 },
	{ 0x9495, 0x9495, PDF_CMAP_SINGLE, 6809 },
	{ 0x9496, 0x9496, PDF_CMAP_SINGLE, 20450 },
	{ 0x9497, 0x9497, PDF_CMAP_SINGLE, 6808 },
	{ 0x9498, 0x9498, PDF_CMAP_SINGLE, 20451 },
	{ 0x9499, 0x9499, PDF_CMAP_SINGLE, 1729 },
	{ 0x949a, 0x949c, PDF_CMAP_RANGE, 6810 },
	{ 0x949d, 0x949d, PDF_CMAP_SINGLE, 1562 },
	{ 0x949e, 0x949e, PDF_CMAP_SINGLE, 1235 },
	{ 0x949f, 0x949f, PDF_CMAP_SINGLE, 4562 },
	{ 0x94a0, 0x94a0, PDF_CMAP_SINGLE, 2857 },
	{ 0x94a1, 0x94a1, PDF_CMAP_SINGLE, 1054 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 1745 },
	{ 0x94a3, 0x94a4, PDF_CMAP_RANGE, 6813 },
	{ 0x94a5, 0x94a5, PDF_CMAP_SINGLE, 4354 },
	{ 0x94a6, 0x94a6, PDF_CMAP_SINGLE, 3154 },
	{ 0x94a7, 0x94a7, PDF_CMAP_SINGLE, 2346 },
	{ 0x94a8, 0x94a8, PDF_CMAP_SINGLE, 3816 },
	{ 0x94a9, 0x94a9, PDF_CMAP_SINGLE, 1804 },
	{ 0x94aa, 0x94aa, PDF_CMAP_SINGLE, 6816 },
	{ 0x94ab, 0x94ab, PDF_CMAP_SINGLE, 6815 },
	{ 0x94ac, 0x94ac, PDF_CMAP_SINGLE, 6818 },
	{ 0x94ad, 0x94ad, PDF_CMAP_SINGLE, 6817 },
	{ 0x94ae, 0x94ae, PDF_CMAP_SINGLE, 2918 },
	{ 0x94af, 0x94b0, PDF_CMAP_RANGE, 6819 },
	{ 0x94b1, 0x94b1, PDF_CMAP_SINGLE, 3115 },
	{ 0x94b2, 0x94b2, PDF_CMAP_SINGLE, 6821 },
	{ 0x94b3, 0x94b3, PDF_CMAP_SINGLE, 3116 },
	{ 0x94b4, 0x94b4, PDF_CMAP_SINGLE, 6822 },
	{ 0x94b5, 0x94b5, PDF_CMAP_SINGLE, 1134 },
	{ 0x94b6, 0x94ba, PDF_CMAP_RANGE, 6823 },
	{ 0x94bb, 0x94bb, PDF_CMAP_SINGLE, 4679 },
	{ 0x94bc, 0x94bd, PDF_CMAP_RANGE, 6828 },
	{ 0x94be, 0x94be, PDF_CMAP_SINGLE, 2123 },
	{ 0x94bf, 0x94bf, PDF_CMAP_SINGLE, 6830 },
	{ 0x94c0, 0x94c0, PDF_CMAP_SINGLE, 4272 },
	{ 0x94c1, 0x94c1, PDF_CMAP_SINGLE, 3661 },
	{ 0x94c2, 0x94c2, PDF_CMAP_SINGLE, 1139 },
	{ 0x94c3, 0x94c3, PDF_CMAP_SINGLE, 2606 },
	{ 0x94c4, 0x94c4, PDF_CMAP_SINGLE, 6831 },
	{ 0x94c5, 0x94c5, PDF_CMAP_SINGLE, 3107 },
	{ 0x94c6, 0x94c6, PDF_CMAP_SINGLE, 2738 },
	{ 0x94c7, 0x94c7, PDF_CMAP_SINGLE, 20452 },
	{ 0x94c8, 0x94ce, PDF_CMAP_RANGE, 6832 },
	{ 0x94cf, 0x94cf, PDF_CMAP_SINGLE, 20453 },
	{ 0x94d0, 0x94d2, PDF_CMAP_RANGE, 6839 },
	{ 0x94d3, 0x94d4, PDF_CMAP_RANGE, 20454 },
	{ 0x94d5, 0x94d7, PDF_CMAP_RANGE, 6842 },
	{ 0x94d8, 0x94d8, PDF_CMAP_SINGLE, 6846 },
	{ 0x94d9, 0x94d9, PDF_CMAP_SINGLE, 6845 },
	{ 0x94da, 0x94da, PDF_CMAP_SINGLE, 20456 },
	{ 0x94db, 0x94db, PDF_CMAP_SINGLE, 6847 },
	{ 0x94dc, 0x94dc, PDF_CMAP_SINGLE, 3678 },
	{ 0x94dd, 0x94dd, PDF_CMAP_SINGLE, 2664 },
	{ 0x94de, 0x94e0, PDF_CMAP_RANGE, 6848 },
	{ 0x94e1, 0x94e1, PDF_CMAP_SINGLE, 4418 },
	{ 0x94e2, 0x94e2, PDF_CMAP_SINGLE, 6851 },
	{ 0x94e3, 0x94e3, PDF_CMAP_SINGLE, 3872 },
	{ 0x94e4, 0x94e5, PDF_CMAP_RANGE, 6852 },
	{ 0x94e6, 0x94e6, PDF_CMAP_SINGLE, 20457 },
	{ 0x94e7, 0x94e8, PDF_CMAP_RANGE, 6854 },
	{ 0x94e9, 0x94e9, PDF_CMAP_SINGLE, 6857 },
	{ 0x94ea, 0x94ea, PDF_CMAP_SINGLE, 6856 },
	{ 0x94eb, 0x94eb, PDF_CMAP_SINGLE, 6858 },
	{ 0x94ec, 0x94ec, PDF_CMAP_SINGLE, 1776 },
	{ 0x94ed, 0x94ed, PDF_CMAP_SINGLE, 2815 },
	{ 0x94ee, 0x94ef, PDF_CMAP_RANGE, 6859 },
	{ 0x94f0, 0x94f0, PDF_CMAP_SINGLE, 2195 },
	{ 0x94f1, 0x94f1, PDF_CMAP_SINGLE, 4166 },
	{ 0x94f2, 0x94f2, PDF_CMAP_SINGLE, 1216 },
	{ 0x94f3, 0x94f5, PDF_CMAP_RANGE, 6861 },
	{ 0x94f6, 0x94f6, PDF_CMAP_SINGLE, 4223 },
	{ 0x94f7, 0x94f7, PDF_CMAP_SINGLE, 6864 },
	{ 0x94f8, 0x94f8, PDF_CMAP_SINGLE, 4604 },
	{ 0x94f9, 0x94f9, PDF_CMAP_SINGLE, 6865 },
	{ 0x94fa, 0x94fa, PDF_CMAP_SINGLE, 3051 },
	{ 0x94fb, 0x94fb, PDF_CMAP_SINGLE, 20458 },
	{ 0x94fc, 0x94fd, PDF_CMAP_RANGE, 6866 },
	{ 0x94fe, 0x94fe, PDF_CMAP_SINGLE, 2557 },
	{ 0x94ff, 0x94ff, PDF_CMAP_SINGLE, 6868 },
	{ 0x9500, 0x9500, PDF_CMAP_SINGLE, 3943 },
	{ 0x9501, 0x9501, PDF_CMAP_SINGLE, 3565 },
	{ 0x9502, 0x9502, PDF_CMAP_SINGLE, 6870 },
	{ 0x9503, 0x9503, PDF_CMAP_SINGLE, 6869 },
	{ 0x9504, 0x9504, PDF_CMAP_SINGLE, 1311 },
	{ 0x9505, 0x9505, PDF_CMAP_SINGLE, 1873 },
	{ 0x9506, 0x9507, PDF_CMAP_RANGE, 6871 },
	{ 0x9508, 0x9508, PDF_CMAP_SINGLE, 4013 },
	{ 0x9509, 0x950a, PDF_CMAP_RANGE, 6873 },
	{ 0x950b, 0x950b, PDF_CMAP_SINGLE, 1667 },
	{ 0x950c, 0x950c, PDF_CMAP_SINGLE, 3978 },
	{ 0x950d, 0x950f, PDF_CMAP_RANGE, 6875 },
	{ 0x9510, 0x9510, PDF_CMAP_SINGLE, 3276 },
	{ 0x9511, 0x9511, PDF_CMAP_SINGLE, 3635 },
	{ 0x9512, 0x9516, PDF_CMAP_RANGE, 6878 },
	{ 0x9517, 0x9517, PDF_CMAP_SINGLE, 4481 },
	{ 0x9518, 0x9518, PDF_CMAP_SINGLE, 6883 },
	{ 0x9519, 0x9519, PDF_CMAP_SINGLE, 1392 },
	{ 0x951a, 0x951a, PDF_CMAP_SINGLE, 2735 },
	{ 0x951b, 0x951b, PDF_CMAP_SINGLE, 6884 },
	{ 0x951c, 0x951c, PDF_CMAP_SINGLE, 20459 },
	{ 0x951d, 0x951f, PDF_CMAP_RANGE, 6885 },
	{ 0x9520, 0x9520, PDF_CMAP_SINGLE, 20460 },
	{ 0x9521, 0x9521, PDF_CMAP_SINGLE, 3852 },
	{ 0x9522, 0x9522, PDF_CMAP_SINGLE, 6888 },
	{ 0x9523, 0x9523, PDF_CMAP_SINGLE, 2695 },
	{ 0x9524, 0x9524, PDF_CMAP_SINGLE, 1339 },
	{ 0x9525, 0x9525, PDF_CMAP_SINGLE, 4627 },
	{ 0x9526, 0x9526, PDF_CMAP_SINGLE, 2246 },
	{ 0x9527, 0x9527, PDF_CMAP_SINGLE, 20461 },
	{ 0x9528, 0x9528, PDF_CMAP_SINGLE, 3892 },
	{ 0x9529, 0x9529, PDF_CMAP_SINGLE, 6891 },
	{ 0x952a, 0x952b, PDF_CMAP_RANGE, 6889 },
	{ 0x952c, 0x952c, PDF_CMAP_SINGLE, 6892 },
	{ 0x952d, 0x952d, PDF_CMAP_SINGLE, 1510 },
	{ 0x952e, 0x952e, PDF_CMAP_SINGLE, 2159 },
	{ 0x952f, 0x952f, PDF_CMAP_SINGLE, 2321 },
	{ 0x9530, 0x9530, PDF_CMAP_SINGLE, 2769 },
	{ 0x9531, 0x9532, PDF_CMAP_RANGE, 6893 },
	{ 0x9533, 0x9533, PDF_CMAP_SINGLE, 20462 },
	{ 0x9534, 0x9534, PDF_CMAP_SINGLE, 6895 },
	{ 0x9535, 0x9535, PDF_CMAP_SINGLE, 6903 },
	{ 0x9536, 0x9538, PDF_CMAP_RANGE, 6896 },
	{ 0x9539, 0x9539, PDF_CMAP_SINGLE, 3135 },
	{ 0x953a, 0x953a, PDF_CMAP_SINGLE, 6937 },
	{ 0x953b, 0x953b, PDF_CMAP_SINGLE, 1548 },
	{ 0x953c, 0x953c, PDF_CMAP_SINGLE, 6899 },
	{ 0x953d, 0x953d, PDF_CMAP_SINGLE, 20463 },
	{ 0x953e, 0x953f, PDF_CMAP_RANGE, 6900 },
	{ 0x9540, 0x9540, PDF_CMAP_SINGLE, 1541 },
	{ 0x9541, 0x9541, PDF_CMAP_SINGLE, 2755 },
	{ 0x9542, 0x9542, PDF_CMAP_SINGLE, 6902 },
	{ 0x9543, 0x9543, PDF_CMAP_SINGLE, 20464 },
	{ 0x9544, 0x9546, PDF_CMAP_RANGE, 6904 },
	{ 0x9547, 0x9547, PDF_CMAP_SINGLE, 4499 },
	{ 0x9548, 0x9548, PDF_CMAP_SINGLE, 20465 },
	{ 0x9549, 0x9549, PDF_CMAP_SINGLE, 6907 },
	{ 0x954a, 0x954a, PDF_CMAP_SINGLE, 2906 },
	{ 0x954b, 0x954b, PDF_CMAP_SINGLE, 20466 },
	{ 0x954c, 0x954c, PDF_CMAP_SINGLE, 6908 },
	{ 0x954d, 0x954d, PDF_CMAP_SINGLE, 2907 },
	{ 0x954e, 0x954f, PDF_CMAP_RANGE, 6909 },
	{ 0x9550, 0x9550, PDF_CMAP_SINGLE, 1759 },
	{ 0x9551, 0x9551, PDF_CMAP_SINGLE, 1026 },
	{ 0x9552, 0x9554, PDF_CMAP_RANGE, 6911 },
	{ 0x9555, 0x9555, PDF_CMAP_SINGLE, 20467 },
	{ 0x9556, 0x9559, PDF_CMAP_RANGE, 6914 },
	{ 0x955a, 0x955a, PDF_CMAP_SINGLE, 20468 },
	{ 0x955b, 0x955b, PDF_CMAP_SINGLE, 6918 },
	{ 0x955c, 0x955c, PDF_CMAP_SINGLE, 2276 },
	{ 0x955d, 0x955d, PDF_CMAP_SINGLE, 6921 },
	{ 0x955e, 0x955f, PDF_CMAP_RANGE, 6919 },
	{ 0x9560, 0x9560, PDF_CMAP_SINGLE, 20469 },
	{ 0x9561, 0x9562, PDF_CMAP_RANGE, 6922 },
	{ 0x9563, 0x9563, PDF_CMAP_SINGLE, 2582 },
	{ 0x9564, 0x956c, PDF_CMAP_RANGE, 6924 },
	{ 0x956d, 0x956d, PDF_CMAP_SINGLE, 2499 },
	{ 0x956e, 0x956e, PDF_CMAP_SINGLE, 20470 },
	{ 0x956f, 0x956f, PDF_CMAP_SINGLE, 6933 },
	{ 0x9570, 0x9570, PDF_CMAP_SINGLE, 2550 },
	{ 0x9571, 0x9573, PDF_CMAP_RANGE, 6934 },
	{ 0x9574, 0x9575, PDF_CMAP_RANGE, 20471 },
	{ 0x9576, 0x9576, PDF_CMAP_SINGLE, 3919 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 7797 },
	{ 0x9578, 0x957e, PDF_CMAP_RANGE, 20473 },
	{ 0x957f, 0x957f, PDF_CMAP_SINGLE, 1225 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 8317 },
	{ 0x9581, 0x9581, PDF_CMAP_SINGLE, 20480 },
	{ 0x9582, 0x9582, PDF_CMAP_SINGLE, 9151 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 8467 },
	{ 0x9584, 0x9585, PDF_CMAP_RANGE, 20481 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 9152 },
	{ 0x9587, 0x9588, PDF_CMAP_RANGE, 20483 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 7749 },
	{ 0x958a, 0x958a, PDF_CMAP_SINGLE, 20485 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 8152 },
	{ 0x958c, 0x958c, PDF_CMAP_SINGLE, 9156 },
	{ 0x958d, 0x958d, PDF_CMAP_SINGLE, 20486 },
	{ 0x958e, 0x958e, PDF_CMAP_SINGLE, 9154 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 8452 },
	{ 0x9590, 0x9590, PDF_CMAP_SINGLE, 20487 },
	{ 0x9591, 0x9591, PDF_CMAP_SINGLE, 8637 },
	{ 0x9592, 0x9592, PDF_CMAP_SINGLE, 20488 },
	{ 0x9593, 0x9593, PDF_CMAP_SINGLE, 8071 },
	{ 0x9594, 0x9594, PDF_CMAP_SINGLE, 9155 },
	{ 0x9595, 0x9597, PDF_CMAP_RANGE, 20489 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 8809 },
	{ 0x9599, 0x95a0, PDF_CMAP_RANGE, 20492 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 8003 },
	{ 0x95a2, 0x95a2, PDF_CMAP_SINGLE, 20500 },
	{ 0x95a3, 0x95a3, PDF_CMAP_SINGLE, 7966 },
	{ 0x95a4, 0x95a4, PDF_CMAP_SINGLE, 9849 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 7915 },
	{ 0x95a6, 0x95a7, PDF_CMAP_RANGE, 20501 },
	{ 0x95a8, 0x95a8, PDF_CMAP_SINGLE, 7989 },
	{ 0x95a9, 0x95a9, PDF_CMAP_SINGLE, 8330 },
	{ 0x95aa, 0x95aa, PDF_CMAP_SINGLE, 20503 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 9159 },
	{ 0x95ac, 0x95ac, PDF_CMAP_SINGLE, 9161 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 9158 },
	{ 0x95ae, 0x95b1, PDF_CMAP_RANGE, 20504 },
	{ 0x95b2, 0x95b2, PDF_CMAP_SINGLE, 8783 },
	{ 0x95b3, 0x95b5, PDF_CMAP_RANGE, 20508 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 9163 },
	{ 0x95b7, 0x95b8, PDF_CMAP_RANGE, 20511 },
	{ 0x95b9, 0x95b9, PDF_CMAP_SINGLE, 8691 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 20513 },
	{ 0x95bb, 0x95bb, PDF_CMAP_SINGLE, 8695 },
	{ 0x95bc, 0x95bc, PDF_CMAP_SINGLE, 9167 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 9166 },
	{ 0x95be, 0x95be, PDF_CMAP_SINGLE, 9162 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 9165 },
	{ 0x95c0, 0x95c2, PDF_CMAP_RANGE, 20514 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 9168 },
	{ 0x95c4, 0x95c5, PDF_CMAP_RANGE, 20517 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 9836 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 20519 },
	{ 0x95c8, 0x95c8, PDF_CMAP_SINGLE, 9153 },
	{ 0x95c9, 0x95c9, PDF_CMAP_SINGLE, 20520 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 8174 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 9169 },
	{ 0x95cc, 0x95cc, PDF_CMAP_SINGLE, 8184 },
	{ 0x95cd, 0x95cf, PDF_CMAP_RANGE, 20521 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 9171 },
	{ 0x95d1, 0x95d3, PDF_CMAP_RANGE, 20524 },
	{ 0x95d4, 0x95d4, PDF_CMAP_SINGLE, 9170 },
	{ 0x95d5, 0x95d5, PDF_CMAP_SINGLE, 9172 },
	{ 0x95d6, 0x95d6, PDF_CMAP_SINGLE, 7832 },
	{ 0x95d7, 0x95db, PDF_CMAP_RANGE, 20527 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 7980 },
	{ 0x95dd, 0x95dd, PDF_CMAP_SINGLE, 20532 },
	{ 0x95de, 0x95de, PDF_CMAP_SINGLE, 9173 },
	{ 0x95df, 0x95e0, PDF_CMAP_RANGE, 20533 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 7793 },
	{ 0x95e2, 0x95e2, PDF_CMAP_SINGLE, 9873 },
	{ 0x95e3, 0x95e4, PDF_CMAP_RANGE, 20535 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 9157 },
	{ 0x95e6, 0x95e7, PDF_CMAP_RANGE, 20537 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 2762 },
	{ 0x95e9, 0x95e9, PDF_CMAP_SINGLE, 5765 },
	{ 0x95ea, 0x95ea, PDF_CMAP_SINGLE, 3322 },
	{ 0x95eb, 0x95eb, PDF_CMAP_SINGLE, 5766 },
	{ 0x95ec, 0x95ec, PDF_CMAP_SINGLE, 20539 },
	{ 0x95ed, 0x95ed, PDF_CMAP_SINGLE, 1086 },
	{ 0x95ee, 0x95ee, PDF_CMAP_SINGLE, 3801 },
	{ 0x95ef, 0x95ef, PDF_CMAP_SINGLE, 1334 },
	{ 0x95f0, 0x95f0, PDF_CMAP_SINGLE, 3277 },
	{ 0x95f1, 0x95f1, PDF_CMAP_SINGLE, 5767 },
	{ 0x95f2, 0x95f2, PDF_CMAP_SINGLE, 3901 },
	{ 0x95f3, 0x95f3, PDF_CMAP_SINGLE, 5768 },
	{ 0x95f4, 0x95f4, PDF_CMAP_SINGLE, 2135 },
	{ 0x95f5, 0x95f6, PDF_CMAP_RANGE, 5769 },
	{ 0x95f7, 0x95f7, PDF_CMAP_SINGLE, 2763 },
	{ 0x95f8, 0x95f8, PDF_CMAP_SINGLE, 4419 },
	{ 0x95f9, 0x95f9, PDF_CMAP_SINGLE, 2873 },
	{ 0x95fa, 0x95fa, PDF_CMAP_SINGLE, 1860 },
	{ 0x95fb, 0x95fb, PDF_CMAP_SINGLE, 3796 },
	{ 0x95fc, 0x95fc, PDF_CMAP_SINGLE, 5771 },
	{ 0x95fd, 0x95fd, PDF_CMAP_SINGLE, 2811 },
	{ 0x95fe, 0x95fe, PDF_CMAP_SINGLE, 5772 },
	{ 0x95ff, 0x95ff, PDF_CMAP_SINGLE, 20540 },
	{ 0x9600, 0x9600, PDF_CMAP_SINGLE, 1604 },
	{ 0x9601, 0x9601, PDF_CMAP_SINGLE, 1774 },
	{ 0x9602, 0x9602, PDF_CMAP_SINGLE, 1929 },
	{ 0x9603, 0x9604, PDF_CMAP_RANGE, 5773 },
	{ 0x9605, 0x9605, PDF_CMAP_SINGLE, 4359 },
	{ 0x9606, 0x9606, PDF_CMAP_SINGLE, 5775 },
	{ 0x9607, 0x9607, PDF_CMAP_SINGLE, 20541 },
	{ 0x9608, 0x9608, PDF_CMAP_SINGLE, 5776 },
	{ 0x9609, 0x9609, PDF_CMAP_SINGLE, 4084 },
	{ 0x960a, 0x960d, PDF_CMAP_RANGE, 5777 },
	{ 0x960e, 0x960e, PDF_CMAP_SINGLE, 4095 },
	{ 0x960f, 0x960f, PDF_CMAP_SINGLE, 5781 },
	{ 0x9610, 0x9610, PDF_CMAP_SINGLE, 1218 },
	{ 0x9611, 0x9611, PDF_CMAP_SINGLE, 2470 },
	{ 0x9612, 0x9612, PDF_CMAP_SINGLE, 5782 },
	{ 0x9613, 0x9613, PDF_CMAP_SINGLE, 20542 },
	{ 0x9614, 0x9614, PDF_CMAP_SINGLE, 2454 },
	{ 0x9615, 0x9617, PDF_CMAP_RANGE, 5783 },
	{ 0x9618, 0x9618, PDF_CMAP_SINGLE, 20543 },
	{ 0x9619, 0x961a, PDF_CMAP_RANGE, 5786 },
	{ 0x961b, 0x961b, PDF_CMAP_SINGLE, 20544 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 1714 },
	{ 0x961d, 0x961d, PDF_CMAP_SINGLE, 4948 },
	{ 0x961e, 0x961e, PDF_CMAP_SINGLE, 20545 },
	{ 0x961f, 0x961f, PDF_CMAP_SINGLE, 1554 },
	{ 0x9620, 0x9620, PDF_CMAP_SINGLE, 20546 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 4950 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 4949 },
	{ 0x9623, 0x9629, PDF_CMAP_RANGE, 20547 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 4952 },
	{ 0x962b, 0x962d, PDF_CMAP_RANGE, 20554 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 3273 },
	{ 0x962f, 0x9630, PDF_CMAP_RANGE, 20557 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 4951 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 1629 },
	{ 0x9633, 0x9633, PDF_CMAP_SINGLE, 4125 },
	{ 0x9634, 0x9634, PDF_CMAP_SINGLE, 4220 },
	{ 0x9635, 0x9635, PDF_CMAP_SINGLE, 4500 },
	{ 0x9636, 0x9636, PDF_CMAP_SINGLE, 2216 },
	{ 0x9637, 0x963a, PDF_CMAP_RANGE, 20559 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 4677 },
	{ 0x963c, 0x963c, PDF_CMAP_SINGLE, 4954 },
	{ 0x963d, 0x963d, PDF_CMAP_SINGLE, 4953 },
	{ 0x963e, 0x963e, PDF_CMAP_SINGLE, 20563 },
	{ 0x963f, 0x963f, PDF_CMAP_SINGLE, 941 },
	{ 0x9640, 0x9640, PDF_CMAP_SINGLE, 3716 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 20564 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 4955 },
	{ 0x9643, 0x9643, PDF_CMAP_SINGLE, 20565 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 1720 },
	{ 0x9645, 0x9645, PDF_CMAP_SINGLE, 2109 },
	{ 0x9646, 0x9646, PDF_CMAP_SINGLE, 2660 },
	{ 0x9647, 0x9647, PDF_CMAP_SINGLE, 2635 },
	{ 0x9648, 0x9648, PDF_CMAP_SINGLE, 1255 },
	{ 0x9649, 0x9649, PDF_CMAP_SINGLE, 4956 },
	{ 0x964a, 0x964a, PDF_CMAP_SINGLE, 20566 },
	{ 0x964b, 0x964b, PDF_CMAP_SINGLE, 2641 },
	{ 0x964c, 0x964c, PDF_CMAP_SINGLE, 2835 },
	{ 0x964d, 0x964d, PDF_CMAP_SINGLE, 2182 },
	{ 0x964e, 0x964f, PDF_CMAP_RANGE, 20567 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 3915 },
	{ 0x9651, 0x9653, PDF_CMAP_RANGE, 20569 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 4957 },
	{ 0x9655, 0x9655, PDF_CMAP_SINGLE, 3323 },
	{ 0x9656, 0x9657, PDF_CMAP_RANGE, 20572 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 8986 },
	{ 0x9659, 0x965a, PDF_CMAP_RANGE, 20574 },
	{ 0x965b, 0x965b, PDF_CMAP_SINGLE, 1094 },
	{ 0x965c, 0x965c, PDF_CMAP_SINGLE, 20576 },
	{ 0x965d, 0x965d, PDF_CMAP_SINGLE, 8468 },
	{ 0x965e, 0x965e, PDF_CMAP_SINGLE, 20577 },
	{ 0x965f, 0x965f, PDF_CMAP_SINGLE, 4958 },
	{ 0x9660, 0x9660, PDF_CMAP_SINGLE, 20578 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 1527 },
	{ 0x9662, 0x9662, PDF_CMAP_SINGLE, 4349 },
	{ 0x9663, 0x9663, PDF_CMAP_SINGLE, 8836 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 1314 },
	{ 0x9665, 0x9666, PDF_CMAP_RANGE, 20579 },
	{ 0x9667, 0x9667, PDF_CMAP_SINGLE, 4959 },
	{ 0x9668, 0x9668, PDF_CMAP_SINGLE, 4364 },
	{ 0x9669, 0x9669, PDF_CMAP_SINGLE, 3906 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 2980 },
	{ 0x966b, 0x966b, PDF_CMAP_SINGLE, 20581 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 4960 },
	{ 0x966d, 0x966f, PDF_CMAP_RANGE, 20582 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 8732 },
	{ 0x9671, 0x9671, PDF_CMAP_SINGLE, 20585 },
	{ 0x9672, 0x9672, PDF_CMAP_SINGLE, 4961 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 7806 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 4962 },
	{ 0x9675, 0x9675, PDF_CMAP_SINGLE, 2611 },
	{ 0x9676, 0x9676, PDF_CMAP_SINGLE, 3624 },
	{ 0x9677, 0x9677, PDF_CMAP_SINGLE, 3914 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 8269 },
	{ 0x9679, 0x967c, PDF_CMAP_RANGE, 20586 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 8705 },
	{ 0x967e, 0x9684, PDF_CMAP_RANGE, 20590 },
	{ 0x9685, 0x9685, PDF_CMAP_SINGLE, 4300 },
	{ 0x9686, 0x9686, PDF_CMAP_SINGLE, 2632 },
	{ 0x9687, 0x9687, PDF_CMAP_SINGLE, 20597 },
	{ 0x9688, 0x9688, PDF_CMAP_SINGLE, 4963 },
	{ 0x9689, 0x9689, PDF_CMAP_SINGLE, 20598 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 7897 },
	{ 0x968b, 0x968b, PDF_CMAP_SINGLE, 3546 },
	{ 0x968c, 0x968c, PDF_CMAP_SINGLE, 20599 },
	{ 0x968d, 0x968d, PDF_CMAP_SINGLE, 4964 },
	{ 0x968e, 0x968e, PDF_CMAP_SINGLE, 8114 },
	{ 0x968f, 0x968f, PDF_CMAP_SINGLE, 3547 },
	{ 0x9690, 0x9690, PDF_CMAP_SINGLE, 4229 },
	{ 0x9691, 0x9693, PDF_CMAP_RANGE, 20600 },
	{ 0x9694, 0x9694, PDF_CMAP_SINGLE, 1775 },
	{ 0x9695, 0x9695, PDF_CMAP_SINGLE, 8786 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 20603 },
	{ 0x9697, 0x9697, PDF_CMAP_SINGLE, 4965 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 954 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 3875 },
	{ 0x969a, 0x969a, PDF_CMAP_SINGLE, 20604 },
	{ 0x969b, 0x969b, PDF_CMAP_SINGLE, 8057 },
	{ 0x969c, 0x969c, PDF_CMAP_SINGLE, 4464 },
	{ 0x969d, 0x96a6, PDF_CMAP_RANGE, 20605 },
	{ 0x96a7, 0x96a7, PDF_CMAP_SINGLE, 3554 },
	{ 0x96a8, 0x96a8, PDF_CMAP_SINGLE, 8535 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 20615 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 8639 },
	{ 0x96ab, 0x96af, PDF_CMAP_RANGE, 20616 },
	{ 0x96b0, 0x96b0, PDF_CMAP_SINGLE, 4966 },
	{ 0x96b1, 0x96b1, PDF_CMAP_SINGLE, 8735 },
	{ 0x96b2, 0x96b2, PDF_CMAP_SINGLE, 20621 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 5764 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 8253 },
	{ 0x96b5, 0x96b5, PDF_CMAP_SINGLE, 20622 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 2542 },
	{ 0x96b7, 0x96b7, PDF_CMAP_SINGLE, 20623 },
	{ 0x96b8, 0x96b8, PDF_CMAP_SINGLE, 8213 },
	{ 0x96b9, 0x96b9, PDF_CMAP_SINGLE, 7545 },
	{ 0x96ba, 0x96ba, PDF_CMAP_SINGLE, 20624 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 9893 },
	{ 0x96bc, 0x96bd, PDF_CMAP_RANGE, 7546 },
	{ 0x96be, 0x96be, PDF_CMAP_SINGLE, 2868 },
	{ 0x96bf, 0x96bf, PDF_CMAP_SINGLE, 20625 },
	{ 0x96c0, 0x96c0, PDF_CMAP_SINGLE, 3219 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 4108 },
	{ 0x96c2, 0x96c3, PDF_CMAP_RANGE, 20626 },
	{ 0x96c4, 0x96c4, PDF_CMAP_SINGLE, 4006 },
	{ 0x96c5, 0x96c5, PDF_CMAP_SINGLE, 4078 },
	{ 0x96c6, 0x96c6, PDF_CMAP_SINGLE, 2082 },
	{ 0x96c7, 0x96c7, PDF_CMAP_SINGLE, 1830 },
	{ 0x96c8, 0x96c8, PDF_CMAP_SINGLE, 20628 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 6940 },
	{ 0x96ca, 0x96cb, PDF_CMAP_RANGE, 20629 },
	{ 0x96cc, 0x96cc, PDF_CMAP_SINGLE, 1353 },
	{ 0x96cd, 0x96cd, PDF_CMAP_SINGLE, 4255 },
	{ 0x96ce, 0x96ce, PDF_CMAP_SINGLE, 7548 },
	{ 0x96cf, 0x96cf, PDF_CMAP_SINGLE, 1312 },
	{ 0x96d0, 0x96d1, PDF_CMAP_RANGE, 20631 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 7549 },
	{ 0x96d3, 0x96d4, PDF_CMAP_RANGE, 20633 },
	{ 0x96d5, 0x96d5, PDF_CMAP_SINGLE, 1490 },
	{ 0x96d6, 0x96d6, PDF_CMAP_SINGLE, 8534 },
	{ 0x96d7, 0x96d8, PDF_CMAP_RANGE, 20635 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 8516 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 20637 },
	{ 0x96db, 0x96db, PDF_CMAP_SINGLE, 7825 },
	{ 0x96dc, 0x96dc, PDF_CMAP_SINGLE, 8791 },
	{ 0x96dd, 0x96df, PDF_CMAP_RANGE, 20638 },
	{ 0x96e0, 0x96e0, PDF_CMAP_SINGLE, 7551 },
	{ 0x96e1, 0x96e1, PDF_CMAP_SINGLE, 20641 },
	{ 0x96e2, 0x96e2, PDF_CMAP_SINGLE, 8202 },
	{ 0x96e3, 0x96e3, PDF_CMAP_SINGLE, 8338 },
	{ 0x96e4, 0x96e7, PDF_CMAP_RANGE, 20642 },
	{ 0x96e8, 0x96e8, PDF_CMAP_SINGLE, 4303 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 7521 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 4050 },
	{ 0x96eb, 0x96ee, PDF_CMAP_RANGE, 20646 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 7523 },
	{ 0x96f0, 0x96f1, PDF_CMAP_RANGE, 20650 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 8784 },
	{ 0x96f3, 0x96f3, PDF_CMAP_SINGLE, 7522 },
	{ 0x96f4, 0x96f5, PDF_CMAP_RANGE, 20652 },
	{ 0x96f6, 0x96f6, PDF_CMAP_SINGLE, 2604 },
	{ 0x96f7, 0x96f7, PDF_CMAP_SINGLE, 2498 },
	{ 0x96f8, 0x96f8, PDF_CMAP_SINGLE, 20654 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 1035 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 20655 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 7875 },
	{ 0x96fc, 0x96fd, PDF_CMAP_RANGE, 20656 },
	{ 0x96fe, 0x96fe, PDF_CMAP_SINGLE, 3836 },
	{ 0x96ff, 0x96ff, PDF_CMAP_SINGLE, 20658 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 4019 },
	{ 0x9701, 0x9701, PDF_CMAP_SINGLE, 7525 },
	{ 0x9702, 0x9703, PDF_CMAP_RANGE, 20659 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 3939 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 20661 },
	{ 0x9706, 0x9706, PDF_CMAP_SINGLE, 7524 },
	{ 0x9707, 0x9707, PDF_CMAP_SINGLE, 4497 },
	{ 0x9708, 0x9708, PDF_CMAP_SINGLE, 7526 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 2750 },
	{ 0x970a, 0x970c, PDF_CMAP_RANGE, 20662 },
	{ 0x970d, 0x970d, PDF_CMAP_SINGLE, 2057 },
	{ 0x970e, 0x970e, PDF_CMAP_SINGLE, 7528 },
	{ 0x970f, 0x970f, PDF_CMAP_SINGLE, 7527 },
	{ 0x9710, 0x9712, PDF_CMAP_RANGE, 20665 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 2881 },
	{ 0x9714, 0x9715, PDF_CMAP_RANGE, 20668 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 2593 },
	{ 0x9717, 0x971b, PDF_CMAP_RANGE, 20670 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 3487 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 20675 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 3881 },
	{ 0x971f, 0x9726, PDF_CMAP_RANGE, 20676 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 8616 },
	{ 0x9728, 0x9729, PDF_CMAP_RANGE, 20684 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7529 },
	{ 0x972b, 0x972c, PDF_CMAP_RANGE, 20686 },
	{ 0x972d, 0x972d, PDF_CMAP_SINGLE, 7530 },
	{ 0x972e, 0x972f, PDF_CMAP_RANGE, 20688 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 7531 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 20690 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 2653 },
	{ 0x9733, 0x9737, PDF_CMAP_RANGE, 20691 },
	{ 0x9738, 0x9738, PDF_CMAP_SINGLE, 991 },
	{ 0x9739, 0x9739, PDF_CMAP_SINGLE, 3002 },
	{ 0x973a, 0x973c, PDF_CMAP_RANGE, 20696 },
	{ 0x973d, 0x973d, PDF_CMAP_SINGLE, 9741 },
	{ 0x973e, 0x973e, PDF_CMAP_SINGLE, 7532 },
	{ 0x973f, 0x9741, PDF_CMAP_RANGE, 20699 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 9740 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 20702 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 9742 },
	{ 0x9745, 0x9747, PDF_CMAP_RANGE, 20703 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 8242 },
	{ 0x9749, 0x9751, PDF_CMAP_RANGE, 20706 },
	{ 0x9752, 0x9752, PDF_CMAP_SINGLE, 3165 },
	{ 0x9753, 0x9753, PDF_CMAP_SINGLE, 7520 },
	{ 0x9754, 0x9755, PDF_CMAP_RANGE, 20715 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 2279 },
	{ 0x9757, 0x9758, PDF_CMAP_RANGE, 20717 },
	{ 0x9759, 0x9759, PDF_CMAP_SINGLE, 2273 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 9739 },
	{ 0x975b, 0x975b, PDF_CMAP_SINGLE, 1478 },
	{ 0x975c, 0x975d, PDF_CMAP_RANGE, 20719 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 1636 },
	{ 0x975f, 0x975f, PDF_CMAP_SINGLE, 20721 },
	{ 0x9760, 0x9760, PDF_CMAP_SINGLE, 2380 },
	{ 0x9761, 0x9761, PDF_CMAP_SINGLE, 2775 },
	{ 0x9762, 0x9762, PDF_CMAP_SINGLE, 2795 },
	{ 0x9763, 0x9764, PDF_CMAP_RANGE, 20722 },
	{ 0x9765, 0x9765, PDF_CMAP_SINGLE, 4738 },
	{ 0x9766, 0x9767, PDF_CMAP_RANGE, 20724 },
	{ 0x9768, 0x9768, PDF_CMAP_SINGLE, 8899 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 1770 },
	{ 0x976a, 0x9772, PDF_CMAP_RANGE, 20726 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 2250 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 4046 },
	{ 0x9775, 0x9775, PDF_CMAP_SINGLE, 20735 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 987 },
	{ 0x9777, 0x977b, PDF_CMAP_RANGE, 20736 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7624 },
	{ 0x977d, 0x9784, PDF_CMAP_RANGE, 20741 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 7625 },
	{ 0x9786, 0x978a, PDF_CMAP_RANGE, 20749 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 3959 },
	{ 0x978c, 0x978c, PDF_CMAP_SINGLE, 20754 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 955 },
	{ 0x978e, 0x978e, PDF_CMAP_SINGLE, 20755 },
	{ 0x978f, 0x978f, PDF_CMAP_SINGLE, 7971 },
	{ 0x9790, 0x9790, PDF_CMAP_SINGLE, 20756 },
	{ 0x9791, 0x9792, PDF_CMAP_RANGE, 7626 },
	{ 0x9793, 0x9793, PDF_CMAP_SINGLE, 20757 },
	{ 0x9794, 0x9794, PDF_CMAP_SINGLE, 7628 },
	{ 0x9795, 0x9797, PDF_CMAP_RANGE, 20758 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 3143 },
	{ 0x9799, 0x979f, PDF_CMAP_RANGE, 20761 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 2302 },
	{ 0x97a1, 0x97a2, PDF_CMAP_RANGE, 20768 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 7631 },
	{ 0x97a4, 0x97a5, PDF_CMAP_RANGE, 20770 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 9877 },
	{ 0x97a7, 0x97aa, PDF_CMAP_RANGE, 20772 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 7630 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 20776 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 1095 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 20777 },
	{ 0x97af, 0x97af, PDF_CMAP_SINGLE, 7629 },
	{ 0x97b0, 0x97b1, PDF_CMAP_RANGE, 20778 },
	{ 0x97b2, 0x97b2, PDF_CMAP_SINGLE, 7632 },
	{ 0x97b3, 0x97b3, PDF_CMAP_SINGLE, 20780 },
	{ 0x97b4, 0x97b4, PDF_CMAP_SINGLE, 7633 },
	{ 0x97b5, 0x97bc, PDF_CMAP_RANGE, 20781 },
	{ 0x97bd, 0x97bd, PDF_CMAP_SINGLE, 9822 },
	{ 0x97be, 0x97c2, PDF_CMAP_RANGE, 20789 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 9821 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 20794 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 9874 },
	{ 0x97c7, 0x97c8, PDF_CMAP_RANGE, 20796 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 9823 },
	{ 0x97ca, 0x97ca, PDF_CMAP_SINGLE, 20798 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 8589 },
	{ 0x97cc, 0x97cc, PDF_CMAP_SINGLE, 8445 },
	{ 0x97cd, 0x97d2, PDF_CMAP_RANGE, 20799 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 8000 },
	{ 0x97d4, 0x97d8, PDF_CMAP_RANGE, 20805 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 9317 },
	{ 0x97da, 0x97db, PDF_CMAP_RANGE, 20810 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 9319 },
	{ 0x97dd, 0x97dd, PDF_CMAP_SINGLE, 20812 },
	{ 0x97de, 0x97de, PDF_CMAP_SINGLE, 9318 },
	{ 0x97df, 0x97e5, PDF_CMAP_RANGE, 20813 },
	{ 0x97e6, 0x97e6, PDF_CMAP_SINGLE, 3763 },
	{ 0x97e7, 0x97e7, PDF_CMAP_SINGLE, 3240 },
	{ 0x97e8, 0x97e8, PDF_CMAP_SINGLE, 20820 },
	{ 0x97e9, 0x97e9, PDF_CMAP_SINGLE, 1890 },
	{ 0x97ea, 0x97ec, PDF_CMAP_RANGE, 6222 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 2289 },
	{ 0x97ee, 0x97f2, PDF_CMAP_RANGE, 20821 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 4219 },
	{ 0x97f4, 0x97f4, PDF_CMAP_SINGLE, 20826 },
	{ 0x97f5, 0x97f5, PDF_CMAP_SINGLE, 4370 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 3345 },
	{ 0x97f7, 0x97fe, PDF_CMAP_RANGE, 20827 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 8649 },
	{ 0x9800, 0x9800, PDF_CMAP_SINGLE, 20835 },
	{ 0x9801, 0x9801, PDF_CMAP_SINGLE, 8713 },
	{ 0x9802, 0x9802, PDF_CMAP_SINGLE, 7881 },
	{ 0x9803, 0x9803, PDF_CMAP_SINGLE, 8425 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 20836 },
	{ 0x9805, 0x9805, PDF_CMAP_SINGLE, 8650 },
	{ 0x9806, 0x9806, PDF_CMAP_SINGLE, 8518 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 9669 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 8668 },
	{ 0x9809, 0x9809, PDF_CMAP_SINGLE, 20837 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 9310 },
	{ 0x980b, 0x980b, PDF_CMAP_SINGLE, 20838 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 8527 },
	{ 0x980d, 0x980d, PDF_CMAP_SINGLE, 20839 },
	{ 0x980e, 0x980f, PDF_CMAP_RANGE, 9670 },
	{ 0x9810, 0x9810, PDF_CMAP_SINGLE, 8769 },
	{ 0x9811, 0x9811, PDF_CMAP_SINGLE, 8586 },
	{ 0x9812, 0x9812, PDF_CMAP_SINGLE, 7727 },
	{ 0x9813, 0x9813, PDF_CMAP_SINGLE, 7900 },
	{ 0x9814, 0x9816, PDF_CMAP_RANGE, 20840 },
	{ 0x9817, 0x9817, PDF_CMAP_SINGLE, 8383 },
	{ 0x9818, 0x9818, PDF_CMAP_SINGLE, 8244 },
	{ 0x9819, 0x981b, PDF_CMAP_RANGE, 20843 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 9673 },
	{ 0x981d, 0x9820, PDF_CMAP_RANGE, 20846 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 9672 },
	{ 0x9822, 0x9823, PDF_CMAP_RANGE, 20850 },
	{ 0x9824, 0x9824, PDF_CMAP_SINGLE, 8718 },
	{ 0x9825, 0x9825, PDF_CMAP_SINGLE, 20852 },
	{ 0x9826, 0x9826, PDF_CMAP_SINGLE, 9675 },
	{ 0x9827, 0x982c, PDF_CMAP_RANGE, 20853 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 8573 },
	{ 0x982e, 0x982f, PDF_CMAP_RANGE, 20859 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 8062 },
	{ 0x9831, 0x9836, PDF_CMAP_RANGE, 20861 },
	{ 0x9837, 0x9837, PDF_CMAP_SINGLE, 9676 },
	{ 0x9838, 0x9838, PDF_CMAP_SINGLE, 8131 },
	{ 0x9839, 0x983a, PDF_CMAP_RANGE, 20867 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 8377 },
	{ 0x983c, 0x983c, PDF_CMAP_SINGLE, 20869 },
	{ 0x983d, 0x983d, PDF_CMAP_SINGLE, 8577 },
	{ 0x983e, 0x9845, PDF_CMAP_RANGE, 20870 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 8154 },
	{ 0x9847, 0x984b, PDF_CMAP_RANGE, 20878 },
	{ 0x984c, 0x984c, PDF_CMAP_SINGLE, 8563 },
	{ 0x984d, 0x984d, PDF_CMAP_SINGLE, 7905 },
	{ 0x984e, 0x984e, PDF_CMAP_SINGLE, 9677 },
	{ 0x984f, 0x9852, PDF_CMAP_RANGE, 20883 },
	{ 0x9853, 0x9853, PDF_CMAP_SINGLE, 9678 },
	{ 0x9854, 0x9854, PDF_CMAP_SINGLE, 8694 },
	{ 0x9855, 0x9857, PDF_CMAP_RANGE, 20887 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 8779 },
	{ 0x9859, 0x9859, PDF_CMAP_SINGLE, 9681 },
	{ 0x985a, 0x985a, PDF_CMAP_SINGLE, 20890 },
	{ 0x985b, 0x985b, PDF_CMAP_SINGLE, 7872 },
	{ 0x985c, 0x985d, PDF_CMAP_RANGE, 20891 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 8200 },
	{ 0x985f, 0x9861, PDF_CMAP_RANGE, 20893 },
	{ 0x9862, 0x9862, PDF_CMAP_SINGLE, 9680 },
	{ 0x9863, 0x9864, PDF_CMAP_RANGE, 20896 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 9682 },
	{ 0x9866, 0x9866, PDF_CMAP_SINGLE, 20898 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 7978 },
	{ 0x9868, 0x986a, PDF_CMAP_RANGE, 20899 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 7794 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 9683 },
	{ 0x986d, 0x986e, PDF_CMAP_RANGE, 20902 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 8638 },
	{ 0x9870, 0x9870, PDF_CMAP_SINGLE, 9684 },
	{ 0x9871, 0x9871, PDF_CMAP_SINGLE, 8260 },
	{ 0x9872, 0x9872, PDF_CMAP_SINGLE, 20904 },
	{ 0x9873, 0x9873, PDF_CMAP_SINGLE, 9679 },
	{ 0x9874, 0x9874, PDF_CMAP_SINGLE, 8435 },
	{ 0x9875, 0x9875, PDF_CMAP_SINGLE, 4154 },
	{ 0x9876, 0x9876, PDF_CMAP_SINGLE, 1508 },
	{ 0x9877, 0x9877, PDF_CMAP_SINGLE, 3175 },
	{ 0x9878, 0x9878, PDF_CMAP_SINGLE, 7136 },
	{ 0x9879, 0x9879, PDF_CMAP_SINGLE, 3931 },
	{ 0x987a, 0x987a, PDF_CMAP_SINGLE, 3496 },
	{ 0x987b, 0x987b, PDF_CMAP_SINGLE, 4022 },
	{ 0x987c, 0x987c, PDF_CMAP_SINGLE, 6183 },
	{ 0x987d, 0x987d, PDF_CMAP_SINGLE, 3736 },
	{ 0x987e, 0x987e, PDF_CMAP_SINGLE, 1828 },
	{ 0x987f, 0x987f, PDF_CMAP_SINGLE, 1560 },
	{ 0x9880, 0x9880, PDF_CMAP_SINGLE, 7137 },
	{ 0x9881, 0x9881, PDF_CMAP_SINGLE, 1007 },
	{ 0x9882, 0x9882, PDF_CMAP_SINGLE, 3521 },
	{ 0x9883, 0x9883, PDF_CMAP_SINGLE, 7138 },
	{ 0x9884, 0x9884, PDF_CMAP_SINGLE, 4327 },
	{ 0x9885, 0x9885, PDF_CMAP_SINGLE, 2644 },
	{ 0x9886, 0x9886, PDF_CMAP_SINGLE, 2613 },
	{ 0x9887, 0x9887, PDF_CMAP_SINGLE, 3043 },
	{ 0x9888, 0x9888, PDF_CMAP_SINGLE, 2272 },
	{ 0x9889, 0x9889, PDF_CMAP_SINGLE, 7139 },
	{ 0x988a, 0x988a, PDF_CMAP_SINGLE, 2120 },
	{ 0x988b, 0x988b, PDF_CMAP_SINGLE, 20905 },
	{ 0x988c, 0x988d, PDF_CMAP_RANGE, 7140 },
	{ 0x988e, 0x988e, PDF_CMAP_SINGLE, 20906 },
	{ 0x988f, 0x988f, PDF_CMAP_SINGLE, 7142 },
	{ 0x9890, 0x9890, PDF_CMAP_SINGLE, 4170 },
	{ 0x9891, 0x9891, PDF_CMAP_SINGLE, 3028 },
	{ 0x9892, 0x9892, PDF_CMAP_SINGLE, 20907 },
	{ 0x9893, 0x9893, PDF_CMAP_SINGLE, 3704 },
	{ 0x9894, 0x9894, PDF_CMAP_SINGLE, 7143 },
	{ 0x9895, 0x9895, PDF_CMAP_SINGLE, 20908 },
	{ 0x9896, 0x9896, PDF_CMAP_SINGLE, 4246 },
	{ 0x9897, 0x9897, PDF_CMAP_SINGLE, 2386 },
	{ 0x9898, 0x9898, PDF_CMAP_SINGLE, 3637 },
	{ 0x9899, 0x9899, PDF_CMAP_SINGLE, 20909 },
	{ 0x989a, 0x989b, PDF_CMAP_RANGE, 7144 },
	{ 0x989c, 0x989c, PDF_CMAP_SINGLE, 4094 },
	{ 0x989d, 0x989d, PDF_CMAP_SINGLE, 1581 },
	{ 0x989e, 0x989f, PDF_CMAP_RANGE, 7146 },
	{ 0x98a0, 0x98a0, PDF_CMAP_SINGLE, 1472 },
	{ 0x98a1, 0x98a2, PDF_CMAP_RANGE, 7148 },
	{ 0x98a3, 0x98a3, PDF_CMAP_SINGLE, 20910 },
	{ 0x98a4, 0x98a4, PDF_CMAP_SINGLE, 1219 },
	{ 0x98a5, 0x98a6, PDF_CMAP_RANGE, 7150 },
	{ 0x98a7, 0x98a7, PDF_CMAP_SINGLE, 3202 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 7936 },
	{ 0x98a9, 0x98ad, PDF_CMAP_RANGE, 20911 },
	{ 0x98ae, 0x98af, PDF_CMAP_RANGE, 9420 },
	{ 0x98b0, 0x98b0, PDF_CMAP_SINGLE, 20916 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 9880 },
	{ 0x98b2, 0x98b2, PDF_CMAP_SINGLE, 20917 },
	{ 0x98b3, 0x98b3, PDF_CMAP_SINGLE, 9848 },
	{ 0x98b4, 0x98b5, PDF_CMAP_RANGE, 20918 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 9422 },
	{ 0x98b7, 0x98bb, PDF_CMAP_RANGE, 20920 },
	{ 0x98bc, 0x98bc, PDF_CMAP_SINGLE, 9423 },
	{ 0x98bd, 0x98c3, PDF_CMAP_RANGE, 20925 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 8376 },
	{ 0x98c5, 0x98c5, PDF_CMAP_SINGLE, 20932 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 9424 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 20933 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 9425 },
	{ 0x98c9, 0x98cd, PDF_CMAP_RANGE, 20934 },
	{ 0x98ce, 0x98ce, PDF_CMAP_SINGLE, 1668 },
	{ 0x98cf, 0x98d0, PDF_CMAP_RANGE, 20939 },
	{ 0x98d1, 0x98d3, PDF_CMAP_RANGE, 6583 },
	{ 0x98d4, 0x98d4, PDF_CMAP_SINGLE, 20941 },
	{ 0x98d5, 0x98d5, PDF_CMAP_SINGLE, 6586 },
	{ 0x98d6, 0x98d7, PDF_CMAP_RANGE, 20942 },
	{ 0x98d8, 0x98d8, PDF_CMAP_SINGLE, 3021 },
	{ 0x98d9, 0x98da, PDF_CMAP_RANGE, 6587 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 7924 },
	{ 0x98dc, 0x98dd, PDF_CMAP_RANGE, 20944 },
	{ 0x98de, 0x98de, PDF_CMAP_SINGLE, 1638 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 3402 },
	{ 0x98e0, 0x98e0, PDF_CMAP_SINGLE, 9115 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 20946 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 9854 },
	{ 0x98e3, 0x98e6, PDF_CMAP_RANGE, 20947 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 5658 },
	{ 0x98e8, 0x98e8, PDF_CMAP_SINGLE, 7655 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 9117 },
	{ 0x98ea, 0x98eb, PDF_CMAP_RANGE, 9119 },
	{ 0x98ec, 0x98ec, PDF_CMAP_SINGLE, 20951 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 9121 },
	{ 0x98ee, 0x98ee, PDF_CMAP_SINGLE, 20952 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 7921 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 20953 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 8734 },
	{ 0x98f3, 0x98f3, PDF_CMAP_SINGLE, 20955 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 9122 },
	{ 0x98f5, 0x98fb, PDF_CMAP_RANGE, 20956 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 8523 },
	{ 0x98fd, 0x98fd, PDF_CMAP_SINGLE, 7734 },
	{ 0x98fe, 0x98fe, PDF_CMAP_SINGLE, 8501 },
	{ 0x98ff, 0x98ff, PDF_CMAP_SINGLE, 20963 },
	{ 0x9900, 0x9902, PDF_CMAP_RANGE, 20964 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 8109 },
	{ 0x9904, 0x9904, PDF_CMAP_SINGLE, 20967 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 7763 },
	{ 0x9906, 0x9908, PDF_CMAP_RANGE, 20968 },
	{ 0x9909, 0x9909, PDF_CMAP_SINGLE, 9123 },
	{ 0x990a, 0x990a, PDF_CMAP_SINGLE, 8707 },
	{ 0x990b, 0x990b, PDF_CMAP_SINGLE, 20971 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 7911 },
	{ 0x990d, 0x990d, PDF_CMAP_SINGLE, 7656 },
	{ 0x990e, 0x990f, PDF_CMAP_RANGE, 20972 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 1172 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 9124 },
	{ 0x9912, 0x9912, PDF_CMAP_SINGLE, 8343 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 7908 },
	{ 0x9914, 0x9917, PDF_CMAP_RANGE, 20974 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 8760 },
	{ 0x9919, 0x991a, PDF_CMAP_RANGE, 20978 },
	{ 0x991b, 0x991b, PDF_CMAP_SINGLE, 9125 },
	{ 0x991c, 0x991d, PDF_CMAP_RANGE, 20980 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 8090 },
	{ 0x991f, 0x9920, PDF_CMAP_RANGE, 20982 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 8643 },
	{ 0x9922, 0x9927, PDF_CMAP_RANGE, 20984 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 7982 },
	{ 0x9929, 0x992d, PDF_CMAP_RANGE, 20990 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7657 },
	{ 0x992f, 0x9932, PDF_CMAP_RANGE, 20995 },
	{ 0x9933, 0x9933, PDF_CMAP_SINGLE, 9116 },
	{ 0x9934, 0x9936, PDF_CMAP_RANGE, 20999 },
	{ 0x9937, 0x9937, PDF_CMAP_SINGLE, 9126 },
	{ 0x9938, 0x993b, PDF_CMAP_RANGE, 21002 },
	{ 0x993c, 0x993c, PDF_CMAP_SINGLE, 9118 },
	{ 0x993d, 0x993d, PDF_CMAP_SINGLE, 21006 },
	{ 0x993e, 0x993e, PDF_CMAP_SINGLE, 8245 },
	{ 0x993f, 0x993f, PDF_CMAP_SINGLE, 9127 },
	{ 0x9940, 0x9942, PDF_CMAP_RANGE, 21007 },
	{ 0x9943, 0x9943, PDF_CMAP_SINGLE, 9128 },
	{ 0x9944, 0x9944, PDF_CMAP_SINGLE, 21010 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 8309 },
	{ 0x9946, 0x9947, PDF_CMAP_RANGE, 21011 },
	{ 0x9948, 0x994a, PDF_CMAP_RANGE, 9129 },
	{ 0x994b, 0x994b, PDF_CMAP_SINGLE, 8171 },
	{ 0x994c, 0x994c, PDF_CMAP_SINGLE, 9132 },
	{ 0x994d, 0x9950, PDF_CMAP_RANGE, 21013 },
	{ 0x9951, 0x9951, PDF_CMAP_SINGLE, 8042 },
	{ 0x9952, 0x9952, PDF_CMAP_SINGLE, 8441 },
	{ 0x9953, 0x9953, PDF_CMAP_SINGLE, 21017 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 7659 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 7658 },
	{ 0x9956, 0x9956, PDF_CMAP_SINGLE, 21018 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 9830 },
	{ 0x9958, 0x995b, PDF_CMAP_RANGE, 21019 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 9831 },
	{ 0x995d, 0x995d, PDF_CMAP_SINGLE, 21023 },
	{ 0x995e, 0x995e, PDF_CMAP_SINGLE, 7788 },
	{ 0x995f, 0x9961, PDF_CMAP_RANGE, 21024 },
	{ 0x9962, 0x9962, PDF_CMAP_SINGLE, 9133 },
	{ 0x9963, 0x9963, PDF_CMAP_SINGLE, 5661 },
	{ 0x9964, 0x9964, PDF_CMAP_SINGLE, 21027 },
	{ 0x9965, 0x9965, PDF_CMAP_SINGLE, 2069 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 21028 },
	{ 0x9967, 0x996c, PDF_CMAP_RANGE, 5662 },
	{ 0x996d, 0x996d, PDF_CMAP_SINGLE, 1622 },
	{ 0x996e, 0x996e, PDF_CMAP_SINGLE, 4226 },
	{ 0x996f, 0x996f, PDF_CMAP_SINGLE, 2165 },
	{ 0x9970, 0x9970, PDF_CMAP_SINGLE, 3429 },
	{ 0x9971, 0x9971, PDF_CMAP_SINGLE, 1038 },
	{ 0x9972, 0x9972, PDF_CMAP_SINGLE, 3516 },
	{ 0x9973, 0x9973, PDF_CMAP_SINGLE, 21029 },
	{ 0x9974, 0x9974, PDF_CMAP_SINGLE, 5668 },
	{ 0x9975, 0x9975, PDF_CMAP_SINGLE, 1595 },
	{ 0x9976, 0x9976, PDF_CMAP_SINGLE, 3231 },
	{ 0x9977, 0x9977, PDF_CMAP_SINGLE, 5669 },
	{ 0x9978, 0x9979, PDF_CMAP_RANGE, 21030 },
	{ 0x997a, 0x997a, PDF_CMAP_SINGLE, 2201 },
	{ 0x997b, 0x997b, PDF_CMAP_SINGLE, 21032 },
	{ 0x997c, 0x997c, PDF_CMAP_SINGLE, 1126 },
	{ 0x997d, 0x997d, PDF_CMAP_SINGLE, 5670 },
	{ 0x997e, 0x997e, PDF_CMAP_SINGLE, 21033 },
	{ 0x997f, 0x997f, PDF_CMAP_SINGLE, 1589 },
	{ 0x9980, 0x9980, PDF_CMAP_SINGLE, 5671 },
	{ 0x9981, 0x9981, PDF_CMAP_SINGLE, 2876 },
	{ 0x9982, 0x9983, PDF_CMAP_RANGE, 21034 },
	{ 0x9984, 0x9984, PDF_CMAP_SINGLE, 5672 },
	{ 0x9985, 0x9985, PDF_CMAP_SINGLE, 3911 },
	{ 0x9986, 0x9986, PDF_CMAP_SINGLE, 1846 },
	{ 0x9987, 0x9987, PDF_CMAP_SINGLE, 5673 },
	{ 0x9988, 0x9988, PDF_CMAP_SINGLE, 2444 },
	{ 0x9989, 0x9989, PDF_CMAP_SINGLE, 21036 },
	{ 0x998a, 0x998a, PDF_CMAP_SINGLE, 5674 },
	{ 0x998b, 0x998b, PDF_CMAP_SINGLE, 1213 },
	{ 0x998c, 0x998c, PDF_CMAP_SINGLE, 21037 },
	{ 0x998d, 0x998d, PDF_CMAP_SINGLE, 5675 },
	{ 0x998e, 0x998e, PDF_CMAP_SINGLE, 21038 },
	{ 0x998f, 0x998f, PDF_CMAP_SINGLE, 2620 },
	{ 0x9990, 0x9991, PDF_CMAP_RANGE, 5676 },
	{ 0x9992, 0x9992, PDF_CMAP_SINGLE, 2719 },
	{ 0x9993, 0x9995, PDF_CMAP_RANGE, 5678 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 3438 },
	{ 0x9997, 0x9997, PDF_CMAP_SINGLE, 4718 },
	{ 0x9998, 0x9998, PDF_CMAP_SINGLE, 4857 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 3920 },
	{ 0x999a, 0x99a4, PDF_CMAP_RANGE, 21039 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 6955 },
	{ 0x99a6, 0x99a7, PDF_CMAP_RANGE, 21050 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 5086 },
	{ 0x99a9, 0x99ab, PDF_CMAP_RANGE, 21052 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 8301 },
	{ 0x99ad, 0x99ad, PDF_CMAP_SINGLE, 8770 },
	{ 0x99ae, 0x99ae, PDF_CMAP_SINGLE, 7938 },
	{ 0x99af, 0x99b0, PDF_CMAP_RANGE, 21055 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 8579 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 21057 },
	{ 0x99b3, 0x99b3, PDF_CMAP_SINGLE, 7813 },
	{ 0x99b4, 0x99b4, PDF_CMAP_SINGLE, 8681 },
	{ 0x99b5, 0x99c0, PDF_CMAP_RANGE, 21058 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 7767 },
	{ 0x99c2, 0x99cf, PDF_CMAP_RANGE, 21070 },
	{ 0x99d0, 0x99d0, PDF_CMAP_SINGLE, 8869 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 9223 },
	{ 0x99d2, 0x99d2, PDF_CMAP_SINGLE, 8138 },
	{ 0x99d3, 0x99d3, PDF_CMAP_SINGLE, 21084 },
	{ 0x99d4, 0x99d4, PDF_CMAP_SINGLE, 9218 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 8066 },
	{ 0x99d6, 0x99d7, PDF_CMAP_RANGE, 21085 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 9224 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 9220 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 21087 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 8497 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 21088 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 8580 },
	{ 0x99de, 0x99de, PDF_CMAP_SINGLE, 21089 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 9219 },
	{ 0x99e0, 0x99e0, PDF_CMAP_SINGLE, 21090 },
	{ 0x99e1, 0x99e1, PDF_CMAP_SINGLE, 8302 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 9227 },
	{ 0x99e3, 0x99ec, PDF_CMAP_RANGE, 21091 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 7999 },
	{ 0x99ee, 0x99f0, PDF_CMAP_RANGE, 21101 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 8295 },
	{ 0x99f2, 0x99fe, PDF_CMAP_RANGE, 21104 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 8151 },
	{ 0x9a00, 0x9a00, PDF_CMAP_SINGLE, 21117 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 7811 },
	{ 0x9a02, 0x9a04, PDF_CMAP_RANGE, 21118 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 9231 },
	{ 0x9a06, 0x9a0c, PDF_CMAP_RANGE, 21121 },
	{ 0x9a0d, 0x9a0d, PDF_CMAP_SINGLE, 9230 },
	{ 0x9a0e, 0x9a0e, PDF_CMAP_SINGLE, 8391 },
	{ 0x9a0f, 0x9a0f, PDF_CMAP_SINGLE, 9229 },
	{ 0x9a10, 0x9a15, PDF_CMAP_RANGE, 21128 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 9234 },
	{ 0x9a17, 0x9a18, PDF_CMAP_RANGE, 21134 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 8375 },
	{ 0x9a1a, 0x9a2a, PDF_CMAP_RANGE, 21136 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 9200 },
	{ 0x9a2c, 0x9a2c, PDF_CMAP_SINGLE, 21153 },
	{ 0x9a2d, 0x9a2d, PDF_CMAP_SINGLE, 9233 },
	{ 0x9a2e, 0x9a2e, PDF_CMAP_SINGLE, 9236 },
	{ 0x9a2f, 0x9a2f, PDF_CMAP_SINGLE, 21154 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 8560 },
	{ 0x9a31, 0x9a35, PDF_CMAP_RANGE, 21155 },
	{ 0x9a36, 0x9a36, PDF_CMAP_SINGLE, 9221 },
	{ 0x9a37, 0x9a37, PDF_CMAP_SINGLE, 8460 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 9237 },
	{ 0x9a39, 0x9a3d, PDF_CMAP_RANGE, 21160 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 8294 },
	{ 0x9a3f, 0x9a3f, PDF_CMAP_SINGLE, 21165 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 9042 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 9235 },
	{ 0x9a42, 0x9a42, PDF_CMAP_SINGLE, 9232 },
	{ 0x9a43, 0x9a44, PDF_CMAP_RANGE, 9238 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 8433 },
	{ 0x9a46, 0x9a49, PDF_CMAP_RANGE, 21166 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 9226 },
	{ 0x9a4b, 0x9a4c, PDF_CMAP_RANGE, 21170 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 9225 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 21172 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 9240 },
	{ 0x9a50, 0x9a54, PDF_CMAP_RANGE, 21173 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 8103 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 21178 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 8700 },
	{ 0x9a58, 0x9a59, PDF_CMAP_RANGE, 21179 },
	{ 0x9a5a, 0x9a5a, PDF_CMAP_SINGLE, 8129 },
	{ 0x9a5b, 0x9a5b, PDF_CMAP_SINGLE, 9222 },
	{ 0x9a5c, 0x9a5e, PDF_CMAP_RANGE, 21181 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 8860 },
	{ 0x9a60, 0x9a61, PDF_CMAP_RANGE, 21184 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 8270 },
	{ 0x9a63, 0x9a63, PDF_CMAP_SINGLE, 21186 },
	{ 0x9a64, 0x9a64, PDF_CMAP_SINGLE, 9242 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 9241 },
	{ 0x9a66, 0x9a69, PDF_CMAP_RANGE, 21187 },
	{ 0x9a6a, 0x9a6a, PDF_CMAP_SINGLE, 9228 },
	{ 0x9a6b, 0x9a6b, PDF_CMAP_SINGLE, 21191 },
	{ 0x9a6c, 0x9a6c, PDF_CMAP_SINGLE, 2708 },
	{ 0x9a6d, 0x9a6d, PDF_CMAP_SINGLE, 4329 },
	{ 0x9a6e, 0x9a6e, PDF_CMAP_SINGLE, 3717 },
	{ 0x9a6f, 0x9a6f, PDF_CMAP_SINGLE, 4058 },
	{ 0x9a70, 0x9a70, PDF_CMAP_SINGLE, 1280 },
	{ 0x9a71, 0x9a71, PDF_CMAP_SINGLE, 3194 },
	{ 0x9a72, 0x9a72, PDF_CMAP_SINGLE, 21192 },
	{ 0x9a73, 0x9a73, PDF_CMAP_SINGLE, 1148 },
	{ 0x9a74, 0x9a74, PDF_CMAP_SINGLE, 2662 },
	{ 0x9a75, 0x9a75, PDF_CMAP_SINGLE, 6074 },
	{ 0x9a76, 0x9a76, PDF_CMAP_SINGLE, 3410 },
	{ 0x9a77, 0x9a78, PDF_CMAP_RANGE, 6075 },
	{ 0x9a79, 0x9a79, PDF_CMAP_SINGLE, 2307 },
	{ 0x9a7a, 0x9a7a, PDF_CMAP_SINGLE, 6077 },
	{ 0x9a7b, 0x9a7b, PDF_CMAP_SINGLE, 4609 },
	{ 0x9a7c, 0x9a7c, PDF_CMAP_SINGLE, 3718 },
	{ 0x9a7d, 0x9a7d, PDF_CMAP_SINGLE, 6079 },
	{ 0x9a7e, 0x9a7e, PDF_CMAP_SINGLE, 2128 },
	{ 0x9a7f, 0x9a7f, PDF_CMAP_SINGLE, 6078 },
	{ 0x9a80, 0x9a81, PDF_CMAP_RANGE, 6080 },
	{ 0x9a82, 0x9a82, PDF_CMAP_SINGLE, 2709 },
	{ 0x9a83, 0x9a83, PDF_CMAP_SINGLE, 21193 },
	{ 0x9a84, 0x9a84, PDF_CMAP_SINGLE, 2191 },
	{ 0x9a85, 0x9a85, PDF_CMAP_SINGLE, 6082 },
	{ 0x9a86, 0x9a86, PDF_CMAP_SINGLE, 2701 },
	{ 0x9a87, 0x9a87, PDF_CMAP_SINGLE, 1886 },
	{ 0x9a88, 0x9a88, PDF_CMAP_SINGLE, 6083 },
	{ 0x9a89, 0x9a89, PDF_CMAP_SINGLE, 21194 },
	{ 0x9a8a, 0x9a8a, PDF_CMAP_SINGLE, 6084 },
	{ 0x9a8b, 0x9a8b, PDF_CMAP_SINGLE, 1271 },
	{ 0x9a8c, 0x9a8c, PDF_CMAP_SINGLE, 4114 },
	{ 0x9a8d, 0x9a8e, PDF_CMAP_RANGE, 21195 },
	{ 0x9a8f, 0x9a8f, PDF_CMAP_SINGLE, 2354 },
	{ 0x9a90, 0x9a90, PDF_CMAP_SINGLE, 6085 },
	{ 0x9a91, 0x9a91, PDF_CMAP_SINGLE, 3086 },
	{ 0x9a92, 0x9a93, PDF_CMAP_RANGE, 6086 },
	{ 0x9a94, 0x9a95, PDF_CMAP_RANGE, 21197 },
	{ 0x9a96, 0x9a96, PDF_CMAP_SINGLE, 6088 },
	{ 0x9a97, 0x9a97, PDF_CMAP_SINGLE, 3020 },
	{ 0x9a98, 0x9a98, PDF_CMAP_SINGLE, 6089 },
	{ 0x9a99, 0x9a99, PDF_CMAP_SINGLE, 21199 },
	{ 0x9a9a, 0x9a9a, PDF_CMAP_SINGLE, 3296 },
	{ 0x9a9b, 0x9a9d, PDF_CMAP_RANGE, 6090 },
	{ 0x9a9e, 0x9a9e, PDF_CMAP_SINGLE, 5941 },
	{ 0x9a9f, 0x9aa0, PDF_CMAP_RANGE, 6093 },
	{ 0x9aa1, 0x9aa1, PDF_CMAP_SINGLE, 2697 },
	{ 0x9aa2, 0x9aa3, PDF_CMAP_RANGE, 6095 },
	{ 0x9aa4, 0x9aa4, PDF_CMAP_SINGLE, 4583 },
	{ 0x9aa5, 0x9aa5, PDF_CMAP_SINGLE, 6097 },
	{ 0x9aa6, 0x9aa6, PDF_CMAP_SINGLE, 21200 },
	{ 0x9aa7, 0x9aa7, PDF_CMAP_SINGLE, 6098 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 1824 },
	{ 0x9aa9, 0x9aae, PDF_CMAP_RANGE, 21201 },
	{ 0x9aaf, 0x9aaf, PDF_CMAP_SINGLE, 7721 },
	{ 0x9ab0, 0x9ab0, PDF_CMAP_SINGLE, 7635 },
	{ 0x9ab1, 0x9ab1, PDF_CMAP_SINGLE, 7634 },
	{ 0x9ab2, 0x9ab5, PDF_CMAP_RANGE, 21207 },
	{ 0x9ab6, 0x9ab6, PDF_CMAP_SINGLE, 7638 },
	{ 0x9ab7, 0x9ab7, PDF_CMAP_SINGLE, 7636 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 1880 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 21211 },
	{ 0x9aba, 0x9aba, PDF_CMAP_SINGLE, 7639 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 21212 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 7640 },
	{ 0x9abd, 0x9abf, PDF_CMAP_RANGE, 21213 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 7642 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 7641 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 7644 },
	{ 0x9ac3, 0x9ac4, PDF_CMAP_RANGE, 21216 },
	{ 0x9ac5, 0x9ac5, PDF_CMAP_SINGLE, 7643 },
	{ 0x9ac6, 0x9aca, PDF_CMAP_RANGE, 21218 },
	{ 0x9acb, 0x9acc, PDF_CMAP_RANGE, 7645 },
	{ 0x9acd, 0x9ace, PDF_CMAP_RANGE, 21223 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 9825 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 21225 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 7647 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 9892 },
	{ 0x9ad3, 0x9ad3, PDF_CMAP_SINGLE, 3549 },
	{ 0x9ad4, 0x9ad4, PDF_CMAP_SINGLE, 8564 },
	{ 0x9ad5, 0x9ad5, PDF_CMAP_SINGLE, 9827 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 9826 },
	{ 0x9ad7, 0x9ad7, PDF_CMAP_SINGLE, 21226 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 1754 },
	{ 0x9ad9, 0x9ade, PDF_CMAP_RANGE, 21227 },
	{ 0x9adf, 0x9adf, PDF_CMAP_SINGLE, 7660 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 21233 },
	{ 0x9ae1, 0x9ae1, PDF_CMAP_SINGLE, 7661 },
	{ 0x9ae2, 0x9ae5, PDF_CMAP_RANGE, 21234 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7662 },
	{ 0x9ae7, 0x9aea, PDF_CMAP_RANGE, 21238 },
	{ 0x9aeb, 0x9aeb, PDF_CMAP_SINGLE, 7664 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 21242 },
	{ 0x9aed, 0x9aed, PDF_CMAP_SINGLE, 7666 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 9845 },
	{ 0x9aef, 0x9aef, PDF_CMAP_SINGLE, 7663 },
	{ 0x9af0, 0x9af8, PDF_CMAP_RANGE, 21243 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 7667 },
	{ 0x9afa, 0x9afa, PDF_CMAP_SINGLE, 21252 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 7665 },
	{ 0x9afc, 0x9aff, PDF_CMAP_RANGE, 21253 },
	{ 0x9b00, 0x9b02, PDF_CMAP_RANGE, 21257 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 4660 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 21260 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 8524 },
	{ 0x9b07, 0x9b07, PDF_CMAP_SINGLE, 21262 },
	{ 0x9b08, 0x9b08, PDF_CMAP_SINGLE, 7668 },
	{ 0x9b09, 0x9b0c, PDF_CMAP_RANGE, 21263 },
	{ 0x9b0d, 0x9b0d, PDF_CMAP_SINGLE, 9850 },
	{ 0x9b0e, 0x9b0e, PDF_CMAP_SINGLE, 21267 },
	{ 0x9b0f, 0x9b0f, PDF_CMAP_SINGLE, 7669 },
	{ 0x9b10, 0x9b12, PDF_CMAP_RANGE, 21268 },
	{ 0x9b13, 0x9b13, PDF_CMAP_SINGLE, 7670 },
	{ 0x9b14, 0x9b19, PDF_CMAP_RANGE, 21271 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 9888 },
	{ 0x9b1b, 0x9b1e, PDF_CMAP_RANGE, 21277 },
	{ 0x9b1f, 0x9b1f, PDF_CMAP_SINGLE, 7671 },
	{ 0x9b20, 0x9b21, PDF_CMAP_RANGE, 21281 },
	{ 0x9b22, 0x9b22, PDF_CMAP_SINGLE, 9832 },
	{ 0x9b23, 0x9b23, PDF_CMAP_SINGLE, 7672 },
	{ 0x9b24, 0x9b24, PDF_CMAP_SINGLE, 21283 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 7888 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 21284 },
	{ 0x9b27, 0x9b27, PDF_CMAP_SINGLE, 8342 },
	{ 0x9b28, 0x9b28, PDF_CMAP_SINGLE, 21285 },
	{ 0x9b29, 0x9b29, PDF_CMAP_SINGLE, 9164 },
	{ 0x9b2a, 0x9b2d, PDF_CMAP_RANGE, 21286 },
	{ 0x9b2e, 0x9b2e, PDF_CMAP_SINGLE, 9160 },
	{ 0x9b2f, 0x9b2f, PDF_CMAP_SINGLE, 5019 },
	{ 0x9b30, 0x9b30, PDF_CMAP_SINGLE, 21290 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 8766 },
	{ 0x9b32, 0x9b32, PDF_CMAP_SINGLE, 4704 },
	{ 0x9b33, 0x9b3a, PDF_CMAP_RANGE, 21291 },
	{ 0x9b3b, 0x9b3b, PDF_CMAP_SINGLE, 6003 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 1862 },
	{ 0x9b3d, 0x9b40, PDF_CMAP_RANGE, 21299 },
	{ 0x9b41, 0x9b41, PDF_CMAP_SINGLE, 2442 },
	{ 0x9b42, 0x9b42, PDF_CMAP_SINGLE, 2047 },
	{ 0x9b43, 0x9b43, PDF_CMAP_SINGLE, 7649 },
	{ 0x9b44, 0x9b44, PDF_CMAP_SINGLE, 3046 },
	{ 0x9b45, 0x9b45, PDF_CMAP_SINGLE, 7648 },
	{ 0x9b46, 0x9b46, PDF_CMAP_SINGLE, 21303 },
	{ 0x9b47, 0x9b47, PDF_CMAP_SINGLE, 7650 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 7652 },
	{ 0x9b49, 0x9b49, PDF_CMAP_SINGLE, 7651 },
	{ 0x9b4a, 0x9b4c, PDF_CMAP_RANGE, 21304 },
	{ 0x9b4d, 0x9b4d, PDF_CMAP_SINGLE, 7653 },
	{ 0x9b4e, 0x9b4e, PDF_CMAP_SINGLE, 9829 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 3785 },
	{ 0x9b50, 0x9b50, PDF_CMAP_SINGLE, 21307 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 7654 },
	{ 0x9b52, 0x9b53, PDF_CMAP_RANGE, 21308 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 2826 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 21310 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 9828 },
	{ 0x9b59, 0x9b59, PDF_CMAP_SINGLE, 21313 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 8761 },
	{ 0x9b5b, 0x9b6e, PDF_CMAP_RANGE, 21314 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 8266 },
	{ 0x9b70, 0x9b73, PDF_CMAP_RANGE, 21334 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 9759 },
	{ 0x9b75, 0x9b76, PDF_CMAP_RANGE, 21338 },
	{ 0x9b77, 0x9b77, PDF_CMAP_SINGLE, 9758 },
	{ 0x9b78, 0x9b80, PDF_CMAP_RANGE, 21340 },
	{ 0x9b81, 0x9b81, PDF_CMAP_SINGLE, 9760 },
	{ 0x9b82, 0x9b82, PDF_CMAP_SINGLE, 21349 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 9761 },
	{ 0x9b84, 0x9b8d, PDF_CMAP_RANGE, 21350 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 9762 },
	{ 0x9b8f, 0x9b8f, PDF_CMAP_SINGLE, 21360 },
	{ 0x9b90, 0x9b90, PDF_CMAP_SINGLE, 9767 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 7737 },
	{ 0x9b92, 0x9b92, PDF_CMAP_SINGLE, 9765 },
	{ 0x9b93, 0x9b99, PDF_CMAP_RANGE, 21361 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 9769 },
	{ 0x9b9b, 0x9b9c, PDF_CMAP_RANGE, 21368 },
	{ 0x9b9d, 0x9b9d, PDF_CMAP_SINGLE, 9774 },
	{ 0x9b9e, 0x9b9e, PDF_CMAP_SINGLE, 9771 },
	{ 0x9b9f, 0x9ba9, PDF_CMAP_RANGE, 21370 },
	{ 0x9baa, 0x9baa, PDF_CMAP_SINGLE, 9770 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 9773 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 21381 },
	{ 0x9bad, 0x9bad, PDF_CMAP_SINGLE, 9768 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 8633 },
	{ 0x9baf, 0x9bbf, PDF_CMAP_RANGE, 21382 },
	{ 0x9bc0, 0x9bc0, PDF_CMAP_SINGLE, 9782 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 9776 },
	{ 0x9bc2, 0x9bc6, PDF_CMAP_RANGE, 21399 },
	{ 0x9bc7, 0x9bc7, PDF_CMAP_SINGLE, 9784 },
	{ 0x9bc8, 0x9bc8, PDF_CMAP_SINGLE, 21404 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 8205 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 9783 },
	{ 0x9bcb, 0x9bd3, PDF_CMAP_RANGE, 21405 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 9797 },
	{ 0x9bd5, 0x9bd5, PDF_CMAP_SINGLE, 21414 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 9786 },
	{ 0x9bd7, 0x9bda, PDF_CMAP_RANGE, 21415 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 9795 },
	{ 0x9bdc, 0x9bdc, PDF_CMAP_SINGLE, 21419 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 9792 },
	{ 0x9bde, 0x9be0, PDF_CMAP_RANGE, 21420 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 9789 },
	{ 0x9be2, 0x9be2, PDF_CMAP_SINGLE, 9793 },
	{ 0x9be3, 0x9be3, PDF_CMAP_SINGLE, 21423 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 9790 },
	{ 0x9be5, 0x9be6, PDF_CMAP_RANGE, 21424 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 9791 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 8128 },
	{ 0x9be9, 0x9be9, PDF_CMAP_SINGLE, 21426 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 9787 },
	{ 0x9bec, 0x9bef, PDF_CMAP_RANGE, 21427 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 9794 },
	{ 0x9bf1, 0x9bf3, PDF_CMAP_RANGE, 21431 },
	{ 0x9bf4, 0x9bf4, PDF_CMAP_SINGLE, 9796 },
	{ 0x9bf5, 0x9bfc, PDF_CMAP_RANGE, 21434 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 9785 },
	{ 0x9bfe, 0x9bfe, PDF_CMAP_SINGLE, 21442 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 9804 },
	{ 0x9c00, 0x9c07, PDF_CMAP_RANGE, 21443 },
	{ 0x9c08, 0x9c08, PDF_CMAP_SINGLE, 9799 },
	{ 0x9c09, 0x9c09, PDF_CMAP_SINGLE, 9803 },
	{ 0x9c0a, 0x9c0c, PDF_CMAP_RANGE, 21451 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 9801 },
	{ 0x9c0e, 0x9c0f, PDF_CMAP_RANGE, 21454 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 9800 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 21456 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 9802 },
	{ 0x9c13, 0x9c13, PDF_CMAP_SINGLE, 8456 },
	{ 0x9c14, 0x9c1f, PDF_CMAP_RANGE, 21457 },
	{ 0x9c20, 0x9c20, PDF_CMAP_SINGLE, 9805 },
	{ 0x9c21, 0x9c22, PDF_CMAP_RANGE, 21469 },
	{ 0x9c23, 0x9c23, PDF_CMAP_SINGLE, 9780 },
	{ 0x9c24, 0x9c24, PDF_CMAP_SINGLE, 21471 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 9809 },
	{ 0x9c26, 0x9c27, PDF_CMAP_RANGE, 21472 },
	{ 0x9c28, 0x9c28, PDF_CMAP_SINGLE, 9808 },
	{ 0x9c29, 0x9c29, PDF_CMAP_SINGLE, 9810 },
	{ 0x9c2a, 0x9c2c, PDF_CMAP_RANGE, 21474 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 9807 },
	{ 0x9c2e, 0x9c30, PDF_CMAP_RANGE, 21477 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 9778 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 9806 },
	{ 0x9c33, 0x9c33, PDF_CMAP_SINGLE, 9811 },
	{ 0x9c34, 0x9c34, PDF_CMAP_SINGLE, 21480 },
	{ 0x9c35, 0x9c35, PDF_CMAP_SINGLE, 9815 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 21481 },
	{ 0x9c37, 0x9c37, PDF_CMAP_SINGLE, 9781 },
	{ 0x9c38, 0x9c38, PDF_CMAP_SINGLE, 21482 },
	{ 0x9c39, 0x9c39, PDF_CMAP_SINGLE, 9779 },
	{ 0x9c3a, 0x9c3a, PDF_CMAP_SINGLE, 21483 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 9814 },
	{ 0x9c3c, 0x9c3d, PDF_CMAP_RANGE, 21484 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 9812 },
	{ 0x9c3f, 0x9c44, PDF_CMAP_RANGE, 21486 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 9816 },
	{ 0x9c46, 0x9c47, PDF_CMAP_RANGE, 21492 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 9813 },
	{ 0x9c49, 0x9c49, PDF_CMAP_SINGLE, 7757 },
	{ 0x9c4a, 0x9c51, PDF_CMAP_RANGE, 21494 },
	{ 0x9c52, 0x9c52, PDF_CMAP_SINGLE, 9819 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 21502 },
	{ 0x9c54, 0x9c54, PDF_CMAP_SINGLE, 9818 },
	{ 0x9c55, 0x9c55, PDF_CMAP_SINGLE, 21503 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 9817 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 8238 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 9775 },
	{ 0x9c59, 0x9c5c, PDF_CMAP_RANGE, 21504 },
	{ 0x9c5d, 0x9c5d, PDF_CMAP_SINGLE, 9798 },
	{ 0x9c5e, 0x9c5e, PDF_CMAP_SINGLE, 21508 },
	{ 0x9c5f, 0x9c5f, PDF_CMAP_SINGLE, 9766 },
	{ 0x9c60, 0x9c66, PDF_CMAP_RANGE, 21509 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 9820 },
	{ 0x9c68, 0x9c6c, PDF_CMAP_RANGE, 21516 },
	{ 0x9c6d, 0x9c6d, PDF_CMAP_SINGLE, 9772 },
	{ 0x9c6e, 0x9c77, PDF_CMAP_RANGE, 21521 },
	{ 0x9c78, 0x9c78, PDF_CMAP_SINGLE, 9763 },
	{ 0x9c79, 0x9c79, PDF_CMAP_SINGLE, 21531 },
	{ 0x9c7a, 0x9c7a, PDF_CMAP_SINGLE, 9777 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 21532 },
	{ 0x9c7c, 0x9c7c, PDF_CMAP_SINGLE, 4296 },
	{ 0x9c7d, 0x9c7e, PDF_CMAP_RANGE, 21533 },
	{ 0x9c7f, 0x9c7f, PDF_CMAP_SINGLE, 7561 },
	{ 0x9c80, 0x9c80, PDF_CMAP_SINGLE, 21535 },
	{ 0x9c81, 0x9c81, PDF_CMAP_SINGLE, 2650 },
	{ 0x9c82, 0x9c82, PDF_CMAP_SINGLE, 7562 },
	{ 0x9c83, 0x9c84, PDF_CMAP_RANGE, 21536 },
	{ 0x9c85, 0x9c88, PDF_CMAP_RANGE, 7563 },
	{ 0x9c89, 0x9c8a, PDF_CMAP_RANGE, 21538 },
	{ 0x9c8b, 0x9c8b, PDF_CMAP_SINGLE, 7568 },
	{ 0x9c8c, 0x9c8c, PDF_CMAP_SINGLE, 21540 },
	{ 0x9c8d, 0x9c8d, PDF_CMAP_SINGLE, 1044 },
	{ 0x9c8e, 0x9c8e, PDF_CMAP_SINGLE, 7569 },
	{ 0x9c8f, 0x9c8f, PDF_CMAP_SINGLE, 21541 },
	{ 0x9c90, 0x9c92, PDF_CMAP_RANGE, 7570 },
	{ 0x9c93, 0x9c93, PDF_CMAP_SINGLE, 21542 },
	{ 0x9c94, 0x9c95, PDF_CMAP_RANGE, 7573 },
	{ 0x9c96, 0x9c99, PDF_CMAP_RANGE, 21543 },
	{ 0x9c9a, 0x9c9b, PDF_CMAP_RANGE, 7575 },
	{ 0x9c9c, 0x9c9c, PDF_CMAP_SINGLE, 3895 },
	{ 0x9c9d, 0x9c9d, PDF_CMAP_SINGLE, 21547 },
	{ 0x9c9e, 0x9ca3, PDF_CMAP_RANGE, 7577 },
	{ 0x9ca4, 0x9ca4, PDF_CMAP_SINGLE, 2523 },
	{ 0x9ca5, 0x9ca9, PDF_CMAP_RANGE, 7583 },
	{ 0x9caa, 0x9caa, PDF_CMAP_SINGLE, 21548 },
	{ 0x9cab, 0x9cab, PDF_CMAP_SINGLE, 7588 },
	{ 0x9cac, 0x9cac, PDF_CMAP_SINGLE, 21549 },
	{ 0x9cad, 0x9cae, PDF_CMAP_RANGE, 7589 },
	{ 0x9caf, 0x9caf, PDF_CMAP_SINGLE, 21550 },
	{ 0x9cb0, 0x9cb7, PDF_CMAP_RANGE, 7591 },
	{ 0x9cb8, 0x9cb8, PDF_CMAP_SINGLE, 2263 },
	{ 0x9cb9, 0x9cb9, PDF_CMAP_SINGLE, 21551 },
	{ 0x9cba, 0x9cbd, PDF_CMAP_RANGE, 7599 },
	{ 0x9cbe, 0x9cc2, PDF_CMAP_RANGE, 21552 },
	{ 0x9cc3, 0x9cc3, PDF_CMAP_SINGLE, 3285 },
	{ 0x9cc4, 0x9cc7, PDF_CMAP_RANGE, 7603 },
	{ 0x9cc8, 0x9cc9, PDF_CMAP_RANGE, 21557 },
	{ 0x9cca, 0x9cd0, PDF_CMAP_RANGE, 7607 },
	{ 0x9cd1, 0x9cd2, PDF_CMAP_RANGE, 21559 },
	{ 0x9cd3, 0x9cd5, PDF_CMAP_RANGE, 7614 },
	{ 0x9cd6, 0x9cd6, PDF_CMAP_SINGLE, 1111 },
	{ 0x9cd7, 0x9cd9, PDF_CMAP_RANGE, 7617 },
	{ 0x9cda, 0x9cdb, PDF_CMAP_RANGE, 21561 },
	{ 0x9cdc, 0x9cdd, PDF_CMAP_RANGE, 7620 },
	{ 0x9cde, 0x9cde, PDF_CMAP_SINGLE, 2596 },
	{ 0x9cdf, 0x9cdf, PDF_CMAP_SINGLE, 7622 },
	{ 0x9ce0, 0x9ce1, PDF_CMAP_RANGE, 21563 },
	{ 0x9ce2, 0x9ce2, PDF_CMAP_SINGLE, 7623 },
	{ 0x9ce3, 0x9ce4, PDF_CMAP_RANGE, 21565 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 8348 },
	{ 0x9ce6, 0x9ce8, PDF_CMAP_RANGE, 21567 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 9600 },
	{ 0x9cea, 0x9ceb, PDF_CMAP_RANGE, 21570 },
	{ 0x9cec, 0x9cec, PDF_CMAP_SINGLE, 8924 },
	{ 0x9ced, 0x9cf2, PDF_CMAP_RANGE, 21572 },
	{ 0x9cf3, 0x9cf3, PDF_CMAP_SINGLE, 7941 },
	{ 0x9cf4, 0x9cf4, PDF_CMAP_SINGLE, 8331 },
	{ 0x9cf5, 0x9cf5, PDF_CMAP_SINGLE, 21578 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 9601 },
	{ 0x9cf7, 0x9cff, PDF_CMAP_RANGE, 21579 },
	{ 0x9d00, 0x9d05, PDF_CMAP_RANGE, 21588 },
	{ 0x9d06, 0x9d06, PDF_CMAP_SINGLE, 9603 },
	{ 0x9d07, 0x9d07, PDF_CMAP_SINGLE, 9602 },
	{ 0x9d08, 0x9d08, PDF_CMAP_SINGLE, 21594 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 8686 },
	{ 0x9d0a, 0x9d14, PDF_CMAP_RANGE, 21595 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 8578 },
	{ 0x9d16, 0x9d1a, PDF_CMAP_RANGE, 21606 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 8771 },
	{ 0x9d1c, 0x9d1c, PDF_CMAP_SINGLE, 21611 },
	{ 0x9d1d, 0x9d1d, PDF_CMAP_SINGLE, 9607 },
	{ 0x9d1e, 0x9d1e, PDF_CMAP_SINGLE, 21612 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 9608 },
	{ 0x9d20, 0x9d22, PDF_CMAP_RANGE, 21613 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 9604 },
	{ 0x9d24, 0x9d25, PDF_CMAP_RANGE, 21616 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 8701 },
	{ 0x9d27, 0x9d27, PDF_CMAP_SINGLE, 21618 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 8687 },
	{ 0x9d29, 0x9d2e, PDF_CMAP_RANGE, 21619 },
	{ 0x9d2f, 0x9d2f, PDF_CMAP_SINGLE, 9610 },
	{ 0x9d30, 0x9d30, PDF_CMAP_SINGLE, 9612 },
	{ 0x9d31, 0x9d3a, PDF_CMAP_RANGE, 21625 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 8007 },
	{ 0x9d3c, 0x9d3e, PDF_CMAP_RANGE, 21635 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 7965 },
	{ 0x9d40, 0x9d41, PDF_CMAP_RANGE, 21638 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 9613 },
	{ 0x9d43, 0x9d50, PDF_CMAP_RANGE, 21640 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 8144 },
	{ 0x9d52, 0x9d52, PDF_CMAP_SINGLE, 9618 },
	{ 0x9d53, 0x9d53, PDF_CMAP_SINGLE, 9615 },
	{ 0x9d54, 0x9d5b, PDF_CMAP_RANGE, 21654 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 9620 },
	{ 0x9d5d, 0x9d5d, PDF_CMAP_SINGLE, 7904 },
	{ 0x9d5e, 0x9d5f, PDF_CMAP_RANGE, 21662 },
	{ 0x9d60, 0x9d60, PDF_CMAP_SINGLE, 9617 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 9621 },
	{ 0x9d62, 0x9d69, PDF_CMAP_RANGE, 21664 },
	{ 0x9d6a, 0x9d6a, PDF_CMAP_SINGLE, 9623 },
	{ 0x9d6b, 0x9d6b, PDF_CMAP_SINGLE, 21672 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 8374 },
	{ 0x9d6d, 0x9d6e, PDF_CMAP_RANGE, 21673 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 9624 },
	{ 0x9d70, 0x9d71, PDF_CMAP_RANGE, 21675 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 8438 },
	{ 0x9d73, 0x9d86, PDF_CMAP_RANGE, 21677 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 9605 },
	{ 0x9d88, 0x9d88, PDF_CMAP_SINGLE, 21697 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 9625 },
	{ 0x9d8a, 0x9d92, PDF_CMAP_RANGE, 21698 },
	{ 0x9d93, 0x9d93, PDF_CMAP_SINGLE, 9622 },
	{ 0x9d94, 0x9d97, PDF_CMAP_RANGE, 21707 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 9626 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 21711 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 9627 },
	{ 0x9d9b, 0x9da4, PDF_CMAP_RANGE, 21712 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 9629 },
	{ 0x9da6, 0x9da8, PDF_CMAP_RANGE, 21722 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 9630 },
	{ 0x9daa, 0x9dae, PDF_CMAP_RANGE, 21725 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 9036 },
	{ 0x9db0, 0x9db3, PDF_CMAP_RANGE, 21730 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 8004 },
	{ 0x9db5, 0x9dba, PDF_CMAP_RANGE, 21734 },
	{ 0x9dbb, 0x9dbb, PDF_CMAP_SINGLE, 9824 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 9632 },
	{ 0x9dbd, 0x9dbf, PDF_CMAP_RANGE, 21740 },
	{ 0x9dc0, 0x9dc0, PDF_CMAP_SINGLE, 9628 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 21743 },
	{ 0x9dc2, 0x9dc2, PDF_CMAP_SINGLE, 9631 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 21744 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 8044 },
	{ 0x9dc5, 0x9dd2, PDF_CMAP_RANGE, 21745 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 9634 },
	{ 0x9dd4, 0x9dd6, PDF_CMAP_RANGE, 21759 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 8366 },
	{ 0x9dd8, 0x9dd8, PDF_CMAP_SINGLE, 21762 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 9611 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 9635 },
	{ 0x9ddb, 0x9de4, PDF_CMAP_RANGE, 21763 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 9609 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 9637 },
	{ 0x9de7, 0x9dee, PDF_CMAP_RANGE, 21773 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 9636 },
	{ 0x9df0, 0x9df1, PDF_CMAP_RANGE, 21781 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 9638 },
	{ 0x9df3, 0x9df3, PDF_CMAP_SINGLE, 9619 },
	{ 0x9df4, 0x9df7, PDF_CMAP_RANGE, 21783 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 9639 },
	{ 0x9df9, 0x9df9, PDF_CMAP_SINGLE, 8738 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 9641 },
	{ 0x9dfb, 0x9dff, PDF_CMAP_RANGE, 21787 },
	{ 0x9e00, 0x9e0b, PDF_CMAP_RANGE, 21792 },
	{ 0x9e0c, 0x9e0c, PDF_CMAP_SINGLE, 9640 },
	{ 0x9e0d, 0x9e14, PDF_CMAP_RANGE, 21804 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 9606 },
	{ 0x9e16, 0x9e19, PDF_CMAP_RANGE, 21812 },
	{ 0x9e1a, 0x9e1a, PDF_CMAP_SINGLE, 9633 },
	{ 0x9e1b, 0x9e1b, PDF_CMAP_SINGLE, 9642 },
	{ 0x9e1c, 0x9e1c, PDF_CMAP_SINGLE, 21816 },
	{ 0x9e1d, 0x9e1d, PDF_CMAP_SINGLE, 9616 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 9614 },
	{ 0x9e1f, 0x9e1f, PDF_CMAP_SINGLE, 2900 },
	{ 0x9e20, 0x9e20, PDF_CMAP_SINGLE, 6965 },
	{ 0x9e21, 0x9e21, PDF_CMAP_SINGLE, 2073 },
	{ 0x9e22, 0x9e22, PDF_CMAP_SINGLE, 6966 },
	{ 0x9e23, 0x9e23, PDF_CMAP_SINGLE, 2814 },
	{ 0x9e24, 0x9e24, PDF_CMAP_SINGLE, 21817 },
	{ 0x9e25, 0x9e25, PDF_CMAP_SINGLE, 2937 },
	{ 0x9e26, 0x9e26, PDF_CMAP_SINGLE, 4068 },
	{ 0x9e27, 0x9e27, PDF_CMAP_SINGLE, 21818 },
	{ 0x9e28, 0x9e2c, PDF_CMAP_RANGE, 6967 },
	{ 0x9e2d, 0x9e2d, PDF_CMAP_SINGLE, 4069 },
	{ 0x9e2e, 0x9e2e, PDF_CMAP_SINGLE, 21819 },
	{ 0x9e2f, 0x9e2f, PDF_CMAP_SINGLE, 4117 },
	{ 0x9e30, 0x9e30, PDF_CMAP_SINGLE, 21820 },
	{ 0x9e31, 0x9e31, PDF_CMAP_SINGLE, 6973 },
	{ 0x9e32, 0x9e32, PDF_CMAP_SINGLE, 6972 },
	{ 0x9e33, 0x9e33, PDF_CMAP_SINGLE, 4330 },
	{ 0x9e34, 0x9e34, PDF_CMAP_SINGLE, 21821 },
	{ 0x9e35, 0x9e35, PDF_CMAP_SINGLE, 3715 },
	{ 0x9e36, 0x9e36, PDF_CMAP_SINGLE, 6974 },
	{ 0x9e37, 0x9e37, PDF_CMAP_SINGLE, 6976 },
	{ 0x9e38, 0x9e38, PDF_CMAP_SINGLE, 6975 },
	{ 0x9e39, 0x9e3a, PDF_CMAP_RANGE, 6977 },
	{ 0x9e3b, 0x9e3c, PDF_CMAP_RANGE, 21822 },
	{ 0x9e3d, 0x9e3d, PDF_CMAP_SINGLE, 1766 },
	{ 0x9e3e, 0x9e3e, PDF_CMAP_SINGLE, 6979 },
	{ 0x9e3f, 0x9e3f, PDF_CMAP_SINGLE, 1951 },
	{ 0x9e40, 0x9e40, PDF_CMAP_SINGLE, 21824 },
	{ 0x9e41, 0x9e42, PDF_CMAP_RANGE, 6980 },
	{ 0x9e43, 0x9e43, PDF_CMAP_SINGLE, 2328 },
	{ 0x9e44, 0x9e44, PDF_CMAP_SINGLE, 6982 },
	{ 0x9e45, 0x9e45, PDF_CMAP_SINGLE, 1579 },
	{ 0x9e46, 0x9e49, PDF_CMAP_RANGE, 6983 },
	{ 0x9e4a, 0x9e4a, PDF_CMAP_SINGLE, 3216 },
	{ 0x9e4b, 0x9e4c, PDF_CMAP_RANGE, 6987 },
	{ 0x9e4d, 0x9e4d, PDF_CMAP_SINGLE, 21825 },
	{ 0x9e4e, 0x9e4e, PDF_CMAP_SINGLE, 6989 },
	{ 0x9e4f, 0x9e4f, PDF_CMAP_SINGLE, 2997 },
	{ 0x9e50, 0x9e50, PDF_CMAP_SINGLE, 21826 },
	{ 0x9e51, 0x9e51, PDF_CMAP_SINGLE, 6990 },
	{ 0x9e52, 0x9e54, PDF_CMAP_RANGE, 21827 },
	{ 0x9e55, 0x9e55, PDF_CMAP_SINGLE, 6991 },
	{ 0x9e56, 0x9e56, PDF_CMAP_SINGLE, 21830 },
	{ 0x9e57, 0x9e57, PDF_CMAP_SINGLE, 6992 },
	{ 0x9e58, 0x9e58, PDF_CMAP_SINGLE, 7637 },
	{ 0x9e59, 0x9e59, PDF_CMAP_SINGLE, 21831 },
	{ 0x9e5a, 0x9e5c, PDF_CMAP_RANGE, 6993 },
	{ 0x9e5d, 0x9e5d, PDF_CMAP_SINGLE, 21832 },
	{ 0x9e5e, 0x9e5e, PDF_CMAP_SINGLE, 6996 },
	{ 0x9e5f, 0x9e62, PDF_CMAP_RANGE, 21833 },
	{ 0x9e63, 0x9e63, PDF_CMAP_SINGLE, 6997 },
	{ 0x9e64, 0x9e64, PDF_CMAP_SINGLE, 1934 },
	{ 0x9e65, 0x9e65, PDF_CMAP_SINGLE, 21837 },
	{ 0x9e66, 0x9e6c, PDF_CMAP_RANGE, 6998 },
	{ 0x9e6d, 0x9e6d, PDF_CMAP_SINGLE, 7006 },
	{ 0x9e6e, 0x9e6f, PDF_CMAP_RANGE, 21838 },
	{ 0x9e70, 0x9e70, PDF_CMAP_SINGLE, 4234 },
	{ 0x9e71, 0x9e71, PDF_CMAP_SINGLE, 7005 },
	{ 0x9e72, 0x9e72, PDF_CMAP_SINGLE, 21840 },
	{ 0x9e73, 0x9e73, PDF_CMAP_SINGLE, 7007 },
	{ 0x9e74, 0x9e74, PDF_CMAP_SINGLE, 21841 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 9864 },
	{ 0x9e76, 0x9e78, PDF_CMAP_RANGE, 21842 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 9886 },
	{ 0x9e7a, 0x9e7a, PDF_CMAP_SINGLE, 9723 },
	{ 0x9e7b, 0x9e7b, PDF_CMAP_SINGLE, 21845 },
	{ 0x9e7c, 0x9e7c, PDF_CMAP_SINGLE, 8076 },
	{ 0x9e7d, 0x9e7d, PDF_CMAP_SINGLE, 8692 },
	{ 0x9e7e, 0x9e7e, PDF_CMAP_SINGLE, 7446 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 2656 },
	{ 0x9e80, 0x9e81, PDF_CMAP_RANGE, 21846 },
	{ 0x9e82, 0x9e82, PDF_CMAP_SINGLE, 7676 },
	{ 0x9e83, 0x9e86, PDF_CMAP_RANGE, 21848 },
	{ 0x9e87, 0x9e88, PDF_CMAP_RANGE, 7677 },
	{ 0x9e89, 0x9e8a, PDF_CMAP_RANGE, 21852 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 7679 },
	{ 0x9e8c, 0x9e91, PDF_CMAP_RANGE, 21854 },
	{ 0x9e92, 0x9e92, PDF_CMAP_SINGLE, 7680 },
	{ 0x9e93, 0x9e93, PDF_CMAP_SINGLE, 2651 },
	{ 0x9e94, 0x9e96, PDF_CMAP_RANGE, 21860 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 8207 },
	{ 0x9e98, 0x9e9c, PDF_CMAP_RANGE, 21863 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 7682 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 21868 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 7683 },
	{ 0x9ea0, 0x9ea4, PDF_CMAP_RANGE, 21869 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 8305 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 2714 },
	{ 0x9ea7, 0x9ea8, PDF_CMAP_RANGE, 21874 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 9719 },
	{ 0x9eaa, 0x9eae, PDF_CMAP_RANGE, 21876 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 9878 },
	{ 0x9eb0, 0x9eb3, PDF_CMAP_RANGE, 21881 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 7406 },
	{ 0x9eb5, 0x9eb5, PDF_CMAP_SINGLE, 9871 },
	{ 0x9eb6, 0x9eb7, PDF_CMAP_RANGE, 21885 },
	{ 0x9eb8, 0x9eb8, PDF_CMAP_SINGLE, 7405 },
	{ 0x9eb9, 0x9eba, PDF_CMAP_RANGE, 21887 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 2704 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 21889 },
	{ 0x9ebd, 0x9ebe, PDF_CMAP_RANGE, 7673 },
	{ 0x9ebf, 0x9ec3, PDF_CMAP_RANGE, 21890 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 2011 },
	{ 0x9ec5, 0x9ec8, PDF_CMAP_RANGE, 21895 },
	{ 0x9ec9, 0x9ec9, PDF_CMAP_SINGLE, 4856 },
	{ 0x9eca, 0x9ecb, PDF_CMAP_RANGE, 21899 },
	{ 0x9ecc, 0x9ecc, PDF_CMAP_SINGLE, 8922 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 3465 },
	{ 0x9ece, 0x9ece, PDF_CMAP_SINGLE, 2515 },
	{ 0x9ecf, 0x9ecf, PDF_CMAP_SINGLE, 6954 },
	{ 0x9ed0, 0x9ed0, PDF_CMAP_SINGLE, 21901 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 1937 },
	{ 0x9ed2, 0x9ed3, PDF_CMAP_RANGE, 21902 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 3114 },
	{ 0x9ed5, 0x9ed7, PDF_CMAP_RANGE, 21904 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 2831 },
	{ 0x9ed9, 0x9eda, PDF_CMAP_RANGE, 21907 },
	{ 0x9edb, 0x9edd, PDF_CMAP_RANGE, 7684 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 7873 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 7688 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 7687 },
	{ 0x9ee1, 0x9ee1, PDF_CMAP_SINGLE, 21909 },
	{ 0x9ee2, 0x9ee2, PDF_CMAP_SINGLE, 7689 },
	{ 0x9ee3, 0x9ee4, PDF_CMAP_RANGE, 21910 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 7692 },
	{ 0x9ee6, 0x9ee6, PDF_CMAP_SINGLE, 21912 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 7691 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 7859 },
	{ 0x9ee9, 0x9ee9, PDF_CMAP_SINGLE, 7690 },
	{ 0x9eea, 0x9eea, PDF_CMAP_SINGLE, 7693 },
	{ 0x9eeb, 0x9eee, PDF_CMAP_RANGE, 21913 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 7694 },
	{ 0x9ef0, 0x9ef1, PDF_CMAP_RANGE, 21917 },
	{ 0x9ef2, 0x9ef2, PDF_CMAP_SINGLE, 9834 },
	{ 0x9ef3, 0x9ef3, PDF_CMAP_SINGLE, 21919 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 9866 },
	{ 0x9ef5, 0x9ef6, PDF_CMAP_RANGE, 21920 },
	{ 0x9ef7, 0x9ef7, PDF_CMAP_SINGLE, 9833 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 21922 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 6741 },
	{ 0x9efa, 0x9efa, PDF_CMAP_SINGLE, 21923 },
	{ 0x9efb, 0x9efc, PDF_CMAP_RANGE, 6742 },
	{ 0x9efd, 0x9efd, PDF_CMAP_SINGLE, 9752 },
	{ 0x9efe, 0x9efe, PDF_CMAP_SINGLE, 7542 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 9753 },
	{ 0x9f00, 0x9f08, PDF_CMAP_RANGE, 21924 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 9754 },
	{ 0x9f0a, 0x9f0a, PDF_CMAP_SINGLE, 21933 },
	{ 0x9f0b, 0x9f0b, PDF_CMAP_SINGLE, 7543 },
	{ 0x9f0c, 0x9f0c, PDF_CMAP_SINGLE, 21934 },
	{ 0x9f0d, 0x9f0d, PDF_CMAP_SINGLE, 7544 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 1509 },
	{ 0x9f0f, 0x9f0f, PDF_CMAP_SINGLE, 21935 },
	{ 0x9f10, 0x9f10, PDF_CMAP_SINGLE, 4724 },
	{ 0x9f11, 0x9f12, PDF_CMAP_RANGE, 21936 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 1821 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 21938 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 9843 },
	{ 0x9f16, 0x9f16, PDF_CMAP_SINGLE, 21939 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 4721 },
	{ 0x9f18, 0x9f18, PDF_CMAP_SINGLE, 21940 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 5087 },
	{ 0x9f1a, 0x9f1f, PDF_CMAP_RANGE, 21941 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 3466 },
	{ 0x9f21, 0x9f21, PDF_CMAP_SINGLE, 21947 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 7695 },
	{ 0x9f23, 0x9f2b, PDF_CMAP_RANGE, 21948 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7696 },
	{ 0x9f2d, 0x9f2e, PDF_CMAP_RANGE, 21957 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 7697 },
	{ 0x9f30, 0x9f36, PDF_CMAP_RANGE, 21959 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 7699 },
	{ 0x9f38, 0x9f38, PDF_CMAP_SINGLE, 21966 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 7698 },
	{ 0x9f3a, 0x9f3a, PDF_CMAP_SINGLE, 21967 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 1072 },
	{ 0x9f3c, 0x9f3c, PDF_CMAP_SINGLE, 21968 },
	{ 0x9f3d, 0x9f3e, PDF_CMAP_RANGE, 7700 },
	{ 0x9f3f, 0x9f43, PDF_CMAP_RANGE, 21969 },
	{ 0x9f44, 0x9f44, PDF_CMAP_SINGLE, 7702 },
	{ 0x9f45, 0x9f49, PDF_CMAP_RANGE, 21974 },
	{ 0x9f4a, 0x9f4a, PDF_CMAP_SINGLE, 8390 },
	{ 0x9f4b, 0x9f4b, PDF_CMAP_SINGLE, 8811 },
	{ 0x9f4c, 0x9f4e, PDF_CMAP_RANGE, 21979 },
	{ 0x9f4f, 0x9f4f, PDF_CMAP_SINGLE, 9427 },
	{ 0x9f50, 0x9f50, PDF_CMAP_SINGLE, 3082 },
	{ 0x9f51, 0x9f51, PDF_CMAP_SINGLE, 6594 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7814 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 21982 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 9743 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 21983 },
	{ 0x9f59, 0x9f59, PDF_CMAP_SINGLE, 9745 },
	{ 0x9f5a, 0x9f5b, PDF_CMAP_RANGE, 21987 },
	{ 0x9f5c, 0x9f5c, PDF_CMAP_SINGLE, 9747 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 21989 },
	{ 0x9f5f, 0x9f5f, PDF_CMAP_SINGLE, 9744 },
	{ 0x9f60, 0x9f60, PDF_CMAP_SINGLE, 9746 },
	{ 0x9f61, 0x9f61, PDF_CMAP_SINGLE, 8240 },
	{ 0x9f62, 0x9f62, PDF_CMAP_SINGLE, 21991 },
	{ 0x9f63, 0x9f63, PDF_CMAP_SINGLE, 9841 },
	{ 0x9f64, 0x9f65, PDF_CMAP_RANGE, 21992 },
	{ 0x9f66, 0x9f66, PDF_CMAP_SINGLE, 9748 },
	{ 0x9f67, 0x9f69, PDF_CMAP_RANGE, 21994 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 9750 },
	{ 0x9f6b, 0x9f6b, PDF_CMAP_SINGLE, 21997 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 9749 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 21998 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 8434 },
	{ 0x9f73, 0x9f76, PDF_CMAP_RANGE, 22003 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 9751 },
	{ 0x9f78, 0x9f7e, PDF_CMAP_RANGE, 22007 },
	{ 0x9f7f, 0x9f7f, PDF_CMAP_SINGLE, 1282 },
	{ 0x9f80, 0x9f80, PDF_CMAP_SINGLE, 7533 },
	{ 0x9f81, 0x9f82, PDF_CMAP_RANGE, 22014 },
	{ 0x9f83, 0x9f83, PDF_CMAP_SINGLE, 7534 },
	{ 0x9f84, 0x9f84, PDF_CMAP_SINGLE, 2605 },
	{ 0x9f85, 0x9f8a, PDF_CMAP_RANGE, 7535 },
	{ 0x9f8b, 0x9f8b, PDF_CMAP_SINGLE, 3198 },
	{ 0x9f8c, 0x9f8c, PDF_CMAP_SINGLE, 7541 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 8247 },
	{ 0x9f8e, 0x9f8f, PDF_CMAP_RANGE, 22016 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 8371 },
	{ 0x9f91, 0x9f93, PDF_CMAP_RANGE, 22018 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 7970 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 9453 },
	{ 0x9f96, 0x9f98, PDF_CMAP_RANGE, 22021 },
	{ 0x9f99, 0x9f99, PDF_CMAP_SINGLE, 2627 },
	{ 0x9f9a, 0x9f9a, PDF_CMAP_SINGLE, 1793 },
	{ 0x9f9b, 0x9f9b, PDF_CMAP_SINGLE, 6740 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 7988 },
	{ 0x9f9d, 0x9f9e, PDF_CMAP_RANGE, 22024 },
	{ 0x9f9f, 0x9f9f, PDF_CMAP_SINGLE, 1859 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 4851 },
	{ 0x9fa1, 0x9fa5, PDF_CMAP_RANGE, 22026 },
	{ 0x9fb4, 0x9fb4, PDF_CMAP_SINGLE, 22056 },
	{ 0x9fb5, 0x9fb5, PDF_CMAP_SINGLE, 22064 },
	{ 0x9fb6, 0x9fb7, PDF_CMAP_RANGE, 22069 },
	{ 0x9fb8, 0x9fb8, PDF_CMAP_SINGLE, 22076 },
	{ 0x9fb9, 0x9fb9, PDF_CMAP_SINGLE, 22093 },
	{ 0x9fba, 0x9fba, PDF_CMAP_SINGLE, 22110 },
	{ 0x9fbb, 0x9fbb, PDF_CMAP_SINGLE, 22126 },
	{ 0xa000, 0xa0ff, PDF_CMAP_RANGE, 29064 },
	{ 0xa100, 0xa1ff, PDF_CMAP_RANGE, 29320 },
	{ 0xa200, 0xa2ff, PDF_CMAP_RANGE, 29576 },
	{ 0xa300, 0xa3ff, PDF_CMAP_RANGE, 29832 },
	{ 0xa400, 0xa48c, PDF_CMAP_RANGE, 30088 },
	{ 0xa490, 0xa4c6, PDF_CMAP_RANGE, 30229 },
	{ 0xdc87, 0xdc87, PDF_CMAP_SINGLE, 22048 },
	{ 0xdc89, 0xdc89, PDF_CMAP_SINGLE, 22049 },
	{ 0xdccc, 0xdccc, PDF_CMAP_SINGLE, 22050 },
	{ 0xdd09, 0xdd09, PDF_CMAP_SINGLE, 22093 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 22085 },
	{ 0xdd9d, 0xdd9d, PDF_CMAP_SINGLE, 22110 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 22075 },
	{ 0xddfe, 0xddfe, PDF_CMAP_SINGLE, 22111 },
	{ 0xe76c, 0xe76c, PDF_CMAP_SINGLE, 22353 },
	{ 0xe78d, 0xe793, PDF_CMAP_RANGE, 573 },
	{ 0xe794, 0xe795, PDF_CMAP_RANGE, 592 },
	{ 0xe796, 0xe796, PDF_CMAP_SINGLE, 599 },
	{ 0xe7c7, 0xe7c7, PDF_CMAP_SINGLE, 695 },
	{ 0xe7c8, 0xe7c8, PDF_CMAP_SINGLE, 698 },
	{ 0xe7e7, 0xe7f3, PDF_CMAP_RANGE, 10059 },
	{ 0xe815, 0xe864, PDF_CMAP_RANGE, 22047 },
	{ 0xf92c, 0xf92c, PDF_CMAP_SINGLE, 22031 },
	{ 0xf979, 0xf979, PDF_CMAP_SINGLE, 2562 },
	{ 0xf995, 0xf995, PDF_CMAP_SINGLE, 16595 },
	{ 0xf9e7, 0xf9e7, PDF_CMAP_SINGLE, 8204 },
	{ 0xf9f1, 0xf9f1, PDF_CMAP_SINGLE, 20611 },
	{ 0xfa0c, 0xfa0c, PDF_CMAP_SINGLE, 4697 },
	{ 0xfa0d, 0xfa0f, PDF_CMAP_RANGE, 22032 },
	{ 0xfa11, 0xfa11, PDF_CMAP_SINGLE, 22035 },
	{ 0xfa13, 0xfa14, PDF_CMAP_RANGE, 22036 },
	{ 0xfa18, 0xfa18, PDF_CMAP_SINGLE, 22038 },
	{ 0xfa1f, 0xfa21, PDF_CMAP_RANGE, 22039 },
	{ 0xfa23, 0xfa24, PDF_CMAP_RANGE, 22042 },
	{ 0xfa27, 0xfa29, PDF_CMAP_RANGE, 22044 },
	{ 0xfe10, 0xfe10, PDF_CMAP_SINGLE, 573 },
	{ 0xfe11, 0xfe11, PDF_CMAP_SINGLE, 575 },
	{ 0xfe12, 0xfe12, PDF_CMAP_SINGLE, 574 },
	{ 0xfe13, 0xfe16, PDF_CMAP_RANGE, 576 },
	{ 0xfe17, 0xfe18, PDF_CMAP_RANGE, 592 },
	{ 0xfe19, 0xfe19, PDF_CMAP_SINGLE, 599 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 10013 },
	{ 0xfe31, 0xfe31, PDF_CMAP_SINGLE, 598 },
	{ 0xfe33, 0xfe34, PDF_CMAP_RANGE, 600 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 580 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 596 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 582 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 594 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 586 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 584 },
	{ 0xfe41, 0xfe44, PDF_CMAP_RANGE, 588 },
	{ 0xfe49, 0xfe52, PDF_CMAP_RANGE, 10027 },
	{ 0xfe54, 0xfe57, PDF_CMAP_RANGE, 10037 },
	{ 0xfe59, 0xfe66, PDF_CMAP_RANGE, 10041 },
	{ 0xfe68, 0xfe6b, PDF_CMAP_RANGE, 10055 },
	{ 0xff01, 0xff03, PDF_CMAP_RANGE, 262 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 166 },
	{ 0xff05, 0xff5d, PDF_CMAP_RANGE, 266 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 106 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 168 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 10014 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 355 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 10015 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 265 },
};

static const unsigned short pdf_cmap_UniGB_UTF16_H_table[9548] =
{
	167,22354,171,102,7713,162,127,671,
	669,675,673,693,679,677,683,681,
	687,685,9910,105,9911,107,9912,108,
	10016,7714,185,188,151,157,9921,142,
	136,135,145,159,158,130,138,152,
	147,181,180,183,182,179,178,177,
	176,174,173,22428,22047,22055,22060,4162,
	4707,4722,4709,4185,10131,1597,4867,3238,
	1592,3270,982,4765,4884,4879,2091,5017,
	1431,2543,4860,4710,4740,10778,3397,1150,
	4946,1228,5020,4283,2407,5523,3698,3414,
	5660,11565,3859,1398,2927,4656,5934,1386,
	3948,5302,3395,6004,3318,6165,1789,2093,
	2238,1732,6163,1852,5016,5293,5366,1798,
	5986,5614,5600,3983,1765,12946,3437,4518,
	6409,3795,1526,2240,1626,3821,3248,4350,
	4357,2849,3124,4536,1400,6589,3826,1073,
	2736,3430,3095,3491,2053,4611,1715,4713,
	5789,3019,4073,2916,3209,4041,4310,1832,
	3728,1733,3379,4264,3650,7110,7008,16063,
	994,3011,2808,2850,2737,3407,3398,3413,
	16587,1923,4049,2539,4592,2780,7399,7262,
	3753,4123,4309,2490,1591,7115,1593,6686,
	3261,1249,4657,4544,2297,3353,5656,4570,
	7388,3300,17826,7152,1291,4051,3995,4169,
	18908,8086,2200,4093,1825,1528,7445,7504,
	7739,1285,4668,4672,3366,7803,3980,1250,
	19731,4191,4276,19992,2522,2241,7797,8317,
	1714,2542,7545,4303,20714,1636,2795,1770,
	8589,2289,4219,8713,7936,7924,3402,3438,
	3920,8301,1824,1754,7660,7888,5019,4704,
	1862,8761,8348,9864,2656,8305,2704,21894,
	3465,1937,6741,9752,1509,1821,3466,1072,
	8390,7814,8247,7988,4851,104,10024,7703,
	9989,189,10059,22357,22390,22392,22391,22393,
	99,10019,22065,22068,22109,22108,22117,22112,
	4162,1504,10072,3070,3747,4458,3288,3336,
	3887,4696,1154,4304,10076,4698,1304,10077,
	4613,3151,4701,3415,10078,3181,1124,4156,
	1367,1514,3508,4703,1513,10082,2566,4088,
	10083,3294,4707,10084,1777,4071,5788,4559,
	1662,10087,1329,10088,2594,10089,4722,10090,
	3737,1413,3769,4598,10091,2529,2312,4709,
	2862,10095,2290,10096,4711,2745,4204,10097,
	4525,3817,4424,1964,1603,2497,10098,3032,
	2963,3140,10099,1837,10100,1264,4185,4725,
	2292,3089,4153,3869,3924,2713,2681,10115,
	3268,2580,10132,4301,4506,10133,3417,1597,
	4695,4287,2436,10134,4361,1978,4727,3828,
	2269,4702,10137,4080,3956,8689,4723,4867,
	3751,2375,10140,2188,1884,4199,1217,1943,
	2841,10141,3930,2264,3669,2570,3156,4869,
	10145,4872,3238,4767,4193,3401,3237,4770,
	4768,4732,2247,3052,1301,10153,4769,2242,
	2234,10154,3247,1366,2687,10157,1181,4653,
	3426,3568,4461,1713,3894,4846,4774,3111,
	10161,4772,10162,1405,2615,4187,4771,10165,
	4174,4773,2764,4127,10169,4568,4776,10170,
	4779,2161,2126,3241,10174,1658,10175,1631,
	10176,3090,4782,4168,3832,2098,1689,1602,
	4008,4569,4266,2052,2038,4775,3290,3775,
	1326,10193,4777,10194,3332,4780,2686,4781,
	3776,4783,1141,1817,10200,4791,10201,1012,
	10202,2607,10203,3365,10204,3514,10205,3515,
	4792,3786,1454,4606,4689,4280,10219,3640,
	10220,1925,10221,4790,4848,4293,4787,1677,
	4692,4788,4784,4789,2886,4251,4778,4849,
	10224,4785,10225,2982,4121,998,2116,4794,
	10232,4793,4800,4802,10237,4799,3408,4797,
	4533,10241,8178,10242,1283,4796,10243,2536,
	10244,3427,10245,4798,10246,4795,4804,10249,
	8285,1520,1794,10253,4167,2665,10258,2197,
	4493,1191,3141,2424,4801,10259,4803,10260,
	3833,1957,9884,1372,1580,4808,4850,3147,
	2537,4812,3532,1690,10289,4809,10290,4811,
	1036,4294,4813,8629,3984,10291,4810,4805,
	10294,4806,2546,4807,2150,4009,1700,10297,
	2322,10298,4817,4814,10303,958,4822,10307,
	8908,7778,10315,7968,4824,1055,10316,4819,
	10317,8319,1434,10318,2338,3611,1961,4183,
	10322,4823,2233,10325,1232,4825,2330,10329,
	4826,4815,2882,8284,4818,4821,4820,4431,
	10341,4532,10342,3168,2124,4830,8596,4816,
	10352,4831,3018,4691,10362,3668,7781,8833,
	2941,3686,4827,1226,2443,8909,10407,8458,
	7742,9855,1401,10416,4834,10417,4835,1317,
	4836,1378,8750,972,7830,8907,8812,10424,
	8471,4837,3310,3934,10442,8415,8386,4838,
	8108,4842,3303,8915,4841,4840,4843,3536,
	10465,2170,8065,10469,3014,8720,10474,8911,
	10475,8723,10476,4839,4844,8164,8080,10477,
	4845,8918,10482,3264,10483,8912,8910,7798,
	10492,2503,8919,8917,8913,1592,4697,4365,
	10516,4333,4001,1289,4472,10517,3893,1851,
	10518,2392,10519,2791,1553,7909,10523,3700,
	4866,4868,1428,10527,1524,3206,8229,10537,
	982,1796,2626,4854,10538,2471,1803,10539,
	1841,3990,1121,3075,2318,1477,4645,10540,
	4129,2137,3445,2096,4858,4765,1743,3224,
	1192,4380,2790,4884,3258,10555,3967,10556,
	2347,2922,1843,10560,4885,10561,4332,4886,
	4879,1515,1672,1122,4880,1290,2341,10569,
	2435,4152,2511,1522,4882,4881,3071,10578,
	4633,4883,10579,2562,10580,1491,2609,7887,
	10581,2152,10582,1368,2598,10592,2912,2091,
	1615,4864,10603,3037,10604,2362,2016,8153,
	10605,1451,10606,5017,4002,10607,3690,967,
	1307,2060,5018,1894,10608,4391,1431,1492,
	4748,3243,1651,3149,4749,10611,2364,5003,
	4750,3991,1987,6510,2585,2622,4405,1744,
	1335,10619,1306,2970,2534,10628,1113,10629,
	4751,1831,10630,1438,4549,3479,3210,3307,
	1359,2393,10635,1869,10636,1574,2100,3645,
	8904,3940,9859,4755,3117,1833,2164,3633,
	10645,3049,7959,4758,10650,4756,1033,10656,
	2326,10657,3386,2151,7979,1708,1769,10663,
	8906,7833,4760,10671,2204,10672,4762,4761,
	8015,8143,3005,8246,7994,10676,8905,8089,
	4763,8053,10679,4764,2543,10687,3211,1015,
	1791,2118,3840,5005,2588,1518,4601,2925,
	2218,2531,2257,2488,5008,3421,10705,8126,
	10706,1137,4263,10710,2792,10711,4052,5010,
	10716,2496,7885,5011,10719,2366,8617,10720,
	8678,10721,8487,8195,2847,8498,10724,3159,
	5012,8995,8437,4860,3344,1805,3839,4363,
	1031,1365,10748,4004,4861,10753,5301,4863,
	4710,1988,1050,10758,1276,4740,2180,2428,
	10763,3880,8901,4743,8030,10771,8902,3012,
	3189,4164,4744,2887,8431,3397,10781,3108,
	10782,4700,10783,3382,3830,2033,1014,1983,
	3960,10787,1049,4673,4636,8656,1414,2715,
	2866,1136,10790,1150,10791,1102,5370,4445,
	2357,2643,4747,2648,10792,4746,3811,10793,
	4946,10794,3791,4714,2739,4230,3762,10797,
	2087,3215,2680,10798,2332,3969,10799,4947,
	1228,10806,1585,3663,2533,2530,10809,4066,
	4106,4733,2512,8897,1960,4734,10821,4336,
	7780,10822,3918,4735,10823,4736,3888,10824,
	1309,2294,8697,4737,8208,10831,8898,10832,
	5020,3909,10840,3289,1173,7770,4283,1197,
	2083,4278,3488,1617,3452,10850,3196,3443,
	1101,4027,10851,2962,5013,1503,10855,7842,
	2407,1822,2323,2614,10856,5374,5373,4538,
	2209,4474,979,1506,2390,3579,5371,3406,
	4279,10857,4741,4157,1916,3507,3601,10858,
	5375,1489,5372,4314,10862,1273,1778,10863,
	5378,10864,1926,2077,1494,10865,3677,2816,
	1962,2527,3699,3935,5376,3890,10866,2663,
	5377,2711,2348,10870,2600,3709,4222,1642,
	5385,10871,5388,1678,980,1557,1649,10874,
	1891,3664,2401,3494,3091,10875,4519,5389,
	10876,3825,1240,3851,1336,10879,3798,1959,
	10880,3824,10881,4070,1399,10886,1263,10887,
	1761,5379,2856,10892,2940,5383,5386,4340,
	5387,10893,3127,3815,2875,10899,5394,10900,
	5399,10901,4571,5393,5410,3781,10910,1918,
	5398,5392,2975,3364,1963,2817,2310,10915,
	5390,10916,5397,10917,2969,4423,1924,10922,
	2299,4258,1723,10923,4579,10924,5391,1815,
	2356,10927,5400,5411,5421,10933,5407,5406,
	4646,5991,4142,5402,10934,2358,10935,4382,
	10936,2389,5404,5413,9069,4083,10942,5414,
	946,3030,5403,10943,1948,10944,1566,3724,
	1879,4376,5415,3929,944,5423,5401,4079,
	5405,5412,10947,1982,10948,10949,5418,5422,
	5424,4249,10950,8775,1762,2935,5426,3347,
	2545,2855,2411,3941,4477,5430,1151,10965,
	1942,5428,10966,5009,10967,4109,9068,10970,
	3561,1344,10971,945,5433,3609,5434,5427,
	10989,5436,10994,1976,10995,3442,3767,5453,
	1231,10996,5452,5446,11004,3722,5444,11005,
	5442,11006,2397,4641,11007,3333,5440,940,
	8604,5445,8393,11016,5443,5448,5454,11022,
	8688,3008,3311,2461,5437,2943,11029,4730,
	5441,2905,5466,3639,11038,5463,11039,2355,
	5461,3784,5457,3327,2457,5460,1956,1895,
	5455,5438,11045,5465,1328,5471,3871,1919,
	11053,5462,8459,11063,8414,11064,7850,5458,
	8748,4414,11067,5439,11068,2984,11069,5459,
	11070,4316,11071,5482,4012,8408,8896,11077,
	5474,5476,5456,3293,5479,11082,5464,8609,
	11086,3423,5481,11087,5467,11088,3802,11089,
	3512,5491,5484,5480,11090,5489,9081,5472,
	11095,5490,9074,5473,3529,5499,11101,5500,
	5496,8554,11106,5494,2113,5495,9084,1725,
	4731,8368,11113,9082,7796,4021,2710,9079,
	11116,5493,5475,5498,5497,5501,11122,8012,
	5506,5502,9080,8655,9064,4597,1237,11125,
	4681,9073,3504,11126,9065,5504,11127,3850,
	1936,11131,9844,5511,5507,4148,5505,5503,
	5509,5510,9071,9072,11153,5508,11154,5514,
	9078,9075,11155,3094,4706,4398,5516,3424,
	9087,11158,5515,9076,11159,8373,11160,1724,
	11161,7899,9842,11162,5519,5513,8631,9077,
	11174,1910,3642,5520,11178,9090,9067,11193,
	8249,9887,5521,9085,8693,11202,9089,3229,
	9083,9086,8652,9066,9865,2869,11214,9879,
	3185,3513,11225,5524,2029,4716,4217,5525,
	3702,11226,1561,5527,11233,4339,2450,1364,
	3766,5526,5528,1829,1875,3693,5529,3059,
	5530,11246,4341,9091,3201,5532,5531,7997,
	11247,8591,8774,8776,8574,11254,8576,4381,
	5031,5033,11267,5032,1856,5035,1222,5039,
	2061,11277,4534,11278,5040,1624,11285,5024,
	3585,2367,1994,4693,2400,2132,3590,5038,
	990,3834,1653,4630,3041,2447,11295,3596,
	11296,5047,5041,3033,5043,11297,5048,11298,
	3000,5049,2381,5046,5045,1340,2455,2633,
	5042,5044,3992,5053,2504,5058,5059,5026,
	1809,4334,5052,11332,2398,5056,11333,5025,
	11334,1479,11335,5051,2418,5054,11339,5057,
	1786,942,1260,5055,5064,11363,3057,5060,
	5061,5063,5062,5070,11368,4311,1153,9002,
	5074,11379,5067,8843,5068,2977,2062,5075,
	11390,3606,8069,1552,5196,8997,5071,11395,
	5072,1576,9004,9006,5076,11407,5079,1037,
	8709,4104,7736,7795,1537,8163,9017,3567,
	6563,11440,9003,11441,3537,9005,11442,3571,
	8575,3604,8615,11453,9007,5080,3649,5081,
	5027,11474,5085,5082,11475,2274,11476,3474,
	5083,7874,3331,2844,3130,5084,11492,8882,
	11493,4409,4017,2830,1556,8409,5028,11517,
	8157,5029,11523,8551,5030,11533,8685,11534,
	1909,8199,8998,9001,8018,8251,9000,11540,
	8999,11541,3227,7723,11546,3414,3236,11547,
	4624,8878,3378,4163,8009,5660,11560,1321,
	1711,11568,3889,4859,3859,3731,4865,1567,
	11575,4160,8321,11580,5659,5657,11581,1398,
	11582,3647,3582,1679,11583,4712,4116,1906,
	11584,3390,4171,2417,2115,1568,11589,5295,
	11590,8060,5296,5004,11593,4098,3077,2865,
	1675,11596,1657,2441,4669,11599,3092,1061,
	5298,2178,3626,5300,11602,5299,1485,11608,
	3350,9051,7902,11614,8098,11615,7930,2927,
	2924,11620,2863,11621,2140,3570,6005,3266,
	6006,3758,11628,4622,1721,2703,11629,3244,
	1545,2110,4134,6011,11638,2803,8877,6014,
	6010,6015,3720,2880,6018,2760,11658,3069,
	6012,3411,2228,1820,6016,4000,3774,6019,
	6026,11677,4141,11678,2171,6023,6025,11684,
	2492,6027,11702,4221,4648,11706,3759,11707,
	3727,2637,2192,6024,6029,6032,11714,6034,
	2329,3368,6033,11726,1583,4302,6030,11737,
	6031,11738,3197,6035,8255,3746,6037,7950,
	6036,4233,6041,3373,6045,2761,6044,6049,
	3870,11838,6564,8297,6047,2129,3298,6050,
	11860,6051,11861,6056,9207,6058,6053,1462,
	11868,6054,2878,11873,6048,9210,6060,11898,
	9214,8104,6062,6061,11908,9217,8737,11929,
	6063,11930,4876,6064,8482,4656,6071,11954,
	6072,2404,4371,11955,4659,1385,3556,6068,
	4729,4651,3949,11956,2772,6073,2097,1819,
	6069,4048,1881,2678,8538,4705,3458,5994,
	11964,6070,11965,1682,8677,11968,3265,2904,
	11971,8279,5934,2913,11972,3569,5935,4429,
	11973,4307,3439,957,11974,3523,3739,5937,
	11980,5936,11981,4663,1842,4581,1511,3745,
	4178,1039,3404,11982,1293,3372,2394,4038,
	3433,5938,2007,3913,1797,1885,4112,3945,
	2117,11992,5939,3256,2426,1119,3539,2104,
	11998,2103,4225,2785,2409,2759,12006,1893,
	4325,3163,2834,1203,12016,1834,8421,12017,
	5943,2577,8495,8355,4432,8481,12018,8660,
	8165,12019,5944,12020,5946,7818,7735,12025,
	1386,1555,3511,4057,1437,3440,12028,1663,
	3357,12031,2172,8094,8870,3789,4685,8680,
	12032,7898,7865,3948,12033,3346,1594,6066,
	2133,12036,1251,12037,3337,12038,6067,1223,
	5302,12043,4269,5303,12044,4137,9052,3395,
	4227,1284,5990,2884,2256,3777,2901,2309,
	3015,1194,3193,3646,2237,3820,3409,3040,
	5992,3975,3467,12073,3697,2668,8272,5995,
	7783,2667,5996,12074,9204,8510,12078,6004,
	3710,12079,3318,4192,5548,4305,12091,3551,
	3088,5547,5549,5550,5555,1204,12104,5551,
	1749,5556,1435,5557,7958,5559,5564,4091,
	12114,5562,5561,2612,5563,12118,4355,12119,
	5558,12120,5566,961,5560,12125,2438,12126,
	5565,5569,5571,2676,12156,1578,12157,4317,
	9097,12165,7863,2349,12170,8628,5579,1292,
	9103,3080,1377,12187,4075,7962,5577,12198,
	5575,5574,12199,1065,9099,4441,5576,5584,
	5582,5591,5590,3123,5586,12237,5588,5592,
	12249,5589,5585,9105,5594,12268,8817,9102,
	12286,5595,5598,9104,12313,8243,12314,8764,
	8169,12328,3760,8277,12333,9106,6165,12340,
	1323,4572,4059,1239,1789,4688,3142,2317,
	1799,12345,3814,1205,5023,8996,2093,4184,
	3517,984,4855,2238,12355,12356,1083,3431,
	1155,12357,3484,1608,12358,3389,5534,4459,
	5537,5536,2946,3662,12366,2554,5535,4578,
	1142,4547,1468,8515,1403,4512,8489,12377,
	3868,1018,5538,12380,8823,7847,5541,1224,
	8837,12387,2786,9092,5542,1686,5543,2846,
	12401,9095,9094,5545,8848,12406,5546,1332,
	7748,7730,9093,1732,3036,2893,12419,1129,
	12420,3997,7955,6163,2008,4284,4265,8051,
	1852,5681,4620,12424,3177,1084,1333,5683,
	4029,2645,5682,12430,2415,4235,1464,5684,
	1483,12431,2802,1784,12432,1705,12433,2964,
	1644,5686,5685,1543,4694,8160,12441,3670,
	5690,12447,5688,3475,2370,4254,5687,2551,
	2483,5693,5692,2453,8327,7800,9134,7926,
	7985,5695,12481,5696,12482,8261,8568,5016,
	12489,4092,3667,5293,4699,2359,5021,4211,
	3097,2923,5294,12498,1088,5366,3412,12502,
	5369,12503,1798,12504,4228,12505,1696,1954,
	1469,4454,2779,3903,1974,12515,5999,5998,
	6000,12518,3733,12519,3280,12520,9205,12521,
	8821,1424,3132,12525,6002,9838,12534,7856,
	8323,12538,8584,12539,5986,12540,1858,1426,
	12541,2659,5988,5987,5989,9852,5614,3993,
	12548,3679,12549,4110,1169,1108,12552,1115,
	2990,4452,4245,12555,5600,5601,12559,4194,
	12560,1246,1076,3754,4504,5602,12564,2277,
	1408,12565,5603,1939,5604,1991,2672,5605,
	4023,8133,3694,5606,12571,1444,2952,5607,
	5608,12574,7841,12575,9107,4318,5609,7947,
	4054,5610,3761,5611,12590,1443,12591,7804,
	5612,2026,3983,5698,1089,4203,2108,3239,
	12603,5701,6668,6667,5367,5700,4541,3757,
	2731,4561,5705,5710,2425,12618,5711,5703,
	3982,12628,1965,5707,1659,1992,3583,3520,
	5702,5704,5718,4408,5717,2926,12636,4507,
	2947,1159,5713,12639,5716,2552,3505,1410,
	5723,2084,5715,3999,4348,5719,1839,5720,
	4665,6669,5722,12660,6673,5728,3432,2558,
	12668,2021,2403,12671,1946,6674,6671,2027,
	6675,4031,6672,1941,1590,5729,1521,3652,
	1792,12685,3855,3102,1584,12690,5727,5726,
	2872,5730,12691,4262,5735,3137,5737,1902,
	5736,12705,2031,12706,5731,5732,5738,12710,
	5734,12711,3841,4267,4358,12716,2909,6676,
	4039,5733,12719,2810,12720,5741,1048,12721,
	5746,9140,8318,12722,2101,5740,1440,3174,
	5744,2265,3744,5743,12746,5745,12747,3860,
	5742,12748,3768,2034,7907,1484,2324,1177,
	1266,12753,1058,5739,1176,1421,1848,1575,
	8341,9147,3928,5751,12754,2017,3234,3989,
	9145,5752,1299,6677,12765,4319,4297,6678,
	5753,4201,4291,7720,9149,1739,5747,5750,
	1660,12782,5748,2445,9139,4347,1355,12810,
	5755,8546,2010,12811,3376,7774,12819,7773,
	6679,12822,9143,2724,7983,9439,2032,2363,
	12827,9138,8526,8274,12830,3790,9148,12833,
	5756,8427,2371,7743,1112,4410,12853,8219,
	8380,9150,12854,5758,5759,1888,6680,12870,
	8329,5757,12871,9137,8724,5760,8158,3971,
	8739,974,6682,9142,6683,2476,12895,5761,
	9868,9438,2932,12910,9144,5762,8190,8017,
	8673,12922,9136,12923,8142,12924,8478,5088,
	8225,6684,9441,1765,12930,3835,6393,4018,
	3472,3249,3876,1262,3809,2229,12931,9379,
	5790,2055,6394,4446,12932,3068,6395,6396,
	12936,6398,6397,12937,6400,6399,12938,9380,
	2217,12941,6401,12942,2661,12943,8819,12944,
	8624,1348,1402,6644,6643,1628,3566,1099,
	12952,6645,12956,3437,5306,1164,4413,3050,
	978,1397,3246,3105,4531,12973,2452,5307,
	3297,4120,2917,1010,1243,3232,1586,12982,
	4467,1269,2095,2336,988,4189,3450,4610,
	12996,3687,1525,2374,4476,12997,1698,2968,
	5308,2406,2684,3133,13001,1977,1041,3004,
	3578,2827,13016,5309,4067,1294,13017,2807,
	1684,13020,4595,1412,1207,2839,2892,2456,
	5310,13021,1011,2949,2601,13022,1838,13023,
	2315,3721,985,13024,3712,5312,2303,4635,
	5311,4465,1000,2634,2147,13029,4250,2468,
	2914,1133,4404,2451,3418,5313,4509,13032,
	1801,13033,3208,3486,13034,5315,2378,3027,
	4612,3399,2854,13039,1275,1835,4535,6478,
	959,4542,2677,13057,3805,3573,3961,2870,
	1427,5314,4502,2090,2024,943,13060,2931,
	1391,5316,3671,3741,8657,3829,5318,13077,
	3682,2449,4634,13080,5317,977,1899,3340,
	2902,2327,2487,3557,13093,2148,2000,1432,
	2998,8476,5328,13097,9053,13098,5325,2316,
	5321,9858,1338,2222,5322,2896,3891,13110,
	1473,8461,8282,1565,3441,1493,5327,13113,
	4455,13114,5323,3616,3101,13115,2950,4155,
	13119,2337,2682,13127,3600,1245,13128,2212,
	13129,2405,3703,4099,1390,13130,5326,5319,
	5329,13131,6480,2647,5324,4543,1416,13136,
	1211,13137,5330,13138,5340,13139,8077,5335,
	13143,5339,4670,5337,2797,3636,13149,1196,
	5336,13159,5333,3812,13160,1322,2360,2285,
	2211,8024,5331,13171,4337,13172,5320,13173,
	5332,2474,13178,5334,1210,1764,2638,5343,
	5346,8539,13186,1138,1319,1389,3295,5344,
	3526,13194,1758,13195,5345,5348,3605,13203,
	1004,1393,5942,13210,8411,1202,13216,6481,
	13217,5342,3356,5341,997,4136,1120,13220,
	3586,9056,5338,13227,3481,9057,13234,5349,
	8256,1376,13242,2825,5351,13246,8846,9054,
	13252,5353,7786,2581,13259,5350,2334,13260,
	3025,8194,1258,3281,8339,13269,3503,5352,
	4623,9055,7852,1244,7764,2572,13282,7944,
	3144,1132,1388,13283,4616,13284,8384,9058,
	13285,2895,13286,13287,5357,8544,1898,13288,
	8605,8078,5358,8749,2505,13289,8263,3324,
	13290,8802,8039,7858,13293,1184,3171,13294,
	5359,13295,3161,13296,7849,5360,6482,13299,
	8140,3528,13303,8050,13304,5362,13305,5361,
	13306,1160,7762,8356,7964,8847,13314,8173,
	7725,8530,9062,13319,8442,13320,2955,9059,
	13324,8346,4383,13334,8182,13335,9060,13336,
	3228,7785,13337,9063,13338,8477,8793,8278,
	8547,5364,8105,2335,8188,13347,5365,4518,
	6409,6502,3436,13352,4786,1727,13353,1790,
	1634,4511,3954,7371,2809,13367,2295,6503,
	968,7726,13371,2205,13372,2555,13373,1087,
	1229,1741,3291,6504,2275,3476,13386,3136,
	13387,4508,7868,13388,1680,8514,8222,7747,
	4428,1116,6593,1002,13407,6595,13408,9428,
	13409,1526,13410,2584,13411,7510,3964,4486,
	13414,3810,2240,1287,13417,1702,13418,4439,
	13419,6697,8815,1550,13420,3502,3981,7895,
	13427,1626,6596,3392,2965,13433,6599,6598,
	2666,6597,4040,6600,13438,6601,4674,6603,
	3083,3821,13452,2107,3248,1418,2296,4539,
	4394,4055,4028,3400,2434,6418,13468,965,
	6416,6413,13474,1220,13475,2812,2045,4190,
	3843,6417,6421,13485,3986,4248,1341,13490,
	2758,4687,4466,13495,3422,13496,6423,6422,
	6425,6424,4582,13504,3905,6429,8493,2019,
	2251,3335,3314,3947,6428,4369,6431,6433,
	8859,13522,6427,13523,6432,3837,13526,2035,
	13527,1252,3060,2271,3849,3172,13536,2262,
	6434,4384,13545,6435,3883,8790,9384,2928,
	960,6437,7389,9383,13593,9862,9382,13594,
	8654,3462,13609,4350,13624,3191,4158,1783,
	6420,8508,1187,2723,13629,4411,3641,4683,
	4357,4277,6515,2996,13636,1691,3500,6542,
	13642,2485,3756,13646,1236,13647,3065,6572,
	9412,2849,13654,3779,2828,1063,4416,13655,
	3468,13656,4587,3058,1571,3306,13667,4374,
	3203,1734,13670,6228,3317,6225,13673,2521,
	3998,1163,1384,13674,6226,1540,13682,6227,
	3471,1751,3656,4119,6229,6232,1907,13690,
	1046,2221,7884,6415,6233,13691,6236,13692,
	6241,6242,13697,3518,1008,13698,2078,6231,
	13703,3752,13704,6240,4494,13713,2591,6234,
	13714,2747,13715,1876,4517,6238,3447,4393,
	13719,6230,13720,6235,6237,13721,3126,1664,
	13722,6239,13723,2410,6248,6253,13726,6251,
	2127,2114,6257,6260,13735,6256,1123,995,
	2838,1735,3073,3225,3260,6245,6250,6252,
	13749,1866,6254,4690,13750,2910,13751,6258,
	2383,6243,4600,13761,2625,1208,6261,13770,
	3416,6255,4421,13775,1107,4444,6244,6246,
	1519,6249,13776,6259,2467,13777,3470,13778,
	3485,3067,2528,6280,4585,4130,1922,1780,
	1772,4375,6277,13808,6276,6274,1865,3621,
	6268,3765,2431,13809,963,6279,6278,13810,
	4637,13811,6266,13812,3674,3292,13813,1997,
	2220,6272,6267,1430,6269,3138,6273,6275,
	2177,4619,13824,6286,6283,13833,3681,6284,
	2563,13843,6270,13844,2748,1019,1788,9322,
	8565,13863,9325,2771,3823,2513,3560,13873,
	3632,3968,3448,13876,6281,2143,13887,6287,
	2787,13894,3076,13895,1872,4661,9323,8799,
	2079,13902,2992,7886,3607,3302,13918,6294,
	2509,6292,1840,13925,6289,6295,13938,4529,
	4626,9331,6299,13939,2184,1325,13975,1342,
	7934,3955,6304,2510,13999,6302,6316,14004,
	9333,6306,14007,8714,6288,8047,14013,2361,
	6309,6318,6307,14019,1728,14020,6305,14021,
	4289,2481,6329,6319,1020,6320,4422,14045,
	9320,6322,14046,6323,8448,9334,14059,6326,
	6327,7975,6311,8407,6314,14068,1990,6328,
	6330,6338,8097,1186,14105,6332,14106,8874,
	8197,8506,4450,14128,2822,14129,8708,6348,
	14134,1944,6333,14139,4232,8387,8512,9335,
	6347,6340,3134,9332,6349,1261,14167,6343,
	8040,14171,3933,8581,6345,14208,6352,6351,
	6341,14220,8075,9348,6353,14226,6355,2767,
	9344,9326,14266,9339,9350,9329,9347,9328,
	14274,9321,14275,9343,9338,14317,3124,1361,
	1995,3979,6577,6578,14334,6579,3066,8419,
	2427,6581,3957,14343,3125,9419,14362,8019,
	4536,4510,1358,1156,3827,3078,8211,7987,
	1400,14376,3509,2130,4115,6361,14381,1404,
	6360,14382,4060,3449,1175,14383,6364,4530,
	14390,7772,14391,6368,9354,9353,9355,8067,
	6589,2938,1549,14407,4218,8463,8155,1487,
	14413,2030,6591,4202,8367,3826,14420,2843,
	14421,2756,1533,4719,1073,1080,1082,3007,
	14424,1081,14425,2736,6484,1912,6485,14445,
	6487,6486,14452,9408,6489,6491,6490,8813,
	9409,6492,14461,3430,4715,2806,14462,2730,
	3095,6493,2861,14463,1687,14467,6497,3167,
	8394,6499,14468,1883,4126,956,6498,6500,
	8423,9410,14469,1419,2671,3173,14470,6501,
	14471,3491,5791,3666,4524,3184,4852,2040,
	14483,1905,5794,5792,3328,14492,1904,4061,
	5793,3269,1800,2174,1277,3818,3584,14506,
	2086,14507,5802,14508,5803,3098,1655,3164,
	4177,3813,14516,5796,5804,14517,3371,1254,
	3074,5797,6687,5798,3308,14528,2983,1806,
	14532,2752,14533,5795,2942,2541,2688,1182,
	14534,5805,1979,2832,14535,5808,2313,5819,
	5813,1930,1645,4274,14542,4557,4468,1818,
	4437,4097,3972,3187,3205,1147,14550,2783,
	5820,5807,1605,5815,5812,2915,14561,5814,
	2974,1135,3099,14562,2883,4607,14565,2508,
	5817,5818,5821,3580,5811,14568,4259,14569,
	1068,6688,14570,5816,3973,3042,4406,5822,
	14571,14572,2225,4124,5825,14580,5831,3873,
	14588,5830,5836,2700,2243,14597,5824,1952,
	5832,1596,4573,5840,14604,5835,5823,14608,
	2051,3726,3103,2954,2624,14611,5826,14612,
	3120,2173,2190,5827,14613,4644,1193,14614,
	5833,2102,5837,14615,2048,5838,2921,5839,
	4484,2352,14620,5848,14621,5845,14622,5849,
	1917,2486,1692,5842,1882,2255,9180,2908,
	14647,9179,3944,3359,4260,14650,3902,5841,
	14653,3617,14659,2495,5843,2553,5844,3807,
	14660,2006,1460,14661,3278,2168,4456,3301,
	1693,5864,5866,4077,5857,1486,4650,5854,
	3946,5853,2597,3613,2874,14690,3623,5862,
	5861,5855,14694,5858,1422,4286,14697,5860,
	8286,4224,1382,14701,1993,3367,14704,1345,
	14705,8772,9186,2049,14706,4086,8404,3648,
	6689,4331,14719,5865,4658,5856,14720,2166,
	5859,4299,14723,5863,3377,3195,1544,14730,
	4415,1146,5877,8607,3793,14733,5867,7782,
	3787,14734,1750,5876,14737,2391,4275,14741,
	2801,2953,5878,3701,5869,14761,1973,14762,
	3923,9181,5872,5868,8555,3734,3393,2446,
	14797,2167,5873,5893,4343,8885,14810,5881,
	2616,7974,14814,5895,5885,14819,5886,3538,
	14826,5880,5871,14827,5891,14828,3255,5889,
	2890,5888,14831,5887,1313,5894,14835,7779,
	8328,14836,1474,4649,7869,14840,9025,5892,
	14841,1985,14842,4655,3618,6568,14843,5890,
	4556,5879,5882,2721,5884,14849,2674,2479,
	2679,14850,1118,3589,8484,14857,1455,8264,
	9184,5901,8311,14865,8762,3022,2519,4102,
	5900,2833,14888,8001,8220,5899,5908,5906,
	2725,8888,5883,14893,5902,14894,3477,8822,
	4453,5903,14897,8091,4131,8095,14903,9674,
	5904,14911,3770,2956,9175,3118,14922,2657,
	5896,14926,8453,14927,2579,3593,1238,9185,
	8172,14934,5913,14935,5905,9194,5912,14938,
	5915,14939,5914,9187,8462,1267,14945,8102,
	8196,1247,5909,5911,5910,2989,9189,4395,
	5918,14967,9442,7876,14974,975,2071,8886,
	5921,8361,5916,1117,5924,8054,5925,5922,
	5923,5926,8593,7760,8092,15023,9178,15024,
	8275,15025,9193,9188,8661,15033,9190,9183,
	15037,3064,9177,15043,5927,5929,15044,8212,
	15045,9196,9195,9176,9198,9870,15058,9197,
	9174,9192,5932,15081,8454,9199,5933,8585,
	8280,2053,6639,2805,15100,1447,2023,2610,
	4401,15105,2291,4643,15109,4377,1178,6604,
	2646,1337,1241,15123,3213,2376,6606,6605,
	6607,6611,2325,3602,2971,2283,15141,6612,
	15142,1127,6610,4425,1476,15146,6608,2559,
	1288,6609,3501,2478,3665,2587,15153,6614,
	1949,2494,15166,4593,2379,15174,1616,3342,
	6613,2039,15175,3615,2254,3235,15176,3862,
	3738,15183,2988,4082,1903,6616,15205,2005,
	6617,15206,6640,1059,1654,6618,4111,6619,
	8226,6625,15246,2018,15247,2136,6621,15260,
	3312,9018,15264,2751,15265,6641,4470,6622,
	7918,9429,15268,4594,6624,6620,6626,15276,
	6627,8744,15296,3254,9431,6628,3844,3459,
	6632,8444,15318,6629,15319,6630,9432,15329,
	3223,8473,15342,6634,4105,8743,6633,4402,
	7775,6635,8863,5014,8124,15370,9434,15371,
	8192,15396,6637,6638,15407,4611,15408,2945,
	6508,953,8592,2339,1715,4150,993,1498,
	8712,4713,15414,3489,7910,5789,3019,1009,
	2951,6505,6507,15432,9411,4073,15433,2916,
	15434,6468,15435,2837,15436,2840,2489,6469,
	2852,15440,3838,6466,6470,3627,3853,3865,
	2514,1534,6474,15461,6475,15462,6476,3209,
	5615,15486,1621,5616,5617,15490,4625,8879,
	5621,2430,5620,1459,5623,5622,15504,1971,
	15505,5624,1808,15510,2304,2911,15515,1940,
	2199,5625,5627,1535,3886,3391,5626,4505,
	4321,5628,5632,5629,5630,2517,8630,2482,
	7741,5631,15529,5633,5637,15543,5642,1221,
	5636,2770,1161,5641,5640,5638,5643,3988,
	4588,2733,5646,15554,3908,1958,15559,8757,
	6356,5647,5644,15560,9111,1984,4342,15563,
	9109,8767,8490,5649,15575,6357,7890,15595,
	9110,9112,5653,3572,15596,5654,8354,15597,
	8036,8235,15600,9108,8505,15601,8543,8641,
	9114,15602,5655,15603,9113,2673,15609,4310,
	15610,3750,3735,15634,2746,6170,1996,3907,
	15637,2602,6177,6188,1130,6178,15647,6174,
	6180,6179,3315,4485,15655,6173,1606,6175,
	6187,15667,4584,6181,15672,6186,15673,6185,
	8640,15692,3183,15696,2480,2520,2617,6184,
	6190,3563,6200,6199,6194,6193,15720,6195,
	15721,6191,15722,6198,15723,6197,15724,6196,
	2590,3158,3006,2948,9312,15734,6201,6204,
	15753,6203,15754,6205,1966,6192,6202,15755,
	3275,3299,8741,8298,4135,6206,6208,15779,
	15780,2544,6213,15784,9313,15785,6214,6210,
	15788,6218,6209,15800,6215,9314,6219,6220,
	15842,9315,9316,15848,1832,15849,6962,15850,
	6963,15851,3023,1013,3226,15852,3728,3804,
	6403,6404,15864,3038,1356,9381,5253,15883,
	6406,15884,6407,15885,6408,1733,5368,3374,
	15890,3651,7792,15896,3380,4264,3483,15899,
	1697,6964,1067,15900,5940,3650,4270,2122,
	3363,15901,1480,15902,2867,1482,15903,6776,
	1986,6166,15906,6777,6779,2232,15915,6778,
	3782,2621,5022,6781,4030,8335,2683,3079,
	1609,8014,6782,15939,1296,15940,7857,15941,
	2064,6783,2175,7819,3455,15958,4176,7008,
	15959,7009,15960,7010,2575,15961,1768,2301,
	7012,15964,2930,7011,4122,15965,7014,983,
	2235,4198,7013,15971,1330,1669,7020,7019,
	3010,7017,4495,3630,2305,2085,7022,7021,
	7018,1128,15979,4513,4253,2278,3207,4128,
	15986,4555,1938,7023,15987,1530,8134,15988,
	3685,2538,7025,7028,15994,7027,7030,7026,
	15995,2003,7029,3592,7032,7034,7036,1380,
	9646,7031,16010,7037,7040,7937,7038,8704,
	7043,16021,7039,7042,7044,16022,7045,9650,
	3792,7048,7831,7047,16025,2623,7041,3444,
	8363,16026,1396,1114,3588,16027,7050,7056,
	4463,7053,9651,7046,16036,7055,7052,7049,
	16037,8232,7054,16040,9649,948,7057,7059,
	16049,7061,16050,9644,7060,16054,7058,7758,
	8706,4043,9643,8838,16057,9645,16058,9654,
	16059,7062,8675,7063,8751,8549,9655,1448,
	7913,994,996,4400,16070,1445,16071,2213,
	2015,6957,947,16078,6959,6960,7717,4580,
	7112,16103,7113,9664,16107,8858,4560,2985,
	16116,4244,16117,4205,6795,966,4438,4087,
	2131,1927,16120,2437,16121,1730,1442,2957,
	8814,2768,16126,8125,16127,8068,8370,6796,
	16128,8259,2850,1505,16135,6744,2729,16136,
	4528,3917,6747,2959,16142,1563,16143,16144,
	3384,2753,16149,2369,16150,6746,6752,6750,
	4487,2788,16166,6751,6754,16170,4420,4044,
	6753,16174,2773,6755,2433,2331,6756,16180,
	3658,16181,4100,4642,4503,16185,6760,6761,
	2261,3492,6763,1532,16208,6764,2851,16209,
	6762,2223,1166,6767,4720,6765,6768,2798,
	1303,6769,6766,3878,6770,2718,16248,5269,
	3026,16251,3139,16252,4596,1450,16253,3495,
	9863,3676,16258,6774,4433,9456,6775,16264,
	7550,8864,2737,7114,3407,4186,16293,4521,
	16294,6938,16295,2311,16296,2196,6939,1547,
	950,8107,6690,16302,6691,3848,1612,2432,
	6692,2706,3305,3093,2368,6696,3001,16318,
	4089,16319,4614,16320,4107,16321,16322,6704,
	6708,16326,6706,6714,4489,16327,6709,6707,
	6698,3362,4373,6705,16337,2532,16338,1316,
	9896,16341,1857,16342,6716,6717,16347,6710,
	16348,6715,16349,3847,9445,16366,6718,2619,
	4247,6711,3218,8698,6722,16388,1488,2652,
	952,3550,1047,16393,6720,3740,1475,16397,
	6721,6719,6723,1499,16400,6724,16401,6725,
	16402,6728,16403,1077,16404,8520,2999,2145,
	6726,3599,1201,9446,6727,8439,16413,8299,
	16414,2894,6731,2501,1387,6729,2385,6730,
	8871,9451,2824,6733,16451,6735,6737,16490,
	8166,9447,8210,7916,3413,6648,2524,16505,
	3360,16506,6649,3085,6650,16511,3084,6651,
	4675,6657,4608,3370,3555,6658,16525,6656,
	3926,16528,6660,3024,2099,16533,6659,1436,
	2059,16541,6661,4875,2252,2658,6662,8038,
	9436,1694,9890,6665,8206,16581,9435,7864,
	16582,6666,4306,4708,2518,16588,3162,1923,
	16589,4014,3506,16590,3691,1125,16595,3180,
	16596,4565,2387,2800,6943,1272,16613,3157,
	4118,16614,4551,16615,6944,16616,6942,16617,
	2066,1259,4173,16627,2037,3854,16630,6948,
	6947,1265,16637,3341,3493,6949,16650,1298,
	7808,16665,3799,6952,16669,6951,16670,1439,
	2125,2065,16671,1760,9847,9764,8041,8747,
	9599,8027,8603,16704,9853,4049,16712,2286,
	3179,1324,7068,3692,16717,3153,4430,16718,
	7069,16719,7070,4139,4558,7071,2210,1331,
	2284,1375,3808,16732,2412,7073,2439,7072,
	16737,7075,8608,8582,16738,7074,7076,8429,
	7844,8417,16756,9656,8800,16757,8418,2539,
	2281,2280,4451,2351,16776,3680,7065,2224,
	16782,1546,4592,7267,7268,16793,1736,16797,
	981,16798,7269,16799,7273,3558,7275,16803,
	3953,1075,7272,7279,16809,1458,7287,16812,
	7282,7284,7283,1688,16816,1064,16817,7278,
	7274,1467,16818,7280,7281,16821,7285,7277,
	16826,2134,16827,2630,16828,7286,7290,7745,
	7276,16835,1449,16836,2239,7292,1601,2429,
	4605,3683,16839,1395,16840,1190,16841,7288,
	16842,7289,3313,16843,7293,7294,16846,7297,
	7299,7298,7291,16861,2423,16862,1300,16863,
	7296,3110,16866,2149,8070,16876,1816,1140,
	2067,16882,3544,7309,7305,1845,7310,7306,
	2696,7308,7311,7304,2160,8115,7314,7919,
	16910,4618,3017,16911,8868,16912,9704,7315,
	7313,16917,2639,1752,7317,8465,7320,2516,
	16937,9702,7323,16945,7322,16946,9703,7326,
	7321,8257,16956,7324,9706,7327,16970,8079,
	16971,9708,7329,2014,7328,9707,8400,8221,
	1157,7332,7331,16992,8183,7821,2081,9710,
	8250,9701,9709,17024,8201,17025,8293,9891,
	2780,4853,2507,7370,4654,7372,2540,3048,
	17055,1369,4436,7374,7373,7375,3535,7376,
	17061,4356,4575,2564,7377,2267,7385,17083,
	7381,7384,17084,1972,17085,1757,3610,7386,
	17093,1185,2776,9717,7932,4390,2372,8228,
	7387,2933,9883,17108,9715,17109,8921,17110,
	9716,17111,7399,9243,17112,3874,8136,17115,
	8059,17116,9245,17117,8780,8008,9244,8447,
	3800,8602,17118,8337,9251,7835,9250,17123,
	8464,17124,8845,8049,7928,9249,3533,7923,
	3564,2502,8625,9254,9253,8480,8474,9252,
	17143,9256,9258,9257,17146,8852,17147,8894,
	17148,7729,9260,17156,8117,9259,8111,8296,
	8676,7969,17174,8449,8572,8522,9261,8148,
	7400,17184,8145,7731,17192,9263,9262,17195,
	9264,8667,8889,17212,9270,7822,9273,8645,
	9271,8594,7402,17222,9274,7961,8588,7744,
	8883,8287,9272,9266,8820,17226,7836,9265,
	8325,8119,9267,8276,8670,17240,9268,9277,
	8073,9276,8046,7896,17246,7871,9285,17247,
	8777,9281,17250,7751,8022,8598,17255,9283,
	9279,17256,8227,17257,9282,9278,17260,9894,
	9038,9286,9291,9284,9255,9292,7951,17286,
	9287,9289,9288,7939,17296,9290,8540,8891,
	9296,17299,9876,9295,9718,8273,17300,9294,
	17301,7675,17302,8890,8045,9298,9297,7403,
	9301,17318,8841,8470,8486,8033,9885,17335,
	8074,9302,9305,9304,8058,9293,9303,9299,
	17353,9248,17354,8671,9861,17355,7790,8740,
	9840,17359,8634,17360,9306,7404,8191,6099,
	2287,6100,1955,6101,3896,6102,4351,2089,
	2112,3245,3778,6105,17365,1346,6106,3309,
	1748,2860,17366,4666,2689,1652,4540,3797,
	1633,2919,6107,3916,2560,4678,3369,3877,
	4526,4564,6111,1016,3349,4214,2268,6114,
	1022,3257,2226,6115,3233,17369,6116,2043,
	1779,4045,6117,2702,2343,2203,3684,2333,
	4016,17370,3548,3619,2111,6120,2075,4034,
	6121,17371,4035,3383,3771,2789,6127,1066,
	1302,17372,4664,4449,6130,2675,4631,2141,
	2794,2477,2076,17373,6126,1551,6138,17374,
	1999,1471,2669,1097,6141,4344,6142,1722,
	6144,6143,1673,17375,6145,1215,4236,3562,
	2202,6162,7262,17376,1746,17377,3212,7263,
	17385,9698,17392,9882,17393,1847,3753,4766,
	1896,17396,2693,6785,17397,1600,6787,17402,
	6786,992,6789,4471,4684,4546,17411,7914,
	6791,3463,17412,6790,7724,17415,6792,8290,
	9457,17425,9458,17426,4123,17427,3129,17428,
	2757,7364,4010,7365,17441,3912,3221,9714,
	17444,7366,17445,8725,17451,7369,4878,1785,
	5997,4309,17459,7390,17460,3803,7064,17468,
	4213,17469,7391,8622,17473,3925,7392,1461,
	1383,7394,7393,7395,7397,17491,1897,970,
	17492,7398,8416,17498,1610,4212,4146,2490,
	17502,2377,6483,4480,6426,7127,1591,3480,
	2864,17509,7115,17510,7116,1782,7117,1915,
	4360,989,7119,17516,7120,7125,7124,2966,
	17519,9666,17520,9665,7126,17523,1593,17524,
	7128,4149,5297,3519,1281,17527,1411,17528,
	1787,2903,7129,2573,2628,4527,7131,7132,
	17540,2547,17541,8488,17542,3031,17543,2314,
	7133,1362,8215,7840,7134,8485,8525,17561,
	9668,8349,8842,17562,9667,8569,8248,6686,
	6685,3541,4197,8533,3510,4473,17568,3261,
	17569,2506,2068,3951,17578,4577,17579,1542,
	1747,6512,1737,17580,6511,1227,1826,4522,
	17581,1681,1639,2138,1627,6518,17585,6519,
	964,2396,17586,6517,4322,17587,6520,1643,
	17592,6514,6516,3375,4566,4462,3965,6527,
	3783,6528,17593,1417,1052,6530,3576,2967,
	6531,17606,6529,2976,6526,3387,6533,1030,
	4717,7111,17611,6534,6042,6537,17612,2421,
	4175,6535,6540,1767,6536,17613,2187,17614,
	4003,17615,962,17616,6541,2879,8658,1379,
	2717,2092,4387,3081,2871,6543,2920,4873,
	17627,1144,17628,6548,17629,2198,9415,1703,
	17647,3714,6549,6545,17651,2556,8825,3654,
	17663,6550,17664,2459,4159,6551,17665,8483,
	17666,1706,1704,17667,6552,3128,3748,9413,
	6557,9417,3987,8340,6562,17678,6558,17679,
	8854,17680,6561,3284,17681,4133,6556,7799,
	1716,3910,2888,3629,3705,1021,17687,6566,
	6565,17693,1145,1755,17698,6567,1109,17705,
	7942,3608,2823,3858,6576,17712,2995,8345,
	6570,5697,6574,17727,7853,9416,8360,3711,
	6575,1092,4252,8223,6573,17732,6571,8389,
	17733,9418,8176,17742,9414,8797,8926,6402,
	8236,17752,4657,17753,7337,1305,8545,4490,
	2297,17764,7333,17765,4143,17766,7336,2298,
	4292,8763,8665,8139,8137,17767,7266,17770,
	3453,17771,3653,5656,3497,17778,3831,4570,
	17779,7344,17784,1908,7345,1006,7342,17785,
	7343,2163,1180,17786,1573,1143,3900,7346,
	1327,17787,7347,3672,17797,7352,17798,7353,
	3527,7777,7355,9711,17818,8088,17819,7357,
	7388,2565,2139,8072,3300,4103,6001,8696,
	17826,5089,4188,5090,951,5091,2219,17831,
	5095,4728,17835,5093,4312,17836,3343,5096,
	5092,17837,5097,2727,5098,5107,5099,3822,
	4516,17844,5117,17845,5115,5120,2231,2642,
	17848,5094,5113,5116,5100,1647,976,5109,
	3977,5103,1981,17849,1625,5114,5108,5101,
	3160,17852,8994,17853,4072,5102,3772,5104,
	17860,5105,1179,5119,3530,17861,4346,5129,
	5133,3577,5140,17862,2796,5130,2382,5127,
	17865,1029,1807,5139,5121,17866,5106,5124,
	3279,2413,9014,1062,17873,4231,5123,17878,
	3034,17879,5132,4639,2740,1619,3150,2734,
	5136,5126,5145,5122,5131,17887,2260,5125,
	17888,5134,17891,5158,5135,5166,5141,2142,
	1351,2728,1198,5160,17906,5152,17907,5149,
	17908,5162,5148,4215,1199,17909,3250,3262,
	5161,17910,5147,2258,5154,17918,1188,5153,
	2153,5142,2009,17924,2526,17930,5151,5156,
	5159,1429,17931,3252,2044,5164,5163,4240,
	5165,5167,5169,4216,5168,1920,5173,5186,
	5181,5184,5180,3053,2525,8875,5146,5178,
	8127,17958,5187,5150,5179,17961,5188,17962,
	5176,17963,8061,9011,5189,5183,5177,2829,
	5172,2462,2548,17972,5182,2054,5185,4237,
	5190,17973,5191,2732,5214,5192,5213,17979,
	1814,2308,17982,2345,5206,17989,5201,17990,
	5195,1170,5199,17994,5207,1131,5217,3055,
	5212,8013,5216,2603,1635,5193,18016,5209,
	3620,18017,5205,9012,8177,5198,2765,3035,
	3773,5208,18020,5204,5202,2691,5215,3937,
	3283,5228,2699,5219,9030,4599,18072,5221,
	5220,1771,5218,3054,18077,1516,9031,18078,
	8595,5230,18081,1968,4388,5235,18082,9889,
	18083,1363,18084,5222,18085,2440,5231,8034,
	5227,18086,5225,5226,18098,2176,5232,18099,
	5233,9037,9032,5249,18106,2766,3056,18128,
	5248,4501,5247,5244,18132,7776,5240,18133,
	5243,9029,5236,3251,5242,7954,18140,5237,
	5238,3559,18143,5241,2465,18152,2094,5245,
	5251,18155,5250,5239,2991,18161,8216,9013,
	5255,5262,9020,18173,5261,2804,18190,2722,
	9839,9041,5257,18198,1171,18199,8096,2891,
	3446,8731,3131,5254,5256,5258,5260,949,
	1079,9026,18218,5270,9039,18221,5264,2183,
	3274,9028,9009,18228,9035,5259,18229,9019,
	5263,9040,18237,5266,5265,7860,8614,5271,
	18247,4367,9045,18250,5277,5268,5272,1034,
	5280,18258,5275,9022,18259,8052,18260,9008,
	5276,18263,9856,4047,5279,5273,18278,8081,
	18279,5274,8455,3976,5278,3460,5283,5282,
	18289,5281,9023,8180,9027,1183,2799,5284,
	18308,2939,5286,8722,3628,8711,1607,9049,
	8788,18333,9010,7718,9047,4392,9048,5289,
	8258,8531,5288,18368,5290,4443,18382,9043,
	18383,5292,7152,1975,2649,2929,2670,7153,
	7829,4020,18405,8265,18406,4290,8002,1291,
	7154,18416,7155,1950,7157,7159,7158,3545,
	3879,7156,3403,4182,2707,3794,7162,1025,
	7161,7168,18438,1174,4074,7164,7166,4396,
	7165,7160,7169,7167,18452,7163,7177,7174,
	7176,7180,7172,7170,7171,18468,3190,3352,
	18469,7178,1823,1425,7173,7179,7186,7192,
	7191,3725,18477,4586,7188,7190,7184,2720,
	18491,4478,7185,7189,7195,4257,9690,1577,
	18498,3464,18499,1665,7193,9687,7194,7196,
	7199,7197,4090,7187,18508,3706,18509,3806,
	4520,18510,7205,18511,2784,18512,7202,2458,
	7214,7200,18515,7203,7206,7212,4241,7207,
	1212,7220,18543,3958,7223,18548,8494,18549,
	2013,18550,7227,7224,7226,7228,8626,7221,
	18566,7219,1970,18570,1500,18571,8606,7217,
	7225,7213,7238,9692,7235,7222,18581,3253,
	8300,2813,7239,7241,7236,18605,7230,7243,
	18609,7242,2692,9696,18614,7246,8827,18620,
	7233,18621,9694,18622,7249,7244,7247,7232,
	7245,9685,18640,7240,7251,18645,7787,18646,
	7253,9691,3970,18654,8721,7256,18657,4877,
	18661,8745,9686,9693,9695,18670,7255,18671,
	3263,7254,8175,18679,7259,1347,9688,7977,
	6797,7260,18698,8310,7261,7338,3985,8855,
	9872,18708,3995,4101,8511,3899,2215,18716,
	4076,18717,8600,18718,7816,1945,5613,4169,
	7078,1152,1110,7079,18724,3321,1257,18725,
	4870,18726,3482,18727,7080,4563,18732,8844,
	7081,7358,7082,18736,4335,7083,18737,971,
	7359,1407,18742,2972,7084,18760,4871,1060,
	18767,3867,1162,2586,4621,7085,7088,9713,
	7090,8204,4874,18793,4200,4326,7361,3220,
	7768,8876,18798,7362,7089,7091,2416,7092,
	7098,7093,18810,3338,2978,2698,1877,7095,
	9895,7097,18816,1933,18830,1032,7101,7100,
	7094,7102,3707,7105,5945,18852,8161,9659,
	7104,18853,7106,18854,9662,9660,7363,2244,
	9658,9663,18889,7108,8583,18893,9835,3846,
	18909,4145,18910,7135,8502,18925,9400,8420,
	18941,9401,8189,18954,9403,7981,2158,1844,
	18955,1855,2782,3434,6458,2475,2340,7513,
	18963,7514,18964,7512,2227,18969,7515,1320,
	6592,9737,18980,9738,18981,7828,4093,8927,
	7883,7949,4862,8055,18992,8683,18993,8929,
	18994,8559,18995,8928,8682,18998,8930,8395,
	18999,8056,7906,19002,8690,19003,8528,8147,
	19007,8933,8532,19019,8935,8936,19032,6788,
	8932,19038,8810,19039,8939,19040,8937,8381,
	8950,8679,8726,8492,19053,7784,8946,7991,
	8947,19054,8943,8016,7952,8648,19055,8945,
	8941,8862,8940,8162,19067,4323,3631,3419,
	19073,7855,8758,19076,8951,8765,19080,7810,
	8118,19081,8612,8618,8952,8529,19082,8032,
	8517,19089,8156,8960,19101,8884,8553,19104,
	8957,19105,8426,19106,8948,19107,8955,19108,
	8956,8231,8288,8959,8958,7879,19115,8972,
	8949,19119,8966,19120,8970,8659,8963,19124,
	8967,8971,8031,19127,8969,8962,7940,8861,
	19130,8699,19131,8968,19132,8364,19133,8334,
	8965,8599,19134,8561,8856,19135,5948,8322,
	19141,8977,8964,19145,8975,7733,19146,8401,
	8976,8099,19147,8662,7519,19155,8973,8978,
	8333,8979,8982,8043,8980,19192,8496,8981,
	8552,19193,8388,8984,3016,8729,8727,7789,
	8440,19236,8187,8985,8974,19242,8983,19243,
	4887,2105,1512,1719,3242,2072,3625,3230,
	4890,3100,19244,4062,4208,4063,2106,19245,
	2179,2041,4081,4893,4024,1582,2690,19246,
	3524,1674,3361,1632,2342,4515,3039,4676,
	3405,19247,4426,3540,4496,4896,4574,1357,
	4898,4897,19248,4210,3435,4902,3394,1268,
	4590,4905,1989,1423,1863,4056,4207,4908,
	1726,3927,1206,19249,2236,3819,4308,4911,
	3842,4912,4282,2042,4913,3498,3525,4914,
	3176,4589,4915,2934,1536,4916,1641,2395,
	4917,4918,3490,4919,1496,4920,2571,4632,
	4921,3595,19250,4209,2836,4922,1502,2022,
	4923,3966,1214,4931,4929,4113,4930,2778,
	4932,19251,3974,4140,1028,4936,3112,4937,
	2248,2726,2818,3594,2473,3062,1528,7414,
	8392,7415,3936,2004,1911,4328,7504,1043,
	1209,1928,7506,19314,2743,7739,8831,19329,
	7948,7769,7972,8378,8037,7920,8548,7984,
	8801,8866,9387,19336,9391,7912,7993,19337,
	7752,8304,7848,19338,9388,7927,8566,9389,
	19339,8315,8005,9386,8267,8239,8026,9392,
	19340,8887,8063,19341,8805,9394,8475,7761,
	19348,9396,9395,19353,7839,19354,8472,19355,
	8372,9135,8635,8306,8085,19356,7946,9398,
	8849,9397,8824,7892,8873,9399,7976,8457,
	8903,19370,9390,8881,8806,19377,8795,8900,
	19378,8469,19379,8746,9393,7957,8796,1053,
	4491,1717,19389,1802,1165,4403,3898,999,
	4460,2058,4553,1620,3587,3029,1098,1811,
	4603,1850,1598,2157,3660,1868,6446,1406,
	2744,1646,1935,6447,4407,6448,2121,2036,
	6449,2599,2655,4386,4647,3351,1710,1539,
	6455,3457,3334,1360,5691,2979,6456,2464,
	19392,4629,6457,4617,3287,4745,4739,4385,
	19393,4412,3325,4243,1742,1285,19394,3355,
	7412,1932,19398,7413,7407,1707,4469,1738,
	3087,7408,1233,4352,3188,7409,7956,7411,
	9720,4672,2944,7452,7447,19450,7455,19451,
	7454,19452,4537,7453,4353,7456,19456,7464,
	986,1497,19461,19462,2973,7459,7463,19469,
	2319,7460,1781,7468,7471,2420,19476,1867,
	7448,7465,3659,19482,2156,19483,7469,1572,
	7470,19484,7473,7472,4256,19495,1297,3575,
	8084,7475,2320,7476,3634,7479,1167,4662,
	19516,7477,19517,7478,7480,19518,7486,8752,
	7484,7483,7481,19528,3638,5947,1433,7487,
	7492,3574,9725,1446,1195,19555,7496,7493,
	19556,1558,19557,7497,19558,7494,9727,19562,
	7495,1373,19565,4397,7498,19569,1310,19570,
	9724,7820,9730,19571,8781,19572,7499,7501,
	9732,9726,9731,7500,9729,19578,7502,19579,
	7503,7843,9736,9735,3366,1795,7803,8807,
	7990,8150,6392,8672,19616,9356,8857,9361,
	9366,9359,9362,19647,9367,19648,8113,19652,
	9370,19653,9369,19654,8792,9368,9371,19662,
	7945,8422,8230,9375,8025,7995,9372,7738,
	8283,8816,8759,9426,19692,8627,8773,9377,
	8828,8112,8006,19713,9088,9365,19714,9360,
	19715,1242,4417,1861,4036,19716,6369,4615,
	6370,2685,3272,1947,6377,6376,3166,6380,
	4379,6381,2207,19717,2208,6384,1699,2567,
	6385,1051,2025,1870,6386,19718,1685,2080,
	19719,3451,5492,4338,3882,4440,6390,4479,
	6391,3980,1813,19720,1354,1090,7728,19726,
	19727,1105,19728,7837,7755,7754,1250,3267,
	8362,2578,1394,3109,4285,19739,3096,4065,
	19740,1878,2716,4242,19746,4366,2253,19747,
	5950,1618,5951,1998,4483,19750,2249,4345,
	3764,2549,1278,3657,19753,5954,5952,5956,
	19754,5958,5955,1456,3047,19755,1501,5953,
	19756,3469,5957,9851,2777,1070,2070,3708,
	3522,3425,3622,5960,5959,2889,4042,4064,
	5961,19768,5964,19769,3689,4591,5963,1470,
	19770,3695,9202,5965,1529,19771,8830,3673,
	1853,19772,3420,1270,3534,4399,5966,1671,
	8217,5969,1409,5970,19785,4196,2694,1071,
	19788,4295,1564,3553,19791,5971,8787,19797,
	1106,7998,1587,5974,1441,7846,8590,19798,
	4172,5976,5978,8684,19801,7870,19802,8778,
	19803,5977,3119,19804,4138,5975,8499,4389,
	4475,7812,19813,5980,4686,19814,8399,8674,
	19815,8719,19816,8233,5981,19817,1093,4132,
	8307,5982,5984,8021,9201,5983,19820,7750,
	5985,8291,9203,4191,19824,1452,19825,6167,
	19826,4967,19827,4970,19828,4968,19829,4969,
	4972,3994,2858,3963,19838,4971,19839,4271,
	1889,4978,3182,19840,4974,4973,3348,4975,
	19841,4977,4667,4976,2595,4983,4980,19848,
	4984,2484,4979,4982,4514,19854,4985,4990,
	4988,1913,19861,8990,19862,2353,4987,4986,
	4362,1158,4991,19868,1874,19869,4992,1248,
	8755,1531,4993,19880,1588,19883,4994,19884,
	8992,8892,19895,8988,19896,8785,4995,4997,
	8839,19914,4999,8237,4998,7851,19915,8989,
	19916,8991,5000,8987,5002,19929,8993,4276,
	7416,3186,4640,2981,7417,19930,2293,4368,
	3581,7422,7421,1887,7420,3531,7424,2493,
	19944,1295,19945,3675,7425,7423,2181,7428,
	19946,7429,2206,2749,2414,3542,7430,7432,
	19955,1343,19956,4682,19957,1370,7431,7434,
	7433,7435,3996,2774,3204,7823,7438,8716,
	8100,7443,7442,8347,8664,19990,9722,19991,
	9721,19992,1168,19993,4281,3428,8500,2522,
	4567,4151,2568,19994,2241,9464,19995,9463,
	7880,9462,1701,8832,7877,9467,9466,20005,
	7917,9469,20017,9465,20018,9470,8397,9480,
	9476,9471,8336,7901,7973,20032,9475,9474,
	7802,8358,7953,20047,9479,9472,9477,9497,
	20054,9493,9484,8241,9483,9487,9498,9481,
	9486,8756,9491,8064,9495,9494,9485,20076,
	8403,8314,8398,8106,20101,9502,7967,9517,
	8733,9522,20107,8571,8623,20118,9516,9512,
	20121,8332,20122,9519,20123,8636,9525,20129,
	8717,9510,20130,9524,9514,9503,9521,9500,
	20131,7553,8653,20139,8666,20140,8562,9534,
	8271,20145,9539,20146,8663,20147,7740,7554,
	9535,20165,9540,9507,7824,9530,9541,20169,
	9533,20170,8385,8451,9504,9532,9531,9528,
	9536,20177,8141,8880,20196,9542,7834,9554,
	9520,9545,9553,7882,20204,8402,8120,20208,
	8313,9549,7845,8268,8320,9837,20217,9527,
	7555,20223,9546,8632,20224,9550,9468,9556,
	7996,20230,7893,9559,9555,9544,7556,20253,
	8412,8087,9557,8316,20270,9562,8542,20285,
	9568,8610,9044,9571,9511,20297,9518,9560,
	20298,7963,20299,8835,20300,9572,8352,20303,
	9573,9581,8224,20318,7557,20319,9567,9578,
	7681,9529,9564,9579,7791,20330,8132,9575,
	20331,9563,9515,20347,9585,20348,9591,8234,
	20369,9526,9587,9583,8851,9592,9594,8198,
	20391,8567,9499,20394,9508,7559,9595,9593,
	20407,9574,8083,20413,9596,9597,20427,9586,
	7560,20433,9588,8782,20436,8646,8351,20441,
	9590,8292,8895,9756,8798,4492,1507,6802,
	6801,20449,1613,1495,6807,6809,20450,6808,
	20451,1729,1562,1235,4562,2857,1054,1745,
	4354,3154,2346,3816,1804,6816,6815,6818,
	6817,2918,3115,6821,3116,6822,1134,2123,
	6830,4272,3661,1139,2606,6831,3107,2738,
	20452,6846,6845,20456,6847,3678,2664,4418,
	6851,3872,6857,6856,6858,1776,2815,2195,
	4166,1216,4223,6864,4604,6865,3051,20458,
	2557,6868,3943,3565,6870,6869,1311,1873,
	1667,3978,3276,3635,4481,6883,1392,2735,
	6884,20459,20460,3852,6888,2695,1339,4627,
	2246,20461,3892,6891,6892,1510,2159,2321,
	2769,20462,6895,6903,3135,6937,1548,6899,
	20463,1541,2755,6902,20464,4499,20465,6907,
	2906,20466,6908,2907,1759,1026,20468,6918,
	2276,6921,2499,20470,6933,2550,3919,7797,
	1225,8317,20480,9151,8467,7749,20485,8152,
	9156,20486,9154,8452,20487,8637,20488,8071,
	9155,8003,20500,7966,9849,7915,7989,8330,
	20503,9159,9161,9158,8691,20513,8695,9167,
	9166,9162,9165,9836,20519,9153,20520,8174,
	9169,8184,9170,9172,7832,7980,20532,9173,
	7793,9873,2762,5765,3322,5766,20539,1086,
	3801,1334,3277,5767,3901,5768,2135,2763,
	4419,2873,1860,3796,5771,2811,5772,20540,
	1604,1774,1929,4359,5775,20541,5776,4084,
	4095,5781,1218,2470,5782,20542,2454,20544,
	1714,4948,20545,1554,20546,4950,4949,4951,
	1629,4125,4220,4500,2216,4677,4954,4953,
	20563,941,3716,20564,4955,20565,1720,2109,
	2660,2635,1255,4956,20566,2641,2835,2182,
	4957,3323,1094,20576,8468,20577,4958,20578,
	1527,4349,8836,1314,4959,4364,3906,2980,
	20581,4960,8732,20585,4961,7806,4962,2611,
	3624,3914,8269,4300,2632,20597,4963,20598,
	7897,3546,20599,4964,8114,3547,4229,1775,
	8786,20603,4965,954,3875,20604,8057,4464,
	3554,8535,20615,8639,4966,8735,20621,5764,
	8253,20622,2542,20623,8213,7545,20624,9893,
	2868,20625,3219,4108,4006,4078,2082,1830,
	20628,6940,1353,4255,7548,1312,1490,8534,
	8516,20637,7825,8791,7551,20641,8202,8338,
	4303,7521,4050,8784,7522,2604,2498,20654,
	1035,20655,7875,3836,20658,4019,7525,3939,
	20661,7524,4497,7526,2750,2057,7528,7527,
	3487,20675,3881,7531,20690,2653,991,3002,
	9741,7532,9740,20702,9742,3165,7520,2273,
	9739,1478,1636,20721,2380,2775,2795,8899,
	1770,2250,4046,20735,987,3959,20754,955,
	20755,7971,20756,20757,7628,7630,20776,1095,
	20777,7629,7632,20780,7633,9823,20798,8589,
	8445,9319,20812,9318,3763,3240,20820,1890,
	4219,20826,4370,3345,8649,20835,8713,7881,
	8425,20836,8650,8518,9669,8668,20837,9310,
	20838,8527,20839,8769,8586,7727,7900,8383,
	8244,8718,20852,9675,9676,8131,8377,20869,
	8577,8563,7905,9677,9678,8694,8779,9681,
	20890,7872,9682,20898,7978,7794,9683,8638,
	9684,8260,20904,9679,8435,4154,1508,3175,
	7136,3931,3496,4022,6183,3736,1828,1560,
	7137,1007,3521,7138,4327,2644,2613,3043,
	2272,7139,2120,20905,20906,7142,4170,3028,
	20907,3704,7143,20908,4246,2386,3637,20909,
	4094,1581,20910,1219,3202,7936,20916,9880,
	20917,9848,8376,20932,9424,20933,9425,20941,
	6586,1638,3402,9115,20946,9854,5658,7655,
	9117,20951,9121,20952,7921,8734,20955,9122,
	8523,7734,8501,20963,8109,20967,7763,9123,
	8707,20971,7911,7656,1172,9124,8343,7908,
	9118,21006,8245,9127,9128,21010,8309,8171,
	9132,8042,8441,21017,7659,7658,21018,9830,
	9831,21023,7788,9133,5661,21027,2069,21028,
	1622,4226,2165,3429,1038,3516,21029,5668,
	1595,3231,5669,2201,21032,1126,5670,21033,
	1589,5671,2876,5672,3911,1846,5673,2444,
	21036,5674,1213,21037,5675,21038,2620,3438,
	4718,4857,3920,8301,8770,7938,8579,21057,
	7813,8681,8869,9223,8138,21084,9218,8066,
	9224,9220,21087,8497,21088,8580,21089,9219,
	21090,8302,9227,8151,21117,7811,9230,8391,
	9229,9200,21153,9233,9236,21154,8560,9221,
	8460,9237,8294,21165,9042,9235,9232,9225,
	21172,9240,8103,21178,8700,8129,9222,8270,
	21186,9242,9241,9228,21191,2708,4329,3717,
	4058,1280,3194,21192,1148,2662,6074,3410,
	2307,6077,4609,3718,6079,2128,6078,2709,
	21193,2191,6082,2701,1886,6083,21194,6084,
	1271,4114,2354,6085,3086,6088,3020,6089,
	21199,3296,4583,6097,21200,6098,1824,7721,
	7635,7634,7638,7636,1880,21211,7639,21212,
	7640,7642,7641,7644,9825,21225,7647,9892,
	3549,8564,9827,9826,21226,1754,7660,21233,
	7661,7664,21242,7666,9845,7663,7667,21252,
	7665,8524,21262,7668,9850,21267,7669,9832,
	7672,21283,7888,21284,8342,21285,9164,9160,
	5019,21290,8766,4704,6003,1862,2442,2047,
	7649,3046,7648,21303,7650,7652,7651,7653,
	9829,3785,21307,7654,9828,21313,8761,9760,
	21349,9761,9762,21360,9767,7737,9765,9774,
	9771,9770,9773,21381,9768,8633,9782,9776,
	9784,21404,8205,9783,9797,21414,9786,9795,
	21419,9792,9789,9793,21423,9790,9791,8128,
	21426,9785,21442,9804,9799,9803,9800,21456,
	9802,8456,9780,21471,9809,9808,9810,9778,
	9806,9811,21480,9815,21481,9781,21482,9779,
	21483,9814,9813,7757,9819,21502,9818,21503,
	9817,8238,9775,9798,21508,9766,9763,21531,
	9777,21532,4296,7561,21535,2650,7562,7568,
	21540,1044,7569,21541,3895,21547,21548,7588,
	21549,2263,21551,2596,7622,7941,8331,21578,
	9601,9603,9602,21594,8686,8771,21611,9607,
	21612,9608,8701,21618,8687,9610,9612,8144,
	9618,9615,9620,7904,9617,9621,9623,21672,
	8374,9605,21697,9625,9626,21711,9627,9824,
	9632,9628,21743,9631,21744,8044,8366,21762,
	9611,9635,9609,9637,9638,9619,9639,8738,
	9641,9633,9642,21816,9616,9614,2900,6965,
	2073,6966,2814,21817,2937,4068,21818,4069,
	21819,4117,21820,6973,6972,4330,21821,3715,
	6974,6976,6975,1766,6979,1951,21824,2328,
	6982,1579,21825,6989,2997,21826,6990,6991,
	21830,6992,7637,21831,21832,6996,6997,1934,
	21837,4234,7005,21840,7007,21841,9864,9886,
	9723,21845,8076,8692,7446,2656,7680,2651,
	7682,21868,7683,8305,2714,7406,9871,2704,
	21889,8922,3465,2515,6954,21901,1937,7873,
	7688,7687,21909,7689,7692,21912,7691,7859,
	7690,7693,9834,21919,9866,9833,21922,6741,
	21923,9752,7542,9753,9754,21933,7543,21934,
	7544,1509,21935,4724,1821,21938,9843,21939,
	4721,21940,5087,3466,21947,7695,7699,21966,
	7698,21967,1072,21968,8390,8811,9427,3082,
	6594,7814,21982,9743,9744,9746,8240,21991,
	9841,9750,21997,9749,1282,7533,7534,2605,
	3198,7541,8247,7970,9453,2627,1793,6740,
	7988,1859,4851,22056,22064,22076,22093,22110,
	22126,695,698,573,575,574,10013,598,
	10014,355,10015,265,
};

pdf_cmap pdf_cmap_UniGB_UTF16_H =
{
	-1, "UniGB-UTF16-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 4, 0xdc00, 0xdfff },
		{ 2, 0xe000, 0xffff },
	},
	13876, 13876, (pdf_range*) pdf_cmap_UniGB_UTF16_H_ranges,
	9548, 9548, (unsigned short*) pdf_cmap_UniGB_UTF16_H_table,
};

/* UniGB-UTF16-V */

static const pdf_range pdf_cmap_UniGB_UTF16_V_ranges[] =
{
	{ 0x2014, 0x2014, PDF_CMAP_SINGLE, 598 },
	{ 0x2026, 0x2026, PDF_CMAP_SINGLE, 599 },
	{ 0x3001, 0x3002, PDF_CMAP_TABLE, 0 },
	{ 0x3008, 0x300f, PDF_CMAP_RANGE, 584 },
	{ 0x3010, 0x3011, PDF_CMAP_RANGE, 594 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 7706 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 582 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 592 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 578 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 580 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 573 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 7707 },
	{ 0xff1a, 0xff1b, PDF_CMAP_RANGE, 576 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7708 },
	{ 0xff1f, 0xff1f, PDF_CMAP_SINGLE, 579 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7709 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7710 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 600 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 596 },
	{ 0xff5d, 0xff5e, PDF_CMAP_TABLE, 2 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7711 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7704 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7711 },
};

static const unsigned short pdf_cmap_UniGB_UTF16_V_table[4] =
{
	575,574,597,7704,
};

pdf_cmap pdf_cmap_UniGB_UTF16_V =
{
	-1, "UniGB-UTF16-V", "UniGB-UTF16-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	23, 23, (pdf_range*) pdf_cmap_UniGB_UTF16_V_ranges,
	4, 4, (unsigned short*) pdf_cmap_UniGB_UTF16_V_table,
};

