#include "fitz.h"
#include "mupdf.h"

/* 78-EUC-H */

static const pdf_range pdf_cmap_78_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
	{ 0x8ea0, 0x8edf, PDF_CMAP_RANGE, 326 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 633 },
	{ 0xa2a1, 0xa2ae, PDF_CMAP_RANGE, 727 },
	{ 0xa3b0, 0xa3b9, PDF_CMAP_RANGE, 780 },
	{ 0xa3c1, 0xa3da, PDF_CMAP_RANGE, 790 },
	{ 0xa3e1, 0xa3fa, PDF_CMAP_RANGE, 816 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 842 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 925 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 1011 },
	{ 0xa6c1, 0xa6d8, PDF_CMAP_RANGE, 1035 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 1059 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 1092 },
	{ 0xb0a1, 0xb0a2, PDF_CMAP_TABLE, 0 },
	{ 0xb0a3, 0xb0a8, PDF_CMAP_RANGE, 1127 },
	{ 0xb0a9, 0xb0a9, PDF_CMAP_SINGLE, 8266 },
	{ 0xb0aa, 0xb0b1, PDF_CMAP_RANGE, 1134 },
	{ 0xb0b2, 0xb0b3, PDF_CMAP_TABLE, 2 },
	{ 0xb0b4, 0xb0ba, PDF_CMAP_RANGE, 1144 },
	{ 0xb0bb, 0xb0bb, PDF_CMAP_SINGLE, 7634 },
	{ 0xb0bc, 0xb0ed, PDF_CMAP_RANGE, 1152 },
	{ 0xb0ee, 0xb0ee, PDF_CMAP_SINGLE, 7635 },
	{ 0xb0ef, 0xb0f2, PDF_CMAP_RANGE, 1203 },
	{ 0xb0f3, 0xb0f3, PDF_CMAP_SINGLE, 7636 },
	{ 0xb0f4, 0xb0fb, PDF_CMAP_RANGE, 1208 },
	{ 0xb0fc, 0xb0fc, PDF_CMAP_SINGLE, 7637 },
	{ 0xb0fd, 0xb0fe, PDF_CMAP_RANGE, 1217 },
	{ 0xb1a1, 0xb1a9, PDF_CMAP_RANGE, 1219 },
	{ 0xb1aa, 0xb1aa, PDF_CMAP_SINGLE, 7638 },
	{ 0xb1ab, 0xb1b4, PDF_CMAP_RANGE, 1229 },
	{ 0xb1b5, 0xb1b5, PDF_CMAP_SINGLE, 7639 },
	{ 0xb1b6, 0xb1b8, PDF_CMAP_RANGE, 1240 },
	{ 0xb1b9, 0xb1b9, PDF_CMAP_SINGLE, 7640 },
	{ 0xb1ba, 0xb1bc, PDF_CMAP_RANGE, 1244 },
	{ 0xb1bd, 0xb1bd, PDF_CMAP_SINGLE, 7642 },
	{ 0xb1be, 0xb1c1, PDF_CMAP_RANGE, 1248 },
	{ 0xb1c2, 0xb1c2, PDF_CMAP_SINGLE, 7643 },
	{ 0xb1c3, 0xb1ea, PDF_CMAP_RANGE, 1253 },
	{ 0xb1eb, 0xb1eb, PDF_CMAP_SINGLE, 7644 },
	{ 0xb1ec, 0xb1fe, PDF_CMAP_RANGE, 1294 },
	{ 0xb2a1, 0xb2a7, PDF_CMAP_RANGE, 1313 },
	{ 0xb2a8, 0xb2aa, PDF_CMAP_TABLE, 4 },
	{ 0xb2ab, 0xb2df, PDF_CMAP_RANGE, 1323 },
	{ 0xb2e0, 0xb2e0, PDF_CMAP_SINGLE, 7647 },
	{ 0xb2e1, 0xb2f9, PDF_CMAP_RANGE, 1377 },
	{ 0xb2fa, 0xb2fa, PDF_CMAP_SINGLE, 7648 },
	{ 0xb2fb, 0xb2fc, PDF_CMAP_RANGE, 1403 },
	{ 0xb2fd, 0xb2fe, PDF_CMAP_TABLE, 7 },
	{ 0xb3a1, 0xb3a2, PDF_CMAP_TABLE, 9 },
	{ 0xb3a3, 0xb3c1, PDF_CMAP_RANGE, 1409 },
	{ 0xb3c2, 0xb3c2, PDF_CMAP_SINGLE, 6497 },
	{ 0xb3c3, 0xb3c8, PDF_CMAP_RANGE, 1441 },
	{ 0xb3c9, 0xb3c9, PDF_CMAP_SINGLE, 5023 },
	{ 0xb3ca, 0xb3e4, PDF_CMAP_RANGE, 1448 },
	{ 0xb3e5, 0xb3e5, PDF_CMAP_SINGLE, 7651 },
	{ 0xb3e6, 0xb3ea, PDF_CMAP_RANGE, 1476 },
	{ 0xb3eb, 0xb3eb, PDF_CMAP_SINGLE, 7652 },
	{ 0xb3ec, 0xb3f2, PDF_CMAP_RANGE, 1482 },
	{ 0xb3f3, 0xb3f3, PDF_CMAP_SINGLE, 7653 },
	{ 0xb3f4, 0xb3f5, PDF_CMAP_RANGE, 1490 },
	{ 0xb3f6, 0xb3f6, PDF_CMAP_SINGLE, 5937 },
	{ 0xb3f7, 0xb3f9, PDF_CMAP_RANGE, 1493 },
	{ 0xb3fa, 0xb3fa, PDF_CMAP_SINGLE, 7654 },
	{ 0xb3fb, 0xb3fe, PDF_CMAP_RANGE, 1497 },
	{ 0xb4a1, 0xb4c1, PDF_CMAP_RANGE, 1501 },
	{ 0xb4c2, 0xb4c3, PDF_CMAP_TABLE, 11 },
	{ 0xb4c4, 0xb4cc, PDF_CMAP_RANGE, 1536 },
	{ 0xb4cd, 0xb4cd, PDF_CMAP_SINGLE, 7656 },
	{ 0xb4ce, 0xb4d1, PDF_CMAP_RANGE, 1546 },
	{ 0xb4d2, 0xb4d2, PDF_CMAP_SINGLE, 6688 },
	{ 0xb4d3, 0xb4e4, PDF_CMAP_RANGE, 1551 },
	{ 0xb4e5, 0xb4e5, PDF_CMAP_SINGLE, 7657 },
	{ 0xb4e6, 0xb4fe, PDF_CMAP_RANGE, 1570 },
	{ 0xb5a1, 0xb5aa, PDF_CMAP_RANGE, 1595 },
	{ 0xb5ab, 0xb5ab, PDF_CMAP_SINGLE, 7658 },
	{ 0xb5ac, 0xb5bf, PDF_CMAP_RANGE, 1606 },
	{ 0xb5c0, 0xb5c0, PDF_CMAP_SINGLE, 7659 },
	{ 0xb5c1, 0xb5fe, PDF_CMAP_RANGE, 1627 },
	{ 0xb6a1, 0xb6a2, PDF_CMAP_TABLE, 13 },
	{ 0xb6a3, 0xb6a9, PDF_CMAP_RANGE, 1691 },
	{ 0xb6aa, 0xb6aa, PDF_CMAP_SINGLE, 7661 },
	{ 0xb6ab, 0xb6c5, PDF_CMAP_RANGE, 1699 },
	{ 0xb6c6, 0xb6c6, PDF_CMAP_SINGLE, 7474 },
	{ 0xb6c7, 0xb6ce, PDF_CMAP_RANGE, 1727 },
	{ 0xb6cf, 0xb6cf, PDF_CMAP_SINGLE, 7662 },
	{ 0xb6d0, 0xb6ec, PDF_CMAP_RANGE, 1736 },
	{ 0xb6ed, 0xb6ed, PDF_CMAP_SINGLE, 7663 },
	{ 0xb6ee, 0xb6f3, PDF_CMAP_RANGE, 1766 },
	{ 0xb6f4, 0xb6f4, PDF_CMAP_SINGLE, 7664 },
	{ 0xb6f5, 0xb6fa, PDF_CMAP_RANGE, 1773 },
	{ 0xb6fb, 0xb6fe, PDF_CMAP_TABLE, 15 },
	{ 0xb7a1, 0xb7a3, PDF_CMAP_RANGE, 1783 },
	{ 0xb7a4, 0xb7a4, PDF_CMAP_SINGLE, 7667 },
	{ 0xb7a5, 0xb7b6, PDF_CMAP_RANGE, 1787 },
	{ 0xb7b7, 0xb7b7, PDF_CMAP_SINGLE, 7668 },
	{ 0xb7b8, 0xb7c4, PDF_CMAP_RANGE, 1806 },
	{ 0xb7c5, 0xb7c5, PDF_CMAP_SINGLE, 7669 },
	{ 0xb7c6, 0xb7cd, PDF_CMAP_RANGE, 1820 },
	{ 0xb7ce, 0xb7ce, PDF_CMAP_SINGLE, 7670 },
	{ 0xb7cf, 0xb7d1, PDF_CMAP_RANGE, 1829 },
	{ 0xb7d2, 0xb7d2, PDF_CMAP_SINGLE, 7671 },
	{ 0xb7d3, 0xb7d4, PDF_CMAP_RANGE, 1833 },
	{ 0xb7d5, 0xb7d5, PDF_CMAP_SINGLE, 7672 },
	{ 0xb7d6, 0xb7da, PDF_CMAP_RANGE, 1836 },
	{ 0xb7db, 0xb7db, PDF_CMAP_SINGLE, 7181 },
	{ 0xb7dc, 0xb7e3, PDF_CMAP_RANGE, 1842 },
	{ 0xb7e4, 0xb7e4, PDF_CMAP_SINGLE, 7673 },
	{ 0xb7e5, 0xb7f0, PDF_CMAP_RANGE, 1851 },
	{ 0xb7f1, 0xb7f1, PDF_CMAP_SINGLE, 7674 },
	{ 0xb7f2, 0xb7f8, PDF_CMAP_RANGE, 1864 },
	{ 0xb7f9, 0xb7f9, PDF_CMAP_SINGLE, 7675 },
	{ 0xb7fa, 0xb7fd, PDF_CMAP_RANGE, 1872 },
	{ 0xb7fe, 0xb7fe, PDF_CMAP_SINGLE, 7676 },
	{ 0xb8a1, 0xb8b3, PDF_CMAP_RANGE, 1877 },
	{ 0xb8b4, 0xb8b4, PDF_CMAP_SINGLE, 7677 },
	{ 0xb8b5, 0xb8c0, PDF_CMAP_RANGE, 1897 },
	{ 0xb8c1, 0xb8c1, PDF_CMAP_SINGLE, 7678 },
	{ 0xb8c2, 0xb8fe, PDF_CMAP_RANGE, 1910 },
	{ 0xb9a1, 0xb9aa, PDF_CMAP_RANGE, 1971 },
	{ 0xb9ab, 0xb9ab, PDF_CMAP_SINGLE, 7679 },
	{ 0xb9ac, 0xb9b6, PDF_CMAP_RANGE, 1982 },
	{ 0xb9b7, 0xb9b7, PDF_CMAP_SINGLE, 7680 },
	{ 0xb9b8, 0xb9c1, PDF_CMAP_RANGE, 1994 },
	{ 0xb9c2, 0xb9c2, PDF_CMAP_SINGLE, 7681 },
	{ 0xb9c3, 0xb9db, PDF_CMAP_RANGE, 2005 },
	{ 0xb9dc, 0xb9dc, PDF_CMAP_SINGLE, 5853 },
	{ 0xb9dd, 0xb9ec, PDF_CMAP_RANGE, 2031 },
	{ 0xb9ed, 0xb9ed, PDF_CMAP_SINGLE, 7682 },
	{ 0xb9ee, 0xb9f3, PDF_CMAP_RANGE, 2048 },
	{ 0xb9f4, 0xb9f4, PDF_CMAP_SINGLE, 7683 },
	{ 0xb9f5, 0xb9f8, PDF_CMAP_RANGE, 2055 },
	{ 0xb9f9, 0xb9f9, PDF_CMAP_SINGLE, 7684 },
	{ 0xb9fa, 0xb9fe, PDF_CMAP_RANGE, 2060 },
	{ 0xbaa1, 0xbad2, PDF_CMAP_RANGE, 2065 },
	{ 0xbad3, 0xbad3, PDF_CMAP_SINGLE, 7685 },
	{ 0xbad4, 0xbae6, PDF_CMAP_RANGE, 2116 },
	{ 0xbae7, 0xbae7, PDF_CMAP_SINGLE, 7686 },
	{ 0xbae8, 0xbaf3, PDF_CMAP_RANGE, 2136 },
	{ 0xbaf4, 0xbaf4, PDF_CMAP_SINGLE, 7687 },
	{ 0xbaf5, 0xbafe, PDF_CMAP_RANGE, 2149 },
	{ 0xbba1, 0xbba6, PDF_CMAP_RANGE, 2159 },
	{ 0xbba7, 0xbba7, PDF_CMAP_SINGLE, 7688 },
	{ 0xbba8, 0xbba9, PDF_CMAP_RANGE, 2166 },
	{ 0xbbaa, 0xbbac, PDF_CMAP_TABLE, 19 },
	{ 0xbbad, 0xbbb8, PDF_CMAP_RANGE, 2171 },
	{ 0xbbb9, 0xbbb9, PDF_CMAP_SINGLE, 7691 },
	{ 0xbbba, 0xbbfe, PDF_CMAP_RANGE, 2184 },
	{ 0xbca1, 0xbcc7, PDF_CMAP_RANGE, 2253 },
	{ 0xbcc8, 0xbcc9, PDF_CMAP_TABLE, 22 },
	{ 0xbcca, 0xbcd6, PDF_CMAP_RANGE, 2294 },
	{ 0xbcd7, 0xbcd7, PDF_CMAP_SINGLE, 7694 },
	{ 0xbcd8, 0xbcdc, PDF_CMAP_RANGE, 2308 },
	{ 0xbcdd, 0xbcde, PDF_CMAP_RANGE, 7695 },
	{ 0xbcdf, 0xbcfe, PDF_CMAP_RANGE, 2315 },
	{ 0xbda1, 0xbdaa, PDF_CMAP_RANGE, 2347 },
	{ 0xbdab, 0xbdab, PDF_CMAP_SINGLE, 7697 },
	{ 0xbdac, 0xbdb5, PDF_CMAP_RANGE, 2358 },
	{ 0xbdb6, 0xbdb6, PDF_CMAP_SINGLE, 7698 },
	{ 0xbdb7, 0xbdeb, PDF_CMAP_RANGE, 2369 },
	{ 0xbdec, 0xbded, PDF_CMAP_RANGE, 7699 },
	{ 0xbdee, 0xbdf1, PDF_CMAP_RANGE, 2424 },
	{ 0xbdf2, 0xbdf3, PDF_CMAP_RANGE, 7701 },
	{ 0xbdf4, 0xbdfe, PDF_CMAP_RANGE, 2430 },
	{ 0xbea1, 0xbea4, PDF_CMAP_RANGE, 2441 },
	{ 0xbea5, 0xbea5, PDF_CMAP_SINGLE, 7703 },
	{ 0xbea6, 0xbeb2, PDF_CMAP_RANGE, 2446 },
	{ 0xbeb3, 0xbeb3, PDF_CMAP_SINGLE, 7704 },
	{ 0xbeb4, 0xbebe, PDF_CMAP_RANGE, 2460 },
	{ 0xbebf, 0xbebf, PDF_CMAP_SINGLE, 7705 },
	{ 0xbec0, 0xbed4, PDF_CMAP_RANGE, 2472 },
	{ 0xbed5, 0xbed5, PDF_CMAP_SINGLE, 7706 },
	{ 0xbed6, 0xbede, PDF_CMAP_RANGE, 2494 },
	{ 0xbedf, 0xbedf, PDF_CMAP_SINGLE, 7707 },
	{ 0xbee0, 0xbee3, PDF_CMAP_RANGE, 2504 },
	{ 0xbee4, 0xbee4, PDF_CMAP_SINGLE, 7708 },
	{ 0xbee5, 0xbefe, PDF_CMAP_RANGE, 2509 },
	{ 0xbfa1, 0xbfa9, PDF_CMAP_RANGE, 2535 },
	{ 0xbfaa, 0xbfaa, PDF_CMAP_SINGLE, 7709 },
	{ 0xbfab, 0xbfd8, PDF_CMAP_RANGE, 2545 },
	{ 0xbfd9, 0xbfd9, PDF_CMAP_SINGLE, 7710 },
	{ 0xbfda, 0xbfdf, PDF_CMAP_RANGE, 2592 },
	{ 0xbfe0, 0xbfe0, PDF_CMAP_SINGLE, 7711 },
	{ 0xbfe1, 0xbfe8, PDF_CMAP_RANGE, 2599 },
	{ 0xbfe9, 0xbfe9, PDF_CMAP_SINGLE, 7712 },
	{ 0xbfea, 0xbffe, PDF_CMAP_RANGE, 2608 },
	{ 0xc0a1, 0xc0a2, PDF_CMAP_TABLE, 24 },
	{ 0xc0a3, 0xc0c1, PDF_CMAP_RANGE, 2631 },
	{ 0xc0c2, 0xc0c2, PDF_CMAP_SINGLE, 7714 },
	{ 0xc0c3, 0xc0e5, PDF_CMAP_RANGE, 2663 },
	{ 0xc0e6, 0xc0e6, PDF_CMAP_SINGLE, 7715 },
	{ 0xc0e7, 0xc0f0, PDF_CMAP_RANGE, 2699 },
	{ 0xc0f1, 0xc0f2, PDF_CMAP_RANGE, 7716 },
	{ 0xc0f3, 0xc0f8, PDF_CMAP_RANGE, 2711 },
	{ 0xc0f9, 0xc0fa, PDF_CMAP_RANGE, 7718 },
	{ 0xc0fb, 0xc0fe, PDF_CMAP_RANGE, 2719 },
	{ 0xc1a1, 0xc1a6, PDF_CMAP_RANGE, 2723 },
	{ 0xc1a7, 0xc1a8, PDF_CMAP_TABLE, 26 },
	{ 0xc1a9, 0xc1b8, PDF_CMAP_RANGE, 2731 },
	{ 0xc1b9, 0xc1b9, PDF_CMAP_SINGLE, 7721 },
	{ 0xc1ba, 0xc1cb, PDF_CMAP_RANGE, 2748 },
	{ 0xc1cc, 0xc1cc, PDF_CMAP_SINGLE, 7722 },
	{ 0xc1cd, 0xc1ce, PDF_CMAP_RANGE, 2767 },
	{ 0xc1cf, 0xc1cf, PDF_CMAP_SINGLE, 7723 },
	{ 0xc1d0, 0xc1de, PDF_CMAP_RANGE, 2770 },
	{ 0xc1df, 0xc1df, PDF_CMAP_SINGLE, 7724 },
	{ 0xc1e0, 0xc1e8, PDF_CMAP_RANGE, 2786 },
	{ 0xc1e9, 0xc1e9, PDF_CMAP_SINGLE, 7725 },
	{ 0xc1ea, 0xc1fe, PDF_CMAP_RANGE, 2796 },
	{ 0xc2a1, 0xc2bc, PDF_CMAP_RANGE, 2817 },
	{ 0xc2bd, 0xc2bd, PDF_CMAP_SINGLE, 7726 },
	{ 0xc2be, 0xc2cc, PDF_CMAP_RANGE, 2846 },
	{ 0xc2cd, 0xc2cd, PDF_CMAP_SINGLE, 7727 },
	{ 0xc2ce, 0xc2db, PDF_CMAP_RANGE, 2862 },
	{ 0xc2dc, 0xc2dc, PDF_CMAP_SINGLE, 7728 },
	{ 0xc2dd, 0xc2e2, PDF_CMAP_RANGE, 2877 },
	{ 0xc2e3, 0xc2e3, PDF_CMAP_SINGLE, 7729 },
	{ 0xc2e4, 0xc2ee, PDF_CMAP_RANGE, 2884 },
	{ 0xc2ef, 0xc2ef, PDF_CMAP_SINGLE, 7730 },
	{ 0xc2f0, 0xc2f4, PDF_CMAP_RANGE, 2896 },
	{ 0xc2f5, 0xc2f6, PDF_CMAP_RANGE, 7731 },
	{ 0xc2f7, 0xc2fc, PDF_CMAP_RANGE, 2903 },
	{ 0xc2fd, 0xc2fe, PDF_CMAP_TABLE, 28 },
	{ 0xc3a1, 0xc3a6, PDF_CMAP_RANGE, 2911 },
	{ 0xc3a7, 0xc3a8, PDF_CMAP_TABLE, 30 },
	{ 0xc3a9, 0xc3aa, PDF_CMAP_RANGE, 7735 },
	{ 0xc3ab, 0xc3ac, PDF_CMAP_RANGE, 2921 },
	{ 0xc3ad, 0xc3ae, PDF_CMAP_RANGE, 7737 },
	{ 0xc3af, 0xc3bc, PDF_CMAP_RANGE, 2925 },
	{ 0xc3bd, 0xc3bd, PDF_CMAP_SINGLE, 7739 },
	{ 0xc3be, 0xc3ef, PDF_CMAP_RANGE, 2940 },
	{ 0xc3f0, 0xc3f0, PDF_CMAP_SINGLE, 7740 },
	{ 0xc3f1, 0xc3f4, PDF_CMAP_RANGE, 2991 },
	{ 0xc3f5, 0xc3f5, PDF_CMAP_SINGLE, 7741 },
	{ 0xc3f6, 0xc3fb, PDF_CMAP_RANGE, 2996 },
	{ 0xc3fc, 0xc3fc, PDF_CMAP_SINGLE, 7742 },
	{ 0xc3fd, 0xc3fe, PDF_CMAP_RANGE, 3003 },
	{ 0xc4a1, 0xc4bc, PDF_CMAP_RANGE, 3005 },
	{ 0xc4bd, 0xc4bd, PDF_CMAP_SINGLE, 7743 },
	{ 0xc4be, 0xc4c7, PDF_CMAP_RANGE, 3034 },
	{ 0xc4c8, 0xc4ca, PDF_CMAP_TABLE, 32 },
	{ 0xc4cb, 0xc4cc, PDF_CMAP_RANGE, 3047 },
	{ 0xc4cd, 0xc4cf, PDF_CMAP_TABLE, 35 },
	{ 0xc4d0, 0xc4d3, PDF_CMAP_RANGE, 3052 },
	{ 0xc4d4, 0xc4d4, PDF_CMAP_SINGLE, 8267 },
	{ 0xc4d5, 0xc4da, PDF_CMAP_RANGE, 3057 },
	{ 0xc4db, 0xc4db, PDF_CMAP_SINGLE, 4533 },
	{ 0xc4dc, 0xc4fe, PDF_CMAP_RANGE, 3064 },
	{ 0xc5a1, 0xc5a2, PDF_CMAP_TABLE, 38 },
	{ 0xc5a3, 0xc5a6, PDF_CMAP_RANGE, 3101 },
	{ 0xc5a7, 0xc5a7, PDF_CMAP_SINGLE, 7749 },
	{ 0xc5a8, 0xc5ad, PDF_CMAP_RANGE, 3106 },
	{ 0xc5ae, 0xc5ae, PDF_CMAP_SINGLE, 7750 },
	{ 0xc5af, 0xc5b5, PDF_CMAP_RANGE, 3113 },
	{ 0xc5b6, 0xc5b6, PDF_CMAP_SINGLE, 7751 },
	{ 0xc5b7, 0xc5be, PDF_CMAP_RANGE, 3121 },
	{ 0xc5bf, 0xc5bf, PDF_CMAP_SINGLE, 7752 },
	{ 0xc5c0, 0xc5c7, PDF_CMAP_RANGE, 3130 },
	{ 0xc5c8, 0xc5c8, PDF_CMAP_SINGLE, 7753 },
	{ 0xc5c9, 0xc5ca, PDF_CMAP_RANGE, 3139 },
	{ 0xc5cb, 0xc5cb, PDF_CMAP_SINGLE, 7754 },
	{ 0xc5cc, 0xc5d0, PDF_CMAP_RANGE, 3142 },
	{ 0xc5d1, 0xc5d2, PDF_CMAP_RANGE, 7755 },
	{ 0xc5d3, 0xc5d6, PDF_CMAP_RANGE, 3149 },
	{ 0xc5d7, 0xc5d7, PDF_CMAP_SINGLE, 5855 },
	{ 0xc5d8, 0xc5e3, PDF_CMAP_RANGE, 3154 },
	{ 0xc5e4, 0xc5e4, PDF_CMAP_SINGLE, 7757 },
	{ 0xc5e5, 0xc5ed, PDF_CMAP_RANGE, 3167 },
	{ 0xc5ee, 0xc5ee, PDF_CMAP_SINGLE, 5200 },
	{ 0xc5ef, 0xc5f2, PDF_CMAP_RANGE, 3177 },
	{ 0xc5f3, 0xc5f3, PDF_CMAP_SINGLE, 5430 },
	{ 0xc5f4, 0xc5f7, PDF_CMAP_RANGE, 3182 },
	{ 0xc5f8, 0xc5f8, PDF_CMAP_SINGLE, 7758 },
	{ 0xc5f9, 0xc5fe, PDF_CMAP_RANGE, 3187 },
	{ 0xc6a1, 0xc6bd, PDF_CMAP_RANGE, 3193 },
	{ 0xc6be, 0xc6be, PDF_CMAP_SINGLE, 7759 },
	{ 0xc6bf, 0xc6c1, PDF_CMAP_RANGE, 3223 },
	{ 0xc6c2, 0xc6c2, PDF_CMAP_SINGLE, 7760 },
	{ 0xc6c3, 0xc6d3, PDF_CMAP_RANGE, 3227 },
	{ 0xc6d4, 0xc6d5, PDF_CMAP_RANGE, 7761 },
	{ 0xc6d6, 0xc6da, PDF_CMAP_RANGE, 3246 },
	{ 0xc6db, 0xc6dc, PDF_CMAP_RANGE, 7763 },
	{ 0xc6dd, 0xc6e0, PDF_CMAP_RANGE, 3253 },
	{ 0xc6e1, 0xc6e1, PDF_CMAP_SINGLE, 7765 },
	{ 0xc6e2, 0xc6e5, PDF_CMAP_RANGE, 3258 },
	{ 0xc6e6, 0xc6e7, PDF_CMAP_RANGE, 7766 },
	{ 0xc6e8, 0xc6e9, PDF_CMAP_RANGE, 3264 },
	{ 0xc6ea, 0xc6ea, PDF_CMAP_SINGLE, 7768 },
	{ 0xc6eb, 0xc6f5, PDF_CMAP_RANGE, 3267 },
	{ 0xc6f6, 0xc6f6, PDF_CMAP_SINGLE, 6893 },
	{ 0xc6f7, 0xc6fe, PDF_CMAP_RANGE, 3279 },
	{ 0xc7a1, 0xc7a8, PDF_CMAP_RANGE, 3287 },
	{ 0xc7a9, 0xc7a9, PDF_CMAP_SINGLE, 7769 },
	{ 0xc7aa, 0xc7b8, PDF_CMAP_RANGE, 3296 },
	{ 0xc7b9, 0xc7b9, PDF_CMAP_SINGLE, 7770 },
	{ 0xc7ba, 0xc7d6, PDF_CMAP_RANGE, 3312 },
	{ 0xc7d7, 0xc7d7, PDF_CMAP_SINGLE, 7771 },
	{ 0xc7d8, 0xc7e6, PDF_CMAP_RANGE, 3342 },
	{ 0xc7e7, 0xc7e9, PDF_CMAP_TABLE, 40 },
	{ 0xc7ea, 0xc7ec, PDF_CMAP_RANGE, 3360 },
	{ 0xc7ed, 0xc7ed, PDF_CMAP_SINGLE, 7774 },
	{ 0xc7ee, 0xc7fe, PDF_CMAP_RANGE, 3364 },
	{ 0xc8a1, 0xc8a3, PDF_CMAP_RANGE, 3381 },
	{ 0xc8a4, 0xc8a4, PDF_CMAP_SINGLE, 7775 },
	{ 0xc8a5, 0xc8ad, PDF_CMAP_RANGE, 3385 },
	{ 0xc8ae, 0xc8b0, PDF_CMAP_TABLE, 43 },
	{ 0xc8b1, 0xc8d3, PDF_CMAP_RANGE, 3397 },
	{ 0xc8d4, 0xc8d4, PDF_CMAP_SINGLE, 7778 },
	{ 0xc8d5, 0xc8e1, PDF_CMAP_RANGE, 3433 },
	{ 0xc8e2, 0xc8e2, PDF_CMAP_SINGLE, 7779 },
	{ 0xc8e3, 0xc8f4, PDF_CMAP_RANGE, 3447 },
	{ 0xc8f5, 0xc8f5, PDF_CMAP_SINGLE, 7780 },
	{ 0xc8f6, 0xc8fe, PDF_CMAP_RANGE, 3466 },
	{ 0xc9a1, 0xc9a1, PDF_CMAP_SINGLE, 3475 },
	{ 0xc9a2, 0xc9a3, PDF_CMAP_RANGE, 7781 },
	{ 0xc9a4, 0xc9ae, PDF_CMAP_RANGE, 3478 },
	{ 0xc9af, 0xc9b2, PDF_CMAP_TABLE, 46 },
	{ 0xc9b3, 0xc9b4, PDF_CMAP_RANGE, 3493 },
	{ 0xc9b5, 0xc9b5, PDF_CMAP_SINGLE, 7785 },
	{ 0xc9b6, 0xc9bf, PDF_CMAP_RANGE, 3496 },
	{ 0xc9c0, 0xc9c0, PDF_CMAP_SINGLE, 7786 },
	{ 0xc9c1, 0xc9cd, PDF_CMAP_RANGE, 3507 },
	{ 0xc9ce, 0xc9ce, PDF_CMAP_SINGLE, 7787 },
	{ 0xc9cf, 0xc9d0, PDF_CMAP_RANGE, 3521 },
	{ 0xc9d1, 0xc9d1, PDF_CMAP_SINGLE, 7788 },
	{ 0xc9d2, 0xc9fe, PDF_CMAP_RANGE, 3524 },
	{ 0xcaa1, 0xcac2, PDF_CMAP_RANGE, 3569 },
	{ 0xcac3, 0xcac3, PDF_CMAP_SINGLE, 7789 },
	{ 0xcac4, 0xcacc, PDF_CMAP_RANGE, 3604 },
	{ 0xcacd, 0xcacd, PDF_CMAP_SINGLE, 7790 },
	{ 0xcace, 0xcad9, PDF_CMAP_RANGE, 3614 },
	{ 0xcada, 0xcada, PDF_CMAP_SINGLE, 7791 },
	{ 0xcadb, 0xcaf8, PDF_CMAP_RANGE, 3627 },
	{ 0xcaf9, 0xcaf9, PDF_CMAP_SINGLE, 7792 },
	{ 0xcafa, 0xcafe, PDF_CMAP_RANGE, 3658 },
	{ 0xcba1, 0xcba2, PDF_CMAP_TABLE, 50 },
	{ 0xcba3, 0xcba8, PDF_CMAP_RANGE, 3665 },
	{ 0xcba9, 0xcba9, PDF_CMAP_SINGLE, 7794 },
	{ 0xcbaa, 0xcbca, PDF_CMAP_RANGE, 3672 },
	{ 0xcbcb, 0xcbcb, PDF_CMAP_SINGLE, 7795 },
	{ 0xcbcc, 0xcbe9, PDF_CMAP_RANGE, 3706 },
	{ 0xcbea, 0xcbea, PDF_CMAP_SINGLE, 7475 },
	{ 0xcbeb, 0xcbef, PDF_CMAP_RANGE, 3737 },
	{ 0xcbf0, 0xcbf0, PDF_CMAP_SINGLE, 7796 },
	{ 0xcbf1, 0xcbf7, PDF_CMAP_RANGE, 3743 },
	{ 0xcbf8, 0xcbf9, PDF_CMAP_TABLE, 52 },
	{ 0xcbfa, 0xcbfe, PDF_CMAP_RANGE, 3752 },
	{ 0xcca1, 0xcccc, PDF_CMAP_RANGE, 3757 },
	{ 0xcccd, 0xcccd, PDF_CMAP_SINGLE, 7797 },
	{ 0xccce, 0xccd8, PDF_CMAP_RANGE, 3802 },
	{ 0xccd9, 0xccd9, PDF_CMAP_SINGLE, 7798 },
	{ 0xccda, 0xccde, PDF_CMAP_RANGE, 3814 },
	{ 0xccdf, 0xccdf, PDF_CMAP_SINGLE, 7799 },
	{ 0xcce0, 0xcce1, PDF_CMAP_RANGE, 3820 },
	{ 0xcce2, 0xcce2, PDF_CMAP_SINGLE, 7800 },
	{ 0xcce3, 0xccf8, PDF_CMAP_RANGE, 3823 },
	{ 0xccf9, 0xccfe, PDF_CMAP_TABLE, 54 },
	{ 0xcda1, 0xcdb1, PDF_CMAP_RANGE, 3851 },
	{ 0xcdb2, 0xcdb2, PDF_CMAP_SINGLE, 7804 },
	{ 0xcdb3, 0xcdcf, PDF_CMAP_RANGE, 3869 },
	{ 0xcdd0, 0xcdd0, PDF_CMAP_SINGLE, 7805 },
	{ 0xcdd1, 0xcdd3, PDF_CMAP_RANGE, 3899 },
	{ 0xcdd4, 0xcdd4, PDF_CMAP_SINGLE, 7806 },
	{ 0xcdd5, 0xcdd9, PDF_CMAP_RANGE, 3903 },
	{ 0xcdda, 0xcdda, PDF_CMAP_SINGLE, 7476 },
	{ 0xcddb, 0xcde8, PDF_CMAP_RANGE, 3909 },
	{ 0xcde9, 0xcde9, PDF_CMAP_SINGLE, 7807 },
	{ 0xcdea, 0xcdfe, PDF_CMAP_RANGE, 3924 },
	{ 0xcea1, 0xceca, PDF_CMAP_RANGE, 3945 },
	{ 0xcecb, 0xcecb, PDF_CMAP_SINGLE, 7808 },
	{ 0xcecc, 0xcef9, PDF_CMAP_RANGE, 3988 },
	{ 0xcefa, 0xcefb, PDF_CMAP_RANGE, 7809 },
	{ 0xcefc, 0xcefe, PDF_CMAP_RANGE, 4036 },
	{ 0xcfa1, 0xcfa1, PDF_CMAP_SINGLE, 7811 },
	{ 0xcfa2, 0xcfb0, PDF_CMAP_RANGE, 4040 },
	{ 0xcfb1, 0xcfb1, PDF_CMAP_SINGLE, 7812 },
	{ 0xcfb2, 0xcfb5, PDF_CMAP_RANGE, 4056 },
	{ 0xcfb6, 0xcfb6, PDF_CMAP_SINGLE, 6007 },
	{ 0xcfb7, 0xcfb8, PDF_CMAP_RANGE, 4061 },
	{ 0xcfb9, 0xcfb9, PDF_CMAP_SINGLE, 7813 },
	{ 0xcfba, 0xcfd3, PDF_CMAP_RANGE, 4064 },
	{ 0xd0a1, 0xd0d5, PDF_CMAP_RANGE, 4090 },
	{ 0xd0d6, 0xd0d6, PDF_CMAP_SINGLE, 3751 },
	{ 0xd0d7, 0xd0fe, PDF_CMAP_RANGE, 4144 },
	{ 0xd1a1, 0xd1bc, PDF_CMAP_RANGE, 4184 },
	{ 0xd1bd, 0xd1bd, PDF_CMAP_SINGLE, 7814 },
	{ 0xd1be, 0xd1c6, PDF_CMAP_RANGE, 4213 },
	{ 0xd1c7, 0xd1c7, PDF_CMAP_SINGLE, 7815 },
	{ 0xd1c8, 0xd1ca, PDF_CMAP_RANGE, 4223 },
	{ 0xd1cb, 0xd1cd, PDF_CMAP_TABLE, 60 },
	{ 0xd1ce, 0xd1fe, PDF_CMAP_RANGE, 4229 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4278 },
	{ 0xd3a1, 0xd3af, PDF_CMAP_RANGE, 4372 },
	{ 0xd3b0, 0xd3b0, PDF_CMAP_SINGLE, 7818 },
	{ 0xd3b1, 0xd3b9, PDF_CMAP_RANGE, 4388 },
	{ 0xd3ba, 0xd3ba, PDF_CMAP_SINGLE, 7819 },
	{ 0xd3bb, 0xd3dd, PDF_CMAP_RANGE, 4398 },
	{ 0xd3de, 0xd3de, PDF_CMAP_SINGLE, 7821 },
	{ 0xd3df, 0xd3ea, PDF_CMAP_RANGE, 4434 },
	{ 0xd3eb, 0xd3eb, PDF_CMAP_SINGLE, 7822 },
	{ 0xd3ec, 0xd3fe, PDF_CMAP_RANGE, 4447 },
	{ 0xd4a1, 0xd4c3, PDF_CMAP_RANGE, 4466 },
	{ 0xd4c4, 0xd4c4, PDF_CMAP_SINGLE, 7823 },
	{ 0xd4c5, 0xd4e3, PDF_CMAP_RANGE, 4502 },
	{ 0xd4e4, 0xd4e4, PDF_CMAP_SINGLE, 3063 },
	{ 0xd4e5, 0xd4fe, PDF_CMAP_RANGE, 4534 },
	{ 0xd5a1, 0xd5bc, PDF_CMAP_RANGE, 4560 },
	{ 0xd5bd, 0xd5bd, PDF_CMAP_SINGLE, 7824 },
	{ 0xd5be, 0xd5e2, PDF_CMAP_RANGE, 4589 },
	{ 0xd5e3, 0xd5e3, PDF_CMAP_SINGLE, 7825 },
	{ 0xd5e4, 0xd5fe, PDF_CMAP_RANGE, 4627 },
	{ 0xd6a1, 0xd6a2, PDF_CMAP_TABLE, 63 },
	{ 0xd6a3, 0xd6fe, PDF_CMAP_RANGE, 4656 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 4748 },
	{ 0xd8a1, 0xd8a3, PDF_CMAP_RANGE, 4842 },
	{ 0xd8a4, 0xd8a4, PDF_CMAP_SINGLE, 7828 },
	{ 0xd8a5, 0xd8fe, PDF_CMAP_RANGE, 4846 },
	{ 0xd9a1, 0xd9df, PDF_CMAP_RANGE, 4936 },
	{ 0xd9e0, 0xd9e0, PDF_CMAP_SINGLE, 7829 },
	{ 0xd9e1, 0xd9eb, PDF_CMAP_RANGE, 5000 },
	{ 0xd9ec, 0xd9ec, PDF_CMAP_SINGLE, 7830 },
	{ 0xd9ed, 0xd9f7, PDF_CMAP_RANGE, 5012 },
	{ 0xd9f8, 0xd9f8, PDF_CMAP_SINGLE, 1447 },
	{ 0xd9f9, 0xd9fe, PDF_CMAP_RANGE, 5024 },
	{ 0xdaa1, 0xdab8, PDF_CMAP_RANGE, 5030 },
	{ 0xdab9, 0xdab9, PDF_CMAP_SINGLE, 7831 },
	{ 0xdaba, 0xdacc, PDF_CMAP_RANGE, 5055 },
	{ 0xdacd, 0xdacd, PDF_CMAP_SINGLE, 7832 },
	{ 0xdace, 0xdafe, PDF_CMAP_RANGE, 5075 },
	{ 0xdba1, 0xdbc4, PDF_CMAP_RANGE, 5124 },
	{ 0xdbc5, 0xdbc5, PDF_CMAP_SINGLE, 7833 },
	{ 0xdbc6, 0xdbc9, PDF_CMAP_RANGE, 5161 },
	{ 0xdbca, 0xdbca, PDF_CMAP_SINGLE, 7834 },
	{ 0xdbcb, 0xdbd7, PDF_CMAP_RANGE, 5166 },
	{ 0xdbd8, 0xdbd8, PDF_CMAP_SINGLE, 3490 },
	{ 0xdbd9, 0xdbea, PDF_CMAP_RANGE, 5180 },
	{ 0xdbeb, 0xdbed, PDF_CMAP_TABLE, 65 },
	{ 0xdbee, 0xdbf3, PDF_CMAP_RANGE, 5201 },
	{ 0xdbf4, 0xdbf4, PDF_CMAP_SINGLE, 7836 },
	{ 0xdbf5, 0xdbfe, PDF_CMAP_RANGE, 5208 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5218 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5312 },
	{ 0xdea1, 0xdeb8, PDF_CMAP_RANGE, 5406 },
	{ 0xdeb9, 0xdeb9, PDF_CMAP_SINGLE, 3181 },
	{ 0xdeba, 0xdecf, PDF_CMAP_RANGE, 5431 },
	{ 0xded0, 0xded0, PDF_CMAP_SINGLE, 7837 },
	{ 0xded1, 0xdef4, PDF_CMAP_RANGE, 5454 },
	{ 0xdef5, 0xdef5, PDF_CMAP_SINGLE, 1535 },
	{ 0xdef6, 0xdefe, PDF_CMAP_RANGE, 5491 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5500 },
	{ 0xe0a1, 0xe0a5, PDF_CMAP_RANGE, 5594 },
	{ 0xe0a6, 0xe0a6, PDF_CMAP_SINGLE, 7839 },
	{ 0xe0a7, 0xe0de, PDF_CMAP_RANGE, 5600 },
	{ 0xe0df, 0xe0df, PDF_CMAP_SINGLE, 7840 },
	{ 0xe0e0, 0xe0f5, PDF_CMAP_RANGE, 5657 },
	{ 0xe0f6, 0xe0f6, PDF_CMAP_SINGLE, 7477 },
	{ 0xe0f7, 0xe0fe, PDF_CMAP_RANGE, 5680 },
	{ 0xe1a1, 0xe1aa, PDF_CMAP_RANGE, 5688 },
	{ 0xe1ab, 0xe1ab, PDF_CMAP_SINGLE, 7841 },
	{ 0xe1ac, 0xe1af, PDF_CMAP_RANGE, 5699 },
	{ 0xe1b0, 0xe1b1, PDF_CMAP_RANGE, 7842 },
	{ 0xe1b2, 0xe1fe, PDF_CMAP_RANGE, 5705 },
	{ 0xe2a1, 0xe2aa, PDF_CMAP_RANGE, 5782 },
	{ 0xe2ab, 0xe2ab, PDF_CMAP_SINGLE, 7844 },
	{ 0xe2ac, 0xe2e7, PDF_CMAP_RANGE, 5793 },
	{ 0xe2e8, 0xe2ea, PDF_CMAP_TABLE, 68 },
	{ 0xe2eb, 0xe2ee, PDF_CMAP_RANGE, 5856 },
	{ 0xe2ef, 0xe2ef, PDF_CMAP_SINGLE, 7845 },
	{ 0xe2f0, 0xe2fe, PDF_CMAP_RANGE, 5861 },
	{ 0xe3a1, 0xe3c9, PDF_CMAP_RANGE, 5876 },
	{ 0xe3ca, 0xe3ca, PDF_CMAP_SINGLE, 7846 },
	{ 0xe3cb, 0xe3d3, PDF_CMAP_RANGE, 5918 },
	{ 0xe3d4, 0xe3d4, PDF_CMAP_SINGLE, 7847 },
	{ 0xe3d5, 0xe3dd, PDF_CMAP_RANGE, 5928 },
	{ 0xe3de, 0xe3de, PDF_CMAP_SINGLE, 1492 },
	{ 0xe3df, 0xe3fe, PDF_CMAP_RANGE, 5938 },
	{ 0xe4a1, 0xe4b8, PDF_CMAP_RANGE, 5970 },
	{ 0xe4b9, 0xe4b9, PDF_CMAP_SINGLE, 7848 },
	{ 0xe4ba, 0xe4c5, PDF_CMAP_RANGE, 5995 },
	{ 0xe4c6, 0xe4c6, PDF_CMAP_SINGLE, 4060 },
	{ 0xe4c7, 0xe4e3, PDF_CMAP_RANGE, 6008 },
	{ 0xe4e4, 0xe4e4, PDF_CMAP_SINGLE, 7849 },
	{ 0xe4e5, 0xe4ed, PDF_CMAP_RANGE, 6038 },
	{ 0xe4ee, 0xe4ee, PDF_CMAP_SINGLE, 7850 },
	{ 0xe4ef, 0xe4fe, PDF_CMAP_RANGE, 6048 },
	{ 0xe5a1, 0xe5b8, PDF_CMAP_RANGE, 6064 },
	{ 0xe5b9, 0xe5bb, PDF_CMAP_TABLE, 71 },
	{ 0xe5bc, 0xe5c5, PDF_CMAP_RANGE, 6091 },
	{ 0xe5c6, 0xe5c6, PDF_CMAP_SINGLE, 7853 },
	{ 0xe5c7, 0xe5fe, PDF_CMAP_RANGE, 6102 },
	{ 0xe6a1, 0xe6c5, PDF_CMAP_RANGE, 6158 },
	{ 0xe6c6, 0xe6c6, PDF_CMAP_SINGLE, 7854 },
	{ 0xe6c7, 0xe6fe, PDF_CMAP_RANGE, 6196 },
	{ 0xe7a1, 0xe7e3, PDF_CMAP_RANGE, 6252 },
	{ 0xe7e4, 0xe7e4, PDF_CMAP_SINGLE, 7855 },
	{ 0xe7e5, 0xe7e8, PDF_CMAP_RANGE, 6320 },
	{ 0xe7e9, 0xe7e9, PDF_CMAP_SINGLE, 7856 },
	{ 0xe7ea, 0xe7f1, PDF_CMAP_RANGE, 6325 },
	{ 0xe7f2, 0xe7f2, PDF_CMAP_SINGLE, 7857 },
	{ 0xe7f3, 0xe7fe, PDF_CMAP_RANGE, 6334 },
	{ 0xe8a1, 0xe8b3, PDF_CMAP_RANGE, 6346 },
	{ 0xe8b4, 0xe8b4, PDF_CMAP_SINGLE, 7858 },
	{ 0xe8b5, 0xe8ba, PDF_CMAP_RANGE, 6366 },
	{ 0xe8bb, 0xe8bb, PDF_CMAP_SINGLE, 7859 },
	{ 0xe8bc, 0xe8f3, PDF_CMAP_RANGE, 6373 },
	{ 0xe8f4, 0xe8f4, PDF_CMAP_SINGLE, 7860 },
	{ 0xe8f5, 0xe8fe, PDF_CMAP_RANGE, 6430 },
	{ 0xe9a1, 0xe9a2, PDF_CMAP_TABLE, 74 },
	{ 0xe9a3, 0xe9ad, PDF_CMAP_RANGE, 6442 },
	{ 0xe9ae, 0xe9ae, PDF_CMAP_SINGLE, 3845 },
	{ 0xe9af, 0xe9d9, PDF_CMAP_RANGE, 6454 },
	{ 0xe9da, 0xe9da, PDF_CMAP_SINGLE, 1440 },
	{ 0xe9db, 0xe9fe, PDF_CMAP_RANGE, 6498 },
	{ 0xeaa1, 0xeaa3, PDF_CMAP_RANGE, 6534 },
	{ 0xeaa4, 0xeaa4, PDF_CMAP_SINGLE, 3358 },
	{ 0xeaa5, 0xeaa6, PDF_CMAP_RANGE, 6538 },
	{ 0xeaa7, 0xeaa7, PDF_CMAP_SINGLE, 7862 },
	{ 0xeaa8, 0xeabc, PDF_CMAP_RANGE, 6541 },
	{ 0xeabd, 0xeabd, PDF_CMAP_SINGLE, 7863 },
	{ 0xeabe, 0xeaee, PDF_CMAP_RANGE, 6563 },
	{ 0xeaef, 0xeaef, PDF_CMAP_SINGLE, 7864 },
	{ 0xeaf0, 0xeafe, PDF_CMAP_RANGE, 6613 },
	{ 0xeba1, 0xebb1, PDF_CMAP_RANGE, 6628 },
	{ 0xebb2, 0xebb2, PDF_CMAP_SINGLE, 7865 },
	{ 0xebb3, 0xebdc, PDF_CMAP_RANGE, 6646 },
	{ 0xebdd, 0xebdd, PDF_CMAP_SINGLE, 1550 },
	{ 0xebde, 0xebe5, PDF_CMAP_RANGE, 6689 },
	{ 0xebe6, 0xebe6, PDF_CMAP_SINGLE, 7866 },
	{ 0xebe7, 0xebf5, PDF_CMAP_RANGE, 6698 },
	{ 0xebf6, 0xebf6, PDF_CMAP_SINGLE, 7867 },
	{ 0xebf7, 0xebfe, PDF_CMAP_RANGE, 6714 },
	{ 0xeca1, 0xeccc, PDF_CMAP_RANGE, 6722 },
	{ 0xeccd, 0xeccd, PDF_CMAP_SINGLE, 2730 },
	{ 0xecce, 0xece8, PDF_CMAP_RANGE, 6767 },
	{ 0xece9, 0xece9, PDF_CMAP_SINGLE, 7868 },
	{ 0xecea, 0xecf3, PDF_CMAP_RANGE, 6795 },
	{ 0xecf4, 0xecf4, PDF_CMAP_SINGLE, 7869 },
	{ 0xecf5, 0xecfe, PDF_CMAP_RANGE, 6806 },
	{ 0xeda1, 0xedcd, PDF_CMAP_RANGE, 6816 },
	{ 0xedce, 0xedce, PDF_CMAP_SINGLE, 7870 },
	{ 0xedcf, 0xedeb, PDF_CMAP_RANGE, 6862 },
	{ 0xedec, 0xedee, PDF_CMAP_TABLE, 76 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6894 },
	{ 0xeea1, 0xeea8, PDF_CMAP_RANGE, 6910 },
	{ 0xeea9, 0xeea9, PDF_CMAP_SINGLE, 7873 },
	{ 0xeeaa, 0xeebc, PDF_CMAP_RANGE, 6919 },
	{ 0xeebd, 0xeebd, PDF_CMAP_SINGLE, 7874 },
	{ 0xeebe, 0xeed6, PDF_CMAP_RANGE, 6939 },
	{ 0xeed7, 0xeed7, PDF_CMAP_SINGLE, 7875 },
	{ 0xeed8, 0xeefe, PDF_CMAP_RANGE, 6965 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 7004 },
	{ 0xf0a1, 0xf0c4, PDF_CMAP_RANGE, 7098 },
	{ 0xf0c5, 0xf0c5, PDF_CMAP_SINGLE, 7878 },
	{ 0xf0c6, 0xf0d0, PDF_CMAP_RANGE, 7135 },
	{ 0xf0d1, 0xf0d1, PDF_CMAP_SINGLE, 7879 },
	{ 0xf0d2, 0xf0d6, PDF_CMAP_RANGE, 7147 },
	{ 0xf0d7, 0xf0d7, PDF_CMAP_SINGLE, 7880 },
	{ 0xf0d8, 0xf0f3, PDF_CMAP_RANGE, 7153 },
	{ 0xf0f4, 0xf0f5, PDF_CMAP_TABLE, 79 },
	{ 0xf0f6, 0xf0fe, PDF_CMAP_RANGE, 7183 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7192 },
	{ 0xf2a1, 0xf2ac, PDF_CMAP_RANGE, 7286 },
	{ 0xf2ad, 0xf2ad, PDF_CMAP_SINGLE, 7882 },
	{ 0xf2ae, 0xf2bb, PDF_CMAP_RANGE, 7299 },
	{ 0xf2bc, 0xf2bc, PDF_CMAP_SINGLE, 7883 },
	{ 0xf2bd, 0xf2cc, PDF_CMAP_RANGE, 7314 },
	{ 0xf2cd, 0xf2ce, PDF_CMAP_TABLE, 81 },
	{ 0xf2cf, 0xf2f3, PDF_CMAP_RANGE, 7332 },
	{ 0xf2f4, 0xf2f4, PDF_CMAP_SINGLE, 1321 },
	{ 0xf2f5, 0xf2fe, PDF_CMAP_RANGE, 7370 },
	{ 0xf3a1, 0xf3d0, PDF_CMAP_RANGE, 7380 },
	{ 0xf3d1, 0xf3d1, PDF_CMAP_SINGLE, 7885 },
	{ 0xf3d2, 0xf3fc, PDF_CMAP_RANGE, 7429 },
	{ 0xf3fd, 0xf3fe, PDF_CMAP_TABLE, 83 },
	{ 0xebe6, 0xebe6, PDF_CMAP_SINGLE, 7866 },
	{ 0xebe7, 0xebf5, PDF_CMAP_RANGE, 6698 },
	{ 0xebf6, 0xebf6, PDF_CMAP_SINGLE, 7867 },
	{ 0xebf7, 0xebfe, PDF_CMAP_RANGE, 6714 },
	{ 0xeca1, 0xeccc, PDF_CMAP_RANGE, 6722 },
	{ 0xeccd, 0xeccd, PDF_CMAP_SINGLE, 2730 },
	{ 0xecce, 0xece8, PDF_CMAP_RANGE, 6767 },
	{ 0xece9, 0xece9, PDF_CMAP_SINGLE, 7868 },
	{ 0xecea, 0xecf3, PDF_CMAP_RANGE, 6795 },
	{ 0xecf4, 0xecf4, PDF_CMAP_SINGLE, 7869 },
	{ 0xecf5, 0xecfe, PDF_CMAP_RANGE, 6806 },
	{ 0xeda1, 0xedcd, PDF_CMAP_RANGE, 6816 },
	{ 0xedce, 0xedce, PDF_CMAP_SINGLE, 7870 },
	{ 0xedcf, 0xedeb, PDF_CMAP_RANGE, 6862 },
	{ 0xedec, 0xedec, PDF_CMAP_SINGLE, 7871 },
	{ 0xeded, 0xeded, PDF_CMAP_SINGLE, 6892 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 7872 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6894 },
	{ 0xeea1, 0xeea8, PDF_CMAP_RANGE, 6910 },
	{ 0xeea9, 0xeea9, PDF_CMAP_SINGLE, 7873 },
	{ 0xeeaa, 0xeebc, PDF_CMAP_RANGE, 6919 },
	{ 0xeebd, 0xeebd, PDF_CMAP_SINGLE, 7874 },
	{ 0xeebe, 0xeed6, PDF_CMAP_RANGE, 6939 },
	{ 0xeed7, 0xeed7, PDF_CMAP_SINGLE, 7875 },
	{ 0xeed8, 0xeefe, PDF_CMAP_RANGE, 6965 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 7004 },
	{ 0xf0a1, 0xf0c4, PDF_CMAP_RANGE, 7098 },
	{ 0xf0c5, 0xf0c5, PDF_CMAP_SINGLE, 7878 },
	{ 0xf0c6, 0xf0d0, PDF_CMAP_RANGE, 7135 },
	{ 0xf0d1, 0xf0d1, PDF_CMAP_SINGLE, 7879 },
	{ 0xf0d2, 0xf0d6, PDF_CMAP_RANGE, 7147 },
	{ 0xf0d7, 0xf0d7, PDF_CMAP_SINGLE, 7880 },
	{ 0xf0d8, 0xf0f3, PDF_CMAP_RANGE, 7153 },
	{ 0xf0f4, 0xf0f4, PDF_CMAP_SINGLE, 1841 },
	{ 0xf0f5, 0xf0f5, PDF_CMAP_SINGLE, 7881 },
	{ 0xf0f6, 0xf0fe, PDF_CMAP_RANGE, 7183 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7192 },
	{ 0xf2a1, 0xf2ac, PDF_CMAP_RANGE, 7286 },
	{ 0xf2ad, 0xf2ad, PDF_CMAP_SINGLE, 7882 },
	{ 0xf2ae, 0xf2bb, PDF_CMAP_RANGE, 7299 },
	{ 0xf2bc, 0xf2bc, PDF_CMAP_SINGLE, 7883 },
	{ 0xf2bd, 0xf2cc, PDF_CMAP_RANGE, 7314 },
	{ 0xf2cd, 0xf2cd, PDF_CMAP_SINGLE, 1143 },
	{ 0xf2ce, 0xf2ce, PDF_CMAP_SINGLE, 7884 },
	{ 0xf2cf, 0xf2f3, PDF_CMAP_RANGE, 7332 },
	{ 0xf2f4, 0xf2f4, PDF_CMAP_SINGLE, 1321 },
	{ 0xf2f5, 0xf2fe, PDF_CMAP_RANGE, 7370 },
	{ 0xf3a1, 0xf3d0, PDF_CMAP_RANGE, 7380 },
	{ 0xf3d1, 0xf3d1, PDF_CMAP_SINGLE, 7885 },
	{ 0xf3d2, 0xf3fc, PDF_CMAP_RANGE, 7429 },
	{ 0xf3fd, 0xf3fd, PDF_CMAP_SINGLE, 7886 },
	{ 0xf3fe, 0xf3fe, PDF_CMAP_SINGLE, 7473 },
};

static const unsigned short pdf_cmap_78_EUC_H_table[85] =
{
	1125,7633,7961,7330,7645,7369,7646,7649,
	1406,1407,7650,7655,5490,1689,7660,7665,
	1780,7666,1782,7689,2169,7690,7693,6441,
	2629,7713,7720,6766,7733,2910,7734,2918,
	7744,3045,7745,7746,3050,7747,3099,7748,
	7772,6537,7773,7776,3395,7777,7783,5179,
	3491,7784,3663,7793,7980,4143,6453,7801,
	3847,7802,3849,7803,7816,4227,7817,4654,
	7826,7835,5199,3176,2030,5854,3153,7851,
	6089,7852,6440,2293,7871,6892,7872,1841,
	7881,1143,7884,7886,7473,
};

pdf_cmap pdf_cmap_78_EUC_H =
{
	-1, "78-EUC-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8ea0, 0x8edf },
		{ 2, 0xa1a1, 0xfefe },
	},
	625, 625, (pdf_range*) pdf_cmap_78_EUC_H_ranges,
	85, 85, (unsigned short*) pdf_cmap_78_EUC_H_table,
};

/* 78-EUC-V */

static const pdf_range pdf_cmap_78_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 7887 },
	{ 0xa1b1, 0xa1b2, PDF_CMAP_RANGE, 7889 },
	{ 0xa1bc, 0xa1be, PDF_CMAP_RANGE, 7891 },
	{ 0xa1c1, 0xa1c5, PDF_CMAP_RANGE, 7894 },
	{ 0xa1ca, 0xa1db, PDF_CMAP_RANGE, 7899 },
	{ 0xa1e1, 0xa1e1, PDF_CMAP_SINGLE, 7917 },
	{ 0xa4a1, 0xa4a1, PDF_CMAP_SINGLE, 7918 },
	{ 0xa4a3, 0xa4a3, PDF_CMAP_SINGLE, 7919 },
	{ 0xa4a5, 0xa4a5, PDF_CMAP_SINGLE, 7920 },
	{ 0xa4a7, 0xa4a7, PDF_CMAP_SINGLE, 7921 },
	{ 0xa4a9, 0xa4a9, PDF_CMAP_SINGLE, 7922 },
	{ 0xa4c3, 0xa4c3, PDF_CMAP_SINGLE, 7923 },
	{ 0xa4e3, 0xa4e3, PDF_CMAP_SINGLE, 7924 },
	{ 0xa4e5, 0xa4e5, PDF_CMAP_SINGLE, 7925 },
	{ 0xa4e7, 0xa4e7, PDF_CMAP_SINGLE, 7926 },
	{ 0xa4ee, 0xa4ee, PDF_CMAP_SINGLE, 7927 },
	{ 0xa5a1, 0xa5a1, PDF_CMAP_SINGLE, 7928 },
	{ 0xa5a3, 0xa5a3, PDF_CMAP_SINGLE, 7929 },
	{ 0xa5a5, 0xa5a5, PDF_CMAP_SINGLE, 7930 },
	{ 0xa5a7, 0xa5a7, PDF_CMAP_SINGLE, 7931 },
	{ 0xa5a9, 0xa5a9, PDF_CMAP_SINGLE, 7932 },
	{ 0xa5c3, 0xa5c3, PDF_CMAP_SINGLE, 7933 },
	{ 0xa5e3, 0xa5e3, PDF_CMAP_SINGLE, 7934 },
	{ 0xa5e5, 0xa5e5, PDF_CMAP_SINGLE, 7935 },
	{ 0xa5e7, 0xa5e7, PDF_CMAP_SINGLE, 7936 },
	{ 0xa5ee, 0xa5ee, PDF_CMAP_SINGLE, 7937 },
	{ 0xa5f5, 0xa5f6, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_78_EUC_V_table[1] = { 0 };

pdf_cmap pdf_cmap_78_EUC_V =
{
	-1, "78-EUC-V", "78-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	27, 27, (pdf_range*) pdf_cmap_78_EUC_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_78_EUC_V_table,
};

/* 78-H */

static const pdf_range pdf_cmap_78_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 633 },
	{ 0x2221, 0x222e, PDF_CMAP_RANGE, 727 },
	{ 0x2330, 0x2339, PDF_CMAP_RANGE, 780 },
	{ 0x2341, 0x235a, PDF_CMAP_RANGE, 790 },
	{ 0x2361, 0x237a, PDF_CMAP_RANGE, 816 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 842 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 925 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 1011 },
	{ 0x2641, 0x2658, PDF_CMAP_RANGE, 1035 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 1059 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 1092 },
	{ 0x3021, 0x3022, PDF_CMAP_TABLE, 0 },
	{ 0x3023, 0x3028, PDF_CMAP_RANGE, 1127 },
	{ 0x3029, 0x3029, PDF_CMAP_SINGLE, 8266 },
	{ 0x302a, 0x3031, PDF_CMAP_RANGE, 1134 },
	{ 0x3032, 0x3033, PDF_CMAP_TABLE, 2 },
	{ 0x3034, 0x303a, PDF_CMAP_RANGE, 1144 },
	{ 0x303b, 0x303b, PDF_CMAP_SINGLE, 7634 },
	{ 0x303c, 0x306d, PDF_CMAP_RANGE, 1152 },
	{ 0x306e, 0x306e, PDF_CMAP_SINGLE, 7635 },
	{ 0x306f, 0x3072, PDF_CMAP_RANGE, 1203 },
	{ 0x3073, 0x3073, PDF_CMAP_SINGLE, 7636 },
	{ 0x3074, 0x307b, PDF_CMAP_RANGE, 1208 },
	{ 0x307c, 0x307c, PDF_CMAP_SINGLE, 7637 },
	{ 0x307d, 0x307e, PDF_CMAP_RANGE, 1217 },
	{ 0x3121, 0x3129, PDF_CMAP_RANGE, 1219 },
	{ 0x312a, 0x312a, PDF_CMAP_SINGLE, 7638 },
	{ 0x312b, 0x3134, PDF_CMAP_RANGE, 1229 },
	{ 0x3135, 0x3135, PDF_CMAP_SINGLE, 7639 },
	{ 0x3136, 0x3138, PDF_CMAP_RANGE, 1240 },
	{ 0x3139, 0x3139, PDF_CMAP_SINGLE, 7640 },
	{ 0x313a, 0x313c, PDF_CMAP_RANGE, 1244 },
	{ 0x313d, 0x313d, PDF_CMAP_SINGLE, 7642 },
	{ 0x313e, 0x3141, PDF_CMAP_RANGE, 1248 },
	{ 0x3142, 0x3142, PDF_CMAP_SINGLE, 7643 },
	{ 0x3143, 0x316a, PDF_CMAP_RANGE, 1253 },
	{ 0x316b, 0x316b, PDF_CMAP_SINGLE, 7644 },
	{ 0x316c, 0x317e, PDF_CMAP_RANGE, 1294 },
	{ 0x3221, 0x3227, PDF_CMAP_RANGE, 1313 },
	{ 0x3228, 0x322a, PDF_CMAP_TABLE, 4 },
	{ 0x322b, 0x325f, PDF_CMAP_RANGE, 1323 },
	{ 0x3260, 0x3260, PDF_CMAP_SINGLE, 7647 },
	{ 0x3261, 0x3279, PDF_CMAP_RANGE, 1377 },
	{ 0x327a, 0x327a, PDF_CMAP_SINGLE, 7648 },
	{ 0x327b, 0x327c, PDF_CMAP_RANGE, 1403 },
	{ 0x327d, 0x327e, PDF_CMAP_TABLE, 7 },
	{ 0x3321, 0x3322, PDF_CMAP_TABLE, 9 },
	{ 0x3323, 0x3341, PDF_CMAP_RANGE, 1409 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 6497 },
	{ 0x3343, 0x3348, PDF_CMAP_RANGE, 1441 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 5023 },
	{ 0x334a, 0x3364, PDF_CMAP_RANGE, 1448 },
	{ 0x3365, 0x3365, PDF_CMAP_SINGLE, 7651 },
	{ 0x3366, 0x336a, PDF_CMAP_RANGE, 1476 },
	{ 0x336b, 0x336b, PDF_CMAP_SINGLE, 7652 },
	{ 0x336c, 0x3372, PDF_CMAP_RANGE, 1482 },
	{ 0x3373, 0x3373, PDF_CMAP_SINGLE, 7653 },
	{ 0x3374, 0x3375, PDF_CMAP_RANGE, 1490 },
	{ 0x3376, 0x3376, PDF_CMAP_SINGLE, 5937 },
	{ 0x3377, 0x3379, PDF_CMAP_RANGE, 1493 },
	{ 0x337a, 0x337a, PDF_CMAP_SINGLE, 7654 },
	{ 0x337b, 0x337e, PDF_CMAP_RANGE, 1497 },
	{ 0x3421, 0x3441, PDF_CMAP_RANGE, 1501 },
	{ 0x3442, 0x3443, PDF_CMAP_TABLE, 11 },
	{ 0x3444, 0x344c, PDF_CMAP_RANGE, 1536 },
	{ 0x344d, 0x344d, PDF_CMAP_SINGLE, 7656 },
	{ 0x344e, 0x3451, PDF_CMAP_RANGE, 1546 },
	{ 0x3452, 0x3452, PDF_CMAP_SINGLE, 6688 },
	{ 0x3453, 0x3464, PDF_CMAP_RANGE, 1551 },
	{ 0x3465, 0x3465, PDF_CMAP_SINGLE, 7657 },
	{ 0x3466, 0x347e, PDF_CMAP_RANGE, 1570 },
	{ 0x3521, 0x352a, PDF_CMAP_RANGE, 1595 },
	{ 0x352b, 0x352b, PDF_CMAP_SINGLE, 7658 },
	{ 0x352c, 0x353f, PDF_CMAP_RANGE, 1606 },
	{ 0x3540, 0x3540, PDF_CMAP_SINGLE, 7659 },
	{ 0x3541, 0x357e, PDF_CMAP_RANGE, 1627 },
	{ 0x3621, 0x3622, PDF_CMAP_TABLE, 13 },
	{ 0x3623, 0x3629, PDF_CMAP_RANGE, 1691 },
	{ 0x362a, 0x362a, PDF_CMAP_SINGLE, 7661 },
	{ 0x362b, 0x3645, PDF_CMAP_RANGE, 1699 },
	{ 0x3646, 0x3646, PDF_CMAP_SINGLE, 7474 },
	{ 0x3647, 0x364e, PDF_CMAP_RANGE, 1727 },
	{ 0x364f, 0x364f, PDF_CMAP_SINGLE, 7662 },
	{ 0x3650, 0x366c, PDF_CMAP_RANGE, 1736 },
	{ 0x366d, 0x366d, PDF_CMAP_SINGLE, 7663 },
	{ 0x366e, 0x3673, PDF_CMAP_RANGE, 1766 },
	{ 0x3674, 0x3674, PDF_CMAP_SINGLE, 7664 },
	{ 0x3675, 0x367a, PDF_CMAP_RANGE, 1773 },
	{ 0x367b, 0x367e, PDF_CMAP_TABLE, 15 },
	{ 0x3721, 0x3723, PDF_CMAP_RANGE, 1783 },
	{ 0x3724, 0x3724, PDF_CMAP_SINGLE, 7667 },
	{ 0x3725, 0x3736, PDF_CMAP_RANGE, 1787 },
	{ 0x3737, 0x3737, PDF_CMAP_SINGLE, 7668 },
	{ 0x3738, 0x3744, PDF_CMAP_RANGE, 1806 },
	{ 0x3745, 0x3745, PDF_CMAP_SINGLE, 7669 },
	{ 0x3746, 0x374d, PDF_CMAP_RANGE, 1820 },
	{ 0x374e, 0x374e, PDF_CMAP_SINGLE, 7670 },
	{ 0x374f, 0x3751, PDF_CMAP_RANGE, 1829 },
	{ 0x3752, 0x3752, PDF_CMAP_SINGLE, 7671 },
	{ 0x3753, 0x3754, PDF_CMAP_RANGE, 1833 },
	{ 0x3755, 0x3755, PDF_CMAP_SINGLE, 7672 },
	{ 0x3756, 0x375a, PDF_CMAP_RANGE, 1836 },
	{ 0x375b, 0x375b, PDF_CMAP_SINGLE, 7181 },
	{ 0x375c, 0x3763, PDF_CMAP_RANGE, 1842 },
	{ 0x3764, 0x3764, PDF_CMAP_SINGLE, 7673 },
	{ 0x3765, 0x3770, PDF_CMAP_RANGE, 1851 },
	{ 0x3771, 0x3771, PDF_CMAP_SINGLE, 7674 },
	{ 0x3772, 0x3778, PDF_CMAP_RANGE, 1864 },
	{ 0x3779, 0x3779, PDF_CMAP_SINGLE, 7675 },
	{ 0x377a, 0x377d, PDF_CMAP_RANGE, 1872 },
	{ 0x377e, 0x377e, PDF_CMAP_SINGLE, 7676 },
	{ 0x3821, 0x3833, PDF_CMAP_RANGE, 1877 },
	{ 0x3834, 0x3834, PDF_CMAP_SINGLE, 7677 },
	{ 0x3835, 0x3840, PDF_CMAP_RANGE, 1897 },
	{ 0x3841, 0x3841, PDF_CMAP_SINGLE, 7678 },
	{ 0x3842, 0x387e, PDF_CMAP_RANGE, 1910 },
	{ 0x3921, 0x392a, PDF_CMAP_RANGE, 1971 },
	{ 0x392b, 0x392b, PDF_CMAP_SINGLE, 7679 },
	{ 0x392c, 0x3936, PDF_CMAP_RANGE, 1982 },
	{ 0x3937, 0x3937, PDF_CMAP_SINGLE, 7680 },
	{ 0x3938, 0x3941, PDF_CMAP_RANGE, 1994 },
	{ 0x3942, 0x3942, PDF_CMAP_SINGLE, 7681 },
	{ 0x3943, 0x395b, PDF_CMAP_RANGE, 2005 },
	{ 0x395c, 0x395c, PDF_CMAP_SINGLE, 5853 },
	{ 0x395d, 0x396c, PDF_CMAP_RANGE, 2031 },
	{ 0x396d, 0x396d, PDF_CMAP_SINGLE, 7682 },
	{ 0x396e, 0x3973, PDF_CMAP_RANGE, 2048 },
	{ 0x3974, 0x3974, PDF_CMAP_SINGLE, 7683 },
	{ 0x3975, 0x3978, PDF_CMAP_RANGE, 2055 },
	{ 0x3979, 0x3979, PDF_CMAP_SINGLE, 7684 },
	{ 0x397a, 0x397e, PDF_CMAP_RANGE, 2060 },
	{ 0x3a21, 0x3a52, PDF_CMAP_RANGE, 2065 },
	{ 0x3a53, 0x3a53, PDF_CMAP_SINGLE, 7685 },
	{ 0x3a54, 0x3a66, PDF_CMAP_RANGE, 2116 },
	{ 0x3a67, 0x3a67, PDF_CMAP_SINGLE, 7686 },
	{ 0x3a68, 0x3a73, PDF_CMAP_RANGE, 2136 },
	{ 0x3a74, 0x3a74, PDF_CMAP_SINGLE, 7687 },
	{ 0x3a75, 0x3a7e, PDF_CMAP_RANGE, 2149 },
	{ 0x3b21, 0x3b26, PDF_CMAP_RANGE, 2159 },
	{ 0x3b27, 0x3b27, PDF_CMAP_SINGLE, 7688 },
	{ 0x3b28, 0x3b29, PDF_CMAP_RANGE, 2166 },
	{ 0x3b2a, 0x3b2c, PDF_CMAP_TABLE, 19 },
	{ 0x3b2d, 0x3b38, PDF_CMAP_RANGE, 2171 },
	{ 0x3b39, 0x3b39, PDF_CMAP_SINGLE, 7691 },
	{ 0x3b3a, 0x3b7e, PDF_CMAP_RANGE, 2184 },
	{ 0x3c21, 0x3c47, PDF_CMAP_RANGE, 2253 },
	{ 0x3c48, 0x3c49, PDF_CMAP_TABLE, 22 },
	{ 0x3c4a, 0x3c56, PDF_CMAP_RANGE, 2294 },
	{ 0x3c57, 0x3c57, PDF_CMAP_SINGLE, 7694 },
	{ 0x3c58, 0x3c5c, PDF_CMAP_RANGE, 2308 },
	{ 0x3c5d, 0x3c5e, PDF_CMAP_RANGE, 7695 },
	{ 0x3c5f, 0x3c7e, PDF_CMAP_RANGE, 2315 },
	{ 0x3d21, 0x3d2a, PDF_CMAP_RANGE, 2347 },
	{ 0x3d2b, 0x3d2b, PDF_CMAP_SINGLE, 7697 },
	{ 0x3d2c, 0x3d35, PDF_CMAP_RANGE, 2358 },
	{ 0x3d36, 0x3d36, PDF_CMAP_SINGLE, 7698 },
	{ 0x3d37, 0x3d6b, PDF_CMAP_RANGE, 2369 },
	{ 0x3d6c, 0x3d6d, PDF_CMAP_RANGE, 7699 },
	{ 0x3d6e, 0x3d71, PDF_CMAP_RANGE, 2424 },
	{ 0x3d72, 0x3d73, PDF_CMAP_RANGE, 7701 },
	{ 0x3d74, 0x3d7e, PDF_CMAP_RANGE, 2430 },
	{ 0x3e21, 0x3e24, PDF_CMAP_RANGE, 2441 },
	{ 0x3e25, 0x3e25, PDF_CMAP_SINGLE, 7703 },
	{ 0x3e26, 0x3e32, PDF_CMAP_RANGE, 2446 },
	{ 0x3e33, 0x3e33, PDF_CMAP_SINGLE, 7704 },
	{ 0x3e34, 0x3e3e, PDF_CMAP_RANGE, 2460 },
	{ 0x3e3f, 0x3e3f, PDF_CMAP_SINGLE, 7705 },
	{ 0x3e40, 0x3e54, PDF_CMAP_RANGE, 2472 },
	{ 0x3e55, 0x3e55, PDF_CMAP_SINGLE, 7706 },
	{ 0x3e56, 0x3e5e, PDF_CMAP_RANGE, 2494 },
	{ 0x3e5f, 0x3e5f, PDF_CMAP_SINGLE, 7707 },
	{ 0x3e60, 0x3e63, PDF_CMAP_RANGE, 2504 },
	{ 0x3e64, 0x3e64, PDF_CMAP_SINGLE, 7708 },
	{ 0x3e65, 0x3e7e, PDF_CMAP_RANGE, 2509 },
	{ 0x3f21, 0x3f29, PDF_CMAP_RANGE, 2535 },
	{ 0x3f2a, 0x3f2a, PDF_CMAP_SINGLE, 7709 },
	{ 0x3f2b, 0x3f58, PDF_CMAP_RANGE, 2545 },
	{ 0x3f59, 0x3f59, PDF_CMAP_SINGLE, 7710 },
	{ 0x3f5a, 0x3f5f, PDF_CMAP_RANGE, 2592 },
	{ 0x3f60, 0x3f60, PDF_CMAP_SINGLE, 7711 },
	{ 0x3f61, 0x3f68, PDF_CMAP_RANGE, 2599 },
	{ 0x3f69, 0x3f69, PDF_CMAP_SINGLE, 7712 },
	{ 0x3f6a, 0x3f7e, PDF_CMAP_RANGE, 2608 },
	{ 0x4021, 0x4022, PDF_CMAP_TABLE, 24 },
	{ 0x4023, 0x4041, PDF_CMAP_RANGE, 2631 },
	{ 0x4042, 0x4042, PDF_CMAP_SINGLE, 7714 },
	{ 0x4043, 0x4065, PDF_CMAP_RANGE, 2663 },
	{ 0x4066, 0x4066, PDF_CMAP_SINGLE, 7715 },
	{ 0x4067, 0x4070, PDF_CMAP_RANGE, 2699 },
	{ 0x4071, 0x4072, PDF_CMAP_RANGE, 7716 },
	{ 0x4073, 0x4078, PDF_CMAP_RANGE, 2711 },
	{ 0x4079, 0x407a, PDF_CMAP_RANGE, 7718 },
	{ 0x407b, 0x407e, PDF_CMAP_RANGE, 2719 },
	{ 0x4121, 0x4126, PDF_CMAP_RANGE, 2723 },
	{ 0x4127, 0x4128, PDF_CMAP_TABLE, 26 },
	{ 0x4129, 0x4138, PDF_CMAP_RANGE, 2731 },
	{ 0x4139, 0x4139, PDF_CMAP_SINGLE, 7721 },
	{ 0x413a, 0x414b, PDF_CMAP_RANGE, 2748 },
	{ 0x414c, 0x414c, PDF_CMAP_SINGLE, 7722 },
	{ 0x414d, 0x414e, PDF_CMAP_RANGE, 2767 },
	{ 0x414f, 0x414f, PDF_CMAP_SINGLE, 7723 },
	{ 0x4150, 0x415e, PDF_CMAP_RANGE, 2770 },
	{ 0x415f, 0x415f, PDF_CMAP_SINGLE, 7724 },
	{ 0x4160, 0x4168, PDF_CMAP_RANGE, 2786 },
	{ 0x4169, 0x4169, PDF_CMAP_SINGLE, 7725 },
	{ 0x416a, 0x417e, PDF_CMAP_RANGE, 2796 },
	{ 0x4221, 0x423c, PDF_CMAP_RANGE, 2817 },
	{ 0x423d, 0x423d, PDF_CMAP_SINGLE, 7726 },
	{ 0x423e, 0x424c, PDF_CMAP_RANGE, 2846 },
	{ 0x424d, 0x424d, PDF_CMAP_SINGLE, 7727 },
	{ 0x424e, 0x425b, PDF_CMAP_RANGE, 2862 },
	{ 0x425c, 0x425c, PDF_CMAP_SINGLE, 7728 },
	{ 0x425d, 0x4262, PDF_CMAP_RANGE, 2877 },
	{ 0x4263, 0x4263, PDF_CMAP_SINGLE, 7729 },
	{ 0x4264, 0x426e, PDF_CMAP_RANGE, 2884 },
	{ 0x426f, 0x426f, PDF_CMAP_SINGLE, 7730 },
	{ 0x4270, 0x4274, PDF_CMAP_RANGE, 2896 },
	{ 0x4275, 0x4276, PDF_CMAP_RANGE, 7731 },
	{ 0x4277, 0x427c, PDF_CMAP_RANGE, 2903 },
	{ 0x427d, 0x427e, PDF_CMAP_TABLE, 28 },
	{ 0x4321, 0x4326, PDF_CMAP_RANGE, 2911 },
	{ 0x4327, 0x4328, PDF_CMAP_TABLE, 30 },
	{ 0x4329, 0x432a, PDF_CMAP_RANGE, 7735 },
	{ 0x432b, 0x432c, PDF_CMAP_RANGE, 2921 },
	{ 0x432d, 0x432e, PDF_CMAP_RANGE, 7737 },
	{ 0x432f, 0x433c, PDF_CMAP_RANGE, 2925 },
	{ 0x433d, 0x433d, PDF_CMAP_SINGLE, 7739 },
	{ 0x433e, 0x436f, PDF_CMAP_RANGE, 2940 },
	{ 0x4370, 0x4370, PDF_CMAP_SINGLE, 7740 },
	{ 0x4371, 0x4374, PDF_CMAP_RANGE, 2991 },
	{ 0x4375, 0x4375, PDF_CMAP_SINGLE, 7741 },
	{ 0x4376, 0x437b, PDF_CMAP_RANGE, 2996 },
	{ 0x437c, 0x437c, PDF_CMAP_SINGLE, 7742 },
	{ 0x437d, 0x437e, PDF_CMAP_RANGE, 3003 },
	{ 0x4421, 0x443c, PDF_CMAP_RANGE, 3005 },
	{ 0x443d, 0x443d, PDF_CMAP_SINGLE, 7743 },
	{ 0x443e, 0x4447, PDF_CMAP_RANGE, 3034 },
	{ 0x4448, 0x444a, PDF_CMAP_TABLE, 32 },
	{ 0x444b, 0x444c, PDF_CMAP_RANGE, 3047 },
	{ 0x444d, 0x444f, PDF_CMAP_TABLE, 35 },
	{ 0x4450, 0x4453, PDF_CMAP_RANGE, 3052 },
	{ 0x4454, 0x4454, PDF_CMAP_SINGLE, 8267 },
	{ 0x4455, 0x445a, PDF_CMAP_RANGE, 3057 },
	{ 0x445b, 0x445b, PDF_CMAP_SINGLE, 4533 },
	{ 0x445c, 0x447e, PDF_CMAP_RANGE, 3064 },
	{ 0x4521, 0x4522, PDF_CMAP_TABLE, 38 },
	{ 0x4523, 0x4526, PDF_CMAP_RANGE, 3101 },
	{ 0x4527, 0x4527, PDF_CMAP_SINGLE, 7749 },
	{ 0x4528, 0x452d, PDF_CMAP_RANGE, 3106 },
	{ 0x452e, 0x452e, PDF_CMAP_SINGLE, 7750 },
	{ 0x452f, 0x4535, PDF_CMAP_RANGE, 3113 },
	{ 0x4536, 0x4536, PDF_CMAP_SINGLE, 7751 },
	{ 0x4537, 0x453e, PDF_CMAP_RANGE, 3121 },
	{ 0x453f, 0x453f, PDF_CMAP_SINGLE, 7752 },
	{ 0x4540, 0x4547, PDF_CMAP_RANGE, 3130 },
	{ 0x4548, 0x4548, PDF_CMAP_SINGLE, 7753 },
	{ 0x4549, 0x454a, PDF_CMAP_RANGE, 3139 },
	{ 0x454b, 0x454b, PDF_CMAP_SINGLE, 7754 },
	{ 0x454c, 0x4550, PDF_CMAP_RANGE, 3142 },
	{ 0x4551, 0x4552, PDF_CMAP_RANGE, 7755 },
	{ 0x4553, 0x4556, PDF_CMAP_RANGE, 3149 },
	{ 0x4557, 0x4557, PDF_CMAP_SINGLE, 5855 },
	{ 0x4558, 0x4563, PDF_CMAP_RANGE, 3154 },
	{ 0x4564, 0x4564, PDF_CMAP_SINGLE, 7757 },
	{ 0x4565, 0x456d, PDF_CMAP_RANGE, 3167 },
	{ 0x456e, 0x456e, PDF_CMAP_SINGLE, 5200 },
	{ 0x456f, 0x4572, PDF_CMAP_RANGE, 3177 },
	{ 0x4573, 0x4573, PDF_CMAP_SINGLE, 5430 },
	{ 0x4574, 0x4577, PDF_CMAP_RANGE, 3182 },
	{ 0x4578, 0x4578, PDF_CMAP_SINGLE, 7758 },
	{ 0x4579, 0x457e, PDF_CMAP_RANGE, 3187 },
	{ 0x4621, 0x463d, PDF_CMAP_RANGE, 3193 },
	{ 0x463e, 0x463e, PDF_CMAP_SINGLE, 7759 },
	{ 0x463f, 0x4641, PDF_CMAP_RANGE, 3223 },
	{ 0x4642, 0x4642, PDF_CMAP_SINGLE, 7760 },
	{ 0x4643, 0x4653, PDF_CMAP_RANGE, 3227 },
	{ 0x4654, 0x4655, PDF_CMAP_RANGE, 7761 },
	{ 0x4656, 0x465a, PDF_CMAP_RANGE, 3246 },
	{ 0x465b, 0x465c, PDF_CMAP_RANGE, 7763 },
	{ 0x465d, 0x4660, PDF_CMAP_RANGE, 3253 },
	{ 0x4661, 0x4661, PDF_CMAP_SINGLE, 7765 },
	{ 0x4662, 0x4665, PDF_CMAP_RANGE, 3258 },
	{ 0x4666, 0x4667, PDF_CMAP_RANGE, 7766 },
	{ 0x4668, 0x4669, PDF_CMAP_RANGE, 3264 },
	{ 0x466a, 0x466a, PDF_CMAP_SINGLE, 7768 },
	{ 0x466b, 0x4675, PDF_CMAP_RANGE, 3267 },
	{ 0x4676, 0x4676, PDF_CMAP_SINGLE, 6893 },
	{ 0x4677, 0x467e, PDF_CMAP_RANGE, 3279 },
	{ 0x4721, 0x4728, PDF_CMAP_RANGE, 3287 },
	{ 0x4729, 0x4729, PDF_CMAP_SINGLE, 7769 },
	{ 0x472a, 0x4738, PDF_CMAP_RANGE, 3296 },
	{ 0x4739, 0x4739, PDF_CMAP_SINGLE, 7770 },
	{ 0x473a, 0x4756, PDF_CMAP_RANGE, 3312 },
	{ 0x4757, 0x4757, PDF_CMAP_SINGLE, 7771 },
	{ 0x4758, 0x4766, PDF_CMAP_RANGE, 3342 },
	{ 0x4767, 0x4769, PDF_CMAP_TABLE, 40 },
	{ 0x476a, 0x476c, PDF_CMAP_RANGE, 3360 },
	{ 0x476d, 0x476d, PDF_CMAP_SINGLE, 7774 },
	{ 0x476e, 0x477e, PDF_CMAP_RANGE, 3364 },
	{ 0x4821, 0x4823, PDF_CMAP_RANGE, 3381 },
	{ 0x4824, 0x4824, PDF_CMAP_SINGLE, 7775 },
	{ 0x4825, 0x482d, PDF_CMAP_RANGE, 3385 },
	{ 0x482e, 0x4830, PDF_CMAP_TABLE, 43 },
	{ 0x4831, 0x4853, PDF_CMAP_RANGE, 3397 },
	{ 0x4854, 0x4854, PDF_CMAP_SINGLE, 7778 },
	{ 0x4855, 0x4861, PDF_CMAP_RANGE, 3433 },
	{ 0x4862, 0x4862, PDF_CMAP_SINGLE, 7779 },
	{ 0x4863, 0x4874, PDF_CMAP_RANGE, 3447 },
	{ 0x4875, 0x4875, PDF_CMAP_SINGLE, 7780 },
	{ 0x4876, 0x487e, PDF_CMAP_RANGE, 3466 },
	{ 0x4921, 0x4921, PDF_CMAP_SINGLE, 3475 },
	{ 0x4922, 0x4923, PDF_CMAP_RANGE, 7781 },
	{ 0x4924, 0x492e, PDF_CMAP_RANGE, 3478 },
	{ 0x492f, 0x4932, PDF_CMAP_TABLE, 46 },
	{ 0x4933, 0x4934, PDF_CMAP_RANGE, 3493 },
	{ 0x4935, 0x4935, PDF_CMAP_SINGLE, 7785 },
	{ 0x4936, 0x493f, PDF_CMAP_RANGE, 3496 },
	{ 0x4940, 0x4940, PDF_CMAP_SINGLE, 7786 },
	{ 0x4941, 0x494d, PDF_CMAP_RANGE, 3507 },
	{ 0x494e, 0x494e, PDF_CMAP_SINGLE, 7787 },
	{ 0x494f, 0x4950, PDF_CMAP_RANGE, 3521 },
	{ 0x4951, 0x4951, PDF_CMAP_SINGLE, 7788 },
	{ 0x4952, 0x497e, PDF_CMAP_RANGE, 3524 },
	{ 0x4a21, 0x4a42, PDF_CMAP_RANGE, 3569 },
	{ 0x4a43, 0x4a43, PDF_CMAP_SINGLE, 7789 },
	{ 0x4a44, 0x4a4c, PDF_CMAP_RANGE, 3604 },
	{ 0x4a4d, 0x4a4d, PDF_CMAP_SINGLE, 7790 },
	{ 0x4a4e, 0x4a59, PDF_CMAP_RANGE, 3614 },
	{ 0x4a5a, 0x4a5a, PDF_CMAP_SINGLE, 7791 },
	{ 0x4a5b, 0x4a78, PDF_CMAP_RANGE, 3627 },
	{ 0x4a79, 0x4a79, PDF_CMAP_SINGLE, 7792 },
	{ 0x4a7a, 0x4a7e, PDF_CMAP_RANGE, 3658 },
	{ 0x4b21, 0x4b22, PDF_CMAP_TABLE, 50 },
	{ 0x4b23, 0x4b28, PDF_CMAP_RANGE, 3665 },
	{ 0x4b29, 0x4b29, PDF_CMAP_SINGLE, 7794 },
	{ 0x4b2a, 0x4b4a, PDF_CMAP_RANGE, 3672 },
	{ 0x4b4b, 0x4b4b, PDF_CMAP_SINGLE, 7795 },
	{ 0x4b4c, 0x4b69, PDF_CMAP_RANGE, 3706 },
	{ 0x4b6a, 0x4b6a, PDF_CMAP_SINGLE, 7475 },
	{ 0x4b6b, 0x4b6f, PDF_CMAP_RANGE, 3737 },
	{ 0x4b70, 0x4b70, PDF_CMAP_SINGLE, 7796 },
	{ 0x4b71, 0x4b77, PDF_CMAP_RANGE, 3743 },
	{ 0x4b78, 0x4b79, PDF_CMAP_TABLE, 52 },
	{ 0x4b7a, 0x4b7e, PDF_CMAP_RANGE, 3752 },
	{ 0x4c21, 0x4c4c, PDF_CMAP_RANGE, 3757 },
	{ 0x4c4d, 0x4c4d, PDF_CMAP_SINGLE, 7797 },
	{ 0x4c4e, 0x4c58, PDF_CMAP_RANGE, 3802 },
	{ 0x4c59, 0x4c59, PDF_CMAP_SINGLE, 7798 },
	{ 0x4c5a, 0x4c5e, PDF_CMAP_RANGE, 3814 },
	{ 0x4c5f, 0x4c5f, PDF_CMAP_SINGLE, 7799 },
	{ 0x4c60, 0x4c61, PDF_CMAP_RANGE, 3820 },
	{ 0x4c62, 0x4c62, PDF_CMAP_SINGLE, 7800 },
	{ 0x4c63, 0x4c78, PDF_CMAP_RANGE, 3823 },
	{ 0x4c79, 0x4c7e, PDF_CMAP_TABLE, 54 },
	{ 0x4d21, 0x4d31, PDF_CMAP_RANGE, 3851 },
	{ 0x4d32, 0x4d32, PDF_CMAP_SINGLE, 7804 },
	{ 0x4d33, 0x4d4f, PDF_CMAP_RANGE, 3869 },
	{ 0x4d50, 0x4d50, PDF_CMAP_SINGLE, 7805 },
	{ 0x4d51, 0x4d53, PDF_CMAP_RANGE, 3899 },
	{ 0x4d54, 0x4d54, PDF_CMAP_SINGLE, 7806 },
	{ 0x4d55, 0x4d59, PDF_CMAP_RANGE, 3903 },
	{ 0x4d5a, 0x4d5a, PDF_CMAP_SINGLE, 7476 },
	{ 0x4d5b, 0x4d68, PDF_CMAP_RANGE, 3909 },
	{ 0x4d69, 0x4d69, PDF_CMAP_SINGLE, 7807 },
	{ 0x4d6a, 0x4d7e, PDF_CMAP_RANGE, 3924 },
	{ 0x4e21, 0x4e4a, PDF_CMAP_RANGE, 3945 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 7808 },
	{ 0x4e4c, 0x4e79, PDF_CMAP_RANGE, 3988 },
	{ 0x4e7a, 0x4e7b, PDF_CMAP_RANGE, 7809 },
	{ 0x4e7c, 0x4e7e, PDF_CMAP_RANGE, 4036 },
	{ 0x4f21, 0x4f21, PDF_CMAP_SINGLE, 7811 },
	{ 0x4f22, 0x4f30, PDF_CMAP_RANGE, 4040 },
	{ 0x4f31, 0x4f31, PDF_CMAP_SINGLE, 7812 },
	{ 0x4f32, 0x4f35, PDF_CMAP_RANGE, 4056 },
	{ 0x4f36, 0x4f36, PDF_CMAP_SINGLE, 6007 },
	{ 0x4f37, 0x4f38, PDF_CMAP_RANGE, 4061 },
	{ 0x4f39, 0x4f39, PDF_CMAP_SINGLE, 7813 },
	{ 0x4f3a, 0x4f53, PDF_CMAP_RANGE, 4064 },
	{ 0x5021, 0x5055, PDF_CMAP_RANGE, 4090 },
	{ 0x5056, 0x5056, PDF_CMAP_SINGLE, 3751 },
	{ 0x5057, 0x507e, PDF_CMAP_RANGE, 4144 },
	{ 0x5121, 0x513c, PDF_CMAP_RANGE, 4184 },
	{ 0x513d, 0x513d, PDF_CMAP_SINGLE, 7814 },
	{ 0x513e, 0x5146, PDF_CMAP_RANGE, 4213 },
	{ 0x5147, 0x5147, PDF_CMAP_SINGLE, 7815 },
	{ 0x5148, 0x514a, PDF_CMAP_RANGE, 4223 },
	{ 0x514b, 0x514d, PDF_CMAP_TABLE, 60 },
	{ 0x514e, 0x517e, PDF_CMAP_RANGE, 4229 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 4278 },
	{ 0x5321, 0x532f, PDF_CMAP_RANGE, 4372 },
	{ 0x5330, 0x5330, PDF_CMAP_SINGLE, 7818 },
	{ 0x5331, 0x5339, PDF_CMAP_RANGE, 4388 },
	{ 0x533a, 0x533a, PDF_CMAP_SINGLE, 7819 },
	{ 0x533b, 0x535d, PDF_CMAP_RANGE, 4398 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 7821 },
	{ 0x535f, 0x536a, PDF_CMAP_RANGE, 4434 },
	{ 0x536b, 0x536b, PDF_CMAP_SINGLE, 7822 },
	{ 0x536c, 0x537e, PDF_CMAP_RANGE, 4447 },
	{ 0x5421, 0x5443, PDF_CMAP_RANGE, 4466 },
	{ 0x5444, 0x5444, PDF_CMAP_SINGLE, 7823 },
	{ 0x5445, 0x5463, PDF_CMAP_RANGE, 4502 },
	{ 0x5464, 0x5464, PDF_CMAP_SINGLE, 3063 },
	{ 0x5465, 0x547e, PDF_CMAP_RANGE, 4534 },
	{ 0x5521, 0x553c, PDF_CMAP_RANGE, 4560 },
	{ 0x553d, 0x553d, PDF_CMAP_SINGLE, 7824 },
	{ 0x553e, 0x5562, PDF_CMAP_RANGE, 4589 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 7825 },
	{ 0x5564, 0x557e, PDF_CMAP_RANGE, 4627 },
	{ 0x5621, 0x5622, PDF_CMAP_TABLE, 63 },
	{ 0x5623, 0x567e, PDF_CMAP_RANGE, 4656 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 4748 },
	{ 0x5821, 0x5823, PDF_CMAP_RANGE, 4842 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 7828 },
	{ 0x5825, 0x587e, PDF_CMAP_RANGE, 4846 },
	{ 0x5921, 0x595f, PDF_CMAP_RANGE, 4936 },
	{ 0x5960, 0x5960, PDF_CMAP_SINGLE, 7829 },
	{ 0x5961, 0x596b, PDF_CMAP_RANGE, 5000 },
	{ 0x596c, 0x596c, PDF_CMAP_SINGLE, 7830 },
	{ 0x596d, 0x5977, PDF_CMAP_RANGE, 5012 },
	{ 0x5978, 0x5978, PDF_CMAP_SINGLE, 1447 },
	{ 0x5979, 0x597e, PDF_CMAP_RANGE, 5024 },
	{ 0x5a21, 0x5a38, PDF_CMAP_RANGE, 5030 },
	{ 0x5a39, 0x5a39, PDF_CMAP_SINGLE, 7831 },
	{ 0x5a3a, 0x5a4c, PDF_CMAP_RANGE, 5055 },
	{ 0x5a4d, 0x5a4d, PDF_CMAP_SINGLE, 7832 },
	{ 0x5a4e, 0x5a7e, PDF_CMAP_RANGE, 5075 },
	{ 0x5b21, 0x5b44, PDF_CMAP_RANGE, 5124 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 7833 },
	{ 0x5b46, 0x5b49, PDF_CMAP_RANGE, 5161 },
	{ 0x5b4a, 0x5b4a, PDF_CMAP_SINGLE, 7834 },
	{ 0x5b4b, 0x5b57, PDF_CMAP_RANGE, 5166 },
	{ 0x5b58, 0x5b58, PDF_CMAP_SINGLE, 3490 },
	{ 0x5b59, 0x5b6a, PDF_CMAP_RANGE, 5180 },
	{ 0x5b6b, 0x5b6d, PDF_CMAP_TABLE, 65 },
	{ 0x5b6e, 0x5b73, PDF_CMAP_RANGE, 5201 },
	{ 0x5b74, 0x5b74, PDF_CMAP_SINGLE, 7836 },
	{ 0x5b75, 0x5b7e, PDF_CMAP_RANGE, 5208 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5218 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 5312 },
	{ 0x5e21, 0x5e38, PDF_CMAP_RANGE, 5406 },
	{ 0x5e39, 0x5e39, PDF_CMAP_SINGLE, 3181 },
	{ 0x5e3a, 0x5e4f, PDF_CMAP_RANGE, 5431 },
	{ 0x5e50, 0x5e50, PDF_CMAP_SINGLE, 7837 },
	{ 0x5e51, 0x5e74, PDF_CMAP_RANGE, 5454 },
	{ 0x5e75, 0x5e75, PDF_CMAP_SINGLE, 1535 },
	{ 0x5e76, 0x5e7e, PDF_CMAP_RANGE, 5491 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 5500 },
	{ 0x6021, 0x6025, PDF_CMAP_RANGE, 5594 },
	{ 0x6026, 0x6026, PDF_CMAP_SINGLE, 7839 },
	{ 0x6027, 0x605e, PDF_CMAP_RANGE, 5600 },
	{ 0x605f, 0x605f, PDF_CMAP_SINGLE, 7840 },
	{ 0x6060, 0x6075, PDF_CMAP_RANGE, 5657 },
	{ 0x6076, 0x6076, PDF_CMAP_SINGLE, 7477 },
	{ 0x6077, 0x607e, PDF_CMAP_RANGE, 5680 },
	{ 0x6121, 0x612a, PDF_CMAP_RANGE, 5688 },
	{ 0x612b, 0x612b, PDF_CMAP_SINGLE, 7841 },
	{ 0x612c, 0x612f, PDF_CMAP_RANGE, 5699 },
	{ 0x6130, 0x6131, PDF_CMAP_RANGE, 7842 },
	{ 0x6132, 0x617e, PDF_CMAP_RANGE, 5705 },
	{ 0x6221, 0x622a, PDF_CMAP_RANGE, 5782 },
	{ 0x622b, 0x622b, PDF_CMAP_SINGLE, 7844 },
	{ 0x622c, 0x6267, PDF_CMAP_RANGE, 5793 },
	{ 0x6268, 0x626a, PDF_CMAP_TABLE, 68 },
	{ 0x626b, 0x626e, PDF_CMAP_RANGE, 5856 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 7845 },
	{ 0x6270, 0x627e, PDF_CMAP_RANGE, 5861 },
	{ 0x6321, 0x6349, PDF_CMAP_RANGE, 5876 },
	{ 0x634a, 0x634a, PDF_CMAP_SINGLE, 7846 },
	{ 0x634b, 0x6353, PDF_CMAP_RANGE, 5918 },
	{ 0x6354, 0x6354, PDF_CMAP_SINGLE, 7847 },
	{ 0x6355, 0x635d, PDF_CMAP_RANGE, 5928 },
	{ 0x635e, 0x635e, PDF_CMAP_SINGLE, 1492 },
	{ 0x635f, 0x637e, PDF_CMAP_RANGE, 5938 },
	{ 0x6421, 0x6438, PDF_CMAP_RANGE, 5970 },
	{ 0x6439, 0x6439, PDF_CMAP_SINGLE, 7848 },
	{ 0x643a, 0x6445, PDF_CMAP_RANGE, 5995 },
	{ 0x6446, 0x6446, PDF_CMAP_SINGLE, 4060 },
	{ 0x6447, 0x6463, PDF_CMAP_RANGE, 6008 },
	{ 0x6464, 0x6464, PDF_CMAP_SINGLE, 7849 },
	{ 0x6465, 0x646d, PDF_CMAP_RANGE, 6038 },
	{ 0x646e, 0x646e, PDF_CMAP_SINGLE, 7850 },
	{ 0x646f, 0x647e, PDF_CMAP_RANGE, 6048 },
	{ 0x6521, 0x6538, PDF_CMAP_RANGE, 6064 },
	{ 0x6539, 0x653b, PDF_CMAP_TABLE, 71 },
	{ 0x653c, 0x6545, PDF_CMAP_RANGE, 6091 },
	{ 0x6546, 0x6546, PDF_CMAP_SINGLE, 7853 },
	{ 0x6547, 0x657e, PDF_CMAP_RANGE, 6102 },
	{ 0x6621, 0x6645, PDF_CMAP_RANGE, 6158 },
	{ 0x6646, 0x6646, PDF_CMAP_SINGLE, 7854 },
	{ 0x6647, 0x667e, PDF_CMAP_RANGE, 6196 },
	{ 0x6721, 0x6763, PDF_CMAP_RANGE, 6252 },
	{ 0x6764, 0x6764, PDF_CMAP_SINGLE, 7855 },
	{ 0x6765, 0x6768, PDF_CMAP_RANGE, 6320 },
	{ 0x6769, 0x6769, PDF_CMAP_SINGLE, 7856 },
	{ 0x676a, 0x6771, PDF_CMAP_RANGE, 6325 },
	{ 0x6772, 0x6772, PDF_CMAP_SINGLE, 7857 },
	{ 0x6773, 0x677e, PDF_CMAP_RANGE, 6334 },
	{ 0x6821, 0x6833, PDF_CMAP_RANGE, 6346 },
	{ 0x6834, 0x6834, PDF_CMAP_SINGLE, 7858 },
	{ 0x6835, 0x683a, PDF_CMAP_RANGE, 6366 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 7859 },
	{ 0x683c, 0x6873, PDF_CMAP_RANGE, 6373 },
	{ 0x6874, 0x6874, PDF_CMAP_SINGLE, 7860 },
	{ 0x6875, 0x687e, PDF_CMAP_RANGE, 6430 },
	{ 0x6921, 0x6922, PDF_CMAP_TABLE, 74 },
	{ 0x6923, 0x692d, PDF_CMAP_RANGE, 6442 },
	{ 0x692e, 0x692e, PDF_CMAP_SINGLE, 3845 },
	{ 0x692f, 0x6959, PDF_CMAP_RANGE, 6454 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 1440 },
	{ 0x695b, 0x697e, PDF_CMAP_RANGE, 6498 },
	{ 0x6a21, 0x6a23, PDF_CMAP_RANGE, 6534 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 3358 },
	{ 0x6a25, 0x6a26, PDF_CMAP_RANGE, 6538 },
	{ 0x6a27, 0x6a27, PDF_CMAP_SINGLE, 7862 },
	{ 0x6a28, 0x6a3c, PDF_CMAP_RANGE, 6541 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 7863 },
	{ 0x6a3e, 0x6a6e, PDF_CMAP_RANGE, 6563 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 7864 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6613 },
	{ 0x6b21, 0x6b31, PDF_CMAP_RANGE, 6628 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 7865 },
	{ 0x6b33, 0x6b5c, PDF_CMAP_RANGE, 6646 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 1550 },
	{ 0x6b5e, 0x6b65, PDF_CMAP_RANGE, 6689 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 7866 },
	{ 0x6b67, 0x6b75, PDF_CMAP_RANGE, 6698 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 7867 },
	{ 0x6b77, 0x6b7e, PDF_CMAP_RANGE, 6714 },
	{ 0x6c21, 0x6c4c, PDF_CMAP_RANGE, 6722 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2730 },
	{ 0x6c4e, 0x6c68, PDF_CMAP_RANGE, 6767 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 7868 },
	{ 0x6c6a, 0x6c73, PDF_CMAP_RANGE, 6795 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 7869 },
	{ 0x6c75, 0x6c7e, PDF_CMAP_RANGE, 6806 },
	{ 0x6d21, 0x6d4d, PDF_CMAP_RANGE, 6816 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 7870 },
	{ 0x6d4f, 0x6d6b, PDF_CMAP_RANGE, 6862 },
	{ 0x6d6c, 0x6d6e, PDF_CMAP_TABLE, 76 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6894 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e56, PDF_CMAP_RANGE, 6939 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 7875 },
	{ 0x6e58, 0x6e7e, PDF_CMAP_RANGE, 6965 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 7004 },
	{ 0x7021, 0x7044, PDF_CMAP_RANGE, 7098 },
	{ 0x7045, 0x7045, PDF_CMAP_SINGLE, 7878 },
	{ 0x7046, 0x7050, PDF_CMAP_RANGE, 7135 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x7056, PDF_CMAP_RANGE, 7147 },
	{ 0x7057, 0x7057, PDF_CMAP_SINGLE, 7880 },
	{ 0x7058, 0x7073, PDF_CMAP_RANGE, 7153 },
	{ 0x7074, 0x7075, PDF_CMAP_TABLE, 79 },
	{ 0x7076, 0x707e, PDF_CMAP_RANGE, 7183 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724c, PDF_CMAP_RANGE, 7314 },
	{ 0x724d, 0x724e, PDF_CMAP_TABLE, 81 },
	{ 0x724f, 0x7273, PDF_CMAP_RANGE, 7332 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1321 },
	{ 0x7275, 0x727e, PDF_CMAP_RANGE, 7370 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x737c, PDF_CMAP_RANGE, 7429 },
	{ 0x737d, 0x737e, PDF_CMAP_TABLE, 83 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 7866 },
	{ 0x6b67, 0x6b75, PDF_CMAP_RANGE, 6698 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 7867 },
	{ 0x6b77, 0x6b7e, PDF_CMAP_RANGE, 6714 },
	{ 0x6c21, 0x6c4c, PDF_CMAP_RANGE, 6722 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2730 },
	{ 0x6c4e, 0x6c68, PDF_CMAP_RANGE, 6767 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 7868 },
	{ 0x6c6a, 0x6c73, PDF_CMAP_RANGE, 6795 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 7869 },
	{ 0x6c75, 0x6c7e, PDF_CMAP_RANGE, 6806 },
	{ 0x6d21, 0x6d4d, PDF_CMAP_RANGE, 6816 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 7870 },
	{ 0x6d4f, 0x6d6b, PDF_CMAP_RANGE, 6862 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 7871 },
	{ 0x6d6d, 0x6d6d, PDF_CMAP_SINGLE, 6892 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 7872 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6894 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e56, PDF_CMAP_RANGE, 6939 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 7875 },
	{ 0x6e58, 0x6e7e, PDF_CMAP_RANGE, 6965 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 7004 },
	{ 0x7021, 0x7044, PDF_CMAP_RANGE, 7098 },
	{ 0x7045, 0x7045, PDF_CMAP_SINGLE, 7878 },
	{ 0x7046, 0x7050, PDF_CMAP_RANGE, 7135 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x7056, PDF_CMAP_RANGE, 7147 },
	{ 0x7057, 0x7057, PDF_CMAP_SINGLE, 7880 },
	{ 0x7058, 0x7073, PDF_CMAP_RANGE, 7153 },
	{ 0x7074, 0x7074, PDF_CMAP_SINGLE, 1841 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 7881 },
	{ 0x7076, 0x707e, PDF_CMAP_RANGE, 7183 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724c, PDF_CMAP_RANGE, 7314 },
	{ 0x724d, 0x724d, PDF_CMAP_SINGLE, 1143 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 7884 },
	{ 0x724f, 0x7273, PDF_CMAP_RANGE, 7332 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1321 },
	{ 0x7275, 0x727e, PDF_CMAP_RANGE, 7370 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x737c, PDF_CMAP_RANGE, 7429 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 7886 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 7473 },
};

static const unsigned short pdf_cmap_78_H_table[85] =
{
	1125,7633,7961,7330,7645,7369,7646,7649,
	1406,1407,7650,7655,5490,1689,7660,7665,
	1780,7666,1782,7689,2169,7690,7693,6441,
	2629,7713,7720,6766,7733,2910,7734,2918,
	7744,3045,7745,7746,3050,7747,3099,7748,
	7772,6537,7773,7776,3395,7777,7783,5179,
	3491,7784,3663,7793,7980,4143,6453,7801,
	3847,7802,3849,7803,7816,4227,7817,4654,
	7826,7835,5199,3176,2030,5854,3153,7851,
	6089,7852,6440,2293,7871,6892,7872,1841,
	7881,1143,7884,7886,7473,
};

pdf_cmap pdf_cmap_78_H =
{
	-1, "78-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	623, 623, (pdf_range*) pdf_cmap_78_H_ranges,
	85, 85, (unsigned short*) pdf_cmap_78_H_table,
};

/* 78-RKSJ-H */

static const pdf_range pdf_cmap_78_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x889f, 0x88a0, PDF_CMAP_TABLE, 0 },
	{ 0x88a1, 0x88a6, PDF_CMAP_RANGE, 1127 },
	{ 0x88a7, 0x88a7, PDF_CMAP_SINGLE, 8266 },
	{ 0x88a8, 0x88af, PDF_CMAP_RANGE, 1134 },
	{ 0x88b0, 0x88b1, PDF_CMAP_TABLE, 2 },
	{ 0x88b2, 0x88b8, PDF_CMAP_RANGE, 1144 },
	{ 0x88b9, 0x88b9, PDF_CMAP_SINGLE, 7634 },
	{ 0x88ba, 0x88eb, PDF_CMAP_RANGE, 1152 },
	{ 0x88ec, 0x88ec, PDF_CMAP_SINGLE, 7635 },
	{ 0x88ed, 0x88f0, PDF_CMAP_RANGE, 1203 },
	{ 0x88f1, 0x88f1, PDF_CMAP_SINGLE, 7636 },
	{ 0x88f2, 0x88f9, PDF_CMAP_RANGE, 1208 },
	{ 0x88fa, 0x88fa, PDF_CMAP_SINGLE, 7637 },
	{ 0x88fb, 0x88fc, PDF_CMAP_RANGE, 1217 },
	{ 0x8940, 0x8948, PDF_CMAP_RANGE, 1219 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 7638 },
	{ 0x894a, 0x8953, PDF_CMAP_RANGE, 1229 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 7639 },
	{ 0x8955, 0x8957, PDF_CMAP_RANGE, 1240 },
	{ 0x8958, 0x8958, PDF_CMAP_SINGLE, 7640 },
	{ 0x8959, 0x895b, PDF_CMAP_RANGE, 1244 },
	{ 0x895c, 0x895c, PDF_CMAP_SINGLE, 7642 },
	{ 0x895d, 0x8960, PDF_CMAP_RANGE, 1248 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 7643 },
	{ 0x8962, 0x897e, PDF_CMAP_RANGE, 1253 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 1282 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 7644 },
	{ 0x898c, 0x89a5, PDF_CMAP_RANGE, 1294 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 4 },
	{ 0x89a9, 0x89dd, PDF_CMAP_RANGE, 1323 },
	{ 0x89de, 0x89de, PDF_CMAP_SINGLE, 7647 },
	{ 0x89df, 0x89f7, PDF_CMAP_RANGE, 1377 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7648 },
	{ 0x89f9, 0x89fa, PDF_CMAP_RANGE, 1403 },
	{ 0x89fb, 0x89fc, PDF_CMAP_TABLE, 7 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 9 },
	{ 0x8a42, 0x8a60, PDF_CMAP_RANGE, 1409 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 6497 },
	{ 0x8a62, 0x8a67, PDF_CMAP_RANGE, 1441 },
	{ 0x8a68, 0x8a68, PDF_CMAP_SINGLE, 5023 },
	{ 0x8a69, 0x8a7e, PDF_CMAP_RANGE, 1448 },
	{ 0x8a80, 0x8a84, PDF_CMAP_RANGE, 1470 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 7651 },
	{ 0x8a86, 0x8a8a, PDF_CMAP_RANGE, 1476 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 7652 },
	{ 0x8a8c, 0x8a92, PDF_CMAP_RANGE, 1482 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 7653 },
	{ 0x8a94, 0x8a95, PDF_CMAP_RANGE, 1490 },
	{ 0x8a96, 0x8a96, PDF_CMAP_SINGLE, 5937 },
	{ 0x8a97, 0x8a99, PDF_CMAP_RANGE, 1493 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 7654 },
	{ 0x8a9b, 0x8abf, PDF_CMAP_RANGE, 1497 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_TABLE, 11 },
	{ 0x8ac2, 0x8aca, PDF_CMAP_RANGE, 1536 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 7656 },
	{ 0x8acc, 0x8acf, PDF_CMAP_RANGE, 1546 },
	{ 0x8ad0, 0x8ad0, PDF_CMAP_SINGLE, 6688 },
	{ 0x8ad1, 0x8ae2, PDF_CMAP_RANGE, 1551 },
	{ 0x8ae3, 0x8ae3, PDF_CMAP_SINGLE, 7657 },
	{ 0x8ae4, 0x8afc, PDF_CMAP_RANGE, 1570 },
	{ 0x8b40, 0x8b49, PDF_CMAP_RANGE, 1595 },
	{ 0x8b4a, 0x8b4a, PDF_CMAP_SINGLE, 7658 },
	{ 0x8b4b, 0x8b5e, PDF_CMAP_RANGE, 1606 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 7659 },
	{ 0x8b60, 0x8b7e, PDF_CMAP_RANGE, 1627 },
	{ 0x8b80, 0x8b9f, PDF_CMAP_RANGE, 1658 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 7660 },
	{ 0x8ba1, 0x8ba7, PDF_CMAP_RANGE, 1691 },
	{ 0x8ba8, 0x8ba8, PDF_CMAP_SINGLE, 7661 },
	{ 0x8ba9, 0x8bc3, PDF_CMAP_RANGE, 1699 },
	{ 0x8bc4, 0x8bc4, PDF_CMAP_SINGLE, 7474 },
	{ 0x8bc5, 0x8bcc, PDF_CMAP_RANGE, 1727 },
	{ 0x8bcd, 0x8bcd, PDF_CMAP_SINGLE, 7662 },
	{ 0x8bce, 0x8bea, PDF_CMAP_RANGE, 1736 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 7663 },
	{ 0x8bec, 0x8bf1, PDF_CMAP_RANGE, 1766 },
	{ 0x8bf2, 0x8bf2, PDF_CMAP_SINGLE, 7664 },
	{ 0x8bf3, 0x8bf8, PDF_CMAP_RANGE, 1773 },
	{ 0x8bf9, 0x8bfc, PDF_CMAP_TABLE, 13 },
	{ 0x8c40, 0x8c42, PDF_CMAP_RANGE, 1783 },
	{ 0x8c43, 0x8c43, PDF_CMAP_SINGLE, 7667 },
	{ 0x8c44, 0x8c55, PDF_CMAP_RANGE, 1787 },
	{ 0x8c56, 0x8c56, PDF_CMAP_SINGLE, 7668 },
	{ 0x8c57, 0x8c63, PDF_CMAP_RANGE, 1806 },
	{ 0x8c64, 0x8c64, PDF_CMAP_SINGLE, 7669 },
	{ 0x8c65, 0x8c6c, PDF_CMAP_RANGE, 1820 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 7670 },
	{ 0x8c6e, 0x8c70, PDF_CMAP_RANGE, 1829 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 7671 },
	{ 0x8c72, 0x8c73, PDF_CMAP_RANGE, 1833 },
	{ 0x8c74, 0x8c74, PDF_CMAP_SINGLE, 7672 },
	{ 0x8c75, 0x8c79, PDF_CMAP_RANGE, 1836 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 7181 },
	{ 0x8c7b, 0x8c7e, PDF_CMAP_RANGE, 1842 },
	{ 0x8c80, 0x8c83, PDF_CMAP_RANGE, 1846 },
	{ 0x8c84, 0x8c84, PDF_CMAP_SINGLE, 7673 },
	{ 0x8c85, 0x8c90, PDF_CMAP_RANGE, 1851 },
	{ 0x8c91, 0x8c91, PDF_CMAP_SINGLE, 7674 },
	{ 0x8c92, 0x8c98, PDF_CMAP_RANGE, 1864 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 7675 },
	{ 0x8c9a, 0x8c9d, PDF_CMAP_RANGE, 1872 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 7676 },
	{ 0x8c9f, 0x8cb1, PDF_CMAP_RANGE, 1877 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 7677 },
	{ 0x8cb3, 0x8cbe, PDF_CMAP_RANGE, 1897 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 7678 },
	{ 0x8cc0, 0x8cfc, PDF_CMAP_RANGE, 1910 },
	{ 0x8d40, 0x8d49, PDF_CMAP_RANGE, 1971 },
	{ 0x8d4a, 0x8d4a, PDF_CMAP_SINGLE, 7679 },
	{ 0x8d4b, 0x8d55, PDF_CMAP_RANGE, 1982 },
	{ 0x8d56, 0x8d56, PDF_CMAP_SINGLE, 7680 },
	{ 0x8d57, 0x8d60, PDF_CMAP_RANGE, 1994 },
	{ 0x8d61, 0x8d61, PDF_CMAP_SINGLE, 7681 },
	{ 0x8d62, 0x8d7a, PDF_CMAP_RANGE, 2005 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 5853 },
	{ 0x8d7c, 0x8d7e, PDF_CMAP_RANGE, 2031 },
	{ 0x8d80, 0x8d8c, PDF_CMAP_RANGE, 2034 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 7682 },
	{ 0x8d8e, 0x8d93, PDF_CMAP_RANGE, 2048 },
	{ 0x8d94, 0x8d94, PDF_CMAP_SINGLE, 7683 },
	{ 0x8d95, 0x8d98, PDF_CMAP_RANGE, 2055 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 7684 },
	{ 0x8d9a, 0x8dd0, PDF_CMAP_RANGE, 2060 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 7685 },
	{ 0x8dd2, 0x8de4, PDF_CMAP_RANGE, 2116 },
	{ 0x8de5, 0x8de5, PDF_CMAP_SINGLE, 7686 },
	{ 0x8de6, 0x8df1, PDF_CMAP_RANGE, 2136 },
	{ 0x8df2, 0x8df2, PDF_CMAP_SINGLE, 7687 },
	{ 0x8df3, 0x8dfc, PDF_CMAP_RANGE, 2149 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 2159 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 7688 },
	{ 0x8e47, 0x8e48, PDF_CMAP_RANGE, 2166 },
	{ 0x8e49, 0x8e4b, PDF_CMAP_TABLE, 17 },
	{ 0x8e4c, 0x8e57, PDF_CMAP_RANGE, 2171 },
	{ 0x8e58, 0x8e58, PDF_CMAP_SINGLE, 7691 },
	{ 0x8e59, 0x8e7e, PDF_CMAP_RANGE, 2184 },
	{ 0x8e80, 0x8ec5, PDF_CMAP_RANGE, 2222 },
	{ 0x8ec6, 0x8ec7, PDF_CMAP_TABLE, 20 },
	{ 0x8ec8, 0x8ed4, PDF_CMAP_RANGE, 2294 },
	{ 0x8ed5, 0x8ed5, PDF_CMAP_SINGLE, 7694 },
	{ 0x8ed6, 0x8eda, PDF_CMAP_RANGE, 2308 },
	{ 0x8edb, 0x8edc, PDF_CMAP_RANGE, 7695 },
	{ 0x8edd, 0x8efc, PDF_CMAP_RANGE, 2315 },
	{ 0x8f40, 0x8f49, PDF_CMAP_RANGE, 2347 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 7697 },
	{ 0x8f4b, 0x8f54, PDF_CMAP_RANGE, 2358 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 7698 },
	{ 0x8f56, 0x8f7e, PDF_CMAP_RANGE, 2369 },
	{ 0x8f80, 0x8f8b, PDF_CMAP_RANGE, 2410 },
	{ 0x8f8c, 0x8f8d, PDF_CMAP_RANGE, 7699 },
	{ 0x8f8e, 0x8f91, PDF_CMAP_RANGE, 2424 },
	{ 0x8f92, 0x8f93, PDF_CMAP_RANGE, 7701 },
	{ 0x8f94, 0x8fa2, PDF_CMAP_RANGE, 2430 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 7703 },
	{ 0x8fa4, 0x8fb0, PDF_CMAP_RANGE, 2446 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 7704 },
	{ 0x8fb2, 0x8fbc, PDF_CMAP_RANGE, 2460 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 7705 },
	{ 0x8fbe, 0x8fd2, PDF_CMAP_RANGE, 2472 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 7706 },
	{ 0x8fd4, 0x8fdc, PDF_CMAP_RANGE, 2494 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 7707 },
	{ 0x8fde, 0x8fe1, PDF_CMAP_RANGE, 2504 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 7708 },
	{ 0x8fe3, 0x8ffc, PDF_CMAP_RANGE, 2509 },
	{ 0x9040, 0x9048, PDF_CMAP_RANGE, 2535 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 7709 },
	{ 0x904a, 0x9077, PDF_CMAP_RANGE, 2545 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 7710 },
	{ 0x9079, 0x907e, PDF_CMAP_RANGE, 2592 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 7711 },
	{ 0x9081, 0x9088, PDF_CMAP_RANGE, 2599 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 7712 },
	{ 0x908a, 0x909f, PDF_CMAP_RANGE, 2608 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 7713 },
	{ 0x90a1, 0x90bf, PDF_CMAP_RANGE, 2631 },
	{ 0x90c0, 0x90c0, PDF_CMAP_SINGLE, 7714 },
	{ 0x90c1, 0x90e3, PDF_CMAP_RANGE, 2663 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 7715 },
	{ 0x90e5, 0x90ee, PDF_CMAP_RANGE, 2699 },
	{ 0x90ef, 0x90f0, PDF_CMAP_RANGE, 7716 },
	{ 0x90f1, 0x90f6, PDF_CMAP_RANGE, 2711 },
	{ 0x90f7, 0x90f8, PDF_CMAP_RANGE, 7718 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 2719 },
	{ 0x9140, 0x9145, PDF_CMAP_RANGE, 2723 },
	{ 0x9146, 0x9147, PDF_CMAP_TABLE, 22 },
	{ 0x9148, 0x9157, PDF_CMAP_RANGE, 2731 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 7721 },
	{ 0x9159, 0x916a, PDF_CMAP_RANGE, 2748 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 7722 },
	{ 0x916c, 0x916d, PDF_CMAP_RANGE, 2767 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 7723 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 2770 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 7724 },
	{ 0x9180, 0x9188, PDF_CMAP_RANGE, 2786 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 7725 },
	{ 0x918a, 0x91ba, PDF_CMAP_RANGE, 2796 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 7726 },
	{ 0x91bc, 0x91ca, PDF_CMAP_RANGE, 2846 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 7727 },
	{ 0x91cc, 0x91d9, PDF_CMAP_RANGE, 2862 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 7728 },
	{ 0x91db, 0x91e0, PDF_CMAP_RANGE, 2877 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 7729 },
	{ 0x91e2, 0x91ec, PDF_CMAP_RANGE, 2884 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 7730 },
	{ 0x91ee, 0x91f2, PDF_CMAP_RANGE, 2896 },
	{ 0x91f3, 0x91f4, PDF_CMAP_RANGE, 7731 },
	{ 0x91f5, 0x91fa, PDF_CMAP_RANGE, 2903 },
	{ 0x91fb, 0x91fc, PDF_CMAP_TABLE, 24 },
	{ 0x9240, 0x9245, PDF_CMAP_RANGE, 2911 },
	{ 0x9246, 0x9247, PDF_CMAP_TABLE, 26 },
	{ 0x9248, 0x9249, PDF_CMAP_RANGE, 7735 },
	{ 0x924a, 0x924b, PDF_CMAP_RANGE, 2921 },
	{ 0x924c, 0x924d, PDF_CMAP_RANGE, 7737 },
	{ 0x924e, 0x925b, PDF_CMAP_RANGE, 2925 },
	{ 0x925c, 0x925c, PDF_CMAP_SINGLE, 7739 },
	{ 0x925d, 0x927e, PDF_CMAP_RANGE, 2940 },
	{ 0x9280, 0x928f, PDF_CMAP_RANGE, 2974 },
	{ 0x9290, 0x9290, PDF_CMAP_SINGLE, 7740 },
	{ 0x9291, 0x9294, PDF_CMAP_RANGE, 2991 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 7741 },
	{ 0x9296, 0x929b, PDF_CMAP_RANGE, 2996 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 7742 },
	{ 0x929d, 0x92ba, PDF_CMAP_RANGE, 3003 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 7743 },
	{ 0x92bc, 0x92c5, PDF_CMAP_RANGE, 3034 },
	{ 0x92c6, 0x92c8, PDF_CMAP_TABLE, 28 },
	{ 0x92c9, 0x92ca, PDF_CMAP_RANGE, 3047 },
	{ 0x92cb, 0x92cd, PDF_CMAP_TABLE, 31 },
	{ 0x92ce, 0x92d1, PDF_CMAP_RANGE, 3052 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 8267 },
	{ 0x92d3, 0x92d8, PDF_CMAP_RANGE, 3057 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 4533 },
	{ 0x92da, 0x92fc, PDF_CMAP_RANGE, 3064 },
	{ 0x9340, 0x9341, PDF_CMAP_TABLE, 34 },
	{ 0x9342, 0x9345, PDF_CMAP_RANGE, 3101 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 7749 },
	{ 0x9347, 0x934c, PDF_CMAP_RANGE, 3106 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 7750 },
	{ 0x934e, 0x9354, PDF_CMAP_RANGE, 3113 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 7751 },
	{ 0x9356, 0x935d, PDF_CMAP_RANGE, 3121 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 7752 },
	{ 0x935f, 0x9366, PDF_CMAP_RANGE, 3130 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 7753 },
	{ 0x9368, 0x9369, PDF_CMAP_RANGE, 3139 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 7754 },
	{ 0x936b, 0x936f, PDF_CMAP_RANGE, 3142 },
	{ 0x9370, 0x9371, PDF_CMAP_RANGE, 7755 },
	{ 0x9372, 0x9375, PDF_CMAP_RANGE, 3149 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 5855 },
	{ 0x9377, 0x937e, PDF_CMAP_RANGE, 3154 },
	{ 0x9380, 0x9383, PDF_CMAP_RANGE, 3162 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 7757 },
	{ 0x9385, 0x938d, PDF_CMAP_RANGE, 3167 },
	{ 0x938e, 0x938e, PDF_CMAP_SINGLE, 5200 },
	{ 0x938f, 0x9392, PDF_CMAP_RANGE, 3177 },
	{ 0x9393, 0x9393, PDF_CMAP_SINGLE, 5430 },
	{ 0x9394, 0x9397, PDF_CMAP_RANGE, 3182 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 7758 },
	{ 0x9399, 0x93bb, PDF_CMAP_RANGE, 3187 },
	{ 0x93bc, 0x93bc, PDF_CMAP_SINGLE, 7759 },
	{ 0x93bd, 0x93bf, PDF_CMAP_RANGE, 3223 },
	{ 0x93c0, 0x93c0, PDF_CMAP_SINGLE, 7760 },
	{ 0x93c1, 0x93d1, PDF_CMAP_RANGE, 3227 },
	{ 0x93d2, 0x93d3, PDF_CMAP_RANGE, 7761 },
	{ 0x93d4, 0x93d8, PDF_CMAP_RANGE, 3246 },
	{ 0x93d9, 0x93da, PDF_CMAP_RANGE, 7763 },
	{ 0x93db, 0x93de, PDF_CMAP_RANGE, 3253 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 7765 },
	{ 0x93e0, 0x93e3, PDF_CMAP_RANGE, 3258 },
	{ 0x93e4, 0x93e5, PDF_CMAP_RANGE, 7766 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 3264 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7768 },
	{ 0x93e9, 0x93f3, PDF_CMAP_RANGE, 3267 },
	{ 0x93f4, 0x93f4, PDF_CMAP_SINGLE, 6893 },
	{ 0x93f5, 0x93fc, PDF_CMAP_RANGE, 3279 },
	{ 0x9440, 0x9447, PDF_CMAP_RANGE, 3287 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 7769 },
	{ 0x9449, 0x9457, PDF_CMAP_RANGE, 3296 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 7770 },
	{ 0x9459, 0x9475, PDF_CMAP_RANGE, 3312 },
	{ 0x9476, 0x9476, PDF_CMAP_SINGLE, 7771 },
	{ 0x9477, 0x947e, PDF_CMAP_RANGE, 3342 },
	{ 0x9480, 0x9486, PDF_CMAP_RANGE, 3350 },
	{ 0x9487, 0x9489, PDF_CMAP_TABLE, 36 },
	{ 0x948a, 0x948c, PDF_CMAP_RANGE, 3360 },
	{ 0x948d, 0x948d, PDF_CMAP_SINGLE, 7774 },
	{ 0x948e, 0x94a1, PDF_CMAP_RANGE, 3364 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 7775 },
	{ 0x94a3, 0x94ab, PDF_CMAP_RANGE, 3385 },
	{ 0x94ac, 0x94ae, PDF_CMAP_TABLE, 39 },
	{ 0x94af, 0x94d1, PDF_CMAP_RANGE, 3397 },
	{ 0x94d2, 0x94d2, PDF_CMAP_SINGLE, 7778 },
	{ 0x94d3, 0x94df, PDF_CMAP_RANGE, 3433 },
	{ 0x94e0, 0x94e0, PDF_CMAP_SINGLE, 7779 },
	{ 0x94e1, 0x94f2, PDF_CMAP_RANGE, 3447 },
	{ 0x94f3, 0x94f3, PDF_CMAP_SINGLE, 7780 },
	{ 0x94f4, 0x94fc, PDF_CMAP_RANGE, 3466 },
	{ 0x9540, 0x9540, PDF_CMAP_SINGLE, 3475 },
	{ 0x9541, 0x9542, PDF_CMAP_RANGE, 7781 },
	{ 0x9543, 0x954d, PDF_CMAP_RANGE, 3478 },
	{ 0x954e, 0x9551, PDF_CMAP_TABLE, 42 },
	{ 0x9552, 0x9553, PDF_CMAP_RANGE, 3493 },
	{ 0x9554, 0x9554, PDF_CMAP_SINGLE, 7785 },
	{ 0x9555, 0x955e, PDF_CMAP_RANGE, 3496 },
	{ 0x955f, 0x955f, PDF_CMAP_SINGLE, 7786 },
	{ 0x9560, 0x956c, PDF_CMAP_RANGE, 3507 },
	{ 0x956d, 0x956d, PDF_CMAP_SINGLE, 7787 },
	{ 0x956e, 0x956f, PDF_CMAP_RANGE, 3521 },
	{ 0x9570, 0x9570, PDF_CMAP_SINGLE, 7788 },
	{ 0x9571, 0x957e, PDF_CMAP_RANGE, 3524 },
	{ 0x9580, 0x95c0, PDF_CMAP_RANGE, 3538 },
	{ 0x95c1, 0x95c1, PDF_CMAP_SINGLE, 7789 },
	{ 0x95c2, 0x95ca, PDF_CMAP_RANGE, 3604 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 7790 },
	{ 0x95cc, 0x95d7, PDF_CMAP_RANGE, 3614 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 7791 },
	{ 0x95d9, 0x95f6, PDF_CMAP_RANGE, 3627 },
	{ 0x95f7, 0x95f7, PDF_CMAP_SINGLE, 7792 },
	{ 0x95f8, 0x95fc, PDF_CMAP_RANGE, 3658 },
	{ 0x9640, 0x9641, PDF_CMAP_TABLE, 46 },
	{ 0x9642, 0x9647, PDF_CMAP_RANGE, 3665 },
	{ 0x9648, 0x9648, PDF_CMAP_SINGLE, 7794 },
	{ 0x9649, 0x9669, PDF_CMAP_RANGE, 3672 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 7795 },
	{ 0x966b, 0x967e, PDF_CMAP_RANGE, 3706 },
	{ 0x9680, 0x9689, PDF_CMAP_RANGE, 3726 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 7475 },
	{ 0x968b, 0x968f, PDF_CMAP_RANGE, 3737 },
	{ 0x9690, 0x9690, PDF_CMAP_SINGLE, 7796 },
	{ 0x9691, 0x9697, PDF_CMAP_RANGE, 3743 },
	{ 0x9698, 0x9699, PDF_CMAP_TABLE, 48 },
	{ 0x969a, 0x96ca, PDF_CMAP_RANGE, 3752 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 7797 },
	{ 0x96cc, 0x96d6, PDF_CMAP_RANGE, 3802 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 7798 },
	{ 0x96d8, 0x96dc, PDF_CMAP_RANGE, 3814 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 7799 },
	{ 0x96de, 0x96df, PDF_CMAP_RANGE, 3820 },
	{ 0x96e0, 0x96e0, PDF_CMAP_SINGLE, 7800 },
	{ 0x96e1, 0x96f6, PDF_CMAP_RANGE, 3823 },
	{ 0x96f7, 0x96fc, PDF_CMAP_TABLE, 50 },
	{ 0x9740, 0x9750, PDF_CMAP_RANGE, 3851 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 7804 },
	{ 0x9752, 0x976e, PDF_CMAP_RANGE, 3869 },
	{ 0x976f, 0x976f, PDF_CMAP_SINGLE, 7805 },
	{ 0x9770, 0x9772, PDF_CMAP_RANGE, 3899 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 7806 },
	{ 0x9774, 0x9778, PDF_CMAP_RANGE, 3903 },
	{ 0x9779, 0x9779, PDF_CMAP_SINGLE, 7476 },
	{ 0x977a, 0x977e, PDF_CMAP_RANGE, 3909 },
	{ 0x9780, 0x9788, PDF_CMAP_RANGE, 3914 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 7807 },
	{ 0x978a, 0x97c8, PDF_CMAP_RANGE, 3924 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 7808 },
	{ 0x97ca, 0x97f7, PDF_CMAP_RANGE, 3988 },
	{ 0x97f8, 0x97f9, PDF_CMAP_RANGE, 7809 },
	{ 0x97fa, 0x97fc, PDF_CMAP_RANGE, 4036 },
	{ 0x9840, 0x9840, PDF_CMAP_SINGLE, 7811 },
	{ 0x9841, 0x984f, PDF_CMAP_RANGE, 4040 },
	{ 0x9850, 0x9850, PDF_CMAP_SINGLE, 7812 },
	{ 0x9851, 0x9854, PDF_CMAP_RANGE, 4056 },
	{ 0x9855, 0x9855, PDF_CMAP_SINGLE, 6007 },
	{ 0x9856, 0x9857, PDF_CMAP_RANGE, 4061 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 7813 },
	{ 0x9859, 0x9872, PDF_CMAP_RANGE, 4064 },
	{ 0x989f, 0x98d3, PDF_CMAP_RANGE, 4090 },
	{ 0x98d4, 0x98d4, PDF_CMAP_SINGLE, 3751 },
	{ 0x98d5, 0x98fc, PDF_CMAP_RANGE, 4144 },
	{ 0x9940, 0x995b, PDF_CMAP_RANGE, 4184 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 7814 },
	{ 0x995d, 0x9965, PDF_CMAP_RANGE, 4213 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 7815 },
	{ 0x9967, 0x9969, PDF_CMAP_RANGE, 4223 },
	{ 0x996a, 0x996c, PDF_CMAP_TABLE, 56 },
	{ 0x996d, 0x997e, PDF_CMAP_RANGE, 4229 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a4e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 7818 },
	{ 0x9a50, 0x9a58, PDF_CMAP_RANGE, 4388 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 7819 },
	{ 0x9a5a, 0x9a7c, PDF_CMAP_RANGE, 4398 },
	{ 0x9a7d, 0x9a7e, PDF_CMAP_TABLE, 59 },
	{ 0x9a80, 0x9a8a, PDF_CMAP_RANGE, 4435 },
	{ 0x9a8b, 0x9a8b, PDF_CMAP_SINGLE, 7822 },
	{ 0x9a8c, 0x9ac1, PDF_CMAP_RANGE, 4447 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 7823 },
	{ 0x9ac3, 0x9ae1, PDF_CMAP_RANGE, 4502 },
	{ 0x9ae2, 0x9ae2, PDF_CMAP_SINGLE, 3063 },
	{ 0x9ae3, 0x9afc, PDF_CMAP_RANGE, 4534 },
	{ 0x9b40, 0x9b5b, PDF_CMAP_RANGE, 4560 },
	{ 0x9b5c, 0x9b5c, PDF_CMAP_SINGLE, 7824 },
	{ 0x9b5d, 0x9b7e, PDF_CMAP_RANGE, 4589 },
	{ 0x9b80, 0x9b82, PDF_CMAP_RANGE, 4623 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7825 },
	{ 0x9b84, 0x9b9f, PDF_CMAP_RANGE, 4627 },
	{ 0x9ba0, 0x9ba0, PDF_CMAP_SINGLE, 7826 },
	{ 0x9ba1, 0x9bfc, PDF_CMAP_RANGE, 4656 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9ca1, PDF_CMAP_RANGE, 4811 },
	{ 0x9ca2, 0x9ca2, PDF_CMAP_SINGLE, 7828 },
	{ 0x9ca3, 0x9cfc, PDF_CMAP_RANGE, 4846 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9d80, PDF_CMAP_SINGLE, 7829 },
	{ 0x9d81, 0x9d8b, PDF_CMAP_RANGE, 5000 },
	{ 0x9d8c, 0x9d8c, PDF_CMAP_SINGLE, 7830 },
	{ 0x9d8d, 0x9d97, PDF_CMAP_RANGE, 5012 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 1447 },
	{ 0x9d99, 0x9db6, PDF_CMAP_RANGE, 5024 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 7831 },
	{ 0x9db8, 0x9dca, PDF_CMAP_RANGE, 5055 },
	{ 0x9dcb, 0x9dcb, PDF_CMAP_SINGLE, 7832 },
	{ 0x9dcc, 0x9dfc, PDF_CMAP_RANGE, 5075 },
	{ 0x9e40, 0x9e63, PDF_CMAP_RANGE, 5124 },
	{ 0x9e64, 0x9e64, PDF_CMAP_SINGLE, 7833 },
	{ 0x9e65, 0x9e68, PDF_CMAP_RANGE, 5161 },
	{ 0x9e69, 0x9e69, PDF_CMAP_SINGLE, 7834 },
	{ 0x9e6a, 0x9e76, PDF_CMAP_RANGE, 5166 },
	{ 0x9e77, 0x9e77, PDF_CMAP_SINGLE, 3490 },
	{ 0x9e78, 0x9e7e, PDF_CMAP_RANGE, 5180 },
	{ 0x9e80, 0x9e8a, PDF_CMAP_RANGE, 5187 },
	{ 0x9e8b, 0x9e8d, PDF_CMAP_TABLE, 61 },
	{ 0x9e8e, 0x9e93, PDF_CMAP_RANGE, 5201 },
	{ 0x9e94, 0x9e94, PDF_CMAP_SINGLE, 7836 },
	{ 0x9e95, 0x9efc, PDF_CMAP_RANGE, 5208 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9fb6, PDF_CMAP_RANGE, 5375 },
	{ 0x9fb7, 0x9fb7, PDF_CMAP_SINGLE, 3181 },
	{ 0x9fb8, 0x9fcd, PDF_CMAP_RANGE, 5431 },
	{ 0x9fce, 0x9fce, PDF_CMAP_SINGLE, 7837 },
	{ 0x9fcf, 0x9ff2, PDF_CMAP_RANGE, 5454 },
	{ 0x9ff3, 0x9ff3, PDF_CMAP_SINGLE, 1535 },
	{ 0x9ff4, 0x9ffc, PDF_CMAP_RANGE, 5491 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0a3, PDF_CMAP_RANGE, 5563 },
	{ 0xe0a4, 0xe0a4, PDF_CMAP_SINGLE, 7839 },
	{ 0xe0a5, 0xe0dc, PDF_CMAP_RANGE, 5600 },
	{ 0xe0dd, 0xe0dd, PDF_CMAP_SINGLE, 7840 },
	{ 0xe0de, 0xe0f3, PDF_CMAP_RANGE, 5657 },
	{ 0xe0f4, 0xe0f4, PDF_CMAP_SINGLE, 7477 },
	{ 0xe0f5, 0xe0fc, PDF_CMAP_RANGE, 5680 },
	{ 0xe140, 0xe149, PDF_CMAP_RANGE, 5688 },
	{ 0xe14a, 0xe14a, PDF_CMAP_SINGLE, 7841 },
	{ 0xe14b, 0xe14e, PDF_CMAP_RANGE, 5699 },
	{ 0xe14f, 0xe150, PDF_CMAP_RANGE, 7842 },
	{ 0xe151, 0xe17e, PDF_CMAP_RANGE, 5705 },
	{ 0xe180, 0xe1a8, PDF_CMAP_RANGE, 5751 },
	{ 0xe1a9, 0xe1a9, PDF_CMAP_SINGLE, 7844 },
	{ 0xe1aa, 0xe1e5, PDF_CMAP_RANGE, 5793 },
	{ 0xe1e6, 0xe1e8, PDF_CMAP_TABLE, 64 },
	{ 0xe1e9, 0xe1ec, PDF_CMAP_RANGE, 5856 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 7845 },
	{ 0xe1ee, 0xe1fc, PDF_CMAP_RANGE, 5861 },
	{ 0xe240, 0xe268, PDF_CMAP_RANGE, 5876 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 7846 },
	{ 0xe26a, 0xe272, PDF_CMAP_RANGE, 5918 },
	{ 0xe273, 0xe273, PDF_CMAP_SINGLE, 7847 },
	{ 0xe274, 0xe27c, PDF_CMAP_RANGE, 5928 },
	{ 0xe27d, 0xe27e, PDF_CMAP_TABLE, 67 },
	{ 0xe280, 0xe2b6, PDF_CMAP_RANGE, 5939 },
	{ 0xe2b7, 0xe2b7, PDF_CMAP_SINGLE, 7848 },
	{ 0xe2b8, 0xe2c3, PDF_CMAP_RANGE, 5995 },
	{ 0xe2c4, 0xe2c4, PDF_CMAP_SINGLE, 4060 },
	{ 0xe2c5, 0xe2e1, PDF_CMAP_RANGE, 6008 },
	{ 0xe2e2, 0xe2e2, PDF_CMAP_SINGLE, 7849 },
	{ 0xe2e3, 0xe2eb, PDF_CMAP_RANGE, 6038 },
	{ 0xe2ec, 0xe2ec, PDF_CMAP_SINGLE, 7850 },
	{ 0xe2ed, 0xe2fc, PDF_CMAP_RANGE, 6048 },
	{ 0xe340, 0xe357, PDF_CMAP_RANGE, 6064 },
	{ 0xe358, 0xe35a, PDF_CMAP_TABLE, 69 },
	{ 0xe35b, 0xe364, PDF_CMAP_RANGE, 6091 },
	{ 0xe365, 0xe365, PDF_CMAP_SINGLE, 7853 },
	{ 0xe366, 0xe37e, PDF_CMAP_RANGE, 6102 },
	{ 0xe380, 0xe3c3, PDF_CMAP_RANGE, 6127 },
	{ 0xe3c4, 0xe3c4, PDF_CMAP_SINGLE, 7854 },
	{ 0xe3c5, 0xe3fc, PDF_CMAP_RANGE, 6196 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe483, PDF_CMAP_RANGE, 6315 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 7855 },
	{ 0xe485, 0xe488, PDF_CMAP_RANGE, 6320 },
	{ 0xe489, 0xe489, PDF_CMAP_SINGLE, 7856 },
	{ 0xe48a, 0xe491, PDF_CMAP_RANGE, 6325 },
	{ 0xe492, 0xe492, PDF_CMAP_SINGLE, 7857 },
	{ 0xe493, 0xe4b1, PDF_CMAP_RANGE, 6334 },
	{ 0xe4b2, 0xe4b2, PDF_CMAP_SINGLE, 7858 },
	{ 0xe4b3, 0xe4b8, PDF_CMAP_RANGE, 6366 },
	{ 0xe4b9, 0xe4b9, PDF_CMAP_SINGLE, 7859 },
	{ 0xe4ba, 0xe4f1, PDF_CMAP_RANGE, 6373 },
	{ 0xe4f2, 0xe4f2, PDF_CMAP_SINGLE, 7860 },
	{ 0xe4f3, 0xe4fc, PDF_CMAP_RANGE, 6430 },
	{ 0xe540, 0xe541, PDF_CMAP_TABLE, 72 },
	{ 0xe542, 0xe54c, PDF_CMAP_RANGE, 6442 },
	{ 0xe54d, 0xe54d, PDF_CMAP_SINGLE, 3845 },
	{ 0xe54e, 0xe578, PDF_CMAP_RANGE, 6454 },
	{ 0xe579, 0xe579, PDF_CMAP_SINGLE, 1440 },
	{ 0xe57a, 0xe57e, PDF_CMAP_RANGE, 6498 },
	{ 0xe580, 0xe5a1, PDF_CMAP_RANGE, 6503 },
	{ 0xe5a2, 0xe5a2, PDF_CMAP_SINGLE, 3358 },
	{ 0xe5a3, 0xe5a4, PDF_CMAP_RANGE, 6538 },
	{ 0xe5a5, 0xe5a5, PDF_CMAP_SINGLE, 7862 },
	{ 0xe5a6, 0xe5ba, PDF_CMAP_RANGE, 6541 },
	{ 0xe5bb, 0xe5bb, PDF_CMAP_SINGLE, 7863 },
	{ 0xe5bc, 0xe5ec, PDF_CMAP_RANGE, 6563 },
	{ 0xe5ed, 0xe5ed, PDF_CMAP_SINGLE, 7864 },
	{ 0xe5ee, 0xe5fc, PDF_CMAP_RANGE, 6613 },
	{ 0xe640, 0xe650, PDF_CMAP_RANGE, 6628 },
	{ 0xe651, 0xe651, PDF_CMAP_SINGLE, 7865 },
	{ 0xe652, 0xe67b, PDF_CMAP_RANGE, 6646 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 1550 },
	{ 0xe67d, 0xe67e, PDF_CMAP_RANGE, 6689 },
	{ 0xe680, 0xe685, PDF_CMAP_RANGE, 6691 },
	{ 0xe686, 0xe686, PDF_CMAP_SINGLE, 7866 },
	{ 0xe687, 0xe695, PDF_CMAP_RANGE, 6698 },
	{ 0xe696, 0xe696, PDF_CMAP_SINGLE, 7867 },
	{ 0xe697, 0xe6ca, PDF_CMAP_RANGE, 6714 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 2730 },
	{ 0xe6cc, 0xe6e6, PDF_CMAP_RANGE, 6767 },
	{ 0xe6e7, 0xe6e7, PDF_CMAP_SINGLE, 7868 },
	{ 0xe6e8, 0xe6f1, PDF_CMAP_RANGE, 6795 },
	{ 0xe6f2, 0xe6f2, PDF_CMAP_SINGLE, 7869 },
	{ 0xe6f3, 0xe6fc, PDF_CMAP_RANGE, 6806 },
	{ 0xe740, 0xe76c, PDF_CMAP_RANGE, 6816 },
	{ 0xe76d, 0xe76d, PDF_CMAP_SINGLE, 7870 },
	{ 0xe76e, 0xe77e, PDF_CMAP_RANGE, 6862 },
	{ 0xe780, 0xe78b, PDF_CMAP_RANGE, 6879 },
	{ 0xe78c, 0xe78e, PDF_CMAP_TABLE, 74 },
	{ 0xe78f, 0xe7a6, PDF_CMAP_RANGE, 6894 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7d4, PDF_CMAP_RANGE, 6939 },
	{ 0xe7d5, 0xe7d5, PDF_CMAP_SINGLE, 7875 },
	{ 0xe7d6, 0xe7fc, PDF_CMAP_RANGE, 6965 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8c2, PDF_CMAP_RANGE, 7067 },
	{ 0xe8c3, 0xe8c3, PDF_CMAP_SINGLE, 7878 },
	{ 0xe8c4, 0xe8ce, PDF_CMAP_RANGE, 7135 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8d4, PDF_CMAP_RANGE, 7147 },
	{ 0xe8d5, 0xe8d5, PDF_CMAP_SINGLE, 7880 },
	{ 0xe8d6, 0xe8f1, PDF_CMAP_RANGE, 7153 },
	{ 0xe8f2, 0xe8f3, PDF_CMAP_TABLE, 77 },
	{ 0xe8f4, 0xe8fc, PDF_CMAP_RANGE, 7183 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9ca, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cb, 0xe9cc, PDF_CMAP_TABLE, 79 },
	{ 0xe9cd, 0xe9f1, PDF_CMAP_RANGE, 7332 },
	{ 0xe9f2, 0xe9f2, PDF_CMAP_SINGLE, 1321 },
	{ 0xe9f3, 0xe9fc, PDF_CMAP_RANGE, 7370 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea7e, PDF_CMAP_RANGE, 7429 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9e, PDF_CMAP_TABLE, 81 },
	{ 0xe687, 0xe695, PDF_CMAP_RANGE, 6698 },
	{ 0xe696, 0xe696, PDF_CMAP_SINGLE, 7867 },
	{ 0xe697, 0xe6ca, PDF_CMAP_RANGE, 6714 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 2730 },
	{ 0xe6cc, 0xe6e6, PDF_CMAP_RANGE, 6767 },
	{ 0xe6e7, 0xe6e7, PDF_CMAP_SINGLE, 7868 },
	{ 0xe6e8, 0xe6f1, PDF_CMAP_RANGE, 6795 },
	{ 0xe6f2, 0xe6f2, PDF_CMAP_SINGLE, 7869 },
	{ 0xe6f3, 0xe6fc, PDF_CMAP_RANGE, 6806 },
	{ 0xe740, 0xe76c, PDF_CMAP_RANGE, 6816 },
	{ 0xe76d, 0xe76d, PDF_CMAP_SINGLE, 7870 },
	{ 0xe76e, 0xe77e, PDF_CMAP_RANGE, 6862 },
	{ 0xe780, 0xe78b, PDF_CMAP_RANGE, 6879 },
	{ 0xe78c, 0xe78c, PDF_CMAP_SINGLE, 7871 },
	{ 0xe78d, 0xe78d, PDF_CMAP_SINGLE, 6892 },
	{ 0xe78e, 0xe78e, PDF_CMAP_SINGLE, 7872 },
	{ 0xe78f, 0xe7a6, PDF_CMAP_RANGE, 6894 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7d4, PDF_CMAP_RANGE, 6939 },
	{ 0xe7d5, 0xe7d5, PDF_CMAP_SINGLE, 7875 },
	{ 0xe7d6, 0xe7fc, PDF_CMAP_RANGE, 6965 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8c2, PDF_CMAP_RANGE, 7067 },
	{ 0xe8c3, 0xe8c3, PDF_CMAP_SINGLE, 7878 },
	{ 0xe8c4, 0xe8ce, PDF_CMAP_RANGE, 7135 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8d4, PDF_CMAP_RANGE, 7147 },
	{ 0xe8d5, 0xe8d5, PDF_CMAP_SINGLE, 7880 },
	{ 0xe8d6, 0xe8f1, PDF_CMAP_RANGE, 7153 },
	{ 0xe8f2, 0xe8f2, PDF_CMAP_SINGLE, 1841 },
	{ 0xe8f3, 0xe8f3, PDF_CMAP_SINGLE, 7881 },
	{ 0xe8f4, 0xe8fc, PDF_CMAP_RANGE, 7183 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9ca, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cb, 0xe9cb, PDF_CMAP_SINGLE, 1143 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 7884 },
	{ 0xe9cd, 0xe9f1, PDF_CMAP_RANGE, 7332 },
	{ 0xe9f2, 0xe9f2, PDF_CMAP_SINGLE, 1321 },
	{ 0xe9f3, 0xe9fc, PDF_CMAP_RANGE, 7370 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea7e, PDF_CMAP_RANGE, 7429 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9d, PDF_CMAP_SINGLE, 7886 },
	{ 0xea9e, 0xea9e, PDF_CMAP_SINGLE, 7473 },
};

static const unsigned short pdf_cmap_78_RKSJ_H_table[83] =
{
	1125,7633,7961,7330,7645,7369,7646,7649,
	1406,1407,7650,7655,5490,7665,1780,7666,
	1782,7689,2169,7690,7693,6441,7720,6766,
	7733,2910,7734,2918,7744,3045,7745,7746,
	3050,7747,3099,7748,7772,6537,7773,7776,
	3395,7777,7783,5179,3491,7784,3663,7793,
	7980,4143,6453,7801,3847,7802,3849,7803,
	7816,4227,7817,7821,4434,7835,5199,3176,
	2030,5854,3153,1492,5938,7851,6089,7852,
	6440,2293,7871,6892,7872,1841,7881,1143,
	7884,7886,7473,
};

pdf_cmap pdf_cmap_78_RKSJ_H =
{
	-1, "78-RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	626, 626, (pdf_range*) pdf_cmap_78_RKSJ_H_ranges,
	83, 83, (unsigned short*) pdf_cmap_78_RKSJ_H_table,
};

/* 78-RKSJ-V */

static const pdf_range pdf_cmap_78_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 7917 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_78_RKSJ_V_table[1] = { 0 };

pdf_cmap pdf_cmap_78_RKSJ_V =
{
	-1, "78-RKSJ-V", "78-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	27, 27, (pdf_range*) pdf_cmap_78_RKSJ_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_78_RKSJ_V_table,
};

/* 78-V */

static const pdf_range pdf_cmap_78_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_RANGE, 7887 },
	{ 0x2131, 0x2132, PDF_CMAP_RANGE, 7889 },
	{ 0x213c, 0x213e, PDF_CMAP_RANGE, 7891 },
	{ 0x2141, 0x2145, PDF_CMAP_RANGE, 7894 },
	{ 0x214a, 0x215b, PDF_CMAP_RANGE, 7899 },
	{ 0x2161, 0x2161, PDF_CMAP_SINGLE, 7917 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 7918 },
	{ 0x2423, 0x2423, PDF_CMAP_SINGLE, 7919 },
	{ 0x2425, 0x2425, PDF_CMAP_SINGLE, 7920 },
	{ 0x2427, 0x2427, PDF_CMAP_SINGLE, 7921 },
	{ 0x2429, 0x2429, PDF_CMAP_SINGLE, 7922 },
	{ 0x2443, 0x2443, PDF_CMAP_SINGLE, 7923 },
	{ 0x2463, 0x2463, PDF_CMAP_SINGLE, 7924 },
	{ 0x2465, 0x2465, PDF_CMAP_SINGLE, 7925 },
	{ 0x2467, 0x2467, PDF_CMAP_SINGLE, 7926 },
	{ 0x246e, 0x246e, PDF_CMAP_SINGLE, 7927 },
	{ 0x2521, 0x2521, PDF_CMAP_SINGLE, 7928 },
	{ 0x2523, 0x2523, PDF_CMAP_SINGLE, 7929 },
	{ 0x2525, 0x2525, PDF_CMAP_SINGLE, 7930 },
	{ 0x2527, 0x2527, PDF_CMAP_SINGLE, 7931 },
	{ 0x2529, 0x2529, PDF_CMAP_SINGLE, 7932 },
	{ 0x2543, 0x2543, PDF_CMAP_SINGLE, 7933 },
	{ 0x2563, 0x2563, PDF_CMAP_SINGLE, 7934 },
	{ 0x2565, 0x2565, PDF_CMAP_SINGLE, 7935 },
	{ 0x2567, 0x2567, PDF_CMAP_SINGLE, 7936 },
	{ 0x256e, 0x256e, PDF_CMAP_SINGLE, 7937 },
	{ 0x2575, 0x2576, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_78_V_table[1] = { 0 };

pdf_cmap pdf_cmap_78_V =
{
	-1, "78-V", "78-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	27, 27, (pdf_range*) pdf_cmap_78_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_78_V_table,
};

/* 78ms-RKSJ-H */

static const pdf_range pdf_cmap_78ms_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007d, PDF_CMAP_RANGE, 231 },
	{ 0x007e, 0x007e, PDF_CMAP_SINGLE, 631 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 0 },
	{ 0x8740, 0x875d, PDF_CMAP_RANGE, 7555 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7585 },
	{ 0x8761, 0x8765, PDF_CMAP_TABLE, 32 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7592 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8044 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7595 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8043 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7598 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8047 },
	{ 0x876f, 0x8775, PDF_CMAP_RANGE, 7601 },
	{ 0x877e, 0x877e, PDF_CMAP_SINGLE, 8323 },
	{ 0x8780, 0x8783, PDF_CMAP_RANGE, 7608 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 8055 },
	{ 0x8785, 0x878f, PDF_CMAP_RANGE, 7613 },
	{ 0x8790, 0x8792, PDF_CMAP_TABLE, 37 },
	{ 0x8793, 0x8799, PDF_CMAP_RANGE, 7624 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 768 },
	{ 0x879b, 0x879c, PDF_CMAP_RANGE, 7631 },
	{ 0x889f, 0x88a0, PDF_CMAP_TABLE, 40 },
	{ 0x88a1, 0x88a6, PDF_CMAP_RANGE, 1127 },
	{ 0x88a7, 0x88a7, PDF_CMAP_SINGLE, 8266 },
	{ 0x88a8, 0x88af, PDF_CMAP_RANGE, 1134 },
	{ 0x88b0, 0x88b1, PDF_CMAP_TABLE, 42 },
	{ 0x88b2, 0x88b8, PDF_CMAP_RANGE, 1144 },
	{ 0x88b9, 0x88b9, PDF_CMAP_SINGLE, 7634 },
	{ 0x88ba, 0x88eb, PDF_CMAP_RANGE, 1152 },
	{ 0x88ec, 0x88ec, PDF_CMAP_SINGLE, 7635 },
	{ 0x88ed, 0x88f0, PDF_CMAP_RANGE, 1203 },
	{ 0x88f1, 0x88f1, PDF_CMAP_SINGLE, 7636 },
	{ 0x88f2, 0x88f9, PDF_CMAP_RANGE, 1208 },
	{ 0x88fa, 0x88fa, PDF_CMAP_SINGLE, 7637 },
	{ 0x88fb, 0x88fc, PDF_CMAP_RANGE, 1217 },
	{ 0x8940, 0x8948, PDF_CMAP_RANGE, 1219 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 7638 },
	{ 0x894a, 0x8953, PDF_CMAP_RANGE, 1229 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 7639 },
	{ 0x8955, 0x8957, PDF_CMAP_RANGE, 1240 },
	{ 0x8958, 0x8958, PDF_CMAP_SINGLE, 7640 },
	{ 0x8959, 0x895b, PDF_CMAP_RANGE, 1244 },
	{ 0x895c, 0x895c, PDF_CMAP_SINGLE, 7642 },
	{ 0x895d, 0x8960, PDF_CMAP_RANGE, 1248 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 7643 },
	{ 0x8962, 0x897e, PDF_CMAP_RANGE, 1253 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 1282 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 7644 },
	{ 0x898c, 0x89a5, PDF_CMAP_RANGE, 1294 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 44 },
	{ 0x89a9, 0x89dd, PDF_CMAP_RANGE, 1323 },
	{ 0x89de, 0x89de, PDF_CMAP_SINGLE, 7647 },
	{ 0x89df, 0x89f7, PDF_CMAP_RANGE, 1377 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7648 },
	{ 0x89f9, 0x89fa, PDF_CMAP_RANGE, 1403 },
	{ 0x89fb, 0x89fc, PDF_CMAP_TABLE, 47 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 49 },
	{ 0x8a42, 0x8a60, PDF_CMAP_RANGE, 1409 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 6497 },
	{ 0x8a62, 0x8a67, PDF_CMAP_RANGE, 1441 },
	{ 0x8a68, 0x8a68, PDF_CMAP_SINGLE, 5023 },
	{ 0x8a69, 0x8a7e, PDF_CMAP_RANGE, 1448 },
	{ 0x8a80, 0x8a84, PDF_CMAP_RANGE, 1470 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 7651 },
	{ 0x8a86, 0x8a8a, PDF_CMAP_RANGE, 1476 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 7652 },
	{ 0x8a8c, 0x8a92, PDF_CMAP_RANGE, 1482 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 7653 },
	{ 0x8a94, 0x8a95, PDF_CMAP_RANGE, 1490 },
	{ 0x8a96, 0x8a96, PDF_CMAP_SINGLE, 5937 },
	{ 0x8a97, 0x8a99, PDF_CMAP_RANGE, 1493 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 7654 },
	{ 0x8a9b, 0x8abf, PDF_CMAP_RANGE, 1497 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_TABLE, 51 },
	{ 0x8ac2, 0x8aca, PDF_CMAP_RANGE, 1536 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 7656 },
	{ 0x8acc, 0x8acf, PDF_CMAP_RANGE, 1546 },
	{ 0x8ad0, 0x8ad0, PDF_CMAP_SINGLE, 6688 },
	{ 0x8ad1, 0x8ae2, PDF_CMAP_RANGE, 1551 },
	{ 0x8ae3, 0x8ae3, PDF_CMAP_SINGLE, 7657 },
	{ 0x8ae4, 0x8afc, PDF_CMAP_RANGE, 1570 },
	{ 0x8b40, 0x8b49, PDF_CMAP_RANGE, 1595 },
	{ 0x8b4a, 0x8b4a, PDF_CMAP_SINGLE, 7658 },
	{ 0x8b4b, 0x8b5e, PDF_CMAP_RANGE, 1606 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 7659 },
	{ 0x8b60, 0x8b7e, PDF_CMAP_RANGE, 1627 },
	{ 0x8b80, 0x8b9f, PDF_CMAP_RANGE, 1658 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 7660 },
	{ 0x8ba1, 0x8ba7, PDF_CMAP_RANGE, 1691 },
	{ 0x8ba8, 0x8ba8, PDF_CMAP_SINGLE, 7661 },
	{ 0x8ba9, 0x8bc3, PDF_CMAP_RANGE, 1699 },
	{ 0x8bc4, 0x8bc4, PDF_CMAP_SINGLE, 7474 },
	{ 0x8bc5, 0x8bcc, PDF_CMAP_RANGE, 1727 },
	{ 0x8bcd, 0x8bcd, PDF_CMAP_SINGLE, 7662 },
	{ 0x8bce, 0x8bea, PDF_CMAP_RANGE, 1736 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 7663 },
	{ 0x8bec, 0x8bf1, PDF_CMAP_RANGE, 1766 },
	{ 0x8bf2, 0x8bf2, PDF_CMAP_SINGLE, 7664 },
	{ 0x8bf3, 0x8bf8, PDF_CMAP_RANGE, 1773 },
	{ 0x8bf9, 0x8bfc, PDF_CMAP_TABLE, 53 },
	{ 0x8c40, 0x8c42, PDF_CMAP_RANGE, 1783 },
	{ 0x8c43, 0x8c43, PDF_CMAP_SINGLE, 7667 },
	{ 0x8c44, 0x8c55, PDF_CMAP_RANGE, 1787 },
	{ 0x8c56, 0x8c56, PDF_CMAP_SINGLE, 7668 },
	{ 0x8c57, 0x8c63, PDF_CMAP_RANGE, 1806 },
	{ 0x8c64, 0x8c64, PDF_CMAP_SINGLE, 7669 },
	{ 0x8c65, 0x8c6c, PDF_CMAP_RANGE, 1820 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 7670 },
	{ 0x8c6e, 0x8c70, PDF_CMAP_RANGE, 1829 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 7671 },
	{ 0x8c72, 0x8c73, PDF_CMAP_RANGE, 1833 },
	{ 0x8c74, 0x8c74, PDF_CMAP_SINGLE, 7672 },
	{ 0x8c75, 0x8c79, PDF_CMAP_RANGE, 1836 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 7181 },
	{ 0x8c7b, 0x8c7e, PDF_CMAP_RANGE, 1842 },
	{ 0x8c80, 0x8c83, PDF_CMAP_RANGE, 1846 },
	{ 0x8c84, 0x8c84, PDF_CMAP_SINGLE, 7673 },
	{ 0x8c85, 0x8c90, PDF_CMAP_RANGE, 1851 },
	{ 0x8c91, 0x8c91, PDF_CMAP_SINGLE, 7674 },
	{ 0x8c92, 0x8c98, PDF_CMAP_RANGE, 1864 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 7675 },
	{ 0x8c9a, 0x8c9d, PDF_CMAP_RANGE, 1872 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 7676 },
	{ 0x8c9f, 0x8cb1, PDF_CMAP_RANGE, 1877 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 7677 },
	{ 0x8cb3, 0x8cbe, PDF_CMAP_RANGE, 1897 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 7678 },
	{ 0x8cc0, 0x8cfc, PDF_CMAP_RANGE, 1910 },
	{ 0x8d40, 0x8d49, PDF_CMAP_RANGE, 1971 },
	{ 0x8d4a, 0x8d4a, PDF_CMAP_SINGLE, 7679 },
	{ 0x8d4b, 0x8d55, PDF_CMAP_RANGE, 1982 },
	{ 0x8d56, 0x8d56, PDF_CMAP_SINGLE, 7680 },
	{ 0x8d57, 0x8d60, PDF_CMAP_RANGE, 1994 },
	{ 0x8d61, 0x8d61, PDF_CMAP_SINGLE, 7681 },
	{ 0x8d62, 0x8d7a, PDF_CMAP_RANGE, 2005 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 5853 },
	{ 0x8d7c, 0x8d7e, PDF_CMAP_RANGE, 2031 },
	{ 0x8d80, 0x8d8c, PDF_CMAP_RANGE, 2034 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 7682 },
	{ 0x8d8e, 0x8d93, PDF_CMAP_RANGE, 2048 },
	{ 0x8d94, 0x8d94, PDF_CMAP_SINGLE, 7683 },
	{ 0x8d95, 0x8d98, PDF_CMAP_RANGE, 2055 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 7684 },
	{ 0x8d9a, 0x8dd0, PDF_CMAP_RANGE, 2060 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 7685 },
	{ 0x8dd2, 0x8de4, PDF_CMAP_RANGE, 2116 },
	{ 0x8de5, 0x8de5, PDF_CMAP_SINGLE, 7686 },
	{ 0x8de6, 0x8df1, PDF_CMAP_RANGE, 2136 },
	{ 0x8df2, 0x8df2, PDF_CMAP_SINGLE, 7687 },
	{ 0x8df3, 0x8dfc, PDF_CMAP_RANGE, 2149 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 2159 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 7688 },
	{ 0x8e47, 0x8e48, PDF_CMAP_RANGE, 2166 },
	{ 0x8e49, 0x8e4b, PDF_CMAP_TABLE, 57 },
	{ 0x8e4c, 0x8e57, PDF_CMAP_RANGE, 2171 },
	{ 0x8e58, 0x8e58, PDF_CMAP_SINGLE, 7691 },
	{ 0x8e59, 0x8e7e, PDF_CMAP_RANGE, 2184 },
	{ 0x8e80, 0x8ec5, PDF_CMAP_RANGE, 2222 },
	{ 0x8ec6, 0x8ec7, PDF_CMAP_TABLE, 60 },
	{ 0x8ec8, 0x8ed4, PDF_CMAP_RANGE, 2294 },
	{ 0x8ed5, 0x8ed5, PDF_CMAP_SINGLE, 7694 },
	{ 0x8ed6, 0x8eda, PDF_CMAP_RANGE, 2308 },
	{ 0x8edb, 0x8edc, PDF_CMAP_RANGE, 7695 },
	{ 0x8edd, 0x8efc, PDF_CMAP_RANGE, 2315 },
	{ 0x8f40, 0x8f49, PDF_CMAP_RANGE, 2347 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 7697 },
	{ 0x8f4b, 0x8f54, PDF_CMAP_RANGE, 2358 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 7698 },
	{ 0x8f56, 0x8f7e, PDF_CMAP_RANGE, 2369 },
	{ 0x8f80, 0x8f8b, PDF_CMAP_RANGE, 2410 },
	{ 0x8f8c, 0x8f8d, PDF_CMAP_RANGE, 7699 },
	{ 0x8f8e, 0x8f91, PDF_CMAP_RANGE, 2424 },
	{ 0x8f92, 0x8f93, PDF_CMAP_RANGE, 7701 },
	{ 0x8f94, 0x8fa2, PDF_CMAP_RANGE, 2430 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 7703 },
	{ 0x8fa4, 0x8fb0, PDF_CMAP_RANGE, 2446 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 7704 },
	{ 0x8fb2, 0x8fbc, PDF_CMAP_RANGE, 2460 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 7705 },
	{ 0x8fbe, 0x8fd2, PDF_CMAP_RANGE, 2472 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 7706 },
	{ 0x8fd4, 0x8fdc, PDF_CMAP_RANGE, 2494 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 7707 },
	{ 0x8fde, 0x8fe1, PDF_CMAP_RANGE, 2504 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 7708 },
	{ 0x8fe3, 0x8ffc, PDF_CMAP_RANGE, 2509 },
	{ 0x9040, 0x9048, PDF_CMAP_RANGE, 2535 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 7709 },
	{ 0x904a, 0x9077, PDF_CMAP_RANGE, 2545 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 7710 },
	{ 0x9079, 0x907e, PDF_CMAP_RANGE, 2592 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 7711 },
	{ 0x9081, 0x9088, PDF_CMAP_RANGE, 2599 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 7712 },
	{ 0x908a, 0x909f, PDF_CMAP_RANGE, 2608 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 7713 },
	{ 0x90a1, 0x90bf, PDF_CMAP_RANGE, 2631 },
	{ 0x90c0, 0x90c0, PDF_CMAP_SINGLE, 7714 },
	{ 0x90c1, 0x90e3, PDF_CMAP_RANGE, 2663 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 7715 },
	{ 0x90e5, 0x90ee, PDF_CMAP_RANGE, 2699 },
	{ 0x90ef, 0x90f0, PDF_CMAP_RANGE, 7716 },
	{ 0x90f1, 0x90f6, PDF_CMAP_RANGE, 2711 },
	{ 0x90f7, 0x90f8, PDF_CMAP_RANGE, 7718 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 2719 },
	{ 0x9140, 0x9145, PDF_CMAP_RANGE, 2723 },
	{ 0x9146, 0x9147, PDF_CMAP_TABLE, 62 },
	{ 0x9148, 0x9157, PDF_CMAP_RANGE, 2731 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 7721 },
	{ 0x9159, 0x916a, PDF_CMAP_RANGE, 2748 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 7722 },
	{ 0x916c, 0x916d, PDF_CMAP_RANGE, 2767 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 7723 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 2770 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 7724 },
	{ 0x9180, 0x9188, PDF_CMAP_RANGE, 2786 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 7725 },
	{ 0x918a, 0x91ba, PDF_CMAP_RANGE, 2796 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 7726 },
	{ 0x91bc, 0x91ca, PDF_CMAP_RANGE, 2846 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 7727 },
	{ 0x91cc, 0x91d9, PDF_CMAP_RANGE, 2862 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 7728 },
	{ 0x91db, 0x91e0, PDF_CMAP_RANGE, 2877 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 7729 },
	{ 0x91e2, 0x91ec, PDF_CMAP_RANGE, 2884 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 7730 },
	{ 0x91ee, 0x91f2, PDF_CMAP_RANGE, 2896 },
	{ 0x91f3, 0x91f4, PDF_CMAP_RANGE, 7731 },
	{ 0x91f5, 0x91fa, PDF_CMAP_RANGE, 2903 },
	{ 0x91fb, 0x91fc, PDF_CMAP_TABLE, 64 },
	{ 0x9240, 0x9245, PDF_CMAP_RANGE, 2911 },
	{ 0x9246, 0x9247, PDF_CMAP_TABLE, 66 },
	{ 0x9248, 0x9249, PDF_CMAP_RANGE, 7735 },
	{ 0x924a, 0x924b, PDF_CMAP_RANGE, 2921 },
	{ 0x924c, 0x924d, PDF_CMAP_RANGE, 7737 },
	{ 0x924e, 0x925b, PDF_CMAP_RANGE, 2925 },
	{ 0x925c, 0x925c, PDF_CMAP_SINGLE, 7739 },
	{ 0x925d, 0x927e, PDF_CMAP_RANGE, 2940 },
	{ 0x9280, 0x928f, PDF_CMAP_RANGE, 2974 },
	{ 0x9290, 0x9290, PDF_CMAP_SINGLE, 7740 },
	{ 0x9291, 0x9294, PDF_CMAP_RANGE, 2991 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 7741 },
	{ 0x9296, 0x929b, PDF_CMAP_RANGE, 2996 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 7742 },
	{ 0x929d, 0x92ba, PDF_CMAP_RANGE, 3003 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 7743 },
	{ 0x92bc, 0x92c5, PDF_CMAP_RANGE, 3034 },
	{ 0x92c6, 0x92c8, PDF_CMAP_TABLE, 68 },
	{ 0x92c9, 0x92ca, PDF_CMAP_RANGE, 3047 },
	{ 0x92cb, 0x92cd, PDF_CMAP_TABLE, 71 },
	{ 0x92ce, 0x92d1, PDF_CMAP_RANGE, 3052 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 8267 },
	{ 0x92d3, 0x92d8, PDF_CMAP_RANGE, 3057 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 4533 },
	{ 0x92da, 0x92fc, PDF_CMAP_RANGE, 3064 },
	{ 0x9340, 0x9341, PDF_CMAP_TABLE, 74 },
	{ 0x9342, 0x9345, PDF_CMAP_RANGE, 3101 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 7749 },
	{ 0x9347, 0x934c, PDF_CMAP_RANGE, 3106 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 7750 },
	{ 0x934e, 0x9354, PDF_CMAP_RANGE, 3113 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 7751 },
	{ 0x9356, 0x935d, PDF_CMAP_RANGE, 3121 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 7752 },
	{ 0x935f, 0x9366, PDF_CMAP_RANGE, 3130 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 7753 },
	{ 0x9368, 0x9369, PDF_CMAP_RANGE, 3139 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 7754 },
	{ 0x936b, 0x936f, PDF_CMAP_RANGE, 3142 },
	{ 0x9370, 0x9371, PDF_CMAP_RANGE, 7755 },
	{ 0x9372, 0x9375, PDF_CMAP_RANGE, 3149 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 5855 },
	{ 0x9377, 0x937e, PDF_CMAP_RANGE, 3154 },
	{ 0x9380, 0x9383, PDF_CMAP_RANGE, 3162 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 7757 },
	{ 0x9385, 0x938d, PDF_CMAP_RANGE, 3167 },
	{ 0x938e, 0x938e, PDF_CMAP_SINGLE, 5200 },
	{ 0x938f, 0x9392, PDF_CMAP_RANGE, 3177 },
	{ 0x9393, 0x9393, PDF_CMAP_SINGLE, 5430 },
	{ 0x9394, 0x9397, PDF_CMAP_RANGE, 3182 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 7758 },
	{ 0x9399, 0x93bb, PDF_CMAP_RANGE, 3187 },
	{ 0x93bc, 0x93bc, PDF_CMAP_SINGLE, 7759 },
	{ 0x93bd, 0x93bf, PDF_CMAP_RANGE, 3223 },
	{ 0x93c0, 0x93c0, PDF_CMAP_SINGLE, 7760 },
	{ 0x93c1, 0x93d1, PDF_CMAP_RANGE, 3227 },
	{ 0x93d2, 0x93d3, PDF_CMAP_RANGE, 7761 },
	{ 0x93d4, 0x93d8, PDF_CMAP_RANGE, 3246 },
	{ 0x93d9, 0x93da, PDF_CMAP_RANGE, 7763 },
	{ 0x93db, 0x93de, PDF_CMAP_RANGE, 3253 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 7765 },
	{ 0x93e0, 0x93e3, PDF_CMAP_RANGE, 3258 },
	{ 0x93e4, 0x93e5, PDF_CMAP_RANGE, 7766 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 3264 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7768 },
	{ 0x93e9, 0x93f3, PDF_CMAP_RANGE, 3267 },
	{ 0x93f4, 0x93f4, PDF_CMAP_SINGLE, 6893 },
	{ 0x93f5, 0x93fc, PDF_CMAP_RANGE, 3279 },
	{ 0x9440, 0x9447, PDF_CMAP_RANGE, 3287 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 7769 },
	{ 0x9449, 0x9457, PDF_CMAP_RANGE, 3296 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 7770 },
	{ 0x9459, 0x9475, PDF_CMAP_RANGE, 3312 },
	{ 0x9476, 0x9476, PDF_CMAP_SINGLE, 7771 },
	{ 0x9477, 0x947e, PDF_CMAP_RANGE, 3342 },
	{ 0x9480, 0x9486, PDF_CMAP_RANGE, 3350 },
	{ 0x9487, 0x9489, PDF_CMAP_TABLE, 76 },
	{ 0x948a, 0x948c, PDF_CMAP_RANGE, 3360 },
	{ 0x948d, 0x948d, PDF_CMAP_SINGLE, 7774 },
	{ 0x948e, 0x94a1, PDF_CMAP_RANGE, 3364 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 7775 },
	{ 0x94a3, 0x94ab, PDF_CMAP_RANGE, 3385 },
	{ 0x94ac, 0x94ae, PDF_CMAP_TABLE, 79 },
	{ 0x94af, 0x94d1, PDF_CMAP_RANGE, 3397 },
	{ 0x94d2, 0x94d2, PDF_CMAP_SINGLE, 7778 },
	{ 0x94d3, 0x94df, PDF_CMAP_RANGE, 3433 },
	{ 0x94e0, 0x94e0, PDF_CMAP_SINGLE, 7779 },
	{ 0x94e1, 0x94f2, PDF_CMAP_RANGE, 3447 },
	{ 0x94f3, 0x94f3, PDF_CMAP_SINGLE, 7780 },
	{ 0x94f4, 0x94fc, PDF_CMAP_RANGE, 3466 },
	{ 0x9540, 0x9540, PDF_CMAP_SINGLE, 3475 },
	{ 0x9541, 0x9542, PDF_CMAP_RANGE, 7781 },
	{ 0x9543, 0x954d, PDF_CMAP_RANGE, 3478 },
	{ 0x954e, 0x9551, PDF_CMAP_TABLE, 82 },
	{ 0x9552, 0x9553, PDF_CMAP_RANGE, 3493 },
	{ 0x9554, 0x9554, PDF_CMAP_SINGLE, 7785 },
	{ 0x9555, 0x955e, PDF_CMAP_RANGE, 3496 },
	{ 0x955f, 0x955f, PDF_CMAP_SINGLE, 7786 },
	{ 0x9560, 0x956c, PDF_CMAP_RANGE, 3507 },
	{ 0x956d, 0x956d, PDF_CMAP_SINGLE, 7787 },
	{ 0x956e, 0x956f, PDF_CMAP_RANGE, 3521 },
	{ 0x9570, 0x9570, PDF_CMAP_SINGLE, 7788 },
	{ 0x9571, 0x957e, PDF_CMAP_RANGE, 3524 },
	{ 0x9580, 0x95c0, PDF_CMAP_RANGE, 3538 },
	{ 0x95c1, 0x95c1, PDF_CMAP_SINGLE, 7789 },
	{ 0x95c2, 0x95ca, PDF_CMAP_RANGE, 3604 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 7790 },
	{ 0x95cc, 0x95d7, PDF_CMAP_RANGE, 3614 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 7791 },
	{ 0x95d9, 0x95f6, PDF_CMAP_RANGE, 3627 },
	{ 0x95f7, 0x95f7, PDF_CMAP_SINGLE, 7792 },
	{ 0x95f8, 0x95fc, PDF_CMAP_RANGE, 3658 },
	{ 0x9640, 0x9641, PDF_CMAP_TABLE, 86 },
	{ 0x9642, 0x9647, PDF_CMAP_RANGE, 3665 },
	{ 0x9648, 0x9648, PDF_CMAP_SINGLE, 7794 },
	{ 0x9649, 0x9669, PDF_CMAP_RANGE, 3672 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 7795 },
	{ 0x966b, 0x967e, PDF_CMAP_RANGE, 3706 },
	{ 0x9680, 0x9689, PDF_CMAP_RANGE, 3726 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 7475 },
	{ 0x968b, 0x968f, PDF_CMAP_RANGE, 3737 },
	{ 0x9690, 0x9690, PDF_CMAP_SINGLE, 7796 },
	{ 0x9691, 0x9697, PDF_CMAP_RANGE, 3743 },
	{ 0x9698, 0x9699, PDF_CMAP_TABLE, 88 },
	{ 0x969a, 0x96ca, PDF_CMAP_RANGE, 3752 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 7797 },
	{ 0x96cc, 0x96d6, PDF_CMAP_RANGE, 3802 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 7798 },
	{ 0x96d8, 0x96dc, PDF_CMAP_RANGE, 3814 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 7799 },
	{ 0x96de, 0x96df, PDF_CMAP_RANGE, 3820 },
	{ 0x96e0, 0x96e0, PDF_CMAP_SINGLE, 7800 },
	{ 0x96e1, 0x96f6, PDF_CMAP_RANGE, 3823 },
	{ 0x96f7, 0x96fc, PDF_CMAP_TABLE, 90 },
	{ 0x9740, 0x9750, PDF_CMAP_RANGE, 3851 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 7804 },
	{ 0x9752, 0x976e, PDF_CMAP_RANGE, 3869 },
	{ 0x976f, 0x976f, PDF_CMAP_SINGLE, 7805 },
	{ 0x9770, 0x9772, PDF_CMAP_RANGE, 3899 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 7806 },
	{ 0x9774, 0x9778, PDF_CMAP_RANGE, 3903 },
	{ 0x9779, 0x9779, PDF_CMAP_SINGLE, 7476 },
	{ 0x977a, 0x977e, PDF_CMAP_RANGE, 3909 },
	{ 0x9780, 0x9788, PDF_CMAP_RANGE, 3914 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 7807 },
	{ 0x978a, 0x97c8, PDF_CMAP_RANGE, 3924 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 7808 },
	{ 0x97ca, 0x97f7, PDF_CMAP_RANGE, 3988 },
	{ 0x97f8, 0x97f9, PDF_CMAP_RANGE, 7809 },
	{ 0x97fa, 0x97fc, PDF_CMAP_RANGE, 4036 },
	{ 0x9840, 0x9840, PDF_CMAP_SINGLE, 7811 },
	{ 0x9841, 0x984f, PDF_CMAP_RANGE, 4040 },
	{ 0x9850, 0x9850, PDF_CMAP_SINGLE, 7812 },
	{ 0x9851, 0x9854, PDF_CMAP_RANGE, 4056 },
	{ 0x9855, 0x9855, PDF_CMAP_SINGLE, 6007 },
	{ 0x9856, 0x9857, PDF_CMAP_RANGE, 4061 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 7813 },
	{ 0x9859, 0x9872, PDF_CMAP_RANGE, 4064 },
	{ 0x989f, 0x98d3, PDF_CMAP_RANGE, 4090 },
	{ 0x98d4, 0x98d4, PDF_CMAP_SINGLE, 3751 },
	{ 0x98d5, 0x98fc, PDF_CMAP_RANGE, 4144 },
	{ 0x9940, 0x995b, PDF_CMAP_RANGE, 4184 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 7814 },
	{ 0x995d, 0x9965, PDF_CMAP_RANGE, 4213 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 7815 },
	{ 0x9967, 0x9969, PDF_CMAP_RANGE, 4223 },
	{ 0x996a, 0x996c, PDF_CMAP_TABLE, 96 },
	{ 0x996d, 0x997e, PDF_CMAP_RANGE, 4229 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a4e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 7818 },
	{ 0x9a50, 0x9a58, PDF_CMAP_RANGE, 4388 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 7819 },
	{ 0x9a5a, 0x9a7c, PDF_CMAP_RANGE, 4398 },
	{ 0x9a7d, 0x9a7e, PDF_CMAP_TABLE, 99 },
	{ 0x9a80, 0x9a8a, PDF_CMAP_RANGE, 4435 },
	{ 0x9a8b, 0x9a8b, PDF_CMAP_SINGLE, 7822 },
	{ 0x9a8c, 0x9ac1, PDF_CMAP_RANGE, 4447 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 7823 },
	{ 0x9ac3, 0x9ae1, PDF_CMAP_RANGE, 4502 },
	{ 0x9ae2, 0x9ae2, PDF_CMAP_SINGLE, 3063 },
	{ 0x9ae3, 0x9afc, PDF_CMAP_RANGE, 4534 },
	{ 0x9b40, 0x9b5b, PDF_CMAP_RANGE, 4560 },
	{ 0x9b5c, 0x9b5c, PDF_CMAP_SINGLE, 7824 },
	{ 0x9b5d, 0x9b7e, PDF_CMAP_RANGE, 4589 },
	{ 0x9b80, 0x9b82, PDF_CMAP_RANGE, 4623 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7825 },
	{ 0x9b84, 0x9b9f, PDF_CMAP_RANGE, 4627 },
	{ 0x9ba0, 0x9ba0, PDF_CMAP_SINGLE, 7826 },
	{ 0x9ba1, 0x9bfc, PDF_CMAP_RANGE, 4656 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9ca1, PDF_CMAP_RANGE, 4811 },
	{ 0x9ca2, 0x9ca2, PDF_CMAP_SINGLE, 7828 },
	{ 0x9ca3, 0x9cfc, PDF_CMAP_RANGE, 4846 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9d80, PDF_CMAP_SINGLE, 7829 },
	{ 0x9d81, 0x9d8b, PDF_CMAP_RANGE, 5000 },
	{ 0x9d8c, 0x9d8c, PDF_CMAP_SINGLE, 7830 },
	{ 0x9d8d, 0x9d97, PDF_CMAP_RANGE, 5012 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 1447 },
	{ 0x9d99, 0x9db6, PDF_CMAP_RANGE, 5024 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 7831 },
	{ 0x9db8, 0x9dca, PDF_CMAP_RANGE, 5055 },
	{ 0x9dcb, 0x9dcb, PDF_CMAP_SINGLE, 7832 },
	{ 0x9dcc, 0x9dfc, PDF_CMAP_RANGE, 5075 },
	{ 0x9e40, 0x9e63, PDF_CMAP_RANGE, 5124 },
	{ 0x9e64, 0x9e64, PDF_CMAP_SINGLE, 7833 },
	{ 0x9e65, 0x9e68, PDF_CMAP_RANGE, 5161 },
	{ 0x9e69, 0x9e69, PDF_CMAP_SINGLE, 7834 },
	{ 0x9e6a, 0x9e76, PDF_CMAP_RANGE, 5166 },
	{ 0x9e77, 0x9e77, PDF_CMAP_SINGLE, 3490 },
	{ 0x9e78, 0x9e7e, PDF_CMAP_RANGE, 5180 },
	{ 0x9e80, 0x9e8a, PDF_CMAP_RANGE, 5187 },
	{ 0x9e8b, 0x9e8d, PDF_CMAP_TABLE, 101 },
	{ 0x9e8e, 0x9e93, PDF_CMAP_RANGE, 5201 },
	{ 0x9e94, 0x9e94, PDF_CMAP_SINGLE, 7836 },
	{ 0x9e95, 0x9efc, PDF_CMAP_RANGE, 5208 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9fb6, PDF_CMAP_RANGE, 5375 },
	{ 0x9fb7, 0x9fb7, PDF_CMAP_SINGLE, 3181 },
	{ 0x9fb8, 0x9fcd, PDF_CMAP_RANGE, 5431 },
	{ 0x9fce, 0x9fce, PDF_CMAP_SINGLE, 7837 },
	{ 0x9fcf, 0x9ff2, PDF_CMAP_RANGE, 5454 },
	{ 0x9ff3, 0x9ff3, PDF_CMAP_SINGLE, 1535 },
	{ 0x9ff4, 0x9ffc, PDF_CMAP_RANGE, 5491 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0a3, PDF_CMAP_RANGE, 5563 },
	{ 0xe0a4, 0xe0a4, PDF_CMAP_SINGLE, 7839 },
	{ 0xe0a5, 0xe0dc, PDF_CMAP_RANGE, 5600 },
	{ 0xe0dd, 0xe0dd, PDF_CMAP_SINGLE, 7840 },
	{ 0xe0de, 0xe0f3, PDF_CMAP_RANGE, 5657 },
	{ 0xe0f4, 0xe0f4, PDF_CMAP_SINGLE, 7477 },
	{ 0xe0f5, 0xe0fc, PDF_CMAP_RANGE, 5680 },
	{ 0xe140, 0xe149, PDF_CMAP_RANGE, 5688 },
	{ 0xe14a, 0xe14a, PDF_CMAP_SINGLE, 7841 },
	{ 0xe14b, 0xe14e, PDF_CMAP_RANGE, 5699 },
	{ 0xe14f, 0xe150, PDF_CMAP_RANGE, 7842 },
	{ 0xe151, 0xe17e, PDF_CMAP_RANGE, 5705 },
	{ 0xe180, 0xe1a8, PDF_CMAP_RANGE, 5751 },
	{ 0xe1a9, 0xe1a9, PDF_CMAP_SINGLE, 7844 },
	{ 0xe1aa, 0xe1e5, PDF_CMAP_RANGE, 5793 },
	{ 0xe1e6, 0xe1e8, PDF_CMAP_TABLE, 104 },
	{ 0xe1e9, 0xe1ec, PDF_CMAP_RANGE, 5856 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 7845 },
	{ 0xe1ee, 0xe1fc, PDF_CMAP_RANGE, 5861 },
	{ 0xe240, 0xe268, PDF_CMAP_RANGE, 5876 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 7846 },
	{ 0xe26a, 0xe272, PDF_CMAP_RANGE, 5918 },
	{ 0xe273, 0xe273, PDF_CMAP_SINGLE, 7847 },
	{ 0xe274, 0xe27c, PDF_CMAP_RANGE, 5928 },
	{ 0xe27d, 0xe27e, PDF_CMAP_TABLE, 107 },
	{ 0xe280, 0xe2b6, PDF_CMAP_RANGE, 5939 },
	{ 0xe2b7, 0xe2b7, PDF_CMAP_SINGLE, 7848 },
	{ 0xe2b8, 0xe2c3, PDF_CMAP_RANGE, 5995 },
	{ 0xe2c4, 0xe2c4, PDF_CMAP_SINGLE, 4060 },
	{ 0xe2c5, 0xe2e1, PDF_CMAP_RANGE, 6008 },
	{ 0xe2e2, 0xe2e2, PDF_CMAP_SINGLE, 7849 },
	{ 0xe2e3, 0xe2eb, PDF_CMAP_RANGE, 6038 },
	{ 0xe2ec, 0xe2ec, PDF_CMAP_SINGLE, 7850 },
	{ 0xe2ed, 0xe2fc, PDF_CMAP_RANGE, 6048 },
	{ 0xe340, 0xe357, PDF_CMAP_RANGE, 6064 },
	{ 0xe358, 0xe35a, PDF_CMAP_TABLE, 109 },
	{ 0xe35b, 0xe364, PDF_CMAP_RANGE, 6091 },
	{ 0xe365, 0xe365, PDF_CMAP_SINGLE, 7853 },
	{ 0xe366, 0xe37e, PDF_CMAP_RANGE, 6102 },
	{ 0xe380, 0xe3c3, PDF_CMAP_RANGE, 6127 },
	{ 0xe3c4, 0xe3c4, PDF_CMAP_SINGLE, 7854 },
	{ 0xe3c5, 0xe3fc, PDF_CMAP_RANGE, 6196 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe483, PDF_CMAP_RANGE, 6315 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 7855 },
	{ 0xe485, 0xe488, PDF_CMAP_RANGE, 6320 },
	{ 0xe489, 0xe489, PDF_CMAP_SINGLE, 7856 },
	{ 0xe48a, 0xe491, PDF_CMAP_RANGE, 6325 },
	{ 0xe492, 0xe492, PDF_CMAP_SINGLE, 7857 },
	{ 0xe493, 0xe4b1, PDF_CMAP_RANGE, 6334 },
	{ 0xe4b2, 0xe4b2, PDF_CMAP_SINGLE, 7858 },
	{ 0xe4b3, 0xe4b8, PDF_CMAP_RANGE, 6366 },
	{ 0xe4b9, 0xe4b9, PDF_CMAP_SINGLE, 7859 },
	{ 0xe4ba, 0xe4f1, PDF_CMAP_RANGE, 6373 },
	{ 0xe4f2, 0xe4f2, PDF_CMAP_SINGLE, 7860 },
	{ 0xe4f3, 0xe4fc, PDF_CMAP_RANGE, 6430 },
	{ 0xe540, 0xe541, PDF_CMAP_TABLE, 112 },
	{ 0xe542, 0xe54c, PDF_CMAP_RANGE, 6442 },
	{ 0xe54d, 0xe54d, PDF_CMAP_SINGLE, 3845 },
	{ 0xe54e, 0xe578, PDF_CMAP_RANGE, 6454 },
	{ 0xe579, 0xe579, PDF_CMAP_SINGLE, 1440 },
	{ 0xe57a, 0xe57e, PDF_CMAP_RANGE, 6498 },
	{ 0xe580, 0xe5a1, PDF_CMAP_RANGE, 6503 },
	{ 0xe5a2, 0xe5a2, PDF_CMAP_SINGLE, 3358 },
	{ 0xe5a3, 0xe5a4, PDF_CMAP_RANGE, 6538 },
	{ 0xe5a5, 0xe5a5, PDF_CMAP_SINGLE, 7862 },
	{ 0xe5a6, 0xe5ba, PDF_CMAP_RANGE, 6541 },
	{ 0xe5bb, 0xe5bb, PDF_CMAP_SINGLE, 7863 },
	{ 0xe5bc, 0xe5ec, PDF_CMAP_RANGE, 6563 },
	{ 0xe5ed, 0xe5ed, PDF_CMAP_SINGLE, 7864 },
	{ 0xe5ee, 0xe5fc, PDF_CMAP_RANGE, 6613 },
	{ 0xe640, 0xe650, PDF_CMAP_RANGE, 6628 },
	{ 0xe651, 0xe651, PDF_CMAP_SINGLE, 7865 },
	{ 0xe652, 0xe67b, PDF_CMAP_RANGE, 6646 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 1550 },
	{ 0xe67d, 0xe67e, PDF_CMAP_RANGE, 6689 },
	{ 0xe680, 0xe685, PDF_CMAP_RANGE, 6691 },
	{ 0xe686, 0xe686, PDF_CMAP_SINGLE, 7866 },
	{ 0xe687, 0xe695, PDF_CMAP_RANGE, 6698 },
	{ 0xe696, 0xe696, PDF_CMAP_SINGLE, 7867 },
	{ 0xe697, 0xe6ca, PDF_CMAP_RANGE, 6714 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 2730 },
	{ 0xe6cc, 0xe6e6, PDF_CMAP_RANGE, 6767 },
	{ 0xe6e7, 0xe6e7, PDF_CMAP_SINGLE, 7868 },
	{ 0xe6e8, 0xe6f1, PDF_CMAP_RANGE, 6795 },
	{ 0xe6f2, 0xe6f2, PDF_CMAP_SINGLE, 7869 },
	{ 0xe6f3, 0xe6fc, PDF_CMAP_RANGE, 6806 },
	{ 0xe740, 0xe76c, PDF_CMAP_RANGE, 6816 },
	{ 0xe76d, 0xe76d, PDF_CMAP_SINGLE, 7870 },
	{ 0xe76e, 0xe77e, PDF_CMAP_RANGE, 6862 },
	{ 0xe780, 0xe78b, PDF_CMAP_RANGE, 6879 },
	{ 0xe78c, 0xe78e, PDF_CMAP_TABLE, 114 },
	{ 0xe78f, 0xe7a6, PDF_CMAP_RANGE, 6894 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7d4, PDF_CMAP_RANGE, 6939 },
	{ 0xe7d5, 0xe7d5, PDF_CMAP_SINGLE, 7875 },
	{ 0xe7d6, 0xe7fc, PDF_CMAP_RANGE, 6965 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8c2, PDF_CMAP_RANGE, 7067 },
	{ 0xe8c3, 0xe8c3, PDF_CMAP_SINGLE, 7878 },
	{ 0xe8c4, 0xe8ce, PDF_CMAP_RANGE, 7135 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8d4, PDF_CMAP_RANGE, 7147 },
	{ 0xe8d5, 0xe8d5, PDF_CMAP_SINGLE, 7880 },
	{ 0xe8d6, 0xe8f1, PDF_CMAP_RANGE, 7153 },
	{ 0xe8f2, 0xe8f3, PDF_CMAP_TABLE, 117 },
	{ 0xe8f4, 0xe8fc, PDF_CMAP_RANGE, 7183 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9ca, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cb, 0xe9cc, PDF_CMAP_TABLE, 119 },
	{ 0xe9cd, 0xe9f1, PDF_CMAP_RANGE, 7332 },
	{ 0xe9f2, 0xe9f2, PDF_CMAP_SINGLE, 1321 },
	{ 0xe9f3, 0xe9fc, PDF_CMAP_RANGE, 7370 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea7e, PDF_CMAP_RANGE, 7429 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9d, PDF_CMAP_SINGLE, 7886 },
	{ 0xea9e, 0xeaa2, PDF_CMAP_RANGE, 7473 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xfa40, 0xfa49, PDF_CMAP_RANGE, 8092 },
	{ 0xfa4a, 0xfa53, PDF_CMAP_RANGE, 7575 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 751 },
	{ 0xfa55, 0xfa57, PDF_CMAP_RANGE, 8005 },
	{ 0xfa58, 0xfa5b, PDF_CMAP_TABLE, 121 },
	{ 0xfa5c, 0xfa7e, PDF_CMAP_RANGE, 8359 },
	{ 0xfa80, 0xfacf, PDF_CMAP_RANGE, 8394 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 1993 },
	{ 0xfad1, 0xfafc, PDF_CMAP_RANGE, 8474 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 8518 },
	{ 0xfb80, 0xfbfc, PDF_CMAP_RANGE, 8581 },
	{ 0xfc40, 0xfc4b, PDF_CMAP_RANGE, 8706 },
	{ 0xe5a3, 0xe5a4, PDF_CMAP_RANGE, 6538 },
	{ 0xe5a5, 0xe5a5, PDF_CMAP_SINGLE, 7862 },
	{ 0xe5a6, 0xe5ba, PDF_CMAP_RANGE, 6541 },
	{ 0xe5bb, 0xe5bb, PDF_CMAP_SINGLE, 7863 },
	{ 0xe5bc, 0xe5ec, PDF_CMAP_RANGE, 6563 },
	{ 0xe5ed, 0xe5ed, PDF_CMAP_SINGLE, 7864 },
	{ 0xe5ee, 0xe5fc, PDF_CMAP_RANGE, 6613 },
	{ 0xe640, 0xe650, PDF_CMAP_RANGE, 6628 },
	{ 0xe651, 0xe651, PDF_CMAP_SINGLE, 7865 },
	{ 0xe652, 0xe67b, PDF_CMAP_RANGE, 6646 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 1550 },
	{ 0xe67d, 0xe67e, PDF_CMAP_RANGE, 6689 },
	{ 0xe680, 0xe685, PDF_CMAP_RANGE, 6691 },
	{ 0xe686, 0xe686, PDF_CMAP_SINGLE, 7866 },
	{ 0xe687, 0xe695, PDF_CMAP_RANGE, 6698 },
	{ 0xe696, 0xe696, PDF_CMAP_SINGLE, 7867 },
	{ 0xe697, 0xe6ca, PDF_CMAP_RANGE, 6714 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 2730 },
	{ 0xe6cc, 0xe6e6, PDF_CMAP_RANGE, 6767 },
	{ 0xe6e7, 0xe6e7, PDF_CMAP_SINGLE, 7868 },
	{ 0xe6e8, 0xe6f1, PDF_CMAP_RANGE, 6795 },
	{ 0xe6f2, 0xe6f2, PDF_CMAP_SINGLE, 7869 },
	{ 0xe6f3, 0xe6fc, PDF_CMAP_RANGE, 6806 },
	{ 0xe740, 0xe76c, PDF_CMAP_RANGE, 6816 },
	{ 0xe76d, 0xe76d, PDF_CMAP_SINGLE, 7870 },
	{ 0xe76e, 0xe77e, PDF_CMAP_RANGE, 6862 },
	{ 0xe780, 0xe78b, PDF_CMAP_RANGE, 6879 },
	{ 0xe78c, 0xe78c, PDF_CMAP_SINGLE, 7871 },
	{ 0xe78d, 0xe78d, PDF_CMAP_SINGLE, 6892 },
	{ 0xe78e, 0xe78e, PDF_CMAP_SINGLE, 7872 },
	{ 0xe78f, 0xe7a6, PDF_CMAP_RANGE, 6894 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7d4, PDF_CMAP_RANGE, 6939 },
	{ 0xe7d5, 0xe7d5, PDF_CMAP_SINGLE, 7875 },
	{ 0xe7d6, 0xe7fc, PDF_CMAP_RANGE, 6965 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8c2, PDF_CMAP_RANGE, 7067 },
	{ 0xe8c3, 0xe8c3, PDF_CMAP_SINGLE, 7878 },
	{ 0xe8c4, 0xe8ce, PDF_CMAP_RANGE, 7135 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8d4, PDF_CMAP_RANGE, 7147 },
	{ 0xe8d5, 0xe8d5, PDF_CMAP_SINGLE, 7880 },
	{ 0xe8d6, 0xe8f1, PDF_CMAP_RANGE, 7153 },
	{ 0xe8f2, 0xe8f2, PDF_CMAP_SINGLE, 1841 },
	{ 0xe8f3, 0xe8f3, PDF_CMAP_SINGLE, 7881 },
	{ 0xe8f4, 0xe8fc, PDF_CMAP_RANGE, 7183 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9ca, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cb, 0xe9cb, PDF_CMAP_SINGLE, 1143 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 7884 },
	{ 0xe9cd, 0xe9f1, PDF_CMAP_RANGE, 7332 },
	{ 0xe9f2, 0xe9f2, PDF_CMAP_SINGLE, 1321 },
	{ 0xe9f3, 0xe9fc, PDF_CMAP_RANGE, 7370 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea7e, PDF_CMAP_RANGE, 7429 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9d, PDF_CMAP_SINGLE, 7886 },
	{ 0xea9e, 0xeaa2, PDF_CMAP_RANGE, 7473 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xfa40, 0xfa49, PDF_CMAP_RANGE, 8092 },
	{ 0xfa4a, 0xfa53, PDF_CMAP_RANGE, 7575 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 751 },
	{ 0xfa55, 0xfa57, PDF_CMAP_RANGE, 8005 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 7618 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 7610 },
	{ 0xfa5a, 0xfa5a, PDF_CMAP_SINGLE, 8055 },
	{ 0xfa5b, 0xfa5b, PDF_CMAP_SINGLE, 768 },
	{ 0xfa5c, 0xfa7e, PDF_CMAP_RANGE, 8359 },
	{ 0xfa80, 0xfacf, PDF_CMAP_RANGE, 8394 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 1993 },
	{ 0xfad1, 0xfafc, PDF_CMAP_RANGE, 8474 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 8518 },
	{ 0xfb80, 0xfbfc, PDF_CMAP_RANGE, 8581 },
	{ 0xfc40, 0xfc4b, PDF_CMAP_RANGE, 8706 },
};

static const unsigned short pdf_cmap_78ms_RKSJ_H_table[125] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
	8038,7588,8040,7590,8042,762,761,769,
	1125,7633,7961,7330,7645,7369,7646,7649,
	1406,1407,7650,7655,5490,7665,1780,7666,
	1782,7689,2169,7690,7693,6441,7720,6766,
	7733,2910,7734,2918,7744,3045,7745,7746,
	3050,7747,3099,7748,7772,6537,7773,7776,
	3395,7777,7783,5179,3491,7784,3663,7793,
	7980,4143,6453,7801,3847,7802,3849,7803,
	7816,4227,7817,7821,4434,7835,5199,3176,
	2030,5854,3153,1492,5938,7851,6089,7852,
	6440,2293,7871,6892,7872,1841,7881,1143,
	7884,7618,7610,8055,768,
};

pdf_cmap pdf_cmap_78ms_RKSJ_H =
{
	-1, "78ms-RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	713, 713, (pdf_range*) pdf_cmap_78ms_RKSJ_H_ranges,
	125, 125, (unsigned short*) pdf_cmap_78ms_RKSJ_H_table,
};

/* 78ms-RKSJ-V */

static const pdf_range pdf_cmap_78ms_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8143, 0x8144, PDF_CMAP_TABLE, 0 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 7917 },
	{ 0x81a8, 0x81a9, PDF_CMAP_TABLE, 2 },
	{ 0x81aa, 0x81ab, PDF_CMAP_RANGE, 736 },
	{ 0x81ac, 0x81ac, PDF_CMAP_SINGLE, 8270 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 4 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7940 },
	{ 0x8761, 0x8765, PDF_CMAP_TABLE, 36 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7947 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8344 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7950 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8348 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7953 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8349 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
	{ 0x84a7, 0x84a7, PDF_CMAP_SINGLE, 7531 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 7507 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 7539 },
	{ 0x84aa, 0x84aa, PDF_CMAP_SINGLE, 7482 },
	{ 0x84ab, 0x84ab, PDF_CMAP_SINGLE, 7480 },
	{ 0x84ac, 0x84ac, PDF_CMAP_SINGLE, 7498 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 7506 },
	{ 0x84ae, 0x84ae, PDF_CMAP_SINGLE, 7502 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 7494 },
	{ 0x84b0, 0x84b0, PDF_CMAP_SINGLE, 7530 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 7522 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 7538 },
	{ 0x84b3, 0x84b3, PDF_CMAP_SINGLE, 7514 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 7554 },
	{ 0x84b5, 0x84b5, PDF_CMAP_SINGLE, 7526 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 7519 },
	{ 0x84b7, 0x84b7, PDF_CMAP_SINGLE, 7534 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 7511 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 7545 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 7527 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 7516 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 7535 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 7508 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 7542 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7940 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 8329 },
	{ 0x8762, 0x8762, PDF_CMAP_SINGLE, 7943 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 8339 },
	{ 0x8764, 0x8764, PDF_CMAP_SINGLE, 7945 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 8338 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7947 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8344 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7950 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8348 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7953 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8349 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
};

static const unsigned short pdf_cmap_78ms_RKSJ_V_table[41] =
{
	8268,8274,739,738,7481,7479,7495,7503,
	7499,7491,7523,7515,7531,7507,7539,7482,
	7480,7498,7506,7502,7494,7530,7522,7538,
	7514,7554,7526,7519,7534,7511,7545,7527,
	7516,7535,7508,7542,8329,7943,8339,7945,
	8338,
};

pdf_cmap pdf_cmap_78ms_RKSJ_V =
{
	-1, "78ms-RKSJ-V", "78ms-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	78, 78, (pdf_range*) pdf_cmap_78ms_RKSJ_V_ranges,
	41, 41, (unsigned short*) pdf_cmap_78ms_RKSJ_V_table,
};

/* 83pv-RKSJ-H */

static const pdf_range pdf_cmap_83pv_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x0080, 0x0080, PDF_CMAP_SINGLE, 97 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x00fd, 0x00ff, PDF_CMAP_TABLE, 0 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 3 },
	{ 0x8540, 0x857e, PDF_CMAP_RANGE, 232 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 390 },
	{ 0x8581, 0x859e, PDF_CMAP_RANGE, 296 },
	{ 0x859f, 0x85dd, PDF_CMAP_RANGE, 327 },
	{ 0x85de, 0x85fc, PDF_CMAP_RANGE, 391 },
	{ 0x8640, 0x867e, PDF_CMAP_RANGE, 422 },
	{ 0x8680, 0x8691, PDF_CMAP_RANGE, 485 },
	{ 0x8692, 0x8692, PDF_CMAP_SINGLE, 295 },
	{ 0x8693, 0x869e, PDF_CMAP_RANGE, 503 },
	{ 0x86a2, 0x86ed, PDF_CMAP_RANGE, 7479 },
	{ 0x8740, 0x875d, PDF_CMAP_RANGE, 7555 },
	{ 0x875f, 0x8775, PDF_CMAP_RANGE, 7585 },
	{ 0x8780, 0x878f, PDF_CMAP_RANGE, 7608 },
	{ 0x8790, 0x8792, PDF_CMAP_TABLE, 35 },
	{ 0x8793, 0x8799, PDF_CMAP_RANGE, 7624 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 768 },
	{ 0x879b, 0x879c, PDF_CMAP_RANGE, 7631 },
	{ 0x889f, 0x88fc, PDF_CMAP_RANGE, 1125 },
	{ 0x8940, 0x897e, PDF_CMAP_RANGE, 1219 },
	{ 0x8980, 0x89fc, PDF_CMAP_RANGE, 1282 },
	{ 0x8a40, 0x8a7e, PDF_CMAP_RANGE, 1407 },
	{ 0x8a80, 0x8afc, PDF_CMAP_RANGE, 1470 },
	{ 0x8b40, 0x8b7e, PDF_CMAP_RANGE, 1595 },
	{ 0x8b80, 0x8bfc, PDF_CMAP_RANGE, 1658 },
	{ 0x8c40, 0x8c7e, PDF_CMAP_RANGE, 1783 },
	{ 0x8c80, 0x8cfc, PDF_CMAP_RANGE, 1846 },
	{ 0x8d40, 0x8d7e, PDF_CMAP_RANGE, 1971 },
	{ 0x8d80, 0x8dfc, PDF_CMAP_RANGE, 2034 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_RANGE, 2159 },
	{ 0x8e80, 0x8efc, PDF_CMAP_RANGE, 2222 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_RANGE, 2347 },
	{ 0x8f80, 0x8ffc, PDF_CMAP_RANGE, 2410 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 2535 },
	{ 0x9080, 0x90fc, PDF_CMAP_RANGE, 2598 },
	{ 0x9140, 0x917e, PDF_CMAP_RANGE, 2723 },
	{ 0x9180, 0x91fc, PDF_CMAP_RANGE, 2786 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 2911 },
	{ 0x9280, 0x92fc, PDF_CMAP_RANGE, 2974 },
	{ 0x9340, 0x937e, PDF_CMAP_RANGE, 3099 },
	{ 0x9380, 0x93fc, PDF_CMAP_RANGE, 3162 },
	{ 0x9440, 0x947e, PDF_CMAP_RANGE, 3287 },
	{ 0x9480, 0x94fc, PDF_CMAP_RANGE, 3350 },
	{ 0x9540, 0x957e, PDF_CMAP_RANGE, 3475 },
	{ 0x9580, 0x95fc, PDF_CMAP_RANGE, 3538 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 3663 },
	{ 0x9680, 0x96fc, PDF_CMAP_RANGE, 3726 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 3851 },
	{ 0x9780, 0x97fc, PDF_CMAP_RANGE, 3914 },
	{ 0x9840, 0x9872, PDF_CMAP_RANGE, 4039 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x997e, PDF_CMAP_RANGE, 4184 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a7e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a80, 0x9afc, PDF_CMAP_RANGE, 4435 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 4560 },
	{ 0x9b80, 0x9bfc, PDF_CMAP_RANGE, 4623 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xeb40, 0xeb40, PDF_CMAP_SINGLE, 633 },
	{ 0xeb41, 0xeb42, PDF_CMAP_RANGE, 7887 },
	{ 0xeb43, 0xeb4f, PDF_CMAP_RANGE, 636 },
	{ 0xeb50, 0xeb51, PDF_CMAP_RANGE, 7889 },
	{ 0xeb52, 0xeb5a, PDF_CMAP_RANGE, 651 },
	{ 0xeb5b, 0xeb5d, PDF_CMAP_RANGE, 7891 },
	{ 0xeb5e, 0xeb5f, PDF_CMAP_RANGE, 663 },
	{ 0xeb60, 0xeb64, PDF_CMAP_RANGE, 7894 },
	{ 0xeb65, 0xeb68, PDF_CMAP_RANGE, 670 },
	{ 0xeb69, 0xeb7a, PDF_CMAP_RANGE, 7899 },
	{ 0xeb7b, 0xeb7e, PDF_CMAP_RANGE, 692 },
	{ 0xeb80, 0xeb81, PDF_CMAP_TABLE, 38 },
	{ 0xeb82, 0xebac, PDF_CMAP_RANGE, 698 },
	{ 0xebb8, 0xebbf, PDF_CMAP_RANGE, 741 },
	{ 0xebc8, 0xebce, PDF_CMAP_RANGE, 749 },
	{ 0xebda, 0xebe8, PDF_CMAP_RANGE, 756 },
	{ 0xebf0, 0xebf7, PDF_CMAP_RANGE, 771 },
	{ 0xebfc, 0xebfc, PDF_CMAP_SINGLE, 779 },
	{ 0xec4f, 0xec58, PDF_CMAP_RANGE, 780 },
	{ 0xec60, 0xec79, PDF_CMAP_RANGE, 790 },
	{ 0xec81, 0xec9a, PDF_CMAP_RANGE, 816 },
	{ 0xec9f, 0xeca7, PDF_CMAP_TABLE, 40 },
	{ 0xeca8, 0xecc0, PDF_CMAP_RANGE, 851 },
	{ 0xecc1, 0xecc1, PDF_CMAP_SINGLE, 7923 },
	{ 0xecc2, 0xece0, PDF_CMAP_RANGE, 877 },
	{ 0xece1, 0xece5, PDF_CMAP_TABLE, 49 },
	{ 0xece6, 0xeceb, PDF_CMAP_RANGE, 913 },
	{ 0xecec, 0xecec, PDF_CMAP_SINGLE, 7927 },
	{ 0xeced, 0xecf1, PDF_CMAP_RANGE, 920 },
	{ 0xed40, 0xed48, PDF_CMAP_TABLE, 54 },
	{ 0xed49, 0xed61, PDF_CMAP_RANGE, 934 },
	{ 0xed62, 0xed62, PDF_CMAP_SINGLE, 7933 },
	{ 0xed63, 0xed7e, PDF_CMAP_RANGE, 960 },
	{ 0xed80, 0xed82, PDF_CMAP_RANGE, 988 },
	{ 0xed83, 0xed87, PDF_CMAP_TABLE, 63 },
	{ 0xed88, 0xed8d, PDF_CMAP_RANGE, 996 },
	{ 0xed8e, 0xed8e, PDF_CMAP_SINGLE, 7937 },
	{ 0xed8f, 0xed94, PDF_CMAP_RANGE, 1003 },
	{ 0xed95, 0xed96, PDF_CMAP_RANGE, 7938 },
	{ 0xed9f, 0xedb6, PDF_CMAP_RANGE, 1011 },
	{ 0xedbf, 0xedd6, PDF_CMAP_RANGE, 1035 },
	{ 0xee40, 0xee5d, PDF_CMAP_RANGE, 7555 },
	{ 0xee5f, 0xee6e, PDF_CMAP_RANGE, 7940 },
	{ 0xee6f, 0xee75, PDF_CMAP_RANGE, 7601 },
	{ 0xee80, 0xee81, PDF_CMAP_RANGE, 7956 },
	{ 0xee82, 0xee8f, PDF_CMAP_RANGE, 7610 },
	{ 0xee90, 0xee92, PDF_CMAP_TABLE, 68 },
	{ 0xee93, 0xee99, PDF_CMAP_RANGE, 7624 },
	{ 0xee9a, 0xee9a, PDF_CMAP_SINGLE, 768 },
	{ 0xee9b, 0xee9c, PDF_CMAP_RANGE, 7631 },
	{ 0xebc8, 0xebce, PDF_CMAP_RANGE, 749 },
	{ 0xebda, 0xebe8, PDF_CMAP_RANGE, 756 },
	{ 0xebf0, 0xebf7, PDF_CMAP_RANGE, 771 },
	{ 0xebfc, 0xebfc, PDF_CMAP_SINGLE, 779 },
	{ 0xec4f, 0xec58, PDF_CMAP_RANGE, 780 },
	{ 0xec60, 0xec79, PDF_CMAP_RANGE, 790 },
	{ 0xec81, 0xec9a, PDF_CMAP_RANGE, 816 },
	{ 0xec9f, 0xec9f, PDF_CMAP_SINGLE, 7918 },
	{ 0xeca0, 0xeca0, PDF_CMAP_SINGLE, 843 },
	{ 0xeca1, 0xeca1, PDF_CMAP_SINGLE, 7919 },
	{ 0xeca2, 0xeca2, PDF_CMAP_SINGLE, 845 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 7920 },
	{ 0xeca4, 0xeca4, PDF_CMAP_SINGLE, 847 },
	{ 0xeca5, 0xeca5, PDF_CMAP_SINGLE, 7921 },
	{ 0xeca6, 0xeca6, PDF_CMAP_SINGLE, 849 },
	{ 0xeca7, 0xeca7, PDF_CMAP_SINGLE, 7922 },
	{ 0xeca8, 0xecc0, PDF_CMAP_RANGE, 851 },
	{ 0xecc1, 0xecc1, PDF_CMAP_SINGLE, 7923 },
	{ 0xecc2, 0xece0, PDF_CMAP_RANGE, 877 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 7924 },
	{ 0xece2, 0xece2, PDF_CMAP_SINGLE, 909 },
	{ 0xece3, 0xece3, PDF_CMAP_SINGLE, 7925 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 911 },
	{ 0xece5, 0xece5, PDF_CMAP_SINGLE, 7926 },
	{ 0xece6, 0xeceb, PDF_CMAP_RANGE, 913 },
	{ 0xecec, 0xecec, PDF_CMAP_SINGLE, 7927 },
	{ 0xeced, 0xecf1, PDF_CMAP_RANGE, 920 },
	{ 0xed40, 0xed40, PDF_CMAP_SINGLE, 7928 },
	{ 0xed41, 0xed41, PDF_CMAP_SINGLE, 926 },
	{ 0xed42, 0xed42, PDF_CMAP_SINGLE, 7929 },
	{ 0xed43, 0xed43, PDF_CMAP_SINGLE, 928 },
	{ 0xed44, 0xed44, PDF_CMAP_SINGLE, 7930 },
	{ 0xed45, 0xed45, PDF_CMAP_SINGLE, 930 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 7931 },
	{ 0xed47, 0xed47, PDF_CMAP_SINGLE, 932 },
	{ 0xed48, 0xed48, PDF_CMAP_SINGLE, 7932 },
	{ 0xed49, 0xed61, PDF_CMAP_RANGE, 934 },
	{ 0xed62, 0xed62, PDF_CMAP_SINGLE, 7933 },
	{ 0xed63, 0xed7e, PDF_CMAP_RANGE, 960 },
	{ 0xed80, 0xed82, PDF_CMAP_RANGE, 988 },
	{ 0xed83, 0xed83, PDF_CMAP_SINGLE, 7934 },
	{ 0xed84, 0xed84, PDF_CMAP_SINGLE, 992 },
	{ 0xed85, 0xed85, PDF_CMAP_SINGLE, 7935 },
	{ 0xed86, 0xed86, PDF_CMAP_SINGLE, 994 },
	{ 0xed87, 0xed87, PDF_CMAP_SINGLE, 7936 },
	{ 0xed88, 0xed8d, PDF_CMAP_RANGE, 996 },
	{ 0xed8e, 0xed8e, PDF_CMAP_SINGLE, 7937 },
	{ 0xed8f, 0xed94, PDF_CMAP_RANGE, 1003 },
	{ 0xed95, 0xed96, PDF_CMAP_RANGE, 7938 },
	{ 0xed9f, 0xedb6, PDF_CMAP_RANGE, 1011 },
	{ 0xedbf, 0xedd6, PDF_CMAP_RANGE, 1035 },
	{ 0xee40, 0xee5d, PDF_CMAP_RANGE, 7555 },
	{ 0xee5f, 0xee6e, PDF_CMAP_RANGE, 7940 },
	{ 0xee6f, 0xee75, PDF_CMAP_RANGE, 7601 },
	{ 0xee80, 0xee81, PDF_CMAP_RANGE, 7956 },
	{ 0xee82, 0xee8f, PDF_CMAP_RANGE, 7610 },
	{ 0xee90, 0xee90, PDF_CMAP_SINGLE, 762 },
	{ 0xee91, 0xee91, PDF_CMAP_SINGLE, 761 },
	{ 0xee92, 0xee92, PDF_CMAP_SINGLE, 769 },
	{ 0xee93, 0xee99, PDF_CMAP_RANGE, 7624 },
	{ 0xee9a, 0xee9a, PDF_CMAP_SINGLE, 768 },
	{ 0xee9b, 0xee9c, PDF_CMAP_RANGE, 7631 },
};

static const unsigned short pdf_cmap_83pv_RKSJ_H_table[71] =
{
	152,228,124,7479,7481,7491,7495,7503,
	7499,7507,7523,7515,7531,7539,7480,7482,
	7494,7498,7506,7502,7514,7530,7522,7538,
	7554,7511,7526,7519,7534,7542,7508,7527,
	7516,7535,7545,762,761,769,696,7917,
	7918,843,7919,845,7920,847,7921,849,
	7922,7924,909,7925,911,7926,7928,926,
	7929,928,7930,930,7931,932,7932,7934,
	992,7935,994,7936,762,761,769,
};

pdf_cmap pdf_cmap_83pv_RKSJ_H =
{
	-1, "83pv-RKSJ-H", "", nil, 0,
	5, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
		{ 1, 0x00fd, 0x00ff },
	},
	222, 222, (pdf_range*) pdf_cmap_83pv_RKSJ_H_ranges,
	71, 71, (unsigned short*) pdf_cmap_83pv_RKSJ_H_table,
};

/* 90ms-RKSJ-H */

static const pdf_range pdf_cmap_90ms_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007d, PDF_CMAP_RANGE, 231 },
	{ 0x007e, 0x007e, PDF_CMAP_SINGLE, 631 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 0 },
	{ 0x8740, 0x875d, PDF_CMAP_RANGE, 7555 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7585 },
	{ 0x8761, 0x8765, PDF_CMAP_TABLE, 32 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7592 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8044 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7595 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8043 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7598 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8047 },
	{ 0x876f, 0x8775, PDF_CMAP_RANGE, 7601 },
	{ 0x877e, 0x877e, PDF_CMAP_SINGLE, 8323 },
	{ 0x8780, 0x8783, PDF_CMAP_RANGE, 7608 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 8055 },
	{ 0x8785, 0x878f, PDF_CMAP_RANGE, 7613 },
	{ 0x8790, 0x8792, PDF_CMAP_TABLE, 37 },
	{ 0x8793, 0x8799, PDF_CMAP_RANGE, 7624 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 768 },
	{ 0x879b, 0x879c, PDF_CMAP_RANGE, 7631 },
	{ 0x889f, 0x88fc, PDF_CMAP_RANGE, 1125 },
	{ 0x8940, 0x897e, PDF_CMAP_RANGE, 1219 },
	{ 0x8980, 0x89fc, PDF_CMAP_RANGE, 1282 },
	{ 0x8a40, 0x8a7e, PDF_CMAP_RANGE, 1407 },
	{ 0x8a80, 0x8afc, PDF_CMAP_RANGE, 1470 },
	{ 0x8b40, 0x8b7e, PDF_CMAP_RANGE, 1595 },
	{ 0x8b80, 0x8bfc, PDF_CMAP_RANGE, 1658 },
	{ 0x8c40, 0x8c7e, PDF_CMAP_RANGE, 1783 },
	{ 0x8c80, 0x8cfc, PDF_CMAP_RANGE, 1846 },
	{ 0x8d40, 0x8d7e, PDF_CMAP_RANGE, 1971 },
	{ 0x8d80, 0x8dfc, PDF_CMAP_RANGE, 2034 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_RANGE, 2159 },
	{ 0x8e80, 0x8efc, PDF_CMAP_RANGE, 2222 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_RANGE, 2347 },
	{ 0x8f80, 0x8ffc, PDF_CMAP_RANGE, 2410 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 2535 },
	{ 0x9080, 0x90fc, PDF_CMAP_RANGE, 2598 },
	{ 0x9140, 0x917e, PDF_CMAP_RANGE, 2723 },
	{ 0x9180, 0x91fc, PDF_CMAP_RANGE, 2786 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 2911 },
	{ 0x9280, 0x92fc, PDF_CMAP_RANGE, 2974 },
	{ 0x9340, 0x937e, PDF_CMAP_RANGE, 3099 },
	{ 0x9380, 0x93fc, PDF_CMAP_RANGE, 3162 },
	{ 0x9440, 0x947e, PDF_CMAP_RANGE, 3287 },
	{ 0x9480, 0x94fc, PDF_CMAP_RANGE, 3350 },
	{ 0x9540, 0x957e, PDF_CMAP_RANGE, 3475 },
	{ 0x9580, 0x95fc, PDF_CMAP_RANGE, 3538 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 3663 },
	{ 0x9680, 0x96fc, PDF_CMAP_RANGE, 3726 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 3851 },
	{ 0x9780, 0x97fc, PDF_CMAP_RANGE, 3914 },
	{ 0x9840, 0x9872, PDF_CMAP_RANGE, 4039 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x997e, PDF_CMAP_RANGE, 4184 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a7e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a80, 0x9afc, PDF_CMAP_RANGE, 4435 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 4560 },
	{ 0x9b80, 0x9bfc, PDF_CMAP_RANGE, 4623 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xfa40, 0xfa49, PDF_CMAP_RANGE, 8092 },
	{ 0xfa4a, 0xfa53, PDF_CMAP_RANGE, 7575 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 751 },
	{ 0xfa55, 0xfa57, PDF_CMAP_RANGE, 8005 },
	{ 0xfa58, 0xfa5b, PDF_CMAP_TABLE, 40 },
	{ 0xfa5c, 0xfa7e, PDF_CMAP_RANGE, 8359 },
	{ 0xfa80, 0xfacf, PDF_CMAP_RANGE, 8394 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 1993 },
	{ 0xfad1, 0xfafc, PDF_CMAP_RANGE, 8474 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 8518 },
	{ 0xfb80, 0xfbfc, PDF_CMAP_RANGE, 8581 },
	{ 0xfc40, 0xfc4b, PDF_CMAP_RANGE, 8706 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xfa40, 0xfa49, PDF_CMAP_RANGE, 8092 },
	{ 0xfa4a, 0xfa53, PDF_CMAP_RANGE, 7575 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 751 },
	{ 0xfa55, 0xfa57, PDF_CMAP_RANGE, 8005 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 7618 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 7610 },
	{ 0xfa5a, 0xfa5a, PDF_CMAP_SINGLE, 8055 },
	{ 0xfa5b, 0xfa5b, PDF_CMAP_SINGLE, 768 },
	{ 0xfa5c, 0xfa7e, PDF_CMAP_RANGE, 8359 },
	{ 0xfa80, 0xfacf, PDF_CMAP_RANGE, 8394 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 1993 },
	{ 0xfad1, 0xfafc, PDF_CMAP_RANGE, 8474 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 8518 },
	{ 0xfb80, 0xfbfc, PDF_CMAP_RANGE, 8581 },
	{ 0xfc40, 0xfc4b, PDF_CMAP_RANGE, 8706 },
};

static const unsigned short pdf_cmap_90ms_RKSJ_H_table[44] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
	8038,7588,8040,7590,8042,762,761,769,
	7618,7610,8055,768,
};

pdf_cmap pdf_cmap_90ms_RKSJ_H =
{
	-1, "90ms-RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	171, 171, (pdf_range*) pdf_cmap_90ms_RKSJ_H_ranges,
	44, 44, (unsigned short*) pdf_cmap_90ms_RKSJ_H_table,
};

/* 90ms-RKSJ-V */

static const pdf_range pdf_cmap_90ms_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8143, 0x8144, PDF_CMAP_TABLE, 0 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 7917 },
	{ 0x81a8, 0x81a9, PDF_CMAP_TABLE, 2 },
	{ 0x81aa, 0x81ab, PDF_CMAP_RANGE, 736 },
	{ 0x81ac, 0x81ac, PDF_CMAP_SINGLE, 8270 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 4 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7940 },
	{ 0x8761, 0x8765, PDF_CMAP_TABLE, 36 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7947 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8344 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7950 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8348 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7953 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8349 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
	{ 0x84a7, 0x84a7, PDF_CMAP_SINGLE, 7531 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 7507 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 7539 },
	{ 0x84aa, 0x84aa, PDF_CMAP_SINGLE, 7482 },
	{ 0x84ab, 0x84ab, PDF_CMAP_SINGLE, 7480 },
	{ 0x84ac, 0x84ac, PDF_CMAP_SINGLE, 7498 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 7506 },
	{ 0x84ae, 0x84ae, PDF_CMAP_SINGLE, 7502 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 7494 },
	{ 0x84b0, 0x84b0, PDF_CMAP_SINGLE, 7530 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 7522 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 7538 },
	{ 0x84b3, 0x84b3, PDF_CMAP_SINGLE, 7514 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 7554 },
	{ 0x84b5, 0x84b5, PDF_CMAP_SINGLE, 7526 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 7519 },
	{ 0x84b7, 0x84b7, PDF_CMAP_SINGLE, 7534 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 7511 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 7545 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 7527 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 7516 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 7535 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 7508 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 7542 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7940 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 8329 },
	{ 0x8762, 0x8762, PDF_CMAP_SINGLE, 7943 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 8339 },
	{ 0x8764, 0x8764, PDF_CMAP_SINGLE, 7945 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 8338 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7947 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8344 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7950 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8348 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7953 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8349 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
};

static const unsigned short pdf_cmap_90ms_RKSJ_V_table[41] =
{
	8268,8274,739,738,7481,7479,7495,7503,
	7499,7491,7523,7515,7531,7507,7539,7482,
	7480,7498,7506,7502,7494,7530,7522,7538,
	7514,7554,7526,7519,7534,7511,7545,7527,
	7516,7535,7508,7542,8329,7943,8339,7945,
	8338,
};

pdf_cmap pdf_cmap_90ms_RKSJ_V =
{
	-1, "90ms-RKSJ-V", "90ms-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	78, 78, (pdf_range*) pdf_cmap_90ms_RKSJ_V_ranges,
	41, 41, (unsigned short*) pdf_cmap_90ms_RKSJ_V_table,
};

/* 90msp-RKSJ-H */

static const pdf_range pdf_cmap_90msp_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 0 },
	{ 0x8740, 0x875d, PDF_CMAP_RANGE, 7555 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7585 },
	{ 0x8761, 0x8765, PDF_CMAP_TABLE, 32 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7592 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8044 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7595 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8043 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7598 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8047 },
	{ 0x876f, 0x8775, PDF_CMAP_RANGE, 7601 },
	{ 0x877e, 0x877e, PDF_CMAP_SINGLE, 8323 },
	{ 0x8780, 0x8783, PDF_CMAP_RANGE, 7608 },
	{ 0x8784, 0x8784, PDF_CMAP_SINGLE, 8055 },
	{ 0x8785, 0x878f, PDF_CMAP_RANGE, 7613 },
	{ 0x8790, 0x8792, PDF_CMAP_TABLE, 37 },
	{ 0x8793, 0x8799, PDF_CMAP_RANGE, 7624 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 768 },
	{ 0x879b, 0x879c, PDF_CMAP_RANGE, 7631 },
	{ 0x889f, 0x88fc, PDF_CMAP_RANGE, 1125 },
	{ 0x8940, 0x897e, PDF_CMAP_RANGE, 1219 },
	{ 0x8980, 0x89fc, PDF_CMAP_RANGE, 1282 },
	{ 0x8a40, 0x8a7e, PDF_CMAP_RANGE, 1407 },
	{ 0x8a80, 0x8afc, PDF_CMAP_RANGE, 1470 },
	{ 0x8b40, 0x8b7e, PDF_CMAP_RANGE, 1595 },
	{ 0x8b80, 0x8bfc, PDF_CMAP_RANGE, 1658 },
	{ 0x8c40, 0x8c7e, PDF_CMAP_RANGE, 1783 },
	{ 0x8c80, 0x8cfc, PDF_CMAP_RANGE, 1846 },
	{ 0x8d40, 0x8d7e, PDF_CMAP_RANGE, 1971 },
	{ 0x8d80, 0x8dfc, PDF_CMAP_RANGE, 2034 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_RANGE, 2159 },
	{ 0x8e80, 0x8efc, PDF_CMAP_RANGE, 2222 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_RANGE, 2347 },
	{ 0x8f80, 0x8ffc, PDF_CMAP_RANGE, 2410 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 2535 },
	{ 0x9080, 0x90fc, PDF_CMAP_RANGE, 2598 },
	{ 0x9140, 0x917e, PDF_CMAP_RANGE, 2723 },
	{ 0x9180, 0x91fc, PDF_CMAP_RANGE, 2786 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 2911 },
	{ 0x9280, 0x92fc, PDF_CMAP_RANGE, 2974 },
	{ 0x9340, 0x937e, PDF_CMAP_RANGE, 3099 },
	{ 0x9380, 0x93fc, PDF_CMAP_RANGE, 3162 },
	{ 0x9440, 0x947e, PDF_CMAP_RANGE, 3287 },
	{ 0x9480, 0x94fc, PDF_CMAP_RANGE, 3350 },
	{ 0x9540, 0x957e, PDF_CMAP_RANGE, 3475 },
	{ 0x9580, 0x95fc, PDF_CMAP_RANGE, 3538 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 3663 },
	{ 0x9680, 0x96fc, PDF_CMAP_RANGE, 3726 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 3851 },
	{ 0x9780, 0x97fc, PDF_CMAP_RANGE, 3914 },
	{ 0x9840, 0x9872, PDF_CMAP_RANGE, 4039 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x997e, PDF_CMAP_RANGE, 4184 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a7e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a80, 0x9afc, PDF_CMAP_RANGE, 4435 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 4560 },
	{ 0x9b80, 0x9bfc, PDF_CMAP_RANGE, 4623 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xfa40, 0xfa49, PDF_CMAP_RANGE, 8092 },
	{ 0xfa4a, 0xfa53, PDF_CMAP_RANGE, 7575 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 751 },
	{ 0xfa55, 0xfa57, PDF_CMAP_RANGE, 8005 },
	{ 0xfa58, 0xfa5b, PDF_CMAP_TABLE, 40 },
	{ 0xfa5c, 0xfa7e, PDF_CMAP_RANGE, 8359 },
	{ 0xfa80, 0xfacf, PDF_CMAP_RANGE, 8394 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 1993 },
	{ 0xfad1, 0xfafc, PDF_CMAP_RANGE, 8474 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 8518 },
	{ 0xfb80, 0xfbfc, PDF_CMAP_RANGE, 8581 },
	{ 0xfc40, 0xfc4b, PDF_CMAP_RANGE, 8706 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xfa40, 0xfa49, PDF_CMAP_RANGE, 8092 },
	{ 0xfa4a, 0xfa53, PDF_CMAP_RANGE, 7575 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 751 },
	{ 0xfa55, 0xfa57, PDF_CMAP_RANGE, 8005 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 7618 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 7610 },
	{ 0xfa5a, 0xfa5a, PDF_CMAP_SINGLE, 8055 },
	{ 0xfa5b, 0xfa5b, PDF_CMAP_SINGLE, 768 },
	{ 0xfa5c, 0xfa7e, PDF_CMAP_RANGE, 8359 },
	{ 0xfa80, 0xfacf, PDF_CMAP_RANGE, 8394 },
	{ 0xfad0, 0xfad0, PDF_CMAP_SINGLE, 1993 },
	{ 0xfad1, 0xfafc, PDF_CMAP_RANGE, 8474 },
	{ 0xfb40, 0xfb7e, PDF_CMAP_RANGE, 8518 },
	{ 0xfb80, 0xfbfc, PDF_CMAP_RANGE, 8581 },
	{ 0xfc40, 0xfc4b, PDF_CMAP_RANGE, 8706 },
};

static const unsigned short pdf_cmap_90msp_RKSJ_H_table[44] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
	8038,7588,8040,7590,8042,762,761,769,
	7618,7610,8055,768,
};

pdf_cmap pdf_cmap_90msp_RKSJ_H =
{
	-1, "90msp-RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	170, 170, (pdf_range*) pdf_cmap_90msp_RKSJ_H_ranges,
	44, 44, (unsigned short*) pdf_cmap_90msp_RKSJ_H_table,
};

/* 90msp-RKSJ-V */

static const pdf_range pdf_cmap_90msp_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8143, 0x8144, PDF_CMAP_TABLE, 0 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 7917 },
	{ 0x81a8, 0x81a9, PDF_CMAP_TABLE, 2 },
	{ 0x81aa, 0x81ab, PDF_CMAP_RANGE, 736 },
	{ 0x81ac, 0x81ac, PDF_CMAP_SINGLE, 8270 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 4 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7940 },
	{ 0x8761, 0x8765, PDF_CMAP_TABLE, 36 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7947 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8344 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7950 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8348 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7953 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8349 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
	{ 0x84a7, 0x84a7, PDF_CMAP_SINGLE, 7531 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 7507 },
	{ 0x84a9, 0x84a9, PDF_CMAP_SINGLE, 7539 },
	{ 0x84aa, 0x84aa, PDF_CMAP_SINGLE, 7482 },
	{ 0x84ab, 0x84ab, PDF_CMAP_SINGLE, 7480 },
	{ 0x84ac, 0x84ac, PDF_CMAP_SINGLE, 7498 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 7506 },
	{ 0x84ae, 0x84ae, PDF_CMAP_SINGLE, 7502 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 7494 },
	{ 0x84b0, 0x84b0, PDF_CMAP_SINGLE, 7530 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 7522 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 7538 },
	{ 0x84b3, 0x84b3, PDF_CMAP_SINGLE, 7514 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 7554 },
	{ 0x84b5, 0x84b5, PDF_CMAP_SINGLE, 7526 },
	{ 0x84b6, 0x84b6, PDF_CMAP_SINGLE, 7519 },
	{ 0x84b7, 0x84b7, PDF_CMAP_SINGLE, 7534 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 7511 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 7545 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 7527 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 7516 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 7535 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 7508 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 7542 },
	{ 0x875f, 0x8760, PDF_CMAP_RANGE, 7940 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 8329 },
	{ 0x8762, 0x8762, PDF_CMAP_SINGLE, 7943 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 8339 },
	{ 0x8764, 0x8764, PDF_CMAP_SINGLE, 7945 },
	{ 0x8765, 0x8765, PDF_CMAP_SINGLE, 8338 },
	{ 0x8766, 0x8767, PDF_CMAP_RANGE, 7947 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 8344 },
	{ 0x8769, 0x876a, PDF_CMAP_RANGE, 7950 },
	{ 0x876b, 0x876b, PDF_CMAP_SINGLE, 8348 },
	{ 0x876c, 0x876d, PDF_CMAP_RANGE, 7953 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 8349 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
};

static const unsigned short pdf_cmap_90msp_RKSJ_V_table[41] =
{
	8268,8274,739,738,7481,7479,7495,7503,
	7499,7491,7523,7515,7531,7507,7539,7482,
	7480,7498,7506,7502,7494,7530,7522,7538,
	7514,7554,7526,7519,7534,7511,7545,7527,
	7516,7535,7508,7542,8329,7943,8339,7945,
	8338,
};

pdf_cmap pdf_cmap_90msp_RKSJ_V =
{
	-1, "90msp-RKSJ-V", "90msp-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	78, 78, (pdf_range*) pdf_cmap_90msp_RKSJ_V_ranges,
	41, 41, (unsigned short*) pdf_cmap_90msp_RKSJ_V_table,
};

/* 90pv-RKSJ-H */

static const pdf_range pdf_cmap_90pv_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x0080, 0x0080, PDF_CMAP_SINGLE, 97 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x00fd, 0x00ff, PDF_CMAP_TABLE, 0 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 3 },
	{ 0x8540, 0x8553, PDF_CMAP_RANGE, 7555 },
	{ 0x855e, 0x8571, PDF_CMAP_RANGE, 8071 },
	{ 0x857c, 0x857e, PDF_CMAP_RANGE, 8286 },
	{ 0x8580, 0x8585, PDF_CMAP_RANGE, 8289 },
	{ 0x8591, 0x859a, PDF_CMAP_RANGE, 8061 },
	{ 0x859f, 0x85a8, PDF_CMAP_RANGE, 7575 },
	{ 0x85a9, 0x85aa, PDF_CMAP_RANGE, 8225 },
	{ 0x85ab, 0x85ad, PDF_CMAP_RANGE, 8295 },
	{ 0x85b3, 0x85bc, PDF_CMAP_RANGE, 8092 },
	{ 0x85bd, 0x85c1, PDF_CMAP_RANGE, 8298 },
	{ 0x85db, 0x85f4, PDF_CMAP_RANGE, 8112 },
	{ 0x8640, 0x864b, PDF_CMAP_TABLE, 35 },
	{ 0x864c, 0x864d, PDF_CMAP_RANGE, 7605 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 8037 },
	{ 0x864f, 0x8655, PDF_CMAP_RANGE, 8024 },
	{ 0x8656, 0x8657, PDF_CMAP_TABLE, 47 },
	{ 0x8658, 0x8659, PDF_CMAP_RANGE, 8034 },
	{ 0x865a, 0x865c, PDF_CMAP_RANGE, 8031 },
	{ 0x865d, 0x865d, PDF_CMAP_SINGLE, 8306 },
	{ 0x869b, 0x869d, PDF_CMAP_RANGE, 7610 },
	{ 0x869e, 0x869f, PDF_CMAP_TABLE, 49 },
	{ 0x86a0, 0x86a1, PDF_CMAP_RANGE, 8016 },
	{ 0x86a2, 0x86a6, PDF_CMAP_TABLE, 51 },
	{ 0x86b3, 0x86b5, PDF_CMAP_TABLE, 56 },
	{ 0x86c7, 0x86ca, PDF_CMAP_RANGE, 8219 },
	{ 0x86cb, 0x86ce, PDF_CMAP_RANGE, 8309 },
	{ 0x86cf, 0x86d2, PDF_CMAP_TABLE, 59 },
	{ 0x86d3, 0x86d6, PDF_CMAP_RANGE, 8206 },
	{ 0x8740, 0x8746, PDF_CMAP_RANGE, 8197 },
	{ 0x8747, 0x8755, PDF_CMAP_TABLE, 63 },
	{ 0x8756, 0x8757, PDF_CMAP_RANGE, 8139 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 8142 },
	{ 0x8791, 0x8792, PDF_CMAP_RANGE, 8317 },
	{ 0x8793, 0x8797, PDF_CMAP_RANGE, 7613 },
	{ 0x8798, 0x87a4, PDF_CMAP_TABLE, 78 },
	{ 0x87a5, 0x87a6, PDF_CMAP_RANGE, 8327 },
	{ 0x87a7, 0x87a8, PDF_CMAP_TABLE, 91 },
	{ 0x87a9, 0x87aa, PDF_CMAP_RANGE, 8040 },
	{ 0x87ab, 0x87b5, PDF_CMAP_TABLE, 93 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 8048 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 8051 },
	{ 0x87c0, 0x87c1, PDF_CMAP_RANGE, 8049 },
	{ 0x87e5, 0x87e7, PDF_CMAP_RANGE, 7621 },
	{ 0x87e8, 0x87e8, PDF_CMAP_SINGLE, 8323 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 8054 },
	{ 0x87fb, 0x87fc, PDF_CMAP_RANGE, 8321 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 7624 },
	{ 0x8841, 0x8842, PDF_CMAP_RANGE, 7629 },
	{ 0x8854, 0x8855, PDF_CMAP_RANGE, 7608 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 7958 },
	{ 0x886a, 0x886d, PDF_CMAP_RANGE, 8313 },
	{ 0x889f, 0x88fc, PDF_CMAP_RANGE, 1125 },
	{ 0x8940, 0x897e, PDF_CMAP_RANGE, 1219 },
	{ 0x8980, 0x89fc, PDF_CMAP_RANGE, 1282 },
	{ 0x8a40, 0x8a7e, PDF_CMAP_RANGE, 1407 },
	{ 0x8a80, 0x8afc, PDF_CMAP_RANGE, 1470 },
	{ 0x8b40, 0x8b7e, PDF_CMAP_RANGE, 1595 },
	{ 0x8b80, 0x8bfc, PDF_CMAP_RANGE, 1658 },
	{ 0x8c40, 0x8c7e, PDF_CMAP_RANGE, 1783 },
	{ 0x8c80, 0x8cfc, PDF_CMAP_RANGE, 1846 },
	{ 0x8d40, 0x8d7e, PDF_CMAP_RANGE, 1971 },
	{ 0x8d80, 0x8dfc, PDF_CMAP_RANGE, 2034 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_RANGE, 2159 },
	{ 0x8e80, 0x8efc, PDF_CMAP_RANGE, 2222 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_RANGE, 2347 },
	{ 0x8f80, 0x8ffc, PDF_CMAP_RANGE, 2410 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 2535 },
	{ 0x9080, 0x90fc, PDF_CMAP_RANGE, 2598 },
	{ 0x9140, 0x917e, PDF_CMAP_RANGE, 2723 },
	{ 0x9180, 0x91fc, PDF_CMAP_RANGE, 2786 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 2911 },
	{ 0x9280, 0x92fc, PDF_CMAP_RANGE, 2974 },
	{ 0x9340, 0x937e, PDF_CMAP_RANGE, 3099 },
	{ 0x9380, 0x93fc, PDF_CMAP_RANGE, 3162 },
	{ 0x9440, 0x947e, PDF_CMAP_RANGE, 3287 },
	{ 0x9480, 0x94fc, PDF_CMAP_RANGE, 3350 },
	{ 0x9540, 0x957e, PDF_CMAP_RANGE, 3475 },
	{ 0x9580, 0x95fc, PDF_CMAP_RANGE, 3538 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 3663 },
	{ 0x9680, 0x96fc, PDF_CMAP_RANGE, 3726 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 3851 },
	{ 0x9780, 0x97fc, PDF_CMAP_RANGE, 3914 },
	{ 0x9840, 0x9872, PDF_CMAP_RANGE, 4039 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x997e, PDF_CMAP_RANGE, 4184 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a7e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a80, 0x9afc, PDF_CMAP_RANGE, 4435 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 4560 },
	{ 0x9b80, 0x9bfc, PDF_CMAP_RANGE, 4623 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xeb41, 0xeb42, PDF_CMAP_RANGE, 7887 },
	{ 0xeb50, 0xeb51, PDF_CMAP_RANGE, 7889 },
	{ 0xeb5b, 0xeb5d, PDF_CMAP_RANGE, 7891 },
	{ 0xeb60, 0xeb64, PDF_CMAP_RANGE, 7894 },
	{ 0xeb69, 0xeb7a, PDF_CMAP_RANGE, 7899 },
	{ 0xeb81, 0xeb81, PDF_CMAP_SINGLE, 7917 },
	{ 0xec9f, 0xec9f, PDF_CMAP_SINGLE, 7918 },
	{ 0xeca1, 0xeca1, PDF_CMAP_SINGLE, 7919 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 7920 },
	{ 0xeca5, 0xeca5, PDF_CMAP_SINGLE, 7921 },
	{ 0xeca7, 0xeca7, PDF_CMAP_SINGLE, 7922 },
	{ 0xecc1, 0xecc1, PDF_CMAP_SINGLE, 7923 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 7924 },
	{ 0xece3, 0xece3, PDF_CMAP_SINGLE, 7925 },
	{ 0xece5, 0xece5, PDF_CMAP_SINGLE, 7926 },
	{ 0xecec, 0xecec, PDF_CMAP_SINGLE, 7927 },
	{ 0xed40, 0xed40, PDF_CMAP_SINGLE, 7928 },
	{ 0xed42, 0xed42, PDF_CMAP_SINGLE, 7929 },
	{ 0xed44, 0xed44, PDF_CMAP_SINGLE, 7930 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 7931 },
	{ 0xed48, 0xed48, PDF_CMAP_SINGLE, 7932 },
	{ 0xed62, 0xed62, PDF_CMAP_SINGLE, 7933 },
	{ 0xed83, 0xed83, PDF_CMAP_SINGLE, 7934 },
	{ 0xed85, 0xed85, PDF_CMAP_SINGLE, 7935 },
	{ 0xed87, 0xed87, PDF_CMAP_SINGLE, 7936 },
	{ 0xed8e, 0xed8e, PDF_CMAP_SINGLE, 7937 },
	{ 0xed95, 0xed96, PDF_CMAP_RANGE, 7938 },
	{ 0x8b40, 0x8b7e, PDF_CMAP_RANGE, 1595 },
	{ 0x8b80, 0x8bfc, PDF_CMAP_RANGE, 1658 },
	{ 0x8c40, 0x8c7e, PDF_CMAP_RANGE, 1783 },
	{ 0x8c80, 0x8cfc, PDF_CMAP_RANGE, 1846 },
	{ 0x8d40, 0x8d7e, PDF_CMAP_RANGE, 1971 },
	{ 0x8d80, 0x8dfc, PDF_CMAP_RANGE, 2034 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_RANGE, 2159 },
	{ 0x8e80, 0x8efc, PDF_CMAP_RANGE, 2222 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_RANGE, 2347 },
	{ 0x8f80, 0x8ffc, PDF_CMAP_RANGE, 2410 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 2535 },
	{ 0x9080, 0x90fc, PDF_CMAP_RANGE, 2598 },
	{ 0x9140, 0x917e, PDF_CMAP_RANGE, 2723 },
	{ 0x9180, 0x91fc, PDF_CMAP_RANGE, 2786 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 2911 },
	{ 0x9280, 0x92fc, PDF_CMAP_RANGE, 2974 },
	{ 0x9340, 0x937e, PDF_CMAP_RANGE, 3099 },
	{ 0x9380, 0x93fc, PDF_CMAP_RANGE, 3162 },
	{ 0x9440, 0x947e, PDF_CMAP_RANGE, 3287 },
	{ 0x9480, 0x94fc, PDF_CMAP_RANGE, 3350 },
	{ 0x9540, 0x957e, PDF_CMAP_RANGE, 3475 },
	{ 0x9580, 0x95fc, PDF_CMAP_RANGE, 3538 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 3663 },
	{ 0x9680, 0x96fc, PDF_CMAP_RANGE, 3726 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 3851 },
	{ 0x9780, 0x97fc, PDF_CMAP_RANGE, 3914 },
	{ 0x9840, 0x9872, PDF_CMAP_RANGE, 4039 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x997e, PDF_CMAP_RANGE, 4184 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a7e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a80, 0x9afc, PDF_CMAP_RANGE, 4435 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 4560 },
	{ 0x9b80, 0x9bfc, PDF_CMAP_RANGE, 4623 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xeb41, 0xeb42, PDF_CMAP_RANGE, 7887 },
	{ 0xeb50, 0xeb51, PDF_CMAP_RANGE, 7889 },
	{ 0xeb5b, 0xeb5d, PDF_CMAP_RANGE, 7891 },
	{ 0xeb60, 0xeb64, PDF_CMAP_RANGE, 7894 },
	{ 0xeb69, 0xeb7a, PDF_CMAP_RANGE, 7899 },
	{ 0xeb81, 0xeb81, PDF_CMAP_SINGLE, 7917 },
	{ 0xec9f, 0xec9f, PDF_CMAP_SINGLE, 7918 },
	{ 0xeca1, 0xeca1, PDF_CMAP_SINGLE, 7919 },
	{ 0xeca3, 0xeca3, PDF_CMAP_SINGLE, 7920 },
	{ 0xeca5, 0xeca5, PDF_CMAP_SINGLE, 7921 },
	{ 0xeca7, 0xeca7, PDF_CMAP_SINGLE, 7922 },
	{ 0xecc1, 0xecc1, PDF_CMAP_SINGLE, 7923 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 7924 },
	{ 0xece3, 0xece3, PDF_CMAP_SINGLE, 7925 },
	{ 0xece5, 0xece5, PDF_CMAP_SINGLE, 7926 },
	{ 0xecec, 0xecec, PDF_CMAP_SINGLE, 7927 },
	{ 0xed40, 0xed40, PDF_CMAP_SINGLE, 7928 },
	{ 0xed42, 0xed42, PDF_CMAP_SINGLE, 7929 },
	{ 0xed44, 0xed44, PDF_CMAP_SINGLE, 7930 },
	{ 0xed46, 0xed46, PDF_CMAP_SINGLE, 7931 },
	{ 0xed48, 0xed48, PDF_CMAP_SINGLE, 7932 },
	{ 0xed62, 0xed62, PDF_CMAP_SINGLE, 7933 },
	{ 0xed83, 0xed83, PDF_CMAP_SINGLE, 7934 },
	{ 0xed85, 0xed85, PDF_CMAP_SINGLE, 7935 },
	{ 0xed87, 0xed87, PDF_CMAP_SINGLE, 7936 },
	{ 0xed8e, 0xed8e, PDF_CMAP_SINGLE, 7937 },
	{ 0xed95, 0xed96, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_90pv_RKSJ_H_table[104] =
{
	152,228,124,7479,7481,7491,7495,7503,
	7499,7507,7523,7515,7531,7539,7480,7482,
	7494,7498,7506,7502,7514,7530,7522,7538,
	7554,7511,7526,7519,7534,7542,7508,7527,
	7516,7535,7545,7601,8186,7602,8020,8022,
	8303,7607,8023,7603,8021,7604,8304,8305,
	8036,8307,8018,8019,8211,8213,8212,8214,
	8058,8056,8308,8014,8013,8012,8011,8150,
	8204,8145,8138,7620,8151,7618,8146,8141,
	7619,8149,8147,8143,8148,8144,8154,8165,
	8319,8158,8191,8320,8223,7585,8038,7588,
	7586,8039,8183,8042,7592,7590,7593,7599,
	8046,8044,7595,8045,8043,7596,8047,7598,
};

pdf_cmap pdf_cmap_90pv_RKSJ_H =
{
	-1, "90pv-RKSJ-H", "", nil, 0,
	5, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
		{ 1, 0x00fd, 0x00ff },
	},
	263, 263, (pdf_range*) pdf_cmap_90pv_RKSJ_H_ranges,
	104, 104, (unsigned short*) pdf_cmap_90pv_RKSJ_H_table,
};

/* 90pv-RKSJ-V */

static const pdf_range pdf_cmap_90pv_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 7917 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0x879f, 0x87a3, PDF_CMAP_TABLE, 0 },
	{ 0x87a4, 0x87a5, PDF_CMAP_RANGE, 8333 },
	{ 0x87a6, 0x87a7, PDF_CMAP_RANGE, 8337 },
	{ 0x87a8, 0x87a8, PDF_CMAP_SINGLE, 7947 },
	{ 0x87a9, 0x87aa, PDF_CMAP_RANGE, 8339 },
	{ 0x87ab, 0x87ad, PDF_CMAP_TABLE, 5 },
	{ 0x87ae, 0x87af, PDF_CMAP_RANGE, 8343 },
	{ 0x87b0, 0x87b0, PDF_CMAP_SINGLE, 7950 },
	{ 0x87b1, 0x87b2, PDF_CMAP_RANGE, 8347 },
	{ 0x87b3, 0x87b5, PDF_CMAP_TABLE, 8 },
	{ 0x87bd, 0x87c1, PDF_CMAP_TABLE, 11 },
	{ 0x87fa, 0x87fc, PDF_CMAP_RANGE, 8324 },
	{ 0x87ae, 0x87af, PDF_CMAP_RANGE, 8343 },
	{ 0x87b0, 0x87b0, PDF_CMAP_SINGLE, 7950 },
	{ 0x87b1, 0x87b2, PDF_CMAP_RANGE, 8347 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 7951 },
	{ 0x87b4, 0x87b4, PDF_CMAP_SINGLE, 8349 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 7953 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 8350 },
	{ 0x87be, 0x87be, PDF_CMAP_SINGLE, 8353 },
	{ 0x87bf, 0x87bf, PDF_CMAP_SINGLE, 8356 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 8358 },
	{ 0x87c1, 0x87c1, PDF_CMAP_SINGLE, 8357 },
	{ 0x87fa, 0x87fc, PDF_CMAP_RANGE, 8324 },
};

static const unsigned short pdf_cmap_90pv_RKSJ_V_table[16] =
{
	7940,8329,7943,7941,8330,7945,7948,7954,
	7951,8349,7953,8350,8353,8356,8358,8357,
};

pdf_cmap pdf_cmap_90pv_RKSJ_V =
{
	-1, "90pv-RKSJ-V", "90pv-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	51, 51, (pdf_range*) pdf_cmap_90pv_RKSJ_V_ranges,
	16, 16, (unsigned short*) pdf_cmap_90pv_RKSJ_V_table,
};

/* Add-H */

static const pdf_range pdf_cmap_Add_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 633 },
	{ 0x2221, 0x222e, PDF_CMAP_RANGE, 727 },
	{ 0x223a, 0x2241, PDF_CMAP_RANGE, 741 },
	{ 0x224a, 0x2250, PDF_CMAP_RANGE, 749 },
	{ 0x225c, 0x226a, PDF_CMAP_RANGE, 756 },
	{ 0x2272, 0x2279, PDF_CMAP_RANGE, 771 },
	{ 0x227e, 0x227e, PDF_CMAP_SINGLE, 779 },
	{ 0x2330, 0x2339, PDF_CMAP_RANGE, 780 },
	{ 0x2341, 0x235a, PDF_CMAP_RANGE, 790 },
	{ 0x2361, 0x237a, PDF_CMAP_RANGE, 816 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 842 },
	{ 0x2474, 0x2476, PDF_CMAP_RANGE, 7958 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 925 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 1011 },
	{ 0x2641, 0x2658, PDF_CMAP_RANGE, 1035 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 1059 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 1092 },
	{ 0x2821, 0x2840, PDF_CMAP_TABLE, 0 },
	{ 0x3021, 0x3022, PDF_CMAP_TABLE, 32 },
	{ 0x3023, 0x3031, PDF_CMAP_RANGE, 1127 },
	{ 0x3032, 0x3032, PDF_CMAP_SINGLE, 7961 },
	{ 0x3033, 0x303a, PDF_CMAP_RANGE, 1143 },
	{ 0x303b, 0x303b, PDF_CMAP_SINGLE, 7634 },
	{ 0x303c, 0x306d, PDF_CMAP_RANGE, 1152 },
	{ 0x306e, 0x306e, PDF_CMAP_SINGLE, 7635 },
	{ 0x306f, 0x3070, PDF_CMAP_RANGE, 1203 },
	{ 0x3071, 0x3073, PDF_CMAP_TABLE, 34 },
	{ 0x3074, 0x307b, PDF_CMAP_RANGE, 1208 },
	{ 0x307c, 0x307c, PDF_CMAP_SINGLE, 7637 },
	{ 0x307d, 0x307e, PDF_CMAP_RANGE, 1217 },
	{ 0x3121, 0x3129, PDF_CMAP_RANGE, 1219 },
	{ 0x312a, 0x312a, PDF_CMAP_SINGLE, 7638 },
	{ 0x312b, 0x3132, PDF_CMAP_RANGE, 1229 },
	{ 0x3133, 0x3135, PDF_CMAP_TABLE, 37 },
	{ 0x3136, 0x3138, PDF_CMAP_RANGE, 1240 },
	{ 0x3139, 0x3139, PDF_CMAP_SINGLE, 7964 },
	{ 0x313a, 0x313c, PDF_CMAP_RANGE, 1244 },
	{ 0x313d, 0x313d, PDF_CMAP_SINGLE, 7642 },
	{ 0x313e, 0x3141, PDF_CMAP_RANGE, 1248 },
	{ 0x3142, 0x3142, PDF_CMAP_SINGLE, 7643 },
	{ 0x3143, 0x316a, PDF_CMAP_RANGE, 1253 },
	{ 0x316b, 0x316b, PDF_CMAP_SINGLE, 7644 },
	{ 0x316c, 0x317e, PDF_CMAP_RANGE, 1294 },
	{ 0x3221, 0x3227, PDF_CMAP_RANGE, 1313 },
	{ 0x3228, 0x322a, PDF_CMAP_TABLE, 40 },
	{ 0x322b, 0x325f, PDF_CMAP_RANGE, 1323 },
	{ 0x3260, 0x3260, PDF_CMAP_SINGLE, 7647 },
	{ 0x3261, 0x3266, PDF_CMAP_RANGE, 1377 },
	{ 0x3267, 0x3267, PDF_CMAP_SINGLE, 7965 },
	{ 0x3268, 0x3279, PDF_CMAP_RANGE, 1384 },
	{ 0x327a, 0x327a, PDF_CMAP_SINGLE, 7648 },
	{ 0x327b, 0x327e, PDF_CMAP_RANGE, 1403 },
	{ 0x3321, 0x3322, PDF_CMAP_TABLE, 43 },
	{ 0x3323, 0x336a, PDF_CMAP_RANGE, 1409 },
	{ 0x336b, 0x336b, PDF_CMAP_SINGLE, 7652 },
	{ 0x336c, 0x3372, PDF_CMAP_RANGE, 1482 },
	{ 0x3373, 0x3373, PDF_CMAP_SINGLE, 7653 },
	{ 0x3374, 0x3379, PDF_CMAP_RANGE, 1490 },
	{ 0x337a, 0x337a, PDF_CMAP_SINGLE, 7654 },
	{ 0x337b, 0x337e, PDF_CMAP_RANGE, 1497 },
	{ 0x3421, 0x3441, PDF_CMAP_RANGE, 1501 },
	{ 0x3442, 0x3442, PDF_CMAP_SINGLE, 7655 },
	{ 0x3443, 0x344c, PDF_CMAP_RANGE, 1535 },
	{ 0x344d, 0x344d, PDF_CMAP_SINGLE, 7656 },
	{ 0x344e, 0x3464, PDF_CMAP_RANGE, 1546 },
	{ 0x3465, 0x3465, PDF_CMAP_SINGLE, 7657 },
	{ 0x3466, 0x347e, PDF_CMAP_RANGE, 1570 },
	{ 0x3521, 0x352a, PDF_CMAP_RANGE, 1595 },
	{ 0x352b, 0x352b, PDF_CMAP_SINGLE, 7658 },
	{ 0x352c, 0x353f, PDF_CMAP_RANGE, 1606 },
	{ 0x3540, 0x3540, PDF_CMAP_SINGLE, 7659 },
	{ 0x3541, 0x3561, PDF_CMAP_RANGE, 1627 },
	{ 0x3562, 0x3562, PDF_CMAP_SINGLE, 7966 },
	{ 0x3563, 0x3567, PDF_CMAP_RANGE, 1661 },
	{ 0x3568, 0x3568, PDF_CMAP_SINGLE, 7967 },
	{ 0x3569, 0x357e, PDF_CMAP_RANGE, 1667 },
	{ 0x3621, 0x3622, PDF_CMAP_TABLE, 45 },
	{ 0x3623, 0x3629, PDF_CMAP_RANGE, 1691 },
	{ 0x362a, 0x362a, PDF_CMAP_SINGLE, 7661 },
	{ 0x362b, 0x3641, PDF_CMAP_RANGE, 1699 },
	{ 0x3642, 0x3642, PDF_CMAP_SINGLE, 7968 },
	{ 0x3643, 0x364e, PDF_CMAP_RANGE, 1723 },
	{ 0x364f, 0x364f, PDF_CMAP_SINGLE, 7662 },
	{ 0x3650, 0x366c, PDF_CMAP_RANGE, 1736 },
	{ 0x366d, 0x366d, PDF_CMAP_SINGLE, 7663 },
	{ 0x366e, 0x3673, PDF_CMAP_RANGE, 1766 },
	{ 0x3674, 0x3674, PDF_CMAP_SINGLE, 7664 },
	{ 0x3675, 0x367a, PDF_CMAP_RANGE, 1773 },
	{ 0x367b, 0x367e, PDF_CMAP_TABLE, 47 },
	{ 0x3721, 0x3736, PDF_CMAP_RANGE, 1783 },
	{ 0x3737, 0x3737, PDF_CMAP_SINGLE, 7668 },
	{ 0x3738, 0x3751, PDF_CMAP_RANGE, 1806 },
	{ 0x3752, 0x3752, PDF_CMAP_SINGLE, 7671 },
	{ 0x3753, 0x3770, PDF_CMAP_RANGE, 1833 },
	{ 0x3771, 0x3771, PDF_CMAP_SINGLE, 7674 },
	{ 0x3772, 0x377c, PDF_CMAP_RANGE, 1864 },
	{ 0x377d, 0x377e, PDF_CMAP_TABLE, 51 },
	{ 0x3821, 0x3833, PDF_CMAP_RANGE, 1877 },
	{ 0x3834, 0x3834, PDF_CMAP_SINGLE, 7677 },
	{ 0x3835, 0x3840, PDF_CMAP_RANGE, 1897 },
	{ 0x3841, 0x3841, PDF_CMAP_SINGLE, 7678 },
	{ 0x3842, 0x387e, PDF_CMAP_RANGE, 1910 },
	{ 0x3921, 0x392a, PDF_CMAP_RANGE, 1971 },
	{ 0x392b, 0x392b, PDF_CMAP_SINGLE, 7679 },
	{ 0x392c, 0x396c, PDF_CMAP_RANGE, 1982 },
	{ 0x396d, 0x396d, PDF_CMAP_SINGLE, 7682 },
	{ 0x396e, 0x3973, PDF_CMAP_RANGE, 2048 },
	{ 0x3974, 0x3974, PDF_CMAP_SINGLE, 7683 },
	{ 0x3975, 0x3978, PDF_CMAP_RANGE, 2055 },
	{ 0x3979, 0x3979, PDF_CMAP_SINGLE, 7684 },
	{ 0x397a, 0x397e, PDF_CMAP_RANGE, 2060 },
	{ 0x3a21, 0x3a52, PDF_CMAP_RANGE, 2065 },
	{ 0x3a53, 0x3a53, PDF_CMAP_SINGLE, 7685 },
	{ 0x3a54, 0x3a66, PDF_CMAP_RANGE, 2116 },
	{ 0x3a67, 0x3a67, PDF_CMAP_SINGLE, 7686 },
	{ 0x3a68, 0x3a73, PDF_CMAP_RANGE, 2136 },
	{ 0x3a74, 0x3a74, PDF_CMAP_SINGLE, 7687 },
	{ 0x3a75, 0x3a7e, PDF_CMAP_RANGE, 2149 },
	{ 0x3b21, 0x3b26, PDF_CMAP_RANGE, 2159 },
	{ 0x3b27, 0x3b27, PDF_CMAP_SINGLE, 7688 },
	{ 0x3b28, 0x3b29, PDF_CMAP_RANGE, 2166 },
	{ 0x3b2a, 0x3b2c, PDF_CMAP_TABLE, 53 },
	{ 0x3b2d, 0x3b38, PDF_CMAP_RANGE, 2171 },
	{ 0x3b39, 0x3b39, PDF_CMAP_SINGLE, 7691 },
	{ 0x3b3a, 0x3b40, PDF_CMAP_RANGE, 2184 },
	{ 0x3b41, 0x3b41, PDF_CMAP_SINGLE, 7970 },
	{ 0x3b42, 0x3b7e, PDF_CMAP_RANGE, 2192 },
	{ 0x3c21, 0x3c47, PDF_CMAP_RANGE, 2253 },
	{ 0x3c48, 0x3c48, PDF_CMAP_SINGLE, 7693 },
	{ 0x3c49, 0x3c5c, PDF_CMAP_RANGE, 2293 },
	{ 0x3c5d, 0x3c5e, PDF_CMAP_RANGE, 7695 },
	{ 0x3c5f, 0x3c7e, PDF_CMAP_RANGE, 2315 },
	{ 0x3d21, 0x3d2a, PDF_CMAP_RANGE, 2347 },
	{ 0x3d2b, 0x3d2b, PDF_CMAP_SINGLE, 7697 },
	{ 0x3d2c, 0x3d35, PDF_CMAP_RANGE, 2358 },
	{ 0x3d36, 0x3d36, PDF_CMAP_SINGLE, 7698 },
	{ 0x3d37, 0x3d6b, PDF_CMAP_RANGE, 2369 },
	{ 0x3d6c, 0x3d6c, PDF_CMAP_SINGLE, 7699 },
	{ 0x3d6d, 0x3d71, PDF_CMAP_RANGE, 2423 },
	{ 0x3d72, 0x3d73, PDF_CMAP_RANGE, 7701 },
	{ 0x3d74, 0x3d7e, PDF_CMAP_RANGE, 2430 },
	{ 0x3e21, 0x3e24, PDF_CMAP_RANGE, 2441 },
	{ 0x3e25, 0x3e25, PDF_CMAP_SINGLE, 7703 },
	{ 0x3e26, 0x3e32, PDF_CMAP_RANGE, 2446 },
	{ 0x3e33, 0x3e33, PDF_CMAP_SINGLE, 7704 },
	{ 0x3e34, 0x3e54, PDF_CMAP_RANGE, 2460 },
	{ 0x3e55, 0x3e55, PDF_CMAP_SINGLE, 7706 },
	{ 0x3e56, 0x3e5e, PDF_CMAP_RANGE, 2494 },
	{ 0x3e5f, 0x3e5f, PDF_CMAP_SINGLE, 7707 },
	{ 0x3e60, 0x3e63, PDF_CMAP_RANGE, 2504 },
	{ 0x3e64, 0x3e64, PDF_CMAP_SINGLE, 7708 },
	{ 0x3e65, 0x3e7e, PDF_CMAP_RANGE, 2509 },
	{ 0x3f21, 0x3f29, PDF_CMAP_RANGE, 2535 },
	{ 0x3f2a, 0x3f2a, PDF_CMAP_SINGLE, 7709 },
	{ 0x3f2b, 0x3f58, PDF_CMAP_RANGE, 2545 },
	{ 0x3f59, 0x3f59, PDF_CMAP_SINGLE, 7971 },
	{ 0x3f5a, 0x3f5f, PDF_CMAP_RANGE, 2592 },
	{ 0x3f60, 0x3f60, PDF_CMAP_SINGLE, 7711 },
	{ 0x3f61, 0x3f7e, PDF_CMAP_RANGE, 2599 },
	{ 0x4021, 0x4022, PDF_CMAP_TABLE, 56 },
	{ 0x4023, 0x4065, PDF_CMAP_RANGE, 2631 },
	{ 0x4066, 0x4066, PDF_CMAP_SINGLE, 7715 },
	{ 0x4067, 0x4070, PDF_CMAP_RANGE, 2699 },
	{ 0x4071, 0x4071, PDF_CMAP_SINGLE, 7716 },
	{ 0x4072, 0x4078, PDF_CMAP_RANGE, 2710 },
	{ 0x4079, 0x407b, PDF_CMAP_TABLE, 58 },
	{ 0x407c, 0x407d, PDF_CMAP_RANGE, 7973 },
	{ 0x407e, 0x407e, PDF_CMAP_SINGLE, 2722 },
	{ 0x4121, 0x4126, PDF_CMAP_RANGE, 2723 },
	{ 0x4127, 0x4127, PDF_CMAP_SINGLE, 7720 },
	{ 0x4128, 0x4138, PDF_CMAP_RANGE, 2730 },
	{ 0x4139, 0x4139, PDF_CMAP_SINGLE, 7721 },
	{ 0x413a, 0x414b, PDF_CMAP_RANGE, 2748 },
	{ 0x414c, 0x414c, PDF_CMAP_SINGLE, 7722 },
	{ 0x414d, 0x414e, PDF_CMAP_RANGE, 2767 },
	{ 0x414f, 0x414f, PDF_CMAP_SINGLE, 7723 },
	{ 0x4150, 0x415e, PDF_CMAP_RANGE, 2770 },
	{ 0x415f, 0x415f, PDF_CMAP_SINGLE, 7724 },
	{ 0x4160, 0x4168, PDF_CMAP_RANGE, 2786 },
	{ 0x4169, 0x4169, PDF_CMAP_SINGLE, 7725 },
	{ 0x416a, 0x417e, PDF_CMAP_RANGE, 2796 },
	{ 0x4221, 0x4236, PDF_CMAP_RANGE, 2817 },
	{ 0x4237, 0x4237, PDF_CMAP_SINGLE, 7975 },
	{ 0x4238, 0x423c, PDF_CMAP_RANGE, 2840 },
	{ 0x423d, 0x423d, PDF_CMAP_SINGLE, 7726 },
	{ 0x423e, 0x424c, PDF_CMAP_RANGE, 2846 },
	{ 0x424d, 0x424d, PDF_CMAP_SINGLE, 7727 },
	{ 0x424e, 0x425b, PDF_CMAP_RANGE, 2862 },
	{ 0x425c, 0x425c, PDF_CMAP_SINGLE, 7728 },
	{ 0x425d, 0x4262, PDF_CMAP_RANGE, 2877 },
	{ 0x4263, 0x4263, PDF_CMAP_SINGLE, 7729 },
	{ 0x4264, 0x426e, PDF_CMAP_RANGE, 2884 },
	{ 0x426f, 0x426f, PDF_CMAP_SINGLE, 7730 },
	{ 0x4270, 0x427c, PDF_CMAP_RANGE, 2896 },
	{ 0x427d, 0x427e, PDF_CMAP_TABLE, 61 },
	{ 0x4321, 0x4326, PDF_CMAP_RANGE, 2911 },
	{ 0x4327, 0x4329, PDF_CMAP_TABLE, 63 },
	{ 0x432a, 0x432c, PDF_CMAP_RANGE, 2920 },
	{ 0x432d, 0x432e, PDF_CMAP_RANGE, 7737 },
	{ 0x432f, 0x433c, PDF_CMAP_RANGE, 2925 },
	{ 0x433d, 0x433d, PDF_CMAP_SINGLE, 7739 },
	{ 0x433e, 0x436f, PDF_CMAP_RANGE, 2940 },
	{ 0x4370, 0x4370, PDF_CMAP_SINGLE, 7740 },
	{ 0x4371, 0x4374, PDF_CMAP_RANGE, 2991 },
	{ 0x4375, 0x4375, PDF_CMAP_SINGLE, 7741 },
	{ 0x4376, 0x437b, PDF_CMAP_RANGE, 2996 },
	{ 0x437c, 0x437c, PDF_CMAP_SINGLE, 7742 },
	{ 0x437d, 0x437e, PDF_CMAP_RANGE, 3003 },
	{ 0x4421, 0x443c, PDF_CMAP_RANGE, 3005 },
	{ 0x443d, 0x443d, PDF_CMAP_SINGLE, 7743 },
	{ 0x443e, 0x4447, PDF_CMAP_RANGE, 3034 },
	{ 0x4448, 0x444a, PDF_CMAP_TABLE, 66 },
	{ 0x444b, 0x444e, PDF_CMAP_RANGE, 3047 },
	{ 0x444f, 0x444f, PDF_CMAP_SINGLE, 7747 },
	{ 0x4450, 0x447e, PDF_CMAP_RANGE, 3052 },
	{ 0x4521, 0x4522, PDF_CMAP_TABLE, 69 },
	{ 0x4523, 0x4526, PDF_CMAP_RANGE, 3101 },
	{ 0x4527, 0x4527, PDF_CMAP_SINGLE, 7749 },
	{ 0x4528, 0x452d, PDF_CMAP_RANGE, 3106 },
	{ 0x452e, 0x452e, PDF_CMAP_SINGLE, 7750 },
	{ 0x452f, 0x4535, PDF_CMAP_RANGE, 3113 },
	{ 0x4536, 0x4536, PDF_CMAP_SINGLE, 7751 },
	{ 0x4537, 0x453e, PDF_CMAP_RANGE, 3121 },
	{ 0x453f, 0x453f, PDF_CMAP_SINGLE, 7752 },
	{ 0x4540, 0x4547, PDF_CMAP_RANGE, 3130 },
	{ 0x4548, 0x4548, PDF_CMAP_SINGLE, 7753 },
	{ 0x4549, 0x454a, PDF_CMAP_RANGE, 3139 },
	{ 0x454b, 0x454b, PDF_CMAP_SINGLE, 7754 },
	{ 0x454c, 0x4550, PDF_CMAP_RANGE, 3142 },
	{ 0x4551, 0x4552, PDF_CMAP_TABLE, 71 },
	{ 0x4553, 0x4563, PDF_CMAP_RANGE, 3149 },
	{ 0x4564, 0x4564, PDF_CMAP_SINGLE, 7757 },
	{ 0x4565, 0x4577, PDF_CMAP_RANGE, 3167 },
	{ 0x4578, 0x4578, PDF_CMAP_SINGLE, 7758 },
	{ 0x4579, 0x457e, PDF_CMAP_RANGE, 3187 },
	{ 0x4621, 0x4641, PDF_CMAP_RANGE, 3193 },
	{ 0x4642, 0x4642, PDF_CMAP_SINGLE, 7760 },
	{ 0x4643, 0x4653, PDF_CMAP_RANGE, 3227 },
	{ 0x4654, 0x4654, PDF_CMAP_SINGLE, 7761 },
	{ 0x4655, 0x465a, PDF_CMAP_RANGE, 3245 },
	{ 0x465b, 0x465b, PDF_CMAP_SINGLE, 7763 },
	{ 0x465c, 0x4665, PDF_CMAP_RANGE, 3252 },
	{ 0x4666, 0x4667, PDF_CMAP_RANGE, 7766 },
	{ 0x4668, 0x4669, PDF_CMAP_RANGE, 3264 },
	{ 0x466a, 0x466a, PDF_CMAP_SINGLE, 7768 },
	{ 0x466b, 0x4675, PDF_CMAP_RANGE, 3267 },
	{ 0x4676, 0x4676, PDF_CMAP_SINGLE, 7872 },
	{ 0x4677, 0x467e, PDF_CMAP_RANGE, 3279 },
	{ 0x4721, 0x4728, PDF_CMAP_RANGE, 3287 },
	{ 0x4729, 0x472a, PDF_CMAP_TABLE, 73 },
	{ 0x472b, 0x4738, PDF_CMAP_RANGE, 3297 },
	{ 0x4739, 0x4739, PDF_CMAP_SINGLE, 7770 },
	{ 0x473a, 0x4756, PDF_CMAP_RANGE, 3312 },
	{ 0x4757, 0x4757, PDF_CMAP_SINGLE, 7771 },
	{ 0x4758, 0x4766, PDF_CMAP_RANGE, 3342 },
	{ 0x4767, 0x4769, PDF_CMAP_TABLE, 75 },
	{ 0x476a, 0x476c, PDF_CMAP_RANGE, 3360 },
	{ 0x476d, 0x476d, PDF_CMAP_SINGLE, 7774 },
	{ 0x476e, 0x477e, PDF_CMAP_RANGE, 3364 },
	{ 0x4821, 0x4823, PDF_CMAP_RANGE, 3381 },
	{ 0x4824, 0x4824, PDF_CMAP_SINGLE, 7775 },
	{ 0x4825, 0x482d, PDF_CMAP_RANGE, 3385 },
	{ 0x482e, 0x4830, PDF_CMAP_TABLE, 78 },
	{ 0x4831, 0x483f, PDF_CMAP_RANGE, 3397 },
	{ 0x4840, 0x4840, PDF_CMAP_SINGLE, 7978 },
	{ 0x4841, 0x4853, PDF_CMAP_RANGE, 3413 },
	{ 0x4854, 0x4854, PDF_CMAP_SINGLE, 7778 },
	{ 0x4855, 0x4874, PDF_CMAP_RANGE, 3433 },
	{ 0x4875, 0x4875, PDF_CMAP_SINGLE, 7780 },
	{ 0x4876, 0x487e, PDF_CMAP_RANGE, 3466 },
	{ 0x4921, 0x4921, PDF_CMAP_SINGLE, 3475 },
	{ 0x4922, 0x4923, PDF_CMAP_RANGE, 7781 },
	{ 0x4924, 0x492e, PDF_CMAP_RANGE, 3478 },
	{ 0x492f, 0x492f, PDF_CMAP_SINGLE, 7783 },
	{ 0x4930, 0x4931, PDF_CMAP_RANGE, 3490 },
	{ 0x4932, 0x4932, PDF_CMAP_SINGLE, 7784 },
	{ 0x4933, 0x4934, PDF_CMAP_RANGE, 3493 },
	{ 0x4935, 0x4935, PDF_CMAP_SINGLE, 7785 },
	{ 0x4936, 0x493f, PDF_CMAP_RANGE, 3496 },
	{ 0x4940, 0x4940, PDF_CMAP_SINGLE, 7786 },
	{ 0x4941, 0x494d, PDF_CMAP_RANGE, 3507 },
	{ 0x494e, 0x494e, PDF_CMAP_SINGLE, 7787 },
	{ 0x494f, 0x497e, PDF_CMAP_RANGE, 3521 },
	{ 0x4a21, 0x4a42, PDF_CMAP_RANGE, 3569 },
	{ 0x4a43, 0x4a43, PDF_CMAP_SINGLE, 7789 },
	{ 0x4a44, 0x4a4c, PDF_CMAP_RANGE, 3604 },
	{ 0x4a4d, 0x4a4d, PDF_CMAP_SINGLE, 7790 },
	{ 0x4a4e, 0x4a52, PDF_CMAP_RANGE, 3614 },
	{ 0x4a53, 0x4a53, PDF_CMAP_SINGLE, 7979 },
	{ 0x4a54, 0x4a59, PDF_CMAP_RANGE, 3620 },
	{ 0x4a5a, 0x4a5a, PDF_CMAP_SINGLE, 7791 },
	{ 0x4a5b, 0x4a78, PDF_CMAP_RANGE, 3627 },
	{ 0x4a79, 0x4a79, PDF_CMAP_SINGLE, 7792 },
	{ 0x4a7a, 0x4a7e, PDF_CMAP_RANGE, 3658 },
	{ 0x4b21, 0x4b28, PDF_CMAP_RANGE, 3663 },
	{ 0x4b29, 0x4b29, PDF_CMAP_SINGLE, 7794 },
	{ 0x4b2a, 0x4b4a, PDF_CMAP_RANGE, 3672 },
	{ 0x4b4b, 0x4b4b, PDF_CMAP_SINGLE, 7795 },
	{ 0x4b4c, 0x4b6f, PDF_CMAP_RANGE, 3706 },
	{ 0x4b70, 0x4b70, PDF_CMAP_SINGLE, 7796 },
	{ 0x4b71, 0x4b77, PDF_CMAP_RANGE, 3743 },
	{ 0x4b78, 0x4b78, PDF_CMAP_SINGLE, 7980 },
	{ 0x4b79, 0x4b7e, PDF_CMAP_RANGE, 3751 },
	{ 0x4c21, 0x4c4c, PDF_CMAP_RANGE, 3757 },
	{ 0x4c4d, 0x4c4d, PDF_CMAP_SINGLE, 7797 },
	{ 0x4c4e, 0x4c58, PDF_CMAP_RANGE, 3802 },
	{ 0x4c59, 0x4c59, PDF_CMAP_SINGLE, 7798 },
	{ 0x4c5a, 0x4c5e, PDF_CMAP_RANGE, 3814 },
	{ 0x4c5f, 0x4c5f, PDF_CMAP_SINGLE, 7799 },
	{ 0x4c60, 0x4c61, PDF_CMAP_RANGE, 3820 },
	{ 0x4c62, 0x4c62, PDF_CMAP_SINGLE, 7800 },
	{ 0x4c63, 0x4c79, PDF_CMAP_RANGE, 3823 },
	{ 0x4c7a, 0x4c7c, PDF_CMAP_TABLE, 81 },
	{ 0x4c7d, 0x4c7e, PDF_CMAP_RANGE, 3849 },
	{ 0x4d21, 0x4d31, PDF_CMAP_RANGE, 3851 },
	{ 0x4d32, 0x4d32, PDF_CMAP_SINGLE, 7804 },
	{ 0x4d33, 0x4d4f, PDF_CMAP_RANGE, 3869 },
	{ 0x4d50, 0x4d50, PDF_CMAP_SINGLE, 7805 },
	{ 0x4d51, 0x4d53, PDF_CMAP_RANGE, 3899 },
	{ 0x4d54, 0x4d54, PDF_CMAP_SINGLE, 7806 },
	{ 0x4d55, 0x4d68, PDF_CMAP_RANGE, 3903 },
	{ 0x4d69, 0x4d69, PDF_CMAP_SINGLE, 7807 },
	{ 0x4d6a, 0x4d7e, PDF_CMAP_RANGE, 3924 },
	{ 0x4e21, 0x4e79, PDF_CMAP_RANGE, 3945 },
	{ 0x4e7a, 0x4e7b, PDF_CMAP_RANGE, 7809 },
	{ 0x4e7c, 0x4e7c, PDF_CMAP_SINGLE, 7981 },
	{ 0x4e7d, 0x4e7e, PDF_CMAP_RANGE, 4037 },
	{ 0x4f21, 0x4f21, PDF_CMAP_SINGLE, 7811 },
	{ 0x4f22, 0x4f30, PDF_CMAP_RANGE, 4040 },
	{ 0x4f31, 0x4f31, PDF_CMAP_SINGLE, 7812 },
	{ 0x4f32, 0x4f38, PDF_CMAP_RANGE, 4056 },
	{ 0x4f39, 0x4f39, PDF_CMAP_SINGLE, 7813 },
	{ 0x4f3a, 0x4f53, PDF_CMAP_RANGE, 4064 },
	{ 0x5021, 0x507e, PDF_CMAP_RANGE, 4090 },
	{ 0x5121, 0x5122, PDF_CMAP_TABLE, 84 },
	{ 0x5123, 0x513c, PDF_CMAP_RANGE, 4186 },
	{ 0x513d, 0x513d, PDF_CMAP_SINGLE, 7814 },
	{ 0x513e, 0x514c, PDF_CMAP_RANGE, 4213 },
	{ 0x514d, 0x514d, PDF_CMAP_SINGLE, 7817 },
	{ 0x514e, 0x517e, PDF_CMAP_RANGE, 4229 },
	{ 0x5221, 0x5237, PDF_CMAP_RANGE, 4278 },
	{ 0x5238, 0x5238, PDF_CMAP_SINGLE, 7983 },
	{ 0x5239, 0x527e, PDF_CMAP_RANGE, 4302 },
	{ 0x5321, 0x532f, PDF_CMAP_RANGE, 4372 },
	{ 0x5330, 0x5330, PDF_CMAP_SINGLE, 7818 },
	{ 0x5331, 0x5339, PDF_CMAP_RANGE, 4388 },
	{ 0x533a, 0x533a, PDF_CMAP_SINGLE, 7819 },
	{ 0x533b, 0x5347, PDF_CMAP_RANGE, 4398 },
	{ 0x5348, 0x5348, PDF_CMAP_SINGLE, 7984 },
	{ 0x5349, 0x535d, PDF_CMAP_RANGE, 4412 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 7821 },
	{ 0x535f, 0x536a, PDF_CMAP_RANGE, 4434 },
	{ 0x536b, 0x536c, PDF_CMAP_TABLE, 86 },
	{ 0x536d, 0x537e, PDF_CMAP_RANGE, 4448 },
	{ 0x5421, 0x5443, PDF_CMAP_RANGE, 4466 },
	{ 0x5444, 0x5445, PDF_CMAP_TABLE, 88 },
	{ 0x5446, 0x546b, PDF_CMAP_RANGE, 4503 },
	{ 0x546c, 0x546c, PDF_CMAP_SINGLE, 7987 },
	{ 0x546d, 0x547e, PDF_CMAP_RANGE, 4542 },
	{ 0x5521, 0x553c, PDF_CMAP_RANGE, 4560 },
	{ 0x553d, 0x553d, PDF_CMAP_SINGLE, 7824 },
	{ 0x553e, 0x5562, PDF_CMAP_RANGE, 4589 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 7825 },
	{ 0x5564, 0x5577, PDF_CMAP_RANGE, 4627 },
	{ 0x5578, 0x5578, PDF_CMAP_SINGLE, 7988 },
	{ 0x5579, 0x557e, PDF_CMAP_RANGE, 4648 },
	{ 0x5621, 0x5622, PDF_CMAP_TABLE, 90 },
	{ 0x5623, 0x567c, PDF_CMAP_RANGE, 4656 },
	{ 0x567d, 0x567e, PDF_CMAP_RANGE, 7989 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 4748 },
	{ 0x5821, 0x5823, PDF_CMAP_RANGE, 4842 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 7828 },
	{ 0x5825, 0x587e, PDF_CMAP_RANGE, 4846 },
	{ 0x5921, 0x5927, PDF_CMAP_RANGE, 4936 },
	{ 0x5928, 0x5928, PDF_CMAP_SINGLE, 7991 },
	{ 0x5929, 0x595f, PDF_CMAP_RANGE, 4944 },
	{ 0x5960, 0x5960, PDF_CMAP_SINGLE, 7829 },
	{ 0x5961, 0x596b, PDF_CMAP_RANGE, 5000 },
	{ 0x596c, 0x596c, PDF_CMAP_SINGLE, 7830 },
	{ 0x596d, 0x597e, PDF_CMAP_RANGE, 5012 },
	{ 0x5a21, 0x5a38, PDF_CMAP_RANGE, 5030 },
	{ 0x5a39, 0x5a39, PDF_CMAP_SINGLE, 7831 },
	{ 0x5a3a, 0x5a79, PDF_CMAP_RANGE, 5055 },
	{ 0x5a7a, 0x5a7a, PDF_CMAP_SINGLE, 7992 },
	{ 0x5a7b, 0x5a7e, PDF_CMAP_RANGE, 5120 },
	{ 0x5b21, 0x5b44, PDF_CMAP_RANGE, 5124 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 7833 },
	{ 0x5b46, 0x5b6a, PDF_CMAP_RANGE, 5161 },
	{ 0x5b6b, 0x5b6b, PDF_CMAP_SINGLE, 7835 },
	{ 0x5b6c, 0x5b7e, PDF_CMAP_RANGE, 5199 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5218 },
	{ 0x5d21, 0x5d60, PDF_CMAP_RANGE, 5312 },
	{ 0x5d61, 0x5d61, PDF_CMAP_SINGLE, 7993 },
	{ 0x5d62, 0x5d7e, PDF_CMAP_RANGE, 5377 },
	{ 0x5e21, 0x5e4f, PDF_CMAP_RANGE, 5406 },
	{ 0x5e50, 0x5e50, PDF_CMAP_SINGLE, 7837 },
	{ 0x5e51, 0x5e55, PDF_CMAP_RANGE, 5454 },
	{ 0x5e56, 0x5e56, PDF_CMAP_SINGLE, 7994 },
	{ 0x5e57, 0x5e75, PDF_CMAP_RANGE, 5460 },
	{ 0x5e76, 0x5e76, PDF_CMAP_SINGLE, 7995 },
	{ 0x5e77, 0x5e7e, PDF_CMAP_RANGE, 5492 },
	{ 0x5f21, 0x5f72, PDF_CMAP_RANGE, 5500 },
	{ 0x5f73, 0x5f73, PDF_CMAP_SINGLE, 7838 },
	{ 0x5f74, 0x5f7e, PDF_CMAP_RANGE, 5583 },
	{ 0x6021, 0x6025, PDF_CMAP_RANGE, 5594 },
	{ 0x6026, 0x6026, PDF_CMAP_SINGLE, 7839 },
	{ 0x6027, 0x605e, PDF_CMAP_RANGE, 5600 },
	{ 0x605f, 0x605f, PDF_CMAP_SINGLE, 7840 },
	{ 0x6060, 0x607e, PDF_CMAP_RANGE, 5657 },
	{ 0x6121, 0x612a, PDF_CMAP_RANGE, 5688 },
	{ 0x612b, 0x612b, PDF_CMAP_SINGLE, 7841 },
	{ 0x612c, 0x617e, PDF_CMAP_RANGE, 5699 },
	{ 0x6221, 0x626e, PDF_CMAP_RANGE, 5782 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 7845 },
	{ 0x6270, 0x627e, PDF_CMAP_RANGE, 5861 },
	{ 0x6321, 0x6349, PDF_CMAP_RANGE, 5876 },
	{ 0x634a, 0x634a, PDF_CMAP_SINGLE, 7846 },
	{ 0x634b, 0x6353, PDF_CMAP_RANGE, 5918 },
	{ 0x6354, 0x6354, PDF_CMAP_SINGLE, 7847 },
	{ 0x6355, 0x6358, PDF_CMAP_RANGE, 5928 },
	{ 0x6359, 0x6359, PDF_CMAP_SINGLE, 7996 },
	{ 0x635a, 0x637e, PDF_CMAP_RANGE, 5933 },
	{ 0x6421, 0x6438, PDF_CMAP_RANGE, 5970 },
	{ 0x6439, 0x6439, PDF_CMAP_SINGLE, 7848 },
	{ 0x643a, 0x643f, PDF_CMAP_RANGE, 5995 },
	{ 0x6440, 0x6440, PDF_CMAP_SINGLE, 7997 },
	{ 0x6441, 0x6463, PDF_CMAP_RANGE, 6002 },
	{ 0x6464, 0x6464, PDF_CMAP_SINGLE, 7849 },
	{ 0x6465, 0x646d, PDF_CMAP_RANGE, 6038 },
	{ 0x646e, 0x646e, PDF_CMAP_SINGLE, 7850 },
	{ 0x646f, 0x647e, PDF_CMAP_RANGE, 6048 },
	{ 0x6521, 0x6538, PDF_CMAP_RANGE, 6064 },
	{ 0x6539, 0x653b, PDF_CMAP_TABLE, 92 },
	{ 0x653c, 0x6545, PDF_CMAP_RANGE, 6091 },
	{ 0x6546, 0x6546, PDF_CMAP_SINGLE, 7853 },
	{ 0x6547, 0x657e, PDF_CMAP_RANGE, 6102 },
	{ 0x6621, 0x6648, PDF_CMAP_RANGE, 6158 },
	{ 0x6649, 0x6649, PDF_CMAP_SINGLE, 7998 },
	{ 0x664a, 0x667e, PDF_CMAP_RANGE, 6199 },
	{ 0x6721, 0x6763, PDF_CMAP_RANGE, 6252 },
	{ 0x6764, 0x6764, PDF_CMAP_SINGLE, 7855 },
	{ 0x6765, 0x6768, PDF_CMAP_RANGE, 6320 },
	{ 0x6769, 0x6769, PDF_CMAP_SINGLE, 7856 },
	{ 0x676a, 0x6771, PDF_CMAP_RANGE, 6325 },
	{ 0x6772, 0x6772, PDF_CMAP_SINGLE, 7857 },
	{ 0x6773, 0x677e, PDF_CMAP_RANGE, 6334 },
	{ 0x6821, 0x683a, PDF_CMAP_RANGE, 6346 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 7859 },
	{ 0x683c, 0x684c, PDF_CMAP_RANGE, 6373 },
	{ 0x684d, 0x684d, PDF_CMAP_SINGLE, 7999 },
	{ 0x684e, 0x687e, PDF_CMAP_RANGE, 6391 },
	{ 0x6921, 0x697d, PDF_CMAP_RANGE, 6440 },
	{ 0x697e, 0x697e, PDF_CMAP_SINGLE, 8000 },
	{ 0x6a21, 0x6a3b, PDF_CMAP_RANGE, 6534 },
	{ 0x6a3c, 0x6a3d, PDF_CMAP_RANGE, 8001 },
	{ 0x6a3e, 0x6a6e, PDF_CMAP_RANGE, 6563 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 7864 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6613 },
	{ 0x6b21, 0x6b31, PDF_CMAP_RANGE, 6628 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 7865 },
	{ 0x6b33, 0x6b65, PDF_CMAP_RANGE, 6646 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 7866 },
	{ 0x6b67, 0x6b7e, PDF_CMAP_RANGE, 6698 },
	{ 0x6c21, 0x6c68, PDF_CMAP_RANGE, 6722 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 7868 },
	{ 0x6c6a, 0x6c7e, PDF_CMAP_RANGE, 6795 },
	{ 0x6d21, 0x6d4d, PDF_CMAP_RANGE, 6816 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 7870 },
	{ 0x6d4f, 0x6d7e, PDF_CMAP_RANGE, 6862 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e7e, PDF_CMAP_RANGE, 6939 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 7004 },
	{ 0x7021, 0x7050, PDF_CMAP_RANGE, 7098 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x707e, PDF_CMAP_RANGE, 7147 },
	{ 0x7121, 0x7158, PDF_CMAP_RANGE, 7192 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 8003 },
	{ 0x715a, 0x717e, PDF_CMAP_RANGE, 7249 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724d, PDF_CMAP_RANGE, 7314 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 7884 },
	{ 0x724f, 0x727e, PDF_CMAP_RANGE, 7332 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7352, PDF_CMAP_TABLE, 95 },
	{ 0x7353, 0x737c, PDF_CMAP_RANGE, 7430 },
	{ 0x737d, 0x737e, PDF_CMAP_TABLE, 97 },
	{ 0x7421, 0x7424, PDF_CMAP_RANGE, 7474 },
	{ 0x7425, 0x7426, PDF_CMAP_RANGE, 8284 },
	{ 0x7721, 0x7723, PDF_CMAP_RANGE, 8005 },
	{ 0x7727, 0x772a, PDF_CMAP_TABLE, 99 },
	{ 0x772e, 0x7738, PDF_CMAP_RANGE, 8009 },
	{ 0x773c, 0x773e, PDF_CMAP_RANGE, 7601 },
	{ 0x773f, 0x7740, PDF_CMAP_TABLE, 103 },
	{ 0x7741, 0x7743, PDF_CMAP_RANGE, 8021 },
	{ 0x7744, 0x7746, PDF_CMAP_RANGE, 7604 },
	{ 0x7747, 0x7750, PDF_CMAP_RANGE, 8024 },
	{ 0x7751, 0x7753, PDF_CMAP_TABLE, 105 },
	{ 0x7754, 0x7755, PDF_CMAP_RANGE, 8035 },
	{ 0x7757, 0x7757, PDF_CMAP_SINGLE, 8037 },
	{ 0x7759, 0x775c, PDF_CMAP_TABLE, 108 },
	{ 0x775d, 0x775f, PDF_CMAP_RANGE, 8039 },
	{ 0x7760, 0x7766, PDF_CMAP_TABLE, 112 },
	{ 0x7767, 0x7768, PDF_CMAP_RANGE, 8044 },
	{ 0x7769, 0x7769, PDF_CMAP_SINGLE, 7599 },
	{ 0x776a, 0x7770, PDF_CMAP_RANGE, 8046 },
	{ 0x7774, 0x7779, PDF_CMAP_RANGE, 8053 },
	{ 0x777a, 0x777b, PDF_CMAP_TABLE, 119 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 8060 },
	{ 0x7829, 0x7832, PDF_CMAP_RANGE, 8061 },
	{ 0x7834, 0x7847, PDF_CMAP_RANGE, 8071 },
	{ 0x7849, 0x785c, PDF_CMAP_RANGE, 7555 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 8091 },
	{ 0x785e, 0x7867, PDF_CMAP_RANGE, 7575 },
	{ 0x786b, 0x787e, PDF_CMAP_RANGE, 8092 },
	{ 0x7921, 0x793a, PDF_CMAP_RANGE, 8112 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 7958 },
	{ 0x7949, 0x794a, PDF_CMAP_RANGE, 8138 },
	{ 0x794b, 0x794b, PDF_CMAP_SINGLE, 7620 },
	{ 0x794c, 0x794e, PDF_CMAP_RANGE, 8140 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 7619 },
	{ 0x7950, 0x7954, PDF_CMAP_RANGE, 8143 },
	{ 0x7955, 0x7955, PDF_CMAP_SINGLE, 7618 },
	{ 0x7956, 0x7959, PDF_CMAP_RANGE, 8148 },
	{ 0x795d, 0x796a, PDF_CMAP_RANGE, 8152 },
	{ 0x796f, 0x797e, PDF_CMAP_RANGE, 8166 },
	{ 0x7d21, 0x7d22, PDF_CMAP_RANGE, 7887 },
	{ 0x7d23, 0x7d24, PDF_CMAP_TABLE, 121 },
	{ 0x7d25, 0x7d2e, PDF_CMAP_RANGE, 7889 },
	{ 0x7d2f, 0x7d32, PDF_CMAP_TABLE, 123 },
	{ 0x7d33, 0x7d44, PDF_CMAP_RANGE, 7899 },
	{ 0x7d45, 0x7d5a, PDF_CMAP_RANGE, 7918 },
	{ 0x7d5b, 0x7d5c, PDF_CMAP_RANGE, 8264 },
	{ 0x7d6d, 0x7d70, PDF_CMAP_RANGE, 736 },
	{ 0x7d71, 0x7d74, PDF_CMAP_RANGE, 8182 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e7e, PDF_CMAP_RANGE, 6939 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 7004 },
	{ 0x7021, 0x7050, PDF_CMAP_RANGE, 7098 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x707e, PDF_CMAP_RANGE, 7147 },
	{ 0x7121, 0x7158, PDF_CMAP_RANGE, 7192 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 8003 },
	{ 0x715a, 0x717e, PDF_CMAP_RANGE, 7249 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724d, PDF_CMAP_RANGE, 7314 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 7884 },
	{ 0x724f, 0x727e, PDF_CMAP_RANGE, 7332 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 8004 },
	{ 0x7353, 0x737c, PDF_CMAP_RANGE, 7430 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 7886 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 7473 },
	{ 0x7421, 0x7424, PDF_CMAP_RANGE, 7474 },
	{ 0x7425, 0x7426, PDF_CMAP_RANGE, 8284 },
	{ 0x7721, 0x7723, PDF_CMAP_RANGE, 8005 },
	{ 0x7727, 0x7727, PDF_CMAP_SINGLE, 8008 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 768 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 762 },
	{ 0x772a, 0x772a, PDF_CMAP_SINGLE, 761 },
	{ 0x772e, 0x7738, PDF_CMAP_RANGE, 8009 },
	{ 0x773c, 0x773e, PDF_CMAP_RANGE, 7601 },
	{ 0x773f, 0x773f, PDF_CMAP_SINGLE, 8020 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 7607 },
	{ 0x7741, 0x7743, PDF_CMAP_RANGE, 8021 },
	{ 0x7744, 0x7746, PDF_CMAP_RANGE, 7604 },
	{ 0x7747, 0x7750, PDF_CMAP_RANGE, 8024 },
	{ 0x7751, 0x7751, PDF_CMAP_SINGLE, 771 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 8034 },
	{ 0x7753, 0x7753, PDF_CMAP_SINGLE, 772 },
	{ 0x7754, 0x7755, PDF_CMAP_RANGE, 8035 },
	{ 0x7757, 0x7757, PDF_CMAP_SINGLE, 8037 },
	{ 0x7759, 0x7759, PDF_CMAP_SINGLE, 7588 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 7585 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 8038 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 7586 },
	{ 0x775d, 0x775f, PDF_CMAP_RANGE, 8039 },
	{ 0x7760, 0x7760, PDF_CMAP_SINGLE, 7590 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 8042 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 7592 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 7596 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 8043 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 7598 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 7595 },
	{ 0x7767, 0x7768, PDF_CMAP_RANGE, 8044 },
	{ 0x7769, 0x7769, PDF_CMAP_SINGLE, 7599 },
	{ 0x776a, 0x7770, PDF_CMAP_RANGE, 8046 },
	{ 0x7774, 0x7779, PDF_CMAP_RANGE, 8053 },
	{ 0x777a, 0x777a, PDF_CMAP_SINGLE, 7610 },
	{ 0x777b, 0x777b, PDF_CMAP_SINGLE, 8059 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 8060 },
	{ 0x7829, 0x7832, PDF_CMAP_RANGE, 8061 },
	{ 0x7834, 0x7847, PDF_CMAP_RANGE, 8071 },
	{ 0x7849, 0x785c, PDF_CMAP_RANGE, 7555 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 8091 },
	{ 0x785e, 0x7867, PDF_CMAP_RANGE, 7575 },
	{ 0x786b, 0x787e, PDF_CMAP_RANGE, 8092 },
	{ 0x7921, 0x793a, PDF_CMAP_RANGE, 8112 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 7958 },
	{ 0x7949, 0x794a, PDF_CMAP_RANGE, 8138 },
	{ 0x794b, 0x794b, PDF_CMAP_SINGLE, 7620 },
	{ 0x794c, 0x794e, PDF_CMAP_RANGE, 8140 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 7619 },
	{ 0x7950, 0x7954, PDF_CMAP_RANGE, 8143 },
	{ 0x7955, 0x7955, PDF_CMAP_SINGLE, 7618 },
	{ 0x7956, 0x7959, PDF_CMAP_RANGE, 8148 },
	{ 0x795d, 0x796a, PDF_CMAP_RANGE, 8152 },
	{ 0x796f, 0x797e, PDF_CMAP_RANGE, 8166 },
	{ 0x7d21, 0x7d22, PDF_CMAP_RANGE, 7887 },
	{ 0x7d23, 0x7d23, PDF_CMAP_SINGLE, 8268 },
	{ 0x7d24, 0x7d24, PDF_CMAP_SINGLE, 8274 },
	{ 0x7d25, 0x7d2e, PDF_CMAP_RANGE, 7889 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 8282 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 8275 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 8280 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 8277 },
	{ 0x7d33, 0x7d44, PDF_CMAP_RANGE, 7899 },
	{ 0x7d45, 0x7d5a, PDF_CMAP_RANGE, 7918 },
	{ 0x7d5b, 0x7d5c, PDF_CMAP_RANGE, 8264 },
	{ 0x7d6d, 0x7d70, PDF_CMAP_RANGE, 736 },
	{ 0x7d71, 0x7d74, PDF_CMAP_RANGE, 8182 },
};

static const unsigned short pdf_cmap_Add_H_table[127] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
	1125,7633,7962,1206,7636,7963,1238,7639,
	7645,1321,7646,1407,7650,1689,7660,7665,
	1780,7666,1782,7969,7676,7689,2169,7690,
	2629,7713,7718,7972,2719,7733,2910,7734,
	2918,7735,7744,3045,7745,3099,7748,7976,
	7756,7769,7977,7772,3358,7773,7776,3395,
	7777,7801,3847,7802,4184,7982,7822,7985,
	7823,7986,4654,7826,7851,6089,7852,7885,
	8004,7886,7473,8008,768,762,761,8020,
	7607,771,8034,772,7588,7585,8038,7586,
	7590,8042,7592,7596,8043,7598,7595,7610,
	8059,8268,8274,8282,8275,8280,8277,
};

pdf_cmap pdf_cmap_Add_H =
{
	-1, "Add-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	631, 631, (pdf_range*) pdf_cmap_Add_H_ranges,
	127, 127, (unsigned short*) pdf_cmap_Add_H_table,
};

/* Add-RKSJ-H */

static const pdf_range pdf_cmap_Add_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x82f2, 0x82f4, PDF_CMAP_RANGE, 7958 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 0 },
	{ 0x889f, 0x88a0, PDF_CMAP_TABLE, 32 },
	{ 0x88a1, 0x88af, PDF_CMAP_RANGE, 1127 },
	{ 0x88b0, 0x88b0, PDF_CMAP_SINGLE, 7961 },
	{ 0x88b1, 0x88b8, PDF_CMAP_RANGE, 1143 },
	{ 0x88b9, 0x88b9, PDF_CMAP_SINGLE, 7634 },
	{ 0x88ba, 0x88eb, PDF_CMAP_RANGE, 1152 },
	{ 0x88ec, 0x88ec, PDF_CMAP_SINGLE, 7635 },
	{ 0x88ed, 0x88ee, PDF_CMAP_RANGE, 1203 },
	{ 0x88ef, 0x88f1, PDF_CMAP_TABLE, 34 },
	{ 0x88f2, 0x88f9, PDF_CMAP_RANGE, 1208 },
	{ 0x88fa, 0x88fa, PDF_CMAP_SINGLE, 7637 },
	{ 0x88fb, 0x88fc, PDF_CMAP_RANGE, 1217 },
	{ 0x8940, 0x8948, PDF_CMAP_RANGE, 1219 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 7638 },
	{ 0x894a, 0x8951, PDF_CMAP_RANGE, 1229 },
	{ 0x8952, 0x8954, PDF_CMAP_TABLE, 37 },
	{ 0x8955, 0x8957, PDF_CMAP_RANGE, 1240 },
	{ 0x8958, 0x8958, PDF_CMAP_SINGLE, 7964 },
	{ 0x8959, 0x895b, PDF_CMAP_RANGE, 1244 },
	{ 0x895c, 0x895c, PDF_CMAP_SINGLE, 7642 },
	{ 0x895d, 0x8960, PDF_CMAP_RANGE, 1248 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 7643 },
	{ 0x8962, 0x897e, PDF_CMAP_RANGE, 1253 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 1282 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 7644 },
	{ 0x898c, 0x89a5, PDF_CMAP_RANGE, 1294 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 40 },
	{ 0x89a9, 0x89dd, PDF_CMAP_RANGE, 1323 },
	{ 0x89de, 0x89de, PDF_CMAP_SINGLE, 7647 },
	{ 0x89df, 0x89e4, PDF_CMAP_RANGE, 1377 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 7965 },
	{ 0x89e6, 0x89f7, PDF_CMAP_RANGE, 1384 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7648 },
	{ 0x89f9, 0x89fc, PDF_CMAP_RANGE, 1403 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 43 },
	{ 0x8a42, 0x8a7e, PDF_CMAP_RANGE, 1409 },
	{ 0x8a80, 0x8a8a, PDF_CMAP_RANGE, 1470 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 7652 },
	{ 0x8a8c, 0x8a92, PDF_CMAP_RANGE, 1482 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 7653 },
	{ 0x8a94, 0x8a99, PDF_CMAP_RANGE, 1490 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 7654 },
	{ 0x8a9b, 0x8abf, PDF_CMAP_RANGE, 1497 },
	{ 0x8ac0, 0x8ac0, PDF_CMAP_SINGLE, 7655 },
	{ 0x8ac1, 0x8aca, PDF_CMAP_RANGE, 1535 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 7656 },
	{ 0x8acc, 0x8ae2, PDF_CMAP_RANGE, 1546 },
	{ 0x8ae3, 0x8ae3, PDF_CMAP_SINGLE, 7657 },
	{ 0x8ae4, 0x8afc, PDF_CMAP_RANGE, 1570 },
	{ 0x8b40, 0x8b49, PDF_CMAP_RANGE, 1595 },
	{ 0x8b4a, 0x8b4a, PDF_CMAP_SINGLE, 7658 },
	{ 0x8b4b, 0x8b5e, PDF_CMAP_RANGE, 1606 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 7659 },
	{ 0x8b60, 0x8b7e, PDF_CMAP_RANGE, 1627 },
	{ 0x8b80, 0x8b81, PDF_CMAP_RANGE, 1658 },
	{ 0x8b82, 0x8b82, PDF_CMAP_SINGLE, 7966 },
	{ 0x8b83, 0x8b87, PDF_CMAP_RANGE, 1661 },
	{ 0x8b88, 0x8b88, PDF_CMAP_SINGLE, 7967 },
	{ 0x8b89, 0x8b9f, PDF_CMAP_RANGE, 1667 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 7660 },
	{ 0x8ba1, 0x8ba7, PDF_CMAP_RANGE, 1691 },
	{ 0x8ba8, 0x8ba8, PDF_CMAP_SINGLE, 7661 },
	{ 0x8ba9, 0x8bbf, PDF_CMAP_RANGE, 1699 },
	{ 0x8bc0, 0x8bc0, PDF_CMAP_SINGLE, 7968 },
	{ 0x8bc1, 0x8bcc, PDF_CMAP_RANGE, 1723 },
	{ 0x8bcd, 0x8bcd, PDF_CMAP_SINGLE, 7662 },
	{ 0x8bce, 0x8bea, PDF_CMAP_RANGE, 1736 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 7663 },
	{ 0x8bec, 0x8bf1, PDF_CMAP_RANGE, 1766 },
	{ 0x8bf2, 0x8bf2, PDF_CMAP_SINGLE, 7664 },
	{ 0x8bf3, 0x8bf8, PDF_CMAP_RANGE, 1773 },
	{ 0x8bf9, 0x8bfc, PDF_CMAP_TABLE, 45 },
	{ 0x8c40, 0x8c55, PDF_CMAP_RANGE, 1783 },
	{ 0x8c56, 0x8c56, PDF_CMAP_SINGLE, 7668 },
	{ 0x8c57, 0x8c70, PDF_CMAP_RANGE, 1806 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 7671 },
	{ 0x8c72, 0x8c7e, PDF_CMAP_RANGE, 1833 },
	{ 0x8c80, 0x8c90, PDF_CMAP_RANGE, 1846 },
	{ 0x8c91, 0x8c91, PDF_CMAP_SINGLE, 7674 },
	{ 0x8c92, 0x8c9c, PDF_CMAP_RANGE, 1864 },
	{ 0x8c9d, 0x8c9e, PDF_CMAP_TABLE, 49 },
	{ 0x8c9f, 0x8cb1, PDF_CMAP_RANGE, 1877 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 7677 },
	{ 0x8cb3, 0x8cbe, PDF_CMAP_RANGE, 1897 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 7678 },
	{ 0x8cc0, 0x8cfc, PDF_CMAP_RANGE, 1910 },
	{ 0x8d40, 0x8d49, PDF_CMAP_RANGE, 1971 },
	{ 0x8d4a, 0x8d4a, PDF_CMAP_SINGLE, 7679 },
	{ 0x8d4b, 0x8d7e, PDF_CMAP_RANGE, 1982 },
	{ 0x8d80, 0x8d8c, PDF_CMAP_RANGE, 2034 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 7682 },
	{ 0x8d8e, 0x8d93, PDF_CMAP_RANGE, 2048 },
	{ 0x8d94, 0x8d94, PDF_CMAP_SINGLE, 7683 },
	{ 0x8d95, 0x8d98, PDF_CMAP_RANGE, 2055 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 7684 },
	{ 0x8d9a, 0x8dd0, PDF_CMAP_RANGE, 2060 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 7685 },
	{ 0x8dd2, 0x8de4, PDF_CMAP_RANGE, 2116 },
	{ 0x8de5, 0x8de5, PDF_CMAP_SINGLE, 7686 },
	{ 0x8de6, 0x8df1, PDF_CMAP_RANGE, 2136 },
	{ 0x8df2, 0x8df2, PDF_CMAP_SINGLE, 7687 },
	{ 0x8df3, 0x8dfc, PDF_CMAP_RANGE, 2149 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 2159 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 7688 },
	{ 0x8e47, 0x8e48, PDF_CMAP_RANGE, 2166 },
	{ 0x8e49, 0x8e4b, PDF_CMAP_TABLE, 51 },
	{ 0x8e4c, 0x8e57, PDF_CMAP_RANGE, 2171 },
	{ 0x8e58, 0x8e58, PDF_CMAP_SINGLE, 7691 },
	{ 0x8e59, 0x8e5f, PDF_CMAP_RANGE, 2184 },
	{ 0x8e60, 0x8e60, PDF_CMAP_SINGLE, 7970 },
	{ 0x8e61, 0x8e7e, PDF_CMAP_RANGE, 2192 },
	{ 0x8e80, 0x8ec5, PDF_CMAP_RANGE, 2222 },
	{ 0x8ec6, 0x8ec6, PDF_CMAP_SINGLE, 7693 },
	{ 0x8ec7, 0x8eda, PDF_CMAP_RANGE, 2293 },
	{ 0x8edb, 0x8edc, PDF_CMAP_RANGE, 7695 },
	{ 0x8edd, 0x8efc, PDF_CMAP_RANGE, 2315 },
	{ 0x8f40, 0x8f49, PDF_CMAP_RANGE, 2347 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 7697 },
	{ 0x8f4b, 0x8f54, PDF_CMAP_RANGE, 2358 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 7698 },
	{ 0x8f56, 0x8f7e, PDF_CMAP_RANGE, 2369 },
	{ 0x8f80, 0x8f8b, PDF_CMAP_RANGE, 2410 },
	{ 0x8f8c, 0x8f8c, PDF_CMAP_SINGLE, 7699 },
	{ 0x8f8d, 0x8f91, PDF_CMAP_RANGE, 2423 },
	{ 0x8f92, 0x8f93, PDF_CMAP_RANGE, 7701 },
	{ 0x8f94, 0x8fa2, PDF_CMAP_RANGE, 2430 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 7703 },
	{ 0x8fa4, 0x8fb0, PDF_CMAP_RANGE, 2446 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 7704 },
	{ 0x8fb2, 0x8fd2, PDF_CMAP_RANGE, 2460 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 7706 },
	{ 0x8fd4, 0x8fdc, PDF_CMAP_RANGE, 2494 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 7707 },
	{ 0x8fde, 0x8fe1, PDF_CMAP_RANGE, 2504 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 7708 },
	{ 0x8fe3, 0x8ffc, PDF_CMAP_RANGE, 2509 },
	{ 0x9040, 0x9048, PDF_CMAP_RANGE, 2535 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 7709 },
	{ 0x904a, 0x9077, PDF_CMAP_RANGE, 2545 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 7971 },
	{ 0x9079, 0x907e, PDF_CMAP_RANGE, 2592 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 7711 },
	{ 0x9081, 0x909f, PDF_CMAP_RANGE, 2599 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 7713 },
	{ 0x90a1, 0x90e3, PDF_CMAP_RANGE, 2631 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 7715 },
	{ 0x90e5, 0x90ee, PDF_CMAP_RANGE, 2699 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 7716 },
	{ 0x90f0, 0x90f6, PDF_CMAP_RANGE, 2710 },
	{ 0x90f7, 0x90f9, PDF_CMAP_TABLE, 54 },
	{ 0x90fa, 0x90fb, PDF_CMAP_RANGE, 7973 },
	{ 0x90fc, 0x90fc, PDF_CMAP_SINGLE, 2722 },
	{ 0x9140, 0x9145, PDF_CMAP_RANGE, 2723 },
	{ 0x9146, 0x9146, PDF_CMAP_SINGLE, 7720 },
	{ 0x9147, 0x9157, PDF_CMAP_RANGE, 2730 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 7721 },
	{ 0x9159, 0x916a, PDF_CMAP_RANGE, 2748 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 7722 },
	{ 0x916c, 0x916d, PDF_CMAP_RANGE, 2767 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 7723 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 2770 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 7724 },
	{ 0x9180, 0x9188, PDF_CMAP_RANGE, 2786 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 7725 },
	{ 0x918a, 0x91b4, PDF_CMAP_RANGE, 2796 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 7975 },
	{ 0x91b6, 0x91ba, PDF_CMAP_RANGE, 2840 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 7726 },
	{ 0x91bc, 0x91ca, PDF_CMAP_RANGE, 2846 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 7727 },
	{ 0x91cc, 0x91d9, PDF_CMAP_RANGE, 2862 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 7728 },
	{ 0x91db, 0x91e0, PDF_CMAP_RANGE, 2877 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 7729 },
	{ 0x91e2, 0x91ec, PDF_CMAP_RANGE, 2884 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 7730 },
	{ 0x91ee, 0x91fa, PDF_CMAP_RANGE, 2896 },
	{ 0x91fb, 0x91fc, PDF_CMAP_TABLE, 57 },
	{ 0x9240, 0x9245, PDF_CMAP_RANGE, 2911 },
	{ 0x9246, 0x9248, PDF_CMAP_TABLE, 59 },
	{ 0x9249, 0x924b, PDF_CMAP_RANGE, 2920 },
	{ 0x924c, 0x924d, PDF_CMAP_RANGE, 7737 },
	{ 0x924e, 0x925b, PDF_CMAP_RANGE, 2925 },
	{ 0x925c, 0x925c, PDF_CMAP_SINGLE, 7739 },
	{ 0x925d, 0x927e, PDF_CMAP_RANGE, 2940 },
	{ 0x9280, 0x928f, PDF_CMAP_RANGE, 2974 },
	{ 0x9290, 0x9290, PDF_CMAP_SINGLE, 7740 },
	{ 0x9291, 0x9294, PDF_CMAP_RANGE, 2991 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 7741 },
	{ 0x9296, 0x929b, PDF_CMAP_RANGE, 2996 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 7742 },
	{ 0x929d, 0x92ba, PDF_CMAP_RANGE, 3003 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 7743 },
	{ 0x92bc, 0x92c5, PDF_CMAP_RANGE, 3034 },
	{ 0x92c6, 0x92c8, PDF_CMAP_TABLE, 62 },
	{ 0x92c9, 0x92cc, PDF_CMAP_RANGE, 3047 },
	{ 0x92cd, 0x92cd, PDF_CMAP_SINGLE, 7747 },
	{ 0x92ce, 0x92fc, PDF_CMAP_RANGE, 3052 },
	{ 0x9340, 0x9341, PDF_CMAP_TABLE, 65 },
	{ 0x9342, 0x9345, PDF_CMAP_RANGE, 3101 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 7749 },
	{ 0x9347, 0x934c, PDF_CMAP_RANGE, 3106 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 7750 },
	{ 0x934e, 0x9354, PDF_CMAP_RANGE, 3113 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 7751 },
	{ 0x9356, 0x935d, PDF_CMAP_RANGE, 3121 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 7752 },
	{ 0x935f, 0x9366, PDF_CMAP_RANGE, 3130 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 7753 },
	{ 0x9368, 0x9369, PDF_CMAP_RANGE, 3139 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 7754 },
	{ 0x936b, 0x936f, PDF_CMAP_RANGE, 3142 },
	{ 0x9370, 0x9371, PDF_CMAP_TABLE, 67 },
	{ 0x9372, 0x937e, PDF_CMAP_RANGE, 3149 },
	{ 0x9380, 0x9383, PDF_CMAP_RANGE, 3162 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 7757 },
	{ 0x9385, 0x9397, PDF_CMAP_RANGE, 3167 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 7758 },
	{ 0x9399, 0x93bf, PDF_CMAP_RANGE, 3187 },
	{ 0x93c0, 0x93c0, PDF_CMAP_SINGLE, 7760 },
	{ 0x93c1, 0x93d1, PDF_CMAP_RANGE, 3227 },
	{ 0x93d2, 0x93d2, PDF_CMAP_SINGLE, 7761 },
	{ 0x93d3, 0x93d8, PDF_CMAP_RANGE, 3245 },
	{ 0x93d9, 0x93d9, PDF_CMAP_SINGLE, 7763 },
	{ 0x93da, 0x93e3, PDF_CMAP_RANGE, 3252 },
	{ 0x93e4, 0x93e5, PDF_CMAP_RANGE, 7766 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 3264 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7768 },
	{ 0x93e9, 0x93f3, PDF_CMAP_RANGE, 3267 },
	{ 0x93f4, 0x93f4, PDF_CMAP_SINGLE, 7872 },
	{ 0x93f5, 0x93fc, PDF_CMAP_RANGE, 3279 },
	{ 0x9440, 0x9447, PDF_CMAP_RANGE, 3287 },
	{ 0x9448, 0x9449, PDF_CMAP_TABLE, 69 },
	{ 0x944a, 0x9457, PDF_CMAP_RANGE, 3297 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 7770 },
	{ 0x9459, 0x9475, PDF_CMAP_RANGE, 3312 },
	{ 0x9476, 0x9476, PDF_CMAP_SINGLE, 7771 },
	{ 0x9477, 0x947e, PDF_CMAP_RANGE, 3342 },
	{ 0x9480, 0x9486, PDF_CMAP_RANGE, 3350 },
	{ 0x9487, 0x9489, PDF_CMAP_TABLE, 71 },
	{ 0x948a, 0x948c, PDF_CMAP_RANGE, 3360 },
	{ 0x948d, 0x948d, PDF_CMAP_SINGLE, 7774 },
	{ 0x948e, 0x94a1, PDF_CMAP_RANGE, 3364 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 7775 },
	{ 0x94a3, 0x94ab, PDF_CMAP_RANGE, 3385 },
	{ 0x94ac, 0x94ae, PDF_CMAP_TABLE, 74 },
	{ 0x94af, 0x94bd, PDF_CMAP_RANGE, 3397 },
	{ 0x94be, 0x94be, PDF_CMAP_SINGLE, 7978 },
	{ 0x94bf, 0x94d1, PDF_CMAP_RANGE, 3413 },
	{ 0x94d2, 0x94d2, PDF_CMAP_SINGLE, 7778 },
	{ 0x94d3, 0x94f2, PDF_CMAP_RANGE, 3433 },
	{ 0x94f3, 0x94f3, PDF_CMAP_SINGLE, 7780 },
	{ 0x94f4, 0x94fc, PDF_CMAP_RANGE, 3466 },
	{ 0x9540, 0x9540, PDF_CMAP_SINGLE, 3475 },
	{ 0x9541, 0x9542, PDF_CMAP_RANGE, 7781 },
	{ 0x9543, 0x954d, PDF_CMAP_RANGE, 3478 },
	{ 0x954e, 0x954e, PDF_CMAP_SINGLE, 7783 },
	{ 0x954f, 0x9550, PDF_CMAP_RANGE, 3490 },
	{ 0x9551, 0x9551, PDF_CMAP_SINGLE, 7784 },
	{ 0x9552, 0x9553, PDF_CMAP_RANGE, 3493 },
	{ 0x9554, 0x9554, PDF_CMAP_SINGLE, 7785 },
	{ 0x9555, 0x955e, PDF_CMAP_RANGE, 3496 },
	{ 0x955f, 0x955f, PDF_CMAP_SINGLE, 7786 },
	{ 0x9560, 0x956c, PDF_CMAP_RANGE, 3507 },
	{ 0x956d, 0x956d, PDF_CMAP_SINGLE, 7787 },
	{ 0x956e, 0x957e, PDF_CMAP_RANGE, 3521 },
	{ 0x9580, 0x95c0, PDF_CMAP_RANGE, 3538 },
	{ 0x95c1, 0x95c1, PDF_CMAP_SINGLE, 7789 },
	{ 0x95c2, 0x95ca, PDF_CMAP_RANGE, 3604 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 7790 },
	{ 0x95cc, 0x95d0, PDF_CMAP_RANGE, 3614 },
	{ 0x95d1, 0x95d1, PDF_CMAP_SINGLE, 7979 },
	{ 0x95d2, 0x95d7, PDF_CMAP_RANGE, 3620 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 7791 },
	{ 0x95d9, 0x95f6, PDF_CMAP_RANGE, 3627 },
	{ 0x95f7, 0x95f7, PDF_CMAP_SINGLE, 7792 },
	{ 0x95f8, 0x95fc, PDF_CMAP_RANGE, 3658 },
	{ 0x9640, 0x9647, PDF_CMAP_RANGE, 3663 },
	{ 0x9648, 0x9648, PDF_CMAP_SINGLE, 7794 },
	{ 0x9649, 0x9669, PDF_CMAP_RANGE, 3672 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 7795 },
	{ 0x966b, 0x967e, PDF_CMAP_RANGE, 3706 },
	{ 0x9680, 0x968f, PDF_CMAP_RANGE, 3726 },
	{ 0x9690, 0x9690, PDF_CMAP_SINGLE, 7796 },
	{ 0x9691, 0x9697, PDF_CMAP_RANGE, 3743 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 7980 },
	{ 0x9699, 0x96ca, PDF_CMAP_RANGE, 3751 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 7797 },
	{ 0x96cc, 0x96d6, PDF_CMAP_RANGE, 3802 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 7798 },
	{ 0x96d8, 0x96dc, PDF_CMAP_RANGE, 3814 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 7799 },
	{ 0x96de, 0x96df, PDF_CMAP_RANGE, 3820 },
	{ 0x96e0, 0x96e0, PDF_CMAP_SINGLE, 7800 },
	{ 0x96e1, 0x96f7, PDF_CMAP_RANGE, 3823 },
	{ 0x96f8, 0x96fa, PDF_CMAP_TABLE, 77 },
	{ 0x96fb, 0x96fc, PDF_CMAP_RANGE, 3849 },
	{ 0x9740, 0x9750, PDF_CMAP_RANGE, 3851 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 7804 },
	{ 0x9752, 0x976e, PDF_CMAP_RANGE, 3869 },
	{ 0x976f, 0x976f, PDF_CMAP_SINGLE, 7805 },
	{ 0x9770, 0x9772, PDF_CMAP_RANGE, 3899 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 7806 },
	{ 0x9774, 0x977e, PDF_CMAP_RANGE, 3903 },
	{ 0x9780, 0x9788, PDF_CMAP_RANGE, 3914 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 7807 },
	{ 0x978a, 0x97f7, PDF_CMAP_RANGE, 3924 },
	{ 0x97f8, 0x97f9, PDF_CMAP_RANGE, 7809 },
	{ 0x97fa, 0x97fa, PDF_CMAP_SINGLE, 7981 },
	{ 0x97fb, 0x97fc, PDF_CMAP_RANGE, 4037 },
	{ 0x9840, 0x9840, PDF_CMAP_SINGLE, 7811 },
	{ 0x9841, 0x984f, PDF_CMAP_RANGE, 4040 },
	{ 0x9850, 0x9850, PDF_CMAP_SINGLE, 7812 },
	{ 0x9851, 0x9857, PDF_CMAP_RANGE, 4056 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 7813 },
	{ 0x9859, 0x9872, PDF_CMAP_RANGE, 4064 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x9941, PDF_CMAP_TABLE, 80 },
	{ 0x9942, 0x995b, PDF_CMAP_RANGE, 4186 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 7814 },
	{ 0x995d, 0x996b, PDF_CMAP_RANGE, 4213 },
	{ 0x996c, 0x996c, PDF_CMAP_SINGLE, 7817 },
	{ 0x996d, 0x997e, PDF_CMAP_RANGE, 4229 },
	{ 0x9980, 0x99b5, PDF_CMAP_RANGE, 4247 },
	{ 0x99b6, 0x99b6, PDF_CMAP_SINGLE, 7983 },
	{ 0x99b7, 0x99fc, PDF_CMAP_RANGE, 4302 },
	{ 0x9a40, 0x9a4e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 7818 },
	{ 0x9a50, 0x9a58, PDF_CMAP_RANGE, 4388 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 7819 },
	{ 0x9a5a, 0x9a66, PDF_CMAP_RANGE, 4398 },
	{ 0x9a67, 0x9a67, PDF_CMAP_SINGLE, 7984 },
	{ 0x9a68, 0x9a7c, PDF_CMAP_RANGE, 4412 },
	{ 0x9a7d, 0x9a7e, PDF_CMAP_TABLE, 82 },
	{ 0x9a80, 0x9a8a, PDF_CMAP_RANGE, 4435 },
	{ 0x9a8b, 0x9a8c, PDF_CMAP_TABLE, 84 },
	{ 0x9a8d, 0x9ac1, PDF_CMAP_RANGE, 4448 },
	{ 0x9ac2, 0x9ac3, PDF_CMAP_TABLE, 86 },
	{ 0x9ac4, 0x9ae9, PDF_CMAP_RANGE, 4503 },
	{ 0x9aea, 0x9aea, PDF_CMAP_SINGLE, 7987 },
	{ 0x9aeb, 0x9afc, PDF_CMAP_RANGE, 4542 },
	{ 0x9b40, 0x9b5b, PDF_CMAP_RANGE, 4560 },
	{ 0x9b5c, 0x9b5c, PDF_CMAP_SINGLE, 7824 },
	{ 0x9b5d, 0x9b7e, PDF_CMAP_RANGE, 4589 },
	{ 0x9b80, 0x9b82, PDF_CMAP_RANGE, 4623 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7825 },
	{ 0x9b84, 0x9b97, PDF_CMAP_RANGE, 4627 },
	{ 0x9b98, 0x9b98, PDF_CMAP_SINGLE, 7988 },
	{ 0x9b99, 0x9b9f, PDF_CMAP_RANGE, 4648 },
	{ 0x9ba0, 0x9ba0, PDF_CMAP_SINGLE, 7826 },
	{ 0x9ba1, 0x9bfa, PDF_CMAP_RANGE, 4656 },
	{ 0x9bfb, 0x9bfc, PDF_CMAP_RANGE, 7989 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9ca1, PDF_CMAP_RANGE, 4811 },
	{ 0x9ca2, 0x9ca2, PDF_CMAP_SINGLE, 7828 },
	{ 0x9ca3, 0x9cfc, PDF_CMAP_RANGE, 4846 },
	{ 0x9d40, 0x9d46, PDF_CMAP_RANGE, 4936 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 7991 },
	{ 0x9d48, 0x9d7e, PDF_CMAP_RANGE, 4944 },
	{ 0x9d80, 0x9d80, PDF_CMAP_SINGLE, 7829 },
	{ 0x9d81, 0x9d8b, PDF_CMAP_RANGE, 5000 },
	{ 0x9d8c, 0x9d8c, PDF_CMAP_SINGLE, 7830 },
	{ 0x9d8d, 0x9db6, PDF_CMAP_RANGE, 5012 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 7831 },
	{ 0x9db8, 0x9df7, PDF_CMAP_RANGE, 5055 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 7992 },
	{ 0x9df9, 0x9dfc, PDF_CMAP_RANGE, 5120 },
	{ 0x9e40, 0x9e63, PDF_CMAP_RANGE, 5124 },
	{ 0x9e64, 0x9e64, PDF_CMAP_SINGLE, 7833 },
	{ 0x9e65, 0x9e7e, PDF_CMAP_RANGE, 5161 },
	{ 0x9e80, 0x9e8a, PDF_CMAP_RANGE, 5187 },
	{ 0x9e8b, 0x9e8b, PDF_CMAP_SINGLE, 7835 },
	{ 0x9e8c, 0x9efc, PDF_CMAP_RANGE, 5199 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9f81, PDF_CMAP_TABLE, 88 },
	{ 0x9f82, 0x9fcd, PDF_CMAP_RANGE, 5377 },
	{ 0x9fce, 0x9fce, PDF_CMAP_SINGLE, 7837 },
	{ 0x9fcf, 0x9fd3, PDF_CMAP_RANGE, 5454 },
	{ 0x9fd4, 0x9fd4, PDF_CMAP_SINGLE, 7994 },
	{ 0x9fd5, 0x9ff3, PDF_CMAP_RANGE, 5460 },
	{ 0x9ff4, 0x9ff4, PDF_CMAP_SINGLE, 7995 },
	{ 0x9ff5, 0x9ffc, PDF_CMAP_RANGE, 5492 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe092, PDF_CMAP_RANGE, 5563 },
	{ 0xe093, 0xe093, PDF_CMAP_SINGLE, 7838 },
	{ 0xe094, 0xe0a3, PDF_CMAP_RANGE, 5583 },
	{ 0xe0a4, 0xe0a4, PDF_CMAP_SINGLE, 7839 },
	{ 0xe0a5, 0xe0dc, PDF_CMAP_RANGE, 5600 },
	{ 0xe0dd, 0xe0dd, PDF_CMAP_SINGLE, 7840 },
	{ 0xe0de, 0xe0fc, PDF_CMAP_RANGE, 5657 },
	{ 0xe140, 0xe149, PDF_CMAP_RANGE, 5688 },
	{ 0xe14a, 0xe14a, PDF_CMAP_SINGLE, 7841 },
	{ 0xe14b, 0xe17e, PDF_CMAP_RANGE, 5699 },
	{ 0xe180, 0xe1ec, PDF_CMAP_RANGE, 5751 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 7845 },
	{ 0xe1ee, 0xe1fc, PDF_CMAP_RANGE, 5861 },
	{ 0xe240, 0xe268, PDF_CMAP_RANGE, 5876 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 7846 },
	{ 0xe26a, 0xe272, PDF_CMAP_RANGE, 5918 },
	{ 0xe273, 0xe273, PDF_CMAP_SINGLE, 7847 },
	{ 0xe274, 0xe277, PDF_CMAP_RANGE, 5928 },
	{ 0xe278, 0xe278, PDF_CMAP_SINGLE, 7996 },
	{ 0xe279, 0xe27e, PDF_CMAP_RANGE, 5933 },
	{ 0xe280, 0xe2b6, PDF_CMAP_RANGE, 5939 },
	{ 0xe2b7, 0xe2b7, PDF_CMAP_SINGLE, 7848 },
	{ 0xe2b8, 0xe2bd, PDF_CMAP_RANGE, 5995 },
	{ 0xe2be, 0xe2be, PDF_CMAP_SINGLE, 7997 },
	{ 0xe2bf, 0xe2e1, PDF_CMAP_RANGE, 6002 },
	{ 0xe2e2, 0xe2e2, PDF_CMAP_SINGLE, 7849 },
	{ 0xe2e3, 0xe2eb, PDF_CMAP_RANGE, 6038 },
	{ 0xe2ec, 0xe2ec, PDF_CMAP_SINGLE, 7850 },
	{ 0xe2ed, 0xe2fc, PDF_CMAP_RANGE, 6048 },
	{ 0xe340, 0xe357, PDF_CMAP_RANGE, 6064 },
	{ 0xe358, 0xe35a, PDF_CMAP_TABLE, 90 },
	{ 0xe35b, 0xe364, PDF_CMAP_RANGE, 6091 },
	{ 0xe365, 0xe365, PDF_CMAP_SINGLE, 7853 },
	{ 0xe366, 0xe37e, PDF_CMAP_RANGE, 6102 },
	{ 0xe380, 0xe3c6, PDF_CMAP_RANGE, 6127 },
	{ 0xe3c7, 0xe3c7, PDF_CMAP_SINGLE, 7998 },
	{ 0xe3c8, 0xe3fc, PDF_CMAP_RANGE, 6199 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe483, PDF_CMAP_RANGE, 6315 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 7855 },
	{ 0xe485, 0xe488, PDF_CMAP_RANGE, 6320 },
	{ 0xe489, 0xe489, PDF_CMAP_SINGLE, 7856 },
	{ 0xe48a, 0xe491, PDF_CMAP_RANGE, 6325 },
	{ 0xe492, 0xe492, PDF_CMAP_SINGLE, 7857 },
	{ 0xe493, 0xe4b8, PDF_CMAP_RANGE, 6334 },
	{ 0xe4b9, 0xe4b9, PDF_CMAP_SINGLE, 7859 },
	{ 0xe4ba, 0xe4ca, PDF_CMAP_RANGE, 6373 },
	{ 0xe4cb, 0xe4cb, PDF_CMAP_SINGLE, 7999 },
	{ 0xe4cc, 0xe4fc, PDF_CMAP_RANGE, 6391 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe59d, PDF_CMAP_RANGE, 6503 },
	{ 0xe59e, 0xe59e, PDF_CMAP_SINGLE, 8000 },
	{ 0xe59f, 0xe5b9, PDF_CMAP_RANGE, 6534 },
	{ 0xe5ba, 0xe5bb, PDF_CMAP_RANGE, 8001 },
	{ 0xe5bc, 0xe5ec, PDF_CMAP_RANGE, 6563 },
	{ 0xe5ed, 0xe5ed, PDF_CMAP_SINGLE, 7864 },
	{ 0xe5ee, 0xe5fc, PDF_CMAP_RANGE, 6613 },
	{ 0xe640, 0xe650, PDF_CMAP_RANGE, 6628 },
	{ 0xe651, 0xe651, PDF_CMAP_SINGLE, 7865 },
	{ 0xe652, 0xe67e, PDF_CMAP_RANGE, 6646 },
	{ 0xe680, 0xe685, PDF_CMAP_RANGE, 6691 },
	{ 0xe686, 0xe686, PDF_CMAP_SINGLE, 7866 },
	{ 0xe687, 0xe6e6, PDF_CMAP_RANGE, 6698 },
	{ 0xe6e7, 0xe6e7, PDF_CMAP_SINGLE, 7868 },
	{ 0xe6e8, 0xe6fc, PDF_CMAP_RANGE, 6795 },
	{ 0xe740, 0xe76c, PDF_CMAP_RANGE, 6816 },
	{ 0xe76d, 0xe76d, PDF_CMAP_SINGLE, 7870 },
	{ 0xe76e, 0xe77e, PDF_CMAP_RANGE, 6862 },
	{ 0xe780, 0xe7a6, PDF_CMAP_RANGE, 6879 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7fc, PDF_CMAP_RANGE, 6939 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8ce, PDF_CMAP_RANGE, 7067 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8fc, PDF_CMAP_RANGE, 7147 },
	{ 0xe940, 0xe977, PDF_CMAP_RANGE, 7192 },
	{ 0xe978, 0xe978, PDF_CMAP_SINGLE, 8003 },
	{ 0xe979, 0xe97e, PDF_CMAP_RANGE, 7249 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9cb, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 7884 },
	{ 0xe9cd, 0xe9fc, PDF_CMAP_RANGE, 7332 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea71, PDF_CMAP_TABLE, 93 },
	{ 0xea72, 0xea7e, PDF_CMAP_RANGE, 7430 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9d, PDF_CMAP_SINGLE, 7886 },
	{ 0xea9e, 0xeaa2, PDF_CMAP_RANGE, 7473 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xec40, 0xec42, PDF_CMAP_RANGE, 8005 },
	{ 0xec46, 0xec49, PDF_CMAP_TABLE, 95 },
	{ 0xec4d, 0xec57, PDF_CMAP_RANGE, 8009 },
	{ 0xec5b, 0xec5d, PDF_CMAP_RANGE, 7601 },
	{ 0xec5e, 0xec5f, PDF_CMAP_TABLE, 99 },
	{ 0xec60, 0xec62, PDF_CMAP_RANGE, 8021 },
	{ 0xec63, 0xec65, PDF_CMAP_RANGE, 7604 },
	{ 0xec66, 0xec6f, PDF_CMAP_RANGE, 8024 },
	{ 0xec70, 0xec72, PDF_CMAP_TABLE, 101 },
	{ 0xec73, 0xec74, PDF_CMAP_RANGE, 8035 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8037 },
	{ 0xec78, 0xec7b, PDF_CMAP_TABLE, 104 },
	{ 0xec7c, 0xec7e, PDF_CMAP_RANGE, 8039 },
	{ 0xec80, 0xec86, PDF_CMAP_TABLE, 108 },
	{ 0xec87, 0xec88, PDF_CMAP_RANGE, 8044 },
	{ 0xec89, 0xec89, PDF_CMAP_SINGLE, 7599 },
	{ 0xec8a, 0xec90, PDF_CMAP_RANGE, 8046 },
	{ 0xec94, 0xec99, PDF_CMAP_RANGE, 8053 },
	{ 0xec9a, 0xec9b, PDF_CMAP_TABLE, 115 },
	{ 0xec9e, 0xec9e, PDF_CMAP_SINGLE, 8060 },
	{ 0xeca7, 0xecb0, PDF_CMAP_RANGE, 8061 },
	{ 0xecb2, 0xecc5, PDF_CMAP_RANGE, 8071 },
	{ 0xecc7, 0xecda, PDF_CMAP_RANGE, 7555 },
	{ 0xecdb, 0xecdb, PDF_CMAP_SINGLE, 8091 },
	{ 0xecdc, 0xece5, PDF_CMAP_RANGE, 7575 },
	{ 0xece9, 0xecfc, PDF_CMAP_RANGE, 8092 },
	{ 0xed40, 0xed59, PDF_CMAP_RANGE, 8112 },
	{ 0xed64, 0xed64, PDF_CMAP_SINGLE, 7958 },
	{ 0xed68, 0xed69, PDF_CMAP_RANGE, 8138 },
	{ 0xed6a, 0xed6a, PDF_CMAP_SINGLE, 7620 },
	{ 0xed6b, 0xed6d, PDF_CMAP_RANGE, 8140 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 7619 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 8143 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 7618 },
	{ 0xed75, 0xed78, PDF_CMAP_RANGE, 8148 },
	{ 0xed7c, 0xed7e, PDF_CMAP_RANGE, 8152 },
	{ 0xed80, 0xed8a, PDF_CMAP_RANGE, 8155 },
	{ 0xed8f, 0xed9e, PDF_CMAP_RANGE, 8166 },
	{ 0xef40, 0xef41, PDF_CMAP_RANGE, 7887 },
	{ 0xef42, 0xef43, PDF_CMAP_TABLE, 117 },
	{ 0xef44, 0xef4d, PDF_CMAP_RANGE, 7889 },
	{ 0xef4e, 0xef51, PDF_CMAP_TABLE, 119 },
	{ 0xef52, 0xef63, PDF_CMAP_RANGE, 7899 },
	{ 0xef64, 0xef79, PDF_CMAP_RANGE, 7918 },
	{ 0xef7a, 0xef7b, PDF_CMAP_RANGE, 8264 },
	{ 0xef8d, 0xef90, PDF_CMAP_RANGE, 736 },
	{ 0xef91, 0xef94, PDF_CMAP_RANGE, 8182 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8ce, PDF_CMAP_RANGE, 7067 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8fc, PDF_CMAP_RANGE, 7147 },
	{ 0xe940, 0xe977, PDF_CMAP_RANGE, 7192 },
	{ 0xe978, 0xe978, PDF_CMAP_SINGLE, 8003 },
	{ 0xe979, 0xe97e, PDF_CMAP_RANGE, 7249 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9cb, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 7884 },
	{ 0xe9cd, 0xe9fc, PDF_CMAP_RANGE, 7332 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea71, PDF_CMAP_SINGLE, 8004 },
	{ 0xea72, 0xea7e, PDF_CMAP_RANGE, 7430 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9d, PDF_CMAP_SINGLE, 7886 },
	{ 0xea9e, 0xeaa2, PDF_CMAP_RANGE, 7473 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0xec40, 0xec42, PDF_CMAP_RANGE, 8005 },
	{ 0xec46, 0xec46, PDF_CMAP_SINGLE, 8008 },
	{ 0xec47, 0xec47, PDF_CMAP_SINGLE, 768 },
	{ 0xec48, 0xec48, PDF_CMAP_SINGLE, 762 },
	{ 0xec49, 0xec49, PDF_CMAP_SINGLE, 761 },
	{ 0xec4d, 0xec57, PDF_CMAP_RANGE, 8009 },
	{ 0xec5b, 0xec5d, PDF_CMAP_RANGE, 7601 },
	{ 0xec5e, 0xec5e, PDF_CMAP_SINGLE, 8020 },
	{ 0xec5f, 0xec5f, PDF_CMAP_SINGLE, 7607 },
	{ 0xec60, 0xec62, PDF_CMAP_RANGE, 8021 },
	{ 0xec63, 0xec65, PDF_CMAP_RANGE, 7604 },
	{ 0xec66, 0xec6f, PDF_CMAP_RANGE, 8024 },
	{ 0xec70, 0xec70, PDF_CMAP_SINGLE, 771 },
	{ 0xec71, 0xec71, PDF_CMAP_SINGLE, 8034 },
	{ 0xec72, 0xec72, PDF_CMAP_SINGLE, 772 },
	{ 0xec73, 0xec74, PDF_CMAP_RANGE, 8035 },
	{ 0xec76, 0xec76, PDF_CMAP_SINGLE, 8037 },
	{ 0xec78, 0xec78, PDF_CMAP_SINGLE, 7588 },
	{ 0xec79, 0xec79, PDF_CMAP_SINGLE, 7585 },
	{ 0xec7a, 0xec7a, PDF_CMAP_SINGLE, 8038 },
	{ 0xec7b, 0xec7b, PDF_CMAP_SINGLE, 7586 },
	{ 0xec7c, 0xec7e, PDF_CMAP_RANGE, 8039 },
	{ 0xec80, 0xec80, PDF_CMAP_SINGLE, 7590 },
	{ 0xec81, 0xec81, PDF_CMAP_SINGLE, 8042 },
	{ 0xec82, 0xec82, PDF_CMAP_SINGLE, 7592 },
	{ 0xec83, 0xec83, PDF_CMAP_SINGLE, 7596 },
	{ 0xec84, 0xec84, PDF_CMAP_SINGLE, 8043 },
	{ 0xec85, 0xec85, PDF_CMAP_SINGLE, 7598 },
	{ 0xec86, 0xec86, PDF_CMAP_SINGLE, 7595 },
	{ 0xec87, 0xec88, PDF_CMAP_RANGE, 8044 },
	{ 0xec89, 0xec89, PDF_CMAP_SINGLE, 7599 },
	{ 0xec8a, 0xec90, PDF_CMAP_RANGE, 8046 },
	{ 0xec94, 0xec99, PDF_CMAP_RANGE, 8053 },
	{ 0xec9a, 0xec9a, PDF_CMAP_SINGLE, 7610 },
	{ 0xec9b, 0xec9b, PDF_CMAP_SINGLE, 8059 },
	{ 0xec9e, 0xec9e, PDF_CMAP_SINGLE, 8060 },
	{ 0xeca7, 0xecb0, PDF_CMAP_RANGE, 8061 },
	{ 0xecb2, 0xecc5, PDF_CMAP_RANGE, 8071 },
	{ 0xecc7, 0xecda, PDF_CMAP_RANGE, 7555 },
	{ 0xecdb, 0xecdb, PDF_CMAP_SINGLE, 8091 },
	{ 0xecdc, 0xece5, PDF_CMAP_RANGE, 7575 },
	{ 0xece9, 0xecfc, PDF_CMAP_RANGE, 8092 },
	{ 0xed40, 0xed59, PDF_CMAP_RANGE, 8112 },
	{ 0xed64, 0xed64, PDF_CMAP_SINGLE, 7958 },
	{ 0xed68, 0xed69, PDF_CMAP_RANGE, 8138 },
	{ 0xed6a, 0xed6a, PDF_CMAP_SINGLE, 7620 },
	{ 0xed6b, 0xed6d, PDF_CMAP_RANGE, 8140 },
	{ 0xed6e, 0xed6e, PDF_CMAP_SINGLE, 7619 },
	{ 0xed6f, 0xed73, PDF_CMAP_RANGE, 8143 },
	{ 0xed74, 0xed74, PDF_CMAP_SINGLE, 7618 },
	{ 0xed75, 0xed78, PDF_CMAP_RANGE, 8148 },
	{ 0xed7c, 0xed7e, PDF_CMAP_RANGE, 8152 },
	{ 0xed80, 0xed8a, PDF_CMAP_RANGE, 8155 },
	{ 0xed8f, 0xed9e, PDF_CMAP_RANGE, 8166 },
	{ 0xef40, 0xef41, PDF_CMAP_RANGE, 7887 },
	{ 0xef42, 0xef42, PDF_CMAP_SINGLE, 8268 },
	{ 0xef43, 0xef43, PDF_CMAP_SINGLE, 8274 },
	{ 0xef44, 0xef4d, PDF_CMAP_RANGE, 7889 },
	{ 0xef4e, 0xef4e, PDF_CMAP_SINGLE, 8282 },
	{ 0xef4f, 0xef4f, PDF_CMAP_SINGLE, 8275 },
	{ 0xef50, 0xef50, PDF_CMAP_SINGLE, 8280 },
	{ 0xef51, 0xef51, PDF_CMAP_SINGLE, 8277 },
	{ 0xef52, 0xef63, PDF_CMAP_RANGE, 7899 },
	{ 0xef64, 0xef79, PDF_CMAP_RANGE, 7918 },
	{ 0xef7a, 0xef7b, PDF_CMAP_RANGE, 8264 },
	{ 0xef8d, 0xef90, PDF_CMAP_RANGE, 736 },
	{ 0xef91, 0xef94, PDF_CMAP_RANGE, 8182 },
};

static const unsigned short pdf_cmap_Add_RKSJ_H_table[123] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
	1125,7633,7962,1206,7636,7963,1238,7639,
	7645,1321,7646,1407,7650,7665,1780,7666,
	1782,7969,7676,7689,2169,7690,7718,7972,
	2719,7733,2910,7734,2918,7735,7744,3045,
	7745,3099,7748,7976,7756,7769,7977,7772,
	3358,7773,7776,3395,7777,7801,3847,7802,
	4184,7982,7821,4434,7822,7985,7823,7986,
	5375,7993,7851,6089,7852,7885,8004,8008,
	768,762,761,8020,7607,771,8034,772,
	7588,7585,8038,7586,7590,8042,7592,7596,
	8043,7598,7595,7610,8059,8268,8274,8282,
	8275,8280,8277,
};

pdf_cmap pdf_cmap_Add_RKSJ_H =
{
	-1, "Add-RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	635, 635, (pdf_range*) pdf_cmap_Add_RKSJ_H_ranges,
	123, 123, (unsigned short*) pdf_cmap_Add_RKSJ_H_table,
};

/* Add-RKSJ-V */

static const pdf_range pdf_cmap_Add_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8143, 0x8144, PDF_CMAP_TABLE, 0 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8165, 0x8168, PDF_CMAP_TABLE, 2 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x82f3, 0x82f4, PDF_CMAP_RANGE, 8264 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0xec78, 0xec7c, PDF_CMAP_TABLE, 6 },
	{ 0xec7d, 0xec7e, PDF_CMAP_RANGE, 8339 },
	{ 0xec80, 0xec8a, PDF_CMAP_TABLE, 11 },
	{ 0xec8b, 0xec8c, PDF_CMAP_RANGE, 8349 },
	{ 0xec8d, 0xec90, PDF_CMAP_TABLE, 22 },
	{ 0xec95, 0xec95, PDF_CMAP_SINGLE, 8324 },
	{ 0xef92, 0xef92, PDF_CMAP_SINGLE, 8333 },
	{ 0xec7b, 0xec7b, PDF_CMAP_SINGLE, 7941 },
	{ 0xec7c, 0xec7c, PDF_CMAP_SINGLE, 8330 },
	{ 0xec7d, 0xec7e, PDF_CMAP_RANGE, 8339 },
	{ 0xec80, 0xec80, PDF_CMAP_SINGLE, 7945 },
	{ 0xec81, 0xec81, PDF_CMAP_SINGLE, 8338 },
	{ 0xec82, 0xec82, PDF_CMAP_SINGLE, 7947 },
	{ 0xec83, 0xec83, PDF_CMAP_SINGLE, 7951 },
	{ 0xec84, 0xec84, PDF_CMAP_SINGLE, 8348 },
	{ 0xec85, 0xec85, PDF_CMAP_SINGLE, 7953 },
	{ 0xec86, 0xec86, PDF_CMAP_SINGLE, 7950 },
	{ 0xec87, 0xec87, PDF_CMAP_SINGLE, 8344 },
	{ 0xec88, 0xec88, PDF_CMAP_SINGLE, 8347 },
	{ 0xec89, 0xec89, PDF_CMAP_SINGLE, 7954 },
	{ 0xec8a, 0xec8a, PDF_CMAP_SINGLE, 8343 },
	{ 0xec8b, 0xec8c, PDF_CMAP_RANGE, 8349 },
	{ 0xec8d, 0xec8d, PDF_CMAP_SINGLE, 8358 },
	{ 0xec8e, 0xec8e, PDF_CMAP_SINGLE, 8357 },
	{ 0xec8f, 0xec8f, PDF_CMAP_SINGLE, 8353 },
	{ 0xec90, 0xec90, PDF_CMAP_SINGLE, 8356 },
	{ 0xec95, 0xec95, PDF_CMAP_SINGLE, 8324 },
	{ 0xef92, 0xef92, PDF_CMAP_SINGLE, 8333 },
};

static const unsigned short pdf_cmap_Add_RKSJ_V_table[26] =
{
	8268,8274,8282,8275,8280,8277,7943,7940,
	8329,7941,8330,7945,8338,7947,7951,8348,
	7953,7950,8344,8347,7954,8343,8358,8357,
	8353,8356,
};

pdf_cmap pdf_cmap_Add_RKSJ_V =
{
	-1, "Add-RKSJ-V", "Add-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	57, 57, (pdf_range*) pdf_cmap_Add_RKSJ_V_ranges,
	26, 26, (unsigned short*) pdf_cmap_Add_RKSJ_V_table,
};

/* Add-V */

static const pdf_range pdf_cmap_Add_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_RANGE, 7887 },
	{ 0x2124, 0x2125, PDF_CMAP_TABLE, 0 },
	{ 0x2131, 0x2132, PDF_CMAP_RANGE, 7889 },
	{ 0x213c, 0x213e, PDF_CMAP_RANGE, 7891 },
	{ 0x2141, 0x2145, PDF_CMAP_RANGE, 7894 },
	{ 0x2146, 0x2149, PDF_CMAP_TABLE, 2 },
	{ 0x214a, 0x215b, PDF_CMAP_RANGE, 7899 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 7918 },
	{ 0x2423, 0x2423, PDF_CMAP_SINGLE, 7919 },
	{ 0x2425, 0x2425, PDF_CMAP_SINGLE, 7920 },
	{ 0x2427, 0x2427, PDF_CMAP_SINGLE, 7921 },
	{ 0x2429, 0x2429, PDF_CMAP_SINGLE, 7922 },
	{ 0x2443, 0x2443, PDF_CMAP_SINGLE, 7923 },
	{ 0x2463, 0x2463, PDF_CMAP_SINGLE, 7924 },
	{ 0x2465, 0x2465, PDF_CMAP_SINGLE, 7925 },
	{ 0x2467, 0x2467, PDF_CMAP_SINGLE, 7926 },
	{ 0x246e, 0x246e, PDF_CMAP_SINGLE, 7927 },
	{ 0x2475, 0x2476, PDF_CMAP_RANGE, 8264 },
	{ 0x2521, 0x2521, PDF_CMAP_SINGLE, 7928 },
	{ 0x2523, 0x2523, PDF_CMAP_SINGLE, 7929 },
	{ 0x2525, 0x2525, PDF_CMAP_SINGLE, 7930 },
	{ 0x2527, 0x2527, PDF_CMAP_SINGLE, 7931 },
	{ 0x2529, 0x2529, PDF_CMAP_SINGLE, 7932 },
	{ 0x2543, 0x2543, PDF_CMAP_SINGLE, 7933 },
	{ 0x2563, 0x2563, PDF_CMAP_SINGLE, 7934 },
	{ 0x2565, 0x2565, PDF_CMAP_SINGLE, 7935 },
	{ 0x2567, 0x2567, PDF_CMAP_SINGLE, 7936 },
	{ 0x256e, 0x256e, PDF_CMAP_SINGLE, 7937 },
	{ 0x2575, 0x2576, PDF_CMAP_RANGE, 7938 },
	{ 0x7759, 0x775d, PDF_CMAP_TABLE, 6 },
	{ 0x775e, 0x775f, PDF_CMAP_RANGE, 8339 },
	{ 0x7760, 0x776a, PDF_CMAP_TABLE, 11 },
	{ 0x776b, 0x776c, PDF_CMAP_RANGE, 8349 },
	{ 0x776d, 0x7770, PDF_CMAP_TABLE, 22 },
	{ 0x7775, 0x7775, PDF_CMAP_SINGLE, 8324 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 8333 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 7941 },
	{ 0x775d, 0x775d, PDF_CMAP_SINGLE, 8330 },
	{ 0x775e, 0x775f, PDF_CMAP_RANGE, 8339 },
	{ 0x7760, 0x7760, PDF_CMAP_SINGLE, 7945 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 8338 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 7947 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 7951 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 8348 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 7953 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 7950 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 8344 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 8347 },
	{ 0x7769, 0x7769, PDF_CMAP_SINGLE, 7954 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 8343 },
	{ 0x776b, 0x776c, PDF_CMAP_RANGE, 8349 },
	{ 0x776d, 0x776d, PDF_CMAP_SINGLE, 8358 },
	{ 0x776e, 0x776e, PDF_CMAP_SINGLE, 8357 },
	{ 0x776f, 0x776f, PDF_CMAP_SINGLE, 8353 },
	{ 0x7770, 0x7770, PDF_CMAP_SINGLE, 8356 },
	{ 0x7775, 0x7775, PDF_CMAP_SINGLE, 8324 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 8333 },
};

static const unsigned short pdf_cmap_Add_V_table[26] =
{
	8268,8274,8282,8275,8280,8277,7943,7940,
	8329,7941,8330,7945,8338,7947,7951,8348,
	7953,7950,8344,8347,7954,8343,8358,8357,
	8353,8356,
};

pdf_cmap pdf_cmap_Add_V =
{
	-1, "Add-V", "Add-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	57, 57, (pdf_range*) pdf_cmap_Add_V_ranges,
	26, 26, (unsigned short*) pdf_cmap_Add_V_table,
};

/* Adobe-Japan1-0 */

static const pdf_range pdf_cmap_Adobe_Japan1_0_ranges[] =
{
	{ 0x0000, 0x205b, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x205b, PDF_CMAP_RANGE, 8192 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_0_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_0 =
{
	-1, "Adobe-Japan1-0", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x20ff },
	},
	33, 33, (pdf_range*) pdf_cmap_Adobe_Japan1_0_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_0_table,
};

/* Adobe-Japan1-1 */

static const pdf_range pdf_cmap_Adobe_Japan1_1_ranges[] =
{
	{ 0x0000, 0x20a6, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20a6, PDF_CMAP_RANGE, 8192 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_1_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_1 =
{
	-1, "Adobe-Japan1-1", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x20ff },
	},
	33, 33, (pdf_range*) pdf_cmap_Adobe_Japan1_1_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_1_table,
};

/* Adobe-Japan1-2 */

static const pdf_range pdf_cmap_Adobe_Japan1_2_ranges[] =
{
	{ 0x0000, 0x220f, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x220f, PDF_CMAP_RANGE, 8704 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_2_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_2 =
{
	-1, "Adobe-Japan1-2", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x22ff },
	},
	35, 35, (pdf_range*) pdf_cmap_Adobe_Japan1_2_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_2_table,
};

/* Adobe-Japan1-3 */

static const pdf_range pdf_cmap_Adobe_Japan1_3_ranges[] =
{
	{ 0x0000, 0x2489, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x2489, PDF_CMAP_RANGE, 9216 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_3_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_3 =
{
	-1, "Adobe-Japan1-3", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x24ff },
	},
	37, 37, (pdf_range*) pdf_cmap_Adobe_Japan1_3_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_3_table,
};

/* Adobe-Japan1-4 */

static const pdf_range pdf_cmap_Adobe_Japan1_4_ranges[] =
{
	{ 0x0000, 0x3c53, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3c53, PDF_CMAP_RANGE, 15360 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_4_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_4 =
{
	-1, "Adobe-Japan1-4", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x3cff },
	},
	61, 61, (pdf_range*) pdf_cmap_Adobe_Japan1_4_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_4_table,
};

/* Adobe-Japan1-5 */

static const pdf_range pdf_cmap_Adobe_Japan1_5_ranges[] =
{
	{ 0x0000, 0x4f5c, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4aff, PDF_CMAP_RANGE, 18944 },
	{ 0x4b00, 0x4bff, PDF_CMAP_RANGE, 19200 },
	{ 0x4c00, 0x4cff, PDF_CMAP_RANGE, 19456 },
	{ 0x4d00, 0x4dff, PDF_CMAP_RANGE, 19712 },
	{ 0x4e00, 0x4eff, PDF_CMAP_RANGE, 19968 },
	{ 0x4f00, 0x4f5c, PDF_CMAP_RANGE, 20224 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_5_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_5 =
{
	-1, "Adobe-Japan1-5", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x4fff },
	},
	80, 80, (pdf_range*) pdf_cmap_Adobe_Japan1_5_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_5_table,
};

/* Adobe-Japan1-6 */

static const pdf_range pdf_cmap_Adobe_Japan1_6_ranges[] =
{
	{ 0x0000, 0x5a11, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47ff, PDF_CMAP_RANGE, 18176 },
	{ 0x4800, 0x48ff, PDF_CMAP_RANGE, 18432 },
	{ 0x4900, 0x49ff, PDF_CMAP_RANGE, 18688 },
	{ 0x4a00, 0x4aff, PDF_CMAP_RANGE, 18944 },
	{ 0x4b00, 0x4bff, PDF_CMAP_RANGE, 19200 },
	{ 0x4c00, 0x4cff, PDF_CMAP_RANGE, 19456 },
	{ 0x4d00, 0x4dff, PDF_CMAP_RANGE, 19712 },
	{ 0x4e00, 0x4eff, PDF_CMAP_RANGE, 19968 },
	{ 0x4f00, 0x4fff, PDF_CMAP_RANGE, 20224 },
	{ 0x5000, 0x50ff, PDF_CMAP_RANGE, 20480 },
	{ 0x5100, 0x51ff, PDF_CMAP_RANGE, 20736 },
	{ 0x5200, 0x52ff, PDF_CMAP_RANGE, 20992 },
	{ 0x5300, 0x53ff, PDF_CMAP_RANGE, 21248 },
	{ 0x5400, 0x54ff, PDF_CMAP_RANGE, 21504 },
	{ 0x5500, 0x55ff, PDF_CMAP_RANGE, 21760 },
	{ 0x5600, 0x56ff, PDF_CMAP_RANGE, 22016 },
	{ 0x5700, 0x57ff, PDF_CMAP_RANGE, 22272 },
	{ 0x5800, 0x58ff, PDF_CMAP_RANGE, 22528 },
	{ 0x5900, 0x59ff, PDF_CMAP_RANGE, 22784 },
	{ 0x5a00, 0x5a11, PDF_CMAP_RANGE, 23040 },
};

static const unsigned short pdf_cmap_Adobe_Japan1_6_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan1_6 =
{
	-1, "Adobe-Japan1-6", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x5aff },
	},
	91, 91, (pdf_range*) pdf_cmap_Adobe_Japan1_6_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan1_6_table,
};

/* EUC-H */

static const pdf_range pdf_cmap_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
	{ 0x8ea0, 0x8edf, PDF_CMAP_RANGE, 326 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 633 },
	{ 0xa2a1, 0xa2ae, PDF_CMAP_RANGE, 727 },
	{ 0xa2ba, 0xa2c1, PDF_CMAP_RANGE, 741 },
	{ 0xa2ca, 0xa2d0, PDF_CMAP_RANGE, 749 },
	{ 0xa2dc, 0xa2ea, PDF_CMAP_RANGE, 756 },
	{ 0xa2f2, 0xa2f9, PDF_CMAP_RANGE, 771 },
	{ 0xa2fe, 0xa2fe, PDF_CMAP_SINGLE, 779 },
	{ 0xa3b0, 0xa3b9, PDF_CMAP_RANGE, 780 },
	{ 0xa3c1, 0xa3da, PDF_CMAP_RANGE, 790 },
	{ 0xa3e1, 0xa3fa, PDF_CMAP_RANGE, 816 },
	{ 0xa4a1, 0xa4f3, PDF_CMAP_RANGE, 842 },
	{ 0xa5a1, 0xa5f6, PDF_CMAP_RANGE, 925 },
	{ 0xa6a1, 0xa6b8, PDF_CMAP_RANGE, 1011 },
	{ 0xa6c1, 0xa6d8, PDF_CMAP_RANGE, 1035 },
	{ 0xa7a1, 0xa7c1, PDF_CMAP_RANGE, 1059 },
	{ 0xa7d1, 0xa7f1, PDF_CMAP_RANGE, 1092 },
	{ 0xa8a1, 0xa8c0, PDF_CMAP_TABLE, 0 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 1125 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1219 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1313 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1407 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1501 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1595 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1689 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1783 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1877 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1971 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 2065 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 2159 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2253 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2347 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2441 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2535 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2629 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2723 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2817 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2911 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 3005 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 3099 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3193 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3287 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3381 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 3475 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3569 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 3663 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3757 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 3851 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3945 },
	{ 0xcfa1, 0xcfd3, PDF_CMAP_RANGE, 4039 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 4090 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 4184 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 4278 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4372 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 4466 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 4560 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 4654 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 4748 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4842 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4936 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 5030 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 5124 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5218 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5312 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5406 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5500 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5594 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5688 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5782 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5876 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5970 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 6064 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6158 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6252 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6346 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6440 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6534 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6628 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6722 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6816 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6910 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 7004 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 7098 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7192 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7286 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7380 },
	{ 0xf4a1, 0xf4a4, PDF_CMAP_RANGE, 7474 },
	{ 0xf4a5, 0xf4a6, PDF_CMAP_RANGE, 8284 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 4748 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4842 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4936 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 5030 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 5124 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 5218 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 5312 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 5406 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 5500 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 5594 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 5688 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5782 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5876 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5970 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 6064 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 6158 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 6252 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 6346 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 6440 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 6534 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 6628 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 6722 },
	{ 0xeda1, 0xedfe, PDF_CMAP_RANGE, 6816 },
	{ 0xeea1, 0xeefe, PDF_CMAP_RANGE, 6910 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 7004 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 7098 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 7192 },
	{ 0xf2a1, 0xf2fe, PDF_CMAP_RANGE, 7286 },
	{ 0xf3a1, 0xf3fe, PDF_CMAP_RANGE, 7380 },
	{ 0xf4a1, 0xf4a4, PDF_CMAP_RANGE, 7474 },
	{ 0xf4a5, 0xf4a6, PDF_CMAP_RANGE, 8284 },
};

static const unsigned short pdf_cmap_EUC_H_table[32] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
};

pdf_cmap pdf_cmap_EUC_H =
{
	-1, "EUC-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8ea0, 0x8edf },
		{ 2, 0xa1a1, 0xfefe },
	},
	120, 120, (pdf_range*) pdf_cmap_EUC_H_ranges,
	32, 32, (unsigned short*) pdf_cmap_EUC_H_table,
};

/* EUC-V */

static const pdf_range pdf_cmap_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 7887 },
	{ 0xa1b1, 0xa1b2, PDF_CMAP_RANGE, 7889 },
	{ 0xa1bc, 0xa1be, PDF_CMAP_RANGE, 7891 },
	{ 0xa1c1, 0xa1c5, PDF_CMAP_RANGE, 7894 },
	{ 0xa1ca, 0xa1db, PDF_CMAP_RANGE, 7899 },
	{ 0xa1e1, 0xa1e1, PDF_CMAP_SINGLE, 7917 },
	{ 0xa4a1, 0xa4a1, PDF_CMAP_SINGLE, 7918 },
	{ 0xa4a3, 0xa4a3, PDF_CMAP_SINGLE, 7919 },
	{ 0xa4a5, 0xa4a5, PDF_CMAP_SINGLE, 7920 },
	{ 0xa4a7, 0xa4a7, PDF_CMAP_SINGLE, 7921 },
	{ 0xa4a9, 0xa4a9, PDF_CMAP_SINGLE, 7922 },
	{ 0xa4c3, 0xa4c3, PDF_CMAP_SINGLE, 7923 },
	{ 0xa4e3, 0xa4e3, PDF_CMAP_SINGLE, 7924 },
	{ 0xa4e5, 0xa4e5, PDF_CMAP_SINGLE, 7925 },
	{ 0xa4e7, 0xa4e7, PDF_CMAP_SINGLE, 7926 },
	{ 0xa4ee, 0xa4ee, PDF_CMAP_SINGLE, 7927 },
	{ 0xa5a1, 0xa5a1, PDF_CMAP_SINGLE, 7928 },
	{ 0xa5a3, 0xa5a3, PDF_CMAP_SINGLE, 7929 },
	{ 0xa5a5, 0xa5a5, PDF_CMAP_SINGLE, 7930 },
	{ 0xa5a7, 0xa5a7, PDF_CMAP_SINGLE, 7931 },
	{ 0xa5a9, 0xa5a9, PDF_CMAP_SINGLE, 7932 },
	{ 0xa5c3, 0xa5c3, PDF_CMAP_SINGLE, 7933 },
	{ 0xa5e3, 0xa5e3, PDF_CMAP_SINGLE, 7934 },
	{ 0xa5e5, 0xa5e5, PDF_CMAP_SINGLE, 7935 },
	{ 0xa5e7, 0xa5e7, PDF_CMAP_SINGLE, 7936 },
	{ 0xa5ee, 0xa5ee, PDF_CMAP_SINGLE, 7937 },
	{ 0xa5f5, 0xa5f6, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_EUC_V_table[1] = { 0 };

pdf_cmap pdf_cmap_EUC_V =
{
	-1, "EUC-V", "EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	27, 27, (pdf_range*) pdf_cmap_EUC_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_EUC_V_table,
};

/* Ext-H */

static const pdf_range pdf_cmap_Ext_H_ranges[] =
{
	{ 0x2121, 0x2168, PDF_CMAP_RANGE, 633 },
	{ 0x2169, 0x2169, PDF_CMAP_SINGLE, 7478 },
	{ 0x216a, 0x217e, PDF_CMAP_RANGE, 706 },
	{ 0x2221, 0x222e, PDF_CMAP_RANGE, 727 },
	{ 0x2330, 0x2339, PDF_CMAP_RANGE, 780 },
	{ 0x2341, 0x235a, PDF_CMAP_RANGE, 790 },
	{ 0x2361, 0x237a, PDF_CMAP_RANGE, 816 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 842 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 925 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 1011 },
	{ 0x2641, 0x2658, PDF_CMAP_RANGE, 1035 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 1059 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 1092 },
	{ 0x2921, 0x295f, PDF_CMAP_RANGE, 232 },
	{ 0x2960, 0x2960, PDF_CMAP_SINGLE, 390 },
	{ 0x2961, 0x297e, PDF_CMAP_RANGE, 296 },
	{ 0x2a21, 0x2a5f, PDF_CMAP_RANGE, 327 },
	{ 0x2a60, 0x2a7e, PDF_CMAP_RANGE, 391 },
	{ 0x2b21, 0x2b71, PDF_CMAP_RANGE, 422 },
	{ 0x2b72, 0x2b72, PDF_CMAP_SINGLE, 295 },
	{ 0x2b73, 0x2b7e, PDF_CMAP_RANGE, 503 },
	{ 0x2c24, 0x2c6f, PDF_CMAP_RANGE, 7479 },
	{ 0x2d21, 0x2d3e, PDF_CMAP_RANGE, 7555 },
	{ 0x2d40, 0x2d56, PDF_CMAP_RANGE, 7585 },
	{ 0x2d5f, 0x2d5f, PDF_CMAP_SINGLE, 8323 },
	{ 0x2d60, 0x2d6f, PDF_CMAP_RANGE, 7608 },
	{ 0x2d70, 0x2d72, PDF_CMAP_TABLE, 0 },
	{ 0x2d73, 0x2d79, PDF_CMAP_RANGE, 7624 },
	{ 0x2d7a, 0x2d7a, PDF_CMAP_SINGLE, 768 },
	{ 0x2d7b, 0x2d7c, PDF_CMAP_RANGE, 7631 },
	{ 0x3021, 0x3022, PDF_CMAP_TABLE, 3 },
	{ 0x3023, 0x3032, PDF_CMAP_RANGE, 1127 },
	{ 0x3033, 0x3033, PDF_CMAP_SINGLE, 7330 },
	{ 0x3034, 0x303a, PDF_CMAP_RANGE, 1144 },
	{ 0x303b, 0x303b, PDF_CMAP_SINGLE, 7634 },
	{ 0x303c, 0x306d, PDF_CMAP_RANGE, 1152 },
	{ 0x306e, 0x306e, PDF_CMAP_SINGLE, 7635 },
	{ 0x306f, 0x3072, PDF_CMAP_RANGE, 1203 },
	{ 0x3073, 0x3073, PDF_CMAP_SINGLE, 7636 },
	{ 0x3074, 0x307b, PDF_CMAP_RANGE, 1208 },
	{ 0x307c, 0x307c, PDF_CMAP_SINGLE, 7637 },
	{ 0x307d, 0x307e, PDF_CMAP_RANGE, 1217 },
	{ 0x3121, 0x3129, PDF_CMAP_RANGE, 1219 },
	{ 0x312a, 0x312a, PDF_CMAP_SINGLE, 7638 },
	{ 0x312b, 0x3134, PDF_CMAP_RANGE, 1229 },
	{ 0x3135, 0x3135, PDF_CMAP_SINGLE, 7639 },
	{ 0x3136, 0x3138, PDF_CMAP_RANGE, 1240 },
	{ 0x3139, 0x3139, PDF_CMAP_SINGLE, 7640 },
	{ 0x313a, 0x313b, PDF_CMAP_RANGE, 1244 },
	{ 0x313c, 0x313d, PDF_CMAP_RANGE, 7641 },
	{ 0x313e, 0x3141, PDF_CMAP_RANGE, 1248 },
	{ 0x3142, 0x3142, PDF_CMAP_SINGLE, 7643 },
	{ 0x3143, 0x316a, PDF_CMAP_RANGE, 1253 },
	{ 0x316b, 0x316b, PDF_CMAP_SINGLE, 7644 },
	{ 0x316c, 0x317e, PDF_CMAP_RANGE, 1294 },
	{ 0x3221, 0x3227, PDF_CMAP_RANGE, 1313 },
	{ 0x3228, 0x322a, PDF_CMAP_TABLE, 5 },
	{ 0x322b, 0x325f, PDF_CMAP_RANGE, 1323 },
	{ 0x3260, 0x3260, PDF_CMAP_SINGLE, 7647 },
	{ 0x3261, 0x3279, PDF_CMAP_RANGE, 1377 },
	{ 0x327a, 0x327a, PDF_CMAP_SINGLE, 7648 },
	{ 0x327b, 0x327c, PDF_CMAP_RANGE, 1403 },
	{ 0x327d, 0x327e, PDF_CMAP_TABLE, 8 },
	{ 0x3321, 0x3322, PDF_CMAP_TABLE, 10 },
	{ 0x3323, 0x3341, PDF_CMAP_RANGE, 1409 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 6497 },
	{ 0x3343, 0x3348, PDF_CMAP_RANGE, 1441 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 5023 },
	{ 0x334a, 0x3364, PDF_CMAP_RANGE, 1448 },
	{ 0x3365, 0x3365, PDF_CMAP_SINGLE, 7651 },
	{ 0x3366, 0x336a, PDF_CMAP_RANGE, 1476 },
	{ 0x336b, 0x336b, PDF_CMAP_SINGLE, 7652 },
	{ 0x336c, 0x3372, PDF_CMAP_RANGE, 1482 },
	{ 0x3373, 0x3373, PDF_CMAP_SINGLE, 7653 },
	{ 0x3374, 0x3375, PDF_CMAP_RANGE, 1490 },
	{ 0x3376, 0x3376, PDF_CMAP_SINGLE, 5937 },
	{ 0x3377, 0x3379, PDF_CMAP_RANGE, 1493 },
	{ 0x337a, 0x337a, PDF_CMAP_SINGLE, 7654 },
	{ 0x337b, 0x337e, PDF_CMAP_RANGE, 1497 },
	{ 0x3421, 0x3441, PDF_CMAP_RANGE, 1501 },
	{ 0x3442, 0x3443, PDF_CMAP_TABLE, 12 },
	{ 0x3444, 0x344c, PDF_CMAP_RANGE, 1536 },
	{ 0x344d, 0x344d, PDF_CMAP_SINGLE, 7656 },
	{ 0x344e, 0x3451, PDF_CMAP_RANGE, 1546 },
	{ 0x3452, 0x3452, PDF_CMAP_SINGLE, 6688 },
	{ 0x3453, 0x3464, PDF_CMAP_RANGE, 1551 },
	{ 0x3465, 0x3465, PDF_CMAP_SINGLE, 7657 },
	{ 0x3466, 0x347e, PDF_CMAP_RANGE, 1570 },
	{ 0x3521, 0x352a, PDF_CMAP_RANGE, 1595 },
	{ 0x352b, 0x352b, PDF_CMAP_SINGLE, 7658 },
	{ 0x352c, 0x353f, PDF_CMAP_RANGE, 1606 },
	{ 0x3540, 0x3540, PDF_CMAP_SINGLE, 7659 },
	{ 0x3541, 0x357e, PDF_CMAP_RANGE, 1627 },
	{ 0x3621, 0x3622, PDF_CMAP_TABLE, 14 },
	{ 0x3623, 0x3629, PDF_CMAP_RANGE, 1691 },
	{ 0x362a, 0x362a, PDF_CMAP_SINGLE, 7661 },
	{ 0x362b, 0x3645, PDF_CMAP_RANGE, 1699 },
	{ 0x3646, 0x3646, PDF_CMAP_SINGLE, 7474 },
	{ 0x3647, 0x364e, PDF_CMAP_RANGE, 1727 },
	{ 0x364f, 0x364f, PDF_CMAP_SINGLE, 7662 },
	{ 0x3650, 0x366c, PDF_CMAP_RANGE, 1736 },
	{ 0x366d, 0x366d, PDF_CMAP_SINGLE, 7663 },
	{ 0x366e, 0x3673, PDF_CMAP_RANGE, 1766 },
	{ 0x3674, 0x3674, PDF_CMAP_SINGLE, 7664 },
	{ 0x3675, 0x367a, PDF_CMAP_RANGE, 1773 },
	{ 0x367b, 0x367e, PDF_CMAP_TABLE, 16 },
	{ 0x3721, 0x3723, PDF_CMAP_RANGE, 1783 },
	{ 0x3724, 0x3724, PDF_CMAP_SINGLE, 7667 },
	{ 0x3725, 0x3736, PDF_CMAP_RANGE, 1787 },
	{ 0x3737, 0x3737, PDF_CMAP_SINGLE, 7668 },
	{ 0x3738, 0x3744, PDF_CMAP_RANGE, 1806 },
	{ 0x3745, 0x3745, PDF_CMAP_SINGLE, 7669 },
	{ 0x3746, 0x374d, PDF_CMAP_RANGE, 1820 },
	{ 0x374e, 0x374e, PDF_CMAP_SINGLE, 7670 },
	{ 0x374f, 0x3751, PDF_CMAP_RANGE, 1829 },
	{ 0x3752, 0x3752, PDF_CMAP_SINGLE, 7671 },
	{ 0x3753, 0x3754, PDF_CMAP_RANGE, 1833 },
	{ 0x3755, 0x3755, PDF_CMAP_SINGLE, 7672 },
	{ 0x3756, 0x375a, PDF_CMAP_RANGE, 1836 },
	{ 0x375b, 0x375b, PDF_CMAP_SINGLE, 7181 },
	{ 0x375c, 0x3763, PDF_CMAP_RANGE, 1842 },
	{ 0x3764, 0x3764, PDF_CMAP_SINGLE, 7673 },
	{ 0x3765, 0x3770, PDF_CMAP_RANGE, 1851 },
	{ 0x3771, 0x3771, PDF_CMAP_SINGLE, 7674 },
	{ 0x3772, 0x3778, PDF_CMAP_RANGE, 1864 },
	{ 0x3779, 0x3779, PDF_CMAP_SINGLE, 7675 },
	{ 0x377a, 0x377d, PDF_CMAP_RANGE, 1872 },
	{ 0x377e, 0x377e, PDF_CMAP_SINGLE, 7676 },
	{ 0x3821, 0x3833, PDF_CMAP_RANGE, 1877 },
	{ 0x3834, 0x3834, PDF_CMAP_SINGLE, 7677 },
	{ 0x3835, 0x3840, PDF_CMAP_RANGE, 1897 },
	{ 0x3841, 0x3841, PDF_CMAP_SINGLE, 7678 },
	{ 0x3842, 0x387e, PDF_CMAP_RANGE, 1910 },
	{ 0x3921, 0x392a, PDF_CMAP_RANGE, 1971 },
	{ 0x392b, 0x392b, PDF_CMAP_SINGLE, 7679 },
	{ 0x392c, 0x3936, PDF_CMAP_RANGE, 1982 },
	{ 0x3937, 0x3937, PDF_CMAP_SINGLE, 7680 },
	{ 0x3938, 0x3941, PDF_CMAP_RANGE, 1994 },
	{ 0x3942, 0x3942, PDF_CMAP_SINGLE, 7681 },
	{ 0x3943, 0x395b, PDF_CMAP_RANGE, 2005 },
	{ 0x395c, 0x395c, PDF_CMAP_SINGLE, 5853 },
	{ 0x395d, 0x396c, PDF_CMAP_RANGE, 2031 },
	{ 0x396d, 0x396d, PDF_CMAP_SINGLE, 7682 },
	{ 0x396e, 0x3973, PDF_CMAP_RANGE, 2048 },
	{ 0x3974, 0x3974, PDF_CMAP_SINGLE, 7683 },
	{ 0x3975, 0x3978, PDF_CMAP_RANGE, 2055 },
	{ 0x3979, 0x3979, PDF_CMAP_SINGLE, 7684 },
	{ 0x397a, 0x397e, PDF_CMAP_RANGE, 2060 },
	{ 0x3a21, 0x3a52, PDF_CMAP_RANGE, 2065 },
	{ 0x3a53, 0x3a53, PDF_CMAP_SINGLE, 7685 },
	{ 0x3a54, 0x3a66, PDF_CMAP_RANGE, 2116 },
	{ 0x3a67, 0x3a67, PDF_CMAP_SINGLE, 7686 },
	{ 0x3a68, 0x3a73, PDF_CMAP_RANGE, 2136 },
	{ 0x3a74, 0x3a74, PDF_CMAP_SINGLE, 7687 },
	{ 0x3a75, 0x3a7e, PDF_CMAP_RANGE, 2149 },
	{ 0x3b21, 0x3b26, PDF_CMAP_RANGE, 2159 },
	{ 0x3b27, 0x3b27, PDF_CMAP_SINGLE, 7688 },
	{ 0x3b28, 0x3b29, PDF_CMAP_RANGE, 2166 },
	{ 0x3b2a, 0x3b2c, PDF_CMAP_TABLE, 20 },
	{ 0x3b2d, 0x3b38, PDF_CMAP_RANGE, 2171 },
	{ 0x3b39, 0x3b39, PDF_CMAP_SINGLE, 7691 },
	{ 0x3b3a, 0x3b7e, PDF_CMAP_RANGE, 2184 },
	{ 0x3c21, 0x3c37, PDF_CMAP_RANGE, 2253 },
	{ 0x3c38, 0x3c38, PDF_CMAP_SINGLE, 7692 },
	{ 0x3c39, 0x3c47, PDF_CMAP_RANGE, 2277 },
	{ 0x3c48, 0x3c49, PDF_CMAP_TABLE, 23 },
	{ 0x3c4a, 0x3c56, PDF_CMAP_RANGE, 2294 },
	{ 0x3c57, 0x3c57, PDF_CMAP_SINGLE, 7694 },
	{ 0x3c58, 0x3c5c, PDF_CMAP_RANGE, 2308 },
	{ 0x3c5d, 0x3c5e, PDF_CMAP_RANGE, 7695 },
	{ 0x3c5f, 0x3c7e, PDF_CMAP_RANGE, 2315 },
	{ 0x3d21, 0x3d2a, PDF_CMAP_RANGE, 2347 },
	{ 0x3d2b, 0x3d2b, PDF_CMAP_SINGLE, 7697 },
	{ 0x3d2c, 0x3d35, PDF_CMAP_RANGE, 2358 },
	{ 0x3d36, 0x3d36, PDF_CMAP_SINGLE, 7698 },
	{ 0x3d37, 0x3d6b, PDF_CMAP_RANGE, 2369 },
	{ 0x3d6c, 0x3d6d, PDF_CMAP_RANGE, 7699 },
	{ 0x3d6e, 0x3d71, PDF_CMAP_RANGE, 2424 },
	{ 0x3d72, 0x3d73, PDF_CMAP_RANGE, 7701 },
	{ 0x3d74, 0x3d7e, PDF_CMAP_RANGE, 2430 },
	{ 0x3e21, 0x3e24, PDF_CMAP_RANGE, 2441 },
	{ 0x3e25, 0x3e25, PDF_CMAP_SINGLE, 7703 },
	{ 0x3e26, 0x3e32, PDF_CMAP_RANGE, 2446 },
	{ 0x3e33, 0x3e33, PDF_CMAP_SINGLE, 7704 },
	{ 0x3e34, 0x3e3e, PDF_CMAP_RANGE, 2460 },
	{ 0x3e3f, 0x3e3f, PDF_CMAP_SINGLE, 7705 },
	{ 0x3e40, 0x3e54, PDF_CMAP_RANGE, 2472 },
	{ 0x3e55, 0x3e55, PDF_CMAP_SINGLE, 7706 },
	{ 0x3e56, 0x3e5e, PDF_CMAP_RANGE, 2494 },
	{ 0x3e5f, 0x3e5f, PDF_CMAP_SINGLE, 7707 },
	{ 0x3e60, 0x3e63, PDF_CMAP_RANGE, 2504 },
	{ 0x3e64, 0x3e64, PDF_CMAP_SINGLE, 7708 },
	{ 0x3e65, 0x3e7e, PDF_CMAP_RANGE, 2509 },
	{ 0x3f21, 0x3f29, PDF_CMAP_RANGE, 2535 },
	{ 0x3f2a, 0x3f2a, PDF_CMAP_SINGLE, 7709 },
	{ 0x3f2b, 0x3f58, PDF_CMAP_RANGE, 2545 },
	{ 0x3f59, 0x3f59, PDF_CMAP_SINGLE, 7710 },
	{ 0x3f5a, 0x3f5f, PDF_CMAP_RANGE, 2592 },
	{ 0x3f60, 0x3f60, PDF_CMAP_SINGLE, 7711 },
	{ 0x3f61, 0x3f68, PDF_CMAP_RANGE, 2599 },
	{ 0x3f69, 0x3f69, PDF_CMAP_SINGLE, 7712 },
	{ 0x3f6a, 0x3f7e, PDF_CMAP_RANGE, 2608 },
	{ 0x4021, 0x4022, PDF_CMAP_TABLE, 25 },
	{ 0x4023, 0x4041, PDF_CMAP_RANGE, 2631 },
	{ 0x4042, 0x4042, PDF_CMAP_SINGLE, 7714 },
	{ 0x4043, 0x4065, PDF_CMAP_RANGE, 2663 },
	{ 0x4066, 0x4066, PDF_CMAP_SINGLE, 7715 },
	{ 0x4067, 0x4070, PDF_CMAP_RANGE, 2699 },
	{ 0x4071, 0x4072, PDF_CMAP_RANGE, 7716 },
	{ 0x4073, 0x4078, PDF_CMAP_RANGE, 2711 },
	{ 0x4079, 0x407a, PDF_CMAP_RANGE, 7718 },
	{ 0x407b, 0x407e, PDF_CMAP_RANGE, 2719 },
	{ 0x4121, 0x4126, PDF_CMAP_RANGE, 2723 },
	{ 0x4127, 0x4128, PDF_CMAP_TABLE, 27 },
	{ 0x4129, 0x4138, PDF_CMAP_RANGE, 2731 },
	{ 0x4139, 0x4139, PDF_CMAP_SINGLE, 7721 },
	{ 0x413a, 0x414b, PDF_CMAP_RANGE, 2748 },
	{ 0x414c, 0x414c, PDF_CMAP_SINGLE, 7722 },
	{ 0x414d, 0x414e, PDF_CMAP_RANGE, 2767 },
	{ 0x414f, 0x414f, PDF_CMAP_SINGLE, 7723 },
	{ 0x4150, 0x415e, PDF_CMAP_RANGE, 2770 },
	{ 0x415f, 0x415f, PDF_CMAP_SINGLE, 7724 },
	{ 0x4160, 0x4168, PDF_CMAP_RANGE, 2786 },
	{ 0x4169, 0x4169, PDF_CMAP_SINGLE, 7725 },
	{ 0x416a, 0x417e, PDF_CMAP_RANGE, 2796 },
	{ 0x4221, 0x423c, PDF_CMAP_RANGE, 2817 },
	{ 0x423d, 0x423d, PDF_CMAP_SINGLE, 7726 },
	{ 0x423e, 0x424c, PDF_CMAP_RANGE, 2846 },
	{ 0x424d, 0x424d, PDF_CMAP_SINGLE, 7727 },
	{ 0x424e, 0x425b, PDF_CMAP_RANGE, 2862 },
	{ 0x425c, 0x425c, PDF_CMAP_SINGLE, 7728 },
	{ 0x425d, 0x4262, PDF_CMAP_RANGE, 2877 },
	{ 0x4263, 0x4263, PDF_CMAP_SINGLE, 7729 },
	{ 0x4264, 0x426e, PDF_CMAP_RANGE, 2884 },
	{ 0x426f, 0x426f, PDF_CMAP_SINGLE, 7730 },
	{ 0x4270, 0x4274, PDF_CMAP_RANGE, 2896 },
	{ 0x4275, 0x4276, PDF_CMAP_RANGE, 7731 },
	{ 0x4277, 0x427c, PDF_CMAP_RANGE, 2903 },
	{ 0x427d, 0x427e, PDF_CMAP_TABLE, 29 },
	{ 0x4321, 0x4326, PDF_CMAP_RANGE, 2911 },
	{ 0x4327, 0x4328, PDF_CMAP_TABLE, 31 },
	{ 0x4329, 0x432a, PDF_CMAP_RANGE, 7735 },
	{ 0x432b, 0x432c, PDF_CMAP_RANGE, 2921 },
	{ 0x432d, 0x432e, PDF_CMAP_RANGE, 7737 },
	{ 0x432f, 0x433c, PDF_CMAP_RANGE, 2925 },
	{ 0x433d, 0x433d, PDF_CMAP_SINGLE, 7739 },
	{ 0x433e, 0x436f, PDF_CMAP_RANGE, 2940 },
	{ 0x4370, 0x4370, PDF_CMAP_SINGLE, 7740 },
	{ 0x4371, 0x4374, PDF_CMAP_RANGE, 2991 },
	{ 0x4375, 0x4375, PDF_CMAP_SINGLE, 7741 },
	{ 0x4376, 0x437b, PDF_CMAP_RANGE, 2996 },
	{ 0x437c, 0x437c, PDF_CMAP_SINGLE, 7742 },
	{ 0x437d, 0x437e, PDF_CMAP_RANGE, 3003 },
	{ 0x4421, 0x443c, PDF_CMAP_RANGE, 3005 },
	{ 0x443d, 0x443d, PDF_CMAP_SINGLE, 7743 },
	{ 0x443e, 0x4447, PDF_CMAP_RANGE, 3034 },
	{ 0x4448, 0x444a, PDF_CMAP_TABLE, 33 },
	{ 0x444b, 0x444c, PDF_CMAP_RANGE, 3047 },
	{ 0x444d, 0x444f, PDF_CMAP_TABLE, 36 },
	{ 0x4450, 0x445a, PDF_CMAP_RANGE, 3052 },
	{ 0x445b, 0x445b, PDF_CMAP_SINGLE, 4533 },
	{ 0x445c, 0x447e, PDF_CMAP_RANGE, 3064 },
	{ 0x4521, 0x4522, PDF_CMAP_TABLE, 39 },
	{ 0x4523, 0x4526, PDF_CMAP_RANGE, 3101 },
	{ 0x4527, 0x4527, PDF_CMAP_SINGLE, 7749 },
	{ 0x4528, 0x452d, PDF_CMAP_RANGE, 3106 },
	{ 0x452e, 0x452e, PDF_CMAP_SINGLE, 7750 },
	{ 0x452f, 0x4535, PDF_CMAP_RANGE, 3113 },
	{ 0x4536, 0x4536, PDF_CMAP_SINGLE, 7751 },
	{ 0x4537, 0x453e, PDF_CMAP_RANGE, 3121 },
	{ 0x453f, 0x453f, PDF_CMAP_SINGLE, 7752 },
	{ 0x4540, 0x4547, PDF_CMAP_RANGE, 3130 },
	{ 0x4548, 0x4548, PDF_CMAP_SINGLE, 7753 },
	{ 0x4549, 0x454a, PDF_CMAP_RANGE, 3139 },
	{ 0x454b, 0x454b, PDF_CMAP_SINGLE, 7754 },
	{ 0x454c, 0x4550, PDF_CMAP_RANGE, 3142 },
	{ 0x4551, 0x4552, PDF_CMAP_RANGE, 7755 },
	{ 0x4553, 0x4556, PDF_CMAP_RANGE, 3149 },
	{ 0x4557, 0x4557, PDF_CMAP_SINGLE, 5855 },
	{ 0x4558, 0x4563, PDF_CMAP_RANGE, 3154 },
	{ 0x4564, 0x4564, PDF_CMAP_SINGLE, 7757 },
	{ 0x4565, 0x456d, PDF_CMAP_RANGE, 3167 },
	{ 0x456e, 0x456e, PDF_CMAP_SINGLE, 5200 },
	{ 0x456f, 0x4572, PDF_CMAP_RANGE, 3177 },
	{ 0x4573, 0x4573, PDF_CMAP_SINGLE, 5430 },
	{ 0x4574, 0x4577, PDF_CMAP_RANGE, 3182 },
	{ 0x4578, 0x4578, PDF_CMAP_SINGLE, 7758 },
	{ 0x4579, 0x457e, PDF_CMAP_RANGE, 3187 },
	{ 0x4621, 0x463d, PDF_CMAP_RANGE, 3193 },
	{ 0x463e, 0x463e, PDF_CMAP_SINGLE, 7759 },
	{ 0x463f, 0x4641, PDF_CMAP_RANGE, 3223 },
	{ 0x4642, 0x4642, PDF_CMAP_SINGLE, 7760 },
	{ 0x4643, 0x4653, PDF_CMAP_RANGE, 3227 },
	{ 0x4654, 0x4655, PDF_CMAP_RANGE, 7761 },
	{ 0x4656, 0x465a, PDF_CMAP_RANGE, 3246 },
	{ 0x465b, 0x465c, PDF_CMAP_RANGE, 7763 },
	{ 0x465d, 0x4660, PDF_CMAP_RANGE, 3253 },
	{ 0x4661, 0x4661, PDF_CMAP_SINGLE, 7765 },
	{ 0x4662, 0x4665, PDF_CMAP_RANGE, 3258 },
	{ 0x4666, 0x4667, PDF_CMAP_RANGE, 7766 },
	{ 0x4668, 0x4669, PDF_CMAP_RANGE, 3264 },
	{ 0x466a, 0x466a, PDF_CMAP_SINGLE, 7768 },
	{ 0x466b, 0x4675, PDF_CMAP_RANGE, 3267 },
	{ 0x4676, 0x4676, PDF_CMAP_SINGLE, 6893 },
	{ 0x4677, 0x467e, PDF_CMAP_RANGE, 3279 },
	{ 0x4721, 0x4728, PDF_CMAP_RANGE, 3287 },
	{ 0x4729, 0x4729, PDF_CMAP_SINGLE, 7769 },
	{ 0x472a, 0x4738, PDF_CMAP_RANGE, 3296 },
	{ 0x4739, 0x4739, PDF_CMAP_SINGLE, 7770 },
	{ 0x473a, 0x4756, PDF_CMAP_RANGE, 3312 },
	{ 0x4757, 0x4757, PDF_CMAP_SINGLE, 7771 },
	{ 0x4758, 0x4766, PDF_CMAP_RANGE, 3342 },
	{ 0x4767, 0x4769, PDF_CMAP_TABLE, 41 },
	{ 0x476a, 0x476c, PDF_CMAP_RANGE, 3360 },
	{ 0x476d, 0x476d, PDF_CMAP_SINGLE, 7774 },
	{ 0x476e, 0x477e, PDF_CMAP_RANGE, 3364 },
	{ 0x4821, 0x4823, PDF_CMAP_RANGE, 3381 },
	{ 0x4824, 0x4824, PDF_CMAP_SINGLE, 7775 },
	{ 0x4825, 0x482d, PDF_CMAP_RANGE, 3385 },
	{ 0x482e, 0x4830, PDF_CMAP_TABLE, 44 },
	{ 0x4831, 0x4853, PDF_CMAP_RANGE, 3397 },
	{ 0x4854, 0x4854, PDF_CMAP_SINGLE, 7778 },
	{ 0x4855, 0x4861, PDF_CMAP_RANGE, 3433 },
	{ 0x4862, 0x4862, PDF_CMAP_SINGLE, 7779 },
	{ 0x4863, 0x4874, PDF_CMAP_RANGE, 3447 },
	{ 0x4875, 0x4875, PDF_CMAP_SINGLE, 7780 },
	{ 0x4876, 0x487e, PDF_CMAP_RANGE, 3466 },
	{ 0x4921, 0x4921, PDF_CMAP_SINGLE, 3475 },
	{ 0x4922, 0x4923, PDF_CMAP_RANGE, 7781 },
	{ 0x4924, 0x492e, PDF_CMAP_RANGE, 3478 },
	{ 0x492f, 0x4932, PDF_CMAP_TABLE, 47 },
	{ 0x4933, 0x4934, PDF_CMAP_RANGE, 3493 },
	{ 0x4935, 0x4935, PDF_CMAP_SINGLE, 7785 },
	{ 0x4936, 0x493f, PDF_CMAP_RANGE, 3496 },
	{ 0x4940, 0x4940, PDF_CMAP_SINGLE, 7786 },
	{ 0x4941, 0x494d, PDF_CMAP_RANGE, 3507 },
	{ 0x494e, 0x494e, PDF_CMAP_SINGLE, 7787 },
	{ 0x494f, 0x4950, PDF_CMAP_RANGE, 3521 },
	{ 0x4951, 0x4951, PDF_CMAP_SINGLE, 7788 },
	{ 0x4952, 0x497e, PDF_CMAP_RANGE, 3524 },
	{ 0x4a21, 0x4a42, PDF_CMAP_RANGE, 3569 },
	{ 0x4a43, 0x4a43, PDF_CMAP_SINGLE, 7789 },
	{ 0x4a44, 0x4a4c, PDF_CMAP_RANGE, 3604 },
	{ 0x4a4d, 0x4a4d, PDF_CMAP_SINGLE, 7790 },
	{ 0x4a4e, 0x4a59, PDF_CMAP_RANGE, 3614 },
	{ 0x4a5a, 0x4a5a, PDF_CMAP_SINGLE, 7791 },
	{ 0x4a5b, 0x4a78, PDF_CMAP_RANGE, 3627 },
	{ 0x4a79, 0x4a79, PDF_CMAP_SINGLE, 7792 },
	{ 0x4a7a, 0x4a7e, PDF_CMAP_RANGE, 3658 },
	{ 0x4b21, 0x4b22, PDF_CMAP_TABLE, 51 },
	{ 0x4b23, 0x4b28, PDF_CMAP_RANGE, 3665 },
	{ 0x4b29, 0x4b29, PDF_CMAP_SINGLE, 7794 },
	{ 0x4b2a, 0x4b4a, PDF_CMAP_RANGE, 3672 },
	{ 0x4b4b, 0x4b4b, PDF_CMAP_SINGLE, 7795 },
	{ 0x4b4c, 0x4b69, PDF_CMAP_RANGE, 3706 },
	{ 0x4b6a, 0x4b6a, PDF_CMAP_SINGLE, 7475 },
	{ 0x4b6b, 0x4b6f, PDF_CMAP_RANGE, 3737 },
	{ 0x4b70, 0x4b70, PDF_CMAP_SINGLE, 7796 },
	{ 0x4b71, 0x4b78, PDF_CMAP_RANGE, 3743 },
	{ 0x4b79, 0x4b79, PDF_CMAP_SINGLE, 4143 },
	{ 0x4b7a, 0x4b7e, PDF_CMAP_RANGE, 3752 },
	{ 0x4c21, 0x4c4c, PDF_CMAP_RANGE, 3757 },
	{ 0x4c4d, 0x4c4d, PDF_CMAP_SINGLE, 7797 },
	{ 0x4c4e, 0x4c58, PDF_CMAP_RANGE, 3802 },
	{ 0x4c59, 0x4c59, PDF_CMAP_SINGLE, 7798 },
	{ 0x4c5a, 0x4c5e, PDF_CMAP_RANGE, 3814 },
	{ 0x4c5f, 0x4c5f, PDF_CMAP_SINGLE, 7799 },
	{ 0x4c60, 0x4c61, PDF_CMAP_RANGE, 3820 },
	{ 0x4c62, 0x4c62, PDF_CMAP_SINGLE, 7800 },
	{ 0x4c63, 0x4c78, PDF_CMAP_RANGE, 3823 },
	{ 0x4c79, 0x4c7e, PDF_CMAP_TABLE, 53 },
	{ 0x4d21, 0x4d31, PDF_CMAP_RANGE, 3851 },
	{ 0x4d32, 0x4d32, PDF_CMAP_SINGLE, 7804 },
	{ 0x4d33, 0x4d4f, PDF_CMAP_RANGE, 3869 },
	{ 0x4d50, 0x4d50, PDF_CMAP_SINGLE, 7805 },
	{ 0x4d51, 0x4d53, PDF_CMAP_RANGE, 3899 },
	{ 0x4d54, 0x4d54, PDF_CMAP_SINGLE, 7806 },
	{ 0x4d55, 0x4d59, PDF_CMAP_RANGE, 3903 },
	{ 0x4d5a, 0x4d5a, PDF_CMAP_SINGLE, 7476 },
	{ 0x4d5b, 0x4d68, PDF_CMAP_RANGE, 3909 },
	{ 0x4d69, 0x4d69, PDF_CMAP_SINGLE, 7807 },
	{ 0x4d6a, 0x4d7e, PDF_CMAP_RANGE, 3924 },
	{ 0x4e21, 0x4e4a, PDF_CMAP_RANGE, 3945 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 7808 },
	{ 0x4e4c, 0x4e79, PDF_CMAP_RANGE, 3988 },
	{ 0x4e7a, 0x4e7b, PDF_CMAP_RANGE, 7809 },
	{ 0x4e7c, 0x4e7e, PDF_CMAP_RANGE, 4036 },
	{ 0x4f21, 0x4f21, PDF_CMAP_SINGLE, 7811 },
	{ 0x4f22, 0x4f30, PDF_CMAP_RANGE, 4040 },
	{ 0x4f31, 0x4f31, PDF_CMAP_SINGLE, 7812 },
	{ 0x4f32, 0x4f35, PDF_CMAP_RANGE, 4056 },
	{ 0x4f36, 0x4f36, PDF_CMAP_SINGLE, 6007 },
	{ 0x4f37, 0x4f38, PDF_CMAP_RANGE, 4061 },
	{ 0x4f39, 0x4f39, PDF_CMAP_SINGLE, 7813 },
	{ 0x4f3a, 0x4f53, PDF_CMAP_RANGE, 4064 },
	{ 0x5021, 0x5055, PDF_CMAP_RANGE, 4090 },
	{ 0x5056, 0x5056, PDF_CMAP_SINGLE, 3751 },
	{ 0x5057, 0x507e, PDF_CMAP_RANGE, 4144 },
	{ 0x5121, 0x513c, PDF_CMAP_RANGE, 4184 },
	{ 0x513d, 0x513d, PDF_CMAP_SINGLE, 7814 },
	{ 0x513e, 0x5146, PDF_CMAP_RANGE, 4213 },
	{ 0x5147, 0x5147, PDF_CMAP_SINGLE, 7815 },
	{ 0x5148, 0x514a, PDF_CMAP_RANGE, 4223 },
	{ 0x514b, 0x514d, PDF_CMAP_TABLE, 59 },
	{ 0x514e, 0x517e, PDF_CMAP_RANGE, 4229 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 4278 },
	{ 0x5321, 0x532f, PDF_CMAP_RANGE, 4372 },
	{ 0x5330, 0x5330, PDF_CMAP_SINGLE, 7818 },
	{ 0x5331, 0x5339, PDF_CMAP_RANGE, 4388 },
	{ 0x533a, 0x533a, PDF_CMAP_SINGLE, 7819 },
	{ 0x533b, 0x534f, PDF_CMAP_RANGE, 4398 },
	{ 0x5350, 0x5350, PDF_CMAP_SINGLE, 7820 },
	{ 0x5351, 0x535d, PDF_CMAP_RANGE, 4420 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 7821 },
	{ 0x535f, 0x536a, PDF_CMAP_RANGE, 4434 },
	{ 0x536b, 0x536b, PDF_CMAP_SINGLE, 7822 },
	{ 0x536c, 0x537e, PDF_CMAP_RANGE, 4447 },
	{ 0x5421, 0x5443, PDF_CMAP_RANGE, 4466 },
	{ 0x5444, 0x5444, PDF_CMAP_SINGLE, 7823 },
	{ 0x5445, 0x5463, PDF_CMAP_RANGE, 4502 },
	{ 0x5464, 0x5464, PDF_CMAP_SINGLE, 3063 },
	{ 0x5465, 0x547e, PDF_CMAP_RANGE, 4534 },
	{ 0x5521, 0x553c, PDF_CMAP_RANGE, 4560 },
	{ 0x553d, 0x553d, PDF_CMAP_SINGLE, 7824 },
	{ 0x553e, 0x5562, PDF_CMAP_RANGE, 4589 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 7825 },
	{ 0x5564, 0x557e, PDF_CMAP_RANGE, 4627 },
	{ 0x5621, 0x5622, PDF_CMAP_TABLE, 62 },
	{ 0x5623, 0x5671, PDF_CMAP_RANGE, 4656 },
	{ 0x5672, 0x5672, PDF_CMAP_SINGLE, 7827 },
	{ 0x5673, 0x567e, PDF_CMAP_RANGE, 4736 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 4748 },
	{ 0x5821, 0x5823, PDF_CMAP_RANGE, 4842 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 7828 },
	{ 0x5825, 0x587e, PDF_CMAP_RANGE, 4846 },
	{ 0x5921, 0x595f, PDF_CMAP_RANGE, 4936 },
	{ 0x5960, 0x5960, PDF_CMAP_SINGLE, 7829 },
	{ 0x5961, 0x596b, PDF_CMAP_RANGE, 5000 },
	{ 0x596c, 0x596c, PDF_CMAP_SINGLE, 7830 },
	{ 0x596d, 0x5977, PDF_CMAP_RANGE, 5012 },
	{ 0x5978, 0x5978, PDF_CMAP_SINGLE, 1447 },
	{ 0x5979, 0x597e, PDF_CMAP_RANGE, 5024 },
	{ 0x5a21, 0x5a38, PDF_CMAP_RANGE, 5030 },
	{ 0x5a39, 0x5a39, PDF_CMAP_SINGLE, 7831 },
	{ 0x5a3a, 0x5a4c, PDF_CMAP_RANGE, 5055 },
	{ 0x5a4d, 0x5a4d, PDF_CMAP_SINGLE, 7832 },
	{ 0x5a4e, 0x5a7e, PDF_CMAP_RANGE, 5075 },
	{ 0x5b21, 0x5b44, PDF_CMAP_RANGE, 5124 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 7833 },
	{ 0x5b46, 0x5b49, PDF_CMAP_RANGE, 5161 },
	{ 0x5b4a, 0x5b4a, PDF_CMAP_SINGLE, 7834 },
	{ 0x5b4b, 0x5b57, PDF_CMAP_RANGE, 5166 },
	{ 0x5b58, 0x5b58, PDF_CMAP_SINGLE, 3490 },
	{ 0x5b59, 0x5b6a, PDF_CMAP_RANGE, 5180 },
	{ 0x5b6b, 0x5b6d, PDF_CMAP_TABLE, 64 },
	{ 0x5b6e, 0x5b73, PDF_CMAP_RANGE, 5201 },
	{ 0x5b74, 0x5b74, PDF_CMAP_SINGLE, 7836 },
	{ 0x5b75, 0x5b7e, PDF_CMAP_RANGE, 5208 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5218 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 5312 },
	{ 0x5e21, 0x5e38, PDF_CMAP_RANGE, 5406 },
	{ 0x5e39, 0x5e39, PDF_CMAP_SINGLE, 3181 },
	{ 0x5e3a, 0x5e4f, PDF_CMAP_RANGE, 5431 },
	{ 0x5e50, 0x5e50, PDF_CMAP_SINGLE, 7837 },
	{ 0x5e51, 0x5e74, PDF_CMAP_RANGE, 5454 },
	{ 0x5e75, 0x5e75, PDF_CMAP_SINGLE, 1535 },
	{ 0x5e76, 0x5e7e, PDF_CMAP_RANGE, 5491 },
	{ 0x5f21, 0x5f72, PDF_CMAP_RANGE, 5500 },
	{ 0x5f73, 0x5f73, PDF_CMAP_SINGLE, 7838 },
	{ 0x5f74, 0x5f7e, PDF_CMAP_RANGE, 5583 },
	{ 0x6021, 0x6025, PDF_CMAP_RANGE, 5594 },
	{ 0x6026, 0x6026, PDF_CMAP_SINGLE, 7839 },
	{ 0x6027, 0x605e, PDF_CMAP_RANGE, 5600 },
	{ 0x605f, 0x605f, PDF_CMAP_SINGLE, 7840 },
	{ 0x6060, 0x6075, PDF_CMAP_RANGE, 5657 },
	{ 0x6076, 0x6076, PDF_CMAP_SINGLE, 7477 },
	{ 0x6077, 0x607e, PDF_CMAP_RANGE, 5680 },
	{ 0x6121, 0x612a, PDF_CMAP_RANGE, 5688 },
	{ 0x612b, 0x612b, PDF_CMAP_SINGLE, 7841 },
	{ 0x612c, 0x612f, PDF_CMAP_RANGE, 5699 },
	{ 0x6130, 0x6131, PDF_CMAP_RANGE, 7842 },
	{ 0x6132, 0x617e, PDF_CMAP_RANGE, 5705 },
	{ 0x6221, 0x622a, PDF_CMAP_RANGE, 5782 },
	{ 0x622b, 0x622b, PDF_CMAP_SINGLE, 7844 },
	{ 0x622c, 0x6267, PDF_CMAP_RANGE, 5793 },
	{ 0x6268, 0x626a, PDF_CMAP_TABLE, 67 },
	{ 0x626b, 0x626e, PDF_CMAP_RANGE, 5856 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 7845 },
	{ 0x6270, 0x627e, PDF_CMAP_RANGE, 5861 },
	{ 0x6321, 0x6349, PDF_CMAP_RANGE, 5876 },
	{ 0x634a, 0x634a, PDF_CMAP_SINGLE, 7846 },
	{ 0x634b, 0x6353, PDF_CMAP_RANGE, 5918 },
	{ 0x6354, 0x6354, PDF_CMAP_SINGLE, 7847 },
	{ 0x6355, 0x635d, PDF_CMAP_RANGE, 5928 },
	{ 0x635e, 0x635e, PDF_CMAP_SINGLE, 1492 },
	{ 0x635f, 0x637e, PDF_CMAP_RANGE, 5938 },
	{ 0x6421, 0x6438, PDF_CMAP_RANGE, 5970 },
	{ 0x6439, 0x6439, PDF_CMAP_SINGLE, 7848 },
	{ 0x643a, 0x6445, PDF_CMAP_RANGE, 5995 },
	{ 0x6446, 0x6446, PDF_CMAP_SINGLE, 4060 },
	{ 0x6447, 0x6463, PDF_CMAP_RANGE, 6008 },
	{ 0x6464, 0x6464, PDF_CMAP_SINGLE, 7849 },
	{ 0x6465, 0x646d, PDF_CMAP_RANGE, 6038 },
	{ 0x646e, 0x646e, PDF_CMAP_SINGLE, 7850 },
	{ 0x646f, 0x647e, PDF_CMAP_RANGE, 6048 },
	{ 0x6521, 0x6538, PDF_CMAP_RANGE, 6064 },
	{ 0x6539, 0x653b, PDF_CMAP_TABLE, 70 },
	{ 0x653c, 0x6545, PDF_CMAP_RANGE, 6091 },
	{ 0x6546, 0x6546, PDF_CMAP_SINGLE, 7853 },
	{ 0x6547, 0x657e, PDF_CMAP_RANGE, 6102 },
	{ 0x6621, 0x6645, PDF_CMAP_RANGE, 6158 },
	{ 0x6646, 0x6646, PDF_CMAP_SINGLE, 7854 },
	{ 0x6647, 0x667e, PDF_CMAP_RANGE, 6196 },
	{ 0x6721, 0x6763, PDF_CMAP_RANGE, 6252 },
	{ 0x6764, 0x6764, PDF_CMAP_SINGLE, 7855 },
	{ 0x6765, 0x6768, PDF_CMAP_RANGE, 6320 },
	{ 0x6769, 0x6769, PDF_CMAP_SINGLE, 7856 },
	{ 0x676a, 0x6771, PDF_CMAP_RANGE, 6325 },
	{ 0x6772, 0x6772, PDF_CMAP_SINGLE, 7857 },
	{ 0x6773, 0x677e, PDF_CMAP_RANGE, 6334 },
	{ 0x6821, 0x6833, PDF_CMAP_RANGE, 6346 },
	{ 0x6834, 0x6834, PDF_CMAP_SINGLE, 7858 },
	{ 0x6835, 0x683a, PDF_CMAP_RANGE, 6366 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 7859 },
	{ 0x683c, 0x6873, PDF_CMAP_RANGE, 6373 },
	{ 0x6874, 0x6874, PDF_CMAP_SINGLE, 7860 },
	{ 0x6875, 0x687e, PDF_CMAP_RANGE, 6430 },
	{ 0x6921, 0x6922, PDF_CMAP_TABLE, 73 },
	{ 0x6923, 0x692d, PDF_CMAP_RANGE, 6442 },
	{ 0x692e, 0x692e, PDF_CMAP_SINGLE, 3845 },
	{ 0x692f, 0x693b, PDF_CMAP_RANGE, 6454 },
	{ 0x693c, 0x693c, PDF_CMAP_SINGLE, 7861 },
	{ 0x693d, 0x6959, PDF_CMAP_RANGE, 6468 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 1440 },
	{ 0x695b, 0x697e, PDF_CMAP_RANGE, 6498 },
	{ 0x6a21, 0x6a23, PDF_CMAP_RANGE, 6534 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 3358 },
	{ 0x6a25, 0x6a26, PDF_CMAP_RANGE, 6538 },
	{ 0x6a27, 0x6a27, PDF_CMAP_SINGLE, 7862 },
	{ 0x6a28, 0x6a3c, PDF_CMAP_RANGE, 6541 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 7863 },
	{ 0x6a3e, 0x6a6e, PDF_CMAP_RANGE, 6563 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 7864 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6613 },
	{ 0x6b21, 0x6b31, PDF_CMAP_RANGE, 6628 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 7865 },
	{ 0x6b33, 0x6b5c, PDF_CMAP_RANGE, 6646 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 1550 },
	{ 0x6b5e, 0x6b65, PDF_CMAP_RANGE, 6689 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 7866 },
	{ 0x6b67, 0x6b75, PDF_CMAP_RANGE, 6698 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 7867 },
	{ 0x6b77, 0x6b7e, PDF_CMAP_RANGE, 6714 },
	{ 0x6c21, 0x6c4c, PDF_CMAP_RANGE, 6722 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2730 },
	{ 0x6c4e, 0x6c68, PDF_CMAP_RANGE, 6767 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 7868 },
	{ 0x6c6a, 0x6c73, PDF_CMAP_RANGE, 6795 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 7869 },
	{ 0x6c75, 0x6c7e, PDF_CMAP_RANGE, 6806 },
	{ 0x6d21, 0x6d4d, PDF_CMAP_RANGE, 6816 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 7870 },
	{ 0x6d4f, 0x6d6b, PDF_CMAP_RANGE, 6862 },
	{ 0x6d6c, 0x6d6e, PDF_CMAP_TABLE, 75 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6894 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e56, PDF_CMAP_RANGE, 6939 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 7875 },
	{ 0x6e58, 0x6e7e, PDF_CMAP_RANGE, 6965 },
	{ 0x6f21, 0x6f64, PDF_CMAP_RANGE, 7004 },
	{ 0x6f65, 0x6f65, PDF_CMAP_SINGLE, 7876 },
	{ 0x6f66, 0x6f7e, PDF_CMAP_RANGE, 7073 },
	{ 0x7021, 0x7032, PDF_CMAP_RANGE, 7098 },
	{ 0x7033, 0x7033, PDF_CMAP_SINGLE, 7877 },
	{ 0x7034, 0x7044, PDF_CMAP_RANGE, 7117 },
	{ 0x7045, 0x7045, PDF_CMAP_SINGLE, 7878 },
	{ 0x7046, 0x7050, PDF_CMAP_RANGE, 7135 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x7056, PDF_CMAP_RANGE, 7147 },
	{ 0x7057, 0x7057, PDF_CMAP_SINGLE, 7880 },
	{ 0x7058, 0x7073, PDF_CMAP_RANGE, 7153 },
	{ 0x7074, 0x7075, PDF_CMAP_TABLE, 78 },
	{ 0x7076, 0x707e, PDF_CMAP_RANGE, 7183 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724c, PDF_CMAP_RANGE, 7314 },
	{ 0x724d, 0x724e, PDF_CMAP_TABLE, 80 },
	{ 0x724f, 0x7273, PDF_CMAP_RANGE, 7332 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1321 },
	{ 0x7275, 0x727e, PDF_CMAP_RANGE, 7370 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x737c, PDF_CMAP_RANGE, 7429 },
	{ 0x737d, 0x737e, PDF_CMAP_TABLE, 82 },
	{ 0x7921, 0x797e, PDF_CMAP_RANGE, 8359 },
	{ 0x7a21, 0x7a35, PDF_CMAP_RANGE, 8453 },
	{ 0x7a36, 0x7a36, PDF_CMAP_SINGLE, 1993 },
	{ 0x7a37, 0x7a7e, PDF_CMAP_RANGE, 8474 },
	{ 0x7b21, 0x7b7e, PDF_CMAP_RANGE, 8546 },
	{ 0x7c21, 0x7c6e, PDF_CMAP_RANGE, 8640 },
	{ 0x7c71, 0x7c7a, PDF_CMAP_RANGE, 8092 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 751 },
	{ 0x7c7c, 0x7c7e, PDF_CMAP_RANGE, 8005 },
	{ 0x6d4f, 0x6d6b, PDF_CMAP_RANGE, 6862 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 7871 },
	{ 0x6d6d, 0x6d6d, PDF_CMAP_SINGLE, 6892 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 7872 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6894 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e56, PDF_CMAP_RANGE, 6939 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 7875 },
	{ 0x6e58, 0x6e7e, PDF_CMAP_RANGE, 6965 },
	{ 0x6f21, 0x6f64, PDF_CMAP_RANGE, 7004 },
	{ 0x6f65, 0x6f65, PDF_CMAP_SINGLE, 7876 },
	{ 0x6f66, 0x6f7e, PDF_CMAP_RANGE, 7073 },
	{ 0x7021, 0x7032, PDF_CMAP_RANGE, 7098 },
	{ 0x7033, 0x7033, PDF_CMAP_SINGLE, 7877 },
	{ 0x7034, 0x7044, PDF_CMAP_RANGE, 7117 },
	{ 0x7045, 0x7045, PDF_CMAP_SINGLE, 7878 },
	{ 0x7046, 0x7050, PDF_CMAP_RANGE, 7135 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x7056, PDF_CMAP_RANGE, 7147 },
	{ 0x7057, 0x7057, PDF_CMAP_SINGLE, 7880 },
	{ 0x7058, 0x7073, PDF_CMAP_RANGE, 7153 },
	{ 0x7074, 0x7074, PDF_CMAP_SINGLE, 1841 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 7881 },
	{ 0x7076, 0x707e, PDF_CMAP_RANGE, 7183 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724c, PDF_CMAP_RANGE, 7314 },
	{ 0x724d, 0x724d, PDF_CMAP_SINGLE, 1143 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 7884 },
	{ 0x724f, 0x7273, PDF_CMAP_RANGE, 7332 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1321 },
	{ 0x7275, 0x727e, PDF_CMAP_RANGE, 7370 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x737c, PDF_CMAP_RANGE, 7429 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 7886 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 7473 },
	{ 0x7921, 0x797e, PDF_CMAP_RANGE, 8359 },
	{ 0x7a21, 0x7a35, PDF_CMAP_RANGE, 8453 },
	{ 0x7a36, 0x7a36, PDF_CMAP_SINGLE, 1993 },
	{ 0x7a37, 0x7a7e, PDF_CMAP_RANGE, 8474 },
	{ 0x7b21, 0x7b7e, PDF_CMAP_RANGE, 8546 },
	{ 0x7c21, 0x7c6e, PDF_CMAP_RANGE, 8640 },
	{ 0x7c71, 0x7c7a, PDF_CMAP_RANGE, 8092 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 751 },
	{ 0x7c7c, 0x7c7e, PDF_CMAP_RANGE, 8005 },
};

static const unsigned short pdf_cmap_Ext_H_table[84] =
{
	762,761,769,1125,7633,7645,7369,7646,
	7649,1406,1407,7650,7655,5490,1689,7660,
	7665,1780,7666,1782,7689,2169,7690,7693,
	6441,2629,7713,7720,6766,7733,2910,7734,
	2918,7744,3045,7745,7746,3050,7747,3099,
	7748,7772,6537,7773,7776,3395,7777,7783,
	5179,3491,7784,3663,7793,6453,7801,3847,
	7802,3849,7803,7816,4227,7817,4654,7826,
	7835,5199,3176,2030,5854,3153,7851,6089,
	7852,6440,2293,7871,6892,7872,1841,7881,
	1143,7884,7886,7473,
};

pdf_cmap pdf_cmap_Ext_H =
{
	-1, "Ext-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	661, 661, (pdf_range*) pdf_cmap_Ext_H_ranges,
	84, 84, (unsigned short*) pdf_cmap_Ext_H_table,
};

/* Ext-RKSJ-H */

static const pdf_range pdf_cmap_Ext_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x8188, PDF_CMAP_RANGE, 696 },
	{ 0x8189, 0x8189, PDF_CMAP_SINGLE, 7478 },
	{ 0x818a, 0x81ac, PDF_CMAP_RANGE, 706 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x8540, 0x857e, PDF_CMAP_RANGE, 232 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 390 },
	{ 0x8581, 0x859e, PDF_CMAP_RANGE, 296 },
	{ 0x859f, 0x85dd, PDF_CMAP_RANGE, 327 },
	{ 0x85de, 0x85fc, PDF_CMAP_RANGE, 391 },
	{ 0x8640, 0x867e, PDF_CMAP_RANGE, 422 },
	{ 0x8680, 0x8691, PDF_CMAP_RANGE, 485 },
	{ 0x8692, 0x8692, PDF_CMAP_SINGLE, 295 },
	{ 0x8693, 0x869e, PDF_CMAP_RANGE, 503 },
	{ 0x86a2, 0x86ed, PDF_CMAP_RANGE, 7479 },
	{ 0x8740, 0x875d, PDF_CMAP_RANGE, 7555 },
	{ 0x875f, 0x8775, PDF_CMAP_RANGE, 7585 },
	{ 0x877e, 0x877e, PDF_CMAP_SINGLE, 8323 },
	{ 0x8780, 0x878f, PDF_CMAP_RANGE, 7608 },
	{ 0x8790, 0x8792, PDF_CMAP_TABLE, 0 },
	{ 0x8793, 0x8799, PDF_CMAP_RANGE, 7624 },
	{ 0x879a, 0x879a, PDF_CMAP_SINGLE, 768 },
	{ 0x879b, 0x879c, PDF_CMAP_RANGE, 7631 },
	{ 0x889f, 0x88a0, PDF_CMAP_TABLE, 3 },
	{ 0x88a1, 0x88b0, PDF_CMAP_RANGE, 1127 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 7330 },
	{ 0x88b2, 0x88b8, PDF_CMAP_RANGE, 1144 },
	{ 0x88b9, 0x88b9, PDF_CMAP_SINGLE, 7634 },
	{ 0x88ba, 0x88eb, PDF_CMAP_RANGE, 1152 },
	{ 0x88ec, 0x88ec, PDF_CMAP_SINGLE, 7635 },
	{ 0x88ed, 0x88f0, PDF_CMAP_RANGE, 1203 },
	{ 0x88f1, 0x88f1, PDF_CMAP_SINGLE, 7636 },
	{ 0x88f2, 0x88f9, PDF_CMAP_RANGE, 1208 },
	{ 0x88fa, 0x88fa, PDF_CMAP_SINGLE, 7637 },
	{ 0x88fb, 0x88fc, PDF_CMAP_RANGE, 1217 },
	{ 0x8940, 0x8948, PDF_CMAP_RANGE, 1219 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 7638 },
	{ 0x894a, 0x8953, PDF_CMAP_RANGE, 1229 },
	{ 0x8954, 0x8954, PDF_CMAP_SINGLE, 7639 },
	{ 0x8955, 0x8957, PDF_CMAP_RANGE, 1240 },
	{ 0x8958, 0x8958, PDF_CMAP_SINGLE, 7640 },
	{ 0x8959, 0x895a, PDF_CMAP_RANGE, 1244 },
	{ 0x895b, 0x895c, PDF_CMAP_RANGE, 7641 },
	{ 0x895d, 0x8960, PDF_CMAP_RANGE, 1248 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 7643 },
	{ 0x8962, 0x897e, PDF_CMAP_RANGE, 1253 },
	{ 0x8980, 0x898a, PDF_CMAP_RANGE, 1282 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 7644 },
	{ 0x898c, 0x89a5, PDF_CMAP_RANGE, 1294 },
	{ 0x89a6, 0x89a8, PDF_CMAP_TABLE, 5 },
	{ 0x89a9, 0x89dd, PDF_CMAP_RANGE, 1323 },
	{ 0x89de, 0x89de, PDF_CMAP_SINGLE, 7647 },
	{ 0x89df, 0x89f7, PDF_CMAP_RANGE, 1377 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7648 },
	{ 0x89f9, 0x89fa, PDF_CMAP_RANGE, 1403 },
	{ 0x89fb, 0x89fc, PDF_CMAP_TABLE, 8 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 10 },
	{ 0x8a42, 0x8a60, PDF_CMAP_RANGE, 1409 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 6497 },
	{ 0x8a62, 0x8a67, PDF_CMAP_RANGE, 1441 },
	{ 0x8a68, 0x8a68, PDF_CMAP_SINGLE, 5023 },
	{ 0x8a69, 0x8a7e, PDF_CMAP_RANGE, 1448 },
	{ 0x8a80, 0x8a84, PDF_CMAP_RANGE, 1470 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 7651 },
	{ 0x8a86, 0x8a8a, PDF_CMAP_RANGE, 1476 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 7652 },
	{ 0x8a8c, 0x8a92, PDF_CMAP_RANGE, 1482 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 7653 },
	{ 0x8a94, 0x8a95, PDF_CMAP_RANGE, 1490 },
	{ 0x8a96, 0x8a96, PDF_CMAP_SINGLE, 5937 },
	{ 0x8a97, 0x8a99, PDF_CMAP_RANGE, 1493 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 7654 },
	{ 0x8a9b, 0x8abf, PDF_CMAP_RANGE, 1497 },
	{ 0x8ac0, 0x8ac1, PDF_CMAP_TABLE, 12 },
	{ 0x8ac2, 0x8aca, PDF_CMAP_RANGE, 1536 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 7656 },
	{ 0x8acc, 0x8acf, PDF_CMAP_RANGE, 1546 },
	{ 0x8ad0, 0x8ad0, PDF_CMAP_SINGLE, 6688 },
	{ 0x8ad1, 0x8ae2, PDF_CMAP_RANGE, 1551 },
	{ 0x8ae3, 0x8ae3, PDF_CMAP_SINGLE, 7657 },
	{ 0x8ae4, 0x8afc, PDF_CMAP_RANGE, 1570 },
	{ 0x8b40, 0x8b49, PDF_CMAP_RANGE, 1595 },
	{ 0x8b4a, 0x8b4a, PDF_CMAP_SINGLE, 7658 },
	{ 0x8b4b, 0x8b5e, PDF_CMAP_RANGE, 1606 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 7659 },
	{ 0x8b60, 0x8b7e, PDF_CMAP_RANGE, 1627 },
	{ 0x8b80, 0x8b9f, PDF_CMAP_RANGE, 1658 },
	{ 0x8ba0, 0x8ba0, PDF_CMAP_SINGLE, 7660 },
	{ 0x8ba1, 0x8ba7, PDF_CMAP_RANGE, 1691 },
	{ 0x8ba8, 0x8ba8, PDF_CMAP_SINGLE, 7661 },
	{ 0x8ba9, 0x8bc3, PDF_CMAP_RANGE, 1699 },
	{ 0x8bc4, 0x8bc4, PDF_CMAP_SINGLE, 7474 },
	{ 0x8bc5, 0x8bcc, PDF_CMAP_RANGE, 1727 },
	{ 0x8bcd, 0x8bcd, PDF_CMAP_SINGLE, 7662 },
	{ 0x8bce, 0x8bea, PDF_CMAP_RANGE, 1736 },
	{ 0x8beb, 0x8beb, PDF_CMAP_SINGLE, 7663 },
	{ 0x8bec, 0x8bf1, PDF_CMAP_RANGE, 1766 },
	{ 0x8bf2, 0x8bf2, PDF_CMAP_SINGLE, 7664 },
	{ 0x8bf3, 0x8bf8, PDF_CMAP_RANGE, 1773 },
	{ 0x8bf9, 0x8bfc, PDF_CMAP_TABLE, 14 },
	{ 0x8c40, 0x8c42, PDF_CMAP_RANGE, 1783 },
	{ 0x8c43, 0x8c43, PDF_CMAP_SINGLE, 7667 },
	{ 0x8c44, 0x8c55, PDF_CMAP_RANGE, 1787 },
	{ 0x8c56, 0x8c56, PDF_CMAP_SINGLE, 7668 },
	{ 0x8c57, 0x8c63, PDF_CMAP_RANGE, 1806 },
	{ 0x8c64, 0x8c64, PDF_CMAP_SINGLE, 7669 },
	{ 0x8c65, 0x8c6c, PDF_CMAP_RANGE, 1820 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 7670 },
	{ 0x8c6e, 0x8c70, PDF_CMAP_RANGE, 1829 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 7671 },
	{ 0x8c72, 0x8c73, PDF_CMAP_RANGE, 1833 },
	{ 0x8c74, 0x8c74, PDF_CMAP_SINGLE, 7672 },
	{ 0x8c75, 0x8c79, PDF_CMAP_RANGE, 1836 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 7181 },
	{ 0x8c7b, 0x8c7e, PDF_CMAP_RANGE, 1842 },
	{ 0x8c80, 0x8c83, PDF_CMAP_RANGE, 1846 },
	{ 0x8c84, 0x8c84, PDF_CMAP_SINGLE, 7673 },
	{ 0x8c85, 0x8c90, PDF_CMAP_RANGE, 1851 },
	{ 0x8c91, 0x8c91, PDF_CMAP_SINGLE, 7674 },
	{ 0x8c92, 0x8c98, PDF_CMAP_RANGE, 1864 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 7675 },
	{ 0x8c9a, 0x8c9d, PDF_CMAP_RANGE, 1872 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 7676 },
	{ 0x8c9f, 0x8cb1, PDF_CMAP_RANGE, 1877 },
	{ 0x8cb2, 0x8cb2, PDF_CMAP_SINGLE, 7677 },
	{ 0x8cb3, 0x8cbe, PDF_CMAP_RANGE, 1897 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 7678 },
	{ 0x8cc0, 0x8cfc, PDF_CMAP_RANGE, 1910 },
	{ 0x8d40, 0x8d49, PDF_CMAP_RANGE, 1971 },
	{ 0x8d4a, 0x8d4a, PDF_CMAP_SINGLE, 7679 },
	{ 0x8d4b, 0x8d55, PDF_CMAP_RANGE, 1982 },
	{ 0x8d56, 0x8d56, PDF_CMAP_SINGLE, 7680 },
	{ 0x8d57, 0x8d60, PDF_CMAP_RANGE, 1994 },
	{ 0x8d61, 0x8d61, PDF_CMAP_SINGLE, 7681 },
	{ 0x8d62, 0x8d7a, PDF_CMAP_RANGE, 2005 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 5853 },
	{ 0x8d7c, 0x8d7e, PDF_CMAP_RANGE, 2031 },
	{ 0x8d80, 0x8d8c, PDF_CMAP_RANGE, 2034 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 7682 },
	{ 0x8d8e, 0x8d93, PDF_CMAP_RANGE, 2048 },
	{ 0x8d94, 0x8d94, PDF_CMAP_SINGLE, 7683 },
	{ 0x8d95, 0x8d98, PDF_CMAP_RANGE, 2055 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 7684 },
	{ 0x8d9a, 0x8dd0, PDF_CMAP_RANGE, 2060 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 7685 },
	{ 0x8dd2, 0x8de4, PDF_CMAP_RANGE, 2116 },
	{ 0x8de5, 0x8de5, PDF_CMAP_SINGLE, 7686 },
	{ 0x8de6, 0x8df1, PDF_CMAP_RANGE, 2136 },
	{ 0x8df2, 0x8df2, PDF_CMAP_SINGLE, 7687 },
	{ 0x8df3, 0x8dfc, PDF_CMAP_RANGE, 2149 },
	{ 0x8e40, 0x8e45, PDF_CMAP_RANGE, 2159 },
	{ 0x8e46, 0x8e46, PDF_CMAP_SINGLE, 7688 },
	{ 0x8e47, 0x8e48, PDF_CMAP_RANGE, 2166 },
	{ 0x8e49, 0x8e4b, PDF_CMAP_TABLE, 18 },
	{ 0x8e4c, 0x8e57, PDF_CMAP_RANGE, 2171 },
	{ 0x8e58, 0x8e58, PDF_CMAP_SINGLE, 7691 },
	{ 0x8e59, 0x8e7e, PDF_CMAP_RANGE, 2184 },
	{ 0x8e80, 0x8eb5, PDF_CMAP_RANGE, 2222 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 7692 },
	{ 0x8eb7, 0x8ec5, PDF_CMAP_RANGE, 2277 },
	{ 0x8ec6, 0x8ec7, PDF_CMAP_TABLE, 21 },
	{ 0x8ec8, 0x8ed4, PDF_CMAP_RANGE, 2294 },
	{ 0x8ed5, 0x8ed5, PDF_CMAP_SINGLE, 7694 },
	{ 0x8ed6, 0x8eda, PDF_CMAP_RANGE, 2308 },
	{ 0x8edb, 0x8edc, PDF_CMAP_RANGE, 7695 },
	{ 0x8edd, 0x8efc, PDF_CMAP_RANGE, 2315 },
	{ 0x8f40, 0x8f49, PDF_CMAP_RANGE, 2347 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 7697 },
	{ 0x8f4b, 0x8f54, PDF_CMAP_RANGE, 2358 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 7698 },
	{ 0x8f56, 0x8f7e, PDF_CMAP_RANGE, 2369 },
	{ 0x8f80, 0x8f8b, PDF_CMAP_RANGE, 2410 },
	{ 0x8f8c, 0x8f8d, PDF_CMAP_RANGE, 7699 },
	{ 0x8f8e, 0x8f91, PDF_CMAP_RANGE, 2424 },
	{ 0x8f92, 0x8f93, PDF_CMAP_RANGE, 7701 },
	{ 0x8f94, 0x8fa2, PDF_CMAP_RANGE, 2430 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 7703 },
	{ 0x8fa4, 0x8fb0, PDF_CMAP_RANGE, 2446 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 7704 },
	{ 0x8fb2, 0x8fbc, PDF_CMAP_RANGE, 2460 },
	{ 0x8fbd, 0x8fbd, PDF_CMAP_SINGLE, 7705 },
	{ 0x8fbe, 0x8fd2, PDF_CMAP_RANGE, 2472 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 7706 },
	{ 0x8fd4, 0x8fdc, PDF_CMAP_RANGE, 2494 },
	{ 0x8fdd, 0x8fdd, PDF_CMAP_SINGLE, 7707 },
	{ 0x8fde, 0x8fe1, PDF_CMAP_RANGE, 2504 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 7708 },
	{ 0x8fe3, 0x8ffc, PDF_CMAP_RANGE, 2509 },
	{ 0x9040, 0x9048, PDF_CMAP_RANGE, 2535 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 7709 },
	{ 0x904a, 0x9077, PDF_CMAP_RANGE, 2545 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 7710 },
	{ 0x9079, 0x907e, PDF_CMAP_RANGE, 2592 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 7711 },
	{ 0x9081, 0x9088, PDF_CMAP_RANGE, 2599 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 7712 },
	{ 0x908a, 0x909f, PDF_CMAP_RANGE, 2608 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 7713 },
	{ 0x90a1, 0x90bf, PDF_CMAP_RANGE, 2631 },
	{ 0x90c0, 0x90c0, PDF_CMAP_SINGLE, 7714 },
	{ 0x90c1, 0x90e3, PDF_CMAP_RANGE, 2663 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 7715 },
	{ 0x90e5, 0x90ee, PDF_CMAP_RANGE, 2699 },
	{ 0x90ef, 0x90f0, PDF_CMAP_RANGE, 7716 },
	{ 0x90f1, 0x90f6, PDF_CMAP_RANGE, 2711 },
	{ 0x90f7, 0x90f8, PDF_CMAP_RANGE, 7718 },
	{ 0x90f9, 0x90fc, PDF_CMAP_RANGE, 2719 },
	{ 0x9140, 0x9145, PDF_CMAP_RANGE, 2723 },
	{ 0x9146, 0x9147, PDF_CMAP_TABLE, 23 },
	{ 0x9148, 0x9157, PDF_CMAP_RANGE, 2731 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 7721 },
	{ 0x9159, 0x916a, PDF_CMAP_RANGE, 2748 },
	{ 0x916b, 0x916b, PDF_CMAP_SINGLE, 7722 },
	{ 0x916c, 0x916d, PDF_CMAP_RANGE, 2767 },
	{ 0x916e, 0x916e, PDF_CMAP_SINGLE, 7723 },
	{ 0x916f, 0x917d, PDF_CMAP_RANGE, 2770 },
	{ 0x917e, 0x917e, PDF_CMAP_SINGLE, 7724 },
	{ 0x9180, 0x9188, PDF_CMAP_RANGE, 2786 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 7725 },
	{ 0x918a, 0x91ba, PDF_CMAP_RANGE, 2796 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 7726 },
	{ 0x91bc, 0x91ca, PDF_CMAP_RANGE, 2846 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 7727 },
	{ 0x91cc, 0x91d9, PDF_CMAP_RANGE, 2862 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 7728 },
	{ 0x91db, 0x91e0, PDF_CMAP_RANGE, 2877 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 7729 },
	{ 0x91e2, 0x91ec, PDF_CMAP_RANGE, 2884 },
	{ 0x91ed, 0x91ed, PDF_CMAP_SINGLE, 7730 },
	{ 0x91ee, 0x91f2, PDF_CMAP_RANGE, 2896 },
	{ 0x91f3, 0x91f4, PDF_CMAP_RANGE, 7731 },
	{ 0x91f5, 0x91fa, PDF_CMAP_RANGE, 2903 },
	{ 0x91fb, 0x91fc, PDF_CMAP_TABLE, 25 },
	{ 0x9240, 0x9245, PDF_CMAP_RANGE, 2911 },
	{ 0x9246, 0x9247, PDF_CMAP_TABLE, 27 },
	{ 0x9248, 0x9249, PDF_CMAP_RANGE, 7735 },
	{ 0x924a, 0x924b, PDF_CMAP_RANGE, 2921 },
	{ 0x924c, 0x924d, PDF_CMAP_RANGE, 7737 },
	{ 0x924e, 0x925b, PDF_CMAP_RANGE, 2925 },
	{ 0x925c, 0x925c, PDF_CMAP_SINGLE, 7739 },
	{ 0x925d, 0x927e, PDF_CMAP_RANGE, 2940 },
	{ 0x9280, 0x928f, PDF_CMAP_RANGE, 2974 },
	{ 0x9290, 0x9290, PDF_CMAP_SINGLE, 7740 },
	{ 0x9291, 0x9294, PDF_CMAP_RANGE, 2991 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 7741 },
	{ 0x9296, 0x929b, PDF_CMAP_RANGE, 2996 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 7742 },
	{ 0x929d, 0x92ba, PDF_CMAP_RANGE, 3003 },
	{ 0x92bb, 0x92bb, PDF_CMAP_SINGLE, 7743 },
	{ 0x92bc, 0x92c5, PDF_CMAP_RANGE, 3034 },
	{ 0x92c6, 0x92c8, PDF_CMAP_TABLE, 29 },
	{ 0x92c9, 0x92ca, PDF_CMAP_RANGE, 3047 },
	{ 0x92cb, 0x92cd, PDF_CMAP_TABLE, 32 },
	{ 0x92ce, 0x92d8, PDF_CMAP_RANGE, 3052 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 4533 },
	{ 0x92da, 0x92fc, PDF_CMAP_RANGE, 3064 },
	{ 0x9340, 0x9341, PDF_CMAP_TABLE, 35 },
	{ 0x9342, 0x9345, PDF_CMAP_RANGE, 3101 },
	{ 0x9346, 0x9346, PDF_CMAP_SINGLE, 7749 },
	{ 0x9347, 0x934c, PDF_CMAP_RANGE, 3106 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 7750 },
	{ 0x934e, 0x9354, PDF_CMAP_RANGE, 3113 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 7751 },
	{ 0x9356, 0x935d, PDF_CMAP_RANGE, 3121 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 7752 },
	{ 0x935f, 0x9366, PDF_CMAP_RANGE, 3130 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 7753 },
	{ 0x9368, 0x9369, PDF_CMAP_RANGE, 3139 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 7754 },
	{ 0x936b, 0x936f, PDF_CMAP_RANGE, 3142 },
	{ 0x9370, 0x9371, PDF_CMAP_RANGE, 7755 },
	{ 0x9372, 0x9375, PDF_CMAP_RANGE, 3149 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 5855 },
	{ 0x9377, 0x937e, PDF_CMAP_RANGE, 3154 },
	{ 0x9380, 0x9383, PDF_CMAP_RANGE, 3162 },
	{ 0x9384, 0x9384, PDF_CMAP_SINGLE, 7757 },
	{ 0x9385, 0x938d, PDF_CMAP_RANGE, 3167 },
	{ 0x938e, 0x938e, PDF_CMAP_SINGLE, 5200 },
	{ 0x938f, 0x9392, PDF_CMAP_RANGE, 3177 },
	{ 0x9393, 0x9393, PDF_CMAP_SINGLE, 5430 },
	{ 0x9394, 0x9397, PDF_CMAP_RANGE, 3182 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 7758 },
	{ 0x9399, 0x93bb, PDF_CMAP_RANGE, 3187 },
	{ 0x93bc, 0x93bc, PDF_CMAP_SINGLE, 7759 },
	{ 0x93bd, 0x93bf, PDF_CMAP_RANGE, 3223 },
	{ 0x93c0, 0x93c0, PDF_CMAP_SINGLE, 7760 },
	{ 0x93c1, 0x93d1, PDF_CMAP_RANGE, 3227 },
	{ 0x93d2, 0x93d3, PDF_CMAP_RANGE, 7761 },
	{ 0x93d4, 0x93d8, PDF_CMAP_RANGE, 3246 },
	{ 0x93d9, 0x93da, PDF_CMAP_RANGE, 7763 },
	{ 0x93db, 0x93de, PDF_CMAP_RANGE, 3253 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 7765 },
	{ 0x93e0, 0x93e3, PDF_CMAP_RANGE, 3258 },
	{ 0x93e4, 0x93e5, PDF_CMAP_RANGE, 7766 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 3264 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7768 },
	{ 0x93e9, 0x93f3, PDF_CMAP_RANGE, 3267 },
	{ 0x93f4, 0x93f4, PDF_CMAP_SINGLE, 6893 },
	{ 0x93f5, 0x93fc, PDF_CMAP_RANGE, 3279 },
	{ 0x9440, 0x9447, PDF_CMAP_RANGE, 3287 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 7769 },
	{ 0x9449, 0x9457, PDF_CMAP_RANGE, 3296 },
	{ 0x9458, 0x9458, PDF_CMAP_SINGLE, 7770 },
	{ 0x9459, 0x9475, PDF_CMAP_RANGE, 3312 },
	{ 0x9476, 0x9476, PDF_CMAP_SINGLE, 7771 },
	{ 0x9477, 0x947e, PDF_CMAP_RANGE, 3342 },
	{ 0x9480, 0x9486, PDF_CMAP_RANGE, 3350 },
	{ 0x9487, 0x9489, PDF_CMAP_TABLE, 37 },
	{ 0x948a, 0x948c, PDF_CMAP_RANGE, 3360 },
	{ 0x948d, 0x948d, PDF_CMAP_SINGLE, 7774 },
	{ 0x948e, 0x94a1, PDF_CMAP_RANGE, 3364 },
	{ 0x94a2, 0x94a2, PDF_CMAP_SINGLE, 7775 },
	{ 0x94a3, 0x94ab, PDF_CMAP_RANGE, 3385 },
	{ 0x94ac, 0x94ae, PDF_CMAP_TABLE, 40 },
	{ 0x94af, 0x94d1, PDF_CMAP_RANGE, 3397 },
	{ 0x94d2, 0x94d2, PDF_CMAP_SINGLE, 7778 },
	{ 0x94d3, 0x94df, PDF_CMAP_RANGE, 3433 },
	{ 0x94e0, 0x94e0, PDF_CMAP_SINGLE, 7779 },
	{ 0x94e1, 0x94f2, PDF_CMAP_RANGE, 3447 },
	{ 0x94f3, 0x94f3, PDF_CMAP_SINGLE, 7780 },
	{ 0x94f4, 0x94fc, PDF_CMAP_RANGE, 3466 },
	{ 0x9540, 0x9540, PDF_CMAP_SINGLE, 3475 },
	{ 0x9541, 0x9542, PDF_CMAP_RANGE, 7781 },
	{ 0x9543, 0x954d, PDF_CMAP_RANGE, 3478 },
	{ 0x954e, 0x9551, PDF_CMAP_TABLE, 43 },
	{ 0x9552, 0x9553, PDF_CMAP_RANGE, 3493 },
	{ 0x9554, 0x9554, PDF_CMAP_SINGLE, 7785 },
	{ 0x9555, 0x955e, PDF_CMAP_RANGE, 3496 },
	{ 0x955f, 0x955f, PDF_CMAP_SINGLE, 7786 },
	{ 0x9560, 0x956c, PDF_CMAP_RANGE, 3507 },
	{ 0x956d, 0x956d, PDF_CMAP_SINGLE, 7787 },
	{ 0x956e, 0x956f, PDF_CMAP_RANGE, 3521 },
	{ 0x9570, 0x9570, PDF_CMAP_SINGLE, 7788 },
	{ 0x9571, 0x957e, PDF_CMAP_RANGE, 3524 },
	{ 0x9580, 0x95c0, PDF_CMAP_RANGE, 3538 },
	{ 0x95c1, 0x95c1, PDF_CMAP_SINGLE, 7789 },
	{ 0x95c2, 0x95ca, PDF_CMAP_RANGE, 3604 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 7790 },
	{ 0x95cc, 0x95d7, PDF_CMAP_RANGE, 3614 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 7791 },
	{ 0x95d9, 0x95f6, PDF_CMAP_RANGE, 3627 },
	{ 0x95f7, 0x95f7, PDF_CMAP_SINGLE, 7792 },
	{ 0x95f8, 0x95fc, PDF_CMAP_RANGE, 3658 },
	{ 0x9640, 0x9641, PDF_CMAP_TABLE, 47 },
	{ 0x9642, 0x9647, PDF_CMAP_RANGE, 3665 },
	{ 0x9648, 0x9648, PDF_CMAP_SINGLE, 7794 },
	{ 0x9649, 0x9669, PDF_CMAP_RANGE, 3672 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 7795 },
	{ 0x966b, 0x967e, PDF_CMAP_RANGE, 3706 },
	{ 0x9680, 0x9689, PDF_CMAP_RANGE, 3726 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 7475 },
	{ 0x968b, 0x968f, PDF_CMAP_RANGE, 3737 },
	{ 0x9690, 0x9690, PDF_CMAP_SINGLE, 7796 },
	{ 0x9691, 0x9698, PDF_CMAP_RANGE, 3743 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 4143 },
	{ 0x969a, 0x96ca, PDF_CMAP_RANGE, 3752 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 7797 },
	{ 0x96cc, 0x96d6, PDF_CMAP_RANGE, 3802 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 7798 },
	{ 0x96d8, 0x96dc, PDF_CMAP_RANGE, 3814 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 7799 },
	{ 0x96de, 0x96df, PDF_CMAP_RANGE, 3820 },
	{ 0x96e0, 0x96e0, PDF_CMAP_SINGLE, 7800 },
	{ 0x96e1, 0x96f6, PDF_CMAP_RANGE, 3823 },
	{ 0x96f7, 0x96fc, PDF_CMAP_TABLE, 49 },
	{ 0x9740, 0x9750, PDF_CMAP_RANGE, 3851 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 7804 },
	{ 0x9752, 0x976e, PDF_CMAP_RANGE, 3869 },
	{ 0x976f, 0x976f, PDF_CMAP_SINGLE, 7805 },
	{ 0x9770, 0x9772, PDF_CMAP_RANGE, 3899 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 7806 },
	{ 0x9774, 0x9778, PDF_CMAP_RANGE, 3903 },
	{ 0x9779, 0x9779, PDF_CMAP_SINGLE, 7476 },
	{ 0x977a, 0x977e, PDF_CMAP_RANGE, 3909 },
	{ 0x9780, 0x9788, PDF_CMAP_RANGE, 3914 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 7807 },
	{ 0x978a, 0x97c8, PDF_CMAP_RANGE, 3924 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 7808 },
	{ 0x97ca, 0x97f7, PDF_CMAP_RANGE, 3988 },
	{ 0x97f8, 0x97f9, PDF_CMAP_RANGE, 7809 },
	{ 0x97fa, 0x97fc, PDF_CMAP_RANGE, 4036 },
	{ 0x9840, 0x9840, PDF_CMAP_SINGLE, 7811 },
	{ 0x9841, 0x984f, PDF_CMAP_RANGE, 4040 },
	{ 0x9850, 0x9850, PDF_CMAP_SINGLE, 7812 },
	{ 0x9851, 0x9854, PDF_CMAP_RANGE, 4056 },
	{ 0x9855, 0x9855, PDF_CMAP_SINGLE, 6007 },
	{ 0x9856, 0x9857, PDF_CMAP_RANGE, 4061 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 7813 },
	{ 0x9859, 0x9872, PDF_CMAP_RANGE, 4064 },
	{ 0x989f, 0x98d3, PDF_CMAP_RANGE, 4090 },
	{ 0x98d4, 0x98d4, PDF_CMAP_SINGLE, 3751 },
	{ 0x98d5, 0x98fc, PDF_CMAP_RANGE, 4144 },
	{ 0x9940, 0x995b, PDF_CMAP_RANGE, 4184 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 7814 },
	{ 0x995d, 0x9965, PDF_CMAP_RANGE, 4213 },
	{ 0x9966, 0x9966, PDF_CMAP_SINGLE, 7815 },
	{ 0x9967, 0x9969, PDF_CMAP_RANGE, 4223 },
	{ 0x996a, 0x996c, PDF_CMAP_TABLE, 55 },
	{ 0x996d, 0x997e, PDF_CMAP_RANGE, 4229 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a4e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a4f, 0x9a4f, PDF_CMAP_SINGLE, 7818 },
	{ 0x9a50, 0x9a58, PDF_CMAP_RANGE, 4388 },
	{ 0x9a59, 0x9a59, PDF_CMAP_SINGLE, 7819 },
	{ 0x9a5a, 0x9a6e, PDF_CMAP_RANGE, 4398 },
	{ 0x9a6f, 0x9a6f, PDF_CMAP_SINGLE, 7820 },
	{ 0x9a70, 0x9a7c, PDF_CMAP_RANGE, 4420 },
	{ 0x9a7d, 0x9a7e, PDF_CMAP_TABLE, 58 },
	{ 0x9a80, 0x9a8a, PDF_CMAP_RANGE, 4435 },
	{ 0x9a8b, 0x9a8b, PDF_CMAP_SINGLE, 7822 },
	{ 0x9a8c, 0x9ac1, PDF_CMAP_RANGE, 4447 },
	{ 0x9ac2, 0x9ac2, PDF_CMAP_SINGLE, 7823 },
	{ 0x9ac3, 0x9ae1, PDF_CMAP_RANGE, 4502 },
	{ 0x9ae2, 0x9ae2, PDF_CMAP_SINGLE, 3063 },
	{ 0x9ae3, 0x9afc, PDF_CMAP_RANGE, 4534 },
	{ 0x9b40, 0x9b5b, PDF_CMAP_RANGE, 4560 },
	{ 0x9b5c, 0x9b5c, PDF_CMAP_SINGLE, 7824 },
	{ 0x9b5d, 0x9b7e, PDF_CMAP_RANGE, 4589 },
	{ 0x9b80, 0x9b82, PDF_CMAP_RANGE, 4623 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7825 },
	{ 0x9b84, 0x9b9f, PDF_CMAP_RANGE, 4627 },
	{ 0x9ba0, 0x9ba0, PDF_CMAP_SINGLE, 7826 },
	{ 0x9ba1, 0x9bef, PDF_CMAP_RANGE, 4656 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 7827 },
	{ 0x9bf1, 0x9bfc, PDF_CMAP_RANGE, 4736 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9ca1, PDF_CMAP_RANGE, 4811 },
	{ 0x9ca2, 0x9ca2, PDF_CMAP_SINGLE, 7828 },
	{ 0x9ca3, 0x9cfc, PDF_CMAP_RANGE, 4846 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9d80, PDF_CMAP_SINGLE, 7829 },
	{ 0x9d81, 0x9d8b, PDF_CMAP_RANGE, 5000 },
	{ 0x9d8c, 0x9d8c, PDF_CMAP_SINGLE, 7830 },
	{ 0x9d8d, 0x9d97, PDF_CMAP_RANGE, 5012 },
	{ 0x9d98, 0x9d98, PDF_CMAP_SINGLE, 1447 },
	{ 0x9d99, 0x9db6, PDF_CMAP_RANGE, 5024 },
	{ 0x9db7, 0x9db7, PDF_CMAP_SINGLE, 7831 },
	{ 0x9db8, 0x9dca, PDF_CMAP_RANGE, 5055 },
	{ 0x9dcb, 0x9dcb, PDF_CMAP_SINGLE, 7832 },
	{ 0x9dcc, 0x9dfc, PDF_CMAP_RANGE, 5075 },
	{ 0x9e40, 0x9e63, PDF_CMAP_RANGE, 5124 },
	{ 0x9e64, 0x9e64, PDF_CMAP_SINGLE, 7833 },
	{ 0x9e65, 0x9e68, PDF_CMAP_RANGE, 5161 },
	{ 0x9e69, 0x9e69, PDF_CMAP_SINGLE, 7834 },
	{ 0x9e6a, 0x9e76, PDF_CMAP_RANGE, 5166 },
	{ 0x9e77, 0x9e77, PDF_CMAP_SINGLE, 3490 },
	{ 0x9e78, 0x9e7e, PDF_CMAP_RANGE, 5180 },
	{ 0x9e80, 0x9e8a, PDF_CMAP_RANGE, 5187 },
	{ 0x9e8b, 0x9e8d, PDF_CMAP_TABLE, 60 },
	{ 0x9e8e, 0x9e93, PDF_CMAP_RANGE, 5201 },
	{ 0x9e94, 0x9e94, PDF_CMAP_SINGLE, 7836 },
	{ 0x9e95, 0x9efc, PDF_CMAP_RANGE, 5208 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9fb6, PDF_CMAP_RANGE, 5375 },
	{ 0x9fb7, 0x9fb7, PDF_CMAP_SINGLE, 3181 },
	{ 0x9fb8, 0x9fcd, PDF_CMAP_RANGE, 5431 },
	{ 0x9fce, 0x9fce, PDF_CMAP_SINGLE, 7837 },
	{ 0x9fcf, 0x9ff2, PDF_CMAP_RANGE, 5454 },
	{ 0x9ff3, 0x9ff3, PDF_CMAP_SINGLE, 1535 },
	{ 0x9ff4, 0x9ffc, PDF_CMAP_RANGE, 5491 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe092, PDF_CMAP_RANGE, 5563 },
	{ 0xe093, 0xe093, PDF_CMAP_SINGLE, 7838 },
	{ 0xe094, 0xe0a3, PDF_CMAP_RANGE, 5583 },
	{ 0xe0a4, 0xe0a4, PDF_CMAP_SINGLE, 7839 },
	{ 0xe0a5, 0xe0dc, PDF_CMAP_RANGE, 5600 },
	{ 0xe0dd, 0xe0dd, PDF_CMAP_SINGLE, 7840 },
	{ 0xe0de, 0xe0f3, PDF_CMAP_RANGE, 5657 },
	{ 0xe0f4, 0xe0f4, PDF_CMAP_SINGLE, 7477 },
	{ 0xe0f5, 0xe0fc, PDF_CMAP_RANGE, 5680 },
	{ 0xe140, 0xe149, PDF_CMAP_RANGE, 5688 },
	{ 0xe14a, 0xe14a, PDF_CMAP_SINGLE, 7841 },
	{ 0xe14b, 0xe14e, PDF_CMAP_RANGE, 5699 },
	{ 0xe14f, 0xe150, PDF_CMAP_RANGE, 7842 },
	{ 0xe151, 0xe17e, PDF_CMAP_RANGE, 5705 },
	{ 0xe180, 0xe1a8, PDF_CMAP_RANGE, 5751 },
	{ 0xe1a9, 0xe1a9, PDF_CMAP_SINGLE, 7844 },
	{ 0xe1aa, 0xe1e5, PDF_CMAP_RANGE, 5793 },
	{ 0xe1e6, 0xe1e8, PDF_CMAP_TABLE, 63 },
	{ 0xe1e9, 0xe1ec, PDF_CMAP_RANGE, 5856 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 7845 },
	{ 0xe1ee, 0xe1fc, PDF_CMAP_RANGE, 5861 },
	{ 0xe240, 0xe268, PDF_CMAP_RANGE, 5876 },
	{ 0xe269, 0xe269, PDF_CMAP_SINGLE, 7846 },
	{ 0xe26a, 0xe272, PDF_CMAP_RANGE, 5918 },
	{ 0xe273, 0xe273, PDF_CMAP_SINGLE, 7847 },
	{ 0xe274, 0xe27c, PDF_CMAP_RANGE, 5928 },
	{ 0xe27d, 0xe27e, PDF_CMAP_TABLE, 66 },
	{ 0xe280, 0xe2b6, PDF_CMAP_RANGE, 5939 },
	{ 0xe2b7, 0xe2b7, PDF_CMAP_SINGLE, 7848 },
	{ 0xe2b8, 0xe2c3, PDF_CMAP_RANGE, 5995 },
	{ 0xe2c4, 0xe2c4, PDF_CMAP_SINGLE, 4060 },
	{ 0xe2c5, 0xe2e1, PDF_CMAP_RANGE, 6008 },
	{ 0xe2e2, 0xe2e2, PDF_CMAP_SINGLE, 7849 },
	{ 0xe2e3, 0xe2eb, PDF_CMAP_RANGE, 6038 },
	{ 0xe2ec, 0xe2ec, PDF_CMAP_SINGLE, 7850 },
	{ 0xe2ed, 0xe2fc, PDF_CMAP_RANGE, 6048 },
	{ 0xe340, 0xe357, PDF_CMAP_RANGE, 6064 },
	{ 0xe358, 0xe35a, PDF_CMAP_TABLE, 68 },
	{ 0xe35b, 0xe364, PDF_CMAP_RANGE, 6091 },
	{ 0xe365, 0xe365, PDF_CMAP_SINGLE, 7853 },
	{ 0xe366, 0xe37e, PDF_CMAP_RANGE, 6102 },
	{ 0xe380, 0xe3c3, PDF_CMAP_RANGE, 6127 },
	{ 0xe3c4, 0xe3c4, PDF_CMAP_SINGLE, 7854 },
	{ 0xe3c5, 0xe3fc, PDF_CMAP_RANGE, 6196 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe483, PDF_CMAP_RANGE, 6315 },
	{ 0xe484, 0xe484, PDF_CMAP_SINGLE, 7855 },
	{ 0xe485, 0xe488, PDF_CMAP_RANGE, 6320 },
	{ 0xe489, 0xe489, PDF_CMAP_SINGLE, 7856 },
	{ 0xe48a, 0xe491, PDF_CMAP_RANGE, 6325 },
	{ 0xe492, 0xe492, PDF_CMAP_SINGLE, 7857 },
	{ 0xe493, 0xe4b1, PDF_CMAP_RANGE, 6334 },
	{ 0xe4b2, 0xe4b2, PDF_CMAP_SINGLE, 7858 },
	{ 0xe4b3, 0xe4b8, PDF_CMAP_RANGE, 6366 },
	{ 0xe4b9, 0xe4b9, PDF_CMAP_SINGLE, 7859 },
	{ 0xe4ba, 0xe4f1, PDF_CMAP_RANGE, 6373 },
	{ 0xe4f2, 0xe4f2, PDF_CMAP_SINGLE, 7860 },
	{ 0xe4f3, 0xe4fc, PDF_CMAP_RANGE, 6430 },
	{ 0xe540, 0xe541, PDF_CMAP_TABLE, 71 },
	{ 0xe542, 0xe54c, PDF_CMAP_RANGE, 6442 },
	{ 0xe54d, 0xe54d, PDF_CMAP_SINGLE, 3845 },
	{ 0xe54e, 0xe55a, PDF_CMAP_RANGE, 6454 },
	{ 0xe55b, 0xe55b, PDF_CMAP_SINGLE, 7861 },
	{ 0xe55c, 0xe578, PDF_CMAP_RANGE, 6468 },
	{ 0xe579, 0xe579, PDF_CMAP_SINGLE, 1440 },
	{ 0xe57a, 0xe57e, PDF_CMAP_RANGE, 6498 },
	{ 0xe580, 0xe5a1, PDF_CMAP_RANGE, 6503 },
	{ 0xe5a2, 0xe5a2, PDF_CMAP_SINGLE, 3358 },
	{ 0xe5a3, 0xe5a4, PDF_CMAP_RANGE, 6538 },
	{ 0xe5a5, 0xe5a5, PDF_CMAP_SINGLE, 7862 },
	{ 0xe5a6, 0xe5ba, PDF_CMAP_RANGE, 6541 },
	{ 0xe5bb, 0xe5bb, PDF_CMAP_SINGLE, 7863 },
	{ 0xe5bc, 0xe5ec, PDF_CMAP_RANGE, 6563 },
	{ 0xe5ed, 0xe5ed, PDF_CMAP_SINGLE, 7864 },
	{ 0xe5ee, 0xe5fc, PDF_CMAP_RANGE, 6613 },
	{ 0xe640, 0xe650, PDF_CMAP_RANGE, 6628 },
	{ 0xe651, 0xe651, PDF_CMAP_SINGLE, 7865 },
	{ 0xe652, 0xe67b, PDF_CMAP_RANGE, 6646 },
	{ 0xe67c, 0xe67c, PDF_CMAP_SINGLE, 1550 },
	{ 0xe67d, 0xe67e, PDF_CMAP_RANGE, 6689 },
	{ 0xe680, 0xe685, PDF_CMAP_RANGE, 6691 },
	{ 0xe686, 0xe686, PDF_CMAP_SINGLE, 7866 },
	{ 0xe687, 0xe695, PDF_CMAP_RANGE, 6698 },
	{ 0xe696, 0xe696, PDF_CMAP_SINGLE, 7867 },
	{ 0xe697, 0xe6ca, PDF_CMAP_RANGE, 6714 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 2730 },
	{ 0xe6cc, 0xe6e6, PDF_CMAP_RANGE, 6767 },
	{ 0xe6e7, 0xe6e7, PDF_CMAP_SINGLE, 7868 },
	{ 0xe6e8, 0xe6f1, PDF_CMAP_RANGE, 6795 },
	{ 0xe6f2, 0xe6f2, PDF_CMAP_SINGLE, 7869 },
	{ 0xe6f3, 0xe6fc, PDF_CMAP_RANGE, 6806 },
	{ 0xe740, 0xe76c, PDF_CMAP_RANGE, 6816 },
	{ 0xe76d, 0xe76d, PDF_CMAP_SINGLE, 7870 },
	{ 0xe76e, 0xe77e, PDF_CMAP_RANGE, 6862 },
	{ 0xe780, 0xe78b, PDF_CMAP_RANGE, 6879 },
	{ 0xe78c, 0xe78e, PDF_CMAP_TABLE, 73 },
	{ 0xe78f, 0xe7a6, PDF_CMAP_RANGE, 6894 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7d4, PDF_CMAP_RANGE, 6939 },
	{ 0xe7d5, 0xe7d5, PDF_CMAP_SINGLE, 7875 },
	{ 0xe7d6, 0xe7fc, PDF_CMAP_RANGE, 6965 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe884, PDF_CMAP_RANGE, 7067 },
	{ 0xe885, 0xe885, PDF_CMAP_SINGLE, 7876 },
	{ 0xe886, 0xe8b0, PDF_CMAP_RANGE, 7073 },
	{ 0xe8b1, 0xe8b1, PDF_CMAP_SINGLE, 7877 },
	{ 0xe8b2, 0xe8c2, PDF_CMAP_RANGE, 7117 },
	{ 0xe8c3, 0xe8c3, PDF_CMAP_SINGLE, 7878 },
	{ 0xe8c4, 0xe8ce, PDF_CMAP_RANGE, 7135 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8d4, PDF_CMAP_RANGE, 7147 },
	{ 0xe8d5, 0xe8d5, PDF_CMAP_SINGLE, 7880 },
	{ 0xe8d6, 0xe8f1, PDF_CMAP_RANGE, 7153 },
	{ 0xe8f2, 0xe8f3, PDF_CMAP_TABLE, 76 },
	{ 0xe8f4, 0xe8fc, PDF_CMAP_RANGE, 7183 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9ca, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cb, 0xe9cc, PDF_CMAP_TABLE, 78 },
	{ 0xe9cd, 0xe9f1, PDF_CMAP_RANGE, 7332 },
	{ 0xe9f2, 0xe9f2, PDF_CMAP_SINGLE, 1321 },
	{ 0xe9f3, 0xe9fc, PDF_CMAP_RANGE, 7370 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea7e, PDF_CMAP_RANGE, 7429 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9e, PDF_CMAP_TABLE, 80 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
	{ 0xe78c, 0xe78c, PDF_CMAP_SINGLE, 7871 },
	{ 0xe78d, 0xe78d, PDF_CMAP_SINGLE, 6892 },
	{ 0xe78e, 0xe78e, PDF_CMAP_SINGLE, 7872 },
	{ 0xe78f, 0xe7a6, PDF_CMAP_RANGE, 6894 },
	{ 0xe7a7, 0xe7a7, PDF_CMAP_SINGLE, 7873 },
	{ 0xe7a8, 0xe7ba, PDF_CMAP_RANGE, 6919 },
	{ 0xe7bb, 0xe7bb, PDF_CMAP_SINGLE, 7874 },
	{ 0xe7bc, 0xe7d4, PDF_CMAP_RANGE, 6939 },
	{ 0xe7d5, 0xe7d5, PDF_CMAP_SINGLE, 7875 },
	{ 0xe7d6, 0xe7fc, PDF_CMAP_RANGE, 6965 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe884, PDF_CMAP_RANGE, 7067 },
	{ 0xe885, 0xe885, PDF_CMAP_SINGLE, 7876 },
	{ 0xe886, 0xe8b0, PDF_CMAP_RANGE, 7073 },
	{ 0xe8b1, 0xe8b1, PDF_CMAP_SINGLE, 7877 },
	{ 0xe8b2, 0xe8c2, PDF_CMAP_RANGE, 7117 },
	{ 0xe8c3, 0xe8c3, PDF_CMAP_SINGLE, 7878 },
	{ 0xe8c4, 0xe8ce, PDF_CMAP_RANGE, 7135 },
	{ 0xe8cf, 0xe8cf, PDF_CMAP_SINGLE, 7879 },
	{ 0xe8d0, 0xe8d4, PDF_CMAP_RANGE, 7147 },
	{ 0xe8d5, 0xe8d5, PDF_CMAP_SINGLE, 7880 },
	{ 0xe8d6, 0xe8f1, PDF_CMAP_RANGE, 7153 },
	{ 0xe8f2, 0xe8f2, PDF_CMAP_SINGLE, 1841 },
	{ 0xe8f3, 0xe8f3, PDF_CMAP_SINGLE, 7881 },
	{ 0xe8f4, 0xe8fc, PDF_CMAP_RANGE, 7183 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9aa, PDF_CMAP_RANGE, 7255 },
	{ 0xe9ab, 0xe9ab, PDF_CMAP_SINGLE, 7882 },
	{ 0xe9ac, 0xe9b9, PDF_CMAP_RANGE, 7299 },
	{ 0xe9ba, 0xe9ba, PDF_CMAP_SINGLE, 7883 },
	{ 0xe9bb, 0xe9ca, PDF_CMAP_RANGE, 7314 },
	{ 0xe9cb, 0xe9cb, PDF_CMAP_SINGLE, 1143 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 7884 },
	{ 0xe9cd, 0xe9f1, PDF_CMAP_RANGE, 7332 },
	{ 0xe9f2, 0xe9f2, PDF_CMAP_SINGLE, 1321 },
	{ 0xe9f3, 0xe9fc, PDF_CMAP_RANGE, 7370 },
	{ 0xea40, 0xea6f, PDF_CMAP_RANGE, 7380 },
	{ 0xea70, 0xea70, PDF_CMAP_SINGLE, 7885 },
	{ 0xea71, 0xea7e, PDF_CMAP_RANGE, 7429 },
	{ 0xea80, 0xea9c, PDF_CMAP_RANGE, 7443 },
	{ 0xea9d, 0xea9d, PDF_CMAP_SINGLE, 7886 },
	{ 0xea9e, 0xea9e, PDF_CMAP_SINGLE, 7473 },
	{ 0xed40, 0xed7e, PDF_CMAP_RANGE, 8359 },
	{ 0xed80, 0xedb3, PDF_CMAP_RANGE, 8422 },
	{ 0xedb4, 0xedb4, PDF_CMAP_SINGLE, 1993 },
	{ 0xedb5, 0xedfc, PDF_CMAP_RANGE, 8474 },
	{ 0xee40, 0xee7e, PDF_CMAP_RANGE, 8546 },
	{ 0xee80, 0xeeec, PDF_CMAP_RANGE, 8609 },
	{ 0xeeef, 0xeef8, PDF_CMAP_RANGE, 8092 },
	{ 0xeef9, 0xeef9, PDF_CMAP_SINGLE, 751 },
	{ 0xeefa, 0xeefc, PDF_CMAP_RANGE, 8005 },
};

static const unsigned short pdf_cmap_Ext_RKSJ_H_table[82] =
{
	762,761,769,1125,7633,7645,7369,7646,
	7649,1406,1407,7650,7655,5490,7665,1780,
	7666,1782,7689,2169,7690,7693,6441,7720,
	6766,7733,2910,7734,2918,7744,3045,7745,
	7746,3050,7747,3099,7748,7772,6537,7773,
	7776,3395,7777,7783,5179,3491,7784,3663,
	7793,6453,7801,3847,7802,3849,7803,7816,
	4227,7817,7821,4434,7835,5199,3176,2030,
	5854,3153,1492,5938,7851,6089,7852,6440,
	2293,7871,6892,7872,1841,7881,1143,7884,
	7886,7473,
};

pdf_cmap pdf_cmap_Ext_RKSJ_H =
{
	-1, "Ext-RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	665, 665, (pdf_range*) pdf_cmap_Ext_RKSJ_H_ranges,
	82, 82, (unsigned short*) pdf_cmap_Ext_RKSJ_H_table,
};

/* Ext-RKSJ-V */

static const pdf_range pdf_cmap_Ext_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8143, 0x8144, PDF_CMAP_TABLE, 0 },
	{ 0x814a, 0x814b, PDF_CMAP_TABLE, 2 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8165, 0x8168, PDF_CMAP_TABLE, 4 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x818b, 0x818d, PDF_CMAP_TABLE, 8 },
	{ 0x81ac, 0x81ac, PDF_CMAP_SINGLE, 8270 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0x875f, 0x876e, PDF_CMAP_RANGE, 7940 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
	{ 0x875f, 0x876e, PDF_CMAP_RANGE, 7940 },
	{ 0x8780, 0x8781, PDF_CMAP_RANGE, 7956 },
};

static const unsigned short pdf_cmap_Ext_RKSJ_V_table[11] =
{
	8268,8274,8272,8271,8281,8276,8279,8278,
	8269,8273,8283,
};

pdf_cmap pdf_cmap_Ext_RKSJ_V =
{
	-1, "Ext-RKSJ-V", "Ext-RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	39, 39, (pdf_range*) pdf_cmap_Ext_RKSJ_V_ranges,
	11, 11, (unsigned short*) pdf_cmap_Ext_RKSJ_V_table,
};

/* Ext-V */

static const pdf_range pdf_cmap_Ext_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_RANGE, 7887 },
	{ 0x2124, 0x2125, PDF_CMAP_TABLE, 0 },
	{ 0x212b, 0x212c, PDF_CMAP_TABLE, 2 },
	{ 0x213c, 0x213e, PDF_CMAP_RANGE, 7891 },
	{ 0x2141, 0x2145, PDF_CMAP_RANGE, 7894 },
	{ 0x2146, 0x2149, PDF_CMAP_TABLE, 4 },
	{ 0x214a, 0x215b, PDF_CMAP_RANGE, 7899 },
	{ 0x216b, 0x216d, PDF_CMAP_TABLE, 8 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 8270 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 7918 },
	{ 0x2423, 0x2423, PDF_CMAP_SINGLE, 7919 },
	{ 0x2425, 0x2425, PDF_CMAP_SINGLE, 7920 },
	{ 0x2427, 0x2427, PDF_CMAP_SINGLE, 7921 },
	{ 0x2429, 0x2429, PDF_CMAP_SINGLE, 7922 },
	{ 0x2443, 0x2443, PDF_CMAP_SINGLE, 7923 },
	{ 0x2463, 0x2463, PDF_CMAP_SINGLE, 7924 },
	{ 0x2465, 0x2465, PDF_CMAP_SINGLE, 7925 },
	{ 0x2467, 0x2467, PDF_CMAP_SINGLE, 7926 },
	{ 0x246e, 0x246e, PDF_CMAP_SINGLE, 7927 },
	{ 0x2521, 0x2521, PDF_CMAP_SINGLE, 7928 },
	{ 0x2523, 0x2523, PDF_CMAP_SINGLE, 7929 },
	{ 0x2525, 0x2525, PDF_CMAP_SINGLE, 7930 },
	{ 0x2527, 0x2527, PDF_CMAP_SINGLE, 7931 },
	{ 0x2529, 0x2529, PDF_CMAP_SINGLE, 7932 },
	{ 0x2543, 0x2543, PDF_CMAP_SINGLE, 7933 },
	{ 0x2563, 0x2563, PDF_CMAP_SINGLE, 7934 },
	{ 0x2565, 0x2565, PDF_CMAP_SINGLE, 7935 },
	{ 0x2567, 0x2567, PDF_CMAP_SINGLE, 7936 },
	{ 0x256e, 0x256e, PDF_CMAP_SINGLE, 7937 },
	{ 0x2575, 0x2576, PDF_CMAP_RANGE, 7938 },
	{ 0x2d40, 0x2d4f, PDF_CMAP_RANGE, 7940 },
	{ 0x2d60, 0x2d61, PDF_CMAP_RANGE, 7956 },
	{ 0x2563, 0x2563, PDF_CMAP_SINGLE, 7934 },
	{ 0x2565, 0x2565, PDF_CMAP_SINGLE, 7935 },
	{ 0x2567, 0x2567, PDF_CMAP_SINGLE, 7936 },
	{ 0x256e, 0x256e, PDF_CMAP_SINGLE, 7937 },
	{ 0x2575, 0x2576, PDF_CMAP_RANGE, 7938 },
	{ 0x2d40, 0x2d4f, PDF_CMAP_RANGE, 7940 },
	{ 0x2d60, 0x2d61, PDF_CMAP_RANGE, 7956 },
};

static const unsigned short pdf_cmap_Ext_V_table[11] =
{
	8268,8274,8272,8271,8281,8276,8279,8278,
	8269,8273,8283,
};

pdf_cmap pdf_cmap_Ext_V =
{
	-1, "Ext-V", "Ext-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	39, 39, (pdf_range*) pdf_cmap_Ext_V_ranges,
	11, 11, (unsigned short*) pdf_cmap_Ext_V_table,
};

/* H */

static const pdf_range pdf_cmap_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 633 },
	{ 0x2221, 0x222e, PDF_CMAP_RANGE, 727 },
	{ 0x223a, 0x2241, PDF_CMAP_RANGE, 741 },
	{ 0x224a, 0x2250, PDF_CMAP_RANGE, 749 },
	{ 0x225c, 0x226a, PDF_CMAP_RANGE, 756 },
	{ 0x2272, 0x2279, PDF_CMAP_RANGE, 771 },
	{ 0x227e, 0x227e, PDF_CMAP_SINGLE, 779 },
	{ 0x2330, 0x2339, PDF_CMAP_RANGE, 780 },
	{ 0x2341, 0x235a, PDF_CMAP_RANGE, 790 },
	{ 0x2361, 0x237a, PDF_CMAP_RANGE, 816 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 842 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 925 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 1011 },
	{ 0x2641, 0x2658, PDF_CMAP_RANGE, 1035 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 1059 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 1092 },
	{ 0x2821, 0x2840, PDF_CMAP_TABLE, 0 },
	{ 0x3021, 0x307e, PDF_CMAP_RANGE, 1125 },
	{ 0x3121, 0x317e, PDF_CMAP_RANGE, 1219 },
	{ 0x3221, 0x327e, PDF_CMAP_RANGE, 1313 },
	{ 0x3321, 0x337e, PDF_CMAP_RANGE, 1407 },
	{ 0x3421, 0x347e, PDF_CMAP_RANGE, 1501 },
	{ 0x3521, 0x357e, PDF_CMAP_RANGE, 1595 },
	{ 0x3621, 0x367e, PDF_CMAP_RANGE, 1689 },
	{ 0x3721, 0x377e, PDF_CMAP_RANGE, 1783 },
	{ 0x3821, 0x387e, PDF_CMAP_RANGE, 1877 },
	{ 0x3921, 0x397e, PDF_CMAP_RANGE, 1971 },
	{ 0x3a21, 0x3a7e, PDF_CMAP_RANGE, 2065 },
	{ 0x3b21, 0x3b7e, PDF_CMAP_RANGE, 2159 },
	{ 0x3c21, 0x3c7e, PDF_CMAP_RANGE, 2253 },
	{ 0x3d21, 0x3d7e, PDF_CMAP_RANGE, 2347 },
	{ 0x3e21, 0x3e7e, PDF_CMAP_RANGE, 2441 },
	{ 0x3f21, 0x3f7e, PDF_CMAP_RANGE, 2535 },
	{ 0x4021, 0x407e, PDF_CMAP_RANGE, 2629 },
	{ 0x4121, 0x417e, PDF_CMAP_RANGE, 2723 },
	{ 0x4221, 0x427e, PDF_CMAP_RANGE, 2817 },
	{ 0x4321, 0x437e, PDF_CMAP_RANGE, 2911 },
	{ 0x4421, 0x447e, PDF_CMAP_RANGE, 3005 },
	{ 0x4521, 0x457e, PDF_CMAP_RANGE, 3099 },
	{ 0x4621, 0x467e, PDF_CMAP_RANGE, 3193 },
	{ 0x4721, 0x477e, PDF_CMAP_RANGE, 3287 },
	{ 0x4821, 0x487e, PDF_CMAP_RANGE, 3381 },
	{ 0x4921, 0x497e, PDF_CMAP_RANGE, 3475 },
	{ 0x4a21, 0x4a7e, PDF_CMAP_RANGE, 3569 },
	{ 0x4b21, 0x4b7e, PDF_CMAP_RANGE, 3663 },
	{ 0x4c21, 0x4c7e, PDF_CMAP_RANGE, 3757 },
	{ 0x4d21, 0x4d7e, PDF_CMAP_RANGE, 3851 },
	{ 0x4e21, 0x4e7e, PDF_CMAP_RANGE, 3945 },
	{ 0x4f21, 0x4f53, PDF_CMAP_RANGE, 4039 },
	{ 0x5021, 0x507e, PDF_CMAP_RANGE, 4090 },
	{ 0x5121, 0x517e, PDF_CMAP_RANGE, 4184 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 4278 },
	{ 0x5321, 0x537e, PDF_CMAP_RANGE, 4372 },
	{ 0x5421, 0x547e, PDF_CMAP_RANGE, 4466 },
	{ 0x5521, 0x557e, PDF_CMAP_RANGE, 4560 },
	{ 0x5621, 0x567e, PDF_CMAP_RANGE, 4654 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 4748 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 4842 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 4936 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 5030 },
	{ 0x5b21, 0x5b7e, PDF_CMAP_RANGE, 5124 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5218 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 5312 },
	{ 0x5e21, 0x5e7e, PDF_CMAP_RANGE, 5406 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 5500 },
	{ 0x6021, 0x607e, PDF_CMAP_RANGE, 5594 },
	{ 0x6121, 0x617e, PDF_CMAP_RANGE, 5688 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 5782 },
	{ 0x6321, 0x637e, PDF_CMAP_RANGE, 5876 },
	{ 0x6421, 0x647e, PDF_CMAP_RANGE, 5970 },
	{ 0x6521, 0x657e, PDF_CMAP_RANGE, 6064 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 6158 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 6252 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 6346 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 6440 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 6534 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 6628 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 6722 },
	{ 0x6d21, 0x6d7e, PDF_CMAP_RANGE, 6816 },
	{ 0x6e21, 0x6e7e, PDF_CMAP_RANGE, 6910 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 7004 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 7098 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x727e, PDF_CMAP_RANGE, 7286 },
	{ 0x7321, 0x737e, PDF_CMAP_RANGE, 7380 },
	{ 0x7421, 0x7424, PDF_CMAP_RANGE, 7474 },
	{ 0x7425, 0x7426, PDF_CMAP_RANGE, 8284 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 4748 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 4842 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 4936 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 5030 },
	{ 0x5b21, 0x5b7e, PDF_CMAP_RANGE, 5124 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5218 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 5312 },
	{ 0x5e21, 0x5e7e, PDF_CMAP_RANGE, 5406 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 5500 },
	{ 0x6021, 0x607e, PDF_CMAP_RANGE, 5594 },
	{ 0x6121, 0x617e, PDF_CMAP_RANGE, 5688 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 5782 },
	{ 0x6321, 0x637e, PDF_CMAP_RANGE, 5876 },
	{ 0x6421, 0x647e, PDF_CMAP_RANGE, 5970 },
	{ 0x6521, 0x657e, PDF_CMAP_RANGE, 6064 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 6158 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 6252 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 6346 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 6440 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 6534 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 6628 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 6722 },
	{ 0x6d21, 0x6d7e, PDF_CMAP_RANGE, 6816 },
	{ 0x6e21, 0x6e7e, PDF_CMAP_RANGE, 6910 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 7004 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 7098 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x727e, PDF_CMAP_RANGE, 7286 },
	{ 0x7321, 0x737e, PDF_CMAP_RANGE, 7380 },
	{ 0x7421, 0x7424, PDF_CMAP_RANGE, 7474 },
	{ 0x7425, 0x7426, PDF_CMAP_RANGE, 8284 },
};

static const unsigned short pdf_cmap_H_table[32] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
};

pdf_cmap pdf_cmap_H =
{
	-1, "H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	118, 118, (pdf_range*) pdf_cmap_H_ranges,
	32, 32, (unsigned short*) pdf_cmap_H_table,
};

/* Hankaku */

static const pdf_range pdf_cmap_Hankaku_ranges[] =
{
	{ 0x0020, 0x005f, PDF_CMAP_RANGE, 231 },
	{ 0x0060, 0x0060, PDF_CMAP_SINGLE, 231 },
	{ 0x0061, 0x007e, PDF_CMAP_RANGE, 296 },
	{ 0x0081, 0x0085, PDF_CMAP_RANGE, 327 },
	{ 0x0086, 0x008f, PDF_CMAP_RANGE, 516 },
	{ 0x0090, 0x0090, PDF_CMAP_SINGLE, 342 },
	{ 0x0091, 0x009f, PDF_CMAP_RANGE, 526 },
	{ 0x00a1, 0x00df, PDF_CMAP_RANGE, 327 },
	{ 0x00e0, 0x00fd, PDF_CMAP_RANGE, 541 },
	{ 0x00fe, 0x00ff, PDF_CMAP_RANGE, 388 },
};

static const unsigned short pdf_cmap_Hankaku_table[1] = { 0 };

pdf_cmap pdf_cmap_Hankaku =
{
	-1, "Hankaku", "", nil, 0,
	1, /* codespace table */
	{
		{ 1, 0x0000, 0x00ff },
	},
	10, 10, (pdf_range*) pdf_cmap_Hankaku_ranges,
	0, 0, (unsigned short*) pdf_cmap_Hankaku_table,
};

/* Hiragana */

static const pdf_range pdf_cmap_Hiragana_ranges[] =
{
	{ 0x0020, 0x0020, PDF_CMAP_SINGLE, 515 },
	{ 0x0021, 0x0025, PDF_CMAP_RANGE, 327 },
	{ 0x0026, 0x002f, PDF_CMAP_RANGE, 516 },
	{ 0x0030, 0x0030, PDF_CMAP_SINGLE, 342 },
	{ 0x0031, 0x005d, PDF_CMAP_RANGE, 526 },
	{ 0x005e, 0x005f, PDF_CMAP_RANGE, 388 },
	{ 0x0060, 0x0062, PDF_CMAP_RANGE, 571 },
	{ 0x0066, 0x007e, PDF_CMAP_RANGE, 574 },
};

static const unsigned short pdf_cmap_Hiragana_table[1] = { 0 };

pdf_cmap pdf_cmap_Hiragana =
{
	-1, "Hiragana", "", nil, 0,
	1, /* codespace table */
	{
		{ 1, 0x0000, 0x00ff },
	},
	8, 8, (pdf_range*) pdf_cmap_Hiragana_ranges,
	0, 0, (unsigned short*) pdf_cmap_Hiragana_table,
};

/* Katakana */

static const pdf_range pdf_cmap_Katakana_ranges[] =
{
	{ 0x0020, 0x005f, PDF_CMAP_RANGE, 326 },
	{ 0x0060, 0x007e, PDF_CMAP_RANGE, 391 },
};

static const unsigned short pdf_cmap_Katakana_table[1] = { 0 };

pdf_cmap pdf_cmap_Katakana =
{
	-1, "Katakana", "", nil, 0,
	1, /* codespace table */
	{
		{ 1, 0x0000, 0x00ff },
	},
	2, 2, (pdf_range*) pdf_cmap_Katakana_ranges,
	0, 0, (unsigned short*) pdf_cmap_Katakana_table,
};

/* NWP-H */

static const pdf_range pdf_cmap_NWP_H_ranges[] =
{
	{ 0x2121, 0x2168, PDF_CMAP_RANGE, 633 },
	{ 0x2169, 0x2169, PDF_CMAP_SINGLE, 7478 },
	{ 0x216a, 0x217e, PDF_CMAP_RANGE, 706 },
	{ 0x2221, 0x222e, PDF_CMAP_RANGE, 727 },
	{ 0x223a, 0x2241, PDF_CMAP_RANGE, 741 },
	{ 0x224a, 0x2250, PDF_CMAP_RANGE, 749 },
	{ 0x225c, 0x226a, PDF_CMAP_RANGE, 756 },
	{ 0x2272, 0x2279, PDF_CMAP_RANGE, 771 },
	{ 0x227e, 0x227e, PDF_CMAP_SINGLE, 779 },
	{ 0x2330, 0x2339, PDF_CMAP_RANGE, 780 },
	{ 0x2341, 0x235a, PDF_CMAP_RANGE, 790 },
	{ 0x2361, 0x237a, PDF_CMAP_RANGE, 816 },
	{ 0x2421, 0x2473, PDF_CMAP_RANGE, 842 },
	{ 0x2521, 0x2576, PDF_CMAP_RANGE, 925 },
	{ 0x2621, 0x2638, PDF_CMAP_RANGE, 1011 },
	{ 0x2641, 0x2658, PDF_CMAP_RANGE, 1035 },
	{ 0x2721, 0x2741, PDF_CMAP_RANGE, 1059 },
	{ 0x2751, 0x2771, PDF_CMAP_RANGE, 1092 },
	{ 0x2921, 0x295f, PDF_CMAP_RANGE, 232 },
	{ 0x2960, 0x2960, PDF_CMAP_SINGLE, 390 },
	{ 0x2961, 0x297e, PDF_CMAP_RANGE, 296 },
	{ 0x2a21, 0x2a5f, PDF_CMAP_RANGE, 327 },
	{ 0x2a60, 0x2a7e, PDF_CMAP_RANGE, 391 },
	{ 0x2b21, 0x2b71, PDF_CMAP_RANGE, 422 },
	{ 0x2b72, 0x2b72, PDF_CMAP_SINGLE, 295 },
	{ 0x2b73, 0x2b7e, PDF_CMAP_RANGE, 503 },
	{ 0x2c24, 0x2c6f, PDF_CMAP_RANGE, 7479 },
	{ 0x2d21, 0x2d3e, PDF_CMAP_RANGE, 7555 },
	{ 0x2d40, 0x2d56, PDF_CMAP_RANGE, 7585 },
	{ 0x2d60, 0x2d6f, PDF_CMAP_RANGE, 7608 },
	{ 0x2d70, 0x2d72, PDF_CMAP_TABLE, 0 },
	{ 0x2d73, 0x2d79, PDF_CMAP_RANGE, 7624 },
	{ 0x2d7a, 0x2d7a, PDF_CMAP_SINGLE, 768 },
	{ 0x2d7b, 0x2d7c, PDF_CMAP_RANGE, 7631 },
	{ 0x2e21, 0x2e27, PDF_CMAP_TABLE, 3 },
	{ 0x2e28, 0x2e29, PDF_CMAP_RANGE, 674 },
	{ 0x2e2a, 0x2e2f, PDF_CMAP_TABLE, 10 },
	{ 0x2e30, 0x2e39, PDF_CMAP_RANGE, 780 },
	{ 0x2e3a, 0x2e3b, PDF_CMAP_RANGE, 639 },
	{ 0x2e3c, 0x2e40, PDF_CMAP_TABLE, 16 },
	{ 0x2e41, 0x2e5a, PDF_CMAP_RANGE, 790 },
	{ 0x2e5b, 0x2e60, PDF_CMAP_TABLE, 21 },
	{ 0x2e61, 0x2e7a, PDF_CMAP_RANGE, 816 },
	{ 0x2e7b, 0x2e7e, PDF_CMAP_TABLE, 27 },
	{ 0x2f21, 0x2f21, PDF_CMAP_SINGLE, 635 },
	{ 0x2f22, 0x2f23, PDF_CMAP_RANGE, 686 },
	{ 0x2f24, 0x2f34, PDF_CMAP_TABLE, 31 },
	{ 0x2f35, 0x2f36, PDF_CMAP_RANGE, 934 },
	{ 0x2f37, 0x2f44, PDF_CMAP_TABLE, 48 },
	{ 0x2f45, 0x2f4a, PDF_CMAP_RANGE, 966 },
	{ 0x2f4b, 0x2f4e, PDF_CMAP_TABLE, 62 },
	{ 0x2f4f, 0x2f53, PDF_CMAP_RANGE, 986 },
	{ 0x2f54, 0x2f55, PDF_CMAP_TABLE, 66 },
	{ 0x2f56, 0x2f5b, PDF_CMAP_RANGE, 996 },
	{ 0x2f5c, 0x2f5d, PDF_CMAP_TABLE, 68 },
	{ 0x2f5e, 0x2f5f, PDF_CMAP_RANGE, 643 },
	{ 0x2f60, 0x2f61, PDF_CMAP_RANGE, 1004 },
	{ 0x2f62, 0x2f62, PDF_CMAP_SINGLE, 1002 },
	{ 0x2f63, 0x2f64, PDF_CMAP_RANGE, 1009 },
	{ 0x2f65, 0x2f74, PDF_CMAP_TABLE, 70 },
	{ 0x2f75, 0x2f76, PDF_CMAP_RANGE, 972 },
	{ 0x2f77, 0x2f78, PDF_CMAP_RANGE, 975 },
	{ 0x2f79, 0x2f7a, PDF_CMAP_RANGE, 978 },
	{ 0x2f7b, 0x2f7c, PDF_CMAP_RANGE, 981 },
	{ 0x2f7d, 0x2f7e, PDF_CMAP_RANGE, 984 },
	{ 0x3021, 0x3022, PDF_CMAP_TABLE, 86 },
	{ 0x3023, 0x3032, PDF_CMAP_RANGE, 1127 },
	{ 0x3033, 0x3033, PDF_CMAP_SINGLE, 7330 },
	{ 0x3034, 0x303a, PDF_CMAP_RANGE, 1144 },
	{ 0x303b, 0x303b, PDF_CMAP_SINGLE, 7634 },
	{ 0x303c, 0x306d, PDF_CMAP_RANGE, 1152 },
	{ 0x306e, 0x306e, PDF_CMAP_SINGLE, 7635 },
	{ 0x306f, 0x3072, PDF_CMAP_RANGE, 1203 },
	{ 0x3073, 0x3073, PDF_CMAP_SINGLE, 7636 },
	{ 0x3074, 0x307b, PDF_CMAP_RANGE, 1208 },
	{ 0x307c, 0x307c, PDF_CMAP_SINGLE, 7637 },
	{ 0x307d, 0x307e, PDF_CMAP_RANGE, 1217 },
	{ 0x3121, 0x3129, PDF_CMAP_RANGE, 1219 },
	{ 0x312a, 0x312a, PDF_CMAP_SINGLE, 7638 },
	{ 0x312b, 0x3134, PDF_CMAP_RANGE, 1229 },
	{ 0x3135, 0x3135, PDF_CMAP_SINGLE, 7639 },
	{ 0x3136, 0x3138, PDF_CMAP_RANGE, 1240 },
	{ 0x3139, 0x3139, PDF_CMAP_SINGLE, 7640 },
	{ 0x313a, 0x313b, PDF_CMAP_RANGE, 1244 },
	{ 0x313c, 0x313d, PDF_CMAP_RANGE, 7641 },
	{ 0x313e, 0x3141, PDF_CMAP_RANGE, 1248 },
	{ 0x3142, 0x3142, PDF_CMAP_SINGLE, 7643 },
	{ 0x3143, 0x316a, PDF_CMAP_RANGE, 1253 },
	{ 0x316b, 0x316b, PDF_CMAP_SINGLE, 7644 },
	{ 0x316c, 0x317e, PDF_CMAP_RANGE, 1294 },
	{ 0x3221, 0x3227, PDF_CMAP_RANGE, 1313 },
	{ 0x3228, 0x322a, PDF_CMAP_TABLE, 88 },
	{ 0x322b, 0x325f, PDF_CMAP_RANGE, 1323 },
	{ 0x3260, 0x3260, PDF_CMAP_SINGLE, 7647 },
	{ 0x3261, 0x3279, PDF_CMAP_RANGE, 1377 },
	{ 0x327a, 0x327a, PDF_CMAP_SINGLE, 7648 },
	{ 0x327b, 0x327c, PDF_CMAP_RANGE, 1403 },
	{ 0x327d, 0x327e, PDF_CMAP_TABLE, 91 },
	{ 0x3321, 0x3322, PDF_CMAP_TABLE, 93 },
	{ 0x3323, 0x3341, PDF_CMAP_RANGE, 1409 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 6497 },
	{ 0x3343, 0x3348, PDF_CMAP_RANGE, 1441 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 5023 },
	{ 0x334a, 0x3364, PDF_CMAP_RANGE, 1448 },
	{ 0x3365, 0x3365, PDF_CMAP_SINGLE, 7651 },
	{ 0x3366, 0x336a, PDF_CMAP_RANGE, 1476 },
	{ 0x336b, 0x336b, PDF_CMAP_SINGLE, 7652 },
	{ 0x336c, 0x3372, PDF_CMAP_RANGE, 1482 },
	{ 0x3373, 0x3373, PDF_CMAP_SINGLE, 7653 },
	{ 0x3374, 0x3375, PDF_CMAP_RANGE, 1490 },
	{ 0x3376, 0x3376, PDF_CMAP_SINGLE, 5937 },
	{ 0x3377, 0x3379, PDF_CMAP_RANGE, 1493 },
	{ 0x337a, 0x337a, PDF_CMAP_SINGLE, 7654 },
	{ 0x337b, 0x337e, PDF_CMAP_RANGE, 1497 },
	{ 0x3421, 0x3441, PDF_CMAP_RANGE, 1501 },
	{ 0x3442, 0x3443, PDF_CMAP_TABLE, 95 },
	{ 0x3444, 0x344c, PDF_CMAP_RANGE, 1536 },
	{ 0x344d, 0x344d, PDF_CMAP_SINGLE, 7656 },
	{ 0x344e, 0x3451, PDF_CMAP_RANGE, 1546 },
	{ 0x3452, 0x3452, PDF_CMAP_SINGLE, 6688 },
	{ 0x3453, 0x3464, PDF_CMAP_RANGE, 1551 },
	{ 0x3465, 0x3465, PDF_CMAP_SINGLE, 7657 },
	{ 0x3466, 0x347e, PDF_CMAP_RANGE, 1570 },
	{ 0x3521, 0x352a, PDF_CMAP_RANGE, 1595 },
	{ 0x352b, 0x352b, PDF_CMAP_SINGLE, 7658 },
	{ 0x352c, 0x353f, PDF_CMAP_RANGE, 1606 },
	{ 0x3540, 0x3540, PDF_CMAP_SINGLE, 7659 },
	{ 0x3541, 0x357e, PDF_CMAP_RANGE, 1627 },
	{ 0x3621, 0x3622, PDF_CMAP_TABLE, 97 },
	{ 0x3623, 0x3629, PDF_CMAP_RANGE, 1691 },
	{ 0x362a, 0x362a, PDF_CMAP_SINGLE, 7661 },
	{ 0x362b, 0x3645, PDF_CMAP_RANGE, 1699 },
	{ 0x3646, 0x3646, PDF_CMAP_SINGLE, 7474 },
	{ 0x3647, 0x364e, PDF_CMAP_RANGE, 1727 },
	{ 0x364f, 0x364f, PDF_CMAP_SINGLE, 7662 },
	{ 0x3650, 0x366c, PDF_CMAP_RANGE, 1736 },
	{ 0x366d, 0x366d, PDF_CMAP_SINGLE, 7663 },
	{ 0x366e, 0x3673, PDF_CMAP_RANGE, 1766 },
	{ 0x3674, 0x3674, PDF_CMAP_SINGLE, 7664 },
	{ 0x3675, 0x367a, PDF_CMAP_RANGE, 1773 },
	{ 0x367b, 0x367e, PDF_CMAP_TABLE, 99 },
	{ 0x3721, 0x3723, PDF_CMAP_RANGE, 1783 },
	{ 0x3724, 0x3724, PDF_CMAP_SINGLE, 7667 },
	{ 0x3725, 0x3736, PDF_CMAP_RANGE, 1787 },
	{ 0x3737, 0x3737, PDF_CMAP_SINGLE, 7668 },
	{ 0x3738, 0x3744, PDF_CMAP_RANGE, 1806 },
	{ 0x3745, 0x3745, PDF_CMAP_SINGLE, 7669 },
	{ 0x3746, 0x374d, PDF_CMAP_RANGE, 1820 },
	{ 0x374e, 0x374e, PDF_CMAP_SINGLE, 7670 },
	{ 0x374f, 0x3751, PDF_CMAP_RANGE, 1829 },
	{ 0x3752, 0x3752, PDF_CMAP_SINGLE, 7671 },
	{ 0x3753, 0x3754, PDF_CMAP_RANGE, 1833 },
	{ 0x3755, 0x3755, PDF_CMAP_SINGLE, 7672 },
	{ 0x3756, 0x375a, PDF_CMAP_RANGE, 1836 },
	{ 0x375b, 0x375b, PDF_CMAP_SINGLE, 7181 },
	{ 0x375c, 0x3763, PDF_CMAP_RANGE, 1842 },
	{ 0x3764, 0x3764, PDF_CMAP_SINGLE, 7673 },
	{ 0x3765, 0x3770, PDF_CMAP_RANGE, 1851 },
	{ 0x3771, 0x3771, PDF_CMAP_SINGLE, 7674 },
	{ 0x3772, 0x3778, PDF_CMAP_RANGE, 1864 },
	{ 0x3779, 0x3779, PDF_CMAP_SINGLE, 7675 },
	{ 0x377a, 0x377d, PDF_CMAP_RANGE, 1872 },
	{ 0x377e, 0x377e, PDF_CMAP_SINGLE, 7676 },
	{ 0x3821, 0x3833, PDF_CMAP_RANGE, 1877 },
	{ 0x3834, 0x3834, PDF_CMAP_SINGLE, 7677 },
	{ 0x3835, 0x3840, PDF_CMAP_RANGE, 1897 },
	{ 0x3841, 0x3841, PDF_CMAP_SINGLE, 7678 },
	{ 0x3842, 0x387e, PDF_CMAP_RANGE, 1910 },
	{ 0x3921, 0x392a, PDF_CMAP_RANGE, 1971 },
	{ 0x392b, 0x392b, PDF_CMAP_SINGLE, 7679 },
	{ 0x392c, 0x3936, PDF_CMAP_RANGE, 1982 },
	{ 0x3937, 0x3937, PDF_CMAP_SINGLE, 7680 },
	{ 0x3938, 0x3941, PDF_CMAP_RANGE, 1994 },
	{ 0x3942, 0x3942, PDF_CMAP_SINGLE, 7681 },
	{ 0x3943, 0x395b, PDF_CMAP_RANGE, 2005 },
	{ 0x395c, 0x395c, PDF_CMAP_SINGLE, 5853 },
	{ 0x395d, 0x396c, PDF_CMAP_RANGE, 2031 },
	{ 0x396d, 0x396d, PDF_CMAP_SINGLE, 7682 },
	{ 0x396e, 0x3973, PDF_CMAP_RANGE, 2048 },
	{ 0x3974, 0x3974, PDF_CMAP_SINGLE, 7683 },
	{ 0x3975, 0x3978, PDF_CMAP_RANGE, 2055 },
	{ 0x3979, 0x3979, PDF_CMAP_SINGLE, 7684 },
	{ 0x397a, 0x397e, PDF_CMAP_RANGE, 2060 },
	{ 0x3a21, 0x3a52, PDF_CMAP_RANGE, 2065 },
	{ 0x3a53, 0x3a53, PDF_CMAP_SINGLE, 7685 },
	{ 0x3a54, 0x3a66, PDF_CMAP_RANGE, 2116 },
	{ 0x3a67, 0x3a67, PDF_CMAP_SINGLE, 7686 },
	{ 0x3a68, 0x3a73, PDF_CMAP_RANGE, 2136 },
	{ 0x3a74, 0x3a74, PDF_CMAP_SINGLE, 7687 },
	{ 0x3a75, 0x3a7e, PDF_CMAP_RANGE, 2149 },
	{ 0x3b21, 0x3b26, PDF_CMAP_RANGE, 2159 },
	{ 0x3b27, 0x3b27, PDF_CMAP_SINGLE, 7688 },
	{ 0x3b28, 0x3b29, PDF_CMAP_RANGE, 2166 },
	{ 0x3b2a, 0x3b2c, PDF_CMAP_TABLE, 103 },
	{ 0x3b2d, 0x3b38, PDF_CMAP_RANGE, 2171 },
	{ 0x3b39, 0x3b39, PDF_CMAP_SINGLE, 7691 },
	{ 0x3b3a, 0x3b7e, PDF_CMAP_RANGE, 2184 },
	{ 0x3c21, 0x3c37, PDF_CMAP_RANGE, 2253 },
	{ 0x3c38, 0x3c38, PDF_CMAP_SINGLE, 7692 },
	{ 0x3c39, 0x3c47, PDF_CMAP_RANGE, 2277 },
	{ 0x3c48, 0x3c49, PDF_CMAP_TABLE, 106 },
	{ 0x3c4a, 0x3c56, PDF_CMAP_RANGE, 2294 },
	{ 0x3c57, 0x3c57, PDF_CMAP_SINGLE, 7694 },
	{ 0x3c58, 0x3c5c, PDF_CMAP_RANGE, 2308 },
	{ 0x3c5d, 0x3c5e, PDF_CMAP_RANGE, 7695 },
	{ 0x3c5f, 0x3c7e, PDF_CMAP_RANGE, 2315 },
	{ 0x3d21, 0x3d2a, PDF_CMAP_RANGE, 2347 },
	{ 0x3d2b, 0x3d2b, PDF_CMAP_SINGLE, 7697 },
	{ 0x3d2c, 0x3d35, PDF_CMAP_RANGE, 2358 },
	{ 0x3d36, 0x3d36, PDF_CMAP_SINGLE, 7698 },
	{ 0x3d37, 0x3d6b, PDF_CMAP_RANGE, 2369 },
	{ 0x3d6c, 0x3d6d, PDF_CMAP_RANGE, 7699 },
	{ 0x3d6e, 0x3d71, PDF_CMAP_RANGE, 2424 },
	{ 0x3d72, 0x3d73, PDF_CMAP_RANGE, 7701 },
	{ 0x3d74, 0x3d7e, PDF_CMAP_RANGE, 2430 },
	{ 0x3e21, 0x3e24, PDF_CMAP_RANGE, 2441 },
	{ 0x3e25, 0x3e25, PDF_CMAP_SINGLE, 7703 },
	{ 0x3e26, 0x3e32, PDF_CMAP_RANGE, 2446 },
	{ 0x3e33, 0x3e33, PDF_CMAP_SINGLE, 7704 },
	{ 0x3e34, 0x3e3e, PDF_CMAP_RANGE, 2460 },
	{ 0x3e3f, 0x3e3f, PDF_CMAP_SINGLE, 7705 },
	{ 0x3e40, 0x3e54, PDF_CMAP_RANGE, 2472 },
	{ 0x3e55, 0x3e55, PDF_CMAP_SINGLE, 7706 },
	{ 0x3e56, 0x3e5e, PDF_CMAP_RANGE, 2494 },
	{ 0x3e5f, 0x3e5f, PDF_CMAP_SINGLE, 7707 },
	{ 0x3e60, 0x3e63, PDF_CMAP_RANGE, 2504 },
	{ 0x3e64, 0x3e64, PDF_CMAP_SINGLE, 7708 },
	{ 0x3e65, 0x3e7e, PDF_CMAP_RANGE, 2509 },
	{ 0x3f21, 0x3f29, PDF_CMAP_RANGE, 2535 },
	{ 0x3f2a, 0x3f2a, PDF_CMAP_SINGLE, 7709 },
	{ 0x3f2b, 0x3f58, PDF_CMAP_RANGE, 2545 },
	{ 0x3f59, 0x3f59, PDF_CMAP_SINGLE, 7710 },
	{ 0x3f5a, 0x3f5f, PDF_CMAP_RANGE, 2592 },
	{ 0x3f60, 0x3f60, PDF_CMAP_SINGLE, 7711 },
	{ 0x3f61, 0x3f68, PDF_CMAP_RANGE, 2599 },
	{ 0x3f69, 0x3f69, PDF_CMAP_SINGLE, 7712 },
	{ 0x3f6a, 0x3f7e, PDF_CMAP_RANGE, 2608 },
	{ 0x4021, 0x4022, PDF_CMAP_TABLE, 108 },
	{ 0x4023, 0x4041, PDF_CMAP_RANGE, 2631 },
	{ 0x4042, 0x4042, PDF_CMAP_SINGLE, 7714 },
	{ 0x4043, 0x4065, PDF_CMAP_RANGE, 2663 },
	{ 0x4066, 0x4066, PDF_CMAP_SINGLE, 7715 },
	{ 0x4067, 0x4070, PDF_CMAP_RANGE, 2699 },
	{ 0x4071, 0x4072, PDF_CMAP_RANGE, 7716 },
	{ 0x4073, 0x4078, PDF_CMAP_RANGE, 2711 },
	{ 0x4079, 0x407a, PDF_CMAP_RANGE, 7718 },
	{ 0x407b, 0x407e, PDF_CMAP_RANGE, 2719 },
	{ 0x4121, 0x4126, PDF_CMAP_RANGE, 2723 },
	{ 0x4127, 0x4128, PDF_CMAP_TABLE, 110 },
	{ 0x4129, 0x4138, PDF_CMAP_RANGE, 2731 },
	{ 0x4139, 0x4139, PDF_CMAP_SINGLE, 7721 },
	{ 0x413a, 0x414b, PDF_CMAP_RANGE, 2748 },
	{ 0x414c, 0x414c, PDF_CMAP_SINGLE, 7722 },
	{ 0x414d, 0x414e, PDF_CMAP_RANGE, 2767 },
	{ 0x414f, 0x414f, PDF_CMAP_SINGLE, 7723 },
	{ 0x4150, 0x415e, PDF_CMAP_RANGE, 2770 },
	{ 0x415f, 0x415f, PDF_CMAP_SINGLE, 7724 },
	{ 0x4160, 0x4168, PDF_CMAP_RANGE, 2786 },
	{ 0x4169, 0x4169, PDF_CMAP_SINGLE, 7725 },
	{ 0x416a, 0x417e, PDF_CMAP_RANGE, 2796 },
	{ 0x4221, 0x423c, PDF_CMAP_RANGE, 2817 },
	{ 0x423d, 0x423d, PDF_CMAP_SINGLE, 7726 },
	{ 0x423e, 0x424c, PDF_CMAP_RANGE, 2846 },
	{ 0x424d, 0x424d, PDF_CMAP_SINGLE, 7727 },
	{ 0x424e, 0x425b, PDF_CMAP_RANGE, 2862 },
	{ 0x425c, 0x425c, PDF_CMAP_SINGLE, 7728 },
	{ 0x425d, 0x4262, PDF_CMAP_RANGE, 2877 },
	{ 0x4263, 0x4263, PDF_CMAP_SINGLE, 7729 },
	{ 0x4264, 0x426e, PDF_CMAP_RANGE, 2884 },
	{ 0x426f, 0x426f, PDF_CMAP_SINGLE, 7730 },
	{ 0x4270, 0x4274, PDF_CMAP_RANGE, 2896 },
	{ 0x4275, 0x4276, PDF_CMAP_RANGE, 7731 },
	{ 0x4277, 0x427c, PDF_CMAP_RANGE, 2903 },
	{ 0x427d, 0x427e, PDF_CMAP_TABLE, 112 },
	{ 0x4321, 0x4326, PDF_CMAP_RANGE, 2911 },
	{ 0x4327, 0x4328, PDF_CMAP_TABLE, 114 },
	{ 0x4329, 0x432a, PDF_CMAP_RANGE, 7735 },
	{ 0x432b, 0x432c, PDF_CMAP_RANGE, 2921 },
	{ 0x432d, 0x432e, PDF_CMAP_RANGE, 7737 },
	{ 0x432f, 0x433c, PDF_CMAP_RANGE, 2925 },
	{ 0x433d, 0x433d, PDF_CMAP_SINGLE, 7739 },
	{ 0x433e, 0x436f, PDF_CMAP_RANGE, 2940 },
	{ 0x4370, 0x4370, PDF_CMAP_SINGLE, 7740 },
	{ 0x4371, 0x4374, PDF_CMAP_RANGE, 2991 },
	{ 0x4375, 0x4375, PDF_CMAP_SINGLE, 7741 },
	{ 0x4376, 0x437b, PDF_CMAP_RANGE, 2996 },
	{ 0x437c, 0x437c, PDF_CMAP_SINGLE, 7742 },
	{ 0x437d, 0x437e, PDF_CMAP_RANGE, 3003 },
	{ 0x4421, 0x443c, PDF_CMAP_RANGE, 3005 },
	{ 0x443d, 0x443d, PDF_CMAP_SINGLE, 7743 },
	{ 0x443e, 0x4447, PDF_CMAP_RANGE, 3034 },
	{ 0x4448, 0x444a, PDF_CMAP_TABLE, 116 },
	{ 0x444b, 0x444c, PDF_CMAP_RANGE, 3047 },
	{ 0x444d, 0x444f, PDF_CMAP_TABLE, 119 },
	{ 0x4450, 0x445a, PDF_CMAP_RANGE, 3052 },
	{ 0x445b, 0x445b, PDF_CMAP_SINGLE, 4533 },
	{ 0x445c, 0x447e, PDF_CMAP_RANGE, 3064 },
	{ 0x4521, 0x4522, PDF_CMAP_TABLE, 122 },
	{ 0x4523, 0x4526, PDF_CMAP_RANGE, 3101 },
	{ 0x4527, 0x4527, PDF_CMAP_SINGLE, 7749 },
	{ 0x4528, 0x452d, PDF_CMAP_RANGE, 3106 },
	{ 0x452e, 0x452e, PDF_CMAP_SINGLE, 7750 },
	{ 0x452f, 0x4535, PDF_CMAP_RANGE, 3113 },
	{ 0x4536, 0x4536, PDF_CMAP_SINGLE, 7751 },
	{ 0x4537, 0x453e, PDF_CMAP_RANGE, 3121 },
	{ 0x453f, 0x453f, PDF_CMAP_SINGLE, 7752 },
	{ 0x4540, 0x4547, PDF_CMAP_RANGE, 3130 },
	{ 0x4548, 0x4548, PDF_CMAP_SINGLE, 7753 },
	{ 0x4549, 0x454a, PDF_CMAP_RANGE, 3139 },
	{ 0x454b, 0x454b, PDF_CMAP_SINGLE, 7754 },
	{ 0x454c, 0x4550, PDF_CMAP_RANGE, 3142 },
	{ 0x4551, 0x4552, PDF_CMAP_RANGE, 7755 },
	{ 0x4553, 0x4556, PDF_CMAP_RANGE, 3149 },
	{ 0x4557, 0x4557, PDF_CMAP_SINGLE, 5855 },
	{ 0x4558, 0x4563, PDF_CMAP_RANGE, 3154 },
	{ 0x4564, 0x4564, PDF_CMAP_SINGLE, 7757 },
	{ 0x4565, 0x456d, PDF_CMAP_RANGE, 3167 },
	{ 0x456e, 0x456e, PDF_CMAP_SINGLE, 5200 },
	{ 0x456f, 0x4572, PDF_CMAP_RANGE, 3177 },
	{ 0x4573, 0x4573, PDF_CMAP_SINGLE, 5430 },
	{ 0x4574, 0x4577, PDF_CMAP_RANGE, 3182 },
	{ 0x4578, 0x4578, PDF_CMAP_SINGLE, 7758 },
	{ 0x4579, 0x457e, PDF_CMAP_RANGE, 3187 },
	{ 0x4621, 0x463d, PDF_CMAP_RANGE, 3193 },
	{ 0x463e, 0x463e, PDF_CMAP_SINGLE, 7759 },
	{ 0x463f, 0x4641, PDF_CMAP_RANGE, 3223 },
	{ 0x4642, 0x4642, PDF_CMAP_SINGLE, 7760 },
	{ 0x4643, 0x4653, PDF_CMAP_RANGE, 3227 },
	{ 0x4654, 0x4655, PDF_CMAP_RANGE, 7761 },
	{ 0x4656, 0x465a, PDF_CMAP_RANGE, 3246 },
	{ 0x465b, 0x465c, PDF_CMAP_RANGE, 7763 },
	{ 0x465d, 0x4660, PDF_CMAP_RANGE, 3253 },
	{ 0x4661, 0x4661, PDF_CMAP_SINGLE, 7765 },
	{ 0x4662, 0x4665, PDF_CMAP_RANGE, 3258 },
	{ 0x4666, 0x4667, PDF_CMAP_RANGE, 7766 },
	{ 0x4668, 0x4669, PDF_CMAP_RANGE, 3264 },
	{ 0x466a, 0x466a, PDF_CMAP_SINGLE, 7768 },
	{ 0x466b, 0x4675, PDF_CMAP_RANGE, 3267 },
	{ 0x4676, 0x4676, PDF_CMAP_SINGLE, 6893 },
	{ 0x4677, 0x467e, PDF_CMAP_RANGE, 3279 },
	{ 0x4721, 0x4728, PDF_CMAP_RANGE, 3287 },
	{ 0x4729, 0x4729, PDF_CMAP_SINGLE, 7769 },
	{ 0x472a, 0x4738, PDF_CMAP_RANGE, 3296 },
	{ 0x4739, 0x4739, PDF_CMAP_SINGLE, 7770 },
	{ 0x473a, 0x4756, PDF_CMAP_RANGE, 3312 },
	{ 0x4757, 0x4757, PDF_CMAP_SINGLE, 7771 },
	{ 0x4758, 0x4766, PDF_CMAP_RANGE, 3342 },
	{ 0x4767, 0x4769, PDF_CMAP_TABLE, 124 },
	{ 0x476a, 0x476c, PDF_CMAP_RANGE, 3360 },
	{ 0x476d, 0x476d, PDF_CMAP_SINGLE, 7774 },
	{ 0x476e, 0x477e, PDF_CMAP_RANGE, 3364 },
	{ 0x4821, 0x4823, PDF_CMAP_RANGE, 3381 },
	{ 0x4824, 0x4824, PDF_CMAP_SINGLE, 7775 },
	{ 0x4825, 0x482d, PDF_CMAP_RANGE, 3385 },
	{ 0x482e, 0x4830, PDF_CMAP_TABLE, 127 },
	{ 0x4831, 0x4853, PDF_CMAP_RANGE, 3397 },
	{ 0x4854, 0x4854, PDF_CMAP_SINGLE, 7778 },
	{ 0x4855, 0x4861, PDF_CMAP_RANGE, 3433 },
	{ 0x4862, 0x4862, PDF_CMAP_SINGLE, 7779 },
	{ 0x4863, 0x4874, PDF_CMAP_RANGE, 3447 },
	{ 0x4875, 0x4875, PDF_CMAP_SINGLE, 7780 },
	{ 0x4876, 0x487e, PDF_CMAP_RANGE, 3466 },
	{ 0x4921, 0x4921, PDF_CMAP_SINGLE, 3475 },
	{ 0x4922, 0x4923, PDF_CMAP_RANGE, 7781 },
	{ 0x4924, 0x492e, PDF_CMAP_RANGE, 3478 },
	{ 0x492f, 0x4932, PDF_CMAP_TABLE, 130 },
	{ 0x4933, 0x4934, PDF_CMAP_RANGE, 3493 },
	{ 0x4935, 0x4935, PDF_CMAP_SINGLE, 7785 },
	{ 0x4936, 0x493f, PDF_CMAP_RANGE, 3496 },
	{ 0x4940, 0x4940, PDF_CMAP_SINGLE, 7786 },
	{ 0x4941, 0x494d, PDF_CMAP_RANGE, 3507 },
	{ 0x494e, 0x494e, PDF_CMAP_SINGLE, 7787 },
	{ 0x494f, 0x4950, PDF_CMAP_RANGE, 3521 },
	{ 0x4951, 0x4951, PDF_CMAP_SINGLE, 7788 },
	{ 0x4952, 0x497e, PDF_CMAP_RANGE, 3524 },
	{ 0x4a21, 0x4a42, PDF_CMAP_RANGE, 3569 },
	{ 0x4a43, 0x4a43, PDF_CMAP_SINGLE, 7789 },
	{ 0x4a44, 0x4a4c, PDF_CMAP_RANGE, 3604 },
	{ 0x4a4d, 0x4a4d, PDF_CMAP_SINGLE, 7790 },
	{ 0x4a4e, 0x4a59, PDF_CMAP_RANGE, 3614 },
	{ 0x4a5a, 0x4a5a, PDF_CMAP_SINGLE, 7791 },
	{ 0x4a5b, 0x4a78, PDF_CMAP_RANGE, 3627 },
	{ 0x4a79, 0x4a79, PDF_CMAP_SINGLE, 7792 },
	{ 0x4a7a, 0x4a7e, PDF_CMAP_RANGE, 3658 },
	{ 0x4b21, 0x4b22, PDF_CMAP_TABLE, 134 },
	{ 0x4b23, 0x4b28, PDF_CMAP_RANGE, 3665 },
	{ 0x4b29, 0x4b29, PDF_CMAP_SINGLE, 7794 },
	{ 0x4b2a, 0x4b4a, PDF_CMAP_RANGE, 3672 },
	{ 0x4b4b, 0x4b4b, PDF_CMAP_SINGLE, 7795 },
	{ 0x4b4c, 0x4b69, PDF_CMAP_RANGE, 3706 },
	{ 0x4b6a, 0x4b6a, PDF_CMAP_SINGLE, 7475 },
	{ 0x4b6b, 0x4b6f, PDF_CMAP_RANGE, 3737 },
	{ 0x4b70, 0x4b70, PDF_CMAP_SINGLE, 7796 },
	{ 0x4b71, 0x4b78, PDF_CMAP_RANGE, 3743 },
	{ 0x4b79, 0x4b79, PDF_CMAP_SINGLE, 4143 },
	{ 0x4b7a, 0x4b7e, PDF_CMAP_RANGE, 3752 },
	{ 0x4c21, 0x4c4c, PDF_CMAP_RANGE, 3757 },
	{ 0x4c4d, 0x4c4d, PDF_CMAP_SINGLE, 7797 },
	{ 0x4c4e, 0x4c58, PDF_CMAP_RANGE, 3802 },
	{ 0x4c59, 0x4c59, PDF_CMAP_SINGLE, 7798 },
	{ 0x4c5a, 0x4c5e, PDF_CMAP_RANGE, 3814 },
	{ 0x4c5f, 0x4c5f, PDF_CMAP_SINGLE, 7799 },
	{ 0x4c60, 0x4c61, PDF_CMAP_RANGE, 3820 },
	{ 0x4c62, 0x4c62, PDF_CMAP_SINGLE, 7800 },
	{ 0x4c63, 0x4c78, PDF_CMAP_RANGE, 3823 },
	{ 0x4c79, 0x4c7e, PDF_CMAP_TABLE, 136 },
	{ 0x4d21, 0x4d31, PDF_CMAP_RANGE, 3851 },
	{ 0x4d32, 0x4d32, PDF_CMAP_SINGLE, 7804 },
	{ 0x4d33, 0x4d4f, PDF_CMAP_RANGE, 3869 },
	{ 0x4d50, 0x4d50, PDF_CMAP_SINGLE, 7805 },
	{ 0x4d51, 0x4d53, PDF_CMAP_RANGE, 3899 },
	{ 0x4d54, 0x4d54, PDF_CMAP_SINGLE, 7806 },
	{ 0x4d55, 0x4d59, PDF_CMAP_RANGE, 3903 },
	{ 0x4d5a, 0x4d5a, PDF_CMAP_SINGLE, 7476 },
	{ 0x4d5b, 0x4d68, PDF_CMAP_RANGE, 3909 },
	{ 0x4d69, 0x4d69, PDF_CMAP_SINGLE, 7807 },
	{ 0x4d6a, 0x4d7e, PDF_CMAP_RANGE, 3924 },
	{ 0x4e21, 0x4e4a, PDF_CMAP_RANGE, 3945 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 7808 },
	{ 0x4e4c, 0x4e79, PDF_CMAP_RANGE, 3988 },
	{ 0x4e7a, 0x4e7b, PDF_CMAP_RANGE, 7809 },
	{ 0x4e7c, 0x4e7e, PDF_CMAP_RANGE, 4036 },
	{ 0x4f21, 0x4f21, PDF_CMAP_SINGLE, 7811 },
	{ 0x4f22, 0x4f30, PDF_CMAP_RANGE, 4040 },
	{ 0x4f31, 0x4f31, PDF_CMAP_SINGLE, 7812 },
	{ 0x4f32, 0x4f35, PDF_CMAP_RANGE, 4056 },
	{ 0x4f36, 0x4f36, PDF_CMAP_SINGLE, 6007 },
	{ 0x4f37, 0x4f38, PDF_CMAP_RANGE, 4061 },
	{ 0x4f39, 0x4f39, PDF_CMAP_SINGLE, 7813 },
	{ 0x4f3a, 0x4f53, PDF_CMAP_RANGE, 4064 },
	{ 0x5021, 0x5055, PDF_CMAP_RANGE, 4090 },
	{ 0x5056, 0x5056, PDF_CMAP_SINGLE, 3751 },
	{ 0x5057, 0x507e, PDF_CMAP_RANGE, 4144 },
	{ 0x5121, 0x513c, PDF_CMAP_RANGE, 4184 },
	{ 0x513d, 0x513d, PDF_CMAP_SINGLE, 7814 },
	{ 0x513e, 0x5146, PDF_CMAP_RANGE, 4213 },
	{ 0x5147, 0x5147, PDF_CMAP_SINGLE, 7815 },
	{ 0x5148, 0x514a, PDF_CMAP_RANGE, 4223 },
	{ 0x514b, 0x514d, PDF_CMAP_TABLE, 142 },
	{ 0x514e, 0x517e, PDF_CMAP_RANGE, 4229 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 4278 },
	{ 0x5321, 0x532f, PDF_CMAP_RANGE, 4372 },
	{ 0x5330, 0x5330, PDF_CMAP_SINGLE, 7818 },
	{ 0x5331, 0x5339, PDF_CMAP_RANGE, 4388 },
	{ 0x533a, 0x533a, PDF_CMAP_SINGLE, 7819 },
	{ 0x533b, 0x534f, PDF_CMAP_RANGE, 4398 },
	{ 0x5350, 0x5350, PDF_CMAP_SINGLE, 7820 },
	{ 0x5351, 0x535d, PDF_CMAP_RANGE, 4420 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 7821 },
	{ 0x535f, 0x536a, PDF_CMAP_RANGE, 4434 },
	{ 0x536b, 0x536b, PDF_CMAP_SINGLE, 7822 },
	{ 0x536c, 0x537e, PDF_CMAP_RANGE, 4447 },
	{ 0x5421, 0x5443, PDF_CMAP_RANGE, 4466 },
	{ 0x5444, 0x5444, PDF_CMAP_SINGLE, 7823 },
	{ 0x5445, 0x5463, PDF_CMAP_RANGE, 4502 },
	{ 0x5464, 0x5464, PDF_CMAP_SINGLE, 3063 },
	{ 0x5465, 0x547e, PDF_CMAP_RANGE, 4534 },
	{ 0x5521, 0x553c, PDF_CMAP_RANGE, 4560 },
	{ 0x553d, 0x553d, PDF_CMAP_SINGLE, 7824 },
	{ 0x553e, 0x5562, PDF_CMAP_RANGE, 4589 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 7825 },
	{ 0x5564, 0x557e, PDF_CMAP_RANGE, 4627 },
	{ 0x5621, 0x5622, PDF_CMAP_TABLE, 145 },
	{ 0x5623, 0x5671, PDF_CMAP_RANGE, 4656 },
	{ 0x5672, 0x5672, PDF_CMAP_SINGLE, 7827 },
	{ 0x5673, 0x567e, PDF_CMAP_RANGE, 4736 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 4748 },
	{ 0x5821, 0x5823, PDF_CMAP_RANGE, 4842 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 7828 },
	{ 0x5825, 0x587e, PDF_CMAP_RANGE, 4846 },
	{ 0x5921, 0x595f, PDF_CMAP_RANGE, 4936 },
	{ 0x5960, 0x5960, PDF_CMAP_SINGLE, 7829 },
	{ 0x5961, 0x596b, PDF_CMAP_RANGE, 5000 },
	{ 0x596c, 0x596c, PDF_CMAP_SINGLE, 7830 },
	{ 0x596d, 0x5977, PDF_CMAP_RANGE, 5012 },
	{ 0x5978, 0x5978, PDF_CMAP_SINGLE, 1447 },
	{ 0x5979, 0x597e, PDF_CMAP_RANGE, 5024 },
	{ 0x5a21, 0x5a38, PDF_CMAP_RANGE, 5030 },
	{ 0x5a39, 0x5a39, PDF_CMAP_SINGLE, 7831 },
	{ 0x5a3a, 0x5a4c, PDF_CMAP_RANGE, 5055 },
	{ 0x5a4d, 0x5a4d, PDF_CMAP_SINGLE, 7832 },
	{ 0x5a4e, 0x5a7e, PDF_CMAP_RANGE, 5075 },
	{ 0x5b21, 0x5b44, PDF_CMAP_RANGE, 5124 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 7833 },
	{ 0x5b46, 0x5b49, PDF_CMAP_RANGE, 5161 },
	{ 0x5b4a, 0x5b4a, PDF_CMAP_SINGLE, 7834 },
	{ 0x5b4b, 0x5b57, PDF_CMAP_RANGE, 5166 },
	{ 0x5b58, 0x5b58, PDF_CMAP_SINGLE, 3490 },
	{ 0x5b59, 0x5b6a, PDF_CMAP_RANGE, 5180 },
	{ 0x5b6b, 0x5b6d, PDF_CMAP_TABLE, 147 },
	{ 0x5b6e, 0x5b73, PDF_CMAP_RANGE, 5201 },
	{ 0x5b74, 0x5b74, PDF_CMAP_SINGLE, 7836 },
	{ 0x5b75, 0x5b7e, PDF_CMAP_RANGE, 5208 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 5218 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 5312 },
	{ 0x5e21, 0x5e38, PDF_CMAP_RANGE, 5406 },
	{ 0x5e39, 0x5e39, PDF_CMAP_SINGLE, 3181 },
	{ 0x5e3a, 0x5e4f, PDF_CMAP_RANGE, 5431 },
	{ 0x5e50, 0x5e50, PDF_CMAP_SINGLE, 7837 },
	{ 0x5e51, 0x5e74, PDF_CMAP_RANGE, 5454 },
	{ 0x5e75, 0x5e75, PDF_CMAP_SINGLE, 1535 },
	{ 0x5e76, 0x5e7e, PDF_CMAP_RANGE, 5491 },
	{ 0x5f21, 0x5f72, PDF_CMAP_RANGE, 5500 },
	{ 0x5f73, 0x5f73, PDF_CMAP_SINGLE, 7838 },
	{ 0x5f74, 0x5f7e, PDF_CMAP_RANGE, 5583 },
	{ 0x6021, 0x6025, PDF_CMAP_RANGE, 5594 },
	{ 0x6026, 0x6026, PDF_CMAP_SINGLE, 7839 },
	{ 0x6027, 0x605e, PDF_CMAP_RANGE, 5600 },
	{ 0x605f, 0x605f, PDF_CMAP_SINGLE, 7840 },
	{ 0x6060, 0x6075, PDF_CMAP_RANGE, 5657 },
	{ 0x6076, 0x6076, PDF_CMAP_SINGLE, 7477 },
	{ 0x6077, 0x607e, PDF_CMAP_RANGE, 5680 },
	{ 0x6121, 0x612a, PDF_CMAP_RANGE, 5688 },
	{ 0x612b, 0x612b, PDF_CMAP_SINGLE, 7841 },
	{ 0x612c, 0x612f, PDF_CMAP_RANGE, 5699 },
	{ 0x6130, 0x6131, PDF_CMAP_RANGE, 7842 },
	{ 0x6132, 0x617e, PDF_CMAP_RANGE, 5705 },
	{ 0x6221, 0x622a, PDF_CMAP_RANGE, 5782 },
	{ 0x622b, 0x622b, PDF_CMAP_SINGLE, 7844 },
	{ 0x622c, 0x6267, PDF_CMAP_RANGE, 5793 },
	{ 0x6268, 0x626a, PDF_CMAP_TABLE, 150 },
	{ 0x626b, 0x626e, PDF_CMAP_RANGE, 5856 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 7845 },
	{ 0x6270, 0x627e, PDF_CMAP_RANGE, 5861 },
	{ 0x6321, 0x6349, PDF_CMAP_RANGE, 5876 },
	{ 0x634a, 0x634a, PDF_CMAP_SINGLE, 7846 },
	{ 0x634b, 0x6353, PDF_CMAP_RANGE, 5918 },
	{ 0x6354, 0x6354, PDF_CMAP_SINGLE, 7847 },
	{ 0x6355, 0x635d, PDF_CMAP_RANGE, 5928 },
	{ 0x635e, 0x635e, PDF_CMAP_SINGLE, 1492 },
	{ 0x635f, 0x637e, PDF_CMAP_RANGE, 5938 },
	{ 0x6421, 0x6438, PDF_CMAP_RANGE, 5970 },
	{ 0x6439, 0x6439, PDF_CMAP_SINGLE, 7848 },
	{ 0x643a, 0x6445, PDF_CMAP_RANGE, 5995 },
	{ 0x6446, 0x6446, PDF_CMAP_SINGLE, 4060 },
	{ 0x6447, 0x6463, PDF_CMAP_RANGE, 6008 },
	{ 0x6464, 0x6464, PDF_CMAP_SINGLE, 7849 },
	{ 0x6465, 0x646d, PDF_CMAP_RANGE, 6038 },
	{ 0x646e, 0x646e, PDF_CMAP_SINGLE, 7850 },
	{ 0x646f, 0x647e, PDF_CMAP_RANGE, 6048 },
	{ 0x6521, 0x6538, PDF_CMAP_RANGE, 6064 },
	{ 0x6539, 0x653b, PDF_CMAP_TABLE, 153 },
	{ 0x653c, 0x6545, PDF_CMAP_RANGE, 6091 },
	{ 0x6546, 0x6546, PDF_CMAP_SINGLE, 7853 },
	{ 0x6547, 0x657e, PDF_CMAP_RANGE, 6102 },
	{ 0x6621, 0x6645, PDF_CMAP_RANGE, 6158 },
	{ 0x6646, 0x6646, PDF_CMAP_SINGLE, 7854 },
	{ 0x6647, 0x667e, PDF_CMAP_RANGE, 6196 },
	{ 0x6721, 0x6763, PDF_CMAP_RANGE, 6252 },
	{ 0x6764, 0x6764, PDF_CMAP_SINGLE, 7855 },
	{ 0x6765, 0x6768, PDF_CMAP_RANGE, 6320 },
	{ 0x6769, 0x6769, PDF_CMAP_SINGLE, 7856 },
	{ 0x676a, 0x6771, PDF_CMAP_RANGE, 6325 },
	{ 0x6772, 0x6772, PDF_CMAP_SINGLE, 7857 },
	{ 0x6773, 0x677e, PDF_CMAP_RANGE, 6334 },
	{ 0x6821, 0x6833, PDF_CMAP_RANGE, 6346 },
	{ 0x6834, 0x6834, PDF_CMAP_SINGLE, 7858 },
	{ 0x6835, 0x683a, PDF_CMAP_RANGE, 6366 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 7859 },
	{ 0x683c, 0x6873, PDF_CMAP_RANGE, 6373 },
	{ 0x6874, 0x6874, PDF_CMAP_SINGLE, 7860 },
	{ 0x6875, 0x687e, PDF_CMAP_RANGE, 6430 },
	{ 0x6921, 0x6922, PDF_CMAP_TABLE, 156 },
	{ 0x6923, 0x692d, PDF_CMAP_RANGE, 6442 },
	{ 0x692e, 0x692e, PDF_CMAP_SINGLE, 3845 },
	{ 0x692f, 0x693b, PDF_CMAP_RANGE, 6454 },
	{ 0x693c, 0x693c, PDF_CMAP_SINGLE, 7861 },
	{ 0x693d, 0x6959, PDF_CMAP_RANGE, 6468 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 1440 },
	{ 0x695b, 0x697e, PDF_CMAP_RANGE, 6498 },
	{ 0x6a21, 0x6a23, PDF_CMAP_RANGE, 6534 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 3358 },
	{ 0x6a25, 0x6a26, PDF_CMAP_RANGE, 6538 },
	{ 0x6a27, 0x6a27, PDF_CMAP_SINGLE, 7862 },
	{ 0x6a28, 0x6a3c, PDF_CMAP_RANGE, 6541 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 7863 },
	{ 0x6a3e, 0x6a6e, PDF_CMAP_RANGE, 6563 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 7864 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6613 },
	{ 0x6b21, 0x6b31, PDF_CMAP_RANGE, 6628 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 7865 },
	{ 0x6b33, 0x6b5c, PDF_CMAP_RANGE, 6646 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 1550 },
	{ 0x6b5e, 0x6b65, PDF_CMAP_RANGE, 6689 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 7866 },
	{ 0x6b67, 0x6b75, PDF_CMAP_RANGE, 6698 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 7867 },
	{ 0x6b77, 0x6b7e, PDF_CMAP_RANGE, 6714 },
	{ 0x6c21, 0x6c4c, PDF_CMAP_RANGE, 6722 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2730 },
	{ 0x6c4e, 0x6c68, PDF_CMAP_RANGE, 6767 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 7868 },
	{ 0x6c6a, 0x6c73, PDF_CMAP_RANGE, 6795 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 7869 },
	{ 0x6c75, 0x6c7e, PDF_CMAP_RANGE, 6806 },
	{ 0x6d21, 0x6d4d, PDF_CMAP_RANGE, 6816 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 7870 },
	{ 0x6d4f, 0x6d6b, PDF_CMAP_RANGE, 6862 },
	{ 0x6d6c, 0x6d6e, PDF_CMAP_TABLE, 158 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6894 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e56, PDF_CMAP_RANGE, 6939 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 7875 },
	{ 0x6e58, 0x6e7e, PDF_CMAP_RANGE, 6965 },
	{ 0x6f21, 0x6f64, PDF_CMAP_RANGE, 7004 },
	{ 0x6f65, 0x6f65, PDF_CMAP_SINGLE, 7876 },
	{ 0x6f66, 0x6f7e, PDF_CMAP_RANGE, 7073 },
	{ 0x7021, 0x7032, PDF_CMAP_RANGE, 7098 },
	{ 0x7033, 0x7033, PDF_CMAP_SINGLE, 7877 },
	{ 0x7034, 0x7044, PDF_CMAP_RANGE, 7117 },
	{ 0x7045, 0x7045, PDF_CMAP_SINGLE, 7878 },
	{ 0x7046, 0x7050, PDF_CMAP_RANGE, 7135 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x7056, PDF_CMAP_RANGE, 7147 },
	{ 0x7057, 0x7057, PDF_CMAP_SINGLE, 7880 },
	{ 0x7058, 0x7073, PDF_CMAP_RANGE, 7153 },
	{ 0x7074, 0x7075, PDF_CMAP_TABLE, 161 },
	{ 0x7076, 0x707e, PDF_CMAP_RANGE, 7183 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724c, PDF_CMAP_RANGE, 7314 },
	{ 0x724d, 0x724e, PDF_CMAP_TABLE, 163 },
	{ 0x724f, 0x7273, PDF_CMAP_RANGE, 7332 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1321 },
	{ 0x7275, 0x727e, PDF_CMAP_RANGE, 7370 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x737c, PDF_CMAP_RANGE, 7429 },
	{ 0x737d, 0x737e, PDF_CMAP_TABLE, 165 },
	{ 0x6269, 0x6269, PDF_CMAP_SINGLE, 5854 },
	{ 0x626a, 0x626a, PDF_CMAP_SINGLE, 3153 },
	{ 0x626b, 0x626e, PDF_CMAP_RANGE, 5856 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 7845 },
	{ 0x6270, 0x627e, PDF_CMAP_RANGE, 5861 },
	{ 0x6321, 0x6349, PDF_CMAP_RANGE, 5876 },
	{ 0x634a, 0x634a, PDF_CMAP_SINGLE, 7846 },
	{ 0x634b, 0x6353, PDF_CMAP_RANGE, 5918 },
	{ 0x6354, 0x6354, PDF_CMAP_SINGLE, 7847 },
	{ 0x6355, 0x635d, PDF_CMAP_RANGE, 5928 },
	{ 0x635e, 0x635e, PDF_CMAP_SINGLE, 1492 },
	{ 0x635f, 0x637e, PDF_CMAP_RANGE, 5938 },
	{ 0x6421, 0x6438, PDF_CMAP_RANGE, 5970 },
	{ 0x6439, 0x6439, PDF_CMAP_SINGLE, 7848 },
	{ 0x643a, 0x6445, PDF_CMAP_RANGE, 5995 },
	{ 0x6446, 0x6446, PDF_CMAP_SINGLE, 4060 },
	{ 0x6447, 0x6463, PDF_CMAP_RANGE, 6008 },
	{ 0x6464, 0x6464, PDF_CMAP_SINGLE, 7849 },
	{ 0x6465, 0x646d, PDF_CMAP_RANGE, 6038 },
	{ 0x646e, 0x646e, PDF_CMAP_SINGLE, 7850 },
	{ 0x646f, 0x647e, PDF_CMAP_RANGE, 6048 },
	{ 0x6521, 0x6538, PDF_CMAP_RANGE, 6064 },
	{ 0x6539, 0x6539, PDF_CMAP_SINGLE, 7851 },
	{ 0x653a, 0x653a, PDF_CMAP_SINGLE, 6089 },
	{ 0x653b, 0x653b, PDF_CMAP_SINGLE, 7852 },
	{ 0x653c, 0x6545, PDF_CMAP_RANGE, 6091 },
	{ 0x6546, 0x6546, PDF_CMAP_SINGLE, 7853 },
	{ 0x6547, 0x657e, PDF_CMAP_RANGE, 6102 },
	{ 0x6621, 0x6645, PDF_CMAP_RANGE, 6158 },
	{ 0x6646, 0x6646, PDF_CMAP_SINGLE, 7854 },
	{ 0x6647, 0x667e, PDF_CMAP_RANGE, 6196 },
	{ 0x6721, 0x6763, PDF_CMAP_RANGE, 6252 },
	{ 0x6764, 0x6764, PDF_CMAP_SINGLE, 7855 },
	{ 0x6765, 0x6768, PDF_CMAP_RANGE, 6320 },
	{ 0x6769, 0x6769, PDF_CMAP_SINGLE, 7856 },
	{ 0x676a, 0x6771, PDF_CMAP_RANGE, 6325 },
	{ 0x6772, 0x6772, PDF_CMAP_SINGLE, 7857 },
	{ 0x6773, 0x677e, PDF_CMAP_RANGE, 6334 },
	{ 0x6821, 0x6833, PDF_CMAP_RANGE, 6346 },
	{ 0x6834, 0x6834, PDF_CMAP_SINGLE, 7858 },
	{ 0x6835, 0x683a, PDF_CMAP_RANGE, 6366 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 7859 },
	{ 0x683c, 0x6873, PDF_CMAP_RANGE, 6373 },
	{ 0x6874, 0x6874, PDF_CMAP_SINGLE, 7860 },
	{ 0x6875, 0x687e, PDF_CMAP_RANGE, 6430 },
	{ 0x6921, 0x6921, PDF_CMAP_SINGLE, 6440 },
	{ 0x6922, 0x6922, PDF_CMAP_SINGLE, 2293 },
	{ 0x6923, 0x692d, PDF_CMAP_RANGE, 6442 },
	{ 0x692e, 0x692e, PDF_CMAP_SINGLE, 3845 },
	{ 0x692f, 0x693b, PDF_CMAP_RANGE, 6454 },
	{ 0x693c, 0x693c, PDF_CMAP_SINGLE, 7861 },
	{ 0x693d, 0x6959, PDF_CMAP_RANGE, 6468 },
	{ 0x695a, 0x695a, PDF_CMAP_SINGLE, 1440 },
	{ 0x695b, 0x697e, PDF_CMAP_RANGE, 6498 },
	{ 0x6a21, 0x6a23, PDF_CMAP_RANGE, 6534 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 3358 },
	{ 0x6a25, 0x6a26, PDF_CMAP_RANGE, 6538 },
	{ 0x6a27, 0x6a27, PDF_CMAP_SINGLE, 7862 },
	{ 0x6a28, 0x6a3c, PDF_CMAP_RANGE, 6541 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 7863 },
	{ 0x6a3e, 0x6a6e, PDF_CMAP_RANGE, 6563 },
	{ 0x6a6f, 0x6a6f, PDF_CMAP_SINGLE, 7864 },
	{ 0x6a70, 0x6a7e, PDF_CMAP_RANGE, 6613 },
	{ 0x6b21, 0x6b31, PDF_CMAP_RANGE, 6628 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 7865 },
	{ 0x6b33, 0x6b5c, PDF_CMAP_RANGE, 6646 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 1550 },
	{ 0x6b5e, 0x6b65, PDF_CMAP_RANGE, 6689 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 7866 },
	{ 0x6b67, 0x6b75, PDF_CMAP_RANGE, 6698 },
	{ 0x6b76, 0x6b76, PDF_CMAP_SINGLE, 7867 },
	{ 0x6b77, 0x6b7e, PDF_CMAP_RANGE, 6714 },
	{ 0x6c21, 0x6c4c, PDF_CMAP_RANGE, 6722 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2730 },
	{ 0x6c4e, 0x6c68, PDF_CMAP_RANGE, 6767 },
	{ 0x6c69, 0x6c69, PDF_CMAP_SINGLE, 7868 },
	{ 0x6c6a, 0x6c73, PDF_CMAP_RANGE, 6795 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 7869 },
	{ 0x6c75, 0x6c7e, PDF_CMAP_RANGE, 6806 },
	{ 0x6d21, 0x6d4d, PDF_CMAP_RANGE, 6816 },
	{ 0x6d4e, 0x6d4e, PDF_CMAP_SINGLE, 7870 },
	{ 0x6d4f, 0x6d6b, PDF_CMAP_RANGE, 6862 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 7871 },
	{ 0x6d6d, 0x6d6d, PDF_CMAP_SINGLE, 6892 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 7872 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6894 },
	{ 0x6e21, 0x6e28, PDF_CMAP_RANGE, 6910 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 7873 },
	{ 0x6e2a, 0x6e3c, PDF_CMAP_RANGE, 6919 },
	{ 0x6e3d, 0x6e3d, PDF_CMAP_SINGLE, 7874 },
	{ 0x6e3e, 0x6e56, PDF_CMAP_RANGE, 6939 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 7875 },
	{ 0x6e58, 0x6e7e, PDF_CMAP_RANGE, 6965 },
	{ 0x6f21, 0x6f64, PDF_CMAP_RANGE, 7004 },
	{ 0x6f65, 0x6f65, PDF_CMAP_SINGLE, 7876 },
	{ 0x6f66, 0x6f7e, PDF_CMAP_RANGE, 7073 },
	{ 0x7021, 0x7032, PDF_CMAP_RANGE, 7098 },
	{ 0x7033, 0x7033, PDF_CMAP_SINGLE, 7877 },
	{ 0x7034, 0x7044, PDF_CMAP_RANGE, 7117 },
	{ 0x7045, 0x7045, PDF_CMAP_SINGLE, 7878 },
	{ 0x7046, 0x7050, PDF_CMAP_RANGE, 7135 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 7879 },
	{ 0x7052, 0x7056, PDF_CMAP_RANGE, 7147 },
	{ 0x7057, 0x7057, PDF_CMAP_SINGLE, 7880 },
	{ 0x7058, 0x7073, PDF_CMAP_RANGE, 7153 },
	{ 0x7074, 0x7074, PDF_CMAP_SINGLE, 1841 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 7881 },
	{ 0x7076, 0x707e, PDF_CMAP_RANGE, 7183 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 7192 },
	{ 0x7221, 0x722c, PDF_CMAP_RANGE, 7286 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 7882 },
	{ 0x722e, 0x723b, PDF_CMAP_RANGE, 7299 },
	{ 0x723c, 0x723c, PDF_CMAP_SINGLE, 7883 },
	{ 0x723d, 0x724c, PDF_CMAP_RANGE, 7314 },
	{ 0x724d, 0x724d, PDF_CMAP_SINGLE, 1143 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 7884 },
	{ 0x724f, 0x7273, PDF_CMAP_RANGE, 7332 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 1321 },
	{ 0x7275, 0x727e, PDF_CMAP_RANGE, 7370 },
	{ 0x7321, 0x7350, PDF_CMAP_RANGE, 7380 },
	{ 0x7351, 0x7351, PDF_CMAP_SINGLE, 7885 },
	{ 0x7352, 0x737c, PDF_CMAP_RANGE, 7429 },
	{ 0x737d, 0x737d, PDF_CMAP_SINGLE, 7886 },
	{ 0x737e, 0x737e, PDF_CMAP_SINGLE, 7473 },
};

static const unsigned short pdf_cmap_NWP_H_table[167] =
{
	762,761,769,642,673,773,712,715,
	717,671,718,692,636,693,637,663,
	699,697,700,641,719,678,711,679,
	648,650,646,680,667,681,649,634,
	638,1006,925,927,929,931,933,991,
	993,995,959,660,926,928,930,932,
	937,939,941,943,945,947,949,951,
	953,955,957,960,962,964,974,977,
	980,983,992,994,1003,1007,1008,936,
	938,940,942,944,946,948,950,952,
	954,956,958,961,963,965,1125,7633,
	7645,7369,7646,7649,1406,1407,7650,7655,
	5490,1689,7660,7665,1780,7666,1782,7689,
	2169,7690,7693,6441,2629,7713,7720,6766,
	7733,2910,7734,2918,7744,3045,7745,7746,
	3050,7747,3099,7748,7772,6537,7773,7776,
	3395,7777,7783,5179,3491,7784,3663,7793,
	6453,7801,3847,7802,3849,7803,7816,4227,
	7817,4654,7826,7835,5199,3176,2030,5854,
	3153,7851,6089,7852,6440,2293,7871,6892,
	7872,1841,7881,1143,7884,7886,7473,
};

pdf_cmap pdf_cmap_NWP_H =
{
	-1, "NWP-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	759, 759, (pdf_range*) pdf_cmap_NWP_H_ranges,
	167, 167, (unsigned short*) pdf_cmap_NWP_H_table,
};

/* NWP-V */

static const pdf_range pdf_cmap_NWP_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_RANGE, 7887 },
	{ 0x2124, 0x2125, PDF_CMAP_TABLE, 0 },
	{ 0x212b, 0x212c, PDF_CMAP_TABLE, 2 },
	{ 0x213c, 0x213e, PDF_CMAP_RANGE, 7891 },
	{ 0x2141, 0x2145, PDF_CMAP_RANGE, 7894 },
	{ 0x2146, 0x2149, PDF_CMAP_TABLE, 4 },
	{ 0x214a, 0x215b, PDF_CMAP_RANGE, 7899 },
	{ 0x216b, 0x216d, PDF_CMAP_TABLE, 8 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 8270 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 7918 },
	{ 0x2423, 0x2423, PDF_CMAP_SINGLE, 7919 },
	{ 0x2425, 0x2425, PDF_CMAP_SINGLE, 7920 },
	{ 0x2427, 0x2427, PDF_CMAP_SINGLE, 7921 },
	{ 0x2429, 0x2429, PDF_CMAP_SINGLE, 7922 },
	{ 0x2443, 0x2443, PDF_CMAP_SINGLE, 7923 },
	{ 0x2463, 0x2463, PDF_CMAP_SINGLE, 7924 },
	{ 0x2465, 0x2465, PDF_CMAP_SINGLE, 7925 },
	{ 0x2467, 0x2467, PDF_CMAP_SINGLE, 7926 },
	{ 0x246e, 0x246e, PDF_CMAP_SINGLE, 7927 },
	{ 0x2521, 0x2521, PDF_CMAP_SINGLE, 7928 },
	{ 0x2523, 0x2523, PDF_CMAP_SINGLE, 7929 },
	{ 0x2525, 0x2525, PDF_CMAP_SINGLE, 7930 },
	{ 0x2527, 0x2527, PDF_CMAP_SINGLE, 7931 },
	{ 0x2529, 0x2529, PDF_CMAP_SINGLE, 7932 },
	{ 0x2543, 0x2543, PDF_CMAP_SINGLE, 7933 },
	{ 0x2563, 0x2563, PDF_CMAP_SINGLE, 7934 },
	{ 0x2565, 0x2565, PDF_CMAP_SINGLE, 7935 },
	{ 0x2567, 0x2567, PDF_CMAP_SINGLE, 7936 },
	{ 0x256e, 0x256e, PDF_CMAP_SINGLE, 7937 },
	{ 0x2575, 0x2576, PDF_CMAP_RANGE, 7938 },
	{ 0x2d40, 0x2d4f, PDF_CMAP_RANGE, 7940 },
	{ 0x2d60, 0x2d61, PDF_CMAP_RANGE, 7956 },
	{ 0x2f21, 0x2f21, PDF_CMAP_SINGLE, 7888 },
	{ 0x2f22, 0x2f23, PDF_CMAP_RANGE, 7911 },
	{ 0x2f24, 0x2f24, PDF_CMAP_SINGLE, 7887 },
	{ 0x2f27, 0x2f2b, PDF_CMAP_RANGE, 7928 },
	{ 0x2f2c, 0x2f2e, PDF_CMAP_RANGE, 7934 },
	{ 0x2f2f, 0x2f30, PDF_CMAP_TABLE, 11 },
	{ 0x2f5e, 0x2f5f, PDF_CMAP_TABLE, 13 },
	{ 0x2f21, 0x2f21, PDF_CMAP_SINGLE, 7888 },
	{ 0x2f22, 0x2f23, PDF_CMAP_RANGE, 7911 },
	{ 0x2f24, 0x2f24, PDF_CMAP_SINGLE, 7887 },
	{ 0x2f27, 0x2f2b, PDF_CMAP_RANGE, 7928 },
	{ 0x2f2c, 0x2f2e, PDF_CMAP_RANGE, 7934 },
	{ 0x2f2f, 0x2f2f, PDF_CMAP_SINGLE, 7933 },
	{ 0x2f30, 0x2f30, PDF_CMAP_SINGLE, 7891 },
	{ 0x2f5e, 0x2f5e, PDF_CMAP_SINGLE, 8272 },
	{ 0x2f5f, 0x2f5f, PDF_CMAP_SINGLE, 8271 },
};

static const unsigned short pdf_cmap_NWP_V_table[15] =
{
	8268,8274,8272,8271,8281,8276,8279,8278,
	8269,8273,8283,7933,7891,8272,8271,
};

pdf_cmap pdf_cmap_NWP_V =
{
	-1, "NWP-V", "NWP-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	48, 48, (pdf_range*) pdf_cmap_NWP_V_ranges,
	15, 15, (unsigned short*) pdf_cmap_NWP_V_table,
};

/* RKSJ-H */

static const pdf_range pdf_cmap_RKSJ_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
	{ 0x00a0, 0x00df, PDF_CMAP_RANGE, 326 },
	{ 0x8140, 0x817e, PDF_CMAP_RANGE, 633 },
	{ 0x8180, 0x81ac, PDF_CMAP_RANGE, 696 },
	{ 0x81b8, 0x81bf, PDF_CMAP_RANGE, 741 },
	{ 0x81c8, 0x81ce, PDF_CMAP_RANGE, 749 },
	{ 0x81da, 0x81e8, PDF_CMAP_RANGE, 756 },
	{ 0x81f0, 0x81f7, PDF_CMAP_RANGE, 771 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 779 },
	{ 0x824f, 0x8258, PDF_CMAP_RANGE, 780 },
	{ 0x8260, 0x8279, PDF_CMAP_RANGE, 790 },
	{ 0x8281, 0x829a, PDF_CMAP_RANGE, 816 },
	{ 0x829f, 0x82f1, PDF_CMAP_RANGE, 842 },
	{ 0x8340, 0x837e, PDF_CMAP_RANGE, 925 },
	{ 0x8380, 0x8396, PDF_CMAP_RANGE, 988 },
	{ 0x839f, 0x83b6, PDF_CMAP_RANGE, 1011 },
	{ 0x83bf, 0x83d6, PDF_CMAP_RANGE, 1035 },
	{ 0x8440, 0x8460, PDF_CMAP_RANGE, 1059 },
	{ 0x8470, 0x847e, PDF_CMAP_RANGE, 1092 },
	{ 0x8480, 0x8491, PDF_CMAP_RANGE, 1107 },
	{ 0x849f, 0x84be, PDF_CMAP_TABLE, 0 },
	{ 0x889f, 0x88fc, PDF_CMAP_RANGE, 1125 },
	{ 0x8940, 0x897e, PDF_CMAP_RANGE, 1219 },
	{ 0x8980, 0x89fc, PDF_CMAP_RANGE, 1282 },
	{ 0x8a40, 0x8a7e, PDF_CMAP_RANGE, 1407 },
	{ 0x8a80, 0x8afc, PDF_CMAP_RANGE, 1470 },
	{ 0x8b40, 0x8b7e, PDF_CMAP_RANGE, 1595 },
	{ 0x8b80, 0x8bfc, PDF_CMAP_RANGE, 1658 },
	{ 0x8c40, 0x8c7e, PDF_CMAP_RANGE, 1783 },
	{ 0x8c80, 0x8cfc, PDF_CMAP_RANGE, 1846 },
	{ 0x8d40, 0x8d7e, PDF_CMAP_RANGE, 1971 },
	{ 0x8d80, 0x8dfc, PDF_CMAP_RANGE, 2034 },
	{ 0x8e40, 0x8e7e, PDF_CMAP_RANGE, 2159 },
	{ 0x8e80, 0x8efc, PDF_CMAP_RANGE, 2222 },
	{ 0x8f40, 0x8f7e, PDF_CMAP_RANGE, 2347 },
	{ 0x8f80, 0x8ffc, PDF_CMAP_RANGE, 2410 },
	{ 0x9040, 0x907e, PDF_CMAP_RANGE, 2535 },
	{ 0x9080, 0x90fc, PDF_CMAP_RANGE, 2598 },
	{ 0x9140, 0x917e, PDF_CMAP_RANGE, 2723 },
	{ 0x9180, 0x91fc, PDF_CMAP_RANGE, 2786 },
	{ 0x9240, 0x927e, PDF_CMAP_RANGE, 2911 },
	{ 0x9280, 0x92fc, PDF_CMAP_RANGE, 2974 },
	{ 0x9340, 0x937e, PDF_CMAP_RANGE, 3099 },
	{ 0x9380, 0x93fc, PDF_CMAP_RANGE, 3162 },
	{ 0x9440, 0x947e, PDF_CMAP_RANGE, 3287 },
	{ 0x9480, 0x94fc, PDF_CMAP_RANGE, 3350 },
	{ 0x9540, 0x957e, PDF_CMAP_RANGE, 3475 },
	{ 0x9580, 0x95fc, PDF_CMAP_RANGE, 3538 },
	{ 0x9640, 0x967e, PDF_CMAP_RANGE, 3663 },
	{ 0x9680, 0x96fc, PDF_CMAP_RANGE, 3726 },
	{ 0x9740, 0x977e, PDF_CMAP_RANGE, 3851 },
	{ 0x9780, 0x97fc, PDF_CMAP_RANGE, 3914 },
	{ 0x9840, 0x9872, PDF_CMAP_RANGE, 4039 },
	{ 0x989f, 0x98fc, PDF_CMAP_RANGE, 4090 },
	{ 0x9940, 0x997e, PDF_CMAP_RANGE, 4184 },
	{ 0x9980, 0x99fc, PDF_CMAP_RANGE, 4247 },
	{ 0x9a40, 0x9a7e, PDF_CMAP_RANGE, 4372 },
	{ 0x9a80, 0x9afc, PDF_CMAP_RANGE, 4435 },
	{ 0x9b40, 0x9b7e, PDF_CMAP_RANGE, 4560 },
	{ 0x9b80, 0x9bfc, PDF_CMAP_RANGE, 4623 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
	{ 0x9c40, 0x9c7e, PDF_CMAP_RANGE, 4748 },
	{ 0x9c80, 0x9cfc, PDF_CMAP_RANGE, 4811 },
	{ 0x9d40, 0x9d7e, PDF_CMAP_RANGE, 4936 },
	{ 0x9d80, 0x9dfc, PDF_CMAP_RANGE, 4999 },
	{ 0x9e40, 0x9e7e, PDF_CMAP_RANGE, 5124 },
	{ 0x9e80, 0x9efc, PDF_CMAP_RANGE, 5187 },
	{ 0x9f40, 0x9f7e, PDF_CMAP_RANGE, 5312 },
	{ 0x9f80, 0x9ffc, PDF_CMAP_RANGE, 5375 },
	{ 0xe040, 0xe07e, PDF_CMAP_RANGE, 5500 },
	{ 0xe080, 0xe0fc, PDF_CMAP_RANGE, 5563 },
	{ 0xe140, 0xe17e, PDF_CMAP_RANGE, 5688 },
	{ 0xe180, 0xe1fc, PDF_CMAP_RANGE, 5751 },
	{ 0xe240, 0xe27e, PDF_CMAP_RANGE, 5876 },
	{ 0xe280, 0xe2fc, PDF_CMAP_RANGE, 5939 },
	{ 0xe340, 0xe37e, PDF_CMAP_RANGE, 6064 },
	{ 0xe380, 0xe3fc, PDF_CMAP_RANGE, 6127 },
	{ 0xe440, 0xe47e, PDF_CMAP_RANGE, 6252 },
	{ 0xe480, 0xe4fc, PDF_CMAP_RANGE, 6315 },
	{ 0xe540, 0xe57e, PDF_CMAP_RANGE, 6440 },
	{ 0xe580, 0xe5fc, PDF_CMAP_RANGE, 6503 },
	{ 0xe640, 0xe67e, PDF_CMAP_RANGE, 6628 },
	{ 0xe680, 0xe6fc, PDF_CMAP_RANGE, 6691 },
	{ 0xe740, 0xe77e, PDF_CMAP_RANGE, 6816 },
	{ 0xe780, 0xe7fc, PDF_CMAP_RANGE, 6879 },
	{ 0xe840, 0xe87e, PDF_CMAP_RANGE, 7004 },
	{ 0xe880, 0xe8fc, PDF_CMAP_RANGE, 7067 },
	{ 0xe940, 0xe97e, PDF_CMAP_RANGE, 7192 },
	{ 0xe980, 0xe9fc, PDF_CMAP_RANGE, 7255 },
	{ 0xea40, 0xea7e, PDF_CMAP_RANGE, 7380 },
	{ 0xea80, 0xeaa2, PDF_CMAP_RANGE, 7443 },
	{ 0xeaa3, 0xeaa4, PDF_CMAP_RANGE, 8284 },
};

static const unsigned short pdf_cmap_RKSJ_H_table[32] =
{
	7479,7481,7491,7495,7503,7499,7507,7523,
	7515,7531,7539,7480,7482,7494,7498,7506,
	7502,7514,7530,7522,7538,7554,7511,7526,
	7519,7534,7542,7508,7527,7516,7535,7545,
};

pdf_cmap pdf_cmap_RKSJ_H =
{
	-1, "RKSJ-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8140, 0x9ffc },
		{ 1, 0x00a0, 0x00df },
		{ 2, 0xe040, 0xfcfc },
	},
	122, 122, (pdf_range*) pdf_cmap_RKSJ_H_ranges,
	32, 32, (unsigned short*) pdf_cmap_RKSJ_H_table,
};

/* RKSJ-V */

static const pdf_range pdf_cmap_RKSJ_V_ranges[] =
{
	{ 0x8141, 0x8142, PDF_CMAP_RANGE, 7887 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 7889 },
	{ 0x815b, 0x815d, PDF_CMAP_RANGE, 7891 },
	{ 0x8160, 0x8164, PDF_CMAP_RANGE, 7894 },
	{ 0x8169, 0x817a, PDF_CMAP_RANGE, 7899 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 7917 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 7918 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 7919 },
	{ 0x82a3, 0x82a3, PDF_CMAP_SINGLE, 7920 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 7921 },
	{ 0x82a7, 0x82a7, PDF_CMAP_SINGLE, 7922 },
	{ 0x82c1, 0x82c1, PDF_CMAP_SINGLE, 7923 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 7924 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 7925 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 7926 },
	{ 0x82ec, 0x82ec, PDF_CMAP_SINGLE, 7927 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 7928 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 7929 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 7930 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7931 },
	{ 0x8348, 0x8348, PDF_CMAP_SINGLE, 7932 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 7933 },
	{ 0x8383, 0x8383, PDF_CMAP_SINGLE, 7934 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 7935 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 7936 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 7937 },
	{ 0x8395, 0x8396, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_RKSJ_V_table[1] = { 0 };

pdf_cmap pdf_cmap_RKSJ_V =
{
	-1, "RKSJ-V", "RKSJ-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	27, 27, (pdf_range*) pdf_cmap_RKSJ_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_RKSJ_V_table,
};

/* Roman */

static const pdf_range pdf_cmap_Roman_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 231 },
};

static const unsigned short pdf_cmap_Roman_table[1] = { 0 };

pdf_cmap pdf_cmap_Roman =
{
	-1, "Roman", "", nil, 0,
	1, /* codespace table */
	{
		{ 1, 0x0000, 0x00ff },
	},
	1, 1, (pdf_range*) pdf_cmap_Roman_ranges,
	0, 0, (unsigned short*) pdf_cmap_Roman_table,
};

/* UniJIS-UCS2-H */

static const pdf_range pdf_cmap_UniJIS_UCS2_H_ranges[] =
{
	{ 0x0020, 0x005b, PDF_CMAP_RANGE, 1 },
	{ 0x005c, 0x005c, PDF_CMAP_SINGLE, 97 },
	{ 0x005d, 0x007e, PDF_CMAP_RANGE, 62 },
	{ 0x00a1, 0x00a3, PDF_CMAP_RANGE, 101 },
	{ 0x00a4, 0x00b1, PDF_CMAP_TABLE, 0 },
	{ 0x00b2, 0x00b3, PDF_CMAP_RANGE, 157 },
	{ 0x00b4, 0x00bb, PDF_CMAP_TABLE, 14 },
	{ 0x00bc, 0x00be, PDF_CMAP_RANGE, 161 },
	{ 0x00bf, 0x00bf, PDF_CMAP_SINGLE, 126 },
	{ 0x00c0, 0x00c5, PDF_CMAP_RANGE, 164 },
	{ 0x00c6, 0x00c6, PDF_CMAP_SINGLE, 139 },
	{ 0x00c7, 0x00d6, PDF_CMAP_RANGE, 170 },
	{ 0x00d7, 0x00d8, PDF_CMAP_TABLE, 22 },
	{ 0x00d9, 0x00de, PDF_CMAP_RANGE, 187 },
	{ 0x00df, 0x00df, PDF_CMAP_SINGLE, 150 },
	{ 0x00e0, 0x00e5, PDF_CMAP_RANGE, 193 },
	{ 0x00e6, 0x00e6, PDF_CMAP_SINGLE, 145 },
	{ 0x00e7, 0x00f6, PDF_CMAP_RANGE, 199 },
	{ 0x00f7, 0x00f8, PDF_CMAP_TABLE, 24 },
	{ 0x00f9, 0x00ff, PDF_CMAP_RANGE, 216 },
	{ 0x0100, 0x0101, PDF_CMAP_TABLE, 26 },
	{ 0x0112, 0x0113, PDF_CMAP_TABLE, 28 },
	{ 0x011a, 0x011b, PDF_CMAP_TABLE, 30 },
	{ 0x0127, 0x012b, PDF_CMAP_TABLE, 32 },
	{ 0x0131, 0x0131, PDF_CMAP_SINGLE, 146 },
	{ 0x0141, 0x0142, PDF_CMAP_TABLE, 37 },
	{ 0x014b, 0x014d, PDF_CMAP_TABLE, 39 },
	{ 0x0152, 0x0153, PDF_CMAP_TABLE, 42 },
	{ 0x0160, 0x0161, PDF_CMAP_TABLE, 44 },
	{ 0x0168, 0x016b, PDF_CMAP_TABLE, 46 },
	{ 0x016e, 0x016f, PDF_CMAP_TABLE, 50 },
	{ 0x0178, 0x0178, PDF_CMAP_SINGLE, 224 },
	{ 0x017d, 0x017e, PDF_CMAP_TABLE, 52 },
	{ 0x01c0, 0x01c0, PDF_CMAP_SINGLE, 99 },
	{ 0x01cd, 0x01d4, PDF_CMAP_TABLE, 54 },
	{ 0x01fd, 0x01fd, PDF_CMAP_SINGLE, 9421 },
	{ 0x0251, 0x0251, PDF_CMAP_SINGLE, 9418 },
	{ 0x0254, 0x0254, PDF_CMAP_SINGLE, 9423 },
	{ 0x0259, 0x025b, PDF_CMAP_TABLE, 62 },
	{ 0x0275, 0x0275, PDF_CMAP_SINGLE, 9437 },
	{ 0x0283, 0x0283, PDF_CMAP_SINGLE, 9442 },
	{ 0x028c, 0x028c, PDF_CMAP_SINGLE, 9438 },
	{ 0x0292, 0x0292, PDF_CMAP_SINGLE, 9441 },
	{ 0x02d0, 0x02d0, PDF_CMAP_SINGLE, 9443 },
	{ 0x0300, 0x0300, PDF_CMAP_SINGLE, 65 },
	{ 0x0301, 0x0302, PDF_CMAP_RANGE, 127 },
	{ 0x0303, 0x0305, PDF_CMAP_TABLE, 65 },
	{ 0x0306, 0x0308, PDF_CMAP_RANGE, 130 },
	{ 0x030a, 0x030c, PDF_CMAP_TABLE, 68 },
	{ 0x0327, 0x0328, PDF_CMAP_TABLE, 71 },
	{ 0x0332, 0x0332, PDF_CMAP_SINGLE, 64 },
	{ 0x0336, 0x0336, PDF_CMAP_SINGLE, 138 },
	{ 0x0361, 0x0361, PDF_CMAP_SINGLE, 758 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 1011 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 1028 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 1035 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 1052 },
	{ 0x03d0, 0x03d1, PDF_CMAP_TABLE, 73 },
	{ 0x03db, 0x03db, PDF_CMAP_SINGLE, 12095 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 1065 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 1059 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 1066 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 1099 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 1098 },
	{ 0x045b, 0x045b, PDF_CMAP_SINGLE, 12092 },
	{ 0x1ebc, 0x1ebd, PDF_CMAP_TABLE, 75 },
	{ 0x2002, 0x2003, PDF_CMAP_TABLE, 77 },
	{ 0x2010, 0x2016, PDF_CMAP_TABLE, 79 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 670 },
	{ 0x201a, 0x201a, PDF_CMAP_SINGLE, 120 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 672 },
	{ 0x201e, 0x201e, PDF_CMAP_SINGLE, 121 },
	{ 0x2020, 0x2021, PDF_CMAP_RANGE, 776 },
	{ 0x2022, 0x2022, PDF_CMAP_SINGLE, 119 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 86 },
	{ 0x2030, 0x2030, PDF_CMAP_SINGLE, 772 },
	{ 0x2032, 0x2033, PDF_CMAP_RANGE, 708 },
	{ 0x2039, 0x203a, PDF_CMAP_RANGE, 110 },
	{ 0x203b, 0x203c, PDF_CMAP_TABLE, 88 },
	{ 0x203e, 0x203e, PDF_CMAP_SINGLE, 325 },
	{ 0x2044, 0x2044, PDF_CMAP_SINGLE, 104 },
	{ 0x2049, 0x2049, PDF_CMAP_SINGLE, 12112 },
	{ 0x2070, 0x2070, PDF_CMAP_SINGLE, 9377 },
	{ 0x2074, 0x2079, PDF_CMAP_RANGE, 9378 },
	{ 0x2080, 0x2089, PDF_CMAP_RANGE, 9384 },
	{ 0x20ac, 0x20ac, PDF_CMAP_SINGLE, 9354 },
	{ 0x20dd, 0x20dd, PDF_CMAP_SINGLE, 779 },
	{ 0x2100, 0x2100, PDF_CMAP_SINGLE, 11855 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 710 },
	{ 0x2105, 0x2105, PDF_CMAP_SINGLE, 11859 },
	{ 0x2109, 0x210a, PDF_CMAP_TABLE, 90 },
	{ 0x210f, 0x210f, PDF_CMAP_SINGLE, 12092 },
	{ 0x2113, 0x2113, PDF_CMAP_SINGLE, 8025 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 7610 },
	{ 0x2121, 0x2122, PDF_CMAP_TABLE, 92 },
	{ 0x2126, 0x2126, PDF_CMAP_SINGLE, 9355 },
	{ 0x212b, 0x212b, PDF_CMAP_SINGLE, 771 },
	{ 0x2135, 0x2135, PDF_CMAP_SINGLE, 12089 },
	{ 0x2153, 0x2154, PDF_CMAP_RANGE, 9375 },
	{ 0x215b, 0x215e, PDF_CMAP_RANGE, 9371 },
	{ 0x2160, 0x2169, PDF_CMAP_RANGE, 7575 },
	{ 0x216a, 0x216b, PDF_CMAP_RANGE, 8225 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 8092 },
	{ 0x217a, 0x217b, PDF_CMAP_RANGE, 8298 },
	{ 0x217f, 0x217f, PDF_CMAP_SINGLE, 8303 },
	{ 0x2190, 0x2191, PDF_CMAP_RANGE, 737 },
	{ 0x2192, 0x2195, PDF_CMAP_TABLE, 94 },
	{ 0x2196, 0x2197, PDF_CMAP_RANGE, 12204 },
	{ 0x2198, 0x2199, PDF_CMAP_RANGE, 12202 },
	{ 0x21c4, 0x21c5, PDF_CMAP_RANGE, 8310 },
	{ 0x21c6, 0x21c6, PDF_CMAP_SINGLE, 8309 },
	{ 0x21cc, 0x21cc, PDF_CMAP_SINGLE, 12206 },
	{ 0x21d0, 0x21d0, PDF_CMAP_SINGLE, 12200 },
	{ 0x21d2, 0x21d2, PDF_CMAP_SINGLE, 752 },
	{ 0x21d4, 0x21d4, PDF_CMAP_SINGLE, 753 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 98 },
	{ 0x2200, 0x2200, PDF_CMAP_SINGLE, 754 },
	{ 0x2202, 0x2203, PDF_CMAP_TABLE, 102 },
	{ 0x2205, 0x2205, PDF_CMAP_SINGLE, 12184 },
	{ 0x2207, 0x2208, PDF_CMAP_TABLE, 104 },
	{ 0x220a, 0x220b, PDF_CMAP_TABLE, 106 },
	{ 0x2211, 0x2213, PDF_CMAP_TABLE, 108 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 765 },
	{ 0x221d, 0x2220, PDF_CMAP_TABLE, 111 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 666 },
	{ 0x2227, 0x2228, PDF_CMAP_RANGE, 749 },
	{ 0x2229, 0x222a, PDF_CMAP_TABLE, 115 },
	{ 0x222b, 0x222c, PDF_CMAP_RANGE, 769 },
	{ 0x222d, 0x222e, PDF_CMAP_TABLE, 117 },
	{ 0x2234, 0x2235, PDF_CMAP_TABLE, 119 },
	{ 0x223c, 0x223d, PDF_CMAP_TABLE, 121 },
	{ 0x2243, 0x2243, PDF_CMAP_SINGLE, 12120 },
	{ 0x2252, 0x2252, PDF_CMAP_SINGLE, 762 },
	{ 0x2260, 0x2261, PDF_CMAP_TABLE, 123 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 701 },
	{ 0x226a, 0x226b, PDF_CMAP_RANGE, 763 },
	{ 0x2272, 0x2273, PDF_CMAP_RANGE, 12121 },
	{ 0x2282, 0x2283, PDF_CMAP_RANGE, 745 },
	{ 0x2286, 0x2287, PDF_CMAP_RANGE, 743 },
	{ 0x2295, 0x2298, PDF_CMAP_TABLE, 125 },
	{ 0x229d, 0x229e, PDF_CMAP_TABLE, 129 },
	{ 0x22a0, 0x22a0, PDF_CMAP_SINGLE, 12185 },
	{ 0x22a5, 0x22a5, PDF_CMAP_SINGLE, 757 },
	{ 0x22bf, 0x22bf, PDF_CMAP_SINGLE, 7630 },
	{ 0x22ee, 0x22ef, PDF_CMAP_TABLE, 131 },
	{ 0x2300, 0x2300, PDF_CMAP_SINGLE, 12184 },
	{ 0x2307, 0x2307, PDF_CMAP_SINGLE, 12219 },
	{ 0x2312, 0x2312, PDF_CMAP_SINGLE, 758 },
	{ 0x244a, 0x244a, PDF_CMAP_SINGLE, 12116 },
	{ 0x2460, 0x2473, PDF_CMAP_RANGE, 7555 },
	{ 0x2474, 0x2487, PDF_CMAP_RANGE, 8071 },
	{ 0x2488, 0x2490, PDF_CMAP_RANGE, 8062 },
	{ 0x249c, 0x24b5, PDF_CMAP_RANGE, 8112 },
	{ 0x24b6, 0x24cf, PDF_CMAP_RANGE, 10339 },
	{ 0x24d0, 0x24e9, PDF_CMAP_RANGE, 10313 },
	{ 0x24ea, 0x24ea, PDF_CMAP_SINGLE, 8224 },
	{ 0x2500, 0x254b, PDF_CMAP_RANGE, 7479 },
	{ 0x2550, 0x2550, PDF_CMAP_SINGLE, 8251 },
	{ 0x255e, 0x255e, PDF_CMAP_SINGLE, 8252 },
	{ 0x2561, 0x2561, PDF_CMAP_SINGLE, 8254 },
	{ 0x256a, 0x256a, PDF_CMAP_SINGLE, 8253 },
	{ 0x256d, 0x256e, PDF_CMAP_RANGE, 8247 },
	{ 0x256f, 0x2570, PDF_CMAP_TABLE, 133 },
	{ 0x2571, 0x2573, PDF_CMAP_RANGE, 8261 },
	{ 0x2581, 0x2588, PDF_CMAP_RANGE, 8230 },
	{ 0x2589, 0x258f, PDF_CMAP_TABLE, 135 },
	{ 0x2594, 0x2595, PDF_CMAP_RANGE, 8245 },
	{ 0x25a0, 0x25a2, PDF_CMAP_TABLE, 142 },
	{ 0x25aa, 0x25ab, PDF_CMAP_TABLE, 145 },
	{ 0x25b2, 0x25b3, PDF_CMAP_TABLE, 147 },
	{ 0x25b6, 0x25b7, PDF_CMAP_TABLE, 149 },
	{ 0x25bc, 0x25bd, PDF_CMAP_TABLE, 151 },
	{ 0x25c0, 0x25c1, PDF_CMAP_TABLE, 153 },
	{ 0x25c6, 0x25c7, PDF_CMAP_TABLE, 155 },
	{ 0x25c9, 0x25c9, PDF_CMAP_SINGLE, 8210 },
	{ 0x25cb, 0x25cc, PDF_CMAP_TABLE, 157 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 159 },
	{ 0x25e2, 0x25e3, PDF_CMAP_RANGE, 8255 },
	{ 0x25e4, 0x25e6, PDF_CMAP_TABLE, 161 },
	{ 0x25ef, 0x25ef, PDF_CMAP_SINGLE, 779 },
	{ 0x2600, 0x2603, PDF_CMAP_RANGE, 8215 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 164 },
	{ 0x260e, 0x260e, PDF_CMAP_SINGLE, 8056 },
	{ 0x2612, 0x2612, PDF_CMAP_SINGLE, 12185 },
	{ 0x261c, 0x261d, PDF_CMAP_RANGE, 8220 },
	{ 0x261e, 0x261f, PDF_CMAP_TABLE, 166 },
	{ 0x2640, 0x2640, PDF_CMAP_SINGLE, 706 },
	{ 0x2642, 0x2642, PDF_CMAP_SINGLE, 705 },
	{ 0x2660, 0x2667, PDF_CMAP_TABLE, 168 },
	{ 0x2668, 0x2669, PDF_CMAP_RANGE, 12098 },
	{ 0x266a, 0x266a, PDF_CMAP_SINGLE, 775 },
	{ 0x266c, 0x266d, PDF_CMAP_TABLE, 176 },
	{ 0x266f, 0x266f, PDF_CMAP_SINGLE, 773 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12176 },
	{ 0x271a, 0x271a, PDF_CMAP_SINGLE, 12241 },
	{ 0x2756, 0x2756, PDF_CMAP_SINGLE, 12259 },
	{ 0x2776, 0x277e, PDF_CMAP_RANGE, 8286 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8206 },
	{ 0x2e83, 0x2e83, PDF_CMAP_SINGLE, 14305 },
	{ 0x2e85, 0x2e85, PDF_CMAP_SINGLE, 13856 },
	{ 0x2e87, 0x2e87, PDF_CMAP_SINGLE, 14105 },
	{ 0x2e89, 0x2e89, PDF_CMAP_SINGLE, 14356 },
	{ 0x2e8b, 0x2e8b, PDF_CMAP_SINGLE, 14110 },
	{ 0x2e8c, 0x2e8d, PDF_CMAP_RANGE, 13833 },
	{ 0x2e8e, 0x2e90, PDF_CMAP_TABLE, 178 },
	{ 0x2e92, 0x2e99, PDF_CMAP_TABLE, 181 },
	{ 0x2e9b, 0x2e9b, PDF_CMAP_SINGLE, 5089 },
	{ 0x2e9f, 0x2ea0, PDF_CMAP_TABLE, 189 },
	{ 0x2ea1, 0x2ea2, PDF_CMAP_RANGE, 14689 },
	{ 0x2ea3, 0x2ea4, PDF_CMAP_TABLE, 191 },
	{ 0x2ea6, 0x2ea6, PDF_CMAP_SINGLE, 14157 },
	{ 0x2ea8, 0x2eae, PDF_CMAP_TABLE, 193 },
	{ 0x2eb1, 0x2eb3, PDF_CMAP_TABLE, 200 },
	{ 0x2eb7, 0x2eb7, PDF_CMAP_SINGLE, 14078 },
	{ 0x2eb9, 0x2eb9, PDF_CMAP_SINGLE, 14099 },
	{ 0x2ebc, 0x2ebd, PDF_CMAP_TABLE, 203 },
	{ 0x2ebe, 0x2ec0, PDF_CMAP_RANGE, 14197 },
	{ 0x2ec1, 0x2ec4, PDF_CMAP_TABLE, 205 },
	{ 0x2ec6, 0x2ec6, PDF_CMAP_SINGLE, 13682 },
	{ 0x2eca, 0x2eca, PDF_CMAP_SINGLE, 13898 },
	{ 0x2ecc, 0x2ecd, PDF_CMAP_TABLE, 209 },
	{ 0x2ecf, 0x2ecf, PDF_CMAP_SINGLE, 15262 },
	{ 0x2ed1, 0x2ed2, PDF_CMAP_TABLE, 211 },
	{ 0x2ed6, 0x2ed8, PDF_CMAP_TABLE, 213 },
	{ 0x2edd, 0x2edf, PDF_CMAP_TABLE, 216 },
	{ 0x2ee4, 0x2ee4, PDF_CMAP_SINGLE, 1614 },
	{ 0x2ee8, 0x2ee9, PDF_CMAP_TABLE, 219 },
	{ 0x2eeb, 0x2eeb, PDF_CMAP_SINGLE, 2666 },
	{ 0x2eed, 0x2eed, PDF_CMAP_SINGLE, 2243 },
	{ 0x2eef, 0x2eef, PDF_CMAP_SINGLE, 3965 },
	{ 0x2ef2, 0x2ef2, PDF_CMAP_SINGLE, 1615 },
	{ 0x2f00, 0x2f13, PDF_CMAP_TABLE, 221 },
	{ 0x2f14, 0x2f15, PDF_CMAP_RANGE, 4301 },
	{ 0x2f16, 0x2f20, PDF_CMAP_TABLE, 241 },
	{ 0x2f21, 0x2f22, PDF_CMAP_RANGE, 4538 },
	{ 0x2f23, 0x2fbe, PDF_CMAP_TABLE, 252 },
	{ 0x2fbf, 0x2fc0, PDF_CMAP_RANGE, 7299 },
	{ 0x2fc1, 0x2fd0, PDF_CMAP_TABLE, 408 },
	{ 0x2fd1, 0x2fd2, PDF_CMAP_RANGE, 7457 },
	{ 0x2fd3, 0x2fd3, PDF_CMAP_SINGLE, 3966 },
	{ 0x2fd4, 0x2fd5, PDF_CMAP_RANGE, 7472 },
	{ 0x3000, 0x3002, PDF_CMAP_RANGE, 633 },
	{ 0x3003, 0x3004, PDF_CMAP_TABLE, 424 },
	{ 0x3005, 0x3007, PDF_CMAP_RANGE, 657 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 682 },
	{ 0x3012, 0x3013, PDF_CMAP_TABLE, 426 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 676 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 428 },
	{ 0x301f, 0x3020, PDF_CMAP_TABLE, 430 },
	{ 0x3030, 0x3030, PDF_CMAP_SINGLE, 12218 },
	{ 0x3033, 0x3035, PDF_CMAP_RANGE, 12108 },
	{ 0x3036, 0x3036, PDF_CMAP_SINGLE, 8057 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 842 },
	{ 0x3094, 0x3094, PDF_CMAP_SINGLE, 7958 },
	{ 0x309b, 0x309c, PDF_CMAP_RANGE, 643 },
	{ 0x309d, 0x309e, PDF_CMAP_RANGE, 653 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 925 },
	{ 0x30f7, 0x30fa, PDF_CMAP_RANGE, 8313 },
	{ 0x30fb, 0x30fc, PDF_CMAP_TABLE, 432 },
	{ 0x30fd, 0x30fe, PDF_CMAP_RANGE, 651 },
	{ 0x3220, 0x3229, PDF_CMAP_RANGE, 10126 },
	{ 0x322a, 0x322f, PDF_CMAP_RANGE, 8198 },
	{ 0x3230, 0x3230, PDF_CMAP_SINGLE, 8197 },
	{ 0x3231, 0x3232, PDF_CMAP_RANGE, 7618 },
	{ 0x3233, 0x3243, PDF_CMAP_TABLE, 434 },
	{ 0x3280, 0x3289, PDF_CMAP_RANGE, 10461 },
	{ 0x328a, 0x328f, PDF_CMAP_RANGE, 10472 },
	{ 0x3290, 0x329f, PDF_CMAP_TABLE, 451 },
	{ 0x32a0, 0x32a1, PDF_CMAP_RANGE, 10486 },
	{ 0x32a2, 0x32a3, PDF_CMAP_TABLE, 467 },
	{ 0x32a4, 0x32a8, PDF_CMAP_RANGE, 7613 },
	{ 0x32a9, 0x32b0, PDF_CMAP_TABLE, 469 },
	{ 0x32d0, 0x32fe, PDF_CMAP_RANGE, 10413 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8048 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11874 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 477 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11889 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 488 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11900 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8051 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11904 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 494 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11912 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 500 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11915 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 11918 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 502 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 11924 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 505 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 11936 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 513 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 11939 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 515 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 11944 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 519 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 11946 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 521 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 11955 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8044 },
	{ 0x3371, 0x3371, PDF_CMAP_SINGLE, 11861 },
	{ 0x337b, 0x337f, PDF_CMAP_TABLE, 525 },
	{ 0x3385, 0x3387, PDF_CMAP_RANGE, 8031 },
	{ 0x3388, 0x3389, PDF_CMAP_RANGE, 8192 },
	{ 0x338d, 0x338d, PDF_CMAP_SINGLE, 11864 },
	{ 0x338e, 0x338f, PDF_CMAP_RANGE, 7604 },
	{ 0x3390, 0x3390, PDF_CMAP_SINGLE, 8035 },
	{ 0x3396, 0x3398, PDF_CMAP_TABLE, 530 },
	{ 0x339b, 0x339b, PDF_CMAP_SINGLE, 11865 },
	{ 0x339c, 0x339e, PDF_CMAP_RANGE, 7601 },
	{ 0x339f, 0x33a3, PDF_CMAP_TABLE, 533 },
	{ 0x33a4, 0x33a5, PDF_CMAP_RANGE, 8022 },
	{ 0x33a6, 0x33a6, PDF_CMAP_SINGLE, 8188 },
	{ 0x33b0, 0x33b3, PDF_CMAP_TABLE, 538 },
	{ 0x33c2, 0x33c2, PDF_CMAP_SINGLE, 11856 },
	{ 0x33c4, 0x33c4, PDF_CMAP_SINGLE, 7606 },
	{ 0x33c8, 0x33c8, PDF_CMAP_SINGLE, 8194 },
	{ 0x33cb, 0x33cd, PDF_CMAP_TABLE, 542 },
	{ 0x33d4, 0x33d4, PDF_CMAP_SINGLE, 8036 },
	{ 0x33d7, 0x33d8, PDF_CMAP_RANGE, 11869 },
	{ 0x33da, 0x33da, PDF_CMAP_SINGLE, 11851 },
	{ 0x3402, 0x3402, PDF_CMAP_SINGLE, 13698 },
	{ 0x3405, 0x3405, PDF_CMAP_SINGLE, 15387 },
	{ 0x3427, 0x3427, PDF_CMAP_SINGLE, 13910 },
	{ 0x3488, 0x3488, PDF_CMAP_SINGLE, 15442 },
	{ 0x34db, 0x34db, PDF_CMAP_SINGLE, 15425 },
	{ 0x351f, 0x351f, PDF_CMAP_SINGLE, 13865 },
	{ 0x353e, 0x353e, PDF_CMAP_SINGLE, 14110 },
	{ 0x378d, 0x378d, PDF_CMAP_SINGLE, 13850 },
	{ 0x37e2, 0x37e2, PDF_CMAP_SINGLE, 14123 },
	{ 0x3af3, 0x3af3, PDF_CMAP_SINGLE, 15424 },
	{ 0x3b22, 0x3b22, PDF_CMAP_SINGLE, 15433 },
	{ 0x3b88, 0x3b88, PDF_CMAP_SINGLE, 13965 },
	{ 0x3e8a, 0x3e8a, PDF_CMAP_SINGLE, 15427 },
	{ 0x3eda, 0x3eda, PDF_CMAP_SINGLE, 15432 },
	{ 0x3fb1, 0x3fb1, PDF_CMAP_SINGLE, 14164 },
	{ 0x4093, 0x4093, PDF_CMAP_SINGLE, 15436 },
	{ 0x4103, 0x4103, PDF_CMAP_SINGLE, 15439 },
	{ 0x4264, 0x4264, PDF_CMAP_SINGLE, 14176 },
	{ 0x4293, 0x4293, PDF_CMAP_SINGLE, 15440 },
	{ 0x440c, 0x440c, PDF_CMAP_SINGLE, 15426 },
	{ 0x4453, 0x4453, PDF_CMAP_SINGLE, 14195 },
	{ 0x457a, 0x457a, PDF_CMAP_SINGLE, 15435 },
	{ 0x4665, 0x4665, PDF_CMAP_SINGLE, 15438 },
	{ 0x46ae, 0x46ae, PDF_CMAP_SINGLE, 15441 },
	{ 0x4be8, 0x4be8, PDF_CMAP_SINGLE, 15430 },
	{ 0x4e00, 0x4e01, PDF_CMAP_TABLE, 545 },
	{ 0x4e03, 0x4e03, PDF_CMAP_SINGLE, 2275 },
	{ 0x4e04, 0x4e05, PDF_CMAP_RANGE, 14296 },
	{ 0x4e07, 0x4e0b, PDF_CMAP_TABLE, 547 },
	{ 0x4e0d, 0x4e0e, PDF_CMAP_TABLE, 552 },
	{ 0x4e10, 0x4e11, PDF_CMAP_TABLE, 554 },
	{ 0x4e14, 0x4e19, PDF_CMAP_TABLE, 556 },
	{ 0x4e1e, 0x4e1f, PDF_CMAP_TABLE, 562 },
	{ 0x4e21, 0x4e21, PDF_CMAP_SINGLE, 3974 },
	{ 0x4e26, 0x4e26, PDF_CMAP_SINGLE, 3602 },
	{ 0x4e28, 0x4e28, PDF_CMAP_SINGLE, 8371 },
	{ 0x4e2a, 0x4e2d, PDF_CMAP_TABLE, 564 },
	{ 0x4e2f, 0x4e30, PDF_CMAP_RANGE, 14300 },
	{ 0x4e31, 0x4e32, PDF_CMAP_TABLE, 568 },
	{ 0x4e36, 0x4e39, PDF_CMAP_TABLE, 570 },
	{ 0x4e3b, 0x4e3c, PDF_CMAP_TABLE, 574 },
	{ 0x4e3f, 0x4e3f, PDF_CMAP_SINGLE, 4097 },
	{ 0x4e40, 0x4e41, PDF_CMAP_RANGE, 14302 },
	{ 0x4e42, 0x4e45, PDF_CMAP_TABLE, 576 },
	{ 0x4e48, 0x4e48, PDF_CMAP_SINGLE, 14126 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 3309 },
	{ 0x4e4d, 0x4e4f, PDF_CMAP_TABLE, 580 },
	{ 0x4e55, 0x4e5a, PDF_CMAP_TABLE, 583 },
	{ 0x4e5d, 0x4e5f, PDF_CMAP_TABLE, 589 },
	{ 0x4e62, 0x4e62, PDF_CMAP_SINGLE, 4659 },
	{ 0x4e71, 0x4e71, PDF_CMAP_SINGLE, 3930 },
	{ 0x4e73, 0x4e73, PDF_CMAP_SINGLE, 3285 },
	{ 0x4e7e, 0x4e80, PDF_CMAP_TABLE, 592 },
	{ 0x4e82, 0x4e82, PDF_CMAP_SINGLE, 4101 },
	{ 0x4e85, 0x4e86, PDF_CMAP_TABLE, 595 },
	{ 0x4e88, 0x4e8e, PDF_CMAP_TABLE, 597 },
	{ 0x4e91, 0x4e92, PDF_CMAP_TABLE, 604 },
	{ 0x4e94, 0x4e96, PDF_CMAP_TABLE, 606 },
	{ 0x4e98, 0x4e99, PDF_CMAP_TABLE, 609 },
	{ 0x4e9b, 0x4e9c, PDF_CMAP_TABLE, 611 },
	{ 0x4e9e, 0x4ea0, PDF_CMAP_RANGE, 4108 },
	{ 0x4ea1, 0x4ea2, PDF_CMAP_TABLE, 613 },
	{ 0x4ea4, 0x4ea6, PDF_CMAP_TABLE, 615 },
	{ 0x4ea8, 0x4ea8, PDF_CMAP_SINGLE, 1686 },
	{ 0x4eab, 0x4eac, PDF_CMAP_RANGE, 1687 },
	{ 0x4ead, 0x4eae, PDF_CMAP_TABLE, 618 },
	{ 0x4eb0, 0x4eb0, PDF_CMAP_SINGLE, 4112 },
	{ 0x4eb3, 0x4eb3, PDF_CMAP_SINGLE, 4113 },
	{ 0x4eb6, 0x4eb6, PDF_CMAP_SINGLE, 4114 },
	{ 0x4eb9, 0x4ebb, PDF_CMAP_TABLE, 620 },
	{ 0x4ec0, 0x4ec2, PDF_CMAP_TABLE, 623 },
	{ 0x4ec4, 0x4ec4, PDF_CMAP_SINGLE, 4117 },
	{ 0x4ec6, 0x4ec7, PDF_CMAP_TABLE, 626 },
	{ 0x4eca, 0x4ecb, PDF_CMAP_TABLE, 628 },
	{ 0x4ecd, 0x4ed0, PDF_CMAP_TABLE, 630 },
	{ 0x4ed4, 0x4ed9, PDF_CMAP_TABLE, 634 },
	{ 0x4edd, 0x4ee1, PDF_CMAP_TABLE, 640 },
	{ 0x4ee3, 0x4ee5, PDF_CMAP_TABLE, 645 },
	{ 0x4eed, 0x4eee, PDF_CMAP_TABLE, 648 },
	{ 0x4ef0, 0x4ef0, PDF_CMAP_SINGLE, 1724 },
	{ 0x4ef2, 0x4ef2, PDF_CMAP_SINGLE, 2981 },
	{ 0x4ef6, 0x4ef7, PDF_CMAP_TABLE, 650 },
	{ 0x4efb, 0x4efd, PDF_CMAP_TABLE, 652 },
	{ 0x4eff, 0x4f01, PDF_CMAP_TABLE, 655 },
	{ 0x4f03, 0x4f03, PDF_CMAP_SINGLE, 8375 },
	{ 0x4f09, 0x4f0b, PDF_CMAP_TABLE, 658 },
	{ 0x4f0d, 0x4f11, PDF_CMAP_TABLE, 661 },
	{ 0x4f15, 0x4f15, PDF_CMAP_SINGLE, 14315 },
	{ 0x4f1a, 0x4f1a, PDF_CMAP_SINGLE, 1393 },
	{ 0x4f1c, 0x4f1d, PDF_CMAP_TABLE, 666 },
	{ 0x4f2f, 0x4f30, PDF_CMAP_TABLE, 668 },
	{ 0x4f34, 0x4f34, PDF_CMAP_SINGLE, 3408 },
	{ 0x4f36, 0x4f36, PDF_CMAP_SINGLE, 4010 },
	{ 0x4f38, 0x4f3d, PDF_CMAP_TABLE, 670 },
	{ 0x4f43, 0x4f43, PDF_CMAP_SINGLE, 3053 },
	{ 0x4f46, 0x4f47, PDF_CMAP_TABLE, 676 },
	{ 0x4f49, 0x4f49, PDF_CMAP_SINGLE, 14318 },
	{ 0x4f4d, 0x4f51, PDF_CMAP_TABLE, 678 },
	{ 0x4f53, 0x4f57, PDF_CMAP_TABLE, 683 },
	{ 0x4f59, 0x4f5e, PDF_CMAP_TABLE, 688 },
	{ 0x4f60, 0x4f60, PDF_CMAP_SINGLE, 14316 },
	{ 0x4f69, 0x4f69, PDF_CMAP_SINGLE, 4137 },
	{ 0x4f6f, 0x4f70, PDF_CMAP_TABLE, 694 },
	{ 0x4f73, 0x4f73, PDF_CMAP_SINGLE, 1346 },
	{ 0x4f75, 0x4f76, PDF_CMAP_TABLE, 696 },
	{ 0x4f7a, 0x4f7c, PDF_CMAP_TABLE, 698 },
	{ 0x4f7d, 0x4f7e, PDF_CMAP_RANGE, 14321 },
	{ 0x4f7f, 0x4f7f, PDF_CMAP_SINGLE, 2198 },
	{ 0x4f83, 0x4f83, PDF_CMAP_SINGLE, 1506 },
	{ 0x4f86, 0x4f86, PDF_CMAP_SINGLE, 4141 },
	{ 0x4f88, 0x4f88, PDF_CMAP_SINGLE, 4133 },
	{ 0x4f8a, 0x4f8b, PDF_CMAP_TABLE, 701 },
	{ 0x4f8d, 0x4f8d, PDF_CMAP_SINGLE, 2246 },
	{ 0x4f8f, 0x4f8f, PDF_CMAP_SINGLE, 4134 },
	{ 0x4f91, 0x4f92, PDF_CMAP_TABLE, 703 },
	{ 0x4f94, 0x4f94, PDF_CMAP_SINGLE, 8381 },
	{ 0x4f96, 0x4f98, PDF_CMAP_TABLE, 705 },
	{ 0x4f9a, 0x4f9b, PDF_CMAP_TABLE, 708 },
	{ 0x4f9d, 0x4f9d, PDF_CMAP_SINGLE, 1169 },
	{ 0x4fa0, 0x4fa1, PDF_CMAP_TABLE, 710 },
	{ 0x4fab, 0x4fab, PDF_CMAP_SINGLE, 4564 },
	{ 0x4fad, 0x4faf, PDF_CMAP_TABLE, 712 },
	{ 0x4fb5, 0x4fb6, PDF_CMAP_TABLE, 715 },
	{ 0x4fbe, 0x4fbf, PDF_CMAP_TABLE, 717 },
	{ 0x4fc2, 0x4fc4, PDF_CMAP_TABLE, 719 },
	{ 0x4fc9, 0x4fca, PDF_CMAP_TABLE, 722 },
	{ 0x4fcd, 0x4fd1, PDF_CMAP_TABLE, 724 },
	{ 0x4fd3, 0x4fd4, PDF_CMAP_TABLE, 729 },
	{ 0x4fd7, 0x4fd8, PDF_CMAP_TABLE, 731 },
	{ 0x4fda, 0x4fdb, PDF_CMAP_TABLE, 733 },
	{ 0x4fdd, 0x4fdd, PDF_CMAP_SINGLE, 3629 },
	{ 0x4fdf, 0x4fe1, PDF_CMAP_TABLE, 735 },
	{ 0x4fe3, 0x4fe3, PDF_CMAP_SINGLE, 3745 },
	{ 0x4fe4, 0x4fe5, PDF_CMAP_RANGE, 4152 },
	{ 0x4fee, 0x4fef, PDF_CMAP_TABLE, 738 },
	{ 0x4ff3, 0x4ff3, PDF_CMAP_SINGLE, 3334 },
	{ 0x4ff5, 0x4ff6, PDF_CMAP_TABLE, 740 },
	{ 0x4ff8, 0x4ff8, PDF_CMAP_SINGLE, 3648 },
	{ 0x4ffa, 0x4ffa, PDF_CMAP_SINGLE, 1334 },
	{ 0x4ffd, 0x4fff, PDF_CMAP_TABLE, 742 },
	{ 0x5000, 0x5001, PDF_CMAP_RANGE, 14327 },
	{ 0x5002, 0x5002, PDF_CMAP_SINGLE, 13383 },
	{ 0x5005, 0x5006, PDF_CMAP_TABLE, 745 },
	{ 0x5009, 0x5009, PDF_CMAP_SINGLE, 2772 },
	{ 0x500b, 0x500b, PDF_CMAP_SINGLE, 1912 },
	{ 0x500d, 0x500d, PDF_CMAP_SINGLE, 3346 },
	{ 0x500f, 0x5012, PDF_CMAP_TABLE, 747 },
	{ 0x5014, 0x5014, PDF_CMAP_SINGLE, 4156 },
	{ 0x5016, 0x5016, PDF_CMAP_SINGLE, 1962 },
	{ 0x5019, 0x501b, PDF_CMAP_TABLE, 751 },
	{ 0x501e, 0x501f, PDF_CMAP_TABLE, 754 },
	{ 0x5021, 0x502e, PDF_CMAP_TABLE, 756 },
	{ 0x5036, 0x5036, PDF_CMAP_SINGLE, 1758 },
	{ 0x5039, 0x5039, PDF_CMAP_SINGLE, 1862 },
	{ 0x503b, 0x503b, PDF_CMAP_SINGLE, 14336 },
	{ 0x5040, 0x5040, PDF_CMAP_SINGLE, 8383 },
	{ 0x5042, 0x5043, PDF_CMAP_TABLE, 770 },
	{ 0x5046, 0x5049, PDF_CMAP_TABLE, 772 },
	{ 0x504f, 0x5050, PDF_CMAP_TABLE, 776 },
	{ 0x5055, 0x5057, PDF_CMAP_TABLE, 778 },
	{ 0x505a, 0x505a, PDF_CMAP_SINGLE, 4175 },
	{ 0x505c, 0x505c, PDF_CMAP_SINGLE, 3072 },
	{ 0x5065, 0x5066, PDF_CMAP_TABLE, 781 },
	{ 0x506a, 0x506a, PDF_CMAP_SINGLE, 14335 },
	{ 0x506c, 0x506c, PDF_CMAP_SINGLE, 4177 },
	{ 0x5070, 0x5070, PDF_CMAP_SINGLE, 8388 },
	{ 0x5072, 0x5072, PDF_CMAP_SINGLE, 2289 },
	{ 0x5074, 0x5076, PDF_CMAP_TABLE, 783 },
	{ 0x5078, 0x5078, PDF_CMAP_SINGLE, 4178 },
	{ 0x507d, 0x507d, PDF_CMAP_SINGLE, 1616 },
	{ 0x5080, 0x5080, PDF_CMAP_SINGLE, 4179 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 4181 },
	{ 0x508d, 0x508d, PDF_CMAP_SINGLE, 3683 },
	{ 0x508f, 0x508f, PDF_CMAP_SINGLE, 14337 },
	{ 0x5091, 0x5091, PDF_CMAP_SINGLE, 1852 },
	{ 0x5094, 0x5094, PDF_CMAP_SINGLE, 8390 },
	{ 0x5096, 0x5096, PDF_CMAP_SINGLE, 14338 },
	{ 0x5098, 0x509a, PDF_CMAP_TABLE, 786 },
	{ 0x509c, 0x509c, PDF_CMAP_SINGLE, 14339 },
	{ 0x50ac, 0x50ad, PDF_CMAP_TABLE, 789 },
	{ 0x50b2, 0x50b5, PDF_CMAP_TABLE, 791 },
	{ 0x50b7, 0x50b7, PDF_CMAP_SINGLE, 2439 },
	{ 0x50be, 0x50be, PDF_CMAP_SINGLE, 1807 },
	{ 0x50c2, 0x50c2, PDF_CMAP_SINGLE, 4187 },
	{ 0x50c5, 0x50c5, PDF_CMAP_SINGLE, 1735 },
	{ 0x50c9, 0x50ca, PDF_CMAP_RANGE, 4184 },
	{ 0x50cc, 0x50cd, PDF_CMAP_TABLE, 795 },
	{ 0x50cf, 0x50cf, PDF_CMAP_SINGLE, 2814 },
	{ 0x50d1, 0x50d1, PDF_CMAP_SINGLE, 1691 },
	{ 0x50d5, 0x50d6, PDF_CMAP_TABLE, 797 },
	{ 0x50d8, 0x50da, PDF_CMAP_TABLE, 799 },
	{ 0x50de, 0x50de, PDF_CMAP_SINGLE, 4189 },
	{ 0x50e3, 0x50e3, PDF_CMAP_SINGLE, 4192 },
	{ 0x50e5, 0x50e7, PDF_CMAP_TABLE, 802 },
	{ 0x50e9, 0x50e9, PDF_CMAP_SINGLE, 14342 },
	{ 0x50ed, 0x50f0, PDF_CMAP_TABLE, 805 },
	{ 0x50f4, 0x50f5, PDF_CMAP_TABLE, 809 },
	{ 0x50f9, 0x50f9, PDF_CMAP_SINGLE, 4194 },
	{ 0x50fb, 0x50fb, PDF_CMAP_SINGLE, 3608 },
	{ 0x5100, 0x5100, PDF_CMAP_SINGLE, 1617 },
	{ 0x5101, 0x5102, PDF_CMAP_RANGE, 4197 },
	{ 0x5104, 0x5104, PDF_CMAP_SINGLE, 1327 },
	{ 0x5108, 0x5109, PDF_CMAP_TABLE, 811 },
	{ 0x510b, 0x510b, PDF_CMAP_SINGLE, 14345 },
	{ 0x5110, 0x5110, PDF_CMAP_SINGLE, 14346 },
	{ 0x5112, 0x5112, PDF_CMAP_SINGLE, 2336 },
	{ 0x5114, 0x5116, PDF_CMAP_TABLE, 813 },
	{ 0x5118, 0x5118, PDF_CMAP_SINGLE, 4143 },
	{ 0x511a, 0x511b, PDF_CMAP_TABLE, 816 },
	{ 0x511e, 0x511f, PDF_CMAP_TABLE, 818 },
	{ 0x5121, 0x5121, PDF_CMAP_SINGLE, 4203 },
	{ 0x512a, 0x512a, PDF_CMAP_SINGLE, 3855 },
	{ 0x5132, 0x5132, PDF_CMAP_SINGLE, 3813 },
	{ 0x5137, 0x5137, PDF_CMAP_SINGLE, 4205 },
	{ 0x513a, 0x513c, PDF_CMAP_TABLE, 820 },
	{ 0x513f, 0x5140, PDF_CMAP_RANGE, 4208 },
	{ 0x5141, 0x5141, PDF_CMAP_SINGLE, 1208 },
	{ 0x5143, 0x514e, PDF_CMAP_TABLE, 823 },
	{ 0x5150, 0x5150, PDF_CMAP_SINGLE, 2247 },
	{ 0x5152, 0x5152, PDF_CMAP_SINGLE, 4210 },
	{ 0x5154, 0x5154, PDF_CMAP_SINGLE, 4212 },
	{ 0x515a, 0x515a, PDF_CMAP_SINGLE, 3160 },
	{ 0x515c, 0x515c, PDF_CMAP_SINGLE, 1491 },
	{ 0x515f, 0x515f, PDF_CMAP_SINGLE, 14349 },
	{ 0x5162, 0x5162, PDF_CMAP_SINGLE, 4213 },
	{ 0x5164, 0x5165, PDF_CMAP_TABLE, 835 },
	{ 0x5167, 0x5168, PDF_CMAP_TABLE, 837 },
	{ 0x5169, 0x516a, PDF_CMAP_RANGE, 4215 },
	{ 0x516b, 0x516e, PDF_CMAP_TABLE, 839 },
	{ 0x5171, 0x5171, PDF_CMAP_SINGLE, 1694 },
	{ 0x5175, 0x5179, PDF_CMAP_TABLE, 843 },
	{ 0x517c, 0x517c, PDF_CMAP_SINGLE, 1865 },
	{ 0x5180, 0x5180, PDF_CMAP_SINGLE, 4218 },
	{ 0x5182, 0x5182, PDF_CMAP_SINGLE, 4219 },
	{ 0x5185, 0x5186, PDF_CMAP_TABLE, 848 },
	{ 0x5189, 0x518a, PDF_CMAP_TABLE, 850 },
	{ 0x518c, 0x518d, PDF_CMAP_TABLE, 852 },
	{ 0x518f, 0x5193, PDF_CMAP_TABLE, 854 },
	{ 0x5195, 0x5196, PDF_CMAP_RANGE, 4226 },
	{ 0x5197, 0x5197, PDF_CMAP_SINGLE, 2513 },
	{ 0x5199, 0x5199, PDF_CMAP_SINGLE, 2296 },
	{ 0x519d, 0x519d, PDF_CMAP_SINGLE, 8395 },
	{ 0x51a0, 0x51a2, PDF_CMAP_TABLE, 859 },
	{ 0x51a4, 0x51a6, PDF_CMAP_TABLE, 862 },
	{ 0x51a8, 0x51a8, PDF_CMAP_SINGLE, 3532 },
	{ 0x51a9, 0x51ab, PDF_CMAP_RANGE, 4231 },
	{ 0x51ac, 0x51ac, PDF_CMAP_SINGLE, 3161 },
	{ 0x51b0, 0x51b0, PDF_CMAP_SINGLE, 4237 },
	{ 0x51b1, 0x51b2, PDF_CMAP_RANGE, 4235 },
	{ 0x51b3, 0x51b7, PDF_CMAP_TABLE, 865 },
	{ 0x51bc, 0x51be, PDF_CMAP_TABLE, 870 },
	{ 0x51c3, 0x51c6, PDF_CMAP_TABLE, 873 },
	{ 0x51c9, 0x51c9, PDF_CMAP_SINGLE, 4241 },
	{ 0x51cb, 0x51cd, PDF_CMAP_TABLE, 877 },
	{ 0x51d6, 0x51d6, PDF_CMAP_SINGLE, 4314 },
	{ 0x51db, 0x51de, PDF_CMAP_TABLE, 880 },
	{ 0x51e0, 0x51e1, PDF_CMAP_TABLE, 884 },
	{ 0x51e6, 0x51e7, PDF_CMAP_TABLE, 886 },
	{ 0x51e9, 0x51ea, PDF_CMAP_TABLE, 888 },
	{ 0x51ec, 0x51ee, PDF_CMAP_TABLE, 890 },
	{ 0x51f0, 0x51f1, PDF_CMAP_TABLE, 893 },
	{ 0x51f4, 0x51f6, PDF_CMAP_TABLE, 895 },
	{ 0x51f8, 0x51fa, PDF_CMAP_TABLE, 898 },
	{ 0x51fd, 0x51fe, PDF_CMAP_TABLE, 901 },
	{ 0x5200, 0x5200, PDF_CMAP_SINGLE, 3163 },
	{ 0x5201, 0x5202, PDF_CMAP_RANGE, 14355 },
	{ 0x5203, 0x5204, PDF_CMAP_TABLE, 903 },
	{ 0x5206, 0x5208, PDF_CMAP_TABLE, 905 },
	{ 0x520a, 0x520b, PDF_CMAP_TABLE, 908 },
	{ 0x520e, 0x520e, PDF_CMAP_SINGLE, 4253 },
	{ 0x5211, 0x5211, PDF_CMAP_SINGLE, 1808 },
	{ 0x5213, 0x5215, PDF_CMAP_TABLE, 910 },
	{ 0x5217, 0x5217, PDF_CMAP_SINGLE, 4027 },
	{ 0x521d, 0x521d, PDF_CMAP_SINGLE, 2419 },
	{ 0x5224, 0x5225, PDF_CMAP_TABLE, 913 },
	{ 0x5227, 0x5227, PDF_CMAP_SINGLE, 4254 },
	{ 0x5229, 0x522a, PDF_CMAP_TABLE, 915 },
	{ 0x522e, 0x522e, PDF_CMAP_SINGLE, 4256 },
	{ 0x5230, 0x5230, PDF_CMAP_SINGLE, 3192 },
	{ 0x5233, 0x5233, PDF_CMAP_SINGLE, 4257 },
	{ 0x5236, 0x523b, PDF_CMAP_TABLE, 917 },
	{ 0x5243, 0x5244, PDF_CMAP_TABLE, 923 },
	{ 0x5247, 0x5247, PDF_CMAP_SINGLE, 2823 },
	{ 0x5249, 0x524a, PDF_CMAP_TABLE, 925 },
	{ 0x524b, 0x524c, PDF_CMAP_RANGE, 4261 },
	{ 0x524d, 0x524d, PDF_CMAP_SINGLE, 2738 },
	{ 0x524f, 0x524f, PDF_CMAP_SINGLE, 4259 },
	{ 0x5254, 0x5254, PDF_CMAP_SINGLE, 4264 },
	{ 0x5256, 0x5256, PDF_CMAP_SINGLE, 3684 },
	{ 0x525b, 0x525b, PDF_CMAP_SINGLE, 2038 },
	{ 0x525d, 0x525e, PDF_CMAP_TABLE, 927 },
	{ 0x5261, 0x5261, PDF_CMAP_SINGLE, 14359 },
	{ 0x5263, 0x5266, PDF_CMAP_TABLE, 929 },
	{ 0x5269, 0x526a, PDF_CMAP_TABLE, 933 },
	{ 0x526f, 0x5275, PDF_CMAP_TABLE, 935 },
	{ 0x527d, 0x527d, PDF_CMAP_SINGLE, 4270 },
	{ 0x527f, 0x527f, PDF_CMAP_SINGLE, 4269 },
	{ 0x5283, 0x5283, PDF_CMAP_SINGLE, 1442 },
	{ 0x5287, 0x5289, PDF_CMAP_TABLE, 942 },
	{ 0x528d, 0x528d, PDF_CMAP_SINGLE, 4271 },
	{ 0x5291, 0x5294, PDF_CMAP_TABLE, 945 },
	{ 0x529b, 0x529c, PDF_CMAP_TABLE, 949 },
	{ 0x529f, 0x52a0, PDF_CMAP_TABLE, 951 },
	{ 0x52a3, 0x52a3, PDF_CMAP_SINGLE, 4028 },
	{ 0x52a6, 0x52a6, PDF_CMAP_SINGLE, 8400 },
	{ 0x52a9, 0x52ab, PDF_CMAP_TABLE, 953 },
	{ 0x52ac, 0x52ad, PDF_CMAP_RANGE, 4279 },
	{ 0x52af, 0x52af, PDF_CMAP_SINGLE, 8573 },
	{ 0x52b1, 0x52b1, PDF_CMAP_SINGLE, 4013 },
	{ 0x52b4, 0x52b5, PDF_CMAP_TABLE, 956 },
	{ 0x52b9, 0x52b9, PDF_CMAP_SINGLE, 1966 },
	{ 0x52bc, 0x52bc, PDF_CMAP_SINGLE, 4281 },
	{ 0x52be, 0x52be, PDF_CMAP_SINGLE, 1421 },
	{ 0x52c0, 0x52c1, PDF_CMAP_TABLE, 958 },
	{ 0x52c3, 0x52c3, PDF_CMAP_SINGLE, 3716 },
	{ 0x52c5, 0x52c5, PDF_CMAP_SINGLE, 3032 },
	{ 0x52c7, 0x52c9, PDF_CMAP_TABLE, 960 },
	{ 0x52cd, 0x52cd, PDF_CMAP_SINGLE, 4284 },
	{ 0x52d0, 0x52d0, PDF_CMAP_SINGLE, 14056 },
	{ 0x52d2, 0x52d2, PDF_CMAP_SINGLE, 7150 },
	{ 0x52d5, 0x52d5, PDF_CMAP_SINGLE, 3208 },
	{ 0x52d7, 0x52d9, PDF_CMAP_TABLE, 963 },
	{ 0x52db, 0x52db, PDF_CMAP_SINGLE, 8402 },
	{ 0x52dd, 0x52e0, PDF_CMAP_TABLE, 966 },
	{ 0x52e2, 0x52e4, PDF_CMAP_TABLE, 970 },
	{ 0x52e6, 0x52e7, PDF_CMAP_TABLE, 973 },
	{ 0x52f0, 0x52f0, PDF_CMAP_SINGLE, 14363 },
	{ 0x52f2, 0x52f3, PDF_CMAP_TABLE, 975 },
	{ 0x52f5, 0x52f5, PDF_CMAP_SINGLE, 4292 },
	{ 0x52f8, 0x52f9, PDF_CMAP_RANGE, 4293 },
	{ 0x52fa, 0x52fa, PDF_CMAP_SINGLE, 2311 },
	{ 0x52fe, 0x5302, PDF_CMAP_TABLE, 977 },
	{ 0x5305, 0x5308, PDF_CMAP_TABLE, 982 },
	{ 0x530a, 0x530b, PDF_CMAP_RANGE, 14364 },
	{ 0x530d, 0x530d, PDF_CMAP_SINGLE, 4298 },
	{ 0x530f, 0x5310, PDF_CMAP_TABLE, 986 },
	{ 0x5315, 0x5317, PDF_CMAP_TABLE, 988 },
	{ 0x5319, 0x531a, PDF_CMAP_TABLE, 991 },
	{ 0x531d, 0x531d, PDF_CMAP_SINGLE, 2779 },
	{ 0x5320, 0x5321, PDF_CMAP_TABLE, 993 },
	{ 0x5323, 0x5324, PDF_CMAP_TABLE, 995 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 3439 },
	{ 0x532f, 0x532f, PDF_CMAP_SINGLE, 4304 },
	{ 0x5331, 0x5331, PDF_CMAP_SINGLE, 4305 },
	{ 0x5333, 0x5333, PDF_CMAP_SINGLE, 4306 },
	{ 0x5338, 0x533b, PDF_CMAP_TABLE, 997 },
	{ 0x533e, 0x5341, PDF_CMAP_TABLE, 1001 },
	{ 0x5343, 0x5343, PDF_CMAP_SINGLE, 2701 },
	{ 0x5345, 0x534d, PDF_CMAP_TABLE, 1005 },
	{ 0x5351, 0x5354, PDF_CMAP_TABLE, 1014 },
	{ 0x5357, 0x5358, PDF_CMAP_TABLE, 1018 },
	{ 0x535a, 0x535a, PDF_CMAP_SINGLE, 3364 },
	{ 0x535c, 0x535c, PDF_CMAP_SINGLE, 3708 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 4315 },
	{ 0x5360, 0x5361, PDF_CMAP_TABLE, 1020 },
	{ 0x5366, 0x5366, PDF_CMAP_SINGLE, 1803 },
	{ 0x5369, 0x5369, PDF_CMAP_SINGLE, 4316 },
	{ 0x536c, 0x536c, PDF_CMAP_SINGLE, 14370 },
	{ 0x536e, 0x5375, PDF_CMAP_TABLE, 1022 },
	{ 0x5377, 0x5378, PDF_CMAP_TABLE, 1030 },
	{ 0x537b, 0x537b, PDF_CMAP_SINGLE, 4319 },
	{ 0x537d, 0x537d, PDF_CMAP_SINGLE, 13365 },
	{ 0x537f, 0x537f, PDF_CMAP_SINGLE, 1698 },
	{ 0x5382, 0x5382, PDF_CMAP_SINGLE, 4321 },
	{ 0x5384, 0x5384, PDF_CMAP_SINGLE, 3837 },
	{ 0x5389, 0x5389, PDF_CMAP_SINGLE, 14288 },
	{ 0x5393, 0x5393, PDF_CMAP_SINGLE, 8407 },
	{ 0x5396, 0x5396, PDF_CMAP_SINGLE, 4322 },
	{ 0x5398, 0x5398, PDF_CMAP_SINGLE, 3994 },
	{ 0x539a, 0x539a, PDF_CMAP_SINGLE, 1968 },
	{ 0x539f, 0x53a0, PDF_CMAP_TABLE, 1032 },
	{ 0x53a5, 0x53a6, PDF_CMAP_TABLE, 1034 },
	{ 0x53a8, 0x53a9, PDF_CMAP_TABLE, 1036 },
	{ 0x53ab, 0x53ab, PDF_CMAP_SINGLE, 14371 },
	{ 0x53ad, 0x53ae, PDF_CMAP_TABLE, 1038 },
	{ 0x53b0, 0x53b0, PDF_CMAP_SINGLE, 4327 },
	{ 0x53b2, 0x53b3, PDF_CMAP_TABLE, 1040 },
	{ 0x53b6, 0x53b6, PDF_CMAP_SINGLE, 4328 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 1672 },
	{ 0x53c2, 0x53c3, PDF_CMAP_TABLE, 1042 },
	{ 0x53c8, 0x53ce, PDF_CMAP_TABLE, 1044 },
	{ 0x53d4, 0x53d4, PDF_CMAP_SINGLE, 2385 },
	{ 0x53d6, 0x53d7, PDF_CMAP_TABLE, 1051 },
	{ 0x53d9, 0x53db, PDF_CMAP_TABLE, 1053 },
	{ 0x53dd, 0x53dd, PDF_CMAP_SINGLE, 8409 },
	{ 0x53df, 0x53df, PDF_CMAP_SINGLE, 4332 },
	{ 0x53e1, 0x53e6, PDF_CMAP_TABLE, 1056 },
	{ 0x53e8, 0x53f3, PDF_CMAP_TABLE, 1062 },
	{ 0x53f5, 0x53f8, PDF_CMAP_TABLE, 1074 },
	{ 0x53fa, 0x53fa, PDF_CMAP_SINGLE, 4338 },
	{ 0x5401, 0x5401, PDF_CMAP_SINGLE, 4339 },
	{ 0x5403, 0x5404, PDF_CMAP_TABLE, 1078 },
	{ 0x5408, 0x5411, PDF_CMAP_TABLE, 1080 },
	{ 0x541b, 0x541b, PDF_CMAP_SINGLE, 1797 },
	{ 0x541d, 0x541d, PDF_CMAP_SINGLE, 4348 },
	{ 0x541f, 0x5420, PDF_CMAP_TABLE, 1090 },
	{ 0x5426, 0x5427, PDF_CMAP_TABLE, 1092 },
	{ 0x5429, 0x5429, PDF_CMAP_SINGLE, 4347 },
	{ 0x542b, 0x542b, PDF_CMAP_SINGLE, 1562 },
	{ 0x542c, 0x542d, PDF_CMAP_RANGE, 4342 },
	{ 0x542e, 0x542e, PDF_CMAP_SINGLE, 4345 },
	{ 0x5433, 0x5433, PDF_CMAP_SINGLE, 13760 },
	{ 0x5436, 0x5436, PDF_CMAP_SINGLE, 4346 },
	{ 0x5438, 0x5439, PDF_CMAP_TABLE, 1094 },
	{ 0x543b, 0x5440, PDF_CMAP_TABLE, 1096 },
	{ 0x5442, 0x5442, PDF_CMAP_SINGLE, 4042 },
	{ 0x5446, 0x5446, PDF_CMAP_SINGLE, 3650 },
	{ 0x5448, 0x544a, PDF_CMAP_TABLE, 1102 },
	{ 0x544d, 0x544e, PDF_CMAP_TABLE, 1105 },
	{ 0x5451, 0x5451, PDF_CMAP_SINGLE, 3253 },
	{ 0x5455, 0x5455, PDF_CMAP_SINGLE, 14115 },
	{ 0x545f, 0x545f, PDF_CMAP_SINGLE, 4353 },
	{ 0x5466, 0x5466, PDF_CMAP_SINGLE, 14377 },
	{ 0x5468, 0x5468, PDF_CMAP_SINGLE, 2346 },
	{ 0x546a, 0x546b, PDF_CMAP_TABLE, 1107 },
	{ 0x5470, 0x5471, PDF_CMAP_TABLE, 1109 },
	{ 0x5473, 0x5477, PDF_CMAP_TABLE, 1111 },
	{ 0x547b, 0x547d, PDF_CMAP_TABLE, 1116 },
	{ 0x5480, 0x5480, PDF_CMAP_SINGLE, 4359 },
	{ 0x5484, 0x5484, PDF_CMAP_SINGLE, 4361 },
	{ 0x5486, 0x5486, PDF_CMAP_SINGLE, 4363 },
	{ 0x548a, 0x5490, PDF_CMAP_TABLE, 1119 },
	{ 0x5492, 0x5492, PDF_CMAP_SINGLE, 4357 },
	{ 0x5496, 0x5496, PDF_CMAP_SINGLE, 14381 },
	{ 0x549c, 0x549c, PDF_CMAP_SINGLE, 8411 },
	{ 0x54a1, 0x54a2, PDF_CMAP_TABLE, 1126 },
	{ 0x54a4, 0x54a5, PDF_CMAP_TABLE, 1128 },
	{ 0x54a8, 0x54a9, PDF_CMAP_TABLE, 1130 },
	{ 0x54ab, 0x54ad, PDF_CMAP_TABLE, 1132 },
	{ 0x54af, 0x54af, PDF_CMAP_SINGLE, 4401 },
	{ 0x54b2, 0x54b3, PDF_CMAP_TABLE, 1135 },
	{ 0x54b8, 0x54b9, PDF_CMAP_TABLE, 1137 },
	{ 0x54bc, 0x54c2, PDF_CMAP_TABLE, 1139 },
	{ 0x54c4, 0x54c4, PDF_CMAP_SINGLE, 4369 },
	{ 0x54c6, 0x54c9, PDF_CMAP_TABLE, 1146 },
	{ 0x54cd, 0x54cd, PDF_CMAP_SINGLE, 14387 },
	{ 0x54d8, 0x54d8, PDF_CMAP_SINGLE, 4377 },
	{ 0x54e1, 0x54e2, PDF_CMAP_TABLE, 1150 },
	{ 0x54e5, 0x54e6, PDF_CMAP_RANGE, 4378 },
	{ 0x54e8, 0x54e9, PDF_CMAP_TABLE, 1152 },
	{ 0x54ed, 0x54ee, PDF_CMAP_TABLE, 1154 },
	{ 0x54f2, 0x54f2, PDF_CMAP_SINGLE, 3113 },
	{ 0x54fa, 0x54fa, PDF_CMAP_SINGLE, 4385 },
	{ 0x54fd, 0x54fd, PDF_CMAP_SINGLE, 4382 },
	{ 0x54ff, 0x54ff, PDF_CMAP_SINGLE, 8414 },
	{ 0x5504, 0x5504, PDF_CMAP_SINGLE, 1238 },
	{ 0x5506, 0x5507, PDF_CMAP_TABLE, 1156 },
	{ 0x550e, 0x5510, PDF_CMAP_TABLE, 1158 },
	{ 0x5514, 0x5514, PDF_CMAP_SINGLE, 4381 },
	{ 0x5516, 0x5516, PDF_CMAP_SINGLE, 1126 },
	{ 0x552b, 0x552b, PDF_CMAP_SINGLE, 14389 },
	{ 0x552e, 0x552f, PDF_CMAP_TABLE, 1161 },
	{ 0x5531, 0x5531, PDF_CMAP_SINGLE, 2447 },
	{ 0x5533, 0x5533, PDF_CMAP_SINGLE, 4397 },
	{ 0x5535, 0x5535, PDF_CMAP_SINGLE, 14390 },
	{ 0x5538, 0x5539, PDF_CMAP_TABLE, 1163 },
	{ 0x553e, 0x553e, PDF_CMAP_SINGLE, 2851 },
	{ 0x5540, 0x5540, PDF_CMAP_SINGLE, 4388 },
	{ 0x5544, 0x5546, PDF_CMAP_TABLE, 1165 },
	{ 0x554a, 0x554a, PDF_CMAP_SINGLE, 14391 },
	{ 0x554c, 0x554c, PDF_CMAP_SINGLE, 4390 },
	{ 0x554f, 0x554f, PDF_CMAP_SINGLE, 3824 },
	{ 0x5553, 0x5553, PDF_CMAP_SINGLE, 1810 },
	{ 0x5556, 0x5557, PDF_CMAP_RANGE, 4394 },
	{ 0x555c, 0x555e, PDF_CMAP_TABLE, 1168 },
	{ 0x5560, 0x5561, PDF_CMAP_RANGE, 14392 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 4389 },
	{ 0x557b, 0x557c, PDF_CMAP_TABLE, 1171 },
	{ 0x557e, 0x557e, PDF_CMAP_SINGLE, 4405 },
	{ 0x5580, 0x5580, PDF_CMAP_SINGLE, 4400 },
	{ 0x5583, 0x5584, PDF_CMAP_TABLE, 1173 },
	{ 0x5586, 0x558b, PDF_CMAP_TABLE, 1175 },
	{ 0x558e, 0x558e, PDF_CMAP_SINGLE, 14395 },
	{ 0x5598, 0x559a, PDF_CMAP_TABLE, 1181 },
	{ 0x559c, 0x559f, PDF_CMAP_TABLE, 1184 },
	{ 0x55a7, 0x55ac, PDF_CMAP_TABLE, 1188 },
	{ 0x55ae, 0x55ae, PDF_CMAP_SINGLE, 4408 },
	{ 0x55b0, 0x55b0, PDF_CMAP_SINGLE, 1772 },
	{ 0x55b6, 0x55b6, PDF_CMAP_SINGLE, 1254 },
	{ 0x55c4, 0x55c5, PDF_CMAP_TABLE, 1194 },
	{ 0x55c7, 0x55c7, PDF_CMAP_SINGLE, 4472 },
	{ 0x55d4, 0x55d4, PDF_CMAP_SINGLE, 4420 },
	{ 0x55da, 0x55da, PDF_CMAP_SINGLE, 4414 },
	{ 0x55dc, 0x55dc, PDF_CMAP_SINGLE, 4418 },
	{ 0x55df, 0x55df, PDF_CMAP_SINGLE, 4416 },
	{ 0x55e3, 0x55e4, PDF_CMAP_TABLE, 1196 },
	{ 0x55f7, 0x55f7, PDF_CMAP_SINGLE, 4422 },
	{ 0x55f9, 0x55f9, PDF_CMAP_SINGLE, 4427 },
	{ 0x55fd, 0x55fe, PDF_CMAP_TABLE, 1198 },
	{ 0x5606, 0x5606, PDF_CMAP_SINGLE, 2928 },
	{ 0x5608, 0x5609, PDF_CMAP_TABLE, 1200 },
	{ 0x560e, 0x560f, PDF_CMAP_RANGE, 14397 },
	{ 0x5614, 0x5614, PDF_CMAP_SINGLE, 4421 },
	{ 0x5616, 0x5618, PDF_CMAP_TABLE, 1202 },
	{ 0x561b, 0x561b, PDF_CMAP_SINGLE, 4426 },
	{ 0x5620, 0x5620, PDF_CMAP_SINGLE, 15389 },
	{ 0x5629, 0x5629, PDF_CMAP_SINGLE, 1374 },
	{ 0x562f, 0x562f, PDF_CMAP_SINGLE, 4437 },
	{ 0x5631, 0x5632, PDF_CMAP_TABLE, 1205 },
	{ 0x5634, 0x5634, PDF_CMAP_SINGLE, 4431 },
	{ 0x5636, 0x5638, PDF_CMAP_TABLE, 1207 },
	{ 0x563f, 0x563f, PDF_CMAP_SINGLE, 14400 },
	{ 0x5642, 0x5642, PDF_CMAP_SINGLE, 1247 },
	{ 0x5649, 0x5649, PDF_CMAP_SINGLE, 14401 },
	{ 0x564b, 0x564c, PDF_CMAP_TABLE, 1210 },
	{ 0x564e, 0x5650, PDF_CMAP_TABLE, 1212 },
	{ 0x5653, 0x5653, PDF_CMAP_SINGLE, 7963 },
	{ 0x565b, 0x565b, PDF_CMAP_SINGLE, 1496 },
	{ 0x5664, 0x5664, PDF_CMAP_SINGLE, 4436 },
	{ 0x5666, 0x5666, PDF_CMAP_SINGLE, 14404 },
	{ 0x5668, 0x566c, PDF_CMAP_TABLE, 1215 },
	{ 0x566f, 0x566f, PDF_CMAP_SINGLE, 14406 },
	{ 0x5671, 0x5672, PDF_CMAP_RANGE, 14407 },
	{ 0x5674, 0x5674, PDF_CMAP_SINGLE, 3582 },
	{ 0x5676, 0x5676, PDF_CMAP_SINGLE, 15411 },
	{ 0x5678, 0x5678, PDF_CMAP_SINGLE, 3245 },
	{ 0x567a, 0x567a, PDF_CMAP_SINGLE, 3404 },
	{ 0x5680, 0x5680, PDF_CMAP_SINGLE, 4441 },
	{ 0x5686, 0x5687, PDF_CMAP_TABLE, 1220 },
	{ 0x568a, 0x568a, PDF_CMAP_SINGLE, 4442 },
	{ 0x568f, 0x568f, PDF_CMAP_SINGLE, 4445 },
	{ 0x5694, 0x5695, PDF_CMAP_TABLE, 1222 },
	{ 0x5699, 0x569a, PDF_CMAP_TABLE, 1224 },
	{ 0x56a0, 0x56a0, PDF_CMAP_SINGLE, 4443 },
	{ 0x56a2, 0x56a2, PDF_CMAP_SINGLE, 3311 },
	{ 0x56a5, 0x56a5, PDF_CMAP_SINGLE, 4446 },
	{ 0x56ac, 0x56ad, PDF_CMAP_RANGE, 14411 },
	{ 0x56ae, 0x56ae, PDF_CMAP_SINGLE, 4447 },
	{ 0x56b1, 0x56b1, PDF_CMAP_SINGLE, 14413 },
	{ 0x56b4, 0x56b4, PDF_CMAP_SINGLE, 4449 },
	{ 0x56b6, 0x56b6, PDF_CMAP_SINGLE, 4448 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 4451 },
	{ 0x56c0, 0x56c3, PDF_CMAP_TABLE, 1226 },
	{ 0x56c8, 0x56ca, PDF_CMAP_TABLE, 1230 },
	{ 0x56ce, 0x56ce, PDF_CMAP_SINGLE, 4456 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 4457 },
	{ 0x56d3, 0x56d3, PDF_CMAP_SINGLE, 4458 },
	{ 0x56d7, 0x56d8, PDF_CMAP_TABLE, 1233 },
	{ 0x56da, 0x56db, PDF_CMAP_TABLE, 1235 },
	{ 0x56dd, 0x56de, PDF_CMAP_TABLE, 1237 },
	{ 0x56e0, 0x56e0, PDF_CMAP_SINGLE, 1212 },
	{ 0x56e3, 0x56e4, PDF_CMAP_TABLE, 1239 },
	{ 0x56ee, 0x56ee, PDF_CMAP_SINGLE, 4460 },
	{ 0x56f0, 0x56f0, PDF_CMAP_SINGLE, 2068 },
	{ 0x56f2, 0x56f3, PDF_CMAP_TABLE, 1241 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 1243 },
	{ 0x56fd, 0x56fd, PDF_CMAP_SINGLE, 2051 },
	{ 0x56ff, 0x5700, PDF_CMAP_TABLE, 1245 },
	{ 0x5703, 0x5704, PDF_CMAP_TABLE, 1247 },
	{ 0x5708, 0x570b, PDF_CMAP_TABLE, 1249 },
	{ 0x570d, 0x570d, PDF_CMAP_SINGLE, 4468 },
	{ 0x570f, 0x570f, PDF_CMAP_SINGLE, 1869 },
	{ 0x5712, 0x5713, PDF_CMAP_TABLE, 1253 },
	{ 0x5715, 0x5716, PDF_CMAP_TABLE, 1255 },
	{ 0x5718, 0x5718, PDF_CMAP_SINGLE, 4470 },
	{ 0x571c, 0x571c, PDF_CMAP_SINGLE, 4473 },
	{ 0x571f, 0x571f, PDF_CMAP_SINGLE, 3156 },
	{ 0x5721, 0x5721, PDF_CMAP_SINGLE, 13952 },
	{ 0x5723, 0x5723, PDF_CMAP_SINGLE, 14419 },
	{ 0x5726, 0x5728, PDF_CMAP_TABLE, 1257 },
	{ 0x572d, 0x572d, PDF_CMAP_SINGLE, 1811 },
	{ 0x572f, 0x5730, PDF_CMAP_TABLE, 1260 },
	{ 0x5733, 0x5734, PDF_CMAP_RANGE, 14421 },
	{ 0x5737, 0x5738, PDF_CMAP_RANGE, 4475 },
	{ 0x573b, 0x573b, PDF_CMAP_SINGLE, 4478 },
	{ 0x5740, 0x5740, PDF_CMAP_SINGLE, 4479 },
	{ 0x5742, 0x5742, PDF_CMAP_SINGLE, 2132 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 1737 },
	{ 0x574a, 0x574a, PDF_CMAP_SINGLE, 3685 },
	{ 0x574c, 0x574c, PDF_CMAP_SINGLE, 14423 },
	{ 0x574e, 0x5751, PDF_CMAP_TABLE, 1262 },
	{ 0x5759, 0x5759, PDF_CMAP_SINGLE, 8416 },
	{ 0x5761, 0x5761, PDF_CMAP_SINGLE, 4484 },
	{ 0x5764, 0x5766, PDF_CMAP_TABLE, 1266 },
	{ 0x5769, 0x576a, PDF_CMAP_TABLE, 1269 },
	{ 0x5770, 0x5770, PDF_CMAP_SINGLE, 14424 },
	{ 0x577f, 0x577f, PDF_CMAP_SINGLE, 4485 },
	{ 0x5782, 0x5782, PDF_CMAP_SINGLE, 2600 },
	{ 0x5788, 0x5789, PDF_CMAP_TABLE, 1271 },
	{ 0x578b, 0x578c, PDF_CMAP_TABLE, 1273 },
	{ 0x5793, 0x5793, PDF_CMAP_SINGLE, 4487 },
	{ 0x579c, 0x579c, PDF_CMAP_SINGLE, 14426 },
	{ 0x57a0, 0x57a0, PDF_CMAP_SINGLE, 4488 },
	{ 0x57a2, 0x57a4, PDF_CMAP_TABLE, 1275 },
	{ 0x57aa, 0x57aa, PDF_CMAP_SINGLE, 4491 },
	{ 0x57ac, 0x57ac, PDF_CMAP_SINGLE, 8418 },
	{ 0x57b0, 0x57b0, PDF_CMAP_SINGLE, 4492 },
	{ 0x57b3, 0x57b3, PDF_CMAP_SINGLE, 4489 },
	{ 0x57b8, 0x57b8, PDF_CMAP_SINGLE, 14427 },
	{ 0x57c0, 0x57c0, PDF_CMAP_SINGLE, 4482 },
	{ 0x57c3, 0x57c3, PDF_CMAP_SINGLE, 4493 },
	{ 0x57c6, 0x57c8, PDF_CMAP_TABLE, 1278 },
	{ 0x57cb, 0x57cb, PDF_CMAP_SINGLE, 3730 },
	{ 0x57ce, 0x57ce, PDF_CMAP_SINGLE, 2515 },
	{ 0x57d2, 0x57d3, PDF_CMAP_RANGE, 4496 },
	{ 0x57d4, 0x57d4, PDF_CMAP_SINGLE, 4495 },
	{ 0x57d6, 0x57d6, PDF_CMAP_SINGLE, 4499 },
	{ 0x57dc, 0x57dc, PDF_CMAP_SINGLE, 3310 },
	{ 0x57df, 0x57e0, PDF_CMAP_TABLE, 1281 },
	{ 0x57e3, 0x57e3, PDF_CMAP_SINGLE, 4500 },
	{ 0x57e6, 0x57e6, PDF_CMAP_SINGLE, 14428 },
	{ 0x57ed, 0x57ed, PDF_CMAP_SINGLE, 14429 },
	{ 0x57f4, 0x57f4, PDF_CMAP_SINGLE, 2533 },
	{ 0x57f5, 0x57f6, PDF_CMAP_RANGE, 14430 },
	{ 0x57f7, 0x57f7, PDF_CMAP_SINGLE, 2277 },
	{ 0x57f9, 0x57fa, PDF_CMAP_TABLE, 1283 },
	{ 0x57fc, 0x57fc, PDF_CMAP_SINGLE, 2139 },
	{ 0x57ff, 0x5800, PDF_CMAP_TABLE, 1285 },
	{ 0x5802, 0x5802, PDF_CMAP_SINGLE, 3210 },
	{ 0x5805, 0x5806, PDF_CMAP_TABLE, 1287 },
	{ 0x5809, 0x580b, PDF_CMAP_TABLE, 1289 },
	{ 0x5815, 0x5815, PDF_CMAP_SINGLE, 2852 },
	{ 0x5819, 0x5819, PDF_CMAP_SINGLE, 4502 },
	{ 0x581d, 0x581d, PDF_CMAP_SINGLE, 4503 },
	{ 0x5820, 0x5821, PDF_CMAP_TABLE, 1292 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 3077 },
	{ 0x582a, 0x582a, PDF_CMAP_SINGLE, 1514 },
	{ 0x582f, 0x5832, PDF_CMAP_TABLE, 1294 },
	{ 0x5834, 0x5835, PDF_CMAP_TABLE, 1298 },
	{ 0x583a, 0x583a, PDF_CMAP_SINGLE, 2134 },
	{ 0x583d, 0x583d, PDF_CMAP_SINGLE, 4511 },
	{ 0x5840, 0x5841, PDF_CMAP_TABLE, 1300 },
	{ 0x584a, 0x584b, PDF_CMAP_TABLE, 1302 },
	{ 0x5851, 0x5852, PDF_CMAP_TABLE, 1304 },
	{ 0x5854, 0x5854, PDF_CMAP_SINGLE, 3165 },
	{ 0x5857, 0x585a, PDF_CMAP_TABLE, 1306 },
	{ 0x585e, 0x585e, PDF_CMAP_SINGLE, 2105 },
	{ 0x5861, 0x5862, PDF_CMAP_TABLE, 1310 },
	{ 0x5869, 0x5869, PDF_CMAP_SINGLE, 1304 },
	{ 0x586b, 0x586b, PDF_CMAP_SINGLE, 3120 },
	{ 0x5870, 0x5870, PDF_CMAP_SINGLE, 4508 },
	{ 0x5872, 0x5872, PDF_CMAP_SINGLE, 4504 },
	{ 0x5875, 0x5875, PDF_CMAP_SINGLE, 2582 },
	{ 0x5879, 0x5879, PDF_CMAP_SINGLE, 4512 },
	{ 0x587c, 0x587c, PDF_CMAP_SINGLE, 14436 },
	{ 0x587e, 0x587e, PDF_CMAP_SINGLE, 2392 },
	{ 0x5880, 0x5880, PDF_CMAP_SINGLE, 14437 },
	{ 0x5883, 0x5883, PDF_CMAP_SINGLE, 1701 },
	{ 0x5885, 0x5885, PDF_CMAP_SINGLE, 4513 },
	{ 0x5893, 0x5893, PDF_CMAP_SINGLE, 3640 },
	{ 0x5897, 0x5897, PDF_CMAP_SINGLE, 2815 },
	{ 0x589c, 0x589c, PDF_CMAP_SINGLE, 3042 },
	{ 0x589e, 0x589f, PDF_CMAP_TABLE, 1312 },
	{ 0x58a8, 0x58a9, PDF_CMAP_TABLE, 1314 },
	{ 0x58ab, 0x58ab, PDF_CMAP_SINGLE, 4516 },
	{ 0x58ae, 0x58ae, PDF_CMAP_SINGLE, 4521 },
	{ 0x58b2, 0x58b3, PDF_CMAP_TABLE, 1316 },
	{ 0x58b8, 0x58bb, PDF_CMAP_TABLE, 1318 },
	{ 0x58be, 0x58be, PDF_CMAP_SINGLE, 2070 },
	{ 0x58c1, 0x58c1, PDF_CMAP_SINGLE, 3609 },
	{ 0x58c5, 0x58c5, PDF_CMAP_SINGLE, 4522 },
	{ 0x58c7, 0x58c7, PDF_CMAP_SINGLE, 2947 },
	{ 0x58ca, 0x58ca, PDF_CMAP_SINGLE, 1397 },
	{ 0x58cc, 0x58cc, PDF_CMAP_SINGLE, 2517 },
	{ 0x58ce, 0x58ce, PDF_CMAP_SINGLE, 14439 },
	{ 0x58d0, 0x58d1, PDF_CMAP_TABLE, 1322 },
	{ 0x58d3, 0x58d5, PDF_CMAP_TABLE, 1324 },
	{ 0x58d7, 0x58da, PDF_CMAP_TABLE, 1327 },
	{ 0x58dc, 0x58dc, PDF_CMAP_SINGLE, 4529 },
	{ 0x58de, 0x58df, PDF_CMAP_TABLE, 1331 },
	{ 0x58e4, 0x58e5, PDF_CMAP_TABLE, 1333 },
	{ 0x58e9, 0x58e9, PDF_CMAP_SINGLE, 14443 },
	{ 0x58eb, 0x58ec, PDF_CMAP_TABLE, 1335 },
	{ 0x58ee, 0x58f2, PDF_CMAP_TABLE, 1337 },
	{ 0x58f7, 0x58f7, PDF_CMAP_SINGLE, 3063 },
	{ 0x58f9, 0x58fa, PDF_CMAP_TABLE, 1342 },
	{ 0x58fb, 0x58fd, PDF_CMAP_RANGE, 4535 },
	{ 0x5902, 0x5902, PDF_CMAP_SINGLE, 4538 },
	{ 0x5909, 0x590c, PDF_CMAP_TABLE, 1344 },
	{ 0x590f, 0x5910, PDF_CMAP_TABLE, 1348 },
	{ 0x5915, 0x5916, PDF_CMAP_TABLE, 1350 },
	{ 0x5918, 0x591c, PDF_CMAP_TABLE, 1352 },
	{ 0x5922, 0x5922, PDF_CMAP_SINGLE, 3776 },
	{ 0x5924, 0x5925, PDF_CMAP_TABLE, 1357 },
	{ 0x5927, 0x5927, PDF_CMAP_SINGLE, 2887 },
	{ 0x5929, 0x592b, PDF_CMAP_TABLE, 1359 },
	{ 0x592c, 0x592d, PDF_CMAP_RANGE, 4544 },
	{ 0x592e, 0x592f, PDF_CMAP_TABLE, 1362 },
	{ 0x5931, 0x5932, PDF_CMAP_TABLE, 1364 },
	{ 0x5937, 0x5939, PDF_CMAP_TABLE, 1366 },
	{ 0x593e, 0x593e, PDF_CMAP_SINGLE, 4548 },
	{ 0x5944, 0x5944, PDF_CMAP_SINGLE, 1284 },
	{ 0x5947, 0x5949, PDF_CMAP_TABLE, 1369 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 1372 },
	{ 0x5953, 0x5955, PDF_CMAP_TABLE, 1376 },
	{ 0x5957, 0x5958, PDF_CMAP_TABLE, 1379 },
	{ 0x595a, 0x595b, PDF_CMAP_TABLE, 1381 },
	{ 0x595d, 0x595d, PDF_CMAP_SINGLE, 8428 },
	{ 0x5960, 0x5963, PDF_CMAP_TABLE, 1383 },
	{ 0x5965, 0x5965, PDF_CMAP_SINGLE, 1310 },
	{ 0x5967, 0x596a, PDF_CMAP_TABLE, 1387 },
	{ 0x596c, 0x596e, PDF_CMAP_TABLE, 1391 },
	{ 0x5973, 0x5974, PDF_CMAP_TABLE, 1394 },
	{ 0x5978, 0x5978, PDF_CMAP_SINGLE, 4560 },
	{ 0x597d, 0x597d, PDF_CMAP_SINGLE, 1975 },
	{ 0x5981, 0x5984, PDF_CMAP_TABLE, 1396 },
	{ 0x598a, 0x598a, PDF_CMAP_SINGLE, 3291 },
	{ 0x598d, 0x598d, PDF_CMAP_SINGLE, 4570 },
	{ 0x5993, 0x5993, PDF_CMAP_SINGLE, 1618 },
	{ 0x5996, 0x5996, PDF_CMAP_SINGLE, 3887 },
	{ 0x5999, 0x5999, PDF_CMAP_SINGLE, 3771 },
	{ 0x599b, 0x599b, PDF_CMAP_SINGLE, 4665 },
	{ 0x599d, 0x599d, PDF_CMAP_SINGLE, 4562 },
	{ 0x59a3, 0x59a5, PDF_CMAP_TABLE, 1400 },
	{ 0x59a8, 0x59a8, PDF_CMAP_SINGLE, 3686 },
	{ 0x59ac, 0x59ac, PDF_CMAP_SINGLE, 3140 },
	{ 0x59b2, 0x59b2, PDF_CMAP_SINGLE, 4566 },
	{ 0x59b9, 0x59bb, PDF_CMAP_TABLE, 1403 },
	{ 0x59be, 0x59be, PDF_CMAP_SINGLE, 2450 },
	{ 0x59c6, 0x59c6, PDF_CMAP_SINGLE, 4567 },
	{ 0x59c9, 0x59cb, PDF_CMAP_TABLE, 1406 },
	{ 0x59d0, 0x59d4, PDF_CMAP_TABLE, 1409 },
	{ 0x59d9, 0x59da, PDF_CMAP_RANGE, 4571 },
	{ 0x59dc, 0x59dd, PDF_CMAP_TABLE, 1414 },
	{ 0x59e3, 0x59e4, PDF_CMAP_RANGE, 14452 },
	{ 0x59e5, 0x59e6, PDF_CMAP_TABLE, 1416 },
	{ 0x59e8, 0x59e8, PDF_CMAP_SINGLE, 4568 },
	{ 0x59ea, 0x59ec, PDF_CMAP_TABLE, 1418 },
	{ 0x59f6, 0x59f6, PDF_CMAP_SINGLE, 1132 },
	{ 0x59fb, 0x59fb, PDF_CMAP_SINGLE, 1213 },
	{ 0x59ff, 0x59ff, PDF_CMAP_SINGLE, 2207 },
	{ 0x5a01, 0x5a01, PDF_CMAP_SINGLE, 1174 },
	{ 0x5a03, 0x5a04, PDF_CMAP_TABLE, 1421 },
	{ 0x5a09, 0x5a09, PDF_CMAP_SINGLE, 4577 },
	{ 0x5a0c, 0x5a0c, PDF_CMAP_SINGLE, 14455 },
	{ 0x5a11, 0x5a11, PDF_CMAP_SINGLE, 4575 },
	{ 0x5a18, 0x5a18, PDF_CMAP_SINGLE, 3784 },
	{ 0x5a1a, 0x5a1c, PDF_CMAP_TABLE, 1423 },
	{ 0x5a1f, 0x5a20, PDF_CMAP_TABLE, 1426 },
	{ 0x5a23, 0x5a23, PDF_CMAP_SINGLE, 14456 },
	{ 0x5a25, 0x5a25, PDF_CMAP_SINGLE, 4573 },
	{ 0x5a29, 0x5a29, PDF_CMAP_SINGLE, 3626 },
	{ 0x5a2f, 0x5a2f, PDF_CMAP_SINGLE, 1944 },
	{ 0x5a35, 0x5a36, PDF_CMAP_RANGE, 4582 },
	{ 0x5a3c, 0x5a3c, PDF_CMAP_SINGLE, 2451 },
	{ 0x5a40, 0x5a41, PDF_CMAP_TABLE, 1428 },
	{ 0x5a46, 0x5a47, PDF_CMAP_TABLE, 1430 },
	{ 0x5a49, 0x5a49, PDF_CMAP_SINGLE, 4581 },
	{ 0x5a55, 0x5a55, PDF_CMAP_SINGLE, 14458 },
	{ 0x5a5a, 0x5a5a, PDF_CMAP_SINGLE, 2071 },
	{ 0x5a62, 0x5a63, PDF_CMAP_TABLE, 1432 },
	{ 0x5a66, 0x5a66, PDF_CMAP_SINGLE, 3530 },
	{ 0x5a6a, 0x5a6a, PDF_CMAP_SINGLE, 4585 },
	{ 0x5a6c, 0x5a6d, PDF_CMAP_TABLE, 1434 },
	{ 0x5a7e, 0x5a7f, PDF_CMAP_TABLE, 1436 },
	{ 0x5a92, 0x5a92, PDF_CMAP_SINGLE, 3348 },
	{ 0x5a9a, 0x5a9b, PDF_CMAP_TABLE, 1438 },
	{ 0x5a9e, 0x5a9e, PDF_CMAP_SINGLE, 14462 },
	{ 0x5aa7, 0x5aa7, PDF_CMAP_SINGLE, 14463 },
	{ 0x5aac, 0x5aac, PDF_CMAP_SINGLE, 14464 },
	{ 0x5ab3, 0x5ab3, PDF_CMAP_SINGLE, 14465 },
	{ 0x5abc, 0x5abe, PDF_CMAP_TABLE, 1440 },
	{ 0x5ac1, 0x5ac2, PDF_CMAP_TABLE, 1443 },
	{ 0x5ac9, 0x5ac9, PDF_CMAP_SINGLE, 2279 },
	{ 0x5acb, 0x5acc, PDF_CMAP_TABLE, 1445 },
	{ 0x5ad0, 0x5ad0, PDF_CMAP_SINGLE, 4603 },
	{ 0x5ad6, 0x5ad7, PDF_CMAP_TABLE, 1447 },
	{ 0x5ae0, 0x5ae1, PDF_CMAP_TABLE, 1449 },
	{ 0x5ae3, 0x5ae3, PDF_CMAP_SINGLE, 4592 },
	{ 0x5ae6, 0x5ae6, PDF_CMAP_SINGLE, 4594 },
	{ 0x5ae9, 0x5ae9, PDF_CMAP_SINGLE, 4595 },
	{ 0x5afa, 0x5afb, PDF_CMAP_RANGE, 4597 },
	{ 0x5b00, 0x5b00, PDF_CMAP_SINGLE, 14467 },
	{ 0x5b09, 0x5b09, PDF_CMAP_SINGLE, 1582 },
	{ 0x5b0b, 0x5b0c, PDF_CMAP_TABLE, 1451 },
	{ 0x5b16, 0x5b16, PDF_CMAP_SINGLE, 4601 },
	{ 0x5b19, 0x5b19, PDF_CMAP_SINGLE, 14468 },
	{ 0x5b22, 0x5b22, PDF_CMAP_SINGLE, 2518 },
	{ 0x5b25, 0x5b25, PDF_CMAP_SINGLE, 14469 },
	{ 0x5b2a, 0x5b2a, PDF_CMAP_SINGLE, 4604 },
	{ 0x5b2c, 0x5b2d, PDF_CMAP_TABLE, 1453 },
	{ 0x5b30, 0x5b30, PDF_CMAP_SINGLE, 1255 },
	{ 0x5b32, 0x5b32, PDF_CMAP_SINGLE, 4602 },
	{ 0x5b36, 0x5b36, PDF_CMAP_SINGLE, 4605 },
	{ 0x5b3e, 0x5b3e, PDF_CMAP_SINGLE, 4606 },
	{ 0x5b40, 0x5b41, PDF_CMAP_TABLE, 1455 },
	{ 0x5b43, 0x5b43, PDF_CMAP_SINGLE, 4607 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 4608 },
	{ 0x5b50, 0x5b51, PDF_CMAP_TABLE, 1457 },
	{ 0x5b54, 0x5b58, PDF_CMAP_TABLE, 1459 },
	{ 0x5b5a, 0x5b5b, PDF_CMAP_RANGE, 4612 },
	{ 0x5b5c, 0x5b5d, PDF_CMAP_TABLE, 1464 },
	{ 0x5b5f, 0x5b5f, PDF_CMAP_SINGLE, 3806 },
	{ 0x5b63, 0x5b66, PDF_CMAP_TABLE, 1466 },
	{ 0x5b69, 0x5b69, PDF_CMAP_SINGLE, 4615 },
	{ 0x5b6b, 0x5b6b, PDF_CMAP_SINGLE, 2841 },
	{ 0x5b70, 0x5b71, PDF_CMAP_TABLE, 1470 },
	{ 0x5b73, 0x5b73, PDF_CMAP_SINGLE, 4617 },
	{ 0x5b75, 0x5b75, PDF_CMAP_SINGLE, 4618 },
	{ 0x5b78, 0x5b78, PDF_CMAP_SINGLE, 4619 },
	{ 0x5b7a, 0x5b7a, PDF_CMAP_SINGLE, 4621 },
	{ 0x5b7c, 0x5b7c, PDF_CMAP_SINGLE, 14472 },
	{ 0x5b7e, 0x5b7f, PDF_CMAP_RANGE, 14473 },
	{ 0x5b80, 0x5b80, PDF_CMAP_SINGLE, 4622 },
	{ 0x5b82, 0x5b83, PDF_CMAP_TABLE, 1472 },
	{ 0x5b85, 0x5b85, PDF_CMAP_SINGLE, 2896 },
	{ 0x5b87, 0x5b8d, PDF_CMAP_TABLE, 1474 },
	{ 0x5b8f, 0x5b8f, PDF_CMAP_SINGLE, 1978 },
	{ 0x5b95, 0x5b95, PDF_CMAP_SINGLE, 3168 },
	{ 0x5b97, 0x5b9d, PDF_CMAP_TABLE, 1481 },
	{ 0x5b9f, 0x5b9f, PDF_CMAP_SINGLE, 2286 },
	{ 0x5ba2, 0x5ba6, PDF_CMAP_TABLE, 1488 },
	{ 0x5bae, 0x5bae, PDF_CMAP_SINGLE, 1654 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 2107 },
	{ 0x5bb3, 0x5bb6, PDF_CMAP_TABLE, 1493 },
	{ 0x5bb8, 0x5bb9, PDF_CMAP_TABLE, 1497 },
	{ 0x5bbf, 0x5bc0, PDF_CMAP_TABLE, 1499 },
	{ 0x5bc2, 0x5bc7, PDF_CMAP_TABLE, 1501 },
	{ 0x5bc9, 0x5bc9, PDF_CMAP_SINGLE, 4628 },
	{ 0x5bcc, 0x5bcc, PDF_CMAP_SINGLE, 3531 },
	{ 0x5bd0, 0x5bd0, PDF_CMAP_SINGLE, 4630 },
	{ 0x5bd2, 0x5bd4, PDF_CMAP_TABLE, 1507 },
	{ 0x5bd8, 0x5bd8, PDF_CMAP_SINGLE, 8435 },
	{ 0x5bdb, 0x5bdb, PDF_CMAP_SINGLE, 1518 },
	{ 0x5bdd, 0x5bdf, PDF_CMAP_TABLE, 1510 },
	{ 0x5be1, 0x5be2, PDF_CMAP_TABLE, 1513 },
	{ 0x5be4, 0x5be9, PDF_CMAP_TABLE, 1515 },
	{ 0x5beb, 0x5bec, PDF_CMAP_TABLE, 1521 },
	{ 0x5bee, 0x5bee, PDF_CMAP_SINGLE, 3976 },
	{ 0x5bf0, 0x5bf0, PDF_CMAP_SINGLE, 4637 },
	{ 0x5bf3, 0x5bf3, PDF_CMAP_SINGLE, 4639 },
	{ 0x5bf5, 0x5bf6, PDF_CMAP_TABLE, 1523 },
	{ 0x5bf8, 0x5bf8, PDF_CMAP_SINGLE, 2631 },
	{ 0x5bfa, 0x5bfa, PDF_CMAP_SINGLE, 2249 },
	{ 0x5bfe, 0x5bff, PDF_CMAP_TABLE, 1525 },
	{ 0x5c01, 0x5c02, PDF_CMAP_TABLE, 1527 },
	{ 0x5c04, 0x5c06, PDF_CMAP_TABLE, 1529 },
	{ 0x5c07, 0x5c08, PDF_CMAP_RANGE, 4641 },
	{ 0x5c09, 0x5c0b, PDF_CMAP_TABLE, 1532 },
	{ 0x5c0d, 0x5c0f, PDF_CMAP_TABLE, 1535 },
	{ 0x5c11, 0x5c11, PDF_CMAP_SINGLE, 2455 },
	{ 0x5c13, 0x5c14, PDF_CMAP_TABLE, 1538 },
	{ 0x5c16, 0x5c16, PDF_CMAP_SINGLE, 2705 },
	{ 0x5c19, 0x5c1a, PDF_CMAP_TABLE, 1540 },
	{ 0x5c1e, 0x5c1e, PDF_CMAP_SINGLE, 8437 },
	{ 0x5c20, 0x5c20, PDF_CMAP_SINGLE, 4645 },
	{ 0x5c22, 0x5c24, PDF_CMAP_TABLE, 1542 },
	{ 0x5c28, 0x5c28, PDF_CMAP_SINGLE, 4647 },
	{ 0x5c2b, 0x5c2b, PDF_CMAP_SINGLE, 14477 },
	{ 0x5c2d, 0x5c2d, PDF_CMAP_SINGLE, 1726 },
	{ 0x5c30, 0x5c31, PDF_CMAP_TABLE, 1545 },
	{ 0x5c38, 0x5c39, PDF_CMAP_RANGE, 4648 },
	{ 0x5c3a, 0x5c41, PDF_CMAP_TABLE, 1547 },
	{ 0x5c45, 0x5c46, PDF_CMAP_TABLE, 1555 },
	{ 0x5c48, 0x5c48, PDF_CMAP_SINGLE, 1782 },
	{ 0x5c4a, 0x5c4b, PDF_CMAP_TABLE, 1557 },
	{ 0x5c4d, 0x5c51, PDF_CMAP_TABLE, 1559 },
	{ 0x5c53, 0x5c53, PDF_CMAP_SINGLE, 4653 },
	{ 0x5c55, 0x5c55, PDF_CMAP_SINGLE, 3122 },
	{ 0x5c5b, 0x5c5b, PDF_CMAP_SINGLE, 7826 },
	{ 0x5c5e, 0x5c5e, PDF_CMAP_SINGLE, 2832 },
	{ 0x5c60, 0x5c65, PDF_CMAP_TABLE, 1564 },
	{ 0x5c69, 0x5c69, PDF_CMAP_SINGLE, 14480 },
	{ 0x5c6c, 0x5c6c, PDF_CMAP_SINGLE, 4657 },
	{ 0x5c6e, 0x5c6f, PDF_CMAP_TABLE, 1570 },
	{ 0x5c71, 0x5c71, PDF_CMAP_SINGLE, 2177 },
	{ 0x5c76, 0x5c76, PDF_CMAP_SINGLE, 4660 },
	{ 0x5c79, 0x5c79, PDF_CMAP_SINGLE, 4661 },
	{ 0x5c7c, 0x5c7c, PDF_CMAP_SINGLE, 14481 },
	{ 0x5c8c, 0x5c8c, PDF_CMAP_SINGLE, 4662 },
	{ 0x5c90, 0x5c91, PDF_CMAP_TABLE, 1572 },
	{ 0x5c94, 0x5c94, PDF_CMAP_SINGLE, 4664 },
	{ 0x5ca1, 0x5ca1, PDF_CMAP_SINGLE, 1324 },
	{ 0x5ca6, 0x5ca6, PDF_CMAP_SINGLE, 8438 },
	{ 0x5ca8, 0x5ca9, PDF_CMAP_TABLE, 1574 },
	{ 0x5cab, 0x5cac, PDF_CMAP_TABLE, 1576 },
	{ 0x5cb1, 0x5cb1, PDF_CMAP_SINGLE, 2866 },
	{ 0x5cb3, 0x5cb3, PDF_CMAP_SINGLE, 1463 },
	{ 0x5cb6, 0x5cb8, PDF_CMAP_TABLE, 1578 },
	{ 0x5cba, 0x5cbc, PDF_CMAP_TABLE, 1581 },
	{ 0x5cbe, 0x5cbe, PDF_CMAP_SINGLE, 4672 },
	{ 0x5cc5, 0x5cc5, PDF_CMAP_SINGLE, 4671 },
	{ 0x5cc7, 0x5cc7, PDF_CMAP_SINGLE, 4673 },
	{ 0x5ccb, 0x5ccb, PDF_CMAP_SINGLE, 14482 },
	{ 0x5cd2, 0x5cd2, PDF_CMAP_SINGLE, 14483 },
	{ 0x5cd9, 0x5cd9, PDF_CMAP_SINGLE, 4674 },
	{ 0x5ce0, 0x5ce1, PDF_CMAP_TABLE, 1584 },
	{ 0x5ce6, 0x5ce6, PDF_CMAP_SINGLE, 14124 },
	{ 0x5ce8, 0x5cea, PDF_CMAP_TABLE, 1586 },
	{ 0x5ced, 0x5ced, PDF_CMAP_SINGLE, 4678 },
	{ 0x5cef, 0x5cf0, PDF_CMAP_TABLE, 1589 },
	{ 0x5cf4, 0x5cf6, PDF_CMAP_TABLE, 1591 },
	{ 0x5cfa, 0x5cfb, PDF_CMAP_TABLE, 1594 },
	{ 0x5cfd, 0x5cfd, PDF_CMAP_SINGLE, 4676 },
	{ 0x5d07, 0x5d07, PDF_CMAP_SINGLE, 2616 },
	{ 0x5d0b, 0x5d0b, PDF_CMAP_SINGLE, 4681 },
	{ 0x5d0e, 0x5d0e, PDF_CMAP_SINGLE, 2138 },
	{ 0x5d11, 0x5d11, PDF_CMAP_SINGLE, 4687 },
	{ 0x5d14, 0x5d1b, PDF_CMAP_TABLE, 1596 },
	{ 0x5d1f, 0x5d1f, PDF_CMAP_SINGLE, 4685 },
	{ 0x5d22, 0x5d22, PDF_CMAP_SINGLE, 4689 },
	{ 0x5d24, 0x5d24, PDF_CMAP_SINGLE, 14485 },
	{ 0x5d26, 0x5d27, PDF_CMAP_TABLE, 1604 },
	{ 0x5d29, 0x5d29, PDF_CMAP_SINGLE, 3656 },
	{ 0x5d42, 0x5d43, PDF_CMAP_TABLE, 1606 },
	{ 0x5d46, 0x5d46, PDF_CMAP_SINGLE, 14488 },
	{ 0x5d4a, 0x5d4c, PDF_CMAP_TABLE, 1608 },
	{ 0x5d4e, 0x5d4e, PDF_CMAP_SINGLE, 4695 },
	{ 0x5d50, 0x5d50, PDF_CMAP_SINGLE, 3932 },
	{ 0x5d52, 0x5d53, PDF_CMAP_TABLE, 1611 },
	{ 0x5d5c, 0x5d5c, PDF_CMAP_SINGLE, 4684 },
	{ 0x5d69, 0x5d69, PDF_CMAP_SINGLE, 2617 },
	{ 0x5d6c, 0x5d6d, PDF_CMAP_TABLE, 1613 },
	{ 0x5d6f, 0x5d6f, PDF_CMAP_SINGLE, 2087 },
	{ 0x5d73, 0x5d73, PDF_CMAP_SINGLE, 4698 },
	{ 0x5d76, 0x5d76, PDF_CMAP_SINGLE, 4699 },
	{ 0x5d82, 0x5d82, PDF_CMAP_SINGLE, 4702 },
	{ 0x5d84, 0x5d84, PDF_CMAP_SINGLE, 4701 },
	{ 0x5d87, 0x5d87, PDF_CMAP_SINGLE, 4700 },
	{ 0x5d8b, 0x5d8c, PDF_CMAP_TABLE, 1615 },
	{ 0x5d90, 0x5d90, PDF_CMAP_SINGLE, 4708 },
	{ 0x5d92, 0x5d92, PDF_CMAP_SINGLE, 14490 },
	{ 0x5d94, 0x5d94, PDF_CMAP_SINGLE, 14491 },
	{ 0x5d99, 0x5d99, PDF_CMAP_SINGLE, 14492 },
	{ 0x5d9d, 0x5d9d, PDF_CMAP_SINGLE, 4704 },
	{ 0x5da0, 0x5da0, PDF_CMAP_SINGLE, 14493 },
	{ 0x5da2, 0x5da2, PDF_CMAP_SINGLE, 4703 },
	{ 0x5dac, 0x5dac, PDF_CMAP_SINGLE, 4705 },
	{ 0x5dae, 0x5dae, PDF_CMAP_SINGLE, 4706 },
	{ 0x5db2, 0x5db2, PDF_CMAP_SINGLE, 15269 },
	{ 0x5db7, 0x5db7, PDF_CMAP_SINGLE, 4709 },
	{ 0x5db8, 0x5db9, PDF_CMAP_RANGE, 8446 },
	{ 0x5dba, 0x5dba, PDF_CMAP_SINGLE, 4014 },
	{ 0x5dbc, 0x5dbd, PDF_CMAP_TABLE, 1617 },
	{ 0x5dc9, 0x5dc9, PDF_CMAP_SINGLE, 4711 },
	{ 0x5dcc, 0x5dcd, PDF_CMAP_TABLE, 1619 },
	{ 0x5dd0, 0x5dd0, PDF_CMAP_SINGLE, 8448 },
	{ 0x5dd2, 0x5dd3, PDF_CMAP_TABLE, 1621 },
	{ 0x5dd6, 0x5dd6, PDF_CMAP_SINGLE, 4715 },
	{ 0x5dd8, 0x5dd8, PDF_CMAP_SINGLE, 14494 },
	{ 0x5ddb, 0x5ddb, PDF_CMAP_SINGLE, 4716 },
	{ 0x5ddd, 0x5dde, PDF_CMAP_TABLE, 1623 },
	{ 0x5de0, 0x5de3, PDF_CMAP_TABLE, 1625 },
	{ 0x5de5, 0x5de8, PDF_CMAP_TABLE, 1629 },
	{ 0x5deb, 0x5deb, PDF_CMAP_SINGLE, 4717 },
	{ 0x5dee, 0x5dee, PDF_CMAP_SINGLE, 2089 },
	{ 0x5df1, 0x5df5, PDF_CMAP_TABLE, 1633 },
	{ 0x5df7, 0x5df8, PDF_CMAP_TABLE, 1638 },
	{ 0x5dfb, 0x5dfb, PDF_CMAP_SINGLE, 1512 },
	{ 0x5dfd, 0x5e00, PDF_CMAP_TABLE, 1640 },
	{ 0x5e02, 0x5e03, PDF_CMAP_TABLE, 1644 },
	{ 0x5e06, 0x5e06, PDF_CMAP_SINGLE, 3413 },
	{ 0x5e0b, 0x5e0c, PDF_CMAP_TABLE, 1646 },
	{ 0x5e11, 0x5e12, PDF_CMAP_TABLE, 1648 },
	{ 0x5e14, 0x5e15, PDF_CMAP_RANGE, 14499 },
	{ 0x5e16, 0x5e16, PDF_CMAP_SINGLE, 3005 },
	{ 0x5e18, 0x5e1b, PDF_CMAP_TABLE, 1650 },
	{ 0x5e1d, 0x5e1d, PDF_CMAP_SINGLE, 3079 },
	{ 0x5e25, 0x5e25, PDF_CMAP_SINGLE, 2601 },
	{ 0x5e2b, 0x5e2b, PDF_CMAP_SINGLE, 2211 },
	{ 0x5e2d, 0x5e30, PDF_CMAP_TABLE, 1654 },
	{ 0x5e33, 0x5e33, PDF_CMAP_SINGLE, 3006 },
	{ 0x5e36, 0x5e37, PDF_CMAP_RANGE, 4725 },
	{ 0x5e38, 0x5e38, PDF_CMAP_SINGLE, 2519 },
	{ 0x5e3d, 0x5e3d, PDF_CMAP_SINGLE, 3687 },
	{ 0x5e40, 0x5e40, PDF_CMAP_SINGLE, 4729 },
	{ 0x5e43, 0x5e45, PDF_CMAP_TABLE, 1658 },
	{ 0x5e47, 0x5e47, PDF_CMAP_SINGLE, 4736 },
	{ 0x5e4c, 0x5e4c, PDF_CMAP_SINGLE, 3720 },
	{ 0x5e4e, 0x5e4e, PDF_CMAP_SINGLE, 4730 },
	{ 0x5e54, 0x5e55, PDF_CMAP_TABLE, 1661 },
	{ 0x5e57, 0x5e58, PDF_CMAP_TABLE, 1663 },
	{ 0x5e5f, 0x5e5f, PDF_CMAP_SINGLE, 4733 },
	{ 0x5e61, 0x5e64, PDF_CMAP_TABLE, 1665 },
	{ 0x5e6b, 0x5e6c, PDF_CMAP_RANGE, 14504 },
	{ 0x5e72, 0x5e74, PDF_CMAP_TABLE, 1669 },
	{ 0x5e75, 0x5e76, PDF_CMAP_RANGE, 4737 },
	{ 0x5e78, 0x5e7f, PDF_CMAP_TABLE, 1672 },
	{ 0x5e81, 0x5e81, PDF_CMAP_SINGLE, 3007 },
	{ 0x5e83, 0x5e84, PDF_CMAP_TABLE, 1680 },
	{ 0x5e87, 0x5e87, PDF_CMAP_SINGLE, 3443 },
	{ 0x5e8a, 0x5e8a, PDF_CMAP_SINGLE, 2458 },
	{ 0x5e8f, 0x5e8f, PDF_CMAP_SINGLE, 2434 },
	{ 0x5e95, 0x5e97, PDF_CMAP_TABLE, 1682 },
	{ 0x5e99, 0x5e9a, PDF_CMAP_TABLE, 1685 },
	{ 0x5e9c, 0x5e9c, PDF_CMAP_SINGLE, 3534 },
	{ 0x5ea0, 0x5ea0, PDF_CMAP_SINGLE, 4742 },
	{ 0x5ea6, 0x5ea8, PDF_CMAP_TABLE, 1687 },
	{ 0x5eaa, 0x5eab, PDF_CMAP_TABLE, 1690 },
	{ 0x5ead, 0x5ead, PDF_CMAP_SINGLE, 3081 },
	{ 0x5eb5, 0x5eb8, PDF_CMAP_TABLE, 1692 },
	{ 0x5ebe, 0x5ebf, PDF_CMAP_RANGE, 14508 },
	{ 0x5ec1, 0x5ec2, PDF_CMAP_RANGE, 4743 },
	{ 0x5ec3, 0x5ec3, PDF_CMAP_SINGLE, 3335 },
	{ 0x5ec8, 0x5ecb, PDF_CMAP_TABLE, 1696 },
	{ 0x5ecf, 0x5ed0, PDF_CMAP_TABLE, 1700 },
	{ 0x5ed2, 0x5ed3, PDF_CMAP_TABLE, 1702 },
	{ 0x5ed6, 0x5ed6, PDF_CMAP_SINGLE, 4748 },
	{ 0x5eda, 0x5edb, PDF_CMAP_RANGE, 4751 },
	{ 0x5edd, 0x5edd, PDF_CMAP_SINGLE, 4750 },
	{ 0x5edf, 0x5ee3, PDF_CMAP_TABLE, 1704 },
	{ 0x5ee8, 0x5ee9, PDF_CMAP_RANGE, 4755 },
	{ 0x5eec, 0x5eec, PDF_CMAP_SINGLE, 4757 },
	{ 0x5ef0, 0x5ef1, PDF_CMAP_TABLE, 1709 },
	{ 0x5ef3, 0x5ef4, PDF_CMAP_TABLE, 1711 },
	{ 0x5ef6, 0x5ef8, PDF_CMAP_TABLE, 1713 },
	{ 0x5efa, 0x5efc, PDF_CMAP_TABLE, 1716 },
	{ 0x5efe, 0x5eff, PDF_CMAP_TABLE, 1719 },
	{ 0x5f01, 0x5f01, PDF_CMAP_SINGLE, 3627 },
	{ 0x5f03, 0x5f04, PDF_CMAP_TABLE, 1721 },
	{ 0x5f07, 0x5f07, PDF_CMAP_SINGLE, 14512 },
	{ 0x5f09, 0x5f11, PDF_CMAP_TABLE, 1723 },
	{ 0x5f13, 0x5f18, PDF_CMAP_TABLE, 1732 },
	{ 0x5f1b, 0x5f1b, PDF_CMAP_SINGLE, 2958 },
	{ 0x5f1c, 0x5f1d, PDF_CMAP_RANGE, 14514 },
	{ 0x5f1f, 0x5f1f, PDF_CMAP_SINGLE, 3083 },
	{ 0x5f21, 0x5f22, PDF_CMAP_TABLE, 1738 },
	{ 0x5f25, 0x5f29, PDF_CMAP_TABLE, 1740 },
	{ 0x5f2d, 0x5f2d, PDF_CMAP_SINGLE, 4772 },
	{ 0x5f2f, 0x5f2f, PDF_CMAP_SINGLE, 4778 },
	{ 0x5f31, 0x5f31, PDF_CMAP_SINGLE, 2321 },
	{ 0x5f34, 0x5f38, PDF_CMAP_TABLE, 1745 },
	{ 0x5f3a, 0x5f3c, PDF_CMAP_TABLE, 1750 },
	{ 0x5f3e, 0x5f3e, PDF_CMAP_SINGLE, 2948 },
	{ 0x5f40, 0x5f41, PDF_CMAP_TABLE, 1753 },
	{ 0x5f45, 0x5f45, PDF_CMAP_SINGLE, 8370 },
	{ 0x5f48, 0x5f48, PDF_CMAP_SINGLE, 4775 },
	{ 0x5f4a, 0x5f4a, PDF_CMAP_SINGLE, 1704 },
	{ 0x5f4c, 0x5f4c, PDF_CMAP_SINGLE, 4776 },
	{ 0x5f4e, 0x5f4e, PDF_CMAP_SINGLE, 4777 },
	{ 0x5f50, 0x5f51, PDF_CMAP_TABLE, 1755 },
	{ 0x5f53, 0x5f53, PDF_CMAP_SINGLE, 3184 },
	{ 0x5f56, 0x5f57, PDF_CMAP_RANGE, 4780 },
	{ 0x5f58, 0x5f59, PDF_CMAP_TABLE, 1757 },
	{ 0x5f5c, 0x5f5d, PDF_CMAP_TABLE, 1759 },
	{ 0x5f61, 0x5f62, PDF_CMAP_TABLE, 1761 },
	{ 0x5f64, 0x5f67, PDF_CMAP_TABLE, 1763 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 1767 },
	{ 0x5f70, 0x5f71, PDF_CMAP_TABLE, 1772 },
	{ 0x5f73, 0x5f73, PDF_CMAP_SINGLE, 4785 },
	{ 0x5f77, 0x5f77, PDF_CMAP_SINGLE, 4786 },
	{ 0x5f79, 0x5f79, PDF_CMAP_SINGLE, 3838 },
	{ 0x5f7c, 0x5f7c, PDF_CMAP_SINGLE, 3444 },
	{ 0x5f7f, 0x5f85, PDF_CMAP_TABLE, 1774 },
	{ 0x5f87, 0x5f8c, PDF_CMAP_TABLE, 1781 },
	{ 0x5f90, 0x5f93, PDF_CMAP_TABLE, 1787 },
	{ 0x5f97, 0x5f99, PDF_CMAP_TABLE, 1791 },
	{ 0x5f9c, 0x5f9c, PDF_CMAP_SINGLE, 14525 },
	{ 0x5f9e, 0x5f9e, PDF_CMAP_SINGLE, 4794 },
	{ 0x5fa0, 0x5fa1, PDF_CMAP_TABLE, 1794 },
	{ 0x5fa4, 0x5fa4, PDF_CMAP_SINGLE, 14527 },
	{ 0x5fa7, 0x5faa, PDF_CMAP_TABLE, 1796 },
	{ 0x5fad, 0x5faf, PDF_CMAP_TABLE, 1800 },
	{ 0x5fb3, 0x5fb5, PDF_CMAP_TABLE, 1803 },
	{ 0x5fb7, 0x5fb9, PDF_CMAP_TABLE, 1806 },
	{ 0x5fbc, 0x5fbd, PDF_CMAP_TABLE, 1809 },
	{ 0x5fc3, 0x5fc5, PDF_CMAP_TABLE, 1811 },
	{ 0x5fc9, 0x5fc9, PDF_CMAP_SINGLE, 14531 },
	{ 0x5fcc, 0x5fcd, PDF_CMAP_TABLE, 1814 },
	{ 0x5fd6, 0x5fd7, PDF_CMAP_TABLE, 1816 },
	{ 0x5fd8, 0x5fd9, PDF_CMAP_RANGE, 3688 },
	{ 0x5fdc, 0x5fde, PDF_CMAP_TABLE, 1818 },
	{ 0x5fe0, 0x5fe1, PDF_CMAP_TABLE, 1821 },
	{ 0x5fe4, 0x5fe4, PDF_CMAP_SINGLE, 4803 },
	{ 0x5fe9, 0x5fe9, PDF_CMAP_SINGLE, 14533 },
	{ 0x5feb, 0x5feb, PDF_CMAP_SINGLE, 1399 },
	{ 0x5fed, 0x5fed, PDF_CMAP_SINGLE, 14534 },
	{ 0x5ff0, 0x5ff1, PDF_CMAP_TABLE, 1823 },
	{ 0x5ff5, 0x5ff5, PDF_CMAP_SINGLE, 3302 },
	{ 0x5ff8, 0x5ff8, PDF_CMAP_SINGLE, 4804 },
	{ 0x5ffb, 0x5ffd, PDF_CMAP_TABLE, 1825 },
	{ 0x5fff, 0x5fff, PDF_CMAP_SINGLE, 4808 },
	{ 0x600e, 0x6010, PDF_CMAP_TABLE, 1828 },
	{ 0x6012, 0x6012, PDF_CMAP_SINGLE, 3158 },
	{ 0x6015, 0x6017, PDF_CMAP_TABLE, 1831 },
	{ 0x6019, 0x601d, PDF_CMAP_TABLE, 1834 },
	{ 0x6020, 0x6021, PDF_CMAP_TABLE, 1839 },
	{ 0x6025, 0x602b, PDF_CMAP_TABLE, 1841 },
	{ 0x602f, 0x602f, PDF_CMAP_SINGLE, 1705 },
	{ 0x6031, 0x6031, PDF_CMAP_SINGLE, 4815 },
	{ 0x6033, 0x6033, PDF_CMAP_SINGLE, 14538 },
	{ 0x603a, 0x603a, PDF_CMAP_SINGLE, 4821 },
	{ 0x6041, 0x6043, PDF_CMAP_TABLE, 1848 },
	{ 0x6046, 0x6046, PDF_CMAP_SINGLE, 4828 },
	{ 0x604a, 0x604b, PDF_CMAP_TABLE, 1851 },
	{ 0x604d, 0x604d, PDF_CMAP_SINGLE, 4829 },
	{ 0x6050, 0x6050, PDF_CMAP_SINGLE, 1706 },
	{ 0x6052, 0x6052, PDF_CMAP_SINGLE, 1987 },
	{ 0x6055, 0x6055, PDF_CMAP_SINGLE, 2436 },
	{ 0x6059, 0x605a, PDF_CMAP_TABLE, 1853 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 8454 },
	{ 0x605f, 0x6065, PDF_CMAP_TABLE, 1855 },
	{ 0x6068, 0x606d, PDF_CMAP_TABLE, 1862 },
	{ 0x606f, 0x6070, PDF_CMAP_TABLE, 1868 },
	{ 0x6075, 0x6075, PDF_CMAP_SINGLE, 1817 },
	{ 0x6077, 0x6077, PDF_CMAP_SINGLE, 4825 },
	{ 0x607f, 0x607f, PDF_CMAP_SINGLE, 14540 },
	{ 0x6081, 0x6081, PDF_CMAP_SINGLE, 4837 },
	{ 0x6083, 0x6085, PDF_CMAP_TABLE, 1870 },
	{ 0x6089, 0x608d, PDF_CMAP_TABLE, 1873 },
	{ 0x6092, 0x6092, PDF_CMAP_SINGLE, 4846 },
	{ 0x6094, 0x6094, PDF_CMAP_SINGLE, 1401 },
	{ 0x6096, 0x6097, PDF_CMAP_RANGE, 4844 },
	{ 0x609a, 0x609b, PDF_CMAP_TABLE, 1878 },
	{ 0x609e, 0x60a0, PDF_CMAP_TABLE, 1880 },
	{ 0x60a3, 0x60a4, PDF_CMAP_TABLE, 1883 },
	{ 0x60a6, 0x60a7, PDF_CMAP_TABLE, 1885 },
	{ 0x60a9, 0x60aa, PDF_CMAP_TABLE, 1887 },
	{ 0x60b0, 0x60b0, PDF_CMAP_SINGLE, 14543 },
	{ 0x60b2, 0x60b6, PDF_CMAP_TABLE, 1889 },
	{ 0x60b8, 0x60b8, PDF_CMAP_SINGLE, 4850 },
	{ 0x60bc, 0x60bd, PDF_CMAP_TABLE, 1894 },
	{ 0x60c5, 0x60c7, PDF_CMAP_TABLE, 1896 },
	{ 0x60cb, 0x60cb, PDF_CMAP_SINGLE, 14544 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 4077 },
	{ 0x60d3, 0x60d3, PDF_CMAP_SINGLE, 4852 },
	{ 0x60d5, 0x60d5, PDF_CMAP_SINGLE, 8458 },
	{ 0x60d8, 0x60d8, PDF_CMAP_SINGLE, 4858 },
	{ 0x60da, 0x60dc, PDF_CMAP_TABLE, 1899 },
	{ 0x60de, 0x60e1, PDF_CMAP_TABLE, 1902 },
	{ 0x60e3, 0x60e3, PDF_CMAP_SINGLE, 2780 },
	{ 0x60e7, 0x60e8, PDF_CMAP_TABLE, 1906 },
	{ 0x60f0, 0x60f4, PDF_CMAP_TABLE, 1908 },
	{ 0x60f6, 0x60f7, PDF_CMAP_RANGE, 4862 },
	{ 0x60f8, 0x60fb, PDF_CMAP_TABLE, 1913 },
	{ 0x6100, 0x6101, PDF_CMAP_TABLE, 1917 },
	{ 0x6103, 0x6103, PDF_CMAP_SINGLE, 4867 },
	{ 0x6106, 0x6106, PDF_CMAP_SINGLE, 4861 },
	{ 0x6108, 0x6109, PDF_CMAP_TABLE, 1919 },
	{ 0x610d, 0x610e, PDF_CMAP_RANGE, 4871 },
	{ 0x610f, 0x610f, PDF_CMAP_SINGLE, 1177 },
	{ 0x6111, 0x6111, PDF_CMAP_SINGLE, 8461 },
	{ 0x6112, 0x6114, PDF_CMAP_RANGE, 14547 },
	{ 0x6115, 0x6115, PDF_CMAP_SINGLE, 4860 },
	{ 0x611a, 0x611c, PDF_CMAP_TABLE, 1921 },
	{ 0x611f, 0x6121, PDF_CMAP_TABLE, 1924 },
	{ 0x6127, 0x6128, PDF_CMAP_TABLE, 1927 },
	{ 0x612c, 0x612c, PDF_CMAP_SINGLE, 4880 },
	{ 0x6130, 0x6130, PDF_CMAP_SINGLE, 8463 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 4881 },
	{ 0x6137, 0x6137, PDF_CMAP_SINGLE, 8462 },
	{ 0x613c, 0x613f, PDF_CMAP_TABLE, 1929 },
	{ 0x6142, 0x6142, PDF_CMAP_SINGLE, 4883 },
	{ 0x6144, 0x6144, PDF_CMAP_SINGLE, 4884 },
	{ 0x6147, 0x6148, PDF_CMAP_TABLE, 1933 },
	{ 0x614a, 0x614e, PDF_CMAP_TABLE, 1935 },
	{ 0x6153, 0x6153, PDF_CMAP_SINGLE, 4897 },
	{ 0x6155, 0x6155, PDF_CMAP_SINGLE, 3641 },
	{ 0x6158, 0x615a, PDF_CMAP_RANGE, 4887 },
	{ 0x615d, 0x615d, PDF_CMAP_SINGLE, 4896 },
	{ 0x615f, 0x615f, PDF_CMAP_SINGLE, 4895 },
	{ 0x6162, 0x6163, PDF_CMAP_TABLE, 1940 },
	{ 0x6165, 0x6165, PDF_CMAP_SINGLE, 4893 },
	{ 0x6167, 0x6168, PDF_CMAP_TABLE, 1942 },
	{ 0x616b, 0x616b, PDF_CMAP_SINGLE, 4890 },
	{ 0x616e, 0x6171, PDF_CMAP_TABLE, 1944 },
	{ 0x6173, 0x6177, PDF_CMAP_TABLE, 1948 },
	{ 0x617c, 0x617c, PDF_CMAP_SINGLE, 14551 },
	{ 0x617e, 0x617e, PDF_CMAP_SINGLE, 3911 },
	{ 0x6182, 0x6182, PDF_CMAP_SINGLE, 3861 },
	{ 0x6187, 0x6187, PDF_CMAP_SINGLE, 4901 },
	{ 0x618a, 0x618a, PDF_CMAP_SINGLE, 4905 },
	{ 0x618d, 0x618e, PDF_CMAP_TABLE, 1953 },
	{ 0x6190, 0x6191, PDF_CMAP_TABLE, 1955 },
	{ 0x6194, 0x6194, PDF_CMAP_SINGLE, 4903 },
	{ 0x6196, 0x6196, PDF_CMAP_SINGLE, 4900 },
	{ 0x6198, 0x619a, PDF_CMAP_TABLE, 1957 },
	{ 0x619f, 0x619f, PDF_CMAP_SINGLE, 14553 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 3584 },
	{ 0x61a7, 0x61a9, PDF_CMAP_TABLE, 1960 },
	{ 0x61ab, 0x61ac, PDF_CMAP_TABLE, 1963 },
	{ 0x61ae, 0x61ae, PDF_CMAP_SINGLE, 4908 },
	{ 0x61b2, 0x61b2, PDF_CMAP_SINGLE, 1873 },
	{ 0x61b6, 0x61b6, PDF_CMAP_SINGLE, 1329 },
	{ 0x61ba, 0x61ba, PDF_CMAP_SINGLE, 4916 },
	{ 0x61be, 0x61be, PDF_CMAP_SINGLE, 1524 },
	{ 0x61c2, 0x61c3, PDF_CMAP_TABLE, 1965 },
	{ 0x61c6, 0x61cd, PDF_CMAP_TABLE, 1967 },
	{ 0x61d0, 0x61d0, PDF_CMAP_SINGLE, 1403 },
	{ 0x61df, 0x61df, PDF_CMAP_SINGLE, 14556 },
	{ 0x61e3, 0x61e3, PDF_CMAP_SINGLE, 4921 },
	{ 0x61e6, 0x61e6, PDF_CMAP_SINGLE, 4920 },
	{ 0x61f2, 0x61f2, PDF_CMAP_SINGLE, 3012 },
	{ 0x61f4, 0x61f4, PDF_CMAP_SINGLE, 4924 },
	{ 0x61f6, 0x61f8, PDF_CMAP_TABLE, 1975 },
	{ 0x61fa, 0x61fa, PDF_CMAP_SINGLE, 4923 },
	{ 0x61fc, 0x6200, PDF_CMAP_TABLE, 1978 },
	{ 0x6208, 0x6209, PDF_CMAP_RANGE, 4930 },
	{ 0x620a, 0x620a, PDF_CMAP_SINGLE, 3642 },
	{ 0x620c, 0x620e, PDF_CMAP_TABLE, 1983 },
	{ 0x6210, 0x6216, PDF_CMAP_TABLE, 1986 },
	{ 0x621a, 0x621b, PDF_CMAP_TABLE, 1993 },
	{ 0x621d, 0x621f, PDF_CMAP_TABLE, 1995 },
	{ 0x6221, 0x6221, PDF_CMAP_SINGLE, 4937 },
	{ 0x6226, 0x6226, PDF_CMAP_SINGLE, 2707 },
	{ 0x6229, 0x622a, PDF_CMAP_TABLE, 1998 },
	{ 0x622e, 0x6230, PDF_CMAP_TABLE, 2000 },
	{ 0x6232, 0x6233, PDF_CMAP_RANGE, 4941 },
	{ 0x6234, 0x6234, PDF_CMAP_SINGLE, 2871 },
	{ 0x6236, 0x6236, PDF_CMAP_SINGLE, 13757 },
	{ 0x6238, 0x6238, PDF_CMAP_SINGLE, 1921 },
	{ 0x623b, 0x623b, PDF_CMAP_SINGLE, 3821 },
	{ 0x623e, 0x6241, PDF_CMAP_TABLE, 2003 },
	{ 0x6243, 0x6243, PDF_CMAP_SINGLE, 14559 },
	{ 0x6246, 0x6249, PDF_CMAP_TABLE, 2007 },
	{ 0x624b, 0x624e, PDF_CMAP_TABLE, 2011 },
	{ 0x6251, 0x6251, PDF_CMAP_SINGLE, 14562 },
	{ 0x6253, 0x6253, PDF_CMAP_SINGLE, 2855 },
	{ 0x6255, 0x6256, PDF_CMAP_TABLE, 2015 },
	{ 0x6258, 0x6258, PDF_CMAP_SINGLE, 2897 },
	{ 0x625b, 0x625b, PDF_CMAP_SINGLE, 4947 },
	{ 0x625e, 0x625e, PDF_CMAP_SINGLE, 4945 },
	{ 0x6260, 0x6260, PDF_CMAP_SINGLE, 4948 },
	{ 0x6263, 0x6263, PDF_CMAP_SINGLE, 4946 },
	{ 0x6268, 0x6268, PDF_CMAP_SINGLE, 4949 },
	{ 0x626e, 0x626e, PDF_CMAP_SINGLE, 3585 },
	{ 0x6271, 0x6271, PDF_CMAP_SINGLE, 1147 },
	{ 0x6276, 0x6276, PDF_CMAP_SINGLE, 3536 },
	{ 0x6279, 0x6279, PDF_CMAP_SINGLE, 3447 },
	{ 0x627c, 0x627c, PDF_CMAP_SINGLE, 4950 },
	{ 0x627e, 0x6280, PDF_CMAP_TABLE, 2017 },
	{ 0x6282, 0x6285, PDF_CMAP_TABLE, 2020 },
	{ 0x6289, 0x628a, PDF_CMAP_TABLE, 2024 },
	{ 0x6291, 0x6291, PDF_CMAP_SINGLE, 3912 },
	{ 0x6292, 0x6293, PDF_CMAP_RANGE, 4954 },
	{ 0x6294, 0x6298, PDF_CMAP_TABLE, 2026 },
	{ 0x629b, 0x629c, PDF_CMAP_TABLE, 2031 },
	{ 0x629e, 0x629e, PDF_CMAP_SINGLE, 2898 },
	{ 0x62a6, 0x62a6, PDF_CMAP_SINGLE, 8466 },
	{ 0x62ab, 0x62ac, PDF_CMAP_TABLE, 2033 },
	{ 0x62b1, 0x62b1, PDF_CMAP_SINGLE, 3658 },
	{ 0x62b5, 0x62b5, PDF_CMAP_SINGLE, 3085 },
	{ 0x62b9, 0x62b9, PDF_CMAP_SINGLE, 3747 },
	{ 0x62bb, 0x62bd, PDF_CMAP_TABLE, 2035 },
	{ 0x62c2, 0x62c2, PDF_CMAP_SINGLE, 4971 },
	{ 0x62c4, 0x62ca, PDF_CMAP_TABLE, 2038 },
	{ 0x62cc, 0x62cd, PDF_CMAP_TABLE, 2045 },
	{ 0x62cf, 0x62d4, PDF_CMAP_TABLE, 2047 },
	{ 0x62d7, 0x62d9, PDF_CMAP_TABLE, 2053 },
	{ 0x62db, 0x62dd, PDF_CMAP_TABLE, 2056 },
	{ 0x62e0, 0x62e1, PDF_CMAP_TABLE, 2059 },
	{ 0x62ec, 0x62ef, PDF_CMAP_TABLE, 2061 },
	{ 0x62f1, 0x62f1, PDF_CMAP_SINGLE, 4977 },
	{ 0x62f3, 0x62f3, PDF_CMAP_SINGLE, 1875 },
	{ 0x62f5, 0x62f7, PDF_CMAP_TABLE, 2065 },
	{ 0x62fc, 0x62fc, PDF_CMAP_SINGLE, 14565 },
	{ 0x62fe, 0x62ff, PDF_CMAP_TABLE, 2068 },
	{ 0x6301, 0x6302, PDF_CMAP_TABLE, 2070 },
	{ 0x6307, 0x630a, PDF_CMAP_TABLE, 2072 },
	{ 0x630c, 0x630d, PDF_CMAP_TABLE, 2076 },
	{ 0x6311, 0x6311, PDF_CMAP_SINGLE, 3013 },
	{ 0x6318, 0x6319, PDF_CMAP_TABLE, 2078 },
	{ 0x631b, 0x631b, PDF_CMAP_SINGLE, 14135 },
	{ 0x631f, 0x631f, PDF_CMAP_SINGLE, 1708 },
	{ 0x6327, 0x6328, PDF_CMAP_TABLE, 2080 },
	{ 0x632b, 0x632b, PDF_CMAP_SINGLE, 2099 },
	{ 0x632f, 0x632f, PDF_CMAP_SINGLE, 2556 },
	{ 0x6339, 0x633a, PDF_CMAP_TABLE, 2082 },
	{ 0x633d, 0x633f, PDF_CMAP_TABLE, 2084 },
	{ 0x6342, 0x6343, PDF_CMAP_RANGE, 14570 },
	{ 0x6349, 0x6349, PDF_CMAP_SINGLE, 2826 },
	{ 0x634c, 0x634d, PDF_CMAP_TABLE, 2087 },
	{ 0x634f, 0x6350, PDF_CMAP_TABLE, 2089 },
	{ 0x6355, 0x6355, PDF_CMAP_SINGLE, 3633 },
	{ 0x6357, 0x6357, PDF_CMAP_SINGLE, 3033 },
	{ 0x635c, 0x635c, PDF_CMAP_SINGLE, 2782 },
	{ 0x6365, 0x6365, PDF_CMAP_SINGLE, 14572 },
	{ 0x6367, 0x6369, PDF_CMAP_TABLE, 2091 },
	{ 0x636b, 0x636b, PDF_CMAP_SINGLE, 4998 },
	{ 0x636e, 0x636e, PDF_CMAP_SINGLE, 2622 },
	{ 0x6372, 0x6372, PDF_CMAP_SINGLE, 1876 },
	{ 0x6374, 0x6374, PDF_CMAP_SINGLE, 14573 },
	{ 0x6376, 0x6377, PDF_CMAP_TABLE, 2094 },
	{ 0x637a, 0x637b, PDF_CMAP_TABLE, 2096 },
	{ 0x637d, 0x637d, PDF_CMAP_SINGLE, 14574 },
	{ 0x6380, 0x6380, PDF_CMAP_SINGLE, 4990 },
	{ 0x6383, 0x6384, PDF_CMAP_TABLE, 2098 },
	{ 0x6387, 0x6389, PDF_CMAP_TABLE, 2100 },
	{ 0x638c, 0x638c, PDF_CMAP_SINGLE, 2464 },
	{ 0x638e, 0x6390, PDF_CMAP_TABLE, 2103 },
	{ 0x6392, 0x6392, PDF_CMAP_SINGLE, 3337 },
	{ 0x6396, 0x6396, PDF_CMAP_SINGLE, 4988 },
	{ 0x6398, 0x6398, PDF_CMAP_SINGLE, 1783 },
	{ 0x639b, 0x639b, PDF_CMAP_SINGLE, 1467 },
	{ 0x639e, 0x63a3, PDF_CMAP_TABLE, 2106 },
	{ 0x63a5, 0x63a5, PDF_CMAP_SINGLE, 2688 },
	{ 0x63a7, 0x63ac, PDF_CMAP_TABLE, 2112 },
	{ 0x63b2, 0x63b2, PDF_CMAP_SINGLE, 1821 },
	{ 0x63b4, 0x63b5, PDF_CMAP_TABLE, 2118 },
	{ 0x63bb, 0x63bb, PDF_CMAP_SINGLE, 2785 },
	{ 0x63be, 0x63be, PDF_CMAP_SINGLE, 5000 },
	{ 0x63c0, 0x63c0, PDF_CMAP_SINGLE, 5002 },
	{ 0x63c3, 0x63c4, PDF_CMAP_TABLE, 2120 },
	{ 0x63c6, 0x63c6, PDF_CMAP_SINGLE, 5003 },
	{ 0x63c9, 0x63c9, PDF_CMAP_SINGLE, 5005 },
	{ 0x63cf, 0x63d2, PDF_CMAP_TABLE, 2122 },
	{ 0x63d6, 0x63d6, PDF_CMAP_SINGLE, 3862 },
	{ 0x63da, 0x63dc, PDF_CMAP_TABLE, 2126 },
	{ 0x63e1, 0x63e1, PDF_CMAP_SINGLE, 1138 },
	{ 0x63e3, 0x63e3, PDF_CMAP_SINGLE, 5004 },
	{ 0x63e9, 0x63e9, PDF_CMAP_SINGLE, 5001 },
	{ 0x63ed, 0x63ee, PDF_CMAP_TABLE, 2129 },
	{ 0x63f4, 0x63f7, PDF_CMAP_TABLE, 2131 },
	{ 0x63fa, 0x63fa, PDF_CMAP_SINGLE, 3891 },
	{ 0x6406, 0x6406, PDF_CMAP_SINGLE, 5011 },
	{ 0x6409, 0x6409, PDF_CMAP_SINGLE, 14581 },
	{ 0x640d, 0x640d, PDF_CMAP_SINGLE, 2843 },
	{ 0x640f, 0x6410, PDF_CMAP_TABLE, 2135 },
	{ 0x6413, 0x6414, PDF_CMAP_TABLE, 2137 },
	{ 0x6416, 0x6417, PDF_CMAP_TABLE, 2139 },
	{ 0x641c, 0x641c, PDF_CMAP_SINGLE, 4986 },
	{ 0x6422, 0x6422, PDF_CMAP_SINGLE, 14583 },
	{ 0x6426, 0x6426, PDF_CMAP_SINGLE, 5013 },
	{ 0x6428, 0x6428, PDF_CMAP_SINGLE, 5017 },
	{ 0x642c, 0x642d, PDF_CMAP_TABLE, 2141 },
	{ 0x6434, 0x6434, PDF_CMAP_SINGLE, 5010 },
	{ 0x6436, 0x6436, PDF_CMAP_SINGLE, 5014 },
	{ 0x643a, 0x643a, PDF_CMAP_SINGLE, 1822 },
	{ 0x643e, 0x643e, PDF_CMAP_SINGLE, 2145 },
	{ 0x6442, 0x6442, PDF_CMAP_SINGLE, 2689 },
	{ 0x644e, 0x644e, PDF_CMAP_SINGLE, 5022 },
	{ 0x6451, 0x6451, PDF_CMAP_SINGLE, 7747 },
	{ 0x6454, 0x6454, PDF_CMAP_SINGLE, 14584 },
	{ 0x6458, 0x6458, PDF_CMAP_SINGLE, 3104 },
	{ 0x645b, 0x645b, PDF_CMAP_SINGLE, 14585 },
	{ 0x6460, 0x6460, PDF_CMAP_SINGLE, 8468 },
	{ 0x6467, 0x6467, PDF_CMAP_SINGLE, 5019 },
	{ 0x6469, 0x6469, PDF_CMAP_SINGLE, 3726 },
	{ 0x646d, 0x646d, PDF_CMAP_SINGLE, 14586 },
	{ 0x646f, 0x646f, PDF_CMAP_SINGLE, 5020 },
	{ 0x6476, 0x6476, PDF_CMAP_SINGLE, 5021 },
	{ 0x6478, 0x6478, PDF_CMAP_SINGLE, 3802 },
	{ 0x647a, 0x647b, PDF_CMAP_TABLE, 2143 },
	{ 0x6483, 0x6483, PDF_CMAP_SINGLE, 1848 },
	{ 0x6488, 0x6488, PDF_CMAP_SINGLE, 5028 },
	{ 0x6492, 0x6493, PDF_CMAP_TABLE, 2145 },
	{ 0x6495, 0x6495, PDF_CMAP_SINGLE, 5024 },
	{ 0x649a, 0x649a, PDF_CMAP_SINGLE, 3304 },
	{ 0x649d, 0x649e, PDF_CMAP_TABLE, 2147 },
	{ 0x64a4, 0x64a5, PDF_CMAP_TABLE, 2149 },
	{ 0x64a9, 0x64a9, PDF_CMAP_SINGLE, 5027 },
	{ 0x64ab, 0x64ab, PDF_CMAP_SINGLE, 3553 },
	{ 0x64ad, 0x64ae, PDF_CMAP_TABLE, 2151 },
	{ 0x64b0, 0x64b0, PDF_CMAP_SINGLE, 2709 },
	{ 0x64b2, 0x64b2, PDF_CMAP_SINGLE, 3710 },
	{ 0x64b9, 0x64b9, PDF_CMAP_SINGLE, 1447 },
	{ 0x64bb, 0x64bc, PDF_CMAP_TABLE, 2153 },
	{ 0x64be, 0x64bf, PDF_CMAP_RANGE, 14588 },
	{ 0x64c1, 0x64c2, PDF_CMAP_TABLE, 2155 },
	{ 0x64c5, 0x64c5, PDF_CMAP_SINGLE, 5032 },
	{ 0x64c7, 0x64c7, PDF_CMAP_SINGLE, 5033 },
	{ 0x64ca, 0x64ca, PDF_CMAP_SINGLE, 13341 },
	{ 0x64cd, 0x64ce, PDF_CMAP_TABLE, 2157 },
	{ 0x64d2, 0x64d2, PDF_CMAP_SINGLE, 5031 },
	{ 0x64d4, 0x64d4, PDF_CMAP_SINGLE, 4966 },
	{ 0x64d8, 0x64d8, PDF_CMAP_SINGLE, 5035 },
	{ 0x64da, 0x64da, PDF_CMAP_SINGLE, 5030 },
	{ 0x64e0, 0x64e1, PDF_CMAP_RANGE, 5040 },
	{ 0x64e2, 0x64e3, PDF_CMAP_TABLE, 2159 },
	{ 0x64e5, 0x64e7, PDF_CMAP_TABLE, 2161 },
	{ 0x64ec, 0x64ec, PDF_CMAP_SINGLE, 1622 },
	{ 0x64ef, 0x64ef, PDF_CMAP_SINGLE, 5044 },
	{ 0x64f1, 0x64f2, PDF_CMAP_TABLE, 2164 },
	{ 0x64f4, 0x64f4, PDF_CMAP_SINGLE, 5047 },
	{ 0x64f6, 0x64f7, PDF_CMAP_TABLE, 2166 },
	{ 0x64fa, 0x64fb, PDF_CMAP_TABLE, 2168 },
	{ 0x64fd, 0x64fe, PDF_CMAP_TABLE, 2170 },
	{ 0x6500, 0x6500, PDF_CMAP_SINGLE, 5050 },
	{ 0x6504, 0x6505, PDF_CMAP_TABLE, 2172 },
	{ 0x6516, 0x6516, PDF_CMAP_SINGLE, 14594 },
	{ 0x6518, 0x6519, PDF_CMAP_TABLE, 2174 },
	{ 0x651c, 0x651d, PDF_CMAP_TABLE, 2176 },
	{ 0x6522, 0x6524, PDF_CMAP_TABLE, 2178 },
	{ 0x652a, 0x652c, PDF_CMAP_TABLE, 2181 },
	{ 0x652f, 0x652f, PDF_CMAP_SINGLE, 2215 },
	{ 0x6534, 0x6535, PDF_CMAP_RANGE, 5058 },
	{ 0x6536, 0x6539, PDF_CMAP_TABLE, 2184 },
	{ 0x653b, 0x653b, PDF_CMAP_SINGLE, 1992 },
	{ 0x653e, 0x653f, PDF_CMAP_TABLE, 2188 },
	{ 0x6545, 0x6545, PDF_CMAP_SINGLE, 1922 },
	{ 0x6547, 0x6548, PDF_CMAP_TABLE, 2190 },
	{ 0x654d, 0x654f, PDF_CMAP_TABLE, 2192 },
	{ 0x6551, 0x6551, PDF_CMAP_SINGLE, 1657 },
	{ 0x6555, 0x6559, PDF_CMAP_TABLE, 2195 },
	{ 0x655d, 0x655e, PDF_CMAP_TABLE, 2200 },
	{ 0x6562, 0x6563, PDF_CMAP_TABLE, 2202 },
	{ 0x6566, 0x6567, PDF_CMAP_TABLE, 2204 },
	{ 0x656c, 0x656c, PDF_CMAP_SINGLE, 1823 },
	{ 0x6570, 0x6570, PDF_CMAP_SINGLE, 2618 },
	{ 0x6572, 0x6572, PDF_CMAP_SINGLE, 5071 },
	{ 0x6574, 0x6575, PDF_CMAP_TABLE, 2206 },
	{ 0x6577, 0x6578, PDF_CMAP_TABLE, 2208 },
	{ 0x6581, 0x6581, PDF_CMAP_SINGLE, 14598 },
	{ 0x6582, 0x6583, PDF_CMAP_RANGE, 5073 },
	{ 0x6585, 0x6585, PDF_CMAP_SINGLE, 14599 },
	{ 0x6587, 0x6589, PDF_CMAP_TABLE, 2210 },
	{ 0x658c, 0x658c, PDF_CMAP_SINGLE, 3518 },
	{ 0x658e, 0x658e, PDF_CMAP_SINGLE, 2120 },
	{ 0x6590, 0x6591, PDF_CMAP_TABLE, 2213 },
	{ 0x6597, 0x6597, PDF_CMAP_SINGLE, 3143 },
	{ 0x6599, 0x6599, PDF_CMAP_SINGLE, 3977 },
	{ 0x659b, 0x659c, PDF_CMAP_TABLE, 2215 },
	{ 0x659f, 0x659f, PDF_CMAP_SINGLE, 5077 },
	{ 0x65a1, 0x65a1, PDF_CMAP_SINGLE, 1146 },
	{ 0x65a4, 0x65a5, PDF_CMAP_TABLE, 2217 },
	{ 0x65a7, 0x65a7, PDF_CMAP_SINGLE, 3538 },
	{ 0x65ab, 0x65ad, PDF_CMAP_TABLE, 2219 },
	{ 0x65af, 0x65b0, PDF_CMAP_TABLE, 2222 },
	{ 0x65b7, 0x65b7, PDF_CMAP_SINGLE, 5079 },
	{ 0x65b9, 0x65b9, PDF_CMAP_SINGLE, 3661 },
	{ 0x65bc, 0x65bd, PDF_CMAP_TABLE, 2224 },
	{ 0x65c1, 0x65c6, PDF_CMAP_TABLE, 2226 },
	{ 0x65cb, 0x65cc, PDF_CMAP_TABLE, 2232 },
	{ 0x65cf, 0x65cf, PDF_CMAP_SINGLE, 2834 },
	{ 0x65d2, 0x65d2, PDF_CMAP_SINGLE, 5085 },
	{ 0x65d7, 0x65d7, PDF_CMAP_SINGLE, 1590 },
	{ 0x65d9, 0x65d9, PDF_CMAP_SINGLE, 5087 },
	{ 0x65db, 0x65db, PDF_CMAP_SINGLE, 5086 },
	{ 0x65e0, 0x65e1, PDF_CMAP_RANGE, 5088 },
	{ 0x65e2, 0x65e3, PDF_CMAP_TABLE, 2234 },
	{ 0x65e5, 0x65e9, PDF_CMAP_TABLE, 2236 },
	{ 0x65ec, 0x65ed, PDF_CMAP_TABLE, 2241 },
	{ 0x65f0, 0x65f2, PDF_CMAP_TABLE, 2243 },
	{ 0x65fa, 0x65fb, PDF_CMAP_TABLE, 2246 },
	{ 0x6600, 0x6600, PDF_CMAP_SINGLE, 8472 },
	{ 0x6602, 0x6603, PDF_CMAP_TABLE, 2248 },
	{ 0x6606, 0x6607, PDF_CMAP_TABLE, 2250 },
	{ 0x6609, 0x660a, PDF_CMAP_TABLE, 2252 },
	{ 0x660c, 0x660c, PDF_CMAP_SINGLE, 2467 },
	{ 0x660e, 0x660f, PDF_CMAP_TABLE, 2254 },
	{ 0x6613, 0x6615, PDF_CMAP_TABLE, 2256 },
	{ 0x661c, 0x661c, PDF_CMAP_SINGLE, 5099 },
	{ 0x661e, 0x6620, PDF_CMAP_TABLE, 2259 },
	{ 0x6624, 0x6625, PDF_CMAP_TABLE, 2262 },
	{ 0x6627, 0x6628, PDF_CMAP_TABLE, 2264 },
	{ 0x662c, 0x662f, PDF_CMAP_TABLE, 2266 },
	{ 0x6631, 0x6631, PDF_CMAP_SINGLE, 8366 },
	{ 0x6634, 0x6634, PDF_CMAP_SINGLE, 5098 },
	{ 0x6635, 0x6636, PDF_CMAP_RANGE, 5096 },
	{ 0x663b, 0x663c, PDF_CMAP_TABLE, 2270 },
	{ 0x663f, 0x663f, PDF_CMAP_SINGLE, 5129 },
	{ 0x6641, 0x6644, PDF_CMAP_TABLE, 2272 },
	{ 0x6649, 0x6649, PDF_CMAP_SINGLE, 5102 },
	{ 0x664b, 0x664c, PDF_CMAP_TABLE, 2276 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 5100 },
	{ 0x6652, 0x6652, PDF_CMAP_SINGLE, 2173 },
	{ 0x6657, 0x6657, PDF_CMAP_SINGLE, 8479 },
	{ 0x6659, 0x6659, PDF_CMAP_SINGLE, 8480 },
	{ 0x665b, 0x665c, PDF_CMAP_RANGE, 14605 },
	{ 0x665d, 0x665f, PDF_CMAP_TABLE, 2278 },
	{ 0x6661, 0x6666, PDF_CMAP_TABLE, 2281 },
	{ 0x6667, 0x6668, PDF_CMAP_RANGE, 5107 },
	{ 0x6669, 0x6669, PDF_CMAP_SINGLE, 3433 },
	{ 0x666b, 0x666b, PDF_CMAP_SINGLE, 14608 },
	{ 0x666e, 0x6670, PDF_CMAP_TABLE, 2287 },
	{ 0x6673, 0x6674, PDF_CMAP_TABLE, 2290 },
	{ 0x6676, 0x6677, PDF_CMAP_TABLE, 2292 },
	{ 0x667a, 0x667a, PDF_CMAP_SINGLE, 2960 },
	{ 0x6681, 0x6681, PDF_CMAP_SINGLE, 1727 },
	{ 0x6683, 0x6684, PDF_CMAP_TABLE, 2294 },
	{ 0x6687, 0x6689, PDF_CMAP_TABLE, 2296 },
	{ 0x668e, 0x668e, PDF_CMAP_SINGLE, 5114 },
	{ 0x6691, 0x6691, PDF_CMAP_SINGLE, 2421 },
	{ 0x6696, 0x6699, PDF_CMAP_TABLE, 2299 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 5118 },
	{ 0x66a0, 0x66a0, PDF_CMAP_SINGLE, 8484 },
	{ 0x66a2, 0x66a2, PDF_CMAP_SINGLE, 3014 },
	{ 0x66a4, 0x66a4, PDF_CMAP_SINGLE, 14610 },
	{ 0x66a6, 0x66a6, PDF_CMAP_SINGLE, 4025 },
	{ 0x66ab, 0x66ab, PDF_CMAP_SINGLE, 2193 },
	{ 0x66ae, 0x66ae, PDF_CMAP_SINGLE, 3643 },
	{ 0x66b2, 0x66b2, PDF_CMAP_SINGLE, 8485 },
	{ 0x66b4, 0x66b4, PDF_CMAP_SINGLE, 3691 },
	{ 0x66b8, 0x66b9, PDF_CMAP_TABLE, 2303 },
	{ 0x66bc, 0x66bc, PDF_CMAP_SINGLE, 5123 },
	{ 0x66be, 0x66bf, PDF_CMAP_TABLE, 2305 },
	{ 0x66c1, 0x66c1, PDF_CMAP_SINGLE, 5119 },
	{ 0x66c4, 0x66c4, PDF_CMAP_SINGLE, 5124 },
	{ 0x66c6, 0x66c9, PDF_CMAP_TABLE, 2307 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 5126 },
	{ 0x66d9, 0x66da, PDF_CMAP_TABLE, 2311 },
	{ 0x66dc, 0x66dd, PDF_CMAP_TABLE, 2313 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 5128 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 5130 },
	{ 0x66e9, 0x66e9, PDF_CMAP_SINGLE, 5131 },
	{ 0x66ec, 0x66ec, PDF_CMAP_SINGLE, 14612 },
	{ 0x66f0, 0x66f0, PDF_CMAP_SINGLE, 5132 },
	{ 0x66f2, 0x66f5, PDF_CMAP_TABLE, 2315 },
	{ 0x66f7, 0x6700, PDF_CMAP_TABLE, 2319 },
	{ 0x6703, 0x6703, PDF_CMAP_SINGLE, 4171 },
	{ 0x6705, 0x6705, PDF_CMAP_SINGLE, 14613 },
	{ 0x6708, 0x6709, PDF_CMAP_TABLE, 2329 },
	{ 0x670b, 0x670b, PDF_CMAP_SINGLE, 3662 },
	{ 0x670d, 0x670f, PDF_CMAP_TABLE, 2331 },
	{ 0x6713, 0x6717, PDF_CMAP_TABLE, 2334 },
	{ 0x671b, 0x671b, PDF_CMAP_SINGLE, 3692 },
	{ 0x671d, 0x671f, PDF_CMAP_TABLE, 2339 },
	{ 0x6726, 0x6727, PDF_CMAP_RANGE, 5138 },
	{ 0x6728, 0x6728, PDF_CMAP_SINGLE, 3814 },
	{ 0x672a, 0x672e, PDF_CMAP_TABLE, 2342 },
	{ 0x6731, 0x6731, PDF_CMAP_SINGLE, 2327 },
	{ 0x6733, 0x6734, PDF_CMAP_TABLE, 2347 },
	{ 0x6736, 0x6738, PDF_CMAP_TABLE, 2349 },
	{ 0x673a, 0x673a, PDF_CMAP_SINGLE, 1589 },
	{ 0x673d, 0x673d, PDF_CMAP_SINGLE, 1658 },
	{ 0x673f, 0x673f, PDF_CMAP_SINGLE, 5142 },
	{ 0x6741, 0x6741, PDF_CMAP_SINGLE, 5144 },
	{ 0x6743, 0x6743, PDF_CMAP_SINGLE, 13751 },
	{ 0x6746, 0x6746, PDF_CMAP_SINGLE, 5147 },
	{ 0x6748, 0x6749, PDF_CMAP_TABLE, 2352 },
	{ 0x674c, 0x674c, PDF_CMAP_SINGLE, 14617 },
	{ 0x674e, 0x6751, PDF_CMAP_TABLE, 2354 },
	{ 0x6753, 0x6753, PDF_CMAP_SINGLE, 2313 },
	{ 0x6756, 0x6756, PDF_CMAP_SINGLE, 2523 },
	{ 0x6759, 0x6759, PDF_CMAP_SINGLE, 5150 },
	{ 0x675c, 0x675c, PDF_CMAP_SINGLE, 3144 },
	{ 0x675e, 0x6762, PDF_CMAP_TABLE, 2358 },
	{ 0x6763, 0x6764, PDF_CMAP_RANGE, 5151 },
	{ 0x6765, 0x6766, PDF_CMAP_TABLE, 2363 },
	{ 0x676a, 0x676a, PDF_CMAP_SINGLE, 5157 },
	{ 0x676d, 0x6773, PDF_CMAP_TABLE, 2365 },
	{ 0x6775, 0x6777, PDF_CMAP_TABLE, 2372 },
	{ 0x677b, 0x677c, PDF_CMAP_TABLE, 2375 },
	{ 0x677e, 0x677f, PDF_CMAP_TABLE, 2377 },
	{ 0x6785, 0x6785, PDF_CMAP_SINGLE, 5162 },
	{ 0x6787, 0x6787, PDF_CMAP_SINGLE, 3470 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 5153 },
	{ 0x678b, 0x678c, PDF_CMAP_TABLE, 2379 },
	{ 0x6790, 0x6790, PDF_CMAP_SINGLE, 2675 },
	{ 0x6795, 0x6795, PDF_CMAP_SINGLE, 3739 },
	{ 0x6797, 0x6797, PDF_CMAP_SINGLE, 3995 },
	{ 0x679a, 0x679a, PDF_CMAP_SINGLE, 3733 },
	{ 0x679c, 0x679d, PDF_CMAP_TABLE, 2381 },
	{ 0x67a0, 0x67a2, PDF_CMAP_TABLE, 2383 },
	{ 0x67a6, 0x67a6, PDF_CMAP_SINGLE, 5160 },
	{ 0x67a9, 0x67a9, PDF_CMAP_SINGLE, 5155 },
	{ 0x67af, 0x67b0, PDF_CMAP_TABLE, 2386 },
	{ 0x67b2, 0x67b4, PDF_CMAP_TABLE, 2388 },
	{ 0x67b6, 0x67b9, PDF_CMAP_TABLE, 2391 },
	{ 0x67bb, 0x67bb, PDF_CMAP_SINGLE, 8491 },
	{ 0x67c0, 0x67c1, PDF_CMAP_TABLE, 2395 },
	{ 0x67c4, 0x67c4, PDF_CMAP_SINGLE, 3601 },
	{ 0x67c6, 0x67c6, PDF_CMAP_SINGLE, 5177 },
	{ 0x67ca, 0x67ca, PDF_CMAP_SINGLE, 3476 },
	{ 0x67ce, 0x67d1, PDF_CMAP_TABLE, 2397 },
	{ 0x67d3, 0x67d4, PDF_CMAP_TABLE, 2401 },
	{ 0x67d7, 0x67da, PDF_CMAP_TABLE, 2403 },
	{ 0x67dd, 0x67de, PDF_CMAP_TABLE, 2407 },
	{ 0x67e2, 0x67e2, PDF_CMAP_SINGLE, 5173 },
	{ 0x67e4, 0x67e4, PDF_CMAP_SINGLE, 5170 },
	{ 0x67e7, 0x67e7, PDF_CMAP_SINGLE, 5178 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 5168 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 5166 },
	{ 0x67ee, 0x67f1, PDF_CMAP_TABLE, 2409 },
	{ 0x67f3, 0x67f5, PDF_CMAP_TABLE, 2413 },
	{ 0x67f9, 0x67f9, PDF_CMAP_SINGLE, 14622 },
	{ 0x67fb, 0x67fb, PDF_CMAP_SINGLE, 2090 },
	{ 0x67fe, 0x67ff, PDF_CMAP_TABLE, 2416 },
	{ 0x6801, 0x6805, PDF_CMAP_TABLE, 2418 },
	{ 0x6813, 0x6813, PDF_CMAP_SINGLE, 2710 },
	{ 0x6816, 0x6817, PDF_CMAP_TABLE, 2423 },
	{ 0x681e, 0x681e, PDF_CMAP_SINGLE, 5180 },
	{ 0x6821, 0x6822, PDF_CMAP_TABLE, 2425 },
	{ 0x6829, 0x682c, PDF_CMAP_TABLE, 2427 },
	{ 0x6830, 0x6831, PDF_CMAP_RANGE, 14627 },
	{ 0x6832, 0x6832, PDF_CMAP_SINGLE, 5185 },
	{ 0x6834, 0x6834, PDF_CMAP_SINGLE, 2711 },
	{ 0x6838, 0x6839, PDF_CMAP_TABLE, 2431 },
	{ 0x683c, 0x683d, PDF_CMAP_TABLE, 2433 },
	{ 0x6840, 0x6844, PDF_CMAP_TABLE, 2435 },
	{ 0x6846, 0x6846, PDF_CMAP_SINGLE, 5181 },
	{ 0x6848, 0x6848, PDF_CMAP_SINGLE, 1162 },
	{ 0x684d, 0x684e, PDF_CMAP_TABLE, 2440 },
	{ 0x6850, 0x6854, PDF_CMAP_TABLE, 2442 },
	{ 0x6859, 0x6859, PDF_CMAP_SINGLE, 5189 },
	{ 0x685b, 0x685d, PDF_CMAP_TABLE, 2447 },
	{ 0x685f, 0x685f, PDF_CMAP_SINGLE, 2181 },
	{ 0x6863, 0x6863, PDF_CMAP_SINGLE, 5190 },
	{ 0x6867, 0x6867, PDF_CMAP_SINGLE, 3490 },
	{ 0x6872, 0x6872, PDF_CMAP_SINGLE, 14630 },
	{ 0x6874, 0x6877, PDF_CMAP_TABLE, 2450 },
	{ 0x687a, 0x687a, PDF_CMAP_SINGLE, 14632 },
	{ 0x687e, 0x687f, PDF_CMAP_TABLE, 2454 },
	{ 0x6881, 0x6881, PDF_CMAP_SINGLE, 3978 },
	{ 0x6883, 0x6885, PDF_CMAP_TABLE, 2456 },
	{ 0x688d, 0x688f, PDF_CMAP_TABLE, 2459 },
	{ 0x6893, 0x6894, PDF_CMAP_TABLE, 2462 },
	{ 0x6897, 0x6897, PDF_CMAP_SINGLE, 1998 },
	{ 0x689b, 0x689b, PDF_CMAP_SINGLE, 5198 },
	{ 0x689d, 0x689d, PDF_CMAP_SINGLE, 5197 },
	{ 0x689f, 0x68a0, PDF_CMAP_TABLE, 2464 },
	{ 0x68a2, 0x68a2, PDF_CMAP_SINGLE, 2471 },
	{ 0x68a5, 0x68a8, PDF_CMAP_TABLE, 2466 },
	{ 0x68ad, 0x68ad, PDF_CMAP_SINGLE, 5195 },
	{ 0x68af, 0x68b3, PDF_CMAP_TABLE, 2470 },
	{ 0x68b5, 0x68b6, PDF_CMAP_TABLE, 2475 },
	{ 0x68b9, 0x68ba, PDF_CMAP_TABLE, 2477 },
	{ 0x68bc, 0x68bc, PDF_CMAP_SINGLE, 3176 },
	{ 0x68c4, 0x68c4, PDF_CMAP_SINGLE, 1594 },
	{ 0x68c6, 0x68c6, PDF_CMAP_SINGLE, 5235 },
	{ 0x68c8, 0x68cb, PDF_CMAP_TABLE, 2479 },
	{ 0x68cd, 0x68cd, PDF_CMAP_SINGLE, 5217 },
	{ 0x68cf, 0x68d0, PDF_CMAP_TABLE, 2483 },
	{ 0x68d2, 0x68d2, PDF_CMAP_SINGLE, 3694 },
	{ 0x68d4, 0x68d8, PDF_CMAP_TABLE, 2485 },
	{ 0x68da, 0x68da, PDF_CMAP_SINGLE, 2920 },
	{ 0x68df, 0x68e1, PDF_CMAP_TABLE, 2490 },
	{ 0x68e3, 0x68e3, PDF_CMAP_SINGLE, 5225 },
	{ 0x68e7, 0x68e8, PDF_CMAP_TABLE, 2493 },
	{ 0x68ed, 0x68ef, PDF_CMAP_TABLE, 2495 },
	{ 0x68f0, 0x68f1, PDF_CMAP_RANGE, 14640 },
	{ 0x68f2, 0x68f2, PDF_CMAP_SINGLE, 2647 },
	{ 0x68f9, 0x68fa, PDF_CMAP_TABLE, 2498 },
	{ 0x68fc, 0x68fc, PDF_CMAP_SINGLE, 14642 },
	{ 0x6900, 0x6901, PDF_CMAP_TABLE, 2500 },
	{ 0x6904, 0x6905, PDF_CMAP_TABLE, 2502 },
	{ 0x6908, 0x6908, PDF_CMAP_SINGLE, 5211 },
	{ 0x690b, 0x690f, PDF_CMAP_TABLE, 2504 },
	{ 0x6911, 0x6913, PDF_CMAP_TABLE, 2509 },
	{ 0x6919, 0x691c, PDF_CMAP_TABLE, 2512 },
	{ 0x6921, 0x6923, PDF_CMAP_TABLE, 2516 },
	{ 0x6925, 0x6926, PDF_CMAP_TABLE, 2519 },
	{ 0x6928, 0x6928, PDF_CMAP_SINGLE, 5230 },
	{ 0x692a, 0x692a, PDF_CMAP_SINGLE, 5231 },
	{ 0x6930, 0x6930, PDF_CMAP_SINGLE, 5248 },
	{ 0x6934, 0x6936, PDF_CMAP_TABLE, 2521 },
	{ 0x6939, 0x6939, PDF_CMAP_SINGLE, 5244 },
	{ 0x693b, 0x693b, PDF_CMAP_SINGLE, 14646 },
	{ 0x693d, 0x693d, PDF_CMAP_SINGLE, 5246 },
	{ 0x693f, 0x693f, PDF_CMAP_SINGLE, 3060 },
	{ 0x694a, 0x694a, PDF_CMAP_SINGLE, 3894 },
	{ 0x6953, 0x6955, PDF_CMAP_TABLE, 2524 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 14647 },
	{ 0x6959, 0x695a, PDF_CMAP_TABLE, 2527 },
	{ 0x695c, 0x695e, PDF_CMAP_TABLE, 2529 },
	{ 0x6960, 0x6963, PDF_CMAP_TABLE, 2532 },
	{ 0x6968, 0x6968, PDF_CMAP_SINGLE, 8498 },
	{ 0x696a, 0x696b, PDF_CMAP_TABLE, 2536 },
	{ 0x696d, 0x696f, PDF_CMAP_TABLE, 2538 },
	{ 0x6972, 0x6975, PDF_CMAP_TABLE, 2541 },
	{ 0x6977, 0x6979, PDF_CMAP_TABLE, 2545 },
	{ 0x697c, 0x697e, PDF_CMAP_TABLE, 2548 },
	{ 0x697f, 0x6980, PDF_CMAP_RANGE, 14650 },
	{ 0x6981, 0x6982, PDF_CMAP_TABLE, 2551 },
	{ 0x698a, 0x698a, PDF_CMAP_SINGLE, 2135 },
	{ 0x698e, 0x698e, PDF_CMAP_SINGLE, 1279 },
	{ 0x6991, 0x6991, PDF_CMAP_SINGLE, 5269 },
	{ 0x6994, 0x6995, PDF_CMAP_TABLE, 2553 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 8500 },
	{ 0x699b, 0x699c, PDF_CMAP_TABLE, 2555 },
	{ 0x69a0, 0x69a0, PDF_CMAP_SINGLE, 5270 },
	{ 0x69a6, 0x69a7, PDF_CMAP_TABLE, 2557 },
	{ 0x69ad, 0x69ae, PDF_CMAP_TABLE, 2559 },
	{ 0x69b1, 0x69b2, PDF_CMAP_TABLE, 2561 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 5273 },
	{ 0x69b7, 0x69b7, PDF_CMAP_SINGLE, 14654 },
	{ 0x69bb, 0x69bb, PDF_CMAP_SINGLE, 5265 },
	{ 0x69be, 0x69bf, PDF_CMAP_TABLE, 2563 },
	{ 0x69c1, 0x69c1, PDF_CMAP_SINGLE, 5258 },
	{ 0x69c3, 0x69c3, PDF_CMAP_SINGLE, 5266 },
	{ 0x69c7, 0x69c7, PDF_CMAP_SINGLE, 7475 },
	{ 0x69ca, 0x69ce, PDF_CMAP_TABLE, 2565 },
	{ 0x69d0, 0x69d0, PDF_CMAP_SINGLE, 5256 },
	{ 0x69d3, 0x69d3, PDF_CMAP_SINGLE, 5259 },
	{ 0x69d6, 0x69d7, PDF_CMAP_RANGE, 14655 },
	{ 0x69d8, 0x69d9, PDF_CMAP_TABLE, 2570 },
	{ 0x69dd, 0x69de, PDF_CMAP_TABLE, 2572 },
	{ 0x69e2, 0x69e2, PDF_CMAP_SINGLE, 8501 },
	{ 0x69e7, 0x69e8, PDF_CMAP_TABLE, 2574 },
	{ 0x69ea, 0x69eb, PDF_CMAP_TABLE, 2576 },
	{ 0x69ed, 0x69ed, PDF_CMAP_SINGLE, 5286 },
	{ 0x69f2, 0x69f2, PDF_CMAP_SINGLE, 5281 },
	{ 0x69f6, 0x69f6, PDF_CMAP_SINGLE, 14143 },
	{ 0x69f9, 0x69f9, PDF_CMAP_SINGLE, 5280 },
	{ 0x69fb, 0x69fb, PDF_CMAP_SINGLE, 3052 },
	{ 0x69fd, 0x69fd, PDF_CMAP_SINGLE, 2791 },
	{ 0x69ff, 0x69ff, PDF_CMAP_SINGLE, 5278 },
	{ 0x6a01, 0x6a02, PDF_CMAP_TABLE, 2578 },
	{ 0x6a05, 0x6a05, PDF_CMAP_SINGLE, 5283 },
	{ 0x6a0a, 0x6a0c, PDF_CMAP_TABLE, 2580 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 14658 },
	{ 0x6a12, 0x6a15, PDF_CMAP_TABLE, 2583 },
	{ 0x6a17, 0x6a17, PDF_CMAP_SINGLE, 2994 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 3498 },
	{ 0x6a1b, 0x6a1b, PDF_CMAP_SINGLE, 5277 },
	{ 0x6a1e, 0x6a1f, PDF_CMAP_TABLE, 2587 },
	{ 0x6a21, 0x6a23, PDF_CMAP_TABLE, 2589 },
	{ 0x6a28, 0x6a2b, PDF_CMAP_TABLE, 2592 },
	{ 0x6a2e, 0x6a2e, PDF_CMAP_SINGLE, 5268 },
	{ 0x6a30, 0x6a30, PDF_CMAP_SINGLE, 8502 },
	{ 0x6a34, 0x6a36, PDF_CMAP_TABLE, 2596 },
	{ 0x6a38, 0x6a3a, PDF_CMAP_TABLE, 2599 },
	{ 0x6a3d, 0x6a3e, PDF_CMAP_TABLE, 2602 },
	{ 0x6a44, 0x6a48, PDF_CMAP_TABLE, 2604 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 1710 },
	{ 0x6a50, 0x6a51, PDF_CMAP_RANGE, 14664 },
	{ 0x6a54, 0x6a54, PDF_CMAP_SINGLE, 15412 },
	{ 0x6a56, 0x6a56, PDF_CMAP_SINGLE, 14666 },
	{ 0x6a58, 0x6a59, PDF_CMAP_TABLE, 2609 },
	{ 0x6a5b, 0x6a5b, PDF_CMAP_SINGLE, 14667 },
	{ 0x6a5f, 0x6a5f, PDF_CMAP_SINGLE, 1595 },
	{ 0x6a61, 0x6a62, PDF_CMAP_TABLE, 2611 },
	{ 0x6a66, 0x6a66, PDF_CMAP_SINGLE, 5302 },
	{ 0x6a6b, 0x6a6b, PDF_CMAP_SINGLE, 8503 },
	{ 0x6a72, 0x6a73, PDF_CMAP_TABLE, 2613 },
	{ 0x6a78, 0x6a78, PDF_CMAP_SINGLE, 5298 },
	{ 0x6a7e, 0x6a80, PDF_CMAP_TABLE, 2615 },
	{ 0x6a83, 0x6a84, PDF_CMAP_TABLE, 2618 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 14669 },
	{ 0x6a8d, 0x6a8e, PDF_CMAP_TABLE, 2620 },
	{ 0x6a90, 0x6a91, PDF_CMAP_TABLE, 2622 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 5312 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 5179 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 14671 },
	{ 0x6aa0, 0x6aa0, PDF_CMAP_SINGLE, 5308 },
	{ 0x6aa2, 0x6aa3, PDF_CMAP_RANGE, 5310 },
	{ 0x6aaa, 0x6aaa, PDF_CMAP_SINGLE, 5323 },
	{ 0x6aac, 0x6aac, PDF_CMAP_SINGLE, 5319 },
	{ 0x6aae, 0x6aae, PDF_CMAP_SINGLE, 5200 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 5318 },
	{ 0x6ab8, 0x6ab8, PDF_CMAP_SINGLE, 5317 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 5314 },
	{ 0x6ac1, 0x6ac3, PDF_CMAP_TABLE, 2624 },
	{ 0x6ad1, 0x6ad1, PDF_CMAP_SINGLE, 5321 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 4044 },
	{ 0x6ada, 0x6adc, PDF_CMAP_TABLE, 2627 },
	{ 0x6ade, 0x6adf, PDF_CMAP_TABLE, 2630 },
	{ 0x6ae2, 0x6ae2, PDF_CMAP_SINGLE, 8507 },
	{ 0x6ae4, 0x6ae4, PDF_CMAP_SINGLE, 8508 },
	{ 0x6ae7, 0x6ae8, PDF_CMAP_TABLE, 2632 },
	{ 0x6aea, 0x6aea, PDF_CMAP_SINGLE, 5325 },
	{ 0x6aec, 0x6aec, PDF_CMAP_SINGLE, 14676 },
	{ 0x6afa, 0x6afb, PDF_CMAP_TABLE, 2634 },
	{ 0x6b04, 0x6b05, PDF_CMAP_TABLE, 2636 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 5279 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 5330 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 5331 },
	{ 0x6b1d, 0x6b21, PDF_CMAP_TABLE, 2638 },
	{ 0x6b23, 0x6b24, PDF_CMAP_TABLE, 2643 },
	{ 0x6b27, 0x6b27, PDF_CMAP_SINGLE, 1316 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 3913 },
	{ 0x6b35, 0x6b35, PDF_CMAP_SINGLE, 14679 },
	{ 0x6b37, 0x6b3a, PDF_CMAP_TABLE, 2645 },
	{ 0x6b3d, 0x6b3e, PDF_CMAP_TABLE, 2649 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 5340 },
	{ 0x6b46, 0x6b47, PDF_CMAP_TABLE, 2651 },
	{ 0x6b49, 0x6b49, PDF_CMAP_SINGLE, 5341 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 1358 },
	{ 0x6b4e, 0x6b4e, PDF_CMAP_SINGLE, 2933 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 5342 },
	{ 0x6b53, 0x6b54, PDF_CMAP_TABLE, 2653 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 14681 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 5343 },
	{ 0x6b5b, 0x6b5b, PDF_CMAP_SINGLE, 5345 },
	{ 0x6b5f, 0x6b66, PDF_CMAP_TABLE, 2655 },
	{ 0x6b69, 0x6b6a, PDF_CMAP_TABLE, 2663 },
	{ 0x6b6f, 0x6b6f, PDF_CMAP_SINGLE, 2243 },
	{ 0x6b72, 0x6b74, PDF_CMAP_TABLE, 2665 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 13398 },
	{ 0x6b78, 0x6b79, PDF_CMAP_RANGE, 5348 },
	{ 0x6b7b, 0x6b7b, PDF_CMAP_SINGLE, 2222 },
	{ 0x6b7f, 0x6b80, PDF_CMAP_RANGE, 5350 },
	{ 0x6b82, 0x6b84, PDF_CMAP_TABLE, 2668 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 3718 },
	{ 0x6b89, 0x6b8b, PDF_CMAP_TABLE, 2671 },
	{ 0x6b8d, 0x6b8d, PDF_CMAP_SINGLE, 5354 },
	{ 0x6b95, 0x6b96, PDF_CMAP_TABLE, 2674 },
	{ 0x6b98, 0x6b98, PDF_CMAP_SINGLE, 5355 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 5357 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 5358 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 5359 },
	{ 0x6baf, 0x6baf, PDF_CMAP_SINGLE, 5361 },
	{ 0x6bb1, 0x6bb5, PDF_CMAP_TABLE, 2676 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 5365 },
	{ 0x6bba, 0x6bbc, PDF_CMAP_TABLE, 2681 },
	{ 0x6bbe, 0x6bc0, PDF_CMAP_TABLE, 2684 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_TABLE, 2687 },
	{ 0x6bcb, 0x6bcf, PDF_CMAP_TABLE, 2689 },
	{ 0x6bd2, 0x6bd4, PDF_CMAP_TABLE, 2694 },
	{ 0x6bd6, 0x6bd6, PDF_CMAP_SINGLE, 8509 },
	{ 0x6bd8, 0x6bd8, PDF_CMAP_SINGLE, 3471 },
	{ 0x6bdb, 0x6bdb, PDF_CMAP_SINGLE, 3807 },
	{ 0x6bdf, 0x6bdf, PDF_CMAP_SINGLE, 5370 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 14685 },
	{ 0x6beb, 0x6bec, PDF_CMAP_TABLE, 2697 },
	{ 0x6bef, 0x6bef, PDF_CMAP_SINGLE, 5374 },
	{ 0x6bf1, 0x6bf1, PDF_CMAP_SINGLE, 14686 },
	{ 0x6bf3, 0x6bf3, PDF_CMAP_SINGLE, 5373 },
	{ 0x6c08, 0x6c08, PDF_CMAP_SINGLE, 5376 },
	{ 0x6c0f, 0x6c11, PDF_CMAP_TABLE, 2699 },
	{ 0x6c13, 0x6c14, PDF_CMAP_RANGE, 5377 },
	{ 0x6c17, 0x6c17, PDF_CMAP_SINGLE, 1598 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 5379 },
	{ 0x6c23, 0x6c24, PDF_CMAP_TABLE, 2702 },
	{ 0x6c33, 0x6c35, PDF_CMAP_TABLE, 2704 },
	{ 0x6c37, 0x6c38, PDF_CMAP_TABLE, 2707 },
	{ 0x6c3a, 0x6c3a, PDF_CMAP_SINGLE, 14690 },
	{ 0x6c3e, 0x6c42, PDF_CMAP_TABLE, 2709 },
	{ 0x6c4e, 0x6c4e, PDF_CMAP_SINGLE, 3418 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 2266 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 5383 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 1532 },
	{ 0x6c59, 0x6c5a, PDF_CMAP_TABLE, 2714 },
	{ 0x6c5c, 0x6c60, PDF_CMAP_TABLE, 2716 },
	{ 0x6c62, 0x6c62, PDF_CMAP_SINGLE, 5384 },
	{ 0x6c68, 0x6c68, PDF_CMAP_SINGLE, 5392 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 5385 },
	{ 0x6c6f, 0x6c70, PDF_CMAP_TABLE, 2721 },
	{ 0x6c72, 0x6c73, PDF_CMAP_TABLE, 2723 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 14692 },
	{ 0x6c7a, 0x6c7b, PDF_CMAP_TABLE, 2725 },
	{ 0x6c7d, 0x6c7e, PDF_CMAP_TABLE, 2727 },
	{ 0x6c81, 0x6c83, PDF_CMAP_TABLE, 2729 },
	{ 0x6c85, 0x6c86, PDF_CMAP_TABLE, 2732 },
	{ 0x6c88, 0x6c88, PDF_CMAP_SINGLE, 3036 },
	{ 0x6c8c, 0x6c8d, PDF_CMAP_TABLE, 2734 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 5395 },
	{ 0x6c92, 0x6c93, PDF_CMAP_TABLE, 2736 },
	{ 0x6c95, 0x6c96, PDF_CMAP_TABLE, 2738 },
	{ 0x6c99, 0x6c9c, PDF_CMAP_TABLE, 2740 },
	{ 0x6ca1, 0x6ca2, PDF_CMAP_TABLE, 2744 },
	{ 0x6caa, 0x6cab, PDF_CMAP_TABLE, 2746 },
	{ 0x6cae, 0x6cae, PDF_CMAP_SINGLE, 5403 },
	{ 0x6cb1, 0x6cb1, PDF_CMAP_SINGLE, 5404 },
	{ 0x6cb3, 0x6cb3, PDF_CMAP_SINGLE, 1359 },
	{ 0x6cb8, 0x6cbf, PDF_CMAP_TABLE, 2748 },
	{ 0x6cc1, 0x6cc1, PDF_CMAP_SINGLE, 1711 },
	{ 0x6cc4, 0x6cc5, PDF_CMAP_TABLE, 2756 },
	{ 0x6cc9, 0x6cca, PDF_CMAP_TABLE, 2758 },
	{ 0x6ccc, 0x6ccc, PDF_CMAP_SINGLE, 3451 },
	{ 0x6cd0, 0x6cd0, PDF_CMAP_SINGLE, 14697 },
	{ 0x6cd3, 0x6cd7, PDF_CMAP_TABLE, 2760 },
	{ 0x6cd9, 0x6cdb, PDF_CMAP_TABLE, 2765 },
	{ 0x6cdd, 0x6cdd, PDF_CMAP_SINGLE, 5402 },
	{ 0x6ce0, 0x6ce3, PDF_CMAP_TABLE, 2768 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 3103 },
	{ 0x6ce8, 0x6ce8, PDF_CMAP_SINGLE, 2987 },
	{ 0x6cea, 0x6cea, PDF_CMAP_SINGLE, 5410 },
	{ 0x6ceb, 0x6cec, PDF_CMAP_RANGE, 14701 },
	{ 0x6cee, 0x6cf1, PDF_CMAP_TABLE, 2772 },
	{ 0x6cf3, 0x6cf3, PDF_CMAP_SINGLE, 1261 },
	{ 0x6d01, 0x6d01, PDF_CMAP_SINGLE, 15413 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 8515 },
	{ 0x6d0a, 0x6d0c, PDF_CMAP_TABLE, 2776 },
	{ 0x6d0e, 0x6d0e, PDF_CMAP_SINGLE, 14705 },
	{ 0x6d11, 0x6d12, PDF_CMAP_TABLE, 2779 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 2714 },
	{ 0x6d19, 0x6d19, PDF_CMAP_SINGLE, 5417 },
	{ 0x6d1b, 0x6d1b, PDF_CMAP_SINGLE, 3926 },
	{ 0x6d1e, 0x6d1f, PDF_CMAP_TABLE, 2781 },
	{ 0x6d25, 0x6d25, PDF_CMAP_SINGLE, 3041 },
	{ 0x6d29, 0x6d2b, PDF_CMAP_TABLE, 2783 },
	{ 0x6d2e, 0x6d2e, PDF_CMAP_SINGLE, 14707 },
	{ 0x6d32, 0x6d33, PDF_CMAP_TABLE, 2786 },
	{ 0x6d35, 0x6d36, PDF_CMAP_TABLE, 2788 },
	{ 0x6d38, 0x6d38, PDF_CMAP_SINGLE, 5416 },
	{ 0x6d3b, 0x6d3b, PDF_CMAP_SINGLE, 1478 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_TABLE, 2790 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 3958 },
	{ 0x6d44, 0x6d45, PDF_CMAP_TABLE, 2792 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 14708 },
	{ 0x6d59, 0x6d5a, PDF_CMAP_TABLE, 2794 },
	{ 0x6d5c, 0x6d5c, PDF_CMAP_SINGLE, 3519 },
	{ 0x6d5e, 0x6d5e, PDF_CMAP_SINGLE, 14709 },
	{ 0x6d63, 0x6d66, PDF_CMAP_TABLE, 2796 },
	{ 0x6d69, 0x6d6a, PDF_CMAP_TABLE, 2800 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 1435 },
	{ 0x6d6e, 0x6d6f, PDF_CMAP_TABLE, 2802 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 3915 },
	{ 0x6d77, 0x6d79, PDF_CMAP_TABLE, 2804 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 14711 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 5431 },
	{ 0x6d87, 0x6d89, PDF_CMAP_TABLE, 2807 },
	{ 0x6d8c, 0x6d8c, PDF_CMAP_SINGLE, 3866 },
	{ 0x6d8e, 0x6d8e, PDF_CMAP_SINGLE, 5428 },
	{ 0x6d93, 0x6d93, PDF_CMAP_SINGLE, 5423 },
	{ 0x6d95, 0x6d96, PDF_CMAP_TABLE, 2810 },
	{ 0x6d99, 0x6d99, PDF_CMAP_SINGLE, 4006 },
	{ 0x6d9b, 0x6d9c, PDF_CMAP_TABLE, 2812 },
	{ 0x6dac, 0x6dac, PDF_CMAP_SINGLE, 8519 },
	{ 0x6daf, 0x6daf, PDF_CMAP_SINGLE, 1428 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 1271 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 5435 },
	{ 0x6db8, 0x6db8, PDF_CMAP_SINGLE, 5438 },
	{ 0x6dbc, 0x6dbc, PDF_CMAP_SINGLE, 3979 },
	{ 0x6dbf, 0x6dc0, PDF_CMAP_TABLE, 2814 },
	{ 0x6dc4, 0x6dc7, PDF_CMAP_TABLE, 2816 },
	{ 0x6dca, 0x6dcc, PDF_CMAP_TABLE, 2820 },
	{ 0x6dcf, 0x6dd2, PDF_CMAP_TABLE, 2823 },
	{ 0x6dd5, 0x6dd6, PDF_CMAP_TABLE, 2827 },
	{ 0x6dd8, 0x6dda, PDF_CMAP_TABLE, 2829 },
	{ 0x6dde, 0x6dde, PDF_CMAP_SINGLE, 5441 },
	{ 0x6de1, 0x6de1, PDF_CMAP_SINGLE, 2934 },
	{ 0x6de4, 0x6de4, PDF_CMAP_SINGLE, 5448 },
	{ 0x6de6, 0x6de6, PDF_CMAP_SINGLE, 5437 },
	{ 0x6de8, 0x6dec, PDF_CMAP_TABLE, 2832 },
	{ 0x6dee, 0x6dee, PDF_CMAP_SINGLE, 5451 },
	{ 0x6df1, 0x6df3, PDF_CMAP_TABLE, 2837 },
	{ 0x6df5, 0x6df5, PDF_CMAP_SINGLE, 3573 },
	{ 0x6df7, 0x6dfc, PDF_CMAP_TABLE, 2840 },
	{ 0x6e05, 0x6e05, PDF_CMAP_SINGLE, 2650 },
	{ 0x6e07, 0x6e0b, PDF_CMAP_TABLE, 2846 },
	{ 0x6e13, 0x6e13, PDF_CMAP_SINGLE, 1826 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 5433 },
	{ 0x6e17, 0x6e17, PDF_CMAP_SINGLE, 14150 },
	{ 0x6e19, 0x6e1b, PDF_CMAP_TABLE, 2851 },
	{ 0x6e1d, 0x6e1d, PDF_CMAP_SINGLE, 5470 },
	{ 0x6e1f, 0x6e27, PDF_CMAP_TABLE, 2854 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 1337 },
	{ 0x6e2b, 0x6e2f, PDF_CMAP_TABLE, 2863 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 13330 },
	{ 0x6e38, 0x6e3a, PDF_CMAP_TABLE, 2868 },
	{ 0x6e3c, 0x6e3c, PDF_CMAP_SINGLE, 8527 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 5458 },
	{ 0x6e42, 0x6e43, PDF_CMAP_TABLE, 2871 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 3767 },
	{ 0x6e4d, 0x6e4e, PDF_CMAP_TABLE, 2873 },
	{ 0x6e51, 0x6e51, PDF_CMAP_SINGLE, 14718 },
	{ 0x6e56, 0x6e56, PDF_CMAP_SINGLE, 1924 },
	{ 0x6e58, 0x6e58, PDF_CMAP_SINGLE, 2477 },
	{ 0x6e5b, 0x6e5c, PDF_CMAP_TABLE, 2875 },
	{ 0x6e5f, 0x6e5f, PDF_CMAP_SINGLE, 5457 },
	{ 0x6e67, 0x6e67, PDF_CMAP_SINGLE, 3865 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 5460 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 2877 },
	{ 0x6e72, 0x6e72, PDF_CMAP_SINGLE, 5456 },
	{ 0x6e76, 0x6e76, PDF_CMAP_SINGLE, 5462 },
	{ 0x6e7e, 0x6e80, PDF_CMAP_TABLE, 2879 },
	{ 0x6e82, 0x6e82, PDF_CMAP_SINGLE, 5472 },
	{ 0x6e8c, 0x6e8c, PDF_CMAP_SINGLE, 3394 },
	{ 0x6e8f, 0x6e90, PDF_CMAP_TABLE, 2882 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 2410 },
	{ 0x6e98, 0x6e98, PDF_CMAP_SINGLE, 5474 },
	{ 0x6e9c, 0x6e9d, PDF_CMAP_TABLE, 2884 },
	{ 0x6e9f, 0x6e9f, PDF_CMAP_SINGLE, 5487 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 1202 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 5485 },
	{ 0x6eaa, 0x6eab, PDF_CMAP_TABLE, 2886 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 5479 },
	{ 0x6eb2, 0x6eb2, PDF_CMAP_SINGLE, 5481 },
	{ 0x6eb6, 0x6eb7, PDF_CMAP_TABLE, 2888 },
	{ 0x6eba, 0x6eba, PDF_CMAP_SINGLE, 3112 },
	{ 0x6ebd, 0x6ebd, PDF_CMAP_SINGLE, 5478 },
	{ 0x6ebf, 0x6ebf, PDF_CMAP_SINGLE, 8528 },
	{ 0x6ec2, 0x6ec2, PDF_CMAP_SINGLE, 5486 },
	{ 0x6ec4, 0x6ec5, PDF_CMAP_TABLE, 2890 },
	{ 0x6ec7, 0x6ec7, PDF_CMAP_SINGLE, 14719 },
	{ 0x6ec9, 0x6ecc, PDF_CMAP_TABLE, 2892 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 14721 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 1480 },
	{ 0x6ed3, 0x6ed3, PDF_CMAP_SINGLE, 5477 },
	{ 0x6ed4, 0x6ed5, PDF_CMAP_RANGE, 5482 },
	{ 0x6ed9, 0x6ed9, PDF_CMAP_SINGLE, 15416 },
	{ 0x6edd, 0x6ede, PDF_CMAP_TABLE, 2896 },
	{ 0x6ee6, 0x6ee6, PDF_CMAP_SINGLE, 15396 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 5491 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 5497 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 5495 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 3107 },
	{ 0x6ef7, 0x6ef8, PDF_CMAP_TABLE, 2898 },
	{ 0x6efd, 0x6eff, PDF_CMAP_TABLE, 2900 },
	{ 0x6f01, 0x6f02, PDF_CMAP_TABLE, 2903 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 2283 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 2057 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 4057 },
	{ 0x6f11, 0x6f11, PDF_CMAP_SINGLE, 5489 },
	{ 0x6f13, 0x6f15, PDF_CMAP_TABLE, 2905 },
	{ 0x6f1a, 0x6f1a, PDF_CMAP_SINGLE, 14723 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 3375 },
	{ 0x6f22, 0x6f23, PDF_CMAP_TABLE, 2908 },
	{ 0x6f2a, 0x6f2c, PDF_CMAP_TABLE, 2910 },
	{ 0x6f2f, 0x6f2f, PDF_CMAP_SINGLE, 14725 },
	{ 0x6f31, 0x6f33, PDF_CMAP_TABLE, 2913 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 2740 },
	{ 0x6f3e, 0x6f3f, PDF_CMAP_TABLE, 2916 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 5488 },
	{ 0x6f45, 0x6f45, PDF_CMAP_SINGLE, 1535 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 7776 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 1855 },
	{ 0x6f58, 0x6f58, PDF_CMAP_SINGLE, 5514 },
	{ 0x6f5a, 0x6f5c, PDF_CMAP_TABLE, 2918 },
	{ 0x6f5e, 0x6f5f, PDF_CMAP_TABLE, 2921 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 14729 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 2411 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 5518 },
	{ 0x6f6d, 0x6f70, PDF_CMAP_TABLE, 2923 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 5543 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 5505 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 5504 },
	{ 0x6f7c, 0x6f7d, PDF_CMAP_TABLE, 2927 },
	{ 0x6f80, 0x6f82, PDF_CMAP_TABLE, 2929 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 2629 },
	{ 0x6f86, 0x6f86, PDF_CMAP_SINGLE, 5503 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 8529 },
	{ 0x6f8b, 0x6f8b, PDF_CMAP_SINGLE, 14731 },
	{ 0x6f8d, 0x6f8e, PDF_CMAP_TABLE, 2932 },
	{ 0x6f91, 0x6f92, PDF_CMAP_TABLE, 2934 },
	{ 0x6f94, 0x6f94, PDF_CMAP_SINGLE, 14734 },
	{ 0x6f97, 0x6f98, PDF_CMAP_TABLE, 2936 },
	{ 0x6f9a, 0x6f9a, PDF_CMAP_SINGLE, 14735 },
	{ 0x6fa1, 0x6fa1, PDF_CMAP_SINGLE, 5521 },
	{ 0x6fa3, 0x6fa4, PDF_CMAP_TABLE, 2938 },
	{ 0x6fa7, 0x6fa8, PDF_CMAP_RANGE, 14736 },
	{ 0x6faa, 0x6faa, PDF_CMAP_SINGLE, 5525 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 3133 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 5519 },
	{ 0x6fb5, 0x6fb6, PDF_CMAP_TABLE, 2940 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5523 },
	{ 0x6fc0, 0x6fc3, PDF_CMAP_TABLE, 2942 },
	{ 0x6fc6, 0x6fc6, PDF_CMAP_SINGLE, 5524 },
	{ 0x6fd4, 0x6fd5, PDF_CMAP_TABLE, 2946 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 5530 },
	{ 0x6fda, 0x6fdb, PDF_CMAP_TABLE, 2948 },
	{ 0x6fde, 0x6fe1, PDF_CMAP_TABLE, 2950 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 5430 },
	{ 0x6feb, 0x6fec, PDF_CMAP_TABLE, 2954 },
	{ 0x6fee, 0x6fef, PDF_CMAP_TABLE, 2956 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 5531 },
	{ 0x6ff3, 0x6ff3, PDF_CMAP_SINGLE, 5510 },
	{ 0x6ff5, 0x6ff6, PDF_CMAP_TABLE, 2958 },
	{ 0x6ff9, 0x6ffa, PDF_CMAP_TABLE, 2960 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 5540 },
	{ 0x7001, 0x7001, PDF_CMAP_SINGLE, 5538 },
	{ 0x7005, 0x7007, PDF_CMAP_TABLE, 2962 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 5534 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 5535 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 5539 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5537 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 3520 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 5545 },
	{ 0x701a, 0x701b, PDF_CMAP_TABLE, 2965 },
	{ 0x701d, 0x701f, PDF_CMAP_TABLE, 2967 },
	{ 0x7026, 0x7028, PDF_CMAP_TABLE, 2970 },
	{ 0x702c, 0x702c, PDF_CMAP_SINGLE, 2633 },
	{ 0x7030, 0x7030, PDF_CMAP_SINGLE, 5547 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 5549 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 14742 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 14743 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 5548 },
	{ 0x704a, 0x704a, PDF_CMAP_SINGLE, 14744 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 5490 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 5550 },
	{ 0x7054, 0x7054, PDF_CMAP_SINGLE, 14745 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 3263 },
	{ 0x705d, 0x705e, PDF_CMAP_RANGE, 14746 },
	{ 0x7063, 0x7064, PDF_CMAP_TABLE, 2973 },
	{ 0x706b, 0x706c, PDF_CMAP_TABLE, 2975 },
	{ 0x706f, 0x7070, PDF_CMAP_TABLE, 2977 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 1662 },
	{ 0x707c, 0x707e, PDF_CMAP_TABLE, 2979 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 14751 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 8535 },
	{ 0x7089, 0x708a, PDF_CMAP_TABLE, 2982 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 1292 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 5553 },
	{ 0x7095, 0x7095, PDF_CMAP_SINGLE, 14752 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 5552 },
	{ 0x70ab, 0x70af, PDF_CMAP_TABLE, 2984 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 5558 },
	{ 0x70b7, 0x70bb, PDF_CMAP_TABLE, 2989 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 4029 },
	{ 0x70cb, 0x70cb, PDF_CMAP_SINGLE, 5561 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 1226 },
	{ 0x70d3, 0x70d4, PDF_CMAP_RANGE, 14754 },
	{ 0x70d8, 0x70d9, PDF_CMAP_TABLE, 2994 },
	{ 0x70dc, 0x70dd, PDF_CMAP_TABLE, 2996 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 5560 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 5555 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 3665 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 5565 },
	{ 0x7104, 0x7104, PDF_CMAP_SINGLE, 8538 },
	{ 0x7107, 0x7107, PDF_CMAP_SINGLE, 14758 },
	{ 0x7109, 0x7109, PDF_CMAP_SINGLE, 5564 },
	{ 0x710f, 0x710f, PDF_CMAP_SINGLE, 8537 },
	{ 0x7114, 0x7114, PDF_CMAP_SINGLE, 1293 },
	{ 0x7119, 0x711a, PDF_CMAP_TABLE, 2998 },
	{ 0x711c, 0x711c, PDF_CMAP_SINGLE, 5566 },
	{ 0x7120, 0x7121, PDF_CMAP_TABLE, 3000 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2479 },
	{ 0x7130, 0x7131, PDF_CMAP_TABLE, 3002 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 2741 },
	{ 0x713c, 0x713c, PDF_CMAP_SINGLE, 2478 },
	{ 0x7146, 0x7147, PDF_CMAP_RANGE, 8540 },
	{ 0x7149, 0x714a, PDF_CMAP_TABLE, 3004 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 5573 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 2717 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 14762 },
	{ 0x7155, 0x7156, PDF_CMAP_TABLE, 3006 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 1294 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 8539 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 14763 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 5572 },
	{ 0x7164, 0x7167, PDF_CMAP_TABLE, 3008 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 3429 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 5575 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 2301 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 14764 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 2718 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 5578 },
	{ 0x7188, 0x7188, PDF_CMAP_SINGLE, 5570 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 1789 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 5576 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 14765 },
	{ 0x7194, 0x7195, PDF_CMAP_TABLE, 3012 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 8285 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 2393 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 5580 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 5581 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 3300 },
	{ 0x71b3, 0x71b3, PDF_CMAP_SINGLE, 14766 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 5583 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 5584 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 8543 },
	{ 0x71c3, 0x71c3, PDF_CMAP_SINGLE, 3305 },
	{ 0x71c8, 0x71c9, PDF_CMAP_TABLE, 3014 },
	{ 0x71cb, 0x71cb, PDF_CMAP_SINGLE, 14767 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 5588 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 3997 },
	{ 0x71d2, 0x71d7, PDF_CMAP_TABLE, 3016 },
	{ 0x71df, 0x71e0, PDF_CMAP_TABLE, 3022 },
	{ 0x71e5, 0x71e7, PDF_CMAP_TABLE, 3024 },
	{ 0x71ec, 0x71ee, PDF_CMAP_TABLE, 3027 },
	{ 0x71f5, 0x71f5, PDF_CMAP_SINGLE, 5592 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 5594 },
	{ 0x71fb, 0x71fc, PDF_CMAP_TABLE, 3030 },
	{ 0x71fe, 0x7200, PDF_CMAP_TABLE, 3032 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 3376 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 5596 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 5597 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 5598 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 14771 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 5599 },
	{ 0x722a, 0x722d, PDF_CMAP_TABLE, 3035 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 5602 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 5603 },
	{ 0x7235, 0x7236, PDF_CMAP_TABLE, 3039 },
	{ 0x7238, 0x7238, PDF_CMAP_SINGLE, 14773 },
	{ 0x723a, 0x723a, PDF_CMAP_SINGLE, 3832 },
	{ 0x723b, 0x723c, PDF_CMAP_RANGE, 5604 },
	{ 0x723d, 0x723e, PDF_CMAP_TABLE, 3041 },
	{ 0x723f, 0x7240, PDF_CMAP_RANGE, 5606 },
	{ 0x7241, 0x7241, PDF_CMAP_SINGLE, 14774 },
	{ 0x7246, 0x7248, PDF_CMAP_TABLE, 3043 },
	{ 0x724b, 0x724c, PDF_CMAP_TABLE, 3046 },
	{ 0x7252, 0x7253, PDF_CMAP_TABLE, 3048 },
	{ 0x7255, 0x7256, PDF_CMAP_RANGE, 14776 },
	{ 0x7258, 0x7259, PDF_CMAP_TABLE, 3050 },
	{ 0x725b, 0x725d, PDF_CMAP_TABLE, 3052 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 3778 },
	{ 0x7261, 0x7262, PDF_CMAP_TABLE, 3055 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 3712 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 3578 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 2651 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 5611 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 3227 },
	{ 0x727d, 0x727e, PDF_CMAP_TABLE, 3057 },
	{ 0x7280, 0x7282, PDF_CMAP_TABLE, 3059 },
	{ 0x7287, 0x7287, PDF_CMAP_SINGLE, 5615 },
	{ 0x728d, 0x728d, PDF_CMAP_SINGLE, 14779 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 5616 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 5617 },
	{ 0x72a0, 0x72a0, PDF_CMAP_SINGLE, 1624 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 5618 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 5619 },
	{ 0x72ac, 0x72ad, PDF_CMAP_TABLE, 3062 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 3420 },
	{ 0x72b1, 0x72b2, PDF_CMAP_TABLE, 3064 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 14781 },
	{ 0x72b6, 0x72b6, PDF_CMAP_SINGLE, 2525 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 5620 },
	{ 0x72be, 0x72be, PDF_CMAP_SINGLE, 8546 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 13355 },
	{ 0x72c2, 0x72c4, PDF_CMAP_TABLE, 3066 },
	{ 0x72c6, 0x72c7, PDF_CMAP_TABLE, 3069 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5625 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1925 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 5626 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 1761 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 2754 },
	{ 0x72db, 0x72db, PDF_CMAP_SINGLE, 2063 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 5628 },
	{ 0x72e2, 0x72e2, PDF_CMAP_SINGLE, 5627 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 2329 },
	{ 0x72ec, 0x72ed, PDF_CMAP_TABLE, 3071 },
	{ 0x72f7, 0x72f9, PDF_CMAP_TABLE, 3073 },
	{ 0x72fb, 0x72fd, PDF_CMAP_TABLE, 3076 },
	{ 0x7304, 0x7305, PDF_CMAP_RANGE, 14784 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 5634 },
	{ 0x7316, 0x7317, PDF_CMAP_TABLE, 3079 },
	{ 0x731b, 0x731d, PDF_CMAP_TABLE, 3081 },
	{ 0x731f, 0x731f, PDF_CMAP_SINGLE, 3980 },
	{ 0x7324, 0x7325, PDF_CMAP_TABLE, 3084 },
	{ 0x7328, 0x732b, PDF_CMAP_TABLE, 3086 },
	{ 0x732e, 0x732f, PDF_CMAP_TABLE, 3090 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 14787 },
	{ 0x7334, 0x7334, PDF_CMAP_SINGLE, 5638 },
	{ 0x7336, 0x7337, PDF_CMAP_RANGE, 3867 },
	{ 0x733e, 0x733f, PDF_CMAP_TABLE, 3092 },
	{ 0x7343, 0x7345, PDF_CMAP_TABLE, 3094 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 5643 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 5646 },
	{ 0x7363, 0x7363, PDF_CMAP_SINGLE, 2381 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 5648 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 5647 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 14789 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 5649 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 1451 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 5651 },
	{ 0x7377, 0x7378, PDF_CMAP_TABLE, 3097 },
	{ 0x737a, 0x737c, PDF_CMAP_TABLE, 3099 },
	{ 0x7383, 0x7384, PDF_CMAP_TABLE, 3102 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 14792 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 3952 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 1732 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 1318 },
	{ 0x7395, 0x7396, PDF_CMAP_TABLE, 3104 },
	{ 0x739e, 0x73a0, PDF_CMAP_RANGE, 14795 },
	{ 0x73a6, 0x73a6, PDF_CMAP_SINGLE, 14798 },
	{ 0x73a8, 0x73a9, PDF_CMAP_TABLE, 3106 },
	{ 0x73ab, 0x73ab, PDF_CMAP_SINGLE, 14799 },
	{ 0x73b2, 0x73b3, PDF_CMAP_TABLE, 3108 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 14800 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 14801 },
	{ 0x73ba, 0x73bd, PDF_CMAP_TABLE, 3110 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 5658 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 1361 },
	{ 0x73c8, 0x73ca, PDF_CMAP_TABLE, 3114 },
	{ 0x73cd, 0x73cf, PDF_CMAP_TABLE, 3117 },
	{ 0x73d2, 0x73d2, PDF_CMAP_SINGLE, 8554 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 8552 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 14804 },
	{ 0x73de, 0x73de, PDF_CMAP_SINGLE, 5661 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 2330 },
	{ 0x73e3, 0x73e5, PDF_CMAP_TABLE, 3120 },
	{ 0x73e9, 0x73ea, PDF_CMAP_TABLE, 3123 },
	{ 0x73ed, 0x73ee, PDF_CMAP_TABLE, 3125 },
	{ 0x73f1, 0x73f1, PDF_CMAP_SINGLE, 5687 },
	{ 0x73f4, 0x73f5, PDF_CMAP_TABLE, 3127 },
	{ 0x73f8, 0x73f8, PDF_CMAP_SINGLE, 5666 },
	{ 0x73fd, 0x73fe, PDF_CMAP_TABLE, 3129 },
	{ 0x7403, 0x7407, PDF_CMAP_TABLE, 3131 },
	{ 0x7409, 0x740a, PDF_CMAP_TABLE, 3136 },
	{ 0x741a, 0x741b, PDF_CMAP_RANGE, 14810 },
	{ 0x7421, 0x7422, PDF_CMAP_TABLE, 3138 },
	{ 0x7424, 0x7426, PDF_CMAP_TABLE, 3140 },
	{ 0x7428, 0x742a, PDF_CMAP_TABLE, 3143 },
	{ 0x742c, 0x742c, PDF_CMAP_SINGLE, 14814 },
	{ 0x742e, 0x742e, PDF_CMAP_SINGLE, 8560 },
	{ 0x742f, 0x7431, PDF_CMAP_RANGE, 14815 },
	{ 0x7432, 0x7436, PDF_CMAP_TABLE, 3146 },
	{ 0x7439, 0x743a, PDF_CMAP_TABLE, 3151 },
	{ 0x743f, 0x743f, PDF_CMAP_SINGLE, 5670 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 5673 },
	{ 0x7444, 0x7444, PDF_CMAP_SINGLE, 14819 },
	{ 0x7447, 0x7447, PDF_CMAP_SINGLE, 14820 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 14821 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 14822 },
	{ 0x7451, 0x7451, PDF_CMAP_SINGLE, 14823 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 5669 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 14824 },
	{ 0x7459, 0x745c, PDF_CMAP_TABLE, 3153 },
	{ 0x745e, 0x7460, PDF_CMAP_TABLE, 3157 },
	{ 0x7462, 0x7464, PDF_CMAP_TABLE, 3160 },
	{ 0x7466, 0x7466, PDF_CMAP_SINGLE, 14825 },
	{ 0x7469, 0x746b, PDF_CMAP_TABLE, 3163 },
	{ 0x746f, 0x7471, PDF_CMAP_TABLE, 3166 },
	{ 0x7473, 0x7473, PDF_CMAP_SINGLE, 2092 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 5679 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 5680 },
	{ 0x7480, 0x7480, PDF_CMAP_SINGLE, 14828 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 3944 },
	{ 0x7485, 0x7487, PDF_CMAP_RANGE, 14829 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 8562 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 5681 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 14832 },
	{ 0x7498, 0x7498, PDF_CMAP_SINGLE, 14833 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 14834 },
	{ 0x749e, 0x74a0, PDF_CMAP_TABLE, 3169 },
	{ 0x74a2, 0x74a3, PDF_CMAP_TABLE, 3172 },
	{ 0x74a7, 0x74a8, PDF_CMAP_TABLE, 3174 },
	{ 0x74ab, 0x74ab, PDF_CMAP_SINGLE, 14838 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 1536 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 14839 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 2257 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 14840 },
	{ 0x74c8, 0x74c8, PDF_CMAP_SINGLE, 14841 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 5684 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 5685 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 5686 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 14842 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 1245 },
	{ 0x74de, 0x74de, PDF_CMAP_SINGLE, 14843 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 5688 },
	{ 0x74e2, 0x74e3, PDF_CMAP_TABLE, 3176 },
	{ 0x74e6, 0x74e7, PDF_CMAP_TABLE, 3178 },
	{ 0x74e9, 0x74e9, PDF_CMAP_SINGLE, 5691 },
	{ 0x74ee, 0x74ef, PDF_CMAP_TABLE, 3180 },
	{ 0x74f0, 0x74f1, PDF_CMAP_RANGE, 5694 },
	{ 0x74f2, 0x74f2, PDF_CMAP_SINGLE, 5693 },
	{ 0x74f6, 0x74f8, PDF_CMAP_TABLE, 3182 },
	{ 0x7501, 0x7501, PDF_CMAP_SINGLE, 8564 },
	{ 0x7503, 0x7505, PDF_CMAP_TABLE, 3185 },
	{ 0x750c, 0x750e, PDF_CMAP_TABLE, 3188 },
	{ 0x7511, 0x7511, PDF_CMAP_SINGLE, 2059 },
	{ 0x7513, 0x7513, PDF_CMAP_SINGLE, 5705 },
	{ 0x7515, 0x7515, PDF_CMAP_SINGLE, 5704 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 1537 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 2585 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 3126 },
	{ 0x751e, 0x751f, PDF_CMAP_TABLE, 3191 },
	{ 0x7522, 0x7523, PDF_CMAP_TABLE, 3193 },
	{ 0x7525, 0x7526, PDF_CMAP_TABLE, 3195 },
	{ 0x7528, 0x7528, PDF_CMAP_SINGLE, 3899 },
	{ 0x752b, 0x752c, PDF_CMAP_TABLE, 3197 },
	{ 0x752f, 0x7533, PDF_CMAP_TABLE, 3199 },
	{ 0x7537, 0x7538, PDF_CMAP_TABLE, 3204 },
	{ 0x753a, 0x753c, PDF_CMAP_TABLE, 3206 },
	{ 0x7544, 0x7544, PDF_CMAP_SINGLE, 5710 },
	{ 0x7546, 0x7546, PDF_CMAP_SINGLE, 5715 },
	{ 0x7549, 0x754f, PDF_CMAP_TABLE, 3209 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 3390 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 3422 },
	{ 0x7559, 0x755d, PDF_CMAP_TABLE, 3216 },
	{ 0x7560, 0x7560, PDF_CMAP_SINGLE, 3391 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 3487 },
	{ 0x7564, 0x7567, PDF_CMAP_TABLE, 3221 },
	{ 0x7569, 0x756d, PDF_CMAP_TABLE, 3225 },
	{ 0x756f, 0x7570, PDF_CMAP_TABLE, 3230 },
	{ 0x7573, 0x7579, PDF_CMAP_TABLE, 3232 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 1600 },
	{ 0x7581, 0x7582, PDF_CMAP_TABLE, 3239 },
	{ 0x7586, 0x7587, PDF_CMAP_RANGE, 5724 },
	{ 0x7589, 0x758b, PDF_CMAP_TABLE, 3241 },
	{ 0x758e, 0x7591, PDF_CMAP_TABLE, 3244 },
	{ 0x7592, 0x7593, PDF_CMAP_RANGE, 14848 },
	{ 0x7594, 0x7594, PDF_CMAP_SINGLE, 5730 },
	{ 0x759a, 0x759a, PDF_CMAP_SINGLE, 5731 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 5732 },
	{ 0x75a3, 0x75a3, PDF_CMAP_SINGLE, 5734 },
	{ 0x75a5, 0x75a5, PDF_CMAP_SINGLE, 5733 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 1272 },
	{ 0x75b1, 0x75b5, PDF_CMAP_TABLE, 3248 },
	{ 0x75b8, 0x75b9, PDF_CMAP_TABLE, 3253 },
	{ 0x75bc, 0x75be, PDF_CMAP_TABLE, 3255 },
	{ 0x75c2, 0x75c3, PDF_CMAP_TABLE, 3258 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 3508 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 2481 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 5744 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 5743 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 5745 },
	{ 0x75d4, 0x75d5, PDF_CMAP_TABLE, 3260 },
	{ 0x75d8, 0x75d9, PDF_CMAP_TABLE, 3262 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 3047 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 5748 },
	{ 0x75e2, 0x75e4, PDF_CMAP_TABLE, 3264 },
	{ 0x75e9, 0x75e9, PDF_CMAP_SINGLE, 2795 },
	{ 0x75ec, 0x75ec, PDF_CMAP_SINGLE, 14162 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 5753 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 5755 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 2962 },
	{ 0x75f9, 0x75fa, PDF_CMAP_TABLE, 3267 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 5751 },
	{ 0x75fe, 0x75ff, PDF_CMAP_RANGE, 5749 },
	{ 0x7600, 0x7601, PDF_CMAP_TABLE, 3269 },
	{ 0x7609, 0x760b, PDF_CMAP_TABLE, 3271 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 5758 },
	{ 0x7615, 0x7616, PDF_CMAP_RANGE, 14855 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 14857 },
	{ 0x761e, 0x761f, PDF_CMAP_TABLE, 3274 },
	{ 0x7620, 0x7622, PDF_CMAP_RANGE, 5762 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 5765 },
	{ 0x7626, 0x7627, PDF_CMAP_TABLE, 3276 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 14859 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 5767 },
	{ 0x7634, 0x7635, PDF_CMAP_TABLE, 3278 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 5768 },
	{ 0x7642, 0x7643, PDF_CMAP_TABLE, 3280 },
	{ 0x7646, 0x7646, PDF_CMAP_SINGLE, 5771 },
	{ 0x7647, 0x7648, PDF_CMAP_RANGE, 5769 },
	{ 0x764b, 0x764c, PDF_CMAP_TABLE, 3282 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 14163 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 3850 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 3610 },
	{ 0x7658, 0x7658, PDF_CMAP_SINGLE, 5773 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 5772 },
	{ 0x7661, 0x7662, PDF_CMAP_RANGE, 5774 },
	{ 0x7665, 0x7665, PDF_CMAP_SINGLE, 14863 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 5779 },
	{ 0x7668, 0x766a, PDF_CMAP_RANGE, 5776 },
	{ 0x766c, 0x766d, PDF_CMAP_TABLE, 3284 },
	{ 0x766f, 0x7672, PDF_CMAP_TABLE, 3286 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 14867 },
	{ 0x7676, 0x7676, PDF_CMAP_SINGLE, 5783 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 5784 },
	{ 0x767a, 0x767e, PDF_CMAP_TABLE, 3290 },
	{ 0x7680, 0x7680, PDF_CMAP_SINGLE, 5786 },
	{ 0x7682, 0x7684, PDF_CMAP_TABLE, 3295 },
	{ 0x7686, 0x7688, PDF_CMAP_TABLE, 3298 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 5789 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 5790 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 2167 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 5792 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 5791 },
	{ 0x7699, 0x769a, PDF_CMAP_RANGE, 5793 },
	{ 0x769b, 0x769c, PDF_CMAP_TABLE, 3301 },
	{ 0x769e, 0x769e, PDF_CMAP_SINGLE, 8568 },
	{ 0x76a4, 0x76a5, PDF_CMAP_RANGE, 14868 },
	{ 0x76a6, 0x76a6, PDF_CMAP_SINGLE, 8570 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3453 },
	{ 0x76b0, 0x76b0, PDF_CMAP_SINGLE, 5795 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 5796 },
	{ 0x76b7, 0x76b7, PDF_CMAP_SINGLE, 7452 },
	{ 0x76b8, 0x76ba, PDF_CMAP_RANGE, 5797 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2172 },
	{ 0x76c2, 0x76c3, PDF_CMAP_TABLE, 3303 },
	{ 0x76c5, 0x76c6, PDF_CMAP_TABLE, 3305 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 1264 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 1273 },
	{ 0x76cc, 0x76cd, PDF_CMAP_TABLE, 3307 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 5803 },
	{ 0x76d6, 0x76d7, PDF_CMAP_TABLE, 3309 },
	{ 0x76db, 0x76dc, PDF_CMAP_TABLE, 3311 },
	{ 0x76de, 0x76df, PDF_CMAP_TABLE, 3313 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 5805 },
	{ 0x76e3, 0x76e5, PDF_CMAP_TABLE, 3315 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 5807 },
	{ 0x76ea, 0x76ea, PDF_CMAP_SINGLE, 5808 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 14872 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 3816 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 3809 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 3034 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 2796 },
	{ 0x76fb, 0x76fc, PDF_CMAP_TABLE, 3318 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 2412 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 2482 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 5813 },
	{ 0x7707, 0x7709, PDF_CMAP_TABLE, 3320 },
	{ 0x770b, 0x770c, PDF_CMAP_TABLE, 3323 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 5819 },
	{ 0x771e, 0x7720, PDF_CMAP_TABLE, 3325 },
	{ 0x7724, 0x7724, PDF_CMAP_SINGLE, 5815 },
	{ 0x7725, 0x7726, PDF_CMAP_RANGE, 5817 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 5814 },
	{ 0x7734, 0x7734, PDF_CMAP_SINGLE, 14874 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 14875 },
	{ 0x7737, 0x7738, PDF_CMAP_RANGE, 5820 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 3019 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 1567 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 2979 },
	{ 0x7746, 0x7747, PDF_CMAP_TABLE, 3328 },
	{ 0x775a, 0x775c, PDF_CMAP_TABLE, 3330 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 14877 },
	{ 0x7761, 0x7763, PDF_CMAP_TABLE, 3333 },
	{ 0x7765, 0x7766, PDF_CMAP_TABLE, 3336 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 5824 },
	{ 0x776a, 0x776b, PDF_CMAP_TABLE, 3338 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14879 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 5830 },
	{ 0x777d, 0x777f, PDF_CMAP_TABLE, 3340 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 5832 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 5831 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 5833 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 14881 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 5835 },
	{ 0x77a0, 0x77a0, PDF_CMAP_SINGLE, 5834 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 3613 },
	{ 0x77a9, 0x77aa, PDF_CMAP_TABLE, 3343 },
	{ 0x77ac, 0x77ad, PDF_CMAP_TABLE, 3345 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 5836 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 3215 },
	{ 0x77b6, 0x77b6, PDF_CMAP_SINGLE, 5837 },
	{ 0x77b9, 0x77b9, PDF_CMAP_SINGLE, 5838 },
	{ 0x77bb, 0x77bb, PDF_CMAP_SINGLE, 5842 },
	{ 0x77bc, 0x77bd, PDF_CMAP_RANGE, 5840 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 5839 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5843 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5844 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5845 },
	{ 0x77da, 0x77dc, PDF_CMAP_TABLE, 3347 },
	{ 0x77e2, 0x77e3, PDF_CMAP_TABLE, 3350 },
	{ 0x77e5, 0x77e7, PDF_CMAP_TABLE, 3352 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 1763 },
	{ 0x77ed, 0x77f0, PDF_CMAP_TABLE, 3355 },
	{ 0x77f3, 0x77f4, PDF_CMAP_TABLE, 3359 },
	{ 0x77fc, 0x77fc, PDF_CMAP_SINGLE, 5850 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 2093 },
	{ 0x7806, 0x7806, PDF_CMAP_SINGLE, 14886 },
	{ 0x780c, 0x780c, PDF_CMAP_SINGLE, 5851 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 5852 },
	{ 0x7814, 0x7815, PDF_CMAP_TABLE, 3361 },
	{ 0x7820, 0x7822, PDF_CMAP_TABLE, 3363 },
	{ 0x7825, 0x7827, PDF_CMAP_TABLE, 3366 },
	{ 0x782d, 0x782e, PDF_CMAP_RANGE, 14888 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 14890 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 3666 },
	{ 0x7834, 0x7835, PDF_CMAP_TABLE, 3369 },
	{ 0x783a, 0x783a, PDF_CMAP_SINGLE, 3153 },
	{ 0x783f, 0x783f, PDF_CMAP_SINGLE, 2030 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 5856 },
	{ 0x784e, 0x784f, PDF_CMAP_TABLE, 3371 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 15420 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 2483 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 8576 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 14892 },
	{ 0x786b, 0x786c, PDF_CMAP_TABLE, 3373 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 1883 },
	{ 0x7872, 0x7872, PDF_CMAP_SINGLE, 3383 },
	{ 0x7874, 0x7874, PDF_CMAP_SINGLE, 5858 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 8577 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 5860 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 1951 },
	{ 0x7886, 0x7887, PDF_CMAP_TABLE, 3375 },
	{ 0x788c, 0x788e, PDF_CMAP_TABLE, 3377 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3454 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 1234 },
	{ 0x7895, 0x7895, PDF_CMAP_SINGLE, 2140 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 4088 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 5861 },
	{ 0x789e, 0x789e, PDF_CMAP_SINGLE, 14893 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 5863 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 3611 },
	{ 0x78a9, 0x78aa, PDF_CMAP_TABLE, 3380 },
	{ 0x78af, 0x78af, PDF_CMAP_SINGLE, 5866 },
	{ 0x78b5, 0x78b5, PDF_CMAP_SINGLE, 5864 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 1452 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 5872 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 5871 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 2259 },
	{ 0x78c5, 0x78c6, PDF_CMAP_TABLE, 3382 },
	{ 0x78c8, 0x78c8, PDF_CMAP_SINGLE, 14894 },
	{ 0x78ca, 0x78cc, PDF_CMAP_TABLE, 3384 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 14896 },
	{ 0x78d0, 0x78d1, PDF_CMAP_TABLE, 3387 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 5870 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 5877 },
	{ 0x78e0, 0x78e1, PDF_CMAP_RANGE, 14898 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 14897 },
	{ 0x78e7, 0x78e8, PDF_CMAP_TABLE, 3389 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 5875 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 1199 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 14900 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 5879 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 14901 },
	{ 0x78fb, 0x78fb, PDF_CMAP_SINGLE, 14902 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 5878 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2484 },
	{ 0x7907, 0x7907, PDF_CMAP_SINGLE, 5880 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 2757 },
	{ 0x7911, 0x7912, PDF_CMAP_TABLE, 3391 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5883 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 5853 },
	{ 0x792a, 0x792c, PDF_CMAP_TABLE, 3393 },
	{ 0x7930, 0x7931, PDF_CMAP_TABLE, 3396 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 14904 },
	{ 0x793a, 0x793e, PDF_CMAP_TABLE, 3398 },
	{ 0x7940, 0x7941, PDF_CMAP_TABLE, 3403 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 14907 },
	{ 0x7947, 0x7949, PDF_CMAP_TABLE, 3405 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 3870 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 5892 },
	{ 0x7955, 0x7957, PDF_CMAP_TABLE, 3408 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 5890 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 14908 },
	{ 0x795d, 0x7960, PDF_CMAP_TABLE, 3411 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 3296 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 2485 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 3502 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2119 },
	{ 0x7977, 0x7977, PDF_CMAP_SINGLE, 3186 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 5893 },
	{ 0x797f, 0x7981, PDF_CMAP_TABLE, 3415 },
	{ 0x7984, 0x7985, PDF_CMAP_TABLE, 3418 },
	{ 0x798a, 0x798b, PDF_CMAP_TABLE, 3420 },
	{ 0x798d, 0x798f, PDF_CMAP_TABLE, 3422 },
	{ 0x7994, 0x7994, PDF_CMAP_SINGLE, 8582 },
	{ 0x7996, 0x7996, PDF_CMAP_SINGLE, 14911 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 14912 },
	{ 0x799b, 0x799b, PDF_CMAP_SINGLE, 8584 },
	{ 0x799d, 0x799d, PDF_CMAP_SINGLE, 5896 },
	{ 0x79a6, 0x79a7, PDF_CMAP_TABLE, 3425 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5899 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 5900 },
	{ 0x79b0, 0x79b1, PDF_CMAP_TABLE, 3427 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5901 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 14913 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 5902 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 14914 },
	{ 0x79bd, 0x79c1, PDF_CMAP_TABLE, 3429 },
	{ 0x79c9, 0x79cb, PDF_CMAP_TABLE, 3434 },
	{ 0x79d1, 0x79d2, PDF_CMAP_TABLE, 3437 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 5905 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 3455 },
	{ 0x79da, 0x79da, PDF_CMAP_SINGLE, 14916 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 2759 },
	{ 0x79e1, 0x79e1, PDF_CMAP_SINGLE, 5908 },
	{ 0x79e3, 0x79e4, PDF_CMAP_TABLE, 3439 },
	{ 0x79e6, 0x79e7, PDF_CMAP_TABLE, 3441 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 2975 },
	{ 0x79ec, 0x79ec, PDF_CMAP_SINGLE, 5907 },
	{ 0x79f0, 0x79f0, PDF_CMAP_SINGLE, 2486 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 1184 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 1603 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 14917 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 13875 },
	{ 0x7a08, 0x7a09, PDF_CMAP_TABLE, 3443 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 3092 },
	{ 0x7a0d, 0x7a0e, PDF_CMAP_TABLE, 3445 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 14919 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 3769 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 3477 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 5912 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 2963 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 3983 },
	{ 0x7a1e, 0x7a20, PDF_CMAP_TABLE, 3447 },
	{ 0x7a2d, 0x7a2e, PDF_CMAP_TABLE, 3450 },
	{ 0x7a31, 0x7a32, PDF_CMAP_TABLE, 3452 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 5920 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 14922 },
	{ 0x7a3b, 0x7a40, PDF_CMAP_TABLE, 3454 },
	{ 0x7a42, 0x7a43, PDF_CMAP_TABLE, 3460 },
	{ 0x7a45, 0x7a46, PDF_CMAP_TABLE, 3462 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 5923 },
	{ 0x7a4c, 0x7a50, PDF_CMAP_TABLE, 3464 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5922 },
	{ 0x7a5d, 0x7a5d, PDF_CMAP_SINGLE, 14925 },
	{ 0x7a60, 0x7a60, PDF_CMAP_SINGLE, 14926 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5924 },
	{ 0x7a63, 0x7a63, PDF_CMAP_SINGLE, 2527 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5926 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 1453 },
	{ 0x7a6d, 0x7a6d, PDF_CMAP_SINGLE, 14927 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 5928 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 1856 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 1664 },
	{ 0x7a78, 0x7a7a, PDF_CMAP_TABLE, 3469 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 5930 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 2720 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 3237 },
	{ 0x7a83, 0x7a84, PDF_CMAP_TABLE, 3472 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 5931 },
	{ 0x7a92, 0x7a93, PDF_CMAP_TABLE, 3474 },
	{ 0x7a95, 0x7a98, PDF_CMAP_TABLE, 3476 },
	{ 0x7a9f, 0x7aa0, PDF_CMAP_TABLE, 3480 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 14930 },
	{ 0x7aa9, 0x7aaa, PDF_CMAP_TABLE, 3482 },
	{ 0x7aae, 0x7ab0, PDF_CMAP_TABLE, 3484 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 14931 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 5939 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 1232 },
	{ 0x7abb, 0x7abc, PDF_CMAP_RANGE, 14932 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 5942 },
	{ 0x7ac3, 0x7ac8, PDF_CMAP_TABLE, 3487 },
	{ 0x7aca, 0x7acb, PDF_CMAP_TABLE, 3493 },
	{ 0x7acd, 0x7acd, PDF_CMAP_SINGLE, 5946 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 5947 },
	{ 0x7ad1, 0x7ad3, PDF_CMAP_TABLE, 3495 },
	{ 0x7ad5, 0x7ad5, PDF_CMAP_SINGLE, 5948 },
	{ 0x7ad9, 0x7ada, PDF_CMAP_RANGE, 5950 },
	{ 0x7adc, 0x7add, PDF_CMAP_TABLE, 3498 },
	{ 0x7adf, 0x7ae0, PDF_CMAP_TABLE, 3500 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 5953 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 2401 },
	{ 0x7ae5, 0x7ae7, PDF_CMAP_TABLE, 3502 },
	{ 0x7aea, 0x7aeb, PDF_CMAP_TABLE, 3505 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 5956 },
	{ 0x7aef, 0x7af0, PDF_CMAP_TABLE, 3507 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 1693 },
	{ 0x7af8, 0x7afa, PDF_CMAP_TABLE, 3509 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1540 },
	{ 0x7b02, 0x7b02, PDF_CMAP_SINGLE, 5958 },
	{ 0x7b04, 0x7b04, PDF_CMAP_SINGLE, 5971 },
	{ 0x7b06, 0x7b08, PDF_CMAP_TABLE, 3512 },
	{ 0x7b0a, 0x7b0b, PDF_CMAP_TABLE, 3515 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 5959 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 2488 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 14936 },
	{ 0x7b18, 0x7b19, PDF_CMAP_RANGE, 5963 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 3109 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 5965 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 1468 },
	{ 0x7b25, 0x7b28, PDF_CMAP_TABLE, 3517 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 2888 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 14938 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 5962 },
	{ 0x7b35, 0x7b36, PDF_CMAP_TABLE, 3521 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 2155 },
	{ 0x7b45, 0x7b49, PDF_CMAP_TABLE, 3523 },
	{ 0x7b4b, 0x7b54, PDF_CMAP_TABLE, 3528 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 2150 },
	{ 0x7b5d, 0x7b5d, PDF_CMAP_SINGLE, 5993 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 14941 },
	{ 0x7b65, 0x7b65, PDF_CMAP_SINGLE, 5977 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 5979 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 14942 },
	{ 0x7b6c, 0x7b6e, PDF_CMAP_TABLE, 3538 },
	{ 0x7b70, 0x7b71, PDF_CMAP_RANGE, 5980 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 14944 },
	{ 0x7b74, 0x7b75, PDF_CMAP_TABLE, 3541 },
	{ 0x7b7a, 0x7b7a, PDF_CMAP_SINGLE, 5970 },
	{ 0x7b86, 0x7b87, PDF_CMAP_TABLE, 3543 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 5990 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 5987 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 5992 },
	{ 0x7b91, 0x7b92, PDF_CMAP_TABLE, 3545 },
	{ 0x7b94, 0x7b95, PDF_CMAP_TABLE, 3547 },
	{ 0x7b97, 0x7b9a, PDF_CMAP_TABLE, 3549 },
	{ 0x7b9c, 0x7b9f, PDF_CMAP_TABLE, 3553 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 1541 },
	{ 0x7baa, 0x7baa, PDF_CMAP_SINGLE, 2939 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 2721 },
	{ 0x7baf, 0x7baf, PDF_CMAP_SINGLE, 14946 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3382 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 5999 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 3384 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 3557 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 3427 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_TABLE, 3559 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 2969 },
	{ 0x7bcb, 0x7bcc, PDF_CMAP_TABLE, 3561 },
	{ 0x7bcf, 0x7bcf, PDF_CMAP_SINGLE, 5998 },
	{ 0x7bd7, 0x7bd7, PDF_CMAP_SINGLE, 14947 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 14948 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 6001 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 2288 },
	{ 0x7be4, 0x7be6, PDF_CMAP_TABLE, 3563 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 6002 },
	{ 0x7bed, 0x7bed, PDF_CMAP_SINGLE, 4060 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 6011 },
	{ 0x7bf6, 0x7bf7, PDF_CMAP_TABLE, 3566 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 6008 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 6009 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 14949 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 6014 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 14950 },
	{ 0x7c11, 0x7c14, PDF_CMAP_TABLE, 3568 },
	{ 0x7c17, 0x7c17, PDF_CMAP_SINGLE, 6013 },
	{ 0x7c1e, 0x7c21, PDF_CMAP_TABLE, 3572 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 6016 },
	{ 0x7c26, 0x7c27, PDF_CMAP_TABLE, 3576 },
	{ 0x7c2a, 0x7c2b, PDF_CMAP_TABLE, 3578 },
	{ 0x7c31, 0x7c31, PDF_CMAP_SINGLE, 14953 },
	{ 0x7c36, 0x7c38, PDF_CMAP_TABLE, 3580 },
	{ 0x7c3d, 0x7c40, PDF_CMAP_TABLE, 3583 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 6024 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 3587 },
	{ 0x7c4f, 0x7c51, PDF_CMAP_TABLE, 3589 },
	{ 0x7c54, 0x7c54, PDF_CMAP_SINGLE, 6025 },
	{ 0x7c56, 0x7c56, PDF_CMAP_SINGLE, 6032 },
	{ 0x7c58, 0x7c59, PDF_CMAP_TABLE, 3592 },
	{ 0x7c5f, 0x7c60, PDF_CMAP_TABLE, 3594 },
	{ 0x7c64, 0x7c65, PDF_CMAP_TABLE, 3596 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 14957 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 6034 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 14958 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 14959 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 3606 },
	{ 0x7c75, 0x7c75, PDF_CMAP_SINGLE, 6035 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 14094 },
	{ 0x7c7e, 0x7c7e, PDF_CMAP_SINGLE, 3822 },
	{ 0x7c81, 0x7c83, PDF_CMAP_TABLE, 3598 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 3588 },
	{ 0x7c8b, 0x7c8b, PDF_CMAP_SINGLE, 2606 },
	{ 0x7c8d, 0x7c8d, PDF_CMAP_SINGLE, 3772 },
	{ 0x7c90, 0x7c90, PDF_CMAP_SINGLE, 6037 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 3963 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 3370 },
	{ 0x7c97, 0x7c98, PDF_CMAP_TABLE, 3601 },
	{ 0x7c9b, 0x7c9b, PDF_CMAP_SINGLE, 2391 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 1156 },
	{ 0x7ca1, 0x7ca2, PDF_CMAP_TABLE, 3603 },
	{ 0x7ca4, 0x7ca5, PDF_CMAP_TABLE, 3605 },
	{ 0x7ca7, 0x7ca8, PDF_CMAP_TABLE, 3607 },
	{ 0x7cab, 0x7cab, PDF_CMAP_SINGLE, 6041 },
	{ 0x7cad, 0x7cae, PDF_CMAP_TABLE, 3609 },
	{ 0x7cb1, 0x7cb3, PDF_CMAP_TABLE, 3611 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 6048 },
	{ 0x7cbc, 0x7cc0, PDF_CMAP_TABLE, 3614 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 6052 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 6051 },
	{ 0x7cc8, 0x7cc9, PDF_CMAP_RANGE, 14962 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 1926 },
	{ 0x7cce, 0x7cce, PDF_CMAP_SINGLE, 2746 },
	{ 0x7cd2, 0x7cd2, PDF_CMAP_SINGLE, 6054 },
	{ 0x7cd6, 0x7cd9, PDF_CMAP_TABLE, 3619 },
	{ 0x7cdc, 0x7ce0, PDF_CMAP_TABLE, 3623 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 6056 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 3984 },
	{ 0x7ceb, 0x7ceb, PDF_CMAP_SINGLE, 14967 },
	{ 0x7cef, 0x7cef, PDF_CMAP_SINGLE, 6058 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 6059 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 6060 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 6061 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 2227 },
	{ 0x7cfa, 0x7cfb, PDF_CMAP_TABLE, 3628 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 1668 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 1604 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 6064 },
	{ 0x7d04, 0x7d06, PDF_CMAP_TABLE, 3630 },
	{ 0x7d07, 0x7d09, PDF_CMAP_RANGE, 14968 },
	{ 0x7d0a, 0x7d0b, PDF_CMAP_TABLE, 3633 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 3314 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 3493 },
	{ 0x7d13, 0x7d15, PDF_CMAP_TABLE, 3635 },
	{ 0x7d17, 0x7d1d, PDF_CMAP_TABLE, 3638 },
	{ 0x7d20, 0x7d23, PDF_CMAP_TABLE, 3645 },
	{ 0x7d2b, 0x7d2c, PDF_CMAP_TABLE, 3649 },
	{ 0x7d2e, 0x7d30, PDF_CMAP_TABLE, 3651 },
	{ 0x7d32, 0x7d33, PDF_CMAP_TABLE, 3654 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 6073 },
	{ 0x7d39, 0x7d3a, PDF_CMAP_TABLE, 3656 },
	{ 0x7d3f, 0x7d3f, PDF_CMAP_SINGLE, 6072 },
	{ 0x7d41, 0x7d46, PDF_CMAP_TABLE, 3658 },
	{ 0x7d48, 0x7d48, PDF_CMAP_SINGLE, 8591 },
	{ 0x7d4b, 0x7d4c, PDF_CMAP_TABLE, 3664 },
	{ 0x7d4e, 0x7d50, PDF_CMAP_TABLE, 3666 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 14975 },
	{ 0x7d56, 0x7d56, PDF_CMAP_SINGLE, 6076 },
	{ 0x7d59, 0x7d59, PDF_CMAP_SINGLE, 14976 },
	{ 0x7d5b, 0x7d5e, PDF_CMAP_TABLE, 3669 },
	{ 0x7d61, 0x7d63, PDF_CMAP_TABLE, 3673 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 1669 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 6079 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 6080 },
	{ 0x7d71, 0x7d73, PDF_CMAP_TABLE, 3676 },
	{ 0x7d75, 0x7d76, PDF_CMAP_TABLE, 3679 },
	{ 0x7d79, 0x7d7a, PDF_CMAP_TABLE, 3681 },
	{ 0x7d7d, 0x7d7d, PDF_CMAP_SINGLE, 6087 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 14979 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 6084 },
	{ 0x7d8b, 0x7d8c, PDF_CMAP_RANGE, 14980 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 6086 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 6083 },
	{ 0x7d99, 0x7d9c, PDF_CMAP_TABLE, 3683 },
	{ 0x7d9f, 0x7da0, PDF_CMAP_TABLE, 3687 },
	{ 0x7da2, 0x7da3, PDF_CMAP_TABLE, 3689 },
	{ 0x7dab, 0x7db2, PDF_CMAP_TABLE, 3691 },
	{ 0x7db4, 0x7db5, PDF_CMAP_TABLE, 3699 },
	{ 0x7db7, 0x7db8, PDF_CMAP_TABLE, 3701 },
	{ 0x7dba, 0x7dbb, PDF_CMAP_TABLE, 3703 },
	{ 0x7dbd, 0x7dbf, PDF_CMAP_TABLE, 3705 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 6093 },
	{ 0x7dca, 0x7dcc, PDF_CMAP_TABLE, 3708 },
	{ 0x7dcf, 0x7dcf, PDF_CMAP_SINGLE, 2799 },
	{ 0x7dd1, 0x7dd2, PDF_CMAP_TABLE, 3711 },
	{ 0x7dd5, 0x7dd6, PDF_CMAP_TABLE, 3713 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 6103 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 2722 },
	{ 0x7ddc, 0x7dde, PDF_CMAP_TABLE, 3715 },
	{ 0x7de0, 0x7de1, PDF_CMAP_TABLE, 3718 },
	{ 0x7de3, 0x7de4, PDF_CMAP_TABLE, 3720 },
	{ 0x7de8, 0x7de9, PDF_CMAP_TABLE, 3722 },
	{ 0x7deb, 0x7dec, PDF_CMAP_TABLE, 3724 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 1186 },
	{ 0x7df1, 0x7df2, PDF_CMAP_TABLE, 3726 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4037 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 14985 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 6107 },
	{ 0x7e01, 0x7e01, PDF_CMAP_SINGLE, 1297 },
	{ 0x7e04, 0x7e05, PDF_CMAP_TABLE, 3728 },
	{ 0x7e08, 0x7e0b, PDF_CMAP_TABLE, 3730 },
	{ 0x7e11, 0x7e12, PDF_CMAP_TABLE, 3734 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 14988 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 3377 },
	{ 0x7e1e, 0x7e23, PDF_CMAP_TABLE, 3736 },
	{ 0x7e26, 0x7e26, PDF_CMAP_SINGLE, 2382 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 3667 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 2390 },
	{ 0x7e31, 0x7e32, PDF_CMAP_TABLE, 3742 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 6123 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 6126 },
	{ 0x7e39, 0x7e3b, PDF_CMAP_TABLE, 3744 },
	{ 0x7e3d, 0x7e3e, PDF_CMAP_TABLE, 3747 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 3423 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 6125 },
	{ 0x7e46, 0x7e48, PDF_CMAP_TABLE, 3749 },
	{ 0x7e4a, 0x7e4b, PDF_CMAP_TABLE, 3752 },
	{ 0x7e4d, 0x7e4d, PDF_CMAP_SINGLE, 2357 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 8596 },
	{ 0x7e54, 0x7e56, PDF_CMAP_TABLE, 3754 },
	{ 0x7e59, 0x7e5a, PDF_CMAP_RANGE, 6133 },
	{ 0x7e5d, 0x7e5e, PDF_CMAP_TABLE, 3757 },
	{ 0x7e61, 0x7e62, PDF_CMAP_TABLE, 3759 },
	{ 0x7e66, 0x7e67, PDF_CMAP_TABLE, 3761 },
	{ 0x7e69, 0x7e6b, PDF_CMAP_TABLE, 3763 },
	{ 0x7e6d, 0x7e6e, PDF_CMAP_TABLE, 3766 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 1793 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 14993 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 6135 },
	{ 0x7e7b, 0x7e7d, PDF_CMAP_TABLE, 3768 },
	{ 0x7e7f, 0x7e7f, PDF_CMAP_SINGLE, 6144 },
	{ 0x7e82, 0x7e83, PDF_CMAP_TABLE, 3771 },
	{ 0x7e88, 0x7e89, PDF_CMAP_RANGE, 6145 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 8359 },
	{ 0x7e8c, 0x7e92, PDF_CMAP_TABLE, 3773 },
	{ 0x7e93, 0x7e94, PDF_CMAP_RANGE, 6150 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 6152 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 14996 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_RANGE, 6154 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 1544 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 6156 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 6157 },
	{ 0x7f44, 0x7f45, PDF_CMAP_TABLE, 3780 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 8597 },
	{ 0x7f4c, 0x7f4e, PDF_CMAP_RANGE, 6159 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 14998 },
	{ 0x7f50, 0x7f51, PDF_CMAP_RANGE, 6162 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 14999 },
	{ 0x7f54, 0x7f55, PDF_CMAP_TABLE, 3782 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 6166 },
	{ 0x7f5f, 0x7f60, PDF_CMAP_RANGE, 6167 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 15001 },
	{ 0x7f67, 0x7f67, PDF_CMAP_SINGLE, 6171 },
	{ 0x7f68, 0x7f69, PDF_CMAP_RANGE, 6169 },
	{ 0x7f6a, 0x7f6b, PDF_CMAP_TABLE, 3784 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 2964 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 3399 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 2426 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 3331 },
	{ 0x7f77, 0x7f79, PDF_CMAP_TABLE, 3786 },
	{ 0x7f82, 0x7f83, PDF_CMAP_TABLE, 3789 },
	{ 0x7f85, 0x7f88, PDF_CMAP_TABLE, 3791 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 3901 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 6178 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 3474 },
	{ 0x7f91, 0x7f91, PDF_CMAP_SINGLE, 15002 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 6179 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 6182 },
	{ 0x7f9d, 0x7f9e, PDF_CMAP_TABLE, 3795 },
	{ 0x7fa1, 0x7fa1, PDF_CMAP_SINGLE, 8598 },
	{ 0x7fa3, 0x7fa4, PDF_CMAP_TABLE, 3797 },
	{ 0x7fa8, 0x7fa9, PDF_CMAP_TABLE, 3799 },
	{ 0x7fae, 0x7faf, PDF_CMAP_TABLE, 3801 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 6185 },
	{ 0x7fb6, 0x7fb6, PDF_CMAP_SINGLE, 6188 },
	{ 0x7fb8, 0x7fb9, PDF_CMAP_TABLE, 3803 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 1227 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 15003 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 1319 },
	{ 0x7fc5, 0x7fc6, PDF_CMAP_RANGE, 6191 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 6193 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 3916 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 15004 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2358 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_TABLE, 3805 },
	{ 0x7fdf, 0x7fe1, PDF_CMAP_TABLE, 3807 },
	{ 0x7fe5, 0x7fe6, PDF_CMAP_TABLE, 3810 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 6198 },
	{ 0x7feb, 0x7fec, PDF_CMAP_TABLE, 3812 },
	{ 0x7fee, 0x7fef, PDF_CMAP_RANGE, 15008 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 1545 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 6199 },
	{ 0x7ff9, 0x7ffc, PDF_CMAP_TABLE, 3814 },
	{ 0x8000, 0x8006, PDF_CMAP_TABLE, 3818 },
	{ 0x800b, 0x800c, PDF_CMAP_TABLE, 3825 },
	{ 0x800e, 0x800e, PDF_CMAP_SINGLE, 15011 },
	{ 0x8010, 0x8012, PDF_CMAP_TABLE, 3827 },
	{ 0x8014, 0x8015, PDF_CMAP_TABLE, 3830 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 3811 },
	{ 0x8018, 0x8019, PDF_CMAP_RANGE, 6206 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 6208 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 6209 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 15014 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 15015 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 6210 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 2262 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 3833 },
	{ 0x803a, 0x803d, PDF_CMAP_TABLE, 3832 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 6211 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 6214 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 6213 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 6215 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 2655 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 6216 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 6217 },
	{ 0x805e, 0x8062, PDF_CMAP_TABLE, 3836 },
	{ 0x8068, 0x8068, PDF_CMAP_SINGLE, 6220 },
	{ 0x806f, 0x8077, PDF_CMAP_TABLE, 3841 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 6225 },
	{ 0x807d, 0x807f, PDF_CMAP_TABLE, 3850 },
	{ 0x8084, 0x8087, PDF_CMAP_TABLE, 3853 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 3281 },
	{ 0x808b, 0x808c, PDF_CMAP_TABLE, 3857 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6232 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 2491 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 3484 },
	{ 0x809a, 0x809b, PDF_CMAP_TABLE, 3859 },
	{ 0x809d, 0x809e, PDF_CMAP_TABLE, 3861 },
	{ 0x80a1, 0x80a2, PDF_CMAP_TABLE, 3863 },
	{ 0x80a5, 0x80a6, PDF_CMAP_TABLE, 3865 },
	{ 0x80a9, 0x80ad, PDF_CMAP_TABLE, 3867 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 2016 },
	{ 0x80b1, 0x80b2, PDF_CMAP_TABLE, 3872 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 2136 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 3343 },
	{ 0x80c3, 0x80c4, PDF_CMAP_TABLE, 3874 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 2942 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 3342 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 2875 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 6243 },
	{ 0x80d7, 0x80d8, PDF_CMAP_RANGE, 15024 },
	{ 0x80d9, 0x80db, PDF_CMAP_TABLE, 3876 },
	{ 0x80dd, 0x80de, PDF_CMAP_TABLE, 3879 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1929 },
	{ 0x80e4, 0x80e5, PDF_CMAP_TABLE, 3881 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 6245 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 6246 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 3217 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 1715 },
	{ 0x80fc, 0x80fd, PDF_CMAP_TABLE, 3883 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 2231 },
	{ 0x8105, 0x810a, PDF_CMAP_TABLE, 3885 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 15026 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 15027 },
	{ 0x811a, 0x811b, PDF_CMAP_TABLE, 3891 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 6249 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 6248 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 13913 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 6250 },
	{ 0x8131, 0x8131, PDF_CMAP_SINGLE, 2916 },
	{ 0x8133, 0x8133, PDF_CMAP_SINGLE, 3316 },
	{ 0x8139, 0x813a, PDF_CMAP_TABLE, 3893 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 6254 },
	{ 0x8141, 0x8141, PDF_CMAP_SINGLE, 14194 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 6253 },
	{ 0x814a, 0x814c, PDF_CMAP_TABLE, 3895 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 2587 },
	{ 0x8150, 0x8151, PDF_CMAP_TABLE, 3898 },
	{ 0x8153, 0x8155, PDF_CMAP_TABLE, 3900 },
	{ 0x815f, 0x815f, PDF_CMAP_SINGLE, 6272 },
	{ 0x8165, 0x8166, PDF_CMAP_RANGE, 6260 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 2332 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 6259 },
	{ 0x8170, 0x8171, PDF_CMAP_TABLE, 3903 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6262 },
	{ 0x8178, 0x817a, PDF_CMAP_TABLE, 3905 },
	{ 0x817f, 0x8184, PDF_CMAP_TABLE, 3908 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 6264 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 6265 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 2019 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 6273 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 6269 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 3544 },
	{ 0x819c, 0x819d, PDF_CMAP_TABLE, 3914 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 6268 },
	{ 0x81a3, 0x81a4, PDF_CMAP_TABLE, 3916 },
	{ 0x81a8, 0x81a9, PDF_CMAP_TABLE, 3918 },
	{ 0x81b0, 0x81b0, PDF_CMAP_SINGLE, 6275 },
	{ 0x81b3, 0x81b5, PDF_CMAP_TABLE, 3920 },
	{ 0x81b8, 0x81b8, PDF_CMAP_SINGLE, 6278 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 6282 },
	{ 0x81bd, 0x81c0, PDF_CMAP_TABLE, 3923 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 6281 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 1330 },
	{ 0x81c8, 0x81c9, PDF_CMAP_TABLE, 3927 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 6284 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 15034 },
	{ 0x81d1, 0x81d1, PDF_CMAP_SINGLE, 6285 },
	{ 0x81d3, 0x81d3, PDF_CMAP_SINGLE, 2817 },
	{ 0x81d8, 0x81da, PDF_CMAP_TABLE, 3929 },
	{ 0x81df, 0x81e0, PDF_CMAP_RANGE, 6290 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 2569 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 1385 },
	{ 0x81e7, 0x81e8, PDF_CMAP_TABLE, 3932 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 2263 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 2359 },
	{ 0x81f3, 0x81f4, PDF_CMAP_TABLE, 3934 },
	{ 0x81f9, 0x81f9, PDF_CMAP_SINGLE, 15035 },
	{ 0x81fa, 0x81fb, PDF_CMAP_RANGE, 6293 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 1235 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 6295 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 6296 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 15036 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 6298 },
	{ 0x8207, 0x820a, PDF_CMAP_TABLE, 3936 },
	{ 0x820c, 0x820e, PDF_CMAP_TABLE, 3940 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 6302 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 4105 },
	{ 0x8216, 0x8218, PDF_CMAP_TABLE, 3943 },
	{ 0x821b, 0x821c, PDF_CMAP_TABLE, 3946 },
	{ 0x821e, 0x821f, PDF_CMAP_TABLE, 3948 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 15037 },
	{ 0x8229, 0x822c, PDF_CMAP_TABLE, 3950 },
	{ 0x822e, 0x822e, PDF_CMAP_SINGLE, 6319 },
	{ 0x8232, 0x8239, PDF_CMAP_TABLE, 3954 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 6308 },
	{ 0x8246, 0x8247, PDF_CMAP_TABLE, 3962 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 15041 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 15042 },
	{ 0x8258, 0x825a, PDF_CMAP_TABLE, 3964 },
	{ 0x825d, 0x825d, PDF_CMAP_SINGLE, 6311 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 6313 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 6315 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 6314 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 1547 },
	{ 0x8268, 0x8268, PDF_CMAP_SINGLE, 6316 },
	{ 0x826a, 0x826b, PDF_CMAP_RANGE, 6317 },
	{ 0x826e, 0x826f, PDF_CMAP_TABLE, 3967 },
	{ 0x8271, 0x8272, PDF_CMAP_TABLE, 3969 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 1298 },
	{ 0x8277, 0x8278, PDF_CMAP_RANGE, 6321 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 14197 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 6323 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 1206 },
	{ 0x828d, 0x828e, PDF_CMAP_TABLE, 3971 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 6325 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 3545 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 2291 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 6327 },
	{ 0x82a5, 0x82a6, PDF_CMAP_TABLE, 3973 },
	{ 0x82ab, 0x82af, PDF_CMAP_TABLE, 3975 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1366 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 3669 },
	{ 0x82b7, 0x82b9, PDF_CMAP_TABLE, 3980 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 6328 },
	{ 0x82bd, 0x82be, PDF_CMAP_TABLE, 3983 },
	{ 0x82c5, 0x82c6, PDF_CMAP_TABLE, 3985 },
	{ 0x82d1, 0x82d4, PDF_CMAP_TABLE, 3987 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 3510 },
	{ 0x82d9, 0x82d9, PDF_CMAP_SINGLE, 6345 },
	{ 0x82db, 0x82dc, PDF_CMAP_TABLE, 3991 },
	{ 0x82de, 0x82df, PDF_CMAP_TABLE, 3993 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 6330 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 6331 },
	{ 0x82e5, 0x82e7, PDF_CMAP_TABLE, 3995 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3241 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 1267 },
	{ 0x82f3, 0x82f4, PDF_CMAP_TABLE, 3998 },
	{ 0x82f9, 0x82fb, PDF_CMAP_TABLE, 4000 },
	{ 0x82fe, 0x82fe, PDF_CMAP_SINGLE, 15048 },
	{ 0x8301, 0x8306, PDF_CMAP_TABLE, 4003 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 6344 },
	{ 0x830e, 0x830e, PDF_CMAP_SINGLE, 1834 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 6348 },
	{ 0x8317, 0x8318, PDF_CMAP_RANGE, 6357 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 1135 },
	{ 0x8323, 0x8323, PDF_CMAP_SINGLE, 6365 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 1205 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 6356 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 6355 },
	{ 0x8331, 0x8332, PDF_CMAP_TABLE, 4009 },
	{ 0x8334, 0x8336, PDF_CMAP_TABLE, 4011 },
	{ 0x8338, 0x8339, PDF_CMAP_TABLE, 4014 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 6351 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 15049 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 6354 },
	{ 0x8349, 0x834a, PDF_CMAP_TABLE, 4016 },
	{ 0x834f, 0x8352, PDF_CMAP_TABLE, 4018 },
	{ 0x8355, 0x8355, PDF_CMAP_SINGLE, 15051 },
	{ 0x8358, 0x8358, PDF_CMAP_SINGLE, 2803 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 8601 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 6371 },
	{ 0x8375, 0x8375, PDF_CMAP_SINGLE, 6372 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 1369 },
	{ 0x837b, 0x837c, PDF_CMAP_TABLE, 4022 },
	{ 0x837f, 0x837f, PDF_CMAP_SINGLE, 8602 },
	{ 0x8385, 0x8387, PDF_CMAP_TABLE, 4024 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 4027 },
	{ 0x838d, 0x838e, PDF_CMAP_TABLE, 4029 },
	{ 0x8392, 0x8393, PDF_CMAP_TABLE, 4031 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 6364 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 15055 },
	{ 0x839a, 0x839a, PDF_CMAP_SINGLE, 6360 },
	{ 0x839e, 0x83a0, PDF_CMAP_TABLE, 4033 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 6363 },
	{ 0x83a8, 0x83ab, PDF_CMAP_TABLE, 4036 },
	{ 0x83b1, 0x83b1, PDF_CMAP_SINGLE, 3923 },
	{ 0x83b5, 0x83b5, PDF_CMAP_SINGLE, 6370 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 6392 },
	{ 0x83bf, 0x83c0, PDF_CMAP_RANGE, 15057 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 6384 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 2625 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 8603 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 1632 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 1749 },
	{ 0x83ce, 0x83ce, PDF_CMAP_SINGLE, 6379 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 1371 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 2492 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 6382 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 2122 },
	{ 0x83df, 0x83e0, PDF_CMAP_TABLE, 4040 },
	{ 0x83e9, 0x83eb, PDF_CMAP_TABLE, 4042 },
	{ 0x83ef, 0x83f2, PDF_CMAP_TABLE, 4045 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 6376 },
	{ 0x83f6, 0x83f7, PDF_CMAP_TABLE, 4049 },
	{ 0x83fb, 0x83fb, PDF_CMAP_SINGLE, 6395 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 6380 },
	{ 0x8403, 0x8404, PDF_CMAP_TABLE, 4051 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 6386 },
	{ 0x840a, 0x840f, PDF_CMAP_TABLE, 4053 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 15061 },
	{ 0x8413, 0x8413, PDF_CMAP_SINGLE, 6377 },
	{ 0x8420, 0x8420, PDF_CMAP_SINGLE, 6391 },
	{ 0x8422, 0x8422, PDF_CMAP_SINGLE, 6390 },
	{ 0x8429, 0x842a, PDF_CMAP_TABLE, 4059 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 6408 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 1500 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 6411 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 6393 },
	{ 0x843c, 0x843d, PDF_CMAP_TABLE, 4061 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 6407 },
	{ 0x8448, 0x844a, PDF_CMAP_TABLE, 4063 },
	{ 0x844e, 0x844e, PDF_CMAP_SINGLE, 3954 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 2998 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 1481 },
	{ 0x8461, 0x8463, PDF_CMAP_TABLE, 4066 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 1141 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 6406 },
	{ 0x846b, 0x846f, PDF_CMAP_TABLE, 4069 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 3298 },
	{ 0x8475, 0x8477, PDF_CMAP_TABLE, 4074 },
	{ 0x8479, 0x847a, PDF_CMAP_TABLE, 4077 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 6405 },
	{ 0x8484, 0x8484, PDF_CMAP_SINGLE, 6400 },
	{ 0x848b, 0x848b, PDF_CMAP_SINGLE, 2493 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 2361 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 2264 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 3812 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3513 },
	{ 0x849f, 0x849f, PDF_CMAP_SINGLE, 6416 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 6425 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 15064 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 6403 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 15065 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 1493 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 8606 },
	{ 0x84b8, 0x84b9, PDF_CMAP_TABLE, 4079 },
	{ 0x84bb, 0x84bc, PDF_CMAP_TABLE, 4081 },
	{ 0x84bf, 0x84c2, PDF_CMAP_TABLE, 4083 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 2973 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 6423 },
	{ 0x84c9, 0x84cb, PDF_CMAP_TABLE, 4087 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 6418 },
	{ 0x84d0, 0x84d1, PDF_CMAP_TABLE, 4090 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 6424 },
	{ 0x84d9, 0x84da, PDF_CMAP_TABLE, 4092 },
	{ 0x84dc, 0x84dc, PDF_CMAP_SINGLE, 8363 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 3671 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4039 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 15068 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 6428 },
	{ 0x84fc, 0x84fd, PDF_CMAP_TABLE, 4094 },
	{ 0x84ff, 0x8500, PDF_CMAP_TABLE, 4096 },
	{ 0x8506, 0x8506, PDF_CMAP_SINGLE, 6394 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 15070 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 3614 },
	{ 0x8513, 0x8515, PDF_CMAP_TABLE, 4098 },
	{ 0x8517, 0x8518, PDF_CMAP_RANGE, 6429 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 1240 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 6432 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 6426 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 7706 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 3057 },
	{ 0x852c, 0x852d, PDF_CMAP_TABLE, 4101 },
	{ 0x8534, 0x8535, PDF_CMAP_TABLE, 4103 },
	{ 0x853d, 0x853e, PDF_CMAP_TABLE, 4105 },
	{ 0x8540, 0x8541, PDF_CMAP_TABLE, 4107 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 3437 },
	{ 0x8548, 0x854b, PDF_CMAP_TABLE, 4109 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 1718 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 8607 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 6443 },
	{ 0x8557, 0x855a, PDF_CMAP_TABLE, 4113 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 15072 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 6437 },
	{ 0x8568, 0x856b, PDF_CMAP_TABLE, 4117 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 6450 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 6456 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 6457 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 6444 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 3372 },
	{ 0x8587, 0x8588, PDF_CMAP_TABLE, 4121 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 6448 },
	{ 0x858f, 0x8591, PDF_CMAP_TABLE, 4123 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 6451 },
	{ 0x8597, 0x8597, PDF_CMAP_SINGLE, 1300 },
	{ 0x8599, 0x8599, PDF_CMAP_SINGLE, 3261 },
	{ 0x859b, 0x859c, PDF_CMAP_TABLE, 4126 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 6445 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 2728 },
	{ 0x85a8, 0x85b0, PDF_CMAP_TABLE, 4128 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 15074 },
	{ 0x85b9, 0x85ba, PDF_CMAP_TABLE, 4137 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 4084 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 6459 },
	{ 0x85cd, 0x85d0, PDF_CMAP_TABLE, 4139 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 6464 },
	{ 0x85dc, 0x85dd, PDF_CMAP_TABLE, 4143 },
	{ 0x85e4, 0x85e5, PDF_CMAP_TABLE, 4145 },
	{ 0x85e9, 0x85ea, PDF_CMAP_TABLE, 4147 },
	{ 0x85f4, 0x85f4, PDF_CMAP_SINGLE, 14208 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 2429 },
	{ 0x85f9, 0x85fb, PDF_CMAP_TABLE, 4149 },
	{ 0x85fe, 0x85fe, PDF_CMAP_SINGLE, 6472 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 6441 },
	{ 0x8606, 0x8607, PDF_CMAP_TABLE, 4152 },
	{ 0x860a, 0x860b, PDF_CMAP_TABLE, 4154 },
	{ 0x8612, 0x8613, PDF_CMAP_TABLE, 4156 },
	{ 0x8616, 0x8617, PDF_CMAP_TABLE, 4158 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 6476 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 6475 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 15078 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 3936 },
	{ 0x862f, 0x8630, PDF_CMAP_TABLE, 4160 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 6478 },
	{ 0x864d, 0x864e, PDF_CMAP_TABLE, 4162 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 1646 },
	{ 0x8652, 0x8652, PDF_CMAP_SINGLE, 15079 },
	{ 0x8654, 0x8655, PDF_CMAP_TABLE, 4164 },
	{ 0x865a, 0x865c, PDF_CMAP_TABLE, 4166 },
	{ 0x865e, 0x865f, PDF_CMAP_TABLE, 4169 },
	{ 0x8663, 0x8663, PDF_CMAP_SINGLE, 15080 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 6483 },
	{ 0x866b, 0x866c, PDF_CMAP_TABLE, 4171 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 15082 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 6484 },
	{ 0x8679, 0x867b, PDF_CMAP_TABLE, 4173 },
	{ 0x867d, 0x867d, PDF_CMAP_SINGLE, 14214 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 1379 },
	{ 0x868b, 0x868c, PDF_CMAP_RANGE, 6489 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 15084 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 15085 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 6485 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 2187 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 15086 },
	{ 0x86a3, 0x86a4, PDF_CMAP_TABLE, 4176 },
	{ 0x86a7, 0x86a8, PDF_CMAP_RANGE, 15087 },
	{ 0x86a9, 0x86aa, PDF_CMAP_RANGE, 6487 },
	{ 0x86ab, 0x86ab, PDF_CMAP_SINGLE, 6498 },
	{ 0x86af, 0x86b0, PDF_CMAP_TABLE, 4178 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 6491 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 6493 },
	{ 0x86c6, 0x86c7, PDF_CMAP_TABLE, 4180 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 6496 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 2943 },
	{ 0x86cd, 0x86ce, PDF_CMAP_TABLE, 4182 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 6499 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 1437 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 6504 },
	{ 0x86de, 0x86df, PDF_CMAP_TABLE, 4184 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 3406 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 6501 },
	{ 0x86ec, 0x86ef, PDF_CMAP_TABLE, 4186 },
	{ 0x86f8, 0x86fb, PDF_CMAP_TABLE, 4190 },
	{ 0x86fd, 0x86fe, PDF_CMAP_TABLE, 4194 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 6509 },
	{ 0x8702, 0x8703, PDF_CMAP_TABLE, 4196 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 6507 },
	{ 0x8708, 0x870b, PDF_CMAP_TABLE, 4198 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 6514 },
	{ 0x8711, 0x8713, PDF_CMAP_TABLE, 4202 },
	{ 0x8718, 0x871a, PDF_CMAP_TABLE, 4205 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 3766 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 15094 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 6521 },
	{ 0x8728, 0x8729, PDF_CMAP_TABLE, 4208 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 6517 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 6519 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 6520 },
	{ 0x873e, 0x873f, PDF_CMAP_TABLE, 4210 },
	{ 0x8749, 0x8749, PDF_CMAP_SINGLE, 2698 },
	{ 0x874b, 0x874c, PDF_CMAP_TABLE, 4212 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 6528 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 6534 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 2544 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 6530 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 6533 },
	{ 0x875f, 0x8760, PDF_CMAP_TABLE, 4214 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 6535 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 1372 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 6531 },
	{ 0x876a, 0x876a, PDF_CMAP_SINGLE, 6536 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 6532 },
	{ 0x8771, 0x8771, PDF_CMAP_SINGLE, 15097 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 6529 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 3023 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 6526 },
	{ 0x877f, 0x877f, PDF_CMAP_SINGLE, 3358 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 6540 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 15098 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 3877 },
	{ 0x8799, 0x8799, PDF_CMAP_SINGLE, 15099 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 6539 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 6538 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 6547 },
	{ 0x87ac, 0x87ad, PDF_CMAP_RANGE, 15100 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 6541 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 6549 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 15102 },
	{ 0x87ba, 0x87bb, PDF_CMAP_TABLE, 4216 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 6543 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 6544 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 6548 },
	{ 0x87c6, 0x87c7, PDF_CMAP_TABLE, 4218 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 6542 },
	{ 0x87d0, 0x87d0, PDF_CMAP_SINGLE, 6545 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 6562 },
	{ 0x87d6, 0x87d6, PDF_CMAP_SINGLE, 15103 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 6555 },
	{ 0x87eb, 0x87ed, PDF_CMAP_TABLE, 4220 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 6553 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 6554 },
	{ 0x87f5, 0x87f5, PDF_CMAP_SINGLE, 15421 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 6559 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 1416 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 1628 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 6558 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 15106 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 15107 },
	{ 0x8805, 0x8807, PDF_CMAP_TABLE, 4223 },
	{ 0x880b, 0x880b, PDF_CMAP_SINGLE, 15109 },
	{ 0x880d, 0x880f, PDF_CMAP_TABLE, 4226 },
	{ 0x8811, 0x8811, PDF_CMAP_SINGLE, 6563 },
	{ 0x8814, 0x8816, PDF_CMAP_TABLE, 4229 },
	{ 0x881c, 0x881c, PDF_CMAP_SINGLE, 15111 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 7813 },
	{ 0x8821, 0x8823, PDF_CMAP_TABLE, 4232 },
	{ 0x8827, 0x8827, PDF_CMAP_SINGLE, 6571 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 6568 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 6569 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 6570 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 6572 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 1858 },
	{ 0x8842, 0x8842, PDF_CMAP_SINGLE, 6574 },
	{ 0x8844, 0x8844, PDF_CMAP_SINGLE, 6573 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 2362 },
	{ 0x884c, 0x884d, PDF_CMAP_TABLE, 4235 },
	{ 0x8852, 0x8853, PDF_CMAP_TABLE, 4237 },
	{ 0x8856, 0x8857, PDF_CMAP_TABLE, 4239 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 6576 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 1268 },
	{ 0x885d, 0x885f, PDF_CMAP_TABLE, 4241 },
	{ 0x8861, 0x8864, PDF_CMAP_TABLE, 4244 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 3503 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 6579 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 2608 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 6586 },
	{ 0x8875, 0x8875, PDF_CMAP_SINGLE, 6583 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2989 },
	{ 0x887d, 0x887f, PDF_CMAP_TABLE, 4248 },
	{ 0x8881, 0x8882, PDF_CMAP_TABLE, 4251 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 1804 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 2878 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 6593 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 6589 },
	{ 0x8896, 0x8899, PDF_CMAP_TABLE, 4253 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 6582 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 6592 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 6594 },
	{ 0x88aa, 0x88ab, PDF_CMAP_TABLE, 4257 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 6590 },
	{ 0x88b0, 0x88b1, PDF_CMAP_TABLE, 4259 },
	{ 0x88b4, 0x88b5, PDF_CMAP_TABLE, 4261 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 1157 },
	{ 0x88bd, 0x88be, PDF_CMAP_RANGE, 15117 },
	{ 0x88bf, 0x88bf, PDF_CMAP_SINGLE, 6596 },
	{ 0x88c1, 0x88c2, PDF_CMAP_TABLE, 4263 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 6598 },
	{ 0x88c5, 0x88c6, PDF_CMAP_TABLE, 4265 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 15119 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 3946 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 15120 },
	{ 0x88d4, 0x88d5, PDF_CMAP_TABLE, 4267 },
	{ 0x88d8, 0x88d9, PDF_CMAP_RANGE, 6601 },
	{ 0x88db, 0x88dd, PDF_CMAP_TABLE, 4269 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 2096 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3947 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 6608 },
	{ 0x88f0, 0x88f1, PDF_CMAP_RANGE, 15122 },
	{ 0x88f2, 0x88f5, PDF_CMAP_TABLE, 4272 },
	{ 0x88f8, 0x88f9, PDF_CMAP_TABLE, 4276 },
	{ 0x88fc, 0x88fe, PDF_CMAP_TABLE, 4278 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 6605 },
	{ 0x8904, 0x8904, PDF_CMAP_SINGLE, 6610 },
	{ 0x8906, 0x8907, PDF_CMAP_TABLE, 4281 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 6612 },
	{ 0x890c, 0x890c, PDF_CMAP_SINGLE, 6611 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 1482 },
	{ 0x8912, 0x8913, PDF_CMAP_TABLE, 4283 },
	{ 0x8918, 0x891a, PDF_CMAP_RANGE, 15125 },
	{ 0x891c, 0x891e, PDF_CMAP_TABLE, 4285 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 6616 },
	{ 0x8927, 0x8927, PDF_CMAP_SINGLE, 15128 },
	{ 0x892a, 0x892b, PDF_CMAP_RANGE, 6617 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 15129 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 6622 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 6623 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 6621 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 15130 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 6619 },
	{ 0x8943, 0x8944, PDF_CMAP_TABLE, 4288 },
	{ 0x894c, 0x894d, PDF_CMAP_TABLE, 4290 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 1320 },
	{ 0x895e, 0x8960, PDF_CMAP_TABLE, 4292 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 6629 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 6628 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 6631 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 6630 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 6632 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 2363 },
	{ 0x8974, 0x8974, PDF_CMAP_SINGLE, 6633 },
	{ 0x8977, 0x8977, PDF_CMAP_SINGLE, 6634 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 15131 },
	{ 0x897e, 0x8981, PDF_CMAP_TABLE, 4295 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 6636 },
	{ 0x8986, 0x8988, PDF_CMAP_TABLE, 4299 },
	{ 0x898a, 0x898b, PDF_CMAP_TABLE, 4302 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 1606 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 6639 },
	{ 0x8996, 0x8998, PDF_CMAP_TABLE, 4304 },
	{ 0x899a, 0x899a, PDF_CMAP_SINGLE, 1454 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 6641 },
	{ 0x89a6, 0x89a7, PDF_CMAP_TABLE, 4307 },
	{ 0x89a9, 0x89aa, PDF_CMAP_TABLE, 4309 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 6644 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 6645 },
	{ 0x89b2, 0x89b3, PDF_CMAP_TABLE, 4311 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 6647 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 6648 },
	{ 0x89bf, 0x89c0, PDF_CMAP_RANGE, 6649 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 1455 },
	{ 0x89d4, 0x89d4, PDF_CMAP_SINGLE, 15132 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 15133 },
	{ 0x89da, 0x89da, PDF_CMAP_SINGLE, 6651 },
	{ 0x89dc, 0x89dd, PDF_CMAP_RANGE, 6652 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 1394 },
	{ 0x89e5, 0x89e7, PDF_CMAP_TABLE, 4313 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 15135 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 6655 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 6656 },
	{ 0x8a00, 0x8a03, PDF_CMAP_TABLE, 4316 },
	{ 0x8a07, 0x8a08, PDF_CMAP_TABLE, 4320 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2588 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 6660 },
	{ 0x8a0e, 0x8a10, PDF_CMAP_TABLE, 4322 },
	{ 0x8a12, 0x8a13, PDF_CMAP_TABLE, 4325 },
	{ 0x8a15, 0x8a18, PDF_CMAP_TABLE, 4327 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 6661 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 6662 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a22, 0x8a23, PDF_CMAP_TABLE, 4331 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 6663 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 3674 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 2691 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 1680 },
	{ 0x8a33, 0x8a34, PDF_CMAP_TABLE, 4333 },
	{ 0x8a36, 0x8a37, PDF_CMAP_TABLE, 4335 },
	{ 0x8a3a, 0x8a3c, PDF_CMAP_TABLE, 4337 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 6665 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 6668 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 6669 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 15140 },
	{ 0x8a50, 0x8a52, PDF_CMAP_TABLE, 4340 },
	{ 0x8a54, 0x8a55, PDF_CMAP_TABLE, 4343 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 6666 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 2234 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 1269 },
	{ 0x8a62, 0x8a63, PDF_CMAP_TABLE, 4345 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 2236 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 2235 },
	{ 0x8a6b, 0x8a6e, PDF_CMAP_TABLE, 4347 },
	{ 0x8a70, 0x8a73, PDF_CMAP_TABLE, 4351 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8618 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 6670 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 15141 },
	{ 0x8a82, 0x8a82, PDF_CMAP_SINGLE, 6675 },
	{ 0x8a84, 0x8a85, PDF_CMAP_TABLE, 4355 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 1932 },
	{ 0x8a89, 0x8a89, PDF_CMAP_SINGLE, 3882 },
	{ 0x8a8c, 0x8a8d, PDF_CMAP_TABLE, 4357 },
	{ 0x8a91, 0x8a91, PDF_CMAP_SINGLE, 6679 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 2660 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 2944 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 3872 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 6682 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 1952 },
	{ 0x8aa0, 0x8aa1, PDF_CMAP_TABLE, 4359 },
	{ 0x8aa3, 0x8aa4, PDF_CMAP_TABLE, 4361 },
	{ 0x8aa5, 0x8aa6, PDF_CMAP_RANGE, 6680 },
	{ 0x8aa7, 0x8aa8, PDF_CMAP_TABLE, 4363 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 13880 },
	{ 0x8aac, 0x8aad, PDF_CMAP_TABLE, 4365 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 2925 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 1373 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 3460 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 1629 },
	{ 0x8abe, 0x8abf, PDF_CMAP_TABLE, 4367 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 6686 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 6684 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 2954 },
	{ 0x8acb, 0x8acd, PDF_CMAP_TABLE, 4369 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 2593 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 3986 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4070 },
	{ 0x8ada, 0x8adc, PDF_CMAP_TABLE, 4372 },
	{ 0x8ade, 0x8ae2, PDF_CMAP_TABLE, 4375 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 6691 },
	{ 0x8ae6, 0x8ae7, PDF_CMAP_TABLE, 4380 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 6688 },
	{ 0x8aed, 0x8aee, PDF_CMAP_TABLE, 4382 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 6692 },
	{ 0x8af3, 0x8af4, PDF_CMAP_TABLE, 4384 },
	{ 0x8af6, 0x8af8, PDF_CMAP_TABLE, 4386 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 1909 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 2906 },
	{ 0x8b00, 0x8b02, PDF_CMAP_TABLE, 4389 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 3197 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 6700 },
	{ 0x8b0c, 0x8b0c, PDF_CMAP_SINGLE, 6699 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 3262 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 6704 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 6693 },
	{ 0x8b16, 0x8b17, PDF_CMAP_TABLE, 4392 },
	{ 0x8b19, 0x8b1b, PDF_CMAP_TABLE, 4394 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 2305 },
	{ 0x8b1f, 0x8b21, PDF_CMAP_TABLE, 4397 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 6709 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 6712 },
	{ 0x8b2b, 0x8b2c, PDF_CMAP_TABLE, 4400 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 6707 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 15144 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 1752 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 6711 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 6713 },
	{ 0x8b43, 0x8b44, PDF_CMAP_RANGE, 15145 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 6717 },
	{ 0x8b4c, 0x8b4c, PDF_CMAP_SINGLE, 6714 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_TABLE, 4402 },
	{ 0x8b53, 0x8b54, PDF_CMAP_TABLE, 4404 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 6718 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 2269 },
	{ 0x8b5a, 0x8b5c, PDF_CMAP_TABLE, 4406 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 6722 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 1839 },
	{ 0x8b6b, 0x8b6c, PDF_CMAP_TABLE, 4409 },
	{ 0x8b6f, 0x8b72, PDF_CMAP_TABLE, 4411 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 6725 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 1954 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 6726 },
	{ 0x8b7f, 0x8b80, PDF_CMAP_TABLE, 4415 },
	{ 0x8b83, 0x8b83, PDF_CMAP_SINGLE, 2188 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5075 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 6728 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 6729 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 2364 },
	{ 0x8b92, 0x8b93, PDF_CMAP_RANGE, 6730 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 6732 },
	{ 0x8b99, 0x8b9a, PDF_CMAP_RANGE, 6733 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 15148 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 15149 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 2921 },
	{ 0x8c3a, 0x8c3a, PDF_CMAP_SINGLE, 6735 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 6737 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 6736 },
	{ 0x8c46, 0x8c48, PDF_CMAP_TABLE, 4417 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 3675 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 6739 },
	{ 0x8c4e, 0x8c4e, PDF_CMAP_SINGLE, 6740 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 6741 },
	{ 0x8c54, 0x8c55, PDF_CMAP_TABLE, 4420 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 3250 },
	{ 0x8c61, 0x8c62, PDF_CMAP_TABLE, 4422 },
	{ 0x8c6a, 0x8c6c, PDF_CMAP_TABLE, 4424 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 15152 },
	{ 0x8c78, 0x8c7a, PDF_CMAP_TABLE, 4427 },
	{ 0x8c7c, 0x8c7c, PDF_CMAP_SINGLE, 6754 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 6747 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 6749 },
	{ 0x8c89, 0x8c8a, PDF_CMAP_TABLE, 4430 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 3700 },
	{ 0x8c8d, 0x8c8e, PDF_CMAP_RANGE, 6751 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 6753 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 6755 },
	{ 0x8c9d, 0x8c9e, PDF_CMAP_TABLE, 4432 },
	{ 0x8ca0, 0x8ca2, PDF_CMAP_TABLE, 4434 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 15153 },
	{ 0x8ca7, 0x8cb0, PDF_CMAP_TABLE, 4437 },
	{ 0x8cb2, 0x8cb3, PDF_CMAP_RANGE, 6760 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 1608 },
	{ 0x8cb6, 0x8cb8, PDF_CMAP_TABLE, 4447 },
	{ 0x8cbb, 0x8cbd, PDF_CMAP_TABLE, 4450 },
	{ 0x8cbf, 0x8cc4, PDF_CMAP_TABLE, 4453 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_TABLE, 4459 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 2833 },
	{ 0x8ccd, 0x8cce, PDF_CMAP_TABLE, 4461 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 3280 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 3522 },
	{ 0x8cd9, 0x8cdc, PDF_CMAP_TABLE, 4463 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 2502 },
	{ 0x8ce0, 0x8ce4, PDF_CMAP_TABLE, 4467 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 3548 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 2285 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 3148 },
	{ 0x8cf0, 0x8cf0, PDF_CMAP_SINGLE, 8626 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8627 },
	{ 0x8cf8, 0x8cf8, PDF_CMAP_SINGLE, 15156 },
	{ 0x8cfa, 0x8cfb, PDF_CMAP_RANGE, 6770 },
	{ 0x8cfc, 0x8cfe, PDF_CMAP_TABLE, 4472 },
	{ 0x8d04, 0x8d05, PDF_CMAP_RANGE, 6772 },
	{ 0x8d07, 0x8d08, PDF_CMAP_TABLE, 4475 },
	{ 0x8d0a, 0x8d0b, PDF_CMAP_TABLE, 4477 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 6777 },
	{ 0x8d0f, 0x8d10, PDF_CMAP_TABLE, 4479 },
	{ 0x8d12, 0x8d14, PDF_CMAP_TABLE, 4481 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 6783 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 15158 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 2682 },
	{ 0x8d66, 0x8d67, PDF_CMAP_TABLE, 4484 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 15159 },
	{ 0x8d6b, 0x8d6d, PDF_CMAP_TABLE, 4486 },
	{ 0x8d70, 0x8d71, PDF_CMAP_TABLE, 4489 },
	{ 0x8d73, 0x8d74, PDF_CMAP_TABLE, 4491 },
	{ 0x8d76, 0x8d77, PDF_CMAP_TABLE, 4493 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 6788 },
	{ 0x8d84, 0x8d85, PDF_CMAP_TABLE, 4495 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 1277 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 15162 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 15163 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 6789 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 2333 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 15164 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 2620 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 2829 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 6792 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 6791 },
	{ 0x8dc2, 0x8dc2, PDF_CMAP_SINGLE, 6790 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 15165 },
	{ 0x8dcb, 0x8dcc, PDF_CMAP_TABLE, 4497 },
	{ 0x8dce, 0x8dcf, PDF_CMAP_TABLE, 4499 },
	{ 0x8dd6, 0x8dd6, PDF_CMAP_SINGLE, 6795 },
	{ 0x8dda, 0x8ddb, PDF_CMAP_TABLE, 4501 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 1681 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 6801 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 2683 },
	{ 0x8de3, 0x8de4, PDF_CMAP_TABLE, 4503 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 1933 },
	{ 0x8dea, 0x8deb, PDF_CMAP_RANGE, 6799 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 15168 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 4047 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3027 },
	{ 0x8df5, 0x8df5, PDF_CMAP_SINGLE, 2731 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 6803 },
	{ 0x8dff, 0x8dff, PDF_CMAP_SINGLE, 6806 },
	{ 0x8e08, 0x8e09, PDF_CMAP_RANGE, 6804 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 3907 },
	{ 0x8e0f, 0x8e10, PDF_CMAP_TABLE, 4505 },
	{ 0x8e1d, 0x8e1e, PDF_CMAP_RANGE, 6807 },
	{ 0x8e1f, 0x8e20, PDF_CMAP_TABLE, 4507 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 6824 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 6813 },
	{ 0x8e34, 0x8e35, PDF_CMAP_TABLE, 4509 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 6811 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 3097 },
	{ 0x8e47, 0x8e4c, PDF_CMAP_TABLE, 4511 },
	{ 0x8e50, 0x8e50, PDF_CMAP_SINGLE, 6819 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 6826 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 6821 },
	{ 0x8e5f, 0x8e60, PDF_CMAP_TABLE, 4517 },
	{ 0x8e63, 0x8e64, PDF_CMAP_TABLE, 4519 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 15171 },
	{ 0x8e70, 0x8e70, PDF_CMAP_SINGLE, 15172 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 6828 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 2365 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 6827 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 15173 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 6829 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 6830 },
	{ 0x8e84, 0x8e85, PDF_CMAP_TABLE, 4521 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 6831 },
	{ 0x8e8a, 0x8e8b, PDF_CMAP_TABLE, 4523 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 3842 },
	{ 0x8e91, 0x8e94, PDF_CMAP_TABLE, 4525 },
	{ 0x8e99, 0x8e99, PDF_CMAP_SINGLE, 6839 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 6841 },
	{ 0x8eaa, 0x8eac, PDF_CMAP_TABLE, 4529 },
	{ 0x8eae, 0x8eb1, PDF_CMAP_TABLE, 4532 },
	{ 0x8eb3, 0x8eb3, PDF_CMAP_SINGLE, 15176 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14083 },
	{ 0x8ebe, 0x8ebe, PDF_CMAP_SINGLE, 6846 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 7663 },
	{ 0x8ec5, 0x8ec6, PDF_CMAP_TABLE, 4536 },
	{ 0x8ec8, 0x8ec8, PDF_CMAP_SINGLE, 6848 },
	{ 0x8eca, 0x8ecd, PDF_CMAP_TABLE, 4538 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 8631 },
	{ 0x8ed1, 0x8ed2, PDF_CMAP_TABLE, 4542 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 15178 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 6850 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 3272 },
	{ 0x8ee2, 0x8ee3, PDF_CMAP_TABLE, 4544 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 6854 },
	{ 0x8ef8, 0x8ef9, PDF_CMAP_TABLE, 4546 },
	{ 0x8efb, 0x8efe, PDF_CMAP_TABLE, 4548 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 1457 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 6857 },
	{ 0x8f09, 0x8f0a, PDF_CMAP_TABLE, 4552 },
	{ 0x8f0c, 0x8f0c, PDF_CMAP_SINGLE, 6865 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 4554 },
	{ 0x8f17, 0x8f17, PDF_CMAP_SINGLE, 15180 },
	{ 0x8f19, 0x8f19, PDF_CMAP_SINGLE, 6860 },
	{ 0x8f1b, 0x8f1d, PDF_CMAP_TABLE, 4558 },
	{ 0x8f1f, 0x8f1f, PDF_CMAP_SINGLE, 6863 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 6866 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 4561 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 2366 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 6867 },
	{ 0x8f36, 0x8f36, PDF_CMAP_SINGLE, 15181 },
	{ 0x8f38, 0x8f39, PDF_CMAP_TABLE, 4563 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 6868 },
	{ 0x8f3e, 0x8f3f, PDF_CMAP_TABLE, 4565 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 6871 },
	{ 0x8f44, 0x8f46, PDF_CMAP_TABLE, 4567 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 6874 },
	{ 0x8f4c, 0x8f4e, PDF_CMAP_TABLE, 4570 },
	{ 0x8f57, 0x8f57, PDF_CMAP_SINGLE, 6877 },
	{ 0x8f5c, 0x8f5c, PDF_CMAP_SINGLE, 6878 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 2046 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 1787 },
	{ 0x8f62, 0x8f64, PDF_CMAP_RANGE, 6879 },
	{ 0x8f9b, 0x8f9c, PDF_CMAP_TABLE, 4573 },
	{ 0x8f9e, 0x8f9f, PDF_CMAP_TABLE, 4575 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 6884 },
	{ 0x8fa6, 0x8fa8, PDF_CMAP_TABLE, 4577 },
	{ 0x8fad, 0x8fb2, PDF_CMAP_TABLE, 4580 },
	{ 0x8fb5, 0x8fb6, PDF_CMAP_RANGE, 15183 },
	{ 0x8fb7, 0x8fb7, PDF_CMAP_SINGLE, 6887 },
	{ 0x8fba, 0x8fbc, PDF_CMAP_TABLE, 4586 },
	{ 0x8fbf, 0x8fbf, PDF_CMAP_SINGLE, 2919 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 1228 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 4589 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 14233 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 1844 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 1753 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 3622 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 6888 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 15186 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 6890 },
	{ 0x8fe4, 0x8fe6, PDF_CMAP_TABLE, 4592 },
	{ 0x8fe9, 0x8feb, PDF_CMAP_TABLE, 4595 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 3117 },
	{ 0x8fef, 0x8ff0, PDF_CMAP_TABLE, 4598 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 6894 },
	{ 0x8ff6, 0x8ff8, PDF_CMAP_TABLE, 4600 },
	{ 0x8ff9, 0x8ffa, PDF_CMAP_RANGE, 6896 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 3045 },
	{ 0x9000, 0x9003, PDF_CMAP_TABLE, 4603 },
	{ 0x9005, 0x9006, PDF_CMAP_TABLE, 4607 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 6904 },
	{ 0x900d, 0x9011, PDF_CMAP_TABLE, 4609 },
	{ 0x9013, 0x9017, PDF_CMAP_TABLE, 4614 },
	{ 0x9019, 0x901a, PDF_CMAP_TABLE, 4619 },
	{ 0x901d, 0x9023, PDF_CMAP_TABLE, 4621 },
	{ 0x9027, 0x9027, PDF_CMAP_SINGLE, 6905 },
	{ 0x902c, 0x902c, PDF_CMAP_SINGLE, 15190 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 2881 },
	{ 0x9031, 0x9032, PDF_CMAP_TABLE, 4628 },
	{ 0x9035, 0x9036, PDF_CMAP_TABLE, 4630 },
	{ 0x9038, 0x9039, PDF_CMAP_TABLE, 4632 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 3489 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 6916 },
	{ 0x9041, 0x9042, PDF_CMAP_TABLE, 4634 },
	{ 0x9044, 0x9045, PDF_CMAP_TABLE, 4636 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 1776 },
	{ 0x9049, 0x904b, PDF_CMAP_TABLE, 4638 },
	{ 0x904d, 0x904e, PDF_CMAP_TABLE, 4641 },
	{ 0x904f, 0x9052, PDF_CMAP_RANGE, 6910 },
	{ 0x9053, 0x9056, PDF_CMAP_TABLE, 4643 },
	{ 0x9058, 0x9059, PDF_CMAP_TABLE, 4647 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 2845 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 6919 },
	{ 0x9060, 0x9061, PDF_CMAP_TABLE, 4649 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 1891 },
	{ 0x9065, 0x9065, PDF_CMAP_SINGLE, 3908 },
	{ 0x9067, 0x9069, PDF_CMAP_TABLE, 4651 },
	{ 0x906d, 0x906f, PDF_CMAP_TABLE, 4654 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 6924 },
	{ 0x9075, 0x9078, PDF_CMAP_TABLE, 4657 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 1192 },
	{ 0x907c, 0x907d, PDF_CMAP_TABLE, 4661 },
	{ 0x907f, 0x9084, PDF_CMAP_TABLE, 4663 },
	{ 0x9087, 0x908a, PDF_CMAP_TABLE, 4669 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 6931 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 3874 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 15193 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 15194 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 15195 },
	{ 0x90a2, 0x90a3, PDF_CMAP_TABLE, 4673 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 3676 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 6932 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 2309 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 6933 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 6934 },
	{ 0x90b4, 0x90b5, PDF_CMAP_TABLE, 4675 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 3099 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 1198 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2027 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 4064 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 15198 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 6939 },
	{ 0x90dd, 0x90de, PDF_CMAP_TABLE, 4677 },
	{ 0x90e1, 0x90e2, PDF_CMAP_TABLE, 4679 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 6937 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 3558 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 1458 },
	{ 0x90f4, 0x90f5, PDF_CMAP_TABLE, 4681 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 1719 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 3150 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 6940 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 6941 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 8637 },
	{ 0x9117, 0x9117, PDF_CMAP_SINGLE, 15201 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 6942 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 15202 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 8638 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 3100 },
	{ 0x9130, 0x9132, PDF_CMAP_TABLE, 4683 },
	{ 0x913a, 0x913a, PDF_CMAP_SINGLE, 15204 },
	{ 0x913d, 0x913d, PDF_CMAP_SINGLE, 15205 },
	{ 0x9148, 0x914e, PDF_CMAP_TABLE, 4686 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2334 },
	{ 0x9154, 0x9154, PDF_CMAP_SINGLE, 2610 },
	{ 0x9156, 0x9156, PDF_CMAP_SINGLE, 6946 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 6947 },
	{ 0x915b, 0x915b, PDF_CMAP_SINGLE, 15207 },
	{ 0x9161, 0x9165, PDF_CMAP_TABLE, 4693 },
	{ 0x9169, 0x916a, PDF_CMAP_TABLE, 4698 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 2369 },
	{ 0x9172, 0x9173, PDF_CMAP_TABLE, 4700 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 2028 },
	{ 0x9177, 0x9178, PDF_CMAP_TABLE, 4702 },
	{ 0x9182, 0x9182, PDF_CMAP_SINGLE, 6955 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 2416 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 6954 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 6953 },
	{ 0x918d, 0x918e, PDF_CMAP_TABLE, 4704 },
	{ 0x9190, 0x9190, PDF_CMAP_SINGLE, 1955 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 2663 },
	{ 0x9197, 0x9197, PDF_CMAP_SINGLE, 3396 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 2371 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 15211 },
	{ 0x91a2, 0x91a2, PDF_CMAP_SINGLE, 6956 },
	{ 0x91a4, 0x91a4, PDF_CMAP_SINGLE, 2503 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 15212 },
	{ 0x91aa, 0x91ac, PDF_CMAP_TABLE, 4706 },
	{ 0x91ad, 0x91ae, PDF_CMAP_RANGE, 15213 },
	{ 0x91af, 0x91af, PDF_CMAP_SINGLE, 6958 },
	{ 0x91b1, 0x91b2, PDF_CMAP_TABLE, 4709 },
	{ 0x91b4, 0x91b5, PDF_CMAP_TABLE, 4711 },
	{ 0x91b8, 0x91b8, PDF_CMAP_SINGLE, 2530 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 6962 },
	{ 0x91bc, 0x91bc, PDF_CMAP_SINGLE, 15216 },
	{ 0x91c0, 0x91c1, PDF_CMAP_RANGE, 6963 },
	{ 0x91c6, 0x91c9, PDF_CMAP_TABLE, 4713 },
	{ 0x91cb, 0x91d1, PDF_CMAP_TABLE, 4717 },
	{ 0x91d6, 0x91d8, PDF_CMAP_TABLE, 4724 },
	{ 0x91da, 0x91df, PDF_CMAP_TABLE, 4727 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 6970 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 3068 },
	{ 0x91e4, 0x91e5, PDF_CMAP_RANGE, 8644 },
	{ 0x91e6, 0x91e7, PDF_CMAP_TABLE, 4733 },
	{ 0x91ed, 0x91ee, PDF_CMAP_RANGE, 8642 },
	{ 0x91f0, 0x91f0, PDF_CMAP_SINGLE, 15217 },
	{ 0x91f5, 0x91f6, PDF_CMAP_RANGE, 6973 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 15218 },
	{ 0x91fb, 0x91fc, PDF_CMAP_TABLE, 4735 },
	{ 0x91ff, 0x91ff, PDF_CMAP_SINGLE, 6976 },
	{ 0x9206, 0x9207, PDF_CMAP_TABLE, 4737 },
	{ 0x920a, 0x920a, PDF_CMAP_SINGLE, 8648 },
	{ 0x920d, 0x920e, PDF_CMAP_TABLE, 4739 },
	{ 0x9210, 0x9211, PDF_CMAP_TABLE, 4741 },
	{ 0x9214, 0x9215, PDF_CMAP_TABLE, 4743 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 6975 },
	{ 0x9228, 0x9229, PDF_CMAP_TABLE, 4745 },
	{ 0x922c, 0x922c, PDF_CMAP_SINGLE, 6978 },
	{ 0x9233, 0x9234, PDF_CMAP_TABLE, 4747 },
	{ 0x9237, 0x923a, PDF_CMAP_TABLE, 4749 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 8651 },
	{ 0x923f, 0x9240, PDF_CMAP_TABLE, 4753 },
	{ 0x9243, 0x9245, PDF_CMAP_TABLE, 4755 },
	{ 0x9247, 0x9249, PDF_CMAP_TABLE, 4758 },
	{ 0x924b, 0x924b, PDF_CMAP_SINGLE, 6989 },
	{ 0x924e, 0x9251, PDF_CMAP_TABLE, 4761 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 6982 },
	{ 0x9259, 0x925b, PDF_CMAP_TABLE, 4765 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 6981 },
	{ 0x9260, 0x9260, PDF_CMAP_SINGLE, 15227 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 3393 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 6985 },
	{ 0x9266, 0x9267, PDF_CMAP_TABLE, 4768 },
	{ 0x9271, 0x9271, PDF_CMAP_SINGLE, 2029 },
	{ 0x9277, 0x9278, PDF_CMAP_RANGE, 8658 },
	{ 0x927e, 0x927e, PDF_CMAP_SINGLE, 3702 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 1756 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 2384 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 3220 },
	{ 0x9288, 0x9288, PDF_CMAP_SINGLE, 8362 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 2735 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 6993 },
	{ 0x9295, 0x9296, PDF_CMAP_TABLE, 4770 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 3791 },
	{ 0x929a, 0x929c, PDF_CMAP_TABLE, 4772 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 8657 },
	{ 0x92ad, 0x92ad, PDF_CMAP_SINGLE, 2734 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 13652 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 6998 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 6997 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 15228 },
	{ 0x92cb, 0x92cc, PDF_CMAP_RANGE, 15229 },
	{ 0x92cf, 0x92d0, PDF_CMAP_TABLE, 4775 },
	{ 0x92d2, 0x92d3, PDF_CMAP_TABLE, 4777 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 8665 },
	{ 0x92d7, 0x92d7, PDF_CMAP_SINGLE, 8661 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 8662 },
	{ 0x92df, 0x92e0, PDF_CMAP_TABLE, 4779 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 2437 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 8660 },
	{ 0x92e9, 0x92ea, PDF_CMAP_TABLE, 4781 },
	{ 0x92ed, 0x92ed, PDF_CMAP_SINGLE, 1270 },
	{ 0x92f2, 0x92f3, PDF_CMAP_TABLE, 4783 },
	{ 0x92f8, 0x92fc, PDF_CMAP_TABLE, 4785 },
	{ 0x92ff, 0x92ff, PDF_CMAP_SINGLE, 8673 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 8675 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 13402 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 2170 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 15232 },
	{ 0x930f, 0x9310, PDF_CMAP_TABLE, 4790 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 15233 },
	{ 0x9318, 0x931a, PDF_CMAP_TABLE, 4792 },
	{ 0x931d, 0x9323, PDF_CMAP_TABLE, 4795 },
	{ 0x9325, 0x9328, PDF_CMAP_TABLE, 4802 },
	{ 0x932b, 0x932c, PDF_CMAP_TABLE, 4806 },
	{ 0x932e, 0x932f, PDF_CMAP_TABLE, 4808 },
	{ 0x9332, 0x9332, PDF_CMAP_SINGLE, 4069 },
	{ 0x9335, 0x9335, PDF_CMAP_SINGLE, 7009 },
	{ 0x933a, 0x933b, PDF_CMAP_TABLE, 4810 },
	{ 0x9344, 0x9344, PDF_CMAP_SINGLE, 7002 },
	{ 0x9347, 0x9348, PDF_CMAP_TABLE, 4812 },
	{ 0x934a, 0x934b, PDF_CMAP_TABLE, 4814 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 3151 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 15237 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 3059 },
	{ 0x9356, 0x9357, PDF_CMAP_TABLE, 4816 },
	{ 0x935b, 0x935c, PDF_CMAP_TABLE, 4818 },
	{ 0x9360, 0x9360, PDF_CMAP_SINGLE, 7012 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 15238 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 15239 },
	{ 0x936c, 0x936e, PDF_CMAP_TABLE, 4820 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 8676 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 1892 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 7013 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 2505 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 1495 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 7019 },
	{ 0x9396, 0x9397, PDF_CMAP_TABLE, 4823 },
	{ 0x939a, 0x939b, PDF_CMAP_TABLE, 4825 },
	{ 0x93a4, 0x93a4, PDF_CMAP_SINGLE, 8678 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 1433 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 15243 },
	{ 0x93ac, 0x93ad, PDF_CMAP_RANGE, 7017 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 3039 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 7016 },
	{ 0x93b9, 0x93ba, PDF_CMAP_TABLE, 4827 },
	{ 0x93c1, 0x93c1, PDF_CMAP_SINGLE, 15244 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 7026 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 8679 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 7029 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 15245 },
	{ 0x93d0, 0x93d1, PDF_CMAP_TABLE, 4829 },
	{ 0x93d6, 0x93d7, PDF_CMAP_RANGE, 7021 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 7025 },
	{ 0x93dd, 0x93de, PDF_CMAP_TABLE, 4831 },
	{ 0x93e1, 0x93e2, PDF_CMAP_TABLE, 4833 },
	{ 0x93e4, 0x93e5, PDF_CMAP_TABLE, 4835 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7023 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 8681 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 15247 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 15248 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 7034 },
	{ 0x9407, 0x9407, PDF_CMAP_SINGLE, 7035 },
	{ 0x940f, 0x9410, PDF_CMAP_TABLE, 4837 },
	{ 0x9413, 0x9414, PDF_CMAP_TABLE, 4839 },
	{ 0x9418, 0x941a, PDF_CMAP_TABLE, 4841 },
	{ 0x9421, 0x9421, PDF_CMAP_SINGLE, 7040 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 7038 },
	{ 0x9431, 0x9431, PDF_CMAP_SINGLE, 8682 },
	{ 0x9434, 0x9436, PDF_CMAP_TABLE, 4844 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 2904 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 7041 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 15251 },
	{ 0x9441, 0x9441, PDF_CMAP_SINGLE, 7042 },
	{ 0x9444, 0x9445, PDF_CMAP_TABLE, 4847 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 8684 },
	{ 0x9451, 0x9453, PDF_CMAP_TABLE, 4849 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 15252 },
	{ 0x945a, 0x945b, PDF_CMAP_TABLE, 4852 },
	{ 0x945e, 0x945e, PDF_CMAP_SINGLE, 7048 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 7046 },
	{ 0x9462, 0x9462, PDF_CMAP_SINGLE, 7047 },
	{ 0x946a, 0x946b, PDF_CMAP_TABLE, 4854 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 7051 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 15254 },
	{ 0x9475, 0x9475, PDF_CMAP_SINGLE, 7052 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 7053 },
	{ 0x947c, 0x947f, PDF_CMAP_TABLE, 4856 },
	{ 0x9481, 0x9481, PDF_CMAP_SINGLE, 7058 },
	{ 0x9577, 0x9578, PDF_CMAP_TABLE, 4860 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 3827 },
	{ 0x9582, 0x9583, PDF_CMAP_TABLE, 4862 },
	{ 0x9587, 0x9587, PDF_CMAP_SINGLE, 7061 },
	{ 0x9589, 0x958b, PDF_CMAP_TABLE, 4864 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 1246 },
	{ 0x9591, 0x9594, PDF_CMAP_TABLE, 4867 },
	{ 0x9596, 0x9596, PDF_CMAP_SINGLE, 7064 },
	{ 0x9598, 0x9599, PDF_CMAP_RANGE, 7065 },
	{ 0x95a0, 0x95a0, PDF_CMAP_SINGLE, 7067 },
	{ 0x95a2, 0x95a9, PDF_CMAP_TABLE, 4871 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 15258 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 7070 },
	{ 0x95b1, 0x95b2, PDF_CMAP_TABLE, 4879 },
	{ 0x95b4, 0x95b4, PDF_CMAP_SINGLE, 15259 },
	{ 0x95b9, 0x95b9, PDF_CMAP_SINGLE, 7073 },
	{ 0x95bb, 0x95be, PDF_CMAP_TABLE, 4881 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 7077 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 1163 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 7075 },
	{ 0x95cc, 0x95cd, PDF_CMAP_TABLE, 4885 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 4887 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 3206 },
	{ 0x95da, 0x95da, PDF_CMAP_SINGLE, 15261 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 7083 },
	{ 0x95e1, 0x95e2, PDF_CMAP_TABLE, 4890 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 7085 },
	{ 0x961c, 0x961d, PDF_CMAP_TABLE, 4892 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 7087 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 7088 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 2133 },
	{ 0x962e, 0x962f, PDF_CMAP_RANGE, 7089 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 3703 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 2765 },
	{ 0x963f, 0x9642, PDF_CMAP_TABLE, 4894 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 3551 },
	{ 0x964b, 0x964d, PDF_CMAP_TABLE, 4898 },
	{ 0x964f, 0x9650, PDF_CMAP_TABLE, 4901 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 15264 },
	{ 0x965b, 0x965f, PDF_CMAP_TABLE, 4903 },
	{ 0x9662, 0x9666, PDF_CMAP_TABLE, 4908 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 3356 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 7102 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 1220 },
	{ 0x9672, 0x9673, PDF_CMAP_TABLE, 4913 },
	{ 0x9675, 0x9678, PDF_CMAP_TABLE, 4915 },
	{ 0x967a, 0x967a, PDF_CMAP_SINGLE, 1893 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 3909 },
	{ 0x9684, 0x9686, PDF_CMAP_TABLE, 4919 },
	{ 0x9688, 0x9688, PDF_CMAP_SINGLE, 1790 },
	{ 0x968a, 0x968b, PDF_CMAP_TABLE, 4922 },
	{ 0x968d, 0x968f, PDF_CMAP_TABLE, 4924 },
	{ 0x9694, 0x9695, PDF_CMAP_TABLE, 4927 },
	{ 0x9697, 0x9699, PDF_CMAP_TABLE, 4929 },
	{ 0x969b, 0x969d, PDF_CMAP_TABLE, 4932 },
	{ 0x96a0, 0x96a0, PDF_CMAP_SINGLE, 1221 },
	{ 0x96a3, 0x96a4, PDF_CMAP_TABLE, 4935 },
	{ 0x96a7, 0x96aa, PDF_CMAP_TABLE, 4937 },
	{ 0x96af, 0x96b0, PDF_CMAP_TABLE, 4941 },
	{ 0x96b1, 0x96b2, PDF_CMAP_RANGE, 7109 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 7112 },
	{ 0x96b6, 0x96b7, PDF_CMAP_TABLE, 4943 },
	{ 0x96b8, 0x96b9, PDF_CMAP_RANGE, 7114 },
	{ 0x96bb, 0x96bc, PDF_CMAP_TABLE, 4945 },
	{ 0x96c0, 0x96c1, PDF_CMAP_TABLE, 4947 },
	{ 0x96c4, 0x96c7, PDF_CMAP_TABLE, 4949 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 7118 },
	{ 0x96cb, 0x96ce, PDF_CMAP_TABLE, 4953 },
	{ 0x96d1, 0x96d2, PDF_CMAP_TABLE, 4957 },
	{ 0x96d5, 0x96d6, PDF_CMAP_TABLE, 4959 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 4331 },
	{ 0x96db, 0x96dc, PDF_CMAP_TABLE, 4961 },
	{ 0x96de, 0x96de, PDF_CMAP_SINGLE, 15270 },
	{ 0x96e2, 0x96e3, PDF_CMAP_TABLE, 4963 },
	{ 0x96e8, 0x96eb, PDF_CMAP_TABLE, 4965 },
	{ 0x96f0, 0x96f2, PDF_CMAP_TABLE, 4969 },
	{ 0x96f6, 0x96f7, PDF_CMAP_TABLE, 4972 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 7124 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 3135 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 2343 },
	{ 0x9702, 0x9702, PDF_CMAP_SINGLE, 15273 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 7125 },
	{ 0x9706, 0x970a, PDF_CMAP_TABLE, 4974 },
	{ 0x970d, 0x970f, PDF_CMAP_TABLE, 4979 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 7130 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 7128 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 7132 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 7133 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 2812 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 1378 },
	{ 0x9724, 0x9724, PDF_CMAP_SINGLE, 7134 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 3780 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7135 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 7136 },
	{ 0x9732, 0x9733, PDF_CMAP_TABLE, 4982 },
	{ 0x9738, 0x9739, PDF_CMAP_TABLE, 4984 },
	{ 0x973b, 0x973b, PDF_CMAP_SINGLE, 8691 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 7138 },
	{ 0x9742, 0x9744, PDF_CMAP_TABLE, 4986 },
	{ 0x9746, 0x9746, PDF_CMAP_SINGLE, 7141 },
	{ 0x9748, 0x9749, PDF_CMAP_TABLE, 4989 },
	{ 0x974d, 0x974d, PDF_CMAP_SINGLE, 8693 },
	{ 0x974f, 0x974f, PDF_CMAP_SINGLE, 8694 },
	{ 0x9751, 0x9752, PDF_CMAP_TABLE, 4991 },
	{ 0x9755, 0x9756, PDF_CMAP_TABLE, 4993 },
	{ 0x9759, 0x975a, PDF_CMAP_TABLE, 4995 },
	{ 0x975c, 0x975c, PDF_CMAP_SINGLE, 7145 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 3463 },
	{ 0x9760, 0x9764, PDF_CMAP_TABLE, 4997 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 7148 },
	{ 0x9768, 0x9769, PDF_CMAP_TABLE, 5002 },
	{ 0x976b, 0x976b, PDF_CMAP_SINGLE, 7151 },
	{ 0x976d, 0x976e, PDF_CMAP_TABLE, 5004 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 7152 },
	{ 0x9773, 0x9774, PDF_CMAP_TABLE, 5006 },
	{ 0x9779, 0x977a, PDF_CMAP_TABLE, 5008 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7155 },
	{ 0x9781, 0x9781, PDF_CMAP_SINGLE, 7156 },
	{ 0x9784, 0x9786, PDF_CMAP_TABLE, 5010 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 7159 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 1164 },
	{ 0x978f, 0x9790, PDF_CMAP_RANGE, 7160 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 2508 },
	{ 0x979a, 0x979a, PDF_CMAP_SINGLE, 15279 },
	{ 0x979c, 0x979c, PDF_CMAP_SINGLE, 7162 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 1633 },
	{ 0x97a2, 0x97a3, PDF_CMAP_TABLE, 5013 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 7164 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 7163 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 6708 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 3628 },
	{ 0x97b3, 0x97b4, PDF_CMAP_RANGE, 7166 },
	{ 0x97b5, 0x97b6, PDF_CMAP_RANGE, 15281 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 7168 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 7169 },
	{ 0x97c8, 0x97c8, PDF_CMAP_SINGLE, 7170 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 7171 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 1558 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 15283 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 7172 },
	{ 0x97de, 0x97de, PDF_CMAP_SINGLE, 15284 },
	{ 0x97ed, 0x97ee, PDF_CMAP_TABLE, 5015 },
	{ 0x97f2, 0x97f6, PDF_CMAP_TABLE, 5017 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 1222 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 1721 },
	{ 0x9801, 0x9803, PDF_CMAP_TABLE, 5022 },
	{ 0x9805, 0x9806, PDF_CMAP_TABLE, 5025 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 2594 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 15286 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 7180 },
	{ 0x980e, 0x9813, PDF_CMAP_TABLE, 5027 },
	{ 0x9817, 0x9818, PDF_CMAP_TABLE, 5033 },
	{ 0x981a, 0x981a, PDF_CMAP_SINGLE, 1841 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 15288 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 7183 },
	{ 0x9823, 0x9824, PDF_CMAP_TABLE, 5035 },
	{ 0x982b, 0x982d, PDF_CMAP_TABLE, 5037 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 7795 },
	{ 0x9834, 0x9834, PDF_CMAP_SINGLE, 1266 },
	{ 0x9837, 0x9839, PDF_CMAP_TABLE, 5040 },
	{ 0x983b, 0x983e, PDF_CMAP_TABLE, 5043 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 7186 },
	{ 0x984b, 0x984c, PDF_CMAP_TABLE, 5047 },
	{ 0x984d, 0x984e, PDF_CMAP_RANGE, 1465 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 7187 },
	{ 0x9852, 0x9853, PDF_CMAP_RANGE, 15292 },
	{ 0x9854, 0x9855, PDF_CMAP_TABLE, 5049 },
	{ 0x9857, 0x985b, PDF_CMAP_TABLE, 5051 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 4008 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 8698 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 1936 },
	{ 0x986b, 0x986c, PDF_CMAP_TABLE, 5056 },
	{ 0x986f, 0x9871, PDF_CMAP_RANGE, 7190 },
	{ 0x9873, 0x9874, PDF_CMAP_TABLE, 5058 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 3561 },
	{ 0x98aa, 0x98aa, PDF_CMAP_SINGLE, 7195 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 7196 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 7197 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 7198 },
	{ 0x98b8, 0x98b8, PDF_CMAP_SINGLE, 15296 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 15297 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 15298 },
	{ 0x98c3, 0x98c4, PDF_CMAP_TABLE, 5060 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 7201 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 15299 },
	{ 0x98db, 0x98dc, PDF_CMAP_TABLE, 5062 },
	{ 0x98df, 0x98e0, PDF_CMAP_TABLE, 5064 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 1612 },
	{ 0x98e5, 0x98e5, PDF_CMAP_SINGLE, 15300 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 7202 },
	{ 0x98eb, 0x98eb, PDF_CMAP_SINGLE, 7203 },
	{ 0x98ed, 0x98ef, PDF_CMAP_TABLE, 5066 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 1215 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 1151 },
	{ 0x98fc, 0x98fe, PDF_CMAP_TABLE, 5069 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 7204 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 3819 },
	{ 0x9909, 0x990a, PDF_CMAP_TABLE, 5072 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 1252 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 2191 },
	{ 0x9912, 0x9914, PDF_CMAP_TABLE, 5074 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 7208 },
	{ 0x991d, 0x991e, PDF_CMAP_RANGE, 7210 },
	{ 0x9920, 0x9921, PDF_CMAP_TABLE, 5077 },
	{ 0x9924, 0x9924, PDF_CMAP_SINGLE, 7212 },
	{ 0x9927, 0x9928, PDF_CMAP_TABLE, 5079 },
	{ 0x992c, 0x992c, PDF_CMAP_SINGLE, 7214 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7215 },
	{ 0x9932, 0x9933, PDF_CMAP_RANGE, 15301 },
	{ 0x993d, 0x993e, PDF_CMAP_RANGE, 7216 },
	{ 0x9940, 0x9940, PDF_CMAP_SINGLE, 15303 },
	{ 0x9942, 0x9942, PDF_CMAP_SINGLE, 7218 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 7220 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 7219 },
	{ 0x994b, 0x994d, PDF_CMAP_TABLE, 5081 },
	{ 0x9950, 0x9950, PDF_CMAP_SINGLE, 7221 },
	{ 0x9951, 0x9952, PDF_CMAP_RANGE, 7223 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 7226 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 1722 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 15305 },
	{ 0x995f, 0x995f, PDF_CMAP_SINGLE, 15306 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 2335 },
	{ 0x9997, 0x9998, PDF_CMAP_RANGE, 7227 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 2035 },
	{ 0x999e, 0x999e, PDF_CMAP_SINGLE, 8703 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 7229 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 1436 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 3333 },
	{ 0x99ad, 0x99ae, PDF_CMAP_RANGE, 7230 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 15307 },
	{ 0x99b3, 0x99b4, PDF_CMAP_TABLE, 5084 },
	{ 0x99b9, 0x99ba, PDF_CMAP_RANGE, 15308 },
	{ 0x99bc, 0x99bc, PDF_CMAP_SINGLE, 7232 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 3379 },
	{ 0x99c4, 0x99c6, PDF_CMAP_TABLE, 5086 },
	{ 0x99c8, 0x99c9, PDF_CMAP_TABLE, 5089 },
	{ 0x99d0, 0x99d2, PDF_CMAP_TABLE, 5091 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 1391 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 7236 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 7234 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 7235 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 7233 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 7247 },
	{ 0x99ed, 0x99ee, PDF_CMAP_RANGE, 7238 },
	{ 0x99f1, 0x99f2, PDF_CMAP_RANGE, 7240 },
	{ 0x99f8, 0x99f8, PDF_CMAP_SINGLE, 7243 },
	{ 0x99fb, 0x99fb, PDF_CMAP_SINGLE, 7242 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 2403 },
	{ 0x9a01, 0x9a02, PDF_CMAP_TABLE, 5094 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 7246 },
	{ 0x9a08, 0x9a08, PDF_CMAP_SINGLE, 14266 },
	{ 0x9a0e, 0x9a0f, PDF_CMAP_TABLE, 5096 },
	{ 0x9a12, 0x9a13, PDF_CMAP_TABLE, 5098 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 15312 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 7248 },
	{ 0x9a24, 0x9a24, PDF_CMAP_SINGLE, 15313 },
	{ 0x9a27, 0x9a28, PDF_CMAP_TABLE, 5100 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 7249 },
	{ 0x9a2d, 0x9a2e, PDF_CMAP_RANGE, 15315 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 3205 },
	{ 0x9a36, 0x9a38, PDF_CMAP_TABLE, 5102 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 7255 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 7253 },
	{ 0x9a42, 0x9a43, PDF_CMAP_TABLE, 5105 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 7251 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 15319 },
	{ 0x9a4d, 0x9a4e, PDF_CMAP_TABLE, 5107 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 7727 },
	{ 0x9a55, 0x9a57, PDF_CMAP_TABLE, 5109 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_TABLE, 5112 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 7260 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 7261 },
	{ 0x9a64, 0x9a65, PDF_CMAP_TABLE, 5114 },
	{ 0x9a69, 0x9a6b, PDF_CMAP_TABLE, 5116 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2062 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 7267 },
	{ 0x9ab0, 0x9ab0, PDF_CMAP_SINGLE, 7268 },
	{ 0x9ab5, 0x9ab6, PDF_CMAP_RANGE, 15321 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 1434 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 7269 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 7270 },
	{ 0x9ac4, 0x9ac4, PDF_CMAP_SINGLE, 2615 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 7271 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 7272 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 7273 },
	{ 0x9ad8, 0x9ad9, PDF_CMAP_TABLE, 5119 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 8706 },
	{ 0x9ade, 0x9adf, PDF_CMAP_RANGE, 7275 },
	{ 0x9ae2, 0x9ae3, PDF_CMAP_RANGE, 7277 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7279 },
	{ 0x9aea, 0x9aeb, PDF_CMAP_TABLE, 5121 },
	{ 0x9aed, 0x9aef, PDF_CMAP_TABLE, 5123 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 7284 },
	{ 0x9af4, 0x9af4, PDF_CMAP_SINGLE, 7283 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 7285 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 15323 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 7286 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 15324 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 7287 },
	{ 0x9b18, 0x9b18, PDF_CMAP_SINGLE, 7288 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 7289 },
	{ 0x9b1f, 0x9b20, PDF_CMAP_TABLE, 5126 },
	{ 0x9b22, 0x9b23, PDF_CMAP_RANGE, 7291 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 7293 },
	{ 0x9b27, 0x9b2a, PDF_CMAP_RANGE, 7294 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 13372 },
	{ 0x9b2e, 0x9b2f, PDF_CMAP_RANGE, 7298 },
	{ 0x9b31, 0x9b32, PDF_CMAP_TABLE, 5128 },
	{ 0x9b33, 0x9b34, PDF_CMAP_RANGE, 15326 },
	{ 0x9b3b, 0x9b3c, PDF_CMAP_TABLE, 5130 },
	{ 0x9b41, 0x9b45, PDF_CMAP_TABLE, 5132 },
	{ 0x9b4d, 0x9b4e, PDF_CMAP_RANGE, 7304 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 7303 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 7306 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 3728 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 7307 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 1685 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4043 },
	{ 0x9b72, 0x9b75, PDF_CMAP_TABLE, 5137 },
	{ 0x9b79, 0x9b79, PDF_CMAP_SINGLE, 15329 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7310 },
	{ 0x9b8e, 0x9b8f, PDF_CMAP_TABLE, 5141 },
	{ 0x9b91, 0x9b93, PDF_CMAP_TABLE, 5143 },
	{ 0x9b96, 0x9b97, PDF_CMAP_RANGE, 7312 },
	{ 0x9b9f, 0x9ba0, PDF_CMAP_RANGE, 7314 },
	{ 0x9ba7, 0x9ba8, PDF_CMAP_TABLE, 5146 },
	{ 0x9baa, 0x9bab, PDF_CMAP_TABLE, 5148 },
	{ 0x9bad, 0x9bae, PDF_CMAP_TABLE, 5150 },
	{ 0x9bb1, 0x9bb1, PDF_CMAP_SINGLE, 8710 },
	{ 0x9bb4, 0x9bb4, PDF_CMAP_SINGLE, 7317 },
	{ 0x9bb9, 0x9bb9, PDF_CMAP_SINGLE, 7320 },
	{ 0x9bbb, 0x9bbb, PDF_CMAP_SINGLE, 8711 },
	{ 0x9bc0, 0x9bc1, PDF_CMAP_TABLE, 5152 },
	{ 0x9bc6, 0x9bc7, PDF_CMAP_TABLE, 5154 },
	{ 0x9bc9, 0x9bca, PDF_CMAP_TABLE, 5156 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 7322 },
	{ 0x9bd1, 0x9bd2, PDF_CMAP_RANGE, 7323 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 7328 },
	{ 0x9bd6, 0x9bd7, PDF_CMAP_TABLE, 5158 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 2884 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 14271 },
	{ 0x9be1, 0x9be4, PDF_CMAP_TABLE, 5160 },
	{ 0x9be7, 0x9be8, PDF_CMAP_TABLE, 5164 },
	{ 0x9beb, 0x9beb, PDF_CMAP_SINGLE, 15335 },
	{ 0x9bf0, 0x9bf2, PDF_CMAP_TABLE, 5166 },
	{ 0x9bf5, 0x9bf5, PDF_CMAP_SINGLE, 1143 },
	{ 0x9bf7, 0x9bf7, PDF_CMAP_SINGLE, 15336 },
	{ 0x9bfa, 0x9bfa, PDF_CMAP_SINGLE, 15337 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 15338 },
	{ 0x9c00, 0x9c00, PDF_CMAP_SINGLE, 8712 },
	{ 0x9c04, 0x9c04, PDF_CMAP_SINGLE, 7343 },
	{ 0x9c06, 0x9c06, PDF_CMAP_SINGLE, 7339 },
	{ 0x9c08, 0x9c0d, PDF_CMAP_TABLE, 5169 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 4082 },
	{ 0x9c12, 0x9c15, PDF_CMAP_TABLE, 5175 },
	{ 0x9c1b, 0x9c1b, PDF_CMAP_SINGLE, 7345 },
	{ 0x9c21, 0x9c21, PDF_CMAP_SINGLE, 7348 },
	{ 0x9c24, 0x9c25, PDF_CMAP_TABLE, 5179 },
	{ 0x9c27, 0x9c27, PDF_CMAP_SINGLE, 15340 },
	{ 0x9c2a, 0x9c2a, PDF_CMAP_SINGLE, 15341 },
	{ 0x9c2d, 0x9c30, PDF_CMAP_TABLE, 5181 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 7351 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 15342 },
	{ 0x9c39, 0x9c3b, PDF_CMAP_TABLE, 5185 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 7353 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 15343 },
	{ 0x9c46, 0x9c48, PDF_CMAP_TABLE, 5188 },
	{ 0x9c52, 0x9c53, PDF_CMAP_TABLE, 5191 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 4002 },
	{ 0x9c5a, 0x9c5a, PDF_CMAP_SINGLE, 7354 },
	{ 0x9c60, 0x9c60, PDF_CMAP_SINGLE, 7355 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 15345 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 7356 },
	{ 0x9c6a, 0x9c6a, PDF_CMAP_SINGLE, 15346 },
	{ 0x9c76, 0x9c78, PDF_CMAP_TABLE, 5193 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 3031 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 7359 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3403 },
	{ 0x9ceb, 0x9cec, PDF_CMAP_TABLE, 5196 },
	{ 0x9cf0, 0x9cf0, PDF_CMAP_SINGLE, 7361 },
	{ 0x9cf3, 0x9cf4, PDF_CMAP_TABLE, 5198 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 3240 },
	{ 0x9d02, 0x9d03, PDF_CMAP_TABLE, 5200 },
	{ 0x9d06, 0x9d09, PDF_CMAP_TABLE, 5202 },
	{ 0x9d0e, 0x9d0e, PDF_CMAP_SINGLE, 1322 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 7374 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 7373 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 1303 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 7371 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 7370 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 7368 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 1497 },
	{ 0x9d2a, 0x9d2c, PDF_CMAP_TABLE, 5206 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 2037 },
	{ 0x9d3e, 0x9d3f, PDF_CMAP_TABLE, 5209 },
	{ 0x9d41, 0x9d42, PDF_CMAP_TABLE, 5211 },
	{ 0x9d44, 0x9d44, PDF_CMAP_SINGLE, 7372 },
	{ 0x9d46, 0x9d48, PDF_CMAP_TABLE, 5213 },
	{ 0x9d50, 0x9d51, PDF_CMAP_TABLE, 5216 },
	{ 0x9d59, 0x9d59, PDF_CMAP_SINGLE, 7385 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 1231 },
	{ 0x9d5d, 0x9d5e, PDF_CMAP_RANGE, 7380 },
	{ 0x9d60, 0x9d61, PDF_CMAP_TABLE, 5218 },
	{ 0x9d63, 0x9d64, PDF_CMAP_TABLE, 5220 },
	{ 0x9d69, 0x9d69, PDF_CMAP_SINGLE, 15352 },
	{ 0x9d6b, 0x9d6c, PDF_CMAP_TABLE, 5222 },
	{ 0x9d6f, 0x9d70, PDF_CMAP_TABLE, 5224 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 7386 },
	{ 0x9d7a, 0x9d7a, PDF_CMAP_SINGLE, 7391 },
	{ 0x9d7c, 0x9d7c, PDF_CMAP_SINGLE, 15353 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15354 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 7388 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 7387 },
	{ 0x9d8d, 0x9d8d, PDF_CMAP_SINGLE, 15355 },
	{ 0x9d8f, 0x9d8f, PDF_CMAP_SINGLE, 1842 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 7392 },
	{ 0x9da4, 0x9da4, PDF_CMAP_SINGLE, 7393 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 7394 },
	{ 0x9dab, 0x9dab, PDF_CMAP_SINGLE, 7389 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 7369 },
	{ 0x9db1, 0x9db2, PDF_CMAP_TABLE, 5226 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 3069 },
	{ 0x9db8, 0x9db8, PDF_CMAP_SINGLE, 7399 },
	{ 0x9dba, 0x9dbb, PDF_CMAP_TABLE, 5228 },
	{ 0x9dc1, 0x9dc4, PDF_CMAP_TABLE, 5230 },
	{ 0x9dc6, 0x9dc7, PDF_CMAP_TABLE, 5234 },
	{ 0x9dcf, 0x9dcf, PDF_CMAP_SINGLE, 7402 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 7405 },
	{ 0x9dd6, 0x9dd7, PDF_CMAP_TABLE, 5236 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 7404 },
	{ 0x9ddf, 0x9ddf, PDF_CMAP_SINGLE, 15360 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 7407 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 15361 },
	{ 0x9ded, 0x9ded, PDF_CMAP_SINGLE, 7408 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 7409 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 4079 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 15362 },
	{ 0x9df8, 0x9dfa, PDF_CMAP_TABLE, 5238 },
	{ 0x9dfd, 0x9dfd, PDF_CMAP_SINGLE, 7410 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 15363 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 8716 },
	{ 0x9e1a, 0x9e1b, PDF_CMAP_RANGE, 7411 },
	{ 0x9e1d, 0x9e1e, PDF_CMAP_TABLE, 5241 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 7414 },
	{ 0x9e78, 0x9e79, PDF_CMAP_TABLE, 5243 },
	{ 0x9e7c, 0x9e7d, PDF_CMAP_TABLE, 5245 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 2267 },
	{ 0x9e81, 0x9e81, PDF_CMAP_SINGLE, 7417 },
	{ 0x9e88, 0x9e88, PDF_CMAP_SINGLE, 7418 },
	{ 0x9e8b, 0x9e8c, PDF_CMAP_RANGE, 7419 },
	{ 0x9e91, 0x9e93, PDF_CMAP_TABLE, 5247 },
	{ 0x9e95, 0x9e95, PDF_CMAP_SINGLE, 7422 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 4023 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 7424 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 4003 },
	{ 0x9ea4, 0x9ea6, PDF_CMAP_TABLE, 5250 },
	{ 0x9ea8, 0x9eaa, PDF_CMAP_TABLE, 5253 },
	{ 0x9eac, 0x9ead, PDF_CMAP_TABLE, 5256 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 5258 },
	{ 0x9eb8, 0x9ebc, PDF_CMAP_TABLE, 5260 },
	{ 0x9ebe, 0x9ebf, PDF_CMAP_TABLE, 5265 },
	{ 0x9ec3, 0x9ec4, PDF_CMAP_TABLE, 5267 },
	{ 0x9ecc, 0x9ecd, PDF_CMAP_TABLE, 5269 },
	{ 0x9ece, 0x9ed0, PDF_CMAP_RANGE, 7432 },
	{ 0x9ed1, 0x9ed2, PDF_CMAP_TABLE, 5271 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 7435 },
	{ 0x9ed8, 0x9ed9, PDF_CMAP_TABLE, 5273 },
	{ 0x9edb, 0x9ede, PDF_CMAP_TABLE, 5275 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 7439 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 7440 },
	{ 0x9ee7, 0x9ee8, PDF_CMAP_TABLE, 5279 },
	{ 0x9eee, 0x9eef, PDF_CMAP_TABLE, 5281 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 7443 },
	{ 0x9ef6, 0x9ef7, PDF_CMAP_RANGE, 7444 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 7446 },
	{ 0x9efb, 0x9efd, PDF_CMAP_RANGE, 7447 },
	{ 0x9f07, 0x9f08, PDF_CMAP_RANGE, 7450 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3102 },
	{ 0x9f10, 0x9f10, PDF_CMAP_SINGLE, 15370 },
	{ 0x9f12, 0x9f13, PDF_CMAP_TABLE, 5283 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 7453 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 15372 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 15373 },
	{ 0x9f20, 0x9f21, PDF_CMAP_TABLE, 5285 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7455 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 15374 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 15375 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 15376 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 3475 },
	{ 0x9f3e, 0x9f3e, PDF_CMAP_SINGLE, 7456 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 15377 },
	{ 0x9f45, 0x9f45, PDF_CMAP_SINGLE, 15378 },
	{ 0x9f4a, 0x9f4b, PDF_CMAP_TABLE, 5287 },
	{ 0x9f4e, 0x9f4f, PDF_CMAP_TABLE, 5289 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7458 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 7459 },
	{ 0x9f57, 0x9f57, PDF_CMAP_SINGLE, 15379 },
	{ 0x9f5f, 0x9f61, PDF_CMAP_RANGE, 7461 },
	{ 0x9f62, 0x9f63, PDF_CMAP_TABLE, 5291 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 7464 },
	{ 0x9f68, 0x9f68, PDF_CMAP_SINGLE, 15380 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 7467 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 7466 },
	{ 0x9f71, 0x9f72, PDF_CMAP_TABLE, 5293 },
	{ 0x9f75, 0x9f77, PDF_CMAP_TABLE, 5295 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 3966 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 15383 },
	{ 0x9f94, 0x9f95, PDF_CMAP_TABLE, 5298 },
	{ 0x9f9c, 0x9f9d, PDF_CMAP_TABLE, 5300 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 7473 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 15385 },
	{ 0xf929, 0xf929, PDF_CMAP_SINGLE, 8489 },
	{ 0xf9dc, 0xf9dc, PDF_CMAP_SINGLE, 8686 },
	{ 0xfa0e, 0xfa0e, PDF_CMAP_SINGLE, 8410 },
	{ 0xfa0f, 0xfa10, PDF_CMAP_RANGE, 8421 },
	{ 0xfa11, 0xfa17, PDF_CMAP_TABLE, 5302 },
	{ 0xfa18, 0xfa1a, PDF_CMAP_RANGE, 8579 },
	{ 0xfa1b, 0xfa1f, PDF_CMAP_TABLE, 5309 },
	{ 0xfa20, 0xfa21, PDF_CMAP_RANGE, 8612 },
	{ 0xfa22, 0xfa23, PDF_CMAP_TABLE, 5314 },
	{ 0xfa24, 0xfa25, PDF_CMAP_RANGE, 8632 },
	{ 0xfa26, 0xfa29, PDF_CMAP_TABLE, 5316 },
	{ 0xfa2a, 0xfa2b, PDF_CMAP_RANGE, 8699 },
	{ 0xfa2c, 0xfa2d, PDF_CMAP_TABLE, 5320 },
	{ 0xfb00, 0xfb00, PDF_CMAP_SINGLE, 9358 },
	{ 0xfb01, 0xfb02, PDF_CMAP_RANGE, 112 },
	{ 0xfb03, 0xfb04, PDF_CMAP_RANGE, 9359 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 7898 },
	{ 0xfe31, 0xfe32, PDF_CMAP_RANGE, 7892 },
	{ 0xfe33, 0xfe33, PDF_CMAP_SINGLE, 7890 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 7899 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 7905 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 7901 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 7915 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 7909 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 7907 },
	{ 0xfe41, 0xfe44, PDF_CMAP_RANGE, 7911 },
	{ 0xff01, 0xff07, PDF_CMAP_TABLE, 5322 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 674 },
	{ 0xff0a, 0xff0f, PDF_CMAP_TABLE, 5329 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 780 },
	{ 0xff1a, 0xff1b, PDF_CMAP_RANGE, 639 },
	{ 0xff1c, 0xff20, PDF_CMAP_TABLE, 5335 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 790 },
	{ 0xff3b, 0xff40, PDF_CMAP_TABLE, 5340 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 816 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 5346 },
	{ 0xff61, 0xff9f, PDF_CMAP_RANGE, 327 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 713 },
	{ 0xffe2, 0xffe5, PDF_CMAP_TABLE, 5350 },
	{ 0xffe8, 0xffe8, PDF_CMAP_SINGLE, 323 },
	{ 0x7682, 0x7682, PDF_CMAP_SINGLE, 8566 },
	{ 0x7683, 0x7683, PDF_CMAP_SINGLE, 5787 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 3108 },
	{ 0x7686, 0x7686, PDF_CMAP_SINGLE, 1413 },
	{ 0x7687, 0x7687, PDF_CMAP_SINGLE, 2006 },
	{ 0x7688, 0x7688, PDF_CMAP_SINGLE, 5788 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 5789 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 5790 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 2167 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 5792 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 5791 },
	{ 0x7699, 0x769a, PDF_CMAP_RANGE, 5793 },
	{ 0x769b, 0x769b, PDF_CMAP_SINGLE, 8569 },
	{ 0x769c, 0x769c, PDF_CMAP_SINGLE, 8567 },
	{ 0x769e, 0x769e, PDF_CMAP_SINGLE, 8568 },
	{ 0x76a4, 0x76a5, PDF_CMAP_RANGE, 14868 },
	{ 0x76a6, 0x76a6, PDF_CMAP_SINGLE, 8570 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3453 },
	{ 0x76b0, 0x76b0, PDF_CMAP_SINGLE, 5795 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 5796 },
	{ 0x76b7, 0x76b7, PDF_CMAP_SINGLE, 7452 },
	{ 0x76b8, 0x76ba, PDF_CMAP_RANGE, 5797 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2172 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 5800 },
	{ 0x76c3, 0x76c3, PDF_CMAP_SINGLE, 3340 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 14870 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 3725 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 1264 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 1273 },
	{ 0x76cc, 0x76cc, PDF_CMAP_SINGLE, 14871 },
	{ 0x76cd, 0x76cd, PDF_CMAP_SINGLE, 5801 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 5803 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 5802 },
	{ 0x76d7, 0x76d7, PDF_CMAP_SINGLE, 3178 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 2653 },
	{ 0x76dc, 0x76dc, PDF_CMAP_SINGLE, 5336 },
	{ 0x76de, 0x76de, PDF_CMAP_SINGLE, 5804 },
	{ 0x76df, 0x76df, PDF_CMAP_SINGLE, 3789 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 5805 },
	{ 0x76e3, 0x76e3, PDF_CMAP_SINGLE, 1538 },
	{ 0x76e4, 0x76e4, PDF_CMAP_SINGLE, 3435 },
	{ 0x76e5, 0x76e5, PDF_CMAP_SINGLE, 5806 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 5807 },
	{ 0x76ea, 0x76ea, PDF_CMAP_SINGLE, 5808 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 14872 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 3816 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 3809 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 3034 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 2796 },
	{ 0x76fb, 0x76fb, PDF_CMAP_SINGLE, 5810 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 14873 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 2412 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 2482 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 5813 },
	{ 0x7707, 0x7707, PDF_CMAP_SINGLE, 5812 },
	{ 0x7708, 0x7708, PDF_CMAP_SINGLE, 5811 },
	{ 0x7709, 0x7709, PDF_CMAP_SINGLE, 3473 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 1539 },
	{ 0x770c, 0x770c, PDF_CMAP_SINGLE, 1885 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 5819 },
	{ 0x771e, 0x771e, PDF_CMAP_SINGLE, 5816 },
	{ 0x771f, 0x771f, PDF_CMAP_SINGLE, 2565 },
	{ 0x7720, 0x7720, PDF_CMAP_SINGLE, 3774 },
	{ 0x7724, 0x7724, PDF_CMAP_SINGLE, 5815 },
	{ 0x7725, 0x7726, PDF_CMAP_RANGE, 5817 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 5814 },
	{ 0x7734, 0x7734, PDF_CMAP_SINGLE, 14874 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 14875 },
	{ 0x7737, 0x7738, PDF_CMAP_RANGE, 5820 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 3019 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 1567 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 2979 },
	{ 0x7746, 0x7746, PDF_CMAP_SINGLE, 8572 },
	{ 0x7747, 0x7747, PDF_CMAP_SINGLE, 5822 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 5823 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 5826 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 14876 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 14877 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 2605 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 7877 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 3228 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 5827 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 3713 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 5824 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 14165 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 5825 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14879 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 5830 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 14880 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 5829 },
	{ 0x777f, 0x777f, PDF_CMAP_SINGLE, 5828 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 5832 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 5831 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 5833 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 14881 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 5835 },
	{ 0x77a0, 0x77a0, PDF_CMAP_SINGLE, 5834 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 3613 },
	{ 0x77a9, 0x77a9, PDF_CMAP_SINGLE, 14166 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 14882 },
	{ 0x77ac, 0x77ac, PDF_CMAP_SINGLE, 2400 },
	{ 0x77ad, 0x77ad, PDF_CMAP_SINGLE, 3982 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 5836 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 3215 },
	{ 0x77b6, 0x77b6, PDF_CMAP_SINGLE, 5837 },
	{ 0x77b9, 0x77b9, PDF_CMAP_SINGLE, 5838 },
	{ 0x77bb, 0x77bb, PDF_CMAP_SINGLE, 5842 },
	{ 0x77bc, 0x77bd, PDF_CMAP_RANGE, 5840 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 5839 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5843 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5844 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5845 },
	{ 0x77da, 0x77da, PDF_CMAP_SINGLE, 5846 },
	{ 0x77db, 0x77db, PDF_CMAP_SINGLE, 3779 },
	{ 0x77dc, 0x77dc, PDF_CMAP_SINGLE, 5847 },
	{ 0x77e2, 0x77e2, PDF_CMAP_SINGLE, 3836 },
	{ 0x77e3, 0x77e3, PDF_CMAP_SINGLE, 5848 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 2956 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 14883 },
	{ 0x77e7, 0x77e7, PDF_CMAP_SINGLE, 3360 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 1763 },
	{ 0x77ed, 0x77ed, PDF_CMAP_SINGLE, 2937 },
	{ 0x77ee, 0x77ee, PDF_CMAP_SINGLE, 5849 },
	{ 0x77ef, 0x77ef, PDF_CMAP_SINGLE, 1714 },
	{ 0x77f0, 0x77f0, PDF_CMAP_SINGLE, 14884 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 2676 },
	{ 0x77f4, 0x77f4, PDF_CMAP_SINGLE, 14885 },
	{ 0x77fc, 0x77fc, PDF_CMAP_SINGLE, 5850 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 2093 },
	{ 0x7806, 0x7806, PDF_CMAP_SINGLE, 14886 },
	{ 0x780c, 0x780c, PDF_CMAP_SINGLE, 5851 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 5852 },
	{ 0x7814, 0x7814, PDF_CMAP_SINGLE, 1882 },
	{ 0x7815, 0x7815, PDF_CMAP_SINGLE, 2117 },
	{ 0x7820, 0x7820, PDF_CMAP_SINGLE, 5854 },
	{ 0x7821, 0x7821, PDF_CMAP_SINGLE, 8574 },
	{ 0x7822, 0x7822, PDF_CMAP_SINGLE, 14887 },
	{ 0x7825, 0x7825, PDF_CMAP_SINGLE, 3152 },
	{ 0x7826, 0x7826, PDF_CMAP_SINGLE, 2118 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 1640 },
	{ 0x782d, 0x782e, PDF_CMAP_RANGE, 14888 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 14890 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 3666 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 3329 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 14891 },
	{ 0x783a, 0x783a, PDF_CMAP_SINGLE, 3153 },
	{ 0x783f, 0x783f, PDF_CMAP_SINGLE, 2030 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 5856 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 8575 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 13342 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 15420 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 2483 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 8576 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 14892 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 3962 },
	{ 0x786c, 0x786c, PDF_CMAP_SINGLE, 2007 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 1883 },
	{ 0x7872, 0x7872, PDF_CMAP_SINGLE, 3383 },
	{ 0x7874, 0x7874, PDF_CMAP_SINGLE, 5858 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 8577 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 5860 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 1951 },
	{ 0x7886, 0x7886, PDF_CMAP_SINGLE, 5859 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 3090 },
	{ 0x788c, 0x788c, PDF_CMAP_SINGLE, 5862 },
	{ 0x788d, 0x788d, PDF_CMAP_SINGLE, 1429 },
	{ 0x788e, 0x788e, PDF_CMAP_SINGLE, 5857 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3454 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 1234 },
	{ 0x7895, 0x7895, PDF_CMAP_SINGLE, 2140 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 4088 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 5861 },
	{ 0x789e, 0x789e, PDF_CMAP_SINGLE, 14893 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 5863 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 3611 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 2685 },
	{ 0x78aa, 0x78aa, PDF_CMAP_SINGLE, 5865 },
	{ 0x78af, 0x78af, PDF_CMAP_SINGLE, 5866 },
	{ 0x78b5, 0x78b5, PDF_CMAP_SINGLE, 5864 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 1452 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 5872 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 5871 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 2259 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 5873 },
	{ 0x78c6, 0x78c6, PDF_CMAP_SINGLE, 5868 },
	{ 0x78c8, 0x78c8, PDF_CMAP_SINGLE, 14894 },
	{ 0x78ca, 0x78ca, PDF_CMAP_SINGLE, 5874 },
	{ 0x78cb, 0x78cb, PDF_CMAP_SINGLE, 5869 },
	{ 0x78cc, 0x78cc, PDF_CMAP_SINGLE, 14895 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 14896 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 3436 },
	{ 0x78d1, 0x78d1, PDF_CMAP_SINGLE, 5867 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 5870 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 5877 },
	{ 0x78e0, 0x78e1, PDF_CMAP_RANGE, 14898 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 14897 },
	{ 0x78e7, 0x78e7, PDF_CMAP_SINGLE, 5876 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 3727 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 5875 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 1199 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 14900 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 5879 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 14901 },
	{ 0x78fb, 0x78fb, PDF_CMAP_SINGLE, 14902 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 5878 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2484 },
	{ 0x7907, 0x7907, PDF_CMAP_SINGLE, 5880 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 2757 },
	{ 0x7911, 0x7911, PDF_CMAP_SINGLE, 5882 },
	{ 0x7912, 0x7912, PDF_CMAP_SINGLE, 5881 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5883 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 5853 },
	{ 0x792a, 0x792a, PDF_CMAP_SINGLE, 5855 },
	{ 0x792b, 0x792b, PDF_CMAP_SINGLE, 5885 },
	{ 0x792c, 0x792c, PDF_CMAP_SINGLE, 5884 },
	{ 0x7930, 0x7930, PDF_CMAP_SINGLE, 8578 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 14903 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 14904 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 2260 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 14905 },
	{ 0x793c, 0x793c, PDF_CMAP_SINGLE, 4017 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 14906 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 2302 },
	{ 0x7940, 0x7940, PDF_CMAP_SINGLE, 5886 },
	{ 0x7941, 0x7941, PDF_CMAP_SINGLE, 1805 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 14907 },
	{ 0x7947, 0x7947, PDF_CMAP_SINGLE, 1626 },
	{ 0x7948, 0x7948, PDF_CMAP_SINGLE, 1601 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 2225 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 3870 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 5892 },
	{ 0x7955, 0x7955, PDF_CMAP_SINGLE, 5891 },
	{ 0x7956, 0x7956, PDF_CMAP_SINGLE, 2758 },
	{ 0x7957, 0x7957, PDF_CMAP_SINGLE, 5888 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 5890 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 14908 },
	{ 0x795d, 0x795d, PDF_CMAP_SINGLE, 2389 },
	{ 0x795e, 0x795e, PDF_CMAP_SINGLE, 2566 },
	{ 0x795f, 0x795f, PDF_CMAP_SINGLE, 5889 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 5887 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 3296 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 2485 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 3502 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2119 },
	{ 0x7977, 0x7977, PDF_CMAP_SINGLE, 3186 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 5893 },
	{ 0x797f, 0x797f, PDF_CMAP_SINGLE, 5894 },
	{ 0x7980, 0x7980, PDF_CMAP_SINGLE, 5916 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 1744 },
	{ 0x7984, 0x7984, PDF_CMAP_SINGLE, 4067 },
	{ 0x7985, 0x7985, PDF_CMAP_SINGLE, 2743 },
	{ 0x798a, 0x798a, PDF_CMAP_SINGLE, 5895 },
	{ 0x798b, 0x798b, PDF_CMAP_SINGLE, 14910 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 1362 },
	{ 0x798e, 0x798e, PDF_CMAP_SINGLE, 3091 },
	{ 0x798f, 0x798f, PDF_CMAP_SINGLE, 3569 },
	{ 0x7994, 0x7994, PDF_CMAP_SINGLE, 8582 },
	{ 0x7996, 0x7996, PDF_CMAP_SINGLE, 14911 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 14912 },
	{ 0x799b, 0x799b, PDF_CMAP_SINGLE, 8584 },
	{ 0x799d, 0x799d, PDF_CMAP_SINGLE, 5896 },
	{ 0x79a6, 0x79a6, PDF_CMAP_SINGLE, 1684 },
	{ 0x79a7, 0x79a7, PDF_CMAP_SINGLE, 5897 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5899 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 5900 },
	{ 0x79b0, 0x79b0, PDF_CMAP_SINGLE, 3295 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 7758 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5901 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 14913 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 5902 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 14914 },
	{ 0x79bd, 0x79bd, PDF_CMAP_SINGLE, 1745 },
	{ 0x79be, 0x79be, PDF_CMAP_SINGLE, 1363 },
	{ 0x79bf, 0x79bf, PDF_CMAP_SINGLE, 3229 },
	{ 0x79c0, 0x79c0, PDF_CMAP_SINGLE, 2354 },
	{ 0x79c1, 0x79c1, PDF_CMAP_SINGLE, 2226 },
	{ 0x79c9, 0x79c9, PDF_CMAP_SINGLE, 5904 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 14915 },
	{ 0x79cb, 0x79cb, PDF_CMAP_SINGLE, 2355 },
	{ 0x79d1, 0x79d1, PDF_CMAP_SINGLE, 1354 },
	{ 0x79d2, 0x79d2, PDF_CMAP_SINGLE, 3509 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 5905 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 3455 },
	{ 0x79da, 0x79da, PDF_CMAP_SINGLE, 14916 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 2759 },
	{ 0x79e1, 0x79e1, PDF_CMAP_SINGLE, 5908 },
	{ 0x79e3, 0x79e3, PDF_CMAP_SINGLE, 5909 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 3359 },
	{ 0x79e6, 0x79e6, PDF_CMAP_SINGLE, 2567 },
	{ 0x79e7, 0x79e7, PDF_CMAP_SINGLE, 5906 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 2975 },
	{ 0x79ec, 0x79ec, PDF_CMAP_SINGLE, 5907 },
	{ 0x79f0, 0x79f0, PDF_CMAP_SINGLE, 2486 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 1184 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 1603 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 14917 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 13875 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 5910 },
	{ 0x7a09, 0x7a09, PDF_CMAP_SINGLE, 14918 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 3092 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 5911 },
	{ 0x7a0e, 0x7a0e, PDF_CMAP_SINGLE, 2667 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 14919 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 3769 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 3477 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 5912 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 2963 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 3983 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 14920 },
	{ 0x7a1f, 0x7a1f, PDF_CMAP_SINGLE, 5915 },
	{ 0x7a20, 0x7a20, PDF_CMAP_SINGLE, 5914 },
	{ 0x7a2d, 0x7a2d, PDF_CMAP_SINGLE, 14921 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 2331 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 5917 },
	{ 0x7a32, 0x7a32, PDF_CMAP_SINGLE, 1204 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 5920 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 14922 },
	{ 0x7a3b, 0x7a3b, PDF_CMAP_SINGLE, 5918 },
	{ 0x7a3c, 0x7a3c, PDF_CMAP_SINGLE, 1364 },
	{ 0x7a3d, 0x7a3d, PDF_CMAP_SINGLE, 1828 },
	{ 0x7a3e, 0x7a3e, PDF_CMAP_SINGLE, 5919 },
	{ 0x7a3f, 0x7a3f, PDF_CMAP_SINGLE, 2008 },
	{ 0x7a40, 0x7a40, PDF_CMAP_SINGLE, 2052 },
	{ 0x7a42, 0x7a42, PDF_CMAP_SINGLE, 3638 },
	{ 0x7a43, 0x7a43, PDF_CMAP_SINGLE, 5921 },
	{ 0x7a45, 0x7a45, PDF_CMAP_SINGLE, 14923 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 3714 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 5923 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 14924 },
	{ 0x7a4d, 0x7a4d, PDF_CMAP_SINGLE, 2677 },
	{ 0x7a4e, 0x7a4e, PDF_CMAP_SINGLE, 1265 },
	{ 0x7a4f, 0x7a4f, PDF_CMAP_SINGLE, 1338 },
	{ 0x7a50, 0x7a50, PDF_CMAP_SINGLE, 1136 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5922 },
	{ 0x7a5d, 0x7a5d, PDF_CMAP_SINGLE, 14925 },
	{ 0x7a60, 0x7a60, PDF_CMAP_SINGLE, 14926 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5924 },
	{ 0x7a63, 0x7a63, PDF_CMAP_SINGLE, 2527 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5926 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 1453 },
	{ 0x7a6d, 0x7a6d, PDF_CMAP_SINGLE, 14927 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 5928 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 1856 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 1664 },
	{ 0x7a78, 0x7a78, PDF_CMAP_SINGLE, 14928 },
	{ 0x7a79, 0x7a79, PDF_CMAP_SINGLE, 5929 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 1773 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 5930 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 2720 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 3237 },
	{ 0x7a83, 0x7a83, PDF_CMAP_SINGLE, 2692 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 2149 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 5931 },
	{ 0x7a92, 0x7a92, PDF_CMAP_SINGLE, 2976 },
	{ 0x7a93, 0x7a93, PDF_CMAP_SINGLE, 2797 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 5933 },
	{ 0x7a96, 0x7a96, PDF_CMAP_SINGLE, 5935 },
	{ 0x7a97, 0x7a97, PDF_CMAP_SINGLE, 5932 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 5934 },
	{ 0x7a9f, 0x7a9f, PDF_CMAP_SINGLE, 1784 },
	{ 0x7aa0, 0x7aa0, PDF_CMAP_SINGLE, 14929 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 14930 },
	{ 0x7aa9, 0x7aa9, PDF_CMAP_SINGLE, 5936 },
	{ 0x7aaa, 0x7aaa, PDF_CMAP_SINGLE, 1788 },
	{ 0x7aae, 0x7aae, PDF_CMAP_SINGLE, 1665 },
	{ 0x7aaf, 0x7aaf, PDF_CMAP_SINGLE, 3900 },
	{ 0x7ab0, 0x7ab0, PDF_CMAP_SINGLE, 5938 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 14931 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 5939 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 1232 },
	{ 0x7abb, 0x7abc, PDF_CMAP_RANGE, 14932 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 5942 },
	{ 0x7ac3, 0x7ac3, PDF_CMAP_SINGLE, 1492 },
	{ 0x7ac4, 0x7ac4, PDF_CMAP_SINGLE, 5941 },
	{ 0x7ac5, 0x7ac5, PDF_CMAP_SINGLE, 5940 },
	{ 0x7ac6, 0x7ac6, PDF_CMAP_SINGLE, 14934 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 5944 },
	{ 0x7ac8, 0x7ac8, PDF_CMAP_SINGLE, 5937 },
	{ 0x7aca, 0x7aca, PDF_CMAP_SINGLE, 5945 },
	{ 0x7acb, 0x7acb, PDF_CMAP_SINGLE, 3953 },
	{ 0x7acd, 0x7acd, PDF_CMAP_SINGLE, 5946 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 5947 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 8585 },
	{ 0x7ad2, 0x7ad2, PDF_CMAP_SINGLE, 4549 },
	{ 0x7ad3, 0x7ad3, PDF_CMAP_SINGLE, 5949 },
	{ 0x7ad5, 0x7ad5, PDF_CMAP_SINGLE, 5948 },
	{ 0x7ad9, 0x7ada, PDF_CMAP_RANGE, 5950 },
	{ 0x7adc, 0x7adc, PDF_CMAP_SINGLE, 3965 },
	{ 0x7add, 0x7add, PDF_CMAP_SINGLE, 5952 },
	{ 0x7adf, 0x7adf, PDF_CMAP_SINGLE, 7176 },
	{ 0x7ae0, 0x7ae0, PDF_CMAP_SINGLE, 2487 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 5953 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 2401 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 3216 },
	{ 0x7ae6, 0x7ae6, PDF_CMAP_SINGLE, 5955 },
	{ 0x7ae7, 0x7ae7, PDF_CMAP_SINGLE, 8586 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 2918 },
	{ 0x7aeb, 0x7aeb, PDF_CMAP_SINGLE, 8588 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 5956 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 2938 },
	{ 0x7af0, 0x7af0, PDF_CMAP_SINGLE, 5957 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 1693 },
	{ 0x7af8, 0x7af8, PDF_CMAP_SINGLE, 4214 },
	{ 0x7af9, 0x7af9, PDF_CMAP_SINGLE, 2971 },
	{ 0x7afa, 0x7afa, PDF_CMAP_SINGLE, 2271 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1540 },
	{ 0x7b02, 0x7b02, PDF_CMAP_SINGLE, 5958 },
	{ 0x7b04, 0x7b04, PDF_CMAP_SINGLE, 5971 },
	{ 0x7b06, 0x7b06, PDF_CMAP_SINGLE, 5961 },
	{ 0x7b07, 0x7b07, PDF_CMAP_SINGLE, 14935 },
	{ 0x7b08, 0x7b08, PDF_CMAP_SINGLE, 1666 },
	{ 0x7b0a, 0x7b0a, PDF_CMAP_SINGLE, 5960 },
	{ 0x7b0b, 0x7b0b, PDF_CMAP_SINGLE, 5973 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 5959 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 2488 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 14936 },
	{ 0x7b18, 0x7b19, PDF_CMAP_RANGE, 5963 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 3109 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 5965 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 1468 },
	{ 0x7b25, 0x7b25, PDF_CMAP_SINGLE, 2592 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 3542 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 14937 },
	{ 0x7b28, 0x7b28, PDF_CMAP_SINGLE, 5967 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 2888 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 14938 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 5962 },
	{ 0x7b35, 0x7b35, PDF_CMAP_SINGLE, 5966 },
	{ 0x7b36, 0x7b36, PDF_CMAP_SINGLE, 5968 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 2155 },
	{ 0x7b45, 0x7b45, PDF_CMAP_SINGLE, 5975 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 3488 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 14939 },
	{ 0x7b48, 0x7b48, PDF_CMAP_SINGLE, 3386 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 3187 },
	{ 0x7b4b, 0x7b4b, PDF_CMAP_SINGLE, 1746 },
	{ 0x7b4c, 0x7b4c, PDF_CMAP_SINGLE, 5974 },
	{ 0x7b4d, 0x7b4d, PDF_CMAP_SINGLE, 5972 },
	{ 0x7b4e, 0x7b4e, PDF_CMAP_SINGLE, 14940 },
	{ 0x7b4f, 0x7b4f, PDF_CMAP_SINGLE, 3401 },
	{ 0x7b50, 0x7b50, PDF_CMAP_SINGLE, 5969 },
	{ 0x7b51, 0x7b51, PDF_CMAP_SINGLE, 2972 },
	{ 0x7b52, 0x7b52, PDF_CMAP_SINGLE, 3189 },
	{ 0x7b53, 0x7b53, PDF_CMAP_SINGLE, 14173 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 3188 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 2150 },
	{ 0x7b5d, 0x7b5d, PDF_CMAP_SINGLE, 5993 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 14941 },
	{ 0x7b65, 0x7b65, PDF_CMAP_SINGLE, 5977 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 5979 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 14942 },
	{ 0x7b6c, 0x7b6c, PDF_CMAP_SINGLE, 5982 },
	{ 0x7b6d, 0x7b6d, PDF_CMAP_SINGLE, 14943 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 5983 },
	{ 0x7b70, 0x7b71, PDF_CMAP_RANGE, 5980 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 14944 },
	{ 0x7b74, 0x7b74, PDF_CMAP_SINGLE, 5978 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 5976 },
	{ 0x7b7a, 0x7b7a, PDF_CMAP_SINGLE, 5970 },
	{ 0x7b86, 0x7b86, PDF_CMAP_SINGLE, 3615 },
	{ 0x7b87, 0x7b87, PDF_CMAP_SINGLE, 1365 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 5990 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 5987 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 5992 },
	{ 0x7b91, 0x7b91, PDF_CMAP_SINGLE, 14945 },
	{ 0x7b92, 0x7b92, PDF_CMAP_SINGLE, 5991 },
	{ 0x7b94, 0x7b94, PDF_CMAP_SINGLE, 3369 },
	{ 0x7b95, 0x7b95, PDF_CMAP_SINGLE, 3763 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 2185 },
	{ 0x7b98, 0x7b98, PDF_CMAP_SINGLE, 5985 },
	{ 0x7b99, 0x7b99, PDF_CMAP_SINGLE, 5994 },
	{ 0x7b9a, 0x7b9a, PDF_CMAP_SINGLE, 5989 },
	{ 0x7b9c, 0x7b9c, PDF_CMAP_SINGLE, 5988 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 5984 },
	{ 0x7b9e, 0x7b9e, PDF_CMAP_SINGLE, 8589 },
	{ 0x7b9f, 0x7b9f, PDF_CMAP_SINGLE, 5986 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 1541 },
	{ 0x7baa, 0x7baa, PDF_CMAP_SINGLE, 2939 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 2721 },
	{ 0x7baf, 0x7baf, PDF_CMAP_SINGLE, 14946 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3382 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 5999 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 3384 },
	{ 0x7bc0, 0x7bc0, PDF_CMAP_SINGLE, 2693 },
	{ 0x7bc1, 0x7bc1, PDF_CMAP_SINGLE, 5996 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 3427 },
	{ 0x7bc6, 0x7bc6, PDF_CMAP_SINGLE, 6000 },
	{ 0x7bc7, 0x7bc7, PDF_CMAP_SINGLE, 3619 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 2969 },
	{ 0x7bcb, 0x7bcb, PDF_CMAP_SINGLE, 5995 },
	{ 0x7bcc, 0x7bcc, PDF_CMAP_SINGLE, 5997 },
	{ 0x7bcf, 0x7bcf, PDF_CMAP_SINGLE, 5998 },
	{ 0x7bd7, 0x7bd7, PDF_CMAP_SINGLE, 14947 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 14948 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 6001 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 2288 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 3230 },
	{ 0x7be5, 0x7be5, PDF_CMAP_SINGLE, 6006 },
	{ 0x7be6, 0x7be6, PDF_CMAP_SINGLE, 6005 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 6002 },
	{ 0x7bed, 0x7bed, PDF_CMAP_SINGLE, 4060 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 6011 },
	{ 0x7bf6, 0x7bf6, PDF_CMAP_SINGLE, 6015 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 6012 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 6008 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 6009 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 14949 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 6014 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 14950 },
	{ 0x7c11, 0x7c11, PDF_CMAP_SINGLE, 6003 },
	{ 0x7c12, 0x7c12, PDF_CMAP_SINGLE, 4330 },
	{ 0x7c13, 0x7c13, PDF_CMAP_SINGLE, 6010 },
	{ 0x7c14, 0x7c14, PDF_CMAP_SINGLE, 6004 },
	{ 0x7c17, 0x7c17, PDF_CMAP_SINGLE, 6013 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 7739 },
	{ 0x7c1f, 0x7c1f, PDF_CMAP_SINGLE, 6019 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 14951 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 1542 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 6016 },
	{ 0x7c26, 0x7c26, PDF_CMAP_SINGLE, 14952 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 6017 },
	{ 0x7c2a, 0x7c2a, PDF_CMAP_SINGLE, 6018 },
	{ 0x7c2b, 0x7c2b, PDF_CMAP_SINGLE, 6021 },
	{ 0x7c31, 0x7c31, PDF_CMAP_SINGLE, 14953 },
	{ 0x7c36, 0x7c36, PDF_CMAP_SINGLE, 14954 },
	{ 0x7c37, 0x7c37, PDF_CMAP_SINGLE, 6020 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 3466 },
	{ 0x7c3d, 0x7c3d, PDF_CMAP_SINGLE, 6022 },
	{ 0x7c3e, 0x7c3e, PDF_CMAP_SINGLE, 4036 },
	{ 0x7c3f, 0x7c3f, PDF_CMAP_SINGLE, 3645 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 6027 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 6024 },
	{ 0x7c4c, 0x7c4c, PDF_CMAP_SINGLE, 6023 },
	{ 0x7c4d, 0x7c4d, PDF_CMAP_SINGLE, 2678 },
	{ 0x7c4f, 0x7c4f, PDF_CMAP_SINGLE, 6026 },
	{ 0x7c50, 0x7c50, PDF_CMAP_SINGLE, 6028 },
	{ 0x7c51, 0x7c51, PDF_CMAP_SINGLE, 14955 },
	{ 0x7c54, 0x7c54, PDF_CMAP_SINGLE, 6025 },
	{ 0x7c56, 0x7c56, PDF_CMAP_SINGLE, 6032 },
	{ 0x7c58, 0x7c58, PDF_CMAP_SINGLE, 6029 },
	{ 0x7c59, 0x7c59, PDF_CMAP_SINGLE, 14956 },
	{ 0x7c5f, 0x7c5f, PDF_CMAP_SINGLE, 6030 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 6007 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 6031 },
	{ 0x7c65, 0x7c65, PDF_CMAP_SINGLE, 6033 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 14957 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 6034 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 14958 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 14959 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 3606 },
	{ 0x7c75, 0x7c75, PDF_CMAP_SINGLE, 6035 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 14094 },
	{ 0x7c7e, 0x7c7e, PDF_CMAP_SINGLE, 3822 },
	{ 0x7c81, 0x7c81, PDF_CMAP_SINGLE, 1734 },
	{ 0x7c82, 0x7c82, PDF_CMAP_SINGLE, 1791 },
	{ 0x7c83, 0x7c83, PDF_CMAP_SINGLE, 6036 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 3588 },
	{ 0x7c8b, 0x7c8b, PDF_CMAP_SINGLE, 2606 },
	{ 0x7c8d, 0x7c8d, PDF_CMAP_SINGLE, 3772 },
	{ 0x7c90, 0x7c90, PDF_CMAP_SINGLE, 6037 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 3963 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 3370 },
	{ 0x7c97, 0x7c97, PDF_CMAP_SINGLE, 2760 },
	{ 0x7c98, 0x7c98, PDF_CMAP_SINGLE, 3306 },
	{ 0x7c9b, 0x7c9b, PDF_CMAP_SINGLE, 2391 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 1156 },
	{ 0x7ca1, 0x7ca1, PDF_CMAP_SINGLE, 6042 },
	{ 0x7ca2, 0x7ca2, PDF_CMAP_SINGLE, 6040 },
	{ 0x7ca4, 0x7ca4, PDF_CMAP_SINGLE, 6038 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 1501 },
	{ 0x7ca7, 0x7ca7, PDF_CMAP_SINGLE, 2489 },
	{ 0x7ca8, 0x7ca8, PDF_CMAP_SINGLE, 6043 },
	{ 0x7cab, 0x7cab, PDF_CMAP_SINGLE, 6041 },
	{ 0x7cad, 0x7cad, PDF_CMAP_SINGLE, 6039 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 6047 },
	{ 0x7cb1, 0x7cb1, PDF_CMAP_SINGLE, 6046 },
	{ 0x7cb2, 0x7cb2, PDF_CMAP_SINGLE, 6045 },
	{ 0x7cb3, 0x7cb3, PDF_CMAP_SINGLE, 6044 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 6048 },
	{ 0x7cbc, 0x7cbc, PDF_CMAP_SINGLE, 14960 },
	{ 0x7cbd, 0x7cbd, PDF_CMAP_SINGLE, 6049 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 2654 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 14961 },
	{ 0x7cc0, 0x7cc0, PDF_CMAP_SINGLE, 6050 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 6052 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 6051 },
	{ 0x7cc8, 0x7cc9, PDF_CMAP_RANGE, 14962 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 1926 },
	{ 0x7cce, 0x7cce, PDF_CMAP_SINGLE, 2746 },
	{ 0x7cd2, 0x7cd2, PDF_CMAP_SINGLE, 6054 },
	{ 0x7cd6, 0x7cd6, PDF_CMAP_SINGLE, 3190 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 14964 },
	{ 0x7cd8, 0x7cd8, PDF_CMAP_SINGLE, 6053 },
	{ 0x7cd9, 0x7cd9, PDF_CMAP_SINGLE, 14965 },
	{ 0x7cdc, 0x7cdc, PDF_CMAP_SINGLE, 6055 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 14966 },
	{ 0x7cde, 0x7cde, PDF_CMAP_SINGLE, 3589 },
	{ 0x7cdf, 0x7cdf, PDF_CMAP_SINGLE, 2798 },
	{ 0x7ce0, 0x7ce0, PDF_CMAP_SINGLE, 2009 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 6056 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 3984 },
	{ 0x7ceb, 0x7ceb, PDF_CMAP_SINGLE, 14967 },
	{ 0x7cef, 0x7cef, PDF_CMAP_SINGLE, 6058 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 6059 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 6060 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 6061 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 2227 },
	{ 0x7cfa, 0x7cfa, PDF_CMAP_SINGLE, 6062 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 1829 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 1668 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 1604 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 6064 },
	{ 0x7d04, 0x7d04, PDF_CMAP_SINGLE, 3839 },
	{ 0x7d05, 0x7d05, PDF_CMAP_SINGLE, 2010 },
	{ 0x7d06, 0x7d06, PDF_CMAP_SINGLE, 6063 },
	{ 0x7d07, 0x7d09, PDF_CMAP_RANGE, 14968 },
	{ 0x7d0a, 0x7d0a, PDF_CMAP_SINGLE, 6067 },
	{ 0x7d0b, 0x7d0b, PDF_CMAP_SINGLE, 3826 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 3314 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 3493 },
	{ 0x7d13, 0x7d13, PDF_CMAP_SINGLE, 14971 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 2413 },
	{ 0x7d15, 0x7d15, PDF_CMAP_SINGLE, 6066 },
	{ 0x7d17, 0x7d17, PDF_CMAP_SINGLE, 2303 },
	{ 0x7d18, 0x7d18, PDF_CMAP_SINGLE, 2011 },
	{ 0x7d19, 0x7d19, PDF_CMAP_SINGLE, 2228 },
	{ 0x7d1a, 0x7d1a, PDF_CMAP_SINGLE, 1667 },
	{ 0x7d1b, 0x7d1b, PDF_CMAP_SINGLE, 3590 },
	{ 0x7d1c, 0x7d1c, PDF_CMAP_SINGLE, 6065 },
	{ 0x7d1d, 0x7d1d, PDF_CMAP_SINGLE, 14972 },
	{ 0x7d20, 0x7d20, PDF_CMAP_SINGLE, 2761 },
	{ 0x7d21, 0x7d21, PDF_CMAP_SINGLE, 3696 },
	{ 0x7d22, 0x7d22, PDF_CMAP_SINGLE, 2151 },
	{ 0x7d23, 0x7d23, PDF_CMAP_SINGLE, 14973 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 2229 },
	{ 0x7d2c, 0x7d2c, PDF_CMAP_SINGLE, 3065 },
	{ 0x7d2e, 0x7d2e, PDF_CMAP_SINGLE, 6070 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 4007 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 2121 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 6071 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 2568 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 6073 },
	{ 0x7d39, 0x7d39, PDF_CMAP_SINGLE, 2490 },
	{ 0x7d3a, 0x7d3a, PDF_CMAP_SINGLE, 2080 },
	{ 0x7d3f, 0x7d3f, PDF_CMAP_SINGLE, 6072 },
	{ 0x7d41, 0x7d41, PDF_CMAP_SINGLE, 14974 },
	{ 0x7d42, 0x7d42, PDF_CMAP_SINGLE, 2356 },
	{ 0x7d43, 0x7d43, PDF_CMAP_SINGLE, 1906 },
	{ 0x7d44, 0x7d44, PDF_CMAP_SINGLE, 2762 },
	{ 0x7d45, 0x7d45, PDF_CMAP_SINGLE, 6068 },
	{ 0x7d46, 0x7d46, PDF_CMAP_SINGLE, 6074 },
	{ 0x7d48, 0x7d48, PDF_CMAP_SINGLE, 8591 },
	{ 0x7d4b, 0x7d4b, PDF_CMAP_SINGLE, 6069 },
	{ 0x7d4c, 0x7d4c, PDF_CMAP_SINGLE, 1830 },
	{ 0x7d4e, 0x7d4e, PDF_CMAP_SINGLE, 6077 },
	{ 0x7d4f, 0x7d4f, PDF_CMAP_SINGLE, 6081 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 1857 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 14975 },
	{ 0x7d56, 0x7d56, PDF_CMAP_SINGLE, 6076 },
	{ 0x7d59, 0x7d59, PDF_CMAP_SINGLE, 14976 },
	{ 0x7d5b, 0x7d5b, PDF_CMAP_SINGLE, 6085 },
	{ 0x7d5c, 0x7d5c, PDF_CMAP_SINGLE, 8592 },
	{ 0x7d5d, 0x7d5d, PDF_CMAP_SINGLE, 14977 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 2012 },
	{ 0x7d61, 0x7d61, PDF_CMAP_SINGLE, 3927 },
	{ 0x7d62, 0x7d62, PDF_CMAP_SINGLE, 1152 },
	{ 0x7d63, 0x7d63, PDF_CMAP_SINGLE, 6082 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 1669 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 6079 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 6080 },
	{ 0x7d71, 0x7d71, PDF_CMAP_SINGLE, 3191 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 6078 },
	{ 0x7d73, 0x7d73, PDF_CMAP_SINGLE, 6075 },
	{ 0x7d75, 0x7d75, PDF_CMAP_SINGLE, 1414 },
	{ 0x7d76, 0x7d76, PDF_CMAP_SINGLE, 2696 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 1884 },
	{ 0x7d7a, 0x7d7a, PDF_CMAP_SINGLE, 14978 },
	{ 0x7d7d, 0x7d7d, PDF_CMAP_SINGLE, 6087 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 14979 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 6084 },
	{ 0x7d8b, 0x7d8c, PDF_CMAP_RANGE, 14980 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 6086 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 6083 },
	{ 0x7d99, 0x7d99, PDF_CMAP_SINGLE, 1831 },
	{ 0x7d9a, 0x7d9a, PDF_CMAP_SINGLE, 2835 },
	{ 0x7d9b, 0x7d9b, PDF_CMAP_SINGLE, 6088 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 2800 },
	{ 0x7d9f, 0x7d9f, PDF_CMAP_SINGLE, 6101 },
	{ 0x7da0, 0x7da0, PDF_CMAP_SINGLE, 8594 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 6097 },
	{ 0x7da3, 0x7da3, PDF_CMAP_SINGLE, 6091 },
	{ 0x7dab, 0x7dab, PDF_CMAP_SINGLE, 6095 },
	{ 0x7dac, 0x7dac, PDF_CMAP_SINGLE, 2342 },
	{ 0x7dad, 0x7dad, PDF_CMAP_SINGLE, 1185 },
	{ 0x7dae, 0x7dae, PDF_CMAP_SINGLE, 6090 },
	{ 0x7daf, 0x7daf, PDF_CMAP_SINGLE, 6098 },
	{ 0x7db0, 0x7db0, PDF_CMAP_SINGLE, 6102 },
	{ 0x7db1, 0x7db1, PDF_CMAP_SINGLE, 2013 },
	{ 0x7db2, 0x7db2, PDF_CMAP_SINGLE, 3810 },
	{ 0x7db4, 0x7db4, PDF_CMAP_SINGLE, 3058 },
	{ 0x7db5, 0x7db5, PDF_CMAP_SINGLE, 6092 },
	{ 0x7db7, 0x7db7, PDF_CMAP_SINGLE, 8593 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 6100 },
	{ 0x7dba, 0x7dba, PDF_CMAP_SINGLE, 6089 },
	{ 0x7dbb, 0x7dbb, PDF_CMAP_SINGLE, 2940 },
	{ 0x7dbd, 0x7dbd, PDF_CMAP_SINGLE, 6094 },
	{ 0x7dbe, 0x7dbe, PDF_CMAP_SINGLE, 1153 },
	{ 0x7dbf, 0x7dbf, PDF_CMAP_SINGLE, 3798 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 6093 },
	{ 0x7dca, 0x7dca, PDF_CMAP_SINGLE, 1747 },
	{ 0x7dcb, 0x7dcb, PDF_CMAP_SINGLE, 3456 },
	{ 0x7dcc, 0x7dcc, PDF_CMAP_SINGLE, 14982 },
	{ 0x7dcf, 0x7dcf, PDF_CMAP_SINGLE, 2799 },
	{ 0x7dd1, 0x7dd1, PDF_CMAP_SINGLE, 3992 },
	{ 0x7dd2, 0x7dd2, PDF_CMAP_SINGLE, 2425 },
	{ 0x7dd5, 0x7dd5, PDF_CMAP_SINGLE, 6141 },
	{ 0x7dd6, 0x7dd6, PDF_CMAP_SINGLE, 8595 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 6103 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 2722 },
	{ 0x7ddc, 0x7ddc, PDF_CMAP_SINGLE, 6099 },
	{ 0x7ddd, 0x7ddd, PDF_CMAP_SINGLE, 6104 },
	{ 0x7dde, 0x7dde, PDF_CMAP_SINGLE, 6106 },
	{ 0x7de0, 0x7de0, PDF_CMAP_SINGLE, 3093 },
	{ 0x7de1, 0x7de1, PDF_CMAP_SINGLE, 6109 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 13322 },
	{ 0x7de4, 0x7de4, PDF_CMAP_SINGLE, 6105 },
	{ 0x7de8, 0x7de8, PDF_CMAP_SINGLE, 3620 },
	{ 0x7de9, 0x7de9, PDF_CMAP_SINGLE, 1543 },
	{ 0x7deb, 0x7deb, PDF_CMAP_SINGLE, 14983 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 3799 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 1186 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 14984 },
	{ 0x7df2, 0x7df2, PDF_CMAP_SINGLE, 6108 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4037 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 14985 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 6107 },
	{ 0x7e01, 0x7e01, PDF_CMAP_SINGLE, 1297 },
	{ 0x7e04, 0x7e04, PDF_CMAP_SINGLE, 3268 },
	{ 0x7e05, 0x7e05, PDF_CMAP_SINGLE, 6110 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 14986 },
	{ 0x7e09, 0x7e09, PDF_CMAP_SINGLE, 6117 },
	{ 0x7e0a, 0x7e0a, PDF_CMAP_SINGLE, 6111 },
	{ 0x7e0b, 0x7e0b, PDF_CMAP_SINGLE, 6118 },
	{ 0x7e11, 0x7e11, PDF_CMAP_SINGLE, 14987 },
	{ 0x7e12, 0x7e12, PDF_CMAP_SINGLE, 6114 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 14988 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 3377 },
	{ 0x7e1e, 0x7e1e, PDF_CMAP_SINGLE, 2294 },
	{ 0x7e1f, 0x7e1f, PDF_CMAP_SINGLE, 6116 },
	{ 0x7e20, 0x7e20, PDF_CMAP_SINGLE, 14989 },
	{ 0x7e21, 0x7e21, PDF_CMAP_SINGLE, 6113 },
	{ 0x7e22, 0x7e22, PDF_CMAP_SINGLE, 6119 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 6112 },
	{ 0x7e26, 0x7e26, PDF_CMAP_SINGLE, 2382 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 3667 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 2390 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 6115 },
	{ 0x7e32, 0x7e32, PDF_CMAP_SINGLE, 6127 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 6123 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 6126 },
	{ 0x7e39, 0x7e39, PDF_CMAP_SINGLE, 6124 },
	{ 0x7e3a, 0x7e3a, PDF_CMAP_SINGLE, 6128 },
	{ 0x7e3b, 0x7e3b, PDF_CMAP_SINGLE, 6122 },
	{ 0x7e3d, 0x7e3d, PDF_CMAP_SINGLE, 6096 },
	{ 0x7e3e, 0x7e3e, PDF_CMAP_SINGLE, 2679 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 3423 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 6125 },
	{ 0x7e46, 0x7e46, PDF_CMAP_SINGLE, 6120 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 14990 },
	{ 0x7e48, 0x7e48, PDF_CMAP_SINGLE, 14183 },
	{ 0x7e4a, 0x7e4a, PDF_CMAP_SINGLE, 2723 },
	{ 0x7e4b, 0x7e4b, PDF_CMAP_SINGLE, 1832 },
	{ 0x7e4d, 0x7e4d, PDF_CMAP_SINGLE, 2357 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 8596 },
	{ 0x7e54, 0x7e54, PDF_CMAP_SINGLE, 2539 },
	{ 0x7e55, 0x7e55, PDF_CMAP_SINGLE, 2744 },
	{ 0x7e56, 0x7e56, PDF_CMAP_SINGLE, 6131 },
	{ 0x7e59, 0x7e5a, PDF_CMAP_RANGE, 6133 },
	{ 0x7e5d, 0x7e5d, PDF_CMAP_SINGLE, 6130 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 6132 },
	{ 0x7e61, 0x7e61, PDF_CMAP_SINGLE, 7697 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 14991 },
	{ 0x7e66, 0x7e66, PDF_CMAP_SINGLE, 6121 },
	{ 0x7e67, 0x7e67, PDF_CMAP_SINGLE, 6129 },
	{ 0x7e69, 0x7e69, PDF_CMAP_SINGLE, 6137 },
	{ 0x7e6a, 0x7e6a, PDF_CMAP_SINGLE, 6136 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 7671 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 3752 },
	{ 0x7e6e, 0x7e6e, PDF_CMAP_SINGLE, 14992 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 1793 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 14993 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 6135 },
	{ 0x7e7b, 0x7e7b, PDF_CMAP_SINGLE, 6139 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 6138 },
	{ 0x7e7d, 0x7e7d, PDF_CMAP_SINGLE, 6142 },
	{ 0x7e7f, 0x7e7f, PDF_CMAP_SINGLE, 6144 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 2186 },
	{ 0x7e83, 0x7e83, PDF_CMAP_SINGLE, 6140 },
	{ 0x7e88, 0x7e89, PDF_CMAP_RANGE, 6145 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 8359 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 6147 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 14994 },
	{ 0x7e8e, 0x7e8e, PDF_CMAP_SINGLE, 6153 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 3125 },
	{ 0x7e90, 0x7e90, PDF_CMAP_SINGLE, 6149 },
	{ 0x7e91, 0x7e91, PDF_CMAP_SINGLE, 14995 },
	{ 0x7e92, 0x7e92, PDF_CMAP_SINGLE, 6148 },
	{ 0x7e93, 0x7e94, PDF_CMAP_RANGE, 6150 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 6152 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 14996 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_RANGE, 6154 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 1544 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 6156 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 6157 },
	{ 0x7f44, 0x7f44, PDF_CMAP_SINGLE, 14997 },
	{ 0x7f45, 0x7f45, PDF_CMAP_SINGLE, 6158 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 8597 },
	{ 0x7f4c, 0x7f4e, PDF_CMAP_RANGE, 6159 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 14998 },
	{ 0x7f50, 0x7f51, PDF_CMAP_RANGE, 6162 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 14999 },
	{ 0x7f54, 0x7f54, PDF_CMAP_SINGLE, 6165 },
	{ 0x7f55, 0x7f55, PDF_CMAP_SINGLE, 6164 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 6166 },
	{ 0x7f5f, 0x7f60, PDF_CMAP_RANGE, 6167 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 15001 },
	{ 0x7f67, 0x7f67, PDF_CMAP_SINGLE, 6171 },
	{ 0x7f68, 0x7f69, PDF_CMAP_RANGE, 6169 },
	{ 0x7f6a, 0x7f6a, PDF_CMAP_SINGLE, 2129 },
	{ 0x7f6b, 0x7f6b, PDF_CMAP_SINGLE, 1833 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 2964 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 3399 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 2426 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 3331 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 3457 },
	{ 0x7f78, 0x7f78, PDF_CMAP_SINGLE, 6172 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 4918 },
	{ 0x7f82, 0x7f82, PDF_CMAP_SINGLE, 6173 },
	{ 0x7f83, 0x7f83, PDF_CMAP_SINGLE, 6175 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 3919 },
	{ 0x7f86, 0x7f86, PDF_CMAP_SINGLE, 6174 },
	{ 0x7f87, 0x7f87, PDF_CMAP_SINGLE, 6177 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 6176 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 3901 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 6178 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 3474 },
	{ 0x7f91, 0x7f91, PDF_CMAP_SINGLE, 15002 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 6179 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 6182 },
	{ 0x7f9d, 0x7f9d, PDF_CMAP_SINGLE, 6181 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 6180 },
	{ 0x7fa1, 0x7fa1, PDF_CMAP_SINGLE, 8598 },
	{ 0x7fa3, 0x7fa3, PDF_CMAP_SINGLE, 6183 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 1800 },
	{ 0x7fa8, 0x7fa8, PDF_CMAP_SINGLE, 2724 },
	{ 0x7fa9, 0x7fa9, PDF_CMAP_SINGLE, 1627 },
	{ 0x7fae, 0x7fae, PDF_CMAP_SINGLE, 6187 },
	{ 0x7faf, 0x7faf, PDF_CMAP_SINGLE, 6184 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 6185 },
	{ 0x7fb6, 0x7fb6, PDF_CMAP_SINGLE, 6188 },
	{ 0x7fb8, 0x7fb8, PDF_CMAP_SINGLE, 6189 },
	{ 0x7fb9, 0x7fb9, PDF_CMAP_SINGLE, 6186 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 1227 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 15003 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 1319 },
	{ 0x7fc5, 0x7fc6, PDF_CMAP_RANGE, 6191 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 6193 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 3916 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 15004 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2358 },
	{ 0x7fd4, 0x7fd4, PDF_CMAP_SINGLE, 6195 },
	{ 0x7fd5, 0x7fd5, PDF_CMAP_SINGLE, 6194 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 15005 },
	{ 0x7fe0, 0x7fe0, PDF_CMAP_SINGLE, 2607 },
	{ 0x7fe1, 0x7fe1, PDF_CMAP_SINGLE, 6196 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 15006 },
	{ 0x7fe6, 0x7fe6, PDF_CMAP_SINGLE, 6197 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 6198 },
	{ 0x7feb, 0x7feb, PDF_CMAP_SINGLE, 1569 },
	{ 0x7fec, 0x7fec, PDF_CMAP_SINGLE, 15007 },
	{ 0x7fee, 0x7fef, PDF_CMAP_RANGE, 15008 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 1545 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 6199 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 6200 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 15010 },
	{ 0x7ffb, 0x7ffb, PDF_CMAP_SINGLE, 3723 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 3917 },
	{ 0x8000, 0x8000, PDF_CMAP_SINGLE, 3902 },
	{ 0x8001, 0x8001, PDF_CMAP_SINGLE, 4061 },
	{ 0x8002, 0x8002, PDF_CMAP_SINGLE, 14099 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 2015 },
	{ 0x8004, 0x8004, PDF_CMAP_SINGLE, 6203 },
	{ 0x8005, 0x8005, PDF_CMAP_SINGLE, 2304 },
	{ 0x8006, 0x8006, PDF_CMAP_SINGLE, 6202 },
	{ 0x800b, 0x800b, PDF_CMAP_SINGLE, 6204 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 2261 },
	{ 0x800e, 0x800e, PDF_CMAP_SINGLE, 15011 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 2865 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 15012 },
	{ 0x8012, 0x8012, PDF_CMAP_SINGLE, 6205 },
	{ 0x8014, 0x8014, PDF_CMAP_SINGLE, 15013 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 2014 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 3811 },
	{ 0x8018, 0x8019, PDF_CMAP_RANGE, 6206 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 6208 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 6209 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 15014 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 15015 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 6210 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 2262 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 3833 },
	{ 0x803a, 0x803a, PDF_CMAP_SINGLE, 15016 },
	{ 0x803b, 0x803b, PDF_CMAP_SINGLE, 6212 },
	{ 0x803c, 0x803c, PDF_CMAP_SINGLE, 15017 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 2941 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 6211 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 6214 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 6213 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 6215 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 2655 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 6216 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 6217 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 3593 },
	{ 0x805f, 0x805f, PDF_CMAP_SINGLE, 6218 },
	{ 0x8060, 0x8060, PDF_CMAP_SINGLE, 15018 },
	{ 0x8061, 0x8061, PDF_CMAP_SINGLE, 2801 },
	{ 0x8062, 0x8062, PDF_CMAP_SINGLE, 6219 },
	{ 0x8068, 0x8068, PDF_CMAP_SINGLE, 6220 },
	{ 0x806f, 0x806f, PDF_CMAP_SINGLE, 4038 },
	{ 0x8070, 0x8070, PDF_CMAP_SINGLE, 6223 },
	{ 0x8071, 0x8071, PDF_CMAP_SINGLE, 15019 },
	{ 0x8072, 0x8072, PDF_CMAP_SINGLE, 6222 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 6221 },
	{ 0x8074, 0x8074, PDF_CMAP_SINGLE, 3020 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 15020 },
	{ 0x8076, 0x8076, PDF_CMAP_SINGLE, 6224 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 2540 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 6225 },
	{ 0x807d, 0x807d, PDF_CMAP_SINGLE, 6226 },
	{ 0x807e, 0x807e, PDF_CMAP_SINGLE, 4062 },
	{ 0x807f, 0x807f, PDF_CMAP_SINGLE, 6227 },
	{ 0x8084, 0x8084, PDF_CMAP_SINGLE, 6228 },
	{ 0x8085, 0x8085, PDF_CMAP_SINGLE, 6230 },
	{ 0x8086, 0x8086, PDF_CMAP_SINGLE, 6229 },
	{ 0x8087, 0x8087, PDF_CMAP_SINGLE, 3385 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 3281 },
	{ 0x808b, 0x808b, PDF_CMAP_SINGLE, 4068 },
	{ 0x808c, 0x808c, PDF_CMAP_SINGLE, 3389 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6232 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 2491 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 3484 },
	{ 0x809a, 0x809a, PDF_CMAP_SINGLE, 6233 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 6231 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 1546 },
	{ 0x809e, 0x809e, PDF_CMAP_SINGLE, 15021 },
	{ 0x80a1, 0x80a1, PDF_CMAP_SINGLE, 1928 },
	{ 0x80a2, 0x80a2, PDF_CMAP_SINGLE, 2230 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 3458 },
	{ 0x80a6, 0x80a6, PDF_CMAP_SINGLE, 15022 },
	{ 0x80a9, 0x80a9, PDF_CMAP_SINGLE, 1886 },
	{ 0x80aa, 0x80aa, PDF_CMAP_SINGLE, 3697 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 15023 },
	{ 0x80ac, 0x80ac, PDF_CMAP_SINGLE, 6236 },
	{ 0x80ad, 0x80ad, PDF_CMAP_SINGLE, 6234 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 2016 },
	{ 0x80b1, 0x80b1, PDF_CMAP_SINGLE, 2017 },
	{ 0x80b2, 0x80b2, PDF_CMAP_SINGLE, 1197 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 2136 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 3343 },
	{ 0x80c3, 0x80c3, PDF_CMAP_SINGLE, 1187 },
	{ 0x80c4, 0x80c4, PDF_CMAP_SINGLE, 6241 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 2942 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 3342 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 2875 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 6243 },
	{ 0x80d7, 0x80d8, PDF_CMAP_RANGE, 15024 },
	{ 0x80d9, 0x80d9, PDF_CMAP_SINGLE, 6239 },
	{ 0x80da, 0x80da, PDF_CMAP_SINGLE, 6242 },
	{ 0x80db, 0x80db, PDF_CMAP_SINGLE, 6237 },
	{ 0x80dd, 0x80dd, PDF_CMAP_SINGLE, 6240 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 3668 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1929 },
	{ 0x80e4, 0x80e4, PDF_CMAP_SINGLE, 1217 },
	{ 0x80e5, 0x80e5, PDF_CMAP_SINGLE, 6238 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 6245 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 6246 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 3217 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 1715 },
	{ 0x80fc, 0x80fc, PDF_CMAP_SINGLE, 6257 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 3315 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 2231 },
	{ 0x8105, 0x8105, PDF_CMAP_SINGLE, 1716 },
	{ 0x8106, 0x8106, PDF_CMAP_SINGLE, 2668 },
	{ 0x8107, 0x8107, PDF_CMAP_SINGLE, 4076 },
	{ 0x8108, 0x8108, PDF_CMAP_SINGLE, 3770 },
	{ 0x8109, 0x8109, PDF_CMAP_SINGLE, 6244 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 2680 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 15026 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 15027 },
	{ 0x811a, 0x811a, PDF_CMAP_SINGLE, 1645 },
	{ 0x811b, 0x811b, PDF_CMAP_SINGLE, 6247 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 6249 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 6248 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 13913 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 6250 },
	{ 0x8131, 0x8131, PDF_CMAP_SINGLE, 2916 },
	{ 0x8133, 0x8133, PDF_CMAP_SINGLE, 3316 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 3021 },
	{ 0x813a, 0x813a, PDF_CMAP_SINGLE, 15028 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 6254 },
	{ 0x8141, 0x8141, PDF_CMAP_SINGLE, 14194 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 6253 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 15029 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 6251 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 15030 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 2587 },
	{ 0x8150, 0x8150, PDF_CMAP_SINGLE, 3543 },
	{ 0x8151, 0x8151, PDF_CMAP_SINGLE, 6256 },
	{ 0x8153, 0x8153, PDF_CMAP_SINGLE, 6255 },
	{ 0x8154, 0x8154, PDF_CMAP_SINGLE, 2018 },
	{ 0x8155, 0x8155, PDF_CMAP_SINGLE, 4089 },
	{ 0x815f, 0x815f, PDF_CMAP_SINGLE, 6272 },
	{ 0x8165, 0x8166, PDF_CMAP_RANGE, 6260 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 2332 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 6259 },
	{ 0x8170, 0x8170, PDF_CMAP_SINGLE, 2058 },
	{ 0x8171, 0x8171, PDF_CMAP_SINGLE, 6258 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6262 },
	{ 0x8178, 0x8178, PDF_CMAP_SINGLE, 3022 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 3570 },
	{ 0x817a, 0x817a, PDF_CMAP_SINGLE, 2725 },
	{ 0x817f, 0x817f, PDF_CMAP_SINGLE, 2876 },
	{ 0x8180, 0x8180, PDF_CMAP_SINGLE, 6266 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 15031 },
	{ 0x8182, 0x8182, PDF_CMAP_SINGLE, 6267 },
	{ 0x8183, 0x8183, PDF_CMAP_SINGLE, 6263 },
	{ 0x8184, 0x8184, PDF_CMAP_SINGLE, 15032 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 6264 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 6265 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 2019 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 6273 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 6269 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 3544 },
	{ 0x819c, 0x819c, PDF_CMAP_SINGLE, 3738 },
	{ 0x819d, 0x819d, PDF_CMAP_SINGLE, 3482 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 6268 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 6271 },
	{ 0x81a4, 0x81a4, PDF_CMAP_SINGLE, 6270 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 3698 },
	{ 0x81a9, 0x81a9, PDF_CMAP_SINGLE, 6274 },
	{ 0x81b0, 0x81b0, PDF_CMAP_SINGLE, 6275 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 2745 },
	{ 0x81b4, 0x81b4, PDF_CMAP_SINGLE, 15033 },
	{ 0x81b5, 0x81b5, PDF_CMAP_SINGLE, 6276 },
	{ 0x81b8, 0x81b8, PDF_CMAP_SINGLE, 6278 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 6282 },
	{ 0x81bd, 0x81bd, PDF_CMAP_SINGLE, 6279 },
	{ 0x81be, 0x81be, PDF_CMAP_SINGLE, 6277 },
	{ 0x81bf, 0x81bf, PDF_CMAP_SINGLE, 3317 },
	{ 0x81c0, 0x81c0, PDF_CMAP_SINGLE, 6280 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 6281 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 1330 },
	{ 0x81c8, 0x81c8, PDF_CMAP_SINGLE, 6288 },
	{ 0x81c9, 0x81c9, PDF_CMAP_SINGLE, 6283 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 6284 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 15034 },
	{ 0x81d1, 0x81d1, PDF_CMAP_SINGLE, 6285 },
	{ 0x81d3, 0x81d3, PDF_CMAP_SINGLE, 2817 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 6287 },
	{ 0x81d9, 0x81d9, PDF_CMAP_SINGLE, 6286 },
	{ 0x81da, 0x81da, PDF_CMAP_SINGLE, 6289 },
	{ 0x81df, 0x81e0, PDF_CMAP_RANGE, 6290 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 2569 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 1385 },
	{ 0x81e7, 0x81e7, PDF_CMAP_SINGLE, 6292 },
	{ 0x81e8, 0x81e8, PDF_CMAP_SINGLE, 3999 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 2263 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 2359 },
	{ 0x81f3, 0x81f3, PDF_CMAP_SINGLE, 2232 },
	{ 0x81f4, 0x81f4, PDF_CMAP_SINGLE, 2965 },
	{ 0x81f9, 0x81f9, PDF_CMAP_SINGLE, 15035 },
	{ 0x81fa, 0x81fb, PDF_CMAP_RANGE, 6293 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 1235 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 6295 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 6296 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 15036 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 6298 },
	{ 0x8207, 0x8207, PDF_CMAP_SINGLE, 6299 },
	{ 0x8208, 0x8208, PDF_CMAP_SINGLE, 1717 },
	{ 0x8209, 0x8209, PDF_CMAP_SINGLE, 5039 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 6300 },
	{ 0x820c, 0x820c, PDF_CMAP_SINGLE, 2697 },
	{ 0x820d, 0x820d, PDF_CMAP_SINGLE, 6301 },
	{ 0x820e, 0x820e, PDF_CMAP_SINGLE, 2295 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 6302 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 4105 },
	{ 0x8216, 0x8216, PDF_CMAP_SINGLE, 6303 },
	{ 0x8217, 0x8217, PDF_CMAP_SINGLE, 3630 },
	{ 0x8218, 0x8218, PDF_CMAP_SINGLE, 1560 },
	{ 0x821b, 0x821b, PDF_CMAP_SINGLE, 2726 },
	{ 0x821c, 0x821c, PDF_CMAP_SINGLE, 2402 },
	{ 0x821e, 0x821e, PDF_CMAP_SINGLE, 3555 },
	{ 0x821f, 0x821f, PDF_CMAP_SINGLE, 2360 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 15037 },
	{ 0x8229, 0x8229, PDF_CMAP_SINGLE, 6304 },
	{ 0x822a, 0x822a, PDF_CMAP_SINGLE, 2020 },
	{ 0x822b, 0x822b, PDF_CMAP_SINGLE, 6305 },
	{ 0x822c, 0x822c, PDF_CMAP_SINGLE, 3424 },
	{ 0x822e, 0x822e, PDF_CMAP_SINGLE, 6319 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 15038 },
	{ 0x8233, 0x8233, PDF_CMAP_SINGLE, 6307 },
	{ 0x8234, 0x8234, PDF_CMAP_SINGLE, 15039 },
	{ 0x8235, 0x8235, PDF_CMAP_SINGLE, 2857 },
	{ 0x8236, 0x8236, PDF_CMAP_SINGLE, 3371 },
	{ 0x8237, 0x8237, PDF_CMAP_SINGLE, 1907 },
	{ 0x8238, 0x8238, PDF_CMAP_SINGLE, 6306 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 2727 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 6308 },
	{ 0x8246, 0x8246, PDF_CMAP_SINGLE, 15040 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3094 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 15041 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 15042 },
	{ 0x8258, 0x8258, PDF_CMAP_SINGLE, 6310 },
	{ 0x8259, 0x8259, PDF_CMAP_SINGLE, 6309 },
	{ 0x825a, 0x825a, PDF_CMAP_SINGLE, 6312 },
	{ 0x825d, 0x825d, PDF_CMAP_SINGLE, 6311 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 6313 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 6315 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 6314 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 1547 },
	{ 0x8268, 0x8268, PDF_CMAP_SINGLE, 6316 },
	{ 0x826a, 0x826b, PDF_CMAP_RANGE, 6317 },
	{ 0x826e, 0x826e, PDF_CMAP_SINGLE, 2081 },
	{ 0x826f, 0x826f, PDF_CMAP_SINGLE, 3985 },
	{ 0x8271, 0x8271, PDF_CMAP_SINGLE, 6320 },
	{ 0x8272, 0x8272, PDF_CMAP_SINGLE, 2541 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 1298 },
	{ 0x8277, 0x8278, PDF_CMAP_RANGE, 6321 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 14197 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 6323 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 1206 },
	{ 0x828d, 0x828d, PDF_CMAP_SINGLE, 6324 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 15043 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 6325 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 3545 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 2291 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 6327 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 1415 },
	{ 0x82a6, 0x82a6, PDF_CMAP_SINGLE, 1142 },
	{ 0x82ab, 0x82ab, PDF_CMAP_SINGLE, 6326 },
	{ 0x82ac, 0x82ac, PDF_CMAP_SINGLE, 6329 },
	{ 0x82ad, 0x82ad, PDF_CMAP_SINGLE, 3332 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 15044 },
	{ 0x82af, 0x82af, PDF_CMAP_SINGLE, 2570 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1366 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 3669 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 15045 },
	{ 0x82b8, 0x82b8, PDF_CMAP_SINGLE, 1843 },
	{ 0x82b9, 0x82b9, PDF_CMAP_SINGLE, 1748 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 6328 },
	{ 0x82bd, 0x82bd, PDF_CMAP_SINGLE, 1386 },
	{ 0x82be, 0x82be, PDF_CMAP_SINGLE, 15046 },
	{ 0x82c5, 0x82c5, PDF_CMAP_SINGLE, 1503 },
	{ 0x82c6, 0x82c6, PDF_CMAP_SINGLE, 15047 },
	{ 0x82d1, 0x82d1, PDF_CMAP_SINGLE, 1299 },
	{ 0x82d2, 0x82d2, PDF_CMAP_SINGLE, 6333 },
	{ 0x82d3, 0x82d3, PDF_CMAP_SINGLE, 4018 },
	{ 0x82d4, 0x82d4, PDF_CMAP_SINGLE, 2877 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 3510 },
	{ 0x82d9, 0x82d9, PDF_CMAP_SINGLE, 6345 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 1367 },
	{ 0x82dc, 0x82dc, PDF_CMAP_SINGLE, 6343 },
	{ 0x82de, 0x82de, PDF_CMAP_SINGLE, 6341 },
	{ 0x82df, 0x82df, PDF_CMAP_SINGLE, 6332 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 6330 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 6331 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 2319 },
	{ 0x82e6, 0x82e6, PDF_CMAP_SINGLE, 1764 },
	{ 0x82e7, 0x82e7, PDF_CMAP_SINGLE, 2997 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3241 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 1267 },
	{ 0x82f3, 0x82f3, PDF_CMAP_SINGLE, 6335 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 6334 },
	{ 0x82f9, 0x82f9, PDF_CMAP_SINGLE, 6340 },
	{ 0x82fa, 0x82fa, PDF_CMAP_SINGLE, 6336 },
	{ 0x82fb, 0x82fb, PDF_CMAP_SINGLE, 6339 },
	{ 0x82fe, 0x82fe, PDF_CMAP_SINGLE, 15048 },
	{ 0x8301, 0x8301, PDF_CMAP_SINGLE, 8600 },
	{ 0x8302, 0x8302, PDF_CMAP_SINGLE, 3804 },
	{ 0x8303, 0x8303, PDF_CMAP_SINGLE, 6338 },
	{ 0x8304, 0x8304, PDF_CMAP_SINGLE, 1368 },
	{ 0x8305, 0x8305, PDF_CMAP_SINGLE, 1499 },
	{ 0x8306, 0x8306, PDF_CMAP_SINGLE, 6342 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 6344 },
	{ 0x830e, 0x830e, PDF_CMAP_SINGLE, 1834 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 6348 },
	{ 0x8317, 0x8318, PDF_CMAP_RANGE, 6357 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 1135 },
	{ 0x8323, 0x8323, PDF_CMAP_SINGLE, 6365 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 1205 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 6356 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 6355 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 6350 },
	{ 0x8332, 0x8332, PDF_CMAP_SINGLE, 6349 },
	{ 0x8334, 0x8334, PDF_CMAP_SINGLE, 6347 },
	{ 0x8335, 0x8335, PDF_CMAP_SINGLE, 6346 },
	{ 0x8336, 0x8336, PDF_CMAP_SINGLE, 2977 },
	{ 0x8338, 0x8338, PDF_CMAP_SINGLE, 2907 },
	{ 0x8339, 0x8339, PDF_CMAP_SINGLE, 6352 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 6351 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 15049 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 6354 },
	{ 0x8349, 0x8349, PDF_CMAP_SINGLE, 2802 },
	{ 0x834a, 0x834a, PDF_CMAP_SINGLE, 1835 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 1251 },
	{ 0x8350, 0x8350, PDF_CMAP_SINGLE, 6353 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 15050 },
	{ 0x8352, 0x8352, PDF_CMAP_SINGLE, 2021 },
	{ 0x8355, 0x8355, PDF_CMAP_SINGLE, 15051 },
	{ 0x8358, 0x8358, PDF_CMAP_SINGLE, 2803 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 8601 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 6371 },
	{ 0x8375, 0x8375, PDF_CMAP_SINGLE, 6372 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 1369 },
	{ 0x837b, 0x837b, PDF_CMAP_SINGLE, 1326 },
	{ 0x837c, 0x837c, PDF_CMAP_SINGLE, 6369 },
	{ 0x837f, 0x837f, PDF_CMAP_SINGLE, 8602 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 6359 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 15052 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 6367 },
	{ 0x8389, 0x8389, PDF_CMAP_SINGLE, 6374 },
	{ 0x838a, 0x838a, PDF_CMAP_SINGLE, 6368 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 15053 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 6366 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 15054 },
	{ 0x8393, 0x8393, PDF_CMAP_SINGLE, 6337 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 6364 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 15055 },
	{ 0x839a, 0x839a, PDF_CMAP_SINGLE, 6360 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 1548 },
	{ 0x839f, 0x839f, PDF_CMAP_SINGLE, 6362 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 6373 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 6363 },
	{ 0x83a8, 0x83a8, PDF_CMAP_SINGLE, 6375 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 15056 },
	{ 0x83aa, 0x83aa, PDF_CMAP_SINGLE, 6361 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 3378 },
	{ 0x83b1, 0x83b1, PDF_CMAP_SINGLE, 3923 },
	{ 0x83b5, 0x83b5, PDF_CMAP_SINGLE, 6370 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 6392 },
	{ 0x83bf, 0x83c0, PDF_CMAP_RANGE, 15057 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 6384 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 2625 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 8603 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 1632 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 1749 },
	{ 0x83ce, 0x83ce, PDF_CMAP_SINGLE, 6379 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 1371 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 2492 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 6382 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 2122 },
	{ 0x83df, 0x83df, PDF_CMAP_SINGLE, 3147 },
	{ 0x83e0, 0x83e0, PDF_CMAP_SINGLE, 6387 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 3646 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 15059 },
	{ 0x83eb, 0x83eb, PDF_CMAP_SINGLE, 6378 },
	{ 0x83ef, 0x83ef, PDF_CMAP_SINGLE, 1370 },
	{ 0x83f0, 0x83f0, PDF_CMAP_SINGLE, 1930 },
	{ 0x83f1, 0x83f1, PDF_CMAP_SINGLE, 3483 },
	{ 0x83f2, 0x83f2, PDF_CMAP_SINGLE, 6388 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 6376 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 8604 },
	{ 0x83f7, 0x83f7, PDF_CMAP_SINGLE, 6385 },
	{ 0x83fb, 0x83fb, PDF_CMAP_SINGLE, 6395 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 6380 },
	{ 0x8403, 0x8403, PDF_CMAP_SINGLE, 6381 },
	{ 0x8404, 0x8404, PDF_CMAP_SINGLE, 3218 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 6386 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 7807 },
	{ 0x840b, 0x840b, PDF_CMAP_SINGLE, 6383 },
	{ 0x840c, 0x840c, PDF_CMAP_SINGLE, 3670 },
	{ 0x840d, 0x840d, PDF_CMAP_SINGLE, 6389 },
	{ 0x840e, 0x840e, PDF_CMAP_SINGLE, 1188 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 15060 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 15061 },
	{ 0x8413, 0x8413, PDF_CMAP_SINGLE, 6377 },
	{ 0x8420, 0x8420, PDF_CMAP_SINGLE, 6391 },
	{ 0x8422, 0x8422, PDF_CMAP_SINGLE, 6390 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 3361 },
	{ 0x842a, 0x842a, PDF_CMAP_SINGLE, 6397 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 6408 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 1500 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 6411 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 6393 },
	{ 0x843c, 0x843c, PDF_CMAP_SINGLE, 6398 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 3928 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 6407 },
	{ 0x8448, 0x8448, PDF_CMAP_SINGLE, 8605 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 3903 },
	{ 0x844a, 0x844a, PDF_CMAP_SINGLE, 15062 },
	{ 0x844e, 0x844e, PDF_CMAP_SINGLE, 3954 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 2998 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 1481 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 3556 },
	{ 0x8462, 0x8462, PDF_CMAP_SINGLE, 6413 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 3193 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 1141 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 6406 },
	{ 0x846b, 0x846b, PDF_CMAP_SINGLE, 6402 },
	{ 0x846c, 0x846c, PDF_CMAP_SINGLE, 2804 },
	{ 0x846d, 0x846d, PDF_CMAP_SINGLE, 6396 },
	{ 0x846e, 0x846e, PDF_CMAP_SINGLE, 6404 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 6409 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 3298 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 1134 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 15063 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 6401 },
	{ 0x8479, 0x8479, PDF_CMAP_SINGLE, 6410 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 3562 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 6405 },
	{ 0x8484, 0x8484, PDF_CMAP_SINGLE, 6400 },
	{ 0x848b, 0x848b, PDF_CMAP_SINGLE, 2493 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 2361 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 2264 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 3812 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3513 },
	{ 0x849f, 0x849f, PDF_CMAP_SINGLE, 6416 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 6425 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 15064 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 6403 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 15065 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 1493 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 8606 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 2528 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 6414 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 6419 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 2805 },
	{ 0x84bf, 0x84bf, PDF_CMAP_SINGLE, 6415 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 15066 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 6422 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 15067 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 2973 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 6423 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 3904 },
	{ 0x84ca, 0x84ca, PDF_CMAP_SINGLE, 6412 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 1430 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 6418 },
	{ 0x84d0, 0x84d0, PDF_CMAP_SINGLE, 6421 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 3768 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 6424 },
	{ 0x84d9, 0x84d9, PDF_CMAP_SINGLE, 6417 },
	{ 0x84da, 0x84da, PDF_CMAP_SINGLE, 6420 },
	{ 0x84dc, 0x84dc, PDF_CMAP_SINGLE, 8363 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 3671 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4039 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 15068 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 6428 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 6435 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 15069 },
	{ 0x84ff, 0x84ff, PDF_CMAP_SINGLE, 6427 },
	{ 0x8500, 0x8500, PDF_CMAP_SINGLE, 2287 },
	{ 0x8506, 0x8506, PDF_CMAP_SINGLE, 6394 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 15070 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 3614 },
	{ 0x8513, 0x8513, PDF_CMAP_SINGLE, 3758 },
	{ 0x8514, 0x8514, PDF_CMAP_SINGLE, 6434 },
	{ 0x8515, 0x8515, PDF_CMAP_SINGLE, 6433 },
	{ 0x8517, 0x8518, PDF_CMAP_RANGE, 6429 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 1240 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 6432 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 6426 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 7706 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 3057 },
	{ 0x852c, 0x852c, PDF_CMAP_SINGLE, 6431 },
	{ 0x852d, 0x852d, PDF_CMAP_SINGLE, 1218 },
	{ 0x8534, 0x8534, PDF_CMAP_SINGLE, 15071 },
	{ 0x8535, 0x8535, PDF_CMAP_SINGLE, 2818 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 3603 },
	{ 0x853e, 0x853e, PDF_CMAP_SINGLE, 7861 },
	{ 0x8540, 0x8540, PDF_CMAP_SINGLE, 6436 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 6440 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 3437 },
	{ 0x8548, 0x8548, PDF_CMAP_SINGLE, 6439 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 2494 },
	{ 0x854a, 0x854a, PDF_CMAP_SINGLE, 2293 },
	{ 0x854b, 0x854b, PDF_CMAP_SINGLE, 6442 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 1718 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 8607 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 6443 },
	{ 0x8557, 0x8557, PDF_CMAP_SINGLE, 3563 },
	{ 0x8558, 0x8558, PDF_CMAP_SINGLE, 6438 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 8608 },
	{ 0x855a, 0x855a, PDF_CMAP_SINGLE, 6399 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 15072 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 6437 },
	{ 0x8568, 0x8568, PDF_CMAP_SINGLE, 4085 },
	{ 0x8569, 0x8569, PDF_CMAP_SINGLE, 3194 },
	{ 0x856a, 0x856a, PDF_CMAP_SINGLE, 3557 },
	{ 0x856b, 0x856b, PDF_CMAP_SINGLE, 8609 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 6450 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 6456 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 6457 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 6444 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 3372 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 6454 },
	{ 0x8588, 0x8588, PDF_CMAP_SINGLE, 6446 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 6448 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 15073 },
	{ 0x8590, 0x8590, PDF_CMAP_SINGLE, 6458 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 6447 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 6451 },
	{ 0x8597, 0x8597, PDF_CMAP_SINGLE, 1300 },
	{ 0x8599, 0x8599, PDF_CMAP_SINGLE, 3261 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 6452 },
	{ 0x859c, 0x859c, PDF_CMAP_SINGLE, 6455 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 6445 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 2728 },
	{ 0x85a8, 0x85a8, PDF_CMAP_SINGLE, 6449 },
	{ 0x85a9, 0x85a9, PDF_CMAP_SINGLE, 2165 },
	{ 0x85aa, 0x85aa, PDF_CMAP_SINGLE, 2571 },
	{ 0x85ab, 0x85ab, PDF_CMAP_SINGLE, 1798 },
	{ 0x85ac, 0x85ac, PDF_CMAP_SINGLE, 3840 },
	{ 0x85ad, 0x85ad, PDF_CMAP_SINGLE, 15076 },
	{ 0x85ae, 0x85ae, PDF_CMAP_SINGLE, 3845 },
	{ 0x85af, 0x85af, PDF_CMAP_SINGLE, 2428 },
	{ 0x85b0, 0x85b0, PDF_CMAP_SINGLE, 8611 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 15074 },
	{ 0x85b9, 0x85b9, PDF_CMAP_SINGLE, 6462 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 6460 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 4084 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 6459 },
	{ 0x85cd, 0x85cd, PDF_CMAP_SINGLE, 3935 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 15075 },
	{ 0x85cf, 0x85cf, PDF_CMAP_SINGLE, 6461 },
	{ 0x85d0, 0x85d0, PDF_CMAP_SINGLE, 6463 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 6464 },
	{ 0x85dc, 0x85dc, PDF_CMAP_SINGLE, 6467 },
	{ 0x85dd, 0x85dd, PDF_CMAP_SINGLE, 6465 },
	{ 0x85e4, 0x85e4, PDF_CMAP_SINGLE, 3195 },
	{ 0x85e5, 0x85e5, PDF_CMAP_SINGLE, 6466 },
	{ 0x85e9, 0x85e9, PDF_CMAP_SINGLE, 3425 },
	{ 0x85ea, 0x85ea, PDF_CMAP_SINGLE, 6453 },
	{ 0x85f4, 0x85f4, PDF_CMAP_SINGLE, 14208 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 2429 },
	{ 0x85f9, 0x85f9, PDF_CMAP_SINGLE, 6468 },
	{ 0x85fa, 0x85fa, PDF_CMAP_SINGLE, 6473 },
	{ 0x85fb, 0x85fb, PDF_CMAP_SINGLE, 2806 },
	{ 0x85fe, 0x85fe, PDF_CMAP_SINGLE, 6472 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 6441 },
	{ 0x8606, 0x8606, PDF_CMAP_SINGLE, 6474 },
	{ 0x8607, 0x8607, PDF_CMAP_SINGLE, 2763 },
	{ 0x860a, 0x860a, PDF_CMAP_SINGLE, 6469 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 6471 },
	{ 0x8612, 0x8612, PDF_CMAP_SINGLE, 15077 },
	{ 0x8613, 0x8613, PDF_CMAP_SINGLE, 6470 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 5328 },
	{ 0x8617, 0x8617, PDF_CMAP_SINGLE, 5313 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 6476 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 6475 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 15078 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 3936 },
	{ 0x862f, 0x862f, PDF_CMAP_SINGLE, 5809 },
	{ 0x8630, 0x8630, PDF_CMAP_SINGLE, 6477 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 6478 },
	{ 0x864d, 0x864d, PDF_CMAP_SINGLE, 6479 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 1931 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 1646 },
	{ 0x8652, 0x8652, PDF_CMAP_SINGLE, 15079 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 6481 },
	{ 0x8655, 0x8655, PDF_CMAP_SINGLE, 4244 },
	{ 0x865a, 0x865a, PDF_CMAP_SINGLE, 1679 },
	{ 0x865b, 0x865b, PDF_CMAP_SINGLE, 13336 },
	{ 0x865c, 0x865c, PDF_CMAP_SINGLE, 3970 },
	{ 0x865e, 0x865e, PDF_CMAP_SINGLE, 1771 },
	{ 0x865f, 0x865f, PDF_CMAP_SINGLE, 6482 },
	{ 0x8663, 0x8663, PDF_CMAP_SINGLE, 15080 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 6483 },
	{ 0x866b, 0x866b, PDF_CMAP_SINGLE, 2988 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 15081 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 15082 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 6484 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 3282 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 15083 },
	{ 0x867b, 0x867b, PDF_CMAP_SINGLE, 1150 },
	{ 0x867d, 0x867d, PDF_CMAP_SINGLE, 14214 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 1379 },
	{ 0x868b, 0x868c, PDF_CMAP_RANGE, 6489 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 15084 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 15085 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 6485 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 2187 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 15086 },
	{ 0x86a3, 0x86a3, PDF_CMAP_SINGLE, 6486 },
	{ 0x86a4, 0x86a4, PDF_CMAP_SINGLE, 3320 },
	{ 0x86a7, 0x86a8, PDF_CMAP_RANGE, 15087 },
	{ 0x86a9, 0x86aa, PDF_CMAP_RANGE, 6487 },
	{ 0x86ab, 0x86ab, PDF_CMAP_SINGLE, 6498 },
	{ 0x86af, 0x86af, PDF_CMAP_SINGLE, 6492 },
	{ 0x86b0, 0x86b0, PDF_CMAP_SINGLE, 6495 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 6491 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 6493 },
	{ 0x86c6, 0x86c6, PDF_CMAP_SINGLE, 6494 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 2308 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 6496 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 2943 },
	{ 0x86cd, 0x86cd, PDF_CMAP_SINGLE, 1836 },
	{ 0x86ce, 0x86ce, PDF_CMAP_SINGLE, 1440 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 6499 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 1437 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 6504 },
	{ 0x86de, 0x86de, PDF_CMAP_SINGLE, 6500 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 6503 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 3406 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 6501 },
	{ 0x86ec, 0x86ec, PDF_CMAP_SINGLE, 6502 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 3514 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 3438 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 6505 },
	{ 0x86f8, 0x86f8, PDF_CMAP_SINGLE, 2909 },
	{ 0x86f9, 0x86f9, PDF_CMAP_SINGLE, 6515 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 15089 },
	{ 0x86fb, 0x86fb, PDF_CMAP_SINGLE, 6511 },
	{ 0x86fd, 0x86fd, PDF_CMAP_SINGLE, 15090 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 1387 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 6509 },
	{ 0x8702, 0x8702, PDF_CMAP_SINGLE, 3672 },
	{ 0x8703, 0x8703, PDF_CMAP_SINGLE, 6510 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 6507 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 6508 },
	{ 0x8709, 0x8709, PDF_CMAP_SINGLE, 6513 },
	{ 0x870a, 0x870a, PDF_CMAP_SINGLE, 6516 },
	{ 0x870b, 0x870b, PDF_CMAP_SINGLE, 15091 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 6514 },
	{ 0x8711, 0x8711, PDF_CMAP_SINGLE, 6512 },
	{ 0x8712, 0x8712, PDF_CMAP_SINGLE, 6506 },
	{ 0x8713, 0x8713, PDF_CMAP_SINGLE, 15092 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 2966 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 15093 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 6523 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 3766 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 15094 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 6521 },
	{ 0x8728, 0x8728, PDF_CMAP_SINGLE, 15095 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 6522 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 6517 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 6519 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 6520 },
	{ 0x873e, 0x873e, PDF_CMAP_SINGLE, 15096 },
	{ 0x873f, 0x873f, PDF_CMAP_SINGLE, 6518 },
	{ 0x8749, 0x8749, PDF_CMAP_SINGLE, 2698 },
	{ 0x874b, 0x874b, PDF_CMAP_SINGLE, 4063 },
	{ 0x874c, 0x874c, PDF_CMAP_SINGLE, 6527 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 6528 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 6534 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 2544 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 6530 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 6533 },
	{ 0x875f, 0x875f, PDF_CMAP_SINGLE, 6525 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 6524 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 6535 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 1372 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 6531 },
	{ 0x876a, 0x876a, PDF_CMAP_SINGLE, 6536 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 6532 },
	{ 0x8771, 0x8771, PDF_CMAP_SINGLE, 15097 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 6529 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 3023 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 6526 },
	{ 0x877f, 0x877f, PDF_CMAP_SINGLE, 3358 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 6540 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 15098 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 3877 },
	{ 0x8799, 0x8799, PDF_CMAP_SINGLE, 15099 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 6539 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 6538 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 6547 },
	{ 0x87ac, 0x87ad, PDF_CMAP_RANGE, 15100 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 6541 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 6549 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 15102 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 3920 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 6552 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 6543 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 6544 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 6548 },
	{ 0x87c6, 0x87c6, PDF_CMAP_SINGLE, 6551 },
	{ 0x87c7, 0x87c7, PDF_CMAP_SINGLE, 6550 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 6542 },
	{ 0x87d0, 0x87d0, PDF_CMAP_SINGLE, 6545 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 6562 },
	{ 0x87d6, 0x87d6, PDF_CMAP_SINGLE, 15103 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 6555 },
	{ 0x87eb, 0x87eb, PDF_CMAP_SINGLE, 15104 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 7715 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 15105 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 6553 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 6554 },
	{ 0x87f5, 0x87f5, PDF_CMAP_SINGLE, 15421 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 6559 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 1416 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 1628 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 6558 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 15106 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 15107 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 6537 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 15108 },
	{ 0x8807, 0x8807, PDF_CMAP_SINGLE, 8614 },
	{ 0x880b, 0x880b, PDF_CMAP_SINGLE, 15109 },
	{ 0x880d, 0x880d, PDF_CMAP_SINGLE, 6557 },
	{ 0x880e, 0x880e, PDF_CMAP_SINGLE, 6561 },
	{ 0x880f, 0x880f, PDF_CMAP_SINGLE, 6556 },
	{ 0x8811, 0x8811, PDF_CMAP_SINGLE, 6563 },
	{ 0x8814, 0x8814, PDF_CMAP_SINGLE, 15110 },
	{ 0x8815, 0x8815, PDF_CMAP_SINGLE, 6565 },
	{ 0x8816, 0x8816, PDF_CMAP_SINGLE, 6564 },
	{ 0x881c, 0x881c, PDF_CMAP_SINGLE, 15111 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 7813 },
	{ 0x8821, 0x8821, PDF_CMAP_SINGLE, 6567 },
	{ 0x8822, 0x8822, PDF_CMAP_SINGLE, 6566 },
	{ 0x8823, 0x8823, PDF_CMAP_SINGLE, 6497 },
	{ 0x8827, 0x8827, PDF_CMAP_SINGLE, 6571 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 6568 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 6569 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 6570 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 6572 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 1858 },
	{ 0x8842, 0x8842, PDF_CMAP_SINGLE, 6574 },
	{ 0x8844, 0x8844, PDF_CMAP_SINGLE, 6573 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 2362 },
	{ 0x884c, 0x884c, PDF_CMAP_SINGLE, 2022 },
	{ 0x884d, 0x884d, PDF_CMAP_SINGLE, 5412 },
	{ 0x8852, 0x8852, PDF_CMAP_SINGLE, 6575 },
	{ 0x8853, 0x8853, PDF_CMAP_SINGLE, 2395 },
	{ 0x8856, 0x8856, PDF_CMAP_SINGLE, 15112 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 1431 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 6576 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 1268 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 2495 },
	{ 0x885e, 0x885e, PDF_CMAP_SINGLE, 6577 },
	{ 0x885f, 0x885f, PDF_CMAP_SINGLE, 15113 },
	{ 0x8861, 0x8861, PDF_CMAP_SINGLE, 2023 },
	{ 0x8862, 0x8862, PDF_CMAP_SINGLE, 6578 },
	{ 0x8863, 0x8863, PDF_CMAP_SINGLE, 1189 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 15114 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 3503 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 6579 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 2608 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 6586 },
	{ 0x8875, 0x8875, PDF_CMAP_SINGLE, 6583 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2989 },
	{ 0x887d, 0x887d, PDF_CMAP_SINGLE, 6584 },
	{ 0x887e, 0x887e, PDF_CMAP_SINGLE, 6581 },
	{ 0x887f, 0x887f, PDF_CMAP_SINGLE, 1750 },
	{ 0x8881, 0x8881, PDF_CMAP_SINGLE, 6580 },
	{ 0x8882, 0x8882, PDF_CMAP_SINGLE, 6587 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 1804 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 2878 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 6593 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 6589 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 2837 },
	{ 0x8897, 0x8897, PDF_CMAP_SINGLE, 6588 },
	{ 0x8898, 0x8898, PDF_CMAP_SINGLE, 15115 },
	{ 0x8899, 0x8899, PDF_CMAP_SINGLE, 6591 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 6582 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 6592 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 6594 },
	{ 0x88aa, 0x88aa, PDF_CMAP_SINGLE, 15116 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 3459 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 6590 },
	{ 0x88b0, 0x88b0, PDF_CMAP_SINGLE, 6595 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 6597 },
	{ 0x88b4, 0x88b4, PDF_CMAP_SINGLE, 1927 },
	{ 0x88b5, 0x88b5, PDF_CMAP_SINGLE, 6585 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 1157 },
	{ 0x88bd, 0x88be, PDF_CMAP_RANGE, 15117 },
	{ 0x88bf, 0x88bf, PDF_CMAP_SINGLE, 6596 },
	{ 0x88c1, 0x88c1, PDF_CMAP_SINGLE, 2123 },
	{ 0x88c2, 0x88c2, PDF_CMAP_SINGLE, 4030 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 6598 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 2807 },
	{ 0x88c6, 0x88c6, PDF_CMAP_SINGLE, 14217 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 15119 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 3946 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 15120 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 6600 },
	{ 0x88d5, 0x88d5, PDF_CMAP_SINGLE, 3871 },
	{ 0x88d8, 0x88d9, PDF_CMAP_RANGE, 6601 },
	{ 0x88db, 0x88db, PDF_CMAP_SINGLE, 15121 },
	{ 0x88dc, 0x88dc, PDF_CMAP_SINGLE, 3636 },
	{ 0x88dd, 0x88dd, PDF_CMAP_SINGLE, 6603 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 2096 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3947 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 6608 },
	{ 0x88f0, 0x88f1, PDF_CMAP_RANGE, 15122 },
	{ 0x88f2, 0x88f2, PDF_CMAP_SINGLE, 6609 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 2496 },
	{ 0x88f4, 0x88f4, PDF_CMAP_SINGLE, 6607 },
	{ 0x88f5, 0x88f5, PDF_CMAP_SINGLE, 8615 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 3921 },
	{ 0x88f9, 0x88f9, PDF_CMAP_SINGLE, 6604 },
	{ 0x88fc, 0x88fc, PDF_CMAP_SINGLE, 6606 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 2657 },
	{ 0x88fe, 0x88fe, PDF_CMAP_SINGLE, 2628 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 6605 },
	{ 0x8904, 0x8904, PDF_CMAP_SINGLE, 6610 },
	{ 0x8906, 0x8906, PDF_CMAP_SINGLE, 15124 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 3571 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 6612 },
	{ 0x890c, 0x890c, PDF_CMAP_SINGLE, 6611 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 1482 },
	{ 0x8912, 0x8912, PDF_CMAP_SINGLE, 3673 },
	{ 0x8913, 0x8913, PDF_CMAP_SINGLE, 6613 },
	{ 0x8918, 0x891a, PDF_CMAP_RANGE, 15125 },
	{ 0x891c, 0x891c, PDF_CMAP_SINGLE, 8360 },
	{ 0x891d, 0x891d, PDF_CMAP_SINGLE, 6625 },
	{ 0x891e, 0x891e, PDF_CMAP_SINGLE, 6615 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 6616 },
	{ 0x8927, 0x8927, PDF_CMAP_SINGLE, 15128 },
	{ 0x892a, 0x892b, PDF_CMAP_RANGE, 6617 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 15129 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 6622 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 6623 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 6621 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 15130 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 6619 },
	{ 0x8943, 0x8943, PDF_CMAP_SINGLE, 6614 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 6620 },
	{ 0x894c, 0x894c, PDF_CMAP_SINGLE, 6624 },
	{ 0x894d, 0x894d, PDF_CMAP_SINGLE, 7120 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 1320 },
	{ 0x895e, 0x895e, PDF_CMAP_SINGLE, 6627 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 1751 },
	{ 0x8960, 0x8960, PDF_CMAP_SINGLE, 6626 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 6629 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 6628 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 6631 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 6630 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 6632 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 2363 },
	{ 0x8974, 0x8974, PDF_CMAP_SINGLE, 6633 },
	{ 0x8977, 0x8977, PDF_CMAP_SINGLE, 6634 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 15131 },
	{ 0x897e, 0x897e, PDF_CMAP_SINGLE, 6635 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 2658 },
	{ 0x8980, 0x8980, PDF_CMAP_SINGLE, 13870 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 3905 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 6636 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 3572 },
	{ 0x8987, 0x8987, PDF_CMAP_SINGLE, 3324 },
	{ 0x8988, 0x8988, PDF_CMAP_SINGLE, 6637 },
	{ 0x898a, 0x898a, PDF_CMAP_SINGLE, 6638 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 1887 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 1606 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 6639 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 2233 },
	{ 0x8997, 0x8997, PDF_CMAP_SINGLE, 3319 },
	{ 0x8998, 0x8998, PDF_CMAP_SINGLE, 6640 },
	{ 0x899a, 0x899a, PDF_CMAP_SINGLE, 1454 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 6641 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 6643 },
	{ 0x89a7, 0x89a7, PDF_CMAP_SINGLE, 3937 },
	{ 0x89a9, 0x89a9, PDF_CMAP_SINGLE, 6642 },
	{ 0x89aa, 0x89aa, PDF_CMAP_SINGLE, 2572 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 6644 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 6645 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 6646 },
	{ 0x89b3, 0x89b3, PDF_CMAP_SINGLE, 1549 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 6647 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 6648 },
	{ 0x89bf, 0x89c0, PDF_CMAP_RANGE, 6649 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 1455 },
	{ 0x89d4, 0x89d4, PDF_CMAP_SINGLE, 15132 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 15133 },
	{ 0x89da, 0x89da, PDF_CMAP_SINGLE, 6651 },
	{ 0x89dc, 0x89dd, PDF_CMAP_RANGE, 6652 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 1394 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 15134 },
	{ 0x89e6, 0x89e6, PDF_CMAP_SINGLE, 2542 },
	{ 0x89e7, 0x89e7, PDF_CMAP_SINGLE, 6654 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 15135 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 6655 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 6656 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 1908 },
	{ 0x8a01, 0x8a01, PDF_CMAP_SINGLE, 13756 },
	{ 0x8a02, 0x8a02, PDF_CMAP_SINGLE, 3095 },
	{ 0x8a03, 0x8a03, PDF_CMAP_SINGLE, 6657 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 15136 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 1837 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2588 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 6660 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 3196 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 15137 },
	{ 0x8a10, 0x8a10, PDF_CMAP_SINGLE, 6659 },
	{ 0x8a12, 0x8a12, PDF_CMAP_SINGLE, 8616 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 1799 },
	{ 0x8a15, 0x8a15, PDF_CMAP_SINGLE, 15138 },
	{ 0x8a16, 0x8a16, PDF_CMAP_SINGLE, 6658 },
	{ 0x8a17, 0x8a17, PDF_CMAP_SINGLE, 2903 },
	{ 0x8a18, 0x8a18, PDF_CMAP_SINGLE, 1607 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 6661 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 6662 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a22, 0x8a22, PDF_CMAP_SINGLE, 15139 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 1859 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 6663 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 3674 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 2691 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 1680 },
	{ 0x8a33, 0x8a33, PDF_CMAP_SINGLE, 3841 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 2764 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 6664 },
	{ 0x8a37, 0x8a37, PDF_CMAP_SINGLE, 8617 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 2573 },
	{ 0x8a3b, 0x8a3b, PDF_CMAP_SINGLE, 2990 },
	{ 0x8a3c, 0x8a3c, PDF_CMAP_SINGLE, 2498 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 6665 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 6668 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 6669 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 15140 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 2094 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 2850 },
	{ 0x8a52, 0x8a52, PDF_CMAP_SINGLE, 6667 },
	{ 0x8a54, 0x8a54, PDF_CMAP_SINGLE, 2499 },
	{ 0x8a55, 0x8a55, PDF_CMAP_SINGLE, 3504 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 6666 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 2234 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 1269 },
	{ 0x8a62, 0x8a62, PDF_CMAP_SINGLE, 6673 },
	{ 0x8a63, 0x8a63, PDF_CMAP_SINGLE, 1838 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 2236 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 2235 },
	{ 0x8a6b, 0x8a6b, PDF_CMAP_SINGLE, 4083 },
	{ 0x8a6c, 0x8a6c, PDF_CMAP_SINGLE, 6672 },
	{ 0x8a6d, 0x8a6d, PDF_CMAP_SINGLE, 6671 },
	{ 0x8a6e, 0x8a6e, PDF_CMAP_SINGLE, 2729 },
	{ 0x8a70, 0x8a70, PDF_CMAP_SINGLE, 1639 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 4073 },
	{ 0x8a72, 0x8a72, PDF_CMAP_SINGLE, 1432 },
	{ 0x8a73, 0x8a73, PDF_CMAP_SINGLE, 2500 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8618 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 6670 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 15141 },
	{ 0x8a82, 0x8a82, PDF_CMAP_SINGLE, 6675 },
	{ 0x8a84, 0x8a84, PDF_CMAP_SINGLE, 6676 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 6674 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 1932 },
	{ 0x8a89, 0x8a89, PDF_CMAP_SINGLE, 3882 },
	{ 0x8a8c, 0x8a8c, PDF_CMAP_SINGLE, 2237 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 3293 },
	{ 0x8a91, 0x8a91, PDF_CMAP_SINGLE, 6679 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 2660 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 2944 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 3872 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 6682 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 1952 },
	{ 0x8aa0, 0x8aa0, PDF_CMAP_SINGLE, 2659 },
	{ 0x8aa1, 0x8aa1, PDF_CMAP_SINGLE, 6678 },
	{ 0x8aa3, 0x8aa3, PDF_CMAP_SINGLE, 6683 },
	{ 0x8aa4, 0x8aa4, PDF_CMAP_SINGLE, 1953 },
	{ 0x8aa5, 0x8aa6, PDF_CMAP_RANGE, 6680 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 8619 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 6677 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 13880 },
	{ 0x8aac, 0x8aac, PDF_CMAP_SINGLE, 2694 },
	{ 0x8aad, 0x8aad, PDF_CMAP_SINGLE, 3233 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 2925 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 1373 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 3460 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 1629 },
	{ 0x8abe, 0x8abe, PDF_CMAP_SINGLE, 8620 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 3024 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 6686 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 6684 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 2954 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 2661 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 1550 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 6685 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 2593 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 3986 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4070 },
	{ 0x8ada, 0x8ada, PDF_CMAP_SINGLE, 6687 },
	{ 0x8adb, 0x8adb, PDF_CMAP_SINGLE, 6698 },
	{ 0x8adc, 0x8adc, PDF_CMAP_SINGLE, 3025 },
	{ 0x8ade, 0x8ade, PDF_CMAP_SINGLE, 6697 },
	{ 0x8adf, 0x8adf, PDF_CMAP_SINGLE, 8621 },
	{ 0x8ae0, 0x8ae0, PDF_CMAP_SINGLE, 6694 },
	{ 0x8ae1, 0x8ae1, PDF_CMAP_SINGLE, 6702 },
	{ 0x8ae2, 0x8ae2, PDF_CMAP_SINGLE, 6695 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 6691 },
	{ 0x8ae6, 0x8ae6, PDF_CMAP_SINGLE, 3096 },
	{ 0x8ae7, 0x8ae7, PDF_CMAP_SINGLE, 6690 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 6688 },
	{ 0x8aed, 0x8aed, PDF_CMAP_SINGLE, 3851 },
	{ 0x8aee, 0x8aee, PDF_CMAP_SINGLE, 2238 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 6692 },
	{ 0x8af3, 0x8af3, PDF_CMAP_SINGLE, 6689 },
	{ 0x8af4, 0x8af4, PDF_CMAP_SINGLE, 15142 },
	{ 0x8af6, 0x8af6, PDF_CMAP_SINGLE, 8623 },
	{ 0x8af7, 0x8af7, PDF_CMAP_SINGLE, 6696 },
	{ 0x8af8, 0x8af8, PDF_CMAP_SINGLE, 2430 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 1909 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 2906 },
	{ 0x8b00, 0x8b00, PDF_CMAP_SINGLE, 3699 },
	{ 0x8b01, 0x8b01, PDF_CMAP_SINGLE, 1276 },
	{ 0x8b02, 0x8b02, PDF_CMAP_SINGLE, 1190 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 3197 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 6700 },
	{ 0x8b0c, 0x8b0c, PDF_CMAP_SINGLE, 6699 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 3262 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 6704 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 6693 },
	{ 0x8b16, 0x8b16, PDF_CMAP_SINGLE, 6703 },
	{ 0x8b17, 0x8b17, PDF_CMAP_SINGLE, 6705 },
	{ 0x8b19, 0x8b19, PDF_CMAP_SINGLE, 1888 },
	{ 0x8b1a, 0x8b1a, PDF_CMAP_SINGLE, 6701 },
	{ 0x8b1b, 0x8b1b, PDF_CMAP_SINGLE, 2024 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 2305 },
	{ 0x8b1f, 0x8b1f, PDF_CMAP_SINGLE, 15143 },
	{ 0x8b20, 0x8b20, PDF_CMAP_SINGLE, 6706 },
	{ 0x8b21, 0x8b21, PDF_CMAP_SINGLE, 3906 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 6709 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 6712 },
	{ 0x8b2b, 0x8b2b, PDF_CMAP_SINGLE, 6710 },
	{ 0x8b2c, 0x8b2c, PDF_CMAP_SINGLE, 3495 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 6707 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 15144 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 1752 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 6711 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 6713 },
	{ 0x8b43, 0x8b44, PDF_CMAP_RANGE, 15145 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 6717 },
	{ 0x8b4c, 0x8b4c, PDF_CMAP_SINGLE, 6714 },
	{ 0x8b4e, 0x8b4e, PDF_CMAP_SINGLE, 6716 },
	{ 0x8b4f, 0x8b4f, PDF_CMAP_SINGLE, 6715 },
	{ 0x8b53, 0x8b53, PDF_CMAP_SINGLE, 8624 },
	{ 0x8b54, 0x8b54, PDF_CMAP_SINGLE, 15147 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 6718 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 2269 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 6720 },
	{ 0x8b5b, 0x8b5b, PDF_CMAP_SINGLE, 6719 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 3546 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 6722 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 1839 },
	{ 0x8b6b, 0x8b6b, PDF_CMAP_SINGLE, 6721 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 6723 },
	{ 0x8b6f, 0x8b6f, PDF_CMAP_SINGLE, 6724 },
	{ 0x8b70, 0x8b70, PDF_CMAP_SINGLE, 1630 },
	{ 0x8b71, 0x8b71, PDF_CMAP_SINGLE, 6190 },
	{ 0x8b72, 0x8b72, PDF_CMAP_SINGLE, 2529 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 6725 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 1954 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 6726 },
	{ 0x8b7f, 0x8b7f, PDF_CMAP_SINGLE, 8625 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 6727 },
	{ 0x8b83, 0x8b83, PDF_CMAP_SINGLE, 2188 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5075 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 6728 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 6729 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 2364 },
	{ 0x8b92, 0x8b93, PDF_CMAP_RANGE, 6730 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 6732 },
	{ 0x8b99, 0x8b9a, PDF_CMAP_RANGE, 6733 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 15148 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 15149 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 2921 },
	{ 0x8c3a, 0x8c3a, PDF_CMAP_SINGLE, 6735 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 6737 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 6736 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 3198 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 15150 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 6738 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 3675 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 6739 },
	{ 0x8c4e, 0x8c4e, PDF_CMAP_SINGLE, 6740 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 6741 },
	{ 0x8c54, 0x8c54, PDF_CMAP_SINGLE, 15151 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 6742 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 3250 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 2501 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 6743 },
	{ 0x8c6a, 0x8c6a, PDF_CMAP_SINGLE, 2045 },
	{ 0x8c6b, 0x8c6b, PDF_CMAP_SINGLE, 4103 },
	{ 0x8c6c, 0x8c6c, PDF_CMAP_SINGLE, 6744 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 15152 },
	{ 0x8c78, 0x8c78, PDF_CMAP_SINGLE, 6745 },
	{ 0x8c79, 0x8c79, PDF_CMAP_SINGLE, 3505 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 6746 },
	{ 0x8c7c, 0x8c7c, PDF_CMAP_SINGLE, 6754 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 6747 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 6749 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 6748 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 6750 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 3700 },
	{ 0x8c8d, 0x8c8e, PDF_CMAP_RANGE, 6751 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 6753 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 6755 },
	{ 0x8c9d, 0x8c9d, PDF_CMAP_SINGLE, 1419 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 3075 },
	{ 0x8ca0, 0x8ca0, PDF_CMAP_SINGLE, 3547 },
	{ 0x8ca1, 0x8ca1, PDF_CMAP_SINGLE, 2130 },
	{ 0x8ca2, 0x8ca2, PDF_CMAP_SINGLE, 2025 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 15153 },
	{ 0x8ca7, 0x8ca7, PDF_CMAP_SINGLE, 3521 },
	{ 0x8ca8, 0x8ca8, PDF_CMAP_SINGLE, 1375 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 3426 },
	{ 0x8caa, 0x8caa, PDF_CMAP_SINGLE, 6758 },
	{ 0x8cab, 0x8cab, PDF_CMAP_SINGLE, 1551 },
	{ 0x8cac, 0x8cac, PDF_CMAP_SINGLE, 2681 },
	{ 0x8cad, 0x8cad, PDF_CMAP_SINGLE, 6757 },
	{ 0x8cae, 0x8cae, PDF_CMAP_SINGLE, 6762 },
	{ 0x8caf, 0x8caf, PDF_CMAP_SINGLE, 2999 },
	{ 0x8cb0, 0x8cb0, PDF_CMAP_SINGLE, 3823 },
	{ 0x8cb2, 0x8cb3, PDF_CMAP_RANGE, 6760 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 1608 },
	{ 0x8cb6, 0x8cb6, PDF_CMAP_SINGLE, 6763 },
	{ 0x8cb7, 0x8cb7, PDF_CMAP_SINGLE, 3353 },
	{ 0x8cb8, 0x8cb8, PDF_CMAP_SINGLE, 2879 },
	{ 0x8cbb, 0x8cbb, PDF_CMAP_SINGLE, 3461 },
	{ 0x8cbc, 0x8cbc, PDF_CMAP_SINGLE, 3127 },
	{ 0x8cbd, 0x8cbd, PDF_CMAP_SINGLE, 6759 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 3701 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 1388 },
	{ 0x8cc1, 0x8cc1, PDF_CMAP_SINGLE, 6765 },
	{ 0x8cc2, 0x8cc2, PDF_CMAP_SINGLE, 4046 },
	{ 0x8cc3, 0x8cc3, PDF_CMAP_SINGLE, 3038 },
	{ 0x8cc4, 0x8cc4, PDF_CMAP_SINGLE, 4075 },
	{ 0x8cc7, 0x8cc7, PDF_CMAP_SINGLE, 2239 },
	{ 0x8cc8, 0x8cc8, PDF_CMAP_SINGLE, 6764 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 2833 },
	{ 0x8ccd, 0x8ccd, PDF_CMAP_SINGLE, 6781 },
	{ 0x8cce, 0x8cce, PDF_CMAP_SINGLE, 2730 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 3280 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 3522 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 15154 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 6768 },
	{ 0x8cdb, 0x8cdb, PDF_CMAP_SINGLE, 2189 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 2240 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 2502 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 3355 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 15155 },
	{ 0x8ce2, 0x8ce2, PDF_CMAP_SINGLE, 1889 },
	{ 0x8ce3, 0x8ce3, PDF_CMAP_SINGLE, 6767 },
	{ 0x8ce4, 0x8ce4, PDF_CMAP_SINGLE, 6766 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 3548 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 2285 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 3148 },
	{ 0x8cf0, 0x8cf0, PDF_CMAP_SINGLE, 8626 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8627 },
	{ 0x8cf8, 0x8cf8, PDF_CMAP_SINGLE, 15156 },
	{ 0x8cfa, 0x8cfb, PDF_CMAP_RANGE, 6770 },
	{ 0x8cfc, 0x8cfc, PDF_CMAP_SINGLE, 2026 },
	{ 0x8cfd, 0x8cfd, PDF_CMAP_SINGLE, 6769 },
	{ 0x8cfe, 0x8cfe, PDF_CMAP_SINGLE, 15157 },
	{ 0x8d04, 0x8d05, PDF_CMAP_RANGE, 6772 },
	{ 0x8d07, 0x8d07, PDF_CMAP_SINGLE, 6775 },
	{ 0x8d08, 0x8d08, PDF_CMAP_SINGLE, 2819 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 6774 },
	{ 0x8d0b, 0x8d0b, PDF_CMAP_SINGLE, 1570 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 6777 },
	{ 0x8d0f, 0x8d0f, PDF_CMAP_SINGLE, 6776 },
	{ 0x8d10, 0x8d10, PDF_CMAP_SINGLE, 6778 },
	{ 0x8d12, 0x8d12, PDF_CMAP_SINGLE, 8628 },
	{ 0x8d13, 0x8d13, PDF_CMAP_SINGLE, 6780 },
	{ 0x8d14, 0x8d14, PDF_CMAP_SINGLE, 6782 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 6783 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 15158 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 2682 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 2299 },
	{ 0x8d67, 0x8d67, PDF_CMAP_SINGLE, 6784 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 15159 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 1456 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 15160 },
	{ 0x8d6d, 0x8d6d, PDF_CMAP_SINGLE, 6785 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 2808 },
	{ 0x8d71, 0x8d71, PDF_CMAP_SINGLE, 6786 },
	{ 0x8d73, 0x8d73, PDF_CMAP_SINGLE, 6787 },
	{ 0x8d74, 0x8d74, PDF_CMAP_SINGLE, 3549 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 8629 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 1609 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 6788 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 15161 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 3026 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 1277 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 15162 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 15163 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 6789 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 2333 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 15164 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 2620 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 2829 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 6792 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 6791 },
	{ 0x8dc2, 0x8dc2, PDF_CMAP_SINGLE, 6790 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 15165 },
	{ 0x8dcb, 0x8dcb, PDF_CMAP_SINGLE, 6798 },
	{ 0x8dcc, 0x8dcc, PDF_CMAP_SINGLE, 6796 },
	{ 0x8dce, 0x8dce, PDF_CMAP_SINGLE, 15166 },
	{ 0x8dcf, 0x8dcf, PDF_CMAP_SINGLE, 6793 },
	{ 0x8dd6, 0x8dd6, PDF_CMAP_SINGLE, 6795 },
	{ 0x8dda, 0x8dda, PDF_CMAP_SINGLE, 6794 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 6797 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 1681 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 6801 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 2683 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 6802 },
	{ 0x8de4, 0x8de4, PDF_CMAP_SINGLE, 15167 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 1933 },
	{ 0x8dea, 0x8deb, PDF_CMAP_RANGE, 6799 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 15168 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 4047 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3027 },
	{ 0x8df5, 0x8df5, PDF_CMAP_SINGLE, 2731 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 6803 },
	{ 0x8dff, 0x8dff, PDF_CMAP_SINGLE, 6806 },
	{ 0x8e08, 0x8e09, PDF_CMAP_RANGE, 6804 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 3907 },
	{ 0x8e0f, 0x8e0f, PDF_CMAP_SINGLE, 3199 },
	{ 0x8e10, 0x8e10, PDF_CMAP_SINGLE, 6809 },
	{ 0x8e1d, 0x8e1e, PDF_CMAP_RANGE, 6807 },
	{ 0x8e1f, 0x8e1f, PDF_CMAP_SINGLE, 6810 },
	{ 0x8e20, 0x8e20, PDF_CMAP_SINGLE, 15169 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 6824 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 6813 },
	{ 0x8e34, 0x8e34, PDF_CMAP_SINGLE, 6814 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 6812 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 6811 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 3097 },
	{ 0x8e47, 0x8e47, PDF_CMAP_SINGLE, 6816 },
	{ 0x8e48, 0x8e48, PDF_CMAP_SINGLE, 6820 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 6817 },
	{ 0x8e4a, 0x8e4a, PDF_CMAP_SINGLE, 6815 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 15170 },
	{ 0x8e4c, 0x8e4c, PDF_CMAP_SINGLE, 6818 },
	{ 0x8e50, 0x8e50, PDF_CMAP_SINGLE, 6819 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 6826 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 6821 },
	{ 0x8e5f, 0x8e5f, PDF_CMAP_SINGLE, 2684 },
	{ 0x8e60, 0x8e60, PDF_CMAP_SINGLE, 6823 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 6825 },
	{ 0x8e64, 0x8e64, PDF_CMAP_SINGLE, 6822 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 15171 },
	{ 0x8e70, 0x8e70, PDF_CMAP_SINGLE, 15172 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 6828 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 2365 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 6827 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 15173 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 6829 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 6830 },
	{ 0x8e84, 0x8e84, PDF_CMAP_SINGLE, 6833 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 6832 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 6831 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 6835 },
	{ 0x8e8b, 0x8e8b, PDF_CMAP_SINGLE, 6834 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 3842 },
	{ 0x8e91, 0x8e91, PDF_CMAP_SINGLE, 6837 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 15174 },
	{ 0x8e93, 0x8e93, PDF_CMAP_SINGLE, 6836 },
	{ 0x8e94, 0x8e94, PDF_CMAP_SINGLE, 6838 },
	{ 0x8e99, 0x8e99, PDF_CMAP_SINGLE, 6839 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 6841 },
	{ 0x8eaa, 0x8eaa, PDF_CMAP_SINGLE, 6840 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 2574 },
	{ 0x8eac, 0x8eac, PDF_CMAP_SINGLE, 6842 },
	{ 0x8eae, 0x8eae, PDF_CMAP_SINGLE, 15175 },
	{ 0x8eaf, 0x8eaf, PDF_CMAP_SINGLE, 1765 },
	{ 0x8eb0, 0x8eb0, PDF_CMAP_SINGLE, 6843 },
	{ 0x8eb1, 0x8eb1, PDF_CMAP_SINGLE, 6845 },
	{ 0x8eb3, 0x8eb3, PDF_CMAP_SINGLE, 15176 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14083 },
	{ 0x8ebe, 0x8ebe, PDF_CMAP_SINGLE, 6846 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 7663 },
	{ 0x8ec5, 0x8ec5, PDF_CMAP_SINGLE, 6847 },
	{ 0x8ec6, 0x8ec6, PDF_CMAP_SINGLE, 6844 },
	{ 0x8ec8, 0x8ec8, PDF_CMAP_SINGLE, 6848 },
	{ 0x8eca, 0x8eca, PDF_CMAP_SINGLE, 2306 },
	{ 0x8ecb, 0x8ecb, PDF_CMAP_SINGLE, 6849 },
	{ 0x8ecc, 0x8ecc, PDF_CMAP_SINGLE, 1610 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 1801 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 8631 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 15177 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 1890 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 15178 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 6850 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 3272 },
	{ 0x8ee2, 0x8ee2, PDF_CMAP_SINGLE, 3128 },
	{ 0x8ee3, 0x8ee3, PDF_CMAP_SINGLE, 6851 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 6854 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 2272 },
	{ 0x8ef9, 0x8ef9, PDF_CMAP_SINGLE, 15179 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 6853 },
	{ 0x8efc, 0x8efc, PDF_CMAP_SINGLE, 6852 },
	{ 0x8efd, 0x8efd, PDF_CMAP_SINGLE, 1840 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 6855 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 1457 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 6857 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 2124 },
	{ 0x8f0a, 0x8f0a, PDF_CMAP_SINGLE, 6856 },
	{ 0x8f0c, 0x8f0c, PDF_CMAP_SINGLE, 6865 },
	{ 0x8f12, 0x8f12, PDF_CMAP_SINGLE, 6859 },
	{ 0x8f13, 0x8f13, PDF_CMAP_SINGLE, 6861 },
	{ 0x8f14, 0x8f14, PDF_CMAP_SINGLE, 3637 },
	{ 0x8f15, 0x8f15, PDF_CMAP_SINGLE, 6858 },
	{ 0x8f17, 0x8f17, PDF_CMAP_SINGLE, 15180 },
	{ 0x8f19, 0x8f19, PDF_CMAP_SINGLE, 6860 },
	{ 0x8f1b, 0x8f1b, PDF_CMAP_SINGLE, 6864 },
	{ 0x8f1c, 0x8f1c, PDF_CMAP_SINGLE, 6862 },
	{ 0x8f1d, 0x8f1d, PDF_CMAP_SINGLE, 1611 },
	{ 0x8f1f, 0x8f1f, PDF_CMAP_SINGLE, 6863 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 6866 },
	{ 0x8f29, 0x8f29, PDF_CMAP_SINGLE, 3344 },
	{ 0x8f2a, 0x8f2a, PDF_CMAP_SINGLE, 4000 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 2366 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 6867 },
	{ 0x8f36, 0x8f36, PDF_CMAP_SINGLE, 15181 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 3852 },
	{ 0x8f39, 0x8f39, PDF_CMAP_SINGLE, 6869 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 6868 },
	{ 0x8f3e, 0x8f3e, PDF_CMAP_SINGLE, 6872 },
	{ 0x8f3f, 0x8f3f, PDF_CMAP_SINGLE, 3883 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 6871 },
	{ 0x8f44, 0x8f44, PDF_CMAP_SINGLE, 1483 },
	{ 0x8f45, 0x8f45, PDF_CMAP_SINGLE, 6870 },
	{ 0x8f46, 0x8f46, PDF_CMAP_SINGLE, 6875 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 6874 },
	{ 0x8f4c, 0x8f4c, PDF_CMAP_SINGLE, 6873 },
	{ 0x8f4d, 0x8f4d, PDF_CMAP_SINGLE, 3116 },
	{ 0x8f4e, 0x8f4e, PDF_CMAP_SINGLE, 6876 },
	{ 0x8f57, 0x8f57, PDF_CMAP_SINGLE, 6877 },
	{ 0x8f5c, 0x8f5c, PDF_CMAP_SINGLE, 6878 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 2046 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 1787 },
	{ 0x8f62, 0x8f64, PDF_CMAP_RANGE, 6879 },
	{ 0x8f9b, 0x8f9b, PDF_CMAP_SINGLE, 2575 },
	{ 0x8f9c, 0x8f9c, PDF_CMAP_SINGLE, 6882 },
	{ 0x8f9e, 0x8f9e, PDF_CMAP_SINGLE, 2265 },
	{ 0x8f9f, 0x8f9f, PDF_CMAP_SINGLE, 6883 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 6884 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 15182 },
	{ 0x8fa7, 0x8fa7, PDF_CMAP_SINGLE, 4278 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 4277 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 6885 },
	{ 0x8fae, 0x8fae, PDF_CMAP_SINGLE, 6143 },
	{ 0x8faf, 0x8faf, PDF_CMAP_SINGLE, 6886 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 2914 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 2545 },
	{ 0x8fb2, 0x8fb2, PDF_CMAP_SINGLE, 3318 },
	{ 0x8fb5, 0x8fb6, PDF_CMAP_RANGE, 15183 },
	{ 0x8fb7, 0x8fb7, PDF_CMAP_SINGLE, 6887 },
	{ 0x8fba, 0x8fba, PDF_CMAP_SINGLE, 3621 },
	{ 0x8fbb, 0x8fbb, PDF_CMAP_SINGLE, 3056 },
	{ 0x8fbc, 0x8fbc, PDF_CMAP_SINGLE, 2064 },
	{ 0x8fbf, 0x8fbf, PDF_CMAP_SINGLE, 2919 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 1228 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 3750 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 2589 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 15185 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 14233 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 1844 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 1753 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 3622 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 6888 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 15186 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 6890 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 15187 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 6889 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 1376 },
	{ 0x8fe9, 0x8fe9, PDF_CMAP_SINGLE, 3278 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 6891 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 3373 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 3117 },
	{ 0x8fef, 0x8fef, PDF_CMAP_SINGLE, 6892 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 2396 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 6894 },
	{ 0x8ff6, 0x8ff6, PDF_CMAP_SINGLE, 15188 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 3790 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 6909 },
	{ 0x8ff9, 0x8ffa, PDF_CMAP_RANGE, 6896 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 3045 },
	{ 0x9000, 0x9000, PDF_CMAP_SINGLE, 2880 },
	{ 0x9001, 0x9001, PDF_CMAP_SINGLE, 2809 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 15189 },
	{ 0x9003, 0x9003, PDF_CMAP_SINGLE, 3200 },
	{ 0x9005, 0x9005, PDF_CMAP_SINGLE, 6895 },
	{ 0x9006, 0x9006, PDF_CMAP_SINGLE, 1647 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 6904 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 6901 },
	{ 0x900e, 0x900e, PDF_CMAP_SINGLE, 6914 },
	{ 0x900f, 0x900f, PDF_CMAP_SINGLE, 3201 },
	{ 0x9010, 0x9010, PDF_CMAP_SINGLE, 2974 },
	{ 0x9011, 0x9011, PDF_CMAP_SINGLE, 6898 },
	{ 0x9013, 0x9013, PDF_CMAP_SINGLE, 3098 },
	{ 0x9014, 0x9014, PDF_CMAP_SINGLE, 3149 },
	{ 0x9015, 0x9015, PDF_CMAP_SINGLE, 6899 },
	{ 0x9016, 0x9016, PDF_CMAP_SINGLE, 6903 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 2598 },
	{ 0x9019, 0x9019, PDF_CMAP_SINGLE, 3357 },
	{ 0x901a, 0x901a, PDF_CMAP_SINGLE, 3048 },
	{ 0x901d, 0x901d, PDF_CMAP_SINGLE, 2662 },
	{ 0x901e, 0x901e, PDF_CMAP_SINGLE, 6902 },
	{ 0x901f, 0x901f, PDF_CMAP_SINGLE, 2830 },
	{ 0x9020, 0x9020, PDF_CMAP_SINGLE, 2820 },
	{ 0x9021, 0x9021, PDF_CMAP_SINGLE, 6900 },
	{ 0x9022, 0x9022, PDF_CMAP_SINGLE, 1133 },
	{ 0x9023, 0x9023, PDF_CMAP_SINGLE, 4040 },
	{ 0x9027, 0x9027, PDF_CMAP_SINGLE, 6905 },
	{ 0x902c, 0x902c, PDF_CMAP_SINGLE, 15190 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 2881 },
	{ 0x9031, 0x9031, PDF_CMAP_SINGLE, 2367 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 2576 },
	{ 0x9035, 0x9035, PDF_CMAP_SINGLE, 6907 },
	{ 0x9036, 0x9036, PDF_CMAP_SINGLE, 6906 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 1203 },
	{ 0x9039, 0x9039, PDF_CMAP_SINGLE, 6908 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 3489 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 6916 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 3251 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 2609 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 15191 },
	{ 0x9045, 0x9045, PDF_CMAP_SINGLE, 2967 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 1776 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 6915 },
	{ 0x904a, 0x904a, PDF_CMAP_SINGLE, 3873 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 1249 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 3623 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 1377 },
	{ 0x904f, 0x9052, PDF_CMAP_RANGE, 6910 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 3219 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 2913 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 1191 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 6917 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 6918 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 7476 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 2845 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 6919 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 1301 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 2766 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 1891 },
	{ 0x9065, 0x9065, PDF_CMAP_SINGLE, 3908 },
	{ 0x9067, 0x9067, PDF_CMAP_SINGLE, 8634 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 6920 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 3110 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 2810 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 2307 },
	{ 0x906f, 0x906f, PDF_CMAP_SINGLE, 6921 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 6924 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 2415 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 6922 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 2733 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 2732 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 1192 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 3987 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 6926 },
	{ 0x907f, 0x907f, PDF_CMAP_SINGLE, 3462 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 6928 },
	{ 0x9081, 0x9081, PDF_CMAP_SINGLE, 6927 },
	{ 0x9082, 0x9082, PDF_CMAP_SINGLE, 6925 },
	{ 0x9083, 0x9083, PDF_CMAP_SINGLE, 5943 },
	{ 0x9084, 0x9084, PDF_CMAP_SINGLE, 1552 },
	{ 0x9087, 0x9087, PDF_CMAP_SINGLE, 6893 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 15192 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 6930 },
	{ 0x908a, 0x908a, PDF_CMAP_SINGLE, 6929 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 6931 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 3874 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 15193 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 15194 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 15195 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 15196 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 3257 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 3676 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 6932 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 2309 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 6933 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 6934 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 15197 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 6935 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 3099 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 1198 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2027 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 4064 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 15198 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 6939 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 15199 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 8635 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 1802 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 6936 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 6937 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 3558 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 1458 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 15200 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 3875 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 1719 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 3150 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 6940 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 6941 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 8637 },
	{ 0x9117, 0x9117, PDF_CMAP_SINGLE, 15201 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 6942 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 15202 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 8638 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 3100 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 6944 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 15203 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 6943 },
	{ 0x913a, 0x913a, PDF_CMAP_SINGLE, 15204 },
	{ 0x913d, 0x913d, PDF_CMAP_SINGLE, 15205 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 15206 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 3243 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 6945 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 2368 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 2316 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 3345 },
	{ 0x914e, 0x914e, PDF_CMAP_SINGLE, 2991 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2334 },
	{ 0x9154, 0x9154, PDF_CMAP_SINGLE, 2610 },
	{ 0x9156, 0x9156, PDF_CMAP_SINGLE, 6946 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 6947 },
	{ 0x915b, 0x915b, PDF_CMAP_SINGLE, 15207 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 15208 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 2595 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 6948 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 15209 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 6949 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 6950 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 3929 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 2369 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 6952 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 6951 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 2028 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 2053 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 2190 },
	{ 0x9182, 0x9182, PDF_CMAP_SINGLE, 6955 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 2416 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 6954 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 6953 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 2889 },
	{ 0x918e, 0x918e, PDF_CMAP_SINGLE, 15210 },
	{ 0x9190, 0x9190, PDF_CMAP_SINGLE, 1955 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 2663 },
	{ 0x9197, 0x9197, PDF_CMAP_SINGLE, 3396 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 2371 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 15211 },
	{ 0x91a2, 0x91a2, PDF_CMAP_SINGLE, 6956 },
	{ 0x91a4, 0x91a4, PDF_CMAP_SINGLE, 2503 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 15212 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 6959 },
	{ 0x91ab, 0x91ab, PDF_CMAP_SINGLE, 6957 },
	{ 0x91ac, 0x91ac, PDF_CMAP_SINGLE, 7707 },
	{ 0x91ad, 0x91ae, PDF_CMAP_RANGE, 15213 },
	{ 0x91af, 0x91af, PDF_CMAP_SINGLE, 6958 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 7777 },
	{ 0x91b2, 0x91b2, PDF_CMAP_SINGLE, 15215 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 6961 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 6960 },
	{ 0x91b8, 0x91b8, PDF_CMAP_SINGLE, 2530 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 6962 },
	{ 0x91bc, 0x91bc, PDF_CMAP_SINGLE, 15216 },
	{ 0x91c0, 0x91c1, PDF_CMAP_RANGE, 6963 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 3428 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 2115 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 2317 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 6965 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 6966 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 3948 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 2383 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 3834 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 3988 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 6967 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 1754 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 6968 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 8640 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 3101 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 8639 },
	{ 0x91db, 0x91db, PDF_CMAP_SINGLE, 6971 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 1494 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 2577 },
	{ 0x91de, 0x91de, PDF_CMAP_SINGLE, 8641 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 6969 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 6970 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 3068 },
	{ 0x91e4, 0x91e5, PDF_CMAP_RANGE, 8644 },
	{ 0x91e6, 0x91e6, PDF_CMAP_SINGLE, 3715 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 1780 },
	{ 0x91ed, 0x91ee, PDF_CMAP_RANGE, 8642 },
	{ 0x91f0, 0x91f0, PDF_CMAP_SINGLE, 15217 },
	{ 0x91f5, 0x91f6, PDF_CMAP_RANGE, 6973 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 15218 },
	{ 0x91fb, 0x91fb, PDF_CMAP_SINGLE, 15219 },
	{ 0x91fc, 0x91fc, PDF_CMAP_SINGLE, 6972 },
	{ 0x91ff, 0x91ff, PDF_CMAP_SINGLE, 6976 },
	{ 0x9206, 0x9206, PDF_CMAP_SINGLE, 8646 },
	{ 0x9207, 0x9207, PDF_CMAP_SINGLE, 15220 },
	{ 0x920a, 0x920a, PDF_CMAP_SINGLE, 8648 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 3255 },
	{ 0x920e, 0x920e, PDF_CMAP_SINGLE, 1441 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 8647 },
	{ 0x9211, 0x9211, PDF_CMAP_SINGLE, 6980 },
	{ 0x9214, 0x9214, PDF_CMAP_SINGLE, 6977 },
	{ 0x9215, 0x9215, PDF_CMAP_SINGLE, 6979 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 6975 },
	{ 0x9228, 0x9228, PDF_CMAP_SINGLE, 15221 },
	{ 0x9229, 0x9229, PDF_CMAP_SINGLE, 7050 },
	{ 0x922c, 0x922c, PDF_CMAP_SINGLE, 6978 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 15222 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 4019 },
	{ 0x9237, 0x9237, PDF_CMAP_SINGLE, 1934 },
	{ 0x9238, 0x9238, PDF_CMAP_SINGLE, 15223 },
	{ 0x9239, 0x9239, PDF_CMAP_SINGLE, 8655 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 8649 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 8651 },
	{ 0x923f, 0x923f, PDF_CMAP_SINGLE, 6988 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 8650 },
	{ 0x9243, 0x9243, PDF_CMAP_SINGLE, 15224 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 3118 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 6983 },
	{ 0x9247, 0x9247, PDF_CMAP_SINGLE, 15225 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 6986 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 6984 },
	{ 0x924b, 0x924b, PDF_CMAP_SINGLE, 6989 },
	{ 0x924e, 0x924e, PDF_CMAP_SINGLE, 8652 },
	{ 0x924f, 0x924f, PDF_CMAP_SINGLE, 15226 },
	{ 0x9250, 0x9250, PDF_CMAP_SINGLE, 6990 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 8654 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 6982 },
	{ 0x9259, 0x9259, PDF_CMAP_SINGLE, 8653 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 6995 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 1302 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 6981 },
	{ 0x9260, 0x9260, PDF_CMAP_SINGLE, 15227 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 3393 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 6985 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 2504 },
	{ 0x9267, 0x9267, PDF_CMAP_SINGLE, 8656 },
	{ 0x9271, 0x9271, PDF_CMAP_SINGLE, 2029 },
	{ 0x9277, 0x9278, PDF_CMAP_RANGE, 8658 },
	{ 0x927e, 0x927e, PDF_CMAP_SINGLE, 3702 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 1756 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 2384 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 3220 },
	{ 0x9288, 0x9288, PDF_CMAP_SINGLE, 8362 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 2735 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 6993 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 6987 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 6992 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 3791 },
	{ 0x929a, 0x929a, PDF_CMAP_SINGLE, 3028 },
	{ 0x929b, 0x929b, PDF_CMAP_SINGLE, 6994 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 6991 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 8657 },
	{ 0x92ad, 0x92ad, PDF_CMAP_SINGLE, 2734 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 13652 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 6998 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 6997 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 15228 },
	{ 0x92cb, 0x92cc, PDF_CMAP_RANGE, 15229 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 6996 },
	{ 0x92d0, 0x92d0, PDF_CMAP_SINGLE, 8663 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 3677 },
	{ 0x92d3, 0x92d3, PDF_CMAP_SINGLE, 8667 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 8665 },
	{ 0x92d7, 0x92d7, PDF_CMAP_SINGLE, 8661 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 8662 },
	{ 0x92df, 0x92df, PDF_CMAP_SINGLE, 15231 },
	{ 0x92e0, 0x92e0, PDF_CMAP_SINGLE, 8666 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 2437 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 8660 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 6999 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 3631 },
	{ 0x92ed, 0x92ed, PDF_CMAP_SINGLE, 1270 },
	{ 0x92f2, 0x92f2, PDF_CMAP_SINGLE, 3512 },
	{ 0x92f3, 0x92f3, PDF_CMAP_SINGLE, 2992 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 1682 },
	{ 0x92f9, 0x92f9, PDF_CMAP_SINGLE, 8368 },
	{ 0x92fa, 0x92fa, PDF_CMAP_SINGLE, 7001 },
	{ 0x92fb, 0x92fb, PDF_CMAP_SINGLE, 8670 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 2031 },
	{ 0x92ff, 0x92ff, PDF_CMAP_SINGLE, 8673 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 8675 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 13402 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 2170 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 15232 },
	{ 0x930f, 0x930f, PDF_CMAP_SINGLE, 7000 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 2611 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 15233 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 2612 },
	{ 0x9319, 0x9319, PDF_CMAP_SINGLE, 7004 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 7006 },
	{ 0x931d, 0x931d, PDF_CMAP_SINGLE, 8674 },
	{ 0x931e, 0x931e, PDF_CMAP_SINGLE, 8672 },
	{ 0x931f, 0x931f, PDF_CMAP_SINGLE, 15234 },
	{ 0x9320, 0x9320, PDF_CMAP_SINGLE, 2531 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 8669 },
	{ 0x9322, 0x9322, PDF_CMAP_SINGLE, 7005 },
	{ 0x9323, 0x9323, PDF_CMAP_SINGLE, 7007 },
	{ 0x9325, 0x9325, PDF_CMAP_SINGLE, 8668 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 1739 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 15235 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 3511 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 2318 },
	{ 0x932c, 0x932c, PDF_CMAP_SINGLE, 4041 },
	{ 0x932e, 0x932e, PDF_CMAP_SINGLE, 7003 },
	{ 0x932f, 0x932f, PDF_CMAP_SINGLE, 2152 },
	{ 0x9332, 0x9332, PDF_CMAP_SINGLE, 4069 },
	{ 0x9335, 0x9335, PDF_CMAP_SINGLE, 7009 },
	{ 0x933a, 0x933a, PDF_CMAP_SINGLE, 7008 },
	{ 0x933b, 0x933b, PDF_CMAP_SINGLE, 7010 },
	{ 0x9344, 0x9344, PDF_CMAP_SINGLE, 7002 },
	{ 0x9347, 0x9347, PDF_CMAP_SINGLE, 15236 },
	{ 0x9348, 0x9348, PDF_CMAP_SINGLE, 8361 },
	{ 0x934a, 0x934a, PDF_CMAP_SINGLE, 13400 },
	{ 0x934b, 0x934b, PDF_CMAP_SINGLE, 3265 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 3151 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 15237 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 3059 },
	{ 0x9356, 0x9356, PDF_CMAP_SINGLE, 7015 },
	{ 0x9357, 0x9357, PDF_CMAP_SINGLE, 8677 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 2945 },
	{ 0x935c, 0x935c, PDF_CMAP_SINGLE, 7011 },
	{ 0x9360, 0x9360, PDF_CMAP_SINGLE, 7012 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 15238 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 15239 },
	{ 0x936c, 0x936c, PDF_CMAP_SINGLE, 1795 },
	{ 0x936d, 0x936d, PDF_CMAP_SINGLE, 15240 },
	{ 0x936e, 0x936e, PDF_CMAP_SINGLE, 7014 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 8676 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 1892 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 7013 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 2505 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 1495 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 7019 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 2095 },
	{ 0x9397, 0x9397, PDF_CMAP_SINGLE, 2811 },
	{ 0x939a, 0x939a, PDF_CMAP_SINGLE, 3046 },
	{ 0x939b, 0x939b, PDF_CMAP_SINGLE, 15241 },
	{ 0x93a4, 0x93a4, PDF_CMAP_SINGLE, 8678 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 1433 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 15243 },
	{ 0x93ac, 0x93ad, PDF_CMAP_RANGE, 7017 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 3039 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 7016 },
	{ 0x93b9, 0x93b9, PDF_CMAP_SINGLE, 7020 },
	{ 0x93ba, 0x93ba, PDF_CMAP_SINGLE, 15242 },
	{ 0x93c1, 0x93c1, PDF_CMAP_SINGLE, 15244 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 7026 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 8679 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 7029 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 15245 },
	{ 0x93d0, 0x93d0, PDF_CMAP_SINGLE, 7028 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 3111 },
	{ 0x93d6, 0x93d7, PDF_CMAP_RANGE, 7021 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 7025 },
	{ 0x93dd, 0x93dd, PDF_CMAP_SINGLE, 7027 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 8680 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 1720 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 15246 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 7030 },
	{ 0x93e5, 0x93e5, PDF_CMAP_SINGLE, 7024 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7023 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 8681 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 15247 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 15248 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 7034 },
	{ 0x9407, 0x9407, PDF_CMAP_SINGLE, 7035 },
	{ 0x940f, 0x940f, PDF_CMAP_SINGLE, 15249 },
	{ 0x9410, 0x9410, PDF_CMAP_SINGLE, 7036 },
	{ 0x9413, 0x9413, PDF_CMAP_SINGLE, 7033 },
	{ 0x9414, 0x9414, PDF_CMAP_SINGLE, 7032 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 2506 },
	{ 0x9419, 0x9419, PDF_CMAP_SINGLE, 3202 },
	{ 0x941a, 0x941a, PDF_CMAP_SINGLE, 7031 },
	{ 0x9421, 0x9421, PDF_CMAP_SINGLE, 7040 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 7038 },
	{ 0x9431, 0x9431, PDF_CMAP_SINGLE, 8682 },
	{ 0x9434, 0x9434, PDF_CMAP_SINGLE, 15250 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 7039 },
	{ 0x9436, 0x9436, PDF_CMAP_SINGLE, 7037 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 2904 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 7041 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 15251 },
	{ 0x9441, 0x9441, PDF_CMAP_SINGLE, 7042 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 7044 },
	{ 0x9445, 0x9445, PDF_CMAP_SINGLE, 8683 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 8684 },
	{ 0x9451, 0x9451, PDF_CMAP_SINGLE, 1553 },
	{ 0x9452, 0x9452, PDF_CMAP_SINGLE, 7043 },
	{ 0x9453, 0x9453, PDF_CMAP_SINGLE, 3846 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 15252 },
	{ 0x945a, 0x945a, PDF_CMAP_SINGLE, 7055 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 7045 },
	{ 0x945e, 0x945e, PDF_CMAP_SINGLE, 7048 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 7046 },
	{ 0x9462, 0x9462, PDF_CMAP_SINGLE, 7047 },
	{ 0x946a, 0x946a, PDF_CMAP_SINGLE, 7049 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 15253 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 7051 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 15254 },
	{ 0x9475, 0x9475, PDF_CMAP_SINGLE, 7052 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 7053 },
	{ 0x947c, 0x947c, PDF_CMAP_SINGLE, 7056 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 7054 },
	{ 0x947e, 0x947e, PDF_CMAP_SINGLE, 7057 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 7059 },
	{ 0x9481, 0x9481, PDF_CMAP_SINGLE, 7058 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 3029 },
	{ 0x9578, 0x9578, PDF_CMAP_SINGLE, 15255 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 3827 },
	{ 0x9582, 0x9582, PDF_CMAP_SINGLE, 7060 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 2736 },
	{ 0x9587, 0x9587, PDF_CMAP_SINGLE, 7061 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 3604 },
	{ 0x958a, 0x958a, PDF_CMAP_SINGLE, 7062 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 1417 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 1246 },
	{ 0x9591, 0x9591, PDF_CMAP_SINGLE, 1555 },
	{ 0x9592, 0x9592, PDF_CMAP_SINGLE, 8685 },
	{ 0x9593, 0x9593, PDF_CMAP_SINGLE, 1554 },
	{ 0x9594, 0x9594, PDF_CMAP_SINGLE, 7063 },
	{ 0x9596, 0x9596, PDF_CMAP_SINGLE, 7064 },
	{ 0x9598, 0x9599, PDF_CMAP_RANGE, 7065 },
	{ 0x95a0, 0x95a0, PDF_CMAP_SINGLE, 7067 },
	{ 0x95a2, 0x95a2, PDF_CMAP_SINGLE, 1556 },
	{ 0x95a3, 0x95a3, PDF_CMAP_SINGLE, 1459 },
	{ 0x95a4, 0x95a4, PDF_CMAP_SINGLE, 2032 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 3402 },
	{ 0x95a6, 0x95a6, PDF_CMAP_SINGLE, 15256 },
	{ 0x95a7, 0x95a7, PDF_CMAP_SINGLE, 7069 },
	{ 0x95a8, 0x95a8, PDF_CMAP_SINGLE, 7068 },
	{ 0x95a9, 0x95a9, PDF_CMAP_SINGLE, 15257 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 15258 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 7070 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 13653 },
	{ 0x95b2, 0x95b2, PDF_CMAP_SINGLE, 1278 },
	{ 0x95b4, 0x95b4, PDF_CMAP_SINGLE, 15259 },
	{ 0x95b9, 0x95b9, PDF_CMAP_SINGLE, 7073 },
	{ 0x95bb, 0x95bb, PDF_CMAP_SINGLE, 7072 },
	{ 0x95bc, 0x95bc, PDF_CMAP_SINGLE, 7071 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 15260 },
	{ 0x95be, 0x95be, PDF_CMAP_SINGLE, 7074 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 7077 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 1163 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 7075 },
	{ 0x95cc, 0x95cc, PDF_CMAP_SINGLE, 7079 },
	{ 0x95cd, 0x95cd, PDF_CMAP_SINGLE, 7078 },
	{ 0x95d4, 0x95d4, PDF_CMAP_SINGLE, 7081 },
	{ 0x95d5, 0x95d5, PDF_CMAP_SINGLE, 7080 },
	{ 0x95d6, 0x95d6, PDF_CMAP_SINGLE, 7082 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 3206 },
	{ 0x95da, 0x95da, PDF_CMAP_SINGLE, 15261 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 7083 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 7084 },
	{ 0x95e2, 0x95e2, PDF_CMAP_SINGLE, 7086 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 7085 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 3550 },
	{ 0x961d, 0x961d, PDF_CMAP_SINGLE, 15262 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 7087 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 7088 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 2133 },
	{ 0x962e, 0x962f, PDF_CMAP_RANGE, 7089 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 3703 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 2765 },
	{ 0x963f, 0x963f, PDF_CMAP_SINGLE, 1128 },
	{ 0x9640, 0x9640, PDF_CMAP_SINGLE, 2859 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 15263 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 7091 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 3551 },
	{ 0x964b, 0x964b, PDF_CMAP_SINGLE, 7094 },
	{ 0x964c, 0x964c, PDF_CMAP_SINGLE, 7092 },
	{ 0x964d, 0x964d, PDF_CMAP_SINGLE, 2033 },
	{ 0x964f, 0x964f, PDF_CMAP_SINGLE, 7093 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 1910 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 15264 },
	{ 0x965b, 0x965b, PDF_CMAP_SINGLE, 3605 },
	{ 0x965c, 0x965c, PDF_CMAP_SINGLE, 7096 },
	{ 0x965d, 0x965d, PDF_CMAP_SINGLE, 7098 },
	{ 0x965e, 0x965e, PDF_CMAP_SINGLE, 7097 },
	{ 0x965f, 0x965f, PDF_CMAP_SINGLE, 7099 },
	{ 0x9662, 0x9662, PDF_CMAP_SINGLE, 1219 },
	{ 0x9663, 0x9663, PDF_CMAP_SINGLE, 2590 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 2438 },
	{ 0x9665, 0x9665, PDF_CMAP_SINGLE, 1557 },
	{ 0x9666, 0x9666, PDF_CMAP_SINGLE, 7100 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 3356 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 7102 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 1220 },
	{ 0x9672, 0x9672, PDF_CMAP_SINGLE, 7101 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 3040 },
	{ 0x9675, 0x9675, PDF_CMAP_SINGLE, 3989 },
	{ 0x9676, 0x9676, PDF_CMAP_SINGLE, 3203 },
	{ 0x9677, 0x9677, PDF_CMAP_SINGLE, 7095 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 3950 },
	{ 0x967a, 0x967a, PDF_CMAP_SINGLE, 1893 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 3909 },
	{ 0x9684, 0x9684, PDF_CMAP_SINGLE, 15265 },
	{ 0x9685, 0x9685, PDF_CMAP_SINGLE, 1777 },
	{ 0x9686, 0x9686, PDF_CMAP_SINGLE, 3964 },
	{ 0x9688, 0x9688, PDF_CMAP_SINGLE, 1790 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 2882 },
	{ 0x968b, 0x968b, PDF_CMAP_SINGLE, 6252 },
	{ 0x968d, 0x968d, PDF_CMAP_SINGLE, 7103 },
	{ 0x968e, 0x968e, PDF_CMAP_SINGLE, 1418 },
	{ 0x968f, 0x968f, PDF_CMAP_SINGLE, 2613 },
	{ 0x9694, 0x9694, PDF_CMAP_SINGLE, 1460 },
	{ 0x9695, 0x9695, PDF_CMAP_SINGLE, 7105 },
	{ 0x9697, 0x9697, PDF_CMAP_SINGLE, 7106 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 7104 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 1850 },
	{ 0x969b, 0x969b, PDF_CMAP_SINGLE, 2125 },
	{ 0x969c, 0x969c, PDF_CMAP_SINGLE, 2507 },
	{ 0x969d, 0x969d, PDF_CMAP_SINGLE, 8688 },
	{ 0x96a0, 0x96a0, PDF_CMAP_SINGLE, 1221 },
	{ 0x96a3, 0x96a3, PDF_CMAP_SINGLE, 4001 },
	{ 0x96a4, 0x96a4, PDF_CMAP_SINGLE, 15266 },
	{ 0x96a7, 0x96a7, PDF_CMAP_SINGLE, 7108 },
	{ 0x96a8, 0x96a8, PDF_CMAP_SINGLE, 6923 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 15267 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 7107 },
	{ 0x96af, 0x96af, PDF_CMAP_SINGLE, 8689 },
	{ 0x96b0, 0x96b0, PDF_CMAP_SINGLE, 7111 },
	{ 0x96b1, 0x96b2, PDF_CMAP_RANGE, 7109 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 7112 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 7113 },
	{ 0x96b7, 0x96b7, PDF_CMAP_SINGLE, 4020 },
	{ 0x96b8, 0x96b9, PDF_CMAP_RANGE, 7114 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 2669 },
	{ 0x96bc, 0x96bc, PDF_CMAP_SINGLE, 3407 },
	{ 0x96c0, 0x96c0, PDF_CMAP_SINGLE, 2627 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 1571 },
	{ 0x96c4, 0x96c4, PDF_CMAP_SINGLE, 3876 },
	{ 0x96c5, 0x96c5, PDF_CMAP_SINGLE, 1389 },
	{ 0x96c6, 0x96c6, PDF_CMAP_SINGLE, 2370 },
	{ 0x96c7, 0x96c7, PDF_CMAP_SINGLE, 1935 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 7118 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 7117 },
	{ 0x96cc, 0x96cc, PDF_CMAP_SINGLE, 2241 },
	{ 0x96cd, 0x96cd, PDF_CMAP_SINGLE, 7119 },
	{ 0x96ce, 0x96ce, PDF_CMAP_SINGLE, 7116 },
	{ 0x96d1, 0x96d1, PDF_CMAP_SINGLE, 2166 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 15268 },
	{ 0x96d5, 0x96d5, PDF_CMAP_SINGLE, 7123 },
	{ 0x96d6, 0x96d6, PDF_CMAP_SINGLE, 6546 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 4331 },
	{ 0x96db, 0x96db, PDF_CMAP_SINGLE, 2621 },
	{ 0x96dc, 0x96dc, PDF_CMAP_SINGLE, 7121 },
	{ 0x96de, 0x96de, PDF_CMAP_SINGLE, 15270 },
	{ 0x96e2, 0x96e2, PDF_CMAP_SINGLE, 3949 },
	{ 0x96e3, 0x96e3, PDF_CMAP_SINGLE, 3273 },
	{ 0x96e8, 0x96e8, PDF_CMAP_SINGLE, 1229 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 15271 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 2695 },
	{ 0x96eb, 0x96eb, PDF_CMAP_SINGLE, 2274 },
	{ 0x96f0, 0x96f0, PDF_CMAP_SINGLE, 3591 },
	{ 0x96f1, 0x96f1, PDF_CMAP_SINGLE, 15272 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 1250 },
	{ 0x96f6, 0x96f6, PDF_CMAP_SINGLE, 4021 },
	{ 0x96f7, 0x96f7, PDF_CMAP_SINGLE, 3925 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 7124 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 3135 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 2343 },
	{ 0x9702, 0x9702, PDF_CMAP_SINGLE, 15273 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 7125 },
	{ 0x9706, 0x9706, PDF_CMAP_SINGLE, 7126 },
	{ 0x9707, 0x9707, PDF_CMAP_SINGLE, 2578 },
	{ 0x9708, 0x9708, PDF_CMAP_SINGLE, 7127 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 15274 },
	{ 0x970a, 0x970a, PDF_CMAP_SINGLE, 4022 },
	{ 0x970d, 0x970d, PDF_CMAP_SINGLE, 7122 },
	{ 0x970e, 0x970e, PDF_CMAP_SINGLE, 7129 },
	{ 0x970f, 0x970f, PDF_CMAP_SINGLE, 7131 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 7130 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 7128 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 7132 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 7133 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 2812 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 1378 },
	{ 0x9724, 0x9724, PDF_CMAP_SINGLE, 7134 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 3780 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7135 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 7136 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 4048 },
	{ 0x9733, 0x9733, PDF_CMAP_SINGLE, 8690 },
	{ 0x9738, 0x9738, PDF_CMAP_SINGLE, 5140 },
	{ 0x9739, 0x9739, PDF_CMAP_SINGLE, 7137 },
	{ 0x973b, 0x973b, PDF_CMAP_SINGLE, 8691 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 7138 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 7143 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 8692 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 7140 },
	{ 0x9746, 0x9746, PDF_CMAP_SINGLE, 7141 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 7142 },
	{ 0x9749, 0x9749, PDF_CMAP_SINGLE, 7144 },
	{ 0x974d, 0x974d, PDF_CMAP_SINGLE, 8693 },
	{ 0x974f, 0x974f, PDF_CMAP_SINGLE, 8694 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 8695 },
	{ 0x9752, 0x9752, PDF_CMAP_SINGLE, 2664 },
	{ 0x9755, 0x9755, PDF_CMAP_SINGLE, 8696 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 3843 },
	{ 0x9759, 0x9759, PDF_CMAP_SINGLE, 2665 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 15275 },
	{ 0x975c, 0x975c, PDF_CMAP_SINGLE, 7145 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 3463 },
	{ 0x9760, 0x9760, PDF_CMAP_SINGLE, 7146 },
	{ 0x9761, 0x9761, PDF_CMAP_SINGLE, 7430 },
	{ 0x9762, 0x9762, PDF_CMAP_SINGLE, 3800 },
	{ 0x9763, 0x9763, PDF_CMAP_SINGLE, 15276 },
	{ 0x9764, 0x9764, PDF_CMAP_SINGLE, 7147 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 7148 },
	{ 0x9768, 0x9768, PDF_CMAP_SINGLE, 7149 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 1461 },
	{ 0x976b, 0x976b, PDF_CMAP_SINGLE, 7151 },
	{ 0x976d, 0x976d, PDF_CMAP_SINGLE, 2591 },
	{ 0x976e, 0x976e, PDF_CMAP_SINGLE, 15277 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 7152 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 15278 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 1786 },
	{ 0x9779, 0x9779, PDF_CMAP_SINGLE, 7153 },
	{ 0x977a, 0x977a, PDF_CMAP_SINGLE, 7157 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7155 },
	{ 0x9781, 0x9781, PDF_CMAP_SINGLE, 7156 },
	{ 0x9784, 0x9784, PDF_CMAP_SINGLE, 1489 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 7154 },
	{ 0x9786, 0x9786, PDF_CMAP_SINGLE, 7158 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 7159 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 1164 },
	{ 0x978f, 0x9790, PDF_CMAP_RANGE, 7160 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 2508 },
	{ 0x979a, 0x979a, PDF_CMAP_SINGLE, 15279 },
	{ 0x979c, 0x979c, PDF_CMAP_SINGLE, 7162 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 1633 },
	{ 0x97a2, 0x97a2, PDF_CMAP_SINGLE, 15280 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 7165 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 7164 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 7163 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 6708 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 3628 },
	{ 0x97b3, 0x97b4, PDF_CMAP_RANGE, 7166 },
	{ 0x97b5, 0x97b6, PDF_CMAP_RANGE, 15281 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 7168 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 7169 },
	{ 0x97c8, 0x97c8, PDF_CMAP_SINGLE, 7170 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 7171 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 1558 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 15283 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 7172 },
	{ 0x97de, 0x97de, PDF_CMAP_SINGLE, 15284 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 7173 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 3289 },
	{ 0x97f2, 0x97f2, PDF_CMAP_SINGLE, 7175 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 1339 },
	{ 0x97f4, 0x97f4, PDF_CMAP_SINGLE, 15285 },
	{ 0x97f5, 0x97f5, PDF_CMAP_SINGLE, 7178 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 7177 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 1222 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 1721 },
	{ 0x9801, 0x9801, PDF_CMAP_SINGLE, 3607 },
	{ 0x9802, 0x9802, PDF_CMAP_SINGLE, 3030 },
	{ 0x9803, 0x9803, PDF_CMAP_SINGLE, 2066 },
	{ 0x9805, 0x9805, PDF_CMAP_SINGLE, 2034 },
	{ 0x9806, 0x9806, PDF_CMAP_SINGLE, 2417 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 2594 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 15286 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 7180 },
	{ 0x980e, 0x980e, PDF_CMAP_SINGLE, 15287 },
	{ 0x980f, 0x980f, PDF_CMAP_SINGLE, 7179 },
	{ 0x9810, 0x9810, PDF_CMAP_SINGLE, 3884 },
	{ 0x9811, 0x9811, PDF_CMAP_SINGLE, 1572 },
	{ 0x9812, 0x9812, PDF_CMAP_SINGLE, 3430 },
	{ 0x9813, 0x9813, PDF_CMAP_SINGLE, 3252 },
	{ 0x9817, 0x9817, PDF_CMAP_SINGLE, 2626 },
	{ 0x9818, 0x9818, PDF_CMAP_SINGLE, 3990 },
	{ 0x981a, 0x981a, PDF_CMAP_SINGLE, 1841 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 15288 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 7183 },
	{ 0x9823, 0x9823, PDF_CMAP_SINGLE, 15289 },
	{ 0x9824, 0x9824, PDF_CMAP_SINGLE, 7182 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 15290 },
	{ 0x982c, 0x982c, PDF_CMAP_SINGLE, 3705 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 3204 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 7795 },
	{ 0x9834, 0x9834, PDF_CMAP_SINGLE, 1266 },
	{ 0x9837, 0x9837, PDF_CMAP_SINGLE, 7184 },
	{ 0x9838, 0x9838, PDF_CMAP_SINGLE, 7181 },
	{ 0x9839, 0x9839, PDF_CMAP_SINGLE, 14259 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 3523 },
	{ 0x983c, 0x983c, PDF_CMAP_SINGLE, 3924 },
	{ 0x983d, 0x983d, PDF_CMAP_SINGLE, 7185 },
	{ 0x983e, 0x983e, PDF_CMAP_SINGLE, 15291 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 7186 },
	{ 0x984b, 0x984b, PDF_CMAP_SINGLE, 7188 },
	{ 0x984c, 0x984c, PDF_CMAP_SINGLE, 2890 },
	{ 0x984d, 0x984e, PDF_CMAP_RANGE, 1465 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 7187 },
	{ 0x9852, 0x9853, PDF_CMAP_RANGE, 15292 },
	{ 0x9854, 0x9854, PDF_CMAP_SINGLE, 1573 },
	{ 0x9855, 0x9855, PDF_CMAP_SINGLE, 1894 },
	{ 0x9857, 0x9857, PDF_CMAP_SINGLE, 8697 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 1574 },
	{ 0x9859, 0x9859, PDF_CMAP_SINGLE, 15294 },
	{ 0x985a, 0x985a, PDF_CMAP_SINGLE, 7752 },
	{ 0x985b, 0x985b, PDF_CMAP_SINGLE, 3129 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 4008 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 8698 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 1936 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 7189 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 15295 },
	{ 0x986f, 0x9871, PDF_CMAP_RANGE, 7190 },
	{ 0x9873, 0x9873, PDF_CMAP_SINGLE, 7194 },
	{ 0x9874, 0x9874, PDF_CMAP_SINGLE, 7193 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 3561 },
	{ 0x98aa, 0x98aa, PDF_CMAP_SINGLE, 7195 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 7196 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 7197 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 7198 },
	{ 0x98b8, 0x98b8, PDF_CMAP_SINGLE, 15296 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 15297 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 15298 },
	{ 0x98c3, 0x98c3, PDF_CMAP_SINGLE, 7200 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 7199 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 7201 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 15299 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 3464 },
	{ 0x98dc, 0x98dc, PDF_CMAP_SINGLE, 6201 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 2543 },
	{ 0x98e0, 0x98e0, PDF_CMAP_SINGLE, 13848 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 1612 },
	{ 0x98e5, 0x98e5, PDF_CMAP_SINGLE, 15300 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 7202 },
	{ 0x98eb, 0x98eb, PDF_CMAP_SINGLE, 7203 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 4289 },
	{ 0x98ee, 0x98ee, PDF_CMAP_SINGLE, 5338 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 3431 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 1215 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 1151 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 2242 },
	{ 0x98fd, 0x98fd, PDF_CMAP_SINGLE, 3678 },
	{ 0x98fe, 0x98fe, PDF_CMAP_SINGLE, 2534 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 7204 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 3819 },
	{ 0x9909, 0x9909, PDF_CMAP_SINGLE, 7205 },
	{ 0x990a, 0x990a, PDF_CMAP_SINGLE, 3910 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 1252 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 2191 },
	{ 0x9912, 0x9912, PDF_CMAP_SINGLE, 7206 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 1390 },
	{ 0x9914, 0x9914, PDF_CMAP_SINGLE, 7207 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 7208 },
	{ 0x991d, 0x991e, PDF_CMAP_RANGE, 7210 },
	{ 0x9920, 0x9920, PDF_CMAP_SINGLE, 7213 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 7209 },
	{ 0x9924, 0x9924, PDF_CMAP_SINGLE, 7212 },
	{ 0x9927, 0x9927, PDF_CMAP_SINGLE, 8701 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 1559 },
	{ 0x992c, 0x992c, PDF_CMAP_SINGLE, 7214 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7215 },
	{ 0x9932, 0x9933, PDF_CMAP_RANGE, 15301 },
	{ 0x993d, 0x993e, PDF_CMAP_RANGE, 7216 },
	{ 0x9940, 0x9940, PDF_CMAP_SINGLE, 15303 },
	{ 0x9942, 0x9942, PDF_CMAP_SINGLE, 7218 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 7220 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 7219 },
	{ 0x994b, 0x994b, PDF_CMAP_SINGLE, 7222 },
	{ 0x994c, 0x994c, PDF_CMAP_SINGLE, 7225 },
	{ 0x994d, 0x994d, PDF_CMAP_SINGLE, 15304 },
	{ 0x9950, 0x9950, PDF_CMAP_SINGLE, 7221 },
	{ 0x9951, 0x9952, PDF_CMAP_RANGE, 7223 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 7226 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 1722 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 15305 },
	{ 0x995f, 0x995f, PDF_CMAP_SINGLE, 15306 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 2335 },
	{ 0x9997, 0x9998, PDF_CMAP_RANGE, 7227 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 2035 },
	{ 0x999e, 0x999e, PDF_CMAP_SINGLE, 8703 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 7229 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 1436 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 3333 },
	{ 0x99ad, 0x99ae, PDF_CMAP_RANGE, 7230 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 15307 },
	{ 0x99b3, 0x99b3, PDF_CMAP_SINGLE, 2968 },
	{ 0x99b4, 0x99b4, PDF_CMAP_SINGLE, 3267 },
	{ 0x99b9, 0x99ba, PDF_CMAP_RANGE, 15308 },
	{ 0x99bc, 0x99bc, PDF_CMAP_SINGLE, 7232 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 3379 },
	{ 0x99c4, 0x99c4, PDF_CMAP_SINGLE, 2860 },
	{ 0x99c5, 0x99c5, PDF_CMAP_SINGLE, 1274 },
	{ 0x99c6, 0x99c6, PDF_CMAP_SINGLE, 1766 },
	{ 0x99c8, 0x99c8, PDF_CMAP_SINGLE, 1767 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 15310 },
	{ 0x99d0, 0x99d0, PDF_CMAP_SINGLE, 2993 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 7237 },
	{ 0x99d2, 0x99d2, PDF_CMAP_SINGLE, 1768 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 1391 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 7236 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 7234 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 7235 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 7233 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 7247 },
	{ 0x99ed, 0x99ee, PDF_CMAP_RANGE, 7238 },
	{ 0x99f1, 0x99f2, PDF_CMAP_RANGE, 7240 },
	{ 0x99f8, 0x99f8, PDF_CMAP_SINGLE, 7243 },
	{ 0x99fb, 0x99fb, PDF_CMAP_SINGLE, 7242 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 2403 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 7244 },
	{ 0x9a02, 0x9a02, PDF_CMAP_SINGLE, 15311 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 7246 },
	{ 0x9a08, 0x9a08, PDF_CMAP_SINGLE, 14266 },
	{ 0x9a0e, 0x9a0e, PDF_CMAP_SINGLE, 1613 },
	{ 0x9a0f, 0x9a0f, PDF_CMAP_SINGLE, 7245 },
	{ 0x9a12, 0x9a12, PDF_CMAP_SINGLE, 2813 },
	{ 0x9a13, 0x9a13, PDF_CMAP_SINGLE, 1895 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 15312 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 7248 },
	{ 0x9a24, 0x9a24, PDF_CMAP_SINGLE, 15313 },
	{ 0x9a27, 0x9a27, PDF_CMAP_SINGLE, 15314 },
	{ 0x9a28, 0x9a28, PDF_CMAP_SINGLE, 2861 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 7249 },
	{ 0x9a2d, 0x9a2e, PDF_CMAP_RANGE, 15315 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 3205 },
	{ 0x9a36, 0x9a36, PDF_CMAP_SINGLE, 15317 },
	{ 0x9a37, 0x9a37, PDF_CMAP_SINGLE, 7250 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 15318 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 7255 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 7253 },
	{ 0x9a42, 0x9a42, PDF_CMAP_SINGLE, 7252 },
	{ 0x9a43, 0x9a43, PDF_CMAP_SINGLE, 7254 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 7251 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 15319 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 7257 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 8704 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 7727 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 7256 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 15320 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 7259 },
	{ 0x9a5a, 0x9a5a, PDF_CMAP_SINGLE, 1723 },
	{ 0x9a5b, 0x9a5b, PDF_CMAP_SINGLE, 7258 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 7260 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 7261 },
	{ 0x9a64, 0x9a64, PDF_CMAP_SINGLE, 7263 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 7262 },
	{ 0x9a69, 0x9a69, PDF_CMAP_SINGLE, 7264 },
	{ 0x9a6a, 0x9a6a, PDF_CMAP_SINGLE, 7266 },
	{ 0x9a6b, 0x9a6b, PDF_CMAP_SINGLE, 7265 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2062 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 7267 },
	{ 0x9ab0, 0x9ab0, PDF_CMAP_SINGLE, 7268 },
	{ 0x9ab5, 0x9ab6, PDF_CMAP_RANGE, 15321 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 1434 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 7269 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 7270 },
	{ 0x9ac4, 0x9ac4, PDF_CMAP_SINGLE, 2615 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 7271 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 7272 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 7273 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 2036 },
	{ 0x9ad9, 0x9ad9, PDF_CMAP_SINGLE, 8705 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 8706 },
	{ 0x9ade, 0x9adf, PDF_CMAP_RANGE, 7275 },
	{ 0x9ae2, 0x9ae3, PDF_CMAP_RANGE, 7277 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7279 },
	{ 0x9aea, 0x9aea, PDF_CMAP_SINGLE, 3397 },
	{ 0x9aeb, 0x9aeb, PDF_CMAP_SINGLE, 7281 },
	{ 0x9aed, 0x9aed, PDF_CMAP_SINGLE, 3480 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 7282 },
	{ 0x9aef, 0x9aef, PDF_CMAP_SINGLE, 7280 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 7284 },
	{ 0x9af4, 0x9af4, PDF_CMAP_SINGLE, 7283 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 7285 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 15323 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 7286 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 15324 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 7287 },
	{ 0x9b18, 0x9b18, PDF_CMAP_SINGLE, 7288 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 7289 },
	{ 0x9b1f, 0x9b1f, PDF_CMAP_SINGLE, 7290 },
	{ 0x9b20, 0x9b20, PDF_CMAP_SINGLE, 15325 },
	{ 0x9b22, 0x9b23, PDF_CMAP_RANGE, 7291 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 7293 },
	{ 0x9b27, 0x9b2a, PDF_CMAP_RANGE, 7294 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 13372 },
	{ 0x9b2e, 0x9b2f, PDF_CMAP_RANGE, 7298 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 5332 },
	{ 0x9b32, 0x9b32, PDF_CMAP_SINGLE, 7300 },
	{ 0x9b33, 0x9b34, PDF_CMAP_RANGE, 15326 },
	{ 0x9b3b, 0x9b3b, PDF_CMAP_SINGLE, 6057 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 1614 },
	{ 0x9b41, 0x9b41, PDF_CMAP_SINGLE, 1407 },
	{ 0x9b42, 0x9b42, PDF_CMAP_SINGLE, 2082 },
	{ 0x9b43, 0x9b43, PDF_CMAP_SINGLE, 7302 },
	{ 0x9b44, 0x9b44, PDF_CMAP_SINGLE, 7301 },
	{ 0x9b45, 0x9b45, PDF_CMAP_SINGLE, 3761 },
	{ 0x9b4d, 0x9b4e, PDF_CMAP_RANGE, 7304 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 7303 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 7306 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 3728 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 7307 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 1685 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4043 },
	{ 0x9b72, 0x9b72, PDF_CMAP_SINGLE, 8708 },
	{ 0x9b73, 0x9b73, PDF_CMAP_SINGLE, 15328 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 7308 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 8707 },
	{ 0x9b79, 0x9b79, PDF_CMAP_SINGLE, 15329 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7310 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 1154 },
	{ 0x9b8f, 0x9b8f, PDF_CMAP_SINGLE, 8709 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 7311 },
	{ 0x9b92, 0x9b92, PDF_CMAP_SINGLE, 3579 },
	{ 0x9b93, 0x9b93, PDF_CMAP_SINGLE, 7309 },
	{ 0x9b96, 0x9b97, PDF_CMAP_RANGE, 7312 },
	{ 0x9b9f, 0x9ba0, PDF_CMAP_RANGE, 7314 },
	{ 0x9ba7, 0x9ba7, PDF_CMAP_SINGLE, 15330 },
	{ 0x9ba8, 0x9ba8, PDF_CMAP_SINGLE, 7316 },
	{ 0x9baa, 0x9baa, PDF_CMAP_SINGLE, 3740 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 2171 },
	{ 0x9bad, 0x9bad, PDF_CMAP_SINGLE, 2154 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 2737 },
	{ 0x9bb1, 0x9bb1, PDF_CMAP_SINGLE, 8710 },
	{ 0x9bb4, 0x9bb4, PDF_CMAP_SINGLE, 7317 },
	{ 0x9bb9, 0x9bb9, PDF_CMAP_SINGLE, 7320 },
	{ 0x9bbb, 0x9bbb, PDF_CMAP_SINGLE, 8711 },
	{ 0x9bc0, 0x9bc0, PDF_CMAP_SINGLE, 7318 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 15331 },
	{ 0x9bc6, 0x9bc6, PDF_CMAP_SINGLE, 7321 },
	{ 0x9bc7, 0x9bc7, PDF_CMAP_SINGLE, 15332 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 1957 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 7319 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 7322 },
	{ 0x9bd1, 0x9bd2, PDF_CMAP_RANGE, 7323 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 7328 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 2168 },
	{ 0x9bd7, 0x9bd7, PDF_CMAP_SINGLE, 15333 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 2884 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 14271 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 7329 },
	{ 0x9be2, 0x9be2, PDF_CMAP_SINGLE, 7326 },
	{ 0x9be3, 0x9be3, PDF_CMAP_SINGLE, 7325 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 7327 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 15334 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 1845 },
	{ 0x9beb, 0x9beb, PDF_CMAP_SINGLE, 15335 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 7333 },
	{ 0x9bf1, 0x9bf1, PDF_CMAP_SINGLE, 7332 },
	{ 0x9bf2, 0x9bf2, PDF_CMAP_SINGLE, 7331 },
	{ 0x9bf5, 0x9bf5, PDF_CMAP_SINGLE, 1143 },
	{ 0x9bf7, 0x9bf7, PDF_CMAP_SINGLE, 15336 },
	{ 0x9bfa, 0x9bfa, PDF_CMAP_SINGLE, 15337 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 15338 },
	{ 0x9c00, 0x9c00, PDF_CMAP_SINGLE, 8712 },
	{ 0x9c04, 0x9c04, PDF_CMAP_SINGLE, 7343 },
	{ 0x9c06, 0x9c06, PDF_CMAP_SINGLE, 7339 },
	{ 0x9c08, 0x9c08, PDF_CMAP_SINGLE, 7340 },
	{ 0x9c09, 0x9c09, PDF_CMAP_SINGLE, 7336 },
	{ 0x9c0a, 0x9c0a, PDF_CMAP_SINGLE, 7342 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 15339 },
	{ 0x9c0c, 0x9c0c, PDF_CMAP_SINGLE, 7338 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 1472 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 4082 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 7341 },
	{ 0x9c13, 0x9c13, PDF_CMAP_SINGLE, 7337 },
	{ 0x9c14, 0x9c14, PDF_CMAP_SINGLE, 7335 },
	{ 0x9c15, 0x9c15, PDF_CMAP_SINGLE, 7334 },
	{ 0x9c1b, 0x9c1b, PDF_CMAP_SINGLE, 7345 },
	{ 0x9c21, 0x9c21, PDF_CMAP_SINGLE, 7348 },
	{ 0x9c24, 0x9c24, PDF_CMAP_SINGLE, 7347 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 7346 },
	{ 0x9c27, 0x9c27, PDF_CMAP_SINGLE, 15340 },
	{ 0x9c2a, 0x9c2a, PDF_CMAP_SINGLE, 15341 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 3515 },
	{ 0x9c2e, 0x9c2e, PDF_CMAP_SINGLE, 7344 },
	{ 0x9c2f, 0x9c2f, PDF_CMAP_SINGLE, 1207 },
	{ 0x9c30, 0x9c30, PDF_CMAP_SINGLE, 7349 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 7351 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 15342 },
	{ 0x9c39, 0x9c39, PDF_CMAP_SINGLE, 1485 },
	{ 0x9c3a, 0x9c3a, PDF_CMAP_SINGLE, 7330 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 1241 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 7353 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 15343 },
	{ 0x9c46, 0x9c46, PDF_CMAP_SINGLE, 7352 },
	{ 0x9c47, 0x9c47, PDF_CMAP_SINGLE, 7350 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 2923 },
	{ 0x9c52, 0x9c52, PDF_CMAP_SINGLE, 3742 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 15344 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 4002 },
	{ 0x9c5a, 0x9c5a, PDF_CMAP_SINGLE, 7354 },
	{ 0x9c60, 0x9c60, PDF_CMAP_SINGLE, 7355 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 15345 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 7356 },
	{ 0x9c6a, 0x9c6a, PDF_CMAP_SINGLE, 15346 },
	{ 0x9c76, 0x9c76, PDF_CMAP_SINGLE, 7357 },
	{ 0x9c77, 0x9c77, PDF_CMAP_SINGLE, 15347 },
	{ 0x9c78, 0x9c78, PDF_CMAP_SINGLE, 7358 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 3031 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 7359 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3403 },
	{ 0x9ceb, 0x9ceb, PDF_CMAP_SINGLE, 7364 },
	{ 0x9cec, 0x9cec, PDF_CMAP_SINGLE, 7360 },
	{ 0x9cf0, 0x9cf0, PDF_CMAP_SINGLE, 7361 },
	{ 0x9cf3, 0x9cf3, PDF_CMAP_SINGLE, 3679 },
	{ 0x9cf4, 0x9cf4, PDF_CMAP_SINGLE, 3792 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 3240 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 15348 },
	{ 0x9d03, 0x9d03, PDF_CMAP_SINGLE, 7365 },
	{ 0x9d06, 0x9d06, PDF_CMAP_SINGLE, 7366 },
	{ 0x9d07, 0x9d07, PDF_CMAP_SINGLE, 3222 },
	{ 0x9d08, 0x9d08, PDF_CMAP_SINGLE, 7363 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 7362 },
	{ 0x9d0e, 0x9d0e, PDF_CMAP_SINGLE, 1322 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 7374 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 7373 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 1303 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 7371 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 7370 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 7368 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 1497 },
	{ 0x9d2a, 0x9d2a, PDF_CMAP_SINGLE, 7367 },
	{ 0x9d2b, 0x9d2b, PDF_CMAP_SINGLE, 2270 },
	{ 0x9d2c, 0x9d2c, PDF_CMAP_SINGLE, 1321 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 2037 },
	{ 0x9d3e, 0x9d3e, PDF_CMAP_SINGLE, 7377 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 7376 },
	{ 0x9d41, 0x9d41, PDF_CMAP_SINGLE, 7375 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 15349 },
	{ 0x9d44, 0x9d44, PDF_CMAP_SINGLE, 7372 },
	{ 0x9d46, 0x9d46, PDF_CMAP_SINGLE, 7378 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 15350 },
	{ 0x9d48, 0x9d48, PDF_CMAP_SINGLE, 7379 },
	{ 0x9d50, 0x9d50, PDF_CMAP_SINGLE, 7384 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 7383 },
	{ 0x9d59, 0x9d59, PDF_CMAP_SINGLE, 7385 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 1231 },
	{ 0x9d5d, 0x9d5e, PDF_CMAP_RANGE, 7380 },
	{ 0x9d60, 0x9d60, PDF_CMAP_SINGLE, 2054 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 3781 },
	{ 0x9d63, 0x9d63, PDF_CMAP_SINGLE, 15351 },
	{ 0x9d64, 0x9d64, PDF_CMAP_SINGLE, 7382 },
	{ 0x9d69, 0x9d69, PDF_CMAP_SINGLE, 15352 },
	{ 0x9d6b, 0x9d6b, PDF_CMAP_SINGLE, 8714 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 3680 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 7390 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 8713 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 7386 },
	{ 0x9d7a, 0x9d7a, PDF_CMAP_SINGLE, 7391 },
	{ 0x9d7c, 0x9d7c, PDF_CMAP_SINGLE, 15353 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15354 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 7388 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 7387 },
	{ 0x9d8d, 0x9d8d, PDF_CMAP_SINGLE, 15355 },
	{ 0x9d8f, 0x9d8f, PDF_CMAP_SINGLE, 1842 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 7392 },
	{ 0x9da4, 0x9da4, PDF_CMAP_SINGLE, 7393 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 7394 },
	{ 0x9dab, 0x9dab, PDF_CMAP_SINGLE, 7389 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 7369 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 15356 },
	{ 0x9db2, 0x9db2, PDF_CMAP_SINGLE, 7395 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 3069 },
	{ 0x9db8, 0x9db8, PDF_CMAP_SINGLE, 7399 },
	{ 0x9dba, 0x9dba, PDF_CMAP_SINGLE, 7400 },
	{ 0x9dbb, 0x9dbb, PDF_CMAP_SINGLE, 7398 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 7397 },
	{ 0x9dc2, 0x9dc2, PDF_CMAP_SINGLE, 7403 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 15357 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 7396 },
	{ 0x9dc6, 0x9dc6, PDF_CMAP_SINGLE, 7401 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 15358 },
	{ 0x9dcf, 0x9dcf, PDF_CMAP_SINGLE, 7402 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 7405 },
	{ 0x9dd6, 0x9dd6, PDF_CMAP_SINGLE, 15359 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 7646 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 7404 },
	{ 0x9ddf, 0x9ddf, PDF_CMAP_SINGLE, 15360 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 7407 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 15361 },
	{ 0x9ded, 0x9ded, PDF_CMAP_SINGLE, 7408 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 7409 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 4079 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 15362 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 7406 },
	{ 0x9df9, 0x9df9, PDF_CMAP_SINGLE, 2891 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 2141 },
	{ 0x9dfd, 0x9dfd, PDF_CMAP_SINGLE, 7410 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 15363 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 8716 },
	{ 0x9e1a, 0x9e1b, PDF_CMAP_RANGE, 7411 },
	{ 0x9e1d, 0x9e1d, PDF_CMAP_SINGLE, 15364 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 7413 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 7414 },
	{ 0x9e78, 0x9e78, PDF_CMAP_SINGLE, 1896 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 7415 },
	{ 0x9e7c, 0x9e7c, PDF_CMAP_SINGLE, 7677 },
	{ 0x9e7d, 0x9e7d, PDF_CMAP_SINGLE, 7416 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 2267 },
	{ 0x9e81, 0x9e81, PDF_CMAP_SINGLE, 7417 },
	{ 0x9e88, 0x9e88, PDF_CMAP_SINGLE, 7418 },
	{ 0x9e8b, 0x9e8c, PDF_CMAP_RANGE, 7419 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 7423 },
	{ 0x9e92, 0x9e92, PDF_CMAP_SINGLE, 7421 },
	{ 0x9e93, 0x9e93, PDF_CMAP_SINGLE, 4066 },
	{ 0x9e95, 0x9e95, PDF_CMAP_SINGLE, 7422 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 4023 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 7424 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 4003 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 15365 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 7425 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 3380 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 15366 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 7426 },
	{ 0x9eaa, 0x9eaa, PDF_CMAP_SINGLE, 7428 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 15367 },
	{ 0x9ead, 0x9ead, PDF_CMAP_SINGLE, 7429 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 7682 },
	{ 0x9eb5, 0x9eb5, PDF_CMAP_SINGLE, 7797 },
	{ 0x9eb8, 0x9eb8, PDF_CMAP_SINGLE, 7427 },
	{ 0x9eb9, 0x9eb9, PDF_CMAP_SINGLE, 2047 },
	{ 0x9eba, 0x9eba, PDF_CMAP_SINGLE, 3801 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 3729 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 4740 },
	{ 0x9ebe, 0x9ebe, PDF_CMAP_SINGLE, 5375 },
	{ 0x9ebf, 0x9ebf, PDF_CMAP_SINGLE, 3753 },
	{ 0x9ec3, 0x9ec3, PDF_CMAP_SINGLE, 13323 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 1323 },
	{ 0x9ecc, 0x9ecc, PDF_CMAP_SINGLE, 7431 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 1642 },
	{ 0x9ece, 0x9ed0, PDF_CMAP_RANGE, 7432 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 8717 },
	{ 0x9ed2, 0x9ed2, PDF_CMAP_SINGLE, 2055 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 7435 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 5645 },
	{ 0x9ed9, 0x9ed9, PDF_CMAP_SINGLE, 3815 },
	{ 0x9edb, 0x9edb, PDF_CMAP_SINGLE, 2883 },
	{ 0x9edc, 0x9edc, PDF_CMAP_SINGLE, 7436 },
	{ 0x9edd, 0x9edd, PDF_CMAP_SINGLE, 7438 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 7437 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 7439 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 7440 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 15368 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 7441 },
	{ 0x9eee, 0x9eee, PDF_CMAP_SINGLE, 15369 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 7442 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 7443 },
	{ 0x9ef6, 0x9ef7, PDF_CMAP_RANGE, 7444 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 7446 },
	{ 0x9efb, 0x9efd, PDF_CMAP_RANGE, 7447 },
	{ 0x9f07, 0x9f08, PDF_CMAP_RANGE, 7450 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3102 },
	{ 0x9f10, 0x9f10, PDF_CMAP_SINGLE, 15370 },
	{ 0x9f12, 0x9f12, PDF_CMAP_SINGLE, 15371 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 1937 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 7453 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 15372 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 15373 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 2767 },
	{ 0x9f21, 0x9f21, PDF_CMAP_SINGLE, 7454 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7455 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 15374 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 15375 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 15376 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 3475 },
	{ 0x9f3e, 0x9f3e, PDF_CMAP_SINGLE, 7456 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 15377 },
	{ 0x9f45, 0x9f45, PDF_CMAP_SINGLE, 15378 },
	{ 0x9f4a, 0x9f4a, PDF_CMAP_SINGLE, 7457 },
	{ 0x9f4b, 0x9f4b, PDF_CMAP_SINGLE, 5898 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 6779 },
	{ 0x9f4f, 0x9f4f, PDF_CMAP_SINGLE, 7174 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7458 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 7459 },
	{ 0x9f57, 0x9f57, PDF_CMAP_SINGLE, 15379 },
	{ 0x9f5f, 0x9f61, PDF_CMAP_RANGE, 7461 },
	{ 0x9f62, 0x9f62, PDF_CMAP_SINGLE, 4024 },
	{ 0x9f63, 0x9f63, PDF_CMAP_SINGLE, 7460 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 7464 },
	{ 0x9f68, 0x9f68, PDF_CMAP_SINGLE, 15380 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 7467 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 7466 },
	{ 0x9f71, 0x9f71, PDF_CMAP_SINGLE, 15381 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 7469 },
	{ 0x9f75, 0x9f75, PDF_CMAP_SINGLE, 15382 },
	{ 0x9f76, 0x9f76, PDF_CMAP_SINGLE, 7470 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 7468 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 3966 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 15383 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 15384 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 7471 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 7472 },
	{ 0x9f9d, 0x9f9d, PDF_CMAP_SINGLE, 5927 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 7473 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 15385 },
	{ 0xf929, 0xf929, PDF_CMAP_SINGLE, 8489 },
	{ 0xf9dc, 0xf9dc, PDF_CMAP_SINGLE, 8686 },
	{ 0xfa0e, 0xfa0e, PDF_CMAP_SINGLE, 8410 },
	{ 0xfa0f, 0xfa10, PDF_CMAP_RANGE, 8421 },
	{ 0xfa11, 0xfa11, PDF_CMAP_SINGLE, 8443 },
	{ 0xfa12, 0xfa12, PDF_CMAP_SINGLE, 8481 },
	{ 0xfa13, 0xfa13, PDF_CMAP_SINGLE, 8497 },
	{ 0xfa14, 0xfa14, PDF_CMAP_SINGLE, 8499 },
	{ 0xfa15, 0xfa15, PDF_CMAP_SINGLE, 8542 },
	{ 0xfa16, 0xfa16, PDF_CMAP_SINGLE, 8548 },
	{ 0xfa17, 0xfa17, PDF_CMAP_SINGLE, 8571 },
	{ 0xfa18, 0xfa1a, PDF_CMAP_RANGE, 8579 },
	{ 0xfa1b, 0xfa1b, PDF_CMAP_SINGLE, 8583 },
	{ 0xfa1c, 0xfa1c, PDF_CMAP_SINGLE, 8587 },
	{ 0xfa1d, 0xfa1d, PDF_CMAP_SINGLE, 8590 },
	{ 0xfa1e, 0xfa1e, PDF_CMAP_SINGLE, 8599 },
	{ 0xfa1f, 0xfa1f, PDF_CMAP_SINGLE, 8610 },
	{ 0xfa20, 0xfa21, PDF_CMAP_RANGE, 8612 },
	{ 0xfa22, 0xfa22, PDF_CMAP_SINGLE, 8622 },
	{ 0xfa23, 0xfa23, PDF_CMAP_SINGLE, 8630 },
	{ 0xfa24, 0xfa25, PDF_CMAP_RANGE, 8632 },
	{ 0xfa26, 0xfa26, PDF_CMAP_SINGLE, 8636 },
	{ 0xfa27, 0xfa27, PDF_CMAP_SINGLE, 8664 },
	{ 0xfa28, 0xfa28, PDF_CMAP_SINGLE, 8671 },
	{ 0xfa29, 0xfa29, PDF_CMAP_SINGLE, 8687 },
	{ 0xfa2a, 0xfa2b, PDF_CMAP_RANGE, 8699 },
	{ 0xfa2c, 0xfa2c, PDF_CMAP_SINGLE, 8702 },
	{ 0xfa2d, 0xfa2d, PDF_CMAP_SINGLE, 8715 },
	{ 0xfb00, 0xfb00, PDF_CMAP_SINGLE, 9358 },
	{ 0xfb01, 0xfb02, PDF_CMAP_RANGE, 112 },
	{ 0xfb03, 0xfb04, PDF_CMAP_RANGE, 9359 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 7898 },
	{ 0xfe31, 0xfe32, PDF_CMAP_RANGE, 7892 },
	{ 0xfe33, 0xfe33, PDF_CMAP_SINGLE, 7890 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 7899 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 7905 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 7901 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 7915 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 7909 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 7907 },
	{ 0xfe41, 0xfe44, PDF_CMAP_RANGE, 7911 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 642 },
	{ 0xff02, 0xff02, PDF_CMAP_SINGLE, 8007 },
	{ 0xff03, 0xff03, PDF_CMAP_SINGLE, 716 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 712 },
	{ 0xff05, 0xff05, PDF_CMAP_SINGLE, 715 },
	{ 0xff06, 0xff06, PDF_CMAP_SINGLE, 717 },
	{ 0xff07, 0xff07, PDF_CMAP_SINGLE, 8006 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 674 },
	{ 0xff0a, 0xff0a, PDF_CMAP_SINGLE, 718 },
	{ 0xff0b, 0xff0b, PDF_CMAP_SINGLE, 692 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 636 },
	{ 0xff0d, 0xff0d, PDF_CMAP_SINGLE, 693 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 637 },
	{ 0xff0f, 0xff0f, PDF_CMAP_SINGLE, 663 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 780 },
	{ 0xff1a, 0xff1b, PDF_CMAP_RANGE, 639 },
	{ 0xff1c, 0xff1c, PDF_CMAP_SINGLE, 699 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 697 },
	{ 0xff1e, 0xff1e, PDF_CMAP_SINGLE, 700 },
	{ 0xff1f, 0xff1f, PDF_CMAP_SINGLE, 641 },
	{ 0xff20, 0xff20, PDF_CMAP_SINGLE, 719 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 790 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 678 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 664 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 679 },
	{ 0xff3e, 0xff3e, PDF_CMAP_SINGLE, 648 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 650 },
	{ 0xff40, 0xff40, PDF_CMAP_SINGLE, 646 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 816 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 680 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 667 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 681 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 665 },
	{ 0xff61, 0xff9f, PDF_CMAP_RANGE, 327 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 713 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 751 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 649 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 8005 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 711 },
	{ 0xffe8, 0xffe8, PDF_CMAP_SINGLE, 323 },
};

static const unsigned short pdf_cmap_UniJIS_UCS2_H_table[5354] =
{
	107,61,99,720,647,152,140,109,
	153,151,154,129,707,694,645,159,
	778,117,134,160,144,123,695,142,
	696,148,9366,9361,9369,9364,9395,9407,
	12092,9400,9412,9367,9362,141,147,9436,
	9370,9365,143,149,223,227,9405,9417,
	9368,9363,9404,9416,225,229,9394,9406,
	9398,9410,9401,9413,9403,9415,9426,9429,
	9432,95,129,226,133,135,137,134,
	136,12090,12096,9397,9409,231,633,662,
	14,114,114,138,661,666,669,668,
	734,12111,8305,8304,8055,228,736,739,
	12201,12215,8013,8012,8014,8011,759,755,
	760,741,12091,742,7625,693,12118,767,
	703,7629,756,748,747,8195,7624,704,
	768,665,766,698,761,12188,12186,12189,
	12187,12186,12190,7897,668,8250,8249,8244,
	8243,8242,8241,8240,8239,8238,729,728,
	8015,12239,12237,731,730,12195,8010,733,
	732,12194,8009,727,726,723,10502,725,
	724,8258,8257,12254,722,721,8219,8222,
	8211,8017,8019,8213,8018,8212,8214,8016,
	12100,774,4209,14476,4646,3762,4739,4779,
	15391,14530,13852,14561,5059,3644,3773,14749,
	15398,14780,13729,13995,14999,2260,14905,13922,
	15000,14999,14189,13747,13646,1931,15114,13870,
	2658,15403,15184,3029,15255,15262,13645,2664,
	13847,13849,13848,3380,1323,1200,8371,4095,
	4097,1333,4102,3275,4110,2579,4208,3286,
	3392,4219,4227,4233,4243,4248,3163,3991,
	4294,4307,2375,3708,4316,4321,4328,3746,
	1969,4459,3156,2204,3878,2887,2433,2208,
	4622,2631,2454,4646,4648,4658,2177,4716,
	1979,1918,1738,1519,4739,4741,4761,4763,
	4768,1655,14521,4783,4785,2554,4930,1921,
	2326,2215,5058,3592,3143,1740,3661,5088,
	3284,5132,1860,3814,1853,2221,5349,5364,
	5368,3450,3807,2223,5378,2603,1360,3066,
	3541,5604,5606,3618,1383,1671,1880,1904,
	1732,1245,1504,1537,2652,3899,3134,3479,
	14848,5783,3368,3453,2172,3816,3779,3836,
	2676,2260,14913,1363,1856,3953,2971,3606,
	2227,1544,6163,3901,1227,4061,2261,6205,
	2262,6227,3281,2569,2263,2232,1235,2697,
	2726,2360,2081,2541,6322,6479,2988,1858,
	2022,1189,6635,1887,1455,1908,2921,3198,
	6742,6745,1419,2682,2808,2829,2574,2306,
	2575,2914,15183,3874,3243,3428,3948,1754,
	3029,3827,3550,7113,7115,1229,8695,3463,
	3800,1461,7171,7173,1339,3607,3561,3464,
	2543,2335,2035,3333,2062,2036,7276,7293,
	1614,1685,3031,7414,2267,7425,3729,13323,
	1642,2055,7446,7449,3102,1937,2767,3475,
	655,8308,735,740,665,7608,7609,8058,
	638,660,8143,8141,8148,8147,8204,8142,
	7620,8151,8149,8144,8139,8146,8140,8150,
	8205,8145,8138,10471,8161,8160,8162,8156,
	10495,8165,10492,8158,8223,10489,10488,10494,
	8319,8191,10479,10491,10490,8154,8157,8159,
	8163,8153,8164,8155,8152,8042,11876,8183,
	11877,11881,11879,11884,11882,11886,11888,7595,
	7586,8041,8039,11896,8040,11898,8038,8043,
	11907,11909,7596,7590,8052,7598,8049,11921,
	8327,7592,11930,11932,8046,11933,8047,11926,
	11934,11935,8045,8050,11943,7585,7599,7588,
	8328,7593,11950,11954,11951,8323,7623,7622,
	7621,8054,8037,8024,8026,8186,8020,7607,
	8021,8187,8030,8029,8028,8027,8034,8182,
	7611,1200,3000,3754,2510,2174,2509,1340,
	3526,3881,4091,1233,1484,4092,2632,4311,
	1648,3594,2511,14298,4093,14299,14157,2980,
	4094,1778,4095,13981,1561,2926,2323,4096,
	4098,3307,14304,1649,3259,1911,3681,6480,
	4099,2512,4100,1333,14305,1757,1956,3829,
	1505,14306,1615,4102,3971,3879,2794,4104,
	2244,3275,14307,4107,1248,1939,1938,1194,
	14308,4081,4080,2083,1125,3682,4111,1958,
	1195,3744,3070,3972,14309,2579,13856,2372,
	2580,4119,4118,1650,2067,1392,4116,4115,
	3577,14310,2196,2195,2846,4120,3527,2699,
	656,4121,4123,14311,8372,2885,4009,1166,
	4122,1342,1861,4124,3290,8373,14312,14313,
	8374,1575,4125,1167,14314,1940,1576,3564,
	3398,1651,4160,3131,3362,4127,2547,8376,
	2197,14317,2245,1344,2912,4131,1168,3071,
	2373,2084,3854,2862,14319,1343,8377,4130,
	3880,4126,4128,2142,4129,4563,4140,4138,
	3595,4132,14320,4136,1959,8379,4011,4139,
	8378,4142,14323,4135,8380,1689,1690,1345,
	3751,3552,1960,2549,3967,14324,3624,1806,
	2821,1380,8364,2397,8382,4146,14325,4151,
	4149,15407,4144,2831,4147,4150,4148,4145,
	7660,2548,2350,4166,3496,4161,14326,4165,
	8385,4159,4168,5632,14329,4167,3159,1961,
	4154,14330,8386,2310,4162,8384,3647,2955,
	4158,1863,14331,4155,4163,4157,3993,4164,
	4071,14332,8389,4169,8387,4170,4174,1170,
	3616,4173,4172,4176,14333,1864,14334,2822,
	3073,1774,2175,3467,4180,2101,3885,4183,
	4186,4182,2100,14340,3207,3707,4188,8392,
	15408,3973,4190,14341,2768,4191,4193,14343,
	15409,8391,4195,14344,4196,4201,4200,4199,
	4202,14347,14348,2440,4204,4207,4206,1897,
	1809,2374,3001,1692,2700,1963,8393,2048,
	4211,3796,3136,8394,3286,13966,2742,3392,
	1964,4065,4217,3596,2838,1769,3119,14201,
	3258,1281,4222,2157,4221,2102,4223,6235,
	4224,3695,4225,1507,14350,4230,4228,3785,
	4229,4234,2131,4238,3830,4012,14351,4239,
	8396,15410,2636,4240,2404,3002,3975,3162,
	4242,8284,1725,14352,4243,3724,2418,2908,
	4245,3260,8397,4246,14353,4247,1420,14354,
	4248,1695,3236,1308,2394,3381,4249,2581,
	4250,3580,2686,1502,1509,4251,14357,4252,
	8398,3409,3612,3938,4255,2637,2158,1866,
	4258,2199,2049,3074,4260,14358,2143,7774,
	4263,1867,2126,3363,14360,4267,4265,3565,
	2514,4274,1474,4268,4266,2769,1846,4275,
	3957,4276,4273,14361,4272,3991,8399,1965,
	1347,2431,3154,2039,4049,4282,8401,4283,
	3856,14362,3625,4285,1510,3775,2441,4286,
	3639,4290,2638,4287,1736,4288,1511,1796,
	4291,1967,3818,8403,3828,3279,3649,4295,
	8404,4296,4300,4299,4301,1341,3706,2156,
	4302,2442,1697,4303,8405,4307,3478,1760,
	1193,14366,3223,4308,2375,4310,4309,2443,
	1941,4312,3410,14368,14367,4313,3440,2836,
	2894,1696,3270,2927,2702,14369,4317,1230,
	1209,1577,8406,2824,1643,3931,4320,1335,
	1898,4323,4325,4324,2597,1243,1280,4326,
	8408,1899,2176,4329,3746,2085,1652,3857,
	2770,3411,2345,2324,2337,2432,14372,3412,
	1253,2771,1969,1913,1759,14373,4336,2911,
	2910,1699,2444,4337,4335,1348,2886,2276,
	2201,1224,14374,1486,2040,2200,1635,1444,
	2041,1634,3067,1223,3209,3786,1971,3939,
	3137,1970,1755,3704,3441,14375,1653,2599,
	3581,4344,4340,1943,13775,4341,3076,1942,
	2050,14376,4349,2338,14378,4356,4354,3759,
	14379,4351,4360,4355,4358,1914,3787,8412,
	2144,4072,14380,4352,4350,4362,14382,4365,
	4374,4367,4371,8413,4372,4368,14383,2137,
	1423,4366,14384,4376,1210,4375,14385,1129,
	3516,4373,14386,4364,4370,2104,1211,4386,
	2445,3735,4384,4383,2086,2550,14388,4380,
	3164,4391,3853,4396,4387,2895,4393,2446,
	4392,4398,7633,4404,4409,4410,2739,8415,
	4412,14394,1972,4402,3003,4406,4399,1513,
	1578,1475,4407,4403,1868,4413,4411,2773,
	1636,1700,4417,4415,2202,4419,4425,4424,
	14396,1349,4423,2448,1237,2532,4433,4432,
	14399,4434,14402,2747,4428,14403,4429,1579,
	14405,4439,4435,4438,4440,1443,4444,14409,
	7654,14410,4454,4452,4450,4453,4455,14414,
	7770,4459,4220,2344,2203,14415,1395,2946,
	14416,1171,2596,4461,1915,4463,4462,3632,
	4464,4466,4465,14417,4467,1282,4469,14418,
	4471,4474,1145,2127,14420,2957,4477,4480,
	2097,1973,2069,8417,2929,4481,3062,4483,
	4486,1813,14425,1974,1438,4490,4494,8420,
	8419,1196,3528,3347,1580,14432,3719,1870,
	2863,14433,4498,4501,14434,4505,7474,1283,
	3651,14435,2516,3138,3597,4005,1396,4507,
	2748,4510,3139,3166,3405,3049,7751,4506,
	8423,4515,3709,14438,8424,3583,4520,4514,
	4517,4519,14440,4524,4523,14441,2042,4525,
	4527,4526,14442,4518,4531,4530,4528,2204,
	2583,2774,4532,2656,1201,3354,4534,4533,
	3617,4539,8425,14444,1350,4540,3878,1422,
	4318,2386,2847,4541,3831,14445,4543,3121,
	2848,3529,1309,14446,2278,4546,1172,4547,
	14117,1581,3256,3652,4552,2775,4551,1814,
	8426,3721,4550,3167,4554,4553,8427,4556,
	14447,4555,8429,4557,2449,4559,2915,4558,
	14448,3587,2433,3157,4561,3287,3442,3805,
	4565,8430,2853,3731,8431,2106,2206,14449,
	2205,1149,1916,14450,2639,1173,4569,14451,
	1242,1515,3793,3491,13997,1127,14454,4578,
	13761,4576,4574,2551,4579,4050,3330,14457,
	4584,14459,4580,14460,14461,3783,4586,3492,
	4587,4591,4588,1351,4590,4589,1871,4596,
	4593,14466,2978,4600,4599,3064,14470,4609,
	14471,2208,4610,1976,4611,8432,2248,2840,
	2216,1977,1602,1917,4614,1462,4616,4656,
	13840,4623,1225,2325,1158,14475,2777,1516,
	2273,2347,1517,2982,3078,1148,1619,3653,
	1644,2703,2280,3858,4624,1424,1285,2452,
	1352,4625,3888,2387,8433,2320,4626,1583,
	3242,3765,4627,1508,1775,4629,2552,4634,
	2159,1353,4633,4631,4635,4632,3297,5262,
	2553,4636,8436,3004,4638,2864,2339,3559,
	2704,2297,4640,2453,1175,2842,2584,4643,
	3211,2454,4644,14122,13835,2456,4646,14476,
	3820,14478,2348,2312,2546,3276,2586,3468,
	3288,1729,4650,1673,4651,3239,1328,2209,
	4652,4655,4654,1781,3141,2292,7693,14479,
	2778,3940,4658,3246,1584,4663,2749,1568,
	4666,3764,4668,4670,1563,8439,4667,4669,
	3221,1702,1381,4675,4680,3655,3654,14484,
	8440,3169,4677,2398,4688,4682,1425,4683,
	4692,4691,4690,4686,14486,8441,8444,14487,
	14489,4696,4693,4694,8442,4697,8445,3170,
	4679,4710,4707,1564,4712,4714,4713,2706,
	2349,14495,2414,13362,2789,1979,2088,1980,
	1674,1918,4718,3762,3321,4719,1981,14496,
	2917,1738,13794,14497,2210,3533,4720,1585,
	4723,14498,14501,4722,4721,4724,2670,14502,
	2867,1596,4728,4727,3567,4732,3737,4731,
	14503,3388,4734,3598,4735,1519,3599,3301,
	1982,1520,4739,1900,3886,3859,1586,4741,
	1983,2457,3080,3657,3123,14000,1984,3155,
	2098,14506,14507,1919,1159,2424,1985,3889,
	4745,4031,4051,14510,4747,4746,14511,1445,
	3506,2459,4754,4753,4749,4760,4758,4759,
	4761,1286,3082,4762,1872,1398,3308,4763,
	3283,4764,4052,4765,3600,4768,4090,4106,
	14513,2268,3277,4769,1655,3008,1214,4770,
	3574,1986,8449,14516,3835,1901,1920,14517,
	4771,8450,3009,14518,1703,4773,13720,14519,
	3485,14520,4774,14521,4779,14522,4782,4767,
	4766,4783,1815,14523,13996,3481,8451,2108,
	3497,3010,3517,4784,2460,1256,4789,1311,
	2640,4788,4787,1816,2868,4793,4791,14524,
	4790,3951,1945,2435,4792,3142,2376,3224,
	4796,4795,4797,1946,14526,4798,3566,2405,
	4799,3469,14528,3225,3011,13368,8452,14529,
	3114,4800,1605,2554,14530,3486,1587,3292,
	4801,2212,1312,4806,8453,2983,14532,4854,
	4805,4802,14535,2060,4814,4820,4812,4817,
	3535,14536,4811,14537,4816,4015,2213,2869,
	4809,1656,4819,2641,1287,4813,1400,4818,
	4823,4833,4831,4827,4032,4836,4822,4826,
	4810,14539,1402,4830,4832,2959,2072,1336,
	4824,4835,4834,1707,2825,1476,4840,4842,
	8455,2281,8456,4848,3084,4838,4841,4843,
	14541,1947,3860,1521,14542,1275,4847,3312,
	1137,3445,4807,4853,4857,3825,3171,4855,
	2520,4856,3247,2061,14545,2671,8457,1176,
	4851,4849,4839,2178,2854,4870,8460,2781,
	4865,14546,2322,4866,4869,4864,2351,3848,
	3847,1770,1130,14550,1522,8459,4868,4876,
	4875,4879,4882,4874,4878,4873,2250,4877,
	2870,1988,4859,2555,3755,1523,1819,1426,
	3968,4892,1178,4894,4885,4891,4898,1818,
	4886,14552,2816,4033,4906,8464,4899,4904,
	3212,14554,1820,4907,4902,14555,4914,4915,
	2073,4913,4911,4910,4917,4909,4919,4922,
	4912,1874,4927,4926,4928,4925,4929,4933,
	4932,2377,2642,1382,1404,8465,4934,14557,
	1155,2672,4935,6756,4936,1847,14558,4938,
	4939,1620,4940,13390,3690,2420,4943,14560,
	2708,6938,3446,2326,14561,2109,4944,3575,
	14563,4953,2461,1621,4951,4958,2462,13765,
	4952,3322,4959,3172,4956,1989,2690,4973,
	3400,3448,5042,4962,1313,2984,14564,2930,
	4965,4972,4967,4974,4970,4969,3365,4963,
	1405,4961,1675,2899,4957,4960,1990,2687,
	2463,4968,3336,1676,1446,1477,2535,4976,
	4981,4982,2160,2043,2352,4964,2251,4979,
	2214,4980,1160,14566,4975,14567,14568,1677,
	4978,1131,14569,3086,3432,4984,2784,2169,
	4985,4987,4983,3659,2298,4999,4992,2465,
	3264,3303,2783,14575,14576,2340,4995,4989,
	4994,14577,14578,4996,3955,2110,2931,4993,
	1991,2602,1288,2750,4991,1631,3051,4997,
	2839,5008,3507,3087,14579,5006,3890,1525,
	14580,13340,1588,1289,8467,5007,13892,5018,
	14582,5012,7724,5009,5016,3414,3173,2630,
	14587,2179,5025,8469,3213,3115,5026,3323,
	2161,5034,5029,3892,5036,2786,8470,3105,
	5043,14590,2162,5038,5037,5048,5046,14591,
	5049,14592,5051,2521,14593,5054,5052,14595,
	5053,5015,7831,5056,5055,5023,5057,5045,
	5061,5060,5062,1406,3660,2643,14596,5064,
	5067,8471,3524,5066,5065,3338,5068,1709,
	5070,5069,1526,2180,3248,14597,2644,3106,
	3537,5072,3592,4620,2666,3449,3415,5076,
	2300,1740,2673,5078,2192,2949,2217,2557,
	1305,2218,5082,14600,5080,5083,3969,5081,
	2719,5084,1591,13701,3284,2932,1670,2219,
	2787,2406,1140,14601,5090,14602,1314,5094,
	1993,5093,2075,2466,8474,5092,3788,2074,
	1179,2674,8473,8476,2645,1257,8477,2399,
	3732,2146,14603,2468,8475,2635,7680,2985,
	5103,2252,1994,5101,2558,14604,5105,5104,
	5109,14607,5110,14138,5106,8478,1408,3539,
	1824,5111,8482,2646,2469,14609,5112,5116,
	1355,5113,5115,2950,1161,5117,8483,5125,
	5120,5122,8486,13397,3254,14611,5121,2422,
	5127,3893,3374,1730,1258,1995,5133,5134,
	2427,2788,8487,8369,4333,2752,2751,2872,
	2103,1860,3863,3568,8488,5135,14614,2147,
	3035,5136,4053,3015,5137,1592,3760,3748,
	3722,2163,5141,14615,3711,5143,5146,5145,
	14616,2623,3941,1165,2128,2844,5148,2827,
	5149,2522,3817,3922,8490,1996,13681,3339,
	5154,3174,5091,5095,1641,14618,3325,14619,
	5156,2470,3416,5159,5158,1356,2220,4078,
	5161,2619,1923,14620,14621,5167,5165,1357,
	5163,5169,5175,8493,2856,5176,3366,3693,
	1527,2715,2378,14623,3055,14624,3864,5172,
	5171,5174,5164,14625,2986,3844,2290,2148,
	3741,1439,8494,3050,3234,1259,7687,2648,
	1792,1997,1498,5182,1490,5188,14626,1449,
	2076,1448,2111,5183,1851,1825,3175,8495,
	5184,5186,1733,1794,8492,1528,1637,14629,
	2153,3743,5202,14631,1331,5191,5208,5192,
	5199,14633,3349,5207,7836,5194,1144,5196,
	5193,5204,14634,4542,1948,3942,3088,1409,
	2077,14635,5187,5203,1471,5201,5205,8367,
	3797,5210,1593,8496,14636,5218,5220,14637,
	5224,5212,3177,5228,5215,5219,14638,14639,
	2559,5229,5227,1529,4086,5209,5223,1180,
	3782,5216,2536,3043,5206,14643,5222,14644,
	2624,5232,1487,1877,5234,5213,5233,5226,
	5214,3238,14645,5221,3560,5241,2858,5247,
	2753,5238,5251,5250,3271,5249,3266,14648,
	5253,5240,1728,5243,2407,14649,3350,5245,
	1731,5237,5239,5236,4054,1464,5242,5252,
	1427,4055,5272,2560,5271,14652,5267,14653,
	5255,5284,5254,5260,5257,5263,1999,3044,
	2790,5261,3895,3736,5264,5274,5282,5275,
	13679,5288,14657,5276,5289,3465,5295,5290,
	5293,5287,14659,5285,2472,3803,5305,5292,
	14660,1878,1315,1469,14661,2473,5297,5304,
	2341,1488,2924,14662,5294,14663,8504,5299,
	5303,1638,5301,3235,5300,5296,8505,8506,
	1470,2951,14668,5309,5307,1949,5306,14670,
	5291,5316,5315,5324,1779,14674,5320,5322,
	14675,3387,5329,5326,3933,5327,1239,14677,
	5333,1853,2253,1741,14678,5335,5334,5337,
	1623,1742,1530,14680,5339,1531,5344,5346,
	14682,5347,2221,2649,2065,13386,3554,3634,
	4074,13785,2112,4026,14683,5353,5352,2408,
	2328,2194,5356,2537,5363,5362,5364,1317,
	2952,2164,1450,5366,14684,3132,4509,1597,
	5367,5368,14148,3644,3734,13388,3231,5369,
	3450,5372,5371,2223,14687,3773,5381,5380,
	14688,2603,14689,3499,1260,3417,8510,3089,
	2379,1659,14691,1306,8511,3274,5382,2000,
	2961,8513,2849,1660,5393,1854,14693,1599,
	5391,5389,5386,3914,14694,8512,3249,5387,
	5394,1785,14695,1325,2091,5388,5390,14696,
	3717,2900,14153,3749,3576,3849,5406,2255,
	2474,5399,5405,1290,5396,5401,2712,3367,
	5398,14698,3663,14699,5400,5409,8514,5407,
	14700,3664,3326,1661,14703,5408,2873,5397,
	14704,3896,5421,14706,5420,3214,5411,1262,
	2001,5414,2353,5419,5418,5413,5415,3327,
	2524,2713,5427,5425,5422,5424,14710,1244,
	2002,4056,3540,8517,1410,2561,5426,8516,
	2475,13354,5429,8518,3181,3226,14712,3918,
	14713,5445,5439,5436,14714,3996,5442,8520,
	15414,2388,5444,5449,14715,3179,5447,13395,
	5443,14716,5450,1216,5440,2562,8522,2409,
	2078,8521,5432,5446,3124,8523,1479,2113,
	2476,5434,2380,5455,2423,1902,5464,1678,
	3145,14717,5459,5468,1139,1236,8526,5461,
	2828,5452,5454,2003,5471,8524,5466,15415,
	5465,5463,5467,2935,8525,5453,3180,4087,
	2282,3756,5484,1903,3959,2004,5473,13324,
	3897,5476,5480,3795,5475,14720,2254,5499,
	2892,2874,5502,5492,14722,5493,5469,1683,
	3500,5501,1291,2792,1533,4034,14724,3757,
	3054,5496,5498,14726,5500,5494,14727,5509,
	2716,14728,1473,5511,3016,5508,3061,5513,
	14730,5507,5506,5512,14732,5515,5516,14733,
	1534,13884,5520,5522,8530,14738,1849,2905,
	5517,3313,5529,5527,14739,5533,14740,5526,
	2044,3294,3934,5528,5532,2901,8531,7076,
	14741,5536,8532,7760,8533,5542,5541,5544,
	3244,5546,2995,2893,8534,5551,14748,1360,
	14749,3182,1411,2314,2114,14750,4045,2604,
	8536,5556,2936,5559,5554,14753,5557,3130,
	1181,8365,14756,5563,14757,5562,5567,3586,
	14759,3777,7644,14760,4035,14761,5569,5574,
	3351,5568,5571,2480,3898,5579,3183,5586,
	5585,14768,5587,1295,14769,5582,4430,5589,
	2793,2182,5591,5590,2538,4334,5577,5593,
	8544,5595,14770,3066,14772,5601,5600,2315,
	3541,2776,2256,5608,3618,3419,5609,3341,
	3017,14775,5610,1383,1671,14778,3794,1332,
	4058,1879,5612,2116,5614,5613,1880,14780,
	8545,5621,1712,5622,5624,5623,14782,3232,
	1713,5631,2922,5630,14783,4059,3352,5636,
	5633,3808,5635,5637,8547,5641,14786,5640,
	2996,3299,1881,5639,5642,1296,14788,2056,
	2224,8549,5650,5653,5652,14790,14791,1904,
	14794,1762,15423,1565,4016,5655,13802,5657,
	14802,8550,5654,8551,2183,3037,5656,14803,
	8553,15417,5659,14805,1812,3421,5660,14806,
	8556,14807,1905,1663,14808,5663,3943,8555,
	3960,14809,15418,2902,14812,5665,8557,14813,
	8559,8558,5667,3998,1743,3472,3328,14818,
	5668,5672,1950,1263,5674,2614,5671,4004,
	8561,5677,7477,5675,5678,14826,5664,5676,
	14827,5682,8563,14835,5662,14836,5683,14837,
	3501,5689,1504,5690,5692,14160,3525,5697,
	5696,5699,5698,5700,5701,5703,5702,5706,
	2652,13790,2184,1307,5707,3635,5708,8434,
	3134,3869,2005,2563,2953,4297,3018,1384,
	5709,5713,5712,5063,1412,5711,14844,1182,
	3961,5716,5714,2970,2634,5718,3956,1827,
	5719,5717,3434,5720,15419,5721,8565,1183,
	2526,5726,14161,5723,3269,5722,14845,14846,
	5729,5728,5727,3479,2756,2755,14847,1625,
	5742,3452,5736,14850,5738,5740,2564,5741,
	5739,2284,5735,5737,2258,2079,3185,5746,
	3945,5747,14851,14852,5754,14853,5752,5759,
	14854,5757,14858,5760,7725,5761,5766,14860,
	3981,14861,14862,1566,5780,14864,14865,5781,
	14866,5782,3395,3146,5785,3368,3494,8566,
	5787,3108,1413,2006,5788,8569,8567,5800,
	3340,14870,3725,14871,5801,5802,3178,2653,
	5336,5804,3789,1538,3435,5806,5810,14873,
	5812,5811,3473,1539,1885,5816,2565,3774,
	8572,5822,5823,5826,14876,2605,7877,3228,
	5827,3713,14165,5825,14880,5829,5828,14166,
	14882,2400,3982,5846,3779,5847,3836,5848,
	2956,14883,3360,2937,5849,1714,14884,2676,
	14885,1882,2117,5854,8574,14887,3152,2118,
	1640,3329,14891,8575,13342,3962,2007,5859,
	3090,5862,1429,5857,2685,5865,5873,5868,
	5874,5869,14895,3436,5867,5876,3727,5882,
	5881,5855,5885,5884,8578,14903,2260,14905,
	4017,14906,2302,5886,1805,1626,1601,2225,
	5891,2758,5888,2389,2566,5889,5887,5894,
	5916,1744,4067,2743,5895,14910,1362,3091,
	3569,1684,5897,3295,7758,1745,1363,3229,
	2354,2226,5904,14915,2355,1354,3509,5909,
	3359,2567,5906,5910,14918,5911,2667,14920,
	5915,5914,14921,2331,5917,1204,5918,1364,
	1828,5919,2008,2052,3638,5921,14923,3714,
	14924,2677,1265,1338,1136,14928,5929,1773,
	2692,2149,2976,2797,5933,5935,5932,5934,
	1784,14929,5936,1788,1665,3900,5938,1492,
	5941,5940,14934,5944,5937,5945,3953,8585,
	4549,5949,3965,5952,7176,2487,3216,5955,
	8586,2918,8588,2938,5957,4214,2971,2271,
	5961,14935,1666,5960,5973,2592,3542,14937,
	5967,5966,5968,5975,3488,14939,3386,3187,
	1746,5974,5972,14940,3401,5969,2972,3189,
	14173,3188,5982,14943,5983,5978,5976,3615,
	1365,14945,5991,3369,3763,2185,5985,5994,
	5989,5988,5984,8589,5986,2693,5996,6000,
	3619,5995,5997,3230,6006,6005,6015,6012,
	6003,4330,6010,6004,7739,6019,14951,1542,
	14952,6017,6018,6021,14954,6020,3466,6022,
	4036,3645,6027,6023,2678,6026,6028,14955,
	6029,14956,6030,6007,6031,6033,1734,1791,
	6036,2760,3306,6042,6040,6038,1501,2489,
	6043,6039,6047,6046,6045,6044,14960,6049,
	2654,14961,6050,3190,14964,6053,14965,6055,
	14966,3589,2798,2009,6062,1829,3839,2010,
	6063,6067,3826,14971,2413,6066,2303,2011,
	2228,1667,3590,6065,14972,2761,3696,2151,
	14973,2229,3065,6070,4007,2121,6071,2568,
	2490,2080,14974,2356,1906,2762,6068,6074,
	6069,1830,6077,6081,1857,6085,8592,14977,
	2012,3927,1152,6082,3191,6078,6075,1414,
	2696,1884,14978,1831,2835,6088,2800,6101,
	8594,6097,6091,6095,2342,1185,6090,6098,
	6102,2013,3810,3058,6092,8593,6100,6089,
	2940,6094,1153,3798,1747,3456,14982,3992,
	2425,6141,8595,6099,6104,6106,3093,6109,
	13322,6105,3620,1543,14983,3799,14984,6108,
	3268,6110,14986,6117,6111,6118,14987,6114,
	2294,6116,14989,6113,6119,6112,6115,6127,
	6124,6128,6122,6096,2679,6120,14990,14183,
	2723,1832,2539,2744,6131,6130,6132,7697,
	14991,6121,6129,6137,6136,7671,3752,14992,
	6139,6138,6142,2186,6140,6147,14994,6153,
	3125,6149,14995,6148,14997,6158,6165,6164,
	2129,1833,3457,6172,4918,6173,6175,3919,
	6174,6177,6176,6181,6180,6183,1800,2724,
	1627,6187,6184,6189,6186,6195,6194,15005,
	2607,6196,15006,6197,1569,15007,6200,15010,
	3723,3917,3902,4061,14099,2015,6203,2304,
	6202,6204,2261,2865,15012,6205,15013,2014,
	15016,6212,15017,2941,3593,6218,15018,2801,
	6219,4038,6223,15019,6222,6221,3020,15020,
	6224,2540,6226,4062,6227,6228,6230,6229,
	3385,4068,3389,6233,6231,1546,15021,1928,
	2230,3458,15022,1886,3697,15023,6236,6234,
	2017,1197,1187,6241,6239,6242,6237,6240,
	3668,1217,6238,6257,3315,1716,2668,4076,
	3770,6244,2680,1645,6247,3021,15028,15029,
	6251,15030,3543,6256,6255,2018,4089,2058,
	6258,3022,3570,2725,2876,6266,15031,6267,
	6263,15032,3738,3482,6271,6270,3698,6274,
	2745,15033,6276,6279,6277,3317,6280,6288,
	6283,6287,6286,6289,6292,3999,2232,2965,
	6299,1717,5039,6300,2697,6301,2295,6303,
	3630,1560,2726,2402,3555,2360,6304,2020,
	6305,3424,15038,6307,15039,2857,3371,1907,
	6306,2727,15040,3094,6310,6309,6312,2081,
	3985,6320,2541,6324,15043,1415,1142,6326,
	6329,3332,15044,2570,15045,1843,1748,1386,
	15046,1503,15047,1299,6333,4018,2877,1367,
	6343,6341,6332,2319,1764,2997,6335,6334,
	6340,6336,6339,8600,3804,6338,1368,1499,
	6342,6350,6349,6347,6346,2977,2907,6352,
	2802,1835,1251,6353,15050,2021,1326,6369,
	6359,15052,6367,6374,6368,15053,6366,15054,
	6337,1548,6362,6373,6375,15056,6361,3378,
	3147,6387,3646,15059,6378,1370,1930,3483,
	6388,8604,6385,6381,3218,7807,6383,3670,
	6389,1188,15060,3361,6397,6398,3928,8605,
	3903,15062,3556,6413,3193,6402,2804,6396,
	6404,6409,1134,15063,6401,6410,3562,2528,
	6414,6419,2805,6415,15066,6422,15067,3904,
	6412,1430,6421,3768,6417,6420,6435,15069,
	6427,2287,3758,6434,6433,6431,1218,15071,
	2818,3603,7861,6436,6440,6439,2494,2293,
	6442,3563,6438,8608,6399,4085,3194,3557,
	8609,6454,6446,15073,6458,6447,6452,6455,
	6449,2165,2571,1798,3840,15076,3845,2428,
	8611,6462,6460,3935,15075,6461,6463,6467,
	6465,3195,6466,3425,6453,6468,6473,2806,
	6474,2763,6469,6471,15077,6470,5328,5313,
	5809,6477,6479,1931,6481,4244,1679,13336,
	3970,1771,6482,2988,15081,3282,15083,1150,
	6486,3320,6492,6495,6494,2308,1836,1440,
	6500,6503,6502,3514,3438,6505,2909,6515,
	15089,6511,15090,1387,3672,6510,6508,6513,
	6516,15091,6512,6506,15092,2966,15093,6523,
	15095,6522,15096,6518,4063,6527,6525,6524,
	3920,6552,6551,6550,15104,7715,15105,6537,
	15108,8614,6557,6561,6556,15110,6565,6564,
	6567,6566,6497,2022,5412,6575,2395,15112,
	1431,2495,6577,15113,2023,6578,1189,15114,
	6584,6581,1750,6580,6587,2837,6588,15115,
	6591,15116,3459,6595,6597,1927,6585,2123,
	4030,2807,14217,6600,3871,15121,3636,6603,
	6609,2496,6607,8615,3921,6604,6606,2657,
	2628,15124,3571,3673,6613,8360,6625,6615,
	6614,6620,6624,7120,6627,1751,6626,6635,
	2658,13870,3905,3572,3324,6637,6638,1887,
	2233,3319,6640,6643,3937,6642,2572,6646,
	1549,15134,2542,6654,1908,13756,3095,6657,
	15136,1837,3196,15137,6659,8616,1799,15138,
	6658,2903,1607,15139,1859,3841,2764,6664,
	8617,2573,2990,2498,2094,2850,6667,2499,
	3504,6673,1838,4083,6672,6671,2729,1639,
	4073,1432,2500,6676,6674,2237,3293,2659,
	6678,6683,1953,8619,6677,2694,3233,8620,
	3024,2661,1550,6685,6687,6698,3025,6697,
	8621,6694,6702,6695,3096,6690,3851,2238,
	6689,15142,8623,6696,2430,3699,1276,1190,
	6703,6705,1888,6701,2024,15143,6706,3906,
	6710,3495,6716,6715,8624,15147,6720,6719,
	3546,6721,6723,6724,1630,6190,2529,8625,
	6727,3198,15150,6738,15151,6742,2501,6743,
	2045,4103,6744,6745,3505,6746,6748,6750,
	1419,3075,3547,2130,2025,3521,1375,3426,
	6758,1551,2681,6757,6762,2999,3823,6763,
	3353,2879,3461,3127,6759,3701,1388,6765,
	4046,3038,4075,2239,6764,6781,2730,15154,
	6768,2189,2240,3355,15155,1889,6767,6766,
	2026,6769,15157,6775,2819,6774,1570,6776,
	6778,8628,6780,6782,2299,6784,1456,15160,
	6785,2808,6786,6787,3549,8629,1609,15161,
	3026,6798,6796,15166,6793,6794,6797,6802,
	15167,3199,6809,6810,15169,6814,6812,6816,
	6820,6817,6815,15170,6818,2684,6823,6825,
	6822,6833,6832,6835,6834,6837,15174,6836,
	6838,6840,2574,6842,15175,1765,6843,6845,
	6847,6844,2306,6849,1610,1801,15177,1890,
	3128,6851,2272,15179,6853,6852,1840,6855,
	2124,6856,6859,6861,3637,6858,6864,6862,
	1611,3344,4000,3852,6869,6872,3883,1483,
	6870,6875,6873,3116,6876,2575,6882,2265,
	6883,15182,4278,4277,6885,6143,6886,2914,
	2545,3318,3621,3056,2064,3750,2589,15185,
	15187,6889,1376,3278,6891,3373,6892,2396,
	15188,3790,6909,2880,2809,15189,3200,6895,
	1647,6901,6914,3201,2974,6898,3098,3149,
	6899,6903,2598,3357,3048,2662,6902,2830,
	2820,6900,1133,4040,2367,2576,6907,6906,
	1203,6908,3251,2609,15191,2967,6915,3873,
	1249,3623,1377,3219,2913,1191,6917,6918,
	7476,1301,2766,8634,6920,3110,2810,2307,
	6921,2415,6922,2733,2732,3987,6926,3462,
	6928,6927,6925,5943,1552,6893,15192,6930,
	6929,15196,3257,15197,6935,15199,8635,1802,
	6936,15200,3875,6944,15203,6943,15206,3243,
	6945,2368,2316,3345,2991,15208,2595,6948,
	15209,6949,6950,3929,6952,6951,2053,2190,
	2889,15210,6959,6957,7707,7777,15215,6961,
	6960,3428,2115,2317,6965,6966,3948,2383,
	3834,3988,6967,1754,6968,8640,3101,8639,
	6971,1494,2577,8641,6969,3715,1780,15219,
	6972,8646,15220,3255,1441,8647,6980,6977,
	6979,15221,7050,15222,4019,1934,15223,8655,
	8649,6988,8650,15224,3118,6983,15225,6986,
	6984,8652,15226,6990,8654,8653,6995,1302,
	2504,8656,6987,6992,3028,6994,6991,6996,
	8663,3677,8667,15231,8666,6999,3631,3512,
	2992,1682,8368,7001,8670,2031,7000,2611,
	2612,7004,7006,8674,8672,15234,2531,8669,
	7005,7007,8668,1739,15235,3511,2318,4041,
	7003,2152,7008,7010,15236,8361,13400,3265,
	7015,8677,2945,7011,1795,15240,7014,2095,
	2811,3046,15241,7020,15242,7028,3111,7027,
	8680,1720,15246,7030,7024,15249,7036,7033,
	7032,2506,3202,7031,15250,7039,7037,7044,
	8683,1553,7043,3846,7055,7045,7049,15253,
	7056,7054,7057,7059,3029,15255,7060,2736,
	3604,7062,1417,1555,8685,1554,7063,1556,
	1459,2032,3402,15256,7069,7068,15257,13653,
	1278,7072,7071,15260,7074,7079,7078,7081,
	7080,7082,7084,7086,3550,15262,1128,2859,
	15263,7091,7094,7092,2033,7093,1910,3605,
	7096,7098,7097,7099,1219,2590,2438,1557,
	7100,7101,3040,3989,3203,7095,3950,15265,
	1777,3964,2882,6252,7103,1418,2613,1460,
	7105,7106,7104,1850,2125,2507,8688,4001,
	15266,7108,6923,15267,7107,8689,7111,7113,
	4020,2669,3407,2627,1571,3876,1389,2370,
	1935,7117,2241,7119,7116,2166,15268,7123,
	6546,2621,7121,3949,3273,1229,15271,2695,
	2274,3591,15272,1250,4021,3925,7126,2578,
	7127,15274,4022,7122,7129,7131,4048,8690,
	5140,7137,7143,8692,7140,7142,7144,8695,
	2664,8696,3843,2665,15275,7146,7430,3800,
	15276,7147,7149,1461,2591,15277,15278,1786,
	7153,7157,1489,7154,7158,15280,7165,7173,
	3289,7175,1339,15285,7178,7177,3607,3030,
	2066,2034,2417,15287,7179,3884,1572,3430,
	3252,2626,3990,15289,7182,15290,3705,3204,
	7184,7181,14259,3523,3924,7185,15291,7188,
	2890,1573,1894,8697,1574,15294,7752,3129,
	7189,15295,7194,7193,7200,7199,3464,6201,
	2543,13848,4289,5338,3431,2242,3678,2534,
	7205,3910,7206,1390,7207,7213,7209,8701,
	1559,7222,7225,15304,2968,3267,2860,1274,
	1766,1767,15310,2993,7237,1768,7244,15311,
	1613,7245,2813,1895,15314,2861,15317,7250,
	15318,7252,7254,7257,8704,7256,15320,7259,
	1723,7258,7263,7262,7264,7266,7265,2036,
	8705,3397,7281,3480,7282,7280,7290,15325,
	5332,7300,6057,1614,1407,2082,7302,7301,
	3761,8708,15328,7308,8707,1154,8709,7311,
	3579,7309,15330,7316,3740,2171,2154,2737,
	7318,15331,7321,15332,1957,7319,2168,15333,
	7329,7326,7325,7327,15334,1845,7333,7332,
	7331,7340,7336,7342,15339,7338,1472,7341,
	7337,7335,7334,7347,7346,3515,7344,1207,
	7349,1485,7330,1241,7352,7350,2923,3742,
	15344,7357,15347,7358,7364,7360,3679,3792,
	15348,7365,7366,3222,7363,7362,7367,2270,
	1321,7377,7376,7375,15349,7378,15350,7379,
	7384,7383,2054,3781,15351,7382,8714,3680,
	7390,8713,15356,7395,7400,7398,7397,7403,
	15357,7396,7401,15358,15359,7646,7406,2891,
	2141,15364,7413,1896,7415,7677,7416,7423,
	7421,4066,15365,7425,3380,15366,7426,7428,
	15367,7429,7682,7797,7427,2047,3801,3729,
	4740,5375,3753,13323,1323,7431,1642,8717,
	2055,5645,3815,2883,7436,7438,7437,15368,
	7441,15369,7442,15371,1937,2767,7454,7457,
	5898,6779,7174,4024,7460,15381,7469,15382,
	7470,7468,15384,7471,7472,5927,8443,8481,
	8497,8499,8542,8548,8571,8583,8587,8590,
	8599,8610,8622,8630,8636,8664,8671,8687,
	8702,8715,642,8007,716,712,715,717,
	8006,718,692,636,693,637,663,699,
	697,700,641,719,678,664,679,648,
	650,646,680,667,681,665,751,649,
	8005,711,
};

pdf_cmap pdf_cmap_UniJIS_UCS2_H =
{
	-1, "UniJIS-UCS2-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 2, 0xe000, 0xffff },
	},
	8533, 8533, (pdf_range*) pdf_cmap_UniJIS_UCS2_H_ranges,
	5354, 5354, (unsigned short*) pdf_cmap_UniJIS_UCS2_H_table,
};

/* UniJIS-UCS2-HW-H */

static const pdf_range pdf_cmap_UniJIS_UCS2_HW_H_ranges[] =
{
	{ 0x0020, 0x005b, PDF_CMAP_RANGE, 231 },
	{ 0x005c, 0x005c, PDF_CMAP_SINGLE, 8719 },
	{ 0x005d, 0x007e, PDF_CMAP_RANGE, 292 },
	{ 0x00a5, 0x00a5, PDF_CMAP_SINGLE, 291 },
};

static const unsigned short pdf_cmap_UniJIS_UCS2_HW_H_table[1] = { 0 };

pdf_cmap pdf_cmap_UniJIS_UCS2_HW_H =
{
	-1, "UniJIS-UCS2-HW-H", "UniJIS-UCS2-H", nil, 0,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	4, 4, (pdf_range*) pdf_cmap_UniJIS_UCS2_HW_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_UniJIS_UCS2_HW_H_table,
};

/* UniJIS-UCS2-HW-V */

static const pdf_range pdf_cmap_UniJIS_UCS2_HW_V_ranges[] =
{
	{ 0x0020, 0x005b, PDF_CMAP_RANGE, 231 },
	{ 0x005c, 0x005c, PDF_CMAP_SINGLE, 8719 },
	{ 0x005d, 0x007e, PDF_CMAP_RANGE, 292 },
	{ 0x00a5, 0x00a5, PDF_CMAP_SINGLE, 291 },
	{ 0x00b0, 0x00b0, PDF_CMAP_SINGLE, 8269 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 7893 },
	{ 0x2015, 0x2016, PDF_CMAP_TABLE, 0 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 8281 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 8279 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 2 },
	{ 0x2032, 0x2033, PDF_CMAP_TABLE, 4 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 6 },
	{ 0x2195, 0x2195, PDF_CMAP_SINGLE, 12214 },
	{ 0x21c4, 0x21c6, PDF_CMAP_TABLE, 10 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 13 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 7895 },
	{ 0x223c, 0x223c, PDF_CMAP_SINGLE, 7894 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 7897 },
	{ 0x2500, 0x2501, PDF_CMAP_RANGE, 7481 },
	{ 0x2502, 0x2503, PDF_CMAP_RANGE, 7479 },
	{ 0x2504, 0x2505, PDF_CMAP_RANGE, 7485 },
	{ 0x2506, 0x2507, PDF_CMAP_RANGE, 7483 },
	{ 0x2508, 0x2509, PDF_CMAP_RANGE, 7489 },
	{ 0x250a, 0x250b, PDF_CMAP_RANGE, 7487 },
	{ 0x250c, 0x2522, PDF_CMAP_TABLE, 17 },
	{ 0x2523, 0x2524, PDF_CMAP_RANGE, 7530 },
	{ 0x2525, 0x252c, PDF_CMAP_TABLE, 40 },
	{ 0x252d, 0x252f, PDF_CMAP_RANGE, 7517 },
	{ 0x2530, 0x2530, PDF_CMAP_SINGLE, 7516 },
	{ 0x2531, 0x2533, PDF_CMAP_RANGE, 7520 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 7507 },
	{ 0x2535, 0x2537, PDF_CMAP_RANGE, 7509 },
	{ 0x2538, 0x2538, PDF_CMAP_SINGLE, 7508 },
	{ 0x2539, 0x253b, PDF_CMAP_RANGE, 7512 },
	{ 0x253d, 0x253f, PDF_CMAP_RANGE, 7543 },
	{ 0x2540, 0x2548, PDF_CMAP_TABLE, 48 },
	{ 0x2549, 0x254a, PDF_CMAP_RANGE, 7550 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 57 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12178 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8209 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 7887 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 7907 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 8270 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 7901 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 61 },
	{ 0x301f, 0x301f, PDF_CMAP_SINGLE, 7957 },
	{ 0x3041, 0x3041, PDF_CMAP_SINGLE, 7918 },
	{ 0x3043, 0x3043, PDF_CMAP_SINGLE, 7919 },
	{ 0x3045, 0x3045, PDF_CMAP_SINGLE, 7920 },
	{ 0x3047, 0x3047, PDF_CMAP_SINGLE, 7921 },
	{ 0x3049, 0x3049, PDF_CMAP_SINGLE, 7922 },
	{ 0x3063, 0x3063, PDF_CMAP_SINGLE, 7923 },
	{ 0x3083, 0x3083, PDF_CMAP_SINGLE, 7924 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309c, PDF_CMAP_TABLE, 63 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 65 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 76 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 82 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 88 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 90 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 93 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 101 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 103 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 107 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 109 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 113 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309b, PDF_CMAP_SINGLE, 8272 },
	{ 0x309c, 0x309c, PDF_CMAP_SINGLE, 8271 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x3303, PDF_CMAP_SINGLE, 8338 },
	{ 0x3304, 0x3304, PDF_CMAP_SINGLE, 11960 },
	{ 0x3305, 0x3305, PDF_CMAP_SINGLE, 8333 },
	{ 0x3306, 0x3306, PDF_CMAP_SINGLE, 11961 },
	{ 0x3307, 0x3307, PDF_CMAP_SINGLE, 11965 },
	{ 0x3308, 0x3308, PDF_CMAP_SINGLE, 11963 },
	{ 0x3309, 0x3309, PDF_CMAP_SINGLE, 11968 },
	{ 0x330a, 0x330a, PDF_CMAP_SINGLE, 11966 },
	{ 0x330b, 0x330b, PDF_CMAP_SINGLE, 11970 },
	{ 0x330c, 0x330c, PDF_CMAP_SINGLE, 11972 },
	{ 0x330d, 0x330d, PDF_CMAP_SINGLE, 7950 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3314, PDF_CMAP_SINGLE, 7941 },
	{ 0x3315, 0x3315, PDF_CMAP_SINGLE, 8340 },
	{ 0x3316, 0x3316, PDF_CMAP_SINGLE, 8330 },
	{ 0x3317, 0x3317, PDF_CMAP_SINGLE, 11980 },
	{ 0x3318, 0x3318, PDF_CMAP_SINGLE, 8339 },
	{ 0x3319, 0x3319, PDF_CMAP_SINGLE, 11982 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3322, PDF_CMAP_SINGLE, 8329 },
	{ 0x3323, 0x3323, PDF_CMAP_SINGLE, 8348 },
	{ 0x3324, 0x3324, PDF_CMAP_SINGLE, 11991 },
	{ 0x3325, 0x3325, PDF_CMAP_SINGLE, 11993 },
	{ 0x3326, 0x3326, PDF_CMAP_SINGLE, 7951 },
	{ 0x3327, 0x3327, PDF_CMAP_SINGLE, 7945 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332a, PDF_CMAP_SINGLE, 8356 },
	{ 0x332b, 0x332b, PDF_CMAP_SINGLE, 7953 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3331, PDF_CMAP_SINGLE, 8358 },
	{ 0x3332, 0x3332, PDF_CMAP_SINGLE, 12005 },
	{ 0x3333, 0x3333, PDF_CMAP_SINGLE, 8334 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x3336, PDF_CMAP_SINGLE, 7947 },
	{ 0x3337, 0x3337, PDF_CMAP_SINGLE, 12014 },
	{ 0x3338, 0x3338, PDF_CMAP_SINGLE, 12016 },
	{ 0x3339, 0x3339, PDF_CMAP_SINGLE, 8343 },
	{ 0x333a, 0x333a, PDF_CMAP_SINGLE, 12017 },
	{ 0x333b, 0x333b, PDF_CMAP_SINGLE, 8349 },
	{ 0x333c, 0x333c, PDF_CMAP_SINGLE, 12010 },
	{ 0x333d, 0x333d, PDF_CMAP_SINGLE, 12018 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3341, PDF_CMAP_SINGLE, 12019 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 8347 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x3347, PDF_CMAP_SINGLE, 8357 },
	{ 0x3348, 0x3348, PDF_CMAP_SINGLE, 12027 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 7940 },
	{ 0x334a, 0x334a, PDF_CMAP_SINGLE, 7954 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334d, PDF_CMAP_SINGLE, 7943 },
	{ 0x334e, 0x334e, PDF_CMAP_SINGLE, 8337 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3351, PDF_CMAP_SINGLE, 7948 },
	{ 0x3352, 0x3352, PDF_CMAP_SINGLE, 12034 },
	{ 0x3353, 0x3353, PDF_CMAP_SINGLE, 12038 },
	{ 0x3354, 0x3354, PDF_CMAP_SINGLE, 12035 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 7905 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 7896 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 7906 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7894 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
};

static const unsigned short pdf_cmap_UniJIS_UCS2_HW_V_table[117] =
{
	7892,7895,7898,7897,8273,8283,738,736,
	739,737,8311,8310,8312,8012,8014,8011,
	8013,7495,7497,7496,7498,7503,7505,7504,
	7506,7491,7493,7492,7494,7499,7501,7500,
	7502,7523,7527,7525,7524,7526,7529,7528,
	7535,7533,7532,7534,7537,7536,7538,7515,
	7541,7540,7542,7547,7549,7546,7548,7553,
	7552,8221,8219,8222,8220,7894,7956,8272,
	8271,8338,11960,8333,11961,11965,11963,11968,
	11966,11970,11972,7950,7941,8340,8330,11980,
	8339,11982,8329,8348,11991,11993,7951,7945,
	8356,7953,8358,12005,8334,7947,12014,12016,
	8343,12017,8349,12010,12018,12019,8347,8357,
	12027,7940,7954,7943,8337,7948,12034,12038,
	12035,7905,7896,7906,7894,
};

pdf_cmap pdf_cmap_UniJIS_UCS2_HW_V =
{
	-1, "UniJIS-UCS2-HW-V", "UniJIS-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	199, 199, (pdf_range*) pdf_cmap_UniJIS_UCS2_HW_V_ranges,
	117, 117, (unsigned short*) pdf_cmap_UniJIS_UCS2_HW_V_table,
};

/* UniJIS-UCS2-V */

static const pdf_range pdf_cmap_UniJIS_UCS2_V_ranges[] =
{
	{ 0x00b0, 0x00b0, PDF_CMAP_SINGLE, 8269 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 7893 },
	{ 0x2015, 0x2016, PDF_CMAP_TABLE, 0 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 8281 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 8279 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 2 },
	{ 0x2032, 0x2033, PDF_CMAP_TABLE, 4 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 6 },
	{ 0x2195, 0x2195, PDF_CMAP_SINGLE, 12214 },
	{ 0x21c4, 0x21c6, PDF_CMAP_TABLE, 10 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 13 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 7895 },
	{ 0x223c, 0x223c, PDF_CMAP_SINGLE, 7894 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 7897 },
	{ 0x2500, 0x2501, PDF_CMAP_RANGE, 7481 },
	{ 0x2502, 0x2503, PDF_CMAP_RANGE, 7479 },
	{ 0x2504, 0x2505, PDF_CMAP_RANGE, 7485 },
	{ 0x2506, 0x2507, PDF_CMAP_RANGE, 7483 },
	{ 0x2508, 0x2509, PDF_CMAP_RANGE, 7489 },
	{ 0x250a, 0x250b, PDF_CMAP_RANGE, 7487 },
	{ 0x250c, 0x2522, PDF_CMAP_TABLE, 17 },
	{ 0x2523, 0x2524, PDF_CMAP_RANGE, 7530 },
	{ 0x2525, 0x252c, PDF_CMAP_TABLE, 40 },
	{ 0x252d, 0x252f, PDF_CMAP_RANGE, 7517 },
	{ 0x2530, 0x2530, PDF_CMAP_SINGLE, 7516 },
	{ 0x2531, 0x2533, PDF_CMAP_RANGE, 7520 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 7507 },
	{ 0x2535, 0x2537, PDF_CMAP_RANGE, 7509 },
	{ 0x2538, 0x2538, PDF_CMAP_SINGLE, 7508 },
	{ 0x2539, 0x253b, PDF_CMAP_RANGE, 7512 },
	{ 0x253d, 0x253f, PDF_CMAP_RANGE, 7543 },
	{ 0x2540, 0x2548, PDF_CMAP_TABLE, 48 },
	{ 0x2549, 0x254a, PDF_CMAP_RANGE, 7550 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 57 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12178 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8209 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 7887 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 7907 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 8270 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 7901 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 61 },
	{ 0x301f, 0x301f, PDF_CMAP_SINGLE, 7957 },
	{ 0x3041, 0x3041, PDF_CMAP_SINGLE, 7918 },
	{ 0x3043, 0x3043, PDF_CMAP_SINGLE, 7919 },
	{ 0x3045, 0x3045, PDF_CMAP_SINGLE, 7920 },
	{ 0x3047, 0x3047, PDF_CMAP_SINGLE, 7921 },
	{ 0x3049, 0x3049, PDF_CMAP_SINGLE, 7922 },
	{ 0x3063, 0x3063, PDF_CMAP_SINGLE, 7923 },
	{ 0x3083, 0x3083, PDF_CMAP_SINGLE, 7924 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309c, PDF_CMAP_TABLE, 63 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 65 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 76 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 82 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 88 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 90 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 93 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 101 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 103 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 107 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 109 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 113 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309b, PDF_CMAP_SINGLE, 8272 },
	{ 0x309c, 0x309c, PDF_CMAP_SINGLE, 8271 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x3303, PDF_CMAP_SINGLE, 8338 },
	{ 0x3304, 0x3304, PDF_CMAP_SINGLE, 11960 },
	{ 0x3305, 0x3305, PDF_CMAP_SINGLE, 8333 },
	{ 0x3306, 0x3306, PDF_CMAP_SINGLE, 11961 },
	{ 0x3307, 0x3307, PDF_CMAP_SINGLE, 11965 },
	{ 0x3308, 0x3308, PDF_CMAP_SINGLE, 11963 },
	{ 0x3309, 0x3309, PDF_CMAP_SINGLE, 11968 },
	{ 0x330a, 0x330a, PDF_CMAP_SINGLE, 11966 },
	{ 0x330b, 0x330b, PDF_CMAP_SINGLE, 11970 },
	{ 0x330c, 0x330c, PDF_CMAP_SINGLE, 11972 },
	{ 0x330d, 0x330d, PDF_CMAP_SINGLE, 7950 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3314, PDF_CMAP_SINGLE, 7941 },
	{ 0x3315, 0x3315, PDF_CMAP_SINGLE, 8340 },
	{ 0x3316, 0x3316, PDF_CMAP_SINGLE, 8330 },
	{ 0x3317, 0x3317, PDF_CMAP_SINGLE, 11980 },
	{ 0x3318, 0x3318, PDF_CMAP_SINGLE, 8339 },
	{ 0x3319, 0x3319, PDF_CMAP_SINGLE, 11982 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3322, PDF_CMAP_SINGLE, 8329 },
	{ 0x3323, 0x3323, PDF_CMAP_SINGLE, 8348 },
	{ 0x3324, 0x3324, PDF_CMAP_SINGLE, 11991 },
	{ 0x3325, 0x3325, PDF_CMAP_SINGLE, 11993 },
	{ 0x3326, 0x3326, PDF_CMAP_SINGLE, 7951 },
	{ 0x3327, 0x3327, PDF_CMAP_SINGLE, 7945 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332a, PDF_CMAP_SINGLE, 8356 },
	{ 0x332b, 0x332b, PDF_CMAP_SINGLE, 7953 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3331, PDF_CMAP_SINGLE, 8358 },
	{ 0x3332, 0x3332, PDF_CMAP_SINGLE, 12005 },
	{ 0x3333, 0x3333, PDF_CMAP_SINGLE, 8334 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x3336, PDF_CMAP_SINGLE, 7947 },
	{ 0x3337, 0x3337, PDF_CMAP_SINGLE, 12014 },
	{ 0x3338, 0x3338, PDF_CMAP_SINGLE, 12016 },
	{ 0x3339, 0x3339, PDF_CMAP_SINGLE, 8343 },
	{ 0x333a, 0x333a, PDF_CMAP_SINGLE, 12017 },
	{ 0x333b, 0x333b, PDF_CMAP_SINGLE, 8349 },
	{ 0x333c, 0x333c, PDF_CMAP_SINGLE, 12010 },
	{ 0x333d, 0x333d, PDF_CMAP_SINGLE, 12018 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3341, PDF_CMAP_SINGLE, 12019 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 8347 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x3347, PDF_CMAP_SINGLE, 8357 },
	{ 0x3348, 0x3348, PDF_CMAP_SINGLE, 12027 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 7940 },
	{ 0x334a, 0x334a, PDF_CMAP_SINGLE, 7954 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334d, PDF_CMAP_SINGLE, 7943 },
	{ 0x334e, 0x334e, PDF_CMAP_SINGLE, 8337 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3351, PDF_CMAP_SINGLE, 7948 },
	{ 0x3352, 0x3352, PDF_CMAP_SINGLE, 12034 },
	{ 0x3353, 0x3353, PDF_CMAP_SINGLE, 12038 },
	{ 0x3354, 0x3354, PDF_CMAP_SINGLE, 12035 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 7905 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 7896 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 7906 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7894 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
};

static const unsigned short pdf_cmap_UniJIS_UCS2_V_table[117] =
{
	7892,7895,7898,7897,8273,8283,738,736,
	739,737,8311,8310,8312,8012,8014,8011,
	8013,7495,7497,7496,7498,7503,7505,7504,
	7506,7491,7493,7492,7494,7499,7501,7500,
	7502,7523,7527,7525,7524,7526,7529,7528,
	7535,7533,7532,7534,7537,7536,7538,7515,
	7541,7540,7542,7547,7549,7546,7548,7553,
	7552,8221,8219,8222,8220,7894,7956,8272,
	8271,8338,11960,8333,11961,11965,11963,11968,
	11966,11970,11972,7950,7941,8340,8330,11980,
	8339,11982,8329,8348,11991,11993,7951,7945,
	8356,7953,8358,12005,8334,7947,12014,12016,
	8343,12017,8349,12010,12018,12019,8347,8357,
	12027,7940,7954,7943,8337,7948,12034,12038,
	12035,7905,7896,7906,7894,
};

pdf_cmap pdf_cmap_UniJIS_UCS2_V =
{
	-1, "UniJIS-UCS2-V", "UniJIS-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	195, 195, (pdf_range*) pdf_cmap_UniJIS_UCS2_V_ranges,
	117, 117, (unsigned short*) pdf_cmap_UniJIS_UCS2_V_table,
};

/* UniJISPro-UCS2-HW-V */

static const pdf_range pdf_cmap_UniJISPro_UCS2_HW_V_ranges[] =
{
	{ 0x0020, 0x005b, PDF_CMAP_RANGE, 231 },
	{ 0x005c, 0x005c, PDF_CMAP_SINGLE, 8719 },
	{ 0x005d, 0x007e, PDF_CMAP_RANGE, 292 },
	{ 0x00a5, 0x00a5, PDF_CMAP_SINGLE, 291 },
	{ 0x00b0, 0x00b0, PDF_CMAP_SINGLE, 8269 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 7893 },
	{ 0x2015, 0x2016, PDF_CMAP_TABLE, 0 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 12173 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 7956 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 2 },
	{ 0x2032, 0x2033, PDF_CMAP_TABLE, 4 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 6 },
	{ 0x2195, 0x2195, PDF_CMAP_SINGLE, 12214 },
	{ 0x21c4, 0x21c6, PDF_CMAP_TABLE, 10 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 13 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 7895 },
	{ 0x223c, 0x223c, PDF_CMAP_SINGLE, 7894 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 7897 },
	{ 0x2500, 0x2501, PDF_CMAP_RANGE, 7481 },
	{ 0x2502, 0x2503, PDF_CMAP_RANGE, 7479 },
	{ 0x2504, 0x2505, PDF_CMAP_RANGE, 7485 },
	{ 0x2506, 0x2507, PDF_CMAP_RANGE, 7483 },
	{ 0x2508, 0x2509, PDF_CMAP_RANGE, 7489 },
	{ 0x250a, 0x250b, PDF_CMAP_RANGE, 7487 },
	{ 0x250c, 0x2522, PDF_CMAP_TABLE, 17 },
	{ 0x2523, 0x2524, PDF_CMAP_RANGE, 7530 },
	{ 0x2525, 0x252c, PDF_CMAP_TABLE, 40 },
	{ 0x252d, 0x252f, PDF_CMAP_RANGE, 7517 },
	{ 0x2530, 0x2530, PDF_CMAP_SINGLE, 7516 },
	{ 0x2531, 0x2533, PDF_CMAP_RANGE, 7520 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 7507 },
	{ 0x2535, 0x2537, PDF_CMAP_RANGE, 7509 },
	{ 0x2538, 0x2538, PDF_CMAP_SINGLE, 7508 },
	{ 0x2539, 0x253b, PDF_CMAP_RANGE, 7512 },
	{ 0x253d, 0x253f, PDF_CMAP_RANGE, 7543 },
	{ 0x2540, 0x2548, PDF_CMAP_TABLE, 48 },
	{ 0x2549, 0x254a, PDF_CMAP_RANGE, 7550 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 57 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12178 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8209 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 7887 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 7907 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 8270 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 7901 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 61 },
	{ 0x301f, 0x301f, PDF_CMAP_SINGLE, 7957 },
	{ 0x3041, 0x3041, PDF_CMAP_SINGLE, 7918 },
	{ 0x3043, 0x3043, PDF_CMAP_SINGLE, 7919 },
	{ 0x3045, 0x3045, PDF_CMAP_SINGLE, 7920 },
	{ 0x3047, 0x3047, PDF_CMAP_SINGLE, 7921 },
	{ 0x3049, 0x3049, PDF_CMAP_SINGLE, 7922 },
	{ 0x3063, 0x3063, PDF_CMAP_SINGLE, 7923 },
	{ 0x3083, 0x3083, PDF_CMAP_SINGLE, 7924 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309c, PDF_CMAP_TABLE, 63 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 65 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 76 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 82 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 88 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 90 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 93 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 101 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 103 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 107 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 109 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337b, 0x337f, PDF_CMAP_TABLE, 113 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1a, 0xff1a, PDF_CMAP_SINGLE, 12101 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 118 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309b, PDF_CMAP_SINGLE, 8272 },
	{ 0x309c, 0x309c, PDF_CMAP_SINGLE, 8271 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x3303, PDF_CMAP_SINGLE, 8338 },
	{ 0x3304, 0x3304, PDF_CMAP_SINGLE, 11960 },
	{ 0x3305, 0x3305, PDF_CMAP_SINGLE, 8333 },
	{ 0x3306, 0x3306, PDF_CMAP_SINGLE, 11961 },
	{ 0x3307, 0x3307, PDF_CMAP_SINGLE, 11965 },
	{ 0x3308, 0x3308, PDF_CMAP_SINGLE, 11963 },
	{ 0x3309, 0x3309, PDF_CMAP_SINGLE, 11968 },
	{ 0x330a, 0x330a, PDF_CMAP_SINGLE, 11966 },
	{ 0x330b, 0x330b, PDF_CMAP_SINGLE, 11970 },
	{ 0x330c, 0x330c, PDF_CMAP_SINGLE, 11972 },
	{ 0x330d, 0x330d, PDF_CMAP_SINGLE, 7950 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3314, PDF_CMAP_SINGLE, 7941 },
	{ 0x3315, 0x3315, PDF_CMAP_SINGLE, 8340 },
	{ 0x3316, 0x3316, PDF_CMAP_SINGLE, 8330 },
	{ 0x3317, 0x3317, PDF_CMAP_SINGLE, 11980 },
	{ 0x3318, 0x3318, PDF_CMAP_SINGLE, 8339 },
	{ 0x3319, 0x3319, PDF_CMAP_SINGLE, 11982 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3322, PDF_CMAP_SINGLE, 8329 },
	{ 0x3323, 0x3323, PDF_CMAP_SINGLE, 8348 },
	{ 0x3324, 0x3324, PDF_CMAP_SINGLE, 11991 },
	{ 0x3325, 0x3325, PDF_CMAP_SINGLE, 11993 },
	{ 0x3326, 0x3326, PDF_CMAP_SINGLE, 7951 },
	{ 0x3327, 0x3327, PDF_CMAP_SINGLE, 7945 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332a, PDF_CMAP_SINGLE, 8356 },
	{ 0x332b, 0x332b, PDF_CMAP_SINGLE, 7953 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3331, PDF_CMAP_SINGLE, 8358 },
	{ 0x3332, 0x3332, PDF_CMAP_SINGLE, 12005 },
	{ 0x3333, 0x3333, PDF_CMAP_SINGLE, 8334 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x3336, PDF_CMAP_SINGLE, 7947 },
	{ 0x3337, 0x3337, PDF_CMAP_SINGLE, 12014 },
	{ 0x3338, 0x3338, PDF_CMAP_SINGLE, 12016 },
	{ 0x3339, 0x3339, PDF_CMAP_SINGLE, 8343 },
	{ 0x333a, 0x333a, PDF_CMAP_SINGLE, 12017 },
	{ 0x333b, 0x333b, PDF_CMAP_SINGLE, 8349 },
	{ 0x333c, 0x333c, PDF_CMAP_SINGLE, 12010 },
	{ 0x333d, 0x333d, PDF_CMAP_SINGLE, 12018 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3341, PDF_CMAP_SINGLE, 12019 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 8347 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x3347, PDF_CMAP_SINGLE, 8357 },
	{ 0x3348, 0x3348, PDF_CMAP_SINGLE, 12027 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 7940 },
	{ 0x334a, 0x334a, PDF_CMAP_SINGLE, 7954 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334d, PDF_CMAP_SINGLE, 7943 },
	{ 0x334e, 0x334e, PDF_CMAP_SINGLE, 8337 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3351, PDF_CMAP_SINGLE, 7948 },
	{ 0x3352, 0x3352, PDF_CMAP_SINGLE, 12034 },
	{ 0x3353, 0x3353, PDF_CMAP_SINGLE, 12038 },
	{ 0x3354, 0x3354, PDF_CMAP_SINGLE, 12035 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337b, 0x337b, PDF_CMAP_SINGLE, 12044 },
	{ 0x337c, 0x337c, PDF_CMAP_SINGLE, 12043 },
	{ 0x337d, 0x337d, PDF_CMAP_SINGLE, 12042 },
	{ 0x337e, 0x337e, PDF_CMAP_SINGLE, 12041 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1a, 0xff1a, PDF_CMAP_SINGLE, 12101 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 7905 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 7896 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 7906 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7894 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
};

static const unsigned short pdf_cmap_UniJISPro_UCS2_HW_V_table[122] =
{
	7892,7895,7898,7897,8273,8283,738,736,
	739,737,8311,8310,8312,8012,8014,8011,
	8013,7495,7497,7496,7498,7503,7505,7504,
	7506,7491,7493,7492,7494,7499,7501,7500,
	7502,7523,7527,7525,7524,7526,7529,7528,
	7535,7533,7532,7534,7537,7536,7538,7515,
	7541,7540,7542,7547,7549,7546,7548,7553,
	7552,8221,8219,8222,8220,7894,7956,8272,
	8271,8338,11960,8333,11961,11965,11963,11968,
	11966,11970,11972,7950,7941,8340,8330,11980,
	8339,11982,8329,8348,11991,11993,7951,7945,
	8356,7953,8358,12005,8334,7947,12014,12016,
	8343,12017,8349,12010,12018,12019,8347,8357,
	12027,7940,7954,7943,8337,7948,12034,12038,
	12035,12044,12043,12042,12041,8324,7905,7896,
	7906,7894,
};

pdf_cmap pdf_cmap_UniJISPro_UCS2_HW_V =
{
	-1, "UniJISPro-UCS2-HW-V", "UniJIS-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	204, 204, (pdf_range*) pdf_cmap_UniJISPro_UCS2_HW_V_ranges,
	122, 122, (unsigned short*) pdf_cmap_UniJISPro_UCS2_HW_V_table,
};

/* UniJISPro-UCS2-V */

static const pdf_range pdf_cmap_UniJISPro_UCS2_V_ranges[] =
{
	{ 0x00b0, 0x00b0, PDF_CMAP_SINGLE, 8269 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 7893 },
	{ 0x2015, 0x2016, PDF_CMAP_TABLE, 0 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 12173 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 7956 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 2 },
	{ 0x2032, 0x2033, PDF_CMAP_TABLE, 4 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 6 },
	{ 0x2195, 0x2195, PDF_CMAP_SINGLE, 12214 },
	{ 0x21c4, 0x21c6, PDF_CMAP_TABLE, 10 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 13 },
	{ 0x2225, 0x2225, PDF_CMAP_SINGLE, 7895 },
	{ 0x223c, 0x223c, PDF_CMAP_SINGLE, 7894 },
	{ 0x22ef, 0x22ef, PDF_CMAP_SINGLE, 7897 },
	{ 0x2500, 0x2501, PDF_CMAP_RANGE, 7481 },
	{ 0x2502, 0x2503, PDF_CMAP_RANGE, 7479 },
	{ 0x2504, 0x2505, PDF_CMAP_RANGE, 7485 },
	{ 0x2506, 0x2507, PDF_CMAP_RANGE, 7483 },
	{ 0x2508, 0x2509, PDF_CMAP_RANGE, 7489 },
	{ 0x250a, 0x250b, PDF_CMAP_RANGE, 7487 },
	{ 0x250c, 0x2522, PDF_CMAP_TABLE, 17 },
	{ 0x2523, 0x2524, PDF_CMAP_RANGE, 7530 },
	{ 0x2525, 0x252c, PDF_CMAP_TABLE, 40 },
	{ 0x252d, 0x252f, PDF_CMAP_RANGE, 7517 },
	{ 0x2530, 0x2530, PDF_CMAP_SINGLE, 7516 },
	{ 0x2531, 0x2533, PDF_CMAP_RANGE, 7520 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 7507 },
	{ 0x2535, 0x2537, PDF_CMAP_RANGE, 7509 },
	{ 0x2538, 0x2538, PDF_CMAP_SINGLE, 7508 },
	{ 0x2539, 0x253b, PDF_CMAP_RANGE, 7512 },
	{ 0x253d, 0x253f, PDF_CMAP_RANGE, 7543 },
	{ 0x2540, 0x2548, PDF_CMAP_TABLE, 48 },
	{ 0x2549, 0x254a, PDF_CMAP_RANGE, 7550 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 57 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12178 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8209 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 7887 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 7907 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 8270 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 7901 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 61 },
	{ 0x301f, 0x301f, PDF_CMAP_SINGLE, 7957 },
	{ 0x3041, 0x3041, PDF_CMAP_SINGLE, 7918 },
	{ 0x3043, 0x3043, PDF_CMAP_SINGLE, 7919 },
	{ 0x3045, 0x3045, PDF_CMAP_SINGLE, 7920 },
	{ 0x3047, 0x3047, PDF_CMAP_SINGLE, 7921 },
	{ 0x3049, 0x3049, PDF_CMAP_SINGLE, 7922 },
	{ 0x3063, 0x3063, PDF_CMAP_SINGLE, 7923 },
	{ 0x3083, 0x3083, PDF_CMAP_SINGLE, 7924 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309c, PDF_CMAP_TABLE, 63 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 65 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 76 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 82 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 88 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 90 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 93 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 101 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 103 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 107 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 109 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337b, 0x337f, PDF_CMAP_TABLE, 113 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1a, 0xff1a, PDF_CMAP_SINGLE, 12101 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 118 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x309b, 0x309b, PDF_CMAP_SINGLE, 8272 },
	{ 0x309c, 0x309c, PDF_CMAP_SINGLE, 8271 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x3303, PDF_CMAP_SINGLE, 8338 },
	{ 0x3304, 0x3304, PDF_CMAP_SINGLE, 11960 },
	{ 0x3305, 0x3305, PDF_CMAP_SINGLE, 8333 },
	{ 0x3306, 0x3306, PDF_CMAP_SINGLE, 11961 },
	{ 0x3307, 0x3307, PDF_CMAP_SINGLE, 11965 },
	{ 0x3308, 0x3308, PDF_CMAP_SINGLE, 11963 },
	{ 0x3309, 0x3309, PDF_CMAP_SINGLE, 11968 },
	{ 0x330a, 0x330a, PDF_CMAP_SINGLE, 11966 },
	{ 0x330b, 0x330b, PDF_CMAP_SINGLE, 11970 },
	{ 0x330c, 0x330c, PDF_CMAP_SINGLE, 11972 },
	{ 0x330d, 0x330d, PDF_CMAP_SINGLE, 7950 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3314, PDF_CMAP_SINGLE, 7941 },
	{ 0x3315, 0x3315, PDF_CMAP_SINGLE, 8340 },
	{ 0x3316, 0x3316, PDF_CMAP_SINGLE, 8330 },
	{ 0x3317, 0x3317, PDF_CMAP_SINGLE, 11980 },
	{ 0x3318, 0x3318, PDF_CMAP_SINGLE, 8339 },
	{ 0x3319, 0x3319, PDF_CMAP_SINGLE, 11982 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3322, PDF_CMAP_SINGLE, 8329 },
	{ 0x3323, 0x3323, PDF_CMAP_SINGLE, 8348 },
	{ 0x3324, 0x3324, PDF_CMAP_SINGLE, 11991 },
	{ 0x3325, 0x3325, PDF_CMAP_SINGLE, 11993 },
	{ 0x3326, 0x3326, PDF_CMAP_SINGLE, 7951 },
	{ 0x3327, 0x3327, PDF_CMAP_SINGLE, 7945 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332a, PDF_CMAP_SINGLE, 8356 },
	{ 0x332b, 0x332b, PDF_CMAP_SINGLE, 7953 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3331, PDF_CMAP_SINGLE, 8358 },
	{ 0x3332, 0x3332, PDF_CMAP_SINGLE, 12005 },
	{ 0x3333, 0x3333, PDF_CMAP_SINGLE, 8334 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x3336, PDF_CMAP_SINGLE, 7947 },
	{ 0x3337, 0x3337, PDF_CMAP_SINGLE, 12014 },
	{ 0x3338, 0x3338, PDF_CMAP_SINGLE, 12016 },
	{ 0x3339, 0x3339, PDF_CMAP_SINGLE, 8343 },
	{ 0x333a, 0x333a, PDF_CMAP_SINGLE, 12017 },
	{ 0x333b, 0x333b, PDF_CMAP_SINGLE, 8349 },
	{ 0x333c, 0x333c, PDF_CMAP_SINGLE, 12010 },
	{ 0x333d, 0x333d, PDF_CMAP_SINGLE, 12018 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3341, PDF_CMAP_SINGLE, 12019 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 8347 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x3347, PDF_CMAP_SINGLE, 8357 },
	{ 0x3348, 0x3348, PDF_CMAP_SINGLE, 12027 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 7940 },
	{ 0x334a, 0x334a, PDF_CMAP_SINGLE, 7954 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334d, PDF_CMAP_SINGLE, 7943 },
	{ 0x334e, 0x334e, PDF_CMAP_SINGLE, 8337 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3351, PDF_CMAP_SINGLE, 7948 },
	{ 0x3352, 0x3352, PDF_CMAP_SINGLE, 12034 },
	{ 0x3353, 0x3353, PDF_CMAP_SINGLE, 12038 },
	{ 0x3354, 0x3354, PDF_CMAP_SINGLE, 12035 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337b, 0x337b, PDF_CMAP_SINGLE, 12044 },
	{ 0x337c, 0x337c, PDF_CMAP_SINGLE, 12043 },
	{ 0x337d, 0x337d, PDF_CMAP_SINGLE, 12042 },
	{ 0x337e, 0x337e, PDF_CMAP_SINGLE, 12041 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1a, 0xff1a, PDF_CMAP_SINGLE, 12101 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 7905 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 7896 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 7906 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7894 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
};

static const unsigned short pdf_cmap_UniJISPro_UCS2_V_table[122] =
{
	7892,7895,7898,7897,8273,8283,738,736,
	739,737,8311,8310,8312,8012,8014,8011,
	8013,7495,7497,7496,7498,7503,7505,7504,
	7506,7491,7493,7492,7494,7499,7501,7500,
	7502,7523,7527,7525,7524,7526,7529,7528,
	7535,7533,7532,7534,7537,7536,7538,7515,
	7541,7540,7542,7547,7549,7546,7548,7553,
	7552,8221,8219,8222,8220,7894,7956,8272,
	8271,8338,11960,8333,11961,11965,11963,11968,
	11966,11970,11972,7950,7941,8340,8330,11980,
	8339,11982,8329,8348,11991,11993,7951,7945,
	8356,7953,8358,12005,8334,7947,12014,12016,
	8343,12017,8349,12010,12018,12019,8347,8357,
	12027,7940,7954,7943,8337,7948,12034,12038,
	12035,12044,12043,12042,12041,8324,7905,7896,
	7906,7894,
};

pdf_cmap pdf_cmap_UniJISPro_UCS2_V =
{
	-1, "UniJISPro-UCS2-V", "UniJIS-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	200, 200, (pdf_range*) pdf_cmap_UniJISPro_UCS2_V_ranges,
	122, 122, (unsigned short*) pdf_cmap_UniJISPro_UCS2_V_table,
};

/* V */

static const pdf_range pdf_cmap_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_RANGE, 7887 },
	{ 0x2131, 0x2132, PDF_CMAP_RANGE, 7889 },
	{ 0x213c, 0x213e, PDF_CMAP_RANGE, 7891 },
	{ 0x2141, 0x2145, PDF_CMAP_RANGE, 7894 },
	{ 0x214a, 0x215b, PDF_CMAP_RANGE, 7899 },
	{ 0x2161, 0x2161, PDF_CMAP_SINGLE, 7917 },
	{ 0x2421, 0x2421, PDF_CMAP_SINGLE, 7918 },
	{ 0x2423, 0x2423, PDF_CMAP_SINGLE, 7919 },
	{ 0x2425, 0x2425, PDF_CMAP_SINGLE, 7920 },
	{ 0x2427, 0x2427, PDF_CMAP_SINGLE, 7921 },
	{ 0x2429, 0x2429, PDF_CMAP_SINGLE, 7922 },
	{ 0x2443, 0x2443, PDF_CMAP_SINGLE, 7923 },
	{ 0x2463, 0x2463, PDF_CMAP_SINGLE, 7924 },
	{ 0x2465, 0x2465, PDF_CMAP_SINGLE, 7925 },
	{ 0x2467, 0x2467, PDF_CMAP_SINGLE, 7926 },
	{ 0x246e, 0x246e, PDF_CMAP_SINGLE, 7927 },
	{ 0x2521, 0x2521, PDF_CMAP_SINGLE, 7928 },
	{ 0x2523, 0x2523, PDF_CMAP_SINGLE, 7929 },
	{ 0x2525, 0x2525, PDF_CMAP_SINGLE, 7930 },
	{ 0x2527, 0x2527, PDF_CMAP_SINGLE, 7931 },
	{ 0x2529, 0x2529, PDF_CMAP_SINGLE, 7932 },
	{ 0x2543, 0x2543, PDF_CMAP_SINGLE, 7933 },
	{ 0x2563, 0x2563, PDF_CMAP_SINGLE, 7934 },
	{ 0x2565, 0x2565, PDF_CMAP_SINGLE, 7935 },
	{ 0x2567, 0x2567, PDF_CMAP_SINGLE, 7936 },
	{ 0x256e, 0x256e, PDF_CMAP_SINGLE, 7937 },
	{ 0x2575, 0x2576, PDF_CMAP_RANGE, 7938 },
};

static const unsigned short pdf_cmap_V_table[1] = { 0 };

pdf_cmap pdf_cmap_V =
{
	-1, "V", "H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	27, 27, (pdf_range*) pdf_cmap_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_V_table,
};

/* WP-Symbol */

static const pdf_range pdf_cmap_WP_Symbol_ranges[] =
{
	{ 0x0001, 0x0004, PDF_CMAP_TABLE, 0 },
	{ 0x0005, 0x0018, PDF_CMAP_RANGE, 8061 },
	{ 0x0019, 0x0022, PDF_CMAP_RANGE, 8092 },
	{ 0x0023, 0x0023, PDF_CMAP_SINGLE, 8186 },
	{ 0x0024, 0x0025, PDF_CMAP_RANGE, 8020 },
	{ 0x0026, 0x0026, PDF_CMAP_SINGLE, 8187 },
	{ 0x0027, 0x0028, PDF_CMAP_RANGE, 8022 },
	{ 0x0029, 0x002a, PDF_CMAP_TABLE, 4 },
	{ 0x002b, 0x002d, PDF_CMAP_RANGE, 8024 },
	{ 0x002e, 0x002f, PDF_CMAP_RANGE, 8189 },
	{ 0x0030, 0x0031, PDF_CMAP_RANGE, 8192 },
	{ 0x0032, 0x0033, PDF_CMAP_RANGE, 7899 },
	{ 0x0034, 0x0036, PDF_CMAP_TABLE, 6 },
	{ 0x0037, 0x0039, PDF_CMAP_RANGE, 8027 },
	{ 0x003a, 0x003a, PDF_CMAP_SINGLE, 8195 },
	{ 0x003b, 0x0054, PDF_CMAP_RANGE, 8112 },
	{ 0x0055, 0x005d, PDF_CMAP_RANGE, 8197 },
	{ 0x005e, 0x005e, PDF_CMAP_SINGLE, 8151 },
	{ 0x0061, 0x0071, PDF_CMAP_RANGE, 8206 },
	{ 0x0072, 0x0072, PDF_CMAP_SINGLE, 8057 },
	{ 0x0073, 0x0077, PDF_CMAP_RANGE, 8223 },
	{ 0x003b, 0x0054, PDF_CMAP_RANGE, 8112 },
	{ 0x0055, 0x005d, PDF_CMAP_RANGE, 8197 },
	{ 0x005e, 0x005e, PDF_CMAP_SINGLE, 8151 },
	{ 0x0061, 0x0071, PDF_CMAP_RANGE, 8206 },
	{ 0x0072, 0x0072, PDF_CMAP_SINGLE, 8057 },
	{ 0x0073, 0x0077, PDF_CMAP_RANGE, 8223 },
};

static const unsigned short pdf_cmap_WP_Symbol_table[9] =
{
	8056,8058,8191,8196,8188,8037,7894,8194,
	8035,
};

pdf_cmap pdf_cmap_WP_Symbol =
{
	-1, "WP-Symbol", "", nil, 0,
	1, /* codespace table */
	{
		{ 1, 0x0000, 0x00ff },
	},
	27, 27, (pdf_range*) pdf_cmap_WP_Symbol_ranges,
	9, 9, (unsigned short*) pdf_cmap_WP_Symbol_table,
};

/* Adobe-Japan2-0 */

static const pdf_range pdf_cmap_Adobe_Japan2_0_ranges[] =
{
	{ 0x0000, 0x17b3, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17b3, PDF_CMAP_RANGE, 5888 },
};

static const unsigned short pdf_cmap_Adobe_Japan2_0_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Japan2_0 =
{
	-1, "Adobe-Japan2-0", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x17ff },
	},
	24, 24, (pdf_range*) pdf_cmap_Adobe_Japan2_0_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Japan2_0_table,
};

/* Hojo-EUC-H */

static const pdf_range pdf_cmap_Hojo_EUC_H_ranges[] =
{
	{ 0xa2af, 0xa2b9, PDF_CMAP_RANGE, 1 },
	{ 0xa2c2, 0xa2c4, PDF_CMAP_RANGE, 12 },
	{ 0xa2eb, 0xa2f1, PDF_CMAP_RANGE, 15 },
	{ 0xa6e1, 0xa6e5, PDF_CMAP_RANGE, 22 },
	{ 0xa6e7, 0xa6e7, PDF_CMAP_SINGLE, 27 },
	{ 0xa6e9, 0xa6ea, PDF_CMAP_RANGE, 28 },
	{ 0xa6ec, 0xa6ec, PDF_CMAP_SINGLE, 30 },
	{ 0xa6f1, 0xa6fc, PDF_CMAP_RANGE, 31 },
	{ 0xa7c2, 0xa7ce, PDF_CMAP_RANGE, 43 },
	{ 0xa7f2, 0xa7fe, PDF_CMAP_RANGE, 56 },
	{ 0xa9a1, 0xa9a2, PDF_CMAP_RANGE, 69 },
	{ 0xa9a4, 0xa9a4, PDF_CMAP_SINGLE, 71 },
	{ 0xa9a6, 0xa9a6, PDF_CMAP_SINGLE, 72 },
	{ 0xa9a8, 0xa9a9, PDF_CMAP_RANGE, 73 },
	{ 0xa9ab, 0xa9ad, PDF_CMAP_RANGE, 75 },
	{ 0xa9af, 0xa9b0, PDF_CMAP_RANGE, 78 },
	{ 0xa9c1, 0xa9d0, PDF_CMAP_RANGE, 80 },
	{ 0xaaa1, 0xaab8, PDF_CMAP_RANGE, 96 },
	{ 0xaaba, 0xaaf7, PDF_CMAP_RANGE, 120 },
	{ 0xaba1, 0xabbb, PDF_CMAP_RANGE, 182 },
	{ 0xabbd, 0xabc3, PDF_CMAP_RANGE, 209 },
	{ 0xabc5, 0xabf7, PDF_CMAP_RANGE, 216 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 267 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 361 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 455 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 549 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 643 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 737 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 831 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 925 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1019 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1113 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 1207 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 1301 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 1395 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 1489 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 1583 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 1677 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 1771 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 1865 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 1959 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2053 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2147 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 2241 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 2335 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 2429 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 2523 },
	{ 0xc9a1, 0xc9fe, PDF_CMAP_RANGE, 2617 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 2711 },
	{ 0xcba1, 0xcbfe, PDF_CMAP_RANGE, 2805 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 2899 },
	{ 0xcda1, 0xcdfe, PDF_CMAP_RANGE, 2993 },
	{ 0xcea1, 0xcefe, PDF_CMAP_RANGE, 3087 },
	{ 0xcfa1, 0xcffe, PDF_CMAP_RANGE, 3181 },
	{ 0xd0a1, 0xd0fe, PDF_CMAP_RANGE, 3275 },
	{ 0xd1a1, 0xd1fe, PDF_CMAP_RANGE, 3369 },
	{ 0xd2a1, 0xd2fe, PDF_CMAP_RANGE, 3463 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 3557 },
	{ 0xd4a1, 0xd4fe, PDF_CMAP_RANGE, 3651 },
	{ 0xd5a1, 0xd5fe, PDF_CMAP_RANGE, 3745 },
	{ 0xd6a1, 0xd6fe, PDF_CMAP_RANGE, 3839 },
	{ 0xd7a1, 0xd7fe, PDF_CMAP_RANGE, 3933 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4027 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4121 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4215 },
	{ 0xdba1, 0xdbfe, PDF_CMAP_RANGE, 4309 },
	{ 0xdca1, 0xdcfe, PDF_CMAP_RANGE, 4403 },
	{ 0xdda1, 0xddfe, PDF_CMAP_RANGE, 4497 },
	{ 0xdea1, 0xdefe, PDF_CMAP_RANGE, 4591 },
	{ 0xdfa1, 0xdffe, PDF_CMAP_RANGE, 4685 },
	{ 0xe0a1, 0xe0fe, PDF_CMAP_RANGE, 4779 },
	{ 0xe1a1, 0xe1fe, PDF_CMAP_RANGE, 4873 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 4967 },
	{ 0xe3a1, 0xe3fe, PDF_CMAP_RANGE, 5061 },
	{ 0xe4a1, 0xe4fe, PDF_CMAP_RANGE, 5155 },
	{ 0xe5a1, 0xe5fe, PDF_CMAP_RANGE, 5249 },
	{ 0xe6a1, 0xe6fe, PDF_CMAP_RANGE, 5343 },
	{ 0xe7a1, 0xe7fe, PDF_CMAP_RANGE, 5437 },
	{ 0xe8a1, 0xe8fe, PDF_CMAP_RANGE, 5531 },
	{ 0xe9a1, 0xe9fe, PDF_CMAP_RANGE, 5625 },
	{ 0xeaa1, 0xeafe, PDF_CMAP_RANGE, 5719 },
	{ 0xeba1, 0xebfe, PDF_CMAP_RANGE, 5813 },
	{ 0xeca1, 0xecfe, PDF_CMAP_RANGE, 5907 },
	{ 0xeda1, 0xede3, PDF_CMAP_RANGE, 6001 },
};

static const unsigned short pdf_cmap_Hojo_EUC_H_table[1] = { 0 };

pdf_cmap pdf_cmap_Hojo_EUC_H =
{
	-1, "Hojo-EUC-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 3, 0xa1a1, 0xfefe },
	},
	84, 84, (pdf_range*) pdf_cmap_Hojo_EUC_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_Hojo_EUC_H_table,
};

/* Hojo-EUC-V */

static const pdf_range pdf_cmap_Hojo_EUC_V_ranges[] =
{
	/* dummy entry for non-c99 compilers */
	{ 0x0, 0x0, PDF_CMAP_RANGE, 0 }
};

static const unsigned short pdf_cmap_Hojo_EUC_V_table[1] = { 0 };

pdf_cmap pdf_cmap_Hojo_EUC_V =
{
	-1, "Hojo-EUC-V", "Hojo-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	0, 0, (pdf_range*) pdf_cmap_Hojo_EUC_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_Hojo_EUC_V_table,
};

/* Hojo-H */

static const pdf_range pdf_cmap_Hojo_H_ranges[] =
{
	{ 0x222f, 0x2239, PDF_CMAP_RANGE, 1 },
	{ 0x2242, 0x2244, PDF_CMAP_RANGE, 12 },
	{ 0x226b, 0x2271, PDF_CMAP_RANGE, 15 },
	{ 0x2661, 0x2665, PDF_CMAP_RANGE, 22 },
	{ 0x2667, 0x2667, PDF_CMAP_SINGLE, 27 },
	{ 0x2669, 0x266a, PDF_CMAP_RANGE, 28 },
	{ 0x266c, 0x266c, PDF_CMAP_SINGLE, 30 },
	{ 0x2671, 0x267c, PDF_CMAP_RANGE, 31 },
	{ 0x2742, 0x274e, PDF_CMAP_RANGE, 43 },
	{ 0x2772, 0x277e, PDF_CMAP_RANGE, 56 },
	{ 0x2921, 0x2922, PDF_CMAP_RANGE, 69 },
	{ 0x2924, 0x2924, PDF_CMAP_SINGLE, 71 },
	{ 0x2926, 0x2926, PDF_CMAP_SINGLE, 72 },
	{ 0x2928, 0x2929, PDF_CMAP_RANGE, 73 },
	{ 0x292b, 0x292d, PDF_CMAP_RANGE, 75 },
	{ 0x292f, 0x2930, PDF_CMAP_RANGE, 78 },
	{ 0x2941, 0x2950, PDF_CMAP_RANGE, 80 },
	{ 0x2a21, 0x2a38, PDF_CMAP_RANGE, 96 },
	{ 0x2a3a, 0x2a77, PDF_CMAP_RANGE, 120 },
	{ 0x2b21, 0x2b3b, PDF_CMAP_RANGE, 182 },
	{ 0x2b3d, 0x2b43, PDF_CMAP_RANGE, 209 },
	{ 0x2b45, 0x2b77, PDF_CMAP_RANGE, 216 },
	{ 0x3021, 0x307e, PDF_CMAP_RANGE, 267 },
	{ 0x3121, 0x317e, PDF_CMAP_RANGE, 361 },
	{ 0x3221, 0x327e, PDF_CMAP_RANGE, 455 },
	{ 0x3321, 0x337e, PDF_CMAP_RANGE, 549 },
	{ 0x3421, 0x347e, PDF_CMAP_RANGE, 643 },
	{ 0x3521, 0x357e, PDF_CMAP_RANGE, 737 },
	{ 0x3621, 0x367e, PDF_CMAP_RANGE, 831 },
	{ 0x3721, 0x377e, PDF_CMAP_RANGE, 925 },
	{ 0x3821, 0x387e, PDF_CMAP_RANGE, 1019 },
	{ 0x3921, 0x397e, PDF_CMAP_RANGE, 1113 },
	{ 0x3a21, 0x3a7e, PDF_CMAP_RANGE, 1207 },
	{ 0x3b21, 0x3b7e, PDF_CMAP_RANGE, 1301 },
	{ 0x3c21, 0x3c7e, PDF_CMAP_RANGE, 1395 },
	{ 0x3d21, 0x3d7e, PDF_CMAP_RANGE, 1489 },
	{ 0x3e21, 0x3e7e, PDF_CMAP_RANGE, 1583 },
	{ 0x3f21, 0x3f7e, PDF_CMAP_RANGE, 1677 },
	{ 0x4021, 0x407e, PDF_CMAP_RANGE, 1771 },
	{ 0x4121, 0x417e, PDF_CMAP_RANGE, 1865 },
	{ 0x4221, 0x427e, PDF_CMAP_RANGE, 1959 },
	{ 0x4321, 0x437e, PDF_CMAP_RANGE, 2053 },
	{ 0x4421, 0x447e, PDF_CMAP_RANGE, 2147 },
	{ 0x4521, 0x457e, PDF_CMAP_RANGE, 2241 },
	{ 0x4621, 0x467e, PDF_CMAP_RANGE, 2335 },
	{ 0x4721, 0x477e, PDF_CMAP_RANGE, 2429 },
	{ 0x4821, 0x487e, PDF_CMAP_RANGE, 2523 },
	{ 0x4921, 0x497e, PDF_CMAP_RANGE, 2617 },
	{ 0x4a21, 0x4a7e, PDF_CMAP_RANGE, 2711 },
	{ 0x4b21, 0x4b7e, PDF_CMAP_RANGE, 2805 },
	{ 0x4c21, 0x4c7e, PDF_CMAP_RANGE, 2899 },
	{ 0x4d21, 0x4d7e, PDF_CMAP_RANGE, 2993 },
	{ 0x4e21, 0x4e7e, PDF_CMAP_RANGE, 3087 },
	{ 0x4f21, 0x4f7e, PDF_CMAP_RANGE, 3181 },
	{ 0x5021, 0x507e, PDF_CMAP_RANGE, 3275 },
	{ 0x5121, 0x517e, PDF_CMAP_RANGE, 3369 },
	{ 0x5221, 0x527e, PDF_CMAP_RANGE, 3463 },
	{ 0x5321, 0x537e, PDF_CMAP_RANGE, 3557 },
	{ 0x5421, 0x547e, PDF_CMAP_RANGE, 3651 },
	{ 0x5521, 0x557e, PDF_CMAP_RANGE, 3745 },
	{ 0x5621, 0x567e, PDF_CMAP_RANGE, 3839 },
	{ 0x5721, 0x577e, PDF_CMAP_RANGE, 3933 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 4027 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 4121 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 4215 },
	{ 0x5b21, 0x5b7e, PDF_CMAP_RANGE, 4309 },
	{ 0x5c21, 0x5c7e, PDF_CMAP_RANGE, 4403 },
	{ 0x5d21, 0x5d7e, PDF_CMAP_RANGE, 4497 },
	{ 0x5e21, 0x5e7e, PDF_CMAP_RANGE, 4591 },
	{ 0x5f21, 0x5f7e, PDF_CMAP_RANGE, 4685 },
	{ 0x6021, 0x607e, PDF_CMAP_RANGE, 4779 },
	{ 0x6121, 0x617e, PDF_CMAP_RANGE, 4873 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 4967 },
	{ 0x6321, 0x637e, PDF_CMAP_RANGE, 5061 },
	{ 0x6421, 0x647e, PDF_CMAP_RANGE, 5155 },
	{ 0x6521, 0x657e, PDF_CMAP_RANGE, 5249 },
	{ 0x6621, 0x667e, PDF_CMAP_RANGE, 5343 },
	{ 0x6721, 0x677e, PDF_CMAP_RANGE, 5437 },
	{ 0x6821, 0x687e, PDF_CMAP_RANGE, 5531 },
	{ 0x6921, 0x697e, PDF_CMAP_RANGE, 5625 },
	{ 0x6a21, 0x6a7e, PDF_CMAP_RANGE, 5719 },
	{ 0x6b21, 0x6b7e, PDF_CMAP_RANGE, 5813 },
	{ 0x6c21, 0x6c7e, PDF_CMAP_RANGE, 5907 },
	{ 0x6d21, 0x6d63, PDF_CMAP_RANGE, 6001 },
};

static const unsigned short pdf_cmap_Hojo_H_table[1] = { 0 };

pdf_cmap pdf_cmap_Hojo_H =
{
	-1, "Hojo-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	84, 84, (pdf_range*) pdf_cmap_Hojo_H_ranges,
	0, 0, (unsigned short*) pdf_cmap_Hojo_H_table,
};

/* Hojo-V */

static const pdf_range pdf_cmap_Hojo_V_ranges[] =
{
	/* dummy entry for non-c99 compilers */
	{ 0x0, 0x0, PDF_CMAP_RANGE, 0 }
};

static const unsigned short pdf_cmap_Hojo_V_table[1] = { 0 };

pdf_cmap pdf_cmap_Hojo_V =
{
	-1, "Hojo-V", "Hojo-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	0, 0, (pdf_range*) pdf_cmap_Hojo_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_Hojo_V_table,
};

/* UniHojo-UCS2-H */

static const pdf_range pdf_cmap_UniHojo_UCS2_H_ranges[] =
{
	{ 0x007e, 0x007e, PDF_CMAP_SINGLE, 9 },
	{ 0x00a1, 0x00a1, PDF_CMAP_SINGLE, 12 },
	{ 0x00a4, 0x00a4, PDF_CMAP_SINGLE, 20 },
	{ 0x00a6, 0x00a6, PDF_CMAP_SINGLE, 13 },
	{ 0x00a9, 0x00aa, PDF_CMAP_TABLE, 0 },
	{ 0x00ae, 0x00af, PDF_CMAP_TABLE, 2 },
	{ 0x00b8, 0x00b8, PDF_CMAP_SINGLE, 3 },
	{ 0x00ba, 0x00ba, PDF_CMAP_SINGLE, 15 },
	{ 0x00bf, 0x00cf, PDF_CMAP_TABLE, 4 },
	{ 0x00d1, 0x00d6, PDF_CMAP_TABLE, 21 },
	{ 0x00d8, 0x00f6, PDF_CMAP_TABLE, 27 },
	{ 0x00f8, 0x0113, PDF_CMAP_TABLE, 58 },
	{ 0x0116, 0x0122, PDF_CMAP_TABLE, 86 },
	{ 0x0124, 0x012b, PDF_CMAP_TABLE, 99 },
	{ 0x012e, 0x014d, PDF_CMAP_TABLE, 107 },
	{ 0x0150, 0x017e, PDF_CMAP_TABLE, 139 },
	{ 0x01cd, 0x01dc, PDF_CMAP_TABLE, 186 },
	{ 0x01f5, 0x01f5, PDF_CMAP_SINGLE, 206 },
	{ 0x02c7, 0x02c7, PDF_CMAP_SINGLE, 2 },
	{ 0x02d8, 0x02db, PDF_CMAP_TABLE, 202 },
	{ 0x02dd, 0x02dd, PDF_CMAP_SINGLE, 5 },
	{ 0x0384, 0x0385, PDF_CMAP_RANGE, 10 },
	{ 0x0386, 0x0386, PDF_CMAP_SINGLE, 22 },
	{ 0x0388, 0x038a, PDF_CMAP_RANGE, 23 },
	{ 0x038c, 0x038c, PDF_CMAP_SINGLE, 27 },
	{ 0x038e, 0x0390, PDF_CMAP_TABLE, 206 },
	{ 0x03aa, 0x03ab, PDF_CMAP_TABLE, 209 },
	{ 0x03ac, 0x03af, PDF_CMAP_RANGE, 31 },
	{ 0x03b0, 0x03b0, PDF_CMAP_SINGLE, 41 },
	{ 0x03c2, 0x03c2, PDF_CMAP_SINGLE, 38 },
	{ 0x03ca, 0x03ce, PDF_CMAP_TABLE, 211 },
	{ 0x0402, 0x040c, PDF_CMAP_RANGE, 43 },
	{ 0x040e, 0x040f, PDF_CMAP_RANGE, 54 },
	{ 0x0452, 0x045c, PDF_CMAP_RANGE, 56 },
	{ 0x045e, 0x045f, PDF_CMAP_RANGE, 67 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 21 },
	{ 0x2122, 0x2122, PDF_CMAP_SINGLE, 19 },
	{ 0x4e02, 0x4e02, PDF_CMAP_SINGLE, 267 },
	{ 0x4e04, 0x4e05, PDF_CMAP_RANGE, 268 },
	{ 0x4e0c, 0x4e0c, PDF_CMAP_SINGLE, 270 },
	{ 0x4e12, 0x4e12, PDF_CMAP_SINGLE, 271 },
	{ 0x4e1f, 0x4e1f, PDF_CMAP_SINGLE, 272 },
	{ 0x4e23, 0x4e24, PDF_CMAP_RANGE, 273 },
	{ 0x4e28, 0x4e28, PDF_CMAP_SINGLE, 275 },
	{ 0x4e2b, 0x4e2b, PDF_CMAP_SINGLE, 276 },
	{ 0x4e2e, 0x4e30, PDF_CMAP_RANGE, 277 },
	{ 0x4e35, 0x4e35, PDF_CMAP_SINGLE, 280 },
	{ 0x4e40, 0x4e41, PDF_CMAP_RANGE, 281 },
	{ 0x4e44, 0x4e44, PDF_CMAP_SINGLE, 283 },
	{ 0x4e47, 0x4e47, PDF_CMAP_SINGLE, 284 },
	{ 0x4e51, 0x4e51, PDF_CMAP_SINGLE, 285 },
	{ 0x4e5a, 0x4e5a, PDF_CMAP_SINGLE, 286 },
	{ 0x4e5c, 0x4e5c, PDF_CMAP_SINGLE, 287 },
	{ 0x4e63, 0x4e63, PDF_CMAP_SINGLE, 288 },
	{ 0x4e68, 0x4e69, PDF_CMAP_RANGE, 289 },
	{ 0x4e74, 0x4e75, PDF_CMAP_RANGE, 291 },
	{ 0x4e79, 0x4e79, PDF_CMAP_SINGLE, 293 },
	{ 0x4e7f, 0x4e7f, PDF_CMAP_SINGLE, 294 },
	{ 0x4e8d, 0x4e8d, PDF_CMAP_SINGLE, 295 },
	{ 0x4e96, 0x4e97, PDF_CMAP_RANGE, 296 },
	{ 0x4e9d, 0x4e9d, PDF_CMAP_SINGLE, 298 },
	{ 0x4eaf, 0x4eaf, PDF_CMAP_SINGLE, 299 },
	{ 0x4eb9, 0x4eb9, PDF_CMAP_SINGLE, 300 },
	{ 0x4ec3, 0x4ec3, PDF_CMAP_SINGLE, 301 },
	{ 0x4ed0, 0x4ed0, PDF_CMAP_SINGLE, 302 },
	{ 0x4eda, 0x4edb, PDF_CMAP_RANGE, 303 },
	{ 0x4ee0, 0x4ee2, PDF_CMAP_RANGE, 305 },
	{ 0x4ee8, 0x4ee8, PDF_CMAP_SINGLE, 308 },
	{ 0x4eef, 0x4eef, PDF_CMAP_SINGLE, 309 },
	{ 0x4ef1, 0x4ef1, PDF_CMAP_SINGLE, 310 },
	{ 0x4ef3, 0x4ef3, PDF_CMAP_SINGLE, 311 },
	{ 0x4ef5, 0x4ef5, PDF_CMAP_SINGLE, 312 },
	{ 0x4efd, 0x4f00, PDF_CMAP_RANGE, 313 },
	{ 0x4f02, 0x4f03, PDF_CMAP_RANGE, 317 },
	{ 0x4f08, 0x4f08, PDF_CMAP_SINGLE, 319 },
	{ 0x4f0b, 0x4f0c, PDF_CMAP_RANGE, 320 },
	{ 0x4f12, 0x4f12, PDF_CMAP_SINGLE, 322 },
	{ 0x4f15, 0x4f17, PDF_CMAP_RANGE, 323 },
	{ 0x4f19, 0x4f19, PDF_CMAP_SINGLE, 326 },
	{ 0x4f2e, 0x4f2e, PDF_CMAP_SINGLE, 327 },
	{ 0x4f31, 0x4f31, PDF_CMAP_SINGLE, 328 },
	{ 0x4f33, 0x4f33, PDF_CMAP_SINGLE, 330 },
	{ 0x4f35, 0x4f35, PDF_CMAP_SINGLE, 331 },
	{ 0x4f37, 0x4f37, PDF_CMAP_SINGLE, 332 },
	{ 0x4f39, 0x4f39, PDF_CMAP_SINGLE, 333 },
	{ 0x4f3b, 0x4f3b, PDF_CMAP_SINGLE, 334 },
	{ 0x4f3e, 0x4f3e, PDF_CMAP_SINGLE, 335 },
	{ 0x4f40, 0x4f40, PDF_CMAP_SINGLE, 336 },
	{ 0x4f42, 0x4f42, PDF_CMAP_SINGLE, 337 },
	{ 0x4f48, 0x4f49, PDF_CMAP_RANGE, 338 },
	{ 0x4f4b, 0x4f4c, PDF_CMAP_RANGE, 340 },
	{ 0x4f52, 0x4f52, PDF_CMAP_SINGLE, 342 },
	{ 0x4f54, 0x4f54, PDF_CMAP_SINGLE, 343 },
	{ 0x4f56, 0x4f56, PDF_CMAP_SINGLE, 344 },
	{ 0x4f58, 0x4f58, PDF_CMAP_SINGLE, 345 },
	{ 0x4f5f, 0x4f60, PDF_CMAP_TABLE, 216 },
	{ 0x4f63, 0x4f63, PDF_CMAP_SINGLE, 347 },
	{ 0x4f6a, 0x4f6a, PDF_CMAP_SINGLE, 348 },
	{ 0x4f6c, 0x4f6c, PDF_CMAP_SINGLE, 349 },
	{ 0x4f6e, 0x4f6e, PDF_CMAP_SINGLE, 350 },
	{ 0x4f71, 0x4f71, PDF_CMAP_SINGLE, 351 },
	{ 0x4f77, 0x4f7a, PDF_CMAP_RANGE, 352 },
	{ 0x4f7d, 0x4f7e, PDF_CMAP_RANGE, 356 },
	{ 0x4f81, 0x4f82, PDF_CMAP_RANGE, 358 },
	{ 0x4f84, 0x4f85, PDF_CMAP_RANGE, 360 },
	{ 0x4f89, 0x4f8a, PDF_CMAP_RANGE, 362 },
	{ 0x4f8c, 0x4f8c, PDF_CMAP_SINGLE, 364 },
	{ 0x4f8e, 0x4f8e, PDF_CMAP_SINGLE, 365 },
	{ 0x4f90, 0x4f90, PDF_CMAP_SINGLE, 366 },
	{ 0x4f92, 0x4f94, PDF_CMAP_RANGE, 367 },
	{ 0x4f97, 0x4f97, PDF_CMAP_SINGLE, 370 },
	{ 0x4f99, 0x4f9a, PDF_CMAP_RANGE, 371 },
	{ 0x4f9e, 0x4f9f, PDF_CMAP_RANGE, 373 },
	{ 0x4fb2, 0x4fb2, PDF_CMAP_SINGLE, 375 },
	{ 0x4fb7, 0x4fb7, PDF_CMAP_SINGLE, 376 },
	{ 0x4fb9, 0x4fb9, PDF_CMAP_SINGLE, 377 },
	{ 0x4fbb, 0x4fbe, PDF_CMAP_RANGE, 378 },
	{ 0x4fc0, 0x4fc1, PDF_CMAP_RANGE, 382 },
	{ 0x4fc5, 0x4fc6, PDF_CMAP_RANGE, 384 },
	{ 0x4fc8, 0x4fc9, PDF_CMAP_RANGE, 386 },
	{ 0x4fcb, 0x4fcd, PDF_CMAP_RANGE, 388 },
	{ 0x4fcf, 0x4fcf, PDF_CMAP_SINGLE, 391 },
	{ 0x4fd2, 0x4fd2, PDF_CMAP_SINGLE, 392 },
	{ 0x4fdc, 0x4fdc, PDF_CMAP_SINGLE, 393 },
	{ 0x4fe0, 0x4fe0, PDF_CMAP_SINGLE, 394 },
	{ 0x4fe2, 0x4fe2, PDF_CMAP_SINGLE, 395 },
	{ 0x4ff0, 0x4ff0, PDF_CMAP_SINGLE, 396 },
	{ 0x4ff2, 0x4ff2, PDF_CMAP_SINGLE, 397 },
	{ 0x4ffc, 0x4ffd, PDF_CMAP_RANGE, 398 },
	{ 0x4fff, 0x5001, PDF_CMAP_RANGE, 400 },
	{ 0x5004, 0x5004, PDF_CMAP_SINGLE, 403 },
	{ 0x5007, 0x5007, PDF_CMAP_SINGLE, 404 },
	{ 0x500a, 0x500a, PDF_CMAP_SINGLE, 405 },
	{ 0x500c, 0x500c, PDF_CMAP_SINGLE, 406 },
	{ 0x500e, 0x500e, PDF_CMAP_SINGLE, 407 },
	{ 0x5010, 0x5010, PDF_CMAP_SINGLE, 408 },
	{ 0x5013, 0x5013, PDF_CMAP_SINGLE, 409 },
	{ 0x5017, 0x5018, PDF_CMAP_RANGE, 410 },
	{ 0x501b, 0x501e, PDF_CMAP_RANGE, 412 },
	{ 0x5022, 0x5022, PDF_CMAP_SINGLE, 416 },
	{ 0x5027, 0x5027, PDF_CMAP_SINGLE, 417 },
	{ 0x502e, 0x502e, PDF_CMAP_SINGLE, 418 },
	{ 0x5030, 0x5030, PDF_CMAP_SINGLE, 419 },
	{ 0x5032, 0x5033, PDF_CMAP_RANGE, 420 },
	{ 0x5035, 0x5035, PDF_CMAP_SINGLE, 422 },
	{ 0x503b, 0x503b, PDF_CMAP_SINGLE, 446 },
	{ 0x5040, 0x5042, PDF_CMAP_RANGE, 423 },
	{ 0x5045, 0x5046, PDF_CMAP_RANGE, 426 },
	{ 0x504a, 0x504a, PDF_CMAP_SINGLE, 428 },
	{ 0x504c, 0x504c, PDF_CMAP_SINGLE, 429 },
	{ 0x504e, 0x504e, PDF_CMAP_SINGLE, 430 },
	{ 0x5051, 0x5053, PDF_CMAP_RANGE, 431 },
	{ 0x5057, 0x5057, PDF_CMAP_SINGLE, 434 },
	{ 0x5059, 0x5059, PDF_CMAP_SINGLE, 435 },
	{ 0x505f, 0x5060, PDF_CMAP_RANGE, 436 },
	{ 0x5062, 0x5063, PDF_CMAP_RANGE, 438 },
	{ 0x5066, 0x5067, PDF_CMAP_RANGE, 440 },
	{ 0x506a, 0x506a, PDF_CMAP_SINGLE, 442 },
	{ 0x506d, 0x506d, PDF_CMAP_SINGLE, 443 },
	{ 0x5070, 0x5071, PDF_CMAP_RANGE, 444 },
	{ 0x5081, 0x5081, PDF_CMAP_SINGLE, 447 },
	{ 0x5083, 0x5084, PDF_CMAP_RANGE, 448 },
	{ 0x5086, 0x5086, PDF_CMAP_SINGLE, 450 },
	{ 0x508a, 0x508a, PDF_CMAP_SINGLE, 451 },
	{ 0x508e, 0x5090, PDF_CMAP_RANGE, 452 },
	{ 0x5092, 0x5094, PDF_CMAP_RANGE, 455 },
	{ 0x5096, 0x5096, PDF_CMAP_SINGLE, 458 },
	{ 0x509b, 0x509c, PDF_CMAP_RANGE, 459 },
	{ 0x509e, 0x50a2, PDF_CMAP_RANGE, 461 },
	{ 0x50aa, 0x50aa, PDF_CMAP_SINGLE, 466 },
	{ 0x50af, 0x50b0, PDF_CMAP_RANGE, 467 },
	{ 0x50b9, 0x50ba, PDF_CMAP_RANGE, 469 },
	{ 0x50bd, 0x50bd, PDF_CMAP_SINGLE, 471 },
	{ 0x50c0, 0x50c0, PDF_CMAP_SINGLE, 472 },
	{ 0x50c3, 0x50c4, PDF_CMAP_RANGE, 473 },
	{ 0x50c7, 0x50c7, PDF_CMAP_SINGLE, 475 },
	{ 0x50cc, 0x50cc, PDF_CMAP_SINGLE, 476 },
	{ 0x50ce, 0x50ce, PDF_CMAP_SINGLE, 477 },
	{ 0x50d0, 0x50d0, PDF_CMAP_SINGLE, 478 },
	{ 0x50d3, 0x50d4, PDF_CMAP_RANGE, 479 },
	{ 0x50d8, 0x50d8, PDF_CMAP_SINGLE, 481 },
	{ 0x50dc, 0x50dd, PDF_CMAP_RANGE, 482 },
	{ 0x50df, 0x50df, PDF_CMAP_SINGLE, 484 },
	{ 0x50e2, 0x50e2, PDF_CMAP_SINGLE, 485 },
	{ 0x50e4, 0x50e4, PDF_CMAP_SINGLE, 486 },
	{ 0x50e6, 0x50e6, PDF_CMAP_SINGLE, 487 },
	{ 0x50e8, 0x50e9, PDF_CMAP_RANGE, 488 },
	{ 0x50ef, 0x50ef, PDF_CMAP_SINGLE, 490 },
	{ 0x50f1, 0x50f2, PDF_CMAP_TABLE, 218 },
	{ 0x50f6, 0x50f6, PDF_CMAP_SINGLE, 492 },
	{ 0x50fa, 0x50fa, PDF_CMAP_SINGLE, 493 },
	{ 0x50fe, 0x50fe, PDF_CMAP_SINGLE, 494 },
	{ 0x5103, 0x5103, PDF_CMAP_SINGLE, 495 },
	{ 0x5106, 0x5108, PDF_CMAP_RANGE, 496 },
	{ 0x510b, 0x510e, PDF_CMAP_RANGE, 499 },
	{ 0x5110, 0x5110, PDF_CMAP_SINGLE, 504 },
	{ 0x5117, 0x5117, PDF_CMAP_SINGLE, 505 },
	{ 0x5119, 0x5119, PDF_CMAP_SINGLE, 506 },
	{ 0x511b, 0x511e, PDF_CMAP_RANGE, 507 },
	{ 0x5123, 0x5123, PDF_CMAP_SINGLE, 511 },
	{ 0x5127, 0x5128, PDF_CMAP_RANGE, 512 },
	{ 0x512c, 0x512d, PDF_CMAP_RANGE, 514 },
	{ 0x512f, 0x512f, PDF_CMAP_SINGLE, 516 },
	{ 0x5131, 0x5131, PDF_CMAP_SINGLE, 517 },
	{ 0x5133, 0x5135, PDF_CMAP_RANGE, 518 },
	{ 0x5138, 0x5139, PDF_CMAP_RANGE, 521 },
	{ 0x5142, 0x5142, PDF_CMAP_SINGLE, 523 },
	{ 0x514a, 0x514a, PDF_CMAP_SINGLE, 524 },
	{ 0x514f, 0x514f, PDF_CMAP_SINGLE, 525 },
	{ 0x5153, 0x5153, PDF_CMAP_SINGLE, 526 },
	{ 0x5155, 0x5155, PDF_CMAP_SINGLE, 527 },
	{ 0x5157, 0x5158, PDF_CMAP_RANGE, 528 },
	{ 0x515f, 0x515f, PDF_CMAP_SINGLE, 530 },
	{ 0x5164, 0x5164, PDF_CMAP_SINGLE, 531 },
	{ 0x5166, 0x5166, PDF_CMAP_SINGLE, 532 },
	{ 0x517e, 0x517e, PDF_CMAP_SINGLE, 533 },
	{ 0x5183, 0x5184, PDF_CMAP_RANGE, 534 },
	{ 0x518b, 0x518b, PDF_CMAP_SINGLE, 536 },
	{ 0x518e, 0x518e, PDF_CMAP_SINGLE, 537 },
	{ 0x5198, 0x5198, PDF_CMAP_SINGLE, 538 },
	{ 0x519d, 0x519d, PDF_CMAP_SINGLE, 539 },
	{ 0x51a1, 0x51a1, PDF_CMAP_SINGLE, 540 },
	{ 0x51a3, 0x51a3, PDF_CMAP_SINGLE, 541 },
	{ 0x51ad, 0x51ad, PDF_CMAP_SINGLE, 542 },
	{ 0x51b8, 0x51b8, PDF_CMAP_SINGLE, 543 },
	{ 0x51ba, 0x51ba, PDF_CMAP_SINGLE, 544 },
	{ 0x51bc, 0x51bc, PDF_CMAP_SINGLE, 545 },
	{ 0x51be, 0x51bf, PDF_CMAP_RANGE, 546 },
	{ 0x51c2, 0x51c2, PDF_CMAP_SINGLE, 548 },
	{ 0x51c8, 0x51c8, PDF_CMAP_SINGLE, 549 },
	{ 0x51cf, 0x51cf, PDF_CMAP_SINGLE, 550 },
	{ 0x51d1, 0x51d3, PDF_CMAP_RANGE, 551 },
	{ 0x51d5, 0x51d5, PDF_CMAP_SINGLE, 554 },
	{ 0x51d8, 0x51d8, PDF_CMAP_SINGLE, 555 },
	{ 0x51de, 0x51de, PDF_CMAP_SINGLE, 556 },
	{ 0x51e2, 0x51e2, PDF_CMAP_SINGLE, 557 },
	{ 0x51e5, 0x51e5, PDF_CMAP_SINGLE, 558 },
	{ 0x51ee, 0x51ee, PDF_CMAP_SINGLE, 559 },
	{ 0x51f2, 0x51f4, PDF_CMAP_RANGE, 560 },
	{ 0x51f7, 0x51f7, PDF_CMAP_SINGLE, 563 },
	{ 0x5201, 0x5202, PDF_CMAP_RANGE, 564 },
	{ 0x5205, 0x5205, PDF_CMAP_SINGLE, 566 },
	{ 0x5212, 0x5213, PDF_CMAP_RANGE, 567 },
	{ 0x5215, 0x5216, PDF_CMAP_RANGE, 569 },
	{ 0x5218, 0x5218, PDF_CMAP_SINGLE, 571 },
	{ 0x5222, 0x5222, PDF_CMAP_SINGLE, 572 },
	{ 0x5228, 0x5228, PDF_CMAP_SINGLE, 573 },
	{ 0x5231, 0x5232, PDF_CMAP_RANGE, 574 },
	{ 0x5235, 0x5235, PDF_CMAP_SINGLE, 576 },
	{ 0x523c, 0x523c, PDF_CMAP_SINGLE, 577 },
	{ 0x5245, 0x5245, PDF_CMAP_SINGLE, 578 },
	{ 0x5249, 0x5249, PDF_CMAP_SINGLE, 579 },
	{ 0x5255, 0x5255, PDF_CMAP_SINGLE, 580 },
	{ 0x5257, 0x5258, PDF_CMAP_RANGE, 581 },
	{ 0x525a, 0x525a, PDF_CMAP_SINGLE, 583 },
	{ 0x525c, 0x525c, PDF_CMAP_SINGLE, 584 },
	{ 0x525f, 0x5261, PDF_CMAP_RANGE, 585 },
	{ 0x5266, 0x5266, PDF_CMAP_SINGLE, 588 },
	{ 0x526e, 0x526e, PDF_CMAP_SINGLE, 589 },
	{ 0x5277, 0x5279, PDF_CMAP_RANGE, 590 },
	{ 0x5280, 0x5280, PDF_CMAP_SINGLE, 593 },
	{ 0x5282, 0x5282, PDF_CMAP_SINGLE, 594 },
	{ 0x5285, 0x5285, PDF_CMAP_SINGLE, 595 },
	{ 0x528a, 0x528a, PDF_CMAP_SINGLE, 596 },
	{ 0x528c, 0x528c, PDF_CMAP_SINGLE, 597 },
	{ 0x5293, 0x5293, PDF_CMAP_SINGLE, 598 },
	{ 0x5295, 0x5298, PDF_CMAP_RANGE, 599 },
	{ 0x529a, 0x529a, PDF_CMAP_SINGLE, 603 },
	{ 0x529c, 0x529c, PDF_CMAP_SINGLE, 604 },
	{ 0x52a4, 0x52a7, PDF_CMAP_RANGE, 605 },
	{ 0x52af, 0x52b0, PDF_CMAP_RANGE, 609 },
	{ 0x52b6, 0x52b8, PDF_CMAP_RANGE, 611 },
	{ 0x52ba, 0x52bb, PDF_CMAP_RANGE, 614 },
	{ 0x52bd, 0x52bd, PDF_CMAP_SINGLE, 616 },
	{ 0x52c0, 0x52c0, PDF_CMAP_SINGLE, 617 },
	{ 0x52c4, 0x52c4, PDF_CMAP_SINGLE, 618 },
	{ 0x52c6, 0x52c6, PDF_CMAP_SINGLE, 619 },
	{ 0x52c8, 0x52c8, PDF_CMAP_SINGLE, 620 },
	{ 0x52cc, 0x52cc, PDF_CMAP_SINGLE, 621 },
	{ 0x52cf, 0x52cf, PDF_CMAP_SINGLE, 622 },
	{ 0x52d1, 0x52d1, PDF_CMAP_SINGLE, 623 },
	{ 0x52d4, 0x52d4, PDF_CMAP_SINGLE, 624 },
	{ 0x52d6, 0x52d6, PDF_CMAP_SINGLE, 625 },
	{ 0x52db, 0x52dc, PDF_CMAP_RANGE, 626 },
	{ 0x52e1, 0x52e1, PDF_CMAP_SINGLE, 628 },
	{ 0x52e5, 0x52e5, PDF_CMAP_SINGLE, 629 },
	{ 0x52e8, 0x52ea, PDF_CMAP_RANGE, 630 },
	{ 0x52ec, 0x52ec, PDF_CMAP_SINGLE, 633 },
	{ 0x52f0, 0x52f1, PDF_CMAP_RANGE, 634 },
	{ 0x52f4, 0x52f4, PDF_CMAP_SINGLE, 636 },
	{ 0x52f6, 0x52f7, PDF_CMAP_RANGE, 637 },
	{ 0x5300, 0x5300, PDF_CMAP_SINGLE, 639 },
	{ 0x5303, 0x5303, PDF_CMAP_SINGLE, 640 },
	{ 0x530a, 0x530c, PDF_CMAP_RANGE, 641 },
	{ 0x5311, 0x5311, PDF_CMAP_SINGLE, 644 },
	{ 0x5313, 0x5313, PDF_CMAP_SINGLE, 645 },
	{ 0x5318, 0x5318, PDF_CMAP_SINGLE, 646 },
	{ 0x531b, 0x531c, PDF_CMAP_RANGE, 647 },
	{ 0x531e, 0x531f, PDF_CMAP_RANGE, 649 },
	{ 0x5325, 0x5325, PDF_CMAP_SINGLE, 651 },
	{ 0x5327, 0x5329, PDF_CMAP_RANGE, 652 },
	{ 0x532b, 0x532d, PDF_CMAP_RANGE, 655 },
	{ 0x5330, 0x5330, PDF_CMAP_SINGLE, 658 },
	{ 0x5332, 0x5332, PDF_CMAP_SINGLE, 659 },
	{ 0x5335, 0x5335, PDF_CMAP_SINGLE, 660 },
	{ 0x533c, 0x533e, PDF_CMAP_RANGE, 661 },
	{ 0x5342, 0x5342, PDF_CMAP_SINGLE, 664 },
	{ 0x534b, 0x534c, PDF_CMAP_TABLE, 220 },
	{ 0x5359, 0x5359, PDF_CMAP_SINGLE, 667 },
	{ 0x535b, 0x535b, PDF_CMAP_SINGLE, 668 },
	{ 0x5361, 0x5361, PDF_CMAP_SINGLE, 669 },
	{ 0x5363, 0x5363, PDF_CMAP_SINGLE, 670 },
	{ 0x5365, 0x5365, PDF_CMAP_SINGLE, 671 },
	{ 0x536c, 0x536d, PDF_CMAP_RANGE, 672 },
	{ 0x5372, 0x5372, PDF_CMAP_SINGLE, 674 },
	{ 0x5379, 0x5379, PDF_CMAP_SINGLE, 675 },
	{ 0x537e, 0x537e, PDF_CMAP_SINGLE, 676 },
	{ 0x5383, 0x5383, PDF_CMAP_SINGLE, 677 },
	{ 0x5387, 0x5388, PDF_CMAP_RANGE, 678 },
	{ 0x538e, 0x538e, PDF_CMAP_SINGLE, 680 },
	{ 0x5393, 0x5394, PDF_CMAP_RANGE, 681 },
	{ 0x5399, 0x5399, PDF_CMAP_SINGLE, 683 },
	{ 0x539d, 0x539d, PDF_CMAP_SINGLE, 684 },
	{ 0x53a1, 0x53a1, PDF_CMAP_SINGLE, 685 },
	{ 0x53a4, 0x53a4, PDF_CMAP_SINGLE, 686 },
	{ 0x53aa, 0x53ab, PDF_CMAP_RANGE, 687 },
	{ 0x53af, 0x53af, PDF_CMAP_SINGLE, 689 },
	{ 0x53b2, 0x53b2, PDF_CMAP_SINGLE, 690 },
	{ 0x53b4, 0x53b5, PDF_CMAP_RANGE, 691 },
	{ 0x53b7, 0x53b8, PDF_CMAP_RANGE, 693 },
	{ 0x53ba, 0x53ba, PDF_CMAP_SINGLE, 695 },
	{ 0x53bd, 0x53bd, PDF_CMAP_SINGLE, 696 },
	{ 0x53c0, 0x53c0, PDF_CMAP_SINGLE, 697 },
	{ 0x53c5, 0x53c5, PDF_CMAP_SINGLE, 698 },
	{ 0x53cf, 0x53cf, PDF_CMAP_SINGLE, 699 },
	{ 0x53d2, 0x53d3, PDF_CMAP_RANGE, 700 },
	{ 0x53d5, 0x53d5, PDF_CMAP_SINGLE, 702 },
	{ 0x53da, 0x53da, PDF_CMAP_SINGLE, 703 },
	{ 0x53dd, 0x53de, PDF_CMAP_RANGE, 704 },
	{ 0x53e0, 0x53e0, PDF_CMAP_SINGLE, 706 },
	{ 0x53e6, 0x53e7, PDF_CMAP_RANGE, 707 },
	{ 0x53f5, 0x53f5, PDF_CMAP_SINGLE, 709 },
	{ 0x5402, 0x5402, PDF_CMAP_SINGLE, 710 },
	{ 0x5413, 0x5413, PDF_CMAP_SINGLE, 711 },
	{ 0x541a, 0x541a, PDF_CMAP_SINGLE, 712 },
	{ 0x5421, 0x5421, PDF_CMAP_SINGLE, 713 },
	{ 0x5427, 0x5428, PDF_CMAP_RANGE, 714 },
	{ 0x542a, 0x542a, PDF_CMAP_SINGLE, 716 },
	{ 0x542f, 0x542f, PDF_CMAP_SINGLE, 717 },
	{ 0x5431, 0x5431, PDF_CMAP_SINGLE, 718 },
	{ 0x5434, 0x5435, PDF_CMAP_RANGE, 719 },
	{ 0x5443, 0x5444, PDF_CMAP_RANGE, 721 },
	{ 0x5447, 0x5447, PDF_CMAP_SINGLE, 723 },
	{ 0x544d, 0x544d, PDF_CMAP_SINGLE, 724 },
	{ 0x544f, 0x544f, PDF_CMAP_SINGLE, 725 },
	{ 0x545e, 0x545e, PDF_CMAP_SINGLE, 726 },
	{ 0x5462, 0x5462, PDF_CMAP_SINGLE, 727 },
	{ 0x5464, 0x5464, PDF_CMAP_SINGLE, 728 },
	{ 0x5466, 0x5467, PDF_CMAP_RANGE, 729 },
	{ 0x5469, 0x5469, PDF_CMAP_SINGLE, 731 },
	{ 0x546b, 0x546b, PDF_CMAP_SINGLE, 732 },
	{ 0x546d, 0x546e, PDF_CMAP_RANGE, 733 },
	{ 0x5474, 0x5474, PDF_CMAP_SINGLE, 735 },
	{ 0x547f, 0x547f, PDF_CMAP_SINGLE, 736 },
	{ 0x5481, 0x5481, PDF_CMAP_SINGLE, 737 },
	{ 0x5483, 0x5483, PDF_CMAP_SINGLE, 738 },
	{ 0x5485, 0x5485, PDF_CMAP_SINGLE, 739 },
	{ 0x5488, 0x5489, PDF_CMAP_RANGE, 740 },
	{ 0x548d, 0x548d, PDF_CMAP_SINGLE, 742 },
	{ 0x5491, 0x5491, PDF_CMAP_SINGLE, 743 },
	{ 0x5495, 0x5496, PDF_CMAP_RANGE, 744 },
	{ 0x549c, 0x549c, PDF_CMAP_SINGLE, 746 },
	{ 0x549f, 0x549f, PDF_CMAP_SINGLE, 747 },
	{ 0x54a1, 0x54a1, PDF_CMAP_SINGLE, 748 },
	{ 0x54a6, 0x54a7, PDF_CMAP_RANGE, 749 },
	{ 0x54a9, 0x54aa, PDF_CMAP_RANGE, 751 },
	{ 0x54ad, 0x54ae, PDF_CMAP_RANGE, 753 },
	{ 0x54b1, 0x54b1, PDF_CMAP_SINGLE, 755 },
	{ 0x54b7, 0x54b7, PDF_CMAP_SINGLE, 756 },
	{ 0x54b9, 0x54bb, PDF_CMAP_RANGE, 757 },
	{ 0x54bf, 0x54bf, PDF_CMAP_SINGLE, 760 },
	{ 0x54c6, 0x54c6, PDF_CMAP_SINGLE, 761 },
	{ 0x54ca, 0x54ca, PDF_CMAP_SINGLE, 762 },
	{ 0x54cd, 0x54ce, PDF_CMAP_RANGE, 763 },
	{ 0x54e0, 0x54e0, PDF_CMAP_SINGLE, 765 },
	{ 0x54ea, 0x54ea, PDF_CMAP_SINGLE, 766 },
	{ 0x54ec, 0x54ec, PDF_CMAP_SINGLE, 767 },
	{ 0x54ef, 0x54ef, PDF_CMAP_SINGLE, 768 },
	{ 0x54f6, 0x54f6, PDF_CMAP_SINGLE, 769 },
	{ 0x54fc, 0x54fc, PDF_CMAP_SINGLE, 770 },
	{ 0x54fe, 0x5501, PDF_CMAP_RANGE, 771 },
	{ 0x5505, 0x5505, PDF_CMAP_SINGLE, 775 },
	{ 0x5508, 0x5509, PDF_CMAP_RANGE, 776 },
	{ 0x550c, 0x550e, PDF_CMAP_RANGE, 778 },
	{ 0x5515, 0x5515, PDF_CMAP_SINGLE, 781 },
	{ 0x552a, 0x552b, PDF_CMAP_RANGE, 782 },
	{ 0x5532, 0x5532, PDF_CMAP_SINGLE, 784 },
	{ 0x5535, 0x5536, PDF_CMAP_RANGE, 785 },
	{ 0x553b, 0x553d, PDF_CMAP_RANGE, 787 },
	{ 0x5541, 0x5541, PDF_CMAP_SINGLE, 790 },
	{ 0x5547, 0x5547, PDF_CMAP_SINGLE, 791 },
	{ 0x5549, 0x554a, PDF_CMAP_RANGE, 792 },
	{ 0x554d, 0x554d, PDF_CMAP_SINGLE, 794 },
	{ 0x5550, 0x5551, PDF_CMAP_RANGE, 795 },
	{ 0x5558, 0x5558, PDF_CMAP_SINGLE, 797 },
	{ 0x555a, 0x555b, PDF_CMAP_RANGE, 798 },
	{ 0x555e, 0x555e, PDF_CMAP_SINGLE, 800 },
	{ 0x5560, 0x5561, PDF_CMAP_RANGE, 801 },
	{ 0x5564, 0x5564, PDF_CMAP_SINGLE, 803 },
	{ 0x5566, 0x5566, PDF_CMAP_SINGLE, 804 },
	{ 0x557f, 0x557f, PDF_CMAP_SINGLE, 805 },
	{ 0x5581, 0x5582, PDF_CMAP_RANGE, 806 },
	{ 0x5586, 0x5586, PDF_CMAP_SINGLE, 808 },
	{ 0x5588, 0x5588, PDF_CMAP_SINGLE, 809 },
	{ 0x558e, 0x558f, PDF_CMAP_RANGE, 810 },
	{ 0x5591, 0x5594, PDF_CMAP_RANGE, 812 },
	{ 0x5597, 0x5597, PDF_CMAP_SINGLE, 816 },
	{ 0x55a3, 0x55a4, PDF_CMAP_RANGE, 817 },
	{ 0x55ad, 0x55ad, PDF_CMAP_SINGLE, 819 },
	{ 0x55b2, 0x55b2, PDF_CMAP_SINGLE, 820 },
	{ 0x55bf, 0x55bf, PDF_CMAP_SINGLE, 821 },
	{ 0x55c1, 0x55c1, PDF_CMAP_SINGLE, 822 },
	{ 0x55c3, 0x55c3, PDF_CMAP_SINGLE, 823 },
	{ 0x55c6, 0x55c6, PDF_CMAP_SINGLE, 824 },
	{ 0x55c9, 0x55c9, PDF_CMAP_SINGLE, 825 },
	{ 0x55cb, 0x55cc, PDF_CMAP_RANGE, 826 },
	{ 0x55ce, 0x55ce, PDF_CMAP_SINGLE, 828 },
	{ 0x55d1, 0x55d3, PDF_CMAP_RANGE, 829 },
	{ 0x55d7, 0x55d8, PDF_CMAP_RANGE, 832 },
	{ 0x55db, 0x55db, PDF_CMAP_SINGLE, 834 },
	{ 0x55de, 0x55de, PDF_CMAP_SINGLE, 835 },
	{ 0x55e2, 0x55e2, PDF_CMAP_SINGLE, 836 },
	{ 0x55e9, 0x55e9, PDF_CMAP_SINGLE, 837 },
	{ 0x55f6, 0x55f6, PDF_CMAP_SINGLE, 838 },
	{ 0x55ff, 0x55ff, PDF_CMAP_SINGLE, 839 },
	{ 0x5605, 0x5605, PDF_CMAP_SINGLE, 840 },
	{ 0x5608, 0x5608, PDF_CMAP_SINGLE, 841 },
	{ 0x560a, 0x560a, PDF_CMAP_SINGLE, 842 },
	{ 0x560d, 0x5612, PDF_CMAP_RANGE, 843 },
	{ 0x5619, 0x5619, PDF_CMAP_SINGLE, 849 },
	{ 0x562c, 0x562c, PDF_CMAP_SINGLE, 850 },
	{ 0x5630, 0x5630, PDF_CMAP_SINGLE, 851 },
	{ 0x5633, 0x5633, PDF_CMAP_SINGLE, 852 },
	{ 0x5635, 0x5635, PDF_CMAP_SINGLE, 853 },
	{ 0x5637, 0x5637, PDF_CMAP_SINGLE, 854 },
	{ 0x5639, 0x5639, PDF_CMAP_SINGLE, 855 },
	{ 0x563b, 0x563d, PDF_CMAP_RANGE, 856 },
	{ 0x563f, 0x5641, PDF_CMAP_RANGE, 859 },
	{ 0x5643, 0x5644, PDF_CMAP_RANGE, 862 },
	{ 0x5646, 0x5646, PDF_CMAP_SINGLE, 864 },
	{ 0x5649, 0x5649, PDF_CMAP_SINGLE, 865 },
	{ 0x564b, 0x564b, PDF_CMAP_SINGLE, 866 },
	{ 0x564d, 0x564d, PDF_CMAP_SINGLE, 867 },
	{ 0x564f, 0x564f, PDF_CMAP_SINGLE, 868 },
	{ 0x5654, 0x5654, PDF_CMAP_SINGLE, 869 },
	{ 0x565e, 0x565e, PDF_CMAP_SINGLE, 870 },
	{ 0x5660, 0x5663, PDF_CMAP_RANGE, 871 },
	{ 0x5666, 0x5666, PDF_CMAP_SINGLE, 875 },
	{ 0x5669, 0x5669, PDF_CMAP_SINGLE, 876 },
	{ 0x566d, 0x566d, PDF_CMAP_SINGLE, 877 },
	{ 0x566f, 0x566f, PDF_CMAP_SINGLE, 878 },
	{ 0x5671, 0x5672, PDF_CMAP_RANGE, 879 },
	{ 0x5675, 0x5675, PDF_CMAP_SINGLE, 881 },
	{ 0x5684, 0x5685, PDF_CMAP_RANGE, 882 },
	{ 0x5688, 0x5688, PDF_CMAP_SINGLE, 884 },
	{ 0x568b, 0x568c, PDF_CMAP_RANGE, 885 },
	{ 0x5695, 0x5695, PDF_CMAP_SINGLE, 887 },
	{ 0x5699, 0x569a, PDF_CMAP_RANGE, 888 },
	{ 0x569d, 0x569f, PDF_CMAP_RANGE, 890 },
	{ 0x56a6, 0x56a9, PDF_CMAP_RANGE, 893 },
	{ 0x56ab, 0x56ad, PDF_CMAP_RANGE, 897 },
	{ 0x56b1, 0x56b1, PDF_CMAP_SINGLE, 900 },
	{ 0x56b3, 0x56b3, PDF_CMAP_SINGLE, 901 },
	{ 0x56b7, 0x56b7, PDF_CMAP_SINGLE, 902 },
	{ 0x56be, 0x56be, PDF_CMAP_SINGLE, 903 },
	{ 0x56c5, 0x56c5, PDF_CMAP_SINGLE, 904 },
	{ 0x56c9, 0x56cb, PDF_CMAP_RANGE, 905 },
	{ 0x56cc, 0x56cd, PDF_CMAP_RANGE, 910 },
	{ 0x56cf, 0x56d0, PDF_CMAP_RANGE, 908 },
	{ 0x56d9, 0x56d9, PDF_CMAP_SINGLE, 912 },
	{ 0x56dc, 0x56dd, PDF_CMAP_RANGE, 913 },
	{ 0x56df, 0x56df, PDF_CMAP_SINGLE, 915 },
	{ 0x56e1, 0x56e1, PDF_CMAP_SINGLE, 916 },
	{ 0x56e4, 0x56e8, PDF_CMAP_RANGE, 917 },
	{ 0x56eb, 0x56eb, PDF_CMAP_SINGLE, 923 },
	{ 0x56ed, 0x56ed, PDF_CMAP_SINGLE, 924 },
	{ 0x56f1, 0x56f1, PDF_CMAP_SINGLE, 922 },
	{ 0x56f6, 0x56f7, PDF_CMAP_RANGE, 925 },
	{ 0x5701, 0x5702, PDF_CMAP_RANGE, 927 },
	{ 0x5707, 0x5707, PDF_CMAP_SINGLE, 929 },
	{ 0x570a, 0x570a, PDF_CMAP_SINGLE, 930 },
	{ 0x570c, 0x570c, PDF_CMAP_SINGLE, 931 },
	{ 0x5711, 0x5711, PDF_CMAP_SINGLE, 932 },
	{ 0x5715, 0x5715, PDF_CMAP_SINGLE, 933 },
	{ 0x571a, 0x571b, PDF_CMAP_RANGE, 934 },
	{ 0x571d, 0x571d, PDF_CMAP_SINGLE, 936 },
	{ 0x5720, 0x5720, PDF_CMAP_SINGLE, 937 },
	{ 0x5722, 0x5725, PDF_CMAP_RANGE, 938 },
	{ 0x5729, 0x572a, PDF_CMAP_RANGE, 942 },
	{ 0x572c, 0x572c, PDF_CMAP_SINGLE, 944 },
	{ 0x572e, 0x572f, PDF_CMAP_RANGE, 945 },
	{ 0x5733, 0x5734, PDF_CMAP_RANGE, 947 },
	{ 0x573d, 0x573f, PDF_CMAP_RANGE, 949 },
	{ 0x5745, 0x5746, PDF_CMAP_RANGE, 952 },
	{ 0x574c, 0x574d, PDF_CMAP_RANGE, 954 },
	{ 0x5752, 0x5752, PDF_CMAP_SINGLE, 956 },
	{ 0x5762, 0x5762, PDF_CMAP_SINGLE, 957 },
	{ 0x5765, 0x5765, PDF_CMAP_SINGLE, 958 },
	{ 0x5767, 0x5768, PDF_CMAP_RANGE, 959 },
	{ 0x576b, 0x576b, PDF_CMAP_SINGLE, 961 },
	{ 0x576d, 0x5771, PDF_CMAP_RANGE, 962 },
	{ 0x5773, 0x5775, PDF_CMAP_RANGE, 967 },
	{ 0x5777, 0x5777, PDF_CMAP_SINGLE, 970 },
	{ 0x5779, 0x577c, PDF_CMAP_RANGE, 971 },
	{ 0x577e, 0x577e, PDF_CMAP_SINGLE, 975 },
	{ 0x5781, 0x5781, PDF_CMAP_SINGLE, 976 },
	{ 0x5783, 0x5783, PDF_CMAP_SINGLE, 977 },
	{ 0x578c, 0x578c, PDF_CMAP_SINGLE, 978 },
	{ 0x5794, 0x5795, PDF_CMAP_TABLE, 222 },
	{ 0x5797, 0x5797, PDF_CMAP_SINGLE, 980 },
	{ 0x5799, 0x579a, PDF_CMAP_RANGE, 981 },
	{ 0x579c, 0x579f, PDF_CMAP_RANGE, 983 },
	{ 0x57a1, 0x57a1, PDF_CMAP_SINGLE, 987 },
	{ 0x57a7, 0x57a9, PDF_CMAP_RANGE, 989 },
	{ 0x57ac, 0x57ac, PDF_CMAP_SINGLE, 992 },
	{ 0x57b8, 0x57b8, PDF_CMAP_SINGLE, 993 },
	{ 0x57bd, 0x57bd, PDF_CMAP_SINGLE, 994 },
	{ 0x57c7, 0x57c8, PDF_CMAP_RANGE, 995 },
	{ 0x57cc, 0x57cc, PDF_CMAP_SINGLE, 997 },
	{ 0x57cf, 0x57cf, PDF_CMAP_SINGLE, 998 },
	{ 0x57d5, 0x57d5, PDF_CMAP_SINGLE, 999 },
	{ 0x57dd, 0x57de, PDF_CMAP_RANGE, 1000 },
	{ 0x57e1, 0x57e1, PDF_CMAP_SINGLE, 1018 },
	{ 0x57e4, 0x57e4, PDF_CMAP_SINGLE, 1002 },
	{ 0x57e6, 0x57e7, PDF_CMAP_RANGE, 1003 },
	{ 0x57e9, 0x57e9, PDF_CMAP_SINGLE, 1005 },
	{ 0x57ed, 0x57ed, PDF_CMAP_SINGLE, 1006 },
	{ 0x57f0, 0x57f0, PDF_CMAP_SINGLE, 1007 },
	{ 0x57f5, 0x57f6, PDF_CMAP_RANGE, 1008 },
	{ 0x57f8, 0x57f8, PDF_CMAP_SINGLE, 1010 },
	{ 0x57fd, 0x57ff, PDF_CMAP_RANGE, 1011 },
	{ 0x5803, 0x5804, PDF_CMAP_RANGE, 1014 },
	{ 0x5808, 0x5809, PDF_CMAP_RANGE, 1016 },
	{ 0x580c, 0x580d, PDF_CMAP_RANGE, 1019 },
	{ 0x581b, 0x581b, PDF_CMAP_SINGLE, 1021 },
	{ 0x581e, 0x5820, PDF_CMAP_RANGE, 1022 },
	{ 0x5826, 0x5827, PDF_CMAP_RANGE, 1025 },
	{ 0x582d, 0x582d, PDF_CMAP_SINGLE, 1027 },
	{ 0x5832, 0x5832, PDF_CMAP_SINGLE, 1028 },
	{ 0x5839, 0x5839, PDF_CMAP_SINGLE, 1029 },
	{ 0x583f, 0x583f, PDF_CMAP_SINGLE, 1030 },
	{ 0x5849, 0x5849, PDF_CMAP_SINGLE, 1031 },
	{ 0x584c, 0x584d, PDF_CMAP_RANGE, 1032 },
	{ 0x584f, 0x5850, PDF_CMAP_RANGE, 1034 },
	{ 0x5855, 0x5855, PDF_CMAP_SINGLE, 1036 },
	{ 0x585f, 0x585f, PDF_CMAP_SINGLE, 1037 },
	{ 0x5861, 0x5861, PDF_CMAP_SINGLE, 1038 },
	{ 0x5864, 0x5864, PDF_CMAP_SINGLE, 1039 },
	{ 0x5867, 0x5868, PDF_CMAP_RANGE, 1040 },
	{ 0x5878, 0x5878, PDF_CMAP_SINGLE, 1042 },
	{ 0x587c, 0x587c, PDF_CMAP_SINGLE, 1043 },
	{ 0x587f, 0x5881, PDF_CMAP_RANGE, 1044 },
	{ 0x5887, 0x588a, PDF_CMAP_RANGE, 1047 },
	{ 0x588c, 0x588d, PDF_CMAP_RANGE, 1051 },
	{ 0x588f, 0x5890, PDF_CMAP_RANGE, 1053 },
	{ 0x5894, 0x5894, PDF_CMAP_SINGLE, 1055 },
	{ 0x5896, 0x5896, PDF_CMAP_SINGLE, 1056 },
	{ 0x589d, 0x589d, PDF_CMAP_SINGLE, 1057 },
	{ 0x58a0, 0x58a2, PDF_CMAP_RANGE, 1058 },
	{ 0x58a6, 0x58a6, PDF_CMAP_SINGLE, 1061 },
	{ 0x58a9, 0x58a9, PDF_CMAP_SINGLE, 1062 },
	{ 0x58b1, 0x58b2, PDF_CMAP_RANGE, 1063 },
	{ 0x58bc, 0x58bc, PDF_CMAP_SINGLE, 1066 },
	{ 0x58c2, 0x58c2, PDF_CMAP_SINGLE, 1067 },
	{ 0x58c4, 0x58c4, PDF_CMAP_SINGLE, 1065 },
	{ 0x58c8, 0x58c8, PDF_CMAP_SINGLE, 1068 },
	{ 0x58cd, 0x58ce, PDF_CMAP_RANGE, 1069 },
	{ 0x58d0, 0x58d0, PDF_CMAP_SINGLE, 1071 },
	{ 0x58d2, 0x58d2, PDF_CMAP_SINGLE, 1072 },
	{ 0x58d4, 0x58d4, PDF_CMAP_SINGLE, 1073 },
	{ 0x58d6, 0x58d6, PDF_CMAP_SINGLE, 1074 },
	{ 0x58da, 0x58da, PDF_CMAP_SINGLE, 1075 },
	{ 0x58dd, 0x58dd, PDF_CMAP_SINGLE, 1076 },
	{ 0x58e1, 0x58e2, PDF_CMAP_RANGE, 1077 },
	{ 0x58e9, 0x58e9, PDF_CMAP_SINGLE, 1079 },
	{ 0x58f3, 0x58f3, PDF_CMAP_SINGLE, 1080 },
	{ 0x5905, 0x5906, PDF_CMAP_RANGE, 1081 },
	{ 0x590b, 0x590c, PDF_CMAP_RANGE, 1083 },
	{ 0x5912, 0x5914, PDF_CMAP_RANGE, 1085 },
	{ 0x591d, 0x591d, PDF_CMAP_SINGLE, 1089 },
	{ 0x5921, 0x5921, PDF_CMAP_SINGLE, 1090 },
	{ 0x5923, 0x5924, PDF_CMAP_RANGE, 1091 },
	{ 0x5928, 0x5928, PDF_CMAP_SINGLE, 1093 },
	{ 0x592f, 0x5930, PDF_CMAP_RANGE, 1094 },
	{ 0x5933, 0x5933, PDF_CMAP_SINGLE, 1096 },
	{ 0x5935, 0x5936, PDF_CMAP_RANGE, 1097 },
	{ 0x593f, 0x593f, PDF_CMAP_SINGLE, 1099 },
	{ 0x5943, 0x5943, PDF_CMAP_SINGLE, 1100 },
	{ 0x5946, 0x5946, PDF_CMAP_SINGLE, 1101 },
	{ 0x5952, 0x5953, PDF_CMAP_RANGE, 1102 },
	{ 0x5959, 0x5959, PDF_CMAP_SINGLE, 1104 },
	{ 0x595b, 0x595b, PDF_CMAP_SINGLE, 1105 },
	{ 0x595d, 0x595f, PDF_CMAP_RANGE, 1106 },
	{ 0x5961, 0x5961, PDF_CMAP_SINGLE, 1109 },
	{ 0x5963, 0x5963, PDF_CMAP_SINGLE, 1110 },
	{ 0x596b, 0x596b, PDF_CMAP_SINGLE, 1111 },
	{ 0x596d, 0x596d, PDF_CMAP_SINGLE, 1112 },
	{ 0x596f, 0x596f, PDF_CMAP_SINGLE, 1113 },
	{ 0x5972, 0x5972, PDF_CMAP_SINGLE, 1114 },
	{ 0x5975, 0x5976, PDF_CMAP_RANGE, 1115 },
	{ 0x5979, 0x5979, PDF_CMAP_SINGLE, 1117 },
	{ 0x597b, 0x597c, PDF_CMAP_RANGE, 1118 },
	{ 0x598b, 0x598c, PDF_CMAP_RANGE, 1120 },
	{ 0x598e, 0x598e, PDF_CMAP_SINGLE, 1122 },
	{ 0x5992, 0x5992, PDF_CMAP_SINGLE, 1123 },
	{ 0x5995, 0x5995, PDF_CMAP_SINGLE, 1124 },
	{ 0x5997, 0x5997, PDF_CMAP_SINGLE, 1125 },
	{ 0x599f, 0x599f, PDF_CMAP_SINGLE, 1126 },
	{ 0x59a4, 0x59a4, PDF_CMAP_SINGLE, 1127 },
	{ 0x59a7, 0x59a7, PDF_CMAP_SINGLE, 1128 },
	{ 0x59ad, 0x59b0, PDF_CMAP_RANGE, 1129 },
	{ 0x59b3, 0x59b3, PDF_CMAP_SINGLE, 1133 },
	{ 0x59b7, 0x59b7, PDF_CMAP_SINGLE, 1134 },
	{ 0x59ba, 0x59ba, PDF_CMAP_SINGLE, 1135 },
	{ 0x59bc, 0x59bc, PDF_CMAP_SINGLE, 1136 },
	{ 0x59c1, 0x59c1, PDF_CMAP_SINGLE, 1137 },
	{ 0x59c3, 0x59c4, PDF_CMAP_RANGE, 1138 },
	{ 0x59c8, 0x59c8, PDF_CMAP_SINGLE, 1140 },
	{ 0x59ca, 0x59ca, PDF_CMAP_SINGLE, 1141 },
	{ 0x59cd, 0x59cd, PDF_CMAP_SINGLE, 1142 },
	{ 0x59d2, 0x59d2, PDF_CMAP_SINGLE, 1143 },
	{ 0x59dd, 0x59df, PDF_CMAP_RANGE, 1144 },
	{ 0x59e3, 0x59e4, PDF_CMAP_RANGE, 1147 },
	{ 0x59e7, 0x59e7, PDF_CMAP_SINGLE, 1149 },
	{ 0x59ee, 0x59ef, PDF_CMAP_RANGE, 1150 },
	{ 0x59f1, 0x59f2, PDF_CMAP_RANGE, 1152 },
	{ 0x59f4, 0x59f4, PDF_CMAP_SINGLE, 1154 },
	{ 0x59f7, 0x59f7, PDF_CMAP_SINGLE, 1155 },
	{ 0x5a00, 0x5a00, PDF_CMAP_SINGLE, 1156 },
	{ 0x5a04, 0x5a04, PDF_CMAP_SINGLE, 1157 },
	{ 0x5a0c, 0x5a0e, PDF_CMAP_RANGE, 1158 },
	{ 0x5a12, 0x5a13, PDF_CMAP_RANGE, 1161 },
	{ 0x5a1e, 0x5a1e, PDF_CMAP_SINGLE, 1163 },
	{ 0x5a23, 0x5a24, PDF_CMAP_RANGE, 1164 },
	{ 0x5a27, 0x5a28, PDF_CMAP_RANGE, 1166 },
	{ 0x5a2a, 0x5a2a, PDF_CMAP_SINGLE, 1168 },
	{ 0x5a2d, 0x5a2d, PDF_CMAP_SINGLE, 1169 },
	{ 0x5a30, 0x5a30, PDF_CMAP_SINGLE, 1170 },
	{ 0x5a44, 0x5a45, PDF_CMAP_RANGE, 1171 },
	{ 0x5a47, 0x5a48, PDF_CMAP_RANGE, 1173 },
	{ 0x5a4c, 0x5a4c, PDF_CMAP_SINGLE, 1175 },
	{ 0x5a50, 0x5a50, PDF_CMAP_SINGLE, 1176 },
	{ 0x5a55, 0x5a55, PDF_CMAP_SINGLE, 1177 },
	{ 0x5a5e, 0x5a5e, PDF_CMAP_SINGLE, 1178 },
	{ 0x5a63, 0x5a63, PDF_CMAP_SINGLE, 1179 },
	{ 0x5a65, 0x5a65, PDF_CMAP_SINGLE, 1180 },
	{ 0x5a67, 0x5a67, PDF_CMAP_SINGLE, 1181 },
	{ 0x5a6d, 0x5a6d, PDF_CMAP_SINGLE, 1182 },
	{ 0x5a77, 0x5a77, PDF_CMAP_SINGLE, 1183 },
	{ 0x5a7a, 0x5a7b, PDF_CMAP_RANGE, 1184 },
	{ 0x5a7e, 0x5a7e, PDF_CMAP_SINGLE, 1186 },
	{ 0x5a8b, 0x5a8b, PDF_CMAP_SINGLE, 1187 },
	{ 0x5a90, 0x5a90, PDF_CMAP_SINGLE, 1188 },
	{ 0x5a93, 0x5a93, PDF_CMAP_SINGLE, 1189 },
	{ 0x5a96, 0x5a96, PDF_CMAP_SINGLE, 1190 },
	{ 0x5a99, 0x5a99, PDF_CMAP_SINGLE, 1191 },
	{ 0x5a9c, 0x5a9c, PDF_CMAP_SINGLE, 1192 },
	{ 0x5a9e, 0x5aa0, PDF_CMAP_RANGE, 1193 },
	{ 0x5aa2, 0x5aa2, PDF_CMAP_SINGLE, 1196 },
	{ 0x5aa7, 0x5aa7, PDF_CMAP_SINGLE, 1197 },
	{ 0x5aac, 0x5aac, PDF_CMAP_SINGLE, 1198 },
	{ 0x5ab1, 0x5ab3, PDF_CMAP_RANGE, 1199 },
	{ 0x5ab5, 0x5ab5, PDF_CMAP_SINGLE, 1202 },
	{ 0x5ab8, 0x5ab8, PDF_CMAP_SINGLE, 1203 },
	{ 0x5aba, 0x5abb, PDF_CMAP_RANGE, 1204 },
	{ 0x5abf, 0x5abf, PDF_CMAP_SINGLE, 1206 },
	{ 0x5ac4, 0x5ac4, PDF_CMAP_SINGLE, 1207 },
	{ 0x5ac6, 0x5ac6, PDF_CMAP_SINGLE, 1208 },
	{ 0x5ac8, 0x5ac8, PDF_CMAP_SINGLE, 1209 },
	{ 0x5acf, 0x5acf, PDF_CMAP_SINGLE, 1210 },
	{ 0x5ada, 0x5ada, PDF_CMAP_SINGLE, 1211 },
	{ 0x5adc, 0x5adc, PDF_CMAP_SINGLE, 1212 },
	{ 0x5ae0, 0x5ae0, PDF_CMAP_SINGLE, 1213 },
	{ 0x5ae5, 0x5ae5, PDF_CMAP_SINGLE, 1214 },
	{ 0x5aea, 0x5aea, PDF_CMAP_SINGLE, 1215 },
	{ 0x5aee, 0x5aee, PDF_CMAP_SINGLE, 1216 },
	{ 0x5af5, 0x5af6, PDF_CMAP_RANGE, 1217 },
	{ 0x5afd, 0x5afd, PDF_CMAP_SINGLE, 1219 },
	{ 0x5b00, 0x5b01, PDF_CMAP_RANGE, 1220 },
	{ 0x5b08, 0x5b08, PDF_CMAP_SINGLE, 1222 },
	{ 0x5b17, 0x5b17, PDF_CMAP_SINGLE, 1223 },
	{ 0x5b19, 0x5b19, PDF_CMAP_SINGLE, 1225 },
	{ 0x5b1b, 0x5b1b, PDF_CMAP_SINGLE, 1226 },
	{ 0x5b1d, 0x5b1d, PDF_CMAP_SINGLE, 1227 },
	{ 0x5b21, 0x5b21, PDF_CMAP_SINGLE, 1228 },
	{ 0x5b25, 0x5b25, PDF_CMAP_SINGLE, 1229 },
	{ 0x5b2d, 0x5b2d, PDF_CMAP_SINGLE, 1230 },
	{ 0x5b34, 0x5b34, PDF_CMAP_SINGLE, 1224 },
	{ 0x5b38, 0x5b38, PDF_CMAP_SINGLE, 1231 },
	{ 0x5b41, 0x5b41, PDF_CMAP_SINGLE, 1232 },
	{ 0x5b4b, 0x5b4c, PDF_CMAP_RANGE, 1233 },
	{ 0x5b52, 0x5b52, PDF_CMAP_SINGLE, 1235 },
	{ 0x5b56, 0x5b56, PDF_CMAP_SINGLE, 1236 },
	{ 0x5b5e, 0x5b5e, PDF_CMAP_SINGLE, 1237 },
	{ 0x5b68, 0x5b68, PDF_CMAP_SINGLE, 1238 },
	{ 0x5b6e, 0x5b6f, PDF_CMAP_RANGE, 1239 },
	{ 0x5b7c, 0x5b7f, PDF_CMAP_RANGE, 1241 },
	{ 0x5b81, 0x5b81, PDF_CMAP_SINGLE, 1245 },
	{ 0x5b84, 0x5b84, PDF_CMAP_SINGLE, 1246 },
	{ 0x5b86, 0x5b86, PDF_CMAP_SINGLE, 1247 },
	{ 0x5b8a, 0x5b8a, PDF_CMAP_SINGLE, 1248 },
	{ 0x5b8e, 0x5b8e, PDF_CMAP_SINGLE, 1249 },
	{ 0x5b90, 0x5b91, PDF_CMAP_RANGE, 1250 },
	{ 0x5b93, 0x5b94, PDF_CMAP_RANGE, 1252 },
	{ 0x5b96, 0x5b96, PDF_CMAP_SINGLE, 1254 },
	{ 0x5ba8, 0x5ba9, PDF_CMAP_RANGE, 1255 },
	{ 0x5bac, 0x5bad, PDF_CMAP_RANGE, 1257 },
	{ 0x5baf, 0x5baf, PDF_CMAP_SINGLE, 1259 },
	{ 0x5bb1, 0x5bb2, PDF_CMAP_RANGE, 1260 },
	{ 0x5bb7, 0x5bb7, PDF_CMAP_SINGLE, 1262 },
	{ 0x5bba, 0x5bba, PDF_CMAP_SINGLE, 1263 },
	{ 0x5bbc, 0x5bbc, PDF_CMAP_SINGLE, 1264 },
	{ 0x5bc0, 0x5bc1, PDF_CMAP_RANGE, 1265 },
	{ 0x5bcd, 0x5bcd, PDF_CMAP_SINGLE, 1267 },
	{ 0x5bcf, 0x5bcf, PDF_CMAP_SINGLE, 1268 },
	{ 0x5bd6, 0x5bda, PDF_CMAP_RANGE, 1269 },
	{ 0x5be0, 0x5be0, PDF_CMAP_SINGLE, 1274 },
	{ 0x5bef, 0x5bef, PDF_CMAP_SINGLE, 1275 },
	{ 0x5bf1, 0x5bf1, PDF_CMAP_SINGLE, 1276 },
	{ 0x5bf4, 0x5bf4, PDF_CMAP_SINGLE, 1277 },
	{ 0x5bfd, 0x5bfd, PDF_CMAP_SINGLE, 1278 },
	{ 0x5c0c, 0x5c0c, PDF_CMAP_SINGLE, 1279 },
	{ 0x5c17, 0x5c17, PDF_CMAP_SINGLE, 1280 },
	{ 0x5c1e, 0x5c1f, PDF_CMAP_RANGE, 1281 },
	{ 0x5c23, 0x5c23, PDF_CMAP_SINGLE, 1283 },
	{ 0x5c26, 0x5c26, PDF_CMAP_SINGLE, 1284 },
	{ 0x5c29, 0x5c29, PDF_CMAP_SINGLE, 1285 },
	{ 0x5c2b, 0x5c2c, PDF_CMAP_RANGE, 1286 },
	{ 0x5c2e, 0x5c2e, PDF_CMAP_SINGLE, 1288 },
	{ 0x5c30, 0x5c30, PDF_CMAP_SINGLE, 1289 },
	{ 0x5c32, 0x5c32, PDF_CMAP_SINGLE, 1290 },
	{ 0x5c35, 0x5c36, PDF_CMAP_RANGE, 1291 },
	{ 0x5c59, 0x5c5a, PDF_CMAP_RANGE, 1293 },
	{ 0x5c5c, 0x5c5c, PDF_CMAP_SINGLE, 1295 },
	{ 0x5c62, 0x5c63, PDF_CMAP_RANGE, 1296 },
	{ 0x5c67, 0x5c69, PDF_CMAP_RANGE, 1298 },
	{ 0x5c6d, 0x5c6d, PDF_CMAP_SINGLE, 1301 },
	{ 0x5c70, 0x5c70, PDF_CMAP_SINGLE, 1302 },
	{ 0x5c74, 0x5c75, PDF_CMAP_RANGE, 1303 },
	{ 0x5c7a, 0x5c7d, PDF_CMAP_RANGE, 1305 },
	{ 0x5c87, 0x5c88, PDF_CMAP_RANGE, 1309 },
	{ 0x5c8a, 0x5c8a, PDF_CMAP_SINGLE, 1311 },
	{ 0x5c8f, 0x5c8f, PDF_CMAP_SINGLE, 1312 },
	{ 0x5c92, 0x5c92, PDF_CMAP_SINGLE, 1313 },
	{ 0x5c9d, 0x5c9d, PDF_CMAP_SINGLE, 1314 },
	{ 0x5c9f, 0x5ca0, PDF_CMAP_RANGE, 1315 },
	{ 0x5ca2, 0x5ca3, PDF_CMAP_RANGE, 1317 },
	{ 0x5ca6, 0x5ca6, PDF_CMAP_SINGLE, 1319 },
	{ 0x5caa, 0x5caa, PDF_CMAP_SINGLE, 1320 },
	{ 0x5cb2, 0x5cb2, PDF_CMAP_SINGLE, 1321 },
	{ 0x5cb4, 0x5cb5, PDF_CMAP_RANGE, 1322 },
	{ 0x5cba, 0x5cba, PDF_CMAP_SINGLE, 1324 },
	{ 0x5cc9, 0x5cc9, PDF_CMAP_SINGLE, 1325 },
	{ 0x5ccb, 0x5ccb, PDF_CMAP_SINGLE, 1326 },
	{ 0x5cd2, 0x5cd2, PDF_CMAP_SINGLE, 1327 },
	{ 0x5cd7, 0x5cd7, PDF_CMAP_SINGLE, 1329 },
	{ 0x5cdd, 0x5cdd, PDF_CMAP_SINGLE, 1328 },
	{ 0x5cee, 0x5cee, PDF_CMAP_SINGLE, 1330 },
	{ 0x5cf1, 0x5cf2, PDF_CMAP_RANGE, 1331 },
	{ 0x5cf4, 0x5cf4, PDF_CMAP_SINGLE, 1333 },
	{ 0x5d01, 0x5d01, PDF_CMAP_SINGLE, 1334 },
	{ 0x5d06, 0x5d06, PDF_CMAP_SINGLE, 1335 },
	{ 0x5d0d, 0x5d0d, PDF_CMAP_SINGLE, 1336 },
	{ 0x5d12, 0x5d12, PDF_CMAP_SINGLE, 1337 },
	{ 0x5d23, 0x5d24, PDF_CMAP_RANGE, 1339 },
	{ 0x5d26, 0x5d27, PDF_CMAP_RANGE, 1341 },
	{ 0x5d2b, 0x5d2b, PDF_CMAP_SINGLE, 1338 },
	{ 0x5d31, 0x5d31, PDF_CMAP_SINGLE, 1343 },
	{ 0x5d34, 0x5d34, PDF_CMAP_SINGLE, 1344 },
	{ 0x5d39, 0x5d39, PDF_CMAP_SINGLE, 1345 },
	{ 0x5d3d, 0x5d3d, PDF_CMAP_SINGLE, 1346 },
	{ 0x5d3f, 0x5d3f, PDF_CMAP_SINGLE, 1347 },
	{ 0x5d42, 0x5d43, PDF_CMAP_RANGE, 1348 },
	{ 0x5d46, 0x5d46, PDF_CMAP_SINGLE, 1350 },
	{ 0x5d48, 0x5d48, PDF_CMAP_SINGLE, 1351 },
	{ 0x5d4a, 0x5d4a, PDF_CMAP_SINGLE, 1355 },
	{ 0x5d51, 0x5d51, PDF_CMAP_SINGLE, 1353 },
	{ 0x5d55, 0x5d55, PDF_CMAP_SINGLE, 1352 },
	{ 0x5d59, 0x5d59, PDF_CMAP_SINGLE, 1354 },
	{ 0x5d5f, 0x5d62, PDF_CMAP_RANGE, 1356 },
	{ 0x5d64, 0x5d64, PDF_CMAP_SINGLE, 1360 },
	{ 0x5d6a, 0x5d6a, PDF_CMAP_SINGLE, 1361 },
	{ 0x5d6d, 0x5d6d, PDF_CMAP_SINGLE, 1362 },
	{ 0x5d70, 0x5d70, PDF_CMAP_SINGLE, 1363 },
	{ 0x5d79, 0x5d7a, PDF_CMAP_RANGE, 1364 },
	{ 0x5d7e, 0x5d7f, PDF_CMAP_RANGE, 1366 },
	{ 0x5d81, 0x5d81, PDF_CMAP_SINGLE, 1368 },
	{ 0x5d83, 0x5d83, PDF_CMAP_SINGLE, 1369 },
	{ 0x5d88, 0x5d88, PDF_CMAP_SINGLE, 1370 },
	{ 0x5d8a, 0x5d8a, PDF_CMAP_SINGLE, 1371 },
	{ 0x5d92, 0x5d95, PDF_CMAP_RANGE, 1372 },
	{ 0x5d99, 0x5d99, PDF_CMAP_SINGLE, 1376 },
	{ 0x5d9b, 0x5d9b, PDF_CMAP_SINGLE, 1377 },
	{ 0x5d9f, 0x5da0, PDF_CMAP_RANGE, 1378 },
	{ 0x5da7, 0x5da7, PDF_CMAP_SINGLE, 1380 },
	{ 0x5dab, 0x5dab, PDF_CMAP_SINGLE, 1381 },
	{ 0x5db0, 0x5db0, PDF_CMAP_SINGLE, 1382 },
	{ 0x5db2, 0x5db2, PDF_CMAP_SINGLE, 5426 },
	{ 0x5db4, 0x5db4, PDF_CMAP_SINGLE, 1383 },
	{ 0x5db8, 0x5db9, PDF_CMAP_RANGE, 1384 },
	{ 0x5dc3, 0x5dc3, PDF_CMAP_SINGLE, 1386 },
	{ 0x5dc7, 0x5dc7, PDF_CMAP_SINGLE, 1387 },
	{ 0x5dcb, 0x5dcb, PDF_CMAP_SINGLE, 1388 },
	{ 0x5dce, 0x5dce, PDF_CMAP_SINGLE, 1390 },
	{ 0x5dd0, 0x5dd0, PDF_CMAP_SINGLE, 1389 },
	{ 0x5dd8, 0x5dd9, PDF_CMAP_RANGE, 1391 },
	{ 0x5de0, 0x5de0, PDF_CMAP_SINGLE, 1393 },
	{ 0x5de4, 0x5de4, PDF_CMAP_SINGLE, 1394 },
	{ 0x5de9, 0x5de9, PDF_CMAP_SINGLE, 1395 },
	{ 0x5df8, 0x5df9, PDF_CMAP_RANGE, 1396 },
	{ 0x5e00, 0x5e00, PDF_CMAP_SINGLE, 1398 },
	{ 0x5e07, 0x5e07, PDF_CMAP_SINGLE, 1399 },
	{ 0x5e0d, 0x5e0d, PDF_CMAP_SINGLE, 1400 },
	{ 0x5e12, 0x5e12, PDF_CMAP_SINGLE, 1401 },
	{ 0x5e14, 0x5e15, PDF_CMAP_RANGE, 1402 },
	{ 0x5e18, 0x5e18, PDF_CMAP_SINGLE, 1404 },
	{ 0x5e1f, 0x5e20, PDF_CMAP_RANGE, 1405 },
	{ 0x5e28, 0x5e28, PDF_CMAP_SINGLE, 1408 },
	{ 0x5e2e, 0x5e2e, PDF_CMAP_SINGLE, 1407 },
	{ 0x5e32, 0x5e32, PDF_CMAP_SINGLE, 1409 },
	{ 0x5e35, 0x5e35, PDF_CMAP_SINGLE, 1410 },
	{ 0x5e3e, 0x5e3e, PDF_CMAP_SINGLE, 1411 },
	{ 0x5e49, 0x5e49, PDF_CMAP_SINGLE, 1414 },
	{ 0x5e4b, 0x5e4b, PDF_CMAP_SINGLE, 1412 },
	{ 0x5e50, 0x5e51, PDF_CMAP_TABLE, 224 },
	{ 0x5e56, 0x5e56, PDF_CMAP_SINGLE, 1416 },
	{ 0x5e58, 0x5e58, PDF_CMAP_SINGLE, 1417 },
	{ 0x5e5b, 0x5e5c, PDF_CMAP_RANGE, 1418 },
	{ 0x5e5e, 0x5e5e, PDF_CMAP_SINGLE, 1420 },
	{ 0x5e68, 0x5e68, PDF_CMAP_SINGLE, 1421 },
	{ 0x5e6a, 0x5e6e, PDF_CMAP_RANGE, 1422 },
	{ 0x5e70, 0x5e70, PDF_CMAP_SINGLE, 1427 },
	{ 0x5e80, 0x5e80, PDF_CMAP_SINGLE, 1428 },
	{ 0x5e8b, 0x5e8b, PDF_CMAP_SINGLE, 1429 },
	{ 0x5e8e, 0x5e8e, PDF_CMAP_SINGLE, 1430 },
	{ 0x5ea2, 0x5ea2, PDF_CMAP_SINGLE, 1431 },
	{ 0x5ea4, 0x5ea5, PDF_CMAP_RANGE, 1432 },
	{ 0x5ea8, 0x5ea8, PDF_CMAP_SINGLE, 1434 },
	{ 0x5eaa, 0x5eaa, PDF_CMAP_SINGLE, 1435 },
	{ 0x5eac, 0x5eac, PDF_CMAP_SINGLE, 1436 },
	{ 0x5eb1, 0x5eb1, PDF_CMAP_SINGLE, 1437 },
	{ 0x5eb3, 0x5eb3, PDF_CMAP_SINGLE, 1438 },
	{ 0x5ebd, 0x5ebf, PDF_CMAP_RANGE, 1439 },
	{ 0x5ec6, 0x5ec6, PDF_CMAP_SINGLE, 1442 },
	{ 0x5ecb, 0x5ecc, PDF_CMAP_TABLE, 226 },
	{ 0x5ece, 0x5ece, PDF_CMAP_SINGLE, 1445 },
	{ 0x5ed1, 0x5ed2, PDF_CMAP_RANGE, 1446 },
	{ 0x5ed4, 0x5ed5, PDF_CMAP_RANGE, 1448 },
	{ 0x5edc, 0x5edc, PDF_CMAP_SINGLE, 1450 },
	{ 0x5ede, 0x5ede, PDF_CMAP_SINGLE, 1451 },
	{ 0x5ee5, 0x5ee5, PDF_CMAP_SINGLE, 1452 },
	{ 0x5eeb, 0x5eeb, PDF_CMAP_SINGLE, 1453 },
	{ 0x5f02, 0x5f02, PDF_CMAP_SINGLE, 1454 },
	{ 0x5f06, 0x5f08, PDF_CMAP_RANGE, 1455 },
	{ 0x5f0e, 0x5f0e, PDF_CMAP_SINGLE, 1458 },
	{ 0x5f19, 0x5f19, PDF_CMAP_SINGLE, 1459 },
	{ 0x5f1c, 0x5f1d, PDF_CMAP_RANGE, 1460 },
	{ 0x5f21, 0x5f24, PDF_CMAP_RANGE, 1462 },
	{ 0x5f28, 0x5f28, PDF_CMAP_SINGLE, 1466 },
	{ 0x5f2b, 0x5f2c, PDF_CMAP_RANGE, 1467 },
	{ 0x5f2e, 0x5f2e, PDF_CMAP_SINGLE, 1469 },
	{ 0x5f30, 0x5f30, PDF_CMAP_SINGLE, 1470 },
	{ 0x5f34, 0x5f34, PDF_CMAP_SINGLE, 1471 },
	{ 0x5f36, 0x5f36, PDF_CMAP_SINGLE, 1472 },
	{ 0x5f3b, 0x5f3b, PDF_CMAP_SINGLE, 1473 },
	{ 0x5f3d, 0x5f3d, PDF_CMAP_SINGLE, 1474 },
	{ 0x5f3f, 0x5f40, PDF_CMAP_RANGE, 1475 },
	{ 0x5f44, 0x5f45, PDF_CMAP_RANGE, 1477 },
	{ 0x5f47, 0x5f47, PDF_CMAP_SINGLE, 1479 },
	{ 0x5f4d, 0x5f4d, PDF_CMAP_SINGLE, 1480 },
	{ 0x5f50, 0x5f50, PDF_CMAP_SINGLE, 1481 },
	{ 0x5f54, 0x5f54, PDF_CMAP_SINGLE, 1482 },
	{ 0x5f58, 0x5f58, PDF_CMAP_SINGLE, 1483 },
	{ 0x5f5b, 0x5f5b, PDF_CMAP_SINGLE, 1484 },
	{ 0x5f60, 0x5f60, PDF_CMAP_SINGLE, 1485 },
	{ 0x5f63, 0x5f64, PDF_CMAP_RANGE, 1486 },
	{ 0x5f67, 0x5f67, PDF_CMAP_SINGLE, 1488 },
	{ 0x5f6f, 0x5f6f, PDF_CMAP_SINGLE, 1489 },
	{ 0x5f72, 0x5f72, PDF_CMAP_SINGLE, 1490 },
	{ 0x5f74, 0x5f75, PDF_CMAP_RANGE, 1491 },
	{ 0x5f78, 0x5f78, PDF_CMAP_SINGLE, 1493 },
	{ 0x5f7a, 0x5f7a, PDF_CMAP_SINGLE, 1494 },
	{ 0x5f7d, 0x5f7e, PDF_CMAP_RANGE, 1495 },
	{ 0x5f89, 0x5f89, PDF_CMAP_SINGLE, 1497 },
	{ 0x5f8d, 0x5f8d, PDF_CMAP_SINGLE, 1498 },
	{ 0x5f8f, 0x5f8f, PDF_CMAP_SINGLE, 1499 },
	{ 0x5f96, 0x5f96, PDF_CMAP_SINGLE, 1500 },
	{ 0x5f9c, 0x5f9d, PDF_CMAP_RANGE, 1501 },
	{ 0x5fa2, 0x5fa2, PDF_CMAP_SINGLE, 1503 },
	{ 0x5fa4, 0x5fa4, PDF_CMAP_SINGLE, 1506 },
	{ 0x5fa7, 0x5fa7, PDF_CMAP_SINGLE, 1504 },
	{ 0x5fab, 0x5fac, PDF_CMAP_TABLE, 228 },
	{ 0x5faf, 0x5fb1, PDF_CMAP_RANGE, 1508 },
	{ 0x5fb8, 0x5fb8, PDF_CMAP_SINGLE, 1511 },
	{ 0x5fc4, 0x5fc4, PDF_CMAP_SINGLE, 1512 },
	{ 0x5fc7, 0x5fc9, PDF_CMAP_RANGE, 1513 },
	{ 0x5fcb, 0x5fcb, PDF_CMAP_SINGLE, 1516 },
	{ 0x5fd0, 0x5fd4, PDF_CMAP_RANGE, 1517 },
	{ 0x5fde, 0x5fde, PDF_CMAP_SINGLE, 1522 },
	{ 0x5fe1, 0x5fe2, PDF_CMAP_RANGE, 1523 },
	{ 0x5fe8, 0x5fea, PDF_CMAP_RANGE, 1525 },
	{ 0x5fec, 0x5fef, PDF_CMAP_RANGE, 1528 },
	{ 0x5ff2, 0x5ff3, PDF_CMAP_RANGE, 1532 },
	{ 0x5ff6, 0x5ff6, PDF_CMAP_SINGLE, 1534 },
	{ 0x5ffa, 0x5ffa, PDF_CMAP_SINGLE, 1535 },
	{ 0x5ffc, 0x5ffc, PDF_CMAP_SINGLE, 1536 },
	{ 0x6007, 0x6007, PDF_CMAP_SINGLE, 1537 },
	{ 0x600a, 0x600a, PDF_CMAP_SINGLE, 1538 },
	{ 0x600d, 0x600d, PDF_CMAP_SINGLE, 1539 },
	{ 0x6013, 0x6014, PDF_CMAP_RANGE, 1540 },
	{ 0x6017, 0x6018, PDF_CMAP_RANGE, 1542 },
	{ 0x601a, 0x601a, PDF_CMAP_SINGLE, 1544 },
	{ 0x601f, 0x601f, PDF_CMAP_SINGLE, 1545 },
	{ 0x6024, 0x6024, PDF_CMAP_SINGLE, 1546 },
	{ 0x602d, 0x602d, PDF_CMAP_SINGLE, 1547 },
	{ 0x6033, 0x6033, PDF_CMAP_SINGLE, 1548 },
	{ 0x6035, 0x6035, PDF_CMAP_SINGLE, 1549 },
	{ 0x6040, 0x6040, PDF_CMAP_SINGLE, 1550 },
	{ 0x6047, 0x6049, PDF_CMAP_RANGE, 1551 },
	{ 0x604c, 0x604c, PDF_CMAP_SINGLE, 1554 },
	{ 0x6051, 0x6051, PDF_CMAP_SINGLE, 1555 },
	{ 0x6054, 0x6054, PDF_CMAP_SINGLE, 1556 },
	{ 0x6056, 0x6057, PDF_CMAP_RANGE, 1557 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 1559 },
	{ 0x6061, 0x6061, PDF_CMAP_SINGLE, 1560 },
	{ 0x6067, 0x6067, PDF_CMAP_SINGLE, 1561 },
	{ 0x6071, 0x6071, PDF_CMAP_SINGLE, 1562 },
	{ 0x607e, 0x607f, PDF_CMAP_RANGE, 1563 },
	{ 0x6082, 0x6082, PDF_CMAP_SINGLE, 1565 },
	{ 0x6086, 0x6086, PDF_CMAP_SINGLE, 1566 },
	{ 0x6088, 0x6088, PDF_CMAP_SINGLE, 1567 },
	{ 0x608a, 0x608a, PDF_CMAP_SINGLE, 1568 },
	{ 0x608e, 0x608e, PDF_CMAP_SINGLE, 1569 },
	{ 0x6091, 0x6091, PDF_CMAP_SINGLE, 1570 },
	{ 0x6093, 0x6093, PDF_CMAP_SINGLE, 1571 },
	{ 0x6095, 0x6095, PDF_CMAP_SINGLE, 1572 },
	{ 0x6098, 0x6098, PDF_CMAP_SINGLE, 1573 },
	{ 0x609d, 0x609e, PDF_CMAP_RANGE, 1574 },
	{ 0x60a2, 0x60a2, PDF_CMAP_SINGLE, 1576 },
	{ 0x60a4, 0x60a5, PDF_CMAP_RANGE, 1577 },
	{ 0x60a8, 0x60a8, PDF_CMAP_SINGLE, 1579 },
	{ 0x60b0, 0x60b1, PDF_CMAP_RANGE, 1580 },
	{ 0x60b7, 0x60b7, PDF_CMAP_SINGLE, 1582 },
	{ 0x60bb, 0x60bb, PDF_CMAP_SINGLE, 1583 },
	{ 0x60be, 0x60be, PDF_CMAP_SINGLE, 1584 },
	{ 0x60c2, 0x60c2, PDF_CMAP_SINGLE, 1585 },
	{ 0x60c4, 0x60c4, PDF_CMAP_SINGLE, 1586 },
	{ 0x60c8, 0x60cb, PDF_CMAP_RANGE, 1587 },
	{ 0x60ce, 0x60cf, PDF_CMAP_RANGE, 1591 },
	{ 0x60d4, 0x60d5, PDF_CMAP_RANGE, 1593 },
	{ 0x60d9, 0x60d9, PDF_CMAP_SINGLE, 1595 },
	{ 0x60db, 0x60db, PDF_CMAP_SINGLE, 1596 },
	{ 0x60dd, 0x60de, PDF_CMAP_RANGE, 1597 },
	{ 0x60e2, 0x60e2, PDF_CMAP_SINGLE, 1599 },
	{ 0x60e5, 0x60e5, PDF_CMAP_SINGLE, 1600 },
	{ 0x60f2, 0x60f2, PDF_CMAP_SINGLE, 1601 },
	{ 0x60f5, 0x60f5, PDF_CMAP_SINGLE, 1602 },
	{ 0x60f8, 0x60f8, PDF_CMAP_SINGLE, 1603 },
	{ 0x60fc, 0x60fd, PDF_CMAP_RANGE, 1604 },
	{ 0x6102, 0x6102, PDF_CMAP_SINGLE, 1606 },
	{ 0x6107, 0x6107, PDF_CMAP_SINGLE, 1607 },
	{ 0x610a, 0x610a, PDF_CMAP_SINGLE, 1608 },
	{ 0x610c, 0x610c, PDF_CMAP_SINGLE, 1609 },
	{ 0x6110, 0x6114, PDF_CMAP_RANGE, 1610 },
	{ 0x6116, 0x6117, PDF_CMAP_RANGE, 1615 },
	{ 0x6119, 0x6119, PDF_CMAP_SINGLE, 1617 },
	{ 0x611c, 0x611c, PDF_CMAP_SINGLE, 1618 },
	{ 0x611e, 0x611e, PDF_CMAP_SINGLE, 1619 },
	{ 0x6122, 0x6122, PDF_CMAP_SINGLE, 1620 },
	{ 0x612a, 0x612b, PDF_CMAP_RANGE, 1621 },
	{ 0x6130, 0x6131, PDF_CMAP_RANGE, 1623 },
	{ 0x6135, 0x6137, PDF_CMAP_RANGE, 1625 },
	{ 0x6139, 0x6139, PDF_CMAP_SINGLE, 1628 },
	{ 0x6141, 0x6141, PDF_CMAP_SINGLE, 1629 },
	{ 0x6145, 0x6146, PDF_CMAP_RANGE, 1630 },
	{ 0x6149, 0x6149, PDF_CMAP_SINGLE, 1632 },
	{ 0x615e, 0x615e, PDF_CMAP_SINGLE, 1633 },
	{ 0x6160, 0x6160, PDF_CMAP_SINGLE, 1634 },
	{ 0x616c, 0x616c, PDF_CMAP_SINGLE, 1635 },
	{ 0x6172, 0x6172, PDF_CMAP_SINGLE, 1636 },
	{ 0x6178, 0x6178, PDF_CMAP_SINGLE, 1637 },
	{ 0x617b, 0x617c, PDF_CMAP_RANGE, 1638 },
	{ 0x617f, 0x6181, PDF_CMAP_RANGE, 1640 },
	{ 0x6183, 0x6184, PDF_CMAP_RANGE, 1643 },
	{ 0x618b, 0x618b, PDF_CMAP_SINGLE, 1645 },
	{ 0x618d, 0x618d, PDF_CMAP_SINGLE, 1646 },
	{ 0x6192, 0x6193, PDF_CMAP_RANGE, 1647 },
	{ 0x6197, 0x6198, PDF_CMAP_RANGE, 1649 },
	{ 0x619c, 0x619d, PDF_CMAP_RANGE, 1651 },
	{ 0x619f, 0x61a0, PDF_CMAP_RANGE, 1653 },
	{ 0x61a5, 0x61a5, PDF_CMAP_SINGLE, 1655 },
	{ 0x61a8, 0x61a8, PDF_CMAP_SINGLE, 1656 },
	{ 0x61aa, 0x61aa, PDF_CMAP_SINGLE, 1657 },
	{ 0x61ad, 0x61ad, PDF_CMAP_SINGLE, 1658 },
	{ 0x61b8, 0x61b9, PDF_CMAP_RANGE, 1659 },
	{ 0x61bc, 0x61bc, PDF_CMAP_SINGLE, 1661 },
	{ 0x61c0, 0x61c2, PDF_CMAP_RANGE, 1662 },
	{ 0x61ce, 0x61cf, PDF_CMAP_RANGE, 1665 },
	{ 0x61d5, 0x61d5, PDF_CMAP_SINGLE, 1667 },
	{ 0x61dc, 0x61df, PDF_CMAP_RANGE, 1668 },
	{ 0x61e1, 0x61e2, PDF_CMAP_RANGE, 1672 },
	{ 0x61e5, 0x61e5, PDF_CMAP_SINGLE, 1676 },
	{ 0x61e7, 0x61e7, PDF_CMAP_SINGLE, 1674 },
	{ 0x61e9, 0x61e9, PDF_CMAP_SINGLE, 1675 },
	{ 0x61ec, 0x61ed, PDF_CMAP_RANGE, 1677 },
	{ 0x61ef, 0x61ef, PDF_CMAP_SINGLE, 1679 },
	{ 0x6201, 0x6201, PDF_CMAP_SINGLE, 1680 },
	{ 0x6203, 0x6204, PDF_CMAP_RANGE, 1681 },
	{ 0x6207, 0x6207, PDF_CMAP_SINGLE, 1683 },
	{ 0x6213, 0x6213, PDF_CMAP_SINGLE, 1684 },
	{ 0x6215, 0x6215, PDF_CMAP_SINGLE, 1685 },
	{ 0x621c, 0x621c, PDF_CMAP_SINGLE, 1686 },
	{ 0x6220, 0x6220, PDF_CMAP_SINGLE, 1687 },
	{ 0x6222, 0x6223, PDF_CMAP_RANGE, 1688 },
	{ 0x6227, 0x6227, PDF_CMAP_SINGLE, 1690 },
	{ 0x6229, 0x6229, PDF_CMAP_SINGLE, 1691 },
	{ 0x622b, 0x622b, PDF_CMAP_SINGLE, 1692 },
	{ 0x6239, 0x6239, PDF_CMAP_SINGLE, 1693 },
	{ 0x623d, 0x623d, PDF_CMAP_SINGLE, 1694 },
	{ 0x6242, 0x6244, PDF_CMAP_RANGE, 1695 },
	{ 0x6246, 0x6246, PDF_CMAP_SINGLE, 1698 },
	{ 0x624c, 0x624c, PDF_CMAP_SINGLE, 1699 },
	{ 0x6250, 0x6252, PDF_CMAP_RANGE, 1700 },
	{ 0x6254, 0x6254, PDF_CMAP_SINGLE, 1703 },
	{ 0x6256, 0x6256, PDF_CMAP_SINGLE, 1704 },
	{ 0x625a, 0x625a, PDF_CMAP_SINGLE, 1705 },
	{ 0x625c, 0x625c, PDF_CMAP_SINGLE, 1706 },
	{ 0x6264, 0x6264, PDF_CMAP_SINGLE, 1707 },
	{ 0x626d, 0x626d, PDF_CMAP_SINGLE, 1708 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 1709 },
	{ 0x6273, 0x6273, PDF_CMAP_SINGLE, 1710 },
	{ 0x627a, 0x627a, PDF_CMAP_SINGLE, 1711 },
	{ 0x627d, 0x627d, PDF_CMAP_SINGLE, 1712 },
	{ 0x628d, 0x6290, PDF_CMAP_RANGE, 1713 },
	{ 0x62a6, 0x62a6, PDF_CMAP_SINGLE, 1717 },
	{ 0x62a8, 0x62a8, PDF_CMAP_SINGLE, 1718 },
	{ 0x62b3, 0x62b3, PDF_CMAP_SINGLE, 1719 },
	{ 0x62b6, 0x62b7, PDF_CMAP_RANGE, 1720 },
	{ 0x62ba, 0x62ba, PDF_CMAP_SINGLE, 1722 },
	{ 0x62be, 0x62bf, PDF_CMAP_RANGE, 1723 },
	{ 0x62c4, 0x62c4, PDF_CMAP_SINGLE, 1725 },
	{ 0x62ce, 0x62ce, PDF_CMAP_SINGLE, 1726 },
	{ 0x62d5, 0x62d6, PDF_CMAP_RANGE, 1727 },
	{ 0x62da, 0x62da, PDF_CMAP_SINGLE, 1729 },
	{ 0x62ea, 0x62ea, PDF_CMAP_SINGLE, 1730 },
	{ 0x62f2, 0x62f2, PDF_CMAP_SINGLE, 1731 },
	{ 0x62f4, 0x62f4, PDF_CMAP_SINGLE, 1732 },
	{ 0x62fc, 0x62fd, PDF_CMAP_RANGE, 1733 },
	{ 0x6303, 0x6304, PDF_CMAP_RANGE, 1735 },
	{ 0x630a, 0x630b, PDF_CMAP_RANGE, 1737 },
	{ 0x630d, 0x630d, PDF_CMAP_SINGLE, 1739 },
	{ 0x6310, 0x6310, PDF_CMAP_SINGLE, 1740 },
	{ 0x6313, 0x6313, PDF_CMAP_SINGLE, 1741 },
	{ 0x6316, 0x6316, PDF_CMAP_SINGLE, 1742 },
	{ 0x6318, 0x6318, PDF_CMAP_SINGLE, 1743 },
	{ 0x6329, 0x632a, PDF_CMAP_RANGE, 1744 },
	{ 0x632d, 0x632d, PDF_CMAP_SINGLE, 1746 },
	{ 0x6335, 0x6336, PDF_CMAP_RANGE, 1747 },
	{ 0x6339, 0x6339, PDF_CMAP_SINGLE, 1749 },
	{ 0x633c, 0x633c, PDF_CMAP_SINGLE, 1750 },
	{ 0x6341, 0x6344, PDF_CMAP_RANGE, 1751 },
	{ 0x6346, 0x6346, PDF_CMAP_SINGLE, 1755 },
	{ 0x634a, 0x634b, PDF_CMAP_RANGE, 1756 },
	{ 0x634e, 0x634e, PDF_CMAP_SINGLE, 1758 },
	{ 0x6352, 0x6354, PDF_CMAP_RANGE, 1759 },
	{ 0x6358, 0x6358, PDF_CMAP_SINGLE, 1762 },
	{ 0x635b, 0x635b, PDF_CMAP_SINGLE, 1763 },
	{ 0x6365, 0x6366, PDF_CMAP_RANGE, 1764 },
	{ 0x636c, 0x636d, PDF_CMAP_RANGE, 1766 },
	{ 0x6371, 0x6371, PDF_CMAP_SINGLE, 1768 },
	{ 0x6374, 0x6375, PDF_CMAP_RANGE, 1769 },
	{ 0x6378, 0x6378, PDF_CMAP_SINGLE, 1771 },
	{ 0x637c, 0x637d, PDF_CMAP_RANGE, 1772 },
	{ 0x637f, 0x637f, PDF_CMAP_SINGLE, 1774 },
	{ 0x6382, 0x6382, PDF_CMAP_SINGLE, 1775 },
	{ 0x6384, 0x6384, PDF_CMAP_SINGLE, 1776 },
	{ 0x6387, 0x6387, PDF_CMAP_SINGLE, 1777 },
	{ 0x638a, 0x638a, PDF_CMAP_SINGLE, 1778 },
	{ 0x6390, 0x6390, PDF_CMAP_SINGLE, 1779 },
	{ 0x6394, 0x6395, PDF_CMAP_RANGE, 1780 },
	{ 0x6399, 0x639a, PDF_CMAP_RANGE, 1782 },
	{ 0x639e, 0x639e, PDF_CMAP_SINGLE, 1784 },
	{ 0x63a4, 0x63a4, PDF_CMAP_SINGLE, 1785 },
	{ 0x63a6, 0x63a6, PDF_CMAP_SINGLE, 1786 },
	{ 0x63ad, 0x63af, PDF_CMAP_RANGE, 1787 },
	{ 0x63bd, 0x63bd, PDF_CMAP_SINGLE, 1790 },
	{ 0x63c1, 0x63c1, PDF_CMAP_SINGLE, 1791 },
	{ 0x63c5, 0x63c5, PDF_CMAP_SINGLE, 1792 },
	{ 0x63c8, 0x63c8, PDF_CMAP_SINGLE, 1793 },
	{ 0x63ce, 0x63ce, PDF_CMAP_SINGLE, 1794 },
	{ 0x63d1, 0x63d1, PDF_CMAP_SINGLE, 1795 },
	{ 0x63d3, 0x63d5, PDF_CMAP_RANGE, 1796 },
	{ 0x63dc, 0x63dc, PDF_CMAP_SINGLE, 1799 },
	{ 0x63e0, 0x63e0, PDF_CMAP_SINGLE, 1800 },
	{ 0x63e5, 0x63e5, PDF_CMAP_SINGLE, 1801 },
	{ 0x63ea, 0x63ea, PDF_CMAP_SINGLE, 1802 },
	{ 0x63ec, 0x63ec, PDF_CMAP_SINGLE, 1803 },
	{ 0x63f2, 0x63f3, PDF_CMAP_RANGE, 1804 },
	{ 0x63f5, 0x63f5, PDF_CMAP_SINGLE, 1806 },
	{ 0x63f8, 0x63f9, PDF_CMAP_RANGE, 1807 },
	{ 0x6409, 0x640a, PDF_CMAP_RANGE, 1809 },
	{ 0x6410, 0x6410, PDF_CMAP_SINGLE, 1811 },
	{ 0x6412, 0x6412, PDF_CMAP_SINGLE, 1812 },
	{ 0x6414, 0x6414, PDF_CMAP_SINGLE, 1813 },
	{ 0x6418, 0x6418, PDF_CMAP_SINGLE, 1814 },
	{ 0x641e, 0x641e, PDF_CMAP_SINGLE, 1815 },
	{ 0x6420, 0x6420, PDF_CMAP_SINGLE, 1816 },
	{ 0x6422, 0x6422, PDF_CMAP_SINGLE, 1817 },
	{ 0x6424, 0x6425, PDF_CMAP_RANGE, 1818 },
	{ 0x6429, 0x642a, PDF_CMAP_RANGE, 1820 },
	{ 0x642f, 0x6430, PDF_CMAP_RANGE, 1822 },
	{ 0x6435, 0x6435, PDF_CMAP_SINGLE, 1824 },
	{ 0x643d, 0x643d, PDF_CMAP_SINGLE, 1825 },
	{ 0x643f, 0x643f, PDF_CMAP_SINGLE, 1826 },
	{ 0x644b, 0x644b, PDF_CMAP_SINGLE, 1827 },
	{ 0x644f, 0x644f, PDF_CMAP_SINGLE, 1828 },
	{ 0x6451, 0x6454, PDF_CMAP_RANGE, 1829 },
	{ 0x645a, 0x645d, PDF_CMAP_RANGE, 1833 },
	{ 0x645f, 0x6461, PDF_CMAP_RANGE, 1837 },
	{ 0x6463, 0x6463, PDF_CMAP_SINGLE, 1840 },
	{ 0x646d, 0x646d, PDF_CMAP_SINGLE, 1841 },
	{ 0x6473, 0x6474, PDF_CMAP_RANGE, 1842 },
	{ 0x647b, 0x647b, PDF_CMAP_SINGLE, 1844 },
	{ 0x647d, 0x647d, PDF_CMAP_SINGLE, 1845 },
	{ 0x6485, 0x6485, PDF_CMAP_SINGLE, 1846 },
	{ 0x6487, 0x6487, PDF_CMAP_SINGLE, 1847 },
	{ 0x648f, 0x6491, PDF_CMAP_RANGE, 1848 },
	{ 0x6498, 0x6499, PDF_CMAP_RANGE, 1851 },
	{ 0x649b, 0x649b, PDF_CMAP_SINGLE, 1853 },
	{ 0x649d, 0x649d, PDF_CMAP_SINGLE, 1854 },
	{ 0x649f, 0x649f, PDF_CMAP_SINGLE, 1855 },
	{ 0x64a1, 0x64a1, PDF_CMAP_SINGLE, 1856 },
	{ 0x64a3, 0x64a3, PDF_CMAP_SINGLE, 1857 },
	{ 0x64a6, 0x64a6, PDF_CMAP_SINGLE, 1858 },
	{ 0x64a8, 0x64a8, PDF_CMAP_SINGLE, 1859 },
	{ 0x64ac, 0x64ac, PDF_CMAP_SINGLE, 1860 },
	{ 0x64b3, 0x64b3, PDF_CMAP_SINGLE, 1861 },
	{ 0x64bd, 0x64bf, PDF_CMAP_RANGE, 1862 },
	{ 0x64c4, 0x64c4, PDF_CMAP_SINGLE, 1865 },
	{ 0x64c9, 0x64cc, PDF_CMAP_RANGE, 1866 },
	{ 0x64ce, 0x64ce, PDF_CMAP_SINGLE, 1870 },
	{ 0x64d0, 0x64d1, PDF_CMAP_RANGE, 1871 },
	{ 0x64d5, 0x64d5, PDF_CMAP_SINGLE, 1873 },
	{ 0x64d7, 0x64d7, PDF_CMAP_SINGLE, 1874 },
	{ 0x64e4, 0x64e5, PDF_CMAP_RANGE, 1875 },
	{ 0x64e9, 0x64ea, PDF_CMAP_RANGE, 1877 },
	{ 0x64ed, 0x64ed, PDF_CMAP_SINGLE, 1879 },
	{ 0x64f0, 0x64f0, PDF_CMAP_SINGLE, 1880 },
	{ 0x64f5, 0x64f5, PDF_CMAP_SINGLE, 1881 },
	{ 0x64f7, 0x64f7, PDF_CMAP_SINGLE, 1882 },
	{ 0x64fb, 0x64fb, PDF_CMAP_SINGLE, 1883 },
	{ 0x64ff, 0x64ff, PDF_CMAP_SINGLE, 1884 },
	{ 0x6501, 0x6501, PDF_CMAP_SINGLE, 1885 },
	{ 0x6504, 0x6504, PDF_CMAP_SINGLE, 1886 },
	{ 0x6508, 0x650a, PDF_CMAP_RANGE, 1887 },
	{ 0x650f, 0x650f, PDF_CMAP_SINGLE, 1890 },
	{ 0x6513, 0x6514, PDF_CMAP_RANGE, 1891 },
	{ 0x6516, 0x6516, PDF_CMAP_SINGLE, 1893 },
	{ 0x6519, 0x6519, PDF_CMAP_SINGLE, 1894 },
	{ 0x651b, 0x651b, PDF_CMAP_SINGLE, 1895 },
	{ 0x651e, 0x651f, PDF_CMAP_RANGE, 1896 },
	{ 0x6522, 0x6522, PDF_CMAP_SINGLE, 1898 },
	{ 0x6526, 0x6526, PDF_CMAP_SINGLE, 1899 },
	{ 0x6529, 0x6529, PDF_CMAP_SINGLE, 1900 },
	{ 0x652e, 0x652e, PDF_CMAP_SINGLE, 1901 },
	{ 0x6531, 0x6531, PDF_CMAP_SINGLE, 1902 },
	{ 0x653a, 0x653a, PDF_CMAP_SINGLE, 1903 },
	{ 0x653c, 0x653d, PDF_CMAP_RANGE, 1904 },
	{ 0x6543, 0x6543, PDF_CMAP_SINGLE, 1906 },
	{ 0x6547, 0x6547, PDF_CMAP_SINGLE, 1907 },
	{ 0x6549, 0x6549, PDF_CMAP_SINGLE, 1908 },
	{ 0x6550, 0x6550, PDF_CMAP_SINGLE, 1909 },
	{ 0x6552, 0x6552, PDF_CMAP_SINGLE, 1910 },
	{ 0x6554, 0x6554, PDF_CMAP_SINGLE, 1911 },
	{ 0x655f, 0x6560, PDF_CMAP_RANGE, 1912 },
	{ 0x6567, 0x6567, PDF_CMAP_SINGLE, 1914 },
	{ 0x656b, 0x656b, PDF_CMAP_SINGLE, 1915 },
	{ 0x657a, 0x657a, PDF_CMAP_SINGLE, 1916 },
	{ 0x657d, 0x657d, PDF_CMAP_SINGLE, 1917 },
	{ 0x6581, 0x6581, PDF_CMAP_SINGLE, 1918 },
	{ 0x6585, 0x6585, PDF_CMAP_SINGLE, 1919 },
	{ 0x658a, 0x658a, PDF_CMAP_SINGLE, 1920 },
	{ 0x6592, 0x6592, PDF_CMAP_SINGLE, 1921 },
	{ 0x6595, 0x6595, PDF_CMAP_SINGLE, 1922 },
	{ 0x6598, 0x6598, PDF_CMAP_SINGLE, 1923 },
	{ 0x659d, 0x659d, PDF_CMAP_SINGLE, 1924 },
	{ 0x65a0, 0x65a0, PDF_CMAP_SINGLE, 1925 },
	{ 0x65a3, 0x65a3, PDF_CMAP_SINGLE, 1926 },
	{ 0x65a6, 0x65a6, PDF_CMAP_SINGLE, 1927 },
	{ 0x65ae, 0x65ae, PDF_CMAP_SINGLE, 1928 },
	{ 0x65b2, 0x65b4, PDF_CMAP_RANGE, 1929 },
	{ 0x65bf, 0x65bf, PDF_CMAP_SINGLE, 1932 },
	{ 0x65c2, 0x65c2, PDF_CMAP_SINGLE, 1933 },
	{ 0x65c8, 0x65c9, PDF_CMAP_RANGE, 1934 },
	{ 0x65ce, 0x65ce, PDF_CMAP_SINGLE, 1936 },
	{ 0x65d0, 0x65d0, PDF_CMAP_SINGLE, 1937 },
	{ 0x65d4, 0x65d4, PDF_CMAP_SINGLE, 1938 },
	{ 0x65d6, 0x65d6, PDF_CMAP_SINGLE, 1939 },
	{ 0x65d8, 0x65d8, PDF_CMAP_SINGLE, 1940 },
	{ 0x65df, 0x65df, PDF_CMAP_SINGLE, 1941 },
	{ 0x65f0, 0x65f0, PDF_CMAP_SINGLE, 1942 },
	{ 0x65f2, 0x65f2, PDF_CMAP_SINGLE, 1943 },
	{ 0x65f4, 0x65f5, PDF_CMAP_RANGE, 1944 },
	{ 0x65f9, 0x65f9, PDF_CMAP_SINGLE, 1946 },
	{ 0x65fe, 0x6600, PDF_CMAP_RANGE, 1947 },
	{ 0x6604, 0x6604, PDF_CMAP_SINGLE, 1950 },
	{ 0x6608, 0x6609, PDF_CMAP_RANGE, 1951 },
	{ 0x660d, 0x660d, PDF_CMAP_SINGLE, 1953 },
	{ 0x6611, 0x6612, PDF_CMAP_RANGE, 1954 },
	{ 0x6615, 0x6616, PDF_CMAP_RANGE, 1956 },
	{ 0x661d, 0x661e, PDF_CMAP_RANGE, 1958 },
	{ 0x6621, 0x6624, PDF_CMAP_RANGE, 1960 },
	{ 0x6626, 0x6626, PDF_CMAP_SINGLE, 1964 },
	{ 0x6629, 0x662c, PDF_CMAP_RANGE, 1965 },
	{ 0x662e, 0x662e, PDF_CMAP_SINGLE, 1969 },
	{ 0x6630, 0x6631, PDF_CMAP_RANGE, 1970 },
	{ 0x6633, 0x6633, PDF_CMAP_SINGLE, 1972 },
	{ 0x6637, 0x6637, PDF_CMAP_SINGLE, 1974 },
	{ 0x6639, 0x6639, PDF_CMAP_SINGLE, 1973 },
	{ 0x6640, 0x6640, PDF_CMAP_SINGLE, 1975 },
	{ 0x6645, 0x6646, PDF_CMAP_RANGE, 1976 },
	{ 0x664a, 0x664a, PDF_CMAP_SINGLE, 1978 },
	{ 0x664c, 0x664c, PDF_CMAP_SINGLE, 1979 },
	{ 0x664e, 0x664e, PDF_CMAP_SINGLE, 1981 },
	{ 0x6651, 0x6651, PDF_CMAP_SINGLE, 1980 },
	{ 0x6657, 0x6659, PDF_CMAP_RANGE, 1982 },
	{ 0x665b, 0x665c, PDF_CMAP_RANGE, 1985 },
	{ 0x6660, 0x6661, PDF_CMAP_RANGE, 1987 },
	{ 0x666a, 0x666c, PDF_CMAP_RANGE, 1990 },
	{ 0x6673, 0x6673, PDF_CMAP_SINGLE, 1994 },
	{ 0x6675, 0x6675, PDF_CMAP_SINGLE, 1995 },
	{ 0x6677, 0x6679, PDF_CMAP_RANGE, 1997 },
	{ 0x667b, 0x667c, PDF_CMAP_TABLE, 230 },
	{ 0x667e, 0x6680, PDF_CMAP_TABLE, 232 },
	{ 0x668b, 0x668d, PDF_CMAP_RANGE, 2003 },
	{ 0x6690, 0x6690, PDF_CMAP_SINGLE, 2006 },
	{ 0x6692, 0x6692, PDF_CMAP_SINGLE, 2007 },
	{ 0x6699, 0x669c, PDF_CMAP_RANGE, 2008 },
	{ 0x669f, 0x66a0, PDF_CMAP_RANGE, 2012 },
	{ 0x66a4, 0x66a4, PDF_CMAP_SINGLE, 2014 },
	{ 0x66ad, 0x66ad, PDF_CMAP_SINGLE, 2015 },
	{ 0x66b1, 0x66b2, PDF_CMAP_RANGE, 2016 },
	{ 0x66b5, 0x66b5, PDF_CMAP_SINGLE, 2018 },
	{ 0x66bb, 0x66bb, PDF_CMAP_SINGLE, 2019 },
	{ 0x66bf, 0x66c0, PDF_CMAP_RANGE, 2020 },
	{ 0x66c2, 0x66c3, PDF_CMAP_RANGE, 2022 },
	{ 0x66c8, 0x66c8, PDF_CMAP_SINGLE, 2024 },
	{ 0x66cc, 0x66cc, PDF_CMAP_SINGLE, 2025 },
	{ 0x66ce, 0x66cf, PDF_CMAP_RANGE, 2026 },
	{ 0x66d4, 0x66d4, PDF_CMAP_SINGLE, 2028 },
	{ 0x66db, 0x66db, PDF_CMAP_SINGLE, 2029 },
	{ 0x66df, 0x66df, PDF_CMAP_SINGLE, 2030 },
	{ 0x66e8, 0x66e8, PDF_CMAP_SINGLE, 2031 },
	{ 0x66eb, 0x66ec, PDF_CMAP_RANGE, 2032 },
	{ 0x66ee, 0x66ee, PDF_CMAP_SINGLE, 2034 },
	{ 0x66fa, 0x66fb, PDF_CMAP_TABLE, 235 },
	{ 0x6705, 0x6705, PDF_CMAP_SINGLE, 2036 },
	{ 0x6707, 0x6707, PDF_CMAP_SINGLE, 2037 },
	{ 0x670e, 0x670e, PDF_CMAP_SINGLE, 2038 },
	{ 0x6713, 0x6713, PDF_CMAP_SINGLE, 2039 },
	{ 0x6719, 0x6719, PDF_CMAP_SINGLE, 2040 },
	{ 0x671c, 0x671c, PDF_CMAP_SINGLE, 2041 },
	{ 0x6720, 0x6720, PDF_CMAP_SINGLE, 2042 },
	{ 0x6722, 0x6722, PDF_CMAP_SINGLE, 2043 },
	{ 0x6733, 0x6733, PDF_CMAP_SINGLE, 2044 },
	{ 0x673e, 0x673e, PDF_CMAP_SINGLE, 2045 },
	{ 0x6745, 0x6745, PDF_CMAP_SINGLE, 2046 },
	{ 0x6747, 0x6748, PDF_CMAP_RANGE, 2047 },
	{ 0x674c, 0x674c, PDF_CMAP_SINGLE, 2049 },
	{ 0x6754, 0x6755, PDF_CMAP_RANGE, 2050 },
	{ 0x675d, 0x675d, PDF_CMAP_SINGLE, 2052 },
	{ 0x6766, 0x6766, PDF_CMAP_SINGLE, 2053 },
	{ 0x676c, 0x676c, PDF_CMAP_SINGLE, 2054 },
	{ 0x676e, 0x676e, PDF_CMAP_SINGLE, 2055 },
	{ 0x6774, 0x6774, PDF_CMAP_SINGLE, 2056 },
	{ 0x6776, 0x6776, PDF_CMAP_SINGLE, 2057 },
	{ 0x677b, 0x677b, PDF_CMAP_SINGLE, 2058 },
	{ 0x6781, 0x6781, PDF_CMAP_SINGLE, 2059 },
	{ 0x6784, 0x6784, PDF_CMAP_SINGLE, 2060 },
	{ 0x678e, 0x678f, PDF_CMAP_RANGE, 2061 },
	{ 0x6791, 0x6791, PDF_CMAP_SINGLE, 2063 },
	{ 0x6793, 0x6793, PDF_CMAP_SINGLE, 2064 },
	{ 0x6796, 0x6796, PDF_CMAP_SINGLE, 2065 },
	{ 0x6798, 0x6799, PDF_CMAP_RANGE, 2066 },
	{ 0x679b, 0x679b, PDF_CMAP_SINGLE, 2068 },
	{ 0x67b0, 0x67b2, PDF_CMAP_RANGE, 2069 },
	{ 0x67b5, 0x67b5, PDF_CMAP_SINGLE, 2072 },
	{ 0x67bb, 0x67bd, PDF_CMAP_RANGE, 2073 },
	{ 0x67c0, 0x67c0, PDF_CMAP_SINGLE, 2077 },
	{ 0x67c2, 0x67c3, PDF_CMAP_RANGE, 2078 },
	{ 0x67c5, 0x67c5, PDF_CMAP_SINGLE, 2080 },
	{ 0x67c8, 0x67c9, PDF_CMAP_RANGE, 2081 },
	{ 0x67d2, 0x67d2, PDF_CMAP_SINGLE, 2083 },
	{ 0x67d7, 0x67d7, PDF_CMAP_SINGLE, 2084 },
	{ 0x67d9, 0x67d9, PDF_CMAP_SINGLE, 2085 },
	{ 0x67dc, 0x67dc, PDF_CMAP_SINGLE, 2086 },
	{ 0x67e1, 0x67e1, PDF_CMAP_SINGLE, 2087 },
	{ 0x67e6, 0x67e6, PDF_CMAP_SINGLE, 2088 },
	{ 0x67f0, 0x67f0, PDF_CMAP_SINGLE, 2089 },
	{ 0x67f2, 0x67f2, PDF_CMAP_SINGLE, 2090 },
	{ 0x67f6, 0x67f7, PDF_CMAP_RANGE, 2091 },
	{ 0x67f9, 0x67f9, PDF_CMAP_SINGLE, 2076 },
	{ 0x6814, 0x6814, PDF_CMAP_SINGLE, 2094 },
	{ 0x6819, 0x6819, PDF_CMAP_SINGLE, 2095 },
	{ 0x681d, 0x681d, PDF_CMAP_SINGLE, 2096 },
	{ 0x681f, 0x681f, PDF_CMAP_SINGLE, 2097 },
	{ 0x6827, 0x6828, PDF_CMAP_TABLE, 237 },
	{ 0x682c, 0x682d, PDF_CMAP_RANGE, 2100 },
	{ 0x682f, 0x6831, PDF_CMAP_RANGE, 2102 },
	{ 0x6833, 0x6833, PDF_CMAP_SINGLE, 2105 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 2106 },
	{ 0x683f, 0x683f, PDF_CMAP_SINGLE, 2107 },
	{ 0x6844, 0x6845, PDF_CMAP_RANGE, 2108 },
	{ 0x684a, 0x684a, PDF_CMAP_SINGLE, 2110 },
	{ 0x684c, 0x684c, PDF_CMAP_SINGLE, 2111 },
	{ 0x6852, 0x6852, PDF_CMAP_SINGLE, 2093 },
	{ 0x6855, 0x6855, PDF_CMAP_SINGLE, 2112 },
	{ 0x6857, 0x6858, PDF_CMAP_RANGE, 2113 },
	{ 0x685b, 0x685b, PDF_CMAP_SINGLE, 2115 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 2116 },
	{ 0x686e, 0x6872, PDF_CMAP_RANGE, 2117 },
	{ 0x6875, 0x6875, PDF_CMAP_SINGLE, 2122 },
	{ 0x6879, 0x687c, PDF_CMAP_RANGE, 2123 },
	{ 0x6882, 0x6882, PDF_CMAP_SINGLE, 2127 },
	{ 0x6884, 0x6884, PDF_CMAP_SINGLE, 2128 },
	{ 0x6886, 0x6886, PDF_CMAP_SINGLE, 2129 },
	{ 0x6888, 0x6888, PDF_CMAP_SINGLE, 2130 },
	{ 0x6896, 0x6896, PDF_CMAP_SINGLE, 2131 },
	{ 0x6898, 0x6898, PDF_CMAP_SINGLE, 2132 },
	{ 0x689a, 0x689a, PDF_CMAP_SINGLE, 2133 },
	{ 0x689c, 0x689c, PDF_CMAP_SINGLE, 2134 },
	{ 0x68a1, 0x68a1, PDF_CMAP_SINGLE, 2135 },
	{ 0x68a3, 0x68a3, PDF_CMAP_SINGLE, 2136 },
	{ 0x68a5, 0x68a5, PDF_CMAP_SINGLE, 2137 },
	{ 0x68a9, 0x68aa, PDF_CMAP_RANGE, 2138 },
	{ 0x68ae, 0x68ae, PDF_CMAP_SINGLE, 2140 },
	{ 0x68b2, 0x68b2, PDF_CMAP_SINGLE, 2141 },
	{ 0x68bb, 0x68bb, PDF_CMAP_SINGLE, 2142 },
	{ 0x68c5, 0x68c5, PDF_CMAP_SINGLE, 2143 },
	{ 0x68c8, 0x68c8, PDF_CMAP_SINGLE, 2144 },
	{ 0x68cc, 0x68cc, PDF_CMAP_SINGLE, 2145 },
	{ 0x68cf, 0x68d1, PDF_CMAP_RANGE, 2146 },
	{ 0x68d3, 0x68d3, PDF_CMAP_SINGLE, 2149 },
	{ 0x68d6, 0x68d6, PDF_CMAP_SINGLE, 2150 },
	{ 0x68d9, 0x68d9, PDF_CMAP_SINGLE, 2151 },
	{ 0x68dc, 0x68dd, PDF_CMAP_RANGE, 2152 },
	{ 0x68e5, 0x68e5, PDF_CMAP_SINGLE, 2154 },
	{ 0x68e8, 0x68e8, PDF_CMAP_SINGLE, 2155 },
	{ 0x68ea, 0x68ed, PDF_CMAP_RANGE, 2156 },
	{ 0x68f0, 0x68f1, PDF_CMAP_RANGE, 2160 },
	{ 0x68f5, 0x68f6, PDF_CMAP_RANGE, 2162 },
	{ 0x68fb, 0x68fd, PDF_CMAP_RANGE, 2164 },
	{ 0x6906, 0x6906, PDF_CMAP_SINGLE, 2167 },
	{ 0x6909, 0x690a, PDF_CMAP_RANGE, 2168 },
	{ 0x6910, 0x6911, PDF_CMAP_RANGE, 2170 },
	{ 0x6913, 0x6913, PDF_CMAP_SINGLE, 2172 },
	{ 0x6916, 0x6917, PDF_CMAP_RANGE, 2173 },
	{ 0x6931, 0x6931, PDF_CMAP_SINGLE, 2175 },
	{ 0x6933, 0x6933, PDF_CMAP_SINGLE, 2176 },
	{ 0x6935, 0x6935, PDF_CMAP_SINGLE, 2177 },
	{ 0x6938, 0x6938, PDF_CMAP_SINGLE, 2178 },
	{ 0x693b, 0x693b, PDF_CMAP_SINGLE, 2179 },
	{ 0x6942, 0x6942, PDF_CMAP_SINGLE, 2180 },
	{ 0x6945, 0x6945, PDF_CMAP_SINGLE, 2181 },
	{ 0x6949, 0x6949, PDF_CMAP_SINGLE, 2182 },
	{ 0x694e, 0x694e, PDF_CMAP_SINGLE, 2183 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 2184 },
	{ 0x695b, 0x695b, PDF_CMAP_SINGLE, 2185 },
	{ 0x6963, 0x6966, PDF_CMAP_RANGE, 2186 },
	{ 0x6968, 0x6969, PDF_CMAP_RANGE, 2190 },
	{ 0x696c, 0x696c, PDF_CMAP_SINGLE, 2192 },
	{ 0x6970, 0x6972, PDF_CMAP_RANGE, 2193 },
	{ 0x697a, 0x697b, PDF_CMAP_RANGE, 2196 },
	{ 0x697f, 0x6980, PDF_CMAP_RANGE, 2198 },
	{ 0x698d, 0x698d, PDF_CMAP_SINGLE, 2200 },
	{ 0x6992, 0x6992, PDF_CMAP_SINGLE, 2201 },
	{ 0x6996, 0x6996, PDF_CMAP_SINGLE, 2202 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 2203 },
	{ 0x69a1, 0x69a1, PDF_CMAP_SINGLE, 2204 },
	{ 0x69a5, 0x69a6, PDF_CMAP_RANGE, 2205 },
	{ 0x69a8, 0x69a8, PDF_CMAP_SINGLE, 2207 },
	{ 0x69ab, 0x69ab, PDF_CMAP_SINGLE, 2208 },
	{ 0x69ad, 0x69ad, PDF_CMAP_SINGLE, 2209 },
	{ 0x69af, 0x69af, PDF_CMAP_SINGLE, 2210 },
	{ 0x69b7, 0x69b8, PDF_CMAP_RANGE, 2211 },
	{ 0x69ba, 0x69ba, PDF_CMAP_SINGLE, 2213 },
	{ 0x69bc, 0x69bc, PDF_CMAP_SINGLE, 2214 },
	{ 0x69c5, 0x69c5, PDF_CMAP_SINGLE, 2215 },
	{ 0x69c8, 0x69c8, PDF_CMAP_SINGLE, 2216 },
	{ 0x69d1, 0x69d1, PDF_CMAP_SINGLE, 2217 },
	{ 0x69d6, 0x69d7, PDF_CMAP_RANGE, 2218 },
	{ 0x69e2, 0x69e2, PDF_CMAP_SINGLE, 2220 },
	{ 0x69e5, 0x69e5, PDF_CMAP_SINGLE, 2221 },
	{ 0x69ee, 0x69ef, PDF_CMAP_RANGE, 2222 },
	{ 0x69f1, 0x69f1, PDF_CMAP_SINGLE, 2224 },
	{ 0x69f3, 0x69f3, PDF_CMAP_SINGLE, 2225 },
	{ 0x69f5, 0x69f5, PDF_CMAP_SINGLE, 2226 },
	{ 0x69fe, 0x69fe, PDF_CMAP_SINGLE, 2227 },
	{ 0x6a00, 0x6a01, PDF_CMAP_RANGE, 2228 },
	{ 0x6a03, 0x6a03, PDF_CMAP_SINGLE, 2230 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 2231 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 2232 },
	{ 0x6a15, 0x6a15, PDF_CMAP_SINGLE, 2233 },
	{ 0x6a1a, 0x6a1a, PDF_CMAP_SINGLE, 2234 },
	{ 0x6a1d, 0x6a1d, PDF_CMAP_SINGLE, 2235 },
	{ 0x6a20, 0x6a20, PDF_CMAP_SINGLE, 2236 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 2237 },
	{ 0x6a28, 0x6a28, PDF_CMAP_SINGLE, 2238 },
	{ 0x6a30, 0x6a30, PDF_CMAP_SINGLE, 2239 },
	{ 0x6a32, 0x6a32, PDF_CMAP_SINGLE, 2240 },
	{ 0x6a34, 0x6a34, PDF_CMAP_SINGLE, 2241 },
	{ 0x6a37, 0x6a37, PDF_CMAP_SINGLE, 2242 },
	{ 0x6a3b, 0x6a3b, PDF_CMAP_SINGLE, 2243 },
	{ 0x6a3e, 0x6a3f, PDF_CMAP_RANGE, 2244 },
	{ 0x6a45, 0x6a46, PDF_CMAP_RANGE, 2246 },
	{ 0x6a49, 0x6a4a, PDF_CMAP_RANGE, 2248 },
	{ 0x6a4e, 0x6a4e, PDF_CMAP_SINGLE, 2250 },
	{ 0x6a50, 0x6a52, PDF_CMAP_RANGE, 2251 },
	{ 0x6a55, 0x6a56, PDF_CMAP_RANGE, 2254 },
	{ 0x6a5b, 0x6a5b, PDF_CMAP_SINGLE, 2256 },
	{ 0x6a64, 0x6a64, PDF_CMAP_SINGLE, 2257 },
	{ 0x6a67, 0x6a67, PDF_CMAP_SINGLE, 2258 },
	{ 0x6a6a, 0x6a6a, PDF_CMAP_SINGLE, 2259 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 2260 },
	{ 0x6a73, 0x6a73, PDF_CMAP_SINGLE, 2261 },
	{ 0x6a7e, 0x6a7e, PDF_CMAP_SINGLE, 2262 },
	{ 0x6a81, 0x6a81, PDF_CMAP_SINGLE, 2263 },
	{ 0x6a83, 0x6a83, PDF_CMAP_SINGLE, 2264 },
	{ 0x6a86, 0x6a87, PDF_CMAP_RANGE, 2265 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 2267 },
	{ 0x6a8b, 0x6a8b, PDF_CMAP_SINGLE, 2268 },
	{ 0x6a91, 0x6a91, PDF_CMAP_SINGLE, 2269 },
	{ 0x6a9b, 0x6a9b, PDF_CMAP_SINGLE, 2270 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 2271 },
	{ 0x6aa5, 0x6aa5, PDF_CMAP_SINGLE, 2274 },
	{ 0x6aab, 0x6aab, PDF_CMAP_SINGLE, 2275 },
	{ 0x6aaf, 0x6ab1, PDF_CMAP_RANGE, 2276 },
	{ 0x6ab4, 0x6ab4, PDF_CMAP_SINGLE, 2279 },
	{ 0x6abd, 0x6abf, PDF_CMAP_RANGE, 2280 },
	{ 0x6ac6, 0x6ac6, PDF_CMAP_SINGLE, 2283 },
	{ 0x6ac8, 0x6ac9, PDF_CMAP_TABLE, 239 },
	{ 0x6acc, 0x6acc, PDF_CMAP_SINGLE, 2286 },
	{ 0x6ad0, 0x6ad0, PDF_CMAP_SINGLE, 2287 },
	{ 0x6ad4, 0x6ad6, PDF_CMAP_RANGE, 2288 },
	{ 0x6adc, 0x6add, PDF_CMAP_RANGE, 2291 },
	{ 0x6ae4, 0x6ae4, PDF_CMAP_SINGLE, 2293 },
	{ 0x6ae7, 0x6ae7, PDF_CMAP_SINGLE, 2294 },
	{ 0x6aec, 0x6aec, PDF_CMAP_SINGLE, 2295 },
	{ 0x6af0, 0x6af2, PDF_CMAP_RANGE, 2296 },
	{ 0x6afc, 0x6afd, PDF_CMAP_RANGE, 2299 },
	{ 0x6b02, 0x6b03, PDF_CMAP_RANGE, 2301 },
	{ 0x6b06, 0x6b07, PDF_CMAP_RANGE, 2303 },
	{ 0x6b09, 0x6b09, PDF_CMAP_SINGLE, 2305 },
	{ 0x6b0f, 0x6b11, PDF_CMAP_RANGE, 2306 },
	{ 0x6b17, 0x6b17, PDF_CMAP_SINGLE, 2309 },
	{ 0x6b1b, 0x6b1b, PDF_CMAP_SINGLE, 2310 },
	{ 0x6b1e, 0x6b1e, PDF_CMAP_SINGLE, 2311 },
	{ 0x6b24, 0x6b24, PDF_CMAP_SINGLE, 2312 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 2313 },
	{ 0x6b2b, 0x6b2c, PDF_CMAP_RANGE, 2314 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 2316 },
	{ 0x6b35, 0x6b36, PDF_CMAP_RANGE, 2317 },
	{ 0x6b3b, 0x6b3b, PDF_CMAP_SINGLE, 2319 },
	{ 0x6b3f, 0x6b3f, PDF_CMAP_SINGLE, 2320 },
	{ 0x6b46, 0x6b46, PDF_CMAP_SINGLE, 2321 },
	{ 0x6b4a, 0x6b4a, PDF_CMAP_SINGLE, 2322 },
	{ 0x6b4d, 0x6b4d, PDF_CMAP_SINGLE, 2323 },
	{ 0x6b52, 0x6b52, PDF_CMAP_SINGLE, 2324 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 2325 },
	{ 0x6b58, 0x6b58, PDF_CMAP_SINGLE, 2326 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 2327 },
	{ 0x6b60, 0x6b60, PDF_CMAP_SINGLE, 2328 },
	{ 0x6b67, 0x6b67, PDF_CMAP_SINGLE, 2329 },
	{ 0x6b6b, 0x6b6b, PDF_CMAP_SINGLE, 2330 },
	{ 0x6b6e, 0x6b6e, PDF_CMAP_SINGLE, 2331 },
	{ 0x6b70, 0x6b70, PDF_CMAP_SINGLE, 2332 },
	{ 0x6b75, 0x6b75, PDF_CMAP_SINGLE, 2333 },
	{ 0x6b7d, 0x6b7e, PDF_CMAP_RANGE, 2334 },
	{ 0x6b82, 0x6b82, PDF_CMAP_SINGLE, 2336 },
	{ 0x6b85, 0x6b85, PDF_CMAP_SINGLE, 2337 },
	{ 0x6b97, 0x6b97, PDF_CMAP_SINGLE, 2338 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 2339 },
	{ 0x6b9f, 0x6ba0, PDF_CMAP_RANGE, 2340 },
	{ 0x6ba2, 0x6ba3, PDF_CMAP_RANGE, 2342 },
	{ 0x6ba8, 0x6ba9, PDF_CMAP_RANGE, 2344 },
	{ 0x6bac, 0x6bae, PDF_CMAP_RANGE, 2346 },
	{ 0x6bb0, 0x6bb0, PDF_CMAP_SINGLE, 2349 },
	{ 0x6bb8, 0x6bb9, PDF_CMAP_RANGE, 2350 },
	{ 0x6bbd, 0x6bbe, PDF_CMAP_RANGE, 2352 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 2354 },
	{ 0x6bc9, 0x6bc9, PDF_CMAP_SINGLE, 2356 },
	{ 0x6bcc, 0x6bcc, PDF_CMAP_SINGLE, 2357 },
	{ 0x6bd6, 0x6bd6, PDF_CMAP_SINGLE, 2358 },
	{ 0x6bda, 0x6bda, PDF_CMAP_SINGLE, 2359 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 2360 },
	{ 0x6be3, 0x6be3, PDF_CMAP_SINGLE, 2361 },
	{ 0x6be6, 0x6be7, PDF_CMAP_RANGE, 2362 },
	{ 0x6bee, 0x6bee, PDF_CMAP_SINGLE, 2364 },
	{ 0x6bf1, 0x6bf1, PDF_CMAP_SINGLE, 2365 },
	{ 0x6bf7, 0x6bf7, PDF_CMAP_SINGLE, 2366 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 2367 },
	{ 0x6bff, 0x6bff, PDF_CMAP_SINGLE, 2368 },
	{ 0x6c02, 0x6c02, PDF_CMAP_SINGLE, 2369 },
	{ 0x6c04, 0x6c05, PDF_CMAP_RANGE, 2370 },
	{ 0x6c09, 0x6c09, PDF_CMAP_SINGLE, 2372 },
	{ 0x6c0d, 0x6c0e, PDF_CMAP_RANGE, 2373 },
	{ 0x6c10, 0x6c10, PDF_CMAP_SINGLE, 2375 },
	{ 0x6c12, 0x6c12, PDF_CMAP_SINGLE, 2376 },
	{ 0x6c19, 0x6c19, PDF_CMAP_SINGLE, 2377 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 2378 },
	{ 0x6c26, 0x6c28, PDF_CMAP_RANGE, 2379 },
	{ 0x6c2c, 0x6c2c, PDF_CMAP_SINGLE, 2382 },
	{ 0x6c2e, 0x6c2e, PDF_CMAP_SINGLE, 2383 },
	{ 0x6c33, 0x6c33, PDF_CMAP_SINGLE, 2384 },
	{ 0x6c35, 0x6c36, PDF_CMAP_RANGE, 2385 },
	{ 0x6c3a, 0x6c3b, PDF_CMAP_RANGE, 2387 },
	{ 0x6c3f, 0x6c3f, PDF_CMAP_SINGLE, 2389 },
	{ 0x6c4a, 0x6c4b, PDF_CMAP_RANGE, 2390 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2392 },
	{ 0x6c4f, 0x6c4f, PDF_CMAP_SINGLE, 2393 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 2394 },
	{ 0x6c54, 0x6c54, PDF_CMAP_SINGLE, 2395 },
	{ 0x6c59, 0x6c59, PDF_CMAP_SINGLE, 2396 },
	{ 0x6c5b, 0x6c5c, PDF_CMAP_RANGE, 2397 },
	{ 0x6c67, 0x6c67, PDF_CMAP_SINGLE, 2450 },
	{ 0x6c6b, 0x6c6b, PDF_CMAP_SINGLE, 2399 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 2400 },
	{ 0x6c6f, 0x6c6f, PDF_CMAP_SINGLE, 2401 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 2402 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 2403 },
	{ 0x6c78, 0x6c79, PDF_CMAP_RANGE, 2404 },
	{ 0x6c7b, 0x6c7b, PDF_CMAP_SINGLE, 2406 },
	{ 0x6c85, 0x6c87, PDF_CMAP_RANGE, 2407 },
	{ 0x6c89, 0x6c89, PDF_CMAP_SINGLE, 2410 },
	{ 0x6c94, 0x6c95, PDF_CMAP_RANGE, 2411 },
	{ 0x6c97, 0x6c98, PDF_CMAP_RANGE, 2413 },
	{ 0x6c9c, 0x6c9c, PDF_CMAP_SINGLE, 2415 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 2416 },
	{ 0x6cb0, 0x6cb0, PDF_CMAP_SINGLE, 2417 },
	{ 0x6cb2, 0x6cb2, PDF_CMAP_SINGLE, 2418 },
	{ 0x6cb4, 0x6cb4, PDF_CMAP_SINGLE, 2419 },
	{ 0x6cc2, 0x6cc2, PDF_CMAP_SINGLE, 2420 },
	{ 0x6cc6, 0x6cc6, PDF_CMAP_SINGLE, 2421 },
	{ 0x6ccd, 0x6ccd, PDF_CMAP_SINGLE, 2422 },
	{ 0x6ccf, 0x6cd2, PDF_CMAP_RANGE, 2423 },
	{ 0x6cd4, 0x6cd4, PDF_CMAP_SINGLE, 2427 },
	{ 0x6cd6, 0x6cd6, PDF_CMAP_SINGLE, 2428 },
	{ 0x6cda, 0x6cda, PDF_CMAP_SINGLE, 2429 },
	{ 0x6cdc, 0x6cdc, PDF_CMAP_SINGLE, 2430 },
	{ 0x6ce0, 0x6ce0, PDF_CMAP_SINGLE, 2431 },
	{ 0x6ce7, 0x6ce7, PDF_CMAP_SINGLE, 2432 },
	{ 0x6ce9, 0x6ce9, PDF_CMAP_SINGLE, 2433 },
	{ 0x6ceb, 0x6cec, PDF_CMAP_RANGE, 2434 },
	{ 0x6cee, 0x6cee, PDF_CMAP_SINGLE, 2436 },
	{ 0x6cf2, 0x6cf2, PDF_CMAP_SINGLE, 2437 },
	{ 0x6cf4, 0x6cf4, PDF_CMAP_SINGLE, 2438 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 2439 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 2440 },
	{ 0x6d0a, 0x6d0a, PDF_CMAP_SINGLE, 2441 },
	{ 0x6d0e, 0x6d0f, PDF_CMAP_RANGE, 2442 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 2444 },
	{ 0x6d13, 0x6d13, PDF_CMAP_SINGLE, 2445 },
	{ 0x6d1a, 0x6d1a, PDF_CMAP_SINGLE, 2446 },
	{ 0x6d26, 0x6d28, PDF_CMAP_RANGE, 2447 },
	{ 0x6d2e, 0x6d2f, PDF_CMAP_RANGE, 2451 },
	{ 0x6d31, 0x6d31, PDF_CMAP_SINGLE, 2453 },
	{ 0x6d39, 0x6d39, PDF_CMAP_SINGLE, 2454 },
	{ 0x6d3c, 0x6d3c, PDF_CMAP_SINGLE, 2455 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 2456 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 2457 },
	{ 0x6d5e, 0x6d5f, PDF_CMAP_RANGE, 2458 },
	{ 0x6d61, 0x6d61, PDF_CMAP_SINGLE, 2460 },
	{ 0x6d65, 0x6d65, PDF_CMAP_SINGLE, 2461 },
	{ 0x6d67, 0x6d67, PDF_CMAP_SINGLE, 2462 },
	{ 0x6d6f, 0x6d70, PDF_CMAP_RANGE, 2463 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 2465 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 2466 },
	{ 0x6d87, 0x6d87, PDF_CMAP_SINGLE, 2467 },
	{ 0x6d91, 0x6d92, PDF_CMAP_RANGE, 2468 },
	{ 0x6d94, 0x6d94, PDF_CMAP_SINGLE, 2470 },
	{ 0x6d96, 0x6d98, PDF_CMAP_RANGE, 2471 },
	{ 0x6daa, 0x6daa, PDF_CMAP_SINGLE, 2474 },
	{ 0x6dac, 0x6dac, PDF_CMAP_SINGLE, 2475 },
	{ 0x6db4, 0x6db4, PDF_CMAP_SINGLE, 2476 },
	{ 0x6db7, 0x6db7, PDF_CMAP_SINGLE, 2477 },
	{ 0x6db9, 0x6db9, PDF_CMAP_SINGLE, 2478 },
	{ 0x6dbd, 0x6dbd, PDF_CMAP_SINGLE, 2479 },
	{ 0x6dbf, 0x6dbf, PDF_CMAP_SINGLE, 2480 },
	{ 0x6dc4, 0x6dc4, PDF_CMAP_SINGLE, 2481 },
	{ 0x6dc8, 0x6dc8, PDF_CMAP_SINGLE, 2482 },
	{ 0x6dca, 0x6dca, PDF_CMAP_SINGLE, 2483 },
	{ 0x6dce, 0x6dcf, PDF_CMAP_RANGE, 2484 },
	{ 0x6dd6, 0x6dd6, PDF_CMAP_SINGLE, 2486 },
	{ 0x6ddb, 0x6ddb, PDF_CMAP_SINGLE, 2487 },
	{ 0x6ddd, 0x6ddd, PDF_CMAP_SINGLE, 2488 },
	{ 0x6ddf, 0x6de0, PDF_CMAP_RANGE, 2489 },
	{ 0x6de2, 0x6de2, PDF_CMAP_SINGLE, 2491 },
	{ 0x6de5, 0x6de5, PDF_CMAP_SINGLE, 2492 },
	{ 0x6de9, 0x6de9, PDF_CMAP_SINGLE, 2493 },
	{ 0x6def, 0x6df0, PDF_CMAP_RANGE, 2494 },
	{ 0x6df4, 0x6df4, PDF_CMAP_SINGLE, 2496 },
	{ 0x6df6, 0x6df6, PDF_CMAP_SINGLE, 2497 },
	{ 0x6dfc, 0x6dfc, PDF_CMAP_SINGLE, 2498 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 2499 },
	{ 0x6e04, 0x6e04, PDF_CMAP_SINGLE, 2500 },
	{ 0x6e1e, 0x6e1e, PDF_CMAP_SINGLE, 2501 },
	{ 0x6e22, 0x6e22, PDF_CMAP_SINGLE, 2502 },
	{ 0x6e27, 0x6e27, PDF_CMAP_SINGLE, 2503 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 2504 },
	{ 0x6e36, 0x6e36, PDF_CMAP_SINGLE, 2505 },
	{ 0x6e39, 0x6e39, PDF_CMAP_SINGLE, 2506 },
	{ 0x6e3b, 0x6e3c, PDF_CMAP_RANGE, 2507 },
	{ 0x6e44, 0x6e45, PDF_CMAP_RANGE, 2509 },
	{ 0x6e48, 0x6e49, PDF_CMAP_RANGE, 2511 },
	{ 0x6e4b, 0x6e4b, PDF_CMAP_SINGLE, 2513 },
	{ 0x6e4f, 0x6e4f, PDF_CMAP_SINGLE, 2514 },
	{ 0x6e51, 0x6e54, PDF_CMAP_RANGE, 2515 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 2519 },
	{ 0x6e5c, 0x6e5e, PDF_CMAP_RANGE, 2520 },
	{ 0x6e62, 0x6e63, PDF_CMAP_RANGE, 2523 },
	{ 0x6e68, 0x6e68, PDF_CMAP_SINGLE, 2525 },
	{ 0x6e73, 0x6e73, PDF_CMAP_SINGLE, 2526 },
	{ 0x6e7b, 0x6e7b, PDF_CMAP_SINGLE, 2527 },
	{ 0x6e7d, 0x6e7d, PDF_CMAP_SINGLE, 2528 },
	{ 0x6e8d, 0x6e8d, PDF_CMAP_SINGLE, 2529 },
	{ 0x6e93, 0x6e93, PDF_CMAP_SINGLE, 2530 },
	{ 0x6e99, 0x6e99, PDF_CMAP_SINGLE, 2531 },
	{ 0x6ea0, 0x6ea0, PDF_CMAP_SINGLE, 2532 },
	{ 0x6ea7, 0x6ea7, PDF_CMAP_SINGLE, 2533 },
	{ 0x6ead, 0x6eae, PDF_CMAP_RANGE, 2534 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 2536 },
	{ 0x6eb3, 0x6eb3, PDF_CMAP_SINGLE, 2537 },
	{ 0x6ebb, 0x6ebb, PDF_CMAP_SINGLE, 2538 },
	{ 0x6ebf, 0x6ec1, PDF_CMAP_RANGE, 2539 },
	{ 0x6ec3, 0x6ec3, PDF_CMAP_SINGLE, 2542 },
	{ 0x6ec7, 0x6ec8, PDF_CMAP_RANGE, 2543 },
	{ 0x6eca, 0x6eca, PDF_CMAP_SINGLE, 2545 },
	{ 0x6ecd, 0x6ecf, PDF_CMAP_RANGE, 2546 },
	{ 0x6eeb, 0x6eeb, PDF_CMAP_SINGLE, 2549 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 2550 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 2552 },
	{ 0x6efb, 0x6efb, PDF_CMAP_SINGLE, 2553 },
	{ 0x6efd, 0x6efd, PDF_CMAP_SINGLE, 2554 },
	{ 0x6f04, 0x6f04, PDF_CMAP_SINGLE, 2555 },
	{ 0x6f08, 0x6f08, PDF_CMAP_SINGLE, 2556 },
	{ 0x6f0a, 0x6f0a, PDF_CMAP_SINGLE, 2557 },
	{ 0x6f0c, 0x6f0d, PDF_CMAP_RANGE, 2558 },
	{ 0x6f16, 0x6f16, PDF_CMAP_SINGLE, 2560 },
	{ 0x6f18, 0x6f18, PDF_CMAP_SINGLE, 2561 },
	{ 0x6f1a, 0x6f1b, PDF_CMAP_RANGE, 2562 },
	{ 0x6f26, 0x6f26, PDF_CMAP_SINGLE, 2564 },
	{ 0x6f29, 0x6f2a, PDF_CMAP_RANGE, 2565 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 2573 },
	{ 0x6f2f, 0x6f30, PDF_CMAP_RANGE, 2567 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 2569 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 2570 },
	{ 0x6f3b, 0x6f3c, PDF_CMAP_RANGE, 2571 },
	{ 0x6f4f, 0x6f4f, PDF_CMAP_SINGLE, 2574 },
	{ 0x6f51, 0x6f53, PDF_CMAP_RANGE, 2575 },
	{ 0x6f57, 0x6f57, PDF_CMAP_SINGLE, 2578 },
	{ 0x6f59, 0x6f5a, PDF_CMAP_RANGE, 2579 },
	{ 0x6f5d, 0x6f5e, PDF_CMAP_RANGE, 2581 },
	{ 0x6f61, 0x6f62, PDF_CMAP_RANGE, 2583 },
	{ 0x6f68, 0x6f68, PDF_CMAP_SINGLE, 2585 },
	{ 0x6f6c, 0x6f6c, PDF_CMAP_SINGLE, 2586 },
	{ 0x6f7d, 0x6f7e, PDF_CMAP_RANGE, 2587 },
	{ 0x6f83, 0x6f83, PDF_CMAP_SINGLE, 2589 },
	{ 0x6f87, 0x6f88, PDF_CMAP_RANGE, 2590 },
	{ 0x6f8b, 0x6f8d, PDF_CMAP_RANGE, 2592 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 2595 },
	{ 0x6f92, 0x6f94, PDF_CMAP_RANGE, 2596 },
	{ 0x6f96, 0x6f96, PDF_CMAP_SINGLE, 2599 },
	{ 0x6f9a, 0x6f9a, PDF_CMAP_SINGLE, 2600 },
	{ 0x6f9f, 0x6fa0, PDF_CMAP_RANGE, 2601 },
	{ 0x6fa5, 0x6fa8, PDF_CMAP_RANGE, 2603 },
	{ 0x6fae, 0x6fb0, PDF_CMAP_RANGE, 2607 },
	{ 0x6fb5, 0x6fb6, PDF_CMAP_RANGE, 2610 },
	{ 0x6fbc, 0x6fbc, PDF_CMAP_SINGLE, 2612 },
	{ 0x6fc5, 0x6fc5, PDF_CMAP_SINGLE, 2613 },
	{ 0x6fc7, 0x6fc8, PDF_CMAP_RANGE, 2614 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 2616 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 2617 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 2618 },
	{ 0x6fe8, 0x6fe9, PDF_CMAP_RANGE, 2619 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 2621 },
	{ 0x6ff5, 0x6ff5, PDF_CMAP_SINGLE, 2622 },
	{ 0x6ff9, 0x6ff9, PDF_CMAP_SINGLE, 2623 },
	{ 0x6ffc, 0x6ffd, PDF_CMAP_RANGE, 2624 },
	{ 0x7000, 0x7000, PDF_CMAP_SINGLE, 2626 },
	{ 0x7005, 0x7007, PDF_CMAP_RANGE, 2627 },
	{ 0x700d, 0x700d, PDF_CMAP_SINGLE, 2630 },
	{ 0x7017, 0x7017, PDF_CMAP_SINGLE, 2631 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 2632 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 2633 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 2634 },
	{ 0x7034, 0x7034, PDF_CMAP_SINGLE, 2635 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 2636 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 2637 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 2638 },
	{ 0x7043, 0x7044, PDF_CMAP_RANGE, 2639 },
	{ 0x7048, 0x704b, PDF_CMAP_RANGE, 2641 },
	{ 0x704e, 0x704e, PDF_CMAP_SINGLE, 2649 },
	{ 0x7054, 0x7055, PDF_CMAP_RANGE, 2645 },
	{ 0x705d, 0x705e, PDF_CMAP_RANGE, 2647 },
	{ 0x7064, 0x7065, PDF_CMAP_RANGE, 2650 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 2652 },
	{ 0x706e, 0x706e, PDF_CMAP_SINGLE, 2653 },
	{ 0x7075, 0x7076, PDF_CMAP_RANGE, 2654 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 2656 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 2657 },
	{ 0x7085, 0x7086, PDF_CMAP_RANGE, 2658 },
	{ 0x7094, 0x7098, PDF_CMAP_RANGE, 2660 },
	{ 0x709b, 0x709b, PDF_CMAP_SINGLE, 2665 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 2666 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 2667 },
	{ 0x70b0, 0x70b1, PDF_CMAP_RANGE, 2668 },
	{ 0x70b4, 0x70b4, PDF_CMAP_SINGLE, 2670 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 2671 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 2672 },
	{ 0x70d1, 0x70d1, PDF_CMAP_SINGLE, 2673 },
	{ 0x70d3, 0x70d6, PDF_CMAP_RANGE, 2674 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 2678 },
	{ 0x70dc, 0x70dc, PDF_CMAP_SINGLE, 2679 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2680 },
	{ 0x70fa, 0x70fa, PDF_CMAP_SINGLE, 2681 },
	{ 0x7103, 0x7107, PDF_CMAP_RANGE, 2682 },
	{ 0x710b, 0x710c, PDF_CMAP_RANGE, 2687 },
	{ 0x710f, 0x710f, PDF_CMAP_SINGLE, 2689 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 2690 },
	{ 0x7120, 0x7120, PDF_CMAP_SINGLE, 2691 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 2692 },
	{ 0x712d, 0x712d, PDF_CMAP_SINGLE, 2693 },
	{ 0x712f, 0x7131, PDF_CMAP_RANGE, 2694 },
	{ 0x7138, 0x7138, PDF_CMAP_SINGLE, 2697 },
	{ 0x7141, 0x7141, PDF_CMAP_SINGLE, 2698 },
	{ 0x7145, 0x7147, PDF_CMAP_RANGE, 2699 },
	{ 0x714a, 0x714b, PDF_CMAP_RANGE, 2702 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 2704 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 2705 },
	{ 0x7157, 0x7157, PDF_CMAP_SINGLE, 2706 },
	{ 0x715a, 0x715a, PDF_CMAP_SINGLE, 2707 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 2708 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 2709 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 2710 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 2711 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 2712 },
	{ 0x7180, 0x7180, PDF_CMAP_SINGLE, 2713 },
	{ 0x7185, 0x7185, PDF_CMAP_SINGLE, 2714 },
	{ 0x7187, 0x7187, PDF_CMAP_SINGLE, 2715 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 2716 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 2717 },
	{ 0x719a, 0x719b, PDF_CMAP_RANGE, 2718 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 2720 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 2721 },
	{ 0x71af, 0x71b0, PDF_CMAP_RANGE, 2722 },
	{ 0x71b2, 0x71b3, PDF_CMAP_RANGE, 2724 },
	{ 0x71ba, 0x71ba, PDF_CMAP_SINGLE, 2726 },
	{ 0x71bf, 0x71c1, PDF_CMAP_RANGE, 2727 },
	{ 0x71c4, 0x71c4, PDF_CMAP_SINGLE, 2730 },
	{ 0x71cb, 0x71cc, PDF_CMAP_RANGE, 2731 },
	{ 0x71d3, 0x71d3, PDF_CMAP_SINGLE, 2733 },
	{ 0x71d6, 0x71d6, PDF_CMAP_SINGLE, 2734 },
	{ 0x71d9, 0x71da, PDF_CMAP_RANGE, 2735 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 2737 },
	{ 0x71f8, 0x71f8, PDF_CMAP_SINGLE, 2738 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 2739 },
	{ 0x7200, 0x7200, PDF_CMAP_SINGLE, 2740 },
	{ 0x7207, 0x7209, PDF_CMAP_RANGE, 2741 },
	{ 0x7213, 0x7213, PDF_CMAP_SINGLE, 2744 },
	{ 0x7217, 0x7217, PDF_CMAP_SINGLE, 2745 },
	{ 0x721a, 0x721a, PDF_CMAP_SINGLE, 2746 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 2747 },
	{ 0x721f, 0x721f, PDF_CMAP_SINGLE, 2748 },
	{ 0x7224, 0x7224, PDF_CMAP_SINGLE, 2749 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 2750 },
	{ 0x722f, 0x722f, PDF_CMAP_SINGLE, 2751 },
	{ 0x7234, 0x7234, PDF_CMAP_SINGLE, 2752 },
	{ 0x7238, 0x7239, PDF_CMAP_RANGE, 2753 },
	{ 0x7241, 0x7243, PDF_CMAP_RANGE, 2755 },
	{ 0x7245, 0x7245, PDF_CMAP_SINGLE, 2758 },
	{ 0x724e, 0x7250, PDF_CMAP_RANGE, 2759 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 2762 },
	{ 0x7255, 0x7256, PDF_CMAP_RANGE, 2763 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 2765 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 2766 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 2767 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 2768 },
	{ 0x7263, 0x7263, PDF_CMAP_SINGLE, 2769 },
	{ 0x7268, 0x7268, PDF_CMAP_SINGLE, 2770 },
	{ 0x726b, 0x726b, PDF_CMAP_SINGLE, 2771 },
	{ 0x726e, 0x726f, PDF_CMAP_RANGE, 2772 },
	{ 0x7271, 0x7271, PDF_CMAP_SINGLE, 2774 },
	{ 0x7277, 0x7278, PDF_CMAP_RANGE, 2775 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 2777 },
	{ 0x727f, 0x727f, PDF_CMAP_SINGLE, 2779 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 2780 },
	{ 0x7289, 0x7289, PDF_CMAP_SINGLE, 2781 },
	{ 0x728d, 0x728e, PDF_CMAP_RANGE, 2782 },
	{ 0x7293, 0x7293, PDF_CMAP_SINGLE, 2784 },
	{ 0x729b, 0x729b, PDF_CMAP_SINGLE, 2785 },
	{ 0x72a8, 0x72a8, PDF_CMAP_SINGLE, 2786 },
	{ 0x72ad, 0x72ae, PDF_CMAP_RANGE, 2787 },
	{ 0x72b1, 0x72b1, PDF_CMAP_SINGLE, 2789 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 2790 },
	{ 0x72be, 0x72be, PDF_CMAP_SINGLE, 2791 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 2792 },
	{ 0x72c7, 0x72c7, PDF_CMAP_SINGLE, 2793 },
	{ 0x72c9, 0x72c9, PDF_CMAP_SINGLE, 2794 },
	{ 0x72cc, 0x72cc, PDF_CMAP_SINGLE, 2795 },
	{ 0x72d5, 0x72d6, PDF_CMAP_RANGE, 2796 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 2798 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 2799 },
	{ 0x72e5, 0x72e5, PDF_CMAP_SINGLE, 2800 },
	{ 0x72f3, 0x72f4, PDF_CMAP_RANGE, 2801 },
	{ 0x72fa, 0x72fb, PDF_CMAP_RANGE, 2803 },
	{ 0x72fe, 0x72fe, PDF_CMAP_SINGLE, 2805 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 2806 },
	{ 0x7304, 0x7305, PDF_CMAP_RANGE, 2807 },
	{ 0x7307, 0x7307, PDF_CMAP_SINGLE, 2809 },
	{ 0x730b, 0x730b, PDF_CMAP_SINGLE, 2810 },
	{ 0x730d, 0x730d, PDF_CMAP_SINGLE, 2811 },
	{ 0x7312, 0x7313, PDF_CMAP_RANGE, 2812 },
	{ 0x7318, 0x7319, PDF_CMAP_RANGE, 2814 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 2816 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 2817 },
	{ 0x7324, 0x7324, PDF_CMAP_SINGLE, 2818 },
	{ 0x7327, 0x7328, PDF_CMAP_RANGE, 2819 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 2821 },
	{ 0x7331, 0x7332, PDF_CMAP_RANGE, 2822 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 2824 },
	{ 0x733a, 0x733b, PDF_CMAP_RANGE, 2825 },
	{ 0x733d, 0x733d, PDF_CMAP_SINGLE, 2827 },
	{ 0x7343, 0x7343, PDF_CMAP_SINGLE, 2828 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 2829 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 2830 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 2831 },
	{ 0x7356, 0x7356, PDF_CMAP_SINGLE, 2832 },
	{ 0x7358, 0x7358, PDF_CMAP_SINGLE, 2833 },
	{ 0x735d, 0x7360, PDF_CMAP_RANGE, 2834 },
	{ 0x7366, 0x7367, PDF_CMAP_RANGE, 2838 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 2840 },
	{ 0x736b, 0x736c, PDF_CMAP_RANGE, 2841 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 2843 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 2845 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 2846 },
	{ 0x7379, 0x7379, PDF_CMAP_SINGLE, 2847 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 2848 },
	{ 0x7380, 0x7381, PDF_CMAP_RANGE, 2849 },
	{ 0x7383, 0x7383, PDF_CMAP_SINGLE, 2851 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 2852 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 2854 },
	{ 0x7390, 0x7390, PDF_CMAP_SINGLE, 2855 },
	{ 0x7393, 0x7393, PDF_CMAP_SINGLE, 2856 },
	{ 0x7395, 0x7395, PDF_CMAP_SINGLE, 2857 },
	{ 0x7397, 0x7398, PDF_CMAP_RANGE, 2858 },
	{ 0x739c, 0x739c, PDF_CMAP_SINGLE, 2860 },
	{ 0x739e, 0x73a0, PDF_CMAP_RANGE, 2861 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 2864 },
	{ 0x73a5, 0x73a6, PDF_CMAP_RANGE, 2865 },
	{ 0x73aa, 0x73ab, PDF_CMAP_RANGE, 2867 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 2869 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 2870 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 2871 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 2872 },
	{ 0x73bc, 0x73bd, PDF_CMAP_RANGE, 2873 },
	{ 0x73bf, 0x73bf, PDF_CMAP_SINGLE, 2875 },
	{ 0x73c5, 0x73c6, PDF_CMAP_RANGE, 2876 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 2878 },
	{ 0x73cb, 0x73cc, PDF_CMAP_RANGE, 2879 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 2881 },
	{ 0x73d2, 0x73d3, PDF_CMAP_RANGE, 2882 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 2884 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 2885 },
	{ 0x73dd, 0x73dd, PDF_CMAP_SINGLE, 2886 },
	{ 0x73e1, 0x73e1, PDF_CMAP_SINGLE, 2887 },
	{ 0x73e3, 0x73e3, PDF_CMAP_SINGLE, 2888 },
	{ 0x73e6, 0x73e7, PDF_CMAP_RANGE, 2889 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 2891 },
	{ 0x73f4, 0x73f5, PDF_CMAP_RANGE, 2892 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 2894 },
	{ 0x73f9, 0x73fb, PDF_CMAP_RANGE, 2895 },
	{ 0x73fd, 0x73fd, PDF_CMAP_SINGLE, 2898 },
	{ 0x73ff, 0x7401, PDF_CMAP_RANGE, 2899 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 2902 },
	{ 0x7407, 0x7407, PDF_CMAP_SINGLE, 2903 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 2904 },
	{ 0x7411, 0x7411, PDF_CMAP_SINGLE, 2905 },
	{ 0x741a, 0x741b, PDF_CMAP_RANGE, 2906 },
	{ 0x7424, 0x7424, PDF_CMAP_SINGLE, 2908 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 2909 },
	{ 0x7428, 0x7431, PDF_CMAP_RANGE, 2910 },
	{ 0x7439, 0x7439, PDF_CMAP_SINGLE, 2920 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 2921 },
	{ 0x7443, 0x7444, PDF_CMAP_RANGE, 2922 },
	{ 0x7446, 0x7447, PDF_CMAP_RANGE, 2924 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 2926 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 2927 },
	{ 0x7451, 0x7452, PDF_CMAP_RANGE, 2928 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 2930 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 2931 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 2932 },
	{ 0x7466, 0x7468, PDF_CMAP_RANGE, 2933 },
	{ 0x746b, 0x746b, PDF_CMAP_SINGLE, 2936 },
	{ 0x746d, 0x746e, PDF_CMAP_RANGE, 2937 },
	{ 0x7471, 0x7472, PDF_CMAP_RANGE, 2939 },
	{ 0x7480, 0x7481, PDF_CMAP_RANGE, 2941 },
	{ 0x7485, 0x7487, PDF_CMAP_RANGE, 2943 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 2946 },
	{ 0x748f, 0x7492, PDF_CMAP_RANGE, 2947 },
	{ 0x7498, 0x749a, PDF_CMAP_RANGE, 2951 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 2954 },
	{ 0x749f, 0x74a1, PDF_CMAP_RANGE, 2955 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 2958 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 2959 },
	{ 0x74a8, 0x74ab, PDF_CMAP_RANGE, 2960 },
	{ 0x74ae, 0x74af, PDF_CMAP_RANGE, 2964 },
	{ 0x74b1, 0x74b2, PDF_CMAP_RANGE, 2966 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 2968 },
	{ 0x74b9, 0x74b9, PDF_CMAP_SINGLE, 2969 },
	{ 0x74bb, 0x74bb, PDF_CMAP_SINGLE, 2970 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 2971 },
	{ 0x74c8, 0x74c9, PDF_CMAP_RANGE, 2972 },
	{ 0x74cc, 0x74cc, PDF_CMAP_SINGLE, 2974 },
	{ 0x74d0, 0x74d0, PDF_CMAP_SINGLE, 2975 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 2976 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 2977 },
	{ 0x74da, 0x74db, PDF_CMAP_RANGE, 2978 },
	{ 0x74de, 0x74df, PDF_CMAP_RANGE, 2980 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 2982 },
	{ 0x74e8, 0x74e8, PDF_CMAP_SINGLE, 2983 },
	{ 0x74ea, 0x74eb, PDF_CMAP_RANGE, 2984 },
	{ 0x74ef, 0x74ef, PDF_CMAP_SINGLE, 2986 },
	{ 0x74f4, 0x74f4, PDF_CMAP_SINGLE, 2987 },
	{ 0x74fa, 0x74fc, PDF_CMAP_RANGE, 2988 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 2991 },
	{ 0x7506, 0x7506, PDF_CMAP_SINGLE, 2992 },
	{ 0x7512, 0x7512, PDF_CMAP_SINGLE, 2993 },
	{ 0x7516, 0x7517, PDF_CMAP_RANGE, 2994 },
	{ 0x7520, 0x7521, PDF_CMAP_RANGE, 2996 },
	{ 0x7524, 0x7524, PDF_CMAP_SINGLE, 2998 },
	{ 0x7527, 0x7527, PDF_CMAP_SINGLE, 2999 },
	{ 0x7529, 0x752a, PDF_CMAP_RANGE, 3000 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 3002 },
	{ 0x7536, 0x7536, PDF_CMAP_SINGLE, 3003 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 3004 },
	{ 0x753d, 0x7540, PDF_CMAP_RANGE, 3005 },
	{ 0x7543, 0x7543, PDF_CMAP_SINGLE, 3009 },
	{ 0x7547, 0x7548, PDF_CMAP_RANGE, 3010 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 3012 },
	{ 0x7550, 0x7550, PDF_CMAP_SINGLE, 3013 },
	{ 0x7552, 0x7552, PDF_CMAP_SINGLE, 3014 },
	{ 0x7557, 0x7557, PDF_CMAP_SINGLE, 3015 },
	{ 0x755e, 0x755f, PDF_CMAP_RANGE, 3016 },
	{ 0x7561, 0x7561, PDF_CMAP_SINGLE, 3018 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 3019 },
	{ 0x7571, 0x7571, PDF_CMAP_SINGLE, 3020 },
	{ 0x7579, 0x757e, PDF_CMAP_RANGE, 3021 },
	{ 0x7581, 0x7581, PDF_CMAP_SINGLE, 3027 },
	{ 0x7585, 0x7585, PDF_CMAP_SINGLE, 3028 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 3029 },
	{ 0x7592, 0x7593, PDF_CMAP_RANGE, 3030 },
	{ 0x7595, 0x7595, PDF_CMAP_SINGLE, 3032 },
	{ 0x7599, 0x7599, PDF_CMAP_SINGLE, 3033 },
	{ 0x759c, 0x759c, PDF_CMAP_SINGLE, 3034 },
	{ 0x75a2, 0x75a2, PDF_CMAP_SINGLE, 3035 },
	{ 0x75a4, 0x75a4, PDF_CMAP_SINGLE, 3036 },
	{ 0x75b4, 0x75b4, PDF_CMAP_SINGLE, 3037 },
	{ 0x75ba, 0x75ba, PDF_CMAP_SINGLE, 3038 },
	{ 0x75bf, 0x75c1, PDF_CMAP_RANGE, 3039 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 3042 },
	{ 0x75c6, 0x75c6, PDF_CMAP_SINGLE, 3043 },
	{ 0x75cc, 0x75cc, PDF_CMAP_SINGLE, 3044 },
	{ 0x75ce, 0x75cf, PDF_CMAP_RANGE, 3045 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 3047 },
	{ 0x75dc, 0x75dc, PDF_CMAP_SINGLE, 3048 },
	{ 0x75df, 0x75e1, PDF_CMAP_RANGE, 3049 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 3052 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 3053 },
	{ 0x75ec, 0x75ec, PDF_CMAP_SINGLE, 3054 },
	{ 0x75ee, 0x75ef, PDF_CMAP_RANGE, 3055 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 3057 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 3058 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 3059 },
	{ 0x7602, 0x7604, PDF_CMAP_RANGE, 3060 },
	{ 0x7607, 0x7608, PDF_CMAP_RANGE, 3063 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 3065 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 3066 },
	{ 0x760f, 0x760f, PDF_CMAP_SINGLE, 3067 },
	{ 0x7612, 0x7613, PDF_CMAP_RANGE, 3068 },
	{ 0x7615, 0x7616, PDF_CMAP_RANGE, 3070 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 3072 },
	{ 0x761b, 0x761e, PDF_CMAP_RANGE, 3073 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 3077 },
	{ 0x7625, 0x7626, PDF_CMAP_RANGE, 3078 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 3080 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 3081 },
	{ 0x7632, 0x7633, PDF_CMAP_RANGE, 3082 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 3084 },
	{ 0x7638, 0x763a, PDF_CMAP_RANGE, 3085 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 3088 },
	{ 0x7640, 0x7641, PDF_CMAP_RANGE, 3090 },
	{ 0x7643, 0x7645, PDF_CMAP_RANGE, 3092 },
	{ 0x7649, 0x764b, PDF_CMAP_TABLE, 241 },
	{ 0x7655, 0x7655, PDF_CMAP_SINGLE, 3097 },
	{ 0x7659, 0x7659, PDF_CMAP_SINGLE, 3098 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 3099 },
	{ 0x7664, 0x7665, PDF_CMAP_RANGE, 3100 },
	{ 0x766d, 0x766f, PDF_CMAP_RANGE, 3102 },
	{ 0x7671, 0x7671, PDF_CMAP_SINGLE, 3105 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 3106 },
	{ 0x7681, 0x7681, PDF_CMAP_SINGLE, 3107 },
	{ 0x7685, 0x7685, PDF_CMAP_SINGLE, 3108 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 3109 },
	{ 0x7695, 0x7695, PDF_CMAP_SINGLE, 3111 },
	{ 0x769b, 0x769d, PDF_CMAP_RANGE, 3112 },
	{ 0x769f, 0x76a0, PDF_CMAP_RANGE, 3115 },
	{ 0x76a2, 0x76a8, PDF_CMAP_RANGE, 3117 },
	{ 0x76aa, 0x76aa, PDF_CMAP_SINGLE, 3124 },
	{ 0x76ad, 0x76ad, PDF_CMAP_SINGLE, 3125 },
	{ 0x76bd, 0x76bd, PDF_CMAP_SINGLE, 3126 },
	{ 0x76c1, 0x76c1, PDF_CMAP_SINGLE, 3127 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 3128 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 3129 },
	{ 0x76cb, 0x76cc, PDF_CMAP_RANGE, 3130 },
	{ 0x76ce, 0x76ce, PDF_CMAP_SINGLE, 3132 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 3133 },
	{ 0x76d9, 0x76d9, PDF_CMAP_SINGLE, 3134 },
	{ 0x76e0, 0x76e0, PDF_CMAP_SINGLE, 3135 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 3136 },
	{ 0x76e8, 0x76e8, PDF_CMAP_SINGLE, 3137 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 3138 },
	{ 0x76f0, 0x76f1, PDF_CMAP_RANGE, 3139 },
	{ 0x76f6, 0x76f6, PDF_CMAP_SINGLE, 3141 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 3142 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 3143 },
	{ 0x7700, 0x7700, PDF_CMAP_SINGLE, 3144 },
	{ 0x7706, 0x7706, PDF_CMAP_SINGLE, 3145 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 3146 },
	{ 0x770e, 0x770e, PDF_CMAP_SINGLE, 3147 },
	{ 0x7712, 0x7712, PDF_CMAP_SINGLE, 3148 },
	{ 0x7714, 0x7715, PDF_CMAP_RANGE, 3149 },
	{ 0x7717, 0x7717, PDF_CMAP_SINGLE, 3151 },
	{ 0x7719, 0x771a, PDF_CMAP_RANGE, 3152 },
	{ 0x771c, 0x771c, PDF_CMAP_SINGLE, 3154 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 3155 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 3156 },
	{ 0x772d, 0x772f, PDF_CMAP_RANGE, 3157 },
	{ 0x7734, 0x7736, PDF_CMAP_RANGE, 3160 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 3163 },
	{ 0x773d, 0x773e, PDF_CMAP_RANGE, 3164 },
	{ 0x7742, 0x7742, PDF_CMAP_SINGLE, 3166 },
	{ 0x7745, 0x7746, PDF_CMAP_RANGE, 3167 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 3169 },
	{ 0x774d, 0x774f, PDF_CMAP_RANGE, 3170 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 3173 },
	{ 0x7756, 0x7757, PDF_CMAP_RANGE, 3174 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 3176 },
	{ 0x775e, 0x7760, PDF_CMAP_RANGE, 3177 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 3180 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 3181 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 3182 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 3183 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 3184 },
	{ 0x7770, 0x7770, PDF_CMAP_SINGLE, 3185 },
	{ 0x7772, 0x7774, PDF_CMAP_RANGE, 3186 },
	{ 0x777a, 0x777a, PDF_CMAP_SINGLE, 3189 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 3190 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 3191 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 3192 },
	{ 0x778c, 0x778d, PDF_CMAP_RANGE, 3193 },
	{ 0x7794, 0x7796, PDF_CMAP_RANGE, 3195 },
	{ 0x779a, 0x779a, PDF_CMAP_SINGLE, 3198 },
	{ 0x779f, 0x779f, PDF_CMAP_SINGLE, 3199 },
	{ 0x77a2, 0x77a2, PDF_CMAP_SINGLE, 3200 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 3201 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 3202 },
	{ 0x77ae, 0x77af, PDF_CMAP_RANGE, 3203 },
	{ 0x77b1, 0x77b1, PDF_CMAP_SINGLE, 3205 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 3206 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 3207 },
	{ 0x77c3, 0x77c3, PDF_CMAP_SINGLE, 3208 },
	{ 0x77c9, 0x77c9, PDF_CMAP_SINGLE, 3209 },
	{ 0x77d1, 0x77d2, PDF_CMAP_RANGE, 3210 },
	{ 0x77d5, 0x77d5, PDF_CMAP_SINGLE, 3212 },
	{ 0x77d9, 0x77d9, PDF_CMAP_SINGLE, 3213 },
	{ 0x77de, 0x77e0, PDF_CMAP_RANGE, 3214 },
	{ 0x77e4, 0x77e4, PDF_CMAP_SINGLE, 3217 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 3218 },
	{ 0x77ea, 0x77ea, PDF_CMAP_SINGLE, 3219 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 3220 },
	{ 0x77f0, 0x77f1, PDF_CMAP_RANGE, 3221 },
	{ 0x77f4, 0x77f4, PDF_CMAP_SINGLE, 3223 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 3224 },
	{ 0x77fb, 0x77fb, PDF_CMAP_SINGLE, 3225 },
	{ 0x7805, 0x7806, PDF_CMAP_RANGE, 3226 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 3228 },
	{ 0x780d, 0x780e, PDF_CMAP_RANGE, 3229 },
	{ 0x7811, 0x7811, PDF_CMAP_SINGLE, 3231 },
	{ 0x781d, 0x781d, PDF_CMAP_SINGLE, 3232 },
	{ 0x7821, 0x7823, PDF_CMAP_RANGE, 3233 },
	{ 0x782d, 0x782e, PDF_CMAP_RANGE, 3236 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 3238 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 3239 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 3240 },
	{ 0x7843, 0x7844, PDF_CMAP_RANGE, 3241 },
	{ 0x7847, 0x7848, PDF_CMAP_RANGE, 3243 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 3245 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 3246 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 3247 },
	{ 0x785c, 0x785c, PDF_CMAP_SINGLE, 3248 },
	{ 0x785e, 0x785e, PDF_CMAP_SINGLE, 3249 },
	{ 0x7860, 0x7861, PDF_CMAP_RANGE, 3250 },
	{ 0x7863, 0x7864, PDF_CMAP_RANGE, 3252 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 3254 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 3255 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 3256 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 3257 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 3258 },
	{ 0x788a, 0x788a, PDF_CMAP_SINGLE, 3259 },
	{ 0x788f, 0x788f, PDF_CMAP_SINGLE, 3260 },
	{ 0x7894, 0x7894, PDF_CMAP_SINGLE, 3261 },
	{ 0x7898, 0x7898, PDF_CMAP_SINGLE, 3262 },
	{ 0x789d, 0x789f, PDF_CMAP_RANGE, 3264 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 3263 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 3267 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 3268 },
	{ 0x78ac, 0x78ad, PDF_CMAP_RANGE, 3269 },
	{ 0x78b0, 0x78b3, PDF_CMAP_RANGE, 3271 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 3275 },
	{ 0x78bd, 0x78bd, PDF_CMAP_SINGLE, 3276 },
	{ 0x78bf, 0x78bf, PDF_CMAP_SINGLE, 3277 },
	{ 0x78c7, 0x78c9, PDF_CMAP_RANGE, 3278 },
	{ 0x78cc, 0x78cc, PDF_CMAP_SINGLE, 3281 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 3282 },
	{ 0x78d2, 0x78d3, PDF_CMAP_RANGE, 3283 },
	{ 0x78d5, 0x78d6, PDF_CMAP_RANGE, 3285 },
	{ 0x78db, 0x78db, PDF_CMAP_SINGLE, 3288 },
	{ 0x78df, 0x78e1, PDF_CMAP_RANGE, 3289 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 3287 },
	{ 0x78e6, 0x78e6, PDF_CMAP_SINGLE, 3292 },
	{ 0x78ea, 0x78ea, PDF_CMAP_SINGLE, 3293 },
	{ 0x78f2, 0x78f3, PDF_CMAP_RANGE, 3294 },
	{ 0x78f6, 0x78f7, PDF_CMAP_RANGE, 3297 },
	{ 0x78fa, 0x78fb, PDF_CMAP_RANGE, 3299 },
	{ 0x78ff, 0x7900, PDF_CMAP_TABLE, 244 },
	{ 0x7906, 0x7906, PDF_CMAP_SINGLE, 3302 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 3303 },
	{ 0x7910, 0x7910, PDF_CMAP_SINGLE, 3304 },
	{ 0x791a, 0x791a, PDF_CMAP_SINGLE, 3305 },
	{ 0x791c, 0x791c, PDF_CMAP_SINGLE, 3306 },
	{ 0x791e, 0x7920, PDF_CMAP_RANGE, 3307 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 3310 },
	{ 0x7927, 0x7927, PDF_CMAP_SINGLE, 3311 },
	{ 0x7929, 0x7929, PDF_CMAP_SINGLE, 3312 },
	{ 0x792d, 0x792d, PDF_CMAP_SINGLE, 3313 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 3314 },
	{ 0x7934, 0x7935, PDF_CMAP_RANGE, 3315 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 3317 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 3318 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 3319 },
	{ 0x7944, 0x7946, PDF_CMAP_RANGE, 3320 },
	{ 0x794a, 0x794b, PDF_CMAP_RANGE, 3323 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 3325 },
	{ 0x7951, 0x7951, PDF_CMAP_SINGLE, 3326 },
	{ 0x7954, 0x7954, PDF_CMAP_SINGLE, 3327 },
	{ 0x7958, 0x7958, PDF_CMAP_SINGLE, 3328 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 3329 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 3331 },
	{ 0x7969, 0x7969, PDF_CMAP_SINGLE, 3332 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 3333 },
	{ 0x7972, 0x7972, PDF_CMAP_SINGLE, 3334 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 3335 },
	{ 0x797b, 0x797c, PDF_CMAP_RANGE, 3336 },
	{ 0x797e, 0x797e, PDF_CMAP_SINGLE, 3338 },
	{ 0x798b, 0x798c, PDF_CMAP_RANGE, 3339 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 3341 },
	{ 0x7993, 0x7996, PDF_CMAP_RANGE, 3342 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 3346 },
	{ 0x799b, 0x799c, PDF_CMAP_RANGE, 3347 },
	{ 0x79a1, 0x79a1, PDF_CMAP_SINGLE, 3349 },
	{ 0x79a8, 0x79a9, PDF_CMAP_RANGE, 3350 },
	{ 0x79ab, 0x79ab, PDF_CMAP_SINGLE, 3352 },
	{ 0x79af, 0x79af, PDF_CMAP_SINGLE, 3353 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 3354 },
	{ 0x79b4, 0x79b4, PDF_CMAP_SINGLE, 3355 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 3356 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 3357 },
	{ 0x79c2, 0x79c2, PDF_CMAP_SINGLE, 3358 },
	{ 0x79c4, 0x79c4, PDF_CMAP_SINGLE, 3359 },
	{ 0x79c7, 0x79c8, PDF_CMAP_RANGE, 3360 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 3362 },
	{ 0x79cf, 0x79cf, PDF_CMAP_SINGLE, 3363 },
	{ 0x79d4, 0x79d4, PDF_CMAP_SINGLE, 3364 },
	{ 0x79d6, 0x79d6, PDF_CMAP_SINGLE, 3365 },
	{ 0x79da, 0x79da, PDF_CMAP_SINGLE, 3366 },
	{ 0x79dd, 0x79de, PDF_CMAP_RANGE, 3367 },
	{ 0x79e0, 0x79e0, PDF_CMAP_SINGLE, 3369 },
	{ 0x79e2, 0x79e2, PDF_CMAP_SINGLE, 3370 },
	{ 0x79e5, 0x79e5, PDF_CMAP_SINGLE, 3371 },
	{ 0x79ea, 0x79eb, PDF_CMAP_RANGE, 3372 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 3374 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 3375 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 3376 },
	{ 0x79fc, 0x79fc, PDF_CMAP_SINGLE, 3377 },
	{ 0x7a02, 0x7a03, PDF_CMAP_RANGE, 3378 },
	{ 0x7a07, 0x7a07, PDF_CMAP_SINGLE, 3380 },
	{ 0x7a09, 0x7a0a, PDF_CMAP_RANGE, 3381 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 3383 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 3384 },
	{ 0x7a15, 0x7a15, PDF_CMAP_SINGLE, 3385 },
	{ 0x7a1b, 0x7a1b, PDF_CMAP_SINGLE, 3386 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 3387 },
	{ 0x7a21, 0x7a21, PDF_CMAP_SINGLE, 3388 },
	{ 0x7a27, 0x7a27, PDF_CMAP_SINGLE, 3389 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 3390 },
	{ 0x7a2d, 0x7a2d, PDF_CMAP_SINGLE, 3391 },
	{ 0x7a2f, 0x7a30, PDF_CMAP_RANGE, 3392 },
	{ 0x7a34, 0x7a35, PDF_CMAP_RANGE, 3394 },
	{ 0x7a38, 0x7a3a, PDF_CMAP_RANGE, 3396 },
	{ 0x7a44, 0x7a45, PDF_CMAP_RANGE, 3399 },
	{ 0x7a47, 0x7a48, PDF_CMAP_RANGE, 3401 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 3403 },
	{ 0x7a55, 0x7a56, PDF_CMAP_RANGE, 3404 },
	{ 0x7a59, 0x7a59, PDF_CMAP_SINGLE, 3406 },
	{ 0x7a5c, 0x7a5d, PDF_CMAP_RANGE, 3407 },
	{ 0x7a5f, 0x7a60, PDF_CMAP_RANGE, 3409 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 3411 },
	{ 0x7a67, 0x7a67, PDF_CMAP_SINGLE, 3412 },
	{ 0x7a6a, 0x7a6a, PDF_CMAP_SINGLE, 3413 },
	{ 0x7a6d, 0x7a6d, PDF_CMAP_SINGLE, 3414 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 3415 },
	{ 0x7a78, 0x7a78, PDF_CMAP_SINGLE, 3416 },
	{ 0x7a7e, 0x7a7e, PDF_CMAP_SINGLE, 3417 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 3418 },
	{ 0x7a82, 0x7a82, PDF_CMAP_SINGLE, 3419 },
	{ 0x7a85, 0x7a86, PDF_CMAP_RANGE, 3420 },
	{ 0x7a8a, 0x7a8b, PDF_CMAP_RANGE, 3422 },
	{ 0x7a90, 0x7a91, PDF_CMAP_RANGE, 3424 },
	{ 0x7a94, 0x7a94, PDF_CMAP_SINGLE, 3426 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 3427 },
	{ 0x7aa0, 0x7aa0, PDF_CMAP_SINGLE, 3428 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 3429 },
	{ 0x7aac, 0x7aac, PDF_CMAP_SINGLE, 3430 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 3431 },
	{ 0x7ab5, 0x7ab5, PDF_CMAP_SINGLE, 3432 },
	{ 0x7ab9, 0x7ab9, PDF_CMAP_SINGLE, 3433 },
	{ 0x7abb, 0x7abc, PDF_CMAP_RANGE, 3434 },
	{ 0x7ac6, 0x7ac6, PDF_CMAP_SINGLE, 3436 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 3437 },
	{ 0x7acc, 0x7acc, PDF_CMAP_SINGLE, 3438 },
	{ 0x7ace, 0x7ace, PDF_CMAP_SINGLE, 3439 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 3440 },
	{ 0x7adb, 0x7adb, PDF_CMAP_SINGLE, 3441 },
	{ 0x7ae8, 0x7ae9, PDF_CMAP_RANGE, 3442 },
	{ 0x7aeb, 0x7aec, PDF_CMAP_RANGE, 3444 },
	{ 0x7af1, 0x7af1, PDF_CMAP_SINGLE, 3446 },
	{ 0x7af4, 0x7af4, PDF_CMAP_SINGLE, 3447 },
	{ 0x7afb, 0x7afb, PDF_CMAP_SINGLE, 3448 },
	{ 0x7afd, 0x7afe, PDF_CMAP_RANGE, 3449 },
	{ 0x7b07, 0x7b07, PDF_CMAP_SINGLE, 3451 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 3452 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 3453 },
	{ 0x7b23, 0x7b23, PDF_CMAP_SINGLE, 3454 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 3455 },
	{ 0x7b29, 0x7b2b, PDF_CMAP_RANGE, 3456 },
	{ 0x7b2d, 0x7b31, PDF_CMAP_RANGE, 3459 },
	{ 0x7b34, 0x7b34, PDF_CMAP_SINGLE, 3464 },
	{ 0x7b3d, 0x7b3d, PDF_CMAP_SINGLE, 3465 },
	{ 0x7b3f, 0x7b41, PDF_CMAP_RANGE, 3466 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 3469 },
	{ 0x7b4e, 0x7b4e, PDF_CMAP_SINGLE, 3470 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 3471 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 3472 },
	{ 0x7b64, 0x7b64, PDF_CMAP_SINGLE, 3473 },
	{ 0x7b66, 0x7b66, PDF_CMAP_SINGLE, 3474 },
	{ 0x7b69, 0x7b6a, PDF_CMAP_RANGE, 3475 },
	{ 0x7b6d, 0x7b6d, PDF_CMAP_SINGLE, 3477 },
	{ 0x7b6f, 0x7b6f, PDF_CMAP_SINGLE, 3478 },
	{ 0x7b72, 0x7b73, PDF_CMAP_RANGE, 3479 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 3481 },
	{ 0x7b84, 0x7b84, PDF_CMAP_SINGLE, 3482 },
	{ 0x7b89, 0x7b89, PDF_CMAP_SINGLE, 3483 },
	{ 0x7b8e, 0x7b8e, PDF_CMAP_SINGLE, 3484 },
	{ 0x7b90, 0x7b91, PDF_CMAP_RANGE, 3485 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 3487 },
	{ 0x7b9b, 0x7b9b, PDF_CMAP_SINGLE, 3488 },
	{ 0x7b9e, 0x7b9e, PDF_CMAP_SINGLE, 3489 },
	{ 0x7ba0, 0x7ba0, PDF_CMAP_SINGLE, 3490 },
	{ 0x7ba5, 0x7ba5, PDF_CMAP_SINGLE, 3491 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 3492 },
	{ 0x7baf, 0x7bb0, PDF_CMAP_RANGE, 3493 },
	{ 0x7bb2, 0x7bb2, PDF_CMAP_SINGLE, 3495 },
	{ 0x7bb5, 0x7bb6, PDF_CMAP_RANGE, 3496 },
	{ 0x7bba, 0x7bbd, PDF_CMAP_RANGE, 3498 },
	{ 0x7bc2, 0x7bc2, PDF_CMAP_SINGLE, 3502 },
	{ 0x7bc5, 0x7bc5, PDF_CMAP_SINGLE, 3503 },
	{ 0x7bc8, 0x7bc8, PDF_CMAP_SINGLE, 3504 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 3505 },
	{ 0x7bd4, 0x7bd4, PDF_CMAP_SINGLE, 3506 },
	{ 0x7bd6, 0x7bd7, PDF_CMAP_RANGE, 3507 },
	{ 0x7bd9, 0x7bdb, PDF_CMAP_RANGE, 3509 },
	{ 0x7be8, 0x7be8, PDF_CMAP_SINGLE, 3512 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 3513 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 3514 },
	{ 0x7bf4, 0x7bf5, PDF_CMAP_RANGE, 3515 },
	{ 0x7bf8, 0x7bfa, PDF_CMAP_RANGE, 3517 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 3520 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 3521 },
	{ 0x7c01, 0x7c04, PDF_CMAP_RANGE, 3522 },
	{ 0x7c06, 0x7c06, PDF_CMAP_SINGLE, 3526 },
	{ 0x7c09, 0x7c09, PDF_CMAP_SINGLE, 3527 },
	{ 0x7c0b, 0x7c0c, PDF_CMAP_RANGE, 3528 },
	{ 0x7c0e, 0x7c0f, PDF_CMAP_RANGE, 3530 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 3532 },
	{ 0x7c1b, 0x7c1b, PDF_CMAP_SINGLE, 3533 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 3534 },
	{ 0x7c25, 0x7c26, PDF_CMAP_RANGE, 3535 },
	{ 0x7c28, 0x7c28, PDF_CMAP_SINGLE, 3537 },
	{ 0x7c2c, 0x7c2c, PDF_CMAP_SINGLE, 3538 },
	{ 0x7c31, 0x7c31, PDF_CMAP_SINGLE, 3539 },
	{ 0x7c33, 0x7c34, PDF_CMAP_RANGE, 3540 },
	{ 0x7c36, 0x7c36, PDF_CMAP_SINGLE, 3542 },
	{ 0x7c39, 0x7c3a, PDF_CMAP_RANGE, 3543 },
	{ 0x7c46, 0x7c46, PDF_CMAP_SINGLE, 3545 },
	{ 0x7c4a, 0x7c4a, PDF_CMAP_SINGLE, 3546 },
	{ 0x7c51, 0x7c53, PDF_CMAP_RANGE, 3548 },
	{ 0x7c55, 0x7c55, PDF_CMAP_SINGLE, 3547 },
	{ 0x7c59, 0x7c5e, PDF_CMAP_RANGE, 3551 },
	{ 0x7c61, 0x7c61, PDF_CMAP_SINGLE, 3557 },
	{ 0x7c63, 0x7c63, PDF_CMAP_SINGLE, 3558 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 3559 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 3560 },
	{ 0x7c6d, 0x7c6e, PDF_CMAP_RANGE, 3561 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 3563 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 3564 },
	{ 0x7c79, 0x7c79, PDF_CMAP_SINGLE, 3565 },
	{ 0x7c7c, 0x7c7d, PDF_CMAP_RANGE, 3566 },
	{ 0x7c86, 0x7c87, PDF_CMAP_RANGE, 3568 },
	{ 0x7c8f, 0x7c8f, PDF_CMAP_SINGLE, 3570 },
	{ 0x7c94, 0x7c94, PDF_CMAP_SINGLE, 3571 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 3572 },
	{ 0x7ca0, 0x7ca0, PDF_CMAP_SINGLE, 3573 },
	{ 0x7ca6, 0x7ca6, PDF_CMAP_SINGLE, 3574 },
	{ 0x7cb0, 0x7cb0, PDF_CMAP_SINGLE, 3575 },
	{ 0x7cb6, 0x7cb7, PDF_CMAP_RANGE, 3576 },
	{ 0x7cba, 0x7cbc, PDF_CMAP_RANGE, 3578 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 3581 },
	{ 0x7cc4, 0x7cc4, PDF_CMAP_SINGLE, 3582 },
	{ 0x7cc7, 0x7cc9, PDF_CMAP_RANGE, 3583 },
	{ 0x7ccd, 0x7ccd, PDF_CMAP_SINGLE, 3586 },
	{ 0x7ccf, 0x7ccf, PDF_CMAP_SINGLE, 3587 },
	{ 0x7cd3, 0x7cd5, PDF_CMAP_RANGE, 3588 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 3591 },
	{ 0x7cd9, 0x7cda, PDF_CMAP_RANGE, 3592 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 3594 },
	{ 0x7ce6, 0x7ce6, PDF_CMAP_SINGLE, 3595 },
	{ 0x7ce9, 0x7ce9, PDF_CMAP_SINGLE, 3596 },
	{ 0x7ceb, 0x7ceb, PDF_CMAP_SINGLE, 3597 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 3598 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 3599 },
	{ 0x7d07, 0x7d09, PDF_CMAP_RANGE, 3600 },
	{ 0x7d0f, 0x7d0f, PDF_CMAP_SINGLE, 3603 },
	{ 0x7d11, 0x7d13, PDF_CMAP_RANGE, 3604 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 3607 },
	{ 0x7d1d, 0x7d1e, PDF_CMAP_RANGE, 3608 },
	{ 0x7d23, 0x7d23, PDF_CMAP_SINGLE, 3610 },
	{ 0x7d26, 0x7d26, PDF_CMAP_SINGLE, 3611 },
	{ 0x7d2a, 0x7d2a, PDF_CMAP_SINGLE, 3612 },
	{ 0x7d2d, 0x7d2d, PDF_CMAP_SINGLE, 3613 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 3614 },
	{ 0x7d3c, 0x7d3e, PDF_CMAP_RANGE, 3615 },
	{ 0x7d40, 0x7d41, PDF_CMAP_RANGE, 3618 },
	{ 0x7d47, 0x7d48, PDF_CMAP_RANGE, 3620 },
	{ 0x7d4d, 0x7d4d, PDF_CMAP_SINGLE, 3622 },
	{ 0x7d51, 0x7d51, PDF_CMAP_SINGLE, 3623 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 3624 },
	{ 0x7d57, 0x7d57, PDF_CMAP_SINGLE, 3625 },
	{ 0x7d59, 0x7d5a, PDF_CMAP_RANGE, 3626 },
	{ 0x7d5c, 0x7d5d, PDF_CMAP_RANGE, 3628 },
	{ 0x7d65, 0x7d65, PDF_CMAP_SINGLE, 3630 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 3631 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 3632 },
	{ 0x7d70, 0x7d70, PDF_CMAP_SINGLE, 3633 },
	{ 0x7d78, 0x7d78, PDF_CMAP_SINGLE, 3634 },
	{ 0x7d7a, 0x7d7b, PDF_CMAP_RANGE, 3635 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 3637 },
	{ 0x7d81, 0x7d83, PDF_CMAP_RANGE, 3638 },
	{ 0x7d85, 0x7d86, PDF_CMAP_RANGE, 3641 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 3643 },
	{ 0x7d8b, 0x7d8d, PDF_CMAP_RANGE, 3644 },
	{ 0x7d91, 0x7d91, PDF_CMAP_SINGLE, 3647 },
	{ 0x7d96, 0x7d97, PDF_CMAP_RANGE, 3648 },
	{ 0x7d9d, 0x7d9e, PDF_CMAP_RANGE, 3650 },
	{ 0x7da6, 0x7da7, PDF_CMAP_RANGE, 3652 },
	{ 0x7daa, 0x7daa, PDF_CMAP_SINGLE, 3654 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 3655 },
	{ 0x7db6, 0x7db7, PDF_CMAP_RANGE, 3656 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 3658 },
	{ 0x7dc2, 0x7dc6, PDF_CMAP_RANGE, 3659 },
	{ 0x7dcc, 0x7dce, PDF_CMAP_RANGE, 3664 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 3667 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 3668 },
	{ 0x7de2, 0x7de2, PDF_CMAP_SINGLE, 3670 },
	{ 0x7de5, 0x7de6, PDF_CMAP_RANGE, 3671 },
	{ 0x7dea, 0x7deb, PDF_CMAP_RANGE, 3673 },
	{ 0x7ded, 0x7ded, PDF_CMAP_SINGLE, 3675 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 3676 },
	{ 0x7df5, 0x7df6, PDF_CMAP_RANGE, 3677 },
	{ 0x7df9, 0x7dfa, PDF_CMAP_RANGE, 3679 },
	{ 0x7e00, 0x7e00, PDF_CMAP_SINGLE, 3669 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 3681 },
	{ 0x7e10, 0x7e11, PDF_CMAP_RANGE, 3682 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 3684 },
	{ 0x7e17, 0x7e17, PDF_CMAP_SINGLE, 3685 },
	{ 0x7e1c, 0x7e1d, PDF_CMAP_RANGE, 3686 },
	{ 0x7e20, 0x7e20, PDF_CMAP_SINGLE, 3688 },
	{ 0x7e27, 0x7e28, PDF_CMAP_RANGE, 3689 },
	{ 0x7e2c, 0x7e2d, PDF_CMAP_RANGE, 3691 },
	{ 0x7e2f, 0x7e2f, PDF_CMAP_SINGLE, 3693 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 3694 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 3695 },
	{ 0x7e3f, 0x7e3f, PDF_CMAP_SINGLE, 3696 },
	{ 0x7e44, 0x7e45, PDF_CMAP_RANGE, 3697 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 3699 },
	{ 0x7e4e, 0x7e4e, PDF_CMAP_SINGLE, 3700 },
	{ 0x7e50, 0x7e50, PDF_CMAP_SINGLE, 3701 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 3702 },
	{ 0x7e58, 0x7e58, PDF_CMAP_SINGLE, 3703 },
	{ 0x7e5f, 0x7e5f, PDF_CMAP_SINGLE, 3704 },
	{ 0x7e61, 0x7e62, PDF_CMAP_RANGE, 3705 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 3707 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 3708 },
	{ 0x7e6e, 0x7e6f, PDF_CMAP_RANGE, 3709 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 3711 },
	{ 0x7e78, 0x7e78, PDF_CMAP_SINGLE, 3712 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 3713 },
	{ 0x7e81, 0x7e81, PDF_CMAP_SINGLE, 3714 },
	{ 0x7e86, 0x7e87, PDF_CMAP_RANGE, 3715 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 3717 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 3718 },
	{ 0x7e91, 0x7e91, PDF_CMAP_SINGLE, 3719 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 3720 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 3721 },
	{ 0x7e9a, 0x7e9a, PDF_CMAP_SINGLE, 3722 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 3723 },
	{ 0x7f3b, 0x7f3c, PDF_CMAP_TABLE, 246 },
	{ 0x7f3d, 0x7f3f, PDF_CMAP_RANGE, 3727 },
	{ 0x7f43, 0x7f44, PDF_CMAP_RANGE, 3730 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 3732 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 3733 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 3734 },
	{ 0x7f5b, 0x7f5d, PDF_CMAP_RANGE, 3736 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 3739 },
	{ 0x7f63, 0x7f66, PDF_CMAP_RANGE, 3740 },
	{ 0x7f6d, 0x7f6d, PDF_CMAP_SINGLE, 3744 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 3745 },
	{ 0x7f7d, 0x7f80, PDF_CMAP_RANGE, 3746 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 3750 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 3751 },
	{ 0x7f8f, 0x7f91, PDF_CMAP_RANGE, 3752 },
	{ 0x7f96, 0x7f97, PDF_CMAP_RANGE, 3755 },
	{ 0x7f9c, 0x7f9c, PDF_CMAP_SINGLE, 3757 },
	{ 0x7fa1, 0x7fa2, PDF_CMAP_RANGE, 3758 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 3760 },
	{ 0x7faa, 0x7faa, PDF_CMAP_SINGLE, 3761 },
	{ 0x7fad, 0x7fad, PDF_CMAP_SINGLE, 3762 },
	{ 0x7fb4, 0x7fb4, PDF_CMAP_SINGLE, 3763 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 3764 },
	{ 0x7fbf, 0x7fc0, PDF_CMAP_RANGE, 3765 },
	{ 0x7fc3, 0x7fc3, PDF_CMAP_SINGLE, 3767 },
	{ 0x7fc8, 0x7fc8, PDF_CMAP_SINGLE, 3768 },
	{ 0x7fce, 0x7fcf, PDF_CMAP_RANGE, 3769 },
	{ 0x7fdb, 0x7fdb, PDF_CMAP_SINGLE, 3771 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 3772 },
	{ 0x7fe3, 0x7fe3, PDF_CMAP_SINGLE, 3773 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 3774 },
	{ 0x7fe8, 0x7fe8, PDF_CMAP_SINGLE, 3775 },
	{ 0x7fec, 0x7fec, PDF_CMAP_SINGLE, 3776 },
	{ 0x7fee, 0x7fef, PDF_CMAP_RANGE, 3777 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 3779 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 3780 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 3781 },
	{ 0x8007, 0x8008, PDF_CMAP_RANGE, 3784 },
	{ 0x800a, 0x800a, PDF_CMAP_SINGLE, 3786 },
	{ 0x800d, 0x800f, PDF_CMAP_RANGE, 3787 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 3790 },
	{ 0x8013, 0x8014, PDF_CMAP_RANGE, 3791 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 3793 },
	{ 0x801d, 0x8020, PDF_CMAP_RANGE, 3794 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 3798 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 3799 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 3800 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 3801 },
	{ 0x8030, 0x8030, PDF_CMAP_SINGLE, 3802 },
	{ 0x8034, 0x8035, PDF_CMAP_RANGE, 3803 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 3805 },
	{ 0x8039, 0x803a, PDF_CMAP_RANGE, 3806 },
	{ 0x803c, 0x803c, PDF_CMAP_SINGLE, 3808 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 3809 },
	{ 0x8040, 0x8040, PDF_CMAP_SINGLE, 3810 },
	{ 0x8044, 0x8044, PDF_CMAP_SINGLE, 3811 },
	{ 0x8060, 0x8060, PDF_CMAP_SINGLE, 3812 },
	{ 0x8064, 0x8064, PDF_CMAP_SINGLE, 3813 },
	{ 0x8066, 0x8066, PDF_CMAP_SINGLE, 3814 },
	{ 0x806d, 0x806d, PDF_CMAP_SINGLE, 3815 },
	{ 0x8071, 0x8071, PDF_CMAP_SINGLE, 3816 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 3817 },
	{ 0x8081, 0x8081, PDF_CMAP_SINGLE, 3818 },
	{ 0x8088, 0x8088, PDF_CMAP_SINGLE, 3819 },
	{ 0x808e, 0x808e, PDF_CMAP_SINGLE, 3820 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 3821 },
	{ 0x809e, 0x809e, PDF_CMAP_SINGLE, 3822 },
	{ 0x80a6, 0x80a7, PDF_CMAP_RANGE, 3823 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 3825 },
	{ 0x80b8, 0x80b9, PDF_CMAP_RANGE, 3826 },
	{ 0x80c8, 0x80c8, PDF_CMAP_SINGLE, 3828 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 3829 },
	{ 0x80cf, 0x80cf, PDF_CMAP_SINGLE, 3830 },
	{ 0x80d2, 0x80d2, PDF_CMAP_SINGLE, 3831 },
	{ 0x80d4, 0x80d5, PDF_CMAP_RANGE, 3832 },
	{ 0x80d7, 0x80d8, PDF_CMAP_RANGE, 3834 },
	{ 0x80e0, 0x80e0, PDF_CMAP_SINGLE, 3836 },
	{ 0x80ed, 0x80ee, PDF_CMAP_RANGE, 3837 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 3839 },
	{ 0x80f2, 0x80f3, PDF_CMAP_RANGE, 3840 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 3842 },
	{ 0x80f9, 0x80fa, PDF_CMAP_RANGE, 3843 },
	{ 0x80fe, 0x80fe, PDF_CMAP_SINGLE, 3845 },
	{ 0x8103, 0x8103, PDF_CMAP_SINGLE, 3846 },
	{ 0x810b, 0x810b, PDF_CMAP_SINGLE, 3847 },
	{ 0x8116, 0x8118, PDF_CMAP_RANGE, 3848 },
	{ 0x811c, 0x811c, PDF_CMAP_SINGLE, 3851 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 3852 },
	{ 0x8120, 0x8120, PDF_CMAP_SINGLE, 3853 },
	{ 0x8124, 0x8124, PDF_CMAP_SINGLE, 3854 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 3855 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 3856 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 3857 },
	{ 0x8135, 0x8135, PDF_CMAP_SINGLE, 3858 },
	{ 0x813a, 0x813a, PDF_CMAP_SINGLE, 3859 },
	{ 0x813c, 0x813c, PDF_CMAP_SINGLE, 3860 },
	{ 0x8145, 0x8145, PDF_CMAP_SINGLE, 3861 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 3862 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 3863 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 3864 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 3865 },
	{ 0x8157, 0x8157, PDF_CMAP_SINGLE, 3866 },
	{ 0x8160, 0x8161, PDF_CMAP_RANGE, 3867 },
	{ 0x8167, 0x8169, PDF_CMAP_RANGE, 3869 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 3872 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 3873 },
	{ 0x8177, 0x8177, PDF_CMAP_SINGLE, 3874 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 3875 },
	{ 0x8184, 0x8186, PDF_CMAP_RANGE, 3877 },
	{ 0x818b, 0x818b, PDF_CMAP_SINGLE, 3880 },
	{ 0x818e, 0x818e, PDF_CMAP_SINGLE, 3881 },
	{ 0x8190, 0x8190, PDF_CMAP_SINGLE, 3876 },
	{ 0x8196, 0x8196, PDF_CMAP_SINGLE, 3882 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 3883 },
	{ 0x819b, 0x819b, PDF_CMAP_SINGLE, 3884 },
	{ 0x819e, 0x819e, PDF_CMAP_SINGLE, 3885 },
	{ 0x81a2, 0x81a2, PDF_CMAP_SINGLE, 3886 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 3887 },
	{ 0x81b2, 0x81b2, PDF_CMAP_SINGLE, 3888 },
	{ 0x81b4, 0x81b4, PDF_CMAP_SINGLE, 3889 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 3890 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 3892 },
	{ 0x81c5, 0x81c5, PDF_CMAP_SINGLE, 3893 },
	{ 0x81ca, 0x81cb, PDF_CMAP_TABLE, 248 },
	{ 0x81ce, 0x81cf, PDF_CMAP_RANGE, 3895 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 3897 },
	{ 0x81d7, 0x81d7, PDF_CMAP_SINGLE, 3898 },
	{ 0x81db, 0x81db, PDF_CMAP_SINGLE, 3899 },
	{ 0x81dd, 0x81de, PDF_CMAP_RANGE, 3900 },
	{ 0x81e1, 0x81e1, PDF_CMAP_SINGLE, 3902 },
	{ 0x81e4, 0x81e4, PDF_CMAP_SINGLE, 3903 },
	{ 0x81eb, 0x81ec, PDF_CMAP_RANGE, 3904 },
	{ 0x81f0, 0x81f2, PDF_CMAP_RANGE, 3906 },
	{ 0x81f5, 0x81f6, PDF_CMAP_RANGE, 3909 },
	{ 0x81f8, 0x81f9, PDF_CMAP_RANGE, 3911 },
	{ 0x81fd, 0x81fd, PDF_CMAP_SINGLE, 3913 },
	{ 0x81ff, 0x8200, PDF_CMAP_RANGE, 3914 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 3916 },
	{ 0x820f, 0x820f, PDF_CMAP_SINGLE, 3917 },
	{ 0x8213, 0x8214, PDF_CMAP_RANGE, 3918 },
	{ 0x8219, 0x821a, PDF_CMAP_RANGE, 3920 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 3922 },
	{ 0x8221, 0x8222, PDF_CMAP_RANGE, 3923 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 3925 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 3926 },
	{ 0x8234, 0x8234, PDF_CMAP_SINGLE, 3927 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 3928 },
	{ 0x8243, 0x8246, PDF_CMAP_RANGE, 3929 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 3933 },
	{ 0x824e, 0x824f, PDF_CMAP_RANGE, 3934 },
	{ 0x8251, 0x8251, PDF_CMAP_SINGLE, 3936 },
	{ 0x8256, 0x8256, PDF_CMAP_SINGLE, 3937 },
	{ 0x825c, 0x825c, PDF_CMAP_SINGLE, 3938 },
	{ 0x8260, 0x8260, PDF_CMAP_SINGLE, 3939 },
	{ 0x8263, 0x8263, PDF_CMAP_SINGLE, 3940 },
	{ 0x8267, 0x8267, PDF_CMAP_SINGLE, 3941 },
	{ 0x826d, 0x826d, PDF_CMAP_SINGLE, 3942 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 3943 },
	{ 0x827b, 0x827b, PDF_CMAP_SINGLE, 3944 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 3945 },
	{ 0x827f, 0x8281, PDF_CMAP_RANGE, 3946 },
	{ 0x8283, 0x8284, PDF_CMAP_RANGE, 3949 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 3951 },
	{ 0x8289, 0x828a, PDF_CMAP_RANGE, 3952 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 3954 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 3955 },
	{ 0x8294, 0x8294, PDF_CMAP_SINGLE, 3956 },
	{ 0x8296, 0x8296, PDF_CMAP_SINGLE, 3957 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 3958 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 3959 },
	{ 0x82a0, 0x82a1, PDF_CMAP_RANGE, 3961 },
	{ 0x82a3, 0x82a4, PDF_CMAP_RANGE, 3963 },
	{ 0x82a7, 0x82aa, PDF_CMAP_RANGE, 3965 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 3969 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 3970 },
	{ 0x82b2, 0x82b2, PDF_CMAP_SINGLE, 3971 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 3972 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 3973 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 3974 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 3975 },
	{ 0x82be, 0x82bf, PDF_CMAP_RANGE, 3976 },
	{ 0x82c6, 0x82c6, PDF_CMAP_SINGLE, 3978 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 3979 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 3980 },
	{ 0x82da, 0x82da, PDF_CMAP_SINGLE, 3981 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 3982 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 3983 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 3984 },
	{ 0x82e8, 0x82e8, PDF_CMAP_SINGLE, 3985 },
	{ 0x82ea, 0x82ea, PDF_CMAP_SINGLE, 3986 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 3987 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 3988 },
	{ 0x82f6, 0x82f7, PDF_CMAP_RANGE, 3989 },
	{ 0x82fd, 0x82fe, PDF_CMAP_RANGE, 3991 },
	{ 0x8300, 0x8301, PDF_CMAP_RANGE, 3993 },
	{ 0x8307, 0x8308, PDF_CMAP_RANGE, 3995 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 3997 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 4000 },
	{ 0x831d, 0x831f, PDF_CMAP_RANGE, 4001 },
	{ 0x8321, 0x8322, PDF_CMAP_RANGE, 4004 },
	{ 0x832c, 0x832e, PDF_CMAP_RANGE, 4006 },
	{ 0x8330, 0x8330, PDF_CMAP_SINGLE, 4009 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 4010 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 4011 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 4012 },
	{ 0x833c, 0x833d, PDF_CMAP_RANGE, 4013 },
	{ 0x8342, 0x8344, PDF_CMAP_RANGE, 4015 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 4018 },
	{ 0x834d, 0x834e, PDF_CMAP_RANGE, 4019 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 4021 },
	{ 0x8353, 0x8354, PDF_CMAP_TABLE, 250 },
	{ 0x8355, 0x8357, PDF_CMAP_RANGE, 4022 },
	{ 0x8370, 0x8370, PDF_CMAP_SINGLE, 4025 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 4026 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 4027 },
	{ 0x837f, 0x8380, PDF_CMAP_RANGE, 4028 },
	{ 0x8382, 0x8382, PDF_CMAP_SINGLE, 4030 },
	{ 0x8384, 0x8384, PDF_CMAP_SINGLE, 4031 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 4032 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 4033 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 4034 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 4035 },
	{ 0x8398, 0x8399, PDF_CMAP_RANGE, 4037 },
	{ 0x839b, 0x839d, PDF_CMAP_RANGE, 4039 },
	{ 0x83a6, 0x83a7, PDF_CMAP_RANGE, 4042 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 4044 },
	{ 0x83ac, 0x83ad, PDF_CMAP_TABLE, 252 },
	{ 0x83be, 0x83c0, PDF_CMAP_RANGE, 4046 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 4049 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 4050 },
	{ 0x83cf, 0x83d1, PDF_CMAP_RANGE, 4051 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 4054 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 4055 },
	{ 0x83e8, 0x83e8, PDF_CMAP_SINGLE, 4057 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 4058 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 4059 },
	{ 0x83f8, 0x83f9, PDF_CMAP_RANGE, 4060 },
	{ 0x83fc, 0x83fc, PDF_CMAP_SINGLE, 4062 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 4063 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 4064 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 4065 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 4066 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 4067 },
	{ 0x8415, 0x8415, PDF_CMAP_SINGLE, 4068 },
	{ 0x8419, 0x8419, PDF_CMAP_SINGLE, 4069 },
	{ 0x842f, 0x842f, PDF_CMAP_SINGLE, 4071 },
	{ 0x8439, 0x8439, PDF_CMAP_SINGLE, 4072 },
	{ 0x8445, 0x8445, PDF_CMAP_SINGLE, 4073 },
	{ 0x8447, 0x8448, PDF_CMAP_RANGE, 4074 },
	{ 0x844a, 0x844a, PDF_CMAP_SINGLE, 4076 },
	{ 0x844d, 0x844d, PDF_CMAP_SINGLE, 4077 },
	{ 0x844f, 0x844f, PDF_CMAP_SINGLE, 4078 },
	{ 0x8451, 0x8452, PDF_CMAP_RANGE, 4079 },
	{ 0x8456, 0x8456, PDF_CMAP_SINGLE, 4081 },
	{ 0x8458, 0x845a, PDF_CMAP_RANGE, 4082 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 4085 },
	{ 0x8460, 0x8460, PDF_CMAP_SINGLE, 4086 },
	{ 0x8464, 0x8465, PDF_CMAP_RANGE, 4087 },
	{ 0x8467, 0x8467, PDF_CMAP_SINGLE, 4089 },
	{ 0x846a, 0x846a, PDF_CMAP_SINGLE, 4090 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 4091 },
	{ 0x8473, 0x8474, PDF_CMAP_RANGE, 4092 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 4094 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 4095 },
	{ 0x847c, 0x847d, PDF_CMAP_RANGE, 4096 },
	{ 0x8481, 0x8481, PDF_CMAP_SINGLE, 4098 },
	{ 0x8485, 0x8485, PDF_CMAP_SINGLE, 4099 },
	{ 0x8492, 0x8493, PDF_CMAP_RANGE, 4100 },
	{ 0x8495, 0x8495, PDF_CMAP_SINGLE, 4102 },
	{ 0x849e, 0x849e, PDF_CMAP_SINGLE, 4103 },
	{ 0x84a6, 0x84a6, PDF_CMAP_SINGLE, 4104 },
	{ 0x84a8, 0x84aa, PDF_CMAP_RANGE, 4105 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 4108 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 4109 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 4110 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 4111 },
	{ 0x84bd, 0x84be, PDF_CMAP_RANGE, 4112 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 4114 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 4115 },
	{ 0x84c7, 0x84c8, PDF_CMAP_RANGE, 4116 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 4118 },
	{ 0x84cf, 0x84cf, PDF_CMAP_SINGLE, 4119 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 4120 },
	{ 0x84dc, 0x84dc, PDF_CMAP_SINGLE, 4121 },
	{ 0x84e7, 0x84e7, PDF_CMAP_SINGLE, 4122 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 4123 },
	{ 0x84ef, 0x84f2, PDF_CMAP_RANGE, 4124 },
	{ 0x84f7, 0x84f7, PDF_CMAP_SINGLE, 4128 },
	{ 0x84fa, 0x84fb, PDF_CMAP_RANGE, 4130 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 4132 },
	{ 0x8502, 0x8503, PDF_CMAP_RANGE, 4133 },
	{ 0x8507, 0x8507, PDF_CMAP_SINGLE, 4135 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 4136 },
	{ 0x850e, 0x850e, PDF_CMAP_SINGLE, 4137 },
	{ 0x8510, 0x8510, PDF_CMAP_SINGLE, 4138 },
	{ 0x851c, 0x851c, PDF_CMAP_SINGLE, 4139 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 4140 },
	{ 0x8522, 0x8525, PDF_CMAP_RANGE, 4141 },
	{ 0x8527, 0x8527, PDF_CMAP_SINGLE, 4145 },
	{ 0x852a, 0x852b, PDF_CMAP_RANGE, 4146 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 4148 },
	{ 0x8532, 0x8532, PDF_CMAP_SINGLE, 4129 },
	{ 0x8533, 0x8534, PDF_CMAP_RANGE, 4149 },
	{ 0x8536, 0x8536, PDF_CMAP_SINGLE, 4151 },
	{ 0x853f, 0x853f, PDF_CMAP_SINGLE, 4152 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 4153 },
	{ 0x854f, 0x8553, PDF_CMAP_RANGE, 4154 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 4159 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 4160 },
	{ 0x855c, 0x8562, PDF_CMAP_RANGE, 4161 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 4168 },
	{ 0x856b, 0x856b, PDF_CMAP_SINGLE, 4169 },
	{ 0x856f, 0x856f, PDF_CMAP_SINGLE, 4170 },
	{ 0x8579, 0x857b, PDF_CMAP_RANGE, 4171 },
	{ 0x857d, 0x857d, PDF_CMAP_SINGLE, 4174 },
	{ 0x857f, 0x857f, PDF_CMAP_SINGLE, 4175 },
	{ 0x8581, 0x8581, PDF_CMAP_SINGLE, 4176 },
	{ 0x8585, 0x8586, PDF_CMAP_RANGE, 4177 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 4179 },
	{ 0x858b, 0x858c, PDF_CMAP_RANGE, 4180 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 4182 },
	{ 0x8593, 0x8593, PDF_CMAP_SINGLE, 4183 },
	{ 0x8598, 0x8598, PDF_CMAP_SINGLE, 4184 },
	{ 0x859d, 0x859d, PDF_CMAP_SINGLE, 4185 },
	{ 0x859f, 0x85a0, PDF_CMAP_RANGE, 4186 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 4188 },
	{ 0x85a5, 0x85a5, PDF_CMAP_SINGLE, 4189 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 4190 },
	{ 0x85ad, 0x85ad, PDF_CMAP_SINGLE, 4204 },
	{ 0x85b4, 0x85b4, PDF_CMAP_SINGLE, 4191 },
	{ 0x85b6, 0x85b8, PDF_CMAP_RANGE, 4192 },
	{ 0x85bc, 0x85bf, PDF_CMAP_RANGE, 4195 },
	{ 0x85c2, 0x85c2, PDF_CMAP_SINGLE, 4199 },
	{ 0x85c7, 0x85c7, PDF_CMAP_SINGLE, 4200 },
	{ 0x85ca, 0x85cb, PDF_CMAP_RANGE, 4201 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 4203 },
	{ 0x85d8, 0x85d8, PDF_CMAP_SINGLE, 4205 },
	{ 0x85da, 0x85da, PDF_CMAP_SINGLE, 4206 },
	{ 0x85df, 0x85e0, PDF_CMAP_RANGE, 4207 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 4209 },
	{ 0x85e8, 0x85e8, PDF_CMAP_SINGLE, 4210 },
	{ 0x85ed, 0x85ed, PDF_CMAP_SINGLE, 4211 },
	{ 0x85f3, 0x85f3, PDF_CMAP_SINGLE, 4212 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 4213 },
	{ 0x85fc, 0x85fc, PDF_CMAP_SINGLE, 4214 },
	{ 0x85ff, 0x8600, PDF_CMAP_RANGE, 4215 },
	{ 0x8604, 0x8605, PDF_CMAP_RANGE, 4217 },
	{ 0x860d, 0x860e, PDF_CMAP_RANGE, 4219 },
	{ 0x8610, 0x8612, PDF_CMAP_RANGE, 4221 },
	{ 0x8618, 0x8619, PDF_CMAP_RANGE, 4224 },
	{ 0x861b, 0x861b, PDF_CMAP_SINGLE, 4226 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 4227 },
	{ 0x8621, 0x8621, PDF_CMAP_SINGLE, 4228 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 4229 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 4230 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 4231 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 4232 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 4233 },
	{ 0x863c, 0x863d, PDF_CMAP_RANGE, 4234 },
	{ 0x8640, 0x8642, PDF_CMAP_TABLE, 254 },
	{ 0x8646, 0x8646, PDF_CMAP_SINGLE, 4238 },
	{ 0x8652, 0x8653, PDF_CMAP_RANGE, 4239 },
	{ 0x8656, 0x8659, PDF_CMAP_RANGE, 4241 },
	{ 0x865d, 0x865d, PDF_CMAP_SINGLE, 4245 },
	{ 0x8660, 0x8664, PDF_CMAP_RANGE, 4246 },
	{ 0x8669, 0x8669, PDF_CMAP_SINGLE, 4251 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 4252 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 4253 },
	{ 0x8675, 0x8677, PDF_CMAP_RANGE, 4254 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 4257 },
	{ 0x8688, 0x8688, PDF_CMAP_SINGLE, 4291 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 4258 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 4259 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 4260 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 4261 },
	{ 0x869a, 0x869a, PDF_CMAP_SINGLE, 4262 },
	{ 0x869c, 0x869c, PDF_CMAP_SINGLE, 4263 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 4264 },
	{ 0x86a6, 0x86a8, PDF_CMAP_RANGE, 4265 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 4268 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 4269 },
	{ 0x86b3, 0x86b5, PDF_CMAP_RANGE, 4270 },
	{ 0x86b7, 0x86b9, PDF_CMAP_RANGE, 4273 },
	{ 0x86bf, 0x86c1, PDF_CMAP_RANGE, 4276 },
	{ 0x86c3, 0x86c3, PDF_CMAP_SINGLE, 4279 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 4280 },
	{ 0x86d1, 0x86d2, PDF_CMAP_RANGE, 4281 },
	{ 0x86d5, 0x86d5, PDF_CMAP_SINGLE, 4283 },
	{ 0x86d7, 0x86d7, PDF_CMAP_SINGLE, 4284 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 4285 },
	{ 0x86dc, 0x86dc, PDF_CMAP_SINGLE, 4286 },
	{ 0x86e0, 0x86e0, PDF_CMAP_SINGLE, 4287 },
	{ 0x86e3, 0x86e3, PDF_CMAP_SINGLE, 4288 },
	{ 0x86e5, 0x86e5, PDF_CMAP_SINGLE, 4289 },
	{ 0x86e7, 0x86e7, PDF_CMAP_SINGLE, 4290 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 4292 },
	{ 0x86fc, 0x86fd, PDF_CMAP_RANGE, 4293 },
	{ 0x8704, 0x8705, PDF_CMAP_RANGE, 4295 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 4297 },
	{ 0x870b, 0x870b, PDF_CMAP_SINGLE, 4298 },
	{ 0x870e, 0x8710, PDF_CMAP_RANGE, 4299 },
	{ 0x8713, 0x8714, PDF_CMAP_RANGE, 4302 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 4304 },
	{ 0x871e, 0x871f, PDF_CMAP_RANGE, 4305 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 4307 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 4308 },
	{ 0x8728, 0x8728, PDF_CMAP_SINGLE, 4309 },
	{ 0x872e, 0x872f, PDF_CMAP_RANGE, 4310 },
	{ 0x8731, 0x8732, PDF_CMAP_RANGE, 4312 },
	{ 0x8739, 0x873a, PDF_CMAP_RANGE, 4314 },
	{ 0x873c, 0x873e, PDF_CMAP_RANGE, 4316 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 4319 },
	{ 0x8743, 0x8743, PDF_CMAP_SINGLE, 4320 },
	{ 0x8745, 0x8745, PDF_CMAP_SINGLE, 4321 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 4322 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 4323 },
	{ 0x875d, 0x875d, PDF_CMAP_SINGLE, 4324 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 4325 },
	{ 0x8764, 0x8765, PDF_CMAP_RANGE, 4326 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 4328 },
	{ 0x8771, 0x8772, PDF_CMAP_RANGE, 4329 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 4331 },
	{ 0x8783, 0x8789, PDF_CMAP_RANGE, 4332 },
	{ 0x878b, 0x878c, PDF_CMAP_RANGE, 4339 },
	{ 0x8790, 0x8790, PDF_CMAP_SINGLE, 4341 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 4342 },
	{ 0x8795, 0x8795, PDF_CMAP_SINGLE, 4343 },
	{ 0x8797, 0x8799, PDF_CMAP_RANGE, 4344 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 4347 },
	{ 0x87a0, 0x87a0, PDF_CMAP_SINGLE, 4348 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 4349 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 4350 },
	{ 0x87ac, 0x87ae, PDF_CMAP_RANGE, 4351 },
	{ 0x87b1, 0x87b1, PDF_CMAP_SINGLE, 4354 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 4355 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 4356 },
	{ 0x87c1, 0x87c1, PDF_CMAP_SINGLE, 4358 },
	{ 0x87c8, 0x87ca, PDF_CMAP_RANGE, 4359 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 4362 },
	{ 0x87d5, 0x87d6, PDF_CMAP_RANGE, 4363 },
	{ 0x87d9, 0x87da, PDF_CMAP_RANGE, 4365 },
	{ 0x87dc, 0x87dc, PDF_CMAP_SINGLE, 4367 },
	{ 0x87df, 0x87df, PDF_CMAP_SINGLE, 4368 },
	{ 0x87e2, 0x87e4, PDF_CMAP_RANGE, 4369 },
	{ 0x87ea, 0x87eb, PDF_CMAP_RANGE, 4372 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 4374 },
	{ 0x87f1, 0x87f1, PDF_CMAP_SINGLE, 4375 },
	{ 0x87f3, 0x87f3, PDF_CMAP_SINGLE, 4376 },
	{ 0x87f8, 0x87f8, PDF_CMAP_SINGLE, 4377 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 4378 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 4379 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 4380 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 4381 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 4382 },
	{ 0x8809, 0x880b, PDF_CMAP_RANGE, 4383 },
	{ 0x8810, 0x8810, PDF_CMAP_SINGLE, 4386 },
	{ 0x8812, 0x8814, PDF_CMAP_RANGE, 4388 },
	{ 0x8818, 0x8819, PDF_CMAP_TABLE, 257 },
	{ 0x881a, 0x881c, PDF_CMAP_RANGE, 4392 },
	{ 0x881e, 0x881f, PDF_CMAP_RANGE, 4395 },
	{ 0x8828, 0x8828, PDF_CMAP_SINGLE, 4397 },
	{ 0x882d, 0x882e, PDF_CMAP_RANGE, 4398 },
	{ 0x8830, 0x8830, PDF_CMAP_SINGLE, 4400 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 4401 },
	{ 0x8835, 0x8835, PDF_CMAP_SINGLE, 4402 },
	{ 0x883a, 0x883a, PDF_CMAP_SINGLE, 4403 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 4404 },
	{ 0x8841, 0x8841, PDF_CMAP_SINGLE, 4405 },
	{ 0x8843, 0x8843, PDF_CMAP_SINGLE, 4406 },
	{ 0x8845, 0x8845, PDF_CMAP_SINGLE, 4407 },
	{ 0x8848, 0x884b, PDF_CMAP_RANGE, 4408 },
	{ 0x884e, 0x884e, PDF_CMAP_SINGLE, 4412 },
	{ 0x8851, 0x8851, PDF_CMAP_SINGLE, 4413 },
	{ 0x8855, 0x8856, PDF_CMAP_RANGE, 4414 },
	{ 0x8858, 0x8858, PDF_CMAP_SINGLE, 4416 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 4417 },
	{ 0x885c, 0x885c, PDF_CMAP_SINGLE, 4418 },
	{ 0x885f, 0x8860, PDF_CMAP_RANGE, 4419 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 4421 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 4422 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 4423 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 4424 },
	{ 0x887b, 0x887b, PDF_CMAP_SINGLE, 4425 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 4426 },
	{ 0x8898, 0x8898, PDF_CMAP_SINGLE, 4427 },
	{ 0x889a, 0x889c, PDF_CMAP_RANGE, 4428 },
	{ 0x889f, 0x88a0, PDF_CMAP_RANGE, 4431 },
	{ 0x88a8, 0x88a8, PDF_CMAP_SINGLE, 4433 },
	{ 0x88aa, 0x88aa, PDF_CMAP_SINGLE, 4434 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 4435 },
	{ 0x88bd, 0x88be, PDF_CMAP_RANGE, 4436 },
	{ 0x88c0, 0x88c0, PDF_CMAP_SINGLE, 4438 },
	{ 0x88ca, 0x88ce, PDF_CMAP_RANGE, 4439 },
	{ 0x88d1, 0x88d3, PDF_CMAP_RANGE, 4444 },
	{ 0x88db, 0x88db, PDF_CMAP_SINGLE, 4447 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 4448 },
	{ 0x88e7, 0x88e7, PDF_CMAP_SINGLE, 4449 },
	{ 0x88ef, 0x88f1, PDF_CMAP_RANGE, 4450 },
	{ 0x88f5, 0x88f5, PDF_CMAP_SINGLE, 4453 },
	{ 0x88f7, 0x88f7, PDF_CMAP_SINGLE, 4454 },
	{ 0x8901, 0x8901, PDF_CMAP_SINGLE, 4455 },
	{ 0x8906, 0x8906, PDF_CMAP_SINGLE, 4456 },
	{ 0x890d, 0x890f, PDF_CMAP_RANGE, 4457 },
	{ 0x8915, 0x8916, PDF_CMAP_RANGE, 4460 },
	{ 0x8918, 0x891a, PDF_CMAP_RANGE, 4462 },
	{ 0x891c, 0x891c, PDF_CMAP_SINGLE, 4465 },
	{ 0x8920, 0x8920, PDF_CMAP_SINGLE, 4466 },
	{ 0x8926, 0x8928, PDF_CMAP_RANGE, 4467 },
	{ 0x8930, 0x8932, PDF_CMAP_RANGE, 4470 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 4473 },
	{ 0x8939, 0x893a, PDF_CMAP_RANGE, 4474 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 4476 },
	{ 0x8940, 0x8940, PDF_CMAP_SINGLE, 4477 },
	{ 0x8942, 0x8942, PDF_CMAP_SINGLE, 4478 },
	{ 0x8945, 0x8946, PDF_CMAP_RANGE, 4479 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 4481 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 4482 },
	{ 0x8952, 0x8952, PDF_CMAP_SINGLE, 4483 },
	{ 0x8957, 0x8957, PDF_CMAP_SINGLE, 4484 },
	{ 0x895a, 0x895c, PDF_CMAP_RANGE, 4485 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 4488 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 4491 },
	{ 0x896e, 0x896e, PDF_CMAP_SINGLE, 4492 },
	{ 0x8970, 0x8970, PDF_CMAP_SINGLE, 4493 },
	{ 0x8973, 0x8973, PDF_CMAP_SINGLE, 4494 },
	{ 0x8975, 0x8975, PDF_CMAP_SINGLE, 4495 },
	{ 0x897a, 0x897d, PDF_CMAP_RANGE, 4496 },
	{ 0x8989, 0x8989, PDF_CMAP_SINGLE, 4500 },
	{ 0x898d, 0x898d, PDF_CMAP_SINGLE, 4501 },
	{ 0x8990, 0x8990, PDF_CMAP_SINGLE, 4502 },
	{ 0x8994, 0x8995, PDF_CMAP_RANGE, 4503 },
	{ 0x899b, 0x899c, PDF_CMAP_RANGE, 4505 },
	{ 0x899f, 0x89a0, PDF_CMAP_RANGE, 4507 },
	{ 0x89a5, 0x89a5, PDF_CMAP_SINGLE, 4509 },
	{ 0x89b0, 0x89b0, PDF_CMAP_SINGLE, 4510 },
	{ 0x89b4, 0x89b7, PDF_CMAP_RANGE, 4511 },
	{ 0x89bc, 0x89bc, PDF_CMAP_SINGLE, 4515 },
	{ 0x89d4, 0x89d8, PDF_CMAP_RANGE, 4516 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 4521 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 4522 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 4523 },
	{ 0x89ed, 0x89ed, PDF_CMAP_SINGLE, 4524 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 4525 },
	{ 0x89f3, 0x89f3, PDF_CMAP_SINGLE, 4526 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 4527 },
	{ 0x89f9, 0x89f9, PDF_CMAP_SINGLE, 4528 },
	{ 0x89fd, 0x89fd, PDF_CMAP_SINGLE, 4529 },
	{ 0x89ff, 0x89ff, PDF_CMAP_SINGLE, 4530 },
	{ 0x8a04, 0x8a05, PDF_CMAP_RANGE, 4531 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 4533 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 4534 },
	{ 0x8a11, 0x8a12, PDF_CMAP_RANGE, 4535 },
	{ 0x8a14, 0x8a15, PDF_CMAP_RANGE, 4537 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 4539 },
	{ 0x8a20, 0x8a20, PDF_CMAP_SINGLE, 4540 },
	{ 0x8a22, 0x8a22, PDF_CMAP_SINGLE, 4541 },
	{ 0x8a24, 0x8a24, PDF_CMAP_SINGLE, 4542 },
	{ 0x8a26, 0x8a26, PDF_CMAP_SINGLE, 4543 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 4544 },
	{ 0x8a2f, 0x8a2f, PDF_CMAP_SINGLE, 4546 },
	{ 0x8a35, 0x8a35, PDF_CMAP_SINGLE, 4547 },
	{ 0x8a37, 0x8a37, PDF_CMAP_SINGLE, 4548 },
	{ 0x8a3d, 0x8a3e, PDF_CMAP_RANGE, 4549 },
	{ 0x8a40, 0x8a40, PDF_CMAP_SINGLE, 4551 },
	{ 0x8a43, 0x8a43, PDF_CMAP_SINGLE, 4552 },
	{ 0x8a45, 0x8a45, PDF_CMAP_SINGLE, 4553 },
	{ 0x8a47, 0x8a47, PDF_CMAP_SINGLE, 4554 },
	{ 0x8a49, 0x8a49, PDF_CMAP_SINGLE, 4555 },
	{ 0x8a4d, 0x8a4e, PDF_CMAP_RANGE, 4556 },
	{ 0x8a53, 0x8a53, PDF_CMAP_SINGLE, 4558 },
	{ 0x8a56, 0x8a58, PDF_CMAP_RANGE, 4559 },
	{ 0x8a5c, 0x8a5d, PDF_CMAP_RANGE, 4562 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 4564 },
	{ 0x8a65, 0x8a65, PDF_CMAP_SINGLE, 4565 },
	{ 0x8a67, 0x8a67, PDF_CMAP_SINGLE, 4566 },
	{ 0x8a75, 0x8a77, PDF_CMAP_RANGE, 4567 },
	{ 0x8a79, 0x8a7b, PDF_CMAP_RANGE, 4570 },
	{ 0x8a7e, 0x8a80, PDF_CMAP_RANGE, 4573 },
	{ 0x8a83, 0x8a83, PDF_CMAP_SINGLE, 4576 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 4577 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 4578 },
	{ 0x8a8f, 0x8a90, PDF_CMAP_RANGE, 4579 },
	{ 0x8a92, 0x8a92, PDF_CMAP_SINGLE, 4581 },
	{ 0x8a96, 0x8a97, PDF_CMAP_RANGE, 4582 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 4584 },
	{ 0x8a9f, 0x8a9f, PDF_CMAP_SINGLE, 4585 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 4586 },
	{ 0x8aa9, 0x8aa9, PDF_CMAP_SINGLE, 4587 },
	{ 0x8aae, 0x8aaf, PDF_CMAP_RANGE, 4588 },
	{ 0x8ab3, 0x8ab3, PDF_CMAP_SINGLE, 4590 },
	{ 0x8ab6, 0x8ab7, PDF_CMAP_RANGE, 4591 },
	{ 0x8abb, 0x8abb, PDF_CMAP_SINGLE, 4593 },
	{ 0x8abe, 0x8abe, PDF_CMAP_SINGLE, 4594 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 4595 },
	{ 0x8ac6, 0x8ac6, PDF_CMAP_SINGLE, 4596 },
	{ 0x8ac8, 0x8aca, PDF_CMAP_RANGE, 4597 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 4600 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 4601 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 4604 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 4605 },
	{ 0x8adf, 0x8adf, PDF_CMAP_SINGLE, 4606 },
	{ 0x8aec, 0x8aec, PDF_CMAP_SINGLE, 4607 },
	{ 0x8af0, 0x8af0, PDF_CMAP_SINGLE, 4608 },
	{ 0x8af4, 0x8af6, PDF_CMAP_RANGE, 4609 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 4612 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 4613 },
	{ 0x8b05, 0x8b06, PDF_CMAP_RANGE, 4614 },
	{ 0x8b0a, 0x8b0b, PDF_CMAP_TABLE, 259 },
	{ 0x8b11, 0x8b11, PDF_CMAP_SINGLE, 4617 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 4618 },
	{ 0x8b1e, 0x8b1f, PDF_CMAP_RANGE, 4619 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 4622 },
	{ 0x8b30, 0x8b30, PDF_CMAP_SINGLE, 4623 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 4624 },
	{ 0x8b3c, 0x8b3c, PDF_CMAP_SINGLE, 4625 },
	{ 0x8b42, 0x8b46, PDF_CMAP_RANGE, 4626 },
	{ 0x8b48, 0x8b48, PDF_CMAP_SINGLE, 4631 },
	{ 0x8b4d, 0x8b4d, PDF_CMAP_SINGLE, 4636 },
	{ 0x8b52, 0x8b54, PDF_CMAP_RANGE, 4632 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 4635 },
	{ 0x8b5e, 0x8b5e, PDF_CMAP_SINGLE, 4637 },
	{ 0x8b63, 0x8b63, PDF_CMAP_SINGLE, 4638 },
	{ 0x8b6d, 0x8b6d, PDF_CMAP_SINGLE, 4639 },
	{ 0x8b76, 0x8b76, PDF_CMAP_SINGLE, 4640 },
	{ 0x8b78, 0x8b79, PDF_CMAP_RANGE, 4641 },
	{ 0x8b7c, 0x8b7c, PDF_CMAP_SINGLE, 4643 },
	{ 0x8b7e, 0x8b7e, PDF_CMAP_SINGLE, 4644 },
	{ 0x8b81, 0x8b81, PDF_CMAP_SINGLE, 4645 },
	{ 0x8b84, 0x8b85, PDF_CMAP_RANGE, 4646 },
	{ 0x8b8b, 0x8b8b, PDF_CMAP_SINGLE, 4648 },
	{ 0x8b8d, 0x8b8d, PDF_CMAP_SINGLE, 4649 },
	{ 0x8b8f, 0x8b8f, PDF_CMAP_SINGLE, 4650 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 4651 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 4653 },
	{ 0x8b9e, 0x8b9f, PDF_CMAP_RANGE, 4654 },
	{ 0x8c38, 0x8c39, PDF_CMAP_RANGE, 4656 },
	{ 0x8c3d, 0x8c3e, PDF_CMAP_RANGE, 4658 },
	{ 0x8c45, 0x8c45, PDF_CMAP_SINGLE, 4660 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 4661 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 4662 },
	{ 0x8c4b, 0x8c4b, PDF_CMAP_SINGLE, 4663 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 4664 },
	{ 0x8c51, 0x8c51, PDF_CMAP_SINGLE, 4665 },
	{ 0x8c53, 0x8c54, PDF_CMAP_RANGE, 4666 },
	{ 0x8c57, 0x8c58, PDF_CMAP_RANGE, 4668 },
	{ 0x8c59, 0x8c59, PDF_CMAP_SINGLE, 4672 },
	{ 0x8c5b, 0x8c5b, PDF_CMAP_SINGLE, 4670 },
	{ 0x8c5d, 0x8c5d, PDF_CMAP_SINGLE, 4671 },
	{ 0x8c63, 0x8c64, PDF_CMAP_RANGE, 4673 },
	{ 0x8c66, 0x8c66, PDF_CMAP_SINGLE, 4675 },
	{ 0x8c68, 0x8c69, PDF_CMAP_RANGE, 4676 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 4678 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 4679 },
	{ 0x8c75, 0x8c76, PDF_CMAP_RANGE, 4680 },
	{ 0x8c7b, 0x8c7b, PDF_CMAP_SINGLE, 4682 },
	{ 0x8c7e, 0x8c7e, PDF_CMAP_SINGLE, 4683 },
	{ 0x8c86, 0x8c87, PDF_CMAP_RANGE, 4684 },
	{ 0x8c8b, 0x8c8b, PDF_CMAP_SINGLE, 4686 },
	{ 0x8c90, 0x8c90, PDF_CMAP_SINGLE, 4687 },
	{ 0x8c92, 0x8c93, PDF_CMAP_RANGE, 4688 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 4690 },
	{ 0x8c9b, 0x8c9c, PDF_CMAP_RANGE, 4691 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 4693 },
	{ 0x8cb9, 0x8cba, PDF_CMAP_RANGE, 4694 },
	{ 0x8cc5, 0x8cc6, PDF_CMAP_RANGE, 4696 },
	{ 0x8cc9, 0x8cc9, PDF_CMAP_SINGLE, 4698 },
	{ 0x8ccb, 0x8ccb, PDF_CMAP_SINGLE, 4699 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 4700 },
	{ 0x8cd5, 0x8cd6, PDF_CMAP_TABLE, 261 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 4703 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 4704 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 4705 },
	{ 0x8ce8, 0x8ce8, PDF_CMAP_SINGLE, 4706 },
	{ 0x8cec, 0x8cec, PDF_CMAP_SINGLE, 4707 },
	{ 0x8cef, 0x8cf0, PDF_CMAP_RANGE, 4708 },
	{ 0x8cf2, 0x8cf2, PDF_CMAP_SINGLE, 4710 },
	{ 0x8cf5, 0x8cf5, PDF_CMAP_SINGLE, 4711 },
	{ 0x8cf7, 0x8cf8, PDF_CMAP_RANGE, 4712 },
	{ 0x8cfe, 0x8cff, PDF_CMAP_RANGE, 4714 },
	{ 0x8d01, 0x8d01, PDF_CMAP_SINGLE, 4716 },
	{ 0x8d03, 0x8d03, PDF_CMAP_SINGLE, 4717 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 4718 },
	{ 0x8d12, 0x8d12, PDF_CMAP_SINGLE, 4719 },
	{ 0x8d17, 0x8d17, PDF_CMAP_SINGLE, 4720 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 4721 },
	{ 0x8d65, 0x8d65, PDF_CMAP_SINGLE, 4722 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 4723 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 4724 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 4725 },
	{ 0x8d7f, 0x8d7f, PDF_CMAP_SINGLE, 4726 },
	{ 0x8d82, 0x8d82, PDF_CMAP_SINGLE, 4727 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 4728 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 4729 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 4730 },
	{ 0x8d90, 0x8d91, PDF_CMAP_RANGE, 4731 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 4733 },
	{ 0x8d9e, 0x8da0, PDF_CMAP_RANGE, 4734 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 4737 },
	{ 0x8dab, 0x8dac, PDF_CMAP_RANGE, 4738 },
	{ 0x8daf, 0x8daf, PDF_CMAP_SINGLE, 4740 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 4741 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 4742 },
	{ 0x8db7, 0x8db7, PDF_CMAP_SINGLE, 4743 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 4744 },
	{ 0x8dbb, 0x8dbc, PDF_CMAP_TABLE, 263 },
	{ 0x8dc0, 0x8dc0, PDF_CMAP_SINGLE, 4746 },
	{ 0x8dc5, 0x8dc8, PDF_CMAP_RANGE, 4747 },
	{ 0x8dca, 0x8dca, PDF_CMAP_SINGLE, 4751 },
	{ 0x8dce, 0x8dce, PDF_CMAP_SINGLE, 4752 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 4753 },
	{ 0x8dd4, 0x8dd5, PDF_CMAP_RANGE, 4754 },
	{ 0x8dd7, 0x8dd7, PDF_CMAP_SINGLE, 4756 },
	{ 0x8dd9, 0x8dd9, PDF_CMAP_SINGLE, 4757 },
	{ 0x8de4, 0x8de5, PDF_CMAP_RANGE, 4758 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 4760 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 4761 },
	{ 0x8df0, 0x8df0, PDF_CMAP_SINGLE, 4762 },
	{ 0x8df1, 0x8df2, PDF_CMAP_RANGE, 4764 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 4766 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 4767 },
	{ 0x8e01, 0x8e01, PDF_CMAP_SINGLE, 4768 },
	{ 0x8e04, 0x8e06, PDF_CMAP_RANGE, 4769 },
	{ 0x8e0b, 0x8e0b, PDF_CMAP_SINGLE, 4772 },
	{ 0x8e11, 0x8e11, PDF_CMAP_SINGLE, 4773 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 4774 },
	{ 0x8e16, 0x8e16, PDF_CMAP_SINGLE, 4775 },
	{ 0x8e20, 0x8e23, PDF_CMAP_RANGE, 4776 },
	{ 0x8e26, 0x8e27, PDF_CMAP_RANGE, 4780 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 4782 },
	{ 0x8e33, 0x8e33, PDF_CMAP_SINGLE, 4783 },
	{ 0x8e36, 0x8e39, PDF_CMAP_RANGE, 4784 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 4788 },
	{ 0x8e40, 0x8e41, PDF_CMAP_RANGE, 4789 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 4791 },
	{ 0x8e4d, 0x8e4f, PDF_CMAP_RANGE, 4792 },
	{ 0x8e54, 0x8e54, PDF_CMAP_SINGLE, 4795 },
	{ 0x8e5b, 0x8e5e, PDF_CMAP_RANGE, 4796 },
	{ 0x8e61, 0x8e62, PDF_CMAP_RANGE, 4800 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 4802 },
	{ 0x8e6c, 0x8e6d, PDF_CMAP_RANGE, 4803 },
	{ 0x8e6f, 0x8e71, PDF_CMAP_RANGE, 4805 },
	{ 0x8e79, 0x8e7b, PDF_CMAP_RANGE, 4808 },
	{ 0x8e82, 0x8e83, PDF_CMAP_RANGE, 4811 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 4813 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 4814 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 4815 },
	{ 0x8e95, 0x8e95, PDF_CMAP_SINGLE, 4816 },
	{ 0x8e9a, 0x8e9b, PDF_CMAP_RANGE, 4817 },
	{ 0x8e9d, 0x8e9e, PDF_CMAP_RANGE, 4819 },
	{ 0x8ea2, 0x8ea2, PDF_CMAP_SINGLE, 4821 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 4822 },
	{ 0x8ea9, 0x8ea9, PDF_CMAP_SINGLE, 4823 },
	{ 0x8ead, 0x8eae, PDF_CMAP_RANGE, 4824 },
	{ 0x8eb3, 0x8eb3, PDF_CMAP_SINGLE, 4826 },
	{ 0x8eb5, 0x8eb5, PDF_CMAP_SINGLE, 4827 },
	{ 0x8eba, 0x8ebb, PDF_CMAP_RANGE, 4828 },
	{ 0x8ec0, 0x8ec1, PDF_CMAP_RANGE, 4830 },
	{ 0x8ec3, 0x8ec4, PDF_CMAP_RANGE, 4832 },
	{ 0x8ec7, 0x8ec7, PDF_CMAP_SINGLE, 4834 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 4835 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 4836 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 4837 },
	{ 0x8edc, 0x8edc, PDF_CMAP_SINGLE, 4838 },
	{ 0x8ee8, 0x8ee8, PDF_CMAP_SINGLE, 4839 },
	{ 0x8eed, 0x8eee, PDF_CMAP_TABLE, 265 },
	{ 0x8ef0, 0x8ef1, PDF_CMAP_RANGE, 4841 },
	{ 0x8ef7, 0x8ef7, PDF_CMAP_SINGLE, 4843 },
	{ 0x8ef9, 0x8efa, PDF_CMAP_RANGE, 4844 },
	{ 0x8f00, 0x8f00, PDF_CMAP_SINGLE, 4847 },
	{ 0x8f02, 0x8f02, PDF_CMAP_SINGLE, 4848 },
	{ 0x8f07, 0x8f08, PDF_CMAP_RANGE, 4849 },
	{ 0x8f0f, 0x8f10, PDF_CMAP_RANGE, 4851 },
	{ 0x8f16, 0x8f18, PDF_CMAP_RANGE, 4853 },
	{ 0x8f1e, 0x8f1e, PDF_CMAP_SINGLE, 4856 },
	{ 0x8f20, 0x8f21, PDF_CMAP_RANGE, 4857 },
	{ 0x8f23, 0x8f23, PDF_CMAP_SINGLE, 4859 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 4860 },
	{ 0x8f27, 0x8f28, PDF_CMAP_RANGE, 4861 },
	{ 0x8f2c, 0x8f2e, PDF_CMAP_RANGE, 4863 },
	{ 0x8f34, 0x8f37, PDF_CMAP_RANGE, 4866 },
	{ 0x8f3a, 0x8f3a, PDF_CMAP_SINGLE, 4870 },
	{ 0x8f40, 0x8f41, PDF_CMAP_RANGE, 4871 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 4873 },
	{ 0x8f47, 0x8f47, PDF_CMAP_SINGLE, 4874 },
	{ 0x8f4f, 0x8f4f, PDF_CMAP_SINGLE, 4875 },
	{ 0x8f51, 0x8f55, PDF_CMAP_RANGE, 4876 },
	{ 0x8f58, 0x8f58, PDF_CMAP_SINGLE, 4881 },
	{ 0x8f5d, 0x8f5e, PDF_CMAP_RANGE, 4882 },
	{ 0x8f65, 0x8f65, PDF_CMAP_SINGLE, 4884 },
	{ 0x8f9d, 0x8f9d, PDF_CMAP_SINGLE, 4885 },
	{ 0x8fa0, 0x8fa1, PDF_CMAP_RANGE, 4886 },
	{ 0x8fa4, 0x8fa6, PDF_CMAP_RANGE, 4888 },
	{ 0x8fb5, 0x8fb6, PDF_CMAP_RANGE, 4891 },
	{ 0x8fb8, 0x8fb8, PDF_CMAP_SINGLE, 4893 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 4894 },
	{ 0x8fc0, 0x8fc1, PDF_CMAP_RANGE, 4895 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 4897 },
	{ 0x8fca, 0x8fcb, PDF_CMAP_RANGE, 4898 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 4900 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 4901 },
	{ 0x8fd2, 0x8fd3, PDF_CMAP_RANGE, 4902 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 4904 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 4905 },
	{ 0x8fe3, 0x8fe4, PDF_CMAP_RANGE, 4906 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 4908 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 4909 },
	{ 0x8ff1, 0x8ff1, PDF_CMAP_SINGLE, 4910 },
	{ 0x8ff5, 0x8ff6, PDF_CMAP_RANGE, 4911 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 4913 },
	{ 0x8ffe, 0x8ffe, PDF_CMAP_SINGLE, 4914 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 4915 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 4916 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 4917 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 4918 },
	{ 0x9018, 0x9018, PDF_CMAP_SINGLE, 4919 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 4920 },
	{ 0x9028, 0x9029, PDF_CMAP_RANGE, 4921 },
	{ 0x902a, 0x902a, PDF_CMAP_SINGLE, 4924 },
	{ 0x902c, 0x902d, PDF_CMAP_RANGE, 4925 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 4923 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 4927 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 4929 },
	{ 0x903f, 0x903f, PDF_CMAP_SINGLE, 4930 },
	{ 0x9043, 0x9044, PDF_CMAP_RANGE, 4931 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 4933 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 4934 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 4935 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 4936 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 4937 },
	{ 0x906c, 0x906c, PDF_CMAP_SINGLE, 4939 },
	{ 0x9070, 0x9070, PDF_CMAP_SINGLE, 4940 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 4941 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 4942 },
	{ 0x9085, 0x9085, PDF_CMAP_SINGLE, 4943 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 4944 },
	{ 0x908b, 0x908c, PDF_CMAP_RANGE, 4945 },
	{ 0x908e, 0x908e, PDF_CMAP_SINGLE, 4947 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 4948 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 4949 },
	{ 0x9097, 0x9099, PDF_CMAP_RANGE, 4950 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 4953 },
	{ 0x90a0, 0x90a2, PDF_CMAP_RANGE, 4954 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 4957 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 4958 },
	{ 0x90b2, 0x90b4, PDF_CMAP_RANGE, 4959 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 4962 },
	{ 0x90bd, 0x90be, PDF_CMAP_TABLE, 267 },
	{ 0x90c3, 0x90c5, PDF_CMAP_RANGE, 4966 },
	{ 0x90c7, 0x90c8, PDF_CMAP_RANGE, 4969 },
	{ 0x90cc, 0x90cc, PDF_CMAP_SINGLE, 4964 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 4979 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 4971 },
	{ 0x90d7, 0x90d9, PDF_CMAP_RANGE, 4972 },
	{ 0x90dc, 0x90dd, PDF_CMAP_RANGE, 4975 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 4977 },
	{ 0x90e5, 0x90e5, PDF_CMAP_SINGLE, 4978 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 4981 },
	{ 0x90ef, 0x90f0, PDF_CMAP_RANGE, 4982 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 4984 },
	{ 0x90f6, 0x90f6, PDF_CMAP_SINGLE, 4980 },
	{ 0x90fe, 0x9100, PDF_CMAP_RANGE, 4985 },
	{ 0x9104, 0x9106, PDF_CMAP_RANGE, 4988 },
	{ 0x9108, 0x9108, PDF_CMAP_SINGLE, 4991 },
	{ 0x910d, 0x910d, PDF_CMAP_SINGLE, 4992 },
	{ 0x9110, 0x9110, PDF_CMAP_SINGLE, 4993 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 4994 },
	{ 0x9116, 0x9118, PDF_CMAP_RANGE, 4995 },
	{ 0x911a, 0x911a, PDF_CMAP_SINGLE, 4998 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 4999 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 5000 },
	{ 0x9120, 0x9120, PDF_CMAP_SINGLE, 5001 },
	{ 0x9122, 0x9123, PDF_CMAP_RANGE, 5003 },
	{ 0x9125, 0x9125, PDF_CMAP_SINGLE, 5002 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 5005 },
	{ 0x9129, 0x9129, PDF_CMAP_SINGLE, 5006 },
	{ 0x912e, 0x912f, PDF_CMAP_RANGE, 5007 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 5009 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 5010 },
	{ 0x9136, 0x9137, PDF_CMAP_RANGE, 5011 },
	{ 0x9139, 0x913a, PDF_CMAP_RANGE, 5013 },
	{ 0x913c, 0x913d, PDF_CMAP_RANGE, 5015 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 5017 },
	{ 0x9147, 0x9148, PDF_CMAP_RANGE, 5018 },
	{ 0x914f, 0x914f, PDF_CMAP_SINGLE, 5020 },
	{ 0x9153, 0x9153, PDF_CMAP_SINGLE, 5021 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 5022 },
	{ 0x9159, 0x915b, PDF_CMAP_RANGE, 5023 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 5026 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 5027 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 5028 },
	{ 0x916d, 0x916d, PDF_CMAP_SINGLE, 5029 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 5030 },
	{ 0x9179, 0x917b, PDF_CMAP_RANGE, 5031 },
	{ 0x9181, 0x9181, PDF_CMAP_SINGLE, 5034 },
	{ 0x9183, 0x9183, PDF_CMAP_SINGLE, 5035 },
	{ 0x9185, 0x9186, PDF_CMAP_RANGE, 5036 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 5038 },
	{ 0x918e, 0x918e, PDF_CMAP_SINGLE, 5039 },
	{ 0x9191, 0x9191, PDF_CMAP_SINGLE, 5040 },
	{ 0x9193, 0x9195, PDF_CMAP_RANGE, 5041 },
	{ 0x9198, 0x9198, PDF_CMAP_SINGLE, 5044 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 5045 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 5046 },
	{ 0x91a6, 0x91a6, PDF_CMAP_SINGLE, 5047 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 5048 },
	{ 0x91ac, 0x91ae, PDF_CMAP_RANGE, 5049 },
	{ 0x91b0, 0x91b3, PDF_CMAP_RANGE, 5052 },
	{ 0x91b6, 0x91b6, PDF_CMAP_SINGLE, 5056 },
	{ 0x91bb, 0x91bd, PDF_CMAP_RANGE, 5057 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 5060 },
	{ 0x91c2, 0x91c3, PDF_CMAP_RANGE, 5061 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 5063 },
	{ 0x91d3, 0x91d4, PDF_CMAP_RANGE, 5064 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 5066 },
	{ 0x91d9, 0x91da, PDF_CMAP_RANGE, 5067 },
	{ 0x91de, 0x91de, PDF_CMAP_SINGLE, 5069 },
	{ 0x91e4, 0x91e5, PDF_CMAP_RANGE, 5070 },
	{ 0x91e9, 0x91ea, PDF_CMAP_RANGE, 5072 },
	{ 0x91ec, 0x91f1, PDF_CMAP_RANGE, 5074 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 5080 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 5081 },
	{ 0x91fb, 0x91fb, PDF_CMAP_SINGLE, 5082 },
	{ 0x91fd, 0x91fd, PDF_CMAP_SINGLE, 5083 },
	{ 0x9200, 0x9201, PDF_CMAP_RANGE, 5084 },
	{ 0x9204, 0x9207, PDF_CMAP_RANGE, 5086 },
	{ 0x9209, 0x920a, PDF_CMAP_RANGE, 5090 },
	{ 0x920c, 0x920c, PDF_CMAP_SINGLE, 5092 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 5093 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 5094 },
	{ 0x9216, 0x9216, PDF_CMAP_SINGLE, 5096 },
	{ 0x9218, 0x9218, PDF_CMAP_SINGLE, 5097 },
	{ 0x921c, 0x921d, PDF_CMAP_RANGE, 5098 },
	{ 0x9223, 0x9226, PDF_CMAP_RANGE, 5100 },
	{ 0x9228, 0x9228, PDF_CMAP_SINGLE, 5104 },
	{ 0x922e, 0x9230, PDF_CMAP_RANGE, 5105 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 5108 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 5109 },
	{ 0x9238, 0x923a, PDF_CMAP_RANGE, 5111 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 5114 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 5115 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 5116 },
	{ 0x9242, 0x9243, PDF_CMAP_RANGE, 5117 },
	{ 0x9246, 0x9247, PDF_CMAP_RANGE, 5119 },
	{ 0x924a, 0x924a, PDF_CMAP_SINGLE, 5121 },
	{ 0x924d, 0x924f, PDF_CMAP_RANGE, 5122 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 5125 },
	{ 0x9258, 0x9259, PDF_CMAP_RANGE, 5126 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 5128 },
	{ 0x9260, 0x9261, PDF_CMAP_RANGE, 5130 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 5132 },
	{ 0x9267, 0x9269, PDF_CMAP_RANGE, 5133 },
	{ 0x926e, 0x9270, PDF_CMAP_RANGE, 5136 },
	{ 0x9275, 0x9279, PDF_CMAP_RANGE, 5139 },
	{ 0x927b, 0x927d, PDF_CMAP_RANGE, 5144 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 5147 },
	{ 0x9288, 0x928a, PDF_CMAP_RANGE, 5148 },
	{ 0x928d, 0x928e, PDF_CMAP_RANGE, 5151 },
	{ 0x9292, 0x9292, PDF_CMAP_SINGLE, 5153 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 5154 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 5155 },
	{ 0x929f, 0x92a0, PDF_CMAP_RANGE, 5156 },
	{ 0x92a4, 0x92a5, PDF_CMAP_RANGE, 5158 },
	{ 0x92a7, 0x92a8, PDF_CMAP_RANGE, 5160 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 5162 },
	{ 0x92af, 0x92af, PDF_CMAP_SINGLE, 5163 },
	{ 0x92b2, 0x92b2, PDF_CMAP_SINGLE, 5164 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 5165 },
	{ 0x92b8, 0x92b8, PDF_CMAP_SINGLE, 5166 },
	{ 0x92ba, 0x92bd, PDF_CMAP_RANGE, 5167 },
	{ 0x92bf, 0x92c3, PDF_CMAP_RANGE, 5171 },
	{ 0x92c5, 0x92c8, PDF_CMAP_RANGE, 5176 },
	{ 0x92cb, 0x92ce, PDF_CMAP_RANGE, 5180 },
	{ 0x92d0, 0x92d0, PDF_CMAP_SINGLE, 5184 },
	{ 0x92d3, 0x92d3, PDF_CMAP_SINGLE, 5185 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 5186 },
	{ 0x92d7, 0x92d9, PDF_CMAP_RANGE, 5187 },
	{ 0x92dc, 0x92dd, PDF_CMAP_RANGE, 5190 },
	{ 0x92df, 0x92e1, PDF_CMAP_RANGE, 5192 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 5195 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 5196 },
	{ 0x92e7, 0x92e8, PDF_CMAP_RANGE, 5197 },
	{ 0x92ec, 0x92ec, PDF_CMAP_SINGLE, 5199 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 5200 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 5201 },
	{ 0x92f9, 0x92f9, PDF_CMAP_SINGLE, 5202 },
	{ 0x92fb, 0x92fb, PDF_CMAP_SINGLE, 5203 },
	{ 0x92ff, 0x9300, PDF_CMAP_RANGE, 5204 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 5206 },
	{ 0x9308, 0x9308, PDF_CMAP_SINGLE, 5207 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 5208 },
	{ 0x9311, 0x9311, PDF_CMAP_SINGLE, 5209 },
	{ 0x9314, 0x9315, PDF_CMAP_RANGE, 5210 },
	{ 0x931c, 0x931f, PDF_CMAP_RANGE, 5212 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 5216 },
	{ 0x9324, 0x9325, PDF_CMAP_RANGE, 5217 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 5219 },
	{ 0x9329, 0x932a, PDF_CMAP_RANGE, 5220 },
	{ 0x9333, 0x9334, PDF_CMAP_RANGE, 5222 },
	{ 0x9336, 0x9337, PDF_CMAP_RANGE, 5224 },
	{ 0x9347, 0x9349, PDF_CMAP_RANGE, 5226 },
	{ 0x9350, 0x9352, PDF_CMAP_RANGE, 5229 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 5232 },
	{ 0x9357, 0x9358, PDF_CMAP_RANGE, 5233 },
	{ 0x935a, 0x935a, PDF_CMAP_SINGLE, 5235 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 5236 },
	{ 0x9364, 0x9365, PDF_CMAP_RANGE, 5237 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 5239 },
	{ 0x9369, 0x936a, PDF_CMAP_RANGE, 5240 },
	{ 0x936d, 0x936d, PDF_CMAP_SINGLE, 5242 },
	{ 0x936f, 0x9371, PDF_CMAP_RANGE, 5243 },
	{ 0x9373, 0x9374, PDF_CMAP_RANGE, 5246 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 5248 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 5249 },
	{ 0x937d, 0x937d, PDF_CMAP_SINGLE, 5250 },
	{ 0x937f, 0x9382, PDF_CMAP_RANGE, 5251 },
	{ 0x9388, 0x9388, PDF_CMAP_SINGLE, 5255 },
	{ 0x938a, 0x938b, PDF_CMAP_RANGE, 5256 },
	{ 0x938d, 0x938d, PDF_CMAP_SINGLE, 5258 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 5259 },
	{ 0x9392, 0x9392, PDF_CMAP_SINGLE, 5260 },
	{ 0x9395, 0x9395, PDF_CMAP_SINGLE, 5261 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 5262 },
	{ 0x939b, 0x939b, PDF_CMAP_SINGLE, 5263 },
	{ 0x939e, 0x939e, PDF_CMAP_SINGLE, 5264 },
	{ 0x93a1, 0x93a1, PDF_CMAP_SINGLE, 5265 },
	{ 0x93a3, 0x93a4, PDF_CMAP_RANGE, 5266 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 5268 },
	{ 0x93a8, 0x93a9, PDF_CMAP_TABLE, 269 },
	{ 0x93ab, 0x93ab, PDF_CMAP_SINGLE, 5270 },
	{ 0x93b4, 0x93b6, PDF_CMAP_RANGE, 5271 },
	{ 0x93ba, 0x93ba, PDF_CMAP_SINGLE, 5274 },
	{ 0x93c1, 0x93c1, PDF_CMAP_SINGLE, 5276 },
	{ 0x93c4, 0x93c7, PDF_CMAP_RANGE, 5277 },
	{ 0x93c9, 0x93cd, PDF_CMAP_RANGE, 5281 },
	{ 0x93d3, 0x93d3, PDF_CMAP_SINGLE, 5286 },
	{ 0x93d9, 0x93d9, PDF_CMAP_SINGLE, 5287 },
	{ 0x93dc, 0x93dc, PDF_CMAP_SINGLE, 5288 },
	{ 0x93de, 0x93df, PDF_CMAP_RANGE, 5289 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 5291 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 5292 },
	{ 0x93f7, 0x93f8, PDF_CMAP_RANGE, 5295 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 5294 },
	{ 0x93fa, 0x93fb, PDF_CMAP_RANGE, 5297 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 5299 },
	{ 0x9401, 0x9402, PDF_CMAP_RANGE, 5300 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 5302 },
	{ 0x9408, 0x9409, PDF_CMAP_RANGE, 5303 },
	{ 0x940d, 0x940f, PDF_CMAP_RANGE, 5305 },
	{ 0x9415, 0x9417, PDF_CMAP_RANGE, 5308 },
	{ 0x941f, 0x941f, PDF_CMAP_SINGLE, 5311 },
	{ 0x942e, 0x942f, PDF_CMAP_RANGE, 5312 },
	{ 0x9431, 0x9434, PDF_CMAP_RANGE, 5314 },
	{ 0x943b, 0x943b, PDF_CMAP_SINGLE, 5318 },
	{ 0x943d, 0x943d, PDF_CMAP_SINGLE, 5320 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 5319 },
	{ 0x9443, 0x9443, PDF_CMAP_SINGLE, 5321 },
	{ 0x9445, 0x9445, PDF_CMAP_SINGLE, 5322 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 5323 },
	{ 0x944a, 0x944a, PDF_CMAP_SINGLE, 5324 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 5325 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 5326 },
	{ 0x9459, 0x9459, PDF_CMAP_SINGLE, 5327 },
	{ 0x945c, 0x945c, PDF_CMAP_SINGLE, 5328 },
	{ 0x945f, 0x945f, PDF_CMAP_SINGLE, 5329 },
	{ 0x9461, 0x9461, PDF_CMAP_SINGLE, 5330 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 5331 },
	{ 0x9468, 0x9468, PDF_CMAP_SINGLE, 5332 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 5333 },
	{ 0x946d, 0x946f, PDF_CMAP_RANGE, 5334 },
	{ 0x9471, 0x9472, PDF_CMAP_RANGE, 5337 },
	{ 0x9483, 0x9484, PDF_CMAP_TABLE, 271 },
	{ 0x9578, 0x9579, PDF_CMAP_RANGE, 5341 },
	{ 0x957e, 0x957e, PDF_CMAP_SINGLE, 5343 },
	{ 0x9584, 0x9584, PDF_CMAP_SINGLE, 5344 },
	{ 0x9588, 0x9588, PDF_CMAP_SINGLE, 5345 },
	{ 0x958c, 0x958e, PDF_CMAP_RANGE, 5346 },
	{ 0x959d, 0x959f, PDF_CMAP_RANGE, 5349 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 5352 },
	{ 0x95a6, 0x95a6, PDF_CMAP_SINGLE, 5353 },
	{ 0x95a9, 0x95a9, PDF_CMAP_SINGLE, 5354 },
	{ 0x95ab, 0x95ac, PDF_CMAP_RANGE, 5355 },
	{ 0x95b4, 0x95b4, PDF_CMAP_SINGLE, 5357 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 5358 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 5359 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 5360 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 5361 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 5362 },
	{ 0x95c8, 0x95c9, PDF_CMAP_RANGE, 5363 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 5365 },
	{ 0x95d0, 0x95d3, PDF_CMAP_RANGE, 5366 },
	{ 0x95d9, 0x95da, PDF_CMAP_RANGE, 5370 },
	{ 0x95dd, 0x95e0, PDF_CMAP_RANGE, 5372 },
	{ 0x95e4, 0x95e4, PDF_CMAP_SINGLE, 5376 },
	{ 0x95e6, 0x95e6, PDF_CMAP_SINGLE, 5377 },
	{ 0x961d, 0x961e, PDF_CMAP_RANGE, 5378 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 5380 },
	{ 0x9624, 0x9626, PDF_CMAP_RANGE, 5381 },
	{ 0x962c, 0x962c, PDF_CMAP_SINGLE, 5384 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 5385 },
	{ 0x9633, 0x9633, PDF_CMAP_SINGLE, 5386 },
	{ 0x9637, 0x963a, PDF_CMAP_RANGE, 5387 },
	{ 0x963c, 0x963d, PDF_CMAP_RANGE, 5391 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 5393 },
	{ 0x9652, 0x9652, PDF_CMAP_SINGLE, 5394 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 5395 },
	{ 0x9656, 0x9658, PDF_CMAP_RANGE, 5396 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 5399 },
	{ 0x966e, 0x966e, PDF_CMAP_SINGLE, 5400 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 5401 },
	{ 0x967b, 0x967c, PDF_CMAP_RANGE, 5402 },
	{ 0x967e, 0x967f, PDF_CMAP_RANGE, 5404 },
	{ 0x9681, 0x9684, PDF_CMAP_RANGE, 5406 },
	{ 0x9689, 0x9689, PDF_CMAP_SINGLE, 5410 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 5411 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 5412 },
	{ 0x969a, 0x969a, PDF_CMAP_SINGLE, 5413 },
	{ 0x969d, 0x969d, PDF_CMAP_SINGLE, 5414 },
	{ 0x969f, 0x969f, PDF_CMAP_SINGLE, 5415 },
	{ 0x96a4, 0x96a6, PDF_CMAP_RANGE, 5416 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 5419 },
	{ 0x96ae, 0x96af, PDF_CMAP_RANGE, 5420 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 5422 },
	{ 0x96ba, 0x96ba, PDF_CMAP_SINGLE, 5423 },
	{ 0x96ca, 0x96ca, PDF_CMAP_SINGLE, 5424 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 5425 },
	{ 0x96d8, 0x96d8, PDF_CMAP_SINGLE, 5427 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 5428 },
	{ 0x96dd, 0x96df, PDF_CMAP_RANGE, 5429 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 5432 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 5433 },
	{ 0x96f1, 0x96f1, PDF_CMAP_SINGLE, 5434 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 5435 },
	{ 0x9702, 0x9703, PDF_CMAP_RANGE, 5436 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 5438 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 5439 },
	{ 0x971a, 0x971b, PDF_CMAP_RANGE, 5440 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 5442 },
	{ 0x9721, 0x9723, PDF_CMAP_RANGE, 5443 },
	{ 0x9728, 0x9728, PDF_CMAP_SINGLE, 5446 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 5447 },
	{ 0x9733, 0x9733, PDF_CMAP_SINGLE, 5448 },
	{ 0x9741, 0x9741, PDF_CMAP_SINGLE, 5449 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 5450 },
	{ 0x974a, 0x974a, PDF_CMAP_SINGLE, 5451 },
	{ 0x974e, 0x974f, PDF_CMAP_RANGE, 5452 },
	{ 0x9755, 0x9755, PDF_CMAP_SINGLE, 5454 },
	{ 0x9757, 0x9758, PDF_CMAP_RANGE, 5455 },
	{ 0x975a, 0x975b, PDF_CMAP_RANGE, 5457 },
	{ 0x9763, 0x9763, PDF_CMAP_SINGLE, 5459 },
	{ 0x9767, 0x9767, PDF_CMAP_SINGLE, 5460 },
	{ 0x976a, 0x976a, PDF_CMAP_SINGLE, 5461 },
	{ 0x976e, 0x976e, PDF_CMAP_SINGLE, 5462 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 5463 },
	{ 0x9776, 0x9778, PDF_CMAP_RANGE, 5464 },
	{ 0x977b, 0x977b, PDF_CMAP_SINGLE, 5467 },
	{ 0x977d, 0x977d, PDF_CMAP_SINGLE, 5468 },
	{ 0x977f, 0x9780, PDF_CMAP_RANGE, 5469 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 5471 },
	{ 0x9795, 0x9797, PDF_CMAP_RANGE, 5472 },
	{ 0x9799, 0x979a, PDF_CMAP_RANGE, 5475 },
	{ 0x979e, 0x979f, PDF_CMAP_RANGE, 5477 },
	{ 0x97a2, 0x97a2, PDF_CMAP_SINGLE, 5479 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 5480 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 5481 },
	{ 0x97b1, 0x97b2, PDF_CMAP_RANGE, 5482 },
	{ 0x97b5, 0x97b6, PDF_CMAP_RANGE, 5484 },
	{ 0x97b8, 0x97ba, PDF_CMAP_RANGE, 5486 },
	{ 0x97bc, 0x97bc, PDF_CMAP_SINGLE, 5489 },
	{ 0x97be, 0x97bf, PDF_CMAP_RANGE, 5490 },
	{ 0x97c1, 0x97c1, PDF_CMAP_SINGLE, 5492 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 5493 },
	{ 0x97c7, 0x97c7, PDF_CMAP_SINGLE, 5495 },
	{ 0x97c9, 0x97ca, PDF_CMAP_RANGE, 5496 },
	{ 0x97cc, 0x97ce, PDF_CMAP_RANGE, 5498 },
	{ 0x97d0, 0x97d1, PDF_CMAP_RANGE, 5501 },
	{ 0x97d4, 0x97d4, PDF_CMAP_SINGLE, 5503 },
	{ 0x97d7, 0x97d9, PDF_CMAP_RANGE, 5504 },
	{ 0x97db, 0x97db, PDF_CMAP_SINGLE, 5510 },
	{ 0x97dd, 0x97de, PDF_CMAP_RANGE, 5507 },
	{ 0x97e0, 0x97e1, PDF_CMAP_TABLE, 273 },
	{ 0x97e4, 0x97e4, PDF_CMAP_SINGLE, 5512 },
	{ 0x97ef, 0x97ef, PDF_CMAP_SINGLE, 5513 },
	{ 0x97f1, 0x97f1, PDF_CMAP_SINGLE, 5514 },
	{ 0x97f4, 0x97f4, PDF_CMAP_SINGLE, 5515 },
	{ 0x97f7, 0x97f8, PDF_CMAP_RANGE, 5516 },
	{ 0x97fa, 0x97fa, PDF_CMAP_SINGLE, 5518 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 5519 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 5520 },
	{ 0x980d, 0x980e, PDF_CMAP_RANGE, 5522 },
	{ 0x9814, 0x9814, PDF_CMAP_SINGLE, 5524 },
	{ 0x9816, 0x9816, PDF_CMAP_SINGLE, 5525 },
	{ 0x9819, 0x9819, PDF_CMAP_SINGLE, 5521 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 5526 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 5527 },
	{ 0x9820, 0x9820, PDF_CMAP_SINGLE, 5528 },
	{ 0x9823, 0x9823, PDF_CMAP_SINGLE, 5529 },
	{ 0x9825, 0x9826, PDF_CMAP_TABLE, 275 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 5531 },
	{ 0x982e, 0x9830, PDF_CMAP_RANGE, 5532 },
	{ 0x9832, 0x9833, PDF_CMAP_RANGE, 5535 },
	{ 0x9835, 0x9835, PDF_CMAP_SINGLE, 5537 },
	{ 0x983e, 0x983e, PDF_CMAP_SINGLE, 5539 },
	{ 0x9844, 0x9844, PDF_CMAP_SINGLE, 5540 },
	{ 0x9847, 0x9847, PDF_CMAP_SINGLE, 5541 },
	{ 0x984a, 0x984a, PDF_CMAP_SINGLE, 5542 },
	{ 0x9851, 0x9853, PDF_CMAP_RANGE, 5543 },
	{ 0x9856, 0x9857, PDF_CMAP_RANGE, 5546 },
	{ 0x9859, 0x985a, PDF_CMAP_RANGE, 5548 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 5550 },
	{ 0x9865, 0x9866, PDF_CMAP_RANGE, 5552 },
	{ 0x986a, 0x986a, PDF_CMAP_SINGLE, 5554 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 5555 },
	{ 0x98ab, 0x98ab, PDF_CMAP_SINGLE, 5556 },
	{ 0x98ad, 0x98ae, PDF_CMAP_RANGE, 5557 },
	{ 0x98b0, 0x98b0, PDF_CMAP_SINGLE, 5559 },
	{ 0x98b4, 0x98b4, PDF_CMAP_SINGLE, 5560 },
	{ 0x98b7, 0x98b8, PDF_CMAP_RANGE, 5561 },
	{ 0x98ba, 0x98bb, PDF_CMAP_RANGE, 5563 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 5565 },
	{ 0x98c2, 0x98c2, PDF_CMAP_SINGLE, 5566 },
	{ 0x98c5, 0x98c5, PDF_CMAP_SINGLE, 5567 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 5568 },
	{ 0x98cc, 0x98cc, PDF_CMAP_SINGLE, 5569 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 5570 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 5571 },
	{ 0x98e5, 0x98e7, PDF_CMAP_RANGE, 5572 },
	{ 0x98ea, 0x98ea, PDF_CMAP_SINGLE, 5575 },
	{ 0x98f3, 0x98f3, PDF_CMAP_SINGLE, 5576 },
	{ 0x98f6, 0x98f6, PDF_CMAP_SINGLE, 5577 },
	{ 0x9902, 0x9902, PDF_CMAP_SINGLE, 5578 },
	{ 0x9907, 0x9908, PDF_CMAP_RANGE, 5579 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 5581 },
	{ 0x9915, 0x9917, PDF_CMAP_RANGE, 5582 },
	{ 0x991a, 0x991c, PDF_CMAP_RANGE, 5585 },
	{ 0x991f, 0x991f, PDF_CMAP_SINGLE, 5588 },
	{ 0x9922, 0x9922, PDF_CMAP_SINGLE, 5589 },
	{ 0x9926, 0x9927, PDF_CMAP_RANGE, 5590 },
	{ 0x992b, 0x992b, PDF_CMAP_SINGLE, 5592 },
	{ 0x9931, 0x9935, PDF_CMAP_RANGE, 5593 },
	{ 0x9939, 0x993c, PDF_CMAP_RANGE, 5598 },
	{ 0x9940, 0x9941, PDF_CMAP_RANGE, 5602 },
	{ 0x9946, 0x9948, PDF_CMAP_RANGE, 5604 },
	{ 0x994d, 0x994e, PDF_CMAP_RANGE, 5607 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 5609 },
	{ 0x9958, 0x9959, PDF_CMAP_RANGE, 5610 },
	{ 0x995b, 0x995c, PDF_CMAP_RANGE, 5612 },
	{ 0x995e, 0x9960, PDF_CMAP_RANGE, 5614 },
	{ 0x999b, 0x999b, PDF_CMAP_SINGLE, 5617 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 5618 },
	{ 0x999f, 0x999f, PDF_CMAP_SINGLE, 5619 },
	{ 0x99a6, 0x99a6, PDF_CMAP_SINGLE, 5620 },
	{ 0x99b0, 0x99b2, PDF_CMAP_RANGE, 5621 },
	{ 0x99b5, 0x99b5, PDF_CMAP_SINGLE, 5624 },
	{ 0x99b9, 0x99ba, PDF_CMAP_RANGE, 5625 },
	{ 0x99bd, 0x99bd, PDF_CMAP_SINGLE, 5627 },
	{ 0x99bf, 0x99bf, PDF_CMAP_SINGLE, 5628 },
	{ 0x99c3, 0x99c3, PDF_CMAP_SINGLE, 5629 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 5630 },
	{ 0x99d3, 0x99d4, PDF_CMAP_RANGE, 5631 },
	{ 0x99d9, 0x99da, PDF_CMAP_RANGE, 5633 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 5635 },
	{ 0x99de, 0x99de, PDF_CMAP_SINGLE, 5636 },
	{ 0x99e7, 0x99e7, PDF_CMAP_SINGLE, 5637 },
	{ 0x99ea, 0x99ec, PDF_CMAP_RANGE, 5638 },
	{ 0x99f0, 0x99f0, PDF_CMAP_SINGLE, 5641 },
	{ 0x99f4, 0x99f5, PDF_CMAP_RANGE, 5642 },
	{ 0x99f9, 0x99f9, PDF_CMAP_SINGLE, 5644 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 5645 },
	{ 0x9a02, 0x9a04, PDF_CMAP_RANGE, 5647 },
	{ 0x9a0b, 0x9a0c, PDF_CMAP_RANGE, 5650 },
	{ 0x9a10, 0x9a11, PDF_CMAP_RANGE, 5652 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 5654 },
	{ 0x9a1e, 0x9a1e, PDF_CMAP_SINGLE, 5655 },
	{ 0x9a20, 0x9a20, PDF_CMAP_SINGLE, 5656 },
	{ 0x9a22, 0x9a24, PDF_CMAP_RANGE, 5657 },
	{ 0x9a27, 0x9a27, PDF_CMAP_SINGLE, 5660 },
	{ 0x9a2d, 0x9a2e, PDF_CMAP_RANGE, 5661 },
	{ 0x9a33, 0x9a33, PDF_CMAP_SINGLE, 5663 },
	{ 0x9a35, 0x9a36, PDF_CMAP_RANGE, 5664 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 5666 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 5668 },
	{ 0x9a44, 0x9a44, PDF_CMAP_SINGLE, 5669 },
	{ 0x9a47, 0x9a47, PDF_CMAP_SINGLE, 5667 },
	{ 0x9a4a, 0x9a4c, PDF_CMAP_RANGE, 5670 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 5673 },
	{ 0x9a51, 0x9a51, PDF_CMAP_SINGLE, 5674 },
	{ 0x9a54, 0x9a54, PDF_CMAP_SINGLE, 5675 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 5676 },
	{ 0x9a5d, 0x9a5d, PDF_CMAP_SINGLE, 5677 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 5678 },
	{ 0x9aac, 0x9aac, PDF_CMAP_SINGLE, 5679 },
	{ 0x9aae, 0x9aaf, PDF_CMAP_RANGE, 5680 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 5682 },
	{ 0x9ab4, 0x9ab6, PDF_CMAP_RANGE, 5683 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 5686 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 5687 },
	{ 0x9abe, 0x9abf, PDF_CMAP_RANGE, 5688 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 5690 },
	{ 0x9ac3, 0x9ac3, PDF_CMAP_SINGLE, 5691 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 5692 },
	{ 0x9ac8, 0x9ac8, PDF_CMAP_SINGLE, 5693 },
	{ 0x9ace, 0x9ace, PDF_CMAP_SINGLE, 5694 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 5695 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 5696 },
	{ 0x9ad5, 0x9ad7, PDF_CMAP_RANGE, 5697 },
	{ 0x9adb, 0x9adc, PDF_CMAP_RANGE, 5700 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 5702 },
	{ 0x9ae4, 0x9ae5, PDF_CMAP_RANGE, 5703 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 5705 },
	{ 0x9ae9, 0x9ae9, PDF_CMAP_SINGLE, 5706 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 5707 },
	{ 0x9af2, 0x9af3, PDF_CMAP_RANGE, 5708 },
	{ 0x9af5, 0x9af5, PDF_CMAP_SINGLE, 5710 },
	{ 0x9af9, 0x9afa, PDF_CMAP_RANGE, 5711 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 5713 },
	{ 0x9aff, 0x9b05, PDF_CMAP_RANGE, 5714 },
	{ 0x9b08, 0x9b09, PDF_CMAP_RANGE, 5721 },
	{ 0x9b0b, 0x9b0e, PDF_CMAP_RANGE, 5723 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 5727 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 5728 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 5729 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 5730 },
	{ 0x9b1b, 0x9b1c, PDF_CMAP_RANGE, 5731 },
	{ 0x9b20, 0x9b20, PDF_CMAP_SINGLE, 5733 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 5734 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 5735 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 5736 },
	{ 0x9b33, 0x9b35, PDF_CMAP_RANGE, 5737 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 5740 },
	{ 0x9b39, 0x9b3a, PDF_CMAP_RANGE, 5741 },
	{ 0x9b3d, 0x9b3d, PDF_CMAP_SINGLE, 5743 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 5744 },
	{ 0x9b4b, 0x9b4c, PDF_CMAP_RANGE, 5745 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 5747 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 5750 },
	{ 0x9b5e, 0x9b5e, PDF_CMAP_SINGLE, 5751 },
	{ 0x9b61, 0x9b61, PDF_CMAP_SINGLE, 5752 },
	{ 0x9b63, 0x9b63, PDF_CMAP_SINGLE, 5753 },
	{ 0x9b65, 0x9b66, PDF_CMAP_RANGE, 5754 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 5756 },
	{ 0x9b6a, 0x9b6e, PDF_CMAP_RANGE, 5757 },
	{ 0x9b73, 0x9b73, PDF_CMAP_SINGLE, 5762 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 5763 },
	{ 0x9b77, 0x9b79, PDF_CMAP_RANGE, 5764 },
	{ 0x9b7f, 0x9b80, PDF_CMAP_RANGE, 5767 },
	{ 0x9b84, 0x9b87, PDF_CMAP_RANGE, 5769 },
	{ 0x9b89, 0x9b8b, PDF_CMAP_RANGE, 5773 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 5776 },
	{ 0x9b8f, 0x9b90, PDF_CMAP_RANGE, 5777 },
	{ 0x9b94, 0x9b94, PDF_CMAP_SINGLE, 5779 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 5780 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_RANGE, 5781 },
	{ 0x9ba6, 0x9ba7, PDF_CMAP_RANGE, 5783 },
	{ 0x9ba9, 0x9ba9, PDF_CMAP_SINGLE, 5785 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 5786 },
	{ 0x9bb0, 0x9bb2, PDF_CMAP_RANGE, 5787 },
	{ 0x9bb7, 0x9bb8, PDF_CMAP_RANGE, 5790 },
	{ 0x9bbb, 0x9bbc, PDF_CMAP_RANGE, 5792 },
	{ 0x9bbe, 0x9bbf, PDF_CMAP_RANGE, 5794 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 5796 },
	{ 0x9bc7, 0x9bc8, PDF_CMAP_RANGE, 5797 },
	{ 0x9bce, 0x9bce, PDF_CMAP_SINGLE, 5799 },
	{ 0x9bd0, 0x9bd0, PDF_CMAP_SINGLE, 5800 },
	{ 0x9bd7, 0x9bd8, PDF_CMAP_RANGE, 5801 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 5803 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 5804 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 5805 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 5806 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 5807 },
	{ 0x9bef, 0x9bef, PDF_CMAP_SINGLE, 5809 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 5810 },
	{ 0x9bf7, 0x9bfa, PDF_CMAP_RANGE, 5811 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5815 },
	{ 0x9bff, 0x9c00, PDF_CMAP_RANGE, 5816 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 5818 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 5819 },
	{ 0x9c0f, 0x9c0f, PDF_CMAP_SINGLE, 5820 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 5821 },
	{ 0x9c16, 0x9c16, PDF_CMAP_SINGLE, 5822 },
	{ 0x9c18, 0x9c1a, PDF_CMAP_RANGE, 5823 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 5826 },
	{ 0x9c1e, 0x9c1e, PDF_CMAP_SINGLE, 5827 },
	{ 0x9c22, 0x9c23, PDF_CMAP_RANGE, 5828 },
	{ 0x9c26, 0x9c2a, PDF_CMAP_RANGE, 5830 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 5835 },
	{ 0x9c35, 0x9c37, PDF_CMAP_RANGE, 5836 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 5839 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 5840 },
	{ 0x9c43, 0x9c45, PDF_CMAP_RANGE, 5841 },
	{ 0x9c49, 0x9c4a, PDF_CMAP_RANGE, 5844 },
	{ 0x9c4e, 0x9c50, PDF_CMAP_RANGE, 5846 },
	{ 0x9c53, 0x9c54, PDF_CMAP_RANGE, 5849 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 5851 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 5852 },
	{ 0x9c5b, 0x9c5c, PDF_CMAP_TABLE, 277 },
	{ 0x9c5d, 0x9c5f, PDF_CMAP_RANGE, 5854 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 5857 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 5862 },
	{ 0x9c69, 0x9c6a, PDF_CMAP_RANGE, 5858 },
	{ 0x9c6b, 0x9c6b, PDF_CMAP_SINGLE, 5861 },
	{ 0x9c6e, 0x9c6e, PDF_CMAP_SINGLE, 5863 },
	{ 0x9c70, 0x9c70, PDF_CMAP_SINGLE, 5864 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 5865 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 5866 },
	{ 0x9c77, 0x9c77, PDF_CMAP_SINGLE, 5867 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 5868 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 5869 },
	{ 0x9cf2, 0x9cf2, PDF_CMAP_SINGLE, 5870 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 5871 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 5872 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 5874 },
	{ 0x9d0b, 0x9d0b, PDF_CMAP_SINGLE, 5873 },
	{ 0x9d11, 0x9d11, PDF_CMAP_SINGLE, 5875 },
	{ 0x9d17, 0x9d18, PDF_CMAP_RANGE, 5876 },
	{ 0x9d1c, 0x9d1e, PDF_CMAP_RANGE, 5878 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_RANGE, 5881 },
	{ 0x9d32, 0x9d34, PDF_CMAP_RANGE, 5883 },
	{ 0x9d3a, 0x9d3a, PDF_CMAP_SINGLE, 5886 },
	{ 0x9d3c, 0x9d3d, PDF_CMAP_TABLE, 279 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 5890 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 5888 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 5892 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 5893 },
	{ 0x9d53, 0x9d54, PDF_CMAP_RANGE, 5894 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 5896 },
	{ 0x9d62, 0x9d63, PDF_CMAP_TABLE, 281 },
	{ 0x9d65, 0x9d65, PDF_CMAP_SINGLE, 5899 },
	{ 0x9d69, 0x9d6b, PDF_CMAP_RANGE, 5900 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 5903 },
	{ 0x9d76, 0x9d77, PDF_CMAP_RANGE, 5904 },
	{ 0x9d7b, 0x9d7c, PDF_CMAP_RANGE, 5906 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 5908 },
	{ 0x9d83, 0x9d84, PDF_CMAP_RANGE, 5909 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 5911 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 5912 },
	{ 0x9d8d, 0x9d8e, PDF_CMAP_RANGE, 5913 },
	{ 0x9d92, 0x9d93, PDF_CMAP_RANGE, 5915 },
	{ 0x9d95, 0x9d98, PDF_CMAP_RANGE, 5917 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 5921 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 5922 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 5923 },
	{ 0x9dae, 0x9dae, PDF_CMAP_SINGLE, 5924 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 5925 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 5926 },
	{ 0x9db9, 0x9db9, PDF_CMAP_SINGLE, 5927 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 5928 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 5929 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 5930 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 5931 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_RANGE, 5932 },
	{ 0x9dd4, 0x9dd7, PDF_CMAP_RANGE, 5934 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 5938 },
	{ 0x9dde, 0x9de0, PDF_CMAP_RANGE, 5939 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 5942 },
	{ 0x9de7, 0x9de7, PDF_CMAP_SINGLE, 5943 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 5944 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 5945 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 5946 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 5947 },
	{ 0x9df3, 0x9df4, PDF_CMAP_RANGE, 5948 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 5950 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 5952 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 5953 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 5951 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 5954 },
	{ 0x9e10, 0x9e12, PDF_CMAP_RANGE, 5955 },
	{ 0x9e15, 0x9e16, PDF_CMAP_RANGE, 5958 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 5960 },
	{ 0x9e1c, 0x9e1d, PDF_CMAP_RANGE, 5961 },
	{ 0x9e7a, 0x9e7c, PDF_CMAP_RANGE, 5963 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 5966 },
	{ 0x9e82, 0x9e85, PDF_CMAP_RANGE, 5967 },
	{ 0x9e87, 0x9e87, PDF_CMAP_SINGLE, 5971 },
	{ 0x9e8e, 0x9e8f, PDF_CMAP_RANGE, 5972 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 5974 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 5975 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 5976 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 5977 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 5978 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 5979 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 5980 },
	{ 0x9eae, 0x9eb0, PDF_CMAP_RANGE, 5981 },
	{ 0x9eb3, 0x9eb5, PDF_CMAP_RANGE, 5984 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 5987 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 5988 },
	{ 0x9ecb, 0x9ecb, PDF_CMAP_SINGLE, 5989 },
	{ 0x9ed5, 0x9ed5, PDF_CMAP_SINGLE, 5990 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 5991 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 5992 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 5993 },
	{ 0x9eec, 0x9eee, PDF_CMAP_RANGE, 5994 },
	{ 0x9ef0, 0x9ef2, PDF_CMAP_RANGE, 5997 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 6000 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 6001 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 6002 },
	{ 0x9f02, 0x9f03, PDF_CMAP_RANGE, 6003 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 6005 },
	{ 0x9f0f, 0x9f12, PDF_CMAP_RANGE, 6006 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 6010 },
	{ 0x9f16, 0x9f17, PDF_CMAP_RANGE, 6011 },
	{ 0x9f19, 0x9f1b, PDF_CMAP_RANGE, 6013 },
	{ 0x9f1f, 0x9f1f, PDF_CMAP_SINGLE, 6016 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 6017 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 6018 },
	{ 0x9f2a, 0x9f2b, PDF_CMAP_RANGE, 6019 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 6021 },
	{ 0x9f31, 0x9f32, PDF_CMAP_RANGE, 6022 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 6024 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 6025 },
	{ 0x9f39, 0x9f3a, PDF_CMAP_RANGE, 6026 },
	{ 0x9f3c, 0x9f3d, PDF_CMAP_RANGE, 6028 },
	{ 0x9f3f, 0x9f3f, PDF_CMAP_SINGLE, 6030 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 6031 },
	{ 0x9f43, 0x9f47, PDF_CMAP_RANGE, 6032 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 6037 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 6038 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 6042 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 6043 },
	{ 0x9f68, 0x9f69, PDF_CMAP_RANGE, 6045 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 6047 },
	{ 0x9f73, 0x9f73, PDF_CMAP_SINGLE, 6052 },
	{ 0x9f75, 0x9f75, PDF_CMAP_SINGLE, 6053 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 6054 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 6055 },
	{ 0x9f8f, 0x9f92, PDF_CMAP_RANGE, 6056 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 6060 },
	{ 0x9f96, 0x9f97, PDF_CMAP_RANGE, 6061 },
	{ 0x9f9e, 0x9f9e, PDF_CMAP_SINGLE, 6063 },
	{ 0x9fa1, 0x9fa3, PDF_CMAP_RANGE, 6064 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 6067 },
	{ 0x9a5d, 0x9a5d, PDF_CMAP_SINGLE, 5677 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 5678 },
	{ 0x9aac, 0x9aac, PDF_CMAP_SINGLE, 5679 },
	{ 0x9aae, 0x9aaf, PDF_CMAP_RANGE, 5680 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 5682 },
	{ 0x9ab4, 0x9ab6, PDF_CMAP_RANGE, 5683 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 5686 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 5687 },
	{ 0x9abe, 0x9abf, PDF_CMAP_RANGE, 5688 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 5690 },
	{ 0x9ac3, 0x9ac3, PDF_CMAP_SINGLE, 5691 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 5692 },
	{ 0x9ac8, 0x9ac8, PDF_CMAP_SINGLE, 5693 },
	{ 0x9ace, 0x9ace, PDF_CMAP_SINGLE, 5694 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 5695 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 5696 },
	{ 0x9ad5, 0x9ad7, PDF_CMAP_RANGE, 5697 },
	{ 0x9adb, 0x9adc, PDF_CMAP_RANGE, 5700 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 5702 },
	{ 0x9ae4, 0x9ae5, PDF_CMAP_RANGE, 5703 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 5705 },
	{ 0x9ae9, 0x9ae9, PDF_CMAP_SINGLE, 5706 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 5707 },
	{ 0x9af2, 0x9af3, PDF_CMAP_RANGE, 5708 },
	{ 0x9af5, 0x9af5, PDF_CMAP_SINGLE, 5710 },
	{ 0x9af9, 0x9afa, PDF_CMAP_RANGE, 5711 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 5713 },
	{ 0x9aff, 0x9aff, PDF_CMAP_SINGLE, 5714 },
	{ 0x9b00, 0x9b05, PDF_CMAP_RANGE, 5715 },
	{ 0x9b08, 0x9b09, PDF_CMAP_RANGE, 5721 },
	{ 0x9b0b, 0x9b0e, PDF_CMAP_RANGE, 5723 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 5727 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 5728 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 5729 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 5730 },
	{ 0x9b1b, 0x9b1c, PDF_CMAP_RANGE, 5731 },
	{ 0x9b20, 0x9b20, PDF_CMAP_SINGLE, 5733 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 5734 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 5735 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 5736 },
	{ 0x9b33, 0x9b35, PDF_CMAP_RANGE, 5737 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 5740 },
	{ 0x9b39, 0x9b3a, PDF_CMAP_RANGE, 5741 },
	{ 0x9b3d, 0x9b3d, PDF_CMAP_SINGLE, 5743 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 5744 },
	{ 0x9b4b, 0x9b4c, PDF_CMAP_RANGE, 5745 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 5747 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 5750 },
	{ 0x9b5e, 0x9b5e, PDF_CMAP_SINGLE, 5751 },
	{ 0x9b61, 0x9b61, PDF_CMAP_SINGLE, 5752 },
	{ 0x9b63, 0x9b63, PDF_CMAP_SINGLE, 5753 },
	{ 0x9b65, 0x9b66, PDF_CMAP_RANGE, 5754 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 5756 },
	{ 0x9b6a, 0x9b6e, PDF_CMAP_RANGE, 5757 },
	{ 0x9b73, 0x9b73, PDF_CMAP_SINGLE, 5762 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 5763 },
	{ 0x9b77, 0x9b79, PDF_CMAP_RANGE, 5764 },
	{ 0x9b7f, 0x9b80, PDF_CMAP_RANGE, 5767 },
	{ 0x9b84, 0x9b87, PDF_CMAP_RANGE, 5769 },
	{ 0x9b89, 0x9b8b, PDF_CMAP_RANGE, 5773 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 5776 },
	{ 0x9b8f, 0x9b90, PDF_CMAP_RANGE, 5777 },
	{ 0x9b94, 0x9b94, PDF_CMAP_SINGLE, 5779 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 5780 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_RANGE, 5781 },
	{ 0x9ba6, 0x9ba7, PDF_CMAP_RANGE, 5783 },
	{ 0x9ba9, 0x9ba9, PDF_CMAP_SINGLE, 5785 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 5786 },
	{ 0x9bb0, 0x9bb2, PDF_CMAP_RANGE, 5787 },
	{ 0x9bb7, 0x9bb8, PDF_CMAP_RANGE, 5790 },
	{ 0x9bbb, 0x9bbc, PDF_CMAP_RANGE, 5792 },
	{ 0x9bbe, 0x9bbf, PDF_CMAP_RANGE, 5794 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 5796 },
	{ 0x9bc7, 0x9bc8, PDF_CMAP_RANGE, 5797 },
	{ 0x9bce, 0x9bce, PDF_CMAP_SINGLE, 5799 },
	{ 0x9bd0, 0x9bd0, PDF_CMAP_SINGLE, 5800 },
	{ 0x9bd7, 0x9bd8, PDF_CMAP_RANGE, 5801 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 5803 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 5804 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 5805 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 5806 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 5807 },
	{ 0x9bef, 0x9bef, PDF_CMAP_SINGLE, 5809 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 5810 },
	{ 0x9bf7, 0x9bfa, PDF_CMAP_RANGE, 5811 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5815 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 5816 },
	{ 0x9c00, 0x9c00, PDF_CMAP_SINGLE, 5817 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 5818 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 5819 },
	{ 0x9c0f, 0x9c0f, PDF_CMAP_SINGLE, 5820 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 5821 },
	{ 0x9c16, 0x9c16, PDF_CMAP_SINGLE, 5822 },
	{ 0x9c18, 0x9c1a, PDF_CMAP_RANGE, 5823 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 5826 },
	{ 0x9c1e, 0x9c1e, PDF_CMAP_SINGLE, 5827 },
	{ 0x9c22, 0x9c23, PDF_CMAP_RANGE, 5828 },
	{ 0x9c26, 0x9c2a, PDF_CMAP_RANGE, 5830 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 5835 },
	{ 0x9c35, 0x9c37, PDF_CMAP_RANGE, 5836 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 5839 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 5840 },
	{ 0x9c43, 0x9c45, PDF_CMAP_RANGE, 5841 },
	{ 0x9c49, 0x9c4a, PDF_CMAP_RANGE, 5844 },
	{ 0x9c4e, 0x9c50, PDF_CMAP_RANGE, 5846 },
	{ 0x9c53, 0x9c54, PDF_CMAP_RANGE, 5849 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 5851 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 5852 },
	{ 0x9c5b, 0x9c5b, PDF_CMAP_SINGLE, 5853 },
	{ 0x9c5c, 0x9c5c, PDF_CMAP_SINGLE, 5860 },
	{ 0x9c5d, 0x9c5f, PDF_CMAP_RANGE, 5854 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 5857 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 5862 },
	{ 0x9c69, 0x9c6a, PDF_CMAP_RANGE, 5858 },
	{ 0x9c6b, 0x9c6b, PDF_CMAP_SINGLE, 5861 },
	{ 0x9c6e, 0x9c6e, PDF_CMAP_SINGLE, 5863 },
	{ 0x9c70, 0x9c70, PDF_CMAP_SINGLE, 5864 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 5865 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 5866 },
	{ 0x9c77, 0x9c77, PDF_CMAP_SINGLE, 5867 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 5868 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 5869 },
	{ 0x9cf2, 0x9cf2, PDF_CMAP_SINGLE, 5870 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 5871 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 5872 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 5874 },
	{ 0x9d0b, 0x9d0b, PDF_CMAP_SINGLE, 5873 },
	{ 0x9d11, 0x9d11, PDF_CMAP_SINGLE, 5875 },
	{ 0x9d17, 0x9d18, PDF_CMAP_RANGE, 5876 },
	{ 0x9d1c, 0x9d1e, PDF_CMAP_RANGE, 5878 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_RANGE, 5881 },
	{ 0x9d32, 0x9d34, PDF_CMAP_RANGE, 5883 },
	{ 0x9d3a, 0x9d3a, PDF_CMAP_SINGLE, 5886 },
	{ 0x9d3c, 0x9d3c, PDF_CMAP_SINGLE, 5887 },
	{ 0x9d3d, 0x9d3d, PDF_CMAP_SINGLE, 5889 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 5890 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 5888 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 5892 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 5893 },
	{ 0x9d53, 0x9d54, PDF_CMAP_RANGE, 5894 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 5896 },
	{ 0x9d62, 0x9d62, PDF_CMAP_SINGLE, 5898 },
	{ 0x9d63, 0x9d63, PDF_CMAP_SINGLE, 5897 },
	{ 0x9d65, 0x9d65, PDF_CMAP_SINGLE, 5899 },
	{ 0x9d69, 0x9d6b, PDF_CMAP_RANGE, 5900 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 5903 },
	{ 0x9d76, 0x9d77, PDF_CMAP_RANGE, 5904 },
	{ 0x9d7b, 0x9d7c, PDF_CMAP_RANGE, 5906 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 5908 },
	{ 0x9d83, 0x9d84, PDF_CMAP_RANGE, 5909 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 5911 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 5912 },
	{ 0x9d8d, 0x9d8e, PDF_CMAP_RANGE, 5913 },
	{ 0x9d92, 0x9d93, PDF_CMAP_RANGE, 5915 },
	{ 0x9d95, 0x9d98, PDF_CMAP_RANGE, 5917 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 5921 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 5922 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 5923 },
	{ 0x9dae, 0x9dae, PDF_CMAP_SINGLE, 5924 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 5925 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 5926 },
	{ 0x9db9, 0x9db9, PDF_CMAP_SINGLE, 5927 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 5928 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 5929 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 5930 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 5931 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_RANGE, 5932 },
	{ 0x9dd4, 0x9dd7, PDF_CMAP_RANGE, 5934 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 5938 },
	{ 0x9dde, 0x9de0, PDF_CMAP_RANGE, 5939 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 5942 },
	{ 0x9de7, 0x9de7, PDF_CMAP_SINGLE, 5943 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 5944 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 5945 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 5946 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 5947 },
	{ 0x9df3, 0x9df4, PDF_CMAP_RANGE, 5948 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 5950 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 5952 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 5953 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 5951 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 5954 },
	{ 0x9e10, 0x9e12, PDF_CMAP_RANGE, 5955 },
	{ 0x9e15, 0x9e16, PDF_CMAP_RANGE, 5958 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 5960 },
	{ 0x9e1c, 0x9e1d, PDF_CMAP_RANGE, 5961 },
	{ 0x9e7a, 0x9e7c, PDF_CMAP_RANGE, 5963 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 5966 },
	{ 0x9e82, 0x9e85, PDF_CMAP_RANGE, 5967 },
	{ 0x9e87, 0x9e87, PDF_CMAP_SINGLE, 5971 },
	{ 0x9e8e, 0x9e8f, PDF_CMAP_RANGE, 5972 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 5974 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 5975 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 5976 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 5977 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 5978 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 5979 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 5980 },
	{ 0x9eae, 0x9eb0, PDF_CMAP_RANGE, 5981 },
	{ 0x9eb3, 0x9eb5, PDF_CMAP_RANGE, 5984 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 5987 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 5988 },
	{ 0x9ecb, 0x9ecb, PDF_CMAP_SINGLE, 5989 },
	{ 0x9ed5, 0x9ed5, PDF_CMAP_SINGLE, 5990 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 5991 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 5992 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 5993 },
	{ 0x9eec, 0x9eee, PDF_CMAP_RANGE, 5994 },
	{ 0x9ef0, 0x9ef2, PDF_CMAP_RANGE, 5997 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 6000 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 6001 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 6002 },
	{ 0x9f02, 0x9f03, PDF_CMAP_RANGE, 6003 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 6005 },
	{ 0x9f0f, 0x9f12, PDF_CMAP_RANGE, 6006 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 6010 },
	{ 0x9f16, 0x9f17, PDF_CMAP_RANGE, 6011 },
	{ 0x9f19, 0x9f1b, PDF_CMAP_RANGE, 6013 },
	{ 0x9f1f, 0x9f1f, PDF_CMAP_SINGLE, 6016 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 6017 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 6018 },
	{ 0x9f2a, 0x9f2b, PDF_CMAP_RANGE, 6019 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 6021 },
	{ 0x9f31, 0x9f32, PDF_CMAP_RANGE, 6022 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 6024 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 6025 },
	{ 0x9f39, 0x9f3a, PDF_CMAP_RANGE, 6026 },
	{ 0x9f3c, 0x9f3d, PDF_CMAP_RANGE, 6028 },
	{ 0x9f3f, 0x9f3f, PDF_CMAP_SINGLE, 6030 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 6031 },
	{ 0x9f43, 0x9f47, PDF_CMAP_RANGE, 6032 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 6037 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 6038 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 6042 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 6043 },
	{ 0x9f68, 0x9f69, PDF_CMAP_RANGE, 6045 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 6047 },
	{ 0x9f73, 0x9f73, PDF_CMAP_SINGLE, 6052 },
	{ 0x9f75, 0x9f75, PDF_CMAP_SINGLE, 6053 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 6054 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 6055 },
	{ 0x9f8f, 0x9f92, PDF_CMAP_RANGE, 6056 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 6060 },
	{ 0x9f96, 0x9f97, PDF_CMAP_RANGE, 6061 },
	{ 0x9f9e, 0x9f9e, PDF_CMAP_SINGLE, 6063 },
	{ 0x9fa1, 0x9fa3, PDF_CMAP_RANGE, 6064 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 6067 },
};

static const unsigned short pdf_cmap_UniHojo_UCS2_H_table[283] =
{
	17,16,18,6,14,97,96,99,
	105,98,104,69,109,113,112,115,
	114,126,125,128,127,142,144,143,
	146,150,145,76,161,160,163,162,
	176,79,93,183,182,185,191,184,
	190,80,195,199,198,201,200,212,
	211,214,213,82,227,229,228,231,
	235,230,91,246,245,248,247,261,
	95,262,102,188,100,186,103,189,
	106,192,107,193,110,196,108,194,
	111,197,70,81,118,204,117,203,
	119,205,116,202,120,207,121,208,
	123,209,122,124,210,71,83,133,
	218,131,216,132,217,130,84,72,
	85,134,219,135,220,86,136,221,
	138,223,137,222,74,88,73,87,
	139,224,141,226,140,225,89,75,
	90,149,234,148,233,77,92,151,
	236,153,238,152,237,154,239,155,
	240,157,242,156,241,159,244,158,
	243,78,94,170,255,167,252,164,
	249,169,254,166,251,168,253,175,
	260,178,263,177,179,264,181,266,
	180,265,101,187,129,215,147,232,
	165,250,174,259,171,256,173,258,
	172,257,1,4,8,7,28,30,
	36,26,29,35,40,37,39,42,
	346,329,491,503,666,665,979,988,
	1413,1415,1444,1443,1505,1507,2000,2002,
	1993,1996,2001,2035,1989,2099,2098,2285,
	2284,3095,3089,3096,3301,3296,3726,3725,
	3894,3891,4056,3999,4045,4070,4236,1088,
	4237,4391,4387,4621,4616,4702,4701,4745,
	4763,4846,4840,4963,4965,5269,5275,5340,
	5339,5509,5511,5538,5530,5853,5860,5887,
	5889,5898,5897,
};

pdf_cmap pdf_cmap_UniHojo_UCS2_H =
{
	-1, "UniHojo-UCS2-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 2, 0xe000, 0xffff },
	},
	4232, 4232, (pdf_range*) pdf_cmap_UniHojo_UCS2_H_ranges,
	283, 283, (unsigned short*) pdf_cmap_UniHojo_UCS2_H_table,
};

/* UniHojo-UCS2-V */

static const pdf_range pdf_cmap_UniHojo_UCS2_V_ranges[] =
{
	/* dummy entry for non-c99 compilers */
	{ 0x0, 0x0, PDF_CMAP_RANGE, 0 }
};

static const unsigned short pdf_cmap_UniHojo_UCS2_V_table[1] = { 0 };

pdf_cmap pdf_cmap_UniHojo_UCS2_V =
{
	-1, "UniHojo-UCS2-V", "UniHojo-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	0, 0, (pdf_range*) pdf_cmap_UniHojo_UCS2_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_UniHojo_UCS2_V_table,
};

/* UniHojo-UTF16-H */

static const pdf_range pdf_cmap_UniHojo_UTF16_H_ranges[] =
{
	{ 0x007e, 0x007e, PDF_CMAP_SINGLE, 9 },
	{ 0x00a1, 0x00a1, PDF_CMAP_SINGLE, 12 },
	{ 0x00a4, 0x00a4, PDF_CMAP_SINGLE, 20 },
	{ 0x00a6, 0x00a6, PDF_CMAP_SINGLE, 13 },
	{ 0x00a9, 0x00aa, PDF_CMAP_TABLE, 0 },
	{ 0x00ae, 0x00af, PDF_CMAP_TABLE, 2 },
	{ 0x00b8, 0x00b8, PDF_CMAP_SINGLE, 3 },
	{ 0x00ba, 0x00ba, PDF_CMAP_SINGLE, 15 },
	{ 0x00bf, 0x00cf, PDF_CMAP_TABLE, 4 },
	{ 0x00d1, 0x00d6, PDF_CMAP_TABLE, 21 },
	{ 0x00d8, 0x00f6, PDF_CMAP_TABLE, 27 },
	{ 0x00f8, 0x0113, PDF_CMAP_TABLE, 58 },
	{ 0x0116, 0x0122, PDF_CMAP_TABLE, 86 },
	{ 0x0124, 0x012b, PDF_CMAP_TABLE, 99 },
	{ 0x012e, 0x014d, PDF_CMAP_TABLE, 107 },
	{ 0x0150, 0x017e, PDF_CMAP_TABLE, 139 },
	{ 0x01cd, 0x01dc, PDF_CMAP_TABLE, 186 },
	{ 0x01f5, 0x01f5, PDF_CMAP_SINGLE, 206 },
	{ 0x02c7, 0x02c7, PDF_CMAP_SINGLE, 2 },
	{ 0x02d8, 0x02db, PDF_CMAP_TABLE, 202 },
	{ 0x02dd, 0x02dd, PDF_CMAP_SINGLE, 5 },
	{ 0x0384, 0x0385, PDF_CMAP_RANGE, 10 },
	{ 0x0386, 0x0386, PDF_CMAP_SINGLE, 22 },
	{ 0x0388, 0x038a, PDF_CMAP_RANGE, 23 },
	{ 0x038c, 0x038c, PDF_CMAP_SINGLE, 27 },
	{ 0x038e, 0x0390, PDF_CMAP_TABLE, 206 },
	{ 0x03aa, 0x03ab, PDF_CMAP_TABLE, 209 },
	{ 0x03ac, 0x03af, PDF_CMAP_RANGE, 31 },
	{ 0x03b0, 0x03b0, PDF_CMAP_SINGLE, 41 },
	{ 0x03c2, 0x03c2, PDF_CMAP_SINGLE, 38 },
	{ 0x03ca, 0x03ce, PDF_CMAP_TABLE, 211 },
	{ 0x0402, 0x040c, PDF_CMAP_RANGE, 43 },
	{ 0x040e, 0x040f, PDF_CMAP_RANGE, 54 },
	{ 0x0452, 0x045c, PDF_CMAP_RANGE, 56 },
	{ 0x045e, 0x045f, PDF_CMAP_RANGE, 67 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 21 },
	{ 0x2122, 0x2122, PDF_CMAP_SINGLE, 19 },
	{ 0x2e83, 0x2e83, PDF_CMAP_SINGLE, 286 },
	{ 0x2e89, 0x2e89, PDF_CMAP_SINGLE, 565 },
	{ 0x2e8f, 0x2e8f, PDF_CMAP_SINGLE, 1283 },
	{ 0x2e95, 0x2e96, PDF_CMAP_TABLE, 216 },
	{ 0x2e98, 0x2e98, PDF_CMAP_SINGLE, 1699 },
	{ 0x2ea1, 0x2ea3, PDF_CMAP_TABLE, 218 },
	{ 0x2ea5, 0x2ea5, PDF_CMAP_SINGLE, 2750 },
	{ 0x2ea8, 0x2ea8, PDF_CMAP_SINGLE, 2787 },
	{ 0x2ead, 0x2ead, PDF_CMAP_SINGLE, 3317 },
	{ 0x2eb1, 0x2eb2, PDF_CMAP_TABLE, 221 },
	{ 0x2ec2, 0x2ec2, PDF_CMAP_SINGLE, 4421 },
	{ 0x2ecd, 0x2ecd, PDF_CMAP_SINGLE, 4892 },
	{ 0x2ed2, 0x2ed2, PDF_CMAP_SINGLE, 5341 },
	{ 0x2ed6, 0x2ed6, PDF_CMAP_SINGLE, 5378 },
	{ 0x2f01, 0x2f01, PDF_CMAP_SINGLE, 275 },
	{ 0x2f39, 0x2f39, PDF_CMAP_SINGLE, 1481 },
	{ 0x2f67, 0x2f67, PDF_CMAP_SINGLE, 3030 },
	{ 0x2f71, 0x2f71, PDF_CMAP_SINGLE, 3356 },
	{ 0x2fa1, 0x2fa1, PDF_CMAP_SINGLE, 4891 },
	{ 0x4e02, 0x4e02, PDF_CMAP_SINGLE, 267 },
	{ 0x4e04, 0x4e05, PDF_CMAP_RANGE, 268 },
	{ 0x4e0c, 0x4e0c, PDF_CMAP_SINGLE, 270 },
	{ 0x4e12, 0x4e12, PDF_CMAP_SINGLE, 271 },
	{ 0x4e1f, 0x4e1f, PDF_CMAP_SINGLE, 272 },
	{ 0x4e23, 0x4e24, PDF_CMAP_RANGE, 273 },
	{ 0x4e28, 0x4e28, PDF_CMAP_SINGLE, 275 },
	{ 0x4e2b, 0x4e2b, PDF_CMAP_SINGLE, 276 },
	{ 0x4e2e, 0x4e30, PDF_CMAP_RANGE, 277 },
	{ 0x4e35, 0x4e35, PDF_CMAP_SINGLE, 280 },
	{ 0x4e40, 0x4e41, PDF_CMAP_RANGE, 281 },
	{ 0x4e44, 0x4e44, PDF_CMAP_SINGLE, 283 },
	{ 0x4e47, 0x4e47, PDF_CMAP_SINGLE, 284 },
	{ 0x4e51, 0x4e51, PDF_CMAP_SINGLE, 285 },
	{ 0x4e5a, 0x4e5a, PDF_CMAP_SINGLE, 286 },
	{ 0x4e5c, 0x4e5c, PDF_CMAP_SINGLE, 287 },
	{ 0x4e63, 0x4e63, PDF_CMAP_SINGLE, 288 },
	{ 0x4e68, 0x4e69, PDF_CMAP_RANGE, 289 },
	{ 0x4e74, 0x4e75, PDF_CMAP_RANGE, 291 },
	{ 0x4e79, 0x4e79, PDF_CMAP_SINGLE, 293 },
	{ 0x4e7f, 0x4e7f, PDF_CMAP_SINGLE, 294 },
	{ 0x4e8d, 0x4e8d, PDF_CMAP_SINGLE, 295 },
	{ 0x4e96, 0x4e97, PDF_CMAP_RANGE, 296 },
	{ 0x4e9d, 0x4e9d, PDF_CMAP_SINGLE, 298 },
	{ 0x4eaf, 0x4eaf, PDF_CMAP_SINGLE, 299 },
	{ 0x4eb9, 0x4eb9, PDF_CMAP_SINGLE, 300 },
	{ 0x4ec3, 0x4ec3, PDF_CMAP_SINGLE, 301 },
	{ 0x4ed0, 0x4ed0, PDF_CMAP_SINGLE, 302 },
	{ 0x4eda, 0x4edb, PDF_CMAP_RANGE, 303 },
	{ 0x4ee0, 0x4ee2, PDF_CMAP_RANGE, 305 },
	{ 0x4ee8, 0x4ee8, PDF_CMAP_SINGLE, 308 },
	{ 0x4eef, 0x4eef, PDF_CMAP_SINGLE, 309 },
	{ 0x4ef1, 0x4ef1, PDF_CMAP_SINGLE, 310 },
	{ 0x4ef3, 0x4ef3, PDF_CMAP_SINGLE, 311 },
	{ 0x4ef5, 0x4ef5, PDF_CMAP_SINGLE, 312 },
	{ 0x4efd, 0x4f00, PDF_CMAP_RANGE, 313 },
	{ 0x4f02, 0x4f03, PDF_CMAP_RANGE, 317 },
	{ 0x4f08, 0x4f08, PDF_CMAP_SINGLE, 319 },
	{ 0x4f0b, 0x4f0c, PDF_CMAP_RANGE, 320 },
	{ 0x4f12, 0x4f12, PDF_CMAP_SINGLE, 322 },
	{ 0x4f15, 0x4f17, PDF_CMAP_RANGE, 323 },
	{ 0x4f19, 0x4f19, PDF_CMAP_SINGLE, 326 },
	{ 0x4f2e, 0x4f2e, PDF_CMAP_SINGLE, 327 },
	{ 0x4f31, 0x4f31, PDF_CMAP_SINGLE, 328 },
	{ 0x4f33, 0x4f33, PDF_CMAP_SINGLE, 330 },
	{ 0x4f35, 0x4f35, PDF_CMAP_SINGLE, 331 },
	{ 0x4f37, 0x4f37, PDF_CMAP_SINGLE, 332 },
	{ 0x4f39, 0x4f39, PDF_CMAP_SINGLE, 333 },
	{ 0x4f3b, 0x4f3b, PDF_CMAP_SINGLE, 334 },
	{ 0x4f3e, 0x4f3e, PDF_CMAP_SINGLE, 335 },
	{ 0x4f40, 0x4f40, PDF_CMAP_SINGLE, 336 },
	{ 0x4f42, 0x4f42, PDF_CMAP_SINGLE, 337 },
	{ 0x4f48, 0x4f49, PDF_CMAP_RANGE, 338 },
	{ 0x4f4b, 0x4f4c, PDF_CMAP_RANGE, 340 },
	{ 0x4f52, 0x4f52, PDF_CMAP_SINGLE, 342 },
	{ 0x4f54, 0x4f54, PDF_CMAP_SINGLE, 343 },
	{ 0x4f56, 0x4f56, PDF_CMAP_SINGLE, 344 },
	{ 0x4f58, 0x4f58, PDF_CMAP_SINGLE, 345 },
	{ 0x4f5f, 0x4f60, PDF_CMAP_TABLE, 223 },
	{ 0x4f63, 0x4f63, PDF_CMAP_SINGLE, 347 },
	{ 0x4f6a, 0x4f6a, PDF_CMAP_SINGLE, 348 },
	{ 0x4f6c, 0x4f6c, PDF_CMAP_SINGLE, 349 },
	{ 0x4f6e, 0x4f6e, PDF_CMAP_SINGLE, 350 },
	{ 0x4f71, 0x4f71, PDF_CMAP_SINGLE, 351 },
	{ 0x4f77, 0x4f7a, PDF_CMAP_RANGE, 352 },
	{ 0x4f7d, 0x4f7e, PDF_CMAP_RANGE, 356 },
	{ 0x4f81, 0x4f82, PDF_CMAP_RANGE, 358 },
	{ 0x4f84, 0x4f85, PDF_CMAP_RANGE, 360 },
	{ 0x4f89, 0x4f8a, PDF_CMAP_RANGE, 362 },
	{ 0x4f8c, 0x4f8c, PDF_CMAP_SINGLE, 364 },
	{ 0x4f8e, 0x4f8e, PDF_CMAP_SINGLE, 365 },
	{ 0x4f90, 0x4f90, PDF_CMAP_SINGLE, 366 },
	{ 0x4f92, 0x4f94, PDF_CMAP_RANGE, 367 },
	{ 0x4f97, 0x4f97, PDF_CMAP_SINGLE, 370 },
	{ 0x4f99, 0x4f9a, PDF_CMAP_RANGE, 371 },
	{ 0x4f9e, 0x4f9f, PDF_CMAP_RANGE, 373 },
	{ 0x4fb2, 0x4fb2, PDF_CMAP_SINGLE, 375 },
	{ 0x4fb7, 0x4fb7, PDF_CMAP_SINGLE, 376 },
	{ 0x4fb9, 0x4fb9, PDF_CMAP_SINGLE, 377 },
	{ 0x4fbb, 0x4fbe, PDF_CMAP_RANGE, 378 },
	{ 0x4fc0, 0x4fc1, PDF_CMAP_RANGE, 382 },
	{ 0x4fc5, 0x4fc6, PDF_CMAP_RANGE, 384 },
	{ 0x4fc8, 0x4fc9, PDF_CMAP_RANGE, 386 },
	{ 0x4fcb, 0x4fcd, PDF_CMAP_RANGE, 388 },
	{ 0x4fcf, 0x4fcf, PDF_CMAP_SINGLE, 391 },
	{ 0x4fd2, 0x4fd2, PDF_CMAP_SINGLE, 392 },
	{ 0x4fdc, 0x4fdc, PDF_CMAP_SINGLE, 393 },
	{ 0x4fe0, 0x4fe0, PDF_CMAP_SINGLE, 394 },
	{ 0x4fe2, 0x4fe2, PDF_CMAP_SINGLE, 395 },
	{ 0x4ff0, 0x4ff0, PDF_CMAP_SINGLE, 396 },
	{ 0x4ff2, 0x4ff2, PDF_CMAP_SINGLE, 397 },
	{ 0x4ffc, 0x4ffd, PDF_CMAP_RANGE, 398 },
	{ 0x4fff, 0x5001, PDF_CMAP_RANGE, 400 },
	{ 0x5004, 0x5004, PDF_CMAP_SINGLE, 403 },
	{ 0x5007, 0x5007, PDF_CMAP_SINGLE, 404 },
	{ 0x500a, 0x500a, PDF_CMAP_SINGLE, 405 },
	{ 0x500c, 0x500c, PDF_CMAP_SINGLE, 406 },
	{ 0x500e, 0x500e, PDF_CMAP_SINGLE, 407 },
	{ 0x5010, 0x5010, PDF_CMAP_SINGLE, 408 },
	{ 0x5013, 0x5013, PDF_CMAP_SINGLE, 409 },
	{ 0x5017, 0x5018, PDF_CMAP_RANGE, 410 },
	{ 0x501b, 0x501e, PDF_CMAP_RANGE, 412 },
	{ 0x5022, 0x5022, PDF_CMAP_SINGLE, 416 },
	{ 0x5027, 0x5027, PDF_CMAP_SINGLE, 417 },
	{ 0x502e, 0x502e, PDF_CMAP_SINGLE, 418 },
	{ 0x5030, 0x5030, PDF_CMAP_SINGLE, 419 },
	{ 0x5032, 0x5033, PDF_CMAP_RANGE, 420 },
	{ 0x5035, 0x5035, PDF_CMAP_SINGLE, 422 },
	{ 0x503b, 0x503b, PDF_CMAP_SINGLE, 446 },
	{ 0x5040, 0x5042, PDF_CMAP_RANGE, 423 },
	{ 0x5045, 0x5046, PDF_CMAP_RANGE, 426 },
	{ 0x504a, 0x504a, PDF_CMAP_SINGLE, 428 },
	{ 0x504c, 0x504c, PDF_CMAP_SINGLE, 429 },
	{ 0x504e, 0x504e, PDF_CMAP_SINGLE, 430 },
	{ 0x5051, 0x5053, PDF_CMAP_RANGE, 431 },
	{ 0x5057, 0x5057, PDF_CMAP_SINGLE, 434 },
	{ 0x5059, 0x5059, PDF_CMAP_SINGLE, 435 },
	{ 0x505f, 0x5060, PDF_CMAP_RANGE, 436 },
	{ 0x5062, 0x5063, PDF_CMAP_RANGE, 438 },
	{ 0x5066, 0x5067, PDF_CMAP_RANGE, 440 },
	{ 0x506a, 0x506a, PDF_CMAP_SINGLE, 442 },
	{ 0x506d, 0x506d, PDF_CMAP_SINGLE, 443 },
	{ 0x5070, 0x5071, PDF_CMAP_RANGE, 444 },
	{ 0x5081, 0x5081, PDF_CMAP_SINGLE, 447 },
	{ 0x5083, 0x5084, PDF_CMAP_RANGE, 448 },
	{ 0x5086, 0x5086, PDF_CMAP_SINGLE, 450 },
	{ 0x508a, 0x508a, PDF_CMAP_SINGLE, 451 },
	{ 0x508e, 0x5090, PDF_CMAP_RANGE, 452 },
	{ 0x5092, 0x5094, PDF_CMAP_RANGE, 455 },
	{ 0x5096, 0x5096, PDF_CMAP_SINGLE, 458 },
	{ 0x509b, 0x509c, PDF_CMAP_RANGE, 459 },
	{ 0x509e, 0x50a2, PDF_CMAP_RANGE, 461 },
	{ 0x50aa, 0x50aa, PDF_CMAP_SINGLE, 466 },
	{ 0x50af, 0x50b0, PDF_CMAP_RANGE, 467 },
	{ 0x50b9, 0x50ba, PDF_CMAP_RANGE, 469 },
	{ 0x50bd, 0x50bd, PDF_CMAP_SINGLE, 471 },
	{ 0x50c0, 0x50c0, PDF_CMAP_SINGLE, 472 },
	{ 0x50c3, 0x50c4, PDF_CMAP_RANGE, 473 },
	{ 0x50c7, 0x50c7, PDF_CMAP_SINGLE, 475 },
	{ 0x50cc, 0x50cc, PDF_CMAP_SINGLE, 476 },
	{ 0x50ce, 0x50ce, PDF_CMAP_SINGLE, 477 },
	{ 0x50d0, 0x50d0, PDF_CMAP_SINGLE, 478 },
	{ 0x50d3, 0x50d4, PDF_CMAP_RANGE, 479 },
	{ 0x50d8, 0x50d8, PDF_CMAP_SINGLE, 481 },
	{ 0x50dc, 0x50dd, PDF_CMAP_RANGE, 482 },
	{ 0x50df, 0x50df, PDF_CMAP_SINGLE, 484 },
	{ 0x50e2, 0x50e2, PDF_CMAP_SINGLE, 485 },
	{ 0x50e4, 0x50e4, PDF_CMAP_SINGLE, 486 },
	{ 0x50e6, 0x50e6, PDF_CMAP_SINGLE, 487 },
	{ 0x50e8, 0x50e9, PDF_CMAP_RANGE, 488 },
	{ 0x50ef, 0x50ef, PDF_CMAP_SINGLE, 490 },
	{ 0x50f1, 0x50f2, PDF_CMAP_TABLE, 225 },
	{ 0x50f6, 0x50f6, PDF_CMAP_SINGLE, 492 },
	{ 0x50fa, 0x50fa, PDF_CMAP_SINGLE, 493 },
	{ 0x50fe, 0x50fe, PDF_CMAP_SINGLE, 494 },
	{ 0x5103, 0x5103, PDF_CMAP_SINGLE, 495 },
	{ 0x5106, 0x5108, PDF_CMAP_RANGE, 496 },
	{ 0x510b, 0x510e, PDF_CMAP_RANGE, 499 },
	{ 0x5110, 0x5110, PDF_CMAP_SINGLE, 504 },
	{ 0x5117, 0x5117, PDF_CMAP_SINGLE, 505 },
	{ 0x5119, 0x5119, PDF_CMAP_SINGLE, 506 },
	{ 0x511b, 0x511e, PDF_CMAP_RANGE, 507 },
	{ 0x5123, 0x5123, PDF_CMAP_SINGLE, 511 },
	{ 0x5127, 0x5128, PDF_CMAP_RANGE, 512 },
	{ 0x512c, 0x512d, PDF_CMAP_RANGE, 514 },
	{ 0x512f, 0x512f, PDF_CMAP_SINGLE, 516 },
	{ 0x5131, 0x5131, PDF_CMAP_SINGLE, 517 },
	{ 0x5133, 0x5135, PDF_CMAP_RANGE, 518 },
	{ 0x5138, 0x5139, PDF_CMAP_RANGE, 521 },
	{ 0x5142, 0x5142, PDF_CMAP_SINGLE, 523 },
	{ 0x514a, 0x514a, PDF_CMAP_SINGLE, 524 },
	{ 0x514f, 0x514f, PDF_CMAP_SINGLE, 525 },
	{ 0x5153, 0x5153, PDF_CMAP_SINGLE, 526 },
	{ 0x5155, 0x5155, PDF_CMAP_SINGLE, 527 },
	{ 0x5157, 0x5158, PDF_CMAP_RANGE, 528 },
	{ 0x515f, 0x515f, PDF_CMAP_SINGLE, 530 },
	{ 0x5164, 0x5164, PDF_CMAP_SINGLE, 531 },
	{ 0x5166, 0x5166, PDF_CMAP_SINGLE, 532 },
	{ 0x517e, 0x517e, PDF_CMAP_SINGLE, 533 },
	{ 0x5183, 0x5184, PDF_CMAP_RANGE, 534 },
	{ 0x518b, 0x518b, PDF_CMAP_SINGLE, 536 },
	{ 0x518e, 0x518e, PDF_CMAP_SINGLE, 537 },
	{ 0x5198, 0x5198, PDF_CMAP_SINGLE, 538 },
	{ 0x519d, 0x519d, PDF_CMAP_SINGLE, 539 },
	{ 0x51a1, 0x51a1, PDF_CMAP_SINGLE, 540 },
	{ 0x51a3, 0x51a3, PDF_CMAP_SINGLE, 541 },
	{ 0x51ad, 0x51ad, PDF_CMAP_SINGLE, 542 },
	{ 0x51b8, 0x51b8, PDF_CMAP_SINGLE, 543 },
	{ 0x51ba, 0x51ba, PDF_CMAP_SINGLE, 544 },
	{ 0x51bc, 0x51bc, PDF_CMAP_SINGLE, 545 },
	{ 0x51be, 0x51bf, PDF_CMAP_RANGE, 546 },
	{ 0x51c2, 0x51c2, PDF_CMAP_SINGLE, 548 },
	{ 0x51c8, 0x51c8, PDF_CMAP_SINGLE, 549 },
	{ 0x51cf, 0x51cf, PDF_CMAP_SINGLE, 550 },
	{ 0x51d1, 0x51d3, PDF_CMAP_RANGE, 551 },
	{ 0x51d5, 0x51d5, PDF_CMAP_SINGLE, 554 },
	{ 0x51d8, 0x51d8, PDF_CMAP_SINGLE, 555 },
	{ 0x51de, 0x51de, PDF_CMAP_SINGLE, 556 },
	{ 0x51e2, 0x51e2, PDF_CMAP_SINGLE, 557 },
	{ 0x51e5, 0x51e5, PDF_CMAP_SINGLE, 558 },
	{ 0x51ee, 0x51ee, PDF_CMAP_SINGLE, 559 },
	{ 0x51f2, 0x51f4, PDF_CMAP_RANGE, 560 },
	{ 0x51f7, 0x51f7, PDF_CMAP_SINGLE, 563 },
	{ 0x5201, 0x5202, PDF_CMAP_RANGE, 564 },
	{ 0x5205, 0x5205, PDF_CMAP_SINGLE, 566 },
	{ 0x5212, 0x5213, PDF_CMAP_RANGE, 567 },
	{ 0x5215, 0x5216, PDF_CMAP_RANGE, 569 },
	{ 0x5218, 0x5218, PDF_CMAP_SINGLE, 571 },
	{ 0x5222, 0x5222, PDF_CMAP_SINGLE, 572 },
	{ 0x5228, 0x5228, PDF_CMAP_SINGLE, 573 },
	{ 0x5231, 0x5232, PDF_CMAP_RANGE, 574 },
	{ 0x5235, 0x5235, PDF_CMAP_SINGLE, 576 },
	{ 0x523c, 0x523c, PDF_CMAP_SINGLE, 577 },
	{ 0x5245, 0x5245, PDF_CMAP_SINGLE, 578 },
	{ 0x5249, 0x5249, PDF_CMAP_SINGLE, 579 },
	{ 0x5255, 0x5255, PDF_CMAP_SINGLE, 580 },
	{ 0x5257, 0x5258, PDF_CMAP_RANGE, 581 },
	{ 0x525a, 0x525a, PDF_CMAP_SINGLE, 583 },
	{ 0x525c, 0x525c, PDF_CMAP_SINGLE, 584 },
	{ 0x525f, 0x5261, PDF_CMAP_RANGE, 585 },
	{ 0x5266, 0x5266, PDF_CMAP_SINGLE, 588 },
	{ 0x526e, 0x526e, PDF_CMAP_SINGLE, 589 },
	{ 0x5277, 0x5279, PDF_CMAP_RANGE, 590 },
	{ 0x5280, 0x5280, PDF_CMAP_SINGLE, 593 },
	{ 0x5282, 0x5282, PDF_CMAP_SINGLE, 594 },
	{ 0x5285, 0x5285, PDF_CMAP_SINGLE, 595 },
	{ 0x528a, 0x528a, PDF_CMAP_SINGLE, 596 },
	{ 0x528c, 0x528c, PDF_CMAP_SINGLE, 597 },
	{ 0x5293, 0x5293, PDF_CMAP_SINGLE, 598 },
	{ 0x5295, 0x5298, PDF_CMAP_RANGE, 599 },
	{ 0x529a, 0x529a, PDF_CMAP_SINGLE, 603 },
	{ 0x529c, 0x529c, PDF_CMAP_SINGLE, 604 },
	{ 0x52a4, 0x52a7, PDF_CMAP_RANGE, 605 },
	{ 0x52af, 0x52b0, PDF_CMAP_RANGE, 609 },
	{ 0x52b6, 0x52b8, PDF_CMAP_RANGE, 611 },
	{ 0x52ba, 0x52bb, PDF_CMAP_RANGE, 614 },
	{ 0x52bd, 0x52bd, PDF_CMAP_SINGLE, 616 },
	{ 0x52c0, 0x52c0, PDF_CMAP_SINGLE, 617 },
	{ 0x52c4, 0x52c4, PDF_CMAP_SINGLE, 618 },
	{ 0x52c6, 0x52c6, PDF_CMAP_SINGLE, 619 },
	{ 0x52c8, 0x52c8, PDF_CMAP_SINGLE, 620 },
	{ 0x52cc, 0x52cc, PDF_CMAP_SINGLE, 621 },
	{ 0x52cf, 0x52cf, PDF_CMAP_SINGLE, 622 },
	{ 0x52d1, 0x52d1, PDF_CMAP_SINGLE, 623 },
	{ 0x52d4, 0x52d4, PDF_CMAP_SINGLE, 624 },
	{ 0x52d6, 0x52d6, PDF_CMAP_SINGLE, 625 },
	{ 0x52db, 0x52dc, PDF_CMAP_RANGE, 626 },
	{ 0x52e1, 0x52e1, PDF_CMAP_SINGLE, 628 },
	{ 0x52e5, 0x52e5, PDF_CMAP_SINGLE, 629 },
	{ 0x52e8, 0x52ea, PDF_CMAP_RANGE, 630 },
	{ 0x52ec, 0x52ec, PDF_CMAP_SINGLE, 633 },
	{ 0x52f0, 0x52f1, PDF_CMAP_RANGE, 634 },
	{ 0x52f4, 0x52f4, PDF_CMAP_SINGLE, 636 },
	{ 0x52f6, 0x52f7, PDF_CMAP_RANGE, 637 },
	{ 0x5300, 0x5300, PDF_CMAP_SINGLE, 639 },
	{ 0x5303, 0x5303, PDF_CMAP_SINGLE, 640 },
	{ 0x530a, 0x530c, PDF_CMAP_RANGE, 641 },
	{ 0x5311, 0x5311, PDF_CMAP_SINGLE, 644 },
	{ 0x5313, 0x5313, PDF_CMAP_SINGLE, 645 },
	{ 0x5318, 0x5318, PDF_CMAP_SINGLE, 646 },
	{ 0x531b, 0x531c, PDF_CMAP_RANGE, 647 },
	{ 0x531e, 0x531f, PDF_CMAP_RANGE, 649 },
	{ 0x5325, 0x5325, PDF_CMAP_SINGLE, 651 },
	{ 0x5327, 0x5329, PDF_CMAP_RANGE, 652 },
	{ 0x532b, 0x532d, PDF_CMAP_RANGE, 655 },
	{ 0x5330, 0x5330, PDF_CMAP_SINGLE, 658 },
	{ 0x5332, 0x5332, PDF_CMAP_SINGLE, 659 },
	{ 0x5335, 0x5335, PDF_CMAP_SINGLE, 660 },
	{ 0x533c, 0x533e, PDF_CMAP_RANGE, 661 },
	{ 0x5342, 0x5342, PDF_CMAP_SINGLE, 664 },
	{ 0x534b, 0x534c, PDF_CMAP_TABLE, 227 },
	{ 0x5359, 0x5359, PDF_CMAP_SINGLE, 667 },
	{ 0x535b, 0x535b, PDF_CMAP_SINGLE, 668 },
	{ 0x5361, 0x5361, PDF_CMAP_SINGLE, 669 },
	{ 0x5363, 0x5363, PDF_CMAP_SINGLE, 670 },
	{ 0x5365, 0x5365, PDF_CMAP_SINGLE, 671 },
	{ 0x536c, 0x536d, PDF_CMAP_RANGE, 672 },
	{ 0x5372, 0x5372, PDF_CMAP_SINGLE, 674 },
	{ 0x5379, 0x5379, PDF_CMAP_SINGLE, 675 },
	{ 0x537e, 0x537e, PDF_CMAP_SINGLE, 676 },
	{ 0x5383, 0x5383, PDF_CMAP_SINGLE, 677 },
	{ 0x5387, 0x5388, PDF_CMAP_RANGE, 678 },
	{ 0x538e, 0x538e, PDF_CMAP_SINGLE, 680 },
	{ 0x5393, 0x5394, PDF_CMAP_RANGE, 681 },
	{ 0x5399, 0x5399, PDF_CMAP_SINGLE, 683 },
	{ 0x539d, 0x539d, PDF_CMAP_SINGLE, 684 },
	{ 0x53a1, 0x53a1, PDF_CMAP_SINGLE, 685 },
	{ 0x53a4, 0x53a4, PDF_CMAP_SINGLE, 686 },
	{ 0x53aa, 0x53ab, PDF_CMAP_RANGE, 687 },
	{ 0x53af, 0x53af, PDF_CMAP_SINGLE, 689 },
	{ 0x53b2, 0x53b2, PDF_CMAP_SINGLE, 690 },
	{ 0x53b4, 0x53b5, PDF_CMAP_RANGE, 691 },
	{ 0x53b7, 0x53b8, PDF_CMAP_RANGE, 693 },
	{ 0x53ba, 0x53ba, PDF_CMAP_SINGLE, 695 },
	{ 0x53bd, 0x53bd, PDF_CMAP_SINGLE, 696 },
	{ 0x53c0, 0x53c0, PDF_CMAP_SINGLE, 697 },
	{ 0x53c5, 0x53c5, PDF_CMAP_SINGLE, 698 },
	{ 0x53cf, 0x53cf, PDF_CMAP_SINGLE, 699 },
	{ 0x53d2, 0x53d3, PDF_CMAP_RANGE, 700 },
	{ 0x53d5, 0x53d5, PDF_CMAP_SINGLE, 702 },
	{ 0x53da, 0x53da, PDF_CMAP_SINGLE, 703 },
	{ 0x53dd, 0x53de, PDF_CMAP_RANGE, 704 },
	{ 0x53e0, 0x53e0, PDF_CMAP_SINGLE, 706 },
	{ 0x53e6, 0x53e7, PDF_CMAP_RANGE, 707 },
	{ 0x53f5, 0x53f5, PDF_CMAP_SINGLE, 709 },
	{ 0x5402, 0x5402, PDF_CMAP_SINGLE, 710 },
	{ 0x5413, 0x5413, PDF_CMAP_SINGLE, 711 },
	{ 0x541a, 0x541a, PDF_CMAP_SINGLE, 712 },
	{ 0x5421, 0x5421, PDF_CMAP_SINGLE, 713 },
	{ 0x5427, 0x5428, PDF_CMAP_RANGE, 714 },
	{ 0x542a, 0x542a, PDF_CMAP_SINGLE, 716 },
	{ 0x542f, 0x542f, PDF_CMAP_SINGLE, 717 },
	{ 0x5431, 0x5431, PDF_CMAP_SINGLE, 718 },
	{ 0x5434, 0x5435, PDF_CMAP_RANGE, 719 },
	{ 0x5443, 0x5444, PDF_CMAP_RANGE, 721 },
	{ 0x5447, 0x5447, PDF_CMAP_SINGLE, 723 },
	{ 0x544d, 0x544d, PDF_CMAP_SINGLE, 724 },
	{ 0x544f, 0x544f, PDF_CMAP_SINGLE, 725 },
	{ 0x545e, 0x545e, PDF_CMAP_SINGLE, 726 },
	{ 0x5462, 0x5462, PDF_CMAP_SINGLE, 727 },
	{ 0x5464, 0x5464, PDF_CMAP_SINGLE, 728 },
	{ 0x5466, 0x5467, PDF_CMAP_RANGE, 729 },
	{ 0x5469, 0x5469, PDF_CMAP_SINGLE, 731 },
	{ 0x546b, 0x546b, PDF_CMAP_SINGLE, 732 },
	{ 0x546d, 0x546e, PDF_CMAP_RANGE, 733 },
	{ 0x5474, 0x5474, PDF_CMAP_SINGLE, 735 },
	{ 0x547f, 0x547f, PDF_CMAP_SINGLE, 736 },
	{ 0x5481, 0x5481, PDF_CMAP_SINGLE, 737 },
	{ 0x5483, 0x5483, PDF_CMAP_SINGLE, 738 },
	{ 0x5485, 0x5485, PDF_CMAP_SINGLE, 739 },
	{ 0x5488, 0x5489, PDF_CMAP_RANGE, 740 },
	{ 0x548d, 0x548d, PDF_CMAP_SINGLE, 742 },
	{ 0x5491, 0x5491, PDF_CMAP_SINGLE, 743 },
	{ 0x5495, 0x5496, PDF_CMAP_RANGE, 744 },
	{ 0x549c, 0x549c, PDF_CMAP_SINGLE, 746 },
	{ 0x549f, 0x549f, PDF_CMAP_SINGLE, 747 },
	{ 0x54a1, 0x54a1, PDF_CMAP_SINGLE, 748 },
	{ 0x54a6, 0x54a7, PDF_CMAP_RANGE, 749 },
	{ 0x54a9, 0x54aa, PDF_CMAP_RANGE, 751 },
	{ 0x54ad, 0x54ae, PDF_CMAP_RANGE, 753 },
	{ 0x54b1, 0x54b1, PDF_CMAP_SINGLE, 755 },
	{ 0x54b7, 0x54b7, PDF_CMAP_SINGLE, 756 },
	{ 0x54b9, 0x54bb, PDF_CMAP_RANGE, 757 },
	{ 0x54bf, 0x54bf, PDF_CMAP_SINGLE, 760 },
	{ 0x54c6, 0x54c6, PDF_CMAP_SINGLE, 761 },
	{ 0x54ca, 0x54ca, PDF_CMAP_SINGLE, 762 },
	{ 0x54cd, 0x54ce, PDF_CMAP_RANGE, 763 },
	{ 0x54e0, 0x54e0, PDF_CMAP_SINGLE, 765 },
	{ 0x54ea, 0x54ea, PDF_CMAP_SINGLE, 766 },
	{ 0x54ec, 0x54ec, PDF_CMAP_SINGLE, 767 },
	{ 0x54ef, 0x54ef, PDF_CMAP_SINGLE, 768 },
	{ 0x54f6, 0x54f6, PDF_CMAP_SINGLE, 769 },
	{ 0x54fc, 0x54fc, PDF_CMAP_SINGLE, 770 },
	{ 0x54fe, 0x5501, PDF_CMAP_RANGE, 771 },
	{ 0x5505, 0x5505, PDF_CMAP_SINGLE, 775 },
	{ 0x5508, 0x5509, PDF_CMAP_RANGE, 776 },
	{ 0x550c, 0x550e, PDF_CMAP_RANGE, 778 },
	{ 0x5515, 0x5515, PDF_CMAP_SINGLE, 781 },
	{ 0x552a, 0x552b, PDF_CMAP_RANGE, 782 },
	{ 0x5532, 0x5532, PDF_CMAP_SINGLE, 784 },
	{ 0x5535, 0x5536, PDF_CMAP_RANGE, 785 },
	{ 0x553b, 0x553d, PDF_CMAP_RANGE, 787 },
	{ 0x5541, 0x5541, PDF_CMAP_SINGLE, 790 },
	{ 0x5547, 0x5547, PDF_CMAP_SINGLE, 791 },
	{ 0x5549, 0x554a, PDF_CMAP_RANGE, 792 },
	{ 0x554d, 0x554d, PDF_CMAP_SINGLE, 794 },
	{ 0x5550, 0x5551, PDF_CMAP_RANGE, 795 },
	{ 0x5558, 0x5558, PDF_CMAP_SINGLE, 797 },
	{ 0x555a, 0x555b, PDF_CMAP_RANGE, 798 },
	{ 0x555e, 0x555e, PDF_CMAP_SINGLE, 800 },
	{ 0x5560, 0x5561, PDF_CMAP_RANGE, 801 },
	{ 0x5564, 0x5564, PDF_CMAP_SINGLE, 803 },
	{ 0x5566, 0x5566, PDF_CMAP_SINGLE, 804 },
	{ 0x557f, 0x557f, PDF_CMAP_SINGLE, 805 },
	{ 0x5581, 0x5582, PDF_CMAP_RANGE, 806 },
	{ 0x5586, 0x5586, PDF_CMAP_SINGLE, 808 },
	{ 0x5588, 0x5588, PDF_CMAP_SINGLE, 809 },
	{ 0x558e, 0x558f, PDF_CMAP_RANGE, 810 },
	{ 0x5591, 0x5594, PDF_CMAP_RANGE, 812 },
	{ 0x5597, 0x5597, PDF_CMAP_SINGLE, 816 },
	{ 0x55a3, 0x55a4, PDF_CMAP_RANGE, 817 },
	{ 0x55ad, 0x55ad, PDF_CMAP_SINGLE, 819 },
	{ 0x55b2, 0x55b2, PDF_CMAP_SINGLE, 820 },
	{ 0x55bf, 0x55bf, PDF_CMAP_SINGLE, 821 },
	{ 0x55c1, 0x55c1, PDF_CMAP_SINGLE, 822 },
	{ 0x55c3, 0x55c3, PDF_CMAP_SINGLE, 823 },
	{ 0x55c6, 0x55c6, PDF_CMAP_SINGLE, 824 },
	{ 0x55c9, 0x55c9, PDF_CMAP_SINGLE, 825 },
	{ 0x55cb, 0x55cc, PDF_CMAP_RANGE, 826 },
	{ 0x55ce, 0x55ce, PDF_CMAP_SINGLE, 828 },
	{ 0x55d1, 0x55d3, PDF_CMAP_RANGE, 829 },
	{ 0x55d7, 0x55d8, PDF_CMAP_RANGE, 832 },
	{ 0x55db, 0x55db, PDF_CMAP_SINGLE, 834 },
	{ 0x55de, 0x55de, PDF_CMAP_SINGLE, 835 },
	{ 0x55e2, 0x55e2, PDF_CMAP_SINGLE, 836 },
	{ 0x55e9, 0x55e9, PDF_CMAP_SINGLE, 837 },
	{ 0x55f6, 0x55f6, PDF_CMAP_SINGLE, 838 },
	{ 0x55ff, 0x55ff, PDF_CMAP_SINGLE, 839 },
	{ 0x5605, 0x5605, PDF_CMAP_SINGLE, 840 },
	{ 0x5608, 0x5608, PDF_CMAP_SINGLE, 841 },
	{ 0x560a, 0x560a, PDF_CMAP_SINGLE, 842 },
	{ 0x560d, 0x5612, PDF_CMAP_RANGE, 843 },
	{ 0x5619, 0x5619, PDF_CMAP_SINGLE, 849 },
	{ 0x562c, 0x562c, PDF_CMAP_SINGLE, 850 },
	{ 0x5630, 0x5630, PDF_CMAP_SINGLE, 851 },
	{ 0x5633, 0x5633, PDF_CMAP_SINGLE, 852 },
	{ 0x5635, 0x5635, PDF_CMAP_SINGLE, 853 },
	{ 0x5637, 0x5637, PDF_CMAP_SINGLE, 854 },
	{ 0x5639, 0x5639, PDF_CMAP_SINGLE, 855 },
	{ 0x563b, 0x563d, PDF_CMAP_RANGE, 856 },
	{ 0x563f, 0x5641, PDF_CMAP_RANGE, 859 },
	{ 0x5643, 0x5644, PDF_CMAP_RANGE, 862 },
	{ 0x5646, 0x5646, PDF_CMAP_SINGLE, 864 },
	{ 0x5649, 0x5649, PDF_CMAP_SINGLE, 865 },
	{ 0x564b, 0x564b, PDF_CMAP_SINGLE, 866 },
	{ 0x564d, 0x564d, PDF_CMAP_SINGLE, 867 },
	{ 0x564f, 0x564f, PDF_CMAP_SINGLE, 868 },
	{ 0x5654, 0x5654, PDF_CMAP_SINGLE, 869 },
	{ 0x565e, 0x565e, PDF_CMAP_SINGLE, 870 },
	{ 0x5660, 0x5663, PDF_CMAP_RANGE, 871 },
	{ 0x5666, 0x5666, PDF_CMAP_SINGLE, 875 },
	{ 0x5669, 0x5669, PDF_CMAP_SINGLE, 876 },
	{ 0x566d, 0x566d, PDF_CMAP_SINGLE, 877 },
	{ 0x566f, 0x566f, PDF_CMAP_SINGLE, 878 },
	{ 0x5671, 0x5672, PDF_CMAP_RANGE, 879 },
	{ 0x5675, 0x5675, PDF_CMAP_SINGLE, 881 },
	{ 0x5684, 0x5685, PDF_CMAP_RANGE, 882 },
	{ 0x5688, 0x5688, PDF_CMAP_SINGLE, 884 },
	{ 0x568b, 0x568c, PDF_CMAP_RANGE, 885 },
	{ 0x5695, 0x5695, PDF_CMAP_SINGLE, 887 },
	{ 0x5699, 0x569a, PDF_CMAP_RANGE, 888 },
	{ 0x569d, 0x569f, PDF_CMAP_RANGE, 890 },
	{ 0x56a6, 0x56a9, PDF_CMAP_RANGE, 893 },
	{ 0x56ab, 0x56ad, PDF_CMAP_RANGE, 897 },
	{ 0x56b1, 0x56b1, PDF_CMAP_SINGLE, 900 },
	{ 0x56b3, 0x56b3, PDF_CMAP_SINGLE, 901 },
	{ 0x56b7, 0x56b7, PDF_CMAP_SINGLE, 902 },
	{ 0x56be, 0x56be, PDF_CMAP_SINGLE, 903 },
	{ 0x56c5, 0x56c5, PDF_CMAP_SINGLE, 904 },
	{ 0x56c9, 0x56cb, PDF_CMAP_RANGE, 905 },
	{ 0x56cc, 0x56cd, PDF_CMAP_RANGE, 910 },
	{ 0x56cf, 0x56d0, PDF_CMAP_RANGE, 908 },
	{ 0x56d9, 0x56d9, PDF_CMAP_SINGLE, 912 },
	{ 0x56dc, 0x56dd, PDF_CMAP_RANGE, 913 },
	{ 0x56df, 0x56df, PDF_CMAP_SINGLE, 915 },
	{ 0x56e1, 0x56e1, PDF_CMAP_SINGLE, 916 },
	{ 0x56e4, 0x56e8, PDF_CMAP_RANGE, 917 },
	{ 0x56eb, 0x56eb, PDF_CMAP_SINGLE, 923 },
	{ 0x56ed, 0x56ed, PDF_CMAP_SINGLE, 924 },
	{ 0x56f1, 0x56f1, PDF_CMAP_SINGLE, 922 },
	{ 0x56f6, 0x56f7, PDF_CMAP_RANGE, 925 },
	{ 0x5701, 0x5702, PDF_CMAP_RANGE, 927 },
	{ 0x5707, 0x5707, PDF_CMAP_SINGLE, 929 },
	{ 0x570a, 0x570a, PDF_CMAP_SINGLE, 930 },
	{ 0x570c, 0x570c, PDF_CMAP_SINGLE, 931 },
	{ 0x5711, 0x5711, PDF_CMAP_SINGLE, 932 },
	{ 0x5715, 0x5715, PDF_CMAP_SINGLE, 933 },
	{ 0x571a, 0x571b, PDF_CMAP_RANGE, 934 },
	{ 0x571d, 0x571d, PDF_CMAP_SINGLE, 936 },
	{ 0x5720, 0x5720, PDF_CMAP_SINGLE, 937 },
	{ 0x5722, 0x5725, PDF_CMAP_RANGE, 938 },
	{ 0x5729, 0x572a, PDF_CMAP_RANGE, 942 },
	{ 0x572c, 0x572c, PDF_CMAP_SINGLE, 944 },
	{ 0x572e, 0x572f, PDF_CMAP_RANGE, 945 },
	{ 0x5733, 0x5734, PDF_CMAP_RANGE, 947 },
	{ 0x573d, 0x573f, PDF_CMAP_RANGE, 949 },
	{ 0x5745, 0x5746, PDF_CMAP_RANGE, 952 },
	{ 0x574c, 0x574d, PDF_CMAP_RANGE, 954 },
	{ 0x5752, 0x5752, PDF_CMAP_SINGLE, 956 },
	{ 0x5762, 0x5762, PDF_CMAP_SINGLE, 957 },
	{ 0x5765, 0x5765, PDF_CMAP_SINGLE, 958 },
	{ 0x5767, 0x5768, PDF_CMAP_RANGE, 959 },
	{ 0x576b, 0x576b, PDF_CMAP_SINGLE, 961 },
	{ 0x576d, 0x5771, PDF_CMAP_RANGE, 962 },
	{ 0x5773, 0x5775, PDF_CMAP_RANGE, 967 },
	{ 0x5777, 0x5777, PDF_CMAP_SINGLE, 970 },
	{ 0x5779, 0x577c, PDF_CMAP_RANGE, 971 },
	{ 0x577e, 0x577e, PDF_CMAP_SINGLE, 975 },
	{ 0x5781, 0x5781, PDF_CMAP_SINGLE, 976 },
	{ 0x5783, 0x5783, PDF_CMAP_SINGLE, 977 },
	{ 0x578c, 0x578c, PDF_CMAP_SINGLE, 978 },
	{ 0x5794, 0x5795, PDF_CMAP_TABLE, 229 },
	{ 0x5797, 0x5797, PDF_CMAP_SINGLE, 980 },
	{ 0x5799, 0x579a, PDF_CMAP_RANGE, 981 },
	{ 0x579c, 0x579f, PDF_CMAP_RANGE, 983 },
	{ 0x57a1, 0x57a1, PDF_CMAP_SINGLE, 987 },
	{ 0x57a7, 0x57a9, PDF_CMAP_RANGE, 989 },
	{ 0x57ac, 0x57ac, PDF_CMAP_SINGLE, 992 },
	{ 0x57b8, 0x57b8, PDF_CMAP_SINGLE, 993 },
	{ 0x57bd, 0x57bd, PDF_CMAP_SINGLE, 994 },
	{ 0x57c7, 0x57c8, PDF_CMAP_RANGE, 995 },
	{ 0x57cc, 0x57cc, PDF_CMAP_SINGLE, 997 },
	{ 0x57cf, 0x57cf, PDF_CMAP_SINGLE, 998 },
	{ 0x57d5, 0x57d5, PDF_CMAP_SINGLE, 999 },
	{ 0x57dd, 0x57de, PDF_CMAP_RANGE, 1000 },
	{ 0x57e1, 0x57e1, PDF_CMAP_SINGLE, 1018 },
	{ 0x57e4, 0x57e4, PDF_CMAP_SINGLE, 1002 },
	{ 0x57e6, 0x57e7, PDF_CMAP_RANGE, 1003 },
	{ 0x57e9, 0x57e9, PDF_CMAP_SINGLE, 1005 },
	{ 0x57ed, 0x57ed, PDF_CMAP_SINGLE, 1006 },
	{ 0x57f0, 0x57f0, PDF_CMAP_SINGLE, 1007 },
	{ 0x57f5, 0x57f6, PDF_CMAP_RANGE, 1008 },
	{ 0x57f8, 0x57f8, PDF_CMAP_SINGLE, 1010 },
	{ 0x57fd, 0x57ff, PDF_CMAP_RANGE, 1011 },
	{ 0x5803, 0x5804, PDF_CMAP_RANGE, 1014 },
	{ 0x5808, 0x5809, PDF_CMAP_RANGE, 1016 },
	{ 0x580c, 0x580d, PDF_CMAP_RANGE, 1019 },
	{ 0x581b, 0x581b, PDF_CMAP_SINGLE, 1021 },
	{ 0x581e, 0x5820, PDF_CMAP_RANGE, 1022 },
	{ 0x5826, 0x5827, PDF_CMAP_RANGE, 1025 },
	{ 0x582d, 0x582d, PDF_CMAP_SINGLE, 1027 },
	{ 0x5832, 0x5832, PDF_CMAP_SINGLE, 1028 },
	{ 0x5839, 0x5839, PDF_CMAP_SINGLE, 1029 },
	{ 0x583f, 0x583f, PDF_CMAP_SINGLE, 1030 },
	{ 0x5849, 0x5849, PDF_CMAP_SINGLE, 1031 },
	{ 0x584c, 0x584d, PDF_CMAP_RANGE, 1032 },
	{ 0x584f, 0x5850, PDF_CMAP_RANGE, 1034 },
	{ 0x5855, 0x5855, PDF_CMAP_SINGLE, 1036 },
	{ 0x585f, 0x585f, PDF_CMAP_SINGLE, 1037 },
	{ 0x5861, 0x5861, PDF_CMAP_SINGLE, 1038 },
	{ 0x5864, 0x5864, PDF_CMAP_SINGLE, 1039 },
	{ 0x5867, 0x5868, PDF_CMAP_RANGE, 1040 },
	{ 0x5878, 0x5878, PDF_CMAP_SINGLE, 1042 },
	{ 0x587c, 0x587c, PDF_CMAP_SINGLE, 1043 },
	{ 0x587f, 0x5881, PDF_CMAP_RANGE, 1044 },
	{ 0x5887, 0x588a, PDF_CMAP_RANGE, 1047 },
	{ 0x588c, 0x588d, PDF_CMAP_RANGE, 1051 },
	{ 0x588f, 0x5890, PDF_CMAP_RANGE, 1053 },
	{ 0x5894, 0x5894, PDF_CMAP_SINGLE, 1055 },
	{ 0x5896, 0x5896, PDF_CMAP_SINGLE, 1056 },
	{ 0x589d, 0x589d, PDF_CMAP_SINGLE, 1057 },
	{ 0x58a0, 0x58a2, PDF_CMAP_RANGE, 1058 },
	{ 0x58a6, 0x58a6, PDF_CMAP_SINGLE, 1061 },
	{ 0x58a9, 0x58a9, PDF_CMAP_SINGLE, 1062 },
	{ 0x58b1, 0x58b2, PDF_CMAP_RANGE, 1063 },
	{ 0x58bc, 0x58bc, PDF_CMAP_SINGLE, 1066 },
	{ 0x58c2, 0x58c2, PDF_CMAP_SINGLE, 1067 },
	{ 0x58c4, 0x58c4, PDF_CMAP_SINGLE, 1065 },
	{ 0x58c8, 0x58c8, PDF_CMAP_SINGLE, 1068 },
	{ 0x58cd, 0x58ce, PDF_CMAP_RANGE, 1069 },
	{ 0x58d0, 0x58d0, PDF_CMAP_SINGLE, 1071 },
	{ 0x58d2, 0x58d2, PDF_CMAP_SINGLE, 1072 },
	{ 0x58d4, 0x58d4, PDF_CMAP_SINGLE, 1073 },
	{ 0x58d6, 0x58d6, PDF_CMAP_SINGLE, 1074 },
	{ 0x58da, 0x58da, PDF_CMAP_SINGLE, 1075 },
	{ 0x58dd, 0x58dd, PDF_CMAP_SINGLE, 1076 },
	{ 0x58e1, 0x58e2, PDF_CMAP_RANGE, 1077 },
	{ 0x58e9, 0x58e9, PDF_CMAP_SINGLE, 1079 },
	{ 0x58f3, 0x58f3, PDF_CMAP_SINGLE, 1080 },
	{ 0x5905, 0x5906, PDF_CMAP_RANGE, 1081 },
	{ 0x590b, 0x590c, PDF_CMAP_RANGE, 1083 },
	{ 0x5912, 0x5914, PDF_CMAP_RANGE, 1085 },
	{ 0x591d, 0x591d, PDF_CMAP_SINGLE, 1089 },
	{ 0x5921, 0x5921, PDF_CMAP_SINGLE, 1090 },
	{ 0x5923, 0x5924, PDF_CMAP_RANGE, 1091 },
	{ 0x5928, 0x5928, PDF_CMAP_SINGLE, 1093 },
	{ 0x592f, 0x5930, PDF_CMAP_RANGE, 1094 },
	{ 0x5933, 0x5933, PDF_CMAP_SINGLE, 1096 },
	{ 0x5935, 0x5936, PDF_CMAP_RANGE, 1097 },
	{ 0x593f, 0x593f, PDF_CMAP_SINGLE, 1099 },
	{ 0x5943, 0x5943, PDF_CMAP_SINGLE, 1100 },
	{ 0x5946, 0x5946, PDF_CMAP_SINGLE, 1101 },
	{ 0x5952, 0x5953, PDF_CMAP_RANGE, 1102 },
	{ 0x5959, 0x5959, PDF_CMAP_SINGLE, 1104 },
	{ 0x595b, 0x595b, PDF_CMAP_SINGLE, 1105 },
	{ 0x595d, 0x595f, PDF_CMAP_RANGE, 1106 },
	{ 0x5961, 0x5961, PDF_CMAP_SINGLE, 1109 },
	{ 0x5963, 0x5963, PDF_CMAP_SINGLE, 1110 },
	{ 0x596b, 0x596b, PDF_CMAP_SINGLE, 1111 },
	{ 0x596d, 0x596d, PDF_CMAP_SINGLE, 1112 },
	{ 0x596f, 0x596f, PDF_CMAP_SINGLE, 1113 },
	{ 0x5972, 0x5972, PDF_CMAP_SINGLE, 1114 },
	{ 0x5975, 0x5976, PDF_CMAP_RANGE, 1115 },
	{ 0x5979, 0x5979, PDF_CMAP_SINGLE, 1117 },
	{ 0x597b, 0x597c, PDF_CMAP_RANGE, 1118 },
	{ 0x598b, 0x598c, PDF_CMAP_RANGE, 1120 },
	{ 0x598e, 0x598e, PDF_CMAP_SINGLE, 1122 },
	{ 0x5992, 0x5992, PDF_CMAP_SINGLE, 1123 },
	{ 0x5995, 0x5995, PDF_CMAP_SINGLE, 1124 },
	{ 0x5997, 0x5997, PDF_CMAP_SINGLE, 1125 },
	{ 0x599f, 0x599f, PDF_CMAP_SINGLE, 1126 },
	{ 0x59a4, 0x59a4, PDF_CMAP_SINGLE, 1127 },
	{ 0x59a7, 0x59a7, PDF_CMAP_SINGLE, 1128 },
	{ 0x59ad, 0x59b0, PDF_CMAP_RANGE, 1129 },
	{ 0x59b3, 0x59b3, PDF_CMAP_SINGLE, 1133 },
	{ 0x59b7, 0x59b7, PDF_CMAP_SINGLE, 1134 },
	{ 0x59ba, 0x59ba, PDF_CMAP_SINGLE, 1135 },
	{ 0x59bc, 0x59bc, PDF_CMAP_SINGLE, 1136 },
	{ 0x59c1, 0x59c1, PDF_CMAP_SINGLE, 1137 },
	{ 0x59c3, 0x59c4, PDF_CMAP_RANGE, 1138 },
	{ 0x59c8, 0x59c8, PDF_CMAP_SINGLE, 1140 },
	{ 0x59ca, 0x59ca, PDF_CMAP_SINGLE, 1141 },
	{ 0x59cd, 0x59cd, PDF_CMAP_SINGLE, 1142 },
	{ 0x59d2, 0x59d2, PDF_CMAP_SINGLE, 1143 },
	{ 0x59dd, 0x59df, PDF_CMAP_RANGE, 1144 },
	{ 0x59e3, 0x59e4, PDF_CMAP_RANGE, 1147 },
	{ 0x59e7, 0x59e7, PDF_CMAP_SINGLE, 1149 },
	{ 0x59ee, 0x59ef, PDF_CMAP_RANGE, 1150 },
	{ 0x59f1, 0x59f2, PDF_CMAP_RANGE, 1152 },
	{ 0x59f4, 0x59f4, PDF_CMAP_SINGLE, 1154 },
	{ 0x59f7, 0x59f7, PDF_CMAP_SINGLE, 1155 },
	{ 0x5a00, 0x5a00, PDF_CMAP_SINGLE, 1156 },
	{ 0x5a04, 0x5a04, PDF_CMAP_SINGLE, 1157 },
	{ 0x5a0c, 0x5a0e, PDF_CMAP_RANGE, 1158 },
	{ 0x5a12, 0x5a13, PDF_CMAP_RANGE, 1161 },
	{ 0x5a1e, 0x5a1e, PDF_CMAP_SINGLE, 1163 },
	{ 0x5a23, 0x5a24, PDF_CMAP_RANGE, 1164 },
	{ 0x5a27, 0x5a28, PDF_CMAP_RANGE, 1166 },
	{ 0x5a2a, 0x5a2a, PDF_CMAP_SINGLE, 1168 },
	{ 0x5a2d, 0x5a2d, PDF_CMAP_SINGLE, 1169 },
	{ 0x5a30, 0x5a30, PDF_CMAP_SINGLE, 1170 },
	{ 0x5a44, 0x5a45, PDF_CMAP_RANGE, 1171 },
	{ 0x5a47, 0x5a48, PDF_CMAP_RANGE, 1173 },
	{ 0x5a4c, 0x5a4c, PDF_CMAP_SINGLE, 1175 },
	{ 0x5a50, 0x5a50, PDF_CMAP_SINGLE, 1176 },
	{ 0x5a55, 0x5a55, PDF_CMAP_SINGLE, 1177 },
	{ 0x5a5e, 0x5a5e, PDF_CMAP_SINGLE, 1178 },
	{ 0x5a63, 0x5a63, PDF_CMAP_SINGLE, 1179 },
	{ 0x5a65, 0x5a65, PDF_CMAP_SINGLE, 1180 },
	{ 0x5a67, 0x5a67, PDF_CMAP_SINGLE, 1181 },
	{ 0x5a6d, 0x5a6d, PDF_CMAP_SINGLE, 1182 },
	{ 0x5a77, 0x5a77, PDF_CMAP_SINGLE, 1183 },
	{ 0x5a7a, 0x5a7b, PDF_CMAP_RANGE, 1184 },
	{ 0x5a7e, 0x5a7e, PDF_CMAP_SINGLE, 1186 },
	{ 0x5a8b, 0x5a8b, PDF_CMAP_SINGLE, 1187 },
	{ 0x5a90, 0x5a90, PDF_CMAP_SINGLE, 1188 },
	{ 0x5a93, 0x5a93, PDF_CMAP_SINGLE, 1189 },
	{ 0x5a96, 0x5a96, PDF_CMAP_SINGLE, 1190 },
	{ 0x5a99, 0x5a99, PDF_CMAP_SINGLE, 1191 },
	{ 0x5a9c, 0x5a9c, PDF_CMAP_SINGLE, 1192 },
	{ 0x5a9e, 0x5aa0, PDF_CMAP_RANGE, 1193 },
	{ 0x5aa2, 0x5aa2, PDF_CMAP_SINGLE, 1196 },
	{ 0x5aa7, 0x5aa7, PDF_CMAP_SINGLE, 1197 },
	{ 0x5aac, 0x5aac, PDF_CMAP_SINGLE, 1198 },
	{ 0x5ab1, 0x5ab3, PDF_CMAP_RANGE, 1199 },
	{ 0x5ab5, 0x5ab5, PDF_CMAP_SINGLE, 1202 },
	{ 0x5ab8, 0x5ab8, PDF_CMAP_SINGLE, 1203 },
	{ 0x5aba, 0x5abb, PDF_CMAP_RANGE, 1204 },
	{ 0x5abf, 0x5abf, PDF_CMAP_SINGLE, 1206 },
	{ 0x5ac4, 0x5ac4, PDF_CMAP_SINGLE, 1207 },
	{ 0x5ac6, 0x5ac6, PDF_CMAP_SINGLE, 1208 },
	{ 0x5ac8, 0x5ac8, PDF_CMAP_SINGLE, 1209 },
	{ 0x5acf, 0x5acf, PDF_CMAP_SINGLE, 1210 },
	{ 0x5ada, 0x5ada, PDF_CMAP_SINGLE, 1211 },
	{ 0x5adc, 0x5adc, PDF_CMAP_SINGLE, 1212 },
	{ 0x5ae0, 0x5ae0, PDF_CMAP_SINGLE, 1213 },
	{ 0x5ae5, 0x5ae5, PDF_CMAP_SINGLE, 1214 },
	{ 0x5aea, 0x5aea, PDF_CMAP_SINGLE, 1215 },
	{ 0x5aee, 0x5aee, PDF_CMAP_SINGLE, 1216 },
	{ 0x5af5, 0x5af6, PDF_CMAP_RANGE, 1217 },
	{ 0x5afd, 0x5afd, PDF_CMAP_SINGLE, 1219 },
	{ 0x5b00, 0x5b01, PDF_CMAP_RANGE, 1220 },
	{ 0x5b08, 0x5b08, PDF_CMAP_SINGLE, 1222 },
	{ 0x5b17, 0x5b17, PDF_CMAP_SINGLE, 1223 },
	{ 0x5b19, 0x5b19, PDF_CMAP_SINGLE, 1225 },
	{ 0x5b1b, 0x5b1b, PDF_CMAP_SINGLE, 1226 },
	{ 0x5b1d, 0x5b1d, PDF_CMAP_SINGLE, 1227 },
	{ 0x5b21, 0x5b21, PDF_CMAP_SINGLE, 1228 },
	{ 0x5b25, 0x5b25, PDF_CMAP_SINGLE, 1229 },
	{ 0x5b2d, 0x5b2d, PDF_CMAP_SINGLE, 1230 },
	{ 0x5b34, 0x5b34, PDF_CMAP_SINGLE, 1224 },
	{ 0x5b38, 0x5b38, PDF_CMAP_SINGLE, 1231 },
	{ 0x5b41, 0x5b41, PDF_CMAP_SINGLE, 1232 },
	{ 0x5b4b, 0x5b4c, PDF_CMAP_RANGE, 1233 },
	{ 0x5b52, 0x5b52, PDF_CMAP_SINGLE, 1235 },
	{ 0x5b56, 0x5b56, PDF_CMAP_SINGLE, 1236 },
	{ 0x5b5e, 0x5b5e, PDF_CMAP_SINGLE, 1237 },
	{ 0x5b68, 0x5b68, PDF_CMAP_SINGLE, 1238 },
	{ 0x5b6e, 0x5b6f, PDF_CMAP_RANGE, 1239 },
	{ 0x5b7c, 0x5b7f, PDF_CMAP_RANGE, 1241 },
	{ 0x5b81, 0x5b81, PDF_CMAP_SINGLE, 1245 },
	{ 0x5b84, 0x5b84, PDF_CMAP_SINGLE, 1246 },
	{ 0x5b86, 0x5b86, PDF_CMAP_SINGLE, 1247 },
	{ 0x5b8a, 0x5b8a, PDF_CMAP_SINGLE, 1248 },
	{ 0x5b8e, 0x5b8e, PDF_CMAP_SINGLE, 1249 },
	{ 0x5b90, 0x5b91, PDF_CMAP_RANGE, 1250 },
	{ 0x5b93, 0x5b94, PDF_CMAP_RANGE, 1252 },
	{ 0x5b96, 0x5b96, PDF_CMAP_SINGLE, 1254 },
	{ 0x5ba8, 0x5ba9, PDF_CMAP_RANGE, 1255 },
	{ 0x5bac, 0x5bad, PDF_CMAP_RANGE, 1257 },
	{ 0x5baf, 0x5baf, PDF_CMAP_SINGLE, 1259 },
	{ 0x5bb1, 0x5bb2, PDF_CMAP_RANGE, 1260 },
	{ 0x5bb7, 0x5bb7, PDF_CMAP_SINGLE, 1262 },
	{ 0x5bba, 0x5bba, PDF_CMAP_SINGLE, 1263 },
	{ 0x5bbc, 0x5bbc, PDF_CMAP_SINGLE, 1264 },
	{ 0x5bc0, 0x5bc1, PDF_CMAP_RANGE, 1265 },
	{ 0x5bcd, 0x5bcd, PDF_CMAP_SINGLE, 1267 },
	{ 0x5bcf, 0x5bcf, PDF_CMAP_SINGLE, 1268 },
	{ 0x5bd6, 0x5bda, PDF_CMAP_RANGE, 1269 },
	{ 0x5be0, 0x5be0, PDF_CMAP_SINGLE, 1274 },
	{ 0x5bef, 0x5bef, PDF_CMAP_SINGLE, 1275 },
	{ 0x5bf1, 0x5bf1, PDF_CMAP_SINGLE, 1276 },
	{ 0x5bf4, 0x5bf4, PDF_CMAP_SINGLE, 1277 },
	{ 0x5bfd, 0x5bfd, PDF_CMAP_SINGLE, 1278 },
	{ 0x5c0c, 0x5c0c, PDF_CMAP_SINGLE, 1279 },
	{ 0x5c17, 0x5c17, PDF_CMAP_SINGLE, 1280 },
	{ 0x5c1e, 0x5c1f, PDF_CMAP_RANGE, 1281 },
	{ 0x5c23, 0x5c23, PDF_CMAP_SINGLE, 1283 },
	{ 0x5c26, 0x5c26, PDF_CMAP_SINGLE, 1284 },
	{ 0x5c29, 0x5c29, PDF_CMAP_SINGLE, 1285 },
	{ 0x5c2b, 0x5c2c, PDF_CMAP_RANGE, 1286 },
	{ 0x5c2e, 0x5c2e, PDF_CMAP_SINGLE, 1288 },
	{ 0x5c30, 0x5c30, PDF_CMAP_SINGLE, 1289 },
	{ 0x5c32, 0x5c32, PDF_CMAP_SINGLE, 1290 },
	{ 0x5c35, 0x5c36, PDF_CMAP_RANGE, 1291 },
	{ 0x5c59, 0x5c5a, PDF_CMAP_RANGE, 1293 },
	{ 0x5c5c, 0x5c5c, PDF_CMAP_SINGLE, 1295 },
	{ 0x5c62, 0x5c63, PDF_CMAP_RANGE, 1296 },
	{ 0x5c67, 0x5c69, PDF_CMAP_RANGE, 1298 },
	{ 0x5c6d, 0x5c6d, PDF_CMAP_SINGLE, 1301 },
	{ 0x5c70, 0x5c70, PDF_CMAP_SINGLE, 1302 },
	{ 0x5c74, 0x5c75, PDF_CMAP_RANGE, 1303 },
	{ 0x5c7a, 0x5c7d, PDF_CMAP_RANGE, 1305 },
	{ 0x5c87, 0x5c88, PDF_CMAP_RANGE, 1309 },
	{ 0x5c8a, 0x5c8a, PDF_CMAP_SINGLE, 1311 },
	{ 0x5c8f, 0x5c8f, PDF_CMAP_SINGLE, 1312 },
	{ 0x5c92, 0x5c92, PDF_CMAP_SINGLE, 1313 },
	{ 0x5c9d, 0x5c9d, PDF_CMAP_SINGLE, 1314 },
	{ 0x5c9f, 0x5ca0, PDF_CMAP_RANGE, 1315 },
	{ 0x5ca2, 0x5ca3, PDF_CMAP_RANGE, 1317 },
	{ 0x5ca6, 0x5ca6, PDF_CMAP_SINGLE, 1319 },
	{ 0x5caa, 0x5caa, PDF_CMAP_SINGLE, 1320 },
	{ 0x5cb2, 0x5cb2, PDF_CMAP_SINGLE, 1321 },
	{ 0x5cb4, 0x5cb5, PDF_CMAP_RANGE, 1322 },
	{ 0x5cba, 0x5cba, PDF_CMAP_SINGLE, 1324 },
	{ 0x5cc9, 0x5cc9, PDF_CMAP_SINGLE, 1325 },
	{ 0x5ccb, 0x5ccb, PDF_CMAP_SINGLE, 1326 },
	{ 0x5cd2, 0x5cd2, PDF_CMAP_SINGLE, 1327 },
	{ 0x5cd7, 0x5cd7, PDF_CMAP_SINGLE, 1329 },
	{ 0x5cdd, 0x5cdd, PDF_CMAP_SINGLE, 1328 },
	{ 0x5cee, 0x5cee, PDF_CMAP_SINGLE, 1330 },
	{ 0x5cf1, 0x5cf2, PDF_CMAP_RANGE, 1331 },
	{ 0x5cf4, 0x5cf4, PDF_CMAP_SINGLE, 1333 },
	{ 0x5d01, 0x5d01, PDF_CMAP_SINGLE, 1334 },
	{ 0x5d06, 0x5d06, PDF_CMAP_SINGLE, 1335 },
	{ 0x5d0d, 0x5d0d, PDF_CMAP_SINGLE, 1336 },
	{ 0x5d12, 0x5d12, PDF_CMAP_SINGLE, 1337 },
	{ 0x5d23, 0x5d24, PDF_CMAP_RANGE, 1339 },
	{ 0x5d26, 0x5d27, PDF_CMAP_RANGE, 1341 },
	{ 0x5d2b, 0x5d2b, PDF_CMAP_SINGLE, 1338 },
	{ 0x5d31, 0x5d31, PDF_CMAP_SINGLE, 1343 },
	{ 0x5d34, 0x5d34, PDF_CMAP_SINGLE, 1344 },
	{ 0x5d39, 0x5d39, PDF_CMAP_SINGLE, 1345 },
	{ 0x5d3d, 0x5d3d, PDF_CMAP_SINGLE, 1346 },
	{ 0x5d3f, 0x5d3f, PDF_CMAP_SINGLE, 1347 },
	{ 0x5d42, 0x5d43, PDF_CMAP_RANGE, 1348 },
	{ 0x5d46, 0x5d46, PDF_CMAP_SINGLE, 1350 },
	{ 0x5d48, 0x5d48, PDF_CMAP_SINGLE, 1351 },
	{ 0x5d4a, 0x5d4a, PDF_CMAP_SINGLE, 1355 },
	{ 0x5d51, 0x5d51, PDF_CMAP_SINGLE, 1353 },
	{ 0x5d55, 0x5d55, PDF_CMAP_SINGLE, 1352 },
	{ 0x5d59, 0x5d59, PDF_CMAP_SINGLE, 1354 },
	{ 0x5d5f, 0x5d62, PDF_CMAP_RANGE, 1356 },
	{ 0x5d64, 0x5d64, PDF_CMAP_SINGLE, 1360 },
	{ 0x5d6a, 0x5d6a, PDF_CMAP_SINGLE, 1361 },
	{ 0x5d6d, 0x5d6d, PDF_CMAP_SINGLE, 1362 },
	{ 0x5d70, 0x5d70, PDF_CMAP_SINGLE, 1363 },
	{ 0x5d79, 0x5d7a, PDF_CMAP_RANGE, 1364 },
	{ 0x5d7e, 0x5d7f, PDF_CMAP_RANGE, 1366 },
	{ 0x5d81, 0x5d81, PDF_CMAP_SINGLE, 1368 },
	{ 0x5d83, 0x5d83, PDF_CMAP_SINGLE, 1369 },
	{ 0x5d88, 0x5d88, PDF_CMAP_SINGLE, 1370 },
	{ 0x5d8a, 0x5d8a, PDF_CMAP_SINGLE, 1371 },
	{ 0x5d92, 0x5d95, PDF_CMAP_RANGE, 1372 },
	{ 0x5d99, 0x5d99, PDF_CMAP_SINGLE, 1376 },
	{ 0x5d9b, 0x5d9b, PDF_CMAP_SINGLE, 1377 },
	{ 0x5d9f, 0x5da0, PDF_CMAP_RANGE, 1378 },
	{ 0x5da7, 0x5da7, PDF_CMAP_SINGLE, 1380 },
	{ 0x5dab, 0x5dab, PDF_CMAP_SINGLE, 1381 },
	{ 0x5db0, 0x5db0, PDF_CMAP_SINGLE, 1382 },
	{ 0x5db2, 0x5db2, PDF_CMAP_SINGLE, 5426 },
	{ 0x5db4, 0x5db4, PDF_CMAP_SINGLE, 1383 },
	{ 0x5db8, 0x5db9, PDF_CMAP_RANGE, 1384 },
	{ 0x5dc3, 0x5dc3, PDF_CMAP_SINGLE, 1386 },
	{ 0x5dc7, 0x5dc7, PDF_CMAP_SINGLE, 1387 },
	{ 0x5dcb, 0x5dcb, PDF_CMAP_SINGLE, 1388 },
	{ 0x5dce, 0x5dce, PDF_CMAP_SINGLE, 1390 },
	{ 0x5dd0, 0x5dd0, PDF_CMAP_SINGLE, 1389 },
	{ 0x5dd8, 0x5dd9, PDF_CMAP_RANGE, 1391 },
	{ 0x5de0, 0x5de0, PDF_CMAP_SINGLE, 1393 },
	{ 0x5de4, 0x5de4, PDF_CMAP_SINGLE, 1394 },
	{ 0x5de9, 0x5de9, PDF_CMAP_SINGLE, 1395 },
	{ 0x5df8, 0x5df9, PDF_CMAP_RANGE, 1396 },
	{ 0x5e00, 0x5e00, PDF_CMAP_SINGLE, 1398 },
	{ 0x5e07, 0x5e07, PDF_CMAP_SINGLE, 1399 },
	{ 0x5e0d, 0x5e0d, PDF_CMAP_SINGLE, 1400 },
	{ 0x5e12, 0x5e12, PDF_CMAP_SINGLE, 1401 },
	{ 0x5e14, 0x5e15, PDF_CMAP_RANGE, 1402 },
	{ 0x5e18, 0x5e18, PDF_CMAP_SINGLE, 1404 },
	{ 0x5e1f, 0x5e20, PDF_CMAP_RANGE, 1405 },
	{ 0x5e28, 0x5e28, PDF_CMAP_SINGLE, 1408 },
	{ 0x5e2e, 0x5e2e, PDF_CMAP_SINGLE, 1407 },
	{ 0x5e32, 0x5e32, PDF_CMAP_SINGLE, 1409 },
	{ 0x5e35, 0x5e35, PDF_CMAP_SINGLE, 1410 },
	{ 0x5e3e, 0x5e3e, PDF_CMAP_SINGLE, 1411 },
	{ 0x5e49, 0x5e49, PDF_CMAP_SINGLE, 1414 },
	{ 0x5e4b, 0x5e4b, PDF_CMAP_SINGLE, 1412 },
	{ 0x5e50, 0x5e51, PDF_CMAP_TABLE, 231 },
	{ 0x5e56, 0x5e56, PDF_CMAP_SINGLE, 1416 },
	{ 0x5e58, 0x5e58, PDF_CMAP_SINGLE, 1417 },
	{ 0x5e5b, 0x5e5c, PDF_CMAP_RANGE, 1418 },
	{ 0x5e5e, 0x5e5e, PDF_CMAP_SINGLE, 1420 },
	{ 0x5e68, 0x5e68, PDF_CMAP_SINGLE, 1421 },
	{ 0x5e6a, 0x5e6e, PDF_CMAP_RANGE, 1422 },
	{ 0x5e70, 0x5e70, PDF_CMAP_SINGLE, 1427 },
	{ 0x5e80, 0x5e80, PDF_CMAP_SINGLE, 1428 },
	{ 0x5e8b, 0x5e8b, PDF_CMAP_SINGLE, 1429 },
	{ 0x5e8e, 0x5e8e, PDF_CMAP_SINGLE, 1430 },
	{ 0x5ea2, 0x5ea2, PDF_CMAP_SINGLE, 1431 },
	{ 0x5ea4, 0x5ea5, PDF_CMAP_RANGE, 1432 },
	{ 0x5ea8, 0x5ea8, PDF_CMAP_SINGLE, 1434 },
	{ 0x5eaa, 0x5eaa, PDF_CMAP_SINGLE, 1435 },
	{ 0x5eac, 0x5eac, PDF_CMAP_SINGLE, 1436 },
	{ 0x5eb1, 0x5eb1, PDF_CMAP_SINGLE, 1437 },
	{ 0x5eb3, 0x5eb3, PDF_CMAP_SINGLE, 1438 },
	{ 0x5ebd, 0x5ebf, PDF_CMAP_RANGE, 1439 },
	{ 0x5ec6, 0x5ec6, PDF_CMAP_SINGLE, 1442 },
	{ 0x5ecb, 0x5ecc, PDF_CMAP_TABLE, 233 },
	{ 0x5ece, 0x5ece, PDF_CMAP_SINGLE, 1445 },
	{ 0x5ed1, 0x5ed2, PDF_CMAP_RANGE, 1446 },
	{ 0x5ed4, 0x5ed5, PDF_CMAP_RANGE, 1448 },
	{ 0x5edc, 0x5edc, PDF_CMAP_SINGLE, 1450 },
	{ 0x5ede, 0x5ede, PDF_CMAP_SINGLE, 1451 },
	{ 0x5ee5, 0x5ee5, PDF_CMAP_SINGLE, 1452 },
	{ 0x5eeb, 0x5eeb, PDF_CMAP_SINGLE, 1453 },
	{ 0x5f02, 0x5f02, PDF_CMAP_SINGLE, 1454 },
	{ 0x5f06, 0x5f08, PDF_CMAP_RANGE, 1455 },
	{ 0x5f0e, 0x5f0e, PDF_CMAP_SINGLE, 1458 },
	{ 0x5f19, 0x5f19, PDF_CMAP_SINGLE, 1459 },
	{ 0x5f1c, 0x5f1d, PDF_CMAP_RANGE, 1460 },
	{ 0x5f21, 0x5f24, PDF_CMAP_RANGE, 1462 },
	{ 0x5f28, 0x5f28, PDF_CMAP_SINGLE, 1466 },
	{ 0x5f2b, 0x5f2c, PDF_CMAP_RANGE, 1467 },
	{ 0x5f2e, 0x5f2e, PDF_CMAP_SINGLE, 1469 },
	{ 0x5f30, 0x5f30, PDF_CMAP_SINGLE, 1470 },
	{ 0x5f34, 0x5f34, PDF_CMAP_SINGLE, 1471 },
	{ 0x5f36, 0x5f36, PDF_CMAP_SINGLE, 1472 },
	{ 0x5f3b, 0x5f3b, PDF_CMAP_SINGLE, 1473 },
	{ 0x5f3d, 0x5f3d, PDF_CMAP_SINGLE, 1474 },
	{ 0x5f3f, 0x5f40, PDF_CMAP_RANGE, 1475 },
	{ 0x5f44, 0x5f45, PDF_CMAP_RANGE, 1477 },
	{ 0x5f47, 0x5f47, PDF_CMAP_SINGLE, 1479 },
	{ 0x5f4d, 0x5f4d, PDF_CMAP_SINGLE, 1480 },
	{ 0x5f50, 0x5f50, PDF_CMAP_SINGLE, 1481 },
	{ 0x5f54, 0x5f54, PDF_CMAP_SINGLE, 1482 },
	{ 0x5f58, 0x5f58, PDF_CMAP_SINGLE, 1483 },
	{ 0x5f5b, 0x5f5b, PDF_CMAP_SINGLE, 1484 },
	{ 0x5f60, 0x5f60, PDF_CMAP_SINGLE, 1485 },
	{ 0x5f63, 0x5f64, PDF_CMAP_RANGE, 1486 },
	{ 0x5f67, 0x5f67, PDF_CMAP_SINGLE, 1488 },
	{ 0x5f6f, 0x5f6f, PDF_CMAP_SINGLE, 1489 },
	{ 0x5f72, 0x5f72, PDF_CMAP_SINGLE, 1490 },
	{ 0x5f74, 0x5f75, PDF_CMAP_RANGE, 1491 },
	{ 0x5f78, 0x5f78, PDF_CMAP_SINGLE, 1493 },
	{ 0x5f7a, 0x5f7a, PDF_CMAP_SINGLE, 1494 },
	{ 0x5f7d, 0x5f7e, PDF_CMAP_RANGE, 1495 },
	{ 0x5f89, 0x5f89, PDF_CMAP_SINGLE, 1497 },
	{ 0x5f8d, 0x5f8d, PDF_CMAP_SINGLE, 1498 },
	{ 0x5f8f, 0x5f8f, PDF_CMAP_SINGLE, 1499 },
	{ 0x5f96, 0x5f96, PDF_CMAP_SINGLE, 1500 },
	{ 0x5f9c, 0x5f9d, PDF_CMAP_RANGE, 1501 },
	{ 0x5fa2, 0x5fa2, PDF_CMAP_SINGLE, 1503 },
	{ 0x5fa4, 0x5fa4, PDF_CMAP_SINGLE, 1506 },
	{ 0x5fa7, 0x5fa7, PDF_CMAP_SINGLE, 1504 },
	{ 0x5fab, 0x5fac, PDF_CMAP_TABLE, 235 },
	{ 0x5faf, 0x5fb1, PDF_CMAP_RANGE, 1508 },
	{ 0x5fb8, 0x5fb8, PDF_CMAP_SINGLE, 1511 },
	{ 0x5fc4, 0x5fc4, PDF_CMAP_SINGLE, 1512 },
	{ 0x5fc7, 0x5fc9, PDF_CMAP_RANGE, 1513 },
	{ 0x5fcb, 0x5fcb, PDF_CMAP_SINGLE, 1516 },
	{ 0x5fd0, 0x5fd4, PDF_CMAP_RANGE, 1517 },
	{ 0x5fde, 0x5fde, PDF_CMAP_SINGLE, 1522 },
	{ 0x5fe1, 0x5fe2, PDF_CMAP_RANGE, 1523 },
	{ 0x5fe8, 0x5fea, PDF_CMAP_RANGE, 1525 },
	{ 0x5fec, 0x5fef, PDF_CMAP_RANGE, 1528 },
	{ 0x5ff2, 0x5ff3, PDF_CMAP_RANGE, 1532 },
	{ 0x5ff6, 0x5ff6, PDF_CMAP_SINGLE, 1534 },
	{ 0x5ffa, 0x5ffa, PDF_CMAP_SINGLE, 1535 },
	{ 0x5ffc, 0x5ffc, PDF_CMAP_SINGLE, 1536 },
	{ 0x6007, 0x6007, PDF_CMAP_SINGLE, 1537 },
	{ 0x600a, 0x600a, PDF_CMAP_SINGLE, 1538 },
	{ 0x600d, 0x600d, PDF_CMAP_SINGLE, 1539 },
	{ 0x6013, 0x6014, PDF_CMAP_RANGE, 1540 },
	{ 0x6017, 0x6018, PDF_CMAP_RANGE, 1542 },
	{ 0x601a, 0x601a, PDF_CMAP_SINGLE, 1544 },
	{ 0x601f, 0x601f, PDF_CMAP_SINGLE, 1545 },
	{ 0x6024, 0x6024, PDF_CMAP_SINGLE, 1546 },
	{ 0x602d, 0x602d, PDF_CMAP_SINGLE, 1547 },
	{ 0x6033, 0x6033, PDF_CMAP_SINGLE, 1548 },
	{ 0x6035, 0x6035, PDF_CMAP_SINGLE, 1549 },
	{ 0x6040, 0x6040, PDF_CMAP_SINGLE, 1550 },
	{ 0x6047, 0x6049, PDF_CMAP_RANGE, 1551 },
	{ 0x604c, 0x604c, PDF_CMAP_SINGLE, 1554 },
	{ 0x6051, 0x6051, PDF_CMAP_SINGLE, 1555 },
	{ 0x6054, 0x6054, PDF_CMAP_SINGLE, 1556 },
	{ 0x6056, 0x6057, PDF_CMAP_RANGE, 1557 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 1559 },
	{ 0x6061, 0x6061, PDF_CMAP_SINGLE, 1560 },
	{ 0x6067, 0x6067, PDF_CMAP_SINGLE, 1561 },
	{ 0x6071, 0x6071, PDF_CMAP_SINGLE, 1562 },
	{ 0x607e, 0x607f, PDF_CMAP_RANGE, 1563 },
	{ 0x6082, 0x6082, PDF_CMAP_SINGLE, 1565 },
	{ 0x6086, 0x6086, PDF_CMAP_SINGLE, 1566 },
	{ 0x6088, 0x6088, PDF_CMAP_SINGLE, 1567 },
	{ 0x608a, 0x608a, PDF_CMAP_SINGLE, 1568 },
	{ 0x608e, 0x608e, PDF_CMAP_SINGLE, 1569 },
	{ 0x6091, 0x6091, PDF_CMAP_SINGLE, 1570 },
	{ 0x6093, 0x6093, PDF_CMAP_SINGLE, 1571 },
	{ 0x6095, 0x6095, PDF_CMAP_SINGLE, 1572 },
	{ 0x6098, 0x6098, PDF_CMAP_SINGLE, 1573 },
	{ 0x609d, 0x609e, PDF_CMAP_RANGE, 1574 },
	{ 0x60a2, 0x60a2, PDF_CMAP_SINGLE, 1576 },
	{ 0x60a4, 0x60a5, PDF_CMAP_RANGE, 1577 },
	{ 0x60a8, 0x60a8, PDF_CMAP_SINGLE, 1579 },
	{ 0x60b0, 0x60b1, PDF_CMAP_RANGE, 1580 },
	{ 0x60b7, 0x60b7, PDF_CMAP_SINGLE, 1582 },
	{ 0x60bb, 0x60bb, PDF_CMAP_SINGLE, 1583 },
	{ 0x60be, 0x60be, PDF_CMAP_SINGLE, 1584 },
	{ 0x60c2, 0x60c2, PDF_CMAP_SINGLE, 1585 },
	{ 0x60c4, 0x60c4, PDF_CMAP_SINGLE, 1586 },
	{ 0x60c8, 0x60cb, PDF_CMAP_RANGE, 1587 },
	{ 0x60ce, 0x60cf, PDF_CMAP_RANGE, 1591 },
	{ 0x60d4, 0x60d5, PDF_CMAP_RANGE, 1593 },
	{ 0x60d9, 0x60d9, PDF_CMAP_SINGLE, 1595 },
	{ 0x60db, 0x60db, PDF_CMAP_SINGLE, 1596 },
	{ 0x60dd, 0x60de, PDF_CMAP_RANGE, 1597 },
	{ 0x60e2, 0x60e2, PDF_CMAP_SINGLE, 1599 },
	{ 0x60e5, 0x60e5, PDF_CMAP_SINGLE, 1600 },
	{ 0x60f2, 0x60f2, PDF_CMAP_SINGLE, 1601 },
	{ 0x60f5, 0x60f5, PDF_CMAP_SINGLE, 1602 },
	{ 0x60f8, 0x60f8, PDF_CMAP_SINGLE, 1603 },
	{ 0x60fc, 0x60fd, PDF_CMAP_RANGE, 1604 },
	{ 0x6102, 0x6102, PDF_CMAP_SINGLE, 1606 },
	{ 0x6107, 0x6107, PDF_CMAP_SINGLE, 1607 },
	{ 0x610a, 0x610a, PDF_CMAP_SINGLE, 1608 },
	{ 0x610c, 0x610c, PDF_CMAP_SINGLE, 1609 },
	{ 0x6110, 0x6114, PDF_CMAP_RANGE, 1610 },
	{ 0x6116, 0x6117, PDF_CMAP_RANGE, 1615 },
	{ 0x6119, 0x6119, PDF_CMAP_SINGLE, 1617 },
	{ 0x611c, 0x611c, PDF_CMAP_SINGLE, 1618 },
	{ 0x611e, 0x611e, PDF_CMAP_SINGLE, 1619 },
	{ 0x6122, 0x6122, PDF_CMAP_SINGLE, 1620 },
	{ 0x612a, 0x612b, PDF_CMAP_RANGE, 1621 },
	{ 0x6130, 0x6131, PDF_CMAP_RANGE, 1623 },
	{ 0x6135, 0x6137, PDF_CMAP_RANGE, 1625 },
	{ 0x6139, 0x6139, PDF_CMAP_SINGLE, 1628 },
	{ 0x6141, 0x6141, PDF_CMAP_SINGLE, 1629 },
	{ 0x6145, 0x6146, PDF_CMAP_RANGE, 1630 },
	{ 0x6149, 0x6149, PDF_CMAP_SINGLE, 1632 },
	{ 0x615e, 0x615e, PDF_CMAP_SINGLE, 1633 },
	{ 0x6160, 0x6160, PDF_CMAP_SINGLE, 1634 },
	{ 0x616c, 0x616c, PDF_CMAP_SINGLE, 1635 },
	{ 0x6172, 0x6172, PDF_CMAP_SINGLE, 1636 },
	{ 0x6178, 0x6178, PDF_CMAP_SINGLE, 1637 },
	{ 0x617b, 0x617c, PDF_CMAP_RANGE, 1638 },
	{ 0x617f, 0x6181, PDF_CMAP_RANGE, 1640 },
	{ 0x6183, 0x6184, PDF_CMAP_RANGE, 1643 },
	{ 0x618b, 0x618b, PDF_CMAP_SINGLE, 1645 },
	{ 0x618d, 0x618d, PDF_CMAP_SINGLE, 1646 },
	{ 0x6192, 0x6193, PDF_CMAP_RANGE, 1647 },
	{ 0x6197, 0x6198, PDF_CMAP_RANGE, 1649 },
	{ 0x619c, 0x619d, PDF_CMAP_RANGE, 1651 },
	{ 0x619f, 0x61a0, PDF_CMAP_RANGE, 1653 },
	{ 0x61a5, 0x61a5, PDF_CMAP_SINGLE, 1655 },
	{ 0x61a8, 0x61a8, PDF_CMAP_SINGLE, 1656 },
	{ 0x61aa, 0x61aa, PDF_CMAP_SINGLE, 1657 },
	{ 0x61ad, 0x61ad, PDF_CMAP_SINGLE, 1658 },
	{ 0x61b8, 0x61b9, PDF_CMAP_RANGE, 1659 },
	{ 0x61bc, 0x61bc, PDF_CMAP_SINGLE, 1661 },
	{ 0x61c0, 0x61c2, PDF_CMAP_RANGE, 1662 },
	{ 0x61ce, 0x61cf, PDF_CMAP_RANGE, 1665 },
	{ 0x61d5, 0x61d5, PDF_CMAP_SINGLE, 1667 },
	{ 0x61dc, 0x61df, PDF_CMAP_RANGE, 1668 },
	{ 0x61e1, 0x61e2, PDF_CMAP_RANGE, 1672 },
	{ 0x61e5, 0x61e5, PDF_CMAP_SINGLE, 1676 },
	{ 0x61e7, 0x61e7, PDF_CMAP_SINGLE, 1674 },
	{ 0x61e9, 0x61e9, PDF_CMAP_SINGLE, 1675 },
	{ 0x61ec, 0x61ed, PDF_CMAP_RANGE, 1677 },
	{ 0x61ef, 0x61ef, PDF_CMAP_SINGLE, 1679 },
	{ 0x6201, 0x6201, PDF_CMAP_SINGLE, 1680 },
	{ 0x6203, 0x6204, PDF_CMAP_RANGE, 1681 },
	{ 0x6207, 0x6207, PDF_CMAP_SINGLE, 1683 },
	{ 0x6213, 0x6213, PDF_CMAP_SINGLE, 1684 },
	{ 0x6215, 0x6215, PDF_CMAP_SINGLE, 1685 },
	{ 0x621c, 0x621c, PDF_CMAP_SINGLE, 1686 },
	{ 0x6220, 0x6220, PDF_CMAP_SINGLE, 1687 },
	{ 0x6222, 0x6223, PDF_CMAP_RANGE, 1688 },
	{ 0x6227, 0x6227, PDF_CMAP_SINGLE, 1690 },
	{ 0x6229, 0x6229, PDF_CMAP_SINGLE, 1691 },
	{ 0x622b, 0x622b, PDF_CMAP_SINGLE, 1692 },
	{ 0x6239, 0x6239, PDF_CMAP_SINGLE, 1693 },
	{ 0x623d, 0x623d, PDF_CMAP_SINGLE, 1694 },
	{ 0x6242, 0x6244, PDF_CMAP_RANGE, 1695 },
	{ 0x6246, 0x6246, PDF_CMAP_SINGLE, 1698 },
	{ 0x624c, 0x624c, PDF_CMAP_SINGLE, 1699 },
	{ 0x6250, 0x6252, PDF_CMAP_RANGE, 1700 },
	{ 0x6254, 0x6254, PDF_CMAP_SINGLE, 1703 },
	{ 0x6256, 0x6256, PDF_CMAP_SINGLE, 1704 },
	{ 0x625a, 0x625a, PDF_CMAP_SINGLE, 1705 },
	{ 0x625c, 0x625c, PDF_CMAP_SINGLE, 1706 },
	{ 0x6264, 0x6264, PDF_CMAP_SINGLE, 1707 },
	{ 0x626d, 0x626d, PDF_CMAP_SINGLE, 1708 },
	{ 0x626f, 0x626f, PDF_CMAP_SINGLE, 1709 },
	{ 0x6273, 0x6273, PDF_CMAP_SINGLE, 1710 },
	{ 0x627a, 0x627a, PDF_CMAP_SINGLE, 1711 },
	{ 0x627d, 0x627d, PDF_CMAP_SINGLE, 1712 },
	{ 0x628d, 0x6290, PDF_CMAP_RANGE, 1713 },
	{ 0x62a6, 0x62a6, PDF_CMAP_SINGLE, 1717 },
	{ 0x62a8, 0x62a8, PDF_CMAP_SINGLE, 1718 },
	{ 0x62b3, 0x62b3, PDF_CMAP_SINGLE, 1719 },
	{ 0x62b6, 0x62b7, PDF_CMAP_RANGE, 1720 },
	{ 0x62ba, 0x62ba, PDF_CMAP_SINGLE, 1722 },
	{ 0x62be, 0x62bf, PDF_CMAP_RANGE, 1723 },
	{ 0x62c4, 0x62c4, PDF_CMAP_SINGLE, 1725 },
	{ 0x62ce, 0x62ce, PDF_CMAP_SINGLE, 1726 },
	{ 0x62d5, 0x62d6, PDF_CMAP_RANGE, 1727 },
	{ 0x62da, 0x62da, PDF_CMAP_SINGLE, 1729 },
	{ 0x62ea, 0x62ea, PDF_CMAP_SINGLE, 1730 },
	{ 0x62f2, 0x62f2, PDF_CMAP_SINGLE, 1731 },
	{ 0x62f4, 0x62f4, PDF_CMAP_SINGLE, 1732 },
	{ 0x62fc, 0x62fd, PDF_CMAP_RANGE, 1733 },
	{ 0x6303, 0x6304, PDF_CMAP_RANGE, 1735 },
	{ 0x630a, 0x630b, PDF_CMAP_RANGE, 1737 },
	{ 0x630d, 0x630d, PDF_CMAP_SINGLE, 1739 },
	{ 0x6310, 0x6310, PDF_CMAP_SINGLE, 1740 },
	{ 0x6313, 0x6313, PDF_CMAP_SINGLE, 1741 },
	{ 0x6316, 0x6316, PDF_CMAP_SINGLE, 1742 },
	{ 0x6318, 0x6318, PDF_CMAP_SINGLE, 1743 },
	{ 0x6329, 0x632a, PDF_CMAP_RANGE, 1744 },
	{ 0x632d, 0x632d, PDF_CMAP_SINGLE, 1746 },
	{ 0x6335, 0x6336, PDF_CMAP_RANGE, 1747 },
	{ 0x6339, 0x6339, PDF_CMAP_SINGLE, 1749 },
	{ 0x633c, 0x633c, PDF_CMAP_SINGLE, 1750 },
	{ 0x6341, 0x6344, PDF_CMAP_RANGE, 1751 },
	{ 0x6346, 0x6346, PDF_CMAP_SINGLE, 1755 },
	{ 0x634a, 0x634b, PDF_CMAP_RANGE, 1756 },
	{ 0x634e, 0x634e, PDF_CMAP_SINGLE, 1758 },
	{ 0x6352, 0x6354, PDF_CMAP_RANGE, 1759 },
	{ 0x6358, 0x6358, PDF_CMAP_SINGLE, 1762 },
	{ 0x635b, 0x635b, PDF_CMAP_SINGLE, 1763 },
	{ 0x6365, 0x6366, PDF_CMAP_RANGE, 1764 },
	{ 0x636c, 0x636d, PDF_CMAP_RANGE, 1766 },
	{ 0x6371, 0x6371, PDF_CMAP_SINGLE, 1768 },
	{ 0x6374, 0x6375, PDF_CMAP_RANGE, 1769 },
	{ 0x6378, 0x6378, PDF_CMAP_SINGLE, 1771 },
	{ 0x637c, 0x637d, PDF_CMAP_RANGE, 1772 },
	{ 0x637f, 0x637f, PDF_CMAP_SINGLE, 1774 },
	{ 0x6382, 0x6382, PDF_CMAP_SINGLE, 1775 },
	{ 0x6384, 0x6384, PDF_CMAP_SINGLE, 1776 },
	{ 0x6387, 0x6387, PDF_CMAP_SINGLE, 1777 },
	{ 0x638a, 0x638a, PDF_CMAP_SINGLE, 1778 },
	{ 0x6390, 0x6390, PDF_CMAP_SINGLE, 1779 },
	{ 0x6394, 0x6395, PDF_CMAP_RANGE, 1780 },
	{ 0x6399, 0x639a, PDF_CMAP_RANGE, 1782 },
	{ 0x639e, 0x639e, PDF_CMAP_SINGLE, 1784 },
	{ 0x63a4, 0x63a4, PDF_CMAP_SINGLE, 1785 },
	{ 0x63a6, 0x63a6, PDF_CMAP_SINGLE, 1786 },
	{ 0x63ad, 0x63af, PDF_CMAP_RANGE, 1787 },
	{ 0x63bd, 0x63bd, PDF_CMAP_SINGLE, 1790 },
	{ 0x63c1, 0x63c1, PDF_CMAP_SINGLE, 1791 },
	{ 0x63c5, 0x63c5, PDF_CMAP_SINGLE, 1792 },
	{ 0x63c8, 0x63c8, PDF_CMAP_SINGLE, 1793 },
	{ 0x63ce, 0x63ce, PDF_CMAP_SINGLE, 1794 },
	{ 0x63d1, 0x63d1, PDF_CMAP_SINGLE, 1795 },
	{ 0x63d3, 0x63d5, PDF_CMAP_RANGE, 1796 },
	{ 0x63dc, 0x63dc, PDF_CMAP_SINGLE, 1799 },
	{ 0x63e0, 0x63e0, PDF_CMAP_SINGLE, 1800 },
	{ 0x63e5, 0x63e5, PDF_CMAP_SINGLE, 1801 },
	{ 0x63ea, 0x63ea, PDF_CMAP_SINGLE, 1802 },
	{ 0x63ec, 0x63ec, PDF_CMAP_SINGLE, 1803 },
	{ 0x63f2, 0x63f3, PDF_CMAP_RANGE, 1804 },
	{ 0x63f5, 0x63f5, PDF_CMAP_SINGLE, 1806 },
	{ 0x63f8, 0x63f9, PDF_CMAP_RANGE, 1807 },
	{ 0x6409, 0x640a, PDF_CMAP_RANGE, 1809 },
	{ 0x6410, 0x6410, PDF_CMAP_SINGLE, 1811 },
	{ 0x6412, 0x6412, PDF_CMAP_SINGLE, 1812 },
	{ 0x6414, 0x6414, PDF_CMAP_SINGLE, 1813 },
	{ 0x6418, 0x6418, PDF_CMAP_SINGLE, 1814 },
	{ 0x641e, 0x641e, PDF_CMAP_SINGLE, 1815 },
	{ 0x6420, 0x6420, PDF_CMAP_SINGLE, 1816 },
	{ 0x6422, 0x6422, PDF_CMAP_SINGLE, 1817 },
	{ 0x6424, 0x6425, PDF_CMAP_RANGE, 1818 },
	{ 0x6429, 0x642a, PDF_CMAP_RANGE, 1820 },
	{ 0x642f, 0x6430, PDF_CMAP_RANGE, 1822 },
	{ 0x6435, 0x6435, PDF_CMAP_SINGLE, 1824 },
	{ 0x643d, 0x643d, PDF_CMAP_SINGLE, 1825 },
	{ 0x643f, 0x643f, PDF_CMAP_SINGLE, 1826 },
	{ 0x644b, 0x644b, PDF_CMAP_SINGLE, 1827 },
	{ 0x644f, 0x644f, PDF_CMAP_SINGLE, 1828 },
	{ 0x6451, 0x6454, PDF_CMAP_RANGE, 1829 },
	{ 0x645a, 0x645d, PDF_CMAP_RANGE, 1833 },
	{ 0x645f, 0x6461, PDF_CMAP_RANGE, 1837 },
	{ 0x6463, 0x6463, PDF_CMAP_SINGLE, 1840 },
	{ 0x646d, 0x646d, PDF_CMAP_SINGLE, 1841 },
	{ 0x6473, 0x6474, PDF_CMAP_RANGE, 1842 },
	{ 0x647b, 0x647b, PDF_CMAP_SINGLE, 1844 },
	{ 0x647d, 0x647d, PDF_CMAP_SINGLE, 1845 },
	{ 0x6485, 0x6485, PDF_CMAP_SINGLE, 1846 },
	{ 0x6487, 0x6487, PDF_CMAP_SINGLE, 1847 },
	{ 0x648f, 0x6491, PDF_CMAP_RANGE, 1848 },
	{ 0x6498, 0x6499, PDF_CMAP_RANGE, 1851 },
	{ 0x649b, 0x649b, PDF_CMAP_SINGLE, 1853 },
	{ 0x649d, 0x649d, PDF_CMAP_SINGLE, 1854 },
	{ 0x649f, 0x649f, PDF_CMAP_SINGLE, 1855 },
	{ 0x64a1, 0x64a1, PDF_CMAP_SINGLE, 1856 },
	{ 0x64a3, 0x64a3, PDF_CMAP_SINGLE, 1857 },
	{ 0x64a6, 0x64a6, PDF_CMAP_SINGLE, 1858 },
	{ 0x64a8, 0x64a8, PDF_CMAP_SINGLE, 1859 },
	{ 0x64ac, 0x64ac, PDF_CMAP_SINGLE, 1860 },
	{ 0x64b3, 0x64b3, PDF_CMAP_SINGLE, 1861 },
	{ 0x64bd, 0x64bf, PDF_CMAP_RANGE, 1862 },
	{ 0x64c4, 0x64c4, PDF_CMAP_SINGLE, 1865 },
	{ 0x64c9, 0x64cc, PDF_CMAP_RANGE, 1866 },
	{ 0x64ce, 0x64ce, PDF_CMAP_SINGLE, 1870 },
	{ 0x64d0, 0x64d1, PDF_CMAP_RANGE, 1871 },
	{ 0x64d5, 0x64d5, PDF_CMAP_SINGLE, 1873 },
	{ 0x64d7, 0x64d7, PDF_CMAP_SINGLE, 1874 },
	{ 0x64e4, 0x64e5, PDF_CMAP_RANGE, 1875 },
	{ 0x64e9, 0x64ea, PDF_CMAP_RANGE, 1877 },
	{ 0x64ed, 0x64ed, PDF_CMAP_SINGLE, 1879 },
	{ 0x64f0, 0x64f0, PDF_CMAP_SINGLE, 1880 },
	{ 0x64f5, 0x64f5, PDF_CMAP_SINGLE, 1881 },
	{ 0x64f7, 0x64f7, PDF_CMAP_SINGLE, 1882 },
	{ 0x64fb, 0x64fb, PDF_CMAP_SINGLE, 1883 },
	{ 0x64ff, 0x64ff, PDF_CMAP_SINGLE, 1884 },
	{ 0x6501, 0x6501, PDF_CMAP_SINGLE, 1885 },
	{ 0x6504, 0x6504, PDF_CMAP_SINGLE, 1886 },
	{ 0x6508, 0x650a, PDF_CMAP_RANGE, 1887 },
	{ 0x650f, 0x650f, PDF_CMAP_SINGLE, 1890 },
	{ 0x6513, 0x6514, PDF_CMAP_RANGE, 1891 },
	{ 0x6516, 0x6516, PDF_CMAP_SINGLE, 1893 },
	{ 0x6519, 0x6519, PDF_CMAP_SINGLE, 1894 },
	{ 0x651b, 0x651b, PDF_CMAP_SINGLE, 1895 },
	{ 0x651e, 0x651f, PDF_CMAP_RANGE, 1896 },
	{ 0x6522, 0x6522, PDF_CMAP_SINGLE, 1898 },
	{ 0x6526, 0x6526, PDF_CMAP_SINGLE, 1899 },
	{ 0x6529, 0x6529, PDF_CMAP_SINGLE, 1900 },
	{ 0x652e, 0x652e, PDF_CMAP_SINGLE, 1901 },
	{ 0x6531, 0x6531, PDF_CMAP_SINGLE, 1902 },
	{ 0x653a, 0x653a, PDF_CMAP_SINGLE, 1903 },
	{ 0x653c, 0x653d, PDF_CMAP_RANGE, 1904 },
	{ 0x6543, 0x6543, PDF_CMAP_SINGLE, 1906 },
	{ 0x6547, 0x6547, PDF_CMAP_SINGLE, 1907 },
	{ 0x6549, 0x6549, PDF_CMAP_SINGLE, 1908 },
	{ 0x6550, 0x6550, PDF_CMAP_SINGLE, 1909 },
	{ 0x6552, 0x6552, PDF_CMAP_SINGLE, 1910 },
	{ 0x6554, 0x6554, PDF_CMAP_SINGLE, 1911 },
	{ 0x655f, 0x6560, PDF_CMAP_RANGE, 1912 },
	{ 0x6567, 0x6567, PDF_CMAP_SINGLE, 1914 },
	{ 0x656b, 0x656b, PDF_CMAP_SINGLE, 1915 },
	{ 0x657a, 0x657a, PDF_CMAP_SINGLE, 1916 },
	{ 0x657d, 0x657d, PDF_CMAP_SINGLE, 1917 },
	{ 0x6581, 0x6581, PDF_CMAP_SINGLE, 1918 },
	{ 0x6585, 0x6585, PDF_CMAP_SINGLE, 1919 },
	{ 0x658a, 0x658a, PDF_CMAP_SINGLE, 1920 },
	{ 0x6592, 0x6592, PDF_CMAP_SINGLE, 1921 },
	{ 0x6595, 0x6595, PDF_CMAP_SINGLE, 1922 },
	{ 0x6598, 0x6598, PDF_CMAP_SINGLE, 1923 },
	{ 0x659d, 0x659d, PDF_CMAP_SINGLE, 1924 },
	{ 0x65a0, 0x65a0, PDF_CMAP_SINGLE, 1925 },
	{ 0x65a3, 0x65a3, PDF_CMAP_SINGLE, 1926 },
	{ 0x65a6, 0x65a6, PDF_CMAP_SINGLE, 1927 },
	{ 0x65ae, 0x65ae, PDF_CMAP_SINGLE, 1928 },
	{ 0x65b2, 0x65b4, PDF_CMAP_RANGE, 1929 },
	{ 0x65bf, 0x65bf, PDF_CMAP_SINGLE, 1932 },
	{ 0x65c2, 0x65c2, PDF_CMAP_SINGLE, 1933 },
	{ 0x65c8, 0x65c9, PDF_CMAP_RANGE, 1934 },
	{ 0x65ce, 0x65ce, PDF_CMAP_SINGLE, 1936 },
	{ 0x65d0, 0x65d0, PDF_CMAP_SINGLE, 1937 },
	{ 0x65d4, 0x65d4, PDF_CMAP_SINGLE, 1938 },
	{ 0x65d6, 0x65d6, PDF_CMAP_SINGLE, 1939 },
	{ 0x65d8, 0x65d8, PDF_CMAP_SINGLE, 1940 },
	{ 0x65df, 0x65df, PDF_CMAP_SINGLE, 1941 },
	{ 0x65f0, 0x65f0, PDF_CMAP_SINGLE, 1942 },
	{ 0x65f2, 0x65f2, PDF_CMAP_SINGLE, 1943 },
	{ 0x65f4, 0x65f5, PDF_CMAP_RANGE, 1944 },
	{ 0x65f9, 0x65f9, PDF_CMAP_SINGLE, 1946 },
	{ 0x65fe, 0x6600, PDF_CMAP_RANGE, 1947 },
	{ 0x6604, 0x6604, PDF_CMAP_SINGLE, 1950 },
	{ 0x6608, 0x6609, PDF_CMAP_RANGE, 1951 },
	{ 0x660d, 0x660d, PDF_CMAP_SINGLE, 1953 },
	{ 0x6611, 0x6612, PDF_CMAP_RANGE, 1954 },
	{ 0x6615, 0x6616, PDF_CMAP_RANGE, 1956 },
	{ 0x661d, 0x661e, PDF_CMAP_RANGE, 1958 },
	{ 0x6621, 0x6624, PDF_CMAP_RANGE, 1960 },
	{ 0x6626, 0x6626, PDF_CMAP_SINGLE, 1964 },
	{ 0x6629, 0x662c, PDF_CMAP_RANGE, 1965 },
	{ 0x662e, 0x662e, PDF_CMAP_SINGLE, 1969 },
	{ 0x6630, 0x6631, PDF_CMAP_RANGE, 1970 },
	{ 0x6633, 0x6633, PDF_CMAP_SINGLE, 1972 },
	{ 0x6637, 0x6637, PDF_CMAP_SINGLE, 1974 },
	{ 0x6639, 0x6639, PDF_CMAP_SINGLE, 1973 },
	{ 0x6640, 0x6640, PDF_CMAP_SINGLE, 1975 },
	{ 0x6645, 0x6646, PDF_CMAP_RANGE, 1976 },
	{ 0x664a, 0x664a, PDF_CMAP_SINGLE, 1978 },
	{ 0x664c, 0x664c, PDF_CMAP_SINGLE, 1979 },
	{ 0x664e, 0x664e, PDF_CMAP_SINGLE, 1981 },
	{ 0x6651, 0x6651, PDF_CMAP_SINGLE, 1980 },
	{ 0x6657, 0x6659, PDF_CMAP_RANGE, 1982 },
	{ 0x665b, 0x665c, PDF_CMAP_RANGE, 1985 },
	{ 0x6660, 0x6661, PDF_CMAP_RANGE, 1987 },
	{ 0x666a, 0x666c, PDF_CMAP_RANGE, 1990 },
	{ 0x6673, 0x6673, PDF_CMAP_SINGLE, 1994 },
	{ 0x6675, 0x6675, PDF_CMAP_SINGLE, 1995 },
	{ 0x6677, 0x6679, PDF_CMAP_RANGE, 1997 },
	{ 0x667b, 0x667c, PDF_CMAP_TABLE, 237 },
	{ 0x667e, 0x6680, PDF_CMAP_TABLE, 239 },
	{ 0x668b, 0x668d, PDF_CMAP_RANGE, 2003 },
	{ 0x6690, 0x6690, PDF_CMAP_SINGLE, 2006 },
	{ 0x6692, 0x6692, PDF_CMAP_SINGLE, 2007 },
	{ 0x6699, 0x669c, PDF_CMAP_RANGE, 2008 },
	{ 0x669f, 0x66a0, PDF_CMAP_RANGE, 2012 },
	{ 0x66a4, 0x66a4, PDF_CMAP_SINGLE, 2014 },
	{ 0x66ad, 0x66ad, PDF_CMAP_SINGLE, 2015 },
	{ 0x66b1, 0x66b2, PDF_CMAP_RANGE, 2016 },
	{ 0x66b5, 0x66b5, PDF_CMAP_SINGLE, 2018 },
	{ 0x66bb, 0x66bb, PDF_CMAP_SINGLE, 2019 },
	{ 0x66bf, 0x66c0, PDF_CMAP_RANGE, 2020 },
	{ 0x66c2, 0x66c3, PDF_CMAP_RANGE, 2022 },
	{ 0x66c8, 0x66c8, PDF_CMAP_SINGLE, 2024 },
	{ 0x66cc, 0x66cc, PDF_CMAP_SINGLE, 2025 },
	{ 0x66ce, 0x66cf, PDF_CMAP_RANGE, 2026 },
	{ 0x66d4, 0x66d4, PDF_CMAP_SINGLE, 2028 },
	{ 0x66db, 0x66db, PDF_CMAP_SINGLE, 2029 },
	{ 0x66df, 0x66df, PDF_CMAP_SINGLE, 2030 },
	{ 0x66e8, 0x66e8, PDF_CMAP_SINGLE, 2031 },
	{ 0x66eb, 0x66ec, PDF_CMAP_RANGE, 2032 },
	{ 0x66ee, 0x66ee, PDF_CMAP_SINGLE, 2034 },
	{ 0x66fa, 0x66fb, PDF_CMAP_TABLE, 242 },
	{ 0x6705, 0x6705, PDF_CMAP_SINGLE, 2036 },
	{ 0x6707, 0x6707, PDF_CMAP_SINGLE, 2037 },
	{ 0x670e, 0x670e, PDF_CMAP_SINGLE, 2038 },
	{ 0x6713, 0x6713, PDF_CMAP_SINGLE, 2039 },
	{ 0x6719, 0x6719, PDF_CMAP_SINGLE, 2040 },
	{ 0x671c, 0x671c, PDF_CMAP_SINGLE, 2041 },
	{ 0x6720, 0x6720, PDF_CMAP_SINGLE, 2042 },
	{ 0x6722, 0x6722, PDF_CMAP_SINGLE, 2043 },
	{ 0x6733, 0x6733, PDF_CMAP_SINGLE, 2044 },
	{ 0x673e, 0x673e, PDF_CMAP_SINGLE, 2045 },
	{ 0x6745, 0x6745, PDF_CMAP_SINGLE, 2046 },
	{ 0x6747, 0x6748, PDF_CMAP_RANGE, 2047 },
	{ 0x674c, 0x674c, PDF_CMAP_SINGLE, 2049 },
	{ 0x6754, 0x6755, PDF_CMAP_RANGE, 2050 },
	{ 0x675d, 0x675d, PDF_CMAP_SINGLE, 2052 },
	{ 0x6766, 0x6766, PDF_CMAP_SINGLE, 2053 },
	{ 0x676c, 0x676c, PDF_CMAP_SINGLE, 2054 },
	{ 0x676e, 0x676e, PDF_CMAP_SINGLE, 2055 },
	{ 0x6774, 0x6774, PDF_CMAP_SINGLE, 2056 },
	{ 0x6776, 0x6776, PDF_CMAP_SINGLE, 2057 },
	{ 0x677b, 0x677b, PDF_CMAP_SINGLE, 2058 },
	{ 0x6781, 0x6781, PDF_CMAP_SINGLE, 2059 },
	{ 0x6784, 0x6784, PDF_CMAP_SINGLE, 2060 },
	{ 0x678e, 0x678f, PDF_CMAP_RANGE, 2061 },
	{ 0x6791, 0x6791, PDF_CMAP_SINGLE, 2063 },
	{ 0x6793, 0x6793, PDF_CMAP_SINGLE, 2064 },
	{ 0x6796, 0x6796, PDF_CMAP_SINGLE, 2065 },
	{ 0x6798, 0x6799, PDF_CMAP_RANGE, 2066 },
	{ 0x679b, 0x679b, PDF_CMAP_SINGLE, 2068 },
	{ 0x67b0, 0x67b2, PDF_CMAP_RANGE, 2069 },
	{ 0x67b5, 0x67b5, PDF_CMAP_SINGLE, 2072 },
	{ 0x67bb, 0x67bd, PDF_CMAP_RANGE, 2073 },
	{ 0x67c0, 0x67c0, PDF_CMAP_SINGLE, 2077 },
	{ 0x67c2, 0x67c3, PDF_CMAP_RANGE, 2078 },
	{ 0x67c5, 0x67c5, PDF_CMAP_SINGLE, 2080 },
	{ 0x67c8, 0x67c9, PDF_CMAP_RANGE, 2081 },
	{ 0x67d2, 0x67d2, PDF_CMAP_SINGLE, 2083 },
	{ 0x67d7, 0x67d7, PDF_CMAP_SINGLE, 2084 },
	{ 0x67d9, 0x67d9, PDF_CMAP_SINGLE, 2085 },
	{ 0x67dc, 0x67dc, PDF_CMAP_SINGLE, 2086 },
	{ 0x67e1, 0x67e1, PDF_CMAP_SINGLE, 2087 },
	{ 0x67e6, 0x67e6, PDF_CMAP_SINGLE, 2088 },
	{ 0x67f0, 0x67f0, PDF_CMAP_SINGLE, 2089 },
	{ 0x67f2, 0x67f2, PDF_CMAP_SINGLE, 2090 },
	{ 0x67f6, 0x67f7, PDF_CMAP_RANGE, 2091 },
	{ 0x67f9, 0x67f9, PDF_CMAP_SINGLE, 2076 },
	{ 0x6814, 0x6814, PDF_CMAP_SINGLE, 2094 },
	{ 0x6819, 0x6819, PDF_CMAP_SINGLE, 2095 },
	{ 0x681d, 0x681d, PDF_CMAP_SINGLE, 2096 },
	{ 0x681f, 0x681f, PDF_CMAP_SINGLE, 2097 },
	{ 0x6827, 0x6828, PDF_CMAP_TABLE, 244 },
	{ 0x682c, 0x682d, PDF_CMAP_RANGE, 2100 },
	{ 0x682f, 0x6831, PDF_CMAP_RANGE, 2102 },
	{ 0x6833, 0x6833, PDF_CMAP_SINGLE, 2105 },
	{ 0x683b, 0x683b, PDF_CMAP_SINGLE, 2106 },
	{ 0x683f, 0x683f, PDF_CMAP_SINGLE, 2107 },
	{ 0x6844, 0x6845, PDF_CMAP_RANGE, 2108 },
	{ 0x684a, 0x684a, PDF_CMAP_SINGLE, 2110 },
	{ 0x684c, 0x684c, PDF_CMAP_SINGLE, 2111 },
	{ 0x6852, 0x6852, PDF_CMAP_SINGLE, 2093 },
	{ 0x6855, 0x6855, PDF_CMAP_SINGLE, 2112 },
	{ 0x6857, 0x6858, PDF_CMAP_RANGE, 2113 },
	{ 0x685b, 0x685b, PDF_CMAP_SINGLE, 2115 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 2116 },
	{ 0x686e, 0x6872, PDF_CMAP_RANGE, 2117 },
	{ 0x6875, 0x6875, PDF_CMAP_SINGLE, 2122 },
	{ 0x6879, 0x687c, PDF_CMAP_RANGE, 2123 },
	{ 0x6882, 0x6882, PDF_CMAP_SINGLE, 2127 },
	{ 0x6884, 0x6884, PDF_CMAP_SINGLE, 2128 },
	{ 0x6886, 0x6886, PDF_CMAP_SINGLE, 2129 },
	{ 0x6888, 0x6888, PDF_CMAP_SINGLE, 2130 },
	{ 0x6896, 0x6896, PDF_CMAP_SINGLE, 2131 },
	{ 0x6898, 0x6898, PDF_CMAP_SINGLE, 2132 },
	{ 0x689a, 0x689a, PDF_CMAP_SINGLE, 2133 },
	{ 0x689c, 0x689c, PDF_CMAP_SINGLE, 2134 },
	{ 0x68a1, 0x68a1, PDF_CMAP_SINGLE, 2135 },
	{ 0x68a3, 0x68a3, PDF_CMAP_SINGLE, 2136 },
	{ 0x68a5, 0x68a5, PDF_CMAP_SINGLE, 2137 },
	{ 0x68a9, 0x68aa, PDF_CMAP_RANGE, 2138 },
	{ 0x68ae, 0x68ae, PDF_CMAP_SINGLE, 2140 },
	{ 0x68b2, 0x68b2, PDF_CMAP_SINGLE, 2141 },
	{ 0x68bb, 0x68bb, PDF_CMAP_SINGLE, 2142 },
	{ 0x68c5, 0x68c5, PDF_CMAP_SINGLE, 2143 },
	{ 0x68c8, 0x68c8, PDF_CMAP_SINGLE, 2144 },
	{ 0x68cc, 0x68cc, PDF_CMAP_SINGLE, 2145 },
	{ 0x68cf, 0x68d1, PDF_CMAP_RANGE, 2146 },
	{ 0x68d3, 0x68d3, PDF_CMAP_SINGLE, 2149 },
	{ 0x68d6, 0x68d6, PDF_CMAP_SINGLE, 2150 },
	{ 0x68d9, 0x68d9, PDF_CMAP_SINGLE, 2151 },
	{ 0x68dc, 0x68dd, PDF_CMAP_RANGE, 2152 },
	{ 0x68e5, 0x68e5, PDF_CMAP_SINGLE, 2154 },
	{ 0x68e8, 0x68e8, PDF_CMAP_SINGLE, 2155 },
	{ 0x68ea, 0x68ed, PDF_CMAP_RANGE, 2156 },
	{ 0x68f0, 0x68f1, PDF_CMAP_RANGE, 2160 },
	{ 0x68f5, 0x68f6, PDF_CMAP_RANGE, 2162 },
	{ 0x68fb, 0x68fd, PDF_CMAP_RANGE, 2164 },
	{ 0x6906, 0x6906, PDF_CMAP_SINGLE, 2167 },
	{ 0x6909, 0x690a, PDF_CMAP_RANGE, 2168 },
	{ 0x6910, 0x6911, PDF_CMAP_RANGE, 2170 },
	{ 0x6913, 0x6913, PDF_CMAP_SINGLE, 2172 },
	{ 0x6916, 0x6917, PDF_CMAP_RANGE, 2173 },
	{ 0x6931, 0x6931, PDF_CMAP_SINGLE, 2175 },
	{ 0x6933, 0x6933, PDF_CMAP_SINGLE, 2176 },
	{ 0x6935, 0x6935, PDF_CMAP_SINGLE, 2177 },
	{ 0x6938, 0x6938, PDF_CMAP_SINGLE, 2178 },
	{ 0x693b, 0x693b, PDF_CMAP_SINGLE, 2179 },
	{ 0x6942, 0x6942, PDF_CMAP_SINGLE, 2180 },
	{ 0x6945, 0x6945, PDF_CMAP_SINGLE, 2181 },
	{ 0x6949, 0x6949, PDF_CMAP_SINGLE, 2182 },
	{ 0x694e, 0x694e, PDF_CMAP_SINGLE, 2183 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 2184 },
	{ 0x695b, 0x695b, PDF_CMAP_SINGLE, 2185 },
	{ 0x6963, 0x6966, PDF_CMAP_RANGE, 2186 },
	{ 0x6968, 0x6969, PDF_CMAP_RANGE, 2190 },
	{ 0x696c, 0x696c, PDF_CMAP_SINGLE, 2192 },
	{ 0x6970, 0x6972, PDF_CMAP_RANGE, 2193 },
	{ 0x697a, 0x697b, PDF_CMAP_RANGE, 2196 },
	{ 0x697f, 0x6980, PDF_CMAP_RANGE, 2198 },
	{ 0x698d, 0x698d, PDF_CMAP_SINGLE, 2200 },
	{ 0x6992, 0x6992, PDF_CMAP_SINGLE, 2201 },
	{ 0x6996, 0x6996, PDF_CMAP_SINGLE, 2202 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 2203 },
	{ 0x69a1, 0x69a1, PDF_CMAP_SINGLE, 2204 },
	{ 0x69a5, 0x69a6, PDF_CMAP_RANGE, 2205 },
	{ 0x69a8, 0x69a8, PDF_CMAP_SINGLE, 2207 },
	{ 0x69ab, 0x69ab, PDF_CMAP_SINGLE, 2208 },
	{ 0x69ad, 0x69ad, PDF_CMAP_SINGLE, 2209 },
	{ 0x69af, 0x69af, PDF_CMAP_SINGLE, 2210 },
	{ 0x69b7, 0x69b8, PDF_CMAP_RANGE, 2211 },
	{ 0x69ba, 0x69ba, PDF_CMAP_SINGLE, 2213 },
	{ 0x69bc, 0x69bc, PDF_CMAP_SINGLE, 2214 },
	{ 0x69c5, 0x69c5, PDF_CMAP_SINGLE, 2215 },
	{ 0x69c8, 0x69c8, PDF_CMAP_SINGLE, 2216 },
	{ 0x69d1, 0x69d1, PDF_CMAP_SINGLE, 2217 },
	{ 0x69d6, 0x69d7, PDF_CMAP_RANGE, 2218 },
	{ 0x69e2, 0x69e2, PDF_CMAP_SINGLE, 2220 },
	{ 0x69e5, 0x69e5, PDF_CMAP_SINGLE, 2221 },
	{ 0x69ee, 0x69ef, PDF_CMAP_RANGE, 2222 },
	{ 0x69f1, 0x69f1, PDF_CMAP_SINGLE, 2224 },
	{ 0x69f3, 0x69f3, PDF_CMAP_SINGLE, 2225 },
	{ 0x69f5, 0x69f5, PDF_CMAP_SINGLE, 2226 },
	{ 0x69fe, 0x69fe, PDF_CMAP_SINGLE, 2227 },
	{ 0x6a00, 0x6a01, PDF_CMAP_RANGE, 2228 },
	{ 0x6a03, 0x6a03, PDF_CMAP_SINGLE, 2230 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 2231 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 2232 },
	{ 0x6a15, 0x6a15, PDF_CMAP_SINGLE, 2233 },
	{ 0x6a1a, 0x6a1a, PDF_CMAP_SINGLE, 2234 },
	{ 0x6a1d, 0x6a1d, PDF_CMAP_SINGLE, 2235 },
	{ 0x6a20, 0x6a20, PDF_CMAP_SINGLE, 2236 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 2237 },
	{ 0x6a28, 0x6a28, PDF_CMAP_SINGLE, 2238 },
	{ 0x6a30, 0x6a30, PDF_CMAP_SINGLE, 2239 },
	{ 0x6a32, 0x6a32, PDF_CMAP_SINGLE, 2240 },
	{ 0x6a34, 0x6a34, PDF_CMAP_SINGLE, 2241 },
	{ 0x6a37, 0x6a37, PDF_CMAP_SINGLE, 2242 },
	{ 0x6a3b, 0x6a3b, PDF_CMAP_SINGLE, 2243 },
	{ 0x6a3e, 0x6a3f, PDF_CMAP_RANGE, 2244 },
	{ 0x6a45, 0x6a46, PDF_CMAP_RANGE, 2246 },
	{ 0x6a49, 0x6a4a, PDF_CMAP_RANGE, 2248 },
	{ 0x6a4e, 0x6a4e, PDF_CMAP_SINGLE, 2250 },
	{ 0x6a50, 0x6a52, PDF_CMAP_RANGE, 2251 },
	{ 0x6a55, 0x6a56, PDF_CMAP_RANGE, 2254 },
	{ 0x6a5b, 0x6a5b, PDF_CMAP_SINGLE, 2256 },
	{ 0x6a64, 0x6a64, PDF_CMAP_SINGLE, 2257 },
	{ 0x6a67, 0x6a67, PDF_CMAP_SINGLE, 2258 },
	{ 0x6a6a, 0x6a6a, PDF_CMAP_SINGLE, 2259 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 2260 },
	{ 0x6a73, 0x6a73, PDF_CMAP_SINGLE, 2261 },
	{ 0x6a7e, 0x6a7e, PDF_CMAP_SINGLE, 2262 },
	{ 0x6a81, 0x6a81, PDF_CMAP_SINGLE, 2263 },
	{ 0x6a83, 0x6a83, PDF_CMAP_SINGLE, 2264 },
	{ 0x6a86, 0x6a87, PDF_CMAP_RANGE, 2265 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 2267 },
	{ 0x6a8b, 0x6a8b, PDF_CMAP_SINGLE, 2268 },
	{ 0x6a91, 0x6a91, PDF_CMAP_SINGLE, 2269 },
	{ 0x6a9b, 0x6a9b, PDF_CMAP_SINGLE, 2270 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 2271 },
	{ 0x6aa5, 0x6aa5, PDF_CMAP_SINGLE, 2274 },
	{ 0x6aab, 0x6aab, PDF_CMAP_SINGLE, 2275 },
	{ 0x6aaf, 0x6ab1, PDF_CMAP_RANGE, 2276 },
	{ 0x6ab4, 0x6ab4, PDF_CMAP_SINGLE, 2279 },
	{ 0x6abd, 0x6abf, PDF_CMAP_RANGE, 2280 },
	{ 0x6ac6, 0x6ac6, PDF_CMAP_SINGLE, 2283 },
	{ 0x6ac8, 0x6ac9, PDF_CMAP_TABLE, 246 },
	{ 0x6acc, 0x6acc, PDF_CMAP_SINGLE, 2286 },
	{ 0x6ad0, 0x6ad0, PDF_CMAP_SINGLE, 2287 },
	{ 0x6ad4, 0x6ad6, PDF_CMAP_RANGE, 2288 },
	{ 0x6adc, 0x6add, PDF_CMAP_RANGE, 2291 },
	{ 0x6ae4, 0x6ae4, PDF_CMAP_SINGLE, 2293 },
	{ 0x6ae7, 0x6ae7, PDF_CMAP_SINGLE, 2294 },
	{ 0x6aec, 0x6aec, PDF_CMAP_SINGLE, 2295 },
	{ 0x6af0, 0x6af2, PDF_CMAP_RANGE, 2296 },
	{ 0x6afc, 0x6afd, PDF_CMAP_RANGE, 2299 },
	{ 0x6b02, 0x6b03, PDF_CMAP_RANGE, 2301 },
	{ 0x6b06, 0x6b07, PDF_CMAP_RANGE, 2303 },
	{ 0x6b09, 0x6b09, PDF_CMAP_SINGLE, 2305 },
	{ 0x6b0f, 0x6b11, PDF_CMAP_RANGE, 2306 },
	{ 0x6b17, 0x6b17, PDF_CMAP_SINGLE, 2309 },
	{ 0x6b1b, 0x6b1b, PDF_CMAP_SINGLE, 2310 },
	{ 0x6b1e, 0x6b1e, PDF_CMAP_SINGLE, 2311 },
	{ 0x6b24, 0x6b24, PDF_CMAP_SINGLE, 2312 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 2313 },
	{ 0x6b2b, 0x6b2c, PDF_CMAP_RANGE, 2314 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 2316 },
	{ 0x6b35, 0x6b36, PDF_CMAP_RANGE, 2317 },
	{ 0x6b3b, 0x6b3b, PDF_CMAP_SINGLE, 2319 },
	{ 0x6b3f, 0x6b3f, PDF_CMAP_SINGLE, 2320 },
	{ 0x6b46, 0x6b46, PDF_CMAP_SINGLE, 2321 },
	{ 0x6b4a, 0x6b4a, PDF_CMAP_SINGLE, 2322 },
	{ 0x6b4d, 0x6b4d, PDF_CMAP_SINGLE, 2323 },
	{ 0x6b52, 0x6b52, PDF_CMAP_SINGLE, 2324 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 2325 },
	{ 0x6b58, 0x6b58, PDF_CMAP_SINGLE, 2326 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 2327 },
	{ 0x6b60, 0x6b60, PDF_CMAP_SINGLE, 2328 },
	{ 0x6b67, 0x6b67, PDF_CMAP_SINGLE, 2329 },
	{ 0x6b6b, 0x6b6b, PDF_CMAP_SINGLE, 2330 },
	{ 0x6b6e, 0x6b6e, PDF_CMAP_SINGLE, 2331 },
	{ 0x6b70, 0x6b70, PDF_CMAP_SINGLE, 2332 },
	{ 0x6b75, 0x6b75, PDF_CMAP_SINGLE, 2333 },
	{ 0x6b7d, 0x6b7e, PDF_CMAP_RANGE, 2334 },
	{ 0x6b82, 0x6b82, PDF_CMAP_SINGLE, 2336 },
	{ 0x6b85, 0x6b85, PDF_CMAP_SINGLE, 2337 },
	{ 0x6b97, 0x6b97, PDF_CMAP_SINGLE, 2338 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 2339 },
	{ 0x6b9f, 0x6ba0, PDF_CMAP_RANGE, 2340 },
	{ 0x6ba2, 0x6ba3, PDF_CMAP_RANGE, 2342 },
	{ 0x6ba8, 0x6ba9, PDF_CMAP_RANGE, 2344 },
	{ 0x6bac, 0x6bae, PDF_CMAP_RANGE, 2346 },
	{ 0x6bb0, 0x6bb0, PDF_CMAP_SINGLE, 2349 },
	{ 0x6bb8, 0x6bb9, PDF_CMAP_RANGE, 2350 },
	{ 0x6bbd, 0x6bbe, PDF_CMAP_RANGE, 2352 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 2354 },
	{ 0x6bc9, 0x6bc9, PDF_CMAP_SINGLE, 2356 },
	{ 0x6bcc, 0x6bcc, PDF_CMAP_SINGLE, 2357 },
	{ 0x6bd6, 0x6bd6, PDF_CMAP_SINGLE, 2358 },
	{ 0x6bda, 0x6bda, PDF_CMAP_SINGLE, 2359 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 2360 },
	{ 0x6be3, 0x6be3, PDF_CMAP_SINGLE, 2361 },
	{ 0x6be6, 0x6be7, PDF_CMAP_RANGE, 2362 },
	{ 0x6bee, 0x6bee, PDF_CMAP_SINGLE, 2364 },
	{ 0x6bf1, 0x6bf1, PDF_CMAP_SINGLE, 2365 },
	{ 0x6bf7, 0x6bf7, PDF_CMAP_SINGLE, 2366 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 2367 },
	{ 0x6bff, 0x6bff, PDF_CMAP_SINGLE, 2368 },
	{ 0x6c02, 0x6c02, PDF_CMAP_SINGLE, 2369 },
	{ 0x6c04, 0x6c05, PDF_CMAP_RANGE, 2370 },
	{ 0x6c09, 0x6c09, PDF_CMAP_SINGLE, 2372 },
	{ 0x6c0d, 0x6c0e, PDF_CMAP_RANGE, 2373 },
	{ 0x6c10, 0x6c10, PDF_CMAP_SINGLE, 2375 },
	{ 0x6c12, 0x6c12, PDF_CMAP_SINGLE, 2376 },
	{ 0x6c19, 0x6c19, PDF_CMAP_SINGLE, 2377 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 2378 },
	{ 0x6c26, 0x6c28, PDF_CMAP_RANGE, 2379 },
	{ 0x6c2c, 0x6c2c, PDF_CMAP_SINGLE, 2382 },
	{ 0x6c2e, 0x6c2e, PDF_CMAP_SINGLE, 2383 },
	{ 0x6c33, 0x6c33, PDF_CMAP_SINGLE, 2384 },
	{ 0x6c35, 0x6c36, PDF_CMAP_RANGE, 2385 },
	{ 0x6c3a, 0x6c3b, PDF_CMAP_RANGE, 2387 },
	{ 0x6c3f, 0x6c3f, PDF_CMAP_SINGLE, 2389 },
	{ 0x6c4a, 0x6c4b, PDF_CMAP_RANGE, 2390 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 2392 },
	{ 0x6c4f, 0x6c4f, PDF_CMAP_SINGLE, 2393 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 2394 },
	{ 0x6c54, 0x6c54, PDF_CMAP_SINGLE, 2395 },
	{ 0x6c59, 0x6c59, PDF_CMAP_SINGLE, 2396 },
	{ 0x6c5b, 0x6c5c, PDF_CMAP_RANGE, 2397 },
	{ 0x6c67, 0x6c67, PDF_CMAP_SINGLE, 2450 },
	{ 0x6c6b, 0x6c6b, PDF_CMAP_SINGLE, 2399 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 2400 },
	{ 0x6c6f, 0x6c6f, PDF_CMAP_SINGLE, 2401 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 2402 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 2403 },
	{ 0x6c78, 0x6c79, PDF_CMAP_RANGE, 2404 },
	{ 0x6c7b, 0x6c7b, PDF_CMAP_SINGLE, 2406 },
	{ 0x6c85, 0x6c87, PDF_CMAP_RANGE, 2407 },
	{ 0x6c89, 0x6c89, PDF_CMAP_SINGLE, 2410 },
	{ 0x6c94, 0x6c95, PDF_CMAP_RANGE, 2411 },
	{ 0x6c97, 0x6c98, PDF_CMAP_RANGE, 2413 },
	{ 0x6c9c, 0x6c9c, PDF_CMAP_SINGLE, 2415 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 2416 },
	{ 0x6cb0, 0x6cb0, PDF_CMAP_SINGLE, 2417 },
	{ 0x6cb2, 0x6cb2, PDF_CMAP_SINGLE, 2418 },
	{ 0x6cb4, 0x6cb4, PDF_CMAP_SINGLE, 2419 },
	{ 0x6cc2, 0x6cc2, PDF_CMAP_SINGLE, 2420 },
	{ 0x6cc6, 0x6cc6, PDF_CMAP_SINGLE, 2421 },
	{ 0x6ccd, 0x6ccd, PDF_CMAP_SINGLE, 2422 },
	{ 0x6ccf, 0x6cd2, PDF_CMAP_RANGE, 2423 },
	{ 0x6cd4, 0x6cd4, PDF_CMAP_SINGLE, 2427 },
	{ 0x6cd6, 0x6cd6, PDF_CMAP_SINGLE, 2428 },
	{ 0x6cda, 0x6cda, PDF_CMAP_SINGLE, 2429 },
	{ 0x6cdc, 0x6cdc, PDF_CMAP_SINGLE, 2430 },
	{ 0x6ce0, 0x6ce0, PDF_CMAP_SINGLE, 2431 },
	{ 0x6ce7, 0x6ce7, PDF_CMAP_SINGLE, 2432 },
	{ 0x6ce9, 0x6ce9, PDF_CMAP_SINGLE, 2433 },
	{ 0x6ceb, 0x6cec, PDF_CMAP_RANGE, 2434 },
	{ 0x6cee, 0x6cee, PDF_CMAP_SINGLE, 2436 },
	{ 0x6cf2, 0x6cf2, PDF_CMAP_SINGLE, 2437 },
	{ 0x6cf4, 0x6cf4, PDF_CMAP_SINGLE, 2438 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 2439 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 2440 },
	{ 0x6d0a, 0x6d0a, PDF_CMAP_SINGLE, 2441 },
	{ 0x6d0e, 0x6d0f, PDF_CMAP_RANGE, 2442 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 2444 },
	{ 0x6d13, 0x6d13, PDF_CMAP_SINGLE, 2445 },
	{ 0x6d1a, 0x6d1a, PDF_CMAP_SINGLE, 2446 },
	{ 0x6d26, 0x6d28, PDF_CMAP_RANGE, 2447 },
	{ 0x6d2e, 0x6d2f, PDF_CMAP_RANGE, 2451 },
	{ 0x6d31, 0x6d31, PDF_CMAP_SINGLE, 2453 },
	{ 0x6d39, 0x6d39, PDF_CMAP_SINGLE, 2454 },
	{ 0x6d3c, 0x6d3c, PDF_CMAP_SINGLE, 2455 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 2456 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 2457 },
	{ 0x6d5e, 0x6d5f, PDF_CMAP_RANGE, 2458 },
	{ 0x6d61, 0x6d61, PDF_CMAP_SINGLE, 2460 },
	{ 0x6d65, 0x6d65, PDF_CMAP_SINGLE, 2461 },
	{ 0x6d67, 0x6d67, PDF_CMAP_SINGLE, 2462 },
	{ 0x6d6f, 0x6d70, PDF_CMAP_RANGE, 2463 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 2465 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 2466 },
	{ 0x6d87, 0x6d87, PDF_CMAP_SINGLE, 2467 },
	{ 0x6d91, 0x6d92, PDF_CMAP_RANGE, 2468 },
	{ 0x6d94, 0x6d94, PDF_CMAP_SINGLE, 2470 },
	{ 0x6d96, 0x6d98, PDF_CMAP_RANGE, 2471 },
	{ 0x6daa, 0x6daa, PDF_CMAP_SINGLE, 2474 },
	{ 0x6dac, 0x6dac, PDF_CMAP_SINGLE, 2475 },
	{ 0x6db4, 0x6db4, PDF_CMAP_SINGLE, 2476 },
	{ 0x6db7, 0x6db7, PDF_CMAP_SINGLE, 2477 },
	{ 0x6db9, 0x6db9, PDF_CMAP_SINGLE, 2478 },
	{ 0x6dbd, 0x6dbd, PDF_CMAP_SINGLE, 2479 },
	{ 0x6dbf, 0x6dbf, PDF_CMAP_SINGLE, 2480 },
	{ 0x6dc4, 0x6dc4, PDF_CMAP_SINGLE, 2481 },
	{ 0x6dc8, 0x6dc8, PDF_CMAP_SINGLE, 2482 },
	{ 0x6dca, 0x6dca, PDF_CMAP_SINGLE, 2483 },
	{ 0x6dce, 0x6dcf, PDF_CMAP_RANGE, 2484 },
	{ 0x6dd6, 0x6dd6, PDF_CMAP_SINGLE, 2486 },
	{ 0x6ddb, 0x6ddb, PDF_CMAP_SINGLE, 2487 },
	{ 0x6ddd, 0x6ddd, PDF_CMAP_SINGLE, 2488 },
	{ 0x6ddf, 0x6de0, PDF_CMAP_RANGE, 2489 },
	{ 0x6de2, 0x6de2, PDF_CMAP_SINGLE, 2491 },
	{ 0x6de5, 0x6de5, PDF_CMAP_SINGLE, 2492 },
	{ 0x6de9, 0x6de9, PDF_CMAP_SINGLE, 2493 },
	{ 0x6def, 0x6df0, PDF_CMAP_RANGE, 2494 },
	{ 0x6df4, 0x6df4, PDF_CMAP_SINGLE, 2496 },
	{ 0x6df6, 0x6df6, PDF_CMAP_SINGLE, 2497 },
	{ 0x6dfc, 0x6dfc, PDF_CMAP_SINGLE, 2498 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 2499 },
	{ 0x6e04, 0x6e04, PDF_CMAP_SINGLE, 2500 },
	{ 0x6e1e, 0x6e1e, PDF_CMAP_SINGLE, 2501 },
	{ 0x6e22, 0x6e22, PDF_CMAP_SINGLE, 2502 },
	{ 0x6e27, 0x6e27, PDF_CMAP_SINGLE, 2503 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 2504 },
	{ 0x6e36, 0x6e36, PDF_CMAP_SINGLE, 2505 },
	{ 0x6e39, 0x6e39, PDF_CMAP_SINGLE, 2506 },
	{ 0x6e3b, 0x6e3c, PDF_CMAP_RANGE, 2507 },
	{ 0x6e44, 0x6e45, PDF_CMAP_RANGE, 2509 },
	{ 0x6e48, 0x6e49, PDF_CMAP_RANGE, 2511 },
	{ 0x6e4b, 0x6e4b, PDF_CMAP_SINGLE, 2513 },
	{ 0x6e4f, 0x6e4f, PDF_CMAP_SINGLE, 2514 },
	{ 0x6e51, 0x6e54, PDF_CMAP_RANGE, 2515 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 2519 },
	{ 0x6e5c, 0x6e5e, PDF_CMAP_RANGE, 2520 },
	{ 0x6e62, 0x6e63, PDF_CMAP_RANGE, 2523 },
	{ 0x6e68, 0x6e68, PDF_CMAP_SINGLE, 2525 },
	{ 0x6e73, 0x6e73, PDF_CMAP_SINGLE, 2526 },
	{ 0x6e7b, 0x6e7b, PDF_CMAP_SINGLE, 2527 },
	{ 0x6e7d, 0x6e7d, PDF_CMAP_SINGLE, 2528 },
	{ 0x6e8d, 0x6e8d, PDF_CMAP_SINGLE, 2529 },
	{ 0x6e93, 0x6e93, PDF_CMAP_SINGLE, 2530 },
	{ 0x6e99, 0x6e99, PDF_CMAP_SINGLE, 2531 },
	{ 0x6ea0, 0x6ea0, PDF_CMAP_SINGLE, 2532 },
	{ 0x6ea7, 0x6ea7, PDF_CMAP_SINGLE, 2533 },
	{ 0x6ead, 0x6eae, PDF_CMAP_RANGE, 2534 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 2536 },
	{ 0x6eb3, 0x6eb3, PDF_CMAP_SINGLE, 2537 },
	{ 0x6ebb, 0x6ebb, PDF_CMAP_SINGLE, 2538 },
	{ 0x6ebf, 0x6ec1, PDF_CMAP_RANGE, 2539 },
	{ 0x6ec3, 0x6ec3, PDF_CMAP_SINGLE, 2542 },
	{ 0x6ec7, 0x6ec8, PDF_CMAP_RANGE, 2543 },
	{ 0x6eca, 0x6eca, PDF_CMAP_SINGLE, 2545 },
	{ 0x6ecd, 0x6ecf, PDF_CMAP_RANGE, 2546 },
	{ 0x6eeb, 0x6eeb, PDF_CMAP_SINGLE, 2549 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 2550 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 2552 },
	{ 0x6efb, 0x6efb, PDF_CMAP_SINGLE, 2553 },
	{ 0x6efd, 0x6efd, PDF_CMAP_SINGLE, 2554 },
	{ 0x6f04, 0x6f04, PDF_CMAP_SINGLE, 2555 },
	{ 0x6f08, 0x6f08, PDF_CMAP_SINGLE, 2556 },
	{ 0x6f0a, 0x6f0a, PDF_CMAP_SINGLE, 2557 },
	{ 0x6f0c, 0x6f0d, PDF_CMAP_RANGE, 2558 },
	{ 0x6f16, 0x6f16, PDF_CMAP_SINGLE, 2560 },
	{ 0x6f18, 0x6f18, PDF_CMAP_SINGLE, 2561 },
	{ 0x6f1a, 0x6f1b, PDF_CMAP_RANGE, 2562 },
	{ 0x6f26, 0x6f26, PDF_CMAP_SINGLE, 2564 },
	{ 0x6f29, 0x6f2a, PDF_CMAP_RANGE, 2565 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 2573 },
	{ 0x6f2f, 0x6f30, PDF_CMAP_RANGE, 2567 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 2569 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 2570 },
	{ 0x6f3b, 0x6f3c, PDF_CMAP_RANGE, 2571 },
	{ 0x6f4f, 0x6f4f, PDF_CMAP_SINGLE, 2574 },
	{ 0x6f51, 0x6f53, PDF_CMAP_RANGE, 2575 },
	{ 0x6f57, 0x6f57, PDF_CMAP_SINGLE, 2578 },
	{ 0x6f59, 0x6f5a, PDF_CMAP_RANGE, 2579 },
	{ 0x6f5d, 0x6f5e, PDF_CMAP_RANGE, 2581 },
	{ 0x6f61, 0x6f62, PDF_CMAP_RANGE, 2583 },
	{ 0x6f68, 0x6f68, PDF_CMAP_SINGLE, 2585 },
	{ 0x6f6c, 0x6f6c, PDF_CMAP_SINGLE, 2586 },
	{ 0x6f7d, 0x6f7e, PDF_CMAP_RANGE, 2587 },
	{ 0x6f83, 0x6f83, PDF_CMAP_SINGLE, 2589 },
	{ 0x6f87, 0x6f88, PDF_CMAP_RANGE, 2590 },
	{ 0x6f8b, 0x6f8d, PDF_CMAP_RANGE, 2592 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 2595 },
	{ 0x6f92, 0x6f94, PDF_CMAP_RANGE, 2596 },
	{ 0x6f96, 0x6f96, PDF_CMAP_SINGLE, 2599 },
	{ 0x6f9a, 0x6f9a, PDF_CMAP_SINGLE, 2600 },
	{ 0x6f9f, 0x6fa0, PDF_CMAP_RANGE, 2601 },
	{ 0x6fa5, 0x6fa8, PDF_CMAP_RANGE, 2603 },
	{ 0x6fae, 0x6fb0, PDF_CMAP_RANGE, 2607 },
	{ 0x6fb5, 0x6fb6, PDF_CMAP_RANGE, 2610 },
	{ 0x6fbc, 0x6fbc, PDF_CMAP_SINGLE, 2612 },
	{ 0x6fc5, 0x6fc5, PDF_CMAP_SINGLE, 2613 },
	{ 0x6fc7, 0x6fc8, PDF_CMAP_RANGE, 2614 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 2616 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 2617 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 2618 },
	{ 0x6fe8, 0x6fe9, PDF_CMAP_RANGE, 2619 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 2621 },
	{ 0x6ff5, 0x6ff5, PDF_CMAP_SINGLE, 2622 },
	{ 0x6ff9, 0x6ff9, PDF_CMAP_SINGLE, 2623 },
	{ 0x6ffc, 0x6ffd, PDF_CMAP_RANGE, 2624 },
	{ 0x7000, 0x7000, PDF_CMAP_SINGLE, 2626 },
	{ 0x7005, 0x7007, PDF_CMAP_RANGE, 2627 },
	{ 0x700d, 0x700d, PDF_CMAP_SINGLE, 2630 },
	{ 0x7017, 0x7017, PDF_CMAP_SINGLE, 2631 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 2632 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 2633 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 2634 },
	{ 0x7034, 0x7034, PDF_CMAP_SINGLE, 2635 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 2636 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 2637 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 2638 },
	{ 0x7043, 0x7044, PDF_CMAP_RANGE, 2639 },
	{ 0x7048, 0x704b, PDF_CMAP_RANGE, 2641 },
	{ 0x704e, 0x704e, PDF_CMAP_SINGLE, 2649 },
	{ 0x7054, 0x7055, PDF_CMAP_RANGE, 2645 },
	{ 0x705d, 0x705e, PDF_CMAP_RANGE, 2647 },
	{ 0x7064, 0x7065, PDF_CMAP_RANGE, 2650 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 2652 },
	{ 0x706e, 0x706e, PDF_CMAP_SINGLE, 2653 },
	{ 0x7075, 0x7076, PDF_CMAP_RANGE, 2654 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 2656 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 2657 },
	{ 0x7085, 0x7086, PDF_CMAP_RANGE, 2658 },
	{ 0x7094, 0x7098, PDF_CMAP_RANGE, 2660 },
	{ 0x709b, 0x709b, PDF_CMAP_SINGLE, 2665 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 2666 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 2667 },
	{ 0x70b0, 0x70b1, PDF_CMAP_RANGE, 2668 },
	{ 0x70b4, 0x70b4, PDF_CMAP_SINGLE, 2670 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 2671 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 2672 },
	{ 0x70d1, 0x70d1, PDF_CMAP_SINGLE, 2673 },
	{ 0x70d3, 0x70d6, PDF_CMAP_RANGE, 2674 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 2678 },
	{ 0x70dc, 0x70dc, PDF_CMAP_SINGLE, 2679 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 2680 },
	{ 0x70fa, 0x70fa, PDF_CMAP_SINGLE, 2681 },
	{ 0x7103, 0x7107, PDF_CMAP_RANGE, 2682 },
	{ 0x710b, 0x710c, PDF_CMAP_RANGE, 2687 },
	{ 0x710f, 0x710f, PDF_CMAP_SINGLE, 2689 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 2690 },
	{ 0x7120, 0x7120, PDF_CMAP_SINGLE, 2691 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 2692 },
	{ 0x712d, 0x712d, PDF_CMAP_SINGLE, 2693 },
	{ 0x712f, 0x7131, PDF_CMAP_RANGE, 2694 },
	{ 0x7138, 0x7138, PDF_CMAP_SINGLE, 2697 },
	{ 0x7141, 0x7141, PDF_CMAP_SINGLE, 2698 },
	{ 0x7145, 0x7147, PDF_CMAP_RANGE, 2699 },
	{ 0x714a, 0x714b, PDF_CMAP_RANGE, 2702 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 2704 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 2705 },
	{ 0x7157, 0x7157, PDF_CMAP_SINGLE, 2706 },
	{ 0x715a, 0x715a, PDF_CMAP_SINGLE, 2707 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 2708 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 2709 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 2710 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 2711 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 2712 },
	{ 0x7180, 0x7180, PDF_CMAP_SINGLE, 2713 },
	{ 0x7185, 0x7185, PDF_CMAP_SINGLE, 2714 },
	{ 0x7187, 0x7187, PDF_CMAP_SINGLE, 2715 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 2716 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 2717 },
	{ 0x719a, 0x719b, PDF_CMAP_RANGE, 2718 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 2720 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 2721 },
	{ 0x71af, 0x71b0, PDF_CMAP_RANGE, 2722 },
	{ 0x71b2, 0x71b3, PDF_CMAP_RANGE, 2724 },
	{ 0x71ba, 0x71ba, PDF_CMAP_SINGLE, 2726 },
	{ 0x71bf, 0x71c1, PDF_CMAP_RANGE, 2727 },
	{ 0x71c4, 0x71c4, PDF_CMAP_SINGLE, 2730 },
	{ 0x71cb, 0x71cc, PDF_CMAP_RANGE, 2731 },
	{ 0x71d3, 0x71d3, PDF_CMAP_SINGLE, 2733 },
	{ 0x71d6, 0x71d6, PDF_CMAP_SINGLE, 2734 },
	{ 0x71d9, 0x71da, PDF_CMAP_RANGE, 2735 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 2737 },
	{ 0x71f8, 0x71f8, PDF_CMAP_SINGLE, 2738 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 2739 },
	{ 0x7200, 0x7200, PDF_CMAP_SINGLE, 2740 },
	{ 0x7207, 0x7209, PDF_CMAP_RANGE, 2741 },
	{ 0x7213, 0x7213, PDF_CMAP_SINGLE, 2744 },
	{ 0x7217, 0x7217, PDF_CMAP_SINGLE, 2745 },
	{ 0x721a, 0x721a, PDF_CMAP_SINGLE, 2746 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 2747 },
	{ 0x721f, 0x721f, PDF_CMAP_SINGLE, 2748 },
	{ 0x7224, 0x7224, PDF_CMAP_SINGLE, 2749 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 2750 },
	{ 0x722f, 0x722f, PDF_CMAP_SINGLE, 2751 },
	{ 0x7234, 0x7234, PDF_CMAP_SINGLE, 2752 },
	{ 0x7238, 0x7239, PDF_CMAP_RANGE, 2753 },
	{ 0x7241, 0x7243, PDF_CMAP_RANGE, 2755 },
	{ 0x7245, 0x7245, PDF_CMAP_SINGLE, 2758 },
	{ 0x724e, 0x7250, PDF_CMAP_RANGE, 2759 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 2762 },
	{ 0x7255, 0x7256, PDF_CMAP_RANGE, 2763 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 2765 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 2766 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 2767 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 2768 },
	{ 0x7263, 0x7263, PDF_CMAP_SINGLE, 2769 },
	{ 0x7268, 0x7268, PDF_CMAP_SINGLE, 2770 },
	{ 0x726b, 0x726b, PDF_CMAP_SINGLE, 2771 },
	{ 0x726e, 0x726f, PDF_CMAP_RANGE, 2772 },
	{ 0x7271, 0x7271, PDF_CMAP_SINGLE, 2774 },
	{ 0x7277, 0x7278, PDF_CMAP_RANGE, 2775 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 2777 },
	{ 0x727f, 0x727f, PDF_CMAP_SINGLE, 2779 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 2780 },
	{ 0x7289, 0x7289, PDF_CMAP_SINGLE, 2781 },
	{ 0x728d, 0x728e, PDF_CMAP_RANGE, 2782 },
	{ 0x7293, 0x7293, PDF_CMAP_SINGLE, 2784 },
	{ 0x729b, 0x729b, PDF_CMAP_SINGLE, 2785 },
	{ 0x72a8, 0x72a8, PDF_CMAP_SINGLE, 2786 },
	{ 0x72ad, 0x72ae, PDF_CMAP_RANGE, 2787 },
	{ 0x72b1, 0x72b1, PDF_CMAP_SINGLE, 2789 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 2790 },
	{ 0x72be, 0x72be, PDF_CMAP_SINGLE, 2791 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 2792 },
	{ 0x72c7, 0x72c7, PDF_CMAP_SINGLE, 2793 },
	{ 0x72c9, 0x72c9, PDF_CMAP_SINGLE, 2794 },
	{ 0x72cc, 0x72cc, PDF_CMAP_SINGLE, 2795 },
	{ 0x72d5, 0x72d6, PDF_CMAP_RANGE, 2796 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 2798 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 2799 },
	{ 0x72e5, 0x72e5, PDF_CMAP_SINGLE, 2800 },
	{ 0x72f3, 0x72f4, PDF_CMAP_RANGE, 2801 },
	{ 0x72fa, 0x72fb, PDF_CMAP_RANGE, 2803 },
	{ 0x72fe, 0x72fe, PDF_CMAP_SINGLE, 2805 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 2806 },
	{ 0x7304, 0x7305, PDF_CMAP_RANGE, 2807 },
	{ 0x7307, 0x7307, PDF_CMAP_SINGLE, 2809 },
	{ 0x730b, 0x730b, PDF_CMAP_SINGLE, 2810 },
	{ 0x730d, 0x730d, PDF_CMAP_SINGLE, 2811 },
	{ 0x7312, 0x7313, PDF_CMAP_RANGE, 2812 },
	{ 0x7318, 0x7319, PDF_CMAP_RANGE, 2814 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 2816 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 2817 },
	{ 0x7324, 0x7324, PDF_CMAP_SINGLE, 2818 },
	{ 0x7327, 0x7328, PDF_CMAP_RANGE, 2819 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 2821 },
	{ 0x7331, 0x7332, PDF_CMAP_RANGE, 2822 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 2824 },
	{ 0x733a, 0x733b, PDF_CMAP_RANGE, 2825 },
	{ 0x733d, 0x733d, PDF_CMAP_SINGLE, 2827 },
	{ 0x7343, 0x7343, PDF_CMAP_SINGLE, 2828 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 2829 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 2830 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 2831 },
	{ 0x7356, 0x7356, PDF_CMAP_SINGLE, 2832 },
	{ 0x7358, 0x7358, PDF_CMAP_SINGLE, 2833 },
	{ 0x735d, 0x7360, PDF_CMAP_RANGE, 2834 },
	{ 0x7366, 0x7367, PDF_CMAP_RANGE, 2838 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 2840 },
	{ 0x736b, 0x736c, PDF_CMAP_RANGE, 2841 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 2843 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 2845 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 2846 },
	{ 0x7379, 0x7379, PDF_CMAP_SINGLE, 2847 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 2848 },
	{ 0x7380, 0x7381, PDF_CMAP_RANGE, 2849 },
	{ 0x7383, 0x7383, PDF_CMAP_SINGLE, 2851 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 2852 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 2854 },
	{ 0x7390, 0x7390, PDF_CMAP_SINGLE, 2855 },
	{ 0x7393, 0x7393, PDF_CMAP_SINGLE, 2856 },
	{ 0x7395, 0x7395, PDF_CMAP_SINGLE, 2857 },
	{ 0x7397, 0x7398, PDF_CMAP_RANGE, 2858 },
	{ 0x739c, 0x739c, PDF_CMAP_SINGLE, 2860 },
	{ 0x739e, 0x73a0, PDF_CMAP_RANGE, 2861 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 2864 },
	{ 0x73a5, 0x73a6, PDF_CMAP_RANGE, 2865 },
	{ 0x73aa, 0x73ab, PDF_CMAP_RANGE, 2867 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 2869 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 2870 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 2871 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 2872 },
	{ 0x73bc, 0x73bd, PDF_CMAP_RANGE, 2873 },
	{ 0x73bf, 0x73bf, PDF_CMAP_SINGLE, 2875 },
	{ 0x73c5, 0x73c6, PDF_CMAP_RANGE, 2876 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 2878 },
	{ 0x73cb, 0x73cc, PDF_CMAP_RANGE, 2879 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 2881 },
	{ 0x73d2, 0x73d3, PDF_CMAP_RANGE, 2882 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 2884 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 2885 },
	{ 0x73dd, 0x73dd, PDF_CMAP_SINGLE, 2886 },
	{ 0x73e1, 0x73e1, PDF_CMAP_SINGLE, 2887 },
	{ 0x73e3, 0x73e3, PDF_CMAP_SINGLE, 2888 },
	{ 0x73e6, 0x73e7, PDF_CMAP_RANGE, 2889 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 2891 },
	{ 0x73f4, 0x73f5, PDF_CMAP_RANGE, 2892 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 2894 },
	{ 0x73f9, 0x73fb, PDF_CMAP_RANGE, 2895 },
	{ 0x73fd, 0x73fd, PDF_CMAP_SINGLE, 2898 },
	{ 0x73ff, 0x7401, PDF_CMAP_RANGE, 2899 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 2902 },
	{ 0x7407, 0x7407, PDF_CMAP_SINGLE, 2903 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 2904 },
	{ 0x7411, 0x7411, PDF_CMAP_SINGLE, 2905 },
	{ 0x741a, 0x741b, PDF_CMAP_RANGE, 2906 },
	{ 0x7424, 0x7424, PDF_CMAP_SINGLE, 2908 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 2909 },
	{ 0x7428, 0x7431, PDF_CMAP_RANGE, 2910 },
	{ 0x7439, 0x7439, PDF_CMAP_SINGLE, 2920 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 2921 },
	{ 0x7443, 0x7444, PDF_CMAP_RANGE, 2922 },
	{ 0x7446, 0x7447, PDF_CMAP_RANGE, 2924 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 2926 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 2927 },
	{ 0x7451, 0x7452, PDF_CMAP_RANGE, 2928 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 2930 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 2931 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 2932 },
	{ 0x7466, 0x7468, PDF_CMAP_RANGE, 2933 },
	{ 0x746b, 0x746b, PDF_CMAP_SINGLE, 2936 },
	{ 0x746d, 0x746e, PDF_CMAP_RANGE, 2937 },
	{ 0x7471, 0x7472, PDF_CMAP_RANGE, 2939 },
	{ 0x7480, 0x7481, PDF_CMAP_RANGE, 2941 },
	{ 0x7485, 0x7487, PDF_CMAP_RANGE, 2943 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 2946 },
	{ 0x748f, 0x7492, PDF_CMAP_RANGE, 2947 },
	{ 0x7498, 0x749a, PDF_CMAP_RANGE, 2951 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 2954 },
	{ 0x749f, 0x74a1, PDF_CMAP_RANGE, 2955 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 2958 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 2959 },
	{ 0x74a8, 0x74ab, PDF_CMAP_RANGE, 2960 },
	{ 0x74ae, 0x74af, PDF_CMAP_RANGE, 2964 },
	{ 0x74b1, 0x74b2, PDF_CMAP_RANGE, 2966 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 2968 },
	{ 0x74b9, 0x74b9, PDF_CMAP_SINGLE, 2969 },
	{ 0x74bb, 0x74bb, PDF_CMAP_SINGLE, 2970 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 2971 },
	{ 0x74c8, 0x74c9, PDF_CMAP_RANGE, 2972 },
	{ 0x74cc, 0x74cc, PDF_CMAP_SINGLE, 2974 },
	{ 0x74d0, 0x74d0, PDF_CMAP_SINGLE, 2975 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 2976 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 2977 },
	{ 0x74da, 0x74db, PDF_CMAP_RANGE, 2978 },
	{ 0x74de, 0x74df, PDF_CMAP_RANGE, 2980 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 2982 },
	{ 0x74e8, 0x74e8, PDF_CMAP_SINGLE, 2983 },
	{ 0x74ea, 0x74eb, PDF_CMAP_RANGE, 2984 },
	{ 0x74ef, 0x74ef, PDF_CMAP_SINGLE, 2986 },
	{ 0x74f4, 0x74f4, PDF_CMAP_SINGLE, 2987 },
	{ 0x74fa, 0x74fc, PDF_CMAP_RANGE, 2988 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 2991 },
	{ 0x7506, 0x7506, PDF_CMAP_SINGLE, 2992 },
	{ 0x7512, 0x7512, PDF_CMAP_SINGLE, 2993 },
	{ 0x7516, 0x7517, PDF_CMAP_RANGE, 2994 },
	{ 0x7520, 0x7521, PDF_CMAP_RANGE, 2996 },
	{ 0x7524, 0x7524, PDF_CMAP_SINGLE, 2998 },
	{ 0x7527, 0x7527, PDF_CMAP_SINGLE, 2999 },
	{ 0x7529, 0x752a, PDF_CMAP_RANGE, 3000 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 3002 },
	{ 0x7536, 0x7536, PDF_CMAP_SINGLE, 3003 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 3004 },
	{ 0x753d, 0x7540, PDF_CMAP_RANGE, 3005 },
	{ 0x7543, 0x7543, PDF_CMAP_SINGLE, 3009 },
	{ 0x7547, 0x7548, PDF_CMAP_RANGE, 3010 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 3012 },
	{ 0x7550, 0x7550, PDF_CMAP_SINGLE, 3013 },
	{ 0x7552, 0x7552, PDF_CMAP_SINGLE, 3014 },
	{ 0x7557, 0x7557, PDF_CMAP_SINGLE, 3015 },
	{ 0x755e, 0x755f, PDF_CMAP_RANGE, 3016 },
	{ 0x7561, 0x7561, PDF_CMAP_SINGLE, 3018 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 3019 },
	{ 0x7571, 0x7571, PDF_CMAP_SINGLE, 3020 },
	{ 0x7579, 0x757e, PDF_CMAP_RANGE, 3021 },
	{ 0x7581, 0x7581, PDF_CMAP_SINGLE, 3027 },
	{ 0x7585, 0x7585, PDF_CMAP_SINGLE, 3028 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 3029 },
	{ 0x7592, 0x7593, PDF_CMAP_RANGE, 3030 },
	{ 0x7595, 0x7595, PDF_CMAP_SINGLE, 3032 },
	{ 0x7599, 0x7599, PDF_CMAP_SINGLE, 3033 },
	{ 0x759c, 0x759c, PDF_CMAP_SINGLE, 3034 },
	{ 0x75a2, 0x75a2, PDF_CMAP_SINGLE, 3035 },
	{ 0x75a4, 0x75a4, PDF_CMAP_SINGLE, 3036 },
	{ 0x75b4, 0x75b4, PDF_CMAP_SINGLE, 3037 },
	{ 0x75ba, 0x75ba, PDF_CMAP_SINGLE, 3038 },
	{ 0x75bf, 0x75c1, PDF_CMAP_RANGE, 3039 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 3042 },
	{ 0x75c6, 0x75c6, PDF_CMAP_SINGLE, 3043 },
	{ 0x75cc, 0x75cc, PDF_CMAP_SINGLE, 3044 },
	{ 0x75ce, 0x75cf, PDF_CMAP_RANGE, 3045 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 3047 },
	{ 0x75dc, 0x75dc, PDF_CMAP_SINGLE, 3048 },
	{ 0x75df, 0x75e1, PDF_CMAP_RANGE, 3049 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 3052 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 3053 },
	{ 0x75ec, 0x75ec, PDF_CMAP_SINGLE, 3054 },
	{ 0x75ee, 0x75ef, PDF_CMAP_RANGE, 3055 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 3057 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 3058 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 3059 },
	{ 0x7602, 0x7604, PDF_CMAP_RANGE, 3060 },
	{ 0x7607, 0x7608, PDF_CMAP_RANGE, 3063 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 3065 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 3066 },
	{ 0x760f, 0x760f, PDF_CMAP_SINGLE, 3067 },
	{ 0x7612, 0x7613, PDF_CMAP_RANGE, 3068 },
	{ 0x7615, 0x7616, PDF_CMAP_RANGE, 3070 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 3072 },
	{ 0x761b, 0x761e, PDF_CMAP_RANGE, 3073 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 3077 },
	{ 0x7625, 0x7626, PDF_CMAP_RANGE, 3078 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 3080 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 3081 },
	{ 0x7632, 0x7633, PDF_CMAP_RANGE, 3082 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 3084 },
	{ 0x7638, 0x763a, PDF_CMAP_RANGE, 3085 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 3088 },
	{ 0x7640, 0x7641, PDF_CMAP_RANGE, 3090 },
	{ 0x7643, 0x7645, PDF_CMAP_RANGE, 3092 },
	{ 0x7649, 0x764b, PDF_CMAP_TABLE, 248 },
	{ 0x7655, 0x7655, PDF_CMAP_SINGLE, 3097 },
	{ 0x7659, 0x7659, PDF_CMAP_SINGLE, 3098 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 3099 },
	{ 0x7664, 0x7665, PDF_CMAP_RANGE, 3100 },
	{ 0x766d, 0x766f, PDF_CMAP_RANGE, 3102 },
	{ 0x7671, 0x7671, PDF_CMAP_SINGLE, 3105 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 3106 },
	{ 0x7681, 0x7681, PDF_CMAP_SINGLE, 3107 },
	{ 0x7685, 0x7685, PDF_CMAP_SINGLE, 3108 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 3109 },
	{ 0x7695, 0x7695, PDF_CMAP_SINGLE, 3111 },
	{ 0x769b, 0x769d, PDF_CMAP_RANGE, 3112 },
	{ 0x769f, 0x76a0, PDF_CMAP_RANGE, 3115 },
	{ 0x76a2, 0x76a8, PDF_CMAP_RANGE, 3117 },
	{ 0x76aa, 0x76aa, PDF_CMAP_SINGLE, 3124 },
	{ 0x76ad, 0x76ad, PDF_CMAP_SINGLE, 3125 },
	{ 0x76bd, 0x76bd, PDF_CMAP_SINGLE, 3126 },
	{ 0x76c1, 0x76c1, PDF_CMAP_SINGLE, 3127 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 3128 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 3129 },
	{ 0x76cb, 0x76cc, PDF_CMAP_RANGE, 3130 },
	{ 0x76ce, 0x76ce, PDF_CMAP_SINGLE, 3132 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 3133 },
	{ 0x76d9, 0x76d9, PDF_CMAP_SINGLE, 3134 },
	{ 0x76e0, 0x76e0, PDF_CMAP_SINGLE, 3135 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 3136 },
	{ 0x76e8, 0x76e8, PDF_CMAP_SINGLE, 3137 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 3138 },
	{ 0x76f0, 0x76f1, PDF_CMAP_RANGE, 3139 },
	{ 0x76f6, 0x76f6, PDF_CMAP_SINGLE, 3141 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 3142 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 3143 },
	{ 0x7700, 0x7700, PDF_CMAP_SINGLE, 3144 },
	{ 0x7706, 0x7706, PDF_CMAP_SINGLE, 3145 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 3146 },
	{ 0x770e, 0x770e, PDF_CMAP_SINGLE, 3147 },
	{ 0x7712, 0x7712, PDF_CMAP_SINGLE, 3148 },
	{ 0x7714, 0x7715, PDF_CMAP_RANGE, 3149 },
	{ 0x7717, 0x7717, PDF_CMAP_SINGLE, 3151 },
	{ 0x7719, 0x771a, PDF_CMAP_RANGE, 3152 },
	{ 0x771c, 0x771c, PDF_CMAP_SINGLE, 3154 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 3155 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 3156 },
	{ 0x772d, 0x772f, PDF_CMAP_RANGE, 3157 },
	{ 0x7734, 0x7736, PDF_CMAP_RANGE, 3160 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 3163 },
	{ 0x773d, 0x773e, PDF_CMAP_RANGE, 3164 },
	{ 0x7742, 0x7742, PDF_CMAP_SINGLE, 3166 },
	{ 0x7745, 0x7746, PDF_CMAP_RANGE, 3167 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 3169 },
	{ 0x774d, 0x774f, PDF_CMAP_RANGE, 3170 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 3173 },
	{ 0x7756, 0x7757, PDF_CMAP_RANGE, 3174 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 3176 },
	{ 0x775e, 0x7760, PDF_CMAP_RANGE, 3177 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 3180 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 3181 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 3182 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 3183 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 3184 },
	{ 0x7770, 0x7770, PDF_CMAP_SINGLE, 3185 },
	{ 0x7772, 0x7774, PDF_CMAP_RANGE, 3186 },
	{ 0x777a, 0x777a, PDF_CMAP_SINGLE, 3189 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 3190 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 3191 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 3192 },
	{ 0x778c, 0x778d, PDF_CMAP_RANGE, 3193 },
	{ 0x7794, 0x7796, PDF_CMAP_RANGE, 3195 },
	{ 0x779a, 0x779a, PDF_CMAP_SINGLE, 3198 },
	{ 0x779f, 0x779f, PDF_CMAP_SINGLE, 3199 },
	{ 0x77a2, 0x77a2, PDF_CMAP_SINGLE, 3200 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 3201 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 3202 },
	{ 0x77ae, 0x77af, PDF_CMAP_RANGE, 3203 },
	{ 0x77b1, 0x77b1, PDF_CMAP_SINGLE, 3205 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 3206 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 3207 },
	{ 0x77c3, 0x77c3, PDF_CMAP_SINGLE, 3208 },
	{ 0x77c9, 0x77c9, PDF_CMAP_SINGLE, 3209 },
	{ 0x77d1, 0x77d2, PDF_CMAP_RANGE, 3210 },
	{ 0x77d5, 0x77d5, PDF_CMAP_SINGLE, 3212 },
	{ 0x77d9, 0x77d9, PDF_CMAP_SINGLE, 3213 },
	{ 0x77de, 0x77e0, PDF_CMAP_RANGE, 3214 },
	{ 0x77e4, 0x77e4, PDF_CMAP_SINGLE, 3217 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 3218 },
	{ 0x77ea, 0x77ea, PDF_CMAP_SINGLE, 3219 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 3220 },
	{ 0x77f0, 0x77f1, PDF_CMAP_RANGE, 3221 },
	{ 0x77f4, 0x77f4, PDF_CMAP_SINGLE, 3223 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 3224 },
	{ 0x77fb, 0x77fb, PDF_CMAP_SINGLE, 3225 },
	{ 0x7805, 0x7806, PDF_CMAP_RANGE, 3226 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 3228 },
	{ 0x780d, 0x780e, PDF_CMAP_RANGE, 3229 },
	{ 0x7811, 0x7811, PDF_CMAP_SINGLE, 3231 },
	{ 0x781d, 0x781d, PDF_CMAP_SINGLE, 3232 },
	{ 0x7821, 0x7823, PDF_CMAP_RANGE, 3233 },
	{ 0x782d, 0x782e, PDF_CMAP_RANGE, 3236 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 3238 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 3239 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 3240 },
	{ 0x7843, 0x7844, PDF_CMAP_RANGE, 3241 },
	{ 0x7847, 0x7848, PDF_CMAP_RANGE, 3243 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 3245 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 3246 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 3247 },
	{ 0x785c, 0x785c, PDF_CMAP_SINGLE, 3248 },
	{ 0x785e, 0x785e, PDF_CMAP_SINGLE, 3249 },
	{ 0x7860, 0x7861, PDF_CMAP_RANGE, 3250 },
	{ 0x7863, 0x7864, PDF_CMAP_RANGE, 3252 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 3254 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 3255 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 3256 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 3257 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 3258 },
	{ 0x788a, 0x788a, PDF_CMAP_SINGLE, 3259 },
	{ 0x788f, 0x788f, PDF_CMAP_SINGLE, 3260 },
	{ 0x7894, 0x7894, PDF_CMAP_SINGLE, 3261 },
	{ 0x7898, 0x7898, PDF_CMAP_SINGLE, 3262 },
	{ 0x789d, 0x789f, PDF_CMAP_RANGE, 3264 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 3263 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 3267 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 3268 },
	{ 0x78ac, 0x78ad, PDF_CMAP_RANGE, 3269 },
	{ 0x78b0, 0x78b3, PDF_CMAP_RANGE, 3271 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 3275 },
	{ 0x78bd, 0x78bd, PDF_CMAP_SINGLE, 3276 },
	{ 0x78bf, 0x78bf, PDF_CMAP_SINGLE, 3277 },
	{ 0x78c7, 0x78c9, PDF_CMAP_RANGE, 3278 },
	{ 0x78cc, 0x78cc, PDF_CMAP_SINGLE, 3281 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 3282 },
	{ 0x78d2, 0x78d3, PDF_CMAP_RANGE, 3283 },
	{ 0x78d5, 0x78d6, PDF_CMAP_RANGE, 3285 },
	{ 0x78db, 0x78db, PDF_CMAP_SINGLE, 3288 },
	{ 0x78df, 0x78e1, PDF_CMAP_RANGE, 3289 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 3287 },
	{ 0x78e6, 0x78e6, PDF_CMAP_SINGLE, 3292 },
	{ 0x78ea, 0x78ea, PDF_CMAP_SINGLE, 3293 },
	{ 0x78f2, 0x78f3, PDF_CMAP_RANGE, 3294 },
	{ 0x78f6, 0x78f7, PDF_CMAP_RANGE, 3297 },
	{ 0x78fa, 0x78fb, PDF_CMAP_RANGE, 3299 },
	{ 0x78ff, 0x7900, PDF_CMAP_TABLE, 251 },
	{ 0x7906, 0x7906, PDF_CMAP_SINGLE, 3302 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 3303 },
	{ 0x7910, 0x7910, PDF_CMAP_SINGLE, 3304 },
	{ 0x791a, 0x791a, PDF_CMAP_SINGLE, 3305 },
	{ 0x791c, 0x791c, PDF_CMAP_SINGLE, 3306 },
	{ 0x791e, 0x7920, PDF_CMAP_RANGE, 3307 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 3310 },
	{ 0x7927, 0x7927, PDF_CMAP_SINGLE, 3311 },
	{ 0x7929, 0x7929, PDF_CMAP_SINGLE, 3312 },
	{ 0x792d, 0x792d, PDF_CMAP_SINGLE, 3313 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 3314 },
	{ 0x7934, 0x7935, PDF_CMAP_RANGE, 3315 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 3317 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 3318 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 3319 },
	{ 0x7944, 0x7946, PDF_CMAP_RANGE, 3320 },
	{ 0x794a, 0x794b, PDF_CMAP_RANGE, 3323 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 3325 },
	{ 0x7951, 0x7951, PDF_CMAP_SINGLE, 3326 },
	{ 0x7954, 0x7954, PDF_CMAP_SINGLE, 3327 },
	{ 0x7958, 0x7958, PDF_CMAP_SINGLE, 3328 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 3329 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 3331 },
	{ 0x7969, 0x7969, PDF_CMAP_SINGLE, 3332 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 3333 },
	{ 0x7972, 0x7972, PDF_CMAP_SINGLE, 3334 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 3335 },
	{ 0x797b, 0x797c, PDF_CMAP_RANGE, 3336 },
	{ 0x797e, 0x797e, PDF_CMAP_SINGLE, 3338 },
	{ 0x798b, 0x798c, PDF_CMAP_RANGE, 3339 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 3341 },
	{ 0x7993, 0x7996, PDF_CMAP_RANGE, 3342 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 3346 },
	{ 0x799b, 0x799c, PDF_CMAP_RANGE, 3347 },
	{ 0x79a1, 0x79a1, PDF_CMAP_SINGLE, 3349 },
	{ 0x79a8, 0x79a9, PDF_CMAP_RANGE, 3350 },
	{ 0x79ab, 0x79ab, PDF_CMAP_SINGLE, 3352 },
	{ 0x79af, 0x79af, PDF_CMAP_SINGLE, 3353 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 3354 },
	{ 0x79b4, 0x79b4, PDF_CMAP_SINGLE, 3355 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 3356 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 3357 },
	{ 0x79c2, 0x79c2, PDF_CMAP_SINGLE, 3358 },
	{ 0x79c4, 0x79c4, PDF_CMAP_SINGLE, 3359 },
	{ 0x79c7, 0x79c8, PDF_CMAP_RANGE, 3360 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 3362 },
	{ 0x79cf, 0x79cf, PDF_CMAP_SINGLE, 3363 },
	{ 0x79d4, 0x79d4, PDF_CMAP_SINGLE, 3364 },
	{ 0x79d6, 0x79d6, PDF_CMAP_SINGLE, 3365 },
	{ 0x79da, 0x79da, PDF_CMAP_SINGLE, 3366 },
	{ 0x79dd, 0x79de, PDF_CMAP_RANGE, 3367 },
	{ 0x79e0, 0x79e0, PDF_CMAP_SINGLE, 3369 },
	{ 0x79e2, 0x79e2, PDF_CMAP_SINGLE, 3370 },
	{ 0x79e5, 0x79e5, PDF_CMAP_SINGLE, 3371 },
	{ 0x79ea, 0x79eb, PDF_CMAP_RANGE, 3372 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 3374 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 3375 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 3376 },
	{ 0x79fc, 0x79fc, PDF_CMAP_SINGLE, 3377 },
	{ 0x7a02, 0x7a03, PDF_CMAP_RANGE, 3378 },
	{ 0x7a07, 0x7a07, PDF_CMAP_SINGLE, 3380 },
	{ 0x7a09, 0x7a0a, PDF_CMAP_RANGE, 3381 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 3383 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 3384 },
	{ 0x7a15, 0x7a15, PDF_CMAP_SINGLE, 3385 },
	{ 0x7a1b, 0x7a1b, PDF_CMAP_SINGLE, 3386 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 3387 },
	{ 0x7a21, 0x7a21, PDF_CMAP_SINGLE, 3388 },
	{ 0x7a27, 0x7a27, PDF_CMAP_SINGLE, 3389 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 3390 },
	{ 0x7a2d, 0x7a2d, PDF_CMAP_SINGLE, 3391 },
	{ 0x7a2f, 0x7a30, PDF_CMAP_RANGE, 3392 },
	{ 0x7a34, 0x7a35, PDF_CMAP_RANGE, 3394 },
	{ 0x7a38, 0x7a3a, PDF_CMAP_RANGE, 3396 },
	{ 0x7a44, 0x7a45, PDF_CMAP_RANGE, 3399 },
	{ 0x7a47, 0x7a48, PDF_CMAP_RANGE, 3401 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 3403 },
	{ 0x7a55, 0x7a56, PDF_CMAP_RANGE, 3404 },
	{ 0x7a59, 0x7a59, PDF_CMAP_SINGLE, 3406 },
	{ 0x7a5c, 0x7a5d, PDF_CMAP_RANGE, 3407 },
	{ 0x7a5f, 0x7a60, PDF_CMAP_RANGE, 3409 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 3411 },
	{ 0x7a67, 0x7a67, PDF_CMAP_SINGLE, 3412 },
	{ 0x7a6a, 0x7a6a, PDF_CMAP_SINGLE, 3413 },
	{ 0x7a6d, 0x7a6d, PDF_CMAP_SINGLE, 3414 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 3415 },
	{ 0x7a78, 0x7a78, PDF_CMAP_SINGLE, 3416 },
	{ 0x7a7e, 0x7a7e, PDF_CMAP_SINGLE, 3417 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 3418 },
	{ 0x7a82, 0x7a82, PDF_CMAP_SINGLE, 3419 },
	{ 0x7a85, 0x7a86, PDF_CMAP_RANGE, 3420 },
	{ 0x7a8a, 0x7a8b, PDF_CMAP_RANGE, 3422 },
	{ 0x7a90, 0x7a91, PDF_CMAP_RANGE, 3424 },
	{ 0x7a94, 0x7a94, PDF_CMAP_SINGLE, 3426 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 3427 },
	{ 0x7aa0, 0x7aa0, PDF_CMAP_SINGLE, 3428 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 3429 },
	{ 0x7aac, 0x7aac, PDF_CMAP_SINGLE, 3430 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 3431 },
	{ 0x7ab5, 0x7ab5, PDF_CMAP_SINGLE, 3432 },
	{ 0x7ab9, 0x7ab9, PDF_CMAP_SINGLE, 3433 },
	{ 0x7abb, 0x7abc, PDF_CMAP_RANGE, 3434 },
	{ 0x7ac6, 0x7ac6, PDF_CMAP_SINGLE, 3436 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 3437 },
	{ 0x7acc, 0x7acc, PDF_CMAP_SINGLE, 3438 },
	{ 0x7ace, 0x7ace, PDF_CMAP_SINGLE, 3439 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 3440 },
	{ 0x7adb, 0x7adb, PDF_CMAP_SINGLE, 3441 },
	{ 0x7ae8, 0x7ae9, PDF_CMAP_RANGE, 3442 },
	{ 0x7aeb, 0x7aec, PDF_CMAP_RANGE, 3444 },
	{ 0x7af1, 0x7af1, PDF_CMAP_SINGLE, 3446 },
	{ 0x7af4, 0x7af4, PDF_CMAP_SINGLE, 3447 },
	{ 0x7afb, 0x7afb, PDF_CMAP_SINGLE, 3448 },
	{ 0x7afd, 0x7afe, PDF_CMAP_RANGE, 3449 },
	{ 0x7b07, 0x7b07, PDF_CMAP_SINGLE, 3451 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 3452 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 3453 },
	{ 0x7b23, 0x7b23, PDF_CMAP_SINGLE, 3454 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 3455 },
	{ 0x7b29, 0x7b2b, PDF_CMAP_RANGE, 3456 },
	{ 0x7b2d, 0x7b31, PDF_CMAP_RANGE, 3459 },
	{ 0x7b34, 0x7b34, PDF_CMAP_SINGLE, 3464 },
	{ 0x7b3d, 0x7b3d, PDF_CMAP_SINGLE, 3465 },
	{ 0x7b3f, 0x7b41, PDF_CMAP_RANGE, 3466 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 3469 },
	{ 0x7b4e, 0x7b4e, PDF_CMAP_SINGLE, 3470 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 3471 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 3472 },
	{ 0x7b64, 0x7b64, PDF_CMAP_SINGLE, 3473 },
	{ 0x7b66, 0x7b66, PDF_CMAP_SINGLE, 3474 },
	{ 0x7b69, 0x7b6a, PDF_CMAP_RANGE, 3475 },
	{ 0x7b6d, 0x7b6d, PDF_CMAP_SINGLE, 3477 },
	{ 0x7b6f, 0x7b6f, PDF_CMAP_SINGLE, 3478 },
	{ 0x7b72, 0x7b73, PDF_CMAP_RANGE, 3479 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 3481 },
	{ 0x7b84, 0x7b84, PDF_CMAP_SINGLE, 3482 },
	{ 0x7b89, 0x7b89, PDF_CMAP_SINGLE, 3483 },
	{ 0x7b8e, 0x7b8e, PDF_CMAP_SINGLE, 3484 },
	{ 0x7b90, 0x7b91, PDF_CMAP_RANGE, 3485 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 3487 },
	{ 0x7b9b, 0x7b9b, PDF_CMAP_SINGLE, 3488 },
	{ 0x7b9e, 0x7b9e, PDF_CMAP_SINGLE, 3489 },
	{ 0x7ba0, 0x7ba0, PDF_CMAP_SINGLE, 3490 },
	{ 0x7ba5, 0x7ba5, PDF_CMAP_SINGLE, 3491 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 3492 },
	{ 0x7baf, 0x7bb0, PDF_CMAP_RANGE, 3493 },
	{ 0x7bb2, 0x7bb2, PDF_CMAP_SINGLE, 3495 },
	{ 0x7bb5, 0x7bb6, PDF_CMAP_RANGE, 3496 },
	{ 0x7bba, 0x7bbd, PDF_CMAP_RANGE, 3498 },
	{ 0x7bc2, 0x7bc2, PDF_CMAP_SINGLE, 3502 },
	{ 0x7bc5, 0x7bc5, PDF_CMAP_SINGLE, 3503 },
	{ 0x7bc8, 0x7bc8, PDF_CMAP_SINGLE, 3504 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 3505 },
	{ 0x7bd4, 0x7bd4, PDF_CMAP_SINGLE, 3506 },
	{ 0x7bd6, 0x7bd7, PDF_CMAP_RANGE, 3507 },
	{ 0x7bd9, 0x7bdb, PDF_CMAP_RANGE, 3509 },
	{ 0x7be8, 0x7be8, PDF_CMAP_SINGLE, 3512 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 3513 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 3514 },
	{ 0x7bf4, 0x7bf5, PDF_CMAP_RANGE, 3515 },
	{ 0x7bf8, 0x7bfa, PDF_CMAP_RANGE, 3517 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 3520 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 3521 },
	{ 0x7c01, 0x7c04, PDF_CMAP_RANGE, 3522 },
	{ 0x7c06, 0x7c06, PDF_CMAP_SINGLE, 3526 },
	{ 0x7c09, 0x7c09, PDF_CMAP_SINGLE, 3527 },
	{ 0x7c0b, 0x7c0c, PDF_CMAP_RANGE, 3528 },
	{ 0x7c0e, 0x7c0f, PDF_CMAP_RANGE, 3530 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 3532 },
	{ 0x7c1b, 0x7c1b, PDF_CMAP_SINGLE, 3533 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 3534 },
	{ 0x7c25, 0x7c26, PDF_CMAP_RANGE, 3535 },
	{ 0x7c28, 0x7c28, PDF_CMAP_SINGLE, 3537 },
	{ 0x7c2c, 0x7c2c, PDF_CMAP_SINGLE, 3538 },
	{ 0x7c31, 0x7c31, PDF_CMAP_SINGLE, 3539 },
	{ 0x7c33, 0x7c34, PDF_CMAP_RANGE, 3540 },
	{ 0x7c36, 0x7c36, PDF_CMAP_SINGLE, 3542 },
	{ 0x7c39, 0x7c3a, PDF_CMAP_RANGE, 3543 },
	{ 0x7c46, 0x7c46, PDF_CMAP_SINGLE, 3545 },
	{ 0x7c4a, 0x7c4a, PDF_CMAP_SINGLE, 3546 },
	{ 0x7c51, 0x7c53, PDF_CMAP_RANGE, 3548 },
	{ 0x7c55, 0x7c55, PDF_CMAP_SINGLE, 3547 },
	{ 0x7c59, 0x7c5e, PDF_CMAP_RANGE, 3551 },
	{ 0x7c61, 0x7c61, PDF_CMAP_SINGLE, 3557 },
	{ 0x7c63, 0x7c63, PDF_CMAP_SINGLE, 3558 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 3559 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 3560 },
	{ 0x7c6d, 0x7c6e, PDF_CMAP_RANGE, 3561 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 3563 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 3564 },
	{ 0x7c79, 0x7c79, PDF_CMAP_SINGLE, 3565 },
	{ 0x7c7c, 0x7c7d, PDF_CMAP_RANGE, 3566 },
	{ 0x7c86, 0x7c87, PDF_CMAP_RANGE, 3568 },
	{ 0x7c8f, 0x7c8f, PDF_CMAP_SINGLE, 3570 },
	{ 0x7c94, 0x7c94, PDF_CMAP_SINGLE, 3571 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 3572 },
	{ 0x7ca0, 0x7ca0, PDF_CMAP_SINGLE, 3573 },
	{ 0x7ca6, 0x7ca6, PDF_CMAP_SINGLE, 3574 },
	{ 0x7cb0, 0x7cb0, PDF_CMAP_SINGLE, 3575 },
	{ 0x7cb6, 0x7cb7, PDF_CMAP_RANGE, 3576 },
	{ 0x7cba, 0x7cbc, PDF_CMAP_RANGE, 3578 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 3581 },
	{ 0x7cc4, 0x7cc4, PDF_CMAP_SINGLE, 3582 },
	{ 0x7cc7, 0x7cc9, PDF_CMAP_RANGE, 3583 },
	{ 0x7ccd, 0x7ccd, PDF_CMAP_SINGLE, 3586 },
	{ 0x7ccf, 0x7ccf, PDF_CMAP_SINGLE, 3587 },
	{ 0x7cd3, 0x7cd5, PDF_CMAP_RANGE, 3588 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 3591 },
	{ 0x7cd9, 0x7cda, PDF_CMAP_RANGE, 3592 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 3594 },
	{ 0x7ce6, 0x7ce6, PDF_CMAP_SINGLE, 3595 },
	{ 0x7ce9, 0x7ce9, PDF_CMAP_SINGLE, 3596 },
	{ 0x7ceb, 0x7ceb, PDF_CMAP_SINGLE, 3597 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 3598 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 3599 },
	{ 0x7d07, 0x7d09, PDF_CMAP_RANGE, 3600 },
	{ 0x7d0f, 0x7d0f, PDF_CMAP_SINGLE, 3603 },
	{ 0x7d11, 0x7d13, PDF_CMAP_RANGE, 3604 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 3607 },
	{ 0x7d1d, 0x7d1e, PDF_CMAP_RANGE, 3608 },
	{ 0x7d23, 0x7d23, PDF_CMAP_SINGLE, 3610 },
	{ 0x7d26, 0x7d26, PDF_CMAP_SINGLE, 3611 },
	{ 0x7d2a, 0x7d2a, PDF_CMAP_SINGLE, 3612 },
	{ 0x7d2d, 0x7d2d, PDF_CMAP_SINGLE, 3613 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 3614 },
	{ 0x7d3c, 0x7d3e, PDF_CMAP_RANGE, 3615 },
	{ 0x7d40, 0x7d41, PDF_CMAP_RANGE, 3618 },
	{ 0x7d47, 0x7d48, PDF_CMAP_RANGE, 3620 },
	{ 0x7d4d, 0x7d4d, PDF_CMAP_SINGLE, 3622 },
	{ 0x7d51, 0x7d51, PDF_CMAP_SINGLE, 3623 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 3624 },
	{ 0x7d57, 0x7d57, PDF_CMAP_SINGLE, 3625 },
	{ 0x7d59, 0x7d5a, PDF_CMAP_RANGE, 3626 },
	{ 0x7d5c, 0x7d5d, PDF_CMAP_RANGE, 3628 },
	{ 0x7d65, 0x7d65, PDF_CMAP_SINGLE, 3630 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 3631 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 3632 },
	{ 0x7d70, 0x7d70, PDF_CMAP_SINGLE, 3633 },
	{ 0x7d78, 0x7d78, PDF_CMAP_SINGLE, 3634 },
	{ 0x7d7a, 0x7d7b, PDF_CMAP_RANGE, 3635 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 3637 },
	{ 0x7d81, 0x7d83, PDF_CMAP_RANGE, 3638 },
	{ 0x7d85, 0x7d86, PDF_CMAP_RANGE, 3641 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 3643 },
	{ 0x7d8b, 0x7d8d, PDF_CMAP_RANGE, 3644 },
	{ 0x7d91, 0x7d91, PDF_CMAP_SINGLE, 3647 },
	{ 0x7d96, 0x7d97, PDF_CMAP_RANGE, 3648 },
	{ 0x7d9d, 0x7d9e, PDF_CMAP_RANGE, 3650 },
	{ 0x7da6, 0x7da7, PDF_CMAP_RANGE, 3652 },
	{ 0x7daa, 0x7daa, PDF_CMAP_SINGLE, 3654 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 3655 },
	{ 0x7db6, 0x7db7, PDF_CMAP_RANGE, 3656 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 3658 },
	{ 0x7dc2, 0x7dc6, PDF_CMAP_RANGE, 3659 },
	{ 0x7dcc, 0x7dce, PDF_CMAP_RANGE, 3664 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 3667 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 3668 },
	{ 0x7de2, 0x7de2, PDF_CMAP_SINGLE, 3670 },
	{ 0x7de5, 0x7de6, PDF_CMAP_RANGE, 3671 },
	{ 0x7dea, 0x7deb, PDF_CMAP_RANGE, 3673 },
	{ 0x7ded, 0x7ded, PDF_CMAP_SINGLE, 3675 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 3676 },
	{ 0x7df5, 0x7df6, PDF_CMAP_RANGE, 3677 },
	{ 0x7df9, 0x7dfa, PDF_CMAP_RANGE, 3679 },
	{ 0x7e00, 0x7e00, PDF_CMAP_SINGLE, 3669 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 3681 },
	{ 0x7e10, 0x7e11, PDF_CMAP_RANGE, 3682 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 3684 },
	{ 0x7e17, 0x7e17, PDF_CMAP_SINGLE, 3685 },
	{ 0x7e1c, 0x7e1d, PDF_CMAP_RANGE, 3686 },
	{ 0x7e20, 0x7e20, PDF_CMAP_SINGLE, 3688 },
	{ 0x7e27, 0x7e28, PDF_CMAP_RANGE, 3689 },
	{ 0x7e2c, 0x7e2d, PDF_CMAP_RANGE, 3691 },
	{ 0x7e2f, 0x7e2f, PDF_CMAP_SINGLE, 3693 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 3694 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 3695 },
	{ 0x7e3f, 0x7e3f, PDF_CMAP_SINGLE, 3696 },
	{ 0x7e44, 0x7e45, PDF_CMAP_RANGE, 3697 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 3699 },
	{ 0x7e4e, 0x7e4e, PDF_CMAP_SINGLE, 3700 },
	{ 0x7e50, 0x7e50, PDF_CMAP_SINGLE, 3701 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 3702 },
	{ 0x7e58, 0x7e58, PDF_CMAP_SINGLE, 3703 },
	{ 0x7e5f, 0x7e5f, PDF_CMAP_SINGLE, 3704 },
	{ 0x7e61, 0x7e62, PDF_CMAP_RANGE, 3705 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 3707 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 3708 },
	{ 0x7e6e, 0x7e6f, PDF_CMAP_RANGE, 3709 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 3711 },
	{ 0x7e78, 0x7e78, PDF_CMAP_SINGLE, 3712 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 3713 },
	{ 0x7e81, 0x7e81, PDF_CMAP_SINGLE, 3714 },
	{ 0x7e86, 0x7e87, PDF_CMAP_RANGE, 3715 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 3717 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 3718 },
	{ 0x7e91, 0x7e91, PDF_CMAP_SINGLE, 3719 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 3720 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 3721 },
	{ 0x7e9a, 0x7e9a, PDF_CMAP_SINGLE, 3722 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 3723 },
	{ 0x7f3b, 0x7f3c, PDF_CMAP_TABLE, 253 },
	{ 0x7f3d, 0x7f3f, PDF_CMAP_RANGE, 3727 },
	{ 0x7f43, 0x7f44, PDF_CMAP_RANGE, 3730 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 3732 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 3733 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 3734 },
	{ 0x7f5b, 0x7f5d, PDF_CMAP_RANGE, 3736 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 3739 },
	{ 0x7f63, 0x7f66, PDF_CMAP_RANGE, 3740 },
	{ 0x7f6d, 0x7f6d, PDF_CMAP_SINGLE, 3744 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 3745 },
	{ 0x7f7d, 0x7f80, PDF_CMAP_RANGE, 3746 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 3750 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 3751 },
	{ 0x7f8f, 0x7f91, PDF_CMAP_RANGE, 3752 },
	{ 0x7f96, 0x7f97, PDF_CMAP_RANGE, 3755 },
	{ 0x7f9c, 0x7f9c, PDF_CMAP_SINGLE, 3757 },
	{ 0x7fa1, 0x7fa2, PDF_CMAP_RANGE, 3758 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 3760 },
	{ 0x7faa, 0x7faa, PDF_CMAP_SINGLE, 3761 },
	{ 0x7fad, 0x7fad, PDF_CMAP_SINGLE, 3762 },
	{ 0x7fb4, 0x7fb4, PDF_CMAP_SINGLE, 3763 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 3764 },
	{ 0x7fbf, 0x7fc0, PDF_CMAP_RANGE, 3765 },
	{ 0x7fc3, 0x7fc3, PDF_CMAP_SINGLE, 3767 },
	{ 0x7fc8, 0x7fc8, PDF_CMAP_SINGLE, 3768 },
	{ 0x7fce, 0x7fcf, PDF_CMAP_RANGE, 3769 },
	{ 0x7fdb, 0x7fdb, PDF_CMAP_SINGLE, 3771 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 3772 },
	{ 0x7fe3, 0x7fe3, PDF_CMAP_SINGLE, 3773 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 3774 },
	{ 0x7fe8, 0x7fe8, PDF_CMAP_SINGLE, 3775 },
	{ 0x7fec, 0x7fec, PDF_CMAP_SINGLE, 3776 },
	{ 0x7fee, 0x7fef, PDF_CMAP_RANGE, 3777 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 3779 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 3780 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 3781 },
	{ 0x8007, 0x8008, PDF_CMAP_RANGE, 3784 },
	{ 0x800a, 0x800a, PDF_CMAP_SINGLE, 3786 },
	{ 0x800d, 0x800f, PDF_CMAP_RANGE, 3787 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 3790 },
	{ 0x8013, 0x8014, PDF_CMAP_RANGE, 3791 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 3793 },
	{ 0x801d, 0x8020, PDF_CMAP_RANGE, 3794 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 3798 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 3799 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 3800 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 3801 },
	{ 0x8030, 0x8030, PDF_CMAP_SINGLE, 3802 },
	{ 0x8034, 0x8035, PDF_CMAP_RANGE, 3803 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 3805 },
	{ 0x8039, 0x803a, PDF_CMAP_RANGE, 3806 },
	{ 0x803c, 0x803c, PDF_CMAP_SINGLE, 3808 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 3809 },
	{ 0x8040, 0x8040, PDF_CMAP_SINGLE, 3810 },
	{ 0x8044, 0x8044, PDF_CMAP_SINGLE, 3811 },
	{ 0x8060, 0x8060, PDF_CMAP_SINGLE, 3812 },
	{ 0x8064, 0x8064, PDF_CMAP_SINGLE, 3813 },
	{ 0x8066, 0x8066, PDF_CMAP_SINGLE, 3814 },
	{ 0x806d, 0x806d, PDF_CMAP_SINGLE, 3815 },
	{ 0x8071, 0x8071, PDF_CMAP_SINGLE, 3816 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 3817 },
	{ 0x8081, 0x8081, PDF_CMAP_SINGLE, 3818 },
	{ 0x8088, 0x8088, PDF_CMAP_SINGLE, 3819 },
	{ 0x808e, 0x808e, PDF_CMAP_SINGLE, 3820 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 3821 },
	{ 0x809e, 0x809e, PDF_CMAP_SINGLE, 3822 },
	{ 0x80a6, 0x80a7, PDF_CMAP_RANGE, 3823 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 3825 },
	{ 0x80b8, 0x80b9, PDF_CMAP_RANGE, 3826 },
	{ 0x80c8, 0x80c8, PDF_CMAP_SINGLE, 3828 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 3829 },
	{ 0x80cf, 0x80cf, PDF_CMAP_SINGLE, 3830 },
	{ 0x80d2, 0x80d2, PDF_CMAP_SINGLE, 3831 },
	{ 0x80d4, 0x80d5, PDF_CMAP_RANGE, 3832 },
	{ 0x80d7, 0x80d8, PDF_CMAP_RANGE, 3834 },
	{ 0x80e0, 0x80e0, PDF_CMAP_SINGLE, 3836 },
	{ 0x80ed, 0x80ee, PDF_CMAP_RANGE, 3837 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 3839 },
	{ 0x80f2, 0x80f3, PDF_CMAP_RANGE, 3840 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 3842 },
	{ 0x80f9, 0x80fa, PDF_CMAP_RANGE, 3843 },
	{ 0x80fe, 0x80fe, PDF_CMAP_SINGLE, 3845 },
	{ 0x8103, 0x8103, PDF_CMAP_SINGLE, 3846 },
	{ 0x810b, 0x810b, PDF_CMAP_SINGLE, 3847 },
	{ 0x8116, 0x8118, PDF_CMAP_RANGE, 3848 },
	{ 0x811c, 0x811c, PDF_CMAP_SINGLE, 3851 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 3852 },
	{ 0x8120, 0x8120, PDF_CMAP_SINGLE, 3853 },
	{ 0x8124, 0x8124, PDF_CMAP_SINGLE, 3854 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 3855 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 3856 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 3857 },
	{ 0x8135, 0x8135, PDF_CMAP_SINGLE, 3858 },
	{ 0x813a, 0x813a, PDF_CMAP_SINGLE, 3859 },
	{ 0x813c, 0x813c, PDF_CMAP_SINGLE, 3860 },
	{ 0x8145, 0x8145, PDF_CMAP_SINGLE, 3861 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 3862 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 3863 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 3864 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 3865 },
	{ 0x8157, 0x8157, PDF_CMAP_SINGLE, 3866 },
	{ 0x8160, 0x8161, PDF_CMAP_RANGE, 3867 },
	{ 0x8167, 0x8169, PDF_CMAP_RANGE, 3869 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 3872 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 3873 },
	{ 0x8177, 0x8177, PDF_CMAP_SINGLE, 3874 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 3875 },
	{ 0x8184, 0x8186, PDF_CMAP_RANGE, 3877 },
	{ 0x818b, 0x818b, PDF_CMAP_SINGLE, 3880 },
	{ 0x818e, 0x818e, PDF_CMAP_SINGLE, 3881 },
	{ 0x8190, 0x8190, PDF_CMAP_SINGLE, 3876 },
	{ 0x8196, 0x8196, PDF_CMAP_SINGLE, 3882 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 3883 },
	{ 0x819b, 0x819b, PDF_CMAP_SINGLE, 3884 },
	{ 0x819e, 0x819e, PDF_CMAP_SINGLE, 3885 },
	{ 0x81a2, 0x81a2, PDF_CMAP_SINGLE, 3886 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 3887 },
	{ 0x81b2, 0x81b2, PDF_CMAP_SINGLE, 3888 },
	{ 0x81b4, 0x81b4, PDF_CMAP_SINGLE, 3889 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 3890 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 3892 },
	{ 0x81c5, 0x81c5, PDF_CMAP_SINGLE, 3893 },
	{ 0x81ca, 0x81cb, PDF_CMAP_TABLE, 255 },
	{ 0x81ce, 0x81cf, PDF_CMAP_RANGE, 3895 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 3897 },
	{ 0x81d7, 0x81d7, PDF_CMAP_SINGLE, 3898 },
	{ 0x81db, 0x81db, PDF_CMAP_SINGLE, 3899 },
	{ 0x81dd, 0x81de, PDF_CMAP_RANGE, 3900 },
	{ 0x81e1, 0x81e1, PDF_CMAP_SINGLE, 3902 },
	{ 0x81e4, 0x81e4, PDF_CMAP_SINGLE, 3903 },
	{ 0x81eb, 0x81ec, PDF_CMAP_RANGE, 3904 },
	{ 0x81f0, 0x81f2, PDF_CMAP_RANGE, 3906 },
	{ 0x81f5, 0x81f6, PDF_CMAP_RANGE, 3909 },
	{ 0x81f8, 0x81f9, PDF_CMAP_RANGE, 3911 },
	{ 0x81fd, 0x81fd, PDF_CMAP_SINGLE, 3913 },
	{ 0x81ff, 0x8200, PDF_CMAP_RANGE, 3914 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 3916 },
	{ 0x820f, 0x820f, PDF_CMAP_SINGLE, 3917 },
	{ 0x8213, 0x8214, PDF_CMAP_RANGE, 3918 },
	{ 0x8219, 0x821a, PDF_CMAP_RANGE, 3920 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 3922 },
	{ 0x8221, 0x8222, PDF_CMAP_RANGE, 3923 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 3925 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 3926 },
	{ 0x8234, 0x8234, PDF_CMAP_SINGLE, 3927 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 3928 },
	{ 0x8243, 0x8246, PDF_CMAP_RANGE, 3929 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 3933 },
	{ 0x824e, 0x824f, PDF_CMAP_RANGE, 3934 },
	{ 0x8251, 0x8251, PDF_CMAP_SINGLE, 3936 },
	{ 0x8256, 0x8256, PDF_CMAP_SINGLE, 3937 },
	{ 0x825c, 0x825c, PDF_CMAP_SINGLE, 3938 },
	{ 0x8260, 0x8260, PDF_CMAP_SINGLE, 3939 },
	{ 0x8263, 0x8263, PDF_CMAP_SINGLE, 3940 },
	{ 0x8267, 0x8267, PDF_CMAP_SINGLE, 3941 },
	{ 0x826d, 0x826d, PDF_CMAP_SINGLE, 3942 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 3943 },
	{ 0x827b, 0x827b, PDF_CMAP_SINGLE, 3944 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 3945 },
	{ 0x827f, 0x8281, PDF_CMAP_RANGE, 3946 },
	{ 0x8283, 0x8284, PDF_CMAP_RANGE, 3949 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 3951 },
	{ 0x8289, 0x828a, PDF_CMAP_RANGE, 3952 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 3954 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 3955 },
	{ 0x8294, 0x8294, PDF_CMAP_SINGLE, 3956 },
	{ 0x8296, 0x8296, PDF_CMAP_SINGLE, 3957 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 3958 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 3959 },
	{ 0x82a0, 0x82a1, PDF_CMAP_RANGE, 3961 },
	{ 0x82a3, 0x82a4, PDF_CMAP_RANGE, 3963 },
	{ 0x82a7, 0x82aa, PDF_CMAP_RANGE, 3965 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 3969 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 3970 },
	{ 0x82b2, 0x82b2, PDF_CMAP_SINGLE, 3971 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 3972 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 3973 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 3974 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 3975 },
	{ 0x82be, 0x82bf, PDF_CMAP_RANGE, 3976 },
	{ 0x82c6, 0x82c6, PDF_CMAP_SINGLE, 3978 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 3979 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 3980 },
	{ 0x82da, 0x82da, PDF_CMAP_SINGLE, 3981 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 3982 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 3983 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 3984 },
	{ 0x82e8, 0x82e8, PDF_CMAP_SINGLE, 3985 },
	{ 0x82ea, 0x82ea, PDF_CMAP_SINGLE, 3986 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 3987 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 3988 },
	{ 0x82f6, 0x82f7, PDF_CMAP_RANGE, 3989 },
	{ 0x82fd, 0x82fe, PDF_CMAP_RANGE, 3991 },
	{ 0x8300, 0x8301, PDF_CMAP_RANGE, 3993 },
	{ 0x8307, 0x8308, PDF_CMAP_RANGE, 3995 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 3997 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 4000 },
	{ 0x831d, 0x831f, PDF_CMAP_RANGE, 4001 },
	{ 0x8321, 0x8322, PDF_CMAP_RANGE, 4004 },
	{ 0x832c, 0x832e, PDF_CMAP_RANGE, 4006 },
	{ 0x8330, 0x8330, PDF_CMAP_SINGLE, 4009 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 4010 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 4011 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 4012 },
	{ 0x833c, 0x833d, PDF_CMAP_RANGE, 4013 },
	{ 0x8342, 0x8344, PDF_CMAP_RANGE, 4015 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 4018 },
	{ 0x834d, 0x834e, PDF_CMAP_RANGE, 4019 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 4021 },
	{ 0x8353, 0x8354, PDF_CMAP_TABLE, 257 },
	{ 0x8355, 0x8357, PDF_CMAP_RANGE, 4022 },
	{ 0x8370, 0x8370, PDF_CMAP_SINGLE, 4025 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 4026 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 4027 },
	{ 0x837f, 0x8380, PDF_CMAP_RANGE, 4028 },
	{ 0x8382, 0x8382, PDF_CMAP_SINGLE, 4030 },
	{ 0x8384, 0x8384, PDF_CMAP_SINGLE, 4031 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 4032 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 4033 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 4034 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 4035 },
	{ 0x8398, 0x8399, PDF_CMAP_RANGE, 4037 },
	{ 0x839b, 0x839d, PDF_CMAP_RANGE, 4039 },
	{ 0x83a6, 0x83a7, PDF_CMAP_RANGE, 4042 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 4044 },
	{ 0x83ac, 0x83ad, PDF_CMAP_TABLE, 259 },
	{ 0x83be, 0x83c0, PDF_CMAP_RANGE, 4046 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 4049 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 4050 },
	{ 0x83cf, 0x83d1, PDF_CMAP_RANGE, 4051 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 4054 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 4055 },
	{ 0x83e8, 0x83e8, PDF_CMAP_SINGLE, 4057 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 4058 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 4059 },
	{ 0x83f8, 0x83f9, PDF_CMAP_RANGE, 4060 },
	{ 0x83fc, 0x83fc, PDF_CMAP_SINGLE, 4062 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 4063 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 4064 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 4065 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 4066 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 4067 },
	{ 0x8415, 0x8415, PDF_CMAP_SINGLE, 4068 },
	{ 0x8419, 0x8419, PDF_CMAP_SINGLE, 4069 },
	{ 0x842f, 0x842f, PDF_CMAP_SINGLE, 4071 },
	{ 0x8439, 0x8439, PDF_CMAP_SINGLE, 4072 },
	{ 0x8445, 0x8445, PDF_CMAP_SINGLE, 4073 },
	{ 0x8447, 0x8448, PDF_CMAP_RANGE, 4074 },
	{ 0x844a, 0x844a, PDF_CMAP_SINGLE, 4076 },
	{ 0x844d, 0x844d, PDF_CMAP_SINGLE, 4077 },
	{ 0x844f, 0x844f, PDF_CMAP_SINGLE, 4078 },
	{ 0x8451, 0x8452, PDF_CMAP_RANGE, 4079 },
	{ 0x8456, 0x8456, PDF_CMAP_SINGLE, 4081 },
	{ 0x8458, 0x845a, PDF_CMAP_RANGE, 4082 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 4085 },
	{ 0x8460, 0x8460, PDF_CMAP_SINGLE, 4086 },
	{ 0x8464, 0x8465, PDF_CMAP_RANGE, 4087 },
	{ 0x8467, 0x8467, PDF_CMAP_SINGLE, 4089 },
	{ 0x846a, 0x846a, PDF_CMAP_SINGLE, 4090 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 4091 },
	{ 0x8473, 0x8474, PDF_CMAP_RANGE, 4092 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 4094 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 4095 },
	{ 0x847c, 0x847d, PDF_CMAP_RANGE, 4096 },
	{ 0x8481, 0x8481, PDF_CMAP_SINGLE, 4098 },
	{ 0x8485, 0x8485, PDF_CMAP_SINGLE, 4099 },
	{ 0x8492, 0x8493, PDF_CMAP_RANGE, 4100 },
	{ 0x8495, 0x8495, PDF_CMAP_SINGLE, 4102 },
	{ 0x849e, 0x849e, PDF_CMAP_SINGLE, 4103 },
	{ 0x84a6, 0x84a6, PDF_CMAP_SINGLE, 4104 },
	{ 0x84a8, 0x84aa, PDF_CMAP_RANGE, 4105 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 4108 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 4109 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 4110 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 4111 },
	{ 0x84bd, 0x84be, PDF_CMAP_RANGE, 4112 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 4114 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 4115 },
	{ 0x84c7, 0x84c8, PDF_CMAP_RANGE, 4116 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 4118 },
	{ 0x84cf, 0x84cf, PDF_CMAP_SINGLE, 4119 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 4120 },
	{ 0x84dc, 0x84dc, PDF_CMAP_SINGLE, 4121 },
	{ 0x84e7, 0x84e7, PDF_CMAP_SINGLE, 4122 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 4123 },
	{ 0x84ef, 0x84f2, PDF_CMAP_RANGE, 4124 },
	{ 0x84f7, 0x84f7, PDF_CMAP_SINGLE, 4128 },
	{ 0x84fa, 0x84fb, PDF_CMAP_RANGE, 4130 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 4132 },
	{ 0x8502, 0x8503, PDF_CMAP_RANGE, 4133 },
	{ 0x8507, 0x8507, PDF_CMAP_SINGLE, 4135 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 4136 },
	{ 0x850e, 0x850e, PDF_CMAP_SINGLE, 4137 },
	{ 0x8510, 0x8510, PDF_CMAP_SINGLE, 4138 },
	{ 0x851c, 0x851c, PDF_CMAP_SINGLE, 4139 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 4140 },
	{ 0x8522, 0x8525, PDF_CMAP_RANGE, 4141 },
	{ 0x8527, 0x8527, PDF_CMAP_SINGLE, 4145 },
	{ 0x852a, 0x852b, PDF_CMAP_RANGE, 4146 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 4148 },
	{ 0x8532, 0x8532, PDF_CMAP_SINGLE, 4129 },
	{ 0x8533, 0x8534, PDF_CMAP_RANGE, 4149 },
	{ 0x8536, 0x8536, PDF_CMAP_SINGLE, 4151 },
	{ 0x853f, 0x853f, PDF_CMAP_SINGLE, 4152 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 4153 },
	{ 0x854f, 0x8553, PDF_CMAP_RANGE, 4154 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 4159 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 4160 },
	{ 0x855c, 0x8562, PDF_CMAP_RANGE, 4161 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 4168 },
	{ 0x856b, 0x856b, PDF_CMAP_SINGLE, 4169 },
	{ 0x856f, 0x856f, PDF_CMAP_SINGLE, 4170 },
	{ 0x8579, 0x857b, PDF_CMAP_RANGE, 4171 },
	{ 0x857d, 0x857d, PDF_CMAP_SINGLE, 4174 },
	{ 0x857f, 0x857f, PDF_CMAP_SINGLE, 4175 },
	{ 0x8581, 0x8581, PDF_CMAP_SINGLE, 4176 },
	{ 0x8585, 0x8586, PDF_CMAP_RANGE, 4177 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 4179 },
	{ 0x858b, 0x858c, PDF_CMAP_RANGE, 4180 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 4182 },
	{ 0x8593, 0x8593, PDF_CMAP_SINGLE, 4183 },
	{ 0x8598, 0x8598, PDF_CMAP_SINGLE, 4184 },
	{ 0x859d, 0x859d, PDF_CMAP_SINGLE, 4185 },
	{ 0x859f, 0x85a0, PDF_CMAP_RANGE, 4186 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 4188 },
	{ 0x85a5, 0x85a5, PDF_CMAP_SINGLE, 4189 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 4190 },
	{ 0x85ad, 0x85ad, PDF_CMAP_SINGLE, 4204 },
	{ 0x85b4, 0x85b4, PDF_CMAP_SINGLE, 4191 },
	{ 0x85b6, 0x85b8, PDF_CMAP_RANGE, 4192 },
	{ 0x85bc, 0x85bf, PDF_CMAP_RANGE, 4195 },
	{ 0x85c2, 0x85c2, PDF_CMAP_SINGLE, 4199 },
	{ 0x85c7, 0x85c7, PDF_CMAP_SINGLE, 4200 },
	{ 0x85ca, 0x85cb, PDF_CMAP_RANGE, 4201 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 4203 },
	{ 0x85d8, 0x85d8, PDF_CMAP_SINGLE, 4205 },
	{ 0x85da, 0x85da, PDF_CMAP_SINGLE, 4206 },
	{ 0x85df, 0x85e0, PDF_CMAP_RANGE, 4207 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 4209 },
	{ 0x85e8, 0x85e8, PDF_CMAP_SINGLE, 4210 },
	{ 0x85ed, 0x85ed, PDF_CMAP_SINGLE, 4211 },
	{ 0x85f3, 0x85f3, PDF_CMAP_SINGLE, 4212 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 4213 },
	{ 0x85fc, 0x85fc, PDF_CMAP_SINGLE, 4214 },
	{ 0x85ff, 0x8600, PDF_CMAP_RANGE, 4215 },
	{ 0x8604, 0x8605, PDF_CMAP_RANGE, 4217 },
	{ 0x860d, 0x860e, PDF_CMAP_RANGE, 4219 },
	{ 0x8610, 0x8612, PDF_CMAP_RANGE, 4221 },
	{ 0x8618, 0x8619, PDF_CMAP_RANGE, 4224 },
	{ 0x861b, 0x861b, PDF_CMAP_SINGLE, 4226 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 4227 },
	{ 0x8621, 0x8621, PDF_CMAP_SINGLE, 4228 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 4229 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 4230 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 4231 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 4232 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 4233 },
	{ 0x863c, 0x863d, PDF_CMAP_RANGE, 4234 },
	{ 0x8640, 0x8642, PDF_CMAP_TABLE, 261 },
	{ 0x8646, 0x8646, PDF_CMAP_SINGLE, 4238 },
	{ 0x8652, 0x8653, PDF_CMAP_RANGE, 4239 },
	{ 0x8656, 0x8659, PDF_CMAP_RANGE, 4241 },
	{ 0x865d, 0x865d, PDF_CMAP_SINGLE, 4245 },
	{ 0x8660, 0x8664, PDF_CMAP_RANGE, 4246 },
	{ 0x8669, 0x8669, PDF_CMAP_SINGLE, 4251 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 4252 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 4253 },
	{ 0x8675, 0x8677, PDF_CMAP_RANGE, 4254 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 4257 },
	{ 0x8688, 0x8688, PDF_CMAP_SINGLE, 4291 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 4258 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 4259 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 4260 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 4261 },
	{ 0x869a, 0x869a, PDF_CMAP_SINGLE, 4262 },
	{ 0x869c, 0x869c, PDF_CMAP_SINGLE, 4263 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 4264 },
	{ 0x86a6, 0x86a8, PDF_CMAP_RANGE, 4265 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 4268 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 4269 },
	{ 0x86b3, 0x86b5, PDF_CMAP_RANGE, 4270 },
	{ 0x86b7, 0x86b9, PDF_CMAP_RANGE, 4273 },
	{ 0x86bf, 0x86c1, PDF_CMAP_RANGE, 4276 },
	{ 0x86c3, 0x86c3, PDF_CMAP_SINGLE, 4279 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 4280 },
	{ 0x86d1, 0x86d2, PDF_CMAP_RANGE, 4281 },
	{ 0x86d5, 0x86d5, PDF_CMAP_SINGLE, 4283 },
	{ 0x86d7, 0x86d7, PDF_CMAP_SINGLE, 4284 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 4285 },
	{ 0x86dc, 0x86dc, PDF_CMAP_SINGLE, 4286 },
	{ 0x86e0, 0x86e0, PDF_CMAP_SINGLE, 4287 },
	{ 0x86e3, 0x86e3, PDF_CMAP_SINGLE, 4288 },
	{ 0x86e5, 0x86e5, PDF_CMAP_SINGLE, 4289 },
	{ 0x86e7, 0x86e7, PDF_CMAP_SINGLE, 4290 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 4292 },
	{ 0x86fc, 0x86fd, PDF_CMAP_RANGE, 4293 },
	{ 0x8704, 0x8705, PDF_CMAP_RANGE, 4295 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 4297 },
	{ 0x870b, 0x870b, PDF_CMAP_SINGLE, 4298 },
	{ 0x870e, 0x8710, PDF_CMAP_RANGE, 4299 },
	{ 0x8713, 0x8714, PDF_CMAP_RANGE, 4302 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 4304 },
	{ 0x871e, 0x871f, PDF_CMAP_RANGE, 4305 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 4307 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 4308 },
	{ 0x8728, 0x8728, PDF_CMAP_SINGLE, 4309 },
	{ 0x872e, 0x872f, PDF_CMAP_RANGE, 4310 },
	{ 0x8731, 0x8732, PDF_CMAP_RANGE, 4312 },
	{ 0x8739, 0x873a, PDF_CMAP_RANGE, 4314 },
	{ 0x873c, 0x873e, PDF_CMAP_RANGE, 4316 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 4319 },
	{ 0x8743, 0x8743, PDF_CMAP_SINGLE, 4320 },
	{ 0x8745, 0x8745, PDF_CMAP_SINGLE, 4321 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 4322 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 4323 },
	{ 0x875d, 0x875d, PDF_CMAP_SINGLE, 4324 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 4325 },
	{ 0x8764, 0x8765, PDF_CMAP_RANGE, 4326 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 4328 },
	{ 0x8771, 0x8772, PDF_CMAP_RANGE, 4329 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 4331 },
	{ 0x8783, 0x8789, PDF_CMAP_RANGE, 4332 },
	{ 0x878b, 0x878c, PDF_CMAP_RANGE, 4339 },
	{ 0x8790, 0x8790, PDF_CMAP_SINGLE, 4341 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 4342 },
	{ 0x8795, 0x8795, PDF_CMAP_SINGLE, 4343 },
	{ 0x8797, 0x8799, PDF_CMAP_RANGE, 4344 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 4347 },
	{ 0x87a0, 0x87a0, PDF_CMAP_SINGLE, 4348 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 4349 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 4350 },
	{ 0x87ac, 0x87ae, PDF_CMAP_RANGE, 4351 },
	{ 0x87b1, 0x87b1, PDF_CMAP_SINGLE, 4354 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 4355 },
	{ 0x87be, 0x87bf, PDF_CMAP_RANGE, 4356 },
	{ 0x87c1, 0x87c1, PDF_CMAP_SINGLE, 4358 },
	{ 0x87c8, 0x87ca, PDF_CMAP_RANGE, 4359 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 4362 },
	{ 0x87d5, 0x87d6, PDF_CMAP_RANGE, 4363 },
	{ 0x87d9, 0x87da, PDF_CMAP_RANGE, 4365 },
	{ 0x87dc, 0x87dc, PDF_CMAP_SINGLE, 4367 },
	{ 0x87df, 0x87df, PDF_CMAP_SINGLE, 4368 },
	{ 0x87e2, 0x87e4, PDF_CMAP_RANGE, 4369 },
	{ 0x87ea, 0x87eb, PDF_CMAP_RANGE, 4372 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 4374 },
	{ 0x87f1, 0x87f1, PDF_CMAP_SINGLE, 4375 },
	{ 0x87f3, 0x87f3, PDF_CMAP_SINGLE, 4376 },
	{ 0x87f8, 0x87f8, PDF_CMAP_SINGLE, 4377 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 4378 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 4379 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 4380 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 4381 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 4382 },
	{ 0x8809, 0x880b, PDF_CMAP_RANGE, 4383 },
	{ 0x8810, 0x8810, PDF_CMAP_SINGLE, 4386 },
	{ 0x8812, 0x8814, PDF_CMAP_RANGE, 4388 },
	{ 0x8818, 0x8819, PDF_CMAP_TABLE, 264 },
	{ 0x881a, 0x881c, PDF_CMAP_RANGE, 4392 },
	{ 0x881e, 0x881f, PDF_CMAP_RANGE, 4395 },
	{ 0x8828, 0x8828, PDF_CMAP_SINGLE, 4397 },
	{ 0x882d, 0x882e, PDF_CMAP_RANGE, 4398 },
	{ 0x8830, 0x8830, PDF_CMAP_SINGLE, 4400 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 4401 },
	{ 0x8835, 0x8835, PDF_CMAP_SINGLE, 4402 },
	{ 0x883a, 0x883a, PDF_CMAP_SINGLE, 4403 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 4404 },
	{ 0x8841, 0x8841, PDF_CMAP_SINGLE, 4405 },
	{ 0x8843, 0x8843, PDF_CMAP_SINGLE, 4406 },
	{ 0x8845, 0x8845, PDF_CMAP_SINGLE, 4407 },
	{ 0x8848, 0x884b, PDF_CMAP_RANGE, 4408 },
	{ 0x884e, 0x884e, PDF_CMAP_SINGLE, 4412 },
	{ 0x8851, 0x8851, PDF_CMAP_SINGLE, 4413 },
	{ 0x8855, 0x8856, PDF_CMAP_RANGE, 4414 },
	{ 0x8858, 0x8858, PDF_CMAP_SINGLE, 4416 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 4417 },
	{ 0x885c, 0x885c, PDF_CMAP_SINGLE, 4418 },
	{ 0x885f, 0x8860, PDF_CMAP_RANGE, 4419 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 4421 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 4422 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 4423 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 4424 },
	{ 0x887b, 0x887b, PDF_CMAP_SINGLE, 4425 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 4426 },
	{ 0x8898, 0x8898, PDF_CMAP_SINGLE, 4427 },
	{ 0x889a, 0x889c, PDF_CMAP_RANGE, 4428 },
	{ 0x889f, 0x88a0, PDF_CMAP_RANGE, 4431 },
	{ 0x88a8, 0x88a8, PDF_CMAP_SINGLE, 4433 },
	{ 0x88aa, 0x88aa, PDF_CMAP_SINGLE, 4434 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 4435 },
	{ 0x88bd, 0x88be, PDF_CMAP_RANGE, 4436 },
	{ 0x88c0, 0x88c0, PDF_CMAP_SINGLE, 4438 },
	{ 0x88ca, 0x88ce, PDF_CMAP_RANGE, 4439 },
	{ 0x88d1, 0x88d3, PDF_CMAP_RANGE, 4444 },
	{ 0x88db, 0x88db, PDF_CMAP_SINGLE, 4447 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 4448 },
	{ 0x88e7, 0x88e7, PDF_CMAP_SINGLE, 4449 },
	{ 0x88ef, 0x88f1, PDF_CMAP_RANGE, 4450 },
	{ 0x88f5, 0x88f5, PDF_CMAP_SINGLE, 4453 },
	{ 0x88f7, 0x88f7, PDF_CMAP_SINGLE, 4454 },
	{ 0x8901, 0x8901, PDF_CMAP_SINGLE, 4455 },
	{ 0x8906, 0x8906, PDF_CMAP_SINGLE, 4456 },
	{ 0x890d, 0x890f, PDF_CMAP_RANGE, 4457 },
	{ 0x8915, 0x8916, PDF_CMAP_RANGE, 4460 },
	{ 0x8918, 0x891a, PDF_CMAP_RANGE, 4462 },
	{ 0x891c, 0x891c, PDF_CMAP_SINGLE, 4465 },
	{ 0x8920, 0x8920, PDF_CMAP_SINGLE, 4466 },
	{ 0x8926, 0x8928, PDF_CMAP_RANGE, 4467 },
	{ 0x8930, 0x8932, PDF_CMAP_RANGE, 4470 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 4473 },
	{ 0x8939, 0x893a, PDF_CMAP_RANGE, 4474 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 4476 },
	{ 0x8940, 0x8940, PDF_CMAP_SINGLE, 4477 },
	{ 0x8942, 0x8942, PDF_CMAP_SINGLE, 4478 },
	{ 0x8945, 0x8946, PDF_CMAP_RANGE, 4479 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 4481 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 4482 },
	{ 0x8952, 0x8952, PDF_CMAP_SINGLE, 4483 },
	{ 0x8957, 0x8957, PDF_CMAP_SINGLE, 4484 },
	{ 0x895a, 0x895c, PDF_CMAP_RANGE, 4485 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 4488 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 4491 },
	{ 0x896e, 0x896e, PDF_CMAP_SINGLE, 4492 },
	{ 0x8970, 0x8970, PDF_CMAP_SINGLE, 4493 },
	{ 0x8973, 0x8973, PDF_CMAP_SINGLE, 4494 },
	{ 0x8975, 0x8975, PDF_CMAP_SINGLE, 4495 },
	{ 0x897a, 0x897d, PDF_CMAP_RANGE, 4496 },
	{ 0x8989, 0x8989, PDF_CMAP_SINGLE, 4500 },
	{ 0x898d, 0x898d, PDF_CMAP_SINGLE, 4501 },
	{ 0x8990, 0x8990, PDF_CMAP_SINGLE, 4502 },
	{ 0x8994, 0x8995, PDF_CMAP_RANGE, 4503 },
	{ 0x899b, 0x899c, PDF_CMAP_RANGE, 4505 },
	{ 0x899f, 0x89a0, PDF_CMAP_RANGE, 4507 },
	{ 0x89a5, 0x89a5, PDF_CMAP_SINGLE, 4509 },
	{ 0x89b0, 0x89b0, PDF_CMAP_SINGLE, 4510 },
	{ 0x89b4, 0x89b7, PDF_CMAP_RANGE, 4511 },
	{ 0x89bc, 0x89bc, PDF_CMAP_SINGLE, 4515 },
	{ 0x89d4, 0x89d8, PDF_CMAP_RANGE, 4516 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 4521 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 4522 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 4523 },
	{ 0x89ed, 0x89ed, PDF_CMAP_SINGLE, 4524 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 4525 },
	{ 0x89f3, 0x89f3, PDF_CMAP_SINGLE, 4526 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 4527 },
	{ 0x89f9, 0x89f9, PDF_CMAP_SINGLE, 4528 },
	{ 0x89fd, 0x89fd, PDF_CMAP_SINGLE, 4529 },
	{ 0x89ff, 0x89ff, PDF_CMAP_SINGLE, 4530 },
	{ 0x8a04, 0x8a05, PDF_CMAP_RANGE, 4531 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 4533 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 4534 },
	{ 0x8a11, 0x8a12, PDF_CMAP_RANGE, 4535 },
	{ 0x8a14, 0x8a15, PDF_CMAP_RANGE, 4537 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 4539 },
	{ 0x8a20, 0x8a20, PDF_CMAP_SINGLE, 4540 },
	{ 0x8a22, 0x8a22, PDF_CMAP_SINGLE, 4541 },
	{ 0x8a24, 0x8a24, PDF_CMAP_SINGLE, 4542 },
	{ 0x8a26, 0x8a26, PDF_CMAP_SINGLE, 4543 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 4544 },
	{ 0x8a2f, 0x8a2f, PDF_CMAP_SINGLE, 4546 },
	{ 0x8a35, 0x8a35, PDF_CMAP_SINGLE, 4547 },
	{ 0x8a37, 0x8a37, PDF_CMAP_SINGLE, 4548 },
	{ 0x8a3d, 0x8a3e, PDF_CMAP_RANGE, 4549 },
	{ 0x8a40, 0x8a40, PDF_CMAP_SINGLE, 4551 },
	{ 0x8a43, 0x8a43, PDF_CMAP_SINGLE, 4552 },
	{ 0x8a45, 0x8a45, PDF_CMAP_SINGLE, 4553 },
	{ 0x8a47, 0x8a47, PDF_CMAP_SINGLE, 4554 },
	{ 0x8a49, 0x8a49, PDF_CMAP_SINGLE, 4555 },
	{ 0x8a4d, 0x8a4e, PDF_CMAP_RANGE, 4556 },
	{ 0x8a53, 0x8a53, PDF_CMAP_SINGLE, 4558 },
	{ 0x8a56, 0x8a58, PDF_CMAP_RANGE, 4559 },
	{ 0x8a5c, 0x8a5d, PDF_CMAP_RANGE, 4562 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 4564 },
	{ 0x8a65, 0x8a65, PDF_CMAP_SINGLE, 4565 },
	{ 0x8a67, 0x8a67, PDF_CMAP_SINGLE, 4566 },
	{ 0x8a75, 0x8a77, PDF_CMAP_RANGE, 4567 },
	{ 0x8a79, 0x8a7b, PDF_CMAP_RANGE, 4570 },
	{ 0x8a7e, 0x8a80, PDF_CMAP_RANGE, 4573 },
	{ 0x8a83, 0x8a83, PDF_CMAP_SINGLE, 4576 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 4577 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 4578 },
	{ 0x8a8f, 0x8a90, PDF_CMAP_RANGE, 4579 },
	{ 0x8a92, 0x8a92, PDF_CMAP_SINGLE, 4581 },
	{ 0x8a96, 0x8a97, PDF_CMAP_RANGE, 4582 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 4584 },
	{ 0x8a9f, 0x8a9f, PDF_CMAP_SINGLE, 4585 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 4586 },
	{ 0x8aa9, 0x8aa9, PDF_CMAP_SINGLE, 4587 },
	{ 0x8aae, 0x8aaf, PDF_CMAP_RANGE, 4588 },
	{ 0x8ab3, 0x8ab3, PDF_CMAP_SINGLE, 4590 },
	{ 0x8ab6, 0x8ab7, PDF_CMAP_RANGE, 4591 },
	{ 0x8abb, 0x8abb, PDF_CMAP_SINGLE, 4593 },
	{ 0x8abe, 0x8abe, PDF_CMAP_SINGLE, 4594 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 4595 },
	{ 0x8ac6, 0x8ac6, PDF_CMAP_SINGLE, 4596 },
	{ 0x8ac8, 0x8aca, PDF_CMAP_RANGE, 4597 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 4600 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 4601 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 4604 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 4605 },
	{ 0x8adf, 0x8adf, PDF_CMAP_SINGLE, 4606 },
	{ 0x8aec, 0x8aec, PDF_CMAP_SINGLE, 4607 },
	{ 0x8af0, 0x8af0, PDF_CMAP_SINGLE, 4608 },
	{ 0x8af4, 0x8af6, PDF_CMAP_RANGE, 4609 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 4612 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 4613 },
	{ 0x8b05, 0x8b06, PDF_CMAP_RANGE, 4614 },
	{ 0x8b0a, 0x8b0b, PDF_CMAP_TABLE, 266 },
	{ 0x8b11, 0x8b11, PDF_CMAP_SINGLE, 4617 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 4618 },
	{ 0x8b1e, 0x8b1f, PDF_CMAP_RANGE, 4619 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 4622 },
	{ 0x8b30, 0x8b30, PDF_CMAP_SINGLE, 4623 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 4624 },
	{ 0x8b3c, 0x8b3c, PDF_CMAP_SINGLE, 4625 },
	{ 0x8b42, 0x8b46, PDF_CMAP_RANGE, 4626 },
	{ 0x8b48, 0x8b48, PDF_CMAP_SINGLE, 4631 },
	{ 0x8b4d, 0x8b4d, PDF_CMAP_SINGLE, 4636 },
	{ 0x8b52, 0x8b54, PDF_CMAP_RANGE, 4632 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 4635 },
	{ 0x8b5e, 0x8b5e, PDF_CMAP_SINGLE, 4637 },
	{ 0x8b63, 0x8b63, PDF_CMAP_SINGLE, 4638 },
	{ 0x8b6d, 0x8b6d, PDF_CMAP_SINGLE, 4639 },
	{ 0x8b76, 0x8b76, PDF_CMAP_SINGLE, 4640 },
	{ 0x8b78, 0x8b79, PDF_CMAP_RANGE, 4641 },
	{ 0x8b7c, 0x8b7c, PDF_CMAP_SINGLE, 4643 },
	{ 0x8b7e, 0x8b7e, PDF_CMAP_SINGLE, 4644 },
	{ 0x8b81, 0x8b81, PDF_CMAP_SINGLE, 4645 },
	{ 0x8b84, 0x8b85, PDF_CMAP_RANGE, 4646 },
	{ 0x8b8b, 0x8b8b, PDF_CMAP_SINGLE, 4648 },
	{ 0x8b8d, 0x8b8d, PDF_CMAP_SINGLE, 4649 },
	{ 0x8b8f, 0x8b8f, PDF_CMAP_SINGLE, 4650 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 4651 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 4653 },
	{ 0x8b9e, 0x8b9f, PDF_CMAP_RANGE, 4654 },
	{ 0x8c38, 0x8c39, PDF_CMAP_RANGE, 4656 },
	{ 0x8c3d, 0x8c3e, PDF_CMAP_RANGE, 4658 },
	{ 0x8c45, 0x8c45, PDF_CMAP_SINGLE, 4660 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 4661 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 4662 },
	{ 0x8c4b, 0x8c4b, PDF_CMAP_SINGLE, 4663 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 4664 },
	{ 0x8c51, 0x8c51, PDF_CMAP_SINGLE, 4665 },
	{ 0x8c53, 0x8c54, PDF_CMAP_RANGE, 4666 },
	{ 0x8c57, 0x8c58, PDF_CMAP_RANGE, 4668 },
	{ 0x8c59, 0x8c59, PDF_CMAP_SINGLE, 4672 },
	{ 0x8c5b, 0x8c5b, PDF_CMAP_SINGLE, 4670 },
	{ 0x8c5d, 0x8c5d, PDF_CMAP_SINGLE, 4671 },
	{ 0x8c63, 0x8c64, PDF_CMAP_RANGE, 4673 },
	{ 0x8c66, 0x8c66, PDF_CMAP_SINGLE, 4675 },
	{ 0x8c68, 0x8c69, PDF_CMAP_RANGE, 4676 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 4678 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 4679 },
	{ 0x8c75, 0x8c76, PDF_CMAP_RANGE, 4680 },
	{ 0x8c7b, 0x8c7b, PDF_CMAP_SINGLE, 4682 },
	{ 0x8c7e, 0x8c7e, PDF_CMAP_SINGLE, 4683 },
	{ 0x8c86, 0x8c87, PDF_CMAP_RANGE, 4684 },
	{ 0x8c8b, 0x8c8b, PDF_CMAP_SINGLE, 4686 },
	{ 0x8c90, 0x8c90, PDF_CMAP_SINGLE, 4687 },
	{ 0x8c92, 0x8c93, PDF_CMAP_RANGE, 4688 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 4690 },
	{ 0x8c9b, 0x8c9c, PDF_CMAP_RANGE, 4691 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 4693 },
	{ 0x8cb9, 0x8cba, PDF_CMAP_RANGE, 4694 },
	{ 0x8cc5, 0x8cc6, PDF_CMAP_RANGE, 4696 },
	{ 0x8cc9, 0x8cc9, PDF_CMAP_SINGLE, 4698 },
	{ 0x8ccb, 0x8ccb, PDF_CMAP_SINGLE, 4699 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 4700 },
	{ 0x8cd5, 0x8cd6, PDF_CMAP_TABLE, 268 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 4703 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 4704 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 4705 },
	{ 0x8ce8, 0x8ce8, PDF_CMAP_SINGLE, 4706 },
	{ 0x8cec, 0x8cec, PDF_CMAP_SINGLE, 4707 },
	{ 0x8cef, 0x8cf0, PDF_CMAP_RANGE, 4708 },
	{ 0x8cf2, 0x8cf2, PDF_CMAP_SINGLE, 4710 },
	{ 0x8cf5, 0x8cf5, PDF_CMAP_SINGLE, 4711 },
	{ 0x8cf7, 0x8cf8, PDF_CMAP_RANGE, 4712 },
	{ 0x8cfe, 0x8cff, PDF_CMAP_RANGE, 4714 },
	{ 0x8d01, 0x8d01, PDF_CMAP_SINGLE, 4716 },
	{ 0x8d03, 0x8d03, PDF_CMAP_SINGLE, 4717 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 4718 },
	{ 0x8d12, 0x8d12, PDF_CMAP_SINGLE, 4719 },
	{ 0x8d17, 0x8d17, PDF_CMAP_SINGLE, 4720 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 4721 },
	{ 0x8d65, 0x8d65, PDF_CMAP_SINGLE, 4722 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 4723 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 4724 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 4725 },
	{ 0x8d7f, 0x8d7f, PDF_CMAP_SINGLE, 4726 },
	{ 0x8d82, 0x8d82, PDF_CMAP_SINGLE, 4727 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 4728 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 4729 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 4730 },
	{ 0x8d90, 0x8d91, PDF_CMAP_RANGE, 4731 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 4733 },
	{ 0x8d9e, 0x8da0, PDF_CMAP_RANGE, 4734 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 4737 },
	{ 0x8dab, 0x8dac, PDF_CMAP_RANGE, 4738 },
	{ 0x8daf, 0x8daf, PDF_CMAP_SINGLE, 4740 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 4741 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 4742 },
	{ 0x8db7, 0x8db7, PDF_CMAP_SINGLE, 4743 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 4744 },
	{ 0x8dbb, 0x8dbc, PDF_CMAP_TABLE, 270 },
	{ 0x8dc0, 0x8dc0, PDF_CMAP_SINGLE, 4746 },
	{ 0x8dc5, 0x8dc8, PDF_CMAP_RANGE, 4747 },
	{ 0x8dca, 0x8dca, PDF_CMAP_SINGLE, 4751 },
	{ 0x8dce, 0x8dce, PDF_CMAP_SINGLE, 4752 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 4753 },
	{ 0x8dd4, 0x8dd5, PDF_CMAP_RANGE, 4754 },
	{ 0x8dd7, 0x8dd7, PDF_CMAP_SINGLE, 4756 },
	{ 0x8dd9, 0x8dd9, PDF_CMAP_SINGLE, 4757 },
	{ 0x8de4, 0x8de5, PDF_CMAP_RANGE, 4758 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 4760 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 4761 },
	{ 0x8df0, 0x8df0, PDF_CMAP_SINGLE, 4762 },
	{ 0x8df1, 0x8df2, PDF_CMAP_RANGE, 4764 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 4766 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 4767 },
	{ 0x8e01, 0x8e01, PDF_CMAP_SINGLE, 4768 },
	{ 0x8e04, 0x8e06, PDF_CMAP_RANGE, 4769 },
	{ 0x8e0b, 0x8e0b, PDF_CMAP_SINGLE, 4772 },
	{ 0x8e11, 0x8e11, PDF_CMAP_SINGLE, 4773 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 4774 },
	{ 0x8e16, 0x8e16, PDF_CMAP_SINGLE, 4775 },
	{ 0x8e20, 0x8e23, PDF_CMAP_RANGE, 4776 },
	{ 0x8e26, 0x8e27, PDF_CMAP_RANGE, 4780 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 4782 },
	{ 0x8e33, 0x8e33, PDF_CMAP_SINGLE, 4783 },
	{ 0x8e36, 0x8e39, PDF_CMAP_RANGE, 4784 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 4788 },
	{ 0x8e40, 0x8e41, PDF_CMAP_RANGE, 4789 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 4791 },
	{ 0x8e4d, 0x8e4f, PDF_CMAP_RANGE, 4792 },
	{ 0x8e54, 0x8e54, PDF_CMAP_SINGLE, 4795 },
	{ 0x8e5b, 0x8e5e, PDF_CMAP_RANGE, 4796 },
	{ 0x8e61, 0x8e62, PDF_CMAP_RANGE, 4800 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 4802 },
	{ 0x8e6c, 0x8e6d, PDF_CMAP_RANGE, 4803 },
	{ 0x8e6f, 0x8e71, PDF_CMAP_RANGE, 4805 },
	{ 0x8e79, 0x8e7b, PDF_CMAP_RANGE, 4808 },
	{ 0x8e82, 0x8e83, PDF_CMAP_RANGE, 4811 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 4813 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 4814 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 4815 },
	{ 0x8e95, 0x8e95, PDF_CMAP_SINGLE, 4816 },
	{ 0x8e9a, 0x8e9b, PDF_CMAP_RANGE, 4817 },
	{ 0x8e9d, 0x8e9e, PDF_CMAP_RANGE, 4819 },
	{ 0x8ea2, 0x8ea2, PDF_CMAP_SINGLE, 4821 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 4822 },
	{ 0x8ea9, 0x8ea9, PDF_CMAP_SINGLE, 4823 },
	{ 0x8ead, 0x8eae, PDF_CMAP_RANGE, 4824 },
	{ 0x8eb3, 0x8eb3, PDF_CMAP_SINGLE, 4826 },
	{ 0x8eb5, 0x8eb5, PDF_CMAP_SINGLE, 4827 },
	{ 0x8eba, 0x8ebb, PDF_CMAP_RANGE, 4828 },
	{ 0x8ec0, 0x8ec1, PDF_CMAP_RANGE, 4830 },
	{ 0x8ec3, 0x8ec4, PDF_CMAP_RANGE, 4832 },
	{ 0x8ec7, 0x8ec7, PDF_CMAP_SINGLE, 4834 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 4835 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 4836 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 4837 },
	{ 0x8edc, 0x8edc, PDF_CMAP_SINGLE, 4838 },
	{ 0x8ee8, 0x8ee8, PDF_CMAP_SINGLE, 4839 },
	{ 0x8eed, 0x8eee, PDF_CMAP_TABLE, 272 },
	{ 0x8ef0, 0x8ef1, PDF_CMAP_RANGE, 4841 },
	{ 0x8ef7, 0x8ef7, PDF_CMAP_SINGLE, 4843 },
	{ 0x8ef9, 0x8efa, PDF_CMAP_RANGE, 4844 },
	{ 0x8f00, 0x8f00, PDF_CMAP_SINGLE, 4847 },
	{ 0x8f02, 0x8f02, PDF_CMAP_SINGLE, 4848 },
	{ 0x8f07, 0x8f08, PDF_CMAP_RANGE, 4849 },
	{ 0x8f0f, 0x8f10, PDF_CMAP_RANGE, 4851 },
	{ 0x8f16, 0x8f18, PDF_CMAP_RANGE, 4853 },
	{ 0x8f1e, 0x8f1e, PDF_CMAP_SINGLE, 4856 },
	{ 0x8f20, 0x8f21, PDF_CMAP_RANGE, 4857 },
	{ 0x8f23, 0x8f23, PDF_CMAP_SINGLE, 4859 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 4860 },
	{ 0x8f27, 0x8f28, PDF_CMAP_RANGE, 4861 },
	{ 0x8f2c, 0x8f2e, PDF_CMAP_RANGE, 4863 },
	{ 0x8f34, 0x8f37, PDF_CMAP_RANGE, 4866 },
	{ 0x8f3a, 0x8f3a, PDF_CMAP_SINGLE, 4870 },
	{ 0x8f40, 0x8f41, PDF_CMAP_RANGE, 4871 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 4873 },
	{ 0x8f47, 0x8f47, PDF_CMAP_SINGLE, 4874 },
	{ 0x8f4f, 0x8f4f, PDF_CMAP_SINGLE, 4875 },
	{ 0x8f51, 0x8f55, PDF_CMAP_RANGE, 4876 },
	{ 0x8f58, 0x8f58, PDF_CMAP_SINGLE, 4881 },
	{ 0x8f5d, 0x8f5e, PDF_CMAP_RANGE, 4882 },
	{ 0x8f65, 0x8f65, PDF_CMAP_SINGLE, 4884 },
	{ 0x8f9d, 0x8f9d, PDF_CMAP_SINGLE, 4885 },
	{ 0x8fa0, 0x8fa1, PDF_CMAP_RANGE, 4886 },
	{ 0x8fa4, 0x8fa6, PDF_CMAP_RANGE, 4888 },
	{ 0x8fb5, 0x8fb6, PDF_CMAP_RANGE, 4891 },
	{ 0x8fb8, 0x8fb8, PDF_CMAP_SINGLE, 4893 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 4894 },
	{ 0x8fc0, 0x8fc1, PDF_CMAP_RANGE, 4895 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 4897 },
	{ 0x8fca, 0x8fcb, PDF_CMAP_RANGE, 4898 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 4900 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 4901 },
	{ 0x8fd2, 0x8fd3, PDF_CMAP_RANGE, 4902 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 4904 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 4905 },
	{ 0x8fe3, 0x8fe4, PDF_CMAP_RANGE, 4906 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 4908 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 4909 },
	{ 0x8ff1, 0x8ff1, PDF_CMAP_SINGLE, 4910 },
	{ 0x8ff5, 0x8ff6, PDF_CMAP_RANGE, 4911 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 4913 },
	{ 0x8ffe, 0x8ffe, PDF_CMAP_SINGLE, 4914 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 4915 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 4916 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 4917 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 4918 },
	{ 0x9018, 0x9018, PDF_CMAP_SINGLE, 4919 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 4920 },
	{ 0x9028, 0x9029, PDF_CMAP_RANGE, 4921 },
	{ 0x902a, 0x902a, PDF_CMAP_SINGLE, 4924 },
	{ 0x902c, 0x902d, PDF_CMAP_RANGE, 4925 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 4923 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 4927 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 4929 },
	{ 0x903f, 0x903f, PDF_CMAP_SINGLE, 4930 },
	{ 0x9043, 0x9044, PDF_CMAP_RANGE, 4931 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 4933 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 4934 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 4935 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 4936 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 4937 },
	{ 0x906c, 0x906c, PDF_CMAP_SINGLE, 4939 },
	{ 0x9070, 0x9070, PDF_CMAP_SINGLE, 4940 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 4941 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 4942 },
	{ 0x9085, 0x9085, PDF_CMAP_SINGLE, 4943 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 4944 },
	{ 0x908b, 0x908c, PDF_CMAP_RANGE, 4945 },
	{ 0x908e, 0x908e, PDF_CMAP_SINGLE, 4947 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 4948 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 4949 },
	{ 0x9097, 0x9099, PDF_CMAP_RANGE, 4950 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 4953 },
	{ 0x90a0, 0x90a2, PDF_CMAP_RANGE, 4954 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 4957 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 4958 },
	{ 0x90b2, 0x90b4, PDF_CMAP_RANGE, 4959 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 4962 },
	{ 0x90bd, 0x90be, PDF_CMAP_TABLE, 274 },
	{ 0x90c3, 0x90c5, PDF_CMAP_RANGE, 4966 },
	{ 0x90c7, 0x90c8, PDF_CMAP_RANGE, 4969 },
	{ 0x90cc, 0x90cc, PDF_CMAP_SINGLE, 4964 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 4979 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 4971 },
	{ 0x90d7, 0x90d9, PDF_CMAP_RANGE, 4972 },
	{ 0x90dc, 0x90dd, PDF_CMAP_RANGE, 4975 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 4977 },
	{ 0x90e5, 0x90e5, PDF_CMAP_SINGLE, 4978 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 4981 },
	{ 0x90ef, 0x90f0, PDF_CMAP_RANGE, 4982 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 4984 },
	{ 0x90f6, 0x90f6, PDF_CMAP_SINGLE, 4980 },
	{ 0x90fe, 0x9100, PDF_CMAP_RANGE, 4985 },
	{ 0x9104, 0x9106, PDF_CMAP_RANGE, 4988 },
	{ 0x9108, 0x9108, PDF_CMAP_SINGLE, 4991 },
	{ 0x910d, 0x910d, PDF_CMAP_SINGLE, 4992 },
	{ 0x9110, 0x9110, PDF_CMAP_SINGLE, 4993 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 4994 },
	{ 0x9116, 0x9118, PDF_CMAP_RANGE, 4995 },
	{ 0x911a, 0x911a, PDF_CMAP_SINGLE, 4998 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 4999 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 5000 },
	{ 0x9120, 0x9120, PDF_CMAP_SINGLE, 5001 },
	{ 0x9122, 0x9123, PDF_CMAP_RANGE, 5003 },
	{ 0x9125, 0x9125, PDF_CMAP_SINGLE, 5002 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 5005 },
	{ 0x9129, 0x9129, PDF_CMAP_SINGLE, 5006 },
	{ 0x912e, 0x912f, PDF_CMAP_RANGE, 5007 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 5009 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 5010 },
	{ 0x9136, 0x9137, PDF_CMAP_RANGE, 5011 },
	{ 0x9139, 0x913a, PDF_CMAP_RANGE, 5013 },
	{ 0x913c, 0x913d, PDF_CMAP_RANGE, 5015 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 5017 },
	{ 0x9147, 0x9148, PDF_CMAP_RANGE, 5018 },
	{ 0x914f, 0x914f, PDF_CMAP_SINGLE, 5020 },
	{ 0x9153, 0x9153, PDF_CMAP_SINGLE, 5021 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 5022 },
	{ 0x9159, 0x915b, PDF_CMAP_RANGE, 5023 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 5026 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 5027 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 5028 },
	{ 0x916d, 0x916d, PDF_CMAP_SINGLE, 5029 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 5030 },
	{ 0x9179, 0x917b, PDF_CMAP_RANGE, 5031 },
	{ 0x9181, 0x9181, PDF_CMAP_SINGLE, 5034 },
	{ 0x9183, 0x9183, PDF_CMAP_SINGLE, 5035 },
	{ 0x9185, 0x9186, PDF_CMAP_RANGE, 5036 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 5038 },
	{ 0x918e, 0x918e, PDF_CMAP_SINGLE, 5039 },
	{ 0x9191, 0x9191, PDF_CMAP_SINGLE, 5040 },
	{ 0x9193, 0x9195, PDF_CMAP_RANGE, 5041 },
	{ 0x9198, 0x9198, PDF_CMAP_SINGLE, 5044 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 5045 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 5046 },
	{ 0x91a6, 0x91a6, PDF_CMAP_SINGLE, 5047 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 5048 },
	{ 0x91ac, 0x91ae, PDF_CMAP_RANGE, 5049 },
	{ 0x91b0, 0x91b3, PDF_CMAP_RANGE, 5052 },
	{ 0x91b6, 0x91b6, PDF_CMAP_SINGLE, 5056 },
	{ 0x91bb, 0x91bd, PDF_CMAP_RANGE, 5057 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 5060 },
	{ 0x91c2, 0x91c3, PDF_CMAP_RANGE, 5061 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 5063 },
	{ 0x91d3, 0x91d4, PDF_CMAP_RANGE, 5064 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 5066 },
	{ 0x91d9, 0x91da, PDF_CMAP_RANGE, 5067 },
	{ 0x91de, 0x91de, PDF_CMAP_SINGLE, 5069 },
	{ 0x91e4, 0x91e5, PDF_CMAP_RANGE, 5070 },
	{ 0x91e9, 0x91ea, PDF_CMAP_RANGE, 5072 },
	{ 0x91ec, 0x91f1, PDF_CMAP_RANGE, 5074 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 5080 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 5081 },
	{ 0x91fb, 0x91fb, PDF_CMAP_SINGLE, 5082 },
	{ 0x91fd, 0x91fd, PDF_CMAP_SINGLE, 5083 },
	{ 0x9200, 0x9201, PDF_CMAP_RANGE, 5084 },
	{ 0x9204, 0x9207, PDF_CMAP_RANGE, 5086 },
	{ 0x9209, 0x920a, PDF_CMAP_RANGE, 5090 },
	{ 0x920c, 0x920c, PDF_CMAP_SINGLE, 5092 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 5093 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 5094 },
	{ 0x9216, 0x9216, PDF_CMAP_SINGLE, 5096 },
	{ 0x9218, 0x9218, PDF_CMAP_SINGLE, 5097 },
	{ 0x921c, 0x921d, PDF_CMAP_RANGE, 5098 },
	{ 0x9223, 0x9226, PDF_CMAP_RANGE, 5100 },
	{ 0x9228, 0x9228, PDF_CMAP_SINGLE, 5104 },
	{ 0x922e, 0x9230, PDF_CMAP_RANGE, 5105 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 5108 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 5109 },
	{ 0x9238, 0x923a, PDF_CMAP_RANGE, 5111 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 5114 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 5115 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 5116 },
	{ 0x9242, 0x9243, PDF_CMAP_RANGE, 5117 },
	{ 0x9246, 0x9247, PDF_CMAP_RANGE, 5119 },
	{ 0x924a, 0x924a, PDF_CMAP_SINGLE, 5121 },
	{ 0x924d, 0x924f, PDF_CMAP_RANGE, 5122 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 5125 },
	{ 0x9258, 0x9259, PDF_CMAP_RANGE, 5126 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 5128 },
	{ 0x9260, 0x9261, PDF_CMAP_RANGE, 5130 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 5132 },
	{ 0x9267, 0x9269, PDF_CMAP_RANGE, 5133 },
	{ 0x926e, 0x9270, PDF_CMAP_RANGE, 5136 },
	{ 0x9275, 0x9279, PDF_CMAP_RANGE, 5139 },
	{ 0x927b, 0x927d, PDF_CMAP_RANGE, 5144 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 5147 },
	{ 0x9288, 0x928a, PDF_CMAP_RANGE, 5148 },
	{ 0x928d, 0x928e, PDF_CMAP_RANGE, 5151 },
	{ 0x9292, 0x9292, PDF_CMAP_SINGLE, 5153 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 5154 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 5155 },
	{ 0x929f, 0x92a0, PDF_CMAP_RANGE, 5156 },
	{ 0x92a4, 0x92a5, PDF_CMAP_RANGE, 5158 },
	{ 0x92a7, 0x92a8, PDF_CMAP_RANGE, 5160 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 5162 },
	{ 0x92af, 0x92af, PDF_CMAP_SINGLE, 5163 },
	{ 0x92b2, 0x92b2, PDF_CMAP_SINGLE, 5164 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 5165 },
	{ 0x92b8, 0x92b8, PDF_CMAP_SINGLE, 5166 },
	{ 0x92ba, 0x92bd, PDF_CMAP_RANGE, 5167 },
	{ 0x92bf, 0x92c3, PDF_CMAP_RANGE, 5171 },
	{ 0x92c5, 0x92c8, PDF_CMAP_RANGE, 5176 },
	{ 0x92cb, 0x92ce, PDF_CMAP_RANGE, 5180 },
	{ 0x92d0, 0x92d0, PDF_CMAP_SINGLE, 5184 },
	{ 0x92d3, 0x92d3, PDF_CMAP_SINGLE, 5185 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 5186 },
	{ 0x92d7, 0x92d9, PDF_CMAP_RANGE, 5187 },
	{ 0x92dc, 0x92dd, PDF_CMAP_RANGE, 5190 },
	{ 0x92df, 0x92e1, PDF_CMAP_RANGE, 5192 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 5195 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 5196 },
	{ 0x92e7, 0x92e8, PDF_CMAP_RANGE, 5197 },
	{ 0x92ec, 0x92ec, PDF_CMAP_SINGLE, 5199 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 5200 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 5201 },
	{ 0x92f9, 0x92f9, PDF_CMAP_SINGLE, 5202 },
	{ 0x92fb, 0x92fb, PDF_CMAP_SINGLE, 5203 },
	{ 0x92ff, 0x9300, PDF_CMAP_RANGE, 5204 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 5206 },
	{ 0x9308, 0x9308, PDF_CMAP_SINGLE, 5207 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 5208 },
	{ 0x9311, 0x9311, PDF_CMAP_SINGLE, 5209 },
	{ 0x9314, 0x9315, PDF_CMAP_RANGE, 5210 },
	{ 0x931c, 0x931f, PDF_CMAP_RANGE, 5212 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 5216 },
	{ 0x9324, 0x9325, PDF_CMAP_RANGE, 5217 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 5219 },
	{ 0x9329, 0x932a, PDF_CMAP_RANGE, 5220 },
	{ 0x9333, 0x9334, PDF_CMAP_RANGE, 5222 },
	{ 0x9336, 0x9337, PDF_CMAP_RANGE, 5224 },
	{ 0x9347, 0x9349, PDF_CMAP_RANGE, 5226 },
	{ 0x9350, 0x9352, PDF_CMAP_RANGE, 5229 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 5232 },
	{ 0x9357, 0x9358, PDF_CMAP_RANGE, 5233 },
	{ 0x935a, 0x935a, PDF_CMAP_SINGLE, 5235 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 5236 },
	{ 0x9364, 0x9365, PDF_CMAP_RANGE, 5237 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 5239 },
	{ 0x9369, 0x936a, PDF_CMAP_RANGE, 5240 },
	{ 0x936d, 0x936d, PDF_CMAP_SINGLE, 5242 },
	{ 0x936f, 0x9371, PDF_CMAP_RANGE, 5243 },
	{ 0x9373, 0x9374, PDF_CMAP_RANGE, 5246 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 5248 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 5249 },
	{ 0x937d, 0x937d, PDF_CMAP_SINGLE, 5250 },
	{ 0x937f, 0x9382, PDF_CMAP_RANGE, 5251 },
	{ 0x9388, 0x9388, PDF_CMAP_SINGLE, 5255 },
	{ 0x938a, 0x938b, PDF_CMAP_RANGE, 5256 },
	{ 0x938d, 0x938d, PDF_CMAP_SINGLE, 5258 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 5259 },
	{ 0x9392, 0x9392, PDF_CMAP_SINGLE, 5260 },
	{ 0x9395, 0x9395, PDF_CMAP_SINGLE, 5261 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 5262 },
	{ 0x939b, 0x939b, PDF_CMAP_SINGLE, 5263 },
	{ 0x939e, 0x939e, PDF_CMAP_SINGLE, 5264 },
	{ 0x93a1, 0x93a1, PDF_CMAP_SINGLE, 5265 },
	{ 0x93a3, 0x93a4, PDF_CMAP_RANGE, 5266 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 5268 },
	{ 0x93a8, 0x93a9, PDF_CMAP_TABLE, 276 },
	{ 0x93ab, 0x93ab, PDF_CMAP_SINGLE, 5270 },
	{ 0x93b4, 0x93b6, PDF_CMAP_RANGE, 5271 },
	{ 0x93ba, 0x93ba, PDF_CMAP_SINGLE, 5274 },
	{ 0x93c1, 0x93c1, PDF_CMAP_SINGLE, 5276 },
	{ 0x93c4, 0x93c7, PDF_CMAP_RANGE, 5277 },
	{ 0x93c9, 0x93cd, PDF_CMAP_RANGE, 5281 },
	{ 0x93d3, 0x93d3, PDF_CMAP_SINGLE, 5286 },
	{ 0x93d9, 0x93d9, PDF_CMAP_SINGLE, 5287 },
	{ 0x93dc, 0x93dc, PDF_CMAP_SINGLE, 5288 },
	{ 0x93de, 0x93df, PDF_CMAP_RANGE, 5289 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 5291 },
	{ 0x93e6, 0x93e7, PDF_CMAP_RANGE, 5292 },
	{ 0x93f7, 0x93f8, PDF_CMAP_RANGE, 5295 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 5294 },
	{ 0x93fa, 0x93fb, PDF_CMAP_RANGE, 5297 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 5299 },
	{ 0x9401, 0x9402, PDF_CMAP_RANGE, 5300 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 5302 },
	{ 0x9408, 0x9409, PDF_CMAP_RANGE, 5303 },
	{ 0x940d, 0x940f, PDF_CMAP_RANGE, 5305 },
	{ 0x9415, 0x9417, PDF_CMAP_RANGE, 5308 },
	{ 0x941f, 0x941f, PDF_CMAP_SINGLE, 5311 },
	{ 0x942e, 0x942f, PDF_CMAP_RANGE, 5312 },
	{ 0x9431, 0x9434, PDF_CMAP_RANGE, 5314 },
	{ 0x943b, 0x943b, PDF_CMAP_SINGLE, 5318 },
	{ 0x943d, 0x943d, PDF_CMAP_SINGLE, 5320 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 5319 },
	{ 0x9443, 0x9443, PDF_CMAP_SINGLE, 5321 },
	{ 0x9445, 0x9445, PDF_CMAP_SINGLE, 5322 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 5323 },
	{ 0x944a, 0x944a, PDF_CMAP_SINGLE, 5324 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 5325 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 5326 },
	{ 0x9459, 0x9459, PDF_CMAP_SINGLE, 5327 },
	{ 0x945c, 0x945c, PDF_CMAP_SINGLE, 5328 },
	{ 0x945f, 0x945f, PDF_CMAP_SINGLE, 5329 },
	{ 0x9461, 0x9461, PDF_CMAP_SINGLE, 5330 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 5331 },
	{ 0x9468, 0x9468, PDF_CMAP_SINGLE, 5332 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 5333 },
	{ 0x946d, 0x946f, PDF_CMAP_RANGE, 5334 },
	{ 0x9471, 0x9472, PDF_CMAP_RANGE, 5337 },
	{ 0x9483, 0x9484, PDF_CMAP_TABLE, 278 },
	{ 0x9578, 0x9579, PDF_CMAP_RANGE, 5341 },
	{ 0x957e, 0x957e, PDF_CMAP_SINGLE, 5343 },
	{ 0x9584, 0x9584, PDF_CMAP_SINGLE, 5344 },
	{ 0x9588, 0x9588, PDF_CMAP_SINGLE, 5345 },
	{ 0x958c, 0x958e, PDF_CMAP_RANGE, 5346 },
	{ 0x959d, 0x959f, PDF_CMAP_RANGE, 5349 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 5352 },
	{ 0x95a6, 0x95a6, PDF_CMAP_SINGLE, 5353 },
	{ 0x95a9, 0x95a9, PDF_CMAP_SINGLE, 5354 },
	{ 0x95ab, 0x95ac, PDF_CMAP_RANGE, 5355 },
	{ 0x95b4, 0x95b4, PDF_CMAP_SINGLE, 5357 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 5358 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 5359 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 5360 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 5361 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 5362 },
	{ 0x95c8, 0x95c9, PDF_CMAP_RANGE, 5363 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 5365 },
	{ 0x95d0, 0x95d3, PDF_CMAP_RANGE, 5366 },
	{ 0x95d9, 0x95da, PDF_CMAP_RANGE, 5370 },
	{ 0x95dd, 0x95e0, PDF_CMAP_RANGE, 5372 },
	{ 0x95e4, 0x95e4, PDF_CMAP_SINGLE, 5376 },
	{ 0x95e6, 0x95e6, PDF_CMAP_SINGLE, 5377 },
	{ 0x961d, 0x961e, PDF_CMAP_RANGE, 5378 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 5380 },
	{ 0x9624, 0x9626, PDF_CMAP_RANGE, 5381 },
	{ 0x962c, 0x962c, PDF_CMAP_SINGLE, 5384 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 5385 },
	{ 0x9633, 0x9633, PDF_CMAP_SINGLE, 5386 },
	{ 0x9637, 0x963a, PDF_CMAP_RANGE, 5387 },
	{ 0x963c, 0x963d, PDF_CMAP_RANGE, 5391 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 5393 },
	{ 0x9652, 0x9652, PDF_CMAP_SINGLE, 5394 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 5395 },
	{ 0x9656, 0x9658, PDF_CMAP_RANGE, 5396 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 5399 },
	{ 0x966e, 0x966e, PDF_CMAP_SINGLE, 5400 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 5401 },
	{ 0x967b, 0x967c, PDF_CMAP_RANGE, 5402 },
	{ 0x967e, 0x967f, PDF_CMAP_RANGE, 5404 },
	{ 0x9681, 0x9684, PDF_CMAP_RANGE, 5406 },
	{ 0x9689, 0x9689, PDF_CMAP_SINGLE, 5410 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 5411 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 5412 },
	{ 0x969a, 0x969a, PDF_CMAP_SINGLE, 5413 },
	{ 0x969d, 0x969d, PDF_CMAP_SINGLE, 5414 },
	{ 0x969f, 0x969f, PDF_CMAP_SINGLE, 5415 },
	{ 0x96a4, 0x96a6, PDF_CMAP_RANGE, 5416 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 5419 },
	{ 0x96ae, 0x96af, PDF_CMAP_RANGE, 5420 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 5422 },
	{ 0x96ba, 0x96ba, PDF_CMAP_SINGLE, 5423 },
	{ 0x96ca, 0x96ca, PDF_CMAP_SINGLE, 5424 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 5425 },
	{ 0x96d8, 0x96d8, PDF_CMAP_SINGLE, 5427 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 5428 },
	{ 0x96dd, 0x96df, PDF_CMAP_RANGE, 5429 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 5432 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 5433 },
	{ 0x96f1, 0x96f1, PDF_CMAP_SINGLE, 5434 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 5435 },
	{ 0x9702, 0x9703, PDF_CMAP_RANGE, 5436 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 5438 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 5439 },
	{ 0x971a, 0x971b, PDF_CMAP_RANGE, 5440 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 5442 },
	{ 0x9721, 0x9723, PDF_CMAP_RANGE, 5443 },
	{ 0x9728, 0x9728, PDF_CMAP_SINGLE, 5446 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 5447 },
	{ 0x9733, 0x9733, PDF_CMAP_SINGLE, 5448 },
	{ 0x9741, 0x9741, PDF_CMAP_SINGLE, 5449 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 5450 },
	{ 0x974a, 0x974a, PDF_CMAP_SINGLE, 5451 },
	{ 0x974e, 0x974f, PDF_CMAP_RANGE, 5452 },
	{ 0x9755, 0x9755, PDF_CMAP_SINGLE, 5454 },
	{ 0x9757, 0x9758, PDF_CMAP_RANGE, 5455 },
	{ 0x975a, 0x975b, PDF_CMAP_RANGE, 5457 },
	{ 0x9763, 0x9763, PDF_CMAP_SINGLE, 5459 },
	{ 0x9767, 0x9767, PDF_CMAP_SINGLE, 5460 },
	{ 0x976a, 0x976a, PDF_CMAP_SINGLE, 5461 },
	{ 0x976e, 0x976e, PDF_CMAP_SINGLE, 5462 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 5463 },
	{ 0x9776, 0x9778, PDF_CMAP_RANGE, 5464 },
	{ 0x977b, 0x977b, PDF_CMAP_SINGLE, 5467 },
	{ 0x977d, 0x977d, PDF_CMAP_SINGLE, 5468 },
	{ 0x977f, 0x9780, PDF_CMAP_RANGE, 5469 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 5471 },
	{ 0x9795, 0x9797, PDF_CMAP_RANGE, 5472 },
	{ 0x9799, 0x979a, PDF_CMAP_RANGE, 5475 },
	{ 0x979e, 0x979f, PDF_CMAP_RANGE, 5477 },
	{ 0x97a2, 0x97a2, PDF_CMAP_SINGLE, 5479 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 5480 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 5481 },
	{ 0x97b1, 0x97b2, PDF_CMAP_RANGE, 5482 },
	{ 0x97b5, 0x97b6, PDF_CMAP_RANGE, 5484 },
	{ 0x97b8, 0x97ba, PDF_CMAP_RANGE, 5486 },
	{ 0x97bc, 0x97bc, PDF_CMAP_SINGLE, 5489 },
	{ 0x97be, 0x97bf, PDF_CMAP_RANGE, 5490 },
	{ 0x97c1, 0x97c1, PDF_CMAP_SINGLE, 5492 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 5493 },
	{ 0x97c7, 0x97c7, PDF_CMAP_SINGLE, 5495 },
	{ 0x97c9, 0x97ca, PDF_CMAP_RANGE, 5496 },
	{ 0x97cc, 0x97ce, PDF_CMAP_RANGE, 5498 },
	{ 0x97d0, 0x97d1, PDF_CMAP_RANGE, 5501 },
	{ 0x97d4, 0x97d4, PDF_CMAP_SINGLE, 5503 },
	{ 0x97d7, 0x97d9, PDF_CMAP_RANGE, 5504 },
	{ 0x97db, 0x97db, PDF_CMAP_SINGLE, 5510 },
	{ 0x97dd, 0x97de, PDF_CMAP_RANGE, 5507 },
	{ 0x97e0, 0x97e1, PDF_CMAP_TABLE, 280 },
	{ 0x97e4, 0x97e4, PDF_CMAP_SINGLE, 5512 },
	{ 0x97ef, 0x97ef, PDF_CMAP_SINGLE, 5513 },
	{ 0x97f1, 0x97f1, PDF_CMAP_SINGLE, 5514 },
	{ 0x97f4, 0x97f4, PDF_CMAP_SINGLE, 5515 },
	{ 0x97f7, 0x97f8, PDF_CMAP_RANGE, 5516 },
	{ 0x97fa, 0x97fa, PDF_CMAP_SINGLE, 5518 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 5519 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 5520 },
	{ 0x980d, 0x980e, PDF_CMAP_RANGE, 5522 },
	{ 0x9814, 0x9814, PDF_CMAP_SINGLE, 5524 },
	{ 0x9816, 0x9816, PDF_CMAP_SINGLE, 5525 },
	{ 0x9819, 0x9819, PDF_CMAP_SINGLE, 5521 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 5526 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 5527 },
	{ 0x9820, 0x9820, PDF_CMAP_SINGLE, 5528 },
	{ 0x9823, 0x9823, PDF_CMAP_SINGLE, 5529 },
	{ 0x9825, 0x9826, PDF_CMAP_TABLE, 282 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 5531 },
	{ 0x982e, 0x9830, PDF_CMAP_RANGE, 5532 },
	{ 0x9832, 0x9833, PDF_CMAP_RANGE, 5535 },
	{ 0x9835, 0x9835, PDF_CMAP_SINGLE, 5537 },
	{ 0x983e, 0x983e, PDF_CMAP_SINGLE, 5539 },
	{ 0x9844, 0x9844, PDF_CMAP_SINGLE, 5540 },
	{ 0x9847, 0x9847, PDF_CMAP_SINGLE, 5541 },
	{ 0x984a, 0x984a, PDF_CMAP_SINGLE, 5542 },
	{ 0x9851, 0x9853, PDF_CMAP_RANGE, 5543 },
	{ 0x9856, 0x9857, PDF_CMAP_RANGE, 5546 },
	{ 0x9859, 0x985a, PDF_CMAP_RANGE, 5548 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 5550 },
	{ 0x9865, 0x9866, PDF_CMAP_RANGE, 5552 },
	{ 0x986a, 0x986a, PDF_CMAP_SINGLE, 5554 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 5555 },
	{ 0x98ab, 0x98ab, PDF_CMAP_SINGLE, 5556 },
	{ 0x98ad, 0x98ae, PDF_CMAP_RANGE, 5557 },
	{ 0x98b0, 0x98b0, PDF_CMAP_SINGLE, 5559 },
	{ 0x98b4, 0x98b4, PDF_CMAP_SINGLE, 5560 },
	{ 0x98b7, 0x98b8, PDF_CMAP_RANGE, 5561 },
	{ 0x98ba, 0x98bb, PDF_CMAP_RANGE, 5563 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 5565 },
	{ 0x98c2, 0x98c2, PDF_CMAP_SINGLE, 5566 },
	{ 0x98c5, 0x98c5, PDF_CMAP_SINGLE, 5567 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 5568 },
	{ 0x98cc, 0x98cc, PDF_CMAP_SINGLE, 5569 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 5570 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 5571 },
	{ 0x98e5, 0x98e7, PDF_CMAP_RANGE, 5572 },
	{ 0x98ea, 0x98ea, PDF_CMAP_SINGLE, 5575 },
	{ 0x98f3, 0x98f3, PDF_CMAP_SINGLE, 5576 },
	{ 0x98f6, 0x98f6, PDF_CMAP_SINGLE, 5577 },
	{ 0x9902, 0x9902, PDF_CMAP_SINGLE, 5578 },
	{ 0x9907, 0x9908, PDF_CMAP_RANGE, 5579 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 5581 },
	{ 0x9915, 0x9917, PDF_CMAP_RANGE, 5582 },
	{ 0x991a, 0x991c, PDF_CMAP_RANGE, 5585 },
	{ 0x991f, 0x991f, PDF_CMAP_SINGLE, 5588 },
	{ 0x9922, 0x9922, PDF_CMAP_SINGLE, 5589 },
	{ 0x9926, 0x9927, PDF_CMAP_RANGE, 5590 },
	{ 0x992b, 0x992b, PDF_CMAP_SINGLE, 5592 },
	{ 0x9931, 0x9935, PDF_CMAP_RANGE, 5593 },
	{ 0x9939, 0x993c, PDF_CMAP_RANGE, 5598 },
	{ 0x9940, 0x9941, PDF_CMAP_RANGE, 5602 },
	{ 0x9946, 0x9948, PDF_CMAP_RANGE, 5604 },
	{ 0x994d, 0x994e, PDF_CMAP_RANGE, 5607 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 5609 },
	{ 0x9958, 0x9959, PDF_CMAP_RANGE, 5610 },
	{ 0x995b, 0x995c, PDF_CMAP_RANGE, 5612 },
	{ 0x995e, 0x9960, PDF_CMAP_RANGE, 5614 },
	{ 0x999b, 0x999b, PDF_CMAP_SINGLE, 5617 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 5618 },
	{ 0x999f, 0x999f, PDF_CMAP_SINGLE, 5619 },
	{ 0x99a6, 0x99a6, PDF_CMAP_SINGLE, 5620 },
	{ 0x99b0, 0x99b2, PDF_CMAP_RANGE, 5621 },
	{ 0x99b5, 0x99b5, PDF_CMAP_SINGLE, 5624 },
	{ 0x99b9, 0x99ba, PDF_CMAP_RANGE, 5625 },
	{ 0x99bd, 0x99bd, PDF_CMAP_SINGLE, 5627 },
	{ 0x99bf, 0x99bf, PDF_CMAP_SINGLE, 5628 },
	{ 0x99c3, 0x99c3, PDF_CMAP_SINGLE, 5629 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 5630 },
	{ 0x99d3, 0x99d4, PDF_CMAP_RANGE, 5631 },
	{ 0x99d9, 0x99da, PDF_CMAP_RANGE, 5633 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 5635 },
	{ 0x99de, 0x99de, PDF_CMAP_SINGLE, 5636 },
	{ 0x99e7, 0x99e7, PDF_CMAP_SINGLE, 5637 },
	{ 0x99ea, 0x99ec, PDF_CMAP_RANGE, 5638 },
	{ 0x99f0, 0x99f0, PDF_CMAP_SINGLE, 5641 },
	{ 0x99f4, 0x99f5, PDF_CMAP_RANGE, 5642 },
	{ 0x99f9, 0x99f9, PDF_CMAP_SINGLE, 5644 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 5645 },
	{ 0x9a02, 0x9a04, PDF_CMAP_RANGE, 5647 },
	{ 0x9a0b, 0x9a0c, PDF_CMAP_RANGE, 5650 },
	{ 0x9a10, 0x9a11, PDF_CMAP_RANGE, 5652 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 5654 },
	{ 0x9a1e, 0x9a1e, PDF_CMAP_SINGLE, 5655 },
	{ 0x9a20, 0x9a20, PDF_CMAP_SINGLE, 5656 },
	{ 0x9a22, 0x9a24, PDF_CMAP_RANGE, 5657 },
	{ 0x9a27, 0x9a27, PDF_CMAP_SINGLE, 5660 },
	{ 0x9a2d, 0x9a2e, PDF_CMAP_RANGE, 5661 },
	{ 0x9a33, 0x9a33, PDF_CMAP_SINGLE, 5663 },
	{ 0x9a35, 0x9a36, PDF_CMAP_RANGE, 5664 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 5666 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 5668 },
	{ 0x9a44, 0x9a44, PDF_CMAP_SINGLE, 5669 },
	{ 0x9a47, 0x9a47, PDF_CMAP_SINGLE, 5667 },
	{ 0x9a4a, 0x9a4c, PDF_CMAP_RANGE, 5670 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 5673 },
	{ 0x9a51, 0x9a51, PDF_CMAP_SINGLE, 5674 },
	{ 0x9a54, 0x9a54, PDF_CMAP_SINGLE, 5675 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 5676 },
	{ 0x9a5d, 0x9a5d, PDF_CMAP_SINGLE, 5677 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 5678 },
	{ 0x9aac, 0x9aac, PDF_CMAP_SINGLE, 5679 },
	{ 0x9aae, 0x9aaf, PDF_CMAP_RANGE, 5680 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 5682 },
	{ 0x9ab4, 0x9ab6, PDF_CMAP_RANGE, 5683 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 5686 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 5687 },
	{ 0x9abe, 0x9abf, PDF_CMAP_RANGE, 5688 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 5690 },
	{ 0x9ac3, 0x9ac3, PDF_CMAP_SINGLE, 5691 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 5692 },
	{ 0x9ac8, 0x9ac8, PDF_CMAP_SINGLE, 5693 },
	{ 0x9ace, 0x9ace, PDF_CMAP_SINGLE, 5694 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 5695 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 5696 },
	{ 0x9ad5, 0x9ad7, PDF_CMAP_RANGE, 5697 },
	{ 0x9adb, 0x9adc, PDF_CMAP_RANGE, 5700 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 5702 },
	{ 0x9ae4, 0x9ae5, PDF_CMAP_RANGE, 5703 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 5705 },
	{ 0x9ae9, 0x9ae9, PDF_CMAP_SINGLE, 5706 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 5707 },
	{ 0x9af2, 0x9af3, PDF_CMAP_RANGE, 5708 },
	{ 0x9af5, 0x9af5, PDF_CMAP_SINGLE, 5710 },
	{ 0x9af9, 0x9afa, PDF_CMAP_RANGE, 5711 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 5713 },
	{ 0x9aff, 0x9b05, PDF_CMAP_RANGE, 5714 },
	{ 0x9b08, 0x9b09, PDF_CMAP_RANGE, 5721 },
	{ 0x9b0b, 0x9b0e, PDF_CMAP_RANGE, 5723 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 5727 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 5728 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 5729 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 5730 },
	{ 0x9b1b, 0x9b1c, PDF_CMAP_RANGE, 5731 },
	{ 0x9b20, 0x9b20, PDF_CMAP_SINGLE, 5733 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 5734 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 5735 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 5736 },
	{ 0x9b33, 0x9b35, PDF_CMAP_RANGE, 5737 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 5740 },
	{ 0x9b39, 0x9b3a, PDF_CMAP_RANGE, 5741 },
	{ 0x9b3d, 0x9b3d, PDF_CMAP_SINGLE, 5743 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 5744 },
	{ 0x9b4b, 0x9b4c, PDF_CMAP_RANGE, 5745 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 5747 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 5750 },
	{ 0x9b5e, 0x9b5e, PDF_CMAP_SINGLE, 5751 },
	{ 0x9b61, 0x9b61, PDF_CMAP_SINGLE, 5752 },
	{ 0x9b63, 0x9b63, PDF_CMAP_SINGLE, 5753 },
	{ 0x9b65, 0x9b66, PDF_CMAP_RANGE, 5754 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 5756 },
	{ 0x9b6a, 0x9b6e, PDF_CMAP_RANGE, 5757 },
	{ 0x9b73, 0x9b73, PDF_CMAP_SINGLE, 5762 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 5763 },
	{ 0x9b77, 0x9b79, PDF_CMAP_RANGE, 5764 },
	{ 0x9b7f, 0x9b80, PDF_CMAP_RANGE, 5767 },
	{ 0x9b84, 0x9b87, PDF_CMAP_RANGE, 5769 },
	{ 0x9b89, 0x9b8b, PDF_CMAP_RANGE, 5773 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 5776 },
	{ 0x9b8f, 0x9b90, PDF_CMAP_RANGE, 5777 },
	{ 0x9b94, 0x9b94, PDF_CMAP_SINGLE, 5779 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 5780 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_RANGE, 5781 },
	{ 0x9ba6, 0x9ba7, PDF_CMAP_RANGE, 5783 },
	{ 0x9ba9, 0x9ba9, PDF_CMAP_SINGLE, 5785 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 5786 },
	{ 0x9bb0, 0x9bb2, PDF_CMAP_RANGE, 5787 },
	{ 0x9bb7, 0x9bb8, PDF_CMAP_RANGE, 5790 },
	{ 0x9bbb, 0x9bbc, PDF_CMAP_RANGE, 5792 },
	{ 0x9bbe, 0x9bbf, PDF_CMAP_RANGE, 5794 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 5796 },
	{ 0x9bc7, 0x9bc8, PDF_CMAP_RANGE, 5797 },
	{ 0x9bce, 0x9bce, PDF_CMAP_SINGLE, 5799 },
	{ 0x9bd0, 0x9bd0, PDF_CMAP_SINGLE, 5800 },
	{ 0x9bd7, 0x9bd8, PDF_CMAP_RANGE, 5801 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 5803 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 5804 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 5805 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 5806 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 5807 },
	{ 0x9bef, 0x9bef, PDF_CMAP_SINGLE, 5809 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 5810 },
	{ 0x9bf7, 0x9bfa, PDF_CMAP_RANGE, 5811 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5815 },
	{ 0x9bff, 0x9c00, PDF_CMAP_RANGE, 5816 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 5818 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 5819 },
	{ 0x9c0f, 0x9c0f, PDF_CMAP_SINGLE, 5820 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 5821 },
	{ 0x9c16, 0x9c16, PDF_CMAP_SINGLE, 5822 },
	{ 0x9c18, 0x9c1a, PDF_CMAP_RANGE, 5823 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 5826 },
	{ 0x9c1e, 0x9c1e, PDF_CMAP_SINGLE, 5827 },
	{ 0x9c22, 0x9c23, PDF_CMAP_RANGE, 5828 },
	{ 0x9c26, 0x9c2a, PDF_CMAP_RANGE, 5830 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 5835 },
	{ 0x9c35, 0x9c37, PDF_CMAP_RANGE, 5836 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 5839 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 5840 },
	{ 0x9c43, 0x9c45, PDF_CMAP_RANGE, 5841 },
	{ 0x9c49, 0x9c4a, PDF_CMAP_RANGE, 5844 },
	{ 0x9c4e, 0x9c50, PDF_CMAP_RANGE, 5846 },
	{ 0x9c53, 0x9c54, PDF_CMAP_RANGE, 5849 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 5851 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 5852 },
	{ 0x9c5b, 0x9c5c, PDF_CMAP_TABLE, 284 },
	{ 0x9c5d, 0x9c5f, PDF_CMAP_RANGE, 5854 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 5857 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 5862 },
	{ 0x9c69, 0x9c6a, PDF_CMAP_RANGE, 5858 },
	{ 0x9c6b, 0x9c6b, PDF_CMAP_SINGLE, 5861 },
	{ 0x9c6e, 0x9c6e, PDF_CMAP_SINGLE, 5863 },
	{ 0x9c70, 0x9c70, PDF_CMAP_SINGLE, 5864 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 5865 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 5866 },
	{ 0x9c77, 0x9c77, PDF_CMAP_SINGLE, 5867 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 5868 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 5869 },
	{ 0x9cf2, 0x9cf2, PDF_CMAP_SINGLE, 5870 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 5871 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 5872 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 5874 },
	{ 0x9d0b, 0x9d0b, PDF_CMAP_SINGLE, 5873 },
	{ 0x9d11, 0x9d11, PDF_CMAP_SINGLE, 5875 },
	{ 0x9d17, 0x9d18, PDF_CMAP_RANGE, 5876 },
	{ 0x9d1c, 0x9d1e, PDF_CMAP_RANGE, 5878 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_RANGE, 5881 },
	{ 0x9d32, 0x9d34, PDF_CMAP_RANGE, 5883 },
	{ 0x9d3a, 0x9d3a, PDF_CMAP_SINGLE, 5886 },
	{ 0x9d3c, 0x9d3d, PDF_CMAP_TABLE, 286 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 5890 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 5888 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 5892 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 5893 },
	{ 0x9d53, 0x9d54, PDF_CMAP_RANGE, 5894 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 5896 },
	{ 0x9d62, 0x9d63, PDF_CMAP_TABLE, 288 },
	{ 0x9d65, 0x9d65, PDF_CMAP_SINGLE, 5899 },
	{ 0x9d69, 0x9d6b, PDF_CMAP_RANGE, 5900 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 5903 },
	{ 0x9d76, 0x9d77, PDF_CMAP_RANGE, 5904 },
	{ 0x9d7b, 0x9d7c, PDF_CMAP_RANGE, 5906 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 5908 },
	{ 0x9d83, 0x9d84, PDF_CMAP_RANGE, 5909 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 5911 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 5912 },
	{ 0x9d8d, 0x9d8e, PDF_CMAP_RANGE, 5913 },
	{ 0x9d92, 0x9d93, PDF_CMAP_RANGE, 5915 },
	{ 0x9d95, 0x9d98, PDF_CMAP_RANGE, 5917 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 5921 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 5922 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 5923 },
	{ 0x9dae, 0x9dae, PDF_CMAP_SINGLE, 5924 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 5925 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 5926 },
	{ 0x9db9, 0x9db9, PDF_CMAP_SINGLE, 5927 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 5928 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 5929 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 5930 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 5931 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_RANGE, 5932 },
	{ 0x9dd4, 0x9dd7, PDF_CMAP_RANGE, 5934 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 5938 },
	{ 0x9dde, 0x9de0, PDF_CMAP_RANGE, 5939 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 5942 },
	{ 0x9de7, 0x9de7, PDF_CMAP_SINGLE, 5943 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 5944 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 5945 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 5946 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 5947 },
	{ 0x9df3, 0x9df4, PDF_CMAP_RANGE, 5948 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 5950 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 5952 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 5953 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 5951 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 5954 },
	{ 0x9e10, 0x9e12, PDF_CMAP_RANGE, 5955 },
	{ 0x9e15, 0x9e16, PDF_CMAP_RANGE, 5958 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 5960 },
	{ 0x9e1c, 0x9e1d, PDF_CMAP_RANGE, 5961 },
	{ 0x9e7a, 0x9e7c, PDF_CMAP_RANGE, 5963 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 5966 },
	{ 0x9e82, 0x9e85, PDF_CMAP_RANGE, 5967 },
	{ 0x9e87, 0x9e87, PDF_CMAP_SINGLE, 5971 },
	{ 0x9e8e, 0x9e8f, PDF_CMAP_RANGE, 5972 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 5974 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 5975 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 5976 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 5977 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 5978 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 5979 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 5980 },
	{ 0x9eae, 0x9eb0, PDF_CMAP_RANGE, 5981 },
	{ 0x9eb3, 0x9eb5, PDF_CMAP_RANGE, 5984 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 5987 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 5988 },
	{ 0x9ecb, 0x9ecb, PDF_CMAP_SINGLE, 5989 },
	{ 0x9ed5, 0x9ed5, PDF_CMAP_SINGLE, 5990 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 5991 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 5992 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 5993 },
	{ 0x9eec, 0x9eee, PDF_CMAP_RANGE, 5994 },
	{ 0x9ef0, 0x9ef2, PDF_CMAP_RANGE, 5997 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 6000 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 6001 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 6002 },
	{ 0x9f02, 0x9f03, PDF_CMAP_RANGE, 6003 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 6005 },
	{ 0x9f0f, 0x9f12, PDF_CMAP_RANGE, 6006 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 6010 },
	{ 0x9f16, 0x9f17, PDF_CMAP_RANGE, 6011 },
	{ 0x9f19, 0x9f1b, PDF_CMAP_RANGE, 6013 },
	{ 0x9f1f, 0x9f1f, PDF_CMAP_SINGLE, 6016 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 6017 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 6018 },
	{ 0x9f2a, 0x9f2b, PDF_CMAP_RANGE, 6019 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 6021 },
	{ 0x9f31, 0x9f32, PDF_CMAP_RANGE, 6022 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 6024 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 6025 },
	{ 0x9f39, 0x9f3a, PDF_CMAP_RANGE, 6026 },
	{ 0x9f3c, 0x9f3d, PDF_CMAP_RANGE, 6028 },
	{ 0x9f3f, 0x9f3f, PDF_CMAP_SINGLE, 6030 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 6031 },
	{ 0x9f43, 0x9f47, PDF_CMAP_RANGE, 6032 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 6037 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 6038 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 6042 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 6043 },
	{ 0x9f68, 0x9f69, PDF_CMAP_RANGE, 6045 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 6047 },
	{ 0x9f73, 0x9f73, PDF_CMAP_SINGLE, 6052 },
	{ 0x9f75, 0x9f75, PDF_CMAP_SINGLE, 6053 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 6054 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 6055 },
	{ 0x9f8f, 0x9f92, PDF_CMAP_RANGE, 6056 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 6060 },
	{ 0x9f96, 0x9f97, PDF_CMAP_RANGE, 6061 },
	{ 0x9f9e, 0x9f9e, PDF_CMAP_SINGLE, 6063 },
	{ 0x9fa1, 0x9fa3, PDF_CMAP_RANGE, 6064 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 6067 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 5673 },
	{ 0x9a51, 0x9a51, PDF_CMAP_SINGLE, 5674 },
	{ 0x9a54, 0x9a54, PDF_CMAP_SINGLE, 5675 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 5676 },
	{ 0x9a5d, 0x9a5d, PDF_CMAP_SINGLE, 5677 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 5678 },
	{ 0x9aac, 0x9aac, PDF_CMAP_SINGLE, 5679 },
	{ 0x9aae, 0x9aaf, PDF_CMAP_RANGE, 5680 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 5682 },
	{ 0x9ab4, 0x9ab6, PDF_CMAP_RANGE, 5683 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 5686 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 5687 },
	{ 0x9abe, 0x9abf, PDF_CMAP_RANGE, 5688 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 5690 },
	{ 0x9ac3, 0x9ac3, PDF_CMAP_SINGLE, 5691 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 5692 },
	{ 0x9ac8, 0x9ac8, PDF_CMAP_SINGLE, 5693 },
	{ 0x9ace, 0x9ace, PDF_CMAP_SINGLE, 5694 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 5695 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 5696 },
	{ 0x9ad5, 0x9ad7, PDF_CMAP_RANGE, 5697 },
	{ 0x9adb, 0x9adc, PDF_CMAP_RANGE, 5700 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 5702 },
	{ 0x9ae4, 0x9ae5, PDF_CMAP_RANGE, 5703 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 5705 },
	{ 0x9ae9, 0x9ae9, PDF_CMAP_SINGLE, 5706 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 5707 },
	{ 0x9af2, 0x9af3, PDF_CMAP_RANGE, 5708 },
	{ 0x9af5, 0x9af5, PDF_CMAP_SINGLE, 5710 },
	{ 0x9af9, 0x9afa, PDF_CMAP_RANGE, 5711 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 5713 },
	{ 0x9aff, 0x9aff, PDF_CMAP_SINGLE, 5714 },
	{ 0x9b00, 0x9b05, PDF_CMAP_RANGE, 5715 },
	{ 0x9b08, 0x9b09, PDF_CMAP_RANGE, 5721 },
	{ 0x9b0b, 0x9b0e, PDF_CMAP_RANGE, 5723 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 5727 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 5728 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 5729 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 5730 },
	{ 0x9b1b, 0x9b1c, PDF_CMAP_RANGE, 5731 },
	{ 0x9b20, 0x9b20, PDF_CMAP_SINGLE, 5733 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 5734 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 5735 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 5736 },
	{ 0x9b33, 0x9b35, PDF_CMAP_RANGE, 5737 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 5740 },
	{ 0x9b39, 0x9b3a, PDF_CMAP_RANGE, 5741 },
	{ 0x9b3d, 0x9b3d, PDF_CMAP_SINGLE, 5743 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 5744 },
	{ 0x9b4b, 0x9b4c, PDF_CMAP_RANGE, 5745 },
	{ 0x9b55, 0x9b57, PDF_CMAP_RANGE, 5747 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 5750 },
	{ 0x9b5e, 0x9b5e, PDF_CMAP_SINGLE, 5751 },
	{ 0x9b61, 0x9b61, PDF_CMAP_SINGLE, 5752 },
	{ 0x9b63, 0x9b63, PDF_CMAP_SINGLE, 5753 },
	{ 0x9b65, 0x9b66, PDF_CMAP_RANGE, 5754 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 5756 },
	{ 0x9b6a, 0x9b6e, PDF_CMAP_RANGE, 5757 },
	{ 0x9b73, 0x9b73, PDF_CMAP_SINGLE, 5762 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 5763 },
	{ 0x9b77, 0x9b79, PDF_CMAP_RANGE, 5764 },
	{ 0x9b7f, 0x9b80, PDF_CMAP_RANGE, 5767 },
	{ 0x9b84, 0x9b87, PDF_CMAP_RANGE, 5769 },
	{ 0x9b89, 0x9b8b, PDF_CMAP_RANGE, 5773 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 5776 },
	{ 0x9b8f, 0x9b90, PDF_CMAP_RANGE, 5777 },
	{ 0x9b94, 0x9b94, PDF_CMAP_SINGLE, 5779 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 5780 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_RANGE, 5781 },
	{ 0x9ba6, 0x9ba7, PDF_CMAP_RANGE, 5783 },
	{ 0x9ba9, 0x9ba9, PDF_CMAP_SINGLE, 5785 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 5786 },
	{ 0x9bb0, 0x9bb2, PDF_CMAP_RANGE, 5787 },
	{ 0x9bb7, 0x9bb8, PDF_CMAP_RANGE, 5790 },
	{ 0x9bbb, 0x9bbc, PDF_CMAP_RANGE, 5792 },
	{ 0x9bbe, 0x9bbf, PDF_CMAP_RANGE, 5794 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 5796 },
	{ 0x9bc7, 0x9bc8, PDF_CMAP_RANGE, 5797 },
	{ 0x9bce, 0x9bce, PDF_CMAP_SINGLE, 5799 },
	{ 0x9bd0, 0x9bd0, PDF_CMAP_SINGLE, 5800 },
	{ 0x9bd7, 0x9bd8, PDF_CMAP_RANGE, 5801 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 5803 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 5804 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 5805 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 5806 },
	{ 0x9bea, 0x9beb, PDF_CMAP_RANGE, 5807 },
	{ 0x9bef, 0x9bef, PDF_CMAP_SINGLE, 5809 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 5810 },
	{ 0x9bf7, 0x9bfa, PDF_CMAP_RANGE, 5811 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 5815 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 5816 },
	{ 0x9c00, 0x9c00, PDF_CMAP_SINGLE, 5817 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 5818 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 5819 },
	{ 0x9c0f, 0x9c0f, PDF_CMAP_SINGLE, 5820 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 5821 },
	{ 0x9c16, 0x9c16, PDF_CMAP_SINGLE, 5822 },
	{ 0x9c18, 0x9c1a, PDF_CMAP_RANGE, 5823 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 5826 },
	{ 0x9c1e, 0x9c1e, PDF_CMAP_SINGLE, 5827 },
	{ 0x9c22, 0x9c23, PDF_CMAP_RANGE, 5828 },
	{ 0x9c26, 0x9c2a, PDF_CMAP_RANGE, 5830 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 5835 },
	{ 0x9c35, 0x9c37, PDF_CMAP_RANGE, 5836 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 5839 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 5840 },
	{ 0x9c43, 0x9c45, PDF_CMAP_RANGE, 5841 },
	{ 0x9c49, 0x9c4a, PDF_CMAP_RANGE, 5844 },
	{ 0x9c4e, 0x9c50, PDF_CMAP_RANGE, 5846 },
	{ 0x9c53, 0x9c54, PDF_CMAP_RANGE, 5849 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 5851 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 5852 },
	{ 0x9c5b, 0x9c5b, PDF_CMAP_SINGLE, 5853 },
	{ 0x9c5c, 0x9c5c, PDF_CMAP_SINGLE, 5860 },
	{ 0x9c5d, 0x9c5f, PDF_CMAP_RANGE, 5854 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 5857 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 5862 },
	{ 0x9c69, 0x9c6a, PDF_CMAP_RANGE, 5858 },
	{ 0x9c6b, 0x9c6b, PDF_CMAP_SINGLE, 5861 },
	{ 0x9c6e, 0x9c6e, PDF_CMAP_SINGLE, 5863 },
	{ 0x9c70, 0x9c70, PDF_CMAP_SINGLE, 5864 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 5865 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 5866 },
	{ 0x9c77, 0x9c77, PDF_CMAP_SINGLE, 5867 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 5868 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 5869 },
	{ 0x9cf2, 0x9cf2, PDF_CMAP_SINGLE, 5870 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 5871 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 5872 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 5874 },
	{ 0x9d0b, 0x9d0b, PDF_CMAP_SINGLE, 5873 },
	{ 0x9d11, 0x9d11, PDF_CMAP_SINGLE, 5875 },
	{ 0x9d17, 0x9d18, PDF_CMAP_RANGE, 5876 },
	{ 0x9d1c, 0x9d1e, PDF_CMAP_RANGE, 5878 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_RANGE, 5881 },
	{ 0x9d32, 0x9d34, PDF_CMAP_RANGE, 5883 },
	{ 0x9d3a, 0x9d3a, PDF_CMAP_SINGLE, 5886 },
	{ 0x9d3c, 0x9d3c, PDF_CMAP_SINGLE, 5887 },
	{ 0x9d3d, 0x9d3d, PDF_CMAP_SINGLE, 5889 },
	{ 0x9d42, 0x9d43, PDF_CMAP_RANGE, 5890 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 5888 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 5892 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 5893 },
	{ 0x9d53, 0x9d54, PDF_CMAP_RANGE, 5894 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 5896 },
	{ 0x9d62, 0x9d62, PDF_CMAP_SINGLE, 5898 },
	{ 0x9d63, 0x9d63, PDF_CMAP_SINGLE, 5897 },
	{ 0x9d65, 0x9d65, PDF_CMAP_SINGLE, 5899 },
	{ 0x9d69, 0x9d6b, PDF_CMAP_RANGE, 5900 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 5903 },
	{ 0x9d76, 0x9d77, PDF_CMAP_RANGE, 5904 },
	{ 0x9d7b, 0x9d7c, PDF_CMAP_RANGE, 5906 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 5908 },
	{ 0x9d83, 0x9d84, PDF_CMAP_RANGE, 5909 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 5911 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 5912 },
	{ 0x9d8d, 0x9d8e, PDF_CMAP_RANGE, 5913 },
	{ 0x9d92, 0x9d93, PDF_CMAP_RANGE, 5915 },
	{ 0x9d95, 0x9d98, PDF_CMAP_RANGE, 5917 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 5921 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 5922 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 5923 },
	{ 0x9dae, 0x9dae, PDF_CMAP_SINGLE, 5924 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 5925 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 5926 },
	{ 0x9db9, 0x9db9, PDF_CMAP_SINGLE, 5927 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 5928 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 5929 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 5930 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 5931 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_RANGE, 5932 },
	{ 0x9dd4, 0x9dd7, PDF_CMAP_RANGE, 5934 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 5938 },
	{ 0x9dde, 0x9de0, PDF_CMAP_RANGE, 5939 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 5942 },
	{ 0x9de7, 0x9de7, PDF_CMAP_SINGLE, 5943 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 5944 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 5945 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 5946 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 5947 },
	{ 0x9df3, 0x9df4, PDF_CMAP_RANGE, 5948 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 5950 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 5952 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 5953 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 5951 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 5954 },
	{ 0x9e10, 0x9e12, PDF_CMAP_RANGE, 5955 },
	{ 0x9e15, 0x9e16, PDF_CMAP_RANGE, 5958 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 5960 },
	{ 0x9e1c, 0x9e1d, PDF_CMAP_RANGE, 5961 },
	{ 0x9e7a, 0x9e7c, PDF_CMAP_RANGE, 5963 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 5966 },
	{ 0x9e82, 0x9e85, PDF_CMAP_RANGE, 5967 },
	{ 0x9e87, 0x9e87, PDF_CMAP_SINGLE, 5971 },
	{ 0x9e8e, 0x9e8f, PDF_CMAP_RANGE, 5972 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 5974 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 5975 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 5976 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 5977 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 5978 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 5979 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 5980 },
	{ 0x9eae, 0x9eb0, PDF_CMAP_RANGE, 5981 },
	{ 0x9eb3, 0x9eb5, PDF_CMAP_RANGE, 5984 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 5987 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 5988 },
	{ 0x9ecb, 0x9ecb, PDF_CMAP_SINGLE, 5989 },
	{ 0x9ed5, 0x9ed5, PDF_CMAP_SINGLE, 5990 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 5991 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 5992 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 5993 },
	{ 0x9eec, 0x9eee, PDF_CMAP_RANGE, 5994 },
	{ 0x9ef0, 0x9ef2, PDF_CMAP_RANGE, 5997 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 6000 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 6001 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 6002 },
	{ 0x9f02, 0x9f03, PDF_CMAP_RANGE, 6003 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 6005 },
	{ 0x9f0f, 0x9f12, PDF_CMAP_RANGE, 6006 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 6010 },
	{ 0x9f16, 0x9f17, PDF_CMAP_RANGE, 6011 },
	{ 0x9f19, 0x9f1b, PDF_CMAP_RANGE, 6013 },
	{ 0x9f1f, 0x9f1f, PDF_CMAP_SINGLE, 6016 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 6017 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 6018 },
	{ 0x9f2a, 0x9f2b, PDF_CMAP_RANGE, 6019 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 6021 },
	{ 0x9f31, 0x9f32, PDF_CMAP_RANGE, 6022 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 6024 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 6025 },
	{ 0x9f39, 0x9f3a, PDF_CMAP_RANGE, 6026 },
	{ 0x9f3c, 0x9f3d, PDF_CMAP_RANGE, 6028 },
	{ 0x9f3f, 0x9f3f, PDF_CMAP_SINGLE, 6030 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 6031 },
	{ 0x9f43, 0x9f47, PDF_CMAP_RANGE, 6032 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 6037 },
	{ 0x9f55, 0x9f58, PDF_CMAP_RANGE, 6038 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 6042 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_RANGE, 6043 },
	{ 0x9f68, 0x9f69, PDF_CMAP_RANGE, 6045 },
	{ 0x9f6d, 0x9f71, PDF_CMAP_RANGE, 6047 },
	{ 0x9f73, 0x9f73, PDF_CMAP_SINGLE, 6052 },
	{ 0x9f75, 0x9f75, PDF_CMAP_SINGLE, 6053 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 6054 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 6055 },
	{ 0x9f8f, 0x9f92, PDF_CMAP_RANGE, 6056 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 6060 },
	{ 0x9f96, 0x9f97, PDF_CMAP_RANGE, 6061 },
	{ 0x9f9e, 0x9f9e, PDF_CMAP_SINGLE, 6063 },
	{ 0x9fa1, 0x9fa3, PDF_CMAP_RANGE, 6064 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 6067 },
};

static const unsigned short pdf_cmap_UniHojo_UTF16_H_table[290] =
{
	17,16,18,6,14,97,96,99,
	105,98,104,69,109,113,112,115,
	114,126,125,128,127,142,144,143,
	146,150,145,76,161,160,163,162,
	176,79,93,183,182,185,191,184,
	190,80,195,199,198,201,200,212,
	211,214,213,82,227,229,228,231,
	235,230,91,246,245,248,247,261,
	95,262,102,188,100,186,103,189,
	106,192,107,193,110,196,108,194,
	111,197,70,81,118,204,117,203,
	119,205,116,202,120,207,121,208,
	123,209,122,124,210,71,83,133,
	218,131,216,132,217,130,84,72,
	85,134,219,135,220,86,136,221,
	138,223,137,222,74,88,73,87,
	139,224,141,226,140,225,89,75,
	90,149,234,148,233,77,92,151,
	236,153,238,152,237,154,239,155,
	240,157,242,156,241,159,244,158,
	243,78,94,170,255,167,252,164,
	249,169,254,166,251,168,253,175,
	260,178,263,177,179,264,181,266,
	180,265,101,187,129,215,147,232,
	165,250,174,259,171,256,173,258,
	172,257,1,4,8,7,28,30,
	36,26,29,35,40,37,39,42,
	1481,1512,2385,2387,2652,3735,3734,346,
	329,491,503,666,665,979,988,1413,
	1415,1444,1443,1505,1507,2000,2002,1993,
	1996,2001,2035,1989,2099,2098,2285,2284,
	3095,3089,3096,3301,3296,3726,3725,3894,
	3891,4056,3999,4045,4070,4236,1088,4237,
	4391,4387,4621,4616,4702,4701,4745,4763,
	4846,4840,4963,4965,5269,5275,5340,5339,
	5509,5511,5538,5530,5853,5860,5887,5889,
	5898,5897,
};

pdf_cmap pdf_cmap_UniHojo_UTF16_H =
{
	-1, "UniHojo-UTF16-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 4, 0xdc00, 0xdfff },
		{ 2, 0xe000, 0xffff },
	},
	4255, 4255, (pdf_range*) pdf_cmap_UniHojo_UTF16_H_ranges,
	290, 290, (unsigned short*) pdf_cmap_UniHojo_UTF16_H_table,
};

/* UniHojo-UTF16-V */

static const pdf_range pdf_cmap_UniHojo_UTF16_V_ranges[] =
{
	/* dummy entry for non-c99 compilers */
	{ 0x0, 0x0, PDF_CMAP_RANGE, 0 }
};

static const unsigned short pdf_cmap_UniHojo_UTF16_V_table[1] = { 0 };

pdf_cmap pdf_cmap_UniHojo_UTF16_V =
{
	-1, "UniHojo-UTF16-V", "UniHojo-UTF16-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	0, 0, (pdf_range*) pdf_cmap_UniHojo_UTF16_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_UniHojo_UTF16_V_table,
};

/* UniJIS-UTF16-H */

static const pdf_range pdf_cmap_UniJIS_UTF16_H_ranges[] =
{
	{ 0x0020, 0x005b, PDF_CMAP_RANGE, 1 },
	{ 0x005c, 0x005c, PDF_CMAP_SINGLE, 97 },
	{ 0x005d, 0x007b, PDF_CMAP_RANGE, 62 },
	{ 0x007c, 0x007e, PDF_CMAP_TABLE, 0 },
	{ 0x00a0, 0x00a0, PDF_CMAP_SINGLE, 1 },
	{ 0x00a1, 0x00a3, PDF_CMAP_RANGE, 101 },
	{ 0x00a4, 0x00b1, PDF_CMAP_TABLE, 3 },
	{ 0x00b2, 0x00b3, PDF_CMAP_RANGE, 157 },
	{ 0x00b4, 0x00bb, PDF_CMAP_TABLE, 17 },
	{ 0x00bc, 0x00be, PDF_CMAP_RANGE, 161 },
	{ 0x00bf, 0x00bf, PDF_CMAP_SINGLE, 126 },
	{ 0x00c0, 0x00c5, PDF_CMAP_RANGE, 164 },
	{ 0x00c6, 0x00c6, PDF_CMAP_SINGLE, 139 },
	{ 0x00c7, 0x00d6, PDF_CMAP_RANGE, 170 },
	{ 0x00d7, 0x00d8, PDF_CMAP_TABLE, 25 },
	{ 0x00d9, 0x00de, PDF_CMAP_RANGE, 187 },
	{ 0x00df, 0x00df, PDF_CMAP_SINGLE, 150 },
	{ 0x00e0, 0x00e5, PDF_CMAP_RANGE, 193 },
	{ 0x00e6, 0x00e6, PDF_CMAP_SINGLE, 145 },
	{ 0x00e7, 0x00f6, PDF_CMAP_RANGE, 199 },
	{ 0x00f7, 0x00f8, PDF_CMAP_TABLE, 27 },
	{ 0x00f9, 0x00ff, PDF_CMAP_RANGE, 216 },
	{ 0x0100, 0x0113, PDF_CMAP_TABLE, 29 },
	{ 0x0116, 0x0122, PDF_CMAP_TABLE, 49 },
	{ 0x0124, 0x012b, PDF_CMAP_TABLE, 62 },
	{ 0x012e, 0x014d, PDF_CMAP_TABLE, 70 },
	{ 0x0150, 0x017e, PDF_CMAP_TABLE, 102 },
	{ 0x0192, 0x0193, PDF_CMAP_TABLE, 149 },
	{ 0x01c2, 0x01c2, PDF_CMAP_SINGLE, 15821 },
	{ 0x01cd, 0x01dc, PDF_CMAP_TABLE, 151 },
	{ 0x01f5, 0x01f5, PDF_CMAP_SINGLE, 20354 },
	{ 0x01f8, 0x01f9, PDF_CMAP_RANGE, 15731 },
	{ 0x01fd, 0x01fd, PDF_CMAP_SINGLE, 9421 },
	{ 0x0237, 0x0237, PDF_CMAP_SINGLE, 9435 },
	{ 0x0250, 0x025c, PDF_CMAP_TABLE, 167 },
	{ 0x025e, 0x0261, PDF_CMAP_TABLE, 180 },
	{ 0x0262, 0x0263, PDF_CMAP_RANGE, 15883 },
	{ 0x0264, 0x0268, PDF_CMAP_TABLE, 184 },
	{ 0x026a, 0x026a, PDF_CMAP_SINGLE, 15885 },
	{ 0x026c, 0x0276, PDF_CMAP_TABLE, 189 },
	{ 0x0278, 0x027b, PDF_CMAP_TABLE, 200 },
	{ 0x027d, 0x027e, PDF_CMAP_TABLE, 204 },
	{ 0x0280, 0x0284, PDF_CMAP_TABLE, 206 },
	{ 0x0288, 0x0292, PDF_CMAP_TABLE, 211 },
	{ 0x0294, 0x0295, PDF_CMAP_TABLE, 222 },
	{ 0x0298, 0x0299, PDF_CMAP_TABLE, 224 },
	{ 0x029c, 0x029d, PDF_CMAP_TABLE, 226 },
	{ 0x029f, 0x029f, PDF_CMAP_SINGLE, 15893 },
	{ 0x02a1, 0x02a2, PDF_CMAP_TABLE, 228 },
	{ 0x02b0, 0x02b0, PDF_CMAP_SINGLE, 15894 },
	{ 0x02b2, 0x02b2, PDF_CMAP_SINGLE, 15895 },
	{ 0x02b7, 0x02b7, PDF_CMAP_SINGLE, 15896 },
	{ 0x02bb, 0x02bc, PDF_CMAP_TABLE, 230 },
	{ 0x02c1, 0x02c1, PDF_CMAP_SINGLE, 15897 },
	{ 0x02c6, 0x02c8, PDF_CMAP_TABLE, 232 },
	{ 0x02cc, 0x02cc, PDF_CMAP_SINGLE, 15847 },
	{ 0x02d0, 0x02d1, PDF_CMAP_TABLE, 235 },
	{ 0x02d8, 0x02de, PDF_CMAP_TABLE, 237 },
	{ 0x02e0, 0x02e1, PDF_CMAP_RANGE, 15898 },
	{ 0x02e5, 0x02e9, PDF_CMAP_RANGE, 15851 },
	{ 0x0300, 0x0300, PDF_CMAP_SINGLE, 65 },
	{ 0x0301, 0x0302, PDF_CMAP_RANGE, 127 },
	{ 0x0303, 0x0305, PDF_CMAP_TABLE, 244 },
	{ 0x0306, 0x0308, PDF_CMAP_RANGE, 130 },
	{ 0x030a, 0x030c, PDF_CMAP_TABLE, 247 },
	{ 0x030f, 0x030f, PDF_CMAP_SINGLE, 15850 },
	{ 0x0318, 0x0319, PDF_CMAP_RANGE, 15874 },
	{ 0x031a, 0x031a, PDF_CMAP_SINGLE, 15879 },
	{ 0x031c, 0x031c, PDF_CMAP_SINGLE, 15861 },
	{ 0x031d, 0x031e, PDF_CMAP_RANGE, 15872 },
	{ 0x031f, 0x0320, PDF_CMAP_RANGE, 15862 },
	{ 0x0324, 0x0325, PDF_CMAP_TABLE, 250 },
	{ 0x0327, 0x032a, PDF_CMAP_TABLE, 252 },
	{ 0x032c, 0x032c, PDF_CMAP_SINGLE, 15859 },
	{ 0x032f, 0x0330, PDF_CMAP_TABLE, 256 },
	{ 0x0332, 0x0332, PDF_CMAP_SINGLE, 64 },
	{ 0x0334, 0x0334, PDF_CMAP_SINGLE, 15871 },
	{ 0x0336, 0x0336, PDF_CMAP_SINGLE, 138 },
	{ 0x0339, 0x0339, PDF_CMAP_SINGLE, 15860 },
	{ 0x033a, 0x033b, PDF_CMAP_RANGE, 15877 },
	{ 0x033c, 0x033d, PDF_CMAP_TABLE, 258 },
	{ 0x0361, 0x0361, PDF_CMAP_SINGLE, 15845 },
	{ 0x0384, 0x0385, PDF_CMAP_RANGE, 20317 },
	{ 0x0386, 0x0386, PDF_CMAP_SINGLE, 20427 },
	{ 0x0388, 0x038a, PDF_CMAP_RANGE, 20428 },
	{ 0x038c, 0x038c, PDF_CMAP_SINGLE, 20432 },
	{ 0x038e, 0x0390, PDF_CMAP_TABLE, 260 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 1011 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 1028 },
	{ 0x03aa, 0x03ab, PDF_CMAP_TABLE, 263 },
	{ 0x03ac, 0x03af, PDF_CMAP_RANGE, 20436 },
	{ 0x03b0, 0x03b0, PDF_CMAP_SINGLE, 20445 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 1035 },
	{ 0x03c2, 0x03c2, PDF_CMAP_SINGLE, 16222 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 1052 },
	{ 0x03ca, 0x03cb, PDF_CMAP_TABLE, 265 },
	{ 0x03cc, 0x03cd, PDF_CMAP_RANGE, 20442 },
	{ 0x03ce, 0x03ce, PDF_CMAP_SINGLE, 20446 },
	{ 0x03d0, 0x03d1, PDF_CMAP_TABLE, 267 },
	{ 0x03d5, 0x03d5, PDF_CMAP_SINGLE, 12094 },
	{ 0x03db, 0x03db, PDF_CMAP_SINGLE, 12095 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 1065 },
	{ 0x0402, 0x040c, PDF_CMAP_RANGE, 20447 },
	{ 0x040e, 0x040f, PDF_CMAP_RANGE, 20458 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 1059 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 1066 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 1099 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 1098 },
	{ 0x0452, 0x045c, PDF_CMAP_RANGE, 20460 },
	{ 0x045e, 0x045f, PDF_CMAP_RANGE, 20471 },
	{ 0x1e3e, 0x1e3f, PDF_CMAP_RANGE, 15729 },
	{ 0x1ebc, 0x1ebd, PDF_CMAP_TABLE, 269 },
	{ 0x1f70, 0x1f73, PDF_CMAP_TABLE, 271 },
	{ 0x2002, 0x2003, PDF_CMAP_TABLE, 275 },
	{ 0x2010, 0x2016, PDF_CMAP_TABLE, 277 },
	{ 0x2018, 0x201a, PDF_CMAP_TABLE, 284 },
	{ 0x201c, 0x201e, PDF_CMAP_TABLE, 287 },
	{ 0x2020, 0x2021, PDF_CMAP_RANGE, 776 },
	{ 0x2022, 0x2022, PDF_CMAP_SINGLE, 119 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 290 },
	{ 0x2030, 0x2030, PDF_CMAP_SINGLE, 772 },
	{ 0x2032, 0x2033, PDF_CMAP_RANGE, 708 },
	{ 0x2039, 0x203a, PDF_CMAP_RANGE, 110 },
	{ 0x203b, 0x203c, PDF_CMAP_TABLE, 292 },
	{ 0x203e, 0x203f, PDF_CMAP_TABLE, 294 },
	{ 0x2042, 0x2042, PDF_CMAP_SINGLE, 16282 },
	{ 0x2044, 0x2044, PDF_CMAP_SINGLE, 104 },
	{ 0x2047, 0x2048, PDF_CMAP_RANGE, 16278 },
	{ 0x2049, 0x2049, PDF_CMAP_SINGLE, 12112 },
	{ 0x2051, 0x2051, PDF_CMAP_SINGLE, 16281 },
	{ 0x205a, 0x205a, PDF_CMAP_SINGLE, 7898 },
	{ 0x205d, 0x205d, PDF_CMAP_SINGLE, 7897 },
	{ 0x2070, 0x2070, PDF_CMAP_SINGLE, 9377 },
	{ 0x2074, 0x2079, PDF_CMAP_RANGE, 9378 },
	{ 0x2080, 0x2089, PDF_CMAP_RANGE, 9384 },
	{ 0x20ac, 0x20ac, PDF_CMAP_SINGLE, 9354 },
	{ 0x20dd, 0x20de, PDF_CMAP_TABLE, 296 },
	{ 0x2100, 0x2100, PDF_CMAP_SINGLE, 11855 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 710 },
	{ 0x2105, 0x2105, PDF_CMAP_SINGLE, 11859 },
	{ 0x2109, 0x210a, PDF_CMAP_TABLE, 298 },
	{ 0x210f, 0x210f, PDF_CMAP_SINGLE, 12092 },
	{ 0x2113, 0x2113, PDF_CMAP_SINGLE, 8025 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 7610 },
	{ 0x2121, 0x2122, PDF_CMAP_TABLE, 300 },
	{ 0x2126, 0x2127, PDF_CMAP_TABLE, 302 },
	{ 0x212b, 0x212b, PDF_CMAP_SINGLE, 771 },
	{ 0x212e, 0x212e, PDF_CMAP_SINGLE, 20366 },
	{ 0x2135, 0x2135, PDF_CMAP_SINGLE, 12089 },
	{ 0x213b, 0x213b, PDF_CMAP_SINGLE, 8307 },
	{ 0x2153, 0x2154, PDF_CMAP_RANGE, 9375 },
	{ 0x2155, 0x2155, PDF_CMAP_SINGLE, 15727 },
	{ 0x2156, 0x215a, PDF_CMAP_RANGE, 9785 },
	{ 0x215b, 0x215e, PDF_CMAP_RANGE, 9371 },
	{ 0x2160, 0x2169, PDF_CMAP_RANGE, 7575 },
	{ 0x216a, 0x216b, PDF_CMAP_RANGE, 8225 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 8092 },
	{ 0x217a, 0x217b, PDF_CMAP_RANGE, 8298 },
	{ 0x217f, 0x217f, PDF_CMAP_SINGLE, 8303 },
	{ 0x2190, 0x2191, PDF_CMAP_RANGE, 737 },
	{ 0x2192, 0x2194, PDF_CMAP_TABLE, 304 },
	{ 0x2196, 0x2197, PDF_CMAP_RANGE, 12204 },
	{ 0x2198, 0x2199, PDF_CMAP_RANGE, 12202 },
	{ 0x21c4, 0x21c5, PDF_CMAP_RANGE, 8310 },
	{ 0x21c6, 0x21c6, PDF_CMAP_SINGLE, 8309 },
	{ 0x21cb, 0x21cc, PDF_CMAP_TABLE, 307 },
	{ 0x21d0, 0x21d0, PDF_CMAP_SINGLE, 12200 },
	{ 0x21d2, 0x21d2, PDF_CMAP_SINGLE, 752 },
	{ 0x21d4, 0x21d4, PDF_CMAP_SINGLE, 753 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 309 },
	{ 0x2200, 0x2200, PDF_CMAP_SINGLE, 754 },
	{ 0x2202, 0x2203, PDF_CMAP_TABLE, 313 },
	{ 0x2205, 0x220b, PDF_CMAP_TABLE, 315 },
	{ 0x220f, 0x220f, PDF_CMAP_SINGLE, 20368 },
	{ 0x2211, 0x2213, PDF_CMAP_TABLE, 322 },
	{ 0x2219, 0x221a, PDF_CMAP_TABLE, 325 },
	{ 0x221d, 0x2220, PDF_CMAP_TABLE, 327 },
	{ 0x2225, 0x2226, PDF_CMAP_RANGE, 15489 },
	{ 0x2227, 0x2228, PDF_CMAP_RANGE, 749 },
	{ 0x2229, 0x222a, PDF_CMAP_TABLE, 331 },
	{ 0x222b, 0x222c, PDF_CMAP_RANGE, 769 },
	{ 0x222d, 0x222e, PDF_CMAP_TABLE, 333 },
	{ 0x2234, 0x2235, PDF_CMAP_TABLE, 335 },
	{ 0x223c, 0x223d, PDF_CMAP_TABLE, 337 },
	{ 0x2243, 0x2243, PDF_CMAP_SINGLE, 12120 },
	{ 0x2245, 0x2245, PDF_CMAP_SINGLE, 15507 },
	{ 0x2248, 0x2248, PDF_CMAP_SINGLE, 15508 },
	{ 0x2252, 0x2252, PDF_CMAP_SINGLE, 762 },
	{ 0x2260, 0x2262, PDF_CMAP_TABLE, 339 },
	{ 0x2264, 0x2265, PDF_CMAP_RANGE, 20369 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 701 },
	{ 0x226a, 0x226b, PDF_CMAP_RANGE, 763 },
	{ 0x2272, 0x2273, PDF_CMAP_RANGE, 12121 },
	{ 0x2276, 0x2277, PDF_CMAP_RANGE, 15509 },
	{ 0x2282, 0x2283, PDF_CMAP_RANGE, 745 },
	{ 0x2284, 0x2285, PDF_CMAP_RANGE, 15472 },
	{ 0x2286, 0x2287, PDF_CMAP_RANGE, 743 },
	{ 0x228a, 0x228b, PDF_CMAP_RANGE, 15474 },
	{ 0x2295, 0x2298, PDF_CMAP_TABLE, 342 },
	{ 0x229e, 0x229e, PDF_CMAP_SINGLE, 15906 },
	{ 0x22a0, 0x22a0, PDF_CMAP_SINGLE, 12185 },
	{ 0x22a5, 0x22a5, PDF_CMAP_SINGLE, 757 },
	{ 0x22bf, 0x22bf, PDF_CMAP_SINGLE, 7630 },
	{ 0x22da, 0x22db, PDF_CMAP_RANGE, 15725 },
	{ 0x2305, 0x2306, PDF_CMAP_RANGE, 15478 },
	{ 0x2307, 0x2307, PDF_CMAP_SINGLE, 12219 },
	{ 0x2312, 0x2312, PDF_CMAP_SINGLE, 758 },
	{ 0x2318, 0x2318, PDF_CMAP_SINGLE, 15728 },
	{ 0x239b, 0x239c, PDF_CMAP_TABLE, 346 },
	{ 0x239d, 0x239e, PDF_CMAP_RANGE, 12144 },
	{ 0x239f, 0x23a2, PDF_CMAP_TABLE, 348 },
	{ 0x23a3, 0x23a4, PDF_CMAP_RANGE, 12152 },
	{ 0x23a5, 0x23a6, PDF_CMAP_TABLE, 352 },
	{ 0x23a7, 0x23a9, PDF_CMAP_RANGE, 8178 },
	{ 0x23aa, 0x23aa, PDF_CMAP_SINGLE, 12167 },
	{ 0x23ab, 0x23ad, PDF_CMAP_RANGE, 8174 },
	{ 0x23b0, 0x23b1, PDF_CMAP_RANGE, 16312 },
	{ 0x23be, 0x23cc, PDF_CMAP_RANGE, 16253 },
	{ 0x23ce, 0x23ce, PDF_CMAP_SINGLE, 16273 },
	{ 0x2423, 0x2423, PDF_CMAP_SINGLE, 16272 },
	{ 0x2460, 0x2473, PDF_CMAP_RANGE, 7555 },
	{ 0x2474, 0x2487, PDF_CMAP_RANGE, 8071 },
	{ 0x2488, 0x2490, PDF_CMAP_RANGE, 8062 },
	{ 0x2491, 0x2492, PDF_CMAP_RANGE, 20587 },
	{ 0x249c, 0x24b5, PDF_CMAP_RANGE, 8112 },
	{ 0x24b6, 0x24cf, PDF_CMAP_RANGE, 10339 },
	{ 0x24d0, 0x24e9, PDF_CMAP_RANGE, 10313 },
	{ 0x24ea, 0x24ea, PDF_CMAP_SINGLE, 8224 },
	{ 0x24eb, 0x24f4, PDF_CMAP_RANGE, 10515 },
	{ 0x24f5, 0x24fe, PDF_CMAP_RANGE, 16223 },
	{ 0x24ff, 0x24ff, PDF_CMAP_SINGLE, 10503 },
	{ 0x2500, 0x254b, PDF_CMAP_RANGE, 7479 },
	{ 0x2550, 0x2550, PDF_CMAP_SINGLE, 8251 },
	{ 0x255e, 0x255e, PDF_CMAP_SINGLE, 8252 },
	{ 0x2561, 0x2561, PDF_CMAP_SINGLE, 8254 },
	{ 0x256a, 0x256a, PDF_CMAP_SINGLE, 8253 },
	{ 0x256d, 0x256e, PDF_CMAP_RANGE, 8247 },
	{ 0x256f, 0x2570, PDF_CMAP_TABLE, 354 },
	{ 0x2571, 0x2573, PDF_CMAP_RANGE, 8261 },
	{ 0x2581, 0x2588, PDF_CMAP_RANGE, 8230 },
	{ 0x2589, 0x258f, PDF_CMAP_TABLE, 356 },
	{ 0x2594, 0x2595, PDF_CMAP_RANGE, 8245 },
	{ 0x25a0, 0x25a2, PDF_CMAP_TABLE, 363 },
	{ 0x25aa, 0x25ab, PDF_CMAP_TABLE, 366 },
	{ 0x25b1, 0x25b3, PDF_CMAP_TABLE, 368 },
	{ 0x25b6, 0x25b7, PDF_CMAP_TABLE, 371 },
	{ 0x25bc, 0x25bd, PDF_CMAP_TABLE, 373 },
	{ 0x25c0, 0x25c1, PDF_CMAP_TABLE, 375 },
	{ 0x25c6, 0x25c7, PDF_CMAP_TABLE, 377 },
	{ 0x25c9, 0x25cc, PDF_CMAP_TABLE, 379 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 383 },
	{ 0x25d0, 0x25d3, PDF_CMAP_RANGE, 16274 },
	{ 0x25e2, 0x25e3, PDF_CMAP_RANGE, 8255 },
	{ 0x25e4, 0x25e6, PDF_CMAP_TABLE, 385 },
	{ 0x25ef, 0x25ef, PDF_CMAP_SINGLE, 779 },
	{ 0x2600, 0x2603, PDF_CMAP_RANGE, 8215 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 388 },
	{ 0x260e, 0x260e, PDF_CMAP_SINGLE, 8056 },
	{ 0x2616, 0x2617, PDF_CMAP_RANGE, 16233 },
	{ 0x261c, 0x261d, PDF_CMAP_RANGE, 8220 },
	{ 0x261e, 0x261f, PDF_CMAP_TABLE, 390 },
	{ 0x2640, 0x2640, PDF_CMAP_SINGLE, 706 },
	{ 0x2642, 0x2642, PDF_CMAP_SINGLE, 705 },
	{ 0x2660, 0x2667, PDF_CMAP_TABLE, 392 },
	{ 0x2668, 0x2669, PDF_CMAP_RANGE, 12098 },
	{ 0x266a, 0x266f, PDF_CMAP_TABLE, 400 },
	{ 0x2672, 0x267d, PDF_CMAP_RANGE, 16314 },
	{ 0x26a0, 0x26a0, PDF_CMAP_SINGLE, 12192 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12176 },
	{ 0x2713, 0x2713, PDF_CMAP_SINGLE, 16270 },
	{ 0x271a, 0x271a, PDF_CMAP_SINGLE, 12241 },
	{ 0x273f, 0x2740, PDF_CMAP_TABLE, 406 },
	{ 0x2756, 0x2756, PDF_CMAP_SINGLE, 12259 },
	{ 0x2776, 0x277e, PDF_CMAP_RANGE, 8286 },
	{ 0x277f, 0x277f, PDF_CMAP_SINGLE, 10514 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8206 },
	{ 0x2934, 0x2935, PDF_CMAP_RANGE, 16201 },
	{ 0x29bf, 0x29bf, PDF_CMAP_SINGLE, 16203 },
	{ 0x29fa, 0x29fb, PDF_CMAP_RANGE, 16207 },
	{ 0x2b05, 0x2b07, PDF_CMAP_RANGE, 8207 },
	{ 0x2e83, 0x2e83, PDF_CMAP_SINGLE, 14305 },
	{ 0x2e85, 0x2e85, PDF_CMAP_SINGLE, 13856 },
	{ 0x2e87, 0x2e87, PDF_CMAP_SINGLE, 14105 },
	{ 0x2e89, 0x2e89, PDF_CMAP_SINGLE, 14356 },
	{ 0x2e8b, 0x2e8b, PDF_CMAP_SINGLE, 14110 },
	{ 0x2e8c, 0x2e8d, PDF_CMAP_RANGE, 13833 },
	{ 0x2e8e, 0x2e90, PDF_CMAP_TABLE, 408 },
	{ 0x2e92, 0x2e99, PDF_CMAP_TABLE, 411 },
	{ 0x2e9b, 0x2e9b, PDF_CMAP_SINGLE, 5089 },
	{ 0x2e9e, 0x2ea0, PDF_CMAP_TABLE, 419 },
	{ 0x2ea1, 0x2ea2, PDF_CMAP_RANGE, 14689 },
	{ 0x2ea3, 0x2ea4, PDF_CMAP_TABLE, 422 },
	{ 0x2ea6, 0x2ea6, PDF_CMAP_SINGLE, 14157 },
	{ 0x2ea8, 0x2eae, PDF_CMAP_TABLE, 424 },
	{ 0x2eb1, 0x2eb3, PDF_CMAP_TABLE, 431 },
	{ 0x2eb7, 0x2eb7, PDF_CMAP_SINGLE, 14078 },
	{ 0x2eb9, 0x2eb9, PDF_CMAP_SINGLE, 14099 },
	{ 0x2ebc, 0x2ebd, PDF_CMAP_TABLE, 434 },
	{ 0x2ebe, 0x2ec0, PDF_CMAP_RANGE, 14197 },
	{ 0x2ec1, 0x2ec4, PDF_CMAP_TABLE, 436 },
	{ 0x2ec6, 0x2ec6, PDF_CMAP_SINGLE, 13682 },
	{ 0x2eca, 0x2eca, PDF_CMAP_SINGLE, 13898 },
	{ 0x2ecc, 0x2ecd, PDF_CMAP_TABLE, 440 },
	{ 0x2ecf, 0x2ecf, PDF_CMAP_SINGLE, 15262 },
	{ 0x2ed1, 0x2ed2, PDF_CMAP_TABLE, 442 },
	{ 0x2ed6, 0x2ed8, PDF_CMAP_TABLE, 444 },
	{ 0x2edd, 0x2edf, PDF_CMAP_TABLE, 447 },
	{ 0x2ee4, 0x2ee4, PDF_CMAP_SINGLE, 1614 },
	{ 0x2ee8, 0x2ee9, PDF_CMAP_TABLE, 450 },
	{ 0x2eeb, 0x2eeb, PDF_CMAP_SINGLE, 2666 },
	{ 0x2eed, 0x2eed, PDF_CMAP_SINGLE, 2243 },
	{ 0x2eef, 0x2eef, PDF_CMAP_SINGLE, 3965 },
	{ 0x2ef2, 0x2ef2, PDF_CMAP_SINGLE, 1615 },
	{ 0x2f00, 0x2f13, PDF_CMAP_TABLE, 452 },
	{ 0x2f14, 0x2f15, PDF_CMAP_RANGE, 4301 },
	{ 0x2f16, 0x2f20, PDF_CMAP_TABLE, 472 },
	{ 0x2f21, 0x2f22, PDF_CMAP_RANGE, 4538 },
	{ 0x2f23, 0x2fbe, PDF_CMAP_TABLE, 483 },
	{ 0x2fbf, 0x2fc0, PDF_CMAP_RANGE, 7299 },
	{ 0x2fc1, 0x2fd0, PDF_CMAP_TABLE, 639 },
	{ 0x2fd1, 0x2fd2, PDF_CMAP_RANGE, 7457 },
	{ 0x2fd3, 0x2fd3, PDF_CMAP_SINGLE, 3966 },
	{ 0x2fd4, 0x2fd5, PDF_CMAP_RANGE, 7472 },
	{ 0x3000, 0x3002, PDF_CMAP_RANGE, 633 },
	{ 0x3003, 0x3004, PDF_CMAP_TABLE, 655 },
	{ 0x3005, 0x3007, PDF_CMAP_RANGE, 657 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 682 },
	{ 0x3012, 0x3013, PDF_CMAP_TABLE, 657 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 676 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 16197 },
	{ 0x3018, 0x3019, PDF_CMAP_RANGE, 12129 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 659 },
	{ 0x301f, 0x3020, PDF_CMAP_TABLE, 661 },
	{ 0x3030, 0x3030, PDF_CMAP_SINGLE, 12218 },
	{ 0x3033, 0x3035, PDF_CMAP_RANGE, 12108 },
	{ 0x3036, 0x3036, PDF_CMAP_SINGLE, 8057 },
	{ 0x303b, 0x303d, PDF_CMAP_TABLE, 663 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 842 },
	{ 0x3094, 0x3096, PDF_CMAP_RANGE, 7958 },
	{ 0x3099, 0x309a, PDF_CMAP_RANGE, 16326 },
	{ 0x309b, 0x309c, PDF_CMAP_RANGE, 643 },
	{ 0x309d, 0x309e, PDF_CMAP_RANGE, 653 },
	{ 0x309f, 0x30a0, PDF_CMAP_TABLE, 666 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 925 },
	{ 0x30f7, 0x30fa, PDF_CMAP_RANGE, 8313 },
	{ 0x30fb, 0x30fc, PDF_CMAP_TABLE, 668 },
	{ 0x30fd, 0x30fe, PDF_CMAP_RANGE, 651 },
	{ 0x30ff, 0x30ff, PDF_CMAP_SINGLE, 16195 },
	{ 0x3190, 0x319f, PDF_CMAP_RANGE, 16283 },
	{ 0x31f0, 0x31f9, PDF_CMAP_RANGE, 16236 },
	{ 0x31fa, 0x31ff, PDF_CMAP_RANGE, 16247 },
	{ 0x3220, 0x3229, PDF_CMAP_RANGE, 10126 },
	{ 0x322a, 0x322f, PDF_CMAP_RANGE, 8198 },
	{ 0x3230, 0x3230, PDF_CMAP_SINGLE, 8197 },
	{ 0x3231, 0x3232, PDF_CMAP_RANGE, 7618 },
	{ 0x3233, 0x3243, PDF_CMAP_TABLE, 670 },
	{ 0x3251, 0x3251, PDF_CMAP_SINGLE, 8091 },
	{ 0x3252, 0x325b, PDF_CMAP_RANGE, 8102 },
	{ 0x325c, 0x325f, PDF_CMAP_RANGE, 10244 },
	{ 0x3280, 0x3289, PDF_CMAP_RANGE, 10461 },
	{ 0x328a, 0x328f, PDF_CMAP_RANGE, 10472 },
	{ 0x3290, 0x329f, PDF_CMAP_TABLE, 687 },
	{ 0x32a0, 0x32a1, PDF_CMAP_RANGE, 10486 },
	{ 0x32a2, 0x32a3, PDF_CMAP_TABLE, 703 },
	{ 0x32a4, 0x32a8, PDF_CMAP_RANGE, 7613 },
	{ 0x32a9, 0x32b0, PDF_CMAP_TABLE, 705 },
	{ 0x32b1, 0x32bf, PDF_CMAP_RANGE, 10248 },
	{ 0x32d0, 0x32fe, PDF_CMAP_RANGE, 10413 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8048 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11874 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 713 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11889 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 724 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11900 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8051 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11904 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 730 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11912 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 736 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11915 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 11918 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 738 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 11924 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 741 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 11936 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 749 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 11939 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 751 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 11944 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 755 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 11946 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 757 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 11955 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8044 },
	{ 0x3371, 0x3371, PDF_CMAP_SINGLE, 11861 },
	{ 0x337b, 0x337f, PDF_CMAP_TABLE, 761 },
	{ 0x3385, 0x3387, PDF_CMAP_RANGE, 8031 },
	{ 0x3388, 0x3389, PDF_CMAP_RANGE, 8192 },
	{ 0x338d, 0x338d, PDF_CMAP_SINGLE, 11864 },
	{ 0x338e, 0x338f, PDF_CMAP_RANGE, 7604 },
	{ 0x3390, 0x3390, PDF_CMAP_SINGLE, 8035 },
	{ 0x3396, 0x3398, PDF_CMAP_TABLE, 766 },
	{ 0x339b, 0x339b, PDF_CMAP_SINGLE, 11865 },
	{ 0x339c, 0x339e, PDF_CMAP_RANGE, 7601 },
	{ 0x339f, 0x33a3, PDF_CMAP_TABLE, 769 },
	{ 0x33a4, 0x33a5, PDF_CMAP_RANGE, 8022 },
	{ 0x33a6, 0x33a6, PDF_CMAP_SINGLE, 8188 },
	{ 0x33b0, 0x33b3, PDF_CMAP_TABLE, 774 },
	{ 0x33c2, 0x33c2, PDF_CMAP_SINGLE, 11856 },
	{ 0x33c4, 0x33c4, PDF_CMAP_SINGLE, 7606 },
	{ 0x33c8, 0x33c8, PDF_CMAP_SINGLE, 8194 },
	{ 0x33cb, 0x33cd, PDF_CMAP_TABLE, 778 },
	{ 0x33d4, 0x33d4, PDF_CMAP_SINGLE, 8036 },
	{ 0x33d7, 0x33d8, PDF_CMAP_RANGE, 11869 },
	{ 0x33da, 0x33da, PDF_CMAP_SINGLE, 11851 },
	{ 0x3402, 0x3402, PDF_CMAP_SINGLE, 13698 },
	{ 0x3405, 0x3406, PDF_CMAP_TABLE, 781 },
	{ 0x3427, 0x3427, PDF_CMAP_SINGLE, 13910 },
	{ 0x342c, 0x342c, PDF_CMAP_SINGLE, 17246 },
	{ 0x342e, 0x342e, PDF_CMAP_SINGLE, 14216 },
	{ 0x3468, 0x3468, PDF_CMAP_SINGLE, 14047 },
	{ 0x346a, 0x346a, PDF_CMAP_SINGLE, 17269 },
	{ 0x3488, 0x3488, PDF_CMAP_SINGLE, 15442 },
	{ 0x3492, 0x3492, PDF_CMAP_SINGLE, 17294 },
	{ 0x34b5, 0x34b5, PDF_CMAP_SINGLE, 16793 },
	{ 0x34bc, 0x34bc, PDF_CMAP_SINGLE, 17303 },
	{ 0x34c1, 0x34c1, PDF_CMAP_SINGLE, 18384 },
	{ 0x34c7, 0x34c7, PDF_CMAP_SINGLE, 17307 },
	{ 0x34db, 0x34db, PDF_CMAP_SINGLE, 15425 },
	{ 0x351f, 0x351f, PDF_CMAP_SINGLE, 13865 },
	{ 0x353e, 0x353e, PDF_CMAP_SINGLE, 14110 },
	{ 0x355d, 0x355e, PDF_CMAP_RANGE, 17341 },
	{ 0x3563, 0x3563, PDF_CMAP_SINGLE, 17344 },
	{ 0x356e, 0x356e, PDF_CMAP_SINGLE, 17348 },
	{ 0x35a6, 0x35a6, PDF_CMAP_SINGLE, 17369 },
	{ 0x35a8, 0x35a8, PDF_CMAP_SINGLE, 17371 },
	{ 0x35c5, 0x35c5, PDF_CMAP_SINGLE, 17377 },
	{ 0x35da, 0x35da, PDF_CMAP_SINGLE, 17386 },
	{ 0x35de, 0x35de, PDF_CMAP_SINGLE, 20067 },
	{ 0x35f4, 0x35f4, PDF_CMAP_SINGLE, 17395 },
	{ 0x3605, 0x3605, PDF_CMAP_SINGLE, 17402 },
	{ 0x3614, 0x3614, PDF_CMAP_SINGLE, 19131 },
	{ 0x364a, 0x364a, PDF_CMAP_SINGLE, 17441 },
	{ 0x3691, 0x3691, PDF_CMAP_SINGLE, 17473 },
	{ 0x3696, 0x3696, PDF_CMAP_SINGLE, 17477 },
	{ 0x3699, 0x3699, PDF_CMAP_SINGLE, 17475 },
	{ 0x36cf, 0x36cf, PDF_CMAP_SINGLE, 17494 },
	{ 0x3761, 0x3762, PDF_CMAP_RANGE, 17528 },
	{ 0x376b, 0x376c, PDF_CMAP_TABLE, 783 },
	{ 0x3775, 0x3775, PDF_CMAP_SINGLE, 17536 },
	{ 0x378d, 0x378d, PDF_CMAP_SINGLE, 13850 },
	{ 0x37c1, 0x37c1, PDF_CMAP_SINGLE, 17550 },
	{ 0x37e2, 0x37e2, PDF_CMAP_SINGLE, 14123 },
	{ 0x37e8, 0x37e8, PDF_CMAP_SINGLE, 17570 },
	{ 0x37f4, 0x37f4, PDF_CMAP_SINGLE, 17573 },
	{ 0x37fd, 0x37fd, PDF_CMAP_SINGLE, 17576 },
	{ 0x3800, 0x3800, PDF_CMAP_SINGLE, 17578 },
	{ 0x382f, 0x382f, PDF_CMAP_SINGLE, 17588 },
	{ 0x3836, 0x3836, PDF_CMAP_SINGLE, 17589 },
	{ 0x3840, 0x3840, PDF_CMAP_SINGLE, 17590 },
	{ 0x385c, 0x385c, PDF_CMAP_SINGLE, 17594 },
	{ 0x3861, 0x3861, PDF_CMAP_SINGLE, 17596 },
	{ 0x38a1, 0x38a1, PDF_CMAP_SINGLE, 20171 },
	{ 0x38ad, 0x38ad, PDF_CMAP_SINGLE, 19132 },
	{ 0x38fa, 0x38fa, PDF_CMAP_SINGLE, 13852 },
	{ 0x3917, 0x3917, PDF_CMAP_SINGLE, 17625 },
	{ 0x391a, 0x391a, PDF_CMAP_SINGLE, 17628 },
	{ 0x396f, 0x396f, PDF_CMAP_SINGLE, 17643 },
	{ 0x39a4, 0x39a4, PDF_CMAP_SINGLE, 20122 },
	{ 0x39b8, 0x39b8, PDF_CMAP_SINGLE, 20123 },
	{ 0x3a5c, 0x3a5c, PDF_CMAP_SINGLE, 20127 },
	{ 0x3a6e, 0x3a6e, PDF_CMAP_SINGLE, 17713 },
	{ 0x3a73, 0x3a73, PDF_CMAP_SINGLE, 17716 },
	{ 0x3a85, 0x3a85, PDF_CMAP_SINGLE, 20142 },
	{ 0x3ac4, 0x3ac4, PDF_CMAP_SINGLE, 20135 },
	{ 0x3acb, 0x3acb, PDF_CMAP_SINGLE, 20136 },
	{ 0x3ad6, 0x3ad7, PDF_CMAP_TABLE, 785 },
	{ 0x3aea, 0x3aea, PDF_CMAP_SINGLE, 17741 },
	{ 0x3af3, 0x3af3, PDF_CMAP_SINGLE, 15424 },
	{ 0x3b0e, 0x3b0e, PDF_CMAP_SINGLE, 17752 },
	{ 0x3b1a, 0x3b1a, PDF_CMAP_SINGLE, 17756 },
	{ 0x3b1c, 0x3b1c, PDF_CMAP_SINGLE, 17757 },
	{ 0x3b22, 0x3b22, PDF_CMAP_SINGLE, 15433 },
	{ 0x3b6d, 0x3b6d, PDF_CMAP_SINGLE, 17804 },
	{ 0x3b77, 0x3b77, PDF_CMAP_SINGLE, 17797 },
	{ 0x3b87, 0x3b88, PDF_CMAP_TABLE, 787 },
	{ 0x3b8d, 0x3b8d, PDF_CMAP_SINGLE, 17828 },
	{ 0x3ba4, 0x3ba4, PDF_CMAP_SINGLE, 17834 },
	{ 0x3bb6, 0x3bb6, PDF_CMAP_SINGLE, 16910 },
	{ 0x3bc3, 0x3bc3, PDF_CMAP_SINGLE, 16911 },
	{ 0x3bcd, 0x3bcd, PDF_CMAP_SINGLE, 17848 },
	{ 0x3bf0, 0x3bf0, PDF_CMAP_SINGLE, 17865 },
	{ 0x3bf3, 0x3bf3, PDF_CMAP_SINGLE, 20151 },
	{ 0x3c0f, 0x3c0f, PDF_CMAP_SINGLE, 16919 },
	{ 0x3c26, 0x3c26, PDF_CMAP_SINGLE, 17887 },
	{ 0x3cc3, 0x3cc3, PDF_CMAP_SINGLE, 17916 },
	{ 0x3cd2, 0x3cd2, PDF_CMAP_SINGLE, 17921 },
	{ 0x3d11, 0x3d11, PDF_CMAP_SINGLE, 17957 },
	{ 0x3d1e, 0x3d1e, PDF_CMAP_SINGLE, 17968 },
	{ 0x3d31, 0x3d31, PDF_CMAP_SINGLE, 20158 },
	{ 0x3d4e, 0x3d4e, PDF_CMAP_SINGLE, 7655 },
	{ 0x3d64, 0x3d64, PDF_CMAP_SINGLE, 17994 },
	{ 0x3d9a, 0x3d9a, PDF_CMAP_SINGLE, 18008 },
	{ 0x3dc0, 0x3dc0, PDF_CMAP_SINGLE, 18026 },
	{ 0x3dcc, 0x3dcc, PDF_CMAP_SINGLE, 19133 },
	{ 0x3dd4, 0x3dd4, PDF_CMAP_SINGLE, 18031 },
	{ 0x3e05, 0x3e05, PDF_CMAP_SINGLE, 18043 },
	{ 0x3e3f, 0x3e40, PDF_CMAP_TABLE, 789 },
	{ 0x3e60, 0x3e60, PDF_CMAP_SINGLE, 18059 },
	{ 0x3e66, 0x3e66, PDF_CMAP_SINGLE, 18061 },
	{ 0x3e68, 0x3e68, PDF_CMAP_SINGLE, 18062 },
	{ 0x3e83, 0x3e83, PDF_CMAP_SINGLE, 18069 },
	{ 0x3e8a, 0x3e8a, PDF_CMAP_SINGLE, 15427 },
	{ 0x3e94, 0x3e94, PDF_CMAP_SINGLE, 18075 },
	{ 0x3eda, 0x3eda, PDF_CMAP_SINGLE, 15432 },
	{ 0x3f57, 0x3f57, PDF_CMAP_SINGLE, 18123 },
	{ 0x3f72, 0x3f72, PDF_CMAP_SINGLE, 16984 },
	{ 0x3f75, 0x3f75, PDF_CMAP_SINGLE, 18137 },
	{ 0x3f77, 0x3f77, PDF_CMAP_SINGLE, 18139 },
	{ 0x3fae, 0x3fae, PDF_CMAP_SINGLE, 18157 },
	{ 0x3fb1, 0x3fb1, PDF_CMAP_SINGLE, 14164 },
	{ 0x3fc9, 0x3fc9, PDF_CMAP_SINGLE, 18168 },
	{ 0x3fd7, 0x3fd7, PDF_CMAP_SINGLE, 18173 },
	{ 0x3fdc, 0x3fdc, PDF_CMAP_SINGLE, 19134 },
	{ 0x4039, 0x4039, PDF_CMAP_SINGLE, 18191 },
	{ 0x4058, 0x4058, PDF_CMAP_SINGLE, 18198 },
	{ 0x4093, 0x4093, PDF_CMAP_SINGLE, 15436 },
	{ 0x4103, 0x4103, PDF_CMAP_SINGLE, 15439 },
	{ 0x4105, 0x4105, PDF_CMAP_SINGLE, 18235 },
	{ 0x4148, 0x4148, PDF_CMAP_SINGLE, 18247 },
	{ 0x414f, 0x414f, PDF_CMAP_SINGLE, 18250 },
	{ 0x4163, 0x4163, PDF_CMAP_SINGLE, 18254 },
	{ 0x41b4, 0x41b4, PDF_CMAP_SINGLE, 18269 },
	{ 0x41bf, 0x41bf, PDF_CMAP_SINGLE, 18272 },
	{ 0x41e6, 0x41e6, PDF_CMAP_SINGLE, 18283 },
	{ 0x41ee, 0x41ee, PDF_CMAP_SINGLE, 18287 },
	{ 0x41f3, 0x41f3, PDF_CMAP_SINGLE, 18284 },
	{ 0x4207, 0x4207, PDF_CMAP_SINGLE, 18294 },
	{ 0x420e, 0x420e, PDF_CMAP_SINGLE, 18297 },
	{ 0x4264, 0x4264, PDF_CMAP_SINGLE, 14176 },
	{ 0x4293, 0x4293, PDF_CMAP_SINGLE, 15440 },
	{ 0x42c6, 0x42c6, PDF_CMAP_SINGLE, 18335 },
	{ 0x42d6, 0x42d6, PDF_CMAP_SINGLE, 18346 },
	{ 0x42dd, 0x42dd, PDF_CMAP_SINGLE, 18350 },
	{ 0x4302, 0x4302, PDF_CMAP_SINGLE, 18364 },
	{ 0x432b, 0x432b, PDF_CMAP_SINGLE, 18377 },
	{ 0x4343, 0x4343, PDF_CMAP_SINGLE, 18379 },
	{ 0x43ee, 0x43ee, PDF_CMAP_SINGLE, 18423 },
	{ 0x43f0, 0x43f0, PDF_CMAP_SINGLE, 18426 },
	{ 0x4408, 0x4408, PDF_CMAP_SINGLE, 18432 },
	{ 0x440c, 0x440c, PDF_CMAP_SINGLE, 15426 },
	{ 0x4417, 0x4417, PDF_CMAP_SINGLE, 18434 },
	{ 0x441c, 0x441c, PDF_CMAP_SINGLE, 18435 },
	{ 0x4422, 0x4422, PDF_CMAP_SINGLE, 18437 },
	{ 0x4453, 0x4453, PDF_CMAP_SINGLE, 14195 },
	{ 0x445b, 0x445b, PDF_CMAP_SINGLE, 17060 },
	{ 0x4476, 0x4476, PDF_CMAP_SINGLE, 18460 },
	{ 0x447a, 0x447a, PDF_CMAP_SINGLE, 18461 },
	{ 0x4491, 0x4491, PDF_CMAP_SINGLE, 18467 },
	{ 0x44b3, 0x44b3, PDF_CMAP_SINGLE, 18494 },
	{ 0x44be, 0x44be, PDF_CMAP_SINGLE, 18492 },
	{ 0x44d4, 0x44d4, PDF_CMAP_SINGLE, 18493 },
	{ 0x4508, 0x4508, PDF_CMAP_SINGLE, 18524 },
	{ 0x450d, 0x450d, PDF_CMAP_SINGLE, 18520 },
	{ 0x4525, 0x4525, PDF_CMAP_SINGLE, 14206 },
	{ 0x4543, 0x4543, PDF_CMAP_SINGLE, 18540 },
	{ 0x457a, 0x457a, PDF_CMAP_SINGLE, 15435 },
	{ 0x459d, 0x459d, PDF_CMAP_SINGLE, 17108 },
	{ 0x45b8, 0x45b8, PDF_CMAP_SINGLE, 18592 },
	{ 0x45be, 0x45be, PDF_CMAP_SINGLE, 19135 },
	{ 0x45e5, 0x45e5, PDF_CMAP_SINGLE, 18618 },
	{ 0x45ea, 0x45ea, PDF_CMAP_SINGLE, 17113 },
	{ 0x460f, 0x4610, PDF_CMAP_TABLE, 791 },
	{ 0x4641, 0x4641, PDF_CMAP_SINGLE, 18648 },
	{ 0x4665, 0x4665, PDF_CMAP_SINGLE, 15438 },
	{ 0x46a1, 0x46a1, PDF_CMAP_SINGLE, 18665 },
	{ 0x46ae, 0x46af, PDF_CMAP_TABLE, 793 },
	{ 0x470c, 0x470c, PDF_CMAP_SINGLE, 18690 },
	{ 0x471f, 0x471f, PDF_CMAP_SINGLE, 20219 },
	{ 0x4764, 0x4764, PDF_CMAP_SINGLE, 18712 },
	{ 0x47e6, 0x47e6, PDF_CMAP_SINGLE, 14224 },
	{ 0x47fd, 0x47fd, PDF_CMAP_SINGLE, 18724 },
	{ 0x4816, 0x4816, PDF_CMAP_SINGLE, 18734 },
	{ 0x481e, 0x481e, PDF_CMAP_SINGLE, 20225 },
	{ 0x4844, 0x4844, PDF_CMAP_SINGLE, 17141 },
	{ 0x484e, 0x484e, PDF_CMAP_SINGLE, 18746 },
	{ 0x48b5, 0x48b5, PDF_CMAP_SINGLE, 18779 },
	{ 0x49b0, 0x49b0, PDF_CMAP_SINGLE, 17183 },
	{ 0x49e7, 0x49e7, PDF_CMAP_SINGLE, 18898 },
	{ 0x49fa, 0x49fa, PDF_CMAP_SINGLE, 18902 },
	{ 0x4a04, 0x4a04, PDF_CMAP_SINGLE, 18907 },
	{ 0x4a29, 0x4a29, PDF_CMAP_SINGLE, 18910 },
	{ 0x4abc, 0x4abc, PDF_CMAP_SINGLE, 18934 },
	{ 0x4b38, 0x4b38, PDF_CMAP_SINGLE, 13791 },
	{ 0x4b3b, 0x4b3b, PDF_CMAP_SINGLE, 18958 },
	{ 0x4b7e, 0x4b7e, PDF_CMAP_SINGLE, 19137 },
	{ 0x4bc2, 0x4bc2, PDF_CMAP_SINGLE, 18995 },
	{ 0x4bca, 0x4bca, PDF_CMAP_SINGLE, 18997 },
	{ 0x4bd2, 0x4bd2, PDF_CMAP_SINGLE, 18999 },
	{ 0x4be8, 0x4be8, PDF_CMAP_SINGLE, 15430 },
	{ 0x4c17, 0x4c17, PDF_CMAP_SINGLE, 17204 },
	{ 0x4c20, 0x4c20, PDF_CMAP_SINGLE, 19016 },
	{ 0x4c38, 0x4c38, PDF_CMAP_SINGLE, 19138 },
	{ 0x4cc4, 0x4cc4, PDF_CMAP_SINGLE, 19076 },
	{ 0x4cd1, 0x4cd1, PDF_CMAP_SINGLE, 19079 },
	{ 0x4ce1, 0x4ce1, PDF_CMAP_SINGLE, 19139 },
	{ 0x4d07, 0x4d07, PDF_CMAP_SINGLE, 19099 },
	{ 0x4d77, 0x4d77, PDF_CMAP_SINGLE, 19115 },
	{ 0x4e00, 0x4e03, PDF_CMAP_TABLE, 795 },
	{ 0x4e04, 0x4e05, PDF_CMAP_RANGE, 14296 },
	{ 0x4e07, 0x4e12, PDF_CMAP_TABLE, 799 },
	{ 0x4e14, 0x4e19, PDF_CMAP_TABLE, 811 },
	{ 0x4e1e, 0x4e1f, PDF_CMAP_TABLE, 817 },
	{ 0x4e21, 0x4e21, PDF_CMAP_SINGLE, 3974 },
	{ 0x4e23, 0x4e24, PDF_CMAP_RANGE, 19141 },
	{ 0x4e26, 0x4e26, PDF_CMAP_SINGLE, 3602 },
	{ 0x4e28, 0x4e2e, PDF_CMAP_TABLE, 819 },
	{ 0x4e2f, 0x4e30, PDF_CMAP_RANGE, 14300 },
	{ 0x4e31, 0x4e32, PDF_CMAP_TABLE, 826 },
	{ 0x4e35, 0x4e39, PDF_CMAP_TABLE, 828 },
	{ 0x4e3b, 0x4e3c, PDF_CMAP_TABLE, 833 },
	{ 0x4e3f, 0x4e3f, PDF_CMAP_SINGLE, 4097 },
	{ 0x4e40, 0x4e41, PDF_CMAP_RANGE, 14302 },
	{ 0x4e42, 0x4e45, PDF_CMAP_TABLE, 835 },
	{ 0x4e47, 0x4e48, PDF_CMAP_TABLE, 839 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 3309 },
	{ 0x4e4d, 0x4e4f, PDF_CMAP_TABLE, 841 },
	{ 0x4e51, 0x4e51, PDF_CMAP_SINGLE, 17241 },
	{ 0x4e55, 0x4e5a, PDF_CMAP_TABLE, 844 },
	{ 0x4e5c, 0x4e5f, PDF_CMAP_TABLE, 850 },
	{ 0x4e62, 0x4e63, PDF_CMAP_TABLE, 854 },
	{ 0x4e68, 0x4e69, PDF_CMAP_TABLE, 856 },
	{ 0x4e71, 0x4e71, PDF_CMAP_SINGLE, 3930 },
	{ 0x4e73, 0x4e73, PDF_CMAP_SINGLE, 3285 },
	{ 0x4e74, 0x4e75, PDF_CMAP_RANGE, 21079 },
	{ 0x4e79, 0x4e79, PDF_CMAP_SINGLE, 19143 },
	{ 0x4e7e, 0x4e80, PDF_CMAP_TABLE, 858 },
	{ 0x4e82, 0x4e82, PDF_CMAP_SINGLE, 4101 },
	{ 0x4e85, 0x4e86, PDF_CMAP_TABLE, 861 },
	{ 0x4e88, 0x4e8e, PDF_CMAP_TABLE, 863 },
	{ 0x4e91, 0x4e92, PDF_CMAP_TABLE, 870 },
	{ 0x4e94, 0x4e99, PDF_CMAP_TABLE, 872 },
	{ 0x4e9b, 0x4e9d, PDF_CMAP_TABLE, 878 },
	{ 0x4e9e, 0x4ea0, PDF_CMAP_RANGE, 4108 },
	{ 0x4ea1, 0x4ea2, PDF_CMAP_TABLE, 881 },
	{ 0x4ea4, 0x4ea6, PDF_CMAP_TABLE, 883 },
	{ 0x4ea8, 0x4ea8, PDF_CMAP_SINGLE, 1686 },
	{ 0x4eab, 0x4eac, PDF_CMAP_RANGE, 1687 },
	{ 0x4ead, 0x4eb0, PDF_CMAP_TABLE, 886 },
	{ 0x4eb3, 0x4eb3, PDF_CMAP_SINGLE, 4113 },
	{ 0x4eb6, 0x4eb6, PDF_CMAP_SINGLE, 4114 },
	{ 0x4eb9, 0x4ebc, PDF_CMAP_TABLE, 890 },
	{ 0x4ec0, 0x4ec4, PDF_CMAP_TABLE, 894 },
	{ 0x4ec6, 0x4ec8, PDF_CMAP_TABLE, 899 },
	{ 0x4eca, 0x4ecb, PDF_CMAP_TABLE, 902 },
	{ 0x4ecd, 0x4ed0, PDF_CMAP_TABLE, 904 },
	{ 0x4ed4, 0x4edb, PDF_CMAP_TABLE, 908 },
	{ 0x4edd, 0x4ee5, PDF_CMAP_TABLE, 916 },
	{ 0x4ee8, 0x4ee8, PDF_CMAP_SINGLE, 21085 },
	{ 0x4eeb, 0x4eeb, PDF_CMAP_SINGLE, 17250 },
	{ 0x4eed, 0x4ef3, PDF_CMAP_TABLE, 925 },
	{ 0x4ef5, 0x4ef7, PDF_CMAP_TABLE, 932 },
	{ 0x4efb, 0x4f03, PDF_CMAP_TABLE, 935 },
	{ 0x4f08, 0x4f12, PDF_CMAP_TABLE, 944 },
	{ 0x4f15, 0x4f17, PDF_CMAP_TABLE, 955 },
	{ 0x4f19, 0x4f1a, PDF_CMAP_TABLE, 958 },
	{ 0x4f1c, 0x4f1d, PDF_CMAP_TABLE, 960 },
	{ 0x4f2b, 0x4f2b, PDF_CMAP_SINGLE, 19147 },
	{ 0x4f2e, 0x4f31, PDF_CMAP_TABLE, 962 },
	{ 0x4f33, 0x4f3e, PDF_CMAP_TABLE, 966 },
	{ 0x4f40, 0x4f40, PDF_CMAP_SINGLE, 21094 },
	{ 0x4f42, 0x4f43, PDF_CMAP_TABLE, 978 },
	{ 0x4f46, 0x4f49, PDF_CMAP_TABLE, 980 },
	{ 0x4f4b, 0x4f4c, PDF_CMAP_RANGE, 21096 },
	{ 0x4f4d, 0x4f60, PDF_CMAP_TABLE, 984 },
	{ 0x4f63, 0x4f64, PDF_CMAP_TABLE, 1004 },
	{ 0x4f69, 0x4f6a, PDF_CMAP_TABLE, 1006 },
	{ 0x4f6c, 0x4f6c, PDF_CMAP_SINGLE, 16782 },
	{ 0x4f6e, 0x4f71, PDF_CMAP_TABLE, 1008 },
	{ 0x4f73, 0x4f73, PDF_CMAP_SINGLE, 1346 },
	{ 0x4f75, 0x4f76, PDF_CMAP_TABLE, 1012 },
	{ 0x4f77, 0x4f78, PDF_CMAP_RANGE, 17260 },
	{ 0x4f79, 0x4f7c, PDF_CMAP_TABLE, 1014 },
	{ 0x4f7d, 0x4f7e, PDF_CMAP_RANGE, 14321 },
	{ 0x4f7f, 0x4f7f, PDF_CMAP_SINGLE, 2198 },
	{ 0x4f81, 0x4f86, PDF_CMAP_TABLE, 1018 },
	{ 0x4f88, 0x4f94, PDF_CMAP_TABLE, 1024 },
	{ 0x4f96, 0x4f9b, PDF_CMAP_TABLE, 1037 },
	{ 0x4f9d, 0x4fa1, PDF_CMAP_TABLE, 1043 },
	{ 0x4fab, 0x4fab, PDF_CMAP_SINGLE, 4564 },
	{ 0x4fad, 0x4faf, PDF_CMAP_TABLE, 1048 },
	{ 0x4fb2, 0x4fb2, PDF_CMAP_SINGLE, 17265 },
	{ 0x4fb5, 0x4fb7, PDF_CMAP_TABLE, 1051 },
	{ 0x4fb9, 0x4fb9, PDF_CMAP_SINGLE, 21111 },
	{ 0x4fbb, 0x4fbd, PDF_CMAP_RANGE, 21112 },
	{ 0x4fbe, 0x4fbf, PDF_CMAP_TABLE, 1054 },
	{ 0x4fc0, 0x4fc1, PDF_CMAP_RANGE, 21115 },
	{ 0x4fc2, 0x4fc6, PDF_CMAP_TABLE, 1056 },
	{ 0x4fc8, 0x4fd4, PDF_CMAP_TABLE, 1061 },
	{ 0x4fd7, 0x4fd8, PDF_CMAP_TABLE, 1074 },
	{ 0x4fda, 0x4fdd, PDF_CMAP_TABLE, 1076 },
	{ 0x4fdf, 0x4fe3, PDF_CMAP_TABLE, 1080 },
	{ 0x4fe4, 0x4fe5, PDF_CMAP_RANGE, 4152 },
	{ 0x4fe6, 0x4fe6, PDF_CMAP_SINGLE, 17264 },
	{ 0x4fee, 0x4ff3, PDF_CMAP_TABLE, 1085 },
	{ 0x4ff5, 0x4ff6, PDF_CMAP_TABLE, 1091 },
	{ 0x4ff8, 0x4ff8, PDF_CMAP_SINGLE, 3648 },
	{ 0x4ffa, 0x4ffa, PDF_CMAP_SINGLE, 1334 },
	{ 0x4ffc, 0x4fff, PDF_CMAP_TABLE, 1093 },
	{ 0x5000, 0x5001, PDF_CMAP_RANGE, 14327 },
	{ 0x5002, 0x5002, PDF_CMAP_SINGLE, 20299 },
	{ 0x5004, 0x5007, PDF_CMAP_TABLE, 1097 },
	{ 0x5009, 0x5014, PDF_CMAP_TABLE, 1101 },
	{ 0x5016, 0x501f, PDF_CMAP_TABLE, 1113 },
	{ 0x5021, 0x502e, PDF_CMAP_TABLE, 1123 },
	{ 0x5030, 0x5030, PDF_CMAP_SINGLE, 21128 },
	{ 0x5032, 0x5033, PDF_CMAP_RANGE, 21129 },
	{ 0x5035, 0x5036, PDF_CMAP_TABLE, 1137 },
	{ 0x5039, 0x5039, PDF_CMAP_SINGLE, 1862 },
	{ 0x503b, 0x503b, PDF_CMAP_SINGLE, 14336 },
	{ 0x5040, 0x5043, PDF_CMAP_TABLE, 1139 },
	{ 0x5045, 0x504a, PDF_CMAP_TABLE, 1143 },
	{ 0x504c, 0x504c, PDF_CMAP_SINGLE, 19155 },
	{ 0x504e, 0x5050, PDF_CMAP_TABLE, 1149 },
	{ 0x5051, 0x5052, PDF_CMAP_RANGE, 21134 },
	{ 0x5053, 0x5053, PDF_CMAP_SINGLE, 17274 },
	{ 0x5055, 0x5057, PDF_CMAP_TABLE, 1152 },
	{ 0x5059, 0x505a, PDF_CMAP_TABLE, 1155 },
	{ 0x505c, 0x505c, PDF_CMAP_SINGLE, 3072 },
	{ 0x505f, 0x5060, PDF_CMAP_TABLE, 1157 },
	{ 0x5062, 0x5063, PDF_CMAP_TABLE, 1159 },
	{ 0x5065, 0x5067, PDF_CMAP_TABLE, 1161 },
	{ 0x506a, 0x506a, PDF_CMAP_SINGLE, 14335 },
	{ 0x506c, 0x506d, PDF_CMAP_TABLE, 1164 },
	{ 0x5070, 0x5072, PDF_CMAP_TABLE, 1166 },
	{ 0x5074, 0x5078, PDF_CMAP_TABLE, 1169 },
	{ 0x507d, 0x507d, PDF_CMAP_SINGLE, 1616 },
	{ 0x5080, 0x5081, PDF_CMAP_TABLE, 1174 },
	{ 0x5083, 0x5084, PDF_CMAP_RANGE, 21142 },
	{ 0x5085, 0x5086, PDF_CMAP_TABLE, 1176 },
	{ 0x5088, 0x5088, PDF_CMAP_SINGLE, 17277 },
	{ 0x508a, 0x508a, PDF_CMAP_SINGLE, 21145 },
	{ 0x508d, 0x5091, PDF_CMAP_TABLE, 1178 },
	{ 0x5092, 0x5093, PDF_CMAP_RANGE, 17278 },
	{ 0x5094, 0x5096, PDF_CMAP_TABLE, 1183 },
	{ 0x5098, 0x509c, PDF_CMAP_TABLE, 1186 },
	{ 0x509e, 0x509e, PDF_CMAP_SINGLE, 19160 },
	{ 0x509f, 0x50a1, PDF_CMAP_RANGE, 21148 },
	{ 0x50a2, 0x50a3, PDF_CMAP_TABLE, 1191 },
	{ 0x50aa, 0x50aa, PDF_CMAP_SINGLE, 17281 },
	{ 0x50ac, 0x50ad, PDF_CMAP_TABLE, 1193 },
	{ 0x50af, 0x50b0, PDF_CMAP_RANGE, 21151 },
	{ 0x50b1, 0x50b5, PDF_CMAP_TABLE, 1195 },
	{ 0x50b7, 0x50b7, PDF_CMAP_SINGLE, 2439 },
	{ 0x50b9, 0x50b9, PDF_CMAP_SINGLE, 21153 },
	{ 0x50ba, 0x50bb, PDF_CMAP_RANGE, 17284 },
	{ 0x50bd, 0x50be, PDF_CMAP_TABLE, 1200 },
	{ 0x50c0, 0x50c0, PDF_CMAP_SINGLE, 21155 },
	{ 0x50c2, 0x50c5, PDF_CMAP_TABLE, 1202 },
	{ 0x50c7, 0x50c7, PDF_CMAP_SINGLE, 17287 },
	{ 0x50c9, 0x50ca, PDF_CMAP_RANGE, 4184 },
	{ 0x50cc, 0x50d1, PDF_CMAP_TABLE, 1206 },
	{ 0x50d3, 0x50d6, PDF_CMAP_TABLE, 1212 },
	{ 0x50d8, 0x50da, PDF_CMAP_TABLE, 1216 },
	{ 0x50dc, 0x50dd, PDF_CMAP_RANGE, 21157 },
	{ 0x50de, 0x50df, PDF_CMAP_TABLE, 1219 },
	{ 0x50e1, 0x50e9, PDF_CMAP_TABLE, 1221 },
	{ 0x50ed, 0x50f6, PDF_CMAP_TABLE, 1230 },
	{ 0x50f9, 0x50fb, PDF_CMAP_TABLE, 1240 },
	{ 0x50fe, 0x50fe, PDF_CMAP_SINGLE, 19165 },
	{ 0x5100, 0x5100, PDF_CMAP_SINGLE, 1617 },
	{ 0x5101, 0x5102, PDF_CMAP_RANGE, 4197 },
	{ 0x5103, 0x5104, PDF_CMAP_TABLE, 1243 },
	{ 0x5106, 0x5109, PDF_CMAP_TABLE, 1245 },
	{ 0x510b, 0x510b, PDF_CMAP_SINGLE, 14345 },
	{ 0x510c, 0x510e, PDF_CMAP_RANGE, 19167 },
	{ 0x5110, 0x5110, PDF_CMAP_SINGLE, 14346 },
	{ 0x5112, 0x5112, PDF_CMAP_SINGLE, 2336 },
	{ 0x5114, 0x511b, PDF_CMAP_TABLE, 1249 },
	{ 0x511c, 0x511d, PDF_CMAP_RANGE, 21166 },
	{ 0x511e, 0x511f, PDF_CMAP_TABLE, 1257 },
	{ 0x5121, 0x5121, PDF_CMAP_SINGLE, 4203 },
	{ 0x5123, 0x5123, PDF_CMAP_SINGLE, 21168 },
	{ 0x5127, 0x5128, PDF_CMAP_RANGE, 21169 },
	{ 0x512a, 0x512a, PDF_CMAP_SINGLE, 3855 },
	{ 0x512c, 0x512d, PDF_CMAP_RANGE, 21171 },
	{ 0x512f, 0x512f, PDF_CMAP_SINGLE, 21173 },
	{ 0x5131, 0x5135, PDF_CMAP_TABLE, 1259 },
	{ 0x5137, 0x513c, PDF_CMAP_TABLE, 1264 },
	{ 0x513f, 0x5140, PDF_CMAP_RANGE, 4208 },
	{ 0x5141, 0x5150, PDF_CMAP_TABLE, 1270 },
	{ 0x5152, 0x5155, PDF_CMAP_TABLE, 1286 },
	{ 0x5157, 0x5158, PDF_CMAP_TABLE, 1290 },
	{ 0x515a, 0x515a, PDF_CMAP_SINGLE, 3160 },
	{ 0x515c, 0x515c, PDF_CMAP_SINGLE, 1491 },
	{ 0x515f, 0x5160, PDF_CMAP_TABLE, 1292 },
	{ 0x5162, 0x5162, PDF_CMAP_SINGLE, 4213 },
	{ 0x5164, 0x5168, PDF_CMAP_TABLE, 1294 },
	{ 0x5169, 0x516a, PDF_CMAP_RANGE, 4215 },
	{ 0x516b, 0x516e, PDF_CMAP_TABLE, 1299 },
	{ 0x5171, 0x5171, PDF_CMAP_SINGLE, 1694 },
	{ 0x5173, 0x5179, PDF_CMAP_TABLE, 1303 },
	{ 0x517b, 0x517c, PDF_CMAP_TABLE, 1310 },
	{ 0x517e, 0x517e, PDF_CMAP_SINGLE, 21182 },
	{ 0x5180, 0x5180, PDF_CMAP_SINGLE, 4218 },
	{ 0x5182, 0x5186, PDF_CMAP_TABLE, 1312 },
	{ 0x5189, 0x5193, PDF_CMAP_TABLE, 1317 },
	{ 0x5195, 0x5196, PDF_CMAP_RANGE, 4226 },
	{ 0x5197, 0x5199, PDF_CMAP_TABLE, 1328 },
	{ 0x519d, 0x519d, PDF_CMAP_SINGLE, 8395 },
	{ 0x51a0, 0x51a6, PDF_CMAP_TABLE, 1331 },
	{ 0x51a8, 0x51a8, PDF_CMAP_SINGLE, 3532 },
	{ 0x51a9, 0x51ab, PDF_CMAP_RANGE, 4231 },
	{ 0x51ac, 0x51ad, PDF_CMAP_TABLE, 1338 },
	{ 0x51b0, 0x51b0, PDF_CMAP_SINGLE, 4237 },
	{ 0x51b1, 0x51b2, PDF_CMAP_RANGE, 4235 },
	{ 0x51b3, 0x51b8, PDF_CMAP_TABLE, 1340 },
	{ 0x51ba, 0x51ba, PDF_CMAP_SINGLE, 19175 },
	{ 0x51bc, 0x51bf, PDF_CMAP_TABLE, 1346 },
	{ 0x51c2, 0x51c6, PDF_CMAP_TABLE, 1350 },
	{ 0x51c8, 0x51cd, PDF_CMAP_TABLE, 1355 },
	{ 0x51cf, 0x51cf, PDF_CMAP_SINGLE, 19177 },
	{ 0x51d1, 0x51d2, PDF_CMAP_TABLE, 1361 },
	{ 0x51d3, 0x51d4, PDF_CMAP_RANGE, 19179 },
	{ 0x51d5, 0x51d6, PDF_CMAP_TABLE, 1363 },
	{ 0x51d8, 0x51d8, PDF_CMAP_SINGLE, 19181 },
	{ 0x51db, 0x51e2, PDF_CMAP_TABLE, 1365 },
	{ 0x51e5, 0x51e7, PDF_CMAP_TABLE, 1373 },
	{ 0x51e9, 0x51ea, PDF_CMAP_TABLE, 1376 },
	{ 0x51ec, 0x51ee, PDF_CMAP_TABLE, 1378 },
	{ 0x51f0, 0x51fa, PDF_CMAP_TABLE, 1381 },
	{ 0x51fd, 0x51fe, PDF_CMAP_TABLE, 1392 },
	{ 0x5200, 0x5200, PDF_CMAP_SINGLE, 3163 },
	{ 0x5201, 0x5202, PDF_CMAP_RANGE, 14355 },
	{ 0x5203, 0x5208, PDF_CMAP_TABLE, 1394 },
	{ 0x520a, 0x520b, PDF_CMAP_TABLE, 1400 },
	{ 0x520e, 0x520e, PDF_CMAP_SINGLE, 4253 },
	{ 0x5211, 0x5218, PDF_CMAP_TABLE, 1402 },
	{ 0x521d, 0x521d, PDF_CMAP_SINGLE, 2419 },
	{ 0x5222, 0x5222, PDF_CMAP_SINGLE, 21192 },
	{ 0x5224, 0x522b, PDF_CMAP_TABLE, 1410 },
	{ 0x522e, 0x522e, PDF_CMAP_SINGLE, 4256 },
	{ 0x5230, 0x5230, PDF_CMAP_SINGLE, 3192 },
	{ 0x5231, 0x5232, PDF_CMAP_RANGE, 19187 },
	{ 0x5233, 0x5233, PDF_CMAP_SINGLE, 4257 },
	{ 0x5235, 0x523c, PDF_CMAP_TABLE, 1418 },
	{ 0x5243, 0x5245, PDF_CMAP_TABLE, 1426 },
	{ 0x5247, 0x5247, PDF_CMAP_SINGLE, 2823 },
	{ 0x5249, 0x524a, PDF_CMAP_TABLE, 1429 },
	{ 0x524b, 0x524c, PDF_CMAP_RANGE, 4261 },
	{ 0x524d, 0x524d, PDF_CMAP_SINGLE, 2738 },
	{ 0x524f, 0x524f, PDF_CMAP_SINGLE, 4259 },
	{ 0x5254, 0x5258, PDF_CMAP_TABLE, 1431 },
	{ 0x525a, 0x5261, PDF_CMAP_TABLE, 1436 },
	{ 0x5263, 0x5266, PDF_CMAP_TABLE, 1444 },
	{ 0x5269, 0x526a, PDF_CMAP_TABLE, 1448 },
	{ 0x526c, 0x526c, PDF_CMAP_SINGLE, 17315 },
	{ 0x526e, 0x5275, PDF_CMAP_TABLE, 1450 },
	{ 0x5277, 0x5277, PDF_CMAP_SINGLE, 17316 },
	{ 0x5278, 0x5279, PDF_CMAP_RANGE, 19194 },
	{ 0x527d, 0x527d, PDF_CMAP_SINGLE, 4270 },
	{ 0x527f, 0x5280, PDF_CMAP_TABLE, 1458 },
	{ 0x5282, 0x5285, PDF_CMAP_TABLE, 1460 },
	{ 0x5287, 0x528a, PDF_CMAP_TABLE, 1464 },
	{ 0x528c, 0x528d, PDF_CMAP_TABLE, 1468 },
	{ 0x5291, 0x5294, PDF_CMAP_TABLE, 1470 },
	{ 0x5295, 0x5297, PDF_CMAP_RANGE, 21198 },
	{ 0x5298, 0x5298, PDF_CMAP_SINGLE, 17320 },
	{ 0x529a, 0x529c, PDF_CMAP_TABLE, 1474 },
	{ 0x529f, 0x52a0, PDF_CMAP_TABLE, 1477 },
	{ 0x52a3, 0x52a7, PDF_CMAP_TABLE, 1479 },
	{ 0x52a9, 0x52ab, PDF_CMAP_TABLE, 1484 },
	{ 0x52ac, 0x52ad, PDF_CMAP_RANGE, 4279 },
	{ 0x52af, 0x52b1, PDF_CMAP_TABLE, 1487 },
	{ 0x52b4, 0x52b5, PDF_CMAP_TABLE, 1490 },
	{ 0x52b6, 0x52b8, PDF_CMAP_RANGE, 21205 },
	{ 0x52b9, 0x52b9, PDF_CMAP_SINGLE, 1966 },
	{ 0x52ba, 0x52bb, PDF_CMAP_RANGE, 17323 },
	{ 0x52bc, 0x52be, PDF_CMAP_TABLE, 1492 },
	{ 0x52c0, 0x52c1, PDF_CMAP_TABLE, 1495 },
	{ 0x52c3, 0x52ca, PDF_CMAP_TABLE, 1497 },
	{ 0x52cc, 0x52cd, PDF_CMAP_TABLE, 1505 },
	{ 0x52cf, 0x52d2, PDF_CMAP_TABLE, 1507 },
	{ 0x52d4, 0x52d9, PDF_CMAP_TABLE, 1511 },
	{ 0x52db, 0x52ea, PDF_CMAP_TABLE, 1517 },
	{ 0x52ec, 0x52ec, PDF_CMAP_SINGLE, 21217 },
	{ 0x52f0, 0x52f7, PDF_CMAP_TABLE, 1533 },
	{ 0x52f8, 0x52f9, PDF_CMAP_RANGE, 4293 },
	{ 0x52fa, 0x52fb, PDF_CMAP_TABLE, 1541 },
	{ 0x52fe, 0x5303, PDF_CMAP_TABLE, 1543 },
	{ 0x5305, 0x5308, PDF_CMAP_TABLE, 1549 },
	{ 0x530a, 0x530b, PDF_CMAP_RANGE, 14364 },
	{ 0x530c, 0x530d, PDF_CMAP_TABLE, 1553 },
	{ 0x530f, 0x5311, PDF_CMAP_TABLE, 1555 },
	{ 0x5313, 0x5313, PDF_CMAP_SINGLE, 21221 },
	{ 0x5315, 0x5321, PDF_CMAP_TABLE, 1558 },
	{ 0x5323, 0x5325, PDF_CMAP_TABLE, 1571 },
	{ 0x5327, 0x5329, PDF_CMAP_RANGE, 21226 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 3439 },
	{ 0x532b, 0x532c, PDF_CMAP_RANGE, 21229 },
	{ 0x532d, 0x532d, PDF_CMAP_SINGLE, 19204 },
	{ 0x532f, 0x5333, PDF_CMAP_TABLE, 1574 },
	{ 0x5335, 0x5335, PDF_CMAP_SINGLE, 17329 },
	{ 0x5338, 0x5343, PDF_CMAP_TABLE, 1579 },
	{ 0x5345, 0x534d, PDF_CMAP_TABLE, 1591 },
	{ 0x5351, 0x5354, PDF_CMAP_TABLE, 1600 },
	{ 0x5357, 0x535c, PDF_CMAP_TABLE, 1604 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 4315 },
	{ 0x5360, 0x5361, PDF_CMAP_TABLE, 1610 },
	{ 0x5363, 0x5363, PDF_CMAP_SINGLE, 16801 },
	{ 0x5365, 0x5367, PDF_CMAP_TABLE, 1612 },
	{ 0x5369, 0x5369, PDF_CMAP_SINGLE, 4316 },
	{ 0x536c, 0x5375, PDF_CMAP_TABLE, 1615 },
	{ 0x5377, 0x537b, PDF_CMAP_TABLE, 1625 },
	{ 0x537d, 0x537f, PDF_CMAP_TABLE, 1630 },
	{ 0x5382, 0x5384, PDF_CMAP_TABLE, 1633 },
	{ 0x5387, 0x5388, PDF_CMAP_RANGE, 21237 },
	{ 0x5389, 0x5389, PDF_CMAP_SINGLE, 14288 },
	{ 0x538e, 0x538e, PDF_CMAP_SINGLE, 21239 },
	{ 0x5393, 0x5394, PDF_CMAP_TABLE, 1636 },
	{ 0x5396, 0x5396, PDF_CMAP_SINGLE, 4322 },
	{ 0x5398, 0x539a, PDF_CMAP_TABLE, 1638 },
	{ 0x539d, 0x539d, PDF_CMAP_SINGLE, 16802 },
	{ 0x539f, 0x53a1, PDF_CMAP_TABLE, 1641 },
	{ 0x53a4, 0x53a6, PDF_CMAP_TABLE, 1644 },
	{ 0x53a8, 0x53ab, PDF_CMAP_TABLE, 1647 },
	{ 0x53ad, 0x53b0, PDF_CMAP_TABLE, 1651 },
	{ 0x53b2, 0x53b8, PDF_CMAP_TABLE, 1655 },
	{ 0x53ba, 0x53bb, PDF_CMAP_TABLE, 1662 },
	{ 0x53bd, 0x53bd, PDF_CMAP_SINGLE, 21243 },
	{ 0x53c0, 0x53c3, PDF_CMAP_TABLE, 1664 },
	{ 0x53c4, 0x53c5, PDF_CMAP_RANGE, 19217 },
	{ 0x53c8, 0x53cf, PDF_CMAP_TABLE, 1668 },
	{ 0x53d2, 0x53d3, PDF_CMAP_RANGE, 21245 },
	{ 0x53d4, 0x53d7, PDF_CMAP_TABLE, 1676 },
	{ 0x53d9, 0x53db, PDF_CMAP_TABLE, 1680 },
	{ 0x53dd, 0x53f8, PDF_CMAP_TABLE, 1683 },
	{ 0x53fa, 0x53fa, PDF_CMAP_SINGLE, 4338 },
	{ 0x5401, 0x5404, PDF_CMAP_TABLE, 1711 },
	{ 0x5408, 0x5413, PDF_CMAP_TABLE, 1715 },
	{ 0x541a, 0x541b, PDF_CMAP_TABLE, 1727 },
	{ 0x541d, 0x5421, PDF_CMAP_TABLE, 1729 },
	{ 0x5424, 0x5424, PDF_CMAP_SINGLE, 17346 },
	{ 0x5426, 0x542b, PDF_CMAP_TABLE, 1734 },
	{ 0x542c, 0x542d, PDF_CMAP_RANGE, 4342 },
	{ 0x542e, 0x542f, PDF_CMAP_TABLE, 1740 },
	{ 0x5431, 0x5431, PDF_CMAP_SINGLE, 19222 },
	{ 0x5433, 0x5433, PDF_CMAP_SINGLE, 13760 },
	{ 0x5434, 0x5435, PDF_CMAP_RANGE, 19223 },
	{ 0x5436, 0x5436, PDF_CMAP_SINGLE, 4346 },
	{ 0x5438, 0x5439, PDF_CMAP_TABLE, 1742 },
	{ 0x543b, 0x5440, PDF_CMAP_TABLE, 1744 },
	{ 0x5442, 0x5444, PDF_CMAP_TABLE, 1750 },
	{ 0x5446, 0x544a, PDF_CMAP_TABLE, 1753 },
	{ 0x544c, 0x544f, PDF_CMAP_TABLE, 1758 },
	{ 0x5451, 0x5451, PDF_CMAP_SINGLE, 3253 },
	{ 0x5455, 0x5455, PDF_CMAP_SINGLE, 14115 },
	{ 0x545e, 0x545f, PDF_CMAP_TABLE, 1762 },
	{ 0x5462, 0x5462, PDF_CMAP_SINGLE, 17350 },
	{ 0x5464, 0x5464, PDF_CMAP_SINGLE, 21258 },
	{ 0x5466, 0x546c, PDF_CMAP_TABLE, 1764 },
	{ 0x546d, 0x546e, PDF_CMAP_RANGE, 21261 },
	{ 0x5470, 0x5471, PDF_CMAP_TABLE, 1771 },
	{ 0x5473, 0x5477, PDF_CMAP_TABLE, 1773 },
	{ 0x547b, 0x547d, PDF_CMAP_TABLE, 1778 },
	{ 0x547f, 0x5481, PDF_CMAP_TABLE, 1781 },
	{ 0x5483, 0x5486, PDF_CMAP_TABLE, 1784 },
	{ 0x5488, 0x5492, PDF_CMAP_TABLE, 1788 },
	{ 0x5495, 0x5496, PDF_CMAP_TABLE, 1799 },
	{ 0x549c, 0x549c, PDF_CMAP_SINGLE, 8411 },
	{ 0x549f, 0x54a2, PDF_CMAP_TABLE, 1801 },
	{ 0x54a4, 0x54af, PDF_CMAP_TABLE, 1805 },
	{ 0x54b1, 0x54b3, PDF_CMAP_TABLE, 1817 },
	{ 0x54b7, 0x54c4, PDF_CMAP_TABLE, 1820 },
	{ 0x54c6, 0x54ca, PDF_CMAP_TABLE, 1834 },
	{ 0x54cd, 0x54ce, PDF_CMAP_TABLE, 1839 },
	{ 0x54d8, 0x54d8, PDF_CMAP_SINGLE, 4377 },
	{ 0x54e0, 0x54e2, PDF_CMAP_TABLE, 1841 },
	{ 0x54e5, 0x54e6, PDF_CMAP_RANGE, 4378 },
	{ 0x54e8, 0x54ea, PDF_CMAP_TABLE, 1844 },
	{ 0x54ec, 0x54ef, PDF_CMAP_TABLE, 1847 },
	{ 0x54f1, 0x54f3, PDF_CMAP_TABLE, 1851 },
	{ 0x54f6, 0x54f6, PDF_CMAP_SINGLE, 21271 },
	{ 0x54fa, 0x54fa, PDF_CMAP_SINGLE, 4385 },
	{ 0x54fc, 0x54ff, PDF_CMAP_TABLE, 1854 },
	{ 0x5500, 0x5501, PDF_CMAP_RANGE, 17364 },
	{ 0x5504, 0x5509, PDF_CMAP_TABLE, 1858 },
	{ 0x550c, 0x550d, PDF_CMAP_RANGE, 21273 },
	{ 0x550e, 0x5510, PDF_CMAP_TABLE, 1864 },
	{ 0x5514, 0x5516, PDF_CMAP_TABLE, 1867 },
	{ 0x5527, 0x5527, PDF_CMAP_SINGLE, 19236 },
	{ 0x552a, 0x552b, PDF_CMAP_TABLE, 1870 },
	{ 0x552e, 0x552f, PDF_CMAP_TABLE, 1872 },
	{ 0x5531, 0x5533, PDF_CMAP_TABLE, 1874 },
	{ 0x5535, 0x5536, PDF_CMAP_TABLE, 1877 },
	{ 0x5538, 0x5539, PDF_CMAP_TABLE, 1879 },
	{ 0x553b, 0x553e, PDF_CMAP_TABLE, 1881 },
	{ 0x5540, 0x5541, PDF_CMAP_TABLE, 1885 },
	{ 0x5544, 0x5547, PDF_CMAP_TABLE, 1887 },
	{ 0x5549, 0x554a, PDF_CMAP_TABLE, 1891 },
	{ 0x554c, 0x554d, PDF_CMAP_TABLE, 1893 },
	{ 0x554f, 0x5551, PDF_CMAP_TABLE, 1895 },
	{ 0x5553, 0x5553, PDF_CMAP_SINGLE, 1810 },
	{ 0x5556, 0x5557, PDF_CMAP_RANGE, 4394 },
	{ 0x5558, 0x5558, PDF_CMAP_SINGLE, 21280 },
	{ 0x555a, 0x555b, PDF_CMAP_RANGE, 21281 },
	{ 0x555c, 0x555e, PDF_CMAP_TABLE, 1898 },
	{ 0x5560, 0x5561, PDF_CMAP_TABLE, 1901 },
	{ 0x5563, 0x5564, PDF_CMAP_TABLE, 1903 },
	{ 0x5566, 0x5566, PDF_CMAP_SINGLE, 19240 },
	{ 0x557b, 0x5584, PDF_CMAP_TABLE, 1905 },
	{ 0x5586, 0x558b, PDF_CMAP_TABLE, 1915 },
	{ 0x558e, 0x558f, PDF_CMAP_TABLE, 1921 },
	{ 0x5591, 0x5594, PDF_CMAP_TABLE, 1923 },
	{ 0x5597, 0x559a, PDF_CMAP_TABLE, 1927 },
	{ 0x559c, 0x559f, PDF_CMAP_TABLE, 1931 },
	{ 0x55a3, 0x55a4, PDF_CMAP_TABLE, 1935 },
	{ 0x55a7, 0x55ae, PDF_CMAP_TABLE, 1937 },
	{ 0x55b0, 0x55b0, PDF_CMAP_SINGLE, 1772 },
	{ 0x55b2, 0x55b2, PDF_CMAP_SINGLE, 19245 },
	{ 0x55b6, 0x55b6, PDF_CMAP_SINGLE, 1254 },
	{ 0x55bf, 0x55bf, PDF_CMAP_SINGLE, 17381 },
	{ 0x55c1, 0x55c1, PDF_CMAP_SINGLE, 21287 },
	{ 0x55c3, 0x55c7, PDF_CMAP_TABLE, 1945 },
	{ 0x55c9, 0x55c9, PDF_CMAP_SINGLE, 17382 },
	{ 0x55cb, 0x55cc, PDF_CMAP_TABLE, 1950 },
	{ 0x55ce, 0x55ce, PDF_CMAP_SINGLE, 16809 },
	{ 0x55d1, 0x55d4, PDF_CMAP_TABLE, 1952 },
	{ 0x55d7, 0x55d8, PDF_CMAP_RANGE, 21289 },
	{ 0x55da, 0x55df, PDF_CMAP_TABLE, 1956 },
	{ 0x55e2, 0x55e4, PDF_CMAP_TABLE, 1962 },
	{ 0x55e9, 0x55e9, PDF_CMAP_SINGLE, 17389 },
	{ 0x55ec, 0x55ec, PDF_CMAP_SINGLE, 19250 },
	{ 0x55ee, 0x55ee, PDF_CMAP_SINGLE, 19251 },
	{ 0x55f1, 0x55f1, PDF_CMAP_SINGLE, 19252 },
	{ 0x55f6, 0x55f9, PDF_CMAP_TABLE, 1965 },
	{ 0x55fd, 0x55ff, PDF_CMAP_TABLE, 1969 },
	{ 0x5605, 0x560a, PDF_CMAP_TABLE, 1972 },
	{ 0x560d, 0x560d, PDF_CMAP_SINGLE, 19256 },
	{ 0x560e, 0x560f, PDF_CMAP_RANGE, 14397 },
	{ 0x5610, 0x5610, PDF_CMAP_SINGLE, 17393 },
	{ 0x5611, 0x5612, PDF_CMAP_RANGE, 19257 },
	{ 0x5614, 0x5614, PDF_CMAP_SINGLE, 4421 },
	{ 0x5616, 0x5619, PDF_CMAP_TABLE, 1978 },
	{ 0x561b, 0x561b, PDF_CMAP_SINGLE, 4426 },
	{ 0x5620, 0x5620, PDF_CMAP_SINGLE, 15389 },
	{ 0x5628, 0x5629, PDF_CMAP_TABLE, 1982 },
	{ 0x562c, 0x562c, PDF_CMAP_SINGLE, 19259 },
	{ 0x562f, 0x5639, PDF_CMAP_TABLE, 1984 },
	{ 0x563b, 0x563d, PDF_CMAP_TABLE, 1995 },
	{ 0x563f, 0x5642, PDF_CMAP_TABLE, 1998 },
	{ 0x5643, 0x5644, PDF_CMAP_RANGE, 21299 },
	{ 0x5646, 0x5647, PDF_CMAP_TABLE, 2002 },
	{ 0x5649, 0x5649, PDF_CMAP_SINGLE, 14401 },
	{ 0x564b, 0x5650, PDF_CMAP_TABLE, 2004 },
	{ 0x5653, 0x5654, PDF_CMAP_TABLE, 2010 },
	{ 0x565b, 0x565b, PDF_CMAP_SINGLE, 1496 },
	{ 0x565e, 0x565e, PDF_CMAP_SINGLE, 17399 },
	{ 0x5660, 0x5660, PDF_CMAP_SINGLE, 17400 },
	{ 0x5661, 0x5663, PDF_CMAP_RANGE, 21302 },
	{ 0x5664, 0x5664, PDF_CMAP_SINGLE, 4436 },
	{ 0x5666, 0x5666, PDF_CMAP_SINGLE, 14404 },
	{ 0x5668, 0x566d, PDF_CMAP_TABLE, 2012 },
	{ 0x566f, 0x566f, PDF_CMAP_SINGLE, 14406 },
	{ 0x5671, 0x5672, PDF_CMAP_RANGE, 14407 },
	{ 0x5674, 0x5676, PDF_CMAP_TABLE, 2018 },
	{ 0x5678, 0x5678, PDF_CMAP_SINGLE, 3245 },
	{ 0x567a, 0x567a, PDF_CMAP_SINGLE, 3404 },
	{ 0x5680, 0x5680, PDF_CMAP_SINGLE, 4441 },
	{ 0x5684, 0x5688, PDF_CMAP_TABLE, 2021 },
	{ 0x568a, 0x568c, PDF_CMAP_TABLE, 2026 },
	{ 0x568f, 0x568f, PDF_CMAP_SINGLE, 4445 },
	{ 0x5694, 0x5695, PDF_CMAP_TABLE, 2029 },
	{ 0x5699, 0x569a, PDF_CMAP_TABLE, 2031 },
	{ 0x569d, 0x56a0, PDF_CMAP_TABLE, 2033 },
	{ 0x56a2, 0x56a2, PDF_CMAP_SINGLE, 3311 },
	{ 0x56a5, 0x56a9, PDF_CMAP_TABLE, 2037 },
	{ 0x56ab, 0x56ab, PDF_CMAP_SINGLE, 21309 },
	{ 0x56ac, 0x56ad, PDF_CMAP_RANGE, 14411 },
	{ 0x56ae, 0x56ae, PDF_CMAP_SINGLE, 4447 },
	{ 0x56b1, 0x56b4, PDF_CMAP_TABLE, 2042 },
	{ 0x56b6, 0x56b7, PDF_CMAP_TABLE, 2046 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 4451 },
	{ 0x56be, 0x56be, PDF_CMAP_SINGLE, 21310 },
	{ 0x56c0, 0x56c3, PDF_CMAP_TABLE, 2048 },
	{ 0x56c5, 0x56c5, PDF_CMAP_SINGLE, 17408 },
	{ 0x56c8, 0x56d1, PDF_CMAP_TABLE, 2052 },
	{ 0x56d3, 0x56d3, PDF_CMAP_SINGLE, 4458 },
	{ 0x56d7, 0x56e1, PDF_CMAP_TABLE, 2062 },
	{ 0x56e3, 0x56e4, PDF_CMAP_TABLE, 2073 },
	{ 0x56e5, 0x56e7, PDF_CMAP_RANGE, 21314 },
	{ 0x56e8, 0x56e8, PDF_CMAP_SINGLE, 17411 },
	{ 0x56eb, 0x56eb, PDF_CMAP_SINGLE, 19272 },
	{ 0x56ed, 0x56ee, PDF_CMAP_TABLE, 2075 },
	{ 0x56f0, 0x56f3, PDF_CMAP_TABLE, 2077 },
	{ 0x56f6, 0x56f7, PDF_CMAP_RANGE, 17412 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 2081 },
	{ 0x56fd, 0x56fd, PDF_CMAP_SINGLE, 2051 },
	{ 0x56ff, 0x5700, PDF_CMAP_TABLE, 2083 },
	{ 0x5701, 0x5702, PDF_CMAP_RANGE, 21317 },
	{ 0x5703, 0x5704, PDF_CMAP_TABLE, 2085 },
	{ 0x5707, 0x570d, PDF_CMAP_TABLE, 2087 },
	{ 0x570f, 0x570f, PDF_CMAP_SINGLE, 1869 },
	{ 0x5711, 0x5713, PDF_CMAP_TABLE, 2094 },
	{ 0x5715, 0x5716, PDF_CMAP_TABLE, 2097 },
	{ 0x5718, 0x5718, PDF_CMAP_SINGLE, 4470 },
	{ 0x571a, 0x571b, PDF_CMAP_RANGE, 19277 },
	{ 0x571c, 0x571d, PDF_CMAP_TABLE, 2099 },
	{ 0x571f, 0x5723, PDF_CMAP_TABLE, 2101 },
	{ 0x5724, 0x5725, PDF_CMAP_RANGE, 21322 },
	{ 0x5726, 0x572a, PDF_CMAP_TABLE, 2106 },
	{ 0x572c, 0x5730, PDF_CMAP_TABLE, 2111 },
	{ 0x5733, 0x5734, PDF_CMAP_RANGE, 14421 },
	{ 0x5737, 0x5738, PDF_CMAP_RANGE, 4475 },
	{ 0x573b, 0x573b, PDF_CMAP_SINGLE, 4478 },
	{ 0x573d, 0x573e, PDF_CMAP_RANGE, 19282 },
	{ 0x573f, 0x5740, PDF_CMAP_TABLE, 2116 },
	{ 0x5742, 0x5742, PDF_CMAP_SINGLE, 2132 },
	{ 0x5745, 0x5746, PDF_CMAP_RANGE, 17418 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 1737 },
	{ 0x574a, 0x574a, PDF_CMAP_SINGLE, 3685 },
	{ 0x574c, 0x5752, PDF_CMAP_TABLE, 2118 },
	{ 0x5759, 0x5759, PDF_CMAP_SINGLE, 8416 },
	{ 0x575f, 0x575f, PDF_CMAP_SINGLE, 19284 },
	{ 0x5761, 0x5762, PDF_CMAP_TABLE, 2125 },
	{ 0x5764, 0x576b, PDF_CMAP_TABLE, 2127 },
	{ 0x576d, 0x5771, PDF_CMAP_TABLE, 2135 },
	{ 0x5773, 0x5775, PDF_CMAP_RANGE, 17424 },
	{ 0x5777, 0x5777, PDF_CMAP_SINGLE, 16814 },
	{ 0x5779, 0x577c, PDF_CMAP_TABLE, 2140 },
	{ 0x577e, 0x577f, PDF_CMAP_TABLE, 2144 },
	{ 0x5781, 0x5783, PDF_CMAP_TABLE, 2146 },
	{ 0x5788, 0x5789, PDF_CMAP_TABLE, 2149 },
	{ 0x578b, 0x578c, PDF_CMAP_TABLE, 2151 },
	{ 0x5793, 0x5795, PDF_CMAP_TABLE, 2153 },
	{ 0x5797, 0x5797, PDF_CMAP_SINGLE, 19289 },
	{ 0x5799, 0x579a, PDF_CMAP_TABLE, 2156 },
	{ 0x579c, 0x579c, PDF_CMAP_SINGLE, 14426 },
	{ 0x579d, 0x579e, PDF_CMAP_RANGE, 17431 },
	{ 0x579f, 0x57a4, PDF_CMAP_TABLE, 2158 },
	{ 0x57a7, 0x57aa, PDF_CMAP_TABLE, 2164 },
	{ 0x57ac, 0x57ac, PDF_CMAP_SINGLE, 8418 },
	{ 0x57ae, 0x57ae, PDF_CMAP_SINGLE, 19290 },
	{ 0x57b0, 0x57b0, PDF_CMAP_SINGLE, 4492 },
	{ 0x57b3, 0x57b3, PDF_CMAP_SINGLE, 4489 },
	{ 0x57b8, 0x57b8, PDF_CMAP_SINGLE, 14427 },
	{ 0x57bd, 0x57bd, PDF_CMAP_SINGLE, 21341 },
	{ 0x57c0, 0x57c0, PDF_CMAP_SINGLE, 4482 },
	{ 0x57c3, 0x57c3, PDF_CMAP_SINGLE, 4493 },
	{ 0x57c6, 0x57c8, PDF_CMAP_TABLE, 2168 },
	{ 0x57cb, 0x57cc, PDF_CMAP_TABLE, 2171 },
	{ 0x57ce, 0x57cf, PDF_CMAP_TABLE, 2173 },
	{ 0x57d2, 0x57d3, PDF_CMAP_RANGE, 4496 },
	{ 0x57d4, 0x57d7, PDF_CMAP_TABLE, 2175 },
	{ 0x57dc, 0x57e1, PDF_CMAP_TABLE, 2179 },
	{ 0x57e3, 0x57e4, PDF_CMAP_TABLE, 2185 },
	{ 0x57e6, 0x57e7, PDF_CMAP_TABLE, 2187 },
	{ 0x57e9, 0x57e9, PDF_CMAP_SINGLE, 21343 },
	{ 0x57ed, 0x57ed, PDF_CMAP_SINGLE, 14429 },
	{ 0x57f0, 0x57f0, PDF_CMAP_SINGLE, 17440 },
	{ 0x57f4, 0x57f4, PDF_CMAP_SINGLE, 2533 },
	{ 0x57f5, 0x57f6, PDF_CMAP_RANGE, 14430 },
	{ 0x57f7, 0x5800, PDF_CMAP_TABLE, 2189 },
	{ 0x5802, 0x5806, PDF_CMAP_TABLE, 2199 },
	{ 0x5808, 0x580d, PDF_CMAP_TABLE, 2204 },
	{ 0x5815, 0x5815, PDF_CMAP_SINGLE, 2852 },
	{ 0x5819, 0x5819, PDF_CMAP_SINGLE, 4502 },
	{ 0x581b, 0x581b, PDF_CMAP_SINGLE, 21349 },
	{ 0x581d, 0x5821, PDF_CMAP_TABLE, 2210 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 3077 },
	{ 0x5826, 0x5827, PDF_CMAP_TABLE, 2215 },
	{ 0x582a, 0x582a, PDF_CMAP_SINGLE, 1514 },
	{ 0x582d, 0x582d, PDF_CMAP_SINGLE, 21351 },
	{ 0x582f, 0x5832, PDF_CMAP_TABLE, 2217 },
	{ 0x5834, 0x5835, PDF_CMAP_TABLE, 2221 },
	{ 0x5839, 0x583a, PDF_CMAP_TABLE, 2223 },
	{ 0x583d, 0x583d, PDF_CMAP_SINGLE, 4511 },
	{ 0x583f, 0x5841, PDF_CMAP_TABLE, 2225 },
	{ 0x5849, 0x584d, PDF_CMAP_TABLE, 2228 },
	{ 0x584f, 0x5852, PDF_CMAP_TABLE, 2233 },
	{ 0x5854, 0x5855, PDF_CMAP_TABLE, 2237 },
	{ 0x5857, 0x585a, PDF_CMAP_TABLE, 2239 },
	{ 0x585e, 0x585f, PDF_CMAP_TABLE, 2243 },
	{ 0x5861, 0x5862, PDF_CMAP_TABLE, 2245 },
	{ 0x5864, 0x5864, PDF_CMAP_SINGLE, 16819 },
	{ 0x5867, 0x5869, PDF_CMAP_TABLE, 2247 },
	{ 0x586b, 0x586b, PDF_CMAP_SINGLE, 3120 },
	{ 0x586d, 0x586d, PDF_CMAP_SINGLE, 19298 },
	{ 0x5870, 0x5870, PDF_CMAP_SINGLE, 4508 },
	{ 0x5872, 0x5872, PDF_CMAP_SINGLE, 4504 },
	{ 0x5875, 0x5875, PDF_CMAP_SINGLE, 2582 },
	{ 0x5878, 0x5879, PDF_CMAP_TABLE, 2250 },
	{ 0x587c, 0x587c, PDF_CMAP_SINGLE, 14436 },
	{ 0x587e, 0x5881, PDF_CMAP_TABLE, 2252 },
	{ 0x5883, 0x5883, PDF_CMAP_SINGLE, 1701 },
	{ 0x5885, 0x5885, PDF_CMAP_SINGLE, 4513 },
	{ 0x5887, 0x5888, PDF_CMAP_RANGE, 21357 },
	{ 0x5889, 0x5889, PDF_CMAP_SINGLE, 16820 },
	{ 0x588a, 0x588b, PDF_CMAP_RANGE, 17453 },
	{ 0x588c, 0x588d, PDF_CMAP_TABLE, 2256 },
	{ 0x588f, 0x5890, PDF_CMAP_RANGE, 17456 },
	{ 0x5893, 0x5894, PDF_CMAP_TABLE, 2258 },
	{ 0x5896, 0x5898, PDF_CMAP_TABLE, 2260 },
	{ 0x589c, 0x589f, PDF_CMAP_TABLE, 2263 },
	{ 0x58a0, 0x58a2, PDF_CMAP_RANGE, 21361 },
	{ 0x58a6, 0x58a6, PDF_CMAP_SINGLE, 21364 },
	{ 0x58a8, 0x58ab, PDF_CMAP_TABLE, 2267 },
	{ 0x58ae, 0x58ae, PDF_CMAP_SINGLE, 4521 },
	{ 0x58b1, 0x58b3, PDF_CMAP_TABLE, 2271 },
	{ 0x58b8, 0x58bc, PDF_CMAP_TABLE, 2274 },
	{ 0x58be, 0x58be, PDF_CMAP_SINGLE, 2070 },
	{ 0x58c1, 0x58c5, PDF_CMAP_TABLE, 2279 },
	{ 0x58c7, 0x58c8, PDF_CMAP_TABLE, 2284 },
	{ 0x58ca, 0x58ca, PDF_CMAP_SINGLE, 1397 },
	{ 0x58cc, 0x58ce, PDF_CMAP_TABLE, 2286 },
	{ 0x58d0, 0x58da, PDF_CMAP_TABLE, 2289 },
	{ 0x58dc, 0x58e2, PDF_CMAP_TABLE, 2300 },
	{ 0x58e4, 0x58e5, PDF_CMAP_TABLE, 2307 },
	{ 0x58e9, 0x58e9, PDF_CMAP_SINGLE, 14443 },
	{ 0x58eb, 0x58ec, PDF_CMAP_TABLE, 2309 },
	{ 0x58ee, 0x58f2, PDF_CMAP_TABLE, 2311 },
	{ 0x58f3, 0x58f4, PDF_CMAP_RANGE, 17466 },
	{ 0x58f7, 0x58f7, PDF_CMAP_SINGLE, 3063 },
	{ 0x58f9, 0x58fa, PDF_CMAP_TABLE, 2316 },
	{ 0x58fb, 0x58fd, PDF_CMAP_RANGE, 4535 },
	{ 0x5902, 0x5902, PDF_CMAP_SINGLE, 4538 },
	{ 0x5905, 0x5906, PDF_CMAP_RANGE, 17468 },
	{ 0x5909, 0x590d, PDF_CMAP_TABLE, 2318 },
	{ 0x590f, 0x5910, PDF_CMAP_TABLE, 2323 },
	{ 0x5912, 0x5913, PDF_CMAP_RANGE, 21372 },
	{ 0x5914, 0x5916, PDF_CMAP_TABLE, 2325 },
	{ 0x5918, 0x591d, PDF_CMAP_TABLE, 2328 },
	{ 0x591f, 0x591f, PDF_CMAP_SINGLE, 19303 },
	{ 0x5921, 0x5925, PDF_CMAP_TABLE, 2334 },
	{ 0x5927, 0x592b, PDF_CMAP_TABLE, 2339 },
	{ 0x592c, 0x592d, PDF_CMAP_RANGE, 4544 },
	{ 0x592e, 0x5933, PDF_CMAP_TABLE, 2344 },
	{ 0x5935, 0x5936, PDF_CMAP_RANGE, 21379 },
	{ 0x5937, 0x5939, PDF_CMAP_TABLE, 2350 },
	{ 0x593d, 0x593f, PDF_CMAP_TABLE, 2353 },
	{ 0x5943, 0x5944, PDF_CMAP_TABLE, 2356 },
	{ 0x5946, 0x5949, PDF_CMAP_TABLE, 2358 },
	{ 0x594e, 0x5955, PDF_CMAP_TABLE, 2362 },
	{ 0x5957, 0x595b, PDF_CMAP_TABLE, 2370 },
	{ 0x595d, 0x5963, PDF_CMAP_TABLE, 2375 },
	{ 0x5965, 0x5965, PDF_CMAP_SINGLE, 1310 },
	{ 0x5967, 0x596f, PDF_CMAP_TABLE, 2382 },
	{ 0x5972, 0x5974, PDF_CMAP_TABLE, 2391 },
	{ 0x5975, 0x5976, PDF_CMAP_RANGE, 17481 },
	{ 0x5978, 0x5979, PDF_CMAP_TABLE, 2394 },
	{ 0x597b, 0x597d, PDF_CMAP_TABLE, 2396 },
	{ 0x5981, 0x5984, PDF_CMAP_TABLE, 2399 },
	{ 0x598a, 0x598e, PDF_CMAP_TABLE, 2403 },
	{ 0x5992, 0x5993, PDF_CMAP_TABLE, 2408 },
	{ 0x5995, 0x5997, PDF_CMAP_TABLE, 2410 },
	{ 0x5999, 0x5999, PDF_CMAP_SINGLE, 3771 },
	{ 0x599b, 0x599b, PDF_CMAP_SINGLE, 4665 },
	{ 0x599d, 0x599d, PDF_CMAP_SINGLE, 4562 },
	{ 0x599f, 0x599f, PDF_CMAP_SINGLE, 17484 },
	{ 0x59a3, 0x59a5, PDF_CMAP_TABLE, 2413 },
	{ 0x59a7, 0x59a8, PDF_CMAP_TABLE, 2416 },
	{ 0x59ac, 0x59b0, PDF_CMAP_TABLE, 2418 },
	{ 0x59b2, 0x59b3, PDF_CMAP_TABLE, 2423 },
	{ 0x59b7, 0x59b7, PDF_CMAP_SINGLE, 21395 },
	{ 0x59b9, 0x59bc, PDF_CMAP_TABLE, 2425 },
	{ 0x59be, 0x59be, PDF_CMAP_SINGLE, 2450 },
	{ 0x59c1, 0x59c1, PDF_CMAP_SINGLE, 21396 },
	{ 0x59c3, 0x59c4, PDF_CMAP_TABLE, 2429 },
	{ 0x59c6, 0x59c6, PDF_CMAP_SINGLE, 4567 },
	{ 0x59c8, 0x59cb, PDF_CMAP_TABLE, 2431 },
	{ 0x59cd, 0x59cd, PDF_CMAP_SINGLE, 17488 },
	{ 0x59d0, 0x59d4, PDF_CMAP_TABLE, 2435 },
	{ 0x59d9, 0x59da, PDF_CMAP_RANGE, 4571 },
	{ 0x59dc, 0x59df, PDF_CMAP_TABLE, 2440 },
	{ 0x59e3, 0x59e4, PDF_CMAP_RANGE, 14452 },
	{ 0x59e5, 0x59e8, PDF_CMAP_TABLE, 2444 },
	{ 0x59ea, 0x59ec, PDF_CMAP_TABLE, 2448 },
	{ 0x59ee, 0x59ef, PDF_CMAP_TABLE, 2451 },
	{ 0x59f1, 0x59f2, PDF_CMAP_TABLE, 2453 },
	{ 0x59f4, 0x59f4, PDF_CMAP_SINGLE, 21400 },
	{ 0x59f6, 0x59f8, PDF_CMAP_TABLE, 2455 },
	{ 0x59fb, 0x59fb, PDF_CMAP_SINGLE, 1213 },
	{ 0x59ff, 0x5a01, PDF_CMAP_TABLE, 2458 },
	{ 0x5a03, 0x5a04, PDF_CMAP_TABLE, 2461 },
	{ 0x5a09, 0x5a09, PDF_CMAP_SINGLE, 4577 },
	{ 0x5a0c, 0x5a0e, PDF_CMAP_TABLE, 2463 },
	{ 0x5a11, 0x5a13, PDF_CMAP_TABLE, 2466 },
	{ 0x5a17, 0x5a18, PDF_CMAP_TABLE, 2469 },
	{ 0x5a1a, 0x5a1c, PDF_CMAP_TABLE, 2471 },
	{ 0x5a1e, 0x5a20, PDF_CMAP_TABLE, 2474 },
	{ 0x5a23, 0x5a25, PDF_CMAP_TABLE, 2477 },
	{ 0x5a27, 0x5a2a, PDF_CMAP_TABLE, 2480 },
	{ 0x5a2d, 0x5a2d, PDF_CMAP_SINGLE, 17498 },
	{ 0x5a2f, 0x5a30, PDF_CMAP_TABLE, 2484 },
	{ 0x5a35, 0x5a36, PDF_CMAP_RANGE, 4582 },
	{ 0x5a3c, 0x5a3c, PDF_CMAP_SINGLE, 2451 },
	{ 0x5a40, 0x5a41, PDF_CMAP_TABLE, 2486 },
	{ 0x5a44, 0x5a45, PDF_CMAP_RANGE, 21410 },
	{ 0x5a46, 0x5a49, PDF_CMAP_TABLE, 2488 },
	{ 0x5a4c, 0x5a4c, PDF_CMAP_SINGLE, 21413 },
	{ 0x5a50, 0x5a50, PDF_CMAP_SINGLE, 21414 },
	{ 0x5a55, 0x5a55, PDF_CMAP_SINGLE, 14458 },
	{ 0x5a5a, 0x5a5a, PDF_CMAP_SINGLE, 2071 },
	{ 0x5a5e, 0x5a5e, PDF_CMAP_SINGLE, 21415 },
	{ 0x5a62, 0x5a63, PDF_CMAP_TABLE, 2492 },
	{ 0x5a65, 0x5a67, PDF_CMAP_TABLE, 2494 },
	{ 0x5a6a, 0x5a6a, PDF_CMAP_SINGLE, 4585 },
	{ 0x5a6c, 0x5a6d, PDF_CMAP_TABLE, 2497 },
	{ 0x5a77, 0x5a77, PDF_CMAP_SINGLE, 16830 },
	{ 0x5a7a, 0x5a7b, PDF_CMAP_TABLE, 2499 },
	{ 0x5a7e, 0x5a7f, PDF_CMAP_TABLE, 2501 },
	{ 0x5a84, 0x5a84, PDF_CMAP_SINGLE, 16831 },
	{ 0x5a8b, 0x5a8b, PDF_CMAP_SINGLE, 17501 },
	{ 0x5a90, 0x5a90, PDF_CMAP_SINGLE, 21417 },
	{ 0x5a92, 0x5a93, PDF_CMAP_TABLE, 2503 },
	{ 0x5a96, 0x5a96, PDF_CMAP_SINGLE, 21419 },
	{ 0x5a99, 0x5a9c, PDF_CMAP_TABLE, 2505 },
	{ 0x5a9e, 0x5a9e, PDF_CMAP_SINGLE, 14462 },
	{ 0x5a9f, 0x5aa0, PDF_CMAP_RANGE, 17503 },
	{ 0x5aa2, 0x5aa2, PDF_CMAP_SINGLE, 17505 },
	{ 0x5aa7, 0x5aa7, PDF_CMAP_SINGLE, 14463 },
	{ 0x5aac, 0x5aac, PDF_CMAP_SINGLE, 14464 },
	{ 0x5ab1, 0x5ab3, PDF_CMAP_TABLE, 2509 },
	{ 0x5ab5, 0x5ab5, PDF_CMAP_SINGLE, 17507 },
	{ 0x5ab8, 0x5ab8, PDF_CMAP_SINGLE, 19314 },
	{ 0x5aba, 0x5abf, PDF_CMAP_TABLE, 2512 },
	{ 0x5ac1, 0x5ac2, PDF_CMAP_TABLE, 2518 },
	{ 0x5ac4, 0x5ac4, PDF_CMAP_SINGLE, 16832 },
	{ 0x5ac6, 0x5ac6, PDF_CMAP_SINGLE, 21422 },
	{ 0x5ac8, 0x5ac9, PDF_CMAP_TABLE, 2520 },
	{ 0x5acb, 0x5acc, PDF_CMAP_TABLE, 2522 },
	{ 0x5acf, 0x5ad0, PDF_CMAP_TABLE, 2524 },
	{ 0x5ad6, 0x5ad7, PDF_CMAP_TABLE, 2526 },
	{ 0x5ada, 0x5ada, PDF_CMAP_SINGLE, 17510 },
	{ 0x5adc, 0x5adc, PDF_CMAP_SINGLE, 17511 },
	{ 0x5ae0, 0x5ae1, PDF_CMAP_TABLE, 2528 },
	{ 0x5ae3, 0x5ae3, PDF_CMAP_SINGLE, 4592 },
	{ 0x5ae5, 0x5ae6, PDF_CMAP_TABLE, 2530 },
	{ 0x5ae9, 0x5aea, PDF_CMAP_TABLE, 2532 },
	{ 0x5aee, 0x5aee, PDF_CMAP_SINGLE, 17514 },
	{ 0x5af0, 0x5af0, PDF_CMAP_SINGLE, 17513 },
	{ 0x5af5, 0x5af6, PDF_CMAP_TABLE, 2534 },
	{ 0x5afa, 0x5afb, PDF_CMAP_RANGE, 4597 },
	{ 0x5afd, 0x5afd, PDF_CMAP_SINGLE, 21425 },
	{ 0x5b00, 0x5b01, PDF_CMAP_TABLE, 2536 },
	{ 0x5b08, 0x5b09, PDF_CMAP_TABLE, 2538 },
	{ 0x5b0b, 0x5b0c, PDF_CMAP_TABLE, 2540 },
	{ 0x5b16, 0x5b17, PDF_CMAP_TABLE, 2542 },
	{ 0x5b19, 0x5b19, PDF_CMAP_SINGLE, 14468 },
	{ 0x5b1b, 0x5b1b, PDF_CMAP_SINGLE, 19317 },
	{ 0x5b1d, 0x5b1d, PDF_CMAP_SINGLE, 19318 },
	{ 0x5b21, 0x5b22, PDF_CMAP_TABLE, 2544 },
	{ 0x5b25, 0x5b25, PDF_CMAP_SINGLE, 14469 },
	{ 0x5b2a, 0x5b2a, PDF_CMAP_SINGLE, 4604 },
	{ 0x5b2c, 0x5b2d, PDF_CMAP_TABLE, 2546 },
	{ 0x5b30, 0x5b30, PDF_CMAP_SINGLE, 1255 },
	{ 0x5b32, 0x5b32, PDF_CMAP_SINGLE, 4602 },
	{ 0x5b34, 0x5b34, PDF_CMAP_SINGLE, 17518 },
	{ 0x5b36, 0x5b36, PDF_CMAP_SINGLE, 4605 },
	{ 0x5b38, 0x5b38, PDF_CMAP_SINGLE, 19320 },
	{ 0x5b3e, 0x5b3e, PDF_CMAP_SINGLE, 4606 },
	{ 0x5b40, 0x5b41, PDF_CMAP_TABLE, 2548 },
	{ 0x5b43, 0x5b43, PDF_CMAP_SINGLE, 4607 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 4608 },
	{ 0x5b4b, 0x5b4c, PDF_CMAP_TABLE, 2550 },
	{ 0x5b50, 0x5b52, PDF_CMAP_TABLE, 2552 },
	{ 0x5b54, 0x5b58, PDF_CMAP_TABLE, 2555 },
	{ 0x5b5a, 0x5b5b, PDF_CMAP_RANGE, 4612 },
	{ 0x5b5c, 0x5b5f, PDF_CMAP_TABLE, 2560 },
	{ 0x5b63, 0x5b66, PDF_CMAP_TABLE, 2564 },
	{ 0x5b68, 0x5b69, PDF_CMAP_TABLE, 2568 },
	{ 0x5b6b, 0x5b6b, PDF_CMAP_SINGLE, 2841 },
	{ 0x5b6e, 0x5b71, PDF_CMAP_TABLE, 2570 },
	{ 0x5b73, 0x5b73, PDF_CMAP_SINGLE, 4617 },
	{ 0x5b75, 0x5b76, PDF_CMAP_TABLE, 2574 },
	{ 0x5b78, 0x5b78, PDF_CMAP_SINGLE, 4619 },
	{ 0x5b7a, 0x5b7a, PDF_CMAP_SINGLE, 4621 },
	{ 0x5b7c, 0x5b7d, PDF_CMAP_TABLE, 2576 },
	{ 0x5b7e, 0x5b7f, PDF_CMAP_RANGE, 14473 },
	{ 0x5b80, 0x5b8f, PDF_CMAP_TABLE, 2578 },
	{ 0x5b90, 0x5b91, PDF_CMAP_RANGE, 21432 },
	{ 0x5b93, 0x5b9d, PDF_CMAP_TABLE, 2594 },
	{ 0x5b9f, 0x5b9f, PDF_CMAP_SINGLE, 2286 },
	{ 0x5ba2, 0x5ba6, PDF_CMAP_TABLE, 2605 },
	{ 0x5ba8, 0x5ba9, PDF_CMAP_RANGE, 21435 },
	{ 0x5bac, 0x5bb0, PDF_CMAP_TABLE, 2610 },
	{ 0x5bb1, 0x5bb2, PDF_CMAP_RANGE, 21439 },
	{ 0x5bb3, 0x5bba, PDF_CMAP_TABLE, 2615 },
	{ 0x5bbc, 0x5bbc, PDF_CMAP_SINGLE, 21442 },
	{ 0x5bbf, 0x5bc7, PDF_CMAP_TABLE, 2623 },
	{ 0x5bc9, 0x5bc9, PDF_CMAP_SINGLE, 4628 },
	{ 0x5bcc, 0x5bd0, PDF_CMAP_TABLE, 2632 },
	{ 0x5bd2, 0x5bd4, PDF_CMAP_TABLE, 2637 },
	{ 0x5bd6, 0x5bd8, PDF_CMAP_TABLE, 2640 },
	{ 0x5bd9, 0x5bda, PDF_CMAP_RANGE, 21446 },
	{ 0x5bdb, 0x5bdb, PDF_CMAP_SINGLE, 1518 },
	{ 0x5bdd, 0x5be2, PDF_CMAP_TABLE, 2643 },
	{ 0x5be4, 0x5be9, PDF_CMAP_TABLE, 2649 },
	{ 0x5beb, 0x5bec, PDF_CMAP_TABLE, 2655 },
	{ 0x5bee, 0x5bf1, PDF_CMAP_TABLE, 2657 },
	{ 0x5bf3, 0x5bf6, PDF_CMAP_TABLE, 2661 },
	{ 0x5bf8, 0x5bf8, PDF_CMAP_SINGLE, 2631 },
	{ 0x5bfa, 0x5bfa, PDF_CMAP_SINGLE, 2249 },
	{ 0x5bfd, 0x5bff, PDF_CMAP_TABLE, 2665 },
	{ 0x5c01, 0x5c06, PDF_CMAP_TABLE, 2668 },
	{ 0x5c07, 0x5c08, PDF_CMAP_RANGE, 4641 },
	{ 0x5c09, 0x5c0f, PDF_CMAP_TABLE, 2674 },
	{ 0x5c11, 0x5c14, PDF_CMAP_TABLE, 2681 },
	{ 0x5c16, 0x5c17, PDF_CMAP_TABLE, 2685 },
	{ 0x5c19, 0x5c1a, PDF_CMAP_TABLE, 2687 },
	{ 0x5c1e, 0x5c20, PDF_CMAP_TABLE, 2689 },
	{ 0x5c22, 0x5c24, PDF_CMAP_TABLE, 2692 },
	{ 0x5c26, 0x5c26, PDF_CMAP_SINGLE, 21452 },
	{ 0x5c28, 0x5c2e, PDF_CMAP_TABLE, 2695 },
	{ 0x5c30, 0x5c32, PDF_CMAP_TABLE, 2702 },
	{ 0x5c35, 0x5c36, PDF_CMAP_TABLE, 2705 },
	{ 0x5c38, 0x5c39, PDF_CMAP_RANGE, 4648 },
	{ 0x5c3a, 0x5c41, PDF_CMAP_TABLE, 2707 },
	{ 0x5c45, 0x5c46, PDF_CMAP_TABLE, 2715 },
	{ 0x5c48, 0x5c48, PDF_CMAP_SINGLE, 1782 },
	{ 0x5c4a, 0x5c4b, PDF_CMAP_TABLE, 2717 },
	{ 0x5c4d, 0x5c51, PDF_CMAP_TABLE, 2719 },
	{ 0x5c53, 0x5c53, PDF_CMAP_SINGLE, 4653 },
	{ 0x5c55, 0x5c55, PDF_CMAP_SINGLE, 3122 },
	{ 0x5c59, 0x5c5c, PDF_CMAP_TABLE, 2724 },
	{ 0x5c5e, 0x5c65, PDF_CMAP_TABLE, 2728 },
	{ 0x5c67, 0x5c68, PDF_CMAP_RANGE, 17541 },
	{ 0x5c69, 0x5c69, PDF_CMAP_SINGLE, 14480 },
	{ 0x5c6c, 0x5c71, PDF_CMAP_TABLE, 2736 },
	{ 0x5c74, 0x5c75, PDF_CMAP_RANGE, 21457 },
	{ 0x5c76, 0x5c76, PDF_CMAP_SINGLE, 4660 },
	{ 0x5c79, 0x5c7d, PDF_CMAP_TABLE, 2742 },
	{ 0x5c87, 0x5c88, PDF_CMAP_TABLE, 2747 },
	{ 0x5c8a, 0x5c8a, PDF_CMAP_SINGLE, 17549 },
	{ 0x5c8c, 0x5c8c, PDF_CMAP_SINGLE, 4662 },
	{ 0x5c8f, 0x5c92, PDF_CMAP_TABLE, 2749 },
	{ 0x5c94, 0x5c94, PDF_CMAP_SINGLE, 4664 },
	{ 0x5c9d, 0x5c9d, PDF_CMAP_SINGLE, 21463 },
	{ 0x5c9f, 0x5ca3, PDF_CMAP_TABLE, 2753 },
	{ 0x5ca6, 0x5cad, PDF_CMAP_TABLE, 2758 },
	{ 0x5cb1, 0x5cb8, PDF_CMAP_TABLE, 2766 },
	{ 0x5cba, 0x5cbc, PDF_CMAP_TABLE, 2774 },
	{ 0x5cbe, 0x5cbe, PDF_CMAP_SINGLE, 4672 },
	{ 0x5cc5, 0x5cc5, PDF_CMAP_SINGLE, 4671 },
	{ 0x5cc7, 0x5cc7, PDF_CMAP_SINGLE, 4673 },
	{ 0x5cc9, 0x5cc9, PDF_CMAP_SINGLE, 17560 },
	{ 0x5ccb, 0x5ccb, PDF_CMAP_SINGLE, 14482 },
	{ 0x5cd0, 0x5cd0, PDF_CMAP_SINGLE, 16844 },
	{ 0x5cd2, 0x5cd2, PDF_CMAP_SINGLE, 14483 },
	{ 0x5cd7, 0x5cd7, PDF_CMAP_SINGLE, 21466 },
	{ 0x5cd9, 0x5cd9, PDF_CMAP_SINGLE, 4674 },
	{ 0x5cdd, 0x5cdd, PDF_CMAP_SINGLE, 19331 },
	{ 0x5ce0, 0x5ce1, PDF_CMAP_TABLE, 2777 },
	{ 0x5ce6, 0x5ce6, PDF_CMAP_SINGLE, 14124 },
	{ 0x5ce8, 0x5cea, PDF_CMAP_TABLE, 2779 },
	{ 0x5ced, 0x5cf0, PDF_CMAP_TABLE, 2782 },
	{ 0x5cf1, 0x5cf2, PDF_CMAP_RANGE, 21468 },
	{ 0x5cf4, 0x5cf6, PDF_CMAP_TABLE, 2786 },
	{ 0x5cfa, 0x5cfb, PDF_CMAP_TABLE, 2789 },
	{ 0x5cfd, 0x5cfd, PDF_CMAP_SINGLE, 4676 },
	{ 0x5d01, 0x5d01, PDF_CMAP_SINGLE, 19332 },
	{ 0x5d06, 0x5d07, PDF_CMAP_TABLE, 2791 },
	{ 0x5d0b, 0x5d0b, PDF_CMAP_SINGLE, 4681 },
	{ 0x5d0d, 0x5d0e, PDF_CMAP_TABLE, 2793 },
	{ 0x5d10, 0x5d12, PDF_CMAP_TABLE, 2795 },
	{ 0x5d14, 0x5d1b, PDF_CMAP_TABLE, 2798 },
	{ 0x5d1d, 0x5d1d, PDF_CMAP_SINGLE, 17566 },
	{ 0x5d1f, 0x5d20, PDF_CMAP_TABLE, 2806 },
	{ 0x5d22, 0x5d24, PDF_CMAP_TABLE, 2808 },
	{ 0x5d26, 0x5d27, PDF_CMAP_TABLE, 2811 },
	{ 0x5d29, 0x5d29, PDF_CMAP_SINGLE, 3656 },
	{ 0x5d2b, 0x5d2b, PDF_CMAP_SINGLE, 17565 },
	{ 0x5d31, 0x5d31, PDF_CMAP_SINGLE, 17568 },
	{ 0x5d34, 0x5d34, PDF_CMAP_SINGLE, 19333 },
	{ 0x5d39, 0x5d39, PDF_CMAP_SINGLE, 17569 },
	{ 0x5d3d, 0x5d3d, PDF_CMAP_SINGLE, 19334 },
	{ 0x5d3f, 0x5d3f, PDF_CMAP_SINGLE, 21472 },
	{ 0x5d42, 0x5d43, PDF_CMAP_TABLE, 2813 },
	{ 0x5d46, 0x5d48, PDF_CMAP_TABLE, 2815 },
	{ 0x5d4a, 0x5d4c, PDF_CMAP_TABLE, 2818 },
	{ 0x5d4e, 0x5d4e, PDF_CMAP_SINGLE, 4695 },
	{ 0x5d50, 0x5d53, PDF_CMAP_TABLE, 2821 },
	{ 0x5d55, 0x5d55, PDF_CMAP_SINGLE, 21474 },
	{ 0x5d59, 0x5d59, PDF_CMAP_SINGLE, 19335 },
	{ 0x5d5c, 0x5d5c, PDF_CMAP_SINGLE, 4684 },
	{ 0x5d5f, 0x5d60, PDF_CMAP_RANGE, 21476 },
	{ 0x5d61, 0x5d62, PDF_CMAP_TABLE, 2825 },
	{ 0x5d64, 0x5d64, PDF_CMAP_SINGLE, 21479 },
	{ 0x5d69, 0x5d6a, PDF_CMAP_TABLE, 2827 },
	{ 0x5d6c, 0x5d6d, PDF_CMAP_TABLE, 2829 },
	{ 0x5d6f, 0x5d70, PDF_CMAP_TABLE, 2831 },
	{ 0x5d73, 0x5d73, PDF_CMAP_SINGLE, 4698 },
	{ 0x5d76, 0x5d76, PDF_CMAP_SINGLE, 4699 },
	{ 0x5d79, 0x5d7a, PDF_CMAP_RANGE, 21480 },
	{ 0x5d7e, 0x5d7f, PDF_CMAP_TABLE, 2833 },
	{ 0x5d81, 0x5d84, PDF_CMAP_TABLE, 2835 },
	{ 0x5d87, 0x5d88, PDF_CMAP_TABLE, 2839 },
	{ 0x5d8a, 0x5d8c, PDF_CMAP_TABLE, 2841 },
	{ 0x5d90, 0x5d90, PDF_CMAP_SINGLE, 4708 },
	{ 0x5d92, 0x5d95, PDF_CMAP_TABLE, 2844 },
	{ 0x5d97, 0x5d97, PDF_CMAP_SINGLE, 17579 },
	{ 0x5d99, 0x5d99, PDF_CMAP_SINGLE, 14492 },
	{ 0x5d9b, 0x5d9b, PDF_CMAP_SINGLE, 21486 },
	{ 0x5d9d, 0x5d9d, PDF_CMAP_SINGLE, 4704 },
	{ 0x5d9f, 0x5da0, PDF_CMAP_TABLE, 2848 },
	{ 0x5da2, 0x5da2, PDF_CMAP_SINGLE, 4703 },
	{ 0x5da4, 0x5da4, PDF_CMAP_SINGLE, 16849 },
	{ 0x5da7, 0x5da7, PDF_CMAP_SINGLE, 16850 },
	{ 0x5dab, 0x5dac, PDF_CMAP_TABLE, 2850 },
	{ 0x5dae, 0x5dae, PDF_CMAP_SINGLE, 4706 },
	{ 0x5db0, 0x5db0, PDF_CMAP_SINGLE, 17580 },
	{ 0x5db2, 0x5db2, PDF_CMAP_SINGLE, 15405 },
	{ 0x5db4, 0x5db4, PDF_CMAP_SINGLE, 17581 },
	{ 0x5db7, 0x5db7, PDF_CMAP_SINGLE, 4709 },
	{ 0x5db8, 0x5db9, PDF_CMAP_RANGE, 8446 },
	{ 0x5dba, 0x5dba, PDF_CMAP_SINGLE, 4014 },
	{ 0x5dbc, 0x5dbd, PDF_CMAP_TABLE, 2852 },
	{ 0x5dc3, 0x5dc3, PDF_CMAP_SINGLE, 21489 },
	{ 0x5dc7, 0x5dc7, PDF_CMAP_SINGLE, 19338 },
	{ 0x5dc9, 0x5dc9, PDF_CMAP_SINGLE, 4711 },
	{ 0x5dcb, 0x5dce, PDF_CMAP_TABLE, 2854 },
	{ 0x5dd0, 0x5dd3, PDF_CMAP_TABLE, 2858 },
	{ 0x5dd6, 0x5dd9, PDF_CMAP_TABLE, 2862 },
	{ 0x5ddb, 0x5ddb, PDF_CMAP_SINGLE, 4716 },
	{ 0x5ddd, 0x5dde, PDF_CMAP_TABLE, 2866 },
	{ 0x5de0, 0x5de9, PDF_CMAP_TABLE, 2868 },
	{ 0x5deb, 0x5deb, PDF_CMAP_SINGLE, 4717 },
	{ 0x5dee, 0x5dee, PDF_CMAP_SINGLE, 2089 },
	{ 0x5df1, 0x5df5, PDF_CMAP_TABLE, 2878 },
	{ 0x5df7, 0x5df9, PDF_CMAP_TABLE, 2883 },
	{ 0x5dfb, 0x5dfb, PDF_CMAP_SINGLE, 1512 },
	{ 0x5dfd, 0x5e00, PDF_CMAP_TABLE, 2886 },
	{ 0x5e02, 0x5e03, PDF_CMAP_TABLE, 2890 },
	{ 0x5e06, 0x5e07, PDF_CMAP_TABLE, 2892 },
	{ 0x5e0b, 0x5e0d, PDF_CMAP_TABLE, 2894 },
	{ 0x5e11, 0x5e12, PDF_CMAP_TABLE, 2897 },
	{ 0x5e14, 0x5e15, PDF_CMAP_RANGE, 14499 },
	{ 0x5e16, 0x5e16, PDF_CMAP_SINGLE, 3005 },
	{ 0x5e18, 0x5e1b, PDF_CMAP_TABLE, 2899 },
	{ 0x5e1d, 0x5e1d, PDF_CMAP_SINGLE, 3079 },
	{ 0x5e1f, 0x5e20, PDF_CMAP_TABLE, 2903 },
	{ 0x5e25, 0x5e25, PDF_CMAP_SINGLE, 2601 },
	{ 0x5e28, 0x5e28, PDF_CMAP_SINGLE, 19340 },
	{ 0x5e2b, 0x5e2b, PDF_CMAP_SINGLE, 2211 },
	{ 0x5e2d, 0x5e30, PDF_CMAP_TABLE, 2905 },
	{ 0x5e32, 0x5e33, PDF_CMAP_TABLE, 2909 },
	{ 0x5e35, 0x5e35, PDF_CMAP_SINGLE, 19342 },
	{ 0x5e36, 0x5e37, PDF_CMAP_RANGE, 4725 },
	{ 0x5e38, 0x5e38, PDF_CMAP_SINGLE, 2519 },
	{ 0x5e3d, 0x5e3e, PDF_CMAP_TABLE, 2911 },
	{ 0x5e40, 0x5e40, PDF_CMAP_SINGLE, 4729 },
	{ 0x5e43, 0x5e45, PDF_CMAP_TABLE, 2913 },
	{ 0x5e47, 0x5e47, PDF_CMAP_SINGLE, 4736 },
	{ 0x5e49, 0x5e49, PDF_CMAP_SINGLE, 17593 },
	{ 0x5e4b, 0x5e4c, PDF_CMAP_TABLE, 2916 },
	{ 0x5e4e, 0x5e4e, PDF_CMAP_SINGLE, 4730 },
	{ 0x5e50, 0x5e51, PDF_CMAP_RANGE, 21496 },
	{ 0x5e54, 0x5e58, PDF_CMAP_TABLE, 2918 },
	{ 0x5e5b, 0x5e5c, PDF_CMAP_TABLE, 2923 },
	{ 0x5e5e, 0x5e5f, PDF_CMAP_TABLE, 2925 },
	{ 0x5e61, 0x5e64, PDF_CMAP_TABLE, 2927 },
	{ 0x5e68, 0x5e68, PDF_CMAP_SINGLE, 19344 },
	{ 0x5e6a, 0x5e6a, PDF_CMAP_SINGLE, 19345 },
	{ 0x5e6b, 0x5e6c, PDF_CMAP_RANGE, 14504 },
	{ 0x5e6d, 0x5e6e, PDF_CMAP_RANGE, 17597 },
	{ 0x5e70, 0x5e70, PDF_CMAP_SINGLE, 21499 },
	{ 0x5e72, 0x5e74, PDF_CMAP_TABLE, 2931 },
	{ 0x5e75, 0x5e76, PDF_CMAP_RANGE, 4737 },
	{ 0x5e77, 0x5e81, PDF_CMAP_TABLE, 2934 },
	{ 0x5e83, 0x5e84, PDF_CMAP_TABLE, 2945 },
	{ 0x5e87, 0x5e87, PDF_CMAP_SINGLE, 3443 },
	{ 0x5e8a, 0x5e8b, PDF_CMAP_TABLE, 2947 },
	{ 0x5e8e, 0x5e8f, PDF_CMAP_TABLE, 2949 },
	{ 0x5e95, 0x5e97, PDF_CMAP_TABLE, 2951 },
	{ 0x5e99, 0x5e9a, PDF_CMAP_TABLE, 2954 },
	{ 0x5e9c, 0x5e9c, PDF_CMAP_SINGLE, 3534 },
	{ 0x5ea0, 0x5ea0, PDF_CMAP_SINGLE, 4742 },
	{ 0x5ea2, 0x5ea2, PDF_CMAP_SINGLE, 21501 },
	{ 0x5ea4, 0x5ea8, PDF_CMAP_TABLE, 2956 },
	{ 0x5eaa, 0x5ead, PDF_CMAP_TABLE, 2961 },
	{ 0x5eb1, 0x5eb1, PDF_CMAP_SINGLE, 21503 },
	{ 0x5eb3, 0x5eb3, PDF_CMAP_SINGLE, 19349 },
	{ 0x5eb5, 0x5eb9, PDF_CMAP_TABLE, 2965 },
	{ 0x5ebd, 0x5ebd, PDF_CMAP_SINGLE, 19350 },
	{ 0x5ebe, 0x5ebf, PDF_CMAP_RANGE, 14508 },
	{ 0x5ec1, 0x5ec2, PDF_CMAP_RANGE, 4743 },
	{ 0x5ec3, 0x5ec3, PDF_CMAP_SINGLE, 3335 },
	{ 0x5ec6, 0x5ec6, PDF_CMAP_SINGLE, 17603 },
	{ 0x5ec8, 0x5ecc, PDF_CMAP_TABLE, 2970 },
	{ 0x5ece, 0x5ed3, PDF_CMAP_TABLE, 2975 },
	{ 0x5ed4, 0x5ed5, PDF_CMAP_RANGE, 19352 },
	{ 0x5ed6, 0x5ed6, PDF_CMAP_SINGLE, 4748 },
	{ 0x5ed9, 0x5ed9, PDF_CMAP_SINGLE, 17604 },
	{ 0x5eda, 0x5edb, PDF_CMAP_RANGE, 4751 },
	{ 0x5edc, 0x5ee3, PDF_CMAP_TABLE, 2981 },
	{ 0x5ee5, 0x5ee5, PDF_CMAP_SINGLE, 21508 },
	{ 0x5ee8, 0x5ee9, PDF_CMAP_RANGE, 4755 },
	{ 0x5eeb, 0x5eec, PDF_CMAP_TABLE, 2989 },
	{ 0x5ef0, 0x5ef1, PDF_CMAP_TABLE, 2991 },
	{ 0x5ef3, 0x5ef4, PDF_CMAP_TABLE, 2993 },
	{ 0x5ef6, 0x5f04, PDF_CMAP_TABLE, 2995 },
	{ 0x5f06, 0x5f11, PDF_CMAP_TABLE, 3010 },
	{ 0x5f13, 0x5f19, PDF_CMAP_TABLE, 3022 },
	{ 0x5f1b, 0x5f1b, PDF_CMAP_SINGLE, 2958 },
	{ 0x5f1c, 0x5f1d, PDF_CMAP_RANGE, 14514 },
	{ 0x5f1e, 0x5f1f, PDF_CMAP_TABLE, 3029 },
	{ 0x5f21, 0x5f29, PDF_CMAP_TABLE, 3031 },
	{ 0x5f2b, 0x5f2c, PDF_CMAP_RANGE, 21513 },
	{ 0x5f2d, 0x5f31, PDF_CMAP_TABLE, 3040 },
	{ 0x5f34, 0x5f38, PDF_CMAP_TABLE, 3045 },
	{ 0x5f3a, 0x5f41, PDF_CMAP_TABLE, 3050 },
	{ 0x5f44, 0x5f45, PDF_CMAP_TABLE, 3058 },
	{ 0x5f47, 0x5f48, PDF_CMAP_TABLE, 3060 },
	{ 0x5f4a, 0x5f4a, PDF_CMAP_SINGLE, 1704 },
	{ 0x5f4c, 0x5f4e, PDF_CMAP_TABLE, 3062 },
	{ 0x5f50, 0x5f51, PDF_CMAP_TABLE, 3065 },
	{ 0x5f53, 0x5f54, PDF_CMAP_TABLE, 3067 },
	{ 0x5f56, 0x5f57, PDF_CMAP_RANGE, 4780 },
	{ 0x5f58, 0x5f59, PDF_CMAP_TABLE, 3069 },
	{ 0x5f5b, 0x5f5d, PDF_CMAP_TABLE, 3071 },
	{ 0x5f60, 0x5f67, PDF_CMAP_TABLE, 3074 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 3082 },
	{ 0x5f6f, 0x5f73, PDF_CMAP_TABLE, 3087 },
	{ 0x5f74, 0x5f75, PDF_CMAP_RANGE, 21522 },
	{ 0x5f77, 0x5f7a, PDF_CMAP_TABLE, 3092 },
	{ 0x5f7c, 0x5f85, PDF_CMAP_TABLE, 3096 },
	{ 0x5f87, 0x5f8d, PDF_CMAP_TABLE, 3106 },
	{ 0x5f8f, 0x5f93, PDF_CMAP_TABLE, 3113 },
	{ 0x5f96, 0x5f99, PDF_CMAP_TABLE, 3118 },
	{ 0x5f9c, 0x5f9e, PDF_CMAP_TABLE, 3122 },
	{ 0x5fa0, 0x5fa2, PDF_CMAP_TABLE, 3125 },
	{ 0x5fa4, 0x5fa4, PDF_CMAP_SINGLE, 14527 },
	{ 0x5fa7, 0x5faf, PDF_CMAP_TABLE, 3128 },
	{ 0x5fb0, 0x5fb1, PDF_CMAP_RANGE, 21530 },
	{ 0x5fb3, 0x5fb5, PDF_CMAP_TABLE, 3137 },
	{ 0x5fb7, 0x5fb9, PDF_CMAP_TABLE, 3140 },
	{ 0x5fbc, 0x5fbd, PDF_CMAP_TABLE, 3143 },
	{ 0x5fc3, 0x5fc5, PDF_CMAP_TABLE, 3145 },
	{ 0x5fc7, 0x5fc9, PDF_CMAP_TABLE, 3148 },
	{ 0x5fcb, 0x5fcd, PDF_CMAP_TABLE, 3151 },
	{ 0x5fd0, 0x5fd1, PDF_CMAP_RANGE, 21533 },
	{ 0x5fd2, 0x5fd4, PDF_CMAP_RANGE, 17618 },
	{ 0x5fd6, 0x5fd7, PDF_CMAP_TABLE, 3154 },
	{ 0x5fd8, 0x5fd9, PDF_CMAP_RANGE, 3688 },
	{ 0x5fdc, 0x5fde, PDF_CMAP_TABLE, 3156 },
	{ 0x5fe0, 0x5fe2, PDF_CMAP_TABLE, 3159 },
	{ 0x5fe4, 0x5fe4, PDF_CMAP_SINGLE, 4803 },
	{ 0x5fe8, 0x5fed, PDF_CMAP_TABLE, 3162 },
	{ 0x5fee, 0x5fef, PDF_CMAP_RANGE, 17622 },
	{ 0x5ff0, 0x5ff3, PDF_CMAP_TABLE, 3168 },
	{ 0x5ff5, 0x5ff6, PDF_CMAP_TABLE, 3172 },
	{ 0x5ff8, 0x5ff8, PDF_CMAP_SINGLE, 4804 },
	{ 0x5ffa, 0x5ffd, PDF_CMAP_TABLE, 3174 },
	{ 0x5fff, 0x5fff, PDF_CMAP_SINGLE, 4808 },
	{ 0x6007, 0x6007, PDF_CMAP_SINGLE, 19357 },
	{ 0x600a, 0x600a, PDF_CMAP_SINGLE, 21540 },
	{ 0x600d, 0x6010, PDF_CMAP_TABLE, 3178 },
	{ 0x6012, 0x601d, PDF_CMAP_TABLE, 3182 },
	{ 0x601f, 0x6022, PDF_CMAP_TABLE, 3194 },
	{ 0x6024, 0x602b, PDF_CMAP_TABLE, 3198 },
	{ 0x602d, 0x602d, PDF_CMAP_SINGLE, 21543 },
	{ 0x602f, 0x602f, PDF_CMAP_SINGLE, 1705 },
	{ 0x6031, 0x6031, PDF_CMAP_SINGLE, 4815 },
	{ 0x6033, 0x6033, PDF_CMAP_SINGLE, 14538 },
	{ 0x6035, 0x6035, PDF_CMAP_SINGLE, 16863 },
	{ 0x603a, 0x603a, PDF_CMAP_SINGLE, 4821 },
	{ 0x6040, 0x6043, PDF_CMAP_TABLE, 3206 },
	{ 0x6046, 0x604d, PDF_CMAP_TABLE, 3210 },
	{ 0x6050, 0x6052, PDF_CMAP_TABLE, 3218 },
	{ 0x6054, 0x6055, PDF_CMAP_TABLE, 3221 },
	{ 0x6056, 0x6057, PDF_CMAP_RANGE, 21547 },
	{ 0x6059, 0x605a, PDF_CMAP_TABLE, 3223 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 8454 },
	{ 0x605f, 0x6065, PDF_CMAP_TABLE, 3225 },
	{ 0x6067, 0x606d, PDF_CMAP_TABLE, 3232 },
	{ 0x606f, 0x6071, PDF_CMAP_TABLE, 3239 },
	{ 0x6075, 0x6075, PDF_CMAP_SINGLE, 1817 },
	{ 0x6077, 0x6077, PDF_CMAP_SINGLE, 4825 },
	{ 0x607e, 0x607f, PDF_CMAP_TABLE, 3242 },
	{ 0x6081, 0x6086, PDF_CMAP_TABLE, 3244 },
	{ 0x6088, 0x608e, PDF_CMAP_TABLE, 3250 },
	{ 0x6091, 0x6095, PDF_CMAP_TABLE, 3257 },
	{ 0x6096, 0x6097, PDF_CMAP_RANGE, 4844 },
	{ 0x6098, 0x6098, PDF_CMAP_SINGLE, 21557 },
	{ 0x609a, 0x609b, PDF_CMAP_TABLE, 3262 },
	{ 0x609d, 0x60a0, PDF_CMAP_TABLE, 3264 },
	{ 0x60a2, 0x60aa, PDF_CMAP_TABLE, 3268 },
	{ 0x60b0, 0x60b8, PDF_CMAP_TABLE, 3277 },
	{ 0x60bb, 0x60be, PDF_CMAP_TABLE, 3286 },
	{ 0x60c2, 0x60c2, PDF_CMAP_SINGLE, 21562 },
	{ 0x60c4, 0x60c8, PDF_CMAP_TABLE, 3290 },
	{ 0x60c9, 0x60ca, PDF_CMAP_RANGE, 21563 },
	{ 0x60cb, 0x60cb, PDF_CMAP_SINGLE, 14544 },
	{ 0x60ce, 0x60cf, PDF_CMAP_RANGE, 21565 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 4077 },
	{ 0x60d3, 0x60d5, PDF_CMAP_TABLE, 3295 },
	{ 0x60d8, 0x60e3, PDF_CMAP_TABLE, 3298 },
	{ 0x60e5, 0x60e5, PDF_CMAP_SINGLE, 21568 },
	{ 0x60e7, 0x60e8, PDF_CMAP_TABLE, 3310 },
	{ 0x60ee, 0x60ee, PDF_CMAP_SINGLE, 17637 },
	{ 0x60f0, 0x60f5, PDF_CMAP_TABLE, 3312 },
	{ 0x60f6, 0x60f7, PDF_CMAP_RANGE, 4862 },
	{ 0x60f8, 0x60fd, PDF_CMAP_TABLE, 3318 },
	{ 0x6100, 0x6103, PDF_CMAP_TABLE, 3324 },
	{ 0x6106, 0x610a, PDF_CMAP_TABLE, 3328 },
	{ 0x610c, 0x610c, PDF_CMAP_SINGLE, 21572 },
	{ 0x610d, 0x610e, PDF_CMAP_RANGE, 4871 },
	{ 0x610f, 0x6111, PDF_CMAP_TABLE, 3333 },
	{ 0x6112, 0x6114, PDF_CMAP_RANGE, 14547 },
	{ 0x6115, 0x6117, PDF_CMAP_TABLE, 3336 },
	{ 0x6119, 0x611c, PDF_CMAP_TABLE, 3339 },
	{ 0x611e, 0x6122, PDF_CMAP_TABLE, 3343 },
	{ 0x6127, 0x6128, PDF_CMAP_TABLE, 3348 },
	{ 0x612a, 0x612c, PDF_CMAP_TABLE, 3350 },
	{ 0x6130, 0x6131, PDF_CMAP_TABLE, 3353 },
	{ 0x6134, 0x6137, PDF_CMAP_TABLE, 3355 },
	{ 0x6139, 0x613a, PDF_CMAP_TABLE, 3359 },
	{ 0x613c, 0x613f, PDF_CMAP_TABLE, 3361 },
	{ 0x6141, 0x6142, PDF_CMAP_TABLE, 3365 },
	{ 0x6144, 0x614e, PDF_CMAP_TABLE, 3367 },
	{ 0x6153, 0x6153, PDF_CMAP_SINGLE, 4897 },
	{ 0x6155, 0x6155, PDF_CMAP_SINGLE, 3641 },
	{ 0x6158, 0x615a, PDF_CMAP_RANGE, 4887 },
	{ 0x615d, 0x6160, PDF_CMAP_TABLE, 3378 },
	{ 0x6162, 0x6165, PDF_CMAP_TABLE, 3382 },
	{ 0x6167, 0x6168, PDF_CMAP_TABLE, 3386 },
	{ 0x616b, 0x616c, PDF_CMAP_TABLE, 3388 },
	{ 0x616e, 0x6178, PDF_CMAP_TABLE, 3390 },
	{ 0x617b, 0x617f, PDF_CMAP_TABLE, 3401 },
	{ 0x6180, 0x6181, PDF_CMAP_RANGE, 21583 },
	{ 0x6182, 0x6182, PDF_CMAP_SINGLE, 3861 },
	{ 0x6183, 0x6184, PDF_CMAP_RANGE, 21585 },
	{ 0x6187, 0x6187, PDF_CMAP_SINGLE, 4901 },
	{ 0x618a, 0x618b, PDF_CMAP_TABLE, 3406 },
	{ 0x618d, 0x618e, PDF_CMAP_TABLE, 3408 },
	{ 0x6190, 0x6191, PDF_CMAP_TABLE, 3410 },
	{ 0x6192, 0x6193, PDF_CMAP_RANGE, 17648 },
	{ 0x6194, 0x6194, PDF_CMAP_SINGLE, 4903 },
	{ 0x6196, 0x619a, PDF_CMAP_TABLE, 3412 },
	{ 0x619c, 0x619d, PDF_CMAP_TABLE, 3417 },
	{ 0x619f, 0x61a0, PDF_CMAP_TABLE, 3419 },
	{ 0x61a4, 0x61a5, PDF_CMAP_TABLE, 3421 },
	{ 0x61a7, 0x61ae, PDF_CMAP_TABLE, 3423 },
	{ 0x61b2, 0x61b2, PDF_CMAP_SINGLE, 1873 },
	{ 0x61b6, 0x61b6, PDF_CMAP_SINGLE, 1329 },
	{ 0x61b8, 0x61ba, PDF_CMAP_TABLE, 3431 },
	{ 0x61bc, 0x61bc, PDF_CMAP_SINGLE, 16869 },
	{ 0x61be, 0x61be, PDF_CMAP_SINGLE, 1524 },
	{ 0x61c0, 0x61c1, PDF_CMAP_RANGE, 21591 },
	{ 0x61c2, 0x61c3, PDF_CMAP_TABLE, 3434 },
	{ 0x61c6, 0x61cd, PDF_CMAP_TABLE, 3436 },
	{ 0x61ce, 0x61cf, PDF_CMAP_RANGE, 21593 },
	{ 0x61d0, 0x61d0, PDF_CMAP_SINGLE, 1403 },
	{ 0x61d5, 0x61d5, PDF_CMAP_SINGLE, 17654 },
	{ 0x61dc, 0x61df, PDF_CMAP_TABLE, 3444 },
	{ 0x61e1, 0x61e3, PDF_CMAP_TABLE, 3448 },
	{ 0x61e5, 0x61e9, PDF_CMAP_TABLE, 3451 },
	{ 0x61ec, 0x61ed, PDF_CMAP_RANGE, 21599 },
	{ 0x61ef, 0x61ef, PDF_CMAP_SINGLE, 21601 },
	{ 0x61f2, 0x61f2, PDF_CMAP_SINGLE, 3012 },
	{ 0x61f4, 0x61f8, PDF_CMAP_TABLE, 3456 },
	{ 0x61fa, 0x61fa, PDF_CMAP_SINGLE, 4923 },
	{ 0x61fc, 0x6201, PDF_CMAP_TABLE, 3461 },
	{ 0x6203, 0x6204, PDF_CMAP_TABLE, 3467 },
	{ 0x6207, 0x6207, PDF_CMAP_SINGLE, 19380 },
	{ 0x6208, 0x6209, PDF_CMAP_RANGE, 4930 },
	{ 0x620a, 0x620a, PDF_CMAP_SINGLE, 3642 },
	{ 0x620c, 0x620e, PDF_CMAP_TABLE, 3469 },
	{ 0x6210, 0x6216, PDF_CMAP_TABLE, 3472 },
	{ 0x621a, 0x6223, PDF_CMAP_TABLE, 3479 },
	{ 0x6226, 0x6227, PDF_CMAP_TABLE, 3489 },
	{ 0x6229, 0x622b, PDF_CMAP_TABLE, 3491 },
	{ 0x622e, 0x6231, PDF_CMAP_TABLE, 3494 },
	{ 0x6232, 0x6233, PDF_CMAP_RANGE, 4941 },
	{ 0x6234, 0x6234, PDF_CMAP_SINGLE, 2871 },
	{ 0x6236, 0x6236, PDF_CMAP_SINGLE, 13757 },
	{ 0x6238, 0x6239, PDF_CMAP_TABLE, 3498 },
	{ 0x623b, 0x623b, PDF_CMAP_SINGLE, 3821 },
	{ 0x623d, 0x6244, PDF_CMAP_TABLE, 3500 },
	{ 0x6246, 0x6249, PDF_CMAP_TABLE, 3508 },
	{ 0x624b, 0x624e, PDF_CMAP_TABLE, 3512 },
	{ 0x6250, 0x6256, PDF_CMAP_TABLE, 3516 },
	{ 0x6258, 0x6258, PDF_CMAP_SINGLE, 2897 },
	{ 0x625a, 0x625c, PDF_CMAP_TABLE, 3523 },
	{ 0x625e, 0x625e, PDF_CMAP_SINGLE, 4945 },
	{ 0x6260, 0x6261, PDF_CMAP_TABLE, 3526 },
	{ 0x6263, 0x6264, PDF_CMAP_TABLE, 3528 },
	{ 0x6268, 0x6268, PDF_CMAP_SINGLE, 4949 },
	{ 0x626d, 0x626f, PDF_CMAP_TABLE, 3530 },
	{ 0x6271, 0x6271, PDF_CMAP_SINGLE, 1147 },
	{ 0x6273, 0x6273, PDF_CMAP_SINGLE, 17664 },
	{ 0x6276, 0x6276, PDF_CMAP_SINGLE, 3536 },
	{ 0x6279, 0x6280, PDF_CMAP_TABLE, 3533 },
	{ 0x6282, 0x6285, PDF_CMAP_TABLE, 3541 },
	{ 0x6289, 0x628a, PDF_CMAP_TABLE, 3545 },
	{ 0x628d, 0x628f, PDF_CMAP_RANGE, 21614 },
	{ 0x6290, 0x6291, PDF_CMAP_TABLE, 3547 },
	{ 0x6292, 0x6293, PDF_CMAP_RANGE, 4954 },
	{ 0x6294, 0x6299, PDF_CMAP_TABLE, 3549 },
	{ 0x629b, 0x629c, PDF_CMAP_TABLE, 3555 },
	{ 0x629e, 0x629e, PDF_CMAP_SINGLE, 2898 },
	{ 0x62a6, 0x62a6, PDF_CMAP_SINGLE, 8466 },
	{ 0x62a8, 0x62a8, PDF_CMAP_SINGLE, 19386 },
	{ 0x62ab, 0x62ac, PDF_CMAP_TABLE, 3557 },
	{ 0x62b1, 0x62b1, PDF_CMAP_SINGLE, 3658 },
	{ 0x62b3, 0x62b3, PDF_CMAP_SINGLE, 21617 },
	{ 0x62b5, 0x62b5, PDF_CMAP_SINGLE, 3085 },
	{ 0x62b6, 0x62b7, PDF_CMAP_RANGE, 21618 },
	{ 0x62b9, 0x62bd, PDF_CMAP_TABLE, 3559 },
	{ 0x62be, 0x62bf, PDF_CMAP_RANGE, 21621 },
	{ 0x62c2, 0x62c2, PDF_CMAP_SINGLE, 4971 },
	{ 0x62c4, 0x62ca, PDF_CMAP_TABLE, 3564 },
	{ 0x62cc, 0x62dd, PDF_CMAP_TABLE, 3571 },
	{ 0x62e0, 0x62e1, PDF_CMAP_TABLE, 3589 },
	{ 0x62ea, 0x62ea, PDF_CMAP_SINGLE, 21624 },
	{ 0x62ec, 0x62ef, PDF_CMAP_TABLE, 3591 },
	{ 0x62f1, 0x62f7, PDF_CMAP_TABLE, 3595 },
	{ 0x62fc, 0x62ff, PDF_CMAP_TABLE, 3602 },
	{ 0x6301, 0x6304, PDF_CMAP_TABLE, 3606 },
	{ 0x6307, 0x630d, PDF_CMAP_TABLE, 3610 },
	{ 0x6310, 0x6311, PDF_CMAP_TABLE, 3617 },
	{ 0x6313, 0x6313, PDF_CMAP_SINGLE, 21628 },
	{ 0x6316, 0x6316, PDF_CMAP_SINGLE, 19389 },
	{ 0x6318, 0x6319, PDF_CMAP_TABLE, 3619 },
	{ 0x631b, 0x631b, PDF_CMAP_SINGLE, 14135 },
	{ 0x631f, 0x631f, PDF_CMAP_SINGLE, 1708 },
	{ 0x6327, 0x632b, PDF_CMAP_TABLE, 3621 },
	{ 0x632d, 0x632d, PDF_CMAP_SINGLE, 21630 },
	{ 0x632f, 0x632f, PDF_CMAP_SINGLE, 2556 },
	{ 0x6332, 0x6332, PDF_CMAP_SINGLE, 17673 },
	{ 0x6335, 0x6336, PDF_CMAP_TABLE, 3626 },
	{ 0x6339, 0x633a, PDF_CMAP_TABLE, 3628 },
	{ 0x633b, 0x633c, PDF_CMAP_RANGE, 17675 },
	{ 0x633d, 0x633f, PDF_CMAP_TABLE, 3630 },
	{ 0x6341, 0x6341, PDF_CMAP_SINGLE, 17677 },
	{ 0x6342, 0x6343, PDF_CMAP_RANGE, 14570 },
	{ 0x6344, 0x6344, PDF_CMAP_SINGLE, 17678 },
	{ 0x6346, 0x6346, PDF_CMAP_SINGLE, 19392 },
	{ 0x6349, 0x6350, PDF_CMAP_TABLE, 3633 },
	{ 0x6352, 0x6355, PDF_CMAP_TABLE, 3641 },
	{ 0x6357, 0x6359, PDF_CMAP_TABLE, 3645 },
	{ 0x635b, 0x635c, PDF_CMAP_TABLE, 3648 },
	{ 0x6365, 0x6369, PDF_CMAP_TABLE, 3650 },
	{ 0x636b, 0x636e, PDF_CMAP_TABLE, 3655 },
	{ 0x6371, 0x6372, PDF_CMAP_TABLE, 3659 },
	{ 0x6374, 0x6378, PDF_CMAP_TABLE, 3661 },
	{ 0x637a, 0x637d, PDF_CMAP_TABLE, 3666 },
	{ 0x637f, 0x6380, PDF_CMAP_TABLE, 3670 },
	{ 0x6382, 0x6384, PDF_CMAP_TABLE, 3672 },
	{ 0x6387, 0x638a, PDF_CMAP_TABLE, 3675 },
	{ 0x638c, 0x638c, PDF_CMAP_SINGLE, 2464 },
	{ 0x638e, 0x6390, PDF_CMAP_TABLE, 3679 },
	{ 0x6392, 0x6392, PDF_CMAP_SINGLE, 3337 },
	{ 0x6394, 0x6396, PDF_CMAP_TABLE, 3682 },
	{ 0x6398, 0x639b, PDF_CMAP_TABLE, 3685 },
	{ 0x639e, 0x63ad, PDF_CMAP_TABLE, 3689 },
	{ 0x63ae, 0x63af, PDF_CMAP_RANGE, 19400 },
	{ 0x63b2, 0x63b2, PDF_CMAP_SINGLE, 1821 },
	{ 0x63b4, 0x63b5, PDF_CMAP_TABLE, 3705 },
	{ 0x63bb, 0x63bb, PDF_CMAP_SINGLE, 2785 },
	{ 0x63bd, 0x63be, PDF_CMAP_TABLE, 3707 },
	{ 0x63c0, 0x63c1, PDF_CMAP_TABLE, 3709 },
	{ 0x63c3, 0x63c6, PDF_CMAP_TABLE, 3711 },
	{ 0x63c8, 0x63c9, PDF_CMAP_TABLE, 3715 },
	{ 0x63ce, 0x63d3, PDF_CMAP_TABLE, 3717 },
	{ 0x63d4, 0x63d5, PDF_CMAP_RANGE, 17689 },
	{ 0x63d6, 0x63d6, PDF_CMAP_SINGLE, 3862 },
	{ 0x63da, 0x63dc, PDF_CMAP_TABLE, 3723 },
	{ 0x63e0, 0x63e1, PDF_CMAP_TABLE, 3726 },
	{ 0x63e3, 0x63e3, PDF_CMAP_SINGLE, 5004 },
	{ 0x63e5, 0x63e5, PDF_CMAP_SINGLE, 16876 },
	{ 0x63e9, 0x63ea, PDF_CMAP_TABLE, 3728 },
	{ 0x63eb, 0x63ec, PDF_CMAP_RANGE, 17692 },
	{ 0x63ed, 0x63ee, PDF_CMAP_TABLE, 3730 },
	{ 0x63f2, 0x63f7, PDF_CMAP_TABLE, 3732 },
	{ 0x63f8, 0x63f9, PDF_CMAP_RANGE, 19403 },
	{ 0x63fa, 0x63fa, PDF_CMAP_SINGLE, 3891 },
	{ 0x6406, 0x6406, PDF_CMAP_SINGLE, 5011 },
	{ 0x6409, 0x640a, PDF_CMAP_TABLE, 3738 },
	{ 0x640d, 0x640d, PDF_CMAP_SINGLE, 2843 },
	{ 0x640f, 0x6410, PDF_CMAP_TABLE, 3740 },
	{ 0x6412, 0x6414, PDF_CMAP_TABLE, 3742 },
	{ 0x6416, 0x6418, PDF_CMAP_TABLE, 3745 },
	{ 0x641c, 0x641c, PDF_CMAP_SINGLE, 4986 },
	{ 0x641e, 0x641e, PDF_CMAP_SINGLE, 17695 },
	{ 0x6420, 0x6420, PDF_CMAP_SINGLE, 19407 },
	{ 0x6422, 0x6422, PDF_CMAP_SINGLE, 14583 },
	{ 0x6424, 0x6426, PDF_CMAP_TABLE, 3748 },
	{ 0x6428, 0x642a, PDF_CMAP_TABLE, 3751 },
	{ 0x642c, 0x642d, PDF_CMAP_TABLE, 3754 },
	{ 0x642f, 0x6430, PDF_CMAP_TABLE, 3756 },
	{ 0x6434, 0x6436, PDF_CMAP_TABLE, 3758 },
	{ 0x643a, 0x643a, PDF_CMAP_SINGLE, 1822 },
	{ 0x643d, 0x643f, PDF_CMAP_TABLE, 3761 },
	{ 0x6442, 0x6442, PDF_CMAP_SINGLE, 2689 },
	{ 0x644b, 0x644b, PDF_CMAP_SINGLE, 21652 },
	{ 0x644e, 0x644f, PDF_CMAP_TABLE, 3764 },
	{ 0x6451, 0x6454, PDF_CMAP_TABLE, 3766 },
	{ 0x6458, 0x6458, PDF_CMAP_SINGLE, 3104 },
	{ 0x645a, 0x645d, PDF_CMAP_TABLE, 3770 },
	{ 0x645f, 0x6461, PDF_CMAP_TABLE, 3774 },
	{ 0x6463, 0x6463, PDF_CMAP_SINGLE, 21657 },
	{ 0x6467, 0x6467, PDF_CMAP_SINGLE, 5019 },
	{ 0x6469, 0x6469, PDF_CMAP_SINGLE, 3726 },
	{ 0x646d, 0x646d, PDF_CMAP_SINGLE, 14586 },
	{ 0x646f, 0x646f, PDF_CMAP_SINGLE, 5020 },
	{ 0x6473, 0x6474, PDF_CMAP_TABLE, 3777 },
	{ 0x6476, 0x6476, PDF_CMAP_SINGLE, 5021 },
	{ 0x6478, 0x647b, PDF_CMAP_TABLE, 3779 },
	{ 0x647d, 0x647d, PDF_CMAP_SINGLE, 17702 },
	{ 0x6483, 0x6483, PDF_CMAP_SINGLE, 1848 },
	{ 0x6485, 0x6485, PDF_CMAP_SINGLE, 21658 },
	{ 0x6487, 0x6488, PDF_CMAP_TABLE, 3783 },
	{ 0x648f, 0x6493, PDF_CMAP_TABLE, 3785 },
	{ 0x6495, 0x6495, PDF_CMAP_SINGLE, 5024 },
	{ 0x6498, 0x6499, PDF_CMAP_RANGE, 19417 },
	{ 0x649a, 0x649b, PDF_CMAP_TABLE, 3790 },
	{ 0x649d, 0x649f, PDF_CMAP_TABLE, 3792 },
	{ 0x64a1, 0x64a1, PDF_CMAP_SINGLE, 21661 },
	{ 0x64a3, 0x64a6, PDF_CMAP_TABLE, 3795 },
	{ 0x64a8, 0x64a9, PDF_CMAP_TABLE, 3799 },
	{ 0x64ab, 0x64ae, PDF_CMAP_TABLE, 3801 },
	{ 0x64b0, 0x64b0, PDF_CMAP_SINGLE, 2709 },
	{ 0x64b2, 0x64b3, PDF_CMAP_TABLE, 3805 },
	{ 0x64b9, 0x64b9, PDF_CMAP_SINGLE, 1447 },
	{ 0x64bb, 0x64bd, PDF_CMAP_TABLE, 3807 },
	{ 0x64be, 0x64bf, PDF_CMAP_RANGE, 14588 },
	{ 0x64c1, 0x64c2, PDF_CMAP_TABLE, 3810 },
	{ 0x64c4, 0x64c5, PDF_CMAP_TABLE, 3812 },
	{ 0x64c7, 0x64c7, PDF_CMAP_SINGLE, 5033 },
	{ 0x64c9, 0x64ca, PDF_CMAP_TABLE, 3814 },
	{ 0x64cb, 0x64cc, PDF_CMAP_RANGE, 17706 },
	{ 0x64cd, 0x64ce, PDF_CMAP_TABLE, 3816 },
	{ 0x64d0, 0x64d2, PDF_CMAP_TABLE, 3818 },
	{ 0x64d4, 0x64d5, PDF_CMAP_TABLE, 3821 },
	{ 0x64d7, 0x64d8, PDF_CMAP_TABLE, 3823 },
	{ 0x64da, 0x64da, PDF_CMAP_SINGLE, 5030 },
	{ 0x64e0, 0x64e1, PDF_CMAP_RANGE, 5040 },
	{ 0x64e2, 0x64e7, PDF_CMAP_TABLE, 3825 },
	{ 0x64e9, 0x64ea, PDF_CMAP_RANGE, 21668 },
	{ 0x64ec, 0x64ed, PDF_CMAP_TABLE, 3831 },
	{ 0x64ef, 0x64f2, PDF_CMAP_TABLE, 3833 },
	{ 0x64f4, 0x64f7, PDF_CMAP_TABLE, 3837 },
	{ 0x64fa, 0x64fb, PDF_CMAP_TABLE, 3841 },
	{ 0x64fd, 0x6501, PDF_CMAP_TABLE, 3843 },
	{ 0x6504, 0x6505, PDF_CMAP_TABLE, 3848 },
	{ 0x6508, 0x650a, PDF_CMAP_RANGE, 21672 },
	{ 0x650f, 0x650f, PDF_CMAP_SINGLE, 17714 },
	{ 0x6513, 0x6514, PDF_CMAP_TABLE, 3850 },
	{ 0x6516, 0x6516, PDF_CMAP_SINGLE, 14594 },
	{ 0x6518, 0x6519, PDF_CMAP_TABLE, 3852 },
	{ 0x651b, 0x651f, PDF_CMAP_TABLE, 3854 },
	{ 0x6522, 0x6524, PDF_CMAP_TABLE, 3859 },
	{ 0x6526, 0x6526, PDF_CMAP_SINGLE, 21676 },
	{ 0x6529, 0x652c, PDF_CMAP_TABLE, 3862 },
	{ 0x652e, 0x652f, PDF_CMAP_TABLE, 3866 },
	{ 0x6531, 0x6532, PDF_CMAP_TABLE, 3868 },
	{ 0x6534, 0x6535, PDF_CMAP_RANGE, 5058 },
	{ 0x6536, 0x653b, PDF_CMAP_TABLE, 3870 },
	{ 0x653c, 0x653d, PDF_CMAP_RANGE, 21679 },
	{ 0x653e, 0x653f, PDF_CMAP_TABLE, 3876 },
	{ 0x6543, 0x6545, PDF_CMAP_TABLE, 3878 },
	{ 0x6547, 0x6549, PDF_CMAP_TABLE, 3881 },
	{ 0x654d, 0x6552, PDF_CMAP_TABLE, 3884 },
	{ 0x6554, 0x6559, PDF_CMAP_TABLE, 3890 },
	{ 0x655d, 0x6560, PDF_CMAP_TABLE, 3896 },
	{ 0x6562, 0x6563, PDF_CMAP_TABLE, 3900 },
	{ 0x6566, 0x6567, PDF_CMAP_TABLE, 3902 },
	{ 0x656b, 0x656c, PDF_CMAP_TABLE, 3904 },
	{ 0x6570, 0x6570, PDF_CMAP_SINGLE, 2618 },
	{ 0x6572, 0x6572, PDF_CMAP_SINGLE, 5071 },
	{ 0x6574, 0x6575, PDF_CMAP_TABLE, 3906 },
	{ 0x6577, 0x6578, PDF_CMAP_TABLE, 3908 },
	{ 0x657a, 0x657a, PDF_CMAP_SINGLE, 17722 },
	{ 0x657d, 0x657d, PDF_CMAP_SINGLE, 21685 },
	{ 0x6581, 0x6581, PDF_CMAP_SINGLE, 14598 },
	{ 0x6582, 0x6583, PDF_CMAP_RANGE, 5073 },
	{ 0x6584, 0x6585, PDF_CMAP_TABLE, 3910 },
	{ 0x6587, 0x658a, PDF_CMAP_TABLE, 3912 },
	{ 0x658c, 0x658c, PDF_CMAP_SINGLE, 3518 },
	{ 0x658e, 0x658e, PDF_CMAP_SINGLE, 2120 },
	{ 0x6590, 0x6592, PDF_CMAP_TABLE, 3916 },
	{ 0x6595, 0x6595, PDF_CMAP_SINGLE, 19429 },
	{ 0x6597, 0x6599, PDF_CMAP_TABLE, 3919 },
	{ 0x659b, 0x659d, PDF_CMAP_TABLE, 3922 },
	{ 0x659f, 0x65a1, PDF_CMAP_TABLE, 3925 },
	{ 0x65a3, 0x65a7, PDF_CMAP_TABLE, 3928 },
	{ 0x65ab, 0x65b0, PDF_CMAP_TABLE, 3933 },
	{ 0x65b2, 0x65b5, PDF_CMAP_TABLE, 3939 },
	{ 0x65b7, 0x65b9, PDF_CMAP_TABLE, 3943 },
	{ 0x65bc, 0x65bf, PDF_CMAP_TABLE, 3946 },
	{ 0x65c1, 0x65c6, PDF_CMAP_TABLE, 3950 },
	{ 0x65c8, 0x65c9, PDF_CMAP_TABLE, 3956 },
	{ 0x65cb, 0x65cc, PDF_CMAP_TABLE, 3958 },
	{ 0x65ce, 0x65d0, PDF_CMAP_TABLE, 3960 },
	{ 0x65d2, 0x65d2, PDF_CMAP_SINGLE, 5085 },
	{ 0x65d4, 0x65d4, PDF_CMAP_SINGLE, 17730 },
	{ 0x65d6, 0x65d9, PDF_CMAP_TABLE, 3963 },
	{ 0x65db, 0x65db, PDF_CMAP_SINGLE, 5086 },
	{ 0x65df, 0x65df, PDF_CMAP_SINGLE, 19435 },
	{ 0x65e0, 0x65e1, PDF_CMAP_RANGE, 5088 },
	{ 0x65e2, 0x65e3, PDF_CMAP_TABLE, 3967 },
	{ 0x65e5, 0x65e9, PDF_CMAP_TABLE, 3969 },
	{ 0x65ec, 0x65ed, PDF_CMAP_TABLE, 3974 },
	{ 0x65f0, 0x65f2, PDF_CMAP_TABLE, 3976 },
	{ 0x65f4, 0x65f5, PDF_CMAP_RANGE, 21695 },
	{ 0x65f9, 0x65fc, PDF_CMAP_TABLE, 3979 },
	{ 0x65fe, 0x65ff, PDF_CMAP_RANGE, 21697 },
	{ 0x6600, 0x6600, PDF_CMAP_SINGLE, 8472 },
	{ 0x6602, 0x6604, PDF_CMAP_TABLE, 3983 },
	{ 0x6606, 0x660a, PDF_CMAP_TABLE, 3986 },
	{ 0x660c, 0x660f, PDF_CMAP_TABLE, 3991 },
	{ 0x6611, 0x6612, PDF_CMAP_RANGE, 21700 },
	{ 0x6613, 0x6616, PDF_CMAP_TABLE, 3995 },
	{ 0x661c, 0x6631, PDF_CMAP_TABLE, 3999 },
	{ 0x6633, 0x6634, PDF_CMAP_TABLE, 4021 },
	{ 0x6635, 0x6636, PDF_CMAP_RANGE, 5096 },
	{ 0x6637, 0x6637, PDF_CMAP_SINGLE, 21708 },
	{ 0x6639, 0x663c, PDF_CMAP_TABLE, 4023 },
	{ 0x663f, 0x6646, PDF_CMAP_TABLE, 4027 },
	{ 0x6648, 0x664c, PDF_CMAP_TABLE, 4035 },
	{ 0x664e, 0x664f, PDF_CMAP_TABLE, 4040 },
	{ 0x6651, 0x6652, PDF_CMAP_TABLE, 4042 },
	{ 0x6657, 0x665a, PDF_CMAP_TABLE, 4044 },
	{ 0x665b, 0x665c, PDF_CMAP_RANGE, 14605 },
	{ 0x665d, 0x6666, PDF_CMAP_TABLE, 4048 },
	{ 0x6667, 0x6668, PDF_CMAP_RANGE, 5107 },
	{ 0x6669, 0x666b, PDF_CMAP_TABLE, 4058 },
	{ 0x666c, 0x666d, PDF_CMAP_RANGE, 17746 },
	{ 0x666e, 0x6670, PDF_CMAP_TABLE, 4061 },
	{ 0x6673, 0x6676, PDF_CMAP_TABLE, 4064 },
	{ 0x6677, 0x6678, PDF_CMAP_RANGE, 16889 },
	{ 0x6679, 0x667c, PDF_CMAP_TABLE, 4068 },
	{ 0x667e, 0x6681, PDF_CMAP_TABLE, 4072 },
	{ 0x6683, 0x6684, PDF_CMAP_TABLE, 4076 },
	{ 0x6687, 0x6689, PDF_CMAP_TABLE, 4078 },
	{ 0x668b, 0x668c, PDF_CMAP_RANGE, 19437 },
	{ 0x668d, 0x668e, PDF_CMAP_TABLE, 4081 },
	{ 0x6690, 0x6692, PDF_CMAP_TABLE, 4083 },
	{ 0x6696, 0x6699, PDF_CMAP_TABLE, 4086 },
	{ 0x669a, 0x669c, PDF_CMAP_RANGE, 21718 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 5118 },
	{ 0x669f, 0x66a0, PDF_CMAP_TABLE, 4090 },
	{ 0x66a2, 0x66a2, PDF_CMAP_SINGLE, 3014 },
	{ 0x66a4, 0x66a4, PDF_CMAP_SINGLE, 14610 },
	{ 0x66a6, 0x66a6, PDF_CMAP_SINGLE, 4025 },
	{ 0x66ab, 0x66ab, PDF_CMAP_SINGLE, 2193 },
	{ 0x66ad, 0x66ae, PDF_CMAP_TABLE, 4092 },
	{ 0x66b1, 0x66b5, PDF_CMAP_TABLE, 4094 },
	{ 0x66b8, 0x66b9, PDF_CMAP_TABLE, 4099 },
	{ 0x66bb, 0x66bc, PDF_CMAP_TABLE, 4101 },
	{ 0x66be, 0x66c1, PDF_CMAP_TABLE, 4103 },
	{ 0x66c2, 0x66c3, PDF_CMAP_RANGE, 21723 },
	{ 0x66c4, 0x66c4, PDF_CMAP_SINGLE, 5124 },
	{ 0x66c6, 0x66c9, PDF_CMAP_TABLE, 4107 },
	{ 0x66cc, 0x66cc, PDF_CMAP_SINGLE, 21725 },
	{ 0x66ce, 0x66cf, PDF_CMAP_TABLE, 4111 },
	{ 0x66d4, 0x66d4, PDF_CMAP_SINGLE, 21727 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 5126 },
	{ 0x66d9, 0x66dd, PDF_CMAP_TABLE, 4113 },
	{ 0x66df, 0x66e0, PDF_CMAP_TABLE, 4118 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 5130 },
	{ 0x66e8, 0x66e9, PDF_CMAP_TABLE, 4120 },
	{ 0x66eb, 0x66ec, PDF_CMAP_TABLE, 4122 },
	{ 0x66ee, 0x66ee, PDF_CMAP_SINGLE, 21730 },
	{ 0x66f0, 0x66f0, PDF_CMAP_SINGLE, 5132 },
	{ 0x66f2, 0x66f5, PDF_CMAP_TABLE, 4124 },
	{ 0x66f7, 0x6701, PDF_CMAP_TABLE, 4128 },
	{ 0x6703, 0x6703, PDF_CMAP_SINGLE, 4171 },
	{ 0x6705, 0x6705, PDF_CMAP_SINGLE, 14613 },
	{ 0x6707, 0x6709, PDF_CMAP_TABLE, 4139 },
	{ 0x670b, 0x670b, PDF_CMAP_SINGLE, 3662 },
	{ 0x670d, 0x670f, PDF_CMAP_TABLE, 4142 },
	{ 0x6712, 0x6717, PDF_CMAP_TABLE, 4145 },
	{ 0x6719, 0x6719, PDF_CMAP_SINGLE, 17762 },
	{ 0x671b, 0x6720, PDF_CMAP_TABLE, 4151 },
	{ 0x6722, 0x6722, PDF_CMAP_SINGLE, 21734 },
	{ 0x6725, 0x6725, PDF_CMAP_SINGLE, 19442 },
	{ 0x6726, 0x6727, PDF_CMAP_RANGE, 5138 },
	{ 0x6728, 0x6728, PDF_CMAP_SINGLE, 3814 },
	{ 0x672a, 0x672e, PDF_CMAP_TABLE, 4157 },
	{ 0x6731, 0x6731, PDF_CMAP_SINGLE, 2327 },
	{ 0x6733, 0x6738, PDF_CMAP_TABLE, 4162 },
	{ 0x673a, 0x673a, PDF_CMAP_SINGLE, 1589 },
	{ 0x673d, 0x673f, PDF_CMAP_TABLE, 4168 },
	{ 0x6741, 0x6741, PDF_CMAP_SINGLE, 5144 },
	{ 0x6743, 0x6743, PDF_CMAP_SINGLE, 13751 },
	{ 0x6745, 0x6749, PDF_CMAP_TABLE, 4171 },
	{ 0x674c, 0x6751, PDF_CMAP_TABLE, 4176 },
	{ 0x6753, 0x6756, PDF_CMAP_TABLE, 4182 },
	{ 0x6759, 0x6759, PDF_CMAP_SINGLE, 5150 },
	{ 0x675c, 0x6762, PDF_CMAP_TABLE, 4186 },
	{ 0x6763, 0x6764, PDF_CMAP_RANGE, 5151 },
	{ 0x6765, 0x6766, PDF_CMAP_TABLE, 4193 },
	{ 0x676a, 0x676a, PDF_CMAP_SINGLE, 5157 },
	{ 0x676c, 0x6777, PDF_CMAP_TABLE, 4195 },
	{ 0x677b, 0x677c, PDF_CMAP_TABLE, 4207 },
	{ 0x677e, 0x6781, PDF_CMAP_TABLE, 4209 },
	{ 0x6784, 0x6785, PDF_CMAP_TABLE, 4213 },
	{ 0x6787, 0x6787, PDF_CMAP_SINGLE, 3470 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 5153 },
	{ 0x678b, 0x678c, PDF_CMAP_TABLE, 4215 },
	{ 0x678e, 0x6793, PDF_CMAP_TABLE, 4217 },
	{ 0x6795, 0x679d, PDF_CMAP_TABLE, 4223 },
	{ 0x67a0, 0x67a2, PDF_CMAP_TABLE, 4232 },
	{ 0x67a4, 0x67a4, PDF_CMAP_SINGLE, 19448 },
	{ 0x67a6, 0x67a6, PDF_CMAP_SINGLE, 5160 },
	{ 0x67a9, 0x67a9, PDF_CMAP_SINGLE, 5155 },
	{ 0x67af, 0x67b9, PDF_CMAP_TABLE, 4235 },
	{ 0x67bb, 0x67bb, PDF_CMAP_SINGLE, 8491 },
	{ 0x67bc, 0x67bd, PDF_CMAP_RANGE, 21742 },
	{ 0x67be, 0x67be, PDF_CMAP_SINGLE, 19451 },
	{ 0x67c0, 0x67c6, PDF_CMAP_TABLE, 4246 },
	{ 0x67c8, 0x67ca, PDF_CMAP_TABLE, 4253 },
	{ 0x67ce, 0x67d4, PDF_CMAP_TABLE, 4256 },
	{ 0x67d7, 0x67de, PDF_CMAP_TABLE, 4263 },
	{ 0x67e1, 0x67e2, PDF_CMAP_TABLE, 4271 },
	{ 0x67e4, 0x67e4, PDF_CMAP_SINGLE, 5170 },
	{ 0x67e6, 0x67e7, PDF_CMAP_TABLE, 4273 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 5168 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 5166 },
	{ 0x67ee, 0x67f7, PDF_CMAP_TABLE, 4275 },
	{ 0x67f9, 0x67fc, PDF_CMAP_TABLE, 4285 },
	{ 0x67fe, 0x67ff, PDF_CMAP_TABLE, 4289 },
	{ 0x6801, 0x6805, PDF_CMAP_TABLE, 4291 },
	{ 0x6810, 0x6810, PDF_CMAP_SINGLE, 17776 },
	{ 0x6813, 0x6814, PDF_CMAP_TABLE, 4296 },
	{ 0x6816, 0x6819, PDF_CMAP_TABLE, 4298 },
	{ 0x681d, 0x681f, PDF_CMAP_TABLE, 4302 },
	{ 0x6821, 0x6822, PDF_CMAP_TABLE, 4305 },
	{ 0x6827, 0x682d, PDF_CMAP_TABLE, 4307 },
	{ 0x682f, 0x682f, PDF_CMAP_SINGLE, 21755 },
	{ 0x6830, 0x6831, PDF_CMAP_RANGE, 14627 },
	{ 0x6832, 0x6834, PDF_CMAP_TABLE, 4314 },
	{ 0x6838, 0x6839, PDF_CMAP_TABLE, 4317 },
	{ 0x683b, 0x6846, PDF_CMAP_TABLE, 4319 },
	{ 0x6848, 0x684a, PDF_CMAP_TABLE, 4331 },
	{ 0x684c, 0x684e, PDF_CMAP_TABLE, 4334 },
	{ 0x6850, 0x6855, PDF_CMAP_TABLE, 4337 },
	{ 0x6857, 0x6859, PDF_CMAP_TABLE, 4343 },
	{ 0x685b, 0x685d, PDF_CMAP_TABLE, 4346 },
	{ 0x685f, 0x685f, PDF_CMAP_SINGLE, 2181 },
	{ 0x6863, 0x6863, PDF_CMAP_SINGLE, 5190 },
	{ 0x6867, 0x6867, PDF_CMAP_SINGLE, 3490 },
	{ 0x686b, 0x686b, PDF_CMAP_SINGLE, 17798 },
	{ 0x686e, 0x686e, PDF_CMAP_SINGLE, 17799 },
	{ 0x686f, 0x6871, PDF_CMAP_RANGE, 21759 },
	{ 0x6872, 0x6872, PDF_CMAP_SINGLE, 14630 },
	{ 0x6874, 0x6877, PDF_CMAP_TABLE, 4349 },
	{ 0x6879, 0x687c, PDF_CMAP_TABLE, 4353 },
	{ 0x687e, 0x687f, PDF_CMAP_TABLE, 4357 },
	{ 0x6881, 0x6886, PDF_CMAP_TABLE, 4359 },
	{ 0x6888, 0x6888, PDF_CMAP_SINGLE, 21764 },
	{ 0x688d, 0x6890, PDF_CMAP_TABLE, 4365 },
	{ 0x6893, 0x6894, PDF_CMAP_TABLE, 4369 },
	{ 0x6896, 0x6897, PDF_CMAP_TABLE, 4371 },
	{ 0x6898, 0x689a, PDF_CMAP_RANGE, 17805 },
	{ 0x689b, 0x689d, PDF_CMAP_TABLE, 4373 },
	{ 0x689f, 0x68a3, PDF_CMAP_TABLE, 4376 },
	{ 0x68a5, 0x68a9, PDF_CMAP_TABLE, 4381 },
	{ 0x68aa, 0x68ab, PDF_CMAP_RANGE, 17809 },
	{ 0x68ad, 0x68b6, PDF_CMAP_TABLE, 4386 },
	{ 0x68b9, 0x68bc, PDF_CMAP_TABLE, 4396 },
	{ 0x68c3, 0x68c6, PDF_CMAP_TABLE, 4400 },
	{ 0x68c8, 0x68cd, PDF_CMAP_TABLE, 4404 },
	{ 0x68cf, 0x68da, PDF_CMAP_TABLE, 4410 },
	{ 0x68dc, 0x68dd, PDF_CMAP_RANGE, 21770 },
	{ 0x68df, 0x68e1, PDF_CMAP_TABLE, 4422 },
	{ 0x68e3, 0x68e3, PDF_CMAP_SINGLE, 5225 },
	{ 0x68e4, 0x68e5, PDF_CMAP_RANGE, 17820 },
	{ 0x68e7, 0x68e8, PDF_CMAP_TABLE, 4425 },
	{ 0x68ea, 0x68ef, PDF_CMAP_TABLE, 4427 },
	{ 0x68f0, 0x68f1, PDF_CMAP_RANGE, 14640 },
	{ 0x68f2, 0x68f2, PDF_CMAP_SINGLE, 2647 },
	{ 0x68f5, 0x68f7, PDF_CMAP_TABLE, 4433 },
	{ 0x68f9, 0x68fd, PDF_CMAP_TABLE, 4436 },
	{ 0x6900, 0x6901, PDF_CMAP_TABLE, 4441 },
	{ 0x6903, 0x6913, PDF_CMAP_TABLE, 4443 },
	{ 0x6916, 0x6917, PDF_CMAP_TABLE, 4460 },
	{ 0x6919, 0x691c, PDF_CMAP_TABLE, 4462 },
	{ 0x6921, 0x6923, PDF_CMAP_TABLE, 4466 },
	{ 0x6925, 0x6926, PDF_CMAP_TABLE, 4469 },
	{ 0x6928, 0x6928, PDF_CMAP_SINGLE, 5230 },
	{ 0x692a, 0x692a, PDF_CMAP_SINGLE, 5231 },
	{ 0x6930, 0x6931, PDF_CMAP_TABLE, 4471 },
	{ 0x6933, 0x6936, PDF_CMAP_TABLE, 4473 },
	{ 0x6938, 0x6939, PDF_CMAP_TABLE, 4477 },
	{ 0x693b, 0x693b, PDF_CMAP_SINGLE, 14646 },
	{ 0x693d, 0x693d, PDF_CMAP_SINGLE, 5246 },
	{ 0x693f, 0x693f, PDF_CMAP_SINGLE, 3060 },
	{ 0x6942, 0x6942, PDF_CMAP_SINGLE, 16906 },
	{ 0x6945, 0x6946, PDF_CMAP_TABLE, 4479 },
	{ 0x6949, 0x694a, PDF_CMAP_TABLE, 4481 },
	{ 0x694e, 0x694e, PDF_CMAP_SINGLE, 21781 },
	{ 0x6953, 0x6955, PDF_CMAP_TABLE, 4483 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 14647 },
	{ 0x6959, 0x695e, PDF_CMAP_TABLE, 4486 },
	{ 0x6960, 0x6966, PDF_CMAP_TABLE, 4492 },
	{ 0x6968, 0x696f, PDF_CMAP_TABLE, 4499 },
	{ 0x6970, 0x6971, PDF_CMAP_RANGE, 21783 },
	{ 0x6972, 0x6975, PDF_CMAP_TABLE, 4507 },
	{ 0x6977, 0x697e, PDF_CMAP_TABLE, 4511 },
	{ 0x697f, 0x6980, PDF_CMAP_RANGE, 14650 },
	{ 0x6981, 0x6982, PDF_CMAP_TABLE, 4519 },
	{ 0x6986, 0x6986, PDF_CMAP_SINGLE, 14144 },
	{ 0x698a, 0x698a, PDF_CMAP_SINGLE, 2135 },
	{ 0x698d, 0x698e, PDF_CMAP_TABLE, 4521 },
	{ 0x6991, 0x6992, PDF_CMAP_TABLE, 4523 },
	{ 0x6994, 0x6996, PDF_CMAP_TABLE, 4525 },
	{ 0x6998, 0x6998, PDF_CMAP_SINGLE, 8500 },
	{ 0x699b, 0x699c, PDF_CMAP_TABLE, 4528 },
	{ 0x69a0, 0x69a1, PDF_CMAP_TABLE, 4530 },
	{ 0x69a5, 0x69a8, PDF_CMAP_TABLE, 4532 },
	{ 0x69ab, 0x69ab, PDF_CMAP_SINGLE, 19462 },
	{ 0x69ad, 0x69b2, PDF_CMAP_TABLE, 4536 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 5273 },
	{ 0x69b7, 0x69b8, PDF_CMAP_TABLE, 4542 },
	{ 0x69ba, 0x69bc, PDF_CMAP_TABLE, 4544 },
	{ 0x69be, 0x69c1, PDF_CMAP_TABLE, 4547 },
	{ 0x69c3, 0x69c3, PDF_CMAP_SINGLE, 5266 },
	{ 0x69c5, 0x69c5, PDF_CMAP_SINGLE, 21789 },
	{ 0x69c7, 0x69c8, PDF_CMAP_TABLE, 4551 },
	{ 0x69ca, 0x69d1, PDF_CMAP_TABLE, 4553 },
	{ 0x69d3, 0x69d3, PDF_CMAP_SINGLE, 5259 },
	{ 0x69d6, 0x69d7, PDF_CMAP_RANGE, 14655 },
	{ 0x69d8, 0x69d9, PDF_CMAP_TABLE, 4561 },
	{ 0x69dd, 0x69de, PDF_CMAP_TABLE, 4563 },
	{ 0x69e2, 0x69e3, PDF_CMAP_TABLE, 4565 },
	{ 0x69e5, 0x69e5, PDF_CMAP_SINGLE, 19464 },
	{ 0x69e7, 0x69eb, PDF_CMAP_TABLE, 4567 },
	{ 0x69ed, 0x69ed, PDF_CMAP_SINGLE, 5286 },
	{ 0x69ee, 0x69ef, PDF_CMAP_RANGE, 17845 },
	{ 0x69f1, 0x69f6, PDF_CMAP_TABLE, 4572 },
	{ 0x69f9, 0x69f9, PDF_CMAP_SINGLE, 5280 },
	{ 0x69fb, 0x69fb, PDF_CMAP_SINGLE, 3052 },
	{ 0x69fd, 0x6a03, PDF_CMAP_TABLE, 4578 },
	{ 0x6a05, 0x6a05, PDF_CMAP_SINGLE, 5283 },
	{ 0x6a0a, 0x6a0c, PDF_CMAP_TABLE, 4585 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 14658 },
	{ 0x6a11, 0x6a15, PDF_CMAP_TABLE, 4588 },
	{ 0x6a17, 0x6a17, PDF_CMAP_SINGLE, 2994 },
	{ 0x6a19, 0x6a1b, PDF_CMAP_TABLE, 4593 },
	{ 0x6a1d, 0x6a24, PDF_CMAP_TABLE, 4596 },
	{ 0x6a28, 0x6a2b, PDF_CMAP_TABLE, 4604 },
	{ 0x6a2e, 0x6a2e, PDF_CMAP_SINGLE, 5268 },
	{ 0x6a30, 0x6a30, PDF_CMAP_SINGLE, 8502 },
	{ 0x6a32, 0x6a33, PDF_CMAP_RANGE, 17855 },
	{ 0x6a34, 0x6a3b, PDF_CMAP_TABLE, 4608 },
	{ 0x6a3d, 0x6a3f, PDF_CMAP_TABLE, 4616 },
	{ 0x6a44, 0x6a4b, PDF_CMAP_TABLE, 4619 },
	{ 0x6a4e, 0x6a4e, PDF_CMAP_SINGLE, 17860 },
	{ 0x6a50, 0x6a51, PDF_CMAP_RANGE, 14664 },
	{ 0x6a52, 0x6a52, PDF_CMAP_SINGLE, 17861 },
	{ 0x6a54, 0x6a56, PDF_CMAP_TABLE, 4627 },
	{ 0x6a58, 0x6a59, PDF_CMAP_TABLE, 4630 },
	{ 0x6a5b, 0x6a5b, PDF_CMAP_SINGLE, 14667 },
	{ 0x6a5f, 0x6a5f, PDF_CMAP_SINGLE, 1595 },
	{ 0x6a61, 0x6a62, PDF_CMAP_TABLE, 4632 },
	{ 0x6a64, 0x6a64, PDF_CMAP_SINGLE, 17862 },
	{ 0x6a66, 0x6a67, PDF_CMAP_TABLE, 4634 },
	{ 0x6a6a, 0x6a6b, PDF_CMAP_TABLE, 4636 },
	{ 0x6a71, 0x6a73, PDF_CMAP_TABLE, 4638 },
	{ 0x6a78, 0x6a78, PDF_CMAP_SINGLE, 5298 },
	{ 0x6a7a, 0x6a7a, PDF_CMAP_SINGLE, 17859 },
	{ 0x6a7e, 0x6a81, PDF_CMAP_TABLE, 4641 },
	{ 0x6a83, 0x6a84, PDF_CMAP_TABLE, 4645 },
	{ 0x6a86, 0x6a87, PDF_CMAP_RANGE, 21800 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 14669 },
	{ 0x6a8b, 0x6a8b, PDF_CMAP_SINGLE, 17864 },
	{ 0x6a8d, 0x6a8e, PDF_CMAP_TABLE, 4647 },
	{ 0x6a90, 0x6a91, PDF_CMAP_TABLE, 4649 },
	{ 0x6a94, 0x6a94, PDF_CMAP_SINGLE, 16917 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 5312 },
	{ 0x6a9b, 0x6a9c, PDF_CMAP_TABLE, 4651 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 14671 },
	{ 0x6aa0, 0x6aa1, PDF_CMAP_TABLE, 4653 },
	{ 0x6aa2, 0x6aa3, PDF_CMAP_RANGE, 5310 },
	{ 0x6aa5, 0x6aa5, PDF_CMAP_SINGLE, 16918 },
	{ 0x6aaa, 0x6aac, PDF_CMAP_TABLE, 4655 },
	{ 0x6aae, 0x6aaf, PDF_CMAP_TABLE, 4658 },
	{ 0x6ab0, 0x6ab1, PDF_CMAP_RANGE, 21803 },
	{ 0x6ab3, 0x6ab4, PDF_CMAP_TABLE, 4660 },
	{ 0x6ab8, 0x6ab8, PDF_CMAP_SINGLE, 5317 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 5314 },
	{ 0x6abd, 0x6abd, PDF_CMAP_SINGLE, 17869 },
	{ 0x6abe, 0x6abf, PDF_CMAP_RANGE, 21806 },
	{ 0x6ac1, 0x6ac3, PDF_CMAP_TABLE, 4662 },
	{ 0x6ac6, 0x6ac6, PDF_CMAP_SINGLE, 17870 },
	{ 0x6ac8, 0x6ac9, PDF_CMAP_RANGE, 19471 },
	{ 0x6acc, 0x6acc, PDF_CMAP_SINGLE, 21808 },
	{ 0x6ad0, 0x6ad1, PDF_CMAP_TABLE, 4665 },
	{ 0x6ad3, 0x6ad4, PDF_CMAP_TABLE, 4667 },
	{ 0x6ad5, 0x6ad6, PDF_CMAP_RANGE, 21809 },
	{ 0x6ada, 0x6adf, PDF_CMAP_TABLE, 4669 },
	{ 0x6ae2, 0x6ae2, PDF_CMAP_SINGLE, 8507 },
	{ 0x6ae4, 0x6ae4, PDF_CMAP_SINGLE, 8508 },
	{ 0x6ae7, 0x6ae8, PDF_CMAP_TABLE, 4675 },
	{ 0x6aea, 0x6aea, PDF_CMAP_SINGLE, 5325 },
	{ 0x6aec, 0x6aec, PDF_CMAP_SINGLE, 14676 },
	{ 0x6af0, 0x6af0, PDF_CMAP_SINGLE, 21811 },
	{ 0x6af1, 0x6af3, PDF_CMAP_RANGE, 17876 },
	{ 0x6af8, 0x6af8, PDF_CMAP_SINGLE, 20152 },
	{ 0x6afa, 0x6afd, PDF_CMAP_TABLE, 4677 },
	{ 0x6b02, 0x6b05, PDF_CMAP_TABLE, 4681 },
	{ 0x6b06, 0x6b07, PDF_CMAP_RANGE, 21814 },
	{ 0x6b09, 0x6b0b, PDF_CMAP_TABLE, 4685 },
	{ 0x6b0f, 0x6b11, PDF_CMAP_RANGE, 17882 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 5330 },
	{ 0x6b16, 0x6b17, PDF_CMAP_TABLE, 4688 },
	{ 0x6b1b, 0x6b1b, PDF_CMAP_SINGLE, 16920 },
	{ 0x6b1d, 0x6b21, PDF_CMAP_TABLE, 4690 },
	{ 0x6b23, 0x6b24, PDF_CMAP_TABLE, 4695 },
	{ 0x6b27, 0x6b28, PDF_CMAP_TABLE, 4697 },
	{ 0x6b2b, 0x6b2c, PDF_CMAP_TABLE, 4699 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 17888 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 3913 },
	{ 0x6b35, 0x6b3b, PDF_CMAP_TABLE, 4701 },
	{ 0x6b3d, 0x6b3f, PDF_CMAP_TABLE, 4708 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 5340 },
	{ 0x6b46, 0x6b47, PDF_CMAP_TABLE, 4711 },
	{ 0x6b49, 0x6b4a, PDF_CMAP_TABLE, 4713 },
	{ 0x6b4c, 0x6b4e, PDF_CMAP_TABLE, 4715 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 5342 },
	{ 0x6b52, 0x6b54, PDF_CMAP_TABLE, 4718 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 14681 },
	{ 0x6b58, 0x6b59, PDF_CMAP_TABLE, 4721 },
	{ 0x6b5b, 0x6b5b, PDF_CMAP_SINGLE, 5345 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 21822 },
	{ 0x6b5f, 0x6b67, PDF_CMAP_TABLE, 4723 },
	{ 0x6b69, 0x6b6c, PDF_CMAP_TABLE, 4732 },
	{ 0x6b6e, 0x6b70, PDF_CMAP_TABLE, 4736 },
	{ 0x6b72, 0x6b75, PDF_CMAP_TABLE, 4739 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 13398 },
	{ 0x6b78, 0x6b79, PDF_CMAP_RANGE, 5348 },
	{ 0x6b7a, 0x6b7b, PDF_CMAP_TABLE, 4743 },
	{ 0x6b7d, 0x6b7e, PDF_CMAP_RANGE, 19476 },
	{ 0x6b7f, 0x6b80, PDF_CMAP_RANGE, 5350 },
	{ 0x6b81, 0x6b86, PDF_CMAP_TABLE, 4745 },
	{ 0x6b89, 0x6b8b, PDF_CMAP_TABLE, 4751 },
	{ 0x6b8d, 0x6b8d, PDF_CMAP_SINGLE, 5354 },
	{ 0x6b95, 0x6b98, PDF_CMAP_TABLE, 4754 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 17895 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 5357 },
	{ 0x6b9f, 0x6ba0, PDF_CMAP_RANGE, 21828 },
	{ 0x6ba2, 0x6ba3, PDF_CMAP_RANGE, 21830 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 5358 },
	{ 0x6ba8, 0x6ba9, PDF_CMAP_TABLE, 4758 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 5359 },
	{ 0x6bac, 0x6bb5, PDF_CMAP_TABLE, 4760 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 5365 },
	{ 0x6bb8, 0x6bb9, PDF_CMAP_RANGE, 21834 },
	{ 0x6bba, 0x6bc0, PDF_CMAP_TABLE, 4770 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 21836 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_TABLE, 4777 },
	{ 0x6bc7, 0x6bc9, PDF_CMAP_RANGE, 17899 },
	{ 0x6bcb, 0x6bcf, PDF_CMAP_TABLE, 4779 },
	{ 0x6bd2, 0x6bd4, PDF_CMAP_TABLE, 4784 },
	{ 0x6bd6, 0x6bd8, PDF_CMAP_TABLE, 4787 },
	{ 0x6bda, 0x6bdb, PDF_CMAP_TABLE, 4790 },
	{ 0x6bdf, 0x6bdf, PDF_CMAP_SINGLE, 5370 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 14685 },
	{ 0x6be3, 0x6be3, PDF_CMAP_SINGLE, 21838 },
	{ 0x6be6, 0x6be7, PDF_CMAP_RANGE, 17903 },
	{ 0x6beb, 0x6bec, PDF_CMAP_TABLE, 4792 },
	{ 0x6bee, 0x6bef, PDF_CMAP_TABLE, 4794 },
	{ 0x6bf1, 0x6bf1, PDF_CMAP_SINGLE, 14686 },
	{ 0x6bf3, 0x6bf3, PDF_CMAP_SINGLE, 5373 },
	{ 0x6bf7, 0x6bf7, PDF_CMAP_SINGLE, 19479 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 19480 },
	{ 0x6bff, 0x6bff, PDF_CMAP_SINGLE, 16926 },
	{ 0x6c02, 0x6c02, PDF_CMAP_SINGLE, 17906 },
	{ 0x6c04, 0x6c05, PDF_CMAP_TABLE, 4796 },
	{ 0x6c08, 0x6c0a, PDF_CMAP_TABLE, 4798 },
	{ 0x6c0d, 0x6c12, PDF_CMAP_TABLE, 4801 },
	{ 0x6c13, 0x6c14, PDF_CMAP_RANGE, 5377 },
	{ 0x6c17, 0x6c17, PDF_CMAP_SINGLE, 1598 },
	{ 0x6c19, 0x6c19, PDF_CMAP_SINGLE, 21840 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 5379 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 21841 },
	{ 0x6c23, 0x6c24, PDF_CMAP_TABLE, 4807 },
	{ 0x6c26, 0x6c28, PDF_CMAP_RANGE, 21842 },
	{ 0x6c2c, 0x6c2c, PDF_CMAP_SINGLE, 19484 },
	{ 0x6c2e, 0x6c2e, PDF_CMAP_SINGLE, 21845 },
	{ 0x6c33, 0x6c38, PDF_CMAP_TABLE, 4809 },
	{ 0x6c3a, 0x6c3b, PDF_CMAP_TABLE, 4815 },
	{ 0x6c3e, 0x6c42, PDF_CMAP_TABLE, 4817 },
	{ 0x6c4a, 0x6c4b, PDF_CMAP_TABLE, 4822 },
	{ 0x6c4d, 0x6c50, PDF_CMAP_TABLE, 4824 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 19486 },
	{ 0x6c54, 0x6c55, PDF_CMAP_TABLE, 4828 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 1532 },
	{ 0x6c59, 0x6c60, PDF_CMAP_TABLE, 4830 },
	{ 0x6c62, 0x6c62, PDF_CMAP_SINGLE, 5384 },
	{ 0x6c67, 0x6c68, PDF_CMAP_TABLE, 4838 },
	{ 0x6c6a, 0x6c6b, PDF_CMAP_TABLE, 4840 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 17913 },
	{ 0x6c6f, 0x6c70, PDF_CMAP_TABLE, 4842 },
	{ 0x6c72, 0x6c74, PDF_CMAP_TABLE, 4844 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 14692 },
	{ 0x6c78, 0x6c7b, PDF_CMAP_TABLE, 4847 },
	{ 0x6c7d, 0x6c7e, PDF_CMAP_TABLE, 4851 },
	{ 0x6c81, 0x6c89, PDF_CMAP_TABLE, 4853 },
	{ 0x6c8c, 0x6c8d, PDF_CMAP_TABLE, 4862 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 5395 },
	{ 0x6c92, 0x6c9c, PDF_CMAP_TABLE, 4864 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 21852 },
	{ 0x6ca1, 0x6ca2, PDF_CMAP_TABLE, 4875 },
	{ 0x6caa, 0x6cae, PDF_CMAP_TABLE, 4877 },
	{ 0x6cb0, 0x6cb4, PDF_CMAP_TABLE, 4882 },
	{ 0x6cb8, 0x6cbf, PDF_CMAP_TABLE, 4887 },
	{ 0x6cc1, 0x6cc2, PDF_CMAP_TABLE, 4895 },
	{ 0x6cc4, 0x6cc6, PDF_CMAP_TABLE, 4897 },
	{ 0x6cc9, 0x6cca, PDF_CMAP_TABLE, 4900 },
	{ 0x6ccc, 0x6ccd, PDF_CMAP_TABLE, 4902 },
	{ 0x6ccf, 0x6cd7, PDF_CMAP_TABLE, 4904 },
	{ 0x6cd9, 0x6cdd, PDF_CMAP_TABLE, 4913 },
	{ 0x6ce0, 0x6ce3, PDF_CMAP_TABLE, 4918 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 3103 },
	{ 0x6ce7, 0x6cea, PDF_CMAP_TABLE, 4922 },
	{ 0x6ceb, 0x6cec, PDF_CMAP_RANGE, 14701 },
	{ 0x6ced, 0x6cf4, PDF_CMAP_TABLE, 4926 },
	{ 0x6cfb, 0x6cfb, PDF_CMAP_SINGLE, 16930 },
	{ 0x6d00, 0x6d01, PDF_CMAP_TABLE, 4934 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 8515 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 21861 },
	{ 0x6d0a, 0x6d0c, PDF_CMAP_TABLE, 4936 },
	{ 0x6d0e, 0x6d0f, PDF_CMAP_TABLE, 4939 },
	{ 0x6d11, 0x6d13, PDF_CMAP_TABLE, 4941 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 2714 },
	{ 0x6d19, 0x6d1b, PDF_CMAP_TABLE, 4944 },
	{ 0x6d1e, 0x6d1f, PDF_CMAP_TABLE, 4947 },
	{ 0x6d24, 0x6d25, PDF_CMAP_TABLE, 4949 },
	{ 0x6d26, 0x6d27, PDF_CMAP_RANGE, 17928 },
	{ 0x6d28, 0x6d2b, PDF_CMAP_TABLE, 4951 },
	{ 0x6d2e, 0x6d2f, PDF_CMAP_TABLE, 4955 },
	{ 0x6d31, 0x6d36, PDF_CMAP_TABLE, 4957 },
	{ 0x6d38, 0x6d39, PDF_CMAP_TABLE, 4963 },
	{ 0x6d3b, 0x6d3f, PDF_CMAP_TABLE, 4965 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 3958 },
	{ 0x6d44, 0x6d45, PDF_CMAP_TABLE, 4970 },
	{ 0x6d57, 0x6d5c, PDF_CMAP_TABLE, 4972 },
	{ 0x6d5e, 0x6d61, PDF_CMAP_TABLE, 4978 },
	{ 0x6d63, 0x6d67, PDF_CMAP_TABLE, 4982 },
	{ 0x6d69, 0x6d6a, PDF_CMAP_TABLE, 4987 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 1435 },
	{ 0x6d6e, 0x6d70, PDF_CMAP_TABLE, 4989 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 3915 },
	{ 0x6d77, 0x6d79, PDF_CMAP_TABLE, 4992 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 19493 },
	{ 0x6d80, 0x6d81, PDF_CMAP_RANGE, 17936 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 14711 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 5431 },
	{ 0x6d87, 0x6d8a, PDF_CMAP_TABLE, 4995 },
	{ 0x6d8c, 0x6d8e, PDF_CMAP_TABLE, 4999 },
	{ 0x6d91, 0x6d99, PDF_CMAP_TABLE, 5002 },
	{ 0x6d9b, 0x6d9c, PDF_CMAP_TABLE, 5011 },
	{ 0x6daa, 0x6dac, PDF_CMAP_TABLE, 5013 },
	{ 0x6dae, 0x6daf, PDF_CMAP_TABLE, 5016 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 1271 },
	{ 0x6db4, 0x6db5, PDF_CMAP_TABLE, 5018 },
	{ 0x6db7, 0x6db9, PDF_CMAP_TABLE, 5020 },
	{ 0x6dbc, 0x6dbd, PDF_CMAP_TABLE, 5023 },
	{ 0x6dbf, 0x6dc0, PDF_CMAP_TABLE, 5025 },
	{ 0x6dc2, 0x6dc2, PDF_CMAP_SINGLE, 17949 },
	{ 0x6dc4, 0x6dc8, PDF_CMAP_TABLE, 5027 },
	{ 0x6dca, 0x6dcc, PDF_CMAP_TABLE, 5032 },
	{ 0x6dce, 0x6dd2, PDF_CMAP_TABLE, 5035 },
	{ 0x6dd5, 0x6dd6, PDF_CMAP_TABLE, 5040 },
	{ 0x6dd8, 0x6ddb, PDF_CMAP_TABLE, 5042 },
	{ 0x6ddd, 0x6de2, PDF_CMAP_TABLE, 5046 },
	{ 0x6de4, 0x6de6, PDF_CMAP_TABLE, 5052 },
	{ 0x6de8, 0x6dec, PDF_CMAP_TABLE, 5055 },
	{ 0x6dee, 0x6dfc, PDF_CMAP_TABLE, 5060 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 21877 },
	{ 0x6e04, 0x6e05, PDF_CMAP_TABLE, 5075 },
	{ 0x6e07, 0x6e0b, PDF_CMAP_TABLE, 5077 },
	{ 0x6e13, 0x6e13, PDF_CMAP_SINGLE, 1826 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 5433 },
	{ 0x6e17, 0x6e17, PDF_CMAP_SINGLE, 14150 },
	{ 0x6e19, 0x6e1b, PDF_CMAP_TABLE, 5082 },
	{ 0x6e1d, 0x6e27, PDF_CMAP_TABLE, 5085 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 1337 },
	{ 0x6e2b, 0x6e2f, PDF_CMAP_TABLE, 5096 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 17958 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 13330 },
	{ 0x6e36, 0x6e36, PDF_CMAP_SINGLE, 17955 },
	{ 0x6e38, 0x6e3c, PDF_CMAP_TABLE, 5101 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 5458 },
	{ 0x6e42, 0x6e45, PDF_CMAP_TABLE, 5106 },
	{ 0x6e48, 0x6e49, PDF_CMAP_RANGE, 17959 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 3767 },
	{ 0x6e4b, 0x6e4c, PDF_CMAP_RANGE, 17961 },
	{ 0x6e4d, 0x6e4f, PDF_CMAP_TABLE, 5110 },
	{ 0x6e51, 0x6e52, PDF_CMAP_TABLE, 5113 },
	{ 0x6e53, 0x6e54, PDF_CMAP_RANGE, 17964 },
	{ 0x6e56, 0x6e58, PDF_CMAP_TABLE, 5115 },
	{ 0x6e5b, 0x6e5f, PDF_CMAP_TABLE, 5118 },
	{ 0x6e62, 0x6e63, PDF_CMAP_TABLE, 5123 },
	{ 0x6e67, 0x6e68, PDF_CMAP_TABLE, 5125 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 5460 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 5127 },
	{ 0x6e72, 0x6e73, PDF_CMAP_TABLE, 5129 },
	{ 0x6e76, 0x6e76, PDF_CMAP_SINGLE, 5462 },
	{ 0x6e7b, 0x6e7b, PDF_CMAP_SINGLE, 19498 },
	{ 0x6e7d, 0x6e80, PDF_CMAP_TABLE, 5131 },
	{ 0x6e82, 0x6e82, PDF_CMAP_SINGLE, 5472 },
	{ 0x6e89, 0x6e89, PDF_CMAP_SINGLE, 19500 },
	{ 0x6e8c, 0x6e8d, PDF_CMAP_TABLE, 5135 },
	{ 0x6e8f, 0x6e90, PDF_CMAP_TABLE, 5137 },
	{ 0x6e93, 0x6e93, PDF_CMAP_SINGLE, 17969 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 2410 },
	{ 0x6e98, 0x6e99, PDF_CMAP_TABLE, 5139 },
	{ 0x6e9c, 0x6e9d, PDF_CMAP_TABLE, 5141 },
	{ 0x6e9f, 0x6ea0, PDF_CMAP_TABLE, 5143 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 1202 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 5485 },
	{ 0x6ea7, 0x6ea7, PDF_CMAP_SINGLE, 17970 },
	{ 0x6eaa, 0x6eab, PDF_CMAP_TABLE, 5145 },
	{ 0x6ead, 0x6eae, PDF_CMAP_RANGE, 21887 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 5479 },
	{ 0x6eb1, 0x6eb4, PDF_CMAP_TABLE, 5147 },
	{ 0x6eb6, 0x6eb7, PDF_CMAP_TABLE, 5151 },
	{ 0x6eba, 0x6ebd, PDF_CMAP_TABLE, 5153 },
	{ 0x6ebf, 0x6ec5, PDF_CMAP_TABLE, 5157 },
	{ 0x6ec7, 0x6ecf, PDF_CMAP_TABLE, 5164 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 1480 },
	{ 0x6ed3, 0x6ed3, PDF_CMAP_SINGLE, 5477 },
	{ 0x6ed4, 0x6ed5, PDF_CMAP_RANGE, 5482 },
	{ 0x6ed9, 0x6ed9, PDF_CMAP_SINGLE, 15416 },
	{ 0x6eda, 0x6edb, PDF_CMAP_RANGE, 19502 },
	{ 0x6edd, 0x6ede, PDF_CMAP_TABLE, 5173 },
	{ 0x6ee6, 0x6ee6, PDF_CMAP_SINGLE, 15396 },
	{ 0x6eeb, 0x6eec, PDF_CMAP_TABLE, 5175 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 21895 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 5497 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 5495 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 3107 },
	{ 0x6ef7, 0x6ef9, PDF_CMAP_TABLE, 5177 },
	{ 0x6efb, 0x6efb, PDF_CMAP_SINGLE, 17976 },
	{ 0x6efd, 0x6eff, PDF_CMAP_TABLE, 5180 },
	{ 0x6f01, 0x6f02, PDF_CMAP_TABLE, 5183 },
	{ 0x6f04, 0x6f04, PDF_CMAP_SINGLE, 21897 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 2283 },
	{ 0x6f08, 0x6f0a, PDF_CMAP_TABLE, 5185 },
	{ 0x6f0c, 0x6f0d, PDF_CMAP_TABLE, 5188 },
	{ 0x6f0f, 0x6f11, PDF_CMAP_TABLE, 5190 },
	{ 0x6f13, 0x6f16, PDF_CMAP_TABLE, 5193 },
	{ 0x6f18, 0x6f18, PDF_CMAP_SINGLE, 17979 },
	{ 0x6f1a, 0x6f1b, PDF_CMAP_TABLE, 5197 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 3375 },
	{ 0x6f22, 0x6f23, PDF_CMAP_TABLE, 5199 },
	{ 0x6f25, 0x6f26, PDF_CMAP_TABLE, 5201 },
	{ 0x6f29, 0x6f2d, PDF_CMAP_TABLE, 5203 },
	{ 0x6f2f, 0x6f33, PDF_CMAP_TABLE, 5208 },
	{ 0x6f35, 0x6f36, PDF_CMAP_TABLE, 5213 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 2740 },
	{ 0x6f3b, 0x6f3c, PDF_CMAP_TABLE, 5215 },
	{ 0x6f3e, 0x6f3f, PDF_CMAP_TABLE, 5217 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 5488 },
	{ 0x6f45, 0x6f45, PDF_CMAP_SINGLE, 1535 },
	{ 0x6f4f, 0x6f4f, PDF_CMAP_SINGLE, 21904 },
	{ 0x6f51, 0x6f54, PDF_CMAP_TABLE, 5219 },
	{ 0x6f57, 0x6f62, PDF_CMAP_TABLE, 5223 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 2411 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 5518 },
	{ 0x6f68, 0x6f68, PDF_CMAP_SINGLE, 17987 },
	{ 0x6f6c, 0x6f70, PDF_CMAP_TABLE, 5235 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 5543 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 5505 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 5504 },
	{ 0x6f7c, 0x6f7e, PDF_CMAP_TABLE, 5240 },
	{ 0x6f80, 0x6f84, PDF_CMAP_TABLE, 5243 },
	{ 0x6f86, 0x6f88, PDF_CMAP_TABLE, 5248 },
	{ 0x6f8b, 0x6f8e, PDF_CMAP_TABLE, 5251 },
	{ 0x6f90, 0x6f94, PDF_CMAP_TABLE, 5255 },
	{ 0x6f96, 0x6f98, PDF_CMAP_TABLE, 5260 },
	{ 0x6f9a, 0x6f9a, PDF_CMAP_SINGLE, 14735 },
	{ 0x6f9d, 0x6f9d, PDF_CMAP_SINGLE, 19508 },
	{ 0x6f9f, 0x6fa1, PDF_CMAP_TABLE, 5263 },
	{ 0x6fa3, 0x6fa6, PDF_CMAP_TABLE, 5266 },
	{ 0x6fa7, 0x6fa8, PDF_CMAP_RANGE, 14736 },
	{ 0x6faa, 0x6faa, PDF_CMAP_SINGLE, 5525 },
	{ 0x6fae, 0x6fb1, PDF_CMAP_TABLE, 5270 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 5519 },
	{ 0x6fb5, 0x6fb7, PDF_CMAP_TABLE, 5274 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5523 },
	{ 0x6fbc, 0x6fbc, PDF_CMAP_SINGLE, 16950 },
	{ 0x6fbe, 0x6fbe, PDF_CMAP_SINGLE, 17990 },
	{ 0x6fc0, 0x6fc3, PDF_CMAP_TABLE, 5277 },
	{ 0x6fc5, 0x6fc7, PDF_CMAP_TABLE, 5281 },
	{ 0x6fc8, 0x6fc9, PDF_CMAP_RANGE, 17995 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 16952 },
	{ 0x6fd4, 0x6fd5, PDF_CMAP_TABLE, 5284 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 5530 },
	{ 0x6fda, 0x6fdb, PDF_CMAP_TABLE, 5286 },
	{ 0x6fde, 0x6fe1, PDF_CMAP_TABLE, 5288 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 5430 },
	{ 0x6fe8, 0x6fe9, PDF_CMAP_TABLE, 5292 },
	{ 0x6feb, 0x6fec, PDF_CMAP_TABLE, 5294 },
	{ 0x6fee, 0x6ff1, PDF_CMAP_TABLE, 5296 },
	{ 0x6ff3, 0x6ff3, PDF_CMAP_SINGLE, 5510 },
	{ 0x6ff5, 0x6ff6, PDF_CMAP_TABLE, 5300 },
	{ 0x6ff9, 0x6ffa, PDF_CMAP_TABLE, 5302 },
	{ 0x6ffc, 0x6ffe, PDF_CMAP_TABLE, 5304 },
	{ 0x7000, 0x7001, PDF_CMAP_TABLE, 5307 },
	{ 0x7005, 0x7007, PDF_CMAP_TABLE, 5309 },
	{ 0x7009, 0x700b, PDF_CMAP_TABLE, 5312 },
	{ 0x700d, 0x700d, PDF_CMAP_SINGLE, 19511 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 5539 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5537 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 3520 },
	{ 0x7017, 0x7018, PDF_CMAP_TABLE, 5315 },
	{ 0x701a, 0x701b, PDF_CMAP_TABLE, 5317 },
	{ 0x701d, 0x7020, PDF_CMAP_TABLE, 5319 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 18002 },
	{ 0x7026, 0x7028, PDF_CMAP_TABLE, 5323 },
	{ 0x702c, 0x702c, PDF_CMAP_SINGLE, 2633 },
	{ 0x702f, 0x7030, PDF_CMAP_TABLE, 5326 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 5549 },
	{ 0x7034, 0x7034, PDF_CMAP_SINGLE, 21917 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 21918 },
	{ 0x7039, 0x703a, PDF_CMAP_TABLE, 5328 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 14743 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 5548 },
	{ 0x7043, 0x7044, PDF_CMAP_TABLE, 5330 },
	{ 0x7047, 0x704c, PDF_CMAP_TABLE, 5332 },
	{ 0x704e, 0x704e, PDF_CMAP_SINGLE, 16954 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 5550 },
	{ 0x7054, 0x7055, PDF_CMAP_TABLE, 5338 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 3263 },
	{ 0x705d, 0x705e, PDF_CMAP_RANGE, 14746 },
	{ 0x7063, 0x7065, PDF_CMAP_TABLE, 5340 },
	{ 0x7069, 0x7069, PDF_CMAP_SINGLE, 18010 },
	{ 0x706b, 0x706c, PDF_CMAP_TABLE, 5343 },
	{ 0x706e, 0x7070, PDF_CMAP_TABLE, 5345 },
	{ 0x7075, 0x7076, PDF_CMAP_TABLE, 5348 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 1662 },
	{ 0x707c, 0x707e, PDF_CMAP_TABLE, 5350 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 14751 },
	{ 0x7085, 0x7086, PDF_CMAP_TABLE, 5353 },
	{ 0x7089, 0x708a, PDF_CMAP_TABLE, 5355 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 1292 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 5553 },
	{ 0x7094, 0x7099, PDF_CMAP_TABLE, 5357 },
	{ 0x709b, 0x709b, PDF_CMAP_SINGLE, 21924 },
	{ 0x709f, 0x709f, PDF_CMAP_SINGLE, 18016 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 16956 },
	{ 0x70ab, 0x70b1, PDF_CMAP_TABLE, 5363 },
	{ 0x70b3, 0x70b4, PDF_CMAP_TABLE, 5370 },
	{ 0x70b7, 0x70bb, PDF_CMAP_TABLE, 5372 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 4029 },
	{ 0x70ca, 0x70cb, PDF_CMAP_TABLE, 5377 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 1226 },
	{ 0x70d1, 0x70d1, PDF_CMAP_SINGLE, 18021 },
	{ 0x70d3, 0x70d4, PDF_CMAP_RANGE, 14754 },
	{ 0x70d5, 0x70d6, PDF_CMAP_RANGE, 19516 },
	{ 0x70d8, 0x70d9, PDF_CMAP_TABLE, 5379 },
	{ 0x70dc, 0x70dd, PDF_CMAP_TABLE, 5381 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 5560 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 16957 },
	{ 0x70ec, 0x70ec, PDF_CMAP_SINGLE, 18019 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 5555 },
	{ 0x70f9, 0x70fa, PDF_CMAP_TABLE, 5383 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 5565 },
	{ 0x7103, 0x7109, PDF_CMAP_TABLE, 5385 },
	{ 0x710b, 0x710c, PDF_CMAP_TABLE, 5392 },
	{ 0x710f, 0x710f, PDF_CMAP_SINGLE, 8537 },
	{ 0x7114, 0x7114, PDF_CMAP_SINGLE, 1293 },
	{ 0x7119, 0x711a, PDF_CMAP_TABLE, 5394 },
	{ 0x711c, 0x711c, PDF_CMAP_SINGLE, 5566 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 16959 },
	{ 0x7120, 0x7121, PDF_CMAP_TABLE, 5396 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2479 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 16958 },
	{ 0x712d, 0x7131, PDF_CMAP_TABLE, 5398 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 2741 },
	{ 0x7138, 0x7138, PDF_CMAP_SINGLE, 21930 },
	{ 0x713c, 0x713c, PDF_CMAP_SINGLE, 2478 },
	{ 0x7141, 0x7141, PDF_CMAP_SINGLE, 21931 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 19518 },
	{ 0x7146, 0x7147, PDF_CMAP_RANGE, 8540 },
	{ 0x7149, 0x714c, PDF_CMAP_TABLE, 5403 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 2717 },
	{ 0x7150, 0x7153, PDF_CMAP_TABLE, 5407 },
	{ 0x7155, 0x7157, PDF_CMAP_TABLE, 5411 },
	{ 0x7159, 0x715a, PDF_CMAP_TABLE, 5414 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 8539 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 18030 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 14763 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 5572 },
	{ 0x7164, 0x7169, PDF_CMAP_TABLE, 5416 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 5575 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 2301 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 14764 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 2718 },
	{ 0x7180, 0x7180, PDF_CMAP_SINGLE, 18033 },
	{ 0x7184, 0x7185, PDF_CMAP_TABLE, 5422 },
	{ 0x7187, 0x7188, PDF_CMAP_TABLE, 5424 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 1789 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 21936 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 5576 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 14765 },
	{ 0x7194, 0x7196, PDF_CMAP_TABLE, 5426 },
	{ 0x7199, 0x719b, PDF_CMAP_TABLE, 5429 },
	{ 0x719f, 0x71a0, PDF_CMAP_TABLE, 5432 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 18036 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 5580 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 5581 },
	{ 0x71ae, 0x71af, PDF_CMAP_RANGE, 18037 },
	{ 0x71b0, 0x71b3, PDF_CMAP_TABLE, 5434 },
	{ 0x71b9, 0x71ba, PDF_CMAP_TABLE, 5438 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 5584 },
	{ 0x71bf, 0x71c0, PDF_CMAP_RANGE, 21939 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 8543 },
	{ 0x71c3, 0x71c4, PDF_CMAP_TABLE, 5440 },
	{ 0x71c8, 0x71c9, PDF_CMAP_TABLE, 5442 },
	{ 0x71cb, 0x71cc, PDF_CMAP_TABLE, 5444 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 5588 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 3997 },
	{ 0x71d2, 0x71d7, PDF_CMAP_TABLE, 5446 },
	{ 0x71d9, 0x71da, PDF_CMAP_TABLE, 5452 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 18041 },
	{ 0x71df, 0x71e0, PDF_CMAP_TABLE, 5454 },
	{ 0x71e5, 0x71e7, PDF_CMAP_TABLE, 5456 },
	{ 0x71ec, 0x71ee, PDF_CMAP_TABLE, 5459 },
	{ 0x71f4, 0x71f5, PDF_CMAP_TABLE, 5462 },
	{ 0x71f8, 0x71f9, PDF_CMAP_TABLE, 5464 },
	{ 0x71fb, 0x71fc, PDF_CMAP_TABLE, 5466 },
	{ 0x71fe, 0x7200, PDF_CMAP_TABLE, 5468 },
	{ 0x7206, 0x7207, PDF_CMAP_TABLE, 5471 },
	{ 0x7208, 0x7209, PDF_CMAP_RANGE, 21944 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 5596 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 5597 },
	{ 0x7213, 0x7213, PDF_CMAP_SINGLE, 21946 },
	{ 0x7215, 0x7215, PDF_CMAP_SINGLE, 16967 },
	{ 0x7217, 0x7217, PDF_CMAP_SINGLE, 19521 },
	{ 0x721a, 0x721b, PDF_CMAP_TABLE, 5473 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 14771 },
	{ 0x721f, 0x721f, PDF_CMAP_SINGLE, 19522 },
	{ 0x7224, 0x7224, PDF_CMAP_SINGLE, 21948 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 5599 },
	{ 0x722a, 0x722d, PDF_CMAP_TABLE, 5475 },
	{ 0x722f, 0x7230, PDF_CMAP_TABLE, 5479 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 5603 },
	{ 0x7234, 0x7236, PDF_CMAP_TABLE, 5481 },
	{ 0x7238, 0x723a, PDF_CMAP_TABLE, 5484 },
	{ 0x723b, 0x723c, PDF_CMAP_RANGE, 5604 },
	{ 0x723d, 0x723e, PDF_CMAP_TABLE, 5487 },
	{ 0x723f, 0x7240, PDF_CMAP_RANGE, 5606 },
	{ 0x7241, 0x7243, PDF_CMAP_TABLE, 5489 },
	{ 0x7245, 0x7248, PDF_CMAP_TABLE, 5492 },
	{ 0x724b, 0x724c, PDF_CMAP_TABLE, 5496 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 21951 },
	{ 0x724f, 0x7250, PDF_CMAP_RANGE, 19524 },
	{ 0x7252, 0x7253, PDF_CMAP_TABLE, 5498 },
	{ 0x7255, 0x7256, PDF_CMAP_RANGE, 14776 },
	{ 0x7257, 0x7263, PDF_CMAP_TABLE, 5500 },
	{ 0x7267, 0x7269, PDF_CMAP_TABLE, 5513 },
	{ 0x726b, 0x726b, PDF_CMAP_SINGLE, 21953 },
	{ 0x726e, 0x726f, PDF_CMAP_RANGE, 18050 },
	{ 0x7271, 0x7272, PDF_CMAP_TABLE, 5516 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 5611 },
	{ 0x7277, 0x7279, PDF_CMAP_TABLE, 5518 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 21955 },
	{ 0x727d, 0x7282, PDF_CMAP_TABLE, 5521 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 19530 },
	{ 0x7287, 0x7287, PDF_CMAP_SINGLE, 5615 },
	{ 0x7289, 0x7289, PDF_CMAP_SINGLE, 21957 },
	{ 0x728d, 0x728e, PDF_CMAP_TABLE, 5527 },
	{ 0x7292, 0x7293, PDF_CMAP_TABLE, 5529 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 5617 },
	{ 0x729b, 0x729b, PDF_CMAP_SINGLE, 16969 },
	{ 0x72a0, 0x72a0, PDF_CMAP_SINGLE, 1624 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 5618 },
	{ 0x72a7, 0x72a8, PDF_CMAP_TABLE, 5531 },
	{ 0x72ac, 0x72b2, PDF_CMAP_TABLE, 5533 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 14781 },
	{ 0x72b6, 0x72b6, PDF_CMAP_SINGLE, 2525 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 5620 },
	{ 0x72be, 0x72be, PDF_CMAP_SINGLE, 8546 },
	{ 0x72c0, 0x72c4, PDF_CMAP_TABLE, 5540 },
	{ 0x72c6, 0x72c7, PDF_CMAP_TABLE, 5545 },
	{ 0x72c9, 0x72c9, PDF_CMAP_SINGLE, 19531 },
	{ 0x72cc, 0x72cc, PDF_CMAP_SINGLE, 18060 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5625 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1925 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 5626 },
	{ 0x72d5, 0x72d6, PDF_CMAP_RANGE, 21960 },
	{ 0x72d7, 0x72d9, PDF_CMAP_TABLE, 5547 },
	{ 0x72db, 0x72db, PDF_CMAP_SINGLE, 2063 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 21963 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 5628 },
	{ 0x72e2, 0x72e2, PDF_CMAP_SINGLE, 5627 },
	{ 0x72e5, 0x72e5, PDF_CMAP_SINGLE, 19532 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 2329 },
	{ 0x72ec, 0x72ed, PDF_CMAP_TABLE, 5550 },
	{ 0x72f3, 0x72f4, PDF_CMAP_TABLE, 5552 },
	{ 0x72f7, 0x72fe, PDF_CMAP_TABLE, 5554 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 19534 },
	{ 0x7304, 0x7305, PDF_CMAP_RANGE, 14784 },
	{ 0x7307, 0x7307, PDF_CMAP_SINGLE, 18065 },
	{ 0x730a, 0x730b, PDF_CMAP_TABLE, 5562 },
	{ 0x730d, 0x730d, PDF_CMAP_SINGLE, 21965 },
	{ 0x7312, 0x7313, PDF_CMAP_TABLE, 5564 },
	{ 0x7316, 0x7317, PDF_CMAP_TABLE, 5566 },
	{ 0x7318, 0x7319, PDF_CMAP_RANGE, 18067 },
	{ 0x731b, 0x731f, PDF_CMAP_TABLE, 5568 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 19537 },
	{ 0x7324, 0x7325, PDF_CMAP_TABLE, 5573 },
	{ 0x7327, 0x732c, PDF_CMAP_TABLE, 5575 },
	{ 0x732e, 0x732f, PDF_CMAP_TABLE, 5581 },
	{ 0x7331, 0x7335, PDF_CMAP_TABLE, 5583 },
	{ 0x7336, 0x7337, PDF_CMAP_RANGE, 3867 },
	{ 0x7339, 0x7339, PDF_CMAP_SINGLE, 18070 },
	{ 0x733a, 0x733b, PDF_CMAP_RANGE, 19538 },
	{ 0x733d, 0x733f, PDF_CMAP_TABLE, 5588 },
	{ 0x7343, 0x7345, PDF_CMAP_TABLE, 5591 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 19540 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 5643 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 16972 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 18074 },
	{ 0x7356, 0x7358, PDF_CMAP_TABLE, 5594 },
	{ 0x735d, 0x7360, PDF_CMAP_RANGE, 21970 },
	{ 0x7363, 0x7363, PDF_CMAP_SINGLE, 2381 },
	{ 0x7366, 0x736c, PDF_CMAP_TABLE, 5597 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 18078 },
	{ 0x7370, 0x7372, PDF_CMAP_TABLE, 5604 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 5651 },
	{ 0x7377, 0x737c, PDF_CMAP_TABLE, 5607 },
	{ 0x7380, 0x7381, PDF_CMAP_TABLE, 5613 },
	{ 0x7383, 0x7384, PDF_CMAP_TABLE, 5615 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 14792 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 3952 },
	{ 0x7389, 0x738b, PDF_CMAP_TABLE, 5617 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 21977 },
	{ 0x7390, 0x7390, PDF_CMAP_SINGLE, 21978 },
	{ 0x7393, 0x7398, PDF_CMAP_TABLE, 5620 },
	{ 0x739c, 0x739c, PDF_CMAP_SINGLE, 18085 },
	{ 0x739e, 0x73a0, PDF_CMAP_RANGE, 14795 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 16974 },
	{ 0x73a5, 0x73a6, PDF_CMAP_TABLE, 5626 },
	{ 0x73a8, 0x73ab, PDF_CMAP_TABLE, 5628 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 21982 },
	{ 0x73b2, 0x73b3, PDF_CMAP_TABLE, 5632 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 14800 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 14801 },
	{ 0x73b9, 0x73bd, PDF_CMAP_TABLE, 5634 },
	{ 0x73bf, 0x73c0, PDF_CMAP_TABLE, 5639 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 1361 },
	{ 0x73c5, 0x73c6, PDF_CMAP_TABLE, 5641 },
	{ 0x73c8, 0x73cf, PDF_CMAP_TABLE, 5643 },
	{ 0x73d2, 0x73d3, PDF_CMAP_TABLE, 5651 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 8552 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 14804 },
	{ 0x73dd, 0x73de, PDF_CMAP_TABLE, 5653 },
	{ 0x73e0, 0x73e1, PDF_CMAP_TABLE, 5655 },
	{ 0x73e3, 0x73e7, PDF_CMAP_TABLE, 5657 },
	{ 0x73e9, 0x73ea, PDF_CMAP_TABLE, 5662 },
	{ 0x73ed, 0x73ee, PDF_CMAP_TABLE, 5664 },
	{ 0x73f1, 0x73f1, PDF_CMAP_SINGLE, 5687 },
	{ 0x73f4, 0x73f5, PDF_CMAP_TABLE, 5666 },
	{ 0x73f7, 0x73fb, PDF_CMAP_TABLE, 5668 },
	{ 0x73fd, 0x7401, PDF_CMAP_TABLE, 5673 },
	{ 0x7403, 0x7407, PDF_CMAP_TABLE, 5678 },
	{ 0x7409, 0x740a, PDF_CMAP_TABLE, 5683 },
	{ 0x7411, 0x7411, PDF_CMAP_SINGLE, 21992 },
	{ 0x7413, 0x7413, PDF_CMAP_SINGLE, 18094 },
	{ 0x741a, 0x741b, PDF_CMAP_RANGE, 14810 },
	{ 0x7421, 0x7422, PDF_CMAP_TABLE, 5685 },
	{ 0x7424, 0x7426, PDF_CMAP_TABLE, 5687 },
	{ 0x7428, 0x742e, PDF_CMAP_TABLE, 5690 },
	{ 0x742f, 0x7431, PDF_CMAP_RANGE, 14815 },
	{ 0x7432, 0x7436, PDF_CMAP_TABLE, 5697 },
	{ 0x7439, 0x743a, PDF_CMAP_TABLE, 5702 },
	{ 0x743f, 0x7441, PDF_CMAP_TABLE, 5704 },
	{ 0x7443, 0x7444, PDF_CMAP_TABLE, 5707 },
	{ 0x7446, 0x7447, PDF_CMAP_TABLE, 5709 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 14821 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 14822 },
	{ 0x7451, 0x7453, PDF_CMAP_TABLE, 5711 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 5669 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 14824 },
	{ 0x7459, 0x7460, PDF_CMAP_TABLE, 5714 },
	{ 0x7462, 0x7464, PDF_CMAP_TABLE, 5722 },
	{ 0x7466, 0x7466, PDF_CMAP_SINGLE, 14825 },
	{ 0x7467, 0x7468, PDF_CMAP_RANGE, 21994 },
	{ 0x7469, 0x746b, PDF_CMAP_TABLE, 5725 },
	{ 0x746d, 0x7473, PDF_CMAP_TABLE, 5728 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 5679 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 5680 },
	{ 0x7480, 0x7481, PDF_CMAP_TABLE, 5735 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 3944 },
	{ 0x7485, 0x7487, PDF_CMAP_RANGE, 14829 },
	{ 0x7488, 0x7489, PDF_CMAP_TABLE, 5737 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 5681 },
	{ 0x748f, 0x7492, PDF_CMAP_TABLE, 5739 },
	{ 0x7497, 0x749a, PDF_CMAP_TABLE, 5743 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 14834 },
	{ 0x749e, 0x74a3, PDF_CMAP_TABLE, 5747 },
	{ 0x74a5, 0x74ab, PDF_CMAP_TABLE, 5753 },
	{ 0x74ae, 0x74b0, PDF_CMAP_TABLE, 5760 },
	{ 0x74b1, 0x74b2, PDF_CMAP_RANGE, 22001 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 14839 },
	{ 0x74b9, 0x74bb, PDF_CMAP_TABLE, 5763 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 2257 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 14840 },
	{ 0x74c8, 0x74ca, PDF_CMAP_TABLE, 5766 },
	{ 0x74cc, 0x74cc, PDF_CMAP_SINGLE, 22003 },
	{ 0x74cf, 0x74d0, PDF_CMAP_TABLE, 5769 },
	{ 0x74d3, 0x74d4, PDF_CMAP_TABLE, 5771 },
	{ 0x74d6, 0x74d6, PDF_CMAP_SINGLE, 18114 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 18115 },
	{ 0x74da, 0x74dc, PDF_CMAP_TABLE, 5773 },
	{ 0x74de, 0x74e0, PDF_CMAP_TABLE, 5776 },
	{ 0x74e2, 0x74e4, PDF_CMAP_TABLE, 5779 },
	{ 0x74e6, 0x74eb, PDF_CMAP_TABLE, 5782 },
	{ 0x74ee, 0x74ef, PDF_CMAP_TABLE, 5788 },
	{ 0x74f0, 0x74f1, PDF_CMAP_RANGE, 5694 },
	{ 0x74f2, 0x74f2, PDF_CMAP_SINGLE, 5693 },
	{ 0x74f4, 0x74f4, PDF_CMAP_SINGLE, 19547 },
	{ 0x74f6, 0x74f8, PDF_CMAP_TABLE, 5790 },
	{ 0x74fa, 0x74fc, PDF_CMAP_TABLE, 5793 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 16982 },
	{ 0x7501, 0x7501, PDF_CMAP_SINGLE, 8564 },
	{ 0x7503, 0x7506, PDF_CMAP_TABLE, 5796 },
	{ 0x750c, 0x750e, PDF_CMAP_TABLE, 5800 },
	{ 0x7511, 0x7513, PDF_CMAP_TABLE, 5803 },
	{ 0x7515, 0x7518, PDF_CMAP_TABLE, 5806 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 2585 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 3126 },
	{ 0x751e, 0x752c, PDF_CMAP_TABLE, 5810 },
	{ 0x752f, 0x7533, PDF_CMAP_TABLE, 5825 },
	{ 0x7536, 0x753c, PDF_CMAP_TABLE, 5830 },
	{ 0x753d, 0x753e, PDF_CMAP_RANGE, 18125 },
	{ 0x753f, 0x7540, PDF_CMAP_TABLE, 5837 },
	{ 0x7543, 0x7544, PDF_CMAP_TABLE, 5839 },
	{ 0x7546, 0x7552, PDF_CMAP_TABLE, 5841 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 3422 },
	{ 0x7557, 0x7557, PDF_CMAP_SINGLE, 22020 },
	{ 0x7559, 0x7562, PDF_CMAP_TABLE, 5854 },
	{ 0x7564, 0x7567, PDF_CMAP_TABLE, 5864 },
	{ 0x7569, 0x756d, PDF_CMAP_TABLE, 5868 },
	{ 0x756f, 0x757a, PDF_CMAP_TABLE, 5873 },
	{ 0x757b, 0x757c, PDF_CMAP_RANGE, 22023 },
	{ 0x757d, 0x757e, PDF_CMAP_RANGE, 18134 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 1600 },
	{ 0x7581, 0x7582, PDF_CMAP_TABLE, 5885 },
	{ 0x7585, 0x7585, PDF_CMAP_SINGLE, 22025 },
	{ 0x7586, 0x7587, PDF_CMAP_RANGE, 5724 },
	{ 0x7589, 0x758c, PDF_CMAP_TABLE, 5887 },
	{ 0x758e, 0x7591, PDF_CMAP_TABLE, 5891 },
	{ 0x7592, 0x7593, PDF_CMAP_RANGE, 14848 },
	{ 0x7594, 0x7595, PDF_CMAP_TABLE, 5895 },
	{ 0x7599, 0x759a, PDF_CMAP_TABLE, 5897 },
	{ 0x759c, 0x759d, PDF_CMAP_TABLE, 5899 },
	{ 0x75a2, 0x75a5, PDF_CMAP_TABLE, 5901 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 1272 },
	{ 0x75b0, 0x75b5, PDF_CMAP_TABLE, 5905 },
	{ 0x75b7, 0x75ba, PDF_CMAP_TABLE, 5911 },
	{ 0x75bc, 0x75be, PDF_CMAP_TABLE, 5915 },
	{ 0x75bf, 0x75c0, PDF_CMAP_RANGE, 18142 },
	{ 0x75c1, 0x75c7, PDF_CMAP_TABLE, 5918 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 5744 },
	{ 0x75cc, 0x75cf, PDF_CMAP_TABLE, 5925 },
	{ 0x75d2, 0x75d5, PDF_CMAP_TABLE, 5929 },
	{ 0x75d7, 0x75d9, PDF_CMAP_TABLE, 5933 },
	{ 0x75db, 0x75de, PDF_CMAP_TABLE, 5936 },
	{ 0x75df, 0x75e0, PDF_CMAP_RANGE, 18148 },
	{ 0x75e1, 0x75e4, PDF_CMAP_TABLE, 5940 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 18150 },
	{ 0x75e9, 0x75e9, PDF_CMAP_SINGLE, 2795 },
	{ 0x75ec, 0x75ec, PDF_CMAP_SINGLE, 14162 },
	{ 0x75ee, 0x75f1, PDF_CMAP_TABLE, 5944 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 5755 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 2962 },
	{ 0x75f9, 0x75fa, PDF_CMAP_TABLE, 5948 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 5751 },
	{ 0x75fe, 0x75ff, PDF_CMAP_RANGE, 5749 },
	{ 0x7600, 0x7604, PDF_CMAP_TABLE, 5950 },
	{ 0x7607, 0x760d, PDF_CMAP_TABLE, 5955 },
	{ 0x760f, 0x760f, PDF_CMAP_SINGLE, 18156 },
	{ 0x7612, 0x7613, PDF_CMAP_TABLE, 5962 },
	{ 0x7615, 0x7616, PDF_CMAP_RANGE, 14855 },
	{ 0x7618, 0x7619, PDF_CMAP_TABLE, 5964 },
	{ 0x761b, 0x761c, PDF_CMAP_RANGE, 18160 },
	{ 0x761d, 0x761f, PDF_CMAP_TABLE, 5966 },
	{ 0x7620, 0x7622, PDF_CMAP_RANGE, 5762 },
	{ 0x7623, 0x7629, PDF_CMAP_TABLE, 5969 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 14859 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 5767 },
	{ 0x7632, 0x7635, PDF_CMAP_TABLE, 5976 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 19566 },
	{ 0x7639, 0x763a, PDF_CMAP_RANGE, 22032 },
	{ 0x763b, 0x763c, PDF_CMAP_TABLE, 5980 },
	{ 0x7640, 0x7646, PDF_CMAP_TABLE, 5982 },
	{ 0x7647, 0x7648, PDF_CMAP_RANGE, 5769 },
	{ 0x7649, 0x764c, PDF_CMAP_TABLE, 5989 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 14163 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 3850 },
	{ 0x7655, 0x7656, PDF_CMAP_TABLE, 5993 },
	{ 0x7658, 0x7659, PDF_CMAP_TABLE, 5995 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 5772 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 19569 },
	{ 0x7661, 0x7662, PDF_CMAP_RANGE, 5774 },
	{ 0x7664, 0x7665, PDF_CMAP_TABLE, 5997 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 5779 },
	{ 0x7668, 0x766a, PDF_CMAP_RANGE, 5776 },
	{ 0x766c, 0x7672, PDF_CMAP_TABLE, 5999 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 14867 },
	{ 0x7676, 0x7676, PDF_CMAP_SINGLE, 5783 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 5784 },
	{ 0x767a, 0x767e, PDF_CMAP_TABLE, 6006 },
	{ 0x7680, 0x7688, PDF_CMAP_TABLE, 6011 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 5789 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 22038 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 5790 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 2167 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 5792 },
	{ 0x7695, 0x7696, PDF_CMAP_TABLE, 6020 },
	{ 0x7699, 0x769a, PDF_CMAP_RANGE, 5793 },
	{ 0x769b, 0x76a1, PDF_CMAP_TABLE, 6022 },
	{ 0x76a2, 0x76a3, PDF_CMAP_RANGE, 22041 },
	{ 0x76a4, 0x76a5, PDF_CMAP_RANGE, 14868 },
	{ 0x76a6, 0x76a6, PDF_CMAP_SINGLE, 8570 },
	{ 0x76a7, 0x76a8, PDF_CMAP_RANGE, 18178 },
	{ 0x76aa, 0x76aa, PDF_CMAP_SINGLE, 16991 },
	{ 0x76ad, 0x76b0, PDF_CMAP_TABLE, 6029 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 5796 },
	{ 0x76b6, 0x76b7, PDF_CMAP_TABLE, 6033 },
	{ 0x76b8, 0x76ba, PDF_CMAP_RANGE, 5797 },
	{ 0x76bd, 0x76bd, PDF_CMAP_SINGLE, 19571 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2172 },
	{ 0x76c1, 0x76c3, PDF_CMAP_TABLE, 6035 },
	{ 0x76c5, 0x76c6, PDF_CMAP_TABLE, 6038 },
	{ 0x76c8, 0x76ce, PDF_CMAP_TABLE, 6040 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 5803 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 16994 },
	{ 0x76d6, 0x76d7, PDF_CMAP_TABLE, 6047 },
	{ 0x76d9, 0x76d9, PDF_CMAP_SINGLE, 19572 },
	{ 0x76db, 0x76dc, PDF_CMAP_TABLE, 6049 },
	{ 0x76de, 0x76e1, PDF_CMAP_TABLE, 6051 },
	{ 0x76e3, 0x76e8, PDF_CMAP_TABLE, 6055 },
	{ 0x76ea, 0x76ec, PDF_CMAP_TABLE, 6061 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 3816 },
	{ 0x76f0, 0x76f2, PDF_CMAP_TABLE, 6064 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 3034 },
	{ 0x76f6, 0x76f6, PDF_CMAP_SINGLE, 22047 },
	{ 0x76f8, 0x76f9, PDF_CMAP_TABLE, 6067 },
	{ 0x76fb, 0x76fc, PDF_CMAP_TABLE, 6069 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 2412 },
	{ 0x7700, 0x7701, PDF_CMAP_TABLE, 6071 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 5813 },
	{ 0x7706, 0x770c, PDF_CMAP_TABLE, 6073 },
	{ 0x770e, 0x770e, PDF_CMAP_SINGLE, 19577 },
	{ 0x7712, 0x7712, PDF_CMAP_SINGLE, 22049 },
	{ 0x7714, 0x7715, PDF_CMAP_RANGE, 22050 },
	{ 0x7717, 0x7717, PDF_CMAP_SINGLE, 18186 },
	{ 0x7719, 0x771c, PDF_CMAP_TABLE, 6080 },
	{ 0x771e, 0x7720, PDF_CMAP_TABLE, 6084 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 19578 },
	{ 0x7724, 0x7724, PDF_CMAP_SINGLE, 5815 },
	{ 0x7725, 0x7726, PDF_CMAP_RANGE, 5817 },
	{ 0x7728, 0x7729, PDF_CMAP_TABLE, 6087 },
	{ 0x772d, 0x772f, PDF_CMAP_TABLE, 6089 },
	{ 0x7734, 0x7736, PDF_CMAP_TABLE, 6092 },
	{ 0x7737, 0x7738, PDF_CMAP_RANGE, 5820 },
	{ 0x7739, 0x773a, PDF_CMAP_TABLE, 6095 },
	{ 0x773c, 0x773e, PDF_CMAP_TABLE, 6097 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 2979 },
	{ 0x7742, 0x7742, PDF_CMAP_SINGLE, 22055 },
	{ 0x7745, 0x7747, PDF_CMAP_TABLE, 6100 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 19584 },
	{ 0x774d, 0x774e, PDF_CMAP_RANGE, 16999 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 19585 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 22056 },
	{ 0x7756, 0x7757, PDF_CMAP_RANGE, 22057 },
	{ 0x7758, 0x7758, PDF_CMAP_SINGLE, 18194 },
	{ 0x775a, 0x775c, PDF_CMAP_TABLE, 6103 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 19586 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 14877 },
	{ 0x7761, 0x7768, PDF_CMAP_TABLE, 6106 },
	{ 0x776a, 0x776c, PDF_CMAP_TABLE, 6114 },
	{ 0x7770, 0x7770, PDF_CMAP_SINGLE, 22059 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14879 },
	{ 0x7773, 0x7774, PDF_CMAP_RANGE, 22060 },
	{ 0x7779, 0x777a, PDF_CMAP_TABLE, 6117 },
	{ 0x777c, 0x7780, PDF_CMAP_TABLE, 6119 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 19590 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 5832 },
	{ 0x778c, 0x778d, PDF_CMAP_RANGE, 19591 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 5831 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 5833 },
	{ 0x7794, 0x7796, PDF_CMAP_TABLE, 6124 },
	{ 0x779a, 0x779a, PDF_CMAP_SINGLE, 18199 },
	{ 0x779e, 0x77a0, PDF_CMAP_TABLE, 6127 },
	{ 0x77a2, 0x77a2, PDF_CMAP_SINGLE, 18201 },
	{ 0x77a4, 0x77a5, PDF_CMAP_TABLE, 6130 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 19594 },
	{ 0x77a9, 0x77aa, PDF_CMAP_TABLE, 6132 },
	{ 0x77ac, 0x77b1, PDF_CMAP_TABLE, 6134 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 3215 },
	{ 0x77b5, 0x77b7, PDF_CMAP_TABLE, 6140 },
	{ 0x77b9, 0x77b9, PDF_CMAP_SINGLE, 5838 },
	{ 0x77bb, 0x77bb, PDF_CMAP_SINGLE, 5842 },
	{ 0x77bc, 0x77bd, PDF_CMAP_RANGE, 5840 },
	{ 0x77be, 0x77bf, PDF_CMAP_TABLE, 6143 },
	{ 0x77c3, 0x77c3, PDF_CMAP_SINGLE, 22067 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5843 },
	{ 0x77c9, 0x77c9, PDF_CMAP_SINGLE, 19598 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5844 },
	{ 0x77d1, 0x77d2, PDF_CMAP_TABLE, 6145 },
	{ 0x77d5, 0x77d5, PDF_CMAP_SINGLE, 22069 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5845 },
	{ 0x77d9, 0x77dc, PDF_CMAP_TABLE, 6147 },
	{ 0x77de, 0x77df, PDF_CMAP_RANGE, 18203 },
	{ 0x77e0, 0x77e0, PDF_CMAP_SINGLE, 17004 },
	{ 0x77e2, 0x77e7, PDF_CMAP_TABLE, 6151 },
	{ 0x77e9, 0x77ea, PDF_CMAP_TABLE, 6157 },
	{ 0x77ec, 0x77f1, PDF_CMAP_TABLE, 6159 },
	{ 0x77f3, 0x77f4, PDF_CMAP_TABLE, 6165 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 22070 },
	{ 0x77fb, 0x77fc, PDF_CMAP_TABLE, 6167 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 2093 },
	{ 0x7805, 0x7806, PDF_CMAP_TABLE, 6169 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 18211 },
	{ 0x780c, 0x780e, PDF_CMAP_TABLE, 6171 },
	{ 0x7811, 0x7812, PDF_CMAP_TABLE, 6174 },
	{ 0x7814, 0x7815, PDF_CMAP_TABLE, 6176 },
	{ 0x7819, 0x7819, PDF_CMAP_SINGLE, 18213 },
	{ 0x781d, 0x781d, PDF_CMAP_SINGLE, 22073 },
	{ 0x7820, 0x7823, PDF_CMAP_TABLE, 6178 },
	{ 0x7825, 0x7827, PDF_CMAP_TABLE, 6182 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 18214 },
	{ 0x782d, 0x782e, PDF_CMAP_RANGE, 14888 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 14890 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 3666 },
	{ 0x7834, 0x7835, PDF_CMAP_TABLE, 6185 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 19602 },
	{ 0x783a, 0x783a, PDF_CMAP_SINGLE, 3153 },
	{ 0x783f, 0x783f, PDF_CMAP_SINGLE, 2030 },
	{ 0x7843, 0x7845, PDF_CMAP_TABLE, 6187 },
	{ 0x7847, 0x7848, PDF_CMAP_TABLE, 6190 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 22077 },
	{ 0x784e, 0x784f, PDF_CMAP_TABLE, 6192 },
	{ 0x7851, 0x7852, PDF_CMAP_TABLE, 6194 },
	{ 0x785c, 0x785e, PDF_CMAP_TABLE, 6196 },
	{ 0x7860, 0x7861, PDF_CMAP_RANGE, 22080 },
	{ 0x7863, 0x7864, PDF_CMAP_TABLE, 6199 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 14892 },
	{ 0x786a, 0x786c, PDF_CMAP_TABLE, 6201 },
	{ 0x786e, 0x786f, PDF_CMAP_TABLE, 6204 },
	{ 0x7872, 0x7872, PDF_CMAP_SINGLE, 3383 },
	{ 0x7874, 0x7874, PDF_CMAP_SINGLE, 5858 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 8577 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 5860 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 19604 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 1951 },
	{ 0x7886, 0x7887, PDF_CMAP_TABLE, 6206 },
	{ 0x788a, 0x788a, PDF_CMAP_SINGLE, 18218 },
	{ 0x788c, 0x788f, PDF_CMAP_TABLE, 6208 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3454 },
	{ 0x7893, 0x7895, PDF_CMAP_TABLE, 6212 },
	{ 0x7897, 0x7898, PDF_CMAP_TABLE, 6215 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 5861 },
	{ 0x789d, 0x789f, PDF_CMAP_TABLE, 6217 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 19606 },
	{ 0x78a3, 0x78a4, PDF_CMAP_TABLE, 6220 },
	{ 0x78a7, 0x78aa, PDF_CMAP_TABLE, 6222 },
	{ 0x78ac, 0x78ad, PDF_CMAP_TABLE, 6226 },
	{ 0x78af, 0x78b3, PDF_CMAP_TABLE, 6228 },
	{ 0x78b5, 0x78b5, PDF_CMAP_SINGLE, 5864 },
	{ 0x78ba, 0x78bf, PDF_CMAP_TABLE, 6233 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 2259 },
	{ 0x78c5, 0x78cc, PDF_CMAP_TABLE, 6239 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 14896 },
	{ 0x78d0, 0x78d6, PDF_CMAP_TABLE, 6247 },
	{ 0x78da, 0x78db, PDF_CMAP_TABLE, 6254 },
	{ 0x78df, 0x78df, PDF_CMAP_SINGLE, 22093 },
	{ 0x78e0, 0x78e1, PDF_CMAP_RANGE, 14898 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 14897 },
	{ 0x78e6, 0x78e8, PDF_CMAP_TABLE, 6256 },
	{ 0x78ea, 0x78ea, PDF_CMAP_SINGLE, 22094 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 5875 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 1199 },
	{ 0x78f2, 0x78f4, PDF_CMAP_TABLE, 6259 },
	{ 0x78f6, 0x78f7, PDF_CMAP_TABLE, 6262 },
	{ 0x78f9, 0x78fa, PDF_CMAP_RANGE, 18226 },
	{ 0x78fb, 0x78fb, PDF_CMAP_SINGLE, 14902 },
	{ 0x78fd, 0x7901, PDF_CMAP_TABLE, 6264 },
	{ 0x7906, 0x7907, PDF_CMAP_TABLE, 6269 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 19611 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 2757 },
	{ 0x7910, 0x7912, PDF_CMAP_TABLE, 6271 },
	{ 0x7919, 0x791c, PDF_CMAP_TABLE, 6274 },
	{ 0x791e, 0x7920, PDF_CMAP_TABLE, 6278 },
	{ 0x7925, 0x7926, PDF_CMAP_TABLE, 6281 },
	{ 0x7927, 0x7928, PDF_CMAP_RANGE, 19613 },
	{ 0x7929, 0x792e, PDF_CMAP_TABLE, 6283 },
	{ 0x7930, 0x7931, PDF_CMAP_TABLE, 6289 },
	{ 0x7934, 0x7935, PDF_CMAP_TABLE, 6291 },
	{ 0x793a, 0x7942, PDF_CMAP_TABLE, 6293 },
	{ 0x7944, 0x794b, PDF_CMAP_TABLE, 6302 },
	{ 0x794f, 0x7951, PDF_CMAP_TABLE, 6310 },
	{ 0x7953, 0x7958, PDF_CMAP_TABLE, 6313 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 5890 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 14908 },
	{ 0x795d, 0x7960, PDF_CMAP_TABLE, 6319 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 3296 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 2485 },
	{ 0x7967, 0x7969, PDF_CMAP_TABLE, 6323 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 19618 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2119 },
	{ 0x7972, 0x7972, PDF_CMAP_SINGLE, 18237 },
	{ 0x7977, 0x7977, PDF_CMAP_SINGLE, 3186 },
	{ 0x7979, 0x797c, PDF_CMAP_TABLE, 6326 },
	{ 0x797e, 0x7981, PDF_CMAP_TABLE, 6330 },
	{ 0x7984, 0x7985, PDF_CMAP_TABLE, 6334 },
	{ 0x798a, 0x798f, PDF_CMAP_TABLE, 6336 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 22113 },
	{ 0x7993, 0x7996, PDF_CMAP_TABLE, 6342 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 14912 },
	{ 0x799b, 0x799d, PDF_CMAP_TABLE, 6346 },
	{ 0x79a1, 0x79a1, PDF_CMAP_SINGLE, 18239 },
	{ 0x79a6, 0x79ab, PDF_CMAP_TABLE, 6349 },
	{ 0x79ae, 0x79b1, PDF_CMAP_TABLE, 6355 },
	{ 0x79b3, 0x79b4, PDF_CMAP_TABLE, 6359 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 14913 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 5902 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 14914 },
	{ 0x79bd, 0x79c2, PDF_CMAP_TABLE, 6361 },
	{ 0x79c4, 0x79c4, PDF_CMAP_SINGLE, 19621 },
	{ 0x79c7, 0x79cb, PDF_CMAP_TABLE, 6367 },
	{ 0x79cc, 0x79cd, PDF_CMAP_RANGE, 18244 },
	{ 0x79cf, 0x79cf, PDF_CMAP_SINGLE, 22118 },
	{ 0x79d1, 0x79d2, PDF_CMAP_TABLE, 6372 },
	{ 0x79d4, 0x79d6, PDF_CMAP_TABLE, 6374 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 3455 },
	{ 0x79da, 0x79da, PDF_CMAP_SINGLE, 14916 },
	{ 0x79dd, 0x79e7, PDF_CMAP_TABLE, 6377 },
	{ 0x79e9, 0x79ed, PDF_CMAP_TABLE, 6388 },
	{ 0x79f0, 0x79f1, PDF_CMAP_TABLE, 6393 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 22124 },
	{ 0x79fb, 0x79fc, PDF_CMAP_TABLE, 6395 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 1603 },
	{ 0x7a02, 0x7a03, PDF_CMAP_TABLE, 6397 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 13875 },
	{ 0x7a07, 0x7a0e, PDF_CMAP_TABLE, 6399 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 14919 },
	{ 0x7a14, 0x7a15, PDF_CMAP_TABLE, 6407 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 3477 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 5912 },
	{ 0x7a1a, 0x7a1c, PDF_CMAP_TABLE, 6409 },
	{ 0x7a1e, 0x7a21, PDF_CMAP_TABLE, 6412 },
	{ 0x7a27, 0x7a27, PDF_CMAP_SINGLE, 22128 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 22129 },
	{ 0x7a2d, 0x7a32, PDF_CMAP_TABLE, 6416 },
	{ 0x7a34, 0x7a35, PDF_CMAP_RANGE, 22131 },
	{ 0x7a37, 0x7a40, PDF_CMAP_TABLE, 6422 },
	{ 0x7a42, 0x7a49, PDF_CMAP_TABLE, 6432 },
	{ 0x7a4c, 0x7a50, PDF_CMAP_TABLE, 6440 },
	{ 0x7a55, 0x7a57, PDF_CMAP_TABLE, 6445 },
	{ 0x7a59, 0x7a59, PDF_CMAP_SINGLE, 18258 },
	{ 0x7a5c, 0x7a5d, PDF_CMAP_TABLE, 6448 },
	{ 0x7a5f, 0x7a60, PDF_CMAP_TABLE, 6450 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5924 },
	{ 0x7a63, 0x7a63, PDF_CMAP_SINGLE, 2527 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 22135 },
	{ 0x7a67, 0x7a67, PDF_CMAP_SINGLE, 18261 },
	{ 0x7a69, 0x7a6b, PDF_CMAP_TABLE, 6452 },
	{ 0x7a6d, 0x7a6d, PDF_CMAP_SINGLE, 14927 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 5928 },
	{ 0x7a74, 0x7a76, PDF_CMAP_TABLE, 6455 },
	{ 0x7a78, 0x7a7a, PDF_CMAP_TABLE, 6458 },
	{ 0x7a7d, 0x7a86, PDF_CMAP_TABLE, 6461 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 5931 },
	{ 0x7a8a, 0x7a8b, PDF_CMAP_TABLE, 6471 },
	{ 0x7a90, 0x7a98, PDF_CMAP_TABLE, 6473 },
	{ 0x7a9e, 0x7aa0, PDF_CMAP_TABLE, 6482 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 14930 },
	{ 0x7aa9, 0x7aaa, PDF_CMAP_TABLE, 6485 },
	{ 0x7aac, 0x7aac, PDF_CMAP_SINGLE, 18267 },
	{ 0x7aae, 0x7ab0, PDF_CMAP_TABLE, 6487 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 14931 },
	{ 0x7ab5, 0x7ab6, PDF_CMAP_TABLE, 6490 },
	{ 0x7ab9, 0x7aba, PDF_CMAP_TABLE, 6492 },
	{ 0x7abb, 0x7abc, PDF_CMAP_RANGE, 14932 },
	{ 0x7abd, 0x7abf, PDF_CMAP_TABLE, 6494 },
	{ 0x7ac3, 0x7acf, PDF_CMAP_TABLE, 6497 },
	{ 0x7ad1, 0x7ad3, PDF_CMAP_TABLE, 6510 },
	{ 0x7ad5, 0x7ad5, PDF_CMAP_SINGLE, 5948 },
	{ 0x7ad9, 0x7ada, PDF_CMAP_RANGE, 5950 },
	{ 0x7adb, 0x7add, PDF_CMAP_TABLE, 6513 },
	{ 0x7adf, 0x7ae0, PDF_CMAP_TABLE, 6516 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 5953 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 2401 },
	{ 0x7ae5, 0x7aed, PDF_CMAP_TABLE, 6518 },
	{ 0x7aef, 0x7af1, PDF_CMAP_TABLE, 6527 },
	{ 0x7af4, 0x7af4, PDF_CMAP_SINGLE, 18275 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 1693 },
	{ 0x7af8, 0x7afb, PDF_CMAP_TABLE, 6530 },
	{ 0x7afd, 0x7aff, PDF_CMAP_TABLE, 6534 },
	{ 0x7b02, 0x7b02, PDF_CMAP_SINGLE, 5958 },
	{ 0x7b04, 0x7b04, PDF_CMAP_SINGLE, 5971 },
	{ 0x7b06, 0x7b08, PDF_CMAP_TABLE, 6537 },
	{ 0x7b0a, 0x7b0b, PDF_CMAP_TABLE, 6540 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 5959 },
	{ 0x7b11, 0x7b12, PDF_CMAP_TABLE, 6542 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 14936 },
	{ 0x7b18, 0x7b19, PDF_CMAP_RANGE, 5963 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 3109 },
	{ 0x7b1e, 0x7b20, PDF_CMAP_TABLE, 6544 },
	{ 0x7b23, 0x7b23, PDF_CMAP_SINGLE, 22147 },
	{ 0x7b25, 0x7b2d, PDF_CMAP_TABLE, 6547 },
	{ 0x7b2e, 0x7b2f, PDF_CMAP_RANGE, 18281 },
	{ 0x7b30, 0x7b31, PDF_CMAP_TABLE, 6556 },
	{ 0x7b33, 0x7b36, PDF_CMAP_TABLE, 6558 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 2155 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 17029 },
	{ 0x7b3d, 0x7b3d, PDF_CMAP_SINGLE, 18279 },
	{ 0x7b3f, 0x7b40, PDF_CMAP_RANGE, 22151 },
	{ 0x7b41, 0x7b41, PDF_CMAP_SINGLE, 18286 },
	{ 0x7b45, 0x7b49, PDF_CMAP_TABLE, 6562 },
	{ 0x7b4b, 0x7b56, PDF_CMAP_TABLE, 6567 },
	{ 0x7b5d, 0x7b5d, PDF_CMAP_SINGLE, 5993 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 14941 },
	{ 0x7b64, 0x7b67, PDF_CMAP_TABLE, 6579 },
	{ 0x7b69, 0x7b6a, PDF_CMAP_TABLE, 6583 },
	{ 0x7b6c, 0x7b6f, PDF_CMAP_TABLE, 6585 },
	{ 0x7b70, 0x7b71, PDF_CMAP_RANGE, 5980 },
	{ 0x7b72, 0x7b75, PDF_CMAP_TABLE, 6589 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 19635 },
	{ 0x7b79, 0x7b7a, PDF_CMAP_TABLE, 6593 },
	{ 0x7b7f, 0x7b7f, PDF_CMAP_SINGLE, 18285 },
	{ 0x7b84, 0x7b84, PDF_CMAP_SINGLE, 22154 },
	{ 0x7b86, 0x7b87, PDF_CMAP_TABLE, 6595 },
	{ 0x7b89, 0x7b89, PDF_CMAP_SINGLE, 22155 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 5990 },
	{ 0x7b8d, 0x7b92, PDF_CMAP_TABLE, 6597 },
	{ 0x7b94, 0x7ba1, PDF_CMAP_TABLE, 6603 },
	{ 0x7ba5, 0x7ba5, PDF_CMAP_SINGLE, 22158 },
	{ 0x7baa, 0x7baa, PDF_CMAP_SINGLE, 2939 },
	{ 0x7bac, 0x7bad, PDF_CMAP_TABLE, 6617 },
	{ 0x7baf, 0x7bb2, PDF_CMAP_TABLE, 6619 },
	{ 0x7bb4, 0x7bb6, PDF_CMAP_TABLE, 6623 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 3384 },
	{ 0x7bba, 0x7bbb, PDF_CMAP_RANGE, 22161 },
	{ 0x7bbc, 0x7bbd, PDF_CMAP_TABLE, 6626 },
	{ 0x7bc0, 0x7bc2, PDF_CMAP_TABLE, 6628 },
	{ 0x7bc4, 0x7bcc, PDF_CMAP_TABLE, 6631 },
	{ 0x7bcf, 0x7bcf, PDF_CMAP_SINGLE, 5998 },
	{ 0x7bd4, 0x7bd4, PDF_CMAP_SINGLE, 18304 },
	{ 0x7bd6, 0x7bd7, PDF_CMAP_TABLE, 6640 },
	{ 0x7bd9, 0x7bdb, PDF_CMAP_TABLE, 6642 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 6001 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 2288 },
	{ 0x7be4, 0x7be6, PDF_CMAP_TABLE, 6645 },
	{ 0x7be8, 0x7bea, PDF_CMAP_TABLE, 6648 },
	{ 0x7bed, 0x7bed, PDF_CMAP_SINGLE, 4060 },
	{ 0x7bf0, 0x7bf0, PDF_CMAP_SINGLE, 18308 },
	{ 0x7bf2, 0x7bf3, PDF_CMAP_TABLE, 6651 },
	{ 0x7bf4, 0x7bf5, PDF_CMAP_RANGE, 22167 },
	{ 0x7bf6, 0x7bf8, PDF_CMAP_TABLE, 6653 },
	{ 0x7bf9, 0x7bfa, PDF_CMAP_RANGE, 22169 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 19642 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 19643 },
	{ 0x7c00, 0x7c04, PDF_CMAP_TABLE, 6656 },
	{ 0x7c06, 0x7c07, PDF_CMAP_TABLE, 6661 },
	{ 0x7c09, 0x7c09, PDF_CMAP_SINGLE, 19644 },
	{ 0x7c0b, 0x7c0f, PDF_CMAP_TABLE, 6663 },
	{ 0x7c11, 0x7c14, PDF_CMAP_TABLE, 6668 },
	{ 0x7c17, 0x7c17, PDF_CMAP_SINGLE, 6013 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 22175 },
	{ 0x7c1b, 0x7c1b, PDF_CMAP_SINGLE, 22176 },
	{ 0x7c1e, 0x7c21, PDF_CMAP_TABLE, 6672 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 6016 },
	{ 0x7c25, 0x7c28, PDF_CMAP_TABLE, 6676 },
	{ 0x7c2a, 0x7c2c, PDF_CMAP_TABLE, 6680 },
	{ 0x7c2f, 0x7c2f, PDF_CMAP_SINGLE, 19646 },
	{ 0x7c31, 0x7c31, PDF_CMAP_SINGLE, 14953 },
	{ 0x7c33, 0x7c34, PDF_CMAP_TABLE, 6683 },
	{ 0x7c36, 0x7c38, PDF_CMAP_TABLE, 6685 },
	{ 0x7c39, 0x7c3a, PDF_CMAP_RANGE, 22180 },
	{ 0x7c3d, 0x7c40, PDF_CMAP_TABLE, 6688 },
	{ 0x7c42, 0x7c43, PDF_CMAP_TABLE, 6692 },
	{ 0x7c45, 0x7c46, PDF_CMAP_TABLE, 6694 },
	{ 0x7c4a, 0x7c4a, PDF_CMAP_SINGLE, 18312 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 6696 },
	{ 0x7c4f, 0x7c51, PDF_CMAP_TABLE, 6698 },
	{ 0x7c52, 0x7c53, PDF_CMAP_RANGE, 19648 },
	{ 0x7c54, 0x7c5a, PDF_CMAP_TABLE, 6701 },
	{ 0x7c5b, 0x7c5d, PDF_CMAP_RANGE, 19650 },
	{ 0x7c5e, 0x7c61, PDF_CMAP_TABLE, 6708 },
	{ 0x7c63, 0x7c65, PDF_CMAP_TABLE, 6712 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 14957 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 18316 },
	{ 0x7c6c, 0x7c70, PDF_CMAP_TABLE, 6715 },
	{ 0x7c72, 0x7c73, PDF_CMAP_TABLE, 6720 },
	{ 0x7c75, 0x7c75, PDF_CMAP_SINGLE, 6035 },
	{ 0x7c79, 0x7c79, PDF_CMAP_SINGLE, 17035 },
	{ 0x7c7b, 0x7c7e, PDF_CMAP_TABLE, 6722 },
	{ 0x7c81, 0x7c83, PDF_CMAP_TABLE, 6726 },
	{ 0x7c86, 0x7c87, PDF_CMAP_TABLE, 6729 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 3588 },
	{ 0x7c8b, 0x7c8b, PDF_CMAP_SINGLE, 2606 },
	{ 0x7c8d, 0x7c8d, PDF_CMAP_SINGLE, 3772 },
	{ 0x7c8f, 0x7c90, PDF_CMAP_TABLE, 6731 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 3963 },
	{ 0x7c94, 0x7c95, PDF_CMAP_TABLE, 6733 },
	{ 0x7c97, 0x7c98, PDF_CMAP_TABLE, 6735 },
	{ 0x7c9b, 0x7c9b, PDF_CMAP_SINGLE, 2391 },
	{ 0x7c9e, 0x7ca2, PDF_CMAP_TABLE, 6737 },
	{ 0x7ca4, 0x7ca8, PDF_CMAP_TABLE, 6742 },
	{ 0x7cab, 0x7cab, PDF_CMAP_SINGLE, 6041 },
	{ 0x7cad, 0x7cae, PDF_CMAP_TABLE, 6747 },
	{ 0x7cb0, 0x7cb3, PDF_CMAP_TABLE, 6749 },
	{ 0x7cb6, 0x7cb7, PDF_CMAP_RANGE, 18323 },
	{ 0x7cb9, 0x7cc0, PDF_CMAP_TABLE, 6753 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 6052 },
	{ 0x7cc4, 0x7cc5, PDF_CMAP_TABLE, 6761 },
	{ 0x7cc7, 0x7cc7, PDF_CMAP_SINGLE, 19658 },
	{ 0x7cc8, 0x7cc9, PDF_CMAP_RANGE, 14962 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 1926 },
	{ 0x7ccd, 0x7ccf, PDF_CMAP_TABLE, 6763 },
	{ 0x7cd2, 0x7cda, PDF_CMAP_TABLE, 6766 },
	{ 0x7cdc, 0x7ce0, PDF_CMAP_TABLE, 6775 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 6056 },
	{ 0x7ce6, 0x7ce7, PDF_CMAP_TABLE, 6780 },
	{ 0x7ce9, 0x7ce9, PDF_CMAP_SINGLE, 22193 },
	{ 0x7ceb, 0x7ceb, PDF_CMAP_SINGLE, 14967 },
	{ 0x7cef, 0x7cef, PDF_CMAP_SINGLE, 6058 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 6059 },
	{ 0x7cf4, 0x7cf6, PDF_CMAP_TABLE, 6782 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 2227 },
	{ 0x7cfa, 0x7cfb, PDF_CMAP_TABLE, 6785 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 1668 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 1604 },
	{ 0x7d02, 0x7d06, PDF_CMAP_TABLE, 6787 },
	{ 0x7d07, 0x7d09, PDF_CMAP_RANGE, 14968 },
	{ 0x7d0a, 0x7d0b, PDF_CMAP_TABLE, 6792 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 3314 },
	{ 0x7d0f, 0x7d1e, PDF_CMAP_TABLE, 6794 },
	{ 0x7d20, 0x7d23, PDF_CMAP_TABLE, 6810 },
	{ 0x7d26, 0x7d26, PDF_CMAP_SINGLE, 22197 },
	{ 0x7d2a, 0x7d33, PDF_CMAP_TABLE, 6814 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 6073 },
	{ 0x7d39, 0x7d3a, PDF_CMAP_TABLE, 6824 },
	{ 0x7d3c, 0x7d3c, PDF_CMAP_SINGLE, 19661 },
	{ 0x7d3d, 0x7d3e, PDF_CMAP_RANGE, 18340 },
	{ 0x7d3f, 0x7d48, PDF_CMAP_TABLE, 6826 },
	{ 0x7d4b, 0x7d51, PDF_CMAP_TABLE, 6836 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 14975 },
	{ 0x7d55, 0x7d57, PDF_CMAP_TABLE, 6843 },
	{ 0x7d59, 0x7d5e, PDF_CMAP_TABLE, 6846 },
	{ 0x7d61, 0x7d63, PDF_CMAP_TABLE, 6852 },
	{ 0x7d65, 0x7d68, PDF_CMAP_TABLE, 6855 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 18348 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 6080 },
	{ 0x7d70, 0x7d73, PDF_CMAP_TABLE, 6859 },
	{ 0x7d75, 0x7d76, PDF_CMAP_TABLE, 6863 },
	{ 0x7d78, 0x7d7b, PDF_CMAP_TABLE, 6865 },
	{ 0x7d7d, 0x7d7d, PDF_CMAP_SINGLE, 6087 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 18351 },
	{ 0x7d81, 0x7d83, PDF_CMAP_TABLE, 6869 },
	{ 0x7d85, 0x7d86, PDF_CMAP_TABLE, 6872 },
	{ 0x7d88, 0x7d89, PDF_CMAP_TABLE, 6874 },
	{ 0x7d8b, 0x7d8c, PDF_CMAP_RANGE, 14980 },
	{ 0x7d8d, 0x7d8d, PDF_CMAP_SINGLE, 19665 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 6086 },
	{ 0x7d91, 0x7d91, PDF_CMAP_SINGLE, 19666 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 6083 },
	{ 0x7d96, 0x7d97, PDF_CMAP_TABLE, 6876 },
	{ 0x7d99, 0x7da0, PDF_CMAP_TABLE, 6878 },
	{ 0x7da2, 0x7da3, PDF_CMAP_TABLE, 6886 },
	{ 0x7da6, 0x7da7, PDF_CMAP_TABLE, 6888 },
	{ 0x7daa, 0x7dbb, PDF_CMAP_TABLE, 6890 },
	{ 0x7dbd, 0x7dc0, PDF_CMAP_TABLE, 6908 },
	{ 0x7dc2, 0x7dc2, PDF_CMAP_SINGLE, 17044 },
	{ 0x7dc3, 0x7dc6, PDF_CMAP_RANGE, 22208 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 6093 },
	{ 0x7dca, 0x7dcc, PDF_CMAP_TABLE, 6912 },
	{ 0x7dcd, 0x7dce, PDF_CMAP_RANGE, 22212 },
	{ 0x7dcf, 0x7dd2, PDF_CMAP_TABLE, 6915 },
	{ 0x7dd5, 0x7dda, PDF_CMAP_TABLE, 6919 },
	{ 0x7ddc, 0x7dde, PDF_CMAP_TABLE, 6925 },
	{ 0x7de0, 0x7de6, PDF_CMAP_TABLE, 6928 },
	{ 0x7de8, 0x7ded, PDF_CMAP_TABLE, 6935 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 1186 },
	{ 0x7df1, 0x7df2, PDF_CMAP_TABLE, 6941 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4037 },
	{ 0x7df5, 0x7df6, PDF_CMAP_RANGE, 19672 },
	{ 0x7df9, 0x7dfb, PDF_CMAP_TABLE, 6943 },
	{ 0x7e00, 0x7e01, PDF_CMAP_TABLE, 6946 },
	{ 0x7e04, 0x7e05, PDF_CMAP_TABLE, 6948 },
	{ 0x7e08, 0x7e0b, PDF_CMAP_TABLE, 6950 },
	{ 0x7e10, 0x7e12, PDF_CMAP_TABLE, 6954 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 14988 },
	{ 0x7e17, 0x7e17, PDF_CMAP_SINGLE, 18368 },
	{ 0x7e1b, 0x7e23, PDF_CMAP_TABLE, 6957 },
	{ 0x7e26, 0x7e28, PDF_CMAP_TABLE, 6966 },
	{ 0x7e2b, 0x7e2f, PDF_CMAP_TABLE, 6969 },
	{ 0x7e31, 0x7e33, PDF_CMAP_TABLE, 6974 },
	{ 0x7e35, 0x7e37, PDF_CMAP_TABLE, 6977 },
	{ 0x7e39, 0x7e3b, PDF_CMAP_TABLE, 6980 },
	{ 0x7e3d, 0x7e3f, PDF_CMAP_TABLE, 6983 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 3423 },
	{ 0x7e43, 0x7e48, PDF_CMAP_TABLE, 6986 },
	{ 0x7e4a, 0x7e4b, PDF_CMAP_TABLE, 6992 },
	{ 0x7e4d, 0x7e4e, PDF_CMAP_TABLE, 6994 },
	{ 0x7e50, 0x7e50, PDF_CMAP_SINGLE, 22224 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 8596 },
	{ 0x7e54, 0x7e56, PDF_CMAP_TABLE, 6996 },
	{ 0x7e58, 0x7e58, PDF_CMAP_SINGLE, 22225 },
	{ 0x7e59, 0x7e5a, PDF_CMAP_RANGE, 6133 },
	{ 0x7e5d, 0x7e5f, PDF_CMAP_TABLE, 6999 },
	{ 0x7e61, 0x7e62, PDF_CMAP_TABLE, 7002 },
	{ 0x7e65, 0x7e67, PDF_CMAP_TABLE, 7004 },
	{ 0x7e69, 0x7e6b, PDF_CMAP_TABLE, 7007 },
	{ 0x7e6d, 0x7e70, PDF_CMAP_TABLE, 7010 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 14993 },
	{ 0x7e75, 0x7e75, PDF_CMAP_SINGLE, 18373 },
	{ 0x7e78, 0x7e79, PDF_CMAP_TABLE, 7014 },
	{ 0x7e7b, 0x7e7f, PDF_CMAP_TABLE, 7016 },
	{ 0x7e81, 0x7e83, PDF_CMAP_TABLE, 7021 },
	{ 0x7e86, 0x7e87, PDF_CMAP_RANGE, 18375 },
	{ 0x7e88, 0x7e89, PDF_CMAP_RANGE, 6145 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 8359 },
	{ 0x7e8c, 0x7e92, PDF_CMAP_TABLE, 7024 },
	{ 0x7e93, 0x7e94, PDF_CMAP_RANGE, 6150 },
	{ 0x7e95, 0x7e96, PDF_CMAP_TABLE, 7031 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 14996 },
	{ 0x7e9a, 0x7e9a, PDF_CMAP_SINGLE, 18378 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_RANGE, 6154 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 22229 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 1544 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 6156 },
	{ 0x7f3a, 0x7f3f, PDF_CMAP_TABLE, 7033 },
	{ 0x7f43, 0x7f45, PDF_CMAP_TABLE, 7039 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 8597 },
	{ 0x7f4c, 0x7f4e, PDF_CMAP_RANGE, 6159 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 14998 },
	{ 0x7f50, 0x7f51, PDF_CMAP_RANGE, 6162 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 14999 },
	{ 0x7f54, 0x7f55, PDF_CMAP_TABLE, 7042 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 6166 },
	{ 0x7f5b, 0x7f5d, PDF_CMAP_TABLE, 7044 },
	{ 0x7f5f, 0x7f60, PDF_CMAP_RANGE, 6167 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 15001 },
	{ 0x7f63, 0x7f64, PDF_CMAP_RANGE, 18387 },
	{ 0x7f65, 0x7f67, PDF_CMAP_TABLE, 7047 },
	{ 0x7f68, 0x7f69, PDF_CMAP_RANGE, 6169 },
	{ 0x7f6a, 0x7f6b, PDF_CMAP_TABLE, 7050 },
	{ 0x7f6d, 0x7f6e, PDF_CMAP_TABLE, 7052 },
	{ 0x7f70, 0x7f72, PDF_CMAP_TABLE, 7054 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 3331 },
	{ 0x7f77, 0x7f79, PDF_CMAP_TABLE, 7057 },
	{ 0x7f7d, 0x7f7e, PDF_CMAP_RANGE, 18390 },
	{ 0x7f7f, 0x7f80, PDF_CMAP_RANGE, 19685 },
	{ 0x7f82, 0x7f83, PDF_CMAP_TABLE, 7060 },
	{ 0x7f85, 0x7f88, PDF_CMAP_TABLE, 7062 },
	{ 0x7f8a, 0x7f91, PDF_CMAP_TABLE, 7066 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 6179 },
	{ 0x7f96, 0x7f97, PDF_CMAP_TABLE, 7074 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 6182 },
	{ 0x7f9c, 0x7f9e, PDF_CMAP_TABLE, 7076 },
	{ 0x7fa1, 0x7fa4, PDF_CMAP_TABLE, 7079 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 22237 },
	{ 0x7fa8, 0x7faa, PDF_CMAP_TABLE, 7083 },
	{ 0x7fad, 0x7faf, PDF_CMAP_TABLE, 7086 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 6185 },
	{ 0x7fb4, 0x7fb4, PDF_CMAP_SINGLE, 22239 },
	{ 0x7fb6, 0x7fb6, PDF_CMAP_SINGLE, 6188 },
	{ 0x7fb8, 0x7fb9, PDF_CMAP_TABLE, 7089 },
	{ 0x7fbc, 0x7fbd, PDF_CMAP_TABLE, 7091 },
	{ 0x7fbf, 0x7fc1, PDF_CMAP_TABLE, 7093 },
	{ 0x7fc3, 0x7fc3, PDF_CMAP_SINGLE, 18399 },
	{ 0x7fc5, 0x7fc6, PDF_CMAP_RANGE, 6191 },
	{ 0x7fc8, 0x7fc8, PDF_CMAP_SINGLE, 22242 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 6193 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 3916 },
	{ 0x7fce, 0x7fcf, PDF_CMAP_TABLE, 7096 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2358 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_TABLE, 7098 },
	{ 0x7fdb, 0x7fdb, PDF_CMAP_SINGLE, 17047 },
	{ 0x7fdf, 0x7fe1, PDF_CMAP_TABLE, 7100 },
	{ 0x7fe3, 0x7fe3, PDF_CMAP_SINGLE, 18401 },
	{ 0x7fe5, 0x7fe6, PDF_CMAP_TABLE, 7103 },
	{ 0x7fe8, 0x7fe9, PDF_CMAP_TABLE, 7105 },
	{ 0x7feb, 0x7fec, PDF_CMAP_TABLE, 7107 },
	{ 0x7fee, 0x7fef, PDF_CMAP_RANGE, 15008 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 1545 },
	{ 0x7ff2, 0x7ff3, PDF_CMAP_TABLE, 7109 },
	{ 0x7ff9, 0x7ffc, PDF_CMAP_TABLE, 7111 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 19689 },
	{ 0x8000, 0x8008, PDF_CMAP_TABLE, 7115 },
	{ 0x800a, 0x8017, PDF_CMAP_TABLE, 7124 },
	{ 0x8018, 0x8019, PDF_CMAP_RANGE, 6206 },
	{ 0x801c, 0x801e, PDF_CMAP_TABLE, 7138 },
	{ 0x801f, 0x8020, PDF_CMAP_RANGE, 22247 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 6209 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 15014 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 15015 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 6210 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 18406 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 22249 },
	{ 0x8030, 0x8030, PDF_CMAP_SINGLE, 18407 },
	{ 0x8033, 0x8037, PDF_CMAP_TABLE, 7141 },
	{ 0x8039, 0x8040, PDF_CMAP_TABLE, 7146 },
	{ 0x8043, 0x8044, PDF_CMAP_TABLE, 7154 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 6214 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 6213 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 6215 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 2655 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 6216 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 6217 },
	{ 0x805e, 0x8062, PDF_CMAP_TABLE, 7156 },
	{ 0x8064, 0x8064, PDF_CMAP_SINGLE, 22254 },
	{ 0x8066, 0x8066, PDF_CMAP_SINGLE, 18409 },
	{ 0x8068, 0x8068, PDF_CMAP_SINGLE, 6220 },
	{ 0x806d, 0x806d, PDF_CMAP_SINGLE, 22255 },
	{ 0x806f, 0x8077, PDF_CMAP_TABLE, 7161 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 6225 },
	{ 0x807b, 0x807b, PDF_CMAP_SINGLE, 18410 },
	{ 0x807d, 0x807f, PDF_CMAP_TABLE, 7170 },
	{ 0x8081, 0x8081, PDF_CMAP_SINGLE, 22256 },
	{ 0x8084, 0x8089, PDF_CMAP_TABLE, 7173 },
	{ 0x808b, 0x808c, PDF_CMAP_TABLE, 7179 },
	{ 0x808e, 0x808e, PDF_CMAP_SINGLE, 19697 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6232 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 2491 },
	{ 0x8098, 0x809e, PDF_CMAP_TABLE, 7181 },
	{ 0x80a1, 0x80a2, PDF_CMAP_TABLE, 7188 },
	{ 0x80a4, 0x80a7, PDF_CMAP_TABLE, 7190 },
	{ 0x80a9, 0x80ad, PDF_CMAP_TABLE, 7194 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 2016 },
	{ 0x80b1, 0x80b2, PDF_CMAP_TABLE, 7199 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 2136 },
	{ 0x80b8, 0x80ba, PDF_CMAP_TABLE, 7201 },
	{ 0x80c3, 0x80c6, PDF_CMAP_TABLE, 7204 },
	{ 0x80c8, 0x80c8, PDF_CMAP_SINGLE, 22258 },
	{ 0x80ca, 0x80ca, PDF_CMAP_SINGLE, 17050 },
	{ 0x80cc, 0x80cf, PDF_CMAP_TABLE, 7208 },
	{ 0x80d2, 0x80d2, PDF_CMAP_SINGLE, 22260 },
	{ 0x80d4, 0x80d6, PDF_CMAP_TABLE, 7212 },
	{ 0x80d7, 0x80d8, PDF_CMAP_RANGE, 15024 },
	{ 0x80d9, 0x80db, PDF_CMAP_TABLE, 7215 },
	{ 0x80dd, 0x80de, PDF_CMAP_TABLE, 7218 },
	{ 0x80e0, 0x80e1, PDF_CMAP_TABLE, 7220 },
	{ 0x80e4, 0x80e6, PDF_CMAP_TABLE, 7222 },
	{ 0x80ed, 0x80fe, PDF_CMAP_TABLE, 7225 },
	{ 0x8102, 0x8103, PDF_CMAP_TABLE, 7243 },
	{ 0x8105, 0x810b, PDF_CMAP_TABLE, 7245 },
	{ 0x810d, 0x810d, PDF_CMAP_SINGLE, 18420 },
	{ 0x8116, 0x8118, PDF_CMAP_TABLE, 7252 },
	{ 0x811a, 0x811c, PDF_CMAP_TABLE, 7255 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 18425 },
	{ 0x8120, 0x8120, PDF_CMAP_SINGLE, 22267 },
	{ 0x8123, 0x8124, PDF_CMAP_TABLE, 7258 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 18428 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 6248 },
	{ 0x812b, 0x812c, PDF_CMAP_TABLE, 7260 },
	{ 0x812f, 0x8131, PDF_CMAP_TABLE, 7262 },
	{ 0x8133, 0x8133, PDF_CMAP_SINGLE, 3316 },
	{ 0x8135, 0x8135, PDF_CMAP_SINGLE, 18424 },
	{ 0x8139, 0x813a, PDF_CMAP_TABLE, 7265 },
	{ 0x813c, 0x813e, PDF_CMAP_TABLE, 7267 },
	{ 0x8141, 0x8141, PDF_CMAP_SINGLE, 14194 },
	{ 0x8145, 0x8147, PDF_CMAP_TABLE, 7270 },
	{ 0x814a, 0x814c, PDF_CMAP_TABLE, 7273 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 2587 },
	{ 0x8150, 0x8155, PDF_CMAP_TABLE, 7276 },
	{ 0x8157, 0x8157, PDF_CMAP_SINGLE, 19708 },
	{ 0x815f, 0x8161, PDF_CMAP_TABLE, 7282 },
	{ 0x8165, 0x8166, PDF_CMAP_RANGE, 6260 },
	{ 0x8167, 0x8168, PDF_CMAP_RANGE, 17054 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 18433 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 2332 },
	{ 0x816d, 0x8171, PDF_CMAP_TABLE, 7285 },
	{ 0x8173, 0x8174, PDF_CMAP_TABLE, 7290 },
	{ 0x8177, 0x817a, PDF_CMAP_TABLE, 7292 },
	{ 0x817f, 0x8186, PDF_CMAP_TABLE, 7296 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 6264 },
	{ 0x818a, 0x818b, PDF_CMAP_TABLE, 7304 },
	{ 0x818e, 0x8190, PDF_CMAP_TABLE, 7306 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 6273 },
	{ 0x8195, 0x8196, PDF_CMAP_TABLE, 7309 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 18438 },
	{ 0x819a, 0x819e, PDF_CMAP_TABLE, 7311 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 6268 },
	{ 0x81a2, 0x81a4, PDF_CMAP_TABLE, 7316 },
	{ 0x81a8, 0x81a9, PDF_CMAP_TABLE, 7319 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 22278 },
	{ 0x81b0, 0x81b0, PDF_CMAP_SINGLE, 6275 },
	{ 0x81b2, 0x81b5, PDF_CMAP_TABLE, 7321 },
	{ 0x81b8, 0x81b8, PDF_CMAP_SINGLE, 6278 },
	{ 0x81ba, 0x81bb, PDF_CMAP_TABLE, 7325 },
	{ 0x81bd, 0x81c3, PDF_CMAP_TABLE, 7327 },
	{ 0x81c5, 0x81c6, PDF_CMAP_TABLE, 7334 },
	{ 0x81c8, 0x81cb, PDF_CMAP_TABLE, 7336 },
	{ 0x81cd, 0x81cf, PDF_CMAP_TABLE, 7340 },
	{ 0x81d1, 0x81d1, PDF_CMAP_SINGLE, 6285 },
	{ 0x81d3, 0x81d3, PDF_CMAP_SINGLE, 2817 },
	{ 0x81d5, 0x81db, PDF_CMAP_TABLE, 7343 },
	{ 0x81dd, 0x81de, PDF_CMAP_RANGE, 19717 },
	{ 0x81df, 0x81e0, PDF_CMAP_RANGE, 6290 },
	{ 0x81e1, 0x81e1, PDF_CMAP_SINGLE, 19719 },
	{ 0x81e3, 0x81e5, PDF_CMAP_TABLE, 7350 },
	{ 0x81e7, 0x81e8, PDF_CMAP_TABLE, 7353 },
	{ 0x81ea, 0x81ed, PDF_CMAP_TABLE, 7355 },
	{ 0x81ef, 0x81ef, PDF_CMAP_SINGLE, 19720 },
	{ 0x81f0, 0x81f2, PDF_CMAP_RANGE, 22282 },
	{ 0x81f3, 0x81f6, PDF_CMAP_TABLE, 7359 },
	{ 0x81f8, 0x81f9, PDF_CMAP_TABLE, 7363 },
	{ 0x81fa, 0x81fb, PDF_CMAP_RANGE, 6293 },
	{ 0x81fc, 0x8200, PDF_CMAP_TABLE, 7365 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 6296 },
	{ 0x8203, 0x8205, PDF_CMAP_TABLE, 7370 },
	{ 0x8207, 0x8210, PDF_CMAP_TABLE, 7373 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 4105 },
	{ 0x8213, 0x8214, PDF_CMAP_RANGE, 19724 },
	{ 0x8216, 0x821f, PDF_CMAP_TABLE, 7383 },
	{ 0x8221, 0x8222, PDF_CMAP_TABLE, 7393 },
	{ 0x8228, 0x822c, PDF_CMAP_TABLE, 7395 },
	{ 0x822e, 0x822e, PDF_CMAP_SINGLE, 6319 },
	{ 0x8232, 0x823a, PDF_CMAP_TABLE, 7400 },
	{ 0x823c, 0x823c, PDF_CMAP_SINGLE, 18456 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 6308 },
	{ 0x8243, 0x8247, PDF_CMAP_TABLE, 7409 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 18457 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 15041 },
	{ 0x824e, 0x824f, PDF_CMAP_TABLE, 7414 },
	{ 0x8251, 0x8251, PDF_CMAP_SINGLE, 22293 },
	{ 0x8256, 0x825a, PDF_CMAP_TABLE, 7416 },
	{ 0x825c, 0x825d, PDF_CMAP_TABLE, 7421 },
	{ 0x825f, 0x8260, PDF_CMAP_TABLE, 7423 },
	{ 0x8262, 0x8264, PDF_CMAP_TABLE, 7425 },
	{ 0x8266, 0x8268, PDF_CMAP_TABLE, 7428 },
	{ 0x826a, 0x826b, PDF_CMAP_RANGE, 6317 },
	{ 0x826d, 0x826f, PDF_CMAP_TABLE, 7431 },
	{ 0x8271, 0x8272, PDF_CMAP_TABLE, 7434 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 17062 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 1298 },
	{ 0x8277, 0x8278, PDF_CMAP_RANGE, 6321 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 14197 },
	{ 0x827b, 0x827b, PDF_CMAP_SINGLE, 22296 },
	{ 0x827d, 0x827f, PDF_CMAP_TABLE, 7436 },
	{ 0x8280, 0x8281, PDF_CMAP_RANGE, 22297 },
	{ 0x8283, 0x8284, PDF_CMAP_TABLE, 7439 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 22299 },
	{ 0x8289, 0x828b, PDF_CMAP_TABLE, 7441 },
	{ 0x828d, 0x828e, PDF_CMAP_TABLE, 7444 },
	{ 0x8291, 0x8294, PDF_CMAP_TABLE, 7446 },
	{ 0x8296, 0x8296, PDF_CMAP_SINGLE, 22301 },
	{ 0x8298, 0x8299, PDF_CMAP_TABLE, 7450 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 22303 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 2291 },
	{ 0x829f, 0x82a1, PDF_CMAP_TABLE, 7452 },
	{ 0x82a3, 0x82a4, PDF_CMAP_RANGE, 17065 },
	{ 0x82a5, 0x82a6, PDF_CMAP_TABLE, 7455 },
	{ 0x82a7, 0x82a8, PDF_CMAP_RANGE, 18473 },
	{ 0x82a9, 0x82b4, PDF_CMAP_TABLE, 7457 },
	{ 0x82b7, 0x82bf, PDF_CMAP_TABLE, 7469 },
	{ 0x82c5, 0x82c6, PDF_CMAP_TABLE, 7478 },
	{ 0x82d0, 0x82d5, PDF_CMAP_TABLE, 7480 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 3510 },
	{ 0x82d9, 0x82dc, PDF_CMAP_TABLE, 7486 },
	{ 0x82de, 0x82e8, PDF_CMAP_TABLE, 7490 },
	{ 0x82ea, 0x82eb, PDF_CMAP_TABLE, 7501 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 22309 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 19737 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 1267 },
	{ 0x82f3, 0x82f4, PDF_CMAP_TABLE, 7503 },
	{ 0x82f6, 0x82f7, PDF_CMAP_TABLE, 7505 },
	{ 0x82f9, 0x82fb, PDF_CMAP_TABLE, 7507 },
	{ 0x82fd, 0x82fe, PDF_CMAP_TABLE, 7510 },
	{ 0x8300, 0x8306, PDF_CMAP_TABLE, 7512 },
	{ 0x8307, 0x8308, PDF_CMAP_RANGE, 18482 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 6344 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 22310 },
	{ 0x830c, 0x830c, PDF_CMAP_SINGLE, 18484 },
	{ 0x830e, 0x830e, PDF_CMAP_SINGLE, 1834 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 6348 },
	{ 0x8317, 0x8318, PDF_CMAP_RANGE, 6357 },
	{ 0x831b, 0x831d, PDF_CMAP_TABLE, 7519 },
	{ 0x831e, 0x831f, PDF_CMAP_RANGE, 22312 },
	{ 0x8321, 0x8323, PDF_CMAP_TABLE, 7522 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 1205 },
	{ 0x832b, 0x833a, PDF_CMAP_TABLE, 7525 },
	{ 0x833c, 0x833d, PDF_CMAP_TABLE, 7541 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 6351 },
	{ 0x8342, 0x8347, PDF_CMAP_TABLE, 7543 },
	{ 0x8349, 0x834a, PDF_CMAP_TABLE, 7549 },
	{ 0x834d, 0x834e, PDF_CMAP_RANGE, 22321 },
	{ 0x834f, 0x8358, PDF_CMAP_TABLE, 7551 },
	{ 0x835a, 0x835a, PDF_CMAP_SINGLE, 14202 },
	{ 0x8362, 0x8363, PDF_CMAP_TABLE, 7561 },
	{ 0x8370, 0x8370, PDF_CMAP_SINGLE, 22323 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 6371 },
	{ 0x8375, 0x8375, PDF_CMAP_SINGLE, 6372 },
	{ 0x8377, 0x8378, PDF_CMAP_TABLE, 7563 },
	{ 0x837b, 0x837d, PDF_CMAP_TABLE, 7565 },
	{ 0x837f, 0x8380, PDF_CMAP_TABLE, 7568 },
	{ 0x8382, 0x8382, PDF_CMAP_SINGLE, 22325 },
	{ 0x8384, 0x8387, PDF_CMAP_TABLE, 7570 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 7574 },
	{ 0x838d, 0x838e, PDF_CMAP_TABLE, 7576 },
	{ 0x8392, 0x8393, PDF_CMAP_TABLE, 7578 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 18495 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 6364 },
	{ 0x8398, 0x83a0, PDF_CMAP_TABLE, 7580 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 6363 },
	{ 0x83a6, 0x83ad, PDF_CMAP_TABLE, 7589 },
	{ 0x83b1, 0x83b1, PDF_CMAP_SINGLE, 3923 },
	{ 0x83b5, 0x83b5, PDF_CMAP_SINGLE, 6370 },
	{ 0x83bd, 0x83be, PDF_CMAP_TABLE, 7597 },
	{ 0x83bf, 0x83c0, PDF_CMAP_RANGE, 15057 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 6384 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 2625 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 8603 },
	{ 0x83c9, 0x83ca, PDF_CMAP_TABLE, 7599 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 1749 },
	{ 0x83ce, 0x83d1, PDF_CMAP_TABLE, 7601 },
	{ 0x83d3, 0x83d4, PDF_CMAP_TABLE, 7605 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 2492 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 6382 },
	{ 0x83dc, 0x83dd, PDF_CMAP_TABLE, 7607 },
	{ 0x83df, 0x83e1, PDF_CMAP_TABLE, 7609 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 18503 },
	{ 0x83e8, 0x83eb, PDF_CMAP_TABLE, 7612 },
	{ 0x83ef, 0x83f2, PDF_CMAP_TABLE, 7616 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 6376 },
	{ 0x83f6, 0x83f9, PDF_CMAP_TABLE, 7620 },
	{ 0x83fb, 0x83fd, PDF_CMAP_TABLE, 7624 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 17081 },
	{ 0x8403, 0x8404, PDF_CMAP_TABLE, 7627 },
	{ 0x8406, 0x8407, PDF_CMAP_TABLE, 7629 },
	{ 0x840a, 0x840f, PDF_CMAP_TABLE, 7631 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 15061 },
	{ 0x8413, 0x8413, PDF_CMAP_SINGLE, 6377 },
	{ 0x8415, 0x8415, PDF_CMAP_SINGLE, 18505 },
	{ 0x8417, 0x8417, PDF_CMAP_SINGLE, 18507 },
	{ 0x8419, 0x8419, PDF_CMAP_SINGLE, 22334 },
	{ 0x8420, 0x8420, PDF_CMAP_SINGLE, 6391 },
	{ 0x8422, 0x8422, PDF_CMAP_SINGLE, 6390 },
	{ 0x8429, 0x842a, PDF_CMAP_TABLE, 7637 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 6408 },
	{ 0x842f, 0x842f, PDF_CMAP_SINGLE, 22336 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 1500 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 6411 },
	{ 0x8438, 0x8439, PDF_CMAP_TABLE, 7639 },
	{ 0x843c, 0x843d, PDF_CMAP_TABLE, 7641 },
	{ 0x8445, 0x844a, PDF_CMAP_TABLE, 7643 },
	{ 0x844d, 0x844f, PDF_CMAP_TABLE, 7649 },
	{ 0x8451, 0x8452, PDF_CMAP_RANGE, 18510 },
	{ 0x8456, 0x8458, PDF_CMAP_TABLE, 7652 },
	{ 0x8459, 0x845a, PDF_CMAP_RANGE, 18512 },
	{ 0x845b, 0x845c, PDF_CMAP_TABLE, 7655 },
	{ 0x845f, 0x8467, PDF_CMAP_TABLE, 7657 },
	{ 0x8469, 0x8471, PDF_CMAP_TABLE, 7666 },
	{ 0x8473, 0x847a, PDF_CMAP_TABLE, 7675 },
	{ 0x847c, 0x847d, PDF_CMAP_TABLE, 7683 },
	{ 0x8481, 0x8482, PDF_CMAP_TABLE, 7685 },
	{ 0x8484, 0x8485, PDF_CMAP_TABLE, 7687 },
	{ 0x848b, 0x848b, PDF_CMAP_SINGLE, 2493 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 2361 },
	{ 0x8492, 0x8495, PDF_CMAP_TABLE, 7689 },
	{ 0x8497, 0x8497, PDF_CMAP_SINGLE, 18521 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 3812 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3513 },
	{ 0x849e, 0x849f, PDF_CMAP_TABLE, 7693 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 6425 },
	{ 0x84a6, 0x84a6, PDF_CMAP_SINGLE, 18522 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 15064 },
	{ 0x84a9, 0x84aa, PDF_CMAP_RANGE, 22350 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 6403 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 15065 },
	{ 0x84b1, 0x84b2, PDF_CMAP_TABLE, 7695 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 8606 },
	{ 0x84b8, 0x84c2, PDF_CMAP_TABLE, 7697 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 2973 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 6423 },
	{ 0x84c7, 0x84c8, PDF_CMAP_RANGE, 22352 },
	{ 0x84c9, 0x84cd, PDF_CMAP_TABLE, 7708 },
	{ 0x84ce, 0x84cf, PDF_CMAP_RANGE, 18525 },
	{ 0x84d0, 0x84d1, PDF_CMAP_TABLE, 7713 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 18527 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 6424 },
	{ 0x84d9, 0x84da, PDF_CMAP_TABLE, 7715 },
	{ 0x84dc, 0x84dc, PDF_CMAP_SINGLE, 8363 },
	{ 0x84e7, 0x84e7, PDF_CMAP_SINGLE, 18529 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 18530 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 3671 },
	{ 0x84ee, 0x84f2, PDF_CMAP_TABLE, 7717 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 6428 },
	{ 0x84f7, 0x84f7, PDF_CMAP_SINGLE, 22356 },
	{ 0x84fa, 0x84fd, PDF_CMAP_TABLE, 7722 },
	{ 0x84ff, 0x8500, PDF_CMAP_TABLE, 7726 },
	{ 0x8502, 0x8503, PDF_CMAP_RANGE, 22357 },
	{ 0x8506, 0x8507, PDF_CMAP_TABLE, 7728 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 15070 },
	{ 0x850e, 0x850e, PDF_CMAP_SINGLE, 22360 },
	{ 0x8510, 0x8511, PDF_CMAP_TABLE, 7730 },
	{ 0x8513, 0x8515, PDF_CMAP_TABLE, 7732 },
	{ 0x8517, 0x8518, PDF_CMAP_RANGE, 6429 },
	{ 0x851a, 0x851c, PDF_CMAP_TABLE, 7735 },
	{ 0x851e, 0x851f, PDF_CMAP_TABLE, 7738 },
	{ 0x8521, 0x8523, PDF_CMAP_TABLE, 7740 },
	{ 0x8524, 0x8525, PDF_CMAP_RANGE, 18535 },
	{ 0x8526, 0x8527, PDF_CMAP_TABLE, 7743 },
	{ 0x852a, 0x852d, PDF_CMAP_TABLE, 7745 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 17092 },
	{ 0x8532, 0x8536, PDF_CMAP_TABLE, 7749 },
	{ 0x853d, 0x8541, PDF_CMAP_TABLE, 7754 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 3437 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 19748 },
	{ 0x8548, 0x854b, PDF_CMAP_TABLE, 7759 },
	{ 0x854e, 0x8553, PDF_CMAP_TABLE, 7763 },
	{ 0x8555, 0x855a, PDF_CMAP_TABLE, 7769 },
	{ 0x855c, 0x855e, PDF_CMAP_TABLE, 7775 },
	{ 0x855f, 0x8560, PDF_CMAP_RANGE, 22373 },
	{ 0x8561, 0x8562, PDF_CMAP_RANGE, 18542 },
	{ 0x8563, 0x8564, PDF_CMAP_TABLE, 7778 },
	{ 0x8568, 0x856b, PDF_CMAP_TABLE, 7780 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 6450 },
	{ 0x856f, 0x856f, PDF_CMAP_SINGLE, 18539 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 6456 },
	{ 0x8579, 0x857b, PDF_CMAP_TABLE, 7784 },
	{ 0x857d, 0x8581, PDF_CMAP_TABLE, 7787 },
	{ 0x8584, 0x858c, PDF_CMAP_TABLE, 7792 },
	{ 0x858f, 0x8591, PDF_CMAP_TABLE, 7801 },
	{ 0x8593, 0x8594, PDF_CMAP_TABLE, 7804 },
	{ 0x8597, 0x8599, PDF_CMAP_TABLE, 7806 },
	{ 0x859b, 0x859d, PDF_CMAP_TABLE, 7809 },
	{ 0x859f, 0x85a0, PDF_CMAP_TABLE, 7812 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 17096 },
	{ 0x85a4, 0x85b0, PDF_CMAP_TABLE, 7814 },
	{ 0x85b4, 0x85b4, PDF_CMAP_SINGLE, 22381 },
	{ 0x85b6, 0x85ba, PDF_CMAP_TABLE, 7827 },
	{ 0x85bc, 0x85bc, PDF_CMAP_SINGLE, 18556 },
	{ 0x85bd, 0x85bf, PDF_CMAP_RANGE, 22384 },
	{ 0x85c1, 0x85c2, PDF_CMAP_TABLE, 7832 },
	{ 0x85c7, 0x85c7, PDF_CMAP_SINGLE, 18557 },
	{ 0x85c9, 0x85cb, PDF_CMAP_TABLE, 7834 },
	{ 0x85cd, 0x85d0, PDF_CMAP_TABLE, 7837 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 6464 },
	{ 0x85d8, 0x85d9, PDF_CMAP_RANGE, 18559 },
	{ 0x85da, 0x85da, PDF_CMAP_SINGLE, 22388 },
	{ 0x85dc, 0x85dd, PDF_CMAP_TABLE, 7841 },
	{ 0x85df, 0x85e1, PDF_CMAP_TABLE, 7843 },
	{ 0x85e4, 0x85e6, PDF_CMAP_TABLE, 7846 },
	{ 0x85e8, 0x85ea, PDF_CMAP_TABLE, 7849 },
	{ 0x85ed, 0x85ed, PDF_CMAP_SINGLE, 17098 },
	{ 0x85f3, 0x85f4, PDF_CMAP_TABLE, 7852 },
	{ 0x85f6, 0x85f7, PDF_CMAP_TABLE, 7854 },
	{ 0x85f9, 0x85fc, PDF_CMAP_TABLE, 7856 },
	{ 0x85fe, 0x8600, PDF_CMAP_TABLE, 7860 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 6441 },
	{ 0x8604, 0x8605, PDF_CMAP_RANGE, 17100 },
	{ 0x8606, 0x8607, PDF_CMAP_TABLE, 7863 },
	{ 0x860a, 0x860b, PDF_CMAP_TABLE, 7865 },
	{ 0x860d, 0x860e, PDF_CMAP_RANGE, 22393 },
	{ 0x8610, 0x8613, PDF_CMAP_TABLE, 7867 },
	{ 0x8616, 0x861b, PDF_CMAP_TABLE, 7871 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 18567 },
	{ 0x8621, 0x8622, PDF_CMAP_TABLE, 7877 },
	{ 0x8624, 0x8624, PDF_CMAP_SINGLE, 18569 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 18570 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 15078 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 3936 },
	{ 0x862f, 0x8630, PDF_CMAP_TABLE, 7879 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 22397 },
	{ 0x8638, 0x863a, PDF_CMAP_TABLE, 7881 },
	{ 0x863c, 0x863d, PDF_CMAP_TABLE, 7884 },
	{ 0x863f, 0x8642, PDF_CMAP_TABLE, 7886 },
	{ 0x8646, 0x8646, PDF_CMAP_SINGLE, 19754 },
	{ 0x864d, 0x864e, PDF_CMAP_TABLE, 7890 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 1646 },
	{ 0x8652, 0x8657, PDF_CMAP_TABLE, 7892 },
	{ 0x8658, 0x8659, PDF_CMAP_RANGE, 22400 },
	{ 0x865a, 0x865f, PDF_CMAP_TABLE, 7898 },
	{ 0x8660, 0x8661, PDF_CMAP_RANGE, 22403 },
	{ 0x8662, 0x8664, PDF_CMAP_TABLE, 7904 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 6483 },
	{ 0x8669, 0x8669, PDF_CMAP_SINGLE, 22406 },
	{ 0x866b, 0x866c, PDF_CMAP_TABLE, 7907 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 15082 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 6484 },
	{ 0x8675, 0x8677, PDF_CMAP_TABLE, 7909 },
	{ 0x8679, 0x867b, PDF_CMAP_TABLE, 7912 },
	{ 0x867d, 0x867d, PDF_CMAP_SINGLE, 14214 },
	{ 0x8687, 0x868a, PDF_CMAP_TABLE, 7915 },
	{ 0x868b, 0x868c, PDF_CMAP_RANGE, 6489 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 15084 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 15085 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 6485 },
	{ 0x8695, 0x8696, PDF_CMAP_TABLE, 7919 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 15086 },
	{ 0x869a, 0x869a, PDF_CMAP_SINGLE, 22409 },
	{ 0x869c, 0x869d, PDF_CMAP_RANGE, 18581 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 22410 },
	{ 0x86a3, 0x86a4, PDF_CMAP_TABLE, 7921 },
	{ 0x86a6, 0x86a6, PDF_CMAP_SINGLE, 22411 },
	{ 0x86a7, 0x86a8, PDF_CMAP_RANGE, 15087 },
	{ 0x86a9, 0x86aa, PDF_CMAP_RANGE, 6487 },
	{ 0x86ab, 0x86ab, PDF_CMAP_SINGLE, 6498 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 22412 },
	{ 0x86af, 0x86b1, PDF_CMAP_TABLE, 7923 },
	{ 0x86b3, 0x86b3, PDF_CMAP_SINGLE, 18584 },
	{ 0x86b4, 0x86b5, PDF_CMAP_RANGE, 22413 },
	{ 0x86b6, 0x86b9, PDF_CMAP_TABLE, 7926 },
	{ 0x86bf, 0x86c1, PDF_CMAP_TABLE, 7930 },
	{ 0x86c3, 0x86c7, PDF_CMAP_TABLE, 7933 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 6496 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 2943 },
	{ 0x86cd, 0x86ce, PDF_CMAP_TABLE, 7938 },
	{ 0x86d1, 0x86d2, PDF_CMAP_TABLE, 7940 },
	{ 0x86d4, 0x86d5, PDF_CMAP_TABLE, 7942 },
	{ 0x86d7, 0x86d7, PDF_CMAP_SINGLE, 18589 },
	{ 0x86d9, 0x86dc, PDF_CMAP_TABLE, 7944 },
	{ 0x86de, 0x86e0, PDF_CMAP_TABLE, 7948 },
	{ 0x86e3, 0x86e7, PDF_CMAP_TABLE, 7951 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 6501 },
	{ 0x86ec, 0x86ef, PDF_CMAP_TABLE, 7956 },
	{ 0x86f8, 0x86fe, PDF_CMAP_TABLE, 7960 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 6509 },
	{ 0x8702, 0x870b, PDF_CMAP_TABLE, 7967 },
	{ 0x870d, 0x8714, PDF_CMAP_TABLE, 7977 },
	{ 0x8718, 0x871a, PDF_CMAP_TABLE, 7985 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 3766 },
	{ 0x871e, 0x871f, PDF_CMAP_TABLE, 7988 },
	{ 0x8721, 0x8723, PDF_CMAP_TABLE, 7990 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 6521 },
	{ 0x8728, 0x8729, PDF_CMAP_TABLE, 7993 },
	{ 0x872e, 0x872f, PDF_CMAP_TABLE, 7995 },
	{ 0x8731, 0x8732, PDF_CMAP_TABLE, 7997 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 6517 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 6519 },
	{ 0x8739, 0x873b, PDF_CMAP_TABLE, 7999 },
	{ 0x873c, 0x873d, PDF_CMAP_RANGE, 22430 },
	{ 0x873e, 0x8740, PDF_CMAP_TABLE, 8002 },
	{ 0x8743, 0x8743, PDF_CMAP_SINGLE, 18603 },
	{ 0x8745, 0x8745, PDF_CMAP_SINGLE, 22432 },
	{ 0x8749, 0x8749, PDF_CMAP_SINGLE, 2698 },
	{ 0x874b, 0x874e, PDF_CMAP_TABLE, 8005 },
	{ 0x8751, 0x8751, PDF_CMAP_SINGLE, 18604 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 6534 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 2544 },
	{ 0x8757, 0x8759, PDF_CMAP_TABLE, 8009 },
	{ 0x875d, 0x875d, PDF_CMAP_SINGLE, 19760 },
	{ 0x875f, 0x8761, PDF_CMAP_TABLE, 8012 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 6535 },
	{ 0x8764, 0x8765, PDF_CMAP_RANGE, 18606 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 1372 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 6531 },
	{ 0x876a, 0x876a, PDF_CMAP_SINGLE, 6536 },
	{ 0x876e, 0x876f, PDF_CMAP_TABLE, 8015 },
	{ 0x8771, 0x8772, PDF_CMAP_TABLE, 8017 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 6529 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 3023 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 6526 },
	{ 0x877b, 0x877c, PDF_CMAP_TABLE, 8019 },
	{ 0x877f, 0x877f, PDF_CMAP_SINGLE, 3358 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 6540 },
	{ 0x8783, 0x8786, PDF_CMAP_RANGE, 22436 },
	{ 0x8787, 0x8789, PDF_CMAP_TABLE, 8021 },
	{ 0x878b, 0x878e, PDF_CMAP_TABLE, 8024 },
	{ 0x8790, 0x8790, PDF_CMAP_SINGLE, 22440 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 18615 },
	{ 0x8795, 0x8795, PDF_CMAP_SINGLE, 22441 },
	{ 0x8797, 0x8798, PDF_CMAP_RANGE, 19764 },
	{ 0x8799, 0x8799, PDF_CMAP_SINGLE, 15099 },
	{ 0x879e, 0x87a0, PDF_CMAP_TABLE, 8028 },
	{ 0x87a2, 0x87a3, PDF_CMAP_TABLE, 8031 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 18612 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 6547 },
	{ 0x87ac, 0x87ad, PDF_CMAP_RANGE, 15100 },
	{ 0x87ae, 0x87af, PDF_CMAP_TABLE, 8033 },
	{ 0x87b1, 0x87b1, PDF_CMAP_SINGLE, 22443 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 6549 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 15102 },
	{ 0x87ba, 0x87bb, PDF_CMAP_TABLE, 8035 },
	{ 0x87bd, 0x87c1, PDF_CMAP_TABLE, 8037 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 6548 },
	{ 0x87c6, 0x87cb, PDF_CMAP_TABLE, 8042 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 18622 },
	{ 0x87d0, 0x87d0, PDF_CMAP_SINGLE, 6545 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 6562 },
	{ 0x87d5, 0x87d6, PDF_CMAP_TABLE, 8048 },
	{ 0x87d9, 0x87da, PDF_CMAP_TABLE, 8050 },
	{ 0x87dc, 0x87dc, PDF_CMAP_SINGLE, 22448 },
	{ 0x87df, 0x87e0, PDF_CMAP_TABLE, 8052 },
	{ 0x87e2, 0x87e4, PDF_CMAP_TABLE, 8054 },
	{ 0x87e5, 0x87e6, PDF_CMAP_RANGE, 18626 },
	{ 0x87ea, 0x87ed, PDF_CMAP_TABLE, 8057 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 6553 },
	{ 0x87f1, 0x87f3, PDF_CMAP_TABLE, 8061 },
	{ 0x87f5, 0x87f5, PDF_CMAP_SINGLE, 15421 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 6559 },
	{ 0x87f8, 0x87fb, PDF_CMAP_TABLE, 8064 },
	{ 0x87fe, 0x87ff, PDF_CMAP_TABLE, 8068 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 15106 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 20312 },
	{ 0x8805, 0x8807, PDF_CMAP_TABLE, 8070 },
	{ 0x8809, 0x880b, PDF_CMAP_TABLE, 8073 },
	{ 0x880d, 0x8816, PDF_CMAP_TABLE, 8076 },
	{ 0x8818, 0x881c, PDF_CMAP_TABLE, 8086 },
	{ 0x881e, 0x881f, PDF_CMAP_TABLE, 8091 },
	{ 0x8821, 0x8823, PDF_CMAP_TABLE, 8093 },
	{ 0x8827, 0x8828, PDF_CMAP_TABLE, 8096 },
	{ 0x882d, 0x882e, PDF_CMAP_TABLE, 8098 },
	{ 0x8830, 0x8832, PDF_CMAP_TABLE, 8100 },
	{ 0x8835, 0x8836, PDF_CMAP_TABLE, 8103 },
	{ 0x8839, 0x883c, PDF_CMAP_TABLE, 8105 },
	{ 0x8840, 0x8846, PDF_CMAP_TABLE, 8109 },
	{ 0x8848, 0x8849, PDF_CMAP_RANGE, 22465 },
	{ 0x884a, 0x884e, PDF_CMAP_TABLE, 8116 },
	{ 0x8851, 0x8853, PDF_CMAP_TABLE, 8121 },
	{ 0x8855, 0x8864, PDF_CMAP_TABLE, 8124 },
	{ 0x8868, 0x8869, PDF_CMAP_TABLE, 8140 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 6579 },
	{ 0x886e, 0x8872, PDF_CMAP_TABLE, 8142 },
	{ 0x8875, 0x8875, PDF_CMAP_SINGLE, 6583 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2989 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 22472 },
	{ 0x887b, 0x887b, PDF_CMAP_SINGLE, 22473 },
	{ 0x887d, 0x8882, PDF_CMAP_TABLE, 8147 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 1804 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 2878 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 6593 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 6589 },
	{ 0x8896, 0x8899, PDF_CMAP_TABLE, 8153 },
	{ 0x889a, 0x889c, PDF_CMAP_RANGE, 19782 },
	{ 0x889e, 0x88a0, PDF_CMAP_TABLE, 8157 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 6592 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 6594 },
	{ 0x88a8, 0x88a8, PDF_CMAP_SINGLE, 22476 },
	{ 0x88aa, 0x88ab, PDF_CMAP_TABLE, 8160 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 6590 },
	{ 0x88b0, 0x88b1, PDF_CMAP_TABLE, 8162 },
	{ 0x88b4, 0x88b5, PDF_CMAP_TABLE, 8164 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 1157 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 22477 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 18643 },
	{ 0x88bd, 0x88be, PDF_CMAP_RANGE, 15117 },
	{ 0x88bf, 0x88c2, PDF_CMAP_TABLE, 8166 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 6598 },
	{ 0x88c5, 0x88c6, PDF_CMAP_TABLE, 8170 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 15119 },
	{ 0x88cb, 0x88cc, PDF_CMAP_RANGE, 22478 },
	{ 0x88cd, 0x88cf, PDF_CMAP_TABLE, 8172 },
	{ 0x88d1, 0x88d5, PDF_CMAP_TABLE, 8175 },
	{ 0x88d8, 0x88d9, PDF_CMAP_RANGE, 6601 },
	{ 0x88db, 0x88e1, PDF_CMAP_TABLE, 8180 },
	{ 0x88e7, 0x88e8, PDF_CMAP_TABLE, 8187 },
	{ 0x88ef, 0x88ef, PDF_CMAP_SINGLE, 19787 },
	{ 0x88f0, 0x88f1, PDF_CMAP_RANGE, 15122 },
	{ 0x88f2, 0x88f5, PDF_CMAP_TABLE, 8189 },
	{ 0x88f7, 0x88f9, PDF_CMAP_TABLE, 8193 },
	{ 0x88fc, 0x88fe, PDF_CMAP_TABLE, 8196 },
	{ 0x8901, 0x8902, PDF_CMAP_TABLE, 8199 },
	{ 0x8904, 0x8904, PDF_CMAP_SINGLE, 6610 },
	{ 0x8906, 0x8907, PDF_CMAP_TABLE, 8201 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 6612 },
	{ 0x890c, 0x890d, PDF_CMAP_TABLE, 8203 },
	{ 0x890e, 0x890f, PDF_CMAP_RANGE, 19788 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 1482 },
	{ 0x8912, 0x8913, PDF_CMAP_TABLE, 8205 },
	{ 0x8915, 0x8916, PDF_CMAP_RANGE, 22484 },
	{ 0x8918, 0x891a, PDF_CMAP_RANGE, 15125 },
	{ 0x891c, 0x891e, PDF_CMAP_TABLE, 8207 },
	{ 0x8920, 0x8920, PDF_CMAP_SINGLE, 22486 },
	{ 0x8925, 0x8928, PDF_CMAP_TABLE, 8210 },
	{ 0x892a, 0x892b, PDF_CMAP_RANGE, 6617 },
	{ 0x8930, 0x8932, PDF_CMAP_TABLE, 8214 },
	{ 0x8935, 0x893b, PDF_CMAP_TABLE, 8217 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 15130 },
	{ 0x8940, 0x8946, PDF_CMAP_TABLE, 8224 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 18655 },
	{ 0x894c, 0x894d, PDF_CMAP_TABLE, 8231 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 22491 },
	{ 0x8952, 0x8952, PDF_CMAP_SINGLE, 22492 },
	{ 0x8956, 0x8957, PDF_CMAP_TABLE, 8233 },
	{ 0x895a, 0x895c, PDF_CMAP_TABLE, 8235 },
	{ 0x895e, 0x8964, PDF_CMAP_TABLE, 8238 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 6628 },
	{ 0x896a, 0x896b, PDF_CMAP_TABLE, 8245 },
	{ 0x896d, 0x8970, PDF_CMAP_TABLE, 8247 },
	{ 0x8972, 0x8975, PDF_CMAP_TABLE, 8251 },
	{ 0x8977, 0x8977, PDF_CMAP_SINGLE, 6634 },
	{ 0x897a, 0x8981, PDF_CMAP_TABLE, 8255 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 6636 },
	{ 0x8986, 0x898b, PDF_CMAP_TABLE, 8263 },
	{ 0x898d, 0x898d, PDF_CMAP_SINGLE, 22502 },
	{ 0x898f, 0x8990, PDF_CMAP_TABLE, 8269 },
	{ 0x8993, 0x8998, PDF_CMAP_TABLE, 8271 },
	{ 0x899a, 0x899a, PDF_CMAP_SINGLE, 1454 },
	{ 0x899b, 0x899c, PDF_CMAP_RANGE, 22504 },
	{ 0x899f, 0x89a1, PDF_CMAP_TABLE, 8277 },
	{ 0x89a5, 0x89a7, PDF_CMAP_TABLE, 8280 },
	{ 0x89a9, 0x89aa, PDF_CMAP_TABLE, 8283 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 6644 },
	{ 0x89af, 0x89b0, PDF_CMAP_TABLE, 8285 },
	{ 0x89b2, 0x89b7, PDF_CMAP_TABLE, 8287 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 6647 },
	{ 0x89bc, 0x89bd, PDF_CMAP_TABLE, 8293 },
	{ 0x89bf, 0x89c0, PDF_CMAP_RANGE, 6649 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 1455 },
	{ 0x89d4, 0x89d8, PDF_CMAP_TABLE, 8295 },
	{ 0x89da, 0x89da, PDF_CMAP_SINGLE, 6651 },
	{ 0x89dc, 0x89dd, PDF_CMAP_RANGE, 6652 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 1394 },
	{ 0x89e5, 0x89e7, PDF_CMAP_TABLE, 8300 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 22510 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 18664 },
	{ 0x89ed, 0x89ed, PDF_CMAP_SINGLE, 22511 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 15135 },
	{ 0x89f3, 0x89f4, PDF_CMAP_TABLE, 8303 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 17122 },
	{ 0x89f8, 0x89f9, PDF_CMAP_TABLE, 8305 },
	{ 0x89fd, 0x89fd, PDF_CMAP_SINGLE, 18667 },
	{ 0x89ff, 0x8a03, PDF_CMAP_TABLE, 8307 },
	{ 0x8a04, 0x8a05, PDF_CMAP_RANGE, 22513 },
	{ 0x8a07, 0x8a08, PDF_CMAP_TABLE, 8312 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2588 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 6660 },
	{ 0x8a0e, 0x8a18, PDF_CMAP_TABLE, 8314 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 6661 },
	{ 0x8a1d, 0x8a26, PDF_CMAP_TABLE, 8325 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 3674 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 22519 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 2691 },
	{ 0x8a2f, 0x8a2f, PDF_CMAP_SINGLE, 22521 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 1680 },
	{ 0x8a33, 0x8a37, PDF_CMAP_TABLE, 8335 },
	{ 0x8a3a, 0x8a3e, PDF_CMAP_TABLE, 8340 },
	{ 0x8a40, 0x8a41, PDF_CMAP_TABLE, 8345 },
	{ 0x8a43, 0x8a43, PDF_CMAP_SINGLE, 22524 },
	{ 0x8a45, 0x8a49, PDF_CMAP_TABLE, 8347 },
	{ 0x8a4d, 0x8a4e, PDF_CMAP_TABLE, 8352 },
	{ 0x8a50, 0x8a58, PDF_CMAP_TABLE, 8354 },
	{ 0x8a5b, 0x8a5e, PDF_CMAP_TABLE, 8363 },
	{ 0x8a60, 0x8a63, PDF_CMAP_TABLE, 8367 },
	{ 0x8a65, 0x8a67, PDF_CMAP_TABLE, 8371 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 2235 },
	{ 0x8a6b, 0x8a6e, PDF_CMAP_TABLE, 8374 },
	{ 0x8a70, 0x8a73, PDF_CMAP_TABLE, 8378 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 17126 },
	{ 0x8a76, 0x8a77, PDF_CMAP_RANGE, 22529 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8618 },
	{ 0x8a7a, 0x8a7b, PDF_CMAP_RANGE, 22531 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 6670 },
	{ 0x8a7e, 0x8a80, PDF_CMAP_TABLE, 8382 },
	{ 0x8a82, 0x8a87, PDF_CMAP_TABLE, 8385 },
	{ 0x8a89, 0x8a89, PDF_CMAP_SINGLE, 3882 },
	{ 0x8a8b, 0x8a8d, PDF_CMAP_TABLE, 8391 },
	{ 0x8a8f, 0x8a93, PDF_CMAP_TABLE, 8394 },
	{ 0x8a95, 0x8a9a, PDF_CMAP_TABLE, 8399 },
	{ 0x8a9e, 0x8aa1, PDF_CMAP_TABLE, 8405 },
	{ 0x8aa3, 0x8aa4, PDF_CMAP_TABLE, 8409 },
	{ 0x8aa5, 0x8aa6, PDF_CMAP_RANGE, 6680 },
	{ 0x8aa7, 0x8aaa, PDF_CMAP_TABLE, 8411 },
	{ 0x8aac, 0x8ab0, PDF_CMAP_TABLE, 8415 },
	{ 0x8ab2, 0x8ab3, PDF_CMAP_TABLE, 8420 },
	{ 0x8ab6, 0x8ab7, PDF_CMAP_TABLE, 8422 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 3460 },
	{ 0x8abb, 0x8abc, PDF_CMAP_TABLE, 8424 },
	{ 0x8abe, 0x8abf, PDF_CMAP_TABLE, 8426 },
	{ 0x8ac2, 0x8ac4, PDF_CMAP_TABLE, 8428 },
	{ 0x8ac6, 0x8acd, PDF_CMAP_TABLE, 8431 },
	{ 0x8acf, 0x8ad2, PDF_CMAP_TABLE, 8439 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 22549 },
	{ 0x8ad6, 0x8ad7, PDF_CMAP_TABLE, 8443 },
	{ 0x8ada, 0x8ae2, PDF_CMAP_TABLE, 8445 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 6691 },
	{ 0x8ae6, 0x8ae7, PDF_CMAP_TABLE, 8454 },
	{ 0x8aeb, 0x8aee, PDF_CMAP_TABLE, 8456 },
	{ 0x8af0, 0x8af1, PDF_CMAP_TABLE, 8460 },
	{ 0x8af3, 0x8af8, PDF_CMAP_TABLE, 8462 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 1909 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 18683 },
	{ 0x8afe, 0x8b02, PDF_CMAP_TABLE, 8468 },
	{ 0x8b04, 0x8b07, PDF_CMAP_TABLE, 8473 },
	{ 0x8b0a, 0x8b11, PDF_CMAP_TABLE, 8477 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 6693 },
	{ 0x8b16, 0x8b17, PDF_CMAP_TABLE, 8485 },
	{ 0x8b19, 0x8b21, PDF_CMAP_TABLE, 8487 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 6709 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 6712 },
	{ 0x8b2b, 0x8b2d, PDF_CMAP_TABLE, 8496 },
	{ 0x8b30, 0x8b30, PDF_CMAP_SINGLE, 22556 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 6707 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 15144 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 1752 },
	{ 0x8b3c, 0x8b3c, PDF_CMAP_SINGLE, 22557 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 6711 },
	{ 0x8b41, 0x8b42, PDF_CMAP_TABLE, 8499 },
	{ 0x8b43, 0x8b44, PDF_CMAP_RANGE, 15145 },
	{ 0x8b45, 0x8b46, PDF_CMAP_TABLE, 8501 },
	{ 0x8b48, 0x8b49, PDF_CMAP_TABLE, 8503 },
	{ 0x8b4c, 0x8b4f, PDF_CMAP_TABLE, 8505 },
	{ 0x8b51, 0x8b54, PDF_CMAP_TABLE, 8509 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 6718 },
	{ 0x8b58, 0x8b5c, PDF_CMAP_TABLE, 8513 },
	{ 0x8b5e, 0x8b5f, PDF_CMAP_TABLE, 8518 },
	{ 0x8b63, 0x8b63, PDF_CMAP_SINGLE, 22561 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 1839 },
	{ 0x8b69, 0x8b69, PDF_CMAP_SINGLE, 17130 },
	{ 0x8b6b, 0x8b6d, PDF_CMAP_TABLE, 8520 },
	{ 0x8b6f, 0x8b72, PDF_CMAP_TABLE, 8523 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 6725 },
	{ 0x8b76, 0x8b79, PDF_CMAP_TABLE, 8527 },
	{ 0x8b7c, 0x8b81, PDF_CMAP_TABLE, 8531 },
	{ 0x8b83, 0x8b85, PDF_CMAP_TABLE, 8537 },
	{ 0x8b8a, 0x8b90, PDF_CMAP_TABLE, 8540 },
	{ 0x8b92, 0x8b93, PDF_CMAP_RANGE, 6730 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 18696 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 6732 },
	{ 0x8b99, 0x8b9a, PDF_CMAP_RANGE, 6733 },
	{ 0x8b9c, 0x8b9f, PDF_CMAP_TABLE, 8547 },
	{ 0x8c37, 0x8c3a, PDF_CMAP_TABLE, 8551 },
	{ 0x8c3d, 0x8c3f, PDF_CMAP_TABLE, 8555 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 6736 },
	{ 0x8c45, 0x8c4c, PDF_CMAP_TABLE, 8558 },
	{ 0x8c4e, 0x8c51, PDF_CMAP_TABLE, 8566 },
	{ 0x8c53, 0x8c55, PDF_CMAP_TABLE, 8570 },
	{ 0x8c57, 0x8c5b, PDF_CMAP_TABLE, 8573 },
	{ 0x8c5d, 0x8c5d, PDF_CMAP_SINGLE, 22571 },
	{ 0x8c61, 0x8c62, PDF_CMAP_TABLE, 8578 },
	{ 0x8c63, 0x8c64, PDF_CMAP_RANGE, 22573 },
	{ 0x8c66, 0x8c66, PDF_CMAP_SINGLE, 22575 },
	{ 0x8c68, 0x8c6d, PDF_CMAP_TABLE, 8580 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 15152 },
	{ 0x8c75, 0x8c76, PDF_CMAP_RANGE, 22576 },
	{ 0x8c78, 0x8c7c, PDF_CMAP_TABLE, 8586 },
	{ 0x8c7e, 0x8c7e, PDF_CMAP_SINGLE, 22578 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 6747 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 6749 },
	{ 0x8c86, 0x8c87, PDF_CMAP_RANGE, 22579 },
	{ 0x8c89, 0x8c8c, PDF_CMAP_TABLE, 8591 },
	{ 0x8c8d, 0x8c8e, PDF_CMAP_RANGE, 6751 },
	{ 0x8c90, 0x8c90, PDF_CMAP_SINGLE, 22582 },
	{ 0x8c92, 0x8c94, PDF_CMAP_TABLE, 8595 },
	{ 0x8c98, 0x8c99, PDF_CMAP_TABLE, 8598 },
	{ 0x8c9b, 0x8ca2, PDF_CMAP_TABLE, 8600 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 15153 },
	{ 0x8ca7, 0x8cb0, PDF_CMAP_TABLE, 8608 },
	{ 0x8cb2, 0x8cb3, PDF_CMAP_RANGE, 6760 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 1608 },
	{ 0x8cb6, 0x8cbd, PDF_CMAP_TABLE, 8618 },
	{ 0x8cbf, 0x8ccb, PDF_CMAP_TABLE, 8626 },
	{ 0x8ccd, 0x8ccf, PDF_CMAP_TABLE, 8639 },
	{ 0x8cd1, 0x8cd3, PDF_CMAP_TABLE, 8642 },
	{ 0x8cd5, 0x8cd6, PDF_CMAP_TABLE, 8645 },
	{ 0x8cd9, 0x8cde, PDF_CMAP_TABLE, 8647 },
	{ 0x8ce0, 0x8ce4, PDF_CMAP_TABLE, 8653 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 3548 },
	{ 0x8ce8, 0x8ce8, PDF_CMAP_SINGLE, 22590 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 2285 },
	{ 0x8cec, 0x8ced, PDF_CMAP_TABLE, 8658 },
	{ 0x8cef, 0x8cf2, PDF_CMAP_TABLE, 8660 },
	{ 0x8cf4, 0x8cf5, PDF_CMAP_TABLE, 8664 },
	{ 0x8cf7, 0x8cf8, PDF_CMAP_TABLE, 8666 },
	{ 0x8cfa, 0x8cfb, PDF_CMAP_RANGE, 6770 },
	{ 0x8cfc, 0x8cff, PDF_CMAP_TABLE, 8668 },
	{ 0x8d01, 0x8d01, PDF_CMAP_SINGLE, 19834 },
	{ 0x8d03, 0x8d03, PDF_CMAP_SINGLE, 19835 },
	{ 0x8d04, 0x8d05, PDF_CMAP_RANGE, 6772 },
	{ 0x8d07, 0x8d0b, PDF_CMAP_TABLE, 8672 },
	{ 0x8d0d, 0x8d10, PDF_CMAP_TABLE, 8677 },
	{ 0x8d12, 0x8d14, PDF_CMAP_TABLE, 8681 },
	{ 0x8d16, 0x8d17, PDF_CMAP_TABLE, 8684 },
	{ 0x8d1b, 0x8d1c, PDF_CMAP_TABLE, 8686 },
	{ 0x8d64, 0x8d67, PDF_CMAP_TABLE, 8688 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 15159 },
	{ 0x8d6b, 0x8d6e, PDF_CMAP_TABLE, 8692 },
	{ 0x8d70, 0x8d71, PDF_CMAP_TABLE, 8696 },
	{ 0x8d73, 0x8d74, PDF_CMAP_TABLE, 8698 },
	{ 0x8d76, 0x8d77, PDF_CMAP_TABLE, 8700 },
	{ 0x8d7f, 0x8d7f, PDF_CMAP_SINGLE, 22595 },
	{ 0x8d81, 0x8d82, PDF_CMAP_TABLE, 8702 },
	{ 0x8d84, 0x8d85, PDF_CMAP_TABLE, 8704 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 22597 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 1277 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 15162 },
	{ 0x8d90, 0x8d91, PDF_CMAP_TABLE, 8706 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 15163 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 6789 },
	{ 0x8d9e, 0x8da0, PDF_CMAP_TABLE, 8708 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 2333 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 15164 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 2620 },
	{ 0x8dab, 0x8dac, PDF_CMAP_TABLE, 8711 },
	{ 0x8daf, 0x8daf, PDF_CMAP_SINGLE, 17134 },
	{ 0x8db2, 0x8db3, PDF_CMAP_TABLE, 8713 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 22602 },
	{ 0x8db7, 0x8db7, PDF_CMAP_SINGLE, 22603 },
	{ 0x8db9, 0x8dbc, PDF_CMAP_TABLE, 8715 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 6791 },
	{ 0x8dc0, 0x8dc0, PDF_CMAP_SINGLE, 22606 },
	{ 0x8dc2, 0x8dc2, PDF_CMAP_SINGLE, 6790 },
	{ 0x8dc5, 0x8dc8, PDF_CMAP_TABLE, 8719 },
	{ 0x8dca, 0x8dcc, PDF_CMAP_TABLE, 8723 },
	{ 0x8dce, 0x8dcf, PDF_CMAP_TABLE, 8726 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 17135 },
	{ 0x8dd4, 0x8dd7, PDF_CMAP_TABLE, 8728 },
	{ 0x8dd9, 0x8ddb, PDF_CMAP_TABLE, 8732 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 1681 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 6801 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 2683 },
	{ 0x8de3, 0x8de5, PDF_CMAP_TABLE, 8735 },
	{ 0x8de7, 0x8de8, PDF_CMAP_TABLE, 8738 },
	{ 0x8dea, 0x8deb, PDF_CMAP_RANGE, 6799 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 15168 },
	{ 0x8def, 0x8df0, PDF_CMAP_TABLE, 8740 },
	{ 0x8df1, 0x8df2, PDF_CMAP_RANGE, 19845 },
	{ 0x8df3, 0x8df5, PDF_CMAP_TABLE, 8742 },
	{ 0x8dfc, 0x8dfd, PDF_CMAP_TABLE, 8745 },
	{ 0x8dff, 0x8dff, PDF_CMAP_SINGLE, 6806 },
	{ 0x8e01, 0x8e01, PDF_CMAP_SINGLE, 19848 },
	{ 0x8e04, 0x8e05, PDF_CMAP_RANGE, 22614 },
	{ 0x8e06, 0x8e06, PDF_CMAP_SINGLE, 18726 },
	{ 0x8e08, 0x8e09, PDF_CMAP_RANGE, 6804 },
	{ 0x8e0a, 0x8e0c, PDF_CMAP_TABLE, 8747 },
	{ 0x8e0f, 0x8e11, PDF_CMAP_TABLE, 8750 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 18728 },
	{ 0x8e16, 0x8e16, PDF_CMAP_SINGLE, 18729 },
	{ 0x8e1d, 0x8e1e, PDF_CMAP_RANGE, 6807 },
	{ 0x8e1f, 0x8e20, PDF_CMAP_TABLE, 8753 },
	{ 0x8e21, 0x8e22, PDF_CMAP_RANGE, 18730 },
	{ 0x8e23, 0x8e23, PDF_CMAP_SINGLE, 17137 },
	{ 0x8e26, 0x8e27, PDF_CMAP_TABLE, 8755 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 6824 },
	{ 0x8e30, 0x8e31, PDF_CMAP_TABLE, 8757 },
	{ 0x8e33, 0x8e36, PDF_CMAP_TABLE, 8759 },
	{ 0x8e37, 0x8e38, PDF_CMAP_RANGE, 22618 },
	{ 0x8e39, 0x8e39, PDF_CMAP_SINGLE, 18736 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 17138 },
	{ 0x8e40, 0x8e41, PDF_CMAP_RANGE, 19852 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 6811 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 3097 },
	{ 0x8e47, 0x8e50, PDF_CMAP_TABLE, 8763 },
	{ 0x8e54, 0x8e55, PDF_CMAP_TABLE, 8773 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 6821 },
	{ 0x8e5b, 0x8e5c, PDF_CMAP_TABLE, 8775 },
	{ 0x8e5d, 0x8e5e, PDF_CMAP_RANGE, 22622 },
	{ 0x8e5f, 0x8e64, PDF_CMAP_TABLE, 8777 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 19858 },
	{ 0x8e6c, 0x8e6d, PDF_CMAP_TABLE, 8783 },
	{ 0x8e6f, 0x8e72, PDF_CMAP_TABLE, 8785 },
	{ 0x8e74, 0x8e77, PDF_CMAP_TABLE, 8789 },
	{ 0x8e79, 0x8e7c, PDF_CMAP_TABLE, 8793 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 6830 },
	{ 0x8e82, 0x8e83, PDF_CMAP_RANGE, 22625 },
	{ 0x8e84, 0x8e85, PDF_CMAP_TABLE, 8797 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 6831 },
	{ 0x8e89, 0x8e8b, PDF_CMAP_TABLE, 8799 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 3842 },
	{ 0x8e90, 0x8e95, PDF_CMAP_TABLE, 8802 },
	{ 0x8e98, 0x8e9b, PDF_CMAP_TABLE, 8808 },
	{ 0x8e9d, 0x8e9e, PDF_CMAP_TABLE, 8812 },
	{ 0x8ea1, 0x8ea2, PDF_CMAP_TABLE, 8814 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 19866 },
	{ 0x8ea9, 0x8eb1, PDF_CMAP_TABLE, 8816 },
	{ 0x8eb3, 0x8eb3, PDF_CMAP_SINGLE, 15176 },
	{ 0x8eb5, 0x8eb6, PDF_CMAP_TABLE, 8825 },
	{ 0x8eba, 0x8ebb, PDF_CMAP_TABLE, 8827 },
	{ 0x8ebe, 0x8ebe, PDF_CMAP_SINGLE, 6846 },
	{ 0x8ec0, 0x8ec1, PDF_CMAP_TABLE, 8829 },
	{ 0x8ec3, 0x8ec4, PDF_CMAP_RANGE, 22632 },
	{ 0x8ec5, 0x8ec8, PDF_CMAP_TABLE, 8831 },
	{ 0x8eca, 0x8ecd, PDF_CMAP_TABLE, 8835 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 8631 },
	{ 0x8ed1, 0x8ed2, PDF_CMAP_TABLE, 8839 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 15178 },
	{ 0x8edb, 0x8edc, PDF_CMAP_TABLE, 8841 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 3272 },
	{ 0x8ee2, 0x8ee3, PDF_CMAP_TABLE, 8843 },
	{ 0x8ee8, 0x8ee8, PDF_CMAP_SINGLE, 19869 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 6854 },
	{ 0x8eed, 0x8eee, PDF_CMAP_TABLE, 8845 },
	{ 0x8ef0, 0x8ef1, PDF_CMAP_TABLE, 8847 },
	{ 0x8ef7, 0x8efe, PDF_CMAP_TABLE, 8849 },
	{ 0x8f00, 0x8f00, PDF_CMAP_SINGLE, 18748 },
	{ 0x8f02, 0x8f03, PDF_CMAP_TABLE, 8857 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 6857 },
	{ 0x8f07, 0x8f0a, PDF_CMAP_TABLE, 8859 },
	{ 0x8f0c, 0x8f0c, PDF_CMAP_SINGLE, 6865 },
	{ 0x8f0f, 0x8f10, PDF_CMAP_RANGE, 22641 },
	{ 0x8f12, 0x8f19, PDF_CMAP_TABLE, 8863 },
	{ 0x8f1b, 0x8f1f, PDF_CMAP_TABLE, 8871 },
	{ 0x8f20, 0x8f21, PDF_CMAP_RANGE, 22644 },
	{ 0x8f23, 0x8f23, PDF_CMAP_SINGLE, 22646 },
	{ 0x8f25, 0x8f2f, PDF_CMAP_TABLE, 8876 },
	{ 0x8f33, 0x8f3b, PDF_CMAP_TABLE, 8887 },
	{ 0x8f3e, 0x8f47, PDF_CMAP_TABLE, 8896 },
	{ 0x8f49, 0x8f4a, PDF_CMAP_TABLE, 8906 },
	{ 0x8f4c, 0x8f4f, PDF_CMAP_TABLE, 8908 },
	{ 0x8f51, 0x8f51, PDF_CMAP_SINGLE, 19880 },
	{ 0x8f52, 0x8f53, PDF_CMAP_RANGE, 22653 },
	{ 0x8f54, 0x8f55, PDF_CMAP_TABLE, 8912 },
	{ 0x8f57, 0x8f58, PDF_CMAP_TABLE, 8914 },
	{ 0x8f5c, 0x8f5c, PDF_CMAP_SINGLE, 6878 },
	{ 0x8f5d, 0x8f5e, PDF_CMAP_RANGE, 22655 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 2046 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 1787 },
	{ 0x8f62, 0x8f64, PDF_CMAP_RANGE, 6879 },
	{ 0x8f65, 0x8f65, PDF_CMAP_SINGLE, 22657 },
	{ 0x8f9b, 0x8f9f, PDF_CMAP_TABLE, 8916 },
	{ 0x8fa0, 0x8fa2, PDF_CMAP_RANGE, 19882 },
	{ 0x8fa3, 0x8fa8, PDF_CMAP_TABLE, 8921 },
	{ 0x8fad, 0x8fb2, PDF_CMAP_TABLE, 8927 },
	{ 0x8fb4, 0x8fb4, PDF_CMAP_SINGLE, 18756 },
	{ 0x8fb5, 0x8fb6, PDF_CMAP_RANGE, 15183 },
	{ 0x8fb7, 0x8fb8, PDF_CMAP_TABLE, 8933 },
	{ 0x8fba, 0x8fbc, PDF_CMAP_TABLE, 8935 },
	{ 0x8fbe, 0x8fc2, PDF_CMAP_TABLE, 8938 },
	{ 0x8fc4, 0x8fc6, PDF_CMAP_TABLE, 8943 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 14233 },
	{ 0x8fca, 0x8fcb, PDF_CMAP_TABLE, 8946 },
	{ 0x8fcd, 0x8fce, PDF_CMAP_TABLE, 8948 },
	{ 0x8fd0, 0x8fd5, PDF_CMAP_TABLE, 8950 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 6888 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 15186 },
	{ 0x8fe2, 0x8fe6, PDF_CMAP_TABLE, 8956 },
	{ 0x8fe8, 0x8feb, PDF_CMAP_TABLE, 8961 },
	{ 0x8fed, 0x8ff1, PDF_CMAP_TABLE, 8965 },
	{ 0x8ff4, 0x8ff8, PDF_CMAP_TABLE, 8970 },
	{ 0x8ff9, 0x8ffa, PDF_CMAP_RANGE, 6896 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 18767 },
	{ 0x8ffd, 0x8ffe, PDF_CMAP_TABLE, 8975 },
	{ 0x9000, 0x9006, PDF_CMAP_TABLE, 8977 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 17149 },
	{ 0x900b, 0x9011, PDF_CMAP_TABLE, 8984 },
	{ 0x9013, 0x901b, PDF_CMAP_TABLE, 8991 },
	{ 0x901d, 0x9023, PDF_CMAP_TABLE, 9000 },
	{ 0x9027, 0x9027, PDF_CMAP_SINGLE, 6905 },
	{ 0x9028, 0x902a, PDF_CMAP_RANGE, 22668 },
	{ 0x902c, 0x902f, PDF_CMAP_TABLE, 9007 },
	{ 0x9031, 0x9032, PDF_CMAP_TABLE, 9011 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 22671 },
	{ 0x9035, 0x9039, PDF_CMAP_TABLE, 9013 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 3489 },
	{ 0x903e, 0x903f, PDF_CMAP_TABLE, 9018 },
	{ 0x9041, 0x9045, PDF_CMAP_TABLE, 9020 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 1776 },
	{ 0x9049, 0x904e, PDF_CMAP_TABLE, 9025 },
	{ 0x904f, 0x9052, PDF_CMAP_RANGE, 6910 },
	{ 0x9053, 0x9056, PDF_CMAP_TABLE, 9031 },
	{ 0x9058, 0x9059, PDF_CMAP_TABLE, 9035 },
	{ 0x905b, 0x905e, PDF_CMAP_TABLE, 9037 },
	{ 0x9060, 0x9063, PDF_CMAP_TABLE, 9041 },
	{ 0x9065, 0x9069, PDF_CMAP_TABLE, 9045 },
	{ 0x906c, 0x9070, PDF_CMAP_TABLE, 9050 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 6924 },
	{ 0x9074, 0x907a, PDF_CMAP_TABLE, 9055 },
	{ 0x907c, 0x907d, PDF_CMAP_TABLE, 9062 },
	{ 0x907f, 0x9085, PDF_CMAP_TABLE, 9064 },
	{ 0x9087, 0x908c, PDF_CMAP_TABLE, 9071 },
	{ 0x908e, 0x9091, PDF_CMAP_TABLE, 9077 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 15193 },
	{ 0x9097, 0x9099, PDF_CMAP_TABLE, 9081 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 15195 },
	{ 0x90a0, 0x90a3, PDF_CMAP_TABLE, 9084 },
	{ 0x90a5, 0x90a6, PDF_CMAP_TABLE, 9088 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 6932 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 2309 },
	{ 0x90af, 0x90b6, PDF_CMAP_TABLE, 9090 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 3099 },
	{ 0x90bd, 0x90be, PDF_CMAP_TABLE, 9098 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 1198 },
	{ 0x90c3, 0x90c3, PDF_CMAP_SINGLE, 18782 },
	{ 0x90c4, 0x90c5, PDF_CMAP_RANGE, 17154 },
	{ 0x90c7, 0x90ca, PDF_CMAP_TABLE, 9100 },
	{ 0x90cc, 0x90cc, PDF_CMAP_SINGLE, 22679 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 4064 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 22684 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 22680 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 15198 },
	{ 0x90d8, 0x90d9, PDF_CMAP_RANGE, 22681 },
	{ 0x90db, 0x90df, PDF_CMAP_TABLE, 9104 },
	{ 0x90e1, 0x90e2, PDF_CMAP_TABLE, 9109 },
	{ 0x90e4, 0x90e5, PDF_CMAP_TABLE, 9111 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 3558 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 18791 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 1458 },
	{ 0x90ef, 0x90f0, PDF_CMAP_TABLE, 9113 },
	{ 0x90f2, 0x90f2, PDF_CMAP_SINGLE, 18789 },
	{ 0x90f4, 0x90f7, PDF_CMAP_TABLE, 9115 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 3150 },
	{ 0x90fe, 0x90ff, PDF_CMAP_RANGE, 18792 },
	{ 0x9100, 0x9100, PDF_CMAP_SINGLE, 18790 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 6940 },
	{ 0x9104, 0x9106, PDF_CMAP_TABLE, 9119 },
	{ 0x9108, 0x9108, PDF_CMAP_SINGLE, 22685 },
	{ 0x910d, 0x910d, PDF_CMAP_SINGLE, 22686 },
	{ 0x9110, 0x9110, PDF_CMAP_SINGLE, 22687 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 6941 },
	{ 0x9114, 0x911a, PDF_CMAP_TABLE, 9122 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 15202 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 18797 },
	{ 0x9120, 0x9120, PDF_CMAP_SINGLE, 22689 },
	{ 0x9122, 0x9123, PDF_CMAP_RANGE, 17160 },
	{ 0x9125, 0x9125, PDF_CMAP_SINGLE, 19902 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 8638 },
	{ 0x9129, 0x9129, PDF_CMAP_SINGLE, 22690 },
	{ 0x912d, 0x9132, PDF_CMAP_TABLE, 9129 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 17163 },
	{ 0x9136, 0x9137, PDF_CMAP_TABLE, 9135 },
	{ 0x9139, 0x913a, PDF_CMAP_TABLE, 9137 },
	{ 0x913c, 0x913d, PDF_CMAP_TABLE, 9139 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 22694 },
	{ 0x9146, 0x9147, PDF_CMAP_RANGE, 18800 },
	{ 0x9148, 0x914f, PDF_CMAP_TABLE, 9141 },
	{ 0x9152, 0x9154, PDF_CMAP_TABLE, 9149 },
	{ 0x9156, 0x915b, PDF_CMAP_TABLE, 9152 },
	{ 0x9161, 0x9165, PDF_CMAP_TABLE, 9158 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 19904 },
	{ 0x9169, 0x916a, PDF_CMAP_TABLE, 9163 },
	{ 0x916c, 0x916d, PDF_CMAP_TABLE, 9165 },
	{ 0x9172, 0x9175, PDF_CMAP_TABLE, 9167 },
	{ 0x9177, 0x917b, PDF_CMAP_TABLE, 9171 },
	{ 0x9181, 0x9183, PDF_CMAP_TABLE, 9176 },
	{ 0x9185, 0x9187, PDF_CMAP_TABLE, 9179 },
	{ 0x9189, 0x918b, PDF_CMAP_TABLE, 9182 },
	{ 0x918d, 0x918e, PDF_CMAP_TABLE, 9185 },
	{ 0x9190, 0x9192, PDF_CMAP_TABLE, 9187 },
	{ 0x9193, 0x9194, PDF_CMAP_RANGE, 22701 },
	{ 0x9195, 0x9195, PDF_CMAP_SINGLE, 19908 },
	{ 0x9197, 0x9198, PDF_CMAP_TABLE, 9190 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 2371 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 15211 },
	{ 0x91a1, 0x91a2, PDF_CMAP_TABLE, 9192 },
	{ 0x91a4, 0x91a4, PDF_CMAP_SINGLE, 2503 },
	{ 0x91a6, 0x91a6, PDF_CMAP_SINGLE, 22705 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 15212 },
	{ 0x91aa, 0x91ac, PDF_CMAP_TABLE, 9194 },
	{ 0x91ad, 0x91ae, PDF_CMAP_RANGE, 15213 },
	{ 0x91af, 0x91b6, PDF_CMAP_TABLE, 9197 },
	{ 0x91b8, 0x91b8, PDF_CMAP_SINGLE, 2530 },
	{ 0x91ba, 0x91bd, PDF_CMAP_TABLE, 9205 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 22706 },
	{ 0x91c0, 0x91c1, PDF_CMAP_RANGE, 6963 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 19912 },
	{ 0x91c3, 0x91c4, PDF_CMAP_RANGE, 18809 },
	{ 0x91c5, 0x91c9, PDF_CMAP_TABLE, 9209 },
	{ 0x91cb, 0x91d1, PDF_CMAP_TABLE, 9214 },
	{ 0x91d3, 0x91d4, PDF_CMAP_RANGE, 22707 },
	{ 0x91d6, 0x91df, PDF_CMAP_TABLE, 9221 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 6970 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 3068 },
	{ 0x91e4, 0x91e5, PDF_CMAP_RANGE, 8644 },
	{ 0x91e6, 0x91e7, PDF_CMAP_TABLE, 9231 },
	{ 0x91e9, 0x91ea, PDF_CMAP_RANGE, 22710 },
	{ 0x91ec, 0x91ec, PDF_CMAP_SINGLE, 18813 },
	{ 0x91ed, 0x91ee, PDF_CMAP_RANGE, 8642 },
	{ 0x91ef, 0x91f1, PDF_CMAP_TABLE, 9233 },
	{ 0x91f5, 0x91f6, PDF_CMAP_RANGE, 6973 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 15218 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 22713 },
	{ 0x91fb, 0x91fd, PDF_CMAP_TABLE, 9236 },
	{ 0x91ff, 0x9201, PDF_CMAP_TABLE, 9239 },
	{ 0x9204, 0x9205, PDF_CMAP_RANGE, 22715 },
	{ 0x9206, 0x9207, PDF_CMAP_TABLE, 9242 },
	{ 0x9209, 0x920a, PDF_CMAP_TABLE, 9244 },
	{ 0x920c, 0x920e, PDF_CMAP_TABLE, 9246 },
	{ 0x9210, 0x9211, PDF_CMAP_TABLE, 9249 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 22718 },
	{ 0x9214, 0x9215, PDF_CMAP_TABLE, 9251 },
	{ 0x9216, 0x9217, PDF_CMAP_RANGE, 18815 },
	{ 0x9218, 0x9218, PDF_CMAP_SINGLE, 22720 },
	{ 0x921c, 0x921d, PDF_CMAP_RANGE, 22721 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 6975 },
	{ 0x9223, 0x9223, PDF_CMAP_SINGLE, 19916 },
	{ 0x9224, 0x9226, PDF_CMAP_RANGE, 22723 },
	{ 0x9228, 0x9229, PDF_CMAP_TABLE, 9253 },
	{ 0x922c, 0x922c, PDF_CMAP_SINGLE, 6978 },
	{ 0x922e, 0x9230, PDF_CMAP_RANGE, 22726 },
	{ 0x9233, 0x9234, PDF_CMAP_TABLE, 9255 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 22729 },
	{ 0x9237, 0x923a, PDF_CMAP_TABLE, 9257 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 8651 },
	{ 0x923e, 0x9240, PDF_CMAP_TABLE, 9261 },
	{ 0x9242, 0x924b, PDF_CMAP_TABLE, 9264 },
	{ 0x924d, 0x9251, PDF_CMAP_TABLE, 9274 },
	{ 0x9256, 0x925b, PDF_CMAP_TABLE, 9279 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 22735 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 6981 },
	{ 0x9260, 0x9262, PDF_CMAP_TABLE, 9285 },
	{ 0x9264, 0x9269, PDF_CMAP_TABLE, 9288 },
	{ 0x926e, 0x9270, PDF_CMAP_RANGE, 22738 },
	{ 0x9271, 0x9271, PDF_CMAP_SINGLE, 2029 },
	{ 0x9275, 0x9276, PDF_CMAP_TABLE, 9294 },
	{ 0x9277, 0x9278, PDF_CMAP_RANGE, 8658 },
	{ 0x9279, 0x9279, PDF_CMAP_SINGLE, 22742 },
	{ 0x927b, 0x927b, PDF_CMAP_SINGLE, 22743 },
	{ 0x927c, 0x927d, PDF_CMAP_RANGE, 18825 },
	{ 0x927e, 0x9280, PDF_CMAP_TABLE, 9296 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 2384 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 3220 },
	{ 0x9288, 0x928a, PDF_CMAP_TABLE, 9299 },
	{ 0x928d, 0x928e, PDF_CMAP_TABLE, 9302 },
	{ 0x9291, 0x9293, PDF_CMAP_TABLE, 9304 },
	{ 0x9295, 0x929c, PDF_CMAP_TABLE, 9307 },
	{ 0x929f, 0x92a0, PDF_CMAP_TABLE, 9315 },
	{ 0x92a4, 0x92a5, PDF_CMAP_RANGE, 22747 },
	{ 0x92a7, 0x92a8, PDF_CMAP_TABLE, 9317 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 18833 },
	{ 0x92ad, 0x92ad, PDF_CMAP_SINGLE, 2734 },
	{ 0x92af, 0x92af, PDF_CMAP_SINGLE, 19919 },
	{ 0x92b2, 0x92b3, PDF_CMAP_TABLE, 9319 },
	{ 0x92b6, 0x92ba, PDF_CMAP_TABLE, 9321 },
	{ 0x92bb, 0x92bc, PDF_CMAP_RANGE, 19920 },
	{ 0x92bd, 0x92bd, PDF_CMAP_SINGLE, 22753 },
	{ 0x92bf, 0x92c0, PDF_CMAP_RANGE, 18837 },
	{ 0x92c1, 0x92c3, PDF_CMAP_TABLE, 9326 },
	{ 0x92c5, 0x92c8, PDF_CMAP_TABLE, 9329 },
	{ 0x92cb, 0x92cc, PDF_CMAP_RANGE, 15229 },
	{ 0x92cd, 0x92d0, PDF_CMAP_TABLE, 9333 },
	{ 0x92d2, 0x92d3, PDF_CMAP_TABLE, 9337 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 8665 },
	{ 0x92d7, 0x92d9, PDF_CMAP_TABLE, 9339 },
	{ 0x92dc, 0x92dd, PDF_CMAP_RANGE, 22757 },
	{ 0x92df, 0x92e1, PDF_CMAP_TABLE, 9342 },
	{ 0x92e3, 0x92e5, PDF_CMAP_TABLE, 9345 },
	{ 0x92e7, 0x92ea, PDF_CMAP_TABLE, 9348 },
	{ 0x92ec, 0x92ee, PDF_CMAP_TABLE, 9352 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 22764 },
	{ 0x92f2, 0x92f3, PDF_CMAP_TABLE, 9355 },
	{ 0x92f7, 0x92fc, PDF_CMAP_TABLE, 9357 },
	{ 0x92ff, 0x9300, PDF_CMAP_TABLE, 9363 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 8675 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 13402 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 2170 },
	{ 0x9308, 0x9308, PDF_CMAP_SINGLE, 22766 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 15232 },
	{ 0x930f, 0x9311, PDF_CMAP_TABLE, 9365 },
	{ 0x9314, 0x9315, PDF_CMAP_TABLE, 9368 },
	{ 0x9318, 0x931a, PDF_CMAP_TABLE, 9370 },
	{ 0x931c, 0x932c, PDF_CMAP_TABLE, 9373 },
	{ 0x932e, 0x932f, PDF_CMAP_TABLE, 9390 },
	{ 0x9332, 0x9337, PDF_CMAP_TABLE, 9392 },
	{ 0x933a, 0x933b, PDF_CMAP_TABLE, 9398 },
	{ 0x9344, 0x9344, PDF_CMAP_SINGLE, 7002 },
	{ 0x9347, 0x934b, PDF_CMAP_TABLE, 9400 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 3151 },
	{ 0x9350, 0x9352, PDF_CMAP_TABLE, 9405 },
	{ 0x9354, 0x9358, PDF_CMAP_TABLE, 9408 },
	{ 0x935a, 0x935c, PDF_CMAP_TABLE, 9413 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 22774 },
	{ 0x9360, 0x9360, PDF_CMAP_SINGLE, 7012 },
	{ 0x9364, 0x9365, PDF_CMAP_RANGE, 17167 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 22775 },
	{ 0x9369, 0x9371, PDF_CMAP_TABLE, 9416 },
	{ 0x9373, 0x9376, PDF_CMAP_TABLE, 9425 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 22780 },
	{ 0x937c, 0x937f, PDF_CMAP_TABLE, 9429 },
	{ 0x9380, 0x9381, PDF_CMAP_RANGE, 22782 },
	{ 0x9382, 0x9382, PDF_CMAP_SINGLE, 19931 },
	{ 0x9388, 0x9388, PDF_CMAP_SINGLE, 18857 },
	{ 0x938a, 0x938d, PDF_CMAP_TABLE, 9433 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 18859 },
	{ 0x9392, 0x9392, PDF_CMAP_SINGLE, 22785 },
	{ 0x9394, 0x9398, PDF_CMAP_TABLE, 9437 },
	{ 0x939a, 0x939b, PDF_CMAP_TABLE, 9442 },
	{ 0x939e, 0x939e, PDF_CMAP_SINGLE, 18860 },
	{ 0x93a1, 0x93a1, PDF_CMAP_SINGLE, 18854 },
	{ 0x93a3, 0x93a4, PDF_CMAP_TABLE, 9444 },
	{ 0x93a6, 0x93a9, PDF_CMAP_TABLE, 9446 },
	{ 0x93ab, 0x93ab, PDF_CMAP_SINGLE, 22791 },
	{ 0x93ac, 0x93ad, PDF_CMAP_RANGE, 7017 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 3039 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 7016 },
	{ 0x93b4, 0x93b6, PDF_CMAP_RANGE, 22792 },
	{ 0x93b9, 0x93bb, PDF_CMAP_TABLE, 9450 },
	{ 0x93c1, 0x93c1, PDF_CMAP_SINGLE, 15244 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 7026 },
	{ 0x93c4, 0x93c5, PDF_CMAP_RANGE, 22795 },
	{ 0x93c6, 0x93cd, PDF_CMAP_TABLE, 9453 },
	{ 0x93d0, 0x93d1, PDF_CMAP_TABLE, 9461 },
	{ 0x93d3, 0x93d3, PDF_CMAP_SINGLE, 22800 },
	{ 0x93d6, 0x93d7, PDF_CMAP_RANGE, 7021 },
	{ 0x93d8, 0x93d9, PDF_CMAP_TABLE, 9463 },
	{ 0x93dc, 0x93df, PDF_CMAP_TABLE, 9465 },
	{ 0x93e1, 0x93e2, PDF_CMAP_TABLE, 9469 },
	{ 0x93e4, 0x93e8, PDF_CMAP_TABLE, 9471 },
	{ 0x93f1, 0x93f1, PDF_CMAP_SINGLE, 18864 },
	{ 0x93f5, 0x93f5, PDF_CMAP_SINGLE, 18861 },
	{ 0x93f7, 0x93fb, PDF_CMAP_TABLE, 9476 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 15248 },
	{ 0x9401, 0x9404, PDF_CMAP_TABLE, 9481 },
	{ 0x9407, 0x9409, PDF_CMAP_TABLE, 9485 },
	{ 0x940d, 0x940e, PDF_CMAP_RANGE, 19938 },
	{ 0x940f, 0x9410, PDF_CMAP_TABLE, 9488 },
	{ 0x9413, 0x9415, PDF_CMAP_TABLE, 9490 },
	{ 0x9416, 0x9417, PDF_CMAP_RANGE, 18869 },
	{ 0x9418, 0x941a, PDF_CMAP_TABLE, 9493 },
	{ 0x941f, 0x941f, PDF_CMAP_SINGLE, 22806 },
	{ 0x9421, 0x9421, PDF_CMAP_SINGLE, 7040 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 7038 },
	{ 0x942e, 0x942f, PDF_CMAP_TABLE, 9496 },
	{ 0x9431, 0x9436, PDF_CMAP_TABLE, 9498 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 2904 },
	{ 0x943a, 0x943b, PDF_CMAP_TABLE, 9504 },
	{ 0x943d, 0x943d, PDF_CMAP_SINGLE, 22808 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 15251 },
	{ 0x9441, 0x9441, PDF_CMAP_SINGLE, 7042 },
	{ 0x9443, 0x9445, PDF_CMAP_TABLE, 9506 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 8684 },
	{ 0x944a, 0x944a, PDF_CMAP_SINGLE, 17173 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 19941 },
	{ 0x9451, 0x9453, PDF_CMAP_TABLE, 9509 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 15252 },
	{ 0x9459, 0x945c, PDF_CMAP_TABLE, 9512 },
	{ 0x945e, 0x9463, PDF_CMAP_TABLE, 9516 },
	{ 0x9468, 0x9468, PDF_CMAP_SINGLE, 22814 },
	{ 0x946a, 0x946b, PDF_CMAP_TABLE, 9522 },
	{ 0x946d, 0x9472, PDF_CMAP_TABLE, 9524 },
	{ 0x9475, 0x9475, PDF_CMAP_SINGLE, 7052 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 7053 },
	{ 0x947c, 0x947f, PDF_CMAP_TABLE, 9530 },
	{ 0x9481, 0x9481, PDF_CMAP_SINGLE, 7058 },
	{ 0x9483, 0x9484, PDF_CMAP_TABLE, 9534 },
	{ 0x9577, 0x9579, PDF_CMAP_TABLE, 9536 },
	{ 0x957e, 0x957e, PDF_CMAP_SINGLE, 22818 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 3827 },
	{ 0x9582, 0x9584, PDF_CMAP_TABLE, 9539 },
	{ 0x9586, 0x958b, PDF_CMAP_TABLE, 9542 },
	{ 0x958c, 0x958d, PDF_CMAP_RANGE, 18880 },
	{ 0x958e, 0x958f, PDF_CMAP_TABLE, 9548 },
	{ 0x9591, 0x9594, PDF_CMAP_TABLE, 9550 },
	{ 0x9596, 0x9596, PDF_CMAP_SINGLE, 7064 },
	{ 0x9598, 0x9599, PDF_CMAP_RANGE, 7065 },
	{ 0x959d, 0x959e, PDF_CMAP_RANGE, 22820 },
	{ 0x959f, 0x95a9, PDF_CMAP_TABLE, 9554 },
	{ 0x95ab, 0x95ad, PDF_CMAP_TABLE, 9565 },
	{ 0x95b1, 0x95b2, PDF_CMAP_TABLE, 9568 },
	{ 0x95b4, 0x95b4, PDF_CMAP_SINGLE, 15259 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 17179 },
	{ 0x95b9, 0x95bf, PDF_CMAP_TABLE, 9570 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 7077 },
	{ 0x95c6, 0x95cd, PDF_CMAP_TABLE, 9577 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 17181 },
	{ 0x95d1, 0x95d2, PDF_CMAP_RANGE, 19947 },
	{ 0x95d3, 0x95d6, PDF_CMAP_TABLE, 9585 },
	{ 0x95d8, 0x95da, PDF_CMAP_TABLE, 9589 },
	{ 0x95dc, 0x95e2, PDF_CMAP_TABLE, 9592 },
	{ 0x95e4, 0x95e6, PDF_CMAP_TABLE, 9599 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 14061 },
	{ 0x961c, 0x961e, PDF_CMAP_TABLE, 9602 },
	{ 0x9621, 0x9622, PDF_CMAP_TABLE, 9605 },
	{ 0x9624, 0x9624, PDF_CMAP_SINGLE, 19952 },
	{ 0x9625, 0x9626, PDF_CMAP_RANGE, 22828 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 7088 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 2133 },
	{ 0x962c, 0x962c, PDF_CMAP_SINGLE, 18887 },
	{ 0x962e, 0x962f, PDF_CMAP_RANGE, 7089 },
	{ 0x9631, 0x9632, PDF_CMAP_TABLE, 9607 },
	{ 0x9633, 0x9634, PDF_CMAP_RANGE, 18888 },
	{ 0x9637, 0x9638, PDF_CMAP_TABLE, 9609 },
	{ 0x9639, 0x963a, PDF_CMAP_RANGE, 22831 },
	{ 0x963b, 0x963d, PDF_CMAP_TABLE, 9611 },
	{ 0x963f, 0x9642, PDF_CMAP_TABLE, 9614 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 3551 },
	{ 0x964b, 0x964d, PDF_CMAP_TABLE, 9618 },
	{ 0x964f, 0x9650, PDF_CMAP_TABLE, 9621 },
	{ 0x9652, 0x9652, PDF_CMAP_SINGLE, 22833 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 19956 },
	{ 0x9656, 0x9657, PDF_CMAP_RANGE, 22834 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 15264 },
	{ 0x965b, 0x965f, PDF_CMAP_TABLE, 9623 },
	{ 0x9661, 0x9666, PDF_CMAP_TABLE, 9628 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 3356 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 7102 },
	{ 0x966e, 0x966e, PDF_CMAP_SINGLE, 22836 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 1220 },
	{ 0x9672, 0x9678, PDF_CMAP_TABLE, 9634 },
	{ 0x967a, 0x967f, PDF_CMAP_TABLE, 9641 },
	{ 0x9681, 0x9686, PDF_CMAP_TABLE, 9647 },
	{ 0x9688, 0x968b, PDF_CMAP_TABLE, 9653 },
	{ 0x968d, 0x968f, PDF_CMAP_TABLE, 9657 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 22839 },
	{ 0x9694, 0x969d, PDF_CMAP_TABLE, 9660 },
	{ 0x969f, 0x96a0, PDF_CMAP_TABLE, 9670 },
	{ 0x96a3, 0x96aa, PDF_CMAP_TABLE, 9672 },
	{ 0x96ae, 0x96b0, PDF_CMAP_TABLE, 9680 },
	{ 0x96b1, 0x96b2, PDF_CMAP_RANGE, 7109 },
	{ 0x96b3, 0x96b4, PDF_CMAP_TABLE, 9683 },
	{ 0x96b6, 0x96b7, PDF_CMAP_TABLE, 9685 },
	{ 0x96b8, 0x96b9, PDF_CMAP_RANGE, 7114 },
	{ 0x96ba, 0x96bd, PDF_CMAP_TABLE, 9687 },
	{ 0x96c0, 0x96c1, PDF_CMAP_TABLE, 9691 },
	{ 0x96c4, 0x96c7, PDF_CMAP_TABLE, 9693 },
	{ 0x96c9, 0x96ce, PDF_CMAP_TABLE, 9697 },
	{ 0x96d1, 0x96d2, PDF_CMAP_TABLE, 9703 },
	{ 0x96d5, 0x96d6, PDF_CMAP_TABLE, 9705 },
	{ 0x96d8, 0x96df, PDF_CMAP_TABLE, 9707 },
	{ 0x96e2, 0x96e3, PDF_CMAP_TABLE, 9715 },
	{ 0x96e8, 0x96eb, PDF_CMAP_TABLE, 9717 },
	{ 0x96ef, 0x96f2, PDF_CMAP_TABLE, 9721 },
	{ 0x96f6, 0x96f7, PDF_CMAP_TABLE, 9725 },
	{ 0x96f9, 0x96fb, PDF_CMAP_TABLE, 9727 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 2343 },
	{ 0x9702, 0x970a, PDF_CMAP_TABLE, 9730 },
	{ 0x970d, 0x970f, PDF_CMAP_TABLE, 9739 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 7130 },
	{ 0x9713, 0x9714, PDF_CMAP_TABLE, 9742 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 7132 },
	{ 0x9719, 0x971e, PDF_CMAP_TABLE, 9744 },
	{ 0x9721, 0x9722, PDF_CMAP_RANGE, 19967 },
	{ 0x9723, 0x9724, PDF_CMAP_TABLE, 9750 },
	{ 0x9727, 0x9728, PDF_CMAP_TABLE, 9752 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7135 },
	{ 0x9730, 0x9733, PDF_CMAP_TABLE, 9754 },
	{ 0x9736, 0x9736, PDF_CMAP_SINGLE, 18911 },
	{ 0x9738, 0x9739, PDF_CMAP_TABLE, 9758 },
	{ 0x973b, 0x973b, PDF_CMAP_SINGLE, 8691 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 7138 },
	{ 0x9741, 0x9744, PDF_CMAP_TABLE, 9760 },
	{ 0x9746, 0x974a, PDF_CMAP_TABLE, 9764 },
	{ 0x974d, 0x974f, PDF_CMAP_TABLE, 9769 },
	{ 0x9751, 0x9752, PDF_CMAP_TABLE, 9772 },
	{ 0x9755, 0x975c, PDF_CMAP_TABLE, 9774 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 3463 },
	{ 0x9760, 0x9764, PDF_CMAP_TABLE, 9782 },
	{ 0x9766, 0x976b, PDF_CMAP_TABLE, 9787 },
	{ 0x976d, 0x976e, PDF_CMAP_TABLE, 9793 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 7152 },
	{ 0x9773, 0x9774, PDF_CMAP_TABLE, 9795 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 19972 },
	{ 0x9777, 0x9778, PDF_CMAP_RANGE, 22852 },
	{ 0x9779, 0x977d, PDF_CMAP_TABLE, 9797 },
	{ 0x977f, 0x9781, PDF_CMAP_TABLE, 9802 },
	{ 0x9784, 0x9786, PDF_CMAP_TABLE, 9805 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 22856 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 7159 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 1164 },
	{ 0x978f, 0x9790, PDF_CMAP_RANGE, 7160 },
	{ 0x9795, 0x979a, PDF_CMAP_TABLE, 9808 },
	{ 0x979c, 0x979c, PDF_CMAP_SINGLE, 7162 },
	{ 0x979e, 0x97a0, PDF_CMAP_TABLE, 9814 },
	{ 0x97a2, 0x97a3, PDF_CMAP_TABLE, 9817 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 7164 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 7163 },
	{ 0x97ab, 0x97ae, PDF_CMAP_TABLE, 9819 },
	{ 0x97b1, 0x97b2, PDF_CMAP_RANGE, 18921 },
	{ 0x97b3, 0x97b4, PDF_CMAP_RANGE, 7166 },
	{ 0x97b5, 0x97b6, PDF_CMAP_RANGE, 15281 },
	{ 0x97b8, 0x97ba, PDF_CMAP_TABLE, 9823 },
	{ 0x97bc, 0x97bc, PDF_CMAP_SINGLE, 22859 },
	{ 0x97be, 0x97bf, PDF_CMAP_TABLE, 9826 },
	{ 0x97c1, 0x97c1, PDF_CMAP_SINGLE, 17191 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 7168 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 22861 },
	{ 0x97c6, 0x97ce, PDF_CMAP_TABLE, 9828 },
	{ 0x97d0, 0x97d1, PDF_CMAP_TABLE, 9837 },
	{ 0x97d3, 0x97d4, PDF_CMAP_TABLE, 9839 },
	{ 0x97d7, 0x97d9, PDF_CMAP_TABLE, 9841 },
	{ 0x97db, 0x97de, PDF_CMAP_TABLE, 9844 },
	{ 0x97e0, 0x97e1, PDF_CMAP_TABLE, 9848 },
	{ 0x97e4, 0x97e4, PDF_CMAP_SINGLE, 22869 },
	{ 0x97ed, 0x97ef, PDF_CMAP_TABLE, 9850 },
	{ 0x97f1, 0x97f6, PDF_CMAP_TABLE, 9853 },
	{ 0x97f7, 0x97f8, PDF_CMAP_RANGE, 22870 },
	{ 0x97fa, 0x97fb, PDF_CMAP_TABLE, 9859 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 1721 },
	{ 0x9801, 0x9808, PDF_CMAP_TABLE, 9861 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 15286 },
	{ 0x980c, 0x9814, PDF_CMAP_TABLE, 9869 },
	{ 0x9816, 0x981a, PDF_CMAP_TABLE, 9878 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 22874 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 15288 },
	{ 0x9820, 0x9821, PDF_CMAP_TABLE, 9883 },
	{ 0x9823, 0x9826, PDF_CMAP_TABLE, 9885 },
	{ 0x982b, 0x9830, PDF_CMAP_TABLE, 9889 },
	{ 0x9832, 0x9833, PDF_CMAP_RANGE, 18936 },
	{ 0x9834, 0x9835, PDF_CMAP_TABLE, 9895 },
	{ 0x9837, 0x9839, PDF_CMAP_TABLE, 9897 },
	{ 0x983b, 0x983e, PDF_CMAP_TABLE, 9900 },
	{ 0x9844, 0x9844, PDF_CMAP_SINGLE, 22878 },
	{ 0x9846, 0x9847, PDF_CMAP_TABLE, 9904 },
	{ 0x984a, 0x984c, PDF_CMAP_TABLE, 9906 },
	{ 0x984d, 0x984e, PDF_CMAP_RANGE, 1465 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 7187 },
	{ 0x9851, 0x9851, PDF_CMAP_SINGLE, 22880 },
	{ 0x9852, 0x9853, PDF_CMAP_RANGE, 15292 },
	{ 0x9854, 0x985b, PDF_CMAP_TABLE, 9909 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 4008 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 19985 },
	{ 0x9865, 0x9867, PDF_CMAP_TABLE, 9917 },
	{ 0x986a, 0x986c, PDF_CMAP_TABLE, 9920 },
	{ 0x986f, 0x9871, PDF_CMAP_RANGE, 7190 },
	{ 0x9873, 0x9874, PDF_CMAP_TABLE, 9923 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 3561 },
	{ 0x98aa, 0x98ab, PDF_CMAP_TABLE, 9925 },
	{ 0x98ad, 0x98b1, PDF_CMAP_TABLE, 9927 },
	{ 0x98b4, 0x98b4, PDF_CMAP_SINGLE, 19987 },
	{ 0x98b6, 0x98b8, PDF_CMAP_TABLE, 9932 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 15297 },
	{ 0x98bb, 0x98bc, PDF_CMAP_RANGE, 18946 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 15298 },
	{ 0x98c2, 0x98c8, PDF_CMAP_TABLE, 9935 },
	{ 0x98cb, 0x98cc, PDF_CMAP_TABLE, 9942 },
	{ 0x98db, 0x98dc, PDF_CMAP_TABLE, 9944 },
	{ 0x98df, 0x98e3, PDF_CMAP_TABLE, 9946 },
	{ 0x98e5, 0x98e7, PDF_CMAP_TABLE, 9951 },
	{ 0x98e9, 0x98eb, PDF_CMAP_TABLE, 9954 },
	{ 0x98ed, 0x98ef, PDF_CMAP_TABLE, 9957 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 18954 },
	{ 0x98f2, 0x98f4, PDF_CMAP_TABLE, 9960 },
	{ 0x98f6, 0x98f6, PDF_CMAP_SINGLE, 22885 },
	{ 0x98fc, 0x98fe, PDF_CMAP_TABLE, 9963 },
	{ 0x9902, 0x9903, PDF_CMAP_TABLE, 9966 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 3819 },
	{ 0x9907, 0x990a, PDF_CMAP_TABLE, 9968 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 1252 },
	{ 0x9910, 0x9915, PDF_CMAP_TABLE, 9972 },
	{ 0x9916, 0x9917, PDF_CMAP_RANGE, 18960 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 7208 },
	{ 0x991a, 0x991c, PDF_CMAP_RANGE, 18963 },
	{ 0x991d, 0x991e, PDF_CMAP_RANGE, 7210 },
	{ 0x991f, 0x9922, PDF_CMAP_TABLE, 9978 },
	{ 0x9924, 0x9924, PDF_CMAP_SINGLE, 7212 },
	{ 0x9926, 0x9928, PDF_CMAP_TABLE, 9982 },
	{ 0x992b, 0x992c, PDF_CMAP_TABLE, 9985 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7215 },
	{ 0x9931, 0x9931, PDF_CMAP_SINGLE, 18967 },
	{ 0x9932, 0x9933, PDF_CMAP_RANGE, 15301 },
	{ 0x9934, 0x9935, PDF_CMAP_TABLE, 9987 },
	{ 0x9939, 0x9939, PDF_CMAP_SINGLE, 22892 },
	{ 0x993a, 0x993c, PDF_CMAP_RANGE, 18968 },
	{ 0x993d, 0x993e, PDF_CMAP_RANGE, 7216 },
	{ 0x9940, 0x9942, PDF_CMAP_TABLE, 9989 },
	{ 0x9945, 0x9949, PDF_CMAP_TABLE, 9992 },
	{ 0x994b, 0x994e, PDF_CMAP_TABLE, 9997 },
	{ 0x9950, 0x9950, PDF_CMAP_SINGLE, 7221 },
	{ 0x9951, 0x9952, PDF_CMAP_RANGE, 7223 },
	{ 0x9954, 0x9955, PDF_CMAP_TABLE, 10001 },
	{ 0x9957, 0x9959, PDF_CMAP_TABLE, 10003 },
	{ 0x995b, 0x995c, PDF_CMAP_TABLE, 10006 },
	{ 0x995e, 0x9960, PDF_CMAP_TABLE, 10008 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 2335 },
	{ 0x9997, 0x9998, PDF_CMAP_RANGE, 7227 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 2035 },
	{ 0x999b, 0x999b, PDF_CMAP_SINGLE, 22896 },
	{ 0x999d, 0x999f, PDF_CMAP_TABLE, 10011 },
	{ 0x99a3, 0x99a3, PDF_CMAP_SINGLE, 18975 },
	{ 0x99a5, 0x99a6, PDF_CMAP_TABLE, 10014 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 1436 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 3333 },
	{ 0x99ad, 0x99ae, PDF_CMAP_RANGE, 7230 },
	{ 0x99b0, 0x99b5, PDF_CMAP_TABLE, 10016 },
	{ 0x99b9, 0x99ba, PDF_CMAP_RANGE, 15308 },
	{ 0x99bc, 0x99bd, PDF_CMAP_TABLE, 10022 },
	{ 0x99bf, 0x99bf, PDF_CMAP_SINGLE, 18978 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 3379 },
	{ 0x99c3, 0x99c6, PDF_CMAP_TABLE, 10024 },
	{ 0x99c8, 0x99c9, PDF_CMAP_TABLE, 10028 },
	{ 0x99d0, 0x99d5, PDF_CMAP_TABLE, 10030 },
	{ 0x99d8, 0x99df, PDF_CMAP_TABLE, 10036 },
	{ 0x99e1, 0x99e2, PDF_CMAP_TABLE, 10044 },
	{ 0x99e7, 0x99e7, PDF_CMAP_SINGLE, 22905 },
	{ 0x99ea, 0x99ec, PDF_CMAP_RANGE, 22906 },
	{ 0x99ed, 0x99ee, PDF_CMAP_RANGE, 7238 },
	{ 0x99f0, 0x99f0, PDF_CMAP_SINGLE, 18984 },
	{ 0x99f1, 0x99f2, PDF_CMAP_RANGE, 7240 },
	{ 0x99f4, 0x99f5, PDF_CMAP_RANGE, 22909 },
	{ 0x99f8, 0x99f9, PDF_CMAP_TABLE, 10046 },
	{ 0x99fb, 0x99fc, PDF_CMAP_TABLE, 10048 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 22911 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 2403 },
	{ 0x9a01, 0x9a05, PDF_CMAP_TABLE, 10050 },
	{ 0x9a08, 0x9a08, PDF_CMAP_SINGLE, 14266 },
	{ 0x9a0a, 0x9a0c, PDF_CMAP_TABLE, 10055 },
	{ 0x9a0e, 0x9a13, PDF_CMAP_TABLE, 10058 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 15312 },
	{ 0x9a19, 0x9a1a, PDF_CMAP_TABLE, 10064 },
	{ 0x9a1e, 0x9a1e, PDF_CMAP_SINGLE, 22915 },
	{ 0x9a20, 0x9a20, PDF_CMAP_SINGLE, 18990 },
	{ 0x9a22, 0x9a24, PDF_CMAP_TABLE, 10066 },
	{ 0x9a27, 0x9a28, PDF_CMAP_TABLE, 10069 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 7249 },
	{ 0x9a2d, 0x9a2e, PDF_CMAP_RANGE, 15315 },
	{ 0x9a30, 0x9a31, PDF_CMAP_TABLE, 10071 },
	{ 0x9a33, 0x9a33, PDF_CMAP_SINGLE, 22917 },
	{ 0x9a35, 0x9a38, PDF_CMAP_TABLE, 10073 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 7255 },
	{ 0x9a40, 0x9a45, PDF_CMAP_TABLE, 10077 },
	{ 0x9a47, 0x9a47, PDF_CMAP_SINGLE, 22919 },
	{ 0x9a4a, 0x9a4e, PDF_CMAP_TABLE, 10083 },
	{ 0x9a51, 0x9a52, PDF_CMAP_TABLE, 10088 },
	{ 0x9a54, 0x9a58, PDF_CMAP_TABLE, 10090 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_TABLE, 10095 },
	{ 0x9a5d, 0x9a5d, PDF_CMAP_SINGLE, 22923 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 7260 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 7261 },
	{ 0x9a64, 0x9a65, PDF_CMAP_TABLE, 10097 },
	{ 0x9a69, 0x9a6b, PDF_CMAP_TABLE, 10099 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2062 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 22924 },
	{ 0x9aac, 0x9ab0, PDF_CMAP_TABLE, 10102 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 22927 },
	{ 0x9ab4, 0x9ab4, PDF_CMAP_SINGLE, 22928 },
	{ 0x9ab5, 0x9ab6, PDF_CMAP_RANGE, 15321 },
	{ 0x9ab7, 0x9ab9, PDF_CMAP_TABLE, 10107 },
	{ 0x9abb, 0x9abc, PDF_CMAP_TABLE, 10110 },
	{ 0x9abd, 0x9abe, PDF_CMAP_RANGE, 20002 },
	{ 0x9abf, 0x9ac1, PDF_CMAP_TABLE, 10112 },
	{ 0x9ac3, 0x9ac4, PDF_CMAP_TABLE, 10115 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 19002 },
	{ 0x9ac8, 0x9ac8, PDF_CMAP_SINGLE, 22931 },
	{ 0x9ace, 0x9ad2, PDF_CMAP_TABLE, 10117 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 7273 },
	{ 0x9ad5, 0x9ad9, PDF_CMAP_TABLE, 10122 },
	{ 0x9adb, 0x9adc, PDF_CMAP_TABLE, 10127 },
	{ 0x9ade, 0x9adf, PDF_CMAP_RANGE, 7275 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 19006 },
	{ 0x9ae2, 0x9ae3, PDF_CMAP_RANGE, 7277 },
	{ 0x9ae4, 0x9ae7, PDF_CMAP_TABLE, 10129 },
	{ 0x9ae9, 0x9aef, PDF_CMAP_TABLE, 10133 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 7284 },
	{ 0x9af2, 0x9af3, PDF_CMAP_RANGE, 22937 },
	{ 0x9af4, 0x9af5, PDF_CMAP_TABLE, 10140 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 7285 },
	{ 0x9af9, 0x9afb, PDF_CMAP_TABLE, 10142 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 22941 },
	{ 0x9aff, 0x9b00, PDF_CMAP_RANGE, 22942 },
	{ 0x9b01, 0x9b03, PDF_CMAP_TABLE, 10145 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 22944 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 7287 },
	{ 0x9b08, 0x9b09, PDF_CMAP_TABLE, 10148 },
	{ 0x9b0b, 0x9b0c, PDF_CMAP_TABLE, 10150 },
	{ 0x9b0d, 0x9b0e, PDF_CMAP_RANGE, 20007 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 19010 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 19011 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 19012 },
	{ 0x9b18, 0x9b1d, PDF_CMAP_TABLE, 10152 },
	{ 0x9b1f, 0x9b20, PDF_CMAP_TABLE, 10158 },
	{ 0x9b22, 0x9b23, PDF_CMAP_RANGE, 7291 },
	{ 0x9b25, 0x9b26, PDF_CMAP_TABLE, 10160 },
	{ 0x9b27, 0x9b2a, PDF_CMAP_RANGE, 7294 },
	{ 0x9b2b, 0x9b2d, PDF_CMAP_TABLE, 10162 },
	{ 0x9b2e, 0x9b2f, PDF_CMAP_RANGE, 7298 },
	{ 0x9b31, 0x9b32, PDF_CMAP_TABLE, 10165 },
	{ 0x9b33, 0x9b34, PDF_CMAP_RANGE, 15326 },
	{ 0x9b35, 0x9b35, PDF_CMAP_SINGLE, 20010 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 22949 },
	{ 0x9b39, 0x9b3a, PDF_CMAP_RANGE, 22950 },
	{ 0x9b3b, 0x9b3d, PDF_CMAP_TABLE, 10167 },
	{ 0x9b41, 0x9b45, PDF_CMAP_TABLE, 10170 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 20011 },
	{ 0x9b4b, 0x9b4c, PDF_CMAP_TABLE, 10175 },
	{ 0x9b4d, 0x9b4e, PDF_CMAP_RANGE, 7304 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 7303 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 7306 },
	{ 0x9b54, 0x9b55, PDF_CMAP_TABLE, 10177 },
	{ 0x9b56, 0x9b57, PDF_CMAP_RANGE, 22953 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 7307 },
	{ 0x9b5a, 0x9b5b, PDF_CMAP_TABLE, 10179 },
	{ 0x9b5e, 0x9b5e, PDF_CMAP_SINGLE, 17206 },
	{ 0x9b61, 0x9b61, PDF_CMAP_SINGLE, 22956 },
	{ 0x9b63, 0x9b63, PDF_CMAP_SINGLE, 19018 },
	{ 0x9b65, 0x9b66, PDF_CMAP_TABLE, 10181 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 20013 },
	{ 0x9b6a, 0x9b6a, PDF_CMAP_SINGLE, 22957 },
	{ 0x9b6b, 0x9b6c, PDF_CMAP_RANGE, 19020 },
	{ 0x9b6d, 0x9b6e, PDF_CMAP_RANGE, 22958 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4043 },
	{ 0x9b72, 0x9b75, PDF_CMAP_TABLE, 10183 },
	{ 0x9b76, 0x9b77, PDF_CMAP_RANGE, 19022 },
	{ 0x9b78, 0x9b79, PDF_CMAP_TABLE, 10187 },
	{ 0x9b7f, 0x9b80, PDF_CMAP_TABLE, 10189 },
	{ 0x9b83, 0x9b87, PDF_CMAP_TABLE, 10191 },
	{ 0x9b89, 0x9b8b, PDF_CMAP_TABLE, 10196 },
	{ 0x9b8d, 0x9b94, PDF_CMAP_TABLE, 10199 },
	{ 0x9b96, 0x9b97, PDF_CMAP_RANGE, 7312 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 22968 },
	{ 0x9b9d, 0x9b9e, PDF_CMAP_TABLE, 10207 },
	{ 0x9b9f, 0x9ba0, PDF_CMAP_RANGE, 7314 },
	{ 0x9ba6, 0x9bae, PDF_CMAP_TABLE, 10209 },
	{ 0x9bb0, 0x9bb2, PDF_CMAP_TABLE, 10218 },
	{ 0x9bb4, 0x9bb4, PDF_CMAP_SINGLE, 7317 },
	{ 0x9bb7, 0x9bb9, PDF_CMAP_TABLE, 10221 },
	{ 0x9bbb, 0x9bbc, PDF_CMAP_TABLE, 10224 },
	{ 0x9bbe, 0x9bc1, PDF_CMAP_TABLE, 10226 },
	{ 0x9bc6, 0x9bca, PDF_CMAP_TABLE, 10230 },
	{ 0x9bce, 0x9bd0, PDF_CMAP_TABLE, 10235 },
	{ 0x9bd1, 0x9bd2, PDF_CMAP_RANGE, 7323 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 7328 },
	{ 0x9bd6, 0x9bd8, PDF_CMAP_TABLE, 10238 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 2884 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 19032 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 14271 },
	{ 0x9be1, 0x9be5, PDF_CMAP_TABLE, 10241 },
	{ 0x9be7, 0x9be8, PDF_CMAP_TABLE, 10246 },
	{ 0x9bea, 0x9beb, PDF_CMAP_TABLE, 10248 },
	{ 0x9bee, 0x9bf3, PDF_CMAP_TABLE, 10250 },
	{ 0x9bf5, 0x9bf5, PDF_CMAP_SINGLE, 1143 },
	{ 0x9bf7, 0x9bfa, PDF_CMAP_TABLE, 10256 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 15338 },
	{ 0x9bff, 0x9c00, PDF_CMAP_TABLE, 10260 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 20022 },
	{ 0x9c04, 0x9c04, PDF_CMAP_SINGLE, 7343 },
	{ 0x9c06, 0x9c06, PDF_CMAP_SINGLE, 7339 },
	{ 0x9c08, 0x9c0d, PDF_CMAP_TABLE, 10262 },
	{ 0x9c0f, 0x9c16, PDF_CMAP_TABLE, 10268 },
	{ 0x9c18, 0x9c1a, PDF_CMAP_RANGE, 19040 },
	{ 0x9c1b, 0x9c1e, PDF_CMAP_TABLE, 10276 },
	{ 0x9c21, 0x9c2a, PDF_CMAP_TABLE, 10280 },
	{ 0x9c2d, 0x9c32, PDF_CMAP_TABLE, 10290 },
	{ 0x9c35, 0x9c37, PDF_CMAP_TABLE, 10296 },
	{ 0x9c39, 0x9c3b, PDF_CMAP_TABLE, 10299 },
	{ 0x9c3d, 0x9c3e, PDF_CMAP_TABLE, 10302 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 15343 },
	{ 0x9c43, 0x9c48, PDF_CMAP_TABLE, 10304 },
	{ 0x9c49, 0x9c4a, PDF_CMAP_RANGE, 19052 },
	{ 0x9c4e, 0x9c4e, PDF_CMAP_SINGLE, 22981 },
	{ 0x9c4f, 0x9c50, PDF_CMAP_RANGE, 17215 },
	{ 0x9c52, 0x9c54, PDF_CMAP_TABLE, 10310 },
	{ 0x9c56, 0x9c58, PDF_CMAP_TABLE, 10313 },
	{ 0x9c5a, 0x9c61, PDF_CMAP_TABLE, 10316 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 15345 },
	{ 0x9c65, 0x9c65, PDF_CMAP_SINGLE, 17217 },
	{ 0x9c67, 0x9c68, PDF_CMAP_TABLE, 10324 },
	{ 0x9c69, 0x9c6b, PDF_CMAP_RANGE, 19060 },
	{ 0x9c6d, 0x9c6e, PDF_CMAP_RANGE, 19063 },
	{ 0x9c70, 0x9c70, PDF_CMAP_SINGLE, 15346 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 19065 },
	{ 0x9c75, 0x9c78, PDF_CMAP_TABLE, 10326 },
	{ 0x9c7a, 0x9c7b, PDF_CMAP_TABLE, 10330 },
	{ 0x9ce5, 0x9ce7, PDF_CMAP_TABLE, 10332 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3403 },
	{ 0x9ceb, 0x9cec, PDF_CMAP_TABLE, 10335 },
	{ 0x9cf0, 0x9cf0, PDF_CMAP_SINGLE, 7361 },
	{ 0x9cf2, 0x9cf4, PDF_CMAP_TABLE, 10337 },
	{ 0x9cf6, 0x9cf7, PDF_CMAP_TABLE, 10340 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 22985 },
	{ 0x9d02, 0x9d03, PDF_CMAP_TABLE, 10342 },
	{ 0x9d06, 0x9d09, PDF_CMAP_TABLE, 10344 },
	{ 0x9d0b, 0x9d0b, PDF_CMAP_SINGLE, 19070 },
	{ 0x9d0e, 0x9d0e, PDF_CMAP_SINGLE, 1322 },
	{ 0x9d11, 0x9d12, PDF_CMAP_TABLE, 10348 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 7373 },
	{ 0x9d17, 0x9d18, PDF_CMAP_RANGE, 19073 },
	{ 0x9d1b, 0x9d1c, PDF_CMAP_TABLE, 10350 },
	{ 0x9d1d, 0x9d1e, PDF_CMAP_RANGE, 17218 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 7371 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 7370 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 7368 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 1497 },
	{ 0x9d2a, 0x9d2c, PDF_CMAP_TABLE, 10352 },
	{ 0x9d2f, 0x9d30, PDF_CMAP_TABLE, 10355 },
	{ 0x9d32, 0x9d32, PDF_CMAP_SINGLE, 19078 },
	{ 0x9d33, 0x9d34, PDF_CMAP_RANGE, 22988 },
	{ 0x9d3a, 0x9d3f, PDF_CMAP_TABLE, 10357 },
	{ 0x9d41, 0x9d48, PDF_CMAP_TABLE, 10363 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 19080 },
	{ 0x9d50, 0x9d52, PDF_CMAP_TABLE, 10371 },
	{ 0x9d53, 0x9d54, PDF_CMAP_RANGE, 22993 },
	{ 0x9d59, 0x9d59, PDF_CMAP_SINGLE, 7385 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 1231 },
	{ 0x9d5d, 0x9d5e, PDF_CMAP_RANGE, 7380 },
	{ 0x9d5f, 0x9d65, PDF_CMAP_TABLE, 10374 },
	{ 0x9d69, 0x9d6c, PDF_CMAP_TABLE, 10381 },
	{ 0x9d6f, 0x9d70, PDF_CMAP_TABLE, 10385 },
	{ 0x9d72, 0x9d73, PDF_CMAP_TABLE, 10387 },
	{ 0x9d76, 0x9d77, PDF_CMAP_RANGE, 19086 },
	{ 0x9d7a, 0x9d7c, PDF_CMAP_TABLE, 10389 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15354 },
	{ 0x9d83, 0x9d84, PDF_CMAP_TABLE, 10392 },
	{ 0x9d86, 0x9d87, PDF_CMAP_TABLE, 10394 },
	{ 0x9d89, 0x9d8a, PDF_CMAP_TABLE, 10396 },
	{ 0x9d8d, 0x9d8f, PDF_CMAP_TABLE, 10398 },
	{ 0x9d92, 0x9d93, PDF_CMAP_RANGE, 22999 },
	{ 0x9d95, 0x9d96, PDF_CMAP_TABLE, 10401 },
	{ 0x9d97, 0x9d98, PDF_CMAP_RANGE, 23002 },
	{ 0x9d99, 0x9d9a, PDF_CMAP_TABLE, 10403 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 19090 },
	{ 0x9da4, 0x9da4, PDF_CMAP_SINGLE, 7393 },
	{ 0x9da9, 0x9dac, PDF_CMAP_TABLE, 10405 },
	{ 0x9dae, 0x9daf, PDF_CMAP_TABLE, 10409 },
	{ 0x9db1, 0x9db2, PDF_CMAP_TABLE, 10411 },
	{ 0x9db4, 0x9db5, PDF_CMAP_TABLE, 10413 },
	{ 0x9db8, 0x9dbd, PDF_CMAP_TABLE, 10415 },
	{ 0x9dbf, 0x9dc4, PDF_CMAP_TABLE, 10421 },
	{ 0x9dc6, 0x9dc7, PDF_CMAP_TABLE, 10427 },
	{ 0x9dc9, 0x9dca, PDF_CMAP_TABLE, 10429 },
	{ 0x9dcf, 0x9dcf, PDF_CMAP_SINGLE, 7402 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 7405 },
	{ 0x9dd4, 0x9dd5, PDF_CMAP_RANGE, 23008 },
	{ 0x9dd6, 0x9dd7, PDF_CMAP_TABLE, 10431 },
	{ 0x9dd9, 0x9dda, PDF_CMAP_TABLE, 10433 },
	{ 0x9dde, 0x9de0, PDF_CMAP_TABLE, 10435 },
	{ 0x9de3, 0x9de3, PDF_CMAP_SINGLE, 19098 },
	{ 0x9de5, 0x9de7, PDF_CMAP_TABLE, 10438 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 20034 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 15361 },
	{ 0x9ded, 0x9df0, PDF_CMAP_TABLE, 10441 },
	{ 0x9df2, 0x9df4, PDF_CMAP_TABLE, 10445 },
	{ 0x9df8, 0x9dfa, PDF_CMAP_TABLE, 10448 },
	{ 0x9dfd, 0x9dfe, PDF_CMAP_TABLE, 10451 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 19101 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 17229 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 19100 },
	{ 0x9e0d, 0x9e0e, PDF_CMAP_TABLE, 10453 },
	{ 0x9e10, 0x9e12, PDF_CMAP_RANGE, 23015 },
	{ 0x9e15, 0x9e16, PDF_CMAP_TABLE, 10455 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 8716 },
	{ 0x9e1a, 0x9e1b, PDF_CMAP_RANGE, 7411 },
	{ 0x9e1c, 0x9e1e, PDF_CMAP_TABLE, 10457 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 7414 },
	{ 0x9e78, 0x9e7d, PDF_CMAP_TABLE, 10460 },
	{ 0x9e7f, 0x9e81, PDF_CMAP_TABLE, 10466 },
	{ 0x9e82, 0x9e84, PDF_CMAP_RANGE, 20037 },
	{ 0x9e85, 0x9e85, PDF_CMAP_SINGLE, 19107 },
	{ 0x9e87, 0x9e88, PDF_CMAP_TABLE, 10469 },
	{ 0x9e8b, 0x9e8c, PDF_CMAP_RANGE, 7419 },
	{ 0x9e8e, 0x9e8f, PDF_CMAP_RANGE, 23020 },
	{ 0x9e91, 0x9e93, PDF_CMAP_TABLE, 10471 },
	{ 0x9e95, 0x9e98, PDF_CMAP_TABLE, 10474 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 19108 },
	{ 0x9e9d, 0x9e9f, PDF_CMAP_TABLE, 10478 },
	{ 0x9ea4, 0x9ea6, PDF_CMAP_TABLE, 10481 },
	{ 0x9ea8, 0x9eaa, PDF_CMAP_TABLE, 10484 },
	{ 0x9eac, 0x9eb0, PDF_CMAP_TABLE, 10487 },
	{ 0x9eb3, 0x9eb5, PDF_CMAP_TABLE, 10492 },
	{ 0x9eb8, 0x9ebf, PDF_CMAP_TABLE, 10495 },
	{ 0x9ec3, 0x9ec4, PDF_CMAP_TABLE, 10503 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 23026 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 23027 },
	{ 0x9ecb, 0x9ecd, PDF_CMAP_TABLE, 10505 },
	{ 0x9ece, 0x9ed0, PDF_CMAP_RANGE, 7432 },
	{ 0x9ed1, 0x9ed2, PDF_CMAP_TABLE, 10508 },
	{ 0x9ed4, 0x9ed5, PDF_CMAP_TABLE, 10510 },
	{ 0x9ed8, 0x9ed9, PDF_CMAP_TABLE, 10512 },
	{ 0x9edb, 0x9ee0, PDF_CMAP_TABLE, 10514 },
	{ 0x9ee4, 0x9ee5, PDF_CMAP_TABLE, 10520 },
	{ 0x9ee7, 0x9ee8, PDF_CMAP_TABLE, 10522 },
	{ 0x9eec, 0x9eed, PDF_CMAP_RANGE, 23030 },
	{ 0x9eee, 0x9ef2, PDF_CMAP_TABLE, 10524 },
	{ 0x9ef4, 0x9ef5, PDF_CMAP_TABLE, 10529 },
	{ 0x9ef6, 0x9ef7, PDF_CMAP_RANGE, 7444 },
	{ 0x9ef8, 0x9ef9, PDF_CMAP_TABLE, 10531 },
	{ 0x9efb, 0x9efd, PDF_CMAP_RANGE, 7447 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 19113 },
	{ 0x9f02, 0x9f03, PDF_CMAP_TABLE, 10533 },
	{ 0x9f07, 0x9f08, PDF_CMAP_RANGE, 7450 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 20044 },
	{ 0x9f0e, 0x9f17, PDF_CMAP_TABLE, 10535 },
	{ 0x9f19, 0x9f1b, PDF_CMAP_TABLE, 10545 },
	{ 0x9f1f, 0x9f22, PDF_CMAP_TABLE, 10548 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 20049 },
	{ 0x9f2a, 0x9f2b, PDF_CMAP_RANGE, 20050 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7455 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 15374 },
	{ 0x9f31, 0x9f32, PDF_CMAP_RANGE, 23039 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 20052 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 15375 },
	{ 0x9f39, 0x9f3f, PDF_CMAP_TABLE, 10552 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 15377 },
	{ 0x9f43, 0x9f44, PDF_CMAP_RANGE, 23043 },
	{ 0x9f45, 0x9f47, PDF_CMAP_TABLE, 10559 },
	{ 0x9f4a, 0x9f4b, PDF_CMAP_TABLE, 10562 },
	{ 0x9f4e, 0x9f4f, PDF_CMAP_TABLE, 10564 },
	{ 0x9f52, 0x9f58, PDF_CMAP_TABLE, 10566 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 20053 },
	{ 0x9f5d, 0x9f5e, PDF_CMAP_TABLE, 10573 },
	{ 0x9f5f, 0x9f61, PDF_CMAP_RANGE, 7461 },
	{ 0x9f62, 0x9f63, PDF_CMAP_TABLE, 10575 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 7464 },
	{ 0x9f68, 0x9f6a, PDF_CMAP_TABLE, 10577 },
	{ 0x9f6c, 0x9f73, PDF_CMAP_TABLE, 10580 },
	{ 0x9f75, 0x9f77, PDF_CMAP_TABLE, 10588 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 23050 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 23051 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 3966 },
	{ 0x9f8f, 0x9f90, PDF_CMAP_TABLE, 10591 },
	{ 0x9f91, 0x9f92, PDF_CMAP_RANGE, 23053 },
	{ 0x9f94, 0x9f97, PDF_CMAP_TABLE, 10593 },
	{ 0x9f9c, 0x9f9e, PDF_CMAP_TABLE, 10597 },
	{ 0x9fa0, 0x9fa3, PDF_CMAP_TABLE, 10600 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 20056 },
	{ 0x9fb4, 0x9fb4, PDF_CMAP_SINGLE, 14048 },
	{ 0x9fbc, 0x9fbe, PDF_CMAP_TABLE, 10604 },
	{ 0x9fbf, 0x9fc2, PDF_CMAP_RANGE, 20068 },
	{ 0xdc02, 0xdc02, PDF_CMAP_SINGLE, 18398 },
	{ 0xdc04, 0xdc04, PDF_CMAP_SINGLE, 15388 },
	{ 0xdc07, 0xdc07, PDF_CMAP_SINGLE, 17319 },
	{ 0xdc0b, 0xdc0c, PDF_CMAP_TABLE, 10607 },
	{ 0xdc0c, 0xdc0c, PDF_CMAP_SINGLE, 18754 },
	{ 0xdc0f, 0xdc10, PDF_CMAP_TABLE, 10609 },
	{ 0xdc15, 0xdc18, PDF_CMAP_TABLE, 10611 },
	{ 0xdc1a, 0xdc1a, PDF_CMAP_SINGLE, 19077 },
	{ 0xdc1a, 0xdc1a, PDF_CMAP_SINGLE, 13954 },
	{ 0xdc1d, 0xdc1d, PDF_CMAP_SINGLE, 17682 },
	{ 0xdc22, 0xdc22, PDF_CMAP_SINGLE, 13684 },
	{ 0xdc24, 0xdc24, PDF_CMAP_SINGLE, 17686 },
	{ 0xdc24, 0xdc24, PDF_CMAP_SINGLE, 17880 },
	{ 0xdc28, 0xdc29, PDF_CMAP_TABLE, 10615 },
	{ 0xdc2c, 0xdc2c, PDF_CMAP_SINGLE, 14109 },
	{ 0xdc2e, 0xdc2f, PDF_CMAP_TABLE, 10617 },
	{ 0xdc33, 0xdc33, PDF_CMAP_SINGLE, 13719 },
	{ 0xdc37, 0xdc37, PDF_CMAP_SINGLE, 19111 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 17321 },
	{ 0xdc3c, 0xdc3d, PDF_CMAP_TABLE, 10619 },
	{ 0xdc3f, 0xdc3f, PDF_CMAP_SINGLE, 13815 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 20062 },
	{ 0xdc49, 0xdc4a, PDF_CMAP_TABLE, 10621 },
	{ 0xdc4a, 0xdc4a, PDF_CMAP_SINGLE, 17782 },
	{ 0xdc4a, 0xdc4b, PDF_CMAP_TABLE, 10623 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 20168 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 18302 },
	{ 0xdc51, 0xdc52, PDF_CMAP_TABLE, 10625 },
	{ 0xdc55, 0xdc55, PDF_CMAP_SINGLE, 18183 },
	{ 0xdc55, 0xdc56, PDF_CMAP_TABLE, 10627 },
	{ 0xdc5b, 0xdc5b, PDF_CMAP_SINGLE, 17647 },
	{ 0xdc60, 0xdc60, PDF_CMAP_SINGLE, 18355 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 13998 },
	{ 0xdc64, 0xdc65, PDF_CMAP_TABLE, 10629 },
	{ 0xdc6d, 0xdc6d, PDF_CMAP_SINGLE, 17462 },
	{ 0xdc6d, 0xdc6e, PDF_CMAP_TABLE, 10631 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 18506 },
	{ 0xdc73, 0xdc74, PDF_CMAP_TABLE, 10633 },
	{ 0xdc77, 0xdc77, PDF_CMAP_SINGLE, 7754 },
	{ 0xdc7f, 0xdc7f, PDF_CMAP_SINGLE, 17910 },
	{ 0xdc82, 0xdc82, PDF_CMAP_SINGLE, 19084 },
	{ 0xdc84, 0xdc84, PDF_CMAP_SINGLE, 7734 },
	{ 0xdc89, 0xdc8a, PDF_CMAP_TABLE, 10635 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 18727 },
	{ 0xdc8e, 0xdc8e, PDF_CMAP_SINGLE, 17005 },
	{ 0xdc90, 0xdc90, PDF_CMAP_SINGLE, 18935 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 17998 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 18077 },
	{ 0xdc99, 0xdc9a, PDF_CMAP_TABLE, 10637 },
	{ 0xdc9e, 0xdc9e, PDF_CMAP_SINGLE, 20206 },
	{ 0xdca2, 0xdca2, PDF_CMAP_SINGLE, 17240 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 17243 },
	{ 0xdca6, 0xdca6, PDF_CMAP_SINGLE, 20064 },
	{ 0xdcab, 0xdcac, PDF_CMAP_TABLE, 10639 },
	{ 0xdcb0, 0xdcb0, PDF_CMAP_SINGLE, 14209 },
	{ 0xdcb2, 0xdcb2, PDF_CMAP_SINGLE, 13867 },
	{ 0xdcb6, 0xdcb6, PDF_CMAP_SINGLE, 14129 },
	{ 0xdcb8, 0xdcb9, PDF_CMAP_TABLE, 10641 },
	{ 0xdcbb, 0xdcbb, PDF_CMAP_SINGLE, 18733 },
	{ 0xdcbd, 0xdcbe, PDF_CMAP_TABLE, 10643 },
	{ 0xdcc6, 0xdcc6, PDF_CMAP_SINGLE, 18983 },
	{ 0xdcd3, 0xdcd3, PDF_CMAP_SINGLE, 7816 },
	{ 0xdcd9, 0xdcd9, PDF_CMAP_SINGLE, 18217 },
	{ 0xdcdb, 0xdcdd, PDF_CMAP_TABLE, 10645 },
	{ 0xdcdd, 0xdcdd, PDF_CMAP_SINGLE, 18515 },
	{ 0xdcdd, 0xdcdd, PDF_CMAP_SINGLE, 7641 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14291 },
	{ 0xdce4, 0xdce5, PDF_CMAP_TABLE, 10648 },
	{ 0xdcea, 0xdcea, PDF_CMAP_SINGLE, 18446 },
	{ 0xdcea, 0xdcea, PDF_CMAP_SINGLE, 13679 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 18365 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 7665 },
	{ 0xdcf4, 0xdcf5, PDF_CMAP_TABLE, 10650 },
	{ 0xdcf9, 0xdcf9, PDF_CMAP_SINGLE, 19083 },
	{ 0xdcfc, 0xdcfc, PDF_CMAP_SINGLE, 13656 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 13904 },
	{ 0xdd00, 0xdd00, PDF_CMAP_SINGLE, 17925 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 18003 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 13768 },
	{ 0xdd09, 0xdd09, PDF_CMAP_SINGLE, 17299 },
	{ 0xdd0b, 0xdd0b, PDF_CMAP_SINGLE, 13801 },
	{ 0xdd0d, 0xdd0e, PDF_CMAP_TABLE, 10652 },
	{ 0xdd0e, 0xdd0e, PDF_CMAP_SINGLE, 18394 },
	{ 0xdd0e, 0xdd10, PDF_CMAP_TABLE, 10654 },
	{ 0xdd14, 0xdd14, PDF_CMAP_SINGLE, 13995 },
	{ 0xdd1a, 0xdd1a, PDF_CMAP_SINGLE, 13916 },
	{ 0xdd20, 0xdd22, PDF_CMAP_TABLE, 10657 },
	{ 0xdd2d, 0xdd2d, PDF_CMAP_SINGLE, 17544 },
	{ 0xdd39, 0xdd39, PDF_CMAP_SINGLE, 18574 },
	{ 0xdd3f, 0xdd40, PDF_CMAP_TABLE, 10660 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 17359 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 17545 },
	{ 0xdd45, 0xdd47, PDF_CMAP_TABLE, 10662 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 18811 },
	{ 0xdd4b, 0xdd4b, PDF_CMAP_SINGLE, 13717 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 13646 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 20075 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 20090 },
	{ 0xdd5a, 0xdd5a, PDF_CMAP_SINGLE, 17815 },
	{ 0xdd62, 0xdd62, PDF_CMAP_SINGLE, 17547 },
	{ 0xdd6b, 0xdd6b, PDF_CMAP_SINGLE, 18770 },
	{ 0xdd6b, 0xdd6c, PDF_CMAP_TABLE, 10665 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 18450 },
	{ 0xdd71, 0xdd71, PDF_CMAP_SINGLE, 18883 },
	{ 0xdd73, 0xdd73, PDF_CMAP_SINGLE, 20060 },
	{ 0xdd78, 0xdd78, PDF_CMAP_SINGLE, 17546 },
	{ 0xdd7b, 0xdd7c, PDF_CMAP_TABLE, 10667 },
	{ 0xdd84, 0xdd85, PDF_CMAP_TABLE, 10669 },
	{ 0xdd87, 0xdd88, PDF_CMAP_TABLE, 10671 },
	{ 0xdd8f, 0xdd90, PDF_CMAP_TABLE, 10673 },
	{ 0xdd92, 0xdd92, PDF_CMAP_SINGLE, 17556 },
	{ 0xdd94, 0xdd95, PDF_CMAP_TABLE, 10675 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 14134 },
	{ 0xdd9c, 0xdd9d, PDF_CMAP_TABLE, 10677 },
	{ 0xdda0, 0xdda1, PDF_CMAP_TABLE, 10679 },
	{ 0xdda1, 0xdda2, PDF_CMAP_TABLE, 10681 },
	{ 0xdda7, 0xdda7, PDF_CMAP_SINGLE, 18229 },
	{ 0xdda9, 0xdda9, PDF_CMAP_SINGLE, 18190 },
	{ 0xddb1, 0xddb1, PDF_CMAP_SINGLE, 20080 },
	{ 0xddb4, 0xddb4, PDF_CMAP_SINGLE, 20133 },
	{ 0xddb6, 0xddb7, PDF_CMAP_TABLE, 10683 },
	{ 0xddba, 0xddbb, PDF_CMAP_RANGE, 18834 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 17525 },
	{ 0xddc3, 0xddc4, PDF_CMAP_TABLE, 10685 },
	{ 0xddc4, 0xddc4, PDF_CMAP_SINGLE, 16905 },
	{ 0xddc4, 0xddc4, PDF_CMAP_SINGLE, 17024 },
	{ 0xddc6, 0xddc6, PDF_CMAP_SINGLE, 18015 },
	{ 0xddc8, 0xddc9, PDF_CMAP_RANGE, 18773 },
	{ 0xddcc, 0xddcd, PDF_CMAP_TABLE, 10687 },
	{ 0xddcf, 0xddd0, PDF_CMAP_TABLE, 10689 },
	{ 0xddd3, 0xddd4, PDF_CMAP_TABLE, 10691 },
	{ 0xddd6, 0xddd7, PDF_CMAP_TABLE, 10693 },
	{ 0xdddb, 0xdddb, PDF_CMAP_SINGLE, 19026 },
	{ 0xdddd, 0xdde1, PDF_CMAP_TABLE, 10695 },
	{ 0xdde1, 0xdde1, PDF_CMAP_SINGLE, 17710 },
	{ 0xdde5, 0xdde5, PDF_CMAP_SINGLE, 18192 },
	{ 0xddf1, 0xddf1, PDF_CMAP_SINGLE, 19123 },
	{ 0xddf4, 0xddf5, PDF_CMAP_TABLE, 10700 },
	{ 0xddf9, 0xddfb, PDF_CMAP_TABLE, 10702 },
	{ 0xddfe, 0xddfe, PDF_CMAP_SINGLE, 18018 },
	{ 0xde01, 0xde02, PDF_CMAP_TABLE, 10705 },
	{ 0xde04, 0xde04, PDF_CMAP_SINGLE, 20058 },
	{ 0xde09, 0xde09, PDF_CMAP_SINGLE, 15443 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 18158 },
	{ 0xde10, 0xde11, PDF_CMAP_TABLE, 10707 },
	{ 0xde13, 0xde15, PDF_CMAP_TABLE, 10709 },
	{ 0xde15, 0xde15, PDF_CMAP_SINGLE, 19036 },
	{ 0xde17, 0xde18, PDF_CMAP_TABLE, 10712 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 7825 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 20316 },
	{ 0xde1e, 0xde1e, PDF_CMAP_SINGLE, 18195 },
	{ 0xde1e, 0xde1e, PDF_CMAP_SINGLE, 18455 },
	{ 0xde1e, 0xde1f, PDF_CMAP_TABLE, 10714 },
	{ 0xde22, 0xde22, PDF_CMAP_SINGLE, 13691 },
	{ 0xde28, 0xde29, PDF_CMAP_TABLE, 10716 },
	{ 0xde29, 0xde29, PDF_CMAP_SINGLE, 18844 },
	{ 0xde2e, 0xde2e, PDF_CMAP_SINGLE, 18318 },
	{ 0xde31, 0xde31, PDF_CMAP_SINGLE, 18640 },
	{ 0xde33, 0xde34, PDF_CMAP_TABLE, 10718 },
	{ 0xde36, 0xde3a, PDF_CMAP_TABLE, 10720 },
	{ 0xde3d, 0xde3d, PDF_CMAP_SINGLE, 13953 },
	{ 0xde3d, 0xde3d, PDF_CMAP_SINGLE, 20315 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 17089 },
	{ 0xde42, 0xde43, PDF_CMAP_TABLE, 10725 },
	{ 0xde47, 0xde47, PDF_CMAP_SINGLE, 17480 },
	{ 0xde47, 0xde47, PDF_CMAP_SINGLE, 17842 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 19038 },
	{ 0xde4c, 0xde4d, PDF_CMAP_TABLE, 10727 },
	{ 0xde55, 0xde56, PDF_CMAP_TABLE, 10729 },
	{ 0xde58, 0xde58, PDF_CMAP_SINGLE, 18459 },
	{ 0xde62, 0xde62, PDF_CMAP_SINGLE, 18322 },
	{ 0xde64, 0xde64, PDF_CMAP_SINGLE, 13755 },
	{ 0xde64, 0xde65, PDF_CMAP_TABLE, 10731 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 18528 },
	{ 0xde6a, 0xde6a, PDF_CMAP_SINGLE, 18167 },
	{ 0xde6a, 0xde6a, PDF_CMAP_SINGLE, 14189 },
	{ 0xde6d, 0xde6d, PDF_CMAP_SINGLE, 17380 },
	{ 0xde70, 0xde72, PDF_CMAP_TABLE, 10733 },
	{ 0xde74, 0xde74, PDF_CMAP_SINGLE, 17421 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 17140 },
	{ 0xde7b, 0xde7b, PDF_CMAP_SINGLE, 17417 },
	{ 0xde7e, 0xde7f, PDF_CMAP_TABLE, 10736 },
	{ 0xde7f, 0xde7f, PDF_CMAP_SINGLE, 13849 },
	{ 0xde82, 0xde82, PDF_CMAP_SINGLE, 18745 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 17117 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 18684 },
	{ 0xde86, 0xde86, PDF_CMAP_SINGLE, 18386 },
	{ 0xde89, 0xde8c, PDF_CMAP_TABLE, 10738 },
	{ 0xde93, 0xde93, PDF_CMAP_SINGLE, 18645 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 17379 },
	{ 0xde98, 0xde99, PDF_CMAP_TABLE, 10742 },
	{ 0xdea0, 0xdea0, PDF_CMAP_SINGLE, 18917 },
	{ 0xdea5, 0xdea5, PDF_CMAP_SINGLE, 18055 },
	{ 0xdeb0, 0xdeb2, PDF_CMAP_TABLE, 10744 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 19129 },
	{ 0xdeb4, 0xdeb4, PDF_CMAP_SINGLE, 16838 },
	{ 0xdeb7, 0xdeb8, PDF_CMAP_TABLE, 10747 },
	{ 0xdec2, 0xdec2, PDF_CMAP_SINGLE, 18327 },
	{ 0xdec4, 0xdec4, PDF_CMAP_SINGLE, 19046 },
	{ 0xdecd, 0xdecd, PDF_CMAP_SINGLE, 18856 },
	{ 0xded3, 0xded3, PDF_CMAP_SINGLE, 17337 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 17429 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 13922 },
	{ 0xded7, 0xded8, PDF_CMAP_TABLE, 10749 },
	{ 0xdedb, 0xdedb, PDF_CMAP_SINGLE, 19054 },
	{ 0xdedd, 0xdedd, PDF_CMAP_SINGLE, 18863 },
	{ 0xdee3, 0xdee4, PDF_CMAP_TABLE, 10751 },
	{ 0xdee4, 0xdee4, PDF_CMAP_SINGLE, 18276 },
	{ 0xdee4, 0xdee4, PDF_CMAP_SINGLE, 18862 },
	{ 0xdee8, 0xdee9, PDF_CMAP_TABLE, 10753 },
	{ 0xdeeb, 0xdeec, PDF_CMAP_TABLE, 10755 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 14282 },
	{ 0xdef0, 0xdef1, PDF_CMAP_TABLE, 10757 },
	{ 0xdef3, 0xdef3, PDF_CMAP_SINGLE, 17632 },
	{ 0xdef3, 0xdef3, PDF_CMAP_SINGLE, 18747 },
	{ 0xdef6, 0xdef6, PDF_CMAP_SINGLE, 7673 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 18787 },
	{ 0xdefd, 0xdefd, PDF_CMAP_SINGLE, 17435 },
	{ 0xdeff, 0xdeff, PDF_CMAP_SINGLE, 17063 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 17492 },
	{ 0xdf0c, 0xdf0c, PDF_CMAP_SINGLE, 17863 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18650 },
	{ 0xdf19, 0xdf19, PDF_CMAP_SINGLE, 18962 },
	{ 0xdf1b, 0xdf1d, PDF_CMAP_TABLE, 10759 },
	{ 0xdf1d, 0xdf1e, PDF_CMAP_TABLE, 10762 },
	{ 0xdf1e, 0xdf1e, PDF_CMAP_SINGLE, 17605 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18330 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18652 },
	{ 0xdf2b, 0xdf2b, PDF_CMAP_SINGLE, 17282 },
	{ 0xdf32, 0xdf32, PDF_CMAP_SINGLE, 18897 },
	{ 0xdf36, 0xdf36, PDF_CMAP_SINGLE, 17437 },
	{ 0xdf3f, 0xdf3f, PDF_CMAP_SINGLE, 16914 },
	{ 0xdf42, 0xdf42, PDF_CMAP_SINGLE, 17493 },
	{ 0xdf44, 0xdf44, PDF_CMAP_SINGLE, 17438 },
	{ 0xdf46, 0xdf46, PDF_CMAP_SINGLE, 17680 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 20311 },
	{ 0xdf4f, 0xdf4f, PDF_CMAP_SINGLE, 17312 },
	{ 0xdf4f, 0xdf50, PDF_CMAP_RANGE, 17671 },
	{ 0xdf50, 0xdf50, PDF_CMAP_SINGLE, 18966 },
	{ 0xdf52, 0xdf52, PDF_CMAP_SINGLE, 18656 },
	{ 0xdf56, 0xdf56, PDF_CMAP_SINGLE, 18117 },
	{ 0xdf5c, 0xdf5c, PDF_CMAP_SINGLE, 18332 },
	{ 0xdf5f, 0xdf5f, PDF_CMAP_SINGLE, 17391 },
	{ 0xdf63, 0xdf64, PDF_CMAP_TABLE, 10764 },
	{ 0xdf6f, 0xdf6f, PDF_CMAP_SINGLE, 18119 },
	{ 0xdf71, 0xdf71, PDF_CMAP_SINGLE, 17291 },
	{ 0xdf71, 0xdf72, PDF_CMAP_TABLE, 10766 },
	{ 0xdf76, 0xdf76, PDF_CMAP_SINGLE, 17582 },
	{ 0xdf7e, 0xdf7e, PDF_CMAP_SINGLE, 17983 },
	{ 0xdf81, 0xdf81, PDF_CMAP_SINGLE, 17289 },
	{ 0xdf8c, 0xdf8c, PDF_CMAP_SINGLE, 19109 },
	{ 0xdf94, 0xdf94, PDF_CMAP_SINGLE, 18544 },
	{ 0xdf9f, 0xdf9f, PDF_CMAP_SINGLE, 13803 },
	{ 0xdfa6, 0xdfa6, PDF_CMAP_SINGLE, 17683 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 18248 },
	{ 0xdfad, 0xdfad, PDF_CMAP_SINGLE, 17608 },
	{ 0xdfb2, 0xdfb4, PDF_CMAP_TABLE, 10768 },
	{ 0xdfb7, 0xdfb7, PDF_CMAP_SINGLE, 13706 },
	{ 0xdfb7, 0xdfb7, PDF_CMAP_SINGLE, 13898 },
	{ 0xdfbc, 0xdfbc, PDF_CMAP_SINGLE, 18039 },
	{ 0xdfbe, 0xdfbe, PDF_CMAP_SINGLE, 18701 },
	{ 0xdfc1, 0xdfc1, PDF_CMAP_SINGLE, 18874 },
	{ 0xdfc4, 0xdfc4, PDF_CMAP_SINGLE, 17449 },
	{ 0xdfc7, 0xdfc8, PDF_CMAP_TABLE, 10771 },
	{ 0xdfcd, 0xdfce, PDF_CMAP_TABLE, 10773 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 17768 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 7838 },
	{ 0xdfd2, 0xdfd5, PDF_CMAP_TABLE, 10775 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 19071 },
	{ 0xdfda, 0xdfda, PDF_CMAP_SINGLE, 17772 },
	{ 0xdfda, 0xdfdb, PDF_CMAP_TABLE, 10779 },
	{ 0xdfdf, 0xdfe0, PDF_CMAP_TABLE, 10781 },
	{ 0xdfe4, 0xdfe4, PDF_CMAP_SINGLE, 17769 },
	{ 0xdfe7, 0xdfe7, PDF_CMAP_SINGLE, 17875 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 20130 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 18875 },
	{ 0xdff1, 0xdff2, PDF_CMAP_TABLE, 10783 },
	{ 0xdff6, 0xdff7, PDF_CMAP_RANGE, 18554 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 18553 },
	{ 0xdff8, 0xdffb, PDF_CMAP_TABLE, 10785 },
	{ 0xdffe, 0xdffe, PDF_CMAP_SINGLE, 15422 },
	{ 0xdffe, 0xdfff, PDF_CMAP_TABLE, 10789 },
	{ 0xf860, 0xf862, PDF_CMAP_RANGE, 15444 },
	{ 0xf87a, 0xf87a, PDF_CMAP_SINGLE, 15447 },
	{ 0xf87f, 0xf87f, PDF_CMAP_SINGLE, 15448 },
	{ 0xf909, 0xf909, PDF_CMAP_SINGLE, 13739 },
	{ 0xf91d, 0xf91d, PDF_CMAP_SINGLE, 13392 },
	{ 0xf91f, 0xf91f, PDF_CMAP_SINGLE, 14084 },
	{ 0xf928, 0xf929, PDF_CMAP_TABLE, 10791 },
	{ 0xf936, 0xf936, PDF_CMAP_SINGLE, 13394 },
	{ 0xf95f, 0xf95f, PDF_CMAP_SINGLE, 13971 },
	{ 0xf970, 0xf970, PDF_CMAP_SINGLE, 13344 },
	{ 0xf983, 0xf983, PDF_CMAP_SINGLE, 14088 },
	{ 0xf992, 0xf993, PDF_CMAP_RANGE, 7809 },
	{ 0xf999, 0xf99a, PDF_CMAP_TABLE, 10793 },
	{ 0xf9a2, 0xf9a2, PDF_CMAP_SINGLE, 14095 },
	{ 0xf9c3, 0xf9c3, PDF_CMAP_SINGLE, 7808 },
	{ 0xf9d0, 0xf9d0, PDF_CMAP_SINGLE, 13396 },
	{ 0xf9dc, 0xf9dc, PDF_CMAP_SINGLE, 13393 },
	{ 0xf9ec, 0xf9ec, PDF_CMAP_SINGLE, 7750 },
	{ 0xfa03, 0xfa03, PDF_CMAP_SINGLE, 13956 },
	{ 0xfa0e, 0xfa17, PDF_CMAP_TABLE, 10795 },
	{ 0xfa18, 0xfa1a, PDF_CMAP_RANGE, 8579 },
	{ 0xfa1b, 0xfa1f, PDF_CMAP_TABLE, 10805 },
	{ 0xfa20, 0xfa21, PDF_CMAP_RANGE, 8612 },
	{ 0xfa22, 0xfa29, PDF_CMAP_TABLE, 10810 },
	{ 0xfa2a, 0xfa2b, PDF_CMAP_RANGE, 8699 },
	{ 0xfa2c, 0xfa2d, PDF_CMAP_TABLE, 10818 },
	{ 0xfa30, 0xfa5a, PDF_CMAP_TABLE, 10820 },
	{ 0xfa5b, 0xfa5c, PDF_CMAP_RANGE, 13349 },
	{ 0xfa5d, 0xfa6a, PDF_CMAP_TABLE, 10863 },
	{ 0xfb00, 0xfb00, PDF_CMAP_SINGLE, 9358 },
	{ 0xfb01, 0xfb02, PDF_CMAP_RANGE, 112 },
	{ 0xfb03, 0xfb04, PDF_CMAP_RANGE, 9359 },
	{ 0xfe10, 0xfe10, PDF_CMAP_SINGLE, 8268 },
	{ 0xfe11, 0xfe12, PDF_CMAP_RANGE, 7887 },
	{ 0xfe19, 0xfe19, PDF_CMAP_SINGLE, 7897 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 7898 },
	{ 0xfe31, 0xfe32, PDF_CMAP_RANGE, 7892 },
	{ 0xfe33, 0xfe33, PDF_CMAP_SINGLE, 7890 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 7899 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 7905 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 7901 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 7915 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 7909 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 7907 },
	{ 0xfe41, 0xfe44, PDF_CMAP_RANGE, 7911 },
	{ 0xfe45, 0xfe46, PDF_CMAP_RANGE, 12639 },
	{ 0xfe47, 0xfe48, PDF_CMAP_RANGE, 7903 },
	{ 0xff01, 0xff07, PDF_CMAP_TABLE, 10877 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 674 },
	{ 0xff0a, 0xff0f, PDF_CMAP_TABLE, 10884 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 780 },
	{ 0xff1a, 0xff1b, PDF_CMAP_RANGE, 639 },
	{ 0xff1c, 0xff20, PDF_CMAP_TABLE, 10890 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 790 },
	{ 0xff3b, 0xff40, PDF_CMAP_TABLE, 10895 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 816 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 10901 },
	{ 0xff5f, 0xff60, PDF_CMAP_RANGE, 12131 },
	{ 0xff61, 0xff9f, PDF_CMAP_RANGE, 327 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 713 },
	{ 0xffe2, 0xffe5, PDF_CMAP_TABLE, 10905 },
	{ 0xffe8, 0xffe8, PDF_CMAP_SINGLE, 323 },
	{ 0x69bf, 0x69bf, PDF_CMAP_SINGLE, 5257 },
	{ 0x69c0, 0x69c0, PDF_CMAP_SINGLE, 17839 },
	{ 0x69c1, 0x69c1, PDF_CMAP_SINGLE, 5258 },
	{ 0x69c3, 0x69c3, PDF_CMAP_SINGLE, 5266 },
	{ 0x69c5, 0x69c5, PDF_CMAP_SINGLE, 21789 },
	{ 0x69c7, 0x69c7, PDF_CMAP_SINGLE, 7475 },
	{ 0x69c8, 0x69c8, PDF_CMAP_SINGLE, 21790 },
	{ 0x69ca, 0x69ca, PDF_CMAP_SINGLE, 5263 },
	{ 0x69cb, 0x69cb, PDF_CMAP_SINGLE, 1999 },
	{ 0x69cc, 0x69cc, PDF_CMAP_SINGLE, 3044 },
	{ 0x69cd, 0x69cd, PDF_CMAP_SINGLE, 2790 },
	{ 0x69ce, 0x69ce, PDF_CMAP_SINGLE, 5261 },
	{ 0x69cf, 0x69cf, PDF_CMAP_SINGLE, 16909 },
	{ 0x69d0, 0x69d0, PDF_CMAP_SINGLE, 5256 },
	{ 0x69d1, 0x69d1, PDF_CMAP_SINGLE, 17840 },
	{ 0x69d3, 0x69d3, PDF_CMAP_SINGLE, 5259 },
	{ 0x69d6, 0x69d7, PDF_CMAP_RANGE, 14655 },
	{ 0x69d8, 0x69d8, PDF_CMAP_SINGLE, 3895 },
	{ 0x69d9, 0x69d9, PDF_CMAP_SINGLE, 3736 },
	{ 0x69dd, 0x69dd, PDF_CMAP_SINGLE, 5264 },
	{ 0x69de, 0x69de, PDF_CMAP_SINGLE, 5274 },
	{ 0x69e2, 0x69e2, PDF_CMAP_SINGLE, 8501 },
	{ 0x69e3, 0x69e3, PDF_CMAP_SINGLE, 17844 },
	{ 0x69e5, 0x69e5, PDF_CMAP_SINGLE, 19464 },
	{ 0x69e7, 0x69e7, PDF_CMAP_SINGLE, 5282 },
	{ 0x69e8, 0x69e8, PDF_CMAP_SINGLE, 5275 },
	{ 0x69e9, 0x69e9, PDF_CMAP_SINGLE, 16912 },
	{ 0x69ea, 0x69ea, PDF_CMAP_SINGLE, 13329 },
	{ 0x69eb, 0x69eb, PDF_CMAP_SINGLE, 5288 },
	{ 0x69ed, 0x69ed, PDF_CMAP_SINGLE, 5286 },
	{ 0x69ee, 0x69ef, PDF_CMAP_RANGE, 17845 },
	{ 0x69f1, 0x69f1, PDF_CMAP_SINGLE, 19465 },
	{ 0x69f2, 0x69f2, PDF_CMAP_SINGLE, 5281 },
	{ 0x69f3, 0x69f3, PDF_CMAP_SINGLE, 17847 },
	{ 0x69f4, 0x69f4, PDF_CMAP_SINGLE, 17849 },
	{ 0x69f5, 0x69f5, PDF_CMAP_SINGLE, 16913 },
	{ 0x69f6, 0x69f6, PDF_CMAP_SINGLE, 14143 },
	{ 0x69f9, 0x69f9, PDF_CMAP_SINGLE, 5280 },
	{ 0x69fb, 0x69fb, PDF_CMAP_SINGLE, 3052 },
	{ 0x69fd, 0x69fd, PDF_CMAP_SINGLE, 2791 },
	{ 0x69fe, 0x69fe, PDF_CMAP_SINGLE, 17850 },
	{ 0x69ff, 0x69ff, PDF_CMAP_SINGLE, 5278 },
	{ 0x6a00, 0x6a00, PDF_CMAP_SINGLE, 21792 },
	{ 0x6a01, 0x6a01, PDF_CMAP_SINGLE, 14657 },
	{ 0x6a02, 0x6a02, PDF_CMAP_SINGLE, 5276 },
	{ 0x6a03, 0x6a03, PDF_CMAP_SINGLE, 21793 },
	{ 0x6a05, 0x6a05, PDF_CMAP_SINGLE, 5283 },
	{ 0x6a0a, 0x6a0a, PDF_CMAP_SINGLE, 5289 },
	{ 0x6a0b, 0x6a0b, PDF_CMAP_SINGLE, 3465 },
	{ 0x6a0c, 0x6a0c, PDF_CMAP_SINGLE, 5295 },
	{ 0x6a0f, 0x6a0f, PDF_CMAP_SINGLE, 14658 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 17851 },
	{ 0x6a12, 0x6a12, PDF_CMAP_SINGLE, 5290 },
	{ 0x6a13, 0x6a13, PDF_CMAP_SINGLE, 5293 },
	{ 0x6a14, 0x6a14, PDF_CMAP_SINGLE, 5287 },
	{ 0x6a15, 0x6a15, PDF_CMAP_SINGLE, 14659 },
	{ 0x6a17, 0x6a17, PDF_CMAP_SINGLE, 2994 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 3498 },
	{ 0x6a1a, 0x6a1a, PDF_CMAP_SINGLE, 17852 },
	{ 0x6a1b, 0x6a1b, PDF_CMAP_SINGLE, 5277 },
	{ 0x6a1d, 0x6a1d, PDF_CMAP_SINGLE, 17853 },
	{ 0x6a1e, 0x6a1e, PDF_CMAP_SINGLE, 5285 },
	{ 0x6a1f, 0x6a1f, PDF_CMAP_SINGLE, 2472 },
	{ 0x6a20, 0x6a20, PDF_CMAP_SINGLE, 21794 },
	{ 0x6a21, 0x6a21, PDF_CMAP_SINGLE, 3803 },
	{ 0x6a22, 0x6a22, PDF_CMAP_SINGLE, 5305 },
	{ 0x6a23, 0x6a23, PDF_CMAP_SINGLE, 5292 },
	{ 0x6a24, 0x6a24, PDF_CMAP_SINGLE, 21795 },
	{ 0x6a28, 0x6a28, PDF_CMAP_SINGLE, 14660 },
	{ 0x6a29, 0x6a29, PDF_CMAP_SINGLE, 1878 },
	{ 0x6a2a, 0x6a2a, PDF_CMAP_SINGLE, 1315 },
	{ 0x6a2b, 0x6a2b, PDF_CMAP_SINGLE, 1469 },
	{ 0x6a2e, 0x6a2e, PDF_CMAP_SINGLE, 5268 },
	{ 0x6a30, 0x6a30, PDF_CMAP_SINGLE, 8502 },
	{ 0x6a32, 0x6a33, PDF_CMAP_RANGE, 17855 },
	{ 0x6a34, 0x6a34, PDF_CMAP_SINGLE, 14661 },
	{ 0x6a35, 0x6a35, PDF_CMAP_SINGLE, 2473 },
	{ 0x6a36, 0x6a36, PDF_CMAP_SINGLE, 5297 },
	{ 0x6a37, 0x6a37, PDF_CMAP_SINGLE, 21796 },
	{ 0x6a38, 0x6a38, PDF_CMAP_SINGLE, 5304 },
	{ 0x6a39, 0x6a39, PDF_CMAP_SINGLE, 2341 },
	{ 0x6a3a, 0x6a3a, PDF_CMAP_SINGLE, 1488 },
	{ 0x6a3b, 0x6a3b, PDF_CMAP_SINGLE, 16915 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 2924 },
	{ 0x6a3e, 0x6a3e, PDF_CMAP_SINGLE, 14662 },
	{ 0x6a3f, 0x6a3f, PDF_CMAP_SINGLE, 17857 },
	{ 0x6a44, 0x6a44, PDF_CMAP_SINGLE, 5294 },
	{ 0x6a45, 0x6a45, PDF_CMAP_SINGLE, 14663 },
	{ 0x6a46, 0x6a46, PDF_CMAP_SINGLE, 8504 },
	{ 0x6a47, 0x6a47, PDF_CMAP_SINGLE, 5299 },
	{ 0x6a48, 0x6a48, PDF_CMAP_SINGLE, 5303 },
	{ 0x6a49, 0x6a49, PDF_CMAP_SINGLE, 17858 },
	{ 0x6a4a, 0x6a4a, PDF_CMAP_SINGLE, 19466 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 1710 },
	{ 0x6a4e, 0x6a4e, PDF_CMAP_SINGLE, 17860 },
	{ 0x6a50, 0x6a51, PDF_CMAP_RANGE, 14664 },
	{ 0x6a52, 0x6a52, PDF_CMAP_SINGLE, 17861 },
	{ 0x6a54, 0x6a54, PDF_CMAP_SINGLE, 15412 },
	{ 0x6a55, 0x6a55, PDF_CMAP_SINGLE, 19467 },
	{ 0x6a56, 0x6a56, PDF_CMAP_SINGLE, 14666 },
	{ 0x6a58, 0x6a58, PDF_CMAP_SINGLE, 1638 },
	{ 0x6a59, 0x6a59, PDF_CMAP_SINGLE, 5301 },
	{ 0x6a5b, 0x6a5b, PDF_CMAP_SINGLE, 14667 },
	{ 0x6a5f, 0x6a5f, PDF_CMAP_SINGLE, 1595 },
	{ 0x6a61, 0x6a61, PDF_CMAP_SINGLE, 3235 },
	{ 0x6a62, 0x6a62, PDF_CMAP_SINGLE, 5300 },
	{ 0x6a64, 0x6a64, PDF_CMAP_SINGLE, 17862 },
	{ 0x6a66, 0x6a66, PDF_CMAP_SINGLE, 5302 },
	{ 0x6a67, 0x6a67, PDF_CMAP_SINGLE, 19468 },
	{ 0x6a6a, 0x6a6a, PDF_CMAP_SINGLE, 21798 },
	{ 0x6a6b, 0x6a6b, PDF_CMAP_SINGLE, 8503 },
	{ 0x6a71, 0x6a71, PDF_CMAP_SINGLE, 19469 },
	{ 0x6a72, 0x6a72, PDF_CMAP_SINGLE, 5296 },
	{ 0x6a73, 0x6a73, PDF_CMAP_SINGLE, 8505 },
	{ 0x6a78, 0x6a78, PDF_CMAP_SINGLE, 5298 },
	{ 0x6a7a, 0x6a7a, PDF_CMAP_SINGLE, 17859 },
	{ 0x6a7e, 0x6a7e, PDF_CMAP_SINGLE, 8506 },
	{ 0x6a7f, 0x6a7f, PDF_CMAP_SINGLE, 1470 },
	{ 0x6a80, 0x6a80, PDF_CMAP_SINGLE, 2951 },
	{ 0x6a81, 0x6a81, PDF_CMAP_SINGLE, 21799 },
	{ 0x6a83, 0x6a83, PDF_CMAP_SINGLE, 14668 },
	{ 0x6a84, 0x6a84, PDF_CMAP_SINGLE, 5309 },
	{ 0x6a86, 0x6a87, PDF_CMAP_RANGE, 21800 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 14669 },
	{ 0x6a8b, 0x6a8b, PDF_CMAP_SINGLE, 17864 },
	{ 0x6a8d, 0x6a8d, PDF_CMAP_SINGLE, 5307 },
	{ 0x6a8e, 0x6a8e, PDF_CMAP_SINGLE, 1949 },
	{ 0x6a90, 0x6a90, PDF_CMAP_SINGLE, 5306 },
	{ 0x6a91, 0x6a91, PDF_CMAP_SINGLE, 14670 },
	{ 0x6a94, 0x6a94, PDF_CMAP_SINGLE, 16917 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 5312 },
	{ 0x6a9b, 0x6a9b, PDF_CMAP_SINGLE, 21802 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 5179 },
	{ 0x6a9d, 0x6a9f, PDF_CMAP_RANGE, 14671 },
	{ 0x6aa0, 0x6aa0, PDF_CMAP_SINGLE, 5308 },
	{ 0x6aa1, 0x6aa1, PDF_CMAP_SINGLE, 17866 },
	{ 0x6aa2, 0x6aa3, PDF_CMAP_RANGE, 5310 },
	{ 0x6aa5, 0x6aa5, PDF_CMAP_SINGLE, 16918 },
	{ 0x6aaa, 0x6aaa, PDF_CMAP_SINGLE, 5323 },
	{ 0x6aab, 0x6aab, PDF_CMAP_SINGLE, 17868 },
	{ 0x6aac, 0x6aac, PDF_CMAP_SINGLE, 5319 },
	{ 0x6aae, 0x6aae, PDF_CMAP_SINGLE, 5200 },
	{ 0x6aaf, 0x6aaf, PDF_CMAP_SINGLE, 19470 },
	{ 0x6ab0, 0x6ab1, PDF_CMAP_RANGE, 21803 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 5318 },
	{ 0x6ab4, 0x6ab4, PDF_CMAP_SINGLE, 21805 },
	{ 0x6ab8, 0x6ab8, PDF_CMAP_SINGLE, 5317 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 5314 },
	{ 0x6abd, 0x6abd, PDF_CMAP_SINGLE, 17869 },
	{ 0x6abe, 0x6abf, PDF_CMAP_RANGE, 21806 },
	{ 0x6ac1, 0x6ac1, PDF_CMAP_SINGLE, 5291 },
	{ 0x6ac2, 0x6ac2, PDF_CMAP_SINGLE, 5316 },
	{ 0x6ac3, 0x6ac3, PDF_CMAP_SINGLE, 5315 },
	{ 0x6ac6, 0x6ac6, PDF_CMAP_SINGLE, 17870 },
	{ 0x6ac8, 0x6ac9, PDF_CMAP_RANGE, 19471 },
	{ 0x6acc, 0x6acc, PDF_CMAP_SINGLE, 21808 },
	{ 0x6ad0, 0x6ad0, PDF_CMAP_SINGLE, 17872 },
	{ 0x6ad1, 0x6ad1, PDF_CMAP_SINGLE, 5321 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 4044 },
	{ 0x6ad4, 0x6ad4, PDF_CMAP_SINGLE, 17871 },
	{ 0x6ad5, 0x6ad6, PDF_CMAP_RANGE, 21809 },
	{ 0x6ada, 0x6ada, PDF_CMAP_SINGLE, 5324 },
	{ 0x6adb, 0x6adb, PDF_CMAP_SINGLE, 1779 },
	{ 0x6adc, 0x6adc, PDF_CMAP_SINGLE, 14674 },
	{ 0x6add, 0x6add, PDF_CMAP_SINGLE, 17873 },
	{ 0x6ade, 0x6ade, PDF_CMAP_SINGLE, 5320 },
	{ 0x6adf, 0x6adf, PDF_CMAP_SINGLE, 5322 },
	{ 0x6ae2, 0x6ae2, PDF_CMAP_SINGLE, 8507 },
	{ 0x6ae4, 0x6ae4, PDF_CMAP_SINGLE, 8508 },
	{ 0x6ae7, 0x6ae7, PDF_CMAP_SINGLE, 14675 },
	{ 0x6ae8, 0x6ae8, PDF_CMAP_SINGLE, 3387 },
	{ 0x6aea, 0x6aea, PDF_CMAP_SINGLE, 5325 },
	{ 0x6aec, 0x6aec, PDF_CMAP_SINGLE, 14676 },
	{ 0x6af0, 0x6af0, PDF_CMAP_SINGLE, 21811 },
	{ 0x6af1, 0x6af3, PDF_CMAP_RANGE, 17876 },
	{ 0x6af8, 0x6af8, PDF_CMAP_SINGLE, 20152 },
	{ 0x6afa, 0x6afa, PDF_CMAP_SINGLE, 5329 },
	{ 0x6afb, 0x6afb, PDF_CMAP_SINGLE, 5326 },
	{ 0x6afc, 0x6afc, PDF_CMAP_SINGLE, 21812 },
	{ 0x6afd, 0x6afd, PDF_CMAP_SINGLE, 17879 },
	{ 0x6b02, 0x6b02, PDF_CMAP_SINGLE, 21813 },
	{ 0x6b03, 0x6b03, PDF_CMAP_SINGLE, 19473 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 3933 },
	{ 0x6b05, 0x6b05, PDF_CMAP_SINGLE, 5327 },
	{ 0x6b06, 0x6b07, PDF_CMAP_RANGE, 21814 },
	{ 0x6b09, 0x6b09, PDF_CMAP_SINGLE, 21816 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 5279 },
	{ 0x6b0b, 0x6b0b, PDF_CMAP_SINGLE, 17881 },
	{ 0x6b0f, 0x6b11, PDF_CMAP_RANGE, 17882 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 5330 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 5331 },
	{ 0x6b17, 0x6b17, PDF_CMAP_SINGLE, 17886 },
	{ 0x6b1b, 0x6b1b, PDF_CMAP_SINGLE, 16920 },
	{ 0x6b1d, 0x6b1d, PDF_CMAP_SINGLE, 1239 },
	{ 0x6b1e, 0x6b1e, PDF_CMAP_SINGLE, 14677 },
	{ 0x6b1f, 0x6b1f, PDF_CMAP_SINGLE, 5333 },
	{ 0x6b20, 0x6b20, PDF_CMAP_SINGLE, 1853 },
	{ 0x6b21, 0x6b21, PDF_CMAP_SINGLE, 2253 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 1741 },
	{ 0x6b24, 0x6b24, PDF_CMAP_SINGLE, 14678 },
	{ 0x6b27, 0x6b27, PDF_CMAP_SINGLE, 1316 },
	{ 0x6b28, 0x6b28, PDF_CMAP_SINGLE, 21817 },
	{ 0x6b2b, 0x6b2b, PDF_CMAP_SINGLE, 21818 },
	{ 0x6b2c, 0x6b2c, PDF_CMAP_SINGLE, 16921 },
	{ 0x6b2f, 0x6b2f, PDF_CMAP_SINGLE, 17888 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 3913 },
	{ 0x6b35, 0x6b35, PDF_CMAP_SINGLE, 14679 },
	{ 0x6b36, 0x6b36, PDF_CMAP_SINGLE, 21819 },
	{ 0x6b37, 0x6b37, PDF_CMAP_SINGLE, 5335 },
	{ 0x6b38, 0x6b38, PDF_CMAP_SINGLE, 5334 },
	{ 0x6b39, 0x6b39, PDF_CMAP_SINGLE, 5337 },
	{ 0x6b3a, 0x6b3a, PDF_CMAP_SINGLE, 1623 },
	{ 0x6b3b, 0x6b3b, PDF_CMAP_SINGLE, 19474 },
	{ 0x6b3d, 0x6b3d, PDF_CMAP_SINGLE, 1742 },
	{ 0x6b3e, 0x6b3e, PDF_CMAP_SINGLE, 1530 },
	{ 0x6b3f, 0x6b3f, PDF_CMAP_SINGLE, 19475 },
	{ 0x6b43, 0x6b43, PDF_CMAP_SINGLE, 5340 },
	{ 0x6b46, 0x6b46, PDF_CMAP_SINGLE, 14680 },
	{ 0x6b47, 0x6b47, PDF_CMAP_SINGLE, 5339 },
	{ 0x6b49, 0x6b49, PDF_CMAP_SINGLE, 5341 },
	{ 0x6b4a, 0x6b4a, PDF_CMAP_SINGLE, 17889 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 1358 },
	{ 0x6b4d, 0x6b4d, PDF_CMAP_SINGLE, 21820 },
	{ 0x6b4e, 0x6b4e, PDF_CMAP_SINGLE, 2933 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 5342 },
	{ 0x6b52, 0x6b52, PDF_CMAP_SINGLE, 21821 },
	{ 0x6b53, 0x6b53, PDF_CMAP_SINGLE, 1531 },
	{ 0x6b54, 0x6b54, PDF_CMAP_SINGLE, 5344 },
	{ 0x6b56, 0x6b56, PDF_CMAP_SINGLE, 14681 },
	{ 0x6b58, 0x6b58, PDF_CMAP_SINGLE, 17890 },
	{ 0x6b59, 0x6b59, PDF_CMAP_SINGLE, 5343 },
	{ 0x6b5b, 0x6b5b, PDF_CMAP_SINGLE, 5345 },
	{ 0x6b5d, 0x6b5d, PDF_CMAP_SINGLE, 21822 },
	{ 0x6b5f, 0x6b5f, PDF_CMAP_SINGLE, 5346 },
	{ 0x6b60, 0x6b60, PDF_CMAP_SINGLE, 14682 },
	{ 0x6b61, 0x6b61, PDF_CMAP_SINGLE, 5347 },
	{ 0x6b62, 0x6b62, PDF_CMAP_SINGLE, 2221 },
	{ 0x6b63, 0x6b63, PDF_CMAP_SINGLE, 2649 },
	{ 0x6b64, 0x6b64, PDF_CMAP_SINGLE, 2065 },
	{ 0x6b65, 0x6b65, PDF_CMAP_SINGLE, 13386 },
	{ 0x6b66, 0x6b66, PDF_CMAP_SINGLE, 3554 },
	{ 0x6b67, 0x6b67, PDF_CMAP_SINGLE, 16922 },
	{ 0x6b69, 0x6b69, PDF_CMAP_SINGLE, 3634 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 4074 },
	{ 0x6b6b, 0x6b6b, PDF_CMAP_SINGLE, 21823 },
	{ 0x6b6c, 0x6b6c, PDF_CMAP_SINGLE, 17891 },
	{ 0x6b6e, 0x6b6e, PDF_CMAP_SINGLE, 21824 },
	{ 0x6b6f, 0x6b6f, PDF_CMAP_SINGLE, 2243 },
	{ 0x6b70, 0x6b70, PDF_CMAP_SINGLE, 21825 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 13785 },
	{ 0x6b73, 0x6b73, PDF_CMAP_SINGLE, 2112 },
	{ 0x6b74, 0x6b74, PDF_CMAP_SINGLE, 4026 },
	{ 0x6b75, 0x6b75, PDF_CMAP_SINGLE, 17892 },
	{ 0x6b77, 0x6b77, PDF_CMAP_SINGLE, 13398 },
	{ 0x6b78, 0x6b79, PDF_CMAP_RANGE, 5348 },
	{ 0x6b7a, 0x6b7a, PDF_CMAP_SINGLE, 17893 },
	{ 0x6b7b, 0x6b7b, PDF_CMAP_SINGLE, 2222 },
	{ 0x6b7d, 0x6b7e, PDF_CMAP_RANGE, 19476 },
	{ 0x6b7f, 0x6b80, PDF_CMAP_RANGE, 5350 },
	{ 0x6b81, 0x6b81, PDF_CMAP_SINGLE, 17894 },
	{ 0x6b82, 0x6b82, PDF_CMAP_SINGLE, 14683 },
	{ 0x6b83, 0x6b83, PDF_CMAP_SINGLE, 5353 },
	{ 0x6b84, 0x6b84, PDF_CMAP_SINGLE, 5352 },
	{ 0x6b85, 0x6b85, PDF_CMAP_SINGLE, 21826 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 3718 },
	{ 0x6b89, 0x6b89, PDF_CMAP_SINGLE, 2408 },
	{ 0x6b8a, 0x6b8a, PDF_CMAP_SINGLE, 2328 },
	{ 0x6b8b, 0x6b8b, PDF_CMAP_SINGLE, 2194 },
	{ 0x6b8d, 0x6b8d, PDF_CMAP_SINGLE, 5354 },
	{ 0x6b95, 0x6b95, PDF_CMAP_SINGLE, 5356 },
	{ 0x6b96, 0x6b96, PDF_CMAP_SINGLE, 2537 },
	{ 0x6b97, 0x6b97, PDF_CMAP_SINGLE, 21827 },
	{ 0x6b98, 0x6b98, PDF_CMAP_SINGLE, 5355 },
	{ 0x6b9b, 0x6b9b, PDF_CMAP_SINGLE, 17895 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 5357 },
	{ 0x6b9f, 0x6ba0, PDF_CMAP_RANGE, 21828 },
	{ 0x6ba2, 0x6ba3, PDF_CMAP_RANGE, 21830 },
	{ 0x6ba4, 0x6ba4, PDF_CMAP_SINGLE, 5358 },
	{ 0x6ba8, 0x6ba8, PDF_CMAP_SINGLE, 21832 },
	{ 0x6ba9, 0x6ba9, PDF_CMAP_SINGLE, 16923 },
	{ 0x6baa, 0x6bab, PDF_CMAP_RANGE, 5359 },
	{ 0x6bac, 0x6bac, PDF_CMAP_SINGLE, 21833 },
	{ 0x6bad, 0x6bad, PDF_CMAP_SINGLE, 16924 },
	{ 0x6bae, 0x6bae, PDF_CMAP_SINGLE, 17896 },
	{ 0x6baf, 0x6baf, PDF_CMAP_SINGLE, 5361 },
	{ 0x6bb0, 0x6bb0, PDF_CMAP_SINGLE, 19478 },
	{ 0x6bb1, 0x6bb1, PDF_CMAP_SINGLE, 5363 },
	{ 0x6bb2, 0x6bb2, PDF_CMAP_SINGLE, 5362 },
	{ 0x6bb3, 0x6bb3, PDF_CMAP_SINGLE, 5364 },
	{ 0x6bb4, 0x6bb4, PDF_CMAP_SINGLE, 1317 },
	{ 0x6bb5, 0x6bb5, PDF_CMAP_SINGLE, 2952 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 5365 },
	{ 0x6bb8, 0x6bb9, PDF_CMAP_RANGE, 21834 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 2164 },
	{ 0x6bbb, 0x6bbb, PDF_CMAP_SINGLE, 1450 },
	{ 0x6bbc, 0x6bbc, PDF_CMAP_SINGLE, 5366 },
	{ 0x6bbd, 0x6bbd, PDF_CMAP_SINGLE, 17898 },
	{ 0x6bbe, 0x6bbe, PDF_CMAP_SINGLE, 14684 },
	{ 0x6bbf, 0x6bbf, PDF_CMAP_SINGLE, 3132 },
	{ 0x6bc0, 0x6bc0, PDF_CMAP_SINGLE, 4509 },
	{ 0x6bc3, 0x6bc4, PDF_CMAP_RANGE, 21836 },
	{ 0x6bc5, 0x6bc5, PDF_CMAP_SINGLE, 1597 },
	{ 0x6bc6, 0x6bc6, PDF_CMAP_SINGLE, 5367 },
	{ 0x6bc7, 0x6bc9, PDF_CMAP_RANGE, 17899 },
	{ 0x6bcb, 0x6bcb, PDF_CMAP_SINGLE, 5368 },
	{ 0x6bcc, 0x6bcc, PDF_CMAP_SINGLE, 14148 },
	{ 0x6bcd, 0x6bcd, PDF_CMAP_SINGLE, 3644 },
	{ 0x6bce, 0x6bce, PDF_CMAP_SINGLE, 3734 },
	{ 0x6bcf, 0x6bcf, PDF_CMAP_SINGLE, 13388 },
	{ 0x6bd2, 0x6bd2, PDF_CMAP_SINGLE, 3231 },
	{ 0x6bd3, 0x6bd3, PDF_CMAP_SINGLE, 5369 },
	{ 0x6bd4, 0x6bd4, PDF_CMAP_SINGLE, 3450 },
	{ 0x6bd6, 0x6bd6, PDF_CMAP_SINGLE, 8509 },
	{ 0x6bd7, 0x6bd7, PDF_CMAP_SINGLE, 16925 },
	{ 0x6bd8, 0x6bd8, PDF_CMAP_SINGLE, 3471 },
	{ 0x6bda, 0x6bda, PDF_CMAP_SINGLE, 17902 },
	{ 0x6bdb, 0x6bdb, PDF_CMAP_SINGLE, 3807 },
	{ 0x6bdf, 0x6bdf, PDF_CMAP_SINGLE, 5370 },
	{ 0x6be1, 0x6be1, PDF_CMAP_SINGLE, 14685 },
	{ 0x6be3, 0x6be3, PDF_CMAP_SINGLE, 21838 },
	{ 0x6be6, 0x6be7, PDF_CMAP_RANGE, 17903 },
	{ 0x6beb, 0x6beb, PDF_CMAP_SINGLE, 5372 },
	{ 0x6bec, 0x6bec, PDF_CMAP_SINGLE, 5371 },
	{ 0x6bee, 0x6bee, PDF_CMAP_SINGLE, 17905 },
	{ 0x6bef, 0x6bef, PDF_CMAP_SINGLE, 5374 },
	{ 0x6bf1, 0x6bf1, PDF_CMAP_SINGLE, 14686 },
	{ 0x6bf3, 0x6bf3, PDF_CMAP_SINGLE, 5373 },
	{ 0x6bf7, 0x6bf7, PDF_CMAP_SINGLE, 19479 },
	{ 0x6bf9, 0x6bf9, PDF_CMAP_SINGLE, 19480 },
	{ 0x6bff, 0x6bff, PDF_CMAP_SINGLE, 16926 },
	{ 0x6c02, 0x6c02, PDF_CMAP_SINGLE, 17906 },
	{ 0x6c04, 0x6c04, PDF_CMAP_SINGLE, 19481 },
	{ 0x6c05, 0x6c05, PDF_CMAP_SINGLE, 16927 },
	{ 0x6c08, 0x6c08, PDF_CMAP_SINGLE, 5376 },
	{ 0x6c09, 0x6c09, PDF_CMAP_SINGLE, 19482 },
	{ 0x6c0a, 0x6c0a, PDF_CMAP_SINGLE, 17907 },
	{ 0x6c0d, 0x6c0d, PDF_CMAP_SINGLE, 19483 },
	{ 0x6c0e, 0x6c0e, PDF_CMAP_SINGLE, 17908 },
	{ 0x6c0f, 0x6c0f, PDF_CMAP_SINGLE, 2223 },
	{ 0x6c10, 0x6c10, PDF_CMAP_SINGLE, 14687 },
	{ 0x6c11, 0x6c11, PDF_CMAP_SINGLE, 3773 },
	{ 0x6c12, 0x6c12, PDF_CMAP_SINGLE, 21839 },
	{ 0x6c13, 0x6c14, PDF_CMAP_RANGE, 5377 },
	{ 0x6c17, 0x6c17, PDF_CMAP_SINGLE, 1598 },
	{ 0x6c19, 0x6c19, PDF_CMAP_SINGLE, 21840 },
	{ 0x6c1b, 0x6c1b, PDF_CMAP_SINGLE, 5379 },
	{ 0x6c1f, 0x6c1f, PDF_CMAP_SINGLE, 21841 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 5381 },
	{ 0x6c24, 0x6c24, PDF_CMAP_SINGLE, 5380 },
	{ 0x6c26, 0x6c28, PDF_CMAP_RANGE, 21842 },
	{ 0x6c2c, 0x6c2c, PDF_CMAP_SINGLE, 19484 },
	{ 0x6c2e, 0x6c2e, PDF_CMAP_SINGLE, 21845 },
	{ 0x6c33, 0x6c33, PDF_CMAP_SINGLE, 14688 },
	{ 0x6c34, 0x6c34, PDF_CMAP_SINGLE, 2603 },
	{ 0x6c35, 0x6c35, PDF_CMAP_SINGLE, 14689 },
	{ 0x6c36, 0x6c36, PDF_CMAP_SINGLE, 17909 },
	{ 0x6c37, 0x6c37, PDF_CMAP_SINGLE, 3499 },
	{ 0x6c38, 0x6c38, PDF_CMAP_SINGLE, 1260 },
	{ 0x6c3a, 0x6c3a, PDF_CMAP_SINGLE, 20309 },
	{ 0x6c3b, 0x6c3b, PDF_CMAP_SINGLE, 21846 },
	{ 0x6c3e, 0x6c3e, PDF_CMAP_SINGLE, 3417 },
	{ 0x6c3f, 0x6c3f, PDF_CMAP_SINGLE, 8510 },
	{ 0x6c40, 0x6c40, PDF_CMAP_SINGLE, 3089 },
	{ 0x6c41, 0x6c41, PDF_CMAP_SINGLE, 2379 },
	{ 0x6c42, 0x6c42, PDF_CMAP_SINGLE, 1659 },
	{ 0x6c4a, 0x6c4a, PDF_CMAP_SINGLE, 19485 },
	{ 0x6c4b, 0x6c4b, PDF_CMAP_SINGLE, 21847 },
	{ 0x6c4d, 0x6c4d, PDF_CMAP_SINGLE, 17911 },
	{ 0x6c4e, 0x6c4e, PDF_CMAP_SINGLE, 3418 },
	{ 0x6c4f, 0x6c4f, PDF_CMAP_SINGLE, 21848 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 2266 },
	{ 0x6c52, 0x6c52, PDF_CMAP_SINGLE, 19486 },
	{ 0x6c54, 0x6c54, PDF_CMAP_SINGLE, 19487 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 5383 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 1532 },
	{ 0x6c59, 0x6c59, PDF_CMAP_SINGLE, 14691 },
	{ 0x6c5a, 0x6c5a, PDF_CMAP_SINGLE, 1306 },
	{ 0x6c5b, 0x6c5b, PDF_CMAP_SINGLE, 17912 },
	{ 0x6c5c, 0x6c5c, PDF_CMAP_SINGLE, 8511 },
	{ 0x6c5d, 0x6c5d, PDF_CMAP_SINGLE, 3274 },
	{ 0x6c5e, 0x6c5e, PDF_CMAP_SINGLE, 5382 },
	{ 0x6c5f, 0x6c5f, PDF_CMAP_SINGLE, 2000 },
	{ 0x6c60, 0x6c60, PDF_CMAP_SINGLE, 2961 },
	{ 0x6c62, 0x6c62, PDF_CMAP_SINGLE, 5384 },
	{ 0x6c67, 0x6c67, PDF_CMAP_SINGLE, 17930 },
	{ 0x6c68, 0x6c68, PDF_CMAP_SINGLE, 5392 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 5385 },
	{ 0x6c6b, 0x6c6b, PDF_CMAP_SINGLE, 21849 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 17913 },
	{ 0x6c6f, 0x6c6f, PDF_CMAP_SINGLE, 8513 },
	{ 0x6c70, 0x6c70, PDF_CMAP_SINGLE, 2849 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 1660 },
	{ 0x6c73, 0x6c73, PDF_CMAP_SINGLE, 5393 },
	{ 0x6c74, 0x6c74, PDF_CMAP_SINGLE, 16928 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 14692 },
	{ 0x6c78, 0x6c78, PDF_CMAP_SINGLE, 21850 },
	{ 0x6c79, 0x6c79, PDF_CMAP_SINGLE, 19488 },
	{ 0x6c7a, 0x6c7a, PDF_CMAP_SINGLE, 1854 },
	{ 0x6c7b, 0x6c7b, PDF_CMAP_SINGLE, 14693 },
	{ 0x6c7d, 0x6c7d, PDF_CMAP_SINGLE, 1599 },
	{ 0x6c7e, 0x6c7e, PDF_CMAP_SINGLE, 5391 },
	{ 0x6c81, 0x6c81, PDF_CMAP_SINGLE, 5389 },
	{ 0x6c82, 0x6c82, PDF_CMAP_SINGLE, 5386 },
	{ 0x6c83, 0x6c83, PDF_CMAP_SINGLE, 3914 },
	{ 0x6c84, 0x6c84, PDF_CMAP_SINGLE, 17914 },
	{ 0x6c85, 0x6c85, PDF_CMAP_SINGLE, 14694 },
	{ 0x6c86, 0x6c86, PDF_CMAP_SINGLE, 8512 },
	{ 0x6c87, 0x6c87, PDF_CMAP_SINGLE, 21851 },
	{ 0x6c88, 0x6c88, PDF_CMAP_SINGLE, 3036 },
	{ 0x6c89, 0x6c89, PDF_CMAP_SINGLE, 17915 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 3249 },
	{ 0x6c8d, 0x6c8d, PDF_CMAP_SINGLE, 5387 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 5395 },
	{ 0x6c92, 0x6c92, PDF_CMAP_SINGLE, 5394 },
	{ 0x6c93, 0x6c93, PDF_CMAP_SINGLE, 1785 },
	{ 0x6c94, 0x6c94, PDF_CMAP_SINGLE, 17917 },
	{ 0x6c95, 0x6c95, PDF_CMAP_SINGLE, 14695 },
	{ 0x6c96, 0x6c96, PDF_CMAP_SINGLE, 1325 },
	{ 0x6c97, 0x6c97, PDF_CMAP_SINGLE, 17918 },
	{ 0x6c98, 0x6c98, PDF_CMAP_SINGLE, 16929 },
	{ 0x6c99, 0x6c99, PDF_CMAP_SINGLE, 2091 },
	{ 0x6c9a, 0x6c9a, PDF_CMAP_SINGLE, 5388 },
	{ 0x6c9b, 0x6c9b, PDF_CMAP_SINGLE, 5390 },
	{ 0x6c9c, 0x6c9c, PDF_CMAP_SINGLE, 14696 },
	{ 0x6c9f, 0x6c9f, PDF_CMAP_SINGLE, 21852 },
	{ 0x6ca1, 0x6ca1, PDF_CMAP_SINGLE, 3717 },
	{ 0x6ca2, 0x6ca2, PDF_CMAP_SINGLE, 2900 },
	{ 0x6caa, 0x6caa, PDF_CMAP_SINGLE, 14153 },
	{ 0x6cab, 0x6cab, PDF_CMAP_SINGLE, 3749 },
	{ 0x6cac, 0x6cac, PDF_CMAP_SINGLE, 19489 },
	{ 0x6cad, 0x6cad, PDF_CMAP_SINGLE, 17919 },
	{ 0x6cae, 0x6cae, PDF_CMAP_SINGLE, 5403 },
	{ 0x6cb0, 0x6cb0, PDF_CMAP_SINGLE, 21853 },
	{ 0x6cb1, 0x6cb1, PDF_CMAP_SINGLE, 5404 },
	{ 0x6cb2, 0x6cb2, PDF_CMAP_SINGLE, 21854 },
	{ 0x6cb3, 0x6cb3, PDF_CMAP_SINGLE, 1359 },
	{ 0x6cb4, 0x6cb4, PDF_CMAP_SINGLE, 19490 },
	{ 0x6cb8, 0x6cb8, PDF_CMAP_SINGLE, 3576 },
	{ 0x6cb9, 0x6cb9, PDF_CMAP_SINGLE, 3849 },
	{ 0x6cba, 0x6cba, PDF_CMAP_SINGLE, 5406 },
	{ 0x6cbb, 0x6cbb, PDF_CMAP_SINGLE, 2255 },
	{ 0x6cbc, 0x6cbc, PDF_CMAP_SINGLE, 2474 },
	{ 0x6cbd, 0x6cbd, PDF_CMAP_SINGLE, 5399 },
	{ 0x6cbe, 0x6cbe, PDF_CMAP_SINGLE, 5405 },
	{ 0x6cbf, 0x6cbf, PDF_CMAP_SINGLE, 1290 },
	{ 0x6cc1, 0x6cc1, PDF_CMAP_SINGLE, 1711 },
	{ 0x6cc2, 0x6cc2, PDF_CMAP_SINGLE, 17920 },
	{ 0x6cc4, 0x6cc4, PDF_CMAP_SINGLE, 5396 },
	{ 0x6cc5, 0x6cc5, PDF_CMAP_SINGLE, 5401 },
	{ 0x6cc6, 0x6cc6, PDF_CMAP_SINGLE, 16931 },
	{ 0x6cc9, 0x6cc9, PDF_CMAP_SINGLE, 2712 },
	{ 0x6cca, 0x6cca, PDF_CMAP_SINGLE, 3367 },
	{ 0x6ccc, 0x6ccc, PDF_CMAP_SINGLE, 3451 },
	{ 0x6ccd, 0x6ccd, PDF_CMAP_SINGLE, 21855 },
	{ 0x6ccf, 0x6ccf, PDF_CMAP_SINGLE, 21856 },
	{ 0x6cd0, 0x6cd0, PDF_CMAP_SINGLE, 14697 },
	{ 0x6cd1, 0x6cd1, PDF_CMAP_SINGLE, 21857 },
	{ 0x6cd2, 0x6cd2, PDF_CMAP_SINGLE, 19491 },
	{ 0x6cd3, 0x6cd3, PDF_CMAP_SINGLE, 5398 },
	{ 0x6cd4, 0x6cd4, PDF_CMAP_SINGLE, 14698 },
	{ 0x6cd5, 0x6cd5, PDF_CMAP_SINGLE, 3663 },
	{ 0x6cd6, 0x6cd6, PDF_CMAP_SINGLE, 14699 },
	{ 0x6cd7, 0x6cd7, PDF_CMAP_SINGLE, 5400 },
	{ 0x6cd9, 0x6cd9, PDF_CMAP_SINGLE, 5409 },
	{ 0x6cda, 0x6cda, PDF_CMAP_SINGLE, 8514 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 5407 },
	{ 0x6cdc, 0x6cdc, PDF_CMAP_SINGLE, 17922 },
	{ 0x6cdd, 0x6cdd, PDF_CMAP_SINGLE, 5402 },
	{ 0x6ce0, 0x6ce0, PDF_CMAP_SINGLE, 14700 },
	{ 0x6ce1, 0x6ce1, PDF_CMAP_SINGLE, 3664 },
	{ 0x6ce2, 0x6ce2, PDF_CMAP_SINGLE, 3326 },
	{ 0x6ce3, 0x6ce3, PDF_CMAP_SINGLE, 1661 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 3103 },
	{ 0x6ce7, 0x6ce7, PDF_CMAP_SINGLE, 21858 },
	{ 0x6ce8, 0x6ce8, PDF_CMAP_SINGLE, 2987 },
	{ 0x6ce9, 0x6ce9, PDF_CMAP_SINGLE, 17923 },
	{ 0x6cea, 0x6cea, PDF_CMAP_SINGLE, 5410 },
	{ 0x6ceb, 0x6cec, PDF_CMAP_RANGE, 14701 },
	{ 0x6ced, 0x6ced, PDF_CMAP_SINGLE, 17924 },
	{ 0x6cee, 0x6cee, PDF_CMAP_SINGLE, 14703 },
	{ 0x6cef, 0x6cef, PDF_CMAP_SINGLE, 5408 },
	{ 0x6cf0, 0x6cf0, PDF_CMAP_SINGLE, 2873 },
	{ 0x6cf1, 0x6cf1, PDF_CMAP_SINGLE, 5397 },
	{ 0x6cf2, 0x6cf2, PDF_CMAP_SINGLE, 21859 },
	{ 0x6cf3, 0x6cf3, PDF_CMAP_SINGLE, 1261 },
	{ 0x6cf4, 0x6cf4, PDF_CMAP_SINGLE, 21860 },
	{ 0x6cfb, 0x6cfb, PDF_CMAP_SINGLE, 16930 },
	{ 0x6d00, 0x6d00, PDF_CMAP_SINGLE, 17926 },
	{ 0x6d01, 0x6d01, PDF_CMAP_SINGLE, 15413 },
	{ 0x6d04, 0x6d04, PDF_CMAP_SINGLE, 8515 },
	{ 0x6d07, 0x6d07, PDF_CMAP_SINGLE, 21861 },
	{ 0x6d0a, 0x6d0a, PDF_CMAP_SINGLE, 14704 },
	{ 0x6d0b, 0x6d0b, PDF_CMAP_SINGLE, 3896 },
	{ 0x6d0c, 0x6d0c, PDF_CMAP_SINGLE, 5421 },
	{ 0x6d0e, 0x6d0e, PDF_CMAP_SINGLE, 14705 },
	{ 0x6d0f, 0x6d0f, PDF_CMAP_SINGLE, 21862 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 14706 },
	{ 0x6d12, 0x6d12, PDF_CMAP_SINGLE, 5420 },
	{ 0x6d13, 0x6d13, PDF_CMAP_SINGLE, 21863 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 2714 },
	{ 0x6d19, 0x6d19, PDF_CMAP_SINGLE, 5417 },
	{ 0x6d1a, 0x6d1a, PDF_CMAP_SINGLE, 21864 },
	{ 0x6d1b, 0x6d1b, PDF_CMAP_SINGLE, 3926 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 3214 },
	{ 0x6d1f, 0x6d1f, PDF_CMAP_SINGLE, 5411 },
	{ 0x6d24, 0x6d24, PDF_CMAP_SINGLE, 17927 },
	{ 0x6d25, 0x6d25, PDF_CMAP_SINGLE, 3041 },
	{ 0x6d26, 0x6d27, PDF_CMAP_RANGE, 17928 },
	{ 0x6d28, 0x6d28, PDF_CMAP_SINGLE, 21865 },
	{ 0x6d29, 0x6d29, PDF_CMAP_SINGLE, 1262 },
	{ 0x6d2a, 0x6d2a, PDF_CMAP_SINGLE, 2001 },
	{ 0x6d2b, 0x6d2b, PDF_CMAP_SINGLE, 5414 },
	{ 0x6d2e, 0x6d2e, PDF_CMAP_SINGLE, 14707 },
	{ 0x6d2f, 0x6d2f, PDF_CMAP_SINGLE, 17931 },
	{ 0x6d31, 0x6d31, PDF_CMAP_SINGLE, 16932 },
	{ 0x6d32, 0x6d32, PDF_CMAP_SINGLE, 2353 },
	{ 0x6d33, 0x6d33, PDF_CMAP_SINGLE, 5419 },
	{ 0x6d34, 0x6d34, PDF_CMAP_SINGLE, 17950 },
	{ 0x6d35, 0x6d35, PDF_CMAP_SINGLE, 5418 },
	{ 0x6d36, 0x6d36, PDF_CMAP_SINGLE, 5413 },
	{ 0x6d38, 0x6d38, PDF_CMAP_SINGLE, 5416 },
	{ 0x6d39, 0x6d39, PDF_CMAP_SINGLE, 16933 },
	{ 0x6d3b, 0x6d3b, PDF_CMAP_SINGLE, 1478 },
	{ 0x6d3c, 0x6d3c, PDF_CMAP_SINGLE, 17932 },
	{ 0x6d3d, 0x6d3d, PDF_CMAP_SINGLE, 5415 },
	{ 0x6d3e, 0x6d3e, PDF_CMAP_SINGLE, 3327 },
	{ 0x6d3f, 0x6d3f, PDF_CMAP_SINGLE, 16934 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 3958 },
	{ 0x6d44, 0x6d44, PDF_CMAP_SINGLE, 2524 },
	{ 0x6d45, 0x6d45, PDF_CMAP_SINGLE, 2713 },
	{ 0x6d57, 0x6d57, PDF_CMAP_SINGLE, 14708 },
	{ 0x6d58, 0x6d58, PDF_CMAP_SINGLE, 16935 },
	{ 0x6d59, 0x6d59, PDF_CMAP_SINGLE, 5427 },
	{ 0x6d5a, 0x6d5a, PDF_CMAP_SINGLE, 5425 },
	{ 0x6d5b, 0x6d5b, PDF_CMAP_SINGLE, 17933 },
	{ 0x6d5c, 0x6d5c, PDF_CMAP_SINGLE, 3519 },
	{ 0x6d5e, 0x6d5e, PDF_CMAP_SINGLE, 14709 },
	{ 0x6d5f, 0x6d5f, PDF_CMAP_SINGLE, 21866 },
	{ 0x6d60, 0x6d60, PDF_CMAP_SINGLE, 17934 },
	{ 0x6d61, 0x6d61, PDF_CMAP_SINGLE, 19492 },
	{ 0x6d63, 0x6d63, PDF_CMAP_SINGLE, 5422 },
	{ 0x6d64, 0x6d64, PDF_CMAP_SINGLE, 5424 },
	{ 0x6d65, 0x6d65, PDF_CMAP_SINGLE, 14710 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 1244 },
	{ 0x6d67, 0x6d67, PDF_CMAP_SINGLE, 21867 },
	{ 0x6d69, 0x6d69, PDF_CMAP_SINGLE, 2002 },
	{ 0x6d6a, 0x6d6a, PDF_CMAP_SINGLE, 4056 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 1435 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 3540 },
	{ 0x6d6f, 0x6d6f, PDF_CMAP_SINGLE, 8517 },
	{ 0x6d70, 0x6d70, PDF_CMAP_SINGLE, 17935 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 3915 },
	{ 0x6d77, 0x6d77, PDF_CMAP_SINGLE, 1410 },
	{ 0x6d78, 0x6d78, PDF_CMAP_SINGLE, 2561 },
	{ 0x6d79, 0x6d79, PDF_CMAP_SINGLE, 5426 },
	{ 0x6d7c, 0x6d7c, PDF_CMAP_SINGLE, 19493 },
	{ 0x6d80, 0x6d81, PDF_CMAP_RANGE, 17936 },
	{ 0x6d82, 0x6d82, PDF_CMAP_SINGLE, 14711 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 5431 },
	{ 0x6d87, 0x6d87, PDF_CMAP_SINGLE, 8516 },
	{ 0x6d88, 0x6d88, PDF_CMAP_SINGLE, 2475 },
	{ 0x6d89, 0x6d89, PDF_CMAP_SINGLE, 13354 },
	{ 0x6d8a, 0x6d8a, PDF_CMAP_SINGLE, 17938 },
	{ 0x6d8c, 0x6d8c, PDF_CMAP_SINGLE, 3866 },
	{ 0x6d8d, 0x6d8d, PDF_CMAP_SINGLE, 17939 },
	{ 0x6d8e, 0x6d8e, PDF_CMAP_SINGLE, 5428 },
	{ 0x6d91, 0x6d91, PDF_CMAP_SINGLE, 17940 },
	{ 0x6d92, 0x6d92, PDF_CMAP_SINGLE, 21868 },
	{ 0x6d93, 0x6d93, PDF_CMAP_SINGLE, 5423 },
	{ 0x6d94, 0x6d94, PDF_CMAP_SINGLE, 16936 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 5429 },
	{ 0x6d96, 0x6d96, PDF_CMAP_SINGLE, 8518 },
	{ 0x6d97, 0x6d97, PDF_CMAP_SINGLE, 21869 },
	{ 0x6d98, 0x6d98, PDF_CMAP_SINGLE, 17941 },
	{ 0x6d99, 0x6d99, PDF_CMAP_SINGLE, 4006 },
	{ 0x6d9b, 0x6d9b, PDF_CMAP_SINGLE, 3181 },
	{ 0x6d9c, 0x6d9c, PDF_CMAP_SINGLE, 3226 },
	{ 0x6daa, 0x6daa, PDF_CMAP_SINGLE, 16937 },
	{ 0x6dab, 0x6dab, PDF_CMAP_SINGLE, 17946 },
	{ 0x6dac, 0x6dac, PDF_CMAP_SINGLE, 8519 },
	{ 0x6dae, 0x6dae, PDF_CMAP_SINGLE, 17947 },
	{ 0x6daf, 0x6daf, PDF_CMAP_SINGLE, 1428 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 1271 },
	{ 0x6db4, 0x6db4, PDF_CMAP_SINGLE, 17948 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 5435 },
	{ 0x6db7, 0x6db7, PDF_CMAP_SINGLE, 21870 },
	{ 0x6db8, 0x6db8, PDF_CMAP_SINGLE, 5438 },
	{ 0x6db9, 0x6db9, PDF_CMAP_SINGLE, 19494 },
	{ 0x6dbc, 0x6dbc, PDF_CMAP_SINGLE, 3979 },
	{ 0x6dbd, 0x6dbd, PDF_CMAP_SINGLE, 21871 },
	{ 0x6dbf, 0x6dbf, PDF_CMAP_SINGLE, 14712 },
	{ 0x6dc0, 0x6dc0, PDF_CMAP_SINGLE, 3918 },
	{ 0x6dc2, 0x6dc2, PDF_CMAP_SINGLE, 17949 },
	{ 0x6dc4, 0x6dc4, PDF_CMAP_SINGLE, 14713 },
	{ 0x6dc5, 0x6dc5, PDF_CMAP_SINGLE, 5445 },
	{ 0x6dc6, 0x6dc6, PDF_CMAP_SINGLE, 5439 },
	{ 0x6dc7, 0x6dc7, PDF_CMAP_SINGLE, 5436 },
	{ 0x6dc8, 0x6dc8, PDF_CMAP_SINGLE, 17951 },
	{ 0x6dca, 0x6dca, PDF_CMAP_SINGLE, 14714 },
	{ 0x6dcb, 0x6dcb, PDF_CMAP_SINGLE, 3996 },
	{ 0x6dcc, 0x6dcc, PDF_CMAP_SINGLE, 5442 },
	{ 0x6dce, 0x6dce, PDF_CMAP_SINGLE, 17952 },
	{ 0x6dcf, 0x6dcf, PDF_CMAP_SINGLE, 8520 },
	{ 0x6dd0, 0x6dd0, PDF_CMAP_SINGLE, 15414 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 2388 },
	{ 0x6dd2, 0x6dd2, PDF_CMAP_SINGLE, 5444 },
	{ 0x6dd5, 0x6dd5, PDF_CMAP_SINGLE, 5449 },
	{ 0x6dd6, 0x6dd6, PDF_CMAP_SINGLE, 14715 },
	{ 0x6dd8, 0x6dd8, PDF_CMAP_SINGLE, 3179 },
	{ 0x6dd9, 0x6dd9, PDF_CMAP_SINGLE, 5447 },
	{ 0x6dda, 0x6dda, PDF_CMAP_SINGLE, 13395 },
	{ 0x6ddb, 0x6ddb, PDF_CMAP_SINGLE, 16938 },
	{ 0x6ddd, 0x6ddd, PDF_CMAP_SINGLE, 16939 },
	{ 0x6dde, 0x6dde, PDF_CMAP_SINGLE, 5441 },
	{ 0x6ddf, 0x6ddf, PDF_CMAP_SINGLE, 17953 },
	{ 0x6de0, 0x6de0, PDF_CMAP_SINGLE, 21872 },
	{ 0x6de1, 0x6de1, PDF_CMAP_SINGLE, 2934 },
	{ 0x6de2, 0x6de2, PDF_CMAP_SINGLE, 21873 },
	{ 0x6de4, 0x6de4, PDF_CMAP_SINGLE, 5448 },
	{ 0x6de5, 0x6de5, PDF_CMAP_SINGLE, 21874 },
	{ 0x6de6, 0x6de6, PDF_CMAP_SINGLE, 5437 },
	{ 0x6de8, 0x6de8, PDF_CMAP_SINGLE, 5443 },
	{ 0x6de9, 0x6de9, PDF_CMAP_SINGLE, 14716 },
	{ 0x6dea, 0x6dea, PDF_CMAP_SINGLE, 5450 },
	{ 0x6deb, 0x6deb, PDF_CMAP_SINGLE, 1216 },
	{ 0x6dec, 0x6dec, PDF_CMAP_SINGLE, 5440 },
	{ 0x6dee, 0x6dee, PDF_CMAP_SINGLE, 5451 },
	{ 0x6def, 0x6def, PDF_CMAP_SINGLE, 21875 },
	{ 0x6df0, 0x6df0, PDF_CMAP_SINGLE, 19495 },
	{ 0x6df1, 0x6df1, PDF_CMAP_SINGLE, 2562 },
	{ 0x6df2, 0x6df2, PDF_CMAP_SINGLE, 8522 },
	{ 0x6df3, 0x6df3, PDF_CMAP_SINGLE, 2409 },
	{ 0x6df4, 0x6df4, PDF_CMAP_SINGLE, 21876 },
	{ 0x6df5, 0x6df5, PDF_CMAP_SINGLE, 3573 },
	{ 0x6df6, 0x6df6, PDF_CMAP_SINGLE, 17954 },
	{ 0x6df7, 0x6df7, PDF_CMAP_SINGLE, 2078 },
	{ 0x6df8, 0x6df8, PDF_CMAP_SINGLE, 8521 },
	{ 0x6df9, 0x6df9, PDF_CMAP_SINGLE, 5432 },
	{ 0x6dfa, 0x6dfa, PDF_CMAP_SINGLE, 5446 },
	{ 0x6dfb, 0x6dfb, PDF_CMAP_SINGLE, 3124 },
	{ 0x6dfc, 0x6dfc, PDF_CMAP_SINGLE, 8523 },
	{ 0x6e00, 0x6e00, PDF_CMAP_SINGLE, 21877 },
	{ 0x6e04, 0x6e04, PDF_CMAP_SINGLE, 21878 },
	{ 0x6e05, 0x6e05, PDF_CMAP_SINGLE, 2650 },
	{ 0x6e07, 0x6e07, PDF_CMAP_SINGLE, 1479 },
	{ 0x6e08, 0x6e08, PDF_CMAP_SINGLE, 2113 },
	{ 0x6e09, 0x6e09, PDF_CMAP_SINGLE, 2476 },
	{ 0x6e0a, 0x6e0a, PDF_CMAP_SINGLE, 5434 },
	{ 0x6e0b, 0x6e0b, PDF_CMAP_SINGLE, 2380 },
	{ 0x6e13, 0x6e13, PDF_CMAP_SINGLE, 1826 },
	{ 0x6e15, 0x6e15, PDF_CMAP_SINGLE, 5433 },
	{ 0x6e17, 0x6e17, PDF_CMAP_SINGLE, 14150 },
	{ 0x6e19, 0x6e19, PDF_CMAP_SINGLE, 5455 },
	{ 0x6e1a, 0x6e1a, PDF_CMAP_SINGLE, 2423 },
	{ 0x6e1b, 0x6e1b, PDF_CMAP_SINGLE, 1902 },
	{ 0x6e1d, 0x6e1d, PDF_CMAP_SINGLE, 5470 },
	{ 0x6e1e, 0x6e1e, PDF_CMAP_SINGLE, 17956 },
	{ 0x6e1f, 0x6e1f, PDF_CMAP_SINGLE, 5464 },
	{ 0x6e20, 0x6e20, PDF_CMAP_SINGLE, 1678 },
	{ 0x6e21, 0x6e21, PDF_CMAP_SINGLE, 3145 },
	{ 0x6e22, 0x6e22, PDF_CMAP_SINGLE, 14717 },
	{ 0x6e23, 0x6e23, PDF_CMAP_SINGLE, 5459 },
	{ 0x6e24, 0x6e24, PDF_CMAP_SINGLE, 5468 },
	{ 0x6e25, 0x6e25, PDF_CMAP_SINGLE, 1139 },
	{ 0x6e26, 0x6e26, PDF_CMAP_SINGLE, 1236 },
	{ 0x6e27, 0x6e27, PDF_CMAP_SINGLE, 8526 },
	{ 0x6e29, 0x6e29, PDF_CMAP_SINGLE, 1337 },
	{ 0x6e2b, 0x6e2b, PDF_CMAP_SINGLE, 5461 },
	{ 0x6e2c, 0x6e2c, PDF_CMAP_SINGLE, 2828 },
	{ 0x6e2d, 0x6e2d, PDF_CMAP_SINGLE, 5452 },
	{ 0x6e2e, 0x6e2e, PDF_CMAP_SINGLE, 5454 },
	{ 0x6e2f, 0x6e2f, PDF_CMAP_SINGLE, 2003 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 17958 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 13330 },
	{ 0x6e36, 0x6e36, PDF_CMAP_SINGLE, 17955 },
	{ 0x6e38, 0x6e38, PDF_CMAP_SINGLE, 5471 },
	{ 0x6e39, 0x6e39, PDF_CMAP_SINGLE, 8524 },
	{ 0x6e3a, 0x6e3a, PDF_CMAP_SINGLE, 5466 },
	{ 0x6e3b, 0x6e3b, PDF_CMAP_SINGLE, 21879 },
	{ 0x6e3c, 0x6e3c, PDF_CMAP_SINGLE, 8527 },
	{ 0x6e3e, 0x6e3e, PDF_CMAP_SINGLE, 5458 },
	{ 0x6e42, 0x6e42, PDF_CMAP_SINGLE, 15415 },
	{ 0x6e43, 0x6e43, PDF_CMAP_SINGLE, 5465 },
	{ 0x6e44, 0x6e44, PDF_CMAP_SINGLE, 16940 },
	{ 0x6e45, 0x6e45, PDF_CMAP_SINGLE, 19496 },
	{ 0x6e48, 0x6e49, PDF_CMAP_RANGE, 17959 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 3767 },
	{ 0x6e4b, 0x6e4c, PDF_CMAP_RANGE, 17961 },
	{ 0x6e4d, 0x6e4d, PDF_CMAP_SINGLE, 5463 },
	{ 0x6e4e, 0x6e4e, PDF_CMAP_SINGLE, 5467 },
	{ 0x6e4f, 0x6e4f, PDF_CMAP_SINGLE, 17963 },
	{ 0x6e51, 0x6e51, PDF_CMAP_SINGLE, 14718 },
	{ 0x6e52, 0x6e52, PDF_CMAP_SINGLE, 21880 },
	{ 0x6e53, 0x6e54, PDF_CMAP_RANGE, 17964 },
	{ 0x6e56, 0x6e56, PDF_CMAP_SINGLE, 1924 },
	{ 0x6e57, 0x6e57, PDF_CMAP_SINGLE, 17966 },
	{ 0x6e58, 0x6e58, PDF_CMAP_SINGLE, 2477 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 2935 },
	{ 0x6e5c, 0x6e5c, PDF_CMAP_SINGLE, 8525 },
	{ 0x6e5d, 0x6e5d, PDF_CMAP_SINGLE, 21881 },
	{ 0x6e5e, 0x6e5e, PDF_CMAP_SINGLE, 16941 },
	{ 0x6e5f, 0x6e5f, PDF_CMAP_SINGLE, 5457 },
	{ 0x6e62, 0x6e62, PDF_CMAP_SINGLE, 21882 },
	{ 0x6e63, 0x6e63, PDF_CMAP_SINGLE, 17967 },
	{ 0x6e67, 0x6e67, PDF_CMAP_SINGLE, 3865 },
	{ 0x6e68, 0x6e68, PDF_CMAP_SINGLE, 21883 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 5460 },
	{ 0x6e6e, 0x6e6e, PDF_CMAP_SINGLE, 5453 },
	{ 0x6e6f, 0x6e6f, PDF_CMAP_SINGLE, 3180 },
	{ 0x6e72, 0x6e72, PDF_CMAP_SINGLE, 5456 },
	{ 0x6e73, 0x6e73, PDF_CMAP_SINGLE, 19497 },
	{ 0x6e76, 0x6e76, PDF_CMAP_SINGLE, 5462 },
	{ 0x6e7b, 0x6e7b, PDF_CMAP_SINGLE, 19498 },
	{ 0x6e7d, 0x6e7d, PDF_CMAP_SINGLE, 19499 },
	{ 0x6e7e, 0x6e7e, PDF_CMAP_SINGLE, 4087 },
	{ 0x6e7f, 0x6e7f, PDF_CMAP_SINGLE, 2282 },
	{ 0x6e80, 0x6e80, PDF_CMAP_SINGLE, 3756 },
	{ 0x6e82, 0x6e82, PDF_CMAP_SINGLE, 5472 },
	{ 0x6e89, 0x6e89, PDF_CMAP_SINGLE, 19500 },
	{ 0x6e8c, 0x6e8c, PDF_CMAP_SINGLE, 3394 },
	{ 0x6e8d, 0x6e8d, PDF_CMAP_SINGLE, 21884 },
	{ 0x6e8f, 0x6e8f, PDF_CMAP_SINGLE, 5484 },
	{ 0x6e90, 0x6e90, PDF_CMAP_SINGLE, 1903 },
	{ 0x6e93, 0x6e93, PDF_CMAP_SINGLE, 17969 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 2410 },
	{ 0x6e98, 0x6e98, PDF_CMAP_SINGLE, 5474 },
	{ 0x6e99, 0x6e99, PDF_CMAP_SINGLE, 21885 },
	{ 0x6e9c, 0x6e9c, PDF_CMAP_SINGLE, 3959 },
	{ 0x6e9d, 0x6e9d, PDF_CMAP_SINGLE, 2004 },
	{ 0x6e9f, 0x6e9f, PDF_CMAP_SINGLE, 5487 },
	{ 0x6ea0, 0x6ea0, PDF_CMAP_SINGLE, 21886 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 1202 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 5485 },
	{ 0x6ea7, 0x6ea7, PDF_CMAP_SINGLE, 17970 },
	{ 0x6eaa, 0x6eaa, PDF_CMAP_SINGLE, 5473 },
	{ 0x6eab, 0x6eab, PDF_CMAP_SINGLE, 13324 },
	{ 0x6ead, 0x6eae, PDF_CMAP_RANGE, 21887 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 5479 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 16942 },
	{ 0x6eb2, 0x6eb2, PDF_CMAP_SINGLE, 5481 },
	{ 0x6eb3, 0x6eb3, PDF_CMAP_SINGLE, 21889 },
	{ 0x6eb4, 0x6eb4, PDF_CMAP_SINGLE, 17971 },
	{ 0x6eb6, 0x6eb6, PDF_CMAP_SINGLE, 3897 },
	{ 0x6eb7, 0x6eb7, PDF_CMAP_SINGLE, 5476 },
	{ 0x6eba, 0x6eba, PDF_CMAP_SINGLE, 3112 },
	{ 0x6ebb, 0x6ebb, PDF_CMAP_SINGLE, 21890 },
	{ 0x6ebc, 0x6ebc, PDF_CMAP_SINGLE, 19501 },
	{ 0x6ebd, 0x6ebd, PDF_CMAP_SINGLE, 5478 },
	{ 0x6ebf, 0x6ebf, PDF_CMAP_SINGLE, 8528 },
	{ 0x6ec0, 0x6ec0, PDF_CMAP_SINGLE, 21891 },
	{ 0x6ec1, 0x6ec1, PDF_CMAP_SINGLE, 16943 },
	{ 0x6ec2, 0x6ec2, PDF_CMAP_SINGLE, 5486 },
	{ 0x6ec3, 0x6ec3, PDF_CMAP_SINGLE, 17972 },
	{ 0x6ec4, 0x6ec4, PDF_CMAP_SINGLE, 5480 },
	{ 0x6ec5, 0x6ec5, PDF_CMAP_SINGLE, 3795 },
	{ 0x6ec7, 0x6ec7, PDF_CMAP_SINGLE, 14719 },
	{ 0x6ec8, 0x6ec8, PDF_CMAP_SINGLE, 21892 },
	{ 0x6ec9, 0x6ec9, PDF_CMAP_SINGLE, 5475 },
	{ 0x6eca, 0x6eca, PDF_CMAP_SINGLE, 14720 },
	{ 0x6ecb, 0x6ecb, PDF_CMAP_SINGLE, 2254 },
	{ 0x6ecc, 0x6ecc, PDF_CMAP_SINGLE, 5499 },
	{ 0x6ecd, 0x6ecd, PDF_CMAP_SINGLE, 21893 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 14721 },
	{ 0x6ecf, 0x6ecf, PDF_CMAP_SINGLE, 21894 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 1480 },
	{ 0x6ed3, 0x6ed3, PDF_CMAP_SINGLE, 5477 },
	{ 0x6ed4, 0x6ed5, PDF_CMAP_RANGE, 5482 },
	{ 0x6ed9, 0x6ed9, PDF_CMAP_SINGLE, 15416 },
	{ 0x6eda, 0x6edb, PDF_CMAP_RANGE, 19502 },
	{ 0x6edd, 0x6edd, PDF_CMAP_SINGLE, 2892 },
	{ 0x6ede, 0x6ede, PDF_CMAP_SINGLE, 2874 },
	{ 0x6ee6, 0x6ee6, PDF_CMAP_SINGLE, 15396 },
	{ 0x6eeb, 0x6eeb, PDF_CMAP_SINGLE, 17974 },
	{ 0x6eec, 0x6eec, PDF_CMAP_SINGLE, 5491 },
	{ 0x6eed, 0x6eee, PDF_CMAP_RANGE, 21895 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 5497 },
	{ 0x6ef2, 0x6ef2, PDF_CMAP_SINGLE, 5495 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 3107 },
	{ 0x6ef7, 0x6ef7, PDF_CMAP_SINGLE, 5502 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 5492 },
	{ 0x6ef9, 0x6ef9, PDF_CMAP_SINGLE, 17975 },
	{ 0x6efb, 0x6efb, PDF_CMAP_SINGLE, 17976 },
	{ 0x6efd, 0x6efd, PDF_CMAP_SINGLE, 14722 },
	{ 0x6efe, 0x6efe, PDF_CMAP_SINGLE, 5493 },
	{ 0x6eff, 0x6eff, PDF_CMAP_SINGLE, 5469 },
	{ 0x6f01, 0x6f01, PDF_CMAP_SINGLE, 1683 },
	{ 0x6f02, 0x6f02, PDF_CMAP_SINGLE, 3500 },
	{ 0x6f04, 0x6f04, PDF_CMAP_SINGLE, 21897 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 2283 },
	{ 0x6f08, 0x6f08, PDF_CMAP_SINGLE, 21898 },
	{ 0x6f09, 0x6f09, PDF_CMAP_SINGLE, 2057 },
	{ 0x6f0a, 0x6f0a, PDF_CMAP_SINGLE, 17977 },
	{ 0x6f0c, 0x6f0c, PDF_CMAP_SINGLE, 17978 },
	{ 0x6f0d, 0x6f0d, PDF_CMAP_SINGLE, 21899 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 4057 },
	{ 0x6f10, 0x6f10, PDF_CMAP_SINGLE, 16944 },
	{ 0x6f11, 0x6f11, PDF_CMAP_SINGLE, 5489 },
	{ 0x6f13, 0x6f13, PDF_CMAP_SINGLE, 5501 },
	{ 0x6f14, 0x6f14, PDF_CMAP_SINGLE, 1291 },
	{ 0x6f15, 0x6f15, PDF_CMAP_SINGLE, 2792 },
	{ 0x6f16, 0x6f16, PDF_CMAP_SINGLE, 21900 },
	{ 0x6f18, 0x6f18, PDF_CMAP_SINGLE, 17979 },
	{ 0x6f1a, 0x6f1a, PDF_CMAP_SINGLE, 14723 },
	{ 0x6f1b, 0x6f1b, PDF_CMAP_SINGLE, 21901 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 3375 },
	{ 0x6f22, 0x6f22, PDF_CMAP_SINGLE, 1533 },
	{ 0x6f23, 0x6f23, PDF_CMAP_SINGLE, 4034 },
	{ 0x6f25, 0x6f25, PDF_CMAP_SINGLE, 17980 },
	{ 0x6f26, 0x6f26, PDF_CMAP_SINGLE, 19504 },
	{ 0x6f29, 0x6f29, PDF_CMAP_SINGLE, 19505 },
	{ 0x6f2a, 0x6f2a, PDF_CMAP_SINGLE, 14724 },
	{ 0x6f2b, 0x6f2b, PDF_CMAP_SINGLE, 3757 },
	{ 0x6f2c, 0x6f2c, PDF_CMAP_SINGLE, 3054 },
	{ 0x6f2d, 0x6f2d, PDF_CMAP_SINGLE, 21903 },
	{ 0x6f2f, 0x6f2f, PDF_CMAP_SINGLE, 14725 },
	{ 0x6f30, 0x6f30, PDF_CMAP_SINGLE, 19506 },
	{ 0x6f31, 0x6f31, PDF_CMAP_SINGLE, 5496 },
	{ 0x6f32, 0x6f32, PDF_CMAP_SINGLE, 5498 },
	{ 0x6f33, 0x6f33, PDF_CMAP_SINGLE, 14726 },
	{ 0x6f35, 0x6f35, PDF_CMAP_SINGLE, 17973 },
	{ 0x6f36, 0x6f36, PDF_CMAP_SINGLE, 17981 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 2740 },
	{ 0x6f3b, 0x6f3b, PDF_CMAP_SINGLE, 21902 },
	{ 0x6f3c, 0x6f3c, PDF_CMAP_SINGLE, 17982 },
	{ 0x6f3e, 0x6f3e, PDF_CMAP_SINGLE, 5500 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 5494 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 5488 },
	{ 0x6f45, 0x6f45, PDF_CMAP_SINGLE, 1535 },
	{ 0x6f4f, 0x6f4f, PDF_CMAP_SINGLE, 21904 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 7776 },
	{ 0x6f52, 0x6f52, PDF_CMAP_SINGLE, 17984 },
	{ 0x6f53, 0x6f53, PDF_CMAP_SINGLE, 21905 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 1855 },
	{ 0x6f57, 0x6f57, PDF_CMAP_SINGLE, 17985 },
	{ 0x6f58, 0x6f58, PDF_CMAP_SINGLE, 5514 },
	{ 0x6f59, 0x6f59, PDF_CMAP_SINGLE, 16945 },
	{ 0x6f5a, 0x6f5a, PDF_CMAP_SINGLE, 14727 },
	{ 0x6f5b, 0x6f5b, PDF_CMAP_SINGLE, 5509 },
	{ 0x6f5c, 0x6f5c, PDF_CMAP_SINGLE, 2716 },
	{ 0x6f5d, 0x6f5d, PDF_CMAP_SINGLE, 21906 },
	{ 0x6f5e, 0x6f5e, PDF_CMAP_SINGLE, 14728 },
	{ 0x6f5f, 0x6f5f, PDF_CMAP_SINGLE, 1473 },
	{ 0x6f60, 0x6f60, PDF_CMAP_SINGLE, 17986 },
	{ 0x6f61, 0x6f61, PDF_CMAP_SINGLE, 16946 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 14729 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 2411 },
	{ 0x6f66, 0x6f66, PDF_CMAP_SINGLE, 5518 },
	{ 0x6f68, 0x6f68, PDF_CMAP_SINGLE, 17987 },
	{ 0x6f6c, 0x6f6c, PDF_CMAP_SINGLE, 21907 },
	{ 0x6f6d, 0x6f6d, PDF_CMAP_SINGLE, 5511 },
	{ 0x6f6e, 0x6f6e, PDF_CMAP_SINGLE, 3016 },
	{ 0x6f6f, 0x6f6f, PDF_CMAP_SINGLE, 5508 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 3061 },
	{ 0x6f74, 0x6f74, PDF_CMAP_SINGLE, 5543 },
	{ 0x6f78, 0x6f78, PDF_CMAP_SINGLE, 5505 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 5504 },
	{ 0x6f7c, 0x6f7c, PDF_CMAP_SINGLE, 5513 },
	{ 0x6f7d, 0x6f7d, PDF_CMAP_SINGLE, 14730 },
	{ 0x6f7e, 0x6f7e, PDF_CMAP_SINGLE, 16947 },
	{ 0x6f80, 0x6f80, PDF_CMAP_SINGLE, 5507 },
	{ 0x6f81, 0x6f81, PDF_CMAP_SINGLE, 5506 },
	{ 0x6f82, 0x6f82, PDF_CMAP_SINGLE, 5512 },
	{ 0x6f83, 0x6f83, PDF_CMAP_SINGLE, 21908 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 2629 },
	{ 0x6f86, 0x6f86, PDF_CMAP_SINGLE, 5503 },
	{ 0x6f87, 0x6f87, PDF_CMAP_SINGLE, 19507 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 8529 },
	{ 0x6f8b, 0x6f8b, PDF_CMAP_SINGLE, 14731 },
	{ 0x6f8c, 0x6f8c, PDF_CMAP_SINGLE, 16948 },
	{ 0x6f8d, 0x6f8d, PDF_CMAP_SINGLE, 14732 },
	{ 0x6f8e, 0x6f8e, PDF_CMAP_SINGLE, 5515 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 17988 },
	{ 0x6f91, 0x6f91, PDF_CMAP_SINGLE, 5516 },
	{ 0x6f92, 0x6f92, PDF_CMAP_SINGLE, 14733 },
	{ 0x6f93, 0x6f93, PDF_CMAP_SINGLE, 21909 },
	{ 0x6f94, 0x6f94, PDF_CMAP_SINGLE, 14734 },
	{ 0x6f96, 0x6f96, PDF_CMAP_SINGLE, 17989 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 1534 },
	{ 0x6f98, 0x6f98, PDF_CMAP_SINGLE, 13884 },
	{ 0x6f9a, 0x6f9a, PDF_CMAP_SINGLE, 14735 },
	{ 0x6f9d, 0x6f9d, PDF_CMAP_SINGLE, 19508 },
	{ 0x6f9f, 0x6f9f, PDF_CMAP_SINGLE, 17991 },
	{ 0x6fa0, 0x6fa0, PDF_CMAP_SINGLE, 16949 },
	{ 0x6fa1, 0x6fa1, PDF_CMAP_SINGLE, 5521 },
	{ 0x6fa3, 0x6fa3, PDF_CMAP_SINGLE, 5520 },
	{ 0x6fa4, 0x6fa4, PDF_CMAP_SINGLE, 5522 },
	{ 0x6fa5, 0x6fa5, PDF_CMAP_SINGLE, 17992 },
	{ 0x6fa6, 0x6fa6, PDF_CMAP_SINGLE, 21910 },
	{ 0x6fa7, 0x6fa8, PDF_CMAP_RANGE, 14736 },
	{ 0x6faa, 0x6faa, PDF_CMAP_SINGLE, 5525 },
	{ 0x6fae, 0x6fae, PDF_CMAP_SINGLE, 19509 },
	{ 0x6faf, 0x6faf, PDF_CMAP_SINGLE, 17993 },
	{ 0x6fb0, 0x6fb0, PDF_CMAP_SINGLE, 21911 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 3133 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 5519 },
	{ 0x6fb5, 0x6fb5, PDF_CMAP_SINGLE, 8530 },
	{ 0x6fb6, 0x6fb6, PDF_CMAP_SINGLE, 14738 },
	{ 0x6fb7, 0x6fb7, PDF_CMAP_SINGLE, 19510 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 5523 },
	{ 0x6fbc, 0x6fbc, PDF_CMAP_SINGLE, 16950 },
	{ 0x6fbe, 0x6fbe, PDF_CMAP_SINGLE, 17990 },
	{ 0x6fc0, 0x6fc0, PDF_CMAP_SINGLE, 1849 },
	{ 0x6fc1, 0x6fc1, PDF_CMAP_SINGLE, 2905 },
	{ 0x6fc2, 0x6fc2, PDF_CMAP_SINGLE, 5517 },
	{ 0x6fc3, 0x6fc3, PDF_CMAP_SINGLE, 3313 },
	{ 0x6fc5, 0x6fc5, PDF_CMAP_SINGLE, 21912 },
	{ 0x6fc6, 0x6fc6, PDF_CMAP_SINGLE, 5524 },
	{ 0x6fc7, 0x6fc7, PDF_CMAP_SINGLE, 16951 },
	{ 0x6fc8, 0x6fc9, PDF_CMAP_RANGE, 17995 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 16952 },
	{ 0x6fd4, 0x6fd4, PDF_CMAP_SINGLE, 5529 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 5527 },
	{ 0x6fd8, 0x6fd8, PDF_CMAP_SINGLE, 5530 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 14739 },
	{ 0x6fdb, 0x6fdb, PDF_CMAP_SINGLE, 5533 },
	{ 0x6fde, 0x6fde, PDF_CMAP_SINGLE, 14740 },
	{ 0x6fdf, 0x6fdf, PDF_CMAP_SINGLE, 5526 },
	{ 0x6fe0, 0x6fe0, PDF_CMAP_SINGLE, 2044 },
	{ 0x6fe1, 0x6fe1, PDF_CMAP_SINGLE, 3294 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 5430 },
	{ 0x6fe8, 0x6fe8, PDF_CMAP_SINGLE, 21913 },
	{ 0x6fe9, 0x6fe9, PDF_CMAP_SINGLE, 17997 },
	{ 0x6feb, 0x6feb, PDF_CMAP_SINGLE, 3934 },
	{ 0x6fec, 0x6fec, PDF_CMAP_SINGLE, 5528 },
	{ 0x6fee, 0x6fee, PDF_CMAP_SINGLE, 5532 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 2901 },
	{ 0x6ff0, 0x6ff0, PDF_CMAP_SINGLE, 16953 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 5531 },
	{ 0x6ff3, 0x6ff3, PDF_CMAP_SINGLE, 5510 },
	{ 0x6ff5, 0x6ff5, PDF_CMAP_SINGLE, 8531 },
	{ 0x6ff6, 0x6ff6, PDF_CMAP_SINGLE, 7076 },
	{ 0x6ff9, 0x6ff9, PDF_CMAP_SINGLE, 15395 },
	{ 0x6ffa, 0x6ffa, PDF_CMAP_SINGLE, 5536 },
	{ 0x6ffc, 0x6ffc, PDF_CMAP_SINGLE, 17999 },
	{ 0x6ffd, 0x6ffd, PDF_CMAP_SINGLE, 21914 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 5540 },
	{ 0x7000, 0x7000, PDF_CMAP_SINGLE, 18000 },
	{ 0x7001, 0x7001, PDF_CMAP_SINGLE, 5538 },
	{ 0x7005, 0x7005, PDF_CMAP_SINGLE, 8532 },
	{ 0x7006, 0x7006, PDF_CMAP_SINGLE, 7760 },
	{ 0x7007, 0x7007, PDF_CMAP_SINGLE, 8533 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 5534 },
	{ 0x700a, 0x700a, PDF_CMAP_SINGLE, 18001 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 5535 },
	{ 0x700d, 0x700d, PDF_CMAP_SINGLE, 19511 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 5539 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 5537 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 3520 },
	{ 0x7017, 0x7017, PDF_CMAP_SINGLE, 21915 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 5545 },
	{ 0x701a, 0x701a, PDF_CMAP_SINGLE, 5542 },
	{ 0x701b, 0x701b, PDF_CMAP_SINGLE, 5541 },
	{ 0x701d, 0x701d, PDF_CMAP_SINGLE, 5544 },
	{ 0x701e, 0x701e, PDF_CMAP_SINGLE, 3244 },
	{ 0x701f, 0x701f, PDF_CMAP_SINGLE, 5546 },
	{ 0x7020, 0x7020, PDF_CMAP_SINGLE, 19512 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 18002 },
	{ 0x7026, 0x7026, PDF_CMAP_SINGLE, 2995 },
	{ 0x7027, 0x7027, PDF_CMAP_SINGLE, 2893 },
	{ 0x7028, 0x7028, PDF_CMAP_SINGLE, 8534 },
	{ 0x702c, 0x702c, PDF_CMAP_SINGLE, 2633 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 21916 },
	{ 0x7030, 0x7030, PDF_CMAP_SINGLE, 5547 },
	{ 0x7032, 0x7032, PDF_CMAP_SINGLE, 5549 },
	{ 0x7034, 0x7034, PDF_CMAP_SINGLE, 21917 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 21918 },
	{ 0x7039, 0x7039, PDF_CMAP_SINGLE, 14742 },
	{ 0x703a, 0x703a, PDF_CMAP_SINGLE, 18004 },
	{ 0x703c, 0x703c, PDF_CMAP_SINGLE, 14743 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 5548 },
	{ 0x7043, 0x7043, PDF_CMAP_SINGLE, 18005 },
	{ 0x7044, 0x7044, PDF_CMAP_SINGLE, 21919 },
	{ 0x7047, 0x7047, PDF_CMAP_SINGLE, 18006 },
	{ 0x7048, 0x7048, PDF_CMAP_SINGLE, 21920 },
	{ 0x7049, 0x7049, PDF_CMAP_SINGLE, 19513 },
	{ 0x704a, 0x704a, PDF_CMAP_SINGLE, 20306 },
	{ 0x704b, 0x704b, PDF_CMAP_SINGLE, 18007 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 5490 },
	{ 0x704e, 0x704e, PDF_CMAP_SINGLE, 16954 },
	{ 0x7051, 0x7051, PDF_CMAP_SINGLE, 5550 },
	{ 0x7054, 0x7054, PDF_CMAP_SINGLE, 14745 },
	{ 0x7055, 0x7055, PDF_CMAP_SINGLE, 21921 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 3263 },
	{ 0x705d, 0x705e, PDF_CMAP_RANGE, 14746 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 5551 },
	{ 0x7064, 0x7064, PDF_CMAP_SINGLE, 14748 },
	{ 0x7065, 0x7065, PDF_CMAP_SINGLE, 18009 },
	{ 0x7069, 0x7069, PDF_CMAP_SINGLE, 18010 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 1360 },
	{ 0x706c, 0x706c, PDF_CMAP_SINGLE, 14749 },
	{ 0x706e, 0x706e, PDF_CMAP_SINGLE, 18011 },
	{ 0x706f, 0x706f, PDF_CMAP_SINGLE, 3182 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 1411 },
	{ 0x7075, 0x7075, PDF_CMAP_SINGLE, 16955 },
	{ 0x7076, 0x7076, PDF_CMAP_SINGLE, 18012 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 1662 },
	{ 0x707c, 0x707c, PDF_CMAP_SINGLE, 2314 },
	{ 0x707d, 0x707d, PDF_CMAP_SINGLE, 2114 },
	{ 0x707e, 0x707e, PDF_CMAP_SINGLE, 14750 },
	{ 0x7081, 0x7081, PDF_CMAP_SINGLE, 14751 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 8535 },
	{ 0x7086, 0x7086, PDF_CMAP_SINGLE, 18013 },
	{ 0x7089, 0x7089, PDF_CMAP_SINGLE, 4045 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 2604 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 1292 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 5553 },
	{ 0x7094, 0x7094, PDF_CMAP_SINGLE, 21922 },
	{ 0x7095, 0x7095, PDF_CMAP_SINGLE, 14752 },
	{ 0x7096, 0x7096, PDF_CMAP_SINGLE, 21923 },
	{ 0x7097, 0x7097, PDF_CMAP_SINGLE, 18014 },
	{ 0x7098, 0x7098, PDF_CMAP_SINGLE, 19514 },
	{ 0x7099, 0x7099, PDF_CMAP_SINGLE, 5552 },
	{ 0x709b, 0x709b, PDF_CMAP_SINGLE, 21924 },
	{ 0x709f, 0x709f, PDF_CMAP_SINGLE, 18016 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 16956 },
	{ 0x70ab, 0x70ab, PDF_CMAP_SINGLE, 8536 },
	{ 0x70ac, 0x70ac, PDF_CMAP_SINGLE, 5556 },
	{ 0x70ad, 0x70ad, PDF_CMAP_SINGLE, 2936 },
	{ 0x70ae, 0x70ae, PDF_CMAP_SINGLE, 5559 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 5554 },
	{ 0x70b0, 0x70b0, PDF_CMAP_SINGLE, 19515 },
	{ 0x70b1, 0x70b1, PDF_CMAP_SINGLE, 18017 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 5558 },
	{ 0x70b4, 0x70b4, PDF_CMAP_SINGLE, 21925 },
	{ 0x70b7, 0x70b7, PDF_CMAP_SINGLE, 14753 },
	{ 0x70b8, 0x70b8, PDF_CMAP_SINGLE, 5557 },
	{ 0x70b9, 0x70b9, PDF_CMAP_SINGLE, 3130 },
	{ 0x70ba, 0x70ba, PDF_CMAP_SINGLE, 1181 },
	{ 0x70bb, 0x70bb, PDF_CMAP_SINGLE, 8365 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 4029 },
	{ 0x70ca, 0x70ca, PDF_CMAP_SINGLE, 18020 },
	{ 0x70cb, 0x70cb, PDF_CMAP_SINGLE, 5561 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 1226 },
	{ 0x70d1, 0x70d1, PDF_CMAP_SINGLE, 18021 },
	{ 0x70d3, 0x70d4, PDF_CMAP_RANGE, 14754 },
	{ 0x70d5, 0x70d6, PDF_CMAP_RANGE, 19516 },
	{ 0x70d8, 0x70d8, PDF_CMAP_SINGLE, 14756 },
	{ 0x70d9, 0x70d9, PDF_CMAP_SINGLE, 5563 },
	{ 0x70dc, 0x70dc, PDF_CMAP_SINGLE, 14757 },
	{ 0x70dd, 0x70dd, PDF_CMAP_SINGLE, 5562 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 5560 },
	{ 0x70e4, 0x70e4, PDF_CMAP_SINGLE, 16957 },
	{ 0x70ec, 0x70ec, PDF_CMAP_SINGLE, 18019 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 5555 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 3665 },
	{ 0x70fa, 0x70fa, PDF_CMAP_SINGLE, 21926 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 5565 },
	{ 0x7103, 0x7103, PDF_CMAP_SINGLE, 18022 },
	{ 0x7104, 0x7104, PDF_CMAP_SINGLE, 8538 },
	{ 0x7105, 0x7105, PDF_CMAP_SINGLE, 21927 },
	{ 0x7106, 0x7106, PDF_CMAP_SINGLE, 18023 },
	{ 0x7107, 0x7107, PDF_CMAP_SINGLE, 14758 },
	{ 0x7108, 0x7108, PDF_CMAP_SINGLE, 18024 },
	{ 0x7109, 0x7109, PDF_CMAP_SINGLE, 5564 },
	{ 0x710b, 0x710b, PDF_CMAP_SINGLE, 21928 },
	{ 0x710c, 0x710c, PDF_CMAP_SINGLE, 18025 },
	{ 0x710f, 0x710f, PDF_CMAP_SINGLE, 8537 },
	{ 0x7114, 0x7114, PDF_CMAP_SINGLE, 1293 },
	{ 0x7119, 0x7119, PDF_CMAP_SINGLE, 5567 },
	{ 0x711a, 0x711a, PDF_CMAP_SINGLE, 3586 },
	{ 0x711c, 0x711c, PDF_CMAP_SINGLE, 5566 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 16959 },
	{ 0x7120, 0x7120, PDF_CMAP_SINGLE, 14759 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 3777 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 2479 },
	{ 0x712b, 0x712b, PDF_CMAP_SINGLE, 16958 },
	{ 0x712d, 0x712d, PDF_CMAP_SINGLE, 21929 },
	{ 0x712e, 0x712e, PDF_CMAP_SINGLE, 16960 },
	{ 0x712f, 0x712f, PDF_CMAP_SINGLE, 18027 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 7644 },
	{ 0x7131, 0x7131, PDF_CMAP_SINGLE, 14760 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 2741 },
	{ 0x7138, 0x7138, PDF_CMAP_SINGLE, 21930 },
	{ 0x713c, 0x713c, PDF_CMAP_SINGLE, 2478 },
	{ 0x7141, 0x7141, PDF_CMAP_SINGLE, 21931 },
	{ 0x7145, 0x7145, PDF_CMAP_SINGLE, 19518 },
	{ 0x7146, 0x7147, PDF_CMAP_RANGE, 8540 },
	{ 0x7149, 0x7149, PDF_CMAP_SINGLE, 4035 },
	{ 0x714a, 0x714a, PDF_CMAP_SINGLE, 14761 },
	{ 0x714b, 0x714b, PDF_CMAP_SINGLE, 21932 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 5573 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 2717 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 18028 },
	{ 0x7151, 0x7151, PDF_CMAP_SINGLE, 16961 },
	{ 0x7152, 0x7152, PDF_CMAP_SINGLE, 14762 },
	{ 0x7153, 0x7153, PDF_CMAP_SINGLE, 18029 },
	{ 0x7155, 0x7155, PDF_CMAP_SINGLE, 5569 },
	{ 0x7156, 0x7156, PDF_CMAP_SINGLE, 5574 },
	{ 0x7157, 0x7157, PDF_CMAP_SINGLE, 21934 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 1294 },
	{ 0x715a, 0x715a, PDF_CMAP_SINGLE, 21935 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 8539 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 18030 },
	{ 0x7160, 0x7160, PDF_CMAP_SINGLE, 14763 },
	{ 0x7162, 0x7162, PDF_CMAP_SINGLE, 5572 },
	{ 0x7164, 0x7164, PDF_CMAP_SINGLE, 3351 },
	{ 0x7165, 0x7165, PDF_CMAP_SINGLE, 5568 },
	{ 0x7166, 0x7166, PDF_CMAP_SINGLE, 5571 },
	{ 0x7167, 0x7167, PDF_CMAP_SINGLE, 2480 },
	{ 0x7168, 0x7168, PDF_CMAP_SINGLE, 16962 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 3429 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 5575 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 2301 },
	{ 0x7179, 0x7179, PDF_CMAP_SINGLE, 14764 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 2718 },
	{ 0x7180, 0x7180, PDF_CMAP_SINGLE, 18033 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 5578 },
	{ 0x7185, 0x7185, PDF_CMAP_SINGLE, 16963 },
	{ 0x7187, 0x7187, PDF_CMAP_SINGLE, 16964 },
	{ 0x7188, 0x7188, PDF_CMAP_SINGLE, 5570 },
	{ 0x718a, 0x718a, PDF_CMAP_SINGLE, 1789 },
	{ 0x718c, 0x718c, PDF_CMAP_SINGLE, 21936 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 5576 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 14765 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 3898 },
	{ 0x7195, 0x7195, PDF_CMAP_SINGLE, 5579 },
	{ 0x7196, 0x7196, PDF_CMAP_SINGLE, 18032 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 8285 },
	{ 0x719a, 0x719a, PDF_CMAP_SINGLE, 21937 },
	{ 0x719b, 0x719b, PDF_CMAP_SINGLE, 18034 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 2393 },
	{ 0x71a0, 0x71a0, PDF_CMAP_SINGLE, 18035 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 18036 },
	{ 0x71a8, 0x71a8, PDF_CMAP_SINGLE, 5580 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 5581 },
	{ 0x71ae, 0x71af, PDF_CMAP_RANGE, 18037 },
	{ 0x71b0, 0x71b0, PDF_CMAP_SINGLE, 21938 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 3300 },
	{ 0x71b2, 0x71b2, PDF_CMAP_SINGLE, 19519 },
	{ 0x71b3, 0x71b3, PDF_CMAP_SINGLE, 15397 },
	{ 0x71b9, 0x71b9, PDF_CMAP_SINGLE, 5583 },
	{ 0x71ba, 0x71ba, PDF_CMAP_SINGLE, 16965 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 5584 },
	{ 0x71bf, 0x71c0, PDF_CMAP_RANGE, 21939 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 8543 },
	{ 0x71c3, 0x71c3, PDF_CMAP_SINGLE, 3305 },
	{ 0x71c4, 0x71c4, PDF_CMAP_SINGLE, 16966 },
	{ 0x71c8, 0x71c8, PDF_CMAP_SINGLE, 3183 },
	{ 0x71c9, 0x71c9, PDF_CMAP_SINGLE, 5586 },
	{ 0x71cb, 0x71cb, PDF_CMAP_SINGLE, 14767 },
	{ 0x71cc, 0x71cc, PDF_CMAP_SINGLE, 21941 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 5588 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 3997 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 5585 },
	{ 0x71d3, 0x71d3, PDF_CMAP_SINGLE, 14768 },
	{ 0x71d4, 0x71d4, PDF_CMAP_SINGLE, 5587 },
	{ 0x71d5, 0x71d5, PDF_CMAP_SINGLE, 1295 },
	{ 0x71d6, 0x71d6, PDF_CMAP_SINGLE, 14769 },
	{ 0x71d7, 0x71d7, PDF_CMAP_SINGLE, 5582 },
	{ 0x71d9, 0x71d9, PDF_CMAP_SINGLE, 18040 },
	{ 0x71da, 0x71da, PDF_CMAP_SINGLE, 21942 },
	{ 0x71dc, 0x71dc, PDF_CMAP_SINGLE, 18041 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 4430 },
	{ 0x71e0, 0x71e0, PDF_CMAP_SINGLE, 5589 },
	{ 0x71e5, 0x71e5, PDF_CMAP_SINGLE, 2793 },
	{ 0x71e6, 0x71e6, PDF_CMAP_SINGLE, 2182 },
	{ 0x71e7, 0x71e7, PDF_CMAP_SINGLE, 5591 },
	{ 0x71ec, 0x71ec, PDF_CMAP_SINGLE, 5590 },
	{ 0x71ed, 0x71ed, PDF_CMAP_SINGLE, 2538 },
	{ 0x71ee, 0x71ee, PDF_CMAP_SINGLE, 4334 },
	{ 0x71f4, 0x71f4, PDF_CMAP_SINGLE, 19520 },
	{ 0x71f5, 0x71f5, PDF_CMAP_SINGLE, 5592 },
	{ 0x71f8, 0x71f8, PDF_CMAP_SINGLE, 21943 },
	{ 0x71f9, 0x71f9, PDF_CMAP_SINGLE, 5594 },
	{ 0x71fb, 0x71fb, PDF_CMAP_SINGLE, 5577 },
	{ 0x71fc, 0x71fc, PDF_CMAP_SINGLE, 5593 },
	{ 0x71fe, 0x71fe, PDF_CMAP_SINGLE, 8544 },
	{ 0x71ff, 0x71ff, PDF_CMAP_SINGLE, 5595 },
	{ 0x7200, 0x7200, PDF_CMAP_SINGLE, 14770 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 3376 },
	{ 0x7207, 0x7207, PDF_CMAP_SINGLE, 18042 },
	{ 0x7208, 0x7209, PDF_CMAP_RANGE, 21944 },
	{ 0x720d, 0x720d, PDF_CMAP_SINGLE, 5596 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 5597 },
	{ 0x7213, 0x7213, PDF_CMAP_SINGLE, 21946 },
	{ 0x7215, 0x7215, PDF_CMAP_SINGLE, 16967 },
	{ 0x7217, 0x7217, PDF_CMAP_SINGLE, 19521 },
	{ 0x721a, 0x721a, PDF_CMAP_SINGLE, 21947 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 5598 },
	{ 0x721d, 0x721d, PDF_CMAP_SINGLE, 14771 },
	{ 0x721f, 0x721f, PDF_CMAP_SINGLE, 19522 },
	{ 0x7224, 0x7224, PDF_CMAP_SINGLE, 21948 },
	{ 0x7228, 0x7228, PDF_CMAP_SINGLE, 5599 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 3066 },
	{ 0x722b, 0x722b, PDF_CMAP_SINGLE, 14772 },
	{ 0x722c, 0x722c, PDF_CMAP_SINGLE, 5601 },
	{ 0x722d, 0x722d, PDF_CMAP_SINGLE, 5600 },
	{ 0x722f, 0x722f, PDF_CMAP_SINGLE, 21949 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 5602 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 5603 },
	{ 0x7234, 0x7234, PDF_CMAP_SINGLE, 18044 },
	{ 0x7235, 0x7235, PDF_CMAP_SINGLE, 2315 },
	{ 0x7236, 0x7236, PDF_CMAP_SINGLE, 3541 },
	{ 0x7238, 0x7238, PDF_CMAP_SINGLE, 14773 },
	{ 0x7239, 0x7239, PDF_CMAP_SINGLE, 18045 },
	{ 0x723a, 0x723a, PDF_CMAP_SINGLE, 3832 },
	{ 0x723b, 0x723c, PDF_CMAP_RANGE, 5604 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 2776 },
	{ 0x723e, 0x723e, PDF_CMAP_SINGLE, 2256 },
	{ 0x723f, 0x7240, PDF_CMAP_RANGE, 5606 },
	{ 0x7241, 0x7241, PDF_CMAP_SINGLE, 14774 },
	{ 0x7242, 0x7242, PDF_CMAP_SINGLE, 18046 },
	{ 0x7243, 0x7243, PDF_CMAP_SINGLE, 19523 },
	{ 0x7245, 0x7245, PDF_CMAP_SINGLE, 21950 },
	{ 0x7246, 0x7246, PDF_CMAP_SINGLE, 5608 },
	{ 0x7247, 0x7247, PDF_CMAP_SINGLE, 3618 },
	{ 0x7248, 0x7248, PDF_CMAP_SINGLE, 3419 },
	{ 0x724b, 0x724b, PDF_CMAP_SINGLE, 5609 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 3341 },
	{ 0x724e, 0x724e, PDF_CMAP_SINGLE, 21951 },
	{ 0x724f, 0x7250, PDF_CMAP_RANGE, 19524 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 3017 },
	{ 0x7253, 0x7253, PDF_CMAP_SINGLE, 14775 },
	{ 0x7255, 0x7256, PDF_CMAP_RANGE, 14776 },
	{ 0x7257, 0x7257, PDF_CMAP_SINGLE, 18047 },
	{ 0x7258, 0x7258, PDF_CMAP_SINGLE, 5610 },
	{ 0x7259, 0x7259, PDF_CMAP_SINGLE, 1383 },
	{ 0x725a, 0x725a, PDF_CMAP_SINGLE, 19526 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 1671 },
	{ 0x725c, 0x725c, PDF_CMAP_SINGLE, 14778 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 3794 },
	{ 0x725e, 0x725e, PDF_CMAP_SINGLE, 21952 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 3778 },
	{ 0x7260, 0x7260, PDF_CMAP_SINGLE, 19527 },
	{ 0x7261, 0x7261, PDF_CMAP_SINGLE, 1332 },
	{ 0x7262, 0x7262, PDF_CMAP_SINGLE, 4058 },
	{ 0x7263, 0x7263, PDF_CMAP_SINGLE, 18048 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 3712 },
	{ 0x7268, 0x7268, PDF_CMAP_SINGLE, 19528 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 3578 },
	{ 0x726b, 0x726b, PDF_CMAP_SINGLE, 21953 },
	{ 0x726e, 0x726f, PDF_CMAP_RANGE, 18050 },
	{ 0x7271, 0x7271, PDF_CMAP_SINGLE, 21954 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 2651 },
	{ 0x7274, 0x7274, PDF_CMAP_SINGLE, 5611 },
	{ 0x7277, 0x7277, PDF_CMAP_SINGLE, 19529 },
	{ 0x7278, 0x7278, PDF_CMAP_SINGLE, 18052 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 3227 },
	{ 0x727b, 0x727c, PDF_CMAP_RANGE, 21955 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 1879 },
	{ 0x727e, 0x727e, PDF_CMAP_SINGLE, 5612 },
	{ 0x727f, 0x727f, PDF_CMAP_SINGLE, 18053 },
	{ 0x7280, 0x7280, PDF_CMAP_SINGLE, 2116 },
	{ 0x7281, 0x7281, PDF_CMAP_SINGLE, 5614 },
	{ 0x7282, 0x7282, PDF_CMAP_SINGLE, 5613 },
	{ 0x7284, 0x7284, PDF_CMAP_SINGLE, 19530 },
	{ 0x7287, 0x7287, PDF_CMAP_SINGLE, 5615 },
	{ 0x7289, 0x7289, PDF_CMAP_SINGLE, 21957 },
	{ 0x728d, 0x728d, PDF_CMAP_SINGLE, 14779 },
	{ 0x728e, 0x728e, PDF_CMAP_SINGLE, 18054 },
	{ 0x7292, 0x7292, PDF_CMAP_SINGLE, 5616 },
	{ 0x7293, 0x7293, PDF_CMAP_SINGLE, 21958 },
	{ 0x7296, 0x7296, PDF_CMAP_SINGLE, 5617 },
	{ 0x729b, 0x729b, PDF_CMAP_SINGLE, 16969 },
	{ 0x72a0, 0x72a0, PDF_CMAP_SINGLE, 1624 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 5618 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 5619 },
	{ 0x72a8, 0x72a8, PDF_CMAP_SINGLE, 21959 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 1880 },
	{ 0x72ad, 0x72ad, PDF_CMAP_SINGLE, 14780 },
	{ 0x72ae, 0x72ae, PDF_CMAP_SINGLE, 18056 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 3420 },
	{ 0x72b0, 0x72b0, PDF_CMAP_SINGLE, 18057 },
	{ 0x72b1, 0x72b1, PDF_CMAP_SINGLE, 8545 },
	{ 0x72b2, 0x72b2, PDF_CMAP_SINGLE, 5621 },
	{ 0x72b4, 0x72b4, PDF_CMAP_SINGLE, 14781 },
	{ 0x72b6, 0x72b6, PDF_CMAP_SINGLE, 2525 },
	{ 0x72b9, 0x72b9, PDF_CMAP_SINGLE, 5620 },
	{ 0x72be, 0x72be, PDF_CMAP_SINGLE, 8546 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 13355 },
	{ 0x72c1, 0x72c1, PDF_CMAP_SINGLE, 18058 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 1712 },
	{ 0x72c3, 0x72c3, PDF_CMAP_SINGLE, 5622 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 5624 },
	{ 0x72c6, 0x72c6, PDF_CMAP_SINGLE, 5623 },
	{ 0x72c7, 0x72c7, PDF_CMAP_SINGLE, 14782 },
	{ 0x72c9, 0x72c9, PDF_CMAP_SINGLE, 19531 },
	{ 0x72cc, 0x72cc, PDF_CMAP_SINGLE, 18060 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5625 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 1925 },
	{ 0x72d2, 0x72d2, PDF_CMAP_SINGLE, 5626 },
	{ 0x72d5, 0x72d6, PDF_CMAP_RANGE, 21960 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 1761 },
	{ 0x72d8, 0x72d8, PDF_CMAP_SINGLE, 21962 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 2754 },
	{ 0x72db, 0x72db, PDF_CMAP_SINGLE, 2063 },
	{ 0x72df, 0x72df, PDF_CMAP_SINGLE, 21963 },
	{ 0x72e0, 0x72e1, PDF_CMAP_RANGE, 5628 },
	{ 0x72e2, 0x72e2, PDF_CMAP_SINGLE, 5627 },
	{ 0x72e5, 0x72e5, PDF_CMAP_SINGLE, 19532 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 2329 },
	{ 0x72ec, 0x72ec, PDF_CMAP_SINGLE, 3232 },
	{ 0x72ed, 0x72ed, PDF_CMAP_SINGLE, 1713 },
	{ 0x72f3, 0x72f3, PDF_CMAP_SINGLE, 18063 },
	{ 0x72f4, 0x72f4, PDF_CMAP_SINGLE, 19533 },
	{ 0x72f7, 0x72f7, PDF_CMAP_SINGLE, 5631 },
	{ 0x72f8, 0x72f8, PDF_CMAP_SINGLE, 2922 },
	{ 0x72f9, 0x72f9, PDF_CMAP_SINGLE, 5630 },
	{ 0x72fa, 0x72fa, PDF_CMAP_SINGLE, 18064 },
	{ 0x72fb, 0x72fb, PDF_CMAP_SINGLE, 14783 },
	{ 0x72fc, 0x72fc, PDF_CMAP_SINGLE, 4059 },
	{ 0x72fd, 0x72fd, PDF_CMAP_SINGLE, 3352 },
	{ 0x72fe, 0x72fe, PDF_CMAP_SINGLE, 21964 },
	{ 0x7302, 0x7302, PDF_CMAP_SINGLE, 19534 },
	{ 0x7304, 0x7305, PDF_CMAP_RANGE, 14784 },
	{ 0x7307, 0x7307, PDF_CMAP_SINGLE, 18065 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 5634 },
	{ 0x730b, 0x730b, PDF_CMAP_SINGLE, 19535 },
	{ 0x730d, 0x730d, PDF_CMAP_SINGLE, 21965 },
	{ 0x7312, 0x7312, PDF_CMAP_SINGLE, 18066 },
	{ 0x7313, 0x7313, PDF_CMAP_SINGLE, 21966 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 5636 },
	{ 0x7317, 0x7317, PDF_CMAP_SINGLE, 5633 },
	{ 0x7318, 0x7319, PDF_CMAP_RANGE, 18067 },
	{ 0x731b, 0x731b, PDF_CMAP_SINGLE, 3808 },
	{ 0x731c, 0x731c, PDF_CMAP_SINGLE, 5635 },
	{ 0x731d, 0x731d, PDF_CMAP_SINGLE, 5637 },
	{ 0x731e, 0x731e, PDF_CMAP_SINGLE, 19536 },
	{ 0x731f, 0x731f, PDF_CMAP_SINGLE, 3980 },
	{ 0x7322, 0x7322, PDF_CMAP_SINGLE, 19537 },
	{ 0x7324, 0x7324, PDF_CMAP_SINGLE, 8547 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 5641 },
	{ 0x7327, 0x7327, PDF_CMAP_SINGLE, 16971 },
	{ 0x7328, 0x7328, PDF_CMAP_SINGLE, 14786 },
	{ 0x7329, 0x7329, PDF_CMAP_SINGLE, 5640 },
	{ 0x732a, 0x732a, PDF_CMAP_SINGLE, 2996 },
	{ 0x732b, 0x732b, PDF_CMAP_SINGLE, 3299 },
	{ 0x732c, 0x732c, PDF_CMAP_SINGLE, 18071 },
	{ 0x732e, 0x732e, PDF_CMAP_SINGLE, 1881 },
	{ 0x732f, 0x732f, PDF_CMAP_SINGLE, 5639 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 14787 },
	{ 0x7332, 0x7332, PDF_CMAP_SINGLE, 21967 },
	{ 0x7333, 0x7333, PDF_CMAP_SINGLE, 18072 },
	{ 0x7334, 0x7334, PDF_CMAP_SINGLE, 5638 },
	{ 0x7335, 0x7335, PDF_CMAP_SINGLE, 21968 },
	{ 0x7336, 0x7337, PDF_CMAP_RANGE, 3867 },
	{ 0x7339, 0x7339, PDF_CMAP_SINGLE, 18070 },
	{ 0x733a, 0x733b, PDF_CMAP_RANGE, 19538 },
	{ 0x733d, 0x733d, PDF_CMAP_SINGLE, 18073 },
	{ 0x733e, 0x733e, PDF_CMAP_SINGLE, 5642 },
	{ 0x733f, 0x733f, PDF_CMAP_SINGLE, 1296 },
	{ 0x7343, 0x7343, PDF_CMAP_SINGLE, 14788 },
	{ 0x7344, 0x7344, PDF_CMAP_SINGLE, 2056 },
	{ 0x7345, 0x7345, PDF_CMAP_SINGLE, 2224 },
	{ 0x734d, 0x734d, PDF_CMAP_SINGLE, 19540 },
	{ 0x734e, 0x734f, PDF_CMAP_RANGE, 5643 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 16972 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 18074 },
	{ 0x7356, 0x7356, PDF_CMAP_SINGLE, 21969 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 5646 },
	{ 0x7358, 0x7358, PDF_CMAP_SINGLE, 19541 },
	{ 0x735d, 0x7360, PDF_CMAP_RANGE, 21970 },
	{ 0x7363, 0x7363, PDF_CMAP_SINGLE, 2381 },
	{ 0x7366, 0x7366, PDF_CMAP_SINGLE, 16973 },
	{ 0x7367, 0x7367, PDF_CMAP_SINGLE, 19542 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 5648 },
	{ 0x7369, 0x7369, PDF_CMAP_SINGLE, 21974 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 5647 },
	{ 0x736b, 0x736b, PDF_CMAP_SINGLE, 18076 },
	{ 0x736c, 0x736c, PDF_CMAP_SINGLE, 14789 },
	{ 0x736e, 0x736f, PDF_CMAP_RANGE, 18078 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 5649 },
	{ 0x7371, 0x7371, PDF_CMAP_SINGLE, 18080 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 1451 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 5651 },
	{ 0x7377, 0x7377, PDF_CMAP_SINGLE, 8549 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 5650 },
	{ 0x7379, 0x7379, PDF_CMAP_SINGLE, 21975 },
	{ 0x737a, 0x737a, PDF_CMAP_SINGLE, 5653 },
	{ 0x737b, 0x737b, PDF_CMAP_SINGLE, 5652 },
	{ 0x737c, 0x737c, PDF_CMAP_SINGLE, 14790 },
	{ 0x7380, 0x7380, PDF_CMAP_SINGLE, 21976 },
	{ 0x7381, 0x7381, PDF_CMAP_SINGLE, 18081 },
	{ 0x7383, 0x7383, PDF_CMAP_SINGLE, 14791 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 1904 },
	{ 0x7385, 0x7386, PDF_CMAP_RANGE, 14792 },
	{ 0x7387, 0x7387, PDF_CMAP_SINGLE, 3952 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 1732 },
	{ 0x738a, 0x738a, PDF_CMAP_SINGLE, 18082 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 1318 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 21977 },
	{ 0x7390, 0x7390, PDF_CMAP_SINGLE, 21978 },
	{ 0x7393, 0x7393, PDF_CMAP_SINGLE, 21979 },
	{ 0x7394, 0x7394, PDF_CMAP_SINGLE, 18083 },
	{ 0x7395, 0x7395, PDF_CMAP_SINGLE, 14794 },
	{ 0x7396, 0x7396, PDF_CMAP_SINGLE, 1762 },
	{ 0x7397, 0x7397, PDF_CMAP_SINGLE, 21980 },
	{ 0x7398, 0x7398, PDF_CMAP_SINGLE, 18084 },
	{ 0x739c, 0x739c, PDF_CMAP_SINGLE, 18085 },
	{ 0x739e, 0x73a0, PDF_CMAP_RANGE, 14795 },
	{ 0x73a2, 0x73a2, PDF_CMAP_SINGLE, 16974 },
	{ 0x73a5, 0x73a5, PDF_CMAP_SINGLE, 18086 },
	{ 0x73a6, 0x73a6, PDF_CMAP_SINGLE, 14798 },
	{ 0x73a8, 0x73a8, PDF_CMAP_SINGLE, 15423 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 1565 },
	{ 0x73aa, 0x73aa, PDF_CMAP_SINGLE, 21981 },
	{ 0x73ab, 0x73ab, PDF_CMAP_SINGLE, 14799 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 21982 },
	{ 0x73b2, 0x73b2, PDF_CMAP_SINGLE, 4016 },
	{ 0x73b3, 0x73b3, PDF_CMAP_SINGLE, 5655 },
	{ 0x73b5, 0x73b5, PDF_CMAP_SINGLE, 14800 },
	{ 0x73b7, 0x73b7, PDF_CMAP_SINGLE, 14801 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 18087 },
	{ 0x73ba, 0x73ba, PDF_CMAP_SINGLE, 13802 },
	{ 0x73bb, 0x73bb, PDF_CMAP_SINGLE, 5657 },
	{ 0x73bc, 0x73bc, PDF_CMAP_SINGLE, 14802 },
	{ 0x73bd, 0x73bd, PDF_CMAP_SINGLE, 8550 },
	{ 0x73bf, 0x73bf, PDF_CMAP_SINGLE, 18088 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 5658 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 1361 },
	{ 0x73c5, 0x73c5, PDF_CMAP_SINGLE, 18089 },
	{ 0x73c6, 0x73c6, PDF_CMAP_SINGLE, 21983 },
	{ 0x73c8, 0x73c8, PDF_CMAP_SINGLE, 5654 },
	{ 0x73c9, 0x73c9, PDF_CMAP_SINGLE, 8551 },
	{ 0x73ca, 0x73ca, PDF_CMAP_SINGLE, 2183 },
	{ 0x73cb, 0x73cb, PDF_CMAP_SINGLE, 18090 },
	{ 0x73cc, 0x73cc, PDF_CMAP_SINGLE, 21984 },
	{ 0x73cd, 0x73cd, PDF_CMAP_SINGLE, 3037 },
	{ 0x73ce, 0x73ce, PDF_CMAP_SINGLE, 5656 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 14803 },
	{ 0x73d2, 0x73d2, PDF_CMAP_SINGLE, 8554 },
	{ 0x73d3, 0x73d3, PDF_CMAP_SINGLE, 21985 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 8552 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 14804 },
	{ 0x73dd, 0x73dd, PDF_CMAP_SINGLE, 21986 },
	{ 0x73de, 0x73de, PDF_CMAP_SINGLE, 5661 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 2330 },
	{ 0x73e1, 0x73e1, PDF_CMAP_SINGLE, 18091 },
	{ 0x73e3, 0x73e3, PDF_CMAP_SINGLE, 8553 },
	{ 0x73e4, 0x73e4, PDF_CMAP_SINGLE, 15417 },
	{ 0x73e5, 0x73e5, PDF_CMAP_SINGLE, 5659 },
	{ 0x73e6, 0x73e6, PDF_CMAP_SINGLE, 21987 },
	{ 0x73e7, 0x73e7, PDF_CMAP_SINGLE, 18092 },
	{ 0x73e9, 0x73e9, PDF_CMAP_SINGLE, 14805 },
	{ 0x73ea, 0x73ea, PDF_CMAP_SINGLE, 1812 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 3421 },
	{ 0x73ee, 0x73ee, PDF_CMAP_SINGLE, 5660 },
	{ 0x73f1, 0x73f1, PDF_CMAP_SINGLE, 5687 },
	{ 0x73f4, 0x73f4, PDF_CMAP_SINGLE, 14806 },
	{ 0x73f5, 0x73f5, PDF_CMAP_SINGLE, 8556 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 21988 },
	{ 0x73f8, 0x73f8, PDF_CMAP_SINGLE, 5666 },
	{ 0x73f9, 0x73f9, PDF_CMAP_SINGLE, 18093 },
	{ 0x73fa, 0x73fa, PDF_CMAP_SINGLE, 18095 },
	{ 0x73fb, 0x73fb, PDF_CMAP_SINGLE, 21989 },
	{ 0x73fd, 0x73fd, PDF_CMAP_SINGLE, 14807 },
	{ 0x73fe, 0x73fe, PDF_CMAP_SINGLE, 1905 },
	{ 0x73ff, 0x73ff, PDF_CMAP_SINGLE, 21990 },
	{ 0x7400, 0x7400, PDF_CMAP_SINGLE, 21991 },
	{ 0x7401, 0x7401, PDF_CMAP_SINGLE, 18096 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 1663 },
	{ 0x7404, 0x7404, PDF_CMAP_SINGLE, 14808 },
	{ 0x7405, 0x7405, PDF_CMAP_SINGLE, 5663 },
	{ 0x7406, 0x7406, PDF_CMAP_SINGLE, 3943 },
	{ 0x7407, 0x7407, PDF_CMAP_SINGLE, 8555 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 3960 },
	{ 0x740a, 0x740a, PDF_CMAP_SINGLE, 14809 },
	{ 0x7411, 0x7411, PDF_CMAP_SINGLE, 21992 },
	{ 0x7413, 0x7413, PDF_CMAP_SINGLE, 18094 },
	{ 0x741a, 0x741b, PDF_CMAP_RANGE, 14810 },
	{ 0x7421, 0x7421, PDF_CMAP_SINGLE, 15418 },
	{ 0x7422, 0x7422, PDF_CMAP_SINGLE, 2902 },
	{ 0x7424, 0x7424, PDF_CMAP_SINGLE, 14812 },
	{ 0x7425, 0x7425, PDF_CMAP_SINGLE, 5665 },
	{ 0x7426, 0x7426, PDF_CMAP_SINGLE, 8557 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 14813 },
	{ 0x7429, 0x7429, PDF_CMAP_SINGLE, 8559 },
	{ 0x742a, 0x742a, PDF_CMAP_SINGLE, 8558 },
	{ 0x742b, 0x742b, PDF_CMAP_SINGLE, 16975 },
	{ 0x742c, 0x742c, PDF_CMAP_SINGLE, 14814 },
	{ 0x742d, 0x742d, PDF_CMAP_SINGLE, 21993 },
	{ 0x742e, 0x742e, PDF_CMAP_SINGLE, 8560 },
	{ 0x742f, 0x7431, PDF_CMAP_RANGE, 14815 },
	{ 0x7432, 0x7432, PDF_CMAP_SINGLE, 5667 },
	{ 0x7433, 0x7433, PDF_CMAP_SINGLE, 3998 },
	{ 0x7434, 0x7434, PDF_CMAP_SINGLE, 1743 },
	{ 0x7435, 0x7435, PDF_CMAP_SINGLE, 3472 },
	{ 0x7436, 0x7436, PDF_CMAP_SINGLE, 3328 },
	{ 0x7439, 0x7439, PDF_CMAP_SINGLE, 14818 },
	{ 0x743a, 0x743a, PDF_CMAP_SINGLE, 5668 },
	{ 0x743f, 0x743f, PDF_CMAP_SINGLE, 5670 },
	{ 0x7440, 0x7440, PDF_CMAP_SINGLE, 18098 },
	{ 0x7441, 0x7441, PDF_CMAP_SINGLE, 5673 },
	{ 0x7443, 0x7443, PDF_CMAP_SINGLE, 18099 },
	{ 0x7444, 0x7444, PDF_CMAP_SINGLE, 14819 },
	{ 0x7446, 0x7446, PDF_CMAP_SINGLE, 16976 },
	{ 0x7447, 0x7447, PDF_CMAP_SINGLE, 14820 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 14821 },
	{ 0x744d, 0x744d, PDF_CMAP_SINGLE, 14822 },
	{ 0x7451, 0x7451, PDF_CMAP_SINGLE, 14823 },
	{ 0x7452, 0x7452, PDF_CMAP_SINGLE, 18100 },
	{ 0x7453, 0x7453, PDF_CMAP_SINGLE, 18097 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 5669 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 14824 },
	{ 0x7459, 0x7459, PDF_CMAP_SINGLE, 5672 },
	{ 0x745a, 0x745a, PDF_CMAP_SINGLE, 1950 },
	{ 0x745b, 0x745b, PDF_CMAP_SINGLE, 1263 },
	{ 0x745c, 0x745c, PDF_CMAP_SINGLE, 5674 },
	{ 0x745d, 0x745d, PDF_CMAP_SINGLE, 18101 },
	{ 0x745e, 0x745e, PDF_CMAP_SINGLE, 2614 },
	{ 0x745f, 0x745f, PDF_CMAP_SINGLE, 5671 },
	{ 0x7460, 0x7460, PDF_CMAP_SINGLE, 4004 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 16977 },
	{ 0x7463, 0x7463, PDF_CMAP_SINGLE, 5677 },
	{ 0x7464, 0x7464, PDF_CMAP_SINGLE, 7477 },
	{ 0x7466, 0x7466, PDF_CMAP_SINGLE, 14825 },
	{ 0x7467, 0x7468, PDF_CMAP_RANGE, 21994 },
	{ 0x7469, 0x7469, PDF_CMAP_SINGLE, 5675 },
	{ 0x746a, 0x746a, PDF_CMAP_SINGLE, 5678 },
	{ 0x746b, 0x746b, PDF_CMAP_SINGLE, 14826 },
	{ 0x746d, 0x746d, PDF_CMAP_SINGLE, 16978 },
	{ 0x746e, 0x746e, PDF_CMAP_SINGLE, 21996 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 5664 },
	{ 0x7470, 0x7470, PDF_CMAP_SINGLE, 5676 },
	{ 0x7471, 0x7471, PDF_CMAP_SINGLE, 14827 },
	{ 0x7472, 0x7472, PDF_CMAP_SINGLE, 19543 },
	{ 0x7473, 0x7473, PDF_CMAP_SINGLE, 2092 },
	{ 0x7476, 0x7476, PDF_CMAP_SINGLE, 5679 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 5680 },
	{ 0x7480, 0x7480, PDF_CMAP_SINGLE, 14828 },
	{ 0x7481, 0x7481, PDF_CMAP_SINGLE, 18102 },
	{ 0x7483, 0x7483, PDF_CMAP_SINGLE, 3944 },
	{ 0x7485, 0x7487, PDF_CMAP_RANGE, 14829 },
	{ 0x7488, 0x7488, PDF_CMAP_SINGLE, 18103 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 8562 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 5681 },
	{ 0x748f, 0x748f, PDF_CMAP_SINGLE, 21997 },
	{ 0x7490, 0x7490, PDF_CMAP_SINGLE, 14832 },
	{ 0x7491, 0x7491, PDF_CMAP_SINGLE, 21998 },
	{ 0x7492, 0x7492, PDF_CMAP_SINGLE, 18105 },
	{ 0x7497, 0x7497, PDF_CMAP_SINGLE, 18106 },
	{ 0x7498, 0x7498, PDF_CMAP_SINGLE, 14833 },
	{ 0x7499, 0x7499, PDF_CMAP_SINGLE, 18107 },
	{ 0x749a, 0x749a, PDF_CMAP_SINGLE, 21999 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 14834 },
	{ 0x749e, 0x749e, PDF_CMAP_SINGLE, 5682 },
	{ 0x749f, 0x749f, PDF_CMAP_SINGLE, 8563 },
	{ 0x74a0, 0x74a0, PDF_CMAP_SINGLE, 14835 },
	{ 0x74a1, 0x74a1, PDF_CMAP_SINGLE, 18108 },
	{ 0x74a2, 0x74a2, PDF_CMAP_SINGLE, 5662 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 14836 },
	{ 0x74a5, 0x74a5, PDF_CMAP_SINGLE, 18109 },
	{ 0x74a6, 0x74a6, PDF_CMAP_SINGLE, 16979 },
	{ 0x74a7, 0x74a7, PDF_CMAP_SINGLE, 5683 },
	{ 0x74a8, 0x74a8, PDF_CMAP_SINGLE, 14837 },
	{ 0x74a9, 0x74a9, PDF_CMAP_SINGLE, 16980 },
	{ 0x74aa, 0x74aa, PDF_CMAP_SINGLE, 18110 },
	{ 0x74ab, 0x74ab, PDF_CMAP_SINGLE, 14838 },
	{ 0x74ae, 0x74ae, PDF_CMAP_SINGLE, 22000 },
	{ 0x74af, 0x74af, PDF_CMAP_SINGLE, 19544 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 1536 },
	{ 0x74b1, 0x74b2, PDF_CMAP_RANGE, 22001 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 14839 },
	{ 0x74b9, 0x74b9, PDF_CMAP_SINGLE, 18111 },
	{ 0x74ba, 0x74ba, PDF_CMAP_SINGLE, 18113 },
	{ 0x74bb, 0x74bb, PDF_CMAP_SINGLE, 18112 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 2257 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 14840 },
	{ 0x74c8, 0x74c8, PDF_CMAP_SINGLE, 14841 },
	{ 0x74c9, 0x74c9, PDF_CMAP_SINGLE, 16981 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 5684 },
	{ 0x74cc, 0x74cc, PDF_CMAP_SINGLE, 22003 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 5685 },
	{ 0x74d0, 0x74d0, PDF_CMAP_SINGLE, 22004 },
	{ 0x74d3, 0x74d3, PDF_CMAP_SINGLE, 22005 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 5686 },
	{ 0x74d6, 0x74d6, PDF_CMAP_SINGLE, 18114 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 18115 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 14842 },
	{ 0x74db, 0x74db, PDF_CMAP_SINGLE, 22007 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 1245 },
	{ 0x74de, 0x74de, PDF_CMAP_SINGLE, 14843 },
	{ 0x74df, 0x74df, PDF_CMAP_SINGLE, 19545 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 5688 },
	{ 0x74e2, 0x74e2, PDF_CMAP_SINGLE, 3501 },
	{ 0x74e3, 0x74e3, PDF_CMAP_SINGLE, 5689 },
	{ 0x74e4, 0x74e4, PDF_CMAP_SINGLE, 19546 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 1504 },
	{ 0x74e7, 0x74e7, PDF_CMAP_SINGLE, 5690 },
	{ 0x74e8, 0x74e8, PDF_CMAP_SINGLE, 22008 },
	{ 0x74e9, 0x74e9, PDF_CMAP_SINGLE, 5691 },
	{ 0x74ea, 0x74ea, PDF_CMAP_SINGLE, 22009 },
	{ 0x74eb, 0x74eb, PDF_CMAP_SINGLE, 18116 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 5692 },
	{ 0x74ef, 0x74ef, PDF_CMAP_SINGLE, 14160 },
	{ 0x74f0, 0x74f1, PDF_CMAP_RANGE, 5694 },
	{ 0x74f2, 0x74f2, PDF_CMAP_SINGLE, 5693 },
	{ 0x74f4, 0x74f4, PDF_CMAP_SINGLE, 19547 },
	{ 0x74f6, 0x74f6, PDF_CMAP_SINGLE, 3525 },
	{ 0x74f7, 0x74f7, PDF_CMAP_SINGLE, 5697 },
	{ 0x74f8, 0x74f8, PDF_CMAP_SINGLE, 5696 },
	{ 0x74fa, 0x74fa, PDF_CMAP_SINGLE, 18118 },
	{ 0x74fb, 0x74fb, PDF_CMAP_SINGLE, 19548 },
	{ 0x74fc, 0x74fc, PDF_CMAP_SINGLE, 22011 },
	{ 0x74ff, 0x74ff, PDF_CMAP_SINGLE, 16982 },
	{ 0x7501, 0x7501, PDF_CMAP_SINGLE, 8564 },
	{ 0x7503, 0x7503, PDF_CMAP_SINGLE, 5699 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 5698 },
	{ 0x7505, 0x7505, PDF_CMAP_SINGLE, 5700 },
	{ 0x7506, 0x7506, PDF_CMAP_SINGLE, 22012 },
	{ 0x750c, 0x750c, PDF_CMAP_SINGLE, 5701 },
	{ 0x750d, 0x750d, PDF_CMAP_SINGLE, 5703 },
	{ 0x750e, 0x750e, PDF_CMAP_SINGLE, 5702 },
	{ 0x7511, 0x7511, PDF_CMAP_SINGLE, 2059 },
	{ 0x7512, 0x7512, PDF_CMAP_SINGLE, 22013 },
	{ 0x7513, 0x7513, PDF_CMAP_SINGLE, 5705 },
	{ 0x7515, 0x7515, PDF_CMAP_SINGLE, 5704 },
	{ 0x7516, 0x7516, PDF_CMAP_SINGLE, 19549 },
	{ 0x7517, 0x7517, PDF_CMAP_SINGLE, 16983 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 1537 },
	{ 0x751a, 0x751a, PDF_CMAP_SINGLE, 2585 },
	{ 0x751c, 0x751c, PDF_CMAP_SINGLE, 3126 },
	{ 0x751e, 0x751e, PDF_CMAP_SINGLE, 5706 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 2652 },
	{ 0x7520, 0x7520, PDF_CMAP_SINGLE, 18120 },
	{ 0x7521, 0x7521, PDF_CMAP_SINGLE, 19550 },
	{ 0x7522, 0x7522, PDF_CMAP_SINGLE, 13790 },
	{ 0x7523, 0x7523, PDF_CMAP_SINGLE, 2184 },
	{ 0x7524, 0x7524, PDF_CMAP_SINGLE, 18121 },
	{ 0x7525, 0x7525, PDF_CMAP_SINGLE, 1307 },
	{ 0x7526, 0x7526, PDF_CMAP_SINGLE, 5707 },
	{ 0x7527, 0x7527, PDF_CMAP_SINGLE, 22014 },
	{ 0x7528, 0x7528, PDF_CMAP_SINGLE, 3899 },
	{ 0x7529, 0x7529, PDF_CMAP_SINGLE, 22015 },
	{ 0x752a, 0x752a, PDF_CMAP_SINGLE, 18122 },
	{ 0x752b, 0x752b, PDF_CMAP_SINGLE, 3635 },
	{ 0x752c, 0x752c, PDF_CMAP_SINGLE, 5708 },
	{ 0x752f, 0x752f, PDF_CMAP_SINGLE, 8434 },
	{ 0x7530, 0x7530, PDF_CMAP_SINGLE, 3134 },
	{ 0x7531, 0x7531, PDF_CMAP_SINGLE, 3869 },
	{ 0x7532, 0x7532, PDF_CMAP_SINGLE, 2005 },
	{ 0x7533, 0x7533, PDF_CMAP_SINGLE, 2563 },
	{ 0x7536, 0x7536, PDF_CMAP_SINGLE, 22016 },
	{ 0x7537, 0x7537, PDF_CMAP_SINGLE, 2953 },
	{ 0x7538, 0x7538, PDF_CMAP_SINGLE, 4297 },
	{ 0x7539, 0x7539, PDF_CMAP_SINGLE, 22017 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 3018 },
	{ 0x753b, 0x753b, PDF_CMAP_SINGLE, 1384 },
	{ 0x753c, 0x753c, PDF_CMAP_SINGLE, 5709 },
	{ 0x753d, 0x753e, PDF_CMAP_RANGE, 18125 },
	{ 0x753f, 0x753f, PDF_CMAP_SINGLE, 19551 },
	{ 0x7540, 0x7540, PDF_CMAP_SINGLE, 18127 },
	{ 0x7543, 0x7543, PDF_CMAP_SINGLE, 22018 },
	{ 0x7544, 0x7544, PDF_CMAP_SINGLE, 5710 },
	{ 0x7546, 0x7546, PDF_CMAP_SINGLE, 5715 },
	{ 0x7547, 0x7547, PDF_CMAP_SINGLE, 22019 },
	{ 0x7548, 0x7548, PDF_CMAP_SINGLE, 18128 },
	{ 0x7549, 0x7549, PDF_CMAP_SINGLE, 5713 },
	{ 0x754a, 0x754a, PDF_CMAP_SINGLE, 5712 },
	{ 0x754b, 0x754b, PDF_CMAP_SINGLE, 5063 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 1412 },
	{ 0x754d, 0x754d, PDF_CMAP_SINGLE, 5711 },
	{ 0x754e, 0x754e, PDF_CMAP_SINGLE, 14844 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 1182 },
	{ 0x7550, 0x7550, PDF_CMAP_SINGLE, 18129 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 3390 },
	{ 0x7552, 0x7552, PDF_CMAP_SINGLE, 18130 },
	{ 0x7554, 0x7554, PDF_CMAP_SINGLE, 3422 },
	{ 0x7557, 0x7557, PDF_CMAP_SINGLE, 22020 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 3961 },
	{ 0x755a, 0x755a, PDF_CMAP_SINGLE, 5716 },
	{ 0x755b, 0x755b, PDF_CMAP_SINGLE, 5714 },
	{ 0x755c, 0x755c, PDF_CMAP_SINGLE, 2970 },
	{ 0x755d, 0x755d, PDF_CMAP_SINGLE, 2634 },
	{ 0x755e, 0x755e, PDF_CMAP_SINGLE, 19552 },
	{ 0x755f, 0x755f, PDF_CMAP_SINGLE, 22021 },
	{ 0x7560, 0x7560, PDF_CMAP_SINGLE, 3391 },
	{ 0x7561, 0x7561, PDF_CMAP_SINGLE, 22022 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 3487 },
	{ 0x7564, 0x7564, PDF_CMAP_SINGLE, 5718 },
	{ 0x7565, 0x7565, PDF_CMAP_SINGLE, 3956 },
	{ 0x7566, 0x7566, PDF_CMAP_SINGLE, 1827 },
	{ 0x7567, 0x7567, PDF_CMAP_SINGLE, 5719 },
	{ 0x7569, 0x7569, PDF_CMAP_SINGLE, 5717 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 3434 },
	{ 0x756b, 0x756b, PDF_CMAP_SINGLE, 5720 },
	{ 0x756c, 0x756c, PDF_CMAP_SINGLE, 15419 },
	{ 0x756d, 0x756d, PDF_CMAP_SINGLE, 5721 },
	{ 0x756f, 0x756f, PDF_CMAP_SINGLE, 8565 },
	{ 0x7570, 0x7570, PDF_CMAP_SINGLE, 1183 },
	{ 0x7571, 0x7571, PDF_CMAP_SINGLE, 18132 },
	{ 0x7572, 0x7572, PDF_CMAP_SINGLE, 18131 },
	{ 0x7573, 0x7573, PDF_CMAP_SINGLE, 2526 },
	{ 0x7574, 0x7574, PDF_CMAP_SINGLE, 5726 },
	{ 0x7575, 0x7575, PDF_CMAP_SINGLE, 14161 },
	{ 0x7576, 0x7576, PDF_CMAP_SINGLE, 5723 },
	{ 0x7577, 0x7577, PDF_CMAP_SINGLE, 3269 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 5722 },
	{ 0x7579, 0x7579, PDF_CMAP_SINGLE, 14845 },
	{ 0x757a, 0x757a, PDF_CMAP_SINGLE, 18133 },
	{ 0x757b, 0x757c, PDF_CMAP_RANGE, 22023 },
	{ 0x757d, 0x757e, PDF_CMAP_RANGE, 18134 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 1600 },
	{ 0x7581, 0x7581, PDF_CMAP_SINGLE, 14846 },
	{ 0x7582, 0x7582, PDF_CMAP_SINGLE, 5729 },
	{ 0x7585, 0x7585, PDF_CMAP_SINGLE, 22025 },
	{ 0x7586, 0x7587, PDF_CMAP_RANGE, 5724 },
	{ 0x7589, 0x7589, PDF_CMAP_SINGLE, 5728 },
	{ 0x758a, 0x758a, PDF_CMAP_SINGLE, 5727 },
	{ 0x758b, 0x758b, PDF_CMAP_SINGLE, 3479 },
	{ 0x758c, 0x758c, PDF_CMAP_SINGLE, 18136 },
	{ 0x758e, 0x758e, PDF_CMAP_SINGLE, 2756 },
	{ 0x758f, 0x758f, PDF_CMAP_SINGLE, 2755 },
	{ 0x7590, 0x7590, PDF_CMAP_SINGLE, 14847 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 1625 },
	{ 0x7592, 0x7593, PDF_CMAP_RANGE, 14848 },
	{ 0x7594, 0x7594, PDF_CMAP_SINGLE, 5730 },
	{ 0x7595, 0x7595, PDF_CMAP_SINGLE, 22026 },
	{ 0x7599, 0x7599, PDF_CMAP_SINGLE, 19553 },
	{ 0x759a, 0x759a, PDF_CMAP_SINGLE, 5731 },
	{ 0x759c, 0x759c, PDF_CMAP_SINGLE, 22027 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 5732 },
	{ 0x75a2, 0x75a2, PDF_CMAP_SINGLE, 18138 },
	{ 0x75a3, 0x75a3, PDF_CMAP_SINGLE, 5734 },
	{ 0x75a4, 0x75a4, PDF_CMAP_SINGLE, 19554 },
	{ 0x75a5, 0x75a5, PDF_CMAP_SINGLE, 5733 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 1272 },
	{ 0x75b0, 0x75b0, PDF_CMAP_SINGLE, 18140 },
	{ 0x75b1, 0x75b1, PDF_CMAP_SINGLE, 5742 },
	{ 0x75b2, 0x75b2, PDF_CMAP_SINGLE, 3452 },
	{ 0x75b3, 0x75b3, PDF_CMAP_SINGLE, 5736 },
	{ 0x75b4, 0x75b4, PDF_CMAP_SINGLE, 14850 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 5738 },
	{ 0x75b7, 0x75b7, PDF_CMAP_SINGLE, 18141 },
	{ 0x75b8, 0x75b8, PDF_CMAP_SINGLE, 5740 },
	{ 0x75b9, 0x75b9, PDF_CMAP_SINGLE, 2564 },
	{ 0x75ba, 0x75ba, PDF_CMAP_SINGLE, 22028 },
	{ 0x75bc, 0x75bc, PDF_CMAP_SINGLE, 5741 },
	{ 0x75bd, 0x75bd, PDF_CMAP_SINGLE, 5739 },
	{ 0x75be, 0x75be, PDF_CMAP_SINGLE, 2284 },
	{ 0x75bf, 0x75c0, PDF_CMAP_RANGE, 18142 },
	{ 0x75c1, 0x75c1, PDF_CMAP_SINGLE, 19555 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 5735 },
	{ 0x75c3, 0x75c3, PDF_CMAP_SINGLE, 5737 },
	{ 0x75c4, 0x75c4, PDF_CMAP_SINGLE, 19556 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 3508 },
	{ 0x75c6, 0x75c6, PDF_CMAP_SINGLE, 18144 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 2481 },
	{ 0x75ca, 0x75ca, PDF_CMAP_SINGLE, 5744 },
	{ 0x75cc, 0x75cc, PDF_CMAP_SINGLE, 19557 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 5743 },
	{ 0x75ce, 0x75ce, PDF_CMAP_SINGLE, 16985 },
	{ 0x75cf, 0x75cf, PDF_CMAP_SINGLE, 18145 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 5745 },
	{ 0x75d3, 0x75d3, PDF_CMAP_SINGLE, 18146 },
	{ 0x75d4, 0x75d4, PDF_CMAP_SINGLE, 2258 },
	{ 0x75d5, 0x75d5, PDF_CMAP_SINGLE, 2079 },
	{ 0x75d7, 0x75d7, PDF_CMAP_SINGLE, 19558 },
	{ 0x75d8, 0x75d8, PDF_CMAP_SINGLE, 3185 },
	{ 0x75d9, 0x75d9, PDF_CMAP_SINGLE, 5746 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 3047 },
	{ 0x75dc, 0x75dc, PDF_CMAP_SINGLE, 19559 },
	{ 0x75dd, 0x75dd, PDF_CMAP_SINGLE, 18147 },
	{ 0x75de, 0x75de, PDF_CMAP_SINGLE, 5748 },
	{ 0x75df, 0x75e0, PDF_CMAP_RANGE, 18148 },
	{ 0x75e1, 0x75e1, PDF_CMAP_SINGLE, 19560 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 3945 },
	{ 0x75e3, 0x75e3, PDF_CMAP_SINGLE, 5747 },
	{ 0x75e4, 0x75e4, PDF_CMAP_SINGLE, 14851 },
	{ 0x75e7, 0x75e7, PDF_CMAP_SINGLE, 18150 },
	{ 0x75e9, 0x75e9, PDF_CMAP_SINGLE, 2795 },
	{ 0x75ec, 0x75ec, PDF_CMAP_SINGLE, 14162 },
	{ 0x75ee, 0x75ee, PDF_CMAP_SINGLE, 18151 },
	{ 0x75ef, 0x75ef, PDF_CMAP_SINGLE, 19561 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 5753 },
	{ 0x75f1, 0x75f1, PDF_CMAP_SINGLE, 18152 },
	{ 0x75f2, 0x75f3, PDF_CMAP_RANGE, 5755 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 2962 },
	{ 0x75f9, 0x75f9, PDF_CMAP_SINGLE, 14852 },
	{ 0x75fa, 0x75fa, PDF_CMAP_SINGLE, 5754 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 5751 },
	{ 0x75fe, 0x75ff, PDF_CMAP_RANGE, 5749 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 14853 },
	{ 0x7601, 0x7601, PDF_CMAP_SINGLE, 5752 },
	{ 0x7602, 0x7602, PDF_CMAP_SINGLE, 16986 },
	{ 0x7603, 0x7603, PDF_CMAP_SINGLE, 18153 },
	{ 0x7604, 0x7604, PDF_CMAP_SINGLE, 19562 },
	{ 0x7607, 0x7607, PDF_CMAP_SINGLE, 18155 },
	{ 0x7608, 0x7608, PDF_CMAP_SINGLE, 16987 },
	{ 0x7609, 0x7609, PDF_CMAP_SINGLE, 5759 },
	{ 0x760a, 0x760a, PDF_CMAP_SINGLE, 14854 },
	{ 0x760b, 0x760b, PDF_CMAP_SINGLE, 5757 },
	{ 0x760c, 0x760c, PDF_CMAP_SINGLE, 19563 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 5758 },
	{ 0x760f, 0x760f, PDF_CMAP_SINGLE, 18156 },
	{ 0x7612, 0x7612, PDF_CMAP_SINGLE, 22029 },
	{ 0x7613, 0x7613, PDF_CMAP_SINGLE, 18159 },
	{ 0x7615, 0x7616, PDF_CMAP_RANGE, 14855 },
	{ 0x7618, 0x7618, PDF_CMAP_SINGLE, 18154 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 14857 },
	{ 0x761b, 0x761c, PDF_CMAP_RANGE, 18160 },
	{ 0x761d, 0x761d, PDF_CMAP_SINGLE, 19564 },
	{ 0x761e, 0x761e, PDF_CMAP_SINGLE, 14858 },
	{ 0x761f, 0x761f, PDF_CMAP_SINGLE, 5760 },
	{ 0x7620, 0x7622, PDF_CMAP_RANGE, 5762 },
	{ 0x7623, 0x7623, PDF_CMAP_SINGLE, 22030 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 5765 },
	{ 0x7625, 0x7625, PDF_CMAP_SINGLE, 18163 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 13893 },
	{ 0x7627, 0x7627, PDF_CMAP_SINGLE, 5761 },
	{ 0x7628, 0x7628, PDF_CMAP_SINGLE, 18164 },
	{ 0x7629, 0x7629, PDF_CMAP_SINGLE, 22031 },
	{ 0x762d, 0x762d, PDF_CMAP_SINGLE, 14859 },
	{ 0x7630, 0x7630, PDF_CMAP_SINGLE, 5767 },
	{ 0x7632, 0x7632, PDF_CMAP_SINGLE, 19565 },
	{ 0x7633, 0x7633, PDF_CMAP_SINGLE, 18166 },
	{ 0x7634, 0x7634, PDF_CMAP_SINGLE, 5766 },
	{ 0x7635, 0x7635, PDF_CMAP_SINGLE, 14860 },
	{ 0x7638, 0x7638, PDF_CMAP_SINGLE, 19566 },
	{ 0x7639, 0x763a, PDF_CMAP_RANGE, 22032 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 5768 },
	{ 0x763c, 0x763c, PDF_CMAP_SINGLE, 18165 },
	{ 0x7640, 0x7640, PDF_CMAP_SINGLE, 22034 },
	{ 0x7641, 0x7641, PDF_CMAP_SINGLE, 18169 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 3981 },
	{ 0x7643, 0x7643, PDF_CMAP_SINGLE, 14861 },
	{ 0x7644, 0x7644, PDF_CMAP_SINGLE, 22035 },
	{ 0x7645, 0x7645, PDF_CMAP_SINGLE, 19567 },
	{ 0x7646, 0x7646, PDF_CMAP_SINGLE, 5771 },
	{ 0x7647, 0x7648, PDF_CMAP_RANGE, 5769 },
	{ 0x7649, 0x7649, PDF_CMAP_SINGLE, 18171 },
	{ 0x764a, 0x764a, PDF_CMAP_SINGLE, 19568 },
	{ 0x764b, 0x764b, PDF_CMAP_SINGLE, 14862 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 1566 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 14163 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 3850 },
	{ 0x7655, 0x7655, PDF_CMAP_SINGLE, 18172 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 3610 },
	{ 0x7658, 0x7658, PDF_CMAP_SINGLE, 5773 },
	{ 0x7659, 0x7659, PDF_CMAP_SINGLE, 22036 },
	{ 0x765c, 0x765c, PDF_CMAP_SINGLE, 5772 },
	{ 0x765f, 0x765f, PDF_CMAP_SINGLE, 19569 },
	{ 0x7661, 0x7662, PDF_CMAP_RANGE, 5774 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 16988 },
	{ 0x7665, 0x7665, PDF_CMAP_SINGLE, 14863 },
	{ 0x7667, 0x7667, PDF_CMAP_SINGLE, 5779 },
	{ 0x7668, 0x766a, PDF_CMAP_RANGE, 5776 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 5780 },
	{ 0x766d, 0x766d, PDF_CMAP_SINGLE, 14864 },
	{ 0x766e, 0x766e, PDF_CMAP_SINGLE, 18174 },
	{ 0x766f, 0x766f, PDF_CMAP_SINGLE, 14865 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 5781 },
	{ 0x7671, 0x7671, PDF_CMAP_SINGLE, 14866 },
	{ 0x7672, 0x7672, PDF_CMAP_SINGLE, 5782 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 14867 },
	{ 0x7676, 0x7676, PDF_CMAP_SINGLE, 5783 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 5784 },
	{ 0x767a, 0x767a, PDF_CMAP_SINGLE, 3395 },
	{ 0x767b, 0x767b, PDF_CMAP_SINGLE, 3146 },
	{ 0x767c, 0x767c, PDF_CMAP_SINGLE, 5785 },
	{ 0x767d, 0x767d, PDF_CMAP_SINGLE, 3368 },
	{ 0x767e, 0x767e, PDF_CMAP_SINGLE, 3494 },
	{ 0x7680, 0x7680, PDF_CMAP_SINGLE, 5786 },
	{ 0x7681, 0x7681, PDF_CMAP_SINGLE, 16989 },
	{ 0x7682, 0x7682, PDF_CMAP_SINGLE, 8566 },
	{ 0x7683, 0x7683, PDF_CMAP_SINGLE, 5787 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 3108 },
	{ 0x7685, 0x7685, PDF_CMAP_SINGLE, 22037 },
	{ 0x7686, 0x7686, PDF_CMAP_SINGLE, 1413 },
	{ 0x7687, 0x7687, PDF_CMAP_SINGLE, 2006 },
	{ 0x7688, 0x7688, PDF_CMAP_SINGLE, 5788 },
	{ 0x768b, 0x768b, PDF_CMAP_SINGLE, 5789 },
	{ 0x768c, 0x768d, PDF_CMAP_RANGE, 22038 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 5790 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 2167 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 5792 },
	{ 0x7695, 0x7695, PDF_CMAP_SINGLE, 18175 },
	{ 0x7696, 0x7696, PDF_CMAP_SINGLE, 5791 },
	{ 0x7699, 0x769a, PDF_CMAP_RANGE, 5793 },
	{ 0x769b, 0x769b, PDF_CMAP_SINGLE, 8569 },
	{ 0x769c, 0x769c, PDF_CMAP_SINGLE, 8567 },
	{ 0x769d, 0x769d, PDF_CMAP_SINGLE, 16990 },
	{ 0x769e, 0x769e, PDF_CMAP_SINGLE, 8568 },
	{ 0x769f, 0x769f, PDF_CMAP_SINGLE, 22040 },
	{ 0x76a0, 0x76a0, PDF_CMAP_SINGLE, 18177 },
	{ 0x76a1, 0x76a1, PDF_CMAP_SINGLE, 18176 },
	{ 0x76a2, 0x76a3, PDF_CMAP_RANGE, 22041 },
	{ 0x76a4, 0x76a5, PDF_CMAP_RANGE, 14868 },
	{ 0x76a6, 0x76a6, PDF_CMAP_SINGLE, 8570 },
	{ 0x76a7, 0x76a8, PDF_CMAP_RANGE, 18178 },
	{ 0x76aa, 0x76aa, PDF_CMAP_SINGLE, 16991 },
	{ 0x76ad, 0x76ad, PDF_CMAP_SINGLE, 19570 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 3453 },
	{ 0x76af, 0x76af, PDF_CMAP_SINGLE, 18180 },
	{ 0x76b0, 0x76b0, PDF_CMAP_SINGLE, 5795 },
	{ 0x76b4, 0x76b4, PDF_CMAP_SINGLE, 5796 },
	{ 0x76b6, 0x76b6, PDF_CMAP_SINGLE, 16992 },
	{ 0x76b7, 0x76b7, PDF_CMAP_SINGLE, 7452 },
	{ 0x76b8, 0x76ba, PDF_CMAP_RANGE, 5797 },
	{ 0x76bd, 0x76bd, PDF_CMAP_SINGLE, 19571 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 2172 },
	{ 0x76c1, 0x76c1, PDF_CMAP_SINGLE, 22043 },
	{ 0x76c2, 0x76c2, PDF_CMAP_SINGLE, 5800 },
	{ 0x76c3, 0x76c3, PDF_CMAP_SINGLE, 3340 },
	{ 0x76c5, 0x76c5, PDF_CMAP_SINGLE, 14870 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 3725 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 1264 },
	{ 0x76c9, 0x76c9, PDF_CMAP_SINGLE, 18182 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 1273 },
	{ 0x76cb, 0x76cb, PDF_CMAP_SINGLE, 22044 },
	{ 0x76cc, 0x76cc, PDF_CMAP_SINGLE, 14871 },
	{ 0x76cd, 0x76cd, PDF_CMAP_SINGLE, 5801 },
	{ 0x76ce, 0x76ce, PDF_CMAP_SINGLE, 16993 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 5803 },
	{ 0x76d4, 0x76d4, PDF_CMAP_SINGLE, 16994 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 5802 },
	{ 0x76d7, 0x76d7, PDF_CMAP_SINGLE, 3178 },
	{ 0x76d9, 0x76d9, PDF_CMAP_SINGLE, 19572 },
	{ 0x76db, 0x76db, PDF_CMAP_SINGLE, 2653 },
	{ 0x76dc, 0x76dc, PDF_CMAP_SINGLE, 5336 },
	{ 0x76de, 0x76de, PDF_CMAP_SINGLE, 5804 },
	{ 0x76df, 0x76df, PDF_CMAP_SINGLE, 3789 },
	{ 0x76e0, 0x76e0, PDF_CMAP_SINGLE, 22046 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 5805 },
	{ 0x76e3, 0x76e3, PDF_CMAP_SINGLE, 1538 },
	{ 0x76e4, 0x76e4, PDF_CMAP_SINGLE, 3435 },
	{ 0x76e5, 0x76e5, PDF_CMAP_SINGLE, 5806 },
	{ 0x76e6, 0x76e6, PDF_CMAP_SINGLE, 16995 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 5807 },
	{ 0x76e8, 0x76e8, PDF_CMAP_SINGLE, 18184 },
	{ 0x76ea, 0x76ea, PDF_CMAP_SINGLE, 5808 },
	{ 0x76eb, 0x76eb, PDF_CMAP_SINGLE, 19573 },
	{ 0x76ec, 0x76ec, PDF_CMAP_SINGLE, 14872 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 3816 },
	{ 0x76f0, 0x76f0, PDF_CMAP_SINGLE, 19574 },
	{ 0x76f1, 0x76f1, PDF_CMAP_SINGLE, 16996 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 3809 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 3034 },
	{ 0x76f6, 0x76f6, PDF_CMAP_SINGLE, 22047 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 2796 },
	{ 0x76f9, 0x76f9, PDF_CMAP_SINGLE, 19575 },
	{ 0x76fb, 0x76fb, PDF_CMAP_SINGLE, 5810 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 14873 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 2412 },
	{ 0x7700, 0x7700, PDF_CMAP_SINGLE, 19576 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 2482 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 5813 },
	{ 0x7706, 0x7706, PDF_CMAP_SINGLE, 22048 },
	{ 0x7707, 0x7707, PDF_CMAP_SINGLE, 5812 },
	{ 0x7708, 0x7708, PDF_CMAP_SINGLE, 5811 },
	{ 0x7709, 0x7709, PDF_CMAP_SINGLE, 3473 },
	{ 0x770a, 0x770a, PDF_CMAP_SINGLE, 16997 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 1539 },
	{ 0x770c, 0x770c, PDF_CMAP_SINGLE, 1885 },
	{ 0x770e, 0x770e, PDF_CMAP_SINGLE, 19577 },
	{ 0x7712, 0x7712, PDF_CMAP_SINGLE, 22049 },
	{ 0x7714, 0x7715, PDF_CMAP_RANGE, 22050 },
	{ 0x7717, 0x7717, PDF_CMAP_SINGLE, 18186 },
	{ 0x7719, 0x7719, PDF_CMAP_SINGLE, 16998 },
	{ 0x771a, 0x771a, PDF_CMAP_SINGLE, 18187 },
	{ 0x771b, 0x771b, PDF_CMAP_SINGLE, 5819 },
	{ 0x771c, 0x771c, PDF_CMAP_SINGLE, 22052 },
	{ 0x771e, 0x771e, PDF_CMAP_SINGLE, 5816 },
	{ 0x771f, 0x771f, PDF_CMAP_SINGLE, 2565 },
	{ 0x7720, 0x7720, PDF_CMAP_SINGLE, 3774 },
	{ 0x7722, 0x7722, PDF_CMAP_SINGLE, 19578 },
	{ 0x7724, 0x7724, PDF_CMAP_SINGLE, 5815 },
	{ 0x7725, 0x7726, PDF_CMAP_RANGE, 5817 },
	{ 0x7728, 0x7728, PDF_CMAP_SINGLE, 19579 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 5814 },
	{ 0x772d, 0x772d, PDF_CMAP_SINGLE, 18188 },
	{ 0x772e, 0x772e, PDF_CMAP_SINGLE, 22053 },
	{ 0x772f, 0x772f, PDF_CMAP_SINGLE, 19580 },
	{ 0x7734, 0x7734, PDF_CMAP_SINGLE, 14874 },
	{ 0x7735, 0x7735, PDF_CMAP_SINGLE, 18189 },
	{ 0x7736, 0x7736, PDF_CMAP_SINGLE, 14875 },
	{ 0x7737, 0x7738, PDF_CMAP_RANGE, 5820 },
	{ 0x7739, 0x7739, PDF_CMAP_SINGLE, 19581 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 3019 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 1567 },
	{ 0x773d, 0x773d, PDF_CMAP_SINGLE, 22054 },
	{ 0x773e, 0x773e, PDF_CMAP_SINGLE, 19582 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 2979 },
	{ 0x7742, 0x7742, PDF_CMAP_SINGLE, 22055 },
	{ 0x7745, 0x7745, PDF_CMAP_SINGLE, 19583 },
	{ 0x7746, 0x7746, PDF_CMAP_SINGLE, 8572 },
	{ 0x7747, 0x7747, PDF_CMAP_SINGLE, 5822 },
	{ 0x774a, 0x774a, PDF_CMAP_SINGLE, 19584 },
	{ 0x774d, 0x774e, PDF_CMAP_RANGE, 16999 },
	{ 0x774f, 0x774f, PDF_CMAP_SINGLE, 19585 },
	{ 0x7752, 0x7752, PDF_CMAP_SINGLE, 22056 },
	{ 0x7756, 0x7757, PDF_CMAP_RANGE, 22057 },
	{ 0x7758, 0x7758, PDF_CMAP_SINGLE, 18194 },
	{ 0x775a, 0x775a, PDF_CMAP_SINGLE, 5823 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 5826 },
	{ 0x775c, 0x775c, PDF_CMAP_SINGLE, 14876 },
	{ 0x775e, 0x775e, PDF_CMAP_SINGLE, 19586 },
	{ 0x775f, 0x7760, PDF_CMAP_RANGE, 14877 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 2605 },
	{ 0x7762, 0x7762, PDF_CMAP_SINGLE, 7877 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 3228 },
	{ 0x7764, 0x7764, PDF_CMAP_SINGLE, 19587 },
	{ 0x7765, 0x7765, PDF_CMAP_SINGLE, 5827 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 3713 },
	{ 0x7767, 0x7767, PDF_CMAP_SINGLE, 19588 },
	{ 0x7768, 0x7768, PDF_CMAP_SINGLE, 5824 },
	{ 0x776a, 0x776a, PDF_CMAP_SINGLE, 14165 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 5825 },
	{ 0x776c, 0x776c, PDF_CMAP_SINGLE, 19589 },
	{ 0x7770, 0x7770, PDF_CMAP_SINGLE, 22059 },
	{ 0x7772, 0x7772, PDF_CMAP_SINGLE, 14879 },
	{ 0x7773, 0x7774, PDF_CMAP_RANGE, 22060 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 5830 },
	{ 0x777a, 0x777a, PDF_CMAP_SINGLE, 17001 },
	{ 0x777c, 0x777c, PDF_CMAP_SINGLE, 18196 },
	{ 0x777d, 0x777d, PDF_CMAP_SINGLE, 14880 },
	{ 0x777e, 0x777e, PDF_CMAP_SINGLE, 5829 },
	{ 0x777f, 0x777f, PDF_CMAP_SINGLE, 5828 },
	{ 0x7780, 0x7780, PDF_CMAP_SINGLE, 17002 },
	{ 0x7784, 0x7784, PDF_CMAP_SINGLE, 19590 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 5832 },
	{ 0x778c, 0x778d, PDF_CMAP_RANGE, 19591 },
	{ 0x778e, 0x778e, PDF_CMAP_SINGLE, 5831 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 5833 },
	{ 0x7794, 0x7794, PDF_CMAP_SINGLE, 17003 },
	{ 0x7795, 0x7795, PDF_CMAP_SINGLE, 14881 },
	{ 0x7796, 0x7796, PDF_CMAP_SINGLE, 19593 },
	{ 0x779a, 0x779a, PDF_CMAP_SINGLE, 18199 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 5835 },
	{ 0x779f, 0x779f, PDF_CMAP_SINGLE, 18200 },
	{ 0x77a0, 0x77a0, PDF_CMAP_SINGLE, 5834 },
	{ 0x77a2, 0x77a2, PDF_CMAP_SINGLE, 18201 },
	{ 0x77a4, 0x77a4, PDF_CMAP_SINGLE, 18202 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 3613 },
	{ 0x77a7, 0x77a7, PDF_CMAP_SINGLE, 19594 },
	{ 0x77a9, 0x77a9, PDF_CMAP_SINGLE, 14166 },
	{ 0x77aa, 0x77aa, PDF_CMAP_SINGLE, 14882 },
	{ 0x77ac, 0x77ac, PDF_CMAP_SINGLE, 2400 },
	{ 0x77ad, 0x77ad, PDF_CMAP_SINGLE, 3982 },
	{ 0x77ae, 0x77ae, PDF_CMAP_SINGLE, 22064 },
	{ 0x77af, 0x77af, PDF_CMAP_SINGLE, 19595 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 5836 },
	{ 0x77b1, 0x77b1, PDF_CMAP_SINGLE, 22065 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 3215 },
	{ 0x77b5, 0x77b5, PDF_CMAP_SINGLE, 22066 },
	{ 0x77b6, 0x77b6, PDF_CMAP_SINGLE, 5837 },
	{ 0x77b7, 0x77b7, PDF_CMAP_SINGLE, 19596 },
	{ 0x77b9, 0x77b9, PDF_CMAP_SINGLE, 5838 },
	{ 0x77bb, 0x77bb, PDF_CMAP_SINGLE, 5842 },
	{ 0x77bc, 0x77bd, PDF_CMAP_RANGE, 5840 },
	{ 0x77be, 0x77be, PDF_CMAP_SINGLE, 19597 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 5839 },
	{ 0x77c3, 0x77c3, PDF_CMAP_SINGLE, 22067 },
	{ 0x77c7, 0x77c7, PDF_CMAP_SINGLE, 5843 },
	{ 0x77c9, 0x77c9, PDF_CMAP_SINGLE, 19598 },
	{ 0x77cd, 0x77cd, PDF_CMAP_SINGLE, 5844 },
	{ 0x77d1, 0x77d1, PDF_CMAP_SINGLE, 19599 },
	{ 0x77d2, 0x77d2, PDF_CMAP_SINGLE, 22068 },
	{ 0x77d5, 0x77d5, PDF_CMAP_SINGLE, 22069 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 5845 },
	{ 0x77d9, 0x77d9, PDF_CMAP_SINGLE, 19600 },
	{ 0x77da, 0x77da, PDF_CMAP_SINGLE, 5846 },
	{ 0x77db, 0x77db, PDF_CMAP_SINGLE, 3779 },
	{ 0x77dc, 0x77dc, PDF_CMAP_SINGLE, 5847 },
	{ 0x77de, 0x77df, PDF_CMAP_RANGE, 18203 },
	{ 0x77e0, 0x77e0, PDF_CMAP_SINGLE, 17004 },
	{ 0x77e2, 0x77e2, PDF_CMAP_SINGLE, 3836 },
	{ 0x77e3, 0x77e3, PDF_CMAP_SINGLE, 5848 },
	{ 0x77e4, 0x77e4, PDF_CMAP_SINGLE, 18205 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 2956 },
	{ 0x77e6, 0x77e6, PDF_CMAP_SINGLE, 14883 },
	{ 0x77e7, 0x77e7, PDF_CMAP_SINGLE, 3360 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 1763 },
	{ 0x77ea, 0x77ea, PDF_CMAP_SINGLE, 18206 },
	{ 0x77ec, 0x77ec, PDF_CMAP_SINGLE, 18207 },
	{ 0x77ed, 0x77ed, PDF_CMAP_SINGLE, 2937 },
	{ 0x77ee, 0x77ee, PDF_CMAP_SINGLE, 5849 },
	{ 0x77ef, 0x77ef, PDF_CMAP_SINGLE, 1714 },
	{ 0x77f0, 0x77f0, PDF_CMAP_SINGLE, 14884 },
	{ 0x77f1, 0x77f1, PDF_CMAP_SINGLE, 19601 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 2676 },
	{ 0x77f4, 0x77f4, PDF_CMAP_SINGLE, 14885 },
	{ 0x77f8, 0x77f8, PDF_CMAP_SINGLE, 22070 },
	{ 0x77fb, 0x77fb, PDF_CMAP_SINGLE, 18208 },
	{ 0x77fc, 0x77fc, PDF_CMAP_SINGLE, 5850 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 2093 },
	{ 0x7805, 0x7805, PDF_CMAP_SINGLE, 18210 },
	{ 0x7806, 0x7806, PDF_CMAP_SINGLE, 14886 },
	{ 0x7809, 0x7809, PDF_CMAP_SINGLE, 18211 },
	{ 0x780c, 0x780c, PDF_CMAP_SINGLE, 5851 },
	{ 0x780d, 0x780d, PDF_CMAP_SINGLE, 18212 },
	{ 0x780e, 0x780e, PDF_CMAP_SINGLE, 22071 },
	{ 0x7811, 0x7811, PDF_CMAP_SINGLE, 22072 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 5852 },
	{ 0x7814, 0x7814, PDF_CMAP_SINGLE, 1882 },
	{ 0x7815, 0x7815, PDF_CMAP_SINGLE, 2117 },
	{ 0x7819, 0x7819, PDF_CMAP_SINGLE, 18213 },
	{ 0x781d, 0x781d, PDF_CMAP_SINGLE, 22073 },
	{ 0x7820, 0x7820, PDF_CMAP_SINGLE, 5854 },
	{ 0x7821, 0x7821, PDF_CMAP_SINGLE, 8574 },
	{ 0x7822, 0x7822, PDF_CMAP_SINGLE, 14887 },
	{ 0x7823, 0x7823, PDF_CMAP_SINGLE, 22074 },
	{ 0x7825, 0x7825, PDF_CMAP_SINGLE, 3152 },
	{ 0x7826, 0x7826, PDF_CMAP_SINGLE, 2118 },
	{ 0x7827, 0x7827, PDF_CMAP_SINGLE, 1640 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 18214 },
	{ 0x782d, 0x782e, PDF_CMAP_RANGE, 14888 },
	{ 0x7830, 0x7830, PDF_CMAP_SINGLE, 14890 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 3666 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 3329 },
	{ 0x7835, 0x7835, PDF_CMAP_SINGLE, 14891 },
	{ 0x7837, 0x7837, PDF_CMAP_SINGLE, 19602 },
	{ 0x783a, 0x783a, PDF_CMAP_SINGLE, 3153 },
	{ 0x783f, 0x783f, PDF_CMAP_SINGLE, 2030 },
	{ 0x7843, 0x7843, PDF_CMAP_SINGLE, 17006 },
	{ 0x7844, 0x7844, PDF_CMAP_SINGLE, 22075 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 5856 },
	{ 0x7847, 0x7847, PDF_CMAP_SINGLE, 18215 },
	{ 0x7848, 0x7848, PDF_CMAP_SINGLE, 22076 },
	{ 0x784c, 0x784c, PDF_CMAP_SINGLE, 22077 },
	{ 0x784e, 0x784e, PDF_CMAP_SINGLE, 8575 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 13342 },
	{ 0x7851, 0x7851, PDF_CMAP_SINGLE, 15420 },
	{ 0x7852, 0x7852, PDF_CMAP_SINGLE, 22078 },
	{ 0x785c, 0x785c, PDF_CMAP_SINGLE, 19603 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 2483 },
	{ 0x785e, 0x785e, PDF_CMAP_SINGLE, 22079 },
	{ 0x7860, 0x7861, PDF_CMAP_RANGE, 22080 },
	{ 0x7863, 0x7863, PDF_CMAP_SINGLE, 22082 },
	{ 0x7864, 0x7864, PDF_CMAP_SINGLE, 8576 },
	{ 0x7868, 0x7868, PDF_CMAP_SINGLE, 14892 },
	{ 0x786a, 0x786a, PDF_CMAP_SINGLE, 18216 },
	{ 0x786b, 0x786b, PDF_CMAP_SINGLE, 3962 },
	{ 0x786c, 0x786c, PDF_CMAP_SINGLE, 2007 },
	{ 0x786e, 0x786e, PDF_CMAP_SINGLE, 17007 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 1883 },
	{ 0x7872, 0x7872, PDF_CMAP_SINGLE, 3383 },
	{ 0x7874, 0x7874, PDF_CMAP_SINGLE, 5858 },
	{ 0x787a, 0x787a, PDF_CMAP_SINGLE, 8577 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 5860 },
	{ 0x787e, 0x787e, PDF_CMAP_SINGLE, 19604 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 1951 },
	{ 0x7886, 0x7886, PDF_CMAP_SINGLE, 5859 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 3090 },
	{ 0x788a, 0x788a, PDF_CMAP_SINGLE, 18218 },
	{ 0x788c, 0x788c, PDF_CMAP_SINGLE, 5862 },
	{ 0x788d, 0x788d, PDF_CMAP_SINGLE, 1429 },
	{ 0x788e, 0x788e, PDF_CMAP_SINGLE, 5857 },
	{ 0x788f, 0x788f, PDF_CMAP_SINGLE, 22083 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 3454 },
	{ 0x7893, 0x7893, PDF_CMAP_SINGLE, 1234 },
	{ 0x7894, 0x7894, PDF_CMAP_SINGLE, 18219 },
	{ 0x7895, 0x7895, PDF_CMAP_SINGLE, 2140 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 4088 },
	{ 0x7898, 0x7898, PDF_CMAP_SINGLE, 19605 },
	{ 0x789a, 0x789a, PDF_CMAP_SINGLE, 5861 },
	{ 0x789d, 0x789d, PDF_CMAP_SINGLE, 18221 },
	{ 0x789e, 0x789e, PDF_CMAP_SINGLE, 14893 },
	{ 0x789f, 0x789f, PDF_CMAP_SINGLE, 18222 },
	{ 0x78a1, 0x78a1, PDF_CMAP_SINGLE, 19606 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 5863 },
	{ 0x78a4, 0x78a4, PDF_CMAP_SINGLE, 18220 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 3611 },
	{ 0x78a8, 0x78a8, PDF_CMAP_SINGLE, 22084 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 2685 },
	{ 0x78aa, 0x78aa, PDF_CMAP_SINGLE, 5865 },
	{ 0x78ac, 0x78ac, PDF_CMAP_SINGLE, 22085 },
	{ 0x78ad, 0x78ad, PDF_CMAP_SINGLE, 17010 },
	{ 0x78af, 0x78af, PDF_CMAP_SINGLE, 5866 },
	{ 0x78b0, 0x78b0, PDF_CMAP_SINGLE, 17008 },
	{ 0x78b1, 0x78b1, PDF_CMAP_SINGLE, 19607 },
	{ 0x78b2, 0x78b2, PDF_CMAP_SINGLE, 22086 },
	{ 0x78b3, 0x78b3, PDF_CMAP_SINGLE, 19608 },
	{ 0x78b5, 0x78b5, PDF_CMAP_SINGLE, 5864 },
	{ 0x78ba, 0x78ba, PDF_CMAP_SINGLE, 1452 },
	{ 0x78bb, 0x78bb, PDF_CMAP_SINGLE, 18223 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 5872 },
	{ 0x78bd, 0x78bd, PDF_CMAP_SINGLE, 22087 },
	{ 0x78be, 0x78be, PDF_CMAP_SINGLE, 5871 },
	{ 0x78bf, 0x78bf, PDF_CMAP_SINGLE, 22088 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 2259 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 5873 },
	{ 0x78c6, 0x78c6, PDF_CMAP_SINGLE, 5868 },
	{ 0x78c7, 0x78c7, PDF_CMAP_SINGLE, 22089 },
	{ 0x78c8, 0x78c8, PDF_CMAP_SINGLE, 14894 },
	{ 0x78c9, 0x78c9, PDF_CMAP_SINGLE, 19609 },
	{ 0x78ca, 0x78ca, PDF_CMAP_SINGLE, 5874 },
	{ 0x78cb, 0x78cb, PDF_CMAP_SINGLE, 5869 },
	{ 0x78cc, 0x78cc, PDF_CMAP_SINGLE, 14895 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 14896 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 3436 },
	{ 0x78d1, 0x78d1, PDF_CMAP_SINGLE, 5867 },
	{ 0x78d2, 0x78d2, PDF_CMAP_SINGLE, 22090 },
	{ 0x78d3, 0x78d3, PDF_CMAP_SINGLE, 19610 },
	{ 0x78d4, 0x78d4, PDF_CMAP_SINGLE, 5870 },
	{ 0x78d5, 0x78d5, PDF_CMAP_SINGLE, 18224 },
	{ 0x78d6, 0x78d6, PDF_CMAP_SINGLE, 22091 },
	{ 0x78da, 0x78da, PDF_CMAP_SINGLE, 5877 },
	{ 0x78db, 0x78db, PDF_CMAP_SINGLE, 22092 },
	{ 0x78df, 0x78df, PDF_CMAP_SINGLE, 22093 },
	{ 0x78e0, 0x78e1, PDF_CMAP_RANGE, 14898 },
	{ 0x78e4, 0x78e4, PDF_CMAP_SINGLE, 14897 },
	{ 0x78e6, 0x78e6, PDF_CMAP_SINGLE, 18225 },
	{ 0x78e7, 0x78e7, PDF_CMAP_SINGLE, 5876 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 3727 },
	{ 0x78ea, 0x78ea, PDF_CMAP_SINGLE, 22094 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 5875 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 1199 },
	{ 0x78f2, 0x78f2, PDF_CMAP_SINGLE, 14900 },
	{ 0x78f3, 0x78f3, PDF_CMAP_SINGLE, 22095 },
	{ 0x78f4, 0x78f4, PDF_CMAP_SINGLE, 5879 },
	{ 0x78f6, 0x78f6, PDF_CMAP_SINGLE, 22096 },
	{ 0x78f7, 0x78f7, PDF_CMAP_SINGLE, 14901 },
	{ 0x78f9, 0x78fa, PDF_CMAP_RANGE, 18226 },
	{ 0x78fb, 0x78fb, PDF_CMAP_SINGLE, 14902 },
	{ 0x78fd, 0x78fd, PDF_CMAP_SINGLE, 5878 },
	{ 0x78fe, 0x78fe, PDF_CMAP_SINGLE, 18228 },
	{ 0x78ff, 0x78ff, PDF_CMAP_SINGLE, 22097 },
	{ 0x7900, 0x7900, PDF_CMAP_SINGLE, 17011 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 2484 },
	{ 0x7906, 0x7906, PDF_CMAP_SINGLE, 22098 },
	{ 0x7907, 0x7907, PDF_CMAP_SINGLE, 5880 },
	{ 0x790c, 0x790c, PDF_CMAP_SINGLE, 19611 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 2757 },
	{ 0x7910, 0x7910, PDF_CMAP_SINGLE, 18230 },
	{ 0x7911, 0x7911, PDF_CMAP_SINGLE, 5882 },
	{ 0x7912, 0x7912, PDF_CMAP_SINGLE, 5881 },
	{ 0x7919, 0x7919, PDF_CMAP_SINGLE, 5883 },
	{ 0x791a, 0x791a, PDF_CMAP_SINGLE, 22099 },
	{ 0x791b, 0x791b, PDF_CMAP_SINGLE, 18231 },
	{ 0x791c, 0x791c, PDF_CMAP_SINGLE, 17012 },
	{ 0x791e, 0x791e, PDF_CMAP_SINGLE, 22100 },
	{ 0x791f, 0x791f, PDF_CMAP_SINGLE, 19612 },
	{ 0x7920, 0x7920, PDF_CMAP_SINGLE, 22101 },
	{ 0x7925, 0x7925, PDF_CMAP_SINGLE, 18232 },
	{ 0x7926, 0x7926, PDF_CMAP_SINGLE, 5853 },
	{ 0x7927, 0x7928, PDF_CMAP_RANGE, 19613 },
	{ 0x7929, 0x7929, PDF_CMAP_SINGLE, 22102 },
	{ 0x792a, 0x792a, PDF_CMAP_SINGLE, 5855 },
	{ 0x792b, 0x792b, PDF_CMAP_SINGLE, 5885 },
	{ 0x792c, 0x792c, PDF_CMAP_SINGLE, 5884 },
	{ 0x792d, 0x792d, PDF_CMAP_SINGLE, 22103 },
	{ 0x792e, 0x792e, PDF_CMAP_SINGLE, 17013 },
	{ 0x7930, 0x7930, PDF_CMAP_SINGLE, 8578 },
	{ 0x7931, 0x7931, PDF_CMAP_SINGLE, 14903 },
	{ 0x7934, 0x7934, PDF_CMAP_SINGLE, 17014 },
	{ 0x7935, 0x7935, PDF_CMAP_SINGLE, 22104 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 2260 },
	{ 0x793b, 0x793b, PDF_CMAP_SINGLE, 14905 },
	{ 0x793c, 0x793c, PDF_CMAP_SINGLE, 4017 },
	{ 0x793d, 0x793d, PDF_CMAP_SINGLE, 14906 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 2302 },
	{ 0x793f, 0x793f, PDF_CMAP_SINGLE, 19615 },
	{ 0x7940, 0x7940, PDF_CMAP_SINGLE, 5886 },
	{ 0x7941, 0x7941, PDF_CMAP_SINGLE, 1805 },
	{ 0x7942, 0x7942, PDF_CMAP_SINGLE, 19616 },
	{ 0x7944, 0x7944, PDF_CMAP_SINGLE, 22105 },
	{ 0x7945, 0x7945, PDF_CMAP_SINGLE, 14907 },
	{ 0x7946, 0x7946, PDF_CMAP_SINGLE, 17015 },
	{ 0x7947, 0x7947, PDF_CMAP_SINGLE, 1626 },
	{ 0x7948, 0x7948, PDF_CMAP_SINGLE, 1601 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 2225 },
	{ 0x794a, 0x794a, PDF_CMAP_SINGLE, 18233 },
	{ 0x794b, 0x794b, PDF_CMAP_SINGLE, 22106 },
	{ 0x794f, 0x794f, PDF_CMAP_SINGLE, 22107 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 3870 },
	{ 0x7951, 0x7951, PDF_CMAP_SINGLE, 22108 },
	{ 0x7953, 0x7953, PDF_CMAP_SINGLE, 5892 },
	{ 0x7954, 0x7954, PDF_CMAP_SINGLE, 19617 },
	{ 0x7955, 0x7955, PDF_CMAP_SINGLE, 5891 },
	{ 0x7956, 0x7956, PDF_CMAP_SINGLE, 2758 },
	{ 0x7957, 0x7957, PDF_CMAP_SINGLE, 5888 },
	{ 0x7958, 0x7958, PDF_CMAP_SINGLE, 18234 },
	{ 0x795a, 0x795a, PDF_CMAP_SINGLE, 5890 },
	{ 0x795b, 0x795c, PDF_CMAP_RANGE, 14908 },
	{ 0x795d, 0x795d, PDF_CMAP_SINGLE, 2389 },
	{ 0x795e, 0x795e, PDF_CMAP_SINGLE, 2566 },
	{ 0x795f, 0x795f, PDF_CMAP_SINGLE, 5889 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 5887 },
	{ 0x7962, 0x7962, PDF_CMAP_SINGLE, 3296 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 2485 },
	{ 0x7967, 0x7967, PDF_CMAP_SINGLE, 18236 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 3502 },
	{ 0x7969, 0x7969, PDF_CMAP_SINGLE, 22109 },
	{ 0x796b, 0x796b, PDF_CMAP_SINGLE, 19618 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 2119 },
	{ 0x7972, 0x7972, PDF_CMAP_SINGLE, 18237 },
	{ 0x7977, 0x7977, PDF_CMAP_SINGLE, 3186 },
	{ 0x7979, 0x7979, PDF_CMAP_SINGLE, 17016 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 5893 },
	{ 0x797b, 0x797b, PDF_CMAP_SINGLE, 22110 },
	{ 0x797c, 0x797c, PDF_CMAP_SINGLE, 19619 },
	{ 0x797e, 0x797e, PDF_CMAP_SINGLE, 22111 },
	{ 0x797f, 0x797f, PDF_CMAP_SINGLE, 5894 },
	{ 0x7980, 0x7980, PDF_CMAP_SINGLE, 5916 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 1744 },
	{ 0x7984, 0x7984, PDF_CMAP_SINGLE, 4067 },
	{ 0x7985, 0x7985, PDF_CMAP_SINGLE, 2743 },
	{ 0x798a, 0x798a, PDF_CMAP_SINGLE, 5895 },
	{ 0x798b, 0x798b, PDF_CMAP_SINGLE, 14910 },
	{ 0x798c, 0x798c, PDF_CMAP_SINGLE, 22112 },
	{ 0x798d, 0x798d, PDF_CMAP_SINGLE, 1362 },
	{ 0x798e, 0x798e, PDF_CMAP_SINGLE, 3091 },
	{ 0x798f, 0x798f, PDF_CMAP_SINGLE, 3569 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 22113 },
	{ 0x7993, 0x7993, PDF_CMAP_SINGLE, 22114 },
	{ 0x7994, 0x7994, PDF_CMAP_SINGLE, 8582 },
	{ 0x7995, 0x7995, PDF_CMAP_SINGLE, 18238 },
	{ 0x7996, 0x7996, PDF_CMAP_SINGLE, 14911 },
	{ 0x7998, 0x7998, PDF_CMAP_SINGLE, 14912 },
	{ 0x799b, 0x799b, PDF_CMAP_SINGLE, 8584 },
	{ 0x799c, 0x799c, PDF_CMAP_SINGLE, 22115 },
	{ 0x799d, 0x799d, PDF_CMAP_SINGLE, 5896 },
	{ 0x79a1, 0x79a1, PDF_CMAP_SINGLE, 18239 },
	{ 0x79a6, 0x79a6, PDF_CMAP_SINGLE, 1684 },
	{ 0x79a7, 0x79a7, PDF_CMAP_SINGLE, 5897 },
	{ 0x79a8, 0x79a8, PDF_CMAP_SINGLE, 22116 },
	{ 0x79a9, 0x79a9, PDF_CMAP_SINGLE, 18240 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5899 },
	{ 0x79ab, 0x79ab, PDF_CMAP_SINGLE, 19620 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 5900 },
	{ 0x79af, 0x79af, PDF_CMAP_SINGLE, 22117 },
	{ 0x79b0, 0x79b0, PDF_CMAP_SINGLE, 3295 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 7758 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5901 },
	{ 0x79b4, 0x79b4, PDF_CMAP_SINGLE, 18241 },
	{ 0x79b8, 0x79b8, PDF_CMAP_SINGLE, 14913 },
	{ 0x79b9, 0x79ba, PDF_CMAP_RANGE, 5902 },
	{ 0x79bb, 0x79bb, PDF_CMAP_SINGLE, 14914 },
	{ 0x79bd, 0x79bd, PDF_CMAP_SINGLE, 1745 },
	{ 0x79be, 0x79be, PDF_CMAP_SINGLE, 1363 },
	{ 0x79bf, 0x79bf, PDF_CMAP_SINGLE, 3229 },
	{ 0x79c0, 0x79c0, PDF_CMAP_SINGLE, 2354 },
	{ 0x79c1, 0x79c1, PDF_CMAP_SINGLE, 2226 },
	{ 0x79c2, 0x79c2, PDF_CMAP_SINGLE, 18242 },
	{ 0x79c4, 0x79c4, PDF_CMAP_SINGLE, 19621 },
	{ 0x79c7, 0x79c7, PDF_CMAP_SINGLE, 18243 },
	{ 0x79c8, 0x79c8, PDF_CMAP_SINGLE, 17017 },
	{ 0x79c9, 0x79c9, PDF_CMAP_SINGLE, 5904 },
	{ 0x79ca, 0x79ca, PDF_CMAP_SINGLE, 14915 },
	{ 0x79cb, 0x79cb, PDF_CMAP_SINGLE, 2355 },
	{ 0x79cc, 0x79cd, PDF_CMAP_RANGE, 18244 },
	{ 0x79cf, 0x79cf, PDF_CMAP_SINGLE, 22118 },
	{ 0x79d1, 0x79d1, PDF_CMAP_SINGLE, 1354 },
	{ 0x79d2, 0x79d2, PDF_CMAP_SINGLE, 3509 },
	{ 0x79d4, 0x79d4, PDF_CMAP_SINGLE, 17019 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 5905 },
	{ 0x79d6, 0x79d6, PDF_CMAP_SINGLE, 18246 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 3455 },
	{ 0x79da, 0x79da, PDF_CMAP_SINGLE, 14916 },
	{ 0x79dd, 0x79dd, PDF_CMAP_SINGLE, 22119 },
	{ 0x79de, 0x79de, PDF_CMAP_SINGLE, 17020 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 2759 },
	{ 0x79e0, 0x79e0, PDF_CMAP_SINGLE, 22120 },
	{ 0x79e1, 0x79e1, PDF_CMAP_SINGLE, 5908 },
	{ 0x79e2, 0x79e2, PDF_CMAP_SINGLE, 22121 },
	{ 0x79e3, 0x79e3, PDF_CMAP_SINGLE, 5909 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 3359 },
	{ 0x79e5, 0x79e5, PDF_CMAP_SINGLE, 22122 },
	{ 0x79e6, 0x79e6, PDF_CMAP_SINGLE, 2567 },
	{ 0x79e7, 0x79e7, PDF_CMAP_SINGLE, 5906 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 2975 },
	{ 0x79ea, 0x79ea, PDF_CMAP_SINGLE, 19622 },
	{ 0x79eb, 0x79eb, PDF_CMAP_SINGLE, 17021 },
	{ 0x79ec, 0x79ec, PDF_CMAP_SINGLE, 5907 },
	{ 0x79ed, 0x79ed, PDF_CMAP_SINGLE, 17022 },
	{ 0x79f0, 0x79f0, PDF_CMAP_SINGLE, 2486 },
	{ 0x79f1, 0x79f1, PDF_CMAP_SINGLE, 22123 },
	{ 0x79f8, 0x79f8, PDF_CMAP_SINGLE, 22124 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 1184 },
	{ 0x79fc, 0x79fc, PDF_CMAP_SINGLE, 22125 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 1603 },
	{ 0x7a02, 0x7a02, PDF_CMAP_SINGLE, 19623 },
	{ 0x7a03, 0x7a03, PDF_CMAP_SINGLE, 14917 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 13875 },
	{ 0x7a07, 0x7a07, PDF_CMAP_SINGLE, 22126 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 5910 },
	{ 0x7a09, 0x7a09, PDF_CMAP_SINGLE, 14918 },
	{ 0x7a0a, 0x7a0a, PDF_CMAP_SINGLE, 18251 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 3092 },
	{ 0x7a0c, 0x7a0c, PDF_CMAP_SINGLE, 19624 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 5911 },
	{ 0x7a0e, 0x7a0e, PDF_CMAP_SINGLE, 2667 },
	{ 0x7a11, 0x7a11, PDF_CMAP_SINGLE, 14919 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 3769 },
	{ 0x7a15, 0x7a15, PDF_CMAP_SINGLE, 18252 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 3477 },
	{ 0x7a18, 0x7a19, PDF_CMAP_RANGE, 5912 },
	{ 0x7a1a, 0x7a1a, PDF_CMAP_SINGLE, 2963 },
	{ 0x7a1b, 0x7a1b, PDF_CMAP_SINGLE, 18253 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 3983 },
	{ 0x7a1e, 0x7a1e, PDF_CMAP_SINGLE, 14920 },
	{ 0x7a1f, 0x7a1f, PDF_CMAP_SINGLE, 5915 },
	{ 0x7a20, 0x7a20, PDF_CMAP_SINGLE, 5914 },
	{ 0x7a21, 0x7a21, PDF_CMAP_SINGLE, 22127 },
	{ 0x7a27, 0x7a27, PDF_CMAP_SINGLE, 22128 },
	{ 0x7a2b, 0x7a2b, PDF_CMAP_SINGLE, 22129 },
	{ 0x7a2d, 0x7a2d, PDF_CMAP_SINGLE, 14921 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 2331 },
	{ 0x7a2f, 0x7a2f, PDF_CMAP_SINGLE, 22130 },
	{ 0x7a30, 0x7a30, PDF_CMAP_SINGLE, 19625 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 5917 },
	{ 0x7a32, 0x7a32, PDF_CMAP_SINGLE, 1204 },
	{ 0x7a34, 0x7a35, PDF_CMAP_RANGE, 22131 },
	{ 0x7a37, 0x7a37, PDF_CMAP_SINGLE, 5920 },
	{ 0x7a38, 0x7a38, PDF_CMAP_SINGLE, 18255 },
	{ 0x7a39, 0x7a39, PDF_CMAP_SINGLE, 14922 },
	{ 0x7a3a, 0x7a3a, PDF_CMAP_SINGLE, 19626 },
	{ 0x7a3b, 0x7a3b, PDF_CMAP_SINGLE, 5918 },
	{ 0x7a3c, 0x7a3c, PDF_CMAP_SINGLE, 1364 },
	{ 0x7a3d, 0x7a3d, PDF_CMAP_SINGLE, 1828 },
	{ 0x7a3e, 0x7a3e, PDF_CMAP_SINGLE, 5919 },
	{ 0x7a3f, 0x7a3f, PDF_CMAP_SINGLE, 2008 },
	{ 0x7a40, 0x7a40, PDF_CMAP_SINGLE, 2052 },
	{ 0x7a42, 0x7a42, PDF_CMAP_SINGLE, 3638 },
	{ 0x7a43, 0x7a43, PDF_CMAP_SINGLE, 5921 },
	{ 0x7a44, 0x7a44, PDF_CMAP_SINGLE, 19627 },
	{ 0x7a45, 0x7a45, PDF_CMAP_SINGLE, 14923 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 3714 },
	{ 0x7a47, 0x7a47, PDF_CMAP_SINGLE, 18256 },
	{ 0x7a48, 0x7a48, PDF_CMAP_SINGLE, 22133 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 5923 },
	{ 0x7a4c, 0x7a4c, PDF_CMAP_SINGLE, 14924 },
	{ 0x7a4d, 0x7a4d, PDF_CMAP_SINGLE, 2677 },
	{ 0x7a4e, 0x7a4e, PDF_CMAP_SINGLE, 1265 },
	{ 0x7a4f, 0x7a4f, PDF_CMAP_SINGLE, 1338 },
	{ 0x7a50, 0x7a50, PDF_CMAP_SINGLE, 1136 },
	{ 0x7a55, 0x7a55, PDF_CMAP_SINGLE, 22134 },
	{ 0x7a56, 0x7a56, PDF_CMAP_SINGLE, 18257 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5922 },
	{ 0x7a59, 0x7a59, PDF_CMAP_SINGLE, 18258 },
	{ 0x7a5c, 0x7a5c, PDF_CMAP_SINGLE, 18259 },
	{ 0x7a5d, 0x7a5d, PDF_CMAP_SINGLE, 14925 },
	{ 0x7a5f, 0x7a5f, PDF_CMAP_SINGLE, 18260 },
	{ 0x7a60, 0x7a60, PDF_CMAP_SINGLE, 20310 },
	{ 0x7a61, 0x7a62, PDF_CMAP_RANGE, 5924 },
	{ 0x7a63, 0x7a63, PDF_CMAP_SINGLE, 2527 },
	{ 0x7a65, 0x7a65, PDF_CMAP_SINGLE, 22135 },
	{ 0x7a67, 0x7a67, PDF_CMAP_SINGLE, 18261 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 5926 },
	{ 0x7a6a, 0x7a6a, PDF_CMAP_SINGLE, 18262 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 1453 },
	{ 0x7a6d, 0x7a6d, PDF_CMAP_SINGLE, 14927 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 5928 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 1856 },
	{ 0x7a75, 0x7a75, PDF_CMAP_SINGLE, 18263 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 1664 },
	{ 0x7a78, 0x7a78, PDF_CMAP_SINGLE, 14928 },
	{ 0x7a79, 0x7a79, PDF_CMAP_SINGLE, 5929 },
	{ 0x7a7a, 0x7a7a, PDF_CMAP_SINGLE, 1773 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 5930 },
	{ 0x7a7e, 0x7a7e, PDF_CMAP_SINGLE, 22136 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 2720 },
	{ 0x7a80, 0x7a80, PDF_CMAP_SINGLE, 19628 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 3237 },
	{ 0x7a82, 0x7a82, PDF_CMAP_SINGLE, 18264 },
	{ 0x7a83, 0x7a83, PDF_CMAP_SINGLE, 2692 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 2149 },
	{ 0x7a85, 0x7a85, PDF_CMAP_SINGLE, 17023 },
	{ 0x7a86, 0x7a86, PDF_CMAP_SINGLE, 19629 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 5931 },
	{ 0x7a8a, 0x7a8a, PDF_CMAP_SINGLE, 18265 },
	{ 0x7a8b, 0x7a8b, PDF_CMAP_SINGLE, 22137 },
	{ 0x7a90, 0x7a90, PDF_CMAP_SINGLE, 18266 },
	{ 0x7a91, 0x7a91, PDF_CMAP_SINGLE, 22138 },
	{ 0x7a92, 0x7a92, PDF_CMAP_SINGLE, 2976 },
	{ 0x7a93, 0x7a93, PDF_CMAP_SINGLE, 2797 },
	{ 0x7a94, 0x7a94, PDF_CMAP_SINGLE, 19630 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 5933 },
	{ 0x7a96, 0x7a96, PDF_CMAP_SINGLE, 5935 },
	{ 0x7a97, 0x7a97, PDF_CMAP_SINGLE, 5932 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 5934 },
	{ 0x7a9e, 0x7a9e, PDF_CMAP_SINGLE, 22139 },
	{ 0x7a9f, 0x7a9f, PDF_CMAP_SINGLE, 1784 },
	{ 0x7aa0, 0x7aa0, PDF_CMAP_SINGLE, 14929 },
	{ 0x7aa3, 0x7aa3, PDF_CMAP_SINGLE, 14930 },
	{ 0x7aa9, 0x7aa9, PDF_CMAP_SINGLE, 5936 },
	{ 0x7aaa, 0x7aaa, PDF_CMAP_SINGLE, 1788 },
	{ 0x7aac, 0x7aac, PDF_CMAP_SINGLE, 18267 },
	{ 0x7aae, 0x7aae, PDF_CMAP_SINGLE, 1665 },
	{ 0x7aaf, 0x7aaf, PDF_CMAP_SINGLE, 3900 },
	{ 0x7ab0, 0x7ab0, PDF_CMAP_SINGLE, 5938 },
	{ 0x7ab3, 0x7ab3, PDF_CMAP_SINGLE, 14931 },
	{ 0x7ab5, 0x7ab5, PDF_CMAP_SINGLE, 19631 },
	{ 0x7ab6, 0x7ab6, PDF_CMAP_SINGLE, 5939 },
	{ 0x7ab9, 0x7ab9, PDF_CMAP_SINGLE, 18270 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 1232 },
	{ 0x7abb, 0x7abc, PDF_CMAP_RANGE, 14932 },
	{ 0x7abd, 0x7abd, PDF_CMAP_SINGLE, 19632 },
	{ 0x7abe, 0x7abe, PDF_CMAP_SINGLE, 18271 },
	{ 0x7abf, 0x7abf, PDF_CMAP_SINGLE, 5942 },
	{ 0x7ac3, 0x7ac3, PDF_CMAP_SINGLE, 1492 },
	{ 0x7ac4, 0x7ac4, PDF_CMAP_SINGLE, 5941 },
	{ 0x7ac5, 0x7ac5, PDF_CMAP_SINGLE, 5940 },
	{ 0x7ac6, 0x7ac6, PDF_CMAP_SINGLE, 14934 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 5944 },
	{ 0x7ac8, 0x7ac8, PDF_CMAP_SINGLE, 5937 },
	{ 0x7ac9, 0x7ac9, PDF_CMAP_SINGLE, 22140 },
	{ 0x7aca, 0x7aca, PDF_CMAP_SINGLE, 5945 },
	{ 0x7acb, 0x7acb, PDF_CMAP_SINGLE, 3953 },
	{ 0x7acc, 0x7acc, PDF_CMAP_SINGLE, 18273 },
	{ 0x7acd, 0x7acd, PDF_CMAP_SINGLE, 5946 },
	{ 0x7ace, 0x7ace, PDF_CMAP_SINGLE, 17025 },
	{ 0x7acf, 0x7acf, PDF_CMAP_SINGLE, 5947 },
	{ 0x7ad1, 0x7ad1, PDF_CMAP_SINGLE, 8585 },
	{ 0x7ad2, 0x7ad2, PDF_CMAP_SINGLE, 4549 },
	{ 0x7ad3, 0x7ad3, PDF_CMAP_SINGLE, 5949 },
	{ 0x7ad5, 0x7ad5, PDF_CMAP_SINGLE, 5948 },
	{ 0x7ad9, 0x7ada, PDF_CMAP_RANGE, 5950 },
	{ 0x7adb, 0x7adb, PDF_CMAP_SINGLE, 22141 },
	{ 0x7adc, 0x7adc, PDF_CMAP_SINGLE, 3965 },
	{ 0x7add, 0x7add, PDF_CMAP_SINGLE, 5952 },
	{ 0x7adf, 0x7adf, PDF_CMAP_SINGLE, 7176 },
	{ 0x7ae0, 0x7ae0, PDF_CMAP_SINGLE, 2487 },
	{ 0x7ae1, 0x7ae2, PDF_CMAP_RANGE, 5953 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 2401 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 3216 },
	{ 0x7ae6, 0x7ae6, PDF_CMAP_SINGLE, 5955 },
	{ 0x7ae7, 0x7ae7, PDF_CMAP_SINGLE, 8586 },
	{ 0x7ae8, 0x7ae8, PDF_CMAP_SINGLE, 18274 },
	{ 0x7ae9, 0x7ae9, PDF_CMAP_SINGLE, 22142 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 2918 },
	{ 0x7aeb, 0x7aeb, PDF_CMAP_SINGLE, 8588 },
	{ 0x7aec, 0x7aec, PDF_CMAP_SINGLE, 22143 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 5956 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 2938 },
	{ 0x7af0, 0x7af0, PDF_CMAP_SINGLE, 5957 },
	{ 0x7af1, 0x7af1, PDF_CMAP_SINGLE, 22144 },
	{ 0x7af4, 0x7af4, PDF_CMAP_SINGLE, 18275 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 1693 },
	{ 0x7af8, 0x7af8, PDF_CMAP_SINGLE, 4214 },
	{ 0x7af9, 0x7af9, PDF_CMAP_SINGLE, 2971 },
	{ 0x7afa, 0x7afa, PDF_CMAP_SINGLE, 2271 },
	{ 0x7afb, 0x7afb, PDF_CMAP_SINGLE, 22145 },
	{ 0x7afd, 0x7afd, PDF_CMAP_SINGLE, 17026 },
	{ 0x7afe, 0x7afe, PDF_CMAP_SINGLE, 19633 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 1540 },
	{ 0x7b02, 0x7b02, PDF_CMAP_SINGLE, 5958 },
	{ 0x7b04, 0x7b04, PDF_CMAP_SINGLE, 5971 },
	{ 0x7b06, 0x7b06, PDF_CMAP_SINGLE, 5961 },
	{ 0x7b07, 0x7b07, PDF_CMAP_SINGLE, 14935 },
	{ 0x7b08, 0x7b08, PDF_CMAP_SINGLE, 1666 },
	{ 0x7b0a, 0x7b0a, PDF_CMAP_SINGLE, 5960 },
	{ 0x7b0b, 0x7b0b, PDF_CMAP_SINGLE, 5973 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 5959 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 2488 },
	{ 0x7b12, 0x7b12, PDF_CMAP_SINGLE, 17027 },
	{ 0x7b14, 0x7b14, PDF_CMAP_SINGLE, 14936 },
	{ 0x7b18, 0x7b19, PDF_CMAP_RANGE, 5963 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 3109 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 5965 },
	{ 0x7b1f, 0x7b1f, PDF_CMAP_SINGLE, 22146 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 1468 },
	{ 0x7b23, 0x7b23, PDF_CMAP_SINGLE, 22147 },
	{ 0x7b25, 0x7b25, PDF_CMAP_SINGLE, 2592 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 3542 },
	{ 0x7b27, 0x7b27, PDF_CMAP_SINGLE, 14937 },
	{ 0x7b28, 0x7b28, PDF_CMAP_SINGLE, 5967 },
	{ 0x7b29, 0x7b29, PDF_CMAP_SINGLE, 22148 },
	{ 0x7b2a, 0x7b2a, PDF_CMAP_SINGLE, 18280 },
	{ 0x7b2b, 0x7b2b, PDF_CMAP_SINGLE, 19634 },
	{ 0x7b2c, 0x7b2c, PDF_CMAP_SINGLE, 2888 },
	{ 0x7b2d, 0x7b2d, PDF_CMAP_SINGLE, 17028 },
	{ 0x7b2e, 0x7b2f, PDF_CMAP_RANGE, 18281 },
	{ 0x7b30, 0x7b30, PDF_CMAP_SINGLE, 22149 },
	{ 0x7b31, 0x7b31, PDF_CMAP_SINGLE, 14938 },
	{ 0x7b33, 0x7b33, PDF_CMAP_SINGLE, 5962 },
	{ 0x7b34, 0x7b34, PDF_CMAP_SINGLE, 22150 },
	{ 0x7b35, 0x7b35, PDF_CMAP_SINGLE, 5966 },
	{ 0x7b36, 0x7b36, PDF_CMAP_SINGLE, 5968 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 2155 },
	{ 0x7b3b, 0x7b3b, PDF_CMAP_SINGLE, 17029 },
	{ 0x7b3d, 0x7b3d, PDF_CMAP_SINGLE, 18279 },
	{ 0x7b3f, 0x7b40, PDF_CMAP_RANGE, 22151 },
	{ 0x7b41, 0x7b41, PDF_CMAP_SINGLE, 18286 },
	{ 0x7b45, 0x7b45, PDF_CMAP_SINGLE, 5975 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 3488 },
	{ 0x7b47, 0x7b47, PDF_CMAP_SINGLE, 14939 },
	{ 0x7b48, 0x7b48, PDF_CMAP_SINGLE, 3386 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 3187 },
	{ 0x7b4b, 0x7b4b, PDF_CMAP_SINGLE, 1746 },
	{ 0x7b4c, 0x7b4c, PDF_CMAP_SINGLE, 5974 },
	{ 0x7b4d, 0x7b4d, PDF_CMAP_SINGLE, 5972 },
	{ 0x7b4e, 0x7b4e, PDF_CMAP_SINGLE, 14940 },
	{ 0x7b4f, 0x7b4f, PDF_CMAP_SINGLE, 3401 },
	{ 0x7b50, 0x7b50, PDF_CMAP_SINGLE, 5969 },
	{ 0x7b51, 0x7b51, PDF_CMAP_SINGLE, 2972 },
	{ 0x7b52, 0x7b52, PDF_CMAP_SINGLE, 3189 },
	{ 0x7b53, 0x7b53, PDF_CMAP_SINGLE, 14173 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 3188 },
	{ 0x7b55, 0x7b55, PDF_CMAP_SINGLE, 18288 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 2150 },
	{ 0x7b5d, 0x7b5d, PDF_CMAP_SINGLE, 5993 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 14941 },
	{ 0x7b64, 0x7b64, PDF_CMAP_SINGLE, 18290 },
	{ 0x7b65, 0x7b65, PDF_CMAP_SINGLE, 5977 },
	{ 0x7b66, 0x7b66, PDF_CMAP_SINGLE, 18291 },
	{ 0x7b67, 0x7b67, PDF_CMAP_SINGLE, 5979 },
	{ 0x7b69, 0x7b69, PDF_CMAP_SINGLE, 14942 },
	{ 0x7b6a, 0x7b6a, PDF_CMAP_SINGLE, 22153 },
	{ 0x7b6c, 0x7b6c, PDF_CMAP_SINGLE, 5982 },
	{ 0x7b6d, 0x7b6d, PDF_CMAP_SINGLE, 14943 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 5983 },
	{ 0x7b6f, 0x7b6f, PDF_CMAP_SINGLE, 17030 },
	{ 0x7b70, 0x7b71, PDF_CMAP_RANGE, 5980 },
	{ 0x7b72, 0x7b72, PDF_CMAP_SINGLE, 14944 },
	{ 0x7b73, 0x7b73, PDF_CMAP_SINGLE, 18292 },
	{ 0x7b74, 0x7b74, PDF_CMAP_SINGLE, 5978 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 5976 },
	{ 0x7b77, 0x7b77, PDF_CMAP_SINGLE, 19635 },
	{ 0x7b79, 0x7b79, PDF_CMAP_SINGLE, 18289 },
	{ 0x7b7a, 0x7b7a, PDF_CMAP_SINGLE, 5970 },
	{ 0x7b7f, 0x7b7f, PDF_CMAP_SINGLE, 18285 },
	{ 0x7b84, 0x7b84, PDF_CMAP_SINGLE, 22154 },
	{ 0x7b86, 0x7b86, PDF_CMAP_SINGLE, 3615 },
	{ 0x7b87, 0x7b87, PDF_CMAP_SINGLE, 1365 },
	{ 0x7b89, 0x7b89, PDF_CMAP_SINGLE, 22155 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 5990 },
	{ 0x7b8d, 0x7b8d, PDF_CMAP_SINGLE, 5987 },
	{ 0x7b8e, 0x7b8e, PDF_CMAP_SINGLE, 22156 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 5992 },
	{ 0x7b90, 0x7b90, PDF_CMAP_SINGLE, 18295 },
	{ 0x7b91, 0x7b91, PDF_CMAP_SINGLE, 14945 },
	{ 0x7b92, 0x7b92, PDF_CMAP_SINGLE, 5991 },
	{ 0x7b94, 0x7b94, PDF_CMAP_SINGLE, 3369 },
	{ 0x7b95, 0x7b95, PDF_CMAP_SINGLE, 3763 },
	{ 0x7b96, 0x7b96, PDF_CMAP_SINGLE, 22157 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 2185 },
	{ 0x7b98, 0x7b98, PDF_CMAP_SINGLE, 5985 },
	{ 0x7b99, 0x7b99, PDF_CMAP_SINGLE, 5994 },
	{ 0x7b9a, 0x7b9a, PDF_CMAP_SINGLE, 5989 },
	{ 0x7b9b, 0x7b9b, PDF_CMAP_SINGLE, 18296 },
	{ 0x7b9c, 0x7b9c, PDF_CMAP_SINGLE, 5988 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 5984 },
	{ 0x7b9e, 0x7b9e, PDF_CMAP_SINGLE, 8589 },
	{ 0x7b9f, 0x7b9f, PDF_CMAP_SINGLE, 5986 },
	{ 0x7ba0, 0x7ba0, PDF_CMAP_SINGLE, 19636 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 1541 },
	{ 0x7ba5, 0x7ba5, PDF_CMAP_SINGLE, 22158 },
	{ 0x7baa, 0x7baa, PDF_CMAP_SINGLE, 2939 },
	{ 0x7bac, 0x7bac, PDF_CMAP_SINGLE, 19637 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 2721 },
	{ 0x7baf, 0x7baf, PDF_CMAP_SINGLE, 14946 },
	{ 0x7bb0, 0x7bb0, PDF_CMAP_SINGLE, 19638 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 3382 },
	{ 0x7bb2, 0x7bb2, PDF_CMAP_SINGLE, 22159 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 5999 },
	{ 0x7bb5, 0x7bb5, PDF_CMAP_SINGLE, 18298 },
	{ 0x7bb6, 0x7bb6, PDF_CMAP_SINGLE, 22160 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 3384 },
	{ 0x7bba, 0x7bbb, PDF_CMAP_RANGE, 22161 },
	{ 0x7bbc, 0x7bbc, PDF_CMAP_SINGLE, 18299 },
	{ 0x7bbd, 0x7bbd, PDF_CMAP_SINGLE, 22163 },
	{ 0x7bc0, 0x7bc0, PDF_CMAP_SINGLE, 2693 },
	{ 0x7bc1, 0x7bc1, PDF_CMAP_SINGLE, 5996 },
	{ 0x7bc2, 0x7bc2, PDF_CMAP_SINGLE, 22164 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 3427 },
	{ 0x7bc5, 0x7bc5, PDF_CMAP_SINGLE, 18300 },
	{ 0x7bc6, 0x7bc6, PDF_CMAP_SINGLE, 6000 },
	{ 0x7bc7, 0x7bc7, PDF_CMAP_SINGLE, 3619 },
	{ 0x7bc8, 0x7bc8, PDF_CMAP_SINGLE, 22165 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 2969 },
	{ 0x7bca, 0x7bca, PDF_CMAP_SINGLE, 18301 },
	{ 0x7bcb, 0x7bcb, PDF_CMAP_SINGLE, 5995 },
	{ 0x7bcc, 0x7bcc, PDF_CMAP_SINGLE, 5997 },
	{ 0x7bcf, 0x7bcf, PDF_CMAP_SINGLE, 5998 },
	{ 0x7bd4, 0x7bd4, PDF_CMAP_SINGLE, 18304 },
	{ 0x7bd6, 0x7bd6, PDF_CMAP_SINGLE, 18305 },
	{ 0x7bd7, 0x7bd7, PDF_CMAP_SINGLE, 14947 },
	{ 0x7bd9, 0x7bd9, PDF_CMAP_SINGLE, 14948 },
	{ 0x7bda, 0x7bda, PDF_CMAP_SINGLE, 18306 },
	{ 0x7bdb, 0x7bdb, PDF_CMAP_SINGLE, 22166 },
	{ 0x7bdd, 0x7bdd, PDF_CMAP_SINGLE, 6001 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 2288 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 3230 },
	{ 0x7be5, 0x7be5, PDF_CMAP_SINGLE, 6006 },
	{ 0x7be6, 0x7be6, PDF_CMAP_SINGLE, 6005 },
	{ 0x7be8, 0x7be8, PDF_CMAP_SINGLE, 19639 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 6002 },
	{ 0x7bea, 0x7bea, PDF_CMAP_SINGLE, 18307 },
	{ 0x7bed, 0x7bed, PDF_CMAP_SINGLE, 4060 },
	{ 0x7bf0, 0x7bf0, PDF_CMAP_SINGLE, 18308 },
	{ 0x7bf2, 0x7bf2, PDF_CMAP_SINGLE, 19640 },
	{ 0x7bf3, 0x7bf3, PDF_CMAP_SINGLE, 6011 },
	{ 0x7bf4, 0x7bf5, PDF_CMAP_RANGE, 22167 },
	{ 0x7bf6, 0x7bf6, PDF_CMAP_SINGLE, 6015 },
	{ 0x7bf7, 0x7bf7, PDF_CMAP_SINGLE, 6012 },
	{ 0x7bf8, 0x7bf8, PDF_CMAP_SINGLE, 19641 },
	{ 0x7bf9, 0x7bfa, PDF_CMAP_RANGE, 22169 },
	{ 0x7bfc, 0x7bfc, PDF_CMAP_SINGLE, 19642 },
	{ 0x7bfe, 0x7bfe, PDF_CMAP_SINGLE, 19643 },
	{ 0x7c00, 0x7c00, PDF_CMAP_SINGLE, 6008 },
	{ 0x7c01, 0x7c01, PDF_CMAP_SINGLE, 17031 },
	{ 0x7c02, 0x7c02, PDF_CMAP_SINGLE, 22171 },
	{ 0x7c03, 0x7c03, PDF_CMAP_SINGLE, 18309 },
	{ 0x7c04, 0x7c04, PDF_CMAP_SINGLE, 22172 },
	{ 0x7c06, 0x7c06, PDF_CMAP_SINGLE, 22173 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 6009 },
	{ 0x7c09, 0x7c09, PDF_CMAP_SINGLE, 19644 },
	{ 0x7c0b, 0x7c0b, PDF_CMAP_SINGLE, 14949 },
	{ 0x7c0c, 0x7c0c, PDF_CMAP_SINGLE, 22174 },
	{ 0x7c0d, 0x7c0d, PDF_CMAP_SINGLE, 6014 },
	{ 0x7c0e, 0x7c0e, PDF_CMAP_SINGLE, 18310 },
	{ 0x7c0f, 0x7c0f, PDF_CMAP_SINGLE, 14950 },
	{ 0x7c11, 0x7c11, PDF_CMAP_SINGLE, 6003 },
	{ 0x7c12, 0x7c12, PDF_CMAP_SINGLE, 4330 },
	{ 0x7c13, 0x7c13, PDF_CMAP_SINGLE, 6010 },
	{ 0x7c14, 0x7c14, PDF_CMAP_SINGLE, 6004 },
	{ 0x7c17, 0x7c17, PDF_CMAP_SINGLE, 6013 },
	{ 0x7c19, 0x7c19, PDF_CMAP_SINGLE, 22175 },
	{ 0x7c1b, 0x7c1b, PDF_CMAP_SINGLE, 22176 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 7739 },
	{ 0x7c1f, 0x7c1f, PDF_CMAP_SINGLE, 6019 },
	{ 0x7c20, 0x7c20, PDF_CMAP_SINGLE, 14951 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 1542 },
	{ 0x7c23, 0x7c23, PDF_CMAP_SINGLE, 6016 },
	{ 0x7c25, 0x7c25, PDF_CMAP_SINGLE, 22177 },
	{ 0x7c26, 0x7c26, PDF_CMAP_SINGLE, 14952 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 6017 },
	{ 0x7c28, 0x7c28, PDF_CMAP_SINGLE, 19645 },
	{ 0x7c2a, 0x7c2a, PDF_CMAP_SINGLE, 6018 },
	{ 0x7c2b, 0x7c2b, PDF_CMAP_SINGLE, 6021 },
	{ 0x7c2c, 0x7c2c, PDF_CMAP_SINGLE, 22178 },
	{ 0x7c2f, 0x7c2f, PDF_CMAP_SINGLE, 19646 },
	{ 0x7c31, 0x7c31, PDF_CMAP_SINGLE, 14953 },
	{ 0x7c33, 0x7c33, PDF_CMAP_SINGLE, 17032 },
	{ 0x7c34, 0x7c34, PDF_CMAP_SINGLE, 22179 },
	{ 0x7c36, 0x7c36, PDF_CMAP_SINGLE, 14954 },
	{ 0x7c37, 0x7c37, PDF_CMAP_SINGLE, 6020 },
	{ 0x7c38, 0x7c38, PDF_CMAP_SINGLE, 3466 },
	{ 0x7c39, 0x7c3a, PDF_CMAP_RANGE, 22180 },
	{ 0x7c3d, 0x7c3d, PDF_CMAP_SINGLE, 6022 },
	{ 0x7c3e, 0x7c3e, PDF_CMAP_SINGLE, 4036 },
	{ 0x7c3f, 0x7c3f, PDF_CMAP_SINGLE, 3645 },
	{ 0x7c40, 0x7c40, PDF_CMAP_SINGLE, 6027 },
	{ 0x7c42, 0x7c42, PDF_CMAP_SINGLE, 19647 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 6024 },
	{ 0x7c45, 0x7c45, PDF_CMAP_SINGLE, 18311 },
	{ 0x7c46, 0x7c46, PDF_CMAP_SINGLE, 22182 },
	{ 0x7c4a, 0x7c4a, PDF_CMAP_SINGLE, 18312 },
	{ 0x7c4c, 0x7c4c, PDF_CMAP_SINGLE, 6023 },
	{ 0x7c4d, 0x7c4d, PDF_CMAP_SINGLE, 2678 },
	{ 0x7c4f, 0x7c4f, PDF_CMAP_SINGLE, 6026 },
	{ 0x7c50, 0x7c50, PDF_CMAP_SINGLE, 6028 },
	{ 0x7c51, 0x7c51, PDF_CMAP_SINGLE, 14955 },
	{ 0x7c52, 0x7c53, PDF_CMAP_RANGE, 19648 },
	{ 0x7c54, 0x7c54, PDF_CMAP_SINGLE, 6025 },
	{ 0x7c55, 0x7c55, PDF_CMAP_SINGLE, 22183 },
	{ 0x7c56, 0x7c56, PDF_CMAP_SINGLE, 6032 },
	{ 0x7c57, 0x7c57, PDF_CMAP_SINGLE, 18313 },
	{ 0x7c58, 0x7c58, PDF_CMAP_SINGLE, 6029 },
	{ 0x7c59, 0x7c59, PDF_CMAP_SINGLE, 14956 },
	{ 0x7c5a, 0x7c5a, PDF_CMAP_SINGLE, 22184 },
	{ 0x7c5b, 0x7c5d, PDF_CMAP_RANGE, 19650 },
	{ 0x7c5e, 0x7c5e, PDF_CMAP_SINGLE, 18314 },
	{ 0x7c5f, 0x7c5f, PDF_CMAP_SINGLE, 6030 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 6007 },
	{ 0x7c61, 0x7c61, PDF_CMAP_SINGLE, 18315 },
	{ 0x7c63, 0x7c63, PDF_CMAP_SINGLE, 22185 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 6031 },
	{ 0x7c65, 0x7c65, PDF_CMAP_SINGLE, 6033 },
	{ 0x7c67, 0x7c67, PDF_CMAP_SINGLE, 14957 },
	{ 0x7c69, 0x7c69, PDF_CMAP_SINGLE, 18316 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 6034 },
	{ 0x7c6d, 0x7c6d, PDF_CMAP_SINGLE, 17034 },
	{ 0x7c6e, 0x7c6e, PDF_CMAP_SINGLE, 14958 },
	{ 0x7c6f, 0x7c6f, PDF_CMAP_SINGLE, 18317 },
	{ 0x7c70, 0x7c70, PDF_CMAP_SINGLE, 14959 },
	{ 0x7c72, 0x7c72, PDF_CMAP_SINGLE, 19653 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 3606 },
	{ 0x7c75, 0x7c75, PDF_CMAP_SINGLE, 6035 },
	{ 0x7c79, 0x7c79, PDF_CMAP_SINGLE, 17035 },
	{ 0x7c7b, 0x7c7b, PDF_CMAP_SINGLE, 14094 },
	{ 0x7c7c, 0x7c7c, PDF_CMAP_SINGLE, 22187 },
	{ 0x7c7d, 0x7c7d, PDF_CMAP_SINGLE, 19654 },
	{ 0x7c7e, 0x7c7e, PDF_CMAP_SINGLE, 3822 },
	{ 0x7c81, 0x7c81, PDF_CMAP_SINGLE, 1734 },
	{ 0x7c82, 0x7c82, PDF_CMAP_SINGLE, 1791 },
	{ 0x7c83, 0x7c83, PDF_CMAP_SINGLE, 6036 },
	{ 0x7c86, 0x7c86, PDF_CMAP_SINGLE, 22188 },
	{ 0x7c87, 0x7c87, PDF_CMAP_SINGLE, 19655 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 3588 },
	{ 0x7c8b, 0x7c8b, PDF_CMAP_SINGLE, 2606 },
	{ 0x7c8d, 0x7c8d, PDF_CMAP_SINGLE, 3772 },
	{ 0x7c8f, 0x7c8f, PDF_CMAP_SINGLE, 17036 },
	{ 0x7c90, 0x7c90, PDF_CMAP_SINGLE, 6037 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 3963 },
	{ 0x7c94, 0x7c94, PDF_CMAP_SINGLE, 17037 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 3370 },
	{ 0x7c97, 0x7c97, PDF_CMAP_SINGLE, 2760 },
	{ 0x7c98, 0x7c98, PDF_CMAP_SINGLE, 3306 },
	{ 0x7c9b, 0x7c9b, PDF_CMAP_SINGLE, 2391 },
	{ 0x7c9e, 0x7c9e, PDF_CMAP_SINGLE, 19656 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 1156 },
	{ 0x7ca0, 0x7ca0, PDF_CMAP_SINGLE, 17038 },
	{ 0x7ca1, 0x7ca1, PDF_CMAP_SINGLE, 6042 },
	{ 0x7ca2, 0x7ca2, PDF_CMAP_SINGLE, 6040 },
	{ 0x7ca4, 0x7ca4, PDF_CMAP_SINGLE, 6038 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 1501 },
	{ 0x7ca6, 0x7ca6, PDF_CMAP_SINGLE, 18321 },
	{ 0x7ca7, 0x7ca7, PDF_CMAP_SINGLE, 2489 },
	{ 0x7ca8, 0x7ca8, PDF_CMAP_SINGLE, 6043 },
	{ 0x7cab, 0x7cab, PDF_CMAP_SINGLE, 6041 },
	{ 0x7cad, 0x7cad, PDF_CMAP_SINGLE, 6039 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 6047 },
	{ 0x7cb0, 0x7cb0, PDF_CMAP_SINGLE, 22189 },
	{ 0x7cb1, 0x7cb1, PDF_CMAP_SINGLE, 6046 },
	{ 0x7cb2, 0x7cb2, PDF_CMAP_SINGLE, 6045 },
	{ 0x7cb3, 0x7cb3, PDF_CMAP_SINGLE, 6044 },
	{ 0x7cb6, 0x7cb7, PDF_CMAP_RANGE, 18323 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 6048 },
	{ 0x7cba, 0x7cba, PDF_CMAP_SINGLE, 19657 },
	{ 0x7cbb, 0x7cbb, PDF_CMAP_SINGLE, 22190 },
	{ 0x7cbc, 0x7cbc, PDF_CMAP_SINGLE, 14960 },
	{ 0x7cbd, 0x7cbd, PDF_CMAP_SINGLE, 6049 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 2654 },
	{ 0x7cbf, 0x7cbf, PDF_CMAP_SINGLE, 14961 },
	{ 0x7cc0, 0x7cc0, PDF_CMAP_SINGLE, 6050 },
	{ 0x7cc2, 0x7cc2, PDF_CMAP_SINGLE, 6052 },
	{ 0x7cc4, 0x7cc4, PDF_CMAP_SINGLE, 18326 },
	{ 0x7cc5, 0x7cc5, PDF_CMAP_SINGLE, 6051 },
	{ 0x7cc7, 0x7cc7, PDF_CMAP_SINGLE, 19658 },
	{ 0x7cc8, 0x7cc9, PDF_CMAP_RANGE, 14962 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 1926 },
	{ 0x7ccd, 0x7ccd, PDF_CMAP_SINGLE, 18328 },
	{ 0x7cce, 0x7cce, PDF_CMAP_SINGLE, 2746 },
	{ 0x7ccf, 0x7ccf, PDF_CMAP_SINGLE, 22191 },
	{ 0x7cd2, 0x7cd2, PDF_CMAP_SINGLE, 6054 },
	{ 0x7cd3, 0x7cd3, PDF_CMAP_SINGLE, 19659 },
	{ 0x7cd4, 0x7cd4, PDF_CMAP_SINGLE, 22192 },
	{ 0x7cd5, 0x7cd5, PDF_CMAP_SINGLE, 17039 },
	{ 0x7cd6, 0x7cd6, PDF_CMAP_SINGLE, 3190 },
	{ 0x7cd7, 0x7cd7, PDF_CMAP_SINGLE, 14964 },
	{ 0x7cd8, 0x7cd8, PDF_CMAP_SINGLE, 6053 },
	{ 0x7cd9, 0x7cd9, PDF_CMAP_SINGLE, 14965 },
	{ 0x7cda, 0x7cda, PDF_CMAP_SINGLE, 19660 },
	{ 0x7cdc, 0x7cdc, PDF_CMAP_SINGLE, 6055 },
	{ 0x7cdd, 0x7cdd, PDF_CMAP_SINGLE, 14966 },
	{ 0x7cde, 0x7cde, PDF_CMAP_SINGLE, 3589 },
	{ 0x7cdf, 0x7cdf, PDF_CMAP_SINGLE, 2798 },
	{ 0x7ce0, 0x7ce0, PDF_CMAP_SINGLE, 2009 },
	{ 0x7ce2, 0x7ce2, PDF_CMAP_SINGLE, 6056 },
	{ 0x7ce6, 0x7ce6, PDF_CMAP_SINGLE, 18331 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 3984 },
	{ 0x7ce9, 0x7ce9, PDF_CMAP_SINGLE, 22193 },
	{ 0x7ceb, 0x7ceb, PDF_CMAP_SINGLE, 14967 },
	{ 0x7cef, 0x7cef, PDF_CMAP_SINGLE, 6058 },
	{ 0x7cf2, 0x7cf2, PDF_CMAP_SINGLE, 6059 },
	{ 0x7cf4, 0x7cf4, PDF_CMAP_SINGLE, 6060 },
	{ 0x7cf5, 0x7cf5, PDF_CMAP_SINGLE, 18333 },
	{ 0x7cf6, 0x7cf6, PDF_CMAP_SINGLE, 6061 },
	{ 0x7cf8, 0x7cf8, PDF_CMAP_SINGLE, 2227 },
	{ 0x7cfa, 0x7cfa, PDF_CMAP_SINGLE, 6062 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 1829 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 1668 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 1604 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 6064 },
	{ 0x7d03, 0x7d03, PDF_CMAP_SINGLE, 18334 },
	{ 0x7d04, 0x7d04, PDF_CMAP_SINGLE, 3839 },
	{ 0x7d05, 0x7d05, PDF_CMAP_SINGLE, 2010 },
	{ 0x7d06, 0x7d06, PDF_CMAP_SINGLE, 6063 },
	{ 0x7d07, 0x7d09, PDF_CMAP_RANGE, 14968 },
	{ 0x7d0a, 0x7d0a, PDF_CMAP_SINGLE, 6067 },
	{ 0x7d0b, 0x7d0b, PDF_CMAP_SINGLE, 3826 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 3314 },
	{ 0x7d0f, 0x7d0f, PDF_CMAP_SINGLE, 22194 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 3493 },
	{ 0x7d11, 0x7d11, PDF_CMAP_SINGLE, 22195 },
	{ 0x7d12, 0x7d12, PDF_CMAP_SINGLE, 18336 },
	{ 0x7d13, 0x7d13, PDF_CMAP_SINGLE, 14971 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 2413 },
	{ 0x7d15, 0x7d15, PDF_CMAP_SINGLE, 6066 },
	{ 0x7d16, 0x7d16, PDF_CMAP_SINGLE, 22196 },
	{ 0x7d17, 0x7d17, PDF_CMAP_SINGLE, 2303 },
	{ 0x7d18, 0x7d18, PDF_CMAP_SINGLE, 2011 },
	{ 0x7d19, 0x7d19, PDF_CMAP_SINGLE, 2228 },
	{ 0x7d1a, 0x7d1a, PDF_CMAP_SINGLE, 1667 },
	{ 0x7d1b, 0x7d1b, PDF_CMAP_SINGLE, 3590 },
	{ 0x7d1c, 0x7d1c, PDF_CMAP_SINGLE, 6065 },
	{ 0x7d1d, 0x7d1d, PDF_CMAP_SINGLE, 14972 },
	{ 0x7d1e, 0x7d1e, PDF_CMAP_SINGLE, 18337 },
	{ 0x7d20, 0x7d20, PDF_CMAP_SINGLE, 2761 },
	{ 0x7d21, 0x7d21, PDF_CMAP_SINGLE, 3696 },
	{ 0x7d22, 0x7d22, PDF_CMAP_SINGLE, 2151 },
	{ 0x7d23, 0x7d23, PDF_CMAP_SINGLE, 14973 },
	{ 0x7d26, 0x7d26, PDF_CMAP_SINGLE, 22197 },
	{ 0x7d2a, 0x7d2a, PDF_CMAP_SINGLE, 22198 },
	{ 0x7d2b, 0x7d2b, PDF_CMAP_SINGLE, 2229 },
	{ 0x7d2c, 0x7d2c, PDF_CMAP_SINGLE, 3065 },
	{ 0x7d2d, 0x7d2d, PDF_CMAP_SINGLE, 22199 },
	{ 0x7d2e, 0x7d2e, PDF_CMAP_SINGLE, 6070 },
	{ 0x7d2f, 0x7d2f, PDF_CMAP_SINGLE, 4007 },
	{ 0x7d30, 0x7d30, PDF_CMAP_SINGLE, 2121 },
	{ 0x7d31, 0x7d31, PDF_CMAP_SINGLE, 17040 },
	{ 0x7d32, 0x7d32, PDF_CMAP_SINGLE, 6071 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 2568 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 6073 },
	{ 0x7d39, 0x7d39, PDF_CMAP_SINGLE, 2490 },
	{ 0x7d3a, 0x7d3a, PDF_CMAP_SINGLE, 2080 },
	{ 0x7d3c, 0x7d3c, PDF_CMAP_SINGLE, 19661 },
	{ 0x7d3d, 0x7d3e, PDF_CMAP_RANGE, 18340 },
	{ 0x7d3f, 0x7d3f, PDF_CMAP_SINGLE, 6072 },
	{ 0x7d40, 0x7d40, PDF_CMAP_SINGLE, 18342 },
	{ 0x7d41, 0x7d41, PDF_CMAP_SINGLE, 14974 },
	{ 0x7d42, 0x7d42, PDF_CMAP_SINGLE, 2356 },
	{ 0x7d43, 0x7d43, PDF_CMAP_SINGLE, 1906 },
	{ 0x7d44, 0x7d44, PDF_CMAP_SINGLE, 2762 },
	{ 0x7d45, 0x7d45, PDF_CMAP_SINGLE, 6068 },
	{ 0x7d46, 0x7d46, PDF_CMAP_SINGLE, 6074 },
	{ 0x7d47, 0x7d47, PDF_CMAP_SINGLE, 18343 },
	{ 0x7d48, 0x7d48, PDF_CMAP_SINGLE, 8591 },
	{ 0x7d4b, 0x7d4b, PDF_CMAP_SINGLE, 6069 },
	{ 0x7d4c, 0x7d4c, PDF_CMAP_SINGLE, 1830 },
	{ 0x7d4d, 0x7d4d, PDF_CMAP_SINGLE, 19662 },
	{ 0x7d4e, 0x7d4e, PDF_CMAP_SINGLE, 6077 },
	{ 0x7d4f, 0x7d4f, PDF_CMAP_SINGLE, 6081 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 1857 },
	{ 0x7d51, 0x7d51, PDF_CMAP_SINGLE, 22200 },
	{ 0x7d53, 0x7d53, PDF_CMAP_SINGLE, 14975 },
	{ 0x7d55, 0x7d55, PDF_CMAP_SINGLE, 13882 },
	{ 0x7d56, 0x7d56, PDF_CMAP_SINGLE, 6076 },
	{ 0x7d57, 0x7d57, PDF_CMAP_SINGLE, 22201 },
	{ 0x7d59, 0x7d59, PDF_CMAP_SINGLE, 14976 },
	{ 0x7d5a, 0x7d5a, PDF_CMAP_SINGLE, 18347 },
	{ 0x7d5b, 0x7d5b, PDF_CMAP_SINGLE, 6085 },
	{ 0x7d5c, 0x7d5c, PDF_CMAP_SINGLE, 17041 },
	{ 0x7d5d, 0x7d5d, PDF_CMAP_SINGLE, 14977 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 2012 },
	{ 0x7d61, 0x7d61, PDF_CMAP_SINGLE, 3927 },
	{ 0x7d62, 0x7d62, PDF_CMAP_SINGLE, 1152 },
	{ 0x7d63, 0x7d63, PDF_CMAP_SINGLE, 6082 },
	{ 0x7d65, 0x7d65, PDF_CMAP_SINGLE, 22202 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 1669 },
	{ 0x7d67, 0x7d67, PDF_CMAP_SINGLE, 22203 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 6079 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 18348 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 6080 },
	{ 0x7d70, 0x7d70, PDF_CMAP_SINGLE, 18349 },
	{ 0x7d71, 0x7d71, PDF_CMAP_SINGLE, 3191 },
	{ 0x7d72, 0x7d72, PDF_CMAP_SINGLE, 6078 },
	{ 0x7d73, 0x7d73, PDF_CMAP_SINGLE, 6075 },
	{ 0x7d75, 0x7d75, PDF_CMAP_SINGLE, 1414 },
	{ 0x7d76, 0x7d76, PDF_CMAP_SINGLE, 2696 },
	{ 0x7d78, 0x7d78, PDF_CMAP_SINGLE, 22204 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 1884 },
	{ 0x7d7a, 0x7d7a, PDF_CMAP_SINGLE, 14978 },
	{ 0x7d7b, 0x7d7b, PDF_CMAP_SINGLE, 22205 },
	{ 0x7d7d, 0x7d7d, PDF_CMAP_SINGLE, 6087 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 18351 },
	{ 0x7d81, 0x7d81, PDF_CMAP_SINGLE, 22206 },
	{ 0x7d82, 0x7d82, PDF_CMAP_SINGLE, 19663 },
	{ 0x7d83, 0x7d83, PDF_CMAP_SINGLE, 17042 },
	{ 0x7d85, 0x7d85, PDF_CMAP_SINGLE, 19664 },
	{ 0x7d86, 0x7d86, PDF_CMAP_SINGLE, 14979 },
	{ 0x7d88, 0x7d88, PDF_CMAP_SINGLE, 18353 },
	{ 0x7d89, 0x7d89, PDF_CMAP_SINGLE, 6084 },
	{ 0x7d8b, 0x7d8c, PDF_CMAP_RANGE, 14980 },
	{ 0x7d8d, 0x7d8d, PDF_CMAP_SINGLE, 19665 },
	{ 0x7d8f, 0x7d8f, PDF_CMAP_SINGLE, 6086 },
	{ 0x7d91, 0x7d91, PDF_CMAP_SINGLE, 19666 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 6083 },
	{ 0x7d96, 0x7d96, PDF_CMAP_SINGLE, 22207 },
	{ 0x7d97, 0x7d97, PDF_CMAP_SINGLE, 18354 },
	{ 0x7d99, 0x7d99, PDF_CMAP_SINGLE, 1831 },
	{ 0x7d9a, 0x7d9a, PDF_CMAP_SINGLE, 2835 },
	{ 0x7d9b, 0x7d9b, PDF_CMAP_SINGLE, 6088 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 2800 },
	{ 0x7d9d, 0x7d9d, PDF_CMAP_SINGLE, 18356 },
	{ 0x7d9e, 0x7d9e, PDF_CMAP_SINGLE, 19667 },
	{ 0x7d9f, 0x7d9f, PDF_CMAP_SINGLE, 6101 },
	{ 0x7da0, 0x7da0, PDF_CMAP_SINGLE, 8594 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 6097 },
	{ 0x7da3, 0x7da3, PDF_CMAP_SINGLE, 6091 },
	{ 0x7da6, 0x7da6, PDF_CMAP_SINGLE, 17043 },
	{ 0x7da7, 0x7da7, PDF_CMAP_SINGLE, 18357 },
	{ 0x7daa, 0x7daa, PDF_CMAP_SINGLE, 18358 },
	{ 0x7dab, 0x7dab, PDF_CMAP_SINGLE, 6095 },
	{ 0x7dac, 0x7dac, PDF_CMAP_SINGLE, 2342 },
	{ 0x7dad, 0x7dad, PDF_CMAP_SINGLE, 1185 },
	{ 0x7dae, 0x7dae, PDF_CMAP_SINGLE, 6090 },
	{ 0x7daf, 0x7daf, PDF_CMAP_SINGLE, 6098 },
	{ 0x7db0, 0x7db0, PDF_CMAP_SINGLE, 6102 },
	{ 0x7db1, 0x7db1, PDF_CMAP_SINGLE, 2013 },
	{ 0x7db2, 0x7db2, PDF_CMAP_SINGLE, 3810 },
	{ 0x7db3, 0x7db3, PDF_CMAP_SINGLE, 19668 },
	{ 0x7db4, 0x7db4, PDF_CMAP_SINGLE, 3058 },
	{ 0x7db5, 0x7db5, PDF_CMAP_SINGLE, 6092 },
	{ 0x7db6, 0x7db6, PDF_CMAP_SINGLE, 18359 },
	{ 0x7db7, 0x7db7, PDF_CMAP_SINGLE, 8593 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 6100 },
	{ 0x7db9, 0x7db9, PDF_CMAP_SINGLE, 19669 },
	{ 0x7dba, 0x7dba, PDF_CMAP_SINGLE, 6089 },
	{ 0x7dbb, 0x7dbb, PDF_CMAP_SINGLE, 2940 },
	{ 0x7dbd, 0x7dbd, PDF_CMAP_SINGLE, 6094 },
	{ 0x7dbe, 0x7dbe, PDF_CMAP_SINGLE, 1153 },
	{ 0x7dbf, 0x7dbf, PDF_CMAP_SINGLE, 3798 },
	{ 0x7dc0, 0x7dc0, PDF_CMAP_SINGLE, 18360 },
	{ 0x7dc2, 0x7dc2, PDF_CMAP_SINGLE, 17044 },
	{ 0x7dc3, 0x7dc6, PDF_CMAP_RANGE, 22208 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 6093 },
	{ 0x7dca, 0x7dca, PDF_CMAP_SINGLE, 1747 },
	{ 0x7dcb, 0x7dcb, PDF_CMAP_SINGLE, 3456 },
	{ 0x7dcc, 0x7dcc, PDF_CMAP_SINGLE, 14982 },
	{ 0x7dcd, 0x7dce, PDF_CMAP_RANGE, 22212 },
	{ 0x7dcf, 0x7dcf, PDF_CMAP_SINGLE, 2799 },
	{ 0x7dd0, 0x7dd0, PDF_CMAP_SINGLE, 19670 },
	{ 0x7dd1, 0x7dd1, PDF_CMAP_SINGLE, 3992 },
	{ 0x7dd2, 0x7dd2, PDF_CMAP_SINGLE, 2425 },
	{ 0x7dd5, 0x7dd5, PDF_CMAP_SINGLE, 6141 },
	{ 0x7dd6, 0x7dd6, PDF_CMAP_SINGLE, 8595 },
	{ 0x7dd7, 0x7dd7, PDF_CMAP_SINGLE, 18361 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 6103 },
	{ 0x7dd9, 0x7dd9, PDF_CMAP_SINGLE, 18362 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 2722 },
	{ 0x7ddc, 0x7ddc, PDF_CMAP_SINGLE, 6099 },
	{ 0x7ddd, 0x7ddd, PDF_CMAP_SINGLE, 6104 },
	{ 0x7dde, 0x7dde, PDF_CMAP_SINGLE, 6106 },
	{ 0x7de0, 0x7de0, PDF_CMAP_SINGLE, 3093 },
	{ 0x7de1, 0x7de1, PDF_CMAP_SINGLE, 6109 },
	{ 0x7de2, 0x7de2, PDF_CMAP_SINGLE, 22215 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 13322 },
	{ 0x7de4, 0x7de4, PDF_CMAP_SINGLE, 6105 },
	{ 0x7de5, 0x7de5, PDF_CMAP_SINGLE, 19671 },
	{ 0x7de6, 0x7de6, PDF_CMAP_SINGLE, 18363 },
	{ 0x7de8, 0x7de8, PDF_CMAP_SINGLE, 3620 },
	{ 0x7de9, 0x7de9, PDF_CMAP_SINGLE, 1543 },
	{ 0x7dea, 0x7dea, PDF_CMAP_SINGLE, 22216 },
	{ 0x7deb, 0x7deb, PDF_CMAP_SINGLE, 14983 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 3799 },
	{ 0x7ded, 0x7ded, PDF_CMAP_SINGLE, 22217 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 1186 },
	{ 0x7df1, 0x7df1, PDF_CMAP_SINGLE, 14984 },
	{ 0x7df2, 0x7df2, PDF_CMAP_SINGLE, 6108 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4037 },
	{ 0x7df5, 0x7df6, PDF_CMAP_RANGE, 19672 },
	{ 0x7df9, 0x7df9, PDF_CMAP_SINGLE, 14985 },
	{ 0x7dfa, 0x7dfa, PDF_CMAP_SINGLE, 22218 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 6107 },
	{ 0x7e00, 0x7e00, PDF_CMAP_SINGLE, 22214 },
	{ 0x7e01, 0x7e01, PDF_CMAP_SINGLE, 1297 },
	{ 0x7e04, 0x7e04, PDF_CMAP_SINGLE, 3268 },
	{ 0x7e05, 0x7e05, PDF_CMAP_SINGLE, 6110 },
	{ 0x7e08, 0x7e08, PDF_CMAP_SINGLE, 14986 },
	{ 0x7e09, 0x7e09, PDF_CMAP_SINGLE, 6117 },
	{ 0x7e0a, 0x7e0a, PDF_CMAP_SINGLE, 6111 },
	{ 0x7e0b, 0x7e0b, PDF_CMAP_SINGLE, 6118 },
	{ 0x7e10, 0x7e10, PDF_CMAP_SINGLE, 18367 },
	{ 0x7e11, 0x7e11, PDF_CMAP_SINGLE, 14987 },
	{ 0x7e12, 0x7e12, PDF_CMAP_SINGLE, 6114 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 14988 },
	{ 0x7e17, 0x7e17, PDF_CMAP_SINGLE, 18368 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 3377 },
	{ 0x7e1c, 0x7e1c, PDF_CMAP_SINGLE, 22219 },
	{ 0x7e1d, 0x7e1d, PDF_CMAP_SINGLE, 18369 },
	{ 0x7e1e, 0x7e1e, PDF_CMAP_SINGLE, 2294 },
	{ 0x7e1f, 0x7e1f, PDF_CMAP_SINGLE, 6116 },
	{ 0x7e20, 0x7e20, PDF_CMAP_SINGLE, 14989 },
	{ 0x7e21, 0x7e21, PDF_CMAP_SINGLE, 6113 },
	{ 0x7e22, 0x7e22, PDF_CMAP_SINGLE, 6119 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 6112 },
	{ 0x7e26, 0x7e26, PDF_CMAP_SINGLE, 2382 },
	{ 0x7e27, 0x7e27, PDF_CMAP_SINGLE, 18370 },
	{ 0x7e28, 0x7e28, PDF_CMAP_SINGLE, 17045 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 3667 },
	{ 0x7e2c, 0x7e2c, PDF_CMAP_SINGLE, 18371 },
	{ 0x7e2d, 0x7e2d, PDF_CMAP_SINGLE, 22220 },
	{ 0x7e2e, 0x7e2e, PDF_CMAP_SINGLE, 2390 },
	{ 0x7e2f, 0x7e2f, PDF_CMAP_SINGLE, 19674 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 6115 },
	{ 0x7e32, 0x7e32, PDF_CMAP_SINGLE, 6127 },
	{ 0x7e33, 0x7e33, PDF_CMAP_SINGLE, 22221 },
	{ 0x7e35, 0x7e35, PDF_CMAP_SINGLE, 6123 },
	{ 0x7e36, 0x7e36, PDF_CMAP_SINGLE, 19675 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 6126 },
	{ 0x7e39, 0x7e39, PDF_CMAP_SINGLE, 6124 },
	{ 0x7e3a, 0x7e3a, PDF_CMAP_SINGLE, 6128 },
	{ 0x7e3b, 0x7e3b, PDF_CMAP_SINGLE, 6122 },
	{ 0x7e3d, 0x7e3d, PDF_CMAP_SINGLE, 6096 },
	{ 0x7e3e, 0x7e3e, PDF_CMAP_SINGLE, 2679 },
	{ 0x7e3f, 0x7e3f, PDF_CMAP_SINGLE, 22222 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 3423 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 6125 },
	{ 0x7e44, 0x7e44, PDF_CMAP_SINGLE, 19676 },
	{ 0x7e45, 0x7e45, PDF_CMAP_SINGLE, 18372 },
	{ 0x7e46, 0x7e46, PDF_CMAP_SINGLE, 6120 },
	{ 0x7e47, 0x7e47, PDF_CMAP_SINGLE, 14990 },
	{ 0x7e48, 0x7e48, PDF_CMAP_SINGLE, 14183 },
	{ 0x7e4a, 0x7e4a, PDF_CMAP_SINGLE, 2723 },
	{ 0x7e4b, 0x7e4b, PDF_CMAP_SINGLE, 1832 },
	{ 0x7e4d, 0x7e4d, PDF_CMAP_SINGLE, 2357 },
	{ 0x7e4e, 0x7e4e, PDF_CMAP_SINGLE, 22223 },
	{ 0x7e50, 0x7e50, PDF_CMAP_SINGLE, 22224 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 8596 },
	{ 0x7e54, 0x7e54, PDF_CMAP_SINGLE, 2539 },
	{ 0x7e55, 0x7e55, PDF_CMAP_SINGLE, 2744 },
	{ 0x7e56, 0x7e56, PDF_CMAP_SINGLE, 6131 },
	{ 0x7e58, 0x7e58, PDF_CMAP_SINGLE, 22225 },
	{ 0x7e59, 0x7e5a, PDF_CMAP_RANGE, 6133 },
	{ 0x7e5d, 0x7e5d, PDF_CMAP_SINGLE, 6130 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 6132 },
	{ 0x7e5f, 0x7e5f, PDF_CMAP_SINGLE, 22226 },
	{ 0x7e61, 0x7e61, PDF_CMAP_SINGLE, 7697 },
	{ 0x7e62, 0x7e62, PDF_CMAP_SINGLE, 14991 },
	{ 0x7e65, 0x7e65, PDF_CMAP_SINGLE, 22227 },
	{ 0x7e66, 0x7e66, PDF_CMAP_SINGLE, 6121 },
	{ 0x7e67, 0x7e67, PDF_CMAP_SINGLE, 6129 },
	{ 0x7e69, 0x7e69, PDF_CMAP_SINGLE, 6137 },
	{ 0x7e6a, 0x7e6a, PDF_CMAP_SINGLE, 6136 },
	{ 0x7e6b, 0x7e6b, PDF_CMAP_SINGLE, 7671 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 3752 },
	{ 0x7e6e, 0x7e6e, PDF_CMAP_SINGLE, 14992 },
	{ 0x7e6f, 0x7e6f, PDF_CMAP_SINGLE, 19677 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 1793 },
	{ 0x7e73, 0x7e73, PDF_CMAP_SINGLE, 14993 },
	{ 0x7e75, 0x7e75, PDF_CMAP_SINGLE, 18373 },
	{ 0x7e78, 0x7e78, PDF_CMAP_SINGLE, 19678 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 6135 },
	{ 0x7e7b, 0x7e7b, PDF_CMAP_SINGLE, 6139 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 6138 },
	{ 0x7e7d, 0x7e7d, PDF_CMAP_SINGLE, 6142 },
	{ 0x7e7e, 0x7e7e, PDF_CMAP_SINGLE, 18374 },
	{ 0x7e7f, 0x7e7f, PDF_CMAP_SINGLE, 6144 },
	{ 0x7e81, 0x7e81, PDF_CMAP_SINGLE, 19679 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 2186 },
	{ 0x7e83, 0x7e83, PDF_CMAP_SINGLE, 6140 },
	{ 0x7e86, 0x7e87, PDF_CMAP_RANGE, 18375 },
	{ 0x7e88, 0x7e89, PDF_CMAP_RANGE, 6145 },
	{ 0x7e8a, 0x7e8a, PDF_CMAP_SINGLE, 8359 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 6147 },
	{ 0x7e8d, 0x7e8d, PDF_CMAP_SINGLE, 14994 },
	{ 0x7e8e, 0x7e8e, PDF_CMAP_SINGLE, 6153 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 3125 },
	{ 0x7e90, 0x7e90, PDF_CMAP_SINGLE, 6149 },
	{ 0x7e91, 0x7e91, PDF_CMAP_SINGLE, 14995 },
	{ 0x7e92, 0x7e92, PDF_CMAP_SINGLE, 6148 },
	{ 0x7e93, 0x7e94, PDF_CMAP_RANGE, 6150 },
	{ 0x7e95, 0x7e95, PDF_CMAP_SINGLE, 22228 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 6152 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 14996 },
	{ 0x7e9a, 0x7e9a, PDF_CMAP_SINGLE, 18378 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_RANGE, 6154 },
	{ 0x7e9d, 0x7e9e, PDF_CMAP_RANGE, 22229 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 1544 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 6156 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 6157 },
	{ 0x7f3b, 0x7f3b, PDF_CMAP_SINGLE, 18381 },
	{ 0x7f3c, 0x7f3c, PDF_CMAP_SINGLE, 18380 },
	{ 0x7f3d, 0x7f3d, PDF_CMAP_SINGLE, 19680 },
	{ 0x7f3e, 0x7f3e, PDF_CMAP_SINGLE, 18382 },
	{ 0x7f3f, 0x7f3f, PDF_CMAP_SINGLE, 22231 },
	{ 0x7f43, 0x7f43, PDF_CMAP_SINGLE, 18383 },
	{ 0x7f44, 0x7f44, PDF_CMAP_SINGLE, 14997 },
	{ 0x7f45, 0x7f45, PDF_CMAP_SINGLE, 6158 },
	{ 0x7f47, 0x7f47, PDF_CMAP_SINGLE, 8597 },
	{ 0x7f4c, 0x7f4e, PDF_CMAP_RANGE, 6159 },
	{ 0x7f4f, 0x7f4f, PDF_CMAP_SINGLE, 14998 },
	{ 0x7f50, 0x7f51, PDF_CMAP_RANGE, 6162 },
	{ 0x7f52, 0x7f53, PDF_CMAP_RANGE, 14999 },
	{ 0x7f54, 0x7f54, PDF_CMAP_SINGLE, 6165 },
	{ 0x7f55, 0x7f55, PDF_CMAP_SINGLE, 6164 },
	{ 0x7f58, 0x7f58, PDF_CMAP_SINGLE, 6166 },
	{ 0x7f5b, 0x7f5b, PDF_CMAP_SINGLE, 19681 },
	{ 0x7f5c, 0x7f5c, PDF_CMAP_SINGLE, 22232 },
	{ 0x7f5d, 0x7f5d, PDF_CMAP_SINGLE, 19682 },
	{ 0x7f5f, 0x7f60, PDF_CMAP_RANGE, 6167 },
	{ 0x7f61, 0x7f61, PDF_CMAP_SINGLE, 15001 },
	{ 0x7f63, 0x7f64, PDF_CMAP_RANGE, 18387 },
	{ 0x7f65, 0x7f65, PDF_CMAP_SINGLE, 19683 },
	{ 0x7f66, 0x7f66, PDF_CMAP_SINGLE, 22233 },
	{ 0x7f67, 0x7f67, PDF_CMAP_SINGLE, 6171 },
	{ 0x7f68, 0x7f69, PDF_CMAP_RANGE, 6169 },
	{ 0x7f6a, 0x7f6a, PDF_CMAP_SINGLE, 2129 },
	{ 0x7f6b, 0x7f6b, PDF_CMAP_SINGLE, 1833 },
	{ 0x7f6d, 0x7f6d, PDF_CMAP_SINGLE, 18389 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 2964 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 3399 },
	{ 0x7f71, 0x7f71, PDF_CMAP_SINGLE, 19684 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 2426 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 3331 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 3457 },
	{ 0x7f78, 0x7f78, PDF_CMAP_SINGLE, 6172 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 4918 },
	{ 0x7f7d, 0x7f7e, PDF_CMAP_RANGE, 18390 },
	{ 0x7f7f, 0x7f80, PDF_CMAP_RANGE, 19685 },
	{ 0x7f82, 0x7f82, PDF_CMAP_SINGLE, 6173 },
	{ 0x7f83, 0x7f83, PDF_CMAP_SINGLE, 6175 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 3919 },
	{ 0x7f86, 0x7f86, PDF_CMAP_SINGLE, 6174 },
	{ 0x7f87, 0x7f87, PDF_CMAP_SINGLE, 6177 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 6176 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 3901 },
	{ 0x7f8b, 0x7f8b, PDF_CMAP_SINGLE, 19687 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 6178 },
	{ 0x7f8d, 0x7f8d, PDF_CMAP_SINGLE, 22235 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 3474 },
	{ 0x7f8f, 0x7f8f, PDF_CMAP_SINGLE, 22236 },
	{ 0x7f90, 0x7f90, PDF_CMAP_SINGLE, 18392 },
	{ 0x7f91, 0x7f91, PDF_CMAP_SINGLE, 15002 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 6179 },
	{ 0x7f96, 0x7f96, PDF_CMAP_SINGLE, 18395 },
	{ 0x7f97, 0x7f97, PDF_CMAP_SINGLE, 17046 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 6182 },
	{ 0x7f9c, 0x7f9c, PDF_CMAP_SINGLE, 18396 },
	{ 0x7f9d, 0x7f9d, PDF_CMAP_SINGLE, 6181 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 6180 },
	{ 0x7fa1, 0x7fa1, PDF_CMAP_SINGLE, 8598 },
	{ 0x7fa2, 0x7fa2, PDF_CMAP_SINGLE, 19688 },
	{ 0x7fa3, 0x7fa3, PDF_CMAP_SINGLE, 6183 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 1800 },
	{ 0x7fa6, 0x7fa6, PDF_CMAP_SINGLE, 22237 },
	{ 0x7fa8, 0x7fa8, PDF_CMAP_SINGLE, 2724 },
	{ 0x7fa9, 0x7fa9, PDF_CMAP_SINGLE, 1627 },
	{ 0x7faa, 0x7faa, PDF_CMAP_SINGLE, 22238 },
	{ 0x7fad, 0x7fad, PDF_CMAP_SINGLE, 18397 },
	{ 0x7fae, 0x7fae, PDF_CMAP_SINGLE, 6187 },
	{ 0x7faf, 0x7faf, PDF_CMAP_SINGLE, 6184 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 6185 },
	{ 0x7fb4, 0x7fb4, PDF_CMAP_SINGLE, 22239 },
	{ 0x7fb6, 0x7fb6, PDF_CMAP_SINGLE, 6188 },
	{ 0x7fb8, 0x7fb8, PDF_CMAP_SINGLE, 6189 },
	{ 0x7fb9, 0x7fb9, PDF_CMAP_SINGLE, 6186 },
	{ 0x7fbc, 0x7fbc, PDF_CMAP_SINGLE, 22240 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 1227 },
	{ 0x7fbf, 0x7fbf, PDF_CMAP_SINGLE, 15003 },
	{ 0x7fc0, 0x7fc0, PDF_CMAP_SINGLE, 22241 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 1319 },
	{ 0x7fc3, 0x7fc3, PDF_CMAP_SINGLE, 18399 },
	{ 0x7fc5, 0x7fc6, PDF_CMAP_RANGE, 6191 },
	{ 0x7fc8, 0x7fc8, PDF_CMAP_SINGLE, 22242 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 6193 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 3916 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 15004 },
	{ 0x7fcf, 0x7fcf, PDF_CMAP_SINGLE, 18400 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 2358 },
	{ 0x7fd4, 0x7fd4, PDF_CMAP_SINGLE, 6195 },
	{ 0x7fd5, 0x7fd5, PDF_CMAP_SINGLE, 6194 },
	{ 0x7fdb, 0x7fdb, PDF_CMAP_SINGLE, 17047 },
	{ 0x7fdf, 0x7fdf, PDF_CMAP_SINGLE, 15005 },
	{ 0x7fe0, 0x7fe0, PDF_CMAP_SINGLE, 2607 },
	{ 0x7fe1, 0x7fe1, PDF_CMAP_SINGLE, 6196 },
	{ 0x7fe3, 0x7fe3, PDF_CMAP_SINGLE, 18401 },
	{ 0x7fe5, 0x7fe5, PDF_CMAP_SINGLE, 15006 },
	{ 0x7fe6, 0x7fe6, PDF_CMAP_SINGLE, 6197 },
	{ 0x7fe8, 0x7fe8, PDF_CMAP_SINGLE, 22243 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 6198 },
	{ 0x7feb, 0x7feb, PDF_CMAP_SINGLE, 1569 },
	{ 0x7fec, 0x7fec, PDF_CMAP_SINGLE, 15007 },
	{ 0x7fee, 0x7fef, PDF_CMAP_RANGE, 15008 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 1545 },
	{ 0x7ff2, 0x7ff2, PDF_CMAP_SINGLE, 18402 },
	{ 0x7ff3, 0x7ff3, PDF_CMAP_SINGLE, 6199 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 6200 },
	{ 0x7ffa, 0x7ffa, PDF_CMAP_SINGLE, 15010 },
	{ 0x7ffb, 0x7ffb, PDF_CMAP_SINGLE, 3723 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 3917 },
	{ 0x7ffd, 0x7fff, PDF_CMAP_RANGE, 19689 },
	{ 0x8000, 0x8000, PDF_CMAP_SINGLE, 3902 },
	{ 0x8001, 0x8001, PDF_CMAP_SINGLE, 4061 },
	{ 0x8002, 0x8002, PDF_CMAP_SINGLE, 14099 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 2015 },
	{ 0x8004, 0x8004, PDF_CMAP_SINGLE, 6203 },
	{ 0x8005, 0x8005, PDF_CMAP_SINGLE, 2304 },
	{ 0x8006, 0x8006, PDF_CMAP_SINGLE, 6202 },
	{ 0x8007, 0x8007, PDF_CMAP_SINGLE, 19692 },
	{ 0x8008, 0x8008, PDF_CMAP_SINGLE, 18404 },
	{ 0x800a, 0x800a, PDF_CMAP_SINGLE, 18403 },
	{ 0x800b, 0x800b, PDF_CMAP_SINGLE, 6204 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 2261 },
	{ 0x800d, 0x800d, PDF_CMAP_SINGLE, 19693 },
	{ 0x800e, 0x800e, PDF_CMAP_SINGLE, 15011 },
	{ 0x800f, 0x800f, PDF_CMAP_SINGLE, 22244 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 2865 },
	{ 0x8011, 0x8011, PDF_CMAP_SINGLE, 15012 },
	{ 0x8012, 0x8012, PDF_CMAP_SINGLE, 6205 },
	{ 0x8013, 0x8013, PDF_CMAP_SINGLE, 22245 },
	{ 0x8014, 0x8014, PDF_CMAP_SINGLE, 15013 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 2014 },
	{ 0x8016, 0x8016, PDF_CMAP_SINGLE, 18405 },
	{ 0x8017, 0x8017, PDF_CMAP_SINGLE, 3811 },
	{ 0x8018, 0x8019, PDF_CMAP_RANGE, 6206 },
	{ 0x801c, 0x801c, PDF_CMAP_SINGLE, 6208 },
	{ 0x801d, 0x801d, PDF_CMAP_SINGLE, 22246 },
	{ 0x801e, 0x801e, PDF_CMAP_SINGLE, 19694 },
	{ 0x801f, 0x8020, PDF_CMAP_RANGE, 22247 },
	{ 0x8021, 0x8021, PDF_CMAP_SINGLE, 6209 },
	{ 0x8024, 0x8024, PDF_CMAP_SINGLE, 15014 },
	{ 0x8026, 0x8026, PDF_CMAP_SINGLE, 15015 },
	{ 0x8028, 0x8028, PDF_CMAP_SINGLE, 6210 },
	{ 0x802c, 0x802c, PDF_CMAP_SINGLE, 18406 },
	{ 0x802e, 0x802e, PDF_CMAP_SINGLE, 22249 },
	{ 0x8030, 0x8030, PDF_CMAP_SINGLE, 18407 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 2262 },
	{ 0x8034, 0x8034, PDF_CMAP_SINGLE, 22250 },
	{ 0x8035, 0x8035, PDF_CMAP_SINGLE, 17048 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 3833 },
	{ 0x8037, 0x8037, PDF_CMAP_SINGLE, 17049 },
	{ 0x8039, 0x8039, PDF_CMAP_SINGLE, 19695 },
	{ 0x803a, 0x803a, PDF_CMAP_SINGLE, 15016 },
	{ 0x803b, 0x803b, PDF_CMAP_SINGLE, 6212 },
	{ 0x803c, 0x803c, PDF_CMAP_SINGLE, 15017 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 2941 },
	{ 0x803e, 0x803e, PDF_CMAP_SINGLE, 22251 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 6211 },
	{ 0x8040, 0x8040, PDF_CMAP_SINGLE, 22252 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 18408 },
	{ 0x8044, 0x8044, PDF_CMAP_SINGLE, 22253 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 6214 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 6213 },
	{ 0x8052, 0x8052, PDF_CMAP_SINGLE, 6215 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 2655 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 6216 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 6217 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 3593 },
	{ 0x805f, 0x805f, PDF_CMAP_SINGLE, 6218 },
	{ 0x8060, 0x8060, PDF_CMAP_SINGLE, 15018 },
	{ 0x8061, 0x8061, PDF_CMAP_SINGLE, 2801 },
	{ 0x8062, 0x8062, PDF_CMAP_SINGLE, 6219 },
	{ 0x8064, 0x8064, PDF_CMAP_SINGLE, 22254 },
	{ 0x8066, 0x8066, PDF_CMAP_SINGLE, 18409 },
	{ 0x8068, 0x8068, PDF_CMAP_SINGLE, 6220 },
	{ 0x806d, 0x806d, PDF_CMAP_SINGLE, 22255 },
	{ 0x806f, 0x806f, PDF_CMAP_SINGLE, 4038 },
	{ 0x8070, 0x8070, PDF_CMAP_SINGLE, 6223 },
	{ 0x8071, 0x8071, PDF_CMAP_SINGLE, 15019 },
	{ 0x8072, 0x8072, PDF_CMAP_SINGLE, 6222 },
	{ 0x8073, 0x8073, PDF_CMAP_SINGLE, 6221 },
	{ 0x8074, 0x8074, PDF_CMAP_SINGLE, 3020 },
	{ 0x8075, 0x8075, PDF_CMAP_SINGLE, 15020 },
	{ 0x8076, 0x8076, PDF_CMAP_SINGLE, 6224 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 2540 },
	{ 0x8079, 0x8079, PDF_CMAP_SINGLE, 6225 },
	{ 0x807b, 0x807b, PDF_CMAP_SINGLE, 18410 },
	{ 0x807d, 0x807d, PDF_CMAP_SINGLE, 6226 },
	{ 0x807e, 0x807e, PDF_CMAP_SINGLE, 4062 },
	{ 0x807f, 0x807f, PDF_CMAP_SINGLE, 6227 },
	{ 0x8081, 0x8081, PDF_CMAP_SINGLE, 22256 },
	{ 0x8084, 0x8084, PDF_CMAP_SINGLE, 6228 },
	{ 0x8085, 0x8085, PDF_CMAP_SINGLE, 6230 },
	{ 0x8086, 0x8086, PDF_CMAP_SINGLE, 6229 },
	{ 0x8087, 0x8087, PDF_CMAP_SINGLE, 3385 },
	{ 0x8088, 0x8088, PDF_CMAP_SINGLE, 19696 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 3281 },
	{ 0x808b, 0x808b, PDF_CMAP_SINGLE, 4068 },
	{ 0x808c, 0x808c, PDF_CMAP_SINGLE, 3389 },
	{ 0x808e, 0x808e, PDF_CMAP_SINGLE, 19697 },
	{ 0x8093, 0x8093, PDF_CMAP_SINGLE, 6232 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 2491 },
	{ 0x8098, 0x8098, PDF_CMAP_SINGLE, 3484 },
	{ 0x8099, 0x8099, PDF_CMAP_SINGLE, 18411 },
	{ 0x809a, 0x809a, PDF_CMAP_SINGLE, 6233 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 6231 },
	{ 0x809c, 0x809c, PDF_CMAP_SINGLE, 18412 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 1546 },
	{ 0x809e, 0x809e, PDF_CMAP_SINGLE, 15021 },
	{ 0x80a1, 0x80a1, PDF_CMAP_SINGLE, 1928 },
	{ 0x80a2, 0x80a2, PDF_CMAP_SINGLE, 2230 },
	{ 0x80a4, 0x80a4, PDF_CMAP_SINGLE, 18413 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 3458 },
	{ 0x80a6, 0x80a6, PDF_CMAP_SINGLE, 15022 },
	{ 0x80a7, 0x80a7, PDF_CMAP_SINGLE, 18414 },
	{ 0x80a9, 0x80a9, PDF_CMAP_SINGLE, 1886 },
	{ 0x80aa, 0x80aa, PDF_CMAP_SINGLE, 3697 },
	{ 0x80ab, 0x80ab, PDF_CMAP_SINGLE, 15023 },
	{ 0x80ac, 0x80ac, PDF_CMAP_SINGLE, 6236 },
	{ 0x80ad, 0x80ad, PDF_CMAP_SINGLE, 6234 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 2016 },
	{ 0x80b1, 0x80b1, PDF_CMAP_SINGLE, 2017 },
	{ 0x80b2, 0x80b2, PDF_CMAP_SINGLE, 1197 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 2136 },
	{ 0x80b8, 0x80b8, PDF_CMAP_SINGLE, 18415 },
	{ 0x80b9, 0x80b9, PDF_CMAP_SINGLE, 22257 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 3343 },
	{ 0x80c3, 0x80c3, PDF_CMAP_SINGLE, 1187 },
	{ 0x80c4, 0x80c4, PDF_CMAP_SINGLE, 6241 },
	{ 0x80c5, 0x80c5, PDF_CMAP_SINGLE, 18417 },
	{ 0x80c6, 0x80c6, PDF_CMAP_SINGLE, 2942 },
	{ 0x80c8, 0x80c8, PDF_CMAP_SINGLE, 22258 },
	{ 0x80ca, 0x80ca, PDF_CMAP_SINGLE, 17050 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 3342 },
	{ 0x80cd, 0x80cd, PDF_CMAP_SINGLE, 22259 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 2875 },
	{ 0x80cf, 0x80cf, PDF_CMAP_SINGLE, 19698 },
	{ 0x80d2, 0x80d2, PDF_CMAP_SINGLE, 22260 },
	{ 0x80d4, 0x80d4, PDF_CMAP_SINGLE, 19699 },
	{ 0x80d5, 0x80d5, PDF_CMAP_SINGLE, 18418 },
	{ 0x80d6, 0x80d6, PDF_CMAP_SINGLE, 6243 },
	{ 0x80d7, 0x80d8, PDF_CMAP_RANGE, 15024 },
	{ 0x80d9, 0x80d9, PDF_CMAP_SINGLE, 6239 },
	{ 0x80da, 0x80da, PDF_CMAP_SINGLE, 6242 },
	{ 0x80db, 0x80db, PDF_CMAP_SINGLE, 6237 },
	{ 0x80dd, 0x80dd, PDF_CMAP_SINGLE, 6240 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 3668 },
	{ 0x80e0, 0x80e0, PDF_CMAP_SINGLE, 17051 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 1929 },
	{ 0x80e4, 0x80e4, PDF_CMAP_SINGLE, 1217 },
	{ 0x80e5, 0x80e5, PDF_CMAP_SINGLE, 6238 },
	{ 0x80e6, 0x80e6, PDF_CMAP_SINGLE, 18419 },
	{ 0x80ed, 0x80ed, PDF_CMAP_SINGLE, 19700 },
	{ 0x80ee, 0x80ee, PDF_CMAP_SINGLE, 22261 },
	{ 0x80ef, 0x80ef, PDF_CMAP_SINGLE, 6245 },
	{ 0x80f0, 0x80f0, PDF_CMAP_SINGLE, 19701 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 6246 },
	{ 0x80f2, 0x80f2, PDF_CMAP_SINGLE, 22262 },
	{ 0x80f3, 0x80f3, PDF_CMAP_SINGLE, 17052 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 3217 },
	{ 0x80f5, 0x80f5, PDF_CMAP_SINGLE, 18421 },
	{ 0x80f6, 0x80f6, PDF_CMAP_SINGLE, 22263 },
	{ 0x80f7, 0x80f7, PDF_CMAP_SINGLE, 19702 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 1715 },
	{ 0x80f9, 0x80f9, PDF_CMAP_SINGLE, 22264 },
	{ 0x80fa, 0x80fa, PDF_CMAP_SINGLE, 19703 },
	{ 0x80fb, 0x80fb, PDF_CMAP_SINGLE, 18422 },
	{ 0x80fc, 0x80fc, PDF_CMAP_SINGLE, 6257 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 3315 },
	{ 0x80fe, 0x80fe, PDF_CMAP_SINGLE, 19704 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 2231 },
	{ 0x8103, 0x8103, PDF_CMAP_SINGLE, 19705 },
	{ 0x8105, 0x8105, PDF_CMAP_SINGLE, 1716 },
	{ 0x8106, 0x8106, PDF_CMAP_SINGLE, 2668 },
	{ 0x8107, 0x8107, PDF_CMAP_SINGLE, 4076 },
	{ 0x8108, 0x8108, PDF_CMAP_SINGLE, 3770 },
	{ 0x8109, 0x8109, PDF_CMAP_SINGLE, 6244 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 2680 },
	{ 0x810b, 0x810b, PDF_CMAP_SINGLE, 22265 },
	{ 0x810d, 0x810d, PDF_CMAP_SINGLE, 18420 },
	{ 0x8116, 0x8116, PDF_CMAP_SINGLE, 15026 },
	{ 0x8117, 0x8117, PDF_CMAP_SINGLE, 19706 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 15027 },
	{ 0x811a, 0x811a, PDF_CMAP_SINGLE, 1645 },
	{ 0x811b, 0x811b, PDF_CMAP_SINGLE, 6247 },
	{ 0x811c, 0x811c, PDF_CMAP_SINGLE, 22266 },
	{ 0x811e, 0x811e, PDF_CMAP_SINGLE, 18425 },
	{ 0x8120, 0x8120, PDF_CMAP_SINGLE, 22267 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 6249 },
	{ 0x8124, 0x8124, PDF_CMAP_SINGLE, 18427 },
	{ 0x8127, 0x8127, PDF_CMAP_SINGLE, 18428 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 6248 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 13913 },
	{ 0x812c, 0x812c, PDF_CMAP_SINGLE, 18429 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 6250 },
	{ 0x8130, 0x8130, PDF_CMAP_SINGLE, 19707 },
	{ 0x8131, 0x8131, PDF_CMAP_SINGLE, 2916 },
	{ 0x8133, 0x8133, PDF_CMAP_SINGLE, 3316 },
	{ 0x8135, 0x8135, PDF_CMAP_SINGLE, 18424 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 3021 },
	{ 0x813a, 0x813a, PDF_CMAP_SINGLE, 15028 },
	{ 0x813c, 0x813c, PDF_CMAP_SINGLE, 22268 },
	{ 0x813d, 0x813d, PDF_CMAP_SINGLE, 18431 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 6254 },
	{ 0x8141, 0x8141, PDF_CMAP_SINGLE, 14194 },
	{ 0x8145, 0x8145, PDF_CMAP_SINGLE, 22269 },
	{ 0x8146, 0x8146, PDF_CMAP_SINGLE, 6253 },
	{ 0x8147, 0x8147, PDF_CMAP_SINGLE, 22270 },
	{ 0x814a, 0x814a, PDF_CMAP_SINGLE, 15029 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 6251 },
	{ 0x814c, 0x814c, PDF_CMAP_SINGLE, 15030 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 2587 },
	{ 0x8150, 0x8150, PDF_CMAP_SINGLE, 3543 },
	{ 0x8151, 0x8151, PDF_CMAP_SINGLE, 6256 },
	{ 0x8152, 0x8152, PDF_CMAP_SINGLE, 22271 },
	{ 0x8153, 0x8153, PDF_CMAP_SINGLE, 6255 },
	{ 0x8154, 0x8154, PDF_CMAP_SINGLE, 2018 },
	{ 0x8155, 0x8155, PDF_CMAP_SINGLE, 4089 },
	{ 0x8157, 0x8157, PDF_CMAP_SINGLE, 19708 },
	{ 0x815f, 0x815f, PDF_CMAP_SINGLE, 6272 },
	{ 0x8160, 0x8160, PDF_CMAP_SINGLE, 17053 },
	{ 0x8161, 0x8161, PDF_CMAP_SINGLE, 22272 },
	{ 0x8165, 0x8166, PDF_CMAP_RANGE, 6260 },
	{ 0x8167, 0x8168, PDF_CMAP_RANGE, 17054 },
	{ 0x8169, 0x8169, PDF_CMAP_SINGLE, 18433 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 2332 },
	{ 0x816d, 0x816d, PDF_CMAP_SINGLE, 17056 },
	{ 0x816e, 0x816e, PDF_CMAP_SINGLE, 6259 },
	{ 0x816f, 0x816f, PDF_CMAP_SINGLE, 19709 },
	{ 0x8170, 0x8170, PDF_CMAP_SINGLE, 2058 },
	{ 0x8171, 0x8171, PDF_CMAP_SINGLE, 6258 },
	{ 0x8173, 0x8173, PDF_CMAP_SINGLE, 19710 },
	{ 0x8174, 0x8174, PDF_CMAP_SINGLE, 6262 },
	{ 0x8177, 0x8177, PDF_CMAP_SINGLE, 22273 },
	{ 0x8178, 0x8178, PDF_CMAP_SINGLE, 3022 },
	{ 0x8179, 0x8179, PDF_CMAP_SINGLE, 3570 },
	{ 0x817a, 0x817a, PDF_CMAP_SINGLE, 2725 },
	{ 0x817f, 0x817f, PDF_CMAP_SINGLE, 2876 },
	{ 0x8180, 0x8180, PDF_CMAP_SINGLE, 6266 },
	{ 0x8181, 0x8181, PDF_CMAP_SINGLE, 15031 },
	{ 0x8182, 0x8182, PDF_CMAP_SINGLE, 6267 },
	{ 0x8183, 0x8183, PDF_CMAP_SINGLE, 6263 },
	{ 0x8184, 0x8184, PDF_CMAP_SINGLE, 15032 },
	{ 0x8185, 0x8185, PDF_CMAP_SINGLE, 18436 },
	{ 0x8186, 0x8186, PDF_CMAP_SINGLE, 22274 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 6264 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 6265 },
	{ 0x818b, 0x818b, PDF_CMAP_SINGLE, 19711 },
	{ 0x818e, 0x818e, PDF_CMAP_SINGLE, 22275 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 2019 },
	{ 0x8190, 0x8190, PDF_CMAP_SINGLE, 19712 },
	{ 0x8193, 0x8193, PDF_CMAP_SINGLE, 6273 },
	{ 0x8195, 0x8195, PDF_CMAP_SINGLE, 6269 },
	{ 0x8196, 0x8196, PDF_CMAP_SINGLE, 22276 },
	{ 0x8198, 0x8198, PDF_CMAP_SINGLE, 18438 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 3544 },
	{ 0x819b, 0x819b, PDF_CMAP_SINGLE, 19713 },
	{ 0x819c, 0x819c, PDF_CMAP_SINGLE, 3738 },
	{ 0x819d, 0x819d, PDF_CMAP_SINGLE, 3482 },
	{ 0x819e, 0x819e, PDF_CMAP_SINGLE, 19714 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 6268 },
	{ 0x81a2, 0x81a2, PDF_CMAP_SINGLE, 22277 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 6271 },
	{ 0x81a4, 0x81a4, PDF_CMAP_SINGLE, 6270 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 3698 },
	{ 0x81a9, 0x81a9, PDF_CMAP_SINGLE, 6274 },
	{ 0x81ae, 0x81ae, PDF_CMAP_SINGLE, 22278 },
	{ 0x81b0, 0x81b0, PDF_CMAP_SINGLE, 6275 },
	{ 0x81b2, 0x81b2, PDF_CMAP_SINGLE, 18439 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 2745 },
	{ 0x81b4, 0x81b4, PDF_CMAP_SINGLE, 15033 },
	{ 0x81b5, 0x81b5, PDF_CMAP_SINGLE, 6276 },
	{ 0x81b8, 0x81b8, PDF_CMAP_SINGLE, 6278 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 6282 },
	{ 0x81bb, 0x81bb, PDF_CMAP_SINGLE, 17057 },
	{ 0x81bd, 0x81bd, PDF_CMAP_SINGLE, 6279 },
	{ 0x81be, 0x81be, PDF_CMAP_SINGLE, 6277 },
	{ 0x81bf, 0x81bf, PDF_CMAP_SINGLE, 3317 },
	{ 0x81c0, 0x81c0, PDF_CMAP_SINGLE, 6280 },
	{ 0x81c1, 0x81c1, PDF_CMAP_SINGLE, 18440 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 6281 },
	{ 0x81c3, 0x81c3, PDF_CMAP_SINGLE, 18441 },
	{ 0x81c5, 0x81c5, PDF_CMAP_SINGLE, 22279 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 1330 },
	{ 0x81c8, 0x81c8, PDF_CMAP_SINGLE, 6288 },
	{ 0x81c9, 0x81c9, PDF_CMAP_SINGLE, 6283 },
	{ 0x81ca, 0x81ca, PDF_CMAP_SINGLE, 17058 },
	{ 0x81cb, 0x81cb, PDF_CMAP_SINGLE, 19715 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 6284 },
	{ 0x81ce, 0x81ce, PDF_CMAP_SINGLE, 22280 },
	{ 0x81cf, 0x81cf, PDF_CMAP_SINGLE, 15034 },
	{ 0x81d1, 0x81d1, PDF_CMAP_SINGLE, 6285 },
	{ 0x81d3, 0x81d3, PDF_CMAP_SINGLE, 2817 },
	{ 0x81d5, 0x81d5, PDF_CMAP_SINGLE, 19716 },
	{ 0x81d6, 0x81d6, PDF_CMAP_SINGLE, 18442 },
	{ 0x81d7, 0x81d7, PDF_CMAP_SINGLE, 17059 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 6287 },
	{ 0x81d9, 0x81d9, PDF_CMAP_SINGLE, 6286 },
	{ 0x81da, 0x81da, PDF_CMAP_SINGLE, 6289 },
	{ 0x81db, 0x81db, PDF_CMAP_SINGLE, 18443 },
	{ 0x81dd, 0x81de, PDF_CMAP_RANGE, 19717 },
	{ 0x81df, 0x81e0, PDF_CMAP_RANGE, 6290 },
	{ 0x81e1, 0x81e1, PDF_CMAP_SINGLE, 19719 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 2569 },
	{ 0x81e4, 0x81e4, PDF_CMAP_SINGLE, 18445 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 1385 },
	{ 0x81e7, 0x81e7, PDF_CMAP_SINGLE, 6292 },
	{ 0x81e8, 0x81e8, PDF_CMAP_SINGLE, 3999 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 2263 },
	{ 0x81eb, 0x81eb, PDF_CMAP_SINGLE, 22281 },
	{ 0x81ec, 0x81ec, PDF_CMAP_SINGLE, 18447 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 2359 },
	{ 0x81ef, 0x81ef, PDF_CMAP_SINGLE, 19720 },
	{ 0x81f0, 0x81f2, PDF_CMAP_RANGE, 22282 },
	{ 0x81f3, 0x81f3, PDF_CMAP_SINGLE, 2232 },
	{ 0x81f4, 0x81f4, PDF_CMAP_SINGLE, 2965 },
	{ 0x81f5, 0x81f5, PDF_CMAP_SINGLE, 22285 },
	{ 0x81f6, 0x81f6, PDF_CMAP_SINGLE, 19721 },
	{ 0x81f8, 0x81f8, PDF_CMAP_SINGLE, 22286 },
	{ 0x81f9, 0x81f9, PDF_CMAP_SINGLE, 15035 },
	{ 0x81fa, 0x81fb, PDF_CMAP_RANGE, 6293 },
	{ 0x81fc, 0x81fc, PDF_CMAP_SINGLE, 1235 },
	{ 0x81fd, 0x81fd, PDF_CMAP_SINGLE, 18448 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 6295 },
	{ 0x81ff, 0x81ff, PDF_CMAP_SINGLE, 18449 },
	{ 0x8200, 0x8200, PDF_CMAP_SINGLE, 19722 },
	{ 0x8201, 0x8202, PDF_CMAP_RANGE, 6296 },
	{ 0x8203, 0x8203, PDF_CMAP_SINGLE, 15036 },
	{ 0x8204, 0x8204, PDF_CMAP_SINGLE, 18451 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 6298 },
	{ 0x8207, 0x8207, PDF_CMAP_SINGLE, 6299 },
	{ 0x8208, 0x8208, PDF_CMAP_SINGLE, 1717 },
	{ 0x8209, 0x8209, PDF_CMAP_SINGLE, 5039 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 6300 },
	{ 0x820b, 0x820b, PDF_CMAP_SINGLE, 19723 },
	{ 0x820c, 0x820c, PDF_CMAP_SINGLE, 2697 },
	{ 0x820d, 0x820d, PDF_CMAP_SINGLE, 6301 },
	{ 0x820e, 0x820e, PDF_CMAP_SINGLE, 2295 },
	{ 0x820f, 0x820f, PDF_CMAP_SINGLE, 22288 },
	{ 0x8210, 0x8210, PDF_CMAP_SINGLE, 6302 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 4105 },
	{ 0x8213, 0x8214, PDF_CMAP_RANGE, 19724 },
	{ 0x8216, 0x8216, PDF_CMAP_SINGLE, 6303 },
	{ 0x8217, 0x8217, PDF_CMAP_SINGLE, 3630 },
	{ 0x8218, 0x8218, PDF_CMAP_SINGLE, 1560 },
	{ 0x8219, 0x8219, PDF_CMAP_SINGLE, 18453 },
	{ 0x821a, 0x821a, PDF_CMAP_SINGLE, 19726 },
	{ 0x821b, 0x821b, PDF_CMAP_SINGLE, 2726 },
	{ 0x821c, 0x821c, PDF_CMAP_SINGLE, 2402 },
	{ 0x821d, 0x821d, PDF_CMAP_SINGLE, 22289 },
	{ 0x821e, 0x821e, PDF_CMAP_SINGLE, 3555 },
	{ 0x821f, 0x821f, PDF_CMAP_SINGLE, 2360 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 15037 },
	{ 0x8222, 0x8222, PDF_CMAP_SINGLE, 18454 },
	{ 0x8228, 0x8228, PDF_CMAP_SINGLE, 22290 },
	{ 0x8229, 0x8229, PDF_CMAP_SINGLE, 6304 },
	{ 0x822a, 0x822a, PDF_CMAP_SINGLE, 2020 },
	{ 0x822b, 0x822b, PDF_CMAP_SINGLE, 6305 },
	{ 0x822c, 0x822c, PDF_CMAP_SINGLE, 3424 },
	{ 0x822e, 0x822e, PDF_CMAP_SINGLE, 6319 },
	{ 0x8232, 0x8232, PDF_CMAP_SINGLE, 15038 },
	{ 0x8233, 0x8233, PDF_CMAP_SINGLE, 6307 },
	{ 0x8234, 0x8234, PDF_CMAP_SINGLE, 15039 },
	{ 0x8235, 0x8235, PDF_CMAP_SINGLE, 2857 },
	{ 0x8236, 0x8236, PDF_CMAP_SINGLE, 3371 },
	{ 0x8237, 0x8237, PDF_CMAP_SINGLE, 1907 },
	{ 0x8238, 0x8238, PDF_CMAP_SINGLE, 6306 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 2727 },
	{ 0x823a, 0x823a, PDF_CMAP_SINGLE, 19727 },
	{ 0x823c, 0x823c, PDF_CMAP_SINGLE, 18456 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 6308 },
	{ 0x8243, 0x8243, PDF_CMAP_SINGLE, 22291 },
	{ 0x8244, 0x8244, PDF_CMAP_SINGLE, 19728 },
	{ 0x8245, 0x8245, PDF_CMAP_SINGLE, 18458 },
	{ 0x8246, 0x8246, PDF_CMAP_SINGLE, 15040 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 3094 },
	{ 0x8249, 0x8249, PDF_CMAP_SINGLE, 18457 },
	{ 0x824b, 0x824b, PDF_CMAP_SINGLE, 15041 },
	{ 0x824e, 0x824e, PDF_CMAP_SINGLE, 22292 },
	{ 0x824f, 0x824f, PDF_CMAP_SINGLE, 15042 },
	{ 0x8251, 0x8251, PDF_CMAP_SINGLE, 22293 },
	{ 0x8256, 0x8256, PDF_CMAP_SINGLE, 22294 },
	{ 0x8257, 0x8257, PDF_CMAP_SINGLE, 18462 },
	{ 0x8258, 0x8258, PDF_CMAP_SINGLE, 6310 },
	{ 0x8259, 0x8259, PDF_CMAP_SINGLE, 6309 },
	{ 0x825a, 0x825a, PDF_CMAP_SINGLE, 6312 },
	{ 0x825c, 0x825c, PDF_CMAP_SINGLE, 18464 },
	{ 0x825d, 0x825d, PDF_CMAP_SINGLE, 6311 },
	{ 0x825f, 0x825f, PDF_CMAP_SINGLE, 6313 },
	{ 0x8260, 0x8260, PDF_CMAP_SINGLE, 17061 },
	{ 0x8262, 0x8262, PDF_CMAP_SINGLE, 6315 },
	{ 0x8263, 0x8263, PDF_CMAP_SINGLE, 18465 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 6314 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 1547 },
	{ 0x8267, 0x8267, PDF_CMAP_SINGLE, 22295 },
	{ 0x8268, 0x8268, PDF_CMAP_SINGLE, 6316 },
	{ 0x826a, 0x826b, PDF_CMAP_RANGE, 6317 },
	{ 0x826d, 0x826d, PDF_CMAP_SINGLE, 19729 },
	{ 0x826e, 0x826e, PDF_CMAP_SINGLE, 2081 },
	{ 0x826f, 0x826f, PDF_CMAP_SINGLE, 3985 },
	{ 0x8271, 0x8271, PDF_CMAP_SINGLE, 6320 },
	{ 0x8272, 0x8272, PDF_CMAP_SINGLE, 2541 },
	{ 0x8274, 0x8274, PDF_CMAP_SINGLE, 17062 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 1298 },
	{ 0x8277, 0x8278, PDF_CMAP_RANGE, 6321 },
	{ 0x8279, 0x8279, PDF_CMAP_SINGLE, 14197 },
	{ 0x827b, 0x827b, PDF_CMAP_SINGLE, 22296 },
	{ 0x827d, 0x827d, PDF_CMAP_SINGLE, 18468 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 6323 },
	{ 0x827f, 0x827f, PDF_CMAP_SINGLE, 18469 },
	{ 0x8280, 0x8281, PDF_CMAP_RANGE, 22297 },
	{ 0x8283, 0x8283, PDF_CMAP_SINGLE, 18470 },
	{ 0x8284, 0x8284, PDF_CMAP_SINGLE, 19730 },
	{ 0x8287, 0x8287, PDF_CMAP_SINGLE, 22299 },
	{ 0x8289, 0x8289, PDF_CMAP_SINGLE, 19731 },
	{ 0x828a, 0x828a, PDF_CMAP_SINGLE, 18471 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 1206 },
	{ 0x828d, 0x828d, PDF_CMAP_SINGLE, 6324 },
	{ 0x828e, 0x828e, PDF_CMAP_SINGLE, 15043 },
	{ 0x8291, 0x8291, PDF_CMAP_SINGLE, 19732 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 6325 },
	{ 0x8293, 0x8293, PDF_CMAP_SINGLE, 18472 },
	{ 0x8294, 0x8294, PDF_CMAP_SINGLE, 22300 },
	{ 0x8296, 0x8296, PDF_CMAP_SINGLE, 22301 },
	{ 0x8298, 0x8298, PDF_CMAP_SINGLE, 22302 },
	{ 0x8299, 0x8299, PDF_CMAP_SINGLE, 3545 },
	{ 0x829a, 0x829b, PDF_CMAP_RANGE, 22303 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 2291 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 6327 },
	{ 0x82a0, 0x82a0, PDF_CMAP_SINGLE, 22305 },
	{ 0x82a1, 0x82a1, PDF_CMAP_SINGLE, 17064 },
	{ 0x82a3, 0x82a4, PDF_CMAP_RANGE, 17065 },
	{ 0x82a5, 0x82a5, PDF_CMAP_SINGLE, 1415 },
	{ 0x82a6, 0x82a6, PDF_CMAP_SINGLE, 1142 },
	{ 0x82a7, 0x82a8, PDF_CMAP_RANGE, 18473 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 17067 },
	{ 0x82aa, 0x82aa, PDF_CMAP_SINGLE, 19733 },
	{ 0x82ab, 0x82ab, PDF_CMAP_SINGLE, 6326 },
	{ 0x82ac, 0x82ac, PDF_CMAP_SINGLE, 6329 },
	{ 0x82ad, 0x82ad, PDF_CMAP_SINGLE, 3332 },
	{ 0x82ae, 0x82ae, PDF_CMAP_SINGLE, 15044 },
	{ 0x82af, 0x82af, PDF_CMAP_SINGLE, 2570 },
	{ 0x82b0, 0x82b0, PDF_CMAP_SINGLE, 19734 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 1366 },
	{ 0x82b2, 0x82b2, PDF_CMAP_SINGLE, 18475 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 3669 },
	{ 0x82b4, 0x82b4, PDF_CMAP_SINGLE, 18476 },
	{ 0x82b7, 0x82b7, PDF_CMAP_SINGLE, 15045 },
	{ 0x82b8, 0x82b8, PDF_CMAP_SINGLE, 1843 },
	{ 0x82b9, 0x82b9, PDF_CMAP_SINGLE, 1748 },
	{ 0x82ba, 0x82ba, PDF_CMAP_SINGLE, 18477 },
	{ 0x82bb, 0x82bb, PDF_CMAP_SINGLE, 6328 },
	{ 0x82bc, 0x82bc, PDF_CMAP_SINGLE, 18478 },
	{ 0x82bd, 0x82bd, PDF_CMAP_SINGLE, 1386 },
	{ 0x82be, 0x82be, PDF_CMAP_SINGLE, 15046 },
	{ 0x82bf, 0x82bf, PDF_CMAP_SINGLE, 17068 },
	{ 0x82c5, 0x82c5, PDF_CMAP_SINGLE, 1503 },
	{ 0x82c6, 0x82c6, PDF_CMAP_SINGLE, 15047 },
	{ 0x82d0, 0x82d0, PDF_CMAP_SINGLE, 19735 },
	{ 0x82d1, 0x82d1, PDF_CMAP_SINGLE, 1299 },
	{ 0x82d2, 0x82d2, PDF_CMAP_SINGLE, 6333 },
	{ 0x82d3, 0x82d3, PDF_CMAP_SINGLE, 4018 },
	{ 0x82d4, 0x82d4, PDF_CMAP_SINGLE, 2877 },
	{ 0x82d5, 0x82d5, PDF_CMAP_SINGLE, 17069 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 3510 },
	{ 0x82d9, 0x82d9, PDF_CMAP_SINGLE, 6345 },
	{ 0x82da, 0x82da, PDF_CMAP_SINGLE, 22306 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 1367 },
	{ 0x82dc, 0x82dc, PDF_CMAP_SINGLE, 6343 },
	{ 0x82de, 0x82de, PDF_CMAP_SINGLE, 6341 },
	{ 0x82df, 0x82df, PDF_CMAP_SINGLE, 6332 },
	{ 0x82e0, 0x82e0, PDF_CMAP_SINGLE, 22307 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 6330 },
	{ 0x82e2, 0x82e2, PDF_CMAP_SINGLE, 18479 },
	{ 0x82e3, 0x82e3, PDF_CMAP_SINGLE, 6331 },
	{ 0x82e4, 0x82e4, PDF_CMAP_SINGLE, 22308 },
	{ 0x82e5, 0x82e5, PDF_CMAP_SINGLE, 2319 },
	{ 0x82e6, 0x82e6, PDF_CMAP_SINGLE, 1764 },
	{ 0x82e7, 0x82e7, PDF_CMAP_SINGLE, 2997 },
	{ 0x82e8, 0x82e8, PDF_CMAP_SINGLE, 18480 },
	{ 0x82ea, 0x82ea, PDF_CMAP_SINGLE, 19736 },
	{ 0x82eb, 0x82eb, PDF_CMAP_SINGLE, 3241 },
	{ 0x82ed, 0x82ed, PDF_CMAP_SINGLE, 22309 },
	{ 0x82ef, 0x82ef, PDF_CMAP_SINGLE, 19737 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 1267 },
	{ 0x82f3, 0x82f3, PDF_CMAP_SINGLE, 6335 },
	{ 0x82f4, 0x82f4, PDF_CMAP_SINGLE, 6334 },
	{ 0x82f6, 0x82f6, PDF_CMAP_SINGLE, 19738 },
	{ 0x82f7, 0x82f7, PDF_CMAP_SINGLE, 18481 },
	{ 0x82f9, 0x82f9, PDF_CMAP_SINGLE, 6340 },
	{ 0x82fa, 0x82fa, PDF_CMAP_SINGLE, 6336 },
	{ 0x82fb, 0x82fb, PDF_CMAP_SINGLE, 6339 },
	{ 0x82fd, 0x82fd, PDF_CMAP_SINGLE, 17070 },
	{ 0x82fe, 0x82fe, PDF_CMAP_SINGLE, 15048 },
	{ 0x8300, 0x8300, PDF_CMAP_SINGLE, 17071 },
	{ 0x8301, 0x8301, PDF_CMAP_SINGLE, 8600 },
	{ 0x8302, 0x8302, PDF_CMAP_SINGLE, 3804 },
	{ 0x8303, 0x8303, PDF_CMAP_SINGLE, 6338 },
	{ 0x8304, 0x8304, PDF_CMAP_SINGLE, 1368 },
	{ 0x8305, 0x8305, PDF_CMAP_SINGLE, 1499 },
	{ 0x8306, 0x8306, PDF_CMAP_SINGLE, 6342 },
	{ 0x8307, 0x8308, PDF_CMAP_RANGE, 18482 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 6344 },
	{ 0x830a, 0x830b, PDF_CMAP_RANGE, 22310 },
	{ 0x830c, 0x830c, PDF_CMAP_SINGLE, 18484 },
	{ 0x830e, 0x830e, PDF_CMAP_SINGLE, 1834 },
	{ 0x8316, 0x8316, PDF_CMAP_SINGLE, 6348 },
	{ 0x8317, 0x8318, PDF_CMAP_RANGE, 6357 },
	{ 0x831b, 0x831b, PDF_CMAP_SINGLE, 18486 },
	{ 0x831c, 0x831c, PDF_CMAP_SINGLE, 1135 },
	{ 0x831d, 0x831d, PDF_CMAP_SINGLE, 18487 },
	{ 0x831e, 0x831f, PDF_CMAP_RANGE, 22312 },
	{ 0x8321, 0x8321, PDF_CMAP_SINGLE, 22314 },
	{ 0x8322, 0x8322, PDF_CMAP_SINGLE, 17072 },
	{ 0x8323, 0x8323, PDF_CMAP_SINGLE, 6365 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 1205 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 6356 },
	{ 0x832c, 0x832c, PDF_CMAP_SINGLE, 22315 },
	{ 0x832d, 0x832d, PDF_CMAP_SINGLE, 17073 },
	{ 0x832e, 0x832e, PDF_CMAP_SINGLE, 22316 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 6355 },
	{ 0x8330, 0x8330, PDF_CMAP_SINGLE, 18488 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 6350 },
	{ 0x8332, 0x8332, PDF_CMAP_SINGLE, 6349 },
	{ 0x8333, 0x8333, PDF_CMAP_SINGLE, 22317 },
	{ 0x8334, 0x8334, PDF_CMAP_SINGLE, 6347 },
	{ 0x8335, 0x8335, PDF_CMAP_SINGLE, 6346 },
	{ 0x8336, 0x8336, PDF_CMAP_SINGLE, 2977 },
	{ 0x8337, 0x8337, PDF_CMAP_SINGLE, 22318 },
	{ 0x8338, 0x8338, PDF_CMAP_SINGLE, 2907 },
	{ 0x8339, 0x8339, PDF_CMAP_SINGLE, 6352 },
	{ 0x833a, 0x833a, PDF_CMAP_SINGLE, 17074 },
	{ 0x833c, 0x833c, PDF_CMAP_SINGLE, 18489 },
	{ 0x833d, 0x833d, PDF_CMAP_SINGLE, 22319 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 6351 },
	{ 0x8342, 0x8342, PDF_CMAP_SINGLE, 22320 },
	{ 0x8343, 0x8343, PDF_CMAP_SINGLE, 15049 },
	{ 0x8344, 0x8344, PDF_CMAP_SINGLE, 18490 },
	{ 0x8345, 0x8345, PDF_CMAP_SINGLE, 6354 },
	{ 0x8346, 0x8346, PDF_CMAP_SINGLE, 7672 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 17075 },
	{ 0x8349, 0x8349, PDF_CMAP_SINGLE, 2802 },
	{ 0x834a, 0x834a, PDF_CMAP_SINGLE, 1835 },
	{ 0x834d, 0x834e, PDF_CMAP_RANGE, 22321 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 1251 },
	{ 0x8350, 0x8350, PDF_CMAP_SINGLE, 6353 },
	{ 0x8351, 0x8351, PDF_CMAP_SINGLE, 15050 },
	{ 0x8352, 0x8352, PDF_CMAP_SINGLE, 2021 },
	{ 0x8353, 0x8353, PDF_CMAP_SINGLE, 22332 },
	{ 0x8354, 0x8354, PDF_CMAP_SINGLE, 18485 },
	{ 0x8355, 0x8355, PDF_CMAP_SINGLE, 15051 },
	{ 0x8356, 0x8356, PDF_CMAP_SINGLE, 19739 },
	{ 0x8357, 0x8357, PDF_CMAP_SINGLE, 18491 },
	{ 0x8358, 0x8358, PDF_CMAP_SINGLE, 2803 },
	{ 0x835a, 0x835a, PDF_CMAP_SINGLE, 14202 },
	{ 0x8362, 0x8362, PDF_CMAP_SINGLE, 8601 },
	{ 0x8363, 0x8363, PDF_CMAP_SINGLE, 17775 },
	{ 0x8370, 0x8370, PDF_CMAP_SINGLE, 22323 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 6371 },
	{ 0x8375, 0x8375, PDF_CMAP_SINGLE, 6372 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 1369 },
	{ 0x8378, 0x8378, PDF_CMAP_SINGLE, 19740 },
	{ 0x837b, 0x837b, PDF_CMAP_SINGLE, 1326 },
	{ 0x837c, 0x837c, PDF_CMAP_SINGLE, 6369 },
	{ 0x837d, 0x837d, PDF_CMAP_SINGLE, 17076 },
	{ 0x837f, 0x837f, PDF_CMAP_SINGLE, 8602 },
	{ 0x8380, 0x8380, PDF_CMAP_SINGLE, 22324 },
	{ 0x8382, 0x8382, PDF_CMAP_SINGLE, 22325 },
	{ 0x8384, 0x8384, PDF_CMAP_SINGLE, 22326 },
	{ 0x8385, 0x8385, PDF_CMAP_SINGLE, 6359 },
	{ 0x8386, 0x8386, PDF_CMAP_SINGLE, 15052 },
	{ 0x8387, 0x8387, PDF_CMAP_SINGLE, 6367 },
	{ 0x8389, 0x8389, PDF_CMAP_SINGLE, 6374 },
	{ 0x838a, 0x838a, PDF_CMAP_SINGLE, 6368 },
	{ 0x838d, 0x838d, PDF_CMAP_SINGLE, 15053 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 6366 },
	{ 0x8392, 0x8392, PDF_CMAP_SINGLE, 15054 },
	{ 0x8393, 0x8393, PDF_CMAP_SINGLE, 6337 },
	{ 0x8394, 0x8395, PDF_CMAP_RANGE, 18495 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 6364 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 15055 },
	{ 0x8399, 0x8399, PDF_CMAP_SINGLE, 22327 },
	{ 0x839a, 0x839a, PDF_CMAP_SINGLE, 6360 },
	{ 0x839b, 0x839b, PDF_CMAP_SINGLE, 18497 },
	{ 0x839c, 0x839c, PDF_CMAP_SINGLE, 22328 },
	{ 0x839d, 0x839d, PDF_CMAP_SINGLE, 18498 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 1548 },
	{ 0x839f, 0x839f, PDF_CMAP_SINGLE, 6362 },
	{ 0x83a0, 0x83a0, PDF_CMAP_SINGLE, 6373 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 6363 },
	{ 0x83a6, 0x83a6, PDF_CMAP_SINGLE, 22329 },
	{ 0x83a7, 0x83a7, PDF_CMAP_SINGLE, 17077 },
	{ 0x83a8, 0x83a8, PDF_CMAP_SINGLE, 6375 },
	{ 0x83a9, 0x83a9, PDF_CMAP_SINGLE, 15056 },
	{ 0x83aa, 0x83aa, PDF_CMAP_SINGLE, 6361 },
	{ 0x83ab, 0x83ab, PDF_CMAP_SINGLE, 3378 },
	{ 0x83ac, 0x83ac, PDF_CMAP_SINGLE, 22330 },
	{ 0x83ad, 0x83ad, PDF_CMAP_SINGLE, 22335 },
	{ 0x83b1, 0x83b1, PDF_CMAP_SINGLE, 3923 },
	{ 0x83b5, 0x83b5, PDF_CMAP_SINGLE, 6370 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 6392 },
	{ 0x83be, 0x83be, PDF_CMAP_SINGLE, 22331 },
	{ 0x83bf, 0x83c0, PDF_CMAP_RANGE, 15057 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 6384 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 2625 },
	{ 0x83c7, 0x83c7, PDF_CMAP_SINGLE, 8603 },
	{ 0x83c9, 0x83c9, PDF_CMAP_SINGLE, 18499 },
	{ 0x83ca, 0x83ca, PDF_CMAP_SINGLE, 1632 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 1749 },
	{ 0x83ce, 0x83ce, PDF_CMAP_SINGLE, 6379 },
	{ 0x83cf, 0x83cf, PDF_CMAP_SINGLE, 17078 },
	{ 0x83d0, 0x83d0, PDF_CMAP_SINGLE, 18500 },
	{ 0x83d1, 0x83d1, PDF_CMAP_SINGLE, 17079 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 1371 },
	{ 0x83d4, 0x83d4, PDF_CMAP_SINGLE, 18501 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 2492 },
	{ 0x83d8, 0x83d8, PDF_CMAP_SINGLE, 6382 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 2122 },
	{ 0x83dd, 0x83dd, PDF_CMAP_SINGLE, 18502 },
	{ 0x83df, 0x83df, PDF_CMAP_SINGLE, 3147 },
	{ 0x83e0, 0x83e0, PDF_CMAP_SINGLE, 6387 },
	{ 0x83e1, 0x83e1, PDF_CMAP_SINGLE, 17080 },
	{ 0x83e5, 0x83e5, PDF_CMAP_SINGLE, 18503 },
	{ 0x83e8, 0x83e8, PDF_CMAP_SINGLE, 22333 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 3646 },
	{ 0x83ea, 0x83ea, PDF_CMAP_SINGLE, 15059 },
	{ 0x83eb, 0x83eb, PDF_CMAP_SINGLE, 6378 },
	{ 0x83ef, 0x83ef, PDF_CMAP_SINGLE, 1370 },
	{ 0x83f0, 0x83f0, PDF_CMAP_SINGLE, 1930 },
	{ 0x83f1, 0x83f1, PDF_CMAP_SINGLE, 3483 },
	{ 0x83f2, 0x83f2, PDF_CMAP_SINGLE, 6388 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 6376 },
	{ 0x83f6, 0x83f6, PDF_CMAP_SINGLE, 8604 },
	{ 0x83f7, 0x83f7, PDF_CMAP_SINGLE, 6385 },
	{ 0x83f8, 0x83f8, PDF_CMAP_SINGLE, 19741 },
	{ 0x83f9, 0x83f9, PDF_CMAP_SINGLE, 18504 },
	{ 0x83fb, 0x83fb, PDF_CMAP_SINGLE, 6395 },
	{ 0x83fc, 0x83fc, PDF_CMAP_SINGLE, 19742 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 6380 },
	{ 0x8401, 0x8401, PDF_CMAP_SINGLE, 17081 },
	{ 0x8403, 0x8403, PDF_CMAP_SINGLE, 6381 },
	{ 0x8404, 0x8404, PDF_CMAP_SINGLE, 3218 },
	{ 0x8406, 0x8406, PDF_CMAP_SINGLE, 17082 },
	{ 0x8407, 0x8407, PDF_CMAP_SINGLE, 6386 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 7807 },
	{ 0x840b, 0x840b, PDF_CMAP_SINGLE, 6383 },
	{ 0x840c, 0x840c, PDF_CMAP_SINGLE, 3670 },
	{ 0x840d, 0x840d, PDF_CMAP_SINGLE, 6389 },
	{ 0x840e, 0x840e, PDF_CMAP_SINGLE, 1188 },
	{ 0x840f, 0x840f, PDF_CMAP_SINGLE, 15060 },
	{ 0x8411, 0x8411, PDF_CMAP_SINGLE, 15061 },
	{ 0x8413, 0x8413, PDF_CMAP_SINGLE, 6377 },
	{ 0x8415, 0x8415, PDF_CMAP_SINGLE, 18505 },
	{ 0x8417, 0x8417, PDF_CMAP_SINGLE, 18507 },
	{ 0x8419, 0x8419, PDF_CMAP_SINGLE, 22334 },
	{ 0x8420, 0x8420, PDF_CMAP_SINGLE, 6391 },
	{ 0x8422, 0x8422, PDF_CMAP_SINGLE, 6390 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 3361 },
	{ 0x842a, 0x842a, PDF_CMAP_SINGLE, 6397 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 6408 },
	{ 0x842f, 0x842f, PDF_CMAP_SINGLE, 22336 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 1500 },
	{ 0x8435, 0x8435, PDF_CMAP_SINGLE, 6411 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 6393 },
	{ 0x8439, 0x8439, PDF_CMAP_SINGLE, 18508 },
	{ 0x843c, 0x843c, PDF_CMAP_SINGLE, 6398 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 3928 },
	{ 0x8445, 0x8445, PDF_CMAP_SINGLE, 22337 },
	{ 0x8446, 0x8446, PDF_CMAP_SINGLE, 6407 },
	{ 0x8447, 0x8447, PDF_CMAP_SINGLE, 22338 },
	{ 0x8448, 0x8448, PDF_CMAP_SINGLE, 8605 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 3903 },
	{ 0x844a, 0x844a, PDF_CMAP_SINGLE, 15062 },
	{ 0x844d, 0x844d, PDF_CMAP_SINGLE, 22339 },
	{ 0x844e, 0x844e, PDF_CMAP_SINGLE, 3954 },
	{ 0x844f, 0x844f, PDF_CMAP_SINGLE, 18509 },
	{ 0x8451, 0x8452, PDF_CMAP_RANGE, 18510 },
	{ 0x8456, 0x8456, PDF_CMAP_SINGLE, 22340 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 2998 },
	{ 0x8458, 0x8458, PDF_CMAP_SINGLE, 19743 },
	{ 0x8459, 0x845a, PDF_CMAP_RANGE, 18512 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 1481 },
	{ 0x845c, 0x845c, PDF_CMAP_SINGLE, 18514 },
	{ 0x845f, 0x845f, PDF_CMAP_SINGLE, 17083 },
	{ 0x8460, 0x8460, PDF_CMAP_SINGLE, 22342 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 3556 },
	{ 0x8462, 0x8462, PDF_CMAP_SINGLE, 6413 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 3193 },
	{ 0x8464, 0x8464, PDF_CMAP_SINGLE, 22343 },
	{ 0x8465, 0x8465, PDF_CMAP_SINGLE, 18516 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 1141 },
	{ 0x8467, 0x8467, PDF_CMAP_SINGLE, 22344 },
	{ 0x8469, 0x8469, PDF_CMAP_SINGLE, 6406 },
	{ 0x846a, 0x846a, PDF_CMAP_SINGLE, 22345 },
	{ 0x846b, 0x846b, PDF_CMAP_SINGLE, 6402 },
	{ 0x846c, 0x846c, PDF_CMAP_SINGLE, 2804 },
	{ 0x846d, 0x846d, PDF_CMAP_SINGLE, 6396 },
	{ 0x846e, 0x846e, PDF_CMAP_SINGLE, 6404 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 6409 },
	{ 0x8470, 0x8470, PDF_CMAP_SINGLE, 17084 },
	{ 0x8471, 0x8471, PDF_CMAP_SINGLE, 3298 },
	{ 0x8473, 0x8473, PDF_CMAP_SINGLE, 17085 },
	{ 0x8474, 0x8474, PDF_CMAP_SINGLE, 22346 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 1134 },
	{ 0x8476, 0x8476, PDF_CMAP_SINGLE, 15063 },
	{ 0x8477, 0x8477, PDF_CMAP_SINGLE, 6401 },
	{ 0x8478, 0x8478, PDF_CMAP_SINGLE, 18517 },
	{ 0x8479, 0x8479, PDF_CMAP_SINGLE, 6410 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 3562 },
	{ 0x847c, 0x847c, PDF_CMAP_SINGLE, 18518 },
	{ 0x847d, 0x847d, PDF_CMAP_SINGLE, 22347 },
	{ 0x8481, 0x8481, PDF_CMAP_SINGLE, 18519 },
	{ 0x8482, 0x8482, PDF_CMAP_SINGLE, 6405 },
	{ 0x8484, 0x8484, PDF_CMAP_SINGLE, 6400 },
	{ 0x8485, 0x8485, PDF_CMAP_SINGLE, 17086 },
	{ 0x848b, 0x848b, PDF_CMAP_SINGLE, 2493 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 2361 },
	{ 0x8492, 0x8492, PDF_CMAP_SINGLE, 22348 },
	{ 0x8493, 0x8493, PDF_CMAP_SINGLE, 19744 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 2264 },
	{ 0x8495, 0x8495, PDF_CMAP_SINGLE, 22349 },
	{ 0x8497, 0x8497, PDF_CMAP_SINGLE, 18521 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 3812 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 3513 },
	{ 0x849e, 0x849e, PDF_CMAP_SINGLE, 17087 },
	{ 0x849f, 0x849f, PDF_CMAP_SINGLE, 6416 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 6425 },
	{ 0x84a6, 0x84a6, PDF_CMAP_SINGLE, 18522 },
	{ 0x84a8, 0x84a8, PDF_CMAP_SINGLE, 15064 },
	{ 0x84a9, 0x84aa, PDF_CMAP_RANGE, 22350 },
	{ 0x84ad, 0x84ad, PDF_CMAP_SINGLE, 6403 },
	{ 0x84af, 0x84af, PDF_CMAP_SINGLE, 15065 },
	{ 0x84b1, 0x84b1, PDF_CMAP_SINGLE, 19745 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 1493 },
	{ 0x84b4, 0x84b4, PDF_CMAP_SINGLE, 8606 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 2528 },
	{ 0x84b9, 0x84b9, PDF_CMAP_SINGLE, 6414 },
	{ 0x84ba, 0x84ba, PDF_CMAP_SINGLE, 17088 },
	{ 0x84bb, 0x84bb, PDF_CMAP_SINGLE, 6419 },
	{ 0x84bc, 0x84bc, PDF_CMAP_SINGLE, 2805 },
	{ 0x84bd, 0x84bd, PDF_CMAP_SINGLE, 19746 },
	{ 0x84be, 0x84be, PDF_CMAP_SINGLE, 18523 },
	{ 0x84bf, 0x84bf, PDF_CMAP_SINGLE, 6415 },
	{ 0x84c0, 0x84c0, PDF_CMAP_SINGLE, 15066 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 6422 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 15067 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 2973 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 6423 },
	{ 0x84c7, 0x84c8, PDF_CMAP_RANGE, 22352 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 3904 },
	{ 0x84ca, 0x84ca, PDF_CMAP_SINGLE, 6412 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 1430 },
	{ 0x84cc, 0x84cc, PDF_CMAP_SINGLE, 22354 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 6418 },
	{ 0x84ce, 0x84cf, PDF_CMAP_RANGE, 18525 },
	{ 0x84d0, 0x84d0, PDF_CMAP_SINGLE, 6421 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 3768 },
	{ 0x84d3, 0x84d3, PDF_CMAP_SINGLE, 18527 },
	{ 0x84d6, 0x84d6, PDF_CMAP_SINGLE, 6424 },
	{ 0x84d9, 0x84d9, PDF_CMAP_SINGLE, 6417 },
	{ 0x84da, 0x84da, PDF_CMAP_SINGLE, 6420 },
	{ 0x84dc, 0x84dc, PDF_CMAP_SINGLE, 8363 },
	{ 0x84e7, 0x84e7, PDF_CMAP_SINGLE, 18529 },
	{ 0x84ea, 0x84ea, PDF_CMAP_SINGLE, 18530 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 3671 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4039 },
	{ 0x84ef, 0x84ef, PDF_CMAP_SINGLE, 18531 },
	{ 0x84f0, 0x84f0, PDF_CMAP_SINGLE, 15068 },
	{ 0x84f1, 0x84f1, PDF_CMAP_SINGLE, 18532 },
	{ 0x84f2, 0x84f2, PDF_CMAP_SINGLE, 22355 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 6428 },
	{ 0x84f7, 0x84f7, PDF_CMAP_SINGLE, 22356 },
	{ 0x84fa, 0x84fa, PDF_CMAP_SINGLE, 18533 },
	{ 0x84fb, 0x84fb, PDF_CMAP_SINGLE, 19747 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 6435 },
	{ 0x84fd, 0x84fd, PDF_CMAP_SINGLE, 15069 },
	{ 0x84ff, 0x84ff, PDF_CMAP_SINGLE, 6427 },
	{ 0x8500, 0x8500, PDF_CMAP_SINGLE, 2287 },
	{ 0x8502, 0x8503, PDF_CMAP_RANGE, 22357 },
	{ 0x8506, 0x8506, PDF_CMAP_SINGLE, 6394 },
	{ 0x8507, 0x8507, PDF_CMAP_SINGLE, 22359 },
	{ 0x850c, 0x850c, PDF_CMAP_SINGLE, 15070 },
	{ 0x850e, 0x850e, PDF_CMAP_SINGLE, 22360 },
	{ 0x8510, 0x8510, PDF_CMAP_SINGLE, 22361 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 3614 },
	{ 0x8513, 0x8513, PDF_CMAP_SINGLE, 3758 },
	{ 0x8514, 0x8514, PDF_CMAP_SINGLE, 6434 },
	{ 0x8515, 0x8515, PDF_CMAP_SINGLE, 6433 },
	{ 0x8517, 0x8518, PDF_CMAP_RANGE, 6429 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 1240 },
	{ 0x851b, 0x851b, PDF_CMAP_SINGLE, 18534 },
	{ 0x851c, 0x851c, PDF_CMAP_SINGLE, 22362 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 17091 },
	{ 0x851f, 0x851f, PDF_CMAP_SINGLE, 6432 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 6426 },
	{ 0x8522, 0x8522, PDF_CMAP_SINGLE, 22363 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 7706 },
	{ 0x8524, 0x8525, PDF_CMAP_RANGE, 18535 },
	{ 0x8526, 0x8526, PDF_CMAP_SINGLE, 3057 },
	{ 0x8527, 0x8527, PDF_CMAP_SINGLE, 22364 },
	{ 0x852a, 0x852a, PDF_CMAP_SINGLE, 22365 },
	{ 0x852b, 0x852b, PDF_CMAP_SINGLE, 18537 },
	{ 0x852c, 0x852c, PDF_CMAP_SINGLE, 6431 },
	{ 0x852d, 0x852d, PDF_CMAP_SINGLE, 1218 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 17092 },
	{ 0x8532, 0x8532, PDF_CMAP_SINGLE, 17090 },
	{ 0x8533, 0x8533, PDF_CMAP_SINGLE, 22366 },
	{ 0x8534, 0x8534, PDF_CMAP_SINGLE, 15071 },
	{ 0x8535, 0x8535, PDF_CMAP_SINGLE, 2818 },
	{ 0x8536, 0x8536, PDF_CMAP_SINGLE, 22367 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 3603 },
	{ 0x853e, 0x853e, PDF_CMAP_SINGLE, 7861 },
	{ 0x853f, 0x853f, PDF_CMAP_SINGLE, 22368 },
	{ 0x8540, 0x8540, PDF_CMAP_SINGLE, 6436 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 6440 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 3437 },
	{ 0x8546, 0x8546, PDF_CMAP_SINGLE, 19748 },
	{ 0x8548, 0x8548, PDF_CMAP_SINGLE, 6439 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 2494 },
	{ 0x854a, 0x854a, PDF_CMAP_SINGLE, 2293 },
	{ 0x854b, 0x854b, PDF_CMAP_SINGLE, 6442 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 1718 },
	{ 0x854f, 0x854f, PDF_CMAP_SINGLE, 18538 },
	{ 0x8550, 0x8550, PDF_CMAP_SINGLE, 22369 },
	{ 0x8551, 0x8551, PDF_CMAP_SINGLE, 18541 },
	{ 0x8552, 0x8552, PDF_CMAP_SINGLE, 22370 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 8607 },
	{ 0x8555, 0x8555, PDF_CMAP_SINGLE, 6443 },
	{ 0x8556, 0x8556, PDF_CMAP_SINGLE, 19749 },
	{ 0x8557, 0x8557, PDF_CMAP_SINGLE, 3563 },
	{ 0x8558, 0x8558, PDF_CMAP_SINGLE, 6438 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 8608 },
	{ 0x855a, 0x855a, PDF_CMAP_SINGLE, 6399 },
	{ 0x855c, 0x855c, PDF_CMAP_SINGLE, 22371 },
	{ 0x855d, 0x855d, PDF_CMAP_SINGLE, 19750 },
	{ 0x855e, 0x855e, PDF_CMAP_SINGLE, 15072 },
	{ 0x855f, 0x8560, PDF_CMAP_RANGE, 22373 },
	{ 0x8561, 0x8562, PDF_CMAP_RANGE, 18542 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 6437 },
	{ 0x8564, 0x8564, PDF_CMAP_SINGLE, 17093 },
	{ 0x8568, 0x8568, PDF_CMAP_SINGLE, 4085 },
	{ 0x8569, 0x8569, PDF_CMAP_SINGLE, 3194 },
	{ 0x856a, 0x856a, PDF_CMAP_SINGLE, 3557 },
	{ 0x856b, 0x856b, PDF_CMAP_SINGLE, 8609 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 6450 },
	{ 0x856f, 0x856f, PDF_CMAP_SINGLE, 18539 },
	{ 0x8577, 0x8577, PDF_CMAP_SINGLE, 6456 },
	{ 0x8579, 0x8579, PDF_CMAP_SINGLE, 22375 },
	{ 0x857a, 0x857a, PDF_CMAP_SINGLE, 17094 },
	{ 0x857b, 0x857b, PDF_CMAP_SINGLE, 18545 },
	{ 0x857d, 0x857d, PDF_CMAP_SINGLE, 18546 },
	{ 0x857e, 0x857e, PDF_CMAP_SINGLE, 6457 },
	{ 0x857f, 0x857f, PDF_CMAP_SINGLE, 18547 },
	{ 0x8580, 0x8580, PDF_CMAP_SINGLE, 6444 },
	{ 0x8581, 0x8581, PDF_CMAP_SINGLE, 18548 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 3372 },
	{ 0x8585, 0x8585, PDF_CMAP_SINGLE, 19751 },
	{ 0x8586, 0x8586, PDF_CMAP_SINGLE, 18549 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 6454 },
	{ 0x8588, 0x8588, PDF_CMAP_SINGLE, 6446 },
	{ 0x8589, 0x8589, PDF_CMAP_SINGLE, 22376 },
	{ 0x858a, 0x858a, PDF_CMAP_SINGLE, 6448 },
	{ 0x858b, 0x858b, PDF_CMAP_SINGLE, 22377 },
	{ 0x858c, 0x858c, PDF_CMAP_SINGLE, 17095 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 15073 },
	{ 0x8590, 0x8590, PDF_CMAP_SINGLE, 6458 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 6447 },
	{ 0x8593, 0x8593, PDF_CMAP_SINGLE, 18550 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 6451 },
	{ 0x8597, 0x8597, PDF_CMAP_SINGLE, 1300 },
	{ 0x8598, 0x8598, PDF_CMAP_SINGLE, 19752 },
	{ 0x8599, 0x8599, PDF_CMAP_SINGLE, 3261 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 6452 },
	{ 0x859c, 0x859c, PDF_CMAP_SINGLE, 6455 },
	{ 0x859d, 0x859d, PDF_CMAP_SINGLE, 18551 },
	{ 0x859f, 0x859f, PDF_CMAP_SINGLE, 18552 },
	{ 0x85a0, 0x85a0, PDF_CMAP_SINGLE, 22378 },
	{ 0x85a2, 0x85a2, PDF_CMAP_SINGLE, 17096 },
	{ 0x85a4, 0x85a4, PDF_CMAP_SINGLE, 6445 },
	{ 0x85a5, 0x85a5, PDF_CMAP_SINGLE, 22379 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 2728 },
	{ 0x85a7, 0x85a7, PDF_CMAP_SINGLE, 22380 },
	{ 0x85a8, 0x85a8, PDF_CMAP_SINGLE, 6449 },
	{ 0x85a9, 0x85a9, PDF_CMAP_SINGLE, 2165 },
	{ 0x85aa, 0x85aa, PDF_CMAP_SINGLE, 2571 },
	{ 0x85ab, 0x85ab, PDF_CMAP_SINGLE, 1798 },
	{ 0x85ac, 0x85ac, PDF_CMAP_SINGLE, 3840 },
	{ 0x85ad, 0x85ad, PDF_CMAP_SINGLE, 15076 },
	{ 0x85ae, 0x85ae, PDF_CMAP_SINGLE, 3845 },
	{ 0x85af, 0x85af, PDF_CMAP_SINGLE, 2428 },
	{ 0x85b0, 0x85b0, PDF_CMAP_SINGLE, 8611 },
	{ 0x85b4, 0x85b4, PDF_CMAP_SINGLE, 22381 },
	{ 0x85b6, 0x85b6, PDF_CMAP_SINGLE, 22382 },
	{ 0x85b7, 0x85b7, PDF_CMAP_SINGLE, 15074 },
	{ 0x85b8, 0x85b8, PDF_CMAP_SINGLE, 22383 },
	{ 0x85b9, 0x85b9, PDF_CMAP_SINGLE, 6462 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 6460 },
	{ 0x85bc, 0x85bc, PDF_CMAP_SINGLE, 18556 },
	{ 0x85bd, 0x85bf, PDF_CMAP_RANGE, 22384 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 4084 },
	{ 0x85c2, 0x85c2, PDF_CMAP_SINGLE, 22387 },
	{ 0x85c7, 0x85c7, PDF_CMAP_SINGLE, 18557 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 6459 },
	{ 0x85ca, 0x85ca, PDF_CMAP_SINGLE, 18558 },
	{ 0x85cb, 0x85cb, PDF_CMAP_SINGLE, 17097 },
	{ 0x85cd, 0x85cd, PDF_CMAP_SINGLE, 3935 },
	{ 0x85ce, 0x85ce, PDF_CMAP_SINGLE, 15075 },
	{ 0x85cf, 0x85cf, PDF_CMAP_SINGLE, 6461 },
	{ 0x85d0, 0x85d0, PDF_CMAP_SINGLE, 6463 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 6464 },
	{ 0x85d8, 0x85d9, PDF_CMAP_RANGE, 18559 },
	{ 0x85da, 0x85da, PDF_CMAP_SINGLE, 22388 },
	{ 0x85dc, 0x85dc, PDF_CMAP_SINGLE, 6467 },
	{ 0x85dd, 0x85dd, PDF_CMAP_SINGLE, 6465 },
	{ 0x85df, 0x85df, PDF_CMAP_SINGLE, 18561 },
	{ 0x85e0, 0x85e0, PDF_CMAP_SINGLE, 22389 },
	{ 0x85e1, 0x85e1, PDF_CMAP_SINGLE, 18562 },
	{ 0x85e4, 0x85e4, PDF_CMAP_SINGLE, 3195 },
	{ 0x85e5, 0x85e5, PDF_CMAP_SINGLE, 6466 },
	{ 0x85e6, 0x85e6, PDF_CMAP_SINGLE, 18563 },
	{ 0x85e8, 0x85e8, PDF_CMAP_SINGLE, 22390 },
	{ 0x85e9, 0x85e9, PDF_CMAP_SINGLE, 3425 },
	{ 0x85ea, 0x85ea, PDF_CMAP_SINGLE, 6453 },
	{ 0x85ed, 0x85ed, PDF_CMAP_SINGLE, 17098 },
	{ 0x85f3, 0x85f3, PDF_CMAP_SINGLE, 22391 },
	{ 0x85f4, 0x85f4, PDF_CMAP_SINGLE, 14208 },
	{ 0x85f6, 0x85f6, PDF_CMAP_SINGLE, 18564 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 2429 },
	{ 0x85f9, 0x85f9, PDF_CMAP_SINGLE, 6468 },
	{ 0x85fa, 0x85fa, PDF_CMAP_SINGLE, 6473 },
	{ 0x85fb, 0x85fb, PDF_CMAP_SINGLE, 2806 },
	{ 0x85fc, 0x85fc, PDF_CMAP_SINGLE, 22392 },
	{ 0x85fe, 0x85fe, PDF_CMAP_SINGLE, 6472 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 17099 },
	{ 0x8600, 0x8600, PDF_CMAP_SINGLE, 18565 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 6441 },
	{ 0x8604, 0x8605, PDF_CMAP_RANGE, 17100 },
	{ 0x8606, 0x8606, PDF_CMAP_SINGLE, 6474 },
	{ 0x8607, 0x8607, PDF_CMAP_SINGLE, 2763 },
	{ 0x860a, 0x860a, PDF_CMAP_SINGLE, 6469 },
	{ 0x860b, 0x860b, PDF_CMAP_SINGLE, 6471 },
	{ 0x860d, 0x860e, PDF_CMAP_RANGE, 22393 },
	{ 0x8610, 0x8610, PDF_CMAP_SINGLE, 17102 },
	{ 0x8611, 0x8611, PDF_CMAP_SINGLE, 18566 },
	{ 0x8612, 0x8612, PDF_CMAP_SINGLE, 15077 },
	{ 0x8613, 0x8613, PDF_CMAP_SINGLE, 6470 },
	{ 0x8616, 0x8616, PDF_CMAP_SINGLE, 5328 },
	{ 0x8617, 0x8617, PDF_CMAP_SINGLE, 5313 },
	{ 0x8618, 0x8618, PDF_CMAP_SINGLE, 17104 },
	{ 0x8619, 0x8619, PDF_CMAP_SINGLE, 22395 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 6476 },
	{ 0x861b, 0x861b, PDF_CMAP_SINGLE, 22396 },
	{ 0x861e, 0x861e, PDF_CMAP_SINGLE, 18567 },
	{ 0x8621, 0x8621, PDF_CMAP_SINGLE, 18568 },
	{ 0x8622, 0x8622, PDF_CMAP_SINGLE, 6475 },
	{ 0x8624, 0x8624, PDF_CMAP_SINGLE, 18569 },
	{ 0x8627, 0x8627, PDF_CMAP_SINGLE, 18570 },
	{ 0x8629, 0x8629, PDF_CMAP_SINGLE, 15078 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 3936 },
	{ 0x862f, 0x862f, PDF_CMAP_SINGLE, 5809 },
	{ 0x8630, 0x8630, PDF_CMAP_SINGLE, 6477 },
	{ 0x8636, 0x8636, PDF_CMAP_SINGLE, 22397 },
	{ 0x8638, 0x8638, PDF_CMAP_SINGLE, 17105 },
	{ 0x8639, 0x8639, PDF_CMAP_SINGLE, 18572 },
	{ 0x863a, 0x863a, PDF_CMAP_SINGLE, 22398 },
	{ 0x863c, 0x863c, PDF_CMAP_SINGLE, 18573 },
	{ 0x863d, 0x863d, PDF_CMAP_SINGLE, 22399 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 6478 },
	{ 0x8640, 0x8640, PDF_CMAP_SINGLE, 18575 },
	{ 0x8641, 0x8641, PDF_CMAP_SINGLE, 16824 },
	{ 0x8642, 0x8642, PDF_CMAP_SINGLE, 19753 },
	{ 0x8646, 0x8646, PDF_CMAP_SINGLE, 19754 },
	{ 0x864d, 0x864d, PDF_CMAP_SINGLE, 6479 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 1931 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 1646 },
	{ 0x8652, 0x8652, PDF_CMAP_SINGLE, 15079 },
	{ 0x8653, 0x8653, PDF_CMAP_SINGLE, 18576 },
	{ 0x8654, 0x8654, PDF_CMAP_SINGLE, 6481 },
	{ 0x8655, 0x8655, PDF_CMAP_SINGLE, 4244 },
	{ 0x8656, 0x8656, PDF_CMAP_SINGLE, 18577 },
	{ 0x8657, 0x8657, PDF_CMAP_SINGLE, 17106 },
	{ 0x8658, 0x8659, PDF_CMAP_RANGE, 22400 },
	{ 0x865a, 0x865a, PDF_CMAP_SINGLE, 1679 },
	{ 0x865b, 0x865b, PDF_CMAP_SINGLE, 13336 },
	{ 0x865c, 0x865c, PDF_CMAP_SINGLE, 3970 },
	{ 0x865d, 0x865d, PDF_CMAP_SINGLE, 22402 },
	{ 0x865e, 0x865e, PDF_CMAP_SINGLE, 1771 },
	{ 0x865f, 0x865f, PDF_CMAP_SINGLE, 6482 },
	{ 0x8660, 0x8661, PDF_CMAP_RANGE, 22403 },
	{ 0x8662, 0x8662, PDF_CMAP_SINGLE, 17107 },
	{ 0x8663, 0x8663, PDF_CMAP_SINGLE, 15080 },
	{ 0x8664, 0x8664, PDF_CMAP_SINGLE, 22405 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 6483 },
	{ 0x8669, 0x8669, PDF_CMAP_SINGLE, 22406 },
	{ 0x866b, 0x866b, PDF_CMAP_SINGLE, 2988 },
	{ 0x866c, 0x866c, PDF_CMAP_SINGLE, 15081 },
	{ 0x866f, 0x866f, PDF_CMAP_SINGLE, 15082 },
	{ 0x8671, 0x8671, PDF_CMAP_SINGLE, 6484 },
	{ 0x8675, 0x8675, PDF_CMAP_SINGLE, 17109 },
	{ 0x8676, 0x8676, PDF_CMAP_SINGLE, 22407 },
	{ 0x8677, 0x8677, PDF_CMAP_SINGLE, 18578 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 3282 },
	{ 0x867a, 0x867a, PDF_CMAP_SINGLE, 15083 },
	{ 0x867b, 0x867b, PDF_CMAP_SINGLE, 1150 },
	{ 0x867d, 0x867d, PDF_CMAP_SINGLE, 14214 },
	{ 0x8687, 0x8687, PDF_CMAP_SINGLE, 18579 },
	{ 0x8688, 0x8688, PDF_CMAP_SINGLE, 22425 },
	{ 0x8689, 0x8689, PDF_CMAP_SINGLE, 18580 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 1379 },
	{ 0x868b, 0x868c, PDF_CMAP_RANGE, 6489 },
	{ 0x868d, 0x868d, PDF_CMAP_SINGLE, 15084 },
	{ 0x8691, 0x8691, PDF_CMAP_SINGLE, 15085 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 6485 },
	{ 0x8695, 0x8695, PDF_CMAP_SINGLE, 2187 },
	{ 0x8696, 0x8696, PDF_CMAP_SINGLE, 22408 },
	{ 0x8698, 0x8698, PDF_CMAP_SINGLE, 15086 },
	{ 0x869a, 0x869a, PDF_CMAP_SINGLE, 22409 },
	{ 0x869c, 0x869d, PDF_CMAP_RANGE, 18581 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 22410 },
	{ 0x86a3, 0x86a3, PDF_CMAP_SINGLE, 6486 },
	{ 0x86a4, 0x86a4, PDF_CMAP_SINGLE, 3320 },
	{ 0x86a6, 0x86a6, PDF_CMAP_SINGLE, 22411 },
	{ 0x86a7, 0x86a8, PDF_CMAP_RANGE, 15087 },
	{ 0x86a9, 0x86aa, PDF_CMAP_RANGE, 6487 },
	{ 0x86ab, 0x86ab, PDF_CMAP_SINGLE, 6498 },
	{ 0x86ad, 0x86ad, PDF_CMAP_SINGLE, 22412 },
	{ 0x86af, 0x86af, PDF_CMAP_SINGLE, 6492 },
	{ 0x86b0, 0x86b0, PDF_CMAP_SINGLE, 6495 },
	{ 0x86b1, 0x86b1, PDF_CMAP_SINGLE, 18583 },
	{ 0x86b3, 0x86b3, PDF_CMAP_SINGLE, 18584 },
	{ 0x86b4, 0x86b5, PDF_CMAP_RANGE, 22413 },
	{ 0x86b6, 0x86b6, PDF_CMAP_SINGLE, 6491 },
	{ 0x86b7, 0x86b7, PDF_CMAP_SINGLE, 22415 },
	{ 0x86b8, 0x86b8, PDF_CMAP_SINGLE, 17110 },
	{ 0x86b9, 0x86b9, PDF_CMAP_SINGLE, 22416 },
	{ 0x86bf, 0x86bf, PDF_CMAP_SINGLE, 22417 },
	{ 0x86c0, 0x86c0, PDF_CMAP_SINGLE, 19755 },
	{ 0x86c1, 0x86c1, PDF_CMAP_SINGLE, 18585 },
	{ 0x86c3, 0x86c3, PDF_CMAP_SINGLE, 18586 },
	{ 0x86c4, 0x86c4, PDF_CMAP_SINGLE, 6493 },
	{ 0x86c5, 0x86c5, PDF_CMAP_SINGLE, 22418 },
	{ 0x86c6, 0x86c6, PDF_CMAP_SINGLE, 6494 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 2308 },
	{ 0x86c9, 0x86c9, PDF_CMAP_SINGLE, 6496 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 2943 },
	{ 0x86cd, 0x86cd, PDF_CMAP_SINGLE, 1836 },
	{ 0x86ce, 0x86ce, PDF_CMAP_SINGLE, 1440 },
	{ 0x86d1, 0x86d1, PDF_CMAP_SINGLE, 18587 },
	{ 0x86d2, 0x86d2, PDF_CMAP_SINGLE, 22419 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 6499 },
	{ 0x86d5, 0x86d5, PDF_CMAP_SINGLE, 18588 },
	{ 0x86d7, 0x86d7, PDF_CMAP_SINGLE, 18589 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 1437 },
	{ 0x86da, 0x86da, PDF_CMAP_SINGLE, 22420 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 6504 },
	{ 0x86dc, 0x86dc, PDF_CMAP_SINGLE, 22421 },
	{ 0x86de, 0x86de, PDF_CMAP_SINGLE, 6500 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 6503 },
	{ 0x86e0, 0x86e0, PDF_CMAP_SINGLE, 22422 },
	{ 0x86e3, 0x86e3, PDF_CMAP_SINGLE, 18590 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 3406 },
	{ 0x86e5, 0x86e5, PDF_CMAP_SINGLE, 22423 },
	{ 0x86e6, 0x86e6, PDF_CMAP_SINGLE, 18591 },
	{ 0x86e7, 0x86e7, PDF_CMAP_SINGLE, 22424 },
	{ 0x86e9, 0x86e9, PDF_CMAP_SINGLE, 6501 },
	{ 0x86ec, 0x86ec, PDF_CMAP_SINGLE, 6502 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 3514 },
	{ 0x86ee, 0x86ee, PDF_CMAP_SINGLE, 3438 },
	{ 0x86ef, 0x86ef, PDF_CMAP_SINGLE, 6505 },
	{ 0x86f8, 0x86f8, PDF_CMAP_SINGLE, 2909 },
	{ 0x86f9, 0x86f9, PDF_CMAP_SINGLE, 6515 },
	{ 0x86fa, 0x86fa, PDF_CMAP_SINGLE, 15089 },
	{ 0x86fb, 0x86fb, PDF_CMAP_SINGLE, 6511 },
	{ 0x86fc, 0x86fc, PDF_CMAP_SINGLE, 17111 },
	{ 0x86fd, 0x86fd, PDF_CMAP_SINGLE, 15090 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 1387 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 6509 },
	{ 0x8702, 0x8702, PDF_CMAP_SINGLE, 3672 },
	{ 0x8703, 0x8703, PDF_CMAP_SINGLE, 6510 },
	{ 0x8704, 0x8704, PDF_CMAP_SINGLE, 22426 },
	{ 0x8705, 0x8705, PDF_CMAP_SINGLE, 18593 },
	{ 0x8706, 0x8706, PDF_CMAP_SINGLE, 6507 },
	{ 0x8707, 0x8707, PDF_CMAP_SINGLE, 18594 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 6508 },
	{ 0x8709, 0x8709, PDF_CMAP_SINGLE, 6513 },
	{ 0x870a, 0x870a, PDF_CMAP_SINGLE, 6516 },
	{ 0x870b, 0x870b, PDF_CMAP_SINGLE, 15091 },
	{ 0x870d, 0x870d, PDF_CMAP_SINGLE, 6514 },
	{ 0x870e, 0x870e, PDF_CMAP_SINGLE, 18595 },
	{ 0x870f, 0x870f, PDF_CMAP_SINGLE, 22427 },
	{ 0x8710, 0x8710, PDF_CMAP_SINGLE, 18596 },
	{ 0x8711, 0x8711, PDF_CMAP_SINGLE, 6512 },
	{ 0x8712, 0x8712, PDF_CMAP_SINGLE, 6506 },
	{ 0x8713, 0x8713, PDF_CMAP_SINGLE, 15092 },
	{ 0x8714, 0x8714, PDF_CMAP_SINGLE, 19756 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 2966 },
	{ 0x8719, 0x8719, PDF_CMAP_SINGLE, 15093 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 6523 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 3766 },
	{ 0x871e, 0x871e, PDF_CMAP_SINGLE, 15094 },
	{ 0x871f, 0x871f, PDF_CMAP_SINGLE, 18597 },
	{ 0x8721, 0x8721, PDF_CMAP_SINGLE, 18598 },
	{ 0x8722, 0x8722, PDF_CMAP_SINGLE, 19757 },
	{ 0x8723, 0x8723, PDF_CMAP_SINGLE, 18599 },
	{ 0x8725, 0x8725, PDF_CMAP_SINGLE, 6521 },
	{ 0x8728, 0x8728, PDF_CMAP_SINGLE, 15095 },
	{ 0x8729, 0x8729, PDF_CMAP_SINGLE, 6522 },
	{ 0x872e, 0x872e, PDF_CMAP_SINGLE, 19758 },
	{ 0x872f, 0x872f, PDF_CMAP_SINGLE, 22428 },
	{ 0x8731, 0x8731, PDF_CMAP_SINGLE, 18600 },
	{ 0x8732, 0x8732, PDF_CMAP_SINGLE, 22429 },
	{ 0x8734, 0x8734, PDF_CMAP_SINGLE, 6517 },
	{ 0x8737, 0x8737, PDF_CMAP_SINGLE, 6519 },
	{ 0x8739, 0x8739, PDF_CMAP_SINGLE, 19759 },
	{ 0x873a, 0x873a, PDF_CMAP_SINGLE, 18601 },
	{ 0x873b, 0x873b, PDF_CMAP_SINGLE, 6520 },
	{ 0x873c, 0x873d, PDF_CMAP_RANGE, 22430 },
	{ 0x873e, 0x873e, PDF_CMAP_SINGLE, 15096 },
	{ 0x873f, 0x873f, PDF_CMAP_SINGLE, 6518 },
	{ 0x8740, 0x8740, PDF_CMAP_SINGLE, 18602 },
	{ 0x8743, 0x8743, PDF_CMAP_SINGLE, 18603 },
	{ 0x8745, 0x8745, PDF_CMAP_SINGLE, 22432 },
	{ 0x8749, 0x8749, PDF_CMAP_SINGLE, 2698 },
	{ 0x874b, 0x874b, PDF_CMAP_SINGLE, 4063 },
	{ 0x874c, 0x874c, PDF_CMAP_SINGLE, 6527 },
	{ 0x874d, 0x874d, PDF_CMAP_SINGLE, 22433 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 6528 },
	{ 0x8751, 0x8751, PDF_CMAP_SINGLE, 18604 },
	{ 0x8753, 0x8753, PDF_CMAP_SINGLE, 6534 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 2544 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 6530 },
	{ 0x8758, 0x8758, PDF_CMAP_SINGLE, 18605 },
	{ 0x8759, 0x8759, PDF_CMAP_SINGLE, 6533 },
	{ 0x875d, 0x875d, PDF_CMAP_SINGLE, 19760 },
	{ 0x875f, 0x875f, PDF_CMAP_SINGLE, 6525 },
	{ 0x8760, 0x8760, PDF_CMAP_SINGLE, 6524 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 22434 },
	{ 0x8763, 0x8763, PDF_CMAP_SINGLE, 6535 },
	{ 0x8764, 0x8765, PDF_CMAP_RANGE, 18606 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 1372 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 6531 },
	{ 0x876a, 0x876a, PDF_CMAP_SINGLE, 6536 },
	{ 0x876e, 0x876e, PDF_CMAP_SINGLE, 6532 },
	{ 0x876f, 0x876f, PDF_CMAP_SINGLE, 22435 },
	{ 0x8771, 0x8771, PDF_CMAP_SINGLE, 15097 },
	{ 0x8772, 0x8772, PDF_CMAP_SINGLE, 18608 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 6529 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 3023 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 6526 },
	{ 0x877b, 0x877b, PDF_CMAP_SINGLE, 19761 },
	{ 0x877c, 0x877c, PDF_CMAP_SINGLE, 18609 },
	{ 0x877f, 0x877f, PDF_CMAP_SINGLE, 3358 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 6540 },
	{ 0x8783, 0x8786, PDF_CMAP_RANGE, 22436 },
	{ 0x8787, 0x8787, PDF_CMAP_SINGLE, 17112 },
	{ 0x8788, 0x8788, PDF_CMAP_SINGLE, 15098 },
	{ 0x8789, 0x8789, PDF_CMAP_SINGLE, 18613 },
	{ 0x878b, 0x878b, PDF_CMAP_SINGLE, 18614 },
	{ 0x878c, 0x878c, PDF_CMAP_SINGLE, 19762 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 3877 },
	{ 0x878e, 0x878e, PDF_CMAP_SINGLE, 19763 },
	{ 0x8790, 0x8790, PDF_CMAP_SINGLE, 22440 },
	{ 0x8793, 0x8793, PDF_CMAP_SINGLE, 18615 },
	{ 0x8795, 0x8795, PDF_CMAP_SINGLE, 22441 },
	{ 0x8797, 0x8798, PDF_CMAP_RANGE, 19764 },
	{ 0x8799, 0x8799, PDF_CMAP_SINGLE, 15099 },
	{ 0x879e, 0x879e, PDF_CMAP_SINGLE, 19766 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 6539 },
	{ 0x87a0, 0x87a0, PDF_CMAP_SINGLE, 18616 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 6538 },
	{ 0x87a3, 0x87a3, PDF_CMAP_SINGLE, 19767 },
	{ 0x87a7, 0x87a7, PDF_CMAP_SINGLE, 18612 },
	{ 0x87ab, 0x87ab, PDF_CMAP_SINGLE, 6547 },
	{ 0x87ac, 0x87ad, PDF_CMAP_RANGE, 15100 },
	{ 0x87ae, 0x87ae, PDF_CMAP_SINGLE, 19768 },
	{ 0x87af, 0x87af, PDF_CMAP_SINGLE, 6541 },
	{ 0x87b1, 0x87b1, PDF_CMAP_SINGLE, 22443 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 6549 },
	{ 0x87b5, 0x87b5, PDF_CMAP_SINGLE, 15102 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 3920 },
	{ 0x87bb, 0x87bb, PDF_CMAP_SINGLE, 6552 },
	{ 0x87bd, 0x87bd, PDF_CMAP_SINGLE, 6543 },
	{ 0x87be, 0x87be, PDF_CMAP_SINGLE, 18619 },
	{ 0x87bf, 0x87bf, PDF_CMAP_SINGLE, 19769 },
	{ 0x87c0, 0x87c0, PDF_CMAP_SINGLE, 6544 },
	{ 0x87c1, 0x87c1, PDF_CMAP_SINGLE, 18621 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 6548 },
	{ 0x87c6, 0x87c6, PDF_CMAP_SINGLE, 6551 },
	{ 0x87c7, 0x87c7, PDF_CMAP_SINGLE, 6550 },
	{ 0x87c8, 0x87c8, PDF_CMAP_SINGLE, 22444 },
	{ 0x87c9, 0x87c9, PDF_CMAP_SINGLE, 19770 },
	{ 0x87ca, 0x87ca, PDF_CMAP_SINGLE, 22445 },
	{ 0x87cb, 0x87cb, PDF_CMAP_SINGLE, 6542 },
	{ 0x87ce, 0x87ce, PDF_CMAP_SINGLE, 18622 },
	{ 0x87d0, 0x87d0, PDF_CMAP_SINGLE, 6545 },
	{ 0x87d2, 0x87d2, PDF_CMAP_SINGLE, 6562 },
	{ 0x87d5, 0x87d5, PDF_CMAP_SINGLE, 22446 },
	{ 0x87d6, 0x87d6, PDF_CMAP_SINGLE, 15103 },
	{ 0x87d9, 0x87d9, PDF_CMAP_SINGLE, 22447 },
	{ 0x87da, 0x87da, PDF_CMAP_SINGLE, 19771 },
	{ 0x87dc, 0x87dc, PDF_CMAP_SINGLE, 22448 },
	{ 0x87df, 0x87df, PDF_CMAP_SINGLE, 18623 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 6555 },
	{ 0x87e2, 0x87e2, PDF_CMAP_SINGLE, 22449 },
	{ 0x87e3, 0x87e3, PDF_CMAP_SINGLE, 18625 },
	{ 0x87e4, 0x87e4, PDF_CMAP_SINGLE, 22450 },
	{ 0x87e5, 0x87e6, PDF_CMAP_RANGE, 18626 },
	{ 0x87ea, 0x87ea, PDF_CMAP_SINGLE, 18628 },
	{ 0x87eb, 0x87eb, PDF_CMAP_SINGLE, 15104 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 7715 },
	{ 0x87ed, 0x87ed, PDF_CMAP_SINGLE, 15105 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 6553 },
	{ 0x87f1, 0x87f1, PDF_CMAP_SINGLE, 22451 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 6554 },
	{ 0x87f3, 0x87f3, PDF_CMAP_SINGLE, 22452 },
	{ 0x87f5, 0x87f5, PDF_CMAP_SINGLE, 15421 },
	{ 0x87f6, 0x87f7, PDF_CMAP_RANGE, 6559 },
	{ 0x87f8, 0x87f8, PDF_CMAP_SINGLE, 22453 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 1416 },
	{ 0x87fa, 0x87fa, PDF_CMAP_SINGLE, 22454 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 1628 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 6558 },
	{ 0x87ff, 0x87ff, PDF_CMAP_SINGLE, 22455 },
	{ 0x8801, 0x8801, PDF_CMAP_SINGLE, 15106 },
	{ 0x8803, 0x8803, PDF_CMAP_SINGLE, 20312 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 6537 },
	{ 0x8806, 0x8806, PDF_CMAP_SINGLE, 15108 },
	{ 0x8807, 0x8807, PDF_CMAP_SINGLE, 8614 },
	{ 0x8809, 0x8809, PDF_CMAP_SINGLE, 22456 },
	{ 0x880a, 0x880a, PDF_CMAP_SINGLE, 17114 },
	{ 0x880b, 0x880b, PDF_CMAP_SINGLE, 15109 },
	{ 0x880d, 0x880d, PDF_CMAP_SINGLE, 6557 },
	{ 0x880e, 0x880e, PDF_CMAP_SINGLE, 6561 },
	{ 0x880f, 0x880f, PDF_CMAP_SINGLE, 6556 },
	{ 0x8810, 0x8810, PDF_CMAP_SINGLE, 17115 },
	{ 0x8811, 0x8811, PDF_CMAP_SINGLE, 6563 },
	{ 0x8812, 0x8812, PDF_CMAP_SINGLE, 22458 },
	{ 0x8813, 0x8813, PDF_CMAP_SINGLE, 18629 },
	{ 0x8814, 0x8814, PDF_CMAP_SINGLE, 15110 },
	{ 0x8815, 0x8815, PDF_CMAP_SINGLE, 6565 },
	{ 0x8816, 0x8816, PDF_CMAP_SINGLE, 6564 },
	{ 0x8818, 0x8818, PDF_CMAP_SINGLE, 19772 },
	{ 0x8819, 0x8819, PDF_CMAP_SINGLE, 22457 },
	{ 0x881a, 0x881a, PDF_CMAP_SINGLE, 22459 },
	{ 0x881b, 0x881b, PDF_CMAP_SINGLE, 19773 },
	{ 0x881c, 0x881c, PDF_CMAP_SINGLE, 15111 },
	{ 0x881e, 0x881e, PDF_CMAP_SINGLE, 22460 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 7813 },
	{ 0x8821, 0x8821, PDF_CMAP_SINGLE, 6567 },
	{ 0x8822, 0x8822, PDF_CMAP_SINGLE, 6566 },
	{ 0x8823, 0x8823, PDF_CMAP_SINGLE, 6497 },
	{ 0x8827, 0x8827, PDF_CMAP_SINGLE, 6571 },
	{ 0x8828, 0x8828, PDF_CMAP_SINGLE, 18630 },
	{ 0x882d, 0x882d, PDF_CMAP_SINGLE, 19774 },
	{ 0x882e, 0x882e, PDF_CMAP_SINGLE, 18631 },
	{ 0x8830, 0x8830, PDF_CMAP_SINGLE, 22461 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 6568 },
	{ 0x8832, 0x8832, PDF_CMAP_SINGLE, 18632 },
	{ 0x8835, 0x8835, PDF_CMAP_SINGLE, 22462 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 6569 },
	{ 0x8839, 0x8839, PDF_CMAP_SINGLE, 6570 },
	{ 0x883a, 0x883a, PDF_CMAP_SINGLE, 19775 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 6572 },
	{ 0x883c, 0x883c, PDF_CMAP_SINGLE, 18633 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 1858 },
	{ 0x8841, 0x8841, PDF_CMAP_SINGLE, 22463 },
	{ 0x8842, 0x8842, PDF_CMAP_SINGLE, 6574 },
	{ 0x8843, 0x8843, PDF_CMAP_SINGLE, 22464 },
	{ 0x8844, 0x8844, PDF_CMAP_SINGLE, 6573 },
	{ 0x8845, 0x8845, PDF_CMAP_SINGLE, 19776 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 2362 },
	{ 0x8848, 0x8849, PDF_CMAP_RANGE, 22465 },
	{ 0x884a, 0x884a, PDF_CMAP_SINGLE, 18635 },
	{ 0x884b, 0x884b, PDF_CMAP_SINGLE, 19777 },
	{ 0x884c, 0x884c, PDF_CMAP_SINGLE, 2022 },
	{ 0x884d, 0x884d, PDF_CMAP_SINGLE, 5412 },
	{ 0x884e, 0x884e, PDF_CMAP_SINGLE, 19778 },
	{ 0x8851, 0x8851, PDF_CMAP_SINGLE, 22468 },
	{ 0x8852, 0x8852, PDF_CMAP_SINGLE, 6575 },
	{ 0x8853, 0x8853, PDF_CMAP_SINGLE, 2395 },
	{ 0x8855, 0x8855, PDF_CMAP_SINGLE, 19779 },
	{ 0x8856, 0x8856, PDF_CMAP_SINGLE, 15112 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 1431 },
	{ 0x8858, 0x8858, PDF_CMAP_SINGLE, 18636 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 6576 },
	{ 0x885a, 0x885a, PDF_CMAP_SINGLE, 19780 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 1268 },
	{ 0x885c, 0x885c, PDF_CMAP_SINGLE, 22469 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 2495 },
	{ 0x885e, 0x885e, PDF_CMAP_SINGLE, 6577 },
	{ 0x885f, 0x885f, PDF_CMAP_SINGLE, 15113 },
	{ 0x8860, 0x8860, PDF_CMAP_SINGLE, 22470 },
	{ 0x8861, 0x8861, PDF_CMAP_SINGLE, 2023 },
	{ 0x8862, 0x8862, PDF_CMAP_SINGLE, 6578 },
	{ 0x8863, 0x8863, PDF_CMAP_SINGLE, 1189 },
	{ 0x8864, 0x8864, PDF_CMAP_SINGLE, 15114 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 3503 },
	{ 0x8869, 0x8869, PDF_CMAP_SINGLE, 18639 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 6579 },
	{ 0x886e, 0x886e, PDF_CMAP_SINGLE, 19781 },
	{ 0x886f, 0x886f, PDF_CMAP_SINGLE, 18641 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 2608 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 22471 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 6586 },
	{ 0x8875, 0x8875, PDF_CMAP_SINGLE, 6583 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 2989 },
	{ 0x8879, 0x8879, PDF_CMAP_SINGLE, 22472 },
	{ 0x887b, 0x887b, PDF_CMAP_SINGLE, 22473 },
	{ 0x887d, 0x887d, PDF_CMAP_SINGLE, 6584 },
	{ 0x887e, 0x887e, PDF_CMAP_SINGLE, 6581 },
	{ 0x887f, 0x887f, PDF_CMAP_SINGLE, 1750 },
	{ 0x8880, 0x8880, PDF_CMAP_SINGLE, 22474 },
	{ 0x8881, 0x8881, PDF_CMAP_SINGLE, 6580 },
	{ 0x8882, 0x8882, PDF_CMAP_SINGLE, 6587 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 1804 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 2878 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 6593 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 6589 },
	{ 0x8896, 0x8896, PDF_CMAP_SINGLE, 2837 },
	{ 0x8897, 0x8897, PDF_CMAP_SINGLE, 6588 },
	{ 0x8898, 0x8898, PDF_CMAP_SINGLE, 15115 },
	{ 0x8899, 0x8899, PDF_CMAP_SINGLE, 6591 },
	{ 0x889a, 0x889c, PDF_CMAP_RANGE, 19782 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 6582 },
	{ 0x889f, 0x889f, PDF_CMAP_SINGLE, 22475 },
	{ 0x88a0, 0x88a0, PDF_CMAP_SINGLE, 18642 },
	{ 0x88a2, 0x88a2, PDF_CMAP_SINGLE, 6592 },
	{ 0x88a4, 0x88a4, PDF_CMAP_SINGLE, 6594 },
	{ 0x88a8, 0x88a8, PDF_CMAP_SINGLE, 22476 },
	{ 0x88aa, 0x88aa, PDF_CMAP_SINGLE, 15116 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 3459 },
	{ 0x88ae, 0x88ae, PDF_CMAP_SINGLE, 6590 },
	{ 0x88b0, 0x88b0, PDF_CMAP_SINGLE, 6595 },
	{ 0x88b1, 0x88b1, PDF_CMAP_SINGLE, 6597 },
	{ 0x88b4, 0x88b4, PDF_CMAP_SINGLE, 1927 },
	{ 0x88b5, 0x88b5, PDF_CMAP_SINGLE, 6585 },
	{ 0x88b7, 0x88b7, PDF_CMAP_SINGLE, 1157 },
	{ 0x88ba, 0x88ba, PDF_CMAP_SINGLE, 22477 },
	{ 0x88bc, 0x88bc, PDF_CMAP_SINGLE, 18643 },
	{ 0x88bd, 0x88be, PDF_CMAP_RANGE, 15117 },
	{ 0x88bf, 0x88bf, PDF_CMAP_SINGLE, 6596 },
	{ 0x88c0, 0x88c0, PDF_CMAP_SINGLE, 18644 },
	{ 0x88c1, 0x88c1, PDF_CMAP_SINGLE, 2123 },
	{ 0x88c2, 0x88c2, PDF_CMAP_SINGLE, 4030 },
	{ 0x88c3, 0x88c4, PDF_CMAP_RANGE, 6598 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 2807 },
	{ 0x88c6, 0x88c6, PDF_CMAP_SINGLE, 14217 },
	{ 0x88ca, 0x88ca, PDF_CMAP_SINGLE, 15119 },
	{ 0x88cb, 0x88cc, PDF_CMAP_RANGE, 22478 },
	{ 0x88cd, 0x88cd, PDF_CMAP_SINGLE, 19785 },
	{ 0x88ce, 0x88ce, PDF_CMAP_SINGLE, 17116 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 3946 },
	{ 0x88d1, 0x88d1, PDF_CMAP_SINGLE, 18646 },
	{ 0x88d2, 0x88d2, PDF_CMAP_SINGLE, 15120 },
	{ 0x88d3, 0x88d3, PDF_CMAP_SINGLE, 18647 },
	{ 0x88d4, 0x88d4, PDF_CMAP_SINGLE, 6600 },
	{ 0x88d5, 0x88d5, PDF_CMAP_SINGLE, 3871 },
	{ 0x88d8, 0x88d9, PDF_CMAP_RANGE, 6601 },
	{ 0x88db, 0x88db, PDF_CMAP_SINGLE, 15121 },
	{ 0x88dc, 0x88dc, PDF_CMAP_SINGLE, 3636 },
	{ 0x88dd, 0x88dd, PDF_CMAP_SINGLE, 6603 },
	{ 0x88de, 0x88de, PDF_CMAP_SINGLE, 22480 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 2096 },
	{ 0x88e0, 0x88e0, PDF_CMAP_SINGLE, 19786 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 3947 },
	{ 0x88e7, 0x88e7, PDF_CMAP_SINGLE, 22481 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 6608 },
	{ 0x88ef, 0x88ef, PDF_CMAP_SINGLE, 19787 },
	{ 0x88f0, 0x88f1, PDF_CMAP_RANGE, 15122 },
	{ 0x88f2, 0x88f2, PDF_CMAP_SINGLE, 6609 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 2496 },
	{ 0x88f4, 0x88f4, PDF_CMAP_SINGLE, 6607 },
	{ 0x88f5, 0x88f5, PDF_CMAP_SINGLE, 8615 },
	{ 0x88f7, 0x88f7, PDF_CMAP_SINGLE, 22482 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 3921 },
	{ 0x88f9, 0x88f9, PDF_CMAP_SINGLE, 6604 },
	{ 0x88fc, 0x88fc, PDF_CMAP_SINGLE, 6606 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 2657 },
	{ 0x88fe, 0x88fe, PDF_CMAP_SINGLE, 2628 },
	{ 0x8901, 0x8901, PDF_CMAP_SINGLE, 18649 },
	{ 0x8902, 0x8902, PDF_CMAP_SINGLE, 6605 },
	{ 0x8904, 0x8904, PDF_CMAP_SINGLE, 6610 },
	{ 0x8906, 0x8906, PDF_CMAP_SINGLE, 15124 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 3571 },
	{ 0x890a, 0x890a, PDF_CMAP_SINGLE, 6612 },
	{ 0x890c, 0x890c, PDF_CMAP_SINGLE, 6611 },
	{ 0x890d, 0x890d, PDF_CMAP_SINGLE, 22483 },
	{ 0x890e, 0x890f, PDF_CMAP_RANGE, 19788 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 1482 },
	{ 0x8912, 0x8912, PDF_CMAP_SINGLE, 3673 },
	{ 0x8913, 0x8913, PDF_CMAP_SINGLE, 6613 },
	{ 0x8915, 0x8916, PDF_CMAP_RANGE, 22484 },
	{ 0x8918, 0x891a, PDF_CMAP_RANGE, 15125 },
	{ 0x891c, 0x891c, PDF_CMAP_SINGLE, 8360 },
	{ 0x891d, 0x891d, PDF_CMAP_SINGLE, 6625 },
	{ 0x891e, 0x891e, PDF_CMAP_SINGLE, 6615 },
	{ 0x8920, 0x8920, PDF_CMAP_SINGLE, 22486 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 6616 },
	{ 0x8926, 0x8926, PDF_CMAP_SINGLE, 19790 },
	{ 0x8927, 0x8927, PDF_CMAP_SINGLE, 15128 },
	{ 0x8928, 0x8928, PDF_CMAP_SINGLE, 22487 },
	{ 0x892a, 0x892b, PDF_CMAP_RANGE, 6617 },
	{ 0x8930, 0x8930, PDF_CMAP_SINGLE, 15129 },
	{ 0x8931, 0x8931, PDF_CMAP_SINGLE, 22488 },
	{ 0x8932, 0x8932, PDF_CMAP_SINGLE, 17118 },
	{ 0x8935, 0x8935, PDF_CMAP_SINGLE, 19791 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 6622 },
	{ 0x8937, 0x8937, PDF_CMAP_SINGLE, 18651 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 6623 },
	{ 0x8939, 0x8939, PDF_CMAP_SINGLE, 17119 },
	{ 0x893a, 0x893a, PDF_CMAP_SINGLE, 22489 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 6621 },
	{ 0x893e, 0x893e, PDF_CMAP_SINGLE, 15130 },
	{ 0x8940, 0x8940, PDF_CMAP_SINGLE, 17120 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 6619 },
	{ 0x8942, 0x8942, PDF_CMAP_SINGLE, 18653 },
	{ 0x8943, 0x8943, PDF_CMAP_SINGLE, 6614 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 6620 },
	{ 0x8945, 0x8945, PDF_CMAP_SINGLE, 18654 },
	{ 0x8946, 0x8946, PDF_CMAP_SINGLE, 22490 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 18655 },
	{ 0x894c, 0x894c, PDF_CMAP_SINGLE, 6624 },
	{ 0x894d, 0x894d, PDF_CMAP_SINGLE, 7120 },
	{ 0x894f, 0x894f, PDF_CMAP_SINGLE, 22491 },
	{ 0x8952, 0x8952, PDF_CMAP_SINGLE, 22492 },
	{ 0x8956, 0x8956, PDF_CMAP_SINGLE, 1320 },
	{ 0x8957, 0x8957, PDF_CMAP_SINGLE, 22493 },
	{ 0x895a, 0x895a, PDF_CMAP_SINGLE, 19792 },
	{ 0x895b, 0x895b, PDF_CMAP_SINGLE, 22494 },
	{ 0x895c, 0x895c, PDF_CMAP_SINGLE, 19793 },
	{ 0x895e, 0x895e, PDF_CMAP_SINGLE, 6627 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 1751 },
	{ 0x8960, 0x8960, PDF_CMAP_SINGLE, 6626 },
	{ 0x8961, 0x8961, PDF_CMAP_SINGLE, 22495 },
	{ 0x8962, 0x8962, PDF_CMAP_SINGLE, 18657 },
	{ 0x8963, 0x8963, PDF_CMAP_SINGLE, 22496 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 6629 },
	{ 0x8966, 0x8966, PDF_CMAP_SINGLE, 6628 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 6631 },
	{ 0x896b, 0x896b, PDF_CMAP_SINGLE, 19794 },
	{ 0x896d, 0x896d, PDF_CMAP_SINGLE, 6630 },
	{ 0x896e, 0x896e, PDF_CMAP_SINGLE, 22497 },
	{ 0x896f, 0x896f, PDF_CMAP_SINGLE, 6632 },
	{ 0x8970, 0x8970, PDF_CMAP_SINGLE, 19795 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 2363 },
	{ 0x8973, 0x8973, PDF_CMAP_SINGLE, 22498 },
	{ 0x8974, 0x8974, PDF_CMAP_SINGLE, 6633 },
	{ 0x8975, 0x8975, PDF_CMAP_SINGLE, 22499 },
	{ 0x8977, 0x8977, PDF_CMAP_SINGLE, 6634 },
	{ 0x897a, 0x897a, PDF_CMAP_SINGLE, 22500 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 15131 },
	{ 0x897c, 0x897c, PDF_CMAP_SINGLE, 19796 },
	{ 0x897d, 0x897d, PDF_CMAP_SINGLE, 22501 },
	{ 0x897e, 0x897e, PDF_CMAP_SINGLE, 6635 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 2658 },
	{ 0x8980, 0x8980, PDF_CMAP_SINGLE, 13870 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 3905 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 6636 },
	{ 0x8986, 0x8986, PDF_CMAP_SINGLE, 3572 },
	{ 0x8987, 0x8987, PDF_CMAP_SINGLE, 3324 },
	{ 0x8988, 0x8988, PDF_CMAP_SINGLE, 6637 },
	{ 0x8989, 0x8989, PDF_CMAP_SINGLE, 18658 },
	{ 0x898a, 0x898a, PDF_CMAP_SINGLE, 6638 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 1887 },
	{ 0x898d, 0x898d, PDF_CMAP_SINGLE, 22502 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 1606 },
	{ 0x8990, 0x8990, PDF_CMAP_SINGLE, 18659 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 6639 },
	{ 0x8994, 0x8994, PDF_CMAP_SINGLE, 17121 },
	{ 0x8995, 0x8995, PDF_CMAP_SINGLE, 22503 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 2233 },
	{ 0x8997, 0x8997, PDF_CMAP_SINGLE, 3319 },
	{ 0x8998, 0x8998, PDF_CMAP_SINGLE, 6640 },
	{ 0x899a, 0x899a, PDF_CMAP_SINGLE, 1454 },
	{ 0x899b, 0x899c, PDF_CMAP_RANGE, 22504 },
	{ 0x899f, 0x899f, PDF_CMAP_SINGLE, 18660 },
	{ 0x89a0, 0x89a0, PDF_CMAP_SINGLE, 22506 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 6641 },
	{ 0x89a5, 0x89a5, PDF_CMAP_SINGLE, 19797 },
	{ 0x89a6, 0x89a6, PDF_CMAP_SINGLE, 6643 },
	{ 0x89a7, 0x89a7, PDF_CMAP_SINGLE, 3937 },
	{ 0x89a9, 0x89a9, PDF_CMAP_SINGLE, 6642 },
	{ 0x89aa, 0x89aa, PDF_CMAP_SINGLE, 2572 },
	{ 0x89ac, 0x89ac, PDF_CMAP_SINGLE, 6644 },
	{ 0x89af, 0x89af, PDF_CMAP_SINGLE, 6645 },
	{ 0x89b0, 0x89b0, PDF_CMAP_SINGLE, 18661 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 6646 },
	{ 0x89b3, 0x89b3, PDF_CMAP_SINGLE, 1549 },
	{ 0x89b4, 0x89b4, PDF_CMAP_SINGLE, 22507 },
	{ 0x89b5, 0x89b5, PDF_CMAP_SINGLE, 19798 },
	{ 0x89b6, 0x89b6, PDF_CMAP_SINGLE, 22508 },
	{ 0x89b7, 0x89b7, PDF_CMAP_SINGLE, 18662 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 6647 },
	{ 0x89bc, 0x89bc, PDF_CMAP_SINGLE, 19799 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 6648 },
	{ 0x89bf, 0x89c0, PDF_CMAP_RANGE, 6649 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 1455 },
	{ 0x89d4, 0x89d4, PDF_CMAP_SINGLE, 15132 },
	{ 0x89d5, 0x89d5, PDF_CMAP_SINGLE, 19800 },
	{ 0x89d6, 0x89d6, PDF_CMAP_SINGLE, 15133 },
	{ 0x89d7, 0x89d7, PDF_CMAP_SINGLE, 22509 },
	{ 0x89d8, 0x89d8, PDF_CMAP_SINGLE, 18663 },
	{ 0x89da, 0x89da, PDF_CMAP_SINGLE, 6651 },
	{ 0x89dc, 0x89dd, PDF_CMAP_RANGE, 6652 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 1394 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 15134 },
	{ 0x89e6, 0x89e6, PDF_CMAP_SINGLE, 2542 },
	{ 0x89e7, 0x89e7, PDF_CMAP_SINGLE, 6654 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 22510 },
	{ 0x89eb, 0x89eb, PDF_CMAP_SINGLE, 18664 },
	{ 0x89ed, 0x89ed, PDF_CMAP_SINGLE, 22511 },
	{ 0x89f1, 0x89f1, PDF_CMAP_SINGLE, 15135 },
	{ 0x89f3, 0x89f3, PDF_CMAP_SINGLE, 18666 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 6655 },
	{ 0x89f6, 0x89f6, PDF_CMAP_SINGLE, 17122 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 6656 },
	{ 0x89f9, 0x89f9, PDF_CMAP_SINGLE, 22512 },
	{ 0x89fd, 0x89fd, PDF_CMAP_SINGLE, 18667 },
	{ 0x89ff, 0x89ff, PDF_CMAP_SINGLE, 18668 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 1908 },
	{ 0x8a01, 0x8a01, PDF_CMAP_SINGLE, 13756 },
	{ 0x8a02, 0x8a02, PDF_CMAP_SINGLE, 3095 },
	{ 0x8a03, 0x8a03, PDF_CMAP_SINGLE, 6657 },
	{ 0x8a04, 0x8a05, PDF_CMAP_RANGE, 22513 },
	{ 0x8a07, 0x8a07, PDF_CMAP_SINGLE, 15136 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 1837 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 2588 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 6660 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 3196 },
	{ 0x8a0f, 0x8a0f, PDF_CMAP_SINGLE, 15137 },
	{ 0x8a10, 0x8a10, PDF_CMAP_SINGLE, 6659 },
	{ 0x8a11, 0x8a11, PDF_CMAP_SINGLE, 18670 },
	{ 0x8a12, 0x8a12, PDF_CMAP_SINGLE, 8616 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 1799 },
	{ 0x8a14, 0x8a14, PDF_CMAP_SINGLE, 18671 },
	{ 0x8a15, 0x8a15, PDF_CMAP_SINGLE, 15138 },
	{ 0x8a16, 0x8a16, PDF_CMAP_SINGLE, 6658 },
	{ 0x8a17, 0x8a17, PDF_CMAP_SINGLE, 2903 },
	{ 0x8a18, 0x8a18, PDF_CMAP_SINGLE, 1607 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 6661 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 6662 },
	{ 0x8a1e, 0x8a1e, PDF_CMAP_SINGLE, 22515 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 2497 },
	{ 0x8a20, 0x8a20, PDF_CMAP_SINGLE, 22516 },
	{ 0x8a21, 0x8a21, PDF_CMAP_SINGLE, 18673 },
	{ 0x8a22, 0x8a22, PDF_CMAP_SINGLE, 15139 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 1859 },
	{ 0x8a24, 0x8a24, PDF_CMAP_SINGLE, 22517 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 6663 },
	{ 0x8a26, 0x8a26, PDF_CMAP_SINGLE, 22518 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 3674 },
	{ 0x8a2b, 0x8a2c, PDF_CMAP_RANGE, 22519 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 2691 },
	{ 0x8a2f, 0x8a2f, PDF_CMAP_SINGLE, 22521 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 1680 },
	{ 0x8a33, 0x8a33, PDF_CMAP_SINGLE, 3841 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 2764 },
	{ 0x8a35, 0x8a35, PDF_CMAP_SINGLE, 18674 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 6664 },
	{ 0x8a37, 0x8a37, PDF_CMAP_SINGLE, 8617 },
	{ 0x8a3a, 0x8a3a, PDF_CMAP_SINGLE, 2573 },
	{ 0x8a3b, 0x8a3b, PDF_CMAP_SINGLE, 2990 },
	{ 0x8a3c, 0x8a3c, PDF_CMAP_SINGLE, 2498 },
	{ 0x8a3d, 0x8a3d, PDF_CMAP_SINGLE, 22522 },
	{ 0x8a3e, 0x8a3e, PDF_CMAP_SINGLE, 18675 },
	{ 0x8a40, 0x8a40, PDF_CMAP_SINGLE, 22523 },
	{ 0x8a41, 0x8a41, PDF_CMAP_SINGLE, 6665 },
	{ 0x8a43, 0x8a43, PDF_CMAP_SINGLE, 22524 },
	{ 0x8a45, 0x8a45, PDF_CMAP_SINGLE, 18676 },
	{ 0x8a46, 0x8a46, PDF_CMAP_SINGLE, 6668 },
	{ 0x8a47, 0x8a47, PDF_CMAP_SINGLE, 17123 },
	{ 0x8a48, 0x8a48, PDF_CMAP_SINGLE, 6669 },
	{ 0x8a49, 0x8a49, PDF_CMAP_SINGLE, 19801 },
	{ 0x8a4d, 0x8a4d, PDF_CMAP_SINGLE, 18677 },
	{ 0x8a4e, 0x8a4e, PDF_CMAP_SINGLE, 15140 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 2094 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 2850 },
	{ 0x8a52, 0x8a52, PDF_CMAP_SINGLE, 6667 },
	{ 0x8a53, 0x8a53, PDF_CMAP_SINGLE, 22525 },
	{ 0x8a54, 0x8a54, PDF_CMAP_SINGLE, 2499 },
	{ 0x8a55, 0x8a55, PDF_CMAP_SINGLE, 3504 },
	{ 0x8a56, 0x8a56, PDF_CMAP_SINGLE, 22526 },
	{ 0x8a57, 0x8a57, PDF_CMAP_SINGLE, 19802 },
	{ 0x8a58, 0x8a58, PDF_CMAP_SINGLE, 18678 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 6666 },
	{ 0x8a5c, 0x8a5c, PDF_CMAP_SINGLE, 22527 },
	{ 0x8a5d, 0x8a5d, PDF_CMAP_SINGLE, 17124 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 2234 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 1269 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 17125 },
	{ 0x8a62, 0x8a62, PDF_CMAP_SINGLE, 6673 },
	{ 0x8a63, 0x8a63, PDF_CMAP_SINGLE, 1838 },
	{ 0x8a65, 0x8a65, PDF_CMAP_SINGLE, 22528 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 2236 },
	{ 0x8a67, 0x8a67, PDF_CMAP_SINGLE, 19803 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 2235 },
	{ 0x8a6b, 0x8a6b, PDF_CMAP_SINGLE, 4083 },
	{ 0x8a6c, 0x8a6c, PDF_CMAP_SINGLE, 6672 },
	{ 0x8a6d, 0x8a6d, PDF_CMAP_SINGLE, 6671 },
	{ 0x8a6e, 0x8a6e, PDF_CMAP_SINGLE, 2729 },
	{ 0x8a70, 0x8a70, PDF_CMAP_SINGLE, 1639 },
	{ 0x8a71, 0x8a71, PDF_CMAP_SINGLE, 4073 },
	{ 0x8a72, 0x8a72, PDF_CMAP_SINGLE, 1432 },
	{ 0x8a73, 0x8a73, PDF_CMAP_SINGLE, 2500 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 17126 },
	{ 0x8a76, 0x8a77, PDF_CMAP_RANGE, 22529 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 8618 },
	{ 0x8a7a, 0x8a7b, PDF_CMAP_RANGE, 22531 },
	{ 0x8a7c, 0x8a7c, PDF_CMAP_SINGLE, 6670 },
	{ 0x8a7e, 0x8a7e, PDF_CMAP_SINGLE, 19804 },
	{ 0x8a7f, 0x8a7f, PDF_CMAP_SINGLE, 15141 },
	{ 0x8a80, 0x8a80, PDF_CMAP_SINGLE, 22533 },
	{ 0x8a82, 0x8a82, PDF_CMAP_SINGLE, 6675 },
	{ 0x8a83, 0x8a83, PDF_CMAP_SINGLE, 22534 },
	{ 0x8a84, 0x8a84, PDF_CMAP_SINGLE, 6676 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 6674 },
	{ 0x8a86, 0x8a86, PDF_CMAP_SINGLE, 19805 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 1932 },
	{ 0x8a89, 0x8a89, PDF_CMAP_SINGLE, 3882 },
	{ 0x8a8b, 0x8a8b, PDF_CMAP_SINGLE, 22535 },
	{ 0x8a8c, 0x8a8c, PDF_CMAP_SINGLE, 2237 },
	{ 0x8a8d, 0x8a8d, PDF_CMAP_SINGLE, 3293 },
	{ 0x8a8f, 0x8a8f, PDF_CMAP_SINGLE, 22536 },
	{ 0x8a90, 0x8a90, PDF_CMAP_SINGLE, 18680 },
	{ 0x8a91, 0x8a91, PDF_CMAP_SINGLE, 6679 },
	{ 0x8a92, 0x8a92, PDF_CMAP_SINGLE, 22537 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 2660 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 2944 },
	{ 0x8a96, 0x8a96, PDF_CMAP_SINGLE, 19806 },
	{ 0x8a97, 0x8a97, PDF_CMAP_SINGLE, 22538 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 3872 },
	{ 0x8a99, 0x8a99, PDF_CMAP_SINGLE, 22539 },
	{ 0x8a9a, 0x8a9a, PDF_CMAP_SINGLE, 6682 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 1952 },
	{ 0x8a9f, 0x8a9f, PDF_CMAP_SINGLE, 22540 },
	{ 0x8aa0, 0x8aa0, PDF_CMAP_SINGLE, 2659 },
	{ 0x8aa1, 0x8aa1, PDF_CMAP_SINGLE, 6678 },
	{ 0x8aa3, 0x8aa3, PDF_CMAP_SINGLE, 6683 },
	{ 0x8aa4, 0x8aa4, PDF_CMAP_SINGLE, 1953 },
	{ 0x8aa5, 0x8aa6, PDF_CMAP_RANGE, 6680 },
	{ 0x8aa7, 0x8aa7, PDF_CMAP_SINGLE, 8619 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 6677 },
	{ 0x8aa9, 0x8aa9, PDF_CMAP_SINGLE, 22541 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 13880 },
	{ 0x8aac, 0x8aac, PDF_CMAP_SINGLE, 2694 },
	{ 0x8aad, 0x8aad, PDF_CMAP_SINGLE, 3233 },
	{ 0x8aae, 0x8aae, PDF_CMAP_SINGLE, 18679 },
	{ 0x8aaf, 0x8aaf, PDF_CMAP_SINGLE, 22542 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 2925 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 1373 },
	{ 0x8ab3, 0x8ab3, PDF_CMAP_SINGLE, 22543 },
	{ 0x8ab6, 0x8ab6, PDF_CMAP_SINGLE, 19807 },
	{ 0x8ab7, 0x8ab7, PDF_CMAP_SINGLE, 18681 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 3460 },
	{ 0x8abb, 0x8abb, PDF_CMAP_SINGLE, 22544 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 1629 },
	{ 0x8abe, 0x8abe, PDF_CMAP_SINGLE, 8620 },
	{ 0x8abf, 0x8abf, PDF_CMAP_SINGLE, 3024 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 6686 },
	{ 0x8ac3, 0x8ac3, PDF_CMAP_SINGLE, 22545 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 6684 },
	{ 0x8ac6, 0x8ac6, PDF_CMAP_SINGLE, 22546 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 2954 },
	{ 0x8ac8, 0x8ac8, PDF_CMAP_SINGLE, 22547 },
	{ 0x8ac9, 0x8ac9, PDF_CMAP_SINGLE, 19808 },
	{ 0x8aca, 0x8aca, PDF_CMAP_SINGLE, 22548 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 2661 },
	{ 0x8acc, 0x8acc, PDF_CMAP_SINGLE, 1550 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 6685 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 2593 },
	{ 0x8ad0, 0x8ad0, PDF_CMAP_SINGLE, 17127 },
	{ 0x8ad1, 0x8ad1, PDF_CMAP_SINGLE, 19809 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 3986 },
	{ 0x8ad3, 0x8ad5, PDF_CMAP_RANGE, 22549 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4070 },
	{ 0x8ad7, 0x8ad7, PDF_CMAP_SINGLE, 18682 },
	{ 0x8ada, 0x8ada, PDF_CMAP_SINGLE, 6687 },
	{ 0x8adb, 0x8adb, PDF_CMAP_SINGLE, 6698 },
	{ 0x8adc, 0x8adc, PDF_CMAP_SINGLE, 3025 },
	{ 0x8add, 0x8add, PDF_CMAP_SINGLE, 19810 },
	{ 0x8ade, 0x8ade, PDF_CMAP_SINGLE, 6697 },
	{ 0x8adf, 0x8adf, PDF_CMAP_SINGLE, 8621 },
	{ 0x8ae0, 0x8ae0, PDF_CMAP_SINGLE, 6694 },
	{ 0x8ae1, 0x8ae1, PDF_CMAP_SINGLE, 6702 },
	{ 0x8ae2, 0x8ae2, PDF_CMAP_SINGLE, 6695 },
	{ 0x8ae4, 0x8ae4, PDF_CMAP_SINGLE, 6691 },
	{ 0x8ae6, 0x8ae6, PDF_CMAP_SINGLE, 3096 },
	{ 0x8ae7, 0x8ae7, PDF_CMAP_SINGLE, 6690 },
	{ 0x8aeb, 0x8aeb, PDF_CMAP_SINGLE, 6688 },
	{ 0x8aec, 0x8aec, PDF_CMAP_SINGLE, 19811 },
	{ 0x8aed, 0x8aed, PDF_CMAP_SINGLE, 3851 },
	{ 0x8aee, 0x8aee, PDF_CMAP_SINGLE, 2238 },
	{ 0x8af0, 0x8af0, PDF_CMAP_SINGLE, 22552 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 6692 },
	{ 0x8af3, 0x8af3, PDF_CMAP_SINGLE, 6689 },
	{ 0x8af4, 0x8af4, PDF_CMAP_SINGLE, 15142 },
	{ 0x8af5, 0x8af5, PDF_CMAP_SINGLE, 19812 },
	{ 0x8af6, 0x8af6, PDF_CMAP_SINGLE, 8623 },
	{ 0x8af7, 0x8af7, PDF_CMAP_SINGLE, 6696 },
	{ 0x8af8, 0x8af8, PDF_CMAP_SINGLE, 2430 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 1909 },
	{ 0x8afc, 0x8afc, PDF_CMAP_SINGLE, 18683 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 2906 },
	{ 0x8aff, 0x8aff, PDF_CMAP_SINGLE, 22553 },
	{ 0x8b00, 0x8b00, PDF_CMAP_SINGLE, 3699 },
	{ 0x8b01, 0x8b01, PDF_CMAP_SINGLE, 1276 },
	{ 0x8b02, 0x8b02, PDF_CMAP_SINGLE, 1190 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 3197 },
	{ 0x8b05, 0x8b05, PDF_CMAP_SINGLE, 18686 },
	{ 0x8b06, 0x8b06, PDF_CMAP_SINGLE, 19813 },
	{ 0x8b07, 0x8b07, PDF_CMAP_SINGLE, 6700 },
	{ 0x8b0a, 0x8b0a, PDF_CMAP_SINGLE, 18685 },
	{ 0x8b0b, 0x8b0b, PDF_CMAP_SINGLE, 22554 },
	{ 0x8b0c, 0x8b0c, PDF_CMAP_SINGLE, 6699 },
	{ 0x8b0d, 0x8b0d, PDF_CMAP_SINGLE, 18687 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 3262 },
	{ 0x8b0f, 0x8b0f, PDF_CMAP_SINGLE, 19814 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 6704 },
	{ 0x8b11, 0x8b11, PDF_CMAP_SINGLE, 19815 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 6693 },
	{ 0x8b16, 0x8b16, PDF_CMAP_SINGLE, 6703 },
	{ 0x8b17, 0x8b17, PDF_CMAP_SINGLE, 6705 },
	{ 0x8b19, 0x8b19, PDF_CMAP_SINGLE, 1888 },
	{ 0x8b1a, 0x8b1a, PDF_CMAP_SINGLE, 6701 },
	{ 0x8b1b, 0x8b1b, PDF_CMAP_SINGLE, 2024 },
	{ 0x8b1c, 0x8b1c, PDF_CMAP_SINGLE, 18688 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 2305 },
	{ 0x8b1e, 0x8b1e, PDF_CMAP_SINGLE, 22555 },
	{ 0x8b1f, 0x8b1f, PDF_CMAP_SINGLE, 15143 },
	{ 0x8b20, 0x8b20, PDF_CMAP_SINGLE, 6706 },
	{ 0x8b21, 0x8b21, PDF_CMAP_SINGLE, 3906 },
	{ 0x8b26, 0x8b26, PDF_CMAP_SINGLE, 6709 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 6712 },
	{ 0x8b2b, 0x8b2b, PDF_CMAP_SINGLE, 6710 },
	{ 0x8b2c, 0x8b2c, PDF_CMAP_SINGLE, 3495 },
	{ 0x8b2d, 0x8b2d, PDF_CMAP_SINGLE, 18689 },
	{ 0x8b30, 0x8b30, PDF_CMAP_SINGLE, 22556 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 6707 },
	{ 0x8b37, 0x8b37, PDF_CMAP_SINGLE, 15144 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 1752 },
	{ 0x8b3c, 0x8b3c, PDF_CMAP_SINGLE, 22557 },
	{ 0x8b3e, 0x8b3e, PDF_CMAP_SINGLE, 6711 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 6713 },
	{ 0x8b42, 0x8b42, PDF_CMAP_SINGLE, 22558 },
	{ 0x8b43, 0x8b44, PDF_CMAP_RANGE, 15145 },
	{ 0x8b45, 0x8b45, PDF_CMAP_SINGLE, 19816 },
	{ 0x8b46, 0x8b46, PDF_CMAP_SINGLE, 17128 },
	{ 0x8b48, 0x8b48, PDF_CMAP_SINGLE, 22559 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 6717 },
	{ 0x8b4c, 0x8b4c, PDF_CMAP_SINGLE, 6714 },
	{ 0x8b4d, 0x8b4d, PDF_CMAP_SINGLE, 22560 },
	{ 0x8b4e, 0x8b4e, PDF_CMAP_SINGLE, 6716 },
	{ 0x8b4f, 0x8b4f, PDF_CMAP_SINGLE, 6715 },
	{ 0x8b51, 0x8b51, PDF_CMAP_SINGLE, 18691 },
	{ 0x8b52, 0x8b52, PDF_CMAP_SINGLE, 19817 },
	{ 0x8b53, 0x8b53, PDF_CMAP_SINGLE, 8624 },
	{ 0x8b54, 0x8b54, PDF_CMAP_SINGLE, 15147 },
	{ 0x8b56, 0x8b56, PDF_CMAP_SINGLE, 6718 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 2269 },
	{ 0x8b59, 0x8b59, PDF_CMAP_SINGLE, 17129 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 6720 },
	{ 0x8b5b, 0x8b5b, PDF_CMAP_SINGLE, 6719 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 3546 },
	{ 0x8b5e, 0x8b5e, PDF_CMAP_SINGLE, 18692 },
	{ 0x8b5f, 0x8b5f, PDF_CMAP_SINGLE, 6722 },
	{ 0x8b63, 0x8b63, PDF_CMAP_SINGLE, 22561 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 1839 },
	{ 0x8b69, 0x8b69, PDF_CMAP_SINGLE, 17130 },
	{ 0x8b6b, 0x8b6b, PDF_CMAP_SINGLE, 6721 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 6723 },
	{ 0x8b6d, 0x8b6d, PDF_CMAP_SINGLE, 19818 },
	{ 0x8b6f, 0x8b6f, PDF_CMAP_SINGLE, 6724 },
	{ 0x8b70, 0x8b70, PDF_CMAP_SINGLE, 1630 },
	{ 0x8b71, 0x8b71, PDF_CMAP_SINGLE, 6190 },
	{ 0x8b72, 0x8b72, PDF_CMAP_SINGLE, 2529 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 6725 },
	{ 0x8b76, 0x8b76, PDF_CMAP_SINGLE, 18693 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 1954 },
	{ 0x8b78, 0x8b78, PDF_CMAP_SINGLE, 19819 },
	{ 0x8b79, 0x8b79, PDF_CMAP_SINGLE, 22562 },
	{ 0x8b7c, 0x8b7c, PDF_CMAP_SINGLE, 19820 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 6726 },
	{ 0x8b7e, 0x8b7e, PDF_CMAP_SINGLE, 19821 },
	{ 0x8b7f, 0x8b7f, PDF_CMAP_SINGLE, 8625 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 6727 },
	{ 0x8b81, 0x8b81, PDF_CMAP_SINGLE, 18694 },
	{ 0x8b83, 0x8b83, PDF_CMAP_SINGLE, 2188 },
	{ 0x8b84, 0x8b84, PDF_CMAP_SINGLE, 22563 },
	{ 0x8b85, 0x8b85, PDF_CMAP_SINGLE, 19822 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5075 },
	{ 0x8b8b, 0x8b8b, PDF_CMAP_SINGLE, 18695 },
	{ 0x8b8c, 0x8b8c, PDF_CMAP_SINGLE, 6728 },
	{ 0x8b8d, 0x8b8d, PDF_CMAP_SINGLE, 22564 },
	{ 0x8b8e, 0x8b8e, PDF_CMAP_SINGLE, 6729 },
	{ 0x8b8f, 0x8b8f, PDF_CMAP_SINGLE, 22565 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 2364 },
	{ 0x8b92, 0x8b93, PDF_CMAP_RANGE, 6730 },
	{ 0x8b94, 0x8b95, PDF_CMAP_RANGE, 18696 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 6732 },
	{ 0x8b99, 0x8b9a, PDF_CMAP_RANGE, 6733 },
	{ 0x8b9c, 0x8b9c, PDF_CMAP_SINGLE, 15148 },
	{ 0x8b9d, 0x8b9d, PDF_CMAP_SINGLE, 17131 },
	{ 0x8b9e, 0x8b9e, PDF_CMAP_SINGLE, 15149 },
	{ 0x8b9f, 0x8b9f, PDF_CMAP_SINGLE, 19823 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 2921 },
	{ 0x8c38, 0x8c38, PDF_CMAP_SINGLE, 22566 },
	{ 0x8c39, 0x8c39, PDF_CMAP_SINGLE, 18698 },
	{ 0x8c3a, 0x8c3a, PDF_CMAP_SINGLE, 6735 },
	{ 0x8c3d, 0x8c3d, PDF_CMAP_SINGLE, 18700 },
	{ 0x8c3e, 0x8c3e, PDF_CMAP_SINGLE, 22567 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 6737 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 6736 },
	{ 0x8c45, 0x8c45, PDF_CMAP_SINGLE, 18703 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 3198 },
	{ 0x8c47, 0x8c47, PDF_CMAP_SINGLE, 15150 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 6738 },
	{ 0x8c49, 0x8c49, PDF_CMAP_SINGLE, 17132 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 3675 },
	{ 0x8c4b, 0x8c4b, PDF_CMAP_SINGLE, 19824 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 6739 },
	{ 0x8c4e, 0x8c4e, PDF_CMAP_SINGLE, 6740 },
	{ 0x8c4f, 0x8c4f, PDF_CMAP_SINGLE, 18704 },
	{ 0x8c50, 0x8c50, PDF_CMAP_SINGLE, 6741 },
	{ 0x8c51, 0x8c51, PDF_CMAP_SINGLE, 22568 },
	{ 0x8c53, 0x8c53, PDF_CMAP_SINGLE, 19825 },
	{ 0x8c54, 0x8c54, PDF_CMAP_SINGLE, 15151 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 6742 },
	{ 0x8c57, 0x8c57, PDF_CMAP_SINGLE, 18705 },
	{ 0x8c58, 0x8c58, PDF_CMAP_SINGLE, 22569 },
	{ 0x8c59, 0x8c59, PDF_CMAP_SINGLE, 22572 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 3250 },
	{ 0x8c5b, 0x8c5b, PDF_CMAP_SINGLE, 22570 },
	{ 0x8c5d, 0x8c5d, PDF_CMAP_SINGLE, 22571 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 2501 },
	{ 0x8c62, 0x8c62, PDF_CMAP_SINGLE, 6743 },
	{ 0x8c63, 0x8c64, PDF_CMAP_RANGE, 22573 },
	{ 0x8c66, 0x8c66, PDF_CMAP_SINGLE, 22575 },
	{ 0x8c68, 0x8c68, PDF_CMAP_SINGLE, 17133 },
	{ 0x8c69, 0x8c69, PDF_CMAP_SINGLE, 18706 },
	{ 0x8c6a, 0x8c6a, PDF_CMAP_SINGLE, 2045 },
	{ 0x8c6b, 0x8c6b, PDF_CMAP_SINGLE, 4103 },
	{ 0x8c6c, 0x8c6c, PDF_CMAP_SINGLE, 6744 },
	{ 0x8c6d, 0x8c6d, PDF_CMAP_SINGLE, 18707 },
	{ 0x8c73, 0x8c73, PDF_CMAP_SINGLE, 15152 },
	{ 0x8c75, 0x8c76, PDF_CMAP_RANGE, 22576 },
	{ 0x8c78, 0x8c78, PDF_CMAP_SINGLE, 6745 },
	{ 0x8c79, 0x8c79, PDF_CMAP_SINGLE, 3505 },
	{ 0x8c7a, 0x8c7a, PDF_CMAP_SINGLE, 6746 },
	{ 0x8c7b, 0x8c7b, PDF_CMAP_SINGLE, 19826 },
	{ 0x8c7c, 0x8c7c, PDF_CMAP_SINGLE, 6754 },
	{ 0x8c7e, 0x8c7e, PDF_CMAP_SINGLE, 22578 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 6747 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 6749 },
	{ 0x8c86, 0x8c87, PDF_CMAP_RANGE, 22579 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 6748 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 6750 },
	{ 0x8c8b, 0x8c8b, PDF_CMAP_SINGLE, 22581 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 3700 },
	{ 0x8c8d, 0x8c8e, PDF_CMAP_RANGE, 6751 },
	{ 0x8c90, 0x8c90, PDF_CMAP_SINGLE, 22582 },
	{ 0x8c92, 0x8c92, PDF_CMAP_SINGLE, 18710 },
	{ 0x8c93, 0x8c93, PDF_CMAP_SINGLE, 18709 },
	{ 0x8c94, 0x8c94, PDF_CMAP_SINGLE, 6753 },
	{ 0x8c98, 0x8c98, PDF_CMAP_SINGLE, 6755 },
	{ 0x8c99, 0x8c99, PDF_CMAP_SINGLE, 18711 },
	{ 0x8c9b, 0x8c9b, PDF_CMAP_SINGLE, 18713 },
	{ 0x8c9c, 0x8c9c, PDF_CMAP_SINGLE, 22584 },
	{ 0x8c9d, 0x8c9d, PDF_CMAP_SINGLE, 1419 },
	{ 0x8c9e, 0x8c9e, PDF_CMAP_SINGLE, 3075 },
	{ 0x8c9f, 0x8c9f, PDF_CMAP_SINGLE, 13644 },
	{ 0x8ca0, 0x8ca0, PDF_CMAP_SINGLE, 3547 },
	{ 0x8ca1, 0x8ca1, PDF_CMAP_SINGLE, 2130 },
	{ 0x8ca2, 0x8ca2, PDF_CMAP_SINGLE, 2025 },
	{ 0x8ca4, 0x8ca4, PDF_CMAP_SINGLE, 15153 },
	{ 0x8ca7, 0x8ca7, PDF_CMAP_SINGLE, 3521 },
	{ 0x8ca8, 0x8ca8, PDF_CMAP_SINGLE, 1375 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 3426 },
	{ 0x8caa, 0x8caa, PDF_CMAP_SINGLE, 6758 },
	{ 0x8cab, 0x8cab, PDF_CMAP_SINGLE, 1551 },
	{ 0x8cac, 0x8cac, PDF_CMAP_SINGLE, 2681 },
	{ 0x8cad, 0x8cad, PDF_CMAP_SINGLE, 6757 },
	{ 0x8cae, 0x8cae, PDF_CMAP_SINGLE, 6762 },
	{ 0x8caf, 0x8caf, PDF_CMAP_SINGLE, 2999 },
	{ 0x8cb0, 0x8cb0, PDF_CMAP_SINGLE, 3823 },
	{ 0x8cb2, 0x8cb3, PDF_CMAP_RANGE, 6760 },
	{ 0x8cb4, 0x8cb4, PDF_CMAP_SINGLE, 1608 },
	{ 0x8cb6, 0x8cb6, PDF_CMAP_SINGLE, 6763 },
	{ 0x8cb7, 0x8cb7, PDF_CMAP_SINGLE, 3353 },
	{ 0x8cb8, 0x8cb8, PDF_CMAP_SINGLE, 2879 },
	{ 0x8cb9, 0x8cb9, PDF_CMAP_SINGLE, 22585 },
	{ 0x8cba, 0x8cba, PDF_CMAP_SINGLE, 19827 },
	{ 0x8cbb, 0x8cbb, PDF_CMAP_SINGLE, 3461 },
	{ 0x8cbc, 0x8cbc, PDF_CMAP_SINGLE, 3127 },
	{ 0x8cbd, 0x8cbd, PDF_CMAP_SINGLE, 6759 },
	{ 0x8cbf, 0x8cbf, PDF_CMAP_SINGLE, 3701 },
	{ 0x8cc0, 0x8cc0, PDF_CMAP_SINGLE, 1388 },
	{ 0x8cc1, 0x8cc1, PDF_CMAP_SINGLE, 6765 },
	{ 0x8cc2, 0x8cc2, PDF_CMAP_SINGLE, 4046 },
	{ 0x8cc3, 0x8cc3, PDF_CMAP_SINGLE, 3038 },
	{ 0x8cc4, 0x8cc4, PDF_CMAP_SINGLE, 4075 },
	{ 0x8cc5, 0x8cc5, PDF_CMAP_SINGLE, 19828 },
	{ 0x8cc6, 0x8cc6, PDF_CMAP_SINGLE, 22586 },
	{ 0x8cc7, 0x8cc7, PDF_CMAP_SINGLE, 2239 },
	{ 0x8cc8, 0x8cc8, PDF_CMAP_SINGLE, 6764 },
	{ 0x8cc9, 0x8cc9, PDF_CMAP_SINGLE, 19829 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 2833 },
	{ 0x8ccb, 0x8ccb, PDF_CMAP_SINGLE, 22587 },
	{ 0x8ccd, 0x8ccd, PDF_CMAP_SINGLE, 6781 },
	{ 0x8cce, 0x8cce, PDF_CMAP_SINGLE, 2730 },
	{ 0x8ccf, 0x8ccf, PDF_CMAP_SINGLE, 22588 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 3280 },
	{ 0x8cd2, 0x8cd2, PDF_CMAP_SINGLE, 19830 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 3522 },
	{ 0x8cd5, 0x8cd5, PDF_CMAP_SINGLE, 18715 },
	{ 0x8cd6, 0x8cd6, PDF_CMAP_SINGLE, 18714 },
	{ 0x8cd9, 0x8cd9, PDF_CMAP_SINGLE, 15154 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 6768 },
	{ 0x8cdb, 0x8cdb, PDF_CMAP_SINGLE, 2189 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 2240 },
	{ 0x8cdd, 0x8cdd, PDF_CMAP_SINGLE, 22589 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 2502 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 3355 },
	{ 0x8ce1, 0x8ce1, PDF_CMAP_SINGLE, 15155 },
	{ 0x8ce2, 0x8ce2, PDF_CMAP_SINGLE, 1889 },
	{ 0x8ce3, 0x8ce3, PDF_CMAP_SINGLE, 6767 },
	{ 0x8ce4, 0x8ce4, PDF_CMAP_SINGLE, 6766 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 3548 },
	{ 0x8ce8, 0x8ce8, PDF_CMAP_SINGLE, 22590 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 2285 },
	{ 0x8cec, 0x8cec, PDF_CMAP_SINGLE, 19831 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 3148 },
	{ 0x8cef, 0x8cef, PDF_CMAP_SINGLE, 22591 },
	{ 0x8cf0, 0x8cf0, PDF_CMAP_SINGLE, 8626 },
	{ 0x8cf1, 0x8cf1, PDF_CMAP_SINGLE, 18717 },
	{ 0x8cf2, 0x8cf2, PDF_CMAP_SINGLE, 22592 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 8627 },
	{ 0x8cf5, 0x8cf5, PDF_CMAP_SINGLE, 19832 },
	{ 0x8cf7, 0x8cf7, PDF_CMAP_SINGLE, 19833 },
	{ 0x8cf8, 0x8cf8, PDF_CMAP_SINGLE, 15156 },
	{ 0x8cfa, 0x8cfb, PDF_CMAP_RANGE, 6770 },
	{ 0x8cfc, 0x8cfc, PDF_CMAP_SINGLE, 2026 },
	{ 0x8cfd, 0x8cfd, PDF_CMAP_SINGLE, 6769 },
	{ 0x8cfe, 0x8cfe, PDF_CMAP_SINGLE, 15157 },
	{ 0x8cff, 0x8cff, PDF_CMAP_SINGLE, 22593 },
	{ 0x8d01, 0x8d01, PDF_CMAP_SINGLE, 19834 },
	{ 0x8d03, 0x8d03, PDF_CMAP_SINGLE, 19835 },
	{ 0x8d04, 0x8d05, PDF_CMAP_RANGE, 6772 },
	{ 0x8d07, 0x8d07, PDF_CMAP_SINGLE, 6775 },
	{ 0x8d08, 0x8d08, PDF_CMAP_SINGLE, 2819 },
	{ 0x8d09, 0x8d09, PDF_CMAP_SINGLE, 18719 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 6774 },
	{ 0x8d0b, 0x8d0b, PDF_CMAP_SINGLE, 1570 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 6777 },
	{ 0x8d0e, 0x8d0e, PDF_CMAP_SINGLE, 18720 },
	{ 0x8d0f, 0x8d0f, PDF_CMAP_SINGLE, 6776 },
	{ 0x8d10, 0x8d10, PDF_CMAP_SINGLE, 6778 },
	{ 0x8d12, 0x8d12, PDF_CMAP_SINGLE, 8628 },
	{ 0x8d13, 0x8d13, PDF_CMAP_SINGLE, 6780 },
	{ 0x8d14, 0x8d14, PDF_CMAP_SINGLE, 6782 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 6783 },
	{ 0x8d17, 0x8d17, PDF_CMAP_SINGLE, 19836 },
	{ 0x8d1b, 0x8d1b, PDF_CMAP_SINGLE, 15158 },
	{ 0x8d1c, 0x8d1c, PDF_CMAP_SINGLE, 19837 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 2682 },
	{ 0x8d65, 0x8d65, PDF_CMAP_SINGLE, 22594 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 2299 },
	{ 0x8d67, 0x8d67, PDF_CMAP_SINGLE, 6784 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 15159 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 1456 },
	{ 0x8d6c, 0x8d6c, PDF_CMAP_SINGLE, 15160 },
	{ 0x8d6d, 0x8d6d, PDF_CMAP_SINGLE, 6785 },
	{ 0x8d6e, 0x8d6e, PDF_CMAP_SINGLE, 19838 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 2808 },
	{ 0x8d71, 0x8d71, PDF_CMAP_SINGLE, 6786 },
	{ 0x8d73, 0x8d73, PDF_CMAP_SINGLE, 6787 },
	{ 0x8d74, 0x8d74, PDF_CMAP_SINGLE, 3549 },
	{ 0x8d76, 0x8d76, PDF_CMAP_SINGLE, 8629 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 1609 },
	{ 0x8d7f, 0x8d7f, PDF_CMAP_SINGLE, 22595 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 6788 },
	{ 0x8d82, 0x8d82, PDF_CMAP_SINGLE, 22596 },
	{ 0x8d84, 0x8d84, PDF_CMAP_SINGLE, 15161 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 3026 },
	{ 0x8d88, 0x8d88, PDF_CMAP_SINGLE, 22597 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 1277 },
	{ 0x8d8d, 0x8d8d, PDF_CMAP_SINGLE, 15162 },
	{ 0x8d90, 0x8d90, PDF_CMAP_SINGLE, 22598 },
	{ 0x8d91, 0x8d91, PDF_CMAP_SINGLE, 19839 },
	{ 0x8d95, 0x8d95, PDF_CMAP_SINGLE, 15163 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 6789 },
	{ 0x8d9e, 0x8d9e, PDF_CMAP_SINGLE, 22599 },
	{ 0x8d9f, 0x8d9f, PDF_CMAP_SINGLE, 19840 },
	{ 0x8da0, 0x8da0, PDF_CMAP_SINGLE, 22600 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 2333 },
	{ 0x8da6, 0x8da6, PDF_CMAP_SINGLE, 15164 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 2620 },
	{ 0x8dab, 0x8dab, PDF_CMAP_SINGLE, 19841 },
	{ 0x8dac, 0x8dac, PDF_CMAP_SINGLE, 22601 },
	{ 0x8daf, 0x8daf, PDF_CMAP_SINGLE, 17134 },
	{ 0x8db2, 0x8db2, PDF_CMAP_SINGLE, 19842 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 2829 },
	{ 0x8db5, 0x8db5, PDF_CMAP_SINGLE, 22602 },
	{ 0x8db7, 0x8db7, PDF_CMAP_SINGLE, 22603 },
	{ 0x8db9, 0x8db9, PDF_CMAP_SINGLE, 22604 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 6792 },
	{ 0x8dbb, 0x8dbb, PDF_CMAP_SINGLE, 22605 },
	{ 0x8dbc, 0x8dbc, PDF_CMAP_SINGLE, 22613 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 6791 },
	{ 0x8dc0, 0x8dc0, PDF_CMAP_SINGLE, 22606 },
	{ 0x8dc2, 0x8dc2, PDF_CMAP_SINGLE, 6790 },
	{ 0x8dc5, 0x8dc5, PDF_CMAP_SINGLE, 22607 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 15165 },
	{ 0x8dc7, 0x8dc7, PDF_CMAP_SINGLE, 22608 },
	{ 0x8dc8, 0x8dc8, PDF_CMAP_SINGLE, 18721 },
	{ 0x8dca, 0x8dca, PDF_CMAP_SINGLE, 22609 },
	{ 0x8dcb, 0x8dcb, PDF_CMAP_SINGLE, 6798 },
	{ 0x8dcc, 0x8dcc, PDF_CMAP_SINGLE, 6796 },
	{ 0x8dce, 0x8dce, PDF_CMAP_SINGLE, 15166 },
	{ 0x8dcf, 0x8dcf, PDF_CMAP_SINGLE, 6793 },
	{ 0x8dd1, 0x8dd1, PDF_CMAP_SINGLE, 17135 },
	{ 0x8dd4, 0x8dd4, PDF_CMAP_SINGLE, 22610 },
	{ 0x8dd5, 0x8dd5, PDF_CMAP_SINGLE, 19843 },
	{ 0x8dd6, 0x8dd6, PDF_CMAP_SINGLE, 6795 },
	{ 0x8dd7, 0x8dd7, PDF_CMAP_SINGLE, 17136 },
	{ 0x8dd9, 0x8dd9, PDF_CMAP_SINGLE, 18722 },
	{ 0x8dda, 0x8dda, PDF_CMAP_SINGLE, 6794 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 6797 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 1681 },
	{ 0x8ddf, 0x8ddf, PDF_CMAP_SINGLE, 6801 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 2683 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 6802 },
	{ 0x8de4, 0x8de4, PDF_CMAP_SINGLE, 15167 },
	{ 0x8de5, 0x8de5, PDF_CMAP_SINGLE, 22611 },
	{ 0x8de7, 0x8de7, PDF_CMAP_SINGLE, 19844 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 1933 },
	{ 0x8dea, 0x8deb, PDF_CMAP_RANGE, 6799 },
	{ 0x8dec, 0x8dec, PDF_CMAP_SINGLE, 15168 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 4047 },
	{ 0x8df0, 0x8df0, PDF_CMAP_SINGLE, 22612 },
	{ 0x8df1, 0x8df2, PDF_CMAP_RANGE, 19845 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 3027 },
	{ 0x8df4, 0x8df4, PDF_CMAP_SINGLE, 19847 },
	{ 0x8df5, 0x8df5, PDF_CMAP_SINGLE, 2731 },
	{ 0x8dfc, 0x8dfc, PDF_CMAP_SINGLE, 6803 },
	{ 0x8dfd, 0x8dfd, PDF_CMAP_SINGLE, 18725 },
	{ 0x8dff, 0x8dff, PDF_CMAP_SINGLE, 6806 },
	{ 0x8e01, 0x8e01, PDF_CMAP_SINGLE, 19848 },
	{ 0x8e04, 0x8e05, PDF_CMAP_RANGE, 22614 },
	{ 0x8e06, 0x8e06, PDF_CMAP_SINGLE, 18726 },
	{ 0x8e08, 0x8e09, PDF_CMAP_RANGE, 6804 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 3907 },
	{ 0x8e0b, 0x8e0b, PDF_CMAP_SINGLE, 19849 },
	{ 0x8e0c, 0x8e0c, PDF_CMAP_SINGLE, 18723 },
	{ 0x8e0f, 0x8e0f, PDF_CMAP_SINGLE, 3199 },
	{ 0x8e10, 0x8e10, PDF_CMAP_SINGLE, 6809 },
	{ 0x8e11, 0x8e11, PDF_CMAP_SINGLE, 22616 },
	{ 0x8e14, 0x8e14, PDF_CMAP_SINGLE, 18728 },
	{ 0x8e16, 0x8e16, PDF_CMAP_SINGLE, 18729 },
	{ 0x8e1d, 0x8e1e, PDF_CMAP_RANGE, 6807 },
	{ 0x8e1f, 0x8e1f, PDF_CMAP_SINGLE, 6810 },
	{ 0x8e20, 0x8e20, PDF_CMAP_SINGLE, 15169 },
	{ 0x8e21, 0x8e22, PDF_CMAP_RANGE, 18730 },
	{ 0x8e23, 0x8e23, PDF_CMAP_SINGLE, 17137 },
	{ 0x8e26, 0x8e26, PDF_CMAP_SINGLE, 19850 },
	{ 0x8e27, 0x8e27, PDF_CMAP_SINGLE, 18732 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 6824 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 6813 },
	{ 0x8e31, 0x8e31, PDF_CMAP_SINGLE, 19851 },
	{ 0x8e33, 0x8e33, PDF_CMAP_SINGLE, 22617 },
	{ 0x8e34, 0x8e34, PDF_CMAP_SINGLE, 6814 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 6812 },
	{ 0x8e36, 0x8e36, PDF_CMAP_SINGLE, 18735 },
	{ 0x8e37, 0x8e38, PDF_CMAP_RANGE, 22618 },
	{ 0x8e39, 0x8e39, PDF_CMAP_SINGLE, 18736 },
	{ 0x8e3d, 0x8e3d, PDF_CMAP_SINGLE, 17138 },
	{ 0x8e40, 0x8e41, PDF_CMAP_RANGE, 19852 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 6811 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 3097 },
	{ 0x8e47, 0x8e47, PDF_CMAP_SINGLE, 6816 },
	{ 0x8e48, 0x8e48, PDF_CMAP_SINGLE, 6820 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 6817 },
	{ 0x8e4a, 0x8e4a, PDF_CMAP_SINGLE, 6815 },
	{ 0x8e4b, 0x8e4b, PDF_CMAP_SINGLE, 15170 },
	{ 0x8e4c, 0x8e4c, PDF_CMAP_SINGLE, 6818 },
	{ 0x8e4d, 0x8e4d, PDF_CMAP_SINGLE, 19854 },
	{ 0x8e4e, 0x8e4e, PDF_CMAP_SINGLE, 22620 },
	{ 0x8e4f, 0x8e4f, PDF_CMAP_SINGLE, 19855 },
	{ 0x8e50, 0x8e50, PDF_CMAP_SINGLE, 6819 },
	{ 0x8e54, 0x8e54, PDF_CMAP_SINGLE, 18737 },
	{ 0x8e55, 0x8e55, PDF_CMAP_SINGLE, 6826 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 6821 },
	{ 0x8e5b, 0x8e5b, PDF_CMAP_SINGLE, 22621 },
	{ 0x8e5c, 0x8e5c, PDF_CMAP_SINGLE, 19856 },
	{ 0x8e5d, 0x8e5e, PDF_CMAP_RANGE, 22622 },
	{ 0x8e5f, 0x8e5f, PDF_CMAP_SINGLE, 2684 },
	{ 0x8e60, 0x8e60, PDF_CMAP_SINGLE, 6823 },
	{ 0x8e61, 0x8e61, PDF_CMAP_SINGLE, 19857 },
	{ 0x8e62, 0x8e62, PDF_CMAP_SINGLE, 18738 },
	{ 0x8e63, 0x8e63, PDF_CMAP_SINGLE, 6825 },
	{ 0x8e64, 0x8e64, PDF_CMAP_SINGLE, 6822 },
	{ 0x8e69, 0x8e69, PDF_CMAP_SINGLE, 19858 },
	{ 0x8e6c, 0x8e6c, PDF_CMAP_SINGLE, 15171 },
	{ 0x8e6d, 0x8e6d, PDF_CMAP_SINGLE, 18739 },
	{ 0x8e6f, 0x8e6f, PDF_CMAP_SINGLE, 18740 },
	{ 0x8e70, 0x8e70, PDF_CMAP_SINGLE, 15172 },
	{ 0x8e71, 0x8e71, PDF_CMAP_SINGLE, 19859 },
	{ 0x8e72, 0x8e72, PDF_CMAP_SINGLE, 6828 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 2365 },
	{ 0x8e75, 0x8e75, PDF_CMAP_SINGLE, 19860 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 6827 },
	{ 0x8e77, 0x8e77, PDF_CMAP_SINGLE, 19861 },
	{ 0x8e79, 0x8e79, PDF_CMAP_SINGLE, 22624 },
	{ 0x8e7a, 0x8e7a, PDF_CMAP_SINGLE, 15173 },
	{ 0x8e7b, 0x8e7b, PDF_CMAP_SINGLE, 17139 },
	{ 0x8e7c, 0x8e7c, PDF_CMAP_SINGLE, 6829 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 6830 },
	{ 0x8e82, 0x8e83, PDF_CMAP_RANGE, 22625 },
	{ 0x8e84, 0x8e84, PDF_CMAP_SINGLE, 6833 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 6832 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 6831 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 19862 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 6835 },
	{ 0x8e8b, 0x8e8b, PDF_CMAP_SINGLE, 6834 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 3842 },
	{ 0x8e90, 0x8e90, PDF_CMAP_SINGLE, 19863 },
	{ 0x8e91, 0x8e91, PDF_CMAP_SINGLE, 6837 },
	{ 0x8e92, 0x8e92, PDF_CMAP_SINGLE, 15174 },
	{ 0x8e93, 0x8e93, PDF_CMAP_SINGLE, 6836 },
	{ 0x8e94, 0x8e94, PDF_CMAP_SINGLE, 6838 },
	{ 0x8e95, 0x8e95, PDF_CMAP_SINGLE, 19864 },
	{ 0x8e98, 0x8e98, PDF_CMAP_SINGLE, 18741 },
	{ 0x8e99, 0x8e99, PDF_CMAP_SINGLE, 6839 },
	{ 0x8e9a, 0x8e9a, PDF_CMAP_SINGLE, 19865 },
	{ 0x8e9b, 0x8e9b, PDF_CMAP_SINGLE, 22627 },
	{ 0x8e9d, 0x8e9d, PDF_CMAP_SINGLE, 22628 },
	{ 0x8e9e, 0x8e9e, PDF_CMAP_SINGLE, 18742 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 6841 },
	{ 0x8ea2, 0x8ea2, PDF_CMAP_SINGLE, 22629 },
	{ 0x8ea7, 0x8ea7, PDF_CMAP_SINGLE, 19866 },
	{ 0x8ea9, 0x8ea9, PDF_CMAP_SINGLE, 19867 },
	{ 0x8eaa, 0x8eaa, PDF_CMAP_SINGLE, 6840 },
	{ 0x8eab, 0x8eab, PDF_CMAP_SINGLE, 2574 },
	{ 0x8eac, 0x8eac, PDF_CMAP_SINGLE, 6842 },
	{ 0x8ead, 0x8ead, PDF_CMAP_SINGLE, 19868 },
	{ 0x8eae, 0x8eae, PDF_CMAP_SINGLE, 15175 },
	{ 0x8eaf, 0x8eaf, PDF_CMAP_SINGLE, 1765 },
	{ 0x8eb0, 0x8eb0, PDF_CMAP_SINGLE, 6843 },
	{ 0x8eb1, 0x8eb1, PDF_CMAP_SINGLE, 6845 },
	{ 0x8eb3, 0x8eb3, PDF_CMAP_SINGLE, 15176 },
	{ 0x8eb5, 0x8eb5, PDF_CMAP_SINGLE, 18743 },
	{ 0x8eb6, 0x8eb6, PDF_CMAP_SINGLE, 14083 },
	{ 0x8eba, 0x8eba, PDF_CMAP_SINGLE, 22630 },
	{ 0x8ebb, 0x8ebb, PDF_CMAP_SINGLE, 18744 },
	{ 0x8ebe, 0x8ebe, PDF_CMAP_SINGLE, 6846 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 7663 },
	{ 0x8ec1, 0x8ec1, PDF_CMAP_SINGLE, 22631 },
	{ 0x8ec3, 0x8ec4, PDF_CMAP_RANGE, 22632 },
	{ 0x8ec5, 0x8ec5, PDF_CMAP_SINGLE, 6847 },
	{ 0x8ec6, 0x8ec6, PDF_CMAP_SINGLE, 6844 },
	{ 0x8ec7, 0x8ec7, PDF_CMAP_SINGLE, 22634 },
	{ 0x8ec8, 0x8ec8, PDF_CMAP_SINGLE, 6848 },
	{ 0x8eca, 0x8eca, PDF_CMAP_SINGLE, 2306 },
	{ 0x8ecb, 0x8ecb, PDF_CMAP_SINGLE, 6849 },
	{ 0x8ecc, 0x8ecc, PDF_CMAP_SINGLE, 1610 },
	{ 0x8ecd, 0x8ecd, PDF_CMAP_SINGLE, 1801 },
	{ 0x8ecf, 0x8ecf, PDF_CMAP_SINGLE, 8631 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 15177 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 1890 },
	{ 0x8ed4, 0x8ed4, PDF_CMAP_SINGLE, 15178 },
	{ 0x8edb, 0x8edb, PDF_CMAP_SINGLE, 6850 },
	{ 0x8edc, 0x8edc, PDF_CMAP_SINGLE, 22635 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 3272 },
	{ 0x8ee2, 0x8ee2, PDF_CMAP_SINGLE, 3128 },
	{ 0x8ee3, 0x8ee3, PDF_CMAP_SINGLE, 6851 },
	{ 0x8ee8, 0x8ee8, PDF_CMAP_SINGLE, 19869 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 6854 },
	{ 0x8eed, 0x8eed, PDF_CMAP_SINGLE, 22639 },
	{ 0x8eee, 0x8eee, PDF_CMAP_SINGLE, 22636 },
	{ 0x8ef0, 0x8ef0, PDF_CMAP_SINGLE, 19870 },
	{ 0x8ef1, 0x8ef1, PDF_CMAP_SINGLE, 22637 },
	{ 0x8ef7, 0x8ef7, PDF_CMAP_SINGLE, 22638 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 2272 },
	{ 0x8ef9, 0x8ef9, PDF_CMAP_SINGLE, 15179 },
	{ 0x8efa, 0x8efa, PDF_CMAP_SINGLE, 17142 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 6853 },
	{ 0x8efc, 0x8efc, PDF_CMAP_SINGLE, 6852 },
	{ 0x8efd, 0x8efd, PDF_CMAP_SINGLE, 1840 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 6855 },
	{ 0x8f00, 0x8f00, PDF_CMAP_SINGLE, 18748 },
	{ 0x8f02, 0x8f02, PDF_CMAP_SINGLE, 22640 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 1457 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 6857 },
	{ 0x8f07, 0x8f07, PDF_CMAP_SINGLE, 19871 },
	{ 0x8f08, 0x8f08, PDF_CMAP_SINGLE, 18749 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 2124 },
	{ 0x8f0a, 0x8f0a, PDF_CMAP_SINGLE, 6856 },
	{ 0x8f0c, 0x8f0c, PDF_CMAP_SINGLE, 6865 },
	{ 0x8f0f, 0x8f10, PDF_CMAP_RANGE, 22641 },
	{ 0x8f12, 0x8f12, PDF_CMAP_SINGLE, 6859 },
	{ 0x8f13, 0x8f13, PDF_CMAP_SINGLE, 6861 },
	{ 0x8f14, 0x8f14, PDF_CMAP_SINGLE, 3637 },
	{ 0x8f15, 0x8f15, PDF_CMAP_SINGLE, 6858 },
	{ 0x8f16, 0x8f16, PDF_CMAP_SINGLE, 22643 },
	{ 0x8f17, 0x8f17, PDF_CMAP_SINGLE, 15180 },
	{ 0x8f18, 0x8f18, PDF_CMAP_SINGLE, 19872 },
	{ 0x8f19, 0x8f19, PDF_CMAP_SINGLE, 6860 },
	{ 0x8f1b, 0x8f1b, PDF_CMAP_SINGLE, 6864 },
	{ 0x8f1c, 0x8f1c, PDF_CMAP_SINGLE, 6862 },
	{ 0x8f1d, 0x8f1d, PDF_CMAP_SINGLE, 1611 },
	{ 0x8f1e, 0x8f1e, PDF_CMAP_SINGLE, 17143 },
	{ 0x8f1f, 0x8f1f, PDF_CMAP_SINGLE, 6863 },
	{ 0x8f20, 0x8f21, PDF_CMAP_RANGE, 22644 },
	{ 0x8f23, 0x8f23, PDF_CMAP_SINGLE, 22646 },
	{ 0x8f25, 0x8f25, PDF_CMAP_SINGLE, 19873 },
	{ 0x8f26, 0x8f26, PDF_CMAP_SINGLE, 6866 },
	{ 0x8f27, 0x8f27, PDF_CMAP_SINGLE, 19874 },
	{ 0x8f28, 0x8f28, PDF_CMAP_SINGLE, 22647 },
	{ 0x8f29, 0x8f29, PDF_CMAP_SINGLE, 3344 },
	{ 0x8f2a, 0x8f2a, PDF_CMAP_SINGLE, 4000 },
	{ 0x8f2b, 0x8f2b, PDF_CMAP_SINGLE, 18750 },
	{ 0x8f2c, 0x8f2c, PDF_CMAP_SINGLE, 19875 },
	{ 0x8f2d, 0x8f2d, PDF_CMAP_SINGLE, 17144 },
	{ 0x8f2e, 0x8f2e, PDF_CMAP_SINGLE, 22648 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 2366 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 6867 },
	{ 0x8f34, 0x8f34, PDF_CMAP_SINGLE, 22649 },
	{ 0x8f35, 0x8f35, PDF_CMAP_SINGLE, 19876 },
	{ 0x8f36, 0x8f36, PDF_CMAP_SINGLE, 15181 },
	{ 0x8f37, 0x8f37, PDF_CMAP_SINGLE, 22650 },
	{ 0x8f38, 0x8f38, PDF_CMAP_SINGLE, 3852 },
	{ 0x8f39, 0x8f39, PDF_CMAP_SINGLE, 6869 },
	{ 0x8f3a, 0x8f3a, PDF_CMAP_SINGLE, 19877 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 6868 },
	{ 0x8f3e, 0x8f3e, PDF_CMAP_SINGLE, 6872 },
	{ 0x8f3f, 0x8f3f, PDF_CMAP_SINGLE, 3883 },
	{ 0x8f40, 0x8f40, PDF_CMAP_SINGLE, 18751 },
	{ 0x8f41, 0x8f41, PDF_CMAP_SINGLE, 22651 },
	{ 0x8f42, 0x8f42, PDF_CMAP_SINGLE, 6871 },
	{ 0x8f43, 0x8f43, PDF_CMAP_SINGLE, 19878 },
	{ 0x8f44, 0x8f44, PDF_CMAP_SINGLE, 1483 },
	{ 0x8f45, 0x8f45, PDF_CMAP_SINGLE, 6870 },
	{ 0x8f46, 0x8f46, PDF_CMAP_SINGLE, 6875 },
	{ 0x8f47, 0x8f47, PDF_CMAP_SINGLE, 19879 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 6874 },
	{ 0x8f4a, 0x8f4a, PDF_CMAP_SINGLE, 18752 },
	{ 0x8f4c, 0x8f4c, PDF_CMAP_SINGLE, 6873 },
	{ 0x8f4d, 0x8f4d, PDF_CMAP_SINGLE, 3116 },
	{ 0x8f4e, 0x8f4e, PDF_CMAP_SINGLE, 6876 },
	{ 0x8f4f, 0x8f4f, PDF_CMAP_SINGLE, 22652 },
	{ 0x8f51, 0x8f51, PDF_CMAP_SINGLE, 19880 },
	{ 0x8f52, 0x8f53, PDF_CMAP_RANGE, 22653 },
	{ 0x8f54, 0x8f54, PDF_CMAP_SINGLE, 17145 },
	{ 0x8f55, 0x8f55, PDF_CMAP_SINGLE, 19881 },
	{ 0x8f57, 0x8f57, PDF_CMAP_SINGLE, 6877 },
	{ 0x8f58, 0x8f58, PDF_CMAP_SINGLE, 18753 },
	{ 0x8f5c, 0x8f5c, PDF_CMAP_SINGLE, 6878 },
	{ 0x8f5d, 0x8f5e, PDF_CMAP_RANGE, 22655 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 2046 },
	{ 0x8f61, 0x8f61, PDF_CMAP_SINGLE, 1787 },
	{ 0x8f62, 0x8f64, PDF_CMAP_RANGE, 6879 },
	{ 0x8f65, 0x8f65, PDF_CMAP_SINGLE, 22657 },
	{ 0x8f9b, 0x8f9b, PDF_CMAP_SINGLE, 2575 },
	{ 0x8f9c, 0x8f9c, PDF_CMAP_SINGLE, 6882 },
	{ 0x8f9d, 0x8f9d, PDF_CMAP_SINGLE, 22658 },
	{ 0x8f9e, 0x8f9e, PDF_CMAP_SINGLE, 2265 },
	{ 0x8f9f, 0x8f9f, PDF_CMAP_SINGLE, 6883 },
	{ 0x8fa0, 0x8fa2, PDF_CMAP_RANGE, 19882 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 6884 },
	{ 0x8fa4, 0x8fa4, PDF_CMAP_SINGLE, 18755 },
	{ 0x8fa5, 0x8fa5, PDF_CMAP_SINGLE, 19885 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 15182 },
	{ 0x8fa7, 0x8fa7, PDF_CMAP_SINGLE, 4278 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 4277 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 6885 },
	{ 0x8fae, 0x8fae, PDF_CMAP_SINGLE, 6143 },
	{ 0x8faf, 0x8faf, PDF_CMAP_SINGLE, 6886 },
	{ 0x8fb0, 0x8fb0, PDF_CMAP_SINGLE, 2914 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 2545 },
	{ 0x8fb2, 0x8fb2, PDF_CMAP_SINGLE, 3318 },
	{ 0x8fb4, 0x8fb4, PDF_CMAP_SINGLE, 18756 },
	{ 0x8fb5, 0x8fb6, PDF_CMAP_RANGE, 15183 },
	{ 0x8fb7, 0x8fb7, PDF_CMAP_SINGLE, 6887 },
	{ 0x8fb8, 0x8fb8, PDF_CMAP_SINGLE, 22659 },
	{ 0x8fba, 0x8fba, PDF_CMAP_SINGLE, 3621 },
	{ 0x8fbb, 0x8fbb, PDF_CMAP_SINGLE, 3056 },
	{ 0x8fbc, 0x8fbc, PDF_CMAP_SINGLE, 2064 },
	{ 0x8fbe, 0x8fbe, PDF_CMAP_SINGLE, 22660 },
	{ 0x8fbf, 0x8fbf, PDF_CMAP_SINGLE, 2919 },
	{ 0x8fc0, 0x8fc0, PDF_CMAP_SINGLE, 22661 },
	{ 0x8fc1, 0x8fc1, PDF_CMAP_SINGLE, 18758 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 1228 },
	{ 0x8fc4, 0x8fc4, PDF_CMAP_SINGLE, 3750 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 2589 },
	{ 0x8fc6, 0x8fc6, PDF_CMAP_SINGLE, 18759 },
	{ 0x8fc8, 0x8fc8, PDF_CMAP_SINGLE, 14233 },
	{ 0x8fca, 0x8fca, PDF_CMAP_SINGLE, 18761 },
	{ 0x8fcb, 0x8fcb, PDF_CMAP_SINGLE, 22662 },
	{ 0x8fcd, 0x8fcd, PDF_CMAP_SINGLE, 18762 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 1844 },
	{ 0x8fd0, 0x8fd0, PDF_CMAP_SINGLE, 22663 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 1753 },
	{ 0x8fd2, 0x8fd2, PDF_CMAP_SINGLE, 22664 },
	{ 0x8fd3, 0x8fd3, PDF_CMAP_SINGLE, 18763 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 3622 },
	{ 0x8fd5, 0x8fd5, PDF_CMAP_SINGLE, 18764 },
	{ 0x8fda, 0x8fda, PDF_CMAP_SINGLE, 6888 },
	{ 0x8fe0, 0x8fe0, PDF_CMAP_SINGLE, 15186 },
	{ 0x8fe2, 0x8fe2, PDF_CMAP_SINGLE, 6890 },
	{ 0x8fe3, 0x8fe3, PDF_CMAP_SINGLE, 22665 },
	{ 0x8fe4, 0x8fe4, PDF_CMAP_SINGLE, 15187 },
	{ 0x8fe5, 0x8fe5, PDF_CMAP_SINGLE, 6889 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 1376 },
	{ 0x8fe8, 0x8fe8, PDF_CMAP_SINGLE, 17147 },
	{ 0x8fe9, 0x8fe9, PDF_CMAP_SINGLE, 3278 },
	{ 0x8fea, 0x8fea, PDF_CMAP_SINGLE, 6891 },
	{ 0x8feb, 0x8feb, PDF_CMAP_SINGLE, 3373 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 3117 },
	{ 0x8fee, 0x8fee, PDF_CMAP_SINGLE, 17148 },
	{ 0x8fef, 0x8fef, PDF_CMAP_SINGLE, 6892 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 2396 },
	{ 0x8ff1, 0x8ff1, PDF_CMAP_SINGLE, 18765 },
	{ 0x8ff4, 0x8ff4, PDF_CMAP_SINGLE, 6894 },
	{ 0x8ff5, 0x8ff5, PDF_CMAP_SINGLE, 18766 },
	{ 0x8ff6, 0x8ff6, PDF_CMAP_SINGLE, 15188 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 3790 },
	{ 0x8ff8, 0x8ff8, PDF_CMAP_SINGLE, 6909 },
	{ 0x8ff9, 0x8ffa, PDF_CMAP_RANGE, 6896 },
	{ 0x8ffb, 0x8ffb, PDF_CMAP_SINGLE, 18767 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 3045 },
	{ 0x8ffe, 0x8ffe, PDF_CMAP_SINGLE, 22666 },
	{ 0x9000, 0x9000, PDF_CMAP_SINGLE, 2880 },
	{ 0x9001, 0x9001, PDF_CMAP_SINGLE, 2809 },
	{ 0x9002, 0x9002, PDF_CMAP_SINGLE, 15189 },
	{ 0x9003, 0x9003, PDF_CMAP_SINGLE, 3200 },
	{ 0x9004, 0x9004, PDF_CMAP_SINGLE, 19886 },
	{ 0x9005, 0x9005, PDF_CMAP_SINGLE, 6895 },
	{ 0x9006, 0x9006, PDF_CMAP_SINGLE, 1647 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 17149 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 6904 },
	{ 0x900c, 0x900c, PDF_CMAP_SINGLE, 18768 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 6901 },
	{ 0x900e, 0x900e, PDF_CMAP_SINGLE, 6914 },
	{ 0x900f, 0x900f, PDF_CMAP_SINGLE, 3201 },
	{ 0x9010, 0x9010, PDF_CMAP_SINGLE, 2974 },
	{ 0x9011, 0x9011, PDF_CMAP_SINGLE, 6898 },
	{ 0x9013, 0x9013, PDF_CMAP_SINGLE, 3098 },
	{ 0x9014, 0x9014, PDF_CMAP_SINGLE, 3149 },
	{ 0x9015, 0x9015, PDF_CMAP_SINGLE, 6899 },
	{ 0x9016, 0x9016, PDF_CMAP_SINGLE, 6903 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 2598 },
	{ 0x9018, 0x9018, PDF_CMAP_SINGLE, 22667 },
	{ 0x9019, 0x9019, PDF_CMAP_SINGLE, 3357 },
	{ 0x901a, 0x901a, PDF_CMAP_SINGLE, 3048 },
	{ 0x901b, 0x901b, PDF_CMAP_SINGLE, 19887 },
	{ 0x901d, 0x901d, PDF_CMAP_SINGLE, 2662 },
	{ 0x901e, 0x901e, PDF_CMAP_SINGLE, 6902 },
	{ 0x901f, 0x901f, PDF_CMAP_SINGLE, 2830 },
	{ 0x9020, 0x9020, PDF_CMAP_SINGLE, 2820 },
	{ 0x9021, 0x9021, PDF_CMAP_SINGLE, 6900 },
	{ 0x9022, 0x9022, PDF_CMAP_SINGLE, 1133 },
	{ 0x9023, 0x9023, PDF_CMAP_SINGLE, 4040 },
	{ 0x9027, 0x9027, PDF_CMAP_SINGLE, 6905 },
	{ 0x9028, 0x902a, PDF_CMAP_RANGE, 22668 },
	{ 0x902c, 0x902c, PDF_CMAP_SINGLE, 15190 },
	{ 0x902d, 0x902d, PDF_CMAP_SINGLE, 17150 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 2881 },
	{ 0x902f, 0x902f, PDF_CMAP_SINGLE, 19888 },
	{ 0x9031, 0x9031, PDF_CMAP_SINGLE, 2367 },
	{ 0x9032, 0x9032, PDF_CMAP_SINGLE, 2576 },
	{ 0x9033, 0x9034, PDF_CMAP_RANGE, 22671 },
	{ 0x9035, 0x9035, PDF_CMAP_SINGLE, 6907 },
	{ 0x9036, 0x9036, PDF_CMAP_SINGLE, 6906 },
	{ 0x9037, 0x9037, PDF_CMAP_SINGLE, 18769 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 1203 },
	{ 0x9039, 0x9039, PDF_CMAP_SINGLE, 6908 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 3489 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 6916 },
	{ 0x903f, 0x903f, PDF_CMAP_SINGLE, 22673 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 3251 },
	{ 0x9042, 0x9042, PDF_CMAP_SINGLE, 2609 },
	{ 0x9043, 0x9043, PDF_CMAP_SINGLE, 18771 },
	{ 0x9044, 0x9044, PDF_CMAP_SINGLE, 15191 },
	{ 0x9045, 0x9045, PDF_CMAP_SINGLE, 2967 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 1776 },
	{ 0x9049, 0x9049, PDF_CMAP_SINGLE, 6915 },
	{ 0x904a, 0x904a, PDF_CMAP_SINGLE, 3873 },
	{ 0x904b, 0x904b, PDF_CMAP_SINGLE, 1249 },
	{ 0x904c, 0x904c, PDF_CMAP_SINGLE, 19889 },
	{ 0x904d, 0x904d, PDF_CMAP_SINGLE, 3623 },
	{ 0x904e, 0x904e, PDF_CMAP_SINGLE, 1377 },
	{ 0x904f, 0x9052, PDF_CMAP_RANGE, 6910 },
	{ 0x9053, 0x9053, PDF_CMAP_SINGLE, 3219 },
	{ 0x9054, 0x9054, PDF_CMAP_SINGLE, 2913 },
	{ 0x9055, 0x9055, PDF_CMAP_SINGLE, 1191 },
	{ 0x9056, 0x9056, PDF_CMAP_SINGLE, 6917 },
	{ 0x9058, 0x9058, PDF_CMAP_SINGLE, 6918 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 7476 },
	{ 0x905b, 0x905b, PDF_CMAP_SINGLE, 19890 },
	{ 0x905c, 0x905c, PDF_CMAP_SINGLE, 2845 },
	{ 0x905d, 0x905d, PDF_CMAP_SINGLE, 18772 },
	{ 0x905e, 0x905e, PDF_CMAP_SINGLE, 6919 },
	{ 0x9060, 0x9060, PDF_CMAP_SINGLE, 1301 },
	{ 0x9061, 0x9061, PDF_CMAP_SINGLE, 2766 },
	{ 0x9062, 0x9062, PDF_CMAP_SINGLE, 22674 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 1891 },
	{ 0x9065, 0x9065, PDF_CMAP_SINGLE, 3908 },
	{ 0x9066, 0x9066, PDF_CMAP_SINGLE, 22675 },
	{ 0x9067, 0x9067, PDF_CMAP_SINGLE, 8634 },
	{ 0x9068, 0x9068, PDF_CMAP_SINGLE, 6920 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 3110 },
	{ 0x906c, 0x906c, PDF_CMAP_SINGLE, 22676 },
	{ 0x906d, 0x906d, PDF_CMAP_SINGLE, 2810 },
	{ 0x906e, 0x906e, PDF_CMAP_SINGLE, 2307 },
	{ 0x906f, 0x906f, PDF_CMAP_SINGLE, 6921 },
	{ 0x9070, 0x9070, PDF_CMAP_SINGLE, 19891 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 6924 },
	{ 0x9074, 0x9074, PDF_CMAP_SINGLE, 19892 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 2415 },
	{ 0x9076, 0x9076, PDF_CMAP_SINGLE, 6922 },
	{ 0x9077, 0x9077, PDF_CMAP_SINGLE, 2733 },
	{ 0x9078, 0x9078, PDF_CMAP_SINGLE, 2732 },
	{ 0x9079, 0x9079, PDF_CMAP_SINGLE, 19893 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 1192 },
	{ 0x907c, 0x907c, PDF_CMAP_SINGLE, 3987 },
	{ 0x907d, 0x907d, PDF_CMAP_SINGLE, 6926 },
	{ 0x907f, 0x907f, PDF_CMAP_SINGLE, 3462 },
	{ 0x9080, 0x9080, PDF_CMAP_SINGLE, 6928 },
	{ 0x9081, 0x9081, PDF_CMAP_SINGLE, 6927 },
	{ 0x9082, 0x9082, PDF_CMAP_SINGLE, 6925 },
	{ 0x9083, 0x9083, PDF_CMAP_SINGLE, 5943 },
	{ 0x9084, 0x9084, PDF_CMAP_SINGLE, 1552 },
	{ 0x9085, 0x9085, PDF_CMAP_SINGLE, 18775 },
	{ 0x9087, 0x9087, PDF_CMAP_SINGLE, 6893 },
	{ 0x9088, 0x9088, PDF_CMAP_SINGLE, 15192 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 6930 },
	{ 0x908a, 0x908a, PDF_CMAP_SINGLE, 6929 },
	{ 0x908b, 0x908b, PDF_CMAP_SINGLE, 19894 },
	{ 0x908c, 0x908c, PDF_CMAP_SINGLE, 18776 },
	{ 0x908e, 0x908e, PDF_CMAP_SINGLE, 22677 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 6931 },
	{ 0x9090, 0x9090, PDF_CMAP_SINGLE, 18777 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 3874 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 15193 },
	{ 0x9097, 0x9097, PDF_CMAP_SINGLE, 17151 },
	{ 0x9098, 0x9098, PDF_CMAP_SINGLE, 19895 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 15194 },
	{ 0x909b, 0x909b, PDF_CMAP_SINGLE, 15195 },
	{ 0x90a0, 0x90a0, PDF_CMAP_SINGLE, 19896 },
	{ 0x90a1, 0x90a1, PDF_CMAP_SINGLE, 18778 },
	{ 0x90a2, 0x90a2, PDF_CMAP_SINGLE, 15196 },
	{ 0x90a3, 0x90a3, PDF_CMAP_SINGLE, 3257 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 22678 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 3676 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 6932 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 2309 },
	{ 0x90af, 0x90af, PDF_CMAP_SINGLE, 6933 },
	{ 0x90b0, 0x90b0, PDF_CMAP_SINGLE, 18780 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 6934 },
	{ 0x90b2, 0x90b2, PDF_CMAP_SINGLE, 19897 },
	{ 0x90b3, 0x90b3, PDF_CMAP_SINGLE, 17152 },
	{ 0x90b4, 0x90b4, PDF_CMAP_SINGLE, 15197 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 6935 },
	{ 0x90b6, 0x90b6, PDF_CMAP_SINGLE, 18781 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 3099 },
	{ 0x90bd, 0x90bd, PDF_CMAP_SINGLE, 19898 },
	{ 0x90be, 0x90be, PDF_CMAP_SINGLE, 17153 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 1198 },
	{ 0x90c3, 0x90c3, PDF_CMAP_SINGLE, 18782 },
	{ 0x90c4, 0x90c5, PDF_CMAP_RANGE, 17154 },
	{ 0x90c7, 0x90c7, PDF_CMAP_SINGLE, 17156 },
	{ 0x90c8, 0x90c8, PDF_CMAP_SINGLE, 18783 },
	{ 0x90c9, 0x90c9, PDF_CMAP_SINGLE, 19899 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 2027 },
	{ 0x90cc, 0x90cc, PDF_CMAP_SINGLE, 22679 },
	{ 0x90ce, 0x90ce, PDF_CMAP_SINGLE, 4064 },
	{ 0x90d2, 0x90d2, PDF_CMAP_SINGLE, 22684 },
	{ 0x90d5, 0x90d5, PDF_CMAP_SINGLE, 22680 },
	{ 0x90d7, 0x90d7, PDF_CMAP_SINGLE, 15198 },
	{ 0x90d8, 0x90d9, PDF_CMAP_RANGE, 22681 },
	{ 0x90db, 0x90db, PDF_CMAP_SINGLE, 6939 },
	{ 0x90dc, 0x90dc, PDF_CMAP_SINGLE, 18785 },
	{ 0x90dd, 0x90dd, PDF_CMAP_SINGLE, 15199 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 8635 },
	{ 0x90df, 0x90df, PDF_CMAP_SINGLE, 18786 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 1802 },
	{ 0x90e2, 0x90e2, PDF_CMAP_SINGLE, 6936 },
	{ 0x90e4, 0x90e4, PDF_CMAP_SINGLE, 6937 },
	{ 0x90e5, 0x90e5, PDF_CMAP_SINGLE, 22683 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 3558 },
	{ 0x90eb, 0x90eb, PDF_CMAP_SINGLE, 18791 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 1458 },
	{ 0x90ef, 0x90ef, PDF_CMAP_SINGLE, 17157 },
	{ 0x90f0, 0x90f0, PDF_CMAP_SINGLE, 19900 },
	{ 0x90f2, 0x90f2, PDF_CMAP_SINGLE, 18789 },
	{ 0x90f4, 0x90f4, PDF_CMAP_SINGLE, 15200 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 3875 },
	{ 0x90f6, 0x90f6, PDF_CMAP_SINGLE, 18788 },
	{ 0x90f7, 0x90f7, PDF_CMAP_SINGLE, 1719 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 3150 },
	{ 0x90fe, 0x90ff, PDF_CMAP_RANGE, 18792 },
	{ 0x9100, 0x9100, PDF_CMAP_SINGLE, 18790 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 6940 },
	{ 0x9104, 0x9104, PDF_CMAP_SINGLE, 18794 },
	{ 0x9105, 0x9105, PDF_CMAP_SINGLE, 19901 },
	{ 0x9106, 0x9106, PDF_CMAP_SINGLE, 18795 },
	{ 0x9108, 0x9108, PDF_CMAP_SINGLE, 22685 },
	{ 0x910d, 0x910d, PDF_CMAP_SINGLE, 22686 },
	{ 0x9110, 0x9110, PDF_CMAP_SINGLE, 22687 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 6941 },
	{ 0x9114, 0x9114, PDF_CMAP_SINGLE, 17158 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 8637 },
	{ 0x9116, 0x9116, PDF_CMAP_SINGLE, 17159 },
	{ 0x9117, 0x9117, PDF_CMAP_SINGLE, 15201 },
	{ 0x9118, 0x9118, PDF_CMAP_SINGLE, 18796 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 6942 },
	{ 0x911a, 0x911a, PDF_CMAP_SINGLE, 22688 },
	{ 0x911c, 0x911c, PDF_CMAP_SINGLE, 15202 },
	{ 0x911e, 0x911e, PDF_CMAP_SINGLE, 18797 },
	{ 0x9120, 0x9120, PDF_CMAP_SINGLE, 22689 },
	{ 0x9122, 0x9123, PDF_CMAP_RANGE, 17160 },
	{ 0x9125, 0x9125, PDF_CMAP_SINGLE, 19902 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 8638 },
	{ 0x9129, 0x9129, PDF_CMAP_SINGLE, 22690 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 3100 },
	{ 0x912e, 0x912e, PDF_CMAP_SINGLE, 22691 },
	{ 0x912f, 0x912f, PDF_CMAP_SINGLE, 17162 },
	{ 0x9130, 0x9130, PDF_CMAP_SINGLE, 6944 },
	{ 0x9131, 0x9131, PDF_CMAP_SINGLE, 15203 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 6943 },
	{ 0x9134, 0x9134, PDF_CMAP_SINGLE, 17163 },
	{ 0x9136, 0x9136, PDF_CMAP_SINGLE, 22692 },
	{ 0x9137, 0x9137, PDF_CMAP_SINGLE, 18798 },
	{ 0x9139, 0x9139, PDF_CMAP_SINGLE, 18799 },
	{ 0x913a, 0x913a, PDF_CMAP_SINGLE, 15204 },
	{ 0x913c, 0x913c, PDF_CMAP_SINGLE, 22693 },
	{ 0x913d, 0x913d, PDF_CMAP_SINGLE, 15205 },
	{ 0x9143, 0x9143, PDF_CMAP_SINGLE, 22694 },
	{ 0x9146, 0x9147, PDF_CMAP_RANGE, 18800 },
	{ 0x9148, 0x9148, PDF_CMAP_SINGLE, 15206 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 3243 },
	{ 0x914a, 0x914a, PDF_CMAP_SINGLE, 6945 },
	{ 0x914b, 0x914b, PDF_CMAP_SINGLE, 2368 },
	{ 0x914c, 0x914c, PDF_CMAP_SINGLE, 2316 },
	{ 0x914d, 0x914d, PDF_CMAP_SINGLE, 3345 },
	{ 0x914e, 0x914e, PDF_CMAP_SINGLE, 2991 },
	{ 0x914f, 0x914f, PDF_CMAP_SINGLE, 22695 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 2334 },
	{ 0x9153, 0x9153, PDF_CMAP_SINGLE, 22696 },
	{ 0x9154, 0x9154, PDF_CMAP_SINGLE, 2610 },
	{ 0x9156, 0x9156, PDF_CMAP_SINGLE, 6946 },
	{ 0x9157, 0x9157, PDF_CMAP_SINGLE, 18802 },
	{ 0x9158, 0x9158, PDF_CMAP_SINGLE, 6947 },
	{ 0x9159, 0x9159, PDF_CMAP_SINGLE, 18803 },
	{ 0x915a, 0x915a, PDF_CMAP_SINGLE, 19903 },
	{ 0x915b, 0x915b, PDF_CMAP_SINGLE, 15207 },
	{ 0x9161, 0x9161, PDF_CMAP_SINGLE, 15208 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 2595 },
	{ 0x9163, 0x9163, PDF_CMAP_SINGLE, 6948 },
	{ 0x9164, 0x9164, PDF_CMAP_SINGLE, 15209 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 6949 },
	{ 0x9167, 0x9167, PDF_CMAP_SINGLE, 19904 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 6950 },
	{ 0x916a, 0x916a, PDF_CMAP_SINGLE, 3929 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 2369 },
	{ 0x916d, 0x916d, PDF_CMAP_SINGLE, 22697 },
	{ 0x9172, 0x9172, PDF_CMAP_SINGLE, 6952 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 6951 },
	{ 0x9174, 0x9174, PDF_CMAP_SINGLE, 18804 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 2028 },
	{ 0x9177, 0x9177, PDF_CMAP_SINGLE, 2053 },
	{ 0x9178, 0x9178, PDF_CMAP_SINGLE, 2190 },
	{ 0x9179, 0x9179, PDF_CMAP_SINGLE, 18805 },
	{ 0x917a, 0x917a, PDF_CMAP_SINGLE, 19905 },
	{ 0x917b, 0x917b, PDF_CMAP_SINGLE, 22698 },
	{ 0x9181, 0x9181, PDF_CMAP_SINGLE, 22699 },
	{ 0x9182, 0x9182, PDF_CMAP_SINGLE, 6955 },
	{ 0x9183, 0x9183, PDF_CMAP_SINGLE, 17164 },
	{ 0x9185, 0x9185, PDF_CMAP_SINGLE, 18806 },
	{ 0x9186, 0x9186, PDF_CMAP_SINGLE, 22700 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 2416 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 6954 },
	{ 0x918a, 0x918a, PDF_CMAP_SINGLE, 19906 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 6953 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 2889 },
	{ 0x918e, 0x918e, PDF_CMAP_SINGLE, 15210 },
	{ 0x9190, 0x9190, PDF_CMAP_SINGLE, 1955 },
	{ 0x9191, 0x9191, PDF_CMAP_SINGLE, 19907 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 2663 },
	{ 0x9193, 0x9194, PDF_CMAP_RANGE, 22701 },
	{ 0x9195, 0x9195, PDF_CMAP_SINGLE, 19908 },
	{ 0x9197, 0x9197, PDF_CMAP_SINGLE, 3396 },
	{ 0x9198, 0x9198, PDF_CMAP_SINGLE, 22703 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 2371 },
	{ 0x919e, 0x919e, PDF_CMAP_SINGLE, 15211 },
	{ 0x91a1, 0x91a1, PDF_CMAP_SINGLE, 22704 },
	{ 0x91a2, 0x91a2, PDF_CMAP_SINGLE, 6956 },
	{ 0x91a4, 0x91a4, PDF_CMAP_SINGLE, 2503 },
	{ 0x91a6, 0x91a6, PDF_CMAP_SINGLE, 22705 },
	{ 0x91a8, 0x91a8, PDF_CMAP_SINGLE, 15212 },
	{ 0x91aa, 0x91aa, PDF_CMAP_SINGLE, 6959 },
	{ 0x91ab, 0x91ab, PDF_CMAP_SINGLE, 6957 },
	{ 0x91ac, 0x91ac, PDF_CMAP_SINGLE, 7707 },
	{ 0x91ad, 0x91ae, PDF_CMAP_RANGE, 15213 },
	{ 0x91af, 0x91af, PDF_CMAP_SINGLE, 6958 },
	{ 0x91b0, 0x91b0, PDF_CMAP_SINGLE, 19909 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 7777 },
	{ 0x91b2, 0x91b2, PDF_CMAP_SINGLE, 15215 },
	{ 0x91b3, 0x91b3, PDF_CMAP_SINGLE, 18807 },
	{ 0x91b4, 0x91b4, PDF_CMAP_SINGLE, 6961 },
	{ 0x91b5, 0x91b5, PDF_CMAP_SINGLE, 6960 },
	{ 0x91b6, 0x91b6, PDF_CMAP_SINGLE, 18808 },
	{ 0x91b8, 0x91b8, PDF_CMAP_SINGLE, 2530 },
	{ 0x91ba, 0x91ba, PDF_CMAP_SINGLE, 6962 },
	{ 0x91bb, 0x91bb, PDF_CMAP_SINGLE, 19910 },
	{ 0x91bc, 0x91bc, PDF_CMAP_SINGLE, 15216 },
	{ 0x91bd, 0x91bd, PDF_CMAP_SINGLE, 19911 },
	{ 0x91bf, 0x91bf, PDF_CMAP_SINGLE, 22706 },
	{ 0x91c0, 0x91c1, PDF_CMAP_RANGE, 6963 },
	{ 0x91c2, 0x91c2, PDF_CMAP_SINGLE, 19912 },
	{ 0x91c3, 0x91c4, PDF_CMAP_RANGE, 18809 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 19913 },
	{ 0x91c6, 0x91c6, PDF_CMAP_SINGLE, 3428 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 2115 },
	{ 0x91c8, 0x91c8, PDF_CMAP_SINGLE, 2317 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 6965 },
	{ 0x91cb, 0x91cb, PDF_CMAP_SINGLE, 6966 },
	{ 0x91cc, 0x91cc, PDF_CMAP_SINGLE, 3948 },
	{ 0x91cd, 0x91cd, PDF_CMAP_SINGLE, 2383 },
	{ 0x91ce, 0x91ce, PDF_CMAP_SINGLE, 3834 },
	{ 0x91cf, 0x91cf, PDF_CMAP_SINGLE, 3988 },
	{ 0x91d0, 0x91d0, PDF_CMAP_SINGLE, 6967 },
	{ 0x91d1, 0x91d1, PDF_CMAP_SINGLE, 1754 },
	{ 0x91d3, 0x91d4, PDF_CMAP_RANGE, 22707 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 6968 },
	{ 0x91d7, 0x91d7, PDF_CMAP_SINGLE, 8640 },
	{ 0x91d8, 0x91d8, PDF_CMAP_SINGLE, 3101 },
	{ 0x91d9, 0x91d9, PDF_CMAP_SINGLE, 22709 },
	{ 0x91da, 0x91da, PDF_CMAP_SINGLE, 8639 },
	{ 0x91db, 0x91db, PDF_CMAP_SINGLE, 6971 },
	{ 0x91dc, 0x91dc, PDF_CMAP_SINGLE, 1494 },
	{ 0x91dd, 0x91dd, PDF_CMAP_SINGLE, 2577 },
	{ 0x91de, 0x91de, PDF_CMAP_SINGLE, 8641 },
	{ 0x91df, 0x91df, PDF_CMAP_SINGLE, 6969 },
	{ 0x91e1, 0x91e1, PDF_CMAP_SINGLE, 6970 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 3068 },
	{ 0x91e4, 0x91e5, PDF_CMAP_RANGE, 8644 },
	{ 0x91e6, 0x91e6, PDF_CMAP_SINGLE, 3715 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 1780 },
	{ 0x91e9, 0x91ea, PDF_CMAP_RANGE, 22710 },
	{ 0x91ec, 0x91ec, PDF_CMAP_SINGLE, 18813 },
	{ 0x91ed, 0x91ee, PDF_CMAP_RANGE, 8642 },
	{ 0x91ef, 0x91ef, PDF_CMAP_SINGLE, 22712 },
	{ 0x91f0, 0x91f0, PDF_CMAP_SINGLE, 15217 },
	{ 0x91f1, 0x91f1, PDF_CMAP_SINGLE, 17165 },
	{ 0x91f5, 0x91f6, PDF_CMAP_RANGE, 6973 },
	{ 0x91f7, 0x91f7, PDF_CMAP_SINGLE, 15218 },
	{ 0x91f9, 0x91f9, PDF_CMAP_SINGLE, 22713 },
	{ 0x91fb, 0x91fb, PDF_CMAP_SINGLE, 15219 },
	{ 0x91fc, 0x91fc, PDF_CMAP_SINGLE, 6972 },
	{ 0x91fd, 0x91fd, PDF_CMAP_SINGLE, 22714 },
	{ 0x91ff, 0x91ff, PDF_CMAP_SINGLE, 6976 },
	{ 0x9200, 0x9200, PDF_CMAP_SINGLE, 19914 },
	{ 0x9201, 0x9201, PDF_CMAP_SINGLE, 18814 },
	{ 0x9204, 0x9205, PDF_CMAP_RANGE, 22715 },
	{ 0x9206, 0x9206, PDF_CMAP_SINGLE, 8646 },
	{ 0x9207, 0x9207, PDF_CMAP_SINGLE, 15220 },
	{ 0x9209, 0x9209, PDF_CMAP_SINGLE, 19915 },
	{ 0x920a, 0x920a, PDF_CMAP_SINGLE, 8648 },
	{ 0x920c, 0x920c, PDF_CMAP_SINGLE, 22717 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 3255 },
	{ 0x920e, 0x920e, PDF_CMAP_SINGLE, 1441 },
	{ 0x9210, 0x9210, PDF_CMAP_SINGLE, 8647 },
	{ 0x9211, 0x9211, PDF_CMAP_SINGLE, 6980 },
	{ 0x9212, 0x9213, PDF_CMAP_RANGE, 22718 },
	{ 0x9214, 0x9214, PDF_CMAP_SINGLE, 6977 },
	{ 0x9215, 0x9215, PDF_CMAP_SINGLE, 6979 },
	{ 0x9216, 0x9217, PDF_CMAP_RANGE, 18815 },
	{ 0x9218, 0x9218, PDF_CMAP_SINGLE, 22720 },
	{ 0x921c, 0x921d, PDF_CMAP_RANGE, 22721 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 6975 },
	{ 0x9223, 0x9223, PDF_CMAP_SINGLE, 19916 },
	{ 0x9224, 0x9226, PDF_CMAP_RANGE, 22723 },
	{ 0x9228, 0x9228, PDF_CMAP_SINGLE, 15221 },
	{ 0x9229, 0x9229, PDF_CMAP_SINGLE, 7050 },
	{ 0x922c, 0x922c, PDF_CMAP_SINGLE, 6978 },
	{ 0x922e, 0x9230, PDF_CMAP_RANGE, 22726 },
	{ 0x9233, 0x9233, PDF_CMAP_SINGLE, 15222 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 4019 },
	{ 0x9235, 0x9236, PDF_CMAP_RANGE, 22729 },
	{ 0x9237, 0x9237, PDF_CMAP_SINGLE, 1934 },
	{ 0x9238, 0x9238, PDF_CMAP_SINGLE, 15223 },
	{ 0x9239, 0x9239, PDF_CMAP_SINGLE, 8655 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 8649 },
	{ 0x923c, 0x923c, PDF_CMAP_SINGLE, 8651 },
	{ 0x923e, 0x923e, PDF_CMAP_SINGLE, 22731 },
	{ 0x923f, 0x923f, PDF_CMAP_SINGLE, 6988 },
	{ 0x9240, 0x9240, PDF_CMAP_SINGLE, 8650 },
	{ 0x9242, 0x9242, PDF_CMAP_SINGLE, 18818 },
	{ 0x9243, 0x9243, PDF_CMAP_SINGLE, 15224 },
	{ 0x9244, 0x9244, PDF_CMAP_SINGLE, 3118 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 6983 },
	{ 0x9246, 0x9246, PDF_CMAP_SINGLE, 22732 },
	{ 0x9247, 0x9247, PDF_CMAP_SINGLE, 15225 },
	{ 0x9248, 0x9248, PDF_CMAP_SINGLE, 6986 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 6984 },
	{ 0x924a, 0x924a, PDF_CMAP_SINGLE, 18819 },
	{ 0x924b, 0x924b, PDF_CMAP_SINGLE, 6989 },
	{ 0x924d, 0x924d, PDF_CMAP_SINGLE, 22733 },
	{ 0x924e, 0x924e, PDF_CMAP_SINGLE, 8652 },
	{ 0x924f, 0x924f, PDF_CMAP_SINGLE, 15226 },
	{ 0x9250, 0x9250, PDF_CMAP_SINGLE, 6990 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 8654 },
	{ 0x9256, 0x9256, PDF_CMAP_SINGLE, 18820 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 6982 },
	{ 0x9258, 0x9258, PDF_CMAP_SINGLE, 22734 },
	{ 0x9259, 0x9259, PDF_CMAP_SINGLE, 8653 },
	{ 0x925a, 0x925a, PDF_CMAP_SINGLE, 6995 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 1302 },
	{ 0x925c, 0x925d, PDF_CMAP_RANGE, 22735 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 6981 },
	{ 0x9260, 0x9260, PDF_CMAP_SINGLE, 15227 },
	{ 0x9261, 0x9261, PDF_CMAP_SINGLE, 18821 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 3393 },
	{ 0x9264, 0x9264, PDF_CMAP_SINGLE, 6985 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 18822 },
	{ 0x9266, 0x9266, PDF_CMAP_SINGLE, 2504 },
	{ 0x9267, 0x9267, PDF_CMAP_SINGLE, 8656 },
	{ 0x9268, 0x9268, PDF_CMAP_SINGLE, 18823 },
	{ 0x9269, 0x9269, PDF_CMAP_SINGLE, 22737 },
	{ 0x926e, 0x9270, PDF_CMAP_RANGE, 22738 },
	{ 0x9271, 0x9271, PDF_CMAP_SINGLE, 2029 },
	{ 0x9275, 0x9275, PDF_CMAP_SINGLE, 22741 },
	{ 0x9276, 0x9276, PDF_CMAP_SINGLE, 19917 },
	{ 0x9277, 0x9278, PDF_CMAP_RANGE, 8658 },
	{ 0x9279, 0x9279, PDF_CMAP_SINGLE, 22742 },
	{ 0x927b, 0x927b, PDF_CMAP_SINGLE, 22743 },
	{ 0x927c, 0x927d, PDF_CMAP_RANGE, 18825 },
	{ 0x927e, 0x927e, PDF_CMAP_SINGLE, 3702 },
	{ 0x927f, 0x927f, PDF_CMAP_SINGLE, 18827 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 1756 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 2384 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 3220 },
	{ 0x9288, 0x9288, PDF_CMAP_SINGLE, 8362 },
	{ 0x9289, 0x9289, PDF_CMAP_SINGLE, 18828 },
	{ 0x928a, 0x928a, PDF_CMAP_SINGLE, 22744 },
	{ 0x928d, 0x928d, PDF_CMAP_SINGLE, 18829 },
	{ 0x928e, 0x928e, PDF_CMAP_SINGLE, 19918 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 2735 },
	{ 0x9292, 0x9292, PDF_CMAP_SINGLE, 22745 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 6993 },
	{ 0x9295, 0x9295, PDF_CMAP_SINGLE, 6987 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 6992 },
	{ 0x9297, 0x9297, PDF_CMAP_SINGLE, 18830 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 3791 },
	{ 0x9299, 0x9299, PDF_CMAP_SINGLE, 18831 },
	{ 0x929a, 0x929a, PDF_CMAP_SINGLE, 3028 },
	{ 0x929b, 0x929b, PDF_CMAP_SINGLE, 6994 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 6991 },
	{ 0x929f, 0x929f, PDF_CMAP_SINGLE, 18832 },
	{ 0x92a0, 0x92a0, PDF_CMAP_SINGLE, 22746 },
	{ 0x92a4, 0x92a5, PDF_CMAP_RANGE, 22747 },
	{ 0x92a7, 0x92a7, PDF_CMAP_SINGLE, 8657 },
	{ 0x92a8, 0x92a8, PDF_CMAP_SINGLE, 22749 },
	{ 0x92ab, 0x92ab, PDF_CMAP_SINGLE, 18833 },
	{ 0x92ad, 0x92ad, PDF_CMAP_SINGLE, 2734 },
	{ 0x92af, 0x92af, PDF_CMAP_SINGLE, 19919 },
	{ 0x92b2, 0x92b2, PDF_CMAP_SINGLE, 18836 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 13652 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 22750 },
	{ 0x92b7, 0x92b7, PDF_CMAP_SINGLE, 6998 },
	{ 0x92b8, 0x92b8, PDF_CMAP_SINGLE, 22751 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 6997 },
	{ 0x92ba, 0x92ba, PDF_CMAP_SINGLE, 22752 },
	{ 0x92bb, 0x92bc, PDF_CMAP_RANGE, 19920 },
	{ 0x92bd, 0x92bd, PDF_CMAP_SINGLE, 22753 },
	{ 0x92bf, 0x92c0, PDF_CMAP_RANGE, 18837 },
	{ 0x92c1, 0x92c1, PDF_CMAP_SINGLE, 19922 },
	{ 0x92c2, 0x92c2, PDF_CMAP_SINGLE, 15228 },
	{ 0x92c3, 0x92c3, PDF_CMAP_SINGLE, 19923 },
	{ 0x92c5, 0x92c5, PDF_CMAP_SINGLE, 19924 },
	{ 0x92c6, 0x92c6, PDF_CMAP_SINGLE, 18839 },
	{ 0x92c7, 0x92c7, PDF_CMAP_SINGLE, 22754 },
	{ 0x92c8, 0x92c8, PDF_CMAP_SINGLE, 19925 },
	{ 0x92cb, 0x92cc, PDF_CMAP_RANGE, 15229 },
	{ 0x92cd, 0x92cd, PDF_CMAP_SINGLE, 22755 },
	{ 0x92ce, 0x92ce, PDF_CMAP_SINGLE, 18840 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 6996 },
	{ 0x92d0, 0x92d0, PDF_CMAP_SINGLE, 8663 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 3677 },
	{ 0x92d3, 0x92d3, PDF_CMAP_SINGLE, 8667 },
	{ 0x92d5, 0x92d5, PDF_CMAP_SINGLE, 8665 },
	{ 0x92d7, 0x92d7, PDF_CMAP_SINGLE, 8661 },
	{ 0x92d8, 0x92d8, PDF_CMAP_SINGLE, 22756 },
	{ 0x92d9, 0x92d9, PDF_CMAP_SINGLE, 8662 },
	{ 0x92dc, 0x92dd, PDF_CMAP_RANGE, 22757 },
	{ 0x92df, 0x92df, PDF_CMAP_SINGLE, 15231 },
	{ 0x92e0, 0x92e0, PDF_CMAP_SINGLE, 8666 },
	{ 0x92e1, 0x92e1, PDF_CMAP_SINGLE, 22759 },
	{ 0x92e3, 0x92e3, PDF_CMAP_SINGLE, 22760 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 2437 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 18841 },
	{ 0x92e7, 0x92e7, PDF_CMAP_SINGLE, 8660 },
	{ 0x92e8, 0x92e8, PDF_CMAP_SINGLE, 22761 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 6999 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 3631 },
	{ 0x92ec, 0x92ec, PDF_CMAP_SINGLE, 22762 },
	{ 0x92ed, 0x92ed, PDF_CMAP_SINGLE, 1270 },
	{ 0x92ee, 0x92ee, PDF_CMAP_SINGLE, 22763 },
	{ 0x92f0, 0x92f0, PDF_CMAP_SINGLE, 22764 },
	{ 0x92f2, 0x92f2, PDF_CMAP_SINGLE, 3512 },
	{ 0x92f3, 0x92f3, PDF_CMAP_SINGLE, 2992 },
	{ 0x92f7, 0x92f7, PDF_CMAP_SINGLE, 18845 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 1682 },
	{ 0x92f9, 0x92f9, PDF_CMAP_SINGLE, 8368 },
	{ 0x92fa, 0x92fa, PDF_CMAP_SINGLE, 7001 },
	{ 0x92fb, 0x92fb, PDF_CMAP_SINGLE, 8670 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 2031 },
	{ 0x92ff, 0x92ff, PDF_CMAP_SINGLE, 8673 },
	{ 0x9300, 0x9300, PDF_CMAP_SINGLE, 22765 },
	{ 0x9302, 0x9302, PDF_CMAP_SINGLE, 8675 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 13402 },
	{ 0x9306, 0x9306, PDF_CMAP_SINGLE, 2170 },
	{ 0x9308, 0x9308, PDF_CMAP_SINGLE, 22766 },
	{ 0x930d, 0x930d, PDF_CMAP_SINGLE, 15232 },
	{ 0x930f, 0x930f, PDF_CMAP_SINGLE, 7000 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 2611 },
	{ 0x9311, 0x9311, PDF_CMAP_SINGLE, 18842 },
	{ 0x9314, 0x9314, PDF_CMAP_SINGLE, 19926 },
	{ 0x9315, 0x9315, PDF_CMAP_SINGLE, 15233 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 2612 },
	{ 0x9319, 0x9319, PDF_CMAP_SINGLE, 7004 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 7006 },
	{ 0x931c, 0x931c, PDF_CMAP_SINGLE, 22767 },
	{ 0x931d, 0x931d, PDF_CMAP_SINGLE, 8674 },
	{ 0x931e, 0x931e, PDF_CMAP_SINGLE, 8672 },
	{ 0x931f, 0x931f, PDF_CMAP_SINGLE, 15234 },
	{ 0x9320, 0x9320, PDF_CMAP_SINGLE, 2531 },
	{ 0x9321, 0x9321, PDF_CMAP_SINGLE, 8669 },
	{ 0x9322, 0x9322, PDF_CMAP_SINGLE, 7005 },
	{ 0x9323, 0x9323, PDF_CMAP_SINGLE, 7007 },
	{ 0x9324, 0x9324, PDF_CMAP_SINGLE, 22768 },
	{ 0x9325, 0x9325, PDF_CMAP_SINGLE, 8668 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 1739 },
	{ 0x9327, 0x9327, PDF_CMAP_SINGLE, 15235 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 3511 },
	{ 0x9329, 0x9329, PDF_CMAP_SINGLE, 18846 },
	{ 0x932a, 0x932a, PDF_CMAP_SINGLE, 22769 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 2318 },
	{ 0x932c, 0x932c, PDF_CMAP_SINGLE, 4041 },
	{ 0x932e, 0x932e, PDF_CMAP_SINGLE, 7003 },
	{ 0x932f, 0x932f, PDF_CMAP_SINGLE, 2152 },
	{ 0x9332, 0x9332, PDF_CMAP_SINGLE, 4069 },
	{ 0x9333, 0x9333, PDF_CMAP_SINGLE, 19927 },
	{ 0x9334, 0x9334, PDF_CMAP_SINGLE, 22770 },
	{ 0x9335, 0x9335, PDF_CMAP_SINGLE, 7009 },
	{ 0x9336, 0x9336, PDF_CMAP_SINGLE, 19928 },
	{ 0x9337, 0x9337, PDF_CMAP_SINGLE, 22771 },
	{ 0x933a, 0x933a, PDF_CMAP_SINGLE, 7008 },
	{ 0x933b, 0x933b, PDF_CMAP_SINGLE, 7010 },
	{ 0x9344, 0x9344, PDF_CMAP_SINGLE, 7002 },
	{ 0x9347, 0x9347, PDF_CMAP_SINGLE, 15236 },
	{ 0x9348, 0x9348, PDF_CMAP_SINGLE, 8361 },
	{ 0x9349, 0x9349, PDF_CMAP_SINGLE, 17166 },
	{ 0x934a, 0x934a, PDF_CMAP_SINGLE, 13400 },
	{ 0x934b, 0x934b, PDF_CMAP_SINGLE, 3265 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 3151 },
	{ 0x9350, 0x9350, PDF_CMAP_SINGLE, 22772 },
	{ 0x9351, 0x9351, PDF_CMAP_SINGLE, 18849 },
	{ 0x9352, 0x9352, PDF_CMAP_SINGLE, 15237 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 3059 },
	{ 0x9355, 0x9355, PDF_CMAP_SINGLE, 22773 },
	{ 0x9356, 0x9356, PDF_CMAP_SINGLE, 7015 },
	{ 0x9357, 0x9357, PDF_CMAP_SINGLE, 8677 },
	{ 0x9358, 0x9358, PDF_CMAP_SINGLE, 19929 },
	{ 0x935a, 0x935a, PDF_CMAP_SINGLE, 18850 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 2945 },
	{ 0x935c, 0x935c, PDF_CMAP_SINGLE, 7011 },
	{ 0x935e, 0x935e, PDF_CMAP_SINGLE, 22774 },
	{ 0x9360, 0x9360, PDF_CMAP_SINGLE, 7012 },
	{ 0x9364, 0x9365, PDF_CMAP_RANGE, 17167 },
	{ 0x9367, 0x9367, PDF_CMAP_SINGLE, 22775 },
	{ 0x9369, 0x9369, PDF_CMAP_SINGLE, 22776 },
	{ 0x936a, 0x936a, PDF_CMAP_SINGLE, 15239 },
	{ 0x936b, 0x936b, PDF_CMAP_SINGLE, 18851 },
	{ 0x936c, 0x936c, PDF_CMAP_SINGLE, 1795 },
	{ 0x936d, 0x936d, PDF_CMAP_SINGLE, 15240 },
	{ 0x936e, 0x936e, PDF_CMAP_SINGLE, 7014 },
	{ 0x936f, 0x936f, PDF_CMAP_SINGLE, 22777 },
	{ 0x9370, 0x9370, PDF_CMAP_SINGLE, 8676 },
	{ 0x9371, 0x9371, PDF_CMAP_SINGLE, 18852 },
	{ 0x9373, 0x9373, PDF_CMAP_SINGLE, 18853 },
	{ 0x9374, 0x9374, PDF_CMAP_SINGLE, 22778 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 1892 },
	{ 0x9376, 0x9376, PDF_CMAP_SINGLE, 22779 },
	{ 0x937a, 0x937a, PDF_CMAP_SINGLE, 22780 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 7013 },
	{ 0x937d, 0x937d, PDF_CMAP_SINGLE, 22781 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 2505 },
	{ 0x937f, 0x937f, PDF_CMAP_SINGLE, 19930 },
	{ 0x9380, 0x9381, PDF_CMAP_RANGE, 22782 },
	{ 0x9382, 0x9382, PDF_CMAP_SINGLE, 19931 },
	{ 0x9388, 0x9388, PDF_CMAP_SINGLE, 18857 },
	{ 0x938a, 0x938a, PDF_CMAP_SINGLE, 19932 },
	{ 0x938b, 0x938b, PDF_CMAP_SINGLE, 18858 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 1495 },
	{ 0x938d, 0x938d, PDF_CMAP_SINGLE, 22784 },
	{ 0x938f, 0x938f, PDF_CMAP_SINGLE, 18859 },
	{ 0x9392, 0x9392, PDF_CMAP_SINGLE, 22785 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 7019 },
	{ 0x9395, 0x9395, PDF_CMAP_SINGLE, 22786 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 2095 },
	{ 0x9397, 0x9397, PDF_CMAP_SINGLE, 2811 },
	{ 0x9398, 0x9398, PDF_CMAP_SINGLE, 22787 },
	{ 0x939a, 0x939a, PDF_CMAP_SINGLE, 3046 },
	{ 0x939b, 0x939b, PDF_CMAP_SINGLE, 15241 },
	{ 0x939e, 0x939e, PDF_CMAP_SINGLE, 18860 },
	{ 0x93a1, 0x93a1, PDF_CMAP_SINGLE, 18854 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 17169 },
	{ 0x93a4, 0x93a4, PDF_CMAP_SINGLE, 8678 },
	{ 0x93a6, 0x93a6, PDF_CMAP_SINGLE, 22789 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 1433 },
	{ 0x93a8, 0x93a8, PDF_CMAP_SINGLE, 22790 },
	{ 0x93a9, 0x93a9, PDF_CMAP_SINGLE, 15243 },
	{ 0x93ab, 0x93ab, PDF_CMAP_SINGLE, 22791 },
	{ 0x93ac, 0x93ad, PDF_CMAP_RANGE, 7017 },
	{ 0x93ae, 0x93ae, PDF_CMAP_SINGLE, 3039 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 7016 },
	{ 0x93b4, 0x93b6, PDF_CMAP_RANGE, 22792 },
	{ 0x93b9, 0x93b9, PDF_CMAP_SINGLE, 7020 },
	{ 0x93ba, 0x93ba, PDF_CMAP_SINGLE, 15242 },
	{ 0x93bb, 0x93bb, PDF_CMAP_SINGLE, 19933 },
	{ 0x93c1, 0x93c1, PDF_CMAP_SINGLE, 15244 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 7026 },
	{ 0x93c4, 0x93c5, PDF_CMAP_RANGE, 22795 },
	{ 0x93c6, 0x93c6, PDF_CMAP_SINGLE, 8679 },
	{ 0x93c7, 0x93c7, PDF_CMAP_SINGLE, 18865 },
	{ 0x93c8, 0x93c8, PDF_CMAP_SINGLE, 7029 },
	{ 0x93c9, 0x93c9, PDF_CMAP_SINGLE, 22797 },
	{ 0x93ca, 0x93ca, PDF_CMAP_SINGLE, 15245 },
	{ 0x93cb, 0x93cb, PDF_CMAP_SINGLE, 22798 },
	{ 0x93cc, 0x93cc, PDF_CMAP_SINGLE, 19934 },
	{ 0x93cd, 0x93cd, PDF_CMAP_SINGLE, 22799 },
	{ 0x93d0, 0x93d0, PDF_CMAP_SINGLE, 7028 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 3111 },
	{ 0x93d3, 0x93d3, PDF_CMAP_SINGLE, 22800 },
	{ 0x93d6, 0x93d7, PDF_CMAP_RANGE, 7021 },
	{ 0x93d8, 0x93d8, PDF_CMAP_SINGLE, 7025 },
	{ 0x93d9, 0x93d9, PDF_CMAP_SINGLE, 22801 },
	{ 0x93dc, 0x93dc, PDF_CMAP_SINGLE, 18866 },
	{ 0x93dd, 0x93dd, PDF_CMAP_SINGLE, 7027 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 8680 },
	{ 0x93df, 0x93df, PDF_CMAP_SINGLE, 17170 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 1720 },
	{ 0x93e2, 0x93e2, PDF_CMAP_SINGLE, 15246 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 7030 },
	{ 0x93e5, 0x93e5, PDF_CMAP_SINGLE, 7024 },
	{ 0x93e6, 0x93e6, PDF_CMAP_SINGLE, 19935 },
	{ 0x93e7, 0x93e7, PDF_CMAP_SINGLE, 18867 },
	{ 0x93e8, 0x93e8, PDF_CMAP_SINGLE, 7023 },
	{ 0x93f1, 0x93f1, PDF_CMAP_SINGLE, 18864 },
	{ 0x93f5, 0x93f5, PDF_CMAP_SINGLE, 18861 },
	{ 0x93f7, 0x93f7, PDF_CMAP_SINGLE, 22802 },
	{ 0x93f8, 0x93f8, PDF_CMAP_SINGLE, 8681 },
	{ 0x93f9, 0x93f9, PDF_CMAP_SINGLE, 19936 },
	{ 0x93fa, 0x93fa, PDF_CMAP_SINGLE, 15247 },
	{ 0x93fb, 0x93fb, PDF_CMAP_SINGLE, 18871 },
	{ 0x93fd, 0x93fd, PDF_CMAP_SINGLE, 15248 },
	{ 0x9401, 0x9401, PDF_CMAP_SINGLE, 22803 },
	{ 0x9402, 0x9402, PDF_CMAP_SINGLE, 19937 },
	{ 0x9403, 0x9403, PDF_CMAP_SINGLE, 7034 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 17171 },
	{ 0x9407, 0x9407, PDF_CMAP_SINGLE, 7035 },
	{ 0x9408, 0x9408, PDF_CMAP_SINGLE, 22804 },
	{ 0x9409, 0x9409, PDF_CMAP_SINGLE, 18868 },
	{ 0x940d, 0x940e, PDF_CMAP_RANGE, 19938 },
	{ 0x940f, 0x940f, PDF_CMAP_SINGLE, 15249 },
	{ 0x9410, 0x9410, PDF_CMAP_SINGLE, 7036 },
	{ 0x9413, 0x9413, PDF_CMAP_SINGLE, 7033 },
	{ 0x9414, 0x9414, PDF_CMAP_SINGLE, 7032 },
	{ 0x9415, 0x9415, PDF_CMAP_SINGLE, 22805 },
	{ 0x9416, 0x9417, PDF_CMAP_RANGE, 18869 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 2506 },
	{ 0x9419, 0x9419, PDF_CMAP_SINGLE, 3202 },
	{ 0x941a, 0x941a, PDF_CMAP_SINGLE, 7031 },
	{ 0x941f, 0x941f, PDF_CMAP_SINGLE, 22806 },
	{ 0x9421, 0x9421, PDF_CMAP_SINGLE, 7040 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 7038 },
	{ 0x942e, 0x942e, PDF_CMAP_SINGLE, 19940 },
	{ 0x942f, 0x942f, PDF_CMAP_SINGLE, 22807 },
	{ 0x9431, 0x9431, PDF_CMAP_SINGLE, 8682 },
	{ 0x9432, 0x9432, PDF_CMAP_SINGLE, 18872 },
	{ 0x9433, 0x9433, PDF_CMAP_SINGLE, 17172 },
	{ 0x9434, 0x9434, PDF_CMAP_SINGLE, 15250 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 7039 },
	{ 0x9436, 0x9436, PDF_CMAP_SINGLE, 7037 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 2904 },
	{ 0x943a, 0x943a, PDF_CMAP_SINGLE, 7041 },
	{ 0x943b, 0x943b, PDF_CMAP_SINGLE, 18873 },
	{ 0x943d, 0x943d, PDF_CMAP_SINGLE, 22808 },
	{ 0x943f, 0x943f, PDF_CMAP_SINGLE, 15251 },
	{ 0x9441, 0x9441, PDF_CMAP_SINGLE, 7042 },
	{ 0x9443, 0x9443, PDF_CMAP_SINGLE, 22809 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 7044 },
	{ 0x9445, 0x9445, PDF_CMAP_SINGLE, 8683 },
	{ 0x9448, 0x9448, PDF_CMAP_SINGLE, 8684 },
	{ 0x944a, 0x944a, PDF_CMAP_SINGLE, 17173 },
	{ 0x944c, 0x944c, PDF_CMAP_SINGLE, 19941 },
	{ 0x9451, 0x9451, PDF_CMAP_SINGLE, 1553 },
	{ 0x9452, 0x9452, PDF_CMAP_SINGLE, 7043 },
	{ 0x9453, 0x9453, PDF_CMAP_SINGLE, 3846 },
	{ 0x9455, 0x9455, PDF_CMAP_SINGLE, 15252 },
	{ 0x9459, 0x9459, PDF_CMAP_SINGLE, 22810 },
	{ 0x945a, 0x945a, PDF_CMAP_SINGLE, 7055 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 7045 },
	{ 0x945c, 0x945c, PDF_CMAP_SINGLE, 22811 },
	{ 0x945e, 0x945e, PDF_CMAP_SINGLE, 7048 },
	{ 0x945f, 0x945f, PDF_CMAP_SINGLE, 22812 },
	{ 0x9460, 0x9460, PDF_CMAP_SINGLE, 7046 },
	{ 0x9461, 0x9461, PDF_CMAP_SINGLE, 22813 },
	{ 0x9462, 0x9462, PDF_CMAP_SINGLE, 7047 },
	{ 0x9463, 0x9463, PDF_CMAP_SINGLE, 17174 },
	{ 0x9468, 0x9468, PDF_CMAP_SINGLE, 22814 },
	{ 0x946a, 0x946a, PDF_CMAP_SINGLE, 7049 },
	{ 0x946b, 0x946b, PDF_CMAP_SINGLE, 15253 },
	{ 0x946d, 0x946d, PDF_CMAP_SINGLE, 18876 },
	{ 0x946e, 0x946e, PDF_CMAP_SINGLE, 22815 },
	{ 0x946f, 0x946f, PDF_CMAP_SINGLE, 18877 },
	{ 0x9470, 0x9470, PDF_CMAP_SINGLE, 7051 },
	{ 0x9471, 0x9471, PDF_CMAP_SINGLE, 17175 },
	{ 0x9472, 0x9472, PDF_CMAP_SINGLE, 15254 },
	{ 0x9475, 0x9475, PDF_CMAP_SINGLE, 7052 },
	{ 0x9477, 0x9477, PDF_CMAP_SINGLE, 7053 },
	{ 0x947c, 0x947c, PDF_CMAP_SINGLE, 7056 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 7054 },
	{ 0x947e, 0x947e, PDF_CMAP_SINGLE, 7057 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 7059 },
	{ 0x9481, 0x9481, PDF_CMAP_SINGLE, 7058 },
	{ 0x9483, 0x9483, PDF_CMAP_SINGLE, 22817 },
	{ 0x9484, 0x9484, PDF_CMAP_SINGLE, 22816 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 3029 },
	{ 0x9578, 0x9578, PDF_CMAP_SINGLE, 15255 },
	{ 0x9579, 0x9579, PDF_CMAP_SINGLE, 18878 },
	{ 0x957e, 0x957e, PDF_CMAP_SINGLE, 22818 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 3827 },
	{ 0x9582, 0x9582, PDF_CMAP_SINGLE, 7060 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 2736 },
	{ 0x9584, 0x9584, PDF_CMAP_SINGLE, 22819 },
	{ 0x9586, 0x9586, PDF_CMAP_SINGLE, 18879 },
	{ 0x9587, 0x9587, PDF_CMAP_SINGLE, 7061 },
	{ 0x9588, 0x9588, PDF_CMAP_SINGLE, 19942 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 3604 },
	{ 0x958a, 0x958a, PDF_CMAP_SINGLE, 7062 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 1417 },
	{ 0x958c, 0x958d, PDF_CMAP_RANGE, 18880 },
	{ 0x958e, 0x958e, PDF_CMAP_SINGLE, 17176 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 1246 },
	{ 0x9591, 0x9591, PDF_CMAP_SINGLE, 1555 },
	{ 0x9592, 0x9592, PDF_CMAP_SINGLE, 8685 },
	{ 0x9593, 0x9593, PDF_CMAP_SINGLE, 1554 },
	{ 0x9594, 0x9594, PDF_CMAP_SINGLE, 7063 },
	{ 0x9596, 0x9596, PDF_CMAP_SINGLE, 7064 },
	{ 0x9598, 0x9599, PDF_CMAP_RANGE, 7065 },
	{ 0x959d, 0x959e, PDF_CMAP_RANGE, 22820 },
	{ 0x959f, 0x959f, PDF_CMAP_SINGLE, 17177 },
	{ 0x95a0, 0x95a0, PDF_CMAP_SINGLE, 7067 },
	{ 0x95a1, 0x95a1, PDF_CMAP_SINGLE, 19943 },
	{ 0x95a2, 0x95a2, PDF_CMAP_SINGLE, 1556 },
	{ 0x95a3, 0x95a3, PDF_CMAP_SINGLE, 1459 },
	{ 0x95a4, 0x95a4, PDF_CMAP_SINGLE, 2032 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 3402 },
	{ 0x95a6, 0x95a6, PDF_CMAP_SINGLE, 15256 },
	{ 0x95a7, 0x95a7, PDF_CMAP_SINGLE, 7069 },
	{ 0x95a8, 0x95a8, PDF_CMAP_SINGLE, 7068 },
	{ 0x95a9, 0x95a9, PDF_CMAP_SINGLE, 15257 },
	{ 0x95ab, 0x95ab, PDF_CMAP_SINGLE, 20313 },
	{ 0x95ac, 0x95ac, PDF_CMAP_SINGLE, 17178 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 7070 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 13653 },
	{ 0x95b2, 0x95b2, PDF_CMAP_SINGLE, 1278 },
	{ 0x95b4, 0x95b4, PDF_CMAP_SINGLE, 15259 },
	{ 0x95b6, 0x95b6, PDF_CMAP_SINGLE, 17179 },
	{ 0x95b9, 0x95b9, PDF_CMAP_SINGLE, 7073 },
	{ 0x95ba, 0x95ba, PDF_CMAP_SINGLE, 22822 },
	{ 0x95bb, 0x95bb, PDF_CMAP_SINGLE, 7072 },
	{ 0x95bc, 0x95bc, PDF_CMAP_SINGLE, 7071 },
	{ 0x95bd, 0x95bd, PDF_CMAP_SINGLE, 15260 },
	{ 0x95be, 0x95be, PDF_CMAP_SINGLE, 7074 },
	{ 0x95bf, 0x95bf, PDF_CMAP_SINGLE, 19944 },
	{ 0x95c3, 0x95c3, PDF_CMAP_SINGLE, 7077 },
	{ 0x95c6, 0x95c6, PDF_CMAP_SINGLE, 19945 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 1163 },
	{ 0x95c8, 0x95c8, PDF_CMAP_SINGLE, 18884 },
	{ 0x95c9, 0x95c9, PDF_CMAP_SINGLE, 19946 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 7075 },
	{ 0x95cb, 0x95cb, PDF_CMAP_SINGLE, 17180 },
	{ 0x95cc, 0x95cc, PDF_CMAP_SINGLE, 7079 },
	{ 0x95cd, 0x95cd, PDF_CMAP_SINGLE, 7078 },
	{ 0x95d0, 0x95d0, PDF_CMAP_SINGLE, 17181 },
	{ 0x95d1, 0x95d2, PDF_CMAP_RANGE, 19947 },
	{ 0x95d3, 0x95d3, PDF_CMAP_SINGLE, 17182 },
	{ 0x95d4, 0x95d4, PDF_CMAP_SINGLE, 7081 },
	{ 0x95d5, 0x95d5, PDF_CMAP_SINGLE, 7080 },
	{ 0x95d6, 0x95d6, PDF_CMAP_SINGLE, 7082 },
	{ 0x95d8, 0x95d8, PDF_CMAP_SINGLE, 3206 },
	{ 0x95d9, 0x95d9, PDF_CMAP_SINGLE, 22823 },
	{ 0x95da, 0x95da, PDF_CMAP_SINGLE, 15261 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 7083 },
	{ 0x95dd, 0x95dd, PDF_CMAP_SINGLE, 22824 },
	{ 0x95de, 0x95de, PDF_CMAP_SINGLE, 17184 },
	{ 0x95df, 0x95df, PDF_CMAP_SINGLE, 22825 },
	{ 0x95e0, 0x95e0, PDF_CMAP_SINGLE, 19949 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 7084 },
	{ 0x95e2, 0x95e2, PDF_CMAP_SINGLE, 7086 },
	{ 0x95e4, 0x95e4, PDF_CMAP_SINGLE, 19950 },
	{ 0x95e5, 0x95e5, PDF_CMAP_SINGLE, 7085 },
	{ 0x95e6, 0x95e6, PDF_CMAP_SINGLE, 19951 },
	{ 0x95e8, 0x95e8, PDF_CMAP_SINGLE, 14061 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 3550 },
	{ 0x961d, 0x961d, PDF_CMAP_SINGLE, 15262 },
	{ 0x961e, 0x961e, PDF_CMAP_SINGLE, 22826 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 7087 },
	{ 0x9622, 0x9622, PDF_CMAP_SINGLE, 22827 },
	{ 0x9624, 0x9624, PDF_CMAP_SINGLE, 19952 },
	{ 0x9625, 0x9626, PDF_CMAP_RANGE, 22828 },
	{ 0x9628, 0x9628, PDF_CMAP_SINGLE, 7088 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 2133 },
	{ 0x962c, 0x962c, PDF_CMAP_SINGLE, 18887 },
	{ 0x962e, 0x962f, PDF_CMAP_RANGE, 7089 },
	{ 0x9631, 0x9631, PDF_CMAP_SINGLE, 19953 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 3703 },
	{ 0x9633, 0x9634, PDF_CMAP_RANGE, 18888 },
	{ 0x9637, 0x9637, PDF_CMAP_SINGLE, 22830 },
	{ 0x9638, 0x9638, PDF_CMAP_SINGLE, 19954 },
	{ 0x9639, 0x963a, PDF_CMAP_RANGE, 22831 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 2765 },
	{ 0x963c, 0x963c, PDF_CMAP_SINGLE, 18891 },
	{ 0x963d, 0x963d, PDF_CMAP_SINGLE, 19955 },
	{ 0x963f, 0x963f, PDF_CMAP_SINGLE, 1128 },
	{ 0x9640, 0x9640, PDF_CMAP_SINGLE, 2859 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 15263 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 7091 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 3551 },
	{ 0x964b, 0x964b, PDF_CMAP_SINGLE, 7094 },
	{ 0x964c, 0x964c, PDF_CMAP_SINGLE, 7092 },
	{ 0x964d, 0x964d, PDF_CMAP_SINGLE, 2033 },
	{ 0x964f, 0x964f, PDF_CMAP_SINGLE, 7093 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 1910 },
	{ 0x9652, 0x9652, PDF_CMAP_SINGLE, 22833 },
	{ 0x9654, 0x9654, PDF_CMAP_SINGLE, 19956 },
	{ 0x9656, 0x9657, PDF_CMAP_RANGE, 22834 },
	{ 0x9658, 0x9658, PDF_CMAP_SINGLE, 15264 },
	{ 0x965b, 0x965b, PDF_CMAP_SINGLE, 3605 },
	{ 0x965c, 0x965c, PDF_CMAP_SINGLE, 7096 },
	{ 0x965d, 0x965d, PDF_CMAP_SINGLE, 7098 },
	{ 0x965e, 0x965e, PDF_CMAP_SINGLE, 7097 },
	{ 0x965f, 0x965f, PDF_CMAP_SINGLE, 7099 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 18892 },
	{ 0x9662, 0x9662, PDF_CMAP_SINGLE, 1219 },
	{ 0x9663, 0x9663, PDF_CMAP_SINGLE, 2590 },
	{ 0x9664, 0x9664, PDF_CMAP_SINGLE, 2438 },
	{ 0x9665, 0x9665, PDF_CMAP_SINGLE, 1557 },
	{ 0x9666, 0x9666, PDF_CMAP_SINGLE, 7100 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 3356 },
	{ 0x966c, 0x966c, PDF_CMAP_SINGLE, 7102 },
	{ 0x966e, 0x966e, PDF_CMAP_SINGLE, 22836 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 1220 },
	{ 0x9672, 0x9672, PDF_CMAP_SINGLE, 7101 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 3040 },
	{ 0x9674, 0x9674, PDF_CMAP_SINGLE, 19957 },
	{ 0x9675, 0x9675, PDF_CMAP_SINGLE, 3989 },
	{ 0x9676, 0x9676, PDF_CMAP_SINGLE, 3203 },
	{ 0x9677, 0x9677, PDF_CMAP_SINGLE, 7095 },
	{ 0x9678, 0x9678, PDF_CMAP_SINGLE, 3950 },
	{ 0x967a, 0x967a, PDF_CMAP_SINGLE, 1893 },
	{ 0x967b, 0x967b, PDF_CMAP_SINGLE, 19958 },
	{ 0x967c, 0x967c, PDF_CMAP_SINGLE, 22837 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 3909 },
	{ 0x967e, 0x967e, PDF_CMAP_SINGLE, 22838 },
	{ 0x967f, 0x967f, PDF_CMAP_SINGLE, 19959 },
	{ 0x9681, 0x9681, PDF_CMAP_SINGLE, 19960 },
	{ 0x9682, 0x9682, PDF_CMAP_SINGLE, 18894 },
	{ 0x9683, 0x9683, PDF_CMAP_SINGLE, 19961 },
	{ 0x9684, 0x9684, PDF_CMAP_SINGLE, 15265 },
	{ 0x9685, 0x9685, PDF_CMAP_SINGLE, 1777 },
	{ 0x9686, 0x9686, PDF_CMAP_SINGLE, 3964 },
	{ 0x9688, 0x9688, PDF_CMAP_SINGLE, 1790 },
	{ 0x9689, 0x9689, PDF_CMAP_SINGLE, 19962 },
	{ 0x968a, 0x968a, PDF_CMAP_SINGLE, 2882 },
	{ 0x968b, 0x968b, PDF_CMAP_SINGLE, 6252 },
	{ 0x968d, 0x968d, PDF_CMAP_SINGLE, 7103 },
	{ 0x968e, 0x968e, PDF_CMAP_SINGLE, 1418 },
	{ 0x968f, 0x968f, PDF_CMAP_SINGLE, 2613 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 22839 },
	{ 0x9694, 0x9694, PDF_CMAP_SINGLE, 1460 },
	{ 0x9695, 0x9695, PDF_CMAP_SINGLE, 7105 },
	{ 0x9696, 0x9696, PDF_CMAP_SINGLE, 19963 },
	{ 0x9697, 0x9697, PDF_CMAP_SINGLE, 7106 },
	{ 0x9698, 0x9698, PDF_CMAP_SINGLE, 7104 },
	{ 0x9699, 0x9699, PDF_CMAP_SINGLE, 1850 },
	{ 0x969a, 0x969a, PDF_CMAP_SINGLE, 18896 },
	{ 0x969b, 0x969b, PDF_CMAP_SINGLE, 2125 },
	{ 0x969c, 0x969c, PDF_CMAP_SINGLE, 2507 },
	{ 0x969d, 0x969d, PDF_CMAP_SINGLE, 8688 },
	{ 0x969f, 0x969f, PDF_CMAP_SINGLE, 22840 },
	{ 0x96a0, 0x96a0, PDF_CMAP_SINGLE, 1221 },
	{ 0x96a3, 0x96a3, PDF_CMAP_SINGLE, 4001 },
	{ 0x96a4, 0x96a4, PDF_CMAP_SINGLE, 15266 },
	{ 0x96a5, 0x96a5, PDF_CMAP_SINGLE, 17185 },
	{ 0x96a6, 0x96a6, PDF_CMAP_SINGLE, 22841 },
	{ 0x96a7, 0x96a7, PDF_CMAP_SINGLE, 7108 },
	{ 0x96a8, 0x96a8, PDF_CMAP_SINGLE, 6923 },
	{ 0x96a9, 0x96a9, PDF_CMAP_SINGLE, 15267 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 7107 },
	{ 0x96ae, 0x96ae, PDF_CMAP_SINGLE, 19964 },
	{ 0x96af, 0x96af, PDF_CMAP_SINGLE, 8689 },
	{ 0x96b0, 0x96b0, PDF_CMAP_SINGLE, 7111 },
	{ 0x96b1, 0x96b2, PDF_CMAP_RANGE, 7109 },
	{ 0x96b3, 0x96b3, PDF_CMAP_SINGLE, 18899 },
	{ 0x96b4, 0x96b4, PDF_CMAP_SINGLE, 7112 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 7113 },
	{ 0x96b7, 0x96b7, PDF_CMAP_SINGLE, 4020 },
	{ 0x96b8, 0x96b9, PDF_CMAP_RANGE, 7114 },
	{ 0x96ba, 0x96ba, PDF_CMAP_SINGLE, 18900 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 2669 },
	{ 0x96bc, 0x96bc, PDF_CMAP_SINGLE, 3407 },
	{ 0x96bd, 0x96bd, PDF_CMAP_SINGLE, 18901 },
	{ 0x96c0, 0x96c0, PDF_CMAP_SINGLE, 2627 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 1571 },
	{ 0x96c4, 0x96c4, PDF_CMAP_SINGLE, 3876 },
	{ 0x96c5, 0x96c5, PDF_CMAP_SINGLE, 1389 },
	{ 0x96c6, 0x96c6, PDF_CMAP_SINGLE, 2370 },
	{ 0x96c7, 0x96c7, PDF_CMAP_SINGLE, 1935 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 7118 },
	{ 0x96ca, 0x96ca, PDF_CMAP_SINGLE, 22842 },
	{ 0x96cb, 0x96cb, PDF_CMAP_SINGLE, 7117 },
	{ 0x96cc, 0x96cc, PDF_CMAP_SINGLE, 2241 },
	{ 0x96cd, 0x96cd, PDF_CMAP_SINGLE, 7119 },
	{ 0x96ce, 0x96ce, PDF_CMAP_SINGLE, 7116 },
	{ 0x96d1, 0x96d1, PDF_CMAP_SINGLE, 2166 },
	{ 0x96d2, 0x96d2, PDF_CMAP_SINGLE, 15268 },
	{ 0x96d5, 0x96d5, PDF_CMAP_SINGLE, 7123 },
	{ 0x96d6, 0x96d6, PDF_CMAP_SINGLE, 6546 },
	{ 0x96d8, 0x96d8, PDF_CMAP_SINGLE, 18904 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 4331 },
	{ 0x96da, 0x96da, PDF_CMAP_SINGLE, 18905 },
	{ 0x96db, 0x96db, PDF_CMAP_SINGLE, 2621 },
	{ 0x96dc, 0x96dc, PDF_CMAP_SINGLE, 7121 },
	{ 0x96dd, 0x96dd, PDF_CMAP_SINGLE, 18906 },
	{ 0x96de, 0x96de, PDF_CMAP_SINGLE, 15270 },
	{ 0x96df, 0x96df, PDF_CMAP_SINGLE, 22844 },
	{ 0x96e2, 0x96e2, PDF_CMAP_SINGLE, 3949 },
	{ 0x96e3, 0x96e3, PDF_CMAP_SINGLE, 3273 },
	{ 0x96e8, 0x96e8, PDF_CMAP_SINGLE, 1229 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 15271 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 2695 },
	{ 0x96eb, 0x96eb, PDF_CMAP_SINGLE, 2274 },
	{ 0x96ef, 0x96ef, PDF_CMAP_SINGLE, 17186 },
	{ 0x96f0, 0x96f0, PDF_CMAP_SINGLE, 3591 },
	{ 0x96f1, 0x96f1, PDF_CMAP_SINGLE, 15272 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 1250 },
	{ 0x96f6, 0x96f6, PDF_CMAP_SINGLE, 4021 },
	{ 0x96f7, 0x96f7, PDF_CMAP_SINGLE, 3925 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 7124 },
	{ 0x96fa, 0x96fa, PDF_CMAP_SINGLE, 22845 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 3135 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 2343 },
	{ 0x9702, 0x9702, PDF_CMAP_SINGLE, 15273 },
	{ 0x9703, 0x9703, PDF_CMAP_SINGLE, 19965 },
	{ 0x9704, 0x9704, PDF_CMAP_SINGLE, 7125 },
	{ 0x9705, 0x9705, PDF_CMAP_SINGLE, 22846 },
	{ 0x9706, 0x9706, PDF_CMAP_SINGLE, 7126 },
	{ 0x9707, 0x9707, PDF_CMAP_SINGLE, 2578 },
	{ 0x9708, 0x9708, PDF_CMAP_SINGLE, 7127 },
	{ 0x9709, 0x9709, PDF_CMAP_SINGLE, 15274 },
	{ 0x970a, 0x970a, PDF_CMAP_SINGLE, 4022 },
	{ 0x970d, 0x970d, PDF_CMAP_SINGLE, 7122 },
	{ 0x970e, 0x970e, PDF_CMAP_SINGLE, 7129 },
	{ 0x970f, 0x970f, PDF_CMAP_SINGLE, 7131 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 7130 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 7128 },
	{ 0x9714, 0x9714, PDF_CMAP_SINGLE, 18908 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 7132 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 7133 },
	{ 0x971a, 0x971a, PDF_CMAP_SINGLE, 22847 },
	{ 0x971b, 0x971b, PDF_CMAP_SINGLE, 19966 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 2812 },
	{ 0x971d, 0x971d, PDF_CMAP_SINGLE, 22848 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 1378 },
	{ 0x9721, 0x9722, PDF_CMAP_RANGE, 19967 },
	{ 0x9723, 0x9723, PDF_CMAP_SINGLE, 18909 },
	{ 0x9724, 0x9724, PDF_CMAP_SINGLE, 7134 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 3780 },
	{ 0x9728, 0x9728, PDF_CMAP_SINGLE, 19969 },
	{ 0x972a, 0x972a, PDF_CMAP_SINGLE, 7135 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 7136 },
	{ 0x9731, 0x9731, PDF_CMAP_SINGLE, 19970 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 4048 },
	{ 0x9733, 0x9733, PDF_CMAP_SINGLE, 8690 },
	{ 0x9736, 0x9736, PDF_CMAP_SINGLE, 18911 },
	{ 0x9738, 0x9738, PDF_CMAP_SINGLE, 5140 },
	{ 0x9739, 0x9739, PDF_CMAP_SINGLE, 7137 },
	{ 0x973b, 0x973b, PDF_CMAP_SINGLE, 8691 },
	{ 0x973d, 0x973e, PDF_CMAP_RANGE, 7138 },
	{ 0x9741, 0x9741, PDF_CMAP_SINGLE, 18912 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 7143 },
	{ 0x9743, 0x9743, PDF_CMAP_SINGLE, 8692 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 7140 },
	{ 0x9746, 0x9746, PDF_CMAP_SINGLE, 7141 },
	{ 0x9747, 0x9747, PDF_CMAP_SINGLE, 18913 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 7142 },
	{ 0x9749, 0x9749, PDF_CMAP_SINGLE, 7144 },
	{ 0x974a, 0x974a, PDF_CMAP_SINGLE, 22850 },
	{ 0x974d, 0x974d, PDF_CMAP_SINGLE, 8693 },
	{ 0x974e, 0x974e, PDF_CMAP_SINGLE, 17187 },
	{ 0x974f, 0x974f, PDF_CMAP_SINGLE, 8694 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 8695 },
	{ 0x9752, 0x9752, PDF_CMAP_SINGLE, 2664 },
	{ 0x9755, 0x9755, PDF_CMAP_SINGLE, 20314 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 3843 },
	{ 0x9757, 0x9757, PDF_CMAP_SINGLE, 18914 },
	{ 0x9758, 0x9758, PDF_CMAP_SINGLE, 22851 },
	{ 0x9759, 0x9759, PDF_CMAP_SINGLE, 2665 },
	{ 0x975a, 0x975a, PDF_CMAP_SINGLE, 15275 },
	{ 0x975b, 0x975b, PDF_CMAP_SINGLE, 18915 },
	{ 0x975c, 0x975c, PDF_CMAP_SINGLE, 7145 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 3463 },
	{ 0x9760, 0x9760, PDF_CMAP_SINGLE, 7146 },
	{ 0x9761, 0x9761, PDF_CMAP_SINGLE, 7430 },
	{ 0x9762, 0x9762, PDF_CMAP_SINGLE, 3800 },
	{ 0x9763, 0x9763, PDF_CMAP_SINGLE, 15276 },
	{ 0x9764, 0x9764, PDF_CMAP_SINGLE, 7147 },
	{ 0x9766, 0x9766, PDF_CMAP_SINGLE, 7148 },
	{ 0x9767, 0x9767, PDF_CMAP_SINGLE, 19971 },
	{ 0x9768, 0x9768, PDF_CMAP_SINGLE, 7149 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 1461 },
	{ 0x976a, 0x976a, PDF_CMAP_SINGLE, 18916 },
	{ 0x976b, 0x976b, PDF_CMAP_SINGLE, 7151 },
	{ 0x976d, 0x976d, PDF_CMAP_SINGLE, 2591 },
	{ 0x976e, 0x976e, PDF_CMAP_SINGLE, 15277 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 7152 },
	{ 0x9773, 0x9773, PDF_CMAP_SINGLE, 15278 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 1786 },
	{ 0x9776, 0x9776, PDF_CMAP_SINGLE, 19972 },
	{ 0x9777, 0x9778, PDF_CMAP_RANGE, 22852 },
	{ 0x9779, 0x9779, PDF_CMAP_SINGLE, 7153 },
	{ 0x977a, 0x977a, PDF_CMAP_SINGLE, 7157 },
	{ 0x977b, 0x977b, PDF_CMAP_SINGLE, 22854 },
	{ 0x977c, 0x977c, PDF_CMAP_SINGLE, 7155 },
	{ 0x977d, 0x977d, PDF_CMAP_SINGLE, 19973 },
	{ 0x977f, 0x977f, PDF_CMAP_SINGLE, 19974 },
	{ 0x9780, 0x9780, PDF_CMAP_SINGLE, 22855 },
	{ 0x9781, 0x9781, PDF_CMAP_SINGLE, 7156 },
	{ 0x9784, 0x9784, PDF_CMAP_SINGLE, 1489 },
	{ 0x9785, 0x9785, PDF_CMAP_SINGLE, 7154 },
	{ 0x9786, 0x9786, PDF_CMAP_SINGLE, 7158 },
	{ 0x9789, 0x9789, PDF_CMAP_SINGLE, 22856 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 7159 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 1164 },
	{ 0x978f, 0x9790, PDF_CMAP_RANGE, 7160 },
	{ 0x9795, 0x9795, PDF_CMAP_SINGLE, 17188 },
	{ 0x9796, 0x9796, PDF_CMAP_SINGLE, 18919 },
	{ 0x9797, 0x9797, PDF_CMAP_SINGLE, 22857 },
	{ 0x9798, 0x9798, PDF_CMAP_SINGLE, 2508 },
	{ 0x9799, 0x9799, PDF_CMAP_SINGLE, 19975 },
	{ 0x979a, 0x979a, PDF_CMAP_SINGLE, 15279 },
	{ 0x979c, 0x979c, PDF_CMAP_SINGLE, 7162 },
	{ 0x979e, 0x979e, PDF_CMAP_SINGLE, 18920 },
	{ 0x979f, 0x979f, PDF_CMAP_SINGLE, 19976 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 1633 },
	{ 0x97a2, 0x97a2, PDF_CMAP_SINGLE, 15280 },
	{ 0x97a3, 0x97a3, PDF_CMAP_SINGLE, 7165 },
	{ 0x97a6, 0x97a6, PDF_CMAP_SINGLE, 7164 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 7163 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 6708 },
	{ 0x97ac, 0x97ac, PDF_CMAP_SINGLE, 19977 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 3628 },
	{ 0x97ae, 0x97ae, PDF_CMAP_SINGLE, 17189 },
	{ 0x97b1, 0x97b2, PDF_CMAP_RANGE, 18921 },
	{ 0x97b3, 0x97b4, PDF_CMAP_RANGE, 7166 },
	{ 0x97b5, 0x97b6, PDF_CMAP_RANGE, 15281 },
	{ 0x97b8, 0x97b8, PDF_CMAP_SINGLE, 22858 },
	{ 0x97b9, 0x97b9, PDF_CMAP_SINGLE, 19978 },
	{ 0x97ba, 0x97ba, PDF_CMAP_SINGLE, 17190 },
	{ 0x97bc, 0x97bc, PDF_CMAP_SINGLE, 22859 },
	{ 0x97be, 0x97be, PDF_CMAP_SINGLE, 18923 },
	{ 0x97bf, 0x97bf, PDF_CMAP_SINGLE, 22860 },
	{ 0x97c1, 0x97c1, PDF_CMAP_SINGLE, 17191 },
	{ 0x97c3, 0x97c3, PDF_CMAP_SINGLE, 7168 },
	{ 0x97c4, 0x97c5, PDF_CMAP_RANGE, 22861 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 7169 },
	{ 0x97c7, 0x97c7, PDF_CMAP_SINGLE, 22863 },
	{ 0x97c8, 0x97c8, PDF_CMAP_SINGLE, 7170 },
	{ 0x97c9, 0x97c9, PDF_CMAP_SINGLE, 17192 },
	{ 0x97ca, 0x97ca, PDF_CMAP_SINGLE, 22864 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 7171 },
	{ 0x97cc, 0x97cc, PDF_CMAP_SINGLE, 18924 },
	{ 0x97cd, 0x97cd, PDF_CMAP_SINGLE, 19979 },
	{ 0x97ce, 0x97ce, PDF_CMAP_SINGLE, 22865 },
	{ 0x97d0, 0x97d0, PDF_CMAP_SINGLE, 22866 },
	{ 0x97d1, 0x97d1, PDF_CMAP_SINGLE, 18925 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 1558 },
	{ 0x97d4, 0x97d4, PDF_CMAP_SINGLE, 18926 },
	{ 0x97d7, 0x97d7, PDF_CMAP_SINGLE, 22867 },
	{ 0x97d8, 0x97d8, PDF_CMAP_SINGLE, 18927 },
	{ 0x97d9, 0x97d9, PDF_CMAP_SINGLE, 15283 },
	{ 0x97db, 0x97db, PDF_CMAP_SINGLE, 17193 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 7172 },
	{ 0x97dd, 0x97dd, PDF_CMAP_SINGLE, 22868 },
	{ 0x97de, 0x97de, PDF_CMAP_SINGLE, 15284 },
	{ 0x97e0, 0x97e0, PDF_CMAP_SINGLE, 19980 },
	{ 0x97e1, 0x97e1, PDF_CMAP_SINGLE, 18928 },
	{ 0x97e4, 0x97e4, PDF_CMAP_SINGLE, 22869 },
	{ 0x97ed, 0x97ed, PDF_CMAP_SINGLE, 7173 },
	{ 0x97ee, 0x97ee, PDF_CMAP_SINGLE, 3289 },
	{ 0x97ef, 0x97ef, PDF_CMAP_SINGLE, 19981 },
	{ 0x97f1, 0x97f1, PDF_CMAP_SINGLE, 18929 },
	{ 0x97f2, 0x97f2, PDF_CMAP_SINGLE, 7175 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 1339 },
	{ 0x97f4, 0x97f4, PDF_CMAP_SINGLE, 15285 },
	{ 0x97f5, 0x97f5, PDF_CMAP_SINGLE, 7178 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 7177 },
	{ 0x97f7, 0x97f8, PDF_CMAP_RANGE, 22870 },
	{ 0x97fa, 0x97fa, PDF_CMAP_SINGLE, 22872 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 1222 },
	{ 0x97ff, 0x97ff, PDF_CMAP_SINGLE, 1721 },
	{ 0x9801, 0x9801, PDF_CMAP_SINGLE, 3607 },
	{ 0x9802, 0x9802, PDF_CMAP_SINGLE, 3030 },
	{ 0x9803, 0x9803, PDF_CMAP_SINGLE, 2066 },
	{ 0x9804, 0x9804, PDF_CMAP_SINGLE, 18930 },
	{ 0x9805, 0x9805, PDF_CMAP_SINGLE, 2034 },
	{ 0x9806, 0x9806, PDF_CMAP_SINGLE, 2417 },
	{ 0x9807, 0x9807, PDF_CMAP_SINGLE, 19982 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 2594 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 15286 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 7180 },
	{ 0x980d, 0x980d, PDF_CMAP_SINGLE, 18931 },
	{ 0x980e, 0x980e, PDF_CMAP_SINGLE, 15287 },
	{ 0x980f, 0x980f, PDF_CMAP_SINGLE, 7179 },
	{ 0x9810, 0x9810, PDF_CMAP_SINGLE, 3884 },
	{ 0x9811, 0x9811, PDF_CMAP_SINGLE, 1572 },
	{ 0x9812, 0x9812, PDF_CMAP_SINGLE, 3430 },
	{ 0x9813, 0x9813, PDF_CMAP_SINGLE, 3252 },
	{ 0x9814, 0x9814, PDF_CMAP_SINGLE, 18932 },
	{ 0x9816, 0x9816, PDF_CMAP_SINGLE, 18933 },
	{ 0x9817, 0x9817, PDF_CMAP_SINGLE, 2626 },
	{ 0x9818, 0x9818, PDF_CMAP_SINGLE, 3990 },
	{ 0x9819, 0x9819, PDF_CMAP_SINGLE, 22873 },
	{ 0x981a, 0x981a, PDF_CMAP_SINGLE, 1841 },
	{ 0x981c, 0x981c, PDF_CMAP_SINGLE, 22874 },
	{ 0x981e, 0x981e, PDF_CMAP_SINGLE, 15288 },
	{ 0x9820, 0x9820, PDF_CMAP_SINGLE, 22875 },
	{ 0x9821, 0x9821, PDF_CMAP_SINGLE, 7183 },
	{ 0x9823, 0x9823, PDF_CMAP_SINGLE, 15289 },
	{ 0x9824, 0x9824, PDF_CMAP_SINGLE, 7182 },
	{ 0x9825, 0x9825, PDF_CMAP_SINGLE, 18938 },
	{ 0x9826, 0x9826, PDF_CMAP_SINGLE, 19983 },
	{ 0x982b, 0x982b, PDF_CMAP_SINGLE, 15290 },
	{ 0x982c, 0x982c, PDF_CMAP_SINGLE, 3705 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 3204 },
	{ 0x982e, 0x982e, PDF_CMAP_SINGLE, 19984 },
	{ 0x982f, 0x982f, PDF_CMAP_SINGLE, 22876 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 7795 },
	{ 0x9832, 0x9833, PDF_CMAP_RANGE, 18936 },
	{ 0x9834, 0x9834, PDF_CMAP_SINGLE, 1266 },
	{ 0x9835, 0x9835, PDF_CMAP_SINGLE, 22877 },
	{ 0x9837, 0x9837, PDF_CMAP_SINGLE, 7184 },
	{ 0x9838, 0x9838, PDF_CMAP_SINGLE, 7181 },
	{ 0x9839, 0x9839, PDF_CMAP_SINGLE, 14259 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 3523 },
	{ 0x983c, 0x983c, PDF_CMAP_SINGLE, 3924 },
	{ 0x983d, 0x983d, PDF_CMAP_SINGLE, 7185 },
	{ 0x983e, 0x983e, PDF_CMAP_SINGLE, 15291 },
	{ 0x9844, 0x9844, PDF_CMAP_SINGLE, 22878 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 7186 },
	{ 0x9847, 0x9847, PDF_CMAP_SINGLE, 18939 },
	{ 0x984a, 0x984a, PDF_CMAP_SINGLE, 22879 },
	{ 0x984b, 0x984b, PDF_CMAP_SINGLE, 7188 },
	{ 0x984c, 0x984c, PDF_CMAP_SINGLE, 2890 },
	{ 0x984d, 0x984e, PDF_CMAP_RANGE, 1465 },
	{ 0x984f, 0x984f, PDF_CMAP_SINGLE, 7187 },
	{ 0x9851, 0x9851, PDF_CMAP_SINGLE, 22880 },
	{ 0x9852, 0x9853, PDF_CMAP_RANGE, 15292 },
	{ 0x9854, 0x9854, PDF_CMAP_SINGLE, 1573 },
	{ 0x9855, 0x9855, PDF_CMAP_SINGLE, 1894 },
	{ 0x9856, 0x9856, PDF_CMAP_SINGLE, 17194 },
	{ 0x9857, 0x9857, PDF_CMAP_SINGLE, 8697 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 1574 },
	{ 0x9859, 0x9859, PDF_CMAP_SINGLE, 15294 },
	{ 0x985a, 0x985a, PDF_CMAP_SINGLE, 7752 },
	{ 0x985b, 0x985b, PDF_CMAP_SINGLE, 3129 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 4008 },
	{ 0x9862, 0x9863, PDF_CMAP_RANGE, 19985 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 8698 },
	{ 0x9866, 0x9866, PDF_CMAP_SINGLE, 18940 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 1936 },
	{ 0x986a, 0x986a, PDF_CMAP_SINGLE, 22881 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 7189 },
	{ 0x986c, 0x986c, PDF_CMAP_SINGLE, 15295 },
	{ 0x986f, 0x9871, PDF_CMAP_RANGE, 7190 },
	{ 0x9873, 0x9873, PDF_CMAP_SINGLE, 7194 },
	{ 0x9874, 0x9874, PDF_CMAP_SINGLE, 7193 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 3561 },
	{ 0x98aa, 0x98aa, PDF_CMAP_SINGLE, 7195 },
	{ 0x98ab, 0x98ab, PDF_CMAP_SINGLE, 18941 },
	{ 0x98ad, 0x98ad, PDF_CMAP_SINGLE, 18942 },
	{ 0x98ae, 0x98ae, PDF_CMAP_SINGLE, 22882 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 7196 },
	{ 0x98b0, 0x98b0, PDF_CMAP_SINGLE, 18943 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 7197 },
	{ 0x98b4, 0x98b4, PDF_CMAP_SINGLE, 19987 },
	{ 0x98b6, 0x98b6, PDF_CMAP_SINGLE, 7198 },
	{ 0x98b7, 0x98b7, PDF_CMAP_SINGLE, 18945 },
	{ 0x98b8, 0x98b8, PDF_CMAP_SINGLE, 15296 },
	{ 0x98ba, 0x98ba, PDF_CMAP_SINGLE, 15297 },
	{ 0x98bb, 0x98bc, PDF_CMAP_RANGE, 18946 },
	{ 0x98bf, 0x98bf, PDF_CMAP_SINGLE, 15298 },
	{ 0x98c2, 0x98c2, PDF_CMAP_SINGLE, 18948 },
	{ 0x98c3, 0x98c3, PDF_CMAP_SINGLE, 7200 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 7199 },
	{ 0x98c5, 0x98c5, PDF_CMAP_SINGLE, 19988 },
	{ 0x98c6, 0x98c6, PDF_CMAP_SINGLE, 7201 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 18949 },
	{ 0x98c8, 0x98c8, PDF_CMAP_SINGLE, 15299 },
	{ 0x98cb, 0x98cb, PDF_CMAP_SINGLE, 18950 },
	{ 0x98cc, 0x98cc, PDF_CMAP_SINGLE, 22883 },
	{ 0x98db, 0x98db, PDF_CMAP_SINGLE, 3464 },
	{ 0x98dc, 0x98dc, PDF_CMAP_SINGLE, 6201 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 2543 },
	{ 0x98e0, 0x98e0, PDF_CMAP_SINGLE, 13848 },
	{ 0x98e1, 0x98e1, PDF_CMAP_SINGLE, 18951 },
	{ 0x98e2, 0x98e2, PDF_CMAP_SINGLE, 1612 },
	{ 0x98e3, 0x98e3, PDF_CMAP_SINGLE, 18952 },
	{ 0x98e5, 0x98e5, PDF_CMAP_SINGLE, 15300 },
	{ 0x98e6, 0x98e6, PDF_CMAP_SINGLE, 22884 },
	{ 0x98e7, 0x98e7, PDF_CMAP_SINGLE, 17195 },
	{ 0x98e9, 0x98e9, PDF_CMAP_SINGLE, 7202 },
	{ 0x98ea, 0x98ea, PDF_CMAP_SINGLE, 18953 },
	{ 0x98eb, 0x98eb, PDF_CMAP_SINGLE, 7203 },
	{ 0x98ed, 0x98ed, PDF_CMAP_SINGLE, 4289 },
	{ 0x98ee, 0x98ee, PDF_CMAP_SINGLE, 5338 },
	{ 0x98ef, 0x98ef, PDF_CMAP_SINGLE, 3431 },
	{ 0x98f0, 0x98f1, PDF_CMAP_RANGE, 18954 },
	{ 0x98f2, 0x98f2, PDF_CMAP_SINGLE, 1215 },
	{ 0x98f3, 0x98f3, PDF_CMAP_SINGLE, 18956 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 1151 },
	{ 0x98f6, 0x98f6, PDF_CMAP_SINGLE, 22885 },
	{ 0x98fc, 0x98fc, PDF_CMAP_SINGLE, 2242 },
	{ 0x98fd, 0x98fd, PDF_CMAP_SINGLE, 3678 },
	{ 0x98fe, 0x98fe, PDF_CMAP_SINGLE, 2534 },
	{ 0x9902, 0x9902, PDF_CMAP_SINGLE, 19989 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 7204 },
	{ 0x9905, 0x9905, PDF_CMAP_SINGLE, 3819 },
	{ 0x9907, 0x9907, PDF_CMAP_SINGLE, 22886 },
	{ 0x9908, 0x9908, PDF_CMAP_SINGLE, 18957 },
	{ 0x9909, 0x9909, PDF_CMAP_SINGLE, 7205 },
	{ 0x990a, 0x990a, PDF_CMAP_SINGLE, 3910 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 1252 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 2191 },
	{ 0x9911, 0x9911, PDF_CMAP_SINGLE, 19990 },
	{ 0x9912, 0x9912, PDF_CMAP_SINGLE, 7206 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 1390 },
	{ 0x9914, 0x9914, PDF_CMAP_SINGLE, 7207 },
	{ 0x9915, 0x9915, PDF_CMAP_SINGLE, 19991 },
	{ 0x9916, 0x9917, PDF_CMAP_RANGE, 18960 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 7208 },
	{ 0x991a, 0x991c, PDF_CMAP_RANGE, 18963 },
	{ 0x991d, 0x991e, PDF_CMAP_RANGE, 7210 },
	{ 0x991f, 0x991f, PDF_CMAP_SINGLE, 22887 },
	{ 0x9920, 0x9920, PDF_CMAP_SINGLE, 7213 },
	{ 0x9921, 0x9921, PDF_CMAP_SINGLE, 7209 },
	{ 0x9922, 0x9922, PDF_CMAP_SINGLE, 22888 },
	{ 0x9924, 0x9924, PDF_CMAP_SINGLE, 7212 },
	{ 0x9926, 0x9926, PDF_CMAP_SINGLE, 22889 },
	{ 0x9927, 0x9927, PDF_CMAP_SINGLE, 8701 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 1559 },
	{ 0x992b, 0x992b, PDF_CMAP_SINGLE, 22890 },
	{ 0x992c, 0x992c, PDF_CMAP_SINGLE, 7214 },
	{ 0x992e, 0x992e, PDF_CMAP_SINGLE, 7215 },
	{ 0x9931, 0x9931, PDF_CMAP_SINGLE, 18967 },
	{ 0x9932, 0x9933, PDF_CMAP_RANGE, 15301 },
	{ 0x9934, 0x9934, PDF_CMAP_SINGLE, 22891 },
	{ 0x9935, 0x9935, PDF_CMAP_SINGLE, 19992 },
	{ 0x9939, 0x9939, PDF_CMAP_SINGLE, 22892 },
	{ 0x993a, 0x993c, PDF_CMAP_RANGE, 18968 },
	{ 0x993d, 0x993e, PDF_CMAP_RANGE, 7216 },
	{ 0x9940, 0x9940, PDF_CMAP_SINGLE, 15303 },
	{ 0x9941, 0x9941, PDF_CMAP_SINGLE, 18971 },
	{ 0x9942, 0x9942, PDF_CMAP_SINGLE, 7218 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 7220 },
	{ 0x9946, 0x9946, PDF_CMAP_SINGLE, 18972 },
	{ 0x9947, 0x9947, PDF_CMAP_SINGLE, 22893 },
	{ 0x9948, 0x9948, PDF_CMAP_SINGLE, 19993 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 7219 },
	{ 0x994b, 0x994b, PDF_CMAP_SINGLE, 7222 },
	{ 0x994c, 0x994c, PDF_CMAP_SINGLE, 7225 },
	{ 0x994d, 0x994d, PDF_CMAP_SINGLE, 15304 },
	{ 0x994e, 0x994e, PDF_CMAP_SINGLE, 18973 },
	{ 0x9950, 0x9950, PDF_CMAP_SINGLE, 7221 },
	{ 0x9951, 0x9952, PDF_CMAP_RANGE, 7223 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 19994 },
	{ 0x9955, 0x9955, PDF_CMAP_SINGLE, 7226 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 1722 },
	{ 0x9958, 0x9958, PDF_CMAP_SINGLE, 17196 },
	{ 0x9959, 0x9959, PDF_CMAP_SINGLE, 22894 },
	{ 0x995b, 0x995b, PDF_CMAP_SINGLE, 22895 },
	{ 0x995c, 0x995c, PDF_CMAP_SINGLE, 15305 },
	{ 0x995e, 0x995e, PDF_CMAP_SINGLE, 19995 },
	{ 0x995f, 0x995f, PDF_CMAP_SINGLE, 15306 },
	{ 0x9960, 0x9960, PDF_CMAP_SINGLE, 18974 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 2335 },
	{ 0x9997, 0x9998, PDF_CMAP_RANGE, 7227 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 2035 },
	{ 0x999b, 0x999b, PDF_CMAP_SINGLE, 22896 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 22897 },
	{ 0x999e, 0x999e, PDF_CMAP_SINGLE, 8703 },
	{ 0x999f, 0x999f, PDF_CMAP_SINGLE, 22898 },
	{ 0x99a3, 0x99a3, PDF_CMAP_SINGLE, 18975 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 7229 },
	{ 0x99a6, 0x99a6, PDF_CMAP_SINGLE, 18976 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 1436 },
	{ 0x99ac, 0x99ac, PDF_CMAP_SINGLE, 3333 },
	{ 0x99ad, 0x99ae, PDF_CMAP_RANGE, 7230 },
	{ 0x99b0, 0x99b0, PDF_CMAP_SINGLE, 22899 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 15307 },
	{ 0x99b2, 0x99b2, PDF_CMAP_SINGLE, 22900 },
	{ 0x99b3, 0x99b3, PDF_CMAP_SINGLE, 2968 },
	{ 0x99b4, 0x99b4, PDF_CMAP_SINGLE, 3267 },
	{ 0x99b5, 0x99b5, PDF_CMAP_SINGLE, 22901 },
	{ 0x99b9, 0x99ba, PDF_CMAP_RANGE, 15308 },
	{ 0x99bc, 0x99bc, PDF_CMAP_SINGLE, 7232 },
	{ 0x99bd, 0x99bd, PDF_CMAP_SINGLE, 18977 },
	{ 0x99bf, 0x99bf, PDF_CMAP_SINGLE, 18978 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 3379 },
	{ 0x99c3, 0x99c3, PDF_CMAP_SINGLE, 18979 },
	{ 0x99c4, 0x99c4, PDF_CMAP_SINGLE, 2860 },
	{ 0x99c5, 0x99c5, PDF_CMAP_SINGLE, 1274 },
	{ 0x99c6, 0x99c6, PDF_CMAP_SINGLE, 1766 },
	{ 0x99c8, 0x99c8, PDF_CMAP_SINGLE, 1767 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 15310 },
	{ 0x99d0, 0x99d0, PDF_CMAP_SINGLE, 2993 },
	{ 0x99d1, 0x99d1, PDF_CMAP_SINGLE, 7237 },
	{ 0x99d2, 0x99d2, PDF_CMAP_SINGLE, 1768 },
	{ 0x99d3, 0x99d3, PDF_CMAP_SINGLE, 22902 },
	{ 0x99d4, 0x99d4, PDF_CMAP_SINGLE, 18980 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 1391 },
	{ 0x99d8, 0x99d8, PDF_CMAP_SINGLE, 7236 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 18981 },
	{ 0x99da, 0x99da, PDF_CMAP_SINGLE, 22903 },
	{ 0x99db, 0x99db, PDF_CMAP_SINGLE, 7234 },
	{ 0x99dc, 0x99dc, PDF_CMAP_SINGLE, 22904 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 7235 },
	{ 0x99de, 0x99de, PDF_CMAP_SINGLE, 18982 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 7233 },
	{ 0x99e1, 0x99e1, PDF_CMAP_SINGLE, 19996 },
	{ 0x99e2, 0x99e2, PDF_CMAP_SINGLE, 7247 },
	{ 0x99e7, 0x99e7, PDF_CMAP_SINGLE, 22905 },
	{ 0x99ea, 0x99ec, PDF_CMAP_RANGE, 22906 },
	{ 0x99ed, 0x99ee, PDF_CMAP_RANGE, 7238 },
	{ 0x99f0, 0x99f0, PDF_CMAP_SINGLE, 18984 },
	{ 0x99f1, 0x99f2, PDF_CMAP_RANGE, 7240 },
	{ 0x99f4, 0x99f5, PDF_CMAP_RANGE, 22909 },
	{ 0x99f8, 0x99f8, PDF_CMAP_SINGLE, 7243 },
	{ 0x99f9, 0x99f9, PDF_CMAP_SINGLE, 18985 },
	{ 0x99fb, 0x99fb, PDF_CMAP_SINGLE, 7242 },
	{ 0x99fc, 0x99fc, PDF_CMAP_SINGLE, 18986 },
	{ 0x99fd, 0x99fe, PDF_CMAP_RANGE, 22911 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 2403 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 7244 },
	{ 0x9a02, 0x9a02, PDF_CMAP_SINGLE, 15311 },
	{ 0x9a03, 0x9a03, PDF_CMAP_SINGLE, 17197 },
	{ 0x9a04, 0x9a04, PDF_CMAP_SINGLE, 22913 },
	{ 0x9a05, 0x9a05, PDF_CMAP_SINGLE, 7246 },
	{ 0x9a08, 0x9a08, PDF_CMAP_SINGLE, 14266 },
	{ 0x9a0a, 0x9a0a, PDF_CMAP_SINGLE, 18987 },
	{ 0x9a0b, 0x9a0b, PDF_CMAP_SINGLE, 22914 },
	{ 0x9a0c, 0x9a0c, PDF_CMAP_SINGLE, 19997 },
	{ 0x9a0e, 0x9a0e, PDF_CMAP_SINGLE, 1613 },
	{ 0x9a0f, 0x9a0f, PDF_CMAP_SINGLE, 7245 },
	{ 0x9a10, 0x9a10, PDF_CMAP_SINGLE, 19998 },
	{ 0x9a11, 0x9a11, PDF_CMAP_SINGLE, 18988 },
	{ 0x9a12, 0x9a12, PDF_CMAP_SINGLE, 2813 },
	{ 0x9a13, 0x9a13, PDF_CMAP_SINGLE, 1895 },
	{ 0x9a16, 0x9a16, PDF_CMAP_SINGLE, 15312 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 7248 },
	{ 0x9a1a, 0x9a1a, PDF_CMAP_SINGLE, 18989 },
	{ 0x9a1e, 0x9a1e, PDF_CMAP_SINGLE, 22915 },
	{ 0x9a20, 0x9a20, PDF_CMAP_SINGLE, 18990 },
	{ 0x9a22, 0x9a22, PDF_CMAP_SINGLE, 22916 },
	{ 0x9a23, 0x9a23, PDF_CMAP_SINGLE, 19999 },
	{ 0x9a24, 0x9a24, PDF_CMAP_SINGLE, 15313 },
	{ 0x9a27, 0x9a27, PDF_CMAP_SINGLE, 15314 },
	{ 0x9a28, 0x9a28, PDF_CMAP_SINGLE, 2861 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 7249 },
	{ 0x9a2d, 0x9a2e, PDF_CMAP_RANGE, 15315 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 3205 },
	{ 0x9a31, 0x9a31, PDF_CMAP_SINGLE, 18991 },
	{ 0x9a33, 0x9a33, PDF_CMAP_SINGLE, 22917 },
	{ 0x9a35, 0x9a35, PDF_CMAP_SINGLE, 22918 },
	{ 0x9a36, 0x9a36, PDF_CMAP_SINGLE, 15317 },
	{ 0x9a37, 0x9a37, PDF_CMAP_SINGLE, 7250 },
	{ 0x9a38, 0x9a38, PDF_CMAP_SINGLE, 15318 },
	{ 0x9a3e, 0x9a3e, PDF_CMAP_SINGLE, 7255 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 7253 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 20000 },
	{ 0x9a42, 0x9a42, PDF_CMAP_SINGLE, 7252 },
	{ 0x9a43, 0x9a43, PDF_CMAP_SINGLE, 7254 },
	{ 0x9a44, 0x9a44, PDF_CMAP_SINGLE, 18992 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 7251 },
	{ 0x9a47, 0x9a47, PDF_CMAP_SINGLE, 22919 },
	{ 0x9a4a, 0x9a4a, PDF_CMAP_SINGLE, 15319 },
	{ 0x9a4b, 0x9a4b, PDF_CMAP_SINGLE, 22921 },
	{ 0x9a4c, 0x9a4c, PDF_CMAP_SINGLE, 18993 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 7257 },
	{ 0x9a4e, 0x9a4e, PDF_CMAP_SINGLE, 8704 },
	{ 0x9a51, 0x9a51, PDF_CMAP_SINGLE, 20001 },
	{ 0x9a52, 0x9a52, PDF_CMAP_SINGLE, 7727 },
	{ 0x9a54, 0x9a54, PDF_CMAP_SINGLE, 22922 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 7256 },
	{ 0x9a56, 0x9a56, PDF_CMAP_SINGLE, 15320 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 7259 },
	{ 0x9a58, 0x9a58, PDF_CMAP_SINGLE, 18994 },
	{ 0x9a5a, 0x9a5a, PDF_CMAP_SINGLE, 1723 },
	{ 0x9a5b, 0x9a5b, PDF_CMAP_SINGLE, 7258 },
	{ 0x9a5d, 0x9a5d, PDF_CMAP_SINGLE, 22923 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 7260 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 7261 },
	{ 0x9a64, 0x9a64, PDF_CMAP_SINGLE, 7263 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 7262 },
	{ 0x9a69, 0x9a69, PDF_CMAP_SINGLE, 7264 },
	{ 0x9a6a, 0x9a6a, PDF_CMAP_SINGLE, 7266 },
	{ 0x9a6b, 0x9a6b, PDF_CMAP_SINGLE, 7265 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 2062 },
	{ 0x9aaa, 0x9aaa, PDF_CMAP_SINGLE, 22924 },
	{ 0x9aac, 0x9aac, PDF_CMAP_SINGLE, 22925 },
	{ 0x9aad, 0x9aad, PDF_CMAP_SINGLE, 7267 },
	{ 0x9aae, 0x9aae, PDF_CMAP_SINGLE, 22926 },
	{ 0x9aaf, 0x9aaf, PDF_CMAP_SINGLE, 18996 },
	{ 0x9ab0, 0x9ab0, PDF_CMAP_SINGLE, 7268 },
	{ 0x9ab2, 0x9ab2, PDF_CMAP_SINGLE, 22927 },
	{ 0x9ab4, 0x9ab4, PDF_CMAP_SINGLE, 22928 },
	{ 0x9ab5, 0x9ab6, PDF_CMAP_RANGE, 15321 },
	{ 0x9ab7, 0x9ab7, PDF_CMAP_SINGLE, 18998 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 1434 },
	{ 0x9ab9, 0x9ab9, PDF_CMAP_SINGLE, 19000 },
	{ 0x9abb, 0x9abb, PDF_CMAP_SINGLE, 22929 },
	{ 0x9abc, 0x9abc, PDF_CMAP_SINGLE, 7269 },
	{ 0x9abd, 0x9abe, PDF_CMAP_RANGE, 20002 },
	{ 0x9abf, 0x9abf, PDF_CMAP_SINGLE, 22930 },
	{ 0x9ac0, 0x9ac0, PDF_CMAP_SINGLE, 7270 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 17198 },
	{ 0x9ac3, 0x9ac3, PDF_CMAP_SINGLE, 17199 },
	{ 0x9ac4, 0x9ac4, PDF_CMAP_SINGLE, 2615 },
	{ 0x9ac6, 0x9ac6, PDF_CMAP_SINGLE, 19002 },
	{ 0x9ac8, 0x9ac8, PDF_CMAP_SINGLE, 22931 },
	{ 0x9ace, 0x9ace, PDF_CMAP_SINGLE, 17200 },
	{ 0x9acf, 0x9acf, PDF_CMAP_SINGLE, 7271 },
	{ 0x9ad0, 0x9ad0, PDF_CMAP_SINGLE, 19003 },
	{ 0x9ad1, 0x9ad1, PDF_CMAP_SINGLE, 7272 },
	{ 0x9ad2, 0x9ad2, PDF_CMAP_SINGLE, 19004 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_RANGE, 7273 },
	{ 0x9ad5, 0x9ad5, PDF_CMAP_SINGLE, 19005 },
	{ 0x9ad6, 0x9ad6, PDF_CMAP_SINGLE, 17201 },
	{ 0x9ad7, 0x9ad7, PDF_CMAP_SINGLE, 22932 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 2036 },
	{ 0x9ad9, 0x9ad9, PDF_CMAP_SINGLE, 8705 },
	{ 0x9adb, 0x9adb, PDF_CMAP_SINGLE, 22933 },
	{ 0x9adc, 0x9adc, PDF_CMAP_SINGLE, 8706 },
	{ 0x9ade, 0x9adf, PDF_CMAP_RANGE, 7275 },
	{ 0x9ae0, 0x9ae0, PDF_CMAP_SINGLE, 19006 },
	{ 0x9ae2, 0x9ae3, PDF_CMAP_RANGE, 7277 },
	{ 0x9ae4, 0x9ae4, PDF_CMAP_SINGLE, 22934 },
	{ 0x9ae5, 0x9ae5, PDF_CMAP_SINGLE, 19007 },
	{ 0x9ae6, 0x9ae6, PDF_CMAP_SINGLE, 7279 },
	{ 0x9ae7, 0x9ae7, PDF_CMAP_SINGLE, 22935 },
	{ 0x9ae9, 0x9ae9, PDF_CMAP_SINGLE, 19008 },
	{ 0x9aea, 0x9aea, PDF_CMAP_SINGLE, 3397 },
	{ 0x9aeb, 0x9aeb, PDF_CMAP_SINGLE, 7281 },
	{ 0x9aec, 0x9aec, PDF_CMAP_SINGLE, 22936 },
	{ 0x9aed, 0x9aed, PDF_CMAP_SINGLE, 3480 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 7282 },
	{ 0x9aef, 0x9aef, PDF_CMAP_SINGLE, 7280 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 7284 },
	{ 0x9af2, 0x9af3, PDF_CMAP_RANGE, 22937 },
	{ 0x9af4, 0x9af4, PDF_CMAP_SINGLE, 7283 },
	{ 0x9af5, 0x9af5, PDF_CMAP_SINGLE, 22939 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 7285 },
	{ 0x9af9, 0x9af9, PDF_CMAP_SINGLE, 15323 },
	{ 0x9afa, 0x9afa, PDF_CMAP_SINGLE, 22940 },
	{ 0x9afb, 0x9afb, PDF_CMAP_SINGLE, 7286 },
	{ 0x9afd, 0x9afd, PDF_CMAP_SINGLE, 22941 },
	{ 0x9aff, 0x9aff, PDF_CMAP_SINGLE, 22942 },
	{ 0x9b00, 0x9b00, PDF_CMAP_SINGLE, 22943 },
	{ 0x9b01, 0x9b01, PDF_CMAP_SINGLE, 20004 },
	{ 0x9b02, 0x9b02, PDF_CMAP_SINGLE, 17202 },
	{ 0x9b03, 0x9b03, PDF_CMAP_SINGLE, 15324 },
	{ 0x9b04, 0x9b05, PDF_CMAP_RANGE, 22944 },
	{ 0x9b06, 0x9b06, PDF_CMAP_SINGLE, 7287 },
	{ 0x9b08, 0x9b08, PDF_CMAP_SINGLE, 17203 },
	{ 0x9b09, 0x9b09, PDF_CMAP_SINGLE, 20005 },
	{ 0x9b0b, 0x9b0b, PDF_CMAP_SINGLE, 20006 },
	{ 0x9b0c, 0x9b0c, PDF_CMAP_SINGLE, 19009 },
	{ 0x9b0d, 0x9b0e, PDF_CMAP_RANGE, 20007 },
	{ 0x9b10, 0x9b10, PDF_CMAP_SINGLE, 19010 },
	{ 0x9b12, 0x9b12, PDF_CMAP_SINGLE, 19011 },
	{ 0x9b16, 0x9b16, PDF_CMAP_SINGLE, 19012 },
	{ 0x9b18, 0x9b18, PDF_CMAP_SINGLE, 7288 },
	{ 0x9b19, 0x9b19, PDF_CMAP_SINGLE, 20009 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 7289 },
	{ 0x9b1b, 0x9b1b, PDF_CMAP_SINGLE, 22946 },
	{ 0x9b1c, 0x9b1c, PDF_CMAP_SINGLE, 19013 },
	{ 0x9b1d, 0x9b1d, PDF_CMAP_SINGLE, 20297 },
	{ 0x9b1f, 0x9b1f, PDF_CMAP_SINGLE, 7290 },
	{ 0x9b20, 0x9b20, PDF_CMAP_SINGLE, 15325 },
	{ 0x9b22, 0x9b23, PDF_CMAP_RANGE, 7291 },
	{ 0x9b25, 0x9b25, PDF_CMAP_SINGLE, 7293 },
	{ 0x9b26, 0x9b26, PDF_CMAP_SINGLE, 22947 },
	{ 0x9b27, 0x9b2a, PDF_CMAP_RANGE, 7294 },
	{ 0x9b2b, 0x9b2b, PDF_CMAP_SINGLE, 19014 },
	{ 0x9b2c, 0x9b2c, PDF_CMAP_SINGLE, 20298 },
	{ 0x9b2d, 0x9b2d, PDF_CMAP_SINGLE, 17205 },
	{ 0x9b2e, 0x9b2f, PDF_CMAP_RANGE, 7298 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 5332 },
	{ 0x9b32, 0x9b32, PDF_CMAP_SINGLE, 7300 },
	{ 0x9b33, 0x9b34, PDF_CMAP_RANGE, 15326 },
	{ 0x9b35, 0x9b35, PDF_CMAP_SINGLE, 20010 },
	{ 0x9b37, 0x9b37, PDF_CMAP_SINGLE, 22949 },
	{ 0x9b39, 0x9b3a, PDF_CMAP_RANGE, 22950 },
	{ 0x9b3b, 0x9b3b, PDF_CMAP_SINGLE, 6057 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 1614 },
	{ 0x9b3d, 0x9b3d, PDF_CMAP_SINGLE, 19015 },
	{ 0x9b41, 0x9b41, PDF_CMAP_SINGLE, 1407 },
	{ 0x9b42, 0x9b42, PDF_CMAP_SINGLE, 2082 },
	{ 0x9b43, 0x9b43, PDF_CMAP_SINGLE, 7302 },
	{ 0x9b44, 0x9b44, PDF_CMAP_SINGLE, 7301 },
	{ 0x9b45, 0x9b45, PDF_CMAP_SINGLE, 3761 },
	{ 0x9b48, 0x9b48, PDF_CMAP_SINGLE, 20011 },
	{ 0x9b4b, 0x9b4b, PDF_CMAP_SINGLE, 19017 },
	{ 0x9b4c, 0x9b4c, PDF_CMAP_SINGLE, 22952 },
	{ 0x9b4d, 0x9b4e, PDF_CMAP_RANGE, 7304 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 7303 },
	{ 0x9b51, 0x9b51, PDF_CMAP_SINGLE, 7306 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 3728 },
	{ 0x9b55, 0x9b55, PDF_CMAP_SINGLE, 20012 },
	{ 0x9b56, 0x9b57, PDF_CMAP_RANGE, 22953 },
	{ 0x9b58, 0x9b58, PDF_CMAP_SINGLE, 7307 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 1685 },
	{ 0x9b5b, 0x9b5b, PDF_CMAP_SINGLE, 22955 },
	{ 0x9b5e, 0x9b5e, PDF_CMAP_SINGLE, 17206 },
	{ 0x9b61, 0x9b61, PDF_CMAP_SINGLE, 22956 },
	{ 0x9b63, 0x9b63, PDF_CMAP_SINGLE, 19018 },
	{ 0x9b65, 0x9b65, PDF_CMAP_SINGLE, 19019 },
	{ 0x9b66, 0x9b66, PDF_CMAP_SINGLE, 17207 },
	{ 0x9b68, 0x9b68, PDF_CMAP_SINGLE, 20013 },
	{ 0x9b6a, 0x9b6a, PDF_CMAP_SINGLE, 22957 },
	{ 0x9b6b, 0x9b6c, PDF_CMAP_RANGE, 19020 },
	{ 0x9b6d, 0x9b6e, PDF_CMAP_RANGE, 22958 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4043 },
	{ 0x9b72, 0x9b72, PDF_CMAP_SINGLE, 8708 },
	{ 0x9b73, 0x9b73, PDF_CMAP_SINGLE, 15328 },
	{ 0x9b74, 0x9b74, PDF_CMAP_SINGLE, 7308 },
	{ 0x9b75, 0x9b75, PDF_CMAP_SINGLE, 8707 },
	{ 0x9b76, 0x9b77, PDF_CMAP_RANGE, 19022 },
	{ 0x9b78, 0x9b78, PDF_CMAP_SINGLE, 22960 },
	{ 0x9b79, 0x9b79, PDF_CMAP_SINGLE, 15329 },
	{ 0x9b7f, 0x9b7f, PDF_CMAP_SINGLE, 22961 },
	{ 0x9b80, 0x9b80, PDF_CMAP_SINGLE, 20014 },
	{ 0x9b83, 0x9b83, PDF_CMAP_SINGLE, 7310 },
	{ 0x9b84, 0x9b84, PDF_CMAP_SINGLE, 17208 },
	{ 0x9b85, 0x9b85, PDF_CMAP_SINGLE, 22962 },
	{ 0x9b86, 0x9b86, PDF_CMAP_SINGLE, 20015 },
	{ 0x9b87, 0x9b87, PDF_CMAP_SINGLE, 22963 },
	{ 0x9b89, 0x9b89, PDF_CMAP_SINGLE, 22964 },
	{ 0x9b8a, 0x9b8a, PDF_CMAP_SINGLE, 17209 },
	{ 0x9b8b, 0x9b8b, PDF_CMAP_SINGLE, 22965 },
	{ 0x9b8d, 0x9b8d, PDF_CMAP_SINGLE, 22966 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 1154 },
	{ 0x9b8f, 0x9b8f, PDF_CMAP_SINGLE, 8709 },
	{ 0x9b90, 0x9b90, PDF_CMAP_SINGLE, 20016 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 7311 },
	{ 0x9b92, 0x9b92, PDF_CMAP_SINGLE, 3579 },
	{ 0x9b93, 0x9b93, PDF_CMAP_SINGLE, 7309 },
	{ 0x9b94, 0x9b94, PDF_CMAP_SINGLE, 22967 },
	{ 0x9b96, 0x9b97, PDF_CMAP_RANGE, 7312 },
	{ 0x9b9a, 0x9b9a, PDF_CMAP_SINGLE, 22968 },
	{ 0x9b9d, 0x9b9d, PDF_CMAP_SINGLE, 20017 },
	{ 0x9b9e, 0x9b9e, PDF_CMAP_SINGLE, 17210 },
	{ 0x9b9f, 0x9ba0, PDF_CMAP_RANGE, 7314 },
	{ 0x9ba6, 0x9ba6, PDF_CMAP_SINGLE, 19024 },
	{ 0x9ba7, 0x9ba7, PDF_CMAP_SINGLE, 15330 },
	{ 0x9ba8, 0x9ba8, PDF_CMAP_SINGLE, 7316 },
	{ 0x9ba9, 0x9ba9, PDF_CMAP_SINGLE, 22969 },
	{ 0x9baa, 0x9baa, PDF_CMAP_SINGLE, 3740 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 2171 },
	{ 0x9bac, 0x9bac, PDF_CMAP_SINGLE, 19025 },
	{ 0x9bad, 0x9bad, PDF_CMAP_SINGLE, 2154 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 2737 },
	{ 0x9bb0, 0x9bb0, PDF_CMAP_SINGLE, 20018 },
	{ 0x9bb1, 0x9bb1, PDF_CMAP_SINGLE, 8710 },
	{ 0x9bb2, 0x9bb2, PDF_CMAP_SINGLE, 19027 },
	{ 0x9bb4, 0x9bb4, PDF_CMAP_SINGLE, 7317 },
	{ 0x9bb7, 0x9bb7, PDF_CMAP_SINGLE, 22970 },
	{ 0x9bb8, 0x9bb8, PDF_CMAP_SINGLE, 19028 },
	{ 0x9bb9, 0x9bb9, PDF_CMAP_SINGLE, 7320 },
	{ 0x9bbb, 0x9bbb, PDF_CMAP_SINGLE, 8711 },
	{ 0x9bbc, 0x9bbc, PDF_CMAP_SINGLE, 22971 },
	{ 0x9bbe, 0x9bbe, PDF_CMAP_SINGLE, 19029 },
	{ 0x9bbf, 0x9bbf, PDF_CMAP_SINGLE, 20019 },
	{ 0x9bc0, 0x9bc0, PDF_CMAP_SINGLE, 7318 },
	{ 0x9bc1, 0x9bc1, PDF_CMAP_SINGLE, 15331 },
	{ 0x9bc6, 0x9bc6, PDF_CMAP_SINGLE, 7321 },
	{ 0x9bc7, 0x9bc7, PDF_CMAP_SINGLE, 15332 },
	{ 0x9bc8, 0x9bc8, PDF_CMAP_SINGLE, 20020 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 1957 },
	{ 0x9bca, 0x9bca, PDF_CMAP_SINGLE, 7319 },
	{ 0x9bce, 0x9bce, PDF_CMAP_SINGLE, 17211 },
	{ 0x9bcf, 0x9bcf, PDF_CMAP_SINGLE, 7322 },
	{ 0x9bd0, 0x9bd0, PDF_CMAP_SINGLE, 22972 },
	{ 0x9bd1, 0x9bd2, PDF_CMAP_RANGE, 7323 },
	{ 0x9bd4, 0x9bd4, PDF_CMAP_SINGLE, 7328 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 2168 },
	{ 0x9bd7, 0x9bd7, PDF_CMAP_SINGLE, 15333 },
	{ 0x9bd8, 0x9bd8, PDF_CMAP_SINGLE, 19031 },
	{ 0x9bdb, 0x9bdb, PDF_CMAP_SINGLE, 2884 },
	{ 0x9bdd, 0x9bdd, PDF_CMAP_SINGLE, 19032 },
	{ 0x9bdf, 0x9bdf, PDF_CMAP_SINGLE, 14271 },
	{ 0x9be1, 0x9be1, PDF_CMAP_SINGLE, 7329 },
	{ 0x9be2, 0x9be2, PDF_CMAP_SINGLE, 7326 },
	{ 0x9be3, 0x9be3, PDF_CMAP_SINGLE, 7325 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 7327 },
	{ 0x9be5, 0x9be5, PDF_CMAP_SINGLE, 17212 },
	{ 0x9be7, 0x9be7, PDF_CMAP_SINGLE, 15334 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 1845 },
	{ 0x9bea, 0x9bea, PDF_CMAP_SINGLE, 19033 },
	{ 0x9beb, 0x9beb, PDF_CMAP_SINGLE, 15335 },
	{ 0x9bee, 0x9bee, PDF_CMAP_SINGLE, 19035 },
	{ 0x9bef, 0x9bef, PDF_CMAP_SINGLE, 19034 },
	{ 0x9bf0, 0x9bf0, PDF_CMAP_SINGLE, 7333 },
	{ 0x9bf1, 0x9bf1, PDF_CMAP_SINGLE, 7332 },
	{ 0x9bf2, 0x9bf2, PDF_CMAP_SINGLE, 7331 },
	{ 0x9bf3, 0x9bf3, PDF_CMAP_SINGLE, 19030 },
	{ 0x9bf5, 0x9bf5, PDF_CMAP_SINGLE, 1143 },
	{ 0x9bf7, 0x9bf7, PDF_CMAP_SINGLE, 15336 },
	{ 0x9bf8, 0x9bf8, PDF_CMAP_SINGLE, 17213 },
	{ 0x9bf9, 0x9bf9, PDF_CMAP_SINGLE, 22973 },
	{ 0x9bfa, 0x9bfa, PDF_CMAP_SINGLE, 15337 },
	{ 0x9bfd, 0x9bfd, PDF_CMAP_SINGLE, 15338 },
	{ 0x9bff, 0x9bff, PDF_CMAP_SINGLE, 20021 },
	{ 0x9c00, 0x9c00, PDF_CMAP_SINGLE, 8712 },
	{ 0x9c02, 0x9c02, PDF_CMAP_SINGLE, 20022 },
	{ 0x9c04, 0x9c04, PDF_CMAP_SINGLE, 7343 },
	{ 0x9c06, 0x9c06, PDF_CMAP_SINGLE, 7339 },
	{ 0x9c08, 0x9c08, PDF_CMAP_SINGLE, 7340 },
	{ 0x9c09, 0x9c09, PDF_CMAP_SINGLE, 7336 },
	{ 0x9c0a, 0x9c0a, PDF_CMAP_SINGLE, 7342 },
	{ 0x9c0b, 0x9c0b, PDF_CMAP_SINGLE, 15339 },
	{ 0x9c0c, 0x9c0c, PDF_CMAP_SINGLE, 7338 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 1472 },
	{ 0x9c0f, 0x9c0f, PDF_CMAP_SINGLE, 22974 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 4082 },
	{ 0x9c11, 0x9c11, PDF_CMAP_SINGLE, 22975 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 7341 },
	{ 0x9c13, 0x9c13, PDF_CMAP_SINGLE, 7337 },
	{ 0x9c14, 0x9c14, PDF_CMAP_SINGLE, 7335 },
	{ 0x9c15, 0x9c15, PDF_CMAP_SINGLE, 7334 },
	{ 0x9c16, 0x9c16, PDF_CMAP_SINGLE, 19039 },
	{ 0x9c18, 0x9c1a, PDF_CMAP_RANGE, 19040 },
	{ 0x9c1b, 0x9c1b, PDF_CMAP_SINGLE, 7345 },
	{ 0x9c1c, 0x9c1c, PDF_CMAP_SINGLE, 20023 },
	{ 0x9c1d, 0x9c1d, PDF_CMAP_SINGLE, 19043 },
	{ 0x9c1e, 0x9c1e, PDF_CMAP_SINGLE, 22976 },
	{ 0x9c21, 0x9c21, PDF_CMAP_SINGLE, 7348 },
	{ 0x9c22, 0x9c22, PDF_CMAP_SINGLE, 19044 },
	{ 0x9c23, 0x9c23, PDF_CMAP_SINGLE, 17214 },
	{ 0x9c24, 0x9c24, PDF_CMAP_SINGLE, 7347 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 7346 },
	{ 0x9c26, 0x9c26, PDF_CMAP_SINGLE, 22977 },
	{ 0x9c27, 0x9c27, PDF_CMAP_SINGLE, 15340 },
	{ 0x9c28, 0x9c28, PDF_CMAP_SINGLE, 22978 },
	{ 0x9c29, 0x9c29, PDF_CMAP_SINGLE, 19045 },
	{ 0x9c2a, 0x9c2a, PDF_CMAP_SINGLE, 15341 },
	{ 0x9c2d, 0x9c2d, PDF_CMAP_SINGLE, 3515 },
	{ 0x9c2e, 0x9c2e, PDF_CMAP_SINGLE, 7344 },
	{ 0x9c2f, 0x9c2f, PDF_CMAP_SINGLE, 1207 },
	{ 0x9c30, 0x9c30, PDF_CMAP_SINGLE, 7349 },
	{ 0x9c31, 0x9c31, PDF_CMAP_SINGLE, 19047 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 7351 },
	{ 0x9c35, 0x9c35, PDF_CMAP_SINGLE, 20024 },
	{ 0x9c36, 0x9c36, PDF_CMAP_SINGLE, 15342 },
	{ 0x9c37, 0x9c37, PDF_CMAP_SINGLE, 19048 },
	{ 0x9c39, 0x9c39, PDF_CMAP_SINGLE, 1485 },
	{ 0x9c3a, 0x9c3a, PDF_CMAP_SINGLE, 7330 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 1241 },
	{ 0x9c3d, 0x9c3d, PDF_CMAP_SINGLE, 22979 },
	{ 0x9c3e, 0x9c3e, PDF_CMAP_SINGLE, 7353 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 15343 },
	{ 0x9c43, 0x9c43, PDF_CMAP_SINGLE, 22980 },
	{ 0x9c44, 0x9c44, PDF_CMAP_SINGLE, 20025 },
	{ 0x9c45, 0x9c45, PDF_CMAP_SINGLE, 19049 },
	{ 0x9c46, 0x9c46, PDF_CMAP_SINGLE, 7352 },
	{ 0x9c47, 0x9c47, PDF_CMAP_SINGLE, 7350 },
	{ 0x9c48, 0x9c48, PDF_CMAP_SINGLE, 2923 },
	{ 0x9c49, 0x9c4a, PDF_CMAP_RANGE, 19052 },
	{ 0x9c4e, 0x9c4e, PDF_CMAP_SINGLE, 22981 },
	{ 0x9c4f, 0x9c50, PDF_CMAP_RANGE, 17215 },
	{ 0x9c52, 0x9c52, PDF_CMAP_SINGLE, 3742 },
	{ 0x9c53, 0x9c53, PDF_CMAP_SINGLE, 15344 },
	{ 0x9c54, 0x9c54, PDF_CMAP_SINGLE, 19055 },
	{ 0x9c56, 0x9c56, PDF_CMAP_SINGLE, 20026 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 4002 },
	{ 0x9c58, 0x9c58, PDF_CMAP_SINGLE, 19056 },
	{ 0x9c5a, 0x9c5a, PDF_CMAP_SINGLE, 7354 },
	{ 0x9c5b, 0x9c5b, PDF_CMAP_SINGLE, 19057 },
	{ 0x9c5c, 0x9c5c, PDF_CMAP_SINGLE, 19050 },
	{ 0x9c5d, 0x9c5d, PDF_CMAP_SINGLE, 19058 },
	{ 0x9c5e, 0x9c5e, PDF_CMAP_SINGLE, 22982 },
	{ 0x9c5f, 0x9c5f, PDF_CMAP_SINGLE, 19059 },
	{ 0x9c60, 0x9c60, PDF_CMAP_SINGLE, 7355 },
	{ 0x9c61, 0x9c61, PDF_CMAP_SINGLE, 20027 },
	{ 0x9c63, 0x9c63, PDF_CMAP_SINGLE, 15345 },
	{ 0x9c65, 0x9c65, PDF_CMAP_SINGLE, 17217 },
	{ 0x9c67, 0x9c67, PDF_CMAP_SINGLE, 7356 },
	{ 0x9c68, 0x9c68, PDF_CMAP_SINGLE, 20028 },
	{ 0x9c69, 0x9c6b, PDF_CMAP_RANGE, 19060 },
	{ 0x9c6d, 0x9c6e, PDF_CMAP_RANGE, 19063 },
	{ 0x9c70, 0x9c70, PDF_CMAP_SINGLE, 15346 },
	{ 0x9c72, 0x9c72, PDF_CMAP_SINGLE, 19065 },
	{ 0x9c75, 0x9c75, PDF_CMAP_SINGLE, 19066 },
	{ 0x9c76, 0x9c76, PDF_CMAP_SINGLE, 7357 },
	{ 0x9c77, 0x9c77, PDF_CMAP_SINGLE, 15347 },
	{ 0x9c78, 0x9c78, PDF_CMAP_SINGLE, 7358 },
	{ 0x9c7a, 0x9c7a, PDF_CMAP_SINGLE, 19067 },
	{ 0x9c7b, 0x9c7b, PDF_CMAP_SINGLE, 22983 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 3031 },
	{ 0x9ce6, 0x9ce6, PDF_CMAP_SINGLE, 19068 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 7359 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3403 },
	{ 0x9ceb, 0x9ceb, PDF_CMAP_SINGLE, 7364 },
	{ 0x9cec, 0x9cec, PDF_CMAP_SINGLE, 7360 },
	{ 0x9cf0, 0x9cf0, PDF_CMAP_SINGLE, 7361 },
	{ 0x9cf2, 0x9cf2, PDF_CMAP_SINGLE, 19069 },
	{ 0x9cf3, 0x9cf3, PDF_CMAP_SINGLE, 3679 },
	{ 0x9cf4, 0x9cf4, PDF_CMAP_SINGLE, 3792 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 3240 },
	{ 0x9cf7, 0x9cf7, PDF_CMAP_SINGLE, 22984 },
	{ 0x9cf9, 0x9cf9, PDF_CMAP_SINGLE, 22985 },
	{ 0x9d02, 0x9d02, PDF_CMAP_SINGLE, 15348 },
	{ 0x9d03, 0x9d03, PDF_CMAP_SINGLE, 7365 },
	{ 0x9d06, 0x9d06, PDF_CMAP_SINGLE, 7366 },
	{ 0x9d07, 0x9d07, PDF_CMAP_SINGLE, 3222 },
	{ 0x9d08, 0x9d08, PDF_CMAP_SINGLE, 7363 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 7362 },
	{ 0x9d0b, 0x9d0b, PDF_CMAP_SINGLE, 19070 },
	{ 0x9d0e, 0x9d0e, PDF_CMAP_SINGLE, 1322 },
	{ 0x9d11, 0x9d11, PDF_CMAP_SINGLE, 19072 },
	{ 0x9d12, 0x9d12, PDF_CMAP_SINGLE, 7374 },
	{ 0x9d15, 0x9d15, PDF_CMAP_SINGLE, 7373 },
	{ 0x9d17, 0x9d18, PDF_CMAP_RANGE, 19073 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 1303 },
	{ 0x9d1c, 0x9d1c, PDF_CMAP_SINGLE, 22986 },
	{ 0x9d1d, 0x9d1e, PDF_CMAP_RANGE, 17218 },
	{ 0x9d1f, 0x9d1f, PDF_CMAP_SINGLE, 7371 },
	{ 0x9d23, 0x9d23, PDF_CMAP_SINGLE, 7370 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 7368 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 1497 },
	{ 0x9d2a, 0x9d2a, PDF_CMAP_SINGLE, 7367 },
	{ 0x9d2b, 0x9d2b, PDF_CMAP_SINGLE, 2270 },
	{ 0x9d2c, 0x9d2c, PDF_CMAP_SINGLE, 1321 },
	{ 0x9d2f, 0x9d2f, PDF_CMAP_SINGLE, 22987 },
	{ 0x9d30, 0x9d30, PDF_CMAP_SINGLE, 20029 },
	{ 0x9d32, 0x9d32, PDF_CMAP_SINGLE, 19078 },
	{ 0x9d33, 0x9d34, PDF_CMAP_RANGE, 22988 },
	{ 0x9d3a, 0x9d3a, PDF_CMAP_SINGLE, 22990 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 2037 },
	{ 0x9d3c, 0x9d3c, PDF_CMAP_SINGLE, 22991 },
	{ 0x9d3d, 0x9d3d, PDF_CMAP_SINGLE, 20030 },
	{ 0x9d3e, 0x9d3e, PDF_CMAP_SINGLE, 7377 },
	{ 0x9d3f, 0x9d3f, PDF_CMAP_SINGLE, 7376 },
	{ 0x9d41, 0x9d41, PDF_CMAP_SINGLE, 7375 },
	{ 0x9d42, 0x9d42, PDF_CMAP_SINGLE, 15349 },
	{ 0x9d43, 0x9d43, PDF_CMAP_SINGLE, 17220 },
	{ 0x9d44, 0x9d44, PDF_CMAP_SINGLE, 7372 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 22992 },
	{ 0x9d46, 0x9d46, PDF_CMAP_SINGLE, 7378 },
	{ 0x9d47, 0x9d47, PDF_CMAP_SINGLE, 15350 },
	{ 0x9d48, 0x9d48, PDF_CMAP_SINGLE, 7379 },
	{ 0x9d4a, 0x9d4a, PDF_CMAP_SINGLE, 19080 },
	{ 0x9d50, 0x9d50, PDF_CMAP_SINGLE, 7384 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 7383 },
	{ 0x9d52, 0x9d52, PDF_CMAP_SINGLE, 17221 },
	{ 0x9d53, 0x9d54, PDF_CMAP_RANGE, 22993 },
	{ 0x9d59, 0x9d59, PDF_CMAP_SINGLE, 7385 },
	{ 0x9d5c, 0x9d5c, PDF_CMAP_SINGLE, 1231 },
	{ 0x9d5d, 0x9d5e, PDF_CMAP_RANGE, 7380 },
	{ 0x9d5f, 0x9d5f, PDF_CMAP_SINGLE, 19081 },
	{ 0x9d60, 0x9d60, PDF_CMAP_SINGLE, 2054 },
	{ 0x9d61, 0x9d61, PDF_CMAP_SINGLE, 3781 },
	{ 0x9d62, 0x9d62, PDF_CMAP_SINGLE, 19082 },
	{ 0x9d63, 0x9d63, PDF_CMAP_SINGLE, 15351 },
	{ 0x9d64, 0x9d64, PDF_CMAP_SINGLE, 7382 },
	{ 0x9d65, 0x9d65, PDF_CMAP_SINGLE, 22995 },
	{ 0x9d69, 0x9d69, PDF_CMAP_SINGLE, 15352 },
	{ 0x9d6a, 0x9d6a, PDF_CMAP_SINGLE, 20031 },
	{ 0x9d6b, 0x9d6b, PDF_CMAP_SINGLE, 8714 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 3680 },
	{ 0x9d6f, 0x9d6f, PDF_CMAP_SINGLE, 7390 },
	{ 0x9d70, 0x9d70, PDF_CMAP_SINGLE, 8713 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 7386 },
	{ 0x9d73, 0x9d73, PDF_CMAP_SINGLE, 19085 },
	{ 0x9d76, 0x9d77, PDF_CMAP_RANGE, 19086 },
	{ 0x9d7a, 0x9d7a, PDF_CMAP_SINGLE, 7391 },
	{ 0x9d7b, 0x9d7b, PDF_CMAP_SINGLE, 20032 },
	{ 0x9d7c, 0x9d7c, PDF_CMAP_SINGLE, 15353 },
	{ 0x9d7e, 0x9d7e, PDF_CMAP_SINGLE, 15354 },
	{ 0x9d83, 0x9d83, PDF_CMAP_SINGLE, 22996 },
	{ 0x9d84, 0x9d84, PDF_CMAP_SINGLE, 19088 },
	{ 0x9d86, 0x9d86, PDF_CMAP_SINGLE, 22997 },
	{ 0x9d87, 0x9d87, PDF_CMAP_SINGLE, 7388 },
	{ 0x9d89, 0x9d89, PDF_CMAP_SINGLE, 7387 },
	{ 0x9d8a, 0x9d8a, PDF_CMAP_SINGLE, 17222 },
	{ 0x9d8d, 0x9d8d, PDF_CMAP_SINGLE, 15355 },
	{ 0x9d8e, 0x9d8e, PDF_CMAP_SINGLE, 22998 },
	{ 0x9d8f, 0x9d8f, PDF_CMAP_SINGLE, 1842 },
	{ 0x9d92, 0x9d93, PDF_CMAP_RANGE, 22999 },
	{ 0x9d95, 0x9d95, PDF_CMAP_SINGLE, 23001 },
	{ 0x9d96, 0x9d96, PDF_CMAP_SINGLE, 17223 },
	{ 0x9d97, 0x9d98, PDF_CMAP_RANGE, 23002 },
	{ 0x9d99, 0x9d99, PDF_CMAP_SINGLE, 19089 },
	{ 0x9d9a, 0x9d9a, PDF_CMAP_SINGLE, 7392 },
	{ 0x9da1, 0x9da1, PDF_CMAP_SINGLE, 19090 },
	{ 0x9da4, 0x9da4, PDF_CMAP_SINGLE, 7393 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 7394 },
	{ 0x9daa, 0x9daa, PDF_CMAP_SINGLE, 23004 },
	{ 0x9dab, 0x9dab, PDF_CMAP_SINGLE, 7389 },
	{ 0x9dac, 0x9dac, PDF_CMAP_SINGLE, 17225 },
	{ 0x9dae, 0x9dae, PDF_CMAP_SINGLE, 23005 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 7369 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 15356 },
	{ 0x9db2, 0x9db2, PDF_CMAP_SINGLE, 7395 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 3069 },
	{ 0x9db5, 0x9db5, PDF_CMAP_SINGLE, 19092 },
	{ 0x9db8, 0x9db8, PDF_CMAP_SINGLE, 7399 },
	{ 0x9db9, 0x9db9, PDF_CMAP_SINGLE, 19093 },
	{ 0x9dba, 0x9dba, PDF_CMAP_SINGLE, 7400 },
	{ 0x9dbb, 0x9dbb, PDF_CMAP_SINGLE, 7398 },
	{ 0x9dbc, 0x9dbc, PDF_CMAP_SINGLE, 17226 },
	{ 0x9dbd, 0x9dbd, PDF_CMAP_SINGLE, 19094 },
	{ 0x9dbf, 0x9dbf, PDF_CMAP_SINGLE, 19091 },
	{ 0x9dc0, 0x9dc0, PDF_CMAP_SINGLE, 17224 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 7397 },
	{ 0x9dc2, 0x9dc2, PDF_CMAP_SINGLE, 7403 },
	{ 0x9dc3, 0x9dc3, PDF_CMAP_SINGLE, 15357 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 7396 },
	{ 0x9dc6, 0x9dc6, PDF_CMAP_SINGLE, 7401 },
	{ 0x9dc7, 0x9dc7, PDF_CMAP_SINGLE, 15358 },
	{ 0x9dc9, 0x9dc9, PDF_CMAP_SINGLE, 19095 },
	{ 0x9dca, 0x9dca, PDF_CMAP_SINGLE, 23007 },
	{ 0x9dcf, 0x9dcf, PDF_CMAP_SINGLE, 7402 },
	{ 0x9dd3, 0x9dd3, PDF_CMAP_SINGLE, 7405 },
	{ 0x9dd4, 0x9dd5, PDF_CMAP_RANGE, 23008 },
	{ 0x9dd6, 0x9dd6, PDF_CMAP_SINGLE, 15359 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 7646 },
	{ 0x9dd9, 0x9dd9, PDF_CMAP_SINGLE, 7404 },
	{ 0x9dda, 0x9dda, PDF_CMAP_SINGLE, 19096 },
	{ 0x9dde, 0x9dde, PDF_CMAP_SINGLE, 23010 },
	{ 0x9ddf, 0x9ddf, PDF_CMAP_SINGLE, 15360 },
	{ 0x9de0, 0x9de0, PDF_CMAP_SINGLE, 19097 },
	{ 0x9de3, 0x9de3, PDF_CMAP_SINGLE, 19098 },
	{ 0x9de5, 0x9de5, PDF_CMAP_SINGLE, 20033 },
	{ 0x9de6, 0x9de6, PDF_CMAP_SINGLE, 7407 },
	{ 0x9de7, 0x9de7, PDF_CMAP_SINGLE, 17228 },
	{ 0x9de9, 0x9de9, PDF_CMAP_SINGLE, 20034 },
	{ 0x9deb, 0x9deb, PDF_CMAP_SINGLE, 15361 },
	{ 0x9ded, 0x9ded, PDF_CMAP_SINGLE, 7408 },
	{ 0x9dee, 0x9dee, PDF_CMAP_SINGLE, 23011 },
	{ 0x9def, 0x9def, PDF_CMAP_SINGLE, 7409 },
	{ 0x9df0, 0x9df0, PDF_CMAP_SINGLE, 23012 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 4079 },
	{ 0x9df3, 0x9df3, PDF_CMAP_SINGLE, 20035 },
	{ 0x9df4, 0x9df4, PDF_CMAP_SINGLE, 15362 },
	{ 0x9df8, 0x9df8, PDF_CMAP_SINGLE, 7406 },
	{ 0x9df9, 0x9df9, PDF_CMAP_SINGLE, 2891 },
	{ 0x9dfa, 0x9dfa, PDF_CMAP_SINGLE, 2141 },
	{ 0x9dfd, 0x9dfd, PDF_CMAP_SINGLE, 7410 },
	{ 0x9dfe, 0x9dfe, PDF_CMAP_SINGLE, 23013 },
	{ 0x9e02, 0x9e02, PDF_CMAP_SINGLE, 19101 },
	{ 0x9e07, 0x9e07, PDF_CMAP_SINGLE, 17229 },
	{ 0x9e0a, 0x9e0a, PDF_CMAP_SINGLE, 19100 },
	{ 0x9e0d, 0x9e0d, PDF_CMAP_SINGLE, 19102 },
	{ 0x9e0e, 0x9e0e, PDF_CMAP_SINGLE, 23014 },
	{ 0x9e10, 0x9e12, PDF_CMAP_RANGE, 23015 },
	{ 0x9e15, 0x9e15, PDF_CMAP_SINGLE, 15363 },
	{ 0x9e16, 0x9e16, PDF_CMAP_SINGLE, 23018 },
	{ 0x9e19, 0x9e19, PDF_CMAP_SINGLE, 8716 },
	{ 0x9e1a, 0x9e1b, PDF_CMAP_RANGE, 7411 },
	{ 0x9e1c, 0x9e1c, PDF_CMAP_SINGLE, 19103 },
	{ 0x9e1d, 0x9e1d, PDF_CMAP_SINGLE, 15364 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 7413 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 7414 },
	{ 0x9e78, 0x9e78, PDF_CMAP_SINGLE, 1896 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 7415 },
	{ 0x9e7a, 0x9e7a, PDF_CMAP_SINGLE, 20036 },
	{ 0x9e7b, 0x9e7b, PDF_CMAP_SINGLE, 19104 },
	{ 0x9e7c, 0x9e7c, PDF_CMAP_SINGLE, 7677 },
	{ 0x9e7d, 0x9e7d, PDF_CMAP_SINGLE, 7416 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 2267 },
	{ 0x9e80, 0x9e80, PDF_CMAP_SINGLE, 19106 },
	{ 0x9e81, 0x9e81, PDF_CMAP_SINGLE, 7417 },
	{ 0x9e82, 0x9e84, PDF_CMAP_RANGE, 20037 },
	{ 0x9e85, 0x9e85, PDF_CMAP_SINGLE, 19107 },
	{ 0x9e87, 0x9e87, PDF_CMAP_SINGLE, 23019 },
	{ 0x9e88, 0x9e88, PDF_CMAP_SINGLE, 7418 },
	{ 0x9e8b, 0x9e8c, PDF_CMAP_RANGE, 7419 },
	{ 0x9e8e, 0x9e8f, PDF_CMAP_RANGE, 23020 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 7423 },
	{ 0x9e92, 0x9e92, PDF_CMAP_SINGLE, 7421 },
	{ 0x9e93, 0x9e93, PDF_CMAP_SINGLE, 4066 },
	{ 0x9e95, 0x9e95, PDF_CMAP_SINGLE, 7422 },
	{ 0x9e96, 0x9e96, PDF_CMAP_SINGLE, 23022 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 4023 },
	{ 0x9e98, 0x9e98, PDF_CMAP_SINGLE, 23023 },
	{ 0x9e9b, 0x9e9b, PDF_CMAP_SINGLE, 19108 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 7424 },
	{ 0x9e9e, 0x9e9e, PDF_CMAP_SINGLE, 17230 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 4003 },
	{ 0x9ea4, 0x9ea4, PDF_CMAP_SINGLE, 15365 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 7425 },
	{ 0x9ea6, 0x9ea6, PDF_CMAP_SINGLE, 3380 },
	{ 0x9ea8, 0x9ea8, PDF_CMAP_SINGLE, 15366 },
	{ 0x9ea9, 0x9ea9, PDF_CMAP_SINGLE, 7426 },
	{ 0x9eaa, 0x9eaa, PDF_CMAP_SINGLE, 7428 },
	{ 0x9eac, 0x9eac, PDF_CMAP_SINGLE, 15367 },
	{ 0x9ead, 0x9ead, PDF_CMAP_SINGLE, 7429 },
	{ 0x9eae, 0x9eae, PDF_CMAP_SINGLE, 23024 },
	{ 0x9eaf, 0x9eaf, PDF_CMAP_SINGLE, 17231 },
	{ 0x9eb0, 0x9eb0, PDF_CMAP_SINGLE, 20040 },
	{ 0x9eb3, 0x9eb3, PDF_CMAP_SINGLE, 23025 },
	{ 0x9eb4, 0x9eb4, PDF_CMAP_SINGLE, 7682 },
	{ 0x9eb5, 0x9eb5, PDF_CMAP_SINGLE, 7797 },
	{ 0x9eb8, 0x9eb8, PDF_CMAP_SINGLE, 7427 },
	{ 0x9eb9, 0x9eb9, PDF_CMAP_SINGLE, 2047 },
	{ 0x9eba, 0x9eba, PDF_CMAP_SINGLE, 3801 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 3729 },
	{ 0x9ebc, 0x9ebc, PDF_CMAP_SINGLE, 4740 },
	{ 0x9ebd, 0x9ebd, PDF_CMAP_SINGLE, 19110 },
	{ 0x9ebe, 0x9ebe, PDF_CMAP_SINGLE, 5375 },
	{ 0x9ebf, 0x9ebf, PDF_CMAP_SINGLE, 3753 },
	{ 0x9ec3, 0x9ec3, PDF_CMAP_SINGLE, 13323 },
	{ 0x9ec4, 0x9ec4, PDF_CMAP_SINGLE, 1323 },
	{ 0x9ec6, 0x9ec6, PDF_CMAP_SINGLE, 23026 },
	{ 0x9ec8, 0x9ec8, PDF_CMAP_SINGLE, 23027 },
	{ 0x9ecb, 0x9ecb, PDF_CMAP_SINGLE, 23028 },
	{ 0x9ecc, 0x9ecc, PDF_CMAP_SINGLE, 7431 },
	{ 0x9ecd, 0x9ecd, PDF_CMAP_SINGLE, 1642 },
	{ 0x9ece, 0x9ed0, PDF_CMAP_RANGE, 7432 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 8717 },
	{ 0x9ed2, 0x9ed2, PDF_CMAP_SINGLE, 2055 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 7435 },
	{ 0x9ed5, 0x9ed5, PDF_CMAP_SINGLE, 23029 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 5645 },
	{ 0x9ed9, 0x9ed9, PDF_CMAP_SINGLE, 3815 },
	{ 0x9edb, 0x9edb, PDF_CMAP_SINGLE, 2883 },
	{ 0x9edc, 0x9edc, PDF_CMAP_SINGLE, 7436 },
	{ 0x9edd, 0x9edd, PDF_CMAP_SINGLE, 7438 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 7437 },
	{ 0x9edf, 0x9edf, PDF_CMAP_SINGLE, 19112 },
	{ 0x9ee0, 0x9ee0, PDF_CMAP_SINGLE, 7439 },
	{ 0x9ee4, 0x9ee4, PDF_CMAP_SINGLE, 20041 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 7440 },
	{ 0x9ee7, 0x9ee7, PDF_CMAP_SINGLE, 15368 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 7441 },
	{ 0x9eec, 0x9eed, PDF_CMAP_RANGE, 23030 },
	{ 0x9eee, 0x9eee, PDF_CMAP_SINGLE, 15369 },
	{ 0x9eef, 0x9eef, PDF_CMAP_SINGLE, 7442 },
	{ 0x9ef0, 0x9ef0, PDF_CMAP_SINGLE, 20042 },
	{ 0x9ef1, 0x9ef1, PDF_CMAP_SINGLE, 23032 },
	{ 0x9ef2, 0x9ef2, PDF_CMAP_SINGLE, 20043 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 7443 },
	{ 0x9ef5, 0x9ef5, PDF_CMAP_SINGLE, 23033 },
	{ 0x9ef6, 0x9ef7, PDF_CMAP_RANGE, 7444 },
	{ 0x9ef8, 0x9ef8, PDF_CMAP_SINGLE, 23034 },
	{ 0x9ef9, 0x9ef9, PDF_CMAP_SINGLE, 7446 },
	{ 0x9efb, 0x9efd, PDF_CMAP_RANGE, 7447 },
	{ 0x9eff, 0x9eff, PDF_CMAP_SINGLE, 19113 },
	{ 0x9f02, 0x9f02, PDF_CMAP_SINGLE, 19114 },
	{ 0x9f03, 0x9f03, PDF_CMAP_SINGLE, 19116 },
	{ 0x9f07, 0x9f08, PDF_CMAP_RANGE, 7450 },
	{ 0x9f09, 0x9f09, PDF_CMAP_SINGLE, 20044 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 3102 },
	{ 0x9f0f, 0x9f0f, PDF_CMAP_SINGLE, 20045 },
	{ 0x9f10, 0x9f10, PDF_CMAP_SINGLE, 15370 },
	{ 0x9f11, 0x9f11, PDF_CMAP_SINGLE, 23035 },
	{ 0x9f12, 0x9f12, PDF_CMAP_SINGLE, 15371 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 1937 },
	{ 0x9f14, 0x9f14, PDF_CMAP_SINGLE, 20046 },
	{ 0x9f15, 0x9f15, PDF_CMAP_SINGLE, 7453 },
	{ 0x9f16, 0x9f16, PDF_CMAP_SINGLE, 23036 },
	{ 0x9f17, 0x9f17, PDF_CMAP_SINGLE, 15372 },
	{ 0x9f19, 0x9f19, PDF_CMAP_SINGLE, 15373 },
	{ 0x9f1a, 0x9f1a, PDF_CMAP_SINGLE, 23037 },
	{ 0x9f1b, 0x9f1b, PDF_CMAP_SINGLE, 20047 },
	{ 0x9f1f, 0x9f1f, PDF_CMAP_SINGLE, 23038 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 2767 },
	{ 0x9f21, 0x9f21, PDF_CMAP_SINGLE, 7454 },
	{ 0x9f22, 0x9f22, PDF_CMAP_SINGLE, 20048 },
	{ 0x9f26, 0x9f26, PDF_CMAP_SINGLE, 20049 },
	{ 0x9f2a, 0x9f2b, PDF_CMAP_RANGE, 20050 },
	{ 0x9f2c, 0x9f2c, PDF_CMAP_SINGLE, 7455 },
	{ 0x9f2f, 0x9f2f, PDF_CMAP_SINGLE, 15374 },
	{ 0x9f31, 0x9f32, PDF_CMAP_RANGE, 23039 },
	{ 0x9f34, 0x9f34, PDF_CMAP_SINGLE, 20052 },
	{ 0x9f37, 0x9f37, PDF_CMAP_SINGLE, 15375 },
	{ 0x9f39, 0x9f39, PDF_CMAP_SINGLE, 15376 },
	{ 0x9f3a, 0x9f3a, PDF_CMAP_SINGLE, 19117 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 3475 },
	{ 0x9f3c, 0x9f3c, PDF_CMAP_SINGLE, 23041 },
	{ 0x9f3d, 0x9f3d, PDF_CMAP_SINGLE, 19118 },
	{ 0x9f3e, 0x9f3e, PDF_CMAP_SINGLE, 7456 },
	{ 0x9f3f, 0x9f3f, PDF_CMAP_SINGLE, 23042 },
	{ 0x9f41, 0x9f41, PDF_CMAP_SINGLE, 15377 },
	{ 0x9f43, 0x9f44, PDF_CMAP_RANGE, 23043 },
	{ 0x9f45, 0x9f45, PDF_CMAP_SINGLE, 15378 },
	{ 0x9f46, 0x9f46, PDF_CMAP_SINGLE, 19119 },
	{ 0x9f47, 0x9f47, PDF_CMAP_SINGLE, 23045 },
	{ 0x9f4a, 0x9f4a, PDF_CMAP_SINGLE, 7457 },
	{ 0x9f4b, 0x9f4b, PDF_CMAP_SINGLE, 5898 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 6779 },
	{ 0x9f4f, 0x9f4f, PDF_CMAP_SINGLE, 7174 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7458 },
	{ 0x9f53, 0x9f53, PDF_CMAP_SINGLE, 19120 },
	{ 0x9f54, 0x9f54, PDF_CMAP_SINGLE, 7459 },
	{ 0x9f55, 0x9f55, PDF_CMAP_SINGLE, 19121 },
	{ 0x9f56, 0x9f56, PDF_CMAP_SINGLE, 23046 },
	{ 0x9f57, 0x9f57, PDF_CMAP_SINGLE, 15379 },
	{ 0x9f58, 0x9f58, PDF_CMAP_SINGLE, 19122 },
	{ 0x9f5a, 0x9f5a, PDF_CMAP_SINGLE, 20053 },
	{ 0x9f5d, 0x9f5d, PDF_CMAP_SINGLE, 19124 },
	{ 0x9f5e, 0x9f5e, PDF_CMAP_SINGLE, 23047 },
	{ 0x9f5f, 0x9f61, PDF_CMAP_RANGE, 7461 },
	{ 0x9f62, 0x9f62, PDF_CMAP_SINGLE, 4024 },
	{ 0x9f63, 0x9f63, PDF_CMAP_SINGLE, 7460 },
	{ 0x9f66, 0x9f67, PDF_CMAP_RANGE, 7464 },
	{ 0x9f68, 0x9f68, PDF_CMAP_SINGLE, 15380 },
	{ 0x9f69, 0x9f69, PDF_CMAP_SINGLE, 19126 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 7467 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 7466 },
	{ 0x9f6d, 0x9f6d, PDF_CMAP_SINGLE, 19127 },
	{ 0x9f6e, 0x9f6e, PDF_CMAP_SINGLE, 23048 },
	{ 0x9f6f, 0x9f6f, PDF_CMAP_SINGLE, 20054 },
	{ 0x9f70, 0x9f70, PDF_CMAP_SINGLE, 19128 },
	{ 0x9f71, 0x9f71, PDF_CMAP_SINGLE, 15381 },
	{ 0x9f72, 0x9f72, PDF_CMAP_SINGLE, 7469 },
	{ 0x9f73, 0x9f73, PDF_CMAP_SINGLE, 23049 },
	{ 0x9f75, 0x9f75, PDF_CMAP_SINGLE, 15382 },
	{ 0x9f76, 0x9f76, PDF_CMAP_SINGLE, 7470 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 7468 },
	{ 0x9f7a, 0x9f7a, PDF_CMAP_SINGLE, 23050 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 23051 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 3966 },
	{ 0x9f8f, 0x9f8f, PDF_CMAP_SINGLE, 23052 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 15383 },
	{ 0x9f91, 0x9f92, PDF_CMAP_RANGE, 23053 },
	{ 0x9f94, 0x9f94, PDF_CMAP_SINGLE, 15384 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 7471 },
	{ 0x9f96, 0x9f96, PDF_CMAP_SINGLE, 23055 },
	{ 0x9f97, 0x9f97, PDF_CMAP_SINGLE, 17232 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 7472 },
	{ 0x9f9d, 0x9f9d, PDF_CMAP_SINGLE, 5927 },
	{ 0x9f9e, 0x9f9e, PDF_CMAP_SINGLE, 20055 },
	{ 0x9fa0, 0x9fa0, PDF_CMAP_SINGLE, 7473 },
	{ 0x9fa1, 0x9fa1, PDF_CMAP_SINGLE, 23056 },
	{ 0x9fa2, 0x9fa2, PDF_CMAP_SINGLE, 15385 },
	{ 0x9fa3, 0x9fa3, PDF_CMAP_SINGLE, 23057 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 20056 },
	{ 0x9fb4, 0x9fb4, PDF_CMAP_SINGLE, 14048 },
	{ 0x9fbc, 0x9fbc, PDF_CMAP_SINGLE, 15431 },
	{ 0x9fbd, 0x9fbd, PDF_CMAP_SINGLE, 15429 },
	{ 0x9fbe, 0x9fbe, PDF_CMAP_SINGLE, 15434 },
	{ 0x9fbf, 0x9fc2, PDF_CMAP_RANGE, 20068 },
	{ 0xdc02, 0xdc02, PDF_CMAP_SINGLE, 18398 },
	{ 0xdc04, 0xdc04, PDF_CMAP_SINGLE, 15388 },
	{ 0xdc07, 0xdc07, PDF_CMAP_SINGLE, 17319 },
	{ 0xdc0b, 0xdc0b, PDF_CMAP_SINGLE, 13839 },
	{ 0xdc0c, 0xdc0c, PDF_CMAP_SINGLE, 18344 },
	{ 0xdc0c, 0xdc0c, PDF_CMAP_SINGLE, 18754 },
	{ 0xdc0f, 0xdc0f, PDF_CMAP_SINGLE, 7814 },
	{ 0xdc10, 0xdc10, PDF_CMAP_SINGLE, 18620 },
	{ 0xdc15, 0xdc15, PDF_CMAP_SINGLE, 20061 },
	{ 0xdc16, 0xdc16, PDF_CMAP_SINGLE, 18124 },
	{ 0xdc17, 0xdc17, PDF_CMAP_SINGLE, 18352 },
	{ 0xdc18, 0xdc18, PDF_CMAP_SINGLE, 7817 },
	{ 0xdc1a, 0xdc1a, PDF_CMAP_SINGLE, 19077 },
	{ 0xdc1a, 0xdc1a, PDF_CMAP_SINGLE, 13954 },
	{ 0xdc1d, 0xdc1d, PDF_CMAP_SINGLE, 17682 },
	{ 0xdc22, 0xdc22, PDF_CMAP_SINGLE, 13684 },
	{ 0xdc24, 0xdc24, PDF_CMAP_SINGLE, 17686 },
	{ 0xdc24, 0xdc24, PDF_CMAP_SINGLE, 17880 },
	{ 0xdc28, 0xdc28, PDF_CMAP_SINGLE, 13807 },
	{ 0xdc29, 0xdc29, PDF_CMAP_SINGLE, 17478 },
	{ 0xdc2c, 0xdc2c, PDF_CMAP_SINGLE, 14109 },
	{ 0xdc2e, 0xdc2e, PDF_CMAP_SINGLE, 18209 },
	{ 0xdc2f, 0xdc2f, PDF_CMAP_SINGLE, 19075 },
	{ 0xdc33, 0xdc33, PDF_CMAP_SINGLE, 13719 },
	{ 0xdc37, 0xdc37, PDF_CMAP_SINGLE, 19111 },
	{ 0xdc3a, 0xdc3a, PDF_CMAP_SINGLE, 17321 },
	{ 0xdc3c, 0xdc3c, PDF_CMAP_SINGLE, 20220 },
	{ 0xdc3d, 0xdc3d, PDF_CMAP_SINGLE, 17885 },
	{ 0xdc3f, 0xdc3f, PDF_CMAP_SINGLE, 13815 },
	{ 0xdc46, 0xdc46, PDF_CMAP_SINGLE, 20062 },
	{ 0xdc49, 0xdc49, PDF_CMAP_SINGLE, 18624 },
	{ 0xdc4a, 0xdc4a, PDF_CMAP_SINGLE, 17297 },
	{ 0xdc4a, 0xdc4a, PDF_CMAP_SINGLE, 17782 },
	{ 0xdc4a, 0xdc4a, PDF_CMAP_SINGLE, 18181 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 17784 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 20168 },
	{ 0xdc4b, 0xdc4b, PDF_CMAP_SINGLE, 18302 },
	{ 0xdc51, 0xdc51, PDF_CMAP_SINGLE, 17783 },
	{ 0xdc52, 0xdc52, PDF_CMAP_SINGLE, 13841 },
	{ 0xdc55, 0xdc55, PDF_CMAP_SINGLE, 18183 },
	{ 0xdc55, 0xdc55, PDF_CMAP_SINGLE, 18757 },
	{ 0xdc56, 0xdc56, PDF_CMAP_SINGLE, 17539 },
	{ 0xdc5b, 0xdc5b, PDF_CMAP_SINGLE, 17647 },
	{ 0xdc60, 0xdc60, PDF_CMAP_SINGLE, 18355 },
	{ 0xdc62, 0xdc62, PDF_CMAP_SINGLE, 13998 },
	{ 0xdc64, 0xdc64, PDF_CMAP_SINGLE, 18303 },
	{ 0xdc65, 0xdc65, PDF_CMAP_SINGLE, 17788 },
	{ 0xdc6d, 0xdc6d, PDF_CMAP_SINGLE, 17462 },
	{ 0xdc6d, 0xdc6d, PDF_CMAP_SINGLE, 14121 },
	{ 0xdc6e, 0xdc6e, PDF_CMAP_SINGLE, 16821 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 18506 },
	{ 0xdc73, 0xdc73, PDF_CMAP_SINGLE, 13832 },
	{ 0xdc74, 0xdc74, PDF_CMAP_SINGLE, 7670 },
	{ 0xdc77, 0xdc77, PDF_CMAP_SINGLE, 7754 },
	{ 0xdc7f, 0xdc7f, PDF_CMAP_SINGLE, 17910 },
	{ 0xdc82, 0xdc82, PDF_CMAP_SINGLE, 19084 },
	{ 0xdc84, 0xdc84, PDF_CMAP_SINGLE, 7734 },
	{ 0xdc89, 0xdc89, PDF_CMAP_SINGLE, 17233 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 14108 },
	{ 0xdc8a, 0xdc8a, PDF_CMAP_SINGLE, 18727 },
	{ 0xdc8e, 0xdc8e, PDF_CMAP_SINGLE, 17005 },
	{ 0xdc90, 0xdc90, PDF_CMAP_SINGLE, 18935 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 17998 },
	{ 0xdc96, 0xdc96, PDF_CMAP_SINGLE, 18077 },
	{ 0xdc99, 0xdc99, PDF_CMAP_SINGLE, 20063 },
	{ 0xdc9a, 0xdc9a, PDF_CMAP_SINGLE, 13928 },
	{ 0xdc9e, 0xdc9e, PDF_CMAP_SINGLE, 20206 },
	{ 0xdca2, 0xdca2, PDF_CMAP_SINGLE, 17240 },
	{ 0xdca4, 0xdca4, PDF_CMAP_SINGLE, 17243 },
	{ 0xdca6, 0xdca6, PDF_CMAP_SINGLE, 20064 },
	{ 0xdcab, 0xdcab, PDF_CMAP_SINGLE, 17653 },
	{ 0xdcac, 0xdcac, PDF_CMAP_SINGLE, 13750 },
	{ 0xdcb0, 0xdcb0, PDF_CMAP_SINGLE, 14209 },
	{ 0xdcb2, 0xdcb2, PDF_CMAP_SINGLE, 13867 },
	{ 0xdcb6, 0xdcb6, PDF_CMAP_SINGLE, 14129 },
	{ 0xdcb8, 0xdcb8, PDF_CMAP_SINGLE, 18708 },
	{ 0xdcb9, 0xdcb9, PDF_CMAP_SINGLE, 17327 },
	{ 0xdcbb, 0xdcbb, PDF_CMAP_SINGLE, 18733 },
	{ 0xdcbd, 0xdcbd, PDF_CMAP_SINGLE, 16833 },
	{ 0xdcbe, 0xdcbe, PDF_CMAP_SINGLE, 14293 },
	{ 0xdcc6, 0xdcc6, PDF_CMAP_SINGLE, 18983 },
	{ 0xdcd3, 0xdcd3, PDF_CMAP_SINGLE, 7816 },
	{ 0xdcd9, 0xdcd9, PDF_CMAP_SINGLE, 18217 },
	{ 0xdcdb, 0xdcdb, PDF_CMAP_SINGLE, 14140 },
	{ 0xdcdc, 0xdcdc, PDF_CMAP_SINGLE, 7695 },
	{ 0xdcdd, 0xdcdd, PDF_CMAP_SINGLE, 18444 },
	{ 0xdcdd, 0xdcdd, PDF_CMAP_SINGLE, 18515 },
	{ 0xdcdd, 0xdcdd, PDF_CMAP_SINGLE, 7641 },
	{ 0xdce1, 0xdce1, PDF_CMAP_SINGLE, 14291 },
	{ 0xdce4, 0xdce4, PDF_CMAP_SINGLE, 17814 },
	{ 0xdce5, 0xdce5, PDF_CMAP_SINGLE, 20065 },
	{ 0xdcea, 0xdcea, PDF_CMAP_SINGLE, 18446 },
	{ 0xdcea, 0xdcea, PDF_CMAP_SINGLE, 13679 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 18365 },
	{ 0xdced, 0xdced, PDF_CMAP_SINGLE, 7665 },
	{ 0xdcf4, 0xdcf4, PDF_CMAP_SINGLE, 17103 },
	{ 0xdcf5, 0xdcf5, PDF_CMAP_SINGLE, 20057 },
	{ 0xdcf9, 0xdcf9, PDF_CMAP_SINGLE, 19083 },
	{ 0xdcfc, 0xdcfc, PDF_CMAP_SINGLE, 13656 },
	{ 0xdcfe, 0xdcfe, PDF_CMAP_SINGLE, 13904 },
	{ 0xdd00, 0xdd00, PDF_CMAP_SINGLE, 17925 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 18003 },
	{ 0xdd03, 0xdd03, PDF_CMAP_SINGLE, 13768 },
	{ 0xdd09, 0xdd09, PDF_CMAP_SINGLE, 17299 },
	{ 0xdd0b, 0xdd0b, PDF_CMAP_SINGLE, 13801 },
	{ 0xdd0d, 0xdd0d, PDF_CMAP_SINGLE, 18571 },
	{ 0xdd0e, 0xdd0e, PDF_CMAP_SINGLE, 13523 },
	{ 0xdd0e, 0xdd0e, PDF_CMAP_SINGLE, 18394 },
	{ 0xdd0e, 0xdd0e, PDF_CMAP_SINGLE, 17009 },
	{ 0xdd0f, 0xdd0f, PDF_CMAP_SINGLE, 13932 },
	{ 0xdd10, 0xdd10, PDF_CMAP_SINGLE, 18882 },
	{ 0xdd14, 0xdd14, PDF_CMAP_SINGLE, 13995 },
	{ 0xdd1a, 0xdd1a, PDF_CMAP_SINGLE, 13916 },
	{ 0xdd20, 0xdd20, PDF_CMAP_SINGLE, 7839 },
	{ 0xdd21, 0xdd21, PDF_CMAP_SINGLE, 13809 },
	{ 0xdd22, 0xdd22, PDF_CMAP_SINGLE, 18185 },
	{ 0xdd2d, 0xdd2d, PDF_CMAP_SINGLE, 17544 },
	{ 0xdd39, 0xdd39, PDF_CMAP_SINGLE, 18574 },
	{ 0xdd3f, 0xdd3f, PDF_CMAP_SINGLE, 14188 },
	{ 0xdd40, 0xdd40, PDF_CMAP_SINGLE, 17942 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 17359 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 17545 },
	{ 0xdd45, 0xdd45, PDF_CMAP_SINGLE, 13357 },
	{ 0xdd46, 0xdd46, PDF_CMAP_SINGLE, 18812 },
	{ 0xdd47, 0xdd47, PDF_CMAP_SINGLE, 13854 },
	{ 0xdd49, 0xdd49, PDF_CMAP_SINGLE, 18811 },
	{ 0xdd4b, 0xdd4b, PDF_CMAP_SINGLE, 13717 },
	{ 0xdd51, 0xdd51, PDF_CMAP_SINGLE, 13646 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 20075 },
	{ 0xdd58, 0xdd58, PDF_CMAP_SINGLE, 20090 },
	{ 0xdd5a, 0xdd5a, PDF_CMAP_SINGLE, 17815 },
	{ 0xdd62, 0xdd62, PDF_CMAP_SINGLE, 17547 },
	{ 0xdd6b, 0xdd6b, PDF_CMAP_SINGLE, 18770 },
	{ 0xdd6b, 0xdd6b, PDF_CMAP_SINGLE, 18817 },
	{ 0xdd6c, 0xdd6c, PDF_CMAP_SINGLE, 14180 },
	{ 0xdd6f, 0xdd6f, PDF_CMAP_SINGLE, 18450 },
	{ 0xdd71, 0xdd71, PDF_CMAP_SINGLE, 18883 },
	{ 0xdd73, 0xdd73, PDF_CMAP_SINGLE, 20060 },
	{ 0xdd78, 0xdd78, PDF_CMAP_SINGLE, 17546 },
	{ 0xdd7b, 0xdd7b, PDF_CMAP_SINGLE, 17599 },
	{ 0xdd7c, 0xdd7c, PDF_CMAP_SINGLE, 17331 },
	{ 0xdd84, 0xdd84, PDF_CMAP_SINGLE, 14109 },
	{ 0xdd85, 0xdd85, PDF_CMAP_SINGLE, 18672 },
	{ 0xdd87, 0xdd87, PDF_CMAP_SINGLE, 14253 },
	{ 0xdd88, 0xdd88, PDF_CMAP_SINGLE, 18824 },
	{ 0xdd8f, 0xdd8f, PDF_CMAP_SINGLE, 17657 },
	{ 0xdd90, 0xdd90, PDF_CMAP_SINGLE, 17227 },
	{ 0xdd92, 0xdd92, PDF_CMAP_SINGLE, 17556 },
	{ 0xdd94, 0xdd94, PDF_CMAP_SINGLE, 17827 },
	{ 0xdd95, 0xdd95, PDF_CMAP_SINGLE, 13670 },
	{ 0xdd99, 0xdd99, PDF_CMAP_SINGLE, 14134 },
	{ 0xdd9c, 0xdd9c, PDF_CMAP_SINGLE, 17552 },
	{ 0xdd9d, 0xdd9d, PDF_CMAP_SINGLE, 17332 },
	{ 0xdda0, 0xdda0, PDF_CMAP_SINGLE, 18716 },
	{ 0xdda1, 0xdda1, PDF_CMAP_SINGLE, 17551 },
	{ 0xdda1, 0xdda1, PDF_CMAP_SINGLE, 17033 },
	{ 0xdda2, 0xdda2, PDF_CMAP_SINGLE, 13857 },
	{ 0xdda7, 0xdda7, PDF_CMAP_SINGLE, 18229 },
	{ 0xdda9, 0xdda9, PDF_CMAP_SINGLE, 18190 },
	{ 0xddb1, 0xddb1, PDF_CMAP_SINGLE, 20080 },
	{ 0xddb4, 0xddb4, PDF_CMAP_SINGLE, 20133 },
	{ 0xddb6, 0xddb6, PDF_CMAP_SINGLE, 17744 },
	{ 0xddb7, 0xddb7, PDF_CMAP_SINGLE, 17559 },
	{ 0xddba, 0xddbb, PDF_CMAP_RANGE, 18834 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 17525 },
	{ 0xddc3, 0xddc3, PDF_CMAP_SINGLE, 17742 },
	{ 0xddc4, 0xddc4, PDF_CMAP_SINGLE, 16888 },
	{ 0xddc4, 0xddc4, PDF_CMAP_SINGLE, 16905 },
	{ 0xddc4, 0xddc4, PDF_CMAP_SINGLE, 17024 },
	{ 0xddc6, 0xddc6, PDF_CMAP_SINGLE, 18015 },
	{ 0xddc8, 0xddc9, PDF_CMAP_RANGE, 18773 },
	{ 0xddcc, 0xddcc, PDF_CMAP_SINGLE, 20112 },
	{ 0xddcd, 0xddcd, PDF_CMAP_SINGLE, 18193 },
	{ 0xddcf, 0xddcf, PDF_CMAP_SINGLE, 18944 },
	{ 0xddd0, 0xddd0, PDF_CMAP_SINGLE, 14068 },
	{ 0xddd3, 0xddd3, PDF_CMAP_SINGLE, 17945 },
	{ 0xddd4, 0xddd4, PDF_CMAP_SINGLE, 18268 },
	{ 0xddd6, 0xddd6, PDF_CMAP_SINGLE, 17308 },
	{ 0xddd7, 0xddd7, PDF_CMAP_SINGLE, 17472 },
	{ 0xdddb, 0xdddb, PDF_CMAP_SINGLE, 19026 },
	{ 0xdddd, 0xdddd, PDF_CMAP_SINGLE, 18452 },
	{ 0xddde, 0xddde, PDF_CMAP_SINGLE, 20066 },
	{ 0xdddf, 0xdddf, PDF_CMAP_SINGLE, 14069 },
	{ 0xdde0, 0xdde0, PDF_CMAP_SINGLE, 17561 },
	{ 0xdde1, 0xdde1, PDF_CMAP_SINGLE, 17373 },
	{ 0xdde1, 0xdde1, PDF_CMAP_SINGLE, 17710 },
	{ 0xdde5, 0xdde5, PDF_CMAP_SINGLE, 18192 },
	{ 0xddf1, 0xddf1, PDF_CMAP_SINGLE, 19123 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 15269 },
	{ 0xddf5, 0xddf5, PDF_CMAP_SINGLE, 17743 },
	{ 0xddf9, 0xddf9, PDF_CMAP_SINGLE, 17944 },
	{ 0xddfa, 0xddfa, PDF_CMAP_SINGLE, 17943 },
	{ 0xddfb, 0xddfb, PDF_CMAP_SINGLE, 18885 },
	{ 0xddfe, 0xddfe, PDF_CMAP_SINGLE, 18018 },
	{ 0xde01, 0xde01, PDF_CMAP_SINGLE, 17414 },
	{ 0xde02, 0xde02, PDF_CMAP_SINGLE, 19125 },
	{ 0xde04, 0xde04, PDF_CMAP_SINGLE, 20058 },
	{ 0xde09, 0xde09, PDF_CMAP_SINGLE, 15443 },
	{ 0xde0e, 0xde0e, PDF_CMAP_SINGLE, 18158 },
	{ 0xde10, 0xde10, PDF_CMAP_SINGLE, 18718 },
	{ 0xde11, 0xde11, PDF_CMAP_SINGLE, 14294 },
	{ 0xde13, 0xde13, PDF_CMAP_SINGLE, 17259 },
	{ 0xde14, 0xde14, PDF_CMAP_SINGLE, 18638 },
	{ 0xde15, 0xde15, PDF_CMAP_SINGLE, 18637 },
	{ 0xde15, 0xde15, PDF_CMAP_SINGLE, 19036 },
	{ 0xde17, 0xde17, PDF_CMAP_SINGLE, 14256 },
	{ 0xde18, 0xde18, PDF_CMAP_SINGLE, 19105 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 7825 },
	{ 0xde1a, 0xde1a, PDF_CMAP_SINGLE, 20316 },
	{ 0xde1e, 0xde1e, PDF_CMAP_SINGLE, 18195 },
	{ 0xde1e, 0xde1e, PDF_CMAP_SINGLE, 18455 },
	{ 0xde1e, 0xde1e, PDF_CMAP_SINGLE, 18843 },
	{ 0xde1f, 0xde1f, PDF_CMAP_SINGLE, 18886 },
	{ 0xde22, 0xde22, PDF_CMAP_SINGLE, 13691 },
	{ 0xde28, 0xde28, PDF_CMAP_SINGLE, 14105 },
	{ 0xde29, 0xde29, PDF_CMAP_SINGLE, 18049 },
	{ 0xde29, 0xde29, PDF_CMAP_SINGLE, 18844 },
	{ 0xde2e, 0xde2e, PDF_CMAP_SINGLE, 18318 },
	{ 0xde31, 0xde31, PDF_CMAP_SINGLE, 18640 },
	{ 0xde33, 0xde33, PDF_CMAP_SINGLE, 17562 },
	{ 0xde34, 0xde34, PDF_CMAP_SINGLE, 16845 },
	{ 0xde36, 0xde36, PDF_CMAP_SINGLE, 18890 },
	{ 0xde37, 0xde37, PDF_CMAP_SINGLE, 18162 },
	{ 0xde38, 0xde38, PDF_CMAP_SINGLE, 17843 },
	{ 0xde39, 0xde39, PDF_CMAP_SINGLE, 17841 },
	{ 0xde3a, 0xde3a, PDF_CMAP_SINGLE, 15393 },
	{ 0xde3d, 0xde3d, PDF_CMAP_SINGLE, 13953 },
	{ 0xde3d, 0xde3d, PDF_CMAP_SINGLE, 20315 },
	{ 0xde40, 0xde40, PDF_CMAP_SINGLE, 17089 },
	{ 0xde42, 0xde42, PDF_CMAP_SINGLE, 20124 },
	{ 0xde43, 0xde43, PDF_CMAP_SINGLE, 18848 },
	{ 0xde47, 0xde47, PDF_CMAP_SINGLE, 17480 },
	{ 0xde47, 0xde47, PDF_CMAP_SINGLE, 17842 },
	{ 0xde49, 0xde49, PDF_CMAP_SINGLE, 19038 },
	{ 0xde4c, 0xde4c, PDF_CMAP_SINGLE, 18197 },
	{ 0xde4d, 0xde4d, PDF_CMAP_SINGLE, 18104 },
	{ 0xde55, 0xde55, PDF_CMAP_SINGLE, 17415 },
	{ 0xde56, 0xde56, PDF_CMAP_SINGLE, 18319 },
	{ 0xde58, 0xde58, PDF_CMAP_SINGLE, 18459 },
	{ 0xde62, 0xde62, PDF_CMAP_SINGLE, 18322 },
	{ 0xde64, 0xde64, PDF_CMAP_SINGLE, 13755 },
	{ 0xde64, 0xde64, PDF_CMAP_SINGLE, 17388 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 18320 },
	{ 0xde65, 0xde65, PDF_CMAP_SINGLE, 18528 },
	{ 0xde6a, 0xde6a, PDF_CMAP_SINGLE, 18167 },
	{ 0xde6a, 0xde6a, PDF_CMAP_SINGLE, 14189 },
	{ 0xde6d, 0xde6d, PDF_CMAP_SINGLE, 17380 },
	{ 0xde70, 0xde70, PDF_CMAP_SINGLE, 18385 },
	{ 0xde71, 0xde71, PDF_CMAP_SINGLE, 18847 },
	{ 0xde72, 0xde72, PDF_CMAP_SINGLE, 19001 },
	{ 0xde74, 0xde74, PDF_CMAP_SINGLE, 17421 },
	{ 0xde77, 0xde77, PDF_CMAP_SINGLE, 17140 },
	{ 0xde7b, 0xde7b, PDF_CMAP_SINGLE, 17417 },
	{ 0xde7e, 0xde7e, PDF_CMAP_SINGLE, 18416 },
	{ 0xde7f, 0xde7f, PDF_CMAP_SINGLE, 14075 },
	{ 0xde7f, 0xde7f, PDF_CMAP_SINGLE, 13849 },
	{ 0xde82, 0xde82, PDF_CMAP_SINGLE, 18745 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 17117 },
	{ 0xde84, 0xde84, PDF_CMAP_SINGLE, 18684 },
	{ 0xde86, 0xde86, PDF_CMAP_SINGLE, 18386 },
	{ 0xde89, 0xde89, PDF_CMAP_SINGLE, 18893 },
	{ 0xde8a, 0xde8a, PDF_CMAP_SINGLE, 19037 },
	{ 0xde8b, 0xde8b, PDF_CMAP_SINGLE, 18170 },
	{ 0xde8c, 0xde8c, PDF_CMAP_SINGLE, 18463 },
	{ 0xde93, 0xde93, PDF_CMAP_SINGLE, 18645 },
	{ 0xde95, 0xde95, PDF_CMAP_SINGLE, 17379 },
	{ 0xde98, 0xde98, PDF_CMAP_SINGLE, 17897 },
	{ 0xde99, 0xde99, PDF_CMAP_SINGLE, 18855 },
	{ 0xdea0, 0xdea0, PDF_CMAP_SINGLE, 18917 },
	{ 0xdea5, 0xdea5, PDF_CMAP_SINGLE, 18055 },
	{ 0xdeb0, 0xdeb0, PDF_CMAP_SINGLE, 14100 },
	{ 0xdeb1, 0xdeb1, PDF_CMAP_SINGLE, 18918 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 20072 },
	{ 0xdeb2, 0xdeb2, PDF_CMAP_SINGLE, 19129 },
	{ 0xdeb4, 0xdeb4, PDF_CMAP_SINGLE, 16838 },
	{ 0xdeb7, 0xdeb7, PDF_CMAP_SINGLE, 18466 },
	{ 0xdeb8, 0xdeb8, PDF_CMAP_SINGLE, 17667 },
	{ 0xdec2, 0xdec2, PDF_CMAP_SINGLE, 18327 },
	{ 0xdec4, 0xdec4, PDF_CMAP_SINGLE, 19046 },
	{ 0xdecd, 0xdecd, PDF_CMAP_SINGLE, 18856 },
	{ 0xded3, 0xded3, PDF_CMAP_SINGLE, 17337 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 17429 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 13922 },
	{ 0xded7, 0xded7, PDF_CMAP_SINGLE, 18784 },
	{ 0xded8, 0xded8, PDF_CMAP_SINGLE, 18325 },
	{ 0xdedb, 0xdedb, PDF_CMAP_SINGLE, 19054 },
	{ 0xdedd, 0xdedd, PDF_CMAP_SINGLE, 18863 },
	{ 0xdee3, 0xdee3, PDF_CMAP_SINGLE, 18277 },
	{ 0xdee4, 0xdee4, PDF_CMAP_SINGLE, 17428 },
	{ 0xdee4, 0xdee4, PDF_CMAP_SINGLE, 18276 },
	{ 0xdee4, 0xdee4, PDF_CMAP_SINGLE, 18862 },
	{ 0xdee8, 0xdee8, PDF_CMAP_SINGLE, 18329 },
	{ 0xdee9, 0xdee9, PDF_CMAP_SINGLE, 19051 },
	{ 0xdeeb, 0xdeeb, PDF_CMAP_SINGLE, 18895 },
	{ 0xdeec, 0xdeec, PDF_CMAP_SINGLE, 20083 },
	{ 0xdeee, 0xdeee, PDF_CMAP_SINGLE, 14282 },
	{ 0xdef0, 0xdef0, PDF_CMAP_SINGLE, 18959 },
	{ 0xdef1, 0xdef1, PDF_CMAP_SINGLE, 18278 },
	{ 0xdef3, 0xdef3, PDF_CMAP_SINGLE, 17632 },
	{ 0xdef3, 0xdef3, PDF_CMAP_SINGLE, 18747 },
	{ 0xdef6, 0xdef6, PDF_CMAP_SINGLE, 7673 },
	{ 0xdefa, 0xdefa, PDF_CMAP_SINGLE, 18787 },
	{ 0xdefd, 0xdefd, PDF_CMAP_SINGLE, 17435 },
	{ 0xdeff, 0xdeff, PDF_CMAP_SINGLE, 17063 },
	{ 0xdf06, 0xdf06, PDF_CMAP_SINGLE, 17492 },
	{ 0xdf0c, 0xdf0c, PDF_CMAP_SINGLE, 17863 },
	{ 0xdf0e, 0xdf0e, PDF_CMAP_SINGLE, 18650 },
	{ 0xdf19, 0xdf19, PDF_CMAP_SINGLE, 18962 },
	{ 0xdf1b, 0xdf1b, PDF_CMAP_SINGLE, 16816 },
	{ 0xdf1c, 0xdf1c, PDF_CMAP_SINGLE, 17854 },
	{ 0xdf1d, 0xdf1d, PDF_CMAP_SINGLE, 17340 },
	{ 0xdf1d, 0xdf1d, PDF_CMAP_SINGLE, 18430 },
	{ 0xdf1e, 0xdf1e, PDF_CMAP_SINGLE, 17575 },
	{ 0xdf1e, 0xdf1e, PDF_CMAP_SINGLE, 17605 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18330 },
	{ 0xdf23, 0xdf23, PDF_CMAP_SINGLE, 18652 },
	{ 0xdf2b, 0xdf2b, PDF_CMAP_SINGLE, 17282 },
	{ 0xdf32, 0xdf32, PDF_CMAP_SINGLE, 18897 },
	{ 0xdf36, 0xdf36, PDF_CMAP_SINGLE, 17437 },
	{ 0xdf3f, 0xdf3f, PDF_CMAP_SINGLE, 16914 },
	{ 0xdf42, 0xdf42, PDF_CMAP_SINGLE, 17493 },
	{ 0xdf44, 0xdf44, PDF_CMAP_SINGLE, 17438 },
	{ 0xdf46, 0xdf46, PDF_CMAP_SINGLE, 17680 },
	{ 0xdf4c, 0xdf4c, PDF_CMAP_SINGLE, 20311 },
	{ 0xdf4f, 0xdf4f, PDF_CMAP_SINGLE, 17312 },
	{ 0xdf4f, 0xdf50, PDF_CMAP_RANGE, 17671 },
	{ 0xdf50, 0xdf50, PDF_CMAP_SINGLE, 18966 },
	{ 0xdf52, 0xdf52, PDF_CMAP_SINGLE, 18656 },
	{ 0xdf56, 0xdf56, PDF_CMAP_SINGLE, 18117 },
	{ 0xdf5c, 0xdf5c, PDF_CMAP_SINGLE, 18332 },
	{ 0xdf5f, 0xdf5f, PDF_CMAP_SINGLE, 17391 },
	{ 0xdf63, 0xdf63, PDF_CMAP_SINGLE, 16916 },
	{ 0xdf64, 0xdf64, PDF_CMAP_SINGLE, 17867 },
	{ 0xdf6f, 0xdf6f, PDF_CMAP_SINGLE, 18119 },
	{ 0xdf71, 0xdf71, PDF_CMAP_SINGLE, 17291 },
	{ 0xdf71, 0xdf71, PDF_CMAP_SINGLE, 17018 },
	{ 0xdf72, 0xdf72, PDF_CMAP_SINGLE, 17761 },
	{ 0xdf76, 0xdf76, PDF_CMAP_SINGLE, 17582 },
	{ 0xdf7e, 0xdf7e, PDF_CMAP_SINGLE, 17983 },
	{ 0xdf81, 0xdf81, PDF_CMAP_SINGLE, 17289 },
	{ 0xdf8c, 0xdf8c, PDF_CMAP_SINGLE, 19109 },
	{ 0xdf94, 0xdf94, PDF_CMAP_SINGLE, 18544 },
	{ 0xdf9f, 0xdf9f, PDF_CMAP_SINGLE, 13803 },
	{ 0xdfa6, 0xdfa6, PDF_CMAP_SINGLE, 17683 },
	{ 0xdfa9, 0xdfa9, PDF_CMAP_SINGLE, 18248 },
	{ 0xdfad, 0xdfad, PDF_CMAP_SINGLE, 17608 },
	{ 0xdfb2, 0xdfb2, PDF_CMAP_SINGLE, 18293 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 18699 },
	{ 0xdfb4, 0xdfb4, PDF_CMAP_SINGLE, 18249 },
	{ 0xdfb7, 0xdfb7, PDF_CMAP_SINGLE, 13706 },
	{ 0xdfb7, 0xdfb7, PDF_CMAP_SINGLE, 13898 },
	{ 0xdfbc, 0xdfbc, PDF_CMAP_SINGLE, 18039 },
	{ 0xdfbe, 0xdfbe, PDF_CMAP_SINGLE, 18701 },
	{ 0xdfc1, 0xdfc1, PDF_CMAP_SINGLE, 18874 },
	{ 0xdfc4, 0xdfc4, PDF_CMAP_SINGLE, 17449 },
	{ 0xdfc7, 0xdfc7, PDF_CMAP_SINGLE, 18702 },
	{ 0xdfc8, 0xdfc8, PDF_CMAP_SINGLE, 20128 },
	{ 0xdfcd, 0xdfcd, PDF_CMAP_SINGLE, 17146 },
	{ 0xdfce, 0xdfce, PDF_CMAP_SINGLE, 19071 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 17768 },
	{ 0xdfd0, 0xdfd0, PDF_CMAP_SINGLE, 7838 },
	{ 0xdfd2, 0xdfd2, PDF_CMAP_SINGLE, 17764 },
	{ 0xdfd3, 0xdfd3, PDF_CMAP_SINGLE, 17763 },
	{ 0xdfd4, 0xdfd4, PDF_CMAP_SINGLE, 18339 },
	{ 0xdfd5, 0xdfd5, PDF_CMAP_SINGLE, 17770 },
	{ 0xdfd7, 0xdfd7, PDF_CMAP_SINGLE, 19071 },
	{ 0xdfda, 0xdfda, PDF_CMAP_SINGLE, 17772 },
	{ 0xdfda, 0xdfda, PDF_CMAP_SINGLE, 18611 },
	{ 0xdfdb, 0xdfdb, PDF_CMAP_SINGLE, 18610 },
	{ 0xdfdf, 0xdfdf, PDF_CMAP_SINGLE, 17774 },
	{ 0xdfe0, 0xdfe0, PDF_CMAP_SINGLE, 18338 },
	{ 0xdfe4, 0xdfe4, PDF_CMAP_SINGLE, 17769 },
	{ 0xdfe7, 0xdfe7, PDF_CMAP_SINGLE, 17875 },
	{ 0xdfeb, 0xdfeb, PDF_CMAP_SINGLE, 20130 },
	{ 0xdfef, 0xdfef, PDF_CMAP_SINGLE, 18875 },
	{ 0xdff1, 0xdff1, PDF_CMAP_SINGLE, 16970 },
	{ 0xdff2, 0xdff2, PDF_CMAP_SINGLE, 20059 },
	{ 0xdff6, 0xdff7, PDF_CMAP_RANGE, 18554 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 18553 },
	{ 0xdff8, 0xdff8, PDF_CMAP_SINGLE, 18903 },
	{ 0xdff9, 0xdff9, PDF_CMAP_SINGLE, 17295 },
	{ 0xdffa, 0xdffa, PDF_CMAP_SINGLE, 17585 },
	{ 0xdffb, 0xdffb, PDF_CMAP_SINGLE, 18345 },
	{ 0xdffe, 0xdffe, PDF_CMAP_SINGLE, 15422 },
	{ 0xdffe, 0xdffe, PDF_CMAP_SINGLE, 18617 },
	{ 0xdfff, 0xdfff, PDF_CMAP_SINGLE, 17874 },
	{ 0xf860, 0xf862, PDF_CMAP_RANGE, 15444 },
	{ 0xf87a, 0xf87a, PDF_CMAP_SINGLE, 15447 },
	{ 0xf87f, 0xf87f, PDF_CMAP_SINGLE, 15448 },
	{ 0xf909, 0xf909, PDF_CMAP_SINGLE, 13739 },
	{ 0xf91d, 0xf91d, PDF_CMAP_SINGLE, 13392 },
	{ 0xf91f, 0xf91f, PDF_CMAP_SINGLE, 14084 },
	{ 0xf928, 0xf928, PDF_CMAP_SINGLE, 20303 },
	{ 0xf929, 0xf929, PDF_CMAP_SINGLE, 20305 },
	{ 0xf936, 0xf936, PDF_CMAP_SINGLE, 13394 },
	{ 0xf95f, 0xf95f, PDF_CMAP_SINGLE, 13971 },
	{ 0xf970, 0xf970, PDF_CMAP_SINGLE, 13344 },
	{ 0xf983, 0xf983, PDF_CMAP_SINGLE, 14088 },
	{ 0xf992, 0xf993, PDF_CMAP_RANGE, 7809 },
	{ 0xf999, 0xf999, PDF_CMAP_SINGLE, 7811 },
	{ 0xf99a, 0xf99a, PDF_CMAP_SINGLE, 14097 },
	{ 0xf9a2, 0xf9a2, PDF_CMAP_SINGLE, 14095 },
	{ 0xf9c3, 0xf9c3, PDF_CMAP_SINGLE, 7808 },
	{ 0xf9d0, 0xf9d0, PDF_CMAP_SINGLE, 13396 },
	{ 0xf9dc, 0xf9dc, PDF_CMAP_SINGLE, 13393 },
	{ 0xf9ec, 0xf9ec, PDF_CMAP_SINGLE, 7750 },
	{ 0xfa03, 0xfa03, PDF_CMAP_SINGLE, 13956 },
	{ 0xfa0e, 0xfa0e, PDF_CMAP_SINGLE, 8410 },
	{ 0xfa0f, 0xfa0f, PDF_CMAP_SINGLE, 8421 },
	{ 0xfa10, 0xfa10, PDF_CMAP_SINGLE, 7746 },
	{ 0xfa11, 0xfa11, PDF_CMAP_SINGLE, 14290 },
	{ 0xfa12, 0xfa12, PDF_CMAP_SINGLE, 8481 },
	{ 0xfa13, 0xfa13, PDF_CMAP_SINGLE, 8497 },
	{ 0xfa14, 0xfa14, PDF_CMAP_SINGLE, 8499 },
	{ 0xfa15, 0xfa15, PDF_CMAP_SINGLE, 20307 },
	{ 0xfa16, 0xfa16, PDF_CMAP_SINGLE, 8548 },
	{ 0xfa17, 0xfa17, PDF_CMAP_SINGLE, 8571 },
	{ 0xfa18, 0xfa1a, PDF_CMAP_RANGE, 8579 },
	{ 0xfa1b, 0xfa1b, PDF_CMAP_SINGLE, 8583 },
	{ 0xfa1c, 0xfa1c, PDF_CMAP_SINGLE, 8587 },
	{ 0xfa1d, 0xfa1d, PDF_CMAP_SINGLE, 8590 },
	{ 0xfa1e, 0xfa1e, PDF_CMAP_SINGLE, 8599 },
	{ 0xfa1f, 0xfa1f, PDF_CMAP_SINGLE, 8610 },
	{ 0xfa20, 0xfa21, PDF_CMAP_RANGE, 8612 },
	{ 0xfa22, 0xfa22, PDF_CMAP_SINGLE, 8622 },
	{ 0xfa23, 0xfa23, PDF_CMAP_SINGLE, 8630 },
	{ 0xfa24, 0xfa24, PDF_CMAP_SINGLE, 18760 },
	{ 0xfa25, 0xfa25, PDF_CMAP_SINGLE, 8633 },
	{ 0xfa26, 0xfa26, PDF_CMAP_SINGLE, 8636 },
	{ 0xfa27, 0xfa27, PDF_CMAP_SINGLE, 8664 },
	{ 0xfa28, 0xfa28, PDF_CMAP_SINGLE, 8671 },
	{ 0xfa29, 0xfa29, PDF_CMAP_SINGLE, 8687 },
	{ 0xfa2a, 0xfa2b, PDF_CMAP_RANGE, 8699 },
	{ 0xfa2c, 0xfa2c, PDF_CMAP_SINGLE, 8702 },
	{ 0xfa2d, 0xfa2d, PDF_CMAP_SINGLE, 8715 },
	{ 0xfa30, 0xfa30, PDF_CMAP_SINGLE, 13382 },
	{ 0xfa31, 0xfa31, PDF_CMAP_SINGLE, 13360 },
	{ 0xfa32, 0xfa32, PDF_CMAP_SINGLE, 13389 },
	{ 0xfa33, 0xfa33, PDF_CMAP_SINGLE, 13385 },
	{ 0xfa34, 0xfa34, PDF_CMAP_SINGLE, 13338 },
	{ 0xfa35, 0xfa35, PDF_CMAP_SINGLE, 13378 },
	{ 0xfa36, 0xfa36, PDF_CMAP_SINGLE, 7651 },
	{ 0xfa37, 0xfa37, PDF_CMAP_SINGLE, 13366 },
	{ 0xfa38, 0xfa38, PDF_CMAP_SINGLE, 13333 },
	{ 0xfa39, 0xfa39, PDF_CMAP_SINGLE, 13384 },
	{ 0xfa3a, 0xfa3a, PDF_CMAP_SINGLE, 13387 },
	{ 0xfa3b, 0xfa3b, PDF_CMAP_SINGLE, 13361 },
	{ 0xfa3c, 0xfa3c, PDF_CMAP_SINGLE, 16837 },
	{ 0xfa3d, 0xfa3d, PDF_CMAP_SINGLE, 13326 },
	{ 0xfa3e, 0xfa3e, PDF_CMAP_SINGLE, 13328 },
	{ 0xfa3f, 0xfa3f, PDF_CMAP_SINGLE, 13363 },
	{ 0xfa40, 0xfa40, PDF_CMAP_SINGLE, 13369 },
	{ 0xfa41, 0xfa41, PDF_CMAP_SINGLE, 13381 },
	{ 0xfa42, 0xfa42, PDF_CMAP_SINGLE, 13334 },
	{ 0xfa43, 0xfa43, PDF_CMAP_SINGLE, 13352 },
	{ 0xfa44, 0xfa44, PDF_CMAP_SINGLE, 13375 },
	{ 0xfa45, 0xfa45, PDF_CMAP_SINGLE, 13327 },
	{ 0xfa46, 0xfa46, PDF_CMAP_SINGLE, 7700 },
	{ 0xfa47, 0xfa47, PDF_CMAP_SINGLE, 13332 },
	{ 0xfa48, 0xfa48, PDF_CMAP_SINGLE, 13347 },
	{ 0xfa49, 0xfa49, PDF_CMAP_SINGLE, 15398 },
	{ 0xfa4a, 0xfa4a, PDF_CMAP_SINGLE, 7732 },
	{ 0xfa4b, 0xfa4b, PDF_CMAP_SINGLE, 13379 },
	{ 0xfa4c, 0xfa4c, PDF_CMAP_SINGLE, 13348 },
	{ 0xfa4d, 0xfa4d, PDF_CMAP_SINGLE, 13345 },
	{ 0xfa4e, 0xfa4e, PDF_CMAP_SINGLE, 13335 },
	{ 0xfa4f, 0xfa4f, PDF_CMAP_SINGLE, 13391 },
	{ 0xfa50, 0xfa50, PDF_CMAP_SINGLE, 13359 },
	{ 0xfa51, 0xfa51, PDF_CMAP_SINGLE, 13351 },
	{ 0xfa52, 0xfa52, PDF_CMAP_SINGLE, 13325 },
	{ 0xfa53, 0xfa53, PDF_CMAP_SINGLE, 13371 },
	{ 0xfa54, 0xfa54, PDF_CMAP_SINGLE, 13343 },
	{ 0xfa55, 0xfa55, PDF_CMAP_SINGLE, 13373 },
	{ 0xfa56, 0xfa56, PDF_CMAP_SINGLE, 13358 },
	{ 0xfa57, 0xfa57, PDF_CMAP_SINGLE, 13399 },
	{ 0xfa58, 0xfa58, PDF_CMAP_SINGLE, 18366 },
	{ 0xfa59, 0xfa59, PDF_CMAP_SINGLE, 13376 },
	{ 0xfa5a, 0xfa5a, PDF_CMAP_SINGLE, 13353 },
	{ 0xfa5b, 0xfa5c, PDF_CMAP_RANGE, 13349 },
	{ 0xfa5d, 0xfa5d, PDF_CMAP_SINGLE, 14199 },
	{ 0xfa5e, 0xfa5e, PDF_CMAP_SINGLE, 14198 },
	{ 0xfa5f, 0xfa5f, PDF_CMAP_SINGLE, 13367 },
	{ 0xfa60, 0xfa60, PDF_CMAP_SINGLE, 13331 },
	{ 0xfa61, 0xfa61, PDF_CMAP_SINGLE, 13346 },
	{ 0xfa62, 0xfa62, PDF_CMAP_SINGLE, 13321 },
	{ 0xfa63, 0xfa63, PDF_CMAP_SINGLE, 13339 },
	{ 0xfa64, 0xfa64, PDF_CMAP_SINGLE, 13380 },
	{ 0xfa65, 0xfa65, PDF_CMAP_SINGLE, 13364 },
	{ 0xfa66, 0xfa66, PDF_CMAP_SINGLE, 15403 },
	{ 0xfa67, 0xfa67, PDF_CMAP_SINGLE, 13320 },
	{ 0xfa68, 0xfa68, PDF_CMAP_SINGLE, 13374 },
	{ 0xfa69, 0xfa69, PDF_CMAP_SINGLE, 13337 },
	{ 0xfa6a, 0xfa6a, PDF_CMAP_SINGLE, 7788 },
	{ 0xfb00, 0xfb00, PDF_CMAP_SINGLE, 9358 },
	{ 0xfb01, 0xfb02, PDF_CMAP_RANGE, 112 },
	{ 0xfb03, 0xfb04, PDF_CMAP_RANGE, 9359 },
	{ 0xfe10, 0xfe10, PDF_CMAP_SINGLE, 8268 },
	{ 0xfe11, 0xfe12, PDF_CMAP_RANGE, 7887 },
	{ 0xfe19, 0xfe19, PDF_CMAP_SINGLE, 7897 },
	{ 0xfe30, 0xfe30, PDF_CMAP_SINGLE, 7898 },
	{ 0xfe31, 0xfe32, PDF_CMAP_RANGE, 7892 },
	{ 0xfe33, 0xfe33, PDF_CMAP_SINGLE, 7890 },
	{ 0xfe35, 0xfe36, PDF_CMAP_RANGE, 7899 },
	{ 0xfe37, 0xfe38, PDF_CMAP_RANGE, 7905 },
	{ 0xfe39, 0xfe3a, PDF_CMAP_RANGE, 7901 },
	{ 0xfe3b, 0xfe3c, PDF_CMAP_RANGE, 7915 },
	{ 0xfe3d, 0xfe3e, PDF_CMAP_RANGE, 7909 },
	{ 0xfe3f, 0xfe40, PDF_CMAP_RANGE, 7907 },
	{ 0xfe41, 0xfe44, PDF_CMAP_RANGE, 7911 },
	{ 0xfe45, 0xfe46, PDF_CMAP_RANGE, 12639 },
	{ 0xfe47, 0xfe48, PDF_CMAP_RANGE, 7903 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 642 },
	{ 0xff02, 0xff02, PDF_CMAP_SINGLE, 8007 },
	{ 0xff03, 0xff03, PDF_CMAP_SINGLE, 716 },
	{ 0xff04, 0xff04, PDF_CMAP_SINGLE, 712 },
	{ 0xff05, 0xff05, PDF_CMAP_SINGLE, 715 },
	{ 0xff06, 0xff06, PDF_CMAP_SINGLE, 717 },
	{ 0xff07, 0xff07, PDF_CMAP_SINGLE, 8006 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 674 },
	{ 0xff0a, 0xff0a, PDF_CMAP_SINGLE, 718 },
	{ 0xff0b, 0xff0b, PDF_CMAP_SINGLE, 692 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 636 },
	{ 0xff0d, 0xff0d, PDF_CMAP_SINGLE, 693 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 637 },
	{ 0xff0f, 0xff0f, PDF_CMAP_SINGLE, 663 },
	{ 0xff10, 0xff19, PDF_CMAP_RANGE, 780 },
	{ 0xff1a, 0xff1b, PDF_CMAP_RANGE, 639 },
	{ 0xff1c, 0xff1c, PDF_CMAP_SINGLE, 699 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 697 },
	{ 0xff1e, 0xff1e, PDF_CMAP_SINGLE, 700 },
	{ 0xff1f, 0xff1f, PDF_CMAP_SINGLE, 641 },
	{ 0xff20, 0xff20, PDF_CMAP_SINGLE, 719 },
	{ 0xff21, 0xff3a, PDF_CMAP_RANGE, 790 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 678 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 664 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 679 },
	{ 0xff3e, 0xff3e, PDF_CMAP_SINGLE, 648 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 650 },
	{ 0xff40, 0xff40, PDF_CMAP_SINGLE, 646 },
	{ 0xff41, 0xff5a, PDF_CMAP_RANGE, 816 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 680 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 667 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 681 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 665 },
	{ 0xff5f, 0xff60, PDF_CMAP_RANGE, 12131 },
	{ 0xff61, 0xff9f, PDF_CMAP_RANGE, 327 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 713 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 751 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 649 },
	{ 0xffe4, 0xffe4, PDF_CMAP_SINGLE, 8005 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 711 },
	{ 0xffe8, 0xffe8, PDF_CMAP_SINGLE, 323 },
};

static const unsigned short pdf_cmap_UniJIS_UTF16_H_table[10909] =
{
	99,94,100,107,61,93,720,647,
	152,140,109,153,151,154,129,707,
	694,645,159,778,117,134,160,144,
	123,695,142,696,148,9366,9361,15756,
	15769,15737,15745,15758,15771,15783,15789,20333,
	20352,15759,15772,15761,15774,20322,15775,9369,
	9364,20334,20353,15760,15773,9395,9407,15784,
	15790,20335,20355,20337,20356,20336,15785,15791,
	20323,15816,9400,9412,9367,9362,20339,20357,
	20338,146,20324,20328,15786,15792,20340,20358,
	20329,15757,15770,20342,20360,15739,15747,20325,
	20330,141,147,15762,15776,20343,20361,15763,
	15777,20331,20326,9436,9370,9365,15764,15778,
	143,149,15755,15768,20344,20362,15765,15779,
	15740,15748,15787,15793,15741,15750,223,227,
	15767,15781,15742,15751,20327,20332,9405,9417,
	9368,9363,15788,15794,9404,9416,15766,15780,
	20345,20363,20350,20364,20351,20365,224,15743,
	15752,15744,15754,225,229,105,15826,9394,
	9406,9398,9410,9401,9413,9403,9415,20349,
	15733,20346,15734,20348,15735,20347,15736,15832,
	9418,15836,15822,9423,15841,15802,15823,15829,
	9426,9429,9432,15830,15831,15809,15825,15813,
	15835,15838,15819,15844,15827,15798,15808,15799,
	15833,15814,15795,15810,15803,15886,9437,15887,
	15888,15800,15843,15807,15804,15797,15889,15815,
	15805,9442,15824,15801,15828,15834,15796,9438,
	15837,15812,15890,15806,15842,9441,15818,15817,
	15820,15891,15892,15811,15840,15839,98,96,
	128,15749,15846,9443,15848,15738,15782,133,
	15746,95,15753,15867,95,129,226,133,
	135,137,15868,15858,134,136,15865,15876,
	15866,15869,15870,15864,20433,20435,20441,20431,
	20434,20440,20444,12090,12096,9397,9409,9420,
	9419,9434,9433,231,633,662,14,114,
	114,138,661,666,98,96,120,108,
	122,121,669,668,734,12111,226,15849,
	16328,11035,8305,8304,8055,228,9355,15515,
	736,739,12201,12207,12206,8013,8012,8014,
	8011,759,755,12184,20367,760,741,15476,
	12091,742,7625,693,12118,117,765,767,
	703,7629,756,748,747,8195,7624,704,
	768,100,766,698,761,15505,12188,12186,
	12189,12187,12143,12167,12167,12146,12151,12167,
	12167,12154,8250,8249,8244,8243,8242,8241,
	8240,8239,8238,729,728,8015,12239,12237,
	16235,731,730,12195,8010,733,732,12194,
	8009,727,726,8210,20371,723,10502,725,
	724,8258,8257,12254,722,721,8219,8222,
	8211,8017,8019,8213,8018,8212,8214,8016,
	775,16200,12100,774,16199,773,12229,12228,
	4209,14476,4646,3762,4739,4779,15391,14530,
	13852,14561,5059,17893,3644,3773,14749,15398,
	14780,13729,13995,14999,19130,14905,13922,15000,
	14999,14189,13747,13646,1931,15114,13870,2658,
	15403,15184,3029,15255,15262,13645,2664,13847,
	13849,13848,3380,1323,1200,8371,4095,4097,
	1333,4102,3275,4110,2579,4208,3286,3392,
	4219,4227,4233,4243,4248,3163,3991,4294,
	4307,2375,3708,4316,4321,4328,3746,1969,
	4459,3156,2204,3878,2887,2433,2208,4622,
	2631,2454,4646,4648,4658,2177,4716,1979,
	1918,1738,1519,4739,4741,4761,4763,4768,
	1655,14521,4783,4785,2554,4930,1921,2326,
	2215,5058,3592,3143,1740,3661,5088,3284,
	5132,1860,3814,1853,2221,5349,5364,5368,
	3450,3807,2223,5378,2603,1360,3066,3541,
	5604,5606,3618,1383,1671,1880,1904,1732,
	1245,1504,1537,2652,3899,3134,3479,14848,
	5783,3368,3453,2172,3816,3779,3836,2676,
	2260,14913,1363,1856,3953,2971,3606,2227,
	1544,6163,3901,1227,4061,2261,6205,2262,
	6227,3281,2569,2263,2232,1235,2697,2726,
	2360,2081,2541,6322,6479,2988,1858,2022,
	1189,6635,1887,1455,1908,2921,3198,6742,
	6745,1419,2682,2808,2829,2574,2306,2575,
	2914,15183,3874,3243,3428,3948,1754,3029,
	3827,3550,7113,7115,1229,8695,3463,3800,
	1461,7171,7173,1339,3607,3561,3464,2543,
	2335,2035,3333,2062,2036,7276,7293,1614,
	1685,3031,7414,2267,7425,3729,13323,1642,
	2055,7446,7449,3102,1937,2767,3475,655,
	8308,735,740,665,7608,7609,8058,12106,
	16194,12179,12181,16205,638,660,8143,8141,
	8148,8147,8204,8142,7620,8151,8149,8144,
	8139,8146,8140,8150,8205,8145,8138,10471,
	8161,8160,8162,8156,10495,8165,10492,8158,
	8223,10489,10488,10494,8319,8191,10479,10491,
	10490,8154,8157,8159,8163,8153,8164,8155,
	8152,8042,11876,8183,11877,11881,11879,11884,
	11882,11886,11888,7595,7586,8041,8039,11896,
	8040,11898,8038,8043,11907,11909,7596,7590,
	8052,7598,8049,11921,8327,7592,11930,11932,
	8046,11933,8047,11926,11934,11935,8045,8050,
	11943,7585,7599,7588,8328,7593,11950,11954,
	11951,8323,7623,7622,7621,8054,8037,8024,
	8026,8186,8020,7607,8021,8187,8030,8029,
	8028,8027,8034,8182,7611,15387,17242,17533,
	17532,17731,17758,17826,13965,16968,20170,18634,
	19136,15441,18669,1200,3000,17234,2275,3754,
	2510,2174,2509,1340,19140,3526,3881,17235,
	4091,1233,17236,1484,4092,2632,4311,1648,
	3594,2511,14298,8371,17237,4093,14299,14157,
	2980,17238,4094,1778,21075,4095,13981,1561,
	2926,2323,4096,4098,3307,14304,1649,17239,
	14126,3259,1911,3681,6480,4099,2512,4100,
	1333,14305,21076,1757,1956,3829,4659,21077,
	21078,17244,1505,14306,1615,4102,3971,3879,
	2794,4104,2244,3275,14307,4107,1248,1939,
	1938,1194,14308,21081,4081,4080,2083,1125,
	17245,3682,4111,1958,1195,3744,3070,3972,
	21082,4112,14309,2579,13856,17247,2372,2580,
	4119,17248,4117,4118,1650,17249,2067,1392,
	4116,4115,3577,14310,2196,2195,2846,4120,
	3527,2699,17251,21083,656,4121,4123,14311,
	8372,21084,2885,4009,1166,4122,1342,21086,
	1724,17252,2981,19144,17253,1861,4124,3290,
	8373,14312,21087,14313,8374,1575,21088,8375,
	21089,4125,1167,14314,19145,1940,1576,3564,
	3398,1651,21090,14315,17254,21091,19146,1393,
	4160,3131,19148,3362,4127,19149,21092,3408,
	21093,4010,17256,2547,8376,2197,14317,2245,
	1344,17257,21095,3053,2912,4131,16779,14318,
	1168,3071,2373,2084,3854,21098,2862,14319,
	1343,8377,4130,17258,3880,4126,4128,2142,
	4129,4563,16780,14316,21099,17255,4137,16781,
	21100,4140,4138,21101,3595,4132,21102,14320,
	4136,1959,21103,17262,1506,19150,17263,4141,
	4133,21104,8379,4011,21105,2246,21106,4134,
	21107,4139,8378,21108,8381,4142,14323,4135,
	21109,8380,1689,1169,19151,21110,1690,1345,
	3751,3552,1960,2549,3967,19152,14324,3624,
	1806,2821,1380,17266,21117,21118,8364,2397,
	17267,21119,8382,4146,14325,4151,4149,17268,
	15407,4144,2831,4147,4150,4148,21120,3629,
	4145,7660,2548,21121,3745,2350,4166,21122,
	13731,17270,3334,3496,4161,21123,14326,4165,
	8385,19153,4159,4168,21124,2772,21125,1912,
	19154,3346,16783,5632,14329,4167,3159,17271,
	4156,1962,21126,16784,1961,4154,14330,17272,
	21127,8386,2310,4162,8384,3647,2955,4158,
	1863,14331,4155,4163,4157,3993,4164,4071,
	14332,21131,1758,8383,16785,8389,4169,21132,
	8387,4170,4174,1170,21133,17273,3616,4173,
	4172,4176,14333,21136,4175,19156,21137,19157,
	17275,1864,14334,21138,4177,21139,8388,21140,
	2289,2822,3073,1774,19158,4178,4179,21141,
	4181,21144,3683,19159,14337,21146,1852,8390,
	17280,14338,2175,3467,4180,21147,14339,19161,
	17276,2101,3885,17283,4183,4186,4182,2100,
	21154,1807,4187,19162,17286,1735,14340,3207,
	17290,2814,16787,1691,21156,17292,3707,4188,
	8392,15408,3973,4189,21159,17293,21160,4192,
	21161,4190,14341,2768,19163,14342,4191,4193,
	14343,15409,19164,16786,17288,8391,4195,21162,
	4194,21163,3608,16789,1327,16788,19166,14344,
	4196,4201,4200,4199,17296,4143,21165,4202,
	14347,14348,2440,21174,3813,19170,21175,16790,
	4205,19171,21176,4204,4207,4206,1208,21177,
	1897,1809,2374,3001,1692,2700,1963,8393,
	2048,4211,3796,3136,21178,2247,4210,21179,
	4212,16791,16792,21180,14349,17298,8394,3286,
	21181,13966,2742,3392,1964,4065,4217,17300,
	19172,3596,2838,1769,3119,14201,18393,1865,
	4219,17301,19173,3258,1281,4222,2157,17302,
	4221,2102,21183,4223,6235,4224,3695,4225,
	2513,17304,2296,1507,14350,4230,17305,4228,
	3785,4229,3161,17306,4234,2131,4238,3830,
	4012,19174,14351,4239,8396,21184,21185,15410,
	2636,4240,2404,19176,4241,16794,3002,3975,
	3162,19178,21186,21187,4314,4242,8284,1725,
	20300,19182,4243,3724,16795,21188,2418,2908,
	4245,3260,8397,4246,14353,4247,1420,21189,
	17309,14354,4248,1695,21190,3236,1308,2394,
	3381,4249,2581,4250,19183,3580,2686,1502,
	1509,4251,1808,17310,14357,4252,8398,17311,
	4027,21191,3409,3612,19184,4254,19185,3938,
	4255,19186,19189,2637,2158,1866,4258,2199,
	2049,19190,3074,4260,21193,14358,2143,4264,
	17313,3684,16796,21194,19191,2038,17314,7774,
	4263,21195,19192,14359,1867,2126,3363,14360,
	4267,4265,19193,3565,2514,4274,1474,4268,
	4266,2769,4269,21196,17318,1442,17317,21197,
	1846,4275,3957,19196,19197,4271,4276,4273,
	14361,4272,21201,3991,8399,1965,1347,4028,
	17322,21202,8400,21203,2431,3154,2039,8573,
	21204,4013,4049,4282,4281,21208,1421,8401,
	4283,3716,21209,3032,21210,3856,14362,3625,
	17325,16797,4284,21211,14056,17326,7150,21212,
	3208,16798,4285,1510,3775,8402,21213,2441,
	4286,3639,4290,19198,2638,4287,1736,21214,
	4288,1511,21215,19199,21216,14363,19200,1796,
	4291,21218,4292,21219,17328,2311,16799,1967,
	3818,8403,3828,3279,19201,3649,4295,20301,
	4296,21220,4298,4300,4299,19202,4301,1341,
	3706,21222,2156,4302,21223,16800,2779,21224,
	19203,2442,1697,4303,8405,21225,4304,21231,
	4305,19205,4306,4307,3478,1760,1193,21232,
	19206,14366,3223,4308,2375,17330,2701,4310,
	4309,2443,1941,4312,3410,14368,14367,4313,
	3440,2836,2894,1696,3270,2927,21233,3364,
	21234,3708,2702,14369,19207,1803,17333,14370,
	19208,4317,1230,1209,1577,8406,2824,1643,
	3931,4320,1335,19209,17334,4319,13365,19210,
	1698,4321,21236,3837,8407,19211,3994,19212,
	1968,1898,4323,21240,17335,4325,4324,2597,
	1243,19213,14371,1280,4326,19214,4327,8408,
	1899,17336,21241,4328,17338,21242,19215,1672,
	17339,19216,2176,4329,3746,2085,1652,3857,
	2770,3411,2345,21244,2385,17343,2324,2337,
	2432,14372,3412,8409,21247,4332,19219,1253,
	2771,1969,1913,1759,14373,21249,4336,2911,
	2910,1699,2444,4337,4335,1348,2886,2276,
	2201,1224,17345,14374,1486,2040,2200,4339,
	21250,1635,1444,2041,1634,3067,1223,3209,
	3786,1971,3939,3137,1970,16803,19220,21251,
	1797,4348,13964,1755,3704,21252,3441,14375,
	17347,4347,19221,1562,4345,21253,1653,2599,
	3581,4344,4340,1943,13775,4341,4042,17349,
	21254,3650,21255,3076,1942,2050,19225,14376,
	4349,21256,21257,4353,14377,21259,2346,21260,
	2338,14378,17351,4356,4354,3759,14379,4351,
	4360,4355,4358,1914,3787,16804,4359,21263,
	21264,4361,21265,4363,16805,21266,8412,2144,
	4072,14380,4352,4350,4362,21267,4357,17352,
	14381,21268,17353,14382,4365,4374,4367,17354,
	19226,4371,8413,19227,4372,4368,14383,17355,
	4401,19228,2137,1423,17356,4366,14384,17357,
	19229,4376,1210,4375,14385,1129,3516,4373,
	17358,4369,14386,4364,4370,2104,21269,14387,
	19230,21270,1211,4386,2445,3735,19231,17360,
	4384,4383,17361,17362,3113,17363,19232,4382,
	21272,8414,1238,19233,2086,2550,19234,17366,
	14388,4380,3164,4381,19235,1126,19237,14389,
	4391,3853,2447,21275,4397,14390,19238,4396,
	4387,21276,17367,21277,2851,4388,17368,2895,
	4393,2446,17370,21278,14391,4390,21279,3824,
	16806,19239,4392,4398,7633,14392,20308,4389,
	17372,4404,4409,17374,4405,21283,4400,16807,
	17375,4410,2739,8415,4412,14394,1972,4402,
	3003,14395,19241,17376,19242,21284,19243,21285,
	4406,4399,1513,1578,1475,4407,4403,21286,
	19244,1868,4413,4411,2773,1636,1700,16808,
	4408,19246,4417,4415,19247,4472,21288,17383,
	17384,17378,19248,4420,4414,19249,4418,17385,
	21291,4416,17387,2202,4419,19253,4422,19254,
	4427,4425,4424,21292,19255,2928,17392,14396,
	1349,21294,4423,2448,1237,21295,17390,1374,
	4437,17394,2532,4433,21296,4431,19260,4432,
	14399,4434,19261,16810,21297,17396,14400,17397,
	21298,1247,21301,17398,14402,2747,19262,4428,
	14403,4429,7963,19263,1579,14405,4439,4435,
	4438,17401,3582,21305,15411,21306,19264,4440,
	1443,17403,4442,21307,17404,4444,14409,7654,
	14410,17405,16811,19265,4443,4446,19266,21308,
	17406,16812,14413,17407,16813,4449,4448,19267,
	4454,4452,4450,4453,4455,14414,7770,21311,
	19268,17409,4456,19269,21312,4457,4459,4220,
	19270,2344,2203,21313,14415,1395,17410,1212,
	19271,2946,14416,19273,4460,2068,19274,1171,
	2596,4461,1915,4463,4462,3632,4464,19275,
	4466,4465,14417,4467,19276,4468,21319,1282,
	4469,14418,4471,4473,19279,3156,21320,13952,
	21321,14419,4474,1145,2127,17416,21324,19280,
	1811,19281,14420,2957,21325,4479,14423,17420,
	4477,4480,2097,1973,21326,4484,21327,2069,
	8417,2929,21328,17422,4481,3062,19285,19286,
	21329,17423,14424,21330,21331,19287,17427,16815,
	21332,4485,21333,2600,19288,4483,4486,1813,
	14425,4487,21334,21338,21335,17430,21336,4488,
	21337,1974,1438,4490,21339,17433,21340,4491,
	4494,8420,8419,3730,17436,2515,16817,4495,
	19291,4499,17434,3310,21342,17439,1196,3528,
	21347,4500,16818,14428,19292,2277,17442,3347,
	1580,17443,2139,17444,21344,14432,3719,3210,
	21345,17445,1870,2863,21346,14433,4498,4501,
	21348,19293,4503,17446,21350,14434,4505,19294,
	17447,7474,1283,3651,14435,2516,3138,17448,
	2134,21352,3597,4005,17450,1396,4507,17451,
	19295,19296,21353,2748,4510,3165,21354,3139,
	3166,3405,3049,2105,19297,7751,4506,17452,
	21355,1304,21356,4512,2392,19299,14437,19300,
	21359,17455,3640,17458,21360,2815,19301,3042,
	17459,8423,4515,3709,14438,17460,4516,17461,
	8424,3583,4520,4514,4517,4519,19302,3609,
	21366,17463,21365,4522,2947,21367,2517,17464,
	14439,14440,4524,16822,4523,14441,2042,21368,
	4525,4527,4526,14442,4529,21369,4518,4531,
	16823,21370,17465,4530,4528,2204,2583,2774,
	4532,2656,1201,3354,4534,4533,3617,4539,
	8425,14444,17470,1350,4540,17471,3878,1422,
	4318,2386,2847,4541,3831,21374,21375,3776,
	19304,14445,4543,2887,21376,3121,2848,3529,
	1309,14446,21377,2278,4546,21378,1172,4547,
	14117,17474,4548,21381,21382,1284,17476,1581,
	3256,3652,4552,2775,4551,1814,21383,8426,
	3721,4550,3167,4554,19305,4553,8427,8428,
	21384,17479,4556,14447,4555,8429,4557,2449,
	4559,2915,21385,4558,14448,3587,21386,21387,
	2433,3157,4560,19306,21388,17483,1975,4561,
	3287,3442,3805,3291,16825,21389,4570,21390,
	16826,1618,21391,3887,19307,4565,8430,2853,
	21392,3686,3140,21393,17485,19308,21394,4566,
	19309,3731,8431,2106,17486,16827,21397,17487,
	2206,14449,2205,1149,1916,14450,2639,1173,
	4569,14451,17489,19310,1242,1515,17490,4568,
	3793,3491,13997,17491,21398,19311,21399,1132,
	21401,19312,2207,21402,1174,1127,14454,14455,
	17495,21403,4575,21404,16828,17496,3784,4578,
	13761,4576,21405,4574,2551,14456,21406,4573,
	17497,21407,3626,21408,1944,21409,4579,4050,
	3330,14457,21412,4581,4584,14459,17499,3530,
	16829,4580,14460,17500,21416,14461,3783,3348,
	21418,21420,4586,3492,17502,17506,19313,14465,
	17508,21421,4587,4591,4588,17509,1351,4590,
	21423,2279,4589,1871,21424,4603,4596,4593,
	14466,2978,17512,4594,4595,19315,17515,19316,
	14467,21426,17516,1582,4600,4599,4601,17517,
	19319,2518,3064,14470,4609,14471,21427,17519,
	2208,4610,17520,1976,4611,8432,2248,2840,
	2216,1977,21428,3806,1602,1917,4614,1462,
	17521,4615,21429,17522,4616,4656,4618,14120,
	14472,16834,4622,17523,13840,4623,17524,2896,
	21430,1225,2325,1158,14475,2777,1516,2273,
	21431,1978,16835,21434,3168,17526,2347,1517,
	2982,3078,1148,1619,3653,1644,2703,2280,
	3858,4624,17527,21437,1654,21438,2107,1424,
	1285,2452,1352,19321,4625,3888,21441,2387,
	8433,21443,2320,4626,1583,3242,3765,4627,
	3531,21444,17530,21445,4630,1508,1775,4629,
	17531,19322,8435,2552,4634,2159,19323,1353,
	4633,4631,4635,4632,3297,5262,2553,4636,
	20302,3976,21448,4637,17534,4639,21449,3004,
	4638,17535,2864,2339,3559,2704,17537,2297,
	4640,2453,1175,2842,2584,21450,4643,3211,
	2454,2455,16836,4644,14122,2705,21451,13835,
	2456,8437,19324,4645,4646,14476,3820,4647,
	17538,19325,14477,19326,1726,21453,14478,2348,
	21454,21455,19327,2312,2546,3276,2586,3468,
	3288,1729,4650,1673,4651,3239,1328,2209,
	4652,4655,4654,1781,19328,21456,7826,19329,
	2832,17540,3141,2292,7693,14479,2778,3940,
	4657,19330,4658,3246,17543,2177,4661,16839,
	21459,14481,21460,21461,17548,16840,1584,4663,
	21462,16841,17553,1324,17554,16842,8438,17555,
	2749,1568,16843,4666,3764,17557,2866,21464,
	1463,21465,17558,4668,4670,1563,8439,4667,
	4669,3221,1702,1381,4675,4680,4678,21467,
	3655,3654,14484,8440,3169,4677,2398,17563,
	2616,16846,2138,17564,4687,21470,4688,4682,
	1425,4683,4692,4691,4690,4686,4685,17567,
	4689,21471,14485,14486,8441,8444,14487,14488,
	16847,21473,14489,4696,4693,3932,21475,4694,
	8442,17571,21478,2617,17572,4697,8445,2087,
	17574,19336,21482,16848,4702,19337,4701,4700,
	17577,21483,3170,4679,14490,21484,14491,21485,
	21487,14493,21488,4705,4710,4707,16851,1564,
	4712,21490,8448,17583,4714,4713,4715,17584,
	14494,21491,2706,2349,14495,2414,13362,2789,
	17586,1979,2088,1980,1674,17587,1918,4718,
	3762,3321,4719,1981,14496,19339,2917,1738,
	13794,14497,2210,3533,3413,21492,4720,1585,
	21493,4723,14498,14501,4722,4721,4724,17591,
	21494,2670,14502,2867,1596,19341,3006,3687,
	17592,4728,4727,3567,21495,3720,4732,3737,
	17595,4731,14503,19343,21498,16852,4733,3388,
	4734,3598,4735,1519,3599,3301,19346,1982,
	1520,4739,1900,3886,3859,1586,4741,19347,
	3007,1983,2457,2458,19348,21500,2434,3080,
	3657,3123,14000,1984,21502,17600,3155,2098,
	14506,14507,1919,17601,3081,1159,2424,1985,
	3889,17602,4745,4031,4051,15390,21504,21505,
	4747,4746,19351,14511,1445,21506,4750,21507,
	3506,2459,4754,4753,4749,21509,4757,4760,
	4758,4759,4761,1286,3082,4762,16853,1872,
	1398,3308,17606,4763,3283,16854,3627,16855,
	4764,4052,21510,14512,17607,4765,3600,4768,
	4090,4106,14513,2268,3277,4769,1655,3008,
	1214,4770,3574,1986,21511,17609,3083,8449,
	14516,16856,21512,3835,1901,1920,14517,4771,
	4772,21515,4778,21516,2321,8450,3009,14518,
	1703,4773,13720,14519,3485,16857,2948,21517,
	14520,4774,21518,8370,17610,4775,4776,19354,
	4777,14521,4779,3184,16858,14522,4782,21519,
	4767,4766,21520,4783,1815,17611,14523,13996,
	3481,8451,2108,3497,3010,3517,4784,21521,
	2460,1256,17612,4785,4786,21524,3838,21525,
	3444,16859,17613,4789,1311,2640,4788,4787,
	1816,2868,4793,4791,14524,4790,3951,1945,
	21526,17614,2435,4792,3142,2376,21527,3224,
	4796,4795,14525,21528,4794,4797,1946,17615,
	14526,4798,3566,2405,21529,19355,4799,3469,
	14528,3225,3011,13368,8452,14529,3114,4800,
	1605,2554,14530,3486,17616,21532,14531,17617,
	1587,3292,4801,2212,1312,4806,8453,2983,
	14532,17621,21535,14533,19356,1399,21536,14534,
	4854,4805,21537,17624,3302,21538,21539,4802,
	14535,2060,16860,4814,4820,4812,3158,21541,
	16861,4817,3535,14536,16862,4811,14537,4816,
	4015,2213,21542,2869,4809,17626,17627,1656,
	4819,2641,1287,4813,1400,4818,21544,4823,
	4833,4831,4828,16864,21545,19358,4827,4032,
	17629,4829,1706,21546,1987,19359,2436,4836,
	4822,4826,4810,14539,1402,4830,4832,2959,
	19360,2072,1336,4824,4835,4834,1707,2825,
	1476,21549,21550,14540,4837,21551,4840,4842,
	8455,21552,21553,2281,8456,4848,3084,4838,
	21554,21555,4846,21556,1401,17630,4841,4843,
	16865,14541,1947,3860,21559,1521,14542,21560,
	1275,4847,17631,3312,1137,14543,17633,3445,
	4807,4853,4857,3825,21561,4850,19361,3171,
	4855,17634,19362,2520,4856,3247,17635,4852,
	16866,8458,4858,17636,2061,14545,2671,16867,
	8457,1176,4851,4849,21567,2780,4839,2178,
	2854,4870,8460,2781,4865,17638,14546,2322,
	4866,4869,21569,19363,4864,2351,21570,4867,
	4861,21571,3848,3847,19364,1177,17639,8461,
	4860,19365,21573,17640,1770,1130,14550,17641,
	1522,8459,4868,21574,4876,4875,19366,16868,
	4880,8463,21575,4881,21576,19367,8462,21577,
	17642,4879,4882,4874,4878,17644,4883,4884,
	21578,17645,4873,2250,21579,4877,2870,1988,
	4859,2555,4896,19368,4895,17646,3755,1523,
	19369,4893,1819,1426,4890,21580,3968,4892,
	1178,4894,21581,4885,4891,4898,1818,4886,
	21582,19370,14551,19371,3911,19372,4905,21587,
	14552,2816,4033,4906,4900,17650,8464,4899,
	4904,21588,19373,14553,21589,3584,17651,3212,
	14554,1820,21590,4907,4902,17652,4908,19374,
	16870,4916,14555,4914,4915,2073,4913,4911,
	4910,4917,4909,4919,19375,17655,21595,14556,
	21596,19376,4921,19377,4920,21597,19378,21598,
	4924,17656,4922,4912,1874,4927,4926,4928,
	4925,4929,21602,21603,19379,4933,4932,2377,
	2642,1382,1404,8465,4934,14557,1155,2672,
	4935,21604,6756,4936,1847,21605,4937,16871,
	17658,2707,21606,14558,4938,21607,4939,1620,
	4940,19381,1921,19382,19383,13390,3690,2420,
	4943,21608,14559,21609,14560,2708,6938,3446,
	2326,14561,2109,4944,21610,14562,17659,2855,
	21611,3575,14563,16872,4947,21612,4948,17660,
	4946,17661,17663,3585,16873,3447,19384,17662,
	4950,21613,4953,2461,1621,4951,4958,2462,
	13765,4952,3322,19385,3912,4959,3172,4956,
	1989,2690,17665,4973,3400,3448,5042,3747,
	21620,4962,1313,2984,14564,2930,4965,4972,
	4967,4974,4970,4969,3365,21623,4963,1405,
	4961,1675,2899,4957,17666,16874,4960,1990,
	2687,19387,2463,4968,3336,1676,1446,1477,
	2535,4976,4981,4977,21625,1875,19388,4982,
	2160,2043,14565,17668,2352,4964,2251,4979,
	17669,21626,2214,4980,1160,14566,21627,4975,
	14567,17670,3013,14568,1677,4978,1131,21629,
	19390,2099,17674,19391,14569,3086,3432,4984,
	2784,2826,21631,19393,2169,4985,17679,4987,
	4983,21632,19394,21633,3633,3033,21634,17681,
	21635,2782,14572,21636,3659,2298,4999,4998,
	17684,21637,2622,19395,1876,14573,19396,4992,
	2465,21638,3264,3303,16875,14574,19397,4990,
	19398,2783,14575,14576,2340,4995,19399,4989,
	4994,14577,17687,21639,4988,1783,17685,21640,
	1467,14578,4996,3955,2110,2931,4993,21641,
	2688,21642,1991,2602,1288,2750,4991,1631,
	21643,3051,4997,17688,5000,5002,21644,2839,
	5008,21645,5003,21646,5005,21647,3507,3087,
	14579,5006,21648,3890,1525,14580,17691,1138,
	5001,19402,13340,1588,17694,21649,1289,8467,
	5007,13892,14581,21650,5018,14582,19405,5012,
	7724,5009,5016,19406,19408,17696,5013,5017,
	17697,19409,3414,3173,17698,21651,5010,19410,
	5014,19411,2145,19412,5022,21653,7747,19413,
	21654,14584,17699,14585,21655,17700,19414,8468,
	21656,17701,19415,3802,16877,2630,14587,17703,
	5028,21659,19416,17704,2179,5025,3304,21660,
	8469,3213,17705,21662,3115,5026,21663,21664,
	5027,3553,19419,3323,2161,3710,19420,5034,
	5029,21665,3892,5036,16878,5032,21666,13341,
	2786,8470,16879,21667,5031,4966,17708,17709,
	5035,3105,5043,17711,14590,2162,5038,1622,
	19421,5044,19422,5037,5048,5047,21670,5046,
	14591,5049,14592,5051,2521,17712,5050,21671,
	14593,5054,21675,17715,5052,14595,19423,5053,
	5015,17717,19424,7831,5056,5055,16880,5023,
	5057,5045,19425,2215,21677,17718,5061,5060,
	5062,1406,21678,1992,3660,2643,21681,17719,
	1922,14596,5064,19426,5067,8471,3524,21682,
	1657,21683,17720,5066,5065,3338,5068,1709,
	5070,5069,21684,19427,1526,2180,3248,14597,
	17721,1823,2644,3106,3537,5072,17723,14599,
	3592,4620,2666,17724,3449,3415,19428,3143,
	21686,3977,5076,2300,16881,5077,21687,1146,
	21688,1740,2673,21689,3538,5078,2192,2949,
	21690,2217,2557,17725,21691,19430,17726,5079,
	17727,3661,1305,2218,19431,17728,5082,14600,
	5080,5083,3969,5081,19432,17729,2719,5084,
	19433,2834,19434,21692,1590,21693,5087,1591,
	13701,3284,2932,1670,2219,2787,2406,1140,
	14601,5090,14602,17732,1314,5094,17733,1993,
	5093,17734,2075,2466,17735,8474,5092,2467,
	21699,3788,2074,1179,2674,8473,21702,5099,
	21703,20304,2645,1257,17736,16883,21704,8477,
	2399,21705,3732,2146,21706,17737,16884,14603,
	2468,8475,2635,16885,8366,16886,5098,21707,
	16882,7680,2985,5129,21709,5103,2252,1994,
	5101,17738,21710,16887,5102,21711,2558,14604,
	17740,5100,17739,2173,8479,21712,8480,13377,
	5105,5104,5109,21713,14607,5110,14138,5106,
	8478,1408,3433,17745,14608,3539,1824,5111,
	8482,2646,21714,2469,21716,2960,17748,21717,
	19436,21715,17749,1727,5112,5116,1355,5113,
	5115,16891,5114,17750,2421,17751,2950,1161,
	5117,8483,21721,8484,17753,3643,17754,8485,
	19439,3691,17755,5125,5120,16892,5123,5122,
	8486,19440,5119,13397,3254,14611,5121,21726,
	19441,2422,5127,16893,3893,3374,21728,5128,
	16894,5131,21729,14612,1730,1258,1995,5133,
	5134,2427,2788,8487,8369,4333,2752,2751,
	2872,2103,17759,21731,1860,3863,3568,8488,
	5135,17760,14614,2147,3035,5136,4053,3692,
	21732,3015,5137,1592,21733,3760,3748,3722,
	2163,5141,14615,3711,19443,5143,5146,5145,
	1658,21735,5142,21736,5147,16895,14616,2623,
	14617,17765,3941,1165,2128,2844,2313,17766,
	19444,2523,3144,17767,5148,2827,5149,2522,
	3817,3922,8490,21737,1996,13681,3339,5154,
	3174,5091,5095,17771,1641,14618,3325,14619,
	5156,2470,3416,19445,16896,21738,5162,5159,
	5158,21739,19446,2675,19447,17773,16897,3739,
	21740,3995,16898,21741,3733,16899,1356,2220,
	4078,5161,2619,1923,14620,19449,14621,5167,
	5165,19450,1357,5163,5169,5175,8493,2856,
	21744,17777,3601,21745,5177,17778,21746,3476,
	5176,3366,3693,1527,17779,2715,2378,14623,
	3055,14624,3864,17780,21747,5172,5171,21748,
	5173,21749,5178,5174,5164,14625,2986,21750,
	3844,2290,2148,21751,17781,14622,7834,2090,
	16900,3741,1439,14066,3050,3234,1259,7687,
	2710,21752,2648,1792,17785,21753,16901,5180,
	17786,1997,1498,21754,19452,5182,1490,5188,
	14626,17787,5185,17789,2711,1449,2076,17790,
	1448,2111,17791,21756,5183,1851,1825,3175,
	8495,17792,5181,1162,17793,21757,17794,5184,
	5186,1733,1794,8492,1528,1637,17795,17796,
	21758,5189,14629,2153,3743,5202,14631,1331,
	5191,21762,14632,21763,17800,5208,5192,3978,
	17801,5199,14633,3349,19453,5207,7836,5194,
	17802,1144,5196,17803,1998,5198,17808,5197,
	5193,5204,21765,2471,16902,14634,4542,1948,
	3942,21766,5195,21767,3088,1409,2077,14635,
	5187,17811,5203,1471,5201,5205,17812,3176,
	17816,1594,17817,5235,8367,3797,5210,1593,
	17818,5217,8496,14636,21768,3694,21769,5218,
	5220,14637,5224,5212,17819,2920,3177,5228,
	5215,5219,14638,21772,19454,17822,14639,2559,
	5229,19455,21773,17823,5227,1529,17813,14642,
	21774,4086,5209,17824,5223,1180,21775,17825,
	5211,21776,16903,3782,5216,2536,3043,5206,
	21777,14643,5222,14644,21778,19456,2624,5232,
	1487,1877,5234,5213,5233,5226,5214,5248,
	21779,19457,3238,14645,5221,19458,5244,21780,
	17829,16904,3894,3560,5241,2858,5247,2753,
	19459,5238,5251,5250,3271,5249,3266,14648,
	16907,19460,21782,8498,17830,5253,5240,17831,
	1728,5243,2407,14649,3350,5245,1731,5237,
	5239,5236,17832,21785,4054,1464,5242,5252,
	1427,21786,1279,5269,17833,4055,5272,17835,
	2560,5271,5270,21787,16908,14652,5267,19461,
	14653,5255,19463,17836,5284,5254,14654,21788,
	17837,5265,17838,5260,5257,17839,5258,7475,
	21790,5263,1999,3044,2790,5261,16909,5256,
	17840,3895,3736,5264,5274,8501,17844,5282,
	5275,16912,13329,5288,19465,5281,17847,17849,
	16913,14143,2791,17850,5278,21792,14657,5276,
	21793,5289,3465,5295,17851,5290,5293,5287,
	14659,3498,17852,5277,17853,5285,2472,21794,
	3803,5305,5292,21795,14660,1878,1315,1469,
	14661,2473,5297,21796,5304,2341,1488,16915,
	2924,14662,17857,5294,14663,8504,5299,5303,
	17858,19466,1710,15412,19467,14666,1638,5301,
	3235,5300,5302,19468,21798,8503,19469,5296,
	8505,8506,1470,2951,21799,14668,5309,5307,
	1949,5306,14670,21802,5179,5308,17866,5323,
	17868,5319,5200,19470,5318,21805,5291,5316,
	5315,17872,5321,4044,17871,5324,1779,14674,
	17873,5320,5322,14675,3387,5329,5326,21812,
	17879,21813,19473,3933,5327,21816,5279,17881,
	5331,17886,1239,14677,5333,1853,2253,1741,
	14678,1316,21817,21818,16921,14679,21819,5335,
	5334,5337,1623,19474,1742,1530,19475,14680,
	5339,5341,17889,1358,21820,2933,21821,1531,
	5344,17890,5343,5346,14682,5347,2221,2649,
	2065,13386,3554,16922,3634,4074,21823,17891,
	21824,2243,21825,13785,2112,4026,17892,17893,
	2222,17894,14683,5353,5352,21826,3718,2408,
	2328,2194,5356,2537,21827,5355,21832,16923,
	21833,16924,17896,5361,19478,5363,5362,5364,
	1317,2952,2164,1450,5366,17898,14684,3132,
	4509,1597,5367,5368,14148,3644,3734,13388,
	3231,5369,3450,8509,16925,3471,17902,3807,
	5372,5371,17905,5374,19481,16927,5376,19482,
	17907,19483,17908,2223,14687,3773,21839,5381,
	5380,14688,2603,14689,17909,3499,1260,20309,
	21846,3417,8510,3089,2379,1659,19485,21847,
	17911,3418,21848,2266,19487,5383,14691,1306,
	17912,8511,3274,5382,2000,2961,17930,5392,
	5385,21849,8513,2849,1660,5393,16928,21850,
	19488,1854,14693,1599,5391,5389,5386,3914,
	17914,14694,8512,21851,3036,17915,3249,5387,
	5394,1785,17917,14695,1325,17918,16929,2091,
	5388,5390,14696,3717,2900,14153,3749,19489,
	17919,5403,21853,5404,21854,1359,19490,3576,
	3849,5406,2255,2474,5399,5405,1290,1711,
	17920,5396,5401,16931,2712,3367,3451,21855,
	21856,14697,21857,19491,5398,14698,3663,14699,
	5400,5409,8514,5407,17922,5402,14700,3664,
	3326,1661,21858,2987,17923,5410,17924,14703,
	5408,2873,5397,21859,1261,21860,17926,15413,
	14704,3896,5421,14705,21862,14706,5420,21863,
	5417,21864,3926,3214,5411,17927,3041,21865,
	1262,2001,5414,14707,17931,16932,2353,5419,
	17950,5418,5413,5416,16933,1478,17932,5415,
	3327,16934,2524,2713,14708,16935,5427,5425,
	17933,3519,14709,21866,17934,19492,5422,5424,
	14710,1244,21867,2002,4056,3540,8517,17935,
	1410,2561,5426,8516,2475,13354,17938,3866,
	17939,5428,17940,21868,5423,16936,5429,8518,
	21869,17941,4006,3181,3226,16937,17946,8519,
	17947,1428,17948,5435,21870,5438,19494,3979,
	21871,14712,3918,14713,5445,5439,5436,17951,
	14714,3996,5442,17952,8520,15414,2388,5444,
	5449,14715,3179,5447,13395,16938,16939,5441,
	17953,21872,2934,21873,5448,21874,5437,5443,
	14716,5450,1216,5440,5451,21875,19495,2562,
	8522,2409,21876,3573,17954,2078,8521,5432,
	5446,3124,8523,21878,2650,1479,2113,2476,
	5434,2380,5455,2423,1902,5470,17956,5464,
	1678,3145,14717,5459,5468,1139,1236,8526,
	5461,2828,5452,5454,2003,5471,8524,5466,
	21879,8527,15415,5465,16940,19496,5463,5467,
	17963,14718,21880,1924,17966,2477,2935,8525,
	21881,16941,5457,21882,17967,3865,21883,5453,
	3180,5456,19497,19499,4087,2282,3756,3394,
	21884,5484,1903,5474,21885,3959,2004,5487,
	21886,5473,13324,16942,5481,21889,17971,3897,
	5476,3112,21890,19501,5478,8528,21891,16943,
	5486,17972,5480,3795,14719,21892,5475,14720,
	2254,5499,21893,14721,21894,2892,2874,17974,
	5491,5502,5492,17975,14722,5493,5469,1683,
	3500,21898,2057,17977,17978,21899,4057,16944,
	5489,5501,1291,2792,21900,14723,21901,1533,
	4034,17980,19504,19505,14724,3757,3054,21903,
	14725,19506,5496,5498,14726,17973,17981,21902,
	17982,5500,5494,7776,17984,21905,1855,17985,
	5514,16945,14727,5509,2716,21906,14728,1473,
	17986,16946,14729,21907,5511,3016,5508,3061,
	5513,14730,16947,5507,5506,5512,21908,2629,
	5503,19507,8529,14731,16948,14732,5515,17988,
	5516,14733,21909,14734,17989,1534,13884,17991,
	16949,5521,5520,5522,17992,21910,19509,17993,
	21911,3133,8530,14738,19510,1849,2905,5517,
	3313,21912,5524,16951,5529,5527,14739,5533,
	14740,5526,2044,3294,21913,17997,3934,5528,
	5532,2901,16953,5531,8531,7076,15395,5536,
	17999,21914,5540,18000,5538,8532,7760,8533,
	5534,18001,5535,21915,5545,5542,5541,5544,
	3244,5546,19512,2995,2893,8534,21916,5547,
	14742,18004,18005,21919,18006,21920,19513,20306,
	18007,5490,14745,21921,5551,14748,18009,1360,
	14749,18011,3182,1411,16955,18012,2314,2114,
	14750,8535,18013,4045,2604,21922,14752,21923,
	18014,19514,5552,8536,5556,2936,5559,5554,
	19515,18017,5558,21925,14753,5557,3130,1181,
	8365,18020,5561,14756,5563,14757,5562,3665,
	21926,18022,8538,21927,18023,14758,18024,5564,
	21928,18025,5567,3586,14759,3777,21929,16960,
	18027,7644,14760,4035,14761,21932,5573,18028,
	16961,14762,18029,5569,5574,21934,1294,21935,
	3351,5568,5571,2480,16962,3429,5578,16963,
	16964,5570,3898,5579,18032,8285,21937,18034,
	2393,18035,21938,3300,19519,15397,5583,16965,
	3305,16966,3183,5586,14767,21941,5585,14768,
	5587,1295,14769,5582,18040,21942,4430,5589,
	2793,2182,5591,5590,2538,4334,19520,5592,
	21943,5594,5577,5593,8544,5595,14770,3376,
	18042,21947,5598,3066,14772,5601,5600,21949,
	5602,18044,2315,3541,14773,18045,3832,2776,
	2256,14774,18046,19523,21950,5608,3618,3419,
	5609,3341,3017,14775,18047,5610,1383,19526,
	1671,14778,3794,21952,3778,19527,1332,4058,
	18048,3712,19528,3578,21954,2651,19529,18052,
	3227,1879,5612,18053,2116,5614,5613,14779,
	18054,5616,21958,5619,21959,1880,14780,18056,
	3420,18057,8545,5621,13355,18058,1712,5622,
	5624,5623,14782,1761,21962,2754,3232,1713,
	18063,19533,5631,2922,5630,18064,14783,4059,
	3352,21964,5634,19535,18066,21966,5636,5633,
	3808,5635,5637,19536,3980,8547,5641,16971,
	14786,5640,2996,3299,18071,1881,5639,14787,
	21967,18072,5638,21968,18073,5642,1296,14788,
	2056,2224,21969,5646,19541,16973,19542,5648,
	21974,5647,18076,14789,5649,18080,1451,8549,
	5650,21975,5653,5652,14790,21976,18081,14791,
	1904,1732,18082,1318,21979,18083,14794,1762,
	21980,18084,18086,14798,15423,1565,21981,14799,
	4016,5655,18087,13802,5657,14802,8550,18088,
	5658,18089,21983,5654,8551,2183,18090,21984,
	3037,5656,14803,8554,21985,21986,5661,2330,
	18091,8553,15417,5659,21987,18092,14805,1812,
	3421,5660,14806,8556,21988,5666,18093,18095,
	21989,14807,1905,21990,21991,18096,1663,14808,
	5663,3943,8555,3960,14809,15418,2902,14812,
	5665,8557,14813,8559,8558,16975,14814,21993,
	8560,5667,3998,1743,3472,3328,14818,5668,
	5670,18098,5673,18099,14819,16976,14820,14823,
	18100,18097,5672,1950,1263,5674,18101,2614,
	5671,4004,16977,5677,7477,5675,5678,14826,
	16978,21996,5664,5676,14827,19543,2092,14828,
	18102,18103,8562,21997,14832,21998,18105,18106,
	14833,18107,21999,5682,8563,14835,18108,5662,
	14836,18109,16979,5683,14837,16980,18110,14838,
	22000,19544,1536,18111,18113,18112,14841,16981,
	5684,5685,22004,22005,5686,14842,22007,1245,
	14843,19545,5688,3501,5689,19546,1504,5690,
	22008,5691,22009,18116,5692,14160,3525,5697,
	5696,18118,19548,22011,5699,5698,5700,22012,
	5701,5703,5702,2059,22013,5705,5704,19549,
	16983,1537,5706,2652,18120,19550,13790,2184,
	18121,1307,5707,22014,3899,22015,18122,3635,
	5708,8434,3134,3869,2005,2563,22016,2953,
	4297,22017,3018,1384,5709,19551,18127,22018,
	5710,5715,22019,18128,5713,5712,5063,1412,
	5711,14844,1182,18129,3390,18130,3961,5716,
	5714,2970,2634,19552,22021,3391,22022,3487,
	5718,3956,1827,5719,5717,3434,5720,15419,
	5721,8565,1183,18132,18131,2526,5726,14161,
	5723,3269,5722,14845,18133,14846,5729,5728,
	5727,3479,18136,2756,2755,14847,1625,5730,
	22026,19553,5731,22027,5732,18138,5734,19554,
	5733,18140,5742,3452,5736,14850,5738,18141,
	5740,2564,22028,5741,5739,2284,19555,5735,
	5737,19556,3508,18144,2481,19557,5743,16985,
	18145,5745,18146,2258,2079,19558,3185,5746,
	3047,19559,18147,5748,19560,3945,5747,14851,
	18151,19561,5753,18152,14852,5754,14853,5752,
	16986,18153,19562,18155,16987,5759,14854,5757,
	19563,5758,22029,18159,18154,14857,19564,14858,
	5760,22030,5765,18163,13893,5761,18164,22031,
	19565,18166,5766,14860,5768,18165,22034,18169,
	3981,14861,22035,19567,5771,18171,19568,14862,
	1566,18172,3610,5773,22036,16988,14863,5780,
	14864,18174,14865,5781,14866,5782,3395,3146,
	5785,3368,3494,5786,16989,8566,5787,3108,
	22037,1413,2006,5788,18175,5791,8569,8567,
	16990,8568,22040,18177,18176,19570,3453,18180,
	5795,16992,7452,22043,5800,3340,14870,3725,
	1264,18182,1273,22044,14871,5801,16993,5802,
	3178,2653,5336,5804,3789,22046,5805,1538,
	3435,5806,16995,5807,18184,5808,19573,14872,
	19574,16996,3809,2796,19575,5810,14873,19576,
	2482,22048,5812,5811,3473,16997,1539,1885,
	16998,18187,5819,22052,5816,2565,3774,19579,
	5814,18188,22053,19580,14874,18189,14875,19581,
	3019,1567,22054,19582,19583,8572,5822,5823,
	5826,14876,2605,7877,3228,19587,5827,3713,
	19588,5824,14165,5825,19589,5830,17001,18196,
	14880,5829,5828,17002,17003,14881,19593,5835,
	18200,5834,18202,3613,14166,14882,2400,3982,
	22064,19595,5836,22065,22066,5837,19596,19597,
	5839,19599,22068,19600,5846,3779,5847,3836,
	5848,18205,2956,14883,3360,1763,18206,18207,
	2937,5849,1714,14884,19601,2676,14885,18208,
	5850,18210,14886,5851,18212,22071,22072,5852,
	1882,2117,5854,8574,14887,22074,3152,2118,
	1640,3329,14891,17006,22075,5856,18215,22076,
	8575,13342,15420,22078,19603,2483,22079,22082,
	8576,18216,3962,2007,17007,1883,5859,3090,
	5862,1429,5857,22083,1234,18219,2140,4088,
	19605,18221,14893,18222,5863,18220,3611,22084,
	2685,5865,22085,17010,5866,17008,19607,22086,
	19608,1452,18223,5872,22087,5871,22088,5873,
	5868,22089,14894,19609,5874,5869,14895,3436,
	5867,22090,19610,5870,18224,22091,5877,22092,
	18225,5876,3727,14900,22095,5879,22096,14901,
	5878,18228,22097,17011,2484,22098,5880,18230,
	5882,5881,5883,22099,18231,17012,22100,19612,
	22101,18232,5853,22102,5855,5885,5884,22103,
	17013,8578,14903,17014,22104,2260,14905,4017,
	14906,2302,19615,5886,1805,19616,22105,14907,
	17015,1626,1601,2225,18233,22106,22107,3870,
	22108,5892,19617,5891,2758,5888,18234,2389,
	2566,5889,5887,18236,3502,22109,17016,5893,
	22110,19619,22111,5894,5916,1744,4067,2743,
	5895,14910,22112,1362,3091,3569,22114,8582,
	18238,14911,8584,22115,5896,1684,5897,22116,
	18240,5899,19620,5900,22117,3295,7758,5901,
	18241,1745,1363,3229,2354,2226,18242,18243,
	17017,5904,14915,2355,1354,3509,17019,5905,
	18246,22119,17020,2759,22120,5908,22121,5909,
	3359,22122,2567,5906,2975,19622,17021,5907,
	17022,2486,22123,1184,22125,19623,14917,22126,
	5910,14918,18251,3092,19624,5911,2667,3769,
	18252,2963,18253,3983,14920,5915,5914,22127,
	14921,2331,22130,19625,5917,1204,5920,18255,
	14922,19626,5918,1364,1828,5919,2008,2052,
	3638,5921,19627,14923,3714,18256,22133,5923,
	14924,2677,1265,1338,1136,22134,18257,5922,
	18259,14925,18260,20310,5926,18262,1453,1856,
	18263,1664,14928,5929,1773,5930,22136,2720,
	19628,3237,18264,2692,2149,17023,19629,18265,
	22137,18266,22138,2976,2797,19630,5933,5935,
	5932,5934,22139,1784,14929,5936,1788,1665,
	3900,5938,19631,5939,18270,1232,19632,18271,
	5942,1492,5941,5940,14934,5944,5937,22140,
	5945,3953,18273,5946,17025,5947,8585,4549,
	5949,22141,3965,5952,7176,2487,3216,5955,
	8586,18274,22142,2918,8588,22143,5956,2938,
	5957,22144,4214,2971,2271,22145,17026,19633,
	1540,5961,14935,1666,5960,5973,2488,17027,
	5965,22146,1468,2592,3542,14937,5967,22148,
	18280,19634,2888,17028,22149,14938,5962,22150,
	5966,5968,5975,3488,14939,3386,3187,1746,
	5974,5972,14940,3401,5969,2972,3189,14173,
	3188,18288,2150,18290,5977,18291,5979,14942,
	22153,5982,14943,5983,17030,14944,18292,5978,
	5976,18289,5970,3615,1365,5987,22156,5992,
	18295,14945,5991,3369,3763,22157,2185,5985,
	5994,5989,18296,5988,5984,8589,5986,19636,
	1541,19637,2721,14946,19638,3382,22159,5999,
	18298,22160,18299,22163,2693,5996,22164,3427,
	18300,6000,3619,22165,2969,18301,5995,5997,
	18305,14947,14948,18306,22166,3230,6006,6005,
	19639,6002,18307,19640,6011,6015,6012,19641,
	6008,17031,22171,18309,22172,22173,6009,14949,
	22174,6014,18310,14950,6003,4330,6010,6004,
	7739,6019,14951,1542,22177,14952,6017,19645,
	6018,6021,22178,17032,22179,14954,6020,3466,
	6022,4036,3645,6027,19647,6024,18311,22182,
	6023,2678,6026,6028,14955,6025,22183,6032,
	18313,6029,14956,22184,18314,6030,6007,18315,
	22185,6031,6033,6034,17034,14958,18317,14959,
	19653,3606,14094,22187,19654,3822,1734,1791,
	6036,22188,19655,17036,6037,17037,3370,2760,
	3306,19656,1156,17038,6042,6040,6038,1501,
	18321,2489,6043,6039,6047,22189,6046,6045,
	6044,6048,19657,22190,14960,6049,2654,14961,
	6050,18326,6051,18328,2746,22191,6054,19659,
	22192,17039,3190,14964,6053,14965,19660,6055,
	14966,3589,2798,2009,18331,3984,6060,18333,
	6061,6062,1829,6064,18334,3839,2010,6063,
	6067,3826,22194,3493,22195,18336,14971,2413,
	6066,22196,2303,2011,2228,1667,3590,6065,
	14972,18337,2761,3696,2151,14973,22198,2229,
	3065,22199,6070,4007,2121,17040,6071,2568,
	2490,2080,6072,18342,14974,2356,1906,2762,
	6068,6074,18343,8591,6069,1830,19662,6077,
	6081,1857,22200,13882,6076,22201,14976,18347,
	6085,17041,14977,2012,3927,1152,6082,22202,
	1669,22203,6079,18349,3191,6078,6075,1414,
	2696,22204,1884,14978,22205,22206,19663,17042,
	19664,14979,18353,6084,22207,18354,1831,2835,
	6088,2800,18356,19667,6101,8594,6097,6091,
	17043,18357,18358,6095,2342,1185,6090,6098,
	6102,2013,3810,19668,3058,6092,18359,8593,
	6100,19669,6089,2940,6094,1153,3798,18360,
	1747,3456,14982,2799,19670,3992,2425,6141,
	8595,18361,6103,18362,2722,6099,6104,6106,
	3093,6109,22215,13322,6105,19671,18363,3620,
	1543,22216,14983,3799,22217,14984,6108,14985,
	22218,6107,22214,1297,3268,6110,14986,6117,
	6111,6118,18367,14987,6114,3377,22219,18369,
	2294,6116,14989,6113,6119,6112,2382,18370,
	17045,3667,18371,22220,2390,19674,6115,6127,
	22221,6123,19675,6126,6124,6128,6122,6096,
	2679,22222,6125,19676,18372,6120,14990,14183,
	2723,1832,2357,22223,2539,2744,6131,6130,
	6132,22226,7697,14991,22227,6121,6129,6137,
	6136,7671,3752,14992,19677,1793,19678,6135,
	6139,6138,6142,18374,6144,19679,2186,6140,
	6147,14994,6153,3125,6149,14995,6148,22228,
	6152,6157,18381,18380,19680,18382,22231,18383,
	14997,6158,6165,6164,19681,22232,19682,19683,
	22233,6171,2129,1833,18389,2964,3399,19684,
	2426,3457,6172,4918,6173,6175,3919,6174,
	6177,6176,3901,19687,6178,22235,3474,22236,
	18392,15002,18395,17046,18396,6181,6180,8598,
	19688,6183,1800,2724,1627,22238,18397,6187,
	6184,6189,6186,22240,1227,15003,22241,1319,
	15004,18400,6195,6194,15005,2607,6196,15006,
	6197,22243,6198,1569,15007,18402,6199,6200,
	15010,3723,3917,3902,4061,14099,2015,6203,
	2304,6202,19692,18404,18403,6204,2261,19693,
	15011,22244,2865,15012,6205,22245,15013,2014,
	18405,3811,6208,22246,19694,2262,22250,17048,
	3833,17049,19695,15016,6212,15017,2941,22251,
	6211,22252,18408,22253,3593,6218,15018,2801,
	6219,4038,6223,15019,6222,6221,3020,15020,
	6224,2540,6226,4062,6227,6228,6230,6229,
	3385,19696,3281,4068,3389,3484,18411,6233,
	6231,18412,1546,15021,1928,2230,18413,3458,
	15022,18414,1886,3697,15023,6236,6234,2017,
	1197,18415,22257,3343,1187,6241,18417,2942,
	3342,22259,2875,19698,19699,18418,6243,6239,
	6242,6237,6240,3668,17051,1929,1217,6238,
	18419,19700,22261,6245,19701,6246,22262,17052,
	3217,18421,22263,19702,1715,22264,19703,18422,
	6257,3315,19704,2231,19705,1716,2668,4076,
	3770,6244,2680,22265,15026,19706,15027,1645,
	6247,22266,6249,18427,13913,18429,6250,19707,
	2916,3021,15028,22268,18431,6254,22269,6253,
	22270,15029,6251,15030,3543,6256,22271,6255,
	2018,4089,6272,17053,22272,17056,6259,19709,
	2058,6258,19710,6262,22273,3022,3570,2725,
	2876,6266,15031,6267,6263,15032,18436,22274,
	6265,19711,22275,2019,19712,6269,22276,3544,
	19713,3738,3482,19714,22277,6271,6270,3698,
	6274,18439,2745,15033,6276,6282,17057,6279,
	6277,3317,6280,18440,6281,18441,22279,1330,
	6288,6283,17058,19715,6284,22280,15034,19716,
	18442,17059,6287,6286,6289,18443,2569,18445,
	1385,6292,3999,2263,22281,18447,2359,2232,
	2965,22285,19721,22286,15035,1235,18448,6295,
	18449,19722,15036,18451,6298,6299,1717,5039,
	6300,19723,2697,6301,2295,22288,6302,6303,
	3630,1560,18453,19726,2726,2402,22289,3555,
	2360,15037,18454,22290,6304,2020,6305,3424,
	15038,6307,15039,2857,3371,1907,6306,2727,
	19727,22291,19728,18458,15040,3094,22292,15042,
	22294,18462,6310,6309,6312,18464,6311,6313,
	17061,6315,18465,6314,1547,22295,6316,19729,
	2081,3985,6320,2541,18468,6323,18469,18470,
	19730,19731,18471,1206,6324,15043,19732,6325,
	18472,22300,22302,3545,6327,22305,17064,1415,
	1142,17067,19733,6326,6329,3332,15044,2570,
	19734,1366,18475,3669,18476,15045,1843,1748,
	18477,6328,18478,1386,15046,17068,1503,15047,
	19735,1299,6333,4018,2877,17069,6345,22306,
	1367,6343,6341,6332,22307,6330,18479,6331,
	22308,2319,1764,2997,18480,19736,3241,6335,
	6334,19738,18481,6340,6336,6339,17070,15048,
	17071,8600,3804,6338,1368,1499,6342,18486,
	1135,18487,22314,17072,6365,6356,22315,17073,
	22316,6355,18488,6350,6349,22317,6347,6346,
	2977,22318,2907,6352,17074,18489,22319,22320,
	15049,18490,6354,7672,17075,2802,1835,1251,
	6353,15050,2021,22332,18485,15051,19739,18491,
	2803,8601,17775,1369,19740,1326,6369,17076,
	8602,22324,22326,6359,15052,6367,6374,6368,
	15053,6366,15054,6337,15055,22327,6360,18497,
	22328,18498,1548,6362,6373,22329,17077,6375,
	15056,6361,3378,22330,22335,6392,22331,18499,
	1632,6379,17078,18500,17079,1371,18501,2122,
	18502,3147,6387,17080,22333,3646,15059,6378,
	1370,1930,3483,6388,8604,6385,19741,18504,
	6395,19742,6380,6381,3218,17082,6386,7807,
	6383,3670,6389,1188,15060,3361,6397,6393,
	18508,6398,3928,22337,6407,22338,8605,3903,
	15062,22339,3954,18509,22340,2998,19743,1481,
	18514,17083,22342,3556,6413,3193,22343,18516,
	1141,22344,6406,22345,6402,2804,6396,6404,
	6409,17084,3298,17085,22346,1134,15063,6401,
	18517,6410,3562,18518,22347,18519,6405,6400,
	17086,22348,19744,2264,22349,17087,6416,19745,
	1493,2528,6414,17088,6419,2805,19746,18523,
	6415,15066,6422,15067,3904,6412,1430,22354,
	6418,6421,3768,6417,6420,4039,18531,15068,
	18532,22355,18533,19747,6435,15069,6427,2287,
	6394,22359,22361,3614,3758,6434,6433,1240,
	18534,22362,17091,6432,6426,22363,7706,3057,
	22364,22365,18537,6431,1218,17090,22366,15071,
	2818,22367,3603,7861,22368,6436,6440,6439,
	2494,2293,6442,1718,18538,22369,18541,22370,
	8607,6443,19749,3563,6438,8608,6399,22371,
	19750,15072,6437,17093,4085,3194,3557,8609,
	22375,17094,18545,18546,6457,18547,6444,18548,
	3372,19751,18549,6454,6446,22376,6448,22377,
	17095,15073,6458,6447,18550,6451,1300,19752,
	3261,6452,6455,18551,18552,22378,6445,22379,
	2728,22380,6449,2165,2571,1798,3840,15076,
	3845,2428,8611,22382,15074,22383,6462,6460,
	4084,22387,6459,18558,17097,3935,15075,6461,
	6463,6467,6465,18561,22389,18562,3195,6466,
	18563,22390,3425,6453,22391,14208,18564,2429,
	6468,6473,2806,22392,6472,17099,18565,6474,
	2763,6469,6471,17102,18566,15077,6470,5328,
	5313,17104,22395,6476,22396,18568,6475,5809,
	6477,17105,18572,22398,18573,22399,6478,18575,
	16824,19753,6479,1931,15079,18576,6481,4244,
	18577,17106,1679,13336,3970,22402,1771,6482,
	17107,15080,22405,2988,15081,17109,22407,18578,
	3282,15083,1150,18579,22425,18580,1379,2187,
	22408,6486,3320,6492,6495,18583,6491,22415,
	17110,22416,22417,19755,18585,18586,6493,22418,
	6494,2308,1836,1440,18587,22419,6499,18588,
	1437,22420,6504,22421,6500,6503,22422,18590,
	3406,22423,18591,22424,6502,3514,3438,6505,
	2909,6515,15089,6511,17111,15090,1387,3672,
	6510,22426,18593,6507,18594,6508,6513,6516,
	15091,6514,18595,22427,18596,6512,6506,15092,
	19756,2966,15093,6523,15094,18597,18598,19757,
	18599,15095,6522,19758,22428,18600,22429,19759,
	18601,6520,15096,6518,18602,4063,6527,22433,
	6528,6530,18605,6533,6525,6524,22434,6532,
	22435,15097,18608,19761,18609,17112,15098,18613,
	18614,19762,3877,19763,19766,6539,18616,6538,
	19767,19768,6541,3920,6552,6543,18619,19769,
	6544,18621,6551,6550,22444,19770,22445,6542,
	22446,15103,22447,19771,18623,6555,22449,18625,
	22450,18628,15104,7715,15105,22451,6554,22452,
	22453,1416,22454,1628,6558,22455,6537,15108,
	8614,22456,17114,15109,6557,6561,6556,17115,
	6563,22458,18629,15110,6565,6564,19772,22457,
	22459,19773,15111,22460,7813,6567,6566,6497,
	6571,18630,19774,18631,22461,6568,18632,22462,
	6569,6570,19775,6572,18633,1858,22463,6574,
	22464,6573,19776,2362,18635,19777,2022,5412,
	19778,22468,6575,2395,19779,15112,1431,18636,
	6576,19780,1268,22469,2495,6577,15113,22470,
	2023,6578,1189,15114,3503,18639,19781,18641,
	2608,22471,6586,6584,6581,1750,22474,6580,
	6587,2837,6588,15115,6591,6582,22475,18642,
	15116,3459,6595,6597,1927,6585,6596,18644,
	2123,4030,2807,14217,19785,17116,3946,18646,
	15120,18647,6600,3871,15121,3636,6603,22480,
	2096,19786,3947,22481,6608,6609,2496,6607,
	8615,22482,3921,6604,6606,2657,2628,18649,
	6605,15124,3571,6611,22483,3673,6613,8360,
	6625,6615,6616,19790,15128,22487,15129,22488,
	17118,19791,6622,18651,6623,17119,22489,6621,
	17120,6619,18653,6614,6620,18654,22490,6624,
	7120,1320,22493,19792,22494,19793,6627,1751,
	6626,22495,18657,22496,6629,6631,19794,6630,
	22497,6632,19795,2363,22498,6633,22499,22500,
	15131,19796,22501,6635,2658,13870,3905,3572,
	3324,6637,18658,6638,1887,1606,18659,6639,
	17121,22503,2233,3319,6640,18660,22506,6641,
	19797,6643,3937,6642,2572,6645,18661,6646,
	1549,22507,19798,22508,18662,19799,6648,15132,
	19800,15133,22509,18663,15134,2542,6654,18666,
	6655,6656,22512,18668,1908,13756,3095,6657,
	15136,1837,3196,15137,6659,18670,8616,1799,
	18671,15138,6658,2903,1607,6662,22515,2497,
	22516,18673,15139,1859,22517,6663,22518,3841,
	2764,18674,6664,8617,2573,2990,2498,22522,
	18675,22523,6665,18676,6668,17123,6669,19801,
	18677,15140,2094,2850,6667,22525,2499,3504,
	22526,19802,18678,6666,22527,17124,2234,1269,
	17125,6673,1838,22528,2236,19803,4083,6672,
	6671,2729,1639,4073,1432,2500,19804,15141,
	22533,6675,22534,6676,6674,19805,1932,22535,
	2237,3293,22536,18680,6679,22537,2660,2944,
	19806,22538,3872,22539,6682,1952,22540,2659,
	6678,6683,1953,8619,6677,22541,13880,2694,
	3233,18679,22542,2925,1373,22543,19807,18681,
	22544,1629,8620,3024,6686,22545,6684,22546,
	2954,22547,19808,22548,2661,1550,6685,2593,
	17127,19809,3986,4070,18682,6687,6698,3025,
	19810,6697,8621,6694,6702,6695,3096,6690,
	6688,19811,3851,2238,22552,6692,6689,15142,
	19812,8623,6696,2430,2906,22553,3699,1276,
	1190,3197,18686,19813,6700,18685,22554,6699,
	18687,3262,19814,6704,19815,6703,6705,1888,
	6701,2024,18688,2305,22555,15143,6706,3906,
	6710,3495,18689,6713,22558,19816,17128,22559,
	6717,6714,22560,6716,6715,18691,19817,8624,
	15147,2269,17129,6720,6719,3546,18692,6722,
	6721,6723,19818,6724,1630,6190,2529,18693,
	1954,19819,22562,19820,6726,19821,8625,6727,
	18694,2188,22563,19822,5075,18695,6728,22564,
	6729,22565,2364,15148,17131,15149,19823,2921,
	22566,18698,6735,18700,22567,6737,18703,3198,
	15150,6738,17132,3675,19824,6739,6740,18704,
	6741,22568,19825,15151,6742,18705,22569,22572,
	3250,22570,2501,6743,17133,18706,2045,4103,
	6744,18707,6745,3505,6746,19826,6754,6748,
	6750,22581,3700,18710,18709,6753,6755,18711,
	18713,22584,1419,3075,13644,3547,2130,2025,
	3521,1375,3426,6758,1551,2681,6757,6762,
	2999,3823,6763,3353,2879,22585,19827,3461,
	3127,6759,3701,1388,6765,4046,3038,4075,
	19828,22586,2239,6764,19829,2833,22587,6781,
	2730,22588,3280,19830,3522,18715,18714,15154,
	6768,2189,2240,22589,2502,3355,15155,1889,
	6767,6766,19831,3148,22591,8626,18717,22592,
	8627,19832,19833,15156,2026,6769,15157,22593,
	6775,2819,18719,6774,1570,6777,18720,6776,
	6778,8628,6780,6782,6783,19836,15158,19837,
	2682,22594,2299,6784,1456,15160,6785,19838,
	2808,6786,6787,3549,8629,1609,6788,22596,
	15161,3026,22598,19839,22599,19840,22600,19841,
	22601,19842,2829,22604,6792,22605,22613,22607,
	15165,22608,18721,22609,6798,6796,15166,6793,
	22610,19843,6795,17136,18722,6794,6797,6802,
	15167,22611,19844,1933,4047,22612,3027,19847,
	2731,6803,18725,3907,19849,18723,3199,6809,
	22616,6810,15169,19850,18732,6813,19851,22617,
	6814,6812,18735,6816,6820,6817,6815,15170,
	6818,19854,22620,19855,6819,18737,6826,22621,
	19856,2684,6823,19857,18738,6825,6822,15171,
	18739,18740,15172,19859,6828,2365,19860,6827,
	19861,22624,15173,17139,6829,6833,6832,19862,
	6835,6834,19863,6837,15174,6836,6838,19864,
	18741,6839,19865,22627,22628,18742,6841,22629,
	19867,6840,2574,6842,19868,15175,1765,6843,
	6845,18743,14083,22630,18744,7663,22631,6847,
	6844,22634,6848,2306,6849,1610,1801,15177,
	1890,6850,22635,3128,6851,22639,22636,19870,
	22637,22638,2272,15179,17142,6853,6852,1840,
	6855,22640,1457,19871,18749,2124,6856,6859,
	6861,3637,6858,22643,15180,19872,6860,6864,
	6862,1611,17143,6863,19873,6866,19874,22647,
	3344,4000,18750,19875,17144,22648,2366,6867,
	22649,19876,15181,22650,3852,6869,19877,6868,
	6872,3883,18751,22651,6871,19878,1483,6870,
	6875,19879,6874,18752,6873,3116,6876,22652,
	17145,19881,6877,18753,2575,6882,22658,2265,
	6883,6884,18755,19885,15182,4278,4277,6885,
	6143,6886,2914,2545,3318,6887,22659,3621,
	3056,2064,22660,2919,22661,18758,1228,3750,
	2589,18759,18761,22662,18762,1844,22663,1753,
	22664,18763,3622,18764,6890,22665,15187,6889,
	1376,17147,3278,6891,3373,3117,17148,6892,
	2396,18765,6894,18766,15188,3790,6909,3045,
	22666,2880,2809,15189,3200,19886,6895,1647,
	6904,18768,6901,6914,3201,2974,6898,3098,
	3149,6899,6903,2598,22667,3357,3048,19887,
	2662,6902,2830,2820,6900,1133,4040,15190,
	17150,2881,19888,2367,2576,6907,6906,18769,
	1203,6908,6916,22673,3251,2609,18771,15191,
	2967,6915,3873,1249,19889,3623,1377,3219,
	2913,1191,6917,6918,7476,19890,2845,18772,
	6919,1301,2766,22674,1891,3908,22675,8634,
	6920,3110,22676,2810,2307,6921,19891,19892,
	2415,6922,2733,2732,19893,1192,3987,6926,
	3462,6928,6927,6925,5943,1552,18775,6893,
	15192,6930,6929,19894,18776,22677,6931,18777,
	3874,17151,19895,15194,19896,18778,15196,3257,
	22678,3676,6933,18780,6934,19897,17152,15197,
	6935,18781,19898,17153,17156,18783,19899,2027,
	6939,18785,15199,8635,18786,1802,6936,6937,
	22683,17157,19900,15200,3875,18788,1719,18794,
	19901,18795,17158,8637,17159,15201,18796,6942,
	22688,3100,22691,17162,6944,15203,6943,22692,
	18798,18799,15204,22693,15205,15206,3243,6945,
	2368,2316,3345,2991,22695,2334,22696,2610,
	6946,18802,6947,18803,19903,15207,15208,2595,
	6948,15209,6949,6950,3929,2369,22697,6952,
	6951,18804,2028,2053,2190,18805,19905,22698,
	22699,6955,17164,18806,22700,2416,6954,19906,
	6953,2889,15210,1955,19907,2663,3396,22703,
	22704,6956,6959,6957,7707,6958,19909,7777,
	15215,18807,6961,6960,18808,6962,19910,15216,
	19911,19913,3428,2115,2317,6965,6966,3948,
	2383,3834,3988,6967,1754,6968,8640,3101,
	22709,8639,6971,1494,2577,8641,6969,3715,
	1780,22712,15217,17165,15219,6972,22714,6976,
	19914,18814,8646,15220,19915,8648,22717,3255,
	1441,8647,6980,6977,6979,15221,7050,15222,
	4019,1934,15223,8655,8649,22731,6988,8650,
	18818,15224,3118,6983,22732,15225,6986,6984,
	18819,6989,22733,8652,15226,6990,8654,18820,
	6982,22734,8653,6995,1302,15227,18821,3393,
	6985,18822,2504,8656,18823,22737,22741,19917,
	3702,18827,1756,8362,18828,22744,18829,19918,
	2735,22745,6993,6987,6992,18830,3791,18831,
	3028,6994,6991,18832,22746,8657,22749,18836,
	13652,22750,6998,22751,6997,22752,19922,15228,
	19923,19924,18839,22754,19925,22755,18840,6996,
	8663,3677,8667,8661,22756,8662,15231,8666,
	22759,22760,2437,18841,8660,22761,6999,3631,
	22762,1270,22763,3512,2992,18845,1682,8368,
	7001,8670,2031,8673,22765,7000,2611,18842,
	19926,15233,2612,7004,7006,22767,8674,8672,
	15234,2531,8669,7005,7007,22768,8668,1739,
	15235,3511,18846,22769,2318,4041,7003,2152,
	4069,19927,22770,7009,19928,22771,7008,7010,
	15236,8361,17166,13400,3265,22772,18849,15237,
	3059,22773,7015,8677,19929,18850,2945,7011,
	22776,15239,18851,1795,15240,7014,22777,8676,
	18852,18853,22778,1892,22779,7013,22781,2505,
	19930,19932,18858,1495,22784,7019,22786,2095,
	2811,22787,3046,15241,17169,8678,22789,1433,
	22790,15243,7020,15242,19933,8679,18865,7029,
	22797,15245,22798,19934,22799,7028,3111,7025,
	22801,18866,7027,8680,17170,1720,15246,7030,
	7024,19935,18867,7023,22802,8681,19936,15247,
	18871,22803,19937,7034,17171,7035,22804,18868,
	15249,7036,7033,7032,22805,2506,3202,7031,
	19940,22807,8682,18872,17172,15250,7039,7037,
	7041,18873,22809,7044,8683,1553,7043,3846,
	22810,7055,7045,22811,7048,22812,7046,22813,
	7047,17174,7049,15253,18876,22815,18877,7051,
	17175,15254,7056,7054,7057,7059,22817,22816,
	3029,15255,18878,7060,2736,22819,18879,7061,
	19942,3604,7062,1417,17176,1246,1555,8685,
	1554,7063,17177,7067,19943,1556,1459,2032,
	3402,15256,7069,7068,15257,20313,17178,7070,
	13653,1278,7073,22822,7072,7071,15260,7074,
	19944,19945,1163,18884,19946,7075,17180,7079,
	7078,17182,7081,7080,7082,3206,22823,15261,
	7083,22824,17184,22825,19949,7084,7086,19950,
	7085,19951,3550,15262,22826,7087,22827,19953,
	3703,22830,19954,2765,18891,19955,1128,2859,
	15263,7091,7094,7092,2033,7093,1910,3605,
	7096,7098,7097,7099,18892,1219,2590,2438,
	1557,7100,7101,3040,19957,3989,3203,7095,
	3950,1893,19958,22837,3909,22838,19959,19960,
	18894,19961,15265,1777,3964,1790,19962,2882,
	6252,7103,1418,2613,1460,7105,19963,7106,
	7104,1850,18896,2125,2507,8688,22840,1221,
	4001,15266,17185,22841,7108,6923,15267,7107,
	19964,8689,7111,18899,7112,7113,4020,18900,
	2669,3407,18901,2627,1571,3876,1389,2370,
	1935,7118,22842,7117,2241,7119,7116,2166,
	15268,7123,6546,18904,4331,18905,2621,7121,
	18906,15270,22844,3949,3273,1229,15271,2695,
	2274,17186,3591,15272,1250,4021,3925,7124,
	22845,3135,15273,19965,7125,22846,7126,2578,
	7127,15274,4022,7122,7129,7131,7128,18908,
	7133,22847,19966,2812,22848,1378,18909,7134,
	3780,19969,7136,19970,4048,8690,5140,7137,
	18912,7143,8692,7140,7141,18913,7142,7144,
	22850,8693,17187,8694,8695,2664,20314,3843,
	18914,22851,2665,15275,18915,7145,7146,7430,
	3800,15276,7147,7148,19971,7149,1461,18916,
	7151,2591,15277,15278,1786,7153,7157,22854,
	7155,19973,19974,22855,7156,1489,7154,7158,
	17188,18919,22857,2508,19975,15279,18920,19976,
	1633,15280,7165,6708,19977,3628,17189,22858,
	19978,17190,18923,22860,7169,22863,7170,17192,
	22864,7171,18924,19979,22865,22866,18925,1558,
	18926,22867,18927,15283,17193,7172,22868,15284,
	19980,18928,7173,3289,19981,18929,7175,1339,
	15285,7178,7177,22872,1222,3607,3030,2066,
	18930,2034,2417,19982,2594,7180,18931,15287,
	7179,3884,1572,3430,3252,18932,18933,2626,
	3990,22873,1841,22875,7183,15289,7182,18938,
	19983,15290,3705,3204,19984,22876,7795,1266,
	22877,7184,7181,14259,3523,3924,7185,15291,
	7186,18939,22879,7188,2890,1573,1894,17194,
	8697,1574,15294,7752,3129,8698,18940,1936,
	22881,7189,15295,7194,7193,7195,18941,18942,
	22882,7196,18943,7197,7198,18945,15296,18948,
	7200,7199,19988,7201,18949,15299,18950,22883,
	3464,6201,2543,13848,18951,1612,18952,15300,
	22884,17195,7202,18953,7203,4289,5338,3431,
	1215,18956,1151,2242,3678,2534,19989,7204,
	22886,18957,7205,3910,2191,19990,7206,1390,
	7207,19991,22887,7213,7209,22888,22889,8701,
	1559,22890,7214,22891,19992,15303,18971,7218,
	7220,18972,22893,19993,7219,7222,7225,15304,
	18973,19994,7226,1722,17196,22894,22895,15305,
	19995,15306,18974,22897,8703,22898,7229,18976,
	22899,15307,22900,2968,3267,22901,7232,18977,
	18979,2860,1274,1766,1767,15310,2993,7237,
	1768,22902,18980,1391,7236,18981,22903,7234,
	22904,7235,18982,7233,19996,7247,7243,18985,
	7242,18986,7244,15311,17197,22913,7246,18987,
	22914,19997,1613,7245,19998,18988,2813,1895,
	7248,18989,22916,19999,15313,15314,2861,3205,
	18991,22918,15317,7250,15318,7253,20000,7252,
	7254,18992,7251,15319,22921,18993,7257,8704,
	20001,7727,22922,7256,15320,7259,18994,1723,
	7258,7263,7262,7264,7266,7265,22925,7267,
	22926,18996,7268,18998,1434,19000,22929,7269,
	22930,7270,17198,17199,2615,17200,7271,19003,
	7272,19004,19005,17201,22932,2036,8705,22933,
	8706,22934,19007,7279,22935,19008,3397,7281,
	22936,3480,7282,7280,7283,22939,15323,22940,
	7286,20004,17202,15324,17203,20005,20006,19009,
	7288,20009,7289,22946,19013,20297,7290,15325,
	7293,22947,19014,20298,17205,5332,7300,6057,
	1614,19015,1407,2082,7302,7301,3761,19017,
	22952,3728,20012,1685,22955,19019,17207,8708,
	15328,7308,8707,22960,15329,22961,20014,7310,
	17208,22962,20015,22963,22964,17209,22965,22966,
	1154,8709,20016,7311,3579,7309,22967,20017,
	17210,19024,15330,7316,22969,3740,2171,19025,
	2154,2737,20018,8710,19027,22970,19028,7320,
	8711,22971,19029,20019,7318,15331,7321,15332,
	20020,1957,7319,17211,7322,22972,2168,15333,
	19031,7329,7326,7325,7327,17212,15334,1845,
	19033,15335,19035,19034,7333,7332,7331,19030,
	15336,17213,22973,15337,20021,8712,7340,7336,
	7342,15339,7338,1472,22974,4082,22975,7341,
	7337,7335,7334,19039,7345,20023,19043,22976,
	7348,19044,17214,7347,7346,22977,15340,22978,
	19045,15341,3515,7344,1207,7349,19047,7351,
	20024,15342,19048,1485,7330,1241,22979,7353,
	22980,20025,19049,7352,7350,2923,3742,15344,
	19055,20026,4002,19056,7354,19057,19050,19058,
	22982,19059,7355,20027,7356,20028,19066,7357,
	15347,7358,19067,22983,3031,19068,7359,7364,
	7360,19069,3679,3792,3240,22984,15348,7365,
	7366,3222,7363,7362,19072,7374,1303,22986,
	7367,2270,1321,22987,20029,22990,2037,22991,
	20030,7377,7376,7375,15349,17220,7372,22992,
	7378,15350,7379,7384,7383,17221,19081,2054,
	3781,19082,15351,7382,22995,15352,20031,8714,
	3680,7390,8713,7386,19085,7391,20032,15353,
	22996,19088,22997,7388,7387,17222,15355,22998,
	1842,23001,17223,19089,7392,7394,23004,7389,
	17225,23005,7369,15356,7395,3069,19092,7399,
	19093,7400,7398,17226,19094,19091,17224,7397,
	7403,15357,7396,7401,15358,19095,23007,15359,
	7646,7404,19096,23010,15360,19097,20033,7407,
	17228,7408,23011,7409,23012,4079,20035,15362,
	7406,2891,2141,7410,23013,19102,23014,15363,
	23018,19103,15364,7413,1896,7415,20036,19104,
	7677,7416,2267,19106,7417,23019,7418,7423,
	7421,4066,7422,23022,4023,23023,7424,17230,
	4003,15365,7425,3380,15366,7426,7428,15367,
	7429,23024,17231,20040,23025,7682,7797,7427,
	2047,3801,3729,4740,19110,5375,3753,13323,
	1323,23028,7431,1642,8717,2055,7435,23029,
	5645,3815,2883,7436,7438,7437,19112,7439,
	20041,7440,15368,7441,15369,7442,20042,23032,
	20043,7443,23033,23034,7446,19114,19116,3102,
	20045,15370,23035,15371,1937,20046,7453,23036,
	15372,15373,23037,20047,23038,2767,7454,20048,
	15376,19117,3475,23041,19118,7456,23042,15378,
	19119,23045,7457,5898,6779,7174,7458,19120,
	7459,19121,23046,15379,19122,19124,23047,4024,
	7460,15380,19126,7467,7466,19127,23048,20054,
	19128,15381,7469,23049,15382,7470,7468,23052,
	15383,15384,7471,23055,17232,7472,5927,20055,
	7473,23056,15385,23057,15431,15429,15434,13839,
	18344,7814,18620,20061,18124,18352,7817,13807,
	17478,18209,19075,20220,17885,18624,17297,18181,
	17784,17783,13841,18757,17539,18303,17788,14121,
	16821,13832,7670,17233,14108,20063,13928,17653,
	13750,18708,17327,16833,14293,14140,7695,18444,
	17814,20065,17103,20057,18571,13523,17009,13932,
	18882,7839,13809,18185,14188,17942,13357,18812,
	13854,18817,14180,17599,17331,14109,18672,14253,
	18824,17657,17227,17827,13670,17552,17332,18716,
	17551,17033,13857,17744,17559,17742,16888,20112,
	18193,18944,14068,17945,18268,17308,17472,18452,
	20066,14069,17561,17373,15269,17743,17944,17943,
	18885,17414,19125,18718,14294,17259,18638,18637,
	14256,19105,18843,18886,14105,18049,17562,16845,
	18890,18162,17843,17841,15393,20124,18848,18197,
	18104,17415,18319,17388,18320,18385,18847,19001,
	18416,14075,18893,19037,18170,18463,17897,18855,
	14100,18918,20072,18466,17667,18784,18325,18277,
	17428,18329,19051,18895,20083,18959,18278,16816,
	17854,17340,18430,17575,16916,17867,17018,17761,
	18293,18699,18249,18702,20128,17146,19071,17764,
	17763,18339,17770,18611,18610,17774,18338,16970,
	20059,18903,17295,17585,18345,18617,17874,20303,
	20305,7811,14097,8410,8421,7746,14290,8481,
	8497,8499,20307,8548,8571,8583,8587,8590,
	8599,8610,8622,8630,18760,8633,8636,8664,
	8671,8687,8702,8715,13382,13360,13389,13385,
	13338,13378,7651,13366,13333,13384,13387,13361,
	16837,13326,13328,13363,13369,13381,13334,13352,
	13375,13327,7700,13332,13347,15398,7732,13379,
	13348,13345,13335,13391,13359,13351,13325,13371,
	13343,13373,13358,13399,18366,13376,13353,14199,
	14198,13367,13331,13346,13321,13339,13380,13364,
	15403,13320,13374,13337,7788,642,8007,716,
	712,715,717,8006,718,692,636,693,
	637,663,699,697,700,641,719,678,
	664,679,648,650,646,680,667,681,
	665,751,649,8005,711,
};

pdf_cmap pdf_cmap_UniJIS_UTF16_H =
{
	-1, "UniJIS-UTF16-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 4, 0xdc00, 0xdfff },
		{ 2, 0xe000, 0xffff },
	},
	13892, 13892, (pdf_range*) pdf_cmap_UniJIS_UTF16_H_ranges,
	10909, 10909, (unsigned short*) pdf_cmap_UniJIS_UTF16_H_table,
};

/* UniJIS-UTF16-V */

static const pdf_range pdf_cmap_UniJIS_UTF16_V_ranges[] =
{
	{ 0x00b0, 0x00b0, PDF_CMAP_SINGLE, 8269 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 7893 },
	{ 0x2015, 0x2016, PDF_CMAP_TABLE, 0 },
	{ 0x2025, 0x2026, PDF_CMAP_TABLE, 2 },
	{ 0x2032, 0x2033, PDF_CMAP_TABLE, 4 },
	{ 0x2190, 0x2193, PDF_CMAP_TABLE, 6 },
	{ 0x21c4, 0x21c6, PDF_CMAP_TABLE, 10 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 13 },
	{ 0x239b, 0x23ad, PDF_CMAP_TABLE, 17 },
	{ 0x23b0, 0x23b1, PDF_CMAP_RANGE, 16350 },
	{ 0x2500, 0x2501, PDF_CMAP_RANGE, 7481 },
	{ 0x2502, 0x2503, PDF_CMAP_RANGE, 7479 },
	{ 0x2504, 0x2505, PDF_CMAP_RANGE, 7485 },
	{ 0x2506, 0x2507, PDF_CMAP_RANGE, 7483 },
	{ 0x2508, 0x2509, PDF_CMAP_RANGE, 7489 },
	{ 0x250a, 0x250b, PDF_CMAP_RANGE, 7487 },
	{ 0x250c, 0x2522, PDF_CMAP_TABLE, 36 },
	{ 0x2523, 0x2524, PDF_CMAP_RANGE, 7530 },
	{ 0x2525, 0x252c, PDF_CMAP_TABLE, 59 },
	{ 0x252d, 0x252f, PDF_CMAP_RANGE, 7517 },
	{ 0x2530, 0x2530, PDF_CMAP_SINGLE, 7516 },
	{ 0x2531, 0x2533, PDF_CMAP_RANGE, 7520 },
	{ 0x2534, 0x2534, PDF_CMAP_SINGLE, 7507 },
	{ 0x2535, 0x2537, PDF_CMAP_RANGE, 7509 },
	{ 0x2538, 0x2538, PDF_CMAP_SINGLE, 7508 },
	{ 0x2539, 0x253b, PDF_CMAP_RANGE, 7512 },
	{ 0x253d, 0x253f, PDF_CMAP_RANGE, 7543 },
	{ 0x2540, 0x2548, PDF_CMAP_TABLE, 67 },
	{ 0x2549, 0x254a, PDF_CMAP_RANGE, 7550 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 76 },
	{ 0x2702, 0x2702, PDF_CMAP_SINGLE, 12178 },
	{ 0x27a1, 0x27a1, PDF_CMAP_SINGLE, 8209 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 7887 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 7907 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 7901 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 16329 },
	{ 0x3018, 0x3019, PDF_CMAP_RANGE, 12139 },
	{ 0x301c, 0x301d, PDF_CMAP_TABLE, 80 },
	{ 0x301f, 0x301f, PDF_CMAP_SINGLE, 7957 },
	{ 0x3041, 0x3041, PDF_CMAP_SINGLE, 7918 },
	{ 0x3043, 0x3043, PDF_CMAP_SINGLE, 7919 },
	{ 0x3045, 0x3045, PDF_CMAP_SINGLE, 7920 },
	{ 0x3047, 0x3047, PDF_CMAP_SINGLE, 7921 },
	{ 0x3049, 0x3049, PDF_CMAP_SINGLE, 7922 },
	{ 0x3063, 0x3063, PDF_CMAP_SINGLE, 7923 },
	{ 0x3083, 0x3083, PDF_CMAP_SINGLE, 7924 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x3095, 0x3096, PDF_CMAP_RANGE, 8264 },
	{ 0x309b, 0x309c, PDF_CMAP_TABLE, 82 },
	{ 0x30a0, 0x30a1, PDF_CMAP_TABLE, 84 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x31f0, 0x31f9, PDF_CMAP_RANGE, 16333 },
	{ 0x31fa, 0x31ff, PDF_CMAP_RANGE, 16344 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x330d, PDF_CMAP_TABLE, 86 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3319, PDF_CMAP_TABLE, 97 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3327, PDF_CMAP_TABLE, 103 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332b, PDF_CMAP_TABLE, 109 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3333, PDF_CMAP_TABLE, 111 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x333d, PDF_CMAP_TABLE, 114 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3342, PDF_CMAP_TABLE, 122 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x334a, PDF_CMAP_TABLE, 124 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334e, PDF_CMAP_TABLE, 128 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3354, PDF_CMAP_TABLE, 130 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5e, PDF_CMAP_TABLE, 134 },
	{ 0xff5f, 0xff60, PDF_CMAP_RANGE, 12141 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 7907 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 7901 },
	{ 0x3016, 0x3017, PDF_CMAP_RANGE, 16329 },
	{ 0x3018, 0x3019, PDF_CMAP_RANGE, 12139 },
	{ 0x301c, 0x301c, PDF_CMAP_SINGLE, 7894 },
	{ 0x301d, 0x301d, PDF_CMAP_SINGLE, 7956 },
	{ 0x301f, 0x301f, PDF_CMAP_SINGLE, 7957 },
	{ 0x3041, 0x3041, PDF_CMAP_SINGLE, 7918 },
	{ 0x3043, 0x3043, PDF_CMAP_SINGLE, 7919 },
	{ 0x3045, 0x3045, PDF_CMAP_SINGLE, 7920 },
	{ 0x3047, 0x3047, PDF_CMAP_SINGLE, 7921 },
	{ 0x3049, 0x3049, PDF_CMAP_SINGLE, 7922 },
	{ 0x3063, 0x3063, PDF_CMAP_SINGLE, 7923 },
	{ 0x3083, 0x3083, PDF_CMAP_SINGLE, 7924 },
	{ 0x3085, 0x3085, PDF_CMAP_SINGLE, 7925 },
	{ 0x3087, 0x3087, PDF_CMAP_SINGLE, 7926 },
	{ 0x308e, 0x308e, PDF_CMAP_SINGLE, 7927 },
	{ 0x3095, 0x3096, PDF_CMAP_RANGE, 8264 },
	{ 0x309b, 0x309b, PDF_CMAP_SINGLE, 8272 },
	{ 0x309c, 0x309c, PDF_CMAP_SINGLE, 8271 },
	{ 0x30a0, 0x30a0, PDF_CMAP_SINGLE, 16331 },
	{ 0x30a1, 0x30a1, PDF_CMAP_SINGLE, 7928 },
	{ 0x30a3, 0x30a3, PDF_CMAP_SINGLE, 7929 },
	{ 0x30a5, 0x30a5, PDF_CMAP_SINGLE, 7930 },
	{ 0x30a7, 0x30a7, PDF_CMAP_SINGLE, 7931 },
	{ 0x30a9, 0x30a9, PDF_CMAP_SINGLE, 7932 },
	{ 0x30c3, 0x30c3, PDF_CMAP_SINGLE, 7933 },
	{ 0x30e3, 0x30e3, PDF_CMAP_SINGLE, 7934 },
	{ 0x30e5, 0x30e5, PDF_CMAP_SINGLE, 7935 },
	{ 0x30e7, 0x30e7, PDF_CMAP_SINGLE, 7936 },
	{ 0x30ee, 0x30ee, PDF_CMAP_SINGLE, 7937 },
	{ 0x30f5, 0x30f6, PDF_CMAP_RANGE, 7938 },
	{ 0x30fc, 0x30fc, PDF_CMAP_SINGLE, 7891 },
	{ 0x31f0, 0x31f9, PDF_CMAP_RANGE, 16333 },
	{ 0x31fa, 0x31ff, PDF_CMAP_RANGE, 16344 },
	{ 0x3300, 0x3300, PDF_CMAP_SINGLE, 8350 },
	{ 0x3301, 0x3302, PDF_CMAP_RANGE, 11958 },
	{ 0x3303, 0x3303, PDF_CMAP_SINGLE, 8338 },
	{ 0x3304, 0x3304, PDF_CMAP_SINGLE, 11960 },
	{ 0x3305, 0x3305, PDF_CMAP_SINGLE, 8333 },
	{ 0x3306, 0x3306, PDF_CMAP_SINGLE, 11961 },
	{ 0x3307, 0x3307, PDF_CMAP_SINGLE, 11965 },
	{ 0x3308, 0x3308, PDF_CMAP_SINGLE, 11963 },
	{ 0x3309, 0x3309, PDF_CMAP_SINGLE, 11968 },
	{ 0x330a, 0x330a, PDF_CMAP_SINGLE, 11966 },
	{ 0x330b, 0x330b, PDF_CMAP_SINGLE, 11970 },
	{ 0x330c, 0x330c, PDF_CMAP_SINGLE, 11972 },
	{ 0x330d, 0x330d, PDF_CMAP_SINGLE, 7950 },
	{ 0x330e, 0x3313, PDF_CMAP_RANGE, 11973 },
	{ 0x3314, 0x3314, PDF_CMAP_SINGLE, 7941 },
	{ 0x3315, 0x3315, PDF_CMAP_SINGLE, 8340 },
	{ 0x3316, 0x3316, PDF_CMAP_SINGLE, 8330 },
	{ 0x3317, 0x3317, PDF_CMAP_SINGLE, 11980 },
	{ 0x3318, 0x3318, PDF_CMAP_SINGLE, 8339 },
	{ 0x3319, 0x3319, PDF_CMAP_SINGLE, 11982 },
	{ 0x331a, 0x331d, PDF_CMAP_RANGE, 11984 },
	{ 0x331e, 0x331e, PDF_CMAP_SINGLE, 8353 },
	{ 0x331f, 0x3321, PDF_CMAP_RANGE, 11988 },
	{ 0x3322, 0x3322, PDF_CMAP_SINGLE, 8329 },
	{ 0x3323, 0x3323, PDF_CMAP_SINGLE, 8348 },
	{ 0x3324, 0x3324, PDF_CMAP_SINGLE, 11991 },
	{ 0x3325, 0x3325, PDF_CMAP_SINGLE, 11993 },
	{ 0x3326, 0x3326, PDF_CMAP_SINGLE, 7951 },
	{ 0x3327, 0x3327, PDF_CMAP_SINGLE, 7945 },
	{ 0x3328, 0x3329, PDF_CMAP_RANGE, 11996 },
	{ 0x332a, 0x332a, PDF_CMAP_SINGLE, 8356 },
	{ 0x332b, 0x332b, PDF_CMAP_SINGLE, 7953 },
	{ 0x332d, 0x332d, PDF_CMAP_SINGLE, 11999 },
	{ 0x332e, 0x3330, PDF_CMAP_RANGE, 12002 },
	{ 0x3331, 0x3331, PDF_CMAP_SINGLE, 8358 },
	{ 0x3332, 0x3332, PDF_CMAP_SINGLE, 12005 },
	{ 0x3333, 0x3333, PDF_CMAP_SINGLE, 8334 },
	{ 0x3334, 0x3335, PDF_CMAP_RANGE, 12008 },
	{ 0x3336, 0x3336, PDF_CMAP_SINGLE, 7947 },
	{ 0x3337, 0x3337, PDF_CMAP_SINGLE, 12014 },
	{ 0x3338, 0x3338, PDF_CMAP_SINGLE, 12016 },
	{ 0x3339, 0x3339, PDF_CMAP_SINGLE, 8343 },
	{ 0x333a, 0x333a, PDF_CMAP_SINGLE, 12017 },
	{ 0x333b, 0x333b, PDF_CMAP_SINGLE, 8349 },
	{ 0x333c, 0x333c, PDF_CMAP_SINGLE, 12010 },
	{ 0x333d, 0x333d, PDF_CMAP_SINGLE, 12018 },
	{ 0x333e, 0x3340, PDF_CMAP_RANGE, 12020 },
	{ 0x3341, 0x3341, PDF_CMAP_SINGLE, 12019 },
	{ 0x3342, 0x3342, PDF_CMAP_SINGLE, 8347 },
	{ 0x3343, 0x3346, PDF_CMAP_RANGE, 12023 },
	{ 0x3347, 0x3347, PDF_CMAP_SINGLE, 8357 },
	{ 0x3348, 0x3348, PDF_CMAP_SINGLE, 12027 },
	{ 0x3349, 0x3349, PDF_CMAP_SINGLE, 7940 },
	{ 0x334a, 0x334a, PDF_CMAP_SINGLE, 7954 },
	{ 0x334b, 0x334c, PDF_CMAP_RANGE, 12028 },
	{ 0x334d, 0x334d, PDF_CMAP_SINGLE, 7943 },
	{ 0x334e, 0x334e, PDF_CMAP_SINGLE, 8337 },
	{ 0x334f, 0x3350, PDF_CMAP_RANGE, 12030 },
	{ 0x3351, 0x3351, PDF_CMAP_SINGLE, 7948 },
	{ 0x3352, 0x3352, PDF_CMAP_SINGLE, 12034 },
	{ 0x3353, 0x3353, PDF_CMAP_SINGLE, 12038 },
	{ 0x3354, 0x3354, PDF_CMAP_SINGLE, 12035 },
	{ 0x3355, 0x3356, PDF_CMAP_RANGE, 12039 },
	{ 0x3357, 0x3357, PDF_CMAP_SINGLE, 8344 },
	{ 0x337f, 0x337f, PDF_CMAP_SINGLE, 8324 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 7899 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8268 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8274 },
	{ 0xff1d, 0xff1d, PDF_CMAP_SINGLE, 7917 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 7903 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 7904 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 7890 },
	{ 0xff5b, 0xff5b, PDF_CMAP_SINGLE, 7905 },
	{ 0xff5c, 0xff5c, PDF_CMAP_SINGLE, 7896 },
	{ 0xff5d, 0xff5d, PDF_CMAP_SINGLE, 7906 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 7894 },
	{ 0xff5f, 0xff60, PDF_CMAP_RANGE, 12141 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 7889 },
};

static const unsigned short pdf_cmap_UniJIS_UTF16_V_table[138] =
{
	7892,7895,7898,7897,8273,8283,738,736,
	739,737,8311,8310,8312,8012,8014,8011,
	8013,12148,12168,12147,12150,12168,12149,12156,
	12168,12155,12158,12168,12157,8168,8167,8166,
	12168,8172,8171,8170,7495,7497,7496,7498,
	7503,7505,7504,7506,7491,7493,7492,7494,
	7499,7501,7500,7502,7523,7527,7525,7524,
	7526,7529,7528,7535,7533,7532,7534,7537,
	7536,7538,7515,7541,7540,7542,7547,7549,
	7546,7548,7553,7552,8221,8219,8222,8220,
	7894,7956,8272,8271,16331,7928,8338,11960,
	8333,11961,11965,11963,11968,11966,11970,11972,
	7950,7941,8340,8330,11980,8339,11982,8329,
	8348,11991,11993,7951,7945,8356,7953,8358,
	12005,8334,7947,12014,12016,8343,12017,8349,
	12010,12018,12019,8347,8357,12027,7940,7954,
	7943,8337,7948,12034,12038,12035,7905,7896,
	7906,7894,
};

pdf_cmap pdf_cmap_UniJIS_UTF16_V =
{
	-1, "UniJIS-UTF16-V", "UniJIS-UTF16-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	215, 215, (pdf_range*) pdf_cmap_UniJIS_UTF16_V_ranges,
	138, 138, (unsigned short*) pdf_cmap_UniJIS_UTF16_V_table,
};

