#include "fitz.h"
#include "mupdf.h"

/* Adobe-Korea1-0 */

static const pdf_range pdf_cmap_Adobe_Korea1_0_ranges[] =
{
	{ 0x0000, 0x2474, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x2474, PDF_CMAP_RANGE, 9216 },
};

static const unsigned short pdf_cmap_Adobe_Korea1_0_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Korea1_0 =
{
	-1, "Adobe-Korea1-0", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x24ff },
	},
	37, 37, (pdf_range*) pdf_cmap_Adobe_Korea1_0_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Korea1_0_table,
};

/* Adobe-Korea1-1 */

static const pdf_range pdf_cmap_Adobe_Korea1_1_ranges[] =
{
	{ 0x0000, 0x46ea, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ea, PDF_CMAP_RANGE, 17920 },
};

static const unsigned short pdf_cmap_Adobe_Korea1_1_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Korea1_1 =
{
	-1, "Adobe-Korea1-1", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x46ff },
	},
	71, 71, (pdf_range*) pdf_cmap_Adobe_Korea1_1_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Korea1_1_table,
};

/* Adobe-Korea1-2 */

static const pdf_range pdf_cmap_Adobe_Korea1_2_ranges[] =
{
	{ 0x0000, 0x47af, PDF_CMAP_RANGE, 0 },
	{ 0x0100, 0x01ff, PDF_CMAP_RANGE, 256 },
	{ 0x0200, 0x02ff, PDF_CMAP_RANGE, 512 },
	{ 0x0300, 0x03ff, PDF_CMAP_RANGE, 768 },
	{ 0x0400, 0x04ff, PDF_CMAP_RANGE, 1024 },
	{ 0x0500, 0x05ff, PDF_CMAP_RANGE, 1280 },
	{ 0x0600, 0x06ff, PDF_CMAP_RANGE, 1536 },
	{ 0x0700, 0x07ff, PDF_CMAP_RANGE, 1792 },
	{ 0x0800, 0x08ff, PDF_CMAP_RANGE, 2048 },
	{ 0x0900, 0x09ff, PDF_CMAP_RANGE, 2304 },
	{ 0x0a00, 0x0aff, PDF_CMAP_RANGE, 2560 },
	{ 0x0b00, 0x0bff, PDF_CMAP_RANGE, 2816 },
	{ 0x0c00, 0x0cff, PDF_CMAP_RANGE, 3072 },
	{ 0x0d00, 0x0dff, PDF_CMAP_RANGE, 3328 },
	{ 0x0e00, 0x0eff, PDF_CMAP_RANGE, 3584 },
	{ 0x0f00, 0x0fff, PDF_CMAP_RANGE, 3840 },
	{ 0x1000, 0x10ff, PDF_CMAP_RANGE, 4096 },
	{ 0x1100, 0x11ff, PDF_CMAP_RANGE, 4352 },
	{ 0x1200, 0x12ff, PDF_CMAP_RANGE, 4608 },
	{ 0x1300, 0x13ff, PDF_CMAP_RANGE, 4864 },
	{ 0x1400, 0x14ff, PDF_CMAP_RANGE, 5120 },
	{ 0x1500, 0x15ff, PDF_CMAP_RANGE, 5376 },
	{ 0x1600, 0x16ff, PDF_CMAP_RANGE, 5632 },
	{ 0x1700, 0x17ff, PDF_CMAP_RANGE, 5888 },
	{ 0x1800, 0x18ff, PDF_CMAP_RANGE, 6144 },
	{ 0x1900, 0x19ff, PDF_CMAP_RANGE, 6400 },
	{ 0x1a00, 0x1aff, PDF_CMAP_RANGE, 6656 },
	{ 0x1b00, 0x1bff, PDF_CMAP_RANGE, 6912 },
	{ 0x1c00, 0x1cff, PDF_CMAP_RANGE, 7168 },
	{ 0x1d00, 0x1dff, PDF_CMAP_RANGE, 7424 },
	{ 0x1e00, 0x1eff, PDF_CMAP_RANGE, 7680 },
	{ 0x1f00, 0x1fff, PDF_CMAP_RANGE, 7936 },
	{ 0x2000, 0x20ff, PDF_CMAP_RANGE, 8192 },
	{ 0x2100, 0x21ff, PDF_CMAP_RANGE, 8448 },
	{ 0x2200, 0x22ff, PDF_CMAP_RANGE, 8704 },
	{ 0x2300, 0x23ff, PDF_CMAP_RANGE, 8960 },
	{ 0x2400, 0x24ff, PDF_CMAP_RANGE, 9216 },
	{ 0x2500, 0x25ff, PDF_CMAP_RANGE, 9472 },
	{ 0x2600, 0x26ff, PDF_CMAP_RANGE, 9728 },
	{ 0x2700, 0x27ff, PDF_CMAP_RANGE, 9984 },
	{ 0x2800, 0x28ff, PDF_CMAP_RANGE, 10240 },
	{ 0x2900, 0x29ff, PDF_CMAP_RANGE, 10496 },
	{ 0x2a00, 0x2aff, PDF_CMAP_RANGE, 10752 },
	{ 0x2b00, 0x2bff, PDF_CMAP_RANGE, 11008 },
	{ 0x2c00, 0x2cff, PDF_CMAP_RANGE, 11264 },
	{ 0x2d00, 0x2dff, PDF_CMAP_RANGE, 11520 },
	{ 0x2e00, 0x2eff, PDF_CMAP_RANGE, 11776 },
	{ 0x2f00, 0x2fff, PDF_CMAP_RANGE, 12032 },
	{ 0x3000, 0x30ff, PDF_CMAP_RANGE, 12288 },
	{ 0x3100, 0x31ff, PDF_CMAP_RANGE, 12544 },
	{ 0x3200, 0x32ff, PDF_CMAP_RANGE, 12800 },
	{ 0x3300, 0x33ff, PDF_CMAP_RANGE, 13056 },
	{ 0x3400, 0x34ff, PDF_CMAP_RANGE, 13312 },
	{ 0x3500, 0x35ff, PDF_CMAP_RANGE, 13568 },
	{ 0x3600, 0x36ff, PDF_CMAP_RANGE, 13824 },
	{ 0x3700, 0x37ff, PDF_CMAP_RANGE, 14080 },
	{ 0x3800, 0x38ff, PDF_CMAP_RANGE, 14336 },
	{ 0x3900, 0x39ff, PDF_CMAP_RANGE, 14592 },
	{ 0x3a00, 0x3aff, PDF_CMAP_RANGE, 14848 },
	{ 0x3b00, 0x3bff, PDF_CMAP_RANGE, 15104 },
	{ 0x3c00, 0x3cff, PDF_CMAP_RANGE, 15360 },
	{ 0x3d00, 0x3dff, PDF_CMAP_RANGE, 15616 },
	{ 0x3e00, 0x3eff, PDF_CMAP_RANGE, 15872 },
	{ 0x3f00, 0x3fff, PDF_CMAP_RANGE, 16128 },
	{ 0x4000, 0x40ff, PDF_CMAP_RANGE, 16384 },
	{ 0x4100, 0x41ff, PDF_CMAP_RANGE, 16640 },
	{ 0x4200, 0x42ff, PDF_CMAP_RANGE, 16896 },
	{ 0x4300, 0x43ff, PDF_CMAP_RANGE, 17152 },
	{ 0x4400, 0x44ff, PDF_CMAP_RANGE, 17408 },
	{ 0x4500, 0x45ff, PDF_CMAP_RANGE, 17664 },
	{ 0x4600, 0x46ff, PDF_CMAP_RANGE, 17920 },
	{ 0x4700, 0x47af, PDF_CMAP_RANGE, 18176 },
};

static const unsigned short pdf_cmap_Adobe_Korea1_2_table[1] = { 0 };

pdf_cmap pdf_cmap_Adobe_Korea1_2 =
{
	-1, "Adobe-Korea1-2", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x0000, 0x47ff },
	},
	72, 72, (pdf_range*) pdf_cmap_Adobe_Korea1_2_ranges,
	0, 0, (unsigned short*) pdf_cmap_Adobe_Korea1_2_table,
};

/* KSC-EUC-H */

static const pdf_range pdf_cmap_KSC_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 8094 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 101 },
	{ 0xa2a1, 0xa2e5, PDF_CMAP_RANGE, 195 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 264 },
	{ 0xa4a1, 0xa4d3, PDF_CMAP_RANGE, 358 },
	{ 0xa4d5, 0xa4fe, PDF_CMAP_RANGE, 409 },
	{ 0xa5a1, 0xa5aa, PDF_CMAP_RANGE, 451 },
	{ 0xa5b0, 0xa5b9, PDF_CMAP_RANGE, 461 },
	{ 0xa5c1, 0xa5d8, PDF_CMAP_RANGE, 471 },
	{ 0xa5e1, 0xa5f8, PDF_CMAP_RANGE, 495 },
	{ 0xa6a1, 0xa6e4, PDF_CMAP_RANGE, 519 },
	{ 0xa7a1, 0xa7ef, PDF_CMAP_RANGE, 587 },
	{ 0xa8a1, 0xa8a4, PDF_CMAP_RANGE, 666 },
	{ 0xa8a6, 0xa8a6, PDF_CMAP_SINGLE, 670 },
	{ 0xa8a8, 0xa8af, PDF_CMAP_RANGE, 671 },
	{ 0xa8b1, 0xa8fe, PDF_CMAP_RANGE, 679 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 757 },
	{ 0xaaa1, 0xaaf3, PDF_CMAP_RANGE, 851 },
	{ 0xaba1, 0xabf6, PDF_CMAP_RANGE, 934 },
	{ 0xaca1, 0xacc1, PDF_CMAP_RANGE, 1020 },
	{ 0xacd1, 0xacf1, PDF_CMAP_RANGE, 1053 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 1086 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1180 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1274 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1368 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1462 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1556 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1650 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1744 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1838 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1932 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 2026 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 2120 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2214 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2308 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2402 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2496 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2590 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2684 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2778 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2872 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2966 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 3060 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3154 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3248 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3342 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3436 },
	{ 0xcba1, 0xcbcf, PDF_CMAP_RANGE, 3530 },
	{ 0xcbd0, 0xcbd0, PDF_CMAP_SINGLE, 4116 },
	{ 0xcbd1, 0xcbd5, PDF_CMAP_RANGE, 3577 },
	{ 0xcbd6, 0xcbd6, PDF_CMAP_SINGLE, 3678 },
	{ 0xcbd7, 0xcbe6, PDF_CMAP_RANGE, 3582 },
	{ 0xcbe7, 0xcbe7, PDF_CMAP_SINGLE, 7053 },
	{ 0xcbe8, 0xcbfe, PDF_CMAP_RANGE, 3598 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3621 },
	{ 0xcda1, 0xcdce, PDF_CMAP_RANGE, 3715 },
	{ 0xcdcf, 0xcdcf, PDF_CMAP_SINGLE, 3460 },
	{ 0xcdd0, 0xcde7, PDF_CMAP_RANGE, 3761 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 7900 },
	{ 0xcde9, 0xcdfe, PDF_CMAP_RANGE, 3785 },
	{ 0xcea1, 0xceac, PDF_CMAP_RANGE, 3807 },
	{ 0xcead, 0xcead, PDF_CMAP_SINGLE, 3802 },
	{ 0xceae, 0xcefe, PDF_CMAP_RANGE, 3819 },
	{ 0xcfa1, 0xcffa, PDF_CMAP_RANGE, 3900 },
	{ 0xcffb, 0xcffb, PDF_CMAP_SINGLE, 3902 },
	{ 0xcffc, 0xcffe, PDF_CMAP_RANGE, 3990 },
	{ 0xd0a1, 0xd0a2, PDF_CMAP_TABLE, 0 },
	{ 0xd0a3, 0xd0b7, PDF_CMAP_RANGE, 3994 },
	{ 0xd0b8, 0xd0b8, PDF_CMAP_SINGLE, 3946 },
	{ 0xd0b9, 0xd0cf, PDF_CMAP_RANGE, 4015 },
	{ 0xd0d0, 0xd0d0, PDF_CMAP_SINGLE, 3708 },
	{ 0xd0d1, 0xd0dc, PDF_CMAP_RANGE, 4038 },
	{ 0xd0dd, 0xd0dd, PDF_CMAP_SINGLE, 4131 },
	{ 0xd0de, 0xd0fe, PDF_CMAP_RANGE, 4050 },
	{ 0xd1a1, 0xd1d3, PDF_CMAP_RANGE, 4083 },
	{ 0xd1d4, 0xd1d5, PDF_CMAP_TABLE, 2 },
	{ 0xd1d6, 0xd1d7, PDF_CMAP_RANGE, 4134 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 4375 },
	{ 0xd1d9, 0xd1da, PDF_CMAP_RANGE, 4136 },
	{ 0xd1db, 0xd1e0, PDF_CMAP_RANGE, 4376 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_TABLE, 4 },
	{ 0xd1e3, 0xd1e5, PDF_CMAP_RANGE, 4382 },
	{ 0xd1e6, 0xd1e7, PDF_CMAP_TABLE, 6 },
	{ 0xd1e8, 0xd1eb, PDF_CMAP_RANGE, 4387 },
	{ 0xd1ec, 0xd1ee, PDF_CMAP_TABLE, 8 },
	{ 0xd1ef, 0xd1f0, PDF_CMAP_RANGE, 4394 },
	{ 0xd1f1, 0xd1f2, PDF_CMAP_TABLE, 11 },
	{ 0xd1f3, 0xd1f5, PDF_CMAP_RANGE, 4143 },
	{ 0xd1f6, 0xd1f6, PDF_CMAP_SINGLE, 4399 },
	{ 0xd1f7, 0xd1f9, PDF_CMAP_RANGE, 4146 },
	{ 0xd1fa, 0xd1fb, PDF_CMAP_TABLE, 13 },
	{ 0xd1fc, 0xd1fd, PDF_CMAP_RANGE, 4406 },
	{ 0xd1fe, 0xd1fe, PDF_CMAP_SINGLE, 4409 },
	{ 0xd2a1, 0xd2a1, PDF_CMAP_SINGLE, 4150 },
	{ 0xd2a2, 0xd2a3, PDF_CMAP_RANGE, 4410 },
	{ 0xd2a4, 0xd2a6, PDF_CMAP_RANGE, 4151 },
	{ 0xd2a7, 0xd2aa, PDF_CMAP_RANGE, 4412 },
	{ 0xd2ab, 0xd2ad, PDF_CMAP_TABLE, 15 },
	{ 0xd2ae, 0xd2b1, PDF_CMAP_RANGE, 4155 },
	{ 0xd2b2, 0xd2b2, PDF_CMAP_SINGLE, 4424 },
	{ 0xd2b3, 0xd2bd, PDF_CMAP_RANGE, 4159 },
	{ 0xd2be, 0xd2be, PDF_CMAP_SINGLE, 4511 },
	{ 0xd2bf, 0xd2c1, PDF_CMAP_RANGE, 4170 },
	{ 0xd2c2, 0xd2c3, PDF_CMAP_RANGE, 4513 },
	{ 0xd2c4, 0xd2c5, PDF_CMAP_TABLE, 18 },
	{ 0xd2c6, 0xd2ca, PDF_CMAP_RANGE, 4518 },
	{ 0xd2cb, 0xd2cc, PDF_CMAP_TABLE, 20 },
	{ 0xd2cd, 0xd2ce, PDF_CMAP_RANGE, 4525 },
	{ 0xd2cf, 0xd2d4, PDF_CMAP_RANGE, 4528 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 4535 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 4175 },
	{ 0xd2d9, 0xd2da, PDF_CMAP_RANGE, 4541 },
	{ 0xd2db, 0xd2dd, PDF_CMAP_RANGE, 4176 },
	{ 0xd2de, 0xd2df, PDF_CMAP_RANGE, 4545 },
	{ 0xd2e0, 0xd2e4, PDF_CMAP_TABLE, 22 },
	{ 0xd2e5, 0xd2e8, PDF_CMAP_RANGE, 4566 },
	{ 0xd2e9, 0xd2ea, PDF_CMAP_RANGE, 4571 },
	{ 0xd2eb, 0xd2eb, PDF_CMAP_SINGLE, 4576 },
	{ 0xd2ec, 0xd2ef, PDF_CMAP_RANGE, 4181 },
	{ 0xd2f0, 0xd2f3, PDF_CMAP_RANGE, 4604 },
	{ 0xd2f4, 0xd2f5, PDF_CMAP_RANGE, 4609 },
	{ 0xd2f6, 0xd2f6, PDF_CMAP_SINGLE, 4185 },
	{ 0xd2f7, 0xd2f8, PDF_CMAP_RANGE, 4611 },
	{ 0xd2f9, 0xd2fe, PDF_CMAP_RANGE, 4186 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4192 },
	{ 0xd4a1, 0xd4e5, PDF_CMAP_RANGE, 4286 },
	{ 0xd4e6, 0xd4e6, PDF_CMAP_SINGLE, 4318 },
	{ 0xd4e7, 0xd4fb, PDF_CMAP_RANGE, 4355 },
	{ 0xd4fc, 0xd4fc, PDF_CMAP_SINGLE, 4136 },
	{ 0xd4fd, 0xd4fe, PDF_CMAP_RANGE, 4376 },
	{ 0xd5a1, 0xd5a4, PDF_CMAP_RANGE, 4378 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 5800 },
	{ 0xd5a6, 0xd5aa, PDF_CMAP_RANGE, 4382 },
	{ 0xd5ab, 0xd5ab, PDF_CMAP_SINGLE, 4139 },
	{ 0xd5ac, 0xd5ad, PDF_CMAP_RANGE, 4387 },
	{ 0xd5ae, 0xd5ae, PDF_CMAP_SINGLE, 4192 },
	{ 0xd5af, 0xd5fe, PDF_CMAP_RANGE, 4389 },
	{ 0xd6a1, 0xd6b7, PDF_CMAP_RANGE, 4469 },
	{ 0xd6b8, 0xd6b8, PDF_CMAP_SINGLE, 4167 },
	{ 0xd6b9, 0xd6cc, PDF_CMAP_RANGE, 4492 },
	{ 0xd6cd, 0xd6cd, PDF_CMAP_SINGLE, 4172 },
	{ 0xd6ce, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd7a1, 0xd7ca, PDF_CMAP_RANGE, 4561 },
	{ 0xd7cb, 0xd7cb, PDF_CMAP_SINGLE, 5552 },
	{ 0xd7cc, 0xd7e3, PDF_CMAP_RANGE, 4603 },
	{ 0xd7e4, 0xd7e4, PDF_CMAP_SINGLE, 6424 },
	{ 0xd7e5, 0xd7fe, PDF_CMAP_RANGE, 4627 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4653 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4747 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4841 },
	{ 0xdba1, 0xdbc4, PDF_CMAP_RANGE, 4935 },
	{ 0xdbc5, 0xdbc5, PDF_CMAP_SINGLE, 5151 },
	{ 0xdbc6, 0xdbe3, PDF_CMAP_RANGE, 4971 },
	{ 0xdbe4, 0xdbe4, PDF_CMAP_SINGLE, 4922 },
	{ 0xdbe5, 0xdbfe, PDF_CMAP_RANGE, 5001 },
	{ 0xdca1, 0xdca4, PDF_CMAP_RANGE, 5027 },
	{ 0xdca5, 0xdca5, PDF_CMAP_SINGLE, 7518 },
	{ 0xdca6, 0xdcfe, PDF_CMAP_RANGE, 5031 },
	{ 0xdda1, 0xdda4, PDF_CMAP_RANGE, 5120 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 5079 },
	{ 0xdda6, 0xddd4, PDF_CMAP_RANGE, 5124 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 5109 },
	{ 0xddd6, 0xddf3, PDF_CMAP_RANGE, 5171 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 7607 },
	{ 0xddf5, 0xddfe, PDF_CMAP_RANGE, 5201 },
	{ 0xdea1, 0xdefb, PDF_CMAP_RANGE, 5211 },
	{ 0xdefc, 0xdefe, PDF_CMAP_TABLE, 27 },
	{ 0xdfa1, 0xdfb2, PDF_CMAP_RANGE, 5303 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 7083 },
	{ 0xdfb4, 0xdfe0, PDF_CMAP_RANGE, 5321 },
	{ 0xdfe1, 0xdfe1, PDF_CMAP_SINGLE, 5362 },
	{ 0xdfe2, 0xdfe7, PDF_CMAP_RANGE, 5366 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 5485 },
	{ 0xdfe9, 0xdffe, PDF_CMAP_RANGE, 5372 },
	{ 0xe0a1, 0xe0f0, PDF_CMAP_RANGE, 5394 },
	{ 0xe0f1, 0xe0f1, PDF_CMAP_SINGLE, 6001 },
	{ 0xe0f2, 0xe0fe, PDF_CMAP_RANGE, 5474 },
	{ 0xe1a1, 0xe1ac, PDF_CMAP_RANGE, 5487 },
	{ 0xe1ad, 0xe1ad, PDF_CMAP_SINGLE, 5460 },
	{ 0xe1ae, 0xe1ec, PDF_CMAP_RANGE, 5499 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 5317 },
	{ 0xe1ee, 0xe1fe, PDF_CMAP_RANGE, 5562 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5579 },
	{ 0xe3a1, 0xe3f4, PDF_CMAP_RANGE, 5673 },
	{ 0xe3f5, 0xe3f5, PDF_CMAP_SINGLE, 7009 },
	{ 0xe3f6, 0xe3fe, PDF_CMAP_RANGE, 5757 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 7370 },
	{ 0xe4a2, 0xe4a8, PDF_CMAP_RANGE, 5766 },
	{ 0xe4a9, 0xe4a9, PDF_CMAP_SINGLE, 5678 },
	{ 0xe4aa, 0xe4fe, PDF_CMAP_RANGE, 5773 },
	{ 0xe5a1, 0xe5ad, PDF_CMAP_RANGE, 5858 },
	{ 0xe5ae, 0xe5ae, PDF_CMAP_SINGLE, 5874 },
	{ 0xe5af, 0xe5b0, PDF_CMAP_RANGE, 5871 },
	{ 0xe5b1, 0xe5b2, PDF_CMAP_RANGE, 4425 },
	{ 0xe5b3, 0xe5b8, PDF_CMAP_RANGE, 5873 },
	{ 0xe5b9, 0xe5ba, PDF_CMAP_TABLE, 30 },
	{ 0xe5bb, 0xe5bc, PDF_CMAP_RANGE, 4429 },
	{ 0xe5bd, 0xe5c3, PDF_CMAP_RANGE, 5880 },
	{ 0xe5c4, 0xe5c4, PDF_CMAP_SINGLE, 4431 },
	{ 0xe5c5, 0xe5cd, PDF_CMAP_RANGE, 5887 },
	{ 0xe5ce, 0xe5d2, PDF_CMAP_TABLE, 32 },
	{ 0xe5d3, 0xe5d5, PDF_CMAP_RANGE, 5898 },
	{ 0xe5d6, 0xe5d6, PDF_CMAP_SINGLE, 4439 },
	{ 0xe5d7, 0xe5f9, PDF_CMAP_RANGE, 5901 },
	{ 0xe5fa, 0xe5fb, PDF_CMAP_RANGE, 4442 },
	{ 0xe5fc, 0xe5fe, PDF_CMAP_TABLE, 37 },
	{ 0xe6a1, 0xe6a1, PDF_CMAP_SINGLE, 4447 },
	{ 0xe6a2, 0xe6a3, PDF_CMAP_RANGE, 5937 },
	{ 0xe6a4, 0xe6a4, PDF_CMAP_SINGLE, 4449 },
	{ 0xe6a5, 0xe6a6, PDF_CMAP_RANGE, 5939 },
	{ 0xe6a7, 0xe6a7, PDF_CMAP_SINGLE, 4450 },
	{ 0xe6a8, 0xe6ac, PDF_CMAP_RANGE, 5941 },
	{ 0xe6ad, 0xe6ae, PDF_CMAP_TABLE, 40 },
	{ 0xe6af, 0xe6b1, PDF_CMAP_RANGE, 4455 },
	{ 0xe6b2, 0xe6b3, PDF_CMAP_TABLE, 42 },
	{ 0xe6b4, 0xe6b6, PDF_CMAP_RANGE, 5948 },
	{ 0xe6b7, 0xe6b8, PDF_CMAP_RANGE, 4459 },
	{ 0xe6b9, 0xe6bb, PDF_CMAP_RANGE, 5951 },
	{ 0xe6bc, 0xe6bc, PDF_CMAP_SINGLE, 4463 },
	{ 0xe6bd, 0xe6c3, PDF_CMAP_RANGE, 5954 },
	{ 0xe6c4, 0xe6c5, PDF_CMAP_TABLE, 44 },
	{ 0xe6c6, 0xe6c7, PDF_CMAP_RANGE, 4465 },
	{ 0xe6c8, 0xe6c9, PDF_CMAP_RANGE, 5962 },
	{ 0xe6ca, 0xe6ca, PDF_CMAP_SINGLE, 4161 },
	{ 0xe6cb, 0xe6d1, PDF_CMAP_RANGE, 5964 },
	{ 0xe6d2, 0xe6d2, PDF_CMAP_SINGLE, 4468 },
	{ 0xe6d3, 0xe6d5, PDF_CMAP_RANGE, 5971 },
	{ 0xe6d6, 0xe6d6, PDF_CMAP_SINGLE, 4469 },
	{ 0xe6d7, 0xe6d8, PDF_CMAP_RANGE, 5974 },
	{ 0xe6d9, 0xe6d9, PDF_CMAP_SINGLE, 4470 },
	{ 0xe6da, 0xe6db, PDF_CMAP_RANGE, 5976 },
	{ 0xe6dc, 0xe6dc, PDF_CMAP_SINGLE, 4162 },
	{ 0xe6dd, 0xe6de, PDF_CMAP_RANGE, 5978 },
	{ 0xe6df, 0xe6e1, PDF_CMAP_TABLE, 46 },
	{ 0xe6e2, 0xe6e3, PDF_CMAP_RANGE, 5981 },
	{ 0xe6e4, 0xe6e9, PDF_CMAP_TABLE, 49 },
	{ 0xe6ea, 0xe6eb, PDF_CMAP_RANGE, 4478 },
	{ 0xe6ec, 0xe6ec, PDF_CMAP_SINGLE, 6447 },
	{ 0xe6ed, 0xe6ee, PDF_CMAP_RANGE, 5985 },
	{ 0xe6ef, 0xe6f2, PDF_CMAP_TABLE, 55 },
	{ 0xe6f3, 0xe6f4, PDF_CMAP_RANGE, 5988 },
	{ 0xe6f5, 0xe6f9, PDF_CMAP_TABLE, 59 },
	{ 0xe6fa, 0xe6fe, PDF_CMAP_RANGE, 5991 },
	{ 0xe7a1, 0xe7a1, PDF_CMAP_SINGLE, 4487 },
	{ 0xe7a2, 0xe7a5, PDF_CMAP_RANGE, 5996 },
	{ 0xe7a6, 0xe7a6, PDF_CMAP_SINGLE, 4488 },
	{ 0xe7a7, 0xe7a8, PDF_CMAP_RANGE, 6000 },
	{ 0xe7a9, 0xe7ad, PDF_CMAP_TABLE, 64 },
	{ 0xe7ae, 0xe7af, PDF_CMAP_RANGE, 6003 },
	{ 0xe7b0, 0xe7b0, PDF_CMAP_SINGLE, 4494 },
	{ 0xe7b1, 0xe7be, PDF_CMAP_RANGE, 6005 },
	{ 0xe7bf, 0xe7c1, PDF_CMAP_TABLE, 69 },
	{ 0xe7c2, 0xe7c5, PDF_CMAP_RANGE, 6020 },
	{ 0xe7c6, 0xe7c7, PDF_CMAP_TABLE, 72 },
	{ 0xe7c8, 0xe7ca, PDF_CMAP_RANGE, 6024 },
	{ 0xe7cb, 0xe7ce, PDF_CMAP_TABLE, 74 },
	{ 0xe7cf, 0xe7d0, PDF_CMAP_RANGE, 4503 },
	{ 0xe7d1, 0xe7d2, PDF_CMAP_RANGE, 6029 },
	{ 0xe7d3, 0xe7d3, PDF_CMAP_SINGLE, 4506 },
	{ 0xe7d4, 0xe7de, PDF_CMAP_RANGE, 6031 },
	{ 0xe7df, 0xe7df, PDF_CMAP_SINGLE, 4508 },
	{ 0xe7e0, 0xe7e3, PDF_CMAP_RANGE, 6042 },
	{ 0xe7e4, 0xe7e6, PDF_CMAP_TABLE, 78 },
	{ 0xe7e7, 0xe7f6, PDF_CMAP_RANGE, 6047 },
	{ 0xe7f7, 0xe7f7, PDF_CMAP_SINGLE, 5797 },
	{ 0xe7f8, 0xe7fe, PDF_CMAP_RANGE, 6063 },
	{ 0xe8a1, 0xe8e6, PDF_CMAP_RANGE, 6070 },
	{ 0xe8e7, 0xe8e8, PDF_CMAP_RANGE, 4551 },
	{ 0xe8e9, 0xe8ef, PDF_CMAP_RANGE, 6140 },
	{ 0xe8f0, 0xe8f1, PDF_CMAP_TABLE, 81 },
	{ 0xe8f2, 0xe8f6, PDF_CMAP_RANGE, 6147 },
	{ 0xe8f7, 0xe8fb, PDF_CMAP_TABLE, 83 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 6154 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 4557 },
	{ 0xe9a1, 0xe9a6, PDF_CMAP_RANGE, 6156 },
	{ 0xe9a7, 0xe9a7, PDF_CMAP_SINGLE, 4560 },
	{ 0xe9a8, 0xe9ab, PDF_CMAP_RANGE, 6162 },
	{ 0xe9ac, 0xe9ac, PDF_CMAP_SINGLE, 4561 },
	{ 0xe9ad, 0xe9cb, PDF_CMAP_RANGE, 6166 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 4563 },
	{ 0xe9cd, 0xe9f6, PDF_CMAP_RANGE, 6197 },
	{ 0xe9f7, 0xe9f7, PDF_CMAP_SINGLE, 7988 },
	{ 0xe9f8, 0xe9fe, PDF_CMAP_RANGE, 6239 },
	{ 0xeaa1, 0xeac0, PDF_CMAP_RANGE, 6246 },
	{ 0xeac1, 0xeac1, PDF_CMAP_SINGLE, 6123 },
	{ 0xeac2, 0xeae4, PDF_CMAP_RANGE, 6278 },
	{ 0xeae5, 0xeae5, PDF_CMAP_SINGLE, 4577 },
	{ 0xeae6, 0xeaf3, PDF_CMAP_RANGE, 6313 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 4183 },
	{ 0xeaf5, 0xeaf6, PDF_CMAP_RANGE, 6327 },
	{ 0xeaf7, 0xeaf7, PDF_CMAP_SINGLE, 4579 },
	{ 0xeaf8, 0xeafb, PDF_CMAP_RANGE, 6329 },
	{ 0xeafc, 0xeafe, PDF_CMAP_TABLE, 88 },
	{ 0xeba1, 0xeba3, PDF_CMAP_RANGE, 6334 },
	{ 0xeba4, 0xeba4, PDF_CMAP_SINGLE, 4584 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 6337 },
	{ 0xeba7, 0xebaa, PDF_CMAP_TABLE, 91 },
	{ 0xebab, 0xebb9, PDF_CMAP_RANGE, 6340 },
	{ 0xebba, 0xebbb, PDF_CMAP_RANGE, 4590 },
	{ 0xebbc, 0xebbd, PDF_CMAP_TABLE, 95 },
	{ 0xebbe, 0xebc0, PDF_CMAP_RANGE, 6356 },
	{ 0xebc1, 0xebc2, PDF_CMAP_TABLE, 97 },
	{ 0xebc3, 0xebc5, PDF_CMAP_RANGE, 6359 },
	{ 0xebc6, 0xebc7, PDF_CMAP_RANGE, 4596 },
	{ 0xebc8, 0xebcb, PDF_CMAP_RANGE, 6362 },
	{ 0xebcc, 0xebcc, PDF_CMAP_SINGLE, 4599 },
	{ 0xebcd, 0xebce, PDF_CMAP_RANGE, 6366 },
	{ 0xebcf, 0xebd1, PDF_CMAP_RANGE, 4600 },
	{ 0xebd2, 0xebd2, PDF_CMAP_SINGLE, 5552 },
	{ 0xebd3, 0xebd7, PDF_CMAP_RANGE, 6368 },
	{ 0xebd8, 0xebd8, PDF_CMAP_SINGLE, 4603 },
	{ 0xebd9, 0xebfe, PDF_CMAP_RANGE, 6373 },
	{ 0xeca1, 0xeca5, PDF_CMAP_RANGE, 6411 },
	{ 0xeca6, 0xeca7, PDF_CMAP_TABLE, 99 },
	{ 0xeca8, 0xeca9, PDF_CMAP_RANGE, 6416 },
	{ 0xecaa, 0xecaa, PDF_CMAP_SINGLE, 4618 },
	{ 0xecab, 0xecae, PDF_CMAP_RANGE, 6418 },
	{ 0xecaf, 0xecaf, PDF_CMAP_SINGLE, 5950 },
	{ 0xecb0, 0xecb1, PDF_CMAP_RANGE, 4620 },
	{ 0xecb2, 0xecb2, PDF_CMAP_SINGLE, 4187 },
	{ 0xecb3, 0xecb4, PDF_CMAP_RANGE, 6422 },
	{ 0xecb5, 0xecb5, PDF_CMAP_SINGLE, 4625 },
	{ 0xecb6, 0xecb7, PDF_CMAP_RANGE, 6424 },
	{ 0xecb8, 0xecba, PDF_CMAP_TABLE, 101 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc6, PDF_CMAP_TABLE, 104 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecfa, PDF_CMAP_TABLE, 106 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f6, PDF_CMAP_TABLE, 108 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa2, PDF_CMAP_TABLE, 111 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc5, PDF_CMAP_SINGLE, 4634 },
	{ 0xecc6, 0xecc6, PDF_CMAP_SINGLE, 4636 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecf9, PDF_CMAP_SINGLE, 6474 },
	{ 0xecfa, 0xecfa, PDF_CMAP_SINGLE, 4650 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f4, PDF_CMAP_SINGLE, 4279 },
	{ 0xf6f5, 0xf6f5, PDF_CMAP_SINGLE, 7398 },
	{ 0xf6f6, 0xf6f6, PDF_CMAP_SINGLE, 7139 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa1, PDF_CMAP_SINGLE, 7709 },
	{ 0xfaa2, 0xfaa2, PDF_CMAP_SINGLE, 3558 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
};

static const unsigned short pdf_cmap_KSC_EUC_H_table[113] =
{
	3993,3946,4374,4156,4138,5800,4386,4139,
	4140,4391,4141,4142,4396,4403,4149,4419,
	4154,4420,4517,4173,4524,4174,4179,4547,
	4550,4180,4564,5584,5302,5367,4427,5879,
	4435,5896,4436,5897,4437,4159,5936,4444,
	4453,5946,5947,4458,4160,5961,4471,5980,
	4472,4474,4473,4475,5983,4476,5984,4481,
	5987,4482,5460,4483,4163,4166,5990,4485,
	4489,4491,6002,4167,4493,4495,6019,7783,
	4497,4499,4501,6027,4502,6028,4509,6046,
	4510,4553,4180,4555,6152,5800,6153,4556,
	4581,6333,4582,4586,6339,4588,4184,6355,
	4592,4593,4595,4614,4616,4627,6426,4629,
	4634,4636,6474,4650,4279,7398,7139,7709,
	3558,
};

pdf_cmap pdf_cmap_KSC_EUC_H =
{
	-1, "KSC-EUC-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0xa1a1, 0xfefe },
	},
	467, 467, (pdf_range*) pdf_cmap_KSC_EUC_H_ranges,
	113, 113, (unsigned short*) pdf_cmap_KSC_EUC_H_table,
};

/* KSC-EUC-V */

static const pdf_range pdf_cmap_KSC_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 8056 },
	{ 0xa1a5, 0xa1a6, PDF_CMAP_TABLE, 0 },
	{ 0xa1a9, 0xa1ab, PDF_CMAP_RANGE, 8059 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 8062 },
	{ 0xa1b2, 0xa1bd, PDF_CMAP_RANGE, 8063 },
	{ 0xa1eb, 0xa1eb, PDF_CMAP_SINGLE, 8075 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 8076 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 8077 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 8079 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 8080 },
	{ 0xa3ba, 0xa3bf, PDF_CMAP_RANGE, 8081 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 8087 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 8088 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 8089 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
};

static const unsigned short pdf_cmap_KSC_EUC_V_table[2] =
{
	8058,8320,
};

pdf_cmap pdf_cmap_KSC_EUC_V =
{
	-1, "KSC-EUC-V", "KSC-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	16, 16, (pdf_range*) pdf_cmap_KSC_EUC_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_KSC_EUC_V_table,
};

/* KSC-H */

static const pdf_range pdf_cmap_KSC_H_ranges[] =
{
	{ 0x2121, 0x217e, PDF_CMAP_RANGE, 101 },
	{ 0x2221, 0x2265, PDF_CMAP_RANGE, 195 },
	{ 0x2321, 0x237e, PDF_CMAP_RANGE, 264 },
	{ 0x2421, 0x2453, PDF_CMAP_RANGE, 358 },
	{ 0x2455, 0x247e, PDF_CMAP_RANGE, 409 },
	{ 0x2521, 0x252a, PDF_CMAP_RANGE, 451 },
	{ 0x2530, 0x2539, PDF_CMAP_RANGE, 461 },
	{ 0x2541, 0x2558, PDF_CMAP_RANGE, 471 },
	{ 0x2561, 0x2578, PDF_CMAP_RANGE, 495 },
	{ 0x2621, 0x2664, PDF_CMAP_RANGE, 519 },
	{ 0x2721, 0x276f, PDF_CMAP_RANGE, 587 },
	{ 0x2821, 0x2824, PDF_CMAP_RANGE, 666 },
	{ 0x2826, 0x2826, PDF_CMAP_SINGLE, 670 },
	{ 0x2828, 0x282f, PDF_CMAP_RANGE, 671 },
	{ 0x2831, 0x287e, PDF_CMAP_RANGE, 679 },
	{ 0x2921, 0x297e, PDF_CMAP_RANGE, 757 },
	{ 0x2a21, 0x2a73, PDF_CMAP_RANGE, 851 },
	{ 0x2b21, 0x2b76, PDF_CMAP_RANGE, 934 },
	{ 0x2c21, 0x2c41, PDF_CMAP_RANGE, 1020 },
	{ 0x2c51, 0x2c71, PDF_CMAP_RANGE, 1053 },
	{ 0x3021, 0x307e, PDF_CMAP_RANGE, 1086 },
	{ 0x3121, 0x317e, PDF_CMAP_RANGE, 1180 },
	{ 0x3221, 0x327e, PDF_CMAP_RANGE, 1274 },
	{ 0x3321, 0x337e, PDF_CMAP_RANGE, 1368 },
	{ 0x3421, 0x347e, PDF_CMAP_RANGE, 1462 },
	{ 0x3521, 0x357e, PDF_CMAP_RANGE, 1556 },
	{ 0x3621, 0x367e, PDF_CMAP_RANGE, 1650 },
	{ 0x3721, 0x377e, PDF_CMAP_RANGE, 1744 },
	{ 0x3821, 0x387e, PDF_CMAP_RANGE, 1838 },
	{ 0x3921, 0x397e, PDF_CMAP_RANGE, 1932 },
	{ 0x3a21, 0x3a7e, PDF_CMAP_RANGE, 2026 },
	{ 0x3b21, 0x3b7e, PDF_CMAP_RANGE, 2120 },
	{ 0x3c21, 0x3c7e, PDF_CMAP_RANGE, 2214 },
	{ 0x3d21, 0x3d7e, PDF_CMAP_RANGE, 2308 },
	{ 0x3e21, 0x3e7e, PDF_CMAP_RANGE, 2402 },
	{ 0x3f21, 0x3f7e, PDF_CMAP_RANGE, 2496 },
	{ 0x4021, 0x407e, PDF_CMAP_RANGE, 2590 },
	{ 0x4121, 0x417e, PDF_CMAP_RANGE, 2684 },
	{ 0x4221, 0x427e, PDF_CMAP_RANGE, 2778 },
	{ 0x4321, 0x437e, PDF_CMAP_RANGE, 2872 },
	{ 0x4421, 0x447e, PDF_CMAP_RANGE, 2966 },
	{ 0x4521, 0x457e, PDF_CMAP_RANGE, 3060 },
	{ 0x4621, 0x467e, PDF_CMAP_RANGE, 3154 },
	{ 0x4721, 0x477e, PDF_CMAP_RANGE, 3248 },
	{ 0x4821, 0x487e, PDF_CMAP_RANGE, 3342 },
	{ 0x4a21, 0x4a7e, PDF_CMAP_RANGE, 3436 },
	{ 0x4b21, 0x4b4f, PDF_CMAP_RANGE, 3530 },
	{ 0x4b50, 0x4b50, PDF_CMAP_SINGLE, 4116 },
	{ 0x4b51, 0x4b55, PDF_CMAP_RANGE, 3577 },
	{ 0x4b56, 0x4b56, PDF_CMAP_SINGLE, 3678 },
	{ 0x4b57, 0x4b66, PDF_CMAP_RANGE, 3582 },
	{ 0x4b67, 0x4b67, PDF_CMAP_SINGLE, 7053 },
	{ 0x4b68, 0x4b7e, PDF_CMAP_RANGE, 3598 },
	{ 0x4c21, 0x4c7e, PDF_CMAP_RANGE, 3621 },
	{ 0x4d21, 0x4d4e, PDF_CMAP_RANGE, 3715 },
	{ 0x4d4f, 0x4d4f, PDF_CMAP_SINGLE, 3460 },
	{ 0x4d50, 0x4d67, PDF_CMAP_RANGE, 3761 },
	{ 0x4d68, 0x4d68, PDF_CMAP_SINGLE, 7900 },
	{ 0x4d69, 0x4d7e, PDF_CMAP_RANGE, 3785 },
	{ 0x4e21, 0x4e2c, PDF_CMAP_RANGE, 3807 },
	{ 0x4e2d, 0x4e2d, PDF_CMAP_SINGLE, 3802 },
	{ 0x4e2e, 0x4e7e, PDF_CMAP_RANGE, 3819 },
	{ 0x4f21, 0x4f7a, PDF_CMAP_RANGE, 3900 },
	{ 0x4f7b, 0x4f7b, PDF_CMAP_SINGLE, 3902 },
	{ 0x4f7c, 0x4f7e, PDF_CMAP_RANGE, 3990 },
	{ 0x5021, 0x5022, PDF_CMAP_TABLE, 0 },
	{ 0x5023, 0x5037, PDF_CMAP_RANGE, 3994 },
	{ 0x5038, 0x5038, PDF_CMAP_SINGLE, 3946 },
	{ 0x5039, 0x504f, PDF_CMAP_RANGE, 4015 },
	{ 0x5050, 0x5050, PDF_CMAP_SINGLE, 3708 },
	{ 0x5051, 0x505c, PDF_CMAP_RANGE, 4038 },
	{ 0x505d, 0x505d, PDF_CMAP_SINGLE, 4131 },
	{ 0x505e, 0x507e, PDF_CMAP_RANGE, 4050 },
	{ 0x5121, 0x5153, PDF_CMAP_RANGE, 4083 },
	{ 0x5154, 0x5155, PDF_CMAP_TABLE, 2 },
	{ 0x5156, 0x5157, PDF_CMAP_RANGE, 4134 },
	{ 0x5158, 0x5158, PDF_CMAP_SINGLE, 4375 },
	{ 0x5159, 0x515a, PDF_CMAP_RANGE, 4136 },
	{ 0x515b, 0x5160, PDF_CMAP_RANGE, 4376 },
	{ 0x5161, 0x5162, PDF_CMAP_TABLE, 4 },
	{ 0x5163, 0x5165, PDF_CMAP_RANGE, 4382 },
	{ 0x5166, 0x5167, PDF_CMAP_TABLE, 6 },
	{ 0x5168, 0x516b, PDF_CMAP_RANGE, 4387 },
	{ 0x516c, 0x516e, PDF_CMAP_TABLE, 8 },
	{ 0x516f, 0x5170, PDF_CMAP_RANGE, 4394 },
	{ 0x5171, 0x5172, PDF_CMAP_TABLE, 11 },
	{ 0x5173, 0x5175, PDF_CMAP_RANGE, 4143 },
	{ 0x5176, 0x5176, PDF_CMAP_SINGLE, 4399 },
	{ 0x5177, 0x5179, PDF_CMAP_RANGE, 4146 },
	{ 0x517a, 0x517b, PDF_CMAP_TABLE, 13 },
	{ 0x517c, 0x517d, PDF_CMAP_RANGE, 4406 },
	{ 0x517e, 0x517e, PDF_CMAP_SINGLE, 4409 },
	{ 0x5221, 0x5221, PDF_CMAP_SINGLE, 4150 },
	{ 0x5222, 0x5223, PDF_CMAP_RANGE, 4410 },
	{ 0x5224, 0x5226, PDF_CMAP_RANGE, 4151 },
	{ 0x5227, 0x522a, PDF_CMAP_RANGE, 4412 },
	{ 0x522b, 0x522d, PDF_CMAP_TABLE, 15 },
	{ 0x522e, 0x5231, PDF_CMAP_RANGE, 4155 },
	{ 0x5232, 0x5232, PDF_CMAP_SINGLE, 4424 },
	{ 0x5233, 0x523d, PDF_CMAP_RANGE, 4159 },
	{ 0x523e, 0x523e, PDF_CMAP_SINGLE, 4511 },
	{ 0x523f, 0x5241, PDF_CMAP_RANGE, 4170 },
	{ 0x5242, 0x5243, PDF_CMAP_RANGE, 4513 },
	{ 0x5244, 0x5245, PDF_CMAP_TABLE, 18 },
	{ 0x5246, 0x524a, PDF_CMAP_RANGE, 4518 },
	{ 0x524b, 0x524c, PDF_CMAP_TABLE, 20 },
	{ 0x524d, 0x524e, PDF_CMAP_RANGE, 4525 },
	{ 0x524f, 0x5254, PDF_CMAP_RANGE, 4528 },
	{ 0x5255, 0x5257, PDF_CMAP_RANGE, 4535 },
	{ 0x5258, 0x5258, PDF_CMAP_SINGLE, 4175 },
	{ 0x5259, 0x525a, PDF_CMAP_RANGE, 4541 },
	{ 0x525b, 0x525d, PDF_CMAP_RANGE, 4176 },
	{ 0x525e, 0x525f, PDF_CMAP_RANGE, 4545 },
	{ 0x5260, 0x5264, PDF_CMAP_TABLE, 22 },
	{ 0x5265, 0x5268, PDF_CMAP_RANGE, 4566 },
	{ 0x5269, 0x526a, PDF_CMAP_RANGE, 4571 },
	{ 0x526b, 0x526b, PDF_CMAP_SINGLE, 4576 },
	{ 0x526c, 0x526f, PDF_CMAP_RANGE, 4181 },
	{ 0x5270, 0x5273, PDF_CMAP_RANGE, 4604 },
	{ 0x5274, 0x5275, PDF_CMAP_RANGE, 4609 },
	{ 0x5276, 0x5276, PDF_CMAP_SINGLE, 4185 },
	{ 0x5277, 0x5278, PDF_CMAP_RANGE, 4611 },
	{ 0x5279, 0x527e, PDF_CMAP_RANGE, 4186 },
	{ 0x5321, 0x537e, PDF_CMAP_RANGE, 4192 },
	{ 0x5421, 0x5465, PDF_CMAP_RANGE, 4286 },
	{ 0x5466, 0x5466, PDF_CMAP_SINGLE, 4318 },
	{ 0x5467, 0x547b, PDF_CMAP_RANGE, 4355 },
	{ 0x547c, 0x547c, PDF_CMAP_SINGLE, 4136 },
	{ 0x547d, 0x547e, PDF_CMAP_RANGE, 4376 },
	{ 0x5521, 0x5524, PDF_CMAP_RANGE, 4378 },
	{ 0x5525, 0x5525, PDF_CMAP_SINGLE, 5800 },
	{ 0x5526, 0x552a, PDF_CMAP_RANGE, 4382 },
	{ 0x552b, 0x552b, PDF_CMAP_SINGLE, 4139 },
	{ 0x552c, 0x552d, PDF_CMAP_RANGE, 4387 },
	{ 0x552e, 0x552e, PDF_CMAP_SINGLE, 4192 },
	{ 0x552f, 0x557e, PDF_CMAP_RANGE, 4389 },
	{ 0x5621, 0x5637, PDF_CMAP_RANGE, 4469 },
	{ 0x5638, 0x5638, PDF_CMAP_SINGLE, 4167 },
	{ 0x5639, 0x564c, PDF_CMAP_RANGE, 4492 },
	{ 0x564d, 0x564d, PDF_CMAP_SINGLE, 4172 },
	{ 0x564e, 0x567e, PDF_CMAP_RANGE, 4512 },
	{ 0x5721, 0x574a, PDF_CMAP_RANGE, 4561 },
	{ 0x574b, 0x574b, PDF_CMAP_SINGLE, 5552 },
	{ 0x574c, 0x5763, PDF_CMAP_RANGE, 4603 },
	{ 0x5764, 0x5764, PDF_CMAP_SINGLE, 6424 },
	{ 0x5765, 0x577e, PDF_CMAP_RANGE, 4627 },
	{ 0x5821, 0x587e, PDF_CMAP_RANGE, 4653 },
	{ 0x5921, 0x597e, PDF_CMAP_RANGE, 4747 },
	{ 0x5a21, 0x5a7e, PDF_CMAP_RANGE, 4841 },
	{ 0x5b21, 0x5b44, PDF_CMAP_RANGE, 4935 },
	{ 0x5b45, 0x5b45, PDF_CMAP_SINGLE, 5151 },
	{ 0x5b46, 0x5b63, PDF_CMAP_RANGE, 4971 },
	{ 0x5b64, 0x5b64, PDF_CMAP_SINGLE, 4922 },
	{ 0x5b65, 0x5b7e, PDF_CMAP_RANGE, 5001 },
	{ 0x5c21, 0x5c24, PDF_CMAP_RANGE, 5027 },
	{ 0x5c25, 0x5c25, PDF_CMAP_SINGLE, 7518 },
	{ 0x5c26, 0x5c7e, PDF_CMAP_RANGE, 5031 },
	{ 0x5d21, 0x5d24, PDF_CMAP_RANGE, 5120 },
	{ 0x5d25, 0x5d25, PDF_CMAP_SINGLE, 5079 },
	{ 0x5d26, 0x5d54, PDF_CMAP_RANGE, 5124 },
	{ 0x5d55, 0x5d55, PDF_CMAP_SINGLE, 5109 },
	{ 0x5d56, 0x5d73, PDF_CMAP_RANGE, 5171 },
	{ 0x5d74, 0x5d74, PDF_CMAP_SINGLE, 7607 },
	{ 0x5d75, 0x5d7e, PDF_CMAP_RANGE, 5201 },
	{ 0x5e21, 0x5e7b, PDF_CMAP_RANGE, 5211 },
	{ 0x5e7c, 0x5e7e, PDF_CMAP_TABLE, 27 },
	{ 0x5f21, 0x5f32, PDF_CMAP_RANGE, 5303 },
	{ 0x5f33, 0x5f33, PDF_CMAP_SINGLE, 7083 },
	{ 0x5f34, 0x5f60, PDF_CMAP_RANGE, 5321 },
	{ 0x5f61, 0x5f61, PDF_CMAP_SINGLE, 5362 },
	{ 0x5f62, 0x5f67, PDF_CMAP_RANGE, 5366 },
	{ 0x5f68, 0x5f68, PDF_CMAP_SINGLE, 5485 },
	{ 0x5f69, 0x5f7e, PDF_CMAP_RANGE, 5372 },
	{ 0x6021, 0x6070, PDF_CMAP_RANGE, 5394 },
	{ 0x6071, 0x6071, PDF_CMAP_SINGLE, 6001 },
	{ 0x6072, 0x607e, PDF_CMAP_RANGE, 5474 },
	{ 0x6121, 0x612c, PDF_CMAP_RANGE, 5487 },
	{ 0x612d, 0x612d, PDF_CMAP_SINGLE, 5460 },
	{ 0x612e, 0x616c, PDF_CMAP_RANGE, 5499 },
	{ 0x616d, 0x616d, PDF_CMAP_SINGLE, 5317 },
	{ 0x616e, 0x617e, PDF_CMAP_RANGE, 5562 },
	{ 0x6221, 0x627e, PDF_CMAP_RANGE, 5579 },
	{ 0x6321, 0x6374, PDF_CMAP_RANGE, 5673 },
	{ 0x6375, 0x6375, PDF_CMAP_SINGLE, 7009 },
	{ 0x6376, 0x637e, PDF_CMAP_RANGE, 5757 },
	{ 0x6421, 0x6421, PDF_CMAP_SINGLE, 7370 },
	{ 0x6422, 0x6428, PDF_CMAP_RANGE, 5766 },
	{ 0x6429, 0x6429, PDF_CMAP_SINGLE, 5678 },
	{ 0x642a, 0x647e, PDF_CMAP_RANGE, 5773 },
	{ 0x6521, 0x652d, PDF_CMAP_RANGE, 5858 },
	{ 0x652e, 0x652e, PDF_CMAP_SINGLE, 5874 },
	{ 0x652f, 0x6530, PDF_CMAP_RANGE, 5871 },
	{ 0x6531, 0x6532, PDF_CMAP_RANGE, 4425 },
	{ 0x6533, 0x6538, PDF_CMAP_RANGE, 5873 },
	{ 0x6539, 0x653a, PDF_CMAP_TABLE, 30 },
	{ 0x653b, 0x653c, PDF_CMAP_RANGE, 4429 },
	{ 0x653d, 0x6543, PDF_CMAP_RANGE, 5880 },
	{ 0x6544, 0x6544, PDF_CMAP_SINGLE, 4431 },
	{ 0x6545, 0x654d, PDF_CMAP_RANGE, 5887 },
	{ 0x654e, 0x6552, PDF_CMAP_TABLE, 32 },
	{ 0x6553, 0x6555, PDF_CMAP_RANGE, 5898 },
	{ 0x6556, 0x6556, PDF_CMAP_SINGLE, 4439 },
	{ 0x6557, 0x6579, PDF_CMAP_RANGE, 5901 },
	{ 0x657a, 0x657b, PDF_CMAP_RANGE, 4442 },
	{ 0x657c, 0x657e, PDF_CMAP_TABLE, 37 },
	{ 0x6621, 0x6621, PDF_CMAP_SINGLE, 4447 },
	{ 0x6622, 0x6623, PDF_CMAP_RANGE, 5937 },
	{ 0x6624, 0x6624, PDF_CMAP_SINGLE, 4449 },
	{ 0x6625, 0x6626, PDF_CMAP_RANGE, 5939 },
	{ 0x6627, 0x6627, PDF_CMAP_SINGLE, 4450 },
	{ 0x6628, 0x662c, PDF_CMAP_RANGE, 5941 },
	{ 0x662d, 0x662e, PDF_CMAP_TABLE, 40 },
	{ 0x662f, 0x6631, PDF_CMAP_RANGE, 4455 },
	{ 0x6632, 0x6633, PDF_CMAP_TABLE, 42 },
	{ 0x6634, 0x6636, PDF_CMAP_RANGE, 5948 },
	{ 0x6637, 0x6638, PDF_CMAP_RANGE, 4459 },
	{ 0x6639, 0x663b, PDF_CMAP_RANGE, 5951 },
	{ 0x663c, 0x663c, PDF_CMAP_SINGLE, 4463 },
	{ 0x663d, 0x6643, PDF_CMAP_RANGE, 5954 },
	{ 0x6644, 0x6645, PDF_CMAP_TABLE, 44 },
	{ 0x6646, 0x6647, PDF_CMAP_RANGE, 4465 },
	{ 0x6648, 0x6649, PDF_CMAP_RANGE, 5962 },
	{ 0x664a, 0x664a, PDF_CMAP_SINGLE, 4161 },
	{ 0x664b, 0x6651, PDF_CMAP_RANGE, 5964 },
	{ 0x6652, 0x6652, PDF_CMAP_SINGLE, 4468 },
	{ 0x6653, 0x6655, PDF_CMAP_RANGE, 5971 },
	{ 0x6656, 0x6656, PDF_CMAP_SINGLE, 4469 },
	{ 0x6657, 0x6658, PDF_CMAP_RANGE, 5974 },
	{ 0x6659, 0x6659, PDF_CMAP_SINGLE, 4470 },
	{ 0x665a, 0x665b, PDF_CMAP_RANGE, 5976 },
	{ 0x665c, 0x665c, PDF_CMAP_SINGLE, 4162 },
	{ 0x665d, 0x665e, PDF_CMAP_RANGE, 5978 },
	{ 0x665f, 0x6661, PDF_CMAP_TABLE, 46 },
	{ 0x6662, 0x6663, PDF_CMAP_RANGE, 5981 },
	{ 0x6664, 0x6669, PDF_CMAP_TABLE, 49 },
	{ 0x666a, 0x666b, PDF_CMAP_RANGE, 4478 },
	{ 0x666c, 0x666c, PDF_CMAP_SINGLE, 6447 },
	{ 0x666d, 0x666e, PDF_CMAP_RANGE, 5985 },
	{ 0x666f, 0x6672, PDF_CMAP_TABLE, 55 },
	{ 0x6673, 0x6674, PDF_CMAP_RANGE, 5988 },
	{ 0x6675, 0x6679, PDF_CMAP_TABLE, 59 },
	{ 0x667a, 0x667e, PDF_CMAP_RANGE, 5991 },
	{ 0x6721, 0x6721, PDF_CMAP_SINGLE, 4487 },
	{ 0x6722, 0x6725, PDF_CMAP_RANGE, 5996 },
	{ 0x6726, 0x6726, PDF_CMAP_SINGLE, 4488 },
	{ 0x6727, 0x6728, PDF_CMAP_RANGE, 6000 },
	{ 0x6729, 0x672d, PDF_CMAP_TABLE, 64 },
	{ 0x672e, 0x672f, PDF_CMAP_RANGE, 6003 },
	{ 0x6730, 0x6730, PDF_CMAP_SINGLE, 4494 },
	{ 0x6731, 0x673e, PDF_CMAP_RANGE, 6005 },
	{ 0x673f, 0x6741, PDF_CMAP_TABLE, 69 },
	{ 0x6742, 0x6745, PDF_CMAP_RANGE, 6020 },
	{ 0x6746, 0x6747, PDF_CMAP_TABLE, 72 },
	{ 0x6748, 0x674a, PDF_CMAP_RANGE, 6024 },
	{ 0x674b, 0x674e, PDF_CMAP_TABLE, 74 },
	{ 0x674f, 0x6750, PDF_CMAP_RANGE, 4503 },
	{ 0x6751, 0x6752, PDF_CMAP_RANGE, 6029 },
	{ 0x6753, 0x6753, PDF_CMAP_SINGLE, 4506 },
	{ 0x6754, 0x675e, PDF_CMAP_RANGE, 6031 },
	{ 0x675f, 0x675f, PDF_CMAP_SINGLE, 4508 },
	{ 0x6760, 0x6763, PDF_CMAP_RANGE, 6042 },
	{ 0x6764, 0x6766, PDF_CMAP_TABLE, 78 },
	{ 0x6767, 0x6776, PDF_CMAP_RANGE, 6047 },
	{ 0x6777, 0x6777, PDF_CMAP_SINGLE, 5797 },
	{ 0x6778, 0x677e, PDF_CMAP_RANGE, 6063 },
	{ 0x6821, 0x6866, PDF_CMAP_RANGE, 6070 },
	{ 0x6867, 0x6868, PDF_CMAP_RANGE, 4551 },
	{ 0x6869, 0x686f, PDF_CMAP_RANGE, 6140 },
	{ 0x6870, 0x6871, PDF_CMAP_TABLE, 81 },
	{ 0x6872, 0x6876, PDF_CMAP_RANGE, 6147 },
	{ 0x6877, 0x687b, PDF_CMAP_TABLE, 83 },
	{ 0x687c, 0x687d, PDF_CMAP_RANGE, 6154 },
	{ 0x687e, 0x687e, PDF_CMAP_SINGLE, 4557 },
	{ 0x6921, 0x6926, PDF_CMAP_RANGE, 6156 },
	{ 0x6927, 0x6927, PDF_CMAP_SINGLE, 4560 },
	{ 0x6928, 0x692b, PDF_CMAP_RANGE, 6162 },
	{ 0x692c, 0x692c, PDF_CMAP_SINGLE, 4561 },
	{ 0x692d, 0x694b, PDF_CMAP_RANGE, 6166 },
	{ 0x694c, 0x694c, PDF_CMAP_SINGLE, 4563 },
	{ 0x694d, 0x6976, PDF_CMAP_RANGE, 6197 },
	{ 0x6977, 0x6977, PDF_CMAP_SINGLE, 7988 },
	{ 0x6978, 0x697e, PDF_CMAP_RANGE, 6239 },
	{ 0x6a21, 0x6a40, PDF_CMAP_RANGE, 6246 },
	{ 0x6a41, 0x6a41, PDF_CMAP_SINGLE, 6123 },
	{ 0x6a42, 0x6a64, PDF_CMAP_RANGE, 6278 },
	{ 0x6a65, 0x6a65, PDF_CMAP_SINGLE, 4577 },
	{ 0x6a66, 0x6a73, PDF_CMAP_RANGE, 6313 },
	{ 0x6a74, 0x6a74, PDF_CMAP_SINGLE, 4183 },
	{ 0x6a75, 0x6a76, PDF_CMAP_RANGE, 6327 },
	{ 0x6a77, 0x6a77, PDF_CMAP_SINGLE, 4579 },
	{ 0x6a78, 0x6a7b, PDF_CMAP_RANGE, 6329 },
	{ 0x6a7c, 0x6a7e, PDF_CMAP_TABLE, 88 },
	{ 0x6b21, 0x6b23, PDF_CMAP_RANGE, 6334 },
	{ 0x6b24, 0x6b24, PDF_CMAP_SINGLE, 4584 },
	{ 0x6b25, 0x6b26, PDF_CMAP_RANGE, 6337 },
	{ 0x6b27, 0x6b2a, PDF_CMAP_TABLE, 91 },
	{ 0x6b2b, 0x6b39, PDF_CMAP_RANGE, 6340 },
	{ 0x6b3a, 0x6b3b, PDF_CMAP_RANGE, 4590 },
	{ 0x6b3c, 0x6b3d, PDF_CMAP_TABLE, 95 },
	{ 0x6b3e, 0x6b40, PDF_CMAP_RANGE, 6356 },
	{ 0x6b41, 0x6b42, PDF_CMAP_TABLE, 97 },
	{ 0x6b43, 0x6b45, PDF_CMAP_RANGE, 6359 },
	{ 0x6b46, 0x6b47, PDF_CMAP_RANGE, 4596 },
	{ 0x6b48, 0x6b4b, PDF_CMAP_RANGE, 6362 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 4599 },
	{ 0x6b4d, 0x6b4e, PDF_CMAP_RANGE, 6366 },
	{ 0x6b4f, 0x6b51, PDF_CMAP_RANGE, 4600 },
	{ 0x6b52, 0x6b52, PDF_CMAP_SINGLE, 5552 },
	{ 0x6b53, 0x6b57, PDF_CMAP_RANGE, 6368 },
	{ 0x6b58, 0x6b58, PDF_CMAP_SINGLE, 4603 },
	{ 0x6b59, 0x6b7e, PDF_CMAP_RANGE, 6373 },
	{ 0x6c21, 0x6c25, PDF_CMAP_RANGE, 6411 },
	{ 0x6c26, 0x6c27, PDF_CMAP_TABLE, 99 },
	{ 0x6c28, 0x6c29, PDF_CMAP_RANGE, 6416 },
	{ 0x6c2a, 0x6c2a, PDF_CMAP_SINGLE, 4618 },
	{ 0x6c2b, 0x6c2e, PDF_CMAP_RANGE, 6418 },
	{ 0x6c2f, 0x6c2f, PDF_CMAP_SINGLE, 5950 },
	{ 0x6c30, 0x6c31, PDF_CMAP_RANGE, 4620 },
	{ 0x6c32, 0x6c32, PDF_CMAP_SINGLE, 4187 },
	{ 0x6c33, 0x6c34, PDF_CMAP_RANGE, 6422 },
	{ 0x6c35, 0x6c35, PDF_CMAP_SINGLE, 4625 },
	{ 0x6c36, 0x6c37, PDF_CMAP_RANGE, 6424 },
	{ 0x6c38, 0x6c3a, PDF_CMAP_TABLE, 101 },
	{ 0x6c3b, 0x6c3f, PDF_CMAP_RANGE, 6427 },
	{ 0x6c40, 0x6c41, PDF_CMAP_RANGE, 4632 },
	{ 0x6c42, 0x6c44, PDF_CMAP_RANGE, 6432 },
	{ 0x6c45, 0x6c46, PDF_CMAP_TABLE, 104 },
	{ 0x6c47, 0x6c48, PDF_CMAP_RANGE, 6435 },
	{ 0x6c49, 0x6c4a, PDF_CMAP_RANGE, 4188 },
	{ 0x6c4b, 0x6c54, PDF_CMAP_RANGE, 6437 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 4638 },
	{ 0x6c56, 0x6c5c, PDF_CMAP_RANGE, 6447 },
	{ 0x6c5d, 0x6c5e, PDF_CMAP_RANGE, 4640 },
	{ 0x6c5f, 0x6c60, PDF_CMAP_RANGE, 6454 },
	{ 0x6c61, 0x6c61, PDF_CMAP_SINGLE, 4642 },
	{ 0x6c62, 0x6c63, PDF_CMAP_RANGE, 6456 },
	{ 0x6c64, 0x6c64, PDF_CMAP_SINGLE, 4644 },
	{ 0x6c65, 0x6c66, PDF_CMAP_RANGE, 6458 },
	{ 0x6c67, 0x6c68, PDF_CMAP_RANGE, 4645 },
	{ 0x6c69, 0x6c76, PDF_CMAP_RANGE, 6460 },
	{ 0x6c77, 0x6c78, PDF_CMAP_RANGE, 4647 },
	{ 0x6c79, 0x6c7a, PDF_CMAP_TABLE, 106 },
	{ 0x6c7b, 0x6c7e, PDF_CMAP_RANGE, 6475 },
	{ 0x6d21, 0x6d23, PDF_CMAP_RANGE, 4653 },
	{ 0x6d24, 0x6d6d, PDF_CMAP_RANGE, 6479 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 5351 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6553 },
	{ 0x6e21, 0x6e5a, PDF_CMAP_RANGE, 6569 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 6494 },
	{ 0x6e5c, 0x6e7e, PDF_CMAP_RANGE, 6627 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 6662 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 6756 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 6850 },
	{ 0x7221, 0x723c, PDF_CMAP_RANGE, 6944 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 5731 },
	{ 0x723e, 0x7279, PDF_CMAP_RANGE, 6972 },
	{ 0x727a, 0x727a, PDF_CMAP_SINGLE, 5771 },
	{ 0x727b, 0x727e, PDF_CMAP_RANGE, 7032 },
	{ 0x7321, 0x7330, PDF_CMAP_RANGE, 7036 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 4191 },
	{ 0x7332, 0x737e, PDF_CMAP_RANGE, 7052 },
	{ 0x7421, 0x7426, PDF_CMAP_RANGE, 7129 },
	{ 0x7427, 0x7427, PDF_CMAP_SINGLE, 6484 },
	{ 0x7428, 0x746d, PDF_CMAP_RANGE, 7135 },
	{ 0x746e, 0x746e, PDF_CMAP_SINGLE, 6684 },
	{ 0x746f, 0x747e, PDF_CMAP_RANGE, 7205 },
	{ 0x7521, 0x757e, PDF_CMAP_RANGE, 7221 },
	{ 0x7621, 0x7673, PDF_CMAP_RANGE, 7315 },
	{ 0x7674, 0x7676, PDF_CMAP_TABLE, 108 },
	{ 0x7677, 0x767e, PDF_CMAP_RANGE, 7399 },
	{ 0x7721, 0x7737, PDF_CMAP_RANGE, 7407 },
	{ 0x7738, 0x7738, PDF_CMAP_SINGLE, 4247 },
	{ 0x7739, 0x7747, PDF_CMAP_RANGE, 7430 },
	{ 0x7748, 0x7748, PDF_CMAP_SINGLE, 4266 },
	{ 0x7749, 0x7752, PDF_CMAP_RANGE, 7445 },
	{ 0x7753, 0x7753, PDF_CMAP_SINGLE, 4340 },
	{ 0x7754, 0x777e, PDF_CMAP_RANGE, 7455 },
	{ 0x7821, 0x785a, PDF_CMAP_RANGE, 7498 },
	{ 0x785b, 0x785b, PDF_CMAP_SINGLE, 7572 },
	{ 0x785c, 0x786f, PDF_CMAP_RANGE, 7556 },
	{ 0x7870, 0x7870, PDF_CMAP_SINGLE, 5088 },
	{ 0x7871, 0x787e, PDF_CMAP_RANGE, 7576 },
	{ 0x7921, 0x797e, PDF_CMAP_RANGE, 7590 },
	{ 0x7a21, 0x7a22, PDF_CMAP_TABLE, 111 },
	{ 0x7a23, 0x7a65, PDF_CMAP_RANGE, 7684 },
	{ 0x7a66, 0x7a66, PDF_CMAP_SINGLE, 3644 },
	{ 0x7a67, 0x7a7e, PDF_CMAP_RANGE, 7751 },
	{ 0x7b21, 0x7b7e, PDF_CMAP_RANGE, 7775 },
	{ 0x7c21, 0x7c28, PDF_CMAP_RANGE, 7869 },
	{ 0x7c29, 0x7c29, PDF_CMAP_SINGLE, 3815 },
	{ 0x7c2a, 0x7c7e, PDF_CMAP_RANGE, 7877 },
	{ 0x7d21, 0x7d7e, PDF_CMAP_RANGE, 7962 },
	{ 0x6c3b, 0x6c3f, PDF_CMAP_RANGE, 6427 },
	{ 0x6c40, 0x6c41, PDF_CMAP_RANGE, 4632 },
	{ 0x6c42, 0x6c44, PDF_CMAP_RANGE, 6432 },
	{ 0x6c45, 0x6c45, PDF_CMAP_SINGLE, 4634 },
	{ 0x6c46, 0x6c46, PDF_CMAP_SINGLE, 4636 },
	{ 0x6c47, 0x6c48, PDF_CMAP_RANGE, 6435 },
	{ 0x6c49, 0x6c4a, PDF_CMAP_RANGE, 4188 },
	{ 0x6c4b, 0x6c54, PDF_CMAP_RANGE, 6437 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 4638 },
	{ 0x6c56, 0x6c5c, PDF_CMAP_RANGE, 6447 },
	{ 0x6c5d, 0x6c5e, PDF_CMAP_RANGE, 4640 },
	{ 0x6c5f, 0x6c60, PDF_CMAP_RANGE, 6454 },
	{ 0x6c61, 0x6c61, PDF_CMAP_SINGLE, 4642 },
	{ 0x6c62, 0x6c63, PDF_CMAP_RANGE, 6456 },
	{ 0x6c64, 0x6c64, PDF_CMAP_SINGLE, 4644 },
	{ 0x6c65, 0x6c66, PDF_CMAP_RANGE, 6458 },
	{ 0x6c67, 0x6c68, PDF_CMAP_RANGE, 4645 },
	{ 0x6c69, 0x6c76, PDF_CMAP_RANGE, 6460 },
	{ 0x6c77, 0x6c78, PDF_CMAP_RANGE, 4647 },
	{ 0x6c79, 0x6c79, PDF_CMAP_SINGLE, 6474 },
	{ 0x6c7a, 0x6c7a, PDF_CMAP_SINGLE, 4650 },
	{ 0x6c7b, 0x6c7e, PDF_CMAP_RANGE, 6475 },
	{ 0x6d21, 0x6d23, PDF_CMAP_RANGE, 4653 },
	{ 0x6d24, 0x6d6d, PDF_CMAP_RANGE, 6479 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 5351 },
	{ 0x6d6f, 0x6d7e, PDF_CMAP_RANGE, 6553 },
	{ 0x6e21, 0x6e5a, PDF_CMAP_RANGE, 6569 },
	{ 0x6e5b, 0x6e5b, PDF_CMAP_SINGLE, 6494 },
	{ 0x6e5c, 0x6e7e, PDF_CMAP_RANGE, 6627 },
	{ 0x6f21, 0x6f7e, PDF_CMAP_RANGE, 6662 },
	{ 0x7021, 0x707e, PDF_CMAP_RANGE, 6756 },
	{ 0x7121, 0x717e, PDF_CMAP_RANGE, 6850 },
	{ 0x7221, 0x723c, PDF_CMAP_RANGE, 6944 },
	{ 0x723d, 0x723d, PDF_CMAP_SINGLE, 5731 },
	{ 0x723e, 0x7279, PDF_CMAP_RANGE, 6972 },
	{ 0x727a, 0x727a, PDF_CMAP_SINGLE, 5771 },
	{ 0x727b, 0x727e, PDF_CMAP_RANGE, 7032 },
	{ 0x7321, 0x7330, PDF_CMAP_RANGE, 7036 },
	{ 0x7331, 0x7331, PDF_CMAP_SINGLE, 4191 },
	{ 0x7332, 0x737e, PDF_CMAP_RANGE, 7052 },
	{ 0x7421, 0x7426, PDF_CMAP_RANGE, 7129 },
	{ 0x7427, 0x7427, PDF_CMAP_SINGLE, 6484 },
	{ 0x7428, 0x746d, PDF_CMAP_RANGE, 7135 },
	{ 0x746e, 0x746e, PDF_CMAP_SINGLE, 6684 },
	{ 0x746f, 0x747e, PDF_CMAP_RANGE, 7205 },
	{ 0x7521, 0x757e, PDF_CMAP_RANGE, 7221 },
	{ 0x7621, 0x7673, PDF_CMAP_RANGE, 7315 },
	{ 0x7674, 0x7674, PDF_CMAP_SINGLE, 4279 },
	{ 0x7675, 0x7675, PDF_CMAP_SINGLE, 7398 },
	{ 0x7676, 0x7676, PDF_CMAP_SINGLE, 7139 },
	{ 0x7677, 0x767e, PDF_CMAP_RANGE, 7399 },
	{ 0x7721, 0x7737, PDF_CMAP_RANGE, 7407 },
	{ 0x7738, 0x7738, PDF_CMAP_SINGLE, 4247 },
	{ 0x7739, 0x7747, PDF_CMAP_RANGE, 7430 },
	{ 0x7748, 0x7748, PDF_CMAP_SINGLE, 4266 },
	{ 0x7749, 0x7752, PDF_CMAP_RANGE, 7445 },
	{ 0x7753, 0x7753, PDF_CMAP_SINGLE, 4340 },
	{ 0x7754, 0x777e, PDF_CMAP_RANGE, 7455 },
	{ 0x7821, 0x785a, PDF_CMAP_RANGE, 7498 },
	{ 0x785b, 0x785b, PDF_CMAP_SINGLE, 7572 },
	{ 0x785c, 0x786f, PDF_CMAP_RANGE, 7556 },
	{ 0x7870, 0x7870, PDF_CMAP_SINGLE, 5088 },
	{ 0x7871, 0x787e, PDF_CMAP_RANGE, 7576 },
	{ 0x7921, 0x797e, PDF_CMAP_RANGE, 7590 },
	{ 0x7a21, 0x7a21, PDF_CMAP_SINGLE, 7709 },
	{ 0x7a22, 0x7a22, PDF_CMAP_SINGLE, 3558 },
	{ 0x7a23, 0x7a65, PDF_CMAP_RANGE, 7684 },
	{ 0x7a66, 0x7a66, PDF_CMAP_SINGLE, 3644 },
	{ 0x7a67, 0x7a7e, PDF_CMAP_RANGE, 7751 },
	{ 0x7b21, 0x7b7e, PDF_CMAP_RANGE, 7775 },
	{ 0x7c21, 0x7c28, PDF_CMAP_RANGE, 7869 },
	{ 0x7c29, 0x7c29, PDF_CMAP_SINGLE, 3815 },
	{ 0x7c2a, 0x7c7e, PDF_CMAP_RANGE, 7877 },
	{ 0x7d21, 0x7d7e, PDF_CMAP_RANGE, 7962 },
};

static const unsigned short pdf_cmap_KSC_H_table[113] =
{
	3993,3946,4374,4156,4138,5800,4386,4139,
	4140,4391,4141,4142,4396,4403,4149,4419,
	4154,4420,4517,4173,4524,4174,4179,4547,
	4550,4180,4564,5584,5302,5367,4427,5879,
	4435,5896,4436,5897,4437,4159,5936,4444,
	4453,5946,5947,4458,4160,5961,4471,5980,
	4472,4474,4473,4475,5983,4476,5984,4481,
	5987,4482,5460,4483,4163,4166,5990,4485,
	4489,4491,6002,4167,4493,4495,6019,7783,
	4497,4499,4501,6027,4502,6028,4509,6046,
	4510,4553,4180,4555,6152,5800,6153,4556,
	4581,6333,4582,4586,6339,4588,4184,6355,
	4592,4593,4595,4614,4616,4627,6426,4629,
	4634,4636,6474,4650,4279,7398,7139,7709,
	3558,
};

pdf_cmap pdf_cmap_KSC_H =
{
	-1, "KSC-H", "", nil, 0,
	1, /* codespace table */
	{
		{ 2, 0x2121, 0x7e7e },
	},
	466, 466, (pdf_range*) pdf_cmap_KSC_H_ranges,
	113, 113, (unsigned short*) pdf_cmap_KSC_H_table,
};

/* KSC-Johab-H */

static const pdf_range pdf_cmap_KSC_Johab_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8442, 0x8448, PDF_CMAP_RANGE, 358 },
	{ 0x8449, 0x8451, PDF_CMAP_RANGE, 366 },
	{ 0x8453, 0x8453, PDF_CMAP_SINGLE, 375 },
	{ 0x8454, 0x8458, PDF_CMAP_RANGE, 377 },
	{ 0x8459, 0x845d, PDF_CMAP_RANGE, 383 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 388 },
	{ 0x8481, 0x8481, PDF_CMAP_SINGLE, 389 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 390 },
	{ 0x84c1, 0x84c1, PDF_CMAP_SINGLE, 391 },
	{ 0x84e1, 0x84e1, PDF_CMAP_SINGLE, 392 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 393 },
	{ 0x8561, 0x8561, PDF_CMAP_SINGLE, 394 },
	{ 0x8581, 0x8581, PDF_CMAP_SINGLE, 395 },
	{ 0x85a1, 0x85a1, PDF_CMAP_SINGLE, 396 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 397 },
	{ 0x85e1, 0x85e1, PDF_CMAP_SINGLE, 398 },
	{ 0x8641, 0x8641, PDF_CMAP_SINGLE, 399 },
	{ 0x8661, 0x8661, PDF_CMAP_SINGLE, 400 },
	{ 0x8681, 0x8681, PDF_CMAP_SINGLE, 401 },
	{ 0x86a1, 0x86a1, PDF_CMAP_SINGLE, 402 },
	{ 0x86c1, 0x86c1, PDF_CMAP_SINGLE, 403 },
	{ 0x86e1, 0x86e1, PDF_CMAP_SINGLE, 404 },
	{ 0x8741, 0x8741, PDF_CMAP_SINGLE, 405 },
	{ 0x8761, 0x8761, PDF_CMAP_SINGLE, 406 },
	{ 0x8781, 0x8781, PDF_CMAP_SINGLE, 407 },
	{ 0x87a1, 0x87a1, PDF_CMAP_SINGLE, 408 },
	{ 0x8841, 0x8841, PDF_CMAP_SINGLE, 358 },
	{ 0x8861, 0x8862, PDF_CMAP_RANGE, 1086 },
	{ 0x8863, 0x8864, PDF_CMAP_RANGE, 9333 },
	{ 0x8865, 0x8865, PDF_CMAP_SINGLE, 1088 },
	{ 0x8866, 0x8867, PDF_CMAP_RANGE, 9335 },
	{ 0x8868, 0x886b, PDF_CMAP_RANGE, 1089 },
	{ 0x886c, 0x8870, PDF_CMAP_RANGE, 9337 },
	{ 0x8871, 0x8871, PDF_CMAP_SINGLE, 1093 },
	{ 0x8873, 0x8879, PDF_CMAP_RANGE, 1094 },
	{ 0x887a, 0x887a, PDF_CMAP_SINGLE, 9342 },
	{ 0x887b, 0x887d, PDF_CMAP_RANGE, 1101 },
	{ 0x8881, 0x8882, PDF_CMAP_RANGE, 1104 },
	{ 0x8883, 0x8884, PDF_CMAP_RANGE, 9343 },
	{ 0x8885, 0x8885, PDF_CMAP_SINGLE, 1106 },
	{ 0x8886, 0x8888, PDF_CMAP_RANGE, 9345 },
	{ 0x8889, 0x8889, PDF_CMAP_SINGLE, 1107 },
	{ 0x888a, 0x8890, PDF_CMAP_RANGE, 9348 },
	{ 0x8891, 0x8891, PDF_CMAP_SINGLE, 1108 },
	{ 0x8893, 0x8894, PDF_CMAP_TABLE, 0 },
	{ 0x8895, 0x8897, PDF_CMAP_RANGE, 1110 },
	{ 0x8898, 0x889d, PDF_CMAP_RANGE, 9356 },
	{ 0x88a1, 0x88a2, PDF_CMAP_RANGE, 1113 },
	{ 0x88a3, 0x88a4, PDF_CMAP_RANGE, 9362 },
	{ 0x88a5, 0x88a5, PDF_CMAP_SINGLE, 1115 },
	{ 0x88a6, 0x88a8, PDF_CMAP_RANGE, 9364 },
	{ 0x88a9, 0x88a9, PDF_CMAP_SINGLE, 1116 },
	{ 0x88aa, 0x88b1, PDF_CMAP_RANGE, 9367 },
	{ 0x88b3, 0x88b4, PDF_CMAP_RANGE, 9375 },
	{ 0x88b5, 0x88b7, PDF_CMAP_TABLE, 2 },
	{ 0x88b8, 0x88bd, PDF_CMAP_RANGE, 9378 },
	{ 0x88c1, 0x88c1, PDF_CMAP_SINGLE, 1119 },
	{ 0x88c2, 0x88c4, PDF_CMAP_RANGE, 9384 },
	{ 0x88c5, 0x88c5, PDF_CMAP_SINGLE, 1120 },
	{ 0x88c6, 0x88c8, PDF_CMAP_RANGE, 9387 },
	{ 0x88c9, 0x88c9, PDF_CMAP_SINGLE, 1121 },
	{ 0x88ca, 0x88d1, PDF_CMAP_RANGE, 9390 },
	{ 0x88d3, 0x88dd, PDF_CMAP_RANGE, 9398 },
	{ 0x88e1, 0x88e2, PDF_CMAP_RANGE, 1122 },
	{ 0x88e3, 0x88e4, PDF_CMAP_RANGE, 9409 },
	{ 0x88e5, 0x88e5, PDF_CMAP_SINGLE, 1124 },
	{ 0x88e6, 0x88e7, PDF_CMAP_RANGE, 9411 },
	{ 0x88e8, 0x88e9, PDF_CMAP_RANGE, 1125 },
	{ 0x88ea, 0x88eb, PDF_CMAP_TABLE, 5 },
	{ 0x88ec, 0x88f0, PDF_CMAP_RANGE, 9414 },
	{ 0x88f1, 0x88f1, PDF_CMAP_SINGLE, 1128 },
	{ 0x88f3, 0x88f4, PDF_CMAP_TABLE, 7 },
	{ 0x88f5, 0x88f8, PDF_CMAP_RANGE, 1130 },
	{ 0x88f9, 0x88fa, PDF_CMAP_RANGE, 9420 },
	{ 0x88fb, 0x88fd, PDF_CMAP_RANGE, 1134 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 1137 },
	{ 0x8942, 0x8944, PDF_CMAP_RANGE, 9422 },
	{ 0x8945, 0x8945, PDF_CMAP_SINGLE, 1138 },
	{ 0x8946, 0x8948, PDF_CMAP_RANGE, 9425 },
	{ 0x8949, 0x8949, PDF_CMAP_SINGLE, 1139 },
	{ 0x894a, 0x8950, PDF_CMAP_RANGE, 9428 },
	{ 0x8951, 0x8951, PDF_CMAP_SINGLE, 1140 },
	{ 0x8953, 0x8954, PDF_CMAP_TABLE, 9 },
	{ 0x8955, 0x8957, PDF_CMAP_RANGE, 1142 },
	{ 0x8958, 0x895d, PDF_CMAP_RANGE, 9436 },
	{ 0x8961, 0x8963, PDF_CMAP_RANGE, 1145 },
	{ 0x8964, 0x8965, PDF_CMAP_TABLE, 11 },
	{ 0x8966, 0x8967, PDF_CMAP_RANGE, 9443 },
	{ 0x8968, 0x8969, PDF_CMAP_RANGE, 1149 },
	{ 0x896a, 0x8970, PDF_CMAP_RANGE, 9445 },
	{ 0x8971, 0x8971, PDF_CMAP_SINGLE, 1151 },
	{ 0x8973, 0x8974, PDF_CMAP_TABLE, 13 },
	{ 0x8975, 0x8977, PDF_CMAP_RANGE, 1153 },
	{ 0x8978, 0x897a, PDF_CMAP_RANGE, 9453 },
	{ 0x897b, 0x897b, PDF_CMAP_SINGLE, 1156 },
	{ 0x897c, 0x897d, PDF_CMAP_RANGE, 9456 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 1157 },
	{ 0x8982, 0x8984, PDF_CMAP_RANGE, 9458 },
	{ 0x8985, 0x8985, PDF_CMAP_SINGLE, 1158 },
	{ 0x8986, 0x8988, PDF_CMAP_RANGE, 9461 },
	{ 0x8989, 0x8989, PDF_CMAP_SINGLE, 1159 },
	{ 0x898a, 0x8991, PDF_CMAP_RANGE, 9464 },
	{ 0x8993, 0x8995, PDF_CMAP_TABLE, 15 },
	{ 0x8996, 0x899d, PDF_CMAP_RANGE, 9473 },
	{ 0x89a1, 0x89a2, PDF_CMAP_RANGE, 1162 },
	{ 0x89a3, 0x89a4, PDF_CMAP_RANGE, 9481 },
	{ 0x89a5, 0x89a5, PDF_CMAP_SINGLE, 1164 },
	{ 0x89a6, 0x89a7, PDF_CMAP_RANGE, 9483 },
	{ 0x89a8, 0x89a9, PDF_CMAP_RANGE, 1165 },
	{ 0x89aa, 0x89ad, PDF_CMAP_TABLE, 18 },
	{ 0x89ae, 0x89af, PDF_CMAP_RANGE, 9487 },
	{ 0x89b0, 0x89b1, PDF_CMAP_RANGE, 1169 },
	{ 0x89b3, 0x89b6, PDF_CMAP_TABLE, 22 },
	{ 0x89b7, 0x89b8, PDF_CMAP_RANGE, 1173 },
	{ 0x89b9, 0x89bd, PDF_CMAP_RANGE, 9491 },
	{ 0x89c1, 0x89c2, PDF_CMAP_RANGE, 1175 },
	{ 0x89c3, 0x89c4, PDF_CMAP_RANGE, 9496 },
	{ 0x89c5, 0x89c5, PDF_CMAP_SINGLE, 1177 },
	{ 0x89c6, 0x89c8, PDF_CMAP_RANGE, 9498 },
	{ 0x89c9, 0x89cb, PDF_CMAP_TABLE, 26 },
	{ 0x89cc, 0x89d0, PDF_CMAP_RANGE, 9502 },
	{ 0x89d1, 0x89d1, PDF_CMAP_SINGLE, 1180 },
	{ 0x89d3, 0x89d7, PDF_CMAP_TABLE, 29 },
	{ 0x89d8, 0x89dd, PDF_CMAP_RANGE, 9509 },
	{ 0x89e1, 0x89e1, PDF_CMAP_SINGLE, 1184 },
	{ 0x89e2, 0x89e4, PDF_CMAP_RANGE, 9515 },
	{ 0x89e5, 0x89e5, PDF_CMAP_SINGLE, 1185 },
	{ 0x89e6, 0x89e8, PDF_CMAP_RANGE, 9518 },
	{ 0x89e9, 0x89e9, PDF_CMAP_SINGLE, 1186 },
	{ 0x89ea, 0x89f1, PDF_CMAP_RANGE, 9521 },
	{ 0x89f3, 0x89f3, PDF_CMAP_SINGLE, 1187 },
	{ 0x89f4, 0x89f5, PDF_CMAP_RANGE, 9529 },
	{ 0x89f6, 0x89f7, PDF_CMAP_RANGE, 1188 },
	{ 0x89f8, 0x89fd, PDF_CMAP_RANGE, 9531 },
	{ 0x8a41, 0x8a42, PDF_CMAP_RANGE, 1190 },
	{ 0x8a43, 0x8a44, PDF_CMAP_RANGE, 9537 },
	{ 0x8a45, 0x8a45, PDF_CMAP_SINGLE, 1192 },
	{ 0x8a46, 0x8a48, PDF_CMAP_RANGE, 9539 },
	{ 0x8a49, 0x8a49, PDF_CMAP_SINGLE, 1193 },
	{ 0x8a4a, 0x8a50, PDF_CMAP_RANGE, 9542 },
	{ 0x8a51, 0x8a51, PDF_CMAP_SINGLE, 1194 },
	{ 0x8a53, 0x8a57, PDF_CMAP_TABLE, 34 },
	{ 0x8a58, 0x8a5d, PDF_CMAP_RANGE, 9551 },
	{ 0x8a61, 0x8a61, PDF_CMAP_SINGLE, 1198 },
	{ 0x8a62, 0x8a64, PDF_CMAP_RANGE, 9557 },
	{ 0x8a65, 0x8a65, PDF_CMAP_SINGLE, 1199 },
	{ 0x8a66, 0x8a68, PDF_CMAP_RANGE, 9560 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 1200 },
	{ 0x8a6a, 0x8a71, PDF_CMAP_RANGE, 9563 },
	{ 0x8a73, 0x8a75, PDF_CMAP_TABLE, 39 },
	{ 0x8a76, 0x8a7d, PDF_CMAP_RANGE, 9572 },
	{ 0x8a81, 0x8a82, PDF_CMAP_RANGE, 1203 },
	{ 0x8a83, 0x8a84, PDF_CMAP_RANGE, 9580 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 1205 },
	{ 0x8a86, 0x8a87, PDF_CMAP_RANGE, 9582 },
	{ 0x8a88, 0x8a8b, PDF_CMAP_RANGE, 1206 },
	{ 0x8a8c, 0x8a8f, PDF_CMAP_RANGE, 9584 },
	{ 0x8a90, 0x8a91, PDF_CMAP_RANGE, 1210 },
	{ 0x8a93, 0x8a96, PDF_CMAP_TABLE, 42 },
	{ 0x8a97, 0x8a98, PDF_CMAP_RANGE, 1214 },
	{ 0x8a99, 0x8a9d, PDF_CMAP_RANGE, 9590 },
	{ 0x8aa1, 0x8aa2, PDF_CMAP_RANGE, 1216 },
	{ 0x8aa3, 0x8aa4, PDF_CMAP_RANGE, 9595 },
	{ 0x8aa5, 0x8aa5, PDF_CMAP_SINGLE, 1218 },
	{ 0x8aa6, 0x8aa8, PDF_CMAP_RANGE, 9597 },
	{ 0x8aa9, 0x8aa9, PDF_CMAP_SINGLE, 1219 },
	{ 0x8aaa, 0x8ab1, PDF_CMAP_RANGE, 9600 },
	{ 0x8ab3, 0x8ab5, PDF_CMAP_RANGE, 9608 },
	{ 0x8ab6, 0x8ab7, PDF_CMAP_RANGE, 1220 },
	{ 0x8ab8, 0x8abd, PDF_CMAP_RANGE, 9611 },
	{ 0x8ac1, 0x8ac1, PDF_CMAP_SINGLE, 1222 },
	{ 0x8ac2, 0x8ad1, PDF_CMAP_RANGE, 9617 },
	{ 0x8ad3, 0x8ad4, PDF_CMAP_RANGE, 9633 },
	{ 0x8ad5, 0x8ad5, PDF_CMAP_SINGLE, 1223 },
	{ 0x8ad6, 0x8add, PDF_CMAP_RANGE, 9635 },
	{ 0x8ae1, 0x8ae2, PDF_CMAP_RANGE, 1224 },
	{ 0x8ae3, 0x8ae4, PDF_CMAP_RANGE, 9643 },
	{ 0x8ae5, 0x8ae5, PDF_CMAP_SINGLE, 1226 },
	{ 0x8ae6, 0x8ae8, PDF_CMAP_RANGE, 9645 },
	{ 0x8ae9, 0x8ae9, PDF_CMAP_SINGLE, 1227 },
	{ 0x8aea, 0x8af0, PDF_CMAP_RANGE, 9648 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 1228 },
	{ 0x8af3, 0x8af5, PDF_CMAP_TABLE, 46 },
	{ 0x8af6, 0x8afd, PDF_CMAP_RANGE, 9656 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 1231 },
	{ 0x8b42, 0x8b44, PDF_CMAP_RANGE, 9664 },
	{ 0x8b45, 0x8b45, PDF_CMAP_SINGLE, 1232 },
	{ 0x8b46, 0x8b48, PDF_CMAP_RANGE, 9667 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 1233 },
	{ 0x8b4a, 0x8b51, PDF_CMAP_RANGE, 9670 },
	{ 0x8b53, 0x8b5d, PDF_CMAP_RANGE, 9678 },
	{ 0x8b61, 0x8b62, PDF_CMAP_RANGE, 1234 },
	{ 0x8b63, 0x8b64, PDF_CMAP_RANGE, 9689 },
	{ 0x8b65, 0x8b65, PDF_CMAP_SINGLE, 1236 },
	{ 0x8b66, 0x8b67, PDF_CMAP_RANGE, 9691 },
	{ 0x8b68, 0x8b6a, PDF_CMAP_RANGE, 1237 },
	{ 0x8b6b, 0x8b70, PDF_CMAP_RANGE, 9693 },
	{ 0x8b71, 0x8b71, PDF_CMAP_SINGLE, 1240 },
	{ 0x8b73, 0x8b77, PDF_CMAP_TABLE, 49 },
	{ 0x8b78, 0x8b7d, PDF_CMAP_RANGE, 9701 },
	{ 0x8b81, 0x8b81, PDF_CMAP_SINGLE, 1244 },
	{ 0x8b82, 0x8b91, PDF_CMAP_RANGE, 9707 },
	{ 0x8b93, 0x8b9d, PDF_CMAP_RANGE, 9723 },
	{ 0x8ba1, 0x8ba2, PDF_CMAP_RANGE, 1245 },
	{ 0x8ba3, 0x8ba4, PDF_CMAP_RANGE, 9734 },
	{ 0x8ba5, 0x8ba5, PDF_CMAP_SINGLE, 1247 },
	{ 0x8ba6, 0x8ba7, PDF_CMAP_RANGE, 9736 },
	{ 0x8ba8, 0x8ba9, PDF_CMAP_RANGE, 1248 },
	{ 0x8baa, 0x8bab, PDF_CMAP_TABLE, 54 },
	{ 0x8bac, 0x8bb0, PDF_CMAP_RANGE, 9739 },
	{ 0x8bb1, 0x8bb1, PDF_CMAP_SINGLE, 1251 },
	{ 0x8bb3, 0x8bb6, PDF_CMAP_TABLE, 56 },
	{ 0x8bb7, 0x8bb8, PDF_CMAP_RANGE, 1254 },
	{ 0x8bb9, 0x8bbb, PDF_CMAP_RANGE, 9746 },
	{ 0x8bbc, 0x8bbd, PDF_CMAP_TABLE, 60 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 359 },
	{ 0x8c61, 0x8c63, PDF_CMAP_RANGE, 1257 },
	{ 0x8c64, 0x8c65, PDF_CMAP_TABLE, 62 },
	{ 0x8c66, 0x8c68, PDF_CMAP_RANGE, 9751 },
	{ 0x8c69, 0x8c6b, PDF_CMAP_TABLE, 64 },
	{ 0x8c6c, 0x8c70, PDF_CMAP_RANGE, 9755 },
	{ 0x8c71, 0x8c71, PDF_CMAP_SINGLE, 1263 },
	{ 0x8c73, 0x8c74, PDF_CMAP_TABLE, 67 },
	{ 0x8c75, 0x8c77, PDF_CMAP_RANGE, 1265 },
	{ 0x8c78, 0x8c7a, PDF_CMAP_RANGE, 9761 },
	{ 0x8c7b, 0x8c7b, PDF_CMAP_SINGLE, 1268 },
	{ 0x8c7c, 0x8c7d, PDF_CMAP_RANGE, 9764 },
	{ 0x8c81, 0x8c82, PDF_CMAP_RANGE, 1269 },
	{ 0x8c83, 0x8c84, PDF_CMAP_RANGE, 9766 },
	{ 0x8c85, 0x8c85, PDF_CMAP_SINGLE, 1271 },
	{ 0x8c86, 0x8c88, PDF_CMAP_RANGE, 9768 },
	{ 0x8c89, 0x8c89, PDF_CMAP_SINGLE, 1272 },
	{ 0x8c8a, 0x8c90, PDF_CMAP_RANGE, 9771 },
	{ 0x8c91, 0x8c91, PDF_CMAP_SINGLE, 1273 },
	{ 0x8c93, 0x8c94, PDF_CMAP_TABLE, 69 },
	{ 0x8c95, 0x8c97, PDF_CMAP_RANGE, 1275 },
	{ 0x8c98, 0x8c9d, PDF_CMAP_RANGE, 9779 },
	{ 0x8ca1, 0x8ca2, PDF_CMAP_RANGE, 1278 },
	{ 0x8ca3, 0x8ca8, PDF_CMAP_RANGE, 9785 },
	{ 0x8ca9, 0x8ca9, PDF_CMAP_SINGLE, 1280 },
	{ 0x8caa, 0x8cb1, PDF_CMAP_RANGE, 9791 },
	{ 0x8cb3, 0x8cbd, PDF_CMAP_RANGE, 9799 },
	{ 0x8cc1, 0x8cd1, PDF_CMAP_RANGE, 9810 },
	{ 0x8cd3, 0x8cdd, PDF_CMAP_RANGE, 9827 },
	{ 0x8ce1, 0x8ce3, PDF_CMAP_RANGE, 1281 },
	{ 0x8ce4, 0x8ce5, PDF_CMAP_TABLE, 71 },
	{ 0x8ce6, 0x8ce8, PDF_CMAP_RANGE, 9839 },
	{ 0x8ce9, 0x8ce9, PDF_CMAP_SINGLE, 1285 },
	{ 0x8cea, 0x8cf0, PDF_CMAP_RANGE, 9842 },
	{ 0x8cf1, 0x8cf1, PDF_CMAP_SINGLE, 1286 },
	{ 0x8cf3, 0x8cf4, PDF_CMAP_TABLE, 73 },
	{ 0x8cf5, 0x8cf7, PDF_CMAP_RANGE, 1288 },
	{ 0x8cf8, 0x8cfd, PDF_CMAP_RANGE, 9850 },
	{ 0x8d41, 0x8d42, PDF_CMAP_RANGE, 1291 },
	{ 0x8d43, 0x8d44, PDF_CMAP_RANGE, 9856 },
	{ 0x8d45, 0x8d45, PDF_CMAP_SINGLE, 1293 },
	{ 0x8d46, 0x8d50, PDF_CMAP_RANGE, 9858 },
	{ 0x8d51, 0x8d51, PDF_CMAP_SINGLE, 1294 },
	{ 0x8d53, 0x8d54, PDF_CMAP_RANGE, 9869 },
	{ 0x8d55, 0x8d57, PDF_CMAP_TABLE, 75 },
	{ 0x8d58, 0x8d5d, PDF_CMAP_RANGE, 9872 },
	{ 0x8d61, 0x8d61, PDF_CMAP_SINGLE, 1297 },
	{ 0x8d62, 0x8d64, PDF_CMAP_RANGE, 9878 },
	{ 0x8d65, 0x8d65, PDF_CMAP_SINGLE, 1298 },
	{ 0x8d66, 0x8d68, PDF_CMAP_RANGE, 9881 },
	{ 0x8d69, 0x8d69, PDF_CMAP_SINGLE, 1299 },
	{ 0x8d6a, 0x8d71, PDF_CMAP_RANGE, 9884 },
	{ 0x8d73, 0x8d74, PDF_CMAP_RANGE, 9892 },
	{ 0x8d75, 0x8d76, PDF_CMAP_RANGE, 1300 },
	{ 0x8d77, 0x8d7a, PDF_CMAP_RANGE, 9894 },
	{ 0x8d7b, 0x8d7b, PDF_CMAP_SINGLE, 1302 },
	{ 0x8d7c, 0x8d7d, PDF_CMAP_RANGE, 9898 },
	{ 0x8d81, 0x8d81, PDF_CMAP_SINGLE, 1303 },
	{ 0x8d82, 0x8d91, PDF_CMAP_RANGE, 9900 },
	{ 0x8d93, 0x8d9d, PDF_CMAP_RANGE, 9916 },
	{ 0x8da1, 0x8da2, PDF_CMAP_RANGE, 1304 },
	{ 0x8da3, 0x8da4, PDF_CMAP_RANGE, 9927 },
	{ 0x8da5, 0x8da9, PDF_CMAP_TABLE, 78 },
	{ 0x8daa, 0x8db0, PDF_CMAP_RANGE, 9931 },
	{ 0x8db1, 0x8db1, PDF_CMAP_SINGLE, 1309 },
	{ 0x8db3, 0x8db6, PDF_CMAP_TABLE, 83 },
	{ 0x8db7, 0x8db9, PDF_CMAP_RANGE, 1312 },
	{ 0x8dba, 0x8dbd, PDF_CMAP_RANGE, 9940 },
	{ 0x8dc1, 0x8dc2, PDF_CMAP_RANGE, 1315 },
	{ 0x8dc3, 0x8dc8, PDF_CMAP_RANGE, 9944 },
	{ 0x8dc9, 0x8dc9, PDF_CMAP_SINGLE, 1317 },
	{ 0x8dca, 0x8dd1, PDF_CMAP_RANGE, 9950 },
	{ 0x8dd3, 0x8dd5, PDF_CMAP_RANGE, 9958 },
	{ 0x8dd6, 0x8dd7, PDF_CMAP_RANGE, 1318 },
	{ 0x8dd8, 0x8ddd, PDF_CMAP_RANGE, 9961 },
	{ 0x8de1, 0x8de2, PDF_CMAP_RANGE, 1320 },
	{ 0x8de3, 0x8df1, PDF_CMAP_RANGE, 9967 },
	{ 0x8df3, 0x8df6, PDF_CMAP_RANGE, 9982 },
	{ 0x8df7, 0x8df7, PDF_CMAP_SINGLE, 1322 },
	{ 0x8df8, 0x8dfd, PDF_CMAP_RANGE, 9986 },
	{ 0x8e41, 0x8e41, PDF_CMAP_SINGLE, 1323 },
	{ 0x8e42, 0x8e44, PDF_CMAP_RANGE, 9992 },
	{ 0x8e45, 0x8e45, PDF_CMAP_SINGLE, 1324 },
	{ 0x8e46, 0x8e48, PDF_CMAP_RANGE, 9995 },
	{ 0x8e49, 0x8e49, PDF_CMAP_SINGLE, 1325 },
	{ 0x8e4a, 0x8e50, PDF_CMAP_RANGE, 9998 },
	{ 0x8e51, 0x8e51, PDF_CMAP_SINGLE, 1326 },
	{ 0x8e53, 0x8e53, PDF_CMAP_SINGLE, 1327 },
	{ 0x8e54, 0x8e56, PDF_CMAP_RANGE, 10005 },
	{ 0x8e57, 0x8e57, PDF_CMAP_SINGLE, 1328 },
	{ 0x8e58, 0x8e5d, PDF_CMAP_RANGE, 10008 },
	{ 0x8e61, 0x8e61, PDF_CMAP_SINGLE, 1329 },
	{ 0x8e62, 0x8e71, PDF_CMAP_RANGE, 10014 },
	{ 0x8e73, 0x8e7d, PDF_CMAP_RANGE, 10030 },
	{ 0x8e81, 0x8e82, PDF_CMAP_RANGE, 1330 },
	{ 0x8e83, 0x8e84, PDF_CMAP_RANGE, 10041 },
	{ 0x8e85, 0x8e85, PDF_CMAP_SINGLE, 1332 },
	{ 0x8e86, 0x8e88, PDF_CMAP_RANGE, 10043 },
	{ 0x8e89, 0x8e89, PDF_CMAP_SINGLE, 1333 },
	{ 0x8e8a, 0x8e8f, PDF_CMAP_RANGE, 10046 },
	{ 0x8e90, 0x8e91, PDF_CMAP_RANGE, 1334 },
	{ 0x8e93, 0x8e96, PDF_CMAP_TABLE, 87 },
	{ 0x8e97, 0x8e98, PDF_CMAP_RANGE, 1338 },
	{ 0x8e99, 0x8e9d, PDF_CMAP_RANGE, 10054 },
	{ 0x8ea1, 0x8ea1, PDF_CMAP_SINGLE, 1340 },
	{ 0x8ea2, 0x8ea8, PDF_CMAP_RANGE, 10059 },
	{ 0x8ea9, 0x8ea9, PDF_CMAP_SINGLE, 1341 },
	{ 0x8eaa, 0x8eb1, PDF_CMAP_RANGE, 10066 },
	{ 0x8eb3, 0x8eb5, PDF_CMAP_RANGE, 10074 },
	{ 0x8eb6, 0x8eb7, PDF_CMAP_RANGE, 1342 },
	{ 0x8eb8, 0x8ebd, PDF_CMAP_RANGE, 10077 },
	{ 0x8ec1, 0x8ec2, PDF_CMAP_RANGE, 1344 },
	{ 0x8ec3, 0x8ec4, PDF_CMAP_RANGE, 10083 },
	{ 0x8ec5, 0x8ec5, PDF_CMAP_SINGLE, 1346 },
	{ 0x8ec6, 0x8ec8, PDF_CMAP_RANGE, 10085 },
	{ 0x8ec9, 0x8ec9, PDF_CMAP_SINGLE, 1347 },
	{ 0x8eca, 0x8ed0, PDF_CMAP_RANGE, 10088 },
	{ 0x8ed1, 0x8ed1, PDF_CMAP_SINGLE, 1348 },
	{ 0x8ed3, 0x8ed3, PDF_CMAP_SINGLE, 1349 },
	{ 0x8ed4, 0x8ed5, PDF_CMAP_RANGE, 10095 },
	{ 0x8ed6, 0x8ed6, PDF_CMAP_SINGLE, 1350 },
	{ 0x8ed7, 0x8edd, PDF_CMAP_RANGE, 10097 },
	{ 0x8ee1, 0x8ee1, PDF_CMAP_SINGLE, 1351 },
	{ 0x8ee2, 0x8ee4, PDF_CMAP_RANGE, 10104 },
	{ 0x8ee5, 0x8ee5, PDF_CMAP_SINGLE, 1352 },
	{ 0x8ee6, 0x8ee8, PDF_CMAP_RANGE, 10107 },
	{ 0x8ee9, 0x8ee9, PDF_CMAP_SINGLE, 1353 },
	{ 0x8eea, 0x8ef0, PDF_CMAP_RANGE, 10110 },
	{ 0x8ef1, 0x8ef1, PDF_CMAP_SINGLE, 1354 },
	{ 0x8ef3, 0x8ef3, PDF_CMAP_SINGLE, 1355 },
	{ 0x8ef4, 0x8efd, PDF_CMAP_RANGE, 10117 },
	{ 0x8f41, 0x8f41, PDF_CMAP_SINGLE, 1356 },
	{ 0x8f42, 0x8f51, PDF_CMAP_RANGE, 10127 },
	{ 0x8f53, 0x8f5d, PDF_CMAP_RANGE, 10143 },
	{ 0x8f61, 0x8f62, PDF_CMAP_RANGE, 1357 },
	{ 0x8f63, 0x8f64, PDF_CMAP_RANGE, 10154 },
	{ 0x8f65, 0x8f6b, PDF_CMAP_TABLE, 91 },
	{ 0x8f6c, 0x8f6f, PDF_CMAP_RANGE, 10159 },
	{ 0x8f70, 0x8f71, PDF_CMAP_RANGE, 1363 },
	{ 0x8f73, 0x8f77, PDF_CMAP_TABLE, 98 },
	{ 0x8f78, 0x8f7a, PDF_CMAP_RANGE, 10165 },
	{ 0x8f7b, 0x8f7b, PDF_CMAP_SINGLE, 1368 },
	{ 0x8f7c, 0x8f7d, PDF_CMAP_RANGE, 10168 },
	{ 0x8f81, 0x8f91, PDF_CMAP_RANGE, 10170 },
	{ 0x8f93, 0x8f9d, PDF_CMAP_RANGE, 10187 },
	{ 0x8fa1, 0x8fa2, PDF_CMAP_RANGE, 1369 },
	{ 0x8fa3, 0x8fa4, PDF_CMAP_RANGE, 10198 },
	{ 0x8fa5, 0x8fa5, PDF_CMAP_SINGLE, 1371 },
	{ 0x8fa6, 0x8fa8, PDF_CMAP_RANGE, 10200 },
	{ 0x8fa9, 0x8fa9, PDF_CMAP_SINGLE, 1372 },
	{ 0x8faa, 0x8fb0, PDF_CMAP_RANGE, 10203 },
	{ 0x8fb1, 0x8fb1, PDF_CMAP_SINGLE, 1373 },
	{ 0x8fb3, 0x8fb7, PDF_CMAP_TABLE, 103 },
	{ 0x8fb8, 0x8fbd, PDF_CMAP_RANGE, 10212 },
	{ 0x9041, 0x9041, PDF_CMAP_SINGLE, 361 },
	{ 0x9061, 0x9063, PDF_CMAP_RANGE, 1377 },
	{ 0x9064, 0x9065, PDF_CMAP_TABLE, 108 },
	{ 0x9066, 0x9067, PDF_CMAP_RANGE, 10219 },
	{ 0x9068, 0x906b, PDF_CMAP_RANGE, 1381 },
	{ 0x906c, 0x9070, PDF_CMAP_RANGE, 10221 },
	{ 0x9071, 0x9071, PDF_CMAP_SINGLE, 1385 },
	{ 0x9073, 0x9074, PDF_CMAP_TABLE, 110 },
	{ 0x9075, 0x9079, PDF_CMAP_RANGE, 1387 },
	{ 0x907a, 0x907d, PDF_CMAP_TABLE, 112 },
	{ 0x9081, 0x9082, PDF_CMAP_RANGE, 1394 },
	{ 0x9083, 0x9084, PDF_CMAP_RANGE, 10229 },
	{ 0x9085, 0x9085, PDF_CMAP_SINGLE, 1396 },
	{ 0x9086, 0x9088, PDF_CMAP_RANGE, 10231 },
	{ 0x9089, 0x9089, PDF_CMAP_SINGLE, 1397 },
	{ 0x908a, 0x9090, PDF_CMAP_RANGE, 10234 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 1398 },
	{ 0x9093, 0x9094, PDF_CMAP_TABLE, 116 },
	{ 0x9095, 0x9097, PDF_CMAP_RANGE, 1400 },
	{ 0x9098, 0x909d, PDF_CMAP_RANGE, 10242 },
	{ 0x90a1, 0x90a2, PDF_CMAP_RANGE, 1403 },
	{ 0x90a3, 0x90a4, PDF_CMAP_RANGE, 10248 },
	{ 0x90a5, 0x90a5, PDF_CMAP_SINGLE, 1405 },
	{ 0x90a6, 0x90a8, PDF_CMAP_RANGE, 10250 },
	{ 0x90a9, 0x90a9, PDF_CMAP_SINGLE, 1406 },
	{ 0x90aa, 0x90b0, PDF_CMAP_RANGE, 10253 },
	{ 0x90b1, 0x90b1, PDF_CMAP_SINGLE, 1407 },
	{ 0x90b3, 0x90b6, PDF_CMAP_RANGE, 10260 },
	{ 0x90b7, 0x90b7, PDF_CMAP_SINGLE, 1408 },
	{ 0x90b8, 0x90bd, PDF_CMAP_RANGE, 10264 },
	{ 0x90c1, 0x90d1, PDF_CMAP_RANGE, 10270 },
	{ 0x90d3, 0x90dd, PDF_CMAP_RANGE, 10287 },
	{ 0x90e1, 0x90e2, PDF_CMAP_RANGE, 1409 },
	{ 0x90e3, 0x90e3, PDF_CMAP_SINGLE, 10298 },
	{ 0x90e4, 0x90e5, PDF_CMAP_RANGE, 1411 },
	{ 0x90e6, 0x90e8, PDF_CMAP_RANGE, 10299 },
	{ 0x90e9, 0x90ea, PDF_CMAP_TABLE, 118 },
	{ 0x90eb, 0x90ec, PDF_CMAP_RANGE, 1414 },
	{ 0x90ed, 0x90f0, PDF_CMAP_RANGE, 10303 },
	{ 0x90f1, 0x90f1, PDF_CMAP_SINGLE, 1416 },
	{ 0x90f3, 0x90f4, PDF_CMAP_TABLE, 120 },
	{ 0x90f5, 0x90f7, PDF_CMAP_RANGE, 1418 },
	{ 0x90f8, 0x90fc, PDF_CMAP_RANGE, 10308 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 1421 },
	{ 0x9141, 0x9142, PDF_CMAP_RANGE, 1422 },
	{ 0x9143, 0x9144, PDF_CMAP_RANGE, 10313 },
	{ 0x9145, 0x9145, PDF_CMAP_SINGLE, 1424 },
	{ 0x9146, 0x9148, PDF_CMAP_RANGE, 10315 },
	{ 0x9149, 0x9149, PDF_CMAP_SINGLE, 1425 },
	{ 0x914a, 0x9150, PDF_CMAP_RANGE, 10318 },
	{ 0x9151, 0x9151, PDF_CMAP_SINGLE, 1426 },
	{ 0x9153, 0x9154, PDF_CMAP_TABLE, 122 },
	{ 0x9155, 0x9157, PDF_CMAP_RANGE, 1428 },
	{ 0x9158, 0x915d, PDF_CMAP_RANGE, 10326 },
	{ 0x9161, 0x9162, PDF_CMAP_RANGE, 1431 },
	{ 0x9163, 0x9164, PDF_CMAP_RANGE, 10332 },
	{ 0x9165, 0x9165, PDF_CMAP_SINGLE, 1433 },
	{ 0x9166, 0x9168, PDF_CMAP_RANGE, 10334 },
	{ 0x9169, 0x9169, PDF_CMAP_SINGLE, 1434 },
	{ 0x916a, 0x9170, PDF_CMAP_RANGE, 10337 },
	{ 0x9171, 0x9171, PDF_CMAP_SINGLE, 1435 },
	{ 0x9173, 0x9173, PDF_CMAP_SINGLE, 1436 },
	{ 0x9174, 0x9175, PDF_CMAP_RANGE, 10344 },
	{ 0x9176, 0x9177, PDF_CMAP_RANGE, 1437 },
	{ 0x9178, 0x9179, PDF_CMAP_RANGE, 10346 },
	{ 0x917a, 0x917a, PDF_CMAP_SINGLE, 1439 },
	{ 0x917b, 0x917d, PDF_CMAP_RANGE, 10348 },
	{ 0x9181, 0x9181, PDF_CMAP_SINGLE, 1440 },
	{ 0x9182, 0x9184, PDF_CMAP_RANGE, 10351 },
	{ 0x9185, 0x9185, PDF_CMAP_SINGLE, 1441 },
	{ 0x9186, 0x9191, PDF_CMAP_RANGE, 10354 },
	{ 0x9193, 0x919d, PDF_CMAP_RANGE, 10366 },
	{ 0x91a1, 0x91a2, PDF_CMAP_RANGE, 1442 },
	{ 0x91a3, 0x91a4, PDF_CMAP_RANGE, 10377 },
	{ 0x91a5, 0x91a5, PDF_CMAP_SINGLE, 1444 },
	{ 0x91a6, 0x91a8, PDF_CMAP_RANGE, 10379 },
	{ 0x91a9, 0x91ab, PDF_CMAP_TABLE, 124 },
	{ 0x91ac, 0x91b0, PDF_CMAP_RANGE, 10383 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 1447 },
	{ 0x91b3, 0x91b7, PDF_CMAP_TABLE, 127 },
	{ 0x91b8, 0x91bb, PDF_CMAP_RANGE, 10390 },
	{ 0x91bc, 0x91bd, PDF_CMAP_RANGE, 1451 },
	{ 0x91c1, 0x91c1, PDF_CMAP_SINGLE, 1453 },
	{ 0x91c2, 0x91c4, PDF_CMAP_RANGE, 10394 },
	{ 0x91c5, 0x91c5, PDF_CMAP_SINGLE, 1454 },
	{ 0x91c6, 0x91c8, PDF_CMAP_RANGE, 10397 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 1455 },
	{ 0x91ca, 0x91d1, PDF_CMAP_RANGE, 10400 },
	{ 0x91d3, 0x91d5, PDF_CMAP_RANGE, 10408 },
	{ 0x91d6, 0x91d6, PDF_CMAP_SINGLE, 1456 },
	{ 0x91d7, 0x91dd, PDF_CMAP_RANGE, 10411 },
	{ 0x91e1, 0x91f1, PDF_CMAP_RANGE, 10418 },
	{ 0x91f3, 0x91fd, PDF_CMAP_RANGE, 10435 },
	{ 0x9241, 0x9241, PDF_CMAP_SINGLE, 1457 },
	{ 0x9242, 0x9244, PDF_CMAP_RANGE, 10446 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 1458 },
	{ 0x9246, 0x9248, PDF_CMAP_RANGE, 10449 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 1459 },
	{ 0x924a, 0x9250, PDF_CMAP_RANGE, 10452 },
	{ 0x9251, 0x9251, PDF_CMAP_SINGLE, 1460 },
	{ 0x9253, 0x9255, PDF_CMAP_TABLE, 132 },
	{ 0x9256, 0x925d, PDF_CMAP_RANGE, 10460 },
	{ 0x9261, 0x9262, PDF_CMAP_RANGE, 1463 },
	{ 0x9263, 0x9264, PDF_CMAP_RANGE, 10468 },
	{ 0x9265, 0x9265, PDF_CMAP_SINGLE, 1465 },
	{ 0x9266, 0x9268, PDF_CMAP_RANGE, 10470 },
	{ 0x9269, 0x9269, PDF_CMAP_SINGLE, 1466 },
	{ 0x926a, 0x9271, PDF_CMAP_RANGE, 10473 },
	{ 0x9273, 0x9277, PDF_CMAP_TABLE, 135 },
	{ 0x9278, 0x927d, PDF_CMAP_RANGE, 10483 },
	{ 0x9281, 0x9282, PDF_CMAP_RANGE, 1470 },
	{ 0x9283, 0x9284, PDF_CMAP_RANGE, 10489 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 1472 },
	{ 0x9286, 0x9287, PDF_CMAP_RANGE, 10491 },
	{ 0x9288, 0x9289, PDF_CMAP_RANGE, 1473 },
	{ 0x928a, 0x9290, PDF_CMAP_RANGE, 10493 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 1475 },
	{ 0x9293, 0x9297, PDF_CMAP_TABLE, 140 },
	{ 0x9298, 0x929d, PDF_CMAP_RANGE, 10502 },
	{ 0x92a1, 0x92a1, PDF_CMAP_SINGLE, 1479 },
	{ 0x92a2, 0x92b1, PDF_CMAP_RANGE, 10508 },
	{ 0x92b3, 0x92b5, PDF_CMAP_RANGE, 10524 },
	{ 0x92b6, 0x92b6, PDF_CMAP_SINGLE, 1480 },
	{ 0x92b7, 0x92bd, PDF_CMAP_RANGE, 10527 },
	{ 0x92c1, 0x92c1, PDF_CMAP_SINGLE, 1481 },
	{ 0x92c2, 0x92d1, PDF_CMAP_RANGE, 10534 },
	{ 0x92d3, 0x92dd, PDF_CMAP_RANGE, 10550 },
	{ 0x92e1, 0x92e1, PDF_CMAP_SINGLE, 1482 },
	{ 0x92e2, 0x92e4, PDF_CMAP_RANGE, 10561 },
	{ 0x92e5, 0x92e5, PDF_CMAP_SINGLE, 1483 },
	{ 0x92e6, 0x92e8, PDF_CMAP_RANGE, 10564 },
	{ 0x92e9, 0x92e9, PDF_CMAP_SINGLE, 1484 },
	{ 0x92ea, 0x92f0, PDF_CMAP_RANGE, 10567 },
	{ 0x92f1, 0x92f1, PDF_CMAP_SINGLE, 1485 },
	{ 0x92f3, 0x92f3, PDF_CMAP_SINGLE, 1486 },
	{ 0x92f4, 0x92fd, PDF_CMAP_RANGE, 10574 },
	{ 0x9341, 0x9342, PDF_CMAP_RANGE, 1487 },
	{ 0x9343, 0x9348, PDF_CMAP_RANGE, 10584 },
	{ 0x9349, 0x9349, PDF_CMAP_SINGLE, 1489 },
	{ 0x934a, 0x9350, PDF_CMAP_RANGE, 10590 },
	{ 0x9351, 0x9351, PDF_CMAP_SINGLE, 1490 },
	{ 0x9353, 0x9353, PDF_CMAP_SINGLE, 1491 },
	{ 0x9354, 0x9356, PDF_CMAP_RANGE, 10597 },
	{ 0x9357, 0x9357, PDF_CMAP_SINGLE, 1492 },
	{ 0x9358, 0x935d, PDF_CMAP_RANGE, 10600 },
	{ 0x9361, 0x9362, PDF_CMAP_RANGE, 1493 },
	{ 0x9363, 0x9364, PDF_CMAP_RANGE, 10606 },
	{ 0x9365, 0x9365, PDF_CMAP_SINGLE, 1495 },
	{ 0x9366, 0x9368, PDF_CMAP_RANGE, 10608 },
	{ 0x9369, 0x936b, PDF_CMAP_RANGE, 1496 },
	{ 0x936c, 0x9370, PDF_CMAP_RANGE, 10611 },
	{ 0x9371, 0x9371, PDF_CMAP_SINGLE, 1499 },
	{ 0x9373, 0x9376, PDF_CMAP_TABLE, 145 },
	{ 0x9377, 0x9378, PDF_CMAP_RANGE, 1502 },
	{ 0x9379, 0x937b, PDF_CMAP_RANGE, 10618 },
	{ 0x937c, 0x937d, PDF_CMAP_TABLE, 149 },
	{ 0x9381, 0x9381, PDF_CMAP_SINGLE, 1505 },
	{ 0x9382, 0x9384, PDF_CMAP_RANGE, 10622 },
	{ 0x9385, 0x9385, PDF_CMAP_SINGLE, 1506 },
	{ 0x9386, 0x9388, PDF_CMAP_RANGE, 10625 },
	{ 0x9389, 0x9389, PDF_CMAP_SINGLE, 1507 },
	{ 0x938a, 0x9391, PDF_CMAP_RANGE, 10628 },
	{ 0x9393, 0x939d, PDF_CMAP_RANGE, 10636 },
	{ 0x93a1, 0x93a2, PDF_CMAP_RANGE, 1508 },
	{ 0x93a3, 0x93a4, PDF_CMAP_RANGE, 10647 },
	{ 0x93a5, 0x93a5, PDF_CMAP_SINGLE, 1510 },
	{ 0x93a6, 0x93a8, PDF_CMAP_RANGE, 10649 },
	{ 0x93a9, 0x93ab, PDF_CMAP_TABLE, 151 },
	{ 0x93ac, 0x93b0, PDF_CMAP_RANGE, 10653 },
	{ 0x93b1, 0x93b1, PDF_CMAP_SINGLE, 1513 },
	{ 0x93b3, 0x93b7, PDF_CMAP_TABLE, 154 },
	{ 0x93b8, 0x93bb, PDF_CMAP_RANGE, 10660 },
	{ 0x93bc, 0x93bd, PDF_CMAP_TABLE, 159 },
	{ 0x9441, 0x9441, PDF_CMAP_SINGLE, 364 },
	{ 0x9461, 0x9463, PDF_CMAP_RANGE, 1518 },
	{ 0x9464, 0x9465, PDF_CMAP_TABLE, 161 },
	{ 0x9466, 0x9467, PDF_CMAP_RANGE, 10666 },
	{ 0x9468, 0x946c, PDF_CMAP_RANGE, 1522 },
	{ 0x946d, 0x946f, PDF_CMAP_RANGE, 10668 },
	{ 0x9470, 0x9471, PDF_CMAP_RANGE, 1527 },
	{ 0x9473, 0x9474, PDF_CMAP_TABLE, 163 },
	{ 0x9475, 0x9479, PDF_CMAP_RANGE, 1530 },
	{ 0x947a, 0x947c, PDF_CMAP_RANGE, 10672 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 1535 },
	{ 0x9481, 0x9482, PDF_CMAP_RANGE, 1536 },
	{ 0x9483, 0x9484, PDF_CMAP_RANGE, 10675 },
	{ 0x9485, 0x9485, PDF_CMAP_SINGLE, 1538 },
	{ 0x9486, 0x9488, PDF_CMAP_RANGE, 10677 },
	{ 0x9489, 0x9489, PDF_CMAP_SINGLE, 1539 },
	{ 0x948a, 0x9490, PDF_CMAP_RANGE, 10680 },
	{ 0x9491, 0x9491, PDF_CMAP_SINGLE, 1540 },
	{ 0x9493, 0x9494, PDF_CMAP_TABLE, 165 },
	{ 0x9495, 0x9497, PDF_CMAP_RANGE, 1542 },
	{ 0x9498, 0x949d, PDF_CMAP_RANGE, 10688 },
	{ 0x94a1, 0x94a1, PDF_CMAP_SINGLE, 1545 },
	{ 0x94a2, 0x94b1, PDF_CMAP_RANGE, 10694 },
	{ 0x94b3, 0x94bd, PDF_CMAP_RANGE, 10710 },
	{ 0x94c1, 0x94d1, PDF_CMAP_RANGE, 10721 },
	{ 0x94d3, 0x94dd, PDF_CMAP_RANGE, 10738 },
	{ 0x94e1, 0x94e3, PDF_CMAP_RANGE, 1546 },
	{ 0x94e4, 0x94e5, PDF_CMAP_TABLE, 167 },
	{ 0x94e6, 0x94e7, PDF_CMAP_RANGE, 10750 },
	{ 0x94e8, 0x94e9, PDF_CMAP_RANGE, 1550 },
	{ 0x94ea, 0x94ea, PDF_CMAP_SINGLE, 10752 },
	{ 0x94eb, 0x94ec, PDF_CMAP_RANGE, 1552 },
	{ 0x94ed, 0x94f0, PDF_CMAP_RANGE, 10753 },
	{ 0x94f1, 0x94f1, PDF_CMAP_SINGLE, 1554 },
	{ 0x94f3, 0x94f9, PDF_CMAP_TABLE, 169 },
	{ 0x94fa, 0x94fb, PDF_CMAP_RANGE, 10760 },
	{ 0x94fc, 0x94fd, PDF_CMAP_TABLE, 176 },
	{ 0x9541, 0x9542, PDF_CMAP_RANGE, 1560 },
	{ 0x9543, 0x9544, PDF_CMAP_RANGE, 10763 },
	{ 0x9545, 0x9545, PDF_CMAP_SINGLE, 1562 },
	{ 0x9546, 0x9548, PDF_CMAP_RANGE, 10765 },
	{ 0x9549, 0x9549, PDF_CMAP_SINGLE, 1563 },
	{ 0x954a, 0x9550, PDF_CMAP_RANGE, 10768 },
	{ 0x9551, 0x9551, PDF_CMAP_SINGLE, 1564 },
	{ 0x9553, 0x9554, PDF_CMAP_TABLE, 178 },
	{ 0x9555, 0x9557, PDF_CMAP_RANGE, 1566 },
	{ 0x9558, 0x955d, PDF_CMAP_RANGE, 10776 },
	{ 0x9561, 0x9561, PDF_CMAP_SINGLE, 1569 },
	{ 0x9562, 0x9564, PDF_CMAP_RANGE, 10782 },
	{ 0x9565, 0x9565, PDF_CMAP_SINGLE, 1570 },
	{ 0x9566, 0x9568, PDF_CMAP_RANGE, 10785 },
	{ 0x9569, 0x9569, PDF_CMAP_SINGLE, 1571 },
	{ 0x956a, 0x9571, PDF_CMAP_RANGE, 10788 },
	{ 0x9573, 0x9575, PDF_CMAP_RANGE, 10796 },
	{ 0x9576, 0x9577, PDF_CMAP_RANGE, 1572 },
	{ 0x9578, 0x957d, PDF_CMAP_RANGE, 10799 },
	{ 0x9581, 0x9581, PDF_CMAP_SINGLE, 1574 },
	{ 0x9582, 0x9584, PDF_CMAP_RANGE, 10805 },
	{ 0x9585, 0x9585, PDF_CMAP_SINGLE, 1575 },
	{ 0x9586, 0x9591, PDF_CMAP_RANGE, 10808 },
	{ 0x9593, 0x959d, PDF_CMAP_RANGE, 10820 },
	{ 0x95a1, 0x95a2, PDF_CMAP_RANGE, 1576 },
	{ 0x95a3, 0x95a4, PDF_CMAP_RANGE, 10831 },
	{ 0x95a5, 0x95a5, PDF_CMAP_SINGLE, 1578 },
	{ 0x95a6, 0x95a7, PDF_CMAP_RANGE, 10833 },
	{ 0x95a8, 0x95a9, PDF_CMAP_RANGE, 1579 },
	{ 0x95aa, 0x95ad, PDF_CMAP_TABLE, 180 },
	{ 0x95ae, 0x95b0, PDF_CMAP_RANGE, 10837 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 1583 },
	{ 0x95b3, 0x95bb, PDF_CMAP_TABLE, 184 },
	{ 0x95bc, 0x95bd, PDF_CMAP_RANGE, 10844 },
	{ 0x95c1, 0x95c1, PDF_CMAP_SINGLE, 1589 },
	{ 0x95c2, 0x95c4, PDF_CMAP_RANGE, 10846 },
	{ 0x95c5, 0x95c5, PDF_CMAP_SINGLE, 1590 },
	{ 0x95c6, 0x95c8, PDF_CMAP_RANGE, 10849 },
	{ 0x95c9, 0x95c9, PDF_CMAP_SINGLE, 1591 },
	{ 0x95ca, 0x95d1, PDF_CMAP_RANGE, 10852 },
	{ 0x95d3, 0x95dd, PDF_CMAP_RANGE, 10860 },
	{ 0x95e1, 0x95e1, PDF_CMAP_SINGLE, 1592 },
	{ 0x95e2, 0x95f1, PDF_CMAP_RANGE, 10871 },
	{ 0x95f3, 0x95f5, PDF_CMAP_RANGE, 10887 },
	{ 0x95f6, 0x95f6, PDF_CMAP_SINGLE, 1593 },
	{ 0x95f7, 0x95fd, PDF_CMAP_RANGE, 10890 },
	{ 0x9641, 0x9641, PDF_CMAP_SINGLE, 1594 },
	{ 0x9642, 0x9644, PDF_CMAP_RANGE, 10897 },
	{ 0x9645, 0x9645, PDF_CMAP_SINGLE, 1595 },
	{ 0x9646, 0x9648, PDF_CMAP_RANGE, 10900 },
	{ 0x9649, 0x9649, PDF_CMAP_SINGLE, 1596 },
	{ 0x964a, 0x9650, PDF_CMAP_RANGE, 10903 },
	{ 0x9651, 0x9651, PDF_CMAP_SINGLE, 1597 },
	{ 0x9653, 0x9655, PDF_CMAP_TABLE, 193 },
	{ 0x9656, 0x965d, PDF_CMAP_RANGE, 10911 },
	{ 0x9661, 0x9661, PDF_CMAP_SINGLE, 1600 },
	{ 0x9662, 0x9671, PDF_CMAP_RANGE, 10919 },
	{ 0x9673, 0x967d, PDF_CMAP_RANGE, 10935 },
	{ 0x9681, 0x9682, PDF_CMAP_RANGE, 1601 },
	{ 0x9683, 0x9684, PDF_CMAP_RANGE, 10946 },
	{ 0x9685, 0x9685, PDF_CMAP_SINGLE, 1603 },
	{ 0x9686, 0x9688, PDF_CMAP_RANGE, 10948 },
	{ 0x9689, 0x9689, PDF_CMAP_SINGLE, 1604 },
	{ 0x968a, 0x9690, PDF_CMAP_RANGE, 10951 },
	{ 0x9691, 0x9691, PDF_CMAP_SINGLE, 1605 },
	{ 0x9693, 0x9697, PDF_CMAP_TABLE, 196 },
	{ 0x9698, 0x969d, PDF_CMAP_RANGE, 10960 },
	{ 0x96a1, 0x96a1, PDF_CMAP_SINGLE, 1609 },
	{ 0x96a2, 0x96b1, PDF_CMAP_RANGE, 10966 },
	{ 0x96b3, 0x96b5, PDF_CMAP_RANGE, 10982 },
	{ 0x96b6, 0x96b6, PDF_CMAP_SINGLE, 1610 },
	{ 0x96b7, 0x96bd, PDF_CMAP_RANGE, 10985 },
	{ 0x96c1, 0x96c1, PDF_CMAP_SINGLE, 1611 },
	{ 0x96c2, 0x96d1, PDF_CMAP_RANGE, 10992 },
	{ 0x96d3, 0x96d6, PDF_CMAP_RANGE, 11008 },
	{ 0x96d7, 0x96d7, PDF_CMAP_SINGLE, 1612 },
	{ 0x96d8, 0x96dd, PDF_CMAP_RANGE, 11012 },
	{ 0x96e1, 0x96e1, PDF_CMAP_SINGLE, 1613 },
	{ 0x96e2, 0x96e4, PDF_CMAP_RANGE, 11018 },
	{ 0x96e5, 0x96e5, PDF_CMAP_SINGLE, 1614 },
	{ 0x96e6, 0x96e8, PDF_CMAP_RANGE, 11021 },
	{ 0x96e9, 0x96e9, PDF_CMAP_SINGLE, 1615 },
	{ 0x96ea, 0x96f1, PDF_CMAP_RANGE, 11024 },
	{ 0x96f3, 0x96f7, PDF_CMAP_TABLE, 201 },
	{ 0x96f8, 0x96fd, PDF_CMAP_RANGE, 11034 },
	{ 0x9741, 0x9741, PDF_CMAP_SINGLE, 1619 },
	{ 0x9742, 0x9744, PDF_CMAP_RANGE, 11040 },
	{ 0x9745, 0x9745, PDF_CMAP_SINGLE, 1620 },
	{ 0x9746, 0x9748, PDF_CMAP_RANGE, 11043 },
	{ 0x9749, 0x9749, PDF_CMAP_SINGLE, 1621 },
	{ 0x974a, 0x9750, PDF_CMAP_RANGE, 11046 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 1622 },
	{ 0x9753, 0x9756, PDF_CMAP_RANGE, 11053 },
	{ 0x9757, 0x9757, PDF_CMAP_SINGLE, 1623 },
	{ 0x9758, 0x975d, PDF_CMAP_RANGE, 11057 },
	{ 0x9761, 0x9762, PDF_CMAP_RANGE, 1624 },
	{ 0x9763, 0x9764, PDF_CMAP_RANGE, 11063 },
	{ 0x9765, 0x9765, PDF_CMAP_SINGLE, 1626 },
	{ 0x9766, 0x9767, PDF_CMAP_RANGE, 11065 },
	{ 0x9768, 0x9769, PDF_CMAP_RANGE, 1627 },
	{ 0x976a, 0x976b, PDF_CMAP_TABLE, 206 },
	{ 0x976c, 0x9770, PDF_CMAP_RANGE, 11068 },
	{ 0x9771, 0x9771, PDF_CMAP_SINGLE, 1630 },
	{ 0x9773, 0x9777, PDF_CMAP_TABLE, 208 },
	{ 0x9778, 0x977d, PDF_CMAP_RANGE, 11075 },
	{ 0x9781, 0x9781, PDF_CMAP_SINGLE, 1634 },
	{ 0x9782, 0x9791, PDF_CMAP_RANGE, 11081 },
	{ 0x9793, 0x979d, PDF_CMAP_RANGE, 11097 },
	{ 0x97a1, 0x97a2, PDF_CMAP_RANGE, 1635 },
	{ 0x97a3, 0x97a4, PDF_CMAP_RANGE, 11108 },
	{ 0x97a5, 0x97a5, PDF_CMAP_SINGLE, 1637 },
	{ 0x97a6, 0x97a7, PDF_CMAP_RANGE, 11110 },
	{ 0x97a8, 0x97a9, PDF_CMAP_RANGE, 1638 },
	{ 0x97aa, 0x97b0, PDF_CMAP_RANGE, 11112 },
	{ 0x97b1, 0x97b1, PDF_CMAP_SINGLE, 1640 },
	{ 0x97b3, 0x97b4, PDF_CMAP_TABLE, 213 },
	{ 0x97b5, 0x97b8, PDF_CMAP_RANGE, 1642 },
	{ 0x97b9, 0x97bd, PDF_CMAP_RANGE, 11120 },
	{ 0x9841, 0x9841, PDF_CMAP_SINGLE, 365 },
	{ 0x9861, 0x9862, PDF_CMAP_RANGE, 1646 },
	{ 0x9863, 0x9864, PDF_CMAP_RANGE, 11125 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 1648 },
	{ 0x9866, 0x9868, PDF_CMAP_RANGE, 11127 },
	{ 0x9869, 0x9869, PDF_CMAP_SINGLE, 1649 },
	{ 0x986a, 0x9870, PDF_CMAP_RANGE, 11130 },
	{ 0x9871, 0x9871, PDF_CMAP_SINGLE, 1650 },
	{ 0x9873, 0x9874, PDF_CMAP_TABLE, 215 },
	{ 0x9875, 0x9877, PDF_CMAP_RANGE, 1652 },
	{ 0x9878, 0x987c, PDF_CMAP_RANGE, 11138 },
	{ 0x987d, 0x987d, PDF_CMAP_SINGLE, 1655 },
	{ 0x9881, 0x9882, PDF_CMAP_RANGE, 1656 },
	{ 0x9883, 0x9884, PDF_CMAP_RANGE, 11143 },
	{ 0x9885, 0x9885, PDF_CMAP_SINGLE, 1658 },
	{ 0x9886, 0x9888, PDF_CMAP_RANGE, 11145 },
	{ 0x9889, 0x9889, PDF_CMAP_SINGLE, 1659 },
	{ 0x988a, 0x9890, PDF_CMAP_RANGE, 11148 },
	{ 0x9891, 0x9891, PDF_CMAP_SINGLE, 1660 },
	{ 0x9893, 0x9894, PDF_CMAP_TABLE, 217 },
	{ 0x9895, 0x9897, PDF_CMAP_RANGE, 1662 },
	{ 0x9898, 0x989d, PDF_CMAP_RANGE, 11156 },
	{ 0x98a1, 0x98b1, PDF_CMAP_RANGE, 11162 },
	{ 0x98b3, 0x98bd, PDF_CMAP_RANGE, 11179 },
	{ 0x98c1, 0x98d1, PDF_CMAP_RANGE, 11190 },
	{ 0x98d3, 0x98dd, PDF_CMAP_RANGE, 11207 },
	{ 0x98e1, 0x98e2, PDF_CMAP_RANGE, 1665 },
	{ 0x98e3, 0x98e4, PDF_CMAP_RANGE, 11218 },
	{ 0x98e5, 0x98e5, PDF_CMAP_SINGLE, 1667 },
	{ 0x98e6, 0x98e8, PDF_CMAP_RANGE, 11220 },
	{ 0x98e9, 0x98ea, PDF_CMAP_TABLE, 219 },
	{ 0x98eb, 0x98ec, PDF_CMAP_RANGE, 1669 },
	{ 0x98ed, 0x98f0, PDF_CMAP_RANGE, 11224 },
	{ 0x98f1, 0x98f1, PDF_CMAP_SINGLE, 1671 },
	{ 0x98f3, 0x98f4, PDF_CMAP_TABLE, 221 },
	{ 0x98f5, 0x98f7, PDF_CMAP_RANGE, 1673 },
	{ 0x98f8, 0x98fc, PDF_CMAP_RANGE, 11229 },
	{ 0x98fd, 0x98fd, PDF_CMAP_SINGLE, 1676 },
	{ 0x9941, 0x9942, PDF_CMAP_RANGE, 1677 },
	{ 0x9943, 0x9944, PDF_CMAP_RANGE, 11234 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 1679 },
	{ 0x9946, 0x9948, PDF_CMAP_RANGE, 11236 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 1680 },
	{ 0x994a, 0x9950, PDF_CMAP_RANGE, 11239 },
	{ 0x9951, 0x9951, PDF_CMAP_SINGLE, 1681 },
	{ 0x9953, 0x9954, PDF_CMAP_TABLE, 223 },
	{ 0x9955, 0x9957, PDF_CMAP_RANGE, 1683 },
	{ 0x9958, 0x995d, PDF_CMAP_RANGE, 11247 },
	{ 0x9961, 0x9961, PDF_CMAP_SINGLE, 1686 },
	{ 0x9962, 0x9971, PDF_CMAP_RANGE, 11253 },
	{ 0x9973, 0x9975, PDF_CMAP_RANGE, 11269 },
	{ 0x9976, 0x9976, PDF_CMAP_SINGLE, 1687 },
	{ 0x9977, 0x997d, PDF_CMAP_RANGE, 11272 },
	{ 0x9981, 0x9991, PDF_CMAP_RANGE, 11279 },
	{ 0x9993, 0x999d, PDF_CMAP_RANGE, 11296 },
	{ 0x99a1, 0x99a2, PDF_CMAP_RANGE, 1688 },
	{ 0x99a3, 0x99a4, PDF_CMAP_RANGE, 11307 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 1690 },
	{ 0x99a6, 0x99a8, PDF_CMAP_RANGE, 11309 },
	{ 0x99a9, 0x99a9, PDF_CMAP_SINGLE, 1691 },
	{ 0x99aa, 0x99b1, PDF_CMAP_RANGE, 11312 },
	{ 0x99b3, 0x99b6, PDF_CMAP_RANGE, 11320 },
	{ 0x99b7, 0x99b7, PDF_CMAP_SINGLE, 1692 },
	{ 0x99b8, 0x99bd, PDF_CMAP_RANGE, 11324 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 1693 },
	{ 0x99c2, 0x99c8, PDF_CMAP_RANGE, 11330 },
	{ 0x99c9, 0x99c9, PDF_CMAP_SINGLE, 1694 },
	{ 0x99ca, 0x99d1, PDF_CMAP_RANGE, 11337 },
	{ 0x99d3, 0x99dd, PDF_CMAP_RANGE, 11345 },
	{ 0x99e1, 0x99e1, PDF_CMAP_SINGLE, 1695 },
	{ 0x99e2, 0x99f1, PDF_CMAP_RANGE, 11356 },
	{ 0x99f3, 0x99fd, PDF_CMAP_RANGE, 11372 },
	{ 0x9a41, 0x9a41, PDF_CMAP_SINGLE, 1696 },
	{ 0x9a42, 0x9a44, PDF_CMAP_RANGE, 11383 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 1697 },
	{ 0x9a46, 0x9a51, PDF_CMAP_RANGE, 11386 },
	{ 0x9a53, 0x9a5d, PDF_CMAP_RANGE, 11398 },
	{ 0x9a61, 0x9a71, PDF_CMAP_RANGE, 11409 },
	{ 0x9a73, 0x9a7d, PDF_CMAP_RANGE, 11426 },
	{ 0x9a81, 0x9a82, PDF_CMAP_RANGE, 1698 },
	{ 0x9a83, 0x9a84, PDF_CMAP_RANGE, 11437 },
	{ 0x9a85, 0x9a85, PDF_CMAP_SINGLE, 1700 },
	{ 0x9a86, 0x9a88, PDF_CMAP_RANGE, 11439 },
	{ 0x9a89, 0x9a89, PDF_CMAP_SINGLE, 1701 },
	{ 0x9a8a, 0x9a8f, PDF_CMAP_RANGE, 11442 },
	{ 0x9a90, 0x9a91, PDF_CMAP_RANGE, 1702 },
	{ 0x9a93, 0x9a96, PDF_CMAP_RANGE, 11448 },
	{ 0x9a97, 0x9a97, PDF_CMAP_SINGLE, 1704 },
	{ 0x9a98, 0x9a9d, PDF_CMAP_RANGE, 11452 },
	{ 0x9aa1, 0x9ab1, PDF_CMAP_RANGE, 11458 },
	{ 0x9ab3, 0x9abd, PDF_CMAP_RANGE, 11475 },
	{ 0x9ac1, 0x9ac1, PDF_CMAP_SINGLE, 1705 },
	{ 0x9ac2, 0x9ad1, PDF_CMAP_RANGE, 11486 },
	{ 0x9ad3, 0x9add, PDF_CMAP_RANGE, 11502 },
	{ 0x9ae1, 0x9ae1, PDF_CMAP_SINGLE, 1706 },
	{ 0x9ae2, 0x9ae4, PDF_CMAP_RANGE, 11513 },
	{ 0x9ae5, 0x9ae5, PDF_CMAP_SINGLE, 1707 },
	{ 0x9ae6, 0x9ae8, PDF_CMAP_RANGE, 11516 },
	{ 0x9ae9, 0x9ae9, PDF_CMAP_SINGLE, 1708 },
	{ 0x9aea, 0x9af0, PDF_CMAP_RANGE, 11519 },
	{ 0x9af1, 0x9af1, PDF_CMAP_SINGLE, 1709 },
	{ 0x9af3, 0x9af3, PDF_CMAP_SINGLE, 1710 },
	{ 0x9af4, 0x9af6, PDF_CMAP_RANGE, 11526 },
	{ 0x9af7, 0x9af7, PDF_CMAP_SINGLE, 1711 },
	{ 0x9af8, 0x9afd, PDF_CMAP_RANGE, 11529 },
	{ 0x9b41, 0x9b51, PDF_CMAP_RANGE, 11535 },
	{ 0x9b53, 0x9b5d, PDF_CMAP_RANGE, 11552 },
	{ 0x9b61, 0x9b62, PDF_CMAP_RANGE, 1712 },
	{ 0x9b63, 0x9b64, PDF_CMAP_RANGE, 11563 },
	{ 0x9b65, 0x9b65, PDF_CMAP_SINGLE, 1714 },
	{ 0x9b66, 0x9b67, PDF_CMAP_RANGE, 11565 },
	{ 0x9b68, 0x9b69, PDF_CMAP_RANGE, 1715 },
	{ 0x9b6a, 0x9b70, PDF_CMAP_RANGE, 11567 },
	{ 0x9b71, 0x9b71, PDF_CMAP_SINGLE, 1717 },
	{ 0x9b73, 0x9b75, PDF_CMAP_TABLE, 225 },
	{ 0x9b76, 0x9b7d, PDF_CMAP_RANGE, 11575 },
	{ 0x9b81, 0x9b81, PDF_CMAP_SINGLE, 1720 },
	{ 0x9b82, 0x9b84, PDF_CMAP_RANGE, 11583 },
	{ 0x9b85, 0x9b85, PDF_CMAP_SINGLE, 1721 },
	{ 0x9b86, 0x9b88, PDF_CMAP_RANGE, 11586 },
	{ 0x9b89, 0x9b89, PDF_CMAP_SINGLE, 1722 },
	{ 0x9b8a, 0x9b90, PDF_CMAP_RANGE, 11589 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 1723 },
	{ 0x9b93, 0x9b93, PDF_CMAP_SINGLE, 1724 },
	{ 0x9b94, 0x9b9d, PDF_CMAP_RANGE, 11596 },
	{ 0x9ba1, 0x9ba1, PDF_CMAP_SINGLE, 1725 },
	{ 0x9ba2, 0x9ba4, PDF_CMAP_RANGE, 11606 },
	{ 0x9ba5, 0x9ba5, PDF_CMAP_SINGLE, 1726 },
	{ 0x9ba6, 0x9ba8, PDF_CMAP_RANGE, 11609 },
	{ 0x9ba9, 0x9ba9, PDF_CMAP_SINGLE, 1727 },
	{ 0x9baa, 0x9bb0, PDF_CMAP_RANGE, 11612 },
	{ 0x9bb1, 0x9bb1, PDF_CMAP_SINGLE, 1728 },
	{ 0x9bb3, 0x9bb7, PDF_CMAP_TABLE, 228 },
	{ 0x9bb8, 0x9bbd, PDF_CMAP_RANGE, 11621 },
	{ 0x9c41, 0x9c41, PDF_CMAP_SINGLE, 366 },
	{ 0x9c61, 0x9c62, PDF_CMAP_RANGE, 1732 },
	{ 0x9c63, 0x9c64, PDF_CMAP_RANGE, 11627 },
	{ 0x9c65, 0x9c65, PDF_CMAP_SINGLE, 1734 },
	{ 0x9c66, 0x9c68, PDF_CMAP_RANGE, 11629 },
	{ 0x9c69, 0x9c69, PDF_CMAP_SINGLE, 1735 },
	{ 0x9c6a, 0x9c70, PDF_CMAP_RANGE, 11632 },
	{ 0x9c71, 0x9c71, PDF_CMAP_SINGLE, 1736 },
	{ 0x9c73, 0x9c74, PDF_CMAP_TABLE, 233 },
	{ 0x9c75, 0x9c78, PDF_CMAP_RANGE, 1738 },
	{ 0x9c79, 0x9c7b, PDF_CMAP_RANGE, 11640 },
	{ 0x9c7c, 0x9c7d, PDF_CMAP_RANGE, 1742 },
	{ 0x9c81, 0x9c82, PDF_CMAP_RANGE, 1744 },
	{ 0x9c83, 0x9c84, PDF_CMAP_RANGE, 11643 },
	{ 0x9c85, 0x9c85, PDF_CMAP_SINGLE, 1746 },
	{ 0x9c86, 0x9c88, PDF_CMAP_RANGE, 11645 },
	{ 0x9c89, 0x9c89, PDF_CMAP_SINGLE, 1747 },
	{ 0x9c8a, 0x9c90, PDF_CMAP_RANGE, 11648 },
	{ 0x9c91, 0x9c91, PDF_CMAP_SINGLE, 1748 },
	{ 0x9c93, 0x9c94, PDF_CMAP_TABLE, 235 },
	{ 0x9c95, 0x9c97, PDF_CMAP_RANGE, 1750 },
	{ 0x9c98, 0x9c9d, PDF_CMAP_RANGE, 11656 },
	{ 0x9ca1, 0x9ca2, PDF_CMAP_RANGE, 1753 },
	{ 0x9ca3, 0x9ca4, PDF_CMAP_RANGE, 11662 },
	{ 0x9ca5, 0x9ca5, PDF_CMAP_SINGLE, 1755 },
	{ 0x9ca6, 0x9cb1, PDF_CMAP_RANGE, 11664 },
	{ 0x9cb3, 0x9cb4, PDF_CMAP_RANGE, 11676 },
	{ 0x9cb5, 0x9cb7, PDF_CMAP_TABLE, 237 },
	{ 0x9cb8, 0x9cbd, PDF_CMAP_RANGE, 11679 },
	{ 0x9cc1, 0x9cd1, PDF_CMAP_RANGE, 11685 },
	{ 0x9cd3, 0x9cdd, PDF_CMAP_RANGE, 11702 },
	{ 0x9ce1, 0x9ce2, PDF_CMAP_RANGE, 1758 },
	{ 0x9ce3, 0x9ce4, PDF_CMAP_RANGE, 11713 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 1760 },
	{ 0x9ce6, 0x9ce8, PDF_CMAP_RANGE, 11715 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 1761 },
	{ 0x9cea, 0x9cf0, PDF_CMAP_RANGE, 11718 },
	{ 0x9cf1, 0x9cf1, PDF_CMAP_SINGLE, 1762 },
	{ 0x9cf3, 0x9cf4, PDF_CMAP_TABLE, 240 },
	{ 0x9cf5, 0x9cf7, PDF_CMAP_RANGE, 1764 },
	{ 0x9cf8, 0x9cfc, PDF_CMAP_RANGE, 11726 },
	{ 0x9cfd, 0x9cfd, PDF_CMAP_SINGLE, 1767 },
	{ 0x9d41, 0x9d42, PDF_CMAP_RANGE, 1768 },
	{ 0x9d43, 0x9d44, PDF_CMAP_RANGE, 11731 },
	{ 0x9d45, 0x9d45, PDF_CMAP_SINGLE, 1770 },
	{ 0x9d46, 0x9d48, PDF_CMAP_RANGE, 11733 },
	{ 0x9d49, 0x9d49, PDF_CMAP_SINGLE, 1771 },
	{ 0x9d4a, 0x9d50, PDF_CMAP_RANGE, 11736 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 1772 },
	{ 0x9d53, 0x9d57, PDF_CMAP_TABLE, 242 },
	{ 0x9d58, 0x9d5d, PDF_CMAP_RANGE, 11745 },
	{ 0x9d61, 0x9d62, PDF_CMAP_RANGE, 1776 },
	{ 0x9d63, 0x9d64, PDF_CMAP_RANGE, 11751 },
	{ 0x9d65, 0x9d65, PDF_CMAP_SINGLE, 1778 },
	{ 0x9d66, 0x9d68, PDF_CMAP_RANGE, 11753 },
	{ 0x9d69, 0x9d69, PDF_CMAP_SINGLE, 1779 },
	{ 0x9d6a, 0x9d70, PDF_CMAP_RANGE, 11756 },
	{ 0x9d71, 0x9d71, PDF_CMAP_SINGLE, 1780 },
	{ 0x9d73, 0x9d74, PDF_CMAP_TABLE, 247 },
	{ 0x9d75, 0x9d77, PDF_CMAP_RANGE, 1782 },
	{ 0x9d78, 0x9d7d, PDF_CMAP_RANGE, 11764 },
	{ 0x9d81, 0x9d81, PDF_CMAP_SINGLE, 1785 },
	{ 0x9d82, 0x9d84, PDF_CMAP_RANGE, 11770 },
	{ 0x9d85, 0x9d85, PDF_CMAP_SINGLE, 1786 },
	{ 0x9d86, 0x9d91, PDF_CMAP_RANGE, 11773 },
	{ 0x9d93, 0x9d95, PDF_CMAP_TABLE, 249 },
	{ 0x9d96, 0x9d9d, PDF_CMAP_RANGE, 11786 },
	{ 0x9da1, 0x9da2, PDF_CMAP_RANGE, 1789 },
	{ 0x9da3, 0x9da4, PDF_CMAP_RANGE, 11794 },
	{ 0x9da5, 0x9da5, PDF_CMAP_SINGLE, 1791 },
	{ 0x9da6, 0x9da8, PDF_CMAP_RANGE, 11796 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 1792 },
	{ 0x9daa, 0x9db0, PDF_CMAP_RANGE, 11799 },
	{ 0x9db1, 0x9db1, PDF_CMAP_SINGLE, 1793 },
	{ 0x9db3, 0x9db7, PDF_CMAP_TABLE, 252 },
	{ 0x9db8, 0x9dbd, PDF_CMAP_RANGE, 11808 },
	{ 0x9dc1, 0x9dc1, PDF_CMAP_SINGLE, 1797 },
	{ 0x9dc2, 0x9dc4, PDF_CMAP_RANGE, 11814 },
	{ 0x9dc5, 0x9dc5, PDF_CMAP_SINGLE, 1798 },
	{ 0x9dc6, 0x9dd1, PDF_CMAP_RANGE, 11817 },
	{ 0x9dd3, 0x9dd6, PDF_CMAP_RANGE, 11829 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 1799 },
	{ 0x9dd8, 0x9ddd, PDF_CMAP_RANGE, 11833 },
	{ 0x9de1, 0x9df1, PDF_CMAP_RANGE, 11839 },
	{ 0x9df3, 0x9df5, PDF_CMAP_RANGE, 11856 },
	{ 0x9df6, 0x9df6, PDF_CMAP_SINGLE, 1800 },
	{ 0x9df7, 0x9dfd, PDF_CMAP_RANGE, 11859 },
	{ 0x9e41, 0x9e41, PDF_CMAP_SINGLE, 1801 },
	{ 0x9e42, 0x9e44, PDF_CMAP_RANGE, 11866 },
	{ 0x9e45, 0x9e45, PDF_CMAP_SINGLE, 1802 },
	{ 0x9e46, 0x9e48, PDF_CMAP_RANGE, 11869 },
	{ 0x9e49, 0x9e49, PDF_CMAP_SINGLE, 1803 },
	{ 0x9e4a, 0x9e50, PDF_CMAP_RANGE, 11872 },
	{ 0x9e51, 0x9e51, PDF_CMAP_SINGLE, 1804 },
	{ 0x9e53, 0x9e57, PDF_CMAP_TABLE, 257 },
	{ 0x9e58, 0x9e5d, PDF_CMAP_RANGE, 11881 },
	{ 0x9e61, 0x9e61, PDF_CMAP_SINGLE, 1808 },
	{ 0x9e62, 0x9e64, PDF_CMAP_RANGE, 11887 },
	{ 0x9e65, 0x9e65, PDF_CMAP_SINGLE, 1809 },
	{ 0x9e66, 0x9e68, PDF_CMAP_RANGE, 11890 },
	{ 0x9e69, 0x9e69, PDF_CMAP_SINGLE, 1810 },
	{ 0x9e6a, 0x9e71, PDF_CMAP_RANGE, 11893 },
	{ 0x9e73, 0x9e77, PDF_CMAP_TABLE, 262 },
	{ 0x9e78, 0x9e7d, PDF_CMAP_RANGE, 11903 },
	{ 0x9e81, 0x9e82, PDF_CMAP_RANGE, 1814 },
	{ 0x9e83, 0x9e84, PDF_CMAP_RANGE, 11909 },
	{ 0x9e85, 0x9e85, PDF_CMAP_SINGLE, 1816 },
	{ 0x9e86, 0x9e88, PDF_CMAP_RANGE, 11911 },
	{ 0x9e89, 0x9e89, PDF_CMAP_SINGLE, 1817 },
	{ 0x9e8a, 0x9e90, PDF_CMAP_RANGE, 11914 },
	{ 0x9e91, 0x9e91, PDF_CMAP_SINGLE, 1818 },
	{ 0x9e93, 0x9e97, PDF_CMAP_TABLE, 267 },
	{ 0x9e98, 0x9e9d, PDF_CMAP_RANGE, 11923 },
	{ 0x9ea1, 0x9ea1, PDF_CMAP_SINGLE, 1822 },
	{ 0x9ea2, 0x9eb1, PDF_CMAP_RANGE, 11929 },
	{ 0x9eb3, 0x9eb5, PDF_CMAP_RANGE, 11945 },
	{ 0x9eb6, 0x9eb6, PDF_CMAP_SINGLE, 1823 },
	{ 0x9eb7, 0x9ebd, PDF_CMAP_RANGE, 11948 },
	{ 0x9ec1, 0x9ec1, PDF_CMAP_SINGLE, 1824 },
	{ 0x9ec2, 0x9ed1, PDF_CMAP_RANGE, 11955 },
	{ 0x9ed3, 0x9edd, PDF_CMAP_RANGE, 11971 },
	{ 0x9ee1, 0x9ee2, PDF_CMAP_RANGE, 1825 },
	{ 0x9ee3, 0x9ee4, PDF_CMAP_RANGE, 11982 },
	{ 0x9ee5, 0x9ee5, PDF_CMAP_SINGLE, 1827 },
	{ 0x9ee6, 0x9ee8, PDF_CMAP_RANGE, 11984 },
	{ 0x9ee9, 0x9ee9, PDF_CMAP_SINGLE, 1828 },
	{ 0x9eea, 0x9ef0, PDF_CMAP_RANGE, 11987 },
	{ 0x9ef1, 0x9ef1, PDF_CMAP_SINGLE, 1829 },
	{ 0x9ef3, 0x9ef4, PDF_CMAP_RANGE, 11994 },
	{ 0x9ef5, 0x9ef7, PDF_CMAP_TABLE, 272 },
	{ 0x9ef8, 0x9efd, PDF_CMAP_RANGE, 11997 },
	{ 0x9f41, 0x9f42, PDF_CMAP_RANGE, 1832 },
	{ 0x9f43, 0x9f44, PDF_CMAP_RANGE, 12003 },
	{ 0x9f45, 0x9f45, PDF_CMAP_SINGLE, 1834 },
	{ 0x9f46, 0x9f48, PDF_CMAP_RANGE, 12005 },
	{ 0x9f49, 0x9f49, PDF_CMAP_SINGLE, 1835 },
	{ 0x9f4a, 0x9f50, PDF_CMAP_RANGE, 12008 },
	{ 0x9f51, 0x9f51, PDF_CMAP_SINGLE, 1836 },
	{ 0x9f53, 0x9f57, PDF_CMAP_TABLE, 275 },
	{ 0x9f58, 0x9f5d, PDF_CMAP_RANGE, 12017 },
	{ 0x9f61, 0x9f62, PDF_CMAP_RANGE, 1840 },
	{ 0x9f63, 0x9f64, PDF_CMAP_RANGE, 12023 },
	{ 0x9f65, 0x9f65, PDF_CMAP_SINGLE, 1842 },
	{ 0x9f66, 0x9f68, PDF_CMAP_RANGE, 12025 },
	{ 0x9f69, 0x9f69, PDF_CMAP_SINGLE, 1843 },
	{ 0x9f6a, 0x9f70, PDF_CMAP_RANGE, 12028 },
	{ 0x9f71, 0x9f71, PDF_CMAP_SINGLE, 1844 },
	{ 0x9f73, 0x9f76, PDF_CMAP_TABLE, 280 },
	{ 0x9f77, 0x9f78, PDF_CMAP_RANGE, 1847 },
	{ 0x9f79, 0x9f7a, PDF_CMAP_RANGE, 12037 },
	{ 0x9f7b, 0x9f7c, PDF_CMAP_RANGE, 1849 },
	{ 0x9f7d, 0x9f7d, PDF_CMAP_SINGLE, 12039 },
	{ 0x9f81, 0x9f91, PDF_CMAP_RANGE, 12040 },
	{ 0x9f93, 0x9f9d, PDF_CMAP_RANGE, 12057 },
	{ 0x9fa1, 0x9fa2, PDF_CMAP_RANGE, 1851 },
	{ 0x9fa3, 0x9fa4, PDF_CMAP_RANGE, 12068 },
	{ 0x9fa5, 0x9fa5, PDF_CMAP_SINGLE, 1853 },
	{ 0x9fa6, 0x9fa8, PDF_CMAP_RANGE, 12070 },
	{ 0x9fa9, 0x9fa9, PDF_CMAP_SINGLE, 1854 },
	{ 0x9faa, 0x9fb0, PDF_CMAP_RANGE, 12073 },
	{ 0x9fb1, 0x9fb1, PDF_CMAP_SINGLE, 1855 },
	{ 0x9fb3, 0x9fb7, PDF_CMAP_TABLE, 284 },
	{ 0x9fb8, 0x9fbd, PDF_CMAP_RANGE, 12082 },
	{ 0xa041, 0xa041, PDF_CMAP_SINGLE, 374 },
	{ 0xa061, 0xa062, PDF_CMAP_RANGE, 1859 },
	{ 0xa063, 0xa064, PDF_CMAP_RANGE, 12088 },
	{ 0xa065, 0xa066, PDF_CMAP_TABLE, 289 },
	{ 0xa067, 0xa06b, PDF_CMAP_RANGE, 1862 },
	{ 0xa06c, 0xa070, PDF_CMAP_RANGE, 12091 },
	{ 0xa071, 0xa071, PDF_CMAP_SINGLE, 1867 },
	{ 0xa073, 0xa076, PDF_CMAP_TABLE, 291 },
	{ 0xa077, 0xa078, PDF_CMAP_RANGE, 1870 },
	{ 0xa079, 0xa07a, PDF_CMAP_RANGE, 12098 },
	{ 0xa07b, 0xa07d, PDF_CMAP_TABLE, 295 },
	{ 0xa081, 0xa082, PDF_CMAP_RANGE, 1874 },
	{ 0xa083, 0xa084, PDF_CMAP_RANGE, 12101 },
	{ 0xa085, 0xa085, PDF_CMAP_SINGLE, 1876 },
	{ 0xa086, 0xa088, PDF_CMAP_RANGE, 12103 },
	{ 0xa089, 0xa089, PDF_CMAP_SINGLE, 1877 },
	{ 0xa08a, 0xa090, PDF_CMAP_RANGE, 12106 },
	{ 0xa091, 0xa091, PDF_CMAP_SINGLE, 1878 },
	{ 0xa093, 0xa094, PDF_CMAP_TABLE, 298 },
	{ 0xa095, 0xa098, PDF_CMAP_RANGE, 1880 },
	{ 0xa099, 0xa09d, PDF_CMAP_RANGE, 12114 },
	{ 0xa0a1, 0xa0a2, PDF_CMAP_RANGE, 1884 },
	{ 0xa0a3, 0xa0a8, PDF_CMAP_RANGE, 12119 },
	{ 0xa0a9, 0xa0a9, PDF_CMAP_SINGLE, 1886 },
	{ 0xa0aa, 0xa0b1, PDF_CMAP_RANGE, 12125 },
	{ 0xa0b3, 0xa0b6, PDF_CMAP_RANGE, 12133 },
	{ 0xa0b7, 0xa0b7, PDF_CMAP_SINGLE, 1887 },
	{ 0xa0b8, 0xa0bd, PDF_CMAP_RANGE, 12137 },
	{ 0xa0c1, 0xa0d1, PDF_CMAP_RANGE, 12143 },
	{ 0xa0d3, 0xa0dd, PDF_CMAP_RANGE, 12160 },
	{ 0xa0e1, 0xa0e2, PDF_CMAP_RANGE, 1888 },
	{ 0xa0e3, 0xa0e4, PDF_CMAP_RANGE, 12171 },
	{ 0xa0e5, 0xa0e5, PDF_CMAP_SINGLE, 1890 },
	{ 0xa0e6, 0xa0e8, PDF_CMAP_RANGE, 12173 },
	{ 0xa0e9, 0xa0eb, PDF_CMAP_TABLE, 300 },
	{ 0xa0ec, 0xa0f0, PDF_CMAP_RANGE, 12177 },
	{ 0xa0f1, 0xa0f1, PDF_CMAP_SINGLE, 1893 },
	{ 0xa0f3, 0xa0f6, PDF_CMAP_TABLE, 303 },
	{ 0xa0f7, 0xa0f8, PDF_CMAP_RANGE, 1896 },
	{ 0xa0f9, 0xa0fc, PDF_CMAP_RANGE, 12184 },
	{ 0xa0fd, 0xa0fd, PDF_CMAP_SINGLE, 1898 },
	{ 0xa141, 0xa142, PDF_CMAP_RANGE, 1899 },
	{ 0xa143, 0xa144, PDF_CMAP_RANGE, 12188 },
	{ 0xa145, 0xa145, PDF_CMAP_SINGLE, 1901 },
	{ 0xa146, 0xa148, PDF_CMAP_RANGE, 12190 },
	{ 0xa149, 0xa149, PDF_CMAP_SINGLE, 1902 },
	{ 0xa14a, 0xa150, PDF_CMAP_RANGE, 12193 },
	{ 0xa151, 0xa151, PDF_CMAP_SINGLE, 1903 },
	{ 0xa153, 0xa154, PDF_CMAP_TABLE, 307 },
	{ 0xa155, 0xa157, PDF_CMAP_RANGE, 1905 },
	{ 0xa158, 0xa15d, PDF_CMAP_RANGE, 12201 },
	{ 0xa161, 0xa162, PDF_CMAP_RANGE, 1908 },
	{ 0xa163, 0xa164, PDF_CMAP_RANGE, 12207 },
	{ 0xa165, 0xa165, PDF_CMAP_SINGLE, 1910 },
	{ 0xa166, 0xa168, PDF_CMAP_RANGE, 12209 },
	{ 0xa169, 0xa169, PDF_CMAP_SINGLE, 1911 },
	{ 0xa16a, 0xa171, PDF_CMAP_RANGE, 12212 },
	{ 0xa173, 0xa174, PDF_CMAP_RANGE, 12220 },
	{ 0xa175, 0xa177, PDF_CMAP_RANGE, 1912 },
	{ 0xa178, 0xa179, PDF_CMAP_TABLE, 309 },
	{ 0xa17a, 0xa17d, PDF_CMAP_RANGE, 12223 },
	{ 0xa181, 0xa181, PDF_CMAP_SINGLE, 1916 },
	{ 0xa182, 0xa191, PDF_CMAP_RANGE, 12227 },
	{ 0xa193, 0xa19d, PDF_CMAP_RANGE, 12243 },
	{ 0xa1a1, 0xa1a2, PDF_CMAP_RANGE, 1917 },
	{ 0xa1a3, 0xa1a3, PDF_CMAP_SINGLE, 12254 },
	{ 0xa1a4, 0xa1a5, PDF_CMAP_RANGE, 1919 },
	{ 0xa1a6, 0xa1a8, PDF_CMAP_RANGE, 12255 },
	{ 0xa1a9, 0xa1ab, PDF_CMAP_TABLE, 311 },
	{ 0xa1ac, 0xa1b0, PDF_CMAP_RANGE, 12259 },
	{ 0xa1b1, 0xa1b1, PDF_CMAP_SINGLE, 1923 },
	{ 0xa1b3, 0xa1b7, PDF_CMAP_TABLE, 314 },
	{ 0xa1b8, 0xa1bd, PDF_CMAP_RANGE, 12266 },
	{ 0xa1c1, 0xa1c1, PDF_CMAP_SINGLE, 1927 },
	{ 0xa1c2, 0xa1c4, PDF_CMAP_RANGE, 12272 },
	{ 0xa1c5, 0xa1c5, PDF_CMAP_SINGLE, 1928 },
	{ 0xa1c6, 0xa1d1, PDF_CMAP_RANGE, 12275 },
	{ 0xa1d3, 0xa1d5, PDF_CMAP_RANGE, 12287 },
	{ 0xa1d6, 0xa1d7, PDF_CMAP_RANGE, 1929 },
	{ 0xa1d8, 0xa1dd, PDF_CMAP_RANGE, 12290 },
	{ 0xa1e1, 0xa1f1, PDF_CMAP_RANGE, 12296 },
	{ 0xa1f3, 0xa1fd, PDF_CMAP_RANGE, 12313 },
	{ 0xa241, 0xa241, PDF_CMAP_SINGLE, 1931 },
	{ 0xa242, 0xa244, PDF_CMAP_RANGE, 12324 },
	{ 0xa245, 0xa245, PDF_CMAP_SINGLE, 1932 },
	{ 0xa246, 0xa248, PDF_CMAP_RANGE, 12327 },
	{ 0xa249, 0xa249, PDF_CMAP_SINGLE, 1933 },
	{ 0xa24a, 0xa251, PDF_CMAP_RANGE, 12330 },
	{ 0xa253, 0xa257, PDF_CMAP_TABLE, 319 },
	{ 0xa258, 0xa25d, PDF_CMAP_RANGE, 12340 },
	{ 0xa261, 0xa261, PDF_CMAP_SINGLE, 1937 },
	{ 0xa262, 0xa264, PDF_CMAP_RANGE, 12346 },
	{ 0xa265, 0xa265, PDF_CMAP_SINGLE, 1938 },
	{ 0xa266, 0xa268, PDF_CMAP_RANGE, 12349 },
	{ 0xa269, 0xa269, PDF_CMAP_SINGLE, 1939 },
	{ 0xa26a, 0xa271, PDF_CMAP_RANGE, 12352 },
	{ 0xa273, 0xa275, PDF_CMAP_TABLE, 324 },
	{ 0xa276, 0xa27d, PDF_CMAP_RANGE, 12361 },
	{ 0xa281, 0xa283, PDF_CMAP_RANGE, 1942 },
	{ 0xa284, 0xa285, PDF_CMAP_TABLE, 327 },
	{ 0xa286, 0xa287, PDF_CMAP_RANGE, 12370 },
	{ 0xa288, 0xa28b, PDF_CMAP_RANGE, 1946 },
	{ 0xa28c, 0xa290, PDF_CMAP_RANGE, 12372 },
	{ 0xa291, 0xa291, PDF_CMAP_SINGLE, 1950 },
	{ 0xa293, 0xa297, PDF_CMAP_TABLE, 329 },
	{ 0xa298, 0xa29a, PDF_CMAP_RANGE, 12379 },
	{ 0xa29b, 0xa29d, PDF_CMAP_TABLE, 334 },
	{ 0xa2a1, 0xa2a1, PDF_CMAP_SINGLE, 1956 },
	{ 0xa2a2, 0xa2a4, PDF_CMAP_RANGE, 12383 },
	{ 0xa2a5, 0xa2a5, PDF_CMAP_SINGLE, 1957 },
	{ 0xa2a6, 0xa2a8, PDF_CMAP_RANGE, 12386 },
	{ 0xa2a9, 0xa2a9, PDF_CMAP_SINGLE, 1958 },
	{ 0xa2aa, 0xa2b1, PDF_CMAP_RANGE, 12389 },
	{ 0xa2b3, 0xa2b5, PDF_CMAP_TABLE, 337 },
	{ 0xa2b6, 0xa2bd, PDF_CMAP_RANGE, 12398 },
	{ 0xa2c1, 0xa2c1, PDF_CMAP_SINGLE, 1961 },
	{ 0xa2c2, 0xa2d1, PDF_CMAP_RANGE, 12406 },
	{ 0xa2d3, 0xa2dd, PDF_CMAP_RANGE, 12422 },
	{ 0xa2e1, 0xa2e1, PDF_CMAP_SINGLE, 1962 },
	{ 0xa2e2, 0xa2e4, PDF_CMAP_RANGE, 12433 },
	{ 0xa2e5, 0xa2e5, PDF_CMAP_SINGLE, 1963 },
	{ 0xa2e6, 0xa2e8, PDF_CMAP_RANGE, 12436 },
	{ 0xa2e9, 0xa2e9, PDF_CMAP_SINGLE, 1964 },
	{ 0xa2ea, 0xa2f1, PDF_CMAP_RANGE, 12439 },
	{ 0xa2f3, 0xa2fd, PDF_CMAP_RANGE, 12447 },
	{ 0xa341, 0xa341, PDF_CMAP_SINGLE, 1965 },
	{ 0xa342, 0xa344, PDF_CMAP_RANGE, 12458 },
	{ 0xa345, 0xa345, PDF_CMAP_SINGLE, 1966 },
	{ 0xa346, 0xa348, PDF_CMAP_RANGE, 12461 },
	{ 0xa349, 0xa349, PDF_CMAP_SINGLE, 1967 },
	{ 0xa34a, 0xa350, PDF_CMAP_RANGE, 12464 },
	{ 0xa351, 0xa351, PDF_CMAP_SINGLE, 1968 },
	{ 0xa353, 0xa354, PDF_CMAP_RANGE, 12471 },
	{ 0xa355, 0xa355, PDF_CMAP_SINGLE, 1969 },
	{ 0xa356, 0xa35d, PDF_CMAP_RANGE, 12473 },
	{ 0xa361, 0xa361, PDF_CMAP_SINGLE, 1970 },
	{ 0xa362, 0xa364, PDF_CMAP_RANGE, 12481 },
	{ 0xa365, 0xa365, PDF_CMAP_SINGLE, 1971 },
	{ 0xa366, 0xa368, PDF_CMAP_RANGE, 12484 },
	{ 0xa369, 0xa369, PDF_CMAP_SINGLE, 1972 },
	{ 0xa36a, 0xa370, PDF_CMAP_RANGE, 12487 },
	{ 0xa371, 0xa371, PDF_CMAP_SINGLE, 1973 },
	{ 0xa373, 0xa374, PDF_CMAP_RANGE, 12494 },
	{ 0xa375, 0xa375, PDF_CMAP_SINGLE, 1974 },
	{ 0xa376, 0xa37d, PDF_CMAP_RANGE, 12496 },
	{ 0xa381, 0xa391, PDF_CMAP_RANGE, 12504 },
	{ 0xa393, 0xa39d, PDF_CMAP_RANGE, 12521 },
	{ 0xa3a1, 0xa3a2, PDF_CMAP_RANGE, 1975 },
	{ 0xa3a3, 0xa3a4, PDF_CMAP_RANGE, 12532 },
	{ 0xa3a5, 0xa3a5, PDF_CMAP_SINGLE, 1977 },
	{ 0xa3a6, 0xa3a7, PDF_CMAP_RANGE, 12534 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 1978 },
	{ 0xa3aa, 0xa3ab, PDF_CMAP_TABLE, 340 },
	{ 0xa3ac, 0xa3b0, PDF_CMAP_RANGE, 12537 },
	{ 0xa3b1, 0xa3b1, PDF_CMAP_SINGLE, 1981 },
	{ 0xa3b3, 0xa3b4, PDF_CMAP_TABLE, 342 },
	{ 0xa3b5, 0xa3b7, PDF_CMAP_RANGE, 1983 },
	{ 0xa3b8, 0xa3bb, PDF_CMAP_TABLE, 344 },
	{ 0xa3bc, 0xa3bd, PDF_CMAP_RANGE, 12545 },
	{ 0xa441, 0xa441, PDF_CMAP_SINGLE, 375 },
	{ 0xa461, 0xa465, PDF_CMAP_RANGE, 1988 },
	{ 0xa466, 0xa467, PDF_CMAP_RANGE, 12547 },
	{ 0xa468, 0xa46c, PDF_CMAP_RANGE, 1993 },
	{ 0xa46d, 0xa470, PDF_CMAP_RANGE, 12549 },
	{ 0xa471, 0xa471, PDF_CMAP_SINGLE, 1998 },
	{ 0xa473, 0xa477, PDF_CMAP_TABLE, 348 },
	{ 0xa478, 0xa47a, PDF_CMAP_RANGE, 12555 },
	{ 0xa47b, 0xa47b, PDF_CMAP_SINGLE, 2002 },
	{ 0xa47c, 0xa47d, PDF_CMAP_RANGE, 12558 },
	{ 0xa481, 0xa482, PDF_CMAP_RANGE, 2003 },
	{ 0xa483, 0xa484, PDF_CMAP_RANGE, 12560 },
	{ 0xa485, 0xa485, PDF_CMAP_SINGLE, 2005 },
	{ 0xa486, 0xa488, PDF_CMAP_RANGE, 12562 },
	{ 0xa489, 0xa489, PDF_CMAP_SINGLE, 2006 },
	{ 0xa48a, 0xa490, PDF_CMAP_RANGE, 12565 },
	{ 0xa491, 0xa491, PDF_CMAP_SINGLE, 2007 },
	{ 0xa493, 0xa494, PDF_CMAP_TABLE, 353 },
	{ 0xa495, 0xa497, PDF_CMAP_RANGE, 2009 },
	{ 0xa498, 0xa49a, PDF_CMAP_RANGE, 12573 },
	{ 0xa49b, 0xa49b, PDF_CMAP_SINGLE, 2012 },
	{ 0xa49c, 0xa49d, PDF_CMAP_RANGE, 12576 },
	{ 0xa4a1, 0xa4a2, PDF_CMAP_RANGE, 2013 },
	{ 0xa4a3, 0xa4a4, PDF_CMAP_RANGE, 12578 },
	{ 0xa4a5, 0xa4a5, PDF_CMAP_SINGLE, 2015 },
	{ 0xa4a6, 0xa4b1, PDF_CMAP_RANGE, 12580 },
	{ 0xa4b3, 0xa4b3, PDF_CMAP_SINGLE, 2016 },
	{ 0xa4b4, 0xa4bd, PDF_CMAP_RANGE, 12592 },
	{ 0xa4c1, 0xa4d1, PDF_CMAP_RANGE, 12602 },
	{ 0xa4d3, 0xa4dd, PDF_CMAP_RANGE, 12619 },
	{ 0xa4e1, 0xa4e2, PDF_CMAP_RANGE, 2017 },
	{ 0xa4e3, 0xa4e4, PDF_CMAP_RANGE, 12630 },
	{ 0xa4e5, 0xa4e5, PDF_CMAP_SINGLE, 2019 },
	{ 0xa4e6, 0xa4e7, PDF_CMAP_RANGE, 12632 },
	{ 0xa4e8, 0xa4e9, PDF_CMAP_RANGE, 2020 },
	{ 0xa4ea, 0xa4eb, PDF_CMAP_TABLE, 355 },
	{ 0xa4ec, 0xa4f0, PDF_CMAP_RANGE, 12635 },
	{ 0xa4f1, 0xa4f1, PDF_CMAP_SINGLE, 2023 },
	{ 0xa4f3, 0xa4f6, PDF_CMAP_TABLE, 357 },
	{ 0xa4f7, 0xa4f8, PDF_CMAP_RANGE, 2026 },
	{ 0xa4f9, 0xa4fd, PDF_CMAP_RANGE, 12642 },
	{ 0xa541, 0xa542, PDF_CMAP_RANGE, 2028 },
	{ 0xa543, 0xa544, PDF_CMAP_RANGE, 12647 },
	{ 0xa545, 0xa545, PDF_CMAP_SINGLE, 2030 },
	{ 0xa546, 0xa547, PDF_CMAP_RANGE, 12649 },
	{ 0xa548, 0xa549, PDF_CMAP_RANGE, 2031 },
	{ 0xa54a, 0xa550, PDF_CMAP_RANGE, 12651 },
	{ 0xa551, 0xa551, PDF_CMAP_SINGLE, 2033 },
	{ 0xa553, 0xa554, PDF_CMAP_TABLE, 361 },
	{ 0xa555, 0xa557, PDF_CMAP_RANGE, 2035 },
	{ 0xa558, 0xa55d, PDF_CMAP_RANGE, 12659 },
	{ 0xa561, 0xa562, PDF_CMAP_RANGE, 2038 },
	{ 0xa563, 0xa564, PDF_CMAP_RANGE, 12665 },
	{ 0xa565, 0xa565, PDF_CMAP_SINGLE, 2040 },
	{ 0xa566, 0xa568, PDF_CMAP_RANGE, 12667 },
	{ 0xa569, 0xa569, PDF_CMAP_SINGLE, 2041 },
	{ 0xa56a, 0xa571, PDF_CMAP_RANGE, 12670 },
	{ 0xa573, 0xa574, PDF_CMAP_TABLE, 363 },
	{ 0xa575, 0xa577, PDF_CMAP_RANGE, 2043 },
	{ 0xa578, 0xa57a, PDF_CMAP_RANGE, 12679 },
	{ 0xa57b, 0xa57b, PDF_CMAP_SINGLE, 2046 },
	{ 0xa57c, 0xa57d, PDF_CMAP_RANGE, 12682 },
	{ 0xa581, 0xa581, PDF_CMAP_SINGLE, 2047 },
	{ 0xa582, 0xa584, PDF_CMAP_RANGE, 12684 },
	{ 0xa585, 0xa585, PDF_CMAP_SINGLE, 2048 },
	{ 0xa586, 0xa591, PDF_CMAP_RANGE, 12687 },
	{ 0xa593, 0xa59d, PDF_CMAP_RANGE, 12699 },
	{ 0xa5a1, 0xa5a3, PDF_CMAP_RANGE, 2049 },
	{ 0xa5a4, 0xa5a5, PDF_CMAP_TABLE, 365 },
	{ 0xa5a6, 0xa5a8, PDF_CMAP_RANGE, 12711 },
	{ 0xa5a9, 0xa5a9, PDF_CMAP_SINGLE, 2053 },
	{ 0xa5aa, 0xa5b0, PDF_CMAP_RANGE, 12714 },
	{ 0xa5b1, 0xa5b1, PDF_CMAP_SINGLE, 2054 },
	{ 0xa5b3, 0xa5b7, PDF_CMAP_TABLE, 367 },
	{ 0xa5b8, 0xa5bd, PDF_CMAP_RANGE, 12723 },
	{ 0xa5c1, 0xa5c1, PDF_CMAP_SINGLE, 2058 },
	{ 0xa5c2, 0xa5c4, PDF_CMAP_RANGE, 12729 },
	{ 0xa5c5, 0xa5c5, PDF_CMAP_SINGLE, 2059 },
	{ 0xa5c6, 0xa5d1, PDF_CMAP_RANGE, 12732 },
	{ 0xa5d3, 0xa5d5, PDF_CMAP_RANGE, 12744 },
	{ 0xa5d6, 0xa5d6, PDF_CMAP_SINGLE, 2060 },
	{ 0xa5d7, 0xa5dd, PDF_CMAP_RANGE, 12747 },
	{ 0xa5e1, 0xa5e1, PDF_CMAP_SINGLE, 2061 },
	{ 0xa5e2, 0xa5f1, PDF_CMAP_RANGE, 12754 },
	{ 0xa5f3, 0xa5f5, PDF_CMAP_RANGE, 12770 },
	{ 0xa5f6, 0xa5f6, PDF_CMAP_SINGLE, 2062 },
	{ 0xa5f7, 0xa5fd, PDF_CMAP_RANGE, 12773 },
	{ 0xa641, 0xa642, PDF_CMAP_RANGE, 2063 },
	{ 0xa643, 0xa644, PDF_CMAP_RANGE, 12780 },
	{ 0xa645, 0xa645, PDF_CMAP_SINGLE, 2065 },
	{ 0xa646, 0xa648, PDF_CMAP_RANGE, 12782 },
	{ 0xa649, 0xa649, PDF_CMAP_SINGLE, 2066 },
	{ 0xa64a, 0xa650, PDF_CMAP_RANGE, 12785 },
	{ 0xa651, 0xa651, PDF_CMAP_SINGLE, 2067 },
	{ 0xa653, 0xa653, PDF_CMAP_SINGLE, 2068 },
	{ 0xa654, 0xa65d, PDF_CMAP_RANGE, 12792 },
	{ 0xa661, 0xa661, PDF_CMAP_SINGLE, 2069 },
	{ 0xa662, 0xa664, PDF_CMAP_RANGE, 12802 },
	{ 0xa665, 0xa665, PDF_CMAP_SINGLE, 2070 },
	{ 0xa666, 0xa671, PDF_CMAP_RANGE, 12805 },
	{ 0xa673, 0xa67d, PDF_CMAP_RANGE, 12817 },
	{ 0xa681, 0xa682, PDF_CMAP_RANGE, 2071 },
	{ 0xa683, 0xa684, PDF_CMAP_RANGE, 12828 },
	{ 0xa685, 0xa685, PDF_CMAP_SINGLE, 2073 },
	{ 0xa686, 0xa687, PDF_CMAP_RANGE, 12830 },
	{ 0xa688, 0xa68b, PDF_CMAP_RANGE, 2074 },
	{ 0xa68c, 0xa690, PDF_CMAP_RANGE, 12832 },
	{ 0xa691, 0xa691, PDF_CMAP_SINGLE, 2078 },
	{ 0xa693, 0xa697, PDF_CMAP_TABLE, 372 },
	{ 0xa698, 0xa69a, PDF_CMAP_RANGE, 12839 },
	{ 0xa69b, 0xa69c, PDF_CMAP_RANGE, 2082 },
	{ 0xa69d, 0xa69d, PDF_CMAP_SINGLE, 12842 },
	{ 0xa6a1, 0xa6a1, PDF_CMAP_SINGLE, 2084 },
	{ 0xa6a2, 0xa6a8, PDF_CMAP_RANGE, 12843 },
	{ 0xa6a9, 0xa6a9, PDF_CMAP_SINGLE, 2085 },
	{ 0xa6aa, 0xa6b1, PDF_CMAP_RANGE, 12850 },
	{ 0xa6b3, 0xa6b5, PDF_CMAP_RANGE, 12858 },
	{ 0xa6b6, 0xa6b6, PDF_CMAP_SINGLE, 2086 },
	{ 0xa6b7, 0xa6bd, PDF_CMAP_RANGE, 12861 },
	{ 0xa6c1, 0xa6c1, PDF_CMAP_SINGLE, 2087 },
	{ 0xa6c2, 0xa6d1, PDF_CMAP_RANGE, 12868 },
	{ 0xa6d3, 0xa6dd, PDF_CMAP_RANGE, 12884 },
	{ 0xa6e1, 0xa6e2, PDF_CMAP_RANGE, 2088 },
	{ 0xa6e3, 0xa6e4, PDF_CMAP_RANGE, 12895 },
	{ 0xa6e5, 0xa6e5, PDF_CMAP_SINGLE, 2090 },
	{ 0xa6e6, 0xa6e8, PDF_CMAP_RANGE, 12897 },
	{ 0xa6e9, 0xa6e9, PDF_CMAP_SINGLE, 2091 },
	{ 0xa6ea, 0xa6f1, PDF_CMAP_RANGE, 12900 },
	{ 0xa6f3, 0xa6f6, PDF_CMAP_RANGE, 12908 },
	{ 0xa6f7, 0xa6f7, PDF_CMAP_SINGLE, 2092 },
	{ 0xa6f8, 0xa6fd, PDF_CMAP_RANGE, 12912 },
	{ 0xa741, 0xa741, PDF_CMAP_SINGLE, 2093 },
	{ 0xa742, 0xa744, PDF_CMAP_RANGE, 12918 },
	{ 0xa745, 0xa745, PDF_CMAP_SINGLE, 2094 },
	{ 0xa746, 0xa748, PDF_CMAP_RANGE, 12921 },
	{ 0xa749, 0xa749, PDF_CMAP_SINGLE, 2095 },
	{ 0xa74a, 0xa750, PDF_CMAP_RANGE, 12924 },
	{ 0xa751, 0xa751, PDF_CMAP_SINGLE, 2096 },
	{ 0xa753, 0xa754, PDF_CMAP_RANGE, 12931 },
	{ 0xa755, 0xa757, PDF_CMAP_TABLE, 377 },
	{ 0xa758, 0xa75d, PDF_CMAP_RANGE, 12934 },
	{ 0xa761, 0xa762, PDF_CMAP_RANGE, 2099 },
	{ 0xa763, 0xa764, PDF_CMAP_RANGE, 12940 },
	{ 0xa765, 0xa765, PDF_CMAP_SINGLE, 2101 },
	{ 0xa766, 0xa768, PDF_CMAP_RANGE, 12942 },
	{ 0xa769, 0xa769, PDF_CMAP_SINGLE, 2102 },
	{ 0xa76a, 0xa770, PDF_CMAP_RANGE, 12945 },
	{ 0xa771, 0xa771, PDF_CMAP_SINGLE, 2103 },
	{ 0xa773, 0xa775, PDF_CMAP_TABLE, 380 },
	{ 0xa776, 0xa77d, PDF_CMAP_RANGE, 12953 },
	{ 0xa781, 0xa791, PDF_CMAP_RANGE, 12961 },
	{ 0xa793, 0xa79d, PDF_CMAP_RANGE, 12978 },
	{ 0xa7a1, 0xa7a2, PDF_CMAP_RANGE, 2106 },
	{ 0xa7a3, 0xa7a4, PDF_CMAP_RANGE, 12989 },
	{ 0xa7a5, 0xa7a5, PDF_CMAP_SINGLE, 2108 },
	{ 0xa7a6, 0xa7a8, PDF_CMAP_RANGE, 12991 },
	{ 0xa7a9, 0xa7ab, PDF_CMAP_TABLE, 383 },
	{ 0xa7ac, 0xa7b0, PDF_CMAP_RANGE, 12995 },
	{ 0xa7b1, 0xa7b1, PDF_CMAP_SINGLE, 2111 },
	{ 0xa7b3, 0xa7b6, PDF_CMAP_TABLE, 386 },
	{ 0xa7b7, 0xa7b9, PDF_CMAP_RANGE, 2114 },
	{ 0xa7ba, 0xa7bd, PDF_CMAP_RANGE, 13002 },
	{ 0xa841, 0xa841, PDF_CMAP_SINGLE, 376 },
	{ 0xa861, 0xa862, PDF_CMAP_RANGE, 2117 },
	{ 0xa863, 0xa864, PDF_CMAP_RANGE, 13006 },
	{ 0xa865, 0xa865, PDF_CMAP_SINGLE, 2119 },
	{ 0xa866, 0xa868, PDF_CMAP_RANGE, 13008 },
	{ 0xa869, 0xa86b, PDF_CMAP_TABLE, 390 },
	{ 0xa86c, 0xa870, PDF_CMAP_RANGE, 13012 },
	{ 0xa871, 0xa871, PDF_CMAP_SINGLE, 2122 },
	{ 0xa873, 0xa874, PDF_CMAP_TABLE, 393 },
	{ 0xa875, 0xa877, PDF_CMAP_RANGE, 2124 },
	{ 0xa878, 0xa87c, PDF_CMAP_RANGE, 13018 },
	{ 0xa87d, 0xa87d, PDF_CMAP_SINGLE, 2127 },
	{ 0xa881, 0xa882, PDF_CMAP_RANGE, 2128 },
	{ 0xa883, 0xa884, PDF_CMAP_RANGE, 13023 },
	{ 0xa885, 0xa885, PDF_CMAP_SINGLE, 2130 },
	{ 0xa886, 0xa888, PDF_CMAP_RANGE, 13025 },
	{ 0xa889, 0xa889, PDF_CMAP_SINGLE, 2131 },
	{ 0xa88a, 0xa890, PDF_CMAP_RANGE, 13028 },
	{ 0xa891, 0xa891, PDF_CMAP_SINGLE, 2132 },
	{ 0xa893, 0xa894, PDF_CMAP_TABLE, 395 },
	{ 0xa895, 0xa897, PDF_CMAP_RANGE, 2134 },
	{ 0xa898, 0xa89d, PDF_CMAP_RANGE, 13036 },
	{ 0xa8a1, 0xa8a2, PDF_CMAP_RANGE, 2137 },
	{ 0xa8a3, 0xa8b0, PDF_CMAP_RANGE, 13042 },
	{ 0xa8b1, 0xa8b1, PDF_CMAP_SINGLE, 2139 },
	{ 0xa8b3, 0xa8bd, PDF_CMAP_RANGE, 13056 },
	{ 0xa8c1, 0xa8d1, PDF_CMAP_RANGE, 13067 },
	{ 0xa8d3, 0xa8dd, PDF_CMAP_RANGE, 13084 },
	{ 0xa8e1, 0xa8e2, PDF_CMAP_RANGE, 2140 },
	{ 0xa8e3, 0xa8e4, PDF_CMAP_RANGE, 13095 },
	{ 0xa8e5, 0xa8e5, PDF_CMAP_SINGLE, 2142 },
	{ 0xa8e6, 0xa8e7, PDF_CMAP_RANGE, 13097 },
	{ 0xa8e8, 0xa8e9, PDF_CMAP_RANGE, 2143 },
	{ 0xa8ea, 0xa8f0, PDF_CMAP_RANGE, 13099 },
	{ 0xa8f1, 0xa8f1, PDF_CMAP_SINGLE, 2145 },
	{ 0xa8f3, 0xa8f4, PDF_CMAP_RANGE, 13106 },
	{ 0xa8f5, 0xa8f7, PDF_CMAP_RANGE, 2146 },
	{ 0xa8f8, 0xa8fd, PDF_CMAP_RANGE, 13108 },
	{ 0xa941, 0xa941, PDF_CMAP_SINGLE, 2149 },
	{ 0xa942, 0xa951, PDF_CMAP_RANGE, 13114 },
	{ 0xa953, 0xa956, PDF_CMAP_RANGE, 13130 },
	{ 0xa957, 0xa957, PDF_CMAP_SINGLE, 2150 },
	{ 0xa958, 0xa95d, PDF_CMAP_RANGE, 13134 },
	{ 0xa961, 0xa962, PDF_CMAP_RANGE, 2151 },
	{ 0xa963, 0xa970, PDF_CMAP_RANGE, 13140 },
	{ 0xa971, 0xa971, PDF_CMAP_SINGLE, 2153 },
	{ 0xa973, 0xa974, PDF_CMAP_TABLE, 397 },
	{ 0xa975, 0xa977, PDF_CMAP_RANGE, 2155 },
	{ 0xa978, 0xa97d, PDF_CMAP_RANGE, 13155 },
	{ 0xa981, 0xa991, PDF_CMAP_RANGE, 13161 },
	{ 0xa993, 0xa99d, PDF_CMAP_RANGE, 13178 },
	{ 0xa9a1, 0xa9a2, PDF_CMAP_RANGE, 2158 },
	{ 0xa9a3, 0xa9a4, PDF_CMAP_RANGE, 13189 },
	{ 0xa9a5, 0xa9a5, PDF_CMAP_SINGLE, 2160 },
	{ 0xa9a6, 0xa9a8, PDF_CMAP_RANGE, 13191 },
	{ 0xa9a9, 0xa9a9, PDF_CMAP_SINGLE, 2161 },
	{ 0xa9aa, 0xa9b0, PDF_CMAP_RANGE, 13194 },
	{ 0xa9b1, 0xa9b1, PDF_CMAP_SINGLE, 2162 },
	{ 0xa9b3, 0xa9b3, PDF_CMAP_SINGLE, 2163 },
	{ 0xa9b4, 0xa9b6, PDF_CMAP_RANGE, 13201 },
	{ 0xa9b7, 0xa9b7, PDF_CMAP_SINGLE, 2164 },
	{ 0xa9b8, 0xa9bd, PDF_CMAP_RANGE, 13204 },
	{ 0xa9c1, 0xa9d1, PDF_CMAP_RANGE, 13210 },
	{ 0xa9d3, 0xa9dd, PDF_CMAP_RANGE, 13227 },
	{ 0xa9e1, 0xa9f1, PDF_CMAP_RANGE, 13238 },
	{ 0xa9f3, 0xa9fd, PDF_CMAP_RANGE, 13255 },
	{ 0xaa41, 0xaa41, PDF_CMAP_SINGLE, 2165 },
	{ 0xaa42, 0xaa51, PDF_CMAP_RANGE, 13266 },
	{ 0xaa53, 0xaa5d, PDF_CMAP_RANGE, 13282 },
	{ 0xaa61, 0xaa61, PDF_CMAP_SINGLE, 2166 },
	{ 0xaa62, 0xaa71, PDF_CMAP_RANGE, 13293 },
	{ 0xaa73, 0xaa76, PDF_CMAP_RANGE, 13309 },
	{ 0xaa77, 0xaa77, PDF_CMAP_SINGLE, 2167 },
	{ 0xaa78, 0xaa7d, PDF_CMAP_RANGE, 13313 },
	{ 0xaa81, 0xaa82, PDF_CMAP_RANGE, 2168 },
	{ 0xaa83, 0xaa84, PDF_CMAP_RANGE, 13319 },
	{ 0xaa85, 0xaa85, PDF_CMAP_SINGLE, 2170 },
	{ 0xaa86, 0xaa88, PDF_CMAP_RANGE, 13321 },
	{ 0xaa89, 0xaa89, PDF_CMAP_SINGLE, 2171 },
	{ 0xaa8a, 0xaa90, PDF_CMAP_RANGE, 13324 },
	{ 0xaa91, 0xaa91, PDF_CMAP_SINGLE, 2172 },
	{ 0xaa93, 0xaa94, PDF_CMAP_RANGE, 13331 },
	{ 0xaa95, 0xaa97, PDF_CMAP_TABLE, 399 },
	{ 0xaa98, 0xaa9d, PDF_CMAP_RANGE, 13334 },
	{ 0xaaa1, 0xaab1, PDF_CMAP_RANGE, 13340 },
	{ 0xaab3, 0xaabd, PDF_CMAP_RANGE, 13357 },
	{ 0xaac1, 0xaad1, PDF_CMAP_RANGE, 13368 },
	{ 0xaad3, 0xaadd, PDF_CMAP_RANGE, 13385 },
	{ 0xaae1, 0xaaf1, PDF_CMAP_RANGE, 13396 },
	{ 0xaaf3, 0xaafd, PDF_CMAP_RANGE, 13413 },
	{ 0xab41, 0xab41, PDF_CMAP_SINGLE, 2175 },
	{ 0xab42, 0xab51, PDF_CMAP_RANGE, 13424 },
	{ 0xab53, 0xab56, PDF_CMAP_RANGE, 13440 },
	{ 0xab57, 0xab57, PDF_CMAP_SINGLE, 2176 },
	{ 0xab58, 0xab5d, PDF_CMAP_RANGE, 13444 },
	{ 0xab61, 0xab61, PDF_CMAP_SINGLE, 2177 },
	{ 0xab62, 0xab64, PDF_CMAP_RANGE, 13450 },
	{ 0xab65, 0xab65, PDF_CMAP_SINGLE, 2178 },
	{ 0xab66, 0xab68, PDF_CMAP_RANGE, 13453 },
	{ 0xab69, 0xab69, PDF_CMAP_SINGLE, 2179 },
	{ 0xab6a, 0xab70, PDF_CMAP_RANGE, 13456 },
	{ 0xab71, 0xab71, PDF_CMAP_SINGLE, 2180 },
	{ 0xab73, 0xab73, PDF_CMAP_SINGLE, 2181 },
	{ 0xab74, 0xab7d, PDF_CMAP_RANGE, 13463 },
	{ 0xab81, 0xab91, PDF_CMAP_RANGE, 13473 },
	{ 0xab93, 0xab9d, PDF_CMAP_RANGE, 13490 },
	{ 0xaba1, 0xaba2, PDF_CMAP_RANGE, 2182 },
	{ 0xaba3, 0xaba4, PDF_CMAP_RANGE, 13501 },
	{ 0xaba5, 0xaba5, PDF_CMAP_SINGLE, 2184 },
	{ 0xaba6, 0xaba8, PDF_CMAP_RANGE, 13503 },
	{ 0xaba9, 0xaba9, PDF_CMAP_SINGLE, 2185 },
	{ 0xabaa, 0xabb0, PDF_CMAP_RANGE, 13506 },
	{ 0xabb1, 0xabb1, PDF_CMAP_SINGLE, 2186 },
	{ 0xabb3, 0xabb7, PDF_CMAP_TABLE, 402 },
	{ 0xabb8, 0xabbd, PDF_CMAP_RANGE, 13515 },
	{ 0xac41, 0xac41, PDF_CMAP_SINGLE, 378 },
	{ 0xac61, 0xac62, PDF_CMAP_RANGE, 2190 },
	{ 0xac63, 0xac63, PDF_CMAP_SINGLE, 13521 },
	{ 0xac64, 0xac65, PDF_CMAP_RANGE, 2192 },
	{ 0xac66, 0xac67, PDF_CMAP_RANGE, 13522 },
	{ 0xac68, 0xac6b, PDF_CMAP_RANGE, 2194 },
	{ 0xac6c, 0xac70, PDF_CMAP_RANGE, 13524 },
	{ 0xac71, 0xac71, PDF_CMAP_SINGLE, 2198 },
	{ 0xac73, 0xac74, PDF_CMAP_TABLE, 407 },
	{ 0xac75, 0xac77, PDF_CMAP_RANGE, 2200 },
	{ 0xac78, 0xac7a, PDF_CMAP_RANGE, 13530 },
	{ 0xac7b, 0xac7b, PDF_CMAP_SINGLE, 2203 },
	{ 0xac7c, 0xac7d, PDF_CMAP_RANGE, 13533 },
	{ 0xac81, 0xac82, PDF_CMAP_RANGE, 2204 },
	{ 0xac83, 0xac84, PDF_CMAP_RANGE, 13535 },
	{ 0xac85, 0xac85, PDF_CMAP_SINGLE, 2206 },
	{ 0xac86, 0xac88, PDF_CMAP_RANGE, 13537 },
	{ 0xac89, 0xac89, PDF_CMAP_SINGLE, 2207 },
	{ 0xac8a, 0xac90, PDF_CMAP_RANGE, 13540 },
	{ 0xac91, 0xac91, PDF_CMAP_SINGLE, 2208 },
	{ 0xac93, 0xac94, PDF_CMAP_TABLE, 409 },
	{ 0xac95, 0xac97, PDF_CMAP_RANGE, 2210 },
	{ 0xac98, 0xac9d, PDF_CMAP_RANGE, 13548 },
	{ 0xaca1, 0xaca2, PDF_CMAP_RANGE, 2213 },
	{ 0xaca3, 0xaca4, PDF_CMAP_RANGE, 13554 },
	{ 0xaca5, 0xaca5, PDF_CMAP_SINGLE, 2215 },
	{ 0xaca6, 0xaca8, PDF_CMAP_RANGE, 13556 },
	{ 0xaca9, 0xaca9, PDF_CMAP_SINGLE, 2216 },
	{ 0xacaa, 0xacb0, PDF_CMAP_RANGE, 13559 },
	{ 0xacb1, 0xacb1, PDF_CMAP_SINGLE, 2217 },
	{ 0xacb3, 0xacb7, PDF_CMAP_TABLE, 411 },
	{ 0xacb8, 0xacbd, PDF_CMAP_RANGE, 13568 },
	{ 0xacc1, 0xacc1, PDF_CMAP_SINGLE, 2221 },
	{ 0xacc2, 0xacc4, PDF_CMAP_RANGE, 13574 },
	{ 0xacc5, 0xacc5, PDF_CMAP_SINGLE, 2222 },
	{ 0xacc6, 0xacc8, PDF_CMAP_RANGE, 13577 },
	{ 0xacc9, 0xacc9, PDF_CMAP_SINGLE, 2223 },
	{ 0xacca, 0xacd0, PDF_CMAP_RANGE, 13580 },
	{ 0xacd1, 0xacd1, PDF_CMAP_SINGLE, 2224 },
	{ 0xacd3, 0xacd6, PDF_CMAP_RANGE, 13587 },
	{ 0xacd7, 0xacd7, PDF_CMAP_SINGLE, 2225 },
	{ 0xacd8, 0xacdd, PDF_CMAP_RANGE, 13591 },
	{ 0xace1, 0xace5, PDF_CMAP_RANGE, 2226 },
	{ 0xace6, 0xace7, PDF_CMAP_RANGE, 13597 },
	{ 0xace8, 0xace9, PDF_CMAP_RANGE, 2231 },
	{ 0xacea, 0xacea, PDF_CMAP_SINGLE, 13599 },
	{ 0xaceb, 0xacec, PDF_CMAP_RANGE, 2233 },
	{ 0xaced, 0xacf0, PDF_CMAP_RANGE, 13600 },
	{ 0xacf1, 0xacf1, PDF_CMAP_SINGLE, 2235 },
	{ 0xacf3, 0xacf4, PDF_CMAP_TABLE, 416 },
	{ 0xacf5, 0xacf7, PDF_CMAP_RANGE, 2237 },
	{ 0xacf8, 0xacfb, PDF_CMAP_RANGE, 13605 },
	{ 0xacfc, 0xacfd, PDF_CMAP_TABLE, 418 },
	{ 0xad41, 0xad42, PDF_CMAP_RANGE, 2241 },
	{ 0xad43, 0xad44, PDF_CMAP_RANGE, 13610 },
	{ 0xad45, 0xad45, PDF_CMAP_SINGLE, 2243 },
	{ 0xad46, 0xad48, PDF_CMAP_RANGE, 13612 },
	{ 0xad49, 0xad49, PDF_CMAP_SINGLE, 2244 },
	{ 0xad4a, 0xad50, PDF_CMAP_RANGE, 13615 },
	{ 0xad51, 0xad51, PDF_CMAP_SINGLE, 2245 },
	{ 0xad53, 0xad54, PDF_CMAP_TABLE, 420 },
	{ 0xad55, 0xad57, PDF_CMAP_RANGE, 2247 },
	{ 0xad58, 0xad5d, PDF_CMAP_RANGE, 13623 },
	{ 0xad61, 0xad62, PDF_CMAP_RANGE, 2250 },
	{ 0xad63, 0xad64, PDF_CMAP_RANGE, 13629 },
	{ 0xad65, 0xad65, PDF_CMAP_SINGLE, 2252 },
	{ 0xad66, 0xad68, PDF_CMAP_RANGE, 13631 },
	{ 0xad69, 0xad69, PDF_CMAP_SINGLE, 2253 },
	{ 0xad6a, 0xad70, PDF_CMAP_RANGE, 13634 },
	{ 0xad71, 0xad71, PDF_CMAP_SINGLE, 2254 },
	{ 0xad73, 0xad74, PDF_CMAP_TABLE, 422 },
	{ 0xad75, 0xad77, PDF_CMAP_RANGE, 2256 },
	{ 0xad78, 0xad7d, PDF_CMAP_RANGE, 13642 },
	{ 0xad81, 0xad81, PDF_CMAP_SINGLE, 2259 },
	{ 0xad82, 0xad84, PDF_CMAP_RANGE, 13648 },
	{ 0xad85, 0xad85, PDF_CMAP_SINGLE, 2260 },
	{ 0xad86, 0xad88, PDF_CMAP_RANGE, 13651 },
	{ 0xad89, 0xad89, PDF_CMAP_SINGLE, 2261 },
	{ 0xad8a, 0xad91, PDF_CMAP_RANGE, 13654 },
	{ 0xad93, 0xad96, PDF_CMAP_RANGE, 13662 },
	{ 0xad97, 0xad97, PDF_CMAP_SINGLE, 2262 },
	{ 0xad98, 0xad9d, PDF_CMAP_RANGE, 13666 },
	{ 0xada1, 0xada3, PDF_CMAP_RANGE, 2263 },
	{ 0xada4, 0xada5, PDF_CMAP_TABLE, 424 },
	{ 0xada6, 0xada8, PDF_CMAP_RANGE, 13673 },
	{ 0xada9, 0xadab, PDF_CMAP_TABLE, 426 },
	{ 0xadac, 0xadb0, PDF_CMAP_RANGE, 13677 },
	{ 0xadb1, 0xadb1, PDF_CMAP_SINGLE, 2269 },
	{ 0xadb3, 0xadb7, PDF_CMAP_TABLE, 429 },
	{ 0xadb8, 0xadba, PDF_CMAP_RANGE, 13684 },
	{ 0xadbb, 0xadbb, PDF_CMAP_SINGLE, 2273 },
	{ 0xadbc, 0xadbd, PDF_CMAP_RANGE, 13687 },
	{ 0xadc1, 0xadc2, PDF_CMAP_RANGE, 2274 },
	{ 0xadc3, 0xadc4, PDF_CMAP_RANGE, 13689 },
	{ 0xadc5, 0xadc5, PDF_CMAP_SINGLE, 2276 },
	{ 0xadc6, 0xadc8, PDF_CMAP_RANGE, 13691 },
	{ 0xadc9, 0xadc9, PDF_CMAP_SINGLE, 2277 },
	{ 0xadca, 0xadd1, PDF_CMAP_RANGE, 13694 },
	{ 0xadd3, 0xadd6, PDF_CMAP_RANGE, 13702 },
	{ 0xadd7, 0xadd7, PDF_CMAP_SINGLE, 2278 },
	{ 0xadd8, 0xaddd, PDF_CMAP_RANGE, 13706 },
	{ 0xade1, 0xade1, PDF_CMAP_SINGLE, 2279 },
	{ 0xade2, 0xade4, PDF_CMAP_RANGE, 13712 },
	{ 0xade5, 0xade5, PDF_CMAP_SINGLE, 2280 },
	{ 0xade6, 0xade8, PDF_CMAP_RANGE, 13715 },
	{ 0xade9, 0xade9, PDF_CMAP_SINGLE, 2281 },
	{ 0xadea, 0xadf0, PDF_CMAP_RANGE, 13718 },
	{ 0xadf1, 0xadf1, PDF_CMAP_SINGLE, 2282 },
	{ 0xadf3, 0xadf4, PDF_CMAP_RANGE, 13725 },
	{ 0xadf5, 0xadf6, PDF_CMAP_RANGE, 2283 },
	{ 0xadf7, 0xadfd, PDF_CMAP_RANGE, 13727 },
	{ 0xae41, 0xae41, PDF_CMAP_SINGLE, 2285 },
	{ 0xae42, 0xae44, PDF_CMAP_RANGE, 13734 },
	{ 0xae45, 0xae45, PDF_CMAP_SINGLE, 2286 },
	{ 0xae46, 0xae48, PDF_CMAP_RANGE, 13737 },
	{ 0xae49, 0xae49, PDF_CMAP_SINGLE, 2287 },
	{ 0xae4a, 0xae50, PDF_CMAP_RANGE, 13740 },
	{ 0xae51, 0xae51, PDF_CMAP_SINGLE, 2288 },
	{ 0xae53, 0xae55, PDF_CMAP_TABLE, 434 },
	{ 0xae56, 0xae5d, PDF_CMAP_RANGE, 13748 },
	{ 0xae61, 0xae62, PDF_CMAP_RANGE, 2291 },
	{ 0xae63, 0xae64, PDF_CMAP_RANGE, 13756 },
	{ 0xae65, 0xae65, PDF_CMAP_SINGLE, 2293 },
	{ 0xae66, 0xae68, PDF_CMAP_RANGE, 13758 },
	{ 0xae69, 0xae69, PDF_CMAP_SINGLE, 2294 },
	{ 0xae6a, 0xae70, PDF_CMAP_RANGE, 13761 },
	{ 0xae71, 0xae71, PDF_CMAP_SINGLE, 2295 },
	{ 0xae73, 0xae77, PDF_CMAP_TABLE, 437 },
	{ 0xae78, 0xae7d, PDF_CMAP_RANGE, 13770 },
	{ 0xae81, 0xae82, PDF_CMAP_RANGE, 2299 },
	{ 0xae83, 0xae84, PDF_CMAP_RANGE, 13776 },
	{ 0xae85, 0xae85, PDF_CMAP_SINGLE, 2301 },
	{ 0xae86, 0xae87, PDF_CMAP_RANGE, 13778 },
	{ 0xae88, 0xae89, PDF_CMAP_RANGE, 2302 },
	{ 0xae8a, 0xae90, PDF_CMAP_RANGE, 13780 },
	{ 0xae91, 0xae91, PDF_CMAP_SINGLE, 2304 },
	{ 0xae93, 0xae9a, PDF_CMAP_TABLE, 442 },
	{ 0xae9b, 0xae9c, PDF_CMAP_RANGE, 2309 },
	{ 0xae9d, 0xae9d, PDF_CMAP_SINGLE, 13791 },
	{ 0xaea1, 0xaea1, PDF_CMAP_SINGLE, 2311 },
	{ 0xaea2, 0xaeb1, PDF_CMAP_RANGE, 13792 },
	{ 0xaeb3, 0xaeb5, PDF_CMAP_RANGE, 13808 },
	{ 0xaeb6, 0xaeb6, PDF_CMAP_SINGLE, 2312 },
	{ 0xaeb7, 0xaebd, PDF_CMAP_RANGE, 13811 },
	{ 0xaec1, 0xaec2, PDF_CMAP_RANGE, 2313 },
	{ 0xaec3, 0xaec4, PDF_CMAP_RANGE, 13818 },
	{ 0xaec5, 0xaec5, PDF_CMAP_SINGLE, 2315 },
	{ 0xaec6, 0xaec8, PDF_CMAP_RANGE, 13820 },
	{ 0xaec9, 0xaec9, PDF_CMAP_SINGLE, 2316 },
	{ 0xaeca, 0xaed0, PDF_CMAP_RANGE, 13823 },
	{ 0xaed1, 0xaed1, PDF_CMAP_SINGLE, 2317 },
	{ 0xaed3, 0xaed6, PDF_CMAP_RANGE, 13830 },
	{ 0xaed7, 0xaed7, PDF_CMAP_SINGLE, 2318 },
	{ 0xaed8, 0xaedd, PDF_CMAP_RANGE, 13834 },
	{ 0xaee1, 0xaee2, PDF_CMAP_RANGE, 2319 },
	{ 0xaee3, 0xaee4, PDF_CMAP_RANGE, 13840 },
	{ 0xaee5, 0xaee5, PDF_CMAP_SINGLE, 2321 },
	{ 0xaee6, 0xaee8, PDF_CMAP_RANGE, 13842 },
	{ 0xaee9, 0xaee9, PDF_CMAP_SINGLE, 2322 },
	{ 0xaeea, 0xaef0, PDF_CMAP_RANGE, 13845 },
	{ 0xaef1, 0xaef1, PDF_CMAP_SINGLE, 2323 },
	{ 0xaef3, 0xaef7, PDF_CMAP_TABLE, 450 },
	{ 0xaef8, 0xaefd, PDF_CMAP_RANGE, 13854 },
	{ 0xaf41, 0xaf42, PDF_CMAP_RANGE, 2327 },
	{ 0xaf43, 0xaf48, PDF_CMAP_RANGE, 13860 },
	{ 0xaf49, 0xaf49, PDF_CMAP_SINGLE, 2329 },
	{ 0xaf4a, 0xaf50, PDF_CMAP_RANGE, 13866 },
	{ 0xaf51, 0xaf51, PDF_CMAP_SINGLE, 2330 },
	{ 0xaf53, 0xaf54, PDF_CMAP_RANGE, 13873 },
	{ 0xaf55, 0xaf57, PDF_CMAP_TABLE, 455 },
	{ 0xaf58, 0xaf5d, PDF_CMAP_RANGE, 13876 },
	{ 0xaf61, 0xaf62, PDF_CMAP_RANGE, 2333 },
	{ 0xaf63, 0xaf64, PDF_CMAP_RANGE, 13882 },
	{ 0xaf65, 0xaf65, PDF_CMAP_SINGLE, 2335 },
	{ 0xaf66, 0xaf68, PDF_CMAP_RANGE, 13884 },
	{ 0xaf69, 0xaf6a, PDF_CMAP_RANGE, 2336 },
	{ 0xaf6b, 0xaf70, PDF_CMAP_RANGE, 13887 },
	{ 0xaf71, 0xaf71, PDF_CMAP_SINGLE, 2338 },
	{ 0xaf73, 0xaf77, PDF_CMAP_TABLE, 458 },
	{ 0xaf78, 0xaf7d, PDF_CMAP_RANGE, 13895 },
	{ 0xaf81, 0xaf91, PDF_CMAP_RANGE, 13901 },
	{ 0xaf93, 0xaf9d, PDF_CMAP_RANGE, 13918 },
	{ 0xafa1, 0xafa2, PDF_CMAP_RANGE, 2342 },
	{ 0xafa3, 0xafa4, PDF_CMAP_RANGE, 13929 },
	{ 0xafa5, 0xafa5, PDF_CMAP_SINGLE, 2344 },
	{ 0xafa6, 0xafa7, PDF_CMAP_RANGE, 13931 },
	{ 0xafa8, 0xafa9, PDF_CMAP_RANGE, 2345 },
	{ 0xafaa, 0xafaf, PDF_CMAP_RANGE, 13933 },
	{ 0xafb0, 0xafb1, PDF_CMAP_RANGE, 2347 },
	{ 0xafb3, 0xafb7, PDF_CMAP_TABLE, 463 },
	{ 0xafb8, 0xafbb, PDF_CMAP_RANGE, 13941 },
	{ 0xafbc, 0xafbd, PDF_CMAP_TABLE, 468 },
	{ 0xb041, 0xb041, PDF_CMAP_SINGLE, 379 },
	{ 0xb061, 0xb062, PDF_CMAP_RANGE, 2353 },
	{ 0xb063, 0xb063, PDF_CMAP_SINGLE, 13946 },
	{ 0xb064, 0xb065, PDF_CMAP_RANGE, 2355 },
	{ 0xb066, 0xb068, PDF_CMAP_RANGE, 13947 },
	{ 0xb069, 0xb069, PDF_CMAP_SINGLE, 2357 },
	{ 0xb06a, 0xb070, PDF_CMAP_RANGE, 13950 },
	{ 0xb071, 0xb071, PDF_CMAP_SINGLE, 2358 },
	{ 0xb073, 0xb073, PDF_CMAP_SINGLE, 2359 },
	{ 0xb074, 0xb075, PDF_CMAP_RANGE, 13957 },
	{ 0xb076, 0xb077, PDF_CMAP_RANGE, 2360 },
	{ 0xb078, 0xb07c, PDF_CMAP_RANGE, 13959 },
	{ 0xb07d, 0xb07d, PDF_CMAP_SINGLE, 2362 },
	{ 0xb081, 0xb082, PDF_CMAP_RANGE, 2363 },
	{ 0xb083, 0xb084, PDF_CMAP_RANGE, 13964 },
	{ 0xb085, 0xb085, PDF_CMAP_SINGLE, 2365 },
	{ 0xb086, 0xb088, PDF_CMAP_RANGE, 13966 },
	{ 0xb089, 0xb089, PDF_CMAP_SINGLE, 2366 },
	{ 0xb08a, 0xb090, PDF_CMAP_RANGE, 13969 },
	{ 0xb091, 0xb091, PDF_CMAP_SINGLE, 2367 },
	{ 0xb093, 0xb093, PDF_CMAP_SINGLE, 2368 },
	{ 0xb094, 0xb095, PDF_CMAP_RANGE, 13976 },
	{ 0xb096, 0xb097, PDF_CMAP_RANGE, 2369 },
	{ 0xb098, 0xb09d, PDF_CMAP_RANGE, 13978 },
	{ 0xb0a1, 0xb0b1, PDF_CMAP_RANGE, 13984 },
	{ 0xb0b3, 0xb0b6, PDF_CMAP_RANGE, 14001 },
	{ 0xb0b7, 0xb0b7, PDF_CMAP_SINGLE, 2371 },
	{ 0xb0b8, 0xb0bd, PDF_CMAP_RANGE, 14005 },
	{ 0xb0c1, 0xb0d1, PDF_CMAP_RANGE, 14011 },
	{ 0xb0d3, 0xb0dd, PDF_CMAP_RANGE, 14028 },
	{ 0xb0e1, 0xb0e2, PDF_CMAP_RANGE, 2372 },
	{ 0xb0e3, 0xb0e4, PDF_CMAP_RANGE, 14039 },
	{ 0xb0e5, 0xb0e5, PDF_CMAP_SINGLE, 2374 },
	{ 0xb0e6, 0xb0e8, PDF_CMAP_RANGE, 14041 },
	{ 0xb0e9, 0xb0eb, PDF_CMAP_TABLE, 470 },
	{ 0xb0ec, 0xb0f0, PDF_CMAP_RANGE, 14045 },
	{ 0xb0f1, 0xb0f1, PDF_CMAP_SINGLE, 2377 },
	{ 0xb0f3, 0xb0f3, PDF_CMAP_SINGLE, 2378 },
	{ 0xb0f4, 0xb0f5, PDF_CMAP_RANGE, 14050 },
	{ 0xb0f6, 0xb0f7, PDF_CMAP_RANGE, 2379 },
	{ 0xb0f8, 0xb0fd, PDF_CMAP_RANGE, 14052 },
	{ 0xb141, 0xb141, PDF_CMAP_SINGLE, 2381 },
	{ 0xb142, 0xb144, PDF_CMAP_RANGE, 14058 },
	{ 0xb145, 0xb145, PDF_CMAP_SINGLE, 2382 },
	{ 0xb146, 0xb148, PDF_CMAP_RANGE, 14061 },
	{ 0xb149, 0xb149, PDF_CMAP_SINGLE, 2383 },
	{ 0xb14a, 0xb151, PDF_CMAP_RANGE, 14064 },
	{ 0xb153, 0xb15d, PDF_CMAP_RANGE, 14072 },
	{ 0xb161, 0xb171, PDF_CMAP_RANGE, 14083 },
	{ 0xb173, 0xb17d, PDF_CMAP_RANGE, 14100 },
	{ 0xb181, 0xb184, PDF_CMAP_RANGE, 14111 },
	{ 0xb185, 0xb185, PDF_CMAP_SINGLE, 2384 },
	{ 0xb186, 0xb191, PDF_CMAP_RANGE, 14115 },
	{ 0xb193, 0xb19d, PDF_CMAP_RANGE, 14127 },
	{ 0xb1a1, 0xb1a2, PDF_CMAP_RANGE, 2385 },
	{ 0xb1a3, 0xb1a4, PDF_CMAP_RANGE, 14138 },
	{ 0xb1a5, 0xb1a5, PDF_CMAP_SINGLE, 2387 },
	{ 0xb1a6, 0xb1a7, PDF_CMAP_RANGE, 14140 },
	{ 0xb1a8, 0xb1a9, PDF_CMAP_RANGE, 2388 },
	{ 0xb1aa, 0xb1ab, PDF_CMAP_TABLE, 473 },
	{ 0xb1ac, 0xb1b0, PDF_CMAP_RANGE, 14143 },
	{ 0xb1b1, 0xb1b1, PDF_CMAP_SINGLE, 2391 },
	{ 0xb1b3, 0xb1b3, PDF_CMAP_SINGLE, 2392 },
	{ 0xb1b4, 0xb1b6, PDF_CMAP_RANGE, 14148 },
	{ 0xb1b7, 0xb1b7, PDF_CMAP_SINGLE, 2393 },
	{ 0xb1b8, 0xb1bd, PDF_CMAP_RANGE, 14151 },
	{ 0xb1c1, 0xb1c2, PDF_CMAP_RANGE, 2394 },
	{ 0xb1c3, 0xb1c4, PDF_CMAP_RANGE, 14157 },
	{ 0xb1c5, 0xb1c5, PDF_CMAP_SINGLE, 2396 },
	{ 0xb1c6, 0xb1d1, PDF_CMAP_RANGE, 14159 },
	{ 0xb1d3, 0xb1d5, PDF_CMAP_RANGE, 14171 },
	{ 0xb1d6, 0xb1d6, PDF_CMAP_SINGLE, 2397 },
	{ 0xb1d7, 0xb1dd, PDF_CMAP_RANGE, 14174 },
	{ 0xb1e1, 0xb1e1, PDF_CMAP_SINGLE, 2398 },
	{ 0xb1e2, 0xb1f1, PDF_CMAP_RANGE, 14181 },
	{ 0xb1f3, 0xb1f5, PDF_CMAP_RANGE, 14197 },
	{ 0xb1f6, 0xb1f6, PDF_CMAP_SINGLE, 2399 },
	{ 0xb1f7, 0xb1fd, PDF_CMAP_RANGE, 14200 },
	{ 0xb241, 0xb241, PDF_CMAP_SINGLE, 2400 },
	{ 0xb242, 0xb244, PDF_CMAP_RANGE, 14207 },
	{ 0xb245, 0xb245, PDF_CMAP_SINGLE, 2401 },
	{ 0xb246, 0xb248, PDF_CMAP_RANGE, 14210 },
	{ 0xb249, 0xb249, PDF_CMAP_SINGLE, 2402 },
	{ 0xb24a, 0xb250, PDF_CMAP_RANGE, 14213 },
	{ 0xb251, 0xb251, PDF_CMAP_SINGLE, 2403 },
	{ 0xb253, 0xb253, PDF_CMAP_SINGLE, 2404 },
	{ 0xb254, 0xb25d, PDF_CMAP_RANGE, 14220 },
	{ 0xb261, 0xb261, PDF_CMAP_SINGLE, 2405 },
	{ 0xb262, 0xb271, PDF_CMAP_RANGE, 14230 },
	{ 0xb273, 0xb27d, PDF_CMAP_RANGE, 14246 },
	{ 0xb281, 0xb282, PDF_CMAP_RANGE, 2406 },
	{ 0xb283, 0xb284, PDF_CMAP_RANGE, 14257 },
	{ 0xb285, 0xb285, PDF_CMAP_SINGLE, 2408 },
	{ 0xb286, 0xb288, PDF_CMAP_RANGE, 14259 },
	{ 0xb289, 0xb289, PDF_CMAP_SINGLE, 2409 },
	{ 0xb28a, 0xb290, PDF_CMAP_RANGE, 14262 },
	{ 0xb291, 0xb291, PDF_CMAP_SINGLE, 2410 },
	{ 0xb293, 0xb293, PDF_CMAP_SINGLE, 2411 },
	{ 0xb294, 0xb296, PDF_CMAP_RANGE, 14269 },
	{ 0xb297, 0xb297, PDF_CMAP_SINGLE, 2412 },
	{ 0xb298, 0xb29d, PDF_CMAP_RANGE, 14272 },
	{ 0xb2a1, 0xb2a1, PDF_CMAP_SINGLE, 2413 },
	{ 0xb2a2, 0xb2b1, PDF_CMAP_RANGE, 14278 },
	{ 0xb2b3, 0xb2b5, PDF_CMAP_RANGE, 14294 },
	{ 0xb2b6, 0xb2b6, PDF_CMAP_SINGLE, 2414 },
	{ 0xb2b7, 0xb2bd, PDF_CMAP_RANGE, 14297 },
	{ 0xb2c1, 0xb2c1, PDF_CMAP_SINGLE, 2415 },
	{ 0xb2c2, 0xb2d1, PDF_CMAP_RANGE, 14304 },
	{ 0xb2d3, 0xb2dd, PDF_CMAP_RANGE, 14320 },
	{ 0xb2e1, 0xb2e1, PDF_CMAP_SINGLE, 2416 },
	{ 0xb2e2, 0xb2e4, PDF_CMAP_RANGE, 14331 },
	{ 0xb2e5, 0xb2e5, PDF_CMAP_SINGLE, 2417 },
	{ 0xb2e6, 0xb2f1, PDF_CMAP_RANGE, 14334 },
	{ 0xb2f3, 0xb2fd, PDF_CMAP_RANGE, 14346 },
	{ 0xb341, 0xb351, PDF_CMAP_RANGE, 14357 },
	{ 0xb353, 0xb356, PDF_CMAP_RANGE, 14374 },
	{ 0xb357, 0xb357, PDF_CMAP_SINGLE, 2418 },
	{ 0xb358, 0xb35d, PDF_CMAP_RANGE, 14378 },
	{ 0xb361, 0xb362, PDF_CMAP_RANGE, 2419 },
	{ 0xb363, 0xb364, PDF_CMAP_RANGE, 14384 },
	{ 0xb365, 0xb365, PDF_CMAP_SINGLE, 2421 },
	{ 0xb366, 0xb368, PDF_CMAP_RANGE, 14386 },
	{ 0xb369, 0xb36b, PDF_CMAP_TABLE, 475 },
	{ 0xb36c, 0xb36f, PDF_CMAP_RANGE, 14390 },
	{ 0xb370, 0xb371, PDF_CMAP_RANGE, 2424 },
	{ 0xb373, 0xb373, PDF_CMAP_SINGLE, 2426 },
	{ 0xb374, 0xb37d, PDF_CMAP_RANGE, 14394 },
	{ 0xb381, 0xb381, PDF_CMAP_SINGLE, 2427 },
	{ 0xb382, 0xb384, PDF_CMAP_RANGE, 14404 },
	{ 0xb385, 0xb385, PDF_CMAP_SINGLE, 2428 },
	{ 0xb386, 0xb388, PDF_CMAP_RANGE, 14407 },
	{ 0xb389, 0xb389, PDF_CMAP_SINGLE, 2429 },
	{ 0xb38a, 0xb390, PDF_CMAP_RANGE, 14410 },
	{ 0xb391, 0xb391, PDF_CMAP_SINGLE, 2430 },
	{ 0xb393, 0xb39d, PDF_CMAP_RANGE, 14417 },
	{ 0xb3a1, 0xb3a2, PDF_CMAP_RANGE, 2431 },
	{ 0xb3a3, 0xb3a4, PDF_CMAP_RANGE, 14428 },
	{ 0xb3a5, 0xb3a5, PDF_CMAP_SINGLE, 2433 },
	{ 0xb3a6, 0xb3a8, PDF_CMAP_RANGE, 14430 },
	{ 0xb3a9, 0xb3a9, PDF_CMAP_SINGLE, 2434 },
	{ 0xb3aa, 0xb3b0, PDF_CMAP_RANGE, 14433 },
	{ 0xb3b1, 0xb3b1, PDF_CMAP_SINGLE, 2435 },
	{ 0xb3b3, 0xb3b7, PDF_CMAP_TABLE, 478 },
	{ 0xb3b8, 0xb3bd, PDF_CMAP_RANGE, 14442 },
	{ 0xb441, 0xb441, PDF_CMAP_SINGLE, 380 },
	{ 0xb461, 0xb462, PDF_CMAP_RANGE, 2439 },
	{ 0xb463, 0xb464, PDF_CMAP_RANGE, 14448 },
	{ 0xb465, 0xb467, PDF_CMAP_RANGE, 2441 },
	{ 0xb468, 0xb468, PDF_CMAP_SINGLE, 14450 },
	{ 0xb469, 0xb46b, PDF_CMAP_RANGE, 2444 },
	{ 0xb46c, 0xb46f, PDF_CMAP_RANGE, 14451 },
	{ 0xb470, 0xb471, PDF_CMAP_RANGE, 2447 },
	{ 0xb473, 0xb474, PDF_CMAP_TABLE, 483 },
	{ 0xb475, 0xb477, PDF_CMAP_RANGE, 2450 },
	{ 0xb478, 0xb47a, PDF_CMAP_RANGE, 14456 },
	{ 0xb47b, 0xb47c, PDF_CMAP_RANGE, 2453 },
	{ 0xb47d, 0xb47d, PDF_CMAP_SINGLE, 14459 },
	{ 0xb481, 0xb482, PDF_CMAP_RANGE, 2455 },
	{ 0xb483, 0xb484, PDF_CMAP_RANGE, 14460 },
	{ 0xb485, 0xb485, PDF_CMAP_SINGLE, 2457 },
	{ 0xb486, 0xb488, PDF_CMAP_RANGE, 14462 },
	{ 0xb489, 0xb489, PDF_CMAP_SINGLE, 2458 },
	{ 0xb48a, 0xb490, PDF_CMAP_RANGE, 14465 },
	{ 0xb491, 0xb491, PDF_CMAP_SINGLE, 2459 },
	{ 0xb493, 0xb494, PDF_CMAP_TABLE, 485 },
	{ 0xb495, 0xb497, PDF_CMAP_RANGE, 2461 },
	{ 0xb498, 0xb49d, PDF_CMAP_RANGE, 14473 },
	{ 0xb4a1, 0xb4a2, PDF_CMAP_RANGE, 2464 },
	{ 0xb4a3, 0xb4a4, PDF_CMAP_RANGE, 14479 },
	{ 0xb4a5, 0xb4a5, PDF_CMAP_SINGLE, 2466 },
	{ 0xb4a6, 0xb4a8, PDF_CMAP_RANGE, 14481 },
	{ 0xb4a9, 0xb4a9, PDF_CMAP_SINGLE, 2467 },
	{ 0xb4aa, 0xb4ab, PDF_CMAP_RANGE, 14484 },
	{ 0xb4ac, 0xb4ac, PDF_CMAP_SINGLE, 2468 },
	{ 0xb4ad, 0xb4b0, PDF_CMAP_RANGE, 14486 },
	{ 0xb4b1, 0xb4b1, PDF_CMAP_SINGLE, 2469 },
	{ 0xb4b3, 0xb4b7, PDF_CMAP_TABLE, 487 },
	{ 0xb4b8, 0xb4ba, PDF_CMAP_RANGE, 14492 },
	{ 0xb4bb, 0xb4bd, PDF_CMAP_TABLE, 492 },
	{ 0xb4c1, 0xb4c1, PDF_CMAP_SINGLE, 2475 },
	{ 0xb4c2, 0xb4c4, PDF_CMAP_RANGE, 14496 },
	{ 0xb4c5, 0xb4c5, PDF_CMAP_SINGLE, 2476 },
	{ 0xb4c6, 0xb4c8, PDF_CMAP_RANGE, 14499 },
	{ 0xb4c9, 0xb4c9, PDF_CMAP_SINGLE, 2477 },
	{ 0xb4ca, 0xb4d1, PDF_CMAP_RANGE, 14502 },
	{ 0xb4d3, 0xb4d3, PDF_CMAP_SINGLE, 2478 },
	{ 0xb4d4, 0xb4dd, PDF_CMAP_RANGE, 14510 },
	{ 0xb4e1, 0xb4e2, PDF_CMAP_RANGE, 2479 },
	{ 0xb4e3, 0xb4e4, PDF_CMAP_RANGE, 14520 },
	{ 0xb4e5, 0xb4e6, PDF_CMAP_RANGE, 2481 },
	{ 0xb4e7, 0xb4e7, PDF_CMAP_SINGLE, 14522 },
	{ 0xb4e8, 0xb4eb, PDF_CMAP_RANGE, 2483 },
	{ 0xb4ec, 0xb4f0, PDF_CMAP_RANGE, 14523 },
	{ 0xb4f1, 0xb4f1, PDF_CMAP_SINGLE, 2487 },
	{ 0xb4f3, 0xb4f8, PDF_CMAP_RANGE, 2488 },
	{ 0xb4f9, 0xb4fd, PDF_CMAP_TABLE, 495 },
	{ 0xb541, 0xb542, PDF_CMAP_RANGE, 2496 },
	{ 0xb543, 0xb544, PDF_CMAP_RANGE, 14531 },
	{ 0xb545, 0xb545, PDF_CMAP_SINGLE, 2498 },
	{ 0xb546, 0xb548, PDF_CMAP_RANGE, 14533 },
	{ 0xb549, 0xb549, PDF_CMAP_SINGLE, 2499 },
	{ 0xb54a, 0xb550, PDF_CMAP_RANGE, 14536 },
	{ 0xb551, 0xb551, PDF_CMAP_SINGLE, 2500 },
	{ 0xb553, 0xb557, PDF_CMAP_TABLE, 500 },
	{ 0xb558, 0xb55d, PDF_CMAP_RANGE, 14545 },
	{ 0xb561, 0xb563, PDF_CMAP_RANGE, 2504 },
	{ 0xb564, 0xb565, PDF_CMAP_TABLE, 505 },
	{ 0xb566, 0xb568, PDF_CMAP_RANGE, 14552 },
	{ 0xb569, 0xb56a, PDF_CMAP_TABLE, 507 },
	{ 0xb56b, 0xb56c, PDF_CMAP_RANGE, 2509 },
	{ 0xb56d, 0xb570, PDF_CMAP_RANGE, 14556 },
	{ 0xb571, 0xb571, PDF_CMAP_SINGLE, 2511 },
	{ 0xb573, 0xb577, PDF_CMAP_RANGE, 2512 },
	{ 0xb578, 0xb57a, PDF_CMAP_RANGE, 14560 },
	{ 0xb57b, 0xb57d, PDF_CMAP_RANGE, 2517 },
	{ 0xb581, 0xb581, PDF_CMAP_SINGLE, 2520 },
	{ 0xb582, 0xb584, PDF_CMAP_RANGE, 14563 },
	{ 0xb585, 0xb585, PDF_CMAP_SINGLE, 2521 },
	{ 0xb586, 0xb588, PDF_CMAP_RANGE, 14566 },
	{ 0xb589, 0xb589, PDF_CMAP_SINGLE, 2522 },
	{ 0xb58a, 0xb590, PDF_CMAP_RANGE, 14569 },
	{ 0xb591, 0xb591, PDF_CMAP_SINGLE, 2523 },
	{ 0xb593, 0xb594, PDF_CMAP_TABLE, 509 },
	{ 0xb595, 0xb596, PDF_CMAP_RANGE, 2525 },
	{ 0xb597, 0xb59d, PDF_CMAP_RANGE, 14577 },
	{ 0xb5a1, 0xb5a2, PDF_CMAP_RANGE, 2527 },
	{ 0xb5a3, 0xb5a4, PDF_CMAP_RANGE, 14584 },
	{ 0xb5a5, 0xb5a5, PDF_CMAP_SINGLE, 2529 },
	{ 0xb5a6, 0xb5a8, PDF_CMAP_RANGE, 14586 },
	{ 0xb5a9, 0xb5ab, PDF_CMAP_RANGE, 2530 },
	{ 0xb5ac, 0xb5ad, PDF_CMAP_TABLE, 511 },
	{ 0xb5ae, 0xb5af, PDF_CMAP_RANGE, 14590 },
	{ 0xb5b0, 0xb5b1, PDF_CMAP_RANGE, 2534 },
	{ 0xb5b3, 0xb5b9, PDF_CMAP_TABLE, 513 },
	{ 0xb5ba, 0xb5bd, PDF_CMAP_RANGE, 14595 },
	{ 0xb5c1, 0xb5c2, PDF_CMAP_RANGE, 2540 },
	{ 0xb5c3, 0xb5c4, PDF_CMAP_RANGE, 14599 },
	{ 0xb5c5, 0xb5c5, PDF_CMAP_SINGLE, 2542 },
	{ 0xb5c6, 0xb5c8, PDF_CMAP_RANGE, 14601 },
	{ 0xb5c9, 0xb5c9, PDF_CMAP_SINGLE, 2543 },
	{ 0xb5ca, 0xb5d0, PDF_CMAP_RANGE, 14604 },
	{ 0xb5d1, 0xb5d1, PDF_CMAP_SINGLE, 2544 },
	{ 0xb5d3, 0xb5d4, PDF_CMAP_TABLE, 520 },
	{ 0xb5d5, 0xb5d7, PDF_CMAP_RANGE, 2546 },
	{ 0xb5d8, 0xb5dd, PDF_CMAP_RANGE, 14612 },
	{ 0xb5e1, 0xb5e2, PDF_CMAP_RANGE, 2549 },
	{ 0xb5e3, 0xb5e4, PDF_CMAP_RANGE, 14618 },
	{ 0xb5e5, 0xb5e5, PDF_CMAP_SINGLE, 2551 },
	{ 0xb5e6, 0xb5f0, PDF_CMAP_RANGE, 14620 },
	{ 0xb5f1, 0xb5f1, PDF_CMAP_SINGLE, 2552 },
	{ 0xb5f3, 0xb5f4, PDF_CMAP_RANGE, 14631 },
	{ 0xb5f5, 0xb5f7, PDF_CMAP_TABLE, 522 },
	{ 0xb5f8, 0xb5fd, PDF_CMAP_RANGE, 14634 },
	{ 0xb641, 0xb642, PDF_CMAP_RANGE, 2555 },
	{ 0xb643, 0xb644, PDF_CMAP_RANGE, 14640 },
	{ 0xb645, 0xb645, PDF_CMAP_SINGLE, 2557 },
	{ 0xb646, 0xb648, PDF_CMAP_RANGE, 14642 },
	{ 0xb649, 0xb649, PDF_CMAP_SINGLE, 2558 },
	{ 0xb64a, 0xb650, PDF_CMAP_RANGE, 14645 },
	{ 0xb651, 0xb651, PDF_CMAP_SINGLE, 2559 },
	{ 0xb653, 0xb657, PDF_CMAP_TABLE, 525 },
	{ 0xb658, 0xb65d, PDF_CMAP_RANGE, 14654 },
	{ 0xb661, 0xb662, PDF_CMAP_RANGE, 2563 },
	{ 0xb663, 0xb664, PDF_CMAP_RANGE, 14660 },
	{ 0xb665, 0xb665, PDF_CMAP_SINGLE, 2565 },
	{ 0xb666, 0xb668, PDF_CMAP_RANGE, 14662 },
	{ 0xb669, 0xb669, PDF_CMAP_SINGLE, 2566 },
	{ 0xb66a, 0xb670, PDF_CMAP_RANGE, 14665 },
	{ 0xb671, 0xb671, PDF_CMAP_SINGLE, 2567 },
	{ 0xb673, 0xb677, PDF_CMAP_TABLE, 530 },
	{ 0xb678, 0xb67d, PDF_CMAP_RANGE, 14674 },
	{ 0xb681, 0xb682, PDF_CMAP_RANGE, 2571 },
	{ 0xb683, 0xb684, PDF_CMAP_RANGE, 14680 },
	{ 0xb685, 0xb685, PDF_CMAP_SINGLE, 2573 },
	{ 0xb686, 0xb688, PDF_CMAP_RANGE, 14682 },
	{ 0xb689, 0xb68b, PDF_CMAP_RANGE, 2574 },
	{ 0xb68c, 0xb690, PDF_CMAP_RANGE, 14685 },
	{ 0xb691, 0xb691, PDF_CMAP_SINGLE, 2577 },
	{ 0xb693, 0xb697, PDF_CMAP_TABLE, 535 },
	{ 0xb698, 0xb69d, PDF_CMAP_RANGE, 14692 },
	{ 0xb6a1, 0xb6a2, PDF_CMAP_RANGE, 2581 },
	{ 0xb6a3, 0xb6a4, PDF_CMAP_RANGE, 14698 },
	{ 0xb6a5, 0xb6a5, PDF_CMAP_SINGLE, 2583 },
	{ 0xb6a6, 0xb6a8, PDF_CMAP_RANGE, 14700 },
	{ 0xb6a9, 0xb6a9, PDF_CMAP_SINGLE, 2584 },
	{ 0xb6aa, 0xb6b0, PDF_CMAP_RANGE, 14703 },
	{ 0xb6b1, 0xb6b1, PDF_CMAP_SINGLE, 2585 },
	{ 0xb6b3, 0xb6b3, PDF_CMAP_SINGLE, 2586 },
	{ 0xb6b4, 0xb6b5, PDF_CMAP_RANGE, 14710 },
	{ 0xb6b6, 0xb6b7, PDF_CMAP_RANGE, 2587 },
	{ 0xb6b8, 0xb6bd, PDF_CMAP_RANGE, 14712 },
	{ 0xb6c1, 0xb6c2, PDF_CMAP_RANGE, 2589 },
	{ 0xb6c3, 0xb6c4, PDF_CMAP_RANGE, 14718 },
	{ 0xb6c5, 0xb6c5, PDF_CMAP_SINGLE, 2591 },
	{ 0xb6c6, 0xb6c8, PDF_CMAP_RANGE, 14720 },
	{ 0xb6c9, 0xb6c9, PDF_CMAP_SINGLE, 2592 },
	{ 0xb6ca, 0xb6d0, PDF_CMAP_RANGE, 14723 },
	{ 0xb6d1, 0xb6d1, PDF_CMAP_SINGLE, 2593 },
	{ 0xb6d3, 0xb6d3, PDF_CMAP_SINGLE, 2594 },
	{ 0xb6d4, 0xb6d6, PDF_CMAP_RANGE, 14730 },
	{ 0xb6d7, 0xb6d7, PDF_CMAP_SINGLE, 2595 },
	{ 0xb6d8, 0xb6dd, PDF_CMAP_RANGE, 14733 },
	{ 0xb6e1, 0xb6e2, PDF_CMAP_RANGE, 2596 },
	{ 0xb6e3, 0xb6e4, PDF_CMAP_RANGE, 14739 },
	{ 0xb6e5, 0xb6e5, PDF_CMAP_SINGLE, 2598 },
	{ 0xb6e6, 0xb6e8, PDF_CMAP_RANGE, 14741 },
	{ 0xb6e9, 0xb6e9, PDF_CMAP_SINGLE, 2599 },
	{ 0xb6ea, 0xb6f0, PDF_CMAP_RANGE, 14744 },
	{ 0xb6f1, 0xb6f1, PDF_CMAP_SINGLE, 2600 },
	{ 0xb6f3, 0xb6f7, PDF_CMAP_TABLE, 540 },
	{ 0xb6f8, 0xb6fd, PDF_CMAP_RANGE, 14753 },
	{ 0xb741, 0xb742, PDF_CMAP_RANGE, 2604 },
	{ 0xb743, 0xb744, PDF_CMAP_RANGE, 14759 },
	{ 0xb745, 0xb745, PDF_CMAP_SINGLE, 2606 },
	{ 0xb746, 0xb748, PDF_CMAP_RANGE, 14761 },
	{ 0xb749, 0xb749, PDF_CMAP_SINGLE, 2607 },
	{ 0xb74a, 0xb750, PDF_CMAP_RANGE, 14764 },
	{ 0xb751, 0xb751, PDF_CMAP_SINGLE, 2608 },
	{ 0xb753, 0xb759, PDF_CMAP_TABLE, 545 },
	{ 0xb75a, 0xb75d, PDF_CMAP_RANGE, 14774 },
	{ 0xb761, 0xb762, PDF_CMAP_RANGE, 2613 },
	{ 0xb763, 0xb764, PDF_CMAP_RANGE, 14778 },
	{ 0xb765, 0xb765, PDF_CMAP_SINGLE, 2615 },
	{ 0xb766, 0xb768, PDF_CMAP_RANGE, 14780 },
	{ 0xb769, 0xb769, PDF_CMAP_SINGLE, 2616 },
	{ 0xb76a, 0xb76e, PDF_CMAP_RANGE, 14783 },
	{ 0xb76f, 0xb771, PDF_CMAP_TABLE, 552 },
	{ 0xb773, 0xb776, PDF_CMAP_TABLE, 555 },
	{ 0xb777, 0xb77d, PDF_CMAP_RANGE, 2621 },
	{ 0xb781, 0xb781, PDF_CMAP_SINGLE, 2628 },
	{ 0xb782, 0xb784, PDF_CMAP_RANGE, 14791 },
	{ 0xb785, 0xb785, PDF_CMAP_SINGLE, 2629 },
	{ 0xb786, 0xb788, PDF_CMAP_RANGE, 14794 },
	{ 0xb789, 0xb789, PDF_CMAP_SINGLE, 2630 },
	{ 0xb78a, 0xb790, PDF_CMAP_RANGE, 14797 },
	{ 0xb791, 0xb791, PDF_CMAP_SINGLE, 2631 },
	{ 0xb793, 0xb794, PDF_CMAP_RANGE, 14804 },
	{ 0xb795, 0xb795, PDF_CMAP_SINGLE, 2632 },
	{ 0xb796, 0xb79d, PDF_CMAP_RANGE, 14806 },
	{ 0xb7a1, 0xb7a2, PDF_CMAP_RANGE, 2633 },
	{ 0xb7a3, 0xb7a4, PDF_CMAP_RANGE, 14814 },
	{ 0xb7a5, 0xb7a5, PDF_CMAP_SINGLE, 2635 },
	{ 0xb7a6, 0xb7a8, PDF_CMAP_RANGE, 14816 },
	{ 0xb7a9, 0xb7ab, PDF_CMAP_RANGE, 2636 },
	{ 0xb7ac, 0xb7af, PDF_CMAP_RANGE, 14819 },
	{ 0xb7b0, 0xb7b1, PDF_CMAP_RANGE, 2639 },
	{ 0xb7b3, 0xb7b4, PDF_CMAP_TABLE, 559 },
	{ 0xb7b5, 0xb7b8, PDF_CMAP_RANGE, 2642 },
	{ 0xb7b9, 0xb7bb, PDF_CMAP_RANGE, 14824 },
	{ 0xb7bc, 0xb7bd, PDF_CMAP_TABLE, 561 },
	{ 0xb841, 0xb841, PDF_CMAP_SINGLE, 381 },
	{ 0xb861, 0xb862, PDF_CMAP_RANGE, 2647 },
	{ 0xb863, 0xb864, PDF_CMAP_RANGE, 14828 },
	{ 0xb865, 0xb866, PDF_CMAP_TABLE, 563 },
	{ 0xb867, 0xb869, PDF_CMAP_RANGE, 2650 },
	{ 0xb86a, 0xb86b, PDF_CMAP_TABLE, 565 },
	{ 0xb86c, 0xb870, PDF_CMAP_RANGE, 14832 },
	{ 0xb871, 0xb871, PDF_CMAP_SINGLE, 2654 },
	{ 0xb873, 0xb874, PDF_CMAP_TABLE, 567 },
	{ 0xb875, 0xb878, PDF_CMAP_RANGE, 2656 },
	{ 0xb879, 0xb87d, PDF_CMAP_RANGE, 14838 },
	{ 0xb881, 0xb882, PDF_CMAP_RANGE, 2660 },
	{ 0xb883, 0xb884, PDF_CMAP_RANGE, 14843 },
	{ 0xb885, 0xb885, PDF_CMAP_SINGLE, 2662 },
	{ 0xb886, 0xb888, PDF_CMAP_RANGE, 14845 },
	{ 0xb889, 0xb889, PDF_CMAP_SINGLE, 2663 },
	{ 0xb88a, 0xb890, PDF_CMAP_RANGE, 14848 },
	{ 0xb891, 0xb891, PDF_CMAP_SINGLE, 2664 },
	{ 0xb893, 0xb894, PDF_CMAP_TABLE, 569 },
	{ 0xb895, 0xb897, PDF_CMAP_RANGE, 2666 },
	{ 0xb898, 0xb89d, PDF_CMAP_RANGE, 14856 },
	{ 0xb8a1, 0xb8a2, PDF_CMAP_RANGE, 2669 },
	{ 0xb8a3, 0xb8a4, PDF_CMAP_RANGE, 14862 },
	{ 0xb8a5, 0xb8a9, PDF_CMAP_TABLE, 571 },
	{ 0xb8aa, 0xb8b0, PDF_CMAP_RANGE, 14866 },
	{ 0xb8b1, 0xb8b1, PDF_CMAP_SINGLE, 2674 },
	{ 0xb8b3, 0xb8b6, PDF_CMAP_RANGE, 14873 },
	{ 0xb8b7, 0xb8b7, PDF_CMAP_SINGLE, 2675 },
	{ 0xb8b8, 0xb8bd, PDF_CMAP_RANGE, 14877 },
	{ 0xb8c1, 0xb8c1, PDF_CMAP_SINGLE, 2676 },
	{ 0xb8c2, 0xb8c4, PDF_CMAP_RANGE, 14883 },
	{ 0xb8c5, 0xb8c5, PDF_CMAP_SINGLE, 2677 },
	{ 0xb8c6, 0xb8c8, PDF_CMAP_RANGE, 14886 },
	{ 0xb8c9, 0xb8c9, PDF_CMAP_SINGLE, 2678 },
	{ 0xb8ca, 0xb8d1, PDF_CMAP_RANGE, 14889 },
	{ 0xb8d3, 0xb8dd, PDF_CMAP_RANGE, 14897 },
	{ 0xb8e1, 0xb8e2, PDF_CMAP_RANGE, 2679 },
	{ 0xb8e3, 0xb8e4, PDF_CMAP_RANGE, 14908 },
	{ 0xb8e5, 0xb8e5, PDF_CMAP_SINGLE, 2681 },
	{ 0xb8e6, 0xb8e8, PDF_CMAP_RANGE, 14910 },
	{ 0xb8e9, 0xb8eb, PDF_CMAP_TABLE, 576 },
	{ 0xb8ec, 0xb8f0, PDF_CMAP_RANGE, 14914 },
	{ 0xb8f1, 0xb8f1, PDF_CMAP_SINGLE, 2684 },
	{ 0xb8f3, 0xb8f6, PDF_CMAP_TABLE, 579 },
	{ 0xb8f7, 0xb8f8, PDF_CMAP_RANGE, 2687 },
	{ 0xb8f9, 0xb8fd, PDF_CMAP_RANGE, 14921 },
	{ 0xb941, 0xb942, PDF_CMAP_RANGE, 2689 },
	{ 0xb943, 0xb944, PDF_CMAP_RANGE, 14926 },
	{ 0xb945, 0xb945, PDF_CMAP_SINGLE, 2691 },
	{ 0xb946, 0xb948, PDF_CMAP_RANGE, 14928 },
	{ 0xb949, 0xb949, PDF_CMAP_SINGLE, 2692 },
	{ 0xb94a, 0xb950, PDF_CMAP_RANGE, 14931 },
	{ 0xb951, 0xb951, PDF_CMAP_SINGLE, 2693 },
	{ 0xb953, 0xb957, PDF_CMAP_TABLE, 583 },
	{ 0xb958, 0xb95d, PDF_CMAP_RANGE, 14940 },
	{ 0xb961, 0xb961, PDF_CMAP_SINGLE, 2697 },
	{ 0xb962, 0xb964, PDF_CMAP_RANGE, 14946 },
	{ 0xb965, 0xb965, PDF_CMAP_SINGLE, 2698 },
	{ 0xb966, 0xb968, PDF_CMAP_RANGE, 14949 },
	{ 0xb969, 0xb969, PDF_CMAP_SINGLE, 2699 },
	{ 0xb96a, 0xb970, PDF_CMAP_RANGE, 14952 },
	{ 0xb971, 0xb971, PDF_CMAP_SINGLE, 2700 },
	{ 0xb973, 0xb973, PDF_CMAP_SINGLE, 2701 },
	{ 0xb974, 0xb975, PDF_CMAP_RANGE, 14959 },
	{ 0xb976, 0xb977, PDF_CMAP_RANGE, 2702 },
	{ 0xb978, 0xb97d, PDF_CMAP_RANGE, 14961 },
	{ 0xb981, 0xb981, PDF_CMAP_SINGLE, 2704 },
	{ 0xb982, 0xb991, PDF_CMAP_RANGE, 14967 },
	{ 0xb993, 0xb99d, PDF_CMAP_RANGE, 14983 },
	{ 0xb9a1, 0xb9a2, PDF_CMAP_RANGE, 2705 },
	{ 0xb9a3, 0xb9a4, PDF_CMAP_RANGE, 14994 },
	{ 0xb9a5, 0xb9a5, PDF_CMAP_SINGLE, 2707 },
	{ 0xb9a6, 0xb9a8, PDF_CMAP_RANGE, 14996 },
	{ 0xb9a9, 0xb9ab, PDF_CMAP_TABLE, 588 },
	{ 0xb9ac, 0xb9b0, PDF_CMAP_RANGE, 15000 },
	{ 0xb9b1, 0xb9b1, PDF_CMAP_SINGLE, 2710 },
	{ 0xb9b3, 0xb9b6, PDF_CMAP_TABLE, 591 },
	{ 0xb9b7, 0xb9b9, PDF_CMAP_RANGE, 2713 },
	{ 0xb9ba, 0xb9bc, PDF_CMAP_RANGE, 15007 },
	{ 0xb9bd, 0xb9bd, PDF_CMAP_SINGLE, 2716 },
	{ 0xb9c1, 0xb9c2, PDF_CMAP_RANGE, 2717 },
	{ 0xb9c3, 0xb9c8, PDF_CMAP_RANGE, 15010 },
	{ 0xb9c9, 0xb9c9, PDF_CMAP_SINGLE, 2719 },
	{ 0xb9ca, 0xb9d1, PDF_CMAP_RANGE, 15016 },
	{ 0xb9d3, 0xb9d7, PDF_CMAP_TABLE, 595 },
	{ 0xb9d8, 0xb9dd, PDF_CMAP_RANGE, 15026 },
	{ 0xb9e1, 0xb9e1, PDF_CMAP_SINGLE, 2723 },
	{ 0xb9e2, 0xb9f1, PDF_CMAP_RANGE, 15032 },
	{ 0xb9f3, 0xb9f5, PDF_CMAP_RANGE, 15048 },
	{ 0xb9f6, 0xb9f7, PDF_CMAP_RANGE, 2724 },
	{ 0xb9f8, 0xb9fd, PDF_CMAP_RANGE, 15051 },
	{ 0xba41, 0xba41, PDF_CMAP_SINGLE, 2726 },
	{ 0xba42, 0xba44, PDF_CMAP_RANGE, 15057 },
	{ 0xba45, 0xba45, PDF_CMAP_SINGLE, 2727 },
	{ 0xba46, 0xba48, PDF_CMAP_RANGE, 15060 },
	{ 0xba49, 0xba49, PDF_CMAP_SINGLE, 2728 },
	{ 0xba4a, 0xba50, PDF_CMAP_RANGE, 15063 },
	{ 0xba51, 0xba51, PDF_CMAP_SINGLE, 2729 },
	{ 0xba53, 0xba57, PDF_CMAP_TABLE, 600 },
	{ 0xba58, 0xba5d, PDF_CMAP_RANGE, 15072 },
	{ 0xba61, 0xba62, PDF_CMAP_RANGE, 2733 },
	{ 0xba63, 0xba64, PDF_CMAP_RANGE, 15078 },
	{ 0xba65, 0xba65, PDF_CMAP_SINGLE, 2735 },
	{ 0xba66, 0xba71, PDF_CMAP_RANGE, 15080 },
	{ 0xba73, 0xba76, PDF_CMAP_RANGE, 15092 },
	{ 0xba77, 0xba77, PDF_CMAP_SINGLE, 2736 },
	{ 0xba78, 0xba7d, PDF_CMAP_RANGE, 15096 },
	{ 0xba81, 0xba82, PDF_CMAP_RANGE, 2737 },
	{ 0xba83, 0xba84, PDF_CMAP_RANGE, 15102 },
	{ 0xba85, 0xba85, PDF_CMAP_SINGLE, 2739 },
	{ 0xba86, 0xba88, PDF_CMAP_RANGE, 15104 },
	{ 0xba89, 0xba8b, PDF_CMAP_RANGE, 2740 },
	{ 0xba8c, 0xba90, PDF_CMAP_RANGE, 15107 },
	{ 0xba91, 0xba91, PDF_CMAP_SINGLE, 2743 },
	{ 0xba93, 0xba97, PDF_CMAP_TABLE, 605 },
	{ 0xba98, 0xba9d, PDF_CMAP_RANGE, 15114 },
	{ 0xbaa1, 0xbaa1, PDF_CMAP_SINGLE, 2747 },
	{ 0xbaa2, 0xbab1, PDF_CMAP_RANGE, 15120 },
	{ 0xbab3, 0xbab5, PDF_CMAP_RANGE, 15136 },
	{ 0xbab6, 0xbab6, PDF_CMAP_SINGLE, 2748 },
	{ 0xbab7, 0xbabd, PDF_CMAP_RANGE, 15139 },
	{ 0xbac1, 0xbac1, PDF_CMAP_SINGLE, 2749 },
	{ 0xbac2, 0xbad1, PDF_CMAP_RANGE, 15146 },
	{ 0xbad3, 0xbadd, PDF_CMAP_RANGE, 15162 },
	{ 0xbae1, 0xbae2, PDF_CMAP_RANGE, 2750 },
	{ 0xbae3, 0xbae4, PDF_CMAP_RANGE, 15173 },
	{ 0xbae5, 0xbae5, PDF_CMAP_SINGLE, 2752 },
	{ 0xbae6, 0xbae8, PDF_CMAP_RANGE, 15175 },
	{ 0xbae9, 0xbae9, PDF_CMAP_SINGLE, 2753 },
	{ 0xbaea, 0xbaf0, PDF_CMAP_RANGE, 15178 },
	{ 0xbaf1, 0xbaf1, PDF_CMAP_SINGLE, 2754 },
	{ 0xbaf3, 0xbaf5, PDF_CMAP_TABLE, 610 },
	{ 0xbaf6, 0xbafd, PDF_CMAP_RANGE, 15186 },
	{ 0xbb41, 0xbb41, PDF_CMAP_SINGLE, 2757 },
	{ 0xbb42, 0xbb44, PDF_CMAP_RANGE, 15194 },
	{ 0xbb45, 0xbb45, PDF_CMAP_SINGLE, 2758 },
	{ 0xbb46, 0xbb48, PDF_CMAP_RANGE, 15197 },
	{ 0xbb49, 0xbb49, PDF_CMAP_SINGLE, 2759 },
	{ 0xbb4a, 0xbb50, PDF_CMAP_RANGE, 15200 },
	{ 0xbb51, 0xbb51, PDF_CMAP_SINGLE, 2760 },
	{ 0xbb53, 0xbb5d, PDF_CMAP_RANGE, 15207 },
	{ 0xbb61, 0xbb62, PDF_CMAP_RANGE, 2761 },
	{ 0xbb63, 0xbb64, PDF_CMAP_RANGE, 15218 },
	{ 0xbb65, 0xbb65, PDF_CMAP_SINGLE, 2763 },
	{ 0xbb66, 0xbb68, PDF_CMAP_RANGE, 15220 },
	{ 0xbb69, 0xbb69, PDF_CMAP_SINGLE, 2764 },
	{ 0xbb6a, 0xbb70, PDF_CMAP_RANGE, 15223 },
	{ 0xbb71, 0xbb71, PDF_CMAP_SINGLE, 2765 },
	{ 0xbb73, 0xbb77, PDF_CMAP_TABLE, 613 },
	{ 0xbb78, 0xbb7d, PDF_CMAP_RANGE, 15232 },
	{ 0xbb81, 0xbb91, PDF_CMAP_RANGE, 15238 },
	{ 0xbb93, 0xbb9d, PDF_CMAP_RANGE, 15255 },
	{ 0xbba1, 0xbba2, PDF_CMAP_RANGE, 2769 },
	{ 0xbba3, 0xbba4, PDF_CMAP_RANGE, 15266 },
	{ 0xbba5, 0xbba5, PDF_CMAP_SINGLE, 2771 },
	{ 0xbba6, 0xbba7, PDF_CMAP_RANGE, 15268 },
	{ 0xbba8, 0xbba9, PDF_CMAP_RANGE, 2772 },
	{ 0xbbaa, 0xbbab, PDF_CMAP_TABLE, 618 },
	{ 0xbbac, 0xbbb0, PDF_CMAP_RANGE, 15271 },
	{ 0xbbb1, 0xbbb1, PDF_CMAP_SINGLE, 2775 },
	{ 0xbbb3, 0xbbb6, PDF_CMAP_TABLE, 620 },
	{ 0xbbb7, 0xbbb8, PDF_CMAP_RANGE, 2778 },
	{ 0xbbb9, 0xbbba, PDF_CMAP_RANGE, 15278 },
	{ 0xbbbb, 0xbbbc, PDF_CMAP_RANGE, 2780 },
	{ 0xbbbd, 0xbbbd, PDF_CMAP_SINGLE, 15280 },
	{ 0xbc41, 0xbc41, PDF_CMAP_SINGLE, 382 },
	{ 0xbc61, 0xbc62, PDF_CMAP_RANGE, 2782 },
	{ 0xbc63, 0xbc64, PDF_CMAP_RANGE, 15281 },
	{ 0xbc65, 0xbc69, PDF_CMAP_TABLE, 624 },
	{ 0xbc6a, 0xbc6b, PDF_CMAP_RANGE, 15285 },
	{ 0xbc6c, 0xbc6c, PDF_CMAP_SINGLE, 2787 },
	{ 0xbc6d, 0xbc70, PDF_CMAP_RANGE, 15287 },
	{ 0xbc71, 0xbc71, PDF_CMAP_SINGLE, 2788 },
	{ 0xbc73, 0xbc74, PDF_CMAP_TABLE, 629 },
	{ 0xbc75, 0xbc77, PDF_CMAP_RANGE, 2790 },
	{ 0xbc78, 0xbc7d, PDF_CMAP_RANGE, 15292 },
	{ 0xbc81, 0xbc82, PDF_CMAP_RANGE, 2793 },
	{ 0xbc83, 0xbc84, PDF_CMAP_RANGE, 15298 },
	{ 0xbc85, 0xbc85, PDF_CMAP_SINGLE, 2795 },
	{ 0xbc86, 0xbc88, PDF_CMAP_RANGE, 15300 },
	{ 0xbc89, 0xbc89, PDF_CMAP_SINGLE, 2796 },
	{ 0xbc8a, 0xbc90, PDF_CMAP_RANGE, 15303 },
	{ 0xbc91, 0xbc91, PDF_CMAP_SINGLE, 2797 },
	{ 0xbc93, 0xbc94, PDF_CMAP_TABLE, 631 },
	{ 0xbc95, 0xbc97, PDF_CMAP_RANGE, 2799 },
	{ 0xbc98, 0xbc9d, PDF_CMAP_RANGE, 15311 },
	{ 0xbca1, 0xbca1, PDF_CMAP_SINGLE, 2802 },
	{ 0xbca2, 0xbca4, PDF_CMAP_RANGE, 15317 },
	{ 0xbca5, 0xbca5, PDF_CMAP_SINGLE, 2803 },
	{ 0xbca6, 0xbcb1, PDF_CMAP_RANGE, 15320 },
	{ 0xbcb3, 0xbcb6, PDF_CMAP_RANGE, 15332 },
	{ 0xbcb7, 0xbcb7, PDF_CMAP_SINGLE, 2804 },
	{ 0xbcb8, 0xbcbd, PDF_CMAP_RANGE, 15336 },
	{ 0xbcc1, 0xbcd1, PDF_CMAP_RANGE, 15342 },
	{ 0xbcd3, 0xbcdd, PDF_CMAP_RANGE, 15359 },
	{ 0xbce1, 0xbce2, PDF_CMAP_RANGE, 2805 },
	{ 0xbce3, 0xbce4, PDF_CMAP_RANGE, 15370 },
	{ 0xbce5, 0xbce5, PDF_CMAP_SINGLE, 2807 },
	{ 0xbce6, 0xbce8, PDF_CMAP_RANGE, 15372 },
	{ 0xbce9, 0xbce9, PDF_CMAP_SINGLE, 2808 },
	{ 0xbcea, 0xbcf0, PDF_CMAP_RANGE, 15375 },
	{ 0xbcf1, 0xbcf1, PDF_CMAP_SINGLE, 2809 },
	{ 0xbcf3, 0xbcf4, PDF_CMAP_TABLE, 633 },
	{ 0xbcf5, 0xbcf7, PDF_CMAP_RANGE, 2811 },
	{ 0xbcf8, 0xbcfd, PDF_CMAP_RANGE, 15383 },
	{ 0xbd41, 0xbd41, PDF_CMAP_SINGLE, 2814 },
	{ 0xbd42, 0xbd51, PDF_CMAP_RANGE, 15389 },
	{ 0xbd53, 0xbd56, PDF_CMAP_RANGE, 15405 },
	{ 0xbd57, 0xbd57, PDF_CMAP_SINGLE, 2815 },
	{ 0xbd58, 0xbd5d, PDF_CMAP_RANGE, 15409 },
	{ 0xbd61, 0xbd61, PDF_CMAP_SINGLE, 2816 },
	{ 0xbd62, 0xbd71, PDF_CMAP_RANGE, 15415 },
	{ 0xbd73, 0xbd75, PDF_CMAP_RANGE, 15431 },
	{ 0xbd76, 0xbd76, PDF_CMAP_SINGLE, 2817 },
	{ 0xbd77, 0xbd7d, PDF_CMAP_RANGE, 15434 },
	{ 0xbd81, 0xbd91, PDF_CMAP_RANGE, 15441 },
	{ 0xbd93, 0xbd9d, PDF_CMAP_RANGE, 15458 },
	{ 0xbda1, 0xbda2, PDF_CMAP_RANGE, 2818 },
	{ 0xbda3, 0xbda4, PDF_CMAP_RANGE, 15469 },
	{ 0xbda5, 0xbda5, PDF_CMAP_SINGLE, 2820 },
	{ 0xbda6, 0xbda8, PDF_CMAP_RANGE, 15471 },
	{ 0xbda9, 0xbda9, PDF_CMAP_SINGLE, 2821 },
	{ 0xbdaa, 0xbdb0, PDF_CMAP_RANGE, 15474 },
	{ 0xbdb1, 0xbdb1, PDF_CMAP_SINGLE, 2822 },
	{ 0xbdb3, 0xbdb9, PDF_CMAP_TABLE, 635 },
	{ 0xbdba, 0xbdbd, PDF_CMAP_RANGE, 15484 },
	{ 0xbdc1, 0xbdc2, PDF_CMAP_RANGE, 2827 },
	{ 0xbdc3, 0xbdc8, PDF_CMAP_RANGE, 15488 },
	{ 0xbdc9, 0xbdc9, PDF_CMAP_SINGLE, 2829 },
	{ 0xbdca, 0xbdd1, PDF_CMAP_RANGE, 15494 },
	{ 0xbdd3, 0xbdd5, PDF_CMAP_RANGE, 15502 },
	{ 0xbdd6, 0xbdd6, PDF_CMAP_SINGLE, 2830 },
	{ 0xbdd7, 0xbddd, PDF_CMAP_RANGE, 15505 },
	{ 0xbde1, 0xbde1, PDF_CMAP_SINGLE, 2831 },
	{ 0xbde2, 0xbdf1, PDF_CMAP_RANGE, 15512 },
	{ 0xbdf3, 0xbdf5, PDF_CMAP_RANGE, 15528 },
	{ 0xbdf6, 0xbdf6, PDF_CMAP_SINGLE, 2832 },
	{ 0xbdf7, 0xbdfd, PDF_CMAP_RANGE, 15531 },
	{ 0xbe41, 0xbe41, PDF_CMAP_SINGLE, 2833 },
	{ 0xbe42, 0xbe44, PDF_CMAP_RANGE, 15538 },
	{ 0xbe45, 0xbe45, PDF_CMAP_SINGLE, 2834 },
	{ 0xbe46, 0xbe48, PDF_CMAP_RANGE, 15541 },
	{ 0xbe49, 0xbe49, PDF_CMAP_SINGLE, 2835 },
	{ 0xbe4a, 0xbe50, PDF_CMAP_RANGE, 15544 },
	{ 0xbe51, 0xbe51, PDF_CMAP_SINGLE, 2836 },
	{ 0xbe53, 0xbe53, PDF_CMAP_SINGLE, 2837 },
	{ 0xbe54, 0xbe5d, PDF_CMAP_RANGE, 15551 },
	{ 0xbe61, 0xbe71, PDF_CMAP_RANGE, 15561 },
	{ 0xbe73, 0xbe76, PDF_CMAP_RANGE, 15578 },
	{ 0xbe77, 0xbe77, PDF_CMAP_SINGLE, 2838 },
	{ 0xbe78, 0xbe7d, PDF_CMAP_RANGE, 15582 },
	{ 0xbe81, 0xbe82, PDF_CMAP_RANGE, 2839 },
	{ 0xbe83, 0xbe84, PDF_CMAP_RANGE, 15588 },
	{ 0xbe85, 0xbe85, PDF_CMAP_SINGLE, 2841 },
	{ 0xbe86, 0xbe88, PDF_CMAP_RANGE, 15590 },
	{ 0xbe89, 0xbe89, PDF_CMAP_SINGLE, 2842 },
	{ 0xbe8a, 0xbe90, PDF_CMAP_RANGE, 15593 },
	{ 0xbe91, 0xbe91, PDF_CMAP_SINGLE, 2843 },
	{ 0xbe93, 0xbe93, PDF_CMAP_SINGLE, 2844 },
	{ 0xbe94, 0xbe96, PDF_CMAP_RANGE, 15600 },
	{ 0xbe97, 0xbe97, PDF_CMAP_SINGLE, 2845 },
	{ 0xbe98, 0xbe9d, PDF_CMAP_RANGE, 15603 },
	{ 0xbea1, 0xbea1, PDF_CMAP_SINGLE, 2846 },
	{ 0xbea2, 0xbeb1, PDF_CMAP_RANGE, 15609 },
	{ 0xbeb3, 0xbeb5, PDF_CMAP_RANGE, 15625 },
	{ 0xbeb6, 0xbeb7, PDF_CMAP_RANGE, 2847 },
	{ 0xbeb8, 0xbebd, PDF_CMAP_RANGE, 15628 },
	{ 0xbec1, 0xbed1, PDF_CMAP_RANGE, 15634 },
	{ 0xbed3, 0xbedd, PDF_CMAP_RANGE, 15651 },
	{ 0xbee1, 0xbee1, PDF_CMAP_SINGLE, 2849 },
	{ 0xbee2, 0xbef1, PDF_CMAP_RANGE, 15662 },
	{ 0xbef3, 0xbefd, PDF_CMAP_RANGE, 15678 },
	{ 0xbf41, 0xbf41, PDF_CMAP_SINGLE, 2850 },
	{ 0xbf42, 0xbf51, PDF_CMAP_RANGE, 15689 },
	{ 0xbf53, 0xbf5d, PDF_CMAP_RANGE, 15705 },
	{ 0xbf61, 0xbf61, PDF_CMAP_SINGLE, 2851 },
	{ 0xbf62, 0xbf70, PDF_CMAP_RANGE, 15716 },
	{ 0xbf71, 0xbf71, PDF_CMAP_SINGLE, 2852 },
	{ 0xbf73, 0xbf74, PDF_CMAP_RANGE, 15731 },
	{ 0xbf75, 0xbf77, PDF_CMAP_TABLE, 642 },
	{ 0xbf78, 0xbf7d, PDF_CMAP_RANGE, 15734 },
	{ 0xbf81, 0xbf91, PDF_CMAP_RANGE, 15740 },
	{ 0xbf93, 0xbf9d, PDF_CMAP_RANGE, 15757 },
	{ 0xbfa1, 0xbfa2, PDF_CMAP_RANGE, 2855 },
	{ 0xbfa3, 0xbfa4, PDF_CMAP_RANGE, 15768 },
	{ 0xbfa5, 0xbfa5, PDF_CMAP_SINGLE, 2857 },
	{ 0xbfa6, 0xbfa8, PDF_CMAP_RANGE, 15770 },
	{ 0xbfa9, 0xbfa9, PDF_CMAP_SINGLE, 2858 },
	{ 0xbfaa, 0xbfb0, PDF_CMAP_RANGE, 15773 },
	{ 0xbfb1, 0xbfb1, PDF_CMAP_SINGLE, 2859 },
	{ 0xbfb3, 0xbfb3, PDF_CMAP_SINGLE, 2860 },
	{ 0xbfb4, 0xbfb6, PDF_CMAP_RANGE, 15780 },
	{ 0xbfb7, 0xbfb8, PDF_CMAP_RANGE, 2861 },
	{ 0xbfb9, 0xbfbc, PDF_CMAP_RANGE, 15783 },
	{ 0xbfbd, 0xbfbd, PDF_CMAP_SINGLE, 2863 },
	{ 0xc041, 0xc041, PDF_CMAP_SINGLE, 383 },
	{ 0xc061, 0xc062, PDF_CMAP_RANGE, 2864 },
	{ 0xc063, 0xc064, PDF_CMAP_RANGE, 15787 },
	{ 0xc065, 0xc069, PDF_CMAP_TABLE, 645 },
	{ 0xc06a, 0xc070, PDF_CMAP_RANGE, 15791 },
	{ 0xc071, 0xc071, PDF_CMAP_SINGLE, 2869 },
	{ 0xc073, 0xc074, PDF_CMAP_TABLE, 650 },
	{ 0xc075, 0xc078, PDF_CMAP_RANGE, 2871 },
	{ 0xc079, 0xc07d, PDF_CMAP_RANGE, 15799 },
	{ 0xc081, 0xc082, PDF_CMAP_RANGE, 2875 },
	{ 0xc083, 0xc084, PDF_CMAP_RANGE, 15804 },
	{ 0xc085, 0xc085, PDF_CMAP_SINGLE, 2877 },
	{ 0xc086, 0xc088, PDF_CMAP_RANGE, 15806 },
	{ 0xc089, 0xc089, PDF_CMAP_SINGLE, 2878 },
	{ 0xc08a, 0xc090, PDF_CMAP_RANGE, 15809 },
	{ 0xc091, 0xc091, PDF_CMAP_SINGLE, 2879 },
	{ 0xc093, 0xc094, PDF_CMAP_TABLE, 652 },
	{ 0xc095, 0xc097, PDF_CMAP_RANGE, 2881 },
	{ 0xc098, 0xc09d, PDF_CMAP_RANGE, 15817 },
	{ 0xc0a1, 0xc0a1, PDF_CMAP_SINGLE, 2884 },
	{ 0xc0a2, 0xc0a4, PDF_CMAP_RANGE, 15823 },
	{ 0xc0a5, 0xc0a9, PDF_CMAP_TABLE, 654 },
	{ 0xc0aa, 0xc0b0, PDF_CMAP_RANGE, 15828 },
	{ 0xc0b1, 0xc0b1, PDF_CMAP_SINGLE, 2888 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 15835 },
	{ 0xc0b7, 0xc0b7, PDF_CMAP_SINGLE, 2889 },
	{ 0xc0b8, 0xc0bd, PDF_CMAP_RANGE, 15839 },
	{ 0xc0c1, 0xc0d1, PDF_CMAP_RANGE, 15845 },
	{ 0xc0d3, 0xc0dd, PDF_CMAP_RANGE, 15862 },
	{ 0xc0e1, 0xc0e2, PDF_CMAP_RANGE, 2890 },
	{ 0xc0e3, 0xc0e4, PDF_CMAP_RANGE, 15873 },
	{ 0xc0e5, 0xc0e5, PDF_CMAP_SINGLE, 2892 },
	{ 0xc0e6, 0xc0e8, PDF_CMAP_RANGE, 15875 },
	{ 0xc0e9, 0xc0e9, PDF_CMAP_SINGLE, 2893 },
	{ 0xc0ea, 0xc0f0, PDF_CMAP_RANGE, 15878 },
	{ 0xc0f1, 0xc0f1, PDF_CMAP_SINGLE, 2894 },
	{ 0xc0f3, 0xc0f4, PDF_CMAP_TABLE, 659 },
	{ 0xc0f5, 0xc0f7, PDF_CMAP_RANGE, 2896 },
	{ 0xc0f8, 0xc0fd, PDF_CMAP_RANGE, 15886 },
	{ 0xc141, 0xc142, PDF_CMAP_RANGE, 2899 },
	{ 0xc143, 0xc144, PDF_CMAP_RANGE, 15892 },
	{ 0xc145, 0xc145, PDF_CMAP_SINGLE, 2901 },
	{ 0xc146, 0xc148, PDF_CMAP_RANGE, 15894 },
	{ 0xc149, 0xc149, PDF_CMAP_SINGLE, 2902 },
	{ 0xc14a, 0xc150, PDF_CMAP_RANGE, 15897 },
	{ 0xc151, 0xc151, PDF_CMAP_SINGLE, 2903 },
	{ 0xc153, 0xc157, PDF_CMAP_TABLE, 661 },
	{ 0xc158, 0xc15d, PDF_CMAP_RANGE, 15906 },
	{ 0xc161, 0xc161, PDF_CMAP_SINGLE, 2907 },
	{ 0xc162, 0xc164, PDF_CMAP_RANGE, 15912 },
	{ 0xc165, 0xc165, PDF_CMAP_SINGLE, 2908 },
	{ 0xc166, 0xc171, PDF_CMAP_RANGE, 15915 },
	{ 0xc173, 0xc175, PDF_CMAP_RANGE, 15927 },
	{ 0xc176, 0xc176, PDF_CMAP_SINGLE, 2909 },
	{ 0xc177, 0xc17d, PDF_CMAP_RANGE, 15930 },
	{ 0xc181, 0xc181, PDF_CMAP_SINGLE, 2910 },
	{ 0xc182, 0xc184, PDF_CMAP_RANGE, 15937 },
	{ 0xc185, 0xc185, PDF_CMAP_SINGLE, 2911 },
	{ 0xc186, 0xc191, PDF_CMAP_RANGE, 15940 },
	{ 0xc193, 0xc196, PDF_CMAP_RANGE, 15952 },
	{ 0xc197, 0xc197, PDF_CMAP_SINGLE, 2912 },
	{ 0xc198, 0xc19d, PDF_CMAP_RANGE, 15956 },
	{ 0xc1a1, 0xc1a2, PDF_CMAP_RANGE, 2913 },
	{ 0xc1a3, 0xc1a4, PDF_CMAP_RANGE, 15962 },
	{ 0xc1a5, 0xc1a5, PDF_CMAP_SINGLE, 2915 },
	{ 0xc1a6, 0xc1a8, PDF_CMAP_RANGE, 15964 },
	{ 0xc1a9, 0xc1a9, PDF_CMAP_SINGLE, 2916 },
	{ 0xc1aa, 0xc1b0, PDF_CMAP_RANGE, 15967 },
	{ 0xc1b1, 0xc1b1, PDF_CMAP_SINGLE, 2917 },
	{ 0xc1b3, 0xc1b7, PDF_CMAP_TABLE, 666 },
	{ 0xc1b8, 0xc1bd, PDF_CMAP_RANGE, 15976 },
	{ 0xc1c1, 0xc1c1, PDF_CMAP_SINGLE, 2921 },
	{ 0xc1c2, 0xc1c4, PDF_CMAP_RANGE, 15982 },
	{ 0xc1c5, 0xc1c5, PDF_CMAP_SINGLE, 2922 },
	{ 0xc1c6, 0xc1c8, PDF_CMAP_RANGE, 15985 },
	{ 0xc1c9, 0xc1c9, PDF_CMAP_SINGLE, 2923 },
	{ 0xc1ca, 0xc1d1, PDF_CMAP_RANGE, 15988 },
	{ 0xc1d3, 0xc1d6, PDF_CMAP_RANGE, 15996 },
	{ 0xc1d7, 0xc1d7, PDF_CMAP_SINGLE, 2924 },
	{ 0xc1d8, 0xc1dd, PDF_CMAP_RANGE, 16000 },
	{ 0xc1e1, 0xc1f1, PDF_CMAP_RANGE, 16006 },
	{ 0xc1f3, 0xc1fd, PDF_CMAP_RANGE, 16023 },
	{ 0xc241, 0xc241, PDF_CMAP_SINGLE, 2925 },
	{ 0xc242, 0xc244, PDF_CMAP_RANGE, 16034 },
	{ 0xc245, 0xc245, PDF_CMAP_SINGLE, 2926 },
	{ 0xc246, 0xc248, PDF_CMAP_RANGE, 16037 },
	{ 0xc249, 0xc249, PDF_CMAP_SINGLE, 2927 },
	{ 0xc24a, 0xc250, PDF_CMAP_RANGE, 16040 },
	{ 0xc251, 0xc251, PDF_CMAP_SINGLE, 2928 },
	{ 0xc253, 0xc257, PDF_CMAP_TABLE, 671 },
	{ 0xc258, 0xc25d, PDF_CMAP_RANGE, 16049 },
	{ 0xc261, 0xc261, PDF_CMAP_SINGLE, 2932 },
	{ 0xc262, 0xc270, PDF_CMAP_RANGE, 16055 },
	{ 0xc271, 0xc271, PDF_CMAP_SINGLE, 2933 },
	{ 0xc273, 0xc27d, PDF_CMAP_RANGE, 16070 },
	{ 0xc281, 0xc282, PDF_CMAP_RANGE, 2934 },
	{ 0xc283, 0xc284, PDF_CMAP_RANGE, 16081 },
	{ 0xc285, 0xc285, PDF_CMAP_SINGLE, 2936 },
	{ 0xc286, 0xc288, PDF_CMAP_RANGE, 16083 },
	{ 0xc289, 0xc289, PDF_CMAP_SINGLE, 2937 },
	{ 0xc28a, 0xc290, PDF_CMAP_RANGE, 16086 },
	{ 0xc291, 0xc291, PDF_CMAP_SINGLE, 2938 },
	{ 0xc293, 0xc297, PDF_CMAP_TABLE, 676 },
	{ 0xc298, 0xc29d, PDF_CMAP_RANGE, 16095 },
	{ 0xc2a1, 0xc2a1, PDF_CMAP_SINGLE, 2942 },
	{ 0xc2a2, 0xc2b1, PDF_CMAP_RANGE, 16101 },
	{ 0xc2b3, 0xc2b5, PDF_CMAP_RANGE, 16117 },
	{ 0xc2b6, 0xc2b6, PDF_CMAP_SINGLE, 2943 },
	{ 0xc2b7, 0xc2bd, PDF_CMAP_RANGE, 16120 },
	{ 0xc2c1, 0xc2c1, PDF_CMAP_SINGLE, 2944 },
	{ 0xc2c2, 0xc2c4, PDF_CMAP_RANGE, 16127 },
	{ 0xc2c5, 0xc2c5, PDF_CMAP_SINGLE, 2945 },
	{ 0xc2c6, 0xc2d1, PDF_CMAP_RANGE, 16130 },
	{ 0xc2d3, 0xc2dd, PDF_CMAP_RANGE, 16142 },
	{ 0xc2e1, 0xc2e1, PDF_CMAP_SINGLE, 2946 },
	{ 0xc2e2, 0xc2e4, PDF_CMAP_RANGE, 16153 },
	{ 0xc2e5, 0xc2e5, PDF_CMAP_SINGLE, 2947 },
	{ 0xc2e6, 0xc2e8, PDF_CMAP_RANGE, 16156 },
	{ 0xc2e9, 0xc2e9, PDF_CMAP_SINGLE, 2948 },
	{ 0xc2ea, 0xc2f0, PDF_CMAP_RANGE, 16159 },
	{ 0xc2f1, 0xc2f1, PDF_CMAP_SINGLE, 2949 },
	{ 0xc2f3, 0xc2f7, PDF_CMAP_TABLE, 681 },
	{ 0xc2f8, 0xc2fd, PDF_CMAP_RANGE, 16168 },
	{ 0xc341, 0xc341, PDF_CMAP_SINGLE, 2953 },
	{ 0xc342, 0xc344, PDF_CMAP_RANGE, 16174 },
	{ 0xc345, 0xc345, PDF_CMAP_SINGLE, 2954 },
	{ 0xc346, 0xc348, PDF_CMAP_RANGE, 16177 },
	{ 0xc349, 0xc349, PDF_CMAP_SINGLE, 2955 },
	{ 0xc34a, 0xc350, PDF_CMAP_RANGE, 16180 },
	{ 0xc351, 0xc351, PDF_CMAP_SINGLE, 2956 },
	{ 0xc353, 0xc356, PDF_CMAP_RANGE, 16187 },
	{ 0xc357, 0xc357, PDF_CMAP_SINGLE, 2957 },
	{ 0xc358, 0xc35d, PDF_CMAP_RANGE, 16191 },
	{ 0xc361, 0xc362, PDF_CMAP_RANGE, 2958 },
	{ 0xc363, 0xc364, PDF_CMAP_RANGE, 16197 },
	{ 0xc365, 0xc365, PDF_CMAP_SINGLE, 2960 },
	{ 0xc366, 0xc368, PDF_CMAP_RANGE, 16199 },
	{ 0xc369, 0xc369, PDF_CMAP_SINGLE, 2961 },
	{ 0xc36a, 0xc370, PDF_CMAP_RANGE, 16202 },
	{ 0xc371, 0xc371, PDF_CMAP_SINGLE, 2962 },
	{ 0xc373, 0xc377, PDF_CMAP_TABLE, 686 },
	{ 0xc378, 0xc37d, PDF_CMAP_RANGE, 16211 },
	{ 0xc381, 0xc391, PDF_CMAP_RANGE, 16217 },
	{ 0xc393, 0xc39d, PDF_CMAP_RANGE, 16234 },
	{ 0xc3a1, 0xc3a2, PDF_CMAP_RANGE, 2966 },
	{ 0xc3a3, 0xc3a4, PDF_CMAP_RANGE, 16245 },
	{ 0xc3a5, 0xc3a5, PDF_CMAP_SINGLE, 2968 },
	{ 0xc3a6, 0xc3a7, PDF_CMAP_RANGE, 16247 },
	{ 0xc3a8, 0xc3aa, PDF_CMAP_RANGE, 2969 },
	{ 0xc3ab, 0xc3b0, PDF_CMAP_RANGE, 16249 },
	{ 0xc3b1, 0xc3b1, PDF_CMAP_SINGLE, 2972 },
	{ 0xc3b3, 0xc3b7, PDF_CMAP_TABLE, 691 },
	{ 0xc3b8, 0xc3bd, PDF_CMAP_RANGE, 16257 },
	{ 0xc441, 0xc441, PDF_CMAP_SINGLE, 384 },
	{ 0xc461, 0xc462, PDF_CMAP_RANGE, 2976 },
	{ 0xc463, 0xc464, PDF_CMAP_RANGE, 16263 },
	{ 0xc465, 0xc465, PDF_CMAP_SINGLE, 2978 },
	{ 0xc466, 0xc468, PDF_CMAP_RANGE, 16265 },
	{ 0xc469, 0xc469, PDF_CMAP_SINGLE, 2979 },
	{ 0xc46a, 0xc470, PDF_CMAP_RANGE, 16268 },
	{ 0xc471, 0xc471, PDF_CMAP_SINGLE, 2980 },
	{ 0xc473, 0xc477, PDF_CMAP_TABLE, 696 },
	{ 0xc478, 0xc47d, PDF_CMAP_RANGE, 16277 },
	{ 0xc481, 0xc482, PDF_CMAP_RANGE, 2984 },
	{ 0xc483, 0xc484, PDF_CMAP_RANGE, 16283 },
	{ 0xc485, 0xc485, PDF_CMAP_SINGLE, 2986 },
	{ 0xc486, 0xc488, PDF_CMAP_RANGE, 16285 },
	{ 0xc489, 0xc489, PDF_CMAP_SINGLE, 2987 },
	{ 0xc48a, 0xc490, PDF_CMAP_RANGE, 16288 },
	{ 0xc491, 0xc491, PDF_CMAP_SINGLE, 2988 },
	{ 0xc493, 0xc494, PDF_CMAP_TABLE, 701 },
	{ 0xc495, 0xc497, PDF_CMAP_RANGE, 2990 },
	{ 0xc498, 0xc49d, PDF_CMAP_RANGE, 16296 },
	{ 0xc4a1, 0xc4a2, PDF_CMAP_RANGE, 2993 },
	{ 0xc4a3, 0xc4b1, PDF_CMAP_RANGE, 16302 },
	{ 0xc4b3, 0xc4b6, PDF_CMAP_RANGE, 16317 },
	{ 0xc4b7, 0xc4b7, PDF_CMAP_SINGLE, 2995 },
	{ 0xc4b8, 0xc4bd, PDF_CMAP_RANGE, 16321 },
	{ 0xc4c1, 0xc4d1, PDF_CMAP_RANGE, 16327 },
	{ 0xc4d3, 0xc4dd, PDF_CMAP_RANGE, 16344 },
	{ 0xc4e1, 0xc4e2, PDF_CMAP_RANGE, 2996 },
	{ 0xc4e3, 0xc4e4, PDF_CMAP_RANGE, 16355 },
	{ 0xc4e5, 0xc4e5, PDF_CMAP_SINGLE, 2998 },
	{ 0xc4e6, 0xc4e7, PDF_CMAP_RANGE, 16357 },
	{ 0xc4e8, 0xc4e9, PDF_CMAP_RANGE, 2999 },
	{ 0xc4ea, 0xc4f0, PDF_CMAP_RANGE, 16359 },
	{ 0xc4f1, 0xc4f1, PDF_CMAP_SINGLE, 3001 },
	{ 0xc4f3, 0xc4f4, PDF_CMAP_TABLE, 703 },
	{ 0xc4f5, 0xc4f7, PDF_CMAP_RANGE, 3003 },
	{ 0xc4f8, 0xc4fd, PDF_CMAP_RANGE, 16367 },
	{ 0xc541, 0xc542, PDF_CMAP_RANGE, 3006 },
	{ 0xc543, 0xc544, PDF_CMAP_RANGE, 16373 },
	{ 0xc545, 0xc545, PDF_CMAP_SINGLE, 3008 },
	{ 0xc546, 0xc548, PDF_CMAP_RANGE, 16375 },
	{ 0xc549, 0xc549, PDF_CMAP_SINGLE, 3009 },
	{ 0xc54a, 0xc550, PDF_CMAP_RANGE, 16378 },
	{ 0xc551, 0xc551, PDF_CMAP_SINGLE, 3010 },
	{ 0xc553, 0xc557, PDF_CMAP_TABLE, 705 },
	{ 0xc558, 0xc55d, PDF_CMAP_RANGE, 16387 },
	{ 0xc561, 0xc561, PDF_CMAP_SINGLE, 3014 },
	{ 0xc562, 0xc564, PDF_CMAP_RANGE, 16393 },
	{ 0xc565, 0xc565, PDF_CMAP_SINGLE, 3015 },
	{ 0xc566, 0xc568, PDF_CMAP_RANGE, 16396 },
	{ 0xc569, 0xc569, PDF_CMAP_SINGLE, 3016 },
	{ 0xc56a, 0xc570, PDF_CMAP_RANGE, 16399 },
	{ 0xc571, 0xc571, PDF_CMAP_SINGLE, 3017 },
	{ 0xc573, 0xc574, PDF_CMAP_TABLE, 710 },
	{ 0xc575, 0xc577, PDF_CMAP_RANGE, 3019 },
	{ 0xc578, 0xc57d, PDF_CMAP_RANGE, 16407 },
	{ 0xc581, 0xc581, PDF_CMAP_SINGLE, 3022 },
	{ 0xc582, 0xc591, PDF_CMAP_RANGE, 16413 },
	{ 0xc593, 0xc59d, PDF_CMAP_RANGE, 16429 },
	{ 0xc5a1, 0xc5a2, PDF_CMAP_RANGE, 3023 },
	{ 0xc5a3, 0xc5a4, PDF_CMAP_RANGE, 16440 },
	{ 0xc5a5, 0xc5a5, PDF_CMAP_SINGLE, 3025 },
	{ 0xc5a6, 0xc5a8, PDF_CMAP_RANGE, 16442 },
	{ 0xc5a9, 0xc5a9, PDF_CMAP_SINGLE, 3026 },
	{ 0xc5aa, 0xc5b0, PDF_CMAP_RANGE, 16445 },
	{ 0xc5b1, 0xc5b1, PDF_CMAP_SINGLE, 3027 },
	{ 0xc5b3, 0xc5b7, PDF_CMAP_TABLE, 712 },
	{ 0xc5b8, 0xc5bd, PDF_CMAP_RANGE, 16454 },
	{ 0xc5c1, 0xc5c2, PDF_CMAP_RANGE, 3031 },
	{ 0xc5c3, 0xc5c4, PDF_CMAP_RANGE, 16460 },
	{ 0xc5c5, 0xc5c5, PDF_CMAP_SINGLE, 3033 },
	{ 0xc5c6, 0xc5c8, PDF_CMAP_RANGE, 16462 },
	{ 0xc5c9, 0xc5c9, PDF_CMAP_SINGLE, 3034 },
	{ 0xc5ca, 0xc5d0, PDF_CMAP_RANGE, 16465 },
	{ 0xc5d1, 0xc5d1, PDF_CMAP_SINGLE, 3035 },
	{ 0xc5d3, 0xc5d6, PDF_CMAP_RANGE, 16472 },
	{ 0xc5d7, 0xc5d7, PDF_CMAP_SINGLE, 3036 },
	{ 0xc5d8, 0xc5dd, PDF_CMAP_RANGE, 16476 },
	{ 0xc5e1, 0xc5e1, PDF_CMAP_SINGLE, 3037 },
	{ 0xc5e2, 0xc5f1, PDF_CMAP_RANGE, 16482 },
	{ 0xc5f3, 0xc5f6, PDF_CMAP_RANGE, 16498 },
	{ 0xc5f7, 0xc5f7, PDF_CMAP_SINGLE, 3038 },
	{ 0xc5f8, 0xc5fd, PDF_CMAP_RANGE, 16502 },
	{ 0xc641, 0xc641, PDF_CMAP_SINGLE, 3039 },
	{ 0xc642, 0xc648, PDF_CMAP_RANGE, 16508 },
	{ 0xc649, 0xc649, PDF_CMAP_SINGLE, 3040 },
	{ 0xc64a, 0xc651, PDF_CMAP_RANGE, 16515 },
	{ 0xc653, 0xc65d, PDF_CMAP_RANGE, 16523 },
	{ 0xc661, 0xc661, PDF_CMAP_SINGLE, 3041 },
	{ 0xc662, 0xc671, PDF_CMAP_RANGE, 16534 },
	{ 0xc673, 0xc67d, PDF_CMAP_RANGE, 16550 },
	{ 0xc681, 0xc682, PDF_CMAP_RANGE, 3042 },
	{ 0xc683, 0xc684, PDF_CMAP_RANGE, 16561 },
	{ 0xc685, 0xc685, PDF_CMAP_SINGLE, 3044 },
	{ 0xc686, 0xc688, PDF_CMAP_RANGE, 16563 },
	{ 0xc689, 0xc689, PDF_CMAP_SINGLE, 3045 },
	{ 0xc68a, 0xc690, PDF_CMAP_RANGE, 16566 },
	{ 0xc691, 0xc691, PDF_CMAP_SINGLE, 3046 },
	{ 0xc693, 0xc697, PDF_CMAP_TABLE, 717 },
	{ 0xc698, 0xc69d, PDF_CMAP_RANGE, 16575 },
	{ 0xc6a1, 0xc6a1, PDF_CMAP_SINGLE, 3050 },
	{ 0xc6a2, 0xc6a4, PDF_CMAP_RANGE, 16581 },
	{ 0xc6a5, 0xc6a5, PDF_CMAP_SINGLE, 3051 },
	{ 0xc6a6, 0xc6a8, PDF_CMAP_RANGE, 16584 },
	{ 0xc6a9, 0xc6a9, PDF_CMAP_SINGLE, 3052 },
	{ 0xc6aa, 0xc6b1, PDF_CMAP_RANGE, 16587 },
	{ 0xc6b3, 0xc6b6, PDF_CMAP_RANGE, 16595 },
	{ 0xc6b7, 0xc6b7, PDF_CMAP_SINGLE, 3053 },
	{ 0xc6b8, 0xc6bd, PDF_CMAP_RANGE, 16599 },
	{ 0xc6c1, 0xc6c1, PDF_CMAP_SINGLE, 3054 },
	{ 0xc6c2, 0xc6d1, PDF_CMAP_RANGE, 16605 },
	{ 0xc6d3, 0xc6d6, PDF_CMAP_RANGE, 16621 },
	{ 0xc6d7, 0xc6d7, PDF_CMAP_SINGLE, 3055 },
	{ 0xc6d8, 0xc6dd, PDF_CMAP_RANGE, 16625 },
	{ 0xc6e1, 0xc6e2, PDF_CMAP_RANGE, 3056 },
	{ 0xc6e3, 0xc6e4, PDF_CMAP_RANGE, 16631 },
	{ 0xc6e5, 0xc6e5, PDF_CMAP_SINGLE, 3058 },
	{ 0xc6e6, 0xc6e8, PDF_CMAP_RANGE, 16633 },
	{ 0xc6e9, 0xc6e9, PDF_CMAP_SINGLE, 3059 },
	{ 0xc6ea, 0xc6f0, PDF_CMAP_RANGE, 16636 },
	{ 0xc6f1, 0xc6f1, PDF_CMAP_SINGLE, 3060 },
	{ 0xc6f3, 0xc6f7, PDF_CMAP_TABLE, 722 },
	{ 0xc6f8, 0xc6fd, PDF_CMAP_RANGE, 16645 },
	{ 0xc741, 0xc741, PDF_CMAP_SINGLE, 3064 },
	{ 0xc742, 0xc744, PDF_CMAP_RANGE, 16651 },
	{ 0xc745, 0xc745, PDF_CMAP_SINGLE, 3065 },
	{ 0xc746, 0xc748, PDF_CMAP_RANGE, 16654 },
	{ 0xc749, 0xc749, PDF_CMAP_SINGLE, 3066 },
	{ 0xc74a, 0xc750, PDF_CMAP_RANGE, 16657 },
	{ 0xc751, 0xc751, PDF_CMAP_SINGLE, 3067 },
	{ 0xc753, 0xc75d, PDF_CMAP_RANGE, 16664 },
	{ 0xc761, 0xc762, PDF_CMAP_RANGE, 3068 },
	{ 0xc763, 0xc764, PDF_CMAP_RANGE, 16675 },
	{ 0xc765, 0xc765, PDF_CMAP_SINGLE, 3070 },
	{ 0xc766, 0xc768, PDF_CMAP_RANGE, 16677 },
	{ 0xc769, 0xc769, PDF_CMAP_SINGLE, 3071 },
	{ 0xc76a, 0xc770, PDF_CMAP_RANGE, 16680 },
	{ 0xc771, 0xc771, PDF_CMAP_SINGLE, 3072 },
	{ 0xc773, 0xc773, PDF_CMAP_SINGLE, 3073 },
	{ 0xc774, 0xc776, PDF_CMAP_RANGE, 16687 },
	{ 0xc777, 0xc777, PDF_CMAP_SINGLE, 3074 },
	{ 0xc778, 0xc77d, PDF_CMAP_RANGE, 16690 },
	{ 0xc781, 0xc791, PDF_CMAP_RANGE, 16696 },
	{ 0xc793, 0xc79d, PDF_CMAP_RANGE, 16713 },
	{ 0xc7a1, 0xc7a2, PDF_CMAP_RANGE, 3075 },
	{ 0xc7a3, 0xc7a4, PDF_CMAP_RANGE, 16724 },
	{ 0xc7a5, 0xc7a5, PDF_CMAP_SINGLE, 3077 },
	{ 0xc7a6, 0xc7a8, PDF_CMAP_RANGE, 16726 },
	{ 0xc7a9, 0xc7a9, PDF_CMAP_SINGLE, 3078 },
	{ 0xc7aa, 0xc7b0, PDF_CMAP_RANGE, 16729 },
	{ 0xc7b1, 0xc7b1, PDF_CMAP_SINGLE, 3079 },
	{ 0xc7b3, 0xc7b7, PDF_CMAP_TABLE, 727 },
	{ 0xc7b8, 0xc7bd, PDF_CMAP_RANGE, 16738 },
	{ 0xc841, 0xc841, PDF_CMAP_SINGLE, 385 },
	{ 0xc861, 0xc862, PDF_CMAP_RANGE, 3083 },
	{ 0xc863, 0xc864, PDF_CMAP_RANGE, 16744 },
	{ 0xc865, 0xc865, PDF_CMAP_SINGLE, 3085 },
	{ 0xc866, 0xc868, PDF_CMAP_RANGE, 16746 },
	{ 0xc869, 0xc86a, PDF_CMAP_RANGE, 3086 },
	{ 0xc86b, 0xc870, PDF_CMAP_RANGE, 16749 },
	{ 0xc871, 0xc871, PDF_CMAP_SINGLE, 3088 },
	{ 0xc873, 0xc874, PDF_CMAP_TABLE, 732 },
	{ 0xc875, 0xc877, PDF_CMAP_RANGE, 3090 },
	{ 0xc878, 0xc87d, PDF_CMAP_RANGE, 16756 },
	{ 0xc881, 0xc882, PDF_CMAP_RANGE, 3093 },
	{ 0xc883, 0xc884, PDF_CMAP_RANGE, 16762 },
	{ 0xc885, 0xc885, PDF_CMAP_SINGLE, 3095 },
	{ 0xc886, 0xc888, PDF_CMAP_RANGE, 16764 },
	{ 0xc889, 0xc889, PDF_CMAP_SINGLE, 3096 },
	{ 0xc88a, 0xc890, PDF_CMAP_RANGE, 16767 },
	{ 0xc891, 0xc891, PDF_CMAP_SINGLE, 3097 },
	{ 0xc893, 0xc894, PDF_CMAP_TABLE, 734 },
	{ 0xc895, 0xc897, PDF_CMAP_RANGE, 3099 },
	{ 0xc898, 0xc89d, PDF_CMAP_RANGE, 16775 },
	{ 0xc8a1, 0xc8a1, PDF_CMAP_SINGLE, 3102 },
	{ 0xc8a2, 0xc8b1, PDF_CMAP_RANGE, 16781 },
	{ 0xc8b3, 0xc8b6, PDF_CMAP_RANGE, 16797 },
	{ 0xc8b7, 0xc8b7, PDF_CMAP_SINGLE, 3103 },
	{ 0xc8b8, 0xc8bd, PDF_CMAP_RANGE, 16801 },
	{ 0xc8c1, 0xc8d1, PDF_CMAP_RANGE, 16807 },
	{ 0xc8d3, 0xc8dd, PDF_CMAP_RANGE, 16824 },
	{ 0xc8e1, 0xc8e2, PDF_CMAP_RANGE, 3104 },
	{ 0xc8e3, 0xc8e4, PDF_CMAP_RANGE, 16835 },
	{ 0xc8e5, 0xc8e5, PDF_CMAP_SINGLE, 3106 },
	{ 0xc8e6, 0xc8e8, PDF_CMAP_RANGE, 16837 },
	{ 0xc8e9, 0xc8eb, PDF_CMAP_TABLE, 736 },
	{ 0xc8ec, 0xc8f0, PDF_CMAP_RANGE, 16841 },
	{ 0xc8f1, 0xc8f1, PDF_CMAP_SINGLE, 3109 },
	{ 0xc8f3, 0xc8f4, PDF_CMAP_TABLE, 739 },
	{ 0xc8f5, 0xc8f7, PDF_CMAP_RANGE, 3111 },
	{ 0xc8f8, 0xc8fd, PDF_CMAP_RANGE, 16847 },
	{ 0xc941, 0xc942, PDF_CMAP_RANGE, 3114 },
	{ 0xc943, 0xc944, PDF_CMAP_RANGE, 16853 },
	{ 0xc945, 0xc945, PDF_CMAP_SINGLE, 3116 },
	{ 0xc946, 0xc948, PDF_CMAP_RANGE, 16855 },
	{ 0xc949, 0xc949, PDF_CMAP_SINGLE, 3117 },
	{ 0xc94a, 0xc950, PDF_CMAP_RANGE, 16858 },
	{ 0xc951, 0xc951, PDF_CMAP_SINGLE, 3118 },
	{ 0xc953, 0xc957, PDF_CMAP_TABLE, 741 },
	{ 0xc958, 0xc95d, PDF_CMAP_RANGE, 16867 },
	{ 0xc961, 0xc961, PDF_CMAP_SINGLE, 3122 },
	{ 0xc962, 0xc964, PDF_CMAP_RANGE, 16873 },
	{ 0xc965, 0xc965, PDF_CMAP_SINGLE, 3123 },
	{ 0xc966, 0xc971, PDF_CMAP_RANGE, 16876 },
	{ 0xc973, 0xc975, PDF_CMAP_RANGE, 16888 },
	{ 0xc976, 0xc976, PDF_CMAP_SINGLE, 3124 },
	{ 0xc977, 0xc97d, PDF_CMAP_RANGE, 16891 },
	{ 0xc981, 0xc981, PDF_CMAP_SINGLE, 3125 },
	{ 0xc982, 0xc984, PDF_CMAP_RANGE, 16898 },
	{ 0xc985, 0xc985, PDF_CMAP_SINGLE, 3126 },
	{ 0xc986, 0xc991, PDF_CMAP_RANGE, 16901 },
	{ 0xc993, 0xc99d, PDF_CMAP_RANGE, 16913 },
	{ 0xc9a1, 0xc9a2, PDF_CMAP_RANGE, 3127 },
	{ 0xc9a3, 0xc9a4, PDF_CMAP_RANGE, 16924 },
	{ 0xc9a5, 0xc9a5, PDF_CMAP_SINGLE, 3129 },
	{ 0xc9a6, 0xc9a8, PDF_CMAP_RANGE, 16926 },
	{ 0xc9a9, 0xc9a9, PDF_CMAP_SINGLE, 3130 },
	{ 0xc9aa, 0xc9b0, PDF_CMAP_RANGE, 16929 },
	{ 0xc9b1, 0xc9b1, PDF_CMAP_SINGLE, 3131 },
	{ 0xc9b3, 0xc9b7, PDF_CMAP_TABLE, 746 },
	{ 0xc9b8, 0xc9bb, PDF_CMAP_RANGE, 16938 },
	{ 0xc9bc, 0xc9bd, PDF_CMAP_TABLE, 751 },
	{ 0xc9c1, 0xc9c1, PDF_CMAP_SINGLE, 3136 },
	{ 0xc9c2, 0xc9c4, PDF_CMAP_RANGE, 16943 },
	{ 0xc9c5, 0xc9c5, PDF_CMAP_SINGLE, 3137 },
	{ 0xc9c6, 0xc9d1, PDF_CMAP_RANGE, 16946 },
	{ 0xc9d3, 0xc9dd, PDF_CMAP_RANGE, 16958 },
	{ 0xc9e1, 0xc9e1, PDF_CMAP_SINGLE, 3138 },
	{ 0xc9e2, 0xc9f1, PDF_CMAP_RANGE, 16969 },
	{ 0xc9f3, 0xc9fd, PDF_CMAP_RANGE, 16985 },
	{ 0xca41, 0xca41, PDF_CMAP_SINGLE, 3139 },
	{ 0xca42, 0xca44, PDF_CMAP_RANGE, 16996 },
	{ 0xca45, 0xca45, PDF_CMAP_SINGLE, 3140 },
	{ 0xca46, 0xca51, PDF_CMAP_RANGE, 16999 },
	{ 0xca53, 0xca54, PDF_CMAP_RANGE, 17011 },
	{ 0xca55, 0xca57, PDF_CMAP_TABLE, 753 },
	{ 0xca58, 0xca5d, PDF_CMAP_RANGE, 17014 },
	{ 0xca61, 0xca61, PDF_CMAP_SINGLE, 3143 },
	{ 0xca62, 0xca71, PDF_CMAP_RANGE, 17020 },
	{ 0xca73, 0xca7d, PDF_CMAP_RANGE, 17036 },
	{ 0xca81, 0xca82, PDF_CMAP_RANGE, 3144 },
	{ 0xca83, 0xca84, PDF_CMAP_RANGE, 17047 },
	{ 0xca85, 0xca85, PDF_CMAP_SINGLE, 3146 },
	{ 0xca86, 0xca88, PDF_CMAP_RANGE, 17049 },
	{ 0xca89, 0xca89, PDF_CMAP_SINGLE, 3147 },
	{ 0xca8a, 0xca90, PDF_CMAP_RANGE, 17052 },
	{ 0xca91, 0xca91, PDF_CMAP_SINGLE, 3148 },
	{ 0xca93, 0xca97, PDF_CMAP_TABLE, 756 },
	{ 0xca98, 0xca9d, PDF_CMAP_RANGE, 17061 },
	{ 0xcaa1, 0xcaa1, PDF_CMAP_SINGLE, 3152 },
	{ 0xcaa2, 0xcab1, PDF_CMAP_RANGE, 17067 },
	{ 0xcab3, 0xcab5, PDF_CMAP_RANGE, 17083 },
	{ 0xcab6, 0xcab6, PDF_CMAP_SINGLE, 3153 },
	{ 0xcab7, 0xcabd, PDF_CMAP_RANGE, 17086 },
	{ 0xcac1, 0xcac1, PDF_CMAP_SINGLE, 3154 },
	{ 0xcac2, 0xcad1, PDF_CMAP_RANGE, 17093 },
	{ 0xcad3, 0xcadd, PDF_CMAP_RANGE, 17109 },
	{ 0xcae1, 0xcae2, PDF_CMAP_RANGE, 3155 },
	{ 0xcae3, 0xcae4, PDF_CMAP_RANGE, 17120 },
	{ 0xcae5, 0xcae5, PDF_CMAP_SINGLE, 3157 },
	{ 0xcae6, 0xcae8, PDF_CMAP_RANGE, 17122 },
	{ 0xcae9, 0xcae9, PDF_CMAP_SINGLE, 3158 },
	{ 0xcaea, 0xcaf0, PDF_CMAP_RANGE, 17125 },
	{ 0xcaf1, 0xcaf1, PDF_CMAP_SINGLE, 3159 },
	{ 0xcaf3, 0xcaf3, PDF_CMAP_SINGLE, 3160 },
	{ 0xcaf4, 0xcaf6, PDF_CMAP_RANGE, 17132 },
	{ 0xcaf7, 0xcaf7, PDF_CMAP_SINGLE, 3161 },
	{ 0xcaf8, 0xcafd, PDF_CMAP_RANGE, 17135 },
	{ 0xcb41, 0xcb41, PDF_CMAP_SINGLE, 3162 },
	{ 0xcb42, 0xcb44, PDF_CMAP_RANGE, 17141 },
	{ 0xcb45, 0xcb45, PDF_CMAP_SINGLE, 3163 },
	{ 0xcb46, 0xcb48, PDF_CMAP_RANGE, 17144 },
	{ 0xcb49, 0xcb49, PDF_CMAP_SINGLE, 3164 },
	{ 0xcb4a, 0xcb50, PDF_CMAP_RANGE, 17147 },
	{ 0xcb51, 0xcb51, PDF_CMAP_SINGLE, 3165 },
	{ 0xcb53, 0xcb56, PDF_CMAP_RANGE, 17154 },
	{ 0xcb57, 0xcb57, PDF_CMAP_SINGLE, 3166 },
	{ 0xcb58, 0xcb5d, PDF_CMAP_RANGE, 17158 },
	{ 0xcb61, 0xcb62, PDF_CMAP_RANGE, 3167 },
	{ 0xcb63, 0xcb64, PDF_CMAP_RANGE, 17164 },
	{ 0xcb65, 0xcb65, PDF_CMAP_SINGLE, 3169 },
	{ 0xcb66, 0xcb67, PDF_CMAP_RANGE, 17166 },
	{ 0xcb68, 0xcb69, PDF_CMAP_RANGE, 3170 },
	{ 0xcb6a, 0xcb6b, PDF_CMAP_TABLE, 761 },
	{ 0xcb6c, 0xcb70, PDF_CMAP_RANGE, 17169 },
	{ 0xcb71, 0xcb71, PDF_CMAP_SINGLE, 3173 },
	{ 0xcb73, 0xcb75, PDF_CMAP_TABLE, 763 },
	{ 0xcb76, 0xcb7d, PDF_CMAP_RANGE, 17175 },
	{ 0xcb81, 0xcb81, PDF_CMAP_SINGLE, 3176 },
	{ 0xcb82, 0xcb84, PDF_CMAP_RANGE, 17183 },
	{ 0xcb85, 0xcb85, PDF_CMAP_SINGLE, 3177 },
	{ 0xcb86, 0xcb88, PDF_CMAP_RANGE, 17186 },
	{ 0xcb89, 0xcb89, PDF_CMAP_SINGLE, 3178 },
	{ 0xcb8a, 0xcb90, PDF_CMAP_RANGE, 17189 },
	{ 0xcb91, 0xcb91, PDF_CMAP_SINGLE, 3179 },
	{ 0xcb93, 0xcb93, PDF_CMAP_SINGLE, 3180 },
	{ 0xcb94, 0xcb9d, PDF_CMAP_RANGE, 17196 },
	{ 0xcba1, 0xcba2, PDF_CMAP_RANGE, 3181 },
	{ 0xcba3, 0xcba4, PDF_CMAP_RANGE, 17206 },
	{ 0xcba5, 0xcba5, PDF_CMAP_SINGLE, 3183 },
	{ 0xcba6, 0xcba8, PDF_CMAP_RANGE, 17208 },
	{ 0xcba9, 0xcba9, PDF_CMAP_SINGLE, 3184 },
	{ 0xcbaa, 0xcbb0, PDF_CMAP_RANGE, 17211 },
	{ 0xcbb1, 0xcbb1, PDF_CMAP_SINGLE, 3185 },
	{ 0xcbb3, 0xcbb7, PDF_CMAP_TABLE, 766 },
	{ 0xcbb8, 0xcbbd, PDF_CMAP_RANGE, 17220 },
	{ 0xcc41, 0xcc41, PDF_CMAP_SINGLE, 386 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 3189 },
	{ 0xcc64, 0xcc65, PDF_CMAP_TABLE, 771 },
	{ 0xcc66, 0xcc68, PDF_CMAP_RANGE, 17227 },
	{ 0xcc69, 0xcc6b, PDF_CMAP_TABLE, 773 },
	{ 0xcc6c, 0xcc70, PDF_CMAP_RANGE, 17231 },
	{ 0xcc71, 0xcc71, PDF_CMAP_SINGLE, 3195 },
	{ 0xcc73, 0xcc74, PDF_CMAP_TABLE, 776 },
	{ 0xcc75, 0xcc77, PDF_CMAP_RANGE, 3197 },
	{ 0xcc78, 0xcc7a, PDF_CMAP_RANGE, 17237 },
	{ 0xcc7b, 0xcc7b, PDF_CMAP_SINGLE, 3200 },
	{ 0xcc7c, 0xcc7d, PDF_CMAP_RANGE, 17240 },
	{ 0xcc81, 0xcc82, PDF_CMAP_RANGE, 3201 },
	{ 0xcc83, 0xcc84, PDF_CMAP_RANGE, 17242 },
	{ 0xcc85, 0xcc85, PDF_CMAP_SINGLE, 3203 },
	{ 0xcc86, 0xcc88, PDF_CMAP_RANGE, 17244 },
	{ 0xcc89, 0xcc89, PDF_CMAP_SINGLE, 3204 },
	{ 0xcc8a, 0xcc90, PDF_CMAP_RANGE, 17247 },
	{ 0xcc91, 0xcc91, PDF_CMAP_SINGLE, 3205 },
	{ 0xcc93, 0xcc94, PDF_CMAP_TABLE, 778 },
	{ 0xcc95, 0xcc97, PDF_CMAP_RANGE, 3207 },
	{ 0xcc98, 0xcc9d, PDF_CMAP_RANGE, 17255 },
	{ 0xcca1, 0xcca2, PDF_CMAP_RANGE, 3210 },
	{ 0xcca3, 0xccb1, PDF_CMAP_RANGE, 17261 },
	{ 0xccb3, 0xccbd, PDF_CMAP_RANGE, 17276 },
	{ 0xccc1, 0xccd1, PDF_CMAP_RANGE, 17287 },
	{ 0xccd3, 0xccdd, PDF_CMAP_RANGE, 17304 },
	{ 0xcce1, 0xcce2, PDF_CMAP_RANGE, 3212 },
	{ 0xcce3, 0xcce4, PDF_CMAP_RANGE, 17315 },
	{ 0xcce5, 0xcce5, PDF_CMAP_SINGLE, 3214 },
	{ 0xcce6, 0xcce8, PDF_CMAP_RANGE, 17317 },
	{ 0xcce9, 0xcce9, PDF_CMAP_SINGLE, 3215 },
	{ 0xccea, 0xccf0, PDF_CMAP_RANGE, 17320 },
	{ 0xccf1, 0xccf1, PDF_CMAP_SINGLE, 3216 },
	{ 0xccf3, 0xccf4, PDF_CMAP_TABLE, 780 },
	{ 0xccf5, 0xccf7, PDF_CMAP_RANGE, 3218 },
	{ 0xccf8, 0xccfd, PDF_CMAP_RANGE, 17328 },
	{ 0xcd41, 0xcd42, PDF_CMAP_RANGE, 3221 },
	{ 0xcd43, 0xcd44, PDF_CMAP_RANGE, 17334 },
	{ 0xcd45, 0xcd45, PDF_CMAP_SINGLE, 3223 },
	{ 0xcd46, 0xcd48, PDF_CMAP_RANGE, 17336 },
	{ 0xcd49, 0xcd49, PDF_CMAP_SINGLE, 3224 },
	{ 0xcd4a, 0xcd50, PDF_CMAP_RANGE, 17339 },
	{ 0xcd51, 0xcd51, PDF_CMAP_SINGLE, 3225 },
	{ 0xcd53, 0xcd57, PDF_CMAP_TABLE, 782 },
	{ 0xcd58, 0xcd5d, PDF_CMAP_RANGE, 17348 },
	{ 0xcd61, 0xcd61, PDF_CMAP_SINGLE, 3229 },
	{ 0xcd62, 0xcd64, PDF_CMAP_RANGE, 17354 },
	{ 0xcd65, 0xcd65, PDF_CMAP_SINGLE, 3230 },
	{ 0xcd66, 0xcd68, PDF_CMAP_RANGE, 17357 },
	{ 0xcd69, 0xcd69, PDF_CMAP_SINGLE, 3231 },
	{ 0xcd6a, 0xcd70, PDF_CMAP_RANGE, 17360 },
	{ 0xcd71, 0xcd71, PDF_CMAP_SINGLE, 3232 },
	{ 0xcd73, 0xcd73, PDF_CMAP_SINGLE, 3233 },
	{ 0xcd74, 0xcd75, PDF_CMAP_RANGE, 17367 },
	{ 0xcd76, 0xcd77, PDF_CMAP_RANGE, 3234 },
	{ 0xcd78, 0xcd7d, PDF_CMAP_RANGE, 17369 },
	{ 0xcd81, 0xcd81, PDF_CMAP_SINGLE, 3236 },
	{ 0xcd82, 0xcd88, PDF_CMAP_RANGE, 17375 },
	{ 0xcd89, 0xcd89, PDF_CMAP_SINGLE, 3237 },
	{ 0xcd8a, 0xcd91, PDF_CMAP_RANGE, 17382 },
	{ 0xcd93, 0xcd95, PDF_CMAP_TABLE, 787 },
	{ 0xcd96, 0xcd9d, PDF_CMAP_RANGE, 17391 },
	{ 0xcda1, 0xcda2, PDF_CMAP_RANGE, 3240 },
	{ 0xcda3, 0xcda4, PDF_CMAP_RANGE, 17399 },
	{ 0xcda5, 0xcda5, PDF_CMAP_SINGLE, 3242 },
	{ 0xcda6, 0xcda8, PDF_CMAP_RANGE, 17401 },
	{ 0xcda9, 0xcda9, PDF_CMAP_SINGLE, 3243 },
	{ 0xcdaa, 0xcdb0, PDF_CMAP_RANGE, 17404 },
	{ 0xcdb1, 0xcdb1, PDF_CMAP_SINGLE, 3244 },
	{ 0xcdb3, 0xcdb7, PDF_CMAP_TABLE, 790 },
	{ 0xcdb8, 0xcdbd, PDF_CMAP_RANGE, 17413 },
	{ 0xcdc1, 0xcdc1, PDF_CMAP_SINGLE, 3248 },
	{ 0xcdc2, 0xcdd1, PDF_CMAP_RANGE, 17419 },
	{ 0xcdd3, 0xcdd6, PDF_CMAP_RANGE, 17435 },
	{ 0xcdd7, 0xcdd7, PDF_CMAP_SINGLE, 3249 },
	{ 0xcdd8, 0xcddd, PDF_CMAP_RANGE, 17439 },
	{ 0xcde1, 0xcdf1, PDF_CMAP_RANGE, 17445 },
	{ 0xcdf3, 0xcdfd, PDF_CMAP_RANGE, 17462 },
	{ 0xce41, 0xce41, PDF_CMAP_SINGLE, 3250 },
	{ 0xce42, 0xce44, PDF_CMAP_RANGE, 17473 },
	{ 0xce45, 0xce45, PDF_CMAP_SINGLE, 3251 },
	{ 0xce46, 0xce51, PDF_CMAP_RANGE, 17476 },
	{ 0xce53, 0xce5d, PDF_CMAP_RANGE, 17488 },
	{ 0xce61, 0xce61, PDF_CMAP_SINGLE, 3252 },
	{ 0xce62, 0xce64, PDF_CMAP_RANGE, 17499 },
	{ 0xce65, 0xce65, PDF_CMAP_SINGLE, 3253 },
	{ 0xce66, 0xce68, PDF_CMAP_RANGE, 17502 },
	{ 0xce69, 0xce69, PDF_CMAP_SINGLE, 3254 },
	{ 0xce6a, 0xce71, PDF_CMAP_RANGE, 17505 },
	{ 0xce73, 0xce75, PDF_CMAP_TABLE, 795 },
	{ 0xce76, 0xce7d, PDF_CMAP_RANGE, 17514 },
	{ 0xce81, 0xce82, PDF_CMAP_RANGE, 3257 },
	{ 0xce83, 0xce84, PDF_CMAP_RANGE, 17522 },
	{ 0xce85, 0xce85, PDF_CMAP_SINGLE, 3259 },
	{ 0xce86, 0xce87, PDF_CMAP_RANGE, 17524 },
	{ 0xce88, 0xce89, PDF_CMAP_RANGE, 3260 },
	{ 0xce8a, 0xce8b, PDF_CMAP_TABLE, 798 },
	{ 0xce8c, 0xce90, PDF_CMAP_RANGE, 17527 },
	{ 0xce91, 0xce91, PDF_CMAP_SINGLE, 3263 },
	{ 0xce93, 0xce97, PDF_CMAP_TABLE, 800 },
	{ 0xce98, 0xce9d, PDF_CMAP_RANGE, 17534 },
	{ 0xcea1, 0xcea1, PDF_CMAP_SINGLE, 3267 },
	{ 0xcea2, 0xceb1, PDF_CMAP_RANGE, 17540 },
	{ 0xceb3, 0xceb6, PDF_CMAP_RANGE, 17556 },
	{ 0xceb7, 0xceb7, PDF_CMAP_SINGLE, 3268 },
	{ 0xceb8, 0xcebd, PDF_CMAP_RANGE, 17560 },
	{ 0xcec1, 0xced1, PDF_CMAP_RANGE, 17566 },
	{ 0xced3, 0xcedd, PDF_CMAP_RANGE, 17583 },
	{ 0xcee1, 0xcee1, PDF_CMAP_SINGLE, 3269 },
	{ 0xcee2, 0xcee4, PDF_CMAP_RANGE, 17594 },
	{ 0xcee5, 0xcee5, PDF_CMAP_SINGLE, 3270 },
	{ 0xcee6, 0xcee8, PDF_CMAP_RANGE, 17597 },
	{ 0xcee9, 0xcee9, PDF_CMAP_SINGLE, 3271 },
	{ 0xceea, 0xcef0, PDF_CMAP_RANGE, 17600 },
	{ 0xcef1, 0xcef1, PDF_CMAP_SINGLE, 3272 },
	{ 0xcef3, 0xcef4, PDF_CMAP_RANGE, 17607 },
	{ 0xcef5, 0xcef5, PDF_CMAP_SINGLE, 3273 },
	{ 0xcef6, 0xcefd, PDF_CMAP_RANGE, 17609 },
	{ 0xcf41, 0xcf41, PDF_CMAP_SINGLE, 3274 },
	{ 0xcf42, 0xcf44, PDF_CMAP_RANGE, 17617 },
	{ 0xcf45, 0xcf45, PDF_CMAP_SINGLE, 3275 },
	{ 0xcf46, 0xcf48, PDF_CMAP_RANGE, 17620 },
	{ 0xcf49, 0xcf49, PDF_CMAP_SINGLE, 3276 },
	{ 0xcf4a, 0xcf50, PDF_CMAP_RANGE, 17623 },
	{ 0xcf51, 0xcf51, PDF_CMAP_SINGLE, 3277 },
	{ 0xcf53, 0xcf54, PDF_CMAP_RANGE, 17630 },
	{ 0xcf55, 0xcf57, PDF_CMAP_TABLE, 805 },
	{ 0xcf58, 0xcf5d, PDF_CMAP_RANGE, 17633 },
	{ 0xcf61, 0xcf61, PDF_CMAP_SINGLE, 3280 },
	{ 0xcf62, 0xcf64, PDF_CMAP_RANGE, 17639 },
	{ 0xcf65, 0xcf65, PDF_CMAP_SINGLE, 3281 },
	{ 0xcf66, 0xcf68, PDF_CMAP_RANGE, 17642 },
	{ 0xcf69, 0xcf69, PDF_CMAP_SINGLE, 3282 },
	{ 0xcf6a, 0xcf70, PDF_CMAP_RANGE, 17645 },
	{ 0xcf71, 0xcf71, PDF_CMAP_SINGLE, 3283 },
	{ 0xcf73, 0xcf75, PDF_CMAP_TABLE, 808 },
	{ 0xcf76, 0xcf7d, PDF_CMAP_RANGE, 17653 },
	{ 0xcf81, 0xcf91, PDF_CMAP_RANGE, 17661 },
	{ 0xcf93, 0xcf9d, PDF_CMAP_RANGE, 17678 },
	{ 0xcfa1, 0xcfa2, PDF_CMAP_RANGE, 3286 },
	{ 0xcfa3, 0xcfa4, PDF_CMAP_RANGE, 17689 },
	{ 0xcfa5, 0xcfa5, PDF_CMAP_SINGLE, 3288 },
	{ 0xcfa6, 0xcfa8, PDF_CMAP_RANGE, 17691 },
	{ 0xcfa9, 0xcfa9, PDF_CMAP_SINGLE, 3289 },
	{ 0xcfaa, 0xcfb0, PDF_CMAP_RANGE, 17694 },
	{ 0xcfb1, 0xcfb1, PDF_CMAP_SINGLE, 3290 },
	{ 0xcfb3, 0xcfb7, PDF_CMAP_TABLE, 811 },
	{ 0xcfb8, 0xcfbd, PDF_CMAP_RANGE, 17703 },
	{ 0xd041, 0xd041, PDF_CMAP_SINGLE, 387 },
	{ 0xd061, 0xd062, PDF_CMAP_RANGE, 3294 },
	{ 0xd063, 0xd064, PDF_CMAP_RANGE, 17709 },
	{ 0xd065, 0xd065, PDF_CMAP_SINGLE, 3296 },
	{ 0xd066, 0xd068, PDF_CMAP_RANGE, 17711 },
	{ 0xd069, 0xd069, PDF_CMAP_SINGLE, 3297 },
	{ 0xd06a, 0xd06d, PDF_CMAP_RANGE, 17714 },
	{ 0xd06e, 0xd06e, PDF_CMAP_SINGLE, 3298 },
	{ 0xd06f, 0xd070, PDF_CMAP_RANGE, 17718 },
	{ 0xd071, 0xd071, PDF_CMAP_SINGLE, 3299 },
	{ 0xd073, 0xd077, PDF_CMAP_TABLE, 816 },
	{ 0xd078, 0xd07d, PDF_CMAP_RANGE, 17722 },
	{ 0xd081, 0xd082, PDF_CMAP_RANGE, 3303 },
	{ 0xd083, 0xd084, PDF_CMAP_RANGE, 17728 },
	{ 0xd085, 0xd085, PDF_CMAP_SINGLE, 3305 },
	{ 0xd086, 0xd088, PDF_CMAP_RANGE, 17730 },
	{ 0xd089, 0xd089, PDF_CMAP_SINGLE, 3306 },
	{ 0xd08a, 0xd090, PDF_CMAP_RANGE, 17733 },
	{ 0xd091, 0xd091, PDF_CMAP_SINGLE, 3307 },
	{ 0xd093, 0xd094, PDF_CMAP_TABLE, 821 },
	{ 0xd095, 0xd097, PDF_CMAP_RANGE, 3309 },
	{ 0xd098, 0xd09d, PDF_CMAP_RANGE, 17741 },
	{ 0xd0a1, 0xd0a1, PDF_CMAP_SINGLE, 3312 },
	{ 0xd0a2, 0xd0b1, PDF_CMAP_RANGE, 17747 },
	{ 0xd0b3, 0xd0b6, PDF_CMAP_RANGE, 17763 },
	{ 0xd0b7, 0xd0b7, PDF_CMAP_SINGLE, 3313 },
	{ 0xd0b8, 0xd0bd, PDF_CMAP_RANGE, 17767 },
	{ 0xd0c1, 0xd0d1, PDF_CMAP_RANGE, 17773 },
	{ 0xd0d3, 0xd0dd, PDF_CMAP_RANGE, 17790 },
	{ 0xd0e1, 0xd0e2, PDF_CMAP_RANGE, 3314 },
	{ 0xd0e3, 0xd0e4, PDF_CMAP_RANGE, 17801 },
	{ 0xd0e5, 0xd0e5, PDF_CMAP_SINGLE, 3316 },
	{ 0xd0e6, 0xd0e8, PDF_CMAP_RANGE, 17803 },
	{ 0xd0e9, 0xd0eb, PDF_CMAP_TABLE, 823 },
	{ 0xd0ec, 0xd0f0, PDF_CMAP_RANGE, 17807 },
	{ 0xd0f1, 0xd0f1, PDF_CMAP_SINGLE, 3319 },
	{ 0xd0f3, 0xd0f7, PDF_CMAP_TABLE, 826 },
	{ 0xd0f8, 0xd0fd, PDF_CMAP_RANGE, 17814 },
	{ 0xd141, 0xd142, PDF_CMAP_RANGE, 3323 },
	{ 0xd143, 0xd144, PDF_CMAP_RANGE, 17820 },
	{ 0xd145, 0xd145, PDF_CMAP_SINGLE, 3325 },
	{ 0xd146, 0xd148, PDF_CMAP_RANGE, 17822 },
	{ 0xd149, 0xd149, PDF_CMAP_SINGLE, 3326 },
	{ 0xd14a, 0xd150, PDF_CMAP_RANGE, 17825 },
	{ 0xd151, 0xd151, PDF_CMAP_SINGLE, 3327 },
	{ 0xd153, 0xd157, PDF_CMAP_TABLE, 831 },
	{ 0xd158, 0xd15d, PDF_CMAP_RANGE, 17834 },
	{ 0xd161, 0xd162, PDF_CMAP_RANGE, 3331 },
	{ 0xd163, 0xd164, PDF_CMAP_RANGE, 17840 },
	{ 0xd165, 0xd165, PDF_CMAP_SINGLE, 3333 },
	{ 0xd166, 0xd168, PDF_CMAP_RANGE, 17842 },
	{ 0xd169, 0xd169, PDF_CMAP_SINGLE, 3334 },
	{ 0xd16a, 0xd170, PDF_CMAP_RANGE, 17845 },
	{ 0xd171, 0xd171, PDF_CMAP_SINGLE, 3335 },
	{ 0xd173, 0xd174, PDF_CMAP_TABLE, 836 },
	{ 0xd175, 0xd177, PDF_CMAP_RANGE, 3337 },
	{ 0xd178, 0xd17d, PDF_CMAP_RANGE, 17853 },
	{ 0xd181, 0xd181, PDF_CMAP_SINGLE, 3340 },
	{ 0xd182, 0xd184, PDF_CMAP_RANGE, 17859 },
	{ 0xd185, 0xd185, PDF_CMAP_SINGLE, 3341 },
	{ 0xd186, 0xd188, PDF_CMAP_RANGE, 17862 },
	{ 0xd189, 0xd189, PDF_CMAP_SINGLE, 3342 },
	{ 0xd18a, 0xd191, PDF_CMAP_RANGE, 17865 },
	{ 0xd193, 0xd193, PDF_CMAP_SINGLE, 3343 },
	{ 0xd194, 0xd19d, PDF_CMAP_RANGE, 17873 },
	{ 0xd1a1, 0xd1a2, PDF_CMAP_RANGE, 3344 },
	{ 0xd1a3, 0xd1a4, PDF_CMAP_RANGE, 17883 },
	{ 0xd1a5, 0xd1a5, PDF_CMAP_SINGLE, 3346 },
	{ 0xd1a6, 0xd1a8, PDF_CMAP_RANGE, 17885 },
	{ 0xd1a9, 0xd1a9, PDF_CMAP_SINGLE, 3347 },
	{ 0xd1aa, 0xd1ad, PDF_CMAP_RANGE, 17888 },
	{ 0xd1ae, 0xd1ae, PDF_CMAP_SINGLE, 3348 },
	{ 0xd1af, 0xd1b0, PDF_CMAP_RANGE, 17892 },
	{ 0xd1b1, 0xd1b1, PDF_CMAP_SINGLE, 3349 },
	{ 0xd1b3, 0xd1b7, PDF_CMAP_TABLE, 838 },
	{ 0xd1b8, 0xd1ba, PDF_CMAP_RANGE, 17896 },
	{ 0xd1bb, 0xd1bb, PDF_CMAP_SINGLE, 3353 },
	{ 0xd1bc, 0xd1bd, PDF_CMAP_RANGE, 17899 },
	{ 0xd1c1, 0xd1c2, PDF_CMAP_RANGE, 3354 },
	{ 0xd1c3, 0xd1c4, PDF_CMAP_RANGE, 17901 },
	{ 0xd1c5, 0xd1c5, PDF_CMAP_SINGLE, 3356 },
	{ 0xd1c6, 0xd1c8, PDF_CMAP_RANGE, 17903 },
	{ 0xd1c9, 0xd1c9, PDF_CMAP_SINGLE, 3357 },
	{ 0xd1ca, 0xd1d1, PDF_CMAP_RANGE, 17906 },
	{ 0xd1d3, 0xd1d4, PDF_CMAP_RANGE, 17914 },
	{ 0xd1d5, 0xd1d7, PDF_CMAP_TABLE, 843 },
	{ 0xd1d8, 0xd1dd, PDF_CMAP_RANGE, 17917 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_RANGE, 3360 },
	{ 0xd1e3, 0xd1e4, PDF_CMAP_RANGE, 17923 },
	{ 0xd1e5, 0xd1e5, PDF_CMAP_SINGLE, 3362 },
	{ 0xd1e6, 0xd1f1, PDF_CMAP_RANGE, 17925 },
	{ 0xd1f3, 0xd1f4, PDF_CMAP_RANGE, 17937 },
	{ 0xd1f5, 0xd1f7, PDF_CMAP_TABLE, 846 },
	{ 0xd1f8, 0xd1fd, PDF_CMAP_RANGE, 17940 },
	{ 0xd241, 0xd242, PDF_CMAP_RANGE, 3365 },
	{ 0xd243, 0xd244, PDF_CMAP_RANGE, 17946 },
	{ 0xd245, 0xd245, PDF_CMAP_SINGLE, 3367 },
	{ 0xd246, 0xd248, PDF_CMAP_RANGE, 17948 },
	{ 0xd249, 0xd249, PDF_CMAP_SINGLE, 3368 },
	{ 0xd24a, 0xd251, PDF_CMAP_RANGE, 17951 },
	{ 0xd253, 0xd257, PDF_CMAP_TABLE, 849 },
	{ 0xd258, 0xd25d, PDF_CMAP_RANGE, 17961 },
	{ 0xd261, 0xd261, PDF_CMAP_SINGLE, 3372 },
	{ 0xd262, 0xd264, PDF_CMAP_RANGE, 17967 },
	{ 0xd265, 0xd265, PDF_CMAP_SINGLE, 3373 },
	{ 0xd266, 0xd268, PDF_CMAP_RANGE, 17970 },
	{ 0xd269, 0xd269, PDF_CMAP_SINGLE, 3374 },
	{ 0xd26a, 0xd271, PDF_CMAP_RANGE, 17973 },
	{ 0xd273, 0xd275, PDF_CMAP_TABLE, 854 },
	{ 0xd276, 0xd27d, PDF_CMAP_RANGE, 17982 },
	{ 0xd281, 0xd282, PDF_CMAP_RANGE, 3377 },
	{ 0xd283, 0xd284, PDF_CMAP_RANGE, 17990 },
	{ 0xd285, 0xd285, PDF_CMAP_SINGLE, 3379 },
	{ 0xd286, 0xd288, PDF_CMAP_RANGE, 17992 },
	{ 0xd289, 0xd289, PDF_CMAP_SINGLE, 3380 },
	{ 0xd28a, 0xd28d, PDF_CMAP_RANGE, 17995 },
	{ 0xd28e, 0xd28e, PDF_CMAP_SINGLE, 3381 },
	{ 0xd28f, 0xd290, PDF_CMAP_RANGE, 17999 },
	{ 0xd291, 0xd291, PDF_CMAP_SINGLE, 3382 },
	{ 0xd293, 0xd294, PDF_CMAP_RANGE, 18001 },
	{ 0xd295, 0xd297, PDF_CMAP_TABLE, 857 },
	{ 0xd298, 0xd29d, PDF_CMAP_RANGE, 18004 },
	{ 0xd2a1, 0xd2a1, PDF_CMAP_SINGLE, 3385 },
	{ 0xd2a2, 0xd2a4, PDF_CMAP_RANGE, 18010 },
	{ 0xd2a5, 0xd2a5, PDF_CMAP_SINGLE, 3386 },
	{ 0xd2a6, 0xd2a8, PDF_CMAP_RANGE, 18013 },
	{ 0xd2a9, 0xd2a9, PDF_CMAP_SINGLE, 3387 },
	{ 0xd2aa, 0xd2b0, PDF_CMAP_RANGE, 18016 },
	{ 0xd2b1, 0xd2b1, PDF_CMAP_SINGLE, 3388 },
	{ 0xd2b3, 0xd2b6, PDF_CMAP_RANGE, 18023 },
	{ 0xd2b7, 0xd2b7, PDF_CMAP_SINGLE, 3389 },
	{ 0xd2b8, 0xd2bd, PDF_CMAP_RANGE, 18027 },
	{ 0xd2c1, 0xd2c2, PDF_CMAP_RANGE, 3390 },
	{ 0xd2c3, 0xd2c4, PDF_CMAP_RANGE, 18033 },
	{ 0xd2c5, 0xd2c5, PDF_CMAP_SINGLE, 3392 },
	{ 0xd2c6, 0xd2c8, PDF_CMAP_RANGE, 18035 },
	{ 0xd2c9, 0xd2c9, PDF_CMAP_SINGLE, 3393 },
	{ 0xd2ca, 0xd2d1, PDF_CMAP_RANGE, 18038 },
	{ 0xd2d3, 0xd2d6, PDF_CMAP_RANGE, 18046 },
	{ 0xd2d7, 0xd2d7, PDF_CMAP_SINGLE, 3394 },
	{ 0xd2d8, 0xd2dd, PDF_CMAP_RANGE, 18050 },
	{ 0xd2e1, 0xd2e2, PDF_CMAP_RANGE, 3395 },
	{ 0xd2e3, 0xd2e4, PDF_CMAP_RANGE, 18056 },
	{ 0xd2e5, 0xd2e5, PDF_CMAP_SINGLE, 3397 },
	{ 0xd2e6, 0xd2e8, PDF_CMAP_RANGE, 18058 },
	{ 0xd2e9, 0xd2e9, PDF_CMAP_SINGLE, 3398 },
	{ 0xd2ea, 0xd2f0, PDF_CMAP_RANGE, 18061 },
	{ 0xd2f1, 0xd2f1, PDF_CMAP_SINGLE, 3399 },
	{ 0xd2f3, 0xd2f7, PDF_CMAP_TABLE, 860 },
	{ 0xd2f8, 0xd2fd, PDF_CMAP_RANGE, 18070 },
	{ 0xd341, 0xd342, PDF_CMAP_RANGE, 3403 },
	{ 0xd343, 0xd344, PDF_CMAP_RANGE, 18076 },
	{ 0xd345, 0xd345, PDF_CMAP_SINGLE, 3405 },
	{ 0xd346, 0xd348, PDF_CMAP_RANGE, 18078 },
	{ 0xd349, 0xd349, PDF_CMAP_SINGLE, 3406 },
	{ 0xd34a, 0xd350, PDF_CMAP_RANGE, 18081 },
	{ 0xd351, 0xd351, PDF_CMAP_SINGLE, 3407 },
	{ 0xd353, 0xd354, PDF_CMAP_RANGE, 18088 },
	{ 0xd355, 0xd357, PDF_CMAP_TABLE, 865 },
	{ 0xd358, 0xd35d, PDF_CMAP_RANGE, 18091 },
	{ 0xd361, 0xd362, PDF_CMAP_RANGE, 3410 },
	{ 0xd363, 0xd364, PDF_CMAP_RANGE, 18097 },
	{ 0xd365, 0xd366, PDF_CMAP_TABLE, 868 },
	{ 0xd367, 0xd36a, PDF_CMAP_RANGE, 3413 },
	{ 0xd36b, 0xd370, PDF_CMAP_RANGE, 18100 },
	{ 0xd371, 0xd371, PDF_CMAP_SINGLE, 3417 },
	{ 0xd373, 0xd377, PDF_CMAP_TABLE, 870 },
	{ 0xd378, 0xd37a, PDF_CMAP_RANGE, 18108 },
	{ 0xd37b, 0xd37b, PDF_CMAP_SINGLE, 3421 },
	{ 0xd37c, 0xd37d, PDF_CMAP_RANGE, 18111 },
	{ 0xd381, 0xd381, PDF_CMAP_SINGLE, 3422 },
	{ 0xd382, 0xd384, PDF_CMAP_RANGE, 18113 },
	{ 0xd385, 0xd385, PDF_CMAP_SINGLE, 3423 },
	{ 0xd386, 0xd388, PDF_CMAP_RANGE, 18116 },
	{ 0xd389, 0xd389, PDF_CMAP_SINGLE, 3424 },
	{ 0xd38a, 0xd390, PDF_CMAP_RANGE, 18119 },
	{ 0xd391, 0xd391, PDF_CMAP_SINGLE, 3425 },
	{ 0xd393, 0xd393, PDF_CMAP_SINGLE, 3426 },
	{ 0xd394, 0xd396, PDF_CMAP_RANGE, 18126 },
	{ 0xd397, 0xd397, PDF_CMAP_SINGLE, 3427 },
	{ 0xd398, 0xd39d, PDF_CMAP_RANGE, 18129 },
	{ 0xd3a1, 0xd3a2, PDF_CMAP_RANGE, 3428 },
	{ 0xd3a3, 0xd3a4, PDF_CMAP_RANGE, 18135 },
	{ 0xd3a5, 0xd3a5, PDF_CMAP_SINGLE, 3430 },
	{ 0xd3a6, 0xd3a8, PDF_CMAP_RANGE, 18137 },
	{ 0xd3a9, 0xd3a9, PDF_CMAP_SINGLE, 3431 },
	{ 0xd3aa, 0xd3b0, PDF_CMAP_RANGE, 18140 },
	{ 0xd3b1, 0xd3b1, PDF_CMAP_SINGLE, 3432 },
	{ 0xd3b3, 0xd3b7, PDF_CMAP_TABLE, 875 },
	{ 0xd3b8, 0xd3bd, PDF_CMAP_RANGE, 18149 },
	{ 0xd931, 0xd97e, PDF_CMAP_RANGE, 101 },
	{ 0xd991, 0xd9e5, PDF_CMAP_RANGE, 179 },
	{ 0xda31, 0xda7e, PDF_CMAP_RANGE, 264 },
	{ 0xda91, 0xdaa0, PDF_CMAP_RANGE, 342 },
	{ 0xdad5, 0xdafe, PDF_CMAP_RANGE, 409 },
	{ 0xdb31, 0xdb3a, PDF_CMAP_RANGE, 451 },
	{ 0xdb40, 0xdb49, PDF_CMAP_RANGE, 461 },
	{ 0xdb51, 0xdb68, PDF_CMAP_RANGE, 471 },
	{ 0xdb71, 0xdb7e, PDF_CMAP_RANGE, 495 },
	{ 0xdb91, 0xdb9a, PDF_CMAP_RANGE, 509 },
	{ 0xdba1, 0xdbe4, PDF_CMAP_RANGE, 519 },
	{ 0xdc31, 0xdc7e, PDF_CMAP_RANGE, 587 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 665 },
	{ 0xdca1, 0xdca4, PDF_CMAP_RANGE, 666 },
	{ 0xdca6, 0xdca6, PDF_CMAP_SINGLE, 670 },
	{ 0xdca8, 0xdcaf, PDF_CMAP_RANGE, 671 },
	{ 0xdcb1, 0xdcfe, PDF_CMAP_RANGE, 679 },
	{ 0xdd31, 0xdd7e, PDF_CMAP_RANGE, 757 },
	{ 0xdd91, 0xddf3, PDF_CMAP_RANGE, 835 },
	{ 0xde31, 0xde7e, PDF_CMAP_RANGE, 934 },
	{ 0xde91, 0xde98, PDF_CMAP_RANGE, 1012 },
	{ 0xdea1, 0xdec1, PDF_CMAP_RANGE, 1020 },
	{ 0xded1, 0xdef1, PDF_CMAP_RANGE, 1053 },
	{ 0xe031, 0xe07e, PDF_CMAP_RANGE, 3436 },
	{ 0xe091, 0xe0cf, PDF_CMAP_RANGE, 3514 },
	{ 0xe0d0, 0xe0d0, PDF_CMAP_SINGLE, 4116 },
	{ 0xe0d1, 0xe0d5, PDF_CMAP_RANGE, 3577 },
	{ 0xe0d6, 0xe0d6, PDF_CMAP_SINGLE, 3678 },
	{ 0xe0d7, 0xe0e6, PDF_CMAP_RANGE, 3582 },
	{ 0xe0e7, 0xe0e7, PDF_CMAP_SINGLE, 7053 },
	{ 0xe0e8, 0xe0fe, PDF_CMAP_RANGE, 3598 },
	{ 0xe131, 0xe17e, PDF_CMAP_RANGE, 3621 },
	{ 0xe191, 0xe1ce, PDF_CMAP_RANGE, 3699 },
	{ 0xe1cf, 0xe1cf, PDF_CMAP_SINGLE, 3460 },
	{ 0xe1d0, 0xe1e7, PDF_CMAP_RANGE, 3761 },
	{ 0xe1e8, 0xe1e8, PDF_CMAP_SINGLE, 7900 },
	{ 0xe1e9, 0xe1fe, PDF_CMAP_RANGE, 3785 },
	{ 0xe231, 0xe23c, PDF_CMAP_RANGE, 3807 },
	{ 0xe23d, 0xe23d, PDF_CMAP_SINGLE, 3802 },
	{ 0xe23e, 0xe27e, PDF_CMAP_RANGE, 3819 },
	{ 0xe291, 0xe2fa, PDF_CMAP_RANGE, 3884 },
	{ 0xe2fb, 0xe2fb, PDF_CMAP_SINGLE, 3902 },
	{ 0xe2fc, 0xe2fe, PDF_CMAP_RANGE, 3990 },
	{ 0xe331, 0xe332, PDF_CMAP_TABLE, 880 },
	{ 0xe333, 0xe347, PDF_CMAP_RANGE, 3994 },
	{ 0xe348, 0xe348, PDF_CMAP_SINGLE, 3946 },
	{ 0xe349, 0xe35f, PDF_CMAP_RANGE, 4015 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 3708 },
	{ 0xe361, 0xe36c, PDF_CMAP_RANGE, 4038 },
	{ 0xe36d, 0xe36d, PDF_CMAP_SINGLE, 4131 },
	{ 0xe36e, 0xe37e, PDF_CMAP_RANGE, 4050 },
	{ 0xe391, 0xe3d3, PDF_CMAP_RANGE, 4067 },
	{ 0xe3d4, 0xe3d5, PDF_CMAP_TABLE, 882 },
	{ 0xe3d6, 0xe3d7, PDF_CMAP_RANGE, 4134 },
	{ 0xe3d8, 0xe3d8, PDF_CMAP_SINGLE, 4375 },
	{ 0xe3d9, 0xe3da, PDF_CMAP_RANGE, 4136 },
	{ 0xe3db, 0xe3e0, PDF_CMAP_RANGE, 4376 },
	{ 0xe3e1, 0xe3e2, PDF_CMAP_TABLE, 884 },
	{ 0xe3e3, 0xe3e5, PDF_CMAP_RANGE, 4382 },
	{ 0xe3e6, 0xe3e7, PDF_CMAP_TABLE, 886 },
	{ 0xe3e8, 0xe3eb, PDF_CMAP_RANGE, 4387 },
	{ 0xe3ec, 0xe3ee, PDF_CMAP_TABLE, 888 },
	{ 0xe3ef, 0xe3f0, PDF_CMAP_RANGE, 4394 },
	{ 0xe3f1, 0xe3f2, PDF_CMAP_TABLE, 891 },
	{ 0xe3f3, 0xe3f5, PDF_CMAP_RANGE, 4143 },
	{ 0xe3f6, 0xe3f6, PDF_CMAP_SINGLE, 4399 },
	{ 0xe3f7, 0xe3f9, PDF_CMAP_RANGE, 4146 },
	{ 0xe3fa, 0xe3fb, PDF_CMAP_TABLE, 893 },
	{ 0xe3fc, 0xe3fd, PDF_CMAP_RANGE, 4406 },
	{ 0xe3fe, 0xe3fe, PDF_CMAP_SINGLE, 4409 },
	{ 0xe431, 0xe431, PDF_CMAP_SINGLE, 4150 },
	{ 0xe432, 0xe433, PDF_CMAP_RANGE, 4410 },
	{ 0xe434, 0xe436, PDF_CMAP_RANGE, 4151 },
	{ 0xe437, 0xe43a, PDF_CMAP_RANGE, 4412 },
	{ 0xe43b, 0xe43d, PDF_CMAP_TABLE, 895 },
	{ 0xe43e, 0xe441, PDF_CMAP_RANGE, 4155 },
	{ 0xe442, 0xe442, PDF_CMAP_SINGLE, 4424 },
	{ 0xe443, 0xe44d, PDF_CMAP_RANGE, 4159 },
	{ 0xe44e, 0xe44e, PDF_CMAP_SINGLE, 4511 },
	{ 0xe44f, 0xe451, PDF_CMAP_RANGE, 4170 },
	{ 0xe452, 0xe453, PDF_CMAP_RANGE, 4513 },
	{ 0xe454, 0xe455, PDF_CMAP_TABLE, 898 },
	{ 0xe456, 0xe45a, PDF_CMAP_RANGE, 4518 },
	{ 0xe45b, 0xe45c, PDF_CMAP_TABLE, 900 },
	{ 0xe45d, 0xe45e, PDF_CMAP_RANGE, 4525 },
	{ 0xe45f, 0xe464, PDF_CMAP_RANGE, 4528 },
	{ 0xe465, 0xe467, PDF_CMAP_RANGE, 4535 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 4175 },
	{ 0xe469, 0xe46a, PDF_CMAP_RANGE, 4541 },
	{ 0xe46b, 0xe46d, PDF_CMAP_RANGE, 4176 },
	{ 0xe46e, 0xe46f, PDF_CMAP_RANGE, 4545 },
	{ 0xe470, 0xe474, PDF_CMAP_TABLE, 902 },
	{ 0xe475, 0xe478, PDF_CMAP_RANGE, 4566 },
	{ 0xe479, 0xe47a, PDF_CMAP_RANGE, 4571 },
	{ 0xe47b, 0xe47b, PDF_CMAP_SINGLE, 4576 },
	{ 0xe47c, 0xe47e, PDF_CMAP_RANGE, 4181 },
	{ 0xe491, 0xe491, PDF_CMAP_SINGLE, 4184 },
	{ 0xe492, 0xe495, PDF_CMAP_RANGE, 4604 },
	{ 0xe496, 0xe497, PDF_CMAP_RANGE, 4609 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 4185 },
	{ 0xe499, 0xe49a, PDF_CMAP_RANGE, 4611 },
	{ 0xe49b, 0xe4fe, PDF_CMAP_RANGE, 4186 },
	{ 0xe531, 0xe575, PDF_CMAP_RANGE, 4286 },
	{ 0xe576, 0xe576, PDF_CMAP_SINGLE, 4318 },
	{ 0xe577, 0xe57e, PDF_CMAP_RANGE, 4355 },
	{ 0xe591, 0xe59d, PDF_CMAP_RANGE, 4363 },
	{ 0xe59e, 0xe59e, PDF_CMAP_SINGLE, 4136 },
	{ 0xe59f, 0xe5a4, PDF_CMAP_RANGE, 4376 },
	{ 0xe5a5, 0xe5a5, PDF_CMAP_SINGLE, 5800 },
	{ 0xe5a6, 0xe5aa, PDF_CMAP_RANGE, 4382 },
	{ 0xe5ab, 0xe5ab, PDF_CMAP_SINGLE, 4139 },
	{ 0xe5ac, 0xe5ad, PDF_CMAP_RANGE, 4387 },
	{ 0xe5ae, 0xe5ae, PDF_CMAP_SINGLE, 4192 },
	{ 0xe5af, 0xe5fe, PDF_CMAP_RANGE, 4389 },
	{ 0xe631, 0xe647, PDF_CMAP_RANGE, 4469 },
	{ 0xe648, 0xe648, PDF_CMAP_SINGLE, 4167 },
	{ 0xe649, 0xe65c, PDF_CMAP_RANGE, 4492 },
	{ 0xe65d, 0xe65d, PDF_CMAP_SINGLE, 4172 },
	{ 0xe65e, 0xe67e, PDF_CMAP_RANGE, 4512 },
	{ 0xe691, 0xe6ca, PDF_CMAP_RANGE, 4545 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 5552 },
	{ 0xe6cc, 0xe6e3, PDF_CMAP_RANGE, 4603 },
	{ 0xe6e4, 0xe6e4, PDF_CMAP_SINGLE, 6424 },
	{ 0xe6e5, 0xe6fe, PDF_CMAP_RANGE, 4627 },
	{ 0xe731, 0xe77e, PDF_CMAP_RANGE, 4653 },
	{ 0xe791, 0xe7fe, PDF_CMAP_RANGE, 4731 },
	{ 0xe831, 0xe87e, PDF_CMAP_RANGE, 4841 },
	{ 0xe891, 0xe8c4, PDF_CMAP_RANGE, 4919 },
	{ 0xe8c5, 0xe8c5, PDF_CMAP_SINGLE, 5151 },
	{ 0xe8c6, 0xe8e3, PDF_CMAP_RANGE, 4971 },
	{ 0xe8e4, 0xe8e4, PDF_CMAP_SINGLE, 4922 },
	{ 0xe8e5, 0xe8fe, PDF_CMAP_RANGE, 5001 },
	{ 0xe931, 0xe934, PDF_CMAP_RANGE, 5027 },
	{ 0xe935, 0xe935, PDF_CMAP_SINGLE, 7518 },
	{ 0xe936, 0xe97e, PDF_CMAP_RANGE, 5031 },
	{ 0xe991, 0xe9a4, PDF_CMAP_RANGE, 5104 },
	{ 0xe9a5, 0xe9a5, PDF_CMAP_SINGLE, 5079 },
	{ 0xe9a6, 0xe9d4, PDF_CMAP_RANGE, 5124 },
	{ 0xe9d5, 0xe9d5, PDF_CMAP_SINGLE, 5109 },
	{ 0xe9d6, 0xe9f3, PDF_CMAP_RANGE, 5171 },
	{ 0xe9f4, 0xe9f4, PDF_CMAP_SINGLE, 7607 },
	{ 0xe9f5, 0xe9fe, PDF_CMAP_RANGE, 5201 },
	{ 0xea31, 0xea7e, PDF_CMAP_RANGE, 5211 },
	{ 0xea91, 0xea9d, PDF_CMAP_RANGE, 5289 },
	{ 0xea9e, 0xeaa0, PDF_CMAP_TABLE, 907 },
	{ 0xeaa1, 0xeab2, PDF_CMAP_RANGE, 5303 },
	{ 0xeab3, 0xeab3, PDF_CMAP_SINGLE, 7083 },
	{ 0xeab4, 0xeae0, PDF_CMAP_RANGE, 5321 },
	{ 0xeae1, 0xeae1, PDF_CMAP_SINGLE, 5362 },
	{ 0xeae2, 0xeae7, PDF_CMAP_RANGE, 5366 },
	{ 0xeae8, 0xeae8, PDF_CMAP_SINGLE, 5485 },
	{ 0xeae9, 0xeafe, PDF_CMAP_RANGE, 5372 },
	{ 0xeb31, 0xeb7e, PDF_CMAP_RANGE, 5394 },
	{ 0xeb91, 0xeb92, PDF_CMAP_RANGE, 5472 },
	{ 0xeb93, 0xeb93, PDF_CMAP_SINGLE, 6001 },
	{ 0xeb94, 0xebac, PDF_CMAP_RANGE, 5474 },
	{ 0xebad, 0xebad, PDF_CMAP_SINGLE, 5460 },
	{ 0xebae, 0xebec, PDF_CMAP_RANGE, 5499 },
	{ 0xebed, 0xebed, PDF_CMAP_SINGLE, 5317 },
	{ 0xebee, 0xebfe, PDF_CMAP_RANGE, 5562 },
	{ 0xec31, 0xec7e, PDF_CMAP_RANGE, 5579 },
	{ 0xec91, 0xecf4, PDF_CMAP_RANGE, 5657 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 7009 },
	{ 0xecf6, 0xecfe, PDF_CMAP_RANGE, 5757 },
	{ 0xed31, 0xed31, PDF_CMAP_SINGLE, 7370 },
	{ 0xed32, 0xed38, PDF_CMAP_RANGE, 5766 },
	{ 0xed39, 0xed39, PDF_CMAP_SINGLE, 5678 },
	{ 0xed3a, 0xed7e, PDF_CMAP_RANGE, 5773 },
	{ 0xed91, 0xedad, PDF_CMAP_RANGE, 5842 },
	{ 0xedae, 0xedae, PDF_CMAP_SINGLE, 5874 },
	{ 0xedaf, 0xedb0, PDF_CMAP_RANGE, 5871 },
	{ 0xedb1, 0xedb2, PDF_CMAP_RANGE, 4425 },
	{ 0xedb3, 0xedb8, PDF_CMAP_RANGE, 5873 },
	{ 0xedb9, 0xedba, PDF_CMAP_TABLE, 910 },
	{ 0xedbb, 0xedbc, PDF_CMAP_RANGE, 4429 },
	{ 0xedbd, 0xedc3, PDF_CMAP_RANGE, 5880 },
	{ 0xedc4, 0xedc4, PDF_CMAP_SINGLE, 4431 },
	{ 0xedc5, 0xedcd, PDF_CMAP_RANGE, 5887 },
	{ 0xedce, 0xedd2, PDF_CMAP_TABLE, 912 },
	{ 0xedd3, 0xedd5, PDF_CMAP_RANGE, 5898 },
	{ 0xedd6, 0xedd6, PDF_CMAP_SINGLE, 4439 },
	{ 0xedd7, 0xedf9, PDF_CMAP_RANGE, 5901 },
	{ 0xedfa, 0xedfb, PDF_CMAP_RANGE, 4442 },
	{ 0xedfc, 0xedfe, PDF_CMAP_TABLE, 917 },
	{ 0xee31, 0xee31, PDF_CMAP_SINGLE, 4447 },
	{ 0xee32, 0xee33, PDF_CMAP_RANGE, 5937 },
	{ 0xee34, 0xee34, PDF_CMAP_SINGLE, 4449 },
	{ 0xee35, 0xee36, PDF_CMAP_RANGE, 5939 },
	{ 0xee37, 0xee37, PDF_CMAP_SINGLE, 4450 },
	{ 0xee38, 0xee3c, PDF_CMAP_RANGE, 5941 },
	{ 0xee3d, 0xee3e, PDF_CMAP_TABLE, 920 },
	{ 0xee3f, 0xee41, PDF_CMAP_RANGE, 4455 },
	{ 0xee42, 0xee43, PDF_CMAP_TABLE, 922 },
	{ 0xee44, 0xee46, PDF_CMAP_RANGE, 5948 },
	{ 0xee47, 0xee48, PDF_CMAP_RANGE, 4459 },
	{ 0xee49, 0xee4b, PDF_CMAP_RANGE, 5951 },
	{ 0xee4c, 0xee4c, PDF_CMAP_SINGLE, 4463 },
	{ 0xee4d, 0xee53, PDF_CMAP_RANGE, 5954 },
	{ 0xee54, 0xee55, PDF_CMAP_TABLE, 924 },
	{ 0xee56, 0xee57, PDF_CMAP_RANGE, 4465 },
	{ 0xee58, 0xee59, PDF_CMAP_RANGE, 5962 },
	{ 0xee5a, 0xee5a, PDF_CMAP_SINGLE, 4161 },
	{ 0xee5b, 0xee61, PDF_CMAP_RANGE, 5964 },
	{ 0xee62, 0xee62, PDF_CMAP_SINGLE, 4468 },
	{ 0xee63, 0xee65, PDF_CMAP_RANGE, 5971 },
	{ 0xee66, 0xee66, PDF_CMAP_SINGLE, 4469 },
	{ 0xee67, 0xee68, PDF_CMAP_RANGE, 5974 },
	{ 0xee69, 0xee69, PDF_CMAP_SINGLE, 4470 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 5976 },
	{ 0xee6c, 0xee6c, PDF_CMAP_SINGLE, 4162 },
	{ 0xee6d, 0xee6e, PDF_CMAP_RANGE, 5978 },
	{ 0xee6f, 0xee71, PDF_CMAP_TABLE, 926 },
	{ 0xee72, 0xee73, PDF_CMAP_RANGE, 5981 },
	{ 0xee74, 0xee79, PDF_CMAP_TABLE, 929 },
	{ 0xee7a, 0xee7b, PDF_CMAP_RANGE, 4478 },
	{ 0xee7c, 0xee7c, PDF_CMAP_SINGLE, 6447 },
	{ 0xee7d, 0xee7e, PDF_CMAP_RANGE, 5985 },
	{ 0xee91, 0xee94, PDF_CMAP_TABLE, 935 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 5988 },
	{ 0xee97, 0xee9b, PDF_CMAP_TABLE, 939 },
	{ 0xee9c, 0xeea0, PDF_CMAP_RANGE, 5991 },
	{ 0xeea1, 0xeea1, PDF_CMAP_SINGLE, 4487 },
	{ 0xeea2, 0xeea5, PDF_CMAP_RANGE, 5996 },
	{ 0xeea6, 0xeea6, PDF_CMAP_SINGLE, 4488 },
	{ 0xeea7, 0xeea8, PDF_CMAP_RANGE, 6000 },
	{ 0xeea9, 0xeead, PDF_CMAP_TABLE, 944 },
	{ 0xeeae, 0xeeaf, PDF_CMAP_RANGE, 6003 },
	{ 0xeeb0, 0xeeb0, PDF_CMAP_SINGLE, 4494 },
	{ 0xeeb1, 0xeebe, PDF_CMAP_RANGE, 6005 },
	{ 0xeebf, 0xeec1, PDF_CMAP_TABLE, 949 },
	{ 0xeec2, 0xeec5, PDF_CMAP_RANGE, 6020 },
	{ 0xeec6, 0xeec7, PDF_CMAP_TABLE, 952 },
	{ 0xeec8, 0xeeca, PDF_CMAP_RANGE, 6024 },
	{ 0xeecb, 0xeece, PDF_CMAP_TABLE, 954 },
	{ 0xeecf, 0xeed0, PDF_CMAP_RANGE, 4503 },
	{ 0xeed1, 0xeed2, PDF_CMAP_RANGE, 6029 },
	{ 0xeed3, 0xeed3, PDF_CMAP_SINGLE, 4506 },
	{ 0xeed4, 0xeede, PDF_CMAP_RANGE, 6031 },
	{ 0xeedf, 0xeedf, PDF_CMAP_SINGLE, 4508 },
	{ 0xeee0, 0xeee3, PDF_CMAP_RANGE, 6042 },
	{ 0xeee4, 0xeee6, PDF_CMAP_TABLE, 958 },
	{ 0xeee7, 0xeef6, PDF_CMAP_RANGE, 6047 },
	{ 0xeef7, 0xeef7, PDF_CMAP_SINGLE, 5797 },
	{ 0xeef8, 0xeefe, PDF_CMAP_RANGE, 6063 },
	{ 0xef31, 0xef76, PDF_CMAP_RANGE, 6070 },
	{ 0xef77, 0xef78, PDF_CMAP_RANGE, 4551 },
	{ 0xef79, 0xef7e, PDF_CMAP_RANGE, 6140 },
	{ 0xef91, 0xef93, PDF_CMAP_TABLE, 961 },
	{ 0xef94, 0xef98, PDF_CMAP_RANGE, 6147 },
	{ 0xef99, 0xef9d, PDF_CMAP_TABLE, 964 },
	{ 0xef9e, 0xef9f, PDF_CMAP_RANGE, 6154 },
	{ 0xefa0, 0xefa0, PDF_CMAP_SINGLE, 4557 },
	{ 0xefa1, 0xefa6, PDF_CMAP_RANGE, 6156 },
	{ 0xefa7, 0xefa7, PDF_CMAP_SINGLE, 4560 },
	{ 0xefa8, 0xefab, PDF_CMAP_RANGE, 6162 },
	{ 0xefac, 0xefac, PDF_CMAP_SINGLE, 4561 },
	{ 0xefad, 0xefcb, PDF_CMAP_RANGE, 6166 },
	{ 0xefcc, 0xefcc, PDF_CMAP_SINGLE, 4563 },
	{ 0xefcd, 0xeff6, PDF_CMAP_RANGE, 6197 },
	{ 0xeff7, 0xeff7, PDF_CMAP_SINGLE, 7988 },
	{ 0xeff8, 0xeffe, PDF_CMAP_RANGE, 6239 },
	{ 0xf031, 0xf050, PDF_CMAP_RANGE, 6246 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 6123 },
	{ 0xf052, 0xf074, PDF_CMAP_RANGE, 6278 },
	{ 0xf075, 0xf075, PDF_CMAP_SINGLE, 4577 },
	{ 0xf076, 0xf07e, PDF_CMAP_RANGE, 6313 },
	{ 0xf091, 0xf095, PDF_CMAP_RANGE, 6322 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 4183 },
	{ 0xf097, 0xf098, PDF_CMAP_RANGE, 6327 },
	{ 0xf099, 0xf099, PDF_CMAP_SINGLE, 4579 },
	{ 0xf09a, 0xf09d, PDF_CMAP_RANGE, 6329 },
	{ 0xf09e, 0xf0a0, PDF_CMAP_TABLE, 969 },
	{ 0xf0a1, 0xf0a3, PDF_CMAP_RANGE, 6334 },
	{ 0xf0a4, 0xf0a4, PDF_CMAP_SINGLE, 4584 },
	{ 0xf0a5, 0xf0a6, PDF_CMAP_RANGE, 6337 },
	{ 0xf0a7, 0xf0aa, PDF_CMAP_TABLE, 972 },
	{ 0xf0ab, 0xf0b9, PDF_CMAP_RANGE, 6340 },
	{ 0xf0ba, 0xf0bb, PDF_CMAP_RANGE, 4590 },
	{ 0xf0bc, 0xf0bd, PDF_CMAP_TABLE, 976 },
	{ 0xf0be, 0xf0c0, PDF_CMAP_RANGE, 6356 },
	{ 0xf0c1, 0xf0c2, PDF_CMAP_TABLE, 978 },
	{ 0xf0c3, 0xf0c5, PDF_CMAP_RANGE, 6359 },
	{ 0xf0c6, 0xf0c7, PDF_CMAP_RANGE, 4596 },
	{ 0xf0c8, 0xf0cb, PDF_CMAP_RANGE, 6362 },
	{ 0xf0cc, 0xf0cc, PDF_CMAP_SINGLE, 4599 },
	{ 0xf0cd, 0xf0ce, PDF_CMAP_RANGE, 6366 },
	{ 0xf0cf, 0xf0d1, PDF_CMAP_RANGE, 4600 },
	{ 0xf0d2, 0xf0d2, PDF_CMAP_SINGLE, 5552 },
	{ 0xf0d3, 0xf0d7, PDF_CMAP_RANGE, 6368 },
	{ 0xf0d8, 0xf0d8, PDF_CMAP_SINGLE, 4603 },
	{ 0xf0d9, 0xf0fe, PDF_CMAP_RANGE, 6373 },
	{ 0xf131, 0xf135, PDF_CMAP_RANGE, 6411 },
	{ 0xf136, 0xf137, PDF_CMAP_TABLE, 980 },
	{ 0xf138, 0xf139, PDF_CMAP_RANGE, 6416 },
	{ 0xf13a, 0xf13a, PDF_CMAP_SINGLE, 4618 },
	{ 0xf13b, 0xf13e, PDF_CMAP_RANGE, 6418 },
	{ 0xf13f, 0xf13f, PDF_CMAP_SINGLE, 5950 },
	{ 0xf140, 0xf141, PDF_CMAP_RANGE, 4620 },
	{ 0xf142, 0xf142, PDF_CMAP_SINGLE, 4187 },
	{ 0xf143, 0xf144, PDF_CMAP_RANGE, 6422 },
	{ 0xf145, 0xf145, PDF_CMAP_SINGLE, 4625 },
	{ 0xf146, 0xf147, PDF_CMAP_RANGE, 6424 },
	{ 0xf148, 0xf14a, PDF_CMAP_TABLE, 982 },
	{ 0xf14b, 0xf14f, PDF_CMAP_RANGE, 6427 },
	{ 0xf150, 0xf151, PDF_CMAP_RANGE, 4632 },
	{ 0xf152, 0xf154, PDF_CMAP_RANGE, 6432 },
	{ 0xf155, 0xf156, PDF_CMAP_TABLE, 985 },
	{ 0xf157, 0xf158, PDF_CMAP_RANGE, 6435 },
	{ 0xf159, 0xf15a, PDF_CMAP_RANGE, 4188 },
	{ 0xf15b, 0xf164, PDF_CMAP_RANGE, 6437 },
	{ 0xf165, 0xf165, PDF_CMAP_SINGLE, 4638 },
	{ 0xf166, 0xf16c, PDF_CMAP_RANGE, 6447 },
	{ 0xf16d, 0xf16e, PDF_CMAP_RANGE, 4640 },
	{ 0xf16f, 0xf170, PDF_CMAP_RANGE, 6454 },
	{ 0xf171, 0xf171, PDF_CMAP_SINGLE, 4642 },
	{ 0xf172, 0xf173, PDF_CMAP_RANGE, 6456 },
	{ 0xf174, 0xf174, PDF_CMAP_SINGLE, 4644 },
	{ 0xf175, 0xf176, PDF_CMAP_RANGE, 6458 },
	{ 0xf177, 0xf178, PDF_CMAP_RANGE, 4645 },
	{ 0xf179, 0xf17e, PDF_CMAP_RANGE, 6460 },
	{ 0xf191, 0xf198, PDF_CMAP_RANGE, 6466 },
	{ 0xf199, 0xf19a, PDF_CMAP_RANGE, 4647 },
	{ 0xf19b, 0xf19c, PDF_CMAP_TABLE, 987 },
	{ 0xf19d, 0xf1a0, PDF_CMAP_RANGE, 6475 },
	{ 0xf1a1, 0xf1a3, PDF_CMAP_RANGE, 4653 },
	{ 0xf1a4, 0xf1ed, PDF_CMAP_RANGE, 6479 },
	{ 0xf1ee, 0xf1ee, PDF_CMAP_SINGLE, 5351 },
	{ 0xf1ef, 0xf1fe, PDF_CMAP_RANGE, 6553 },
	{ 0xf231, 0xf26a, PDF_CMAP_RANGE, 6569 },
	{ 0xf26b, 0xf26b, PDF_CMAP_SINGLE, 6494 },
	{ 0xf26c, 0xf27e, PDF_CMAP_RANGE, 6627 },
	{ 0xf291, 0xf2fe, PDF_CMAP_RANGE, 6646 },
	{ 0xf331, 0xf37e, PDF_CMAP_RANGE, 6756 },
	{ 0xf391, 0xf3fe, PDF_CMAP_RANGE, 6834 },
	{ 0xf431, 0xf44c, PDF_CMAP_RANGE, 6944 },
	{ 0xf44d, 0xf44d, PDF_CMAP_SINGLE, 5731 },
	{ 0xf44e, 0xf47e, PDF_CMAP_RANGE, 6972 },
	{ 0xf491, 0xf49b, PDF_CMAP_RANGE, 7021 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 5771 },
	{ 0xf49d, 0xf4b0, PDF_CMAP_RANGE, 7032 },
	{ 0xf4b1, 0xf4b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf4b2, 0xf4fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf531, 0xf536, PDF_CMAP_RANGE, 7129 },
	{ 0xf537, 0xf537, PDF_CMAP_SINGLE, 6484 },
	{ 0xf538, 0xf57d, PDF_CMAP_RANGE, 7135 },
	{ 0xf57e, 0xf57e, PDF_CMAP_SINGLE, 6684 },
	{ 0xf591, 0xf5fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf631, 0xf67e, PDF_CMAP_RANGE, 7315 },
	{ 0xf691, 0xf695, PDF_CMAP_RANGE, 7393 },
	{ 0xf696, 0xf698, PDF_CMAP_TABLE, 989 },
	{ 0xf699, 0xf6b7, PDF_CMAP_RANGE, 7399 },
	{ 0xf6b8, 0xf6b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf6b9, 0xf6c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf6c8, 0xf6c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf6c9, 0xf6d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf6d3, 0xf6d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf6d4, 0xf6fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf731, 0xf76a, PDF_CMAP_RANGE, 7498 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 7572 },
	{ 0xf76c, 0xf77e, PDF_CMAP_RANGE, 7556 },
	{ 0xf791, 0xf792, PDF_CMAP_TABLE, 992 },
	{ 0xf793, 0xf7fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf831, 0xf832, PDF_CMAP_TABLE, 994 },
	{ 0xf833, 0xf875, PDF_CMAP_RANGE, 7684 },
	{ 0xf876, 0xf876, PDF_CMAP_SINGLE, 3644 },
	{ 0xf877, 0xf87e, PDF_CMAP_RANGE, 7751 },
	{ 0xf891, 0xf8fe, PDF_CMAP_RANGE, 7759 },
	{ 0xf931, 0xf938, PDF_CMAP_RANGE, 7869 },
	{ 0xf939, 0xf939, PDF_CMAP_SINGLE, 3815 },
	{ 0xf93a, 0xf97e, PDF_CMAP_RANGE, 7877 },
	{ 0xf991, 0xf9fe, PDF_CMAP_RANGE, 7946 },
	{ 0xcf76, 0xcf7d, PDF_CMAP_RANGE, 17653 },
	{ 0xcf81, 0xcf91, PDF_CMAP_RANGE, 17661 },
	{ 0xcf93, 0xcf9d, PDF_CMAP_RANGE, 17678 },
	{ 0xcfa1, 0xcfa2, PDF_CMAP_RANGE, 3286 },
	{ 0xcfa3, 0xcfa4, PDF_CMAP_RANGE, 17689 },
	{ 0xcfa5, 0xcfa5, PDF_CMAP_SINGLE, 3288 },
	{ 0xcfa6, 0xcfa8, PDF_CMAP_RANGE, 17691 },
	{ 0xcfa9, 0xcfa9, PDF_CMAP_SINGLE, 3289 },
	{ 0xcfaa, 0xcfb0, PDF_CMAP_RANGE, 17694 },
	{ 0xcfb1, 0xcfb1, PDF_CMAP_SINGLE, 3290 },
	{ 0xcfb3, 0xcfb3, PDF_CMAP_SINGLE, 3291 },
	{ 0xcfb4, 0xcfb4, PDF_CMAP_SINGLE, 17701 },
	{ 0xcfb5, 0xcfb5, PDF_CMAP_SINGLE, 3292 },
	{ 0xcfb6, 0xcfb6, PDF_CMAP_SINGLE, 17702 },
	{ 0xcfb7, 0xcfb7, PDF_CMAP_SINGLE, 3293 },
	{ 0xcfb8, 0xcfbd, PDF_CMAP_RANGE, 17703 },
	{ 0xd041, 0xd041, PDF_CMAP_SINGLE, 387 },
	{ 0xd061, 0xd062, PDF_CMAP_RANGE, 3294 },
	{ 0xd063, 0xd064, PDF_CMAP_RANGE, 17709 },
	{ 0xd065, 0xd065, PDF_CMAP_SINGLE, 3296 },
	{ 0xd066, 0xd068, PDF_CMAP_RANGE, 17711 },
	{ 0xd069, 0xd069, PDF_CMAP_SINGLE, 3297 },
	{ 0xd06a, 0xd06d, PDF_CMAP_RANGE, 17714 },
	{ 0xd06e, 0xd06e, PDF_CMAP_SINGLE, 3298 },
	{ 0xd06f, 0xd070, PDF_CMAP_RANGE, 17718 },
	{ 0xd071, 0xd071, PDF_CMAP_SINGLE, 3299 },
	{ 0xd073, 0xd073, PDF_CMAP_SINGLE, 3300 },
	{ 0xd074, 0xd074, PDF_CMAP_SINGLE, 17720 },
	{ 0xd075, 0xd075, PDF_CMAP_SINGLE, 3301 },
	{ 0xd076, 0xd076, PDF_CMAP_SINGLE, 17721 },
	{ 0xd077, 0xd077, PDF_CMAP_SINGLE, 3302 },
	{ 0xd078, 0xd07d, PDF_CMAP_RANGE, 17722 },
	{ 0xd081, 0xd082, PDF_CMAP_RANGE, 3303 },
	{ 0xd083, 0xd084, PDF_CMAP_RANGE, 17728 },
	{ 0xd085, 0xd085, PDF_CMAP_SINGLE, 3305 },
	{ 0xd086, 0xd088, PDF_CMAP_RANGE, 17730 },
	{ 0xd089, 0xd089, PDF_CMAP_SINGLE, 3306 },
	{ 0xd08a, 0xd090, PDF_CMAP_RANGE, 17733 },
	{ 0xd091, 0xd091, PDF_CMAP_SINGLE, 3307 },
	{ 0xd093, 0xd093, PDF_CMAP_SINGLE, 3308 },
	{ 0xd094, 0xd094, PDF_CMAP_SINGLE, 17740 },
	{ 0xd095, 0xd097, PDF_CMAP_RANGE, 3309 },
	{ 0xd098, 0xd09d, PDF_CMAP_RANGE, 17741 },
	{ 0xd0a1, 0xd0a1, PDF_CMAP_SINGLE, 3312 },
	{ 0xd0a2, 0xd0b1, PDF_CMAP_RANGE, 17747 },
	{ 0xd0b3, 0xd0b6, PDF_CMAP_RANGE, 17763 },
	{ 0xd0b7, 0xd0b7, PDF_CMAP_SINGLE, 3313 },
	{ 0xd0b8, 0xd0bd, PDF_CMAP_RANGE, 17767 },
	{ 0xd0c1, 0xd0d1, PDF_CMAP_RANGE, 17773 },
	{ 0xd0d3, 0xd0dd, PDF_CMAP_RANGE, 17790 },
	{ 0xd0e1, 0xd0e2, PDF_CMAP_RANGE, 3314 },
	{ 0xd0e3, 0xd0e4, PDF_CMAP_RANGE, 17801 },
	{ 0xd0e5, 0xd0e5, PDF_CMAP_SINGLE, 3316 },
	{ 0xd0e6, 0xd0e8, PDF_CMAP_RANGE, 17803 },
	{ 0xd0e9, 0xd0e9, PDF_CMAP_SINGLE, 3317 },
	{ 0xd0ea, 0xd0ea, PDF_CMAP_SINGLE, 17806 },
	{ 0xd0eb, 0xd0eb, PDF_CMAP_SINGLE, 3318 },
	{ 0xd0ec, 0xd0f0, PDF_CMAP_RANGE, 17807 },
	{ 0xd0f1, 0xd0f1, PDF_CMAP_SINGLE, 3319 },
	{ 0xd0f3, 0xd0f3, PDF_CMAP_SINGLE, 3320 },
	{ 0xd0f4, 0xd0f4, PDF_CMAP_SINGLE, 17812 },
	{ 0xd0f5, 0xd0f5, PDF_CMAP_SINGLE, 3321 },
	{ 0xd0f6, 0xd0f6, PDF_CMAP_SINGLE, 17813 },
	{ 0xd0f7, 0xd0f7, PDF_CMAP_SINGLE, 3322 },
	{ 0xd0f8, 0xd0fd, PDF_CMAP_RANGE, 17814 },
	{ 0xd141, 0xd142, PDF_CMAP_RANGE, 3323 },
	{ 0xd143, 0xd144, PDF_CMAP_RANGE, 17820 },
	{ 0xd145, 0xd145, PDF_CMAP_SINGLE, 3325 },
	{ 0xd146, 0xd148, PDF_CMAP_RANGE, 17822 },
	{ 0xd149, 0xd149, PDF_CMAP_SINGLE, 3326 },
	{ 0xd14a, 0xd150, PDF_CMAP_RANGE, 17825 },
	{ 0xd151, 0xd151, PDF_CMAP_SINGLE, 3327 },
	{ 0xd153, 0xd153, PDF_CMAP_SINGLE, 3328 },
	{ 0xd154, 0xd154, PDF_CMAP_SINGLE, 17832 },
	{ 0xd155, 0xd155, PDF_CMAP_SINGLE, 3329 },
	{ 0xd156, 0xd156, PDF_CMAP_SINGLE, 17833 },
	{ 0xd157, 0xd157, PDF_CMAP_SINGLE, 3330 },
	{ 0xd158, 0xd15d, PDF_CMAP_RANGE, 17834 },
	{ 0xd161, 0xd162, PDF_CMAP_RANGE, 3331 },
	{ 0xd163, 0xd164, PDF_CMAP_RANGE, 17840 },
	{ 0xd165, 0xd165, PDF_CMAP_SINGLE, 3333 },
	{ 0xd166, 0xd168, PDF_CMAP_RANGE, 17842 },
	{ 0xd169, 0xd169, PDF_CMAP_SINGLE, 3334 },
	{ 0xd16a, 0xd170, PDF_CMAP_RANGE, 17845 },
	{ 0xd171, 0xd171, PDF_CMAP_SINGLE, 3335 },
	{ 0xd173, 0xd173, PDF_CMAP_SINGLE, 3336 },
	{ 0xd174, 0xd174, PDF_CMAP_SINGLE, 17852 },
	{ 0xd175, 0xd177, PDF_CMAP_RANGE, 3337 },
	{ 0xd178, 0xd17d, PDF_CMAP_RANGE, 17853 },
	{ 0xd181, 0xd181, PDF_CMAP_SINGLE, 3340 },
	{ 0xd182, 0xd184, PDF_CMAP_RANGE, 17859 },
	{ 0xd185, 0xd185, PDF_CMAP_SINGLE, 3341 },
	{ 0xd186, 0xd188, PDF_CMAP_RANGE, 17862 },
	{ 0xd189, 0xd189, PDF_CMAP_SINGLE, 3342 },
	{ 0xd18a, 0xd191, PDF_CMAP_RANGE, 17865 },
	{ 0xd193, 0xd193, PDF_CMAP_SINGLE, 3343 },
	{ 0xd194, 0xd19d, PDF_CMAP_RANGE, 17873 },
	{ 0xd1a1, 0xd1a2, PDF_CMAP_RANGE, 3344 },
	{ 0xd1a3, 0xd1a4, PDF_CMAP_RANGE, 17883 },
	{ 0xd1a5, 0xd1a5, PDF_CMAP_SINGLE, 3346 },
	{ 0xd1a6, 0xd1a8, PDF_CMAP_RANGE, 17885 },
	{ 0xd1a9, 0xd1a9, PDF_CMAP_SINGLE, 3347 },
	{ 0xd1aa, 0xd1ad, PDF_CMAP_RANGE, 17888 },
	{ 0xd1ae, 0xd1ae, PDF_CMAP_SINGLE, 3348 },
	{ 0xd1af, 0xd1b0, PDF_CMAP_RANGE, 17892 },
	{ 0xd1b1, 0xd1b1, PDF_CMAP_SINGLE, 3349 },
	{ 0xd1b3, 0xd1b3, PDF_CMAP_SINGLE, 3350 },
	{ 0xd1b4, 0xd1b4, PDF_CMAP_SINGLE, 17894 },
	{ 0xd1b5, 0xd1b5, PDF_CMAP_SINGLE, 3351 },
	{ 0xd1b6, 0xd1b6, PDF_CMAP_SINGLE, 17895 },
	{ 0xd1b7, 0xd1b7, PDF_CMAP_SINGLE, 3352 },
	{ 0xd1b8, 0xd1ba, PDF_CMAP_RANGE, 17896 },
	{ 0xd1bb, 0xd1bb, PDF_CMAP_SINGLE, 3353 },
	{ 0xd1bc, 0xd1bd, PDF_CMAP_RANGE, 17899 },
	{ 0xd1c1, 0xd1c2, PDF_CMAP_RANGE, 3354 },
	{ 0xd1c3, 0xd1c4, PDF_CMAP_RANGE, 17901 },
	{ 0xd1c5, 0xd1c5, PDF_CMAP_SINGLE, 3356 },
	{ 0xd1c6, 0xd1c8, PDF_CMAP_RANGE, 17903 },
	{ 0xd1c9, 0xd1c9, PDF_CMAP_SINGLE, 3357 },
	{ 0xd1ca, 0xd1d1, PDF_CMAP_RANGE, 17906 },
	{ 0xd1d3, 0xd1d4, PDF_CMAP_RANGE, 17914 },
	{ 0xd1d5, 0xd1d5, PDF_CMAP_SINGLE, 3358 },
	{ 0xd1d6, 0xd1d6, PDF_CMAP_SINGLE, 17916 },
	{ 0xd1d7, 0xd1d7, PDF_CMAP_SINGLE, 3359 },
	{ 0xd1d8, 0xd1dd, PDF_CMAP_RANGE, 17917 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_RANGE, 3360 },
	{ 0xd1e3, 0xd1e4, PDF_CMAP_RANGE, 17923 },
	{ 0xd1e5, 0xd1e5, PDF_CMAP_SINGLE, 3362 },
	{ 0xd1e6, 0xd1f1, PDF_CMAP_RANGE, 17925 },
	{ 0xd1f3, 0xd1f4, PDF_CMAP_RANGE, 17937 },
	{ 0xd1f5, 0xd1f5, PDF_CMAP_SINGLE, 3363 },
	{ 0xd1f6, 0xd1f6, PDF_CMAP_SINGLE, 17939 },
	{ 0xd1f7, 0xd1f7, PDF_CMAP_SINGLE, 3364 },
	{ 0xd1f8, 0xd1fd, PDF_CMAP_RANGE, 17940 },
	{ 0xd241, 0xd242, PDF_CMAP_RANGE, 3365 },
	{ 0xd243, 0xd244, PDF_CMAP_RANGE, 17946 },
	{ 0xd245, 0xd245, PDF_CMAP_SINGLE, 3367 },
	{ 0xd246, 0xd248, PDF_CMAP_RANGE, 17948 },
	{ 0xd249, 0xd249, PDF_CMAP_SINGLE, 3368 },
	{ 0xd24a, 0xd251, PDF_CMAP_RANGE, 17951 },
	{ 0xd253, 0xd253, PDF_CMAP_SINGLE, 3369 },
	{ 0xd254, 0xd254, PDF_CMAP_SINGLE, 17959 },
	{ 0xd255, 0xd255, PDF_CMAP_SINGLE, 3370 },
	{ 0xd256, 0xd256, PDF_CMAP_SINGLE, 17960 },
	{ 0xd257, 0xd257, PDF_CMAP_SINGLE, 3371 },
	{ 0xd258, 0xd25d, PDF_CMAP_RANGE, 17961 },
	{ 0xd261, 0xd261, PDF_CMAP_SINGLE, 3372 },
	{ 0xd262, 0xd264, PDF_CMAP_RANGE, 17967 },
	{ 0xd265, 0xd265, PDF_CMAP_SINGLE, 3373 },
	{ 0xd266, 0xd268, PDF_CMAP_RANGE, 17970 },
	{ 0xd269, 0xd269, PDF_CMAP_SINGLE, 3374 },
	{ 0xd26a, 0xd271, PDF_CMAP_RANGE, 17973 },
	{ 0xd273, 0xd273, PDF_CMAP_SINGLE, 3375 },
	{ 0xd274, 0xd274, PDF_CMAP_SINGLE, 17981 },
	{ 0xd275, 0xd275, PDF_CMAP_SINGLE, 3376 },
	{ 0xd276, 0xd27d, PDF_CMAP_RANGE, 17982 },
	{ 0xd281, 0xd282, PDF_CMAP_RANGE, 3377 },
	{ 0xd283, 0xd284, PDF_CMAP_RANGE, 17990 },
	{ 0xd285, 0xd285, PDF_CMAP_SINGLE, 3379 },
	{ 0xd286, 0xd288, PDF_CMAP_RANGE, 17992 },
	{ 0xd289, 0xd289, PDF_CMAP_SINGLE, 3380 },
	{ 0xd28a, 0xd28d, PDF_CMAP_RANGE, 17995 },
	{ 0xd28e, 0xd28e, PDF_CMAP_SINGLE, 3381 },
	{ 0xd28f, 0xd290, PDF_CMAP_RANGE, 17999 },
	{ 0xd291, 0xd291, PDF_CMAP_SINGLE, 3382 },
	{ 0xd293, 0xd294, PDF_CMAP_RANGE, 18001 },
	{ 0xd295, 0xd295, PDF_CMAP_SINGLE, 3383 },
	{ 0xd296, 0xd296, PDF_CMAP_SINGLE, 18003 },
	{ 0xd297, 0xd297, PDF_CMAP_SINGLE, 3384 },
	{ 0xd298, 0xd29d, PDF_CMAP_RANGE, 18004 },
	{ 0xd2a1, 0xd2a1, PDF_CMAP_SINGLE, 3385 },
	{ 0xd2a2, 0xd2a4, PDF_CMAP_RANGE, 18010 },
	{ 0xd2a5, 0xd2a5, PDF_CMAP_SINGLE, 3386 },
	{ 0xd2a6, 0xd2a8, PDF_CMAP_RANGE, 18013 },
	{ 0xd2a9, 0xd2a9, PDF_CMAP_SINGLE, 3387 },
	{ 0xd2aa, 0xd2b0, PDF_CMAP_RANGE, 18016 },
	{ 0xd2b1, 0xd2b1, PDF_CMAP_SINGLE, 3388 },
	{ 0xd2b3, 0xd2b6, PDF_CMAP_RANGE, 18023 },
	{ 0xd2b7, 0xd2b7, PDF_CMAP_SINGLE, 3389 },
	{ 0xd2b8, 0xd2bd, PDF_CMAP_RANGE, 18027 },
	{ 0xd2c1, 0xd2c2, PDF_CMAP_RANGE, 3390 },
	{ 0xd2c3, 0xd2c4, PDF_CMAP_RANGE, 18033 },
	{ 0xd2c5, 0xd2c5, PDF_CMAP_SINGLE, 3392 },
	{ 0xd2c6, 0xd2c8, PDF_CMAP_RANGE, 18035 },
	{ 0xd2c9, 0xd2c9, PDF_CMAP_SINGLE, 3393 },
	{ 0xd2ca, 0xd2d1, PDF_CMAP_RANGE, 18038 },
	{ 0xd2d3, 0xd2d6, PDF_CMAP_RANGE, 18046 },
	{ 0xd2d7, 0xd2d7, PDF_CMAP_SINGLE, 3394 },
	{ 0xd2d8, 0xd2dd, PDF_CMAP_RANGE, 18050 },
	{ 0xd2e1, 0xd2e2, PDF_CMAP_RANGE, 3395 },
	{ 0xd2e3, 0xd2e4, PDF_CMAP_RANGE, 18056 },
	{ 0xd2e5, 0xd2e5, PDF_CMAP_SINGLE, 3397 },
	{ 0xd2e6, 0xd2e8, PDF_CMAP_RANGE, 18058 },
	{ 0xd2e9, 0xd2e9, PDF_CMAP_SINGLE, 3398 },
	{ 0xd2ea, 0xd2f0, PDF_CMAP_RANGE, 18061 },
	{ 0xd2f1, 0xd2f1, PDF_CMAP_SINGLE, 3399 },
	{ 0xd2f3, 0xd2f3, PDF_CMAP_SINGLE, 3400 },
	{ 0xd2f4, 0xd2f4, PDF_CMAP_SINGLE, 18068 },
	{ 0xd2f5, 0xd2f5, PDF_CMAP_SINGLE, 3401 },
	{ 0xd2f6, 0xd2f6, PDF_CMAP_SINGLE, 18069 },
	{ 0xd2f7, 0xd2f7, PDF_CMAP_SINGLE, 3402 },
	{ 0xd2f8, 0xd2fd, PDF_CMAP_RANGE, 18070 },
	{ 0xd341, 0xd342, PDF_CMAP_RANGE, 3403 },
	{ 0xd343, 0xd344, PDF_CMAP_RANGE, 18076 },
	{ 0xd345, 0xd345, PDF_CMAP_SINGLE, 3405 },
	{ 0xd346, 0xd348, PDF_CMAP_RANGE, 18078 },
	{ 0xd349, 0xd349, PDF_CMAP_SINGLE, 3406 },
	{ 0xd34a, 0xd350, PDF_CMAP_RANGE, 18081 },
	{ 0xd351, 0xd351, PDF_CMAP_SINGLE, 3407 },
	{ 0xd353, 0xd354, PDF_CMAP_RANGE, 18088 },
	{ 0xd355, 0xd355, PDF_CMAP_SINGLE, 3408 },
	{ 0xd356, 0xd356, PDF_CMAP_SINGLE, 18090 },
	{ 0xd357, 0xd357, PDF_CMAP_SINGLE, 3409 },
	{ 0xd358, 0xd35d, PDF_CMAP_RANGE, 18091 },
	{ 0xd361, 0xd362, PDF_CMAP_RANGE, 3410 },
	{ 0xd363, 0xd364, PDF_CMAP_RANGE, 18097 },
	{ 0xd365, 0xd365, PDF_CMAP_SINGLE, 3412 },
	{ 0xd366, 0xd366, PDF_CMAP_SINGLE, 18099 },
	{ 0xd367, 0xd36a, PDF_CMAP_RANGE, 3413 },
	{ 0xd36b, 0xd370, PDF_CMAP_RANGE, 18100 },
	{ 0xd371, 0xd371, PDF_CMAP_SINGLE, 3417 },
	{ 0xd373, 0xd373, PDF_CMAP_SINGLE, 3418 },
	{ 0xd374, 0xd374, PDF_CMAP_SINGLE, 18106 },
	{ 0xd375, 0xd375, PDF_CMAP_SINGLE, 3419 },
	{ 0xd376, 0xd376, PDF_CMAP_SINGLE, 18107 },
	{ 0xd377, 0xd377, PDF_CMAP_SINGLE, 3420 },
	{ 0xd378, 0xd37a, PDF_CMAP_RANGE, 18108 },
	{ 0xd37b, 0xd37b, PDF_CMAP_SINGLE, 3421 },
	{ 0xd37c, 0xd37d, PDF_CMAP_RANGE, 18111 },
	{ 0xd381, 0xd381, PDF_CMAP_SINGLE, 3422 },
	{ 0xd382, 0xd384, PDF_CMAP_RANGE, 18113 },
	{ 0xd385, 0xd385, PDF_CMAP_SINGLE, 3423 },
	{ 0xd386, 0xd388, PDF_CMAP_RANGE, 18116 },
	{ 0xd389, 0xd389, PDF_CMAP_SINGLE, 3424 },
	{ 0xd38a, 0xd390, PDF_CMAP_RANGE, 18119 },
	{ 0xd391, 0xd391, PDF_CMAP_SINGLE, 3425 },
	{ 0xd393, 0xd393, PDF_CMAP_SINGLE, 3426 },
	{ 0xd394, 0xd396, PDF_CMAP_RANGE, 18126 },
	{ 0xd397, 0xd397, PDF_CMAP_SINGLE, 3427 },
	{ 0xd398, 0xd39d, PDF_CMAP_RANGE, 18129 },
	{ 0xd3a1, 0xd3a2, PDF_CMAP_RANGE, 3428 },
	{ 0xd3a3, 0xd3a4, PDF_CMAP_RANGE, 18135 },
	{ 0xd3a5, 0xd3a5, PDF_CMAP_SINGLE, 3430 },
	{ 0xd3a6, 0xd3a8, PDF_CMAP_RANGE, 18137 },
	{ 0xd3a9, 0xd3a9, PDF_CMAP_SINGLE, 3431 },
	{ 0xd3aa, 0xd3b0, PDF_CMAP_RANGE, 18140 },
	{ 0xd3b1, 0xd3b1, PDF_CMAP_SINGLE, 3432 },
	{ 0xd3b3, 0xd3b3, PDF_CMAP_SINGLE, 3433 },
	{ 0xd3b4, 0xd3b4, PDF_CMAP_SINGLE, 18147 },
	{ 0xd3b5, 0xd3b5, PDF_CMAP_SINGLE, 3434 },
	{ 0xd3b6, 0xd3b6, PDF_CMAP_SINGLE, 18148 },
	{ 0xd3b7, 0xd3b7, PDF_CMAP_SINGLE, 3435 },
	{ 0xd3b8, 0xd3bd, PDF_CMAP_RANGE, 18149 },
	{ 0xd931, 0xd97e, PDF_CMAP_RANGE, 101 },
	{ 0xd991, 0xd9e5, PDF_CMAP_RANGE, 179 },
	{ 0xda31, 0xda7e, PDF_CMAP_RANGE, 264 },
	{ 0xda91, 0xdaa0, PDF_CMAP_RANGE, 342 },
	{ 0xdad5, 0xdafe, PDF_CMAP_RANGE, 409 },
	{ 0xdb31, 0xdb3a, PDF_CMAP_RANGE, 451 },
	{ 0xdb40, 0xdb49, PDF_CMAP_RANGE, 461 },
	{ 0xdb51, 0xdb68, PDF_CMAP_RANGE, 471 },
	{ 0xdb71, 0xdb7e, PDF_CMAP_RANGE, 495 },
	{ 0xdb91, 0xdb9a, PDF_CMAP_RANGE, 509 },
	{ 0xdba1, 0xdbe4, PDF_CMAP_RANGE, 519 },
	{ 0xdc31, 0xdc7e, PDF_CMAP_RANGE, 587 },
	{ 0xdc91, 0xdc91, PDF_CMAP_SINGLE, 665 },
	{ 0xdca1, 0xdca4, PDF_CMAP_RANGE, 666 },
	{ 0xdca6, 0xdca6, PDF_CMAP_SINGLE, 670 },
	{ 0xdca8, 0xdcaf, PDF_CMAP_RANGE, 671 },
	{ 0xdcb1, 0xdcfe, PDF_CMAP_RANGE, 679 },
	{ 0xdd31, 0xdd7e, PDF_CMAP_RANGE, 757 },
	{ 0xdd91, 0xddf3, PDF_CMAP_RANGE, 835 },
	{ 0xde31, 0xde7e, PDF_CMAP_RANGE, 934 },
	{ 0xde91, 0xde98, PDF_CMAP_RANGE, 1012 },
	{ 0xdea1, 0xdec1, PDF_CMAP_RANGE, 1020 },
	{ 0xded1, 0xdef1, PDF_CMAP_RANGE, 1053 },
	{ 0xe031, 0xe07e, PDF_CMAP_RANGE, 3436 },
	{ 0xe091, 0xe0cf, PDF_CMAP_RANGE, 3514 },
	{ 0xe0d0, 0xe0d0, PDF_CMAP_SINGLE, 4116 },
	{ 0xe0d1, 0xe0d5, PDF_CMAP_RANGE, 3577 },
	{ 0xe0d6, 0xe0d6, PDF_CMAP_SINGLE, 3678 },
	{ 0xe0d7, 0xe0e6, PDF_CMAP_RANGE, 3582 },
	{ 0xe0e7, 0xe0e7, PDF_CMAP_SINGLE, 7053 },
	{ 0xe0e8, 0xe0fe, PDF_CMAP_RANGE, 3598 },
	{ 0xe131, 0xe17e, PDF_CMAP_RANGE, 3621 },
	{ 0xe191, 0xe1ce, PDF_CMAP_RANGE, 3699 },
	{ 0xe1cf, 0xe1cf, PDF_CMAP_SINGLE, 3460 },
	{ 0xe1d0, 0xe1e7, PDF_CMAP_RANGE, 3761 },
	{ 0xe1e8, 0xe1e8, PDF_CMAP_SINGLE, 7900 },
	{ 0xe1e9, 0xe1fe, PDF_CMAP_RANGE, 3785 },
	{ 0xe231, 0xe23c, PDF_CMAP_RANGE, 3807 },
	{ 0xe23d, 0xe23d, PDF_CMAP_SINGLE, 3802 },
	{ 0xe23e, 0xe27e, PDF_CMAP_RANGE, 3819 },
	{ 0xe291, 0xe2fa, PDF_CMAP_RANGE, 3884 },
	{ 0xe2fb, 0xe2fb, PDF_CMAP_SINGLE, 3902 },
	{ 0xe2fc, 0xe2fe, PDF_CMAP_RANGE, 3990 },
	{ 0xe331, 0xe331, PDF_CMAP_SINGLE, 3993 },
	{ 0xe332, 0xe332, PDF_CMAP_SINGLE, 3946 },
	{ 0xe333, 0xe347, PDF_CMAP_RANGE, 3994 },
	{ 0xe348, 0xe348, PDF_CMAP_SINGLE, 3946 },
	{ 0xe349, 0xe35f, PDF_CMAP_RANGE, 4015 },
	{ 0xe360, 0xe360, PDF_CMAP_SINGLE, 3708 },
	{ 0xe361, 0xe36c, PDF_CMAP_RANGE, 4038 },
	{ 0xe36d, 0xe36d, PDF_CMAP_SINGLE, 4131 },
	{ 0xe36e, 0xe37e, PDF_CMAP_RANGE, 4050 },
	{ 0xe391, 0xe3d3, PDF_CMAP_RANGE, 4067 },
	{ 0xe3d4, 0xe3d4, PDF_CMAP_SINGLE, 4374 },
	{ 0xe3d5, 0xe3d5, PDF_CMAP_SINGLE, 4156 },
	{ 0xe3d6, 0xe3d7, PDF_CMAP_RANGE, 4134 },
	{ 0xe3d8, 0xe3d8, PDF_CMAP_SINGLE, 4375 },
	{ 0xe3d9, 0xe3da, PDF_CMAP_RANGE, 4136 },
	{ 0xe3db, 0xe3e0, PDF_CMAP_RANGE, 4376 },
	{ 0xe3e1, 0xe3e1, PDF_CMAP_SINGLE, 4138 },
	{ 0xe3e2, 0xe3e2, PDF_CMAP_SINGLE, 5800 },
	{ 0xe3e3, 0xe3e5, PDF_CMAP_RANGE, 4382 },
	{ 0xe3e6, 0xe3e6, PDF_CMAP_SINGLE, 4386 },
	{ 0xe3e7, 0xe3e7, PDF_CMAP_SINGLE, 4139 },
	{ 0xe3e8, 0xe3eb, PDF_CMAP_RANGE, 4387 },
	{ 0xe3ec, 0xe3ec, PDF_CMAP_SINGLE, 4140 },
	{ 0xe3ed, 0xe3ed, PDF_CMAP_SINGLE, 4391 },
	{ 0xe3ee, 0xe3ee, PDF_CMAP_SINGLE, 4141 },
	{ 0xe3ef, 0xe3f0, PDF_CMAP_RANGE, 4394 },
	{ 0xe3f1, 0xe3f1, PDF_CMAP_SINGLE, 4142 },
	{ 0xe3f2, 0xe3f2, PDF_CMAP_SINGLE, 4396 },
	{ 0xe3f3, 0xe3f5, PDF_CMAP_RANGE, 4143 },
	{ 0xe3f6, 0xe3f6, PDF_CMAP_SINGLE, 4399 },
	{ 0xe3f7, 0xe3f9, PDF_CMAP_RANGE, 4146 },
	{ 0xe3fa, 0xe3fa, PDF_CMAP_SINGLE, 4403 },
	{ 0xe3fb, 0xe3fb, PDF_CMAP_SINGLE, 4149 },
	{ 0xe3fc, 0xe3fd, PDF_CMAP_RANGE, 4406 },
	{ 0xe3fe, 0xe3fe, PDF_CMAP_SINGLE, 4409 },
	{ 0xe431, 0xe431, PDF_CMAP_SINGLE, 4150 },
	{ 0xe432, 0xe433, PDF_CMAP_RANGE, 4410 },
	{ 0xe434, 0xe436, PDF_CMAP_RANGE, 4151 },
	{ 0xe437, 0xe43a, PDF_CMAP_RANGE, 4412 },
	{ 0xe43b, 0xe43b, PDF_CMAP_SINGLE, 4419 },
	{ 0xe43c, 0xe43c, PDF_CMAP_SINGLE, 4154 },
	{ 0xe43d, 0xe43d, PDF_CMAP_SINGLE, 4420 },
	{ 0xe43e, 0xe441, PDF_CMAP_RANGE, 4155 },
	{ 0xe442, 0xe442, PDF_CMAP_SINGLE, 4424 },
	{ 0xe443, 0xe44d, PDF_CMAP_RANGE, 4159 },
	{ 0xe44e, 0xe44e, PDF_CMAP_SINGLE, 4511 },
	{ 0xe44f, 0xe451, PDF_CMAP_RANGE, 4170 },
	{ 0xe452, 0xe453, PDF_CMAP_RANGE, 4513 },
	{ 0xe454, 0xe454, PDF_CMAP_SINGLE, 4517 },
	{ 0xe455, 0xe455, PDF_CMAP_SINGLE, 4173 },
	{ 0xe456, 0xe45a, PDF_CMAP_RANGE, 4518 },
	{ 0xe45b, 0xe45b, PDF_CMAP_SINGLE, 4524 },
	{ 0xe45c, 0xe45c, PDF_CMAP_SINGLE, 4174 },
	{ 0xe45d, 0xe45e, PDF_CMAP_RANGE, 4525 },
	{ 0xe45f, 0xe464, PDF_CMAP_RANGE, 4528 },
	{ 0xe465, 0xe467, PDF_CMAP_RANGE, 4535 },
	{ 0xe468, 0xe468, PDF_CMAP_SINGLE, 4175 },
	{ 0xe469, 0xe46a, PDF_CMAP_RANGE, 4541 },
	{ 0xe46b, 0xe46d, PDF_CMAP_RANGE, 4176 },
	{ 0xe46e, 0xe46f, PDF_CMAP_RANGE, 4545 },
	{ 0xe470, 0xe470, PDF_CMAP_SINGLE, 4179 },
	{ 0xe471, 0xe471, PDF_CMAP_SINGLE, 4547 },
	{ 0xe472, 0xe472, PDF_CMAP_SINGLE, 4550 },
	{ 0xe473, 0xe473, PDF_CMAP_SINGLE, 4180 },
	{ 0xe474, 0xe474, PDF_CMAP_SINGLE, 4564 },
	{ 0xe475, 0xe478, PDF_CMAP_RANGE, 4566 },
	{ 0xe479, 0xe47a, PDF_CMAP_RANGE, 4571 },
	{ 0xe47b, 0xe47b, PDF_CMAP_SINGLE, 4576 },
	{ 0xe47c, 0xe47e, PDF_CMAP_RANGE, 4181 },
	{ 0xe491, 0xe491, PDF_CMAP_SINGLE, 4184 },
	{ 0xe492, 0xe495, PDF_CMAP_RANGE, 4604 },
	{ 0xe496, 0xe497, PDF_CMAP_RANGE, 4609 },
	{ 0xe498, 0xe498, PDF_CMAP_SINGLE, 4185 },
	{ 0xe499, 0xe49a, PDF_CMAP_RANGE, 4611 },
	{ 0xe49b, 0xe4fe, PDF_CMAP_RANGE, 4186 },
	{ 0xe531, 0xe575, PDF_CMAP_RANGE, 4286 },
	{ 0xe576, 0xe576, PDF_CMAP_SINGLE, 4318 },
	{ 0xe577, 0xe57e, PDF_CMAP_RANGE, 4355 },
	{ 0xe591, 0xe59d, PDF_CMAP_RANGE, 4363 },
	{ 0xe59e, 0xe59e, PDF_CMAP_SINGLE, 4136 },
	{ 0xe59f, 0xe5a4, PDF_CMAP_RANGE, 4376 },
	{ 0xe5a5, 0xe5a5, PDF_CMAP_SINGLE, 5800 },
	{ 0xe5a6, 0xe5aa, PDF_CMAP_RANGE, 4382 },
	{ 0xe5ab, 0xe5ab, PDF_CMAP_SINGLE, 4139 },
	{ 0xe5ac, 0xe5ad, PDF_CMAP_RANGE, 4387 },
	{ 0xe5ae, 0xe5ae, PDF_CMAP_SINGLE, 4192 },
	{ 0xe5af, 0xe5fe, PDF_CMAP_RANGE, 4389 },
	{ 0xe631, 0xe647, PDF_CMAP_RANGE, 4469 },
	{ 0xe648, 0xe648, PDF_CMAP_SINGLE, 4167 },
	{ 0xe649, 0xe65c, PDF_CMAP_RANGE, 4492 },
	{ 0xe65d, 0xe65d, PDF_CMAP_SINGLE, 4172 },
	{ 0xe65e, 0xe67e, PDF_CMAP_RANGE, 4512 },
	{ 0xe691, 0xe6ca, PDF_CMAP_RANGE, 4545 },
	{ 0xe6cb, 0xe6cb, PDF_CMAP_SINGLE, 5552 },
	{ 0xe6cc, 0xe6e3, PDF_CMAP_RANGE, 4603 },
	{ 0xe6e4, 0xe6e4, PDF_CMAP_SINGLE, 6424 },
	{ 0xe6e5, 0xe6fe, PDF_CMAP_RANGE, 4627 },
	{ 0xe731, 0xe77e, PDF_CMAP_RANGE, 4653 },
	{ 0xe791, 0xe7fe, PDF_CMAP_RANGE, 4731 },
	{ 0xe831, 0xe87e, PDF_CMAP_RANGE, 4841 },
	{ 0xe891, 0xe8c4, PDF_CMAP_RANGE, 4919 },
	{ 0xe8c5, 0xe8c5, PDF_CMAP_SINGLE, 5151 },
	{ 0xe8c6, 0xe8e3, PDF_CMAP_RANGE, 4971 },
	{ 0xe8e4, 0xe8e4, PDF_CMAP_SINGLE, 4922 },
	{ 0xe8e5, 0xe8fe, PDF_CMAP_RANGE, 5001 },
	{ 0xe931, 0xe934, PDF_CMAP_RANGE, 5027 },
	{ 0xe935, 0xe935, PDF_CMAP_SINGLE, 7518 },
	{ 0xe936, 0xe97e, PDF_CMAP_RANGE, 5031 },
	{ 0xe991, 0xe9a4, PDF_CMAP_RANGE, 5104 },
	{ 0xe9a5, 0xe9a5, PDF_CMAP_SINGLE, 5079 },
	{ 0xe9a6, 0xe9d4, PDF_CMAP_RANGE, 5124 },
	{ 0xe9d5, 0xe9d5, PDF_CMAP_SINGLE, 5109 },
	{ 0xe9d6, 0xe9f3, PDF_CMAP_RANGE, 5171 },
	{ 0xe9f4, 0xe9f4, PDF_CMAP_SINGLE, 7607 },
	{ 0xe9f5, 0xe9fe, PDF_CMAP_RANGE, 5201 },
	{ 0xea31, 0xea7e, PDF_CMAP_RANGE, 5211 },
	{ 0xea91, 0xea9d, PDF_CMAP_RANGE, 5289 },
	{ 0xea9e, 0xea9e, PDF_CMAP_SINGLE, 5584 },
	{ 0xea9f, 0xea9f, PDF_CMAP_SINGLE, 5302 },
	{ 0xeaa0, 0xeaa0, PDF_CMAP_SINGLE, 5367 },
	{ 0xeaa1, 0xeab2, PDF_CMAP_RANGE, 5303 },
	{ 0xeab3, 0xeab3, PDF_CMAP_SINGLE, 7083 },
	{ 0xeab4, 0xeae0, PDF_CMAP_RANGE, 5321 },
	{ 0xeae1, 0xeae1, PDF_CMAP_SINGLE, 5362 },
	{ 0xeae2, 0xeae7, PDF_CMAP_RANGE, 5366 },
	{ 0xeae8, 0xeae8, PDF_CMAP_SINGLE, 5485 },
	{ 0xeae9, 0xeafe, PDF_CMAP_RANGE, 5372 },
	{ 0xeb31, 0xeb7e, PDF_CMAP_RANGE, 5394 },
	{ 0xeb91, 0xeb92, PDF_CMAP_RANGE, 5472 },
	{ 0xeb93, 0xeb93, PDF_CMAP_SINGLE, 6001 },
	{ 0xeb94, 0xebac, PDF_CMAP_RANGE, 5474 },
	{ 0xebad, 0xebad, PDF_CMAP_SINGLE, 5460 },
	{ 0xebae, 0xebec, PDF_CMAP_RANGE, 5499 },
	{ 0xebed, 0xebed, PDF_CMAP_SINGLE, 5317 },
	{ 0xebee, 0xebfe, PDF_CMAP_RANGE, 5562 },
	{ 0xec31, 0xec7e, PDF_CMAP_RANGE, 5579 },
	{ 0xec91, 0xecf4, PDF_CMAP_RANGE, 5657 },
	{ 0xecf5, 0xecf5, PDF_CMAP_SINGLE, 7009 },
	{ 0xecf6, 0xecfe, PDF_CMAP_RANGE, 5757 },
	{ 0xed31, 0xed31, PDF_CMAP_SINGLE, 7370 },
	{ 0xed32, 0xed38, PDF_CMAP_RANGE, 5766 },
	{ 0xed39, 0xed39, PDF_CMAP_SINGLE, 5678 },
	{ 0xed3a, 0xed7e, PDF_CMAP_RANGE, 5773 },
	{ 0xed91, 0xedad, PDF_CMAP_RANGE, 5842 },
	{ 0xedae, 0xedae, PDF_CMAP_SINGLE, 5874 },
	{ 0xedaf, 0xedb0, PDF_CMAP_RANGE, 5871 },
	{ 0xedb1, 0xedb2, PDF_CMAP_RANGE, 4425 },
	{ 0xedb3, 0xedb8, PDF_CMAP_RANGE, 5873 },
	{ 0xedb9, 0xedb9, PDF_CMAP_SINGLE, 4427 },
	{ 0xedba, 0xedba, PDF_CMAP_SINGLE, 5879 },
	{ 0xedbb, 0xedbc, PDF_CMAP_RANGE, 4429 },
	{ 0xedbd, 0xedc3, PDF_CMAP_RANGE, 5880 },
	{ 0xedc4, 0xedc4, PDF_CMAP_SINGLE, 4431 },
	{ 0xedc5, 0xedcd, PDF_CMAP_RANGE, 5887 },
	{ 0xedce, 0xedce, PDF_CMAP_SINGLE, 4435 },
	{ 0xedcf, 0xedcf, PDF_CMAP_SINGLE, 5896 },
	{ 0xedd0, 0xedd0, PDF_CMAP_SINGLE, 4436 },
	{ 0xedd1, 0xedd1, PDF_CMAP_SINGLE, 5897 },
	{ 0xedd2, 0xedd2, PDF_CMAP_SINGLE, 4437 },
	{ 0xedd3, 0xedd5, PDF_CMAP_RANGE, 5898 },
	{ 0xedd6, 0xedd6, PDF_CMAP_SINGLE, 4439 },
	{ 0xedd7, 0xedf9, PDF_CMAP_RANGE, 5901 },
	{ 0xedfa, 0xedfb, PDF_CMAP_RANGE, 4442 },
	{ 0xedfc, 0xedfc, PDF_CMAP_SINGLE, 4159 },
	{ 0xedfd, 0xedfd, PDF_CMAP_SINGLE, 5936 },
	{ 0xedfe, 0xedfe, PDF_CMAP_SINGLE, 4444 },
	{ 0xee31, 0xee31, PDF_CMAP_SINGLE, 4447 },
	{ 0xee32, 0xee33, PDF_CMAP_RANGE, 5937 },
	{ 0xee34, 0xee34, PDF_CMAP_SINGLE, 4449 },
	{ 0xee35, 0xee36, PDF_CMAP_RANGE, 5939 },
	{ 0xee37, 0xee37, PDF_CMAP_SINGLE, 4450 },
	{ 0xee38, 0xee3c, PDF_CMAP_RANGE, 5941 },
	{ 0xee3d, 0xee3d, PDF_CMAP_SINGLE, 4453 },
	{ 0xee3e, 0xee3e, PDF_CMAP_SINGLE, 5946 },
	{ 0xee3f, 0xee41, PDF_CMAP_RANGE, 4455 },
	{ 0xee42, 0xee42, PDF_CMAP_SINGLE, 5947 },
	{ 0xee43, 0xee43, PDF_CMAP_SINGLE, 4458 },
	{ 0xee44, 0xee46, PDF_CMAP_RANGE, 5948 },
	{ 0xee47, 0xee48, PDF_CMAP_RANGE, 4459 },
	{ 0xee49, 0xee4b, PDF_CMAP_RANGE, 5951 },
	{ 0xee4c, 0xee4c, PDF_CMAP_SINGLE, 4463 },
	{ 0xee4d, 0xee53, PDF_CMAP_RANGE, 5954 },
	{ 0xee54, 0xee54, PDF_CMAP_SINGLE, 4160 },
	{ 0xee55, 0xee55, PDF_CMAP_SINGLE, 5961 },
	{ 0xee56, 0xee57, PDF_CMAP_RANGE, 4465 },
	{ 0xee58, 0xee59, PDF_CMAP_RANGE, 5962 },
	{ 0xee5a, 0xee5a, PDF_CMAP_SINGLE, 4161 },
	{ 0xee5b, 0xee61, PDF_CMAP_RANGE, 5964 },
	{ 0xee62, 0xee62, PDF_CMAP_SINGLE, 4468 },
	{ 0xee63, 0xee65, PDF_CMAP_RANGE, 5971 },
	{ 0xee66, 0xee66, PDF_CMAP_SINGLE, 4469 },
	{ 0xee67, 0xee68, PDF_CMAP_RANGE, 5974 },
	{ 0xee69, 0xee69, PDF_CMAP_SINGLE, 4470 },
	{ 0xee6a, 0xee6b, PDF_CMAP_RANGE, 5976 },
	{ 0xee6c, 0xee6c, PDF_CMAP_SINGLE, 4162 },
	{ 0xee6d, 0xee6e, PDF_CMAP_RANGE, 5978 },
	{ 0xee6f, 0xee6f, PDF_CMAP_SINGLE, 4471 },
	{ 0xee70, 0xee70, PDF_CMAP_SINGLE, 5980 },
	{ 0xee71, 0xee71, PDF_CMAP_SINGLE, 4472 },
	{ 0xee72, 0xee73, PDF_CMAP_RANGE, 5981 },
	{ 0xee74, 0xee74, PDF_CMAP_SINGLE, 4474 },
	{ 0xee75, 0xee75, PDF_CMAP_SINGLE, 4473 },
	{ 0xee76, 0xee76, PDF_CMAP_SINGLE, 4475 },
	{ 0xee77, 0xee77, PDF_CMAP_SINGLE, 5983 },
	{ 0xee78, 0xee78, PDF_CMAP_SINGLE, 4476 },
	{ 0xee79, 0xee79, PDF_CMAP_SINGLE, 5984 },
	{ 0xee7a, 0xee7b, PDF_CMAP_RANGE, 4478 },
	{ 0xee7c, 0xee7c, PDF_CMAP_SINGLE, 6447 },
	{ 0xee7d, 0xee7e, PDF_CMAP_RANGE, 5985 },
	{ 0xee91, 0xee91, PDF_CMAP_SINGLE, 4481 },
	{ 0xee92, 0xee92, PDF_CMAP_SINGLE, 5987 },
	{ 0xee93, 0xee93, PDF_CMAP_SINGLE, 4482 },
	{ 0xee94, 0xee94, PDF_CMAP_SINGLE, 5460 },
	{ 0xee95, 0xee96, PDF_CMAP_RANGE, 5988 },
	{ 0xee97, 0xee97, PDF_CMAP_SINGLE, 4483 },
	{ 0xee98, 0xee98, PDF_CMAP_SINGLE, 4163 },
	{ 0xee99, 0xee99, PDF_CMAP_SINGLE, 4166 },
	{ 0xee9a, 0xee9a, PDF_CMAP_SINGLE, 5990 },
	{ 0xee9b, 0xee9b, PDF_CMAP_SINGLE, 4485 },
	{ 0xee9c, 0xeea0, PDF_CMAP_RANGE, 5991 },
	{ 0xeea1, 0xeea1, PDF_CMAP_SINGLE, 4487 },
	{ 0xeea2, 0xeea5, PDF_CMAP_RANGE, 5996 },
	{ 0xeea6, 0xeea6, PDF_CMAP_SINGLE, 4488 },
	{ 0xeea7, 0xeea8, PDF_CMAP_RANGE, 6000 },
	{ 0xeea9, 0xeea9, PDF_CMAP_SINGLE, 4489 },
	{ 0xeeaa, 0xeeaa, PDF_CMAP_SINGLE, 4491 },
	{ 0xeeab, 0xeeab, PDF_CMAP_SINGLE, 6002 },
	{ 0xeeac, 0xeeac, PDF_CMAP_SINGLE, 4167 },
	{ 0xeead, 0xeead, PDF_CMAP_SINGLE, 4493 },
	{ 0xeeae, 0xeeaf, PDF_CMAP_RANGE, 6003 },
	{ 0xeeb0, 0xeeb0, PDF_CMAP_SINGLE, 4494 },
	{ 0xeeb1, 0xeebe, PDF_CMAP_RANGE, 6005 },
	{ 0xeebf, 0xeebf, PDF_CMAP_SINGLE, 4495 },
	{ 0xeec0, 0xeec0, PDF_CMAP_SINGLE, 6019 },
	{ 0xeec1, 0xeec1, PDF_CMAP_SINGLE, 7783 },
	{ 0xeec2, 0xeec5, PDF_CMAP_RANGE, 6020 },
	{ 0xeec6, 0xeec6, PDF_CMAP_SINGLE, 4497 },
	{ 0xeec7, 0xeec7, PDF_CMAP_SINGLE, 4499 },
	{ 0xeec8, 0xeeca, PDF_CMAP_RANGE, 6024 },
	{ 0xeecb, 0xeecb, PDF_CMAP_SINGLE, 4501 },
	{ 0xeecc, 0xeecc, PDF_CMAP_SINGLE, 6027 },
	{ 0xeecd, 0xeecd, PDF_CMAP_SINGLE, 4502 },
	{ 0xeece, 0xeece, PDF_CMAP_SINGLE, 6028 },
	{ 0xeecf, 0xeed0, PDF_CMAP_RANGE, 4503 },
	{ 0xeed1, 0xeed2, PDF_CMAP_RANGE, 6029 },
	{ 0xeed3, 0xeed3, PDF_CMAP_SINGLE, 4506 },
	{ 0xeed4, 0xeede, PDF_CMAP_RANGE, 6031 },
	{ 0xeedf, 0xeedf, PDF_CMAP_SINGLE, 4508 },
	{ 0xeee0, 0xeee3, PDF_CMAP_RANGE, 6042 },
	{ 0xeee4, 0xeee4, PDF_CMAP_SINGLE, 4509 },
	{ 0xeee5, 0xeee5, PDF_CMAP_SINGLE, 6046 },
	{ 0xeee6, 0xeee6, PDF_CMAP_SINGLE, 4510 },
	{ 0xeee7, 0xeef6, PDF_CMAP_RANGE, 6047 },
	{ 0xeef7, 0xeef7, PDF_CMAP_SINGLE, 5797 },
	{ 0xeef8, 0xeefe, PDF_CMAP_RANGE, 6063 },
	{ 0xef31, 0xef76, PDF_CMAP_RANGE, 6070 },
	{ 0xef77, 0xef78, PDF_CMAP_RANGE, 4551 },
	{ 0xef79, 0xef7e, PDF_CMAP_RANGE, 6140 },
	{ 0xef91, 0xef91, PDF_CMAP_SINGLE, 6146 },
	{ 0xef92, 0xef92, PDF_CMAP_SINGLE, 4553 },
	{ 0xef93, 0xef93, PDF_CMAP_SINGLE, 4180 },
	{ 0xef94, 0xef98, PDF_CMAP_RANGE, 6147 },
	{ 0xef99, 0xef99, PDF_CMAP_SINGLE, 4555 },
	{ 0xef9a, 0xef9a, PDF_CMAP_SINGLE, 6152 },
	{ 0xef9b, 0xef9b, PDF_CMAP_SINGLE, 5800 },
	{ 0xef9c, 0xef9c, PDF_CMAP_SINGLE, 6153 },
	{ 0xef9d, 0xef9d, PDF_CMAP_SINGLE, 4556 },
	{ 0xef9e, 0xef9f, PDF_CMAP_RANGE, 6154 },
	{ 0xefa0, 0xefa0, PDF_CMAP_SINGLE, 4557 },
	{ 0xefa1, 0xefa6, PDF_CMAP_RANGE, 6156 },
	{ 0xefa7, 0xefa7, PDF_CMAP_SINGLE, 4560 },
	{ 0xefa8, 0xefab, PDF_CMAP_RANGE, 6162 },
	{ 0xefac, 0xefac, PDF_CMAP_SINGLE, 4561 },
	{ 0xefad, 0xefcb, PDF_CMAP_RANGE, 6166 },
	{ 0xefcc, 0xefcc, PDF_CMAP_SINGLE, 4563 },
	{ 0xefcd, 0xeff6, PDF_CMAP_RANGE, 6197 },
	{ 0xeff7, 0xeff7, PDF_CMAP_SINGLE, 7988 },
	{ 0xeff8, 0xeffe, PDF_CMAP_RANGE, 6239 },
	{ 0xf031, 0xf050, PDF_CMAP_RANGE, 6246 },
	{ 0xf051, 0xf051, PDF_CMAP_SINGLE, 6123 },
	{ 0xf052, 0xf074, PDF_CMAP_RANGE, 6278 },
	{ 0xf075, 0xf075, PDF_CMAP_SINGLE, 4577 },
	{ 0xf076, 0xf07e, PDF_CMAP_RANGE, 6313 },
	{ 0xf091, 0xf095, PDF_CMAP_RANGE, 6322 },
	{ 0xf096, 0xf096, PDF_CMAP_SINGLE, 4183 },
	{ 0xf097, 0xf098, PDF_CMAP_RANGE, 6327 },
	{ 0xf099, 0xf099, PDF_CMAP_SINGLE, 4579 },
	{ 0xf09a, 0xf09d, PDF_CMAP_RANGE, 6329 },
	{ 0xf09e, 0xf09e, PDF_CMAP_SINGLE, 4581 },
	{ 0xf09f, 0xf09f, PDF_CMAP_SINGLE, 6333 },
	{ 0xf0a0, 0xf0a0, PDF_CMAP_SINGLE, 4582 },
	{ 0xf0a1, 0xf0a3, PDF_CMAP_RANGE, 6334 },
	{ 0xf0a4, 0xf0a4, PDF_CMAP_SINGLE, 4584 },
	{ 0xf0a5, 0xf0a6, PDF_CMAP_RANGE, 6337 },
	{ 0xf0a7, 0xf0a7, PDF_CMAP_SINGLE, 4586 },
	{ 0xf0a8, 0xf0a8, PDF_CMAP_SINGLE, 6339 },
	{ 0xf0a9, 0xf0a9, PDF_CMAP_SINGLE, 4588 },
	{ 0xf0aa, 0xf0aa, PDF_CMAP_SINGLE, 4184 },
	{ 0xf0ab, 0xf0b9, PDF_CMAP_RANGE, 6340 },
	{ 0xf0ba, 0xf0bb, PDF_CMAP_RANGE, 4590 },
	{ 0xf0bc, 0xf0bc, PDF_CMAP_SINGLE, 6355 },
	{ 0xf0bd, 0xf0bd, PDF_CMAP_SINGLE, 4592 },
	{ 0xf0be, 0xf0c0, PDF_CMAP_RANGE, 6356 },
	{ 0xf0c1, 0xf0c1, PDF_CMAP_SINGLE, 4593 },
	{ 0xf0c2, 0xf0c2, PDF_CMAP_SINGLE, 4595 },
	{ 0xf0c3, 0xf0c5, PDF_CMAP_RANGE, 6359 },
	{ 0xf0c6, 0xf0c7, PDF_CMAP_RANGE, 4596 },
	{ 0xf0c8, 0xf0cb, PDF_CMAP_RANGE, 6362 },
	{ 0xf0cc, 0xf0cc, PDF_CMAP_SINGLE, 4599 },
	{ 0xf0cd, 0xf0ce, PDF_CMAP_RANGE, 6366 },
	{ 0xf0cf, 0xf0d1, PDF_CMAP_RANGE, 4600 },
	{ 0xf0d2, 0xf0d2, PDF_CMAP_SINGLE, 5552 },
	{ 0xf0d3, 0xf0d7, PDF_CMAP_RANGE, 6368 },
	{ 0xf0d8, 0xf0d8, PDF_CMAP_SINGLE, 4603 },
	{ 0xf0d9, 0xf0fe, PDF_CMAP_RANGE, 6373 },
	{ 0xf131, 0xf135, PDF_CMAP_RANGE, 6411 },
	{ 0xf136, 0xf136, PDF_CMAP_SINGLE, 4614 },
	{ 0xf137, 0xf137, PDF_CMAP_SINGLE, 4616 },
	{ 0xf138, 0xf139, PDF_CMAP_RANGE, 6416 },
	{ 0xf13a, 0xf13a, PDF_CMAP_SINGLE, 4618 },
	{ 0xf13b, 0xf13e, PDF_CMAP_RANGE, 6418 },
	{ 0xf13f, 0xf13f, PDF_CMAP_SINGLE, 5950 },
	{ 0xf140, 0xf141, PDF_CMAP_RANGE, 4620 },
	{ 0xf142, 0xf142, PDF_CMAP_SINGLE, 4187 },
	{ 0xf143, 0xf144, PDF_CMAP_RANGE, 6422 },
	{ 0xf145, 0xf145, PDF_CMAP_SINGLE, 4625 },
	{ 0xf146, 0xf147, PDF_CMAP_RANGE, 6424 },
	{ 0xf148, 0xf148, PDF_CMAP_SINGLE, 4627 },
	{ 0xf149, 0xf149, PDF_CMAP_SINGLE, 6426 },
	{ 0xf14a, 0xf14a, PDF_CMAP_SINGLE, 4629 },
	{ 0xf14b, 0xf14f, PDF_CMAP_RANGE, 6427 },
	{ 0xf150, 0xf151, PDF_CMAP_RANGE, 4632 },
	{ 0xf152, 0xf154, PDF_CMAP_RANGE, 6432 },
	{ 0xf155, 0xf155, PDF_CMAP_SINGLE, 4634 },
	{ 0xf156, 0xf156, PDF_CMAP_SINGLE, 4636 },
	{ 0xf157, 0xf158, PDF_CMAP_RANGE, 6435 },
	{ 0xf159, 0xf15a, PDF_CMAP_RANGE, 4188 },
	{ 0xf15b, 0xf164, PDF_CMAP_RANGE, 6437 },
	{ 0xf165, 0xf165, PDF_CMAP_SINGLE, 4638 },
	{ 0xf166, 0xf16c, PDF_CMAP_RANGE, 6447 },
	{ 0xf16d, 0xf16e, PDF_CMAP_RANGE, 4640 },
	{ 0xf16f, 0xf170, PDF_CMAP_RANGE, 6454 },
	{ 0xf171, 0xf171, PDF_CMAP_SINGLE, 4642 },
	{ 0xf172, 0xf173, PDF_CMAP_RANGE, 6456 },
	{ 0xf174, 0xf174, PDF_CMAP_SINGLE, 4644 },
	{ 0xf175, 0xf176, PDF_CMAP_RANGE, 6458 },
	{ 0xf177, 0xf178, PDF_CMAP_RANGE, 4645 },
	{ 0xf179, 0xf17e, PDF_CMAP_RANGE, 6460 },
	{ 0xf191, 0xf198, PDF_CMAP_RANGE, 6466 },
	{ 0xf199, 0xf19a, PDF_CMAP_RANGE, 4647 },
	{ 0xf19b, 0xf19b, PDF_CMAP_SINGLE, 6474 },
	{ 0xf19c, 0xf19c, PDF_CMAP_SINGLE, 4650 },
	{ 0xf19d, 0xf1a0, PDF_CMAP_RANGE, 6475 },
	{ 0xf1a1, 0xf1a3, PDF_CMAP_RANGE, 4653 },
	{ 0xf1a4, 0xf1ed, PDF_CMAP_RANGE, 6479 },
	{ 0xf1ee, 0xf1ee, PDF_CMAP_SINGLE, 5351 },
	{ 0xf1ef, 0xf1fe, PDF_CMAP_RANGE, 6553 },
	{ 0xf231, 0xf26a, PDF_CMAP_RANGE, 6569 },
	{ 0xf26b, 0xf26b, PDF_CMAP_SINGLE, 6494 },
	{ 0xf26c, 0xf27e, PDF_CMAP_RANGE, 6627 },
	{ 0xf291, 0xf2fe, PDF_CMAP_RANGE, 6646 },
	{ 0xf331, 0xf37e, PDF_CMAP_RANGE, 6756 },
	{ 0xf391, 0xf3fe, PDF_CMAP_RANGE, 6834 },
	{ 0xf431, 0xf44c, PDF_CMAP_RANGE, 6944 },
	{ 0xf44d, 0xf44d, PDF_CMAP_SINGLE, 5731 },
	{ 0xf44e, 0xf47e, PDF_CMAP_RANGE, 6972 },
	{ 0xf491, 0xf49b, PDF_CMAP_RANGE, 7021 },
	{ 0xf49c, 0xf49c, PDF_CMAP_SINGLE, 5771 },
	{ 0xf49d, 0xf4b0, PDF_CMAP_RANGE, 7032 },
	{ 0xf4b1, 0xf4b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf4b2, 0xf4fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf531, 0xf536, PDF_CMAP_RANGE, 7129 },
	{ 0xf537, 0xf537, PDF_CMAP_SINGLE, 6484 },
	{ 0xf538, 0xf57d, PDF_CMAP_RANGE, 7135 },
	{ 0xf57e, 0xf57e, PDF_CMAP_SINGLE, 6684 },
	{ 0xf591, 0xf5fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf631, 0xf67e, PDF_CMAP_RANGE, 7315 },
	{ 0xf691, 0xf695, PDF_CMAP_RANGE, 7393 },
	{ 0xf696, 0xf696, PDF_CMAP_SINGLE, 4279 },
	{ 0xf697, 0xf697, PDF_CMAP_SINGLE, 7398 },
	{ 0xf698, 0xf698, PDF_CMAP_SINGLE, 7139 },
	{ 0xf699, 0xf6b7, PDF_CMAP_RANGE, 7399 },
	{ 0xf6b8, 0xf6b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf6b9, 0xf6c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf6c8, 0xf6c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf6c9, 0xf6d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf6d3, 0xf6d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf6d4, 0xf6fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf731, 0xf76a, PDF_CMAP_RANGE, 7498 },
	{ 0xf76b, 0xf76b, PDF_CMAP_SINGLE, 7572 },
	{ 0xf76c, 0xf77e, PDF_CMAP_RANGE, 7556 },
	{ 0xf791, 0xf791, PDF_CMAP_SINGLE, 7575 },
	{ 0xf792, 0xf792, PDF_CMAP_SINGLE, 5088 },
	{ 0xf793, 0xf7fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf831, 0xf831, PDF_CMAP_SINGLE, 7709 },
	{ 0xf832, 0xf832, PDF_CMAP_SINGLE, 3558 },
	{ 0xf833, 0xf875, PDF_CMAP_RANGE, 7684 },
	{ 0xf876, 0xf876, PDF_CMAP_SINGLE, 3644 },
	{ 0xf877, 0xf87e, PDF_CMAP_RANGE, 7751 },
	{ 0xf891, 0xf8fe, PDF_CMAP_RANGE, 7759 },
	{ 0xf931, 0xf938, PDF_CMAP_RANGE, 7869 },
	{ 0xf939, 0xf939, PDF_CMAP_SINGLE, 3815 },
	{ 0xf93a, 0xf97e, PDF_CMAP_RANGE, 7877 },
	{ 0xf991, 0xf9fe, PDF_CMAP_RANGE, 7946 },
};

static const unsigned short pdf_cmap_KSC_Johab_H_table[996] =
{
	1109,9355,1117,9377,1118,9413,1127,1129,
	9419,1141,9435,9442,1148,1152,9452,1160,
	9472,1161,9485,1167,9486,1168,1171,9489,
	1172,9490,1178,9501,1179,1181,9507,1182,
	9508,1183,1195,9549,1196,9550,1197,1201,
	9571,1202,1212,9588,1213,9589,1229,9655,
	1230,1241,9699,1242,9700,1243,9738,1250,
	1252,9744,1253,9745,1256,9749,9750,1260,
	1261,9754,1262,1264,9760,1274,9778,9838,
	1284,1287,9849,1295,9871,1296,1306,9929,
	1307,9930,1308,1310,9938,1311,9939,1336,
	10052,1337,10053,1359,10156,1360,10157,1361,
	10158,1362,1365,10163,1366,10164,1367,1374,
	10210,1375,10211,1376,10218,1380,1386,10226,
	10227,1392,10228,1393,1399,10241,1413,10302,
	1417,10307,1427,10325,1445,10382,1446,1448,
	10388,1449,10389,1450,1461,10459,1462,1467,
	10481,1468,10482,1469,1476,10500,1477,10501,
	1478,1500,10616,1501,10617,1504,10621,1511,
	10652,1512,1514,10658,1515,10659,1516,1517,
	10664,10665,1521,1529,10671,1541,10687,10749,
	1549,1555,10757,1556,10758,1557,10759,1558,
	1559,10762,1565,10775,10835,1581,10836,1582,
	1584,10840,1585,10841,1586,10842,1587,10843,
	1588,1598,10910,1599,1606,10958,1607,10959,
	1608,1616,11032,1617,11033,1618,11067,1629,
	1631,11073,1632,11074,1633,1641,11119,1651,
	11137,1661,11155,1668,11223,1672,11228,1682,
	11246,1718,11574,1719,1729,11619,1730,11620,
	1731,1737,11639,1749,11655,1756,11678,1757,
	1763,11725,1773,11743,1774,11744,1775,1781,
	11763,1787,11785,1788,1794,11806,1795,11807,
	1796,1805,11879,1806,11880,1807,1811,11901,
	1812,11902,1813,1819,11921,1820,11922,1821,
	1830,11996,1831,1837,12015,1838,12016,1839,
	1845,12035,1846,12036,1856,12080,1857,12081,
	1858,1861,12090,1868,12096,1869,12097,1872,
	12100,1873,1879,12113,1891,12176,1892,1894,
	12182,1895,12183,1904,12200,12222,1915,1921,
	12258,1922,1924,12264,1925,12265,1926,1934,
	12338,1935,12339,1936,1940,12360,1941,12369,
	1945,1951,12377,1952,12378,1953,1954,12382,
	1955,1959,12397,1960,12536,1980,1982,12542,
	12543,1986,12544,1987,1999,12553,2000,12554,
	2001,2008,12572,12634,2022,2024,12640,2025,
	12641,2034,12658,2042,12678,12710,2052,2055,
	12721,2056,12722,2057,2079,12837,2080,12838,
	2081,2097,12933,2098,2104,12952,2105,2109,
	12994,2110,2112,13000,2113,13001,2120,13011,
	2121,2123,13017,2133,13035,2154,13154,2173,
	13333,2174,2187,13513,2188,13514,2189,2199,
	13529,2209,13547,2218,13566,2219,13567,2220,
	2236,13604,2240,13609,2246,13622,2255,13641,
	13672,2266,2267,13676,2268,2270,13682,2271,
	13683,2272,2289,13747,2290,2296,13768,2297,
	13769,2298,2305,13787,2306,13788,2307,13789,
	2308,13790,2324,13852,2325,13853,2326,2331,
	13875,2332,2339,13893,2340,13894,2341,2349,
	13939,2350,13940,2351,2352,13945,2375,14044,
	2376,14142,2390,2422,14389,2423,2436,14440,
	2437,14441,2438,2449,14455,2460,14472,2470,
	14490,2471,14491,2472,2473,14495,2474,14528,
	2494,14529,2495,14530,2501,14543,2502,14544,
	2503,14551,2507,2508,14555,2524,14576,14589,
	2533,2536,14592,2537,14593,2538,14594,2539,
	2545,14611,2553,14633,2554,2560,14652,2561,
	14653,2562,2568,14672,2569,14673,2570,2578,
	14690,2579,14691,2580,2601,14751,2602,14752,
	2603,2609,14771,2610,14772,2611,14773,2612,
	2617,14788,2618,2619,14789,2620,14790,2641,
	14823,2646,14827,2649,14830,14831,2653,2655,
	14837,2665,14855,2671,14864,2672,14865,2673,
	2682,14913,2683,2685,14919,2686,14920,2694,
	14938,2695,14939,2696,2708,14999,2709,2711,
	15005,2712,15006,2720,15024,2721,15025,2722,
	2730,15070,2731,15071,2732,2744,15112,2745,
	15113,2746,2755,15185,2756,2766,15230,2767,
	15231,2768,15270,2774,2776,15276,2777,15277,
	2784,15283,2785,15284,2786,2789,15291,2798,
	15310,2810,15382,2823,15481,2824,15482,2825,
	15483,2826,2853,15733,2854,2866,15789,2867,
	15790,2868,2870,15798,2880,15816,2885,15826,
	2886,15827,2887,2895,15885,2904,15904,2905,
	15905,2906,2918,15974,2919,15975,2920,2929,
	16047,2930,16048,2931,2939,16093,2940,16094,
	2941,2950,16166,2951,16167,2952,2963,16209,
	2964,16210,2965,2973,16255,2974,16256,2975,
	2981,16275,2982,16276,2983,2989,16295,3002,
	16366,3011,16385,3012,16386,3013,3018,16406,
	3028,16452,3029,16453,3030,3047,16573,3048,
	16574,3049,3061,16643,3062,16644,3063,3080,
	16736,3081,16737,3082,3089,16755,3098,16774,
	3107,16840,3108,3110,16846,3119,16865,3120,
	16866,3121,3132,16936,3133,16937,3134,3135,
	16942,3141,17013,3142,3149,17059,3150,17060,
	3151,17168,3172,3174,17174,3175,3186,17218,
	3187,17219,3188,17226,3192,3193,17230,3194,
	3196,17236,3206,17254,3217,17327,3226,17346,
	3227,17347,3228,3238,17390,3239,3245,17411,
	3246,17412,3247,3255,17513,3256,17526,3262,
	3264,17532,3265,17533,3266,3278,17632,3279,
	3284,17652,3285,3291,17701,3292,17702,3293,
	3300,17720,3301,17721,3302,3308,17740,3317,
	17806,3318,3320,17812,3321,17813,3322,3328,
	17832,3329,17833,3330,3336,17852,3350,17894,
	3351,17895,3352,3358,17916,3359,3363,17939,
	3364,3369,17959,3370,17960,3371,3375,17981,
	3376,3383,18003,3384,3400,18068,3401,18069,
	3402,3408,18090,3409,3412,18099,3418,18106,
	3419,18107,3420,3433,18147,3434,18148,3435,
	3993,3946,4374,4156,4138,5800,4386,4139,
	4140,4391,4141,4142,4396,4403,4149,4419,
	4154,4420,4517,4173,4524,4174,4179,4547,
	4550,4180,4564,5584,5302,5367,4427,5879,
	4435,5896,4436,5897,4437,4159,5936,4444,
	4453,5946,5947,4458,4160,5961,4471,5980,
	4472,4474,4473,4475,5983,4476,5984,4481,
	5987,4482,5460,4483,4163,4166,5990,4485,
	4489,4491,6002,4167,4493,4495,6019,7783,
	4497,4499,4501,6027,4502,6028,4509,6046,
	4510,6146,4553,4180,4555,6152,5800,6153,
	4556,4581,6333,4582,4586,6339,4588,4184,
	6355,4592,4593,4595,4614,4616,4627,6426,
	4629,4634,4636,6474,4650,4279,7398,7139,
	7575,5088,7709,3558,
};

pdf_cmap pdf_cmap_KSC_Johab_H =
{
	-1, "KSC-Johab-H", "", nil, 0,
	4, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8441, 0xd3fe },
		{ 2, 0xd831, 0xdefe },
		{ 2, 0xe031, 0xf9fe },
	},
	4137, 4137, (pdf_range*) pdf_cmap_KSC_Johab_H_ranges,
	996, 996, (unsigned short*) pdf_cmap_KSC_Johab_H_table,
};

/* KSC-Johab-V */

static const pdf_range pdf_cmap_KSC_Johab_V_ranges[] =
{
	{ 0xd932, 0xd933, PDF_CMAP_RANGE, 8056 },
	{ 0xd935, 0xd936, PDF_CMAP_TABLE, 0 },
	{ 0xd939, 0xd93b, PDF_CMAP_RANGE, 8059 },
	{ 0xd93d, 0xd93d, PDF_CMAP_SINGLE, 8062 },
	{ 0xd942, 0xd94d, PDF_CMAP_RANGE, 8063 },
	{ 0xd97b, 0xd97b, PDF_CMAP_SINGLE, 8075 },
	{ 0xda31, 0xda31, PDF_CMAP_SINGLE, 8076 },
	{ 0xda38, 0xda39, PDF_CMAP_RANGE, 8077 },
	{ 0xda3c, 0xda3c, PDF_CMAP_SINGLE, 8079 },
	{ 0xda3e, 0xda3e, PDF_CMAP_SINGLE, 8080 },
	{ 0xda4a, 0xda4f, PDF_CMAP_RANGE, 8081 },
	{ 0xda6b, 0xda6b, PDF_CMAP_SINGLE, 8087 },
	{ 0xda6d, 0xda6d, PDF_CMAP_SINGLE, 8088 },
	{ 0xda6f, 0xda6f, PDF_CMAP_SINGLE, 8089 },
	{ 0xda9d, 0xdaa0, PDF_CMAP_RANGE, 8090 },
	{ 0xda9d, 0xdaa0, PDF_CMAP_RANGE, 8090 },
};

static const unsigned short pdf_cmap_KSC_Johab_V_table[2] =
{
	8058,8320,
};

pdf_cmap pdf_cmap_KSC_Johab_V =
{
	-1, "KSC-Johab-V", "KSC-Johab-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	16, 16, (pdf_range*) pdf_cmap_KSC_Johab_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_KSC_Johab_V_table,
};

/* KSC-V */

static const pdf_range pdf_cmap_KSC_V_ranges[] =
{
	{ 0x2122, 0x2123, PDF_CMAP_RANGE, 8056 },
	{ 0x2125, 0x2126, PDF_CMAP_TABLE, 0 },
	{ 0x2129, 0x212b, PDF_CMAP_RANGE, 8059 },
	{ 0x212d, 0x212d, PDF_CMAP_SINGLE, 8062 },
	{ 0x2132, 0x213d, PDF_CMAP_RANGE, 8063 },
	{ 0x216b, 0x216b, PDF_CMAP_SINGLE, 8075 },
	{ 0x2321, 0x2321, PDF_CMAP_SINGLE, 8076 },
	{ 0x2328, 0x2329, PDF_CMAP_RANGE, 8077 },
	{ 0x232c, 0x232c, PDF_CMAP_SINGLE, 8079 },
	{ 0x232e, 0x232e, PDF_CMAP_SINGLE, 8080 },
	{ 0x233a, 0x233f, PDF_CMAP_RANGE, 8081 },
	{ 0x235b, 0x235b, PDF_CMAP_SINGLE, 8087 },
	{ 0x235d, 0x235d, PDF_CMAP_SINGLE, 8088 },
	{ 0x235f, 0x235f, PDF_CMAP_SINGLE, 8089 },
	{ 0x237b, 0x237e, PDF_CMAP_RANGE, 8090 },
	{ 0x237b, 0x237e, PDF_CMAP_RANGE, 8090 },
};

static const unsigned short pdf_cmap_KSC_V_table[2] =
{
	8058,8320,
};

pdf_cmap pdf_cmap_KSC_V =
{
	-1, "KSC-V", "KSC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	16, 16, (pdf_range*) pdf_cmap_KSC_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_KSC_V_table,
};

/* KSCms-UHC-H */

static const pdf_range pdf_cmap_KSCms_UHC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x8141, 0x815a, PDF_CMAP_RANGE, 9333 },
	{ 0x8161, 0x817a, PDF_CMAP_RANGE, 9359 },
	{ 0x8181, 0x81fe, PDF_CMAP_RANGE, 9385 },
	{ 0x8241, 0x825a, PDF_CMAP_RANGE, 9511 },
	{ 0x8261, 0x827a, PDF_CMAP_RANGE, 9537 },
	{ 0x8281, 0x82fe, PDF_CMAP_RANGE, 9563 },
	{ 0x8341, 0x835a, PDF_CMAP_RANGE, 9689 },
	{ 0x8361, 0x837a, PDF_CMAP_RANGE, 9715 },
	{ 0x8381, 0x83fe, PDF_CMAP_RANGE, 9741 },
	{ 0x8441, 0x845a, PDF_CMAP_RANGE, 9867 },
	{ 0x8461, 0x847a, PDF_CMAP_RANGE, 9893 },
	{ 0x8481, 0x84fe, PDF_CMAP_RANGE, 9919 },
	{ 0x8541, 0x855a, PDF_CMAP_RANGE, 10045 },
	{ 0x8561, 0x857a, PDF_CMAP_RANGE, 10071 },
	{ 0x8581, 0x85fe, PDF_CMAP_RANGE, 10097 },
	{ 0x8641, 0x865a, PDF_CMAP_RANGE, 10223 },
	{ 0x8661, 0x867a, PDF_CMAP_RANGE, 10249 },
	{ 0x8681, 0x86fe, PDF_CMAP_RANGE, 10275 },
	{ 0x8741, 0x875a, PDF_CMAP_RANGE, 10401 },
	{ 0x8761, 0x877a, PDF_CMAP_RANGE, 10427 },
	{ 0x8781, 0x87fe, PDF_CMAP_RANGE, 10453 },
	{ 0x8841, 0x885a, PDF_CMAP_RANGE, 10579 },
	{ 0x8861, 0x887a, PDF_CMAP_RANGE, 10605 },
	{ 0x8881, 0x88fe, PDF_CMAP_RANGE, 10631 },
	{ 0x8941, 0x895a, PDF_CMAP_RANGE, 10757 },
	{ 0x8961, 0x897a, PDF_CMAP_RANGE, 10783 },
	{ 0x8981, 0x89fe, PDF_CMAP_RANGE, 10809 },
	{ 0x8a41, 0x8a5a, PDF_CMAP_RANGE, 10935 },
	{ 0x8a61, 0x8a7a, PDF_CMAP_RANGE, 10961 },
	{ 0x8a81, 0x8afe, PDF_CMAP_RANGE, 10987 },
	{ 0x8b41, 0x8b5a, PDF_CMAP_RANGE, 11113 },
	{ 0x8b61, 0x8b7a, PDF_CMAP_RANGE, 11139 },
	{ 0x8b81, 0x8bfe, PDF_CMAP_RANGE, 11165 },
	{ 0x8c41, 0x8c5a, PDF_CMAP_RANGE, 11291 },
	{ 0x8c61, 0x8c7a, PDF_CMAP_RANGE, 11317 },
	{ 0x8c81, 0x8cfe, PDF_CMAP_RANGE, 11343 },
	{ 0x8d41, 0x8d5a, PDF_CMAP_RANGE, 11469 },
	{ 0x8d61, 0x8d7a, PDF_CMAP_RANGE, 11495 },
	{ 0x8d81, 0x8dfe, PDF_CMAP_RANGE, 11521 },
	{ 0x8e41, 0x8e5a, PDF_CMAP_RANGE, 11647 },
	{ 0x8e61, 0x8e7a, PDF_CMAP_RANGE, 11673 },
	{ 0x8e81, 0x8efe, PDF_CMAP_RANGE, 11699 },
	{ 0x8f41, 0x8f5a, PDF_CMAP_RANGE, 11825 },
	{ 0x8f61, 0x8f7a, PDF_CMAP_RANGE, 11851 },
	{ 0x8f81, 0x8ffe, PDF_CMAP_RANGE, 11877 },
	{ 0x9041, 0x905a, PDF_CMAP_RANGE, 12003 },
	{ 0x9061, 0x907a, PDF_CMAP_RANGE, 12029 },
	{ 0x9081, 0x90fe, PDF_CMAP_RANGE, 12055 },
	{ 0x9141, 0x915a, PDF_CMAP_RANGE, 12181 },
	{ 0x9161, 0x917a, PDF_CMAP_RANGE, 12207 },
	{ 0x9181, 0x91fe, PDF_CMAP_RANGE, 12233 },
	{ 0x9241, 0x925a, PDF_CMAP_RANGE, 12359 },
	{ 0x9261, 0x927a, PDF_CMAP_RANGE, 12385 },
	{ 0x9281, 0x92fe, PDF_CMAP_RANGE, 12411 },
	{ 0x9341, 0x935a, PDF_CMAP_RANGE, 12537 },
	{ 0x9361, 0x937a, PDF_CMAP_RANGE, 12563 },
	{ 0x9381, 0x93fe, PDF_CMAP_RANGE, 12589 },
	{ 0x9441, 0x945a, PDF_CMAP_RANGE, 12715 },
	{ 0x9461, 0x947a, PDF_CMAP_RANGE, 12741 },
	{ 0x9481, 0x94fe, PDF_CMAP_RANGE, 12767 },
	{ 0x9541, 0x955a, PDF_CMAP_RANGE, 12893 },
	{ 0x9561, 0x957a, PDF_CMAP_RANGE, 12919 },
	{ 0x9581, 0x95fe, PDF_CMAP_RANGE, 12945 },
	{ 0x9641, 0x965a, PDF_CMAP_RANGE, 13071 },
	{ 0x9661, 0x967a, PDF_CMAP_RANGE, 13097 },
	{ 0x9681, 0x96fe, PDF_CMAP_RANGE, 13123 },
	{ 0x9741, 0x975a, PDF_CMAP_RANGE, 13249 },
	{ 0x9761, 0x977a, PDF_CMAP_RANGE, 13275 },
	{ 0x9781, 0x97fe, PDF_CMAP_RANGE, 13301 },
	{ 0x9841, 0x985a, PDF_CMAP_RANGE, 13427 },
	{ 0x9861, 0x987a, PDF_CMAP_RANGE, 13453 },
	{ 0x9881, 0x98fe, PDF_CMAP_RANGE, 13479 },
	{ 0x9941, 0x995a, PDF_CMAP_RANGE, 13605 },
	{ 0x9961, 0x997a, PDF_CMAP_RANGE, 13631 },
	{ 0x9981, 0x99fe, PDF_CMAP_RANGE, 13657 },
	{ 0x9a41, 0x9a5a, PDF_CMAP_RANGE, 13783 },
	{ 0x9a61, 0x9a7a, PDF_CMAP_RANGE, 13809 },
	{ 0x9a81, 0x9afe, PDF_CMAP_RANGE, 13835 },
	{ 0x9b41, 0x9b5a, PDF_CMAP_RANGE, 13961 },
	{ 0x9b61, 0x9b7a, PDF_CMAP_RANGE, 13987 },
	{ 0x9b81, 0x9bfe, PDF_CMAP_RANGE, 14013 },
	{ 0x9c41, 0x9c5a, PDF_CMAP_RANGE, 14139 },
	{ 0x9c61, 0x9c7a, PDF_CMAP_RANGE, 14165 },
	{ 0x9c81, 0x9cfe, PDF_CMAP_RANGE, 14191 },
	{ 0x9d41, 0x9d5a, PDF_CMAP_RANGE, 14317 },
	{ 0x9d61, 0x9d7a, PDF_CMAP_RANGE, 14343 },
	{ 0x9d81, 0x9dfe, PDF_CMAP_RANGE, 14369 },
	{ 0x9e41, 0x9e5a, PDF_CMAP_RANGE, 14495 },
	{ 0x9e61, 0x9e7a, PDF_CMAP_RANGE, 14521 },
	{ 0x9e81, 0x9efe, PDF_CMAP_RANGE, 14547 },
	{ 0x9f41, 0x9f5a, PDF_CMAP_RANGE, 14673 },
	{ 0x9f61, 0x9f7a, PDF_CMAP_RANGE, 14699 },
	{ 0x9f81, 0x9ffe, PDF_CMAP_RANGE, 14725 },
	{ 0xa041, 0xa05a, PDF_CMAP_RANGE, 14851 },
	{ 0xa061, 0xa07a, PDF_CMAP_RANGE, 14877 },
	{ 0xa081, 0xa0fe, PDF_CMAP_RANGE, 14903 },
	{ 0xa141, 0xa15a, PDF_CMAP_RANGE, 15029 },
	{ 0xa161, 0xa17a, PDF_CMAP_RANGE, 15055 },
	{ 0xa181, 0xa1a0, PDF_CMAP_RANGE, 15081 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 101 },
	{ 0xa241, 0xa25a, PDF_CMAP_RANGE, 15113 },
	{ 0xa261, 0xa27a, PDF_CMAP_RANGE, 15139 },
	{ 0xa281, 0xa2a0, PDF_CMAP_RANGE, 15165 },
	{ 0xa2a1, 0xa2e5, PDF_CMAP_RANGE, 195 },
	{ 0xa341, 0xa35a, PDF_CMAP_RANGE, 15197 },
	{ 0xa361, 0xa37a, PDF_CMAP_RANGE, 15223 },
	{ 0xa381, 0xa3a0, PDF_CMAP_RANGE, 15249 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 264 },
	{ 0xa441, 0xa45a, PDF_CMAP_RANGE, 15281 },
	{ 0xa461, 0xa47a, PDF_CMAP_RANGE, 15307 },
	{ 0xa481, 0xa4a0, PDF_CMAP_RANGE, 15333 },
	{ 0xa4a1, 0xa4d3, PDF_CMAP_RANGE, 358 },
	{ 0xa4d5, 0xa4fe, PDF_CMAP_RANGE, 409 },
	{ 0xa541, 0xa55a, PDF_CMAP_RANGE, 15365 },
	{ 0xa561, 0xa57a, PDF_CMAP_RANGE, 15391 },
	{ 0xa581, 0xa5a0, PDF_CMAP_RANGE, 15417 },
	{ 0xa5a1, 0xa5aa, PDF_CMAP_RANGE, 451 },
	{ 0xa5b0, 0xa5b9, PDF_CMAP_RANGE, 461 },
	{ 0xa5c1, 0xa5d8, PDF_CMAP_RANGE, 471 },
	{ 0xa5e1, 0xa5f8, PDF_CMAP_RANGE, 495 },
	{ 0xa641, 0xa65a, PDF_CMAP_RANGE, 15449 },
	{ 0xa661, 0xa67a, PDF_CMAP_RANGE, 15475 },
	{ 0xa681, 0xa6a0, PDF_CMAP_RANGE, 15501 },
	{ 0xa6a1, 0xa6e4, PDF_CMAP_RANGE, 519 },
	{ 0xa741, 0xa75a, PDF_CMAP_RANGE, 15533 },
	{ 0xa761, 0xa77a, PDF_CMAP_RANGE, 15559 },
	{ 0xa781, 0xa7a0, PDF_CMAP_RANGE, 15585 },
	{ 0xa7a1, 0xa7ef, PDF_CMAP_RANGE, 587 },
	{ 0xa841, 0xa85a, PDF_CMAP_RANGE, 15617 },
	{ 0xa861, 0xa87a, PDF_CMAP_RANGE, 15643 },
	{ 0xa881, 0xa8a0, PDF_CMAP_RANGE, 15669 },
	{ 0xa8a1, 0xa8a4, PDF_CMAP_RANGE, 666 },
	{ 0xa8a6, 0xa8a6, PDF_CMAP_SINGLE, 670 },
	{ 0xa8a8, 0xa8af, PDF_CMAP_RANGE, 671 },
	{ 0xa8b1, 0xa8fe, PDF_CMAP_RANGE, 679 },
	{ 0xa941, 0xa95a, PDF_CMAP_RANGE, 15701 },
	{ 0xa961, 0xa97a, PDF_CMAP_RANGE, 15727 },
	{ 0xa981, 0xa9a0, PDF_CMAP_RANGE, 15753 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 757 },
	{ 0xaa41, 0xaa5a, PDF_CMAP_RANGE, 15785 },
	{ 0xaa61, 0xaa7a, PDF_CMAP_RANGE, 15811 },
	{ 0xaa81, 0xaaa0, PDF_CMAP_RANGE, 15837 },
	{ 0xaaa1, 0xaaf3, PDF_CMAP_RANGE, 851 },
	{ 0xab41, 0xab5a, PDF_CMAP_RANGE, 15869 },
	{ 0xab61, 0xab7a, PDF_CMAP_RANGE, 15895 },
	{ 0xab81, 0xaba0, PDF_CMAP_RANGE, 15921 },
	{ 0xaba1, 0xabf6, PDF_CMAP_RANGE, 934 },
	{ 0xac41, 0xac5a, PDF_CMAP_RANGE, 15953 },
	{ 0xac61, 0xac7a, PDF_CMAP_RANGE, 15979 },
	{ 0xac81, 0xaca0, PDF_CMAP_RANGE, 16005 },
	{ 0xaca1, 0xacc1, PDF_CMAP_RANGE, 1020 },
	{ 0xacd1, 0xacf1, PDF_CMAP_RANGE, 1053 },
	{ 0xad41, 0xad5a, PDF_CMAP_RANGE, 16037 },
	{ 0xad61, 0xad7a, PDF_CMAP_RANGE, 16063 },
	{ 0xad81, 0xada0, PDF_CMAP_RANGE, 16089 },
	{ 0xae41, 0xae5a, PDF_CMAP_RANGE, 16121 },
	{ 0xae61, 0xae7a, PDF_CMAP_RANGE, 16147 },
	{ 0xae81, 0xaea0, PDF_CMAP_RANGE, 16173 },
	{ 0xaf41, 0xaf5a, PDF_CMAP_RANGE, 16205 },
	{ 0xaf61, 0xaf7a, PDF_CMAP_RANGE, 16231 },
	{ 0xaf81, 0xafa0, PDF_CMAP_RANGE, 16257 },
	{ 0xb041, 0xb05a, PDF_CMAP_RANGE, 16289 },
	{ 0xb061, 0xb07a, PDF_CMAP_RANGE, 16315 },
	{ 0xb081, 0xb0a0, PDF_CMAP_RANGE, 16341 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 1086 },
	{ 0xb141, 0xb15a, PDF_CMAP_RANGE, 16373 },
	{ 0xb161, 0xb17a, PDF_CMAP_RANGE, 16399 },
	{ 0xb181, 0xb1a0, PDF_CMAP_RANGE, 16425 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1180 },
	{ 0xb241, 0xb25a, PDF_CMAP_RANGE, 16457 },
	{ 0xb261, 0xb27a, PDF_CMAP_RANGE, 16483 },
	{ 0xb281, 0xb2a0, PDF_CMAP_RANGE, 16509 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1274 },
	{ 0xb341, 0xb35a, PDF_CMAP_RANGE, 16541 },
	{ 0xb361, 0xb37a, PDF_CMAP_RANGE, 16567 },
	{ 0xb381, 0xb3a0, PDF_CMAP_RANGE, 16593 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1368 },
	{ 0xb441, 0xb45a, PDF_CMAP_RANGE, 16625 },
	{ 0xb461, 0xb47a, PDF_CMAP_RANGE, 16651 },
	{ 0xb481, 0xb4a0, PDF_CMAP_RANGE, 16677 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1462 },
	{ 0xb541, 0xb55a, PDF_CMAP_RANGE, 16709 },
	{ 0xb561, 0xb57a, PDF_CMAP_RANGE, 16735 },
	{ 0xb581, 0xb5a0, PDF_CMAP_RANGE, 16761 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1556 },
	{ 0xb641, 0xb65a, PDF_CMAP_RANGE, 16793 },
	{ 0xb661, 0xb67a, PDF_CMAP_RANGE, 16819 },
	{ 0xb681, 0xb6a0, PDF_CMAP_RANGE, 16845 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1650 },
	{ 0xb741, 0xb75a, PDF_CMAP_RANGE, 16877 },
	{ 0xb761, 0xb77a, PDF_CMAP_RANGE, 16903 },
	{ 0xb781, 0xb7a0, PDF_CMAP_RANGE, 16929 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1744 },
	{ 0xb841, 0xb85a, PDF_CMAP_RANGE, 16961 },
	{ 0xb861, 0xb87a, PDF_CMAP_RANGE, 16987 },
	{ 0xb881, 0xb8a0, PDF_CMAP_RANGE, 17013 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1838 },
	{ 0xb941, 0xb95a, PDF_CMAP_RANGE, 17045 },
	{ 0xb961, 0xb97a, PDF_CMAP_RANGE, 17071 },
	{ 0xb981, 0xb9a0, PDF_CMAP_RANGE, 17097 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1932 },
	{ 0xba41, 0xba5a, PDF_CMAP_RANGE, 17129 },
	{ 0xba61, 0xba7a, PDF_CMAP_RANGE, 17155 },
	{ 0xba81, 0xbaa0, PDF_CMAP_RANGE, 17181 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 2026 },
	{ 0xbb41, 0xbb5a, PDF_CMAP_RANGE, 17213 },
	{ 0xbb61, 0xbb7a, PDF_CMAP_RANGE, 17239 },
	{ 0xbb81, 0xbba0, PDF_CMAP_RANGE, 17265 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 2120 },
	{ 0xbc41, 0xbc5a, PDF_CMAP_RANGE, 17297 },
	{ 0xbc61, 0xbc7a, PDF_CMAP_RANGE, 17323 },
	{ 0xbc81, 0xbca0, PDF_CMAP_RANGE, 17349 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2214 },
	{ 0xbd41, 0xbd5a, PDF_CMAP_RANGE, 17381 },
	{ 0xbd61, 0xbd7a, PDF_CMAP_RANGE, 17407 },
	{ 0xbd81, 0xbda0, PDF_CMAP_RANGE, 17433 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2308 },
	{ 0xbe41, 0xbe5a, PDF_CMAP_RANGE, 17465 },
	{ 0xbe61, 0xbe7a, PDF_CMAP_RANGE, 17491 },
	{ 0xbe81, 0xbea0, PDF_CMAP_RANGE, 17517 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2402 },
	{ 0xbf41, 0xbf5a, PDF_CMAP_RANGE, 17549 },
	{ 0xbf61, 0xbf7a, PDF_CMAP_RANGE, 17575 },
	{ 0xbf81, 0xbfa0, PDF_CMAP_RANGE, 17601 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2496 },
	{ 0xc041, 0xc05a, PDF_CMAP_RANGE, 17633 },
	{ 0xc061, 0xc07a, PDF_CMAP_RANGE, 17659 },
	{ 0xc081, 0xc0a0, PDF_CMAP_RANGE, 17685 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2590 },
	{ 0xc141, 0xc15a, PDF_CMAP_RANGE, 17717 },
	{ 0xc161, 0xc17a, PDF_CMAP_RANGE, 17743 },
	{ 0xc181, 0xc1a0, PDF_CMAP_RANGE, 17769 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2684 },
	{ 0xc241, 0xc25a, PDF_CMAP_RANGE, 17801 },
	{ 0xc261, 0xc27a, PDF_CMAP_RANGE, 17827 },
	{ 0xc281, 0xc2a0, PDF_CMAP_RANGE, 17853 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2778 },
	{ 0xc341, 0xc35a, PDF_CMAP_RANGE, 17885 },
	{ 0xc361, 0xc37a, PDF_CMAP_RANGE, 17911 },
	{ 0xc381, 0xc3a0, PDF_CMAP_RANGE, 17937 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2872 },
	{ 0xc441, 0xc45a, PDF_CMAP_RANGE, 17969 },
	{ 0xc461, 0xc47a, PDF_CMAP_RANGE, 17995 },
	{ 0xc481, 0xc4a0, PDF_CMAP_RANGE, 18021 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2966 },
	{ 0xc541, 0xc55a, PDF_CMAP_RANGE, 18053 },
	{ 0xc561, 0xc57a, PDF_CMAP_RANGE, 18079 },
	{ 0xc581, 0xc5a0, PDF_CMAP_RANGE, 18105 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 3060 },
	{ 0xc641, 0xc652, PDF_CMAP_RANGE, 18137 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3154 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3248 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3342 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3436 },
	{ 0xcba1, 0xcbcf, PDF_CMAP_RANGE, 3530 },
	{ 0xcbd0, 0xcbd0, PDF_CMAP_SINGLE, 4116 },
	{ 0xcbd1, 0xcbd5, PDF_CMAP_RANGE, 3577 },
	{ 0xcbd6, 0xcbd6, PDF_CMAP_SINGLE, 3678 },
	{ 0xcbd7, 0xcbe6, PDF_CMAP_RANGE, 3582 },
	{ 0xcbe7, 0xcbe7, PDF_CMAP_SINGLE, 7053 },
	{ 0xcbe8, 0xcbfe, PDF_CMAP_RANGE, 3598 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3621 },
	{ 0xcda1, 0xcdce, PDF_CMAP_RANGE, 3715 },
	{ 0xcdcf, 0xcdcf, PDF_CMAP_SINGLE, 3460 },
	{ 0xcdd0, 0xcde7, PDF_CMAP_RANGE, 3761 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 7900 },
	{ 0xcde9, 0xcdfe, PDF_CMAP_RANGE, 3785 },
	{ 0xcea1, 0xceac, PDF_CMAP_RANGE, 3807 },
	{ 0xcead, 0xcead, PDF_CMAP_SINGLE, 3802 },
	{ 0xceae, 0xcefe, PDF_CMAP_RANGE, 3819 },
	{ 0xcfa1, 0xcffa, PDF_CMAP_RANGE, 3900 },
	{ 0xcffb, 0xcffb, PDF_CMAP_SINGLE, 3902 },
	{ 0xcffc, 0xcffe, PDF_CMAP_RANGE, 3990 },
	{ 0xd0a1, 0xd0a2, PDF_CMAP_TABLE, 0 },
	{ 0xd0a3, 0xd0b7, PDF_CMAP_RANGE, 3994 },
	{ 0xd0b8, 0xd0b8, PDF_CMAP_SINGLE, 3946 },
	{ 0xd0b9, 0xd0cf, PDF_CMAP_RANGE, 4015 },
	{ 0xd0d0, 0xd0d0, PDF_CMAP_SINGLE, 3708 },
	{ 0xd0d1, 0xd0dc, PDF_CMAP_RANGE, 4038 },
	{ 0xd0dd, 0xd0dd, PDF_CMAP_SINGLE, 4131 },
	{ 0xd0de, 0xd0fe, PDF_CMAP_RANGE, 4050 },
	{ 0xd1a1, 0xd1d3, PDF_CMAP_RANGE, 4083 },
	{ 0xd1d4, 0xd1d5, PDF_CMAP_TABLE, 2 },
	{ 0xd1d6, 0xd1d7, PDF_CMAP_RANGE, 4134 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 4375 },
	{ 0xd1d9, 0xd1da, PDF_CMAP_RANGE, 4136 },
	{ 0xd1db, 0xd1e0, PDF_CMAP_RANGE, 4376 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_TABLE, 4 },
	{ 0xd1e3, 0xd1e5, PDF_CMAP_RANGE, 4382 },
	{ 0xd1e6, 0xd1e7, PDF_CMAP_TABLE, 6 },
	{ 0xd1e8, 0xd1eb, PDF_CMAP_RANGE, 4387 },
	{ 0xd1ec, 0xd1ee, PDF_CMAP_TABLE, 8 },
	{ 0xd1ef, 0xd1f0, PDF_CMAP_RANGE, 4394 },
	{ 0xd1f1, 0xd1f2, PDF_CMAP_TABLE, 11 },
	{ 0xd1f3, 0xd1f5, PDF_CMAP_RANGE, 4143 },
	{ 0xd1f6, 0xd1f6, PDF_CMAP_SINGLE, 4399 },
	{ 0xd1f7, 0xd1f9, PDF_CMAP_RANGE, 4146 },
	{ 0xd1fa, 0xd1fb, PDF_CMAP_TABLE, 13 },
	{ 0xd1fc, 0xd1fd, PDF_CMAP_RANGE, 4406 },
	{ 0xd1fe, 0xd1fe, PDF_CMAP_SINGLE, 4409 },
	{ 0xd2a1, 0xd2a1, PDF_CMAP_SINGLE, 4150 },
	{ 0xd2a2, 0xd2a3, PDF_CMAP_RANGE, 4410 },
	{ 0xd2a4, 0xd2a6, PDF_CMAP_RANGE, 4151 },
	{ 0xd2a7, 0xd2aa, PDF_CMAP_RANGE, 4412 },
	{ 0xd2ab, 0xd2ad, PDF_CMAP_TABLE, 15 },
	{ 0xd2ae, 0xd2b1, PDF_CMAP_RANGE, 4155 },
	{ 0xd2b2, 0xd2b2, PDF_CMAP_SINGLE, 4424 },
	{ 0xd2b3, 0xd2bd, PDF_CMAP_RANGE, 4159 },
	{ 0xd2be, 0xd2be, PDF_CMAP_SINGLE, 4511 },
	{ 0xd2bf, 0xd2c1, PDF_CMAP_RANGE, 4170 },
	{ 0xd2c2, 0xd2c3, PDF_CMAP_RANGE, 4513 },
	{ 0xd2c4, 0xd2c5, PDF_CMAP_TABLE, 18 },
	{ 0xd2c6, 0xd2ca, PDF_CMAP_RANGE, 4518 },
	{ 0xd2cb, 0xd2cc, PDF_CMAP_TABLE, 20 },
	{ 0xd2cd, 0xd2ce, PDF_CMAP_RANGE, 4525 },
	{ 0xd2cf, 0xd2d4, PDF_CMAP_RANGE, 4528 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 4535 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 4175 },
	{ 0xd2d9, 0xd2da, PDF_CMAP_RANGE, 4541 },
	{ 0xd2db, 0xd2dd, PDF_CMAP_RANGE, 4176 },
	{ 0xd2de, 0xd2df, PDF_CMAP_RANGE, 4545 },
	{ 0xd2e0, 0xd2e4, PDF_CMAP_TABLE, 22 },
	{ 0xd2e5, 0xd2e8, PDF_CMAP_RANGE, 4566 },
	{ 0xd2e9, 0xd2ea, PDF_CMAP_RANGE, 4571 },
	{ 0xd2eb, 0xd2eb, PDF_CMAP_SINGLE, 4576 },
	{ 0xd2ec, 0xd2ef, PDF_CMAP_RANGE, 4181 },
	{ 0xd2f0, 0xd2f3, PDF_CMAP_RANGE, 4604 },
	{ 0xd2f4, 0xd2f5, PDF_CMAP_RANGE, 4609 },
	{ 0xd2f6, 0xd2f6, PDF_CMAP_SINGLE, 4185 },
	{ 0xd2f7, 0xd2f8, PDF_CMAP_RANGE, 4611 },
	{ 0xd2f9, 0xd2fe, PDF_CMAP_RANGE, 4186 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4192 },
	{ 0xd4a1, 0xd4e5, PDF_CMAP_RANGE, 4286 },
	{ 0xd4e6, 0xd4e6, PDF_CMAP_SINGLE, 4318 },
	{ 0xd4e7, 0xd4fb, PDF_CMAP_RANGE, 4355 },
	{ 0xd4fc, 0xd4fc, PDF_CMAP_SINGLE, 4136 },
	{ 0xd4fd, 0xd4fe, PDF_CMAP_RANGE, 4376 },
	{ 0xd5a1, 0xd5a4, PDF_CMAP_RANGE, 4378 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 5800 },
	{ 0xd5a6, 0xd5aa, PDF_CMAP_RANGE, 4382 },
	{ 0xd5ab, 0xd5ab, PDF_CMAP_SINGLE, 4139 },
	{ 0xd5ac, 0xd5ad, PDF_CMAP_RANGE, 4387 },
	{ 0xd5ae, 0xd5ae, PDF_CMAP_SINGLE, 4192 },
	{ 0xd5af, 0xd5fe, PDF_CMAP_RANGE, 4389 },
	{ 0xd6a1, 0xd6b7, PDF_CMAP_RANGE, 4469 },
	{ 0xd6b8, 0xd6b8, PDF_CMAP_SINGLE, 4167 },
	{ 0xd6b9, 0xd6cc, PDF_CMAP_RANGE, 4492 },
	{ 0xd6cd, 0xd6cd, PDF_CMAP_SINGLE, 4172 },
	{ 0xd6ce, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd7a1, 0xd7ca, PDF_CMAP_RANGE, 4561 },
	{ 0xd7cb, 0xd7cb, PDF_CMAP_SINGLE, 5552 },
	{ 0xd7cc, 0xd7e3, PDF_CMAP_RANGE, 4603 },
	{ 0xd7e4, 0xd7e4, PDF_CMAP_SINGLE, 6424 },
	{ 0xd7e5, 0xd7fe, PDF_CMAP_RANGE, 4627 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4653 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4747 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4841 },
	{ 0xdba1, 0xdbc4, PDF_CMAP_RANGE, 4935 },
	{ 0xdbc5, 0xdbc5, PDF_CMAP_SINGLE, 5151 },
	{ 0xdbc6, 0xdbe3, PDF_CMAP_RANGE, 4971 },
	{ 0xdbe4, 0xdbe4, PDF_CMAP_SINGLE, 4922 },
	{ 0xdbe5, 0xdbfe, PDF_CMAP_RANGE, 5001 },
	{ 0xdca1, 0xdca4, PDF_CMAP_RANGE, 5027 },
	{ 0xdca5, 0xdca5, PDF_CMAP_SINGLE, 7518 },
	{ 0xdca6, 0xdcfe, PDF_CMAP_RANGE, 5031 },
	{ 0xdda1, 0xdda4, PDF_CMAP_RANGE, 5120 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 5079 },
	{ 0xdda6, 0xddd4, PDF_CMAP_RANGE, 5124 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 5109 },
	{ 0xddd6, 0xddf3, PDF_CMAP_RANGE, 5171 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 7607 },
	{ 0xddf5, 0xddfe, PDF_CMAP_RANGE, 5201 },
	{ 0xdea1, 0xdefb, PDF_CMAP_RANGE, 5211 },
	{ 0xdefc, 0xdefe, PDF_CMAP_TABLE, 27 },
	{ 0xdfa1, 0xdfb2, PDF_CMAP_RANGE, 5303 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 7083 },
	{ 0xdfb4, 0xdfe0, PDF_CMAP_RANGE, 5321 },
	{ 0xdfe1, 0xdfe1, PDF_CMAP_SINGLE, 5362 },
	{ 0xdfe2, 0xdfe7, PDF_CMAP_RANGE, 5366 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 5485 },
	{ 0xdfe9, 0xdffe, PDF_CMAP_RANGE, 5372 },
	{ 0xe0a1, 0xe0f0, PDF_CMAP_RANGE, 5394 },
	{ 0xe0f1, 0xe0f1, PDF_CMAP_SINGLE, 6001 },
	{ 0xe0f2, 0xe0fe, PDF_CMAP_RANGE, 5474 },
	{ 0xe1a1, 0xe1ac, PDF_CMAP_RANGE, 5487 },
	{ 0xe1ad, 0xe1ad, PDF_CMAP_SINGLE, 5460 },
	{ 0xe1ae, 0xe1ec, PDF_CMAP_RANGE, 5499 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 5317 },
	{ 0xe1ee, 0xe1fe, PDF_CMAP_RANGE, 5562 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5579 },
	{ 0xe3a1, 0xe3f4, PDF_CMAP_RANGE, 5673 },
	{ 0xe3f5, 0xe3f5, PDF_CMAP_SINGLE, 7009 },
	{ 0xe3f6, 0xe3fe, PDF_CMAP_RANGE, 5757 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 7370 },
	{ 0xe4a2, 0xe4a8, PDF_CMAP_RANGE, 5766 },
	{ 0xe4a9, 0xe4a9, PDF_CMAP_SINGLE, 5678 },
	{ 0xe4aa, 0xe4fe, PDF_CMAP_RANGE, 5773 },
	{ 0xe5a1, 0xe5ad, PDF_CMAP_RANGE, 5858 },
	{ 0xe5ae, 0xe5ae, PDF_CMAP_SINGLE, 5874 },
	{ 0xe5af, 0xe5b0, PDF_CMAP_RANGE, 5871 },
	{ 0xe5b1, 0xe5b2, PDF_CMAP_RANGE, 4425 },
	{ 0xe5b3, 0xe5b8, PDF_CMAP_RANGE, 5873 },
	{ 0xe5b9, 0xe5ba, PDF_CMAP_TABLE, 30 },
	{ 0xe5bb, 0xe5bc, PDF_CMAP_RANGE, 4429 },
	{ 0xe5bd, 0xe5c3, PDF_CMAP_RANGE, 5880 },
	{ 0xe5c4, 0xe5c4, PDF_CMAP_SINGLE, 4431 },
	{ 0xe5c5, 0xe5cd, PDF_CMAP_RANGE, 5887 },
	{ 0xe5ce, 0xe5d2, PDF_CMAP_TABLE, 32 },
	{ 0xe5d3, 0xe5d5, PDF_CMAP_RANGE, 5898 },
	{ 0xe5d6, 0xe5d6, PDF_CMAP_SINGLE, 4439 },
	{ 0xe5d7, 0xe5f9, PDF_CMAP_RANGE, 5901 },
	{ 0xe5fa, 0xe5fb, PDF_CMAP_RANGE, 4442 },
	{ 0xe5fc, 0xe5fe, PDF_CMAP_TABLE, 37 },
	{ 0xe6a1, 0xe6a1, PDF_CMAP_SINGLE, 4447 },
	{ 0xe6a2, 0xe6a3, PDF_CMAP_RANGE, 5937 },
	{ 0xe6a4, 0xe6a4, PDF_CMAP_SINGLE, 4449 },
	{ 0xe6a5, 0xe6a6, PDF_CMAP_RANGE, 5939 },
	{ 0xe6a7, 0xe6a7, PDF_CMAP_SINGLE, 4450 },
	{ 0xe6a8, 0xe6ac, PDF_CMAP_RANGE, 5941 },
	{ 0xe6ad, 0xe6ae, PDF_CMAP_TABLE, 40 },
	{ 0xe6af, 0xe6b1, PDF_CMAP_RANGE, 4455 },
	{ 0xe6b2, 0xe6b3, PDF_CMAP_TABLE, 42 },
	{ 0xe6b4, 0xe6b6, PDF_CMAP_RANGE, 5948 },
	{ 0xe6b7, 0xe6b8, PDF_CMAP_RANGE, 4459 },
	{ 0xe6b9, 0xe6bb, PDF_CMAP_RANGE, 5951 },
	{ 0xe6bc, 0xe6bc, PDF_CMAP_SINGLE, 4463 },
	{ 0xe6bd, 0xe6c3, PDF_CMAP_RANGE, 5954 },
	{ 0xe6c4, 0xe6c5, PDF_CMAP_TABLE, 44 },
	{ 0xe6c6, 0xe6c7, PDF_CMAP_RANGE, 4465 },
	{ 0xe6c8, 0xe6c9, PDF_CMAP_RANGE, 5962 },
	{ 0xe6ca, 0xe6ca, PDF_CMAP_SINGLE, 4161 },
	{ 0xe6cb, 0xe6d1, PDF_CMAP_RANGE, 5964 },
	{ 0xe6d2, 0xe6d2, PDF_CMAP_SINGLE, 4468 },
	{ 0xe6d3, 0xe6d5, PDF_CMAP_RANGE, 5971 },
	{ 0xe6d6, 0xe6d6, PDF_CMAP_SINGLE, 4469 },
	{ 0xe6d7, 0xe6d8, PDF_CMAP_RANGE, 5974 },
	{ 0xe6d9, 0xe6d9, PDF_CMAP_SINGLE, 4470 },
	{ 0xe6da, 0xe6db, PDF_CMAP_RANGE, 5976 },
	{ 0xe6dc, 0xe6dc, PDF_CMAP_SINGLE, 4162 },
	{ 0xe6dd, 0xe6de, PDF_CMAP_RANGE, 5978 },
	{ 0xe6df, 0xe6e1, PDF_CMAP_TABLE, 46 },
	{ 0xe6e2, 0xe6e3, PDF_CMAP_RANGE, 5981 },
	{ 0xe6e4, 0xe6e9, PDF_CMAP_TABLE, 49 },
	{ 0xe6ea, 0xe6eb, PDF_CMAP_RANGE, 4478 },
	{ 0xe6ec, 0xe6ec, PDF_CMAP_SINGLE, 6447 },
	{ 0xe6ed, 0xe6ee, PDF_CMAP_RANGE, 5985 },
	{ 0xe6ef, 0xe6f2, PDF_CMAP_TABLE, 55 },
	{ 0xe6f3, 0xe6f4, PDF_CMAP_RANGE, 5988 },
	{ 0xe6f5, 0xe6f9, PDF_CMAP_TABLE, 59 },
	{ 0xe6fa, 0xe6fe, PDF_CMAP_RANGE, 5991 },
	{ 0xe7a1, 0xe7a1, PDF_CMAP_SINGLE, 4487 },
	{ 0xe7a2, 0xe7a5, PDF_CMAP_RANGE, 5996 },
	{ 0xe7a6, 0xe7a6, PDF_CMAP_SINGLE, 4488 },
	{ 0xe7a7, 0xe7a8, PDF_CMAP_RANGE, 6000 },
	{ 0xe7a9, 0xe7ad, PDF_CMAP_TABLE, 64 },
	{ 0xe7ae, 0xe7af, PDF_CMAP_RANGE, 6003 },
	{ 0xe7b0, 0xe7b0, PDF_CMAP_SINGLE, 4494 },
	{ 0xe7b1, 0xe7be, PDF_CMAP_RANGE, 6005 },
	{ 0xe7bf, 0xe7c1, PDF_CMAP_TABLE, 69 },
	{ 0xe7c2, 0xe7c5, PDF_CMAP_RANGE, 6020 },
	{ 0xe7c6, 0xe7c7, PDF_CMAP_TABLE, 72 },
	{ 0xe7c8, 0xe7ca, PDF_CMAP_RANGE, 6024 },
	{ 0xe7cb, 0xe7ce, PDF_CMAP_TABLE, 74 },
	{ 0xe7cf, 0xe7d0, PDF_CMAP_RANGE, 4503 },
	{ 0xe7d1, 0xe7d2, PDF_CMAP_RANGE, 6029 },
	{ 0xe7d3, 0xe7d3, PDF_CMAP_SINGLE, 4506 },
	{ 0xe7d4, 0xe7de, PDF_CMAP_RANGE, 6031 },
	{ 0xe7df, 0xe7df, PDF_CMAP_SINGLE, 4508 },
	{ 0xe7e0, 0xe7e3, PDF_CMAP_RANGE, 6042 },
	{ 0xe7e4, 0xe7e6, PDF_CMAP_TABLE, 78 },
	{ 0xe7e7, 0xe7f6, PDF_CMAP_RANGE, 6047 },
	{ 0xe7f7, 0xe7f7, PDF_CMAP_SINGLE, 5797 },
	{ 0xe7f8, 0xe7fe, PDF_CMAP_RANGE, 6063 },
	{ 0xe8a1, 0xe8e6, PDF_CMAP_RANGE, 6070 },
	{ 0xe8e7, 0xe8e8, PDF_CMAP_RANGE, 4551 },
	{ 0xe8e9, 0xe8ef, PDF_CMAP_RANGE, 6140 },
	{ 0xe8f0, 0xe8f1, PDF_CMAP_TABLE, 81 },
	{ 0xe8f2, 0xe8f6, PDF_CMAP_RANGE, 6147 },
	{ 0xe8f7, 0xe8fb, PDF_CMAP_TABLE, 83 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 6154 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 4557 },
	{ 0xe9a1, 0xe9a6, PDF_CMAP_RANGE, 6156 },
	{ 0xe9a7, 0xe9a7, PDF_CMAP_SINGLE, 4560 },
	{ 0xe9a8, 0xe9ab, PDF_CMAP_RANGE, 6162 },
	{ 0xe9ac, 0xe9ac, PDF_CMAP_SINGLE, 4561 },
	{ 0xe9ad, 0xe9cb, PDF_CMAP_RANGE, 6166 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 4563 },
	{ 0xe9cd, 0xe9f6, PDF_CMAP_RANGE, 6197 },
	{ 0xe9f7, 0xe9f7, PDF_CMAP_SINGLE, 7988 },
	{ 0xe9f8, 0xe9fe, PDF_CMAP_RANGE, 6239 },
	{ 0xeaa1, 0xeac0, PDF_CMAP_RANGE, 6246 },
	{ 0xeac1, 0xeac1, PDF_CMAP_SINGLE, 6123 },
	{ 0xeac2, 0xeae4, PDF_CMAP_RANGE, 6278 },
	{ 0xeae5, 0xeae5, PDF_CMAP_SINGLE, 4577 },
	{ 0xeae6, 0xeaf3, PDF_CMAP_RANGE, 6313 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 4183 },
	{ 0xeaf5, 0xeaf6, PDF_CMAP_RANGE, 6327 },
	{ 0xeaf7, 0xeaf7, PDF_CMAP_SINGLE, 4579 },
	{ 0xeaf8, 0xeafb, PDF_CMAP_RANGE, 6329 },
	{ 0xeafc, 0xeafe, PDF_CMAP_TABLE, 88 },
	{ 0xeba1, 0xeba3, PDF_CMAP_RANGE, 6334 },
	{ 0xeba4, 0xeba4, PDF_CMAP_SINGLE, 4584 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 6337 },
	{ 0xeba7, 0xebaa, PDF_CMAP_TABLE, 91 },
	{ 0xebab, 0xebb9, PDF_CMAP_RANGE, 6340 },
	{ 0xebba, 0xebbb, PDF_CMAP_RANGE, 4590 },
	{ 0xebbc, 0xebbd, PDF_CMAP_TABLE, 95 },
	{ 0xebbe, 0xebc0, PDF_CMAP_RANGE, 6356 },
	{ 0xebc1, 0xebc2, PDF_CMAP_TABLE, 97 },
	{ 0xebc3, 0xebc5, PDF_CMAP_RANGE, 6359 },
	{ 0xebc6, 0xebc7, PDF_CMAP_RANGE, 4596 },
	{ 0xebc8, 0xebcb, PDF_CMAP_RANGE, 6362 },
	{ 0xebcc, 0xebcc, PDF_CMAP_SINGLE, 4599 },
	{ 0xebcd, 0xebce, PDF_CMAP_RANGE, 6366 },
	{ 0xebcf, 0xebd1, PDF_CMAP_RANGE, 4600 },
	{ 0xebd2, 0xebd2, PDF_CMAP_SINGLE, 5552 },
	{ 0xebd3, 0xebd7, PDF_CMAP_RANGE, 6368 },
	{ 0xebd8, 0xebd8, PDF_CMAP_SINGLE, 4603 },
	{ 0xebd9, 0xebfe, PDF_CMAP_RANGE, 6373 },
	{ 0xeca1, 0xeca5, PDF_CMAP_RANGE, 6411 },
	{ 0xeca6, 0xeca7, PDF_CMAP_TABLE, 99 },
	{ 0xeca8, 0xeca9, PDF_CMAP_RANGE, 6416 },
	{ 0xecaa, 0xecaa, PDF_CMAP_SINGLE, 4618 },
	{ 0xecab, 0xecae, PDF_CMAP_RANGE, 6418 },
	{ 0xecaf, 0xecaf, PDF_CMAP_SINGLE, 5950 },
	{ 0xecb0, 0xecb1, PDF_CMAP_RANGE, 4620 },
	{ 0xecb2, 0xecb2, PDF_CMAP_SINGLE, 4187 },
	{ 0xecb3, 0xecb4, PDF_CMAP_RANGE, 6422 },
	{ 0xecb5, 0xecb5, PDF_CMAP_SINGLE, 4625 },
	{ 0xecb6, 0xecb7, PDF_CMAP_RANGE, 6424 },
	{ 0xecb8, 0xecba, PDF_CMAP_TABLE, 101 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc6, PDF_CMAP_TABLE, 104 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecfa, PDF_CMAP_TABLE, 106 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f6, PDF_CMAP_TABLE, 108 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa2, PDF_CMAP_TABLE, 111 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc5, PDF_CMAP_SINGLE, 4634 },
	{ 0xecc6, 0xecc6, PDF_CMAP_SINGLE, 4636 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecf9, PDF_CMAP_SINGLE, 6474 },
	{ 0xecfa, 0xecfa, PDF_CMAP_SINGLE, 4650 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f4, PDF_CMAP_SINGLE, 4279 },
	{ 0xf6f5, 0xf6f5, PDF_CMAP_SINGLE, 7398 },
	{ 0xf6f6, 0xf6f6, PDF_CMAP_SINGLE, 7139 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa1, PDF_CMAP_SINGLE, 7709 },
	{ 0xfaa2, 0xfaa2, PDF_CMAP_SINGLE, 3558 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
};

static const unsigned short pdf_cmap_KSCms_UHC_H_table[113] =
{
	3993,3946,4374,4156,4138,5800,4386,4139,
	4140,4391,4141,4142,4396,4403,4149,4419,
	4154,4420,4517,4173,4524,4174,4179,4547,
	4550,4180,4564,5584,5302,5367,4427,5879,
	4435,5896,4436,5897,4437,4159,5936,4444,
	4453,5946,5947,4458,4160,5961,4471,5980,
	4472,4474,4473,4475,5983,4476,5984,4481,
	5987,4482,5460,4483,4163,4166,5990,4485,
	4489,4491,6002,4167,4493,4495,6019,7783,
	4497,4499,4501,6027,4502,6028,4509,6046,
	4510,4553,4180,4555,6152,5800,6153,4556,
	4581,6333,4582,4586,6339,4588,4184,6355,
	4592,4593,4595,4614,4616,4627,6426,4629,
	4634,4636,6474,4650,4279,7398,7139,7709,
	3558,
};

pdf_cmap pdf_cmap_KSCms_UHC_H =
{
	-1, "KSCms-UHC-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8141, 0xfefe },
	},
	675, 675, (pdf_range*) pdf_cmap_KSCms_UHC_H_ranges,
	113, 113, (unsigned short*) pdf_cmap_KSCms_UHC_H_table,
};

/* KSCms-UHC-HW-H */

static const pdf_range pdf_cmap_KSCms_UHC_HW_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 8094 },
	{ 0x8141, 0x815a, PDF_CMAP_RANGE, 9333 },
	{ 0x8161, 0x817a, PDF_CMAP_RANGE, 9359 },
	{ 0x8181, 0x81fe, PDF_CMAP_RANGE, 9385 },
	{ 0x8241, 0x825a, PDF_CMAP_RANGE, 9511 },
	{ 0x8261, 0x827a, PDF_CMAP_RANGE, 9537 },
	{ 0x8281, 0x82fe, PDF_CMAP_RANGE, 9563 },
	{ 0x8341, 0x835a, PDF_CMAP_RANGE, 9689 },
	{ 0x8361, 0x837a, PDF_CMAP_RANGE, 9715 },
	{ 0x8381, 0x83fe, PDF_CMAP_RANGE, 9741 },
	{ 0x8441, 0x845a, PDF_CMAP_RANGE, 9867 },
	{ 0x8461, 0x847a, PDF_CMAP_RANGE, 9893 },
	{ 0x8481, 0x84fe, PDF_CMAP_RANGE, 9919 },
	{ 0x8541, 0x855a, PDF_CMAP_RANGE, 10045 },
	{ 0x8561, 0x857a, PDF_CMAP_RANGE, 10071 },
	{ 0x8581, 0x85fe, PDF_CMAP_RANGE, 10097 },
	{ 0x8641, 0x865a, PDF_CMAP_RANGE, 10223 },
	{ 0x8661, 0x867a, PDF_CMAP_RANGE, 10249 },
	{ 0x8681, 0x86fe, PDF_CMAP_RANGE, 10275 },
	{ 0x8741, 0x875a, PDF_CMAP_RANGE, 10401 },
	{ 0x8761, 0x877a, PDF_CMAP_RANGE, 10427 },
	{ 0x8781, 0x87fe, PDF_CMAP_RANGE, 10453 },
	{ 0x8841, 0x885a, PDF_CMAP_RANGE, 10579 },
	{ 0x8861, 0x887a, PDF_CMAP_RANGE, 10605 },
	{ 0x8881, 0x88fe, PDF_CMAP_RANGE, 10631 },
	{ 0x8941, 0x895a, PDF_CMAP_RANGE, 10757 },
	{ 0x8961, 0x897a, PDF_CMAP_RANGE, 10783 },
	{ 0x8981, 0x89fe, PDF_CMAP_RANGE, 10809 },
	{ 0x8a41, 0x8a5a, PDF_CMAP_RANGE, 10935 },
	{ 0x8a61, 0x8a7a, PDF_CMAP_RANGE, 10961 },
	{ 0x8a81, 0x8afe, PDF_CMAP_RANGE, 10987 },
	{ 0x8b41, 0x8b5a, PDF_CMAP_RANGE, 11113 },
	{ 0x8b61, 0x8b7a, PDF_CMAP_RANGE, 11139 },
	{ 0x8b81, 0x8bfe, PDF_CMAP_RANGE, 11165 },
	{ 0x8c41, 0x8c5a, PDF_CMAP_RANGE, 11291 },
	{ 0x8c61, 0x8c7a, PDF_CMAP_RANGE, 11317 },
	{ 0x8c81, 0x8cfe, PDF_CMAP_RANGE, 11343 },
	{ 0x8d41, 0x8d5a, PDF_CMAP_RANGE, 11469 },
	{ 0x8d61, 0x8d7a, PDF_CMAP_RANGE, 11495 },
	{ 0x8d81, 0x8dfe, PDF_CMAP_RANGE, 11521 },
	{ 0x8e41, 0x8e5a, PDF_CMAP_RANGE, 11647 },
	{ 0x8e61, 0x8e7a, PDF_CMAP_RANGE, 11673 },
	{ 0x8e81, 0x8efe, PDF_CMAP_RANGE, 11699 },
	{ 0x8f41, 0x8f5a, PDF_CMAP_RANGE, 11825 },
	{ 0x8f61, 0x8f7a, PDF_CMAP_RANGE, 11851 },
	{ 0x8f81, 0x8ffe, PDF_CMAP_RANGE, 11877 },
	{ 0x9041, 0x905a, PDF_CMAP_RANGE, 12003 },
	{ 0x9061, 0x907a, PDF_CMAP_RANGE, 12029 },
	{ 0x9081, 0x90fe, PDF_CMAP_RANGE, 12055 },
	{ 0x9141, 0x915a, PDF_CMAP_RANGE, 12181 },
	{ 0x9161, 0x917a, PDF_CMAP_RANGE, 12207 },
	{ 0x9181, 0x91fe, PDF_CMAP_RANGE, 12233 },
	{ 0x9241, 0x925a, PDF_CMAP_RANGE, 12359 },
	{ 0x9261, 0x927a, PDF_CMAP_RANGE, 12385 },
	{ 0x9281, 0x92fe, PDF_CMAP_RANGE, 12411 },
	{ 0x9341, 0x935a, PDF_CMAP_RANGE, 12537 },
	{ 0x9361, 0x937a, PDF_CMAP_RANGE, 12563 },
	{ 0x9381, 0x93fe, PDF_CMAP_RANGE, 12589 },
	{ 0x9441, 0x945a, PDF_CMAP_RANGE, 12715 },
	{ 0x9461, 0x947a, PDF_CMAP_RANGE, 12741 },
	{ 0x9481, 0x94fe, PDF_CMAP_RANGE, 12767 },
	{ 0x9541, 0x955a, PDF_CMAP_RANGE, 12893 },
	{ 0x9561, 0x957a, PDF_CMAP_RANGE, 12919 },
	{ 0x9581, 0x95fe, PDF_CMAP_RANGE, 12945 },
	{ 0x9641, 0x965a, PDF_CMAP_RANGE, 13071 },
	{ 0x9661, 0x967a, PDF_CMAP_RANGE, 13097 },
	{ 0x9681, 0x96fe, PDF_CMAP_RANGE, 13123 },
	{ 0x9741, 0x975a, PDF_CMAP_RANGE, 13249 },
	{ 0x9761, 0x977a, PDF_CMAP_RANGE, 13275 },
	{ 0x9781, 0x97fe, PDF_CMAP_RANGE, 13301 },
	{ 0x9841, 0x985a, PDF_CMAP_RANGE, 13427 },
	{ 0x9861, 0x987a, PDF_CMAP_RANGE, 13453 },
	{ 0x9881, 0x98fe, PDF_CMAP_RANGE, 13479 },
	{ 0x9941, 0x995a, PDF_CMAP_RANGE, 13605 },
	{ 0x9961, 0x997a, PDF_CMAP_RANGE, 13631 },
	{ 0x9981, 0x99fe, PDF_CMAP_RANGE, 13657 },
	{ 0x9a41, 0x9a5a, PDF_CMAP_RANGE, 13783 },
	{ 0x9a61, 0x9a7a, PDF_CMAP_RANGE, 13809 },
	{ 0x9a81, 0x9afe, PDF_CMAP_RANGE, 13835 },
	{ 0x9b41, 0x9b5a, PDF_CMAP_RANGE, 13961 },
	{ 0x9b61, 0x9b7a, PDF_CMAP_RANGE, 13987 },
	{ 0x9b81, 0x9bfe, PDF_CMAP_RANGE, 14013 },
	{ 0x9c41, 0x9c5a, PDF_CMAP_RANGE, 14139 },
	{ 0x9c61, 0x9c7a, PDF_CMAP_RANGE, 14165 },
	{ 0x9c81, 0x9cfe, PDF_CMAP_RANGE, 14191 },
	{ 0x9d41, 0x9d5a, PDF_CMAP_RANGE, 14317 },
	{ 0x9d61, 0x9d7a, PDF_CMAP_RANGE, 14343 },
	{ 0x9d81, 0x9dfe, PDF_CMAP_RANGE, 14369 },
	{ 0x9e41, 0x9e5a, PDF_CMAP_RANGE, 14495 },
	{ 0x9e61, 0x9e7a, PDF_CMAP_RANGE, 14521 },
	{ 0x9e81, 0x9efe, PDF_CMAP_RANGE, 14547 },
	{ 0x9f41, 0x9f5a, PDF_CMAP_RANGE, 14673 },
	{ 0x9f61, 0x9f7a, PDF_CMAP_RANGE, 14699 },
	{ 0x9f81, 0x9ffe, PDF_CMAP_RANGE, 14725 },
	{ 0xa041, 0xa05a, PDF_CMAP_RANGE, 14851 },
	{ 0xa061, 0xa07a, PDF_CMAP_RANGE, 14877 },
	{ 0xa081, 0xa0fe, PDF_CMAP_RANGE, 14903 },
	{ 0xa141, 0xa15a, PDF_CMAP_RANGE, 15029 },
	{ 0xa161, 0xa17a, PDF_CMAP_RANGE, 15055 },
	{ 0xa181, 0xa1a0, PDF_CMAP_RANGE, 15081 },
	{ 0xa1a1, 0xa1fe, PDF_CMAP_RANGE, 101 },
	{ 0xa241, 0xa25a, PDF_CMAP_RANGE, 15113 },
	{ 0xa261, 0xa27a, PDF_CMAP_RANGE, 15139 },
	{ 0xa281, 0xa2a0, PDF_CMAP_RANGE, 15165 },
	{ 0xa2a1, 0xa2e5, PDF_CMAP_RANGE, 195 },
	{ 0xa341, 0xa35a, PDF_CMAP_RANGE, 15197 },
	{ 0xa361, 0xa37a, PDF_CMAP_RANGE, 15223 },
	{ 0xa381, 0xa3a0, PDF_CMAP_RANGE, 15249 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 264 },
	{ 0xa441, 0xa45a, PDF_CMAP_RANGE, 15281 },
	{ 0xa461, 0xa47a, PDF_CMAP_RANGE, 15307 },
	{ 0xa481, 0xa4a0, PDF_CMAP_RANGE, 15333 },
	{ 0xa4a1, 0xa4d3, PDF_CMAP_RANGE, 358 },
	{ 0xa4d5, 0xa4fe, PDF_CMAP_RANGE, 409 },
	{ 0xa541, 0xa55a, PDF_CMAP_RANGE, 15365 },
	{ 0xa561, 0xa57a, PDF_CMAP_RANGE, 15391 },
	{ 0xa581, 0xa5a0, PDF_CMAP_RANGE, 15417 },
	{ 0xa5a1, 0xa5aa, PDF_CMAP_RANGE, 451 },
	{ 0xa5b0, 0xa5b9, PDF_CMAP_RANGE, 461 },
	{ 0xa5c1, 0xa5d8, PDF_CMAP_RANGE, 471 },
	{ 0xa5e1, 0xa5f8, PDF_CMAP_RANGE, 495 },
	{ 0xa641, 0xa65a, PDF_CMAP_RANGE, 15449 },
	{ 0xa661, 0xa67a, PDF_CMAP_RANGE, 15475 },
	{ 0xa681, 0xa6a0, PDF_CMAP_RANGE, 15501 },
	{ 0xa6a1, 0xa6e4, PDF_CMAP_RANGE, 519 },
	{ 0xa741, 0xa75a, PDF_CMAP_RANGE, 15533 },
	{ 0xa761, 0xa77a, PDF_CMAP_RANGE, 15559 },
	{ 0xa781, 0xa7a0, PDF_CMAP_RANGE, 15585 },
	{ 0xa7a1, 0xa7ef, PDF_CMAP_RANGE, 587 },
	{ 0xa841, 0xa85a, PDF_CMAP_RANGE, 15617 },
	{ 0xa861, 0xa87a, PDF_CMAP_RANGE, 15643 },
	{ 0xa881, 0xa8a0, PDF_CMAP_RANGE, 15669 },
	{ 0xa8a1, 0xa8a4, PDF_CMAP_RANGE, 666 },
	{ 0xa8a6, 0xa8a6, PDF_CMAP_SINGLE, 670 },
	{ 0xa8a8, 0xa8af, PDF_CMAP_RANGE, 671 },
	{ 0xa8b1, 0xa8fe, PDF_CMAP_RANGE, 679 },
	{ 0xa941, 0xa95a, PDF_CMAP_RANGE, 15701 },
	{ 0xa961, 0xa97a, PDF_CMAP_RANGE, 15727 },
	{ 0xa981, 0xa9a0, PDF_CMAP_RANGE, 15753 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 757 },
	{ 0xaa41, 0xaa5a, PDF_CMAP_RANGE, 15785 },
	{ 0xaa61, 0xaa7a, PDF_CMAP_RANGE, 15811 },
	{ 0xaa81, 0xaaa0, PDF_CMAP_RANGE, 15837 },
	{ 0xaaa1, 0xaaf3, PDF_CMAP_RANGE, 851 },
	{ 0xab41, 0xab5a, PDF_CMAP_RANGE, 15869 },
	{ 0xab61, 0xab7a, PDF_CMAP_RANGE, 15895 },
	{ 0xab81, 0xaba0, PDF_CMAP_RANGE, 15921 },
	{ 0xaba1, 0xabf6, PDF_CMAP_RANGE, 934 },
	{ 0xac41, 0xac5a, PDF_CMAP_RANGE, 15953 },
	{ 0xac61, 0xac7a, PDF_CMAP_RANGE, 15979 },
	{ 0xac81, 0xaca0, PDF_CMAP_RANGE, 16005 },
	{ 0xaca1, 0xacc1, PDF_CMAP_RANGE, 1020 },
	{ 0xacd1, 0xacf1, PDF_CMAP_RANGE, 1053 },
	{ 0xad41, 0xad5a, PDF_CMAP_RANGE, 16037 },
	{ 0xad61, 0xad7a, PDF_CMAP_RANGE, 16063 },
	{ 0xad81, 0xada0, PDF_CMAP_RANGE, 16089 },
	{ 0xae41, 0xae5a, PDF_CMAP_RANGE, 16121 },
	{ 0xae61, 0xae7a, PDF_CMAP_RANGE, 16147 },
	{ 0xae81, 0xaea0, PDF_CMAP_RANGE, 16173 },
	{ 0xaf41, 0xaf5a, PDF_CMAP_RANGE, 16205 },
	{ 0xaf61, 0xaf7a, PDF_CMAP_RANGE, 16231 },
	{ 0xaf81, 0xafa0, PDF_CMAP_RANGE, 16257 },
	{ 0xb041, 0xb05a, PDF_CMAP_RANGE, 16289 },
	{ 0xb061, 0xb07a, PDF_CMAP_RANGE, 16315 },
	{ 0xb081, 0xb0a0, PDF_CMAP_RANGE, 16341 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 1086 },
	{ 0xb141, 0xb15a, PDF_CMAP_RANGE, 16373 },
	{ 0xb161, 0xb17a, PDF_CMAP_RANGE, 16399 },
	{ 0xb181, 0xb1a0, PDF_CMAP_RANGE, 16425 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1180 },
	{ 0xb241, 0xb25a, PDF_CMAP_RANGE, 16457 },
	{ 0xb261, 0xb27a, PDF_CMAP_RANGE, 16483 },
	{ 0xb281, 0xb2a0, PDF_CMAP_RANGE, 16509 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1274 },
	{ 0xb341, 0xb35a, PDF_CMAP_RANGE, 16541 },
	{ 0xb361, 0xb37a, PDF_CMAP_RANGE, 16567 },
	{ 0xb381, 0xb3a0, PDF_CMAP_RANGE, 16593 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1368 },
	{ 0xb441, 0xb45a, PDF_CMAP_RANGE, 16625 },
	{ 0xb461, 0xb47a, PDF_CMAP_RANGE, 16651 },
	{ 0xb481, 0xb4a0, PDF_CMAP_RANGE, 16677 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1462 },
	{ 0xb541, 0xb55a, PDF_CMAP_RANGE, 16709 },
	{ 0xb561, 0xb57a, PDF_CMAP_RANGE, 16735 },
	{ 0xb581, 0xb5a0, PDF_CMAP_RANGE, 16761 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1556 },
	{ 0xb641, 0xb65a, PDF_CMAP_RANGE, 16793 },
	{ 0xb661, 0xb67a, PDF_CMAP_RANGE, 16819 },
	{ 0xb681, 0xb6a0, PDF_CMAP_RANGE, 16845 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1650 },
	{ 0xb741, 0xb75a, PDF_CMAP_RANGE, 16877 },
	{ 0xb761, 0xb77a, PDF_CMAP_RANGE, 16903 },
	{ 0xb781, 0xb7a0, PDF_CMAP_RANGE, 16929 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1744 },
	{ 0xb841, 0xb85a, PDF_CMAP_RANGE, 16961 },
	{ 0xb861, 0xb87a, PDF_CMAP_RANGE, 16987 },
	{ 0xb881, 0xb8a0, PDF_CMAP_RANGE, 17013 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1838 },
	{ 0xb941, 0xb95a, PDF_CMAP_RANGE, 17045 },
	{ 0xb961, 0xb97a, PDF_CMAP_RANGE, 17071 },
	{ 0xb981, 0xb9a0, PDF_CMAP_RANGE, 17097 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1932 },
	{ 0xba41, 0xba5a, PDF_CMAP_RANGE, 17129 },
	{ 0xba61, 0xba7a, PDF_CMAP_RANGE, 17155 },
	{ 0xba81, 0xbaa0, PDF_CMAP_RANGE, 17181 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 2026 },
	{ 0xbb41, 0xbb5a, PDF_CMAP_RANGE, 17213 },
	{ 0xbb61, 0xbb7a, PDF_CMAP_RANGE, 17239 },
	{ 0xbb81, 0xbba0, PDF_CMAP_RANGE, 17265 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 2120 },
	{ 0xbc41, 0xbc5a, PDF_CMAP_RANGE, 17297 },
	{ 0xbc61, 0xbc7a, PDF_CMAP_RANGE, 17323 },
	{ 0xbc81, 0xbca0, PDF_CMAP_RANGE, 17349 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2214 },
	{ 0xbd41, 0xbd5a, PDF_CMAP_RANGE, 17381 },
	{ 0xbd61, 0xbd7a, PDF_CMAP_RANGE, 17407 },
	{ 0xbd81, 0xbda0, PDF_CMAP_RANGE, 17433 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2308 },
	{ 0xbe41, 0xbe5a, PDF_CMAP_RANGE, 17465 },
	{ 0xbe61, 0xbe7a, PDF_CMAP_RANGE, 17491 },
	{ 0xbe81, 0xbea0, PDF_CMAP_RANGE, 17517 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2402 },
	{ 0xbf41, 0xbf5a, PDF_CMAP_RANGE, 17549 },
	{ 0xbf61, 0xbf7a, PDF_CMAP_RANGE, 17575 },
	{ 0xbf81, 0xbfa0, PDF_CMAP_RANGE, 17601 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2496 },
	{ 0xc041, 0xc05a, PDF_CMAP_RANGE, 17633 },
	{ 0xc061, 0xc07a, PDF_CMAP_RANGE, 17659 },
	{ 0xc081, 0xc0a0, PDF_CMAP_RANGE, 17685 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2590 },
	{ 0xc141, 0xc15a, PDF_CMAP_RANGE, 17717 },
	{ 0xc161, 0xc17a, PDF_CMAP_RANGE, 17743 },
	{ 0xc181, 0xc1a0, PDF_CMAP_RANGE, 17769 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2684 },
	{ 0xc241, 0xc25a, PDF_CMAP_RANGE, 17801 },
	{ 0xc261, 0xc27a, PDF_CMAP_RANGE, 17827 },
	{ 0xc281, 0xc2a0, PDF_CMAP_RANGE, 17853 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2778 },
	{ 0xc341, 0xc35a, PDF_CMAP_RANGE, 17885 },
	{ 0xc361, 0xc37a, PDF_CMAP_RANGE, 17911 },
	{ 0xc381, 0xc3a0, PDF_CMAP_RANGE, 17937 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2872 },
	{ 0xc441, 0xc45a, PDF_CMAP_RANGE, 17969 },
	{ 0xc461, 0xc47a, PDF_CMAP_RANGE, 17995 },
	{ 0xc481, 0xc4a0, PDF_CMAP_RANGE, 18021 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2966 },
	{ 0xc541, 0xc55a, PDF_CMAP_RANGE, 18053 },
	{ 0xc561, 0xc57a, PDF_CMAP_RANGE, 18079 },
	{ 0xc581, 0xc5a0, PDF_CMAP_RANGE, 18105 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 3060 },
	{ 0xc641, 0xc652, PDF_CMAP_RANGE, 18137 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3154 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3248 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3342 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3436 },
	{ 0xcba1, 0xcbcf, PDF_CMAP_RANGE, 3530 },
	{ 0xcbd0, 0xcbd0, PDF_CMAP_SINGLE, 4116 },
	{ 0xcbd1, 0xcbd5, PDF_CMAP_RANGE, 3577 },
	{ 0xcbd6, 0xcbd6, PDF_CMAP_SINGLE, 3678 },
	{ 0xcbd7, 0xcbe6, PDF_CMAP_RANGE, 3582 },
	{ 0xcbe7, 0xcbe7, PDF_CMAP_SINGLE, 7053 },
	{ 0xcbe8, 0xcbfe, PDF_CMAP_RANGE, 3598 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3621 },
	{ 0xcda1, 0xcdce, PDF_CMAP_RANGE, 3715 },
	{ 0xcdcf, 0xcdcf, PDF_CMAP_SINGLE, 3460 },
	{ 0xcdd0, 0xcde7, PDF_CMAP_RANGE, 3761 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 7900 },
	{ 0xcde9, 0xcdfe, PDF_CMAP_RANGE, 3785 },
	{ 0xcea1, 0xceac, PDF_CMAP_RANGE, 3807 },
	{ 0xcead, 0xcead, PDF_CMAP_SINGLE, 3802 },
	{ 0xceae, 0xcefe, PDF_CMAP_RANGE, 3819 },
	{ 0xcfa1, 0xcffa, PDF_CMAP_RANGE, 3900 },
	{ 0xcffb, 0xcffb, PDF_CMAP_SINGLE, 3902 },
	{ 0xcffc, 0xcffe, PDF_CMAP_RANGE, 3990 },
	{ 0xd0a1, 0xd0a2, PDF_CMAP_TABLE, 0 },
	{ 0xd0a3, 0xd0b7, PDF_CMAP_RANGE, 3994 },
	{ 0xd0b8, 0xd0b8, PDF_CMAP_SINGLE, 3946 },
	{ 0xd0b9, 0xd0cf, PDF_CMAP_RANGE, 4015 },
	{ 0xd0d0, 0xd0d0, PDF_CMAP_SINGLE, 3708 },
	{ 0xd0d1, 0xd0dc, PDF_CMAP_RANGE, 4038 },
	{ 0xd0dd, 0xd0dd, PDF_CMAP_SINGLE, 4131 },
	{ 0xd0de, 0xd0fe, PDF_CMAP_RANGE, 4050 },
	{ 0xd1a1, 0xd1d3, PDF_CMAP_RANGE, 4083 },
	{ 0xd1d4, 0xd1d5, PDF_CMAP_TABLE, 2 },
	{ 0xd1d6, 0xd1d7, PDF_CMAP_RANGE, 4134 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 4375 },
	{ 0xd1d9, 0xd1da, PDF_CMAP_RANGE, 4136 },
	{ 0xd1db, 0xd1e0, PDF_CMAP_RANGE, 4376 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_TABLE, 4 },
	{ 0xd1e3, 0xd1e5, PDF_CMAP_RANGE, 4382 },
	{ 0xd1e6, 0xd1e7, PDF_CMAP_TABLE, 6 },
	{ 0xd1e8, 0xd1eb, PDF_CMAP_RANGE, 4387 },
	{ 0xd1ec, 0xd1ee, PDF_CMAP_TABLE, 8 },
	{ 0xd1ef, 0xd1f0, PDF_CMAP_RANGE, 4394 },
	{ 0xd1f1, 0xd1f2, PDF_CMAP_TABLE, 11 },
	{ 0xd1f3, 0xd1f5, PDF_CMAP_RANGE, 4143 },
	{ 0xd1f6, 0xd1f6, PDF_CMAP_SINGLE, 4399 },
	{ 0xd1f7, 0xd1f9, PDF_CMAP_RANGE, 4146 },
	{ 0xd1fa, 0xd1fb, PDF_CMAP_TABLE, 13 },
	{ 0xd1fc, 0xd1fd, PDF_CMAP_RANGE, 4406 },
	{ 0xd1fe, 0xd1fe, PDF_CMAP_SINGLE, 4409 },
	{ 0xd2a1, 0xd2a1, PDF_CMAP_SINGLE, 4150 },
	{ 0xd2a2, 0xd2a3, PDF_CMAP_RANGE, 4410 },
	{ 0xd2a4, 0xd2a6, PDF_CMAP_RANGE, 4151 },
	{ 0xd2a7, 0xd2aa, PDF_CMAP_RANGE, 4412 },
	{ 0xd2ab, 0xd2ad, PDF_CMAP_TABLE, 15 },
	{ 0xd2ae, 0xd2b1, PDF_CMAP_RANGE, 4155 },
	{ 0xd2b2, 0xd2b2, PDF_CMAP_SINGLE, 4424 },
	{ 0xd2b3, 0xd2bd, PDF_CMAP_RANGE, 4159 },
	{ 0xd2be, 0xd2be, PDF_CMAP_SINGLE, 4511 },
	{ 0xd2bf, 0xd2c1, PDF_CMAP_RANGE, 4170 },
	{ 0xd2c2, 0xd2c3, PDF_CMAP_RANGE, 4513 },
	{ 0xd2c4, 0xd2c5, PDF_CMAP_TABLE, 18 },
	{ 0xd2c6, 0xd2ca, PDF_CMAP_RANGE, 4518 },
	{ 0xd2cb, 0xd2cc, PDF_CMAP_TABLE, 20 },
	{ 0xd2cd, 0xd2ce, PDF_CMAP_RANGE, 4525 },
	{ 0xd2cf, 0xd2d4, PDF_CMAP_RANGE, 4528 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 4535 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 4175 },
	{ 0xd2d9, 0xd2da, PDF_CMAP_RANGE, 4541 },
	{ 0xd2db, 0xd2dd, PDF_CMAP_RANGE, 4176 },
	{ 0xd2de, 0xd2df, PDF_CMAP_RANGE, 4545 },
	{ 0xd2e0, 0xd2e4, PDF_CMAP_TABLE, 22 },
	{ 0xd2e5, 0xd2e8, PDF_CMAP_RANGE, 4566 },
	{ 0xd2e9, 0xd2ea, PDF_CMAP_RANGE, 4571 },
	{ 0xd2eb, 0xd2eb, PDF_CMAP_SINGLE, 4576 },
	{ 0xd2ec, 0xd2ef, PDF_CMAP_RANGE, 4181 },
	{ 0xd2f0, 0xd2f3, PDF_CMAP_RANGE, 4604 },
	{ 0xd2f4, 0xd2f5, PDF_CMAP_RANGE, 4609 },
	{ 0xd2f6, 0xd2f6, PDF_CMAP_SINGLE, 4185 },
	{ 0xd2f7, 0xd2f8, PDF_CMAP_RANGE, 4611 },
	{ 0xd2f9, 0xd2fe, PDF_CMAP_RANGE, 4186 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4192 },
	{ 0xd4a1, 0xd4e5, PDF_CMAP_RANGE, 4286 },
	{ 0xd4e6, 0xd4e6, PDF_CMAP_SINGLE, 4318 },
	{ 0xd4e7, 0xd4fb, PDF_CMAP_RANGE, 4355 },
	{ 0xd4fc, 0xd4fc, PDF_CMAP_SINGLE, 4136 },
	{ 0xd4fd, 0xd4fe, PDF_CMAP_RANGE, 4376 },
	{ 0xd5a1, 0xd5a4, PDF_CMAP_RANGE, 4378 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 5800 },
	{ 0xd5a6, 0xd5aa, PDF_CMAP_RANGE, 4382 },
	{ 0xd5ab, 0xd5ab, PDF_CMAP_SINGLE, 4139 },
	{ 0xd5ac, 0xd5ad, PDF_CMAP_RANGE, 4387 },
	{ 0xd5ae, 0xd5ae, PDF_CMAP_SINGLE, 4192 },
	{ 0xd5af, 0xd5fe, PDF_CMAP_RANGE, 4389 },
	{ 0xd6a1, 0xd6b7, PDF_CMAP_RANGE, 4469 },
	{ 0xd6b8, 0xd6b8, PDF_CMAP_SINGLE, 4167 },
	{ 0xd6b9, 0xd6cc, PDF_CMAP_RANGE, 4492 },
	{ 0xd6cd, 0xd6cd, PDF_CMAP_SINGLE, 4172 },
	{ 0xd6ce, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd7a1, 0xd7ca, PDF_CMAP_RANGE, 4561 },
	{ 0xd7cb, 0xd7cb, PDF_CMAP_SINGLE, 5552 },
	{ 0xd7cc, 0xd7e3, PDF_CMAP_RANGE, 4603 },
	{ 0xd7e4, 0xd7e4, PDF_CMAP_SINGLE, 6424 },
	{ 0xd7e5, 0xd7fe, PDF_CMAP_RANGE, 4627 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4653 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4747 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4841 },
	{ 0xdba1, 0xdbc4, PDF_CMAP_RANGE, 4935 },
	{ 0xdbc5, 0xdbc5, PDF_CMAP_SINGLE, 5151 },
	{ 0xdbc6, 0xdbe3, PDF_CMAP_RANGE, 4971 },
	{ 0xdbe4, 0xdbe4, PDF_CMAP_SINGLE, 4922 },
	{ 0xdbe5, 0xdbfe, PDF_CMAP_RANGE, 5001 },
	{ 0xdca1, 0xdca4, PDF_CMAP_RANGE, 5027 },
	{ 0xdca5, 0xdca5, PDF_CMAP_SINGLE, 7518 },
	{ 0xdca6, 0xdcfe, PDF_CMAP_RANGE, 5031 },
	{ 0xdda1, 0xdda4, PDF_CMAP_RANGE, 5120 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 5079 },
	{ 0xdda6, 0xddd4, PDF_CMAP_RANGE, 5124 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 5109 },
	{ 0xddd6, 0xddf3, PDF_CMAP_RANGE, 5171 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 7607 },
	{ 0xddf5, 0xddfe, PDF_CMAP_RANGE, 5201 },
	{ 0xdea1, 0xdefb, PDF_CMAP_RANGE, 5211 },
	{ 0xdefc, 0xdefe, PDF_CMAP_TABLE, 27 },
	{ 0xdfa1, 0xdfb2, PDF_CMAP_RANGE, 5303 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 7083 },
	{ 0xdfb4, 0xdfe0, PDF_CMAP_RANGE, 5321 },
	{ 0xdfe1, 0xdfe1, PDF_CMAP_SINGLE, 5362 },
	{ 0xdfe2, 0xdfe7, PDF_CMAP_RANGE, 5366 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 5485 },
	{ 0xdfe9, 0xdffe, PDF_CMAP_RANGE, 5372 },
	{ 0xe0a1, 0xe0f0, PDF_CMAP_RANGE, 5394 },
	{ 0xe0f1, 0xe0f1, PDF_CMAP_SINGLE, 6001 },
	{ 0xe0f2, 0xe0fe, PDF_CMAP_RANGE, 5474 },
	{ 0xe1a1, 0xe1ac, PDF_CMAP_RANGE, 5487 },
	{ 0xe1ad, 0xe1ad, PDF_CMAP_SINGLE, 5460 },
	{ 0xe1ae, 0xe1ec, PDF_CMAP_RANGE, 5499 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 5317 },
	{ 0xe1ee, 0xe1fe, PDF_CMAP_RANGE, 5562 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5579 },
	{ 0xe3a1, 0xe3f4, PDF_CMAP_RANGE, 5673 },
	{ 0xe3f5, 0xe3f5, PDF_CMAP_SINGLE, 7009 },
	{ 0xe3f6, 0xe3fe, PDF_CMAP_RANGE, 5757 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 7370 },
	{ 0xe4a2, 0xe4a8, PDF_CMAP_RANGE, 5766 },
	{ 0xe4a9, 0xe4a9, PDF_CMAP_SINGLE, 5678 },
	{ 0xe4aa, 0xe4fe, PDF_CMAP_RANGE, 5773 },
	{ 0xe5a1, 0xe5ad, PDF_CMAP_RANGE, 5858 },
	{ 0xe5ae, 0xe5ae, PDF_CMAP_SINGLE, 5874 },
	{ 0xe5af, 0xe5b0, PDF_CMAP_RANGE, 5871 },
	{ 0xe5b1, 0xe5b2, PDF_CMAP_RANGE, 4425 },
	{ 0xe5b3, 0xe5b8, PDF_CMAP_RANGE, 5873 },
	{ 0xe5b9, 0xe5ba, PDF_CMAP_TABLE, 30 },
	{ 0xe5bb, 0xe5bc, PDF_CMAP_RANGE, 4429 },
	{ 0xe5bd, 0xe5c3, PDF_CMAP_RANGE, 5880 },
	{ 0xe5c4, 0xe5c4, PDF_CMAP_SINGLE, 4431 },
	{ 0xe5c5, 0xe5cd, PDF_CMAP_RANGE, 5887 },
	{ 0xe5ce, 0xe5d2, PDF_CMAP_TABLE, 32 },
	{ 0xe5d3, 0xe5d5, PDF_CMAP_RANGE, 5898 },
	{ 0xe5d6, 0xe5d6, PDF_CMAP_SINGLE, 4439 },
	{ 0xe5d7, 0xe5f9, PDF_CMAP_RANGE, 5901 },
	{ 0xe5fa, 0xe5fb, PDF_CMAP_RANGE, 4442 },
	{ 0xe5fc, 0xe5fe, PDF_CMAP_TABLE, 37 },
	{ 0xe6a1, 0xe6a1, PDF_CMAP_SINGLE, 4447 },
	{ 0xe6a2, 0xe6a3, PDF_CMAP_RANGE, 5937 },
	{ 0xe6a4, 0xe6a4, PDF_CMAP_SINGLE, 4449 },
	{ 0xe6a5, 0xe6a6, PDF_CMAP_RANGE, 5939 },
	{ 0xe6a7, 0xe6a7, PDF_CMAP_SINGLE, 4450 },
	{ 0xe6a8, 0xe6ac, PDF_CMAP_RANGE, 5941 },
	{ 0xe6ad, 0xe6ae, PDF_CMAP_TABLE, 40 },
	{ 0xe6af, 0xe6b1, PDF_CMAP_RANGE, 4455 },
	{ 0xe6b2, 0xe6b3, PDF_CMAP_TABLE, 42 },
	{ 0xe6b4, 0xe6b6, PDF_CMAP_RANGE, 5948 },
	{ 0xe6b7, 0xe6b8, PDF_CMAP_RANGE, 4459 },
	{ 0xe6b9, 0xe6bb, PDF_CMAP_RANGE, 5951 },
	{ 0xe6bc, 0xe6bc, PDF_CMAP_SINGLE, 4463 },
	{ 0xe6bd, 0xe6c3, PDF_CMAP_RANGE, 5954 },
	{ 0xe6c4, 0xe6c5, PDF_CMAP_TABLE, 44 },
	{ 0xe6c6, 0xe6c7, PDF_CMAP_RANGE, 4465 },
	{ 0xe6c8, 0xe6c9, PDF_CMAP_RANGE, 5962 },
	{ 0xe6ca, 0xe6ca, PDF_CMAP_SINGLE, 4161 },
	{ 0xe6cb, 0xe6d1, PDF_CMAP_RANGE, 5964 },
	{ 0xe6d2, 0xe6d2, PDF_CMAP_SINGLE, 4468 },
	{ 0xe6d3, 0xe6d5, PDF_CMAP_RANGE, 5971 },
	{ 0xe6d6, 0xe6d6, PDF_CMAP_SINGLE, 4469 },
	{ 0xe6d7, 0xe6d8, PDF_CMAP_RANGE, 5974 },
	{ 0xe6d9, 0xe6d9, PDF_CMAP_SINGLE, 4470 },
	{ 0xe6da, 0xe6db, PDF_CMAP_RANGE, 5976 },
	{ 0xe6dc, 0xe6dc, PDF_CMAP_SINGLE, 4162 },
	{ 0xe6dd, 0xe6de, PDF_CMAP_RANGE, 5978 },
	{ 0xe6df, 0xe6e1, PDF_CMAP_TABLE, 46 },
	{ 0xe6e2, 0xe6e3, PDF_CMAP_RANGE, 5981 },
	{ 0xe6e4, 0xe6e9, PDF_CMAP_TABLE, 49 },
	{ 0xe6ea, 0xe6eb, PDF_CMAP_RANGE, 4478 },
	{ 0xe6ec, 0xe6ec, PDF_CMAP_SINGLE, 6447 },
	{ 0xe6ed, 0xe6ee, PDF_CMAP_RANGE, 5985 },
	{ 0xe6ef, 0xe6f2, PDF_CMAP_TABLE, 55 },
	{ 0xe6f3, 0xe6f4, PDF_CMAP_RANGE, 5988 },
	{ 0xe6f5, 0xe6f9, PDF_CMAP_TABLE, 59 },
	{ 0xe6fa, 0xe6fe, PDF_CMAP_RANGE, 5991 },
	{ 0xe7a1, 0xe7a1, PDF_CMAP_SINGLE, 4487 },
	{ 0xe7a2, 0xe7a5, PDF_CMAP_RANGE, 5996 },
	{ 0xe7a6, 0xe7a6, PDF_CMAP_SINGLE, 4488 },
	{ 0xe7a7, 0xe7a8, PDF_CMAP_RANGE, 6000 },
	{ 0xe7a9, 0xe7ad, PDF_CMAP_TABLE, 64 },
	{ 0xe7ae, 0xe7af, PDF_CMAP_RANGE, 6003 },
	{ 0xe7b0, 0xe7b0, PDF_CMAP_SINGLE, 4494 },
	{ 0xe7b1, 0xe7be, PDF_CMAP_RANGE, 6005 },
	{ 0xe7bf, 0xe7c1, PDF_CMAP_TABLE, 69 },
	{ 0xe7c2, 0xe7c5, PDF_CMAP_RANGE, 6020 },
	{ 0xe7c6, 0xe7c7, PDF_CMAP_TABLE, 72 },
	{ 0xe7c8, 0xe7ca, PDF_CMAP_RANGE, 6024 },
	{ 0xe7cb, 0xe7ce, PDF_CMAP_TABLE, 74 },
	{ 0xe7cf, 0xe7d0, PDF_CMAP_RANGE, 4503 },
	{ 0xe7d1, 0xe7d2, PDF_CMAP_RANGE, 6029 },
	{ 0xe7d3, 0xe7d3, PDF_CMAP_SINGLE, 4506 },
	{ 0xe7d4, 0xe7de, PDF_CMAP_RANGE, 6031 },
	{ 0xe7df, 0xe7df, PDF_CMAP_SINGLE, 4508 },
	{ 0xe7e0, 0xe7e3, PDF_CMAP_RANGE, 6042 },
	{ 0xe7e4, 0xe7e6, PDF_CMAP_TABLE, 78 },
	{ 0xe7e7, 0xe7f6, PDF_CMAP_RANGE, 6047 },
	{ 0xe7f7, 0xe7f7, PDF_CMAP_SINGLE, 5797 },
	{ 0xe7f8, 0xe7fe, PDF_CMAP_RANGE, 6063 },
	{ 0xe8a1, 0xe8e6, PDF_CMAP_RANGE, 6070 },
	{ 0xe8e7, 0xe8e8, PDF_CMAP_RANGE, 4551 },
	{ 0xe8e9, 0xe8ef, PDF_CMAP_RANGE, 6140 },
	{ 0xe8f0, 0xe8f1, PDF_CMAP_TABLE, 81 },
	{ 0xe8f2, 0xe8f6, PDF_CMAP_RANGE, 6147 },
	{ 0xe8f7, 0xe8fb, PDF_CMAP_TABLE, 83 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 6154 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 4557 },
	{ 0xe9a1, 0xe9a6, PDF_CMAP_RANGE, 6156 },
	{ 0xe9a7, 0xe9a7, PDF_CMAP_SINGLE, 4560 },
	{ 0xe9a8, 0xe9ab, PDF_CMAP_RANGE, 6162 },
	{ 0xe9ac, 0xe9ac, PDF_CMAP_SINGLE, 4561 },
	{ 0xe9ad, 0xe9cb, PDF_CMAP_RANGE, 6166 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 4563 },
	{ 0xe9cd, 0xe9f6, PDF_CMAP_RANGE, 6197 },
	{ 0xe9f7, 0xe9f7, PDF_CMAP_SINGLE, 7988 },
	{ 0xe9f8, 0xe9fe, PDF_CMAP_RANGE, 6239 },
	{ 0xeaa1, 0xeac0, PDF_CMAP_RANGE, 6246 },
	{ 0xeac1, 0xeac1, PDF_CMAP_SINGLE, 6123 },
	{ 0xeac2, 0xeae4, PDF_CMAP_RANGE, 6278 },
	{ 0xeae5, 0xeae5, PDF_CMAP_SINGLE, 4577 },
	{ 0xeae6, 0xeaf3, PDF_CMAP_RANGE, 6313 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 4183 },
	{ 0xeaf5, 0xeaf6, PDF_CMAP_RANGE, 6327 },
	{ 0xeaf7, 0xeaf7, PDF_CMAP_SINGLE, 4579 },
	{ 0xeaf8, 0xeafb, PDF_CMAP_RANGE, 6329 },
	{ 0xeafc, 0xeafe, PDF_CMAP_TABLE, 88 },
	{ 0xeba1, 0xeba3, PDF_CMAP_RANGE, 6334 },
	{ 0xeba4, 0xeba4, PDF_CMAP_SINGLE, 4584 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 6337 },
	{ 0xeba7, 0xebaa, PDF_CMAP_TABLE, 91 },
	{ 0xebab, 0xebb9, PDF_CMAP_RANGE, 6340 },
	{ 0xebba, 0xebbb, PDF_CMAP_RANGE, 4590 },
	{ 0xebbc, 0xebbd, PDF_CMAP_TABLE, 95 },
	{ 0xebbe, 0xebc0, PDF_CMAP_RANGE, 6356 },
	{ 0xebc1, 0xebc2, PDF_CMAP_TABLE, 97 },
	{ 0xebc3, 0xebc5, PDF_CMAP_RANGE, 6359 },
	{ 0xebc6, 0xebc7, PDF_CMAP_RANGE, 4596 },
	{ 0xebc8, 0xebcb, PDF_CMAP_RANGE, 6362 },
	{ 0xebcc, 0xebcc, PDF_CMAP_SINGLE, 4599 },
	{ 0xebcd, 0xebce, PDF_CMAP_RANGE, 6366 },
	{ 0xebcf, 0xebd1, PDF_CMAP_RANGE, 4600 },
	{ 0xebd2, 0xebd2, PDF_CMAP_SINGLE, 5552 },
	{ 0xebd3, 0xebd7, PDF_CMAP_RANGE, 6368 },
	{ 0xebd8, 0xebd8, PDF_CMAP_SINGLE, 4603 },
	{ 0xebd9, 0xebfe, PDF_CMAP_RANGE, 6373 },
	{ 0xeca1, 0xeca5, PDF_CMAP_RANGE, 6411 },
	{ 0xeca6, 0xeca7, PDF_CMAP_TABLE, 99 },
	{ 0xeca8, 0xeca9, PDF_CMAP_RANGE, 6416 },
	{ 0xecaa, 0xecaa, PDF_CMAP_SINGLE, 4618 },
	{ 0xecab, 0xecae, PDF_CMAP_RANGE, 6418 },
	{ 0xecaf, 0xecaf, PDF_CMAP_SINGLE, 5950 },
	{ 0xecb0, 0xecb1, PDF_CMAP_RANGE, 4620 },
	{ 0xecb2, 0xecb2, PDF_CMAP_SINGLE, 4187 },
	{ 0xecb3, 0xecb4, PDF_CMAP_RANGE, 6422 },
	{ 0xecb5, 0xecb5, PDF_CMAP_SINGLE, 4625 },
	{ 0xecb6, 0xecb7, PDF_CMAP_RANGE, 6424 },
	{ 0xecb8, 0xecba, PDF_CMAP_TABLE, 101 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc6, PDF_CMAP_TABLE, 104 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecfa, PDF_CMAP_TABLE, 106 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f6, PDF_CMAP_TABLE, 108 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa2, PDF_CMAP_TABLE, 111 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc5, PDF_CMAP_SINGLE, 4634 },
	{ 0xecc6, 0xecc6, PDF_CMAP_SINGLE, 4636 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecf9, PDF_CMAP_SINGLE, 6474 },
	{ 0xecfa, 0xecfa, PDF_CMAP_SINGLE, 4650 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f4, PDF_CMAP_SINGLE, 4279 },
	{ 0xf6f5, 0xf6f5, PDF_CMAP_SINGLE, 7398 },
	{ 0xf6f6, 0xf6f6, PDF_CMAP_SINGLE, 7139 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa1, PDF_CMAP_SINGLE, 7709 },
	{ 0xfaa2, 0xfaa2, PDF_CMAP_SINGLE, 3558 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
};

static const unsigned short pdf_cmap_KSCms_UHC_HW_H_table[113] =
{
	3993,3946,4374,4156,4138,5800,4386,4139,
	4140,4391,4141,4142,4396,4403,4149,4419,
	4154,4420,4517,4173,4524,4174,4179,4547,
	4550,4180,4564,5584,5302,5367,4427,5879,
	4435,5896,4436,5897,4437,4159,5936,4444,
	4453,5946,5947,4458,4160,5961,4471,5980,
	4472,4474,4473,4475,5983,4476,5984,4481,
	5987,4482,5460,4483,4163,4166,5990,4485,
	4489,4491,6002,4167,4493,4495,6019,7783,
	4497,4499,4501,6027,4502,6028,4509,6046,
	4510,4553,4180,4555,6152,5800,6153,4556,
	4581,6333,4582,4586,6339,4588,4184,6355,
	4592,4593,4595,4614,4616,4627,6426,4629,
	4634,4636,6474,4650,4279,7398,7139,7709,
	3558,
};

pdf_cmap pdf_cmap_KSCms_UHC_HW_H =
{
	-1, "KSCms-UHC-HW-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 1, 0x0000, 0x0080 },
		{ 2, 0x8141, 0xfefe },
	},
	675, 675, (pdf_range*) pdf_cmap_KSCms_UHC_HW_H_ranges,
	113, 113, (unsigned short*) pdf_cmap_KSCms_UHC_HW_H_table,
};

/* KSCms-UHC-HW-V */

static const pdf_range pdf_cmap_KSCms_UHC_HW_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 8056 },
	{ 0xa1a5, 0xa1a6, PDF_CMAP_TABLE, 0 },
	{ 0xa1a9, 0xa1ab, PDF_CMAP_RANGE, 8059 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 8062 },
	{ 0xa1b2, 0xa1bd, PDF_CMAP_RANGE, 8063 },
	{ 0xa1eb, 0xa1eb, PDF_CMAP_SINGLE, 8075 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 8076 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 8077 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 8079 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 8080 },
	{ 0xa3ba, 0xa3bf, PDF_CMAP_RANGE, 8081 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 8087 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 8088 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 8089 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
};

static const unsigned short pdf_cmap_KSCms_UHC_HW_V_table[2] =
{
	8058,8320,
};

pdf_cmap pdf_cmap_KSCms_UHC_HW_V =
{
	-1, "KSCms-UHC-HW-V", "KSCms-UHC-HW-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	16, 16, (pdf_range*) pdf_cmap_KSCms_UHC_HW_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_KSCms_UHC_HW_V_table,
};

/* KSCms-UHC-V */

static const pdf_range pdf_cmap_KSCms_UHC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 8056 },
	{ 0xa1a5, 0xa1a6, PDF_CMAP_TABLE, 0 },
	{ 0xa1a9, 0xa1ab, PDF_CMAP_RANGE, 8059 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 8062 },
	{ 0xa1b2, 0xa1bd, PDF_CMAP_RANGE, 8063 },
	{ 0xa1eb, 0xa1eb, PDF_CMAP_SINGLE, 8075 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 8076 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 8077 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 8079 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 8080 },
	{ 0xa3ba, 0xa3bf, PDF_CMAP_RANGE, 8081 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 8087 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 8088 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 8089 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
};

static const unsigned short pdf_cmap_KSCms_UHC_V_table[2] =
{
	8058,8320,
};

pdf_cmap pdf_cmap_KSCms_UHC_V =
{
	-1, "KSCms-UHC-V", "KSCms-UHC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	16, 16, (pdf_range*) pdf_cmap_KSCms_UHC_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_KSCms_UHC_V_table,
};

/* KSCpc-EUC-H */

static const pdf_range pdf_cmap_KSCpc_EUC_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x0081, 0x0083, PDF_CMAP_RANGE, 96 },
	{ 0x00fe, 0x00ff, PDF_CMAP_RANGE, 99 },
	{ 0xa141, 0xa17d, PDF_CMAP_RANGE, 8191 },
	{ 0xa181, 0xa19a, PDF_CMAP_RANGE, 8252 },
	{ 0xa19c, 0xa1a0, PDF_CMAP_RANGE, 8278 },
	{ 0xa1a1, 0xa1a1, PDF_CMAP_SINGLE, 101 },
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 8283 },
	{ 0xa1a4, 0xa1fe, PDF_CMAP_RANGE, 104 },
	{ 0xa241, 0xa24c, PDF_CMAP_RANGE, 8285 },
	{ 0xa2a1, 0xa2e5, PDF_CMAP_RANGE, 195 },
	{ 0xa2e6, 0xa2fe, PDF_CMAP_RANGE, 8297 },
	{ 0xa341, 0xa37d, PDF_CMAP_RANGE, 8322 },
	{ 0xa381, 0xa39f, PDF_CMAP_RANGE, 8383 },
	{ 0xa3a1, 0xa3fe, PDF_CMAP_RANGE, 264 },
	{ 0xa441, 0xa47d, PDF_CMAP_RANGE, 8414 },
	{ 0xa481, 0xa49f, PDF_CMAP_RANGE, 8475 },
	{ 0xa4a1, 0xa4d3, PDF_CMAP_RANGE, 358 },
	{ 0xa4d5, 0xa4fe, PDF_CMAP_RANGE, 409 },
	{ 0xa541, 0xa57d, PDF_CMAP_RANGE, 8506 },
	{ 0xa581, 0xa58b, PDF_CMAP_RANGE, 8567 },
	{ 0xa5a1, 0xa5aa, PDF_CMAP_RANGE, 451 },
	{ 0xa5b0, 0xa5b9, PDF_CMAP_RANGE, 461 },
	{ 0xa5c1, 0xa5d8, PDF_CMAP_RANGE, 471 },
	{ 0xa5da, 0xa5de, PDF_CMAP_RANGE, 8578 },
	{ 0xa5e1, 0xa5f8, PDF_CMAP_RANGE, 495 },
	{ 0xa5f9, 0xa5fc, PDF_CMAP_RANGE, 8583 },
	{ 0xa641, 0xa67d, PDF_CMAP_RANGE, 8587 },
	{ 0xa681, 0xa68a, PDF_CMAP_RANGE, 8648 },
	{ 0xa68d, 0xa693, PDF_CMAP_RANGE, 8658 },
	{ 0xa696, 0xa69b, PDF_CMAP_RANGE, 8665 },
	{ 0xa69e, 0xa69f, PDF_CMAP_RANGE, 8671 },
	{ 0xa6a1, 0xa6e4, PDF_CMAP_RANGE, 519 },
	{ 0xa6e5, 0xa6fe, PDF_CMAP_RANGE, 8673 },
	{ 0xa741, 0xa77d, PDF_CMAP_RANGE, 8699 },
	{ 0xa781, 0xa79f, PDF_CMAP_RANGE, 8760 },
	{ 0xa7a1, 0xa7ef, PDF_CMAP_RANGE, 587 },
	{ 0xa7f0, 0xa7fe, PDF_CMAP_RANGE, 8791 },
	{ 0xa841, 0xa87d, PDF_CMAP_RANGE, 8806 },
	{ 0xa881, 0xa89f, PDF_CMAP_RANGE, 8867 },
	{ 0xa8a1, 0xa8a4, PDF_CMAP_RANGE, 666 },
	{ 0xa8a6, 0xa8a6, PDF_CMAP_SINGLE, 670 },
	{ 0xa8a8, 0xa8af, PDF_CMAP_RANGE, 671 },
	{ 0xa8b1, 0xa8fe, PDF_CMAP_RANGE, 679 },
	{ 0xa941, 0xa974, PDF_CMAP_RANGE, 8898 },
	{ 0xa9a1, 0xa9fe, PDF_CMAP_RANGE, 757 },
	{ 0xaa41, 0xaa7d, PDF_CMAP_RANGE, 8950 },
	{ 0xaa81, 0xaa9f, PDF_CMAP_RANGE, 9011 },
	{ 0xaaa1, 0xaaf3, PDF_CMAP_RANGE, 851 },
	{ 0xaaf4, 0xaafe, PDF_CMAP_RANGE, 9042 },
	{ 0xab41, 0xab7d, PDF_CMAP_RANGE, 9053 },
	{ 0xab81, 0xab9f, PDF_CMAP_RANGE, 9114 },
	{ 0xaba1, 0xabf6, PDF_CMAP_RANGE, 934 },
	{ 0xabf7, 0xabfa, PDF_CMAP_RANGE, 9145 },
	{ 0xabfb, 0xabfd, PDF_CMAP_RANGE, 9330 },
	{ 0xac41, 0xac7d, PDF_CMAP_RANGE, 9149 },
	{ 0xac81, 0xac97, PDF_CMAP_RANGE, 9210 },
	{ 0xaca1, 0xacc1, PDF_CMAP_RANGE, 1020 },
	{ 0xacc2, 0xacd0, PDF_CMAP_RANGE, 9233 },
	{ 0xacd1, 0xacf1, PDF_CMAP_RANGE, 1053 },
	{ 0xacf2, 0xacf6, PDF_CMAP_RANGE, 9248 },
	{ 0xad41, 0xad7d, PDF_CMAP_RANGE, 9253 },
	{ 0xada1, 0xadb0, PDF_CMAP_RANGE, 9314 },
	{ 0xb0a1, 0xb0fe, PDF_CMAP_RANGE, 1086 },
	{ 0xb1a1, 0xb1fe, PDF_CMAP_RANGE, 1180 },
	{ 0xb2a1, 0xb2fe, PDF_CMAP_RANGE, 1274 },
	{ 0xb3a1, 0xb3fe, PDF_CMAP_RANGE, 1368 },
	{ 0xb4a1, 0xb4fe, PDF_CMAP_RANGE, 1462 },
	{ 0xb5a1, 0xb5fe, PDF_CMAP_RANGE, 1556 },
	{ 0xb6a1, 0xb6fe, PDF_CMAP_RANGE, 1650 },
	{ 0xb7a1, 0xb7fe, PDF_CMAP_RANGE, 1744 },
	{ 0xb8a1, 0xb8fe, PDF_CMAP_RANGE, 1838 },
	{ 0xb9a1, 0xb9fe, PDF_CMAP_RANGE, 1932 },
	{ 0xbaa1, 0xbafe, PDF_CMAP_RANGE, 2026 },
	{ 0xbba1, 0xbbfe, PDF_CMAP_RANGE, 2120 },
	{ 0xbca1, 0xbcfe, PDF_CMAP_RANGE, 2214 },
	{ 0xbda1, 0xbdfe, PDF_CMAP_RANGE, 2308 },
	{ 0xbea1, 0xbefe, PDF_CMAP_RANGE, 2402 },
	{ 0xbfa1, 0xbffe, PDF_CMAP_RANGE, 2496 },
	{ 0xc0a1, 0xc0fe, PDF_CMAP_RANGE, 2590 },
	{ 0xc1a1, 0xc1fe, PDF_CMAP_RANGE, 2684 },
	{ 0xc2a1, 0xc2fe, PDF_CMAP_RANGE, 2778 },
	{ 0xc3a1, 0xc3fe, PDF_CMAP_RANGE, 2872 },
	{ 0xc4a1, 0xc4fe, PDF_CMAP_RANGE, 2966 },
	{ 0xc5a1, 0xc5fe, PDF_CMAP_RANGE, 3060 },
	{ 0xc6a1, 0xc6fe, PDF_CMAP_RANGE, 3154 },
	{ 0xc7a1, 0xc7fe, PDF_CMAP_RANGE, 3248 },
	{ 0xc8a1, 0xc8fe, PDF_CMAP_RANGE, 3342 },
	{ 0xcaa1, 0xcafe, PDF_CMAP_RANGE, 3436 },
	{ 0xcba1, 0xcbcf, PDF_CMAP_RANGE, 3530 },
	{ 0xcbd0, 0xcbd0, PDF_CMAP_SINGLE, 4116 },
	{ 0xcbd1, 0xcbd5, PDF_CMAP_RANGE, 3577 },
	{ 0xcbd6, 0xcbd6, PDF_CMAP_SINGLE, 3678 },
	{ 0xcbd7, 0xcbe6, PDF_CMAP_RANGE, 3582 },
	{ 0xcbe7, 0xcbe7, PDF_CMAP_SINGLE, 7053 },
	{ 0xcbe8, 0xcbfe, PDF_CMAP_RANGE, 3598 },
	{ 0xcca1, 0xccfe, PDF_CMAP_RANGE, 3621 },
	{ 0xcda1, 0xcdce, PDF_CMAP_RANGE, 3715 },
	{ 0xcdcf, 0xcdcf, PDF_CMAP_SINGLE, 3460 },
	{ 0xcdd0, 0xcde7, PDF_CMAP_RANGE, 3761 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 7900 },
	{ 0xcde9, 0xcdfe, PDF_CMAP_RANGE, 3785 },
	{ 0xcea1, 0xceac, PDF_CMAP_RANGE, 3807 },
	{ 0xcead, 0xcead, PDF_CMAP_SINGLE, 3802 },
	{ 0xceae, 0xcefe, PDF_CMAP_RANGE, 3819 },
	{ 0xcfa1, 0xcffa, PDF_CMAP_RANGE, 3900 },
	{ 0xcffb, 0xcffb, PDF_CMAP_SINGLE, 3902 },
	{ 0xcffc, 0xcffe, PDF_CMAP_RANGE, 3990 },
	{ 0xd0a1, 0xd0a2, PDF_CMAP_TABLE, 0 },
	{ 0xd0a3, 0xd0b7, PDF_CMAP_RANGE, 3994 },
	{ 0xd0b8, 0xd0b8, PDF_CMAP_SINGLE, 3946 },
	{ 0xd0b9, 0xd0cf, PDF_CMAP_RANGE, 4015 },
	{ 0xd0d0, 0xd0d0, PDF_CMAP_SINGLE, 3708 },
	{ 0xd0d1, 0xd0dc, PDF_CMAP_RANGE, 4038 },
	{ 0xd0dd, 0xd0dd, PDF_CMAP_SINGLE, 4131 },
	{ 0xd0de, 0xd0fe, PDF_CMAP_RANGE, 4050 },
	{ 0xd1a1, 0xd1d3, PDF_CMAP_RANGE, 4083 },
	{ 0xd1d4, 0xd1d5, PDF_CMAP_TABLE, 2 },
	{ 0xd1d6, 0xd1d7, PDF_CMAP_RANGE, 4134 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 4375 },
	{ 0xd1d9, 0xd1da, PDF_CMAP_RANGE, 4136 },
	{ 0xd1db, 0xd1e0, PDF_CMAP_RANGE, 4376 },
	{ 0xd1e1, 0xd1e2, PDF_CMAP_TABLE, 4 },
	{ 0xd1e3, 0xd1e5, PDF_CMAP_RANGE, 4382 },
	{ 0xd1e6, 0xd1e7, PDF_CMAP_TABLE, 6 },
	{ 0xd1e8, 0xd1eb, PDF_CMAP_RANGE, 4387 },
	{ 0xd1ec, 0xd1ee, PDF_CMAP_TABLE, 8 },
	{ 0xd1ef, 0xd1f0, PDF_CMAP_RANGE, 4394 },
	{ 0xd1f1, 0xd1f2, PDF_CMAP_TABLE, 11 },
	{ 0xd1f3, 0xd1f5, PDF_CMAP_RANGE, 4143 },
	{ 0xd1f6, 0xd1f6, PDF_CMAP_SINGLE, 4399 },
	{ 0xd1f7, 0xd1f9, PDF_CMAP_RANGE, 4146 },
	{ 0xd1fa, 0xd1fb, PDF_CMAP_TABLE, 13 },
	{ 0xd1fc, 0xd1fd, PDF_CMAP_RANGE, 4406 },
	{ 0xd1fe, 0xd1fe, PDF_CMAP_SINGLE, 4409 },
	{ 0xd2a1, 0xd2a1, PDF_CMAP_SINGLE, 4150 },
	{ 0xd2a2, 0xd2a3, PDF_CMAP_RANGE, 4410 },
	{ 0xd2a4, 0xd2a6, PDF_CMAP_RANGE, 4151 },
	{ 0xd2a7, 0xd2aa, PDF_CMAP_RANGE, 4412 },
	{ 0xd2ab, 0xd2ad, PDF_CMAP_TABLE, 15 },
	{ 0xd2ae, 0xd2b1, PDF_CMAP_RANGE, 4155 },
	{ 0xd2b2, 0xd2b2, PDF_CMAP_SINGLE, 4424 },
	{ 0xd2b3, 0xd2bd, PDF_CMAP_RANGE, 4159 },
	{ 0xd2be, 0xd2be, PDF_CMAP_SINGLE, 4511 },
	{ 0xd2bf, 0xd2c1, PDF_CMAP_RANGE, 4170 },
	{ 0xd2c2, 0xd2c3, PDF_CMAP_RANGE, 4513 },
	{ 0xd2c4, 0xd2c5, PDF_CMAP_TABLE, 18 },
	{ 0xd2c6, 0xd2ca, PDF_CMAP_RANGE, 4518 },
	{ 0xd2cb, 0xd2cc, PDF_CMAP_TABLE, 20 },
	{ 0xd2cd, 0xd2ce, PDF_CMAP_RANGE, 4525 },
	{ 0xd2cf, 0xd2d4, PDF_CMAP_RANGE, 4528 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 4535 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 4175 },
	{ 0xd2d9, 0xd2da, PDF_CMAP_RANGE, 4541 },
	{ 0xd2db, 0xd2dd, PDF_CMAP_RANGE, 4176 },
	{ 0xd2de, 0xd2df, PDF_CMAP_RANGE, 4545 },
	{ 0xd2e0, 0xd2e4, PDF_CMAP_TABLE, 22 },
	{ 0xd2e5, 0xd2e8, PDF_CMAP_RANGE, 4566 },
	{ 0xd2e9, 0xd2ea, PDF_CMAP_RANGE, 4571 },
	{ 0xd2eb, 0xd2eb, PDF_CMAP_SINGLE, 4576 },
	{ 0xd2ec, 0xd2ef, PDF_CMAP_RANGE, 4181 },
	{ 0xd2f0, 0xd2f3, PDF_CMAP_RANGE, 4604 },
	{ 0xd2f4, 0xd2f5, PDF_CMAP_RANGE, 4609 },
	{ 0xd2f6, 0xd2f6, PDF_CMAP_SINGLE, 4185 },
	{ 0xd2f7, 0xd2f8, PDF_CMAP_RANGE, 4611 },
	{ 0xd2f9, 0xd2fe, PDF_CMAP_RANGE, 4186 },
	{ 0xd3a1, 0xd3fe, PDF_CMAP_RANGE, 4192 },
	{ 0xd4a1, 0xd4e5, PDF_CMAP_RANGE, 4286 },
	{ 0xd4e6, 0xd4e6, PDF_CMAP_SINGLE, 4318 },
	{ 0xd4e7, 0xd4fb, PDF_CMAP_RANGE, 4355 },
	{ 0xd4fc, 0xd4fc, PDF_CMAP_SINGLE, 4136 },
	{ 0xd4fd, 0xd4fe, PDF_CMAP_RANGE, 4376 },
	{ 0xd5a1, 0xd5a4, PDF_CMAP_RANGE, 4378 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 5800 },
	{ 0xd5a6, 0xd5aa, PDF_CMAP_RANGE, 4382 },
	{ 0xd5ab, 0xd5ab, PDF_CMAP_SINGLE, 4139 },
	{ 0xd5ac, 0xd5ad, PDF_CMAP_RANGE, 4387 },
	{ 0xd5ae, 0xd5ae, PDF_CMAP_SINGLE, 4192 },
	{ 0xd5af, 0xd5fe, PDF_CMAP_RANGE, 4389 },
	{ 0xd6a1, 0xd6b7, PDF_CMAP_RANGE, 4469 },
	{ 0xd6b8, 0xd6b8, PDF_CMAP_SINGLE, 4167 },
	{ 0xd6b9, 0xd6cc, PDF_CMAP_RANGE, 4492 },
	{ 0xd6cd, 0xd6cd, PDF_CMAP_SINGLE, 4172 },
	{ 0xd6ce, 0xd6fe, PDF_CMAP_RANGE, 4512 },
	{ 0xd7a1, 0xd7ca, PDF_CMAP_RANGE, 4561 },
	{ 0xd7cb, 0xd7cb, PDF_CMAP_SINGLE, 5552 },
	{ 0xd7cc, 0xd7e3, PDF_CMAP_RANGE, 4603 },
	{ 0xd7e4, 0xd7e4, PDF_CMAP_SINGLE, 6424 },
	{ 0xd7e5, 0xd7fe, PDF_CMAP_RANGE, 4627 },
	{ 0xd8a1, 0xd8fe, PDF_CMAP_RANGE, 4653 },
	{ 0xd9a1, 0xd9fe, PDF_CMAP_RANGE, 4747 },
	{ 0xdaa1, 0xdafe, PDF_CMAP_RANGE, 4841 },
	{ 0xdba1, 0xdbc4, PDF_CMAP_RANGE, 4935 },
	{ 0xdbc5, 0xdbc5, PDF_CMAP_SINGLE, 5151 },
	{ 0xdbc6, 0xdbe3, PDF_CMAP_RANGE, 4971 },
	{ 0xdbe4, 0xdbe4, PDF_CMAP_SINGLE, 4922 },
	{ 0xdbe5, 0xdbfe, PDF_CMAP_RANGE, 5001 },
	{ 0xdca1, 0xdca4, PDF_CMAP_RANGE, 5027 },
	{ 0xdca5, 0xdca5, PDF_CMAP_SINGLE, 7518 },
	{ 0xdca6, 0xdcfe, PDF_CMAP_RANGE, 5031 },
	{ 0xdda1, 0xdda4, PDF_CMAP_RANGE, 5120 },
	{ 0xdda5, 0xdda5, PDF_CMAP_SINGLE, 5079 },
	{ 0xdda6, 0xddd4, PDF_CMAP_RANGE, 5124 },
	{ 0xddd5, 0xddd5, PDF_CMAP_SINGLE, 5109 },
	{ 0xddd6, 0xddf3, PDF_CMAP_RANGE, 5171 },
	{ 0xddf4, 0xddf4, PDF_CMAP_SINGLE, 7607 },
	{ 0xddf5, 0xddfe, PDF_CMAP_RANGE, 5201 },
	{ 0xdea1, 0xdefb, PDF_CMAP_RANGE, 5211 },
	{ 0xdefc, 0xdefe, PDF_CMAP_TABLE, 27 },
	{ 0xdfa1, 0xdfb2, PDF_CMAP_RANGE, 5303 },
	{ 0xdfb3, 0xdfb3, PDF_CMAP_SINGLE, 7083 },
	{ 0xdfb4, 0xdfe0, PDF_CMAP_RANGE, 5321 },
	{ 0xdfe1, 0xdfe1, PDF_CMAP_SINGLE, 5362 },
	{ 0xdfe2, 0xdfe7, PDF_CMAP_RANGE, 5366 },
	{ 0xdfe8, 0xdfe8, PDF_CMAP_SINGLE, 5485 },
	{ 0xdfe9, 0xdffe, PDF_CMAP_RANGE, 5372 },
	{ 0xe0a1, 0xe0f0, PDF_CMAP_RANGE, 5394 },
	{ 0xe0f1, 0xe0f1, PDF_CMAP_SINGLE, 6001 },
	{ 0xe0f2, 0xe0fe, PDF_CMAP_RANGE, 5474 },
	{ 0xe1a1, 0xe1ac, PDF_CMAP_RANGE, 5487 },
	{ 0xe1ad, 0xe1ad, PDF_CMAP_SINGLE, 5460 },
	{ 0xe1ae, 0xe1ec, PDF_CMAP_RANGE, 5499 },
	{ 0xe1ed, 0xe1ed, PDF_CMAP_SINGLE, 5317 },
	{ 0xe1ee, 0xe1fe, PDF_CMAP_RANGE, 5562 },
	{ 0xe2a1, 0xe2fe, PDF_CMAP_RANGE, 5579 },
	{ 0xe3a1, 0xe3f4, PDF_CMAP_RANGE, 5673 },
	{ 0xe3f5, 0xe3f5, PDF_CMAP_SINGLE, 7009 },
	{ 0xe3f6, 0xe3fe, PDF_CMAP_RANGE, 5757 },
	{ 0xe4a1, 0xe4a1, PDF_CMAP_SINGLE, 7370 },
	{ 0xe4a2, 0xe4a8, PDF_CMAP_RANGE, 5766 },
	{ 0xe4a9, 0xe4a9, PDF_CMAP_SINGLE, 5678 },
	{ 0xe4aa, 0xe4fe, PDF_CMAP_RANGE, 5773 },
	{ 0xe5a1, 0xe5ad, PDF_CMAP_RANGE, 5858 },
	{ 0xe5ae, 0xe5ae, PDF_CMAP_SINGLE, 5874 },
	{ 0xe5af, 0xe5b0, PDF_CMAP_RANGE, 5871 },
	{ 0xe5b1, 0xe5b2, PDF_CMAP_RANGE, 4425 },
	{ 0xe5b3, 0xe5b8, PDF_CMAP_RANGE, 5873 },
	{ 0xe5b9, 0xe5ba, PDF_CMAP_TABLE, 30 },
	{ 0xe5bb, 0xe5bc, PDF_CMAP_RANGE, 4429 },
	{ 0xe5bd, 0xe5c3, PDF_CMAP_RANGE, 5880 },
	{ 0xe5c4, 0xe5c4, PDF_CMAP_SINGLE, 4431 },
	{ 0xe5c5, 0xe5cd, PDF_CMAP_RANGE, 5887 },
	{ 0xe5ce, 0xe5d2, PDF_CMAP_TABLE, 32 },
	{ 0xe5d3, 0xe5d5, PDF_CMAP_RANGE, 5898 },
	{ 0xe5d6, 0xe5d6, PDF_CMAP_SINGLE, 4439 },
	{ 0xe5d7, 0xe5f9, PDF_CMAP_RANGE, 5901 },
	{ 0xe5fa, 0xe5fb, PDF_CMAP_RANGE, 4442 },
	{ 0xe5fc, 0xe5fe, PDF_CMAP_TABLE, 37 },
	{ 0xe6a1, 0xe6a1, PDF_CMAP_SINGLE, 4447 },
	{ 0xe6a2, 0xe6a3, PDF_CMAP_RANGE, 5937 },
	{ 0xe6a4, 0xe6a4, PDF_CMAP_SINGLE, 4449 },
	{ 0xe6a5, 0xe6a6, PDF_CMAP_RANGE, 5939 },
	{ 0xe6a7, 0xe6a7, PDF_CMAP_SINGLE, 4450 },
	{ 0xe6a8, 0xe6ac, PDF_CMAP_RANGE, 5941 },
	{ 0xe6ad, 0xe6ae, PDF_CMAP_TABLE, 40 },
	{ 0xe6af, 0xe6b1, PDF_CMAP_RANGE, 4455 },
	{ 0xe6b2, 0xe6b3, PDF_CMAP_TABLE, 42 },
	{ 0xe6b4, 0xe6b6, PDF_CMAP_RANGE, 5948 },
	{ 0xe6b7, 0xe6b8, PDF_CMAP_RANGE, 4459 },
	{ 0xe6b9, 0xe6bb, PDF_CMAP_RANGE, 5951 },
	{ 0xe6bc, 0xe6bc, PDF_CMAP_SINGLE, 4463 },
	{ 0xe6bd, 0xe6c3, PDF_CMAP_RANGE, 5954 },
	{ 0xe6c4, 0xe6c5, PDF_CMAP_TABLE, 44 },
	{ 0xe6c6, 0xe6c7, PDF_CMAP_RANGE, 4465 },
	{ 0xe6c8, 0xe6c9, PDF_CMAP_RANGE, 5962 },
	{ 0xe6ca, 0xe6ca, PDF_CMAP_SINGLE, 4161 },
	{ 0xe6cb, 0xe6d1, PDF_CMAP_RANGE, 5964 },
	{ 0xe6d2, 0xe6d2, PDF_CMAP_SINGLE, 4468 },
	{ 0xe6d3, 0xe6d5, PDF_CMAP_RANGE, 5971 },
	{ 0xe6d6, 0xe6d6, PDF_CMAP_SINGLE, 4469 },
	{ 0xe6d7, 0xe6d8, PDF_CMAP_RANGE, 5974 },
	{ 0xe6d9, 0xe6d9, PDF_CMAP_SINGLE, 4470 },
	{ 0xe6da, 0xe6db, PDF_CMAP_RANGE, 5976 },
	{ 0xe6dc, 0xe6dc, PDF_CMAP_SINGLE, 4162 },
	{ 0xe6dd, 0xe6de, PDF_CMAP_RANGE, 5978 },
	{ 0xe6df, 0xe6e1, PDF_CMAP_TABLE, 46 },
	{ 0xe6e2, 0xe6e3, PDF_CMAP_RANGE, 5981 },
	{ 0xe6e4, 0xe6e9, PDF_CMAP_TABLE, 49 },
	{ 0xe6ea, 0xe6eb, PDF_CMAP_RANGE, 4478 },
	{ 0xe6ec, 0xe6ec, PDF_CMAP_SINGLE, 6447 },
	{ 0xe6ed, 0xe6ee, PDF_CMAP_RANGE, 5985 },
	{ 0xe6ef, 0xe6f2, PDF_CMAP_TABLE, 55 },
	{ 0xe6f3, 0xe6f4, PDF_CMAP_RANGE, 5988 },
	{ 0xe6f5, 0xe6f9, PDF_CMAP_TABLE, 59 },
	{ 0xe6fa, 0xe6fe, PDF_CMAP_RANGE, 5991 },
	{ 0xe7a1, 0xe7a1, PDF_CMAP_SINGLE, 4487 },
	{ 0xe7a2, 0xe7a5, PDF_CMAP_RANGE, 5996 },
	{ 0xe7a6, 0xe7a6, PDF_CMAP_SINGLE, 4488 },
	{ 0xe7a7, 0xe7a8, PDF_CMAP_RANGE, 6000 },
	{ 0xe7a9, 0xe7ad, PDF_CMAP_TABLE, 64 },
	{ 0xe7ae, 0xe7af, PDF_CMAP_RANGE, 6003 },
	{ 0xe7b0, 0xe7b0, PDF_CMAP_SINGLE, 4494 },
	{ 0xe7b1, 0xe7be, PDF_CMAP_RANGE, 6005 },
	{ 0xe7bf, 0xe7c1, PDF_CMAP_TABLE, 69 },
	{ 0xe7c2, 0xe7c5, PDF_CMAP_RANGE, 6020 },
	{ 0xe7c6, 0xe7c7, PDF_CMAP_TABLE, 72 },
	{ 0xe7c8, 0xe7ca, PDF_CMAP_RANGE, 6024 },
	{ 0xe7cb, 0xe7ce, PDF_CMAP_TABLE, 74 },
	{ 0xe7cf, 0xe7d0, PDF_CMAP_RANGE, 4503 },
	{ 0xe7d1, 0xe7d2, PDF_CMAP_RANGE, 6029 },
	{ 0xe7d3, 0xe7d3, PDF_CMAP_SINGLE, 4506 },
	{ 0xe7d4, 0xe7de, PDF_CMAP_RANGE, 6031 },
	{ 0xe7df, 0xe7df, PDF_CMAP_SINGLE, 4508 },
	{ 0xe7e0, 0xe7e3, PDF_CMAP_RANGE, 6042 },
	{ 0xe7e4, 0xe7e6, PDF_CMAP_TABLE, 78 },
	{ 0xe7e7, 0xe7f6, PDF_CMAP_RANGE, 6047 },
	{ 0xe7f7, 0xe7f7, PDF_CMAP_SINGLE, 5797 },
	{ 0xe7f8, 0xe7fe, PDF_CMAP_RANGE, 6063 },
	{ 0xe8a1, 0xe8e6, PDF_CMAP_RANGE, 6070 },
	{ 0xe8e7, 0xe8e8, PDF_CMAP_RANGE, 4551 },
	{ 0xe8e9, 0xe8ef, PDF_CMAP_RANGE, 6140 },
	{ 0xe8f0, 0xe8f1, PDF_CMAP_TABLE, 81 },
	{ 0xe8f2, 0xe8f6, PDF_CMAP_RANGE, 6147 },
	{ 0xe8f7, 0xe8fb, PDF_CMAP_TABLE, 83 },
	{ 0xe8fc, 0xe8fd, PDF_CMAP_RANGE, 6154 },
	{ 0xe8fe, 0xe8fe, PDF_CMAP_SINGLE, 4557 },
	{ 0xe9a1, 0xe9a6, PDF_CMAP_RANGE, 6156 },
	{ 0xe9a7, 0xe9a7, PDF_CMAP_SINGLE, 4560 },
	{ 0xe9a8, 0xe9ab, PDF_CMAP_RANGE, 6162 },
	{ 0xe9ac, 0xe9ac, PDF_CMAP_SINGLE, 4561 },
	{ 0xe9ad, 0xe9cb, PDF_CMAP_RANGE, 6166 },
	{ 0xe9cc, 0xe9cc, PDF_CMAP_SINGLE, 4563 },
	{ 0xe9cd, 0xe9f6, PDF_CMAP_RANGE, 6197 },
	{ 0xe9f7, 0xe9f7, PDF_CMAP_SINGLE, 7988 },
	{ 0xe9f8, 0xe9fe, PDF_CMAP_RANGE, 6239 },
	{ 0xeaa1, 0xeac0, PDF_CMAP_RANGE, 6246 },
	{ 0xeac1, 0xeac1, PDF_CMAP_SINGLE, 6123 },
	{ 0xeac2, 0xeae4, PDF_CMAP_RANGE, 6278 },
	{ 0xeae5, 0xeae5, PDF_CMAP_SINGLE, 4577 },
	{ 0xeae6, 0xeaf3, PDF_CMAP_RANGE, 6313 },
	{ 0xeaf4, 0xeaf4, PDF_CMAP_SINGLE, 4183 },
	{ 0xeaf5, 0xeaf6, PDF_CMAP_RANGE, 6327 },
	{ 0xeaf7, 0xeaf7, PDF_CMAP_SINGLE, 4579 },
	{ 0xeaf8, 0xeafb, PDF_CMAP_RANGE, 6329 },
	{ 0xeafc, 0xeafe, PDF_CMAP_TABLE, 88 },
	{ 0xeba1, 0xeba3, PDF_CMAP_RANGE, 6334 },
	{ 0xeba4, 0xeba4, PDF_CMAP_SINGLE, 4584 },
	{ 0xeba5, 0xeba6, PDF_CMAP_RANGE, 6337 },
	{ 0xeba7, 0xebaa, PDF_CMAP_TABLE, 91 },
	{ 0xebab, 0xebb9, PDF_CMAP_RANGE, 6340 },
	{ 0xebba, 0xebbb, PDF_CMAP_RANGE, 4590 },
	{ 0xebbc, 0xebbd, PDF_CMAP_TABLE, 95 },
	{ 0xebbe, 0xebc0, PDF_CMAP_RANGE, 6356 },
	{ 0xebc1, 0xebc2, PDF_CMAP_TABLE, 97 },
	{ 0xebc3, 0xebc5, PDF_CMAP_RANGE, 6359 },
	{ 0xebc6, 0xebc7, PDF_CMAP_RANGE, 4596 },
	{ 0xebc8, 0xebcb, PDF_CMAP_RANGE, 6362 },
	{ 0xebcc, 0xebcc, PDF_CMAP_SINGLE, 4599 },
	{ 0xebcd, 0xebce, PDF_CMAP_RANGE, 6366 },
	{ 0xebcf, 0xebd1, PDF_CMAP_RANGE, 4600 },
	{ 0xebd2, 0xebd2, PDF_CMAP_SINGLE, 5552 },
	{ 0xebd3, 0xebd7, PDF_CMAP_RANGE, 6368 },
	{ 0xebd8, 0xebd8, PDF_CMAP_SINGLE, 4603 },
	{ 0xebd9, 0xebfe, PDF_CMAP_RANGE, 6373 },
	{ 0xeca1, 0xeca5, PDF_CMAP_RANGE, 6411 },
	{ 0xeca6, 0xeca7, PDF_CMAP_TABLE, 99 },
	{ 0xeca8, 0xeca9, PDF_CMAP_RANGE, 6416 },
	{ 0xecaa, 0xecaa, PDF_CMAP_SINGLE, 4618 },
	{ 0xecab, 0xecae, PDF_CMAP_RANGE, 6418 },
	{ 0xecaf, 0xecaf, PDF_CMAP_SINGLE, 5950 },
	{ 0xecb0, 0xecb1, PDF_CMAP_RANGE, 4620 },
	{ 0xecb2, 0xecb2, PDF_CMAP_SINGLE, 4187 },
	{ 0xecb3, 0xecb4, PDF_CMAP_RANGE, 6422 },
	{ 0xecb5, 0xecb5, PDF_CMAP_SINGLE, 4625 },
	{ 0xecb6, 0xecb7, PDF_CMAP_RANGE, 6424 },
	{ 0xecb8, 0xecba, PDF_CMAP_TABLE, 101 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc6, PDF_CMAP_TABLE, 104 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecfa, PDF_CMAP_TABLE, 106 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f6, PDF_CMAP_TABLE, 108 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa2, PDF_CMAP_TABLE, 111 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
	{ 0xecbb, 0xecbf, PDF_CMAP_RANGE, 6427 },
	{ 0xecc0, 0xecc1, PDF_CMAP_RANGE, 4632 },
	{ 0xecc2, 0xecc4, PDF_CMAP_RANGE, 6432 },
	{ 0xecc5, 0xecc5, PDF_CMAP_SINGLE, 4634 },
	{ 0xecc6, 0xecc6, PDF_CMAP_SINGLE, 4636 },
	{ 0xecc7, 0xecc8, PDF_CMAP_RANGE, 6435 },
	{ 0xecc9, 0xecca, PDF_CMAP_RANGE, 4188 },
	{ 0xeccb, 0xecd4, PDF_CMAP_RANGE, 6437 },
	{ 0xecd5, 0xecd5, PDF_CMAP_SINGLE, 4638 },
	{ 0xecd6, 0xecdc, PDF_CMAP_RANGE, 6447 },
	{ 0xecdd, 0xecde, PDF_CMAP_RANGE, 4640 },
	{ 0xecdf, 0xece0, PDF_CMAP_RANGE, 6454 },
	{ 0xece1, 0xece1, PDF_CMAP_SINGLE, 4642 },
	{ 0xece2, 0xece3, PDF_CMAP_RANGE, 6456 },
	{ 0xece4, 0xece4, PDF_CMAP_SINGLE, 4644 },
	{ 0xece5, 0xece6, PDF_CMAP_RANGE, 6458 },
	{ 0xece7, 0xece8, PDF_CMAP_RANGE, 4645 },
	{ 0xece9, 0xecf6, PDF_CMAP_RANGE, 6460 },
	{ 0xecf7, 0xecf8, PDF_CMAP_RANGE, 4647 },
	{ 0xecf9, 0xecf9, PDF_CMAP_SINGLE, 6474 },
	{ 0xecfa, 0xecfa, PDF_CMAP_SINGLE, 4650 },
	{ 0xecfb, 0xecfe, PDF_CMAP_RANGE, 6475 },
	{ 0xeda1, 0xeda3, PDF_CMAP_RANGE, 4653 },
	{ 0xeda4, 0xeded, PDF_CMAP_RANGE, 6479 },
	{ 0xedee, 0xedee, PDF_CMAP_SINGLE, 5351 },
	{ 0xedef, 0xedfe, PDF_CMAP_RANGE, 6553 },
	{ 0xeea1, 0xeeda, PDF_CMAP_RANGE, 6569 },
	{ 0xeedb, 0xeedb, PDF_CMAP_SINGLE, 6494 },
	{ 0xeedc, 0xeefe, PDF_CMAP_RANGE, 6627 },
	{ 0xefa1, 0xeffe, PDF_CMAP_RANGE, 6662 },
	{ 0xf0a1, 0xf0fe, PDF_CMAP_RANGE, 6756 },
	{ 0xf1a1, 0xf1fe, PDF_CMAP_RANGE, 6850 },
	{ 0xf2a1, 0xf2bc, PDF_CMAP_RANGE, 6944 },
	{ 0xf2bd, 0xf2bd, PDF_CMAP_SINGLE, 5731 },
	{ 0xf2be, 0xf2f9, PDF_CMAP_RANGE, 6972 },
	{ 0xf2fa, 0xf2fa, PDF_CMAP_SINGLE, 5771 },
	{ 0xf2fb, 0xf2fe, PDF_CMAP_RANGE, 7032 },
	{ 0xf3a1, 0xf3b0, PDF_CMAP_RANGE, 7036 },
	{ 0xf3b1, 0xf3b1, PDF_CMAP_SINGLE, 4191 },
	{ 0xf3b2, 0xf3fe, PDF_CMAP_RANGE, 7052 },
	{ 0xf4a1, 0xf4a6, PDF_CMAP_RANGE, 7129 },
	{ 0xf4a7, 0xf4a7, PDF_CMAP_SINGLE, 6484 },
	{ 0xf4a8, 0xf4ed, PDF_CMAP_RANGE, 7135 },
	{ 0xf4ee, 0xf4ee, PDF_CMAP_SINGLE, 6684 },
	{ 0xf4ef, 0xf4fe, PDF_CMAP_RANGE, 7205 },
	{ 0xf5a1, 0xf5fe, PDF_CMAP_RANGE, 7221 },
	{ 0xf6a1, 0xf6f3, PDF_CMAP_RANGE, 7315 },
	{ 0xf6f4, 0xf6f4, PDF_CMAP_SINGLE, 4279 },
	{ 0xf6f5, 0xf6f5, PDF_CMAP_SINGLE, 7398 },
	{ 0xf6f6, 0xf6f6, PDF_CMAP_SINGLE, 7139 },
	{ 0xf6f7, 0xf6fe, PDF_CMAP_RANGE, 7399 },
	{ 0xf7a1, 0xf7b7, PDF_CMAP_RANGE, 7407 },
	{ 0xf7b8, 0xf7b8, PDF_CMAP_SINGLE, 4247 },
	{ 0xf7b9, 0xf7c7, PDF_CMAP_RANGE, 7430 },
	{ 0xf7c8, 0xf7c8, PDF_CMAP_SINGLE, 4266 },
	{ 0xf7c9, 0xf7d2, PDF_CMAP_RANGE, 7445 },
	{ 0xf7d3, 0xf7d3, PDF_CMAP_SINGLE, 4340 },
	{ 0xf7d4, 0xf7fe, PDF_CMAP_RANGE, 7455 },
	{ 0xf8a1, 0xf8da, PDF_CMAP_RANGE, 7498 },
	{ 0xf8db, 0xf8db, PDF_CMAP_SINGLE, 7572 },
	{ 0xf8dc, 0xf8ef, PDF_CMAP_RANGE, 7556 },
	{ 0xf8f0, 0xf8f0, PDF_CMAP_SINGLE, 5088 },
	{ 0xf8f1, 0xf8fe, PDF_CMAP_RANGE, 7576 },
	{ 0xf9a1, 0xf9fe, PDF_CMAP_RANGE, 7590 },
	{ 0xfaa1, 0xfaa1, PDF_CMAP_SINGLE, 7709 },
	{ 0xfaa2, 0xfaa2, PDF_CMAP_SINGLE, 3558 },
	{ 0xfaa3, 0xfae5, PDF_CMAP_RANGE, 7684 },
	{ 0xfae6, 0xfae6, PDF_CMAP_SINGLE, 3644 },
	{ 0xfae7, 0xfafe, PDF_CMAP_RANGE, 7751 },
	{ 0xfba1, 0xfbfe, PDF_CMAP_RANGE, 7775 },
	{ 0xfca1, 0xfca8, PDF_CMAP_RANGE, 7869 },
	{ 0xfca9, 0xfca9, PDF_CMAP_SINGLE, 3815 },
	{ 0xfcaa, 0xfcfe, PDF_CMAP_RANGE, 7877 },
	{ 0xfda1, 0xfdfe, PDF_CMAP_RANGE, 7962 },
};

static const unsigned short pdf_cmap_KSCpc_EUC_H_table[113] =
{
	3993,3946,4374,4156,4138,5800,4386,4139,
	4140,4391,4141,4142,4396,4403,4149,4419,
	4154,4420,4517,4173,4524,4174,4179,4547,
	4550,4180,4564,5584,5302,5367,4427,5879,
	4435,5896,4436,5897,4437,4159,5936,4444,
	4453,5946,5947,4458,4160,5961,4471,5980,
	4472,4474,4473,4475,5983,4476,5984,4481,
	5987,4482,5460,4483,4163,4166,5990,4485,
	4489,4491,6002,4167,4493,4495,6019,7783,
	4497,4499,4501,6027,4502,6028,4509,6046,
	4510,4553,4180,4555,6152,5800,6153,4556,
	4581,6333,4582,4586,6339,4588,4184,6355,
	4592,4593,4595,4614,4616,4627,6426,4629,
	4634,4636,6474,4650,4279,7398,7139,7709,
	3558,
};

pdf_cmap pdf_cmap_KSCpc_EUC_H =
{
	-1, "KSCpc-EUC-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 1, 0x0000, 0x0084 },
		{ 2, 0xa141, 0xfdfe },
		{ 1, 0x00fe, 0x00ff },
	},
	509, 509, (pdf_range*) pdf_cmap_KSCpc_EUC_H_ranges,
	113, 113, (unsigned short*) pdf_cmap_KSCpc_EUC_H_table,
};

/* KSCpc-EUC-V */

static const pdf_range pdf_cmap_KSCpc_EUC_V_ranges[] =
{
	{ 0xa1a2, 0xa1a3, PDF_CMAP_RANGE, 8056 },
	{ 0xa1a5, 0xa1a6, PDF_CMAP_TABLE, 0 },
	{ 0xa1a9, 0xa1ab, PDF_CMAP_RANGE, 8059 },
	{ 0xa1ad, 0xa1ad, PDF_CMAP_SINGLE, 8062 },
	{ 0xa1b2, 0xa1bd, PDF_CMAP_RANGE, 8063 },
	{ 0xa1eb, 0xa1eb, PDF_CMAP_SINGLE, 8075 },
	{ 0xa3a1, 0xa3a1, PDF_CMAP_SINGLE, 8076 },
	{ 0xa3a8, 0xa3a9, PDF_CMAP_RANGE, 8077 },
	{ 0xa3ac, 0xa3ac, PDF_CMAP_SINGLE, 8079 },
	{ 0xa3ae, 0xa3ae, PDF_CMAP_SINGLE, 8080 },
	{ 0xa3ba, 0xa3bf, PDF_CMAP_RANGE, 8081 },
	{ 0xa3db, 0xa3db, PDF_CMAP_SINGLE, 8087 },
	{ 0xa3dd, 0xa3dd, PDF_CMAP_SINGLE, 8088 },
	{ 0xa3df, 0xa3df, PDF_CMAP_SINGLE, 8089 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
	{ 0xa3fb, 0xa3fe, PDF_CMAP_RANGE, 8090 },
};

static const unsigned short pdf_cmap_KSCpc_EUC_V_table[2] =
{
	8058,8320,
};

pdf_cmap pdf_cmap_KSCpc_EUC_V =
{
	-1, "KSCpc-EUC-V", "KSCpc-EUC-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	16, 16, (pdf_range*) pdf_cmap_KSCpc_EUC_V_ranges,
	2, 2, (unsigned short*) pdf_cmap_KSCpc_EUC_V_table,
};

/* UniKS-UCS2-H */

static const pdf_range pdf_cmap_UniKS_UCS2_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a1, 0x00a1, PDF_CMAP_SINGLE, 208 },
	{ 0x00a4, 0x00a4, PDF_CMAP_SINGLE, 214 },
	{ 0x00a7, 0x00a8, PDF_CMAP_TABLE, 0 },
	{ 0x00aa, 0x00ab, PDF_CMAP_TABLE, 2 },
	{ 0x00b0, 0x00b1, PDF_CMAP_TABLE, 4 },
	{ 0x00b2, 0x00b3, PDF_CMAP_RANGE, 843 },
	{ 0x00b4, 0x00b4, PDF_CMAP_SINGLE, 199 },
	{ 0x00b6, 0x00b6, PDF_CMAP_SINGLE, 244 },
	{ 0x00b8, 0x00bf, PDF_CMAP_TABLE, 6 },
	{ 0x00c6, 0x00c6, PDF_CMAP_SINGLE, 666 },
	{ 0x00d0, 0x00d0, PDF_CMAP_SINGLE, 667 },
	{ 0x00d7, 0x00d8, PDF_CMAP_TABLE, 14 },
	{ 0x00de, 0x00df, PDF_CMAP_TABLE, 16 },
	{ 0x00e6, 0x00e6, PDF_CMAP_SINGLE, 757 },
	{ 0x00f0, 0x00f0, PDF_CMAP_SINGLE, 759 },
	{ 0x00f7, 0x00f8, PDF_CMAP_TABLE, 18 },
	{ 0x00fe, 0x00fe, PDF_CMAP_SINGLE, 769 },
	{ 0x0111, 0x0111, PDF_CMAP_SINGLE, 758 },
	{ 0x0126, 0x0127, PDF_CMAP_TABLE, 20 },
	{ 0x0131, 0x0133, PDF_CMAP_TABLE, 22 },
	{ 0x0138, 0x0138, PDF_CMAP_SINGLE, 763 },
	{ 0x013f, 0x0142, PDF_CMAP_TABLE, 25 },
	{ 0x0149, 0x014b, PDF_CMAP_TABLE, 29 },
	{ 0x0152, 0x0153, PDF_CMAP_TABLE, 32 },
	{ 0x0166, 0x0167, PDF_CMAP_TABLE, 34 },
	{ 0x02bc, 0x02bc, PDF_CMAP_SINGLE, 8275 },
	{ 0x02c7, 0x02c7, PDF_CMAP_SINGLE, 201 },
	{ 0x02d8, 0x02dd, PDF_CMAP_TABLE, 36 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 471 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 488 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 495 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 512 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 1026 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 1020 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 1027 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 1060 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 1059 },
	{ 0x2013, 0x2014, PDF_CMAP_RANGE, 109 },
	{ 0x2016, 0x2016, PDF_CMAP_SINGLE, 111 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 114 },
	{ 0x201b, 0x201b, PDF_CMAP_SINGLE, 8238 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 116 },
	{ 0x201f, 0x201f, PDF_CMAP_SINGLE, 8237 },
	{ 0x2020, 0x2021, PDF_CMAP_RANGE, 245 },
	{ 0x2022, 0x2022, PDF_CMAP_SINGLE, 8607 },
	{ 0x2025, 0x2026, PDF_CMAP_RANGE, 105 },
	{ 0x2030, 0x2030, PDF_CMAP_SINGLE, 216 },
	{ 0x2032, 0x2033, PDF_CMAP_RANGE, 139 },
	{ 0x2034, 0x2036, PDF_CMAP_TABLE, 42 },
	{ 0x2039, 0x203a, PDF_CMAP_RANGE, 8612 },
	{ 0x203b, 0x203c, PDF_CMAP_TABLE, 45 },
	{ 0x2042, 0x2042, PDF_CMAP_SINGLE, 8599 },
	{ 0x2074, 0x2074, PDF_CMAP_SINGLE, 845 },
	{ 0x207a, 0x207b, PDF_CMAP_RANGE, 8239 },
	{ 0x207c, 0x207c, PDF_CMAP_SINGLE, 8248 },
	{ 0x207d, 0x207e, PDF_CMAP_RANGE, 8250 },
	{ 0x207f, 0x207f, PDF_CMAP_SINGLE, 846 },
	{ 0x2081, 0x2084, PDF_CMAP_RANGE, 847 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 141 },
	{ 0x2109, 0x2109, PDF_CMAP_SINGLE, 215 },
	{ 0x2113, 0x2113, PDF_CMAP_SINGLE, 590 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 258 },
	{ 0x2121, 0x2122, PDF_CMAP_TABLE, 47 },
	{ 0x2126, 0x2126, PDF_CMAP_SINGLE, 643 },
	{ 0x212b, 0x212b, PDF_CMAP_SINGLE, 142 },
	{ 0x2153, 0x2154, PDF_CMAP_RANGE, 749 },
	{ 0x215b, 0x215e, PDF_CMAP_RANGE, 753 },
	{ 0x2160, 0x2169, PDF_CMAP_RANGE, 461 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 451 },
	{ 0x2190, 0x2191, PDF_CMAP_RANGE, 171 },
	{ 0x2192, 0x2192, PDF_CMAP_SINGLE, 170 },
	{ 0x2193, 0x2194, PDF_CMAP_RANGE, 173 },
	{ 0x2195, 0x2199, PDF_CMAP_TABLE, 49 },
	{ 0x21b0, 0x21b4, PDF_CMAP_TABLE, 54 },
	{ 0x21bc, 0x21bc, PDF_CMAP_SINGLE, 8884 },
	{ 0x21c0, 0x21c0, PDF_CMAP_SINGLE, 8885 },
	{ 0x21c4, 0x21c5, PDF_CMAP_RANGE, 8896 },
	{ 0x21cd, 0x21cd, PDF_CMAP_SINGLE, 8816 },
	{ 0x21cf, 0x21d4, PDF_CMAP_TABLE, 59 },
	{ 0x21e0, 0x21e3, PDF_CMAP_TABLE, 65 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 69 },
	{ 0x2200, 0x2200, PDF_CMAP_SINGLE, 197 },
	{ 0x2202, 0x2203, PDF_CMAP_TABLE, 73 },
	{ 0x2206, 0x2209, PDF_CMAP_TABLE, 75 },
	{ 0x220b, 0x220c, PDF_CMAP_TABLE, 79 },
	{ 0x220f, 0x220f, PDF_CMAP_SINGLE, 213 },
	{ 0x2211, 0x2211, PDF_CMAP_SINGLE, 212 },
	{ 0x2213, 0x2213, PDF_CMAP_SINGLE, 8726 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 178 },
	{ 0x221d, 0x2220, PDF_CMAP_TABLE, 81 },
	{ 0x2222, 0x2222, PDF_CMAP_SINGLE, 8738 },
	{ 0x2225, 0x2226, PDF_CMAP_RANGE, 8719 },
	{ 0x2227, 0x2228, PDF_CMAP_RANGE, 192 },
	{ 0x2229, 0x222a, PDF_CMAP_TABLE, 85 },
	{ 0x222b, 0x222c, PDF_CMAP_RANGE, 182 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 211 },
	{ 0x2234, 0x2237, PDF_CMAP_TABLE, 87 },
	{ 0x223d, 0x223d, PDF_CMAP_SINGLE, 179 },
	{ 0x2243, 0x2243, PDF_CMAP_SINGLE, 8500 },
	{ 0x2245, 0x2245, PDF_CMAP_SINGLE, 8499 },
	{ 0x2248, 0x2248, PDF_CMAP_SINGLE, 8501 },
	{ 0x2250, 0x2253, PDF_CMAP_TABLE, 91 },
	{ 0x225a, 0x225a, PDF_CMAP_SINGLE, 8753 },
	{ 0x2260, 0x2262, PDF_CMAP_TABLE, 95 },
	{ 0x2264, 0x2265, PDF_CMAP_RANGE, 134 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 8724 },
	{ 0x226e, 0x226f, PDF_CMAP_RANGE, 8745 },
	{ 0x2270, 0x2273, PDF_CMAP_RANGE, 8481 },
	{ 0x2276, 0x2277, PDF_CMAP_RANGE, 8489 },
	{ 0x2279, 0x2279, PDF_CMAP_SINGLE, 8491 },
	{ 0x227a, 0x227b, PDF_CMAP_RANGE, 8475 },
	{ 0x2280, 0x2281, PDF_CMAP_RANGE, 8479 },
	{ 0x2282, 0x2283, PDF_CMAP_RANGE, 188 },
	{ 0x2284, 0x2285, PDF_CMAP_TABLE, 98 },
	{ 0x2286, 0x2287, PDF_CMAP_RANGE, 186 },
	{ 0x228a, 0x228b, PDF_CMAP_TABLE, 100 },
	{ 0x2295, 0x2297, PDF_CMAP_RANGE, 8727 },
	{ 0x22a3, 0x22a5, PDF_CMAP_TABLE, 102 },
	{ 0x22bb, 0x22bc, PDF_CMAP_RANGE, 8751 },
	{ 0x22ce, 0x22cf, PDF_CMAP_RANGE, 8477 },
	{ 0x22da, 0x22db, PDF_CMAP_RANGE, 8492 },
	{ 0x22ee, 0x22ef, PDF_CMAP_TABLE, 105 },
	{ 0x2306, 0x2306, PDF_CMAP_SINGLE, 8754 },
	{ 0x2312, 0x2312, PDF_CMAP_SINGLE, 150 },
	{ 0x2314, 0x2314, PDF_CMAP_SINGLE, 8731 },
	{ 0x2460, 0x246e, PDF_CMAP_RANGE, 733 },
	{ 0x246f, 0x2473, PDF_CMAP_RANGE, 8791 },
	{ 0x2474, 0x2482, PDF_CMAP_RANGE, 827 },
	{ 0x2483, 0x2487, PDF_CMAP_RANGE, 9042 },
	{ 0x249c, 0x24b5, PDF_CMAP_RANGE, 801 },
	{ 0x24b6, 0x24cf, PDF_CMAP_RANGE, 8388 },
	{ 0x24d0, 0x24e9, PDF_CMAP_RANGE, 707 },
	{ 0x2500, 0x2503, PDF_CMAP_TABLE, 107 },
	{ 0x250c, 0x251d, PDF_CMAP_TABLE, 111 },
	{ 0x251e, 0x251f, PDF_CMAP_RANGE, 559 },
	{ 0x2520, 0x2520, PDF_CMAP_SINGLE, 541 },
	{ 0x2521, 0x2522, PDF_CMAP_RANGE, 561 },
	{ 0x2523, 0x2525, PDF_CMAP_TABLE, 129 },
	{ 0x2526, 0x2527, PDF_CMAP_RANGE, 563 },
	{ 0x2528, 0x2528, PDF_CMAP_SINGLE, 543 },
	{ 0x2529, 0x252a, PDF_CMAP_RANGE, 565 },
	{ 0x252b, 0x252c, PDF_CMAP_TABLE, 132 },
	{ 0x252d, 0x252e, PDF_CMAP_RANGE, 567 },
	{ 0x252f, 0x2530, PDF_CMAP_TABLE, 134 },
	{ 0x2531, 0x2532, PDF_CMAP_RANGE, 569 },
	{ 0x2533, 0x2534, PDF_CMAP_TABLE, 136 },
	{ 0x2535, 0x2536, PDF_CMAP_RANGE, 571 },
	{ 0x2537, 0x2538, PDF_CMAP_TABLE, 138 },
	{ 0x2539, 0x253a, PDF_CMAP_RANGE, 573 },
	{ 0x253b, 0x253c, PDF_CMAP_TABLE, 140 },
	{ 0x253d, 0x253e, PDF_CMAP_RANGE, 575 },
	{ 0x253f, 0x253f, PDF_CMAP_SINGLE, 545 },
	{ 0x2540, 0x2541, PDF_CMAP_RANGE, 577 },
	{ 0x2542, 0x2542, PDF_CMAP_SINGLE, 550 },
	{ 0x2543, 0x254a, PDF_CMAP_RANGE, 579 },
	{ 0x254b, 0x254b, PDF_CMAP_SINGLE, 540 },
	{ 0x2592, 0x2592, PDF_CMAP_SINGLE, 232 },
	{ 0x25a0, 0x25a1, PDF_CMAP_TABLE, 142 },
	{ 0x25a3, 0x25a3, PDF_CMAP_SINGLE, 229 },
	{ 0x25a4, 0x25a5, PDF_CMAP_RANGE, 233 },
	{ 0x25a6, 0x25a9, PDF_CMAP_TABLE, 144 },
	{ 0x25b1, 0x25b3, PDF_CMAP_TABLE, 148 },
	{ 0x25b5, 0x25b7, PDF_CMAP_TABLE, 151 },
	{ 0x25b9, 0x25b9, PDF_CMAP_SINGLE, 8781 },
	{ 0x25bc, 0x25bd, PDF_CMAP_TABLE, 154 },
	{ 0x25bf, 0x25c1, PDF_CMAP_TABLE, 156 },
	{ 0x25c3, 0x25c3, PDF_CMAP_SINGLE, 8782 },
	{ 0x25c6, 0x25cc, PDF_CMAP_TABLE, 159 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 166 },
	{ 0x25d0, 0x25d1, PDF_CMAP_RANGE, 230 },
	{ 0x25e6, 0x25e6, PDF_CMAP_SINGLE, 8775 },
	{ 0x25ef, 0x25ef, PDF_CMAP_SINGLE, 8633 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 168 },
	{ 0x260e, 0x260f, PDF_CMAP_TABLE, 170 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 172 },
	{ 0x262f, 0x262f, PDF_CMAP_SINGLE, 8664 },
	{ 0x2640, 0x2640, PDF_CMAP_SINGLE, 147 },
	{ 0x2642, 0x2642, PDF_CMAP_SINGLE, 146 },
	{ 0x2660, 0x2661, PDF_CMAP_RANGE, 222 },
	{ 0x2663, 0x2665, PDF_CMAP_TABLE, 176 },
	{ 0x2667, 0x2668, PDF_CMAP_TABLE, 179 },
	{ 0x2669, 0x266a, PDF_CMAP_RANGE, 253 },
	{ 0x266c, 0x266d, PDF_CMAP_TABLE, 181 },
	{ 0x266f, 0x266f, PDF_CMAP_SINGLE, 8594 },
	{ 0x2716, 0x2716, PDF_CMAP_SINGLE, 8631 },
	{ 0x271a, 0x271a, PDF_CMAP_SINGLE, 8630 },
	{ 0x273d, 0x273d, PDF_CMAP_SINGLE, 8604 },
	{ 0x2756, 0x2756, PDF_CMAP_SINGLE, 8637 },
	{ 0x2776, 0x277f, PDF_CMAP_RANGE, 8673 },
	{ 0x278a, 0x2793, PDF_CMAP_RANGE, 8342 },
	{ 0x3000, 0x3002, PDF_CMAP_RANGE, 101 },
	{ 0x3003, 0x3003, PDF_CMAP_SINGLE, 108 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 120 },
	{ 0x3012, 0x3013, PDF_CMAP_TABLE, 183 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 118 },
	{ 0x3016, 0x3019, PDF_CMAP_RANGE, 8219 },
	{ 0x301e, 0x301f, PDF_CMAP_RANGE, 9322 },
	{ 0x3020, 0x3020, PDF_CMAP_SINGLE, 8671 },
	{ 0x3036, 0x3036, PDF_CMAP_SINGLE, 8701 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 851 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 934 },
	{ 0x30fb, 0x30fc, PDF_CMAP_TABLE, 185 },
	{ 0x3131, 0x3163, PDF_CMAP_RANGE, 358 },
	{ 0x3164, 0x3164, PDF_CMAP_SINGLE, 101 },
	{ 0x3165, 0x318e, PDF_CMAP_RANGE, 409 },
	{ 0x3200, 0x321b, PDF_CMAP_RANGE, 773 },
	{ 0x321c, 0x321c, PDF_CMAP_SINGLE, 257 },
	{ 0x3231, 0x3231, PDF_CMAP_SINGLE, 8788 },
	{ 0x3239, 0x3239, PDF_CMAP_SINGLE, 8789 },
	{ 0x3260, 0x327b, PDF_CMAP_RANGE, 679 },
	{ 0x327f, 0x327f, PDF_CMAP_SINGLE, 256 },
	{ 0x328a, 0x328f, PDF_CMAP_RANGE, 9301 },
	{ 0x3290, 0x3290, PDF_CMAP_SINGLE, 9300 },
	{ 0x3294, 0x3294, PDF_CMAP_SINGLE, 9080 },
	{ 0x329e, 0x329e, PDF_CMAP_SINGLE, 8761 },
	{ 0x32a5, 0x32a5, PDF_CMAP_SINGLE, 9096 },
	{ 0x3380, 0x3384, PDF_CMAP_RANGE, 627 },
	{ 0x3388, 0x3389, PDF_CMAP_RANGE, 612 },
	{ 0x338a, 0x338c, PDF_CMAP_RANGE, 646 },
	{ 0x338d, 0x338f, PDF_CMAP_RANGE, 608 },
	{ 0x3390, 0x3394, PDF_CMAP_RANGE, 638 },
	{ 0x3395, 0x3397, PDF_CMAP_RANGE, 587 },
	{ 0x3398, 0x3398, PDF_CMAP_SINGLE, 591 },
	{ 0x3399, 0x33a2, PDF_CMAP_RANGE, 597 },
	{ 0x33a3, 0x33a6, PDF_CMAP_RANGE, 593 },
	{ 0x33a7, 0x33a8, PDF_CMAP_RANGE, 615 },
	{ 0x33a9, 0x33ac, PDF_CMAP_RANGE, 655 },
	{ 0x33ad, 0x33af, PDF_CMAP_RANGE, 651 },
	{ 0x33b0, 0x33b9, PDF_CMAP_RANGE, 617 },
	{ 0x33ba, 0x33bf, PDF_CMAP_RANGE, 632 },
	{ 0x33c0, 0x33c1, PDF_CMAP_RANGE, 644 },
	{ 0x33c2, 0x33cb, PDF_CMAP_TABLE, 187 },
	{ 0x33cf, 0x33d0, PDF_CMAP_TABLE, 197 },
	{ 0x33d3, 0x33d3, PDF_CMAP_SINGLE, 661 },
	{ 0x33d6, 0x33d6, PDF_CMAP_SINGLE, 649 },
	{ 0x33d8, 0x33d8, PDF_CMAP_SINGLE, 262 },
	{ 0x33db, 0x33dd, PDF_CMAP_TABLE, 199 },
	{ 0x4e00, 0x4e01, PDF_CMAP_TABLE, 202 },
	{ 0x4e03, 0x4e03, PDF_CMAP_SINGLE, 7364 },
	{ 0x4e07, 0x4e0b, PDF_CMAP_TABLE, 204 },
	{ 0x4e0d, 0x4e0d, PDF_CMAP_SINGLE, 5109 },
	{ 0x4e11, 0x4e11, PDF_CMAP_SINGLE, 7288 },
	{ 0x4e14, 0x4e16, PDF_CMAP_TABLE, 209 },
	{ 0x4e18, 0x4e19, PDF_CMAP_TABLE, 212 },
	{ 0x4e1e, 0x4e1e, PDF_CMAP_SINGLE, 5682 },
	{ 0x4e2d, 0x4e2d, PDF_CMAP_SINGLE, 6922 },
	{ 0x4e32, 0x4e32, PDF_CMAP_SINGLE, 3802 },
	{ 0x4e38, 0x4e39, PDF_CMAP_TABLE, 214 },
	{ 0x4e3b, 0x4e3b, PDF_CMAP_SINGLE, 6860 },
	{ 0x4e42, 0x4e43, PDF_CMAP_TABLE, 216 },
	{ 0x4e45, 0x4e45, PDF_CMAP_SINGLE, 3894 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 6942 },
	{ 0x4e4d, 0x4e4f, PDF_CMAP_TABLE, 218 },
	{ 0x4e56, 0x4e56, PDF_CMAP_SINGLE, 3855 },
	{ 0x4e58, 0x4e59, PDF_CMAP_TABLE, 221 },
	{ 0x4e5d, 0x4e5f, PDF_CMAP_TABLE, 223 },
	{ 0x4e6b, 0x4e6b, PDF_CMAP_SINGLE, 3500 },
	{ 0x4e6d, 0x4e6d, PDF_CMAP_SINGLE, 4329 },
	{ 0x4e73, 0x4e73, PDF_CMAP_SINGLE, 6309 },
	{ 0x4e76, 0x4e77, PDF_CMAP_TABLE, 226 },
	{ 0x4e7e, 0x4e7e, PDF_CMAP_SINGLE, 3601 },
	{ 0x4e82, 0x4e82, PDF_CMAP_SINGLE, 4389 },
	{ 0x4e86, 0x4e86, PDF_CMAP_SINGLE, 4551 },
	{ 0x4e88, 0x4e88, PDF_CMAP_SINGLE, 5934 },
	{ 0x4e8b, 0x4e8c, PDF_CMAP_TABLE, 228 },
	{ 0x4e8e, 0x4e8e, PDF_CMAP_SINGLE, 6197 },
	{ 0x4e90, 0x4e92, PDF_CMAP_TABLE, 230 },
	{ 0x4e94, 0x4e95, PDF_CMAP_TABLE, 233 },
	{ 0x4e98, 0x4e98, PDF_CMAP_SINGLE, 4058 },
	{ 0x4e9b, 0x4e9b, PDF_CMAP_SINGLE, 5243 },
	{ 0x4e9e, 0x4e9e, PDF_CMAP_SINGLE, 5775 },
	{ 0x4ea1, 0x4ea2, PDF_CMAP_TABLE, 235 },
	{ 0x4ea4, 0x4ea6, PDF_CMAP_TABLE, 237 },
	{ 0x4ea8, 0x4ea8, PDF_CMAP_SINGLE, 7771 },
	{ 0x4eab, 0x4eae, PDF_CMAP_TABLE, 240 },
	{ 0x4eb6, 0x4eb6, PDF_CMAP_SINGLE, 4193 },
	{ 0x4eba, 0x4eba, PDF_CMAP_SINGLE, 6443 },
	{ 0x4ec0, 0x4ec1, PDF_CMAP_TABLE, 244 },
	{ 0x4ec4, 0x4ec4, PDF_CMAP_SINGLE, 7331 },
	{ 0x4ec7, 0x4ec7, PDF_CMAP_SINGLE, 3896 },
	{ 0x4eca, 0x4ecb, PDF_CMAP_TABLE, 246 },
	{ 0x4ecd, 0x4ecd, PDF_CMAP_SINGLE, 6479 },
	{ 0x4ed4, 0x4ed9, PDF_CMAP_TABLE, 248 },
	{ 0x4edd, 0x4edd, PDF_CMAP_SINGLE, 4331 },
	{ 0x4edf, 0x4edf, PDF_CMAP_SINGLE, 7148 },
	{ 0x4ee3, 0x4ee5, PDF_CMAP_TABLE, 254 },
	{ 0x4ef0, 0x4ef0, PDF_CMAP_SINGLE, 5833 },
	{ 0x4ef2, 0x4ef2, PDF_CMAP_SINGLE, 6923 },
	{ 0x4ef6, 0x4ef7, PDF_CMAP_TABLE, 257 },
	{ 0x4efb, 0x4efb, PDF_CMAP_SINGLE, 6469 },
	{ 0x4f01, 0x4f01, PDF_CMAP_SINGLE, 4062 },
	{ 0x4f09, 0x4f0b, PDF_CMAP_TABLE, 259 },
	{ 0x4f0d, 0x4f11, PDF_CMAP_TABLE, 262 },
	{ 0x4f2f, 0x4f2f, PDF_CMAP_SINGLE, 4988 },
	{ 0x4f34, 0x4f34, PDF_CMAP_SINGLE, 4905 },
	{ 0x4f36, 0x4f36, PDF_CMAP_SINGLE, 4490 },
	{ 0x4f38, 0x4f38, PDF_CMAP_SINGLE, 5735 },
	{ 0x4f3a, 0x4f3a, PDF_CMAP_SINGLE, 5245 },
	{ 0x4f3c, 0x4f3d, PDF_CMAP_TABLE, 267 },
	{ 0x4f43, 0x4f43, PDF_CMAP_SINGLE, 6643 },
	{ 0x4f46, 0x4f48, PDF_CMAP_TABLE, 269 },
	{ 0x4f4d, 0x4f51, PDF_CMAP_TABLE, 272 },
	{ 0x4f55, 0x4f55, PDF_CMAP_SINGLE, 7617 },
	{ 0x4f59, 0x4f5c, PDF_CMAP_TABLE, 277 },
	{ 0x4f69, 0x4f69, PDF_CMAP_SINGLE, 7502 },
	{ 0x4f6f, 0x4f70, PDF_CMAP_TABLE, 281 },
	{ 0x4f73, 0x4f73, PDF_CMAP_SINGLE, 3437 },
	{ 0x4f76, 0x4f76, PDF_CMAP_SINGLE, 4127 },
	{ 0x4f7a, 0x4f7a, PDF_CMAP_SINGLE, 6644 },
	{ 0x4f7e, 0x4f7f, PDF_CMAP_TABLE, 283 },
	{ 0x4f81, 0x4f81, PDF_CMAP_SINGLE, 5736 },
	{ 0x4f83, 0x4f84, PDF_CMAP_TABLE, 285 },
	{ 0x4f86, 0x4f86, PDF_CMAP_SINGLE, 4420 },
	{ 0x4f88, 0x4f88, PDF_CMAP_SINGLE, 7336 },
	{ 0x4f8a, 0x4f8b, PDF_CMAP_TABLE, 287 },
	{ 0x4f8d, 0x4f8d, PDF_CMAP_SINGLE, 5692 },
	{ 0x4f8f, 0x4f8f, PDF_CMAP_SINGLE, 6862 },
	{ 0x4f91, 0x4f91, PDF_CMAP_SINGLE, 6310 },
	{ 0x4f96, 0x4f96, PDF_CMAP_SINGLE, 4594 },
	{ 0x4f98, 0x4f98, PDF_CMAP_SINGLE, 7042 },
	{ 0x4f9b, 0x4f9b, PDF_CMAP_SINGLE, 3786 },
	{ 0x4f9d, 0x4f9d, PDF_CMAP_SINGLE, 6394 },
	{ 0x4fae, 0x4faf, PDF_CMAP_TABLE, 289 },
	{ 0x4fb5, 0x4fb6, PDF_CMAP_TABLE, 291 },
	{ 0x4fbf, 0x4fbf, PDF_CMAP_SINGLE, 7518 },
	{ 0x4fc2, 0x4fc4, PDF_CMAP_TABLE, 293 },
	{ 0x4fc9, 0x4fca, PDF_CMAP_TABLE, 296 },
	{ 0x4fce, 0x4fce, PDF_CMAP_SINGLE, 6782 },
	{ 0x4fd1, 0x4fd1, PDF_CMAP_SINGLE, 6174 },
	{ 0x4fd3, 0x4fd4, PDF_CMAP_TABLE, 298 },
	{ 0x4fd7, 0x4fd7, PDF_CMAP_SINGLE, 5537 },
	{ 0x4fda, 0x4fda, PDF_CMAP_SINGLE, 4613 },
	{ 0x4fdd, 0x4fdd, PDF_CMAP_SINGLE, 5058 },
	{ 0x4fdf, 0x4fe1, PDF_CMAP_TABLE, 300 },
	{ 0x4fee, 0x4fef, PDF_CMAP_TABLE, 303 },
	{ 0x4ff1, 0x4ff1, PDF_CMAP_SINGLE, 3897 },
	{ 0x4ff3, 0x4ff3, PDF_CMAP_SINGLE, 4970 },
	{ 0x4ff5, 0x4ff5, PDF_CMAP_SINGLE, 7576 },
	{ 0x4ff8, 0x4ff8, PDF_CMAP_SINGLE, 5093 },
	{ 0x4ffa, 0x4ffa, PDF_CMAP_SINGLE, 5925 },
	{ 0x5002, 0x5002, PDF_CMAP_SINGLE, 5042 },
	{ 0x5006, 0x5006, PDF_CMAP_SINGLE, 4428 },
	{ 0x5009, 0x5009, PDF_CMAP_SINGLE, 7092 },
	{ 0x500b, 0x500b, PDF_CMAP_SINGLE, 3562 },
	{ 0x500d, 0x500d, PDF_CMAP_SINGLE, 4969 },
	{ 0x5011, 0x5012, PDF_CMAP_TABLE, 305 },
	{ 0x5016, 0x5016, PDF_CMAP_SINGLE, 7705 },
	{ 0x5019, 0x501a, PDF_CMAP_TABLE, 307 },
	{ 0x501c, 0x501c, PDF_CMAP_SINGLE, 7134 },
	{ 0x501e, 0x501f, PDF_CMAP_TABLE, 309 },
	{ 0x5021, 0x5021, PDF_CMAP_SINGLE, 7093 },
	{ 0x5023, 0x5024, PDF_CMAP_TABLE, 311 },
	{ 0x5026, 0x5028, PDF_CMAP_TABLE, 313 },
	{ 0x502a, 0x502d, PDF_CMAP_TABLE, 316 },
	{ 0x503b, 0x503b, PDF_CMAP_SINGLE, 5863 },
	{ 0x5043, 0x5043, PDF_CMAP_SINGLE, 5917 },
	{ 0x5047, 0x5049, PDF_CMAP_TABLE, 320 },
	{ 0x504f, 0x504f, PDF_CMAP_SINGLE, 7519 },
	{ 0x5055, 0x5055, PDF_CMAP_SINGLE, 7686 },
	{ 0x505a, 0x505a, PDF_CMAP_SINGLE, 6863 },
	{ 0x505c, 0x505c, PDF_CMAP_SINGLE, 6707 },
	{ 0x5065, 0x5065, PDF_CMAP_SINGLE, 3603 },
	{ 0x5074, 0x5076, PDF_CMAP_TABLE, 323 },
	{ 0x5078, 0x5078, PDF_CMAP_SINGLE, 7465 },
	{ 0x5080, 0x5080, PDF_CMAP_SINGLE, 3856 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 5112 },
	{ 0x508d, 0x508d, PDF_CMAP_SINGLE, 4942 },
	{ 0x5091, 0x5091, PDF_CMAP_SINGLE, 3614 },
	{ 0x5098, 0x5099, PDF_CMAP_TABLE, 326 },
	{ 0x50ac, 0x50ad, PDF_CMAP_TABLE, 328 },
	{ 0x50b2, 0x50b3, PDF_CMAP_TABLE, 330 },
	{ 0x50b5, 0x50b5, PDF_CMAP_SINGLE, 7114 },
	{ 0x50b7, 0x50b7, PDF_CMAP_SINGLE, 5332 },
	{ 0x50be, 0x50be, PDF_CMAP_SINGLE, 3663 },
	{ 0x50c5, 0x50c5, PDF_CMAP_SINGLE, 4023 },
	{ 0x50c9, 0x50ca, PDF_CMAP_TABLE, 332 },
	{ 0x50cf, 0x50cf, PDF_CMAP_SINGLE, 5333 },
	{ 0x50d1, 0x50d1, PDF_CMAP_SINGLE, 3869 },
	{ 0x50d5, 0x50d6, PDF_CMAP_TABLE, 334 },
	{ 0x50da, 0x50da, PDF_CMAP_SINGLE, 4552 },
	{ 0x50de, 0x50de, PDF_CMAP_SINGLE, 6286 },
	{ 0x50e5, 0x50e5, PDF_CMAP_SINGLE, 6140 },
	{ 0x50e7, 0x50e7, PDF_CMAP_SINGLE, 5684 },
	{ 0x50ed, 0x50ed, PDF_CMAP_SINGLE, 7082 },
	{ 0x50f9, 0x50f9, PDF_CMAP_SINGLE, 3439 },
	{ 0x50fb, 0x50fb, PDF_CMAP_SINGLE, 5020 },
	{ 0x50ff, 0x5101, PDF_CMAP_TABLE, 336 },
	{ 0x5104, 0x5104, PDF_CMAP_SINGLE, 5912 },
	{ 0x5106, 0x5106, PDF_CMAP_SINGLE, 3664 },
	{ 0x5109, 0x5109, PDF_CMAP_SINGLE, 3617 },
	{ 0x5112, 0x5112, PDF_CMAP_SINGLE, 6311 },
	{ 0x511f, 0x511f, PDF_CMAP_SINGLE, 5334 },
	{ 0x5121, 0x5121, PDF_CMAP_SINGLE, 4543 },
	{ 0x512a, 0x512a, PDF_CMAP_SINGLE, 6200 },
	{ 0x5132, 0x5132, PDF_CMAP_SINGLE, 6593 },
	{ 0x5137, 0x5137, PDF_CMAP_SINGLE, 4441 },
	{ 0x513a, 0x513a, PDF_CMAP_SINGLE, 4133 },
	{ 0x513c, 0x513c, PDF_CMAP_SINGLE, 5926 },
	{ 0x5140, 0x5141, PDF_CMAP_TABLE, 339 },
	{ 0x5143, 0x5149, PDF_CMAP_TABLE, 341 },
	{ 0x514b, 0x514e, PDF_CMAP_TABLE, 348 },
	{ 0x5152, 0x5152, PDF_CMAP_SINGLE, 5777 },
	{ 0x515c, 0x515c, PDF_CMAP_SINGLE, 4348 },
	{ 0x5162, 0x5162, PDF_CMAP_SINGLE, 4059 },
	{ 0x5165, 0x5165, PDF_CMAP_SINGLE, 6477 },
	{ 0x5167, 0x516e, PDF_CMAP_TABLE, 352 },
	{ 0x5171, 0x5171, PDF_CMAP_SINGLE, 3788 },
	{ 0x5175, 0x5178, PDF_CMAP_TABLE, 360 },
	{ 0x517c, 0x517c, PDF_CMAP_SINGLE, 3654 },
	{ 0x5180, 0x5180, PDF_CMAP_SINGLE, 4065 },
	{ 0x5186, 0x5186, PDF_CMAP_SINGLE, 5933 },
	{ 0x518a, 0x518a, PDF_CMAP_SINGLE, 7126 },
	{ 0x518d, 0x518d, PDF_CMAP_SINGLE, 6570 },
	{ 0x5192, 0x5192, PDF_CMAP_SINGLE, 4765 },
	{ 0x5195, 0x5195, PDF_CMAP_SINGLE, 4736 },
	{ 0x5197, 0x5197, PDF_CMAP_SINGLE, 6176 },
	{ 0x51a0, 0x51a0, PDF_CMAP_SINGLE, 3819 },
	{ 0x51a5, 0x51a5, PDF_CMAP_SINGLE, 4748 },
	{ 0x51aa, 0x51aa, PDF_CMAP_SINGLE, 4733 },
	{ 0x51ac, 0x51ac, PDF_CMAP_SINGLE, 4332 },
	{ 0x51b6, 0x51b7, PDF_CMAP_TABLE, 364 },
	{ 0x51bd, 0x51bd, PDF_CMAP_SINGLE, 4477 },
	{ 0x51c4, 0x51c4, PDF_CMAP_SINGLE, 7130 },
	{ 0x51c6, 0x51c6, PDF_CMAP_SINGLE, 6904 },
	{ 0x51c9, 0x51c9, PDF_CMAP_SINGLE, 4430 },
	{ 0x51cb, 0x51cd, PDF_CMAP_TABLE, 366 },
	{ 0x51dc, 0x51de, PDF_CMAP_TABLE, 369 },
	{ 0x51e1, 0x51e1, PDF_CMAP_SINGLE, 5009 },
	{ 0x51f0, 0x51f1, PDF_CMAP_TABLE, 372 },
	{ 0x51f6, 0x51f6, PDF_CMAP_SINGLE, 8014 },
	{ 0x51f8, 0x51fa, PDF_CMAP_TABLE, 374 },
	{ 0x51fd, 0x51fd, PDF_CMAP_SINGLE, 7651 },
	{ 0x5200, 0x5200, PDF_CMAP_SINGLE, 4270 },
	{ 0x5203, 0x5203, PDF_CMAP_SINGLE, 6445 },
	{ 0x5206, 0x5208, PDF_CMAP_TABLE, 377 },
	{ 0x520a, 0x520a, PDF_CMAP_SINGLE, 3477 },
	{ 0x520e, 0x520e, PDF_CMAP_SINGLE, 4837 },
	{ 0x5211, 0x5211, PDF_CMAP_SINGLE, 7773 },
	{ 0x5217, 0x5217, PDF_CMAP_SINGLE, 4478 },
	{ 0x521d, 0x521d, PDF_CMAP_SINGLE, 7214 },
	{ 0x5224, 0x5225, PDF_CMAP_TABLE, 380 },
	{ 0x5229, 0x522a, PDF_CMAP_TABLE, 382 },
	{ 0x522e, 0x522e, PDF_CMAP_SINGLE, 3835 },
	{ 0x5230, 0x5230, PDF_CMAP_SINGLE, 4271 },
	{ 0x5236, 0x523b, PDF_CMAP_TABLE, 384 },
	{ 0x5243, 0x5243, PDF_CMAP_SINGLE, 7205 },
	{ 0x5247, 0x5247, PDF_CMAP_SINGLE, 7360 },
	{ 0x524a, 0x524d, PDF_CMAP_TABLE, 390 },
	{ 0x5254, 0x5254, PDF_CMAP_SINGLE, 7135 },
	{ 0x5256, 0x5256, PDF_CMAP_SINGLE, 5113 },
	{ 0x525b, 0x525b, PDF_CMAP_SINGLE, 3536 },
	{ 0x525d, 0x525d, PDF_CMAP_SINGLE, 4886 },
	{ 0x5261, 0x5261, PDF_CMAP_SINGLE, 5463 },
	{ 0x5269, 0x526a, PDF_CMAP_TABLE, 394 },
	{ 0x526f, 0x526f, PDF_CMAP_SINGLE, 5114 },
	{ 0x5272, 0x5272, PDF_CMAP_SINGLE, 7649 },
	{ 0x5275, 0x5275, PDF_CMAP_SINGLE, 7094 },
	{ 0x527d, 0x527d, PDF_CMAP_SINGLE, 7577 },
	{ 0x527f, 0x527f, PDF_CMAP_SINGLE, 7215 },
	{ 0x5283, 0x5283, PDF_CMAP_SINGLE, 7948 },
	{ 0x5287, 0x5289, PDF_CMAP_TABLE, 396 },
	{ 0x528d, 0x528d, PDF_CMAP_SINGLE, 3618 },
	{ 0x5291, 0x5292, PDF_CMAP_TABLE, 399 },
	{ 0x529b, 0x529b, PDF_CMAP_SINGLE, 4458 },
	{ 0x529f, 0x52a0, PDF_CMAP_TABLE, 401 },
	{ 0x52a3, 0x52a4, PDF_CMAP_TABLE, 403 },
	{ 0x52a9, 0x52ab, PDF_CMAP_TABLE, 405 },
	{ 0x52be, 0x52be, PDF_CMAP_SINGLE, 7703 },
	{ 0x52c1, 0x52c1, PDF_CMAP_SINGLE, 3665 },
	{ 0x52c3, 0x52c3, PDF_CMAP_SINGLE, 4930 },
	{ 0x52c5, 0x52c5, PDF_CMAP_SINGLE, 7361 },
	{ 0x52c7, 0x52c7, PDF_CMAP_SINGLE, 6177 },
	{ 0x52c9, 0x52c9, PDF_CMAP_SINGLE, 4737 },
	{ 0x52cd, 0x52cd, PDF_CMAP_SINGLE, 3666 },
	{ 0x52d2, 0x52d2, PDF_CMAP_SINGLE, 4604 },
	{ 0x52d5, 0x52d6, PDF_CMAP_TABLE, 408 },
	{ 0x52d8, 0x52d9, PDF_CMAP_TABLE, 410 },
	{ 0x52db, 0x52db, PDF_CMAP_SINGLE, 7979 },
	{ 0x52dd, 0x52df, PDF_CMAP_TABLE, 412 },
	{ 0x52e2, 0x52e4, PDF_CMAP_TABLE, 415 },
	{ 0x52f3, 0x52f3, PDF_CMAP_SINGLE, 7980 },
	{ 0x52f5, 0x52f5, PDF_CMAP_SINGLE, 4442 },
	{ 0x52f8, 0x52f8, PDF_CMAP_SINGLE, 3971 },
	{ 0x52fa, 0x52fb, PDF_CMAP_TABLE, 418 },
	{ 0x52fe, 0x52ff, PDF_CMAP_TABLE, 420 },
	{ 0x5305, 0x5305, PDF_CMAP_SINGLE, 7545 },
	{ 0x5308, 0x5308, PDF_CMAP_SINGLE, 8015 },
	{ 0x530d, 0x530d, PDF_CMAP_SINGLE, 7546 },
	{ 0x530f, 0x5310, PDF_CMAP_TABLE, 422 },
	{ 0x5315, 0x5317, PDF_CMAP_TABLE, 424 },
	{ 0x5319, 0x5319, PDF_CMAP_SINGLE, 5693 },
	{ 0x5320, 0x5321, PDF_CMAP_TABLE, 427 },
	{ 0x5323, 0x5323, PDF_CMAP_SINGLE, 3530 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 5184 },
	{ 0x532f, 0x532f, PDF_CMAP_SINGLE, 7928 },
	{ 0x5339, 0x5339, PDF_CMAP_SINGLE, 7604 },
	{ 0x533f, 0x5341, PDF_CMAP_TABLE, 429 },
	{ 0x5343, 0x5344, PDF_CMAP_TABLE, 432 },
	{ 0x5347, 0x534a, PDF_CMAP_TABLE, 434 },
	{ 0x534d, 0x534d, PDF_CMAP_SINGLE, 4671 },
	{ 0x5351, 0x5354, PDF_CMAP_TABLE, 438 },
	{ 0x5357, 0x5357, PDF_CMAP_SINGLE, 4145 },
	{ 0x535a, 0x535a, PDF_CMAP_SINGLE, 4887 },
	{ 0x535c, 0x535c, PDF_CMAP_SINGLE, 5077 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 5031 },
	{ 0x5360, 0x5360, PDF_CMAP_SINGLE, 6692 },
	{ 0x5366, 0x5366, PDF_CMAP_SINGLE, 3852 },
	{ 0x5368, 0x5368, PDF_CMAP_SINGLE, 5450 },
	{ 0x536f, 0x5371, PDF_CMAP_TABLE, 442 },
	{ 0x5374, 0x5375, PDF_CMAP_TABLE, 445 },
	{ 0x5377, 0x5377, PDF_CMAP_SINGLE, 3972 },
	{ 0x537d, 0x537d, PDF_CMAP_SINGLE, 6926 },
	{ 0x537f, 0x537f, PDF_CMAP_SINGLE, 3667 },
	{ 0x5384, 0x5384, PDF_CMAP_SINGLE, 5851 },
	{ 0x5393, 0x5393, PDF_CMAP_SINGLE, 5840 },
	{ 0x5398, 0x5398, PDF_CMAP_SINGLE, 4615 },
	{ 0x539a, 0x539a, PDF_CMAP_SINGLE, 7968 },
	{ 0x539f, 0x53a0, PDF_CMAP_TABLE, 447 },
	{ 0x53a5, 0x53a6, PDF_CMAP_TABLE, 449 },
	{ 0x53ad, 0x53ad, PDF_CMAP_SINGLE, 5989 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 3586 },
	{ 0x53c3, 0x53c3, PDF_CMAP_SINGLE, 7083 },
	{ 0x53c8, 0x53cb, PDF_CMAP_TABLE, 451 },
	{ 0x53cd, 0x53cd, PDF_CMAP_SINGLE, 4907 },
	{ 0x53d4, 0x53d4, PDF_CMAP_SINGLE, 5628 },
	{ 0x53d6, 0x53d7, PDF_CMAP_TABLE, 455 },
	{ 0x53db, 0x53db, PDF_CMAP_SINGLE, 4908 },
	{ 0x53e1, 0x53e5, PDF_CMAP_TABLE, 457 },
	{ 0x53e9, 0x53ed, PDF_CMAP_TABLE, 462 },
	{ 0x53ef, 0x53f3, PDF_CMAP_TABLE, 467 },
	{ 0x53f8, 0x53f8, PDF_CMAP_SINGLE, 5251 },
	{ 0x5403, 0x5404, PDF_CMAP_TABLE, 472 },
	{ 0x5408, 0x540a, PDF_CMAP_TABLE, 474 },
	{ 0x540c, 0x5411, PDF_CMAP_TABLE, 477 },
	{ 0x541b, 0x541b, PDF_CMAP_SINGLE, 3953 },
	{ 0x541d, 0x541d, PDF_CMAP_SINGLE, 4638 },
	{ 0x541f, 0x5420, PDF_CMAP_TABLE, 483 },
	{ 0x5426, 0x5426, PDF_CMAP_SINGLE, 5115 },
	{ 0x5429, 0x5429, PDF_CMAP_SINGLE, 5153 },
	{ 0x542b, 0x542b, PDF_CMAP_SINGLE, 7652 },
	{ 0x5433, 0x5433, PDF_CMAP_SINGLE, 6055 },
	{ 0x5438, 0x5439, PDF_CMAP_TABLE, 485 },
	{ 0x543b, 0x543c, PDF_CMAP_TABLE, 487 },
	{ 0x543e, 0x543e, PDF_CMAP_SINGLE, 6054 },
	{ 0x5442, 0x5442, PDF_CMAP_SINGLE, 4443 },
	{ 0x5448, 0x5448, PDF_CMAP_SINGLE, 6709 },
	{ 0x544a, 0x544a, PDF_CMAP_SINGLE, 3731 },
	{ 0x5451, 0x5451, PDF_CMAP_SINGLE, 7408 },
	{ 0x5468, 0x5468, PDF_CMAP_SINGLE, 6867 },
	{ 0x546a, 0x546a, PDF_CMAP_SINGLE, 6866 },
	{ 0x5471, 0x5471, PDF_CMAP_SINGLE, 3732 },
	{ 0x5473, 0x5473, PDF_CMAP_SINGLE, 4851 },
	{ 0x5475, 0x5475, PDF_CMAP_SINGLE, 3442 },
	{ 0x547b, 0x547d, PDF_CMAP_TABLE, 489 },
	{ 0x5480, 0x5480, PDF_CMAP_SINGLE, 6594 },
	{ 0x5486, 0x5486, PDF_CMAP_SINGLE, 7548 },
	{ 0x548c, 0x548c, PDF_CMAP_SINGLE, 7864 },
	{ 0x548e, 0x548e, PDF_CMAP_SINGLE, 3903 },
	{ 0x5490, 0x5490, PDF_CMAP_SINGLE, 5116 },
	{ 0x54a4, 0x54a4, PDF_CMAP_SINGLE, 7381 },
	{ 0x54a8, 0x54a8, PDF_CMAP_SINGLE, 6485 },
	{ 0x54ab, 0x54ac, PDF_CMAP_TABLE, 492 },
	{ 0x54b3, 0x54b3, PDF_CMAP_SINGLE, 7687 },
	{ 0x54b8, 0x54b8, PDF_CMAP_SINGLE, 7653 },
	{ 0x54bd, 0x54bd, PDF_CMAP_SINGLE, 6447 },
	{ 0x54c0, 0x54c1, PDF_CMAP_TABLE, 494 },
	{ 0x54c4, 0x54c4, PDF_CMAP_SINGLE, 7853 },
	{ 0x54c8, 0x54c9, PDF_CMAP_TABLE, 496 },
	{ 0x54e1, 0x54e1, PDF_CMAP_SINGLE, 6257 },
	{ 0x54e5, 0x54e5, PDF_CMAP_SINGLE, 3443 },
	{ 0x54e8, 0x54e8, PDF_CMAP_SINGLE, 7216 },
	{ 0x54ed, 0x54ee, PDF_CMAP_TABLE, 498 },
	{ 0x54f2, 0x54f2, PDF_CMAP_SINGLE, 7168 },
	{ 0x54fa, 0x54fa, PDF_CMAP_SINGLE, 7549 },
	{ 0x5504, 0x5504, PDF_CMAP_SINGLE, 7503 },
	{ 0x5506, 0x5507, PDF_CMAP_TABLE, 500 },
	{ 0x550e, 0x550e, PDF_CMAP_SINGLE, 4617 },
	{ 0x5510, 0x5510, PDF_CMAP_SINGLE, 4239 },
	{ 0x551c, 0x551c, PDF_CMAP_SINGLE, 4689 },
	{ 0x552f, 0x552f, PDF_CMAP_SINGLE, 6313 },
	{ 0x5531, 0x5531, PDF_CMAP_SINGLE, 7095 },
	{ 0x5535, 0x5535, PDF_CMAP_SINGLE, 5821 },
	{ 0x553e, 0x553e, PDF_CMAP_SINGLE, 7382 },
	{ 0x5544, 0x5544, PDF_CMAP_SINGLE, 7396 },
	{ 0x5546, 0x5546, PDF_CMAP_SINGLE, 5335 },
	{ 0x554f, 0x554f, PDF_CMAP_SINGLE, 4839 },
	{ 0x5553, 0x5553, PDF_CMAP_SINGLE, 3706 },
	{ 0x5556, 0x5556, PDF_CMAP_SINGLE, 4217 },
	{ 0x555e, 0x555e, PDF_CMAP_SINGLE, 5778 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 7654 },
	{ 0x557c, 0x557c, PDF_CMAP_SINGLE, 6761 },
	{ 0x5580, 0x5580, PDF_CMAP_SINGLE, 3579 },
	{ 0x5584, 0x5584, PDF_CMAP_SINGLE, 5421 },
	{ 0x5586, 0x5587, PDF_CMAP_TABLE, 502 },
	{ 0x5589, 0x558a, PDF_CMAP_TABLE, 504 },
	{ 0x5598, 0x559a, PDF_CMAP_TABLE, 506 },
	{ 0x559c, 0x559d, PDF_CMAP_TABLE, 509 },
	{ 0x55a7, 0x55a7, PDF_CMAP_SINGLE, 7990 },
	{ 0x55a9, 0x55ac, PDF_CMAP_TABLE, 511 },
	{ 0x55ae, 0x55ae, PDF_CMAP_SINGLE, 4195 },
	{ 0x55c5, 0x55c5, PDF_CMAP_SINGLE, 7972 },
	{ 0x55c7, 0x55c7, PDF_CMAP_SINGLE, 5365 },
	{ 0x55d4, 0x55d4, PDF_CMAP_SINGLE, 6981 },
	{ 0x55da, 0x55da, PDF_CMAP_SINGLE, 6056 },
	{ 0x55dc, 0x55dc, PDF_CMAP_SINGLE, 4066 },
	{ 0x55df, 0x55df, PDF_CMAP_SINGLE, 7045 },
	{ 0x55e3, 0x55e4, PDF_CMAP_TABLE, 515 },
	{ 0x55fd, 0x55fe, PDF_CMAP_TABLE, 517 },
	{ 0x5606, 0x5606, PDF_CMAP_SINGLE, 7409 },
	{ 0x5609, 0x5609, PDF_CMAP_SINGLE, 3444 },
	{ 0x5614, 0x5614, PDF_CMAP_SINGLE, 3904 },
	{ 0x5617, 0x5617, PDF_CMAP_SINGLE, 5337 },
	{ 0x562f, 0x562f, PDF_CMAP_SINGLE, 5501 },
	{ 0x5632, 0x5632, PDF_CMAP_SINGLE, 6786 },
	{ 0x5634, 0x5634, PDF_CMAP_SINGLE, 7318 },
	{ 0x5636, 0x5636, PDF_CMAP_SINGLE, 5694 },
	{ 0x5653, 0x5653, PDF_CMAP_SINGLE, 7719 },
	{ 0x5668, 0x5668, PDF_CMAP_SINGLE, 4067 },
	{ 0x566b, 0x566b, PDF_CMAP_SINGLE, 8038 },
	{ 0x5674, 0x5674, PDF_CMAP_SINGLE, 5154 },
	{ 0x5686, 0x5686, PDF_CMAP_SINGLE, 7954 },
	{ 0x56a5, 0x56a5, PDF_CMAP_SINGLE, 5956 },
	{ 0x56ac, 0x56ac, PDF_CMAP_SINGLE, 5223 },
	{ 0x56ae, 0x56ae, PDF_CMAP_SINGLE, 7712 },
	{ 0x56b4, 0x56b4, PDF_CMAP_SINGLE, 5927 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 6511 },
	{ 0x56ca, 0x56ca, PDF_CMAP_SINGLE, 4152 },
	{ 0x56cd, 0x56cd, PDF_CMAP_SINGLE, 8039 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 7242 },
	{ 0x56da, 0x56db, PDF_CMAP_TABLE, 519 },
	{ 0x56de, 0x56de, PDF_CMAP_SINGLE, 7929 },
	{ 0x56e0, 0x56e0, PDF_CMAP_SINGLE, 6448 },
	{ 0x56f0, 0x56f0, PDF_CMAP_SINGLE, 3774 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 521 },
	{ 0x5703, 0x5704, PDF_CMAP_TABLE, 523 },
	{ 0x5708, 0x5708, PDF_CMAP_SINGLE, 3973 },
	{ 0x570b, 0x570b, PDF_CMAP_SINGLE, 3947 },
	{ 0x570d, 0x570d, PDF_CMAP_SINGLE, 6288 },
	{ 0x5712, 0x5713, PDF_CMAP_TABLE, 525 },
	{ 0x5716, 0x5716, PDF_CMAP_SINGLE, 4272 },
	{ 0x5718, 0x5718, PDF_CMAP_SINGLE, 4196 },
	{ 0x571f, 0x571f, PDF_CMAP_SINGLE, 7451 },
	{ 0x5728, 0x5728, PDF_CMAP_SINGLE, 6572 },
	{ 0x572d, 0x572d, PDF_CMAP_SINGLE, 3995 },
	{ 0x5730, 0x5730, PDF_CMAP_SINGLE, 6945 },
	{ 0x573b, 0x573b, PDF_CMAP_SINGLE, 4068 },
	{ 0x5740, 0x5740, PDF_CMAP_SINGLE, 6946 },
	{ 0x5742, 0x5742, PDF_CMAP_SINGLE, 7491 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 4010 },
	{ 0x574a, 0x574a, PDF_CMAP_SINGLE, 4943 },
	{ 0x574d, 0x574e, PDF_CMAP_TABLE, 527 },
	{ 0x5750, 0x5751, PDF_CMAP_TABLE, 529 },
	{ 0x5761, 0x5761, PDF_CMAP_SINGLE, 7474 },
	{ 0x5764, 0x5764, PDF_CMAP_SINGLE, 3775 },
	{ 0x5766, 0x5766, PDF_CMAP_SINGLE, 7410 },
	{ 0x576a, 0x576a, PDF_CMAP_SINGLE, 7529 },
	{ 0x576e, 0x576e, PDF_CMAP_SINGLE, 4252 },
	{ 0x5770, 0x5770, PDF_CMAP_SINGLE, 3668 },
	{ 0x5775, 0x5775, PDF_CMAP_SINGLE, 3905 },
	{ 0x577c, 0x577c, PDF_CMAP_SINGLE, 7397 },
	{ 0x5782, 0x5782, PDF_CMAP_SINGLE, 5571 },
	{ 0x5788, 0x5788, PDF_CMAP_SINGLE, 4251 },
	{ 0x578b, 0x578b, PDF_CMAP_SINGLE, 7774 },
	{ 0x5793, 0x5793, PDF_CMAP_SINGLE, 7688 },
	{ 0x57a0, 0x57a0, PDF_CMAP_SINGLE, 6373 },
	{ 0x57a2, 0x57a3, PDF_CMAP_TABLE, 531 },
	{ 0x57c3, 0x57c3, PDF_CMAP_SINGLE, 5842 },
	{ 0x57c7, 0x57c8, PDF_CMAP_TABLE, 533 },
	{ 0x57cb, 0x57cb, PDF_CMAP_SINGLE, 4708 },
	{ 0x57ce, 0x57ce, PDF_CMAP_SINGLE, 5474 },
	{ 0x57df, 0x57e0, PDF_CMAP_TABLE, 535 },
	{ 0x57f0, 0x57f0, PDF_CMAP_SINGLE, 7115 },
	{ 0x57f4, 0x57f4, PDF_CMAP_SINGLE, 5720 },
	{ 0x57f7, 0x57f7, PDF_CMAP_SINGLE, 7032 },
	{ 0x57f9, 0x57fa, PDF_CMAP_TABLE, 537 },
	{ 0x57fc, 0x57fc, PDF_CMAP_SINGLE, 4070 },
	{ 0x5800, 0x5800, PDF_CMAP_SINGLE, 3959 },
	{ 0x5802, 0x5802, PDF_CMAP_SINGLE, 4240 },
	{ 0x5805, 0x5806, PDF_CMAP_TABLE, 539 },
	{ 0x5808, 0x580a, PDF_CMAP_TABLE, 541 },
	{ 0x581e, 0x581e, PDF_CMAP_SINGLE, 7187 },
	{ 0x5821, 0x5821, PDF_CMAP_SINGLE, 5059 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 6762 },
	{ 0x5827, 0x5827, PDF_CMAP_SINGLE, 5957 },
	{ 0x582a, 0x582a, PDF_CMAP_SINGLE, 3512 },
	{ 0x582f, 0x5831, PDF_CMAP_TABLE, 544 },
	{ 0x5834, 0x5835, PDF_CMAP_TABLE, 547 },
	{ 0x583a, 0x583a, PDF_CMAP_SINGLE, 3707 },
	{ 0x584a, 0x584b, PDF_CMAP_TABLE, 549 },
	{ 0x584f, 0x584f, PDF_CMAP_SINGLE, 3564 },
	{ 0x5851, 0x5851, PDF_CMAP_SINGLE, 5502 },
	{ 0x5854, 0x5854, PDF_CMAP_SINGLE, 7424 },
	{ 0x5857, 0x5858, PDF_CMAP_TABLE, 551 },
	{ 0x585a, 0x585a, PDF_CMAP_SINGLE, 7252 },
	{ 0x585e, 0x585e, PDF_CMAP_SINGLE, 5362 },
	{ 0x5861, 0x5862, PDF_CMAP_TABLE, 553 },
	{ 0x5864, 0x5864, PDF_CMAP_SINGLE, 7981 },
	{ 0x5875, 0x5875, PDF_CMAP_SINGLE, 6982 },
	{ 0x5879, 0x5879, PDF_CMAP_SINGLE, 7084 },
	{ 0x587c, 0x587c, PDF_CMAP_SINGLE, 6651 },
	{ 0x587e, 0x587e, PDF_CMAP_SINGLE, 5629 },
	{ 0x5883, 0x5883, PDF_CMAP_SINGLE, 3669 },
	{ 0x5885, 0x5885, PDF_CMAP_SINGLE, 5373 },
	{ 0x5889, 0x5889, PDF_CMAP_SINGLE, 6179 },
	{ 0x5893, 0x5893, PDF_CMAP_SINGLE, 4801 },
	{ 0x589c, 0x589c, PDF_CMAP_SINGLE, 7265 },
	{ 0x589e, 0x589f, PDF_CMAP_TABLE, 555 },
	{ 0x58a8, 0x58a9, PDF_CMAP_TABLE, 557 },
	{ 0x58ae, 0x58ae, PDF_CMAP_SINGLE, 7383 },
	{ 0x58b3, 0x58b3, PDF_CMAP_SINGLE, 5155 },
	{ 0x58ba, 0x58bb, PDF_CMAP_TABLE, 559 },
	{ 0x58be, 0x58be, PDF_CMAP_SINGLE, 3478 },
	{ 0x58c1, 0x58c1, PDF_CMAP_SINGLE, 5022 },
	{ 0x58c5, 0x58c5, PDF_CMAP_SINGLE, 6090 },
	{ 0x58c7, 0x58c7, PDF_CMAP_SINGLE, 4197 },
	{ 0x58ce, 0x58ce, PDF_CMAP_SINGLE, 7982 },
	{ 0x58d1, 0x58d1, PDF_CMAP_SINGLE, 7630 },
	{ 0x58d3, 0x58d3, PDF_CMAP_SINGLE, 5829 },
	{ 0x58d5, 0x58d5, PDF_CMAP_SINGLE, 7803 },
	{ 0x58d8, 0x58d9, PDF_CMAP_TABLE, 561 },
	{ 0x58de, 0x58df, PDF_CMAP_TABLE, 563 },
	{ 0x58e4, 0x58e4, PDF_CMAP_SINGLE, 5880 },
	{ 0x58eb, 0x58ec, PDF_CMAP_TABLE, 565 },
	{ 0x58ef, 0x58ef, PDF_CMAP_SINGLE, 6539 },
	{ 0x58f9, 0x58fb, PDF_CMAP_TABLE, 567 },
	{ 0x58fd, 0x58fd, PDF_CMAP_SINGLE, 5572 },
	{ 0x590f, 0x590f, PDF_CMAP_SINGLE, 7619 },
	{ 0x5914, 0x5916, PDF_CMAP_TABLE, 570 },
	{ 0x5919, 0x591a, PDF_CMAP_TABLE, 573 },
	{ 0x591c, 0x591c, PDF_CMAP_SINGLE, 5865 },
	{ 0x5922, 0x5922, PDF_CMAP_SINGLE, 4797 },
	{ 0x5927, 0x5927, PDF_CMAP_SINGLE, 4253 },
	{ 0x5929, 0x592b, PDF_CMAP_TABLE, 575 },
	{ 0x592d, 0x592e, PDF_CMAP_TABLE, 578 },
	{ 0x5931, 0x5931, PDF_CMAP_SINGLE, 5758 },
	{ 0x5937, 0x5937, PDF_CMAP_SINGLE, 6416 },
	{ 0x593e, 0x593e, PDF_CMAP_SINGLE, 7761 },
	{ 0x5944, 0x5944, PDF_CMAP_SINGLE, 5928 },
	{ 0x5947, 0x5949, PDF_CMAP_TABLE, 580 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 583 },
	{ 0x5954, 0x5955, PDF_CMAP_TABLE, 587 },
	{ 0x5957, 0x5957, PDF_CMAP_SINGLE, 7466 },
	{ 0x595a, 0x595a, PDF_CMAP_SINGLE, 7689 },
	{ 0x5960, 0x5960, PDF_CMAP_SINGLE, 6652 },
	{ 0x5962, 0x5962, PDF_CMAP_SINGLE, 5256 },
	{ 0x5967, 0x5967, PDF_CMAP_SINGLE, 6059 },
	{ 0x596a, 0x596e, PDF_CMAP_TABLE, 589 },
	{ 0x5973, 0x5974, PDF_CMAP_TABLE, 594 },
	{ 0x5978, 0x5978, PDF_CMAP_SINGLE, 3479 },
	{ 0x597d, 0x597d, PDF_CMAP_SINGLE, 7805 },
	{ 0x5982, 0x5984, PDF_CMAP_TABLE, 596 },
	{ 0x598a, 0x598a, PDF_CMAP_SINGLE, 6471 },
	{ 0x5993, 0x5993, PDF_CMAP_SINGLE, 4073 },
	{ 0x5996, 0x5997, PDF_CMAP_TABLE, 599 },
	{ 0x5999, 0x5999, PDF_CMAP_SINGLE, 4802 },
	{ 0x59a5, 0x59a5, PDF_CMAP_SINGLE, 7384 },
	{ 0x59a8, 0x59a8, PDF_CMAP_SINGLE, 4944 },
	{ 0x59ac, 0x59ac, PDF_CMAP_SINGLE, 7467 },
	{ 0x59b9, 0x59b9, PDF_CMAP_SINGLE, 4709 },
	{ 0x59bb, 0x59bb, PDF_CMAP_SINGLE, 7131 },
	{ 0x59be, 0x59be, PDF_CMAP_SINGLE, 7188 },
	{ 0x59c3, 0x59c3, PDF_CMAP_SINGLE, 6710 },
	{ 0x59c6, 0x59c6, PDF_CMAP_SINGLE, 4767 },
	{ 0x59c9, 0x59c9, PDF_CMAP_SINGLE, 6486 },
	{ 0x59cb, 0x59cb, PDF_CMAP_SINGLE, 5695 },
	{ 0x59d0, 0x59d1, PDF_CMAP_TABLE, 601 },
	{ 0x59d3, 0x59d4, PDF_CMAP_TABLE, 603 },
	{ 0x59d9, 0x59da, PDF_CMAP_TABLE, 605 },
	{ 0x59dc, 0x59dd, PDF_CMAP_TABLE, 607 },
	{ 0x59e6, 0x59e6, PDF_CMAP_SINGLE, 3480 },
	{ 0x59e8, 0x59e8, PDF_CMAP_SINGLE, 6417 },
	{ 0x59ea, 0x59ea, PDF_CMAP_SINGLE, 7017 },
	{ 0x59ec, 0x59ec, PDF_CMAP_SINGLE, 8040 },
	{ 0x59ee, 0x59ee, PDF_CMAP_SINGLE, 7672 },
	{ 0x59f8, 0x59f8, PDF_CMAP_SINGLE, 5958 },
	{ 0x59fb, 0x59fb, PDF_CMAP_SINGLE, 6449 },
	{ 0x59ff, 0x59ff, PDF_CMAP_SINGLE, 6487 },
	{ 0x5a01, 0x5a01, PDF_CMAP_SINGLE, 6290 },
	{ 0x5a03, 0x5a03, PDF_CMAP_SINGLE, 6132 },
	{ 0x5a11, 0x5a11, PDF_CMAP_SINGLE, 5257 },
	{ 0x5a18, 0x5a18, PDF_CMAP_SINGLE, 4153 },
	{ 0x5a1b, 0x5a1c, PDF_CMAP_TABLE, 609 },
	{ 0x5a1f, 0x5a20, PDF_CMAP_TABLE, 611 },
	{ 0x5a25, 0x5a25, PDF_CMAP_SINGLE, 5779 },
	{ 0x5a29, 0x5a29, PDF_CMAP_SINGLE, 4672 },
	{ 0x5a36, 0x5a36, PDF_CMAP_SINGLE, 7319 },
	{ 0x5a3c, 0x5a3c, PDF_CMAP_SINGLE, 7096 },
	{ 0x5a41, 0x5a41, PDF_CMAP_SINGLE, 4565 },
	{ 0x5a46, 0x5a46, PDF_CMAP_SINGLE, 7475 },
	{ 0x5a49, 0x5a49, PDF_CMAP_SINGLE, 6107 },
	{ 0x5a5a, 0x5a5a, PDF_CMAP_SINGLE, 7844 },
	{ 0x5a62, 0x5a62, PDF_CMAP_SINGLE, 5187 },
	{ 0x5a66, 0x5a66, PDF_CMAP_SINGLE, 5119 },
	{ 0x5a92, 0x5a92, PDF_CMAP_SINGLE, 4710 },
	{ 0x5a9a, 0x5a9b, PDF_CMAP_TABLE, 613 },
	{ 0x5aa4, 0x5aa4, PDF_CMAP_SINGLE, 5696 },
	{ 0x5ac1, 0x5ac2, PDF_CMAP_TABLE, 615 },
	{ 0x5ac4, 0x5ac4, PDF_CMAP_SINGLE, 6262 },
	{ 0x5ac9, 0x5ac9, PDF_CMAP_SINGLE, 7018 },
	{ 0x5acc, 0x5acc, PDF_CMAP_SINGLE, 7758 },
	{ 0x5ae1, 0x5ae1, PDF_CMAP_SINGLE, 6621 },
	{ 0x5ae6, 0x5ae6, PDF_CMAP_SINGLE, 7673 },
	{ 0x5ae9, 0x5ae9, PDF_CMAP_SINGLE, 4181 },
	{ 0x5b05, 0x5b05, PDF_CMAP_SINGLE, 7865 },
	{ 0x5b09, 0x5b09, PDF_CMAP_SINGLE, 8041 },
	{ 0x5b0b, 0x5b0c, PDF_CMAP_TABLE, 617 },
	{ 0x5b16, 0x5b16, PDF_CMAP_SINGLE, 7535 },
	{ 0x5b2a, 0x5b2a, PDF_CMAP_SINGLE, 5224 },
	{ 0x5b40, 0x5b40, PDF_CMAP_SINGLE, 5338 },
	{ 0x5b43, 0x5b43, PDF_CMAP_SINGLE, 5881 },
	{ 0x5b50, 0x5b51, PDF_CMAP_TABLE, 619 },
	{ 0x5b54, 0x5b55, PDF_CMAP_TABLE, 621 },
	{ 0x5b57, 0x5b58, PDF_CMAP_TABLE, 623 },
	{ 0x5b5a, 0x5b5a, PDF_CMAP_SINGLE, 5120 },
	{ 0x5b5c, 0x5b5d, PDF_CMAP_TABLE, 625 },
	{ 0x5b5f, 0x5b5f, PDF_CMAP_SINGLE, 4727 },
	{ 0x5b63, 0x5b64, PDF_CMAP_TABLE, 627 },
	{ 0x5b69, 0x5b69, PDF_CMAP_SINGLE, 7690 },
	{ 0x5b6b, 0x5b6b, PDF_CMAP_SINGLE, 5546 },
	{ 0x5b70, 0x5b71, PDF_CMAP_TABLE, 629 },
	{ 0x5b75, 0x5b75, PDF_CMAP_SINGLE, 5121 },
	{ 0x5b78, 0x5b78, PDF_CMAP_SINGLE, 7631 },
	{ 0x5b7a, 0x5b7a, PDF_CMAP_SINGLE, 6315 },
	{ 0x5b7c, 0x5b7c, PDF_CMAP_SINGLE, 5923 },
	{ 0x5b85, 0x5b85, PDF_CMAP_SINGLE, 4266 },
	{ 0x5b87, 0x5b89, PDF_CMAP_TABLE, 631 },
	{ 0x5b8b, 0x5b8c, PDF_CMAP_TABLE, 634 },
	{ 0x5b8f, 0x5b8f, PDF_CMAP_SINGLE, 3864 },
	{ 0x5b93, 0x5b93, PDF_CMAP_SINGLE, 5078 },
	{ 0x5b95, 0x5b9c, PDF_CMAP_TABLE, 636 },
	{ 0x5ba2, 0x5ba6, PDF_CMAP_TABLE, 644 },
	{ 0x5bac, 0x5bac, PDF_CMAP_SINGLE, 5476 },
	{ 0x5bae, 0x5bae, PDF_CMAP_SINGLE, 3963 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 6573 },
	{ 0x5bb3, 0x5bb6, PDF_CMAP_TABLE, 649 },
	{ 0x5bb8, 0x5bb9, PDF_CMAP_TABLE, 653 },
	{ 0x5bbf, 0x5bc0, PDF_CMAP_TABLE, 655 },
	{ 0x5bc2, 0x5bc7, PDF_CMAP_TABLE, 657 },
	{ 0x5bcc, 0x5bcc, PDF_CMAP_SINGLE, 5122 },
	{ 0x5bd0, 0x5bd0, PDF_CMAP_SINGLE, 4711 },
	{ 0x5bd2, 0x5bd4, PDF_CMAP_TABLE, 663 },
	{ 0x5bd7, 0x5bd7, PDF_CMAP_SINGLE, 4168 },
	{ 0x5bde, 0x5bdf, PDF_CMAP_TABLE, 666 },
	{ 0x5be1, 0x5be2, PDF_CMAP_TABLE, 668 },
	{ 0x5be4, 0x5be9, PDF_CMAP_TABLE, 670 },
	{ 0x5beb, 0x5bec, PDF_CMAP_TABLE, 676 },
	{ 0x5bee, 0x5bef, PDF_CMAP_TABLE, 678 },
	{ 0x5bf5, 0x5bf6, PDF_CMAP_TABLE, 680 },
	{ 0x5bf8, 0x5bf8, PDF_CMAP_SINGLE, 7247 },
	{ 0x5bfa, 0x5bfa, PDF_CMAP_SINGLE, 5259 },
	{ 0x5c01, 0x5c01, PDF_CMAP_SINGLE, 5095 },
	{ 0x5c04, 0x5c04, PDF_CMAP_SINGLE, 5260 },
	{ 0x5c07, 0x5c0b, PDF_CMAP_TABLE, 682 },
	{ 0x5c0d, 0x5c0f, PDF_CMAP_TABLE, 687 },
	{ 0x5c11, 0x5c11, PDF_CMAP_SINGLE, 5505 },
	{ 0x5c16, 0x5c16, PDF_CMAP_SINGLE, 7178 },
	{ 0x5c19, 0x5c19, PDF_CMAP_SINGLE, 5339 },
	{ 0x5c24, 0x5c24, PDF_CMAP_SINGLE, 6206 },
	{ 0x5c28, 0x5c28, PDF_CMAP_SINGLE, 4945 },
	{ 0x5c31, 0x5c31, PDF_CMAP_SINGLE, 7320 },
	{ 0x5c38, 0x5c3c, PDF_CMAP_TABLE, 690 },
	{ 0x5c3e, 0x5c40, PDF_CMAP_TABLE, 695 },
	{ 0x5c45, 0x5c46, PDF_CMAP_TABLE, 698 },
	{ 0x5c48, 0x5c48, PDF_CMAP_SINGLE, 3960 },
	{ 0x5c4b, 0x5c4b, PDF_CMAP_SINGLE, 6078 },
	{ 0x5c4d, 0x5c4e, PDF_CMAP_TABLE, 700 },
	{ 0x5c51, 0x5c51, PDF_CMAP_SINGLE, 5451 },
	{ 0x5c55, 0x5c55, PDF_CMAP_SINGLE, 6654 },
	{ 0x5c5b, 0x5c5b, PDF_CMAP_SINGLE, 5044 },
	{ 0x5c60, 0x5c60, PDF_CMAP_SINGLE, 4276 },
	{ 0x5c62, 0x5c62, PDF_CMAP_SINGLE, 4566 },
	{ 0x5c64, 0x5c65, PDF_CMAP_TABLE, 702 },
	{ 0x5c6c, 0x5c6c, PDF_CMAP_SINGLE, 5538 },
	{ 0x5c6f, 0x5c6f, PDF_CMAP_SINGLE, 4358 },
	{ 0x5c71, 0x5c71, PDF_CMAP_SINGLE, 5305 },
	{ 0x5c79, 0x5c79, PDF_CMAP_SINGLE, 8024 },
	{ 0x5c90, 0x5c91, PDF_CMAP_TABLE, 704 },
	{ 0x5ca1, 0x5ca1, PDF_CMAP_SINGLE, 3539 },
	{ 0x5ca9, 0x5ca9, PDF_CMAP_SINGLE, 5822 },
	{ 0x5cab, 0x5cac, PDF_CMAP_TABLE, 706 },
	{ 0x5cb1, 0x5cb1, PDF_CMAP_SINGLE, 4255 },
	{ 0x5cb3, 0x5cb3, PDF_CMAP_SINGLE, 5794 },
	{ 0x5cb5, 0x5cb5, PDF_CMAP_SINGLE, 7806 },
	{ 0x5cb7, 0x5cb8, PDF_CMAP_TABLE, 708 },
	{ 0x5cba, 0x5cba, PDF_CMAP_SINGLE, 4492 },
	{ 0x5cbe, 0x5cbe, PDF_CMAP_SINGLE, 6693 },
	{ 0x5cc0, 0x5cc0, PDF_CMAP_SINGLE, 5576 },
	{ 0x5cd9, 0x5cd9, PDF_CMAP_SINGLE, 7339 },
	{ 0x5ce0, 0x5ce0, PDF_CMAP_SINGLE, 5340 },
	{ 0x5ce8, 0x5ce8, PDF_CMAP_SINGLE, 5780 },
	{ 0x5cef, 0x5cf0, PDF_CMAP_RANGE, 5096 },
	{ 0x5cf4, 0x5cf4, PDF_CMAP_SINGLE, 7735 },
	{ 0x5cf6, 0x5cf6, PDF_CMAP_SINGLE, 4277 },
	{ 0x5cfb, 0x5cfb, PDF_CMAP_SINGLE, 6907 },
	{ 0x5cfd, 0x5cfd, PDF_CMAP_SINGLE, 7762 },
	{ 0x5d07, 0x5d07, PDF_CMAP_SINGLE, 5671 },
	{ 0x5d0d, 0x5d0e, PDF_CMAP_TABLE, 710 },
	{ 0x5d11, 0x5d11, PDF_CMAP_SINGLE, 3776 },
	{ 0x5d14, 0x5d14, PDF_CMAP_SINGLE, 7263 },
	{ 0x5d16, 0x5d17, PDF_CMAP_TABLE, 712 },
	{ 0x5d19, 0x5d19, PDF_CMAP_SINGLE, 4596 },
	{ 0x5d27, 0x5d27, PDF_CMAP_SINGLE, 5672 },
	{ 0x5d29, 0x5d29, PDF_CMAP_SINGLE, 5175 },
	{ 0x5d4b, 0x5d4c, PDF_CMAP_TABLE, 714 },
	{ 0x5d50, 0x5d50, PDF_CMAP_SINGLE, 4399 },
	{ 0x5d69, 0x5d69, PDF_CMAP_SINGLE, 5673 },
	{ 0x5d6c, 0x5d6c, PDF_CMAP_SINGLE, 6136 },
	{ 0x5d6f, 0x5d6f, PDF_CMAP_SINGLE, 7046 },
	{ 0x5d87, 0x5d87, PDF_CMAP_SINGLE, 3908 },
	{ 0x5d8b, 0x5d8b, PDF_CMAP_SINGLE, 4278 },
	{ 0x5d9d, 0x5d9d, PDF_CMAP_SINGLE, 4365 },
	{ 0x5da0, 0x5da0, PDF_CMAP_SINGLE, 3873 },
	{ 0x5da2, 0x5da2, PDF_CMAP_SINGLE, 6147 },
	{ 0x5daa, 0x5daa, PDF_CMAP_SINGLE, 5931 },
	{ 0x5db8, 0x5db8, PDF_CMAP_SINGLE, 6003 },
	{ 0x5dba, 0x5dba, PDF_CMAP_SINGLE, 4493 },
	{ 0x5dbc, 0x5dbd, PDF_CMAP_TABLE, 716 },
	{ 0x5dcd, 0x5dcd, PDF_CMAP_SINGLE, 6137 },
	{ 0x5dd2, 0x5dd2, PDF_CMAP_SINGLE, 4673 },
	{ 0x5dd6, 0x5dd6, PDF_CMAP_SINGLE, 5823 },
	{ 0x5ddd, 0x5dde, PDF_CMAP_TABLE, 718 },
	{ 0x5de1, 0x5de2, PDF_CMAP_TABLE, 720 },
	{ 0x5de5, 0x5de8, PDF_CMAP_TABLE, 722 },
	{ 0x5deb, 0x5deb, PDF_CMAP_SINGLE, 4813 },
	{ 0x5dee, 0x5dee, PDF_CMAP_SINGLE, 7047 },
	{ 0x5df1, 0x5df4, PDF_CMAP_TABLE, 726 },
	{ 0x5df7, 0x5df7, PDF_CMAP_SINGLE, 7674 },
	{ 0x5dfd, 0x5dfe, PDF_CMAP_TABLE, 730 },
	{ 0x5e02, 0x5e03, PDF_CMAP_TABLE, 732 },
	{ 0x5e06, 0x5e06, PDF_CMAP_SINGLE, 5010 },
	{ 0x5e0c, 0x5e0c, PDF_CMAP_SINGLE, 8042 },
	{ 0x5e11, 0x5e11, PDF_CMAP_SINGLE, 7428 },
	{ 0x5e16, 0x5e16, PDF_CMAP_SINGLE, 7189 },
	{ 0x5e19, 0x5e19, PDF_CMAP_SINGLE, 7019 },
	{ 0x5e1b, 0x5e1b, PDF_CMAP_SINGLE, 4990 },
	{ 0x5e1d, 0x5e1d, PDF_CMAP_SINGLE, 6763 },
	{ 0x5e25, 0x5e25, PDF_CMAP_SINGLE, 5577 },
	{ 0x5e2b, 0x5e2b, PDF_CMAP_SINGLE, 5262 },
	{ 0x5e2d, 0x5e2d, PDF_CMAP_SINGLE, 5405 },
	{ 0x5e33, 0x5e33, PDF_CMAP_SINGLE, 6542 },
	{ 0x5e36, 0x5e36, PDF_CMAP_SINGLE, 4256 },
	{ 0x5e38, 0x5e38, PDF_CMAP_SINGLE, 5341 },
	{ 0x5e3d, 0x5e3d, PDF_CMAP_SINGLE, 4768 },
	{ 0x5e3f, 0x5e40, PDF_CMAP_TABLE, 734 },
	{ 0x5e44, 0x5e45, PDF_CMAP_TABLE, 736 },
	{ 0x5e47, 0x5e47, PDF_CMAP_SINGLE, 4946 },
	{ 0x5e4c, 0x5e4c, PDF_CMAP_SINGLE, 7905 },
	{ 0x5e55, 0x5e55, PDF_CMAP_SINGLE, 4665 },
	{ 0x5e5f, 0x5e5f, PDF_CMAP_SINGLE, 7340 },
	{ 0x5e61, 0x5e63, PDF_CMAP_TABLE, 738 },
	{ 0x5e72, 0x5e74, PDF_CMAP_TABLE, 741 },
	{ 0x5e77, 0x5e79, PDF_CMAP_TABLE, 744 },
	{ 0x5e7b, 0x5e7b, PDF_CMAP_SINGLE, 7886 },
	{ 0x5e7c, 0x5e7d, PDF_CMAP_RANGE, 6317 },
	{ 0x5e7e, 0x5e7e, PDF_CMAP_SINGLE, 4078 },
	{ 0x5e84, 0x5e84, PDF_CMAP_SINGLE, 6543 },
	{ 0x5e87, 0x5e87, PDF_CMAP_SINGLE, 5188 },
	{ 0x5e8a, 0x5e8a, PDF_CMAP_SINGLE, 5342 },
	{ 0x5e8f, 0x5e8f, PDF_CMAP_SINGLE, 5376 },
	{ 0x5e95, 0x5e95, PDF_CMAP_SINGLE, 6596 },
	{ 0x5e97, 0x5e97, PDF_CMAP_SINGLE, 6694 },
	{ 0x5e9a, 0x5e9a, PDF_CMAP_SINGLE, 3670 },
	{ 0x5e9c, 0x5e9c, PDF_CMAP_SINGLE, 5123 },
	{ 0x5ea0, 0x5ea0, PDF_CMAP_SINGLE, 5343 },
	{ 0x5ea6, 0x5ea7, PDF_CMAP_TABLE, 747 },
	{ 0x5eab, 0x5eab, PDF_CMAP_SINGLE, 3737 },
	{ 0x5ead, 0x5ead, PDF_CMAP_SINGLE, 6713 },
	{ 0x5eb5, 0x5eb8, PDF_CMAP_TABLE, 749 },
	{ 0x5ebe, 0x5ebe, PDF_CMAP_SINGLE, 6319 },
	{ 0x5ec2, 0x5ec2, PDF_CMAP_SINGLE, 5344 },
	{ 0x5ec8, 0x5eca, PDF_CMAP_TABLE, 753 },
	{ 0x5ed0, 0x5ed0, PDF_CMAP_SINGLE, 3909 },
	{ 0x5ed3, 0x5ed3, PDF_CMAP_SINGLE, 3815 },
	{ 0x5ed6, 0x5ed6, PDF_CMAP_SINGLE, 4554 },
	{ 0x5eda, 0x5edb, PDF_CMAP_TABLE, 756 },
	{ 0x5edf, 0x5ee0, PDF_CMAP_TABLE, 758 },
	{ 0x5ee2, 0x5ee3, PDF_CMAP_TABLE, 760 },
	{ 0x5eec, 0x5eec, PDF_CMAP_SINGLE, 4444 },
	{ 0x5ef3, 0x5ef3, PDF_CMAP_SINGLE, 7197 },
	{ 0x5ef6, 0x5ef7, PDF_CMAP_TABLE, 762 },
	{ 0x5efa, 0x5efb, PDF_CMAP_TABLE, 764 },
	{ 0x5f01, 0x5f01, PDF_CMAP_SINGLE, 5032 },
	{ 0x5f04, 0x5f04, PDF_CMAP_SINGLE, 4537 },
	{ 0x5f0a, 0x5f0a, PDF_CMAP_SINGLE, 7538 },
	{ 0x5f0f, 0x5f0f, PDF_CMAP_SINGLE, 5722 },
	{ 0x5f11, 0x5f11, PDF_CMAP_SINGLE, 5701 },
	{ 0x5f13, 0x5f15, PDF_CMAP_TABLE, 766 },
	{ 0x5f17, 0x5f18, PDF_CMAP_TABLE, 769 },
	{ 0x5f1b, 0x5f1b, PDF_CMAP_SINGLE, 6419 },
	{ 0x5f1f, 0x5f1f, PDF_CMAP_SINGLE, 6764 },
	{ 0x5f26, 0x5f27, PDF_CMAP_TABLE, 771 },
	{ 0x5f29, 0x5f29, PDF_CMAP_SINGLE, 4171 },
	{ 0x5f31, 0x5f31, PDF_CMAP_SINGLE, 5872 },
	{ 0x5f35, 0x5f35, PDF_CMAP_SINGLE, 6544 },
	{ 0x5f3a, 0x5f3a, PDF_CMAP_SINGLE, 3542 },
	{ 0x5f3c, 0x5f3c, PDF_CMAP_SINGLE, 7605 },
	{ 0x5f48, 0x5f48, PDF_CMAP_SINGLE, 7411 },
	{ 0x5f4a, 0x5f4a, PDF_CMAP_SINGLE, 3543 },
	{ 0x5f4c, 0x5f4c, PDF_CMAP_SINGLE, 4855 },
	{ 0x5f4e, 0x5f4e, PDF_CMAP_SINGLE, 4674 },
	{ 0x5f56, 0x5f57, PDF_CMAP_TABLE, 773 },
	{ 0x5f59, 0x5f59, PDF_CMAP_SINGLE, 7997 },
	{ 0x5f5b, 0x5f5b, PDF_CMAP_SINGLE, 6420 },
	{ 0x5f62, 0x5f62, PDF_CMAP_SINGLE, 7775 },
	{ 0x5f66, 0x5f67, PDF_CMAP_TABLE, 775 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 777 },
	{ 0x5f70, 0x5f71, PDF_CMAP_TABLE, 782 },
	{ 0x5f77, 0x5f77, PDF_CMAP_SINGLE, 4947 },
	{ 0x5f79, 0x5f79, PDF_CMAP_SINGLE, 5949 },
	{ 0x5f7c, 0x5f7c, PDF_CMAP_SINGLE, 7597 },
	{ 0x5f7f, 0x5f81, PDF_CMAP_TABLE, 784 },
	{ 0x5f85, 0x5f85, PDF_CMAP_SINGLE, 4257 },
	{ 0x5f87, 0x5f87, PDF_CMAP_SINGLE, 5641 },
	{ 0x5f8a, 0x5f8c, PDF_CMAP_TABLE, 787 },
	{ 0x5f90, 0x5f92, PDF_CMAP_TABLE, 790 },
	{ 0x5f97, 0x5f99, PDF_CMAP_TABLE, 793 },
	{ 0x5f9e, 0x5f9e, PDF_CMAP_SINGLE, 6839 },
	{ 0x5fa0, 0x5fa1, PDF_CMAP_TABLE, 796 },
	{ 0x5fa8, 0x5faa, PDF_CMAP_TABLE, 798 },
	{ 0x5fae, 0x5fae, PDF_CMAP_SINGLE, 4856 },
	{ 0x5fb5, 0x5fb5, PDF_CMAP_SINGLE, 7038 },
	{ 0x5fb7, 0x5fb7, PDF_CMAP_SINGLE, 4267 },
	{ 0x5fb9, 0x5fb9, PDF_CMAP_SINGLE, 7170 },
	{ 0x5fbd, 0x5fbd, PDF_CMAP_SINGLE, 7998 },
	{ 0x5fc3, 0x5fc3, PDF_CMAP_SINGLE, 5764 },
	{ 0x5fc5, 0x5fc5, PDF_CMAP_SINGLE, 7606 },
	{ 0x5fcc, 0x5fcd, PDF_CMAP_TABLE, 801 },
	{ 0x5fd6, 0x5fd7, PDF_CMAP_TABLE, 803 },
	{ 0x5fd8, 0x5fd9, PDF_CMAP_RANGE, 4698 },
	{ 0x5fe0, 0x5fe0, PDF_CMAP_SINGLE, 7307 },
	{ 0x5feb, 0x5feb, PDF_CMAP_SINGLE, 7379 },
	{ 0x5ff5, 0x5ff5, PDF_CMAP_SINGLE, 4163 },
	{ 0x5ffd, 0x5ffd, PDF_CMAP_SINGLE, 7850 },
	{ 0x5fff, 0x5fff, PDF_CMAP_SINGLE, 5158 },
	{ 0x600f, 0x600f, PDF_CMAP_SINGLE, 5835 },
	{ 0x6012, 0x6012, PDF_CMAP_SINGLE, 4172 },
	{ 0x6016, 0x6016, PDF_CMAP_SINGLE, 7552 },
	{ 0x601c, 0x601d, PDF_CMAP_TABLE, 805 },
	{ 0x6020, 0x6021, PDF_CMAP_TABLE, 807 },
	{ 0x6025, 0x6025, PDF_CMAP_SINGLE, 4053 },
	{ 0x6027, 0x6028, PDF_CMAP_TABLE, 809 },
	{ 0x602a, 0x602a, PDF_CMAP_SINGLE, 3859 },
	{ 0x602f, 0x602f, PDF_CMAP_SINGLE, 3625 },
	{ 0x6041, 0x6043, PDF_CMAP_TABLE, 811 },
	{ 0x604d, 0x604d, PDF_CMAP_SINGLE, 7907 },
	{ 0x6050, 0x6050, PDF_CMAP_SINGLE, 3792 },
	{ 0x6052, 0x6052, PDF_CMAP_SINGLE, 7675 },
	{ 0x6055, 0x6055, PDF_CMAP_SINGLE, 5379 },
	{ 0x6059, 0x6059, PDF_CMAP_SINGLE, 5882 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 3836 },
	{ 0x6062, 0x6065, PDF_CMAP_TABLE, 814 },
	{ 0x6068, 0x606a, PDF_CMAP_TABLE, 818 },
	{ 0x606c, 0x606d, PDF_CMAP_TABLE, 821 },
	{ 0x606f, 0x6070, PDF_CMAP_TABLE, 823 },
	{ 0x6085, 0x6085, PDF_CMAP_SINGLE, 5985 },
	{ 0x6089, 0x6089, PDF_CMAP_SINGLE, 5761 },
	{ 0x608c, 0x608d, PDF_CMAP_TABLE, 825 },
	{ 0x6094, 0x6094, PDF_CMAP_SINGLE, 7933 },
	{ 0x6096, 0x6096, PDF_CMAP_SINGLE, 7504 },
	{ 0x609a, 0x609b, PDF_CMAP_TABLE, 827 },
	{ 0x609f, 0x60a0, PDF_CMAP_TABLE, 829 },
	{ 0x60a3, 0x60a4, PDF_CMAP_TABLE, 831 },
	{ 0x60a7, 0x60a7, PDF_CMAP_SINGLE, 4619 },
	{ 0x60b0, 0x60b0, PDF_CMAP_SINGLE, 6840 },
	{ 0x60b2, 0x60b4, PDF_CMAP_TABLE, 833 },
	{ 0x60b6, 0x60b6, PDF_CMAP_SINGLE, 4871 },
	{ 0x60b8, 0x60b8, PDF_CMAP_SINGLE, 3711 },
	{ 0x60bc, 0x60bd, PDF_CMAP_TABLE, 836 },
	{ 0x60c5, 0x60c5, PDF_CMAP_SINGLE, 6716 },
	{ 0x60c7, 0x60c7, PDF_CMAP_SINGLE, 4320 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 7841 },
	{ 0x60da, 0x60da, PDF_CMAP_SINGLE, 7851 },
	{ 0x60dc, 0x60dc, PDF_CMAP_SINGLE, 5406 },
	{ 0x60df, 0x60e1, PDF_CMAP_TABLE, 838 },
	{ 0x60f0, 0x60f1, PDF_CMAP_TABLE, 841 },
	{ 0x60f3, 0x60f3, PDF_CMAP_SINGLE, 5345 },
	{ 0x60f6, 0x60f6, PDF_CMAP_SINGLE, 7908 },
	{ 0x60f9, 0x60fb, PDF_CMAP_TABLE, 843 },
	{ 0x6101, 0x6101, PDF_CMAP_SINGLE, 5578 },
	{ 0x6106, 0x6106, PDF_CMAP_SINGLE, 3606 },
	{ 0x6108, 0x6109, PDF_CMAP_RANGE, 6322 },
	{ 0x610d, 0x610f, PDF_CMAP_TABLE, 846 },
	{ 0x6115, 0x6115, PDF_CMAP_SINGLE, 5798 },
	{ 0x611a, 0x611b, PDF_CMAP_TABLE, 849 },
	{ 0x611f, 0x611f, PDF_CMAP_SINGLE, 3514 },
	{ 0x6127, 0x6127, PDF_CMAP_SINGLE, 3860 },
	{ 0x6130, 0x6130, PDF_CMAP_SINGLE, 7909 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 7099 },
	{ 0x6137, 0x6137, PDF_CMAP_SINGLE, 3565 },
	{ 0x613c, 0x613c, PDF_CMAP_SINGLE, 5741 },
	{ 0x613e, 0x613f, PDF_CMAP_TABLE, 851 },
	{ 0x6142, 0x6142, PDF_CMAP_SINGLE, 6182 },
	{ 0x6144, 0x6144, PDF_CMAP_SINGLE, 4601 },
	{ 0x6147, 0x6148, PDF_CMAP_TABLE, 853 },
	{ 0x614a, 0x614c, PDF_CMAP_TABLE, 855 },
	{ 0x6153, 0x6153, PDF_CMAP_SINGLE, 7579 },
	{ 0x6155, 0x6155, PDF_CMAP_SINGLE, 4769 },
	{ 0x6158, 0x6159, PDF_CMAP_RANGE, 7085 },
	{ 0x615d, 0x615d, PDF_CMAP_SINGLE, 7471 },
	{ 0x615f, 0x615f, PDF_CMAP_SINGLE, 7453 },
	{ 0x6162, 0x6164, PDF_CMAP_TABLE, 858 },
	{ 0x6167, 0x6168, PDF_CMAP_TABLE, 861 },
	{ 0x616b, 0x616b, PDF_CMAP_SINGLE, 6841 },
	{ 0x616e, 0x616e, PDF_CMAP_SINGLE, 4445 },
	{ 0x6170, 0x6170, PDF_CMAP_SINGLE, 6292 },
	{ 0x6176, 0x6177, PDF_CMAP_TABLE, 863 },
	{ 0x617d, 0x617e, PDF_CMAP_TABLE, 865 },
	{ 0x6181, 0x6182, PDF_CMAP_TABLE, 867 },
	{ 0x618a, 0x618a, PDF_CMAP_SINGLE, 5190 },
	{ 0x618e, 0x618e, PDF_CMAP_SINGLE, 6932 },
	{ 0x6190, 0x6191, PDF_CMAP_TABLE, 869 },
	{ 0x6194, 0x6194, PDF_CMAP_SINGLE, 7217 },
	{ 0x6198, 0x619a, PDF_CMAP_TABLE, 871 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 5159 },
	{ 0x61a7, 0x61a7, PDF_CMAP_SINGLE, 4336 },
	{ 0x61a9, 0x61a9, PDF_CMAP_SINGLE, 3628 },
	{ 0x61ab, 0x61ac, PDF_CMAP_TABLE, 874 },
	{ 0x61ae, 0x61ae, PDF_CMAP_SINGLE, 4814 },
	{ 0x61b2, 0x61b2, PDF_CMAP_SINGLE, 7723 },
	{ 0x61b6, 0x61b6, PDF_CMAP_SINGLE, 5913 },
	{ 0x61ba, 0x61ba, PDF_CMAP_SINGLE, 4219 },
	{ 0x61be, 0x61be, PDF_CMAP_SINGLE, 3515 },
	{ 0x61c3, 0x61c3, PDF_CMAP_SINGLE, 4026 },
	{ 0x61c7, 0x61cb, PDF_CMAP_TABLE, 876 },
	{ 0x61e6, 0x61e6, PDF_CMAP_SINGLE, 4135 },
	{ 0x61f2, 0x61f2, PDF_CMAP_SINGLE, 7039 },
	{ 0x61f6, 0x61f8, PDF_CMAP_TABLE, 881 },
	{ 0x61fa, 0x61fa, PDF_CMAP_SINGLE, 7087 },
	{ 0x61fc, 0x61fc, PDF_CMAP_SINGLE, 3910 },
	{ 0x61ff, 0x6200, PDF_CMAP_TABLE, 884 },
	{ 0x6207, 0x6208, PDF_CMAP_TABLE, 886 },
	{ 0x620a, 0x620a, PDF_CMAP_SINGLE, 4816 },
	{ 0x620c, 0x620e, PDF_CMAP_TABLE, 888 },
	{ 0x6210, 0x6212, PDF_CMAP_TABLE, 891 },
	{ 0x6216, 0x6216, PDF_CMAP_SINGLE, 7842 },
	{ 0x621a, 0x621a, PDF_CMAP_SINGLE, 7138 },
	{ 0x621f, 0x621f, PDF_CMAP_SINGLE, 4019 },
	{ 0x6221, 0x6221, PDF_CMAP_SINGLE, 3516 },
	{ 0x622a, 0x622a, PDF_CMAP_SINGLE, 6685 },
	{ 0x622e, 0x622e, PDF_CMAP_SINGLE, 4592 },
	{ 0x6230, 0x6231, PDF_CMAP_TABLE, 894 },
	{ 0x6234, 0x6234, PDF_CMAP_SINGLE, 4258 },
	{ 0x6236, 0x6236, PDF_CMAP_SINGLE, 7808 },
	{ 0x623e, 0x6241, PDF_CMAP_TABLE, 896 },
	{ 0x6247, 0x6249, PDF_CMAP_TABLE, 900 },
	{ 0x624b, 0x624b, PDF_CMAP_SINGLE, 5580 },
	{ 0x624d, 0x624d, PDF_CMAP_SINGLE, 6574 },
	{ 0x6253, 0x6253, PDF_CMAP_SINGLE, 7386 },
	{ 0x6258, 0x6258, PDF_CMAP_SINGLE, 7398 },
	{ 0x626e, 0x626e, PDF_CMAP_SINGLE, 5160 },
	{ 0x6271, 0x6271, PDF_CMAP_SINGLE, 4054 },
	{ 0x6276, 0x6276, PDF_CMAP_SINGLE, 5124 },
	{ 0x6279, 0x6279, PDF_CMAP_SINGLE, 5192 },
	{ 0x627c, 0x627c, PDF_CMAP_SINGLE, 5852 },
	{ 0x627f, 0x6280, PDF_CMAP_TABLE, 903 },
	{ 0x6284, 0x6284, PDF_CMAP_SINGLE, 7218 },
	{ 0x6289, 0x628a, PDF_CMAP_TABLE, 905 },
	{ 0x6291, 0x6292, PDF_CMAP_TABLE, 907 },
	{ 0x6295, 0x6295, PDF_CMAP_SINGLE, 7468 },
	{ 0x6297, 0x6298, PDF_CMAP_TABLE, 909 },
	{ 0x629b, 0x629b, PDF_CMAP_SINGLE, 7553 },
	{ 0x62ab, 0x62ab, PDF_CMAP_SINGLE, 7598 },
	{ 0x62b1, 0x62b1, PDF_CMAP_SINGLE, 7554 },
	{ 0x62b5, 0x62b5, PDF_CMAP_SINGLE, 6597 },
	{ 0x62b9, 0x62b9, PDF_CMAP_SINGLE, 4690 },
	{ 0x62bc, 0x62bd, PDF_CMAP_TABLE, 911 },
	{ 0x62c2, 0x62c2, PDF_CMAP_SINGLE, 5174 },
	{ 0x62c7, 0x62c9, PDF_CMAP_TABLE, 913 },
	{ 0x62cc, 0x62cd, PDF_CMAP_TABLE, 916 },
	{ 0x62cf, 0x62d0, PDF_CMAP_TABLE, 918 },
	{ 0x62d2, 0x62d4, PDF_CMAP_TABLE, 920 },
	{ 0x62d6, 0x62d9, PDF_CMAP_TABLE, 923 },
	{ 0x62db, 0x62dc, PDF_CMAP_TABLE, 927 },
	{ 0x62ec, 0x62ef, PDF_CMAP_TABLE, 929 },
	{ 0x62f1, 0x62f1, PDF_CMAP_SINGLE, 3794 },
	{ 0x62f3, 0x62f3, PDF_CMAP_SINGLE, 3974 },
	{ 0x62f7, 0x62f7, PDF_CMAP_SINGLE, 3738 },
	{ 0x62fe, 0x62ff, PDF_CMAP_TABLE, 933 },
	{ 0x6301, 0x6301, PDF_CMAP_SINGLE, 6948 },
	{ 0x6307, 0x6307, PDF_CMAP_SINGLE, 6949 },
	{ 0x6309, 0x6309, PDF_CMAP_SINGLE, 5809 },
	{ 0x6311, 0x6311, PDF_CMAP_SINGLE, 4282 },
	{ 0x632b, 0x632b, PDF_CMAP_SINGLE, 6858 },
	{ 0x632f, 0x632f, PDF_CMAP_SINGLE, 6983 },
	{ 0x633a, 0x633b, PDF_CMAP_TABLE, 935 },
	{ 0x633d, 0x633e, PDF_CMAP_TABLE, 937 },
	{ 0x6349, 0x6349, PDF_CMAP_SINGLE, 7055 },
	{ 0x634c, 0x634c, PDF_CMAP_SINGLE, 7501 },
	{ 0x634f, 0x6350, PDF_CMAP_TABLE, 939 },
	{ 0x6355, 0x6355, PDF_CMAP_SINGLE, 7555 },
	{ 0x6367, 0x6368, PDF_CMAP_TABLE, 941 },
	{ 0x636e, 0x636e, PDF_CMAP_SINGLE, 3590 },
	{ 0x6372, 0x6372, PDF_CMAP_SINGLE, 3975 },
	{ 0x6377, 0x6377, PDF_CMAP_SINGLE, 7190 },
	{ 0x637a, 0x637b, PDF_CMAP_TABLE, 943 },
	{ 0x637f, 0x637f, PDF_CMAP_SINGLE, 5381 },
	{ 0x6383, 0x6383, PDF_CMAP_SINGLE, 5508 },
	{ 0x6388, 0x6389, PDF_CMAP_TABLE, 945 },
	{ 0x638c, 0x638c, PDF_CMAP_SINGLE, 6545 },
	{ 0x6392, 0x6392, PDF_CMAP_SINGLE, 4974 },
	{ 0x6396, 0x6396, PDF_CMAP_SINGLE, 5853 },
	{ 0x6398, 0x6398, PDF_CMAP_SINGLE, 3961 },
	{ 0x639b, 0x639b, PDF_CMAP_SINGLE, 3853 },
	{ 0x63a0, 0x63a2, PDF_CMAP_TABLE, 947 },
	{ 0x63a5, 0x63a5, PDF_CMAP_SINGLE, 6701 },
	{ 0x63a7, 0x63aa, PDF_CMAP_TABLE, 950 },
	{ 0x63c0, 0x63c0, PDF_CMAP_SINGLE, 3484 },
	{ 0x63c4, 0x63c4, PDF_CMAP_SINGLE, 6324 },
	{ 0x63c6, 0x63c6, PDF_CMAP_SINGLE, 3997 },
	{ 0x63cf, 0x63d0, PDF_CMAP_TABLE, 954 },
	{ 0x63d6, 0x63d6, PDF_CMAP_SINGLE, 6387 },
	{ 0x63da, 0x63db, PDF_CMAP_TABLE, 956 },
	{ 0x63e1, 0x63e1, PDF_CMAP_SINGLE, 5799 },
	{ 0x63ed, 0x63ee, PDF_CMAP_TABLE, 958 },
	{ 0x63f4, 0x63f4, PDF_CMAP_SINGLE, 6266 },
	{ 0x63f6, 0x63f7, PDF_CMAP_TABLE, 960 },
	{ 0x640d, 0x640d, PDF_CMAP_SINGLE, 5548 },
	{ 0x640f, 0x640f, PDF_CMAP_SINGLE, 4889 },
	{ 0x6414, 0x6414, PDF_CMAP_SINGLE, 5509 },
	{ 0x6416, 0x6417, PDF_CMAP_TABLE, 962 },
	{ 0x641c, 0x641c, PDF_CMAP_SINGLE, 5582 },
	{ 0x6422, 0x6422, PDF_CMAP_SINGLE, 6984 },
	{ 0x642c, 0x642d, PDF_CMAP_TABLE, 964 },
	{ 0x643a, 0x643a, PDF_CMAP_SINGLE, 8006 },
	{ 0x643e, 0x643e, PDF_CMAP_SINGLE, 7056 },
	{ 0x6458, 0x6458, PDF_CMAP_SINGLE, 6623 },
	{ 0x6460, 0x6460, PDF_CMAP_SINGLE, 7256 },
	{ 0x6469, 0x6469, PDF_CMAP_SINGLE, 4656 },
	{ 0x646f, 0x646f, PDF_CMAP_SINGLE, 6950 },
	{ 0x6478, 0x6479, PDF_CMAP_RANGE, 4770 },
	{ 0x647a, 0x647a, PDF_CMAP_SINGLE, 6702 },
	{ 0x6488, 0x6488, PDF_CMAP_SINGLE, 4512 },
	{ 0x6491, 0x6493, PDF_CMAP_TABLE, 966 },
	{ 0x649a, 0x649a, PDF_CMAP_SINGLE, 4161 },
	{ 0x649e, 0x649e, PDF_CMAP_SINGLE, 4244 },
	{ 0x64a4, 0x64a5, PDF_CMAP_TABLE, 969 },
	{ 0x64ab, 0x64ab, PDF_CMAP_SINGLE, 4818 },
	{ 0x64ad, 0x64ae, PDF_CMAP_TABLE, 971 },
	{ 0x64b0, 0x64b0, PDF_CMAP_SINGLE, 7062 },
	{ 0x64b2, 0x64b2, PDF_CMAP_SINGLE, 4890 },
	{ 0x64bb, 0x64bb, PDF_CMAP_SINGLE, 4212 },
	{ 0x64c1, 0x64c1, PDF_CMAP_SINGLE, 6091 },
	{ 0x64c4, 0x64c5, PDF_CMAP_TABLE, 973 },
	{ 0x64c7, 0x64c7, PDF_CMAP_SINGLE, 7445 },
	{ 0x64ca, 0x64ca, PDF_CMAP_SINGLE, 3630 },
	{ 0x64cd, 0x64ce, PDF_CMAP_TABLE, 975 },
	{ 0x64d2, 0x64d2, PDF_CMAP_SINGLE, 4040 },
	{ 0x64d4, 0x64d4, PDF_CMAP_SINGLE, 4220 },
	{ 0x64d8, 0x64d8, PDF_CMAP_SINGLE, 5023 },
	{ 0x64da, 0x64da, PDF_CMAP_SINGLE, 3591 },
	{ 0x64e1, 0x64e2, PDF_CMAP_TABLE, 977 },
	{ 0x64e5, 0x64e7, PDF_CMAP_TABLE, 979 },
	{ 0x64ec, 0x64ec, PDF_CMAP_SINGLE, 6400 },
	{ 0x64f2, 0x64f2, PDF_CMAP_SINGLE, 7140 },
	{ 0x64f4, 0x64f4, PDF_CMAP_SINGLE, 7877 },
	{ 0x64fa, 0x64fa, PDF_CMAP_SINGLE, 7479 },
	{ 0x64fe, 0x64fe, PDF_CMAP_SINGLE, 6151 },
	{ 0x6500, 0x6500, PDF_CMAP_SINGLE, 4911 },
	{ 0x6504, 0x6504, PDF_CMAP_SINGLE, 7448 },
	{ 0x6518, 0x6518, PDF_CMAP_SINGLE, 5884 },
	{ 0x651d, 0x651d, PDF_CMAP_SINGLE, 5471 },
	{ 0x6523, 0x6523, PDF_CMAP_SINGLE, 4467 },
	{ 0x652a, 0x652c, PDF_CMAP_TABLE, 982 },
	{ 0x652f, 0x652f, PDF_CMAP_SINGLE, 6951 },
	{ 0x6536, 0x6539, PDF_CMAP_TABLE, 985 },
	{ 0x653b, 0x653b, PDF_CMAP_SINGLE, 3796 },
	{ 0x653e, 0x653f, PDF_CMAP_TABLE, 989 },
	{ 0x6545, 0x6545, PDF_CMAP_SINGLE, 3740 },
	{ 0x6548, 0x6548, PDF_CMAP_SINGLE, 7956 },
	{ 0x654d, 0x654f, PDF_CMAP_TABLE, 991 },
	{ 0x6551, 0x6551, PDF_CMAP_SINGLE, 3912 },
	{ 0x6556, 0x6557, PDF_CMAP_TABLE, 994 },
	{ 0x655e, 0x655e, PDF_CMAP_SINGLE, 7100 },
	{ 0x6562, 0x6563, PDF_CMAP_TABLE, 996 },
	{ 0x6566, 0x6566, PDF_CMAP_SINGLE, 4321 },
	{ 0x656c, 0x656d, PDF_CMAP_TABLE, 998 },
	{ 0x6572, 0x6572, PDF_CMAP_SINGLE, 3741 },
	{ 0x6574, 0x6575, PDF_CMAP_TABLE, 1000 },
	{ 0x6577, 0x6578, PDF_CMAP_TABLE, 1002 },
	{ 0x657e, 0x657e, PDF_CMAP_SINGLE, 5425 },
	{ 0x6582, 0x6583, PDF_CMAP_TABLE, 1004 },
	{ 0x6585, 0x6585, PDF_CMAP_SINGLE, 7957 },
	{ 0x6587, 0x6587, PDF_CMAP_SINGLE, 4840 },
	{ 0x658c, 0x658c, PDF_CMAP_SINGLE, 5226 },
	{ 0x6590, 0x6591, PDF_CMAP_TABLE, 1006 },
	{ 0x6597, 0x6597, PDF_CMAP_SINGLE, 4349 },
	{ 0x6599, 0x6599, PDF_CMAP_SINGLE, 4555 },
	{ 0x659b, 0x659c, PDF_CMAP_TABLE, 1008 },
	{ 0x659f, 0x659f, PDF_CMAP_SINGLE, 7030 },
	{ 0x65a1, 0x65a1, PDF_CMAP_SINGLE, 5817 },
	{ 0x65a4, 0x65a5, PDF_CMAP_TABLE, 1010 },
	{ 0x65a7, 0x65a7, PDF_CMAP_SINGLE, 5126 },
	{ 0x65ab, 0x65ac, PDF_CMAP_TABLE, 1012 },
	{ 0x65af, 0x65b0, PDF_CMAP_TABLE, 1014 },
	{ 0x65b7, 0x65b7, PDF_CMAP_SINGLE, 4199 },
	{ 0x65b9, 0x65b9, PDF_CMAP_SINGLE, 4950 },
	{ 0x65bc, 0x65bd, PDF_CMAP_TABLE, 1016 },
	{ 0x65c1, 0x65c1, PDF_CMAP_SINGLE, 4951 },
	{ 0x65c5, 0x65c5, PDF_CMAP_SINGLE, 4447 },
	{ 0x65cb, 0x65cc, PDF_CMAP_TABLE, 1018 },
	{ 0x65cf, 0x65cf, PDF_CMAP_SINGLE, 6828 },
	{ 0x65d2, 0x65d2, PDF_CMAP_SINGLE, 4578 },
	{ 0x65d7, 0x65d7, PDF_CMAP_SINGLE, 4081 },
	{ 0x65e0, 0x65e0, PDF_CMAP_SINGLE, 4819 },
	{ 0x65e3, 0x65e3, PDF_CMAP_SINGLE, 4082 },
	{ 0x65e5, 0x65e6, PDF_CMAP_TABLE, 1020 },
	{ 0x65e8, 0x65e9, PDF_CMAP_TABLE, 1022 },
	{ 0x65ec, 0x65ed, PDF_CMAP_TABLE, 1024 },
	{ 0x65f1, 0x65f1, PDF_CMAP_SINGLE, 7638 },
	{ 0x65f4, 0x65f4, PDF_CMAP_SINGLE, 6209 },
	{ 0x65fa, 0x65fa, PDF_CMAP_SINGLE, 6127 },
	{ 0x65fb, 0x65fc, PDF_CMAP_RANGE, 4875 },
	{ 0x65fd, 0x65fd, PDF_CMAP_SINGLE, 4322 },
	{ 0x65ff, 0x65ff, PDF_CMAP_SINGLE, 6065 },
	{ 0x6606, 0x6607, PDF_CMAP_TABLE, 1026 },
	{ 0x6609, 0x660a, PDF_CMAP_TABLE, 1028 },
	{ 0x660c, 0x660c, PDF_CMAP_SINGLE, 7101 },
	{ 0x660e, 0x6611, PDF_CMAP_TABLE, 1030 },
	{ 0x6613, 0x6615, PDF_CMAP_TABLE, 1034 },
	{ 0x661e, 0x6620, PDF_CMAP_TABLE, 1037 },
	{ 0x6625, 0x6625, PDF_CMAP_SINGLE, 7300 },
	{ 0x6627, 0x6628, PDF_CMAP_TABLE, 1040 },
	{ 0x662d, 0x662d, PDF_CMAP_SINGLE, 5510 },
	{ 0x662f, 0x6631, PDF_CMAP_TABLE, 1042 },
	{ 0x6634, 0x6634, PDF_CMAP_SINGLE, 4805 },
	{ 0x6636, 0x6636, PDF_CMAP_SINGLE, 7102 },
	{ 0x663a, 0x663b, PDF_CMAP_TABLE, 1045 },
	{ 0x6641, 0x6642, PDF_CMAP_TABLE, 1047 },
	{ 0x6643, 0x6644, PDF_CMAP_RANGE, 7911 },
	{ 0x6649, 0x6649, PDF_CMAP_SINGLE, 6985 },
	{ 0x664b, 0x664b, PDF_CMAP_SINGLE, 6986 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 5810 },
	{ 0x6659, 0x6659, PDF_CMAP_SINGLE, 6908 },
	{ 0x665b, 0x665b, PDF_CMAP_SINGLE, 7738 },
	{ 0x665d, 0x665f, PDF_CMAP_TABLE, 1049 },
	{ 0x6664, 0x6669, PDF_CMAP_TABLE, 1052 },
	{ 0x666b, 0x666b, PDF_CMAP_SINGLE, 7400 },
	{ 0x666e, 0x666f, PDF_CMAP_TABLE, 1058 },
	{ 0x6673, 0x6674, PDF_CMAP_TABLE, 1060 },
	{ 0x6676, 0x6678, PDF_CMAP_TABLE, 1062 },
	{ 0x667a, 0x667a, PDF_CMAP_SINGLE, 6953 },
	{ 0x6684, 0x6684, PDF_CMAP_SINGLE, 7991 },
	{ 0x6687, 0x6689, PDF_CMAP_TABLE, 1065 },
	{ 0x668e, 0x668e, PDF_CMAP_SINGLE, 6006 },
	{ 0x6690, 0x6691, PDF_CMAP_TABLE, 1068 },
	{ 0x6696, 0x6698, PDF_CMAP_TABLE, 1070 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 4752 },
	{ 0x66a0, 0x66a0, PDF_CMAP_SINGLE, 3742 },
	{ 0x66a2, 0x66a2, PDF_CMAP_SINGLE, 7103 },
	{ 0x66ab, 0x66ab, PDF_CMAP_SINGLE, 6528 },
	{ 0x66ae, 0x66ae, PDF_CMAP_SINGLE, 4772 },
	{ 0x66b2, 0x66b4, PDF_CMAP_TABLE, 1073 },
	{ 0x66b9, 0x66b9, PDF_CMAP_SINGLE, 5464 },
	{ 0x66bb, 0x66bb, PDF_CMAP_SINGLE, 3677 },
	{ 0x66be, 0x66be, PDF_CMAP_SINGLE, 4323 },
	{ 0x66c4, 0x66c4, PDF_CMAP_SINGLE, 5999 },
	{ 0x66c6, 0x66c7, PDF_CMAP_TABLE, 1076 },
	{ 0x66c9, 0x66c9, PDF_CMAP_SINGLE, 7958 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 5845 },
	{ 0x66d9, 0x66d9, PDF_CMAP_SINGLE, 5384 },
	{ 0x66dc, 0x66dd, PDF_CMAP_TABLE, 1078 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 3844 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 8047 },
	{ 0x66f0, 0x66f0, PDF_CMAP_SINGLE, 6125 },
	{ 0x66f2, 0x66f4, PDF_CMAP_TABLE, 1080 },
	{ 0x66f7, 0x66fa, PDF_CMAP_TABLE, 1083 },
	{ 0x66fc, 0x66fc, PDF_CMAP_SINGLE, 4678 },
	{ 0x66fe, 0x6700, PDF_CMAP_TABLE, 1087 },
	{ 0x6703, 0x6703, PDF_CMAP_SINGLE, 7936 },
	{ 0x6708, 0x6709, PDF_CMAP_TABLE, 1090 },
	{ 0x670b, 0x670b, PDF_CMAP_SINGLE, 5176 },
	{ 0x670d, 0x670d, PDF_CMAP_SINGLE, 5080 },
	{ 0x6714, 0x6715, PDF_CMAP_TABLE, 1092 },
	{ 0x6717, 0x6717, PDF_CMAP_SINGLE, 4413 },
	{ 0x671b, 0x671b, PDF_CMAP_SINGLE, 4700 },
	{ 0x671d, 0x671d, PDF_CMAP_SINGLE, 6795 },
	{ 0x671e, 0x671f, PDF_CMAP_RANGE, 4083 },
	{ 0x6726, 0x6728, PDF_CMAP_TABLE, 1094 },
	{ 0x672a, 0x672e, PDF_CMAP_TABLE, 1097 },
	{ 0x6731, 0x6731, PDF_CMAP_SINGLE, 6874 },
	{ 0x6734, 0x6734, PDF_CMAP_SINGLE, 4891 },
	{ 0x6736, 0x6736, PDF_CMAP_SINGLE, 7388 },
	{ 0x673a, 0x673a, PDF_CMAP_SINGLE, 3984 },
	{ 0x673d, 0x673d, PDF_CMAP_SINGLE, 7975 },
	{ 0x6746, 0x6746, PDF_CMAP_SINGLE, 3485 },
	{ 0x6749, 0x6749, PDF_CMAP_SINGLE, 5321 },
	{ 0x674e, 0x6751, PDF_CMAP_TABLE, 1102 },
	{ 0x6753, 0x6753, PDF_CMAP_SINGLE, 7580 },
	{ 0x6756, 0x6756, PDF_CMAP_SINGLE, 6547 },
	{ 0x675c, 0x675c, PDF_CMAP_SINGLE, 4350 },
	{ 0x675e, 0x675f, PDF_CMAP_TABLE, 1106 },
	{ 0x676d, 0x676d, PDF_CMAP_SINGLE, 7677 },
	{ 0x676f, 0x6771, PDF_CMAP_TABLE, 1108 },
	{ 0x6773, 0x6773, PDF_CMAP_SINGLE, 4806 },
	{ 0x6775, 0x6775, PDF_CMAP_SINGLE, 6598 },
	{ 0x6777, 0x6777, PDF_CMAP_SINGLE, 7480 },
	{ 0x677b, 0x677b, PDF_CMAP_SINGLE, 4183 },
	{ 0x677e, 0x677f, PDF_CMAP_TABLE, 1111 },
	{ 0x6787, 0x6787, PDF_CMAP_SINGLE, 5194 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 6128 },
	{ 0x678b, 0x678b, PDF_CMAP_SINGLE, 4953 },
	{ 0x678f, 0x6790, PDF_CMAP_TABLE, 1113 },
	{ 0x6793, 0x6793, PDF_CMAP_SINGLE, 4351 },
	{ 0x6795, 0x6795, PDF_CMAP_SINGLE, 7369 },
	{ 0x6797, 0x6797, PDF_CMAP_SINGLE, 4647 },
	{ 0x679a, 0x679a, PDF_CMAP_SINGLE, 4713 },
	{ 0x679c, 0x679d, PDF_CMAP_TABLE, 1115 },
	{ 0x67af, 0x67b0, PDF_CMAP_TABLE, 1117 },
	{ 0x67b3, 0x67b3, PDF_CMAP_SINGLE, 6955 },
	{ 0x67b6, 0x67b7, PDF_CMAP_RANGE, 3448 },
	{ 0x67b8, 0x67b8, PDF_CMAP_SINGLE, 3913 },
	{ 0x67be, 0x67be, PDF_CMAP_SINGLE, 5706 },
	{ 0x67c4, 0x67c4, PDF_CMAP_SINGLE, 5048 },
	{ 0x67cf, 0x67d4, PDF_CMAP_TABLE, 1119 },
	{ 0x67da, 0x67da, PDF_CMAP_SINGLE, 6328 },
	{ 0x67dd, 0x67dd, PDF_CMAP_SINGLE, 7401 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 3914 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 3486 },
	{ 0x67ef, 0x67f1, PDF_CMAP_TABLE, 1125 },
	{ 0x67f3, 0x67f6, PDF_CMAP_TABLE, 1128 },
	{ 0x67fb, 0x67fb, PDF_CMAP_SINGLE, 5269 },
	{ 0x67fe, 0x67fe, PDF_CMAP_SINGLE, 6723 },
	{ 0x6812, 0x6813, PDF_CMAP_TABLE, 1132 },
	{ 0x6816, 0x6817, PDF_CMAP_TABLE, 1134 },
	{ 0x6821, 0x6822, PDF_CMAP_TABLE, 1136 },
	{ 0x682a, 0x682a, PDF_CMAP_SINGLE, 6876 },
	{ 0x682f, 0x682f, PDF_CMAP_SINGLE, 6233 },
	{ 0x6838, 0x6839, PDF_CMAP_TABLE, 1138 },
	{ 0x683c, 0x683d, PDF_CMAP_TABLE, 1140 },
	{ 0x6840, 0x6843, PDF_CMAP_TABLE, 1142 },
	{ 0x6848, 0x6848, PDF_CMAP_SINGLE, 5811 },
	{ 0x684e, 0x684e, PDF_CMAP_SINGLE, 7020 },
	{ 0x6850, 0x6851, PDF_CMAP_TABLE, 1146 },
	{ 0x6853, 0x6854, PDF_CMAP_TABLE, 1148 },
	{ 0x686d, 0x686d, PDF_CMAP_SINGLE, 6987 },
	{ 0x6876, 0x6876, PDF_CMAP_SINGLE, 7454 },
	{ 0x687f, 0x687f, PDF_CMAP_SINGLE, 3487 },
	{ 0x6881, 0x6881, PDF_CMAP_SINGLE, 4431 },
	{ 0x6885, 0x6885, PDF_CMAP_SINGLE, 4714 },
	{ 0x688f, 0x688f, PDF_CMAP_SINGLE, 3770 },
	{ 0x6893, 0x6894, PDF_CMAP_TABLE, 1150 },
	{ 0x6897, 0x6897, PDF_CMAP_SINGLE, 3679 },
	{ 0x689d, 0x689d, PDF_CMAP_SINGLE, 6796 },
	{ 0x689f, 0x689f, PDF_CMAP_SINGLE, 7959 },
	{ 0x68a1, 0x68a2, PDF_CMAP_TABLE, 1152 },
	{ 0x68a7, 0x68a8, PDF_CMAP_TABLE, 1154 },
	{ 0x68ad, 0x68ad, PDF_CMAP_SINGLE, 5270 },
	{ 0x68af, 0x68b1, PDF_CMAP_TABLE, 1156 },
	{ 0x68b3, 0x68b3, PDF_CMAP_SINGLE, 5511 },
	{ 0x68b5, 0x68b6, PDF_CMAP_TABLE, 1159 },
	{ 0x68c4, 0x68c5, PDF_CMAP_TABLE, 1161 },
	{ 0x68c9, 0x68c9, PDF_CMAP_SINGLE, 4738 },
	{ 0x68cb, 0x68cb, PDF_CMAP_SINGLE, 4086 },
	{ 0x68cd, 0x68cd, PDF_CMAP_SINGLE, 3779 },
	{ 0x68d2, 0x68d2, PDF_CMAP_SINGLE, 5099 },
	{ 0x68d5, 0x68d5, PDF_CMAP_SINGLE, 6842 },
	{ 0x68d7, 0x68d8, PDF_CMAP_TABLE, 1163 },
	{ 0x68da, 0x68da, PDF_CMAP_SINGLE, 5177 },
	{ 0x68df, 0x68e0, PDF_CMAP_TABLE, 1165 },
	{ 0x68e7, 0x68e8, PDF_CMAP_TABLE, 1167 },
	{ 0x68ee, 0x68ee, PDF_CMAP_SINGLE, 5322 },
	{ 0x68f2, 0x68f2, PDF_CMAP_SINGLE, 5387 },
	{ 0x68f9, 0x68fa, PDF_CMAP_TABLE, 1169 },
	{ 0x6900, 0x6900, PDF_CMAP_SINGLE, 6111 },
	{ 0x6905, 0x6905, PDF_CMAP_SINGLE, 6401 },
	{ 0x690d, 0x690e, PDF_CMAP_TABLE, 1171 },
	{ 0x6912, 0x6912, PDF_CMAP_SINGLE, 7221 },
	{ 0x6927, 0x6927, PDF_CMAP_SINGLE, 4753 },
	{ 0x6930, 0x6930, PDF_CMAP_SINGLE, 5868 },
	{ 0x693d, 0x693d, PDF_CMAP_SINGLE, 5964 },
	{ 0x693f, 0x693f, PDF_CMAP_SINGLE, 7301 },
	{ 0x694a, 0x694a, PDF_CMAP_SINGLE, 5887 },
	{ 0x6953, 0x6955, PDF_CMAP_TABLE, 1173 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 3607 },
	{ 0x6959, 0x695a, PDF_CMAP_TABLE, 1176 },
	{ 0x695e, 0x695e, PDF_CMAP_SINGLE, 4608 },
	{ 0x6960, 0x6960, PDF_CMAP_SINGLE, 4147 },
	{ 0x6961, 0x6962, PDF_CMAP_RANGE, 6329 },
	{ 0x6963, 0x6963, PDF_CMAP_SINGLE, 4859 },
	{ 0x6968, 0x6968, PDF_CMAP_SINGLE, 6724 },
	{ 0x696b, 0x696b, PDF_CMAP_SINGLE, 6928 },
	{ 0x696d, 0x696f, PDF_CMAP_TABLE, 1178 },
	{ 0x6975, 0x6975, PDF_CMAP_SINGLE, 4021 },
	{ 0x6977, 0x6979, PDF_CMAP_TABLE, 1181 },
	{ 0x6995, 0x6995, PDF_CMAP_SINGLE, 6183 },
	{ 0x699b, 0x699c, PDF_CMAP_TABLE, 1184 },
	{ 0x69a5, 0x69a5, PDF_CMAP_SINGLE, 7913 },
	{ 0x69a7, 0x69a7, PDF_CMAP_SINGLE, 5195 },
	{ 0x69ae, 0x69ae, PDF_CMAP_SINGLE, 6008 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 4580 },
	{ 0x69bb, 0x69bb, PDF_CMAP_SINGLE, 7426 },
	{ 0x69c1, 0x69c1, PDF_CMAP_SINGLE, 3744 },
	{ 0x69c3, 0x69c3, PDF_CMAP_SINGLE, 4913 },
	{ 0x69cb, 0x69cd, PDF_CMAP_TABLE, 1186 },
	{ 0x69d0, 0x69d0, PDF_CMAP_SINGLE, 3862 },
	{ 0x69e8, 0x69e8, PDF_CMAP_SINGLE, 3816 },
	{ 0x69ea, 0x69ea, PDF_CMAP_SINGLE, 3569 },
	{ 0x69fb, 0x69fb, PDF_CMAP_SINGLE, 3998 },
	{ 0x69fd, 0x69fd, PDF_CMAP_SINGLE, 6798 },
	{ 0x69ff, 0x69ff, PDF_CMAP_SINGLE, 4029 },
	{ 0x6a02, 0x6a02, PDF_CMAP_SINGLE, 5800 },
	{ 0x6a0a, 0x6a0a, PDF_CMAP_SINGLE, 4997 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 4432 },
	{ 0x6a13, 0x6a13, PDF_CMAP_SINGLE, 4567 },
	{ 0x6a17, 0x6a17, PDF_CMAP_SINGLE, 6600 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 7581 },
	{ 0x6a1e, 0x6a1f, PDF_CMAP_TABLE, 1189 },
	{ 0x6a21, 0x6a21, PDF_CMAP_SINGLE, 4774 },
	{ 0x6a23, 0x6a23, PDF_CMAP_SINGLE, 5888 },
	{ 0x6a35, 0x6a35, PDF_CMAP_SINGLE, 7223 },
	{ 0x6a38, 0x6a3a, PDF_CMAP_TABLE, 1191 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 6909 },
	{ 0x6a44, 0x6a44, PDF_CMAP_SINGLE, 3519 },
	{ 0x6a48, 0x6a48, PDF_CMAP_SINGLE, 6153 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 3878 },
	{ 0x6a52, 0x6a53, PDF_CMAP_TABLE, 1194 },
	{ 0x6a58, 0x6a59, PDF_CMAP_TABLE, 1196 },
	{ 0x6a5f, 0x6a5f, PDF_CMAP_SINGLE, 4088 },
	{ 0x6a61, 0x6a61, PDF_CMAP_SINGLE, 5347 },
	{ 0x6a6b, 0x6a6b, PDF_CMAP_SINGLE, 7951 },
	{ 0x6a80, 0x6a80, PDF_CMAP_SINGLE, 4201 },
	{ 0x6a84, 0x6a84, PDF_CMAP_SINGLE, 3632 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 6725 },
	{ 0x6a8d, 0x6a8e, PDF_CMAP_TABLE, 1198 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 5024 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 7937 },
	{ 0x6aa2, 0x6aa3, PDF_CMAP_TABLE, 1200 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 5227 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 7656 },
	{ 0x6ac2, 0x6ac3, PDF_CMAP_TABLE, 1202 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 4514 },
	{ 0x6ada, 0x6adb, PDF_CMAP_TABLE, 1204 },
	{ 0x6af6, 0x6af6, PDF_CMAP_SINGLE, 7724 },
	{ 0x6afb, 0x6afb, PDF_CMAP_SINGLE, 5858 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 4391 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 3976 },
	{ 0x6b0c, 0x6b0c, PDF_CMAP_SINGLE, 6550 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 4392 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 4402 },
	{ 0x6b20, 0x6b21, PDF_CMAP_TABLE, 1206 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 8020 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 6169 },
	{ 0x6b3a, 0x6b3a, PDF_CMAP_SINGLE, 4089 },
	{ 0x6b3d, 0x6b3e, PDF_CMAP_TABLE, 1208 },
	{ 0x6b46, 0x6b47, PDF_CMAP_TABLE, 1210 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 3451 },
	{ 0x6b4e, 0x6b4e, PDF_CMAP_SINGLE, 7413 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 3916 },
	{ 0x6b5f, 0x6b5f, PDF_CMAP_SINGLE, 5937 },
	{ 0x6b61, 0x6b66, PDF_CMAP_TABLE, 1212 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 6133 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 5494 },
	{ 0x6b77, 0x6b78, PDF_CMAP_TABLE, 1218 },
	{ 0x6b7b, 0x6b7b, PDF_CMAP_SINGLE, 5271 },
	{ 0x6b7f, 0x6b7f, PDF_CMAP_SINGLE, 4795 },
	{ 0x6b83, 0x6b84, PDF_CMAP_TABLE, 1220 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 7436 },
	{ 0x6b89, 0x6b8a, PDF_CMAP_TABLE, 1222 },
	{ 0x6b96, 0x6b96, PDF_CMAP_SINGLE, 5726 },
	{ 0x6b98, 0x6b98, PDF_CMAP_SINGLE, 6524 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 6240 },
	{ 0x6bae, 0x6baf, PDF_CMAP_TABLE, 1224 },
	{ 0x6bb2, 0x6bb2, PDF_CMAP_SINGLE, 5465 },
	{ 0x6bb5, 0x6bb5, PDF_CMAP_SINGLE, 4202 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 6376 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 5317 },
	{ 0x6bbc, 0x6bbc, PDF_CMAP_SINGLE, 3470 },
	{ 0x6bbf, 0x6bbf, PDF_CMAP_SINGLE, 6659 },
	{ 0x6bc1, 0x6bc1, PDF_CMAP_SINGLE, 7996 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_TABLE, 1226 },
	{ 0x6bcb, 0x6bcb, PDF_CMAP_SINGLE, 4822 },
	{ 0x6bcd, 0x6bcd, PDF_CMAP_SINGLE, 4775 },
	{ 0x6bcf, 0x6bcf, PDF_CMAP_SINGLE, 4715 },
	{ 0x6bd2, 0x6bd4, PDF_CMAP_TABLE, 1228 },
	{ 0x6bd6, 0x6bd8, PDF_CMAP_RANGE, 5197 },
	{ 0x6bdb, 0x6bdb, PDF_CMAP_SINGLE, 4776 },
	{ 0x6beb, 0x6bec, PDF_CMAP_TABLE, 1231 },
	{ 0x6c08, 0x6c08, PDF_CMAP_SINGLE, 6660 },
	{ 0x6c0f, 0x6c0f, PDF_CMAP_SINGLE, 5774 },
	{ 0x6c11, 0x6c11, PDF_CMAP_SINGLE, 4877 },
	{ 0x6c13, 0x6c13, PDF_CMAP_SINGLE, 4728 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 4090 },
	{ 0x6c34, 0x6c34, PDF_CMAP_SINGLE, 5587 },
	{ 0x6c37, 0x6c38, PDF_CMAP_TABLE, 1233 },
	{ 0x6c3e, 0x6c3e, PDF_CMAP_SINGLE, 5012 },
	{ 0x6c40, 0x6c42, PDF_CMAP_TABLE, 1235 },
	{ 0x6c4e, 0x6c4e, PDF_CMAP_SINGLE, 5013 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 5410 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 5307 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 7639 },
	{ 0x6c5a, 0x6c5a, PDF_CMAP_SINGLE, 6068 },
	{ 0x6c5d, 0x6c60, PDF_CMAP_TABLE, 1238 },
	{ 0x6c68, 0x6c68, PDF_CMAP_SINGLE, 3784 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 6129 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 6034 },
	{ 0x6c70, 0x6c70, PDF_CMAP_SINGLE, 7437 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 4055 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 4841 },
	{ 0x6c7a, 0x6c7a, PDF_CMAP_SINGLE, 3649 },
	{ 0x6c7d, 0x6c7e, PDF_CMAP_TABLE, 1242 },
	{ 0x6c81, 0x6c83, PDF_CMAP_TABLE, 1244 },
	{ 0x6c85, 0x6c88, PDF_CMAP_TABLE, 1247 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 4324 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 4789 },
	{ 0x6c92, 0x6c96, PDF_CMAP_TABLE, 1251 },
	{ 0x6c99, 0x6c9b, PDF_CMAP_TABLE, 1256 },
	{ 0x6cab, 0x6cab, PDF_CMAP_SINGLE, 4692 },
	{ 0x6cae, 0x6cae, PDF_CMAP_SINGLE, 6601 },
	{ 0x6cb3, 0x6cb3, PDF_CMAP_SINGLE, 7622 },
	{ 0x6cb8, 0x6cb9, PDF_CMAP_TABLE, 1259 },
	{ 0x6cbb, 0x6cbf, PDF_CMAP_TABLE, 1261 },
	{ 0x6cc1, 0x6cc2, PDF_CMAP_TABLE, 1266 },
	{ 0x6cc4, 0x6cc4, PDF_CMAP_SINGLE, 5453 },
	{ 0x6cc9, 0x6cca, PDF_CMAP_TABLE, 1268 },
	{ 0x6ccc, 0x6ccc, PDF_CMAP_SINGLE, 7607 },
	{ 0x6cd3, 0x6cd3, PDF_CMAP_SINGLE, 7856 },
	{ 0x6cd5, 0x6cd5, PDF_CMAP_SINGLE, 5018 },
	{ 0x6cd7, 0x6cd7, PDF_CMAP_SINGLE, 5273 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 5014 },
	{ 0x6ce1, 0x6ce3, PDF_CMAP_TABLE, 1270 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 4187 },
	{ 0x6ce8, 0x6ce8, PDF_CMAP_SINGLE, 6877 },
	{ 0x6ceb, 0x6ceb, PDF_CMAP_SINGLE, 7739 },
	{ 0x6cee, 0x6cf0, PDF_CMAP_TABLE, 1273 },
	{ 0x6cf3, 0x6cf3, PDF_CMAP_SINGLE, 6010 },
	{ 0x6d0b, 0x6d0c, PDF_CMAP_TABLE, 1276 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 5064 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 5495 },
	{ 0x6d19, 0x6d19, PDF_CMAP_SINGLE, 5588 },
	{ 0x6d1b, 0x6d1b, PDF_CMAP_SINGLE, 4382 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 4340 },
	{ 0x6d25, 0x6d25, PDF_CMAP_SINGLE, 6990 },
	{ 0x6d27, 0x6d27, PDF_CMAP_SINGLE, 6332 },
	{ 0x6d29, 0x6d2a, PDF_CMAP_TABLE, 1278 },
	{ 0x6d32, 0x6d32, PDF_CMAP_SINGLE, 6878 },
	{ 0x6d35, 0x6d36, PDF_CMAP_TABLE, 1280 },
	{ 0x6d38, 0x6d39, PDF_CMAP_TABLE, 1282 },
	{ 0x6d3b, 0x6d3b, PDF_CMAP_SINGLE, 7899 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_TABLE, 1284 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 4581 },
	{ 0x6d59, 0x6d5a, PDF_CMAP_TABLE, 1286 },
	{ 0x6d5c, 0x6d5c, PDF_CMAP_SINGLE, 5229 },
	{ 0x6d63, 0x6d63, PDF_CMAP_SINGLE, 6112 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 7557 },
	{ 0x6d69, 0x6d6a, PDF_CMAP_TABLE, 1288 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 4622 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 5127 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 6170 },
	{ 0x6d77, 0x6d79, PDF_CMAP_TABLE, 1290 },
	{ 0x6d7f, 0x6d7f, PDF_CMAP_SINGLE, 7507 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 5986 },
	{ 0x6d87, 0x6d89, PDF_CMAP_TABLE, 1293 },
	{ 0x6d8c, 0x6d8e, PDF_CMAP_TABLE, 1296 },
	{ 0x6d91, 0x6d91, PDF_CMAP_SINGLE, 5540 },
	{ 0x6d93, 0x6d93, PDF_CMAP_SINGLE, 5968 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 7207 },
	{ 0x6daf, 0x6daf, PDF_CMAP_SINGLE, 5846 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 5854 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 7657 },
	{ 0x6dc0, 0x6dc0, PDF_CMAP_SINGLE, 6728 },
	{ 0x6dc3, 0x6dc7, PDF_CMAP_TABLE, 1299 },
	{ 0x6dcb, 0x6dcb, PDF_CMAP_SINGLE, 4648 },
	{ 0x6dcf, 0x6dcf, PDF_CMAP_SINGLE, 7814 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 5633 },
	{ 0x6dd8, 0x6dda, PDF_CMAP_TABLE, 1304 },
	{ 0x6dde, 0x6dde, PDF_CMAP_SINGLE, 5556 },
	{ 0x6de1, 0x6de1, PDF_CMAP_SINGLE, 4222 },
	{ 0x6de8, 0x6de8, PDF_CMAP_SINGLE, 6729 },
	{ 0x6dea, 0x6deb, PDF_CMAP_TABLE, 1307 },
	{ 0x6dee, 0x6dee, PDF_CMAP_SINGLE, 7938 },
	{ 0x6df1, 0x6df1, PDF_CMAP_SINGLE, 5766 },
	{ 0x6df3, 0x6df3, PDF_CMAP_SINGLE, 5650 },
	{ 0x6df5, 0x6df5, PDF_CMAP_SINGLE, 5969 },
	{ 0x6df7, 0x6dfb, PDF_CMAP_TABLE, 1309 },
	{ 0x6e17, 0x6e17, PDF_CMAP_SINGLE, 5323 },
	{ 0x6e19, 0x6e1b, PDF_CMAP_TABLE, 1314 },
	{ 0x6e1f, 0x6e21, PDF_CMAP_TABLE, 1317 },
	{ 0x6e23, 0x6e26, PDF_CMAP_TABLE, 1320 },
	{ 0x6e2b, 0x6e2d, PDF_CMAP_TABLE, 1324 },
	{ 0x6e2f, 0x6e2f, PDF_CMAP_SINGLE, 7680 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 5427 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 3503 },
	{ 0x6e36, 0x6e36, PDF_CMAP_SINGLE, 6011 },
	{ 0x6e38, 0x6e38, PDF_CMAP_SINGLE, 6333 },
	{ 0x6e3a, 0x6e3a, PDF_CMAP_SINGLE, 4807 },
	{ 0x6e3c, 0x6e3e, PDF_CMAP_TABLE, 1327 },
	{ 0x6e43, 0x6e44, PDF_CMAP_TABLE, 1330 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 6879 },
	{ 0x6e4d, 0x6e4d, PDF_CMAP_SINGLE, 4203 },
	{ 0x6e56, 0x6e56, PDF_CMAP_SINGLE, 7815 },
	{ 0x6e58, 0x6e58, PDF_CMAP_SINGLE, 5348 },
	{ 0x6e5b, 0x6e5c, PDF_CMAP_TABLE, 1332 },
	{ 0x6e5e, 0x6e5f, PDF_CMAP_TABLE, 1334 },
	{ 0x6e67, 0x6e67, PDF_CMAP_SINGLE, 6185 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 7271 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 1336 },
	{ 0x6e72, 0x6e73, PDF_CMAP_TABLE, 1338 },
	{ 0x6e7a, 0x6e7a, PDF_CMAP_SINGLE, 5065 },
	{ 0x6e90, 0x6e90, PDF_CMAP_SINGLE, 6270 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 6911 },
	{ 0x6e9c, 0x6e9d, PDF_CMAP_TABLE, 1340 },
	{ 0x6e9f, 0x6e9f, PDF_CMAP_SINGLE, 4754 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 6465 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 5128 },
	{ 0x6eaa, 0x6eab, PDF_CMAP_TABLE, 1342 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 5514 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 6991 },
	{ 0x6eb6, 0x6eb6, PDF_CMAP_SINGLE, 6186 },
	{ 0x6eba, 0x6eba, PDF_CMAP_SINGLE, 4189 },
	{ 0x6ec2, 0x6ec2, PDF_CMAP_SINGLE, 4955 },
	{ 0x6ec4, 0x6ec5, PDF_CMAP_TABLE, 1344 },
	{ 0x6ec9, 0x6ec9, PDF_CMAP_SINGLE, 7916 },
	{ 0x6ecb, 0x6ecc, PDF_CMAP_TABLE, 1346 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 7777 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 7900 },
	{ 0x6ed3, 0x6ed4, PDF_CMAP_TABLE, 1348 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 7208 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 6625 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 7816 },
	{ 0x6efe, 0x6eff, PDF_CMAP_TABLE, 1350 },
	{ 0x6f01, 0x6f02, PDF_CMAP_TABLE, 1352 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 7366 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 4569 },
	{ 0x6f11, 0x6f11, PDF_CMAP_SINGLE, 3570 },
	{ 0x6f14, 0x6f15, PDF_CMAP_TABLE, 1354 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 4666 },
	{ 0x6f22, 0x6f23, PDF_CMAP_TABLE, 1356 },
	{ 0x6f2b, 0x6f2c, PDF_CMAP_TABLE, 1358 },
	{ 0x6f31, 0x6f32, PDF_CMAP_TABLE, 1360 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 6695 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 6551 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 6012 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 4934 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 3650 },
	{ 0x6f57, 0x6f58, PDF_CMAP_TABLE, 1362 },
	{ 0x6f5a, 0x6f5b, PDF_CMAP_TABLE, 1364 },
	{ 0x6f5e, 0x6f5f, PDF_CMAP_TABLE, 1366 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 7917 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 6362 },
	{ 0x6f6d, 0x6f6e, PDF_CMAP_TABLE, 1368 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 3986 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 6525 },
	{ 0x6f7c, 0x6f7e, PDF_CMAP_TABLE, 1370 },
	{ 0x6f81, 0x6f81, PDF_CMAP_SINGLE, 5328 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 7040 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 7172 },
	{ 0x6f8d, 0x6f8e, PDF_CMAP_TABLE, 1373 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 6241 },
	{ 0x6f94, 0x6f94, PDF_CMAP_SINGLE, 7817 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 3488 },
	{ 0x6fa3, 0x6fa4, PDF_CMAP_TABLE, 1375 },
	{ 0x6fa7, 0x6fa7, PDF_CMAP_SINGLE, 4507 },
	{ 0x6fae, 0x6faf, PDF_CMAP_TABLE, 1377 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 6661 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 6069 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 4225 },
	{ 0x6fbe, 0x6fbe, PDF_CMAP_SINGLE, 4213 },
	{ 0x6fc0, 0x6fc3, PDF_CMAP_TABLE, 1379 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 6035 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 5677 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 6013 },
	{ 0x6fdf, 0x6fe1, PDF_CMAP_TABLE, 1383 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 4291 },
	{ 0x6fe9, 0x6fe9, PDF_CMAP_SINGLE, 7819 },
	{ 0x6feb, 0x6fec, PDF_CMAP_TABLE, 1386 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 7403 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 5230 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 4449 },
	{ 0x7001, 0x7001, PDF_CMAP_SINGLE, 5890 },
	{ 0x7005, 0x7006, PDF_CMAP_TABLE, 1388 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 5275 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 5767 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 4583 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 7574 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 5231 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 4516 },
	{ 0x701a, 0x701f, PDF_CMAP_TABLE, 1390 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 7695 },
	{ 0x7027, 0x7028, PDF_CMAP_TABLE, 1396 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 6015 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 6437 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 4393 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 3825 },
	{ 0x7050, 0x7051, PDF_CMAP_TABLE, 1398 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 7414 },
	{ 0x705d, 0x705d, PDF_CMAP_SINGLE, 7820 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 4681 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 7867 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 7940 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 3921 },
	{ 0x707c, 0x707d, PDF_CMAP_TABLE, 1400 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 3681 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 7321 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 5991 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 7224 },
	{ 0x7098, 0x709a, PDF_CMAP_TABLE, 1402 },
	{ 0x70a1, 0x70a1, PDF_CMAP_SINGLE, 6733 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 5516 },
	{ 0x70ab, 0x70ad, PDF_CMAP_TABLE, 1405 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 7780 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 5050 },
	{ 0x70b7, 0x70b9, PDF_CMAP_TABLE, 1408 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 4481 },
	{ 0x70cb, 0x70cb, PDF_CMAP_SINGLE, 8007 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 6070 },
	{ 0x70d8, 0x70d9, PDF_CMAP_TABLE, 1411 },
	{ 0x70dd, 0x70dd, PDF_CMAP_SINGLE, 6935 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 5971 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 3682 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 7515 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 5100 },
	{ 0x7104, 0x7104, PDF_CMAP_SINGLE, 7983 },
	{ 0x7109, 0x7109, PDF_CMAP_SINGLE, 5920 },
	{ 0x710c, 0x710c, PDF_CMAP_SINGLE, 6913 },
	{ 0x7119, 0x711a, PDF_CMAP_TABLE, 1413 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 4325 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 4823 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 7225 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 5992 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 5972 },
	{ 0x7147, 0x7147, PDF_CMAP_SINGLE, 8001 },
	{ 0x7149, 0x714a, PDF_CMAP_TABLE, 1415 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 7918 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 6662 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 6016 },
	{ 0x7156, 0x7156, PDF_CMAP_SINGLE, 4141 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 5973 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 6234 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 5318 },
	{ 0x7164, 0x7167, PDF_CMAP_TABLE, 1417 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 4998 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 5891 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 6495 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 5428 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 5728 },
	{ 0x7189, 0x718a, PDF_CMAP_TABLE, 1421 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 7984 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 7781 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 6187 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 8048 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 5635 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 5101 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 6071 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 5987 },
	{ 0x71b9, 0x71ba, PDF_CMAP_RANGE, 8049 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 7345 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 6000 },
	{ 0x71c3, 0x71c3, PDF_CMAP_SINGLE, 5974 },
	{ 0x71c8, 0x71c9, PDF_CMAP_TABLE, 1423 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 4556 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 4640 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 5517 },
	{ 0x71d4, 0x71d5, PDF_CMAP_TABLE, 1425 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 6017 },
	{ 0x71e5, 0x71e7, PDF_CMAP_TABLE, 1427 },
	{ 0x71ed, 0x71ee, PDF_CMAP_TABLE, 1430 },
	{ 0x71fb, 0x71fc, PDF_CMAP_TABLE, 1432 },
	{ 0x71fe, 0x7200, PDF_CMAP_TABLE, 1434 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 7575 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 4517 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 4394 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 6803 },
	{ 0x722c, 0x722d, PDF_CMAP_TABLE, 1437 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 6271 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 6295 },
	{ 0x7235, 0x7236, PDF_CMAP_TABLE, 1439 },
	{ 0x723a, 0x723b, PDF_CMAP_TABLE, 1441 },
	{ 0x723d, 0x723e, PDF_CMAP_TABLE, 1443 },
	{ 0x7240, 0x7240, PDF_CMAP_SINGLE, 5350 },
	{ 0x7246, 0x7248, PDF_CMAP_TABLE, 1445 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 7508 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 7191 },
	{ 0x7258, 0x7259, PDF_CMAP_TABLE, 1448 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 6210 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 5232 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 4777 },
	{ 0x7261, 0x7262, PDF_CMAP_TABLE, 1450 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 4790 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 4850 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 5369 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 7472 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 3638 },
	{ 0x7280, 0x7281, PDF_CMAP_TABLE, 1452 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 4312 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 8051 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 3639 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 5015 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 5351 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 3847 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 6626 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5831 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 7821 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 3922 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 6603 },
	{ 0x72e1, 0x72e1, PDF_CMAP_SINGLE, 3879 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 5591 },
	{ 0x72f8, 0x72f9, PDF_CMAP_TABLE, 1454 },
	{ 0x72fc, 0x72fd, PDF_CMAP_TABLE, 1456 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 6036 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 7107 },
	{ 0x731b, 0x731d, PDF_CMAP_TABLE, 1458 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 6138 },
	{ 0x7329, 0x732b, PDF_CMAP_TABLE, 1461 },
	{ 0x7336, 0x7337, PDF_CMAP_RANGE, 6335 },
	{ 0x733e, 0x733f, PDF_CMAP_TABLE, 1464 },
	{ 0x7344, 0x7345, PDF_CMAP_TABLE, 1466 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 6553 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 6072 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 3980 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 4313 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 7941 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 6018 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 7949 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 4488 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 5592 },
	{ 0x737a, 0x737b, PDF_CMAP_TABLE, 1468 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 7741 },
	{ 0x7386, 0x7387, PDF_CMAP_TABLE, 1470 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 6081 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 6130 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6734 },
	{ 0x7394, 0x7394, PDF_CMAP_SINGLE, 7156 },
	{ 0x7396, 0x7398, PDF_CMAP_TABLE, 1472 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 4879 },
	{ 0x73a7, 0x73a7, PDF_CMAP_SINGLE, 6363 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 6113 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 5233 },
	{ 0x73b2, 0x73b3, PDF_CMAP_TABLE, 1475 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 7742 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 4894 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 3452 },
	{ 0x73c9, 0x73ca, PDF_CMAP_TABLE, 1477 },
	{ 0x73cc, 0x73cd, PDF_CMAP_TABLE, 1479 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 3471 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 3848 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 3797 },
	{ 0x73dd, 0x73de, PDF_CMAP_TABLE, 1481 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 6882 },
	{ 0x73e3, 0x73e6, PDF_CMAP_TABLE, 1483 },
	{ 0x73e9, 0x73ea, PDF_CMAP_TABLE, 1487 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 4916 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 4824 },
	{ 0x73f9, 0x73f9, PDF_CMAP_SINGLE, 5483 },
	{ 0x73fd, 0x73fe, PDF_CMAP_TABLE, 1489 },
	{ 0x7401, 0x7401, PDF_CMAP_SINGLE, 5429 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 3924 },
	{ 0x7405, 0x7407, PDF_CMAP_TABLE, 1491 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 4584 },
	{ 0x7413, 0x7413, PDF_CMAP_SINGLE, 6114 },
	{ 0x741b, 0x741b, PDF_CMAP_SINGLE, 7372 },
	{ 0x7420, 0x7422, PDF_CMAP_TABLE, 1494 },
	{ 0x7425, 0x7426, PDF_CMAP_TABLE, 1497 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 3781 },
	{ 0x742a, 0x742c, PDF_CMAP_TABLE, 1499 },
	{ 0x742e, 0x7430, PDF_CMAP_TABLE, 1502 },
	{ 0x7433, 0x7436, PDF_CMAP_TABLE, 1505 },
	{ 0x7438, 0x7438, PDF_CMAP_SINGLE, 7405 },
	{ 0x743a, 0x743a, PDF_CMAP_SINGLE, 5019 },
	{ 0x743f, 0x7441, PDF_CMAP_TABLE, 1509 },
	{ 0x7443, 0x7444, PDF_CMAP_TABLE, 1512 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 6296 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 7623 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 6273 },
	{ 0x7459, 0x745c, PDF_CMAP_TABLE, 1514 },
	{ 0x745e, 0x7460, PDF_CMAP_TABLE, 1518 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 6188 },
	{ 0x7464, 0x7465, PDF_CMAP_TABLE, 1521 },
	{ 0x7468, 0x746a, PDF_CMAP_TABLE, 1523 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 4417 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 4030 },
	{ 0x7482, 0x7483, PDF_CMAP_TABLE, 1526 },
	{ 0x7487, 0x7487, PDF_CMAP_SINGLE, 5431 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 4470 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 6554 },
	{ 0x7498, 0x7498, PDF_CMAP_SINGLE, 4641 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 7919 },
	{ 0x749e, 0x749f, PDF_CMAP_TABLE, 1528 },
	{ 0x74a1, 0x74a1, PDF_CMAP_SINGLE, 6994 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 4098 },
	{ 0x74a5, 0x74a5, PDF_CMAP_SINGLE, 3684 },
	{ 0x74a7, 0x74a8, PDF_CMAP_TABLE, 1530 },
	{ 0x74aa, 0x74aa, PDF_CMAP_SINGLE, 6804 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 7894 },
	{ 0x74b2, 0x74b2, PDF_CMAP_SINGLE, 5594 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 5939 },
	{ 0x74b9, 0x74b9, PDF_CMAP_SINGLE, 5637 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 5363 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 5432 },
	{ 0x74c6, 0x74c6, PDF_CMAP_SINGLE, 7021 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 3685 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 4540 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 6020 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 3827 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 7066 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 3806 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 7824 },
	{ 0x74e2, 0x74e3, PDF_CMAP_TABLE, 1532 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 6100 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 6092 },
	{ 0x74f7, 0x74f7, PDF_CMAP_SINGLE, 6497 },
	{ 0x7501, 0x7501, PDF_CMAP_SINGLE, 5051 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 3640 },
	{ 0x7511, 0x7511, PDF_CMAP_SINGLE, 6936 },
	{ 0x7515, 0x7515, PDF_CMAP_SINGLE, 6093 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 3521 },
	{ 0x751a, 0x751b, PDF_CMAP_TABLE, 1534 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 5370 },
	{ 0x7523, 0x7523, PDF_CMAP_SINGLE, 5309 },
	{ 0x7525, 0x7526, PDF_CMAP_TABLE, 1536 },
	{ 0x7528, 0x7528, PDF_CMAP_SINGLE, 6189 },
	{ 0x752b, 0x752c, PDF_CMAP_TABLE, 1538 },
	{ 0x7530, 0x7533, PDF_CMAP_TABLE, 1540 },
	{ 0x7537, 0x7538, PDF_CMAP_TABLE, 1544 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 6736 },
	{ 0x7547, 0x7547, PDF_CMAP_SINGLE, 4011 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 3717 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 6139 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 6666 },
	{ 0x7553, 0x7554, PDF_CMAP_TABLE, 1546 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 4586 },
	{ 0x755b, 0x755d, PDF_CMAP_TABLE, 1548 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 7609 },
	{ 0x7565, 0x7566, PDF_CMAP_TABLE, 1551 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 5000 },
	{ 0x756f, 0x7570, PDF_CMAP_TABLE, 1553 },
	{ 0x7575, 0x7576, PDF_CMAP_TABLE, 1555 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 4099 },
	{ 0x757a, 0x757a, PDF_CMAP_SINGLE, 3546 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 4100 },
	{ 0x7586, 0x7587, PDF_CMAP_TABLE, 1557 },
	{ 0x758a, 0x758b, PDF_CMAP_TABLE, 1559 },
	{ 0x758e, 0x758f, PDF_CMAP_TABLE, 1561 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 6403 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 5310 },
	{ 0x75a5, 0x75a5, PDF_CMAP_SINGLE, 3571 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 5951 },
	{ 0x75b1, 0x75b3, PDF_CMAP_TABLE, 1563 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 6498 },
	{ 0x75b8, 0x75b9, PDF_CMAP_TABLE, 1566 },
	{ 0x75bc, 0x75be, PDF_CMAP_TABLE, 1568 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 3453 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 5052 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 6937 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 6425 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 5892 },
	{ 0x75d4, 0x75d5, PDF_CMAP_TABLE, 1571 },
	{ 0x75d8, 0x75d9, PDF_CMAP_TABLE, 1573 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 7455 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 4627 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 4226 },
	{ 0x75f2, 0x75f2, PDF_CMAP_SINGLE, 4658 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 7347 },
	{ 0x75fa, 0x75fa, PDF_CMAP_SINGLE, 5202 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 3746 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 5906 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 5893 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 5521 },
	{ 0x761f, 0x7622, PDF_CMAP_TABLE, 1575 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 4587 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 5595 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 4570 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 4557 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 5826 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 3489 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 6339 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 5026 },
	{ 0x7661, 0x7661, PDF_CMAP_SINGLE, 7348 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 6688 },
	{ 0x7669, 0x7669, PDF_CMAP_SINGLE, 4376 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 5433 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 6094 },
	{ 0x7672, 0x7672, PDF_CMAP_SINGLE, 6667 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 3718 },
	{ 0x767b, 0x767c, PDF_CMAP_TABLE, 1579 },
	{ 0x767d, 0x767e, PDF_CMAP_RANGE, 4993 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 6627 },
	{ 0x7686, 0x7687, PDF_CMAP_TABLE, 1581 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 3880 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 3747 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 7825 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 7600 },
	{ 0x76ba, 0x76ba, PDF_CMAP_SINGLE, 7272 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 4755 },
	{ 0x76c2, 0x76c3, PDF_CMAP_TABLE, 1583 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 5164 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 6021 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 6438 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 7665 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 3573 },
	{ 0x76db, 0x76dc, PDF_CMAP_TABLE, 1585 },
	{ 0x76de, 0x76df, PDF_CMAP_TABLE, 1587 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 6997 },
	{ 0x76e3, 0x76e4, PDF_CMAP_TABLE, 1589 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 4518 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 4791 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 4730 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 6975 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 5352 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 4920 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 5652 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 5485 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 4740 },
	{ 0x7708, 0x7709, PDF_CMAP_TABLE, 1591 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 3490 },
	{ 0x771e, 0x771e, PDF_CMAP_SINGLE, 6998 },
	{ 0x7720, 0x7720, PDF_CMAP_SINGLE, 4741 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 7744 },
	{ 0x7737, 0x7738, PDF_CMAP_TABLE, 1593 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 6805 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 5812 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 7057 },
	{ 0x774d, 0x774d, PDF_CMAP_SINGLE, 7745 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 6737 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 5596 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 4314 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 4792 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 7193 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 4294 },
	{ 0x777e, 0x777f, PDF_CMAP_TABLE, 1595 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 6999 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 4756 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 4682 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 5038 },
	{ 0x77ac, 0x77ad, PDF_CMAP_TABLE, 1597 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 3524 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 4343 },
	{ 0x77bb, 0x77bc, PDF_CMAP_TABLE, 1599 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 3925 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 7244 },
	{ 0x77db, 0x77dc, PDF_CMAP_TABLE, 1601 },
	{ 0x77e2, 0x77e3, PDF_CMAP_TABLE, 1603 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 6960 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 3926 },
	{ 0x77ed, 0x77ef, PDF_CMAP_TABLE, 1605 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 5413 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 5277 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 5203 },
	{ 0x7825, 0x7827, PDF_CMAP_TABLE, 1608 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 4652 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 7559 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 7485 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 4000 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 5976 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 7226 },
	{ 0x786b, 0x786c, PDF_CMAP_TABLE, 1611 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 5977 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 5178 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 4101 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 6738 },
	{ 0x788c, 0x788e, PDF_CMAP_TABLE, 1613 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 5204 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 6116 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 3504 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 5027 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 5414 },
	{ 0x78ba, 0x78bb, PDF_CMAP_RANGE, 7879 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 4659 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 6499 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 4956 },
	{ 0x78ca, 0x78cb, PDF_CMAP_TABLE, 1616 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 3719 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 4921 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 4660 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 3688 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 4102 },
	{ 0x78f5, 0x78f5, PDF_CMAP_SINGLE, 3491 },
	{ 0x78fb, 0x78fb, PDF_CMAP_SINGLE, 4922 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 7227 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 7228 },
	{ 0x7916, 0x7916, PDF_CMAP_SINGLE, 5940 },
	{ 0x792a, 0x792c, PDF_CMAP_TABLE, 1618 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 5710 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 5278 },
	{ 0x7940, 0x7941, PDF_CMAP_TABLE, 1621 },
	{ 0x7947, 0x7948, PDF_CMAP_RANGE, 4104 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 6962 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 6214 },
	{ 0x7956, 0x7957, PDF_CMAP_TABLE, 1623 },
	{ 0x795a, 0x795e, PDF_CMAP_TABLE, 1625 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 5280 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 5353 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 7584 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 6769 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 4106 },
	{ 0x797f, 0x797f, PDF_CMAP_SINGLE, 4529 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 4044 },
	{ 0x798d, 0x798f, PDF_CMAP_TABLE, 1630 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 6215 },
	{ 0x79a6, 0x79a7, PDF_CMAP_TABLE, 1633 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5434 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 4508 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 4295 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5894 },
	{ 0x79b9, 0x79b9, PDF_CMAP_SINGLE, 6216 },
	{ 0x79bd, 0x79c1, PDF_CMAP_TABLE, 1635 },
	{ 0x79c9, 0x79cb, PDF_CMAP_TABLE, 1640 },
	{ 0x79d1, 0x79d2, PDF_CMAP_TABLE, 1643 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 5205 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 5206 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 6808 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 7377 },
	{ 0x79e6, 0x79e7, PDF_CMAP_TABLE, 1645 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 7023 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 6426 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 8053 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 5496 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 3492 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 6740 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 7230 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 6474 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 7510 },
	{ 0x7a19, 0x7a1a, PDF_CMAP_TABLE, 1647 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 4609 },
	{ 0x7a1f, 0x7a20, PDF_CMAP_TABLE, 1649 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 6845 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 7378 },
	{ 0x7a36, 0x7a37, PDF_CMAP_TABLE, 1651 },
	{ 0x7a3b, 0x7a3d, PDF_CMAP_TABLE, 1653 },
	{ 0x7a3f, 0x7a40, PDF_CMAP_TABLE, 1656 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 4793 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 7350 },
	{ 0x7a4d, 0x7a4e, PDF_CMAP_TABLE, 1658 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5598 },
	{ 0x7a61, 0x7a62, PDF_CMAP_TABLE, 1660 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 6086 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 7881 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 5895 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 7755 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 3927 },
	{ 0x7a79, 0x7a7a, PDF_CMAP_TABLE, 1662 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 6741 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 7157 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 4330 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 7058 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 6156 },
	{ 0x7a92, 0x7a93, PDF_CMAP_TABLE, 1664 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 6810 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 3954 },
	{ 0x7a9f, 0x7a9f, PDF_CMAP_SINGLE, 3962 },
	{ 0x7aa9, 0x7aaa, PDF_CMAP_RANGE, 6101 },
	{ 0x7aae, 0x7aaf, PDF_CMAP_TABLE, 1666 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 4001 },
	{ 0x7ac4, 0x7ac5, PDF_CMAP_TABLE, 1668 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 4353 },
	{ 0x7aca, 0x7acb, PDF_CMAP_TABLE, 1670 },
	{ 0x7ad7, 0x7ad7, PDF_CMAP_SINGLE, 4809 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 7089 },
	{ 0x7add, 0x7add, PDF_CMAP_SINGLE, 5054 },
	{ 0x7adf, 0x7ae0, PDF_CMAP_TABLE, 1672 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 6915 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 4344 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 5599 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 3505 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 4205 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 3690 },
	{ 0x7af9, 0x7afa, PDF_CMAP_TABLE, 1674 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 3493 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 7852 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 5522 },
	{ 0x7b19, 0x7b19, PDF_CMAP_SINGLE, 5372 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 6629 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 7439 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 4654 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 5130 },
	{ 0x7b2c, 0x7b2d, PDF_CMAP_TABLE, 1676 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 5497 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 7611 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 4369 },
	{ 0x7b4b, 0x7b4d, PDF_CMAP_TABLE, 1678 },
	{ 0x7b4f, 0x7b52, PDF_CMAP_TABLE, 1681 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 4236 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 7128 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 4012 },
	{ 0x7b6c, 0x7b6c, PDF_CMAP_SINGLE, 5486 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 5390 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 5978 },
	{ 0x7b7d, 0x7b7d, PDF_CMAP_SINGLE, 6073 },
	{ 0x7b87, 0x7b87, PDF_CMAP_SINGLE, 3574 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 6669 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 6588 },
	{ 0x7b94, 0x7b95, PDF_CMAP_TABLE, 1685 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 5311 },
	{ 0x7b9a, 0x7b9a, PDF_CMAP_SINGLE, 7051 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 3656 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 3828 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 6670 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 5354 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 6530 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 6606 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 1687 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 5016 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_TABLE, 1689 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 7293 },
	{ 0x7bd2, 0x7bd2, PDF_CMAP_SINGLE, 5729 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 5523 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 4316 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 5282 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 6829 },
	{ 0x7c12, 0x7c12, PDF_CMAP_SINGLE, 7068 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 4206 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 3494 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 7922 },
	{ 0x7c2a, 0x7c2b, PDF_CMAP_TABLE, 1691 },
	{ 0x7c3d, 0x7c3f, PDF_CMAP_TABLE, 1693 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 4404 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 1696 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 4541 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 7184 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 4628 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 4863 },
	{ 0x7c83, 0x7c83, PDF_CMAP_SINGLE, 5207 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 5165 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 4655 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 4897 },
	{ 0x7c97, 0x7c98, PDF_CMAP_TABLE, 1698 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 5541 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 6901 },
	{ 0x7ca7, 0x7ca7, PDF_CMAP_SINGLE, 6556 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 4433 },
	{ 0x7cb1, 0x7cb3, PDF_CMAP_TABLE, 1700 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 5600 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 6742 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 7827 },
	{ 0x7cd6, 0x7cd6, PDF_CMAP_SINGLE, 4247 },
	{ 0x7cde, 0x7ce0, PDF_CMAP_TABLE, 1703 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 4435 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 3721 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 4003 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 4108 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 6885 },
	{ 0x7d04, 0x7d08, PDF_CMAP_TABLE, 1706 },
	{ 0x7d0a, 0x7d0b, PDF_CMAP_RANGE, 4842 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 4150 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 4184 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 5655 },
	{ 0x7d17, 0x7d1b, PDF_CMAP_TABLE, 1711 },
	{ 0x7d20, 0x7d22, PDF_CMAP_TABLE, 1716 },
	{ 0x7d2b, 0x7d2c, PDF_CMAP_TABLE, 1719 },
	{ 0x7d2e, 0x7d30, PDF_CMAP_TABLE, 1721 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 5747 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 6607 },
	{ 0x7d39, 0x7d3a, PDF_CMAP_TABLE, 1724 },
	{ 0x7d42, 0x7d46, PDF_CMAP_TABLE, 1726 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 3651 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 3882 },
	{ 0x7d61, 0x7d62, PDF_CMAP_TABLE, 1731 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 4057 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 6371 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 6454 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 5391 },
	{ 0x7d71, 0x7d73, PDF_CMAP_TABLE, 1733 },
	{ 0x7d76, 0x7d76, PDF_CMAP_SINGLE, 6691 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 3641 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 3928 },
	{ 0x7d8e, 0x7d8f, PDF_CMAP_TABLE, 1736 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 3692 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 6847 },
	{ 0x7da0, 0x7da0, PDF_CMAP_SINGLE, 4530 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 6887 },
	{ 0x7dac, 0x7dad, PDF_CMAP_TABLE, 1738 },
	{ 0x7db1, 0x7db2, PDF_CMAP_TABLE, 1740 },
	{ 0x7db4, 0x7db5, PDF_CMAP_TABLE, 1742 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 4598 },
	{ 0x7dba, 0x7dbb, PDF_CMAP_TABLE, 1744 },
	{ 0x7dbd, 0x7dbf, PDF_CMAP_TABLE, 1746 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 7351 },
	{ 0x7dca, 0x7dcb, PDF_CMAP_TABLE, 1749 },
	{ 0x7dd6, 0x7dd6, PDF_CMAP_SINGLE, 5392 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 7658 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 5435 },
	{ 0x7ddd, 0x7dde, PDF_CMAP_TABLE, 1751 },
	{ 0x7de0, 0x7de1, PDF_CMAP_TABLE, 1753 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 5979 },
	{ 0x7de8, 0x7de9, PDF_CMAP_TABLE, 1755 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 4743 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 6297 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4471 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 7352 },
	{ 0x7e09, 0x7e0a, PDF_CMAP_TABLE, 1757 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 6087 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 4898 },
	{ 0x7e1d, 0x7e1f, PDF_CMAP_TABLE, 1759 },
	{ 0x7e21, 0x7e21, PDF_CMAP_SINGLE, 6581 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 7748 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 5103 },
	{ 0x7e2e, 0x7e2f, PDF_CMAP_TABLE, 1762 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 6848 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 4572 },
	{ 0x7e3d, 0x7e3e, PDF_CMAP_TABLE, 1764 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 5001 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 5179 },
	{ 0x7e46, 0x7e47, PDF_CMAP_TABLE, 1766 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 6938 },
	{ 0x7e54, 0x7e55, PDF_CMAP_TABLE, 1768 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 6159 },
	{ 0x7e61, 0x7e61, PDF_CMAP_SINGLE, 5603 },
	{ 0x7e69, 0x7e6b, PDF_CMAP_TABLE, 1770 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 3642 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 6814 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 5952 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 3723 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 7069 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 5542 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 6672 },
	{ 0x7e93, 0x7e93, PDF_CMAP_SINGLE, 6023 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 5466 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 7071 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_TABLE, 1773 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 5132 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 7681 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 3652 },
	{ 0x7f4c, 0x7f4c, PDF_CMAP_SINGLE, 5859 },
	{ 0x7f50, 0x7f50, PDF_CMAP_SINGLE, 3829 },
	{ 0x7f54, 0x7f55, PDF_CMAP_TABLE, 1775 },
	{ 0x7f6a, 0x7f6b, PDF_CMAP_TABLE, 1777 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 7353 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 5007 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 5393 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 4717 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 7486 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 4629 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 4377 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 4110 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 5896 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 3551 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 4864 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 3750 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 4497 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 5604 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 3955 },
	{ 0x7fa8, 0x7fa9, PDF_CMAP_TABLE, 1779 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 8054 },
	{ 0x7fb8, 0x7fb9, PDF_CMAP_TABLE, 1781 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 6218 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 6095 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 5711 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 6439 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 6440 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 4498 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 5679 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_TABLE, 1783 },
	{ 0x7fdf, 0x7fe1, PDF_CMAP_TABLE, 1785 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 7524 },
	{ 0x7feb, 0x7feb, PDF_CMAP_SINGLE, 6118 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 7644 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 3883 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 6441 },
	{ 0x8000, 0x8001, PDF_CMAP_TABLE, 1788 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 3751 },
	{ 0x8005, 0x8006, PDF_CMAP_TABLE, 1790 },
	{ 0x8009, 0x8009, PDF_CMAP_SINGLE, 3929 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 6427 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 4158 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 3693 },
	{ 0x8017, 0x8018, PDF_CMAP_TABLE, 1792 },
	{ 0x802d, 0x802d, PDF_CMAP_SINGLE, 4112 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 6428 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 5870 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 7422 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 3694 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 4227 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 4499 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 4559 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 5487 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 5239 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 7323 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 4844 },
	{ 0x806f, 0x8070, PDF_CMAP_TABLE, 1794 },
	{ 0x8072, 0x8073, PDF_CMAP_TABLE, 1796 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 6979 },
	{ 0x807d, 0x807f, PDF_CMAP_TABLE, 1798 },
	{ 0x8084, 0x8087, PDF_CMAP_TABLE, 1801 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 6357 },
	{ 0x808b, 0x808c, PDF_CMAP_TABLE, 1805 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 7231 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 7682 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 3495 },
	{ 0x80a1, 0x80a2, PDF_CMAP_TABLE, 1807 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 5210 },
	{ 0x80a9, 0x80aa, PDF_CMAP_TABLE, 1809 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 4061 },
	{ 0x80b1, 0x80b2, PDF_CMAP_TABLE, 1811 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 7963 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 7540 },
	{ 0x80c3, 0x80c4, PDF_CMAP_TABLE, 1813 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 4979 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 7440 },
	{ 0x80da, 0x80db, PDF_CMAP_TABLE, 1815 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 7560 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 7829 },
	{ 0x80e4, 0x80e5, PDF_CMAP_TABLE, 1817 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 3850 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 4345 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 8017 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 4185 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 6966 },
	{ 0x8105, 0x8108, PDF_CMAP_TABLE, 1819 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 7144 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 6119 },
	{ 0x811a, 0x811b, PDF_CMAP_TABLE, 1823 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 5656 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 5605 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 7419 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 7561 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 7110 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 5211 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 5856 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 5748 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 5133 },
	{ 0x8154, 0x8155, PDF_CMAP_TABLE, 1825 },
	{ 0x8165, 0x8166, PDF_CMAP_TABLE, 1827 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 6849 },
	{ 0x8170, 0x8171, PDF_CMAP_TABLE, 1829 },
	{ 0x8178, 0x817a, PDF_CMAP_TABLE, 1831 },
	{ 0x817f, 0x8180, PDF_CMAP_TABLE, 1834 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 3634 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 4899 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 3753 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 5135 },
	{ 0x819c, 0x819d, PDF_CMAP_TABLE, 1836 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 3884 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 7025 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 7516 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 5439 },
	{ 0x81b5, 0x81b5, PDF_CMAP_SINGLE, 7313 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 6392 },
	{ 0x81bd, 0x81c0, PDF_CMAP_TABLE, 1838 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 5212 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 5916 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 6771 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 4410 },
	{ 0x81df, 0x81df, PDF_CMAP_SINGLE, 6558 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 5749 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 6103 },
	{ 0x81e7, 0x81e8, PDF_CMAP_TABLE, 1842 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 6502 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 7325 },
	{ 0x81f3, 0x81f4, PDF_CMAP_TABLE, 1844 },
	{ 0x81fa, 0x81fc, PDF_CMAP_TABLE, 1846 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 6341 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 3931 },
	{ 0x8207, 0x8208, PDF_CMAP_TABLE, 1849 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 3932 },
	{ 0x820c, 0x820d, PDF_CMAP_TABLE, 1851 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 5395 },
	{ 0x821b, 0x821c, PDF_CMAP_TABLE, 1853 },
	{ 0x821e, 0x821f, PDF_CMAP_TABLE, 1855 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 3553 },
	{ 0x822a, 0x822c, PDF_CMAP_TABLE, 1857 },
	{ 0x8235, 0x8237, PDF_CMAP_TABLE, 1860 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 5440 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 5136 },
	{ 0x8245, 0x8245, PDF_CMAP_SINGLE, 5942 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 6744 },
	{ 0x8259, 0x8259, PDF_CMAP_SINGLE, 7111 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 6406 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 7659 },
	{ 0x826e, 0x826f, PDF_CMAP_TABLE, 1863 },
	{ 0x8271, 0x8272, PDF_CMAP_TABLE, 1865 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 5994 },
	{ 0x8278, 0x8278, PDF_CMAP_SINGLE, 7232 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 5848 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 6219 },
	{ 0x828d, 0x828e, PDF_CMAP_TABLE, 1867 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 4703 },
	{ 0x8299, 0x829a, PDF_CMAP_TABLE, 1869 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 6968 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 5324 },
	{ 0x82a5, 0x82a6, PDF_CMAP_TABLE, 1871 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 4046 },
	{ 0x82ac, 0x82af, PDF_CMAP_TABLE, 1873 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 7871 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 4961 },
	{ 0x82b7, 0x82b9, PDF_CMAP_TABLE, 1877 },
	{ 0x82bb, 0x82bd, PDF_CMAP_TABLE, 1880 },
	{ 0x82bf, 0x82bf, PDF_CMAP_SINGLE, 6482 },
	{ 0x82d1, 0x82d2, PDF_CMAP_TABLE, 1883 },
	{ 0x82d4, 0x82d5, PDF_CMAP_TABLE, 1885 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 4810 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 3455 },
	{ 0x82de, 0x82df, PDF_CMAP_TABLE, 1887 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 6430 },
	{ 0x82e5, 0x82e7, PDF_CMAP_TABLE, 1889 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 6024 },
	{ 0x82fd, 0x82fe, PDF_CMAP_TABLE, 1892 },
	{ 0x8301, 0x8305, PDF_CMAP_TABLE, 1894 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 4693 },
	{ 0x8317, 0x8317, PDF_CMAP_SINGLE, 4757 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 6503 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 4704 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 5083 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 5606 },
	{ 0x8334, 0x8336, PDF_CMAP_TABLE, 1899 },
	{ 0x8338, 0x8339, PDF_CMAP_TABLE, 1902 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 5658 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 7708 },
	{ 0x8349, 0x834a, PDF_CMAP_TABLE, 1904 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 6475 },
	{ 0x8351, 0x8352, PDF_CMAP_TABLE, 1906 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 4354 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 7624 },
	{ 0x837b, 0x837b, PDF_CMAP_SINGLE, 6633 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 1908 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 5287 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 3696 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 5750 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 6121 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 7768 },
	{ 0x83a9, 0x83ab, PDF_CMAP_TABLE, 1910 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 4705 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 7201 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 3830 },
	{ 0x83c9, 0x83ca, PDF_CMAP_TABLE, 1913 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 4013 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 3808 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 7112 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 7122 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 5069 },
	{ 0x83eb, 0x83eb, PDF_CMAP_SINGLE, 4033 },
	{ 0x83ef, 0x83f2, PDF_CMAP_TABLE, 1915 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 5827 },
	{ 0x83f9, 0x83f9, PDF_CMAP_SINGLE, 6609 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 5639 },
	{ 0x8403, 0x8404, PDF_CMAP_TABLE, 1919 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 4423 },
	{ 0x840c, 0x840e, PDF_CMAP_TABLE, 1921 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 7275 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 4683 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 7993 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 6342 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 4386 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 6001 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 6610 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 3506 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 7563 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 4346 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 6300 },
	{ 0x846b, 0x846c, PDF_CMAP_TABLE, 1924 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 5875 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 4004 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 6930 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 5607 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 5712 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 4799 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 5312 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 4962 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 7564 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 6939 },
	{ 0x84bb, 0x84bc, PDF_CMAP_TABLE, 1926 },
	{ 0x84bf, 0x84c0, PDF_CMAP_TABLE, 1928 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 4758 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 7295 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 5415 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 6193 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 3576 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 5713 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 5288 },
	{ 0x84da, 0x84da, PDF_CMAP_SINGLE, 5608 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 5104 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4473 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 5659 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 4560 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 4747 },
	{ 0x8513, 0x8514, PDF_CMAP_TABLE, 1930 },
	{ 0x8517, 0x8518, PDF_CMAP_TABLE, 1932 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 6250 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 4573 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 7123 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 6562 },
	{ 0x8525, 0x8525, PDF_CMAP_SINGLE, 7259 },
	{ 0x852c, 0x852d, PDF_CMAP_TABLE, 1934 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 7004 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 7541 },
	{ 0x853f, 0x853f, PDF_CMAP_SINGLE, 6301 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 4229 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 5002 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 7235 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 3885 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 6245 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 7796 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 5660 },
	{ 0x8568, 0x856a, PDF_CMAP_TABLE, 1936 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 5528 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 4901 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 4865 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 6407 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 3554 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 6563 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 5457 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 7159 },
	{ 0x85a8, 0x85aa, PDF_CMAP_TABLE, 1939 },
	{ 0x85af, 0x85b0, PDF_CMAP_TABLE, 1942 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 6772 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 3757 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 6505 },
	{ 0x85cd, 0x85cf, PDF_CMAP_TABLE, 1944 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 6220 },
	{ 0x85dc, 0x85dd, PDF_CMAP_TABLE, 1947 },
	{ 0x85e4, 0x85e5, PDF_CMAP_TABLE, 1949 },
	{ 0x85e9, 0x85ea, PDF_CMAP_TABLE, 1951 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 6611 },
	{ 0x85fa, 0x85fb, PDF_CMAP_TABLE, 1953 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 3817 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 6041 },
	{ 0x8606, 0x8607, PDF_CMAP_TABLE, 1955 },
	{ 0x860a, 0x860a, PDF_CMAP_SINGLE, 6088 },
	{ 0x8616, 0x8617, PDF_CMAP_TABLE, 1957 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 5441 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 4395 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 4378 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 7833 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 7632 },
	{ 0x8654, 0x8655, PDF_CMAP_TABLE, 1959 },
	{ 0x865b, 0x865c, PDF_CMAP_TABLE, 1961 },
	{ 0x865e, 0x865f, PDF_CMAP_TABLE, 1963 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 8009 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 7860 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 4845 },
	{ 0x868c, 0x868c, PDF_CMAP_SINGLE, 4963 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 6456 },
	{ 0x86a3, 0x86a4, PDF_CMAP_TABLE, 1965 },
	{ 0x86a9, 0x86a9, PDF_CMAP_SINGLE, 7355 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 5289 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 4208 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 7945 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 6104 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 6889 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 3886 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 7666 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 7026 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 5785 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 7245 },
	{ 0x8702, 0x8703, PDF_CMAP_TABLE, 1967 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 6074 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 6970 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 5214 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 4884 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 3508 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 5730 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 7924 },
	{ 0x875f, 0x875f, PDF_CMAP_SINGLE, 6302 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 7625 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 5676 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 7835 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 6703 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 6105 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 4418 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 6372 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 4759 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 7785 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 4248 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 4379 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 7376 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 4926 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 5442 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 6162 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 7309 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 7696 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 6408 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 5467 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 5690 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 4411 },
	{ 0x8822, 0x8823, PDF_CMAP_TABLE, 1969 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 3758 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 6532 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 4685 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 7756 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 6924 },
	{ 0x884c, 0x884d, PDF_CMAP_TABLE, 1971 },
	{ 0x8852, 0x8853, PDF_CMAP_TABLE, 1973 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 3457 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 5786 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 6303 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 7310 },
	{ 0x8861, 0x8863, PDF_CMAP_TABLE, 1975 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 7585 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 5326 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 5565 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 4151 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 7311 },
	{ 0x887e, 0x887f, PDF_CMAP_RANGE, 4047 },
	{ 0x8881, 0x8882, PDF_CMAP_TABLE, 1978 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 3458 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 4262 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 7565 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 4209 },
	{ 0x8896, 0x8897, PDF_CMAP_TABLE, 1980 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 3782 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 7601 },
	{ 0x88b4, 0x88b4, PDF_CMAP_SINGLE, 3759 },
	{ 0x88c1, 0x88c2, PDF_CMAP_TABLE, 1982 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 4632 },
	{ 0x88d4, 0x88d5, PDF_CMAP_TABLE, 1984 },
	{ 0x88d9, 0x88d9, PDF_CMAP_SINGLE, 3956 },
	{ 0x88dc, 0x88dd, PDF_CMAP_TABLE, 1986 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 5290 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 4633 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 5215 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 5356 },
	{ 0x88f4, 0x88f5, PDF_CMAP_RANGE, 4981 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 4380 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 6773 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 5085 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 3507 },
	{ 0x8912, 0x8913, PDF_CMAP_TABLE, 1988 },
	{ 0x8918, 0x8919, PDF_CMAP_TABLE, 1990 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 6172 },
	{ 0x892a, 0x892a, PDF_CMAP_SINGLE, 7462 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 5680 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 4574 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 5458 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 3555 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 5897 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 4049 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 4407 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 4694 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 5681 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 5397 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 6163 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 4230 },
	{ 0x8986, 0x8987, PDF_CMAP_TABLE, 1992 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 3644 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 4005 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 4734 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 5714 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 3635 },
	{ 0x89a9, 0x89aa, PDF_CMAP_TABLE, 1994 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 4034 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 3473 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 4408 },
	{ 0x89c0, 0x89c0, PDF_CMAP_SINGLE, 3831 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 3474 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 7697 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 5357 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7246 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 5921 },
	{ 0x8a02, 0x8a03, PDF_CMAP_TABLE, 1996 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 3724 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 5754 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 7861 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 7452 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 7987 },
	{ 0x8a16, 0x8a18, PDF_CMAP_TABLE, 1998 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 6106 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 5787 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 5557 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 3653 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 4182 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 4964 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 5459 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 7722 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 5530 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 3459 },
	{ 0x8a3a, 0x8a3b, PDF_CMAP_TABLE, 2001 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 5291 },
	{ 0x8a54, 0x8a55, PDF_CMAP_TABLE, 2003 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 6612 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 5292 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 6025 },
	{ 0x8a62, 0x8a63, PDF_CMAP_TABLE, 2005 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 5715 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 5716 },
	{ 0x8a6d, 0x8a6e, PDF_CMAP_TABLE, 2007 },
	{ 0x8a70, 0x8a73, PDF_CMAP_TABLE, 2009 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 5443 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 7185 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 6891 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 3809 },
	{ 0x8a8c, 0x8a8d, PDF_CMAP_TABLE, 2013 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 5398 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 7417 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 6344 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 5908 },
	{ 0x8aa0, 0x8aa1, PDF_CMAP_TABLE, 2015 },
	{ 0x8aa3, 0x8aa6, PDF_CMAP_TABLE, 2017 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 7946 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 5460 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 5611 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 3810 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 5216 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 6410 },
	{ 0x8abe, 0x8abf, PDF_CMAP_TABLE, 2021 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 7186 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 5662 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 4231 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 7202 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 6589 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 7276 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 4437 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4535 },
	{ 0x8adb, 0x8adc, PDF_CMAP_TABLE, 2023 },
	{ 0x8ae1, 0x8ae1, PDF_CMAP_SINGLE, 5717 },
	{ 0x8ae6, 0x8ae7, PDF_CMAP_TABLE, 2025 },
	{ 0x8aea, 0x8aeb, PDF_CMAP_TABLE, 2027 },
	{ 0x8aed, 0x8aee, PDF_CMAP_TABLE, 2029 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 8002 },
	{ 0x8af6, 0x8af8, PDF_CMAP_TABLE, 2031 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 5922 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 4139 },
	{ 0x8b00, 0x8b02, PDF_CMAP_TABLE, 2034 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 4371 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 4866 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 4885 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 7633 },
	{ 0x8b16, 0x8b17, PDF_CMAP_TABLE, 2037 },
	{ 0x8b19, 0x8b1b, PDF_CMAP_TABLE, 2039 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 5293 },
	{ 0x8b20, 0x8b20, PDF_CMAP_SINGLE, 6164 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 4786 },
	{ 0x8b2b, 0x8b2c, PDF_CMAP_TABLE, 2042 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 3935 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 4035 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 7874 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 6940 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_TABLE, 2044 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 5731 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 4232 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 5072 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 3697 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 5217 },
	{ 0x8b6f, 0x8b70, PDF_CMAP_TABLE, 2046 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 3645 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 7836 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 6044 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 4318 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5033 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 5612 },
	{ 0x8b92, 0x8b93, PDF_CMAP_TABLE, 2048 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 7091 },
	{ 0x8b9a, 0x8b9a, PDF_CMAP_SINGLE, 7072 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 3772 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 3726 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 7902 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 4355 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 4116 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 7594 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 6122 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 5718 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 4327 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 5359 },
	{ 0x8c6a, 0x8c6b, PDF_CMAP_TABLE, 2050 },
	{ 0x8c79, 0x8c7a, PDF_CMAP_TABLE, 2052 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 7236 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 4723 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 4787 },
	{ 0x8c9d, 0x8c9e, PDF_CMAP_TABLE, 2054 },
	{ 0x8ca0, 0x8ca2, PDF_CMAP_TABLE, 2056 },
	{ 0x8ca7, 0x8cac, PDF_CMAP_TABLE, 2059 },
	{ 0x8caf, 0x8cb0, PDF_CMAP_TABLE, 2065 },
	{ 0x8cb3, 0x8cb4, PDF_CMAP_TABLE, 2067 },
	{ 0x8cb6, 0x8cb8, PDF_CMAP_TABLE, 2069 },
	{ 0x8cbb, 0x8cbd, PDF_CMAP_TABLE, 2072 },
	{ 0x8cbf, 0x8cc4, PDF_CMAP_TABLE, 2075 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_TABLE, 2081 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 6635 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 7007 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 5235 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 4548 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 5294 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 5360 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 4984 },
	{ 0x8ce2, 0x8ce4, PDF_CMAP_TABLE, 2083 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 5141 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 7027 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 4299 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 4549 },
	{ 0x8cfb, 0x8cfd, PDF_CMAP_TABLE, 2086 },
	{ 0x8d04, 0x8d05, PDF_CMAP_TABLE, 2089 },
	{ 0x8d07, 0x8d08, PDF_CMAP_TABLE, 2091 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 7073 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 5468 },
	{ 0x8d13, 0x8d13, PDF_CMAP_SINGLE, 6566 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 5544 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 6636 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 5295 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 7732 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 6892 },
	{ 0x8d73, 0x8d74, PDF_CMAP_TABLE, 2093 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 4117 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 7237 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 6282 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 6820 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 7326 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 7277 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 6830 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 5144 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 6973 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 7442 },
	{ 0x8dcb, 0x8dcc, PDF_CMAP_TABLE, 2095 },
	{ 0x8dcf, 0x8dcf, PDF_CMAP_SINGLE, 3461 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 7488 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 3596 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 6637 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 5444 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 3811 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 4522 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 4300 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 6194 },
	{ 0x8e0f, 0x8e10, PDF_CMAP_TABLE, 2097 },
	{ 0x8e1e, 0x8e1e, PDF_CMAP_SINGLE, 3597 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 6850 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 6347 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 6851 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 6348 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 6775 },
	{ 0x8e47, 0x8e4a, PDF_CMAP_TABLE, 2099 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 7296 },
	{ 0x8e5f, 0x8e60, PDF_CMAP_TABLE, 2103 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 7297 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 3982 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 6821 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 6614 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 6893 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 5878 },
	{ 0x8eaa, 0x8eac, PDF_CMAP_TABLE, 2105 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 3937 },
	{ 0x8eca, 0x8ecd, PDF_CMAP_TABLE, 2108 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 7726 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 5982 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 7008 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 7298 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 3462 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 5732 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 3887 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 4523 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 6584 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 2112 },
	{ 0x8f1b, 0x8f1f, PDF_CMAP_TABLE, 2116 },
	{ 0x8f26, 0x8f27, PDF_CMAP_TABLE, 2121 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 2123 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 7035 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 6894 },
	{ 0x8f38, 0x8f39, PDF_CMAP_TABLE, 2125 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 5088 },
	{ 0x8f3e, 0x8f3f, PDF_CMAP_TABLE, 2127 },
	{ 0x8f44, 0x8f45, PDF_CMAP_TABLE, 2129 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 6675 },
	{ 0x8f4d, 0x8f4e, PDF_CMAP_TABLE, 2131 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 5945 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 3867 },
	{ 0x8f62, 0x8f62, PDF_CMAP_SINGLE, 4463 },
	{ 0x8f9b, 0x8f9c, PDF_CMAP_TABLE, 2133 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 4398 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 7496 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 5034 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 5296 },
	{ 0x8faf, 0x8fb2, PDF_CMAP_TABLE, 2135 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 6222 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 5757 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 6026 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 4036 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 4927 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 3463 },
	{ 0x8fea, 0x8feb, PDF_CMAP_TABLE, 2139 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 7029 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 5669 },
	{ 0x8ff2, 0x8ff2, PDF_CMAP_SINGLE, 3626 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 4867 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 6640 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 7278 },
	{ 0x9000, 0x9003, PDF_CMAP_TABLE, 2141 },
	{ 0x9005, 0x9006, PDF_CMAP_TABLE, 2145 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 7787 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 7567 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 5531 },
	{ 0x900f, 0x9011, PDF_CMAP_TABLE, 2147 },
	{ 0x9014, 0x9015, PDF_CMAP_TABLE, 2150 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 4356 },
	{ 0x9019, 0x901a, PDF_CMAP_TABLE, 2152 },
	{ 0x901d, 0x9023, PDF_CMAP_TABLE, 2154 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 7211 },
	{ 0x9031, 0x9032, PDF_CMAP_TABLE, 2161 },
	{ 0x9035, 0x9035, PDF_CMAP_SINGLE, 4007 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 6466 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 7615 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 6350 },
	{ 0x9041, 0x9042, PDF_CMAP_TABLE, 2163 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 6223 },
	{ 0x904a, 0x904b, PDF_CMAP_TABLE, 2165 },
	{ 0x904d, 0x904e, PDF_CMAP_TABLE, 2167 },
	{ 0x9050, 0x9051, PDF_CMAP_TABLE, 2169 },
	{ 0x9053, 0x9055, PDF_CMAP_TABLE, 2171 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 6165 },
	{ 0x905c, 0x905e, PDF_CMAP_TABLE, 2174 },
	{ 0x9060, 0x9061, PDF_CMAP_TABLE, 2177 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 3646 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 6641 },
	{ 0x906d, 0x906f, PDF_CMAP_TABLE, 2179 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 6974 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 6918 },
	{ 0x9077, 0x9078, PDF_CMAP_TABLE, 2182 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 6351 },
	{ 0x907c, 0x907d, PDF_CMAP_TABLE, 2184 },
	{ 0x907f, 0x9084, PDF_CMAP_TABLE, 2186 },
	{ 0x9087, 0x9088, PDF_CMAP_TABLE, 2192 },
	{ 0x908a, 0x908a, PDF_CMAP_SINGLE, 5036 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 4381 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 6389 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 6096 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 4707 },
	{ 0x90a2, 0x90a3, PDF_CMAP_TABLE, 2194 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 4966 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 7250 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 5297 },
	{ 0x90af, 0x90b1, PDF_CMAP_TABLE, 2196 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 5533 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 6616 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 6236 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 3889 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 4419 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 3958 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 5145 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 3818 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 6224 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 4305 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 5802 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 7279 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 7714 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 5219 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 4372 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 6748 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 4210 },
	{ 0x9149, 0x914e, PDF_CMAP_TABLE, 2199 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 6897 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 7238 },
	{ 0x9169, 0x916a, PDF_CMAP_TABLE, 2205 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 5616 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 7964 },
	{ 0x9177, 0x9178, PDF_CMAP_TABLE, 2207 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 5663 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 7327 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 7239 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 6776 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 5491 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 7281 },
	{ 0x91ab, 0x91ac, PDF_CMAP_TABLE, 2209 },
	{ 0x91ae, 0x91af, PDF_CMAP_TABLE, 2211 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 4937 },
	{ 0x91b4, 0x91b5, PDF_CMAP_TABLE, 2213 },
	{ 0x91c0, 0x91c0, PDF_CMAP_SINGLE, 5899 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 7124 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 6353 },
	{ 0x91cb, 0x91d1, PDF_CMAP_TABLE, 2215 },
	{ 0x91d7, 0x91d8, PDF_CMAP_TABLE, 2222 },
	{ 0x91dc, 0x91dd, PDF_CMAP_TABLE, 2224 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 6824 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 7163 },
	{ 0x91ea, 0x91ea, PDF_CMAP_SINGLE, 6225 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 7125 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 4363 },
	{ 0x9210, 0x9212, PDF_CMAP_TABLE, 2226 },
	{ 0x9217, 0x9217, PDF_CMAP_SINGLE, 6366 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 4014 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 4501 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 6082 },
	{ 0x923f, 0x9240, PDF_CMAP_TABLE, 2229 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 3599 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 7752 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 3658 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 5983 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 6283 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 4938 },
	{ 0x9264, 0x9266, PDF_CMAP_TABLE, 2231 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 6378 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 7260 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 4347 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 5446 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 6677 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 5617 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 4761 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 7660 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 6046 },
	{ 0x92b6, 0x92b7, PDF_CMAP_TABLE, 2234 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 5618 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 6752 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 7769 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 5107 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 5400 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 7568 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 3600 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 3557 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 4532 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 7282 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 7283 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 6590 },
	{ 0x931e, 0x9322, PDF_CMAP_TABLE, 2236 },
	{ 0x9324, 0x9324, PDF_CMAP_SINGLE, 4119 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 4050 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 4811 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 5417 },
	{ 0x932e, 0x932f, PDF_CMAP_TABLE, 2241 },
	{ 0x9348, 0x9348, PDF_CMAP_SINGLE, 6027 },
	{ 0x934a, 0x934b, PDF_CMAP_TABLE, 2243 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 4306 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 5803 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 4211 },
	{ 0x936e, 0x936e, PDF_CMAP_SINGLE, 6354 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 3611 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 7375 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 6852 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 3659 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 6195 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 5564 },
	{ 0x939a, 0x939a, PDF_CMAP_SINGLE, 7284 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 7789 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 3577 },
	{ 0x93ac, 0x93ad, PDF_CMAP_TABLE, 2245 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 6467 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 6831 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 6642 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 6196 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 3700 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 4575 },
	{ 0x93f6, 0x93f6, PDF_CMAP_SINGLE, 7036 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 7952 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 6853 },
	{ 0x9425, 0x9425, PDF_CMAP_SINGLE, 5447 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 6679 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 7176 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 7407 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 6898 },
	{ 0x9451, 0x9452, PDF_CMAP_RANGE, 3527 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 3851 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 7074 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 7060 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 6568 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 4846 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 5469 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 7542 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 3578 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 6367 },
	{ 0x9591, 0x9592, PDF_CMAP_RANGE, 7645 },
	{ 0x9593, 0x9594, PDF_CMAP_TABLE, 2247 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 3535 },
	{ 0x95a3, 0x95a5, PDF_CMAP_TABLE, 2249 },
	{ 0x95a8, 0x95a8, PDF_CMAP_SINGLE, 4008 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 4453 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 5988 },
	{ 0x95bb, 0x95bc, PDF_CMAP_TABLE, 2252 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 5828 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 7903 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 2254 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 3833 },
	{ 0x95e1, 0x95e2, PDF_CMAP_TABLE, 2257 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 5147 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 7165 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 7498 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 6123 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 4967 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 6825 },
	{ 0x963f, 0x9640, PDF_CMAP_TABLE, 2259 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 7603 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 5148 },
	{ 0x964b, 0x964d, PDF_CMAP_TABLE, 2261 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 7647 },
	{ 0x965b, 0x965f, PDF_CMAP_TABLE, 2264 },
	{ 0x9662, 0x9664, PDF_CMAP_TABLE, 2269 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 4987 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 6384 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 7013 },
	{ 0x9675, 0x9678, PDF_CMAP_TABLE, 2272 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 5900 },
	{ 0x9685, 0x9686, PDF_CMAP_TABLE, 2276 },
	{ 0x968a, 0x968b, PDF_CMAP_TABLE, 2278 },
	{ 0x968d, 0x968e, PDF_CMAP_TABLE, 2280 },
	{ 0x9694, 0x9695, PDF_CMAP_TABLE, 2282 },
	{ 0x9698, 0x9699, PDF_CMAP_TABLE, 2284 },
	{ 0x969b, 0x969c, PDF_CMAP_TABLE, 2286 },
	{ 0x96a3, 0x96a3, PDF_CMAP_SINGLE, 4644 },
	{ 0x96a7, 0x96a8, PDF_CMAP_RANGE, 5620 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 7728 },
	{ 0x96b1, 0x96b1, PDF_CMAP_SINGLE, 6379 },
	{ 0x96b7, 0x96b7, PDF_CMAP_SINGLE, 4510 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 7147 },
	{ 0x96c0, 0x96c1, PDF_CMAP_TABLE, 2288 },
	{ 0x96c4, 0x96c7, PDF_CMAP_TABLE, 2290 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 7357 },
	{ 0x96cb, 0x96ce, PDF_CMAP_TABLE, 2294 },
	{ 0x96d5, 0x96d6, PDF_CMAP_TABLE, 2298 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 5773 },
	{ 0x96db, 0x96dc, PDF_CMAP_TABLE, 2300 },
	{ 0x96e2, 0x96e3, PDF_CMAP_TABLE, 2302 },
	{ 0x96e8, 0x96e9, PDF_CMAP_RANGE, 6227 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 5461 },
	{ 0x96ef, 0x96f0, PDF_CMAP_TABLE, 2304 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 6248 },
	{ 0x96f6, 0x96f7, PDF_CMAP_TABLE, 2306 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 4903 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 6680 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 5623 },
	{ 0x9706, 0x9707, PDF_CMAP_TABLE, 2308 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 6698 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 6047 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 4651 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 6028 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 5361 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 7628 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 4832 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 5314 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 4524 },
	{ 0x9739, 0x9739, PDF_CMAP_SINGLE, 5030 },
	{ 0x973d, 0x973d, PDF_CMAP_SINGLE, 6779 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 4464 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 5850 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 4503 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 7203 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 6755 },
	{ 0x975c, 0x975c, PDF_CMAP_SINGLE, 6756 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 5220 },
	{ 0x9761, 0x9762, PDF_CMAP_TABLE, 2310 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 7733 },
	{ 0x976d, 0x976d, PDF_CMAP_SINGLE, 6458 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 7876 },
	{ 0x9777, 0x9777, PDF_CMAP_SINGLE, 6459 },
	{ 0x977a, 0x977a, PDF_CMAP_SINGLE, 4695 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 7799 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 5814 },
	{ 0x978f, 0x978f, PDF_CMAP_SINGLE, 3801 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 3950 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 3509 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 3951 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 7526 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 7166 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 6307 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 7648 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 4308 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 6385 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 5535 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 6249 },
	{ 0x97ff, 0x9803, PDF_CMAP_TABLE, 2312 },
	{ 0x9805, 0x9806, PDF_CMAP_TABLE, 2317 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 5624 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 6237 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 5560 },
	{ 0x9810, 0x9813, PDF_CMAP_TABLE, 2319 },
	{ 0x9817, 0x9818, PDF_CMAP_TABLE, 2323 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 4357 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 7770 },
	{ 0x9838, 0x9839, PDF_CMAP_TABLE, 2325 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 5236 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 3814 },
	{ 0x984c, 0x984e, PDF_CMAP_TABLE, 2327 },
	{ 0x9854, 0x9854, PDF_CMAP_SINGLE, 5815 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 6279 },
	{ 0x985a, 0x985a, PDF_CMAP_SINGLE, 6681 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 4590 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 7840 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 3764 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 6682 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 7753 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 7595 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 5330 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 7444 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 7588 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 7587 },
	{ 0x98db, 0x98dc, PDF_CMAP_TABLE, 2330 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 5733 },
	{ 0x98e1, 0x98e2, PDF_CMAP_TABLE, 2332 },
	{ 0x98ed, 0x98ef, PDF_CMAP_TABLE, 2334 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 6435 },
	{ 0x98fc, 0x98fe, PDF_CMAP_TABLE, 2337 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 3890 },
	{ 0x9909, 0x990a, PDF_CMAP_TABLE, 2340 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 6436 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 7075 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 5790 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 5946 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 6683 },
	{ 0x9920, 0x9920, PDF_CMAP_SINGLE, 5056 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 3834 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 4687 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 4037 },
	{ 0x994b, 0x994d, PDF_CMAP_TABLE, 2342 },
	{ 0x9951, 0x9952, PDF_CMAP_TABLE, 2345 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 6098 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 7717 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 5625 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 7718 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 7613 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 5089 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 7790 },
	{ 0x99ac, 0x99ae, PDF_CMAP_TABLE, 2347 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 7392 },
	{ 0x99b3, 0x99b4, PDF_CMAP_TABLE, 2350 },
	{ 0x99b9, 0x99b9, PDF_CMAP_SINGLE, 6468 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 4904 },
	{ 0x99d0, 0x99d2, PDF_CMAP_TABLE, 2352 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 3464 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 5149 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 7393 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 5299 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 7701 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 4388 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 6920 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 5240 },
	{ 0x9a08, 0x9a08, PDF_CMAP_SINGLE, 5057 },
	{ 0x9a0e, 0x9a0f, PDF_CMAP_RANGE, 4122 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 7527 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 3612 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 4373 },
	{ 0x9a36, 0x9a37, PDF_CMAP_TABLE, 2355 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 4725 },
	{ 0x9a43, 0x9a43, PDF_CMAP_SINGLE, 7589 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 3943 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 7965 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 3891 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 7729 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_TABLE, 2357 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 7328 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 4454 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 4124 },
	{ 0x9a69, 0x9a6a, PDF_CMAP_TABLE, 2359 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 3785 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 7702 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_TABLE, 2361 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 3765 },
	{ 0x9ae5, 0x9ae5, PDF_CMAP_SINGLE, 5997 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 4939 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 5627 },
	{ 0x9b27, 0x9b27, PDF_CMAP_SINGLE, 4562 },
	{ 0x9b2a, 0x9b2a, PDF_CMAP_SINGLE, 7470 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 6251 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 3993 },
	{ 0x9b41, 0x9b45, PDF_CMAP_TABLE, 2363 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 6308 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 4662 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 5910 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4525 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 6699 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 7570 },
	{ 0x9b9f, 0x9b9f, PDF_CMAP_SINGLE, 5816 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 3892 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 5449 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 4637 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 7204 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 3783 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 3704 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 7287 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 5805 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 5090 },
	{ 0x9c15, 0x9c15, PDF_CMAP_SINGLE, 7629 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 7898 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 6076 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 4688 },
	{ 0x9c47, 0x9c47, PDF_CMAP_SINGLE, 3559 },
	{ 0x9c49, 0x9c49, PDF_CMAP_SINGLE, 5039 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 4645 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 6827 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 5150 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3944 },
	{ 0x9cf3, 0x9cf4, PDF_CMAP_TABLE, 2368 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 5984 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 5791 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 6280 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 5839 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 5832 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 7862 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 3647 },
	{ 0x9d5d, 0x9d5d, PDF_CMAP_SINGLE, 5792 },
	{ 0x9d60, 0x9d61, PDF_CMAP_TABLE, 2370 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 5180 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 6521 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 4794 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 5860 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 7634 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 3728 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 3945 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 7329 },
	{ 0x9df8, 0x9dfa, PDF_CMAP_TABLE, 2372 },
	{ 0x9e1a, 0x9e1a, PDF_CMAP_SINGLE, 5861 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 4396 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 4527 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 7662 },
	{ 0x9e7d, 0x9e7d, PDF_CMAP_SINGLE, 5998 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 4533 },
	{ 0x9e92, 0x9e93, PDF_CMAP_TABLE, 2375 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 4456 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 5300 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 4646 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 4726 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 2377 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 4663 },
	{ 0x9ebe, 0x9ebe, PDF_CMAP_SINGLE, 8004 },
	{ 0x9ec3, 0x9ec3, PDF_CMAP_SINGLE, 7927 },
	{ 0x9ecd, 0x9ece, PDF_CMAP_TABLE, 2379 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 8018 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 3623 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 4835 },
	{ 0x9edb, 0x9edc, PDF_CMAP_TABLE, 2381 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 6700 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 4249 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 4869 },
	{ 0x9f07, 0x9f08, PDF_CMAP_TABLE, 2383 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 6758 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 3766 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 5402 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 5222 },
	{ 0x9f4a, 0x9f4b, PDF_CMAP_TABLE, 2385 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 6586 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7359 },
	{ 0x9f5f, 0x9f5f, PDF_CMAP_SINGLE, 6618 },
	{ 0x9f61, 0x9f61, PDF_CMAP_SINGLE, 4505 },
	{ 0x9f67, 0x9f67, PDF_CMAP_SINGLE, 5462 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 7061 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 5911 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 5806 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 4563 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 4968 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 3529 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 3946 },
	{ 0xac00, 0xac01, PDF_CMAP_RANGE, 1086 },
	{ 0xac02, 0xac03, PDF_CMAP_RANGE, 9333 },
	{ 0xac04, 0xac04, PDF_CMAP_SINGLE, 1088 },
	{ 0xac05, 0xac06, PDF_CMAP_RANGE, 9335 },
	{ 0xac07, 0xac0a, PDF_CMAP_RANGE, 1089 },
	{ 0xac0b, 0xac0f, PDF_CMAP_RANGE, 9337 },
	{ 0xac10, 0xac17, PDF_CMAP_RANGE, 1093 },
	{ 0xac18, 0xac18, PDF_CMAP_SINGLE, 9342 },
	{ 0xac19, 0xac1d, PDF_CMAP_RANGE, 1101 },
	{ 0xac1e, 0xac1f, PDF_CMAP_RANGE, 9343 },
	{ 0xac20, 0xac20, PDF_CMAP_SINGLE, 1106 },
	{ 0xac21, 0xac23, PDF_CMAP_RANGE, 9345 },
	{ 0xac24, 0xac24, PDF_CMAP_SINGLE, 1107 },
	{ 0xac25, 0xac2b, PDF_CMAP_RANGE, 9348 },
	{ 0xac2c, 0xac2d, PDF_CMAP_RANGE, 1108 },
	{ 0xac2e, 0xac2e, PDF_CMAP_SINGLE, 9355 },
	{ 0xac2f, 0xac31, PDF_CMAP_RANGE, 1110 },
	{ 0xac32, 0xac37, PDF_CMAP_RANGE, 9356 },
	{ 0xac38, 0xac39, PDF_CMAP_RANGE, 1113 },
	{ 0xac3a, 0xac3b, PDF_CMAP_RANGE, 9362 },
	{ 0xac3c, 0xac3c, PDF_CMAP_SINGLE, 1115 },
	{ 0xac3d, 0xac3f, PDF_CMAP_RANGE, 9364 },
	{ 0xac40, 0xac40, PDF_CMAP_SINGLE, 1116 },
	{ 0xac41, 0xac4a, PDF_CMAP_RANGE, 9367 },
	{ 0xac4b, 0xac4d, PDF_CMAP_TABLE, 2387 },
	{ 0xac4e, 0xac53, PDF_CMAP_RANGE, 9378 },
	{ 0xac54, 0xac54, PDF_CMAP_SINGLE, 1119 },
	{ 0xac55, 0xac57, PDF_CMAP_RANGE, 9384 },
	{ 0xac58, 0xac58, PDF_CMAP_SINGLE, 1120 },
	{ 0xac59, 0xac5b, PDF_CMAP_RANGE, 9387 },
	{ 0xac5c, 0xac5c, PDF_CMAP_SINGLE, 1121 },
	{ 0xac5d, 0xac6f, PDF_CMAP_RANGE, 9390 },
	{ 0xac70, 0xac71, PDF_CMAP_RANGE, 1122 },
	{ 0xac72, 0xac73, PDF_CMAP_RANGE, 9409 },
	{ 0xac74, 0xac74, PDF_CMAP_SINGLE, 1124 },
	{ 0xac75, 0xac76, PDF_CMAP_RANGE, 9411 },
	{ 0xac77, 0xac78, PDF_CMAP_RANGE, 1125 },
	{ 0xac79, 0xac7a, PDF_CMAP_TABLE, 2390 },
	{ 0xac7b, 0xac7f, PDF_CMAP_RANGE, 9414 },
	{ 0xac80, 0xac81, PDF_CMAP_RANGE, 1128 },
	{ 0xac82, 0xac82, PDF_CMAP_SINGLE, 9419 },
	{ 0xac83, 0xac86, PDF_CMAP_RANGE, 1130 },
	{ 0xac87, 0xac88, PDF_CMAP_RANGE, 9420 },
	{ 0xac89, 0xac8c, PDF_CMAP_RANGE, 1134 },
	{ 0xac8d, 0xac8f, PDF_CMAP_RANGE, 9422 },
	{ 0xac90, 0xac90, PDF_CMAP_SINGLE, 1138 },
	{ 0xac91, 0xac93, PDF_CMAP_RANGE, 9425 },
	{ 0xac94, 0xac94, PDF_CMAP_SINGLE, 1139 },
	{ 0xac95, 0xac9b, PDF_CMAP_RANGE, 9428 },
	{ 0xac9c, 0xac9d, PDF_CMAP_RANGE, 1140 },
	{ 0xac9e, 0xac9e, PDF_CMAP_SINGLE, 9435 },
	{ 0xac9f, 0xaca1, PDF_CMAP_RANGE, 1142 },
	{ 0xaca2, 0xaca7, PDF_CMAP_RANGE, 9436 },
	{ 0xaca8, 0xacaa, PDF_CMAP_RANGE, 1145 },
	{ 0xacab, 0xacac, PDF_CMAP_TABLE, 2392 },
	{ 0xacad, 0xacae, PDF_CMAP_RANGE, 9443 },
	{ 0xacaf, 0xacb0, PDF_CMAP_RANGE, 1149 },
	{ 0xacb1, 0xacb7, PDF_CMAP_RANGE, 9445 },
	{ 0xacb8, 0xacb9, PDF_CMAP_RANGE, 1151 },
	{ 0xacba, 0xacba, PDF_CMAP_SINGLE, 9452 },
	{ 0xacbb, 0xacbd, PDF_CMAP_RANGE, 1153 },
	{ 0xacbe, 0xacc0, PDF_CMAP_RANGE, 9453 },
	{ 0xacc1, 0xacc1, PDF_CMAP_SINGLE, 1156 },
	{ 0xacc2, 0xacc3, PDF_CMAP_RANGE, 9456 },
	{ 0xacc4, 0xacc4, PDF_CMAP_SINGLE, 1157 },
	{ 0xacc5, 0xacc7, PDF_CMAP_RANGE, 9458 },
	{ 0xacc8, 0xacc8, PDF_CMAP_SINGLE, 1158 },
	{ 0xacc9, 0xaccb, PDF_CMAP_RANGE, 9461 },
	{ 0xaccc, 0xaccc, PDF_CMAP_SINGLE, 1159 },
	{ 0xaccd, 0xacd4, PDF_CMAP_RANGE, 9464 },
	{ 0xacd5, 0xacd7, PDF_CMAP_TABLE, 2394 },
	{ 0xacd8, 0xacdf, PDF_CMAP_RANGE, 9473 },
	{ 0xace0, 0xace1, PDF_CMAP_RANGE, 1162 },
	{ 0xace2, 0xace3, PDF_CMAP_RANGE, 9481 },
	{ 0xace4, 0xace4, PDF_CMAP_SINGLE, 1164 },
	{ 0xace5, 0xace6, PDF_CMAP_RANGE, 9483 },
	{ 0xace7, 0xace8, PDF_CMAP_RANGE, 1165 },
	{ 0xace9, 0xacec, PDF_CMAP_TABLE, 2397 },
	{ 0xaced, 0xacee, PDF_CMAP_RANGE, 9487 },
	{ 0xacef, 0xacf1, PDF_CMAP_RANGE, 1169 },
	{ 0xacf2, 0xacf4, PDF_CMAP_TABLE, 2401 },
	{ 0xacf5, 0xacf6, PDF_CMAP_RANGE, 1173 },
	{ 0xacf7, 0xacfb, PDF_CMAP_RANGE, 9491 },
	{ 0xacfc, 0xacfd, PDF_CMAP_RANGE, 1175 },
	{ 0xacfe, 0xacff, PDF_CMAP_RANGE, 9496 },
	{ 0xad00, 0xad00, PDF_CMAP_SINGLE, 1177 },
	{ 0xad01, 0xad03, PDF_CMAP_RANGE, 9498 },
	{ 0xad04, 0xad06, PDF_CMAP_TABLE, 2404 },
	{ 0xad07, 0xad0b, PDF_CMAP_RANGE, 9502 },
	{ 0xad0c, 0xad0d, PDF_CMAP_RANGE, 1180 },
	{ 0xad0e, 0xad11, PDF_CMAP_TABLE, 2407 },
	{ 0xad12, 0xad17, PDF_CMAP_RANGE, 9509 },
	{ 0xad18, 0xad18, PDF_CMAP_SINGLE, 1184 },
	{ 0xad19, 0xad1b, PDF_CMAP_RANGE, 9515 },
	{ 0xad1c, 0xad1c, PDF_CMAP_SINGLE, 1185 },
	{ 0xad1d, 0xad1f, PDF_CMAP_RANGE, 9518 },
	{ 0xad20, 0xad20, PDF_CMAP_SINGLE, 1186 },
	{ 0xad21, 0xad28, PDF_CMAP_RANGE, 9521 },
	{ 0xad29, 0xad29, PDF_CMAP_SINGLE, 1187 },
	{ 0xad2a, 0xad2b, PDF_CMAP_RANGE, 9529 },
	{ 0xad2c, 0xad2d, PDF_CMAP_RANGE, 1188 },
	{ 0xad2e, 0xad33, PDF_CMAP_RANGE, 9531 },
	{ 0xad34, 0xad35, PDF_CMAP_RANGE, 1190 },
	{ 0xad36, 0xad37, PDF_CMAP_RANGE, 9537 },
	{ 0xad38, 0xad38, PDF_CMAP_SINGLE, 1192 },
	{ 0xad39, 0xad3b, PDF_CMAP_RANGE, 9539 },
	{ 0xad3c, 0xad3c, PDF_CMAP_SINGLE, 1193 },
	{ 0xad3d, 0xad43, PDF_CMAP_RANGE, 9542 },
	{ 0xad44, 0xad45, PDF_CMAP_RANGE, 1194 },
	{ 0xad46, 0xad49, PDF_CMAP_TABLE, 2411 },
	{ 0xad4a, 0xad4f, PDF_CMAP_RANGE, 9551 },
	{ 0xad50, 0xad50, PDF_CMAP_SINGLE, 1198 },
	{ 0xad51, 0xad53, PDF_CMAP_RANGE, 9557 },
	{ 0xad54, 0xad54, PDF_CMAP_SINGLE, 1199 },
	{ 0xad55, 0xad57, PDF_CMAP_RANGE, 9560 },
	{ 0xad58, 0xad58, PDF_CMAP_SINGLE, 1200 },
	{ 0xad59, 0xad60, PDF_CMAP_RANGE, 9563 },
	{ 0xad61, 0xad63, PDF_CMAP_TABLE, 2415 },
	{ 0xad64, 0xad6b, PDF_CMAP_RANGE, 9572 },
	{ 0xad6c, 0xad6d, PDF_CMAP_RANGE, 1203 },
	{ 0xad6e, 0xad6f, PDF_CMAP_RANGE, 9580 },
	{ 0xad70, 0xad70, PDF_CMAP_SINGLE, 1205 },
	{ 0xad71, 0xad72, PDF_CMAP_RANGE, 9582 },
	{ 0xad73, 0xad76, PDF_CMAP_RANGE, 1206 },
	{ 0xad77, 0xad7a, PDF_CMAP_RANGE, 9584 },
	{ 0xad7b, 0xad7d, PDF_CMAP_RANGE, 1210 },
	{ 0xad7e, 0xad80, PDF_CMAP_TABLE, 2418 },
	{ 0xad81, 0xad82, PDF_CMAP_RANGE, 1214 },
	{ 0xad83, 0xad87, PDF_CMAP_RANGE, 9590 },
	{ 0xad88, 0xad89, PDF_CMAP_RANGE, 1216 },
	{ 0xad8a, 0xad8b, PDF_CMAP_RANGE, 9595 },
	{ 0xad8c, 0xad8c, PDF_CMAP_SINGLE, 1218 },
	{ 0xad8d, 0xad8f, PDF_CMAP_RANGE, 9597 },
	{ 0xad90, 0xad90, PDF_CMAP_SINGLE, 1219 },
	{ 0xad91, 0xad9b, PDF_CMAP_RANGE, 9600 },
	{ 0xad9c, 0xad9d, PDF_CMAP_RANGE, 1220 },
	{ 0xad9e, 0xada3, PDF_CMAP_RANGE, 9611 },
	{ 0xada4, 0xada4, PDF_CMAP_SINGLE, 1222 },
	{ 0xada5, 0xadb6, PDF_CMAP_RANGE, 9617 },
	{ 0xadb7, 0xadb7, PDF_CMAP_SINGLE, 1223 },
	{ 0xadb8, 0xadbf, PDF_CMAP_RANGE, 9635 },
	{ 0xadc0, 0xadc1, PDF_CMAP_RANGE, 1224 },
	{ 0xadc2, 0xadc3, PDF_CMAP_RANGE, 9643 },
	{ 0xadc4, 0xadc4, PDF_CMAP_SINGLE, 1226 },
	{ 0xadc5, 0xadc7, PDF_CMAP_RANGE, 9645 },
	{ 0xadc8, 0xadc8, PDF_CMAP_SINGLE, 1227 },
	{ 0xadc9, 0xadcf, PDF_CMAP_RANGE, 9648 },
	{ 0xadd0, 0xadd1, PDF_CMAP_RANGE, 1228 },
	{ 0xadd2, 0xadd3, PDF_CMAP_TABLE, 2421 },
	{ 0xadd4, 0xaddb, PDF_CMAP_RANGE, 9656 },
	{ 0xaddc, 0xaddc, PDF_CMAP_SINGLE, 1231 },
	{ 0xaddd, 0xaddf, PDF_CMAP_RANGE, 9664 },
	{ 0xade0, 0xade0, PDF_CMAP_SINGLE, 1232 },
	{ 0xade1, 0xade3, PDF_CMAP_RANGE, 9667 },
	{ 0xade4, 0xade4, PDF_CMAP_SINGLE, 1233 },
	{ 0xade5, 0xadf7, PDF_CMAP_RANGE, 9670 },
	{ 0xadf8, 0xadf9, PDF_CMAP_RANGE, 1234 },
	{ 0xadfa, 0xadfb, PDF_CMAP_RANGE, 9689 },
	{ 0xadfc, 0xadfc, PDF_CMAP_SINGLE, 1236 },
	{ 0xadfd, 0xadfe, PDF_CMAP_RANGE, 9691 },
	{ 0xadff, 0xae01, PDF_CMAP_RANGE, 1237 },
	{ 0xae02, 0xae07, PDF_CMAP_RANGE, 9693 },
	{ 0xae08, 0xae09, PDF_CMAP_RANGE, 1240 },
	{ 0xae0a, 0xae0d, PDF_CMAP_TABLE, 2423 },
	{ 0xae0e, 0xae13, PDF_CMAP_RANGE, 9701 },
	{ 0xae14, 0xae14, PDF_CMAP_SINGLE, 1244 },
	{ 0xae15, 0xae2f, PDF_CMAP_RANGE, 9707 },
	{ 0xae30, 0xae31, PDF_CMAP_RANGE, 1245 },
	{ 0xae32, 0xae33, PDF_CMAP_RANGE, 9734 },
	{ 0xae34, 0xae34, PDF_CMAP_SINGLE, 1247 },
	{ 0xae35, 0xae36, PDF_CMAP_RANGE, 9736 },
	{ 0xae37, 0xae38, PDF_CMAP_RANGE, 1248 },
	{ 0xae39, 0xae3a, PDF_CMAP_TABLE, 2427 },
	{ 0xae3b, 0xae3f, PDF_CMAP_RANGE, 9739 },
	{ 0xae40, 0xae41, PDF_CMAP_RANGE, 1251 },
	{ 0xae42, 0xae44, PDF_CMAP_TABLE, 2429 },
	{ 0xae45, 0xae46, PDF_CMAP_RANGE, 1254 },
	{ 0xae47, 0xae49, PDF_CMAP_RANGE, 9746 },
	{ 0xae4a, 0xae4b, PDF_CMAP_TABLE, 2432 },
	{ 0xae4c, 0xae4e, PDF_CMAP_RANGE, 1257 },
	{ 0xae4f, 0xae50, PDF_CMAP_TABLE, 2434 },
	{ 0xae51, 0xae53, PDF_CMAP_RANGE, 9751 },
	{ 0xae54, 0xae56, PDF_CMAP_TABLE, 2436 },
	{ 0xae57, 0xae5b, PDF_CMAP_RANGE, 9755 },
	{ 0xae5c, 0xae5d, PDF_CMAP_RANGE, 1263 },
	{ 0xae5e, 0xae5e, PDF_CMAP_SINGLE, 9760 },
	{ 0xae5f, 0xae61, PDF_CMAP_RANGE, 1265 },
	{ 0xae62, 0xae64, PDF_CMAP_RANGE, 9761 },
	{ 0xae65, 0xae65, PDF_CMAP_SINGLE, 1268 },
	{ 0xae66, 0xae67, PDF_CMAP_RANGE, 9764 },
	{ 0xae68, 0xae69, PDF_CMAP_RANGE, 1269 },
	{ 0xae6a, 0xae6b, PDF_CMAP_RANGE, 9766 },
	{ 0xae6c, 0xae6c, PDF_CMAP_SINGLE, 1271 },
	{ 0xae6d, 0xae6f, PDF_CMAP_RANGE, 9768 },
	{ 0xae70, 0xae70, PDF_CMAP_SINGLE, 1272 },
	{ 0xae71, 0xae77, PDF_CMAP_RANGE, 9771 },
	{ 0xae78, 0xae79, PDF_CMAP_RANGE, 1273 },
	{ 0xae7a, 0xae7a, PDF_CMAP_SINGLE, 9778 },
	{ 0xae7b, 0xae7d, PDF_CMAP_RANGE, 1275 },
	{ 0xae7e, 0xae83, PDF_CMAP_RANGE, 9779 },
	{ 0xae84, 0xae85, PDF_CMAP_RANGE, 1278 },
	{ 0xae86, 0xae8b, PDF_CMAP_RANGE, 9785 },
	{ 0xae8c, 0xae8c, PDF_CMAP_SINGLE, 1280 },
	{ 0xae8d, 0xaebb, PDF_CMAP_RANGE, 9791 },
	{ 0xaebc, 0xaebe, PDF_CMAP_RANGE, 1281 },
	{ 0xaebf, 0xaec0, PDF_CMAP_TABLE, 2439 },
	{ 0xaec1, 0xaec3, PDF_CMAP_RANGE, 9839 },
	{ 0xaec4, 0xaec4, PDF_CMAP_SINGLE, 1285 },
	{ 0xaec5, 0xaecb, PDF_CMAP_RANGE, 9842 },
	{ 0xaecc, 0xaecd, PDF_CMAP_RANGE, 1286 },
	{ 0xaece, 0xaece, PDF_CMAP_SINGLE, 9849 },
	{ 0xaecf, 0xaed1, PDF_CMAP_RANGE, 1288 },
	{ 0xaed2, 0xaed7, PDF_CMAP_RANGE, 9850 },
	{ 0xaed8, 0xaed9, PDF_CMAP_RANGE, 1291 },
	{ 0xaeda, 0xaedb, PDF_CMAP_RANGE, 9856 },
	{ 0xaedc, 0xaedc, PDF_CMAP_SINGLE, 1293 },
	{ 0xaedd, 0xaee7, PDF_CMAP_RANGE, 9858 },
	{ 0xaee8, 0xaee8, PDF_CMAP_SINGLE, 1294 },
	{ 0xaee9, 0xaeea, PDF_CMAP_RANGE, 9869 },
	{ 0xaeeb, 0xaeed, PDF_CMAP_TABLE, 2441 },
	{ 0xaeee, 0xaef3, PDF_CMAP_RANGE, 9872 },
	{ 0xaef4, 0xaef4, PDF_CMAP_SINGLE, 1297 },
	{ 0xaef5, 0xaef7, PDF_CMAP_RANGE, 9878 },
	{ 0xaef8, 0xaef8, PDF_CMAP_SINGLE, 1298 },
	{ 0xaef9, 0xaefb, PDF_CMAP_RANGE, 9881 },
	{ 0xaefc, 0xaefc, PDF_CMAP_SINGLE, 1299 },
	{ 0xaefd, 0xaf06, PDF_CMAP_RANGE, 9884 },
	{ 0xaf07, 0xaf08, PDF_CMAP_RANGE, 1300 },
	{ 0xaf09, 0xaf0c, PDF_CMAP_RANGE, 9894 },
	{ 0xaf0d, 0xaf0d, PDF_CMAP_SINGLE, 1302 },
	{ 0xaf0e, 0xaf0f, PDF_CMAP_RANGE, 9898 },
	{ 0xaf10, 0xaf10, PDF_CMAP_SINGLE, 1303 },
	{ 0xaf11, 0xaf2b, PDF_CMAP_RANGE, 9900 },
	{ 0xaf2c, 0xaf2d, PDF_CMAP_RANGE, 1304 },
	{ 0xaf2e, 0xaf2f, PDF_CMAP_RANGE, 9927 },
	{ 0xaf30, 0xaf34, PDF_CMAP_TABLE, 2444 },
	{ 0xaf35, 0xaf3b, PDF_CMAP_RANGE, 9931 },
	{ 0xaf3c, 0xaf3d, PDF_CMAP_RANGE, 1309 },
	{ 0xaf3e, 0xaf40, PDF_CMAP_TABLE, 2449 },
	{ 0xaf41, 0xaf43, PDF_CMAP_RANGE, 1312 },
	{ 0xaf44, 0xaf47, PDF_CMAP_RANGE, 9940 },
	{ 0xaf48, 0xaf49, PDF_CMAP_RANGE, 1315 },
	{ 0xaf4a, 0xaf4f, PDF_CMAP_RANGE, 9944 },
	{ 0xaf50, 0xaf50, PDF_CMAP_SINGLE, 1317 },
	{ 0xaf51, 0xaf5b, PDF_CMAP_RANGE, 9950 },
	{ 0xaf5c, 0xaf5d, PDF_CMAP_RANGE, 1318 },
	{ 0xaf5e, 0xaf63, PDF_CMAP_RANGE, 9961 },
	{ 0xaf64, 0xaf65, PDF_CMAP_RANGE, 1320 },
	{ 0xaf66, 0xaf78, PDF_CMAP_RANGE, 9967 },
	{ 0xaf79, 0xaf79, PDF_CMAP_SINGLE, 1322 },
	{ 0xaf7a, 0xaf7f, PDF_CMAP_RANGE, 9986 },
	{ 0xaf80, 0xaf80, PDF_CMAP_SINGLE, 1323 },
	{ 0xaf81, 0xaf83, PDF_CMAP_RANGE, 9992 },
	{ 0xaf84, 0xaf84, PDF_CMAP_SINGLE, 1324 },
	{ 0xaf85, 0xaf87, PDF_CMAP_RANGE, 9995 },
	{ 0xaf88, 0xaf88, PDF_CMAP_SINGLE, 1325 },
	{ 0xaf89, 0xaf8f, PDF_CMAP_RANGE, 9998 },
	{ 0xaf90, 0xaf91, PDF_CMAP_RANGE, 1326 },
	{ 0xaf92, 0xaf94, PDF_CMAP_RANGE, 10005 },
	{ 0xaf95, 0xaf95, PDF_CMAP_SINGLE, 1328 },
	{ 0xaf96, 0xaf9b, PDF_CMAP_RANGE, 10008 },
	{ 0xaf9c, 0xaf9c, PDF_CMAP_SINGLE, 1329 },
	{ 0xaf9d, 0xafb7, PDF_CMAP_RANGE, 10014 },
	{ 0xafb8, 0xafb9, PDF_CMAP_RANGE, 1330 },
	{ 0xafba, 0xafbb, PDF_CMAP_RANGE, 10041 },
	{ 0xafbc, 0xafbc, PDF_CMAP_SINGLE, 1332 },
	{ 0xafbd, 0xafbf, PDF_CMAP_RANGE, 10043 },
	{ 0xafc0, 0xafc0, PDF_CMAP_SINGLE, 1333 },
	{ 0xafc1, 0xafc6, PDF_CMAP_RANGE, 10046 },
	{ 0xafc7, 0xafc9, PDF_CMAP_RANGE, 1334 },
	{ 0xafca, 0xafcc, PDF_CMAP_TABLE, 2452 },
	{ 0xafcd, 0xafce, PDF_CMAP_RANGE, 1338 },
	{ 0xafcf, 0xafd3, PDF_CMAP_RANGE, 10054 },
	{ 0xafd4, 0xafd4, PDF_CMAP_SINGLE, 1340 },
	{ 0xafd5, 0xafdb, PDF_CMAP_RANGE, 10059 },
	{ 0xafdc, 0xafdc, PDF_CMAP_SINGLE, 1341 },
	{ 0xafdd, 0xafe7, PDF_CMAP_RANGE, 10066 },
	{ 0xafe8, 0xafe9, PDF_CMAP_RANGE, 1342 },
	{ 0xafea, 0xafef, PDF_CMAP_RANGE, 10077 },
	{ 0xaff0, 0xaff1, PDF_CMAP_RANGE, 1344 },
	{ 0xaff2, 0xaff3, PDF_CMAP_RANGE, 10083 },
	{ 0xaff4, 0xaff4, PDF_CMAP_SINGLE, 1346 },
	{ 0xaff5, 0xaff7, PDF_CMAP_RANGE, 10085 },
	{ 0xaff8, 0xaff8, PDF_CMAP_SINGLE, 1347 },
	{ 0xaff9, 0xafff, PDF_CMAP_RANGE, 10088 },
	{ 0xb000, 0xb001, PDF_CMAP_RANGE, 1348 },
	{ 0xb002, 0xb003, PDF_CMAP_RANGE, 10095 },
	{ 0xb004, 0xb004, PDF_CMAP_SINGLE, 1350 },
	{ 0xb005, 0xb00b, PDF_CMAP_RANGE, 10097 },
	{ 0xb00c, 0xb00c, PDF_CMAP_SINGLE, 1351 },
	{ 0xb00d, 0xb00f, PDF_CMAP_RANGE, 10104 },
	{ 0xb010, 0xb010, PDF_CMAP_SINGLE, 1352 },
	{ 0xb011, 0xb013, PDF_CMAP_RANGE, 10107 },
	{ 0xb014, 0xb014, PDF_CMAP_SINGLE, 1353 },
	{ 0xb015, 0xb01b, PDF_CMAP_RANGE, 10110 },
	{ 0xb01c, 0xb01d, PDF_CMAP_RANGE, 1354 },
	{ 0xb01e, 0xb027, PDF_CMAP_RANGE, 10117 },
	{ 0xb028, 0xb028, PDF_CMAP_SINGLE, 1356 },
	{ 0xb029, 0xb043, PDF_CMAP_RANGE, 10127 },
	{ 0xb044, 0xb045, PDF_CMAP_RANGE, 1357 },
	{ 0xb046, 0xb047, PDF_CMAP_RANGE, 10154 },
	{ 0xb048, 0xb04e, PDF_CMAP_TABLE, 2455 },
	{ 0xb04f, 0xb052, PDF_CMAP_RANGE, 10159 },
	{ 0xb053, 0xb055, PDF_CMAP_RANGE, 1363 },
	{ 0xb056, 0xb059, PDF_CMAP_TABLE, 2462 },
	{ 0xb05a, 0xb05c, PDF_CMAP_RANGE, 10165 },
	{ 0xb05d, 0xb05d, PDF_CMAP_SINGLE, 1368 },
	{ 0xb05e, 0xb07b, PDF_CMAP_RANGE, 10168 },
	{ 0xb07c, 0xb07d, PDF_CMAP_RANGE, 1369 },
	{ 0xb07e, 0xb07f, PDF_CMAP_RANGE, 10198 },
	{ 0xb080, 0xb080, PDF_CMAP_SINGLE, 1371 },
	{ 0xb081, 0xb083, PDF_CMAP_RANGE, 10200 },
	{ 0xb084, 0xb084, PDF_CMAP_SINGLE, 1372 },
	{ 0xb085, 0xb08b, PDF_CMAP_RANGE, 10203 },
	{ 0xb08c, 0xb08d, PDF_CMAP_RANGE, 1373 },
	{ 0xb08e, 0xb091, PDF_CMAP_TABLE, 2466 },
	{ 0xb092, 0xb097, PDF_CMAP_RANGE, 10212 },
	{ 0xb098, 0xb09a, PDF_CMAP_RANGE, 1377 },
	{ 0xb09b, 0xb09c, PDF_CMAP_TABLE, 2470 },
	{ 0xb09d, 0xb09e, PDF_CMAP_RANGE, 10219 },
	{ 0xb09f, 0xb0a2, PDF_CMAP_RANGE, 1381 },
	{ 0xb0a3, 0xb0a7, PDF_CMAP_RANGE, 10221 },
	{ 0xb0a8, 0xb0a9, PDF_CMAP_RANGE, 1385 },
	{ 0xb0aa, 0xb0aa, PDF_CMAP_SINGLE, 10226 },
	{ 0xb0ab, 0xb0af, PDF_CMAP_RANGE, 1387 },
	{ 0xb0b0, 0xb0b2, PDF_CMAP_TABLE, 2472 },
	{ 0xb0b3, 0xb0b5, PDF_CMAP_RANGE, 1393 },
	{ 0xb0b6, 0xb0b7, PDF_CMAP_RANGE, 10229 },
	{ 0xb0b8, 0xb0b8, PDF_CMAP_SINGLE, 1396 },
	{ 0xb0b9, 0xb0bb, PDF_CMAP_RANGE, 10231 },
	{ 0xb0bc, 0xb0bc, PDF_CMAP_SINGLE, 1397 },
	{ 0xb0bd, 0xb0c3, PDF_CMAP_RANGE, 10234 },
	{ 0xb0c4, 0xb0c5, PDF_CMAP_RANGE, 1398 },
	{ 0xb0c6, 0xb0c6, PDF_CMAP_SINGLE, 10241 },
	{ 0xb0c7, 0xb0c9, PDF_CMAP_RANGE, 1400 },
	{ 0xb0ca, 0xb0cf, PDF_CMAP_RANGE, 10242 },
	{ 0xb0d0, 0xb0d1, PDF_CMAP_RANGE, 1403 },
	{ 0xb0d2, 0xb0d3, PDF_CMAP_RANGE, 10248 },
	{ 0xb0d4, 0xb0d4, PDF_CMAP_SINGLE, 1405 },
	{ 0xb0d5, 0xb0d7, PDF_CMAP_RANGE, 10250 },
	{ 0xb0d8, 0xb0d8, PDF_CMAP_SINGLE, 1406 },
	{ 0xb0d9, 0xb0df, PDF_CMAP_RANGE, 10253 },
	{ 0xb0e0, 0xb0e0, PDF_CMAP_SINGLE, 1407 },
	{ 0xb0e1, 0xb0e4, PDF_CMAP_RANGE, 10260 },
	{ 0xb0e5, 0xb0e5, PDF_CMAP_SINGLE, 1408 },
	{ 0xb0e6, 0xb107, PDF_CMAP_RANGE, 10264 },
	{ 0xb108, 0xb109, PDF_CMAP_RANGE, 1409 },
	{ 0xb10a, 0xb10a, PDF_CMAP_SINGLE, 10298 },
	{ 0xb10b, 0xb10c, PDF_CMAP_RANGE, 1411 },
	{ 0xb10d, 0xb10f, PDF_CMAP_RANGE, 10299 },
	{ 0xb110, 0xb111, PDF_CMAP_TABLE, 2475 },
	{ 0xb112, 0xb113, PDF_CMAP_RANGE, 1414 },
	{ 0xb114, 0xb117, PDF_CMAP_RANGE, 10303 },
	{ 0xb118, 0xb119, PDF_CMAP_RANGE, 1416 },
	{ 0xb11a, 0xb11a, PDF_CMAP_SINGLE, 10307 },
	{ 0xb11b, 0xb11d, PDF_CMAP_RANGE, 1418 },
	{ 0xb11e, 0xb122, PDF_CMAP_RANGE, 10308 },
	{ 0xb123, 0xb125, PDF_CMAP_RANGE, 1421 },
	{ 0xb126, 0xb127, PDF_CMAP_RANGE, 10313 },
	{ 0xb128, 0xb128, PDF_CMAP_SINGLE, 1424 },
	{ 0xb129, 0xb12b, PDF_CMAP_RANGE, 10315 },
	{ 0xb12c, 0xb12c, PDF_CMAP_SINGLE, 1425 },
	{ 0xb12d, 0xb133, PDF_CMAP_RANGE, 10318 },
	{ 0xb134, 0xb135, PDF_CMAP_RANGE, 1426 },
	{ 0xb136, 0xb136, PDF_CMAP_SINGLE, 10325 },
	{ 0xb137, 0xb139, PDF_CMAP_RANGE, 1428 },
	{ 0xb13a, 0xb13f, PDF_CMAP_RANGE, 10326 },
	{ 0xb140, 0xb141, PDF_CMAP_RANGE, 1431 },
	{ 0xb142, 0xb143, PDF_CMAP_RANGE, 10332 },
	{ 0xb144, 0xb144, PDF_CMAP_SINGLE, 1433 },
	{ 0xb145, 0xb147, PDF_CMAP_RANGE, 10334 },
	{ 0xb148, 0xb148, PDF_CMAP_SINGLE, 1434 },
	{ 0xb149, 0xb14f, PDF_CMAP_RANGE, 10337 },
	{ 0xb150, 0xb151, PDF_CMAP_RANGE, 1435 },
	{ 0xb152, 0xb153, PDF_CMAP_RANGE, 10344 },
	{ 0xb154, 0xb155, PDF_CMAP_RANGE, 1437 },
	{ 0xb156, 0xb157, PDF_CMAP_RANGE, 10346 },
	{ 0xb158, 0xb158, PDF_CMAP_SINGLE, 1439 },
	{ 0xb159, 0xb15b, PDF_CMAP_RANGE, 10348 },
	{ 0xb15c, 0xb15c, PDF_CMAP_SINGLE, 1440 },
	{ 0xb15d, 0xb15f, PDF_CMAP_RANGE, 10351 },
	{ 0xb160, 0xb160, PDF_CMAP_SINGLE, 1441 },
	{ 0xb161, 0xb177, PDF_CMAP_RANGE, 10354 },
	{ 0xb178, 0xb179, PDF_CMAP_RANGE, 1442 },
	{ 0xb17a, 0xb17b, PDF_CMAP_RANGE, 10377 },
	{ 0xb17c, 0xb17c, PDF_CMAP_SINGLE, 1444 },
	{ 0xb17d, 0xb17f, PDF_CMAP_RANGE, 10379 },
	{ 0xb180, 0xb182, PDF_CMAP_TABLE, 2477 },
	{ 0xb183, 0xb187, PDF_CMAP_RANGE, 10383 },
	{ 0xb188, 0xb189, PDF_CMAP_RANGE, 1447 },
	{ 0xb18a, 0xb18d, PDF_CMAP_TABLE, 2480 },
	{ 0xb18e, 0xb191, PDF_CMAP_RANGE, 10390 },
	{ 0xb192, 0xb194, PDF_CMAP_RANGE, 1451 },
	{ 0xb195, 0xb197, PDF_CMAP_RANGE, 10394 },
	{ 0xb198, 0xb198, PDF_CMAP_SINGLE, 1454 },
	{ 0xb199, 0xb19b, PDF_CMAP_RANGE, 10397 },
	{ 0xb19c, 0xb19c, PDF_CMAP_SINGLE, 1455 },
	{ 0xb19d, 0xb1a7, PDF_CMAP_RANGE, 10400 },
	{ 0xb1a8, 0xb1a8, PDF_CMAP_SINGLE, 1456 },
	{ 0xb1a9, 0xb1cb, PDF_CMAP_RANGE, 10411 },
	{ 0xb1cc, 0xb1cc, PDF_CMAP_SINGLE, 1457 },
	{ 0xb1cd, 0xb1cf, PDF_CMAP_RANGE, 10446 },
	{ 0xb1d0, 0xb1d0, PDF_CMAP_SINGLE, 1458 },
	{ 0xb1d1, 0xb1d3, PDF_CMAP_RANGE, 10449 },
	{ 0xb1d4, 0xb1d4, PDF_CMAP_SINGLE, 1459 },
	{ 0xb1d5, 0xb1db, PDF_CMAP_RANGE, 10452 },
	{ 0xb1dc, 0xb1dd, PDF_CMAP_RANGE, 1460 },
	{ 0xb1de, 0xb1df, PDF_CMAP_TABLE, 2484 },
	{ 0xb1e0, 0xb1e7, PDF_CMAP_RANGE, 10460 },
	{ 0xb1e8, 0xb1e9, PDF_CMAP_RANGE, 1463 },
	{ 0xb1ea, 0xb1eb, PDF_CMAP_RANGE, 10468 },
	{ 0xb1ec, 0xb1ec, PDF_CMAP_SINGLE, 1465 },
	{ 0xb1ed, 0xb1ef, PDF_CMAP_RANGE, 10470 },
	{ 0xb1f0, 0xb1f0, PDF_CMAP_SINGLE, 1466 },
	{ 0xb1f1, 0xb1f8, PDF_CMAP_RANGE, 10473 },
	{ 0xb1f9, 0xb1fd, PDF_CMAP_TABLE, 2486 },
	{ 0xb1fe, 0xb203, PDF_CMAP_RANGE, 10483 },
	{ 0xb204, 0xb205, PDF_CMAP_RANGE, 1470 },
	{ 0xb206, 0xb207, PDF_CMAP_RANGE, 10489 },
	{ 0xb208, 0xb208, PDF_CMAP_SINGLE, 1472 },
	{ 0xb209, 0xb20a, PDF_CMAP_RANGE, 10491 },
	{ 0xb20b, 0xb20c, PDF_CMAP_RANGE, 1473 },
	{ 0xb20d, 0xb213, PDF_CMAP_RANGE, 10493 },
	{ 0xb214, 0xb215, PDF_CMAP_RANGE, 1475 },
	{ 0xb216, 0xb219, PDF_CMAP_TABLE, 2491 },
	{ 0xb21a, 0xb21f, PDF_CMAP_RANGE, 10502 },
	{ 0xb220, 0xb220, PDF_CMAP_SINGLE, 1479 },
	{ 0xb221, 0xb233, PDF_CMAP_RANGE, 10508 },
	{ 0xb234, 0xb234, PDF_CMAP_SINGLE, 1480 },
	{ 0xb235, 0xb23b, PDF_CMAP_RANGE, 10527 },
	{ 0xb23c, 0xb23c, PDF_CMAP_SINGLE, 1481 },
	{ 0xb23d, 0xb257, PDF_CMAP_RANGE, 10534 },
	{ 0xb258, 0xb258, PDF_CMAP_SINGLE, 1482 },
	{ 0xb259, 0xb25b, PDF_CMAP_RANGE, 10561 },
	{ 0xb25c, 0xb25c, PDF_CMAP_SINGLE, 1483 },
	{ 0xb25d, 0xb25f, PDF_CMAP_RANGE, 10564 },
	{ 0xb260, 0xb260, PDF_CMAP_SINGLE, 1484 },
	{ 0xb261, 0xb267, PDF_CMAP_RANGE, 10567 },
	{ 0xb268, 0xb269, PDF_CMAP_RANGE, 1485 },
	{ 0xb26a, 0xb273, PDF_CMAP_RANGE, 10574 },
	{ 0xb274, 0xb275, PDF_CMAP_RANGE, 1487 },
	{ 0xb276, 0xb27b, PDF_CMAP_RANGE, 10584 },
	{ 0xb27c, 0xb27c, PDF_CMAP_SINGLE, 1489 },
	{ 0xb27d, 0xb283, PDF_CMAP_RANGE, 10590 },
	{ 0xb284, 0xb285, PDF_CMAP_RANGE, 1490 },
	{ 0xb286, 0xb288, PDF_CMAP_RANGE, 10597 },
	{ 0xb289, 0xb289, PDF_CMAP_SINGLE, 1492 },
	{ 0xb28a, 0xb28f, PDF_CMAP_RANGE, 10600 },
	{ 0xb290, 0xb291, PDF_CMAP_RANGE, 1493 },
	{ 0xb292, 0xb293, PDF_CMAP_RANGE, 10606 },
	{ 0xb294, 0xb294, PDF_CMAP_SINGLE, 1495 },
	{ 0xb295, 0xb297, PDF_CMAP_RANGE, 10608 },
	{ 0xb298, 0xb29a, PDF_CMAP_RANGE, 1496 },
	{ 0xb29b, 0xb29f, PDF_CMAP_RANGE, 10611 },
	{ 0xb2a0, 0xb2a1, PDF_CMAP_RANGE, 1499 },
	{ 0xb2a2, 0xb2a4, PDF_CMAP_TABLE, 2495 },
	{ 0xb2a5, 0xb2a6, PDF_CMAP_RANGE, 1502 },
	{ 0xb2a7, 0xb2a9, PDF_CMAP_RANGE, 10618 },
	{ 0xb2aa, 0xb2ac, PDF_CMAP_TABLE, 2498 },
	{ 0xb2ad, 0xb2af, PDF_CMAP_RANGE, 10622 },
	{ 0xb2b0, 0xb2b0, PDF_CMAP_SINGLE, 1506 },
	{ 0xb2b1, 0xb2b3, PDF_CMAP_RANGE, 10625 },
	{ 0xb2b4, 0xb2b4, PDF_CMAP_SINGLE, 1507 },
	{ 0xb2b5, 0xb2c7, PDF_CMAP_RANGE, 10628 },
	{ 0xb2c8, 0xb2c9, PDF_CMAP_RANGE, 1508 },
	{ 0xb2ca, 0xb2cb, PDF_CMAP_RANGE, 10647 },
	{ 0xb2cc, 0xb2cc, PDF_CMAP_SINGLE, 1510 },
	{ 0xb2cd, 0xb2cf, PDF_CMAP_RANGE, 10649 },
	{ 0xb2d0, 0xb2d2, PDF_CMAP_TABLE, 2501 },
	{ 0xb2d3, 0xb2d7, PDF_CMAP_RANGE, 10653 },
	{ 0xb2d8, 0xb2d9, PDF_CMAP_RANGE, 1513 },
	{ 0xb2da, 0xb2dd, PDF_CMAP_TABLE, 2504 },
	{ 0xb2de, 0xb2e1, PDF_CMAP_RANGE, 10660 },
	{ 0xb2e2, 0xb2e3, PDF_CMAP_TABLE, 2508 },
	{ 0xb2e4, 0xb2e6, PDF_CMAP_RANGE, 1518 },
	{ 0xb2e7, 0xb2e8, PDF_CMAP_TABLE, 2510 },
	{ 0xb2e9, 0xb2ea, PDF_CMAP_RANGE, 10666 },
	{ 0xb2eb, 0xb2ef, PDF_CMAP_RANGE, 1522 },
	{ 0xb2f0, 0xb2f2, PDF_CMAP_RANGE, 10668 },
	{ 0xb2f3, 0xb2f5, PDF_CMAP_RANGE, 1527 },
	{ 0xb2f6, 0xb2f6, PDF_CMAP_SINGLE, 10671 },
	{ 0xb2f7, 0xb2fb, PDF_CMAP_RANGE, 1530 },
	{ 0xb2fc, 0xb2fe, PDF_CMAP_RANGE, 10672 },
	{ 0xb2ff, 0xb301, PDF_CMAP_RANGE, 1535 },
	{ 0xb302, 0xb303, PDF_CMAP_RANGE, 10675 },
	{ 0xb304, 0xb304, PDF_CMAP_SINGLE, 1538 },
	{ 0xb305, 0xb307, PDF_CMAP_RANGE, 10677 },
	{ 0xb308, 0xb308, PDF_CMAP_SINGLE, 1539 },
	{ 0xb309, 0xb30f, PDF_CMAP_RANGE, 10680 },
	{ 0xb310, 0xb311, PDF_CMAP_RANGE, 1540 },
	{ 0xb312, 0xb312, PDF_CMAP_SINGLE, 10687 },
	{ 0xb313, 0xb315, PDF_CMAP_RANGE, 1542 },
	{ 0xb316, 0xb31b, PDF_CMAP_RANGE, 10688 },
	{ 0xb31c, 0xb31c, PDF_CMAP_SINGLE, 1545 },
	{ 0xb31d, 0xb353, PDF_CMAP_RANGE, 10694 },
	{ 0xb354, 0xb356, PDF_CMAP_RANGE, 1546 },
	{ 0xb357, 0xb358, PDF_CMAP_TABLE, 2512 },
	{ 0xb359, 0xb35a, PDF_CMAP_RANGE, 10750 },
	{ 0xb35b, 0xb35c, PDF_CMAP_RANGE, 1550 },
	{ 0xb35d, 0xb35d, PDF_CMAP_SINGLE, 10752 },
	{ 0xb35e, 0xb35f, PDF_CMAP_RANGE, 1552 },
	{ 0xb360, 0xb363, PDF_CMAP_RANGE, 10753 },
	{ 0xb364, 0xb365, PDF_CMAP_RANGE, 1554 },
	{ 0xb366, 0xb36b, PDF_CMAP_TABLE, 2514 },
	{ 0xb36c, 0xb36d, PDF_CMAP_RANGE, 10760 },
	{ 0xb36e, 0xb36f, PDF_CMAP_TABLE, 2520 },
	{ 0xb370, 0xb371, PDF_CMAP_RANGE, 1560 },
	{ 0xb372, 0xb373, PDF_CMAP_RANGE, 10763 },
	{ 0xb374, 0xb374, PDF_CMAP_SINGLE, 1562 },
	{ 0xb375, 0xb377, PDF_CMAP_RANGE, 10765 },
	{ 0xb378, 0xb378, PDF_CMAP_SINGLE, 1563 },
	{ 0xb379, 0xb37f, PDF_CMAP_RANGE, 10768 },
	{ 0xb380, 0xb381, PDF_CMAP_RANGE, 1564 },
	{ 0xb382, 0xb382, PDF_CMAP_SINGLE, 10775 },
	{ 0xb383, 0xb385, PDF_CMAP_RANGE, 1566 },
	{ 0xb386, 0xb38b, PDF_CMAP_RANGE, 10776 },
	{ 0xb38c, 0xb38c, PDF_CMAP_SINGLE, 1569 },
	{ 0xb38d, 0xb38f, PDF_CMAP_RANGE, 10782 },
	{ 0xb390, 0xb390, PDF_CMAP_SINGLE, 1570 },
	{ 0xb391, 0xb393, PDF_CMAP_RANGE, 10785 },
	{ 0xb394, 0xb394, PDF_CMAP_SINGLE, 1571 },
	{ 0xb395, 0xb39f, PDF_CMAP_RANGE, 10788 },
	{ 0xb3a0, 0xb3a1, PDF_CMAP_RANGE, 1572 },
	{ 0xb3a2, 0xb3a7, PDF_CMAP_RANGE, 10799 },
	{ 0xb3a8, 0xb3a8, PDF_CMAP_SINGLE, 1574 },
	{ 0xb3a9, 0xb3ab, PDF_CMAP_RANGE, 10805 },
	{ 0xb3ac, 0xb3ac, PDF_CMAP_SINGLE, 1575 },
	{ 0xb3ad, 0xb3c3, PDF_CMAP_RANGE, 10808 },
	{ 0xb3c4, 0xb3c5, PDF_CMAP_RANGE, 1576 },
	{ 0xb3c6, 0xb3c7, PDF_CMAP_RANGE, 10831 },
	{ 0xb3c8, 0xb3c8, PDF_CMAP_SINGLE, 1578 },
	{ 0xb3c9, 0xb3ca, PDF_CMAP_RANGE, 10833 },
	{ 0xb3cb, 0xb3cc, PDF_CMAP_RANGE, 1579 },
	{ 0xb3cd, 0xb3d0, PDF_CMAP_TABLE, 2522 },
	{ 0xb3d1, 0xb3d3, PDF_CMAP_RANGE, 10837 },
	{ 0xb3d4, 0xb3d5, PDF_CMAP_RANGE, 1583 },
	{ 0xb3d6, 0xb3dd, PDF_CMAP_TABLE, 2526 },
	{ 0xb3de, 0xb3df, PDF_CMAP_RANGE, 10844 },
	{ 0xb3e0, 0xb3e0, PDF_CMAP_SINGLE, 1589 },
	{ 0xb3e1, 0xb3e3, PDF_CMAP_RANGE, 10846 },
	{ 0xb3e4, 0xb3e4, PDF_CMAP_SINGLE, 1590 },
	{ 0xb3e5, 0xb3e7, PDF_CMAP_RANGE, 10849 },
	{ 0xb3e8, 0xb3e8, PDF_CMAP_SINGLE, 1591 },
	{ 0xb3e9, 0xb3fb, PDF_CMAP_RANGE, 10852 },
	{ 0xb3fc, 0xb3fc, PDF_CMAP_SINGLE, 1592 },
	{ 0xb3fd, 0xb40f, PDF_CMAP_RANGE, 10871 },
	{ 0xb410, 0xb410, PDF_CMAP_SINGLE, 1593 },
	{ 0xb411, 0xb417, PDF_CMAP_RANGE, 10890 },
	{ 0xb418, 0xb418, PDF_CMAP_SINGLE, 1594 },
	{ 0xb419, 0xb41b, PDF_CMAP_RANGE, 10897 },
	{ 0xb41c, 0xb41c, PDF_CMAP_SINGLE, 1595 },
	{ 0xb41d, 0xb41f, PDF_CMAP_RANGE, 10900 },
	{ 0xb420, 0xb420, PDF_CMAP_SINGLE, 1596 },
	{ 0xb421, 0xb427, PDF_CMAP_RANGE, 10903 },
	{ 0xb428, 0xb429, PDF_CMAP_RANGE, 1597 },
	{ 0xb42a, 0xb42b, PDF_CMAP_TABLE, 2534 },
	{ 0xb42c, 0xb433, PDF_CMAP_RANGE, 10911 },
	{ 0xb434, 0xb434, PDF_CMAP_SINGLE, 1600 },
	{ 0xb435, 0xb44f, PDF_CMAP_RANGE, 10919 },
	{ 0xb450, 0xb451, PDF_CMAP_RANGE, 1601 },
	{ 0xb452, 0xb453, PDF_CMAP_RANGE, 10946 },
	{ 0xb454, 0xb454, PDF_CMAP_SINGLE, 1603 },
	{ 0xb455, 0xb457, PDF_CMAP_RANGE, 10948 },
	{ 0xb458, 0xb458, PDF_CMAP_SINGLE, 1604 },
	{ 0xb459, 0xb45f, PDF_CMAP_RANGE, 10951 },
	{ 0xb460, 0xb461, PDF_CMAP_RANGE, 1605 },
	{ 0xb462, 0xb465, PDF_CMAP_TABLE, 2536 },
	{ 0xb466, 0xb46b, PDF_CMAP_RANGE, 10960 },
	{ 0xb46c, 0xb46c, PDF_CMAP_SINGLE, 1609 },
	{ 0xb46d, 0xb47f, PDF_CMAP_RANGE, 10966 },
	{ 0xb480, 0xb480, PDF_CMAP_SINGLE, 1610 },
	{ 0xb481, 0xb487, PDF_CMAP_RANGE, 10985 },
	{ 0xb488, 0xb488, PDF_CMAP_SINGLE, 1611 },
	{ 0xb489, 0xb49c, PDF_CMAP_RANGE, 10992 },
	{ 0xb49d, 0xb49d, PDF_CMAP_SINGLE, 1612 },
	{ 0xb49e, 0xb4a3, PDF_CMAP_RANGE, 11012 },
	{ 0xb4a4, 0xb4a4, PDF_CMAP_SINGLE, 1613 },
	{ 0xb4a5, 0xb4a7, PDF_CMAP_RANGE, 11018 },
	{ 0xb4a8, 0xb4a8, PDF_CMAP_SINGLE, 1614 },
	{ 0xb4a9, 0xb4ab, PDF_CMAP_RANGE, 11021 },
	{ 0xb4ac, 0xb4ac, PDF_CMAP_SINGLE, 1615 },
	{ 0xb4ad, 0xb4b4, PDF_CMAP_RANGE, 11024 },
	{ 0xb4b5, 0xb4b9, PDF_CMAP_TABLE, 2540 },
	{ 0xb4ba, 0xb4bf, PDF_CMAP_RANGE, 11034 },
	{ 0xb4c0, 0xb4c0, PDF_CMAP_SINGLE, 1619 },
	{ 0xb4c1, 0xb4c3, PDF_CMAP_RANGE, 11040 },
	{ 0xb4c4, 0xb4c4, PDF_CMAP_SINGLE, 1620 },
	{ 0xb4c5, 0xb4c7, PDF_CMAP_RANGE, 11043 },
	{ 0xb4c8, 0xb4c8, PDF_CMAP_SINGLE, 1621 },
	{ 0xb4c9, 0xb4cf, PDF_CMAP_RANGE, 11046 },
	{ 0xb4d0, 0xb4d0, PDF_CMAP_SINGLE, 1622 },
	{ 0xb4d1, 0xb4d4, PDF_CMAP_RANGE, 11053 },
	{ 0xb4d5, 0xb4d5, PDF_CMAP_SINGLE, 1623 },
	{ 0xb4d6, 0xb4db, PDF_CMAP_RANGE, 11057 },
	{ 0xb4dc, 0xb4dd, PDF_CMAP_RANGE, 1624 },
	{ 0xb4de, 0xb4df, PDF_CMAP_RANGE, 11063 },
	{ 0xb4e0, 0xb4e0, PDF_CMAP_SINGLE, 1626 },
	{ 0xb4e1, 0xb4e2, PDF_CMAP_RANGE, 11065 },
	{ 0xb4e3, 0xb4e4, PDF_CMAP_RANGE, 1627 },
	{ 0xb4e5, 0xb4e6, PDF_CMAP_TABLE, 2545 },
	{ 0xb4e7, 0xb4eb, PDF_CMAP_RANGE, 11068 },
	{ 0xb4ec, 0xb4ed, PDF_CMAP_RANGE, 1630 },
	{ 0xb4ee, 0xb4f1, PDF_CMAP_TABLE, 2547 },
	{ 0xb4f2, 0xb4f7, PDF_CMAP_RANGE, 11075 },
	{ 0xb4f8, 0xb4f8, PDF_CMAP_SINGLE, 1634 },
	{ 0xb4f9, 0xb513, PDF_CMAP_RANGE, 11081 },
	{ 0xb514, 0xb515, PDF_CMAP_RANGE, 1635 },
	{ 0xb516, 0xb517, PDF_CMAP_RANGE, 11108 },
	{ 0xb518, 0xb518, PDF_CMAP_SINGLE, 1637 },
	{ 0xb519, 0xb51a, PDF_CMAP_RANGE, 11110 },
	{ 0xb51b, 0xb51c, PDF_CMAP_RANGE, 1638 },
	{ 0xb51d, 0xb523, PDF_CMAP_RANGE, 11112 },
	{ 0xb524, 0xb525, PDF_CMAP_RANGE, 1640 },
	{ 0xb526, 0xb526, PDF_CMAP_SINGLE, 11119 },
	{ 0xb527, 0xb52a, PDF_CMAP_RANGE, 1642 },
	{ 0xb52b, 0xb52f, PDF_CMAP_RANGE, 11120 },
	{ 0xb530, 0xb531, PDF_CMAP_RANGE, 1646 },
	{ 0xb532, 0xb533, PDF_CMAP_RANGE, 11125 },
	{ 0xb534, 0xb534, PDF_CMAP_SINGLE, 1648 },
	{ 0xb535, 0xb537, PDF_CMAP_RANGE, 11127 },
	{ 0xb538, 0xb538, PDF_CMAP_SINGLE, 1649 },
	{ 0xb539, 0xb53f, PDF_CMAP_RANGE, 11130 },
	{ 0xb540, 0xb541, PDF_CMAP_RANGE, 1650 },
	{ 0xb542, 0xb542, PDF_CMAP_SINGLE, 11137 },
	{ 0xb543, 0xb545, PDF_CMAP_RANGE, 1652 },
	{ 0xb546, 0xb54a, PDF_CMAP_RANGE, 11138 },
	{ 0xb54b, 0xb54d, PDF_CMAP_RANGE, 1655 },
	{ 0xb54e, 0xb54f, PDF_CMAP_RANGE, 11143 },
	{ 0xb550, 0xb550, PDF_CMAP_SINGLE, 1658 },
	{ 0xb551, 0xb553, PDF_CMAP_RANGE, 11145 },
	{ 0xb554, 0xb554, PDF_CMAP_SINGLE, 1659 },
	{ 0xb555, 0xb55b, PDF_CMAP_RANGE, 11148 },
	{ 0xb55c, 0xb55d, PDF_CMAP_RANGE, 1660 },
	{ 0xb55e, 0xb55e, PDF_CMAP_SINGLE, 11155 },
	{ 0xb55f, 0xb561, PDF_CMAP_RANGE, 1662 },
	{ 0xb562, 0xb59f, PDF_CMAP_RANGE, 11156 },
	{ 0xb5a0, 0xb5a1, PDF_CMAP_RANGE, 1665 },
	{ 0xb5a2, 0xb5a3, PDF_CMAP_RANGE, 11218 },
	{ 0xb5a4, 0xb5a4, PDF_CMAP_SINGLE, 1667 },
	{ 0xb5a5, 0xb5a7, PDF_CMAP_RANGE, 11220 },
	{ 0xb5a8, 0xb5a9, PDF_CMAP_TABLE, 2551 },
	{ 0xb5aa, 0xb5ab, PDF_CMAP_RANGE, 1669 },
	{ 0xb5ac, 0xb5af, PDF_CMAP_RANGE, 11224 },
	{ 0xb5b0, 0xb5b1, PDF_CMAP_RANGE, 1671 },
	{ 0xb5b2, 0xb5b2, PDF_CMAP_SINGLE, 11228 },
	{ 0xb5b3, 0xb5b5, PDF_CMAP_RANGE, 1673 },
	{ 0xb5b6, 0xb5ba, PDF_CMAP_RANGE, 11229 },
	{ 0xb5bb, 0xb5bd, PDF_CMAP_RANGE, 1676 },
	{ 0xb5be, 0xb5bf, PDF_CMAP_RANGE, 11234 },
	{ 0xb5c0, 0xb5c0, PDF_CMAP_SINGLE, 1679 },
	{ 0xb5c1, 0xb5c3, PDF_CMAP_RANGE, 11236 },
	{ 0xb5c4, 0xb5c4, PDF_CMAP_SINGLE, 1680 },
	{ 0xb5c5, 0xb5cb, PDF_CMAP_RANGE, 11239 },
	{ 0xb5cc, 0xb5cd, PDF_CMAP_RANGE, 1681 },
	{ 0xb5ce, 0xb5ce, PDF_CMAP_SINGLE, 11246 },
	{ 0xb5cf, 0xb5d1, PDF_CMAP_RANGE, 1683 },
	{ 0xb5d2, 0xb5d7, PDF_CMAP_RANGE, 11247 },
	{ 0xb5d8, 0xb5d8, PDF_CMAP_SINGLE, 1686 },
	{ 0xb5d9, 0xb5eb, PDF_CMAP_RANGE, 11253 },
	{ 0xb5ec, 0xb5ec, PDF_CMAP_SINGLE, 1687 },
	{ 0xb5ed, 0xb60f, PDF_CMAP_RANGE, 11272 },
	{ 0xb610, 0xb611, PDF_CMAP_RANGE, 1688 },
	{ 0xb612, 0xb613, PDF_CMAP_RANGE, 11307 },
	{ 0xb614, 0xb614, PDF_CMAP_SINGLE, 1690 },
	{ 0xb615, 0xb617, PDF_CMAP_RANGE, 11309 },
	{ 0xb618, 0xb618, PDF_CMAP_SINGLE, 1691 },
	{ 0xb619, 0xb624, PDF_CMAP_RANGE, 11312 },
	{ 0xb625, 0xb625, PDF_CMAP_SINGLE, 1692 },
	{ 0xb626, 0xb62b, PDF_CMAP_RANGE, 11324 },
	{ 0xb62c, 0xb62c, PDF_CMAP_SINGLE, 1693 },
	{ 0xb62d, 0xb633, PDF_CMAP_RANGE, 11330 },
	{ 0xb634, 0xb634, PDF_CMAP_SINGLE, 1694 },
	{ 0xb635, 0xb647, PDF_CMAP_RANGE, 11337 },
	{ 0xb648, 0xb648, PDF_CMAP_SINGLE, 1695 },
	{ 0xb649, 0xb663, PDF_CMAP_RANGE, 11356 },
	{ 0xb664, 0xb664, PDF_CMAP_SINGLE, 1696 },
	{ 0xb665, 0xb667, PDF_CMAP_RANGE, 11383 },
	{ 0xb668, 0xb668, PDF_CMAP_SINGLE, 1697 },
	{ 0xb669, 0xb69b, PDF_CMAP_RANGE, 11386 },
	{ 0xb69c, 0xb69d, PDF_CMAP_RANGE, 1698 },
	{ 0xb69e, 0xb69f, PDF_CMAP_RANGE, 11437 },
	{ 0xb6a0, 0xb6a0, PDF_CMAP_SINGLE, 1700 },
	{ 0xb6a1, 0xb6a3, PDF_CMAP_RANGE, 11439 },
	{ 0xb6a4, 0xb6a4, PDF_CMAP_SINGLE, 1701 },
	{ 0xb6a5, 0xb6aa, PDF_CMAP_RANGE, 11442 },
	{ 0xb6ab, 0xb6ac, PDF_CMAP_RANGE, 1702 },
	{ 0xb6ad, 0xb6b0, PDF_CMAP_RANGE, 11448 },
	{ 0xb6b1, 0xb6b1, PDF_CMAP_SINGLE, 1704 },
	{ 0xb6b2, 0xb6d3, PDF_CMAP_RANGE, 11452 },
	{ 0xb6d4, 0xb6d4, PDF_CMAP_SINGLE, 1705 },
	{ 0xb6d5, 0xb6ef, PDF_CMAP_RANGE, 11486 },
	{ 0xb6f0, 0xb6f0, PDF_CMAP_SINGLE, 1706 },
	{ 0xb6f1, 0xb6f3, PDF_CMAP_RANGE, 11513 },
	{ 0xb6f4, 0xb6f4, PDF_CMAP_SINGLE, 1707 },
	{ 0xb6f5, 0xb6f7, PDF_CMAP_RANGE, 11516 },
	{ 0xb6f8, 0xb6f8, PDF_CMAP_SINGLE, 1708 },
	{ 0xb6f9, 0xb6ff, PDF_CMAP_RANGE, 11519 },
	{ 0xb700, 0xb701, PDF_CMAP_RANGE, 1709 },
	{ 0xb702, 0xb704, PDF_CMAP_RANGE, 11526 },
	{ 0xb705, 0xb705, PDF_CMAP_SINGLE, 1711 },
	{ 0xb706, 0xb727, PDF_CMAP_RANGE, 11529 },
	{ 0xb728, 0xb729, PDF_CMAP_RANGE, 1712 },
	{ 0xb72a, 0xb72b, PDF_CMAP_RANGE, 11563 },
	{ 0xb72c, 0xb72c, PDF_CMAP_SINGLE, 1714 },
	{ 0xb72d, 0xb72e, PDF_CMAP_RANGE, 11565 },
	{ 0xb72f, 0xb730, PDF_CMAP_RANGE, 1715 },
	{ 0xb731, 0xb737, PDF_CMAP_RANGE, 11567 },
	{ 0xb738, 0xb739, PDF_CMAP_RANGE, 1717 },
	{ 0xb73a, 0xb73b, PDF_CMAP_TABLE, 2553 },
	{ 0xb73c, 0xb743, PDF_CMAP_RANGE, 11575 },
	{ 0xb744, 0xb744, PDF_CMAP_SINGLE, 1720 },
	{ 0xb745, 0xb747, PDF_CMAP_RANGE, 11583 },
	{ 0xb748, 0xb748, PDF_CMAP_SINGLE, 1721 },
	{ 0xb749, 0xb74b, PDF_CMAP_RANGE, 11586 },
	{ 0xb74c, 0xb74c, PDF_CMAP_SINGLE, 1722 },
	{ 0xb74d, 0xb753, PDF_CMAP_RANGE, 11589 },
	{ 0xb754, 0xb755, PDF_CMAP_RANGE, 1723 },
	{ 0xb756, 0xb75f, PDF_CMAP_RANGE, 11596 },
	{ 0xb760, 0xb760, PDF_CMAP_SINGLE, 1725 },
	{ 0xb761, 0xb763, PDF_CMAP_RANGE, 11606 },
	{ 0xb764, 0xb764, PDF_CMAP_SINGLE, 1726 },
	{ 0xb765, 0xb767, PDF_CMAP_RANGE, 11609 },
	{ 0xb768, 0xb768, PDF_CMAP_SINGLE, 1727 },
	{ 0xb769, 0xb76f, PDF_CMAP_RANGE, 11612 },
	{ 0xb770, 0xb771, PDF_CMAP_RANGE, 1728 },
	{ 0xb772, 0xb775, PDF_CMAP_TABLE, 2555 },
	{ 0xb776, 0xb77b, PDF_CMAP_RANGE, 11621 },
	{ 0xb77c, 0xb77d, PDF_CMAP_RANGE, 1732 },
	{ 0xb77e, 0xb77f, PDF_CMAP_RANGE, 11627 },
	{ 0xb780, 0xb780, PDF_CMAP_SINGLE, 1734 },
	{ 0xb781, 0xb783, PDF_CMAP_RANGE, 11629 },
	{ 0xb784, 0xb784, PDF_CMAP_SINGLE, 1735 },
	{ 0xb785, 0xb78b, PDF_CMAP_RANGE, 11632 },
	{ 0xb78c, 0xb78d, PDF_CMAP_RANGE, 1736 },
	{ 0xb78e, 0xb78e, PDF_CMAP_SINGLE, 11639 },
	{ 0xb78f, 0xb792, PDF_CMAP_RANGE, 1738 },
	{ 0xb793, 0xb795, PDF_CMAP_RANGE, 11640 },
	{ 0xb796, 0xb799, PDF_CMAP_RANGE, 1742 },
	{ 0xb79a, 0xb79b, PDF_CMAP_RANGE, 11643 },
	{ 0xb79c, 0xb79c, PDF_CMAP_SINGLE, 1746 },
	{ 0xb79d, 0xb79f, PDF_CMAP_RANGE, 11645 },
	{ 0xb7a0, 0xb7a0, PDF_CMAP_SINGLE, 1747 },
	{ 0xb7a1, 0xb7a7, PDF_CMAP_RANGE, 11648 },
	{ 0xb7a8, 0xb7a9, PDF_CMAP_RANGE, 1748 },
	{ 0xb7aa, 0xb7aa, PDF_CMAP_SINGLE, 11655 },
	{ 0xb7ab, 0xb7ad, PDF_CMAP_RANGE, 1750 },
	{ 0xb7ae, 0xb7b3, PDF_CMAP_RANGE, 11656 },
	{ 0xb7b4, 0xb7b5, PDF_CMAP_RANGE, 1753 },
	{ 0xb7b6, 0xb7b7, PDF_CMAP_RANGE, 11662 },
	{ 0xb7b8, 0xb7b8, PDF_CMAP_SINGLE, 1755 },
	{ 0xb7b9, 0xb7c6, PDF_CMAP_RANGE, 11664 },
	{ 0xb7c7, 0xb7c9, PDF_CMAP_TABLE, 2559 },
	{ 0xb7ca, 0xb7eb, PDF_CMAP_RANGE, 11679 },
	{ 0xb7ec, 0xb7ed, PDF_CMAP_RANGE, 1758 },
	{ 0xb7ee, 0xb7ef, PDF_CMAP_RANGE, 11713 },
	{ 0xb7f0, 0xb7f0, PDF_CMAP_SINGLE, 1760 },
	{ 0xb7f1, 0xb7f3, PDF_CMAP_RANGE, 11715 },
	{ 0xb7f4, 0xb7f4, PDF_CMAP_SINGLE, 1761 },
	{ 0xb7f5, 0xb7fb, PDF_CMAP_RANGE, 11718 },
	{ 0xb7fc, 0xb7fd, PDF_CMAP_RANGE, 1762 },
	{ 0xb7fe, 0xb7ff, PDF_CMAP_TABLE, 2562 },
	{ 0xb800, 0xb801, PDF_CMAP_RANGE, 1765 },
	{ 0xb802, 0xb806, PDF_CMAP_RANGE, 11726 },
	{ 0xb807, 0xb809, PDF_CMAP_RANGE, 1767 },
	{ 0xb80a, 0xb80b, PDF_CMAP_RANGE, 11731 },
	{ 0xb80c, 0xb80c, PDF_CMAP_SINGLE, 1770 },
	{ 0xb80d, 0xb80f, PDF_CMAP_RANGE, 11733 },
	{ 0xb810, 0xb810, PDF_CMAP_SINGLE, 1771 },
	{ 0xb811, 0xb817, PDF_CMAP_RANGE, 11736 },
	{ 0xb818, 0xb819, PDF_CMAP_RANGE, 1772 },
	{ 0xb81a, 0xb81d, PDF_CMAP_TABLE, 2564 },
	{ 0xb81e, 0xb823, PDF_CMAP_RANGE, 11745 },
	{ 0xb824, 0xb825, PDF_CMAP_RANGE, 1776 },
	{ 0xb826, 0xb827, PDF_CMAP_RANGE, 11751 },
	{ 0xb828, 0xb828, PDF_CMAP_SINGLE, 1778 },
	{ 0xb829, 0xb82b, PDF_CMAP_RANGE, 11753 },
	{ 0xb82c, 0xb82c, PDF_CMAP_SINGLE, 1779 },
	{ 0xb82d, 0xb833, PDF_CMAP_RANGE, 11756 },
	{ 0xb834, 0xb835, PDF_CMAP_RANGE, 1780 },
	{ 0xb836, 0xb836, PDF_CMAP_SINGLE, 11763 },
	{ 0xb837, 0xb839, PDF_CMAP_RANGE, 1782 },
	{ 0xb83a, 0xb83f, PDF_CMAP_RANGE, 11764 },
	{ 0xb840, 0xb840, PDF_CMAP_SINGLE, 1785 },
	{ 0xb841, 0xb843, PDF_CMAP_RANGE, 11770 },
	{ 0xb844, 0xb844, PDF_CMAP_SINGLE, 1786 },
	{ 0xb845, 0xb850, PDF_CMAP_RANGE, 11773 },
	{ 0xb851, 0xb853, PDF_CMAP_TABLE, 2568 },
	{ 0xb854, 0xb85b, PDF_CMAP_RANGE, 11786 },
	{ 0xb85c, 0xb85d, PDF_CMAP_RANGE, 1789 },
	{ 0xb85e, 0xb85f, PDF_CMAP_RANGE, 11794 },
	{ 0xb860, 0xb860, PDF_CMAP_SINGLE, 1791 },
	{ 0xb861, 0xb863, PDF_CMAP_RANGE, 11796 },
	{ 0xb864, 0xb864, PDF_CMAP_SINGLE, 1792 },
	{ 0xb865, 0xb86b, PDF_CMAP_RANGE, 11799 },
	{ 0xb86c, 0xb86d, PDF_CMAP_RANGE, 1793 },
	{ 0xb86e, 0xb871, PDF_CMAP_TABLE, 2571 },
	{ 0xb872, 0xb877, PDF_CMAP_RANGE, 11808 },
	{ 0xb878, 0xb878, PDF_CMAP_SINGLE, 1797 },
	{ 0xb879, 0xb87b, PDF_CMAP_RANGE, 11814 },
	{ 0xb87c, 0xb87c, PDF_CMAP_SINGLE, 1798 },
	{ 0xb87d, 0xb88c, PDF_CMAP_RANGE, 11817 },
	{ 0xb88d, 0xb88d, PDF_CMAP_SINGLE, 1799 },
	{ 0xb88e, 0xb8a7, PDF_CMAP_RANGE, 11833 },
	{ 0xb8a8, 0xb8a8, PDF_CMAP_SINGLE, 1800 },
	{ 0xb8a9, 0xb8af, PDF_CMAP_RANGE, 11859 },
	{ 0xb8b0, 0xb8b0, PDF_CMAP_SINGLE, 1801 },
	{ 0xb8b1, 0xb8b3, PDF_CMAP_RANGE, 11866 },
	{ 0xb8b4, 0xb8b4, PDF_CMAP_SINGLE, 1802 },
	{ 0xb8b5, 0xb8b7, PDF_CMAP_RANGE, 11869 },
	{ 0xb8b8, 0xb8b8, PDF_CMAP_SINGLE, 1803 },
	{ 0xb8b9, 0xb8bf, PDF_CMAP_RANGE, 11872 },
	{ 0xb8c0, 0xb8c1, PDF_CMAP_RANGE, 1804 },
	{ 0xb8c2, 0xb8c5, PDF_CMAP_TABLE, 2575 },
	{ 0xb8c6, 0xb8cb, PDF_CMAP_RANGE, 11881 },
	{ 0xb8cc, 0xb8cc, PDF_CMAP_SINGLE, 1808 },
	{ 0xb8cd, 0xb8cf, PDF_CMAP_RANGE, 11887 },
	{ 0xb8d0, 0xb8d0, PDF_CMAP_SINGLE, 1809 },
	{ 0xb8d1, 0xb8d3, PDF_CMAP_RANGE, 11890 },
	{ 0xb8d4, 0xb8d4, PDF_CMAP_SINGLE, 1810 },
	{ 0xb8d5, 0xb8dc, PDF_CMAP_RANGE, 11893 },
	{ 0xb8dd, 0xb8e1, PDF_CMAP_TABLE, 2579 },
	{ 0xb8e2, 0xb8e7, PDF_CMAP_RANGE, 11903 },
	{ 0xb8e8, 0xb8e9, PDF_CMAP_RANGE, 1814 },
	{ 0xb8ea, 0xb8eb, PDF_CMAP_RANGE, 11909 },
	{ 0xb8ec, 0xb8ec, PDF_CMAP_SINGLE, 1816 },
	{ 0xb8ed, 0xb8ef, PDF_CMAP_RANGE, 11911 },
	{ 0xb8f0, 0xb8f0, PDF_CMAP_SINGLE, 1817 },
	{ 0xb8f1, 0xb8f7, PDF_CMAP_RANGE, 11914 },
	{ 0xb8f8, 0xb8f9, PDF_CMAP_RANGE, 1818 },
	{ 0xb8fa, 0xb8fd, PDF_CMAP_TABLE, 2584 },
	{ 0xb8fe, 0xb903, PDF_CMAP_RANGE, 11923 },
	{ 0xb904, 0xb904, PDF_CMAP_SINGLE, 1822 },
	{ 0xb905, 0xb917, PDF_CMAP_RANGE, 11929 },
	{ 0xb918, 0xb918, PDF_CMAP_SINGLE, 1823 },
	{ 0xb919, 0xb91f, PDF_CMAP_RANGE, 11948 },
	{ 0xb920, 0xb920, PDF_CMAP_SINGLE, 1824 },
	{ 0xb921, 0xb93b, PDF_CMAP_RANGE, 11955 },
	{ 0xb93c, 0xb93d, PDF_CMAP_RANGE, 1825 },
	{ 0xb93e, 0xb93f, PDF_CMAP_RANGE, 11982 },
	{ 0xb940, 0xb940, PDF_CMAP_SINGLE, 1827 },
	{ 0xb941, 0xb943, PDF_CMAP_RANGE, 11984 },
	{ 0xb944, 0xb944, PDF_CMAP_SINGLE, 1828 },
	{ 0xb945, 0xb94b, PDF_CMAP_RANGE, 11987 },
	{ 0xb94c, 0xb94c, PDF_CMAP_SINGLE, 1829 },
	{ 0xb94d, 0xb94e, PDF_CMAP_RANGE, 11994 },
	{ 0xb94f, 0xb951, PDF_CMAP_TABLE, 2588 },
	{ 0xb952, 0xb957, PDF_CMAP_RANGE, 11997 },
	{ 0xb958, 0xb959, PDF_CMAP_RANGE, 1832 },
	{ 0xb95a, 0xb95b, PDF_CMAP_RANGE, 12003 },
	{ 0xb95c, 0xb95c, PDF_CMAP_SINGLE, 1834 },
	{ 0xb95d, 0xb95f, PDF_CMAP_RANGE, 12005 },
	{ 0xb960, 0xb960, PDF_CMAP_SINGLE, 1835 },
	{ 0xb961, 0xb967, PDF_CMAP_RANGE, 12008 },
	{ 0xb968, 0xb969, PDF_CMAP_RANGE, 1836 },
	{ 0xb96a, 0xb96d, PDF_CMAP_TABLE, 2591 },
	{ 0xb96e, 0xb973, PDF_CMAP_RANGE, 12017 },
	{ 0xb974, 0xb975, PDF_CMAP_RANGE, 1840 },
	{ 0xb976, 0xb977, PDF_CMAP_RANGE, 12023 },
	{ 0xb978, 0xb978, PDF_CMAP_SINGLE, 1842 },
	{ 0xb979, 0xb97b, PDF_CMAP_RANGE, 12025 },
	{ 0xb97c, 0xb97c, PDF_CMAP_SINGLE, 1843 },
	{ 0xb97d, 0xb983, PDF_CMAP_RANGE, 12028 },
	{ 0xb984, 0xb985, PDF_CMAP_RANGE, 1844 },
	{ 0xb986, 0xb988, PDF_CMAP_TABLE, 2595 },
	{ 0xb989, 0xb98a, PDF_CMAP_RANGE, 1847 },
	{ 0xb98b, 0xb98c, PDF_CMAP_RANGE, 12037 },
	{ 0xb98d, 0xb98e, PDF_CMAP_RANGE, 1849 },
	{ 0xb98f, 0xb9ab, PDF_CMAP_RANGE, 12039 },
	{ 0xb9ac, 0xb9ad, PDF_CMAP_RANGE, 1851 },
	{ 0xb9ae, 0xb9af, PDF_CMAP_RANGE, 12068 },
	{ 0xb9b0, 0xb9b0, PDF_CMAP_SINGLE, 1853 },
	{ 0xb9b1, 0xb9b3, PDF_CMAP_RANGE, 12070 },
	{ 0xb9b4, 0xb9b4, PDF_CMAP_SINGLE, 1854 },
	{ 0xb9b5, 0xb9bb, PDF_CMAP_RANGE, 12073 },
	{ 0xb9bc, 0xb9bd, PDF_CMAP_RANGE, 1855 },
	{ 0xb9be, 0xb9c1, PDF_CMAP_TABLE, 2598 },
	{ 0xb9c2, 0xb9c7, PDF_CMAP_RANGE, 12082 },
	{ 0xb9c8, 0xb9c9, PDF_CMAP_RANGE, 1859 },
	{ 0xb9ca, 0xb9cb, PDF_CMAP_RANGE, 12088 },
	{ 0xb9cc, 0xb9cd, PDF_CMAP_TABLE, 2602 },
	{ 0xb9ce, 0xb9d2, PDF_CMAP_RANGE, 1862 },
	{ 0xb9d3, 0xb9d7, PDF_CMAP_RANGE, 12091 },
	{ 0xb9d8, 0xb9d9, PDF_CMAP_RANGE, 1867 },
	{ 0xb9da, 0xb9dc, PDF_CMAP_TABLE, 2604 },
	{ 0xb9dd, 0xb9de, PDF_CMAP_RANGE, 1870 },
	{ 0xb9df, 0xb9e0, PDF_CMAP_RANGE, 12098 },
	{ 0xb9e1, 0xb9e2, PDF_CMAP_TABLE, 2607 },
	{ 0xb9e3, 0xb9e5, PDF_CMAP_RANGE, 1873 },
	{ 0xb9e6, 0xb9e7, PDF_CMAP_RANGE, 12101 },
	{ 0xb9e8, 0xb9e8, PDF_CMAP_SINGLE, 1876 },
	{ 0xb9e9, 0xb9eb, PDF_CMAP_RANGE, 12103 },
	{ 0xb9ec, 0xb9ec, PDF_CMAP_SINGLE, 1877 },
	{ 0xb9ed, 0xb9f3, PDF_CMAP_RANGE, 12106 },
	{ 0xb9f4, 0xb9f5, PDF_CMAP_RANGE, 1878 },
	{ 0xb9f6, 0xb9f6, PDF_CMAP_SINGLE, 12113 },
	{ 0xb9f7, 0xb9fa, PDF_CMAP_RANGE, 1880 },
	{ 0xb9fb, 0xb9ff, PDF_CMAP_RANGE, 12114 },
	{ 0xba00, 0xba01, PDF_CMAP_RANGE, 1884 },
	{ 0xba02, 0xba07, PDF_CMAP_RANGE, 12119 },
	{ 0xba08, 0xba08, PDF_CMAP_SINGLE, 1886 },
	{ 0xba09, 0xba14, PDF_CMAP_RANGE, 12125 },
	{ 0xba15, 0xba15, PDF_CMAP_SINGLE, 1887 },
	{ 0xba16, 0xba37, PDF_CMAP_RANGE, 12137 },
	{ 0xba38, 0xba39, PDF_CMAP_RANGE, 1888 },
	{ 0xba3a, 0xba3b, PDF_CMAP_RANGE, 12171 },
	{ 0xba3c, 0xba3c, PDF_CMAP_SINGLE, 1890 },
	{ 0xba3d, 0xba3f, PDF_CMAP_RANGE, 12173 },
	{ 0xba40, 0xba42, PDF_CMAP_TABLE, 2609 },
	{ 0xba43, 0xba47, PDF_CMAP_RANGE, 12177 },
	{ 0xba48, 0xba49, PDF_CMAP_RANGE, 1893 },
	{ 0xba4a, 0xba4c, PDF_CMAP_TABLE, 2612 },
	{ 0xba4d, 0xba4e, PDF_CMAP_RANGE, 1896 },
	{ 0xba4f, 0xba52, PDF_CMAP_RANGE, 12184 },
	{ 0xba53, 0xba55, PDF_CMAP_RANGE, 1898 },
	{ 0xba56, 0xba57, PDF_CMAP_RANGE, 12188 },
	{ 0xba58, 0xba58, PDF_CMAP_SINGLE, 1901 },
	{ 0xba59, 0xba5b, PDF_CMAP_RANGE, 12190 },
	{ 0xba5c, 0xba5c, PDF_CMAP_SINGLE, 1902 },
	{ 0xba5d, 0xba63, PDF_CMAP_RANGE, 12193 },
	{ 0xba64, 0xba65, PDF_CMAP_RANGE, 1903 },
	{ 0xba66, 0xba66, PDF_CMAP_SINGLE, 12200 },
	{ 0xba67, 0xba69, PDF_CMAP_RANGE, 1905 },
	{ 0xba6a, 0xba6f, PDF_CMAP_RANGE, 12201 },
	{ 0xba70, 0xba71, PDF_CMAP_RANGE, 1908 },
	{ 0xba72, 0xba73, PDF_CMAP_RANGE, 12207 },
	{ 0xba74, 0xba74, PDF_CMAP_SINGLE, 1910 },
	{ 0xba75, 0xba77, PDF_CMAP_RANGE, 12209 },
	{ 0xba78, 0xba78, PDF_CMAP_SINGLE, 1911 },
	{ 0xba79, 0xba82, PDF_CMAP_RANGE, 12212 },
	{ 0xba83, 0xba85, PDF_CMAP_RANGE, 1912 },
	{ 0xba86, 0xba87, PDF_CMAP_TABLE, 2615 },
	{ 0xba88, 0xba8b, PDF_CMAP_RANGE, 12223 },
	{ 0xba8c, 0xba8c, PDF_CMAP_SINGLE, 1916 },
	{ 0xba8d, 0xbaa7, PDF_CMAP_RANGE, 12227 },
	{ 0xbaa8, 0xbaa9, PDF_CMAP_RANGE, 1917 },
	{ 0xbaaa, 0xbaaa, PDF_CMAP_SINGLE, 12254 },
	{ 0xbaab, 0xbaac, PDF_CMAP_RANGE, 1919 },
	{ 0xbaad, 0xbaaf, PDF_CMAP_RANGE, 12255 },
	{ 0xbab0, 0xbab2, PDF_CMAP_TABLE, 2617 },
	{ 0xbab3, 0xbab7, PDF_CMAP_RANGE, 12259 },
	{ 0xbab8, 0xbab9, PDF_CMAP_RANGE, 1923 },
	{ 0xbaba, 0xbabd, PDF_CMAP_TABLE, 2620 },
	{ 0xbabe, 0xbac3, PDF_CMAP_RANGE, 12266 },
	{ 0xbac4, 0xbac4, PDF_CMAP_SINGLE, 1927 },
	{ 0xbac5, 0xbac7, PDF_CMAP_RANGE, 12272 },
	{ 0xbac8, 0xbac8, PDF_CMAP_SINGLE, 1928 },
	{ 0xbac9, 0xbad7, PDF_CMAP_RANGE, 12275 },
	{ 0xbad8, 0xbad9, PDF_CMAP_RANGE, 1929 },
	{ 0xbada, 0xbafb, PDF_CMAP_RANGE, 12290 },
	{ 0xbafc, 0xbafc, PDF_CMAP_SINGLE, 1931 },
	{ 0xbafd, 0xbaff, PDF_CMAP_RANGE, 12324 },
	{ 0xbb00, 0xbb00, PDF_CMAP_SINGLE, 1932 },
	{ 0xbb01, 0xbb03, PDF_CMAP_RANGE, 12327 },
	{ 0xbb04, 0xbb04, PDF_CMAP_SINGLE, 1933 },
	{ 0xbb05, 0xbb0c, PDF_CMAP_RANGE, 12330 },
	{ 0xbb0d, 0xbb11, PDF_CMAP_TABLE, 2624 },
	{ 0xbb12, 0xbb17, PDF_CMAP_RANGE, 12340 },
	{ 0xbb18, 0xbb18, PDF_CMAP_SINGLE, 1937 },
	{ 0xbb19, 0xbb1b, PDF_CMAP_RANGE, 12346 },
	{ 0xbb1c, 0xbb1c, PDF_CMAP_SINGLE, 1938 },
	{ 0xbb1d, 0xbb1f, PDF_CMAP_RANGE, 12349 },
	{ 0xbb20, 0xbb20, PDF_CMAP_SINGLE, 1939 },
	{ 0xbb21, 0xbb28, PDF_CMAP_RANGE, 12352 },
	{ 0xbb29, 0xbb2b, PDF_CMAP_TABLE, 2629 },
	{ 0xbb2c, 0xbb33, PDF_CMAP_RANGE, 12361 },
	{ 0xbb34, 0xbb36, PDF_CMAP_RANGE, 1942 },
	{ 0xbb37, 0xbb38, PDF_CMAP_TABLE, 2632 },
	{ 0xbb39, 0xbb3a, PDF_CMAP_RANGE, 12370 },
	{ 0xbb3b, 0xbb3e, PDF_CMAP_RANGE, 1946 },
	{ 0xbb3f, 0xbb43, PDF_CMAP_RANGE, 12372 },
	{ 0xbb44, 0xbb45, PDF_CMAP_RANGE, 1950 },
	{ 0xbb46, 0xbb49, PDF_CMAP_TABLE, 2634 },
	{ 0xbb4a, 0xbb4c, PDF_CMAP_RANGE, 12379 },
	{ 0xbb4d, 0xbb4e, PDF_CMAP_TABLE, 2638 },
	{ 0xbb4f, 0xbb50, PDF_CMAP_RANGE, 1955 },
	{ 0xbb51, 0xbb53, PDF_CMAP_RANGE, 12383 },
	{ 0xbb54, 0xbb54, PDF_CMAP_SINGLE, 1957 },
	{ 0xbb55, 0xbb57, PDF_CMAP_RANGE, 12386 },
	{ 0xbb58, 0xbb58, PDF_CMAP_SINGLE, 1958 },
	{ 0xbb59, 0xbb60, PDF_CMAP_RANGE, 12389 },
	{ 0xbb61, 0xbb63, PDF_CMAP_TABLE, 2640 },
	{ 0xbb64, 0xbb6b, PDF_CMAP_RANGE, 12398 },
	{ 0xbb6c, 0xbb6c, PDF_CMAP_SINGLE, 1961 },
	{ 0xbb6d, 0xbb87, PDF_CMAP_RANGE, 12406 },
	{ 0xbb88, 0xbb88, PDF_CMAP_SINGLE, 1962 },
	{ 0xbb89, 0xbb8b, PDF_CMAP_RANGE, 12433 },
	{ 0xbb8c, 0xbb8c, PDF_CMAP_SINGLE, 1963 },
	{ 0xbb8d, 0xbb8f, PDF_CMAP_RANGE, 12436 },
	{ 0xbb90, 0xbb90, PDF_CMAP_SINGLE, 1964 },
	{ 0xbb91, 0xbba3, PDF_CMAP_RANGE, 12439 },
	{ 0xbba4, 0xbba4, PDF_CMAP_SINGLE, 1965 },
	{ 0xbba5, 0xbba7, PDF_CMAP_RANGE, 12458 },
	{ 0xbba8, 0xbba8, PDF_CMAP_SINGLE, 1966 },
	{ 0xbba9, 0xbbab, PDF_CMAP_RANGE, 12461 },
	{ 0xbbac, 0xbbac, PDF_CMAP_SINGLE, 1967 },
	{ 0xbbad, 0xbbb3, PDF_CMAP_RANGE, 12464 },
	{ 0xbbb4, 0xbbb4, PDF_CMAP_SINGLE, 1968 },
	{ 0xbbb5, 0xbbb6, PDF_CMAP_RANGE, 12471 },
	{ 0xbbb7, 0xbbb7, PDF_CMAP_SINGLE, 1969 },
	{ 0xbbb8, 0xbbbf, PDF_CMAP_RANGE, 12473 },
	{ 0xbbc0, 0xbbc0, PDF_CMAP_SINGLE, 1970 },
	{ 0xbbc1, 0xbbc3, PDF_CMAP_RANGE, 12481 },
	{ 0xbbc4, 0xbbc4, PDF_CMAP_SINGLE, 1971 },
	{ 0xbbc5, 0xbbc7, PDF_CMAP_RANGE, 12484 },
	{ 0xbbc8, 0xbbc8, PDF_CMAP_SINGLE, 1972 },
	{ 0xbbc9, 0xbbcf, PDF_CMAP_RANGE, 12487 },
	{ 0xbbd0, 0xbbd0, PDF_CMAP_SINGLE, 1973 },
	{ 0xbbd1, 0xbbd2, PDF_CMAP_RANGE, 12494 },
	{ 0xbbd3, 0xbbd3, PDF_CMAP_SINGLE, 1974 },
	{ 0xbbd4, 0xbbf7, PDF_CMAP_RANGE, 12496 },
	{ 0xbbf8, 0xbbf9, PDF_CMAP_RANGE, 1975 },
	{ 0xbbfa, 0xbbfb, PDF_CMAP_RANGE, 12532 },
	{ 0xbbfc, 0xbbfc, PDF_CMAP_SINGLE, 1977 },
	{ 0xbbfd, 0xbbfe, PDF_CMAP_RANGE, 12534 },
	{ 0xbbff, 0xbc00, PDF_CMAP_RANGE, 1978 },
	{ 0xbc01, 0xbc02, PDF_CMAP_TABLE, 2643 },
	{ 0xbc03, 0xbc07, PDF_CMAP_RANGE, 12537 },
	{ 0xbc08, 0xbc09, PDF_CMAP_RANGE, 1981 },
	{ 0xbc0a, 0xbc0a, PDF_CMAP_SINGLE, 12542 },
	{ 0xbc0b, 0xbc0d, PDF_CMAP_RANGE, 1983 },
	{ 0xbc0e, 0xbc11, PDF_CMAP_TABLE, 2645 },
	{ 0xbc12, 0xbc13, PDF_CMAP_RANGE, 12545 },
	{ 0xbc14, 0xbc18, PDF_CMAP_RANGE, 1988 },
	{ 0xbc19, 0xbc1a, PDF_CMAP_RANGE, 12547 },
	{ 0xbc1b, 0xbc1f, PDF_CMAP_RANGE, 1993 },
	{ 0xbc20, 0xbc23, PDF_CMAP_RANGE, 12549 },
	{ 0xbc24, 0xbc25, PDF_CMAP_RANGE, 1998 },
	{ 0xbc26, 0xbc29, PDF_CMAP_TABLE, 2649 },
	{ 0xbc2a, 0xbc2c, PDF_CMAP_RANGE, 12555 },
	{ 0xbc2d, 0xbc2d, PDF_CMAP_SINGLE, 2002 },
	{ 0xbc2e, 0xbc2f, PDF_CMAP_RANGE, 12558 },
	{ 0xbc30, 0xbc31, PDF_CMAP_RANGE, 2003 },
	{ 0xbc32, 0xbc33, PDF_CMAP_RANGE, 12560 },
	{ 0xbc34, 0xbc34, PDF_CMAP_SINGLE, 2005 },
	{ 0xbc35, 0xbc37, PDF_CMAP_RANGE, 12562 },
	{ 0xbc38, 0xbc38, PDF_CMAP_SINGLE, 2006 },
	{ 0xbc39, 0xbc3f, PDF_CMAP_RANGE, 12565 },
	{ 0xbc40, 0xbc41, PDF_CMAP_RANGE, 2007 },
	{ 0xbc42, 0xbc42, PDF_CMAP_SINGLE, 12572 },
	{ 0xbc43, 0xbc45, PDF_CMAP_RANGE, 2009 },
	{ 0xbc46, 0xbc48, PDF_CMAP_RANGE, 12573 },
	{ 0xbc49, 0xbc49, PDF_CMAP_SINGLE, 2012 },
	{ 0xbc4a, 0xbc4b, PDF_CMAP_RANGE, 12576 },
	{ 0xbc4c, 0xbc4d, PDF_CMAP_RANGE, 2013 },
	{ 0xbc4e, 0xbc4f, PDF_CMAP_RANGE, 12578 },
	{ 0xbc50, 0xbc50, PDF_CMAP_SINGLE, 2015 },
	{ 0xbc51, 0xbc5c, PDF_CMAP_RANGE, 12580 },
	{ 0xbc5d, 0xbc5d, PDF_CMAP_SINGLE, 2016 },
	{ 0xbc5e, 0xbc83, PDF_CMAP_RANGE, 12592 },
	{ 0xbc84, 0xbc85, PDF_CMAP_RANGE, 2017 },
	{ 0xbc86, 0xbc87, PDF_CMAP_RANGE, 12630 },
	{ 0xbc88, 0xbc88, PDF_CMAP_SINGLE, 2019 },
	{ 0xbc89, 0xbc8a, PDF_CMAP_RANGE, 12632 },
	{ 0xbc8b, 0xbc8c, PDF_CMAP_RANGE, 2020 },
	{ 0xbc8d, 0xbc8e, PDF_CMAP_TABLE, 2653 },
	{ 0xbc8f, 0xbc93, PDF_CMAP_RANGE, 12635 },
	{ 0xbc94, 0xbc95, PDF_CMAP_RANGE, 2023 },
	{ 0xbc96, 0xbc98, PDF_CMAP_TABLE, 2655 },
	{ 0xbc99, 0xbc9a, PDF_CMAP_RANGE, 2026 },
	{ 0xbc9b, 0xbc9f, PDF_CMAP_RANGE, 12642 },
	{ 0xbca0, 0xbca1, PDF_CMAP_RANGE, 2028 },
	{ 0xbca2, 0xbca3, PDF_CMAP_RANGE, 12647 },
	{ 0xbca4, 0xbca4, PDF_CMAP_SINGLE, 2030 },
	{ 0xbca5, 0xbca6, PDF_CMAP_RANGE, 12649 },
	{ 0xbca7, 0xbca8, PDF_CMAP_RANGE, 2031 },
	{ 0xbca9, 0xbcaf, PDF_CMAP_RANGE, 12651 },
	{ 0xbcb0, 0xbcb1, PDF_CMAP_RANGE, 2033 },
	{ 0xbcb2, 0xbcb2, PDF_CMAP_SINGLE, 12658 },
	{ 0xbcb3, 0xbcb5, PDF_CMAP_RANGE, 2035 },
	{ 0xbcb6, 0xbcbb, PDF_CMAP_RANGE, 12659 },
	{ 0xbcbc, 0xbcbd, PDF_CMAP_RANGE, 2038 },
	{ 0xbcbe, 0xbcbf, PDF_CMAP_RANGE, 12665 },
	{ 0xbcc0, 0xbcc0, PDF_CMAP_SINGLE, 2040 },
	{ 0xbcc1, 0xbcc3, PDF_CMAP_RANGE, 12667 },
	{ 0xbcc4, 0xbcc4, PDF_CMAP_SINGLE, 2041 },
	{ 0xbcc5, 0xbccc, PDF_CMAP_RANGE, 12670 },
	{ 0xbccd, 0xbcce, PDF_CMAP_TABLE, 2658 },
	{ 0xbccf, 0xbcd1, PDF_CMAP_RANGE, 2043 },
	{ 0xbcd2, 0xbcd4, PDF_CMAP_RANGE, 12679 },
	{ 0xbcd5, 0xbcd5, PDF_CMAP_SINGLE, 2046 },
	{ 0xbcd6, 0xbcd7, PDF_CMAP_RANGE, 12682 },
	{ 0xbcd8, 0xbcd8, PDF_CMAP_SINGLE, 2047 },
	{ 0xbcd9, 0xbcdb, PDF_CMAP_RANGE, 12684 },
	{ 0xbcdc, 0xbcdc, PDF_CMAP_SINGLE, 2048 },
	{ 0xbcdd, 0xbcf3, PDF_CMAP_RANGE, 12687 },
	{ 0xbcf4, 0xbcf6, PDF_CMAP_RANGE, 2049 },
	{ 0xbcf7, 0xbcf8, PDF_CMAP_TABLE, 2660 },
	{ 0xbcf9, 0xbcfb, PDF_CMAP_RANGE, 12711 },
	{ 0xbcfc, 0xbcfc, PDF_CMAP_SINGLE, 2053 },
	{ 0xbcfd, 0xbd03, PDF_CMAP_RANGE, 12714 },
	{ 0xbd04, 0xbd05, PDF_CMAP_RANGE, 2054 },
	{ 0xbd06, 0xbd09, PDF_CMAP_TABLE, 2662 },
	{ 0xbd0a, 0xbd0f, PDF_CMAP_RANGE, 12723 },
	{ 0xbd10, 0xbd10, PDF_CMAP_SINGLE, 2058 },
	{ 0xbd11, 0xbd13, PDF_CMAP_RANGE, 12729 },
	{ 0xbd14, 0xbd14, PDF_CMAP_SINGLE, 2059 },
	{ 0xbd15, 0xbd23, PDF_CMAP_RANGE, 12732 },
	{ 0xbd24, 0xbd24, PDF_CMAP_SINGLE, 2060 },
	{ 0xbd25, 0xbd2b, PDF_CMAP_RANGE, 12747 },
	{ 0xbd2c, 0xbd2c, PDF_CMAP_SINGLE, 2061 },
	{ 0xbd2d, 0xbd3f, PDF_CMAP_RANGE, 12754 },
	{ 0xbd40, 0xbd40, PDF_CMAP_SINGLE, 2062 },
	{ 0xbd41, 0xbd47, PDF_CMAP_RANGE, 12773 },
	{ 0xbd48, 0xbd49, PDF_CMAP_RANGE, 2063 },
	{ 0xbd4a, 0xbd4b, PDF_CMAP_RANGE, 12780 },
	{ 0xbd4c, 0xbd4c, PDF_CMAP_SINGLE, 2065 },
	{ 0xbd4d, 0xbd4f, PDF_CMAP_RANGE, 12782 },
	{ 0xbd50, 0xbd50, PDF_CMAP_SINGLE, 2066 },
	{ 0xbd51, 0xbd57, PDF_CMAP_RANGE, 12785 },
	{ 0xbd58, 0xbd59, PDF_CMAP_RANGE, 2067 },
	{ 0xbd5a, 0xbd63, PDF_CMAP_RANGE, 12792 },
	{ 0xbd64, 0xbd64, PDF_CMAP_SINGLE, 2069 },
	{ 0xbd65, 0xbd67, PDF_CMAP_RANGE, 12802 },
	{ 0xbd68, 0xbd68, PDF_CMAP_SINGLE, 2070 },
	{ 0xbd69, 0xbd7f, PDF_CMAP_RANGE, 12805 },
	{ 0xbd80, 0xbd81, PDF_CMAP_RANGE, 2071 },
	{ 0xbd82, 0xbd83, PDF_CMAP_RANGE, 12828 },
	{ 0xbd84, 0xbd84, PDF_CMAP_SINGLE, 2073 },
	{ 0xbd85, 0xbd86, PDF_CMAP_RANGE, 12830 },
	{ 0xbd87, 0xbd8a, PDF_CMAP_RANGE, 2074 },
	{ 0xbd8b, 0xbd8f, PDF_CMAP_RANGE, 12832 },
	{ 0xbd90, 0xbd91, PDF_CMAP_RANGE, 2078 },
	{ 0xbd92, 0xbd95, PDF_CMAP_TABLE, 2666 },
	{ 0xbd96, 0xbd98, PDF_CMAP_RANGE, 12839 },
	{ 0xbd99, 0xbd9a, PDF_CMAP_RANGE, 2082 },
	{ 0xbd9b, 0xbd9c, PDF_CMAP_TABLE, 2670 },
	{ 0xbd9d, 0xbda3, PDF_CMAP_RANGE, 12843 },
	{ 0xbda4, 0xbda4, PDF_CMAP_SINGLE, 2085 },
	{ 0xbda5, 0xbdaf, PDF_CMAP_RANGE, 12850 },
	{ 0xbdb0, 0xbdb0, PDF_CMAP_SINGLE, 2086 },
	{ 0xbdb1, 0xbdb7, PDF_CMAP_RANGE, 12861 },
	{ 0xbdb8, 0xbdb8, PDF_CMAP_SINGLE, 2087 },
	{ 0xbdb9, 0xbdd3, PDF_CMAP_RANGE, 12868 },
	{ 0xbdd4, 0xbdd5, PDF_CMAP_RANGE, 2088 },
	{ 0xbdd6, 0xbdd7, PDF_CMAP_RANGE, 12895 },
	{ 0xbdd8, 0xbdd8, PDF_CMAP_SINGLE, 2090 },
	{ 0xbdd9, 0xbddb, PDF_CMAP_RANGE, 12897 },
	{ 0xbddc, 0xbddc, PDF_CMAP_SINGLE, 2091 },
	{ 0xbddd, 0xbde8, PDF_CMAP_RANGE, 12900 },
	{ 0xbde9, 0xbde9, PDF_CMAP_SINGLE, 2092 },
	{ 0xbdea, 0xbdef, PDF_CMAP_RANGE, 12912 },
	{ 0xbdf0, 0xbdf0, PDF_CMAP_SINGLE, 2093 },
	{ 0xbdf1, 0xbdf3, PDF_CMAP_RANGE, 12918 },
	{ 0xbdf4, 0xbdf4, PDF_CMAP_SINGLE, 2094 },
	{ 0xbdf5, 0xbdf7, PDF_CMAP_RANGE, 12921 },
	{ 0xbdf8, 0xbdf8, PDF_CMAP_SINGLE, 2095 },
	{ 0xbdf9, 0xbdff, PDF_CMAP_RANGE, 12924 },
	{ 0xbe00, 0xbe00, PDF_CMAP_SINGLE, 2096 },
	{ 0xbe01, 0xbe02, PDF_CMAP_RANGE, 12931 },
	{ 0xbe03, 0xbe05, PDF_CMAP_TABLE, 2672 },
	{ 0xbe06, 0xbe0b, PDF_CMAP_RANGE, 12934 },
	{ 0xbe0c, 0xbe0d, PDF_CMAP_RANGE, 2099 },
	{ 0xbe0e, 0xbe0f, PDF_CMAP_RANGE, 12940 },
	{ 0xbe10, 0xbe10, PDF_CMAP_SINGLE, 2101 },
	{ 0xbe11, 0xbe13, PDF_CMAP_RANGE, 12942 },
	{ 0xbe14, 0xbe14, PDF_CMAP_SINGLE, 2102 },
	{ 0xbe15, 0xbe1b, PDF_CMAP_RANGE, 12945 },
	{ 0xbe1c, 0xbe1d, PDF_CMAP_RANGE, 2103 },
	{ 0xbe1e, 0xbe1f, PDF_CMAP_TABLE, 2675 },
	{ 0xbe20, 0xbe43, PDF_CMAP_RANGE, 12953 },
	{ 0xbe44, 0xbe45, PDF_CMAP_RANGE, 2106 },
	{ 0xbe46, 0xbe47, PDF_CMAP_RANGE, 12989 },
	{ 0xbe48, 0xbe48, PDF_CMAP_SINGLE, 2108 },
	{ 0xbe49, 0xbe4b, PDF_CMAP_RANGE, 12991 },
	{ 0xbe4c, 0xbe4e, PDF_CMAP_TABLE, 2677 },
	{ 0xbe4f, 0xbe53, PDF_CMAP_RANGE, 12995 },
	{ 0xbe54, 0xbe55, PDF_CMAP_RANGE, 2111 },
	{ 0xbe56, 0xbe58, PDF_CMAP_TABLE, 2680 },
	{ 0xbe59, 0xbe5b, PDF_CMAP_RANGE, 2114 },
	{ 0xbe5c, 0xbe5f, PDF_CMAP_RANGE, 13002 },
	{ 0xbe60, 0xbe61, PDF_CMAP_RANGE, 2117 },
	{ 0xbe62, 0xbe63, PDF_CMAP_RANGE, 13006 },
	{ 0xbe64, 0xbe64, PDF_CMAP_SINGLE, 2119 },
	{ 0xbe65, 0xbe67, PDF_CMAP_RANGE, 13008 },
	{ 0xbe68, 0xbe6a, PDF_CMAP_TABLE, 2683 },
	{ 0xbe6b, 0xbe6f, PDF_CMAP_RANGE, 13012 },
	{ 0xbe70, 0xbe71, PDF_CMAP_RANGE, 2122 },
	{ 0xbe72, 0xbe72, PDF_CMAP_SINGLE, 13017 },
	{ 0xbe73, 0xbe75, PDF_CMAP_RANGE, 2124 },
	{ 0xbe76, 0xbe7a, PDF_CMAP_RANGE, 13018 },
	{ 0xbe7b, 0xbe7d, PDF_CMAP_RANGE, 2127 },
	{ 0xbe7e, 0xbe7f, PDF_CMAP_RANGE, 13023 },
	{ 0xbe80, 0xbe80, PDF_CMAP_SINGLE, 2130 },
	{ 0xbe81, 0xbe83, PDF_CMAP_RANGE, 13025 },
	{ 0xbe84, 0xbe84, PDF_CMAP_SINGLE, 2131 },
	{ 0xbe85, 0xbe8b, PDF_CMAP_RANGE, 13028 },
	{ 0xbe8c, 0xbe8d, PDF_CMAP_RANGE, 2132 },
	{ 0xbe8e, 0xbe8e, PDF_CMAP_SINGLE, 13035 },
	{ 0xbe8f, 0xbe91, PDF_CMAP_RANGE, 2134 },
	{ 0xbe92, 0xbe97, PDF_CMAP_RANGE, 13036 },
	{ 0xbe98, 0xbe99, PDF_CMAP_RANGE, 2137 },
	{ 0xbe9a, 0xbea7, PDF_CMAP_RANGE, 13042 },
	{ 0xbea8, 0xbea8, PDF_CMAP_SINGLE, 2139 },
	{ 0xbea9, 0xbecf, PDF_CMAP_RANGE, 13056 },
	{ 0xbed0, 0xbed1, PDF_CMAP_RANGE, 2140 },
	{ 0xbed2, 0xbed3, PDF_CMAP_RANGE, 13095 },
	{ 0xbed4, 0xbed4, PDF_CMAP_SINGLE, 2142 },
	{ 0xbed5, 0xbed6, PDF_CMAP_RANGE, 13097 },
	{ 0xbed7, 0xbed8, PDF_CMAP_RANGE, 2143 },
	{ 0xbed9, 0xbedf, PDF_CMAP_RANGE, 13099 },
	{ 0xbee0, 0xbee0, PDF_CMAP_SINGLE, 2145 },
	{ 0xbee1, 0xbee2, PDF_CMAP_RANGE, 13106 },
	{ 0xbee3, 0xbee5, PDF_CMAP_RANGE, 2146 },
	{ 0xbee6, 0xbeeb, PDF_CMAP_RANGE, 13108 },
	{ 0xbeec, 0xbeec, PDF_CMAP_SINGLE, 2149 },
	{ 0xbeed, 0xbf00, PDF_CMAP_RANGE, 13114 },
	{ 0xbf01, 0xbf01, PDF_CMAP_SINGLE, 2150 },
	{ 0xbf02, 0xbf07, PDF_CMAP_RANGE, 13134 },
	{ 0xbf08, 0xbf09, PDF_CMAP_RANGE, 2151 },
	{ 0xbf0a, 0xbf17, PDF_CMAP_RANGE, 13140 },
	{ 0xbf18, 0xbf19, PDF_CMAP_RANGE, 2153 },
	{ 0xbf1a, 0xbf1a, PDF_CMAP_SINGLE, 13154 },
	{ 0xbf1b, 0xbf1d, PDF_CMAP_RANGE, 2155 },
	{ 0xbf1e, 0xbf3f, PDF_CMAP_RANGE, 13155 },
	{ 0xbf40, 0xbf41, PDF_CMAP_RANGE, 2158 },
	{ 0xbf42, 0xbf43, PDF_CMAP_RANGE, 13189 },
	{ 0xbf44, 0xbf44, PDF_CMAP_SINGLE, 2160 },
	{ 0xbf45, 0xbf47, PDF_CMAP_RANGE, 13191 },
	{ 0xbf48, 0xbf48, PDF_CMAP_SINGLE, 2161 },
	{ 0xbf49, 0xbf4f, PDF_CMAP_RANGE, 13194 },
	{ 0xbf50, 0xbf51, PDF_CMAP_RANGE, 2162 },
	{ 0xbf52, 0xbf54, PDF_CMAP_RANGE, 13201 },
	{ 0xbf55, 0xbf55, PDF_CMAP_SINGLE, 2164 },
	{ 0xbf56, 0xbf93, PDF_CMAP_RANGE, 13204 },
	{ 0xbf94, 0xbf94, PDF_CMAP_SINGLE, 2165 },
	{ 0xbf95, 0xbfaf, PDF_CMAP_RANGE, 13266 },
	{ 0xbfb0, 0xbfb0, PDF_CMAP_SINGLE, 2166 },
	{ 0xbfb1, 0xbfc4, PDF_CMAP_RANGE, 13293 },
	{ 0xbfc5, 0xbfc5, PDF_CMAP_SINGLE, 2167 },
	{ 0xbfc6, 0xbfcb, PDF_CMAP_RANGE, 13313 },
	{ 0xbfcc, 0xbfcd, PDF_CMAP_RANGE, 2168 },
	{ 0xbfce, 0xbfcf, PDF_CMAP_RANGE, 13319 },
	{ 0xbfd0, 0xbfd0, PDF_CMAP_SINGLE, 2170 },
	{ 0xbfd1, 0xbfd3, PDF_CMAP_RANGE, 13321 },
	{ 0xbfd4, 0xbfd4, PDF_CMAP_SINGLE, 2171 },
	{ 0xbfd5, 0xbfdb, PDF_CMAP_RANGE, 13324 },
	{ 0xbfdc, 0xbfdc, PDF_CMAP_SINGLE, 2172 },
	{ 0xbfdd, 0xbfde, PDF_CMAP_RANGE, 13331 },
	{ 0xbfdf, 0xbfe1, PDF_CMAP_TABLE, 2686 },
	{ 0xbfe2, 0xc03b, PDF_CMAP_RANGE, 13334 },
	{ 0xc03c, 0xc03c, PDF_CMAP_SINGLE, 2175 },
	{ 0xc03d, 0xc050, PDF_CMAP_RANGE, 13424 },
	{ 0xc051, 0xc051, PDF_CMAP_SINGLE, 2176 },
	{ 0xc052, 0xc057, PDF_CMAP_RANGE, 13444 },
	{ 0xc058, 0xc058, PDF_CMAP_SINGLE, 2177 },
	{ 0xc059, 0xc05b, PDF_CMAP_RANGE, 13450 },
	{ 0xc05c, 0xc05c, PDF_CMAP_SINGLE, 2178 },
	{ 0xc05d, 0xc05f, PDF_CMAP_RANGE, 13453 },
	{ 0xc060, 0xc060, PDF_CMAP_SINGLE, 2179 },
	{ 0xc061, 0xc067, PDF_CMAP_RANGE, 13456 },
	{ 0xc068, 0xc069, PDF_CMAP_RANGE, 2180 },
	{ 0xc06a, 0xc08f, PDF_CMAP_RANGE, 13463 },
	{ 0xc090, 0xc091, PDF_CMAP_RANGE, 2182 },
	{ 0xc092, 0xc093, PDF_CMAP_RANGE, 13501 },
	{ 0xc094, 0xc094, PDF_CMAP_SINGLE, 2184 },
	{ 0xc095, 0xc097, PDF_CMAP_RANGE, 13503 },
	{ 0xc098, 0xc098, PDF_CMAP_SINGLE, 2185 },
	{ 0xc099, 0xc09f, PDF_CMAP_RANGE, 13506 },
	{ 0xc0a0, 0xc0a1, PDF_CMAP_RANGE, 2186 },
	{ 0xc0a2, 0xc0a5, PDF_CMAP_TABLE, 2689 },
	{ 0xc0a6, 0xc0ab, PDF_CMAP_RANGE, 13515 },
	{ 0xc0ac, 0xc0ad, PDF_CMAP_RANGE, 2190 },
	{ 0xc0ae, 0xc0ae, PDF_CMAP_SINGLE, 13521 },
	{ 0xc0af, 0xc0b0, PDF_CMAP_RANGE, 2192 },
	{ 0xc0b1, 0xc0b2, PDF_CMAP_RANGE, 13522 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 2194 },
	{ 0xc0b7, 0xc0bb, PDF_CMAP_RANGE, 13524 },
	{ 0xc0bc, 0xc0bd, PDF_CMAP_RANGE, 2198 },
	{ 0xc0be, 0xc0be, PDF_CMAP_SINGLE, 13529 },
	{ 0xc0bf, 0xc0c1, PDF_CMAP_RANGE, 2200 },
	{ 0xc0c2, 0xc0c4, PDF_CMAP_RANGE, 13530 },
	{ 0xc0c5, 0xc0c5, PDF_CMAP_SINGLE, 2203 },
	{ 0xc0c6, 0xc0c7, PDF_CMAP_RANGE, 13533 },
	{ 0xc0c8, 0xc0c9, PDF_CMAP_RANGE, 2204 },
	{ 0xc0ca, 0xc0cb, PDF_CMAP_RANGE, 13535 },
	{ 0xc0cc, 0xc0cc, PDF_CMAP_SINGLE, 2206 },
	{ 0xc0cd, 0xc0cf, PDF_CMAP_RANGE, 13537 },
	{ 0xc0d0, 0xc0d0, PDF_CMAP_SINGLE, 2207 },
	{ 0xc0d1, 0xc0d7, PDF_CMAP_RANGE, 13540 },
	{ 0xc0d8, 0xc0d9, PDF_CMAP_RANGE, 2208 },
	{ 0xc0da, 0xc0da, PDF_CMAP_SINGLE, 13547 },
	{ 0xc0db, 0xc0dd, PDF_CMAP_RANGE, 2210 },
	{ 0xc0de, 0xc0e3, PDF_CMAP_RANGE, 13548 },
	{ 0xc0e4, 0xc0e5, PDF_CMAP_RANGE, 2213 },
	{ 0xc0e6, 0xc0e7, PDF_CMAP_RANGE, 13554 },
	{ 0xc0e8, 0xc0e8, PDF_CMAP_SINGLE, 2215 },
	{ 0xc0e9, 0xc0eb, PDF_CMAP_RANGE, 13556 },
	{ 0xc0ec, 0xc0ec, PDF_CMAP_SINGLE, 2216 },
	{ 0xc0ed, 0xc0f3, PDF_CMAP_RANGE, 13559 },
	{ 0xc0f4, 0xc0f5, PDF_CMAP_RANGE, 2217 },
	{ 0xc0f6, 0xc0f9, PDF_CMAP_TABLE, 2693 },
	{ 0xc0fa, 0xc0ff, PDF_CMAP_RANGE, 13568 },
	{ 0xc100, 0xc100, PDF_CMAP_SINGLE, 2221 },
	{ 0xc101, 0xc103, PDF_CMAP_RANGE, 13574 },
	{ 0xc104, 0xc104, PDF_CMAP_SINGLE, 2222 },
	{ 0xc105, 0xc107, PDF_CMAP_RANGE, 13577 },
	{ 0xc108, 0xc108, PDF_CMAP_SINGLE, 2223 },
	{ 0xc109, 0xc10f, PDF_CMAP_RANGE, 13580 },
	{ 0xc110, 0xc110, PDF_CMAP_SINGLE, 2224 },
	{ 0xc111, 0xc114, PDF_CMAP_RANGE, 13587 },
	{ 0xc115, 0xc115, PDF_CMAP_SINGLE, 2225 },
	{ 0xc116, 0xc11b, PDF_CMAP_RANGE, 13591 },
	{ 0xc11c, 0xc120, PDF_CMAP_RANGE, 2226 },
	{ 0xc121, 0xc122, PDF_CMAP_RANGE, 13597 },
	{ 0xc123, 0xc124, PDF_CMAP_RANGE, 2231 },
	{ 0xc125, 0xc125, PDF_CMAP_SINGLE, 13599 },
	{ 0xc126, 0xc127, PDF_CMAP_RANGE, 2233 },
	{ 0xc128, 0xc12b, PDF_CMAP_RANGE, 13600 },
	{ 0xc12c, 0xc12d, PDF_CMAP_RANGE, 2235 },
	{ 0xc12e, 0xc12e, PDF_CMAP_SINGLE, 13604 },
	{ 0xc12f, 0xc131, PDF_CMAP_RANGE, 2237 },
	{ 0xc132, 0xc135, PDF_CMAP_RANGE, 13605 },
	{ 0xc136, 0xc137, PDF_CMAP_TABLE, 2697 },
	{ 0xc138, 0xc139, PDF_CMAP_RANGE, 2241 },
	{ 0xc13a, 0xc13b, PDF_CMAP_RANGE, 13610 },
	{ 0xc13c, 0xc13c, PDF_CMAP_SINGLE, 2243 },
	{ 0xc13d, 0xc13f, PDF_CMAP_RANGE, 13612 },
	{ 0xc140, 0xc140, PDF_CMAP_SINGLE, 2244 },
	{ 0xc141, 0xc147, PDF_CMAP_RANGE, 13615 },
	{ 0xc148, 0xc149, PDF_CMAP_RANGE, 2245 },
	{ 0xc14a, 0xc14a, PDF_CMAP_SINGLE, 13622 },
	{ 0xc14b, 0xc14d, PDF_CMAP_RANGE, 2247 },
	{ 0xc14e, 0xc153, PDF_CMAP_RANGE, 13623 },
	{ 0xc154, 0xc155, PDF_CMAP_RANGE, 2250 },
	{ 0xc156, 0xc157, PDF_CMAP_RANGE, 13629 },
	{ 0xc158, 0xc158, PDF_CMAP_SINGLE, 2252 },
	{ 0xc159, 0xc15b, PDF_CMAP_RANGE, 13631 },
	{ 0xc15c, 0xc15c, PDF_CMAP_SINGLE, 2253 },
	{ 0xc15d, 0xc163, PDF_CMAP_RANGE, 13634 },
	{ 0xc164, 0xc165, PDF_CMAP_RANGE, 2254 },
	{ 0xc166, 0xc166, PDF_CMAP_SINGLE, 13641 },
	{ 0xc167, 0xc169, PDF_CMAP_RANGE, 2256 },
	{ 0xc16a, 0xc16f, PDF_CMAP_RANGE, 13642 },
	{ 0xc170, 0xc170, PDF_CMAP_SINGLE, 2259 },
	{ 0xc171, 0xc173, PDF_CMAP_RANGE, 13648 },
	{ 0xc174, 0xc174, PDF_CMAP_SINGLE, 2260 },
	{ 0xc175, 0xc177, PDF_CMAP_RANGE, 13651 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 2261 },
	{ 0xc179, 0xc184, PDF_CMAP_RANGE, 13654 },
	{ 0xc185, 0xc185, PDF_CMAP_SINGLE, 2262 },
	{ 0xc186, 0xc18b, PDF_CMAP_RANGE, 13666 },
	{ 0xc18c, 0xc18e, PDF_CMAP_RANGE, 2263 },
	{ 0xc18f, 0xc190, PDF_CMAP_TABLE, 2699 },
	{ 0xc191, 0xc193, PDF_CMAP_RANGE, 13673 },
	{ 0xc194, 0xc196, PDF_CMAP_TABLE, 2701 },
	{ 0xc197, 0xc19b, PDF_CMAP_RANGE, 13677 },
	{ 0xc19c, 0xc19d, PDF_CMAP_RANGE, 2269 },
	{ 0xc19e, 0xc1a1, PDF_CMAP_TABLE, 2704 },
	{ 0xc1a2, 0xc1a4, PDF_CMAP_RANGE, 13684 },
	{ 0xc1a5, 0xc1a5, PDF_CMAP_SINGLE, 2273 },
	{ 0xc1a6, 0xc1a7, PDF_CMAP_RANGE, 13687 },
	{ 0xc1a8, 0xc1a9, PDF_CMAP_RANGE, 2274 },
	{ 0xc1aa, 0xc1ab, PDF_CMAP_RANGE, 13689 },
	{ 0xc1ac, 0xc1ac, PDF_CMAP_SINGLE, 2276 },
	{ 0xc1ad, 0xc1af, PDF_CMAP_RANGE, 13691 },
	{ 0xc1b0, 0xc1b0, PDF_CMAP_SINGLE, 2277 },
	{ 0xc1b1, 0xc1bc, PDF_CMAP_RANGE, 13694 },
	{ 0xc1bd, 0xc1bd, PDF_CMAP_SINGLE, 2278 },
	{ 0xc1be, 0xc1c3, PDF_CMAP_RANGE, 13706 },
	{ 0xc1c4, 0xc1c4, PDF_CMAP_SINGLE, 2279 },
	{ 0xc1c5, 0xc1c7, PDF_CMAP_RANGE, 13712 },
	{ 0xc1c8, 0xc1c8, PDF_CMAP_SINGLE, 2280 },
	{ 0xc1c9, 0xc1cb, PDF_CMAP_RANGE, 13715 },
	{ 0xc1cc, 0xc1cc, PDF_CMAP_SINGLE, 2281 },
	{ 0xc1cd, 0xc1d3, PDF_CMAP_RANGE, 13718 },
	{ 0xc1d4, 0xc1d4, PDF_CMAP_SINGLE, 2282 },
	{ 0xc1d5, 0xc1d6, PDF_CMAP_RANGE, 13725 },
	{ 0xc1d7, 0xc1d8, PDF_CMAP_RANGE, 2283 },
	{ 0xc1d9, 0xc1df, PDF_CMAP_RANGE, 13727 },
	{ 0xc1e0, 0xc1e0, PDF_CMAP_SINGLE, 2285 },
	{ 0xc1e1, 0xc1e3, PDF_CMAP_RANGE, 13734 },
	{ 0xc1e4, 0xc1e4, PDF_CMAP_SINGLE, 2286 },
	{ 0xc1e5, 0xc1e7, PDF_CMAP_RANGE, 13737 },
	{ 0xc1e8, 0xc1e8, PDF_CMAP_SINGLE, 2287 },
	{ 0xc1e9, 0xc1ef, PDF_CMAP_RANGE, 13740 },
	{ 0xc1f0, 0xc1f1, PDF_CMAP_RANGE, 2288 },
	{ 0xc1f2, 0xc1f3, PDF_CMAP_TABLE, 2708 },
	{ 0xc1f4, 0xc1fb, PDF_CMAP_RANGE, 13748 },
	{ 0xc1fc, 0xc1fd, PDF_CMAP_RANGE, 2291 },
	{ 0xc1fe, 0xc1ff, PDF_CMAP_RANGE, 13756 },
	{ 0xc200, 0xc200, PDF_CMAP_SINGLE, 2293 },
	{ 0xc201, 0xc203, PDF_CMAP_RANGE, 13758 },
	{ 0xc204, 0xc204, PDF_CMAP_SINGLE, 2294 },
	{ 0xc205, 0xc20b, PDF_CMAP_RANGE, 13761 },
	{ 0xc20c, 0xc20d, PDF_CMAP_RANGE, 2295 },
	{ 0xc20e, 0xc211, PDF_CMAP_TABLE, 2710 },
	{ 0xc212, 0xc217, PDF_CMAP_RANGE, 13770 },
	{ 0xc218, 0xc219, PDF_CMAP_RANGE, 2299 },
	{ 0xc21a, 0xc21b, PDF_CMAP_RANGE, 13776 },
	{ 0xc21c, 0xc21c, PDF_CMAP_SINGLE, 2301 },
	{ 0xc21d, 0xc21e, PDF_CMAP_RANGE, 13778 },
	{ 0xc21f, 0xc220, PDF_CMAP_RANGE, 2302 },
	{ 0xc221, 0xc227, PDF_CMAP_RANGE, 13780 },
	{ 0xc228, 0xc229, PDF_CMAP_RANGE, 2304 },
	{ 0xc22a, 0xc230, PDF_CMAP_TABLE, 2714 },
	{ 0xc231, 0xc232, PDF_CMAP_RANGE, 2309 },
	{ 0xc233, 0xc234, PDF_CMAP_TABLE, 2721 },
	{ 0xc235, 0xc247, PDF_CMAP_RANGE, 13792 },
	{ 0xc248, 0xc248, PDF_CMAP_SINGLE, 2312 },
	{ 0xc249, 0xc24f, PDF_CMAP_RANGE, 13811 },
	{ 0xc250, 0xc251, PDF_CMAP_RANGE, 2313 },
	{ 0xc252, 0xc253, PDF_CMAP_RANGE, 13818 },
	{ 0xc254, 0xc254, PDF_CMAP_SINGLE, 2315 },
	{ 0xc255, 0xc257, PDF_CMAP_RANGE, 13820 },
	{ 0xc258, 0xc258, PDF_CMAP_SINGLE, 2316 },
	{ 0xc259, 0xc25f, PDF_CMAP_RANGE, 13823 },
	{ 0xc260, 0xc260, PDF_CMAP_SINGLE, 2317 },
	{ 0xc261, 0xc264, PDF_CMAP_RANGE, 13830 },
	{ 0xc265, 0xc265, PDF_CMAP_SINGLE, 2318 },
	{ 0xc266, 0xc26b, PDF_CMAP_RANGE, 13834 },
	{ 0xc26c, 0xc26d, PDF_CMAP_RANGE, 2319 },
	{ 0xc26e, 0xc26f, PDF_CMAP_RANGE, 13840 },
	{ 0xc270, 0xc270, PDF_CMAP_SINGLE, 2321 },
	{ 0xc271, 0xc273, PDF_CMAP_RANGE, 13842 },
	{ 0xc274, 0xc274, PDF_CMAP_SINGLE, 2322 },
	{ 0xc275, 0xc27b, PDF_CMAP_RANGE, 13845 },
	{ 0xc27c, 0xc27d, PDF_CMAP_RANGE, 2323 },
	{ 0xc27e, 0xc281, PDF_CMAP_TABLE, 2723 },
	{ 0xc282, 0xc287, PDF_CMAP_RANGE, 13854 },
	{ 0xc288, 0xc289, PDF_CMAP_RANGE, 2327 },
	{ 0xc28a, 0xc28f, PDF_CMAP_RANGE, 13860 },
	{ 0xc290, 0xc290, PDF_CMAP_SINGLE, 2329 },
	{ 0xc291, 0xc297, PDF_CMAP_RANGE, 13866 },
	{ 0xc298, 0xc298, PDF_CMAP_SINGLE, 2330 },
	{ 0xc299, 0xc29a, PDF_CMAP_RANGE, 13873 },
	{ 0xc29b, 0xc29d, PDF_CMAP_TABLE, 2727 },
	{ 0xc29e, 0xc2a3, PDF_CMAP_RANGE, 13876 },
	{ 0xc2a4, 0xc2a5, PDF_CMAP_RANGE, 2333 },
	{ 0xc2a6, 0xc2a7, PDF_CMAP_RANGE, 13882 },
	{ 0xc2a8, 0xc2a8, PDF_CMAP_SINGLE, 2335 },
	{ 0xc2a9, 0xc2ab, PDF_CMAP_RANGE, 13884 },
	{ 0xc2ac, 0xc2ad, PDF_CMAP_RANGE, 2336 },
	{ 0xc2ae, 0xc2b3, PDF_CMAP_RANGE, 13887 },
	{ 0xc2b4, 0xc2b5, PDF_CMAP_RANGE, 2338 },
	{ 0xc2b6, 0xc2b9, PDF_CMAP_TABLE, 2730 },
	{ 0xc2ba, 0xc2db, PDF_CMAP_RANGE, 13895 },
	{ 0xc2dc, 0xc2dd, PDF_CMAP_RANGE, 2342 },
	{ 0xc2de, 0xc2df, PDF_CMAP_RANGE, 13929 },
	{ 0xc2e0, 0xc2e0, PDF_CMAP_SINGLE, 2344 },
	{ 0xc2e1, 0xc2e2, PDF_CMAP_RANGE, 13931 },
	{ 0xc2e3, 0xc2e4, PDF_CMAP_RANGE, 2345 },
	{ 0xc2e5, 0xc2ea, PDF_CMAP_RANGE, 13933 },
	{ 0xc2eb, 0xc2ed, PDF_CMAP_RANGE, 2347 },
	{ 0xc2ee, 0xc2f1, PDF_CMAP_TABLE, 2734 },
	{ 0xc2f2, 0xc2f5, PDF_CMAP_RANGE, 13941 },
	{ 0xc2f6, 0xc2f7, PDF_CMAP_TABLE, 2738 },
	{ 0xc2f8, 0xc2f9, PDF_CMAP_RANGE, 2353 },
	{ 0xc2fa, 0xc2fa, PDF_CMAP_SINGLE, 13946 },
	{ 0xc2fb, 0xc2fc, PDF_CMAP_RANGE, 2355 },
	{ 0xc2fd, 0xc2ff, PDF_CMAP_RANGE, 13947 },
	{ 0xc300, 0xc300, PDF_CMAP_SINGLE, 2357 },
	{ 0xc301, 0xc307, PDF_CMAP_RANGE, 13950 },
	{ 0xc308, 0xc309, PDF_CMAP_RANGE, 2358 },
	{ 0xc30a, 0xc30b, PDF_CMAP_RANGE, 13957 },
	{ 0xc30c, 0xc30d, PDF_CMAP_RANGE, 2360 },
	{ 0xc30e, 0xc312, PDF_CMAP_RANGE, 13959 },
	{ 0xc313, 0xc315, PDF_CMAP_RANGE, 2362 },
	{ 0xc316, 0xc317, PDF_CMAP_RANGE, 13964 },
	{ 0xc318, 0xc318, PDF_CMAP_SINGLE, 2365 },
	{ 0xc319, 0xc31b, PDF_CMAP_RANGE, 13966 },
	{ 0xc31c, 0xc31c, PDF_CMAP_SINGLE, 2366 },
	{ 0xc31d, 0xc323, PDF_CMAP_RANGE, 13969 },
	{ 0xc324, 0xc325, PDF_CMAP_RANGE, 2367 },
	{ 0xc326, 0xc327, PDF_CMAP_RANGE, 13976 },
	{ 0xc328, 0xc329, PDF_CMAP_RANGE, 2369 },
	{ 0xc32a, 0xc344, PDF_CMAP_RANGE, 13978 },
	{ 0xc345, 0xc345, PDF_CMAP_SINGLE, 2371 },
	{ 0xc346, 0xc367, PDF_CMAP_RANGE, 14005 },
	{ 0xc368, 0xc369, PDF_CMAP_RANGE, 2372 },
	{ 0xc36a, 0xc36b, PDF_CMAP_RANGE, 14039 },
	{ 0xc36c, 0xc36c, PDF_CMAP_SINGLE, 2374 },
	{ 0xc36d, 0xc36f, PDF_CMAP_RANGE, 14041 },
	{ 0xc370, 0xc372, PDF_CMAP_TABLE, 2740 },
	{ 0xc373, 0xc377, PDF_CMAP_RANGE, 14045 },
	{ 0xc378, 0xc379, PDF_CMAP_RANGE, 2377 },
	{ 0xc37a, 0xc37b, PDF_CMAP_RANGE, 14050 },
	{ 0xc37c, 0xc37d, PDF_CMAP_RANGE, 2379 },
	{ 0xc37e, 0xc383, PDF_CMAP_RANGE, 14052 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 2381 },
	{ 0xc385, 0xc387, PDF_CMAP_RANGE, 14058 },
	{ 0xc388, 0xc388, PDF_CMAP_SINGLE, 2382 },
	{ 0xc389, 0xc38b, PDF_CMAP_RANGE, 14061 },
	{ 0xc38c, 0xc38c, PDF_CMAP_SINGLE, 2383 },
	{ 0xc38d, 0xc3bf, PDF_CMAP_RANGE, 14064 },
	{ 0xc3c0, 0xc3c0, PDF_CMAP_SINGLE, 2384 },
	{ 0xc3c1, 0xc3d7, PDF_CMAP_RANGE, 14115 },
	{ 0xc3d8, 0xc3d9, PDF_CMAP_RANGE, 2385 },
	{ 0xc3da, 0xc3db, PDF_CMAP_RANGE, 14138 },
	{ 0xc3dc, 0xc3dc, PDF_CMAP_SINGLE, 2387 },
	{ 0xc3dd, 0xc3de, PDF_CMAP_RANGE, 14140 },
	{ 0xc3df, 0xc3e0, PDF_CMAP_RANGE, 2388 },
	{ 0xc3e1, 0xc3e2, PDF_CMAP_TABLE, 2743 },
	{ 0xc3e3, 0xc3e7, PDF_CMAP_RANGE, 14143 },
	{ 0xc3e8, 0xc3e9, PDF_CMAP_RANGE, 2391 },
	{ 0xc3ea, 0xc3ec, PDF_CMAP_RANGE, 14148 },
	{ 0xc3ed, 0xc3ed, PDF_CMAP_SINGLE, 2393 },
	{ 0xc3ee, 0xc3f3, PDF_CMAP_RANGE, 14151 },
	{ 0xc3f4, 0xc3f5, PDF_CMAP_RANGE, 2394 },
	{ 0xc3f6, 0xc3f7, PDF_CMAP_RANGE, 14157 },
	{ 0xc3f8, 0xc3f8, PDF_CMAP_SINGLE, 2396 },
	{ 0xc3f9, 0xc407, PDF_CMAP_RANGE, 14159 },
	{ 0xc408, 0xc408, PDF_CMAP_SINGLE, 2397 },
	{ 0xc409, 0xc40f, PDF_CMAP_RANGE, 14174 },
	{ 0xc410, 0xc410, PDF_CMAP_SINGLE, 2398 },
	{ 0xc411, 0xc423, PDF_CMAP_RANGE, 14181 },
	{ 0xc424, 0xc424, PDF_CMAP_SINGLE, 2399 },
	{ 0xc425, 0xc42b, PDF_CMAP_RANGE, 14200 },
	{ 0xc42c, 0xc42c, PDF_CMAP_SINGLE, 2400 },
	{ 0xc42d, 0xc42f, PDF_CMAP_RANGE, 14207 },
	{ 0xc430, 0xc430, PDF_CMAP_SINGLE, 2401 },
	{ 0xc431, 0xc433, PDF_CMAP_RANGE, 14210 },
	{ 0xc434, 0xc434, PDF_CMAP_SINGLE, 2402 },
	{ 0xc435, 0xc43b, PDF_CMAP_RANGE, 14213 },
	{ 0xc43c, 0xc43d, PDF_CMAP_RANGE, 2403 },
	{ 0xc43e, 0xc447, PDF_CMAP_RANGE, 14220 },
	{ 0xc448, 0xc448, PDF_CMAP_SINGLE, 2405 },
	{ 0xc449, 0xc463, PDF_CMAP_RANGE, 14230 },
	{ 0xc464, 0xc465, PDF_CMAP_RANGE, 2406 },
	{ 0xc466, 0xc467, PDF_CMAP_RANGE, 14257 },
	{ 0xc468, 0xc468, PDF_CMAP_SINGLE, 2408 },
	{ 0xc469, 0xc46b, PDF_CMAP_RANGE, 14259 },
	{ 0xc46c, 0xc46c, PDF_CMAP_SINGLE, 2409 },
	{ 0xc46d, 0xc473, PDF_CMAP_RANGE, 14262 },
	{ 0xc474, 0xc475, PDF_CMAP_RANGE, 2410 },
	{ 0xc476, 0xc478, PDF_CMAP_RANGE, 14269 },
	{ 0xc479, 0xc479, PDF_CMAP_SINGLE, 2412 },
	{ 0xc47a, 0xc47f, PDF_CMAP_RANGE, 14272 },
	{ 0xc480, 0xc480, PDF_CMAP_SINGLE, 2413 },
	{ 0xc481, 0xc493, PDF_CMAP_RANGE, 14278 },
	{ 0xc494, 0xc494, PDF_CMAP_SINGLE, 2414 },
	{ 0xc495, 0xc49b, PDF_CMAP_RANGE, 14297 },
	{ 0xc49c, 0xc49c, PDF_CMAP_SINGLE, 2415 },
	{ 0xc49d, 0xc4b7, PDF_CMAP_RANGE, 14304 },
	{ 0xc4b8, 0xc4b8, PDF_CMAP_SINGLE, 2416 },
	{ 0xc4b9, 0xc4bb, PDF_CMAP_RANGE, 14331 },
	{ 0xc4bc, 0xc4bc, PDF_CMAP_SINGLE, 2417 },
	{ 0xc4bd, 0xc4e8, PDF_CMAP_RANGE, 14334 },
	{ 0xc4e9, 0xc4e9, PDF_CMAP_SINGLE, 2418 },
	{ 0xc4ea, 0xc4ef, PDF_CMAP_RANGE, 14378 },
	{ 0xc4f0, 0xc4f1, PDF_CMAP_RANGE, 2419 },
	{ 0xc4f2, 0xc4f3, PDF_CMAP_RANGE, 14384 },
	{ 0xc4f4, 0xc4f4, PDF_CMAP_SINGLE, 2421 },
	{ 0xc4f5, 0xc4f7, PDF_CMAP_RANGE, 14386 },
	{ 0xc4f8, 0xc4fa, PDF_CMAP_TABLE, 2745 },
	{ 0xc4fb, 0xc4fe, PDF_CMAP_RANGE, 14390 },
	{ 0xc4ff, 0xc501, PDF_CMAP_RANGE, 2424 },
	{ 0xc502, 0xc50b, PDF_CMAP_RANGE, 14394 },
	{ 0xc50c, 0xc50c, PDF_CMAP_SINGLE, 2427 },
	{ 0xc50d, 0xc50f, PDF_CMAP_RANGE, 14404 },
	{ 0xc510, 0xc510, PDF_CMAP_SINGLE, 2428 },
	{ 0xc511, 0xc513, PDF_CMAP_RANGE, 14407 },
	{ 0xc514, 0xc514, PDF_CMAP_SINGLE, 2429 },
	{ 0xc515, 0xc51b, PDF_CMAP_RANGE, 14410 },
	{ 0xc51c, 0xc51c, PDF_CMAP_SINGLE, 2430 },
	{ 0xc51d, 0xc527, PDF_CMAP_RANGE, 14417 },
	{ 0xc528, 0xc529, PDF_CMAP_RANGE, 2431 },
	{ 0xc52a, 0xc52b, PDF_CMAP_RANGE, 14428 },
	{ 0xc52c, 0xc52c, PDF_CMAP_SINGLE, 2433 },
	{ 0xc52d, 0xc52f, PDF_CMAP_RANGE, 14430 },
	{ 0xc530, 0xc530, PDF_CMAP_SINGLE, 2434 },
	{ 0xc531, 0xc537, PDF_CMAP_RANGE, 14433 },
	{ 0xc538, 0xc539, PDF_CMAP_RANGE, 2435 },
	{ 0xc53a, 0xc53d, PDF_CMAP_TABLE, 2748 },
	{ 0xc53e, 0xc543, PDF_CMAP_RANGE, 14442 },
	{ 0xc544, 0xc545, PDF_CMAP_RANGE, 2439 },
	{ 0xc546, 0xc547, PDF_CMAP_RANGE, 14448 },
	{ 0xc548, 0xc54a, PDF_CMAP_RANGE, 2441 },
	{ 0xc54b, 0xc54b, PDF_CMAP_SINGLE, 14450 },
	{ 0xc54c, 0xc54e, PDF_CMAP_RANGE, 2444 },
	{ 0xc54f, 0xc552, PDF_CMAP_RANGE, 14451 },
	{ 0xc553, 0xc555, PDF_CMAP_RANGE, 2447 },
	{ 0xc556, 0xc556, PDF_CMAP_SINGLE, 14455 },
	{ 0xc557, 0xc559, PDF_CMAP_RANGE, 2450 },
	{ 0xc55a, 0xc55c, PDF_CMAP_RANGE, 14456 },
	{ 0xc55d, 0xc55e, PDF_CMAP_RANGE, 2453 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 14459 },
	{ 0xc560, 0xc561, PDF_CMAP_RANGE, 2455 },
	{ 0xc562, 0xc563, PDF_CMAP_RANGE, 14460 },
	{ 0xc564, 0xc564, PDF_CMAP_SINGLE, 2457 },
	{ 0xc565, 0xc567, PDF_CMAP_RANGE, 14462 },
	{ 0xc568, 0xc568, PDF_CMAP_SINGLE, 2458 },
	{ 0xc569, 0xc56f, PDF_CMAP_RANGE, 14465 },
	{ 0xc570, 0xc571, PDF_CMAP_RANGE, 2459 },
	{ 0xc572, 0xc572, PDF_CMAP_SINGLE, 14472 },
	{ 0xc573, 0xc575, PDF_CMAP_RANGE, 2461 },
	{ 0xc576, 0xc57b, PDF_CMAP_RANGE, 14473 },
	{ 0xc57c, 0xc57d, PDF_CMAP_RANGE, 2464 },
	{ 0xc57e, 0xc57f, PDF_CMAP_RANGE, 14479 },
	{ 0xc580, 0xc580, PDF_CMAP_SINGLE, 2466 },
	{ 0xc581, 0xc583, PDF_CMAP_RANGE, 14481 },
	{ 0xc584, 0xc584, PDF_CMAP_SINGLE, 2467 },
	{ 0xc585, 0xc586, PDF_CMAP_RANGE, 14484 },
	{ 0xc587, 0xc587, PDF_CMAP_SINGLE, 2468 },
	{ 0xc588, 0xc58b, PDF_CMAP_RANGE, 14486 },
	{ 0xc58c, 0xc58d, PDF_CMAP_RANGE, 2469 },
	{ 0xc58e, 0xc591, PDF_CMAP_TABLE, 2752 },
	{ 0xc592, 0xc594, PDF_CMAP_RANGE, 14492 },
	{ 0xc595, 0xc596, PDF_CMAP_TABLE, 2756 },
	{ 0xc597, 0xc598, PDF_CMAP_RANGE, 2474 },
	{ 0xc599, 0xc59b, PDF_CMAP_RANGE, 14496 },
	{ 0xc59c, 0xc59c, PDF_CMAP_SINGLE, 2476 },
	{ 0xc59d, 0xc59f, PDF_CMAP_RANGE, 14499 },
	{ 0xc5a0, 0xc5a0, PDF_CMAP_SINGLE, 2477 },
	{ 0xc5a1, 0xc5a8, PDF_CMAP_RANGE, 14502 },
	{ 0xc5a9, 0xc5a9, PDF_CMAP_SINGLE, 2478 },
	{ 0xc5aa, 0xc5b3, PDF_CMAP_RANGE, 14510 },
	{ 0xc5b4, 0xc5b5, PDF_CMAP_RANGE, 2479 },
	{ 0xc5b6, 0xc5b7, PDF_CMAP_RANGE, 14520 },
	{ 0xc5b8, 0xc5b9, PDF_CMAP_RANGE, 2481 },
	{ 0xc5ba, 0xc5ba, PDF_CMAP_SINGLE, 14522 },
	{ 0xc5bb, 0xc5be, PDF_CMAP_RANGE, 2483 },
	{ 0xc5bf, 0xc5c3, PDF_CMAP_RANGE, 14523 },
	{ 0xc5c4, 0xc5ca, PDF_CMAP_RANGE, 2487 },
	{ 0xc5cb, 0xc5cf, PDF_CMAP_TABLE, 2758 },
	{ 0xc5d0, 0xc5d1, PDF_CMAP_RANGE, 2496 },
	{ 0xc5d2, 0xc5d3, PDF_CMAP_RANGE, 14531 },
	{ 0xc5d4, 0xc5d4, PDF_CMAP_SINGLE, 2498 },
	{ 0xc5d5, 0xc5d7, PDF_CMAP_RANGE, 14533 },
	{ 0xc5d8, 0xc5d8, PDF_CMAP_SINGLE, 2499 },
	{ 0xc5d9, 0xc5df, PDF_CMAP_RANGE, 14536 },
	{ 0xc5e0, 0xc5e1, PDF_CMAP_RANGE, 2500 },
	{ 0xc5e2, 0xc5e5, PDF_CMAP_TABLE, 2763 },
	{ 0xc5e6, 0xc5eb, PDF_CMAP_RANGE, 14545 },
	{ 0xc5ec, 0xc5ee, PDF_CMAP_RANGE, 2504 },
	{ 0xc5ef, 0xc5f0, PDF_CMAP_TABLE, 2767 },
	{ 0xc5f1, 0xc5f3, PDF_CMAP_RANGE, 14552 },
	{ 0xc5f4, 0xc5f5, PDF_CMAP_TABLE, 2769 },
	{ 0xc5f6, 0xc5f7, PDF_CMAP_RANGE, 2509 },
	{ 0xc5f8, 0xc5fb, PDF_CMAP_RANGE, 14556 },
	{ 0xc5fc, 0xc601, PDF_CMAP_RANGE, 2511 },
	{ 0xc602, 0xc604, PDF_CMAP_RANGE, 14560 },
	{ 0xc605, 0xc608, PDF_CMAP_RANGE, 2517 },
	{ 0xc609, 0xc60b, PDF_CMAP_RANGE, 14563 },
	{ 0xc60c, 0xc60c, PDF_CMAP_SINGLE, 2521 },
	{ 0xc60d, 0xc60f, PDF_CMAP_RANGE, 14566 },
	{ 0xc610, 0xc610, PDF_CMAP_SINGLE, 2522 },
	{ 0xc611, 0xc617, PDF_CMAP_RANGE, 14569 },
	{ 0xc618, 0xc619, PDF_CMAP_RANGE, 2523 },
	{ 0xc61a, 0xc61a, PDF_CMAP_SINGLE, 14576 },
	{ 0xc61b, 0xc61c, PDF_CMAP_RANGE, 2525 },
	{ 0xc61d, 0xc623, PDF_CMAP_RANGE, 14577 },
	{ 0xc624, 0xc625, PDF_CMAP_RANGE, 2527 },
	{ 0xc626, 0xc627, PDF_CMAP_RANGE, 14584 },
	{ 0xc628, 0xc628, PDF_CMAP_SINGLE, 2529 },
	{ 0xc629, 0xc62b, PDF_CMAP_RANGE, 14586 },
	{ 0xc62c, 0xc62e, PDF_CMAP_RANGE, 2530 },
	{ 0xc62f, 0xc630, PDF_CMAP_TABLE, 2771 },
	{ 0xc631, 0xc632, PDF_CMAP_RANGE, 14590 },
	{ 0xc633, 0xc635, PDF_CMAP_RANGE, 2534 },
	{ 0xc636, 0xc63b, PDF_CMAP_TABLE, 2773 },
	{ 0xc63c, 0xc63f, PDF_CMAP_RANGE, 14595 },
	{ 0xc640, 0xc641, PDF_CMAP_RANGE, 2540 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 14599 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 2542 },
	{ 0xc645, 0xc647, PDF_CMAP_RANGE, 14601 },
	{ 0xc648, 0xc648, PDF_CMAP_SINGLE, 2543 },
	{ 0xc649, 0xc64f, PDF_CMAP_RANGE, 14604 },
	{ 0xc650, 0xc651, PDF_CMAP_RANGE, 2544 },
	{ 0xc652, 0xc652, PDF_CMAP_SINGLE, 14611 },
	{ 0xc653, 0xc655, PDF_CMAP_RANGE, 2546 },
	{ 0xc656, 0xc65b, PDF_CMAP_RANGE, 14612 },
	{ 0xc65c, 0xc65d, PDF_CMAP_RANGE, 2549 },
	{ 0xc65e, 0xc65f, PDF_CMAP_RANGE, 14618 },
	{ 0xc660, 0xc660, PDF_CMAP_SINGLE, 2551 },
	{ 0xc661, 0xc66b, PDF_CMAP_RANGE, 14620 },
	{ 0xc66c, 0xc66c, PDF_CMAP_SINGLE, 2552 },
	{ 0xc66d, 0xc66e, PDF_CMAP_RANGE, 14631 },
	{ 0xc66f, 0xc671, PDF_CMAP_TABLE, 2779 },
	{ 0xc672, 0xc677, PDF_CMAP_RANGE, 14634 },
	{ 0xc678, 0xc679, PDF_CMAP_RANGE, 2555 },
	{ 0xc67a, 0xc67b, PDF_CMAP_RANGE, 14640 },
	{ 0xc67c, 0xc67c, PDF_CMAP_SINGLE, 2557 },
	{ 0xc67d, 0xc67f, PDF_CMAP_RANGE, 14642 },
	{ 0xc680, 0xc680, PDF_CMAP_SINGLE, 2558 },
	{ 0xc681, 0xc687, PDF_CMAP_RANGE, 14645 },
	{ 0xc688, 0xc689, PDF_CMAP_RANGE, 2559 },
	{ 0xc68a, 0xc68d, PDF_CMAP_TABLE, 2782 },
	{ 0xc68e, 0xc693, PDF_CMAP_RANGE, 14654 },
	{ 0xc694, 0xc695, PDF_CMAP_RANGE, 2563 },
	{ 0xc696, 0xc697, PDF_CMAP_RANGE, 14660 },
	{ 0xc698, 0xc698, PDF_CMAP_SINGLE, 2565 },
	{ 0xc699, 0xc69b, PDF_CMAP_RANGE, 14662 },
	{ 0xc69c, 0xc69c, PDF_CMAP_SINGLE, 2566 },
	{ 0xc69d, 0xc6a3, PDF_CMAP_RANGE, 14665 },
	{ 0xc6a4, 0xc6a5, PDF_CMAP_RANGE, 2567 },
	{ 0xc6a6, 0xc6a9, PDF_CMAP_TABLE, 2786 },
	{ 0xc6aa, 0xc6af, PDF_CMAP_RANGE, 14674 },
	{ 0xc6b0, 0xc6b1, PDF_CMAP_RANGE, 2571 },
	{ 0xc6b2, 0xc6b3, PDF_CMAP_RANGE, 14680 },
	{ 0xc6b4, 0xc6b4, PDF_CMAP_SINGLE, 2573 },
	{ 0xc6b5, 0xc6b7, PDF_CMAP_RANGE, 14682 },
	{ 0xc6b8, 0xc6ba, PDF_CMAP_RANGE, 2574 },
	{ 0xc6bb, 0xc6bf, PDF_CMAP_RANGE, 14685 },
	{ 0xc6c0, 0xc6c1, PDF_CMAP_RANGE, 2577 },
	{ 0xc6c2, 0xc6c5, PDF_CMAP_TABLE, 2790 },
	{ 0xc6c6, 0xc6cb, PDF_CMAP_RANGE, 14692 },
	{ 0xc6cc, 0xc6cd, PDF_CMAP_RANGE, 2581 },
	{ 0xc6ce, 0xc6cf, PDF_CMAP_RANGE, 14698 },
	{ 0xc6d0, 0xc6d0, PDF_CMAP_SINGLE, 2583 },
	{ 0xc6d1, 0xc6d3, PDF_CMAP_RANGE, 14700 },
	{ 0xc6d4, 0xc6d4, PDF_CMAP_SINGLE, 2584 },
	{ 0xc6d5, 0xc6db, PDF_CMAP_RANGE, 14703 },
	{ 0xc6dc, 0xc6dd, PDF_CMAP_RANGE, 2585 },
	{ 0xc6de, 0xc6df, PDF_CMAP_RANGE, 14710 },
	{ 0xc6e0, 0xc6e1, PDF_CMAP_RANGE, 2587 },
	{ 0xc6e2, 0xc6e7, PDF_CMAP_RANGE, 14712 },
	{ 0xc6e8, 0xc6e9, PDF_CMAP_RANGE, 2589 },
	{ 0xc6ea, 0xc6eb, PDF_CMAP_RANGE, 14718 },
	{ 0xc6ec, 0xc6ec, PDF_CMAP_SINGLE, 2591 },
	{ 0xc6ed, 0xc6ef, PDF_CMAP_RANGE, 14720 },
	{ 0xc6f0, 0xc6f0, PDF_CMAP_SINGLE, 2592 },
	{ 0xc6f1, 0xc6f7, PDF_CMAP_RANGE, 14723 },
	{ 0xc6f8, 0xc6f9, PDF_CMAP_RANGE, 2593 },
	{ 0xc6fa, 0xc6fc, PDF_CMAP_RANGE, 14730 },
	{ 0xc6fd, 0xc6fd, PDF_CMAP_SINGLE, 2595 },
	{ 0xc6fe, 0xc703, PDF_CMAP_RANGE, 14733 },
	{ 0xc704, 0xc705, PDF_CMAP_RANGE, 2596 },
	{ 0xc706, 0xc707, PDF_CMAP_RANGE, 14739 },
	{ 0xc708, 0xc708, PDF_CMAP_SINGLE, 2598 },
	{ 0xc709, 0xc70b, PDF_CMAP_RANGE, 14741 },
	{ 0xc70c, 0xc70c, PDF_CMAP_SINGLE, 2599 },
	{ 0xc70d, 0xc713, PDF_CMAP_RANGE, 14744 },
	{ 0xc714, 0xc715, PDF_CMAP_RANGE, 2600 },
	{ 0xc716, 0xc719, PDF_CMAP_TABLE, 2794 },
	{ 0xc71a, 0xc71f, PDF_CMAP_RANGE, 14753 },
	{ 0xc720, 0xc721, PDF_CMAP_RANGE, 2604 },
	{ 0xc722, 0xc723, PDF_CMAP_RANGE, 14759 },
	{ 0xc724, 0xc724, PDF_CMAP_SINGLE, 2606 },
	{ 0xc725, 0xc727, PDF_CMAP_RANGE, 14761 },
	{ 0xc728, 0xc728, PDF_CMAP_SINGLE, 2607 },
	{ 0xc729, 0xc72f, PDF_CMAP_RANGE, 14764 },
	{ 0xc730, 0xc731, PDF_CMAP_RANGE, 2608 },
	{ 0xc732, 0xc737, PDF_CMAP_TABLE, 2798 },
	{ 0xc738, 0xc73b, PDF_CMAP_RANGE, 14774 },
	{ 0xc73c, 0xc73d, PDF_CMAP_RANGE, 2613 },
	{ 0xc73e, 0xc73f, PDF_CMAP_RANGE, 14778 },
	{ 0xc740, 0xc740, PDF_CMAP_SINGLE, 2615 },
	{ 0xc741, 0xc743, PDF_CMAP_RANGE, 14780 },
	{ 0xc744, 0xc744, PDF_CMAP_SINGLE, 2616 },
	{ 0xc745, 0xc749, PDF_CMAP_RANGE, 14783 },
	{ 0xc74a, 0xc74b, PDF_CMAP_TABLE, 2804 },
	{ 0xc74c, 0xc74d, PDF_CMAP_RANGE, 2618 },
	{ 0xc74e, 0xc750, PDF_CMAP_TABLE, 2806 },
	{ 0xc751, 0xc758, PDF_CMAP_RANGE, 2621 },
	{ 0xc759, 0xc75b, PDF_CMAP_RANGE, 14791 },
	{ 0xc75c, 0xc75c, PDF_CMAP_SINGLE, 2629 },
	{ 0xc75d, 0xc75f, PDF_CMAP_RANGE, 14794 },
	{ 0xc760, 0xc760, PDF_CMAP_SINGLE, 2630 },
	{ 0xc761, 0xc767, PDF_CMAP_RANGE, 14797 },
	{ 0xc768, 0xc768, PDF_CMAP_SINGLE, 2631 },
	{ 0xc769, 0xc76a, PDF_CMAP_RANGE, 14804 },
	{ 0xc76b, 0xc76b, PDF_CMAP_SINGLE, 2632 },
	{ 0xc76c, 0xc773, PDF_CMAP_RANGE, 14806 },
	{ 0xc774, 0xc775, PDF_CMAP_RANGE, 2633 },
	{ 0xc776, 0xc777, PDF_CMAP_RANGE, 14814 },
	{ 0xc778, 0xc778, PDF_CMAP_SINGLE, 2635 },
	{ 0xc779, 0xc77b, PDF_CMAP_RANGE, 14816 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 2636 },
	{ 0xc77f, 0xc782, PDF_CMAP_RANGE, 14819 },
	{ 0xc783, 0xc785, PDF_CMAP_RANGE, 2639 },
	{ 0xc786, 0xc786, PDF_CMAP_SINGLE, 14823 },
	{ 0xc787, 0xc78a, PDF_CMAP_RANGE, 2642 },
	{ 0xc78b, 0xc78d, PDF_CMAP_RANGE, 14824 },
	{ 0xc78e, 0xc78f, PDF_CMAP_TABLE, 2809 },
	{ 0xc790, 0xc791, PDF_CMAP_RANGE, 2647 },
	{ 0xc792, 0xc793, PDF_CMAP_RANGE, 14828 },
	{ 0xc794, 0xc795, PDF_CMAP_TABLE, 2811 },
	{ 0xc796, 0xc798, PDF_CMAP_RANGE, 2650 },
	{ 0xc799, 0xc79a, PDF_CMAP_TABLE, 2813 },
	{ 0xc79b, 0xc79f, PDF_CMAP_RANGE, 14832 },
	{ 0xc7a0, 0xc7a1, PDF_CMAP_RANGE, 2654 },
	{ 0xc7a2, 0xc7a2, PDF_CMAP_SINGLE, 14837 },
	{ 0xc7a3, 0xc7a6, PDF_CMAP_RANGE, 2656 },
	{ 0xc7a7, 0xc7ab, PDF_CMAP_RANGE, 14838 },
	{ 0xc7ac, 0xc7ad, PDF_CMAP_RANGE, 2660 },
	{ 0xc7ae, 0xc7af, PDF_CMAP_RANGE, 14843 },
	{ 0xc7b0, 0xc7b0, PDF_CMAP_SINGLE, 2662 },
	{ 0xc7b1, 0xc7b3, PDF_CMAP_RANGE, 14845 },
	{ 0xc7b4, 0xc7b4, PDF_CMAP_SINGLE, 2663 },
	{ 0xc7b5, 0xc7bb, PDF_CMAP_RANGE, 14848 },
	{ 0xc7bc, 0xc7bd, PDF_CMAP_RANGE, 2664 },
	{ 0xc7be, 0xc7be, PDF_CMAP_SINGLE, 14855 },
	{ 0xc7bf, 0xc7c1, PDF_CMAP_RANGE, 2666 },
	{ 0xc7c2, 0xc7c7, PDF_CMAP_RANGE, 14856 },
	{ 0xc7c8, 0xc7c9, PDF_CMAP_RANGE, 2669 },
	{ 0xc7ca, 0xc7cb, PDF_CMAP_RANGE, 14862 },
	{ 0xc7cc, 0xc7d0, PDF_CMAP_TABLE, 2815 },
	{ 0xc7d1, 0xc7d7, PDF_CMAP_RANGE, 14866 },
	{ 0xc7d8, 0xc7d8, PDF_CMAP_SINGLE, 2674 },
	{ 0xc7d9, 0xc7dc, PDF_CMAP_RANGE, 14873 },
	{ 0xc7dd, 0xc7dd, PDF_CMAP_SINGLE, 2675 },
	{ 0xc7de, 0xc7e3, PDF_CMAP_RANGE, 14877 },
	{ 0xc7e4, 0xc7e4, PDF_CMAP_SINGLE, 2676 },
	{ 0xc7e5, 0xc7e7, PDF_CMAP_RANGE, 14883 },
	{ 0xc7e8, 0xc7e8, PDF_CMAP_SINGLE, 2677 },
	{ 0xc7e9, 0xc7eb, PDF_CMAP_RANGE, 14886 },
	{ 0xc7ec, 0xc7ec, PDF_CMAP_SINGLE, 2678 },
	{ 0xc7ed, 0xc7ff, PDF_CMAP_RANGE, 14889 },
	{ 0xc800, 0xc801, PDF_CMAP_RANGE, 2679 },
	{ 0xc802, 0xc803, PDF_CMAP_RANGE, 14908 },
	{ 0xc804, 0xc804, PDF_CMAP_SINGLE, 2681 },
	{ 0xc805, 0xc807, PDF_CMAP_RANGE, 14910 },
	{ 0xc808, 0xc80a, PDF_CMAP_TABLE, 2820 },
	{ 0xc80b, 0xc80f, PDF_CMAP_RANGE, 14914 },
	{ 0xc810, 0xc811, PDF_CMAP_RANGE, 2684 },
	{ 0xc812, 0xc814, PDF_CMAP_TABLE, 2823 },
	{ 0xc815, 0xc816, PDF_CMAP_RANGE, 2687 },
	{ 0xc817, 0xc81b, PDF_CMAP_RANGE, 14921 },
	{ 0xc81c, 0xc81d, PDF_CMAP_RANGE, 2689 },
	{ 0xc81e, 0xc81f, PDF_CMAP_RANGE, 14926 },
	{ 0xc820, 0xc820, PDF_CMAP_SINGLE, 2691 },
	{ 0xc821, 0xc823, PDF_CMAP_RANGE, 14928 },
	{ 0xc824, 0xc824, PDF_CMAP_SINGLE, 2692 },
	{ 0xc825, 0xc82b, PDF_CMAP_RANGE, 14931 },
	{ 0xc82c, 0xc82d, PDF_CMAP_RANGE, 2693 },
	{ 0xc82e, 0xc831, PDF_CMAP_TABLE, 2826 },
	{ 0xc832, 0xc837, PDF_CMAP_RANGE, 14940 },
	{ 0xc838, 0xc838, PDF_CMAP_SINGLE, 2697 },
	{ 0xc839, 0xc83b, PDF_CMAP_RANGE, 14946 },
	{ 0xc83c, 0xc83c, PDF_CMAP_SINGLE, 2698 },
	{ 0xc83d, 0xc83f, PDF_CMAP_RANGE, 14949 },
	{ 0xc840, 0xc840, PDF_CMAP_SINGLE, 2699 },
	{ 0xc841, 0xc847, PDF_CMAP_RANGE, 14952 },
	{ 0xc848, 0xc849, PDF_CMAP_RANGE, 2700 },
	{ 0xc84a, 0xc84b, PDF_CMAP_RANGE, 14959 },
	{ 0xc84c, 0xc84d, PDF_CMAP_RANGE, 2702 },
	{ 0xc84e, 0xc853, PDF_CMAP_RANGE, 14961 },
	{ 0xc854, 0xc854, PDF_CMAP_SINGLE, 2704 },
	{ 0xc855, 0xc86f, PDF_CMAP_RANGE, 14967 },
	{ 0xc870, 0xc871, PDF_CMAP_RANGE, 2705 },
	{ 0xc872, 0xc873, PDF_CMAP_RANGE, 14994 },
	{ 0xc874, 0xc874, PDF_CMAP_SINGLE, 2707 },
	{ 0xc875, 0xc877, PDF_CMAP_RANGE, 14996 },
	{ 0xc878, 0xc87a, PDF_CMAP_TABLE, 2830 },
	{ 0xc87b, 0xc87f, PDF_CMAP_RANGE, 15000 },
	{ 0xc880, 0xc881, PDF_CMAP_RANGE, 2710 },
	{ 0xc882, 0xc884, PDF_CMAP_TABLE, 2833 },
	{ 0xc885, 0xc887, PDF_CMAP_RANGE, 2713 },
	{ 0xc888, 0xc88a, PDF_CMAP_RANGE, 15007 },
	{ 0xc88b, 0xc88d, PDF_CMAP_RANGE, 2716 },
	{ 0xc88e, 0xc893, PDF_CMAP_RANGE, 15010 },
	{ 0xc894, 0xc894, PDF_CMAP_SINGLE, 2719 },
	{ 0xc895, 0xc89c, PDF_CMAP_RANGE, 15016 },
	{ 0xc89d, 0xc8a1, PDF_CMAP_TABLE, 2836 },
	{ 0xc8a2, 0xc8a7, PDF_CMAP_RANGE, 15026 },
	{ 0xc8a8, 0xc8a8, PDF_CMAP_SINGLE, 2723 },
	{ 0xc8a9, 0xc8bb, PDF_CMAP_RANGE, 15032 },
	{ 0xc8bc, 0xc8bd, PDF_CMAP_RANGE, 2724 },
	{ 0xc8be, 0xc8c3, PDF_CMAP_RANGE, 15051 },
	{ 0xc8c4, 0xc8c4, PDF_CMAP_SINGLE, 2726 },
	{ 0xc8c5, 0xc8c7, PDF_CMAP_RANGE, 15057 },
	{ 0xc8c8, 0xc8c8, PDF_CMAP_SINGLE, 2727 },
	{ 0xc8c9, 0xc8cb, PDF_CMAP_RANGE, 15060 },
	{ 0xc8cc, 0xc8cc, PDF_CMAP_SINGLE, 2728 },
	{ 0xc8cd, 0xc8d3, PDF_CMAP_RANGE, 15063 },
	{ 0xc8d4, 0xc8d5, PDF_CMAP_RANGE, 2729 },
	{ 0xc8d6, 0xc8d9, PDF_CMAP_TABLE, 2841 },
	{ 0xc8da, 0xc8df, PDF_CMAP_RANGE, 15072 },
	{ 0xc8e0, 0xc8e1, PDF_CMAP_RANGE, 2733 },
	{ 0xc8e2, 0xc8e3, PDF_CMAP_RANGE, 15078 },
	{ 0xc8e4, 0xc8e4, PDF_CMAP_SINGLE, 2735 },
	{ 0xc8e5, 0xc8f4, PDF_CMAP_RANGE, 15080 },
	{ 0xc8f5, 0xc8f5, PDF_CMAP_SINGLE, 2736 },
	{ 0xc8f6, 0xc8fb, PDF_CMAP_RANGE, 15096 },
	{ 0xc8fc, 0xc8fd, PDF_CMAP_RANGE, 2737 },
	{ 0xc8fe, 0xc8ff, PDF_CMAP_RANGE, 15102 },
	{ 0xc900, 0xc900, PDF_CMAP_SINGLE, 2739 },
	{ 0xc901, 0xc903, PDF_CMAP_RANGE, 15104 },
	{ 0xc904, 0xc906, PDF_CMAP_RANGE, 2740 },
	{ 0xc907, 0xc90b, PDF_CMAP_RANGE, 15107 },
	{ 0xc90c, 0xc90d, PDF_CMAP_RANGE, 2743 },
	{ 0xc90e, 0xc911, PDF_CMAP_TABLE, 2845 },
	{ 0xc912, 0xc917, PDF_CMAP_RANGE, 15114 },
	{ 0xc918, 0xc918, PDF_CMAP_SINGLE, 2747 },
	{ 0xc919, 0xc92b, PDF_CMAP_RANGE, 15120 },
	{ 0xc92c, 0xc92c, PDF_CMAP_SINGLE, 2748 },
	{ 0xc92d, 0xc933, PDF_CMAP_RANGE, 15139 },
	{ 0xc934, 0xc934, PDF_CMAP_SINGLE, 2749 },
	{ 0xc935, 0xc94f, PDF_CMAP_RANGE, 15146 },
	{ 0xc950, 0xc951, PDF_CMAP_RANGE, 2750 },
	{ 0xc952, 0xc953, PDF_CMAP_RANGE, 15173 },
	{ 0xc954, 0xc954, PDF_CMAP_SINGLE, 2752 },
	{ 0xc955, 0xc957, PDF_CMAP_RANGE, 15175 },
	{ 0xc958, 0xc958, PDF_CMAP_SINGLE, 2753 },
	{ 0xc959, 0xc95f, PDF_CMAP_RANGE, 15178 },
	{ 0xc960, 0xc961, PDF_CMAP_RANGE, 2754 },
	{ 0xc962, 0xc963, PDF_CMAP_TABLE, 2849 },
	{ 0xc964, 0xc96b, PDF_CMAP_RANGE, 15186 },
	{ 0xc96c, 0xc96c, PDF_CMAP_SINGLE, 2757 },
	{ 0xc96d, 0xc96f, PDF_CMAP_RANGE, 15194 },
	{ 0xc970, 0xc970, PDF_CMAP_SINGLE, 2758 },
	{ 0xc971, 0xc973, PDF_CMAP_RANGE, 15197 },
	{ 0xc974, 0xc974, PDF_CMAP_SINGLE, 2759 },
	{ 0xc975, 0xc97b, PDF_CMAP_RANGE, 15200 },
	{ 0xc97c, 0xc97c, PDF_CMAP_SINGLE, 2760 },
	{ 0xc97d, 0xc987, PDF_CMAP_RANGE, 15207 },
	{ 0xc988, 0xc989, PDF_CMAP_RANGE, 2761 },
	{ 0xc98a, 0xc98b, PDF_CMAP_RANGE, 15218 },
	{ 0xc98c, 0xc98c, PDF_CMAP_SINGLE, 2763 },
	{ 0xc98d, 0xc98f, PDF_CMAP_RANGE, 15220 },
	{ 0xc990, 0xc990, PDF_CMAP_SINGLE, 2764 },
	{ 0xc991, 0xc997, PDF_CMAP_RANGE, 15223 },
	{ 0xc998, 0xc999, PDF_CMAP_RANGE, 2765 },
	{ 0xc99a, 0xc99d, PDF_CMAP_TABLE, 2851 },
	{ 0xc99e, 0xc9bf, PDF_CMAP_RANGE, 15232 },
	{ 0xc9c0, 0xc9c1, PDF_CMAP_RANGE, 2769 },
	{ 0xc9c2, 0xc9c3, PDF_CMAP_RANGE, 15266 },
	{ 0xc9c4, 0xc9c4, PDF_CMAP_SINGLE, 2771 },
	{ 0xc9c5, 0xc9c6, PDF_CMAP_RANGE, 15268 },
	{ 0xc9c7, 0xc9c8, PDF_CMAP_RANGE, 2772 },
	{ 0xc9c9, 0xc9ca, PDF_CMAP_TABLE, 2855 },
	{ 0xc9cb, 0xc9cf, PDF_CMAP_RANGE, 15271 },
	{ 0xc9d0, 0xc9d1, PDF_CMAP_RANGE, 2775 },
	{ 0xc9d2, 0xc9d4, PDF_CMAP_TABLE, 2857 },
	{ 0xc9d5, 0xc9d6, PDF_CMAP_RANGE, 2778 },
	{ 0xc9d7, 0xc9d8, PDF_CMAP_RANGE, 15278 },
	{ 0xc9d9, 0xc9da, PDF_CMAP_RANGE, 2780 },
	{ 0xc9db, 0xc9db, PDF_CMAP_SINGLE, 15280 },
	{ 0xc9dc, 0xc9dd, PDF_CMAP_RANGE, 2782 },
	{ 0xc9de, 0xc9df, PDF_CMAP_RANGE, 15281 },
	{ 0xc9e0, 0xc9e4, PDF_CMAP_TABLE, 2860 },
	{ 0xc9e5, 0xc9e6, PDF_CMAP_RANGE, 15285 },
	{ 0xc9e7, 0xc9e7, PDF_CMAP_SINGLE, 2787 },
	{ 0xc9e8, 0xc9eb, PDF_CMAP_RANGE, 15287 },
	{ 0xc9ec, 0xc9ed, PDF_CMAP_RANGE, 2788 },
	{ 0xc9ee, 0xc9ee, PDF_CMAP_SINGLE, 15291 },
	{ 0xc9ef, 0xc9f1, PDF_CMAP_RANGE, 2790 },
	{ 0xc9f2, 0xc9f7, PDF_CMAP_RANGE, 15292 },
	{ 0xc9f8, 0xc9f9, PDF_CMAP_RANGE, 2793 },
	{ 0xc9fa, 0xc9fb, PDF_CMAP_RANGE, 15298 },
	{ 0xc9fc, 0xc9fc, PDF_CMAP_SINGLE, 2795 },
	{ 0xc9fd, 0xc9ff, PDF_CMAP_RANGE, 15300 },
	{ 0xca00, 0xca00, PDF_CMAP_SINGLE, 2796 },
	{ 0xca01, 0xca07, PDF_CMAP_RANGE, 15303 },
	{ 0xca08, 0xca09, PDF_CMAP_RANGE, 2797 },
	{ 0xca0a, 0xca0a, PDF_CMAP_SINGLE, 15310 },
	{ 0xca0b, 0xca0d, PDF_CMAP_RANGE, 2799 },
	{ 0xca0e, 0xca13, PDF_CMAP_RANGE, 15311 },
	{ 0xca14, 0xca14, PDF_CMAP_SINGLE, 2802 },
	{ 0xca15, 0xca17, PDF_CMAP_RANGE, 15317 },
	{ 0xca18, 0xca18, PDF_CMAP_SINGLE, 2803 },
	{ 0xca19, 0xca28, PDF_CMAP_RANGE, 15320 },
	{ 0xca29, 0xca29, PDF_CMAP_SINGLE, 2804 },
	{ 0xca2a, 0xca4b, PDF_CMAP_RANGE, 15336 },
	{ 0xca4c, 0xca4d, PDF_CMAP_RANGE, 2805 },
	{ 0xca4e, 0xca4f, PDF_CMAP_RANGE, 15370 },
	{ 0xca50, 0xca50, PDF_CMAP_SINGLE, 2807 },
	{ 0xca51, 0xca53, PDF_CMAP_RANGE, 15372 },
	{ 0xca54, 0xca54, PDF_CMAP_SINGLE, 2808 },
	{ 0xca55, 0xca5b, PDF_CMAP_RANGE, 15375 },
	{ 0xca5c, 0xca5d, PDF_CMAP_RANGE, 2809 },
	{ 0xca5e, 0xca5e, PDF_CMAP_SINGLE, 15382 },
	{ 0xca5f, 0xca61, PDF_CMAP_RANGE, 2811 },
	{ 0xca62, 0xca67, PDF_CMAP_RANGE, 15383 },
	{ 0xca68, 0xca68, PDF_CMAP_SINGLE, 2814 },
	{ 0xca69, 0xca7c, PDF_CMAP_RANGE, 15389 },
	{ 0xca7d, 0xca7d, PDF_CMAP_SINGLE, 2815 },
	{ 0xca7e, 0xca83, PDF_CMAP_RANGE, 15409 },
	{ 0xca84, 0xca84, PDF_CMAP_SINGLE, 2816 },
	{ 0xca85, 0xca97, PDF_CMAP_RANGE, 15415 },
	{ 0xca98, 0xca98, PDF_CMAP_SINGLE, 2817 },
	{ 0xca99, 0xcabb, PDF_CMAP_RANGE, 15434 },
	{ 0xcabc, 0xcabd, PDF_CMAP_RANGE, 2818 },
	{ 0xcabe, 0xcabf, PDF_CMAP_RANGE, 15469 },
	{ 0xcac0, 0xcac0, PDF_CMAP_SINGLE, 2820 },
	{ 0xcac1, 0xcac3, PDF_CMAP_RANGE, 15471 },
	{ 0xcac4, 0xcac4, PDF_CMAP_SINGLE, 2821 },
	{ 0xcac5, 0xcacb, PDF_CMAP_RANGE, 15474 },
	{ 0xcacc, 0xcacd, PDF_CMAP_RANGE, 2822 },
	{ 0xcace, 0xcad3, PDF_CMAP_TABLE, 2865 },
	{ 0xcad4, 0xcad7, PDF_CMAP_RANGE, 15484 },
	{ 0xcad8, 0xcad9, PDF_CMAP_RANGE, 2827 },
	{ 0xcada, 0xcadf, PDF_CMAP_RANGE, 15488 },
	{ 0xcae0, 0xcae0, PDF_CMAP_SINGLE, 2829 },
	{ 0xcae1, 0xcaeb, PDF_CMAP_RANGE, 15494 },
	{ 0xcaec, 0xcaec, PDF_CMAP_SINGLE, 2830 },
	{ 0xcaed, 0xcaf3, PDF_CMAP_RANGE, 15505 },
	{ 0xcaf4, 0xcaf4, PDF_CMAP_SINGLE, 2831 },
	{ 0xcaf5, 0xcb07, PDF_CMAP_RANGE, 15512 },
	{ 0xcb08, 0xcb08, PDF_CMAP_SINGLE, 2832 },
	{ 0xcb09, 0xcb0f, PDF_CMAP_RANGE, 15531 },
	{ 0xcb10, 0xcb10, PDF_CMAP_SINGLE, 2833 },
	{ 0xcb11, 0xcb13, PDF_CMAP_RANGE, 15538 },
	{ 0xcb14, 0xcb14, PDF_CMAP_SINGLE, 2834 },
	{ 0xcb15, 0xcb17, PDF_CMAP_RANGE, 15541 },
	{ 0xcb18, 0xcb18, PDF_CMAP_SINGLE, 2835 },
	{ 0xcb19, 0xcb1f, PDF_CMAP_RANGE, 15544 },
	{ 0xcb20, 0xcb21, PDF_CMAP_RANGE, 2836 },
	{ 0xcb22, 0xcb40, PDF_CMAP_RANGE, 15551 },
	{ 0xcb41, 0xcb41, PDF_CMAP_SINGLE, 2838 },
	{ 0xcb42, 0xcb47, PDF_CMAP_RANGE, 15582 },
	{ 0xcb48, 0xcb49, PDF_CMAP_RANGE, 2839 },
	{ 0xcb4a, 0xcb4b, PDF_CMAP_RANGE, 15588 },
	{ 0xcb4c, 0xcb4c, PDF_CMAP_SINGLE, 2841 },
	{ 0xcb4d, 0xcb4f, PDF_CMAP_RANGE, 15590 },
	{ 0xcb50, 0xcb50, PDF_CMAP_SINGLE, 2842 },
	{ 0xcb51, 0xcb57, PDF_CMAP_RANGE, 15593 },
	{ 0xcb58, 0xcb59, PDF_CMAP_RANGE, 2843 },
	{ 0xcb5a, 0xcb5c, PDF_CMAP_RANGE, 15600 },
	{ 0xcb5d, 0xcb5d, PDF_CMAP_SINGLE, 2845 },
	{ 0xcb5e, 0xcb63, PDF_CMAP_RANGE, 15603 },
	{ 0xcb64, 0xcb64, PDF_CMAP_SINGLE, 2846 },
	{ 0xcb65, 0xcb77, PDF_CMAP_RANGE, 15609 },
	{ 0xcb78, 0xcb79, PDF_CMAP_RANGE, 2847 },
	{ 0xcb7a, 0xcb9b, PDF_CMAP_RANGE, 15628 },
	{ 0xcb9c, 0xcb9c, PDF_CMAP_SINGLE, 2849 },
	{ 0xcb9d, 0xcbb7, PDF_CMAP_RANGE, 15662 },
	{ 0xcbb8, 0xcbb8, PDF_CMAP_SINGLE, 2850 },
	{ 0xcbb9, 0xcbd3, PDF_CMAP_RANGE, 15689 },
	{ 0xcbd4, 0xcbd4, PDF_CMAP_SINGLE, 2851 },
	{ 0xcbd5, 0xcbe3, PDF_CMAP_RANGE, 15716 },
	{ 0xcbe4, 0xcbe4, PDF_CMAP_SINGLE, 2852 },
	{ 0xcbe5, 0xcbe6, PDF_CMAP_RANGE, 15731 },
	{ 0xcbe7, 0xcbe9, PDF_CMAP_TABLE, 2871 },
	{ 0xcbea, 0xcc0b, PDF_CMAP_RANGE, 15734 },
	{ 0xcc0c, 0xcc0d, PDF_CMAP_RANGE, 2855 },
	{ 0xcc0e, 0xcc0f, PDF_CMAP_RANGE, 15768 },
	{ 0xcc10, 0xcc10, PDF_CMAP_SINGLE, 2857 },
	{ 0xcc11, 0xcc13, PDF_CMAP_RANGE, 15770 },
	{ 0xcc14, 0xcc14, PDF_CMAP_SINGLE, 2858 },
	{ 0xcc15, 0xcc1b, PDF_CMAP_RANGE, 15773 },
	{ 0xcc1c, 0xcc1d, PDF_CMAP_RANGE, 2859 },
	{ 0xcc1e, 0xcc20, PDF_CMAP_RANGE, 15780 },
	{ 0xcc21, 0xcc22, PDF_CMAP_RANGE, 2861 },
	{ 0xcc23, 0xcc26, PDF_CMAP_RANGE, 15783 },
	{ 0xcc27, 0xcc29, PDF_CMAP_RANGE, 2863 },
	{ 0xcc2a, 0xcc2b, PDF_CMAP_RANGE, 15787 },
	{ 0xcc2c, 0xcc30, PDF_CMAP_TABLE, 2874 },
	{ 0xcc31, 0xcc37, PDF_CMAP_RANGE, 15791 },
	{ 0xcc38, 0xcc39, PDF_CMAP_RANGE, 2869 },
	{ 0xcc3a, 0xcc3a, PDF_CMAP_SINGLE, 15798 },
	{ 0xcc3b, 0xcc3e, PDF_CMAP_RANGE, 2871 },
	{ 0xcc3f, 0xcc43, PDF_CMAP_RANGE, 15799 },
	{ 0xcc44, 0xcc45, PDF_CMAP_RANGE, 2875 },
	{ 0xcc46, 0xcc47, PDF_CMAP_RANGE, 15804 },
	{ 0xcc48, 0xcc48, PDF_CMAP_SINGLE, 2877 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_RANGE, 15806 },
	{ 0xcc4c, 0xcc4c, PDF_CMAP_SINGLE, 2878 },
	{ 0xcc4d, 0xcc53, PDF_CMAP_RANGE, 15809 },
	{ 0xcc54, 0xcc55, PDF_CMAP_RANGE, 2879 },
	{ 0xcc56, 0xcc56, PDF_CMAP_SINGLE, 15816 },
	{ 0xcc57, 0xcc59, PDF_CMAP_RANGE, 2881 },
	{ 0xcc5a, 0xcc5f, PDF_CMAP_RANGE, 15817 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 2884 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 15823 },
	{ 0xcc64, 0xcc68, PDF_CMAP_TABLE, 2879 },
	{ 0xcc69, 0xcc6f, PDF_CMAP_RANGE, 15828 },
	{ 0xcc70, 0xcc70, PDF_CMAP_SINGLE, 2888 },
	{ 0xcc71, 0xcc74, PDF_CMAP_RANGE, 15835 },
	{ 0xcc75, 0xcc75, PDF_CMAP_SINGLE, 2889 },
	{ 0xcc76, 0xcc97, PDF_CMAP_RANGE, 15839 },
	{ 0xcc98, 0xcc99, PDF_CMAP_RANGE, 2890 },
	{ 0xcc9a, 0xcc9b, PDF_CMAP_RANGE, 15873 },
	{ 0xcc9c, 0xcc9c, PDF_CMAP_SINGLE, 2892 },
	{ 0xcc9d, 0xcc9f, PDF_CMAP_RANGE, 15875 },
	{ 0xcca0, 0xcca0, PDF_CMAP_SINGLE, 2893 },
	{ 0xcca1, 0xcca7, PDF_CMAP_RANGE, 15878 },
	{ 0xcca8, 0xcca9, PDF_CMAP_RANGE, 2894 },
	{ 0xccaa, 0xccaa, PDF_CMAP_SINGLE, 15885 },
	{ 0xccab, 0xccad, PDF_CMAP_RANGE, 2896 },
	{ 0xccae, 0xccb3, PDF_CMAP_RANGE, 15886 },
	{ 0xccb4, 0xccb5, PDF_CMAP_RANGE, 2899 },
	{ 0xccb6, 0xccb7, PDF_CMAP_RANGE, 15892 },
	{ 0xccb8, 0xccb8, PDF_CMAP_SINGLE, 2901 },
	{ 0xccb9, 0xccbb, PDF_CMAP_RANGE, 15894 },
	{ 0xccbc, 0xccbc, PDF_CMAP_SINGLE, 2902 },
	{ 0xccbd, 0xccc3, PDF_CMAP_RANGE, 15897 },
	{ 0xccc4, 0xccc5, PDF_CMAP_RANGE, 2903 },
	{ 0xccc6, 0xccc9, PDF_CMAP_TABLE, 2884 },
	{ 0xccca, 0xcccf, PDF_CMAP_RANGE, 15906 },
	{ 0xccd0, 0xccd0, PDF_CMAP_SINGLE, 2907 },
	{ 0xccd1, 0xccd3, PDF_CMAP_RANGE, 15912 },
	{ 0xccd4, 0xccd4, PDF_CMAP_SINGLE, 2908 },
	{ 0xccd5, 0xcce3, PDF_CMAP_RANGE, 15915 },
	{ 0xcce4, 0xcce4, PDF_CMAP_SINGLE, 2909 },
	{ 0xcce5, 0xcceb, PDF_CMAP_RANGE, 15930 },
	{ 0xccec, 0xccec, PDF_CMAP_SINGLE, 2910 },
	{ 0xcced, 0xccef, PDF_CMAP_RANGE, 15937 },
	{ 0xccf0, 0xccf0, PDF_CMAP_SINGLE, 2911 },
	{ 0xccf1, 0xcd00, PDF_CMAP_RANGE, 15940 },
	{ 0xcd01, 0xcd01, PDF_CMAP_SINGLE, 2912 },
	{ 0xcd02, 0xcd07, PDF_CMAP_RANGE, 15956 },
	{ 0xcd08, 0xcd09, PDF_CMAP_RANGE, 2913 },
	{ 0xcd0a, 0xcd0b, PDF_CMAP_RANGE, 15962 },
	{ 0xcd0c, 0xcd0c, PDF_CMAP_SINGLE, 2915 },
	{ 0xcd0d, 0xcd0f, PDF_CMAP_RANGE, 15964 },
	{ 0xcd10, 0xcd10, PDF_CMAP_SINGLE, 2916 },
	{ 0xcd11, 0xcd17, PDF_CMAP_RANGE, 15967 },
	{ 0xcd18, 0xcd19, PDF_CMAP_RANGE, 2917 },
	{ 0xcd1a, 0xcd1d, PDF_CMAP_TABLE, 2888 },
	{ 0xcd1e, 0xcd23, PDF_CMAP_RANGE, 15976 },
	{ 0xcd24, 0xcd24, PDF_CMAP_SINGLE, 2921 },
	{ 0xcd25, 0xcd27, PDF_CMAP_RANGE, 15982 },
	{ 0xcd28, 0xcd28, PDF_CMAP_SINGLE, 2922 },
	{ 0xcd29, 0xcd2b, PDF_CMAP_RANGE, 15985 },
	{ 0xcd2c, 0xcd2c, PDF_CMAP_SINGLE, 2923 },
	{ 0xcd2d, 0xcd38, PDF_CMAP_RANGE, 15988 },
	{ 0xcd39, 0xcd39, PDF_CMAP_SINGLE, 2924 },
	{ 0xcd3a, 0xcd5b, PDF_CMAP_RANGE, 16000 },
	{ 0xcd5c, 0xcd5c, PDF_CMAP_SINGLE, 2925 },
	{ 0xcd5d, 0xcd5f, PDF_CMAP_RANGE, 16034 },
	{ 0xcd60, 0xcd60, PDF_CMAP_SINGLE, 2926 },
	{ 0xcd61, 0xcd63, PDF_CMAP_RANGE, 16037 },
	{ 0xcd64, 0xcd64, PDF_CMAP_SINGLE, 2927 },
	{ 0xcd65, 0xcd6b, PDF_CMAP_RANGE, 16040 },
	{ 0xcd6c, 0xcd6d, PDF_CMAP_RANGE, 2928 },
	{ 0xcd6e, 0xcd71, PDF_CMAP_TABLE, 2892 },
	{ 0xcd72, 0xcd77, PDF_CMAP_RANGE, 16049 },
	{ 0xcd78, 0xcd78, PDF_CMAP_SINGLE, 2932 },
	{ 0xcd79, 0xcd87, PDF_CMAP_RANGE, 16055 },
	{ 0xcd88, 0xcd88, PDF_CMAP_SINGLE, 2933 },
	{ 0xcd89, 0xcd93, PDF_CMAP_RANGE, 16070 },
	{ 0xcd94, 0xcd95, PDF_CMAP_RANGE, 2934 },
	{ 0xcd96, 0xcd97, PDF_CMAP_RANGE, 16081 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 2936 },
	{ 0xcd99, 0xcd9b, PDF_CMAP_RANGE, 16083 },
	{ 0xcd9c, 0xcd9c, PDF_CMAP_SINGLE, 2937 },
	{ 0xcd9d, 0xcda3, PDF_CMAP_RANGE, 16086 },
	{ 0xcda4, 0xcda5, PDF_CMAP_RANGE, 2938 },
	{ 0xcda6, 0xcda9, PDF_CMAP_TABLE, 2896 },
	{ 0xcdaa, 0xcdaf, PDF_CMAP_RANGE, 16095 },
	{ 0xcdb0, 0xcdb0, PDF_CMAP_SINGLE, 2942 },
	{ 0xcdb1, 0xcdc3, PDF_CMAP_RANGE, 16101 },
	{ 0xcdc4, 0xcdc4, PDF_CMAP_SINGLE, 2943 },
	{ 0xcdc5, 0xcdcb, PDF_CMAP_RANGE, 16120 },
	{ 0xcdcc, 0xcdcc, PDF_CMAP_SINGLE, 2944 },
	{ 0xcdcd, 0xcdcf, PDF_CMAP_RANGE, 16127 },
	{ 0xcdd0, 0xcdd0, PDF_CMAP_SINGLE, 2945 },
	{ 0xcdd1, 0xcde7, PDF_CMAP_RANGE, 16130 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 2946 },
	{ 0xcde9, 0xcdeb, PDF_CMAP_RANGE, 16153 },
	{ 0xcdec, 0xcdec, PDF_CMAP_SINGLE, 2947 },
	{ 0xcded, 0xcdef, PDF_CMAP_RANGE, 16156 },
	{ 0xcdf0, 0xcdf0, PDF_CMAP_SINGLE, 2948 },
	{ 0xcdf1, 0xcdf7, PDF_CMAP_RANGE, 16159 },
	{ 0xcdf8, 0xcdf9, PDF_CMAP_RANGE, 2949 },
	{ 0xcdfa, 0xcdfd, PDF_CMAP_TABLE, 2900 },
	{ 0xcdfe, 0xce03, PDF_CMAP_RANGE, 16168 },
	{ 0xce04, 0xce04, PDF_CMAP_SINGLE, 2953 },
	{ 0xce05, 0xce07, PDF_CMAP_RANGE, 16174 },
	{ 0xce08, 0xce08, PDF_CMAP_SINGLE, 2954 },
	{ 0xce09, 0xce0b, PDF_CMAP_RANGE, 16177 },
	{ 0xce0c, 0xce0c, PDF_CMAP_SINGLE, 2955 },
	{ 0xce0d, 0xce13, PDF_CMAP_RANGE, 16180 },
	{ 0xce14, 0xce14, PDF_CMAP_SINGLE, 2956 },
	{ 0xce15, 0xce18, PDF_CMAP_RANGE, 16187 },
	{ 0xce19, 0xce19, PDF_CMAP_SINGLE, 2957 },
	{ 0xce1a, 0xce1f, PDF_CMAP_RANGE, 16191 },
	{ 0xce20, 0xce21, PDF_CMAP_RANGE, 2958 },
	{ 0xce22, 0xce23, PDF_CMAP_RANGE, 16197 },
	{ 0xce24, 0xce24, PDF_CMAP_SINGLE, 2960 },
	{ 0xce25, 0xce27, PDF_CMAP_RANGE, 16199 },
	{ 0xce28, 0xce28, PDF_CMAP_SINGLE, 2961 },
	{ 0xce29, 0xce2f, PDF_CMAP_RANGE, 16202 },
	{ 0xce30, 0xce31, PDF_CMAP_RANGE, 2962 },
	{ 0xce32, 0xce35, PDF_CMAP_TABLE, 2904 },
	{ 0xce36, 0xce57, PDF_CMAP_RANGE, 16211 },
	{ 0xce58, 0xce59, PDF_CMAP_RANGE, 2966 },
	{ 0xce5a, 0xce5b, PDF_CMAP_RANGE, 16245 },
	{ 0xce5c, 0xce5c, PDF_CMAP_SINGLE, 2968 },
	{ 0xce5d, 0xce5e, PDF_CMAP_RANGE, 16247 },
	{ 0xce5f, 0xce61, PDF_CMAP_RANGE, 2969 },
	{ 0xce62, 0xce67, PDF_CMAP_RANGE, 16249 },
	{ 0xce68, 0xce69, PDF_CMAP_RANGE, 2972 },
	{ 0xce6a, 0xce6d, PDF_CMAP_TABLE, 2908 },
	{ 0xce6e, 0xce73, PDF_CMAP_RANGE, 16257 },
	{ 0xce74, 0xce75, PDF_CMAP_RANGE, 2976 },
	{ 0xce76, 0xce77, PDF_CMAP_RANGE, 16263 },
	{ 0xce78, 0xce78, PDF_CMAP_SINGLE, 2978 },
	{ 0xce79, 0xce7b, PDF_CMAP_RANGE, 16265 },
	{ 0xce7c, 0xce7c, PDF_CMAP_SINGLE, 2979 },
	{ 0xce7d, 0xce83, PDF_CMAP_RANGE, 16268 },
	{ 0xce84, 0xce85, PDF_CMAP_RANGE, 2980 },
	{ 0xce86, 0xce89, PDF_CMAP_TABLE, 2912 },
	{ 0xce8a, 0xce8f, PDF_CMAP_RANGE, 16277 },
	{ 0xce90, 0xce91, PDF_CMAP_RANGE, 2984 },
	{ 0xce92, 0xce93, PDF_CMAP_RANGE, 16283 },
	{ 0xce94, 0xce94, PDF_CMAP_SINGLE, 2986 },
	{ 0xce95, 0xce97, PDF_CMAP_RANGE, 16285 },
	{ 0xce98, 0xce98, PDF_CMAP_SINGLE, 2987 },
	{ 0xce99, 0xce9f, PDF_CMAP_RANGE, 16288 },
	{ 0xcea0, 0xcea1, PDF_CMAP_RANGE, 2988 },
	{ 0xcea2, 0xcea2, PDF_CMAP_SINGLE, 16295 },
	{ 0xcea3, 0xcea5, PDF_CMAP_RANGE, 2990 },
	{ 0xcea6, 0xceab, PDF_CMAP_RANGE, 16296 },
	{ 0xceac, 0xcead, PDF_CMAP_RANGE, 2993 },
	{ 0xceae, 0xcec0, PDF_CMAP_RANGE, 16302 },
	{ 0xcec1, 0xcec1, PDF_CMAP_SINGLE, 2995 },
	{ 0xcec2, 0xcee3, PDF_CMAP_RANGE, 16321 },
	{ 0xcee4, 0xcee5, PDF_CMAP_RANGE, 2996 },
	{ 0xcee6, 0xcee7, PDF_CMAP_RANGE, 16355 },
	{ 0xcee8, 0xcee8, PDF_CMAP_SINGLE, 2998 },
	{ 0xcee9, 0xceea, PDF_CMAP_RANGE, 16357 },
	{ 0xceeb, 0xceec, PDF_CMAP_RANGE, 2999 },
	{ 0xceed, 0xcef3, PDF_CMAP_RANGE, 16359 },
	{ 0xcef4, 0xcef5, PDF_CMAP_RANGE, 3001 },
	{ 0xcef6, 0xcef6, PDF_CMAP_SINGLE, 16366 },
	{ 0xcef7, 0xcef9, PDF_CMAP_RANGE, 3003 },
	{ 0xcefa, 0xceff, PDF_CMAP_RANGE, 16367 },
	{ 0xcf00, 0xcf01, PDF_CMAP_RANGE, 3006 },
	{ 0xcf02, 0xcf03, PDF_CMAP_RANGE, 16373 },
	{ 0xcf04, 0xcf04, PDF_CMAP_SINGLE, 3008 },
	{ 0xcf05, 0xcf07, PDF_CMAP_RANGE, 16375 },
	{ 0xcf08, 0xcf08, PDF_CMAP_SINGLE, 3009 },
	{ 0xcf09, 0xcf0f, PDF_CMAP_RANGE, 16378 },
	{ 0xcf10, 0xcf11, PDF_CMAP_RANGE, 3010 },
	{ 0xcf12, 0xcf15, PDF_CMAP_TABLE, 2916 },
	{ 0xcf16, 0xcf1b, PDF_CMAP_RANGE, 16387 },
	{ 0xcf1c, 0xcf1c, PDF_CMAP_SINGLE, 3014 },
	{ 0xcf1d, 0xcf1f, PDF_CMAP_RANGE, 16393 },
	{ 0xcf20, 0xcf20, PDF_CMAP_SINGLE, 3015 },
	{ 0xcf21, 0xcf23, PDF_CMAP_RANGE, 16396 },
	{ 0xcf24, 0xcf24, PDF_CMAP_SINGLE, 3016 },
	{ 0xcf25, 0xcf2b, PDF_CMAP_RANGE, 16399 },
	{ 0xcf2c, 0xcf2d, PDF_CMAP_RANGE, 3017 },
	{ 0xcf2e, 0xcf2e, PDF_CMAP_SINGLE, 16406 },
	{ 0xcf2f, 0xcf31, PDF_CMAP_RANGE, 3019 },
	{ 0xcf32, 0xcf37, PDF_CMAP_RANGE, 16407 },
	{ 0xcf38, 0xcf38, PDF_CMAP_SINGLE, 3022 },
	{ 0xcf39, 0xcf53, PDF_CMAP_RANGE, 16413 },
	{ 0xcf54, 0xcf55, PDF_CMAP_RANGE, 3023 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 16440 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 3025 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 16442 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 3026 },
	{ 0xcf5d, 0xcf63, PDF_CMAP_RANGE, 16445 },
	{ 0xcf64, 0xcf65, PDF_CMAP_RANGE, 3027 },
	{ 0xcf66, 0xcf69, PDF_CMAP_TABLE, 2920 },
	{ 0xcf6a, 0xcf6f, PDF_CMAP_RANGE, 16454 },
	{ 0xcf70, 0xcf71, PDF_CMAP_RANGE, 3031 },
	{ 0xcf72, 0xcf73, PDF_CMAP_RANGE, 16460 },
	{ 0xcf74, 0xcf74, PDF_CMAP_SINGLE, 3033 },
	{ 0xcf75, 0xcf77, PDF_CMAP_RANGE, 16462 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 3034 },
	{ 0xcf79, 0xcf7f, PDF_CMAP_RANGE, 16465 },
	{ 0xcf80, 0xcf80, PDF_CMAP_SINGLE, 3035 },
	{ 0xcf81, 0xcf84, PDF_CMAP_RANGE, 16472 },
	{ 0xcf85, 0xcf85, PDF_CMAP_SINGLE, 3036 },
	{ 0xcf86, 0xcf8b, PDF_CMAP_RANGE, 16476 },
	{ 0xcf8c, 0xcf8c, PDF_CMAP_SINGLE, 3037 },
	{ 0xcf8d, 0xcfa0, PDF_CMAP_RANGE, 16482 },
	{ 0xcfa1, 0xcfa1, PDF_CMAP_SINGLE, 3038 },
	{ 0xcfa2, 0xcfa7, PDF_CMAP_RANGE, 16502 },
	{ 0xcfa8, 0xcfa8, PDF_CMAP_SINGLE, 3039 },
	{ 0xcfa9, 0xcfaf, PDF_CMAP_RANGE, 16508 },
	{ 0xcfb0, 0xcfb0, PDF_CMAP_SINGLE, 3040 },
	{ 0xcfb1, 0xcfc3, PDF_CMAP_RANGE, 16515 },
	{ 0xcfc4, 0xcfc4, PDF_CMAP_SINGLE, 3041 },
	{ 0xcfc5, 0xcfdf, PDF_CMAP_RANGE, 16534 },
	{ 0xcfe0, 0xcfe1, PDF_CMAP_RANGE, 3042 },
	{ 0xcfe2, 0xcfe3, PDF_CMAP_RANGE, 16561 },
	{ 0xcfe4, 0xcfe4, PDF_CMAP_SINGLE, 3044 },
	{ 0xcfe5, 0xcfe7, PDF_CMAP_RANGE, 16563 },
	{ 0xcfe8, 0xcfe8, PDF_CMAP_SINGLE, 3045 },
	{ 0xcfe9, 0xcfef, PDF_CMAP_RANGE, 16566 },
	{ 0xcff0, 0xcff1, PDF_CMAP_RANGE, 3046 },
	{ 0xcff2, 0xcff5, PDF_CMAP_TABLE, 2924 },
	{ 0xcff6, 0xcffb, PDF_CMAP_RANGE, 16575 },
	{ 0xcffc, 0xcffc, PDF_CMAP_SINGLE, 3050 },
	{ 0xcffd, 0xcfff, PDF_CMAP_RANGE, 16581 },
	{ 0xd000, 0xd000, PDF_CMAP_SINGLE, 3051 },
	{ 0xd001, 0xd003, PDF_CMAP_RANGE, 16584 },
	{ 0xd004, 0xd004, PDF_CMAP_SINGLE, 3052 },
	{ 0xd005, 0xd010, PDF_CMAP_RANGE, 16587 },
	{ 0xd011, 0xd011, PDF_CMAP_SINGLE, 3053 },
	{ 0xd012, 0xd017, PDF_CMAP_RANGE, 16599 },
	{ 0xd018, 0xd018, PDF_CMAP_SINGLE, 3054 },
	{ 0xd019, 0xd02c, PDF_CMAP_RANGE, 16605 },
	{ 0xd02d, 0xd02d, PDF_CMAP_SINGLE, 3055 },
	{ 0xd02e, 0xd033, PDF_CMAP_RANGE, 16625 },
	{ 0xd034, 0xd035, PDF_CMAP_RANGE, 3056 },
	{ 0xd036, 0xd037, PDF_CMAP_RANGE, 16631 },
	{ 0xd038, 0xd038, PDF_CMAP_SINGLE, 3058 },
	{ 0xd039, 0xd03b, PDF_CMAP_RANGE, 16633 },
	{ 0xd03c, 0xd03c, PDF_CMAP_SINGLE, 3059 },
	{ 0xd03d, 0xd043, PDF_CMAP_RANGE, 16636 },
	{ 0xd044, 0xd045, PDF_CMAP_RANGE, 3060 },
	{ 0xd046, 0xd049, PDF_CMAP_TABLE, 2928 },
	{ 0xd04a, 0xd04f, PDF_CMAP_RANGE, 16645 },
	{ 0xd050, 0xd050, PDF_CMAP_SINGLE, 3064 },
	{ 0xd051, 0xd053, PDF_CMAP_RANGE, 16651 },
	{ 0xd054, 0xd054, PDF_CMAP_SINGLE, 3065 },
	{ 0xd055, 0xd057, PDF_CMAP_RANGE, 16654 },
	{ 0xd058, 0xd058, PDF_CMAP_SINGLE, 3066 },
	{ 0xd059, 0xd05f, PDF_CMAP_RANGE, 16657 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 3067 },
	{ 0xd061, 0xd06b, PDF_CMAP_RANGE, 16664 },
	{ 0xd06c, 0xd06d, PDF_CMAP_RANGE, 3068 },
	{ 0xd06e, 0xd06f, PDF_CMAP_RANGE, 16675 },
	{ 0xd070, 0xd070, PDF_CMAP_SINGLE, 3070 },
	{ 0xd071, 0xd073, PDF_CMAP_RANGE, 16677 },
	{ 0xd074, 0xd074, PDF_CMAP_SINGLE, 3071 },
	{ 0xd075, 0xd07b, PDF_CMAP_RANGE, 16680 },
	{ 0xd07c, 0xd07d, PDF_CMAP_RANGE, 3072 },
	{ 0xd07e, 0xd080, PDF_CMAP_RANGE, 16687 },
	{ 0xd081, 0xd081, PDF_CMAP_SINGLE, 3074 },
	{ 0xd082, 0xd0a3, PDF_CMAP_RANGE, 16690 },
	{ 0xd0a4, 0xd0a5, PDF_CMAP_RANGE, 3075 },
	{ 0xd0a6, 0xd0a7, PDF_CMAP_RANGE, 16724 },
	{ 0xd0a8, 0xd0a8, PDF_CMAP_SINGLE, 3077 },
	{ 0xd0a9, 0xd0ab, PDF_CMAP_RANGE, 16726 },
	{ 0xd0ac, 0xd0ac, PDF_CMAP_SINGLE, 3078 },
	{ 0xd0ad, 0xd0b3, PDF_CMAP_RANGE, 16729 },
	{ 0xd0b4, 0xd0b5, PDF_CMAP_RANGE, 3079 },
	{ 0xd0b6, 0xd0b9, PDF_CMAP_TABLE, 2932 },
	{ 0xd0ba, 0xd0bf, PDF_CMAP_RANGE, 16738 },
	{ 0xd0c0, 0xd0c1, PDF_CMAP_RANGE, 3083 },
	{ 0xd0c2, 0xd0c3, PDF_CMAP_RANGE, 16744 },
	{ 0xd0c4, 0xd0c4, PDF_CMAP_SINGLE, 3085 },
	{ 0xd0c5, 0xd0c7, PDF_CMAP_RANGE, 16746 },
	{ 0xd0c8, 0xd0c9, PDF_CMAP_RANGE, 3086 },
	{ 0xd0ca, 0xd0cf, PDF_CMAP_RANGE, 16749 },
	{ 0xd0d0, 0xd0d1, PDF_CMAP_RANGE, 3088 },
	{ 0xd0d2, 0xd0d2, PDF_CMAP_SINGLE, 16755 },
	{ 0xd0d3, 0xd0d5, PDF_CMAP_RANGE, 3090 },
	{ 0xd0d6, 0xd0db, PDF_CMAP_RANGE, 16756 },
	{ 0xd0dc, 0xd0dd, PDF_CMAP_RANGE, 3093 },
	{ 0xd0de, 0xd0df, PDF_CMAP_RANGE, 16762 },
	{ 0xd0e0, 0xd0e0, PDF_CMAP_SINGLE, 3095 },
	{ 0xd0e1, 0xd0e3, PDF_CMAP_RANGE, 16764 },
	{ 0xd0e4, 0xd0e4, PDF_CMAP_SINGLE, 3096 },
	{ 0xd0e5, 0xd0eb, PDF_CMAP_RANGE, 16767 },
	{ 0xd0ec, 0xd0ed, PDF_CMAP_RANGE, 3097 },
	{ 0xd0ee, 0xd0ee, PDF_CMAP_SINGLE, 16774 },
	{ 0xd0ef, 0xd0f1, PDF_CMAP_RANGE, 3099 },
	{ 0xd0f2, 0xd0f7, PDF_CMAP_RANGE, 16775 },
	{ 0xd0f8, 0xd0f8, PDF_CMAP_SINGLE, 3102 },
	{ 0xd0f9, 0xd10c, PDF_CMAP_RANGE, 16781 },
	{ 0xd10d, 0xd10d, PDF_CMAP_SINGLE, 3103 },
	{ 0xd10e, 0xd12f, PDF_CMAP_RANGE, 16801 },
	{ 0xd130, 0xd131, PDF_CMAP_RANGE, 3104 },
	{ 0xd132, 0xd133, PDF_CMAP_RANGE, 16835 },
	{ 0xd134, 0xd134, PDF_CMAP_SINGLE, 3106 },
	{ 0xd135, 0xd137, PDF_CMAP_RANGE, 16837 },
	{ 0xd138, 0xd13a, PDF_CMAP_TABLE, 2936 },
	{ 0xd13b, 0xd13f, PDF_CMAP_RANGE, 16841 },
	{ 0xd140, 0xd141, PDF_CMAP_RANGE, 3109 },
	{ 0xd142, 0xd142, PDF_CMAP_SINGLE, 16846 },
	{ 0xd143, 0xd145, PDF_CMAP_RANGE, 3111 },
	{ 0xd146, 0xd14b, PDF_CMAP_RANGE, 16847 },
	{ 0xd14c, 0xd14d, PDF_CMAP_RANGE, 3114 },
	{ 0xd14e, 0xd14f, PDF_CMAP_RANGE, 16853 },
	{ 0xd150, 0xd150, PDF_CMAP_SINGLE, 3116 },
	{ 0xd151, 0xd153, PDF_CMAP_RANGE, 16855 },
	{ 0xd154, 0xd154, PDF_CMAP_SINGLE, 3117 },
	{ 0xd155, 0xd15b, PDF_CMAP_RANGE, 16858 },
	{ 0xd15c, 0xd15d, PDF_CMAP_RANGE, 3118 },
	{ 0xd15e, 0xd161, PDF_CMAP_TABLE, 2939 },
	{ 0xd162, 0xd167, PDF_CMAP_RANGE, 16867 },
	{ 0xd168, 0xd168, PDF_CMAP_SINGLE, 3122 },
	{ 0xd169, 0xd16b, PDF_CMAP_RANGE, 16873 },
	{ 0xd16c, 0xd16c, PDF_CMAP_SINGLE, 3123 },
	{ 0xd16d, 0xd17b, PDF_CMAP_RANGE, 16876 },
	{ 0xd17c, 0xd17c, PDF_CMAP_SINGLE, 3124 },
	{ 0xd17d, 0xd183, PDF_CMAP_RANGE, 16891 },
	{ 0xd184, 0xd184, PDF_CMAP_SINGLE, 3125 },
	{ 0xd185, 0xd187, PDF_CMAP_RANGE, 16898 },
	{ 0xd188, 0xd188, PDF_CMAP_SINGLE, 3126 },
	{ 0xd189, 0xd19f, PDF_CMAP_RANGE, 16901 },
	{ 0xd1a0, 0xd1a1, PDF_CMAP_RANGE, 3127 },
	{ 0xd1a2, 0xd1a3, PDF_CMAP_RANGE, 16924 },
	{ 0xd1a4, 0xd1a4, PDF_CMAP_SINGLE, 3129 },
	{ 0xd1a5, 0xd1a7, PDF_CMAP_RANGE, 16926 },
	{ 0xd1a8, 0xd1a8, PDF_CMAP_SINGLE, 3130 },
	{ 0xd1a9, 0xd1af, PDF_CMAP_RANGE, 16929 },
	{ 0xd1b0, 0xd1b1, PDF_CMAP_RANGE, 3131 },
	{ 0xd1b2, 0xd1b5, PDF_CMAP_TABLE, 2943 },
	{ 0xd1b6, 0xd1b9, PDF_CMAP_RANGE, 16938 },
	{ 0xd1ba, 0xd1bc, PDF_CMAP_TABLE, 2947 },
	{ 0xd1bd, 0xd1bf, PDF_CMAP_RANGE, 16943 },
	{ 0xd1c0, 0xd1c0, PDF_CMAP_SINGLE, 3137 },
	{ 0xd1c1, 0xd1d7, PDF_CMAP_RANGE, 16946 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 3138 },
	{ 0xd1d9, 0xd1f3, PDF_CMAP_RANGE, 16969 },
	{ 0xd1f4, 0xd1f4, PDF_CMAP_SINGLE, 3139 },
	{ 0xd1f5, 0xd1f7, PDF_CMAP_RANGE, 16996 },
	{ 0xd1f8, 0xd1f8, PDF_CMAP_SINGLE, 3140 },
	{ 0xd1f9, 0xd206, PDF_CMAP_RANGE, 16999 },
	{ 0xd207, 0xd209, PDF_CMAP_TABLE, 2950 },
	{ 0xd20a, 0xd20f, PDF_CMAP_RANGE, 17014 },
	{ 0xd210, 0xd210, PDF_CMAP_SINGLE, 3143 },
	{ 0xd211, 0xd22b, PDF_CMAP_RANGE, 17020 },
	{ 0xd22c, 0xd22d, PDF_CMAP_RANGE, 3144 },
	{ 0xd22e, 0xd22f, PDF_CMAP_RANGE, 17047 },
	{ 0xd230, 0xd230, PDF_CMAP_SINGLE, 3146 },
	{ 0xd231, 0xd233, PDF_CMAP_RANGE, 17049 },
	{ 0xd234, 0xd234, PDF_CMAP_SINGLE, 3147 },
	{ 0xd235, 0xd23b, PDF_CMAP_RANGE, 17052 },
	{ 0xd23c, 0xd23d, PDF_CMAP_RANGE, 3148 },
	{ 0xd23e, 0xd241, PDF_CMAP_TABLE, 2953 },
	{ 0xd242, 0xd247, PDF_CMAP_RANGE, 17061 },
	{ 0xd248, 0xd248, PDF_CMAP_SINGLE, 3152 },
	{ 0xd249, 0xd25b, PDF_CMAP_RANGE, 17067 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 3153 },
	{ 0xd25d, 0xd263, PDF_CMAP_RANGE, 17086 },
	{ 0xd264, 0xd264, PDF_CMAP_SINGLE, 3154 },
	{ 0xd265, 0xd27f, PDF_CMAP_RANGE, 17093 },
	{ 0xd280, 0xd281, PDF_CMAP_RANGE, 3155 },
	{ 0xd282, 0xd283, PDF_CMAP_RANGE, 17120 },
	{ 0xd284, 0xd284, PDF_CMAP_SINGLE, 3157 },
	{ 0xd285, 0xd287, PDF_CMAP_RANGE, 17122 },
	{ 0xd288, 0xd288, PDF_CMAP_SINGLE, 3158 },
	{ 0xd289, 0xd28f, PDF_CMAP_RANGE, 17125 },
	{ 0xd290, 0xd291, PDF_CMAP_RANGE, 3159 },
	{ 0xd292, 0xd294, PDF_CMAP_RANGE, 17132 },
	{ 0xd295, 0xd295, PDF_CMAP_SINGLE, 3161 },
	{ 0xd296, 0xd29b, PDF_CMAP_RANGE, 17135 },
	{ 0xd29c, 0xd29c, PDF_CMAP_SINGLE, 3162 },
	{ 0xd29d, 0xd29f, PDF_CMAP_RANGE, 17141 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 3163 },
	{ 0xd2a1, 0xd2a3, PDF_CMAP_RANGE, 17144 },
	{ 0xd2a4, 0xd2a4, PDF_CMAP_SINGLE, 3164 },
	{ 0xd2a5, 0xd2ab, PDF_CMAP_RANGE, 17147 },
	{ 0xd2ac, 0xd2ac, PDF_CMAP_SINGLE, 3165 },
	{ 0xd2ad, 0xd2b0, PDF_CMAP_RANGE, 17154 },
	{ 0xd2b1, 0xd2b1, PDF_CMAP_SINGLE, 3166 },
	{ 0xd2b2, 0xd2b7, PDF_CMAP_RANGE, 17158 },
	{ 0xd2b8, 0xd2b9, PDF_CMAP_RANGE, 3167 },
	{ 0xd2ba, 0xd2bb, PDF_CMAP_RANGE, 17164 },
	{ 0xd2bc, 0xd2bc, PDF_CMAP_SINGLE, 3169 },
	{ 0xd2bd, 0xd2be, PDF_CMAP_RANGE, 17166 },
	{ 0xd2bf, 0xd2c0, PDF_CMAP_RANGE, 3170 },
	{ 0xd2c1, 0xd2c2, PDF_CMAP_TABLE, 2957 },
	{ 0xd2c3, 0xd2c7, PDF_CMAP_RANGE, 17169 },
	{ 0xd2c8, 0xd2c9, PDF_CMAP_RANGE, 3173 },
	{ 0xd2ca, 0xd2cb, PDF_CMAP_TABLE, 2959 },
	{ 0xd2cc, 0xd2d3, PDF_CMAP_RANGE, 17175 },
	{ 0xd2d4, 0xd2d4, PDF_CMAP_SINGLE, 3176 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 17183 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 3177 },
	{ 0xd2d9, 0xd2db, PDF_CMAP_RANGE, 17186 },
	{ 0xd2dc, 0xd2dc, PDF_CMAP_SINGLE, 3178 },
	{ 0xd2dd, 0xd2e3, PDF_CMAP_RANGE, 17189 },
	{ 0xd2e4, 0xd2e5, PDF_CMAP_RANGE, 3179 },
	{ 0xd2e6, 0xd2ef, PDF_CMAP_RANGE, 17196 },
	{ 0xd2f0, 0xd2f1, PDF_CMAP_RANGE, 3181 },
	{ 0xd2f2, 0xd2f3, PDF_CMAP_RANGE, 17206 },
	{ 0xd2f4, 0xd2f4, PDF_CMAP_SINGLE, 3183 },
	{ 0xd2f5, 0xd2f7, PDF_CMAP_RANGE, 17208 },
	{ 0xd2f8, 0xd2f8, PDF_CMAP_SINGLE, 3184 },
	{ 0xd2f9, 0xd2ff, PDF_CMAP_RANGE, 17211 },
	{ 0xd300, 0xd301, PDF_CMAP_RANGE, 3185 },
	{ 0xd302, 0xd305, PDF_CMAP_TABLE, 2961 },
	{ 0xd306, 0xd30b, PDF_CMAP_RANGE, 17220 },
	{ 0xd30c, 0xd30e, PDF_CMAP_RANGE, 3189 },
	{ 0xd30f, 0xd310, PDF_CMAP_TABLE, 2965 },
	{ 0xd311, 0xd313, PDF_CMAP_RANGE, 17227 },
	{ 0xd314, 0xd316, PDF_CMAP_TABLE, 2967 },
	{ 0xd317, 0xd31b, PDF_CMAP_RANGE, 17231 },
	{ 0xd31c, 0xd31d, PDF_CMAP_RANGE, 3195 },
	{ 0xd31e, 0xd31e, PDF_CMAP_SINGLE, 17236 },
	{ 0xd31f, 0xd321, PDF_CMAP_RANGE, 3197 },
	{ 0xd322, 0xd324, PDF_CMAP_RANGE, 17237 },
	{ 0xd325, 0xd325, PDF_CMAP_SINGLE, 3200 },
	{ 0xd326, 0xd327, PDF_CMAP_RANGE, 17240 },
	{ 0xd328, 0xd329, PDF_CMAP_RANGE, 3201 },
	{ 0xd32a, 0xd32b, PDF_CMAP_RANGE, 17242 },
	{ 0xd32c, 0xd32c, PDF_CMAP_SINGLE, 3203 },
	{ 0xd32d, 0xd32f, PDF_CMAP_RANGE, 17244 },
	{ 0xd330, 0xd330, PDF_CMAP_SINGLE, 3204 },
	{ 0xd331, 0xd337, PDF_CMAP_RANGE, 17247 },
	{ 0xd338, 0xd339, PDF_CMAP_RANGE, 3205 },
	{ 0xd33a, 0xd33a, PDF_CMAP_SINGLE, 17254 },
	{ 0xd33b, 0xd33d, PDF_CMAP_RANGE, 3207 },
	{ 0xd33e, 0xd343, PDF_CMAP_RANGE, 17255 },
	{ 0xd344, 0xd345, PDF_CMAP_RANGE, 3210 },
	{ 0xd346, 0xd37b, PDF_CMAP_RANGE, 17261 },
	{ 0xd37c, 0xd37d, PDF_CMAP_RANGE, 3212 },
	{ 0xd37e, 0xd37f, PDF_CMAP_RANGE, 17315 },
	{ 0xd380, 0xd380, PDF_CMAP_SINGLE, 3214 },
	{ 0xd381, 0xd383, PDF_CMAP_RANGE, 17317 },
	{ 0xd384, 0xd384, PDF_CMAP_SINGLE, 3215 },
	{ 0xd385, 0xd38b, PDF_CMAP_RANGE, 17320 },
	{ 0xd38c, 0xd38d, PDF_CMAP_RANGE, 3216 },
	{ 0xd38e, 0xd38e, PDF_CMAP_SINGLE, 17327 },
	{ 0xd38f, 0xd391, PDF_CMAP_RANGE, 3218 },
	{ 0xd392, 0xd397, PDF_CMAP_RANGE, 17328 },
	{ 0xd398, 0xd399, PDF_CMAP_RANGE, 3221 },
	{ 0xd39a, 0xd39b, PDF_CMAP_RANGE, 17334 },
	{ 0xd39c, 0xd39c, PDF_CMAP_SINGLE, 3223 },
	{ 0xd39d, 0xd39f, PDF_CMAP_RANGE, 17336 },
	{ 0xd3a0, 0xd3a0, PDF_CMAP_SINGLE, 3224 },
	{ 0xd3a1, 0xd3a7, PDF_CMAP_RANGE, 17339 },
	{ 0xd3a8, 0xd3a9, PDF_CMAP_RANGE, 3225 },
	{ 0xd3aa, 0xd3ad, PDF_CMAP_TABLE, 2970 },
	{ 0xd3ae, 0xd3b3, PDF_CMAP_RANGE, 17348 },
	{ 0xd3b4, 0xd3b4, PDF_CMAP_SINGLE, 3229 },
	{ 0xd3b5, 0xd3b7, PDF_CMAP_RANGE, 17354 },
	{ 0xd3b8, 0xd3b8, PDF_CMAP_SINGLE, 3230 },
	{ 0xd3b9, 0xd3bb, PDF_CMAP_RANGE, 17357 },
	{ 0xd3bc, 0xd3bc, PDF_CMAP_SINGLE, 3231 },
	{ 0xd3bd, 0xd3c3, PDF_CMAP_RANGE, 17360 },
	{ 0xd3c4, 0xd3c5, PDF_CMAP_RANGE, 3232 },
	{ 0xd3c6, 0xd3c7, PDF_CMAP_RANGE, 17367 },
	{ 0xd3c8, 0xd3c9, PDF_CMAP_RANGE, 3234 },
	{ 0xd3ca, 0xd3cf, PDF_CMAP_RANGE, 17369 },
	{ 0xd3d0, 0xd3d0, PDF_CMAP_SINGLE, 3236 },
	{ 0xd3d1, 0xd3d7, PDF_CMAP_RANGE, 17375 },
	{ 0xd3d8, 0xd3d8, PDF_CMAP_SINGLE, 3237 },
	{ 0xd3d9, 0xd3e0, PDF_CMAP_RANGE, 17382 },
	{ 0xd3e1, 0xd3e3, PDF_CMAP_TABLE, 2974 },
	{ 0xd3e4, 0xd3eb, PDF_CMAP_RANGE, 17391 },
	{ 0xd3ec, 0xd3ed, PDF_CMAP_RANGE, 3240 },
	{ 0xd3ee, 0xd3ef, PDF_CMAP_RANGE, 17399 },
	{ 0xd3f0, 0xd3f0, PDF_CMAP_SINGLE, 3242 },
	{ 0xd3f1, 0xd3f3, PDF_CMAP_RANGE, 17401 },
	{ 0xd3f4, 0xd3f4, PDF_CMAP_SINGLE, 3243 },
	{ 0xd3f5, 0xd3fb, PDF_CMAP_RANGE, 17404 },
	{ 0xd3fc, 0xd3fd, PDF_CMAP_RANGE, 3244 },
	{ 0xd3fe, 0xd401, PDF_CMAP_TABLE, 2977 },
	{ 0xd402, 0xd407, PDF_CMAP_RANGE, 17413 },
	{ 0xd408, 0xd408, PDF_CMAP_SINGLE, 3248 },
	{ 0xd409, 0xd41c, PDF_CMAP_RANGE, 17419 },
	{ 0xd41d, 0xd41d, PDF_CMAP_SINGLE, 3249 },
	{ 0xd41e, 0xd43f, PDF_CMAP_RANGE, 17439 },
	{ 0xd440, 0xd440, PDF_CMAP_SINGLE, 3250 },
	{ 0xd441, 0xd443, PDF_CMAP_RANGE, 17473 },
	{ 0xd444, 0xd444, PDF_CMAP_SINGLE, 3251 },
	{ 0xd445, 0xd45b, PDF_CMAP_RANGE, 17476 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 3252 },
	{ 0xd45d, 0xd45f, PDF_CMAP_RANGE, 17499 },
	{ 0xd460, 0xd460, PDF_CMAP_SINGLE, 3253 },
	{ 0xd461, 0xd463, PDF_CMAP_RANGE, 17502 },
	{ 0xd464, 0xd464, PDF_CMAP_SINGLE, 3254 },
	{ 0xd465, 0xd46c, PDF_CMAP_RANGE, 17505 },
	{ 0xd46d, 0xd46f, PDF_CMAP_TABLE, 2981 },
	{ 0xd470, 0xd477, PDF_CMAP_RANGE, 17514 },
	{ 0xd478, 0xd479, PDF_CMAP_RANGE, 3257 },
	{ 0xd47a, 0xd47b, PDF_CMAP_RANGE, 17522 },
	{ 0xd47c, 0xd47c, PDF_CMAP_SINGLE, 3259 },
	{ 0xd47d, 0xd47e, PDF_CMAP_RANGE, 17524 },
	{ 0xd47f, 0xd480, PDF_CMAP_RANGE, 3260 },
	{ 0xd481, 0xd482, PDF_CMAP_TABLE, 2984 },
	{ 0xd483, 0xd487, PDF_CMAP_RANGE, 17527 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 3263 },
	{ 0xd48a, 0xd48d, PDF_CMAP_TABLE, 2986 },
	{ 0xd48e, 0xd493, PDF_CMAP_RANGE, 17534 },
	{ 0xd494, 0xd494, PDF_CMAP_SINGLE, 3267 },
	{ 0xd495, 0xd4a8, PDF_CMAP_RANGE, 17540 },
	{ 0xd4a9, 0xd4a9, PDF_CMAP_SINGLE, 3268 },
	{ 0xd4aa, 0xd4cb, PDF_CMAP_RANGE, 17560 },
	{ 0xd4cc, 0xd4cc, PDF_CMAP_SINGLE, 3269 },
	{ 0xd4cd, 0xd4cf, PDF_CMAP_RANGE, 17594 },
	{ 0xd4d0, 0xd4d0, PDF_CMAP_SINGLE, 3270 },
	{ 0xd4d1, 0xd4d3, PDF_CMAP_RANGE, 17597 },
	{ 0xd4d4, 0xd4d4, PDF_CMAP_SINGLE, 3271 },
	{ 0xd4d5, 0xd4db, PDF_CMAP_RANGE, 17600 },
	{ 0xd4dc, 0xd4dc, PDF_CMAP_SINGLE, 3272 },
	{ 0xd4dd, 0xd4de, PDF_CMAP_RANGE, 17607 },
	{ 0xd4df, 0xd4df, PDF_CMAP_SINGLE, 3273 },
	{ 0xd4e0, 0xd4e7, PDF_CMAP_RANGE, 17609 },
	{ 0xd4e8, 0xd4e8, PDF_CMAP_SINGLE, 3274 },
	{ 0xd4e9, 0xd4eb, PDF_CMAP_RANGE, 17617 },
	{ 0xd4ec, 0xd4ec, PDF_CMAP_SINGLE, 3275 },
	{ 0xd4ed, 0xd4ef, PDF_CMAP_RANGE, 17620 },
	{ 0xd4f0, 0xd4f0, PDF_CMAP_SINGLE, 3276 },
	{ 0xd4f1, 0xd4f7, PDF_CMAP_RANGE, 17623 },
	{ 0xd4f8, 0xd4f8, PDF_CMAP_SINGLE, 3277 },
	{ 0xd4f9, 0xd4fa, PDF_CMAP_RANGE, 17630 },
	{ 0xd4fb, 0xd4fd, PDF_CMAP_TABLE, 2990 },
	{ 0xd4fe, 0xd503, PDF_CMAP_RANGE, 17633 },
	{ 0xd504, 0xd504, PDF_CMAP_SINGLE, 3280 },
	{ 0xd505, 0xd507, PDF_CMAP_RANGE, 17639 },
	{ 0xd508, 0xd508, PDF_CMAP_SINGLE, 3281 },
	{ 0xd509, 0xd50b, PDF_CMAP_RANGE, 17642 },
	{ 0xd50c, 0xd50c, PDF_CMAP_SINGLE, 3282 },
	{ 0xd50d, 0xd513, PDF_CMAP_RANGE, 17645 },
	{ 0xd514, 0xd515, PDF_CMAP_RANGE, 3283 },
	{ 0xd516, 0xd517, PDF_CMAP_TABLE, 2993 },
	{ 0xd518, 0xd53b, PDF_CMAP_RANGE, 17653 },
	{ 0xd53c, 0xd53d, PDF_CMAP_RANGE, 3286 },
	{ 0xd53e, 0xd53f, PDF_CMAP_RANGE, 17689 },
	{ 0xd540, 0xd540, PDF_CMAP_SINGLE, 3288 },
	{ 0xd541, 0xd543, PDF_CMAP_RANGE, 17691 },
	{ 0xd544, 0xd544, PDF_CMAP_SINGLE, 3289 },
	{ 0xd545, 0xd54b, PDF_CMAP_RANGE, 17694 },
	{ 0xd54c, 0xd54d, PDF_CMAP_RANGE, 3290 },
	{ 0xd54e, 0xd551, PDF_CMAP_TABLE, 2995 },
	{ 0xd552, 0xd557, PDF_CMAP_RANGE, 17703 },
	{ 0xd558, 0xd559, PDF_CMAP_RANGE, 3294 },
	{ 0xd55a, 0xd55b, PDF_CMAP_RANGE, 17709 },
	{ 0xd55c, 0xd55c, PDF_CMAP_SINGLE, 3296 },
	{ 0xd55d, 0xd55f, PDF_CMAP_RANGE, 17711 },
	{ 0xd560, 0xd560, PDF_CMAP_SINGLE, 3297 },
	{ 0xd561, 0xd564, PDF_CMAP_RANGE, 17714 },
	{ 0xd565, 0xd565, PDF_CMAP_SINGLE, 3298 },
	{ 0xd566, 0xd567, PDF_CMAP_RANGE, 17718 },
	{ 0xd568, 0xd569, PDF_CMAP_RANGE, 3299 },
	{ 0xd56a, 0xd56d, PDF_CMAP_TABLE, 2999 },
	{ 0xd56e, 0xd573, PDF_CMAP_RANGE, 17722 },
	{ 0xd574, 0xd575, PDF_CMAP_RANGE, 3303 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 17728 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 3305 },
	{ 0xd579, 0xd57b, PDF_CMAP_RANGE, 17730 },
	{ 0xd57c, 0xd57c, PDF_CMAP_SINGLE, 3306 },
	{ 0xd57d, 0xd583, PDF_CMAP_RANGE, 17733 },
	{ 0xd584, 0xd585, PDF_CMAP_RANGE, 3307 },
	{ 0xd586, 0xd586, PDF_CMAP_SINGLE, 17740 },
	{ 0xd587, 0xd589, PDF_CMAP_RANGE, 3309 },
	{ 0xd58a, 0xd58f, PDF_CMAP_RANGE, 17741 },
	{ 0xd590, 0xd590, PDF_CMAP_SINGLE, 3312 },
	{ 0xd591, 0xd5a4, PDF_CMAP_RANGE, 17747 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 3313 },
	{ 0xd5a6, 0xd5c7, PDF_CMAP_RANGE, 17767 },
	{ 0xd5c8, 0xd5c9, PDF_CMAP_RANGE, 3314 },
	{ 0xd5ca, 0xd5cb, PDF_CMAP_RANGE, 17801 },
	{ 0xd5cc, 0xd5cc, PDF_CMAP_SINGLE, 3316 },
	{ 0xd5cd, 0xd5cf, PDF_CMAP_RANGE, 17803 },
	{ 0xd5d0, 0xd5d2, PDF_CMAP_TABLE, 3003 },
	{ 0xd5d3, 0xd5d7, PDF_CMAP_RANGE, 17807 },
	{ 0xd5d8, 0xd5d9, PDF_CMAP_RANGE, 3319 },
	{ 0xd5da, 0xd5dd, PDF_CMAP_TABLE, 3006 },
	{ 0xd5de, 0xd5e3, PDF_CMAP_RANGE, 17814 },
	{ 0xd5e4, 0xd5e5, PDF_CMAP_RANGE, 3323 },
	{ 0xd5e6, 0xd5e7, PDF_CMAP_RANGE, 17820 },
	{ 0xd5e8, 0xd5e8, PDF_CMAP_SINGLE, 3325 },
	{ 0xd5e9, 0xd5eb, PDF_CMAP_RANGE, 17822 },
	{ 0xd5ec, 0xd5ec, PDF_CMAP_SINGLE, 3326 },
	{ 0xd5ed, 0xd5f3, PDF_CMAP_RANGE, 17825 },
	{ 0xd5f4, 0xd5f5, PDF_CMAP_RANGE, 3327 },
	{ 0xd5f6, 0xd5f9, PDF_CMAP_TABLE, 3010 },
	{ 0xd5fa, 0xd5ff, PDF_CMAP_RANGE, 17834 },
	{ 0xd600, 0xd601, PDF_CMAP_RANGE, 3331 },
	{ 0xd602, 0xd603, PDF_CMAP_RANGE, 17840 },
	{ 0xd604, 0xd604, PDF_CMAP_SINGLE, 3333 },
	{ 0xd605, 0xd607, PDF_CMAP_RANGE, 17842 },
	{ 0xd608, 0xd608, PDF_CMAP_SINGLE, 3334 },
	{ 0xd609, 0xd60f, PDF_CMAP_RANGE, 17845 },
	{ 0xd610, 0xd611, PDF_CMAP_RANGE, 3335 },
	{ 0xd612, 0xd612, PDF_CMAP_SINGLE, 17852 },
	{ 0xd613, 0xd615, PDF_CMAP_RANGE, 3337 },
	{ 0xd616, 0xd61b, PDF_CMAP_RANGE, 17853 },
	{ 0xd61c, 0xd61c, PDF_CMAP_SINGLE, 3340 },
	{ 0xd61d, 0xd61f, PDF_CMAP_RANGE, 17859 },
	{ 0xd620, 0xd620, PDF_CMAP_SINGLE, 3341 },
	{ 0xd621, 0xd623, PDF_CMAP_RANGE, 17862 },
	{ 0xd624, 0xd624, PDF_CMAP_SINGLE, 3342 },
	{ 0xd625, 0xd62c, PDF_CMAP_RANGE, 17865 },
	{ 0xd62d, 0xd62d, PDF_CMAP_SINGLE, 3343 },
	{ 0xd62e, 0xd637, PDF_CMAP_RANGE, 17873 },
	{ 0xd638, 0xd639, PDF_CMAP_RANGE, 3344 },
	{ 0xd63a, 0xd63b, PDF_CMAP_RANGE, 17883 },
	{ 0xd63c, 0xd63c, PDF_CMAP_SINGLE, 3346 },
	{ 0xd63d, 0xd63f, PDF_CMAP_RANGE, 17885 },
	{ 0xd640, 0xd640, PDF_CMAP_SINGLE, 3347 },
	{ 0xd641, 0xd644, PDF_CMAP_RANGE, 17888 },
	{ 0xd645, 0xd645, PDF_CMAP_SINGLE, 3348 },
	{ 0xd646, 0xd647, PDF_CMAP_RANGE, 17892 },
	{ 0xd648, 0xd649, PDF_CMAP_RANGE, 3349 },
	{ 0xd64a, 0xd64d, PDF_CMAP_TABLE, 3014 },
	{ 0xd64e, 0xd650, PDF_CMAP_RANGE, 17896 },
	{ 0xd651, 0xd651, PDF_CMAP_SINGLE, 3353 },
	{ 0xd652, 0xd653, PDF_CMAP_RANGE, 17899 },
	{ 0xd654, 0xd655, PDF_CMAP_RANGE, 3354 },
	{ 0xd656, 0xd657, PDF_CMAP_RANGE, 17901 },
	{ 0xd658, 0xd658, PDF_CMAP_SINGLE, 3356 },
	{ 0xd659, 0xd65b, PDF_CMAP_RANGE, 17903 },
	{ 0xd65c, 0xd65c, PDF_CMAP_SINGLE, 3357 },
	{ 0xd65d, 0xd666, PDF_CMAP_RANGE, 17906 },
	{ 0xd667, 0xd669, PDF_CMAP_TABLE, 3018 },
	{ 0xd66a, 0xd66f, PDF_CMAP_RANGE, 17917 },
	{ 0xd670, 0xd671, PDF_CMAP_RANGE, 3360 },
	{ 0xd672, 0xd673, PDF_CMAP_RANGE, 17923 },
	{ 0xd674, 0xd674, PDF_CMAP_SINGLE, 3362 },
	{ 0xd675, 0xd682, PDF_CMAP_RANGE, 17925 },
	{ 0xd683, 0xd685, PDF_CMAP_TABLE, 3021 },
	{ 0xd686, 0xd68b, PDF_CMAP_RANGE, 17940 },
	{ 0xd68c, 0xd68d, PDF_CMAP_RANGE, 3365 },
	{ 0xd68e, 0xd68f, PDF_CMAP_RANGE, 17946 },
	{ 0xd690, 0xd690, PDF_CMAP_SINGLE, 3367 },
	{ 0xd691, 0xd693, PDF_CMAP_RANGE, 17948 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 3368 },
	{ 0xd695, 0xd69c, PDF_CMAP_RANGE, 17951 },
	{ 0xd69d, 0xd6a1, PDF_CMAP_TABLE, 3024 },
	{ 0xd6a2, 0xd6a7, PDF_CMAP_RANGE, 17961 },
	{ 0xd6a8, 0xd6a8, PDF_CMAP_SINGLE, 3372 },
	{ 0xd6a9, 0xd6ab, PDF_CMAP_RANGE, 17967 },
	{ 0xd6ac, 0xd6ac, PDF_CMAP_SINGLE, 3373 },
	{ 0xd6ad, 0xd6af, PDF_CMAP_RANGE, 17970 },
	{ 0xd6b0, 0xd6b0, PDF_CMAP_SINGLE, 3374 },
	{ 0xd6b1, 0xd6b8, PDF_CMAP_RANGE, 17973 },
	{ 0xd6b9, 0xd6bb, PDF_CMAP_TABLE, 3029 },
	{ 0xd6bc, 0xd6c3, PDF_CMAP_RANGE, 17982 },
	{ 0xd6c4, 0xd6c5, PDF_CMAP_RANGE, 3377 },
	{ 0xd6c6, 0xd6c7, PDF_CMAP_RANGE, 17990 },
	{ 0xd6c8, 0xd6c8, PDF_CMAP_SINGLE, 3379 },
	{ 0xd6c9, 0xd6cb, PDF_CMAP_RANGE, 17992 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 3380 },
	{ 0xd6cd, 0xd6d0, PDF_CMAP_RANGE, 17995 },
	{ 0xd6d1, 0xd6d1, PDF_CMAP_SINGLE, 3381 },
	{ 0xd6d2, 0xd6d3, PDF_CMAP_RANGE, 17999 },
	{ 0xd6d4, 0xd6d4, PDF_CMAP_SINGLE, 3382 },
	{ 0xd6d5, 0xd6d6, PDF_CMAP_RANGE, 18001 },
	{ 0xd6d7, 0xd6d9, PDF_CMAP_TABLE, 3032 },
	{ 0xd6da, 0xd6df, PDF_CMAP_RANGE, 18004 },
	{ 0xd6e0, 0xd6e0, PDF_CMAP_SINGLE, 3385 },
	{ 0xd6e1, 0xd6e3, PDF_CMAP_RANGE, 18010 },
	{ 0xd6e4, 0xd6e4, PDF_CMAP_SINGLE, 3386 },
	{ 0xd6e5, 0xd6e7, PDF_CMAP_RANGE, 18013 },
	{ 0xd6e8, 0xd6e8, PDF_CMAP_SINGLE, 3387 },
	{ 0xd6e9, 0xd6ef, PDF_CMAP_RANGE, 18016 },
	{ 0xd6f0, 0xd6f0, PDF_CMAP_SINGLE, 3388 },
	{ 0xd6f1, 0xd6f4, PDF_CMAP_RANGE, 18023 },
	{ 0xd6f5, 0xd6f5, PDF_CMAP_SINGLE, 3389 },
	{ 0xd6f6, 0xd6fb, PDF_CMAP_RANGE, 18027 },
	{ 0xd6fc, 0xd6fd, PDF_CMAP_RANGE, 3390 },
	{ 0xd6fe, 0xd6ff, PDF_CMAP_RANGE, 18033 },
	{ 0xd700, 0xd700, PDF_CMAP_SINGLE, 3392 },
	{ 0xd701, 0xd703, PDF_CMAP_RANGE, 18035 },
	{ 0xd704, 0xd704, PDF_CMAP_SINGLE, 3393 },
	{ 0xd705, 0xd710, PDF_CMAP_RANGE, 18038 },
	{ 0xd711, 0xd711, PDF_CMAP_SINGLE, 3394 },
	{ 0xd712, 0xd717, PDF_CMAP_RANGE, 18050 },
	{ 0xd718, 0xd719, PDF_CMAP_RANGE, 3395 },
	{ 0xd71a, 0xd71b, PDF_CMAP_RANGE, 18056 },
	{ 0xd71c, 0xd71c, PDF_CMAP_SINGLE, 3397 },
	{ 0xd71d, 0xd71f, PDF_CMAP_RANGE, 18058 },
	{ 0xd720, 0xd720, PDF_CMAP_SINGLE, 3398 },
	{ 0xd721, 0xd727, PDF_CMAP_RANGE, 18061 },
	{ 0xd728, 0xd729, PDF_CMAP_RANGE, 3399 },
	{ 0xd72a, 0xd72d, PDF_CMAP_TABLE, 3035 },
	{ 0xd72e, 0xd733, PDF_CMAP_RANGE, 18070 },
	{ 0xd734, 0xd735, PDF_CMAP_RANGE, 3403 },
	{ 0xd736, 0xd737, PDF_CMAP_RANGE, 18076 },
	{ 0xd738, 0xd738, PDF_CMAP_SINGLE, 3405 },
	{ 0xd739, 0xd73b, PDF_CMAP_RANGE, 18078 },
	{ 0xd73c, 0xd73c, PDF_CMAP_SINGLE, 3406 },
	{ 0xd73d, 0xd743, PDF_CMAP_RANGE, 18081 },
	{ 0xd744, 0xd744, PDF_CMAP_SINGLE, 3407 },
	{ 0xd745, 0xd746, PDF_CMAP_RANGE, 18088 },
	{ 0xd747, 0xd749, PDF_CMAP_TABLE, 3039 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 18091 },
	{ 0xd750, 0xd751, PDF_CMAP_RANGE, 3410 },
	{ 0xd752, 0xd753, PDF_CMAP_RANGE, 18097 },
	{ 0xd754, 0xd755, PDF_CMAP_TABLE, 3042 },
	{ 0xd756, 0xd759, PDF_CMAP_RANGE, 3413 },
	{ 0xd75a, 0xd75f, PDF_CMAP_RANGE, 18100 },
	{ 0xd760, 0xd761, PDF_CMAP_RANGE, 3417 },
	{ 0xd762, 0xd765, PDF_CMAP_TABLE, 3044 },
	{ 0xd766, 0xd768, PDF_CMAP_RANGE, 18108 },
	{ 0xd769, 0xd769, PDF_CMAP_SINGLE, 3421 },
	{ 0xd76a, 0xd76b, PDF_CMAP_RANGE, 18111 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 3422 },
	{ 0xd76d, 0xd76f, PDF_CMAP_RANGE, 18113 },
	{ 0xd770, 0xd770, PDF_CMAP_SINGLE, 3423 },
	{ 0xd771, 0xd773, PDF_CMAP_RANGE, 18116 },
	{ 0xd774, 0xd774, PDF_CMAP_SINGLE, 3424 },
	{ 0xd775, 0xd77b, PDF_CMAP_RANGE, 18119 },
	{ 0xd77c, 0xd77d, PDF_CMAP_RANGE, 3425 },
	{ 0xd77e, 0xd780, PDF_CMAP_RANGE, 18126 },
	{ 0xd781, 0xd781, PDF_CMAP_SINGLE, 3427 },
	{ 0xd782, 0xd787, PDF_CMAP_RANGE, 18129 },
	{ 0xd788, 0xd789, PDF_CMAP_RANGE, 3428 },
	{ 0xd78a, 0xd78b, PDF_CMAP_RANGE, 18135 },
	{ 0xd78c, 0xd78c, PDF_CMAP_SINGLE, 3430 },
	{ 0xd78d, 0xd78f, PDF_CMAP_RANGE, 18137 },
	{ 0xd790, 0xd790, PDF_CMAP_SINGLE, 3431 },
	{ 0xd791, 0xd797, PDF_CMAP_RANGE, 18140 },
	{ 0xd798, 0xd799, PDF_CMAP_RANGE, 3432 },
	{ 0xd79a, 0xd79d, PDF_CMAP_TABLE, 3048 },
	{ 0xd79e, 0xd7a3, PDF_CMAP_RANGE, 18149 },
	{ 0xf900, 0xf90c, PDF_CMAP_TABLE, 3052 },
	{ 0xf90d, 0xf913, PDF_CMAP_RANGE, 4375 },
	{ 0xf914, 0xf914, PDF_CMAP_SINGLE, 5800 },
	{ 0xf915, 0xf917, PDF_CMAP_RANGE, 4382 },
	{ 0xf918, 0xf91d, PDF_CMAP_RANGE, 4386 },
	{ 0xf91e, 0xf920, PDF_CMAP_RANGE, 4394 },
	{ 0xf921, 0xf922, PDF_CMAP_TABLE, 3065 },
	{ 0xf923, 0xf924, PDF_CMAP_RANGE, 4406 },
	{ 0xf925, 0xf92b, PDF_CMAP_RANGE, 4409 },
	{ 0xf92c, 0xf92d, PDF_CMAP_RANGE, 4419 },
	{ 0xf92e, 0xf92f, PDF_CMAP_TABLE, 3067 },
	{ 0xf930, 0xf931, PDF_CMAP_RANGE, 4513 },
	{ 0xf932, 0xf937, PDF_CMAP_RANGE, 4517 },
	{ 0xf938, 0xf93a, PDF_CMAP_RANGE, 4524 },
	{ 0xf93b, 0xf940, PDF_CMAP_RANGE, 4528 },
	{ 0xf941, 0xf943, PDF_CMAP_RANGE, 4535 },
	{ 0xf944, 0xf945, PDF_CMAP_RANGE, 4541 },
	{ 0xf946, 0xf948, PDF_CMAP_RANGE, 4545 },
	{ 0xf949, 0xf94a, PDF_CMAP_TABLE, 3069 },
	{ 0xf94b, 0xf94e, PDF_CMAP_RANGE, 4566 },
	{ 0xf94f, 0xf950, PDF_CMAP_RANGE, 4571 },
	{ 0xf951, 0xf951, PDF_CMAP_SINGLE, 4576 },
	{ 0xf952, 0xf955, PDF_CMAP_RANGE, 4604 },
	{ 0xf956, 0xf959, PDF_CMAP_RANGE, 4609 },
	{ 0xf95a, 0xf974, PDF_CMAP_TABLE, 3071 },
	{ 0xf975, 0xf977, PDF_CMAP_RANGE, 4425 },
	{ 0xf978, 0xf97a, PDF_CMAP_RANGE, 4429 },
	{ 0xf97b, 0xf97d, PDF_CMAP_RANGE, 4435 },
	{ 0xf97e, 0xf97e, PDF_CMAP_SINGLE, 4439 },
	{ 0xf97f, 0xf980, PDF_CMAP_RANGE, 4442 },
	{ 0xf981, 0xf983, PDF_CMAP_TABLE, 3098 },
	{ 0xf984, 0xf985, PDF_CMAP_RANGE, 4449 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 4453 },
	{ 0xf987, 0xf98c, PDF_CMAP_RANGE, 4455 },
	{ 0xf98d, 0xf98e, PDF_CMAP_TABLE, 3101 },
	{ 0xf98f, 0xf990, PDF_CMAP_RANGE, 4465 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 4161 },
	{ 0xf992, 0xf994, PDF_CMAP_RANGE, 4468 },
	{ 0xf995, 0xf995, PDF_CMAP_SINGLE, 4162 },
	{ 0xf996, 0xf997, PDF_CMAP_RANGE, 4471 },
	{ 0xf998, 0xf999, PDF_CMAP_TABLE, 3103 },
	{ 0xf99a, 0xf99b, PDF_CMAP_RANGE, 4475 },
	{ 0xf99c, 0xf99d, PDF_CMAP_RANGE, 4478 },
	{ 0xf99e, 0xf99e, PDF_CMAP_SINGLE, 6447 },
	{ 0xf99f, 0xf9a0, PDF_CMAP_RANGE, 4481 },
	{ 0xf9a1, 0xf9a5, PDF_CMAP_TABLE, 3105 },
	{ 0xf9a6, 0xf9a8, PDF_CMAP_RANGE, 4487 },
	{ 0xf9a9, 0xf9aa, PDF_CMAP_TABLE, 3110 },
	{ 0xf9ab, 0xf9ad, PDF_CMAP_RANGE, 4493 },
	{ 0xf9ae, 0xf9b0, PDF_CMAP_TABLE, 3112 },
	{ 0xf9b1, 0xf9b4, PDF_CMAP_RANGE, 4501 },
	{ 0xf9b5, 0xf9b5, PDF_CMAP_SINGLE, 4506 },
	{ 0xf9b6, 0xf9b8, PDF_CMAP_RANGE, 4508 },
	{ 0xf9b9, 0xf9b9, PDF_CMAP_SINGLE, 5797 },
	{ 0xf9ba, 0xf9bc, PDF_CMAP_RANGE, 4551 },
	{ 0xf9bd, 0xf9bf, PDF_CMAP_TABLE, 3115 },
	{ 0xf9c0, 0xf9c1, PDF_CMAP_RANGE, 4556 },
	{ 0xf9c2, 0xf9c3, PDF_CMAP_RANGE, 4560 },
	{ 0xf9c4, 0xf9c9, PDF_CMAP_TABLE, 3118 },
	{ 0xf9ca, 0xf9cb, PDF_CMAP_RANGE, 4581 },
	{ 0xf9cc, 0xf9cf, PDF_CMAP_TABLE, 3124 },
	{ 0xf9d0, 0xf9d3, PDF_CMAP_RANGE, 4590 },
	{ 0xf9d4, 0xf9d6, PDF_CMAP_RANGE, 4595 },
	{ 0xf9d7, 0xf9da, PDF_CMAP_RANGE, 4599 },
	{ 0xf9db, 0xf9e0, PDF_CMAP_TABLE, 3128 },
	{ 0xf9e1, 0xf9e2, PDF_CMAP_RANGE, 4620 },
	{ 0xf9e3, 0xf9e6, PDF_CMAP_TABLE, 3134 },
	{ 0xf9e7, 0xf9e9, PDF_CMAP_RANGE, 4632 },
	{ 0xf9ea, 0xf9ea, PDF_CMAP_SINGLE, 4636 },
	{ 0xf9eb, 0xf9ec, PDF_CMAP_RANGE, 4188 },
	{ 0xf9ed, 0xf9ed, PDF_CMAP_SINGLE, 4638 },
	{ 0xf9ee, 0xf9f0, PDF_CMAP_RANGE, 4640 },
	{ 0xf9f1, 0xf9f5, PDF_CMAP_RANGE, 4644 },
	{ 0xf9f6, 0xf9f6, PDF_CMAP_SINGLE, 4650 },
	{ 0xf9f7, 0xf9f9, PDF_CMAP_RANGE, 4653 },
	{ 0xf9fa, 0xfa0b, PDF_CMAP_TABLE, 3138 },
	{ 0xff01, 0xff3b, PDF_CMAP_RANGE, 264 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 112 },
	{ 0xff3d, 0xff5d, PDF_CMAP_RANGE, 324 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 113 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 143 },
	{ 0xffe2, 0xffe3, PDF_CMAP_TABLE, 3156 },
	{ 0xffe5, 0xffe6, PDF_CMAP_TABLE, 3158 },
	{ 0xbe60, 0xbe61, PDF_CMAP_RANGE, 2117 },
	{ 0xbe62, 0xbe63, PDF_CMAP_RANGE, 13006 },
	{ 0xbe64, 0xbe64, PDF_CMAP_SINGLE, 2119 },
	{ 0xbe65, 0xbe67, PDF_CMAP_RANGE, 13008 },
	{ 0xbe68, 0xbe68, PDF_CMAP_SINGLE, 2120 },
	{ 0xbe69, 0xbe69, PDF_CMAP_SINGLE, 13011 },
	{ 0xbe6a, 0xbe6a, PDF_CMAP_SINGLE, 2121 },
	{ 0xbe6b, 0xbe6f, PDF_CMAP_RANGE, 13012 },
	{ 0xbe70, 0xbe71, PDF_CMAP_RANGE, 2122 },
	{ 0xbe72, 0xbe72, PDF_CMAP_SINGLE, 13017 },
	{ 0xbe73, 0xbe75, PDF_CMAP_RANGE, 2124 },
	{ 0xbe76, 0xbe7a, PDF_CMAP_RANGE, 13018 },
	{ 0xbe7b, 0xbe7d, PDF_CMAP_RANGE, 2127 },
	{ 0xbe7e, 0xbe7f, PDF_CMAP_RANGE, 13023 },
	{ 0xbe80, 0xbe80, PDF_CMAP_SINGLE, 2130 },
	{ 0xbe81, 0xbe83, PDF_CMAP_RANGE, 13025 },
	{ 0xbe84, 0xbe84, PDF_CMAP_SINGLE, 2131 },
	{ 0xbe85, 0xbe8b, PDF_CMAP_RANGE, 13028 },
	{ 0xbe8c, 0xbe8d, PDF_CMAP_RANGE, 2132 },
	{ 0xbe8e, 0xbe8e, PDF_CMAP_SINGLE, 13035 },
	{ 0xbe8f, 0xbe91, PDF_CMAP_RANGE, 2134 },
	{ 0xbe92, 0xbe97, PDF_CMAP_RANGE, 13036 },
	{ 0xbe98, 0xbe99, PDF_CMAP_RANGE, 2137 },
	{ 0xbe9a, 0xbea7, PDF_CMAP_RANGE, 13042 },
	{ 0xbea8, 0xbea8, PDF_CMAP_SINGLE, 2139 },
	{ 0xbea9, 0xbecf, PDF_CMAP_RANGE, 13056 },
	{ 0xbed0, 0xbed1, PDF_CMAP_RANGE, 2140 },
	{ 0xbed2, 0xbed3, PDF_CMAP_RANGE, 13095 },
	{ 0xbed4, 0xbed4, PDF_CMAP_SINGLE, 2142 },
	{ 0xbed5, 0xbed6, PDF_CMAP_RANGE, 13097 },
	{ 0xbed7, 0xbed8, PDF_CMAP_RANGE, 2143 },
	{ 0xbed9, 0xbedf, PDF_CMAP_RANGE, 13099 },
	{ 0xbee0, 0xbee0, PDF_CMAP_SINGLE, 2145 },
	{ 0xbee1, 0xbee2, PDF_CMAP_RANGE, 13106 },
	{ 0xbee3, 0xbee5, PDF_CMAP_RANGE, 2146 },
	{ 0xbee6, 0xbeeb, PDF_CMAP_RANGE, 13108 },
	{ 0xbeec, 0xbeec, PDF_CMAP_SINGLE, 2149 },
	{ 0xbeed, 0xbeff, PDF_CMAP_RANGE, 13114 },
	{ 0xbf00, 0xbf00, PDF_CMAP_SINGLE, 13133 },
	{ 0xbf01, 0xbf01, PDF_CMAP_SINGLE, 2150 },
	{ 0xbf02, 0xbf07, PDF_CMAP_RANGE, 13134 },
	{ 0xbf08, 0xbf09, PDF_CMAP_RANGE, 2151 },
	{ 0xbf0a, 0xbf17, PDF_CMAP_RANGE, 13140 },
	{ 0xbf18, 0xbf19, PDF_CMAP_RANGE, 2153 },
	{ 0xbf1a, 0xbf1a, PDF_CMAP_SINGLE, 13154 },
	{ 0xbf1b, 0xbf1d, PDF_CMAP_RANGE, 2155 },
	{ 0xbf1e, 0xbf3f, PDF_CMAP_RANGE, 13155 },
	{ 0xbf40, 0xbf41, PDF_CMAP_RANGE, 2158 },
	{ 0xbf42, 0xbf43, PDF_CMAP_RANGE, 13189 },
	{ 0xbf44, 0xbf44, PDF_CMAP_SINGLE, 2160 },
	{ 0xbf45, 0xbf47, PDF_CMAP_RANGE, 13191 },
	{ 0xbf48, 0xbf48, PDF_CMAP_SINGLE, 2161 },
	{ 0xbf49, 0xbf4f, PDF_CMAP_RANGE, 13194 },
	{ 0xbf50, 0xbf51, PDF_CMAP_RANGE, 2162 },
	{ 0xbf52, 0xbf54, PDF_CMAP_RANGE, 13201 },
	{ 0xbf55, 0xbf55, PDF_CMAP_SINGLE, 2164 },
	{ 0xbf56, 0xbf93, PDF_CMAP_RANGE, 13204 },
	{ 0xbf94, 0xbf94, PDF_CMAP_SINGLE, 2165 },
	{ 0xbf95, 0xbfaf, PDF_CMAP_RANGE, 13266 },
	{ 0xbfb0, 0xbfb0, PDF_CMAP_SINGLE, 2166 },
	{ 0xbfb1, 0xbfc4, PDF_CMAP_RANGE, 13293 },
	{ 0xbfc5, 0xbfc5, PDF_CMAP_SINGLE, 2167 },
	{ 0xbfc6, 0xbfcb, PDF_CMAP_RANGE, 13313 },
	{ 0xbfcc, 0xbfcd, PDF_CMAP_RANGE, 2168 },
	{ 0xbfce, 0xbfcf, PDF_CMAP_RANGE, 13319 },
	{ 0xbfd0, 0xbfd0, PDF_CMAP_SINGLE, 2170 },
	{ 0xbfd1, 0xbfd3, PDF_CMAP_RANGE, 13321 },
	{ 0xbfd4, 0xbfd4, PDF_CMAP_SINGLE, 2171 },
	{ 0xbfd5, 0xbfdb, PDF_CMAP_RANGE, 13324 },
	{ 0xbfdc, 0xbfdc, PDF_CMAP_SINGLE, 2172 },
	{ 0xbfdd, 0xbfde, PDF_CMAP_RANGE, 13331 },
	{ 0xbfdf, 0xbfdf, PDF_CMAP_SINGLE, 2173 },
	{ 0xbfe0, 0xbfe0, PDF_CMAP_SINGLE, 13333 },
	{ 0xbfe1, 0xbfe1, PDF_CMAP_SINGLE, 2174 },
	{ 0xbfe2, 0xbfff, PDF_CMAP_RANGE, 13334 },
	{ 0xc000, 0xc03b, PDF_CMAP_RANGE, 13364 },
	{ 0xc03c, 0xc03c, PDF_CMAP_SINGLE, 2175 },
	{ 0xc03d, 0xc050, PDF_CMAP_RANGE, 13424 },
	{ 0xc051, 0xc051, PDF_CMAP_SINGLE, 2176 },
	{ 0xc052, 0xc057, PDF_CMAP_RANGE, 13444 },
	{ 0xc058, 0xc058, PDF_CMAP_SINGLE, 2177 },
	{ 0xc059, 0xc05b, PDF_CMAP_RANGE, 13450 },
	{ 0xc05c, 0xc05c, PDF_CMAP_SINGLE, 2178 },
	{ 0xc05d, 0xc05f, PDF_CMAP_RANGE, 13453 },
	{ 0xc060, 0xc060, PDF_CMAP_SINGLE, 2179 },
	{ 0xc061, 0xc067, PDF_CMAP_RANGE, 13456 },
	{ 0xc068, 0xc069, PDF_CMAP_RANGE, 2180 },
	{ 0xc06a, 0xc08f, PDF_CMAP_RANGE, 13463 },
	{ 0xc090, 0xc091, PDF_CMAP_RANGE, 2182 },
	{ 0xc092, 0xc093, PDF_CMAP_RANGE, 13501 },
	{ 0xc094, 0xc094, PDF_CMAP_SINGLE, 2184 },
	{ 0xc095, 0xc097, PDF_CMAP_RANGE, 13503 },
	{ 0xc098, 0xc098, PDF_CMAP_SINGLE, 2185 },
	{ 0xc099, 0xc09f, PDF_CMAP_RANGE, 13506 },
	{ 0xc0a0, 0xc0a1, PDF_CMAP_RANGE, 2186 },
	{ 0xc0a2, 0xc0a2, PDF_CMAP_SINGLE, 13513 },
	{ 0xc0a3, 0xc0a3, PDF_CMAP_SINGLE, 2188 },
	{ 0xc0a4, 0xc0a4, PDF_CMAP_SINGLE, 13514 },
	{ 0xc0a5, 0xc0a5, PDF_CMAP_SINGLE, 2189 },
	{ 0xc0a6, 0xc0ab, PDF_CMAP_RANGE, 13515 },
	{ 0xc0ac, 0xc0ad, PDF_CMAP_RANGE, 2190 },
	{ 0xc0ae, 0xc0ae, PDF_CMAP_SINGLE, 13521 },
	{ 0xc0af, 0xc0b0, PDF_CMAP_RANGE, 2192 },
	{ 0xc0b1, 0xc0b2, PDF_CMAP_RANGE, 13522 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 2194 },
	{ 0xc0b7, 0xc0bb, PDF_CMAP_RANGE, 13524 },
	{ 0xc0bc, 0xc0bd, PDF_CMAP_RANGE, 2198 },
	{ 0xc0be, 0xc0be, PDF_CMAP_SINGLE, 13529 },
	{ 0xc0bf, 0xc0c1, PDF_CMAP_RANGE, 2200 },
	{ 0xc0c2, 0xc0c4, PDF_CMAP_RANGE, 13530 },
	{ 0xc0c5, 0xc0c5, PDF_CMAP_SINGLE, 2203 },
	{ 0xc0c6, 0xc0c7, PDF_CMAP_RANGE, 13533 },
	{ 0xc0c8, 0xc0c9, PDF_CMAP_RANGE, 2204 },
	{ 0xc0ca, 0xc0cb, PDF_CMAP_RANGE, 13535 },
	{ 0xc0cc, 0xc0cc, PDF_CMAP_SINGLE, 2206 },
	{ 0xc0cd, 0xc0cf, PDF_CMAP_RANGE, 13537 },
	{ 0xc0d0, 0xc0d0, PDF_CMAP_SINGLE, 2207 },
	{ 0xc0d1, 0xc0d7, PDF_CMAP_RANGE, 13540 },
	{ 0xc0d8, 0xc0d9, PDF_CMAP_RANGE, 2208 },
	{ 0xc0da, 0xc0da, PDF_CMAP_SINGLE, 13547 },
	{ 0xc0db, 0xc0dd, PDF_CMAP_RANGE, 2210 },
	{ 0xc0de, 0xc0e3, PDF_CMAP_RANGE, 13548 },
	{ 0xc0e4, 0xc0e5, PDF_CMAP_RANGE, 2213 },
	{ 0xc0e6, 0xc0e7, PDF_CMAP_RANGE, 13554 },
	{ 0xc0e8, 0xc0e8, PDF_CMAP_SINGLE, 2215 },
	{ 0xc0e9, 0xc0eb, PDF_CMAP_RANGE, 13556 },
	{ 0xc0ec, 0xc0ec, PDF_CMAP_SINGLE, 2216 },
	{ 0xc0ed, 0xc0f3, PDF_CMAP_RANGE, 13559 },
	{ 0xc0f4, 0xc0f5, PDF_CMAP_RANGE, 2217 },
	{ 0xc0f6, 0xc0f6, PDF_CMAP_SINGLE, 13566 },
	{ 0xc0f7, 0xc0f7, PDF_CMAP_SINGLE, 2219 },
	{ 0xc0f8, 0xc0f8, PDF_CMAP_SINGLE, 13567 },
	{ 0xc0f9, 0xc0f9, PDF_CMAP_SINGLE, 2220 },
	{ 0xc0fa, 0xc0ff, PDF_CMAP_RANGE, 13568 },
	{ 0xc100, 0xc100, PDF_CMAP_SINGLE, 2221 },
	{ 0xc101, 0xc103, PDF_CMAP_RANGE, 13574 },
	{ 0xc104, 0xc104, PDF_CMAP_SINGLE, 2222 },
	{ 0xc105, 0xc107, PDF_CMAP_RANGE, 13577 },
	{ 0xc108, 0xc108, PDF_CMAP_SINGLE, 2223 },
	{ 0xc109, 0xc10f, PDF_CMAP_RANGE, 13580 },
	{ 0xc110, 0xc110, PDF_CMAP_SINGLE, 2224 },
	{ 0xc111, 0xc114, PDF_CMAP_RANGE, 13587 },
	{ 0xc115, 0xc115, PDF_CMAP_SINGLE, 2225 },
	{ 0xc116, 0xc11b, PDF_CMAP_RANGE, 13591 },
	{ 0xc11c, 0xc120, PDF_CMAP_RANGE, 2226 },
	{ 0xc121, 0xc122, PDF_CMAP_RANGE, 13597 },
	{ 0xc123, 0xc124, PDF_CMAP_RANGE, 2231 },
	{ 0xc125, 0xc125, PDF_CMAP_SINGLE, 13599 },
	{ 0xc126, 0xc127, PDF_CMAP_RANGE, 2233 },
	{ 0xc128, 0xc12b, PDF_CMAP_RANGE, 13600 },
	{ 0xc12c, 0xc12d, PDF_CMAP_RANGE, 2235 },
	{ 0xc12e, 0xc12e, PDF_CMAP_SINGLE, 13604 },
	{ 0xc12f, 0xc131, PDF_CMAP_RANGE, 2237 },
	{ 0xc132, 0xc135, PDF_CMAP_RANGE, 13605 },
	{ 0xc136, 0xc136, PDF_CMAP_SINGLE, 2240 },
	{ 0xc137, 0xc137, PDF_CMAP_SINGLE, 13609 },
	{ 0xc138, 0xc139, PDF_CMAP_RANGE, 2241 },
	{ 0xc13a, 0xc13b, PDF_CMAP_RANGE, 13610 },
	{ 0xc13c, 0xc13c, PDF_CMAP_SINGLE, 2243 },
	{ 0xc13d, 0xc13f, PDF_CMAP_RANGE, 13612 },
	{ 0xc140, 0xc140, PDF_CMAP_SINGLE, 2244 },
	{ 0xc141, 0xc147, PDF_CMAP_RANGE, 13615 },
	{ 0xc148, 0xc149, PDF_CMAP_RANGE, 2245 },
	{ 0xc14a, 0xc14a, PDF_CMAP_SINGLE, 13622 },
	{ 0xc14b, 0xc14d, PDF_CMAP_RANGE, 2247 },
	{ 0xc14e, 0xc153, PDF_CMAP_RANGE, 13623 },
	{ 0xc154, 0xc155, PDF_CMAP_RANGE, 2250 },
	{ 0xc156, 0xc157, PDF_CMAP_RANGE, 13629 },
	{ 0xc158, 0xc158, PDF_CMAP_SINGLE, 2252 },
	{ 0xc159, 0xc15b, PDF_CMAP_RANGE, 13631 },
	{ 0xc15c, 0xc15c, PDF_CMAP_SINGLE, 2253 },
	{ 0xc15d, 0xc163, PDF_CMAP_RANGE, 13634 },
	{ 0xc164, 0xc165, PDF_CMAP_RANGE, 2254 },
	{ 0xc166, 0xc166, PDF_CMAP_SINGLE, 13641 },
	{ 0xc167, 0xc169, PDF_CMAP_RANGE, 2256 },
	{ 0xc16a, 0xc16f, PDF_CMAP_RANGE, 13642 },
	{ 0xc170, 0xc170, PDF_CMAP_SINGLE, 2259 },
	{ 0xc171, 0xc173, PDF_CMAP_RANGE, 13648 },
	{ 0xc174, 0xc174, PDF_CMAP_SINGLE, 2260 },
	{ 0xc175, 0xc177, PDF_CMAP_RANGE, 13651 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 2261 },
	{ 0xc179, 0xc184, PDF_CMAP_RANGE, 13654 },
	{ 0xc185, 0xc185, PDF_CMAP_SINGLE, 2262 },
	{ 0xc186, 0xc18b, PDF_CMAP_RANGE, 13666 },
	{ 0xc18c, 0xc18e, PDF_CMAP_RANGE, 2263 },
	{ 0xc18f, 0xc18f, PDF_CMAP_SINGLE, 13672 },
	{ 0xc190, 0xc190, PDF_CMAP_SINGLE, 2266 },
	{ 0xc191, 0xc193, PDF_CMAP_RANGE, 13673 },
	{ 0xc194, 0xc194, PDF_CMAP_SINGLE, 2267 },
	{ 0xc195, 0xc195, PDF_CMAP_SINGLE, 13676 },
	{ 0xc196, 0xc196, PDF_CMAP_SINGLE, 2268 },
	{ 0xc197, 0xc19b, PDF_CMAP_RANGE, 13677 },
	{ 0xc19c, 0xc19d, PDF_CMAP_RANGE, 2269 },
	{ 0xc19e, 0xc19e, PDF_CMAP_SINGLE, 13682 },
	{ 0xc19f, 0xc19f, PDF_CMAP_SINGLE, 2271 },
	{ 0xc1a0, 0xc1a0, PDF_CMAP_SINGLE, 13683 },
	{ 0xc1a1, 0xc1a1, PDF_CMAP_SINGLE, 2272 },
	{ 0xc1a2, 0xc1a4, PDF_CMAP_RANGE, 13684 },
	{ 0xc1a5, 0xc1a5, PDF_CMAP_SINGLE, 2273 },
	{ 0xc1a6, 0xc1a7, PDF_CMAP_RANGE, 13687 },
	{ 0xc1a8, 0xc1a9, PDF_CMAP_RANGE, 2274 },
	{ 0xc1aa, 0xc1ab, PDF_CMAP_RANGE, 13689 },
	{ 0xc1ac, 0xc1ac, PDF_CMAP_SINGLE, 2276 },
	{ 0xc1ad, 0xc1af, PDF_CMAP_RANGE, 13691 },
	{ 0xc1b0, 0xc1b0, PDF_CMAP_SINGLE, 2277 },
	{ 0xc1b1, 0xc1bc, PDF_CMAP_RANGE, 13694 },
	{ 0xc1bd, 0xc1bd, PDF_CMAP_SINGLE, 2278 },
	{ 0xc1be, 0xc1c3, PDF_CMAP_RANGE, 13706 },
	{ 0xc1c4, 0xc1c4, PDF_CMAP_SINGLE, 2279 },
	{ 0xc1c5, 0xc1c7, PDF_CMAP_RANGE, 13712 },
	{ 0xc1c8, 0xc1c8, PDF_CMAP_SINGLE, 2280 },
	{ 0xc1c9, 0xc1cb, PDF_CMAP_RANGE, 13715 },
	{ 0xc1cc, 0xc1cc, PDF_CMAP_SINGLE, 2281 },
	{ 0xc1cd, 0xc1d3, PDF_CMAP_RANGE, 13718 },
	{ 0xc1d4, 0xc1d4, PDF_CMAP_SINGLE, 2282 },
	{ 0xc1d5, 0xc1d6, PDF_CMAP_RANGE, 13725 },
	{ 0xc1d7, 0xc1d8, PDF_CMAP_RANGE, 2283 },
	{ 0xc1d9, 0xc1df, PDF_CMAP_RANGE, 13727 },
	{ 0xc1e0, 0xc1e0, PDF_CMAP_SINGLE, 2285 },
	{ 0xc1e1, 0xc1e3, PDF_CMAP_RANGE, 13734 },
	{ 0xc1e4, 0xc1e4, PDF_CMAP_SINGLE, 2286 },
	{ 0xc1e5, 0xc1e7, PDF_CMAP_RANGE, 13737 },
	{ 0xc1e8, 0xc1e8, PDF_CMAP_SINGLE, 2287 },
	{ 0xc1e9, 0xc1ef, PDF_CMAP_RANGE, 13740 },
	{ 0xc1f0, 0xc1f1, PDF_CMAP_RANGE, 2288 },
	{ 0xc1f2, 0xc1f2, PDF_CMAP_SINGLE, 13747 },
	{ 0xc1f3, 0xc1f3, PDF_CMAP_SINGLE, 2290 },
	{ 0xc1f4, 0xc1fb, PDF_CMAP_RANGE, 13748 },
	{ 0xc1fc, 0xc1fd, PDF_CMAP_RANGE, 2291 },
	{ 0xc1fe, 0xc1ff, PDF_CMAP_RANGE, 13756 },
	{ 0xc200, 0xc200, PDF_CMAP_SINGLE, 2293 },
	{ 0xc201, 0xc203, PDF_CMAP_RANGE, 13758 },
	{ 0xc204, 0xc204, PDF_CMAP_SINGLE, 2294 },
	{ 0xc205, 0xc20b, PDF_CMAP_RANGE, 13761 },
	{ 0xc20c, 0xc20d, PDF_CMAP_RANGE, 2295 },
	{ 0xc20e, 0xc20e, PDF_CMAP_SINGLE, 13768 },
	{ 0xc20f, 0xc20f, PDF_CMAP_SINGLE, 2297 },
	{ 0xc210, 0xc210, PDF_CMAP_SINGLE, 13769 },
	{ 0xc211, 0xc211, PDF_CMAP_SINGLE, 2298 },
	{ 0xc212, 0xc217, PDF_CMAP_RANGE, 13770 },
	{ 0xc218, 0xc219, PDF_CMAP_RANGE, 2299 },
	{ 0xc21a, 0xc21b, PDF_CMAP_RANGE, 13776 },
	{ 0xc21c, 0xc21c, PDF_CMAP_SINGLE, 2301 },
	{ 0xc21d, 0xc21e, PDF_CMAP_RANGE, 13778 },
	{ 0xc21f, 0xc220, PDF_CMAP_RANGE, 2302 },
	{ 0xc221, 0xc227, PDF_CMAP_RANGE, 13780 },
	{ 0xc228, 0xc229, PDF_CMAP_RANGE, 2304 },
	{ 0xc22a, 0xc22a, PDF_CMAP_SINGLE, 13787 },
	{ 0xc22b, 0xc22b, PDF_CMAP_SINGLE, 2306 },
	{ 0xc22c, 0xc22c, PDF_CMAP_SINGLE, 13788 },
	{ 0xc22d, 0xc22d, PDF_CMAP_SINGLE, 2307 },
	{ 0xc22e, 0xc22e, PDF_CMAP_SINGLE, 13789 },
	{ 0xc22f, 0xc22f, PDF_CMAP_SINGLE, 2308 },
	{ 0xc230, 0xc230, PDF_CMAP_SINGLE, 13790 },
	{ 0xc231, 0xc232, PDF_CMAP_RANGE, 2309 },
	{ 0xc233, 0xc233, PDF_CMAP_SINGLE, 13791 },
	{ 0xc234, 0xc234, PDF_CMAP_SINGLE, 2311 },
	{ 0xc235, 0xc247, PDF_CMAP_RANGE, 13792 },
	{ 0xc248, 0xc248, PDF_CMAP_SINGLE, 2312 },
	{ 0xc249, 0xc24f, PDF_CMAP_RANGE, 13811 },
	{ 0xc250, 0xc251, PDF_CMAP_RANGE, 2313 },
	{ 0xc252, 0xc253, PDF_CMAP_RANGE, 13818 },
	{ 0xc254, 0xc254, PDF_CMAP_SINGLE, 2315 },
	{ 0xc255, 0xc257, PDF_CMAP_RANGE, 13820 },
	{ 0xc258, 0xc258, PDF_CMAP_SINGLE, 2316 },
	{ 0xc259, 0xc25f, PDF_CMAP_RANGE, 13823 },
	{ 0xc260, 0xc260, PDF_CMAP_SINGLE, 2317 },
	{ 0xc261, 0xc264, PDF_CMAP_RANGE, 13830 },
	{ 0xc265, 0xc265, PDF_CMAP_SINGLE, 2318 },
	{ 0xc266, 0xc26b, PDF_CMAP_RANGE, 13834 },
	{ 0xc26c, 0xc26d, PDF_CMAP_RANGE, 2319 },
	{ 0xc26e, 0xc26f, PDF_CMAP_RANGE, 13840 },
	{ 0xc270, 0xc270, PDF_CMAP_SINGLE, 2321 },
	{ 0xc271, 0xc273, PDF_CMAP_RANGE, 13842 },
	{ 0xc274, 0xc274, PDF_CMAP_SINGLE, 2322 },
	{ 0xc275, 0xc27b, PDF_CMAP_RANGE, 13845 },
	{ 0xc27c, 0xc27d, PDF_CMAP_RANGE, 2323 },
	{ 0xc27e, 0xc27e, PDF_CMAP_SINGLE, 13852 },
	{ 0xc27f, 0xc27f, PDF_CMAP_SINGLE, 2325 },
	{ 0xc280, 0xc280, PDF_CMAP_SINGLE, 13853 },
	{ 0xc281, 0xc281, PDF_CMAP_SINGLE, 2326 },
	{ 0xc282, 0xc287, PDF_CMAP_RANGE, 13854 },
	{ 0xc288, 0xc289, PDF_CMAP_RANGE, 2327 },
	{ 0xc28a, 0xc28f, PDF_CMAP_RANGE, 13860 },
	{ 0xc290, 0xc290, PDF_CMAP_SINGLE, 2329 },
	{ 0xc291, 0xc297, PDF_CMAP_RANGE, 13866 },
	{ 0xc298, 0xc298, PDF_CMAP_SINGLE, 2330 },
	{ 0xc299, 0xc29a, PDF_CMAP_RANGE, 13873 },
	{ 0xc29b, 0xc29b, PDF_CMAP_SINGLE, 2331 },
	{ 0xc29c, 0xc29c, PDF_CMAP_SINGLE, 13875 },
	{ 0xc29d, 0xc29d, PDF_CMAP_SINGLE, 2332 },
	{ 0xc29e, 0xc2a3, PDF_CMAP_RANGE, 13876 },
	{ 0xc2a4, 0xc2a5, PDF_CMAP_RANGE, 2333 },
	{ 0xc2a6, 0xc2a7, PDF_CMAP_RANGE, 13882 },
	{ 0xc2a8, 0xc2a8, PDF_CMAP_SINGLE, 2335 },
	{ 0xc2a9, 0xc2ab, PDF_CMAP_RANGE, 13884 },
	{ 0xc2ac, 0xc2ad, PDF_CMAP_RANGE, 2336 },
	{ 0xc2ae, 0xc2b3, PDF_CMAP_RANGE, 13887 },
	{ 0xc2b4, 0xc2b5, PDF_CMAP_RANGE, 2338 },
	{ 0xc2b6, 0xc2b6, PDF_CMAP_SINGLE, 13893 },
	{ 0xc2b7, 0xc2b7, PDF_CMAP_SINGLE, 2340 },
	{ 0xc2b8, 0xc2b8, PDF_CMAP_SINGLE, 13894 },
	{ 0xc2b9, 0xc2b9, PDF_CMAP_SINGLE, 2341 },
	{ 0xc2ba, 0xc2db, PDF_CMAP_RANGE, 13895 },
	{ 0xc2dc, 0xc2dd, PDF_CMAP_RANGE, 2342 },
	{ 0xc2de, 0xc2df, PDF_CMAP_RANGE, 13929 },
	{ 0xc2e0, 0xc2e0, PDF_CMAP_SINGLE, 2344 },
	{ 0xc2e1, 0xc2e2, PDF_CMAP_RANGE, 13931 },
	{ 0xc2e3, 0xc2e4, PDF_CMAP_RANGE, 2345 },
	{ 0xc2e5, 0xc2ea, PDF_CMAP_RANGE, 13933 },
	{ 0xc2eb, 0xc2ed, PDF_CMAP_RANGE, 2347 },
	{ 0xc2ee, 0xc2ee, PDF_CMAP_SINGLE, 13939 },
	{ 0xc2ef, 0xc2ef, PDF_CMAP_SINGLE, 2350 },
	{ 0xc2f0, 0xc2f0, PDF_CMAP_SINGLE, 13940 },
	{ 0xc2f1, 0xc2f1, PDF_CMAP_SINGLE, 2351 },
	{ 0xc2f2, 0xc2f5, PDF_CMAP_RANGE, 13941 },
	{ 0xc2f6, 0xc2f6, PDF_CMAP_SINGLE, 2352 },
	{ 0xc2f7, 0xc2f7, PDF_CMAP_SINGLE, 13945 },
	{ 0xc2f8, 0xc2f9, PDF_CMAP_RANGE, 2353 },
	{ 0xc2fa, 0xc2fa, PDF_CMAP_SINGLE, 13946 },
	{ 0xc2fb, 0xc2fc, PDF_CMAP_RANGE, 2355 },
	{ 0xc2fd, 0xc2ff, PDF_CMAP_RANGE, 13947 },
	{ 0xc300, 0xc300, PDF_CMAP_SINGLE, 2357 },
	{ 0xc301, 0xc307, PDF_CMAP_RANGE, 13950 },
	{ 0xc308, 0xc309, PDF_CMAP_RANGE, 2358 },
	{ 0xc30a, 0xc30b, PDF_CMAP_RANGE, 13957 },
	{ 0xc30c, 0xc30d, PDF_CMAP_RANGE, 2360 },
	{ 0xc30e, 0xc312, PDF_CMAP_RANGE, 13959 },
	{ 0xc313, 0xc315, PDF_CMAP_RANGE, 2362 },
	{ 0xc316, 0xc317, PDF_CMAP_RANGE, 13964 },
	{ 0xc318, 0xc318, PDF_CMAP_SINGLE, 2365 },
	{ 0xc319, 0xc31b, PDF_CMAP_RANGE, 13966 },
	{ 0xc31c, 0xc31c, PDF_CMAP_SINGLE, 2366 },
	{ 0xc31d, 0xc323, PDF_CMAP_RANGE, 13969 },
	{ 0xc324, 0xc325, PDF_CMAP_RANGE, 2367 },
	{ 0xc326, 0xc327, PDF_CMAP_RANGE, 13976 },
	{ 0xc328, 0xc329, PDF_CMAP_RANGE, 2369 },
	{ 0xc32a, 0xc344, PDF_CMAP_RANGE, 13978 },
	{ 0xc345, 0xc345, PDF_CMAP_SINGLE, 2371 },
	{ 0xc346, 0xc367, PDF_CMAP_RANGE, 14005 },
	{ 0xc368, 0xc369, PDF_CMAP_RANGE, 2372 },
	{ 0xc36a, 0xc36b, PDF_CMAP_RANGE, 14039 },
	{ 0xc36c, 0xc36c, PDF_CMAP_SINGLE, 2374 },
	{ 0xc36d, 0xc36f, PDF_CMAP_RANGE, 14041 },
	{ 0xc370, 0xc370, PDF_CMAP_SINGLE, 2375 },
	{ 0xc371, 0xc371, PDF_CMAP_SINGLE, 14044 },
	{ 0xc372, 0xc372, PDF_CMAP_SINGLE, 2376 },
	{ 0xc373, 0xc377, PDF_CMAP_RANGE, 14045 },
	{ 0xc378, 0xc379, PDF_CMAP_RANGE, 2377 },
	{ 0xc37a, 0xc37b, PDF_CMAP_RANGE, 14050 },
	{ 0xc37c, 0xc37d, PDF_CMAP_RANGE, 2379 },
	{ 0xc37e, 0xc383, PDF_CMAP_RANGE, 14052 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 2381 },
	{ 0xc385, 0xc387, PDF_CMAP_RANGE, 14058 },
	{ 0xc388, 0xc388, PDF_CMAP_SINGLE, 2382 },
	{ 0xc389, 0xc38b, PDF_CMAP_RANGE, 14061 },
	{ 0xc38c, 0xc38c, PDF_CMAP_SINGLE, 2383 },
	{ 0xc38d, 0xc3bf, PDF_CMAP_RANGE, 14064 },
	{ 0xc3c0, 0xc3c0, PDF_CMAP_SINGLE, 2384 },
	{ 0xc3c1, 0xc3d7, PDF_CMAP_RANGE, 14115 },
	{ 0xc3d8, 0xc3d9, PDF_CMAP_RANGE, 2385 },
	{ 0xc3da, 0xc3db, PDF_CMAP_RANGE, 14138 },
	{ 0xc3dc, 0xc3dc, PDF_CMAP_SINGLE, 2387 },
	{ 0xc3dd, 0xc3de, PDF_CMAP_RANGE, 14140 },
	{ 0xc3df, 0xc3e0, PDF_CMAP_RANGE, 2388 },
	{ 0xc3e1, 0xc3e1, PDF_CMAP_SINGLE, 14142 },
	{ 0xc3e2, 0xc3e2, PDF_CMAP_SINGLE, 2390 },
	{ 0xc3e3, 0xc3e7, PDF_CMAP_RANGE, 14143 },
	{ 0xc3e8, 0xc3e9, PDF_CMAP_RANGE, 2391 },
	{ 0xc3ea, 0xc3ec, PDF_CMAP_RANGE, 14148 },
	{ 0xc3ed, 0xc3ed, PDF_CMAP_SINGLE, 2393 },
	{ 0xc3ee, 0xc3f3, PDF_CMAP_RANGE, 14151 },
	{ 0xc3f4, 0xc3f5, PDF_CMAP_RANGE, 2394 },
	{ 0xc3f6, 0xc3f7, PDF_CMAP_RANGE, 14157 },
	{ 0xc3f8, 0xc3f8, PDF_CMAP_SINGLE, 2396 },
	{ 0xc3f9, 0xc3ff, PDF_CMAP_RANGE, 14159 },
	{ 0xc400, 0xc407, PDF_CMAP_RANGE, 14166 },
	{ 0xc408, 0xc408, PDF_CMAP_SINGLE, 2397 },
	{ 0xc409, 0xc40f, PDF_CMAP_RANGE, 14174 },
	{ 0xc410, 0xc410, PDF_CMAP_SINGLE, 2398 },
	{ 0xc411, 0xc423, PDF_CMAP_RANGE, 14181 },
	{ 0xc424, 0xc424, PDF_CMAP_SINGLE, 2399 },
	{ 0xc425, 0xc42b, PDF_CMAP_RANGE, 14200 },
	{ 0xc42c, 0xc42c, PDF_CMAP_SINGLE, 2400 },
	{ 0xc42d, 0xc42f, PDF_CMAP_RANGE, 14207 },
	{ 0xc430, 0xc430, PDF_CMAP_SINGLE, 2401 },
	{ 0xc431, 0xc433, PDF_CMAP_RANGE, 14210 },
	{ 0xc434, 0xc434, PDF_CMAP_SINGLE, 2402 },
	{ 0xc435, 0xc43b, PDF_CMAP_RANGE, 14213 },
	{ 0xc43c, 0xc43d, PDF_CMAP_RANGE, 2403 },
	{ 0xc43e, 0xc447, PDF_CMAP_RANGE, 14220 },
	{ 0xc448, 0xc448, PDF_CMAP_SINGLE, 2405 },
	{ 0xc449, 0xc463, PDF_CMAP_RANGE, 14230 },
	{ 0xc464, 0xc465, PDF_CMAP_RANGE, 2406 },
	{ 0xc466, 0xc467, PDF_CMAP_RANGE, 14257 },
	{ 0xc468, 0xc468, PDF_CMAP_SINGLE, 2408 },
	{ 0xc469, 0xc46b, PDF_CMAP_RANGE, 14259 },
	{ 0xc46c, 0xc46c, PDF_CMAP_SINGLE, 2409 },
	{ 0xc46d, 0xc473, PDF_CMAP_RANGE, 14262 },
	{ 0xc474, 0xc475, PDF_CMAP_RANGE, 2410 },
	{ 0xc476, 0xc478, PDF_CMAP_RANGE, 14269 },
	{ 0xc479, 0xc479, PDF_CMAP_SINGLE, 2412 },
	{ 0xc47a, 0xc47f, PDF_CMAP_RANGE, 14272 },
	{ 0xc480, 0xc480, PDF_CMAP_SINGLE, 2413 },
	{ 0xc481, 0xc493, PDF_CMAP_RANGE, 14278 },
	{ 0xc494, 0xc494, PDF_CMAP_SINGLE, 2414 },
	{ 0xc495, 0xc49b, PDF_CMAP_RANGE, 14297 },
	{ 0xc49c, 0xc49c, PDF_CMAP_SINGLE, 2415 },
	{ 0xc49d, 0xc4b7, PDF_CMAP_RANGE, 14304 },
	{ 0xc4b8, 0xc4b8, PDF_CMAP_SINGLE, 2416 },
	{ 0xc4b9, 0xc4bb, PDF_CMAP_RANGE, 14331 },
	{ 0xc4bc, 0xc4bc, PDF_CMAP_SINGLE, 2417 },
	{ 0xc4bd, 0xc4e8, PDF_CMAP_RANGE, 14334 },
	{ 0xc4e9, 0xc4e9, PDF_CMAP_SINGLE, 2418 },
	{ 0xc4ea, 0xc4ef, PDF_CMAP_RANGE, 14378 },
	{ 0xc4f0, 0xc4f1, PDF_CMAP_RANGE, 2419 },
	{ 0xc4f2, 0xc4f3, PDF_CMAP_RANGE, 14384 },
	{ 0xc4f4, 0xc4f4, PDF_CMAP_SINGLE, 2421 },
	{ 0xc4f5, 0xc4f7, PDF_CMAP_RANGE, 14386 },
	{ 0xc4f8, 0xc4f8, PDF_CMAP_SINGLE, 2422 },
	{ 0xc4f9, 0xc4f9, PDF_CMAP_SINGLE, 14389 },
	{ 0xc4fa, 0xc4fa, PDF_CMAP_SINGLE, 2423 },
	{ 0xc4fb, 0xc4fe, PDF_CMAP_RANGE, 14390 },
	{ 0xc4ff, 0xc4ff, PDF_CMAP_SINGLE, 2424 },
	{ 0xc500, 0xc501, PDF_CMAP_RANGE, 2425 },
	{ 0xc502, 0xc50b, PDF_CMAP_RANGE, 14394 },
	{ 0xc50c, 0xc50c, PDF_CMAP_SINGLE, 2427 },
	{ 0xc50d, 0xc50f, PDF_CMAP_RANGE, 14404 },
	{ 0xc510, 0xc510, PDF_CMAP_SINGLE, 2428 },
	{ 0xc511, 0xc513, PDF_CMAP_RANGE, 14407 },
	{ 0xc514, 0xc514, PDF_CMAP_SINGLE, 2429 },
	{ 0xc515, 0xc51b, PDF_CMAP_RANGE, 14410 },
	{ 0xc51c, 0xc51c, PDF_CMAP_SINGLE, 2430 },
	{ 0xc51d, 0xc527, PDF_CMAP_RANGE, 14417 },
	{ 0xc528, 0xc529, PDF_CMAP_RANGE, 2431 },
	{ 0xc52a, 0xc52b, PDF_CMAP_RANGE, 14428 },
	{ 0xc52c, 0xc52c, PDF_CMAP_SINGLE, 2433 },
	{ 0xc52d, 0xc52f, PDF_CMAP_RANGE, 14430 },
	{ 0xc530, 0xc530, PDF_CMAP_SINGLE, 2434 },
	{ 0xc531, 0xc537, PDF_CMAP_RANGE, 14433 },
	{ 0xc538, 0xc539, PDF_CMAP_RANGE, 2435 },
	{ 0xc53a, 0xc53a, PDF_CMAP_SINGLE, 14440 },
	{ 0xc53b, 0xc53b, PDF_CMAP_SINGLE, 2437 },
	{ 0xc53c, 0xc53c, PDF_CMAP_SINGLE, 14441 },
	{ 0xc53d, 0xc53d, PDF_CMAP_SINGLE, 2438 },
	{ 0xc53e, 0xc543, PDF_CMAP_RANGE, 14442 },
	{ 0xc544, 0xc545, PDF_CMAP_RANGE, 2439 },
	{ 0xc546, 0xc547, PDF_CMAP_RANGE, 14448 },
	{ 0xc548, 0xc54a, PDF_CMAP_RANGE, 2441 },
	{ 0xc54b, 0xc54b, PDF_CMAP_SINGLE, 14450 },
	{ 0xc54c, 0xc54e, PDF_CMAP_RANGE, 2444 },
	{ 0xc54f, 0xc552, PDF_CMAP_RANGE, 14451 },
	{ 0xc553, 0xc555, PDF_CMAP_RANGE, 2447 },
	{ 0xc556, 0xc556, PDF_CMAP_SINGLE, 14455 },
	{ 0xc557, 0xc559, PDF_CMAP_RANGE, 2450 },
	{ 0xc55a, 0xc55c, PDF_CMAP_RANGE, 14456 },
	{ 0xc55d, 0xc55e, PDF_CMAP_RANGE, 2453 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 14459 },
	{ 0xc560, 0xc561, PDF_CMAP_RANGE, 2455 },
	{ 0xc562, 0xc563, PDF_CMAP_RANGE, 14460 },
	{ 0xc564, 0xc564, PDF_CMAP_SINGLE, 2457 },
	{ 0xc565, 0xc567, PDF_CMAP_RANGE, 14462 },
	{ 0xc568, 0xc568, PDF_CMAP_SINGLE, 2458 },
	{ 0xc569, 0xc56f, PDF_CMAP_RANGE, 14465 },
	{ 0xc570, 0xc571, PDF_CMAP_RANGE, 2459 },
	{ 0xc572, 0xc572, PDF_CMAP_SINGLE, 14472 },
	{ 0xc573, 0xc575, PDF_CMAP_RANGE, 2461 },
	{ 0xc576, 0xc57b, PDF_CMAP_RANGE, 14473 },
	{ 0xc57c, 0xc57d, PDF_CMAP_RANGE, 2464 },
	{ 0xc57e, 0xc57f, PDF_CMAP_RANGE, 14479 },
	{ 0xc580, 0xc580, PDF_CMAP_SINGLE, 2466 },
	{ 0xc581, 0xc583, PDF_CMAP_RANGE, 14481 },
	{ 0xc584, 0xc584, PDF_CMAP_SINGLE, 2467 },
	{ 0xc585, 0xc586, PDF_CMAP_RANGE, 14484 },
	{ 0xc587, 0xc587, PDF_CMAP_SINGLE, 2468 },
	{ 0xc588, 0xc58b, PDF_CMAP_RANGE, 14486 },
	{ 0xc58c, 0xc58d, PDF_CMAP_RANGE, 2469 },
	{ 0xc58e, 0xc58e, PDF_CMAP_SINGLE, 14490 },
	{ 0xc58f, 0xc58f, PDF_CMAP_SINGLE, 2471 },
	{ 0xc590, 0xc590, PDF_CMAP_SINGLE, 14491 },
	{ 0xc591, 0xc591, PDF_CMAP_SINGLE, 2472 },
	{ 0xc592, 0xc594, PDF_CMAP_RANGE, 14492 },
	{ 0xc595, 0xc595, PDF_CMAP_SINGLE, 2473 },
	{ 0xc596, 0xc596, PDF_CMAP_SINGLE, 14495 },
	{ 0xc597, 0xc598, PDF_CMAP_RANGE, 2474 },
	{ 0xc599, 0xc59b, PDF_CMAP_RANGE, 14496 },
	{ 0xc59c, 0xc59c, PDF_CMAP_SINGLE, 2476 },
	{ 0xc59d, 0xc59f, PDF_CMAP_RANGE, 14499 },
	{ 0xc5a0, 0xc5a0, PDF_CMAP_SINGLE, 2477 },
	{ 0xc5a1, 0xc5a8, PDF_CMAP_RANGE, 14502 },
	{ 0xc5a9, 0xc5a9, PDF_CMAP_SINGLE, 2478 },
	{ 0xc5aa, 0xc5b3, PDF_CMAP_RANGE, 14510 },
	{ 0xc5b4, 0xc5b5, PDF_CMAP_RANGE, 2479 },
	{ 0xc5b6, 0xc5b7, PDF_CMAP_RANGE, 14520 },
	{ 0xc5b8, 0xc5b9, PDF_CMAP_RANGE, 2481 },
	{ 0xc5ba, 0xc5ba, PDF_CMAP_SINGLE, 14522 },
	{ 0xc5bb, 0xc5be, PDF_CMAP_RANGE, 2483 },
	{ 0xc5bf, 0xc5c3, PDF_CMAP_RANGE, 14523 },
	{ 0xc5c4, 0xc5ca, PDF_CMAP_RANGE, 2487 },
	{ 0xc5cb, 0xc5cb, PDF_CMAP_SINGLE, 14528 },
	{ 0xc5cc, 0xc5cc, PDF_CMAP_SINGLE, 2494 },
	{ 0xc5cd, 0xc5cd, PDF_CMAP_SINGLE, 14529 },
	{ 0xc5ce, 0xc5ce, PDF_CMAP_SINGLE, 2495 },
	{ 0xc5cf, 0xc5cf, PDF_CMAP_SINGLE, 14530 },
	{ 0xc5d0, 0xc5d1, PDF_CMAP_RANGE, 2496 },
	{ 0xc5d2, 0xc5d3, PDF_CMAP_RANGE, 14531 },
	{ 0xc5d4, 0xc5d4, PDF_CMAP_SINGLE, 2498 },
	{ 0xc5d5, 0xc5d7, PDF_CMAP_RANGE, 14533 },
	{ 0xc5d8, 0xc5d8, PDF_CMAP_SINGLE, 2499 },
	{ 0xc5d9, 0xc5df, PDF_CMAP_RANGE, 14536 },
	{ 0xc5e0, 0xc5e1, PDF_CMAP_RANGE, 2500 },
	{ 0xc5e2, 0xc5e2, PDF_CMAP_SINGLE, 14543 },
	{ 0xc5e3, 0xc5e3, PDF_CMAP_SINGLE, 2502 },
	{ 0xc5e4, 0xc5e4, PDF_CMAP_SINGLE, 14544 },
	{ 0xc5e5, 0xc5e5, PDF_CMAP_SINGLE, 2503 },
	{ 0xc5e6, 0xc5eb, PDF_CMAP_RANGE, 14545 },
	{ 0xc5ec, 0xc5ee, PDF_CMAP_RANGE, 2504 },
	{ 0xc5ef, 0xc5ef, PDF_CMAP_SINGLE, 14551 },
	{ 0xc5f0, 0xc5f0, PDF_CMAP_SINGLE, 2507 },
	{ 0xc5f1, 0xc5f3, PDF_CMAP_RANGE, 14552 },
	{ 0xc5f4, 0xc5f4, PDF_CMAP_SINGLE, 2508 },
	{ 0xc5f5, 0xc5f5, PDF_CMAP_SINGLE, 14555 },
	{ 0xc5f6, 0xc5f7, PDF_CMAP_RANGE, 2509 },
	{ 0xc5f8, 0xc5fb, PDF_CMAP_RANGE, 14556 },
	{ 0xc5fc, 0xc5ff, PDF_CMAP_RANGE, 2511 },
	{ 0xc600, 0xc601, PDF_CMAP_RANGE, 2515 },
	{ 0xc602, 0xc604, PDF_CMAP_RANGE, 14560 },
	{ 0xc605, 0xc608, PDF_CMAP_RANGE, 2517 },
	{ 0xc609, 0xc60b, PDF_CMAP_RANGE, 14563 },
	{ 0xc60c, 0xc60c, PDF_CMAP_SINGLE, 2521 },
	{ 0xc60d, 0xc60f, PDF_CMAP_RANGE, 14566 },
	{ 0xc610, 0xc610, PDF_CMAP_SINGLE, 2522 },
	{ 0xc611, 0xc617, PDF_CMAP_RANGE, 14569 },
	{ 0xc618, 0xc619, PDF_CMAP_RANGE, 2523 },
	{ 0xc61a, 0xc61a, PDF_CMAP_SINGLE, 14576 },
	{ 0xc61b, 0xc61c, PDF_CMAP_RANGE, 2525 },
	{ 0xc61d, 0xc623, PDF_CMAP_RANGE, 14577 },
	{ 0xc624, 0xc625, PDF_CMAP_RANGE, 2527 },
	{ 0xc626, 0xc627, PDF_CMAP_RANGE, 14584 },
	{ 0xc628, 0xc628, PDF_CMAP_SINGLE, 2529 },
	{ 0xc629, 0xc62b, PDF_CMAP_RANGE, 14586 },
	{ 0xc62c, 0xc62e, PDF_CMAP_RANGE, 2530 },
	{ 0xc62f, 0xc62f, PDF_CMAP_SINGLE, 14589 },
	{ 0xc630, 0xc630, PDF_CMAP_SINGLE, 2533 },
	{ 0xc631, 0xc632, PDF_CMAP_RANGE, 14590 },
	{ 0xc633, 0xc635, PDF_CMAP_RANGE, 2534 },
	{ 0xc636, 0xc636, PDF_CMAP_SINGLE, 14592 },
	{ 0xc637, 0xc637, PDF_CMAP_SINGLE, 2537 },
	{ 0xc638, 0xc638, PDF_CMAP_SINGLE, 14593 },
	{ 0xc639, 0xc639, PDF_CMAP_SINGLE, 2538 },
	{ 0xc63a, 0xc63a, PDF_CMAP_SINGLE, 14594 },
	{ 0xc63b, 0xc63b, PDF_CMAP_SINGLE, 2539 },
	{ 0xc63c, 0xc63f, PDF_CMAP_RANGE, 14595 },
	{ 0xc640, 0xc641, PDF_CMAP_RANGE, 2540 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 14599 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 2542 },
	{ 0xc645, 0xc647, PDF_CMAP_RANGE, 14601 },
	{ 0xc648, 0xc648, PDF_CMAP_SINGLE, 2543 },
	{ 0xc649, 0xc64f, PDF_CMAP_RANGE, 14604 },
	{ 0xc650, 0xc651, PDF_CMAP_RANGE, 2544 },
	{ 0xc652, 0xc652, PDF_CMAP_SINGLE, 14611 },
	{ 0xc653, 0xc655, PDF_CMAP_RANGE, 2546 },
	{ 0xc656, 0xc65b, PDF_CMAP_RANGE, 14612 },
	{ 0xc65c, 0xc65d, PDF_CMAP_RANGE, 2549 },
	{ 0xc65e, 0xc65f, PDF_CMAP_RANGE, 14618 },
	{ 0xc660, 0xc660, PDF_CMAP_SINGLE, 2551 },
	{ 0xc661, 0xc66b, PDF_CMAP_RANGE, 14620 },
	{ 0xc66c, 0xc66c, PDF_CMAP_SINGLE, 2552 },
	{ 0xc66d, 0xc66e, PDF_CMAP_RANGE, 14631 },
	{ 0xc66f, 0xc66f, PDF_CMAP_SINGLE, 2553 },
	{ 0xc670, 0xc670, PDF_CMAP_SINGLE, 14633 },
	{ 0xc671, 0xc671, PDF_CMAP_SINGLE, 2554 },
	{ 0xc672, 0xc677, PDF_CMAP_RANGE, 14634 },
	{ 0xc678, 0xc679, PDF_CMAP_RANGE, 2555 },
	{ 0xc67a, 0xc67b, PDF_CMAP_RANGE, 14640 },
	{ 0xc67c, 0xc67c, PDF_CMAP_SINGLE, 2557 },
	{ 0xc67d, 0xc67f, PDF_CMAP_RANGE, 14642 },
	{ 0xc680, 0xc680, PDF_CMAP_SINGLE, 2558 },
	{ 0xc681, 0xc687, PDF_CMAP_RANGE, 14645 },
	{ 0xc688, 0xc689, PDF_CMAP_RANGE, 2559 },
	{ 0xc68a, 0xc68a, PDF_CMAP_SINGLE, 14652 },
	{ 0xc68b, 0xc68b, PDF_CMAP_SINGLE, 2561 },
	{ 0xc68c, 0xc68c, PDF_CMAP_SINGLE, 14653 },
	{ 0xc68d, 0xc68d, PDF_CMAP_SINGLE, 2562 },
	{ 0xc68e, 0xc693, PDF_CMAP_RANGE, 14654 },
	{ 0xc694, 0xc695, PDF_CMAP_RANGE, 2563 },
	{ 0xc696, 0xc697, PDF_CMAP_RANGE, 14660 },
	{ 0xc698, 0xc698, PDF_CMAP_SINGLE, 2565 },
	{ 0xc699, 0xc69b, PDF_CMAP_RANGE, 14662 },
	{ 0xc69c, 0xc69c, PDF_CMAP_SINGLE, 2566 },
	{ 0xc69d, 0xc6a3, PDF_CMAP_RANGE, 14665 },
	{ 0xc6a4, 0xc6a5, PDF_CMAP_RANGE, 2567 },
	{ 0xc6a6, 0xc6a6, PDF_CMAP_SINGLE, 14672 },
	{ 0xc6a7, 0xc6a7, PDF_CMAP_SINGLE, 2569 },
	{ 0xc6a8, 0xc6a8, PDF_CMAP_SINGLE, 14673 },
	{ 0xc6a9, 0xc6a9, PDF_CMAP_SINGLE, 2570 },
	{ 0xc6aa, 0xc6af, PDF_CMAP_RANGE, 14674 },
	{ 0xc6b0, 0xc6b1, PDF_CMAP_RANGE, 2571 },
	{ 0xc6b2, 0xc6b3, PDF_CMAP_RANGE, 14680 },
	{ 0xc6b4, 0xc6b4, PDF_CMAP_SINGLE, 2573 },
	{ 0xc6b5, 0xc6b7, PDF_CMAP_RANGE, 14682 },
	{ 0xc6b8, 0xc6ba, PDF_CMAP_RANGE, 2574 },
	{ 0xc6bb, 0xc6bf, PDF_CMAP_RANGE, 14685 },
	{ 0xc6c0, 0xc6c1, PDF_CMAP_RANGE, 2577 },
	{ 0xc6c2, 0xc6c2, PDF_CMAP_SINGLE, 14690 },
	{ 0xc6c3, 0xc6c3, PDF_CMAP_SINGLE, 2579 },
	{ 0xc6c4, 0xc6c4, PDF_CMAP_SINGLE, 14691 },
	{ 0xc6c5, 0xc6c5, PDF_CMAP_SINGLE, 2580 },
	{ 0xc6c6, 0xc6cb, PDF_CMAP_RANGE, 14692 },
	{ 0xc6cc, 0xc6cd, PDF_CMAP_RANGE, 2581 },
	{ 0xc6ce, 0xc6cf, PDF_CMAP_RANGE, 14698 },
	{ 0xc6d0, 0xc6d0, PDF_CMAP_SINGLE, 2583 },
	{ 0xc6d1, 0xc6d3, PDF_CMAP_RANGE, 14700 },
	{ 0xc6d4, 0xc6d4, PDF_CMAP_SINGLE, 2584 },
	{ 0xc6d5, 0xc6db, PDF_CMAP_RANGE, 14703 },
	{ 0xc6dc, 0xc6dd, PDF_CMAP_RANGE, 2585 },
	{ 0xc6de, 0xc6df, PDF_CMAP_RANGE, 14710 },
	{ 0xc6e0, 0xc6e1, PDF_CMAP_RANGE, 2587 },
	{ 0xc6e2, 0xc6e7, PDF_CMAP_RANGE, 14712 },
	{ 0xc6e8, 0xc6e9, PDF_CMAP_RANGE, 2589 },
	{ 0xc6ea, 0xc6eb, PDF_CMAP_RANGE, 14718 },
	{ 0xc6ec, 0xc6ec, PDF_CMAP_SINGLE, 2591 },
	{ 0xc6ed, 0xc6ef, PDF_CMAP_RANGE, 14720 },
	{ 0xc6f0, 0xc6f0, PDF_CMAP_SINGLE, 2592 },
	{ 0xc6f1, 0xc6f7, PDF_CMAP_RANGE, 14723 },
	{ 0xc6f8, 0xc6f9, PDF_CMAP_RANGE, 2593 },
	{ 0xc6fa, 0xc6fc, PDF_CMAP_RANGE, 14730 },
	{ 0xc6fd, 0xc6fd, PDF_CMAP_SINGLE, 2595 },
	{ 0xc6fe, 0xc6ff, PDF_CMAP_RANGE, 14733 },
	{ 0xc700, 0xc703, PDF_CMAP_RANGE, 14735 },
	{ 0xc704, 0xc705, PDF_CMAP_RANGE, 2596 },
	{ 0xc706, 0xc707, PDF_CMAP_RANGE, 14739 },
	{ 0xc708, 0xc708, PDF_CMAP_SINGLE, 2598 },
	{ 0xc709, 0xc70b, PDF_CMAP_RANGE, 14741 },
	{ 0xc70c, 0xc70c, PDF_CMAP_SINGLE, 2599 },
	{ 0xc70d, 0xc713, PDF_CMAP_RANGE, 14744 },
	{ 0xc714, 0xc715, PDF_CMAP_RANGE, 2600 },
	{ 0xc716, 0xc716, PDF_CMAP_SINGLE, 14751 },
	{ 0xc717, 0xc717, PDF_CMAP_SINGLE, 2602 },
	{ 0xc718, 0xc718, PDF_CMAP_SINGLE, 14752 },
	{ 0xc719, 0xc719, PDF_CMAP_SINGLE, 2603 },
	{ 0xc71a, 0xc71f, PDF_CMAP_RANGE, 14753 },
	{ 0xc720, 0xc721, PDF_CMAP_RANGE, 2604 },
	{ 0xc722, 0xc723, PDF_CMAP_RANGE, 14759 },
	{ 0xc724, 0xc724, PDF_CMAP_SINGLE, 2606 },
	{ 0xc725, 0xc727, PDF_CMAP_RANGE, 14761 },
	{ 0xc728, 0xc728, PDF_CMAP_SINGLE, 2607 },
	{ 0xc729, 0xc72f, PDF_CMAP_RANGE, 14764 },
	{ 0xc730, 0xc731, PDF_CMAP_RANGE, 2608 },
	{ 0xc732, 0xc732, PDF_CMAP_SINGLE, 14771 },
	{ 0xc733, 0xc733, PDF_CMAP_SINGLE, 2610 },
	{ 0xc734, 0xc734, PDF_CMAP_SINGLE, 14772 },
	{ 0xc735, 0xc735, PDF_CMAP_SINGLE, 2611 },
	{ 0xc736, 0xc736, PDF_CMAP_SINGLE, 14773 },
	{ 0xc737, 0xc737, PDF_CMAP_SINGLE, 2612 },
	{ 0xc738, 0xc73b, PDF_CMAP_RANGE, 14774 },
	{ 0xc73c, 0xc73d, PDF_CMAP_RANGE, 2613 },
	{ 0xc73e, 0xc73f, PDF_CMAP_RANGE, 14778 },
	{ 0xc740, 0xc740, PDF_CMAP_SINGLE, 2615 },
	{ 0xc741, 0xc743, PDF_CMAP_RANGE, 14780 },
	{ 0xc744, 0xc744, PDF_CMAP_SINGLE, 2616 },
	{ 0xc745, 0xc749, PDF_CMAP_RANGE, 14783 },
	{ 0xc74a, 0xc74a, PDF_CMAP_SINGLE, 2617 },
	{ 0xc74b, 0xc74b, PDF_CMAP_SINGLE, 14788 },
	{ 0xc74c, 0xc74d, PDF_CMAP_RANGE, 2618 },
	{ 0xc74e, 0xc74e, PDF_CMAP_SINGLE, 14789 },
	{ 0xc74f, 0xc74f, PDF_CMAP_SINGLE, 2620 },
	{ 0xc750, 0xc750, PDF_CMAP_SINGLE, 14790 },
	{ 0xc751, 0xc758, PDF_CMAP_RANGE, 2621 },
	{ 0xc759, 0xc75b, PDF_CMAP_RANGE, 14791 },
	{ 0xc75c, 0xc75c, PDF_CMAP_SINGLE, 2629 },
	{ 0xc75d, 0xc75f, PDF_CMAP_RANGE, 14794 },
	{ 0xc760, 0xc760, PDF_CMAP_SINGLE, 2630 },
	{ 0xc761, 0xc767, PDF_CMAP_RANGE, 14797 },
	{ 0xc768, 0xc768, PDF_CMAP_SINGLE, 2631 },
	{ 0xc769, 0xc76a, PDF_CMAP_RANGE, 14804 },
	{ 0xc76b, 0xc76b, PDF_CMAP_SINGLE, 2632 },
	{ 0xc76c, 0xc773, PDF_CMAP_RANGE, 14806 },
	{ 0xc774, 0xc775, PDF_CMAP_RANGE, 2633 },
	{ 0xc776, 0xc777, PDF_CMAP_RANGE, 14814 },
	{ 0xc778, 0xc778, PDF_CMAP_SINGLE, 2635 },
	{ 0xc779, 0xc77b, PDF_CMAP_RANGE, 14816 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 2636 },
	{ 0xc77f, 0xc782, PDF_CMAP_RANGE, 14819 },
	{ 0xc783, 0xc785, PDF_CMAP_RANGE, 2639 },
	{ 0xc786, 0xc786, PDF_CMAP_SINGLE, 14823 },
	{ 0xc787, 0xc78a, PDF_CMAP_RANGE, 2642 },
	{ 0xc78b, 0xc78d, PDF_CMAP_RANGE, 14824 },
	{ 0xc78e, 0xc78e, PDF_CMAP_SINGLE, 2646 },
	{ 0xc78f, 0xc78f, PDF_CMAP_SINGLE, 14827 },
	{ 0xc790, 0xc791, PDF_CMAP_RANGE, 2647 },
	{ 0xc792, 0xc793, PDF_CMAP_RANGE, 14828 },
	{ 0xc794, 0xc794, PDF_CMAP_SINGLE, 2649 },
	{ 0xc795, 0xc795, PDF_CMAP_SINGLE, 14830 },
	{ 0xc796, 0xc798, PDF_CMAP_RANGE, 2650 },
	{ 0xc799, 0xc799, PDF_CMAP_SINGLE, 14831 },
	{ 0xc79a, 0xc79a, PDF_CMAP_SINGLE, 2653 },
	{ 0xc79b, 0xc79f, PDF_CMAP_RANGE, 14832 },
	{ 0xc7a0, 0xc7a1, PDF_CMAP_RANGE, 2654 },
	{ 0xc7a2, 0xc7a2, PDF_CMAP_SINGLE, 14837 },
	{ 0xc7a3, 0xc7a6, PDF_CMAP_RANGE, 2656 },
	{ 0xc7a7, 0xc7ab, PDF_CMAP_RANGE, 14838 },
	{ 0xc7ac, 0xc7ad, PDF_CMAP_RANGE, 2660 },
	{ 0xc7ae, 0xc7af, PDF_CMAP_RANGE, 14843 },
	{ 0xc7b0, 0xc7b0, PDF_CMAP_SINGLE, 2662 },
	{ 0xc7b1, 0xc7b3, PDF_CMAP_RANGE, 14845 },
	{ 0xc7b4, 0xc7b4, PDF_CMAP_SINGLE, 2663 },
	{ 0xc7b5, 0xc7bb, PDF_CMAP_RANGE, 14848 },
	{ 0xc7bc, 0xc7bd, PDF_CMAP_RANGE, 2664 },
	{ 0xc7be, 0xc7be, PDF_CMAP_SINGLE, 14855 },
	{ 0xc7bf, 0xc7c1, PDF_CMAP_RANGE, 2666 },
	{ 0xc7c2, 0xc7c7, PDF_CMAP_RANGE, 14856 },
	{ 0xc7c8, 0xc7c9, PDF_CMAP_RANGE, 2669 },
	{ 0xc7ca, 0xc7cb, PDF_CMAP_RANGE, 14862 },
	{ 0xc7cc, 0xc7cc, PDF_CMAP_SINGLE, 2671 },
	{ 0xc7cd, 0xc7cd, PDF_CMAP_SINGLE, 14864 },
	{ 0xc7ce, 0xc7ce, PDF_CMAP_SINGLE, 2672 },
	{ 0xc7cf, 0xc7cf, PDF_CMAP_SINGLE, 14865 },
	{ 0xc7d0, 0xc7d0, PDF_CMAP_SINGLE, 2673 },
	{ 0xc7d1, 0xc7d7, PDF_CMAP_RANGE, 14866 },
	{ 0xc7d8, 0xc7d8, PDF_CMAP_SINGLE, 2674 },
	{ 0xc7d9, 0xc7dc, PDF_CMAP_RANGE, 14873 },
	{ 0xc7dd, 0xc7dd, PDF_CMAP_SINGLE, 2675 },
	{ 0xc7de, 0xc7e3, PDF_CMAP_RANGE, 14877 },
	{ 0xc7e4, 0xc7e4, PDF_CMAP_SINGLE, 2676 },
	{ 0xc7e5, 0xc7e7, PDF_CMAP_RANGE, 14883 },
	{ 0xc7e8, 0xc7e8, PDF_CMAP_SINGLE, 2677 },
	{ 0xc7e9, 0xc7eb, PDF_CMAP_RANGE, 14886 },
	{ 0xc7ec, 0xc7ec, PDF_CMAP_SINGLE, 2678 },
	{ 0xc7ed, 0xc7ff, PDF_CMAP_RANGE, 14889 },
	{ 0xc800, 0xc801, PDF_CMAP_RANGE, 2679 },
	{ 0xc802, 0xc803, PDF_CMAP_RANGE, 14908 },
	{ 0xc804, 0xc804, PDF_CMAP_SINGLE, 2681 },
	{ 0xc805, 0xc807, PDF_CMAP_RANGE, 14910 },
	{ 0xc808, 0xc808, PDF_CMAP_SINGLE, 2682 },
	{ 0xc809, 0xc809, PDF_CMAP_SINGLE, 14913 },
	{ 0xc80a, 0xc80a, PDF_CMAP_SINGLE, 2683 },
	{ 0xc80b, 0xc80f, PDF_CMAP_RANGE, 14914 },
	{ 0xc810, 0xc811, PDF_CMAP_RANGE, 2684 },
	{ 0xc812, 0xc812, PDF_CMAP_SINGLE, 14919 },
	{ 0xc813, 0xc813, PDF_CMAP_SINGLE, 2686 },
	{ 0xc814, 0xc814, PDF_CMAP_SINGLE, 14920 },
	{ 0xc815, 0xc816, PDF_CMAP_RANGE, 2687 },
	{ 0xc817, 0xc81b, PDF_CMAP_RANGE, 14921 },
	{ 0xc81c, 0xc81d, PDF_CMAP_RANGE, 2689 },
	{ 0xc81e, 0xc81f, PDF_CMAP_RANGE, 14926 },
	{ 0xc820, 0xc820, PDF_CMAP_SINGLE, 2691 },
	{ 0xc821, 0xc823, PDF_CMAP_RANGE, 14928 },
	{ 0xc824, 0xc824, PDF_CMAP_SINGLE, 2692 },
	{ 0xc825, 0xc82b, PDF_CMAP_RANGE, 14931 },
	{ 0xc82c, 0xc82d, PDF_CMAP_RANGE, 2693 },
	{ 0xc82e, 0xc82e, PDF_CMAP_SINGLE, 14938 },
	{ 0xc82f, 0xc82f, PDF_CMAP_SINGLE, 2695 },
	{ 0xc830, 0xc830, PDF_CMAP_SINGLE, 14939 },
	{ 0xc831, 0xc831, PDF_CMAP_SINGLE, 2696 },
	{ 0xc832, 0xc837, PDF_CMAP_RANGE, 14940 },
	{ 0xc838, 0xc838, PDF_CMAP_SINGLE, 2697 },
	{ 0xc839, 0xc83b, PDF_CMAP_RANGE, 14946 },
	{ 0xc83c, 0xc83c, PDF_CMAP_SINGLE, 2698 },
	{ 0xc83d, 0xc83f, PDF_CMAP_RANGE, 14949 },
	{ 0xc840, 0xc840, PDF_CMAP_SINGLE, 2699 },
	{ 0xc841, 0xc847, PDF_CMAP_RANGE, 14952 },
	{ 0xc848, 0xc849, PDF_CMAP_RANGE, 2700 },
	{ 0xc84a, 0xc84b, PDF_CMAP_RANGE, 14959 },
	{ 0xc84c, 0xc84d, PDF_CMAP_RANGE, 2702 },
	{ 0xc84e, 0xc853, PDF_CMAP_RANGE, 14961 },
	{ 0xc854, 0xc854, PDF_CMAP_SINGLE, 2704 },
	{ 0xc855, 0xc86f, PDF_CMAP_RANGE, 14967 },
	{ 0xc870, 0xc871, PDF_CMAP_RANGE, 2705 },
	{ 0xc872, 0xc873, PDF_CMAP_RANGE, 14994 },
	{ 0xc874, 0xc874, PDF_CMAP_SINGLE, 2707 },
	{ 0xc875, 0xc877, PDF_CMAP_RANGE, 14996 },
	{ 0xc878, 0xc878, PDF_CMAP_SINGLE, 2708 },
	{ 0xc879, 0xc879, PDF_CMAP_SINGLE, 14999 },
	{ 0xc87a, 0xc87a, PDF_CMAP_SINGLE, 2709 },
	{ 0xc87b, 0xc87f, PDF_CMAP_RANGE, 15000 },
	{ 0xc880, 0xc881, PDF_CMAP_RANGE, 2710 },
	{ 0xc882, 0xc882, PDF_CMAP_SINGLE, 15005 },
	{ 0xc883, 0xc883, PDF_CMAP_SINGLE, 2712 },
	{ 0xc884, 0xc884, PDF_CMAP_SINGLE, 15006 },
	{ 0xc885, 0xc887, PDF_CMAP_RANGE, 2713 },
	{ 0xc888, 0xc88a, PDF_CMAP_RANGE, 15007 },
	{ 0xc88b, 0xc88d, PDF_CMAP_RANGE, 2716 },
	{ 0xc88e, 0xc893, PDF_CMAP_RANGE, 15010 },
	{ 0xc894, 0xc894, PDF_CMAP_SINGLE, 2719 },
	{ 0xc895, 0xc89c, PDF_CMAP_RANGE, 15016 },
	{ 0xc89d, 0xc89d, PDF_CMAP_SINGLE, 2720 },
	{ 0xc89e, 0xc89e, PDF_CMAP_SINGLE, 15024 },
	{ 0xc89f, 0xc89f, PDF_CMAP_SINGLE, 2721 },
	{ 0xc8a0, 0xc8a0, PDF_CMAP_SINGLE, 15025 },
	{ 0xc8a1, 0xc8a1, PDF_CMAP_SINGLE, 2722 },
	{ 0xc8a2, 0xc8a7, PDF_CMAP_RANGE, 15026 },
	{ 0xc8a8, 0xc8a8, PDF_CMAP_SINGLE, 2723 },
	{ 0xc8a9, 0xc8bb, PDF_CMAP_RANGE, 15032 },
	{ 0xc8bc, 0xc8bd, PDF_CMAP_RANGE, 2724 },
	{ 0xc8be, 0xc8c3, PDF_CMAP_RANGE, 15051 },
	{ 0xc8c4, 0xc8c4, PDF_CMAP_SINGLE, 2726 },
	{ 0xc8c5, 0xc8c7, PDF_CMAP_RANGE, 15057 },
	{ 0xc8c8, 0xc8c8, PDF_CMAP_SINGLE, 2727 },
	{ 0xc8c9, 0xc8cb, PDF_CMAP_RANGE, 15060 },
	{ 0xc8cc, 0xc8cc, PDF_CMAP_SINGLE, 2728 },
	{ 0xc8cd, 0xc8d3, PDF_CMAP_RANGE, 15063 },
	{ 0xc8d4, 0xc8d5, PDF_CMAP_RANGE, 2729 },
	{ 0xc8d6, 0xc8d6, PDF_CMAP_SINGLE, 15070 },
	{ 0xc8d7, 0xc8d7, PDF_CMAP_SINGLE, 2731 },
	{ 0xc8d8, 0xc8d8, PDF_CMAP_SINGLE, 15071 },
	{ 0xc8d9, 0xc8d9, PDF_CMAP_SINGLE, 2732 },
	{ 0xc8da, 0xc8df, PDF_CMAP_RANGE, 15072 },
	{ 0xc8e0, 0xc8e1, PDF_CMAP_RANGE, 2733 },
	{ 0xc8e2, 0xc8e3, PDF_CMAP_RANGE, 15078 },
	{ 0xc8e4, 0xc8e4, PDF_CMAP_SINGLE, 2735 },
	{ 0xc8e5, 0xc8f4, PDF_CMAP_RANGE, 15080 },
	{ 0xc8f5, 0xc8f5, PDF_CMAP_SINGLE, 2736 },
	{ 0xc8f6, 0xc8fb, PDF_CMAP_RANGE, 15096 },
	{ 0xc8fc, 0xc8fd, PDF_CMAP_RANGE, 2737 },
	{ 0xc8fe, 0xc8ff, PDF_CMAP_RANGE, 15102 },
	{ 0xc900, 0xc900, PDF_CMAP_SINGLE, 2739 },
	{ 0xc901, 0xc903, PDF_CMAP_RANGE, 15104 },
	{ 0xc904, 0xc906, PDF_CMAP_RANGE, 2740 },
	{ 0xc907, 0xc90b, PDF_CMAP_RANGE, 15107 },
	{ 0xc90c, 0xc90d, PDF_CMAP_RANGE, 2743 },
	{ 0xc90e, 0xc90e, PDF_CMAP_SINGLE, 15112 },
	{ 0xc90f, 0xc90f, PDF_CMAP_SINGLE, 2745 },
	{ 0xc910, 0xc910, PDF_CMAP_SINGLE, 15113 },
	{ 0xc911, 0xc911, PDF_CMAP_SINGLE, 2746 },
	{ 0xc912, 0xc917, PDF_CMAP_RANGE, 15114 },
	{ 0xc918, 0xc918, PDF_CMAP_SINGLE, 2747 },
	{ 0xc919, 0xc92b, PDF_CMAP_RANGE, 15120 },
	{ 0xc92c, 0xc92c, PDF_CMAP_SINGLE, 2748 },
	{ 0xc92d, 0xc933, PDF_CMAP_RANGE, 15139 },
	{ 0xc934, 0xc934, PDF_CMAP_SINGLE, 2749 },
	{ 0xc935, 0xc94f, PDF_CMAP_RANGE, 15146 },
	{ 0xc950, 0xc951, PDF_CMAP_RANGE, 2750 },
	{ 0xc952, 0xc953, PDF_CMAP_RANGE, 15173 },
	{ 0xc954, 0xc954, PDF_CMAP_SINGLE, 2752 },
	{ 0xc955, 0xc957, PDF_CMAP_RANGE, 15175 },
	{ 0xc958, 0xc958, PDF_CMAP_SINGLE, 2753 },
	{ 0xc959, 0xc95f, PDF_CMAP_RANGE, 15178 },
	{ 0xc960, 0xc961, PDF_CMAP_RANGE, 2754 },
	{ 0xc962, 0xc962, PDF_CMAP_SINGLE, 15185 },
	{ 0xc963, 0xc963, PDF_CMAP_SINGLE, 2756 },
	{ 0xc964, 0xc96b, PDF_CMAP_RANGE, 15186 },
	{ 0xc96c, 0xc96c, PDF_CMAP_SINGLE, 2757 },
	{ 0xc96d, 0xc96f, PDF_CMAP_RANGE, 15194 },
	{ 0xc970, 0xc970, PDF_CMAP_SINGLE, 2758 },
	{ 0xc971, 0xc973, PDF_CMAP_RANGE, 15197 },
	{ 0xc974, 0xc974, PDF_CMAP_SINGLE, 2759 },
	{ 0xc975, 0xc97b, PDF_CMAP_RANGE, 15200 },
	{ 0xc97c, 0xc97c, PDF_CMAP_SINGLE, 2760 },
	{ 0xc97d, 0xc987, PDF_CMAP_RANGE, 15207 },
	{ 0xc988, 0xc989, PDF_CMAP_RANGE, 2761 },
	{ 0xc98a, 0xc98b, PDF_CMAP_RANGE, 15218 },
	{ 0xc98c, 0xc98c, PDF_CMAP_SINGLE, 2763 },
	{ 0xc98d, 0xc98f, PDF_CMAP_RANGE, 15220 },
	{ 0xc990, 0xc990, PDF_CMAP_SINGLE, 2764 },
	{ 0xc991, 0xc997, PDF_CMAP_RANGE, 15223 },
	{ 0xc998, 0xc999, PDF_CMAP_RANGE, 2765 },
	{ 0xc99a, 0xc99a, PDF_CMAP_SINGLE, 15230 },
	{ 0xc99b, 0xc99b, PDF_CMAP_SINGLE, 2767 },
	{ 0xc99c, 0xc99c, PDF_CMAP_SINGLE, 15231 },
	{ 0xc99d, 0xc99d, PDF_CMAP_SINGLE, 2768 },
	{ 0xc99e, 0xc9bf, PDF_CMAP_RANGE, 15232 },
	{ 0xc9c0, 0xc9c1, PDF_CMAP_RANGE, 2769 },
	{ 0xc9c2, 0xc9c3, PDF_CMAP_RANGE, 15266 },
	{ 0xc9c4, 0xc9c4, PDF_CMAP_SINGLE, 2771 },
	{ 0xc9c5, 0xc9c6, PDF_CMAP_RANGE, 15268 },
	{ 0xc9c7, 0xc9c8, PDF_CMAP_RANGE, 2772 },
	{ 0xc9c9, 0xc9c9, PDF_CMAP_SINGLE, 15270 },
	{ 0xc9ca, 0xc9ca, PDF_CMAP_SINGLE, 2774 },
	{ 0xc9cb, 0xc9cf, PDF_CMAP_RANGE, 15271 },
	{ 0xc9d0, 0xc9d1, PDF_CMAP_RANGE, 2775 },
	{ 0xc9d2, 0xc9d2, PDF_CMAP_SINGLE, 15276 },
	{ 0xc9d3, 0xc9d3, PDF_CMAP_SINGLE, 2777 },
	{ 0xc9d4, 0xc9d4, PDF_CMAP_SINGLE, 15277 },
	{ 0xc9d5, 0xc9d6, PDF_CMAP_RANGE, 2778 },
	{ 0xc9d7, 0xc9d8, PDF_CMAP_RANGE, 15278 },
	{ 0xc9d9, 0xc9da, PDF_CMAP_RANGE, 2780 },
	{ 0xc9db, 0xc9db, PDF_CMAP_SINGLE, 15280 },
	{ 0xc9dc, 0xc9dd, PDF_CMAP_RANGE, 2782 },
	{ 0xc9de, 0xc9df, PDF_CMAP_RANGE, 15281 },
	{ 0xc9e0, 0xc9e0, PDF_CMAP_SINGLE, 2784 },
	{ 0xc9e1, 0xc9e1, PDF_CMAP_SINGLE, 15283 },
	{ 0xc9e2, 0xc9e2, PDF_CMAP_SINGLE, 2785 },
	{ 0xc9e3, 0xc9e3, PDF_CMAP_SINGLE, 15284 },
	{ 0xc9e4, 0xc9e4, PDF_CMAP_SINGLE, 2786 },
	{ 0xc9e5, 0xc9e6, PDF_CMAP_RANGE, 15285 },
	{ 0xc9e7, 0xc9e7, PDF_CMAP_SINGLE, 2787 },
	{ 0xc9e8, 0xc9eb, PDF_CMAP_RANGE, 15287 },
	{ 0xc9ec, 0xc9ed, PDF_CMAP_RANGE, 2788 },
	{ 0xc9ee, 0xc9ee, PDF_CMAP_SINGLE, 15291 },
	{ 0xc9ef, 0xc9f1, PDF_CMAP_RANGE, 2790 },
	{ 0xc9f2, 0xc9f7, PDF_CMAP_RANGE, 15292 },
	{ 0xc9f8, 0xc9f9, PDF_CMAP_RANGE, 2793 },
	{ 0xc9fa, 0xc9fb, PDF_CMAP_RANGE, 15298 },
	{ 0xc9fc, 0xc9fc, PDF_CMAP_SINGLE, 2795 },
	{ 0xc9fd, 0xc9ff, PDF_CMAP_RANGE, 15300 },
	{ 0xca00, 0xca00, PDF_CMAP_SINGLE, 2796 },
	{ 0xca01, 0xca07, PDF_CMAP_RANGE, 15303 },
	{ 0xca08, 0xca09, PDF_CMAP_RANGE, 2797 },
	{ 0xca0a, 0xca0a, PDF_CMAP_SINGLE, 15310 },
	{ 0xca0b, 0xca0d, PDF_CMAP_RANGE, 2799 },
	{ 0xca0e, 0xca13, PDF_CMAP_RANGE, 15311 },
	{ 0xca14, 0xca14, PDF_CMAP_SINGLE, 2802 },
	{ 0xca15, 0xca17, PDF_CMAP_RANGE, 15317 },
	{ 0xca18, 0xca18, PDF_CMAP_SINGLE, 2803 },
	{ 0xca19, 0xca28, PDF_CMAP_RANGE, 15320 },
	{ 0xca29, 0xca29, PDF_CMAP_SINGLE, 2804 },
	{ 0xca2a, 0xca4b, PDF_CMAP_RANGE, 15336 },
	{ 0xca4c, 0xca4d, PDF_CMAP_RANGE, 2805 },
	{ 0xca4e, 0xca4f, PDF_CMAP_RANGE, 15370 },
	{ 0xca50, 0xca50, PDF_CMAP_SINGLE, 2807 },
	{ 0xca51, 0xca53, PDF_CMAP_RANGE, 15372 },
	{ 0xca54, 0xca54, PDF_CMAP_SINGLE, 2808 },
	{ 0xca55, 0xca5b, PDF_CMAP_RANGE, 15375 },
	{ 0xca5c, 0xca5d, PDF_CMAP_RANGE, 2809 },
	{ 0xca5e, 0xca5e, PDF_CMAP_SINGLE, 15382 },
	{ 0xca5f, 0xca61, PDF_CMAP_RANGE, 2811 },
	{ 0xca62, 0xca67, PDF_CMAP_RANGE, 15383 },
	{ 0xca68, 0xca68, PDF_CMAP_SINGLE, 2814 },
	{ 0xca69, 0xca7c, PDF_CMAP_RANGE, 15389 },
	{ 0xca7d, 0xca7d, PDF_CMAP_SINGLE, 2815 },
	{ 0xca7e, 0xca83, PDF_CMAP_RANGE, 15409 },
	{ 0xca84, 0xca84, PDF_CMAP_SINGLE, 2816 },
	{ 0xca85, 0xca97, PDF_CMAP_RANGE, 15415 },
	{ 0xca98, 0xca98, PDF_CMAP_SINGLE, 2817 },
	{ 0xca99, 0xcabb, PDF_CMAP_RANGE, 15434 },
	{ 0xcabc, 0xcabd, PDF_CMAP_RANGE, 2818 },
	{ 0xcabe, 0xcabf, PDF_CMAP_RANGE, 15469 },
	{ 0xcac0, 0xcac0, PDF_CMAP_SINGLE, 2820 },
	{ 0xcac1, 0xcac3, PDF_CMAP_RANGE, 15471 },
	{ 0xcac4, 0xcac4, PDF_CMAP_SINGLE, 2821 },
	{ 0xcac5, 0xcacb, PDF_CMAP_RANGE, 15474 },
	{ 0xcacc, 0xcacd, PDF_CMAP_RANGE, 2822 },
	{ 0xcace, 0xcace, PDF_CMAP_SINGLE, 15481 },
	{ 0xcacf, 0xcacf, PDF_CMAP_SINGLE, 2824 },
	{ 0xcad0, 0xcad0, PDF_CMAP_SINGLE, 15482 },
	{ 0xcad1, 0xcad1, PDF_CMAP_SINGLE, 2825 },
	{ 0xcad2, 0xcad2, PDF_CMAP_SINGLE, 15483 },
	{ 0xcad3, 0xcad3, PDF_CMAP_SINGLE, 2826 },
	{ 0xcad4, 0xcad7, PDF_CMAP_RANGE, 15484 },
	{ 0xcad8, 0xcad9, PDF_CMAP_RANGE, 2827 },
	{ 0xcada, 0xcadf, PDF_CMAP_RANGE, 15488 },
	{ 0xcae0, 0xcae0, PDF_CMAP_SINGLE, 2829 },
	{ 0xcae1, 0xcaeb, PDF_CMAP_RANGE, 15494 },
	{ 0xcaec, 0xcaec, PDF_CMAP_SINGLE, 2830 },
	{ 0xcaed, 0xcaf3, PDF_CMAP_RANGE, 15505 },
	{ 0xcaf4, 0xcaf4, PDF_CMAP_SINGLE, 2831 },
	{ 0xcaf5, 0xcaff, PDF_CMAP_RANGE, 15512 },
	{ 0xcb00, 0xcb07, PDF_CMAP_RANGE, 15523 },
	{ 0xcb08, 0xcb08, PDF_CMAP_SINGLE, 2832 },
	{ 0xcb09, 0xcb0f, PDF_CMAP_RANGE, 15531 },
	{ 0xcb10, 0xcb10, PDF_CMAP_SINGLE, 2833 },
	{ 0xcb11, 0xcb13, PDF_CMAP_RANGE, 15538 },
	{ 0xcb14, 0xcb14, PDF_CMAP_SINGLE, 2834 },
	{ 0xcb15, 0xcb17, PDF_CMAP_RANGE, 15541 },
	{ 0xcb18, 0xcb18, PDF_CMAP_SINGLE, 2835 },
	{ 0xcb19, 0xcb1f, PDF_CMAP_RANGE, 15544 },
	{ 0xcb20, 0xcb21, PDF_CMAP_RANGE, 2836 },
	{ 0xcb22, 0xcb40, PDF_CMAP_RANGE, 15551 },
	{ 0xcb41, 0xcb41, PDF_CMAP_SINGLE, 2838 },
	{ 0xcb42, 0xcb47, PDF_CMAP_RANGE, 15582 },
	{ 0xcb48, 0xcb49, PDF_CMAP_RANGE, 2839 },
	{ 0xcb4a, 0xcb4b, PDF_CMAP_RANGE, 15588 },
	{ 0xcb4c, 0xcb4c, PDF_CMAP_SINGLE, 2841 },
	{ 0xcb4d, 0xcb4f, PDF_CMAP_RANGE, 15590 },
	{ 0xcb50, 0xcb50, PDF_CMAP_SINGLE, 2842 },
	{ 0xcb51, 0xcb57, PDF_CMAP_RANGE, 15593 },
	{ 0xcb58, 0xcb59, PDF_CMAP_RANGE, 2843 },
	{ 0xcb5a, 0xcb5c, PDF_CMAP_RANGE, 15600 },
	{ 0xcb5d, 0xcb5d, PDF_CMAP_SINGLE, 2845 },
	{ 0xcb5e, 0xcb63, PDF_CMAP_RANGE, 15603 },
	{ 0xcb64, 0xcb64, PDF_CMAP_SINGLE, 2846 },
	{ 0xcb65, 0xcb77, PDF_CMAP_RANGE, 15609 },
	{ 0xcb78, 0xcb79, PDF_CMAP_RANGE, 2847 },
	{ 0xcb7a, 0xcb9b, PDF_CMAP_RANGE, 15628 },
	{ 0xcb9c, 0xcb9c, PDF_CMAP_SINGLE, 2849 },
	{ 0xcb9d, 0xcbb7, PDF_CMAP_RANGE, 15662 },
	{ 0xcbb8, 0xcbb8, PDF_CMAP_SINGLE, 2850 },
	{ 0xcbb9, 0xcbd3, PDF_CMAP_RANGE, 15689 },
	{ 0xcbd4, 0xcbd4, PDF_CMAP_SINGLE, 2851 },
	{ 0xcbd5, 0xcbe3, PDF_CMAP_RANGE, 15716 },
	{ 0xcbe4, 0xcbe4, PDF_CMAP_SINGLE, 2852 },
	{ 0xcbe5, 0xcbe6, PDF_CMAP_RANGE, 15731 },
	{ 0xcbe7, 0xcbe7, PDF_CMAP_SINGLE, 2853 },
	{ 0xcbe8, 0xcbe8, PDF_CMAP_SINGLE, 15733 },
	{ 0xcbe9, 0xcbe9, PDF_CMAP_SINGLE, 2854 },
	{ 0xcbea, 0xcbff, PDF_CMAP_RANGE, 15734 },
	{ 0xcc00, 0xcc0b, PDF_CMAP_RANGE, 15756 },
	{ 0xcc0c, 0xcc0d, PDF_CMAP_RANGE, 2855 },
	{ 0xcc0e, 0xcc0f, PDF_CMAP_RANGE, 15768 },
	{ 0xcc10, 0xcc10, PDF_CMAP_SINGLE, 2857 },
	{ 0xcc11, 0xcc13, PDF_CMAP_RANGE, 15770 },
	{ 0xcc14, 0xcc14, PDF_CMAP_SINGLE, 2858 },
	{ 0xcc15, 0xcc1b, PDF_CMAP_RANGE, 15773 },
	{ 0xcc1c, 0xcc1d, PDF_CMAP_RANGE, 2859 },
	{ 0xcc1e, 0xcc20, PDF_CMAP_RANGE, 15780 },
	{ 0xcc21, 0xcc22, PDF_CMAP_RANGE, 2861 },
	{ 0xcc23, 0xcc26, PDF_CMAP_RANGE, 15783 },
	{ 0xcc27, 0xcc29, PDF_CMAP_RANGE, 2863 },
	{ 0xcc2a, 0xcc2b, PDF_CMAP_RANGE, 15787 },
	{ 0xcc2c, 0xcc2c, PDF_CMAP_SINGLE, 2866 },
	{ 0xcc2d, 0xcc2d, PDF_CMAP_SINGLE, 15789 },
	{ 0xcc2e, 0xcc2e, PDF_CMAP_SINGLE, 2867 },
	{ 0xcc2f, 0xcc2f, PDF_CMAP_SINGLE, 15790 },
	{ 0xcc30, 0xcc30, PDF_CMAP_SINGLE, 2868 },
	{ 0xcc31, 0xcc37, PDF_CMAP_RANGE, 15791 },
	{ 0xcc38, 0xcc39, PDF_CMAP_RANGE, 2869 },
	{ 0xcc3a, 0xcc3a, PDF_CMAP_SINGLE, 15798 },
	{ 0xcc3b, 0xcc3e, PDF_CMAP_RANGE, 2871 },
	{ 0xcc3f, 0xcc43, PDF_CMAP_RANGE, 15799 },
	{ 0xcc44, 0xcc45, PDF_CMAP_RANGE, 2875 },
	{ 0xcc46, 0xcc47, PDF_CMAP_RANGE, 15804 },
	{ 0xcc48, 0xcc48, PDF_CMAP_SINGLE, 2877 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_RANGE, 15806 },
	{ 0xcc4c, 0xcc4c, PDF_CMAP_SINGLE, 2878 },
	{ 0xcc4d, 0xcc53, PDF_CMAP_RANGE, 15809 },
	{ 0xcc54, 0xcc55, PDF_CMAP_RANGE, 2879 },
	{ 0xcc56, 0xcc56, PDF_CMAP_SINGLE, 15816 },
	{ 0xcc57, 0xcc59, PDF_CMAP_RANGE, 2881 },
	{ 0xcc5a, 0xcc5f, PDF_CMAP_RANGE, 15817 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 2884 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 15823 },
	{ 0xcc64, 0xcc64, PDF_CMAP_SINGLE, 2885 },
	{ 0xcc65, 0xcc65, PDF_CMAP_SINGLE, 15826 },
	{ 0xcc66, 0xcc66, PDF_CMAP_SINGLE, 2886 },
	{ 0xcc67, 0xcc67, PDF_CMAP_SINGLE, 15827 },
	{ 0xcc68, 0xcc68, PDF_CMAP_SINGLE, 2887 },
	{ 0xcc69, 0xcc6f, PDF_CMAP_RANGE, 15828 },
	{ 0xcc70, 0xcc70, PDF_CMAP_SINGLE, 2888 },
	{ 0xcc71, 0xcc74, PDF_CMAP_RANGE, 15835 },
	{ 0xcc75, 0xcc75, PDF_CMAP_SINGLE, 2889 },
	{ 0xcc76, 0xcc97, PDF_CMAP_RANGE, 15839 },
	{ 0xcc98, 0xcc99, PDF_CMAP_RANGE, 2890 },
	{ 0xcc9a, 0xcc9b, PDF_CMAP_RANGE, 15873 },
	{ 0xcc9c, 0xcc9c, PDF_CMAP_SINGLE, 2892 },
	{ 0xcc9d, 0xcc9f, PDF_CMAP_RANGE, 15875 },
	{ 0xcca0, 0xcca0, PDF_CMAP_SINGLE, 2893 },
	{ 0xcca1, 0xcca7, PDF_CMAP_RANGE, 15878 },
	{ 0xcca8, 0xcca9, PDF_CMAP_RANGE, 2894 },
	{ 0xccaa, 0xccaa, PDF_CMAP_SINGLE, 15885 },
	{ 0xccab, 0xccad, PDF_CMAP_RANGE, 2896 },
	{ 0xccae, 0xccb3, PDF_CMAP_RANGE, 15886 },
	{ 0xccb4, 0xccb5, PDF_CMAP_RANGE, 2899 },
	{ 0xccb6, 0xccb7, PDF_CMAP_RANGE, 15892 },
	{ 0xccb8, 0xccb8, PDF_CMAP_SINGLE, 2901 },
	{ 0xccb9, 0xccbb, PDF_CMAP_RANGE, 15894 },
	{ 0xccbc, 0xccbc, PDF_CMAP_SINGLE, 2902 },
	{ 0xccbd, 0xccc3, PDF_CMAP_RANGE, 15897 },
	{ 0xccc4, 0xccc5, PDF_CMAP_RANGE, 2903 },
	{ 0xccc6, 0xccc6, PDF_CMAP_SINGLE, 15904 },
	{ 0xccc7, 0xccc7, PDF_CMAP_SINGLE, 2905 },
	{ 0xccc8, 0xccc8, PDF_CMAP_SINGLE, 15905 },
	{ 0xccc9, 0xccc9, PDF_CMAP_SINGLE, 2906 },
	{ 0xccca, 0xcccf, PDF_CMAP_RANGE, 15906 },
	{ 0xccd0, 0xccd0, PDF_CMAP_SINGLE, 2907 },
	{ 0xccd1, 0xccd3, PDF_CMAP_RANGE, 15912 },
	{ 0xccd4, 0xccd4, PDF_CMAP_SINGLE, 2908 },
	{ 0xccd5, 0xcce3, PDF_CMAP_RANGE, 15915 },
	{ 0xcce4, 0xcce4, PDF_CMAP_SINGLE, 2909 },
	{ 0xcce5, 0xcceb, PDF_CMAP_RANGE, 15930 },
	{ 0xccec, 0xccec, PDF_CMAP_SINGLE, 2910 },
	{ 0xcced, 0xccef, PDF_CMAP_RANGE, 15937 },
	{ 0xccf0, 0xccf0, PDF_CMAP_SINGLE, 2911 },
	{ 0xccf1, 0xccff, PDF_CMAP_RANGE, 15940 },
	{ 0xcd00, 0xcd00, PDF_CMAP_SINGLE, 15955 },
	{ 0xcd01, 0xcd01, PDF_CMAP_SINGLE, 2912 },
	{ 0xcd02, 0xcd07, PDF_CMAP_RANGE, 15956 },
	{ 0xcd08, 0xcd09, PDF_CMAP_RANGE, 2913 },
	{ 0xcd0a, 0xcd0b, PDF_CMAP_RANGE, 15962 },
	{ 0xcd0c, 0xcd0c, PDF_CMAP_SINGLE, 2915 },
	{ 0xcd0d, 0xcd0f, PDF_CMAP_RANGE, 15964 },
	{ 0xcd10, 0xcd10, PDF_CMAP_SINGLE, 2916 },
	{ 0xcd11, 0xcd17, PDF_CMAP_RANGE, 15967 },
	{ 0xcd18, 0xcd19, PDF_CMAP_RANGE, 2917 },
	{ 0xcd1a, 0xcd1a, PDF_CMAP_SINGLE, 15974 },
	{ 0xcd1b, 0xcd1b, PDF_CMAP_SINGLE, 2919 },
	{ 0xcd1c, 0xcd1c, PDF_CMAP_SINGLE, 15975 },
	{ 0xcd1d, 0xcd1d, PDF_CMAP_SINGLE, 2920 },
	{ 0xcd1e, 0xcd23, PDF_CMAP_RANGE, 15976 },
	{ 0xcd24, 0xcd24, PDF_CMAP_SINGLE, 2921 },
	{ 0xcd25, 0xcd27, PDF_CMAP_RANGE, 15982 },
	{ 0xcd28, 0xcd28, PDF_CMAP_SINGLE, 2922 },
	{ 0xcd29, 0xcd2b, PDF_CMAP_RANGE, 15985 },
	{ 0xcd2c, 0xcd2c, PDF_CMAP_SINGLE, 2923 },
	{ 0xcd2d, 0xcd38, PDF_CMAP_RANGE, 15988 },
	{ 0xcd39, 0xcd39, PDF_CMAP_SINGLE, 2924 },
	{ 0xcd3a, 0xcd5b, PDF_CMAP_RANGE, 16000 },
	{ 0xcd5c, 0xcd5c, PDF_CMAP_SINGLE, 2925 },
	{ 0xcd5d, 0xcd5f, PDF_CMAP_RANGE, 16034 },
	{ 0xcd60, 0xcd60, PDF_CMAP_SINGLE, 2926 },
	{ 0xcd61, 0xcd63, PDF_CMAP_RANGE, 16037 },
	{ 0xcd64, 0xcd64, PDF_CMAP_SINGLE, 2927 },
	{ 0xcd65, 0xcd6b, PDF_CMAP_RANGE, 16040 },
	{ 0xcd6c, 0xcd6d, PDF_CMAP_RANGE, 2928 },
	{ 0xcd6e, 0xcd6e, PDF_CMAP_SINGLE, 16047 },
	{ 0xcd6f, 0xcd6f, PDF_CMAP_SINGLE, 2930 },
	{ 0xcd70, 0xcd70, PDF_CMAP_SINGLE, 16048 },
	{ 0xcd71, 0xcd71, PDF_CMAP_SINGLE, 2931 },
	{ 0xcd72, 0xcd77, PDF_CMAP_RANGE, 16049 },
	{ 0xcd78, 0xcd78, PDF_CMAP_SINGLE, 2932 },
	{ 0xcd79, 0xcd87, PDF_CMAP_RANGE, 16055 },
	{ 0xcd88, 0xcd88, PDF_CMAP_SINGLE, 2933 },
	{ 0xcd89, 0xcd93, PDF_CMAP_RANGE, 16070 },
	{ 0xcd94, 0xcd95, PDF_CMAP_RANGE, 2934 },
	{ 0xcd96, 0xcd97, PDF_CMAP_RANGE, 16081 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 2936 },
	{ 0xcd99, 0xcd9b, PDF_CMAP_RANGE, 16083 },
	{ 0xcd9c, 0xcd9c, PDF_CMAP_SINGLE, 2937 },
	{ 0xcd9d, 0xcda3, PDF_CMAP_RANGE, 16086 },
	{ 0xcda4, 0xcda5, PDF_CMAP_RANGE, 2938 },
	{ 0xcda6, 0xcda6, PDF_CMAP_SINGLE, 16093 },
	{ 0xcda7, 0xcda7, PDF_CMAP_SINGLE, 2940 },
	{ 0xcda8, 0xcda8, PDF_CMAP_SINGLE, 16094 },
	{ 0xcda9, 0xcda9, PDF_CMAP_SINGLE, 2941 },
	{ 0xcdaa, 0xcdaf, PDF_CMAP_RANGE, 16095 },
	{ 0xcdb0, 0xcdb0, PDF_CMAP_SINGLE, 2942 },
	{ 0xcdb1, 0xcdc3, PDF_CMAP_RANGE, 16101 },
	{ 0xcdc4, 0xcdc4, PDF_CMAP_SINGLE, 2943 },
	{ 0xcdc5, 0xcdcb, PDF_CMAP_RANGE, 16120 },
	{ 0xcdcc, 0xcdcc, PDF_CMAP_SINGLE, 2944 },
	{ 0xcdcd, 0xcdcf, PDF_CMAP_RANGE, 16127 },
	{ 0xcdd0, 0xcdd0, PDF_CMAP_SINGLE, 2945 },
	{ 0xcdd1, 0xcde7, PDF_CMAP_RANGE, 16130 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 2946 },
	{ 0xcde9, 0xcdeb, PDF_CMAP_RANGE, 16153 },
	{ 0xcdec, 0xcdec, PDF_CMAP_SINGLE, 2947 },
	{ 0xcded, 0xcdef, PDF_CMAP_RANGE, 16156 },
	{ 0xcdf0, 0xcdf0, PDF_CMAP_SINGLE, 2948 },
	{ 0xcdf1, 0xcdf7, PDF_CMAP_RANGE, 16159 },
	{ 0xcdf8, 0xcdf9, PDF_CMAP_RANGE, 2949 },
	{ 0xcdfa, 0xcdfa, PDF_CMAP_SINGLE, 16166 },
	{ 0xcdfb, 0xcdfb, PDF_CMAP_SINGLE, 2951 },
	{ 0xcdfc, 0xcdfc, PDF_CMAP_SINGLE, 16167 },
	{ 0xcdfd, 0xcdfd, PDF_CMAP_SINGLE, 2952 },
	{ 0xcdfe, 0xcdff, PDF_CMAP_RANGE, 16168 },
	{ 0xce00, 0xce03, PDF_CMAP_RANGE, 16170 },
	{ 0xce04, 0xce04, PDF_CMAP_SINGLE, 2953 },
	{ 0xce05, 0xce07, PDF_CMAP_RANGE, 16174 },
	{ 0xce08, 0xce08, PDF_CMAP_SINGLE, 2954 },
	{ 0xce09, 0xce0b, PDF_CMAP_RANGE, 16177 },
	{ 0xce0c, 0xce0c, PDF_CMAP_SINGLE, 2955 },
	{ 0xce0d, 0xce13, PDF_CMAP_RANGE, 16180 },
	{ 0xce14, 0xce14, PDF_CMAP_SINGLE, 2956 },
	{ 0xce15, 0xce18, PDF_CMAP_RANGE, 16187 },
	{ 0xce19, 0xce19, PDF_CMAP_SINGLE, 2957 },
	{ 0xce1a, 0xce1f, PDF_CMAP_RANGE, 16191 },
	{ 0xce20, 0xce21, PDF_CMAP_RANGE, 2958 },
	{ 0xce22, 0xce23, PDF_CMAP_RANGE, 16197 },
	{ 0xce24, 0xce24, PDF_CMAP_SINGLE, 2960 },
	{ 0xce25, 0xce27, PDF_CMAP_RANGE, 16199 },
	{ 0xce28, 0xce28, PDF_CMAP_SINGLE, 2961 },
	{ 0xce29, 0xce2f, PDF_CMAP_RANGE, 16202 },
	{ 0xce30, 0xce31, PDF_CMAP_RANGE, 2962 },
	{ 0xce32, 0xce32, PDF_CMAP_SINGLE, 16209 },
	{ 0xce33, 0xce33, PDF_CMAP_SINGLE, 2964 },
	{ 0xce34, 0xce34, PDF_CMAP_SINGLE, 16210 },
	{ 0xce35, 0xce35, PDF_CMAP_SINGLE, 2965 },
	{ 0xce36, 0xce57, PDF_CMAP_RANGE, 16211 },
	{ 0xce58, 0xce59, PDF_CMAP_RANGE, 2966 },
	{ 0xce5a, 0xce5b, PDF_CMAP_RANGE, 16245 },
	{ 0xce5c, 0xce5c, PDF_CMAP_SINGLE, 2968 },
	{ 0xce5d, 0xce5e, PDF_CMAP_RANGE, 16247 },
	{ 0xce5f, 0xce61, PDF_CMAP_RANGE, 2969 },
	{ 0xce62, 0xce67, PDF_CMAP_RANGE, 16249 },
	{ 0xce68, 0xce69, PDF_CMAP_RANGE, 2972 },
	{ 0xce6a, 0xce6a, PDF_CMAP_SINGLE, 16255 },
	{ 0xce6b, 0xce6b, PDF_CMAP_SINGLE, 2974 },
	{ 0xce6c, 0xce6c, PDF_CMAP_SINGLE, 16256 },
	{ 0xce6d, 0xce6d, PDF_CMAP_SINGLE, 2975 },
	{ 0xce6e, 0xce73, PDF_CMAP_RANGE, 16257 },
	{ 0xce74, 0xce75, PDF_CMAP_RANGE, 2976 },
	{ 0xce76, 0xce77, PDF_CMAP_RANGE, 16263 },
	{ 0xce78, 0xce78, PDF_CMAP_SINGLE, 2978 },
	{ 0xce79, 0xce7b, PDF_CMAP_RANGE, 16265 },
	{ 0xce7c, 0xce7c, PDF_CMAP_SINGLE, 2979 },
	{ 0xce7d, 0xce83, PDF_CMAP_RANGE, 16268 },
	{ 0xce84, 0xce85, PDF_CMAP_RANGE, 2980 },
	{ 0xce86, 0xce86, PDF_CMAP_SINGLE, 16275 },
	{ 0xce87, 0xce87, PDF_CMAP_SINGLE, 2982 },
	{ 0xce88, 0xce88, PDF_CMAP_SINGLE, 16276 },
	{ 0xce89, 0xce89, PDF_CMAP_SINGLE, 2983 },
	{ 0xce8a, 0xce8f, PDF_CMAP_RANGE, 16277 },
	{ 0xce90, 0xce91, PDF_CMAP_RANGE, 2984 },
	{ 0xce92, 0xce93, PDF_CMAP_RANGE, 16283 },
	{ 0xce94, 0xce94, PDF_CMAP_SINGLE, 2986 },
	{ 0xce95, 0xce97, PDF_CMAP_RANGE, 16285 },
	{ 0xce98, 0xce98, PDF_CMAP_SINGLE, 2987 },
	{ 0xce99, 0xce9f, PDF_CMAP_RANGE, 16288 },
	{ 0xcea0, 0xcea1, PDF_CMAP_RANGE, 2988 },
	{ 0xcea2, 0xcea2, PDF_CMAP_SINGLE, 16295 },
	{ 0xcea3, 0xcea5, PDF_CMAP_RANGE, 2990 },
	{ 0xcea6, 0xceab, PDF_CMAP_RANGE, 16296 },
	{ 0xceac, 0xcead, PDF_CMAP_RANGE, 2993 },
	{ 0xceae, 0xcec0, PDF_CMAP_RANGE, 16302 },
	{ 0xcec1, 0xcec1, PDF_CMAP_SINGLE, 2995 },
	{ 0xcec2, 0xcee3, PDF_CMAP_RANGE, 16321 },
	{ 0xcee4, 0xcee5, PDF_CMAP_RANGE, 2996 },
	{ 0xcee6, 0xcee7, PDF_CMAP_RANGE, 16355 },
	{ 0xcee8, 0xcee8, PDF_CMAP_SINGLE, 2998 },
	{ 0xcee9, 0xceea, PDF_CMAP_RANGE, 16357 },
	{ 0xceeb, 0xceec, PDF_CMAP_RANGE, 2999 },
	{ 0xceed, 0xcef3, PDF_CMAP_RANGE, 16359 },
	{ 0xcef4, 0xcef5, PDF_CMAP_RANGE, 3001 },
	{ 0xcef6, 0xcef6, PDF_CMAP_SINGLE, 16366 },
	{ 0xcef7, 0xcef9, PDF_CMAP_RANGE, 3003 },
	{ 0xcefa, 0xceff, PDF_CMAP_RANGE, 16367 },
	{ 0xcf00, 0xcf01, PDF_CMAP_RANGE, 3006 },
	{ 0xcf02, 0xcf03, PDF_CMAP_RANGE, 16373 },
	{ 0xcf04, 0xcf04, PDF_CMAP_SINGLE, 3008 },
	{ 0xcf05, 0xcf07, PDF_CMAP_RANGE, 16375 },
	{ 0xcf08, 0xcf08, PDF_CMAP_SINGLE, 3009 },
	{ 0xcf09, 0xcf0f, PDF_CMAP_RANGE, 16378 },
	{ 0xcf10, 0xcf11, PDF_CMAP_RANGE, 3010 },
	{ 0xcf12, 0xcf12, PDF_CMAP_SINGLE, 16385 },
	{ 0xcf13, 0xcf13, PDF_CMAP_SINGLE, 3012 },
	{ 0xcf14, 0xcf14, PDF_CMAP_SINGLE, 16386 },
	{ 0xcf15, 0xcf15, PDF_CMAP_SINGLE, 3013 },
	{ 0xcf16, 0xcf1b, PDF_CMAP_RANGE, 16387 },
	{ 0xcf1c, 0xcf1c, PDF_CMAP_SINGLE, 3014 },
	{ 0xcf1d, 0xcf1f, PDF_CMAP_RANGE, 16393 },
	{ 0xcf20, 0xcf20, PDF_CMAP_SINGLE, 3015 },
	{ 0xcf21, 0xcf23, PDF_CMAP_RANGE, 16396 },
	{ 0xcf24, 0xcf24, PDF_CMAP_SINGLE, 3016 },
	{ 0xcf25, 0xcf2b, PDF_CMAP_RANGE, 16399 },
	{ 0xcf2c, 0xcf2d, PDF_CMAP_RANGE, 3017 },
	{ 0xcf2e, 0xcf2e, PDF_CMAP_SINGLE, 16406 },
	{ 0xcf2f, 0xcf31, PDF_CMAP_RANGE, 3019 },
	{ 0xcf32, 0xcf37, PDF_CMAP_RANGE, 16407 },
	{ 0xcf38, 0xcf38, PDF_CMAP_SINGLE, 3022 },
	{ 0xcf39, 0xcf53, PDF_CMAP_RANGE, 16413 },
	{ 0xcf54, 0xcf55, PDF_CMAP_RANGE, 3023 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 16440 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 3025 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 16442 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 3026 },
	{ 0xcf5d, 0xcf63, PDF_CMAP_RANGE, 16445 },
	{ 0xcf64, 0xcf65, PDF_CMAP_RANGE, 3027 },
	{ 0xcf66, 0xcf66, PDF_CMAP_SINGLE, 16452 },
	{ 0xcf67, 0xcf67, PDF_CMAP_SINGLE, 3029 },
	{ 0xcf68, 0xcf68, PDF_CMAP_SINGLE, 16453 },
	{ 0xcf69, 0xcf69, PDF_CMAP_SINGLE, 3030 },
	{ 0xcf6a, 0xcf6f, PDF_CMAP_RANGE, 16454 },
	{ 0xcf70, 0xcf71, PDF_CMAP_RANGE, 3031 },
	{ 0xcf72, 0xcf73, PDF_CMAP_RANGE, 16460 },
	{ 0xcf74, 0xcf74, PDF_CMAP_SINGLE, 3033 },
	{ 0xcf75, 0xcf77, PDF_CMAP_RANGE, 16462 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 3034 },
	{ 0xcf79, 0xcf7f, PDF_CMAP_RANGE, 16465 },
	{ 0xcf80, 0xcf80, PDF_CMAP_SINGLE, 3035 },
	{ 0xcf81, 0xcf84, PDF_CMAP_RANGE, 16472 },
	{ 0xcf85, 0xcf85, PDF_CMAP_SINGLE, 3036 },
	{ 0xcf86, 0xcf8b, PDF_CMAP_RANGE, 16476 },
	{ 0xcf8c, 0xcf8c, PDF_CMAP_SINGLE, 3037 },
	{ 0xcf8d, 0xcfa0, PDF_CMAP_RANGE, 16482 },
	{ 0xcfa1, 0xcfa1, PDF_CMAP_SINGLE, 3038 },
	{ 0xcfa2, 0xcfa7, PDF_CMAP_RANGE, 16502 },
	{ 0xcfa8, 0xcfa8, PDF_CMAP_SINGLE, 3039 },
	{ 0xcfa9, 0xcfaf, PDF_CMAP_RANGE, 16508 },
	{ 0xcfb0, 0xcfb0, PDF_CMAP_SINGLE, 3040 },
	{ 0xcfb1, 0xcfc3, PDF_CMAP_RANGE, 16515 },
	{ 0xcfc4, 0xcfc4, PDF_CMAP_SINGLE, 3041 },
	{ 0xcfc5, 0xcfdf, PDF_CMAP_RANGE, 16534 },
	{ 0xcfe0, 0xcfe1, PDF_CMAP_RANGE, 3042 },
	{ 0xcfe2, 0xcfe3, PDF_CMAP_RANGE, 16561 },
	{ 0xcfe4, 0xcfe4, PDF_CMAP_SINGLE, 3044 },
	{ 0xcfe5, 0xcfe7, PDF_CMAP_RANGE, 16563 },
	{ 0xcfe8, 0xcfe8, PDF_CMAP_SINGLE, 3045 },
	{ 0xcfe9, 0xcfef, PDF_CMAP_RANGE, 16566 },
	{ 0xcff0, 0xcff1, PDF_CMAP_RANGE, 3046 },
	{ 0xcff2, 0xcff2, PDF_CMAP_SINGLE, 16573 },
	{ 0xcff3, 0xcff3, PDF_CMAP_SINGLE, 3048 },
	{ 0xcff4, 0xcff4, PDF_CMAP_SINGLE, 16574 },
	{ 0xcff5, 0xcff5, PDF_CMAP_SINGLE, 3049 },
	{ 0xcff6, 0xcffb, PDF_CMAP_RANGE, 16575 },
	{ 0xcffc, 0xcffc, PDF_CMAP_SINGLE, 3050 },
	{ 0xcffd, 0xcfff, PDF_CMAP_RANGE, 16581 },
	{ 0xd000, 0xd000, PDF_CMAP_SINGLE, 3051 },
	{ 0xd001, 0xd003, PDF_CMAP_RANGE, 16584 },
	{ 0xd004, 0xd004, PDF_CMAP_SINGLE, 3052 },
	{ 0xd005, 0xd010, PDF_CMAP_RANGE, 16587 },
	{ 0xd011, 0xd011, PDF_CMAP_SINGLE, 3053 },
	{ 0xd012, 0xd017, PDF_CMAP_RANGE, 16599 },
	{ 0xd018, 0xd018, PDF_CMAP_SINGLE, 3054 },
	{ 0xd019, 0xd02c, PDF_CMAP_RANGE, 16605 },
	{ 0xd02d, 0xd02d, PDF_CMAP_SINGLE, 3055 },
	{ 0xd02e, 0xd033, PDF_CMAP_RANGE, 16625 },
	{ 0xd034, 0xd035, PDF_CMAP_RANGE, 3056 },
	{ 0xd036, 0xd037, PDF_CMAP_RANGE, 16631 },
	{ 0xd038, 0xd038, PDF_CMAP_SINGLE, 3058 },
	{ 0xd039, 0xd03b, PDF_CMAP_RANGE, 16633 },
	{ 0xd03c, 0xd03c, PDF_CMAP_SINGLE, 3059 },
	{ 0xd03d, 0xd043, PDF_CMAP_RANGE, 16636 },
	{ 0xd044, 0xd045, PDF_CMAP_RANGE, 3060 },
	{ 0xd046, 0xd046, PDF_CMAP_SINGLE, 16643 },
	{ 0xd047, 0xd047, PDF_CMAP_SINGLE, 3062 },
	{ 0xd048, 0xd048, PDF_CMAP_SINGLE, 16644 },
	{ 0xd049, 0xd049, PDF_CMAP_SINGLE, 3063 },
	{ 0xd04a, 0xd04f, PDF_CMAP_RANGE, 16645 },
	{ 0xd050, 0xd050, PDF_CMAP_SINGLE, 3064 },
	{ 0xd051, 0xd053, PDF_CMAP_RANGE, 16651 },
	{ 0xd054, 0xd054, PDF_CMAP_SINGLE, 3065 },
	{ 0xd055, 0xd057, PDF_CMAP_RANGE, 16654 },
	{ 0xd058, 0xd058, PDF_CMAP_SINGLE, 3066 },
	{ 0xd059, 0xd05f, PDF_CMAP_RANGE, 16657 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 3067 },
	{ 0xd061, 0xd06b, PDF_CMAP_RANGE, 16664 },
	{ 0xd06c, 0xd06d, PDF_CMAP_RANGE, 3068 },
	{ 0xd06e, 0xd06f, PDF_CMAP_RANGE, 16675 },
	{ 0xd070, 0xd070, PDF_CMAP_SINGLE, 3070 },
	{ 0xd071, 0xd073, PDF_CMAP_RANGE, 16677 },
	{ 0xd074, 0xd074, PDF_CMAP_SINGLE, 3071 },
	{ 0xd075, 0xd07b, PDF_CMAP_RANGE, 16680 },
	{ 0xd07c, 0xd07d, PDF_CMAP_RANGE, 3072 },
	{ 0xd07e, 0xd080, PDF_CMAP_RANGE, 16687 },
	{ 0xd081, 0xd081, PDF_CMAP_SINGLE, 3074 },
	{ 0xd082, 0xd0a3, PDF_CMAP_RANGE, 16690 },
	{ 0xd0a4, 0xd0a5, PDF_CMAP_RANGE, 3075 },
	{ 0xd0a6, 0xd0a7, PDF_CMAP_RANGE, 16724 },
	{ 0xd0a8, 0xd0a8, PDF_CMAP_SINGLE, 3077 },
	{ 0xd0a9, 0xd0ab, PDF_CMAP_RANGE, 16726 },
	{ 0xd0ac, 0xd0ac, PDF_CMAP_SINGLE, 3078 },
	{ 0xd0ad, 0xd0b3, PDF_CMAP_RANGE, 16729 },
	{ 0xd0b4, 0xd0b5, PDF_CMAP_RANGE, 3079 },
	{ 0xd0b6, 0xd0b6, PDF_CMAP_SINGLE, 16736 },
	{ 0xd0b7, 0xd0b7, PDF_CMAP_SINGLE, 3081 },
	{ 0xd0b8, 0xd0b8, PDF_CMAP_SINGLE, 16737 },
	{ 0xd0b9, 0xd0b9, PDF_CMAP_SINGLE, 3082 },
	{ 0xd0ba, 0xd0bf, PDF_CMAP_RANGE, 16738 },
	{ 0xd0c0, 0xd0c1, PDF_CMAP_RANGE, 3083 },
	{ 0xd0c2, 0xd0c3, PDF_CMAP_RANGE, 16744 },
	{ 0xd0c4, 0xd0c4, PDF_CMAP_SINGLE, 3085 },
	{ 0xd0c5, 0xd0c7, PDF_CMAP_RANGE, 16746 },
	{ 0xd0c8, 0xd0c9, PDF_CMAP_RANGE, 3086 },
	{ 0xd0ca, 0xd0cf, PDF_CMAP_RANGE, 16749 },
	{ 0xd0d0, 0xd0d1, PDF_CMAP_RANGE, 3088 },
	{ 0xd0d2, 0xd0d2, PDF_CMAP_SINGLE, 16755 },
	{ 0xd0d3, 0xd0d5, PDF_CMAP_RANGE, 3090 },
	{ 0xd0d6, 0xd0db, PDF_CMAP_RANGE, 16756 },
	{ 0xd0dc, 0xd0dd, PDF_CMAP_RANGE, 3093 },
	{ 0xd0de, 0xd0df, PDF_CMAP_RANGE, 16762 },
	{ 0xd0e0, 0xd0e0, PDF_CMAP_SINGLE, 3095 },
	{ 0xd0e1, 0xd0e3, PDF_CMAP_RANGE, 16764 },
	{ 0xd0e4, 0xd0e4, PDF_CMAP_SINGLE, 3096 },
	{ 0xd0e5, 0xd0eb, PDF_CMAP_RANGE, 16767 },
	{ 0xd0ec, 0xd0ed, PDF_CMAP_RANGE, 3097 },
	{ 0xd0ee, 0xd0ee, PDF_CMAP_SINGLE, 16774 },
	{ 0xd0ef, 0xd0f1, PDF_CMAP_RANGE, 3099 },
	{ 0xd0f2, 0xd0f7, PDF_CMAP_RANGE, 16775 },
	{ 0xd0f8, 0xd0f8, PDF_CMAP_SINGLE, 3102 },
	{ 0xd0f9, 0xd0ff, PDF_CMAP_RANGE, 16781 },
	{ 0xd100, 0xd10c, PDF_CMAP_RANGE, 16788 },
	{ 0xd10d, 0xd10d, PDF_CMAP_SINGLE, 3103 },
	{ 0xd10e, 0xd12f, PDF_CMAP_RANGE, 16801 },
	{ 0xd130, 0xd131, PDF_CMAP_RANGE, 3104 },
	{ 0xd132, 0xd133, PDF_CMAP_RANGE, 16835 },
	{ 0xd134, 0xd134, PDF_CMAP_SINGLE, 3106 },
	{ 0xd135, 0xd137, PDF_CMAP_RANGE, 16837 },
	{ 0xd138, 0xd138, PDF_CMAP_SINGLE, 3107 },
	{ 0xd139, 0xd139, PDF_CMAP_SINGLE, 16840 },
	{ 0xd13a, 0xd13a, PDF_CMAP_SINGLE, 3108 },
	{ 0xd13b, 0xd13f, PDF_CMAP_RANGE, 16841 },
	{ 0xd140, 0xd141, PDF_CMAP_RANGE, 3109 },
	{ 0xd142, 0xd142, PDF_CMAP_SINGLE, 16846 },
	{ 0xd143, 0xd145, PDF_CMAP_RANGE, 3111 },
	{ 0xd146, 0xd14b, PDF_CMAP_RANGE, 16847 },
	{ 0xd14c, 0xd14d, PDF_CMAP_RANGE, 3114 },
	{ 0xd14e, 0xd14f, PDF_CMAP_RANGE, 16853 },
	{ 0xd150, 0xd150, PDF_CMAP_SINGLE, 3116 },
	{ 0xd151, 0xd153, PDF_CMAP_RANGE, 16855 },
	{ 0xd154, 0xd154, PDF_CMAP_SINGLE, 3117 },
	{ 0xd155, 0xd15b, PDF_CMAP_RANGE, 16858 },
	{ 0xd15c, 0xd15d, PDF_CMAP_RANGE, 3118 },
	{ 0xd15e, 0xd15e, PDF_CMAP_SINGLE, 16865 },
	{ 0xd15f, 0xd15f, PDF_CMAP_SINGLE, 3120 },
	{ 0xd160, 0xd160, PDF_CMAP_SINGLE, 16866 },
	{ 0xd161, 0xd161, PDF_CMAP_SINGLE, 3121 },
	{ 0xd162, 0xd167, PDF_CMAP_RANGE, 16867 },
	{ 0xd168, 0xd168, PDF_CMAP_SINGLE, 3122 },
	{ 0xd169, 0xd16b, PDF_CMAP_RANGE, 16873 },
	{ 0xd16c, 0xd16c, PDF_CMAP_SINGLE, 3123 },
	{ 0xd16d, 0xd17b, PDF_CMAP_RANGE, 16876 },
	{ 0xd17c, 0xd17c, PDF_CMAP_SINGLE, 3124 },
	{ 0xd17d, 0xd183, PDF_CMAP_RANGE, 16891 },
	{ 0xd184, 0xd184, PDF_CMAP_SINGLE, 3125 },
	{ 0xd185, 0xd187, PDF_CMAP_RANGE, 16898 },
	{ 0xd188, 0xd188, PDF_CMAP_SINGLE, 3126 },
	{ 0xd189, 0xd19f, PDF_CMAP_RANGE, 16901 },
	{ 0xd1a0, 0xd1a1, PDF_CMAP_RANGE, 3127 },
	{ 0xd1a2, 0xd1a3, PDF_CMAP_RANGE, 16924 },
	{ 0xd1a4, 0xd1a4, PDF_CMAP_SINGLE, 3129 },
	{ 0xd1a5, 0xd1a7, PDF_CMAP_RANGE, 16926 },
	{ 0xd1a8, 0xd1a8, PDF_CMAP_SINGLE, 3130 },
	{ 0xd1a9, 0xd1af, PDF_CMAP_RANGE, 16929 },
	{ 0xd1b0, 0xd1b1, PDF_CMAP_RANGE, 3131 },
	{ 0xd1b2, 0xd1b2, PDF_CMAP_SINGLE, 16936 },
	{ 0xd1b3, 0xd1b3, PDF_CMAP_SINGLE, 3133 },
	{ 0xd1b4, 0xd1b4, PDF_CMAP_SINGLE, 16937 },
	{ 0xd1b5, 0xd1b5, PDF_CMAP_SINGLE, 3134 },
	{ 0xd1b6, 0xd1b9, PDF_CMAP_RANGE, 16938 },
	{ 0xd1ba, 0xd1ba, PDF_CMAP_SINGLE, 3135 },
	{ 0xd1bb, 0xd1bb, PDF_CMAP_SINGLE, 16942 },
	{ 0xd1bc, 0xd1bc, PDF_CMAP_SINGLE, 3136 },
	{ 0xd1bd, 0xd1bf, PDF_CMAP_RANGE, 16943 },
	{ 0xd1c0, 0xd1c0, PDF_CMAP_SINGLE, 3137 },
	{ 0xd1c1, 0xd1d7, PDF_CMAP_RANGE, 16946 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 3138 },
	{ 0xd1d9, 0xd1f3, PDF_CMAP_RANGE, 16969 },
	{ 0xd1f4, 0xd1f4, PDF_CMAP_SINGLE, 3139 },
	{ 0xd1f5, 0xd1f7, PDF_CMAP_RANGE, 16996 },
	{ 0xd1f8, 0xd1f8, PDF_CMAP_SINGLE, 3140 },
	{ 0xd1f9, 0xd1ff, PDF_CMAP_RANGE, 16999 },
	{ 0xd200, 0xd206, PDF_CMAP_RANGE, 17006 },
	{ 0xd207, 0xd207, PDF_CMAP_SINGLE, 3141 },
	{ 0xd208, 0xd208, PDF_CMAP_SINGLE, 17013 },
	{ 0xd209, 0xd209, PDF_CMAP_SINGLE, 3142 },
	{ 0xd20a, 0xd20f, PDF_CMAP_RANGE, 17014 },
	{ 0xd210, 0xd210, PDF_CMAP_SINGLE, 3143 },
	{ 0xd211, 0xd22b, PDF_CMAP_RANGE, 17020 },
	{ 0xd22c, 0xd22d, PDF_CMAP_RANGE, 3144 },
	{ 0xd22e, 0xd22f, PDF_CMAP_RANGE, 17047 },
	{ 0xd230, 0xd230, PDF_CMAP_SINGLE, 3146 },
	{ 0xd231, 0xd233, PDF_CMAP_RANGE, 17049 },
	{ 0xd234, 0xd234, PDF_CMAP_SINGLE, 3147 },
	{ 0xd235, 0xd23b, PDF_CMAP_RANGE, 17052 },
	{ 0xd23c, 0xd23d, PDF_CMAP_RANGE, 3148 },
	{ 0xd23e, 0xd23e, PDF_CMAP_SINGLE, 17059 },
	{ 0xd23f, 0xd23f, PDF_CMAP_SINGLE, 3150 },
	{ 0xd240, 0xd240, PDF_CMAP_SINGLE, 17060 },
	{ 0xd241, 0xd241, PDF_CMAP_SINGLE, 3151 },
	{ 0xd242, 0xd247, PDF_CMAP_RANGE, 17061 },
	{ 0xd248, 0xd248, PDF_CMAP_SINGLE, 3152 },
	{ 0xd249, 0xd25b, PDF_CMAP_RANGE, 17067 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 3153 },
	{ 0xd25d, 0xd263, PDF_CMAP_RANGE, 17086 },
	{ 0xd264, 0xd264, PDF_CMAP_SINGLE, 3154 },
	{ 0xd265, 0xd27f, PDF_CMAP_RANGE, 17093 },
	{ 0xd280, 0xd281, PDF_CMAP_RANGE, 3155 },
	{ 0xd282, 0xd283, PDF_CMAP_RANGE, 17120 },
	{ 0xd284, 0xd284, PDF_CMAP_SINGLE, 3157 },
	{ 0xd285, 0xd287, PDF_CMAP_RANGE, 17122 },
	{ 0xd288, 0xd288, PDF_CMAP_SINGLE, 3158 },
	{ 0xd289, 0xd28f, PDF_CMAP_RANGE, 17125 },
	{ 0xd290, 0xd291, PDF_CMAP_RANGE, 3159 },
	{ 0xd292, 0xd294, PDF_CMAP_RANGE, 17132 },
	{ 0xd295, 0xd295, PDF_CMAP_SINGLE, 3161 },
	{ 0xd296, 0xd29b, PDF_CMAP_RANGE, 17135 },
	{ 0xd29c, 0xd29c, PDF_CMAP_SINGLE, 3162 },
	{ 0xd29d, 0xd29f, PDF_CMAP_RANGE, 17141 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 3163 },
	{ 0xd2a1, 0xd2a3, PDF_CMAP_RANGE, 17144 },
	{ 0xd2a4, 0xd2a4, PDF_CMAP_SINGLE, 3164 },
	{ 0xd2a5, 0xd2ab, PDF_CMAP_RANGE, 17147 },
	{ 0xd2ac, 0xd2ac, PDF_CMAP_SINGLE, 3165 },
	{ 0xd2ad, 0xd2b0, PDF_CMAP_RANGE, 17154 },
	{ 0xd2b1, 0xd2b1, PDF_CMAP_SINGLE, 3166 },
	{ 0xd2b2, 0xd2b7, PDF_CMAP_RANGE, 17158 },
	{ 0xd2b8, 0xd2b9, PDF_CMAP_RANGE, 3167 },
	{ 0xd2ba, 0xd2bb, PDF_CMAP_RANGE, 17164 },
	{ 0xd2bc, 0xd2bc, PDF_CMAP_SINGLE, 3169 },
	{ 0xd2bd, 0xd2be, PDF_CMAP_RANGE, 17166 },
	{ 0xd2bf, 0xd2c0, PDF_CMAP_RANGE, 3170 },
	{ 0xd2c1, 0xd2c1, PDF_CMAP_SINGLE, 17168 },
	{ 0xd2c2, 0xd2c2, PDF_CMAP_SINGLE, 3172 },
	{ 0xd2c3, 0xd2c7, PDF_CMAP_RANGE, 17169 },
	{ 0xd2c8, 0xd2c9, PDF_CMAP_RANGE, 3173 },
	{ 0xd2ca, 0xd2ca, PDF_CMAP_SINGLE, 17174 },
	{ 0xd2cb, 0xd2cb, PDF_CMAP_SINGLE, 3175 },
	{ 0xd2cc, 0xd2d3, PDF_CMAP_RANGE, 17175 },
	{ 0xd2d4, 0xd2d4, PDF_CMAP_SINGLE, 3176 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 17183 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 3177 },
	{ 0xd2d9, 0xd2db, PDF_CMAP_RANGE, 17186 },
	{ 0xd2dc, 0xd2dc, PDF_CMAP_SINGLE, 3178 },
	{ 0xd2dd, 0xd2e3, PDF_CMAP_RANGE, 17189 },
	{ 0xd2e4, 0xd2e5, PDF_CMAP_RANGE, 3179 },
	{ 0xd2e6, 0xd2ef, PDF_CMAP_RANGE, 17196 },
	{ 0xd2f0, 0xd2f1, PDF_CMAP_RANGE, 3181 },
	{ 0xd2f2, 0xd2f3, PDF_CMAP_RANGE, 17206 },
	{ 0xd2f4, 0xd2f4, PDF_CMAP_SINGLE, 3183 },
	{ 0xd2f5, 0xd2f7, PDF_CMAP_RANGE, 17208 },
	{ 0xd2f8, 0xd2f8, PDF_CMAP_SINGLE, 3184 },
	{ 0xd2f9, 0xd2ff, PDF_CMAP_RANGE, 17211 },
	{ 0xd300, 0xd301, PDF_CMAP_RANGE, 3185 },
	{ 0xd302, 0xd302, PDF_CMAP_SINGLE, 17218 },
	{ 0xd303, 0xd303, PDF_CMAP_SINGLE, 3187 },
	{ 0xd304, 0xd304, PDF_CMAP_SINGLE, 17219 },
	{ 0xd305, 0xd305, PDF_CMAP_SINGLE, 3188 },
	{ 0xd306, 0xd30b, PDF_CMAP_RANGE, 17220 },
	{ 0xd30c, 0xd30e, PDF_CMAP_RANGE, 3189 },
	{ 0xd30f, 0xd30f, PDF_CMAP_SINGLE, 17226 },
	{ 0xd310, 0xd310, PDF_CMAP_SINGLE, 3192 },
	{ 0xd311, 0xd313, PDF_CMAP_RANGE, 17227 },
	{ 0xd314, 0xd314, PDF_CMAP_SINGLE, 3193 },
	{ 0xd315, 0xd315, PDF_CMAP_SINGLE, 17230 },
	{ 0xd316, 0xd316, PDF_CMAP_SINGLE, 3194 },
	{ 0xd317, 0xd31b, PDF_CMAP_RANGE, 17231 },
	{ 0xd31c, 0xd31d, PDF_CMAP_RANGE, 3195 },
	{ 0xd31e, 0xd31e, PDF_CMAP_SINGLE, 17236 },
	{ 0xd31f, 0xd321, PDF_CMAP_RANGE, 3197 },
	{ 0xd322, 0xd324, PDF_CMAP_RANGE, 17237 },
	{ 0xd325, 0xd325, PDF_CMAP_SINGLE, 3200 },
	{ 0xd326, 0xd327, PDF_CMAP_RANGE, 17240 },
	{ 0xd328, 0xd329, PDF_CMAP_RANGE, 3201 },
	{ 0xd32a, 0xd32b, PDF_CMAP_RANGE, 17242 },
	{ 0xd32c, 0xd32c, PDF_CMAP_SINGLE, 3203 },
	{ 0xd32d, 0xd32f, PDF_CMAP_RANGE, 17244 },
	{ 0xd330, 0xd330, PDF_CMAP_SINGLE, 3204 },
	{ 0xd331, 0xd337, PDF_CMAP_RANGE, 17247 },
	{ 0xd338, 0xd339, PDF_CMAP_RANGE, 3205 },
	{ 0xd33a, 0xd33a, PDF_CMAP_SINGLE, 17254 },
	{ 0xd33b, 0xd33d, PDF_CMAP_RANGE, 3207 },
	{ 0xd33e, 0xd343, PDF_CMAP_RANGE, 17255 },
	{ 0xd344, 0xd345, PDF_CMAP_RANGE, 3210 },
	{ 0xd346, 0xd37b, PDF_CMAP_RANGE, 17261 },
	{ 0xd37c, 0xd37d, PDF_CMAP_RANGE, 3212 },
	{ 0xd37e, 0xd37f, PDF_CMAP_RANGE, 17315 },
	{ 0xd380, 0xd380, PDF_CMAP_SINGLE, 3214 },
	{ 0xd381, 0xd383, PDF_CMAP_RANGE, 17317 },
	{ 0xd384, 0xd384, PDF_CMAP_SINGLE, 3215 },
	{ 0xd385, 0xd38b, PDF_CMAP_RANGE, 17320 },
	{ 0xd38c, 0xd38d, PDF_CMAP_RANGE, 3216 },
	{ 0xd38e, 0xd38e, PDF_CMAP_SINGLE, 17327 },
	{ 0xd38f, 0xd391, PDF_CMAP_RANGE, 3218 },
	{ 0xd392, 0xd397, PDF_CMAP_RANGE, 17328 },
	{ 0xd398, 0xd399, PDF_CMAP_RANGE, 3221 },
	{ 0xd39a, 0xd39b, PDF_CMAP_RANGE, 17334 },
	{ 0xd39c, 0xd39c, PDF_CMAP_SINGLE, 3223 },
	{ 0xd39d, 0xd39f, PDF_CMAP_RANGE, 17336 },
	{ 0xd3a0, 0xd3a0, PDF_CMAP_SINGLE, 3224 },
	{ 0xd3a1, 0xd3a7, PDF_CMAP_RANGE, 17339 },
	{ 0xd3a8, 0xd3a9, PDF_CMAP_RANGE, 3225 },
	{ 0xd3aa, 0xd3aa, PDF_CMAP_SINGLE, 17346 },
	{ 0xd3ab, 0xd3ab, PDF_CMAP_SINGLE, 3227 },
	{ 0xd3ac, 0xd3ac, PDF_CMAP_SINGLE, 17347 },
	{ 0xd3ad, 0xd3ad, PDF_CMAP_SINGLE, 3228 },
	{ 0xd3ae, 0xd3b3, PDF_CMAP_RANGE, 17348 },
	{ 0xd3b4, 0xd3b4, PDF_CMAP_SINGLE, 3229 },
	{ 0xd3b5, 0xd3b7, PDF_CMAP_RANGE, 17354 },
	{ 0xd3b8, 0xd3b8, PDF_CMAP_SINGLE, 3230 },
	{ 0xd3b9, 0xd3bb, PDF_CMAP_RANGE, 17357 },
	{ 0xd3bc, 0xd3bc, PDF_CMAP_SINGLE, 3231 },
	{ 0xd3bd, 0xd3c3, PDF_CMAP_RANGE, 17360 },
	{ 0xd3c4, 0xd3c5, PDF_CMAP_RANGE, 3232 },
	{ 0xd3c6, 0xd3c7, PDF_CMAP_RANGE, 17367 },
	{ 0xd3c8, 0xd3c9, PDF_CMAP_RANGE, 3234 },
	{ 0xd3ca, 0xd3cf, PDF_CMAP_RANGE, 17369 },
	{ 0xd3d0, 0xd3d0, PDF_CMAP_SINGLE, 3236 },
	{ 0xd3d1, 0xd3d7, PDF_CMAP_RANGE, 17375 },
	{ 0xd3d8, 0xd3d8, PDF_CMAP_SINGLE, 3237 },
	{ 0xd3d9, 0xd3e0, PDF_CMAP_RANGE, 17382 },
	{ 0xd3e1, 0xd3e1, PDF_CMAP_SINGLE, 3238 },
	{ 0xd3e2, 0xd3e2, PDF_CMAP_SINGLE, 17390 },
	{ 0xd3e3, 0xd3e3, PDF_CMAP_SINGLE, 3239 },
	{ 0xd3e4, 0xd3eb, PDF_CMAP_RANGE, 17391 },
	{ 0xd3ec, 0xd3ed, PDF_CMAP_RANGE, 3240 },
	{ 0xd3ee, 0xd3ef, PDF_CMAP_RANGE, 17399 },
	{ 0xd3f0, 0xd3f0, PDF_CMAP_SINGLE, 3242 },
	{ 0xd3f1, 0xd3f3, PDF_CMAP_RANGE, 17401 },
	{ 0xd3f4, 0xd3f4, PDF_CMAP_SINGLE, 3243 },
	{ 0xd3f5, 0xd3fb, PDF_CMAP_RANGE, 17404 },
	{ 0xd3fc, 0xd3fd, PDF_CMAP_RANGE, 3244 },
	{ 0xd3fe, 0xd3fe, PDF_CMAP_SINGLE, 17411 },
	{ 0xd3ff, 0xd3ff, PDF_CMAP_SINGLE, 3246 },
	{ 0xd400, 0xd400, PDF_CMAP_SINGLE, 17412 },
	{ 0xd401, 0xd401, PDF_CMAP_SINGLE, 3247 },
	{ 0xd402, 0xd407, PDF_CMAP_RANGE, 17413 },
	{ 0xd408, 0xd408, PDF_CMAP_SINGLE, 3248 },
	{ 0xd409, 0xd41c, PDF_CMAP_RANGE, 17419 },
	{ 0xd41d, 0xd41d, PDF_CMAP_SINGLE, 3249 },
	{ 0xd41e, 0xd43f, PDF_CMAP_RANGE, 17439 },
	{ 0xd440, 0xd440, PDF_CMAP_SINGLE, 3250 },
	{ 0xd441, 0xd443, PDF_CMAP_RANGE, 17473 },
	{ 0xd444, 0xd444, PDF_CMAP_SINGLE, 3251 },
	{ 0xd445, 0xd45b, PDF_CMAP_RANGE, 17476 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 3252 },
	{ 0xd45d, 0xd45f, PDF_CMAP_RANGE, 17499 },
	{ 0xd460, 0xd460, PDF_CMAP_SINGLE, 3253 },
	{ 0xd461, 0xd463, PDF_CMAP_RANGE, 17502 },
	{ 0xd464, 0xd464, PDF_CMAP_SINGLE, 3254 },
	{ 0xd465, 0xd46c, PDF_CMAP_RANGE, 17505 },
	{ 0xd46d, 0xd46d, PDF_CMAP_SINGLE, 3255 },
	{ 0xd46e, 0xd46e, PDF_CMAP_SINGLE, 17513 },
	{ 0xd46f, 0xd46f, PDF_CMAP_SINGLE, 3256 },
	{ 0xd470, 0xd477, PDF_CMAP_RANGE, 17514 },
	{ 0xd478, 0xd479, PDF_CMAP_RANGE, 3257 },
	{ 0xd47a, 0xd47b, PDF_CMAP_RANGE, 17522 },
	{ 0xd47c, 0xd47c, PDF_CMAP_SINGLE, 3259 },
	{ 0xd47d, 0xd47e, PDF_CMAP_RANGE, 17524 },
	{ 0xd47f, 0xd480, PDF_CMAP_RANGE, 3260 },
	{ 0xd481, 0xd481, PDF_CMAP_SINGLE, 17526 },
	{ 0xd482, 0xd482, PDF_CMAP_SINGLE, 3262 },
	{ 0xd483, 0xd487, PDF_CMAP_RANGE, 17527 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 3263 },
	{ 0xd48a, 0xd48a, PDF_CMAP_SINGLE, 17532 },
	{ 0xd48b, 0xd48b, PDF_CMAP_SINGLE, 3265 },
	{ 0xd48c, 0xd48c, PDF_CMAP_SINGLE, 17533 },
	{ 0xd48d, 0xd48d, PDF_CMAP_SINGLE, 3266 },
	{ 0xd48e, 0xd493, PDF_CMAP_RANGE, 17534 },
	{ 0xd494, 0xd494, PDF_CMAP_SINGLE, 3267 },
	{ 0xd495, 0xd4a8, PDF_CMAP_RANGE, 17540 },
	{ 0xd4a9, 0xd4a9, PDF_CMAP_SINGLE, 3268 },
	{ 0xd4aa, 0xd4cb, PDF_CMAP_RANGE, 17560 },
	{ 0xd4cc, 0xd4cc, PDF_CMAP_SINGLE, 3269 },
	{ 0xd4cd, 0xd4cf, PDF_CMAP_RANGE, 17594 },
	{ 0xd4d0, 0xd4d0, PDF_CMAP_SINGLE, 3270 },
	{ 0xd4d1, 0xd4d3, PDF_CMAP_RANGE, 17597 },
	{ 0xd4d4, 0xd4d4, PDF_CMAP_SINGLE, 3271 },
	{ 0xd4d5, 0xd4db, PDF_CMAP_RANGE, 17600 },
	{ 0xd4dc, 0xd4dc, PDF_CMAP_SINGLE, 3272 },
	{ 0xd4dd, 0xd4de, PDF_CMAP_RANGE, 17607 },
	{ 0xd4df, 0xd4df, PDF_CMAP_SINGLE, 3273 },
	{ 0xd4e0, 0xd4e7, PDF_CMAP_RANGE, 17609 },
	{ 0xd4e8, 0xd4e8, PDF_CMAP_SINGLE, 3274 },
	{ 0xd4e9, 0xd4eb, PDF_CMAP_RANGE, 17617 },
	{ 0xd4ec, 0xd4ec, PDF_CMAP_SINGLE, 3275 },
	{ 0xd4ed, 0xd4ef, PDF_CMAP_RANGE, 17620 },
	{ 0xd4f0, 0xd4f0, PDF_CMAP_SINGLE, 3276 },
	{ 0xd4f1, 0xd4f7, PDF_CMAP_RANGE, 17623 },
	{ 0xd4f8, 0xd4f8, PDF_CMAP_SINGLE, 3277 },
	{ 0xd4f9, 0xd4fa, PDF_CMAP_RANGE, 17630 },
	{ 0xd4fb, 0xd4fb, PDF_CMAP_SINGLE, 3278 },
	{ 0xd4fc, 0xd4fc, PDF_CMAP_SINGLE, 17632 },
	{ 0xd4fd, 0xd4fd, PDF_CMAP_SINGLE, 3279 },
	{ 0xd4fe, 0xd4ff, PDF_CMAP_RANGE, 17633 },
	{ 0xd500, 0xd503, PDF_CMAP_RANGE, 17635 },
	{ 0xd504, 0xd504, PDF_CMAP_SINGLE, 3280 },
	{ 0xd505, 0xd507, PDF_CMAP_RANGE, 17639 },
	{ 0xd508, 0xd508, PDF_CMAP_SINGLE, 3281 },
	{ 0xd509, 0xd50b, PDF_CMAP_RANGE, 17642 },
	{ 0xd50c, 0xd50c, PDF_CMAP_SINGLE, 3282 },
	{ 0xd50d, 0xd513, PDF_CMAP_RANGE, 17645 },
	{ 0xd514, 0xd515, PDF_CMAP_RANGE, 3283 },
	{ 0xd516, 0xd516, PDF_CMAP_SINGLE, 17652 },
	{ 0xd517, 0xd517, PDF_CMAP_SINGLE, 3285 },
	{ 0xd518, 0xd53b, PDF_CMAP_RANGE, 17653 },
	{ 0xd53c, 0xd53d, PDF_CMAP_RANGE, 3286 },
	{ 0xd53e, 0xd53f, PDF_CMAP_RANGE, 17689 },
	{ 0xd540, 0xd540, PDF_CMAP_SINGLE, 3288 },
	{ 0xd541, 0xd543, PDF_CMAP_RANGE, 17691 },
	{ 0xd544, 0xd544, PDF_CMAP_SINGLE, 3289 },
	{ 0xd545, 0xd54b, PDF_CMAP_RANGE, 17694 },
	{ 0xd54c, 0xd54d, PDF_CMAP_RANGE, 3290 },
	{ 0xd54e, 0xd54e, PDF_CMAP_SINGLE, 17701 },
	{ 0xd54f, 0xd54f, PDF_CMAP_SINGLE, 3292 },
	{ 0xd550, 0xd550, PDF_CMAP_SINGLE, 17702 },
	{ 0xd551, 0xd551, PDF_CMAP_SINGLE, 3293 },
	{ 0xd552, 0xd557, PDF_CMAP_RANGE, 17703 },
	{ 0xd558, 0xd559, PDF_CMAP_RANGE, 3294 },
	{ 0xd55a, 0xd55b, PDF_CMAP_RANGE, 17709 },
	{ 0xd55c, 0xd55c, PDF_CMAP_SINGLE, 3296 },
	{ 0xd55d, 0xd55f, PDF_CMAP_RANGE, 17711 },
	{ 0xd560, 0xd560, PDF_CMAP_SINGLE, 3297 },
	{ 0xd561, 0xd564, PDF_CMAP_RANGE, 17714 },
	{ 0xd565, 0xd565, PDF_CMAP_SINGLE, 3298 },
	{ 0xd566, 0xd567, PDF_CMAP_RANGE, 17718 },
	{ 0xd568, 0xd569, PDF_CMAP_RANGE, 3299 },
	{ 0xd56a, 0xd56a, PDF_CMAP_SINGLE, 17720 },
	{ 0xd56b, 0xd56b, PDF_CMAP_SINGLE, 3301 },
	{ 0xd56c, 0xd56c, PDF_CMAP_SINGLE, 17721 },
	{ 0xd56d, 0xd56d, PDF_CMAP_SINGLE, 3302 },
	{ 0xd56e, 0xd573, PDF_CMAP_RANGE, 17722 },
	{ 0xd574, 0xd575, PDF_CMAP_RANGE, 3303 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 17728 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 3305 },
	{ 0xd579, 0xd57b, PDF_CMAP_RANGE, 17730 },
	{ 0xd57c, 0xd57c, PDF_CMAP_SINGLE, 3306 },
	{ 0xd57d, 0xd583, PDF_CMAP_RANGE, 17733 },
	{ 0xd584, 0xd585, PDF_CMAP_RANGE, 3307 },
	{ 0xd586, 0xd586, PDF_CMAP_SINGLE, 17740 },
	{ 0xd587, 0xd589, PDF_CMAP_RANGE, 3309 },
	{ 0xd58a, 0xd58f, PDF_CMAP_RANGE, 17741 },
	{ 0xd590, 0xd590, PDF_CMAP_SINGLE, 3312 },
	{ 0xd591, 0xd5a4, PDF_CMAP_RANGE, 17747 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 3313 },
	{ 0xd5a6, 0xd5c7, PDF_CMAP_RANGE, 17767 },
	{ 0xd5c8, 0xd5c9, PDF_CMAP_RANGE, 3314 },
	{ 0xd5ca, 0xd5cb, PDF_CMAP_RANGE, 17801 },
	{ 0xd5cc, 0xd5cc, PDF_CMAP_SINGLE, 3316 },
	{ 0xd5cd, 0xd5cf, PDF_CMAP_RANGE, 17803 },
	{ 0xd5d0, 0xd5d0, PDF_CMAP_SINGLE, 3317 },
	{ 0xd5d1, 0xd5d1, PDF_CMAP_SINGLE, 17806 },
	{ 0xd5d2, 0xd5d2, PDF_CMAP_SINGLE, 3318 },
	{ 0xd5d3, 0xd5d7, PDF_CMAP_RANGE, 17807 },
	{ 0xd5d8, 0xd5d9, PDF_CMAP_RANGE, 3319 },
	{ 0xd5da, 0xd5da, PDF_CMAP_SINGLE, 17812 },
	{ 0xd5db, 0xd5db, PDF_CMAP_SINGLE, 3321 },
	{ 0xd5dc, 0xd5dc, PDF_CMAP_SINGLE, 17813 },
	{ 0xd5dd, 0xd5dd, PDF_CMAP_SINGLE, 3322 },
	{ 0xd5de, 0xd5e3, PDF_CMAP_RANGE, 17814 },
	{ 0xd5e4, 0xd5e5, PDF_CMAP_RANGE, 3323 },
	{ 0xd5e6, 0xd5e7, PDF_CMAP_RANGE, 17820 },
	{ 0xd5e8, 0xd5e8, PDF_CMAP_SINGLE, 3325 },
	{ 0xd5e9, 0xd5eb, PDF_CMAP_RANGE, 17822 },
	{ 0xd5ec, 0xd5ec, PDF_CMAP_SINGLE, 3326 },
	{ 0xd5ed, 0xd5f3, PDF_CMAP_RANGE, 17825 },
	{ 0xd5f4, 0xd5f5, PDF_CMAP_RANGE, 3327 },
	{ 0xd5f6, 0xd5f6, PDF_CMAP_SINGLE, 17832 },
	{ 0xd5f7, 0xd5f7, PDF_CMAP_SINGLE, 3329 },
	{ 0xd5f8, 0xd5f8, PDF_CMAP_SINGLE, 17833 },
	{ 0xd5f9, 0xd5f9, PDF_CMAP_SINGLE, 3330 },
	{ 0xd5fa, 0xd5ff, PDF_CMAP_RANGE, 17834 },
	{ 0xd600, 0xd601, PDF_CMAP_RANGE, 3331 },
	{ 0xd602, 0xd603, PDF_CMAP_RANGE, 17840 },
	{ 0xd604, 0xd604, PDF_CMAP_SINGLE, 3333 },
	{ 0xd605, 0xd607, PDF_CMAP_RANGE, 17842 },
	{ 0xd608, 0xd608, PDF_CMAP_SINGLE, 3334 },
	{ 0xd609, 0xd60f, PDF_CMAP_RANGE, 17845 },
	{ 0xd610, 0xd611, PDF_CMAP_RANGE, 3335 },
	{ 0xd612, 0xd612, PDF_CMAP_SINGLE, 17852 },
	{ 0xd613, 0xd615, PDF_CMAP_RANGE, 3337 },
	{ 0xd616, 0xd61b, PDF_CMAP_RANGE, 17853 },
	{ 0xd61c, 0xd61c, PDF_CMAP_SINGLE, 3340 },
	{ 0xd61d, 0xd61f, PDF_CMAP_RANGE, 17859 },
	{ 0xd620, 0xd620, PDF_CMAP_SINGLE, 3341 },
	{ 0xd621, 0xd623, PDF_CMAP_RANGE, 17862 },
	{ 0xd624, 0xd624, PDF_CMAP_SINGLE, 3342 },
	{ 0xd625, 0xd62c, PDF_CMAP_RANGE, 17865 },
	{ 0xd62d, 0xd62d, PDF_CMAP_SINGLE, 3343 },
	{ 0xd62e, 0xd637, PDF_CMAP_RANGE, 17873 },
	{ 0xd638, 0xd639, PDF_CMAP_RANGE, 3344 },
	{ 0xd63a, 0xd63b, PDF_CMAP_RANGE, 17883 },
	{ 0xd63c, 0xd63c, PDF_CMAP_SINGLE, 3346 },
	{ 0xd63d, 0xd63f, PDF_CMAP_RANGE, 17885 },
	{ 0xd640, 0xd640, PDF_CMAP_SINGLE, 3347 },
	{ 0xd641, 0xd644, PDF_CMAP_RANGE, 17888 },
	{ 0xd645, 0xd645, PDF_CMAP_SINGLE, 3348 },
	{ 0xd646, 0xd647, PDF_CMAP_RANGE, 17892 },
	{ 0xd648, 0xd649, PDF_CMAP_RANGE, 3349 },
	{ 0xd64a, 0xd64a, PDF_CMAP_SINGLE, 17894 },
	{ 0xd64b, 0xd64b, PDF_CMAP_SINGLE, 3351 },
	{ 0xd64c, 0xd64c, PDF_CMAP_SINGLE, 17895 },
	{ 0xd64d, 0xd64d, PDF_CMAP_SINGLE, 3352 },
	{ 0xd64e, 0xd650, PDF_CMAP_RANGE, 17896 },
	{ 0xd651, 0xd651, PDF_CMAP_SINGLE, 3353 },
	{ 0xd652, 0xd653, PDF_CMAP_RANGE, 17899 },
	{ 0xd654, 0xd655, PDF_CMAP_RANGE, 3354 },
	{ 0xd656, 0xd657, PDF_CMAP_RANGE, 17901 },
	{ 0xd658, 0xd658, PDF_CMAP_SINGLE, 3356 },
	{ 0xd659, 0xd65b, PDF_CMAP_RANGE, 17903 },
	{ 0xd65c, 0xd65c, PDF_CMAP_SINGLE, 3357 },
	{ 0xd65d, 0xd666, PDF_CMAP_RANGE, 17906 },
	{ 0xd667, 0xd667, PDF_CMAP_SINGLE, 3358 },
	{ 0xd668, 0xd668, PDF_CMAP_SINGLE, 17916 },
	{ 0xd669, 0xd669, PDF_CMAP_SINGLE, 3359 },
	{ 0xd66a, 0xd66f, PDF_CMAP_RANGE, 17917 },
	{ 0xd670, 0xd671, PDF_CMAP_RANGE, 3360 },
	{ 0xd672, 0xd673, PDF_CMAP_RANGE, 17923 },
	{ 0xd674, 0xd674, PDF_CMAP_SINGLE, 3362 },
	{ 0xd675, 0xd682, PDF_CMAP_RANGE, 17925 },
	{ 0xd683, 0xd683, PDF_CMAP_SINGLE, 3363 },
	{ 0xd684, 0xd684, PDF_CMAP_SINGLE, 17939 },
	{ 0xd685, 0xd685, PDF_CMAP_SINGLE, 3364 },
	{ 0xd686, 0xd68b, PDF_CMAP_RANGE, 17940 },
	{ 0xd68c, 0xd68d, PDF_CMAP_RANGE, 3365 },
	{ 0xd68e, 0xd68f, PDF_CMAP_RANGE, 17946 },
	{ 0xd690, 0xd690, PDF_CMAP_SINGLE, 3367 },
	{ 0xd691, 0xd693, PDF_CMAP_RANGE, 17948 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 3368 },
	{ 0xd695, 0xd69c, PDF_CMAP_RANGE, 17951 },
	{ 0xd69d, 0xd69d, PDF_CMAP_SINGLE, 3369 },
	{ 0xd69e, 0xd69e, PDF_CMAP_SINGLE, 17959 },
	{ 0xd69f, 0xd69f, PDF_CMAP_SINGLE, 3370 },
	{ 0xd6a0, 0xd6a0, PDF_CMAP_SINGLE, 17960 },
	{ 0xd6a1, 0xd6a1, PDF_CMAP_SINGLE, 3371 },
	{ 0xd6a2, 0xd6a7, PDF_CMAP_RANGE, 17961 },
	{ 0xd6a8, 0xd6a8, PDF_CMAP_SINGLE, 3372 },
	{ 0xd6a9, 0xd6ab, PDF_CMAP_RANGE, 17967 },
	{ 0xd6ac, 0xd6ac, PDF_CMAP_SINGLE, 3373 },
	{ 0xd6ad, 0xd6af, PDF_CMAP_RANGE, 17970 },
	{ 0xd6b0, 0xd6b0, PDF_CMAP_SINGLE, 3374 },
	{ 0xd6b1, 0xd6b8, PDF_CMAP_RANGE, 17973 },
	{ 0xd6b9, 0xd6b9, PDF_CMAP_SINGLE, 3375 },
	{ 0xd6ba, 0xd6ba, PDF_CMAP_SINGLE, 17981 },
	{ 0xd6bb, 0xd6bb, PDF_CMAP_SINGLE, 3376 },
	{ 0xd6bc, 0xd6c3, PDF_CMAP_RANGE, 17982 },
	{ 0xd6c4, 0xd6c5, PDF_CMAP_RANGE, 3377 },
	{ 0xd6c6, 0xd6c7, PDF_CMAP_RANGE, 17990 },
	{ 0xd6c8, 0xd6c8, PDF_CMAP_SINGLE, 3379 },
	{ 0xd6c9, 0xd6cb, PDF_CMAP_RANGE, 17992 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 3380 },
	{ 0xd6cd, 0xd6d0, PDF_CMAP_RANGE, 17995 },
	{ 0xd6d1, 0xd6d1, PDF_CMAP_SINGLE, 3381 },
	{ 0xd6d2, 0xd6d3, PDF_CMAP_RANGE, 17999 },
	{ 0xd6d4, 0xd6d4, PDF_CMAP_SINGLE, 3382 },
	{ 0xd6d5, 0xd6d6, PDF_CMAP_RANGE, 18001 },
	{ 0xd6d7, 0xd6d7, PDF_CMAP_SINGLE, 3383 },
	{ 0xd6d8, 0xd6d8, PDF_CMAP_SINGLE, 18003 },
	{ 0xd6d9, 0xd6d9, PDF_CMAP_SINGLE, 3384 },
	{ 0xd6da, 0xd6df, PDF_CMAP_RANGE, 18004 },
	{ 0xd6e0, 0xd6e0, PDF_CMAP_SINGLE, 3385 },
	{ 0xd6e1, 0xd6e3, PDF_CMAP_RANGE, 18010 },
	{ 0xd6e4, 0xd6e4, PDF_CMAP_SINGLE, 3386 },
	{ 0xd6e5, 0xd6e7, PDF_CMAP_RANGE, 18013 },
	{ 0xd6e8, 0xd6e8, PDF_CMAP_SINGLE, 3387 },
	{ 0xd6e9, 0xd6ef, PDF_CMAP_RANGE, 18016 },
	{ 0xd6f0, 0xd6f0, PDF_CMAP_SINGLE, 3388 },
	{ 0xd6f1, 0xd6f4, PDF_CMAP_RANGE, 18023 },
	{ 0xd6f5, 0xd6f5, PDF_CMAP_SINGLE, 3389 },
	{ 0xd6f6, 0xd6fb, PDF_CMAP_RANGE, 18027 },
	{ 0xd6fc, 0xd6fd, PDF_CMAP_RANGE, 3390 },
	{ 0xd6fe, 0xd6ff, PDF_CMAP_RANGE, 18033 },
	{ 0xd700, 0xd700, PDF_CMAP_SINGLE, 3392 },
	{ 0xd701, 0xd703, PDF_CMAP_RANGE, 18035 },
	{ 0xd704, 0xd704, PDF_CMAP_SINGLE, 3393 },
	{ 0xd705, 0xd710, PDF_CMAP_RANGE, 18038 },
	{ 0xd711, 0xd711, PDF_CMAP_SINGLE, 3394 },
	{ 0xd712, 0xd717, PDF_CMAP_RANGE, 18050 },
	{ 0xd718, 0xd719, PDF_CMAP_RANGE, 3395 },
	{ 0xd71a, 0xd71b, PDF_CMAP_RANGE, 18056 },
	{ 0xd71c, 0xd71c, PDF_CMAP_SINGLE, 3397 },
	{ 0xd71d, 0xd71f, PDF_CMAP_RANGE, 18058 },
	{ 0xd720, 0xd720, PDF_CMAP_SINGLE, 3398 },
	{ 0xd721, 0xd727, PDF_CMAP_RANGE, 18061 },
	{ 0xd728, 0xd729, PDF_CMAP_RANGE, 3399 },
	{ 0xd72a, 0xd72a, PDF_CMAP_SINGLE, 18068 },
	{ 0xd72b, 0xd72b, PDF_CMAP_SINGLE, 3401 },
	{ 0xd72c, 0xd72c, PDF_CMAP_SINGLE, 18069 },
	{ 0xd72d, 0xd72d, PDF_CMAP_SINGLE, 3402 },
	{ 0xd72e, 0xd733, PDF_CMAP_RANGE, 18070 },
	{ 0xd734, 0xd735, PDF_CMAP_RANGE, 3403 },
	{ 0xd736, 0xd737, PDF_CMAP_RANGE, 18076 },
	{ 0xd738, 0xd738, PDF_CMAP_SINGLE, 3405 },
	{ 0xd739, 0xd73b, PDF_CMAP_RANGE, 18078 },
	{ 0xd73c, 0xd73c, PDF_CMAP_SINGLE, 3406 },
	{ 0xd73d, 0xd743, PDF_CMAP_RANGE, 18081 },
	{ 0xd744, 0xd744, PDF_CMAP_SINGLE, 3407 },
	{ 0xd745, 0xd746, PDF_CMAP_RANGE, 18088 },
	{ 0xd747, 0xd747, PDF_CMAP_SINGLE, 3408 },
	{ 0xd748, 0xd748, PDF_CMAP_SINGLE, 18090 },
	{ 0xd749, 0xd749, PDF_CMAP_SINGLE, 3409 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 18091 },
	{ 0xd750, 0xd751, PDF_CMAP_RANGE, 3410 },
	{ 0xd752, 0xd753, PDF_CMAP_RANGE, 18097 },
	{ 0xd754, 0xd754, PDF_CMAP_SINGLE, 3412 },
	{ 0xd755, 0xd755, PDF_CMAP_SINGLE, 18099 },
	{ 0xd756, 0xd759, PDF_CMAP_RANGE, 3413 },
	{ 0xd75a, 0xd75f, PDF_CMAP_RANGE, 18100 },
	{ 0xd760, 0xd761, PDF_CMAP_RANGE, 3417 },
	{ 0xd762, 0xd762, PDF_CMAP_SINGLE, 18106 },
	{ 0xd763, 0xd763, PDF_CMAP_SINGLE, 3419 },
	{ 0xd764, 0xd764, PDF_CMAP_SINGLE, 18107 },
	{ 0xd765, 0xd765, PDF_CMAP_SINGLE, 3420 },
	{ 0xd766, 0xd768, PDF_CMAP_RANGE, 18108 },
	{ 0xd769, 0xd769, PDF_CMAP_SINGLE, 3421 },
	{ 0xd76a, 0xd76b, PDF_CMAP_RANGE, 18111 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 3422 },
	{ 0xd76d, 0xd76f, PDF_CMAP_RANGE, 18113 },
	{ 0xd770, 0xd770, PDF_CMAP_SINGLE, 3423 },
	{ 0xd771, 0xd773, PDF_CMAP_RANGE, 18116 },
	{ 0xd774, 0xd774, PDF_CMAP_SINGLE, 3424 },
	{ 0xd775, 0xd77b, PDF_CMAP_RANGE, 18119 },
	{ 0xd77c, 0xd77d, PDF_CMAP_RANGE, 3425 },
	{ 0xd77e, 0xd780, PDF_CMAP_RANGE, 18126 },
	{ 0xd781, 0xd781, PDF_CMAP_SINGLE, 3427 },
	{ 0xd782, 0xd787, PDF_CMAP_RANGE, 18129 },
	{ 0xd788, 0xd789, PDF_CMAP_RANGE, 3428 },
	{ 0xd78a, 0xd78b, PDF_CMAP_RANGE, 18135 },
	{ 0xd78c, 0xd78c, PDF_CMAP_SINGLE, 3430 },
	{ 0xd78d, 0xd78f, PDF_CMAP_RANGE, 18137 },
	{ 0xd790, 0xd790, PDF_CMAP_SINGLE, 3431 },
	{ 0xd791, 0xd797, PDF_CMAP_RANGE, 18140 },
	{ 0xd798, 0xd799, PDF_CMAP_RANGE, 3432 },
	{ 0xd79a, 0xd79a, PDF_CMAP_SINGLE, 18147 },
	{ 0xd79b, 0xd79b, PDF_CMAP_SINGLE, 3434 },
	{ 0xd79c, 0xd79c, PDF_CMAP_SINGLE, 18148 },
	{ 0xd79d, 0xd79d, PDF_CMAP_SINGLE, 3435 },
	{ 0xd79e, 0xd7a3, PDF_CMAP_RANGE, 18149 },
	{ 0xf900, 0xf900, PDF_CMAP_SINGLE, 4116 },
	{ 0xf901, 0xf901, PDF_CMAP_SINGLE, 3678 },
	{ 0xf902, 0xf902, PDF_CMAP_SINGLE, 7053 },
	{ 0xf903, 0xf903, PDF_CMAP_SINGLE, 3460 },
	{ 0xf904, 0xf904, PDF_CMAP_SINGLE, 7900 },
	{ 0xf905, 0xf905, PDF_CMAP_SINGLE, 3802 },
	{ 0xf906, 0xf906, PDF_CMAP_SINGLE, 3902 },
	{ 0xf907, 0xf907, PDF_CMAP_SINGLE, 3946 },
	{ 0xf908, 0xf908, PDF_CMAP_SINGLE, 3946 },
	{ 0xf909, 0xf909, PDF_CMAP_SINGLE, 3708 },
	{ 0xf90a, 0xf90a, PDF_CMAP_SINGLE, 4131 },
	{ 0xf90b, 0xf90b, PDF_CMAP_SINGLE, 4374 },
	{ 0xf90c, 0xf90c, PDF_CMAP_SINGLE, 4156 },
	{ 0xf90d, 0xf913, PDF_CMAP_RANGE, 4375 },
	{ 0xf914, 0xf914, PDF_CMAP_SINGLE, 5800 },
	{ 0xf915, 0xf917, PDF_CMAP_RANGE, 4382 },
	{ 0xf918, 0xf91d, PDF_CMAP_RANGE, 4386 },
	{ 0xf91e, 0xf920, PDF_CMAP_RANGE, 4394 },
	{ 0xf921, 0xf921, PDF_CMAP_SINGLE, 4399 },
	{ 0xf922, 0xf922, PDF_CMAP_SINGLE, 4403 },
	{ 0xf923, 0xf924, PDF_CMAP_RANGE, 4406 },
	{ 0xf925, 0xf92b, PDF_CMAP_RANGE, 4409 },
	{ 0xf92c, 0xf92d, PDF_CMAP_RANGE, 4419 },
	{ 0xf92e, 0xf92e, PDF_CMAP_SINGLE, 4424 },
	{ 0xf92f, 0xf92f, PDF_CMAP_SINGLE, 4511 },
	{ 0xf930, 0xf931, PDF_CMAP_RANGE, 4513 },
	{ 0xf932, 0xf937, PDF_CMAP_RANGE, 4517 },
	{ 0xf938, 0xf93a, PDF_CMAP_RANGE, 4524 },
	{ 0xf93b, 0xf940, PDF_CMAP_RANGE, 4528 },
	{ 0xf941, 0xf943, PDF_CMAP_RANGE, 4535 },
	{ 0xf944, 0xf945, PDF_CMAP_RANGE, 4541 },
	{ 0xf946, 0xf948, PDF_CMAP_RANGE, 4545 },
	{ 0xf949, 0xf949, PDF_CMAP_SINGLE, 4550 },
	{ 0xf94a, 0xf94a, PDF_CMAP_SINGLE, 4564 },
	{ 0xf94b, 0xf94e, PDF_CMAP_RANGE, 4566 },
	{ 0xf94f, 0xf950, PDF_CMAP_RANGE, 4571 },
	{ 0xf951, 0xf951, PDF_CMAP_SINGLE, 4576 },
	{ 0xf952, 0xf955, PDF_CMAP_RANGE, 4604 },
	{ 0xf956, 0xf959, PDF_CMAP_RANGE, 4609 },
	{ 0xf95a, 0xf95a, PDF_CMAP_SINGLE, 4318 },
	{ 0xf95b, 0xf95b, PDF_CMAP_SINGLE, 4136 },
	{ 0xf95c, 0xf95c, PDF_CMAP_SINGLE, 5800 },
	{ 0xf95d, 0xf95d, PDF_CMAP_SINGLE, 4139 },
	{ 0xf95e, 0xf95e, PDF_CMAP_SINGLE, 4192 },
	{ 0xf95f, 0xf95f, PDF_CMAP_SINGLE, 4167 },
	{ 0xf960, 0xf960, PDF_CMAP_SINGLE, 4172 },
	{ 0xf961, 0xf961, PDF_CMAP_SINGLE, 5552 },
	{ 0xf962, 0xf962, PDF_CMAP_SINGLE, 6424 },
	{ 0xf963, 0xf963, PDF_CMAP_SINGLE, 5151 },
	{ 0xf964, 0xf964, PDF_CMAP_SINGLE, 4922 },
	{ 0xf965, 0xf965, PDF_CMAP_SINGLE, 7518 },
	{ 0xf966, 0xf966, PDF_CMAP_SINGLE, 5079 },
	{ 0xf967, 0xf967, PDF_CMAP_SINGLE, 5109 },
	{ 0xf968, 0xf968, PDF_CMAP_SINGLE, 7607 },
	{ 0xf969, 0xf969, PDF_CMAP_SINGLE, 5584 },
	{ 0xf96a, 0xf96a, PDF_CMAP_SINGLE, 5367 },
	{ 0xf96b, 0xf96b, PDF_CMAP_SINGLE, 7083 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 5362 },
	{ 0xf96d, 0xf96d, PDF_CMAP_SINGLE, 5485 },
	{ 0xf96e, 0xf96e, PDF_CMAP_SINGLE, 6001 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 5460 },
	{ 0xf970, 0xf970, PDF_CMAP_SINGLE, 5317 },
	{ 0xf971, 0xf971, PDF_CMAP_SINGLE, 7009 },
	{ 0xf972, 0xf972, PDF_CMAP_SINGLE, 7370 },
	{ 0xf973, 0xf973, PDF_CMAP_SINGLE, 5678 },
	{ 0xf974, 0xf974, PDF_CMAP_SINGLE, 5874 },
	{ 0xf975, 0xf977, PDF_CMAP_RANGE, 4425 },
	{ 0xf978, 0xf97a, PDF_CMAP_RANGE, 4429 },
	{ 0xf97b, 0xf97d, PDF_CMAP_RANGE, 4435 },
	{ 0xf97e, 0xf97e, PDF_CMAP_SINGLE, 4439 },
	{ 0xf97f, 0xf980, PDF_CMAP_RANGE, 4442 },
	{ 0xf981, 0xf981, PDF_CMAP_SINGLE, 4159 },
	{ 0xf982, 0xf982, PDF_CMAP_SINGLE, 4444 },
	{ 0xf983, 0xf983, PDF_CMAP_SINGLE, 4447 },
	{ 0xf984, 0xf985, PDF_CMAP_RANGE, 4449 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 4453 },
	{ 0xf987, 0xf98c, PDF_CMAP_RANGE, 4455 },
	{ 0xf98d, 0xf98d, PDF_CMAP_SINGLE, 4463 },
	{ 0xf98e, 0xf98e, PDF_CMAP_SINGLE, 4160 },
	{ 0xf98f, 0xf990, PDF_CMAP_RANGE, 4465 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 4161 },
	{ 0xf992, 0xf994, PDF_CMAP_RANGE, 4468 },
	{ 0xf995, 0xf995, PDF_CMAP_SINGLE, 4162 },
	{ 0xf996, 0xf997, PDF_CMAP_RANGE, 4471 },
	{ 0xf998, 0xf998, PDF_CMAP_SINGLE, 4474 },
	{ 0xf999, 0xf999, PDF_CMAP_SINGLE, 4473 },
	{ 0xf99a, 0xf99b, PDF_CMAP_RANGE, 4475 },
	{ 0xf99c, 0xf99d, PDF_CMAP_RANGE, 4478 },
	{ 0xf99e, 0xf99e, PDF_CMAP_SINGLE, 6447 },
	{ 0xf99f, 0xf9a0, PDF_CMAP_RANGE, 4481 },
	{ 0xf9a1, 0xf9a1, PDF_CMAP_SINGLE, 5460 },
	{ 0xf9a2, 0xf9a2, PDF_CMAP_SINGLE, 4483 },
	{ 0xf9a3, 0xf9a3, PDF_CMAP_SINGLE, 4163 },
	{ 0xf9a4, 0xf9a4, PDF_CMAP_SINGLE, 4166 },
	{ 0xf9a5, 0xf9a5, PDF_CMAP_SINGLE, 4485 },
	{ 0xf9a6, 0xf9a8, PDF_CMAP_RANGE, 4487 },
	{ 0xf9a9, 0xf9a9, PDF_CMAP_SINGLE, 4491 },
	{ 0xf9aa, 0xf9aa, PDF_CMAP_SINGLE, 4167 },
	{ 0xf9ab, 0xf9ad, PDF_CMAP_RANGE, 4493 },
	{ 0xf9ae, 0xf9ae, PDF_CMAP_SINGLE, 7783 },
	{ 0xf9af, 0xf9af, PDF_CMAP_SINGLE, 4497 },
	{ 0xf9b0, 0xf9b0, PDF_CMAP_SINGLE, 4499 },
	{ 0xf9b1, 0xf9b4, PDF_CMAP_RANGE, 4501 },
	{ 0xf9b5, 0xf9b5, PDF_CMAP_SINGLE, 4506 },
	{ 0xf9b6, 0xf9b8, PDF_CMAP_RANGE, 4508 },
	{ 0xf9b9, 0xf9b9, PDF_CMAP_SINGLE, 5797 },
	{ 0xf9ba, 0xf9bc, PDF_CMAP_RANGE, 4551 },
	{ 0xf9bd, 0xf9bd, PDF_CMAP_SINGLE, 4180 },
	{ 0xf9be, 0xf9be, PDF_CMAP_SINGLE, 4555 },
	{ 0xf9bf, 0xf9bf, PDF_CMAP_SINGLE, 5800 },
	{ 0xf9c0, 0xf9c1, PDF_CMAP_RANGE, 4556 },
	{ 0xf9c2, 0xf9c3, PDF_CMAP_RANGE, 4560 },
	{ 0xf9c4, 0xf9c4, PDF_CMAP_SINGLE, 4563 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 7988 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 6123 },
	{ 0xf9c7, 0xf9c7, PDF_CMAP_SINGLE, 4577 },
	{ 0xf9c8, 0xf9c8, PDF_CMAP_SINGLE, 4183 },
	{ 0xf9c9, 0xf9c9, PDF_CMAP_SINGLE, 4579 },
	{ 0xf9ca, 0xf9cb, PDF_CMAP_RANGE, 4581 },
	{ 0xf9cc, 0xf9cc, PDF_CMAP_SINGLE, 4584 },
	{ 0xf9cd, 0xf9cd, PDF_CMAP_SINGLE, 4586 },
	{ 0xf9ce, 0xf9ce, PDF_CMAP_SINGLE, 4588 },
	{ 0xf9cf, 0xf9cf, PDF_CMAP_SINGLE, 4184 },
	{ 0xf9d0, 0xf9d3, PDF_CMAP_RANGE, 4590 },
	{ 0xf9d4, 0xf9d6, PDF_CMAP_RANGE, 4595 },
	{ 0xf9d7, 0xf9da, PDF_CMAP_RANGE, 4599 },
	{ 0xf9db, 0xf9db, PDF_CMAP_SINGLE, 5552 },
	{ 0xf9dc, 0xf9dc, PDF_CMAP_SINGLE, 4603 },
	{ 0xf9dd, 0xf9dd, PDF_CMAP_SINGLE, 4614 },
	{ 0xf9de, 0xf9de, PDF_CMAP_SINGLE, 4616 },
	{ 0xf9df, 0xf9df, PDF_CMAP_SINGLE, 4618 },
	{ 0xf9e0, 0xf9e0, PDF_CMAP_SINGLE, 5950 },
	{ 0xf9e1, 0xf9e2, PDF_CMAP_RANGE, 4620 },
	{ 0xf9e3, 0xf9e3, PDF_CMAP_SINGLE, 4187 },
	{ 0xf9e4, 0xf9e4, PDF_CMAP_SINGLE, 4625 },
	{ 0xf9e5, 0xf9e5, PDF_CMAP_SINGLE, 4627 },
	{ 0xf9e6, 0xf9e6, PDF_CMAP_SINGLE, 4629 },
	{ 0xf9e7, 0xf9e9, PDF_CMAP_RANGE, 4632 },
	{ 0xf9ea, 0xf9ea, PDF_CMAP_SINGLE, 4636 },
	{ 0xf9eb, 0xf9ec, PDF_CMAP_RANGE, 4188 },
	{ 0xf9ed, 0xf9ed, PDF_CMAP_SINGLE, 4638 },
	{ 0xf9ee, 0xf9f0, PDF_CMAP_RANGE, 4640 },
	{ 0xf9f1, 0xf9f5, PDF_CMAP_RANGE, 4644 },
	{ 0xf9f6, 0xf9f6, PDF_CMAP_SINGLE, 4650 },
	{ 0xf9f7, 0xf9f9, PDF_CMAP_RANGE, 4653 },
	{ 0xf9fa, 0xf9fa, PDF_CMAP_SINGLE, 5351 },
	{ 0xf9fb, 0xf9fb, PDF_CMAP_SINGLE, 6494 },
	{ 0xf9fc, 0xf9fc, PDF_CMAP_SINGLE, 5731 },
	{ 0xf9fd, 0xf9fd, PDF_CMAP_SINGLE, 5771 },
	{ 0xf9fe, 0xf9fe, PDF_CMAP_SINGLE, 4191 },
	{ 0xf9ff, 0xf9ff, PDF_CMAP_SINGLE, 6484 },
	{ 0xfa00, 0xfa00, PDF_CMAP_SINGLE, 6684 },
	{ 0xfa01, 0xfa01, PDF_CMAP_SINGLE, 4279 },
	{ 0xfa02, 0xfa02, PDF_CMAP_SINGLE, 7139 },
	{ 0xfa03, 0xfa03, PDF_CMAP_SINGLE, 4247 },
	{ 0xfa04, 0xfa04, PDF_CMAP_SINGLE, 4266 },
	{ 0xfa05, 0xfa05, PDF_CMAP_SINGLE, 4340 },
	{ 0xfa06, 0xfa06, PDF_CMAP_SINGLE, 7572 },
	{ 0xfa07, 0xfa07, PDF_CMAP_SINGLE, 5088 },
	{ 0xfa08, 0xfa08, PDF_CMAP_SINGLE, 7709 },
	{ 0xfa09, 0xfa09, PDF_CMAP_SINGLE, 3558 },
	{ 0xfa0a, 0xfa0a, PDF_CMAP_SINGLE, 3644 },
	{ 0xfa0b, 0xfa0b, PDF_CMAP_SINGLE, 3815 },
	{ 0xff01, 0xff3b, PDF_CMAP_RANGE, 264 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 112 },
	{ 0xff3d, 0xff5d, PDF_CMAP_RANGE, 324 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 113 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 143 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 194 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 357 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 145 },
	{ 0xffe6, 0xffe6, PDF_CMAP_SINGLE, 323 },
};

static const unsigned short pdf_cmap_UniKS_UCS2_H_table[3160] =
{
	155,107,668,176,138,130,206,842,
	675,177,751,748,752,209,131,673,
	676,768,132,766,669,760,761,670,
	762,671,764,672,765,772,678,771,
	674,767,677,770,202,205,204,207,
	200,203,8582,9326,9324,156,8763,263,
	260,247,250,248,251,249,8868,8865,
	8864,8869,8867,8815,8814,8854,195,8855,
	196,9190,9192,9191,9193,9198,9200,9199,
	9201,151,198,8715,152,184,8749,185,
	8750,180,136,8717,148,191,190,137,
	181,210,8321,8739,8723,154,8722,133,
	153,8734,8748,8747,8486,8488,8742,8503,
	149,8320,106,519,530,520,531,521,
	558,557,532,522,552,551,533,524,
	556,555,535,523,554,553,534,525,
	546,536,527,548,538,526,542,547,
	537,528,544,549,539,529,165,164,
	237,236,235,238,8736,167,166,8780,
	220,219,169,168,8779,218,217,163,
	162,228,227,8787,159,8639,161,160,
	158,157,241,240,242,9222,243,9223,
	226,221,224,225,239,255,252,8700,
	175,104,9330,261,662,592,650,665,
	259,614,663,607,8790,611,660,654,
	664,659,6460,6704,4670,6534,5320,5331,
	7616,7041,5181,5492,3893,5041,7882,4192,
	6029,4154,5241,7800,7614,5683,6380,3895,
	3613,5862,5092,5315,5242,6413,6252,6238,
	7801,6049,6705,4696,7670,3868,7685,5947,
	7710,3660,6706,4427,5771,6444,4038,3560,
	6483,5244,7380,6535,5110,5418,4250,4489,
	6414,3602,3561,7671,6415,4051,6050,4063,
	5074,5005,8005,5246,3436,4194,6591,7544,
	6284,6592,6861,6854,6198,5935,6461,5171,
	6509,5879,4989,6462,5247,3476,7015,3839,
	4506,4764,7966,7367,4440,3705,7241,5776,
	6051,6902,3661,7734,5248,7759,5737,5567,
	5111,4836,4269,7967,6395,3662,7043,4941,
	7337,3969,6837,3585,6030,4595,7394,6131,
	3438,3627,6285,7330,6708,6199,5303,5182,
	7262,6175,6052,6645,7177,5419,5075,8035,
	5249,6396,6903,6089,6359,6255,7772,7306,
	6783,8013,5420,3840,4016,7431,4735,7449,
	4155,6646,4429,6312,7499,3787,4591,7791,
	5043,4064,3898,6647,5864,4424,6784,4607,
	4333,4606,6390,8036,7904,3563,7167,6141,
	7303,5152,6684,6031,7490,5037,4614,5304,
	6759,5561,3970,7077,6484,3465,5301,4017,
	4397,6648,6480,6649,4018,5021,4577,6760,
	3619,3789,3440,4479,4024,6785,4169,3624,
	4334,6229,3510,4812,5685,4511,4766,5493,
	6619,4025,6510,4009,3899,4848,7547,5076,
	5183,7863,5151,6536,3841,4188,3900,5772,
	7149,6478,5686,6053,7994,4906,5185,6834,
	7395,7760,4800,6446,6287,3466,4390,6256,
	7332,3979,7618,6201,7044,4052,6202,7316,
	5568,6032,7251,3901,3729,3902,3730,6943,
	3994,5500,7500,3441,7432,7016,5250,6203,
	8023,3467,7663,4128,6620,4335,4749,7969,
	4616,7450,7711,6381,7534,8030,7317,4838,
	7970,5738,7802,4750,6944,3870,5841,7590,
	7664,6571,3767,7953,5252,6980,7169,4374,
	7971,7655,7150,7995,7883,8037,3501,6314,
	5336,4132,3871,5253,7338,5569,6868,5570,
	5254,4491,3733,7550,5902,6259,6258,4218,
	3511,6855,3581,3906,6260,6178,6905,5948,
	5117,4971,4069,3637,7459,3537,6355,5793,
	6142,5918,5060,6537,4273,3857,6002,4274,
	4241,6650,6057,6931,7720,4834,4319,6058,
	6538,4564,3842,3858,4536,5255,6470,6463,
	7804,5374,4071,5403,6135,5630,4190,7151,
	7433,5118,6143,5834,4072,4156,5094,3996,
	6869,7884,3708,5156,7730,7418,6360,6540,
	5404,5157,4159,4170,5936,5186,4697,6144,
	4039,6595,3734,5475,6289,6472,6145,3538,
	6864,6060,4134,5959,5739,4852,6261,3445,
	5573,5422,3872,6488,7754,3790,6481,6489,
	6832,6490,7955,3709,3735,5631,6522,6204,
	5574,5807,5553,6108,7427,7950,6838,3820,
	6870,6711,6109,6397,3580,5423,5759,6316,
	7885,7691,5960,5503,3446,5740,6180,5632,
	7116,6622,6263,4074,6450,4883,3907,7635,
	6205,5721,4664,7078,3803,7368,6061,6146,
	5760,4167,7117,5762,5258,3821,4553,6906,
	7253,5061,6541,6653,6291,6833,5763,4254,
	4275,5504,5697,6361,7136,3736,4186,4853,
	4180,3948,3587,3710,5699,5698,7335,4618,
	4075,6527,5575,3531,4870,5808,4421,4076,
	5843,3540,4854,3513,5375,5795,7152,6871,
	5640,5506,3791,6856,3874,3588,4077,6418,
	5261,7476,5547,3604,5700,7551,7973,6712,
	5796,7571,4996,4242,7536,3481,7530,4160,
	5045,7706,3482,4279,6857,5824,5377,3541,
	6181,7620,4483,4412,6872,6655,4803,7097,
	7537,3843,5961,6714,3605,7930,3964,6787,
	6451,5172,7854,7736,7807,4198,7792,5919,
	6230,7118,7578,6788,5225,7513,7098,6004,
	5173,6126,6715,7931,4600,7974,5378,3671,
	4280,4364,4972,5263,4422,5903,7906,5079,
	5642,4079,6452,7248,6947,4494,5264,7434,
	6421,5477,6264,6473,5643,5702,7932,6491,
	8010,7341,7636,6374,3468,4164,3793,5723,
	8031,6765,7637,5554,6656,6062,6320,7887,
	7254,5189,4268,7312,4281,7132,6321,7793,
	5797,7385,4178,5866,5478,7333,4872,7517,
	6398,6207,5844,3566,6265,6375,6492,3655,
	7435,7910,4675,3822,3469,7794,3567,3672,
	3544,7137,6168,7255,6208,4465,5237,8044,
	8043,7412,4873,3673,3483,7692,6391,6063,
	4815,4375,7934,7737,6399,4466,4243,3804,
	5667,5579,6369,5479,5781,3712,6657,8045,
	4446,4948,5507,7520,5424,7809,5191,5687,
	4080,3648,7477,5914,5380,7676,6686,5830,
	7266,4817,4165,4409,4909,4888,4136,3861,
	3589,7139,4931,7387,6148,3911,6835,7219,
	4973,3837,5724,4129,6934,5678,4137,6717,
	5963,4676,7763,4143,5962,5098,5265,4144,
	4166,5581,4283,4425,7119,7420,3795,7267,
	5929,6789,4804,6766,5883,7888,3629,7999,
	5867,5327,6149,4284,4910,7425,7447,5316,
	6150,7171,4932,7478,7261,4513,7153,6790,
	3674,4259,7399,4400,7079,3592,3875,7878,
	4401,5583,3739,6325,3568,4949,6718,5382,
	3876,4874,6064,7505,3517,5306,3675,5885,
	6719,6624,5125,5584,4484,7539,5193,4912,
	3768,5266,4027,7141,6512,7088,5267,5742,
	5904,5703,5426,6720,6464,4200,6952,6791,
	5644,6231,3777,5688,4952,7810,4751,7845,
	5161,4041,5950,5407,8019,5046,5480,6005,
	4712,6513,5704,7621,6232,5047,5836,6792,
	5705,6873,8046,5481,6066,7890,7935,7811,
	5743,4677,5062,3676,5408,7198,6721,3990,
	6722,3447,7988,8000,6293,5383,4140,5825,
	5886,6546,7795,7572,4459,4221,6152,7573,
	3769,6033,3678,3502,5385,6794,6793,6933,
	7206,7264,6281,6326,5302,7031,4798,4538,
	4788,4857,4691,5091,7080,7304,4620,7707,
	6575,7249,4085,5539,4975,3615,4337,5555,
	7492,4146,5409,3805,6954,3743,7531,4991,
	4773,3518,7365,5990,6327,3450,4157,6875,
	4579,5707,7127,5268,5645,6658,5386,4602,
	3877,4992,7704,4028,3631,6576,3616,7678,
	3713,4285,4338,5346,7891,4130,6577,7342,
	6110,7220,6067,4621,6767,3714,3778,5011,
	4858,4087,5049,6797,4020,4339,4245,6523,
	3715,4286,3823,5725,7268,7592,5452,7389,
	4820,7222,5932,6599,5646,7693,7269,6007,
	6988,4954,3915,7460,7104,7270,6548,4892,
	5585,7866,6239,5647,4015,4366,5915,4042,
	3620,6549,4287,3985,4448,6927,8027,7048,
	8028,3824,8029,7727,7889,6956,6726,7049,
	5063,4821,4460,3991,5837,6989,5648,5586,
	4485,5228,6402,3917,4309,6356,5196,7812,
	3918,5238,6009,6727,6929,3919,5938,7855,
	3545,6957,4091,5162,5765,4092,6079,6267,
	7679,5965,7370,4796,4234,4739,4849,7308,
	5272,6958,7506,5200,6331,7343,5512,3745,
	7179,5966,7914,7776,7154,4893,7556,7481,
	6388,4914,4878,7438,5889,4480,5454,7857,
	5649,8016,3845,6268,8032,7482,6687,6910,
	7813,4414,7694,7371,7764,3680,5513,5472,
	6184,7960,5967,3977,7344,5411,7961,4093,
	4288,6843,4568,4597,6382,7846,7199,5930,
	7155,7180,7892,6602,3520,6730,3593,4289,
	5274,4933,5801,6099,5455,7334,6294,4860,
	6578,7847,4976,4861,4223,5727,6731,7915,
	6453,7429,6269,4148,4582,3920,3716,6083,
	7105,4746,6493,7142,6579,4290,3780,4679,
	5905,7582,5970,6799,7640,4468,4680,6959,
	5589,7106,7033,4915,5634,6529,4515,5412,
	4224,6800,4341,5066,4639,6880,7514,7641,
	7446,7939,7063,3633,7402,4486,4175,6768,
	7818,6334,4403,6912,7778,4310,7642,6014,
	6370,4461,6732,5515,4539,4544,7779,5562,
	6514,6580,8021,6494,3846,7740,3594,7415,
	6881,6515,6696,7858,4383,4977,5163,4469,
	7992,4716,7893,7976,6801,6242,6253,4367,
	4326,4999,5975,6802,7064,5590,7243,5473,
	7985,5744,4292,6154,7731,7483,6587,6516,
	5129,5869,7962,5349,6422,6552,7521,7493,
	4311,5782,4778,4545,5388,4623,4624,7765,
	4415,7509,4729,5708,6836,5482,6604,4808,
	7901,6272,6080,5276,4214,7725,6496,5552,
	3923,6211,4094,4495,4260,4880,5308,7608,
	6992,7977,4384,5651,5067,6423,7713,7782,
	3999,6735,7743,4416,4625,5593,6663,5636,
	7404,7822,4095,4096,5102,6115,6844,3826,
	5993,4649,4043,5201,7484,7848,6212,4779,
	7302,5430,4173,7823,6019,6337,5389,5674,
	4585,6155,6084,6993,7783,4657,4097,4626,
	4895,3683,5025,7065,7583,7494,5768,7181,
	5371,5518,5068,6190,6664,6338,3532,5745,
	4149,6665,4235,4917,6995,7289,4825,4426,
	8008,6914,6424,7868,4246,3547,6883,7192,
	7610,5520,5519,7558,7599,3522,4215,6996,
	4342,6605,7022,7346,8022,4352,3686,6085,
	7143,7108,4918,4368,4935,3572,7920,6213,
	4978,5484,4293,6526,4731,3523,4919,7421,
	4862,3978,4780,3748,6037,5653,4558,7182,
	3621,4781,4060,5709,6404,4204,6134,3881,
	6961,7120,7373,4588,3687,4528,5847,5563,
	4546,7050,4450,4462,4923,5279,4103,6806,
	6963,6807,3595,7826,7290,5746,7869,6739,
	5081,5907,8052,4045,7870,4315,5597,5281,
	5053,4162,7273,3807,7229,7000,5838,6976,
	7349,7591,6809,6235,6977,4296,3454,3720,
	3749,3771,6628,6022,5366,6038,3965,3798,
	7024,7109,3966,6157,7067,4002,6689,4653,
	3689,6555,6900,7291,6770,4496,4031,6668,
	5654,5006,3849,7292,7456,4896,4107,6690,
	7921,6671,7522,6531,5524,7183,4487,5131,
	6884,6630,6811,6697,4434,7070,3582,5166,
	6812,3548,5873,7859,6217,8025,7895,5283,
	3865,6964,4056,5167,5525,4957,5367,6500,
	6886,7081,4571,5498,5526,3525,6846,7746,
	6813,3691,4924,4385,7747,7457,5284,3549,
	6743,5601,5602,6340,3550,4701,7173,7121,
	4109,7416,6517,4610,4742,4126,5208,7034,
	4207,7209,4881,7523,6117,7001,5855,7002,
	7828,6171,7294,5980,7257,6631,4826,6158,
	6978,5436,5689,7942,3722,4317,4405,4702,
	7643,6859,3854,5437,6405,4630,3583,5355,
	8033,6632,7322,5209,6160,4519,6501,4111,
	4782,6243,4472,7258,5488,6191,7200,4542,
	6368,6429,5638,5285,6815,4605,4113,3752,
	6965,3643,4958,3866,6358,6298,6865,4980,
	3533,6364,5394,7766,7324,7767,4722,3472,
	3695,3552,6120,5489,4179,6161,3608,6557,
	5082,5438,7461,4959,4667,5675,4228,7943,
	4176,4359,6559,4650,6967,7354,4261,7003,
	3930,5941,8034,5456,5286,7158,5657,4827,
	6888,7683,4960,4925,7390,4900,7749,3496,
	4436,3497,5368,6518,3967,5137,4360,3575,
	7830,5168,7487,6039,5769,6969,6244,4032,
	7274,4783,5783,6274,5995,7441,7233,7562,
	3933,5874,3754,6608,3755,7612,6921,4828,
	5017,3456,4784,7944,6455,4191,6192,5943,
	7234,7784,6431,7923,4631,6560,5138,5784,
	4668,4531,3949,7872,3756,4611,5213,7314,
	4297,4732,7532,6299,7831,6561,5876,7113,
	7832,5549,4684,5084,6504,5325,5527,6383,
	3981,7430,4829,7989,5319,5751,5396,7986,
	4406,5752,6564,4451,6040,4370,5877,5003,
	5609,4642,6816,4520,5529,5924,5028,3609,
	7133,7721,4521,6221,7834,3799,6817,5105,
	5753,6916,4452,7709,5981,7750,5668,7786,
	3934,6409,6275,4763,5610,7005,6582,4482,
	6042,6343,5070,6565,7566,5071,6304,4983,
	5086,7511,4298,7363,6745,5139,8026,7406,
	4114,7006,6890,6818,7533,5661,6043,3987,
	6673,8055,7873,7698,5358,6971,6457,5490,
	3725,4830,6075,3760,5558,6377,6819,6345,
	7194,7210,7699,6746,3498,6346,6506,5770,
	7593,6774,4785,5818,6305,5543,4965,3657,
	6442,3556,6634,4589,8011,4115,5953,6411,
	7090,5898,7837,6045,7586,5719,7512,6747,
	5140,6583,3800,5234,7875,7495,7423,3832,
	7129,6613,5499,6433,3992,7528,4718,4263,
	5218,7195,6432,4831,7626,5169,4547,6476,
	7947,6507,3460,7751,4719,7160,5142,3936,
	5364,6972,7315,6365,6941,4006,5143,4936,
	7028,4237,7161,3610,4301,7052,7797,6638,
	7145,4643,5755,3968,7053,5819,3988,3957,
	7196,4686,5073,3698,4438,7356,8003,4706,
	7174,4474,5055,4985,4599,5613,5087,6674,
	5944,7650,6276,7175,3888,5756,3761,5035,
	7009,6173,4177,6639,4902,7463,5559,3838,
	4302,7978,5954,7469,7299,3938,4303,3699,
	6615,7458,5399,4500,5545,6822,6917,5106,
	4475,6895,7010,4361,5614,6349,6246,7525,
	3812,7627,7925,4304,4216,6306,5550,4238,
	7212,6277,5532,6823,7054,4362,7162,5445,
	4561,3598,7602,6166,4720,7700,5615,7896,
	6434,4669,7788,4138,3526,7443,3939,6352,
	6749,7280,6519,4986,6896,4760,4387,7843,
	5313,6412,6567,7240,7798,4509,3584,5416,
	4634,6925,5871,4439,4635,4131,5566,6750,
	5146,7374,3622,7497,5329,6676,3534,3940,
	5670,6751,3941,5534,5664,4233,6753,4118,
	6678,3762,7059,4476,3813,7838,7011,3499,
	4882,3475,7667,5008,5996,5820,7668,3983,
	7473,7164,5029,5788,7391,4576,4724,3558,
	7543,7669,5470,5691,7146,6278,7012,6777,
	4612,4307,7661,4593,6226,4603,4264,5619,
	7926,3727,3636,6247,5849,4022,6778,6569,
	6520,5813,6254,5789,7037,3763,6919,6508,
	6097,6617,6826,5622,7285,6533,4636,4142,
	4847,5170,4502,4550,6754,7014,4868,4744,
	7715,7839,7757,6757,3701,7684,5665,6048,
	6124,4928,4328,7489,4504,3702,7464,6780,
	5857,5804,5221,5004,5551,4120,7362,6386,
	4929,5298,7569,5734,7716,5901,3989,7076,
	5448,4121,6167,4661,5909,7596,7358,5666,
	6899,4174,3942,7286,5536,3703,5955,7897,
	4455,5626,7213,3863,7849,4940,4995,4721,
	5108,4762,3773,4833,8012,6393,4526,4125,
	4534,3952,4745,5401,4457,4265,7305,6077,
	5040,6781,6585,1117,9377,1118,9413,1127,
	9442,1148,1160,9472,1161,9485,1167,9486,
	1168,9489,1172,9490,1178,9501,1179,9507,
	1182,9508,1183,9549,1196,9550,1197,1201,
	9571,1202,9588,1213,9589,9655,1230,9699,
	1242,9700,1243,9738,1250,9744,1253,9745,
	1256,9749,9750,1260,1261,9754,1262,9838,
	1284,1295,9871,1296,1306,9929,1307,9930,
	1308,9938,1311,9939,10052,1337,10053,1359,
	10156,1360,10157,1361,10158,1362,10163,1366,
	10164,1367,10210,1375,10211,1376,10218,1380,
	10227,1392,10228,1413,10302,1445,10382,1446,
	10388,1449,10389,1450,10459,1462,1467,10481,
	1468,10482,1469,10500,1477,10501,1478,10616,
	1501,10617,1504,10621,1505,1511,10652,1512,
	10658,1515,10659,1516,1517,10664,10665,1521,
	10749,1549,10757,1556,10758,1557,10759,1558,
	1559,10762,10835,1581,10836,1582,10840,1585,
	10841,1586,10842,1587,10843,1588,10910,1599,
	10958,1607,10959,1608,1616,11032,1617,11033,
	1618,11067,1629,11073,1632,11074,1633,1668,
	11223,11574,1719,11619,1730,11620,1731,1756,
	11678,1757,11725,1764,11743,1774,11744,1775,
	1787,11785,1788,11806,1795,11807,1796,11879,
	1806,11880,1807,1811,11901,1812,11902,1813,
	11921,1820,11922,1821,1830,11996,1831,12015,
	1838,12016,1839,12035,1846,12036,12080,1857,
	12081,1858,1861,12090,12096,1869,12097,1872,
	12100,1891,12176,1892,12182,1895,12183,12222,
	1915,1921,12258,1922,12264,1925,12265,1926,
	1934,12338,1935,12339,1936,1940,12360,1941,
	12369,1945,12377,1952,12378,1953,1954,12382,
	1959,12397,1960,12536,1980,12543,1986,12544,
	1987,12553,2000,12554,2001,12634,2022,12640,
	2025,12641,2042,12678,12710,2052,12721,2056,
	12722,2057,12837,2080,12838,2081,12842,2084,
	2097,12933,2098,12952,2105,2109,12994,2110,
	13000,2113,13001,2120,13011,2121,2173,13333,
	2174,13513,2188,13514,2189,13566,2219,13567,
	2220,2240,13609,13672,2266,2267,13676,2268,
	13682,2271,13683,2272,13747,2290,13768,2297,
	13769,2298,13787,2306,13788,2307,13789,2308,
	13790,13791,2311,13852,2325,13853,2326,2331,
	13875,2332,13893,2340,13894,2341,13939,2350,
	13940,2351,2352,13945,2375,14044,2376,14142,
	2390,2422,14389,2423,14440,2437,14441,2438,
	14490,2471,14491,2472,2473,14495,14528,2494,
	14529,2495,14530,14543,2502,14544,2503,14551,
	2507,2508,14555,14589,2533,14592,2537,14593,
	2538,14594,2539,2553,14633,2554,14652,2561,
	14653,2562,14672,2569,14673,2570,14690,2579,
	14691,2580,14751,2602,14752,2603,14771,2610,
	14772,2611,14773,2612,2617,14788,14789,2620,
	14790,2646,14827,2649,14830,14831,2653,2671,
	14864,2672,14865,2673,2682,14913,2683,14919,
	2686,14920,14938,2695,14939,2696,2708,14999,
	2709,15005,2712,15006,2720,15024,2721,15025,
	2722,15070,2731,15071,2732,15112,2745,15113,
	2746,15185,2756,15230,2767,15231,2768,15270,
	2774,15276,2777,15277,2784,15283,2785,15284,
	2786,15481,2824,15482,2825,15483,2826,2853,
	15733,2854,2866,15789,2867,15790,2868,2885,
	15826,2886,15827,2887,15904,2905,15905,2906,
	15974,2919,15975,2920,16047,2930,16048,2931,
	16093,2940,16094,2941,16166,2951,16167,2952,
	16209,2964,16210,2965,16255,2974,16256,2975,
	16275,2982,16276,2983,16385,3012,16386,3013,
	16452,3029,16453,3030,16573,3048,16574,3049,
	16643,3062,16644,3063,16736,3081,16737,3082,
	3107,16840,3108,16865,3120,16866,3121,16936,
	3133,16937,3134,3135,16942,3136,3141,17013,
	3142,17059,3150,17060,3151,17168,3172,17174,
	3175,17218,3187,17219,3188,17226,3192,3193,
	17230,3194,17346,3227,17347,3228,3238,17390,
	3239,17411,3246,17412,3247,3255,17513,3256,
	17526,3262,17532,3265,17533,3266,3278,17632,
	3279,17652,3285,17701,3292,17702,3293,17720,
	3301,17721,3302,3317,17806,3318,17812,3321,
	17813,3322,17832,3329,17833,3330,17894,3351,
	17895,3352,3358,17916,3359,3363,17939,3364,
	3369,17959,3370,17960,3371,3375,17981,3376,
	3383,18003,3384,18068,3401,18069,3402,3408,
	18090,3409,3412,18099,18106,3419,18107,3420,
	18147,3434,18148,3435,4116,3678,7053,3460,
	7900,3802,3902,3946,3946,3708,4131,4374,
	4156,4399,4403,4424,4511,4550,4564,4318,
	4136,5800,4139,4192,4167,4172,5552,6424,
	5151,4922,7518,5079,5109,7607,5584,5367,
	7083,5362,5485,6001,5460,5317,7009,7370,
	5678,5874,4159,4444,4447,4463,4160,4474,
	4473,5460,4483,4163,4166,4485,4491,4167,
	7783,4497,4499,4180,4555,5800,4563,7988,
	6123,4577,4183,4579,4584,4586,4588,4184,
	5552,4603,4614,4616,4618,5950,4187,4625,
	4627,4629,5351,6494,5731,5771,4191,6484,
	6684,4279,7139,4247,4266,4340,7572,5088,
	7709,3558,3644,3815,194,357,145,323,
};

pdf_cmap pdf_cmap_UniKS_UCS2_H =
{
	-1, "UniKS-UCS2-H", "", nil, 0,
	2, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 2, 0xe000, 0xffff },
	},
	8394, 8394, (pdf_range*) pdf_cmap_UniKS_UCS2_H_ranges,
	3160, 3160, (unsigned short*) pdf_cmap_UniKS_UCS2_H_table,
};

/* UniKS-UCS2-V */

static const pdf_range pdf_cmap_UniKS_UCS2_V_ranges[] =
{
	{ 0x2013, 0x2014, PDF_CMAP_RANGE, 8059 },
	{ 0x2016, 0x2016, PDF_CMAP_SINGLE, 8061 },
	{ 0x2025, 0x2025, PDF_CMAP_SINGLE, 8058 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 8056 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 8065 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 8075 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 8063 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 8076 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 8077 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8079 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8080 },
	{ 0xff1a, 0xff1f, PDF_CMAP_RANGE, 8081 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 8087 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 8088 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 8089 },
	{ 0xff5b, 0xff5d, PDF_CMAP_RANGE, 8090 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 8062 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 8093 },
};

static const unsigned short pdf_cmap_UniKS_UCS2_V_table[1] = { 0 };

pdf_cmap pdf_cmap_UniKS_UCS2_V =
{
	-1, "UniKS-UCS2-V", "UniKS-UCS2-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	18, 18, (pdf_range*) pdf_cmap_UniKS_UCS2_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_UniKS_UCS2_V_table,
};

/* UniKS-UTF16-H */

static const pdf_range pdf_cmap_UniKS_UTF16_H_ranges[] =
{
	{ 0x0020, 0x007e, PDF_CMAP_RANGE, 1 },
	{ 0x00a1, 0x00a1, PDF_CMAP_SINGLE, 208 },
	{ 0x00a4, 0x00a4, PDF_CMAP_SINGLE, 214 },
	{ 0x00a7, 0x00ab, PDF_CMAP_TABLE, 0 },
	{ 0x00b0, 0x00b1, PDF_CMAP_TABLE, 5 },
	{ 0x00b2, 0x00b3, PDF_CMAP_RANGE, 843 },
	{ 0x00b4, 0x00b4, PDF_CMAP_SINGLE, 199 },
	{ 0x00b6, 0x00bf, PDF_CMAP_TABLE, 7 },
	{ 0x00c6, 0x00c6, PDF_CMAP_SINGLE, 666 },
	{ 0x00d0, 0x00d0, PDF_CMAP_SINGLE, 667 },
	{ 0x00d7, 0x00d8, PDF_CMAP_TABLE, 17 },
	{ 0x00de, 0x00df, PDF_CMAP_TABLE, 19 },
	{ 0x00e6, 0x00e6, PDF_CMAP_SINGLE, 757 },
	{ 0x00f0, 0x00f0, PDF_CMAP_SINGLE, 759 },
	{ 0x00f7, 0x00f8, PDF_CMAP_TABLE, 21 },
	{ 0x00fe, 0x00fe, PDF_CMAP_SINGLE, 769 },
	{ 0x0111, 0x0111, PDF_CMAP_SINGLE, 758 },
	{ 0x0126, 0x0127, PDF_CMAP_TABLE, 23 },
	{ 0x0131, 0x0133, PDF_CMAP_TABLE, 25 },
	{ 0x0138, 0x0138, PDF_CMAP_SINGLE, 763 },
	{ 0x013f, 0x0142, PDF_CMAP_TABLE, 28 },
	{ 0x0149, 0x014b, PDF_CMAP_TABLE, 32 },
	{ 0x0152, 0x0153, PDF_CMAP_TABLE, 35 },
	{ 0x0166, 0x0167, PDF_CMAP_TABLE, 37 },
	{ 0x02bc, 0x02bc, PDF_CMAP_SINGLE, 8275 },
	{ 0x02c7, 0x02c7, PDF_CMAP_SINGLE, 201 },
	{ 0x02d8, 0x02dd, PDF_CMAP_TABLE, 39 },
	{ 0x0391, 0x03a1, PDF_CMAP_RANGE, 471 },
	{ 0x03a3, 0x03a9, PDF_CMAP_RANGE, 488 },
	{ 0x03b1, 0x03c1, PDF_CMAP_RANGE, 495 },
	{ 0x03c3, 0x03c9, PDF_CMAP_RANGE, 512 },
	{ 0x0401, 0x0401, PDF_CMAP_SINGLE, 1026 },
	{ 0x0410, 0x0415, PDF_CMAP_RANGE, 1020 },
	{ 0x0416, 0x0435, PDF_CMAP_RANGE, 1027 },
	{ 0x0436, 0x044f, PDF_CMAP_RANGE, 1060 },
	{ 0x0451, 0x0451, PDF_CMAP_SINGLE, 1059 },
	{ 0x2010, 0x2010, PDF_CMAP_SINGLE, 97 },
	{ 0x2013, 0x2014, PDF_CMAP_RANGE, 109 },
	{ 0x2016, 0x2016, PDF_CMAP_SINGLE, 111 },
	{ 0x2018, 0x2019, PDF_CMAP_RANGE, 114 },
	{ 0x201b, 0x201b, PDF_CMAP_SINGLE, 8238 },
	{ 0x201c, 0x201d, PDF_CMAP_RANGE, 116 },
	{ 0x201f, 0x201f, PDF_CMAP_SINGLE, 8237 },
	{ 0x2020, 0x2021, PDF_CMAP_RANGE, 245 },
	{ 0x2022, 0x2022, PDF_CMAP_SINGLE, 8607 },
	{ 0x2025, 0x2026, PDF_CMAP_RANGE, 105 },
	{ 0x2030, 0x2030, PDF_CMAP_SINGLE, 216 },
	{ 0x2032, 0x2033, PDF_CMAP_RANGE, 139 },
	{ 0x2034, 0x2036, PDF_CMAP_TABLE, 45 },
	{ 0x2039, 0x203a, PDF_CMAP_RANGE, 8612 },
	{ 0x203b, 0x203c, PDF_CMAP_TABLE, 48 },
	{ 0x2042, 0x2042, PDF_CMAP_SINGLE, 8599 },
	{ 0x2074, 0x2074, PDF_CMAP_SINGLE, 845 },
	{ 0x207a, 0x207b, PDF_CMAP_RANGE, 8239 },
	{ 0x207c, 0x207c, PDF_CMAP_SINGLE, 8248 },
	{ 0x207d, 0x207e, PDF_CMAP_RANGE, 8250 },
	{ 0x207f, 0x207f, PDF_CMAP_SINGLE, 846 },
	{ 0x2081, 0x2084, PDF_CMAP_RANGE, 847 },
	{ 0x20a9, 0x20a9, PDF_CMAP_SINGLE, 96 },
	{ 0x2103, 0x2103, PDF_CMAP_SINGLE, 141 },
	{ 0x2109, 0x2109, PDF_CMAP_SINGLE, 215 },
	{ 0x2113, 0x2113, PDF_CMAP_SINGLE, 590 },
	{ 0x2116, 0x2116, PDF_CMAP_SINGLE, 258 },
	{ 0x2121, 0x2122, PDF_CMAP_TABLE, 50 },
	{ 0x2126, 0x2126, PDF_CMAP_SINGLE, 643 },
	{ 0x212b, 0x212b, PDF_CMAP_SINGLE, 142 },
	{ 0x2153, 0x2154, PDF_CMAP_RANGE, 749 },
	{ 0x215b, 0x215e, PDF_CMAP_RANGE, 753 },
	{ 0x2160, 0x2169, PDF_CMAP_RANGE, 461 },
	{ 0x2170, 0x2179, PDF_CMAP_RANGE, 451 },
	{ 0x2190, 0x2191, PDF_CMAP_RANGE, 171 },
	{ 0x2192, 0x2192, PDF_CMAP_SINGLE, 170 },
	{ 0x2193, 0x2194, PDF_CMAP_RANGE, 173 },
	{ 0x2195, 0x2199, PDF_CMAP_TABLE, 52 },
	{ 0x21b0, 0x21b4, PDF_CMAP_TABLE, 57 },
	{ 0x21bc, 0x21bc, PDF_CMAP_SINGLE, 8884 },
	{ 0x21c0, 0x21c0, PDF_CMAP_SINGLE, 8885 },
	{ 0x21c4, 0x21c5, PDF_CMAP_RANGE, 8896 },
	{ 0x21cd, 0x21cd, PDF_CMAP_SINGLE, 8816 },
	{ 0x21cf, 0x21d4, PDF_CMAP_TABLE, 62 },
	{ 0x21e0, 0x21e3, PDF_CMAP_TABLE, 68 },
	{ 0x21e6, 0x21e9, PDF_CMAP_TABLE, 72 },
	{ 0x2200, 0x2200, PDF_CMAP_SINGLE, 197 },
	{ 0x2202, 0x2203, PDF_CMAP_TABLE, 76 },
	{ 0x2206, 0x2209, PDF_CMAP_TABLE, 78 },
	{ 0x220b, 0x220c, PDF_CMAP_TABLE, 82 },
	{ 0x220f, 0x220f, PDF_CMAP_SINGLE, 213 },
	{ 0x2211, 0x2211, PDF_CMAP_SINGLE, 212 },
	{ 0x2213, 0x2213, PDF_CMAP_SINGLE, 8726 },
	{ 0x221a, 0x221a, PDF_CMAP_SINGLE, 178 },
	{ 0x221d, 0x2220, PDF_CMAP_TABLE, 84 },
	{ 0x2222, 0x2222, PDF_CMAP_SINGLE, 8738 },
	{ 0x2225, 0x2226, PDF_CMAP_RANGE, 8719 },
	{ 0x2227, 0x2228, PDF_CMAP_RANGE, 192 },
	{ 0x2229, 0x222a, PDF_CMAP_TABLE, 88 },
	{ 0x222b, 0x222c, PDF_CMAP_RANGE, 182 },
	{ 0x222e, 0x222e, PDF_CMAP_SINGLE, 211 },
	{ 0x2234, 0x2237, PDF_CMAP_TABLE, 90 },
	{ 0x223d, 0x223d, PDF_CMAP_SINGLE, 179 },
	{ 0x2243, 0x2243, PDF_CMAP_SINGLE, 8500 },
	{ 0x2245, 0x2245, PDF_CMAP_SINGLE, 8499 },
	{ 0x2248, 0x2248, PDF_CMAP_SINGLE, 8501 },
	{ 0x2250, 0x2253, PDF_CMAP_TABLE, 94 },
	{ 0x225a, 0x225a, PDF_CMAP_SINGLE, 8753 },
	{ 0x2260, 0x2262, PDF_CMAP_TABLE, 98 },
	{ 0x2264, 0x2265, PDF_CMAP_RANGE, 134 },
	{ 0x2266, 0x2267, PDF_CMAP_RANGE, 8724 },
	{ 0x226e, 0x226f, PDF_CMAP_RANGE, 8745 },
	{ 0x2270, 0x2273, PDF_CMAP_RANGE, 8481 },
	{ 0x2276, 0x2277, PDF_CMAP_RANGE, 8489 },
	{ 0x2279, 0x2279, PDF_CMAP_SINGLE, 8491 },
	{ 0x227a, 0x227b, PDF_CMAP_RANGE, 8475 },
	{ 0x2280, 0x2281, PDF_CMAP_RANGE, 8479 },
	{ 0x2282, 0x2283, PDF_CMAP_RANGE, 188 },
	{ 0x2284, 0x2285, PDF_CMAP_TABLE, 101 },
	{ 0x2286, 0x2287, PDF_CMAP_RANGE, 186 },
	{ 0x228a, 0x228b, PDF_CMAP_TABLE, 103 },
	{ 0x2295, 0x2297, PDF_CMAP_RANGE, 8727 },
	{ 0x22a3, 0x22a5, PDF_CMAP_TABLE, 105 },
	{ 0x22bb, 0x22bc, PDF_CMAP_RANGE, 8751 },
	{ 0x22ce, 0x22cf, PDF_CMAP_RANGE, 8477 },
	{ 0x22da, 0x22db, PDF_CMAP_RANGE, 8492 },
	{ 0x22ee, 0x22ef, PDF_CMAP_TABLE, 108 },
	{ 0x2306, 0x2306, PDF_CMAP_SINGLE, 8754 },
	{ 0x2312, 0x2312, PDF_CMAP_SINGLE, 150 },
	{ 0x2314, 0x2314, PDF_CMAP_SINGLE, 8731 },
	{ 0x2460, 0x246e, PDF_CMAP_RANGE, 733 },
	{ 0x246f, 0x2473, PDF_CMAP_RANGE, 8791 },
	{ 0x2474, 0x2482, PDF_CMAP_RANGE, 827 },
	{ 0x2483, 0x2487, PDF_CMAP_RANGE, 9042 },
	{ 0x249c, 0x24b5, PDF_CMAP_RANGE, 801 },
	{ 0x24b6, 0x24cf, PDF_CMAP_RANGE, 8388 },
	{ 0x24d0, 0x24e9, PDF_CMAP_RANGE, 707 },
	{ 0x2500, 0x2503, PDF_CMAP_TABLE, 110 },
	{ 0x250c, 0x251d, PDF_CMAP_TABLE, 114 },
	{ 0x251e, 0x251f, PDF_CMAP_RANGE, 559 },
	{ 0x2520, 0x2520, PDF_CMAP_SINGLE, 541 },
	{ 0x2521, 0x2522, PDF_CMAP_RANGE, 561 },
	{ 0x2523, 0x2525, PDF_CMAP_TABLE, 132 },
	{ 0x2526, 0x2527, PDF_CMAP_RANGE, 563 },
	{ 0x2528, 0x2528, PDF_CMAP_SINGLE, 543 },
	{ 0x2529, 0x252a, PDF_CMAP_RANGE, 565 },
	{ 0x252b, 0x252c, PDF_CMAP_TABLE, 135 },
	{ 0x252d, 0x252e, PDF_CMAP_RANGE, 567 },
	{ 0x252f, 0x2530, PDF_CMAP_TABLE, 137 },
	{ 0x2531, 0x2532, PDF_CMAP_RANGE, 569 },
	{ 0x2533, 0x2534, PDF_CMAP_TABLE, 139 },
	{ 0x2535, 0x2536, PDF_CMAP_RANGE, 571 },
	{ 0x2537, 0x2538, PDF_CMAP_TABLE, 141 },
	{ 0x2539, 0x253a, PDF_CMAP_RANGE, 573 },
	{ 0x253b, 0x253c, PDF_CMAP_TABLE, 143 },
	{ 0x253d, 0x253e, PDF_CMAP_RANGE, 575 },
	{ 0x253f, 0x253f, PDF_CMAP_SINGLE, 545 },
	{ 0x2540, 0x2541, PDF_CMAP_RANGE, 577 },
	{ 0x2542, 0x2542, PDF_CMAP_SINGLE, 550 },
	{ 0x2543, 0x254a, PDF_CMAP_RANGE, 579 },
	{ 0x254b, 0x254b, PDF_CMAP_SINGLE, 540 },
	{ 0x2592, 0x2592, PDF_CMAP_SINGLE, 232 },
	{ 0x25a0, 0x25a1, PDF_CMAP_TABLE, 145 },
	{ 0x25a3, 0x25a3, PDF_CMAP_SINGLE, 229 },
	{ 0x25a4, 0x25a5, PDF_CMAP_RANGE, 233 },
	{ 0x25a6, 0x25a9, PDF_CMAP_TABLE, 147 },
	{ 0x25b1, 0x25b3, PDF_CMAP_TABLE, 151 },
	{ 0x25b5, 0x25b7, PDF_CMAP_TABLE, 154 },
	{ 0x25b9, 0x25b9, PDF_CMAP_SINGLE, 8781 },
	{ 0x25bc, 0x25bd, PDF_CMAP_TABLE, 157 },
	{ 0x25bf, 0x25c1, PDF_CMAP_TABLE, 159 },
	{ 0x25c3, 0x25c3, PDF_CMAP_SINGLE, 8782 },
	{ 0x25c6, 0x25cc, PDF_CMAP_TABLE, 162 },
	{ 0x25ce, 0x25cf, PDF_CMAP_TABLE, 169 },
	{ 0x25d0, 0x25d1, PDF_CMAP_RANGE, 230 },
	{ 0x25e6, 0x25e6, PDF_CMAP_SINGLE, 8775 },
	{ 0x25ef, 0x25ef, PDF_CMAP_SINGLE, 8633 },
	{ 0x2605, 0x2606, PDF_CMAP_TABLE, 171 },
	{ 0x260e, 0x260f, PDF_CMAP_TABLE, 173 },
	{ 0x261c, 0x261f, PDF_CMAP_TABLE, 175 },
	{ 0x262f, 0x262f, PDF_CMAP_SINGLE, 8664 },
	{ 0x2640, 0x2640, PDF_CMAP_SINGLE, 147 },
	{ 0x2642, 0x2642, PDF_CMAP_SINGLE, 146 },
	{ 0x2660, 0x2661, PDF_CMAP_RANGE, 222 },
	{ 0x2663, 0x2665, PDF_CMAP_TABLE, 179 },
	{ 0x2667, 0x2668, PDF_CMAP_TABLE, 182 },
	{ 0x2669, 0x266a, PDF_CMAP_RANGE, 253 },
	{ 0x266c, 0x266d, PDF_CMAP_TABLE, 184 },
	{ 0x266f, 0x266f, PDF_CMAP_SINGLE, 8594 },
	{ 0x2716, 0x2716, PDF_CMAP_SINGLE, 8631 },
	{ 0x271a, 0x271a, PDF_CMAP_SINGLE, 8630 },
	{ 0x273d, 0x273d, PDF_CMAP_SINGLE, 8604 },
	{ 0x2756, 0x2756, PDF_CMAP_SINGLE, 8637 },
	{ 0x2776, 0x277f, PDF_CMAP_RANGE, 8673 },
	{ 0x278a, 0x2793, PDF_CMAP_RANGE, 8342 },
	{ 0x2f00, 0x2f00, PDF_CMAP_SINGLE, 6460 },
	{ 0x2f04, 0x2f04, PDF_CMAP_SINGLE, 6380 },
	{ 0x2f06, 0x2f06, PDF_CMAP_SINGLE, 6413 },
	{ 0x2f08, 0x2f08, PDF_CMAP_SINGLE, 6443 },
	{ 0x2f0a, 0x2f0b, PDF_CMAP_TABLE, 186 },
	{ 0x2f11, 0x2f12, PDF_CMAP_TABLE, 188 },
	{ 0x2f14, 0x2f14, PDF_CMAP_SINGLE, 5183 },
	{ 0x2f17, 0x2f18, PDF_CMAP_TABLE, 190 },
	{ 0x2f1c, 0x2f1d, PDF_CMAP_TABLE, 192 },
	{ 0x2f1f, 0x2f20, PDF_CMAP_TABLE, 194 },
	{ 0x2f23, 0x2f26, PDF_CMAP_TABLE, 196 },
	{ 0x2f28, 0x2f29, PDF_CMAP_TABLE, 200 },
	{ 0x2f2b, 0x2f2b, PDF_CMAP_SINGLE, 5697 },
	{ 0x2f2d, 0x2f2d, PDF_CMAP_SINGLE, 5305 },
	{ 0x2f2f, 0x2f32, PDF_CMAP_TABLE, 202 },
	{ 0x2f38, 0x2f38, PDF_CMAP_SINGLE, 3964 },
	{ 0x2f3c, 0x2f40, PDF_CMAP_TABLE, 206 },
	{ 0x2f42, 0x2f4c, PDF_CMAP_TABLE, 211 },
	{ 0x2f4f, 0x2f52, PDF_CMAP_TABLE, 222 },
	{ 0x2f54, 0x2f58, PDF_CMAP_TABLE, 226 },
	{ 0x2f5a, 0x2f66, PDF_CMAP_TABLE, 231 },
	{ 0x2f69, 0x2f70, PDF_CMAP_TABLE, 244 },
	{ 0x2f72, 0x2f76, PDF_CMAP_TABLE, 252 },
	{ 0x2f78, 0x2f78, PDF_CMAP_SINGLE, 5132 },
	{ 0x2f7a, 0x2f7d, PDF_CMAP_TABLE, 257 },
	{ 0x2f7f, 0x2f8b, PDF_CMAP_TABLE, 261 },
	{ 0x2f8e, 0x2f90, PDF_CMAP_TABLE, 274 },
	{ 0x2f92, 0x2f97, PDF_CMAP_TABLE, 277 },
	{ 0x2f99, 0x2fa0, PDF_CMAP_TABLE, 283 },
	{ 0x2fa2, 0x2fa3, PDF_CMAP_TABLE, 291 },
	{ 0x2fa5, 0x2fa9, PDF_CMAP_TABLE, 293 },
	{ 0x2fac, 0x2fb1, PDF_CMAP_TABLE, 298 },
	{ 0x2fb3, 0x2fbc, PDF_CMAP_TABLE, 304 },
	{ 0x2fc1, 0x2fca, PDF_CMAP_TABLE, 314 },
	{ 0x2fcd, 0x2fd4, PDF_CMAP_TABLE, 324 },
	{ 0x3000, 0x3002, PDF_CMAP_RANGE, 101 },
	{ 0x3003, 0x3003, PDF_CMAP_SINGLE, 108 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 120 },
	{ 0x3012, 0x3013, PDF_CMAP_TABLE, 332 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 118 },
	{ 0x3016, 0x3019, PDF_CMAP_RANGE, 8219 },
	{ 0x301e, 0x301f, PDF_CMAP_RANGE, 9322 },
	{ 0x3020, 0x3020, PDF_CMAP_SINGLE, 8671 },
	{ 0x3036, 0x3036, PDF_CMAP_SINGLE, 8701 },
	{ 0x3041, 0x3093, PDF_CMAP_RANGE, 851 },
	{ 0x30a1, 0x30f6, PDF_CMAP_RANGE, 934 },
	{ 0x30fb, 0x30fc, PDF_CMAP_TABLE, 334 },
	{ 0x3131, 0x3163, PDF_CMAP_RANGE, 358 },
	{ 0x3164, 0x3164, PDF_CMAP_SINGLE, 101 },
	{ 0x3165, 0x318e, PDF_CMAP_RANGE, 409 },
	{ 0x3200, 0x321b, PDF_CMAP_RANGE, 773 },
	{ 0x321c, 0x321c, PDF_CMAP_SINGLE, 257 },
	{ 0x3231, 0x3231, PDF_CMAP_SINGLE, 8788 },
	{ 0x3239, 0x3239, PDF_CMAP_SINGLE, 8789 },
	{ 0x3260, 0x327b, PDF_CMAP_RANGE, 679 },
	{ 0x327f, 0x327f, PDF_CMAP_SINGLE, 256 },
	{ 0x328a, 0x328f, PDF_CMAP_RANGE, 9301 },
	{ 0x3290, 0x3290, PDF_CMAP_SINGLE, 9300 },
	{ 0x3294, 0x3294, PDF_CMAP_SINGLE, 9080 },
	{ 0x329e, 0x329e, PDF_CMAP_SINGLE, 8761 },
	{ 0x32a5, 0x32a5, PDF_CMAP_SINGLE, 9096 },
	{ 0x3380, 0x3384, PDF_CMAP_RANGE, 627 },
	{ 0x3388, 0x3389, PDF_CMAP_RANGE, 612 },
	{ 0x338a, 0x338c, PDF_CMAP_RANGE, 646 },
	{ 0x338d, 0x338f, PDF_CMAP_RANGE, 608 },
	{ 0x3390, 0x3394, PDF_CMAP_RANGE, 638 },
	{ 0x3395, 0x3397, PDF_CMAP_RANGE, 587 },
	{ 0x3398, 0x3398, PDF_CMAP_SINGLE, 591 },
	{ 0x3399, 0x33a2, PDF_CMAP_RANGE, 597 },
	{ 0x33a3, 0x33a6, PDF_CMAP_RANGE, 593 },
	{ 0x33a7, 0x33a8, PDF_CMAP_RANGE, 615 },
	{ 0x33a9, 0x33ac, PDF_CMAP_RANGE, 655 },
	{ 0x33ad, 0x33af, PDF_CMAP_RANGE, 651 },
	{ 0x33b0, 0x33b9, PDF_CMAP_RANGE, 617 },
	{ 0x33ba, 0x33bf, PDF_CMAP_RANGE, 632 },
	{ 0x33c0, 0x33c1, PDF_CMAP_RANGE, 644 },
	{ 0x33c2, 0x33cb, PDF_CMAP_TABLE, 336 },
	{ 0x33cf, 0x33d0, PDF_CMAP_TABLE, 346 },
	{ 0x33d3, 0x33d3, PDF_CMAP_SINGLE, 661 },
	{ 0x33d6, 0x33d6, PDF_CMAP_SINGLE, 649 },
	{ 0x33d8, 0x33d8, PDF_CMAP_SINGLE, 262 },
	{ 0x33db, 0x33dd, PDF_CMAP_TABLE, 348 },
	{ 0x4e00, 0x4e01, PDF_CMAP_TABLE, 351 },
	{ 0x4e03, 0x4e03, PDF_CMAP_SINGLE, 7364 },
	{ 0x4e07, 0x4e0b, PDF_CMAP_TABLE, 353 },
	{ 0x4e0d, 0x4e0d, PDF_CMAP_SINGLE, 5109 },
	{ 0x4e11, 0x4e11, PDF_CMAP_SINGLE, 7288 },
	{ 0x4e14, 0x4e16, PDF_CMAP_TABLE, 358 },
	{ 0x4e18, 0x4e19, PDF_CMAP_TABLE, 361 },
	{ 0x4e1e, 0x4e1e, PDF_CMAP_SINGLE, 5682 },
	{ 0x4e2d, 0x4e2d, PDF_CMAP_SINGLE, 6922 },
	{ 0x4e32, 0x4e32, PDF_CMAP_SINGLE, 3802 },
	{ 0x4e38, 0x4e39, PDF_CMAP_TABLE, 363 },
	{ 0x4e3b, 0x4e3b, PDF_CMAP_SINGLE, 6860 },
	{ 0x4e42, 0x4e43, PDF_CMAP_TABLE, 365 },
	{ 0x4e45, 0x4e45, PDF_CMAP_SINGLE, 3894 },
	{ 0x4e4b, 0x4e4b, PDF_CMAP_SINGLE, 6942 },
	{ 0x4e4d, 0x4e4f, PDF_CMAP_TABLE, 367 },
	{ 0x4e56, 0x4e56, PDF_CMAP_SINGLE, 3855 },
	{ 0x4e58, 0x4e59, PDF_CMAP_TABLE, 370 },
	{ 0x4e5d, 0x4e5f, PDF_CMAP_TABLE, 372 },
	{ 0x4e6b, 0x4e6b, PDF_CMAP_SINGLE, 3500 },
	{ 0x4e6d, 0x4e6d, PDF_CMAP_SINGLE, 4329 },
	{ 0x4e73, 0x4e73, PDF_CMAP_SINGLE, 6309 },
	{ 0x4e76, 0x4e77, PDF_CMAP_TABLE, 375 },
	{ 0x4e7e, 0x4e7e, PDF_CMAP_SINGLE, 3601 },
	{ 0x4e82, 0x4e82, PDF_CMAP_SINGLE, 4389 },
	{ 0x4e86, 0x4e86, PDF_CMAP_SINGLE, 4551 },
	{ 0x4e88, 0x4e88, PDF_CMAP_SINGLE, 5934 },
	{ 0x4e8b, 0x4e8c, PDF_CMAP_TABLE, 377 },
	{ 0x4e8e, 0x4e8e, PDF_CMAP_SINGLE, 6197 },
	{ 0x4e90, 0x4e92, PDF_CMAP_TABLE, 379 },
	{ 0x4e94, 0x4e95, PDF_CMAP_TABLE, 382 },
	{ 0x4e98, 0x4e98, PDF_CMAP_SINGLE, 4058 },
	{ 0x4e9b, 0x4e9b, PDF_CMAP_SINGLE, 5243 },
	{ 0x4e9e, 0x4e9e, PDF_CMAP_SINGLE, 5775 },
	{ 0x4ea1, 0x4ea2, PDF_CMAP_TABLE, 384 },
	{ 0x4ea4, 0x4ea6, PDF_CMAP_TABLE, 386 },
	{ 0x4ea8, 0x4ea8, PDF_CMAP_SINGLE, 7771 },
	{ 0x4eab, 0x4eae, PDF_CMAP_TABLE, 389 },
	{ 0x4eb6, 0x4eb6, PDF_CMAP_SINGLE, 4193 },
	{ 0x4eba, 0x4eba, PDF_CMAP_SINGLE, 6443 },
	{ 0x4ec0, 0x4ec1, PDF_CMAP_TABLE, 393 },
	{ 0x4ec4, 0x4ec4, PDF_CMAP_SINGLE, 7331 },
	{ 0x4ec7, 0x4ec7, PDF_CMAP_SINGLE, 3896 },
	{ 0x4eca, 0x4ecb, PDF_CMAP_TABLE, 395 },
	{ 0x4ecd, 0x4ecd, PDF_CMAP_SINGLE, 6479 },
	{ 0x4ed4, 0x4ed9, PDF_CMAP_TABLE, 397 },
	{ 0x4edd, 0x4edd, PDF_CMAP_SINGLE, 4331 },
	{ 0x4edf, 0x4edf, PDF_CMAP_SINGLE, 7148 },
	{ 0x4ee3, 0x4ee5, PDF_CMAP_TABLE, 403 },
	{ 0x4ef0, 0x4ef0, PDF_CMAP_SINGLE, 5833 },
	{ 0x4ef2, 0x4ef2, PDF_CMAP_SINGLE, 6923 },
	{ 0x4ef6, 0x4ef7, PDF_CMAP_TABLE, 406 },
	{ 0x4efb, 0x4efb, PDF_CMAP_SINGLE, 6469 },
	{ 0x4f01, 0x4f01, PDF_CMAP_SINGLE, 4062 },
	{ 0x4f09, 0x4f0b, PDF_CMAP_TABLE, 408 },
	{ 0x4f0d, 0x4f11, PDF_CMAP_TABLE, 411 },
	{ 0x4f2f, 0x4f2f, PDF_CMAP_SINGLE, 4988 },
	{ 0x4f34, 0x4f34, PDF_CMAP_SINGLE, 4905 },
	{ 0x4f36, 0x4f36, PDF_CMAP_SINGLE, 4490 },
	{ 0x4f38, 0x4f38, PDF_CMAP_SINGLE, 5735 },
	{ 0x4f3a, 0x4f3a, PDF_CMAP_SINGLE, 5245 },
	{ 0x4f3c, 0x4f3d, PDF_CMAP_TABLE, 416 },
	{ 0x4f43, 0x4f43, PDF_CMAP_SINGLE, 6643 },
	{ 0x4f46, 0x4f48, PDF_CMAP_TABLE, 418 },
	{ 0x4f4d, 0x4f51, PDF_CMAP_TABLE, 421 },
	{ 0x4f55, 0x4f55, PDF_CMAP_SINGLE, 7617 },
	{ 0x4f59, 0x4f5c, PDF_CMAP_TABLE, 426 },
	{ 0x4f69, 0x4f69, PDF_CMAP_SINGLE, 7502 },
	{ 0x4f6f, 0x4f70, PDF_CMAP_TABLE, 430 },
	{ 0x4f73, 0x4f73, PDF_CMAP_SINGLE, 3437 },
	{ 0x4f76, 0x4f76, PDF_CMAP_SINGLE, 4127 },
	{ 0x4f7a, 0x4f7a, PDF_CMAP_SINGLE, 6644 },
	{ 0x4f7e, 0x4f7f, PDF_CMAP_TABLE, 432 },
	{ 0x4f81, 0x4f81, PDF_CMAP_SINGLE, 5736 },
	{ 0x4f83, 0x4f84, PDF_CMAP_TABLE, 434 },
	{ 0x4f86, 0x4f86, PDF_CMAP_SINGLE, 4420 },
	{ 0x4f88, 0x4f88, PDF_CMAP_SINGLE, 7336 },
	{ 0x4f8a, 0x4f8b, PDF_CMAP_TABLE, 436 },
	{ 0x4f8d, 0x4f8d, PDF_CMAP_SINGLE, 5692 },
	{ 0x4f8f, 0x4f8f, PDF_CMAP_SINGLE, 6862 },
	{ 0x4f91, 0x4f91, PDF_CMAP_SINGLE, 6310 },
	{ 0x4f96, 0x4f96, PDF_CMAP_SINGLE, 4594 },
	{ 0x4f98, 0x4f98, PDF_CMAP_SINGLE, 7042 },
	{ 0x4f9b, 0x4f9b, PDF_CMAP_SINGLE, 3786 },
	{ 0x4f9d, 0x4f9d, PDF_CMAP_SINGLE, 6394 },
	{ 0x4fae, 0x4faf, PDF_CMAP_TABLE, 438 },
	{ 0x4fb5, 0x4fb6, PDF_CMAP_TABLE, 440 },
	{ 0x4fbf, 0x4fbf, PDF_CMAP_SINGLE, 7518 },
	{ 0x4fc2, 0x4fc4, PDF_CMAP_TABLE, 442 },
	{ 0x4fc9, 0x4fca, PDF_CMAP_TABLE, 445 },
	{ 0x4fce, 0x4fce, PDF_CMAP_SINGLE, 6782 },
	{ 0x4fd1, 0x4fd1, PDF_CMAP_SINGLE, 6174 },
	{ 0x4fd3, 0x4fd4, PDF_CMAP_TABLE, 447 },
	{ 0x4fd7, 0x4fd7, PDF_CMAP_SINGLE, 5537 },
	{ 0x4fda, 0x4fda, PDF_CMAP_SINGLE, 4613 },
	{ 0x4fdd, 0x4fdd, PDF_CMAP_SINGLE, 5058 },
	{ 0x4fdf, 0x4fe1, PDF_CMAP_TABLE, 449 },
	{ 0x4fee, 0x4fef, PDF_CMAP_TABLE, 452 },
	{ 0x4ff1, 0x4ff1, PDF_CMAP_SINGLE, 3897 },
	{ 0x4ff3, 0x4ff3, PDF_CMAP_SINGLE, 4970 },
	{ 0x4ff5, 0x4ff5, PDF_CMAP_SINGLE, 7576 },
	{ 0x4ff8, 0x4ff8, PDF_CMAP_SINGLE, 5093 },
	{ 0x4ffa, 0x4ffa, PDF_CMAP_SINGLE, 5925 },
	{ 0x5002, 0x5002, PDF_CMAP_SINGLE, 5042 },
	{ 0x5006, 0x5006, PDF_CMAP_SINGLE, 4428 },
	{ 0x5009, 0x5009, PDF_CMAP_SINGLE, 7092 },
	{ 0x500b, 0x500b, PDF_CMAP_SINGLE, 3562 },
	{ 0x500d, 0x500d, PDF_CMAP_SINGLE, 4969 },
	{ 0x5011, 0x5012, PDF_CMAP_TABLE, 454 },
	{ 0x5016, 0x5016, PDF_CMAP_SINGLE, 7705 },
	{ 0x5019, 0x501a, PDF_CMAP_TABLE, 456 },
	{ 0x501c, 0x501c, PDF_CMAP_SINGLE, 7134 },
	{ 0x501e, 0x501f, PDF_CMAP_TABLE, 458 },
	{ 0x5021, 0x5021, PDF_CMAP_SINGLE, 7093 },
	{ 0x5023, 0x5024, PDF_CMAP_TABLE, 460 },
	{ 0x5026, 0x5028, PDF_CMAP_TABLE, 462 },
	{ 0x502a, 0x502d, PDF_CMAP_TABLE, 465 },
	{ 0x503b, 0x503b, PDF_CMAP_SINGLE, 5863 },
	{ 0x5043, 0x5043, PDF_CMAP_SINGLE, 5917 },
	{ 0x5047, 0x5049, PDF_CMAP_TABLE, 469 },
	{ 0x504f, 0x504f, PDF_CMAP_SINGLE, 7519 },
	{ 0x5055, 0x5055, PDF_CMAP_SINGLE, 7686 },
	{ 0x505a, 0x505a, PDF_CMAP_SINGLE, 6863 },
	{ 0x505c, 0x505c, PDF_CMAP_SINGLE, 6707 },
	{ 0x5065, 0x5065, PDF_CMAP_SINGLE, 3603 },
	{ 0x5074, 0x5076, PDF_CMAP_TABLE, 472 },
	{ 0x5078, 0x5078, PDF_CMAP_SINGLE, 7465 },
	{ 0x5080, 0x5080, PDF_CMAP_SINGLE, 3856 },
	{ 0x5085, 0x5085, PDF_CMAP_SINGLE, 5112 },
	{ 0x508d, 0x508d, PDF_CMAP_SINGLE, 4942 },
	{ 0x5091, 0x5091, PDF_CMAP_SINGLE, 3614 },
	{ 0x5098, 0x5099, PDF_CMAP_TABLE, 475 },
	{ 0x50ac, 0x50ad, PDF_CMAP_TABLE, 477 },
	{ 0x50b2, 0x50b3, PDF_CMAP_TABLE, 479 },
	{ 0x50b5, 0x50b5, PDF_CMAP_SINGLE, 7114 },
	{ 0x50b7, 0x50b7, PDF_CMAP_SINGLE, 5332 },
	{ 0x50be, 0x50be, PDF_CMAP_SINGLE, 3663 },
	{ 0x50c5, 0x50c5, PDF_CMAP_SINGLE, 4023 },
	{ 0x50c9, 0x50ca, PDF_CMAP_TABLE, 481 },
	{ 0x50cf, 0x50cf, PDF_CMAP_SINGLE, 5333 },
	{ 0x50d1, 0x50d1, PDF_CMAP_SINGLE, 3869 },
	{ 0x50d5, 0x50d6, PDF_CMAP_TABLE, 483 },
	{ 0x50da, 0x50da, PDF_CMAP_SINGLE, 4552 },
	{ 0x50de, 0x50de, PDF_CMAP_SINGLE, 6286 },
	{ 0x50e5, 0x50e5, PDF_CMAP_SINGLE, 6140 },
	{ 0x50e7, 0x50e7, PDF_CMAP_SINGLE, 5684 },
	{ 0x50ed, 0x50ed, PDF_CMAP_SINGLE, 7082 },
	{ 0x50f9, 0x50f9, PDF_CMAP_SINGLE, 3439 },
	{ 0x50fb, 0x50fb, PDF_CMAP_SINGLE, 5020 },
	{ 0x50ff, 0x5101, PDF_CMAP_TABLE, 485 },
	{ 0x5104, 0x5104, PDF_CMAP_SINGLE, 5912 },
	{ 0x5106, 0x5106, PDF_CMAP_SINGLE, 3664 },
	{ 0x5109, 0x5109, PDF_CMAP_SINGLE, 3617 },
	{ 0x5112, 0x5112, PDF_CMAP_SINGLE, 6311 },
	{ 0x511f, 0x511f, PDF_CMAP_SINGLE, 5334 },
	{ 0x5121, 0x5121, PDF_CMAP_SINGLE, 4543 },
	{ 0x512a, 0x512a, PDF_CMAP_SINGLE, 6200 },
	{ 0x5132, 0x5132, PDF_CMAP_SINGLE, 6593 },
	{ 0x5137, 0x5137, PDF_CMAP_SINGLE, 4441 },
	{ 0x513a, 0x513a, PDF_CMAP_SINGLE, 4133 },
	{ 0x513c, 0x513c, PDF_CMAP_SINGLE, 5926 },
	{ 0x5140, 0x5141, PDF_CMAP_TABLE, 488 },
	{ 0x5143, 0x5149, PDF_CMAP_TABLE, 490 },
	{ 0x514b, 0x514e, PDF_CMAP_TABLE, 497 },
	{ 0x5152, 0x5152, PDF_CMAP_SINGLE, 5777 },
	{ 0x515c, 0x515c, PDF_CMAP_SINGLE, 4348 },
	{ 0x5162, 0x5162, PDF_CMAP_SINGLE, 4059 },
	{ 0x5165, 0x5165, PDF_CMAP_SINGLE, 6477 },
	{ 0x5167, 0x516e, PDF_CMAP_TABLE, 501 },
	{ 0x5171, 0x5171, PDF_CMAP_SINGLE, 3788 },
	{ 0x5175, 0x5178, PDF_CMAP_TABLE, 509 },
	{ 0x517c, 0x517c, PDF_CMAP_SINGLE, 3654 },
	{ 0x5180, 0x5180, PDF_CMAP_SINGLE, 4065 },
	{ 0x5186, 0x5186, PDF_CMAP_SINGLE, 5933 },
	{ 0x518a, 0x518a, PDF_CMAP_SINGLE, 7126 },
	{ 0x518d, 0x518d, PDF_CMAP_SINGLE, 6570 },
	{ 0x5192, 0x5192, PDF_CMAP_SINGLE, 4765 },
	{ 0x5195, 0x5195, PDF_CMAP_SINGLE, 4736 },
	{ 0x5197, 0x5197, PDF_CMAP_SINGLE, 6176 },
	{ 0x51a0, 0x51a0, PDF_CMAP_SINGLE, 3819 },
	{ 0x51a5, 0x51a5, PDF_CMAP_SINGLE, 4748 },
	{ 0x51aa, 0x51aa, PDF_CMAP_SINGLE, 4733 },
	{ 0x51ac, 0x51ac, PDF_CMAP_SINGLE, 4332 },
	{ 0x51b6, 0x51b7, PDF_CMAP_TABLE, 513 },
	{ 0x51bd, 0x51bd, PDF_CMAP_SINGLE, 4477 },
	{ 0x51c4, 0x51c4, PDF_CMAP_SINGLE, 7130 },
	{ 0x51c6, 0x51c6, PDF_CMAP_SINGLE, 6904 },
	{ 0x51c9, 0x51c9, PDF_CMAP_SINGLE, 4430 },
	{ 0x51cb, 0x51cd, PDF_CMAP_TABLE, 515 },
	{ 0x51dc, 0x51de, PDF_CMAP_TABLE, 518 },
	{ 0x51e1, 0x51e1, PDF_CMAP_SINGLE, 5009 },
	{ 0x51f0, 0x51f1, PDF_CMAP_TABLE, 521 },
	{ 0x51f6, 0x51f6, PDF_CMAP_SINGLE, 8014 },
	{ 0x51f8, 0x51fa, PDF_CMAP_TABLE, 523 },
	{ 0x51fd, 0x51fd, PDF_CMAP_SINGLE, 7651 },
	{ 0x5200, 0x5200, PDF_CMAP_SINGLE, 4270 },
	{ 0x5203, 0x5203, PDF_CMAP_SINGLE, 6445 },
	{ 0x5206, 0x5208, PDF_CMAP_TABLE, 526 },
	{ 0x520a, 0x520a, PDF_CMAP_SINGLE, 3477 },
	{ 0x520e, 0x520e, PDF_CMAP_SINGLE, 4837 },
	{ 0x5211, 0x5211, PDF_CMAP_SINGLE, 7773 },
	{ 0x5217, 0x5217, PDF_CMAP_SINGLE, 4478 },
	{ 0x521d, 0x521d, PDF_CMAP_SINGLE, 7214 },
	{ 0x5224, 0x5225, PDF_CMAP_TABLE, 529 },
	{ 0x5229, 0x522a, PDF_CMAP_TABLE, 531 },
	{ 0x522e, 0x522e, PDF_CMAP_SINGLE, 3835 },
	{ 0x5230, 0x5230, PDF_CMAP_SINGLE, 4271 },
	{ 0x5236, 0x523b, PDF_CMAP_TABLE, 533 },
	{ 0x5243, 0x5243, PDF_CMAP_SINGLE, 7205 },
	{ 0x5247, 0x5247, PDF_CMAP_SINGLE, 7360 },
	{ 0x524a, 0x524d, PDF_CMAP_TABLE, 539 },
	{ 0x5254, 0x5254, PDF_CMAP_SINGLE, 7135 },
	{ 0x5256, 0x5256, PDF_CMAP_SINGLE, 5113 },
	{ 0x525b, 0x525b, PDF_CMAP_SINGLE, 3536 },
	{ 0x525d, 0x525d, PDF_CMAP_SINGLE, 4886 },
	{ 0x5261, 0x5261, PDF_CMAP_SINGLE, 5463 },
	{ 0x5269, 0x526a, PDF_CMAP_TABLE, 543 },
	{ 0x526f, 0x526f, PDF_CMAP_SINGLE, 5114 },
	{ 0x5272, 0x5272, PDF_CMAP_SINGLE, 7649 },
	{ 0x5275, 0x5275, PDF_CMAP_SINGLE, 7094 },
	{ 0x527d, 0x527d, PDF_CMAP_SINGLE, 7577 },
	{ 0x527f, 0x527f, PDF_CMAP_SINGLE, 7215 },
	{ 0x5283, 0x5283, PDF_CMAP_SINGLE, 7948 },
	{ 0x5287, 0x5289, PDF_CMAP_TABLE, 545 },
	{ 0x528d, 0x528d, PDF_CMAP_SINGLE, 3618 },
	{ 0x5291, 0x5292, PDF_CMAP_TABLE, 548 },
	{ 0x529b, 0x529b, PDF_CMAP_SINGLE, 4458 },
	{ 0x529f, 0x52a0, PDF_CMAP_TABLE, 550 },
	{ 0x52a3, 0x52a4, PDF_CMAP_TABLE, 552 },
	{ 0x52a9, 0x52ab, PDF_CMAP_TABLE, 554 },
	{ 0x52be, 0x52be, PDF_CMAP_SINGLE, 7703 },
	{ 0x52c1, 0x52c1, PDF_CMAP_SINGLE, 3665 },
	{ 0x52c3, 0x52c3, PDF_CMAP_SINGLE, 4930 },
	{ 0x52c5, 0x52c5, PDF_CMAP_SINGLE, 7361 },
	{ 0x52c7, 0x52c7, PDF_CMAP_SINGLE, 6177 },
	{ 0x52c9, 0x52c9, PDF_CMAP_SINGLE, 4737 },
	{ 0x52cd, 0x52cd, PDF_CMAP_SINGLE, 3666 },
	{ 0x52d2, 0x52d2, PDF_CMAP_SINGLE, 4604 },
	{ 0x52d5, 0x52d6, PDF_CMAP_TABLE, 557 },
	{ 0x52d8, 0x52d9, PDF_CMAP_TABLE, 559 },
	{ 0x52db, 0x52db, PDF_CMAP_SINGLE, 7979 },
	{ 0x52dd, 0x52df, PDF_CMAP_TABLE, 561 },
	{ 0x52e2, 0x52e4, PDF_CMAP_TABLE, 564 },
	{ 0x52f3, 0x52f3, PDF_CMAP_SINGLE, 7980 },
	{ 0x52f5, 0x52f5, PDF_CMAP_SINGLE, 4442 },
	{ 0x52f8, 0x52f8, PDF_CMAP_SINGLE, 3971 },
	{ 0x52fa, 0x52fb, PDF_CMAP_TABLE, 567 },
	{ 0x52fe, 0x52ff, PDF_CMAP_TABLE, 569 },
	{ 0x5305, 0x5305, PDF_CMAP_SINGLE, 7545 },
	{ 0x5308, 0x5308, PDF_CMAP_SINGLE, 8015 },
	{ 0x530d, 0x530d, PDF_CMAP_SINGLE, 7546 },
	{ 0x530f, 0x5310, PDF_CMAP_TABLE, 571 },
	{ 0x5315, 0x5317, PDF_CMAP_TABLE, 573 },
	{ 0x5319, 0x5319, PDF_CMAP_SINGLE, 5693 },
	{ 0x5320, 0x5321, PDF_CMAP_TABLE, 576 },
	{ 0x5323, 0x5323, PDF_CMAP_SINGLE, 3530 },
	{ 0x532a, 0x532a, PDF_CMAP_SINGLE, 5184 },
	{ 0x532f, 0x532f, PDF_CMAP_SINGLE, 7928 },
	{ 0x5339, 0x5339, PDF_CMAP_SINGLE, 7604 },
	{ 0x533f, 0x5341, PDF_CMAP_TABLE, 578 },
	{ 0x5343, 0x5344, PDF_CMAP_TABLE, 581 },
	{ 0x5347, 0x534a, PDF_CMAP_TABLE, 583 },
	{ 0x534d, 0x534d, PDF_CMAP_SINGLE, 4671 },
	{ 0x5351, 0x5354, PDF_CMAP_TABLE, 587 },
	{ 0x5357, 0x5357, PDF_CMAP_SINGLE, 4145 },
	{ 0x535a, 0x535a, PDF_CMAP_SINGLE, 4887 },
	{ 0x535c, 0x535c, PDF_CMAP_SINGLE, 5077 },
	{ 0x535e, 0x535e, PDF_CMAP_SINGLE, 5031 },
	{ 0x5360, 0x5360, PDF_CMAP_SINGLE, 6692 },
	{ 0x5366, 0x5366, PDF_CMAP_SINGLE, 3852 },
	{ 0x5368, 0x5368, PDF_CMAP_SINGLE, 5450 },
	{ 0x536f, 0x5371, PDF_CMAP_TABLE, 591 },
	{ 0x5374, 0x5375, PDF_CMAP_TABLE, 594 },
	{ 0x5377, 0x5377, PDF_CMAP_SINGLE, 3972 },
	{ 0x537d, 0x537d, PDF_CMAP_SINGLE, 6926 },
	{ 0x537f, 0x537f, PDF_CMAP_SINGLE, 3667 },
	{ 0x5384, 0x5384, PDF_CMAP_SINGLE, 5851 },
	{ 0x5393, 0x5393, PDF_CMAP_SINGLE, 5840 },
	{ 0x5398, 0x5398, PDF_CMAP_SINGLE, 4615 },
	{ 0x539a, 0x539a, PDF_CMAP_SINGLE, 7968 },
	{ 0x539f, 0x53a0, PDF_CMAP_TABLE, 596 },
	{ 0x53a5, 0x53a6, PDF_CMAP_TABLE, 598 },
	{ 0x53ad, 0x53ad, PDF_CMAP_SINGLE, 5989 },
	{ 0x53bb, 0x53bb, PDF_CMAP_SINGLE, 3586 },
	{ 0x53c3, 0x53c3, PDF_CMAP_SINGLE, 7083 },
	{ 0x53c8, 0x53cb, PDF_CMAP_TABLE, 600 },
	{ 0x53cd, 0x53cd, PDF_CMAP_SINGLE, 4907 },
	{ 0x53d4, 0x53d4, PDF_CMAP_SINGLE, 5628 },
	{ 0x53d6, 0x53d7, PDF_CMAP_TABLE, 604 },
	{ 0x53db, 0x53db, PDF_CMAP_SINGLE, 4908 },
	{ 0x53e1, 0x53e5, PDF_CMAP_TABLE, 606 },
	{ 0x53e9, 0x53ed, PDF_CMAP_TABLE, 611 },
	{ 0x53ef, 0x53f3, PDF_CMAP_TABLE, 616 },
	{ 0x53f8, 0x53f8, PDF_CMAP_SINGLE, 5251 },
	{ 0x5403, 0x5404, PDF_CMAP_TABLE, 621 },
	{ 0x5408, 0x540a, PDF_CMAP_TABLE, 623 },
	{ 0x540c, 0x5411, PDF_CMAP_TABLE, 626 },
	{ 0x541b, 0x541b, PDF_CMAP_SINGLE, 3953 },
	{ 0x541d, 0x541d, PDF_CMAP_SINGLE, 4638 },
	{ 0x541f, 0x5420, PDF_CMAP_TABLE, 632 },
	{ 0x5426, 0x5426, PDF_CMAP_SINGLE, 5115 },
	{ 0x5429, 0x5429, PDF_CMAP_SINGLE, 5153 },
	{ 0x542b, 0x542b, PDF_CMAP_SINGLE, 7652 },
	{ 0x5433, 0x5433, PDF_CMAP_SINGLE, 6055 },
	{ 0x5438, 0x5439, PDF_CMAP_TABLE, 634 },
	{ 0x543b, 0x543c, PDF_CMAP_TABLE, 636 },
	{ 0x543e, 0x543e, PDF_CMAP_SINGLE, 6054 },
	{ 0x5442, 0x5442, PDF_CMAP_SINGLE, 4443 },
	{ 0x5448, 0x5448, PDF_CMAP_SINGLE, 6709 },
	{ 0x544a, 0x544a, PDF_CMAP_SINGLE, 3731 },
	{ 0x5451, 0x5451, PDF_CMAP_SINGLE, 7408 },
	{ 0x5468, 0x5468, PDF_CMAP_SINGLE, 6867 },
	{ 0x546a, 0x546a, PDF_CMAP_SINGLE, 6866 },
	{ 0x5471, 0x5471, PDF_CMAP_SINGLE, 3732 },
	{ 0x5473, 0x5473, PDF_CMAP_SINGLE, 4851 },
	{ 0x5475, 0x5475, PDF_CMAP_SINGLE, 3442 },
	{ 0x547b, 0x547d, PDF_CMAP_TABLE, 638 },
	{ 0x5480, 0x5480, PDF_CMAP_SINGLE, 6594 },
	{ 0x5486, 0x5486, PDF_CMAP_SINGLE, 7548 },
	{ 0x548c, 0x548c, PDF_CMAP_SINGLE, 7864 },
	{ 0x548e, 0x548e, PDF_CMAP_SINGLE, 3903 },
	{ 0x5490, 0x5490, PDF_CMAP_SINGLE, 5116 },
	{ 0x54a4, 0x54a4, PDF_CMAP_SINGLE, 7381 },
	{ 0x54a8, 0x54a8, PDF_CMAP_SINGLE, 6485 },
	{ 0x54ab, 0x54ac, PDF_CMAP_TABLE, 641 },
	{ 0x54b3, 0x54b3, PDF_CMAP_SINGLE, 7687 },
	{ 0x54b8, 0x54b8, PDF_CMAP_SINGLE, 7653 },
	{ 0x54bd, 0x54bd, PDF_CMAP_SINGLE, 6447 },
	{ 0x54c0, 0x54c1, PDF_CMAP_TABLE, 643 },
	{ 0x54c4, 0x54c4, PDF_CMAP_SINGLE, 7853 },
	{ 0x54c8, 0x54c9, PDF_CMAP_TABLE, 645 },
	{ 0x54e1, 0x54e1, PDF_CMAP_SINGLE, 6257 },
	{ 0x54e5, 0x54e5, PDF_CMAP_SINGLE, 3443 },
	{ 0x54e8, 0x54e8, PDF_CMAP_SINGLE, 7216 },
	{ 0x54ed, 0x54ee, PDF_CMAP_TABLE, 647 },
	{ 0x54f2, 0x54f2, PDF_CMAP_SINGLE, 7168 },
	{ 0x54fa, 0x54fa, PDF_CMAP_SINGLE, 7549 },
	{ 0x5504, 0x5504, PDF_CMAP_SINGLE, 7503 },
	{ 0x5506, 0x5507, PDF_CMAP_TABLE, 649 },
	{ 0x550e, 0x550e, PDF_CMAP_SINGLE, 4617 },
	{ 0x5510, 0x5510, PDF_CMAP_SINGLE, 4239 },
	{ 0x551c, 0x551c, PDF_CMAP_SINGLE, 4689 },
	{ 0x552f, 0x552f, PDF_CMAP_SINGLE, 6313 },
	{ 0x5531, 0x5531, PDF_CMAP_SINGLE, 7095 },
	{ 0x5535, 0x5535, PDF_CMAP_SINGLE, 5821 },
	{ 0x553e, 0x553e, PDF_CMAP_SINGLE, 7382 },
	{ 0x5544, 0x5544, PDF_CMAP_SINGLE, 7396 },
	{ 0x5546, 0x5546, PDF_CMAP_SINGLE, 5335 },
	{ 0x554f, 0x554f, PDF_CMAP_SINGLE, 4839 },
	{ 0x5553, 0x5553, PDF_CMAP_SINGLE, 3706 },
	{ 0x5556, 0x5556, PDF_CMAP_SINGLE, 4217 },
	{ 0x555e, 0x555e, PDF_CMAP_SINGLE, 5778 },
	{ 0x5563, 0x5563, PDF_CMAP_SINGLE, 7654 },
	{ 0x557c, 0x557c, PDF_CMAP_SINGLE, 6761 },
	{ 0x5580, 0x5580, PDF_CMAP_SINGLE, 3579 },
	{ 0x5584, 0x5584, PDF_CMAP_SINGLE, 5421 },
	{ 0x5586, 0x5587, PDF_CMAP_TABLE, 651 },
	{ 0x5589, 0x558a, PDF_CMAP_TABLE, 653 },
	{ 0x5598, 0x559a, PDF_CMAP_TABLE, 655 },
	{ 0x559c, 0x559d, PDF_CMAP_TABLE, 658 },
	{ 0x55a7, 0x55a7, PDF_CMAP_SINGLE, 7990 },
	{ 0x55a9, 0x55ac, PDF_CMAP_TABLE, 660 },
	{ 0x55ae, 0x55ae, PDF_CMAP_SINGLE, 4195 },
	{ 0x55c5, 0x55c5, PDF_CMAP_SINGLE, 7972 },
	{ 0x55c7, 0x55c7, PDF_CMAP_SINGLE, 5365 },
	{ 0x55d4, 0x55d4, PDF_CMAP_SINGLE, 6981 },
	{ 0x55da, 0x55da, PDF_CMAP_SINGLE, 6056 },
	{ 0x55dc, 0x55dc, PDF_CMAP_SINGLE, 4066 },
	{ 0x55df, 0x55df, PDF_CMAP_SINGLE, 7045 },
	{ 0x55e3, 0x55e4, PDF_CMAP_TABLE, 664 },
	{ 0x55fd, 0x55fe, PDF_CMAP_TABLE, 666 },
	{ 0x5606, 0x5606, PDF_CMAP_SINGLE, 7409 },
	{ 0x5609, 0x5609, PDF_CMAP_SINGLE, 3444 },
	{ 0x5614, 0x5614, PDF_CMAP_SINGLE, 3904 },
	{ 0x5617, 0x5617, PDF_CMAP_SINGLE, 5337 },
	{ 0x562f, 0x562f, PDF_CMAP_SINGLE, 5501 },
	{ 0x5632, 0x5632, PDF_CMAP_SINGLE, 6786 },
	{ 0x5634, 0x5634, PDF_CMAP_SINGLE, 7318 },
	{ 0x5636, 0x5636, PDF_CMAP_SINGLE, 5694 },
	{ 0x5653, 0x5653, PDF_CMAP_SINGLE, 7719 },
	{ 0x5668, 0x5668, PDF_CMAP_SINGLE, 4067 },
	{ 0x566b, 0x566b, PDF_CMAP_SINGLE, 8038 },
	{ 0x5674, 0x5674, PDF_CMAP_SINGLE, 5154 },
	{ 0x5686, 0x5686, PDF_CMAP_SINGLE, 7954 },
	{ 0x56a5, 0x56a5, PDF_CMAP_SINGLE, 5956 },
	{ 0x56ac, 0x56ac, PDF_CMAP_SINGLE, 5223 },
	{ 0x56ae, 0x56ae, PDF_CMAP_SINGLE, 7712 },
	{ 0x56b4, 0x56b4, PDF_CMAP_SINGLE, 5927 },
	{ 0x56bc, 0x56bc, PDF_CMAP_SINGLE, 6511 },
	{ 0x56ca, 0x56ca, PDF_CMAP_SINGLE, 4152 },
	{ 0x56cd, 0x56cd, PDF_CMAP_SINGLE, 8039 },
	{ 0x56d1, 0x56d1, PDF_CMAP_SINGLE, 7242 },
	{ 0x56da, 0x56db, PDF_CMAP_TABLE, 668 },
	{ 0x56de, 0x56de, PDF_CMAP_SINGLE, 7929 },
	{ 0x56e0, 0x56e0, PDF_CMAP_SINGLE, 6448 },
	{ 0x56f0, 0x56f0, PDF_CMAP_SINGLE, 3774 },
	{ 0x56f9, 0x56fa, PDF_CMAP_TABLE, 670 },
	{ 0x5703, 0x5704, PDF_CMAP_TABLE, 672 },
	{ 0x5708, 0x5708, PDF_CMAP_SINGLE, 3973 },
	{ 0x570b, 0x570b, PDF_CMAP_SINGLE, 3947 },
	{ 0x570d, 0x570d, PDF_CMAP_SINGLE, 6288 },
	{ 0x5712, 0x5713, PDF_CMAP_TABLE, 674 },
	{ 0x5716, 0x5716, PDF_CMAP_SINGLE, 4272 },
	{ 0x5718, 0x5718, PDF_CMAP_SINGLE, 4196 },
	{ 0x571f, 0x571f, PDF_CMAP_SINGLE, 7451 },
	{ 0x5728, 0x5728, PDF_CMAP_SINGLE, 6572 },
	{ 0x572d, 0x572d, PDF_CMAP_SINGLE, 3995 },
	{ 0x5730, 0x5730, PDF_CMAP_SINGLE, 6945 },
	{ 0x573b, 0x573b, PDF_CMAP_SINGLE, 4068 },
	{ 0x5740, 0x5740, PDF_CMAP_SINGLE, 6946 },
	{ 0x5742, 0x5742, PDF_CMAP_SINGLE, 7491 },
	{ 0x5747, 0x5747, PDF_CMAP_SINGLE, 4010 },
	{ 0x574a, 0x574a, PDF_CMAP_SINGLE, 4943 },
	{ 0x574d, 0x574e, PDF_CMAP_TABLE, 676 },
	{ 0x5750, 0x5751, PDF_CMAP_TABLE, 678 },
	{ 0x5761, 0x5761, PDF_CMAP_SINGLE, 7474 },
	{ 0x5764, 0x5764, PDF_CMAP_SINGLE, 3775 },
	{ 0x5766, 0x5766, PDF_CMAP_SINGLE, 7410 },
	{ 0x576a, 0x576a, PDF_CMAP_SINGLE, 7529 },
	{ 0x576e, 0x576e, PDF_CMAP_SINGLE, 4252 },
	{ 0x5770, 0x5770, PDF_CMAP_SINGLE, 3668 },
	{ 0x5775, 0x5775, PDF_CMAP_SINGLE, 3905 },
	{ 0x577c, 0x577c, PDF_CMAP_SINGLE, 7397 },
	{ 0x5782, 0x5782, PDF_CMAP_SINGLE, 5571 },
	{ 0x5788, 0x5788, PDF_CMAP_SINGLE, 4251 },
	{ 0x578b, 0x578b, PDF_CMAP_SINGLE, 7774 },
	{ 0x5793, 0x5793, PDF_CMAP_SINGLE, 7688 },
	{ 0x57a0, 0x57a0, PDF_CMAP_SINGLE, 6373 },
	{ 0x57a2, 0x57a3, PDF_CMAP_TABLE, 680 },
	{ 0x57c3, 0x57c3, PDF_CMAP_SINGLE, 5842 },
	{ 0x57c7, 0x57c8, PDF_CMAP_TABLE, 682 },
	{ 0x57cb, 0x57cb, PDF_CMAP_SINGLE, 4708 },
	{ 0x57ce, 0x57ce, PDF_CMAP_SINGLE, 5474 },
	{ 0x57df, 0x57e0, PDF_CMAP_TABLE, 684 },
	{ 0x57f0, 0x57f0, PDF_CMAP_SINGLE, 7115 },
	{ 0x57f4, 0x57f4, PDF_CMAP_SINGLE, 5720 },
	{ 0x57f7, 0x57f7, PDF_CMAP_SINGLE, 7032 },
	{ 0x57f9, 0x57fa, PDF_CMAP_TABLE, 686 },
	{ 0x57fc, 0x57fc, PDF_CMAP_SINGLE, 4070 },
	{ 0x5800, 0x5800, PDF_CMAP_SINGLE, 3959 },
	{ 0x5802, 0x5802, PDF_CMAP_SINGLE, 4240 },
	{ 0x5805, 0x5806, PDF_CMAP_TABLE, 688 },
	{ 0x5808, 0x580a, PDF_CMAP_TABLE, 690 },
	{ 0x581e, 0x581e, PDF_CMAP_SINGLE, 7187 },
	{ 0x5821, 0x5821, PDF_CMAP_SINGLE, 5059 },
	{ 0x5824, 0x5824, PDF_CMAP_SINGLE, 6762 },
	{ 0x5827, 0x5827, PDF_CMAP_SINGLE, 5957 },
	{ 0x582a, 0x582a, PDF_CMAP_SINGLE, 3512 },
	{ 0x582f, 0x5831, PDF_CMAP_TABLE, 693 },
	{ 0x5834, 0x5835, PDF_CMAP_TABLE, 696 },
	{ 0x583a, 0x583a, PDF_CMAP_SINGLE, 3707 },
	{ 0x584a, 0x584b, PDF_CMAP_TABLE, 698 },
	{ 0x584f, 0x584f, PDF_CMAP_SINGLE, 3564 },
	{ 0x5851, 0x5851, PDF_CMAP_SINGLE, 5502 },
	{ 0x5854, 0x5854, PDF_CMAP_SINGLE, 7424 },
	{ 0x5857, 0x5858, PDF_CMAP_TABLE, 700 },
	{ 0x585a, 0x585a, PDF_CMAP_SINGLE, 7252 },
	{ 0x585e, 0x585e, PDF_CMAP_SINGLE, 5362 },
	{ 0x5861, 0x5862, PDF_CMAP_TABLE, 702 },
	{ 0x5864, 0x5864, PDF_CMAP_SINGLE, 7981 },
	{ 0x5875, 0x5875, PDF_CMAP_SINGLE, 6982 },
	{ 0x5879, 0x5879, PDF_CMAP_SINGLE, 7084 },
	{ 0x587c, 0x587c, PDF_CMAP_SINGLE, 6651 },
	{ 0x587e, 0x587e, PDF_CMAP_SINGLE, 5629 },
	{ 0x5883, 0x5883, PDF_CMAP_SINGLE, 3669 },
	{ 0x5885, 0x5885, PDF_CMAP_SINGLE, 5373 },
	{ 0x5889, 0x5889, PDF_CMAP_SINGLE, 6179 },
	{ 0x5893, 0x5893, PDF_CMAP_SINGLE, 4801 },
	{ 0x589c, 0x589c, PDF_CMAP_SINGLE, 7265 },
	{ 0x589e, 0x589f, PDF_CMAP_TABLE, 704 },
	{ 0x58a8, 0x58a9, PDF_CMAP_TABLE, 706 },
	{ 0x58ae, 0x58ae, PDF_CMAP_SINGLE, 7383 },
	{ 0x58b3, 0x58b3, PDF_CMAP_SINGLE, 5155 },
	{ 0x58ba, 0x58bb, PDF_CMAP_TABLE, 708 },
	{ 0x58be, 0x58be, PDF_CMAP_SINGLE, 3478 },
	{ 0x58c1, 0x58c1, PDF_CMAP_SINGLE, 5022 },
	{ 0x58c5, 0x58c5, PDF_CMAP_SINGLE, 6090 },
	{ 0x58c7, 0x58c7, PDF_CMAP_SINGLE, 4197 },
	{ 0x58ce, 0x58ce, PDF_CMAP_SINGLE, 7982 },
	{ 0x58d1, 0x58d1, PDF_CMAP_SINGLE, 7630 },
	{ 0x58d3, 0x58d3, PDF_CMAP_SINGLE, 5829 },
	{ 0x58d5, 0x58d5, PDF_CMAP_SINGLE, 7803 },
	{ 0x58d8, 0x58d9, PDF_CMAP_TABLE, 710 },
	{ 0x58de, 0x58df, PDF_CMAP_TABLE, 712 },
	{ 0x58e4, 0x58e4, PDF_CMAP_SINGLE, 5880 },
	{ 0x58eb, 0x58ec, PDF_CMAP_TABLE, 714 },
	{ 0x58ef, 0x58ef, PDF_CMAP_SINGLE, 6539 },
	{ 0x58f9, 0x58fb, PDF_CMAP_TABLE, 716 },
	{ 0x58fd, 0x58fd, PDF_CMAP_SINGLE, 5572 },
	{ 0x590f, 0x590f, PDF_CMAP_SINGLE, 7619 },
	{ 0x5914, 0x5916, PDF_CMAP_TABLE, 719 },
	{ 0x5919, 0x591a, PDF_CMAP_TABLE, 722 },
	{ 0x591c, 0x591c, PDF_CMAP_SINGLE, 5865 },
	{ 0x5922, 0x5922, PDF_CMAP_SINGLE, 4797 },
	{ 0x5927, 0x5927, PDF_CMAP_SINGLE, 4253 },
	{ 0x5929, 0x592b, PDF_CMAP_TABLE, 724 },
	{ 0x592d, 0x592e, PDF_CMAP_TABLE, 727 },
	{ 0x5931, 0x5931, PDF_CMAP_SINGLE, 5758 },
	{ 0x5937, 0x5937, PDF_CMAP_SINGLE, 6416 },
	{ 0x593e, 0x593e, PDF_CMAP_SINGLE, 7761 },
	{ 0x5944, 0x5944, PDF_CMAP_SINGLE, 5928 },
	{ 0x5947, 0x5949, PDF_CMAP_TABLE, 729 },
	{ 0x594e, 0x5951, PDF_CMAP_TABLE, 732 },
	{ 0x5954, 0x5955, PDF_CMAP_TABLE, 736 },
	{ 0x5957, 0x5957, PDF_CMAP_SINGLE, 7466 },
	{ 0x595a, 0x595a, PDF_CMAP_SINGLE, 7689 },
	{ 0x5960, 0x5960, PDF_CMAP_SINGLE, 6652 },
	{ 0x5962, 0x5962, PDF_CMAP_SINGLE, 5256 },
	{ 0x5967, 0x5967, PDF_CMAP_SINGLE, 6059 },
	{ 0x596a, 0x596e, PDF_CMAP_TABLE, 738 },
	{ 0x5973, 0x5974, PDF_CMAP_TABLE, 743 },
	{ 0x5978, 0x5978, PDF_CMAP_SINGLE, 3479 },
	{ 0x597d, 0x597d, PDF_CMAP_SINGLE, 7805 },
	{ 0x5982, 0x5984, PDF_CMAP_TABLE, 745 },
	{ 0x598a, 0x598a, PDF_CMAP_SINGLE, 6471 },
	{ 0x5993, 0x5993, PDF_CMAP_SINGLE, 4073 },
	{ 0x5996, 0x5997, PDF_CMAP_TABLE, 748 },
	{ 0x5999, 0x5999, PDF_CMAP_SINGLE, 4802 },
	{ 0x59a5, 0x59a5, PDF_CMAP_SINGLE, 7384 },
	{ 0x59a8, 0x59a8, PDF_CMAP_SINGLE, 4944 },
	{ 0x59ac, 0x59ac, PDF_CMAP_SINGLE, 7467 },
	{ 0x59b9, 0x59b9, PDF_CMAP_SINGLE, 4709 },
	{ 0x59bb, 0x59bb, PDF_CMAP_SINGLE, 7131 },
	{ 0x59be, 0x59be, PDF_CMAP_SINGLE, 7188 },
	{ 0x59c3, 0x59c3, PDF_CMAP_SINGLE, 6710 },
	{ 0x59c6, 0x59c6, PDF_CMAP_SINGLE, 4767 },
	{ 0x59c9, 0x59c9, PDF_CMAP_SINGLE, 6486 },
	{ 0x59cb, 0x59cb, PDF_CMAP_SINGLE, 5695 },
	{ 0x59d0, 0x59d1, PDF_CMAP_TABLE, 750 },
	{ 0x59d3, 0x59d4, PDF_CMAP_TABLE, 752 },
	{ 0x59d9, 0x59da, PDF_CMAP_TABLE, 754 },
	{ 0x59dc, 0x59dd, PDF_CMAP_TABLE, 756 },
	{ 0x59e6, 0x59e6, PDF_CMAP_SINGLE, 3480 },
	{ 0x59e8, 0x59e8, PDF_CMAP_SINGLE, 6417 },
	{ 0x59ea, 0x59ea, PDF_CMAP_SINGLE, 7017 },
	{ 0x59ec, 0x59ec, PDF_CMAP_SINGLE, 8040 },
	{ 0x59ee, 0x59ee, PDF_CMAP_SINGLE, 7672 },
	{ 0x59f8, 0x59f8, PDF_CMAP_SINGLE, 5958 },
	{ 0x59fb, 0x59fb, PDF_CMAP_SINGLE, 6449 },
	{ 0x59ff, 0x59ff, PDF_CMAP_SINGLE, 6487 },
	{ 0x5a01, 0x5a01, PDF_CMAP_SINGLE, 6290 },
	{ 0x5a03, 0x5a03, PDF_CMAP_SINGLE, 6132 },
	{ 0x5a11, 0x5a11, PDF_CMAP_SINGLE, 5257 },
	{ 0x5a18, 0x5a18, PDF_CMAP_SINGLE, 4153 },
	{ 0x5a1b, 0x5a1c, PDF_CMAP_TABLE, 758 },
	{ 0x5a1f, 0x5a20, PDF_CMAP_TABLE, 760 },
	{ 0x5a25, 0x5a25, PDF_CMAP_SINGLE, 5779 },
	{ 0x5a29, 0x5a29, PDF_CMAP_SINGLE, 4672 },
	{ 0x5a36, 0x5a36, PDF_CMAP_SINGLE, 7319 },
	{ 0x5a3c, 0x5a3c, PDF_CMAP_SINGLE, 7096 },
	{ 0x5a41, 0x5a41, PDF_CMAP_SINGLE, 4565 },
	{ 0x5a46, 0x5a46, PDF_CMAP_SINGLE, 7475 },
	{ 0x5a49, 0x5a49, PDF_CMAP_SINGLE, 6107 },
	{ 0x5a5a, 0x5a5a, PDF_CMAP_SINGLE, 7844 },
	{ 0x5a62, 0x5a62, PDF_CMAP_SINGLE, 5187 },
	{ 0x5a66, 0x5a66, PDF_CMAP_SINGLE, 5119 },
	{ 0x5a92, 0x5a92, PDF_CMAP_SINGLE, 4710 },
	{ 0x5a9a, 0x5a9b, PDF_CMAP_TABLE, 762 },
	{ 0x5aa4, 0x5aa4, PDF_CMAP_SINGLE, 5696 },
	{ 0x5ac1, 0x5ac2, PDF_CMAP_TABLE, 764 },
	{ 0x5ac4, 0x5ac4, PDF_CMAP_SINGLE, 6262 },
	{ 0x5ac9, 0x5ac9, PDF_CMAP_SINGLE, 7018 },
	{ 0x5acc, 0x5acc, PDF_CMAP_SINGLE, 7758 },
	{ 0x5ae1, 0x5ae1, PDF_CMAP_SINGLE, 6621 },
	{ 0x5ae6, 0x5ae6, PDF_CMAP_SINGLE, 7673 },
	{ 0x5ae9, 0x5ae9, PDF_CMAP_SINGLE, 4181 },
	{ 0x5b05, 0x5b05, PDF_CMAP_SINGLE, 7865 },
	{ 0x5b09, 0x5b09, PDF_CMAP_SINGLE, 8041 },
	{ 0x5b0b, 0x5b0c, PDF_CMAP_TABLE, 766 },
	{ 0x5b16, 0x5b16, PDF_CMAP_SINGLE, 7535 },
	{ 0x5b2a, 0x5b2a, PDF_CMAP_SINGLE, 5224 },
	{ 0x5b40, 0x5b40, PDF_CMAP_SINGLE, 5338 },
	{ 0x5b43, 0x5b43, PDF_CMAP_SINGLE, 5881 },
	{ 0x5b50, 0x5b51, PDF_CMAP_TABLE, 768 },
	{ 0x5b54, 0x5b55, PDF_CMAP_TABLE, 770 },
	{ 0x5b57, 0x5b58, PDF_CMAP_TABLE, 772 },
	{ 0x5b5a, 0x5b5a, PDF_CMAP_SINGLE, 5120 },
	{ 0x5b5c, 0x5b5d, PDF_CMAP_TABLE, 774 },
	{ 0x5b5f, 0x5b5f, PDF_CMAP_SINGLE, 4727 },
	{ 0x5b63, 0x5b64, PDF_CMAP_TABLE, 776 },
	{ 0x5b69, 0x5b69, PDF_CMAP_SINGLE, 7690 },
	{ 0x5b6b, 0x5b6b, PDF_CMAP_SINGLE, 5546 },
	{ 0x5b70, 0x5b71, PDF_CMAP_TABLE, 778 },
	{ 0x5b75, 0x5b75, PDF_CMAP_SINGLE, 5121 },
	{ 0x5b78, 0x5b78, PDF_CMAP_SINGLE, 7631 },
	{ 0x5b7a, 0x5b7a, PDF_CMAP_SINGLE, 6315 },
	{ 0x5b7c, 0x5b7c, PDF_CMAP_SINGLE, 5923 },
	{ 0x5b85, 0x5b85, PDF_CMAP_SINGLE, 4266 },
	{ 0x5b87, 0x5b89, PDF_CMAP_TABLE, 780 },
	{ 0x5b8b, 0x5b8c, PDF_CMAP_TABLE, 783 },
	{ 0x5b8f, 0x5b8f, PDF_CMAP_SINGLE, 3864 },
	{ 0x5b93, 0x5b93, PDF_CMAP_SINGLE, 5078 },
	{ 0x5b95, 0x5b9c, PDF_CMAP_TABLE, 785 },
	{ 0x5ba2, 0x5ba6, PDF_CMAP_TABLE, 793 },
	{ 0x5bac, 0x5bac, PDF_CMAP_SINGLE, 5476 },
	{ 0x5bae, 0x5bae, PDF_CMAP_SINGLE, 3963 },
	{ 0x5bb0, 0x5bb0, PDF_CMAP_SINGLE, 6573 },
	{ 0x5bb3, 0x5bb6, PDF_CMAP_TABLE, 798 },
	{ 0x5bb8, 0x5bb9, PDF_CMAP_TABLE, 802 },
	{ 0x5bbf, 0x5bc0, PDF_CMAP_TABLE, 804 },
	{ 0x5bc2, 0x5bc7, PDF_CMAP_TABLE, 806 },
	{ 0x5bcc, 0x5bcc, PDF_CMAP_SINGLE, 5122 },
	{ 0x5bd0, 0x5bd0, PDF_CMAP_SINGLE, 4711 },
	{ 0x5bd2, 0x5bd4, PDF_CMAP_TABLE, 812 },
	{ 0x5bd7, 0x5bd7, PDF_CMAP_SINGLE, 4168 },
	{ 0x5bde, 0x5bdf, PDF_CMAP_TABLE, 815 },
	{ 0x5be1, 0x5be2, PDF_CMAP_TABLE, 817 },
	{ 0x5be4, 0x5be9, PDF_CMAP_TABLE, 819 },
	{ 0x5beb, 0x5bec, PDF_CMAP_TABLE, 825 },
	{ 0x5bee, 0x5bef, PDF_CMAP_TABLE, 827 },
	{ 0x5bf5, 0x5bf6, PDF_CMAP_TABLE, 829 },
	{ 0x5bf8, 0x5bf8, PDF_CMAP_SINGLE, 7247 },
	{ 0x5bfa, 0x5bfa, PDF_CMAP_SINGLE, 5259 },
	{ 0x5c01, 0x5c01, PDF_CMAP_SINGLE, 5095 },
	{ 0x5c04, 0x5c04, PDF_CMAP_SINGLE, 5260 },
	{ 0x5c07, 0x5c0b, PDF_CMAP_TABLE, 831 },
	{ 0x5c0d, 0x5c0f, PDF_CMAP_TABLE, 836 },
	{ 0x5c11, 0x5c11, PDF_CMAP_SINGLE, 5505 },
	{ 0x5c16, 0x5c16, PDF_CMAP_SINGLE, 7178 },
	{ 0x5c19, 0x5c19, PDF_CMAP_SINGLE, 5339 },
	{ 0x5c24, 0x5c24, PDF_CMAP_SINGLE, 6206 },
	{ 0x5c28, 0x5c28, PDF_CMAP_SINGLE, 4945 },
	{ 0x5c31, 0x5c31, PDF_CMAP_SINGLE, 7320 },
	{ 0x5c38, 0x5c3c, PDF_CMAP_TABLE, 839 },
	{ 0x5c3e, 0x5c40, PDF_CMAP_TABLE, 844 },
	{ 0x5c45, 0x5c46, PDF_CMAP_TABLE, 847 },
	{ 0x5c48, 0x5c48, PDF_CMAP_SINGLE, 3960 },
	{ 0x5c4b, 0x5c4b, PDF_CMAP_SINGLE, 6078 },
	{ 0x5c4d, 0x5c4e, PDF_CMAP_TABLE, 849 },
	{ 0x5c51, 0x5c51, PDF_CMAP_SINGLE, 5451 },
	{ 0x5c55, 0x5c55, PDF_CMAP_SINGLE, 6654 },
	{ 0x5c5b, 0x5c5b, PDF_CMAP_SINGLE, 5044 },
	{ 0x5c60, 0x5c60, PDF_CMAP_SINGLE, 4276 },
	{ 0x5c62, 0x5c62, PDF_CMAP_SINGLE, 4566 },
	{ 0x5c64, 0x5c65, PDF_CMAP_TABLE, 851 },
	{ 0x5c6c, 0x5c6c, PDF_CMAP_SINGLE, 5538 },
	{ 0x5c6f, 0x5c6f, PDF_CMAP_SINGLE, 4358 },
	{ 0x5c71, 0x5c71, PDF_CMAP_SINGLE, 5305 },
	{ 0x5c79, 0x5c79, PDF_CMAP_SINGLE, 8024 },
	{ 0x5c90, 0x5c91, PDF_CMAP_TABLE, 853 },
	{ 0x5ca1, 0x5ca1, PDF_CMAP_SINGLE, 3539 },
	{ 0x5ca9, 0x5ca9, PDF_CMAP_SINGLE, 5822 },
	{ 0x5cab, 0x5cac, PDF_CMAP_TABLE, 855 },
	{ 0x5cb1, 0x5cb1, PDF_CMAP_SINGLE, 4255 },
	{ 0x5cb3, 0x5cb3, PDF_CMAP_SINGLE, 5794 },
	{ 0x5cb5, 0x5cb5, PDF_CMAP_SINGLE, 7806 },
	{ 0x5cb7, 0x5cb8, PDF_CMAP_TABLE, 857 },
	{ 0x5cba, 0x5cba, PDF_CMAP_SINGLE, 4492 },
	{ 0x5cbe, 0x5cbe, PDF_CMAP_SINGLE, 6693 },
	{ 0x5cc0, 0x5cc0, PDF_CMAP_SINGLE, 5576 },
	{ 0x5cd9, 0x5cd9, PDF_CMAP_SINGLE, 7339 },
	{ 0x5ce0, 0x5ce0, PDF_CMAP_SINGLE, 5340 },
	{ 0x5ce8, 0x5ce8, PDF_CMAP_SINGLE, 5780 },
	{ 0x5cef, 0x5cf0, PDF_CMAP_RANGE, 5096 },
	{ 0x5cf4, 0x5cf4, PDF_CMAP_SINGLE, 7735 },
	{ 0x5cf6, 0x5cf6, PDF_CMAP_SINGLE, 4277 },
	{ 0x5cfb, 0x5cfb, PDF_CMAP_SINGLE, 6907 },
	{ 0x5cfd, 0x5cfd, PDF_CMAP_SINGLE, 7762 },
	{ 0x5d07, 0x5d07, PDF_CMAP_SINGLE, 5671 },
	{ 0x5d0d, 0x5d0e, PDF_CMAP_TABLE, 859 },
	{ 0x5d11, 0x5d11, PDF_CMAP_SINGLE, 3776 },
	{ 0x5d14, 0x5d14, PDF_CMAP_SINGLE, 7263 },
	{ 0x5d16, 0x5d17, PDF_CMAP_TABLE, 861 },
	{ 0x5d19, 0x5d19, PDF_CMAP_SINGLE, 4596 },
	{ 0x5d27, 0x5d27, PDF_CMAP_SINGLE, 5672 },
	{ 0x5d29, 0x5d29, PDF_CMAP_SINGLE, 5175 },
	{ 0x5d4b, 0x5d4c, PDF_CMAP_TABLE, 863 },
	{ 0x5d50, 0x5d50, PDF_CMAP_SINGLE, 4399 },
	{ 0x5d69, 0x5d69, PDF_CMAP_SINGLE, 5673 },
	{ 0x5d6c, 0x5d6c, PDF_CMAP_SINGLE, 6136 },
	{ 0x5d6f, 0x5d6f, PDF_CMAP_SINGLE, 7046 },
	{ 0x5d87, 0x5d87, PDF_CMAP_SINGLE, 3908 },
	{ 0x5d8b, 0x5d8b, PDF_CMAP_SINGLE, 4278 },
	{ 0x5d9d, 0x5d9d, PDF_CMAP_SINGLE, 4365 },
	{ 0x5da0, 0x5da0, PDF_CMAP_SINGLE, 3873 },
	{ 0x5da2, 0x5da2, PDF_CMAP_SINGLE, 6147 },
	{ 0x5daa, 0x5daa, PDF_CMAP_SINGLE, 5931 },
	{ 0x5db8, 0x5db8, PDF_CMAP_SINGLE, 6003 },
	{ 0x5dba, 0x5dba, PDF_CMAP_SINGLE, 4493 },
	{ 0x5dbc, 0x5dbd, PDF_CMAP_TABLE, 865 },
	{ 0x5dcd, 0x5dcd, PDF_CMAP_SINGLE, 6137 },
	{ 0x5dd2, 0x5dd2, PDF_CMAP_SINGLE, 4673 },
	{ 0x5dd6, 0x5dd6, PDF_CMAP_SINGLE, 5823 },
	{ 0x5ddd, 0x5dde, PDF_CMAP_TABLE, 867 },
	{ 0x5de1, 0x5de2, PDF_CMAP_TABLE, 869 },
	{ 0x5de5, 0x5de8, PDF_CMAP_TABLE, 871 },
	{ 0x5deb, 0x5deb, PDF_CMAP_SINGLE, 4813 },
	{ 0x5dee, 0x5dee, PDF_CMAP_SINGLE, 7047 },
	{ 0x5df1, 0x5df4, PDF_CMAP_TABLE, 875 },
	{ 0x5df7, 0x5df7, PDF_CMAP_SINGLE, 7674 },
	{ 0x5dfd, 0x5dfe, PDF_CMAP_TABLE, 879 },
	{ 0x5e02, 0x5e03, PDF_CMAP_TABLE, 881 },
	{ 0x5e06, 0x5e06, PDF_CMAP_SINGLE, 5010 },
	{ 0x5e0c, 0x5e0c, PDF_CMAP_SINGLE, 8042 },
	{ 0x5e11, 0x5e11, PDF_CMAP_SINGLE, 7428 },
	{ 0x5e16, 0x5e16, PDF_CMAP_SINGLE, 7189 },
	{ 0x5e19, 0x5e19, PDF_CMAP_SINGLE, 7019 },
	{ 0x5e1b, 0x5e1b, PDF_CMAP_SINGLE, 4990 },
	{ 0x5e1d, 0x5e1d, PDF_CMAP_SINGLE, 6763 },
	{ 0x5e25, 0x5e25, PDF_CMAP_SINGLE, 5577 },
	{ 0x5e2b, 0x5e2b, PDF_CMAP_SINGLE, 5262 },
	{ 0x5e2d, 0x5e2d, PDF_CMAP_SINGLE, 5405 },
	{ 0x5e33, 0x5e33, PDF_CMAP_SINGLE, 6542 },
	{ 0x5e36, 0x5e36, PDF_CMAP_SINGLE, 4256 },
	{ 0x5e38, 0x5e38, PDF_CMAP_SINGLE, 5341 },
	{ 0x5e3d, 0x5e3d, PDF_CMAP_SINGLE, 4768 },
	{ 0x5e3f, 0x5e40, PDF_CMAP_TABLE, 883 },
	{ 0x5e44, 0x5e45, PDF_CMAP_TABLE, 885 },
	{ 0x5e47, 0x5e47, PDF_CMAP_SINGLE, 4946 },
	{ 0x5e4c, 0x5e4c, PDF_CMAP_SINGLE, 7905 },
	{ 0x5e55, 0x5e55, PDF_CMAP_SINGLE, 4665 },
	{ 0x5e5f, 0x5e5f, PDF_CMAP_SINGLE, 7340 },
	{ 0x5e61, 0x5e63, PDF_CMAP_TABLE, 887 },
	{ 0x5e72, 0x5e74, PDF_CMAP_TABLE, 890 },
	{ 0x5e77, 0x5e79, PDF_CMAP_TABLE, 893 },
	{ 0x5e7b, 0x5e7b, PDF_CMAP_SINGLE, 7886 },
	{ 0x5e7c, 0x5e7d, PDF_CMAP_RANGE, 6317 },
	{ 0x5e7e, 0x5e7e, PDF_CMAP_SINGLE, 4078 },
	{ 0x5e84, 0x5e84, PDF_CMAP_SINGLE, 6543 },
	{ 0x5e87, 0x5e87, PDF_CMAP_SINGLE, 5188 },
	{ 0x5e8a, 0x5e8a, PDF_CMAP_SINGLE, 5342 },
	{ 0x5e8f, 0x5e8f, PDF_CMAP_SINGLE, 5376 },
	{ 0x5e95, 0x5e95, PDF_CMAP_SINGLE, 6596 },
	{ 0x5e97, 0x5e97, PDF_CMAP_SINGLE, 6694 },
	{ 0x5e9a, 0x5e9a, PDF_CMAP_SINGLE, 3670 },
	{ 0x5e9c, 0x5e9c, PDF_CMAP_SINGLE, 5123 },
	{ 0x5ea0, 0x5ea0, PDF_CMAP_SINGLE, 5343 },
	{ 0x5ea6, 0x5ea7, PDF_CMAP_TABLE, 896 },
	{ 0x5eab, 0x5eab, PDF_CMAP_SINGLE, 3737 },
	{ 0x5ead, 0x5ead, PDF_CMAP_SINGLE, 6713 },
	{ 0x5eb5, 0x5eb8, PDF_CMAP_TABLE, 898 },
	{ 0x5ebe, 0x5ebe, PDF_CMAP_SINGLE, 6319 },
	{ 0x5ec2, 0x5ec2, PDF_CMAP_SINGLE, 5344 },
	{ 0x5ec8, 0x5eca, PDF_CMAP_TABLE, 902 },
	{ 0x5ed0, 0x5ed0, PDF_CMAP_SINGLE, 3909 },
	{ 0x5ed3, 0x5ed3, PDF_CMAP_SINGLE, 3815 },
	{ 0x5ed6, 0x5ed6, PDF_CMAP_SINGLE, 4554 },
	{ 0x5eda, 0x5edb, PDF_CMAP_TABLE, 905 },
	{ 0x5edf, 0x5ee0, PDF_CMAP_TABLE, 907 },
	{ 0x5ee2, 0x5ee3, PDF_CMAP_TABLE, 909 },
	{ 0x5eec, 0x5eec, PDF_CMAP_SINGLE, 4444 },
	{ 0x5ef3, 0x5ef3, PDF_CMAP_SINGLE, 7197 },
	{ 0x5ef6, 0x5ef7, PDF_CMAP_TABLE, 911 },
	{ 0x5efa, 0x5efb, PDF_CMAP_TABLE, 913 },
	{ 0x5f01, 0x5f01, PDF_CMAP_SINGLE, 5032 },
	{ 0x5f04, 0x5f04, PDF_CMAP_SINGLE, 4537 },
	{ 0x5f0a, 0x5f0a, PDF_CMAP_SINGLE, 7538 },
	{ 0x5f0f, 0x5f0f, PDF_CMAP_SINGLE, 5722 },
	{ 0x5f11, 0x5f11, PDF_CMAP_SINGLE, 5701 },
	{ 0x5f13, 0x5f15, PDF_CMAP_TABLE, 915 },
	{ 0x5f17, 0x5f18, PDF_CMAP_TABLE, 918 },
	{ 0x5f1b, 0x5f1b, PDF_CMAP_SINGLE, 6419 },
	{ 0x5f1f, 0x5f1f, PDF_CMAP_SINGLE, 6764 },
	{ 0x5f26, 0x5f27, PDF_CMAP_TABLE, 920 },
	{ 0x5f29, 0x5f29, PDF_CMAP_SINGLE, 4171 },
	{ 0x5f31, 0x5f31, PDF_CMAP_SINGLE, 5872 },
	{ 0x5f35, 0x5f35, PDF_CMAP_SINGLE, 6544 },
	{ 0x5f3a, 0x5f3a, PDF_CMAP_SINGLE, 3542 },
	{ 0x5f3c, 0x5f3c, PDF_CMAP_SINGLE, 7605 },
	{ 0x5f48, 0x5f48, PDF_CMAP_SINGLE, 7411 },
	{ 0x5f4a, 0x5f4a, PDF_CMAP_SINGLE, 3543 },
	{ 0x5f4c, 0x5f4c, PDF_CMAP_SINGLE, 4855 },
	{ 0x5f4e, 0x5f4e, PDF_CMAP_SINGLE, 4674 },
	{ 0x5f56, 0x5f57, PDF_CMAP_TABLE, 922 },
	{ 0x5f59, 0x5f59, PDF_CMAP_SINGLE, 7997 },
	{ 0x5f5b, 0x5f5b, PDF_CMAP_SINGLE, 6420 },
	{ 0x5f62, 0x5f62, PDF_CMAP_SINGLE, 7775 },
	{ 0x5f66, 0x5f67, PDF_CMAP_TABLE, 924 },
	{ 0x5f69, 0x5f6d, PDF_CMAP_TABLE, 926 },
	{ 0x5f70, 0x5f71, PDF_CMAP_TABLE, 931 },
	{ 0x5f77, 0x5f77, PDF_CMAP_SINGLE, 4947 },
	{ 0x5f79, 0x5f79, PDF_CMAP_SINGLE, 5949 },
	{ 0x5f7c, 0x5f7c, PDF_CMAP_SINGLE, 7597 },
	{ 0x5f7f, 0x5f81, PDF_CMAP_TABLE, 933 },
	{ 0x5f85, 0x5f85, PDF_CMAP_SINGLE, 4257 },
	{ 0x5f87, 0x5f87, PDF_CMAP_SINGLE, 5641 },
	{ 0x5f8a, 0x5f8c, PDF_CMAP_TABLE, 936 },
	{ 0x5f90, 0x5f92, PDF_CMAP_TABLE, 939 },
	{ 0x5f97, 0x5f99, PDF_CMAP_TABLE, 942 },
	{ 0x5f9e, 0x5f9e, PDF_CMAP_SINGLE, 6839 },
	{ 0x5fa0, 0x5fa1, PDF_CMAP_TABLE, 945 },
	{ 0x5fa8, 0x5faa, PDF_CMAP_TABLE, 947 },
	{ 0x5fae, 0x5fae, PDF_CMAP_SINGLE, 4856 },
	{ 0x5fb5, 0x5fb5, PDF_CMAP_SINGLE, 7038 },
	{ 0x5fb7, 0x5fb7, PDF_CMAP_SINGLE, 4267 },
	{ 0x5fb9, 0x5fb9, PDF_CMAP_SINGLE, 7170 },
	{ 0x5fbd, 0x5fbd, PDF_CMAP_SINGLE, 7998 },
	{ 0x5fc3, 0x5fc3, PDF_CMAP_SINGLE, 5764 },
	{ 0x5fc5, 0x5fc5, PDF_CMAP_SINGLE, 7606 },
	{ 0x5fcc, 0x5fcd, PDF_CMAP_TABLE, 950 },
	{ 0x5fd6, 0x5fd7, PDF_CMAP_TABLE, 952 },
	{ 0x5fd8, 0x5fd9, PDF_CMAP_RANGE, 4698 },
	{ 0x5fe0, 0x5fe0, PDF_CMAP_SINGLE, 7307 },
	{ 0x5feb, 0x5feb, PDF_CMAP_SINGLE, 7379 },
	{ 0x5ff5, 0x5ff5, PDF_CMAP_SINGLE, 4163 },
	{ 0x5ffd, 0x5ffd, PDF_CMAP_SINGLE, 7850 },
	{ 0x5fff, 0x5fff, PDF_CMAP_SINGLE, 5158 },
	{ 0x600f, 0x600f, PDF_CMAP_SINGLE, 5835 },
	{ 0x6012, 0x6012, PDF_CMAP_SINGLE, 4172 },
	{ 0x6016, 0x6016, PDF_CMAP_SINGLE, 7552 },
	{ 0x601c, 0x601d, PDF_CMAP_TABLE, 954 },
	{ 0x6020, 0x6021, PDF_CMAP_TABLE, 956 },
	{ 0x6025, 0x6025, PDF_CMAP_SINGLE, 4053 },
	{ 0x6027, 0x6028, PDF_CMAP_TABLE, 958 },
	{ 0x602a, 0x602a, PDF_CMAP_SINGLE, 3859 },
	{ 0x602f, 0x602f, PDF_CMAP_SINGLE, 3625 },
	{ 0x6041, 0x6043, PDF_CMAP_TABLE, 960 },
	{ 0x604d, 0x604d, PDF_CMAP_SINGLE, 7907 },
	{ 0x6050, 0x6050, PDF_CMAP_SINGLE, 3792 },
	{ 0x6052, 0x6052, PDF_CMAP_SINGLE, 7675 },
	{ 0x6055, 0x6055, PDF_CMAP_SINGLE, 5379 },
	{ 0x6059, 0x6059, PDF_CMAP_SINGLE, 5882 },
	{ 0x605d, 0x605d, PDF_CMAP_SINGLE, 3836 },
	{ 0x6062, 0x6065, PDF_CMAP_TABLE, 963 },
	{ 0x6068, 0x606a, PDF_CMAP_TABLE, 967 },
	{ 0x606c, 0x606d, PDF_CMAP_TABLE, 970 },
	{ 0x606f, 0x6070, PDF_CMAP_TABLE, 972 },
	{ 0x6085, 0x6085, PDF_CMAP_SINGLE, 5985 },
	{ 0x6089, 0x6089, PDF_CMAP_SINGLE, 5761 },
	{ 0x608c, 0x608d, PDF_CMAP_TABLE, 974 },
	{ 0x6094, 0x6094, PDF_CMAP_SINGLE, 7933 },
	{ 0x6096, 0x6096, PDF_CMAP_SINGLE, 7504 },
	{ 0x609a, 0x609b, PDF_CMAP_TABLE, 976 },
	{ 0x609f, 0x60a0, PDF_CMAP_TABLE, 978 },
	{ 0x60a3, 0x60a4, PDF_CMAP_TABLE, 980 },
	{ 0x60a7, 0x60a7, PDF_CMAP_SINGLE, 4619 },
	{ 0x60b0, 0x60b0, PDF_CMAP_SINGLE, 6840 },
	{ 0x60b2, 0x60b4, PDF_CMAP_TABLE, 982 },
	{ 0x60b6, 0x60b6, PDF_CMAP_SINGLE, 4871 },
	{ 0x60b8, 0x60b8, PDF_CMAP_SINGLE, 3711 },
	{ 0x60bc, 0x60bd, PDF_CMAP_TABLE, 985 },
	{ 0x60c5, 0x60c5, PDF_CMAP_SINGLE, 6716 },
	{ 0x60c7, 0x60c7, PDF_CMAP_SINGLE, 4320 },
	{ 0x60d1, 0x60d1, PDF_CMAP_SINGLE, 7841 },
	{ 0x60da, 0x60da, PDF_CMAP_SINGLE, 7851 },
	{ 0x60dc, 0x60dc, PDF_CMAP_SINGLE, 5406 },
	{ 0x60df, 0x60e1, PDF_CMAP_TABLE, 987 },
	{ 0x60f0, 0x60f1, PDF_CMAP_TABLE, 990 },
	{ 0x60f3, 0x60f3, PDF_CMAP_SINGLE, 5345 },
	{ 0x60f6, 0x60f6, PDF_CMAP_SINGLE, 7908 },
	{ 0x60f9, 0x60fb, PDF_CMAP_TABLE, 992 },
	{ 0x6101, 0x6101, PDF_CMAP_SINGLE, 5578 },
	{ 0x6106, 0x6106, PDF_CMAP_SINGLE, 3606 },
	{ 0x6108, 0x6109, PDF_CMAP_RANGE, 6322 },
	{ 0x610d, 0x610f, PDF_CMAP_TABLE, 995 },
	{ 0x6115, 0x6115, PDF_CMAP_SINGLE, 5798 },
	{ 0x611a, 0x611b, PDF_CMAP_TABLE, 998 },
	{ 0x611f, 0x611f, PDF_CMAP_SINGLE, 3514 },
	{ 0x6127, 0x6127, PDF_CMAP_SINGLE, 3860 },
	{ 0x6130, 0x6130, PDF_CMAP_SINGLE, 7909 },
	{ 0x6134, 0x6134, PDF_CMAP_SINGLE, 7099 },
	{ 0x6137, 0x6137, PDF_CMAP_SINGLE, 3565 },
	{ 0x613c, 0x613c, PDF_CMAP_SINGLE, 5741 },
	{ 0x613e, 0x613f, PDF_CMAP_TABLE, 1000 },
	{ 0x6142, 0x6142, PDF_CMAP_SINGLE, 6182 },
	{ 0x6144, 0x6144, PDF_CMAP_SINGLE, 4601 },
	{ 0x6147, 0x6148, PDF_CMAP_TABLE, 1002 },
	{ 0x614a, 0x614c, PDF_CMAP_TABLE, 1004 },
	{ 0x6153, 0x6153, PDF_CMAP_SINGLE, 7579 },
	{ 0x6155, 0x6155, PDF_CMAP_SINGLE, 4769 },
	{ 0x6158, 0x6159, PDF_CMAP_RANGE, 7085 },
	{ 0x615d, 0x615d, PDF_CMAP_SINGLE, 7471 },
	{ 0x615f, 0x615f, PDF_CMAP_SINGLE, 7453 },
	{ 0x6162, 0x6164, PDF_CMAP_TABLE, 1007 },
	{ 0x6167, 0x6168, PDF_CMAP_TABLE, 1010 },
	{ 0x616b, 0x616b, PDF_CMAP_SINGLE, 6841 },
	{ 0x616e, 0x616e, PDF_CMAP_SINGLE, 4445 },
	{ 0x6170, 0x6170, PDF_CMAP_SINGLE, 6292 },
	{ 0x6176, 0x6177, PDF_CMAP_TABLE, 1012 },
	{ 0x617d, 0x617e, PDF_CMAP_TABLE, 1014 },
	{ 0x6181, 0x6182, PDF_CMAP_TABLE, 1016 },
	{ 0x618a, 0x618a, PDF_CMAP_SINGLE, 5190 },
	{ 0x618e, 0x618e, PDF_CMAP_SINGLE, 6932 },
	{ 0x6190, 0x6191, PDF_CMAP_TABLE, 1018 },
	{ 0x6194, 0x6194, PDF_CMAP_SINGLE, 7217 },
	{ 0x6198, 0x619a, PDF_CMAP_TABLE, 1020 },
	{ 0x61a4, 0x61a4, PDF_CMAP_SINGLE, 5159 },
	{ 0x61a7, 0x61a7, PDF_CMAP_SINGLE, 4336 },
	{ 0x61a9, 0x61a9, PDF_CMAP_SINGLE, 3628 },
	{ 0x61ab, 0x61ac, PDF_CMAP_TABLE, 1023 },
	{ 0x61ae, 0x61ae, PDF_CMAP_SINGLE, 4814 },
	{ 0x61b2, 0x61b2, PDF_CMAP_SINGLE, 7723 },
	{ 0x61b6, 0x61b6, PDF_CMAP_SINGLE, 5913 },
	{ 0x61ba, 0x61ba, PDF_CMAP_SINGLE, 4219 },
	{ 0x61be, 0x61be, PDF_CMAP_SINGLE, 3515 },
	{ 0x61c3, 0x61c3, PDF_CMAP_SINGLE, 4026 },
	{ 0x61c7, 0x61cb, PDF_CMAP_TABLE, 1025 },
	{ 0x61e6, 0x61e6, PDF_CMAP_SINGLE, 4135 },
	{ 0x61f2, 0x61f2, PDF_CMAP_SINGLE, 7039 },
	{ 0x61f6, 0x61f8, PDF_CMAP_TABLE, 1030 },
	{ 0x61fa, 0x61fa, PDF_CMAP_SINGLE, 7087 },
	{ 0x61fc, 0x61fc, PDF_CMAP_SINGLE, 3910 },
	{ 0x61ff, 0x6200, PDF_CMAP_TABLE, 1033 },
	{ 0x6207, 0x6208, PDF_CMAP_TABLE, 1035 },
	{ 0x620a, 0x620a, PDF_CMAP_SINGLE, 4816 },
	{ 0x620c, 0x620e, PDF_CMAP_TABLE, 1037 },
	{ 0x6210, 0x6212, PDF_CMAP_TABLE, 1040 },
	{ 0x6216, 0x6216, PDF_CMAP_SINGLE, 7842 },
	{ 0x621a, 0x621a, PDF_CMAP_SINGLE, 7138 },
	{ 0x621f, 0x621f, PDF_CMAP_SINGLE, 4019 },
	{ 0x6221, 0x6221, PDF_CMAP_SINGLE, 3516 },
	{ 0x622a, 0x622a, PDF_CMAP_SINGLE, 6685 },
	{ 0x622e, 0x622e, PDF_CMAP_SINGLE, 4592 },
	{ 0x6230, 0x6231, PDF_CMAP_TABLE, 1043 },
	{ 0x6234, 0x6234, PDF_CMAP_SINGLE, 4258 },
	{ 0x6236, 0x6236, PDF_CMAP_SINGLE, 7808 },
	{ 0x623e, 0x6241, PDF_CMAP_TABLE, 1045 },
	{ 0x6247, 0x6249, PDF_CMAP_TABLE, 1049 },
	{ 0x624b, 0x624b, PDF_CMAP_SINGLE, 5580 },
	{ 0x624d, 0x624d, PDF_CMAP_SINGLE, 6574 },
	{ 0x6253, 0x6253, PDF_CMAP_SINGLE, 7386 },
	{ 0x6258, 0x6258, PDF_CMAP_SINGLE, 7398 },
	{ 0x626e, 0x626e, PDF_CMAP_SINGLE, 5160 },
	{ 0x6271, 0x6271, PDF_CMAP_SINGLE, 4054 },
	{ 0x6276, 0x6276, PDF_CMAP_SINGLE, 5124 },
	{ 0x6279, 0x6279, PDF_CMAP_SINGLE, 5192 },
	{ 0x627c, 0x627c, PDF_CMAP_SINGLE, 5852 },
	{ 0x627f, 0x6280, PDF_CMAP_TABLE, 1052 },
	{ 0x6284, 0x6284, PDF_CMAP_SINGLE, 7218 },
	{ 0x6289, 0x628a, PDF_CMAP_TABLE, 1054 },
	{ 0x6291, 0x6292, PDF_CMAP_TABLE, 1056 },
	{ 0x6295, 0x6295, PDF_CMAP_SINGLE, 7468 },
	{ 0x6297, 0x6298, PDF_CMAP_TABLE, 1058 },
	{ 0x629b, 0x629b, PDF_CMAP_SINGLE, 7553 },
	{ 0x62ab, 0x62ab, PDF_CMAP_SINGLE, 7598 },
	{ 0x62b1, 0x62b1, PDF_CMAP_SINGLE, 7554 },
	{ 0x62b5, 0x62b5, PDF_CMAP_SINGLE, 6597 },
	{ 0x62b9, 0x62b9, PDF_CMAP_SINGLE, 4690 },
	{ 0x62bc, 0x62bd, PDF_CMAP_TABLE, 1060 },
	{ 0x62c2, 0x62c2, PDF_CMAP_SINGLE, 5174 },
	{ 0x62c7, 0x62c9, PDF_CMAP_TABLE, 1062 },
	{ 0x62cc, 0x62cd, PDF_CMAP_TABLE, 1065 },
	{ 0x62cf, 0x62d0, PDF_CMAP_TABLE, 1067 },
	{ 0x62d2, 0x62d4, PDF_CMAP_TABLE, 1069 },
	{ 0x62d6, 0x62d9, PDF_CMAP_TABLE, 1072 },
	{ 0x62db, 0x62dc, PDF_CMAP_TABLE, 1076 },
	{ 0x62ec, 0x62ef, PDF_CMAP_TABLE, 1078 },
	{ 0x62f1, 0x62f1, PDF_CMAP_SINGLE, 3794 },
	{ 0x62f3, 0x62f3, PDF_CMAP_SINGLE, 3974 },
	{ 0x62f7, 0x62f7, PDF_CMAP_SINGLE, 3738 },
	{ 0x62fe, 0x62ff, PDF_CMAP_TABLE, 1082 },
	{ 0x6301, 0x6301, PDF_CMAP_SINGLE, 6948 },
	{ 0x6307, 0x6307, PDF_CMAP_SINGLE, 6949 },
	{ 0x6309, 0x6309, PDF_CMAP_SINGLE, 5809 },
	{ 0x6311, 0x6311, PDF_CMAP_SINGLE, 4282 },
	{ 0x632b, 0x632b, PDF_CMAP_SINGLE, 6858 },
	{ 0x632f, 0x632f, PDF_CMAP_SINGLE, 6983 },
	{ 0x633a, 0x633b, PDF_CMAP_TABLE, 1084 },
	{ 0x633d, 0x633e, PDF_CMAP_TABLE, 1086 },
	{ 0x6349, 0x6349, PDF_CMAP_SINGLE, 7055 },
	{ 0x634c, 0x634c, PDF_CMAP_SINGLE, 7501 },
	{ 0x634f, 0x6350, PDF_CMAP_TABLE, 1088 },
	{ 0x6355, 0x6355, PDF_CMAP_SINGLE, 7555 },
	{ 0x6367, 0x6368, PDF_CMAP_TABLE, 1090 },
	{ 0x636e, 0x636e, PDF_CMAP_SINGLE, 3590 },
	{ 0x6372, 0x6372, PDF_CMAP_SINGLE, 3975 },
	{ 0x6377, 0x6377, PDF_CMAP_SINGLE, 7190 },
	{ 0x637a, 0x637b, PDF_CMAP_TABLE, 1092 },
	{ 0x637f, 0x637f, PDF_CMAP_SINGLE, 5381 },
	{ 0x6383, 0x6383, PDF_CMAP_SINGLE, 5508 },
	{ 0x6388, 0x6389, PDF_CMAP_TABLE, 1094 },
	{ 0x638c, 0x638c, PDF_CMAP_SINGLE, 6545 },
	{ 0x6392, 0x6392, PDF_CMAP_SINGLE, 4974 },
	{ 0x6396, 0x6396, PDF_CMAP_SINGLE, 5853 },
	{ 0x6398, 0x6398, PDF_CMAP_SINGLE, 3961 },
	{ 0x639b, 0x639b, PDF_CMAP_SINGLE, 3853 },
	{ 0x63a0, 0x63a2, PDF_CMAP_TABLE, 1096 },
	{ 0x63a5, 0x63a5, PDF_CMAP_SINGLE, 6701 },
	{ 0x63a7, 0x63aa, PDF_CMAP_TABLE, 1099 },
	{ 0x63c0, 0x63c0, PDF_CMAP_SINGLE, 3484 },
	{ 0x63c4, 0x63c4, PDF_CMAP_SINGLE, 6324 },
	{ 0x63c6, 0x63c6, PDF_CMAP_SINGLE, 3997 },
	{ 0x63cf, 0x63d0, PDF_CMAP_TABLE, 1103 },
	{ 0x63d6, 0x63d6, PDF_CMAP_SINGLE, 6387 },
	{ 0x63da, 0x63db, PDF_CMAP_TABLE, 1105 },
	{ 0x63e1, 0x63e1, PDF_CMAP_SINGLE, 5799 },
	{ 0x63ed, 0x63ee, PDF_CMAP_TABLE, 1107 },
	{ 0x63f4, 0x63f4, PDF_CMAP_SINGLE, 6266 },
	{ 0x63f6, 0x63f7, PDF_CMAP_TABLE, 1109 },
	{ 0x640d, 0x640d, PDF_CMAP_SINGLE, 5548 },
	{ 0x640f, 0x640f, PDF_CMAP_SINGLE, 4889 },
	{ 0x6414, 0x6414, PDF_CMAP_SINGLE, 5509 },
	{ 0x6416, 0x6417, PDF_CMAP_TABLE, 1111 },
	{ 0x641c, 0x641c, PDF_CMAP_SINGLE, 5582 },
	{ 0x6422, 0x6422, PDF_CMAP_SINGLE, 6984 },
	{ 0x642c, 0x642d, PDF_CMAP_TABLE, 1113 },
	{ 0x643a, 0x643a, PDF_CMAP_SINGLE, 8006 },
	{ 0x643e, 0x643e, PDF_CMAP_SINGLE, 7056 },
	{ 0x6458, 0x6458, PDF_CMAP_SINGLE, 6623 },
	{ 0x6460, 0x6460, PDF_CMAP_SINGLE, 7256 },
	{ 0x6469, 0x6469, PDF_CMAP_SINGLE, 4656 },
	{ 0x646f, 0x646f, PDF_CMAP_SINGLE, 6950 },
	{ 0x6478, 0x6479, PDF_CMAP_RANGE, 4770 },
	{ 0x647a, 0x647a, PDF_CMAP_SINGLE, 6702 },
	{ 0x6488, 0x6488, PDF_CMAP_SINGLE, 4512 },
	{ 0x6491, 0x6493, PDF_CMAP_TABLE, 1115 },
	{ 0x649a, 0x649a, PDF_CMAP_SINGLE, 4161 },
	{ 0x649e, 0x649e, PDF_CMAP_SINGLE, 4244 },
	{ 0x64a4, 0x64a5, PDF_CMAP_TABLE, 1118 },
	{ 0x64ab, 0x64ab, PDF_CMAP_SINGLE, 4818 },
	{ 0x64ad, 0x64ae, PDF_CMAP_TABLE, 1120 },
	{ 0x64b0, 0x64b0, PDF_CMAP_SINGLE, 7062 },
	{ 0x64b2, 0x64b2, PDF_CMAP_SINGLE, 4890 },
	{ 0x64bb, 0x64bb, PDF_CMAP_SINGLE, 4212 },
	{ 0x64c1, 0x64c1, PDF_CMAP_SINGLE, 6091 },
	{ 0x64c4, 0x64c5, PDF_CMAP_TABLE, 1122 },
	{ 0x64c7, 0x64c7, PDF_CMAP_SINGLE, 7445 },
	{ 0x64ca, 0x64ca, PDF_CMAP_SINGLE, 3630 },
	{ 0x64cd, 0x64ce, PDF_CMAP_TABLE, 1124 },
	{ 0x64d2, 0x64d2, PDF_CMAP_SINGLE, 4040 },
	{ 0x64d4, 0x64d4, PDF_CMAP_SINGLE, 4220 },
	{ 0x64d8, 0x64d8, PDF_CMAP_SINGLE, 5023 },
	{ 0x64da, 0x64da, PDF_CMAP_SINGLE, 3591 },
	{ 0x64e1, 0x64e2, PDF_CMAP_TABLE, 1126 },
	{ 0x64e5, 0x64e7, PDF_CMAP_TABLE, 1128 },
	{ 0x64ec, 0x64ec, PDF_CMAP_SINGLE, 6400 },
	{ 0x64f2, 0x64f2, PDF_CMAP_SINGLE, 7140 },
	{ 0x64f4, 0x64f4, PDF_CMAP_SINGLE, 7877 },
	{ 0x64fa, 0x64fa, PDF_CMAP_SINGLE, 7479 },
	{ 0x64fe, 0x64fe, PDF_CMAP_SINGLE, 6151 },
	{ 0x6500, 0x6500, PDF_CMAP_SINGLE, 4911 },
	{ 0x6504, 0x6504, PDF_CMAP_SINGLE, 7448 },
	{ 0x6518, 0x6518, PDF_CMAP_SINGLE, 5884 },
	{ 0x651d, 0x651d, PDF_CMAP_SINGLE, 5471 },
	{ 0x6523, 0x6523, PDF_CMAP_SINGLE, 4467 },
	{ 0x652a, 0x652c, PDF_CMAP_TABLE, 1131 },
	{ 0x652f, 0x652f, PDF_CMAP_SINGLE, 6951 },
	{ 0x6536, 0x6539, PDF_CMAP_TABLE, 1134 },
	{ 0x653b, 0x653b, PDF_CMAP_SINGLE, 3796 },
	{ 0x653e, 0x653f, PDF_CMAP_TABLE, 1138 },
	{ 0x6545, 0x6545, PDF_CMAP_SINGLE, 3740 },
	{ 0x6548, 0x6548, PDF_CMAP_SINGLE, 7956 },
	{ 0x654d, 0x654f, PDF_CMAP_TABLE, 1140 },
	{ 0x6551, 0x6551, PDF_CMAP_SINGLE, 3912 },
	{ 0x6556, 0x6557, PDF_CMAP_TABLE, 1143 },
	{ 0x655e, 0x655e, PDF_CMAP_SINGLE, 7100 },
	{ 0x6562, 0x6563, PDF_CMAP_TABLE, 1145 },
	{ 0x6566, 0x6566, PDF_CMAP_SINGLE, 4321 },
	{ 0x656c, 0x656d, PDF_CMAP_TABLE, 1147 },
	{ 0x6572, 0x6572, PDF_CMAP_SINGLE, 3741 },
	{ 0x6574, 0x6575, PDF_CMAP_TABLE, 1149 },
	{ 0x6577, 0x6578, PDF_CMAP_TABLE, 1151 },
	{ 0x657e, 0x657e, PDF_CMAP_SINGLE, 5425 },
	{ 0x6582, 0x6583, PDF_CMAP_TABLE, 1153 },
	{ 0x6585, 0x6585, PDF_CMAP_SINGLE, 7957 },
	{ 0x6587, 0x6587, PDF_CMAP_SINGLE, 4840 },
	{ 0x658c, 0x658c, PDF_CMAP_SINGLE, 5226 },
	{ 0x6590, 0x6591, PDF_CMAP_TABLE, 1155 },
	{ 0x6597, 0x6597, PDF_CMAP_SINGLE, 4349 },
	{ 0x6599, 0x6599, PDF_CMAP_SINGLE, 4555 },
	{ 0x659b, 0x659c, PDF_CMAP_TABLE, 1157 },
	{ 0x659f, 0x659f, PDF_CMAP_SINGLE, 7030 },
	{ 0x65a1, 0x65a1, PDF_CMAP_SINGLE, 5817 },
	{ 0x65a4, 0x65a5, PDF_CMAP_TABLE, 1159 },
	{ 0x65a7, 0x65a7, PDF_CMAP_SINGLE, 5126 },
	{ 0x65ab, 0x65ac, PDF_CMAP_TABLE, 1161 },
	{ 0x65af, 0x65b0, PDF_CMAP_TABLE, 1163 },
	{ 0x65b7, 0x65b7, PDF_CMAP_SINGLE, 4199 },
	{ 0x65b9, 0x65b9, PDF_CMAP_SINGLE, 4950 },
	{ 0x65bc, 0x65bd, PDF_CMAP_TABLE, 1165 },
	{ 0x65c1, 0x65c1, PDF_CMAP_SINGLE, 4951 },
	{ 0x65c5, 0x65c5, PDF_CMAP_SINGLE, 4447 },
	{ 0x65cb, 0x65cc, PDF_CMAP_TABLE, 1167 },
	{ 0x65cf, 0x65cf, PDF_CMAP_SINGLE, 6828 },
	{ 0x65d2, 0x65d2, PDF_CMAP_SINGLE, 4578 },
	{ 0x65d7, 0x65d7, PDF_CMAP_SINGLE, 4081 },
	{ 0x65e0, 0x65e0, PDF_CMAP_SINGLE, 4819 },
	{ 0x65e3, 0x65e3, PDF_CMAP_SINGLE, 4082 },
	{ 0x65e5, 0x65e6, PDF_CMAP_TABLE, 1169 },
	{ 0x65e8, 0x65e9, PDF_CMAP_TABLE, 1171 },
	{ 0x65ec, 0x65ed, PDF_CMAP_TABLE, 1173 },
	{ 0x65f1, 0x65f1, PDF_CMAP_SINGLE, 7638 },
	{ 0x65f4, 0x65f4, PDF_CMAP_SINGLE, 6209 },
	{ 0x65fa, 0x65fa, PDF_CMAP_SINGLE, 6127 },
	{ 0x65fb, 0x65fc, PDF_CMAP_RANGE, 4875 },
	{ 0x65fd, 0x65fd, PDF_CMAP_SINGLE, 4322 },
	{ 0x65ff, 0x65ff, PDF_CMAP_SINGLE, 6065 },
	{ 0x6606, 0x6607, PDF_CMAP_TABLE, 1175 },
	{ 0x6609, 0x660a, PDF_CMAP_TABLE, 1177 },
	{ 0x660c, 0x660c, PDF_CMAP_SINGLE, 7101 },
	{ 0x660e, 0x6611, PDF_CMAP_TABLE, 1179 },
	{ 0x6613, 0x6615, PDF_CMAP_TABLE, 1183 },
	{ 0x661e, 0x6620, PDF_CMAP_TABLE, 1186 },
	{ 0x6625, 0x6625, PDF_CMAP_SINGLE, 7300 },
	{ 0x6627, 0x6628, PDF_CMAP_TABLE, 1189 },
	{ 0x662d, 0x662d, PDF_CMAP_SINGLE, 5510 },
	{ 0x662f, 0x6631, PDF_CMAP_TABLE, 1191 },
	{ 0x6634, 0x6634, PDF_CMAP_SINGLE, 4805 },
	{ 0x6636, 0x6636, PDF_CMAP_SINGLE, 7102 },
	{ 0x663a, 0x663b, PDF_CMAP_TABLE, 1194 },
	{ 0x6641, 0x6642, PDF_CMAP_TABLE, 1196 },
	{ 0x6643, 0x6644, PDF_CMAP_RANGE, 7911 },
	{ 0x6649, 0x6649, PDF_CMAP_SINGLE, 6985 },
	{ 0x664b, 0x664b, PDF_CMAP_SINGLE, 6986 },
	{ 0x664f, 0x664f, PDF_CMAP_SINGLE, 5810 },
	{ 0x6659, 0x6659, PDF_CMAP_SINGLE, 6908 },
	{ 0x665b, 0x665b, PDF_CMAP_SINGLE, 7738 },
	{ 0x665d, 0x665f, PDF_CMAP_TABLE, 1198 },
	{ 0x6664, 0x6669, PDF_CMAP_TABLE, 1201 },
	{ 0x666b, 0x666b, PDF_CMAP_SINGLE, 7400 },
	{ 0x666e, 0x666f, PDF_CMAP_TABLE, 1207 },
	{ 0x6673, 0x6674, PDF_CMAP_TABLE, 1209 },
	{ 0x6676, 0x6678, PDF_CMAP_TABLE, 1211 },
	{ 0x667a, 0x667a, PDF_CMAP_SINGLE, 6953 },
	{ 0x6684, 0x6684, PDF_CMAP_SINGLE, 7991 },
	{ 0x6687, 0x6689, PDF_CMAP_TABLE, 1214 },
	{ 0x668e, 0x668e, PDF_CMAP_SINGLE, 6006 },
	{ 0x6690, 0x6691, PDF_CMAP_TABLE, 1217 },
	{ 0x6696, 0x6698, PDF_CMAP_TABLE, 1219 },
	{ 0x669d, 0x669d, PDF_CMAP_SINGLE, 4752 },
	{ 0x66a0, 0x66a0, PDF_CMAP_SINGLE, 3742 },
	{ 0x66a2, 0x66a2, PDF_CMAP_SINGLE, 7103 },
	{ 0x66ab, 0x66ab, PDF_CMAP_SINGLE, 6528 },
	{ 0x66ae, 0x66ae, PDF_CMAP_SINGLE, 4772 },
	{ 0x66b2, 0x66b4, PDF_CMAP_TABLE, 1222 },
	{ 0x66b9, 0x66b9, PDF_CMAP_SINGLE, 5464 },
	{ 0x66bb, 0x66bb, PDF_CMAP_SINGLE, 3677 },
	{ 0x66be, 0x66be, PDF_CMAP_SINGLE, 4323 },
	{ 0x66c4, 0x66c4, PDF_CMAP_SINGLE, 5999 },
	{ 0x66c6, 0x66c7, PDF_CMAP_TABLE, 1225 },
	{ 0x66c9, 0x66c9, PDF_CMAP_SINGLE, 7958 },
	{ 0x66d6, 0x66d6, PDF_CMAP_SINGLE, 5845 },
	{ 0x66d9, 0x66d9, PDF_CMAP_SINGLE, 5384 },
	{ 0x66dc, 0x66dd, PDF_CMAP_TABLE, 1227 },
	{ 0x66e0, 0x66e0, PDF_CMAP_SINGLE, 3844 },
	{ 0x66e6, 0x66e6, PDF_CMAP_SINGLE, 8047 },
	{ 0x66f0, 0x66f0, PDF_CMAP_SINGLE, 6125 },
	{ 0x66f2, 0x66f4, PDF_CMAP_TABLE, 1229 },
	{ 0x66f7, 0x66fa, PDF_CMAP_TABLE, 1232 },
	{ 0x66fc, 0x66fc, PDF_CMAP_SINGLE, 4678 },
	{ 0x66fe, 0x6700, PDF_CMAP_TABLE, 1236 },
	{ 0x6703, 0x6703, PDF_CMAP_SINGLE, 7936 },
	{ 0x6708, 0x6709, PDF_CMAP_TABLE, 1239 },
	{ 0x670b, 0x670b, PDF_CMAP_SINGLE, 5176 },
	{ 0x670d, 0x670d, PDF_CMAP_SINGLE, 5080 },
	{ 0x6714, 0x6715, PDF_CMAP_TABLE, 1241 },
	{ 0x6717, 0x6717, PDF_CMAP_SINGLE, 4413 },
	{ 0x671b, 0x671b, PDF_CMAP_SINGLE, 4700 },
	{ 0x671d, 0x671d, PDF_CMAP_SINGLE, 6795 },
	{ 0x671e, 0x671f, PDF_CMAP_RANGE, 4083 },
	{ 0x6726, 0x6728, PDF_CMAP_TABLE, 1243 },
	{ 0x672a, 0x672e, PDF_CMAP_TABLE, 1246 },
	{ 0x6731, 0x6731, PDF_CMAP_SINGLE, 6874 },
	{ 0x6734, 0x6734, PDF_CMAP_SINGLE, 4891 },
	{ 0x6736, 0x6736, PDF_CMAP_SINGLE, 7388 },
	{ 0x673a, 0x673a, PDF_CMAP_SINGLE, 3984 },
	{ 0x673d, 0x673d, PDF_CMAP_SINGLE, 7975 },
	{ 0x6746, 0x6746, PDF_CMAP_SINGLE, 3485 },
	{ 0x6749, 0x6749, PDF_CMAP_SINGLE, 5321 },
	{ 0x674e, 0x6751, PDF_CMAP_TABLE, 1251 },
	{ 0x6753, 0x6753, PDF_CMAP_SINGLE, 7580 },
	{ 0x6756, 0x6756, PDF_CMAP_SINGLE, 6547 },
	{ 0x675c, 0x675c, PDF_CMAP_SINGLE, 4350 },
	{ 0x675e, 0x675f, PDF_CMAP_TABLE, 1255 },
	{ 0x676d, 0x676d, PDF_CMAP_SINGLE, 7677 },
	{ 0x676f, 0x6771, PDF_CMAP_TABLE, 1257 },
	{ 0x6773, 0x6773, PDF_CMAP_SINGLE, 4806 },
	{ 0x6775, 0x6775, PDF_CMAP_SINGLE, 6598 },
	{ 0x6777, 0x6777, PDF_CMAP_SINGLE, 7480 },
	{ 0x677b, 0x677b, PDF_CMAP_SINGLE, 4183 },
	{ 0x677e, 0x677f, PDF_CMAP_TABLE, 1260 },
	{ 0x6787, 0x6787, PDF_CMAP_SINGLE, 5194 },
	{ 0x6789, 0x6789, PDF_CMAP_SINGLE, 6128 },
	{ 0x678b, 0x678b, PDF_CMAP_SINGLE, 4953 },
	{ 0x678f, 0x6790, PDF_CMAP_TABLE, 1262 },
	{ 0x6793, 0x6793, PDF_CMAP_SINGLE, 4351 },
	{ 0x6795, 0x6795, PDF_CMAP_SINGLE, 7369 },
	{ 0x6797, 0x6797, PDF_CMAP_SINGLE, 4647 },
	{ 0x679a, 0x679a, PDF_CMAP_SINGLE, 4713 },
	{ 0x679c, 0x679d, PDF_CMAP_TABLE, 1264 },
	{ 0x67af, 0x67b0, PDF_CMAP_TABLE, 1266 },
	{ 0x67b3, 0x67b3, PDF_CMAP_SINGLE, 6955 },
	{ 0x67b6, 0x67b7, PDF_CMAP_RANGE, 3448 },
	{ 0x67b8, 0x67b8, PDF_CMAP_SINGLE, 3913 },
	{ 0x67be, 0x67be, PDF_CMAP_SINGLE, 5706 },
	{ 0x67c4, 0x67c4, PDF_CMAP_SINGLE, 5048 },
	{ 0x67cf, 0x67d4, PDF_CMAP_TABLE, 1268 },
	{ 0x67da, 0x67da, PDF_CMAP_SINGLE, 6328 },
	{ 0x67dd, 0x67dd, PDF_CMAP_SINGLE, 7401 },
	{ 0x67e9, 0x67e9, PDF_CMAP_SINGLE, 3914 },
	{ 0x67ec, 0x67ec, PDF_CMAP_SINGLE, 3486 },
	{ 0x67ef, 0x67f1, PDF_CMAP_TABLE, 1274 },
	{ 0x67f3, 0x67f6, PDF_CMAP_TABLE, 1277 },
	{ 0x67fb, 0x67fb, PDF_CMAP_SINGLE, 5269 },
	{ 0x67fe, 0x67fe, PDF_CMAP_SINGLE, 6723 },
	{ 0x6812, 0x6813, PDF_CMAP_TABLE, 1281 },
	{ 0x6816, 0x6817, PDF_CMAP_TABLE, 1283 },
	{ 0x6821, 0x6822, PDF_CMAP_TABLE, 1285 },
	{ 0x682a, 0x682a, PDF_CMAP_SINGLE, 6876 },
	{ 0x682f, 0x682f, PDF_CMAP_SINGLE, 6233 },
	{ 0x6838, 0x6839, PDF_CMAP_TABLE, 1287 },
	{ 0x683c, 0x683d, PDF_CMAP_TABLE, 1289 },
	{ 0x6840, 0x6843, PDF_CMAP_TABLE, 1291 },
	{ 0x6848, 0x6848, PDF_CMAP_SINGLE, 5811 },
	{ 0x684e, 0x684e, PDF_CMAP_SINGLE, 7020 },
	{ 0x6850, 0x6851, PDF_CMAP_TABLE, 1295 },
	{ 0x6853, 0x6854, PDF_CMAP_TABLE, 1297 },
	{ 0x686d, 0x686d, PDF_CMAP_SINGLE, 6987 },
	{ 0x6876, 0x6876, PDF_CMAP_SINGLE, 7454 },
	{ 0x687f, 0x687f, PDF_CMAP_SINGLE, 3487 },
	{ 0x6881, 0x6881, PDF_CMAP_SINGLE, 4431 },
	{ 0x6885, 0x6885, PDF_CMAP_SINGLE, 4714 },
	{ 0x688f, 0x688f, PDF_CMAP_SINGLE, 3770 },
	{ 0x6893, 0x6894, PDF_CMAP_TABLE, 1299 },
	{ 0x6897, 0x6897, PDF_CMAP_SINGLE, 3679 },
	{ 0x689d, 0x689d, PDF_CMAP_SINGLE, 6796 },
	{ 0x689f, 0x689f, PDF_CMAP_SINGLE, 7959 },
	{ 0x68a1, 0x68a2, PDF_CMAP_TABLE, 1301 },
	{ 0x68a7, 0x68a8, PDF_CMAP_TABLE, 1303 },
	{ 0x68ad, 0x68ad, PDF_CMAP_SINGLE, 5270 },
	{ 0x68af, 0x68b1, PDF_CMAP_TABLE, 1305 },
	{ 0x68b3, 0x68b3, PDF_CMAP_SINGLE, 5511 },
	{ 0x68b5, 0x68b6, PDF_CMAP_TABLE, 1308 },
	{ 0x68c4, 0x68c5, PDF_CMAP_TABLE, 1310 },
	{ 0x68c9, 0x68c9, PDF_CMAP_SINGLE, 4738 },
	{ 0x68cb, 0x68cb, PDF_CMAP_SINGLE, 4086 },
	{ 0x68cd, 0x68cd, PDF_CMAP_SINGLE, 3779 },
	{ 0x68d2, 0x68d2, PDF_CMAP_SINGLE, 5099 },
	{ 0x68d5, 0x68d5, PDF_CMAP_SINGLE, 6842 },
	{ 0x68d7, 0x68d8, PDF_CMAP_TABLE, 1312 },
	{ 0x68da, 0x68da, PDF_CMAP_SINGLE, 5177 },
	{ 0x68df, 0x68e0, PDF_CMAP_TABLE, 1314 },
	{ 0x68e7, 0x68e8, PDF_CMAP_TABLE, 1316 },
	{ 0x68ee, 0x68ee, PDF_CMAP_SINGLE, 5322 },
	{ 0x68f2, 0x68f2, PDF_CMAP_SINGLE, 5387 },
	{ 0x68f9, 0x68fa, PDF_CMAP_TABLE, 1318 },
	{ 0x6900, 0x6900, PDF_CMAP_SINGLE, 6111 },
	{ 0x6905, 0x6905, PDF_CMAP_SINGLE, 6401 },
	{ 0x690d, 0x690e, PDF_CMAP_TABLE, 1320 },
	{ 0x6912, 0x6912, PDF_CMAP_SINGLE, 7221 },
	{ 0x6927, 0x6927, PDF_CMAP_SINGLE, 4753 },
	{ 0x6930, 0x6930, PDF_CMAP_SINGLE, 5868 },
	{ 0x693d, 0x693d, PDF_CMAP_SINGLE, 5964 },
	{ 0x693f, 0x693f, PDF_CMAP_SINGLE, 7301 },
	{ 0x694a, 0x694a, PDF_CMAP_SINGLE, 5887 },
	{ 0x6953, 0x6955, PDF_CMAP_TABLE, 1322 },
	{ 0x6957, 0x6957, PDF_CMAP_SINGLE, 3607 },
	{ 0x6959, 0x695a, PDF_CMAP_TABLE, 1325 },
	{ 0x695e, 0x695e, PDF_CMAP_SINGLE, 4608 },
	{ 0x6960, 0x6960, PDF_CMAP_SINGLE, 4147 },
	{ 0x6961, 0x6962, PDF_CMAP_RANGE, 6329 },
	{ 0x6963, 0x6963, PDF_CMAP_SINGLE, 4859 },
	{ 0x6968, 0x6968, PDF_CMAP_SINGLE, 6724 },
	{ 0x696b, 0x696b, PDF_CMAP_SINGLE, 6928 },
	{ 0x696d, 0x696f, PDF_CMAP_TABLE, 1327 },
	{ 0x6975, 0x6975, PDF_CMAP_SINGLE, 4021 },
	{ 0x6977, 0x6979, PDF_CMAP_TABLE, 1330 },
	{ 0x6995, 0x6995, PDF_CMAP_SINGLE, 6183 },
	{ 0x699b, 0x699c, PDF_CMAP_TABLE, 1333 },
	{ 0x69a5, 0x69a5, PDF_CMAP_SINGLE, 7913 },
	{ 0x69a7, 0x69a7, PDF_CMAP_SINGLE, 5195 },
	{ 0x69ae, 0x69ae, PDF_CMAP_SINGLE, 6008 },
	{ 0x69b4, 0x69b4, PDF_CMAP_SINGLE, 4580 },
	{ 0x69bb, 0x69bb, PDF_CMAP_SINGLE, 7426 },
	{ 0x69c1, 0x69c1, PDF_CMAP_SINGLE, 3744 },
	{ 0x69c3, 0x69c3, PDF_CMAP_SINGLE, 4913 },
	{ 0x69cb, 0x69cd, PDF_CMAP_TABLE, 1335 },
	{ 0x69d0, 0x69d0, PDF_CMAP_SINGLE, 3862 },
	{ 0x69e8, 0x69e8, PDF_CMAP_SINGLE, 3816 },
	{ 0x69ea, 0x69ea, PDF_CMAP_SINGLE, 3569 },
	{ 0x69fb, 0x69fb, PDF_CMAP_SINGLE, 3998 },
	{ 0x69fd, 0x69fd, PDF_CMAP_SINGLE, 6798 },
	{ 0x69ff, 0x69ff, PDF_CMAP_SINGLE, 4029 },
	{ 0x6a02, 0x6a02, PDF_CMAP_SINGLE, 5800 },
	{ 0x6a0a, 0x6a0a, PDF_CMAP_SINGLE, 4997 },
	{ 0x6a11, 0x6a11, PDF_CMAP_SINGLE, 4432 },
	{ 0x6a13, 0x6a13, PDF_CMAP_SINGLE, 4567 },
	{ 0x6a17, 0x6a17, PDF_CMAP_SINGLE, 6600 },
	{ 0x6a19, 0x6a19, PDF_CMAP_SINGLE, 7581 },
	{ 0x6a1e, 0x6a1f, PDF_CMAP_TABLE, 1338 },
	{ 0x6a21, 0x6a21, PDF_CMAP_SINGLE, 4774 },
	{ 0x6a23, 0x6a23, PDF_CMAP_SINGLE, 5888 },
	{ 0x6a35, 0x6a35, PDF_CMAP_SINGLE, 7223 },
	{ 0x6a38, 0x6a3a, PDF_CMAP_TABLE, 1340 },
	{ 0x6a3d, 0x6a3d, PDF_CMAP_SINGLE, 6909 },
	{ 0x6a44, 0x6a44, PDF_CMAP_SINGLE, 3519 },
	{ 0x6a48, 0x6a48, PDF_CMAP_SINGLE, 6153 },
	{ 0x6a4b, 0x6a4b, PDF_CMAP_SINGLE, 3878 },
	{ 0x6a52, 0x6a53, PDF_CMAP_TABLE, 1343 },
	{ 0x6a58, 0x6a59, PDF_CMAP_TABLE, 1345 },
	{ 0x6a5f, 0x6a5f, PDF_CMAP_SINGLE, 4088 },
	{ 0x6a61, 0x6a61, PDF_CMAP_SINGLE, 5347 },
	{ 0x6a6b, 0x6a6b, PDF_CMAP_SINGLE, 7951 },
	{ 0x6a80, 0x6a80, PDF_CMAP_SINGLE, 4201 },
	{ 0x6a84, 0x6a84, PDF_CMAP_SINGLE, 3632 },
	{ 0x6a89, 0x6a89, PDF_CMAP_SINGLE, 6725 },
	{ 0x6a8d, 0x6a8e, PDF_CMAP_TABLE, 1347 },
	{ 0x6a97, 0x6a97, PDF_CMAP_SINGLE, 5024 },
	{ 0x6a9c, 0x6a9c, PDF_CMAP_SINGLE, 7937 },
	{ 0x6aa2, 0x6aa3, PDF_CMAP_TABLE, 1349 },
	{ 0x6ab3, 0x6ab3, PDF_CMAP_SINGLE, 5227 },
	{ 0x6abb, 0x6abb, PDF_CMAP_SINGLE, 7656 },
	{ 0x6ac2, 0x6ac3, PDF_CMAP_TABLE, 1351 },
	{ 0x6ad3, 0x6ad3, PDF_CMAP_SINGLE, 4514 },
	{ 0x6ada, 0x6adb, PDF_CMAP_TABLE, 1353 },
	{ 0x6af6, 0x6af6, PDF_CMAP_SINGLE, 7724 },
	{ 0x6afb, 0x6afb, PDF_CMAP_SINGLE, 5858 },
	{ 0x6b04, 0x6b04, PDF_CMAP_SINGLE, 4391 },
	{ 0x6b0a, 0x6b0a, PDF_CMAP_SINGLE, 3976 },
	{ 0x6b0c, 0x6b0c, PDF_CMAP_SINGLE, 6550 },
	{ 0x6b12, 0x6b12, PDF_CMAP_SINGLE, 4392 },
	{ 0x6b16, 0x6b16, PDF_CMAP_SINGLE, 4402 },
	{ 0x6b20, 0x6b21, PDF_CMAP_TABLE, 1355 },
	{ 0x6b23, 0x6b23, PDF_CMAP_SINGLE, 8020 },
	{ 0x6b32, 0x6b32, PDF_CMAP_SINGLE, 6169 },
	{ 0x6b3a, 0x6b3a, PDF_CMAP_SINGLE, 4089 },
	{ 0x6b3d, 0x6b3e, PDF_CMAP_TABLE, 1357 },
	{ 0x6b46, 0x6b47, PDF_CMAP_TABLE, 1359 },
	{ 0x6b4c, 0x6b4c, PDF_CMAP_SINGLE, 3451 },
	{ 0x6b4e, 0x6b4e, PDF_CMAP_SINGLE, 7413 },
	{ 0x6b50, 0x6b50, PDF_CMAP_SINGLE, 3916 },
	{ 0x6b5f, 0x6b5f, PDF_CMAP_SINGLE, 5937 },
	{ 0x6b61, 0x6b66, PDF_CMAP_TABLE, 1361 },
	{ 0x6b6a, 0x6b6a, PDF_CMAP_SINGLE, 6133 },
	{ 0x6b72, 0x6b72, PDF_CMAP_SINGLE, 5494 },
	{ 0x6b77, 0x6b78, PDF_CMAP_TABLE, 1367 },
	{ 0x6b7b, 0x6b7b, PDF_CMAP_SINGLE, 5271 },
	{ 0x6b7f, 0x6b7f, PDF_CMAP_SINGLE, 4795 },
	{ 0x6b83, 0x6b84, PDF_CMAP_TABLE, 1369 },
	{ 0x6b86, 0x6b86, PDF_CMAP_SINGLE, 7436 },
	{ 0x6b89, 0x6b8a, PDF_CMAP_TABLE, 1371 },
	{ 0x6b96, 0x6b96, PDF_CMAP_SINGLE, 5726 },
	{ 0x6b98, 0x6b98, PDF_CMAP_SINGLE, 6524 },
	{ 0x6b9e, 0x6b9e, PDF_CMAP_SINGLE, 6240 },
	{ 0x6bae, 0x6baf, PDF_CMAP_TABLE, 1373 },
	{ 0x6bb2, 0x6bb2, PDF_CMAP_SINGLE, 5465 },
	{ 0x6bb5, 0x6bb5, PDF_CMAP_SINGLE, 4202 },
	{ 0x6bb7, 0x6bb7, PDF_CMAP_SINGLE, 6376 },
	{ 0x6bba, 0x6bba, PDF_CMAP_SINGLE, 5317 },
	{ 0x6bbc, 0x6bbc, PDF_CMAP_SINGLE, 3470 },
	{ 0x6bbf, 0x6bbf, PDF_CMAP_SINGLE, 6659 },
	{ 0x6bc1, 0x6bc1, PDF_CMAP_SINGLE, 7996 },
	{ 0x6bc5, 0x6bc6, PDF_CMAP_TABLE, 1375 },
	{ 0x6bcb, 0x6bcb, PDF_CMAP_SINGLE, 4822 },
	{ 0x6bcd, 0x6bcd, PDF_CMAP_SINGLE, 4775 },
	{ 0x6bcf, 0x6bcf, PDF_CMAP_SINGLE, 4715 },
	{ 0x6bd2, 0x6bd4, PDF_CMAP_TABLE, 1377 },
	{ 0x6bd6, 0x6bd8, PDF_CMAP_RANGE, 5197 },
	{ 0x6bdb, 0x6bdb, PDF_CMAP_SINGLE, 4776 },
	{ 0x6beb, 0x6bec, PDF_CMAP_TABLE, 1380 },
	{ 0x6c08, 0x6c08, PDF_CMAP_SINGLE, 6660 },
	{ 0x6c0f, 0x6c0f, PDF_CMAP_SINGLE, 5774 },
	{ 0x6c11, 0x6c11, PDF_CMAP_SINGLE, 4877 },
	{ 0x6c13, 0x6c13, PDF_CMAP_SINGLE, 4728 },
	{ 0x6c23, 0x6c23, PDF_CMAP_SINGLE, 4090 },
	{ 0x6c34, 0x6c34, PDF_CMAP_SINGLE, 5587 },
	{ 0x6c37, 0x6c38, PDF_CMAP_TABLE, 1382 },
	{ 0x6c3e, 0x6c3e, PDF_CMAP_SINGLE, 5012 },
	{ 0x6c40, 0x6c42, PDF_CMAP_TABLE, 1384 },
	{ 0x6c4e, 0x6c4e, PDF_CMAP_SINGLE, 5013 },
	{ 0x6c50, 0x6c50, PDF_CMAP_SINGLE, 5410 },
	{ 0x6c55, 0x6c55, PDF_CMAP_SINGLE, 5307 },
	{ 0x6c57, 0x6c57, PDF_CMAP_SINGLE, 7639 },
	{ 0x6c5a, 0x6c5a, PDF_CMAP_SINGLE, 6068 },
	{ 0x6c5d, 0x6c60, PDF_CMAP_TABLE, 1387 },
	{ 0x6c68, 0x6c68, PDF_CMAP_SINGLE, 3784 },
	{ 0x6c6a, 0x6c6a, PDF_CMAP_SINGLE, 6129 },
	{ 0x6c6d, 0x6c6d, PDF_CMAP_SINGLE, 6034 },
	{ 0x6c70, 0x6c70, PDF_CMAP_SINGLE, 7437 },
	{ 0x6c72, 0x6c72, PDF_CMAP_SINGLE, 4055 },
	{ 0x6c76, 0x6c76, PDF_CMAP_SINGLE, 4841 },
	{ 0x6c7a, 0x6c7a, PDF_CMAP_SINGLE, 3649 },
	{ 0x6c7d, 0x6c7e, PDF_CMAP_TABLE, 1391 },
	{ 0x6c81, 0x6c83, PDF_CMAP_TABLE, 1393 },
	{ 0x6c85, 0x6c88, PDF_CMAP_TABLE, 1396 },
	{ 0x6c8c, 0x6c8c, PDF_CMAP_SINGLE, 4324 },
	{ 0x6c90, 0x6c90, PDF_CMAP_SINGLE, 4789 },
	{ 0x6c92, 0x6c96, PDF_CMAP_TABLE, 1400 },
	{ 0x6c99, 0x6c9b, PDF_CMAP_TABLE, 1405 },
	{ 0x6cab, 0x6cab, PDF_CMAP_SINGLE, 4692 },
	{ 0x6cae, 0x6cae, PDF_CMAP_SINGLE, 6601 },
	{ 0x6cb3, 0x6cb3, PDF_CMAP_SINGLE, 7622 },
	{ 0x6cb8, 0x6cb9, PDF_CMAP_TABLE, 1408 },
	{ 0x6cbb, 0x6cbf, PDF_CMAP_TABLE, 1410 },
	{ 0x6cc1, 0x6cc2, PDF_CMAP_TABLE, 1415 },
	{ 0x6cc4, 0x6cc4, PDF_CMAP_SINGLE, 5453 },
	{ 0x6cc9, 0x6cca, PDF_CMAP_TABLE, 1417 },
	{ 0x6ccc, 0x6ccc, PDF_CMAP_SINGLE, 7607 },
	{ 0x6cd3, 0x6cd3, PDF_CMAP_SINGLE, 7856 },
	{ 0x6cd5, 0x6cd5, PDF_CMAP_SINGLE, 5018 },
	{ 0x6cd7, 0x6cd7, PDF_CMAP_SINGLE, 5273 },
	{ 0x6cdb, 0x6cdb, PDF_CMAP_SINGLE, 5014 },
	{ 0x6ce1, 0x6ce3, PDF_CMAP_TABLE, 1419 },
	{ 0x6ce5, 0x6ce5, PDF_CMAP_SINGLE, 4187 },
	{ 0x6ce8, 0x6ce8, PDF_CMAP_SINGLE, 6877 },
	{ 0x6ceb, 0x6ceb, PDF_CMAP_SINGLE, 7739 },
	{ 0x6cee, 0x6cf0, PDF_CMAP_TABLE, 1422 },
	{ 0x6cf3, 0x6cf3, PDF_CMAP_SINGLE, 6010 },
	{ 0x6d0b, 0x6d0c, PDF_CMAP_TABLE, 1425 },
	{ 0x6d11, 0x6d11, PDF_CMAP_SINGLE, 5064 },
	{ 0x6d17, 0x6d17, PDF_CMAP_SINGLE, 5495 },
	{ 0x6d19, 0x6d19, PDF_CMAP_SINGLE, 5588 },
	{ 0x6d1b, 0x6d1b, PDF_CMAP_SINGLE, 4382 },
	{ 0x6d1e, 0x6d1e, PDF_CMAP_SINGLE, 4340 },
	{ 0x6d25, 0x6d25, PDF_CMAP_SINGLE, 6990 },
	{ 0x6d27, 0x6d27, PDF_CMAP_SINGLE, 6332 },
	{ 0x6d29, 0x6d2a, PDF_CMAP_TABLE, 1427 },
	{ 0x6d32, 0x6d32, PDF_CMAP_SINGLE, 6878 },
	{ 0x6d35, 0x6d36, PDF_CMAP_TABLE, 1429 },
	{ 0x6d38, 0x6d39, PDF_CMAP_TABLE, 1431 },
	{ 0x6d3b, 0x6d3b, PDF_CMAP_SINGLE, 7899 },
	{ 0x6d3d, 0x6d3e, PDF_CMAP_TABLE, 1433 },
	{ 0x6d41, 0x6d41, PDF_CMAP_SINGLE, 4581 },
	{ 0x6d59, 0x6d5a, PDF_CMAP_TABLE, 1435 },
	{ 0x6d5c, 0x6d5c, PDF_CMAP_SINGLE, 5229 },
	{ 0x6d63, 0x6d63, PDF_CMAP_SINGLE, 6112 },
	{ 0x6d66, 0x6d66, PDF_CMAP_SINGLE, 7557 },
	{ 0x6d69, 0x6d6a, PDF_CMAP_TABLE, 1437 },
	{ 0x6d6c, 0x6d6c, PDF_CMAP_SINGLE, 4622 },
	{ 0x6d6e, 0x6d6e, PDF_CMAP_SINGLE, 5127 },
	{ 0x6d74, 0x6d74, PDF_CMAP_SINGLE, 6170 },
	{ 0x6d77, 0x6d79, PDF_CMAP_TABLE, 1439 },
	{ 0x6d7f, 0x6d7f, PDF_CMAP_SINGLE, 7507 },
	{ 0x6d85, 0x6d85, PDF_CMAP_SINGLE, 5986 },
	{ 0x6d87, 0x6d89, PDF_CMAP_TABLE, 1442 },
	{ 0x6d8c, 0x6d8e, PDF_CMAP_TABLE, 1445 },
	{ 0x6d91, 0x6d91, PDF_CMAP_SINGLE, 5540 },
	{ 0x6d93, 0x6d93, PDF_CMAP_SINGLE, 5968 },
	{ 0x6d95, 0x6d95, PDF_CMAP_SINGLE, 7207 },
	{ 0x6daf, 0x6daf, PDF_CMAP_SINGLE, 5846 },
	{ 0x6db2, 0x6db2, PDF_CMAP_SINGLE, 5854 },
	{ 0x6db5, 0x6db5, PDF_CMAP_SINGLE, 7657 },
	{ 0x6dc0, 0x6dc0, PDF_CMAP_SINGLE, 6728 },
	{ 0x6dc3, 0x6dc7, PDF_CMAP_TABLE, 1448 },
	{ 0x6dcb, 0x6dcb, PDF_CMAP_SINGLE, 4648 },
	{ 0x6dcf, 0x6dcf, PDF_CMAP_SINGLE, 7814 },
	{ 0x6dd1, 0x6dd1, PDF_CMAP_SINGLE, 5633 },
	{ 0x6dd8, 0x6dda, PDF_CMAP_TABLE, 1453 },
	{ 0x6dde, 0x6dde, PDF_CMAP_SINGLE, 5556 },
	{ 0x6de1, 0x6de1, PDF_CMAP_SINGLE, 4222 },
	{ 0x6de8, 0x6de8, PDF_CMAP_SINGLE, 6729 },
	{ 0x6dea, 0x6deb, PDF_CMAP_TABLE, 1456 },
	{ 0x6dee, 0x6dee, PDF_CMAP_SINGLE, 7938 },
	{ 0x6df1, 0x6df1, PDF_CMAP_SINGLE, 5766 },
	{ 0x6df3, 0x6df3, PDF_CMAP_SINGLE, 5650 },
	{ 0x6df5, 0x6df5, PDF_CMAP_SINGLE, 5969 },
	{ 0x6df7, 0x6dfb, PDF_CMAP_TABLE, 1458 },
	{ 0x6e17, 0x6e17, PDF_CMAP_SINGLE, 5323 },
	{ 0x6e19, 0x6e1b, PDF_CMAP_TABLE, 1463 },
	{ 0x6e1f, 0x6e21, PDF_CMAP_TABLE, 1466 },
	{ 0x6e23, 0x6e26, PDF_CMAP_TABLE, 1469 },
	{ 0x6e2b, 0x6e2d, PDF_CMAP_TABLE, 1473 },
	{ 0x6e2f, 0x6e2f, PDF_CMAP_SINGLE, 7680 },
	{ 0x6e32, 0x6e32, PDF_CMAP_SINGLE, 5427 },
	{ 0x6e34, 0x6e34, PDF_CMAP_SINGLE, 3503 },
	{ 0x6e36, 0x6e36, PDF_CMAP_SINGLE, 6011 },
	{ 0x6e38, 0x6e38, PDF_CMAP_SINGLE, 6333 },
	{ 0x6e3a, 0x6e3a, PDF_CMAP_SINGLE, 4807 },
	{ 0x6e3c, 0x6e3e, PDF_CMAP_TABLE, 1476 },
	{ 0x6e43, 0x6e44, PDF_CMAP_TABLE, 1479 },
	{ 0x6e4a, 0x6e4a, PDF_CMAP_SINGLE, 6879 },
	{ 0x6e4d, 0x6e4d, PDF_CMAP_SINGLE, 4203 },
	{ 0x6e56, 0x6e56, PDF_CMAP_SINGLE, 7815 },
	{ 0x6e58, 0x6e58, PDF_CMAP_SINGLE, 5348 },
	{ 0x6e5b, 0x6e5c, PDF_CMAP_TABLE, 1481 },
	{ 0x6e5e, 0x6e5f, PDF_CMAP_TABLE, 1483 },
	{ 0x6e67, 0x6e67, PDF_CMAP_SINGLE, 6185 },
	{ 0x6e6b, 0x6e6b, PDF_CMAP_SINGLE, 7271 },
	{ 0x6e6e, 0x6e6f, PDF_CMAP_TABLE, 1485 },
	{ 0x6e72, 0x6e73, PDF_CMAP_TABLE, 1487 },
	{ 0x6e7a, 0x6e7a, PDF_CMAP_SINGLE, 5065 },
	{ 0x6e90, 0x6e90, PDF_CMAP_SINGLE, 6270 },
	{ 0x6e96, 0x6e96, PDF_CMAP_SINGLE, 6911 },
	{ 0x6e9c, 0x6e9d, PDF_CMAP_TABLE, 1489 },
	{ 0x6e9f, 0x6e9f, PDF_CMAP_SINGLE, 4754 },
	{ 0x6ea2, 0x6ea2, PDF_CMAP_SINGLE, 6465 },
	{ 0x6ea5, 0x6ea5, PDF_CMAP_SINGLE, 5128 },
	{ 0x6eaa, 0x6eab, PDF_CMAP_TABLE, 1491 },
	{ 0x6eaf, 0x6eaf, PDF_CMAP_SINGLE, 5514 },
	{ 0x6eb1, 0x6eb1, PDF_CMAP_SINGLE, 6991 },
	{ 0x6eb6, 0x6eb6, PDF_CMAP_SINGLE, 6186 },
	{ 0x6eba, 0x6eba, PDF_CMAP_SINGLE, 4189 },
	{ 0x6ec2, 0x6ec2, PDF_CMAP_SINGLE, 4955 },
	{ 0x6ec4, 0x6ec5, PDF_CMAP_TABLE, 1493 },
	{ 0x6ec9, 0x6ec9, PDF_CMAP_SINGLE, 7916 },
	{ 0x6ecb, 0x6ecc, PDF_CMAP_TABLE, 1495 },
	{ 0x6ece, 0x6ece, PDF_CMAP_SINGLE, 7777 },
	{ 0x6ed1, 0x6ed1, PDF_CMAP_SINGLE, 7900 },
	{ 0x6ed3, 0x6ed4, PDF_CMAP_TABLE, 1497 },
	{ 0x6eef, 0x6eef, PDF_CMAP_SINGLE, 7208 },
	{ 0x6ef4, 0x6ef4, PDF_CMAP_SINGLE, 6625 },
	{ 0x6ef8, 0x6ef8, PDF_CMAP_SINGLE, 7816 },
	{ 0x6efe, 0x6eff, PDF_CMAP_TABLE, 1499 },
	{ 0x6f01, 0x6f02, PDF_CMAP_TABLE, 1501 },
	{ 0x6f06, 0x6f06, PDF_CMAP_SINGLE, 7366 },
	{ 0x6f0f, 0x6f0f, PDF_CMAP_SINGLE, 4569 },
	{ 0x6f11, 0x6f11, PDF_CMAP_SINGLE, 3570 },
	{ 0x6f14, 0x6f15, PDF_CMAP_TABLE, 1503 },
	{ 0x6f20, 0x6f20, PDF_CMAP_SINGLE, 4666 },
	{ 0x6f22, 0x6f23, PDF_CMAP_TABLE, 1505 },
	{ 0x6f2b, 0x6f2c, PDF_CMAP_TABLE, 1507 },
	{ 0x6f31, 0x6f32, PDF_CMAP_TABLE, 1509 },
	{ 0x6f38, 0x6f38, PDF_CMAP_SINGLE, 6695 },
	{ 0x6f3f, 0x6f3f, PDF_CMAP_SINGLE, 6551 },
	{ 0x6f41, 0x6f41, PDF_CMAP_SINGLE, 6012 },
	{ 0x6f51, 0x6f51, PDF_CMAP_SINGLE, 4934 },
	{ 0x6f54, 0x6f54, PDF_CMAP_SINGLE, 3650 },
	{ 0x6f57, 0x6f58, PDF_CMAP_TABLE, 1511 },
	{ 0x6f5a, 0x6f5b, PDF_CMAP_TABLE, 1513 },
	{ 0x6f5e, 0x6f5f, PDF_CMAP_TABLE, 1515 },
	{ 0x6f62, 0x6f62, PDF_CMAP_SINGLE, 7917 },
	{ 0x6f64, 0x6f64, PDF_CMAP_SINGLE, 6362 },
	{ 0x6f6d, 0x6f6e, PDF_CMAP_TABLE, 1517 },
	{ 0x6f70, 0x6f70, PDF_CMAP_SINGLE, 3986 },
	{ 0x6f7a, 0x6f7a, PDF_CMAP_SINGLE, 6525 },
	{ 0x6f7c, 0x6f7e, PDF_CMAP_TABLE, 1519 },
	{ 0x6f81, 0x6f81, PDF_CMAP_SINGLE, 5328 },
	{ 0x6f84, 0x6f84, PDF_CMAP_SINGLE, 7040 },
	{ 0x6f88, 0x6f88, PDF_CMAP_SINGLE, 7172 },
	{ 0x6f8d, 0x6f8e, PDF_CMAP_TABLE, 1522 },
	{ 0x6f90, 0x6f90, PDF_CMAP_SINGLE, 6241 },
	{ 0x6f94, 0x6f94, PDF_CMAP_SINGLE, 7817 },
	{ 0x6f97, 0x6f97, PDF_CMAP_SINGLE, 3488 },
	{ 0x6fa3, 0x6fa4, PDF_CMAP_TABLE, 1524 },
	{ 0x6fa7, 0x6fa7, PDF_CMAP_SINGLE, 4507 },
	{ 0x6fae, 0x6faf, PDF_CMAP_TABLE, 1526 },
	{ 0x6fb1, 0x6fb1, PDF_CMAP_SINGLE, 6661 },
	{ 0x6fb3, 0x6fb3, PDF_CMAP_SINGLE, 6069 },
	{ 0x6fb9, 0x6fb9, PDF_CMAP_SINGLE, 4225 },
	{ 0x6fbe, 0x6fbe, PDF_CMAP_SINGLE, 4213 },
	{ 0x6fc0, 0x6fc3, PDF_CMAP_TABLE, 1528 },
	{ 0x6fca, 0x6fca, PDF_CMAP_SINGLE, 6035 },
	{ 0x6fd5, 0x6fd5, PDF_CMAP_SINGLE, 5677 },
	{ 0x6fda, 0x6fda, PDF_CMAP_SINGLE, 6013 },
	{ 0x6fdf, 0x6fe1, PDF_CMAP_TABLE, 1532 },
	{ 0x6fe4, 0x6fe4, PDF_CMAP_SINGLE, 4291 },
	{ 0x6fe9, 0x6fe9, PDF_CMAP_SINGLE, 7819 },
	{ 0x6feb, 0x6fec, PDF_CMAP_TABLE, 1535 },
	{ 0x6fef, 0x6fef, PDF_CMAP_SINGLE, 7403 },
	{ 0x6ff1, 0x6ff1, PDF_CMAP_SINGLE, 5230 },
	{ 0x6ffe, 0x6ffe, PDF_CMAP_SINGLE, 4449 },
	{ 0x7001, 0x7001, PDF_CMAP_SINGLE, 5890 },
	{ 0x7005, 0x7006, PDF_CMAP_TABLE, 1537 },
	{ 0x7009, 0x7009, PDF_CMAP_SINGLE, 5275 },
	{ 0x700b, 0x700b, PDF_CMAP_SINGLE, 5767 },
	{ 0x700f, 0x700f, PDF_CMAP_SINGLE, 4583 },
	{ 0x7011, 0x7011, PDF_CMAP_SINGLE, 7574 },
	{ 0x7015, 0x7015, PDF_CMAP_SINGLE, 5231 },
	{ 0x7018, 0x7018, PDF_CMAP_SINGLE, 4516 },
	{ 0x701a, 0x701f, PDF_CMAP_TABLE, 1539 },
	{ 0x7023, 0x7023, PDF_CMAP_SINGLE, 7695 },
	{ 0x7027, 0x7028, PDF_CMAP_TABLE, 1545 },
	{ 0x702f, 0x702f, PDF_CMAP_SINGLE, 6015 },
	{ 0x7037, 0x7037, PDF_CMAP_SINGLE, 6437 },
	{ 0x703e, 0x703e, PDF_CMAP_SINGLE, 4393 },
	{ 0x704c, 0x704c, PDF_CMAP_SINGLE, 3825 },
	{ 0x7050, 0x7051, PDF_CMAP_TABLE, 1547 },
	{ 0x7058, 0x7058, PDF_CMAP_SINGLE, 7414 },
	{ 0x705d, 0x705d, PDF_CMAP_SINGLE, 7820 },
	{ 0x7063, 0x7063, PDF_CMAP_SINGLE, 4681 },
	{ 0x706b, 0x706b, PDF_CMAP_SINGLE, 7867 },
	{ 0x7070, 0x7070, PDF_CMAP_SINGLE, 7940 },
	{ 0x7078, 0x7078, PDF_CMAP_SINGLE, 3921 },
	{ 0x707c, 0x707d, PDF_CMAP_TABLE, 1549 },
	{ 0x7085, 0x7085, PDF_CMAP_SINGLE, 3681 },
	{ 0x708a, 0x708a, PDF_CMAP_SINGLE, 7321 },
	{ 0x708e, 0x708e, PDF_CMAP_SINGLE, 5991 },
	{ 0x7092, 0x7092, PDF_CMAP_SINGLE, 7224 },
	{ 0x7098, 0x709a, PDF_CMAP_TABLE, 1551 },
	{ 0x70a1, 0x70a1, PDF_CMAP_SINGLE, 6733 },
	{ 0x70a4, 0x70a4, PDF_CMAP_SINGLE, 5516 },
	{ 0x70ab, 0x70ad, PDF_CMAP_TABLE, 1554 },
	{ 0x70af, 0x70af, PDF_CMAP_SINGLE, 7780 },
	{ 0x70b3, 0x70b3, PDF_CMAP_SINGLE, 5050 },
	{ 0x70b7, 0x70b9, PDF_CMAP_TABLE, 1557 },
	{ 0x70c8, 0x70c8, PDF_CMAP_SINGLE, 4481 },
	{ 0x70cb, 0x70cb, PDF_CMAP_SINGLE, 8007 },
	{ 0x70cf, 0x70cf, PDF_CMAP_SINGLE, 6070 },
	{ 0x70d8, 0x70d9, PDF_CMAP_TABLE, 1560 },
	{ 0x70dd, 0x70dd, PDF_CMAP_SINGLE, 6935 },
	{ 0x70df, 0x70df, PDF_CMAP_SINGLE, 5971 },
	{ 0x70f1, 0x70f1, PDF_CMAP_SINGLE, 3682 },
	{ 0x70f9, 0x70f9, PDF_CMAP_SINGLE, 7515 },
	{ 0x70fd, 0x70fd, PDF_CMAP_SINGLE, 5100 },
	{ 0x7104, 0x7104, PDF_CMAP_SINGLE, 7983 },
	{ 0x7109, 0x7109, PDF_CMAP_SINGLE, 5920 },
	{ 0x710c, 0x710c, PDF_CMAP_SINGLE, 6913 },
	{ 0x7119, 0x711a, PDF_CMAP_TABLE, 1562 },
	{ 0x711e, 0x711e, PDF_CMAP_SINGLE, 4325 },
	{ 0x7121, 0x7121, PDF_CMAP_SINGLE, 4823 },
	{ 0x7126, 0x7126, PDF_CMAP_SINGLE, 7225 },
	{ 0x7130, 0x7130, PDF_CMAP_SINGLE, 5992 },
	{ 0x7136, 0x7136, PDF_CMAP_SINGLE, 5972 },
	{ 0x7147, 0x7147, PDF_CMAP_SINGLE, 8001 },
	{ 0x7149, 0x714a, PDF_CMAP_TABLE, 1564 },
	{ 0x714c, 0x714c, PDF_CMAP_SINGLE, 7918 },
	{ 0x714e, 0x714e, PDF_CMAP_SINGLE, 6662 },
	{ 0x7150, 0x7150, PDF_CMAP_SINGLE, 6016 },
	{ 0x7156, 0x7156, PDF_CMAP_SINGLE, 4141 },
	{ 0x7159, 0x7159, PDF_CMAP_SINGLE, 5973 },
	{ 0x715c, 0x715c, PDF_CMAP_SINGLE, 6234 },
	{ 0x715e, 0x715e, PDF_CMAP_SINGLE, 5318 },
	{ 0x7164, 0x7167, PDF_CMAP_TABLE, 1566 },
	{ 0x7169, 0x7169, PDF_CMAP_SINGLE, 4998 },
	{ 0x716c, 0x716c, PDF_CMAP_SINGLE, 5891 },
	{ 0x716e, 0x716e, PDF_CMAP_SINGLE, 6495 },
	{ 0x717d, 0x717d, PDF_CMAP_SINGLE, 5428 },
	{ 0x7184, 0x7184, PDF_CMAP_SINGLE, 5728 },
	{ 0x7189, 0x718a, PDF_CMAP_TABLE, 1570 },
	{ 0x718f, 0x718f, PDF_CMAP_SINGLE, 7984 },
	{ 0x7192, 0x7192, PDF_CMAP_SINGLE, 7781 },
	{ 0x7194, 0x7194, PDF_CMAP_SINGLE, 6187 },
	{ 0x7199, 0x7199, PDF_CMAP_SINGLE, 8048 },
	{ 0x719f, 0x719f, PDF_CMAP_SINGLE, 5635 },
	{ 0x71a2, 0x71a2, PDF_CMAP_SINGLE, 5101 },
	{ 0x71ac, 0x71ac, PDF_CMAP_SINGLE, 6071 },
	{ 0x71b1, 0x71b1, PDF_CMAP_SINGLE, 5987 },
	{ 0x71b9, 0x71ba, PDF_CMAP_RANGE, 8049 },
	{ 0x71be, 0x71be, PDF_CMAP_SINGLE, 7345 },
	{ 0x71c1, 0x71c1, PDF_CMAP_SINGLE, 6000 },
	{ 0x71c3, 0x71c3, PDF_CMAP_SINGLE, 5974 },
	{ 0x71c8, 0x71c9, PDF_CMAP_TABLE, 1572 },
	{ 0x71ce, 0x71ce, PDF_CMAP_SINGLE, 4556 },
	{ 0x71d0, 0x71d0, PDF_CMAP_SINGLE, 4640 },
	{ 0x71d2, 0x71d2, PDF_CMAP_SINGLE, 5517 },
	{ 0x71d4, 0x71d5, PDF_CMAP_TABLE, 1574 },
	{ 0x71df, 0x71df, PDF_CMAP_SINGLE, 6017 },
	{ 0x71e5, 0x71e7, PDF_CMAP_TABLE, 1576 },
	{ 0x71ed, 0x71ee, PDF_CMAP_TABLE, 1579 },
	{ 0x71fb, 0x71fc, PDF_CMAP_TABLE, 1581 },
	{ 0x71fe, 0x7200, PDF_CMAP_TABLE, 1583 },
	{ 0x7206, 0x7206, PDF_CMAP_SINGLE, 7575 },
	{ 0x7210, 0x7210, PDF_CMAP_SINGLE, 4517 },
	{ 0x721b, 0x721b, PDF_CMAP_SINGLE, 4394 },
	{ 0x722a, 0x722a, PDF_CMAP_SINGLE, 6803 },
	{ 0x722c, 0x722d, PDF_CMAP_TABLE, 1586 },
	{ 0x7230, 0x7230, PDF_CMAP_SINGLE, 6271 },
	{ 0x7232, 0x7232, PDF_CMAP_SINGLE, 6295 },
	{ 0x7235, 0x7236, PDF_CMAP_TABLE, 1588 },
	{ 0x723a, 0x723b, PDF_CMAP_TABLE, 1590 },
	{ 0x723d, 0x723e, PDF_CMAP_TABLE, 1592 },
	{ 0x7240, 0x7240, PDF_CMAP_SINGLE, 5350 },
	{ 0x7246, 0x7248, PDF_CMAP_TABLE, 1594 },
	{ 0x724c, 0x724c, PDF_CMAP_SINGLE, 7508 },
	{ 0x7252, 0x7252, PDF_CMAP_SINGLE, 7191 },
	{ 0x7258, 0x7259, PDF_CMAP_TABLE, 1597 },
	{ 0x725b, 0x725b, PDF_CMAP_SINGLE, 6210 },
	{ 0x725d, 0x725d, PDF_CMAP_SINGLE, 5232 },
	{ 0x725f, 0x725f, PDF_CMAP_SINGLE, 4777 },
	{ 0x7261, 0x7262, PDF_CMAP_TABLE, 1599 },
	{ 0x7267, 0x7267, PDF_CMAP_SINGLE, 4790 },
	{ 0x7269, 0x7269, PDF_CMAP_SINGLE, 4850 },
	{ 0x7272, 0x7272, PDF_CMAP_SINGLE, 5369 },
	{ 0x7279, 0x7279, PDF_CMAP_SINGLE, 7472 },
	{ 0x727d, 0x727d, PDF_CMAP_SINGLE, 3638 },
	{ 0x7280, 0x7281, PDF_CMAP_TABLE, 1601 },
	{ 0x72a2, 0x72a2, PDF_CMAP_SINGLE, 4312 },
	{ 0x72a7, 0x72a7, PDF_CMAP_SINGLE, 8051 },
	{ 0x72ac, 0x72ac, PDF_CMAP_SINGLE, 3639 },
	{ 0x72af, 0x72af, PDF_CMAP_SINGLE, 5015 },
	{ 0x72c0, 0x72c0, PDF_CMAP_SINGLE, 5351 },
	{ 0x72c2, 0x72c2, PDF_CMAP_SINGLE, 3847 },
	{ 0x72c4, 0x72c4, PDF_CMAP_SINGLE, 6626 },
	{ 0x72ce, 0x72ce, PDF_CMAP_SINGLE, 5831 },
	{ 0x72d0, 0x72d0, PDF_CMAP_SINGLE, 7821 },
	{ 0x72d7, 0x72d7, PDF_CMAP_SINGLE, 3922 },
	{ 0x72d9, 0x72d9, PDF_CMAP_SINGLE, 6603 },
	{ 0x72e1, 0x72e1, PDF_CMAP_SINGLE, 3879 },
	{ 0x72e9, 0x72e9, PDF_CMAP_SINGLE, 5591 },
	{ 0x72f8, 0x72f9, PDF_CMAP_TABLE, 1603 },
	{ 0x72fc, 0x72fd, PDF_CMAP_TABLE, 1605 },
	{ 0x730a, 0x730a, PDF_CMAP_SINGLE, 6036 },
	{ 0x7316, 0x7316, PDF_CMAP_SINGLE, 7107 },
	{ 0x731b, 0x731d, PDF_CMAP_TABLE, 1607 },
	{ 0x7325, 0x7325, PDF_CMAP_SINGLE, 6138 },
	{ 0x7329, 0x732b, PDF_CMAP_TABLE, 1610 },
	{ 0x7336, 0x7337, PDF_CMAP_RANGE, 6335 },
	{ 0x733e, 0x733f, PDF_CMAP_TABLE, 1613 },
	{ 0x7344, 0x7345, PDF_CMAP_TABLE, 1615 },
	{ 0x7350, 0x7350, PDF_CMAP_SINGLE, 6553 },
	{ 0x7352, 0x7352, PDF_CMAP_SINGLE, 6072 },
	{ 0x7357, 0x7357, PDF_CMAP_SINGLE, 3980 },
	{ 0x7368, 0x7368, PDF_CMAP_SINGLE, 4313 },
	{ 0x736a, 0x736a, PDF_CMAP_SINGLE, 7941 },
	{ 0x7370, 0x7370, PDF_CMAP_SINGLE, 6018 },
	{ 0x7372, 0x7372, PDF_CMAP_SINGLE, 7949 },
	{ 0x7375, 0x7375, PDF_CMAP_SINGLE, 4488 },
	{ 0x7378, 0x7378, PDF_CMAP_SINGLE, 5592 },
	{ 0x737a, 0x737b, PDF_CMAP_TABLE, 1617 },
	{ 0x7384, 0x7384, PDF_CMAP_SINGLE, 7741 },
	{ 0x7386, 0x7387, PDF_CMAP_TABLE, 1619 },
	{ 0x7389, 0x7389, PDF_CMAP_SINGLE, 6081 },
	{ 0x738b, 0x738b, PDF_CMAP_SINGLE, 6130 },
	{ 0x738e, 0x738e, PDF_CMAP_SINGLE, 6734 },
	{ 0x7394, 0x7394, PDF_CMAP_SINGLE, 7156 },
	{ 0x7396, 0x7398, PDF_CMAP_TABLE, 1621 },
	{ 0x739f, 0x739f, PDF_CMAP_SINGLE, 4879 },
	{ 0x73a7, 0x73a7, PDF_CMAP_SINGLE, 6363 },
	{ 0x73a9, 0x73a9, PDF_CMAP_SINGLE, 6113 },
	{ 0x73ad, 0x73ad, PDF_CMAP_SINGLE, 5233 },
	{ 0x73b2, 0x73b3, PDF_CMAP_TABLE, 1624 },
	{ 0x73b9, 0x73b9, PDF_CMAP_SINGLE, 7742 },
	{ 0x73c0, 0x73c0, PDF_CMAP_SINGLE, 4894 },
	{ 0x73c2, 0x73c2, PDF_CMAP_SINGLE, 3452 },
	{ 0x73c9, 0x73ca, PDF_CMAP_TABLE, 1626 },
	{ 0x73cc, 0x73cd, PDF_CMAP_TABLE, 1628 },
	{ 0x73cf, 0x73cf, PDF_CMAP_SINGLE, 3471 },
	{ 0x73d6, 0x73d6, PDF_CMAP_SINGLE, 3848 },
	{ 0x73d9, 0x73d9, PDF_CMAP_SINGLE, 3797 },
	{ 0x73dd, 0x73de, PDF_CMAP_TABLE, 1630 },
	{ 0x73e0, 0x73e0, PDF_CMAP_SINGLE, 6882 },
	{ 0x73e3, 0x73e6, PDF_CMAP_TABLE, 1632 },
	{ 0x73e9, 0x73ea, PDF_CMAP_TABLE, 1636 },
	{ 0x73ed, 0x73ed, PDF_CMAP_SINGLE, 4916 },
	{ 0x73f7, 0x73f7, PDF_CMAP_SINGLE, 4824 },
	{ 0x73f9, 0x73f9, PDF_CMAP_SINGLE, 5483 },
	{ 0x73fd, 0x73fe, PDF_CMAP_TABLE, 1638 },
	{ 0x7401, 0x7401, PDF_CMAP_SINGLE, 5429 },
	{ 0x7403, 0x7403, PDF_CMAP_SINGLE, 3924 },
	{ 0x7405, 0x7407, PDF_CMAP_TABLE, 1640 },
	{ 0x7409, 0x7409, PDF_CMAP_SINGLE, 4584 },
	{ 0x7413, 0x7413, PDF_CMAP_SINGLE, 6114 },
	{ 0x741b, 0x741b, PDF_CMAP_SINGLE, 7372 },
	{ 0x7420, 0x7422, PDF_CMAP_TABLE, 1643 },
	{ 0x7425, 0x7426, PDF_CMAP_TABLE, 1646 },
	{ 0x7428, 0x7428, PDF_CMAP_SINGLE, 3781 },
	{ 0x742a, 0x742c, PDF_CMAP_TABLE, 1648 },
	{ 0x742e, 0x7430, PDF_CMAP_TABLE, 1651 },
	{ 0x7433, 0x7436, PDF_CMAP_TABLE, 1654 },
	{ 0x7438, 0x7438, PDF_CMAP_SINGLE, 7405 },
	{ 0x743a, 0x743a, PDF_CMAP_SINGLE, 5019 },
	{ 0x743f, 0x7441, PDF_CMAP_TABLE, 1658 },
	{ 0x7443, 0x7444, PDF_CMAP_TABLE, 1661 },
	{ 0x744b, 0x744b, PDF_CMAP_SINGLE, 6296 },
	{ 0x7455, 0x7455, PDF_CMAP_SINGLE, 7623 },
	{ 0x7457, 0x7457, PDF_CMAP_SINGLE, 6273 },
	{ 0x7459, 0x745c, PDF_CMAP_TABLE, 1663 },
	{ 0x745e, 0x7460, PDF_CMAP_TABLE, 1667 },
	{ 0x7462, 0x7462, PDF_CMAP_SINGLE, 6188 },
	{ 0x7464, 0x7465, PDF_CMAP_TABLE, 1670 },
	{ 0x7468, 0x746a, PDF_CMAP_TABLE, 1672 },
	{ 0x746f, 0x746f, PDF_CMAP_SINGLE, 4417 },
	{ 0x747e, 0x747e, PDF_CMAP_SINGLE, 4030 },
	{ 0x7482, 0x7483, PDF_CMAP_TABLE, 1675 },
	{ 0x7487, 0x7487, PDF_CMAP_SINGLE, 5431 },
	{ 0x7489, 0x7489, PDF_CMAP_SINGLE, 4470 },
	{ 0x748b, 0x748b, PDF_CMAP_SINGLE, 6554 },
	{ 0x7498, 0x7498, PDF_CMAP_SINGLE, 4641 },
	{ 0x749c, 0x749c, PDF_CMAP_SINGLE, 7919 },
	{ 0x749e, 0x749f, PDF_CMAP_TABLE, 1677 },
	{ 0x74a1, 0x74a1, PDF_CMAP_SINGLE, 6994 },
	{ 0x74a3, 0x74a3, PDF_CMAP_SINGLE, 4098 },
	{ 0x74a5, 0x74a5, PDF_CMAP_SINGLE, 3684 },
	{ 0x74a7, 0x74a8, PDF_CMAP_TABLE, 1679 },
	{ 0x74aa, 0x74aa, PDF_CMAP_SINGLE, 6804 },
	{ 0x74b0, 0x74b0, PDF_CMAP_SINGLE, 7894 },
	{ 0x74b2, 0x74b2, PDF_CMAP_SINGLE, 5594 },
	{ 0x74b5, 0x74b5, PDF_CMAP_SINGLE, 5939 },
	{ 0x74b9, 0x74b9, PDF_CMAP_SINGLE, 5637 },
	{ 0x74bd, 0x74bd, PDF_CMAP_SINGLE, 5363 },
	{ 0x74bf, 0x74bf, PDF_CMAP_SINGLE, 5432 },
	{ 0x74c6, 0x74c6, PDF_CMAP_SINGLE, 7021 },
	{ 0x74ca, 0x74ca, PDF_CMAP_SINGLE, 3685 },
	{ 0x74cf, 0x74cf, PDF_CMAP_SINGLE, 4540 },
	{ 0x74d4, 0x74d4, PDF_CMAP_SINGLE, 6020 },
	{ 0x74d8, 0x74d8, PDF_CMAP_SINGLE, 3827 },
	{ 0x74da, 0x74da, PDF_CMAP_SINGLE, 7066 },
	{ 0x74dc, 0x74dc, PDF_CMAP_SINGLE, 3806 },
	{ 0x74e0, 0x74e0, PDF_CMAP_SINGLE, 7824 },
	{ 0x74e2, 0x74e3, PDF_CMAP_TABLE, 1681 },
	{ 0x74e6, 0x74e6, PDF_CMAP_SINGLE, 6100 },
	{ 0x74ee, 0x74ee, PDF_CMAP_SINGLE, 6092 },
	{ 0x74f7, 0x74f7, PDF_CMAP_SINGLE, 6497 },
	{ 0x7501, 0x7501, PDF_CMAP_SINGLE, 5051 },
	{ 0x7504, 0x7504, PDF_CMAP_SINGLE, 3640 },
	{ 0x7511, 0x7511, PDF_CMAP_SINGLE, 6936 },
	{ 0x7515, 0x7515, PDF_CMAP_SINGLE, 6093 },
	{ 0x7518, 0x7518, PDF_CMAP_SINGLE, 3521 },
	{ 0x751a, 0x751b, PDF_CMAP_TABLE, 1683 },
	{ 0x751f, 0x751f, PDF_CMAP_SINGLE, 5370 },
	{ 0x7523, 0x7523, PDF_CMAP_SINGLE, 5309 },
	{ 0x7525, 0x7526, PDF_CMAP_TABLE, 1685 },
	{ 0x7528, 0x7528, PDF_CMAP_SINGLE, 6189 },
	{ 0x752b, 0x752c, PDF_CMAP_TABLE, 1687 },
	{ 0x7530, 0x7533, PDF_CMAP_TABLE, 1689 },
	{ 0x7537, 0x7538, PDF_CMAP_TABLE, 1693 },
	{ 0x753a, 0x753a, PDF_CMAP_SINGLE, 6736 },
	{ 0x7547, 0x7547, PDF_CMAP_SINGLE, 4011 },
	{ 0x754c, 0x754c, PDF_CMAP_SINGLE, 3717 },
	{ 0x754f, 0x754f, PDF_CMAP_SINGLE, 6139 },
	{ 0x7551, 0x7551, PDF_CMAP_SINGLE, 6666 },
	{ 0x7553, 0x7554, PDF_CMAP_TABLE, 1695 },
	{ 0x7559, 0x7559, PDF_CMAP_SINGLE, 4586 },
	{ 0x755b, 0x755d, PDF_CMAP_TABLE, 1697 },
	{ 0x7562, 0x7562, PDF_CMAP_SINGLE, 7609 },
	{ 0x7565, 0x7566, PDF_CMAP_TABLE, 1700 },
	{ 0x756a, 0x756a, PDF_CMAP_SINGLE, 5000 },
	{ 0x756f, 0x7570, PDF_CMAP_TABLE, 1702 },
	{ 0x7575, 0x7576, PDF_CMAP_TABLE, 1704 },
	{ 0x7578, 0x7578, PDF_CMAP_SINGLE, 4099 },
	{ 0x757a, 0x757a, PDF_CMAP_SINGLE, 3546 },
	{ 0x757f, 0x757f, PDF_CMAP_SINGLE, 4100 },
	{ 0x7586, 0x7587, PDF_CMAP_TABLE, 1706 },
	{ 0x758a, 0x758b, PDF_CMAP_TABLE, 1708 },
	{ 0x758e, 0x758f, PDF_CMAP_TABLE, 1710 },
	{ 0x7591, 0x7591, PDF_CMAP_SINGLE, 6403 },
	{ 0x759d, 0x759d, PDF_CMAP_SINGLE, 5310 },
	{ 0x75a5, 0x75a5, PDF_CMAP_SINGLE, 3571 },
	{ 0x75ab, 0x75ab, PDF_CMAP_SINGLE, 5951 },
	{ 0x75b1, 0x75b3, PDF_CMAP_TABLE, 1712 },
	{ 0x75b5, 0x75b5, PDF_CMAP_SINGLE, 6498 },
	{ 0x75b8, 0x75b9, PDF_CMAP_TABLE, 1715 },
	{ 0x75bc, 0x75be, PDF_CMAP_TABLE, 1717 },
	{ 0x75c2, 0x75c2, PDF_CMAP_SINGLE, 3453 },
	{ 0x75c5, 0x75c5, PDF_CMAP_SINGLE, 5052 },
	{ 0x75c7, 0x75c7, PDF_CMAP_SINGLE, 6937 },
	{ 0x75cd, 0x75cd, PDF_CMAP_SINGLE, 6425 },
	{ 0x75d2, 0x75d2, PDF_CMAP_SINGLE, 5892 },
	{ 0x75d4, 0x75d5, PDF_CMAP_TABLE, 1720 },
	{ 0x75d8, 0x75d9, PDF_CMAP_TABLE, 1722 },
	{ 0x75db, 0x75db, PDF_CMAP_SINGLE, 7455 },
	{ 0x75e2, 0x75e2, PDF_CMAP_SINGLE, 4627 },
	{ 0x75f0, 0x75f0, PDF_CMAP_SINGLE, 4226 },
	{ 0x75f2, 0x75f2, PDF_CMAP_SINGLE, 4658 },
	{ 0x75f4, 0x75f4, PDF_CMAP_SINGLE, 7347 },
	{ 0x75fa, 0x75fa, PDF_CMAP_SINGLE, 5202 },
	{ 0x75fc, 0x75fc, PDF_CMAP_SINGLE, 3746 },
	{ 0x7600, 0x7600, PDF_CMAP_SINGLE, 5906 },
	{ 0x760d, 0x760d, PDF_CMAP_SINGLE, 5893 },
	{ 0x7619, 0x7619, PDF_CMAP_SINGLE, 5521 },
	{ 0x761f, 0x7622, PDF_CMAP_TABLE, 1724 },
	{ 0x7624, 0x7624, PDF_CMAP_SINGLE, 4587 },
	{ 0x7626, 0x7626, PDF_CMAP_SINGLE, 5595 },
	{ 0x763b, 0x763b, PDF_CMAP_SINGLE, 4570 },
	{ 0x7642, 0x7642, PDF_CMAP_SINGLE, 4557 },
	{ 0x764c, 0x764c, PDF_CMAP_SINGLE, 5826 },
	{ 0x764e, 0x764e, PDF_CMAP_SINGLE, 3489 },
	{ 0x7652, 0x7652, PDF_CMAP_SINGLE, 6339 },
	{ 0x7656, 0x7656, PDF_CMAP_SINGLE, 5026 },
	{ 0x7661, 0x7661, PDF_CMAP_SINGLE, 7348 },
	{ 0x7664, 0x7664, PDF_CMAP_SINGLE, 6688 },
	{ 0x7669, 0x7669, PDF_CMAP_SINGLE, 4376 },
	{ 0x766c, 0x766c, PDF_CMAP_SINGLE, 5433 },
	{ 0x7670, 0x7670, PDF_CMAP_SINGLE, 6094 },
	{ 0x7672, 0x7672, PDF_CMAP_SINGLE, 6667 },
	{ 0x7678, 0x7678, PDF_CMAP_SINGLE, 3718 },
	{ 0x767b, 0x767c, PDF_CMAP_TABLE, 1728 },
	{ 0x767d, 0x767e, PDF_CMAP_RANGE, 4993 },
	{ 0x7684, 0x7684, PDF_CMAP_SINGLE, 6627 },
	{ 0x7686, 0x7687, PDF_CMAP_TABLE, 1730 },
	{ 0x768e, 0x768e, PDF_CMAP_SINGLE, 3880 },
	{ 0x7690, 0x7690, PDF_CMAP_SINGLE, 3747 },
	{ 0x7693, 0x7693, PDF_CMAP_SINGLE, 7825 },
	{ 0x76ae, 0x76ae, PDF_CMAP_SINGLE, 7600 },
	{ 0x76ba, 0x76ba, PDF_CMAP_SINGLE, 7272 },
	{ 0x76bf, 0x76bf, PDF_CMAP_SINGLE, 4755 },
	{ 0x76c2, 0x76c3, PDF_CMAP_TABLE, 1732 },
	{ 0x76c6, 0x76c6, PDF_CMAP_SINGLE, 5164 },
	{ 0x76c8, 0x76c8, PDF_CMAP_SINGLE, 6021 },
	{ 0x76ca, 0x76ca, PDF_CMAP_SINGLE, 6438 },
	{ 0x76d2, 0x76d2, PDF_CMAP_SINGLE, 7665 },
	{ 0x76d6, 0x76d6, PDF_CMAP_SINGLE, 3573 },
	{ 0x76db, 0x76dc, PDF_CMAP_TABLE, 1734 },
	{ 0x76de, 0x76df, PDF_CMAP_TABLE, 1736 },
	{ 0x76e1, 0x76e1, PDF_CMAP_SINGLE, 6997 },
	{ 0x76e3, 0x76e4, PDF_CMAP_TABLE, 1738 },
	{ 0x76e7, 0x76e7, PDF_CMAP_SINGLE, 4518 },
	{ 0x76ee, 0x76ee, PDF_CMAP_SINGLE, 4791 },
	{ 0x76f2, 0x76f2, PDF_CMAP_SINGLE, 4730 },
	{ 0x76f4, 0x76f4, PDF_CMAP_SINGLE, 6975 },
	{ 0x76f8, 0x76f8, PDF_CMAP_SINGLE, 5352 },
	{ 0x76fc, 0x76fc, PDF_CMAP_SINGLE, 4920 },
	{ 0x76fe, 0x76fe, PDF_CMAP_SINGLE, 5652 },
	{ 0x7701, 0x7701, PDF_CMAP_SINGLE, 5485 },
	{ 0x7704, 0x7704, PDF_CMAP_SINGLE, 4740 },
	{ 0x7708, 0x7709, PDF_CMAP_TABLE, 1740 },
	{ 0x770b, 0x770b, PDF_CMAP_SINGLE, 3490 },
	{ 0x771e, 0x771e, PDF_CMAP_SINGLE, 6998 },
	{ 0x7720, 0x7720, PDF_CMAP_SINGLE, 4741 },
	{ 0x7729, 0x7729, PDF_CMAP_SINGLE, 7744 },
	{ 0x7737, 0x7738, PDF_CMAP_TABLE, 1742 },
	{ 0x773a, 0x773a, PDF_CMAP_SINGLE, 6805 },
	{ 0x773c, 0x773c, PDF_CMAP_SINGLE, 5812 },
	{ 0x7740, 0x7740, PDF_CMAP_SINGLE, 7057 },
	{ 0x774d, 0x774d, PDF_CMAP_SINGLE, 7745 },
	{ 0x775b, 0x775b, PDF_CMAP_SINGLE, 6737 },
	{ 0x7761, 0x7761, PDF_CMAP_SINGLE, 5596 },
	{ 0x7763, 0x7763, PDF_CMAP_SINGLE, 4314 },
	{ 0x7766, 0x7766, PDF_CMAP_SINGLE, 4792 },
	{ 0x776b, 0x776b, PDF_CMAP_SINGLE, 7193 },
	{ 0x7779, 0x7779, PDF_CMAP_SINGLE, 4294 },
	{ 0x777e, 0x777f, PDF_CMAP_TABLE, 1744 },
	{ 0x778b, 0x778b, PDF_CMAP_SINGLE, 6999 },
	{ 0x7791, 0x7791, PDF_CMAP_SINGLE, 4756 },
	{ 0x779e, 0x779e, PDF_CMAP_SINGLE, 4682 },
	{ 0x77a5, 0x77a5, PDF_CMAP_SINGLE, 5038 },
	{ 0x77ac, 0x77ad, PDF_CMAP_TABLE, 1746 },
	{ 0x77b0, 0x77b0, PDF_CMAP_SINGLE, 3524 },
	{ 0x77b3, 0x77b3, PDF_CMAP_SINGLE, 4343 },
	{ 0x77bb, 0x77bc, PDF_CMAP_TABLE, 1748 },
	{ 0x77bf, 0x77bf, PDF_CMAP_SINGLE, 3925 },
	{ 0x77d7, 0x77d7, PDF_CMAP_SINGLE, 7244 },
	{ 0x77db, 0x77dc, PDF_CMAP_TABLE, 1750 },
	{ 0x77e2, 0x77e3, PDF_CMAP_TABLE, 1752 },
	{ 0x77e5, 0x77e5, PDF_CMAP_SINGLE, 6960 },
	{ 0x77e9, 0x77e9, PDF_CMAP_SINGLE, 3926 },
	{ 0x77ed, 0x77ef, PDF_CMAP_TABLE, 1754 },
	{ 0x77f3, 0x77f3, PDF_CMAP_SINGLE, 5413 },
	{ 0x7802, 0x7802, PDF_CMAP_SINGLE, 5277 },
	{ 0x7812, 0x7812, PDF_CMAP_SINGLE, 5203 },
	{ 0x7825, 0x7827, PDF_CMAP_TABLE, 1757 },
	{ 0x782c, 0x782c, PDF_CMAP_SINGLE, 4652 },
	{ 0x7832, 0x7832, PDF_CMAP_SINGLE, 7559 },
	{ 0x7834, 0x7834, PDF_CMAP_SINGLE, 7485 },
	{ 0x7845, 0x7845, PDF_CMAP_SINGLE, 4000 },
	{ 0x784f, 0x784f, PDF_CMAP_SINGLE, 5976 },
	{ 0x785d, 0x785d, PDF_CMAP_SINGLE, 7226 },
	{ 0x786b, 0x786c, PDF_CMAP_TABLE, 1760 },
	{ 0x786f, 0x786f, PDF_CMAP_SINGLE, 5977 },
	{ 0x787c, 0x787c, PDF_CMAP_SINGLE, 5178 },
	{ 0x7881, 0x7881, PDF_CMAP_SINGLE, 4101 },
	{ 0x7887, 0x7887, PDF_CMAP_SINGLE, 6738 },
	{ 0x788c, 0x788e, PDF_CMAP_TABLE, 1762 },
	{ 0x7891, 0x7891, PDF_CMAP_SINGLE, 5204 },
	{ 0x7897, 0x7897, PDF_CMAP_SINGLE, 6116 },
	{ 0x78a3, 0x78a3, PDF_CMAP_SINGLE, 3504 },
	{ 0x78a7, 0x78a7, PDF_CMAP_SINGLE, 5027 },
	{ 0x78a9, 0x78a9, PDF_CMAP_SINGLE, 5414 },
	{ 0x78ba, 0x78bb, PDF_CMAP_RANGE, 7879 },
	{ 0x78bc, 0x78bc, PDF_CMAP_SINGLE, 4659 },
	{ 0x78c1, 0x78c1, PDF_CMAP_SINGLE, 6499 },
	{ 0x78c5, 0x78c5, PDF_CMAP_SINGLE, 4956 },
	{ 0x78ca, 0x78cb, PDF_CMAP_TABLE, 1765 },
	{ 0x78ce, 0x78ce, PDF_CMAP_SINGLE, 3719 },
	{ 0x78d0, 0x78d0, PDF_CMAP_SINGLE, 4921 },
	{ 0x78e8, 0x78e8, PDF_CMAP_SINGLE, 4660 },
	{ 0x78ec, 0x78ec, PDF_CMAP_SINGLE, 3688 },
	{ 0x78ef, 0x78ef, PDF_CMAP_SINGLE, 4102 },
	{ 0x78f5, 0x78f5, PDF_CMAP_SINGLE, 3491 },
	{ 0x78fb, 0x78fb, PDF_CMAP_SINGLE, 4922 },
	{ 0x7901, 0x7901, PDF_CMAP_SINGLE, 7227 },
	{ 0x790e, 0x790e, PDF_CMAP_SINGLE, 7228 },
	{ 0x7916, 0x7916, PDF_CMAP_SINGLE, 5940 },
	{ 0x792a, 0x792c, PDF_CMAP_TABLE, 1767 },
	{ 0x793a, 0x793a, PDF_CMAP_SINGLE, 5710 },
	{ 0x793e, 0x793e, PDF_CMAP_SINGLE, 5278 },
	{ 0x7940, 0x7941, PDF_CMAP_TABLE, 1770 },
	{ 0x7947, 0x7948, PDF_CMAP_RANGE, 4104 },
	{ 0x7949, 0x7949, PDF_CMAP_SINGLE, 6962 },
	{ 0x7950, 0x7950, PDF_CMAP_SINGLE, 6214 },
	{ 0x7956, 0x7957, PDF_CMAP_TABLE, 1772 },
	{ 0x795a, 0x795e, PDF_CMAP_TABLE, 1774 },
	{ 0x7960, 0x7960, PDF_CMAP_SINGLE, 5280 },
	{ 0x7965, 0x7965, PDF_CMAP_SINGLE, 5353 },
	{ 0x7968, 0x7968, PDF_CMAP_SINGLE, 7584 },
	{ 0x796d, 0x796d, PDF_CMAP_SINGLE, 6769 },
	{ 0x797a, 0x797a, PDF_CMAP_SINGLE, 4106 },
	{ 0x797f, 0x797f, PDF_CMAP_SINGLE, 4529 },
	{ 0x7981, 0x7981, PDF_CMAP_SINGLE, 4044 },
	{ 0x798d, 0x798f, PDF_CMAP_TABLE, 1779 },
	{ 0x7991, 0x7991, PDF_CMAP_SINGLE, 6215 },
	{ 0x79a6, 0x79a7, PDF_CMAP_TABLE, 1782 },
	{ 0x79aa, 0x79aa, PDF_CMAP_SINGLE, 5434 },
	{ 0x79ae, 0x79ae, PDF_CMAP_SINGLE, 4508 },
	{ 0x79b1, 0x79b1, PDF_CMAP_SINGLE, 4295 },
	{ 0x79b3, 0x79b3, PDF_CMAP_SINGLE, 5894 },
	{ 0x79b9, 0x79b9, PDF_CMAP_SINGLE, 6216 },
	{ 0x79bd, 0x79c1, PDF_CMAP_TABLE, 1784 },
	{ 0x79c9, 0x79cb, PDF_CMAP_TABLE, 1789 },
	{ 0x79d1, 0x79d2, PDF_CMAP_TABLE, 1792 },
	{ 0x79d5, 0x79d5, PDF_CMAP_SINGLE, 5205 },
	{ 0x79d8, 0x79d8, PDF_CMAP_SINGLE, 5206 },
	{ 0x79df, 0x79df, PDF_CMAP_SINGLE, 6808 },
	{ 0x79e4, 0x79e4, PDF_CMAP_SINGLE, 7377 },
	{ 0x79e6, 0x79e7, PDF_CMAP_TABLE, 1794 },
	{ 0x79e9, 0x79e9, PDF_CMAP_SINGLE, 7023 },
	{ 0x79fb, 0x79fb, PDF_CMAP_SINGLE, 6426 },
	{ 0x7a00, 0x7a00, PDF_CMAP_SINGLE, 8053 },
	{ 0x7a05, 0x7a05, PDF_CMAP_SINGLE, 5496 },
	{ 0x7a08, 0x7a08, PDF_CMAP_SINGLE, 3492 },
	{ 0x7a0b, 0x7a0b, PDF_CMAP_SINGLE, 6740 },
	{ 0x7a0d, 0x7a0d, PDF_CMAP_SINGLE, 7230 },
	{ 0x7a14, 0x7a14, PDF_CMAP_SINGLE, 6474 },
	{ 0x7a17, 0x7a17, PDF_CMAP_SINGLE, 7510 },
	{ 0x7a19, 0x7a1a, PDF_CMAP_TABLE, 1796 },
	{ 0x7a1c, 0x7a1c, PDF_CMAP_SINGLE, 4609 },
	{ 0x7a1f, 0x7a20, PDF_CMAP_TABLE, 1798 },
	{ 0x7a2e, 0x7a2e, PDF_CMAP_SINGLE, 6845 },
	{ 0x7a31, 0x7a31, PDF_CMAP_SINGLE, 7378 },
	{ 0x7a36, 0x7a37, PDF_CMAP_TABLE, 1800 },
	{ 0x7a3b, 0x7a3d, PDF_CMAP_TABLE, 1802 },
	{ 0x7a3f, 0x7a40, PDF_CMAP_TABLE, 1805 },
	{ 0x7a46, 0x7a46, PDF_CMAP_SINGLE, 4793 },
	{ 0x7a49, 0x7a49, PDF_CMAP_SINGLE, 7350 },
	{ 0x7a4d, 0x7a4e, PDF_CMAP_TABLE, 1807 },
	{ 0x7a57, 0x7a57, PDF_CMAP_SINGLE, 5598 },
	{ 0x7a61, 0x7a62, PDF_CMAP_TABLE, 1809 },
	{ 0x7a69, 0x7a69, PDF_CMAP_SINGLE, 6086 },
	{ 0x7a6b, 0x7a6b, PDF_CMAP_SINGLE, 7881 },
	{ 0x7a70, 0x7a70, PDF_CMAP_SINGLE, 5895 },
	{ 0x7a74, 0x7a74, PDF_CMAP_SINGLE, 7755 },
	{ 0x7a76, 0x7a76, PDF_CMAP_SINGLE, 3927 },
	{ 0x7a79, 0x7a7a, PDF_CMAP_TABLE, 1811 },
	{ 0x7a7d, 0x7a7d, PDF_CMAP_SINGLE, 6741 },
	{ 0x7a7f, 0x7a7f, PDF_CMAP_SINGLE, 7157 },
	{ 0x7a81, 0x7a81, PDF_CMAP_SINGLE, 4330 },
	{ 0x7a84, 0x7a84, PDF_CMAP_SINGLE, 7058 },
	{ 0x7a88, 0x7a88, PDF_CMAP_SINGLE, 6156 },
	{ 0x7a92, 0x7a93, PDF_CMAP_TABLE, 1813 },
	{ 0x7a95, 0x7a95, PDF_CMAP_SINGLE, 6810 },
	{ 0x7a98, 0x7a98, PDF_CMAP_SINGLE, 3954 },
	{ 0x7a9f, 0x7a9f, PDF_CMAP_SINGLE, 3962 },
	{ 0x7aa9, 0x7aaa, PDF_CMAP_RANGE, 6101 },
	{ 0x7aae, 0x7aaf, PDF_CMAP_TABLE, 1815 },
	{ 0x7aba, 0x7aba, PDF_CMAP_SINGLE, 4001 },
	{ 0x7ac4, 0x7ac5, PDF_CMAP_TABLE, 1817 },
	{ 0x7ac7, 0x7ac7, PDF_CMAP_SINGLE, 4353 },
	{ 0x7aca, 0x7acb, PDF_CMAP_TABLE, 1819 },
	{ 0x7ad7, 0x7ad7, PDF_CMAP_SINGLE, 4809 },
	{ 0x7ad9, 0x7ad9, PDF_CMAP_SINGLE, 7089 },
	{ 0x7add, 0x7add, PDF_CMAP_SINGLE, 5054 },
	{ 0x7adf, 0x7ae0, PDF_CMAP_TABLE, 1821 },
	{ 0x7ae3, 0x7ae3, PDF_CMAP_SINGLE, 6915 },
	{ 0x7ae5, 0x7ae5, PDF_CMAP_SINGLE, 4344 },
	{ 0x7aea, 0x7aea, PDF_CMAP_SINGLE, 5599 },
	{ 0x7aed, 0x7aed, PDF_CMAP_SINGLE, 3505 },
	{ 0x7aef, 0x7aef, PDF_CMAP_SINGLE, 4205 },
	{ 0x7af6, 0x7af6, PDF_CMAP_SINGLE, 3690 },
	{ 0x7af9, 0x7afa, PDF_CMAP_TABLE, 1823 },
	{ 0x7aff, 0x7aff, PDF_CMAP_SINGLE, 3493 },
	{ 0x7b0f, 0x7b0f, PDF_CMAP_SINGLE, 7852 },
	{ 0x7b11, 0x7b11, PDF_CMAP_SINGLE, 5522 },
	{ 0x7b19, 0x7b19, PDF_CMAP_SINGLE, 5372 },
	{ 0x7b1b, 0x7b1b, PDF_CMAP_SINGLE, 6629 },
	{ 0x7b1e, 0x7b1e, PDF_CMAP_SINGLE, 7439 },
	{ 0x7b20, 0x7b20, PDF_CMAP_SINGLE, 4654 },
	{ 0x7b26, 0x7b26, PDF_CMAP_SINGLE, 5130 },
	{ 0x7b2c, 0x7b2d, PDF_CMAP_TABLE, 1825 },
	{ 0x7b39, 0x7b39, PDF_CMAP_SINGLE, 5497 },
	{ 0x7b46, 0x7b46, PDF_CMAP_SINGLE, 7611 },
	{ 0x7b49, 0x7b49, PDF_CMAP_SINGLE, 4369 },
	{ 0x7b4b, 0x7b4d, PDF_CMAP_TABLE, 1827 },
	{ 0x7b4f, 0x7b52, PDF_CMAP_TABLE, 1830 },
	{ 0x7b54, 0x7b54, PDF_CMAP_SINGLE, 4236 },
	{ 0x7b56, 0x7b56, PDF_CMAP_SINGLE, 7128 },
	{ 0x7b60, 0x7b60, PDF_CMAP_SINGLE, 4012 },
	{ 0x7b6c, 0x7b6c, PDF_CMAP_SINGLE, 5486 },
	{ 0x7b6e, 0x7b6e, PDF_CMAP_SINGLE, 5390 },
	{ 0x7b75, 0x7b75, PDF_CMAP_SINGLE, 5978 },
	{ 0x7b7d, 0x7b7d, PDF_CMAP_SINGLE, 6073 },
	{ 0x7b87, 0x7b87, PDF_CMAP_SINGLE, 3574 },
	{ 0x7b8b, 0x7b8b, PDF_CMAP_SINGLE, 6669 },
	{ 0x7b8f, 0x7b8f, PDF_CMAP_SINGLE, 6588 },
	{ 0x7b94, 0x7b95, PDF_CMAP_TABLE, 1834 },
	{ 0x7b97, 0x7b97, PDF_CMAP_SINGLE, 5311 },
	{ 0x7b9a, 0x7b9a, PDF_CMAP_SINGLE, 7051 },
	{ 0x7b9d, 0x7b9d, PDF_CMAP_SINGLE, 3656 },
	{ 0x7ba1, 0x7ba1, PDF_CMAP_SINGLE, 3828 },
	{ 0x7bad, 0x7bad, PDF_CMAP_SINGLE, 6670 },
	{ 0x7bb1, 0x7bb1, PDF_CMAP_SINGLE, 5354 },
	{ 0x7bb4, 0x7bb4, PDF_CMAP_SINGLE, 6530 },
	{ 0x7bb8, 0x7bb8, PDF_CMAP_SINGLE, 6606 },
	{ 0x7bc0, 0x7bc1, PDF_CMAP_TABLE, 1836 },
	{ 0x7bc4, 0x7bc4, PDF_CMAP_SINGLE, 5016 },
	{ 0x7bc6, 0x7bc7, PDF_CMAP_TABLE, 1838 },
	{ 0x7bc9, 0x7bc9, PDF_CMAP_SINGLE, 7293 },
	{ 0x7bd2, 0x7bd2, PDF_CMAP_SINGLE, 5729 },
	{ 0x7be0, 0x7be0, PDF_CMAP_SINGLE, 5523 },
	{ 0x7be4, 0x7be4, PDF_CMAP_SINGLE, 4316 },
	{ 0x7be9, 0x7be9, PDF_CMAP_SINGLE, 5282 },
	{ 0x7c07, 0x7c07, PDF_CMAP_SINGLE, 6829 },
	{ 0x7c12, 0x7c12, PDF_CMAP_SINGLE, 7068 },
	{ 0x7c1e, 0x7c1e, PDF_CMAP_SINGLE, 4206 },
	{ 0x7c21, 0x7c21, PDF_CMAP_SINGLE, 3494 },
	{ 0x7c27, 0x7c27, PDF_CMAP_SINGLE, 7922 },
	{ 0x7c2a, 0x7c2b, PDF_CMAP_TABLE, 1840 },
	{ 0x7c3d, 0x7c3f, PDF_CMAP_TABLE, 1842 },
	{ 0x7c43, 0x7c43, PDF_CMAP_SINGLE, 4404 },
	{ 0x7c4c, 0x7c4d, PDF_CMAP_TABLE, 1845 },
	{ 0x7c60, 0x7c60, PDF_CMAP_SINGLE, 4541 },
	{ 0x7c64, 0x7c64, PDF_CMAP_SINGLE, 7184 },
	{ 0x7c6c, 0x7c6c, PDF_CMAP_SINGLE, 4628 },
	{ 0x7c73, 0x7c73, PDF_CMAP_SINGLE, 4863 },
	{ 0x7c83, 0x7c83, PDF_CMAP_SINGLE, 5207 },
	{ 0x7c89, 0x7c89, PDF_CMAP_SINGLE, 5165 },
	{ 0x7c92, 0x7c92, PDF_CMAP_SINGLE, 4655 },
	{ 0x7c95, 0x7c95, PDF_CMAP_SINGLE, 4897 },
	{ 0x7c97, 0x7c98, PDF_CMAP_TABLE, 1847 },
	{ 0x7c9f, 0x7c9f, PDF_CMAP_SINGLE, 5541 },
	{ 0x7ca5, 0x7ca5, PDF_CMAP_SINGLE, 6901 },
	{ 0x7ca7, 0x7ca7, PDF_CMAP_SINGLE, 6556 },
	{ 0x7cae, 0x7cae, PDF_CMAP_SINGLE, 4433 },
	{ 0x7cb1, 0x7cb3, PDF_CMAP_TABLE, 1849 },
	{ 0x7cb9, 0x7cb9, PDF_CMAP_SINGLE, 5600 },
	{ 0x7cbe, 0x7cbe, PDF_CMAP_SINGLE, 6742 },
	{ 0x7cca, 0x7cca, PDF_CMAP_SINGLE, 7827 },
	{ 0x7cd6, 0x7cd6, PDF_CMAP_SINGLE, 4247 },
	{ 0x7cde, 0x7ce0, PDF_CMAP_TABLE, 1852 },
	{ 0x7ce7, 0x7ce7, PDF_CMAP_SINGLE, 4435 },
	{ 0x7cfb, 0x7cfb, PDF_CMAP_SINGLE, 3721 },
	{ 0x7cfe, 0x7cfe, PDF_CMAP_SINGLE, 4003 },
	{ 0x7d00, 0x7d00, PDF_CMAP_SINGLE, 4108 },
	{ 0x7d02, 0x7d02, PDF_CMAP_SINGLE, 6885 },
	{ 0x7d04, 0x7d08, PDF_CMAP_TABLE, 1855 },
	{ 0x7d0a, 0x7d0b, PDF_CMAP_RANGE, 4842 },
	{ 0x7d0d, 0x7d0d, PDF_CMAP_SINGLE, 4150 },
	{ 0x7d10, 0x7d10, PDF_CMAP_SINGLE, 4184 },
	{ 0x7d14, 0x7d14, PDF_CMAP_SINGLE, 5655 },
	{ 0x7d17, 0x7d1b, PDF_CMAP_TABLE, 1860 },
	{ 0x7d20, 0x7d22, PDF_CMAP_TABLE, 1865 },
	{ 0x7d2b, 0x7d2c, PDF_CMAP_TABLE, 1868 },
	{ 0x7d2e, 0x7d30, PDF_CMAP_TABLE, 1870 },
	{ 0x7d33, 0x7d33, PDF_CMAP_SINGLE, 5747 },
	{ 0x7d35, 0x7d35, PDF_CMAP_SINGLE, 6607 },
	{ 0x7d39, 0x7d3a, PDF_CMAP_TABLE, 1873 },
	{ 0x7d42, 0x7d46, PDF_CMAP_TABLE, 1875 },
	{ 0x7d50, 0x7d50, PDF_CMAP_SINGLE, 3651 },
	{ 0x7d5e, 0x7d5e, PDF_CMAP_SINGLE, 3882 },
	{ 0x7d61, 0x7d62, PDF_CMAP_TABLE, 1880 },
	{ 0x7d66, 0x7d66, PDF_CMAP_SINGLE, 4057 },
	{ 0x7d68, 0x7d68, PDF_CMAP_SINGLE, 6371 },
	{ 0x7d6a, 0x7d6a, PDF_CMAP_SINGLE, 6454 },
	{ 0x7d6e, 0x7d6e, PDF_CMAP_SINGLE, 5391 },
	{ 0x7d71, 0x7d73, PDF_CMAP_TABLE, 1882 },
	{ 0x7d76, 0x7d76, PDF_CMAP_SINGLE, 6691 },
	{ 0x7d79, 0x7d79, PDF_CMAP_SINGLE, 3641 },
	{ 0x7d7f, 0x7d7f, PDF_CMAP_SINGLE, 3928 },
	{ 0x7d8e, 0x7d8f, PDF_CMAP_TABLE, 1885 },
	{ 0x7d93, 0x7d93, PDF_CMAP_SINGLE, 3692 },
	{ 0x7d9c, 0x7d9c, PDF_CMAP_SINGLE, 6847 },
	{ 0x7da0, 0x7da0, PDF_CMAP_SINGLE, 4530 },
	{ 0x7da2, 0x7da2, PDF_CMAP_SINGLE, 6887 },
	{ 0x7dac, 0x7dad, PDF_CMAP_TABLE, 1887 },
	{ 0x7db1, 0x7db2, PDF_CMAP_TABLE, 1889 },
	{ 0x7db4, 0x7db5, PDF_CMAP_TABLE, 1891 },
	{ 0x7db8, 0x7db8, PDF_CMAP_SINGLE, 4598 },
	{ 0x7dba, 0x7dbb, PDF_CMAP_TABLE, 1893 },
	{ 0x7dbd, 0x7dbf, PDF_CMAP_TABLE, 1895 },
	{ 0x7dc7, 0x7dc7, PDF_CMAP_SINGLE, 7351 },
	{ 0x7dca, 0x7dcb, PDF_CMAP_TABLE, 1898 },
	{ 0x7dd6, 0x7dd6, PDF_CMAP_SINGLE, 5392 },
	{ 0x7dd8, 0x7dd8, PDF_CMAP_SINGLE, 7658 },
	{ 0x7dda, 0x7dda, PDF_CMAP_SINGLE, 5435 },
	{ 0x7ddd, 0x7dde, PDF_CMAP_TABLE, 1900 },
	{ 0x7de0, 0x7de1, PDF_CMAP_TABLE, 1902 },
	{ 0x7de3, 0x7de3, PDF_CMAP_SINGLE, 5979 },
	{ 0x7de8, 0x7de9, PDF_CMAP_TABLE, 1904 },
	{ 0x7dec, 0x7dec, PDF_CMAP_SINGLE, 4743 },
	{ 0x7def, 0x7def, PDF_CMAP_SINGLE, 6297 },
	{ 0x7df4, 0x7df4, PDF_CMAP_SINGLE, 4471 },
	{ 0x7dfb, 0x7dfb, PDF_CMAP_SINGLE, 7352 },
	{ 0x7e09, 0x7e0a, PDF_CMAP_TABLE, 1906 },
	{ 0x7e15, 0x7e15, PDF_CMAP_SINGLE, 6087 },
	{ 0x7e1b, 0x7e1b, PDF_CMAP_SINGLE, 4898 },
	{ 0x7e1d, 0x7e1f, PDF_CMAP_TABLE, 1908 },
	{ 0x7e21, 0x7e21, PDF_CMAP_SINGLE, 6581 },
	{ 0x7e23, 0x7e23, PDF_CMAP_SINGLE, 7748 },
	{ 0x7e2b, 0x7e2b, PDF_CMAP_SINGLE, 5103 },
	{ 0x7e2e, 0x7e2f, PDF_CMAP_TABLE, 1911 },
	{ 0x7e31, 0x7e31, PDF_CMAP_SINGLE, 6848 },
	{ 0x7e37, 0x7e37, PDF_CMAP_SINGLE, 4572 },
	{ 0x7e3d, 0x7e3e, PDF_CMAP_TABLE, 1913 },
	{ 0x7e41, 0x7e41, PDF_CMAP_SINGLE, 5001 },
	{ 0x7e43, 0x7e43, PDF_CMAP_SINGLE, 5179 },
	{ 0x7e46, 0x7e47, PDF_CMAP_TABLE, 1915 },
	{ 0x7e52, 0x7e52, PDF_CMAP_SINGLE, 6938 },
	{ 0x7e54, 0x7e55, PDF_CMAP_TABLE, 1917 },
	{ 0x7e5e, 0x7e5e, PDF_CMAP_SINGLE, 6159 },
	{ 0x7e61, 0x7e61, PDF_CMAP_SINGLE, 5603 },
	{ 0x7e69, 0x7e6b, PDF_CMAP_TABLE, 1919 },
	{ 0x7e6d, 0x7e6d, PDF_CMAP_SINGLE, 3642 },
	{ 0x7e70, 0x7e70, PDF_CMAP_SINGLE, 6814 },
	{ 0x7e79, 0x7e79, PDF_CMAP_SINGLE, 5952 },
	{ 0x7e7c, 0x7e7c, PDF_CMAP_SINGLE, 3723 },
	{ 0x7e82, 0x7e82, PDF_CMAP_SINGLE, 7069 },
	{ 0x7e8c, 0x7e8c, PDF_CMAP_SINGLE, 5542 },
	{ 0x7e8f, 0x7e8f, PDF_CMAP_SINGLE, 6672 },
	{ 0x7e93, 0x7e93, PDF_CMAP_SINGLE, 6023 },
	{ 0x7e96, 0x7e96, PDF_CMAP_SINGLE, 5466 },
	{ 0x7e98, 0x7e98, PDF_CMAP_SINGLE, 7071 },
	{ 0x7e9b, 0x7e9c, PDF_CMAP_TABLE, 1922 },
	{ 0x7f36, 0x7f36, PDF_CMAP_SINGLE, 5132 },
	{ 0x7f38, 0x7f38, PDF_CMAP_SINGLE, 7681 },
	{ 0x7f3a, 0x7f3a, PDF_CMAP_SINGLE, 3652 },
	{ 0x7f4c, 0x7f4c, PDF_CMAP_SINGLE, 5859 },
	{ 0x7f50, 0x7f50, PDF_CMAP_SINGLE, 3829 },
	{ 0x7f54, 0x7f55, PDF_CMAP_TABLE, 1924 },
	{ 0x7f6a, 0x7f6b, PDF_CMAP_TABLE, 1926 },
	{ 0x7f6e, 0x7f6e, PDF_CMAP_SINGLE, 7353 },
	{ 0x7f70, 0x7f70, PDF_CMAP_SINGLE, 5007 },
	{ 0x7f72, 0x7f72, PDF_CMAP_SINGLE, 5393 },
	{ 0x7f75, 0x7f75, PDF_CMAP_SINGLE, 4717 },
	{ 0x7f77, 0x7f77, PDF_CMAP_SINGLE, 7486 },
	{ 0x7f79, 0x7f79, PDF_CMAP_SINGLE, 4629 },
	{ 0x7f85, 0x7f85, PDF_CMAP_SINGLE, 4377 },
	{ 0x7f88, 0x7f88, PDF_CMAP_SINGLE, 4110 },
	{ 0x7f8a, 0x7f8a, PDF_CMAP_SINGLE, 5896 },
	{ 0x7f8c, 0x7f8c, PDF_CMAP_SINGLE, 3551 },
	{ 0x7f8e, 0x7f8e, PDF_CMAP_SINGLE, 4864 },
	{ 0x7f94, 0x7f94, PDF_CMAP_SINGLE, 3750 },
	{ 0x7f9a, 0x7f9a, PDF_CMAP_SINGLE, 4497 },
	{ 0x7f9e, 0x7f9e, PDF_CMAP_SINGLE, 5604 },
	{ 0x7fa4, 0x7fa4, PDF_CMAP_SINGLE, 3955 },
	{ 0x7fa8, 0x7fa9, PDF_CMAP_TABLE, 1928 },
	{ 0x7fb2, 0x7fb2, PDF_CMAP_SINGLE, 8054 },
	{ 0x7fb8, 0x7fb9, PDF_CMAP_TABLE, 1930 },
	{ 0x7fbd, 0x7fbd, PDF_CMAP_SINGLE, 6218 },
	{ 0x7fc1, 0x7fc1, PDF_CMAP_SINGLE, 6095 },
	{ 0x7fc5, 0x7fc5, PDF_CMAP_SINGLE, 5711 },
	{ 0x7fca, 0x7fca, PDF_CMAP_SINGLE, 6439 },
	{ 0x7fcc, 0x7fcc, PDF_CMAP_SINGLE, 6440 },
	{ 0x7fce, 0x7fce, PDF_CMAP_SINGLE, 4498 },
	{ 0x7fd2, 0x7fd2, PDF_CMAP_SINGLE, 5679 },
	{ 0x7fd4, 0x7fd5, PDF_CMAP_TABLE, 1932 },
	{ 0x7fdf, 0x7fe1, PDF_CMAP_TABLE, 1934 },
	{ 0x7fe9, 0x7fe9, PDF_CMAP_SINGLE, 7524 },
	{ 0x7feb, 0x7feb, PDF_CMAP_SINGLE, 6118 },
	{ 0x7ff0, 0x7ff0, PDF_CMAP_SINGLE, 7644 },
	{ 0x7ff9, 0x7ff9, PDF_CMAP_SINGLE, 3883 },
	{ 0x7ffc, 0x7ffc, PDF_CMAP_SINGLE, 6441 },
	{ 0x8000, 0x8001, PDF_CMAP_TABLE, 1937 },
	{ 0x8003, 0x8003, PDF_CMAP_SINGLE, 3751 },
	{ 0x8005, 0x8006, PDF_CMAP_TABLE, 1939 },
	{ 0x8009, 0x8009, PDF_CMAP_SINGLE, 3929 },
	{ 0x800c, 0x800c, PDF_CMAP_SINGLE, 6427 },
	{ 0x8010, 0x8010, PDF_CMAP_SINGLE, 4158 },
	{ 0x8015, 0x8015, PDF_CMAP_SINGLE, 3693 },
	{ 0x8017, 0x8018, PDF_CMAP_TABLE, 1941 },
	{ 0x802d, 0x802d, PDF_CMAP_SINGLE, 4112 },
	{ 0x8033, 0x8033, PDF_CMAP_SINGLE, 6428 },
	{ 0x8036, 0x8036, PDF_CMAP_SINGLE, 5870 },
	{ 0x803d, 0x803d, PDF_CMAP_SINGLE, 7422 },
	{ 0x803f, 0x803f, PDF_CMAP_SINGLE, 3694 },
	{ 0x8043, 0x8043, PDF_CMAP_SINGLE, 4227 },
	{ 0x8046, 0x8046, PDF_CMAP_SINGLE, 4499 },
	{ 0x804a, 0x804a, PDF_CMAP_SINGLE, 4559 },
	{ 0x8056, 0x8056, PDF_CMAP_SINGLE, 5487 },
	{ 0x8058, 0x8058, PDF_CMAP_SINGLE, 5239 },
	{ 0x805a, 0x805a, PDF_CMAP_SINGLE, 7323 },
	{ 0x805e, 0x805e, PDF_CMAP_SINGLE, 4844 },
	{ 0x806f, 0x8070, PDF_CMAP_TABLE, 1943 },
	{ 0x8072, 0x8073, PDF_CMAP_TABLE, 1945 },
	{ 0x8077, 0x8077, PDF_CMAP_SINGLE, 6979 },
	{ 0x807d, 0x807f, PDF_CMAP_TABLE, 1947 },
	{ 0x8084, 0x8087, PDF_CMAP_TABLE, 1950 },
	{ 0x8089, 0x8089, PDF_CMAP_SINGLE, 6357 },
	{ 0x808b, 0x808c, PDF_CMAP_TABLE, 1954 },
	{ 0x8096, 0x8096, PDF_CMAP_SINGLE, 7231 },
	{ 0x809b, 0x809b, PDF_CMAP_SINGLE, 7682 },
	{ 0x809d, 0x809d, PDF_CMAP_SINGLE, 3495 },
	{ 0x80a1, 0x80a2, PDF_CMAP_TABLE, 1956 },
	{ 0x80a5, 0x80a5, PDF_CMAP_SINGLE, 5210 },
	{ 0x80a9, 0x80aa, PDF_CMAP_TABLE, 1958 },
	{ 0x80af, 0x80af, PDF_CMAP_SINGLE, 4061 },
	{ 0x80b1, 0x80b2, PDF_CMAP_TABLE, 1960 },
	{ 0x80b4, 0x80b4, PDF_CMAP_SINGLE, 7963 },
	{ 0x80ba, 0x80ba, PDF_CMAP_SINGLE, 7540 },
	{ 0x80c3, 0x80c4, PDF_CMAP_TABLE, 1962 },
	{ 0x80cc, 0x80cc, PDF_CMAP_SINGLE, 4979 },
	{ 0x80ce, 0x80ce, PDF_CMAP_SINGLE, 7440 },
	{ 0x80da, 0x80db, PDF_CMAP_TABLE, 1964 },
	{ 0x80de, 0x80de, PDF_CMAP_SINGLE, 7560 },
	{ 0x80e1, 0x80e1, PDF_CMAP_SINGLE, 7829 },
	{ 0x80e4, 0x80e5, PDF_CMAP_TABLE, 1966 },
	{ 0x80f1, 0x80f1, PDF_CMAP_SINGLE, 3850 },
	{ 0x80f4, 0x80f4, PDF_CMAP_SINGLE, 4345 },
	{ 0x80f8, 0x80f8, PDF_CMAP_SINGLE, 8017 },
	{ 0x80fd, 0x80fd, PDF_CMAP_SINGLE, 4185 },
	{ 0x8102, 0x8102, PDF_CMAP_SINGLE, 6966 },
	{ 0x8105, 0x8108, PDF_CMAP_TABLE, 1968 },
	{ 0x810a, 0x810a, PDF_CMAP_SINGLE, 7144 },
	{ 0x8118, 0x8118, PDF_CMAP_SINGLE, 6119 },
	{ 0x811a, 0x811b, PDF_CMAP_TABLE, 1972 },
	{ 0x8123, 0x8123, PDF_CMAP_SINGLE, 5656 },
	{ 0x8129, 0x8129, PDF_CMAP_SINGLE, 5605 },
	{ 0x812b, 0x812b, PDF_CMAP_SINGLE, 7419 },
	{ 0x812f, 0x812f, PDF_CMAP_SINGLE, 7561 },
	{ 0x8139, 0x8139, PDF_CMAP_SINGLE, 7110 },
	{ 0x813e, 0x813e, PDF_CMAP_SINGLE, 5211 },
	{ 0x814b, 0x814b, PDF_CMAP_SINGLE, 5856 },
	{ 0x814e, 0x814e, PDF_CMAP_SINGLE, 5748 },
	{ 0x8150, 0x8151, PDF_CMAP_RANGE, 5133 },
	{ 0x8154, 0x8155, PDF_CMAP_TABLE, 1974 },
	{ 0x8165, 0x8166, PDF_CMAP_TABLE, 1976 },
	{ 0x816b, 0x816b, PDF_CMAP_SINGLE, 6849 },
	{ 0x8170, 0x8171, PDF_CMAP_TABLE, 1978 },
	{ 0x8178, 0x817a, PDF_CMAP_TABLE, 1980 },
	{ 0x817f, 0x8180, PDF_CMAP_TABLE, 1983 },
	{ 0x8188, 0x8188, PDF_CMAP_SINGLE, 3634 },
	{ 0x818a, 0x818a, PDF_CMAP_SINGLE, 4899 },
	{ 0x818f, 0x818f, PDF_CMAP_SINGLE, 3753 },
	{ 0x819a, 0x819a, PDF_CMAP_SINGLE, 5135 },
	{ 0x819c, 0x819d, PDF_CMAP_TABLE, 1985 },
	{ 0x81a0, 0x81a0, PDF_CMAP_SINGLE, 3884 },
	{ 0x81a3, 0x81a3, PDF_CMAP_SINGLE, 7025 },
	{ 0x81a8, 0x81a8, PDF_CMAP_SINGLE, 7516 },
	{ 0x81b3, 0x81b3, PDF_CMAP_SINGLE, 5439 },
	{ 0x81b5, 0x81b5, PDF_CMAP_SINGLE, 7313 },
	{ 0x81ba, 0x81ba, PDF_CMAP_SINGLE, 6392 },
	{ 0x81bd, 0x81c0, PDF_CMAP_TABLE, 1987 },
	{ 0x81c2, 0x81c2, PDF_CMAP_SINGLE, 5212 },
	{ 0x81c6, 0x81c6, PDF_CMAP_SINGLE, 5916 },
	{ 0x81cd, 0x81cd, PDF_CMAP_SINGLE, 6771 },
	{ 0x81d8, 0x81d8, PDF_CMAP_SINGLE, 4410 },
	{ 0x81df, 0x81df, PDF_CMAP_SINGLE, 6558 },
	{ 0x81e3, 0x81e3, PDF_CMAP_SINGLE, 5749 },
	{ 0x81e5, 0x81e5, PDF_CMAP_SINGLE, 6103 },
	{ 0x81e7, 0x81e8, PDF_CMAP_TABLE, 1991 },
	{ 0x81ea, 0x81ea, PDF_CMAP_SINGLE, 6502 },
	{ 0x81ed, 0x81ed, PDF_CMAP_SINGLE, 7325 },
	{ 0x81f3, 0x81f4, PDF_CMAP_TABLE, 1993 },
	{ 0x81fa, 0x81fc, PDF_CMAP_TABLE, 1995 },
	{ 0x81fe, 0x81fe, PDF_CMAP_SINGLE, 6341 },
	{ 0x8205, 0x8205, PDF_CMAP_SINGLE, 3931 },
	{ 0x8207, 0x8208, PDF_CMAP_TABLE, 1998 },
	{ 0x820a, 0x820a, PDF_CMAP_SINGLE, 3932 },
	{ 0x820c, 0x820d, PDF_CMAP_TABLE, 2000 },
	{ 0x8212, 0x8212, PDF_CMAP_SINGLE, 5395 },
	{ 0x821b, 0x821c, PDF_CMAP_TABLE, 2002 },
	{ 0x821e, 0x821f, PDF_CMAP_TABLE, 2004 },
	{ 0x8221, 0x8221, PDF_CMAP_SINGLE, 3553 },
	{ 0x822a, 0x822c, PDF_CMAP_TABLE, 2006 },
	{ 0x8235, 0x8237, PDF_CMAP_TABLE, 2009 },
	{ 0x8239, 0x8239, PDF_CMAP_SINGLE, 5440 },
	{ 0x8240, 0x8240, PDF_CMAP_SINGLE, 5136 },
	{ 0x8245, 0x8245, PDF_CMAP_SINGLE, 5942 },
	{ 0x8247, 0x8247, PDF_CMAP_SINGLE, 6744 },
	{ 0x8259, 0x8259, PDF_CMAP_SINGLE, 7111 },
	{ 0x8264, 0x8264, PDF_CMAP_SINGLE, 6406 },
	{ 0x8266, 0x8266, PDF_CMAP_SINGLE, 7659 },
	{ 0x826e, 0x826f, PDF_CMAP_TABLE, 2012 },
	{ 0x8271, 0x8272, PDF_CMAP_TABLE, 2014 },
	{ 0x8276, 0x8276, PDF_CMAP_SINGLE, 5994 },
	{ 0x8278, 0x8278, PDF_CMAP_SINGLE, 7232 },
	{ 0x827e, 0x827e, PDF_CMAP_SINGLE, 5848 },
	{ 0x828b, 0x828b, PDF_CMAP_SINGLE, 6219 },
	{ 0x828d, 0x828e, PDF_CMAP_TABLE, 2016 },
	{ 0x8292, 0x8292, PDF_CMAP_SINGLE, 4703 },
	{ 0x8299, 0x829a, PDF_CMAP_TABLE, 2018 },
	{ 0x829d, 0x829d, PDF_CMAP_SINGLE, 6968 },
	{ 0x829f, 0x829f, PDF_CMAP_SINGLE, 5324 },
	{ 0x82a5, 0x82a6, PDF_CMAP_TABLE, 2020 },
	{ 0x82a9, 0x82a9, PDF_CMAP_SINGLE, 4046 },
	{ 0x82ac, 0x82af, PDF_CMAP_TABLE, 2022 },
	{ 0x82b1, 0x82b1, PDF_CMAP_SINGLE, 7871 },
	{ 0x82b3, 0x82b3, PDF_CMAP_SINGLE, 4961 },
	{ 0x82b7, 0x82b9, PDF_CMAP_TABLE, 2026 },
	{ 0x82bb, 0x82bd, PDF_CMAP_TABLE, 2029 },
	{ 0x82bf, 0x82bf, PDF_CMAP_SINGLE, 6482 },
	{ 0x82d1, 0x82d2, PDF_CMAP_TABLE, 2032 },
	{ 0x82d4, 0x82d5, PDF_CMAP_TABLE, 2034 },
	{ 0x82d7, 0x82d7, PDF_CMAP_SINGLE, 4810 },
	{ 0x82db, 0x82db, PDF_CMAP_SINGLE, 3455 },
	{ 0x82de, 0x82df, PDF_CMAP_TABLE, 2036 },
	{ 0x82e1, 0x82e1, PDF_CMAP_SINGLE, 6430 },
	{ 0x82e5, 0x82e7, PDF_CMAP_TABLE, 2038 },
	{ 0x82f1, 0x82f1, PDF_CMAP_SINGLE, 6024 },
	{ 0x82fd, 0x82fe, PDF_CMAP_TABLE, 2041 },
	{ 0x8301, 0x8305, PDF_CMAP_TABLE, 2043 },
	{ 0x8309, 0x8309, PDF_CMAP_SINGLE, 4693 },
	{ 0x8317, 0x8317, PDF_CMAP_SINGLE, 4757 },
	{ 0x8328, 0x8328, PDF_CMAP_SINGLE, 6503 },
	{ 0x832b, 0x832b, PDF_CMAP_SINGLE, 4704 },
	{ 0x832f, 0x832f, PDF_CMAP_SINGLE, 5083 },
	{ 0x8331, 0x8331, PDF_CMAP_SINGLE, 5606 },
	{ 0x8334, 0x8336, PDF_CMAP_TABLE, 2048 },
	{ 0x8338, 0x8339, PDF_CMAP_TABLE, 2051 },
	{ 0x8340, 0x8340, PDF_CMAP_SINGLE, 5658 },
	{ 0x8347, 0x8347, PDF_CMAP_SINGLE, 7708 },
	{ 0x8349, 0x834a, PDF_CMAP_TABLE, 2053 },
	{ 0x834f, 0x834f, PDF_CMAP_SINGLE, 6475 },
	{ 0x8351, 0x8352, PDF_CMAP_TABLE, 2055 },
	{ 0x8373, 0x8373, PDF_CMAP_SINGLE, 4354 },
	{ 0x8377, 0x8377, PDF_CMAP_SINGLE, 7624 },
	{ 0x837b, 0x837b, PDF_CMAP_SINGLE, 6633 },
	{ 0x8389, 0x838a, PDF_CMAP_TABLE, 2057 },
	{ 0x838e, 0x838e, PDF_CMAP_SINGLE, 5287 },
	{ 0x8396, 0x8396, PDF_CMAP_SINGLE, 3696 },
	{ 0x8398, 0x8398, PDF_CMAP_SINGLE, 5750 },
	{ 0x839e, 0x839e, PDF_CMAP_SINGLE, 6121 },
	{ 0x83a2, 0x83a2, PDF_CMAP_SINGLE, 7768 },
	{ 0x83a9, 0x83ab, PDF_CMAP_TABLE, 2059 },
	{ 0x83bd, 0x83bd, PDF_CMAP_SINGLE, 4705 },
	{ 0x83c1, 0x83c1, PDF_CMAP_SINGLE, 7201 },
	{ 0x83c5, 0x83c5, PDF_CMAP_SINGLE, 3830 },
	{ 0x83c9, 0x83ca, PDF_CMAP_TABLE, 2062 },
	{ 0x83cc, 0x83cc, PDF_CMAP_SINGLE, 4013 },
	{ 0x83d3, 0x83d3, PDF_CMAP_SINGLE, 3808 },
	{ 0x83d6, 0x83d6, PDF_CMAP_SINGLE, 7112 },
	{ 0x83dc, 0x83dc, PDF_CMAP_SINGLE, 7122 },
	{ 0x83e9, 0x83e9, PDF_CMAP_SINGLE, 5069 },
	{ 0x83eb, 0x83eb, PDF_CMAP_SINGLE, 4033 },
	{ 0x83ef, 0x83f2, PDF_CMAP_TABLE, 2064 },
	{ 0x83f4, 0x83f4, PDF_CMAP_SINGLE, 5827 },
	{ 0x83f9, 0x83f9, PDF_CMAP_SINGLE, 6609 },
	{ 0x83fd, 0x83fd, PDF_CMAP_SINGLE, 5639 },
	{ 0x8403, 0x8404, PDF_CMAP_TABLE, 2068 },
	{ 0x840a, 0x840a, PDF_CMAP_SINGLE, 4423 },
	{ 0x840c, 0x840e, PDF_CMAP_TABLE, 2070 },
	{ 0x8429, 0x8429, PDF_CMAP_SINGLE, 7275 },
	{ 0x842c, 0x842c, PDF_CMAP_SINGLE, 4683 },
	{ 0x8431, 0x8431, PDF_CMAP_SINGLE, 7993 },
	{ 0x8438, 0x8438, PDF_CMAP_SINGLE, 6342 },
	{ 0x843d, 0x843d, PDF_CMAP_SINGLE, 4386 },
	{ 0x8449, 0x8449, PDF_CMAP_SINGLE, 6001 },
	{ 0x8457, 0x8457, PDF_CMAP_SINGLE, 6610 },
	{ 0x845b, 0x845b, PDF_CMAP_SINGLE, 3506 },
	{ 0x8461, 0x8461, PDF_CMAP_SINGLE, 7563 },
	{ 0x8463, 0x8463, PDF_CMAP_SINGLE, 4346 },
	{ 0x8466, 0x8466, PDF_CMAP_SINGLE, 6300 },
	{ 0x846b, 0x846c, PDF_CMAP_TABLE, 2073 },
	{ 0x846f, 0x846f, PDF_CMAP_SINGLE, 5875 },
	{ 0x8475, 0x8475, PDF_CMAP_SINGLE, 4004 },
	{ 0x847a, 0x847a, PDF_CMAP_SINGLE, 6930 },
	{ 0x8490, 0x8490, PDF_CMAP_SINGLE, 5607 },
	{ 0x8494, 0x8494, PDF_CMAP_SINGLE, 5712 },
	{ 0x8499, 0x8499, PDF_CMAP_SINGLE, 4799 },
	{ 0x849c, 0x849c, PDF_CMAP_SINGLE, 5312 },
	{ 0x84a1, 0x84a1, PDF_CMAP_SINGLE, 4962 },
	{ 0x84b2, 0x84b2, PDF_CMAP_SINGLE, 7564 },
	{ 0x84b8, 0x84b8, PDF_CMAP_SINGLE, 6939 },
	{ 0x84bb, 0x84bc, PDF_CMAP_TABLE, 2075 },
	{ 0x84bf, 0x84c0, PDF_CMAP_TABLE, 2077 },
	{ 0x84c2, 0x84c2, PDF_CMAP_SINGLE, 4758 },
	{ 0x84c4, 0x84c4, PDF_CMAP_SINGLE, 7295 },
	{ 0x84c6, 0x84c6, PDF_CMAP_SINGLE, 5415 },
	{ 0x84c9, 0x84c9, PDF_CMAP_SINGLE, 6193 },
	{ 0x84cb, 0x84cb, PDF_CMAP_SINGLE, 3576 },
	{ 0x84cd, 0x84cd, PDF_CMAP_SINGLE, 5713 },
	{ 0x84d1, 0x84d1, PDF_CMAP_SINGLE, 5288 },
	{ 0x84da, 0x84da, PDF_CMAP_SINGLE, 5608 },
	{ 0x84ec, 0x84ec, PDF_CMAP_SINGLE, 5104 },
	{ 0x84ee, 0x84ee, PDF_CMAP_SINGLE, 4473 },
	{ 0x84f4, 0x84f4, PDF_CMAP_SINGLE, 5659 },
	{ 0x84fc, 0x84fc, PDF_CMAP_SINGLE, 4560 },
	{ 0x8511, 0x8511, PDF_CMAP_SINGLE, 4747 },
	{ 0x8513, 0x8514, PDF_CMAP_TABLE, 2079 },
	{ 0x8517, 0x8518, PDF_CMAP_TABLE, 2081 },
	{ 0x851a, 0x851a, PDF_CMAP_SINGLE, 6250 },
	{ 0x851e, 0x851e, PDF_CMAP_SINGLE, 4573 },
	{ 0x8521, 0x8521, PDF_CMAP_SINGLE, 7123 },
	{ 0x8523, 0x8523, PDF_CMAP_SINGLE, 6562 },
	{ 0x8525, 0x8525, PDF_CMAP_SINGLE, 7259 },
	{ 0x852c, 0x852d, PDF_CMAP_TABLE, 2083 },
	{ 0x852f, 0x852f, PDF_CMAP_SINGLE, 7004 },
	{ 0x853d, 0x853d, PDF_CMAP_SINGLE, 7541 },
	{ 0x853f, 0x853f, PDF_CMAP_SINGLE, 6301 },
	{ 0x8541, 0x8541, PDF_CMAP_SINGLE, 4229 },
	{ 0x8543, 0x8543, PDF_CMAP_SINGLE, 5002 },
	{ 0x8549, 0x8549, PDF_CMAP_SINGLE, 7235 },
	{ 0x854e, 0x854e, PDF_CMAP_SINGLE, 3885 },
	{ 0x8553, 0x8553, PDF_CMAP_SINGLE, 6245 },
	{ 0x8559, 0x8559, PDF_CMAP_SINGLE, 7796 },
	{ 0x8563, 0x8563, PDF_CMAP_SINGLE, 5660 },
	{ 0x8568, 0x856a, PDF_CMAP_TABLE, 2085 },
	{ 0x856d, 0x856d, PDF_CMAP_SINGLE, 5528 },
	{ 0x8584, 0x8584, PDF_CMAP_SINGLE, 4901 },
	{ 0x8587, 0x8587, PDF_CMAP_SINGLE, 4865 },
	{ 0x858f, 0x858f, PDF_CMAP_SINGLE, 6407 },
	{ 0x8591, 0x8591, PDF_CMAP_SINGLE, 3554 },
	{ 0x8594, 0x8594, PDF_CMAP_SINGLE, 6563 },
	{ 0x859b, 0x859b, PDF_CMAP_SINGLE, 5457 },
	{ 0x85a6, 0x85a6, PDF_CMAP_SINGLE, 7159 },
	{ 0x85a8, 0x85aa, PDF_CMAP_TABLE, 2088 },
	{ 0x85af, 0x85b0, PDF_CMAP_TABLE, 2091 },
	{ 0x85ba, 0x85ba, PDF_CMAP_SINGLE, 6772 },
	{ 0x85c1, 0x85c1, PDF_CMAP_SINGLE, 3757 },
	{ 0x85c9, 0x85c9, PDF_CMAP_SINGLE, 6505 },
	{ 0x85cd, 0x85cf, PDF_CMAP_TABLE, 2093 },
	{ 0x85d5, 0x85d5, PDF_CMAP_SINGLE, 6220 },
	{ 0x85dc, 0x85dd, PDF_CMAP_TABLE, 2096 },
	{ 0x85e4, 0x85e5, PDF_CMAP_TABLE, 2098 },
	{ 0x85e9, 0x85ea, PDF_CMAP_TABLE, 2100 },
	{ 0x85f7, 0x85f7, PDF_CMAP_SINGLE, 6611 },
	{ 0x85fa, 0x85fb, PDF_CMAP_TABLE, 2102 },
	{ 0x85ff, 0x85ff, PDF_CMAP_SINGLE, 3817 },
	{ 0x8602, 0x8602, PDF_CMAP_SINGLE, 6041 },
	{ 0x8606, 0x8607, PDF_CMAP_TABLE, 2104 },
	{ 0x860a, 0x860a, PDF_CMAP_SINGLE, 6088 },
	{ 0x8616, 0x8617, PDF_CMAP_TABLE, 2106 },
	{ 0x861a, 0x861a, PDF_CMAP_SINGLE, 5441 },
	{ 0x862d, 0x862d, PDF_CMAP_SINGLE, 4395 },
	{ 0x863f, 0x863f, PDF_CMAP_SINGLE, 4378 },
	{ 0x864e, 0x864e, PDF_CMAP_SINGLE, 7833 },
	{ 0x8650, 0x8650, PDF_CMAP_SINGLE, 7632 },
	{ 0x8654, 0x8655, PDF_CMAP_TABLE, 2108 },
	{ 0x865b, 0x865c, PDF_CMAP_TABLE, 2110 },
	{ 0x865e, 0x865f, PDF_CMAP_TABLE, 2112 },
	{ 0x8667, 0x8667, PDF_CMAP_SINGLE, 8009 },
	{ 0x8679, 0x8679, PDF_CMAP_SINGLE, 7860 },
	{ 0x868a, 0x868a, PDF_CMAP_SINGLE, 4845 },
	{ 0x868c, 0x868c, PDF_CMAP_SINGLE, 4963 },
	{ 0x8693, 0x8693, PDF_CMAP_SINGLE, 6456 },
	{ 0x86a3, 0x86a4, PDF_CMAP_TABLE, 2114 },
	{ 0x86a9, 0x86a9, PDF_CMAP_SINGLE, 7355 },
	{ 0x86c7, 0x86c7, PDF_CMAP_SINGLE, 5289 },
	{ 0x86cb, 0x86cb, PDF_CMAP_SINGLE, 4208 },
	{ 0x86d4, 0x86d4, PDF_CMAP_SINGLE, 7945 },
	{ 0x86d9, 0x86d9, PDF_CMAP_SINGLE, 6104 },
	{ 0x86db, 0x86db, PDF_CMAP_SINGLE, 6889 },
	{ 0x86df, 0x86df, PDF_CMAP_SINGLE, 3886 },
	{ 0x86e4, 0x86e4, PDF_CMAP_SINGLE, 7666 },
	{ 0x86ed, 0x86ed, PDF_CMAP_SINGLE, 7026 },
	{ 0x86fe, 0x86fe, PDF_CMAP_SINGLE, 5785 },
	{ 0x8700, 0x8700, PDF_CMAP_SINGLE, 7245 },
	{ 0x8702, 0x8703, PDF_CMAP_TABLE, 2116 },
	{ 0x8708, 0x8708, PDF_CMAP_SINGLE, 6074 },
	{ 0x8718, 0x8718, PDF_CMAP_SINGLE, 6970 },
	{ 0x871a, 0x871a, PDF_CMAP_SINGLE, 5214 },
	{ 0x871c, 0x871c, PDF_CMAP_SINGLE, 4884 },
	{ 0x874e, 0x874e, PDF_CMAP_SINGLE, 3508 },
	{ 0x8755, 0x8755, PDF_CMAP_SINGLE, 5730 },
	{ 0x8757, 0x8757, PDF_CMAP_SINGLE, 7924 },
	{ 0x875f, 0x875f, PDF_CMAP_SINGLE, 6302 },
	{ 0x8766, 0x8766, PDF_CMAP_SINGLE, 7625 },
	{ 0x8768, 0x8768, PDF_CMAP_SINGLE, 5676 },
	{ 0x8774, 0x8774, PDF_CMAP_SINGLE, 7835 },
	{ 0x8776, 0x8776, PDF_CMAP_SINGLE, 6703 },
	{ 0x8778, 0x8778, PDF_CMAP_SINGLE, 6105 },
	{ 0x8782, 0x8782, PDF_CMAP_SINGLE, 4418 },
	{ 0x878d, 0x878d, PDF_CMAP_SINGLE, 6372 },
	{ 0x879f, 0x879f, PDF_CMAP_SINGLE, 4759 },
	{ 0x87a2, 0x87a2, PDF_CMAP_SINGLE, 7785 },
	{ 0x87b3, 0x87b3, PDF_CMAP_SINGLE, 4248 },
	{ 0x87ba, 0x87ba, PDF_CMAP_SINGLE, 4379 },
	{ 0x87c4, 0x87c4, PDF_CMAP_SINGLE, 7376 },
	{ 0x87e0, 0x87e0, PDF_CMAP_SINGLE, 4926 },
	{ 0x87ec, 0x87ec, PDF_CMAP_SINGLE, 5442 },
	{ 0x87ef, 0x87ef, PDF_CMAP_SINGLE, 6162 },
	{ 0x87f2, 0x87f2, PDF_CMAP_SINGLE, 7309 },
	{ 0x87f9, 0x87f9, PDF_CMAP_SINGLE, 7696 },
	{ 0x87fb, 0x87fb, PDF_CMAP_SINGLE, 6408 },
	{ 0x87fe, 0x87fe, PDF_CMAP_SINGLE, 5467 },
	{ 0x8805, 0x8805, PDF_CMAP_SINGLE, 5690 },
	{ 0x881f, 0x881f, PDF_CMAP_SINGLE, 4411 },
	{ 0x8822, 0x8823, PDF_CMAP_TABLE, 2118 },
	{ 0x8831, 0x8831, PDF_CMAP_SINGLE, 3758 },
	{ 0x8836, 0x8836, PDF_CMAP_SINGLE, 6532 },
	{ 0x883b, 0x883b, PDF_CMAP_SINGLE, 4685 },
	{ 0x8840, 0x8840, PDF_CMAP_SINGLE, 7756 },
	{ 0x8846, 0x8846, PDF_CMAP_SINGLE, 6924 },
	{ 0x884c, 0x884d, PDF_CMAP_TABLE, 2120 },
	{ 0x8852, 0x8853, PDF_CMAP_TABLE, 2122 },
	{ 0x8857, 0x8857, PDF_CMAP_SINGLE, 3457 },
	{ 0x8859, 0x8859, PDF_CMAP_SINGLE, 5786 },
	{ 0x885b, 0x885b, PDF_CMAP_SINGLE, 6303 },
	{ 0x885d, 0x885d, PDF_CMAP_SINGLE, 7310 },
	{ 0x8861, 0x8863, PDF_CMAP_TABLE, 2124 },
	{ 0x8868, 0x8868, PDF_CMAP_SINGLE, 7585 },
	{ 0x886b, 0x886b, PDF_CMAP_SINGLE, 5326 },
	{ 0x8870, 0x8870, PDF_CMAP_SINGLE, 5565 },
	{ 0x8872, 0x8872, PDF_CMAP_SINGLE, 4151 },
	{ 0x8877, 0x8877, PDF_CMAP_SINGLE, 7311 },
	{ 0x887e, 0x887f, PDF_CMAP_RANGE, 4047 },
	{ 0x8881, 0x8882, PDF_CMAP_TABLE, 2127 },
	{ 0x8888, 0x8888, PDF_CMAP_SINGLE, 3458 },
	{ 0x888b, 0x888b, PDF_CMAP_SINGLE, 4262 },
	{ 0x888d, 0x888d, PDF_CMAP_SINGLE, 7565 },
	{ 0x8892, 0x8892, PDF_CMAP_SINGLE, 4209 },
	{ 0x8896, 0x8897, PDF_CMAP_TABLE, 2129 },
	{ 0x889e, 0x889e, PDF_CMAP_SINGLE, 3782 },
	{ 0x88ab, 0x88ab, PDF_CMAP_SINGLE, 7601 },
	{ 0x88b4, 0x88b4, PDF_CMAP_SINGLE, 3759 },
	{ 0x88c1, 0x88c2, PDF_CMAP_TABLE, 2131 },
	{ 0x88cf, 0x88cf, PDF_CMAP_SINGLE, 4632 },
	{ 0x88d4, 0x88d5, PDF_CMAP_TABLE, 2133 },
	{ 0x88d9, 0x88d9, PDF_CMAP_SINGLE, 3956 },
	{ 0x88dc, 0x88dd, PDF_CMAP_TABLE, 2135 },
	{ 0x88df, 0x88df, PDF_CMAP_SINGLE, 5290 },
	{ 0x88e1, 0x88e1, PDF_CMAP_SINGLE, 4633 },
	{ 0x88e8, 0x88e8, PDF_CMAP_SINGLE, 5215 },
	{ 0x88f3, 0x88f3, PDF_CMAP_SINGLE, 5356 },
	{ 0x88f4, 0x88f5, PDF_CMAP_RANGE, 4981 },
	{ 0x88f8, 0x88f8, PDF_CMAP_SINGLE, 4380 },
	{ 0x88fd, 0x88fd, PDF_CMAP_SINGLE, 6773 },
	{ 0x8907, 0x8907, PDF_CMAP_SINGLE, 5085 },
	{ 0x8910, 0x8910, PDF_CMAP_SINGLE, 3507 },
	{ 0x8912, 0x8913, PDF_CMAP_TABLE, 2137 },
	{ 0x8918, 0x8919, PDF_CMAP_TABLE, 2139 },
	{ 0x8925, 0x8925, PDF_CMAP_SINGLE, 6172 },
	{ 0x892a, 0x892a, PDF_CMAP_SINGLE, 7462 },
	{ 0x8936, 0x8936, PDF_CMAP_SINGLE, 5680 },
	{ 0x8938, 0x8938, PDF_CMAP_SINGLE, 4574 },
	{ 0x893b, 0x893b, PDF_CMAP_SINGLE, 5458 },
	{ 0x8941, 0x8941, PDF_CMAP_SINGLE, 3555 },
	{ 0x8944, 0x8944, PDF_CMAP_SINGLE, 5897 },
	{ 0x895f, 0x895f, PDF_CMAP_SINGLE, 4049 },
	{ 0x8964, 0x8964, PDF_CMAP_SINGLE, 4407 },
	{ 0x896a, 0x896a, PDF_CMAP_SINGLE, 4694 },
	{ 0x8972, 0x8972, PDF_CMAP_SINGLE, 5681 },
	{ 0x897f, 0x897f, PDF_CMAP_SINGLE, 5397 },
	{ 0x8981, 0x8981, PDF_CMAP_SINGLE, 6163 },
	{ 0x8983, 0x8983, PDF_CMAP_SINGLE, 4230 },
	{ 0x8986, 0x8987, PDF_CMAP_TABLE, 2141 },
	{ 0x898b, 0x898b, PDF_CMAP_SINGLE, 3644 },
	{ 0x898f, 0x898f, PDF_CMAP_SINGLE, 4005 },
	{ 0x8993, 0x8993, PDF_CMAP_SINGLE, 4734 },
	{ 0x8996, 0x8996, PDF_CMAP_SINGLE, 5714 },
	{ 0x89a1, 0x89a1, PDF_CMAP_SINGLE, 3635 },
	{ 0x89a9, 0x89aa, PDF_CMAP_TABLE, 2143 },
	{ 0x89b2, 0x89b2, PDF_CMAP_SINGLE, 4034 },
	{ 0x89ba, 0x89ba, PDF_CMAP_SINGLE, 3473 },
	{ 0x89bd, 0x89bd, PDF_CMAP_SINGLE, 4408 },
	{ 0x89c0, 0x89c0, PDF_CMAP_SINGLE, 3831 },
	{ 0x89d2, 0x89d2, PDF_CMAP_SINGLE, 3474 },
	{ 0x89e3, 0x89e3, PDF_CMAP_SINGLE, 7697 },
	{ 0x89f4, 0x89f4, PDF_CMAP_SINGLE, 5357 },
	{ 0x89f8, 0x89f8, PDF_CMAP_SINGLE, 7246 },
	{ 0x8a00, 0x8a00, PDF_CMAP_SINGLE, 5921 },
	{ 0x8a02, 0x8a03, PDF_CMAP_TABLE, 2145 },
	{ 0x8a08, 0x8a08, PDF_CMAP_SINGLE, 3724 },
	{ 0x8a0a, 0x8a0a, PDF_CMAP_SINGLE, 5754 },
	{ 0x8a0c, 0x8a0c, PDF_CMAP_SINGLE, 7861 },
	{ 0x8a0e, 0x8a0e, PDF_CMAP_SINGLE, 7452 },
	{ 0x8a13, 0x8a13, PDF_CMAP_SINGLE, 7987 },
	{ 0x8a16, 0x8a18, PDF_CMAP_TABLE, 2147 },
	{ 0x8a1b, 0x8a1b, PDF_CMAP_SINGLE, 6106 },
	{ 0x8a1d, 0x8a1d, PDF_CMAP_SINGLE, 5787 },
	{ 0x8a1f, 0x8a1f, PDF_CMAP_SINGLE, 5557 },
	{ 0x8a23, 0x8a23, PDF_CMAP_SINGLE, 3653 },
	{ 0x8a25, 0x8a25, PDF_CMAP_SINGLE, 4182 },
	{ 0x8a2a, 0x8a2a, PDF_CMAP_SINGLE, 4964 },
	{ 0x8a2d, 0x8a2d, PDF_CMAP_SINGLE, 5459 },
	{ 0x8a31, 0x8a31, PDF_CMAP_SINGLE, 7722 },
	{ 0x8a34, 0x8a34, PDF_CMAP_SINGLE, 5530 },
	{ 0x8a36, 0x8a36, PDF_CMAP_SINGLE, 3459 },
	{ 0x8a3a, 0x8a3b, PDF_CMAP_TABLE, 2150 },
	{ 0x8a50, 0x8a50, PDF_CMAP_SINGLE, 5291 },
	{ 0x8a54, 0x8a55, PDF_CMAP_TABLE, 2152 },
	{ 0x8a5b, 0x8a5b, PDF_CMAP_SINGLE, 6612 },
	{ 0x8a5e, 0x8a5e, PDF_CMAP_SINGLE, 5292 },
	{ 0x8a60, 0x8a60, PDF_CMAP_SINGLE, 6025 },
	{ 0x8a62, 0x8a63, PDF_CMAP_TABLE, 2154 },
	{ 0x8a66, 0x8a66, PDF_CMAP_SINGLE, 5715 },
	{ 0x8a69, 0x8a69, PDF_CMAP_SINGLE, 5716 },
	{ 0x8a6d, 0x8a6e, PDF_CMAP_TABLE, 2156 },
	{ 0x8a70, 0x8a73, PDF_CMAP_TABLE, 2158 },
	{ 0x8a75, 0x8a75, PDF_CMAP_SINGLE, 5443 },
	{ 0x8a79, 0x8a79, PDF_CMAP_SINGLE, 7185 },
	{ 0x8a85, 0x8a85, PDF_CMAP_SINGLE, 6891 },
	{ 0x8a87, 0x8a87, PDF_CMAP_SINGLE, 3809 },
	{ 0x8a8c, 0x8a8d, PDF_CMAP_TABLE, 2162 },
	{ 0x8a93, 0x8a93, PDF_CMAP_SINGLE, 5398 },
	{ 0x8a95, 0x8a95, PDF_CMAP_SINGLE, 7417 },
	{ 0x8a98, 0x8a98, PDF_CMAP_SINGLE, 6344 },
	{ 0x8a9e, 0x8a9e, PDF_CMAP_SINGLE, 5908 },
	{ 0x8aa0, 0x8aa1, PDF_CMAP_TABLE, 2164 },
	{ 0x8aa3, 0x8aa6, PDF_CMAP_TABLE, 2166 },
	{ 0x8aa8, 0x8aa8, PDF_CMAP_SINGLE, 7946 },
	{ 0x8aaa, 0x8aaa, PDF_CMAP_SINGLE, 5460 },
	{ 0x8ab0, 0x8ab0, PDF_CMAP_SINGLE, 5611 },
	{ 0x8ab2, 0x8ab2, PDF_CMAP_SINGLE, 3810 },
	{ 0x8ab9, 0x8ab9, PDF_CMAP_SINGLE, 5216 },
	{ 0x8abc, 0x8abc, PDF_CMAP_SINGLE, 6410 },
	{ 0x8abe, 0x8abf, PDF_CMAP_TABLE, 2170 },
	{ 0x8ac2, 0x8ac2, PDF_CMAP_SINGLE, 7186 },
	{ 0x8ac4, 0x8ac4, PDF_CMAP_SINGLE, 5662 },
	{ 0x8ac7, 0x8ac7, PDF_CMAP_SINGLE, 4231 },
	{ 0x8acb, 0x8acb, PDF_CMAP_SINGLE, 7202 },
	{ 0x8acd, 0x8acd, PDF_CMAP_SINGLE, 6589 },
	{ 0x8acf, 0x8acf, PDF_CMAP_SINGLE, 7276 },
	{ 0x8ad2, 0x8ad2, PDF_CMAP_SINGLE, 4437 },
	{ 0x8ad6, 0x8ad6, PDF_CMAP_SINGLE, 4535 },
	{ 0x8adb, 0x8adc, PDF_CMAP_TABLE, 2172 },
	{ 0x8ae1, 0x8ae1, PDF_CMAP_SINGLE, 5717 },
	{ 0x8ae6, 0x8ae7, PDF_CMAP_TABLE, 2174 },
	{ 0x8aea, 0x8aeb, PDF_CMAP_TABLE, 2176 },
	{ 0x8aed, 0x8aee, PDF_CMAP_TABLE, 2178 },
	{ 0x8af1, 0x8af1, PDF_CMAP_SINGLE, 8002 },
	{ 0x8af6, 0x8af8, PDF_CMAP_TABLE, 2180 },
	{ 0x8afa, 0x8afa, PDF_CMAP_SINGLE, 5922 },
	{ 0x8afe, 0x8afe, PDF_CMAP_SINGLE, 4139 },
	{ 0x8b00, 0x8b02, PDF_CMAP_TABLE, 2183 },
	{ 0x8b04, 0x8b04, PDF_CMAP_SINGLE, 4371 },
	{ 0x8b0e, 0x8b0e, PDF_CMAP_SINGLE, 4866 },
	{ 0x8b10, 0x8b10, PDF_CMAP_SINGLE, 4885 },
	{ 0x8b14, 0x8b14, PDF_CMAP_SINGLE, 7633 },
	{ 0x8b16, 0x8b17, PDF_CMAP_TABLE, 2186 },
	{ 0x8b19, 0x8b1b, PDF_CMAP_TABLE, 2188 },
	{ 0x8b1d, 0x8b1d, PDF_CMAP_SINGLE, 5293 },
	{ 0x8b20, 0x8b20, PDF_CMAP_SINGLE, 6164 },
	{ 0x8b28, 0x8b28, PDF_CMAP_SINGLE, 4786 },
	{ 0x8b2b, 0x8b2c, PDF_CMAP_TABLE, 2191 },
	{ 0x8b33, 0x8b33, PDF_CMAP_SINGLE, 3935 },
	{ 0x8b39, 0x8b39, PDF_CMAP_SINGLE, 4035 },
	{ 0x8b41, 0x8b41, PDF_CMAP_SINGLE, 7874 },
	{ 0x8b49, 0x8b49, PDF_CMAP_SINGLE, 6940 },
	{ 0x8b4e, 0x8b4f, PDF_CMAP_TABLE, 2193 },
	{ 0x8b58, 0x8b58, PDF_CMAP_SINGLE, 5731 },
	{ 0x8b5a, 0x8b5a, PDF_CMAP_SINGLE, 4232 },
	{ 0x8b5c, 0x8b5c, PDF_CMAP_SINGLE, 5072 },
	{ 0x8b66, 0x8b66, PDF_CMAP_SINGLE, 3697 },
	{ 0x8b6c, 0x8b6c, PDF_CMAP_SINGLE, 5217 },
	{ 0x8b6f, 0x8b70, PDF_CMAP_TABLE, 2195 },
	{ 0x8b74, 0x8b74, PDF_CMAP_SINGLE, 3645 },
	{ 0x8b77, 0x8b77, PDF_CMAP_SINGLE, 7836 },
	{ 0x8b7d, 0x8b7d, PDF_CMAP_SINGLE, 6044 },
	{ 0x8b80, 0x8b80, PDF_CMAP_SINGLE, 4318 },
	{ 0x8b8a, 0x8b8a, PDF_CMAP_SINGLE, 5033 },
	{ 0x8b90, 0x8b90, PDF_CMAP_SINGLE, 5612 },
	{ 0x8b92, 0x8b93, PDF_CMAP_TABLE, 2197 },
	{ 0x8b96, 0x8b96, PDF_CMAP_SINGLE, 7091 },
	{ 0x8b9a, 0x8b9a, PDF_CMAP_SINGLE, 7072 },
	{ 0x8c37, 0x8c37, PDF_CMAP_SINGLE, 3772 },
	{ 0x8c3f, 0x8c3f, PDF_CMAP_SINGLE, 3726 },
	{ 0x8c41, 0x8c41, PDF_CMAP_SINGLE, 7902 },
	{ 0x8c46, 0x8c46, PDF_CMAP_SINGLE, 4355 },
	{ 0x8c48, 0x8c48, PDF_CMAP_SINGLE, 4116 },
	{ 0x8c4a, 0x8c4a, PDF_CMAP_SINGLE, 7594 },
	{ 0x8c4c, 0x8c4c, PDF_CMAP_SINGLE, 6122 },
	{ 0x8c55, 0x8c55, PDF_CMAP_SINGLE, 5718 },
	{ 0x8c5a, 0x8c5a, PDF_CMAP_SINGLE, 4327 },
	{ 0x8c61, 0x8c61, PDF_CMAP_SINGLE, 5359 },
	{ 0x8c6a, 0x8c6b, PDF_CMAP_TABLE, 2199 },
	{ 0x8c79, 0x8c7a, PDF_CMAP_TABLE, 2201 },
	{ 0x8c82, 0x8c82, PDF_CMAP_SINGLE, 7236 },
	{ 0x8c8a, 0x8c8a, PDF_CMAP_SINGLE, 4723 },
	{ 0x8c8c, 0x8c8c, PDF_CMAP_SINGLE, 4787 },
	{ 0x8c9d, 0x8c9e, PDF_CMAP_TABLE, 2203 },
	{ 0x8ca0, 0x8ca2, PDF_CMAP_TABLE, 2205 },
	{ 0x8ca7, 0x8cac, PDF_CMAP_TABLE, 2208 },
	{ 0x8caf, 0x8cb0, PDF_CMAP_TABLE, 2214 },
	{ 0x8cb3, 0x8cb4, PDF_CMAP_TABLE, 2216 },
	{ 0x8cb6, 0x8cb8, PDF_CMAP_TABLE, 2218 },
	{ 0x8cbb, 0x8cbd, PDF_CMAP_TABLE, 2221 },
	{ 0x8cbf, 0x8cc4, PDF_CMAP_TABLE, 2224 },
	{ 0x8cc7, 0x8cc8, PDF_CMAP_TABLE, 2230 },
	{ 0x8cca, 0x8cca, PDF_CMAP_SINGLE, 6635 },
	{ 0x8cd1, 0x8cd1, PDF_CMAP_SINGLE, 7007 },
	{ 0x8cd3, 0x8cd3, PDF_CMAP_SINGLE, 5235 },
	{ 0x8cda, 0x8cda, PDF_CMAP_SINGLE, 4548 },
	{ 0x8cdc, 0x8cdc, PDF_CMAP_SINGLE, 5294 },
	{ 0x8cde, 0x8cde, PDF_CMAP_SINGLE, 5360 },
	{ 0x8ce0, 0x8ce0, PDF_CMAP_SINGLE, 4984 },
	{ 0x8ce2, 0x8ce4, PDF_CMAP_TABLE, 2232 },
	{ 0x8ce6, 0x8ce6, PDF_CMAP_SINGLE, 5141 },
	{ 0x8cea, 0x8cea, PDF_CMAP_SINGLE, 7027 },
	{ 0x8ced, 0x8ced, PDF_CMAP_SINGLE, 4299 },
	{ 0x8cf4, 0x8cf4, PDF_CMAP_SINGLE, 4549 },
	{ 0x8cfb, 0x8cfd, PDF_CMAP_TABLE, 2235 },
	{ 0x8d04, 0x8d05, PDF_CMAP_TABLE, 2238 },
	{ 0x8d07, 0x8d08, PDF_CMAP_TABLE, 2240 },
	{ 0x8d0a, 0x8d0a, PDF_CMAP_SINGLE, 7073 },
	{ 0x8d0d, 0x8d0d, PDF_CMAP_SINGLE, 5468 },
	{ 0x8d13, 0x8d13, PDF_CMAP_SINGLE, 6566 },
	{ 0x8d16, 0x8d16, PDF_CMAP_SINGLE, 5544 },
	{ 0x8d64, 0x8d64, PDF_CMAP_SINGLE, 6636 },
	{ 0x8d66, 0x8d66, PDF_CMAP_SINGLE, 5295 },
	{ 0x8d6b, 0x8d6b, PDF_CMAP_SINGLE, 7732 },
	{ 0x8d70, 0x8d70, PDF_CMAP_SINGLE, 6892 },
	{ 0x8d73, 0x8d74, PDF_CMAP_TABLE, 2242 },
	{ 0x8d77, 0x8d77, PDF_CMAP_SINGLE, 4117 },
	{ 0x8d85, 0x8d85, PDF_CMAP_SINGLE, 7237 },
	{ 0x8d8a, 0x8d8a, PDF_CMAP_SINGLE, 6282 },
	{ 0x8d99, 0x8d99, PDF_CMAP_SINGLE, 6820 },
	{ 0x8da3, 0x8da3, PDF_CMAP_SINGLE, 7326 },
	{ 0x8da8, 0x8da8, PDF_CMAP_SINGLE, 7277 },
	{ 0x8db3, 0x8db3, PDF_CMAP_SINGLE, 6830 },
	{ 0x8dba, 0x8dba, PDF_CMAP_SINGLE, 5144 },
	{ 0x8dbe, 0x8dbe, PDF_CMAP_SINGLE, 6973 },
	{ 0x8dc6, 0x8dc6, PDF_CMAP_SINGLE, 7442 },
	{ 0x8dcb, 0x8dcc, PDF_CMAP_TABLE, 2244 },
	{ 0x8dcf, 0x8dcf, PDF_CMAP_SINGLE, 3461 },
	{ 0x8ddb, 0x8ddb, PDF_CMAP_SINGLE, 7488 },
	{ 0x8ddd, 0x8ddd, PDF_CMAP_SINGLE, 3596 },
	{ 0x8de1, 0x8de1, PDF_CMAP_SINGLE, 6637 },
	{ 0x8de3, 0x8de3, PDF_CMAP_SINGLE, 5444 },
	{ 0x8de8, 0x8de8, PDF_CMAP_SINGLE, 3811 },
	{ 0x8def, 0x8def, PDF_CMAP_SINGLE, 4522 },
	{ 0x8df3, 0x8df3, PDF_CMAP_SINGLE, 4300 },
	{ 0x8e0a, 0x8e0a, PDF_CMAP_SINGLE, 6194 },
	{ 0x8e0f, 0x8e10, PDF_CMAP_TABLE, 2246 },
	{ 0x8e1e, 0x8e1e, PDF_CMAP_SINGLE, 3597 },
	{ 0x8e2a, 0x8e2a, PDF_CMAP_SINGLE, 6850 },
	{ 0x8e30, 0x8e30, PDF_CMAP_SINGLE, 6347 },
	{ 0x8e35, 0x8e35, PDF_CMAP_SINGLE, 6851 },
	{ 0x8e42, 0x8e42, PDF_CMAP_SINGLE, 6348 },
	{ 0x8e44, 0x8e44, PDF_CMAP_SINGLE, 6775 },
	{ 0x8e47, 0x8e4a, PDF_CMAP_TABLE, 2248 },
	{ 0x8e59, 0x8e59, PDF_CMAP_SINGLE, 7296 },
	{ 0x8e5f, 0x8e60, PDF_CMAP_TABLE, 2252 },
	{ 0x8e74, 0x8e74, PDF_CMAP_SINGLE, 7297 },
	{ 0x8e76, 0x8e76, PDF_CMAP_SINGLE, 3982 },
	{ 0x8e81, 0x8e81, PDF_CMAP_SINGLE, 6821 },
	{ 0x8e87, 0x8e87, PDF_CMAP_SINGLE, 6614 },
	{ 0x8e8a, 0x8e8a, PDF_CMAP_SINGLE, 6893 },
	{ 0x8e8d, 0x8e8d, PDF_CMAP_SINGLE, 5878 },
	{ 0x8eaa, 0x8eac, PDF_CMAP_TABLE, 2254 },
	{ 0x8ec0, 0x8ec0, PDF_CMAP_SINGLE, 3937 },
	{ 0x8eca, 0x8ecd, PDF_CMAP_TABLE, 2257 },
	{ 0x8ed2, 0x8ed2, PDF_CMAP_SINGLE, 7726 },
	{ 0x8edf, 0x8edf, PDF_CMAP_SINGLE, 5982 },
	{ 0x8eeb, 0x8eeb, PDF_CMAP_SINGLE, 7008 },
	{ 0x8ef8, 0x8ef8, PDF_CMAP_SINGLE, 7298 },
	{ 0x8efb, 0x8efb, PDF_CMAP_SINGLE, 3462 },
	{ 0x8efe, 0x8efe, PDF_CMAP_SINGLE, 5732 },
	{ 0x8f03, 0x8f03, PDF_CMAP_SINGLE, 3887 },
	{ 0x8f05, 0x8f05, PDF_CMAP_SINGLE, 4523 },
	{ 0x8f09, 0x8f09, PDF_CMAP_SINGLE, 6584 },
	{ 0x8f12, 0x8f15, PDF_CMAP_TABLE, 2261 },
	{ 0x8f1b, 0x8f1f, PDF_CMAP_TABLE, 2265 },
	{ 0x8f26, 0x8f27, PDF_CMAP_TABLE, 2270 },
	{ 0x8f29, 0x8f2a, PDF_CMAP_TABLE, 2272 },
	{ 0x8f2f, 0x8f2f, PDF_CMAP_SINGLE, 7035 },
	{ 0x8f33, 0x8f33, PDF_CMAP_SINGLE, 6894 },
	{ 0x8f38, 0x8f39, PDF_CMAP_TABLE, 2274 },
	{ 0x8f3b, 0x8f3b, PDF_CMAP_SINGLE, 5088 },
	{ 0x8f3e, 0x8f3f, PDF_CMAP_TABLE, 2276 },
	{ 0x8f44, 0x8f45, PDF_CMAP_TABLE, 2278 },
	{ 0x8f49, 0x8f49, PDF_CMAP_SINGLE, 6675 },
	{ 0x8f4d, 0x8f4e, PDF_CMAP_TABLE, 2280 },
	{ 0x8f5d, 0x8f5d, PDF_CMAP_SINGLE, 5945 },
	{ 0x8f5f, 0x8f5f, PDF_CMAP_SINGLE, 3867 },
	{ 0x8f62, 0x8f62, PDF_CMAP_SINGLE, 4463 },
	{ 0x8f9b, 0x8f9c, PDF_CMAP_TABLE, 2282 },
	{ 0x8fa3, 0x8fa3, PDF_CMAP_SINGLE, 4398 },
	{ 0x8fa6, 0x8fa6, PDF_CMAP_SINGLE, 7496 },
	{ 0x8fa8, 0x8fa8, PDF_CMAP_SINGLE, 5034 },
	{ 0x8fad, 0x8fad, PDF_CMAP_SINGLE, 5296 },
	{ 0x8faf, 0x8fb2, PDF_CMAP_TABLE, 2284 },
	{ 0x8fc2, 0x8fc2, PDF_CMAP_SINGLE, 6222 },
	{ 0x8fc5, 0x8fc5, PDF_CMAP_SINGLE, 5757 },
	{ 0x8fce, 0x8fce, PDF_CMAP_SINGLE, 6026 },
	{ 0x8fd1, 0x8fd1, PDF_CMAP_SINGLE, 4036 },
	{ 0x8fd4, 0x8fd4, PDF_CMAP_SINGLE, 4927 },
	{ 0x8fe6, 0x8fe6, PDF_CMAP_SINGLE, 3463 },
	{ 0x8fea, 0x8feb, PDF_CMAP_TABLE, 2288 },
	{ 0x8fed, 0x8fed, PDF_CMAP_SINGLE, 7029 },
	{ 0x8ff0, 0x8ff0, PDF_CMAP_SINGLE, 5669 },
	{ 0x8ff2, 0x8ff2, PDF_CMAP_SINGLE, 3626 },
	{ 0x8ff7, 0x8ff7, PDF_CMAP_SINGLE, 4867 },
	{ 0x8ff9, 0x8ff9, PDF_CMAP_SINGLE, 6640 },
	{ 0x8ffd, 0x8ffd, PDF_CMAP_SINGLE, 7278 },
	{ 0x9000, 0x9003, PDF_CMAP_TABLE, 2290 },
	{ 0x9005, 0x9006, PDF_CMAP_TABLE, 2294 },
	{ 0x9008, 0x9008, PDF_CMAP_SINGLE, 7787 },
	{ 0x900b, 0x900b, PDF_CMAP_SINGLE, 7567 },
	{ 0x900d, 0x900d, PDF_CMAP_SINGLE, 5531 },
	{ 0x900f, 0x9011, PDF_CMAP_TABLE, 2296 },
	{ 0x9014, 0x9015, PDF_CMAP_TABLE, 2299 },
	{ 0x9017, 0x9017, PDF_CMAP_SINGLE, 4356 },
	{ 0x9019, 0x901a, PDF_CMAP_TABLE, 2301 },
	{ 0x901d, 0x9023, PDF_CMAP_TABLE, 2303 },
	{ 0x902e, 0x902e, PDF_CMAP_SINGLE, 7211 },
	{ 0x9031, 0x9032, PDF_CMAP_TABLE, 2310 },
	{ 0x9035, 0x9035, PDF_CMAP_SINGLE, 4007 },
	{ 0x9038, 0x9038, PDF_CMAP_SINGLE, 6466 },
	{ 0x903c, 0x903c, PDF_CMAP_SINGLE, 7615 },
	{ 0x903e, 0x903e, PDF_CMAP_SINGLE, 6350 },
	{ 0x9041, 0x9042, PDF_CMAP_TABLE, 2312 },
	{ 0x9047, 0x9047, PDF_CMAP_SINGLE, 6223 },
	{ 0x904a, 0x904b, PDF_CMAP_TABLE, 2314 },
	{ 0x904d, 0x904e, PDF_CMAP_TABLE, 2316 },
	{ 0x9050, 0x9051, PDF_CMAP_TABLE, 2318 },
	{ 0x9053, 0x9055, PDF_CMAP_TABLE, 2320 },
	{ 0x9059, 0x9059, PDF_CMAP_SINGLE, 6165 },
	{ 0x905c, 0x905e, PDF_CMAP_TABLE, 2323 },
	{ 0x9060, 0x9061, PDF_CMAP_TABLE, 2326 },
	{ 0x9063, 0x9063, PDF_CMAP_SINGLE, 3646 },
	{ 0x9069, 0x9069, PDF_CMAP_SINGLE, 6641 },
	{ 0x906d, 0x906f, PDF_CMAP_TABLE, 2328 },
	{ 0x9072, 0x9072, PDF_CMAP_SINGLE, 6974 },
	{ 0x9075, 0x9075, PDF_CMAP_SINGLE, 6918 },
	{ 0x9077, 0x9078, PDF_CMAP_TABLE, 2331 },
	{ 0x907a, 0x907a, PDF_CMAP_SINGLE, 6351 },
	{ 0x907c, 0x907d, PDF_CMAP_TABLE, 2333 },
	{ 0x907f, 0x9084, PDF_CMAP_TABLE, 2335 },
	{ 0x9087, 0x9088, PDF_CMAP_TABLE, 2341 },
	{ 0x908a, 0x908a, PDF_CMAP_SINGLE, 5036 },
	{ 0x908f, 0x908f, PDF_CMAP_SINGLE, 4381 },
	{ 0x9091, 0x9091, PDF_CMAP_SINGLE, 6389 },
	{ 0x9095, 0x9095, PDF_CMAP_SINGLE, 6096 },
	{ 0x9099, 0x9099, PDF_CMAP_SINGLE, 4707 },
	{ 0x90a2, 0x90a3, PDF_CMAP_TABLE, 2343 },
	{ 0x90a6, 0x90a6, PDF_CMAP_SINGLE, 4966 },
	{ 0x90a8, 0x90a8, PDF_CMAP_SINGLE, 7250 },
	{ 0x90aa, 0x90aa, PDF_CMAP_SINGLE, 5297 },
	{ 0x90af, 0x90b1, PDF_CMAP_TABLE, 2345 },
	{ 0x90b5, 0x90b5, PDF_CMAP_SINGLE, 5533 },
	{ 0x90b8, 0x90b8, PDF_CMAP_SINGLE, 6616 },
	{ 0x90c1, 0x90c1, PDF_CMAP_SINGLE, 6236 },
	{ 0x90ca, 0x90ca, PDF_CMAP_SINGLE, 3889 },
	{ 0x90de, 0x90de, PDF_CMAP_SINGLE, 4419 },
	{ 0x90e1, 0x90e1, PDF_CMAP_SINGLE, 3958 },
	{ 0x90e8, 0x90e8, PDF_CMAP_SINGLE, 5145 },
	{ 0x90ed, 0x90ed, PDF_CMAP_SINGLE, 3818 },
	{ 0x90f5, 0x90f5, PDF_CMAP_SINGLE, 6224 },
	{ 0x90fd, 0x90fd, PDF_CMAP_SINGLE, 4305 },
	{ 0x9102, 0x9102, PDF_CMAP_SINGLE, 5802 },
	{ 0x9112, 0x9112, PDF_CMAP_SINGLE, 7279 },
	{ 0x9115, 0x9115, PDF_CMAP_SINGLE, 7714 },
	{ 0x9119, 0x9119, PDF_CMAP_SINGLE, 5219 },
	{ 0x9127, 0x9127, PDF_CMAP_SINGLE, 4372 },
	{ 0x912d, 0x912d, PDF_CMAP_SINGLE, 6748 },
	{ 0x9132, 0x9132, PDF_CMAP_SINGLE, 4210 },
	{ 0x9149, 0x914e, PDF_CMAP_TABLE, 2348 },
	{ 0x9152, 0x9152, PDF_CMAP_SINGLE, 6897 },
	{ 0x9162, 0x9162, PDF_CMAP_SINGLE, 7238 },
	{ 0x9169, 0x916a, PDF_CMAP_TABLE, 2354 },
	{ 0x916c, 0x916c, PDF_CMAP_SINGLE, 5616 },
	{ 0x9175, 0x9175, PDF_CMAP_SINGLE, 7964 },
	{ 0x9177, 0x9178, PDF_CMAP_TABLE, 2356 },
	{ 0x9187, 0x9187, PDF_CMAP_SINGLE, 5663 },
	{ 0x9189, 0x9189, PDF_CMAP_SINGLE, 7327 },
	{ 0x918b, 0x918b, PDF_CMAP_SINGLE, 7239 },
	{ 0x918d, 0x918d, PDF_CMAP_SINGLE, 6776 },
	{ 0x9192, 0x9192, PDF_CMAP_SINGLE, 5491 },
	{ 0x919c, 0x919c, PDF_CMAP_SINGLE, 7281 },
	{ 0x91ab, 0x91ac, PDF_CMAP_TABLE, 2358 },
	{ 0x91ae, 0x91af, PDF_CMAP_TABLE, 2360 },
	{ 0x91b1, 0x91b1, PDF_CMAP_SINGLE, 4937 },
	{ 0x91b4, 0x91b5, PDF_CMAP_TABLE, 2362 },
	{ 0x91c0, 0x91c0, PDF_CMAP_SINGLE, 5899 },
	{ 0x91c7, 0x91c7, PDF_CMAP_SINGLE, 7124 },
	{ 0x91c9, 0x91c9, PDF_CMAP_SINGLE, 6353 },
	{ 0x91cb, 0x91d1, PDF_CMAP_TABLE, 2364 },
	{ 0x91d7, 0x91d8, PDF_CMAP_TABLE, 2371 },
	{ 0x91dc, 0x91dd, PDF_CMAP_TABLE, 2373 },
	{ 0x91e3, 0x91e3, PDF_CMAP_SINGLE, 6824 },
	{ 0x91e7, 0x91e7, PDF_CMAP_SINGLE, 7163 },
	{ 0x91ea, 0x91ea, PDF_CMAP_SINGLE, 6225 },
	{ 0x91f5, 0x91f5, PDF_CMAP_SINGLE, 7125 },
	{ 0x920d, 0x920d, PDF_CMAP_SINGLE, 4363 },
	{ 0x9210, 0x9212, PDF_CMAP_TABLE, 2375 },
	{ 0x9217, 0x9217, PDF_CMAP_SINGLE, 6366 },
	{ 0x921e, 0x921e, PDF_CMAP_SINGLE, 4014 },
	{ 0x9234, 0x9234, PDF_CMAP_SINGLE, 4501 },
	{ 0x923a, 0x923a, PDF_CMAP_SINGLE, 6082 },
	{ 0x923f, 0x9240, PDF_CMAP_TABLE, 2378 },
	{ 0x9245, 0x9245, PDF_CMAP_SINGLE, 3599 },
	{ 0x9249, 0x9249, PDF_CMAP_SINGLE, 7752 },
	{ 0x9257, 0x9257, PDF_CMAP_SINGLE, 3658 },
	{ 0x925b, 0x925b, PDF_CMAP_SINGLE, 5983 },
	{ 0x925e, 0x925e, PDF_CMAP_SINGLE, 6283 },
	{ 0x9262, 0x9262, PDF_CMAP_SINGLE, 4938 },
	{ 0x9264, 0x9266, PDF_CMAP_TABLE, 2380 },
	{ 0x9280, 0x9280, PDF_CMAP_SINGLE, 6378 },
	{ 0x9283, 0x9283, PDF_CMAP_SINGLE, 7260 },
	{ 0x9285, 0x9285, PDF_CMAP_SINGLE, 4347 },
	{ 0x9291, 0x9291, PDF_CMAP_SINGLE, 5446 },
	{ 0x9293, 0x9293, PDF_CMAP_SINGLE, 6677 },
	{ 0x9296, 0x9296, PDF_CMAP_SINGLE, 5617 },
	{ 0x9298, 0x9298, PDF_CMAP_SINGLE, 4761 },
	{ 0x929c, 0x929c, PDF_CMAP_SINGLE, 7660 },
	{ 0x92b3, 0x92b3, PDF_CMAP_SINGLE, 6046 },
	{ 0x92b6, 0x92b7, PDF_CMAP_TABLE, 2383 },
	{ 0x92b9, 0x92b9, PDF_CMAP_SINGLE, 5618 },
	{ 0x92cc, 0x92cc, PDF_CMAP_SINGLE, 6752 },
	{ 0x92cf, 0x92cf, PDF_CMAP_SINGLE, 7769 },
	{ 0x92d2, 0x92d2, PDF_CMAP_SINGLE, 5107 },
	{ 0x92e4, 0x92e4, PDF_CMAP_SINGLE, 5400 },
	{ 0x92ea, 0x92ea, PDF_CMAP_SINGLE, 7568 },
	{ 0x92f8, 0x92f8, PDF_CMAP_SINGLE, 3600 },
	{ 0x92fc, 0x92fc, PDF_CMAP_SINGLE, 3557 },
	{ 0x9304, 0x9304, PDF_CMAP_SINGLE, 4532 },
	{ 0x9310, 0x9310, PDF_CMAP_SINGLE, 7282 },
	{ 0x9318, 0x9318, PDF_CMAP_SINGLE, 7283 },
	{ 0x931a, 0x931a, PDF_CMAP_SINGLE, 6590 },
	{ 0x931e, 0x9322, PDF_CMAP_TABLE, 2385 },
	{ 0x9324, 0x9324, PDF_CMAP_SINGLE, 4119 },
	{ 0x9326, 0x9326, PDF_CMAP_SINGLE, 4050 },
	{ 0x9328, 0x9328, PDF_CMAP_SINGLE, 4811 },
	{ 0x932b, 0x932b, PDF_CMAP_SINGLE, 5417 },
	{ 0x932e, 0x932f, PDF_CMAP_TABLE, 2390 },
	{ 0x9348, 0x9348, PDF_CMAP_SINGLE, 6027 },
	{ 0x934a, 0x934b, PDF_CMAP_TABLE, 2392 },
	{ 0x934d, 0x934d, PDF_CMAP_SINGLE, 4306 },
	{ 0x9354, 0x9354, PDF_CMAP_SINGLE, 5803 },
	{ 0x935b, 0x935b, PDF_CMAP_SINGLE, 4211 },
	{ 0x936e, 0x936e, PDF_CMAP_SINGLE, 6354 },
	{ 0x9375, 0x9375, PDF_CMAP_SINGLE, 3611 },
	{ 0x937c, 0x937c, PDF_CMAP_SINGLE, 7375 },
	{ 0x937e, 0x937e, PDF_CMAP_SINGLE, 6852 },
	{ 0x938c, 0x938c, PDF_CMAP_SINGLE, 3659 },
	{ 0x9394, 0x9394, PDF_CMAP_SINGLE, 6195 },
	{ 0x9396, 0x9396, PDF_CMAP_SINGLE, 5564 },
	{ 0x939a, 0x939a, PDF_CMAP_SINGLE, 7284 },
	{ 0x93a3, 0x93a3, PDF_CMAP_SINGLE, 7789 },
	{ 0x93a7, 0x93a7, PDF_CMAP_SINGLE, 3577 },
	{ 0x93ac, 0x93ad, PDF_CMAP_TABLE, 2394 },
	{ 0x93b0, 0x93b0, PDF_CMAP_SINGLE, 6467 },
	{ 0x93c3, 0x93c3, PDF_CMAP_SINGLE, 6831 },
	{ 0x93d1, 0x93d1, PDF_CMAP_SINGLE, 6642 },
	{ 0x93de, 0x93de, PDF_CMAP_SINGLE, 6196 },
	{ 0x93e1, 0x93e1, PDF_CMAP_SINGLE, 3700 },
	{ 0x93e4, 0x93e4, PDF_CMAP_SINGLE, 4575 },
	{ 0x93f6, 0x93f6, PDF_CMAP_SINGLE, 7036 },
	{ 0x9404, 0x9404, PDF_CMAP_SINGLE, 7952 },
	{ 0x9418, 0x9418, PDF_CMAP_SINGLE, 6853 },
	{ 0x9425, 0x9425, PDF_CMAP_SINGLE, 5447 },
	{ 0x942b, 0x942b, PDF_CMAP_SINGLE, 6679 },
	{ 0x9435, 0x9435, PDF_CMAP_SINGLE, 7176 },
	{ 0x9438, 0x9438, PDF_CMAP_SINGLE, 7407 },
	{ 0x9444, 0x9444, PDF_CMAP_SINGLE, 6898 },
	{ 0x9451, 0x9452, PDF_CMAP_RANGE, 3527 },
	{ 0x945b, 0x945b, PDF_CMAP_SINGLE, 3851 },
	{ 0x947d, 0x947d, PDF_CMAP_SINGLE, 7074 },
	{ 0x947f, 0x947f, PDF_CMAP_SINGLE, 7060 },
	{ 0x9577, 0x9577, PDF_CMAP_SINGLE, 6568 },
	{ 0x9580, 0x9580, PDF_CMAP_SINGLE, 4846 },
	{ 0x9583, 0x9583, PDF_CMAP_SINGLE, 5469 },
	{ 0x9589, 0x9589, PDF_CMAP_SINGLE, 7542 },
	{ 0x958b, 0x958b, PDF_CMAP_SINGLE, 3578 },
	{ 0x958f, 0x958f, PDF_CMAP_SINGLE, 6367 },
	{ 0x9591, 0x9592, PDF_CMAP_RANGE, 7645 },
	{ 0x9593, 0x9594, PDF_CMAP_TABLE, 2396 },
	{ 0x9598, 0x9598, PDF_CMAP_SINGLE, 3535 },
	{ 0x95a3, 0x95a5, PDF_CMAP_TABLE, 2398 },
	{ 0x95a8, 0x95a8, PDF_CMAP_SINGLE, 4008 },
	{ 0x95ad, 0x95ad, PDF_CMAP_SINGLE, 4453 },
	{ 0x95b1, 0x95b1, PDF_CMAP_SINGLE, 5988 },
	{ 0x95bb, 0x95bc, PDF_CMAP_TABLE, 2401 },
	{ 0x95c7, 0x95c7, PDF_CMAP_SINGLE, 5828 },
	{ 0x95ca, 0x95ca, PDF_CMAP_SINGLE, 7903 },
	{ 0x95d4, 0x95d6, PDF_CMAP_TABLE, 2403 },
	{ 0x95dc, 0x95dc, PDF_CMAP_SINGLE, 3833 },
	{ 0x95e1, 0x95e2, PDF_CMAP_TABLE, 2406 },
	{ 0x961c, 0x961c, PDF_CMAP_SINGLE, 5147 },
	{ 0x9621, 0x9621, PDF_CMAP_SINGLE, 7165 },
	{ 0x962a, 0x962a, PDF_CMAP_SINGLE, 7498 },
	{ 0x962e, 0x962e, PDF_CMAP_SINGLE, 6123 },
	{ 0x9632, 0x9632, PDF_CMAP_SINGLE, 4967 },
	{ 0x963b, 0x963b, PDF_CMAP_SINGLE, 6825 },
	{ 0x963f, 0x9640, PDF_CMAP_TABLE, 2408 },
	{ 0x9642, 0x9642, PDF_CMAP_SINGLE, 7603 },
	{ 0x9644, 0x9644, PDF_CMAP_SINGLE, 5148 },
	{ 0x964b, 0x964d, PDF_CMAP_TABLE, 2410 },
	{ 0x9650, 0x9650, PDF_CMAP_SINGLE, 7647 },
	{ 0x965b, 0x965f, PDF_CMAP_TABLE, 2413 },
	{ 0x9662, 0x9664, PDF_CMAP_TABLE, 2418 },
	{ 0x966a, 0x966a, PDF_CMAP_SINGLE, 4987 },
	{ 0x9670, 0x9670, PDF_CMAP_SINGLE, 6384 },
	{ 0x9673, 0x9673, PDF_CMAP_SINGLE, 7013 },
	{ 0x9675, 0x9678, PDF_CMAP_TABLE, 2421 },
	{ 0x967d, 0x967d, PDF_CMAP_SINGLE, 5900 },
	{ 0x9685, 0x9686, PDF_CMAP_TABLE, 2425 },
	{ 0x968a, 0x968b, PDF_CMAP_TABLE, 2427 },
	{ 0x968d, 0x968e, PDF_CMAP_TABLE, 2429 },
	{ 0x9694, 0x9695, PDF_CMAP_TABLE, 2431 },
	{ 0x9698, 0x9699, PDF_CMAP_TABLE, 2433 },
	{ 0x969b, 0x969c, PDF_CMAP_TABLE, 2435 },
	{ 0x96a3, 0x96a3, PDF_CMAP_SINGLE, 4644 },
	{ 0x96a7, 0x96a8, PDF_CMAP_RANGE, 5620 },
	{ 0x96aa, 0x96aa, PDF_CMAP_SINGLE, 7728 },
	{ 0x96b1, 0x96b1, PDF_CMAP_SINGLE, 6379 },
	{ 0x96b7, 0x96b7, PDF_CMAP_SINGLE, 4510 },
	{ 0x96bb, 0x96bb, PDF_CMAP_SINGLE, 7147 },
	{ 0x96c0, 0x96c1, PDF_CMAP_TABLE, 2437 },
	{ 0x96c4, 0x96c7, PDF_CMAP_TABLE, 2439 },
	{ 0x96c9, 0x96c9, PDF_CMAP_SINGLE, 7357 },
	{ 0x96cb, 0x96ce, PDF_CMAP_TABLE, 2443 },
	{ 0x96d5, 0x96d6, PDF_CMAP_TABLE, 2447 },
	{ 0x96d9, 0x96d9, PDF_CMAP_SINGLE, 5773 },
	{ 0x96db, 0x96dc, PDF_CMAP_TABLE, 2449 },
	{ 0x96e2, 0x96e3, PDF_CMAP_TABLE, 2451 },
	{ 0x96e8, 0x96e9, PDF_CMAP_RANGE, 6227 },
	{ 0x96ea, 0x96ea, PDF_CMAP_SINGLE, 5461 },
	{ 0x96ef, 0x96f0, PDF_CMAP_TABLE, 2453 },
	{ 0x96f2, 0x96f2, PDF_CMAP_SINGLE, 6248 },
	{ 0x96f6, 0x96f7, PDF_CMAP_TABLE, 2455 },
	{ 0x96f9, 0x96f9, PDF_CMAP_SINGLE, 4903 },
	{ 0x96fb, 0x96fb, PDF_CMAP_SINGLE, 6680 },
	{ 0x9700, 0x9700, PDF_CMAP_SINGLE, 5623 },
	{ 0x9706, 0x9707, PDF_CMAP_TABLE, 2457 },
	{ 0x9711, 0x9711, PDF_CMAP_SINGLE, 6698 },
	{ 0x9713, 0x9713, PDF_CMAP_SINGLE, 6047 },
	{ 0x9716, 0x9716, PDF_CMAP_SINGLE, 4651 },
	{ 0x9719, 0x9719, PDF_CMAP_SINGLE, 6028 },
	{ 0x971c, 0x971c, PDF_CMAP_SINGLE, 5361 },
	{ 0x971e, 0x971e, PDF_CMAP_SINGLE, 7628 },
	{ 0x9727, 0x9727, PDF_CMAP_SINGLE, 4832 },
	{ 0x9730, 0x9730, PDF_CMAP_SINGLE, 5314 },
	{ 0x9732, 0x9732, PDF_CMAP_SINGLE, 4524 },
	{ 0x9739, 0x9739, PDF_CMAP_SINGLE, 5030 },
	{ 0x973d, 0x973d, PDF_CMAP_SINGLE, 6779 },
	{ 0x9742, 0x9742, PDF_CMAP_SINGLE, 4464 },
	{ 0x9744, 0x9744, PDF_CMAP_SINGLE, 5850 },
	{ 0x9748, 0x9748, PDF_CMAP_SINGLE, 4503 },
	{ 0x9751, 0x9751, PDF_CMAP_SINGLE, 7203 },
	{ 0x9756, 0x9756, PDF_CMAP_SINGLE, 6755 },
	{ 0x975c, 0x975c, PDF_CMAP_SINGLE, 6756 },
	{ 0x975e, 0x975e, PDF_CMAP_SINGLE, 5220 },
	{ 0x9761, 0x9762, PDF_CMAP_TABLE, 2459 },
	{ 0x9769, 0x9769, PDF_CMAP_SINGLE, 7733 },
	{ 0x976d, 0x976d, PDF_CMAP_SINGLE, 6458 },
	{ 0x9774, 0x9774, PDF_CMAP_SINGLE, 7876 },
	{ 0x9777, 0x9777, PDF_CMAP_SINGLE, 6459 },
	{ 0x977a, 0x977a, PDF_CMAP_SINGLE, 4695 },
	{ 0x978b, 0x978b, PDF_CMAP_SINGLE, 7799 },
	{ 0x978d, 0x978d, PDF_CMAP_SINGLE, 5814 },
	{ 0x978f, 0x978f, PDF_CMAP_SINGLE, 3801 },
	{ 0x97a0, 0x97a0, PDF_CMAP_SINGLE, 3950 },
	{ 0x97a8, 0x97a8, PDF_CMAP_SINGLE, 3509 },
	{ 0x97ab, 0x97ab, PDF_CMAP_SINGLE, 3951 },
	{ 0x97ad, 0x97ad, PDF_CMAP_SINGLE, 7526 },
	{ 0x97c6, 0x97c6, PDF_CMAP_SINGLE, 7166 },
	{ 0x97cb, 0x97cb, PDF_CMAP_SINGLE, 6307 },
	{ 0x97d3, 0x97d3, PDF_CMAP_SINGLE, 7648 },
	{ 0x97dc, 0x97dc, PDF_CMAP_SINGLE, 4308 },
	{ 0x97f3, 0x97f3, PDF_CMAP_SINGLE, 6385 },
	{ 0x97f6, 0x97f6, PDF_CMAP_SINGLE, 5535 },
	{ 0x97fb, 0x97fb, PDF_CMAP_SINGLE, 6249 },
	{ 0x97ff, 0x9803, PDF_CMAP_TABLE, 2461 },
	{ 0x9805, 0x9806, PDF_CMAP_TABLE, 2466 },
	{ 0x9808, 0x9808, PDF_CMAP_SINGLE, 5624 },
	{ 0x980a, 0x980a, PDF_CMAP_SINGLE, 6237 },
	{ 0x980c, 0x980c, PDF_CMAP_SINGLE, 5560 },
	{ 0x9810, 0x9813, PDF_CMAP_TABLE, 2468 },
	{ 0x9817, 0x9818, PDF_CMAP_TABLE, 2472 },
	{ 0x982d, 0x982d, PDF_CMAP_SINGLE, 4357 },
	{ 0x9830, 0x9830, PDF_CMAP_SINGLE, 7770 },
	{ 0x9838, 0x9839, PDF_CMAP_TABLE, 2474 },
	{ 0x983b, 0x983b, PDF_CMAP_SINGLE, 5236 },
	{ 0x9846, 0x9846, PDF_CMAP_SINGLE, 3814 },
	{ 0x984c, 0x984e, PDF_CMAP_TABLE, 2476 },
	{ 0x9854, 0x9854, PDF_CMAP_SINGLE, 5815 },
	{ 0x9858, 0x9858, PDF_CMAP_SINGLE, 6279 },
	{ 0x985a, 0x985a, PDF_CMAP_SINGLE, 6681 },
	{ 0x985e, 0x985e, PDF_CMAP_SINGLE, 4590 },
	{ 0x9865, 0x9865, PDF_CMAP_SINGLE, 7840 },
	{ 0x9867, 0x9867, PDF_CMAP_SINGLE, 3764 },
	{ 0x986b, 0x986b, PDF_CMAP_SINGLE, 6682 },
	{ 0x986f, 0x986f, PDF_CMAP_SINGLE, 7753 },
	{ 0x98a8, 0x98a8, PDF_CMAP_SINGLE, 7595 },
	{ 0x98af, 0x98af, PDF_CMAP_SINGLE, 5330 },
	{ 0x98b1, 0x98b1, PDF_CMAP_SINGLE, 7444 },
	{ 0x98c4, 0x98c4, PDF_CMAP_SINGLE, 7588 },
	{ 0x98c7, 0x98c7, PDF_CMAP_SINGLE, 7587 },
	{ 0x98db, 0x98dc, PDF_CMAP_TABLE, 2479 },
	{ 0x98df, 0x98df, PDF_CMAP_SINGLE, 5733 },
	{ 0x98e1, 0x98e2, PDF_CMAP_TABLE, 2481 },
	{ 0x98ed, 0x98ef, PDF_CMAP_TABLE, 2483 },
	{ 0x98f4, 0x98f4, PDF_CMAP_SINGLE, 6435 },
	{ 0x98fc, 0x98fe, PDF_CMAP_TABLE, 2486 },
	{ 0x9903, 0x9903, PDF_CMAP_SINGLE, 3890 },
	{ 0x9909, 0x990a, PDF_CMAP_TABLE, 2489 },
	{ 0x990c, 0x990c, PDF_CMAP_SINGLE, 6436 },
	{ 0x9910, 0x9910, PDF_CMAP_SINGLE, 7075 },
	{ 0x9913, 0x9913, PDF_CMAP_SINGLE, 5790 },
	{ 0x9918, 0x9918, PDF_CMAP_SINGLE, 5946 },
	{ 0x991e, 0x991e, PDF_CMAP_SINGLE, 6683 },
	{ 0x9920, 0x9920, PDF_CMAP_SINGLE, 5056 },
	{ 0x9928, 0x9928, PDF_CMAP_SINGLE, 3834 },
	{ 0x9945, 0x9945, PDF_CMAP_SINGLE, 4687 },
	{ 0x9949, 0x9949, PDF_CMAP_SINGLE, 4037 },
	{ 0x994b, 0x994d, PDF_CMAP_TABLE, 2491 },
	{ 0x9951, 0x9952, PDF_CMAP_TABLE, 2494 },
	{ 0x9954, 0x9954, PDF_CMAP_SINGLE, 6098 },
	{ 0x9957, 0x9957, PDF_CMAP_SINGLE, 7717 },
	{ 0x9996, 0x9996, PDF_CMAP_SINGLE, 5625 },
	{ 0x9999, 0x9999, PDF_CMAP_SINGLE, 7718 },
	{ 0x999d, 0x999d, PDF_CMAP_SINGLE, 7613 },
	{ 0x99a5, 0x99a5, PDF_CMAP_SINGLE, 5089 },
	{ 0x99a8, 0x99a8, PDF_CMAP_SINGLE, 7790 },
	{ 0x99ac, 0x99ae, PDF_CMAP_TABLE, 2496 },
	{ 0x99b1, 0x99b1, PDF_CMAP_SINGLE, 7392 },
	{ 0x99b3, 0x99b4, PDF_CMAP_TABLE, 2499 },
	{ 0x99b9, 0x99b9, PDF_CMAP_SINGLE, 6468 },
	{ 0x99c1, 0x99c1, PDF_CMAP_SINGLE, 4904 },
	{ 0x99d0, 0x99d2, PDF_CMAP_TABLE, 2501 },
	{ 0x99d5, 0x99d5, PDF_CMAP_SINGLE, 3464 },
	{ 0x99d9, 0x99d9, PDF_CMAP_SINGLE, 5149 },
	{ 0x99dd, 0x99dd, PDF_CMAP_SINGLE, 7393 },
	{ 0x99df, 0x99df, PDF_CMAP_SINGLE, 5299 },
	{ 0x99ed, 0x99ed, PDF_CMAP_SINGLE, 7701 },
	{ 0x99f1, 0x99f1, PDF_CMAP_SINGLE, 4388 },
	{ 0x99ff, 0x99ff, PDF_CMAP_SINGLE, 6920 },
	{ 0x9a01, 0x9a01, PDF_CMAP_SINGLE, 5240 },
	{ 0x9a08, 0x9a08, PDF_CMAP_SINGLE, 5057 },
	{ 0x9a0e, 0x9a0f, PDF_CMAP_RANGE, 4122 },
	{ 0x9a19, 0x9a19, PDF_CMAP_SINGLE, 7527 },
	{ 0x9a2b, 0x9a2b, PDF_CMAP_SINGLE, 3612 },
	{ 0x9a30, 0x9a30, PDF_CMAP_SINGLE, 4373 },
	{ 0x9a36, 0x9a37, PDF_CMAP_TABLE, 2504 },
	{ 0x9a40, 0x9a40, PDF_CMAP_SINGLE, 4725 },
	{ 0x9a43, 0x9a43, PDF_CMAP_SINGLE, 7589 },
	{ 0x9a45, 0x9a45, PDF_CMAP_SINGLE, 3943 },
	{ 0x9a4d, 0x9a4d, PDF_CMAP_SINGLE, 7965 },
	{ 0x9a55, 0x9a55, PDF_CMAP_SINGLE, 3891 },
	{ 0x9a57, 0x9a57, PDF_CMAP_SINGLE, 7729 },
	{ 0x9a5a, 0x9a5b, PDF_CMAP_TABLE, 2506 },
	{ 0x9a5f, 0x9a5f, PDF_CMAP_SINGLE, 7328 },
	{ 0x9a62, 0x9a62, PDF_CMAP_SINGLE, 4454 },
	{ 0x9a65, 0x9a65, PDF_CMAP_SINGLE, 4124 },
	{ 0x9a69, 0x9a6a, PDF_CMAP_TABLE, 2508 },
	{ 0x9aa8, 0x9aa8, PDF_CMAP_SINGLE, 3785 },
	{ 0x9ab8, 0x9ab8, PDF_CMAP_SINGLE, 7702 },
	{ 0x9ad3, 0x9ad4, PDF_CMAP_TABLE, 2510 },
	{ 0x9ad8, 0x9ad8, PDF_CMAP_SINGLE, 3765 },
	{ 0x9ae5, 0x9ae5, PDF_CMAP_SINGLE, 5997 },
	{ 0x9aee, 0x9aee, PDF_CMAP_SINGLE, 4939 },
	{ 0x9b1a, 0x9b1a, PDF_CMAP_SINGLE, 5627 },
	{ 0x9b27, 0x9b27, PDF_CMAP_SINGLE, 4562 },
	{ 0x9b2a, 0x9b2a, PDF_CMAP_SINGLE, 7470 },
	{ 0x9b31, 0x9b31, PDF_CMAP_SINGLE, 6251 },
	{ 0x9b3c, 0x9b3c, PDF_CMAP_SINGLE, 3993 },
	{ 0x9b41, 0x9b45, PDF_CMAP_TABLE, 2512 },
	{ 0x9b4f, 0x9b4f, PDF_CMAP_SINGLE, 6308 },
	{ 0x9b54, 0x9b54, PDF_CMAP_SINGLE, 4662 },
	{ 0x9b5a, 0x9b5a, PDF_CMAP_SINGLE, 5910 },
	{ 0x9b6f, 0x9b6f, PDF_CMAP_SINGLE, 4525 },
	{ 0x9b8e, 0x9b8e, PDF_CMAP_SINGLE, 6699 },
	{ 0x9b91, 0x9b91, PDF_CMAP_SINGLE, 7570 },
	{ 0x9b9f, 0x9b9f, PDF_CMAP_SINGLE, 5816 },
	{ 0x9bab, 0x9bab, PDF_CMAP_SINGLE, 3892 },
	{ 0x9bae, 0x9bae, PDF_CMAP_SINGLE, 5449 },
	{ 0x9bc9, 0x9bc9, PDF_CMAP_SINGLE, 4637 },
	{ 0x9bd6, 0x9bd6, PDF_CMAP_SINGLE, 7204 },
	{ 0x9be4, 0x9be4, PDF_CMAP_SINGLE, 3783 },
	{ 0x9be8, 0x9be8, PDF_CMAP_SINGLE, 3704 },
	{ 0x9c0d, 0x9c0d, PDF_CMAP_SINGLE, 7287 },
	{ 0x9c10, 0x9c10, PDF_CMAP_SINGLE, 5805 },
	{ 0x9c12, 0x9c12, PDF_CMAP_SINGLE, 5090 },
	{ 0x9c15, 0x9c15, PDF_CMAP_SINGLE, 7629 },
	{ 0x9c25, 0x9c25, PDF_CMAP_SINGLE, 7898 },
	{ 0x9c32, 0x9c32, PDF_CMAP_SINGLE, 6076 },
	{ 0x9c3b, 0x9c3b, PDF_CMAP_SINGLE, 4688 },
	{ 0x9c47, 0x9c47, PDF_CMAP_SINGLE, 3559 },
	{ 0x9c49, 0x9c49, PDF_CMAP_SINGLE, 5039 },
	{ 0x9c57, 0x9c57, PDF_CMAP_SINGLE, 4645 },
	{ 0x9ce5, 0x9ce5, PDF_CMAP_SINGLE, 6827 },
	{ 0x9ce7, 0x9ce7, PDF_CMAP_SINGLE, 5150 },
	{ 0x9ce9, 0x9ce9, PDF_CMAP_SINGLE, 3944 },
	{ 0x9cf3, 0x9cf4, PDF_CMAP_TABLE, 2517 },
	{ 0x9cf6, 0x9cf6, PDF_CMAP_SINGLE, 5984 },
	{ 0x9d09, 0x9d09, PDF_CMAP_SINGLE, 5791 },
	{ 0x9d1b, 0x9d1b, PDF_CMAP_SINGLE, 6280 },
	{ 0x9d26, 0x9d26, PDF_CMAP_SINGLE, 5839 },
	{ 0x9d28, 0x9d28, PDF_CMAP_SINGLE, 5832 },
	{ 0x9d3b, 0x9d3b, PDF_CMAP_SINGLE, 7862 },
	{ 0x9d51, 0x9d51, PDF_CMAP_SINGLE, 3647 },
	{ 0x9d5d, 0x9d5d, PDF_CMAP_SINGLE, 5792 },
	{ 0x9d60, 0x9d61, PDF_CMAP_TABLE, 2519 },
	{ 0x9d6c, 0x9d6c, PDF_CMAP_SINGLE, 5180 },
	{ 0x9d72, 0x9d72, PDF_CMAP_SINGLE, 6521 },
	{ 0x9da9, 0x9da9, PDF_CMAP_SINGLE, 4794 },
	{ 0x9daf, 0x9daf, PDF_CMAP_SINGLE, 5860 },
	{ 0x9db4, 0x9db4, PDF_CMAP_SINGLE, 7634 },
	{ 0x9dc4, 0x9dc4, PDF_CMAP_SINGLE, 3728 },
	{ 0x9dd7, 0x9dd7, PDF_CMAP_SINGLE, 3945 },
	{ 0x9df2, 0x9df2, PDF_CMAP_SINGLE, 7329 },
	{ 0x9df8, 0x9dfa, PDF_CMAP_TABLE, 2521 },
	{ 0x9e1a, 0x9e1a, PDF_CMAP_SINGLE, 5861 },
	{ 0x9e1e, 0x9e1e, PDF_CMAP_SINGLE, 4396 },
	{ 0x9e75, 0x9e75, PDF_CMAP_SINGLE, 4527 },
	{ 0x9e79, 0x9e79, PDF_CMAP_SINGLE, 7662 },
	{ 0x9e7d, 0x9e7d, PDF_CMAP_SINGLE, 5998 },
	{ 0x9e7f, 0x9e7f, PDF_CMAP_SINGLE, 4533 },
	{ 0x9e92, 0x9e93, PDF_CMAP_TABLE, 2524 },
	{ 0x9e97, 0x9e97, PDF_CMAP_SINGLE, 4456 },
	{ 0x9e9d, 0x9e9d, PDF_CMAP_SINGLE, 5300 },
	{ 0x9e9f, 0x9e9f, PDF_CMAP_SINGLE, 4646 },
	{ 0x9ea5, 0x9ea5, PDF_CMAP_SINGLE, 4726 },
	{ 0x9eb4, 0x9eb5, PDF_CMAP_TABLE, 2526 },
	{ 0x9ebb, 0x9ebb, PDF_CMAP_SINGLE, 4663 },
	{ 0x9ebe, 0x9ebe, PDF_CMAP_SINGLE, 8004 },
	{ 0x9ec3, 0x9ec3, PDF_CMAP_SINGLE, 7927 },
	{ 0x9ecd, 0x9ece, PDF_CMAP_TABLE, 2528 },
	{ 0x9ed1, 0x9ed1, PDF_CMAP_SINGLE, 8018 },
	{ 0x9ed4, 0x9ed4, PDF_CMAP_SINGLE, 3623 },
	{ 0x9ed8, 0x9ed8, PDF_CMAP_SINGLE, 4835 },
	{ 0x9edb, 0x9edc, PDF_CMAP_TABLE, 2530 },
	{ 0x9ede, 0x9ede, PDF_CMAP_SINGLE, 6700 },
	{ 0x9ee8, 0x9ee8, PDF_CMAP_SINGLE, 4249 },
	{ 0x9ef4, 0x9ef4, PDF_CMAP_SINGLE, 4869 },
	{ 0x9f07, 0x9f08, PDF_CMAP_TABLE, 2532 },
	{ 0x9f0e, 0x9f0e, PDF_CMAP_SINGLE, 6758 },
	{ 0x9f13, 0x9f13, PDF_CMAP_SINGLE, 3766 },
	{ 0x9f20, 0x9f20, PDF_CMAP_SINGLE, 5402 },
	{ 0x9f3b, 0x9f3b, PDF_CMAP_SINGLE, 5222 },
	{ 0x9f4a, 0x9f4b, PDF_CMAP_TABLE, 2534 },
	{ 0x9f4e, 0x9f4e, PDF_CMAP_SINGLE, 6586 },
	{ 0x9f52, 0x9f52, PDF_CMAP_SINGLE, 7359 },
	{ 0x9f5f, 0x9f5f, PDF_CMAP_SINGLE, 6618 },
	{ 0x9f61, 0x9f61, PDF_CMAP_SINGLE, 4505 },
	{ 0x9f67, 0x9f67, PDF_CMAP_SINGLE, 5462 },
	{ 0x9f6a, 0x9f6a, PDF_CMAP_SINGLE, 7061 },
	{ 0x9f6c, 0x9f6c, PDF_CMAP_SINGLE, 5911 },
	{ 0x9f77, 0x9f77, PDF_CMAP_SINGLE, 5806 },
	{ 0x9f8d, 0x9f8d, PDF_CMAP_SINGLE, 4563 },
	{ 0x9f90, 0x9f90, PDF_CMAP_SINGLE, 4968 },
	{ 0x9f95, 0x9f95, PDF_CMAP_SINGLE, 3529 },
	{ 0x9f9c, 0x9f9c, PDF_CMAP_SINGLE, 3946 },
	{ 0xac00, 0xac01, PDF_CMAP_RANGE, 1086 },
	{ 0xac02, 0xac03, PDF_CMAP_RANGE, 9333 },
	{ 0xac04, 0xac04, PDF_CMAP_SINGLE, 1088 },
	{ 0xac05, 0xac06, PDF_CMAP_RANGE, 9335 },
	{ 0xac07, 0xac0a, PDF_CMAP_RANGE, 1089 },
	{ 0xac0b, 0xac0f, PDF_CMAP_RANGE, 9337 },
	{ 0xac10, 0xac17, PDF_CMAP_RANGE, 1093 },
	{ 0xac18, 0xac18, PDF_CMAP_SINGLE, 9342 },
	{ 0xac19, 0xac1d, PDF_CMAP_RANGE, 1101 },
	{ 0xac1e, 0xac1f, PDF_CMAP_RANGE, 9343 },
	{ 0xac20, 0xac20, PDF_CMAP_SINGLE, 1106 },
	{ 0xac21, 0xac23, PDF_CMAP_RANGE, 9345 },
	{ 0xac24, 0xac24, PDF_CMAP_SINGLE, 1107 },
	{ 0xac25, 0xac2b, PDF_CMAP_RANGE, 9348 },
	{ 0xac2c, 0xac2d, PDF_CMAP_RANGE, 1108 },
	{ 0xac2e, 0xac2e, PDF_CMAP_SINGLE, 9355 },
	{ 0xac2f, 0xac31, PDF_CMAP_RANGE, 1110 },
	{ 0xac32, 0xac37, PDF_CMAP_RANGE, 9356 },
	{ 0xac38, 0xac39, PDF_CMAP_RANGE, 1113 },
	{ 0xac3a, 0xac3b, PDF_CMAP_RANGE, 9362 },
	{ 0xac3c, 0xac3c, PDF_CMAP_SINGLE, 1115 },
	{ 0xac3d, 0xac3f, PDF_CMAP_RANGE, 9364 },
	{ 0xac40, 0xac40, PDF_CMAP_SINGLE, 1116 },
	{ 0xac41, 0xac4a, PDF_CMAP_RANGE, 9367 },
	{ 0xac4b, 0xac4d, PDF_CMAP_TABLE, 2536 },
	{ 0xac4e, 0xac53, PDF_CMAP_RANGE, 9378 },
	{ 0xac54, 0xac54, PDF_CMAP_SINGLE, 1119 },
	{ 0xac55, 0xac57, PDF_CMAP_RANGE, 9384 },
	{ 0xac58, 0xac58, PDF_CMAP_SINGLE, 1120 },
	{ 0xac59, 0xac5b, PDF_CMAP_RANGE, 9387 },
	{ 0xac5c, 0xac5c, PDF_CMAP_SINGLE, 1121 },
	{ 0xac5d, 0xac6f, PDF_CMAP_RANGE, 9390 },
	{ 0xac70, 0xac71, PDF_CMAP_RANGE, 1122 },
	{ 0xac72, 0xac73, PDF_CMAP_RANGE, 9409 },
	{ 0xac74, 0xac74, PDF_CMAP_SINGLE, 1124 },
	{ 0xac75, 0xac76, PDF_CMAP_RANGE, 9411 },
	{ 0xac77, 0xac78, PDF_CMAP_RANGE, 1125 },
	{ 0xac79, 0xac7a, PDF_CMAP_TABLE, 2539 },
	{ 0xac7b, 0xac7f, PDF_CMAP_RANGE, 9414 },
	{ 0xac80, 0xac81, PDF_CMAP_RANGE, 1128 },
	{ 0xac82, 0xac82, PDF_CMAP_SINGLE, 9419 },
	{ 0xac83, 0xac86, PDF_CMAP_RANGE, 1130 },
	{ 0xac87, 0xac88, PDF_CMAP_RANGE, 9420 },
	{ 0xac89, 0xac8c, PDF_CMAP_RANGE, 1134 },
	{ 0xac8d, 0xac8f, PDF_CMAP_RANGE, 9422 },
	{ 0xac90, 0xac90, PDF_CMAP_SINGLE, 1138 },
	{ 0xac91, 0xac93, PDF_CMAP_RANGE, 9425 },
	{ 0xac94, 0xac94, PDF_CMAP_SINGLE, 1139 },
	{ 0xac95, 0xac9b, PDF_CMAP_RANGE, 9428 },
	{ 0xac9c, 0xac9d, PDF_CMAP_RANGE, 1140 },
	{ 0xac9e, 0xac9e, PDF_CMAP_SINGLE, 9435 },
	{ 0xac9f, 0xaca1, PDF_CMAP_RANGE, 1142 },
	{ 0xaca2, 0xaca7, PDF_CMAP_RANGE, 9436 },
	{ 0xaca8, 0xacaa, PDF_CMAP_RANGE, 1145 },
	{ 0xacab, 0xacac, PDF_CMAP_TABLE, 2541 },
	{ 0xacad, 0xacae, PDF_CMAP_RANGE, 9443 },
	{ 0xacaf, 0xacb0, PDF_CMAP_RANGE, 1149 },
	{ 0xacb1, 0xacb7, PDF_CMAP_RANGE, 9445 },
	{ 0xacb8, 0xacb9, PDF_CMAP_RANGE, 1151 },
	{ 0xacba, 0xacba, PDF_CMAP_SINGLE, 9452 },
	{ 0xacbb, 0xacbd, PDF_CMAP_RANGE, 1153 },
	{ 0xacbe, 0xacc0, PDF_CMAP_RANGE, 9453 },
	{ 0xacc1, 0xacc1, PDF_CMAP_SINGLE, 1156 },
	{ 0xacc2, 0xacc3, PDF_CMAP_RANGE, 9456 },
	{ 0xacc4, 0xacc4, PDF_CMAP_SINGLE, 1157 },
	{ 0xacc5, 0xacc7, PDF_CMAP_RANGE, 9458 },
	{ 0xacc8, 0xacc8, PDF_CMAP_SINGLE, 1158 },
	{ 0xacc9, 0xaccb, PDF_CMAP_RANGE, 9461 },
	{ 0xaccc, 0xaccc, PDF_CMAP_SINGLE, 1159 },
	{ 0xaccd, 0xacd4, PDF_CMAP_RANGE, 9464 },
	{ 0xacd5, 0xacd7, PDF_CMAP_TABLE, 2543 },
	{ 0xacd8, 0xacdf, PDF_CMAP_RANGE, 9473 },
	{ 0xace0, 0xace1, PDF_CMAP_RANGE, 1162 },
	{ 0xace2, 0xace3, PDF_CMAP_RANGE, 9481 },
	{ 0xace4, 0xace4, PDF_CMAP_SINGLE, 1164 },
	{ 0xace5, 0xace6, PDF_CMAP_RANGE, 9483 },
	{ 0xace7, 0xace8, PDF_CMAP_RANGE, 1165 },
	{ 0xace9, 0xacec, PDF_CMAP_TABLE, 2546 },
	{ 0xaced, 0xacee, PDF_CMAP_RANGE, 9487 },
	{ 0xacef, 0xacf1, PDF_CMAP_RANGE, 1169 },
	{ 0xacf2, 0xacf4, PDF_CMAP_TABLE, 2550 },
	{ 0xacf5, 0xacf6, PDF_CMAP_RANGE, 1173 },
	{ 0xacf7, 0xacfb, PDF_CMAP_RANGE, 9491 },
	{ 0xacfc, 0xacfd, PDF_CMAP_RANGE, 1175 },
	{ 0xacfe, 0xacff, PDF_CMAP_RANGE, 9496 },
	{ 0xad00, 0xad00, PDF_CMAP_SINGLE, 1177 },
	{ 0xad01, 0xad03, PDF_CMAP_RANGE, 9498 },
	{ 0xad04, 0xad06, PDF_CMAP_TABLE, 2553 },
	{ 0xad07, 0xad0b, PDF_CMAP_RANGE, 9502 },
	{ 0xad0c, 0xad0d, PDF_CMAP_RANGE, 1180 },
	{ 0xad0e, 0xad11, PDF_CMAP_TABLE, 2556 },
	{ 0xad12, 0xad17, PDF_CMAP_RANGE, 9509 },
	{ 0xad18, 0xad18, PDF_CMAP_SINGLE, 1184 },
	{ 0xad19, 0xad1b, PDF_CMAP_RANGE, 9515 },
	{ 0xad1c, 0xad1c, PDF_CMAP_SINGLE, 1185 },
	{ 0xad1d, 0xad1f, PDF_CMAP_RANGE, 9518 },
	{ 0xad20, 0xad20, PDF_CMAP_SINGLE, 1186 },
	{ 0xad21, 0xad28, PDF_CMAP_RANGE, 9521 },
	{ 0xad29, 0xad29, PDF_CMAP_SINGLE, 1187 },
	{ 0xad2a, 0xad2b, PDF_CMAP_RANGE, 9529 },
	{ 0xad2c, 0xad2d, PDF_CMAP_RANGE, 1188 },
	{ 0xad2e, 0xad33, PDF_CMAP_RANGE, 9531 },
	{ 0xad34, 0xad35, PDF_CMAP_RANGE, 1190 },
	{ 0xad36, 0xad37, PDF_CMAP_RANGE, 9537 },
	{ 0xad38, 0xad38, PDF_CMAP_SINGLE, 1192 },
	{ 0xad39, 0xad3b, PDF_CMAP_RANGE, 9539 },
	{ 0xad3c, 0xad3c, PDF_CMAP_SINGLE, 1193 },
	{ 0xad3d, 0xad43, PDF_CMAP_RANGE, 9542 },
	{ 0xad44, 0xad45, PDF_CMAP_RANGE, 1194 },
	{ 0xad46, 0xad49, PDF_CMAP_TABLE, 2560 },
	{ 0xad4a, 0xad4f, PDF_CMAP_RANGE, 9551 },
	{ 0xad50, 0xad50, PDF_CMAP_SINGLE, 1198 },
	{ 0xad51, 0xad53, PDF_CMAP_RANGE, 9557 },
	{ 0xad54, 0xad54, PDF_CMAP_SINGLE, 1199 },
	{ 0xad55, 0xad57, PDF_CMAP_RANGE, 9560 },
	{ 0xad58, 0xad58, PDF_CMAP_SINGLE, 1200 },
	{ 0xad59, 0xad60, PDF_CMAP_RANGE, 9563 },
	{ 0xad61, 0xad63, PDF_CMAP_TABLE, 2564 },
	{ 0xad64, 0xad6b, PDF_CMAP_RANGE, 9572 },
	{ 0xad6c, 0xad6d, PDF_CMAP_RANGE, 1203 },
	{ 0xad6e, 0xad6f, PDF_CMAP_RANGE, 9580 },
	{ 0xad70, 0xad70, PDF_CMAP_SINGLE, 1205 },
	{ 0xad71, 0xad72, PDF_CMAP_RANGE, 9582 },
	{ 0xad73, 0xad76, PDF_CMAP_RANGE, 1206 },
	{ 0xad77, 0xad7a, PDF_CMAP_RANGE, 9584 },
	{ 0xad7b, 0xad7d, PDF_CMAP_RANGE, 1210 },
	{ 0xad7e, 0xad80, PDF_CMAP_TABLE, 2567 },
	{ 0xad81, 0xad82, PDF_CMAP_RANGE, 1214 },
	{ 0xad83, 0xad87, PDF_CMAP_RANGE, 9590 },
	{ 0xad88, 0xad89, PDF_CMAP_RANGE, 1216 },
	{ 0xad8a, 0xad8b, PDF_CMAP_RANGE, 9595 },
	{ 0xad8c, 0xad8c, PDF_CMAP_SINGLE, 1218 },
	{ 0xad8d, 0xad8f, PDF_CMAP_RANGE, 9597 },
	{ 0xad90, 0xad90, PDF_CMAP_SINGLE, 1219 },
	{ 0xad91, 0xad9b, PDF_CMAP_RANGE, 9600 },
	{ 0xad9c, 0xad9d, PDF_CMAP_RANGE, 1220 },
	{ 0xad9e, 0xada3, PDF_CMAP_RANGE, 9611 },
	{ 0xada4, 0xada4, PDF_CMAP_SINGLE, 1222 },
	{ 0xada5, 0xadb6, PDF_CMAP_RANGE, 9617 },
	{ 0xadb7, 0xadb7, PDF_CMAP_SINGLE, 1223 },
	{ 0xadb8, 0xadbf, PDF_CMAP_RANGE, 9635 },
	{ 0xadc0, 0xadc1, PDF_CMAP_RANGE, 1224 },
	{ 0xadc2, 0xadc3, PDF_CMAP_RANGE, 9643 },
	{ 0xadc4, 0xadc4, PDF_CMAP_SINGLE, 1226 },
	{ 0xadc5, 0xadc7, PDF_CMAP_RANGE, 9645 },
	{ 0xadc8, 0xadc8, PDF_CMAP_SINGLE, 1227 },
	{ 0xadc9, 0xadcf, PDF_CMAP_RANGE, 9648 },
	{ 0xadd0, 0xadd1, PDF_CMAP_RANGE, 1228 },
	{ 0xadd2, 0xadd3, PDF_CMAP_TABLE, 2570 },
	{ 0xadd4, 0xaddb, PDF_CMAP_RANGE, 9656 },
	{ 0xaddc, 0xaddc, PDF_CMAP_SINGLE, 1231 },
	{ 0xaddd, 0xaddf, PDF_CMAP_RANGE, 9664 },
	{ 0xade0, 0xade0, PDF_CMAP_SINGLE, 1232 },
	{ 0xade1, 0xade3, PDF_CMAP_RANGE, 9667 },
	{ 0xade4, 0xade4, PDF_CMAP_SINGLE, 1233 },
	{ 0xade5, 0xadf7, PDF_CMAP_RANGE, 9670 },
	{ 0xadf8, 0xadf9, PDF_CMAP_RANGE, 1234 },
	{ 0xadfa, 0xadfb, PDF_CMAP_RANGE, 9689 },
	{ 0xadfc, 0xadfc, PDF_CMAP_SINGLE, 1236 },
	{ 0xadfd, 0xadfe, PDF_CMAP_RANGE, 9691 },
	{ 0xadff, 0xae01, PDF_CMAP_RANGE, 1237 },
	{ 0xae02, 0xae07, PDF_CMAP_RANGE, 9693 },
	{ 0xae08, 0xae09, PDF_CMAP_RANGE, 1240 },
	{ 0xae0a, 0xae0d, PDF_CMAP_TABLE, 2572 },
	{ 0xae0e, 0xae13, PDF_CMAP_RANGE, 9701 },
	{ 0xae14, 0xae14, PDF_CMAP_SINGLE, 1244 },
	{ 0xae15, 0xae2f, PDF_CMAP_RANGE, 9707 },
	{ 0xae30, 0xae31, PDF_CMAP_RANGE, 1245 },
	{ 0xae32, 0xae33, PDF_CMAP_RANGE, 9734 },
	{ 0xae34, 0xae34, PDF_CMAP_SINGLE, 1247 },
	{ 0xae35, 0xae36, PDF_CMAP_RANGE, 9736 },
	{ 0xae37, 0xae38, PDF_CMAP_RANGE, 1248 },
	{ 0xae39, 0xae3a, PDF_CMAP_TABLE, 2576 },
	{ 0xae3b, 0xae3f, PDF_CMAP_RANGE, 9739 },
	{ 0xae40, 0xae41, PDF_CMAP_RANGE, 1251 },
	{ 0xae42, 0xae44, PDF_CMAP_TABLE, 2578 },
	{ 0xae45, 0xae46, PDF_CMAP_RANGE, 1254 },
	{ 0xae47, 0xae49, PDF_CMAP_RANGE, 9746 },
	{ 0xae4a, 0xae4b, PDF_CMAP_TABLE, 2581 },
	{ 0xae4c, 0xae4e, PDF_CMAP_RANGE, 1257 },
	{ 0xae4f, 0xae50, PDF_CMAP_TABLE, 2583 },
	{ 0xae51, 0xae53, PDF_CMAP_RANGE, 9751 },
	{ 0xae54, 0xae56, PDF_CMAP_TABLE, 2585 },
	{ 0xae57, 0xae5b, PDF_CMAP_RANGE, 9755 },
	{ 0xae5c, 0xae5d, PDF_CMAP_RANGE, 1263 },
	{ 0xae5e, 0xae5e, PDF_CMAP_SINGLE, 9760 },
	{ 0xae5f, 0xae61, PDF_CMAP_RANGE, 1265 },
	{ 0xae62, 0xae64, PDF_CMAP_RANGE, 9761 },
	{ 0xae65, 0xae65, PDF_CMAP_SINGLE, 1268 },
	{ 0xae66, 0xae67, PDF_CMAP_RANGE, 9764 },
	{ 0xae68, 0xae69, PDF_CMAP_RANGE, 1269 },
	{ 0xae6a, 0xae6b, PDF_CMAP_RANGE, 9766 },
	{ 0xae6c, 0xae6c, PDF_CMAP_SINGLE, 1271 },
	{ 0xae6d, 0xae6f, PDF_CMAP_RANGE, 9768 },
	{ 0xae70, 0xae70, PDF_CMAP_SINGLE, 1272 },
	{ 0xae71, 0xae77, PDF_CMAP_RANGE, 9771 },
	{ 0xae78, 0xae79, PDF_CMAP_RANGE, 1273 },
	{ 0xae7a, 0xae7a, PDF_CMAP_SINGLE, 9778 },
	{ 0xae7b, 0xae7d, PDF_CMAP_RANGE, 1275 },
	{ 0xae7e, 0xae83, PDF_CMAP_RANGE, 9779 },
	{ 0xae84, 0xae85, PDF_CMAP_RANGE, 1278 },
	{ 0xae86, 0xae8b, PDF_CMAP_RANGE, 9785 },
	{ 0xae8c, 0xae8c, PDF_CMAP_SINGLE, 1280 },
	{ 0xae8d, 0xaebb, PDF_CMAP_RANGE, 9791 },
	{ 0xaebc, 0xaebe, PDF_CMAP_RANGE, 1281 },
	{ 0xaebf, 0xaec0, PDF_CMAP_TABLE, 2588 },
	{ 0xaec1, 0xaec3, PDF_CMAP_RANGE, 9839 },
	{ 0xaec4, 0xaec4, PDF_CMAP_SINGLE, 1285 },
	{ 0xaec5, 0xaecb, PDF_CMAP_RANGE, 9842 },
	{ 0xaecc, 0xaecd, PDF_CMAP_RANGE, 1286 },
	{ 0xaece, 0xaece, PDF_CMAP_SINGLE, 9849 },
	{ 0xaecf, 0xaed1, PDF_CMAP_RANGE, 1288 },
	{ 0xaed2, 0xaed7, PDF_CMAP_RANGE, 9850 },
	{ 0xaed8, 0xaed9, PDF_CMAP_RANGE, 1291 },
	{ 0xaeda, 0xaedb, PDF_CMAP_RANGE, 9856 },
	{ 0xaedc, 0xaedc, PDF_CMAP_SINGLE, 1293 },
	{ 0xaedd, 0xaee7, PDF_CMAP_RANGE, 9858 },
	{ 0xaee8, 0xaee8, PDF_CMAP_SINGLE, 1294 },
	{ 0xaee9, 0xaeea, PDF_CMAP_RANGE, 9869 },
	{ 0xaeeb, 0xaeed, PDF_CMAP_TABLE, 2590 },
	{ 0xaeee, 0xaef3, PDF_CMAP_RANGE, 9872 },
	{ 0xaef4, 0xaef4, PDF_CMAP_SINGLE, 1297 },
	{ 0xaef5, 0xaef7, PDF_CMAP_RANGE, 9878 },
	{ 0xaef8, 0xaef8, PDF_CMAP_SINGLE, 1298 },
	{ 0xaef9, 0xaefb, PDF_CMAP_RANGE, 9881 },
	{ 0xaefc, 0xaefc, PDF_CMAP_SINGLE, 1299 },
	{ 0xaefd, 0xaf06, PDF_CMAP_RANGE, 9884 },
	{ 0xaf07, 0xaf08, PDF_CMAP_RANGE, 1300 },
	{ 0xaf09, 0xaf0c, PDF_CMAP_RANGE, 9894 },
	{ 0xaf0d, 0xaf0d, PDF_CMAP_SINGLE, 1302 },
	{ 0xaf0e, 0xaf0f, PDF_CMAP_RANGE, 9898 },
	{ 0xaf10, 0xaf10, PDF_CMAP_SINGLE, 1303 },
	{ 0xaf11, 0xaf2b, PDF_CMAP_RANGE, 9900 },
	{ 0xaf2c, 0xaf2d, PDF_CMAP_RANGE, 1304 },
	{ 0xaf2e, 0xaf2f, PDF_CMAP_RANGE, 9927 },
	{ 0xaf30, 0xaf34, PDF_CMAP_TABLE, 2593 },
	{ 0xaf35, 0xaf3b, PDF_CMAP_RANGE, 9931 },
	{ 0xaf3c, 0xaf3d, PDF_CMAP_RANGE, 1309 },
	{ 0xaf3e, 0xaf40, PDF_CMAP_TABLE, 2598 },
	{ 0xaf41, 0xaf43, PDF_CMAP_RANGE, 1312 },
	{ 0xaf44, 0xaf47, PDF_CMAP_RANGE, 9940 },
	{ 0xaf48, 0xaf49, PDF_CMAP_RANGE, 1315 },
	{ 0xaf4a, 0xaf4f, PDF_CMAP_RANGE, 9944 },
	{ 0xaf50, 0xaf50, PDF_CMAP_SINGLE, 1317 },
	{ 0xaf51, 0xaf5b, PDF_CMAP_RANGE, 9950 },
	{ 0xaf5c, 0xaf5d, PDF_CMAP_RANGE, 1318 },
	{ 0xaf5e, 0xaf63, PDF_CMAP_RANGE, 9961 },
	{ 0xaf64, 0xaf65, PDF_CMAP_RANGE, 1320 },
	{ 0xaf66, 0xaf78, PDF_CMAP_RANGE, 9967 },
	{ 0xaf79, 0xaf79, PDF_CMAP_SINGLE, 1322 },
	{ 0xaf7a, 0xaf7f, PDF_CMAP_RANGE, 9986 },
	{ 0xaf80, 0xaf80, PDF_CMAP_SINGLE, 1323 },
	{ 0xaf81, 0xaf83, PDF_CMAP_RANGE, 9992 },
	{ 0xaf84, 0xaf84, PDF_CMAP_SINGLE, 1324 },
	{ 0xaf85, 0xaf87, PDF_CMAP_RANGE, 9995 },
	{ 0xaf88, 0xaf88, PDF_CMAP_SINGLE, 1325 },
	{ 0xaf89, 0xaf8f, PDF_CMAP_RANGE, 9998 },
	{ 0xaf90, 0xaf91, PDF_CMAP_RANGE, 1326 },
	{ 0xaf92, 0xaf94, PDF_CMAP_RANGE, 10005 },
	{ 0xaf95, 0xaf95, PDF_CMAP_SINGLE, 1328 },
	{ 0xaf96, 0xaf9b, PDF_CMAP_RANGE, 10008 },
	{ 0xaf9c, 0xaf9c, PDF_CMAP_SINGLE, 1329 },
	{ 0xaf9d, 0xafb7, PDF_CMAP_RANGE, 10014 },
	{ 0xafb8, 0xafb9, PDF_CMAP_RANGE, 1330 },
	{ 0xafba, 0xafbb, PDF_CMAP_RANGE, 10041 },
	{ 0xafbc, 0xafbc, PDF_CMAP_SINGLE, 1332 },
	{ 0xafbd, 0xafbf, PDF_CMAP_RANGE, 10043 },
	{ 0xafc0, 0xafc0, PDF_CMAP_SINGLE, 1333 },
	{ 0xafc1, 0xafc6, PDF_CMAP_RANGE, 10046 },
	{ 0xafc7, 0xafc9, PDF_CMAP_RANGE, 1334 },
	{ 0xafca, 0xafcc, PDF_CMAP_TABLE, 2601 },
	{ 0xafcd, 0xafce, PDF_CMAP_RANGE, 1338 },
	{ 0xafcf, 0xafd3, PDF_CMAP_RANGE, 10054 },
	{ 0xafd4, 0xafd4, PDF_CMAP_SINGLE, 1340 },
	{ 0xafd5, 0xafdb, PDF_CMAP_RANGE, 10059 },
	{ 0xafdc, 0xafdc, PDF_CMAP_SINGLE, 1341 },
	{ 0xafdd, 0xafe7, PDF_CMAP_RANGE, 10066 },
	{ 0xafe8, 0xafe9, PDF_CMAP_RANGE, 1342 },
	{ 0xafea, 0xafef, PDF_CMAP_RANGE, 10077 },
	{ 0xaff0, 0xaff1, PDF_CMAP_RANGE, 1344 },
	{ 0xaff2, 0xaff3, PDF_CMAP_RANGE, 10083 },
	{ 0xaff4, 0xaff4, PDF_CMAP_SINGLE, 1346 },
	{ 0xaff5, 0xaff7, PDF_CMAP_RANGE, 10085 },
	{ 0xaff8, 0xaff8, PDF_CMAP_SINGLE, 1347 },
	{ 0xaff9, 0xafff, PDF_CMAP_RANGE, 10088 },
	{ 0xb000, 0xb001, PDF_CMAP_RANGE, 1348 },
	{ 0xb002, 0xb003, PDF_CMAP_RANGE, 10095 },
	{ 0xb004, 0xb004, PDF_CMAP_SINGLE, 1350 },
	{ 0xb005, 0xb00b, PDF_CMAP_RANGE, 10097 },
	{ 0xb00c, 0xb00c, PDF_CMAP_SINGLE, 1351 },
	{ 0xb00d, 0xb00f, PDF_CMAP_RANGE, 10104 },
	{ 0xb010, 0xb010, PDF_CMAP_SINGLE, 1352 },
	{ 0xb011, 0xb013, PDF_CMAP_RANGE, 10107 },
	{ 0xb014, 0xb014, PDF_CMAP_SINGLE, 1353 },
	{ 0xb015, 0xb01b, PDF_CMAP_RANGE, 10110 },
	{ 0xb01c, 0xb01d, PDF_CMAP_RANGE, 1354 },
	{ 0xb01e, 0xb027, PDF_CMAP_RANGE, 10117 },
	{ 0xb028, 0xb028, PDF_CMAP_SINGLE, 1356 },
	{ 0xb029, 0xb043, PDF_CMAP_RANGE, 10127 },
	{ 0xb044, 0xb045, PDF_CMAP_RANGE, 1357 },
	{ 0xb046, 0xb047, PDF_CMAP_RANGE, 10154 },
	{ 0xb048, 0xb04e, PDF_CMAP_TABLE, 2604 },
	{ 0xb04f, 0xb052, PDF_CMAP_RANGE, 10159 },
	{ 0xb053, 0xb055, PDF_CMAP_RANGE, 1363 },
	{ 0xb056, 0xb059, PDF_CMAP_TABLE, 2611 },
	{ 0xb05a, 0xb05c, PDF_CMAP_RANGE, 10165 },
	{ 0xb05d, 0xb05d, PDF_CMAP_SINGLE, 1368 },
	{ 0xb05e, 0xb07b, PDF_CMAP_RANGE, 10168 },
	{ 0xb07c, 0xb07d, PDF_CMAP_RANGE, 1369 },
	{ 0xb07e, 0xb07f, PDF_CMAP_RANGE, 10198 },
	{ 0xb080, 0xb080, PDF_CMAP_SINGLE, 1371 },
	{ 0xb081, 0xb083, PDF_CMAP_RANGE, 10200 },
	{ 0xb084, 0xb084, PDF_CMAP_SINGLE, 1372 },
	{ 0xb085, 0xb08b, PDF_CMAP_RANGE, 10203 },
	{ 0xb08c, 0xb08d, PDF_CMAP_RANGE, 1373 },
	{ 0xb08e, 0xb091, PDF_CMAP_TABLE, 2615 },
	{ 0xb092, 0xb097, PDF_CMAP_RANGE, 10212 },
	{ 0xb098, 0xb09a, PDF_CMAP_RANGE, 1377 },
	{ 0xb09b, 0xb09c, PDF_CMAP_TABLE, 2619 },
	{ 0xb09d, 0xb09e, PDF_CMAP_RANGE, 10219 },
	{ 0xb09f, 0xb0a2, PDF_CMAP_RANGE, 1381 },
	{ 0xb0a3, 0xb0a7, PDF_CMAP_RANGE, 10221 },
	{ 0xb0a8, 0xb0a9, PDF_CMAP_RANGE, 1385 },
	{ 0xb0aa, 0xb0aa, PDF_CMAP_SINGLE, 10226 },
	{ 0xb0ab, 0xb0af, PDF_CMAP_RANGE, 1387 },
	{ 0xb0b0, 0xb0b2, PDF_CMAP_TABLE, 2621 },
	{ 0xb0b3, 0xb0b5, PDF_CMAP_RANGE, 1393 },
	{ 0xb0b6, 0xb0b7, PDF_CMAP_RANGE, 10229 },
	{ 0xb0b8, 0xb0b8, PDF_CMAP_SINGLE, 1396 },
	{ 0xb0b9, 0xb0bb, PDF_CMAP_RANGE, 10231 },
	{ 0xb0bc, 0xb0bc, PDF_CMAP_SINGLE, 1397 },
	{ 0xb0bd, 0xb0c3, PDF_CMAP_RANGE, 10234 },
	{ 0xb0c4, 0xb0c5, PDF_CMAP_RANGE, 1398 },
	{ 0xb0c6, 0xb0c6, PDF_CMAP_SINGLE, 10241 },
	{ 0xb0c7, 0xb0c9, PDF_CMAP_RANGE, 1400 },
	{ 0xb0ca, 0xb0cf, PDF_CMAP_RANGE, 10242 },
	{ 0xb0d0, 0xb0d1, PDF_CMAP_RANGE, 1403 },
	{ 0xb0d2, 0xb0d3, PDF_CMAP_RANGE, 10248 },
	{ 0xb0d4, 0xb0d4, PDF_CMAP_SINGLE, 1405 },
	{ 0xb0d5, 0xb0d7, PDF_CMAP_RANGE, 10250 },
	{ 0xb0d8, 0xb0d8, PDF_CMAP_SINGLE, 1406 },
	{ 0xb0d9, 0xb0df, PDF_CMAP_RANGE, 10253 },
	{ 0xb0e0, 0xb0e0, PDF_CMAP_SINGLE, 1407 },
	{ 0xb0e1, 0xb0e4, PDF_CMAP_RANGE, 10260 },
	{ 0xb0e5, 0xb0e5, PDF_CMAP_SINGLE, 1408 },
	{ 0xb0e6, 0xb107, PDF_CMAP_RANGE, 10264 },
	{ 0xb108, 0xb109, PDF_CMAP_RANGE, 1409 },
	{ 0xb10a, 0xb10a, PDF_CMAP_SINGLE, 10298 },
	{ 0xb10b, 0xb10c, PDF_CMAP_RANGE, 1411 },
	{ 0xb10d, 0xb10f, PDF_CMAP_RANGE, 10299 },
	{ 0xb110, 0xb111, PDF_CMAP_TABLE, 2624 },
	{ 0xb112, 0xb113, PDF_CMAP_RANGE, 1414 },
	{ 0xb114, 0xb117, PDF_CMAP_RANGE, 10303 },
	{ 0xb118, 0xb119, PDF_CMAP_RANGE, 1416 },
	{ 0xb11a, 0xb11a, PDF_CMAP_SINGLE, 10307 },
	{ 0xb11b, 0xb11d, PDF_CMAP_RANGE, 1418 },
	{ 0xb11e, 0xb122, PDF_CMAP_RANGE, 10308 },
	{ 0xb123, 0xb125, PDF_CMAP_RANGE, 1421 },
	{ 0xb126, 0xb127, PDF_CMAP_RANGE, 10313 },
	{ 0xb128, 0xb128, PDF_CMAP_SINGLE, 1424 },
	{ 0xb129, 0xb12b, PDF_CMAP_RANGE, 10315 },
	{ 0xb12c, 0xb12c, PDF_CMAP_SINGLE, 1425 },
	{ 0xb12d, 0xb133, PDF_CMAP_RANGE, 10318 },
	{ 0xb134, 0xb135, PDF_CMAP_RANGE, 1426 },
	{ 0xb136, 0xb136, PDF_CMAP_SINGLE, 10325 },
	{ 0xb137, 0xb139, PDF_CMAP_RANGE, 1428 },
	{ 0xb13a, 0xb13f, PDF_CMAP_RANGE, 10326 },
	{ 0xb140, 0xb141, PDF_CMAP_RANGE, 1431 },
	{ 0xb142, 0xb143, PDF_CMAP_RANGE, 10332 },
	{ 0xb144, 0xb144, PDF_CMAP_SINGLE, 1433 },
	{ 0xb145, 0xb147, PDF_CMAP_RANGE, 10334 },
	{ 0xb148, 0xb148, PDF_CMAP_SINGLE, 1434 },
	{ 0xb149, 0xb14f, PDF_CMAP_RANGE, 10337 },
	{ 0xb150, 0xb151, PDF_CMAP_RANGE, 1435 },
	{ 0xb152, 0xb153, PDF_CMAP_RANGE, 10344 },
	{ 0xb154, 0xb155, PDF_CMAP_RANGE, 1437 },
	{ 0xb156, 0xb157, PDF_CMAP_RANGE, 10346 },
	{ 0xb158, 0xb158, PDF_CMAP_SINGLE, 1439 },
	{ 0xb159, 0xb15b, PDF_CMAP_RANGE, 10348 },
	{ 0xb15c, 0xb15c, PDF_CMAP_SINGLE, 1440 },
	{ 0xb15d, 0xb15f, PDF_CMAP_RANGE, 10351 },
	{ 0xb160, 0xb160, PDF_CMAP_SINGLE, 1441 },
	{ 0xb161, 0xb177, PDF_CMAP_RANGE, 10354 },
	{ 0xb178, 0xb179, PDF_CMAP_RANGE, 1442 },
	{ 0xb17a, 0xb17b, PDF_CMAP_RANGE, 10377 },
	{ 0xb17c, 0xb17c, PDF_CMAP_SINGLE, 1444 },
	{ 0xb17d, 0xb17f, PDF_CMAP_RANGE, 10379 },
	{ 0xb180, 0xb182, PDF_CMAP_TABLE, 2626 },
	{ 0xb183, 0xb187, PDF_CMAP_RANGE, 10383 },
	{ 0xb188, 0xb189, PDF_CMAP_RANGE, 1447 },
	{ 0xb18a, 0xb18d, PDF_CMAP_TABLE, 2629 },
	{ 0xb18e, 0xb191, PDF_CMAP_RANGE, 10390 },
	{ 0xb192, 0xb194, PDF_CMAP_RANGE, 1451 },
	{ 0xb195, 0xb197, PDF_CMAP_RANGE, 10394 },
	{ 0xb198, 0xb198, PDF_CMAP_SINGLE, 1454 },
	{ 0xb199, 0xb19b, PDF_CMAP_RANGE, 10397 },
	{ 0xb19c, 0xb19c, PDF_CMAP_SINGLE, 1455 },
	{ 0xb19d, 0xb1a7, PDF_CMAP_RANGE, 10400 },
	{ 0xb1a8, 0xb1a8, PDF_CMAP_SINGLE, 1456 },
	{ 0xb1a9, 0xb1cb, PDF_CMAP_RANGE, 10411 },
	{ 0xb1cc, 0xb1cc, PDF_CMAP_SINGLE, 1457 },
	{ 0xb1cd, 0xb1cf, PDF_CMAP_RANGE, 10446 },
	{ 0xb1d0, 0xb1d0, PDF_CMAP_SINGLE, 1458 },
	{ 0xb1d1, 0xb1d3, PDF_CMAP_RANGE, 10449 },
	{ 0xb1d4, 0xb1d4, PDF_CMAP_SINGLE, 1459 },
	{ 0xb1d5, 0xb1db, PDF_CMAP_RANGE, 10452 },
	{ 0xb1dc, 0xb1dd, PDF_CMAP_RANGE, 1460 },
	{ 0xb1de, 0xb1df, PDF_CMAP_TABLE, 2633 },
	{ 0xb1e0, 0xb1e7, PDF_CMAP_RANGE, 10460 },
	{ 0xb1e8, 0xb1e9, PDF_CMAP_RANGE, 1463 },
	{ 0xb1ea, 0xb1eb, PDF_CMAP_RANGE, 10468 },
	{ 0xb1ec, 0xb1ec, PDF_CMAP_SINGLE, 1465 },
	{ 0xb1ed, 0xb1ef, PDF_CMAP_RANGE, 10470 },
	{ 0xb1f0, 0xb1f0, PDF_CMAP_SINGLE, 1466 },
	{ 0xb1f1, 0xb1f8, PDF_CMAP_RANGE, 10473 },
	{ 0xb1f9, 0xb1fd, PDF_CMAP_TABLE, 2635 },
	{ 0xb1fe, 0xb203, PDF_CMAP_RANGE, 10483 },
	{ 0xb204, 0xb205, PDF_CMAP_RANGE, 1470 },
	{ 0xb206, 0xb207, PDF_CMAP_RANGE, 10489 },
	{ 0xb208, 0xb208, PDF_CMAP_SINGLE, 1472 },
	{ 0xb209, 0xb20a, PDF_CMAP_RANGE, 10491 },
	{ 0xb20b, 0xb20c, PDF_CMAP_RANGE, 1473 },
	{ 0xb20d, 0xb213, PDF_CMAP_RANGE, 10493 },
	{ 0xb214, 0xb215, PDF_CMAP_RANGE, 1475 },
	{ 0xb216, 0xb219, PDF_CMAP_TABLE, 2640 },
	{ 0xb21a, 0xb21f, PDF_CMAP_RANGE, 10502 },
	{ 0xb220, 0xb220, PDF_CMAP_SINGLE, 1479 },
	{ 0xb221, 0xb233, PDF_CMAP_RANGE, 10508 },
	{ 0xb234, 0xb234, PDF_CMAP_SINGLE, 1480 },
	{ 0xb235, 0xb23b, PDF_CMAP_RANGE, 10527 },
	{ 0xb23c, 0xb23c, PDF_CMAP_SINGLE, 1481 },
	{ 0xb23d, 0xb257, PDF_CMAP_RANGE, 10534 },
	{ 0xb258, 0xb258, PDF_CMAP_SINGLE, 1482 },
	{ 0xb259, 0xb25b, PDF_CMAP_RANGE, 10561 },
	{ 0xb25c, 0xb25c, PDF_CMAP_SINGLE, 1483 },
	{ 0xb25d, 0xb25f, PDF_CMAP_RANGE, 10564 },
	{ 0xb260, 0xb260, PDF_CMAP_SINGLE, 1484 },
	{ 0xb261, 0xb267, PDF_CMAP_RANGE, 10567 },
	{ 0xb268, 0xb269, PDF_CMAP_RANGE, 1485 },
	{ 0xb26a, 0xb273, PDF_CMAP_RANGE, 10574 },
	{ 0xb274, 0xb275, PDF_CMAP_RANGE, 1487 },
	{ 0xb276, 0xb27b, PDF_CMAP_RANGE, 10584 },
	{ 0xb27c, 0xb27c, PDF_CMAP_SINGLE, 1489 },
	{ 0xb27d, 0xb283, PDF_CMAP_RANGE, 10590 },
	{ 0xb284, 0xb285, PDF_CMAP_RANGE, 1490 },
	{ 0xb286, 0xb288, PDF_CMAP_RANGE, 10597 },
	{ 0xb289, 0xb289, PDF_CMAP_SINGLE, 1492 },
	{ 0xb28a, 0xb28f, PDF_CMAP_RANGE, 10600 },
	{ 0xb290, 0xb291, PDF_CMAP_RANGE, 1493 },
	{ 0xb292, 0xb293, PDF_CMAP_RANGE, 10606 },
	{ 0xb294, 0xb294, PDF_CMAP_SINGLE, 1495 },
	{ 0xb295, 0xb297, PDF_CMAP_RANGE, 10608 },
	{ 0xb298, 0xb29a, PDF_CMAP_RANGE, 1496 },
	{ 0xb29b, 0xb29f, PDF_CMAP_RANGE, 10611 },
	{ 0xb2a0, 0xb2a1, PDF_CMAP_RANGE, 1499 },
	{ 0xb2a2, 0xb2a4, PDF_CMAP_TABLE, 2644 },
	{ 0xb2a5, 0xb2a6, PDF_CMAP_RANGE, 1502 },
	{ 0xb2a7, 0xb2a9, PDF_CMAP_RANGE, 10618 },
	{ 0xb2aa, 0xb2ac, PDF_CMAP_TABLE, 2647 },
	{ 0xb2ad, 0xb2af, PDF_CMAP_RANGE, 10622 },
	{ 0xb2b0, 0xb2b0, PDF_CMAP_SINGLE, 1506 },
	{ 0xb2b1, 0xb2b3, PDF_CMAP_RANGE, 10625 },
	{ 0xb2b4, 0xb2b4, PDF_CMAP_SINGLE, 1507 },
	{ 0xb2b5, 0xb2c7, PDF_CMAP_RANGE, 10628 },
	{ 0xb2c8, 0xb2c9, PDF_CMAP_RANGE, 1508 },
	{ 0xb2ca, 0xb2cb, PDF_CMAP_RANGE, 10647 },
	{ 0xb2cc, 0xb2cc, PDF_CMAP_SINGLE, 1510 },
	{ 0xb2cd, 0xb2cf, PDF_CMAP_RANGE, 10649 },
	{ 0xb2d0, 0xb2d2, PDF_CMAP_TABLE, 2650 },
	{ 0xb2d3, 0xb2d7, PDF_CMAP_RANGE, 10653 },
	{ 0xb2d8, 0xb2d9, PDF_CMAP_RANGE, 1513 },
	{ 0xb2da, 0xb2dd, PDF_CMAP_TABLE, 2653 },
	{ 0xb2de, 0xb2e1, PDF_CMAP_RANGE, 10660 },
	{ 0xb2e2, 0xb2e3, PDF_CMAP_TABLE, 2657 },
	{ 0xb2e4, 0xb2e6, PDF_CMAP_RANGE, 1518 },
	{ 0xb2e7, 0xb2e8, PDF_CMAP_TABLE, 2659 },
	{ 0xb2e9, 0xb2ea, PDF_CMAP_RANGE, 10666 },
	{ 0xb2eb, 0xb2ef, PDF_CMAP_RANGE, 1522 },
	{ 0xb2f0, 0xb2f2, PDF_CMAP_RANGE, 10668 },
	{ 0xb2f3, 0xb2f5, PDF_CMAP_RANGE, 1527 },
	{ 0xb2f6, 0xb2f6, PDF_CMAP_SINGLE, 10671 },
	{ 0xb2f7, 0xb2fb, PDF_CMAP_RANGE, 1530 },
	{ 0xb2fc, 0xb2fe, PDF_CMAP_RANGE, 10672 },
	{ 0xb2ff, 0xb301, PDF_CMAP_RANGE, 1535 },
	{ 0xb302, 0xb303, PDF_CMAP_RANGE, 10675 },
	{ 0xb304, 0xb304, PDF_CMAP_SINGLE, 1538 },
	{ 0xb305, 0xb307, PDF_CMAP_RANGE, 10677 },
	{ 0xb308, 0xb308, PDF_CMAP_SINGLE, 1539 },
	{ 0xb309, 0xb30f, PDF_CMAP_RANGE, 10680 },
	{ 0xb310, 0xb311, PDF_CMAP_RANGE, 1540 },
	{ 0xb312, 0xb312, PDF_CMAP_SINGLE, 10687 },
	{ 0xb313, 0xb315, PDF_CMAP_RANGE, 1542 },
	{ 0xb316, 0xb31b, PDF_CMAP_RANGE, 10688 },
	{ 0xb31c, 0xb31c, PDF_CMAP_SINGLE, 1545 },
	{ 0xb31d, 0xb353, PDF_CMAP_RANGE, 10694 },
	{ 0xb354, 0xb356, PDF_CMAP_RANGE, 1546 },
	{ 0xb357, 0xb358, PDF_CMAP_TABLE, 2661 },
	{ 0xb359, 0xb35a, PDF_CMAP_RANGE, 10750 },
	{ 0xb35b, 0xb35c, PDF_CMAP_RANGE, 1550 },
	{ 0xb35d, 0xb35d, PDF_CMAP_SINGLE, 10752 },
	{ 0xb35e, 0xb35f, PDF_CMAP_RANGE, 1552 },
	{ 0xb360, 0xb363, PDF_CMAP_RANGE, 10753 },
	{ 0xb364, 0xb365, PDF_CMAP_RANGE, 1554 },
	{ 0xb366, 0xb36b, PDF_CMAP_TABLE, 2663 },
	{ 0xb36c, 0xb36d, PDF_CMAP_RANGE, 10760 },
	{ 0xb36e, 0xb36f, PDF_CMAP_TABLE, 2669 },
	{ 0xb370, 0xb371, PDF_CMAP_RANGE, 1560 },
	{ 0xb372, 0xb373, PDF_CMAP_RANGE, 10763 },
	{ 0xb374, 0xb374, PDF_CMAP_SINGLE, 1562 },
	{ 0xb375, 0xb377, PDF_CMAP_RANGE, 10765 },
	{ 0xb378, 0xb378, PDF_CMAP_SINGLE, 1563 },
	{ 0xb379, 0xb37f, PDF_CMAP_RANGE, 10768 },
	{ 0xb380, 0xb381, PDF_CMAP_RANGE, 1564 },
	{ 0xb382, 0xb382, PDF_CMAP_SINGLE, 10775 },
	{ 0xb383, 0xb385, PDF_CMAP_RANGE, 1566 },
	{ 0xb386, 0xb38b, PDF_CMAP_RANGE, 10776 },
	{ 0xb38c, 0xb38c, PDF_CMAP_SINGLE, 1569 },
	{ 0xb38d, 0xb38f, PDF_CMAP_RANGE, 10782 },
	{ 0xb390, 0xb390, PDF_CMAP_SINGLE, 1570 },
	{ 0xb391, 0xb393, PDF_CMAP_RANGE, 10785 },
	{ 0xb394, 0xb394, PDF_CMAP_SINGLE, 1571 },
	{ 0xb395, 0xb39f, PDF_CMAP_RANGE, 10788 },
	{ 0xb3a0, 0xb3a1, PDF_CMAP_RANGE, 1572 },
	{ 0xb3a2, 0xb3a7, PDF_CMAP_RANGE, 10799 },
	{ 0xb3a8, 0xb3a8, PDF_CMAP_SINGLE, 1574 },
	{ 0xb3a9, 0xb3ab, PDF_CMAP_RANGE, 10805 },
	{ 0xb3ac, 0xb3ac, PDF_CMAP_SINGLE, 1575 },
	{ 0xb3ad, 0xb3c3, PDF_CMAP_RANGE, 10808 },
	{ 0xb3c4, 0xb3c5, PDF_CMAP_RANGE, 1576 },
	{ 0xb3c6, 0xb3c7, PDF_CMAP_RANGE, 10831 },
	{ 0xb3c8, 0xb3c8, PDF_CMAP_SINGLE, 1578 },
	{ 0xb3c9, 0xb3ca, PDF_CMAP_RANGE, 10833 },
	{ 0xb3cb, 0xb3cc, PDF_CMAP_RANGE, 1579 },
	{ 0xb3cd, 0xb3d0, PDF_CMAP_TABLE, 2671 },
	{ 0xb3d1, 0xb3d3, PDF_CMAP_RANGE, 10837 },
	{ 0xb3d4, 0xb3d5, PDF_CMAP_RANGE, 1583 },
	{ 0xb3d6, 0xb3dd, PDF_CMAP_TABLE, 2675 },
	{ 0xb3de, 0xb3df, PDF_CMAP_RANGE, 10844 },
	{ 0xb3e0, 0xb3e0, PDF_CMAP_SINGLE, 1589 },
	{ 0xb3e1, 0xb3e3, PDF_CMAP_RANGE, 10846 },
	{ 0xb3e4, 0xb3e4, PDF_CMAP_SINGLE, 1590 },
	{ 0xb3e5, 0xb3e7, PDF_CMAP_RANGE, 10849 },
	{ 0xb3e8, 0xb3e8, PDF_CMAP_SINGLE, 1591 },
	{ 0xb3e9, 0xb3fb, PDF_CMAP_RANGE, 10852 },
	{ 0xb3fc, 0xb3fc, PDF_CMAP_SINGLE, 1592 },
	{ 0xb3fd, 0xb40f, PDF_CMAP_RANGE, 10871 },
	{ 0xb410, 0xb410, PDF_CMAP_SINGLE, 1593 },
	{ 0xb411, 0xb417, PDF_CMAP_RANGE, 10890 },
	{ 0xb418, 0xb418, PDF_CMAP_SINGLE, 1594 },
	{ 0xb419, 0xb41b, PDF_CMAP_RANGE, 10897 },
	{ 0xb41c, 0xb41c, PDF_CMAP_SINGLE, 1595 },
	{ 0xb41d, 0xb41f, PDF_CMAP_RANGE, 10900 },
	{ 0xb420, 0xb420, PDF_CMAP_SINGLE, 1596 },
	{ 0xb421, 0xb427, PDF_CMAP_RANGE, 10903 },
	{ 0xb428, 0xb429, PDF_CMAP_RANGE, 1597 },
	{ 0xb42a, 0xb42b, PDF_CMAP_TABLE, 2683 },
	{ 0xb42c, 0xb433, PDF_CMAP_RANGE, 10911 },
	{ 0xb434, 0xb434, PDF_CMAP_SINGLE, 1600 },
	{ 0xb435, 0xb44f, PDF_CMAP_RANGE, 10919 },
	{ 0xb450, 0xb451, PDF_CMAP_RANGE, 1601 },
	{ 0xb452, 0xb453, PDF_CMAP_RANGE, 10946 },
	{ 0xb454, 0xb454, PDF_CMAP_SINGLE, 1603 },
	{ 0xb455, 0xb457, PDF_CMAP_RANGE, 10948 },
	{ 0xb458, 0xb458, PDF_CMAP_SINGLE, 1604 },
	{ 0xb459, 0xb45f, PDF_CMAP_RANGE, 10951 },
	{ 0xb460, 0xb461, PDF_CMAP_RANGE, 1605 },
	{ 0xb462, 0xb465, PDF_CMAP_TABLE, 2685 },
	{ 0xb466, 0xb46b, PDF_CMAP_RANGE, 10960 },
	{ 0xb46c, 0xb46c, PDF_CMAP_SINGLE, 1609 },
	{ 0xb46d, 0xb47f, PDF_CMAP_RANGE, 10966 },
	{ 0xb480, 0xb480, PDF_CMAP_SINGLE, 1610 },
	{ 0xb481, 0xb487, PDF_CMAP_RANGE, 10985 },
	{ 0xb488, 0xb488, PDF_CMAP_SINGLE, 1611 },
	{ 0xb489, 0xb49c, PDF_CMAP_RANGE, 10992 },
	{ 0xb49d, 0xb49d, PDF_CMAP_SINGLE, 1612 },
	{ 0xb49e, 0xb4a3, PDF_CMAP_RANGE, 11012 },
	{ 0xb4a4, 0xb4a4, PDF_CMAP_SINGLE, 1613 },
	{ 0xb4a5, 0xb4a7, PDF_CMAP_RANGE, 11018 },
	{ 0xb4a8, 0xb4a8, PDF_CMAP_SINGLE, 1614 },
	{ 0xb4a9, 0xb4ab, PDF_CMAP_RANGE, 11021 },
	{ 0xb4ac, 0xb4ac, PDF_CMAP_SINGLE, 1615 },
	{ 0xb4ad, 0xb4b4, PDF_CMAP_RANGE, 11024 },
	{ 0xb4b5, 0xb4b9, PDF_CMAP_TABLE, 2689 },
	{ 0xb4ba, 0xb4bf, PDF_CMAP_RANGE, 11034 },
	{ 0xb4c0, 0xb4c0, PDF_CMAP_SINGLE, 1619 },
	{ 0xb4c1, 0xb4c3, PDF_CMAP_RANGE, 11040 },
	{ 0xb4c4, 0xb4c4, PDF_CMAP_SINGLE, 1620 },
	{ 0xb4c5, 0xb4c7, PDF_CMAP_RANGE, 11043 },
	{ 0xb4c8, 0xb4c8, PDF_CMAP_SINGLE, 1621 },
	{ 0xb4c9, 0xb4cf, PDF_CMAP_RANGE, 11046 },
	{ 0xb4d0, 0xb4d0, PDF_CMAP_SINGLE, 1622 },
	{ 0xb4d1, 0xb4d4, PDF_CMAP_RANGE, 11053 },
	{ 0xb4d5, 0xb4d5, PDF_CMAP_SINGLE, 1623 },
	{ 0xb4d6, 0xb4db, PDF_CMAP_RANGE, 11057 },
	{ 0xb4dc, 0xb4dd, PDF_CMAP_RANGE, 1624 },
	{ 0xb4de, 0xb4df, PDF_CMAP_RANGE, 11063 },
	{ 0xb4e0, 0xb4e0, PDF_CMAP_SINGLE, 1626 },
	{ 0xb4e1, 0xb4e2, PDF_CMAP_RANGE, 11065 },
	{ 0xb4e3, 0xb4e4, PDF_CMAP_RANGE, 1627 },
	{ 0xb4e5, 0xb4e6, PDF_CMAP_TABLE, 2694 },
	{ 0xb4e7, 0xb4eb, PDF_CMAP_RANGE, 11068 },
	{ 0xb4ec, 0xb4ed, PDF_CMAP_RANGE, 1630 },
	{ 0xb4ee, 0xb4f1, PDF_CMAP_TABLE, 2696 },
	{ 0xb4f2, 0xb4f7, PDF_CMAP_RANGE, 11075 },
	{ 0xb4f8, 0xb4f8, PDF_CMAP_SINGLE, 1634 },
	{ 0xb4f9, 0xb513, PDF_CMAP_RANGE, 11081 },
	{ 0xb514, 0xb515, PDF_CMAP_RANGE, 1635 },
	{ 0xb516, 0xb517, PDF_CMAP_RANGE, 11108 },
	{ 0xb518, 0xb518, PDF_CMAP_SINGLE, 1637 },
	{ 0xb519, 0xb51a, PDF_CMAP_RANGE, 11110 },
	{ 0xb51b, 0xb51c, PDF_CMAP_RANGE, 1638 },
	{ 0xb51d, 0xb523, PDF_CMAP_RANGE, 11112 },
	{ 0xb524, 0xb525, PDF_CMAP_RANGE, 1640 },
	{ 0xb526, 0xb526, PDF_CMAP_SINGLE, 11119 },
	{ 0xb527, 0xb52a, PDF_CMAP_RANGE, 1642 },
	{ 0xb52b, 0xb52f, PDF_CMAP_RANGE, 11120 },
	{ 0xb530, 0xb531, PDF_CMAP_RANGE, 1646 },
	{ 0xb532, 0xb533, PDF_CMAP_RANGE, 11125 },
	{ 0xb534, 0xb534, PDF_CMAP_SINGLE, 1648 },
	{ 0xb535, 0xb537, PDF_CMAP_RANGE, 11127 },
	{ 0xb538, 0xb538, PDF_CMAP_SINGLE, 1649 },
	{ 0xb539, 0xb53f, PDF_CMAP_RANGE, 11130 },
	{ 0xb540, 0xb541, PDF_CMAP_RANGE, 1650 },
	{ 0xb542, 0xb542, PDF_CMAP_SINGLE, 11137 },
	{ 0xb543, 0xb545, PDF_CMAP_RANGE, 1652 },
	{ 0xb546, 0xb54a, PDF_CMAP_RANGE, 11138 },
	{ 0xb54b, 0xb54d, PDF_CMAP_RANGE, 1655 },
	{ 0xb54e, 0xb54f, PDF_CMAP_RANGE, 11143 },
	{ 0xb550, 0xb550, PDF_CMAP_SINGLE, 1658 },
	{ 0xb551, 0xb553, PDF_CMAP_RANGE, 11145 },
	{ 0xb554, 0xb554, PDF_CMAP_SINGLE, 1659 },
	{ 0xb555, 0xb55b, PDF_CMAP_RANGE, 11148 },
	{ 0xb55c, 0xb55d, PDF_CMAP_RANGE, 1660 },
	{ 0xb55e, 0xb55e, PDF_CMAP_SINGLE, 11155 },
	{ 0xb55f, 0xb561, PDF_CMAP_RANGE, 1662 },
	{ 0xb562, 0xb59f, PDF_CMAP_RANGE, 11156 },
	{ 0xb5a0, 0xb5a1, PDF_CMAP_RANGE, 1665 },
	{ 0xb5a2, 0xb5a3, PDF_CMAP_RANGE, 11218 },
	{ 0xb5a4, 0xb5a4, PDF_CMAP_SINGLE, 1667 },
	{ 0xb5a5, 0xb5a7, PDF_CMAP_RANGE, 11220 },
	{ 0xb5a8, 0xb5a9, PDF_CMAP_TABLE, 2700 },
	{ 0xb5aa, 0xb5ab, PDF_CMAP_RANGE, 1669 },
	{ 0xb5ac, 0xb5af, PDF_CMAP_RANGE, 11224 },
	{ 0xb5b0, 0xb5b1, PDF_CMAP_RANGE, 1671 },
	{ 0xb5b2, 0xb5b2, PDF_CMAP_SINGLE, 11228 },
	{ 0xb5b3, 0xb5b5, PDF_CMAP_RANGE, 1673 },
	{ 0xb5b6, 0xb5ba, PDF_CMAP_RANGE, 11229 },
	{ 0xb5bb, 0xb5bd, PDF_CMAP_RANGE, 1676 },
	{ 0xb5be, 0xb5bf, PDF_CMAP_RANGE, 11234 },
	{ 0xb5c0, 0xb5c0, PDF_CMAP_SINGLE, 1679 },
	{ 0xb5c1, 0xb5c3, PDF_CMAP_RANGE, 11236 },
	{ 0xb5c4, 0xb5c4, PDF_CMAP_SINGLE, 1680 },
	{ 0xb5c5, 0xb5cb, PDF_CMAP_RANGE, 11239 },
	{ 0xb5cc, 0xb5cd, PDF_CMAP_RANGE, 1681 },
	{ 0xb5ce, 0xb5ce, PDF_CMAP_SINGLE, 11246 },
	{ 0xb5cf, 0xb5d1, PDF_CMAP_RANGE, 1683 },
	{ 0xb5d2, 0xb5d7, PDF_CMAP_RANGE, 11247 },
	{ 0xb5d8, 0xb5d8, PDF_CMAP_SINGLE, 1686 },
	{ 0xb5d9, 0xb5eb, PDF_CMAP_RANGE, 11253 },
	{ 0xb5ec, 0xb5ec, PDF_CMAP_SINGLE, 1687 },
	{ 0xb5ed, 0xb60f, PDF_CMAP_RANGE, 11272 },
	{ 0xb610, 0xb611, PDF_CMAP_RANGE, 1688 },
	{ 0xb612, 0xb613, PDF_CMAP_RANGE, 11307 },
	{ 0xb614, 0xb614, PDF_CMAP_SINGLE, 1690 },
	{ 0xb615, 0xb617, PDF_CMAP_RANGE, 11309 },
	{ 0xb618, 0xb618, PDF_CMAP_SINGLE, 1691 },
	{ 0xb619, 0xb624, PDF_CMAP_RANGE, 11312 },
	{ 0xb625, 0xb625, PDF_CMAP_SINGLE, 1692 },
	{ 0xb626, 0xb62b, PDF_CMAP_RANGE, 11324 },
	{ 0xb62c, 0xb62c, PDF_CMAP_SINGLE, 1693 },
	{ 0xb62d, 0xb633, PDF_CMAP_RANGE, 11330 },
	{ 0xb634, 0xb634, PDF_CMAP_SINGLE, 1694 },
	{ 0xb635, 0xb647, PDF_CMAP_RANGE, 11337 },
	{ 0xb648, 0xb648, PDF_CMAP_SINGLE, 1695 },
	{ 0xb649, 0xb663, PDF_CMAP_RANGE, 11356 },
	{ 0xb664, 0xb664, PDF_CMAP_SINGLE, 1696 },
	{ 0xb665, 0xb667, PDF_CMAP_RANGE, 11383 },
	{ 0xb668, 0xb668, PDF_CMAP_SINGLE, 1697 },
	{ 0xb669, 0xb69b, PDF_CMAP_RANGE, 11386 },
	{ 0xb69c, 0xb69d, PDF_CMAP_RANGE, 1698 },
	{ 0xb69e, 0xb69f, PDF_CMAP_RANGE, 11437 },
	{ 0xb6a0, 0xb6a0, PDF_CMAP_SINGLE, 1700 },
	{ 0xb6a1, 0xb6a3, PDF_CMAP_RANGE, 11439 },
	{ 0xb6a4, 0xb6a4, PDF_CMAP_SINGLE, 1701 },
	{ 0xb6a5, 0xb6aa, PDF_CMAP_RANGE, 11442 },
	{ 0xb6ab, 0xb6ac, PDF_CMAP_RANGE, 1702 },
	{ 0xb6ad, 0xb6b0, PDF_CMAP_RANGE, 11448 },
	{ 0xb6b1, 0xb6b1, PDF_CMAP_SINGLE, 1704 },
	{ 0xb6b2, 0xb6d3, PDF_CMAP_RANGE, 11452 },
	{ 0xb6d4, 0xb6d4, PDF_CMAP_SINGLE, 1705 },
	{ 0xb6d5, 0xb6ef, PDF_CMAP_RANGE, 11486 },
	{ 0xb6f0, 0xb6f0, PDF_CMAP_SINGLE, 1706 },
	{ 0xb6f1, 0xb6f3, PDF_CMAP_RANGE, 11513 },
	{ 0xb6f4, 0xb6f4, PDF_CMAP_SINGLE, 1707 },
	{ 0xb6f5, 0xb6f7, PDF_CMAP_RANGE, 11516 },
	{ 0xb6f8, 0xb6f8, PDF_CMAP_SINGLE, 1708 },
	{ 0xb6f9, 0xb6ff, PDF_CMAP_RANGE, 11519 },
	{ 0xb700, 0xb701, PDF_CMAP_RANGE, 1709 },
	{ 0xb702, 0xb704, PDF_CMAP_RANGE, 11526 },
	{ 0xb705, 0xb705, PDF_CMAP_SINGLE, 1711 },
	{ 0xb706, 0xb727, PDF_CMAP_RANGE, 11529 },
	{ 0xb728, 0xb729, PDF_CMAP_RANGE, 1712 },
	{ 0xb72a, 0xb72b, PDF_CMAP_RANGE, 11563 },
	{ 0xb72c, 0xb72c, PDF_CMAP_SINGLE, 1714 },
	{ 0xb72d, 0xb72e, PDF_CMAP_RANGE, 11565 },
	{ 0xb72f, 0xb730, PDF_CMAP_RANGE, 1715 },
	{ 0xb731, 0xb737, PDF_CMAP_RANGE, 11567 },
	{ 0xb738, 0xb739, PDF_CMAP_RANGE, 1717 },
	{ 0xb73a, 0xb73b, PDF_CMAP_TABLE, 2702 },
	{ 0xb73c, 0xb743, PDF_CMAP_RANGE, 11575 },
	{ 0xb744, 0xb744, PDF_CMAP_SINGLE, 1720 },
	{ 0xb745, 0xb747, PDF_CMAP_RANGE, 11583 },
	{ 0xb748, 0xb748, PDF_CMAP_SINGLE, 1721 },
	{ 0xb749, 0xb74b, PDF_CMAP_RANGE, 11586 },
	{ 0xb74c, 0xb74c, PDF_CMAP_SINGLE, 1722 },
	{ 0xb74d, 0xb753, PDF_CMAP_RANGE, 11589 },
	{ 0xb754, 0xb755, PDF_CMAP_RANGE, 1723 },
	{ 0xb756, 0xb75f, PDF_CMAP_RANGE, 11596 },
	{ 0xb760, 0xb760, PDF_CMAP_SINGLE, 1725 },
	{ 0xb761, 0xb763, PDF_CMAP_RANGE, 11606 },
	{ 0xb764, 0xb764, PDF_CMAP_SINGLE, 1726 },
	{ 0xb765, 0xb767, PDF_CMAP_RANGE, 11609 },
	{ 0xb768, 0xb768, PDF_CMAP_SINGLE, 1727 },
	{ 0xb769, 0xb76f, PDF_CMAP_RANGE, 11612 },
	{ 0xb770, 0xb771, PDF_CMAP_RANGE, 1728 },
	{ 0xb772, 0xb775, PDF_CMAP_TABLE, 2704 },
	{ 0xb776, 0xb77b, PDF_CMAP_RANGE, 11621 },
	{ 0xb77c, 0xb77d, PDF_CMAP_RANGE, 1732 },
	{ 0xb77e, 0xb77f, PDF_CMAP_RANGE, 11627 },
	{ 0xb780, 0xb780, PDF_CMAP_SINGLE, 1734 },
	{ 0xb781, 0xb783, PDF_CMAP_RANGE, 11629 },
	{ 0xb784, 0xb784, PDF_CMAP_SINGLE, 1735 },
	{ 0xb785, 0xb78b, PDF_CMAP_RANGE, 11632 },
	{ 0xb78c, 0xb78d, PDF_CMAP_RANGE, 1736 },
	{ 0xb78e, 0xb78e, PDF_CMAP_SINGLE, 11639 },
	{ 0xb78f, 0xb792, PDF_CMAP_RANGE, 1738 },
	{ 0xb793, 0xb795, PDF_CMAP_RANGE, 11640 },
	{ 0xb796, 0xb799, PDF_CMAP_RANGE, 1742 },
	{ 0xb79a, 0xb79b, PDF_CMAP_RANGE, 11643 },
	{ 0xb79c, 0xb79c, PDF_CMAP_SINGLE, 1746 },
	{ 0xb79d, 0xb79f, PDF_CMAP_RANGE, 11645 },
	{ 0xb7a0, 0xb7a0, PDF_CMAP_SINGLE, 1747 },
	{ 0xb7a1, 0xb7a7, PDF_CMAP_RANGE, 11648 },
	{ 0xb7a8, 0xb7a9, PDF_CMAP_RANGE, 1748 },
	{ 0xb7aa, 0xb7aa, PDF_CMAP_SINGLE, 11655 },
	{ 0xb7ab, 0xb7ad, PDF_CMAP_RANGE, 1750 },
	{ 0xb7ae, 0xb7b3, PDF_CMAP_RANGE, 11656 },
	{ 0xb7b4, 0xb7b5, PDF_CMAP_RANGE, 1753 },
	{ 0xb7b6, 0xb7b7, PDF_CMAP_RANGE, 11662 },
	{ 0xb7b8, 0xb7b8, PDF_CMAP_SINGLE, 1755 },
	{ 0xb7b9, 0xb7c6, PDF_CMAP_RANGE, 11664 },
	{ 0xb7c7, 0xb7c9, PDF_CMAP_TABLE, 2708 },
	{ 0xb7ca, 0xb7eb, PDF_CMAP_RANGE, 11679 },
	{ 0xb7ec, 0xb7ed, PDF_CMAP_RANGE, 1758 },
	{ 0xb7ee, 0xb7ef, PDF_CMAP_RANGE, 11713 },
	{ 0xb7f0, 0xb7f0, PDF_CMAP_SINGLE, 1760 },
	{ 0xb7f1, 0xb7f3, PDF_CMAP_RANGE, 11715 },
	{ 0xb7f4, 0xb7f4, PDF_CMAP_SINGLE, 1761 },
	{ 0xb7f5, 0xb7fb, PDF_CMAP_RANGE, 11718 },
	{ 0xb7fc, 0xb7fd, PDF_CMAP_RANGE, 1762 },
	{ 0xb7fe, 0xb7ff, PDF_CMAP_TABLE, 2711 },
	{ 0xb800, 0xb801, PDF_CMAP_RANGE, 1765 },
	{ 0xb802, 0xb806, PDF_CMAP_RANGE, 11726 },
	{ 0xb807, 0xb809, PDF_CMAP_RANGE, 1767 },
	{ 0xb80a, 0xb80b, PDF_CMAP_RANGE, 11731 },
	{ 0xb80c, 0xb80c, PDF_CMAP_SINGLE, 1770 },
	{ 0xb80d, 0xb80f, PDF_CMAP_RANGE, 11733 },
	{ 0xb810, 0xb810, PDF_CMAP_SINGLE, 1771 },
	{ 0xb811, 0xb817, PDF_CMAP_RANGE, 11736 },
	{ 0xb818, 0xb819, PDF_CMAP_RANGE, 1772 },
	{ 0xb81a, 0xb81d, PDF_CMAP_TABLE, 2713 },
	{ 0xb81e, 0xb823, PDF_CMAP_RANGE, 11745 },
	{ 0xb824, 0xb825, PDF_CMAP_RANGE, 1776 },
	{ 0xb826, 0xb827, PDF_CMAP_RANGE, 11751 },
	{ 0xb828, 0xb828, PDF_CMAP_SINGLE, 1778 },
	{ 0xb829, 0xb82b, PDF_CMAP_RANGE, 11753 },
	{ 0xb82c, 0xb82c, PDF_CMAP_SINGLE, 1779 },
	{ 0xb82d, 0xb833, PDF_CMAP_RANGE, 11756 },
	{ 0xb834, 0xb835, PDF_CMAP_RANGE, 1780 },
	{ 0xb836, 0xb836, PDF_CMAP_SINGLE, 11763 },
	{ 0xb837, 0xb839, PDF_CMAP_RANGE, 1782 },
	{ 0xb83a, 0xb83f, PDF_CMAP_RANGE, 11764 },
	{ 0xb840, 0xb840, PDF_CMAP_SINGLE, 1785 },
	{ 0xb841, 0xb843, PDF_CMAP_RANGE, 11770 },
	{ 0xb844, 0xb844, PDF_CMAP_SINGLE, 1786 },
	{ 0xb845, 0xb850, PDF_CMAP_RANGE, 11773 },
	{ 0xb851, 0xb853, PDF_CMAP_TABLE, 2717 },
	{ 0xb854, 0xb85b, PDF_CMAP_RANGE, 11786 },
	{ 0xb85c, 0xb85d, PDF_CMAP_RANGE, 1789 },
	{ 0xb85e, 0xb85f, PDF_CMAP_RANGE, 11794 },
	{ 0xb860, 0xb860, PDF_CMAP_SINGLE, 1791 },
	{ 0xb861, 0xb863, PDF_CMAP_RANGE, 11796 },
	{ 0xb864, 0xb864, PDF_CMAP_SINGLE, 1792 },
	{ 0xb865, 0xb86b, PDF_CMAP_RANGE, 11799 },
	{ 0xb86c, 0xb86d, PDF_CMAP_RANGE, 1793 },
	{ 0xb86e, 0xb871, PDF_CMAP_TABLE, 2720 },
	{ 0xb872, 0xb877, PDF_CMAP_RANGE, 11808 },
	{ 0xb878, 0xb878, PDF_CMAP_SINGLE, 1797 },
	{ 0xb879, 0xb87b, PDF_CMAP_RANGE, 11814 },
	{ 0xb87c, 0xb87c, PDF_CMAP_SINGLE, 1798 },
	{ 0xb87d, 0xb88c, PDF_CMAP_RANGE, 11817 },
	{ 0xb88d, 0xb88d, PDF_CMAP_SINGLE, 1799 },
	{ 0xb88e, 0xb8a7, PDF_CMAP_RANGE, 11833 },
	{ 0xb8a8, 0xb8a8, PDF_CMAP_SINGLE, 1800 },
	{ 0xb8a9, 0xb8af, PDF_CMAP_RANGE, 11859 },
	{ 0xb8b0, 0xb8b0, PDF_CMAP_SINGLE, 1801 },
	{ 0xb8b1, 0xb8b3, PDF_CMAP_RANGE, 11866 },
	{ 0xb8b4, 0xb8b4, PDF_CMAP_SINGLE, 1802 },
	{ 0xb8b5, 0xb8b7, PDF_CMAP_RANGE, 11869 },
	{ 0xb8b8, 0xb8b8, PDF_CMAP_SINGLE, 1803 },
	{ 0xb8b9, 0xb8bf, PDF_CMAP_RANGE, 11872 },
	{ 0xb8c0, 0xb8c1, PDF_CMAP_RANGE, 1804 },
	{ 0xb8c2, 0xb8c5, PDF_CMAP_TABLE, 2724 },
	{ 0xb8c6, 0xb8cb, PDF_CMAP_RANGE, 11881 },
	{ 0xb8cc, 0xb8cc, PDF_CMAP_SINGLE, 1808 },
	{ 0xb8cd, 0xb8cf, PDF_CMAP_RANGE, 11887 },
	{ 0xb8d0, 0xb8d0, PDF_CMAP_SINGLE, 1809 },
	{ 0xb8d1, 0xb8d3, PDF_CMAP_RANGE, 11890 },
	{ 0xb8d4, 0xb8d4, PDF_CMAP_SINGLE, 1810 },
	{ 0xb8d5, 0xb8dc, PDF_CMAP_RANGE, 11893 },
	{ 0xb8dd, 0xb8e1, PDF_CMAP_TABLE, 2728 },
	{ 0xb8e2, 0xb8e7, PDF_CMAP_RANGE, 11903 },
	{ 0xb8e8, 0xb8e9, PDF_CMAP_RANGE, 1814 },
	{ 0xb8ea, 0xb8eb, PDF_CMAP_RANGE, 11909 },
	{ 0xb8ec, 0xb8ec, PDF_CMAP_SINGLE, 1816 },
	{ 0xb8ed, 0xb8ef, PDF_CMAP_RANGE, 11911 },
	{ 0xb8f0, 0xb8f0, PDF_CMAP_SINGLE, 1817 },
	{ 0xb8f1, 0xb8f7, PDF_CMAP_RANGE, 11914 },
	{ 0xb8f8, 0xb8f9, PDF_CMAP_RANGE, 1818 },
	{ 0xb8fa, 0xb8fd, PDF_CMAP_TABLE, 2733 },
	{ 0xb8fe, 0xb903, PDF_CMAP_RANGE, 11923 },
	{ 0xb904, 0xb904, PDF_CMAP_SINGLE, 1822 },
	{ 0xb905, 0xb917, PDF_CMAP_RANGE, 11929 },
	{ 0xb918, 0xb918, PDF_CMAP_SINGLE, 1823 },
	{ 0xb919, 0xb91f, PDF_CMAP_RANGE, 11948 },
	{ 0xb920, 0xb920, PDF_CMAP_SINGLE, 1824 },
	{ 0xb921, 0xb93b, PDF_CMAP_RANGE, 11955 },
	{ 0xb93c, 0xb93d, PDF_CMAP_RANGE, 1825 },
	{ 0xb93e, 0xb93f, PDF_CMAP_RANGE, 11982 },
	{ 0xb940, 0xb940, PDF_CMAP_SINGLE, 1827 },
	{ 0xb941, 0xb943, PDF_CMAP_RANGE, 11984 },
	{ 0xb944, 0xb944, PDF_CMAP_SINGLE, 1828 },
	{ 0xb945, 0xb94b, PDF_CMAP_RANGE, 11987 },
	{ 0xb94c, 0xb94c, PDF_CMAP_SINGLE, 1829 },
	{ 0xb94d, 0xb94e, PDF_CMAP_RANGE, 11994 },
	{ 0xb94f, 0xb951, PDF_CMAP_TABLE, 2737 },
	{ 0xb952, 0xb957, PDF_CMAP_RANGE, 11997 },
	{ 0xb958, 0xb959, PDF_CMAP_RANGE, 1832 },
	{ 0xb95a, 0xb95b, PDF_CMAP_RANGE, 12003 },
	{ 0xb95c, 0xb95c, PDF_CMAP_SINGLE, 1834 },
	{ 0xb95d, 0xb95f, PDF_CMAP_RANGE, 12005 },
	{ 0xb960, 0xb960, PDF_CMAP_SINGLE, 1835 },
	{ 0xb961, 0xb967, PDF_CMAP_RANGE, 12008 },
	{ 0xb968, 0xb969, PDF_CMAP_RANGE, 1836 },
	{ 0xb96a, 0xb96d, PDF_CMAP_TABLE, 2740 },
	{ 0xb96e, 0xb973, PDF_CMAP_RANGE, 12017 },
	{ 0xb974, 0xb975, PDF_CMAP_RANGE, 1840 },
	{ 0xb976, 0xb977, PDF_CMAP_RANGE, 12023 },
	{ 0xb978, 0xb978, PDF_CMAP_SINGLE, 1842 },
	{ 0xb979, 0xb97b, PDF_CMAP_RANGE, 12025 },
	{ 0xb97c, 0xb97c, PDF_CMAP_SINGLE, 1843 },
	{ 0xb97d, 0xb983, PDF_CMAP_RANGE, 12028 },
	{ 0xb984, 0xb985, PDF_CMAP_RANGE, 1844 },
	{ 0xb986, 0xb988, PDF_CMAP_TABLE, 2744 },
	{ 0xb989, 0xb98a, PDF_CMAP_RANGE, 1847 },
	{ 0xb98b, 0xb98c, PDF_CMAP_RANGE, 12037 },
	{ 0xb98d, 0xb98e, PDF_CMAP_RANGE, 1849 },
	{ 0xb98f, 0xb9ab, PDF_CMAP_RANGE, 12039 },
	{ 0xb9ac, 0xb9ad, PDF_CMAP_RANGE, 1851 },
	{ 0xb9ae, 0xb9af, PDF_CMAP_RANGE, 12068 },
	{ 0xb9b0, 0xb9b0, PDF_CMAP_SINGLE, 1853 },
	{ 0xb9b1, 0xb9b3, PDF_CMAP_RANGE, 12070 },
	{ 0xb9b4, 0xb9b4, PDF_CMAP_SINGLE, 1854 },
	{ 0xb9b5, 0xb9bb, PDF_CMAP_RANGE, 12073 },
	{ 0xb9bc, 0xb9bd, PDF_CMAP_RANGE, 1855 },
	{ 0xb9be, 0xb9c1, PDF_CMAP_TABLE, 2747 },
	{ 0xb9c2, 0xb9c7, PDF_CMAP_RANGE, 12082 },
	{ 0xb9c8, 0xb9c9, PDF_CMAP_RANGE, 1859 },
	{ 0xb9ca, 0xb9cb, PDF_CMAP_RANGE, 12088 },
	{ 0xb9cc, 0xb9cd, PDF_CMAP_TABLE, 2751 },
	{ 0xb9ce, 0xb9d2, PDF_CMAP_RANGE, 1862 },
	{ 0xb9d3, 0xb9d7, PDF_CMAP_RANGE, 12091 },
	{ 0xb9d8, 0xb9d9, PDF_CMAP_RANGE, 1867 },
	{ 0xb9da, 0xb9dc, PDF_CMAP_TABLE, 2753 },
	{ 0xb9dd, 0xb9de, PDF_CMAP_RANGE, 1870 },
	{ 0xb9df, 0xb9e0, PDF_CMAP_RANGE, 12098 },
	{ 0xb9e1, 0xb9e2, PDF_CMAP_TABLE, 2756 },
	{ 0xb9e3, 0xb9e5, PDF_CMAP_RANGE, 1873 },
	{ 0xb9e6, 0xb9e7, PDF_CMAP_RANGE, 12101 },
	{ 0xb9e8, 0xb9e8, PDF_CMAP_SINGLE, 1876 },
	{ 0xb9e9, 0xb9eb, PDF_CMAP_RANGE, 12103 },
	{ 0xb9ec, 0xb9ec, PDF_CMAP_SINGLE, 1877 },
	{ 0xb9ed, 0xb9f3, PDF_CMAP_RANGE, 12106 },
	{ 0xb9f4, 0xb9f5, PDF_CMAP_RANGE, 1878 },
	{ 0xb9f6, 0xb9f6, PDF_CMAP_SINGLE, 12113 },
	{ 0xb9f7, 0xb9fa, PDF_CMAP_RANGE, 1880 },
	{ 0xb9fb, 0xb9ff, PDF_CMAP_RANGE, 12114 },
	{ 0xba00, 0xba01, PDF_CMAP_RANGE, 1884 },
	{ 0xba02, 0xba07, PDF_CMAP_RANGE, 12119 },
	{ 0xba08, 0xba08, PDF_CMAP_SINGLE, 1886 },
	{ 0xba09, 0xba14, PDF_CMAP_RANGE, 12125 },
	{ 0xba15, 0xba15, PDF_CMAP_SINGLE, 1887 },
	{ 0xba16, 0xba37, PDF_CMAP_RANGE, 12137 },
	{ 0xba38, 0xba39, PDF_CMAP_RANGE, 1888 },
	{ 0xba3a, 0xba3b, PDF_CMAP_RANGE, 12171 },
	{ 0xba3c, 0xba3c, PDF_CMAP_SINGLE, 1890 },
	{ 0xba3d, 0xba3f, PDF_CMAP_RANGE, 12173 },
	{ 0xba40, 0xba42, PDF_CMAP_TABLE, 2758 },
	{ 0xba43, 0xba47, PDF_CMAP_RANGE, 12177 },
	{ 0xba48, 0xba49, PDF_CMAP_RANGE, 1893 },
	{ 0xba4a, 0xba4c, PDF_CMAP_TABLE, 2761 },
	{ 0xba4d, 0xba4e, PDF_CMAP_RANGE, 1896 },
	{ 0xba4f, 0xba52, PDF_CMAP_RANGE, 12184 },
	{ 0xba53, 0xba55, PDF_CMAP_RANGE, 1898 },
	{ 0xba56, 0xba57, PDF_CMAP_RANGE, 12188 },
	{ 0xba58, 0xba58, PDF_CMAP_SINGLE, 1901 },
	{ 0xba59, 0xba5b, PDF_CMAP_RANGE, 12190 },
	{ 0xba5c, 0xba5c, PDF_CMAP_SINGLE, 1902 },
	{ 0xba5d, 0xba63, PDF_CMAP_RANGE, 12193 },
	{ 0xba64, 0xba65, PDF_CMAP_RANGE, 1903 },
	{ 0xba66, 0xba66, PDF_CMAP_SINGLE, 12200 },
	{ 0xba67, 0xba69, PDF_CMAP_RANGE, 1905 },
	{ 0xba6a, 0xba6f, PDF_CMAP_RANGE, 12201 },
	{ 0xba70, 0xba71, PDF_CMAP_RANGE, 1908 },
	{ 0xba72, 0xba73, PDF_CMAP_RANGE, 12207 },
	{ 0xba74, 0xba74, PDF_CMAP_SINGLE, 1910 },
	{ 0xba75, 0xba77, PDF_CMAP_RANGE, 12209 },
	{ 0xba78, 0xba78, PDF_CMAP_SINGLE, 1911 },
	{ 0xba79, 0xba82, PDF_CMAP_RANGE, 12212 },
	{ 0xba83, 0xba85, PDF_CMAP_RANGE, 1912 },
	{ 0xba86, 0xba87, PDF_CMAP_TABLE, 2764 },
	{ 0xba88, 0xba8b, PDF_CMAP_RANGE, 12223 },
	{ 0xba8c, 0xba8c, PDF_CMAP_SINGLE, 1916 },
	{ 0xba8d, 0xbaa7, PDF_CMAP_RANGE, 12227 },
	{ 0xbaa8, 0xbaa9, PDF_CMAP_RANGE, 1917 },
	{ 0xbaaa, 0xbaaa, PDF_CMAP_SINGLE, 12254 },
	{ 0xbaab, 0xbaac, PDF_CMAP_RANGE, 1919 },
	{ 0xbaad, 0xbaaf, PDF_CMAP_RANGE, 12255 },
	{ 0xbab0, 0xbab2, PDF_CMAP_TABLE, 2766 },
	{ 0xbab3, 0xbab7, PDF_CMAP_RANGE, 12259 },
	{ 0xbab8, 0xbab9, PDF_CMAP_RANGE, 1923 },
	{ 0xbaba, 0xbabd, PDF_CMAP_TABLE, 2769 },
	{ 0xbabe, 0xbac3, PDF_CMAP_RANGE, 12266 },
	{ 0xbac4, 0xbac4, PDF_CMAP_SINGLE, 1927 },
	{ 0xbac5, 0xbac7, PDF_CMAP_RANGE, 12272 },
	{ 0xbac8, 0xbac8, PDF_CMAP_SINGLE, 1928 },
	{ 0xbac9, 0xbad7, PDF_CMAP_RANGE, 12275 },
	{ 0xbad8, 0xbad9, PDF_CMAP_RANGE, 1929 },
	{ 0xbada, 0xbafb, PDF_CMAP_RANGE, 12290 },
	{ 0xbafc, 0xbafc, PDF_CMAP_SINGLE, 1931 },
	{ 0xbafd, 0xbaff, PDF_CMAP_RANGE, 12324 },
	{ 0xbb00, 0xbb00, PDF_CMAP_SINGLE, 1932 },
	{ 0xbb01, 0xbb03, PDF_CMAP_RANGE, 12327 },
	{ 0xbb04, 0xbb04, PDF_CMAP_SINGLE, 1933 },
	{ 0xbb05, 0xbb0c, PDF_CMAP_RANGE, 12330 },
	{ 0xbb0d, 0xbb11, PDF_CMAP_TABLE, 2773 },
	{ 0xbb12, 0xbb17, PDF_CMAP_RANGE, 12340 },
	{ 0xbb18, 0xbb18, PDF_CMAP_SINGLE, 1937 },
	{ 0xbb19, 0xbb1b, PDF_CMAP_RANGE, 12346 },
	{ 0xbb1c, 0xbb1c, PDF_CMAP_SINGLE, 1938 },
	{ 0xbb1d, 0xbb1f, PDF_CMAP_RANGE, 12349 },
	{ 0xbb20, 0xbb20, PDF_CMAP_SINGLE, 1939 },
	{ 0xbb21, 0xbb28, PDF_CMAP_RANGE, 12352 },
	{ 0xbb29, 0xbb2b, PDF_CMAP_TABLE, 2778 },
	{ 0xbb2c, 0xbb33, PDF_CMAP_RANGE, 12361 },
	{ 0xbb34, 0xbb36, PDF_CMAP_RANGE, 1942 },
	{ 0xbb37, 0xbb38, PDF_CMAP_TABLE, 2781 },
	{ 0xbb39, 0xbb3a, PDF_CMAP_RANGE, 12370 },
	{ 0xbb3b, 0xbb3e, PDF_CMAP_RANGE, 1946 },
	{ 0xbb3f, 0xbb43, PDF_CMAP_RANGE, 12372 },
	{ 0xbb44, 0xbb45, PDF_CMAP_RANGE, 1950 },
	{ 0xbb46, 0xbb49, PDF_CMAP_TABLE, 2783 },
	{ 0xbb4a, 0xbb4c, PDF_CMAP_RANGE, 12379 },
	{ 0xbb4d, 0xbb4e, PDF_CMAP_TABLE, 2787 },
	{ 0xbb4f, 0xbb50, PDF_CMAP_RANGE, 1955 },
	{ 0xbb51, 0xbb53, PDF_CMAP_RANGE, 12383 },
	{ 0xbb54, 0xbb54, PDF_CMAP_SINGLE, 1957 },
	{ 0xbb55, 0xbb57, PDF_CMAP_RANGE, 12386 },
	{ 0xbb58, 0xbb58, PDF_CMAP_SINGLE, 1958 },
	{ 0xbb59, 0xbb60, PDF_CMAP_RANGE, 12389 },
	{ 0xbb61, 0xbb63, PDF_CMAP_TABLE, 2789 },
	{ 0xbb64, 0xbb6b, PDF_CMAP_RANGE, 12398 },
	{ 0xbb6c, 0xbb6c, PDF_CMAP_SINGLE, 1961 },
	{ 0xbb6d, 0xbb87, PDF_CMAP_RANGE, 12406 },
	{ 0xbb88, 0xbb88, PDF_CMAP_SINGLE, 1962 },
	{ 0xbb89, 0xbb8b, PDF_CMAP_RANGE, 12433 },
	{ 0xbb8c, 0xbb8c, PDF_CMAP_SINGLE, 1963 },
	{ 0xbb8d, 0xbb8f, PDF_CMAP_RANGE, 12436 },
	{ 0xbb90, 0xbb90, PDF_CMAP_SINGLE, 1964 },
	{ 0xbb91, 0xbba3, PDF_CMAP_RANGE, 12439 },
	{ 0xbba4, 0xbba4, PDF_CMAP_SINGLE, 1965 },
	{ 0xbba5, 0xbba7, PDF_CMAP_RANGE, 12458 },
	{ 0xbba8, 0xbba8, PDF_CMAP_SINGLE, 1966 },
	{ 0xbba9, 0xbbab, PDF_CMAP_RANGE, 12461 },
	{ 0xbbac, 0xbbac, PDF_CMAP_SINGLE, 1967 },
	{ 0xbbad, 0xbbb3, PDF_CMAP_RANGE, 12464 },
	{ 0xbbb4, 0xbbb4, PDF_CMAP_SINGLE, 1968 },
	{ 0xbbb5, 0xbbb6, PDF_CMAP_RANGE, 12471 },
	{ 0xbbb7, 0xbbb7, PDF_CMAP_SINGLE, 1969 },
	{ 0xbbb8, 0xbbbf, PDF_CMAP_RANGE, 12473 },
	{ 0xbbc0, 0xbbc0, PDF_CMAP_SINGLE, 1970 },
	{ 0xbbc1, 0xbbc3, PDF_CMAP_RANGE, 12481 },
	{ 0xbbc4, 0xbbc4, PDF_CMAP_SINGLE, 1971 },
	{ 0xbbc5, 0xbbc7, PDF_CMAP_RANGE, 12484 },
	{ 0xbbc8, 0xbbc8, PDF_CMAP_SINGLE, 1972 },
	{ 0xbbc9, 0xbbcf, PDF_CMAP_RANGE, 12487 },
	{ 0xbbd0, 0xbbd0, PDF_CMAP_SINGLE, 1973 },
	{ 0xbbd1, 0xbbd2, PDF_CMAP_RANGE, 12494 },
	{ 0xbbd3, 0xbbd3, PDF_CMAP_SINGLE, 1974 },
	{ 0xbbd4, 0xbbf7, PDF_CMAP_RANGE, 12496 },
	{ 0xbbf8, 0xbbf9, PDF_CMAP_RANGE, 1975 },
	{ 0xbbfa, 0xbbfb, PDF_CMAP_RANGE, 12532 },
	{ 0xbbfc, 0xbbfc, PDF_CMAP_SINGLE, 1977 },
	{ 0xbbfd, 0xbbfe, PDF_CMAP_RANGE, 12534 },
	{ 0xbbff, 0xbc00, PDF_CMAP_RANGE, 1978 },
	{ 0xbc01, 0xbc02, PDF_CMAP_TABLE, 2792 },
	{ 0xbc03, 0xbc07, PDF_CMAP_RANGE, 12537 },
	{ 0xbc08, 0xbc09, PDF_CMAP_RANGE, 1981 },
	{ 0xbc0a, 0xbc0a, PDF_CMAP_SINGLE, 12542 },
	{ 0xbc0b, 0xbc0d, PDF_CMAP_RANGE, 1983 },
	{ 0xbc0e, 0xbc11, PDF_CMAP_TABLE, 2794 },
	{ 0xbc12, 0xbc13, PDF_CMAP_RANGE, 12545 },
	{ 0xbc14, 0xbc18, PDF_CMAP_RANGE, 1988 },
	{ 0xbc19, 0xbc1a, PDF_CMAP_RANGE, 12547 },
	{ 0xbc1b, 0xbc1f, PDF_CMAP_RANGE, 1993 },
	{ 0xbc20, 0xbc23, PDF_CMAP_RANGE, 12549 },
	{ 0xbc24, 0xbc25, PDF_CMAP_RANGE, 1998 },
	{ 0xbc26, 0xbc29, PDF_CMAP_TABLE, 2798 },
	{ 0xbc2a, 0xbc2c, PDF_CMAP_RANGE, 12555 },
	{ 0xbc2d, 0xbc2d, PDF_CMAP_SINGLE, 2002 },
	{ 0xbc2e, 0xbc2f, PDF_CMAP_RANGE, 12558 },
	{ 0xbc30, 0xbc31, PDF_CMAP_RANGE, 2003 },
	{ 0xbc32, 0xbc33, PDF_CMAP_RANGE, 12560 },
	{ 0xbc34, 0xbc34, PDF_CMAP_SINGLE, 2005 },
	{ 0xbc35, 0xbc37, PDF_CMAP_RANGE, 12562 },
	{ 0xbc38, 0xbc38, PDF_CMAP_SINGLE, 2006 },
	{ 0xbc39, 0xbc3f, PDF_CMAP_RANGE, 12565 },
	{ 0xbc40, 0xbc41, PDF_CMAP_RANGE, 2007 },
	{ 0xbc42, 0xbc42, PDF_CMAP_SINGLE, 12572 },
	{ 0xbc43, 0xbc45, PDF_CMAP_RANGE, 2009 },
	{ 0xbc46, 0xbc48, PDF_CMAP_RANGE, 12573 },
	{ 0xbc49, 0xbc49, PDF_CMAP_SINGLE, 2012 },
	{ 0xbc4a, 0xbc4b, PDF_CMAP_RANGE, 12576 },
	{ 0xbc4c, 0xbc4d, PDF_CMAP_RANGE, 2013 },
	{ 0xbc4e, 0xbc4f, PDF_CMAP_RANGE, 12578 },
	{ 0xbc50, 0xbc50, PDF_CMAP_SINGLE, 2015 },
	{ 0xbc51, 0xbc5c, PDF_CMAP_RANGE, 12580 },
	{ 0xbc5d, 0xbc5d, PDF_CMAP_SINGLE, 2016 },
	{ 0xbc5e, 0xbc83, PDF_CMAP_RANGE, 12592 },
	{ 0xbc84, 0xbc85, PDF_CMAP_RANGE, 2017 },
	{ 0xbc86, 0xbc87, PDF_CMAP_RANGE, 12630 },
	{ 0xbc88, 0xbc88, PDF_CMAP_SINGLE, 2019 },
	{ 0xbc89, 0xbc8a, PDF_CMAP_RANGE, 12632 },
	{ 0xbc8b, 0xbc8c, PDF_CMAP_RANGE, 2020 },
	{ 0xbc8d, 0xbc8e, PDF_CMAP_TABLE, 2802 },
	{ 0xbc8f, 0xbc93, PDF_CMAP_RANGE, 12635 },
	{ 0xbc94, 0xbc95, PDF_CMAP_RANGE, 2023 },
	{ 0xbc96, 0xbc98, PDF_CMAP_TABLE, 2804 },
	{ 0xbc99, 0xbc9a, PDF_CMAP_RANGE, 2026 },
	{ 0xbc9b, 0xbc9f, PDF_CMAP_RANGE, 12642 },
	{ 0xbca0, 0xbca1, PDF_CMAP_RANGE, 2028 },
	{ 0xbca2, 0xbca3, PDF_CMAP_RANGE, 12647 },
	{ 0xbca4, 0xbca4, PDF_CMAP_SINGLE, 2030 },
	{ 0xbca5, 0xbca6, PDF_CMAP_RANGE, 12649 },
	{ 0xbca7, 0xbca8, PDF_CMAP_RANGE, 2031 },
	{ 0xbca9, 0xbcaf, PDF_CMAP_RANGE, 12651 },
	{ 0xbcb0, 0xbcb1, PDF_CMAP_RANGE, 2033 },
	{ 0xbcb2, 0xbcb2, PDF_CMAP_SINGLE, 12658 },
	{ 0xbcb3, 0xbcb5, PDF_CMAP_RANGE, 2035 },
	{ 0xbcb6, 0xbcbb, PDF_CMAP_RANGE, 12659 },
	{ 0xbcbc, 0xbcbd, PDF_CMAP_RANGE, 2038 },
	{ 0xbcbe, 0xbcbf, PDF_CMAP_RANGE, 12665 },
	{ 0xbcc0, 0xbcc0, PDF_CMAP_SINGLE, 2040 },
	{ 0xbcc1, 0xbcc3, PDF_CMAP_RANGE, 12667 },
	{ 0xbcc4, 0xbcc4, PDF_CMAP_SINGLE, 2041 },
	{ 0xbcc5, 0xbccc, PDF_CMAP_RANGE, 12670 },
	{ 0xbccd, 0xbcce, PDF_CMAP_TABLE, 2807 },
	{ 0xbccf, 0xbcd1, PDF_CMAP_RANGE, 2043 },
	{ 0xbcd2, 0xbcd4, PDF_CMAP_RANGE, 12679 },
	{ 0xbcd5, 0xbcd5, PDF_CMAP_SINGLE, 2046 },
	{ 0xbcd6, 0xbcd7, PDF_CMAP_RANGE, 12682 },
	{ 0xbcd8, 0xbcd8, PDF_CMAP_SINGLE, 2047 },
	{ 0xbcd9, 0xbcdb, PDF_CMAP_RANGE, 12684 },
	{ 0xbcdc, 0xbcdc, PDF_CMAP_SINGLE, 2048 },
	{ 0xbcdd, 0xbcf3, PDF_CMAP_RANGE, 12687 },
	{ 0xbcf4, 0xbcf6, PDF_CMAP_RANGE, 2049 },
	{ 0xbcf7, 0xbcf8, PDF_CMAP_TABLE, 2809 },
	{ 0xbcf9, 0xbcfb, PDF_CMAP_RANGE, 12711 },
	{ 0xbcfc, 0xbcfc, PDF_CMAP_SINGLE, 2053 },
	{ 0xbcfd, 0xbd03, PDF_CMAP_RANGE, 12714 },
	{ 0xbd04, 0xbd05, PDF_CMAP_RANGE, 2054 },
	{ 0xbd06, 0xbd09, PDF_CMAP_TABLE, 2811 },
	{ 0xbd0a, 0xbd0f, PDF_CMAP_RANGE, 12723 },
	{ 0xbd10, 0xbd10, PDF_CMAP_SINGLE, 2058 },
	{ 0xbd11, 0xbd13, PDF_CMAP_RANGE, 12729 },
	{ 0xbd14, 0xbd14, PDF_CMAP_SINGLE, 2059 },
	{ 0xbd15, 0xbd23, PDF_CMAP_RANGE, 12732 },
	{ 0xbd24, 0xbd24, PDF_CMAP_SINGLE, 2060 },
	{ 0xbd25, 0xbd2b, PDF_CMAP_RANGE, 12747 },
	{ 0xbd2c, 0xbd2c, PDF_CMAP_SINGLE, 2061 },
	{ 0xbd2d, 0xbd3f, PDF_CMAP_RANGE, 12754 },
	{ 0xbd40, 0xbd40, PDF_CMAP_SINGLE, 2062 },
	{ 0xbd41, 0xbd47, PDF_CMAP_RANGE, 12773 },
	{ 0xbd48, 0xbd49, PDF_CMAP_RANGE, 2063 },
	{ 0xbd4a, 0xbd4b, PDF_CMAP_RANGE, 12780 },
	{ 0xbd4c, 0xbd4c, PDF_CMAP_SINGLE, 2065 },
	{ 0xbd4d, 0xbd4f, PDF_CMAP_RANGE, 12782 },
	{ 0xbd50, 0xbd50, PDF_CMAP_SINGLE, 2066 },
	{ 0xbd51, 0xbd57, PDF_CMAP_RANGE, 12785 },
	{ 0xbd58, 0xbd59, PDF_CMAP_RANGE, 2067 },
	{ 0xbd5a, 0xbd63, PDF_CMAP_RANGE, 12792 },
	{ 0xbd64, 0xbd64, PDF_CMAP_SINGLE, 2069 },
	{ 0xbd65, 0xbd67, PDF_CMAP_RANGE, 12802 },
	{ 0xbd68, 0xbd68, PDF_CMAP_SINGLE, 2070 },
	{ 0xbd69, 0xbd7f, PDF_CMAP_RANGE, 12805 },
	{ 0xbd80, 0xbd81, PDF_CMAP_RANGE, 2071 },
	{ 0xbd82, 0xbd83, PDF_CMAP_RANGE, 12828 },
	{ 0xbd84, 0xbd84, PDF_CMAP_SINGLE, 2073 },
	{ 0xbd85, 0xbd86, PDF_CMAP_RANGE, 12830 },
	{ 0xbd87, 0xbd8a, PDF_CMAP_RANGE, 2074 },
	{ 0xbd8b, 0xbd8f, PDF_CMAP_RANGE, 12832 },
	{ 0xbd90, 0xbd91, PDF_CMAP_RANGE, 2078 },
	{ 0xbd92, 0xbd95, PDF_CMAP_TABLE, 2815 },
	{ 0xbd96, 0xbd98, PDF_CMAP_RANGE, 12839 },
	{ 0xbd99, 0xbd9a, PDF_CMAP_RANGE, 2082 },
	{ 0xbd9b, 0xbd9c, PDF_CMAP_TABLE, 2819 },
	{ 0xbd9d, 0xbda3, PDF_CMAP_RANGE, 12843 },
	{ 0xbda4, 0xbda4, PDF_CMAP_SINGLE, 2085 },
	{ 0xbda5, 0xbdaf, PDF_CMAP_RANGE, 12850 },
	{ 0xbdb0, 0xbdb0, PDF_CMAP_SINGLE, 2086 },
	{ 0xbdb1, 0xbdb7, PDF_CMAP_RANGE, 12861 },
	{ 0xbdb8, 0xbdb8, PDF_CMAP_SINGLE, 2087 },
	{ 0xbdb9, 0xbdd3, PDF_CMAP_RANGE, 12868 },
	{ 0xbdd4, 0xbdd5, PDF_CMAP_RANGE, 2088 },
	{ 0xbdd6, 0xbdd7, PDF_CMAP_RANGE, 12895 },
	{ 0xbdd8, 0xbdd8, PDF_CMAP_SINGLE, 2090 },
	{ 0xbdd9, 0xbddb, PDF_CMAP_RANGE, 12897 },
	{ 0xbddc, 0xbddc, PDF_CMAP_SINGLE, 2091 },
	{ 0xbddd, 0xbde8, PDF_CMAP_RANGE, 12900 },
	{ 0xbde9, 0xbde9, PDF_CMAP_SINGLE, 2092 },
	{ 0xbdea, 0xbdef, PDF_CMAP_RANGE, 12912 },
	{ 0xbdf0, 0xbdf0, PDF_CMAP_SINGLE, 2093 },
	{ 0xbdf1, 0xbdf3, PDF_CMAP_RANGE, 12918 },
	{ 0xbdf4, 0xbdf4, PDF_CMAP_SINGLE, 2094 },
	{ 0xbdf5, 0xbdf7, PDF_CMAP_RANGE, 12921 },
	{ 0xbdf8, 0xbdf8, PDF_CMAP_SINGLE, 2095 },
	{ 0xbdf9, 0xbdff, PDF_CMAP_RANGE, 12924 },
	{ 0xbe00, 0xbe00, PDF_CMAP_SINGLE, 2096 },
	{ 0xbe01, 0xbe02, PDF_CMAP_RANGE, 12931 },
	{ 0xbe03, 0xbe05, PDF_CMAP_TABLE, 2821 },
	{ 0xbe06, 0xbe0b, PDF_CMAP_RANGE, 12934 },
	{ 0xbe0c, 0xbe0d, PDF_CMAP_RANGE, 2099 },
	{ 0xbe0e, 0xbe0f, PDF_CMAP_RANGE, 12940 },
	{ 0xbe10, 0xbe10, PDF_CMAP_SINGLE, 2101 },
	{ 0xbe11, 0xbe13, PDF_CMAP_RANGE, 12942 },
	{ 0xbe14, 0xbe14, PDF_CMAP_SINGLE, 2102 },
	{ 0xbe15, 0xbe1b, PDF_CMAP_RANGE, 12945 },
	{ 0xbe1c, 0xbe1d, PDF_CMAP_RANGE, 2103 },
	{ 0xbe1e, 0xbe1f, PDF_CMAP_TABLE, 2824 },
	{ 0xbe20, 0xbe43, PDF_CMAP_RANGE, 12953 },
	{ 0xbe44, 0xbe45, PDF_CMAP_RANGE, 2106 },
	{ 0xbe46, 0xbe47, PDF_CMAP_RANGE, 12989 },
	{ 0xbe48, 0xbe48, PDF_CMAP_SINGLE, 2108 },
	{ 0xbe49, 0xbe4b, PDF_CMAP_RANGE, 12991 },
	{ 0xbe4c, 0xbe4e, PDF_CMAP_TABLE, 2826 },
	{ 0xbe4f, 0xbe53, PDF_CMAP_RANGE, 12995 },
	{ 0xbe54, 0xbe55, PDF_CMAP_RANGE, 2111 },
	{ 0xbe56, 0xbe58, PDF_CMAP_TABLE, 2829 },
	{ 0xbe59, 0xbe5b, PDF_CMAP_RANGE, 2114 },
	{ 0xbe5c, 0xbe5f, PDF_CMAP_RANGE, 13002 },
	{ 0xbe60, 0xbe61, PDF_CMAP_RANGE, 2117 },
	{ 0xbe62, 0xbe63, PDF_CMAP_RANGE, 13006 },
	{ 0xbe64, 0xbe64, PDF_CMAP_SINGLE, 2119 },
	{ 0xbe65, 0xbe67, PDF_CMAP_RANGE, 13008 },
	{ 0xbe68, 0xbe6a, PDF_CMAP_TABLE, 2832 },
	{ 0xbe6b, 0xbe6f, PDF_CMAP_RANGE, 13012 },
	{ 0xbe70, 0xbe71, PDF_CMAP_RANGE, 2122 },
	{ 0xbe72, 0xbe72, PDF_CMAP_SINGLE, 13017 },
	{ 0xbe73, 0xbe75, PDF_CMAP_RANGE, 2124 },
	{ 0xbe76, 0xbe7a, PDF_CMAP_RANGE, 13018 },
	{ 0xbe7b, 0xbe7d, PDF_CMAP_RANGE, 2127 },
	{ 0xbe7e, 0xbe7f, PDF_CMAP_RANGE, 13023 },
	{ 0xbe80, 0xbe80, PDF_CMAP_SINGLE, 2130 },
	{ 0xbe81, 0xbe83, PDF_CMAP_RANGE, 13025 },
	{ 0xbe84, 0xbe84, PDF_CMAP_SINGLE, 2131 },
	{ 0xbe85, 0xbe8b, PDF_CMAP_RANGE, 13028 },
	{ 0xbe8c, 0xbe8d, PDF_CMAP_RANGE, 2132 },
	{ 0xbe8e, 0xbe8e, PDF_CMAP_SINGLE, 13035 },
	{ 0xbe8f, 0xbe91, PDF_CMAP_RANGE, 2134 },
	{ 0xbe92, 0xbe97, PDF_CMAP_RANGE, 13036 },
	{ 0xbe98, 0xbe99, PDF_CMAP_RANGE, 2137 },
	{ 0xbe9a, 0xbea7, PDF_CMAP_RANGE, 13042 },
	{ 0xbea8, 0xbea8, PDF_CMAP_SINGLE, 2139 },
	{ 0xbea9, 0xbecf, PDF_CMAP_RANGE, 13056 },
	{ 0xbed0, 0xbed1, PDF_CMAP_RANGE, 2140 },
	{ 0xbed2, 0xbed3, PDF_CMAP_RANGE, 13095 },
	{ 0xbed4, 0xbed4, PDF_CMAP_SINGLE, 2142 },
	{ 0xbed5, 0xbed6, PDF_CMAP_RANGE, 13097 },
	{ 0xbed7, 0xbed8, PDF_CMAP_RANGE, 2143 },
	{ 0xbed9, 0xbedf, PDF_CMAP_RANGE, 13099 },
	{ 0xbee0, 0xbee0, PDF_CMAP_SINGLE, 2145 },
	{ 0xbee1, 0xbee2, PDF_CMAP_RANGE, 13106 },
	{ 0xbee3, 0xbee5, PDF_CMAP_RANGE, 2146 },
	{ 0xbee6, 0xbeeb, PDF_CMAP_RANGE, 13108 },
	{ 0xbeec, 0xbeec, PDF_CMAP_SINGLE, 2149 },
	{ 0xbeed, 0xbf00, PDF_CMAP_RANGE, 13114 },
	{ 0xbf01, 0xbf01, PDF_CMAP_SINGLE, 2150 },
	{ 0xbf02, 0xbf07, PDF_CMAP_RANGE, 13134 },
	{ 0xbf08, 0xbf09, PDF_CMAP_RANGE, 2151 },
	{ 0xbf0a, 0xbf17, PDF_CMAP_RANGE, 13140 },
	{ 0xbf18, 0xbf19, PDF_CMAP_RANGE, 2153 },
	{ 0xbf1a, 0xbf1a, PDF_CMAP_SINGLE, 13154 },
	{ 0xbf1b, 0xbf1d, PDF_CMAP_RANGE, 2155 },
	{ 0xbf1e, 0xbf3f, PDF_CMAP_RANGE, 13155 },
	{ 0xbf40, 0xbf41, PDF_CMAP_RANGE, 2158 },
	{ 0xbf42, 0xbf43, PDF_CMAP_RANGE, 13189 },
	{ 0xbf44, 0xbf44, PDF_CMAP_SINGLE, 2160 },
	{ 0xbf45, 0xbf47, PDF_CMAP_RANGE, 13191 },
	{ 0xbf48, 0xbf48, PDF_CMAP_SINGLE, 2161 },
	{ 0xbf49, 0xbf4f, PDF_CMAP_RANGE, 13194 },
	{ 0xbf50, 0xbf51, PDF_CMAP_RANGE, 2162 },
	{ 0xbf52, 0xbf54, PDF_CMAP_RANGE, 13201 },
	{ 0xbf55, 0xbf55, PDF_CMAP_SINGLE, 2164 },
	{ 0xbf56, 0xbf93, PDF_CMAP_RANGE, 13204 },
	{ 0xbf94, 0xbf94, PDF_CMAP_SINGLE, 2165 },
	{ 0xbf95, 0xbfaf, PDF_CMAP_RANGE, 13266 },
	{ 0xbfb0, 0xbfb0, PDF_CMAP_SINGLE, 2166 },
	{ 0xbfb1, 0xbfc4, PDF_CMAP_RANGE, 13293 },
	{ 0xbfc5, 0xbfc5, PDF_CMAP_SINGLE, 2167 },
	{ 0xbfc6, 0xbfcb, PDF_CMAP_RANGE, 13313 },
	{ 0xbfcc, 0xbfcd, PDF_CMAP_RANGE, 2168 },
	{ 0xbfce, 0xbfcf, PDF_CMAP_RANGE, 13319 },
	{ 0xbfd0, 0xbfd0, PDF_CMAP_SINGLE, 2170 },
	{ 0xbfd1, 0xbfd3, PDF_CMAP_RANGE, 13321 },
	{ 0xbfd4, 0xbfd4, PDF_CMAP_SINGLE, 2171 },
	{ 0xbfd5, 0xbfdb, PDF_CMAP_RANGE, 13324 },
	{ 0xbfdc, 0xbfdc, PDF_CMAP_SINGLE, 2172 },
	{ 0xbfdd, 0xbfde, PDF_CMAP_RANGE, 13331 },
	{ 0xbfdf, 0xbfe1, PDF_CMAP_TABLE, 2835 },
	{ 0xbfe2, 0xc03b, PDF_CMAP_RANGE, 13334 },
	{ 0xc03c, 0xc03c, PDF_CMAP_SINGLE, 2175 },
	{ 0xc03d, 0xc050, PDF_CMAP_RANGE, 13424 },
	{ 0xc051, 0xc051, PDF_CMAP_SINGLE, 2176 },
	{ 0xc052, 0xc057, PDF_CMAP_RANGE, 13444 },
	{ 0xc058, 0xc058, PDF_CMAP_SINGLE, 2177 },
	{ 0xc059, 0xc05b, PDF_CMAP_RANGE, 13450 },
	{ 0xc05c, 0xc05c, PDF_CMAP_SINGLE, 2178 },
	{ 0xc05d, 0xc05f, PDF_CMAP_RANGE, 13453 },
	{ 0xc060, 0xc060, PDF_CMAP_SINGLE, 2179 },
	{ 0xc061, 0xc067, PDF_CMAP_RANGE, 13456 },
	{ 0xc068, 0xc069, PDF_CMAP_RANGE, 2180 },
	{ 0xc06a, 0xc08f, PDF_CMAP_RANGE, 13463 },
	{ 0xc090, 0xc091, PDF_CMAP_RANGE, 2182 },
	{ 0xc092, 0xc093, PDF_CMAP_RANGE, 13501 },
	{ 0xc094, 0xc094, PDF_CMAP_SINGLE, 2184 },
	{ 0xc095, 0xc097, PDF_CMAP_RANGE, 13503 },
	{ 0xc098, 0xc098, PDF_CMAP_SINGLE, 2185 },
	{ 0xc099, 0xc09f, PDF_CMAP_RANGE, 13506 },
	{ 0xc0a0, 0xc0a1, PDF_CMAP_RANGE, 2186 },
	{ 0xc0a2, 0xc0a5, PDF_CMAP_TABLE, 2838 },
	{ 0xc0a6, 0xc0ab, PDF_CMAP_RANGE, 13515 },
	{ 0xc0ac, 0xc0ad, PDF_CMAP_RANGE, 2190 },
	{ 0xc0ae, 0xc0ae, PDF_CMAP_SINGLE, 13521 },
	{ 0xc0af, 0xc0b0, PDF_CMAP_RANGE, 2192 },
	{ 0xc0b1, 0xc0b2, PDF_CMAP_RANGE, 13522 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 2194 },
	{ 0xc0b7, 0xc0bb, PDF_CMAP_RANGE, 13524 },
	{ 0xc0bc, 0xc0bd, PDF_CMAP_RANGE, 2198 },
	{ 0xc0be, 0xc0be, PDF_CMAP_SINGLE, 13529 },
	{ 0xc0bf, 0xc0c1, PDF_CMAP_RANGE, 2200 },
	{ 0xc0c2, 0xc0c4, PDF_CMAP_RANGE, 13530 },
	{ 0xc0c5, 0xc0c5, PDF_CMAP_SINGLE, 2203 },
	{ 0xc0c6, 0xc0c7, PDF_CMAP_RANGE, 13533 },
	{ 0xc0c8, 0xc0c9, PDF_CMAP_RANGE, 2204 },
	{ 0xc0ca, 0xc0cb, PDF_CMAP_RANGE, 13535 },
	{ 0xc0cc, 0xc0cc, PDF_CMAP_SINGLE, 2206 },
	{ 0xc0cd, 0xc0cf, PDF_CMAP_RANGE, 13537 },
	{ 0xc0d0, 0xc0d0, PDF_CMAP_SINGLE, 2207 },
	{ 0xc0d1, 0xc0d7, PDF_CMAP_RANGE, 13540 },
	{ 0xc0d8, 0xc0d9, PDF_CMAP_RANGE, 2208 },
	{ 0xc0da, 0xc0da, PDF_CMAP_SINGLE, 13547 },
	{ 0xc0db, 0xc0dd, PDF_CMAP_RANGE, 2210 },
	{ 0xc0de, 0xc0e3, PDF_CMAP_RANGE, 13548 },
	{ 0xc0e4, 0xc0e5, PDF_CMAP_RANGE, 2213 },
	{ 0xc0e6, 0xc0e7, PDF_CMAP_RANGE, 13554 },
	{ 0xc0e8, 0xc0e8, PDF_CMAP_SINGLE, 2215 },
	{ 0xc0e9, 0xc0eb, PDF_CMAP_RANGE, 13556 },
	{ 0xc0ec, 0xc0ec, PDF_CMAP_SINGLE, 2216 },
	{ 0xc0ed, 0xc0f3, PDF_CMAP_RANGE, 13559 },
	{ 0xc0f4, 0xc0f5, PDF_CMAP_RANGE, 2217 },
	{ 0xc0f6, 0xc0f9, PDF_CMAP_TABLE, 2842 },
	{ 0xc0fa, 0xc0ff, PDF_CMAP_RANGE, 13568 },
	{ 0xc100, 0xc100, PDF_CMAP_SINGLE, 2221 },
	{ 0xc101, 0xc103, PDF_CMAP_RANGE, 13574 },
	{ 0xc104, 0xc104, PDF_CMAP_SINGLE, 2222 },
	{ 0xc105, 0xc107, PDF_CMAP_RANGE, 13577 },
	{ 0xc108, 0xc108, PDF_CMAP_SINGLE, 2223 },
	{ 0xc109, 0xc10f, PDF_CMAP_RANGE, 13580 },
	{ 0xc110, 0xc110, PDF_CMAP_SINGLE, 2224 },
	{ 0xc111, 0xc114, PDF_CMAP_RANGE, 13587 },
	{ 0xc115, 0xc115, PDF_CMAP_SINGLE, 2225 },
	{ 0xc116, 0xc11b, PDF_CMAP_RANGE, 13591 },
	{ 0xc11c, 0xc120, PDF_CMAP_RANGE, 2226 },
	{ 0xc121, 0xc122, PDF_CMAP_RANGE, 13597 },
	{ 0xc123, 0xc124, PDF_CMAP_RANGE, 2231 },
	{ 0xc125, 0xc125, PDF_CMAP_SINGLE, 13599 },
	{ 0xc126, 0xc127, PDF_CMAP_RANGE, 2233 },
	{ 0xc128, 0xc12b, PDF_CMAP_RANGE, 13600 },
	{ 0xc12c, 0xc12d, PDF_CMAP_RANGE, 2235 },
	{ 0xc12e, 0xc12e, PDF_CMAP_SINGLE, 13604 },
	{ 0xc12f, 0xc131, PDF_CMAP_RANGE, 2237 },
	{ 0xc132, 0xc135, PDF_CMAP_RANGE, 13605 },
	{ 0xc136, 0xc137, PDF_CMAP_TABLE, 2846 },
	{ 0xc138, 0xc139, PDF_CMAP_RANGE, 2241 },
	{ 0xc13a, 0xc13b, PDF_CMAP_RANGE, 13610 },
	{ 0xc13c, 0xc13c, PDF_CMAP_SINGLE, 2243 },
	{ 0xc13d, 0xc13f, PDF_CMAP_RANGE, 13612 },
	{ 0xc140, 0xc140, PDF_CMAP_SINGLE, 2244 },
	{ 0xc141, 0xc147, PDF_CMAP_RANGE, 13615 },
	{ 0xc148, 0xc149, PDF_CMAP_RANGE, 2245 },
	{ 0xc14a, 0xc14a, PDF_CMAP_SINGLE, 13622 },
	{ 0xc14b, 0xc14d, PDF_CMAP_RANGE, 2247 },
	{ 0xc14e, 0xc153, PDF_CMAP_RANGE, 13623 },
	{ 0xc154, 0xc155, PDF_CMAP_RANGE, 2250 },
	{ 0xc156, 0xc157, PDF_CMAP_RANGE, 13629 },
	{ 0xc158, 0xc158, PDF_CMAP_SINGLE, 2252 },
	{ 0xc159, 0xc15b, PDF_CMAP_RANGE, 13631 },
	{ 0xc15c, 0xc15c, PDF_CMAP_SINGLE, 2253 },
	{ 0xc15d, 0xc163, PDF_CMAP_RANGE, 13634 },
	{ 0xc164, 0xc165, PDF_CMAP_RANGE, 2254 },
	{ 0xc166, 0xc166, PDF_CMAP_SINGLE, 13641 },
	{ 0xc167, 0xc169, PDF_CMAP_RANGE, 2256 },
	{ 0xc16a, 0xc16f, PDF_CMAP_RANGE, 13642 },
	{ 0xc170, 0xc170, PDF_CMAP_SINGLE, 2259 },
	{ 0xc171, 0xc173, PDF_CMAP_RANGE, 13648 },
	{ 0xc174, 0xc174, PDF_CMAP_SINGLE, 2260 },
	{ 0xc175, 0xc177, PDF_CMAP_RANGE, 13651 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 2261 },
	{ 0xc179, 0xc184, PDF_CMAP_RANGE, 13654 },
	{ 0xc185, 0xc185, PDF_CMAP_SINGLE, 2262 },
	{ 0xc186, 0xc18b, PDF_CMAP_RANGE, 13666 },
	{ 0xc18c, 0xc18e, PDF_CMAP_RANGE, 2263 },
	{ 0xc18f, 0xc190, PDF_CMAP_TABLE, 2848 },
	{ 0xc191, 0xc193, PDF_CMAP_RANGE, 13673 },
	{ 0xc194, 0xc196, PDF_CMAP_TABLE, 2850 },
	{ 0xc197, 0xc19b, PDF_CMAP_RANGE, 13677 },
	{ 0xc19c, 0xc19d, PDF_CMAP_RANGE, 2269 },
	{ 0xc19e, 0xc1a1, PDF_CMAP_TABLE, 2853 },
	{ 0xc1a2, 0xc1a4, PDF_CMAP_RANGE, 13684 },
	{ 0xc1a5, 0xc1a5, PDF_CMAP_SINGLE, 2273 },
	{ 0xc1a6, 0xc1a7, PDF_CMAP_RANGE, 13687 },
	{ 0xc1a8, 0xc1a9, PDF_CMAP_RANGE, 2274 },
	{ 0xc1aa, 0xc1ab, PDF_CMAP_RANGE, 13689 },
	{ 0xc1ac, 0xc1ac, PDF_CMAP_SINGLE, 2276 },
	{ 0xc1ad, 0xc1af, PDF_CMAP_RANGE, 13691 },
	{ 0xc1b0, 0xc1b0, PDF_CMAP_SINGLE, 2277 },
	{ 0xc1b1, 0xc1bc, PDF_CMAP_RANGE, 13694 },
	{ 0xc1bd, 0xc1bd, PDF_CMAP_SINGLE, 2278 },
	{ 0xc1be, 0xc1c3, PDF_CMAP_RANGE, 13706 },
	{ 0xc1c4, 0xc1c4, PDF_CMAP_SINGLE, 2279 },
	{ 0xc1c5, 0xc1c7, PDF_CMAP_RANGE, 13712 },
	{ 0xc1c8, 0xc1c8, PDF_CMAP_SINGLE, 2280 },
	{ 0xc1c9, 0xc1cb, PDF_CMAP_RANGE, 13715 },
	{ 0xc1cc, 0xc1cc, PDF_CMAP_SINGLE, 2281 },
	{ 0xc1cd, 0xc1d3, PDF_CMAP_RANGE, 13718 },
	{ 0xc1d4, 0xc1d4, PDF_CMAP_SINGLE, 2282 },
	{ 0xc1d5, 0xc1d6, PDF_CMAP_RANGE, 13725 },
	{ 0xc1d7, 0xc1d8, PDF_CMAP_RANGE, 2283 },
	{ 0xc1d9, 0xc1df, PDF_CMAP_RANGE, 13727 },
	{ 0xc1e0, 0xc1e0, PDF_CMAP_SINGLE, 2285 },
	{ 0xc1e1, 0xc1e3, PDF_CMAP_RANGE, 13734 },
	{ 0xc1e4, 0xc1e4, PDF_CMAP_SINGLE, 2286 },
	{ 0xc1e5, 0xc1e7, PDF_CMAP_RANGE, 13737 },
	{ 0xc1e8, 0xc1e8, PDF_CMAP_SINGLE, 2287 },
	{ 0xc1e9, 0xc1ef, PDF_CMAP_RANGE, 13740 },
	{ 0xc1f0, 0xc1f1, PDF_CMAP_RANGE, 2288 },
	{ 0xc1f2, 0xc1f3, PDF_CMAP_TABLE, 2857 },
	{ 0xc1f4, 0xc1fb, PDF_CMAP_RANGE, 13748 },
	{ 0xc1fc, 0xc1fd, PDF_CMAP_RANGE, 2291 },
	{ 0xc1fe, 0xc1ff, PDF_CMAP_RANGE, 13756 },
	{ 0xc200, 0xc200, PDF_CMAP_SINGLE, 2293 },
	{ 0xc201, 0xc203, PDF_CMAP_RANGE, 13758 },
	{ 0xc204, 0xc204, PDF_CMAP_SINGLE, 2294 },
	{ 0xc205, 0xc20b, PDF_CMAP_RANGE, 13761 },
	{ 0xc20c, 0xc20d, PDF_CMAP_RANGE, 2295 },
	{ 0xc20e, 0xc211, PDF_CMAP_TABLE, 2859 },
	{ 0xc212, 0xc217, PDF_CMAP_RANGE, 13770 },
	{ 0xc218, 0xc219, PDF_CMAP_RANGE, 2299 },
	{ 0xc21a, 0xc21b, PDF_CMAP_RANGE, 13776 },
	{ 0xc21c, 0xc21c, PDF_CMAP_SINGLE, 2301 },
	{ 0xc21d, 0xc21e, PDF_CMAP_RANGE, 13778 },
	{ 0xc21f, 0xc220, PDF_CMAP_RANGE, 2302 },
	{ 0xc221, 0xc227, PDF_CMAP_RANGE, 13780 },
	{ 0xc228, 0xc229, PDF_CMAP_RANGE, 2304 },
	{ 0xc22a, 0xc230, PDF_CMAP_TABLE, 2863 },
	{ 0xc231, 0xc232, PDF_CMAP_RANGE, 2309 },
	{ 0xc233, 0xc234, PDF_CMAP_TABLE, 2870 },
	{ 0xc235, 0xc247, PDF_CMAP_RANGE, 13792 },
	{ 0xc248, 0xc248, PDF_CMAP_SINGLE, 2312 },
	{ 0xc249, 0xc24f, PDF_CMAP_RANGE, 13811 },
	{ 0xc250, 0xc251, PDF_CMAP_RANGE, 2313 },
	{ 0xc252, 0xc253, PDF_CMAP_RANGE, 13818 },
	{ 0xc254, 0xc254, PDF_CMAP_SINGLE, 2315 },
	{ 0xc255, 0xc257, PDF_CMAP_RANGE, 13820 },
	{ 0xc258, 0xc258, PDF_CMAP_SINGLE, 2316 },
	{ 0xc259, 0xc25f, PDF_CMAP_RANGE, 13823 },
	{ 0xc260, 0xc260, PDF_CMAP_SINGLE, 2317 },
	{ 0xc261, 0xc264, PDF_CMAP_RANGE, 13830 },
	{ 0xc265, 0xc265, PDF_CMAP_SINGLE, 2318 },
	{ 0xc266, 0xc26b, PDF_CMAP_RANGE, 13834 },
	{ 0xc26c, 0xc26d, PDF_CMAP_RANGE, 2319 },
	{ 0xc26e, 0xc26f, PDF_CMAP_RANGE, 13840 },
	{ 0xc270, 0xc270, PDF_CMAP_SINGLE, 2321 },
	{ 0xc271, 0xc273, PDF_CMAP_RANGE, 13842 },
	{ 0xc274, 0xc274, PDF_CMAP_SINGLE, 2322 },
	{ 0xc275, 0xc27b, PDF_CMAP_RANGE, 13845 },
	{ 0xc27c, 0xc27d, PDF_CMAP_RANGE, 2323 },
	{ 0xc27e, 0xc281, PDF_CMAP_TABLE, 2872 },
	{ 0xc282, 0xc287, PDF_CMAP_RANGE, 13854 },
	{ 0xc288, 0xc289, PDF_CMAP_RANGE, 2327 },
	{ 0xc28a, 0xc28f, PDF_CMAP_RANGE, 13860 },
	{ 0xc290, 0xc290, PDF_CMAP_SINGLE, 2329 },
	{ 0xc291, 0xc297, PDF_CMAP_RANGE, 13866 },
	{ 0xc298, 0xc298, PDF_CMAP_SINGLE, 2330 },
	{ 0xc299, 0xc29a, PDF_CMAP_RANGE, 13873 },
	{ 0xc29b, 0xc29d, PDF_CMAP_TABLE, 2876 },
	{ 0xc29e, 0xc2a3, PDF_CMAP_RANGE, 13876 },
	{ 0xc2a4, 0xc2a5, PDF_CMAP_RANGE, 2333 },
	{ 0xc2a6, 0xc2a7, PDF_CMAP_RANGE, 13882 },
	{ 0xc2a8, 0xc2a8, PDF_CMAP_SINGLE, 2335 },
	{ 0xc2a9, 0xc2ab, PDF_CMAP_RANGE, 13884 },
	{ 0xc2ac, 0xc2ad, PDF_CMAP_RANGE, 2336 },
	{ 0xc2ae, 0xc2b3, PDF_CMAP_RANGE, 13887 },
	{ 0xc2b4, 0xc2b5, PDF_CMAP_RANGE, 2338 },
	{ 0xc2b6, 0xc2b9, PDF_CMAP_TABLE, 2879 },
	{ 0xc2ba, 0xc2db, PDF_CMAP_RANGE, 13895 },
	{ 0xc2dc, 0xc2dd, PDF_CMAP_RANGE, 2342 },
	{ 0xc2de, 0xc2df, PDF_CMAP_RANGE, 13929 },
	{ 0xc2e0, 0xc2e0, PDF_CMAP_SINGLE, 2344 },
	{ 0xc2e1, 0xc2e2, PDF_CMAP_RANGE, 13931 },
	{ 0xc2e3, 0xc2e4, PDF_CMAP_RANGE, 2345 },
	{ 0xc2e5, 0xc2ea, PDF_CMAP_RANGE, 13933 },
	{ 0xc2eb, 0xc2ed, PDF_CMAP_RANGE, 2347 },
	{ 0xc2ee, 0xc2f1, PDF_CMAP_TABLE, 2883 },
	{ 0xc2f2, 0xc2f5, PDF_CMAP_RANGE, 13941 },
	{ 0xc2f6, 0xc2f7, PDF_CMAP_TABLE, 2887 },
	{ 0xc2f8, 0xc2f9, PDF_CMAP_RANGE, 2353 },
	{ 0xc2fa, 0xc2fa, PDF_CMAP_SINGLE, 13946 },
	{ 0xc2fb, 0xc2fc, PDF_CMAP_RANGE, 2355 },
	{ 0xc2fd, 0xc2ff, PDF_CMAP_RANGE, 13947 },
	{ 0xc300, 0xc300, PDF_CMAP_SINGLE, 2357 },
	{ 0xc301, 0xc307, PDF_CMAP_RANGE, 13950 },
	{ 0xc308, 0xc309, PDF_CMAP_RANGE, 2358 },
	{ 0xc30a, 0xc30b, PDF_CMAP_RANGE, 13957 },
	{ 0xc30c, 0xc30d, PDF_CMAP_RANGE, 2360 },
	{ 0xc30e, 0xc312, PDF_CMAP_RANGE, 13959 },
	{ 0xc313, 0xc315, PDF_CMAP_RANGE, 2362 },
	{ 0xc316, 0xc317, PDF_CMAP_RANGE, 13964 },
	{ 0xc318, 0xc318, PDF_CMAP_SINGLE, 2365 },
	{ 0xc319, 0xc31b, PDF_CMAP_RANGE, 13966 },
	{ 0xc31c, 0xc31c, PDF_CMAP_SINGLE, 2366 },
	{ 0xc31d, 0xc323, PDF_CMAP_RANGE, 13969 },
	{ 0xc324, 0xc325, PDF_CMAP_RANGE, 2367 },
	{ 0xc326, 0xc327, PDF_CMAP_RANGE, 13976 },
	{ 0xc328, 0xc329, PDF_CMAP_RANGE, 2369 },
	{ 0xc32a, 0xc344, PDF_CMAP_RANGE, 13978 },
	{ 0xc345, 0xc345, PDF_CMAP_SINGLE, 2371 },
	{ 0xc346, 0xc367, PDF_CMAP_RANGE, 14005 },
	{ 0xc368, 0xc369, PDF_CMAP_RANGE, 2372 },
	{ 0xc36a, 0xc36b, PDF_CMAP_RANGE, 14039 },
	{ 0xc36c, 0xc36c, PDF_CMAP_SINGLE, 2374 },
	{ 0xc36d, 0xc36f, PDF_CMAP_RANGE, 14041 },
	{ 0xc370, 0xc372, PDF_CMAP_TABLE, 2889 },
	{ 0xc373, 0xc377, PDF_CMAP_RANGE, 14045 },
	{ 0xc378, 0xc379, PDF_CMAP_RANGE, 2377 },
	{ 0xc37a, 0xc37b, PDF_CMAP_RANGE, 14050 },
	{ 0xc37c, 0xc37d, PDF_CMAP_RANGE, 2379 },
	{ 0xc37e, 0xc383, PDF_CMAP_RANGE, 14052 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 2381 },
	{ 0xc385, 0xc387, PDF_CMAP_RANGE, 14058 },
	{ 0xc388, 0xc388, PDF_CMAP_SINGLE, 2382 },
	{ 0xc389, 0xc38b, PDF_CMAP_RANGE, 14061 },
	{ 0xc38c, 0xc38c, PDF_CMAP_SINGLE, 2383 },
	{ 0xc38d, 0xc3bf, PDF_CMAP_RANGE, 14064 },
	{ 0xc3c0, 0xc3c0, PDF_CMAP_SINGLE, 2384 },
	{ 0xc3c1, 0xc3d7, PDF_CMAP_RANGE, 14115 },
	{ 0xc3d8, 0xc3d9, PDF_CMAP_RANGE, 2385 },
	{ 0xc3da, 0xc3db, PDF_CMAP_RANGE, 14138 },
	{ 0xc3dc, 0xc3dc, PDF_CMAP_SINGLE, 2387 },
	{ 0xc3dd, 0xc3de, PDF_CMAP_RANGE, 14140 },
	{ 0xc3df, 0xc3e0, PDF_CMAP_RANGE, 2388 },
	{ 0xc3e1, 0xc3e2, PDF_CMAP_TABLE, 2892 },
	{ 0xc3e3, 0xc3e7, PDF_CMAP_RANGE, 14143 },
	{ 0xc3e8, 0xc3e9, PDF_CMAP_RANGE, 2391 },
	{ 0xc3ea, 0xc3ec, PDF_CMAP_RANGE, 14148 },
	{ 0xc3ed, 0xc3ed, PDF_CMAP_SINGLE, 2393 },
	{ 0xc3ee, 0xc3f3, PDF_CMAP_RANGE, 14151 },
	{ 0xc3f4, 0xc3f5, PDF_CMAP_RANGE, 2394 },
	{ 0xc3f6, 0xc3f7, PDF_CMAP_RANGE, 14157 },
	{ 0xc3f8, 0xc3f8, PDF_CMAP_SINGLE, 2396 },
	{ 0xc3f9, 0xc407, PDF_CMAP_RANGE, 14159 },
	{ 0xc408, 0xc408, PDF_CMAP_SINGLE, 2397 },
	{ 0xc409, 0xc40f, PDF_CMAP_RANGE, 14174 },
	{ 0xc410, 0xc410, PDF_CMAP_SINGLE, 2398 },
	{ 0xc411, 0xc423, PDF_CMAP_RANGE, 14181 },
	{ 0xc424, 0xc424, PDF_CMAP_SINGLE, 2399 },
	{ 0xc425, 0xc42b, PDF_CMAP_RANGE, 14200 },
	{ 0xc42c, 0xc42c, PDF_CMAP_SINGLE, 2400 },
	{ 0xc42d, 0xc42f, PDF_CMAP_RANGE, 14207 },
	{ 0xc430, 0xc430, PDF_CMAP_SINGLE, 2401 },
	{ 0xc431, 0xc433, PDF_CMAP_RANGE, 14210 },
	{ 0xc434, 0xc434, PDF_CMAP_SINGLE, 2402 },
	{ 0xc435, 0xc43b, PDF_CMAP_RANGE, 14213 },
	{ 0xc43c, 0xc43d, PDF_CMAP_RANGE, 2403 },
	{ 0xc43e, 0xc447, PDF_CMAP_RANGE, 14220 },
	{ 0xc448, 0xc448, PDF_CMAP_SINGLE, 2405 },
	{ 0xc449, 0xc463, PDF_CMAP_RANGE, 14230 },
	{ 0xc464, 0xc465, PDF_CMAP_RANGE, 2406 },
	{ 0xc466, 0xc467, PDF_CMAP_RANGE, 14257 },
	{ 0xc468, 0xc468, PDF_CMAP_SINGLE, 2408 },
	{ 0xc469, 0xc46b, PDF_CMAP_RANGE, 14259 },
	{ 0xc46c, 0xc46c, PDF_CMAP_SINGLE, 2409 },
	{ 0xc46d, 0xc473, PDF_CMAP_RANGE, 14262 },
	{ 0xc474, 0xc475, PDF_CMAP_RANGE, 2410 },
	{ 0xc476, 0xc478, PDF_CMAP_RANGE, 14269 },
	{ 0xc479, 0xc479, PDF_CMAP_SINGLE, 2412 },
	{ 0xc47a, 0xc47f, PDF_CMAP_RANGE, 14272 },
	{ 0xc480, 0xc480, PDF_CMAP_SINGLE, 2413 },
	{ 0xc481, 0xc493, PDF_CMAP_RANGE, 14278 },
	{ 0xc494, 0xc494, PDF_CMAP_SINGLE, 2414 },
	{ 0xc495, 0xc49b, PDF_CMAP_RANGE, 14297 },
	{ 0xc49c, 0xc49c, PDF_CMAP_SINGLE, 2415 },
	{ 0xc49d, 0xc4b7, PDF_CMAP_RANGE, 14304 },
	{ 0xc4b8, 0xc4b8, PDF_CMAP_SINGLE, 2416 },
	{ 0xc4b9, 0xc4bb, PDF_CMAP_RANGE, 14331 },
	{ 0xc4bc, 0xc4bc, PDF_CMAP_SINGLE, 2417 },
	{ 0xc4bd, 0xc4e8, PDF_CMAP_RANGE, 14334 },
	{ 0xc4e9, 0xc4e9, PDF_CMAP_SINGLE, 2418 },
	{ 0xc4ea, 0xc4ef, PDF_CMAP_RANGE, 14378 },
	{ 0xc4f0, 0xc4f1, PDF_CMAP_RANGE, 2419 },
	{ 0xc4f2, 0xc4f3, PDF_CMAP_RANGE, 14384 },
	{ 0xc4f4, 0xc4f4, PDF_CMAP_SINGLE, 2421 },
	{ 0xc4f5, 0xc4f7, PDF_CMAP_RANGE, 14386 },
	{ 0xc4f8, 0xc4fa, PDF_CMAP_TABLE, 2894 },
	{ 0xc4fb, 0xc4fe, PDF_CMAP_RANGE, 14390 },
	{ 0xc4ff, 0xc501, PDF_CMAP_RANGE, 2424 },
	{ 0xc502, 0xc50b, PDF_CMAP_RANGE, 14394 },
	{ 0xc50c, 0xc50c, PDF_CMAP_SINGLE, 2427 },
	{ 0xc50d, 0xc50f, PDF_CMAP_RANGE, 14404 },
	{ 0xc510, 0xc510, PDF_CMAP_SINGLE, 2428 },
	{ 0xc511, 0xc513, PDF_CMAP_RANGE, 14407 },
	{ 0xc514, 0xc514, PDF_CMAP_SINGLE, 2429 },
	{ 0xc515, 0xc51b, PDF_CMAP_RANGE, 14410 },
	{ 0xc51c, 0xc51c, PDF_CMAP_SINGLE, 2430 },
	{ 0xc51d, 0xc527, PDF_CMAP_RANGE, 14417 },
	{ 0xc528, 0xc529, PDF_CMAP_RANGE, 2431 },
	{ 0xc52a, 0xc52b, PDF_CMAP_RANGE, 14428 },
	{ 0xc52c, 0xc52c, PDF_CMAP_SINGLE, 2433 },
	{ 0xc52d, 0xc52f, PDF_CMAP_RANGE, 14430 },
	{ 0xc530, 0xc530, PDF_CMAP_SINGLE, 2434 },
	{ 0xc531, 0xc537, PDF_CMAP_RANGE, 14433 },
	{ 0xc538, 0xc539, PDF_CMAP_RANGE, 2435 },
	{ 0xc53a, 0xc53d, PDF_CMAP_TABLE, 2897 },
	{ 0xc53e, 0xc543, PDF_CMAP_RANGE, 14442 },
	{ 0xc544, 0xc545, PDF_CMAP_RANGE, 2439 },
	{ 0xc546, 0xc547, PDF_CMAP_RANGE, 14448 },
	{ 0xc548, 0xc54a, PDF_CMAP_RANGE, 2441 },
	{ 0xc54b, 0xc54b, PDF_CMAP_SINGLE, 14450 },
	{ 0xc54c, 0xc54e, PDF_CMAP_RANGE, 2444 },
	{ 0xc54f, 0xc552, PDF_CMAP_RANGE, 14451 },
	{ 0xc553, 0xc555, PDF_CMAP_RANGE, 2447 },
	{ 0xc556, 0xc556, PDF_CMAP_SINGLE, 14455 },
	{ 0xc557, 0xc559, PDF_CMAP_RANGE, 2450 },
	{ 0xc55a, 0xc55c, PDF_CMAP_RANGE, 14456 },
	{ 0xc55d, 0xc55e, PDF_CMAP_RANGE, 2453 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 14459 },
	{ 0xc560, 0xc561, PDF_CMAP_RANGE, 2455 },
	{ 0xc562, 0xc563, PDF_CMAP_RANGE, 14460 },
	{ 0xc564, 0xc564, PDF_CMAP_SINGLE, 2457 },
	{ 0xc565, 0xc567, PDF_CMAP_RANGE, 14462 },
	{ 0xc568, 0xc568, PDF_CMAP_SINGLE, 2458 },
	{ 0xc569, 0xc56f, PDF_CMAP_RANGE, 14465 },
	{ 0xc570, 0xc571, PDF_CMAP_RANGE, 2459 },
	{ 0xc572, 0xc572, PDF_CMAP_SINGLE, 14472 },
	{ 0xc573, 0xc575, PDF_CMAP_RANGE, 2461 },
	{ 0xc576, 0xc57b, PDF_CMAP_RANGE, 14473 },
	{ 0xc57c, 0xc57d, PDF_CMAP_RANGE, 2464 },
	{ 0xc57e, 0xc57f, PDF_CMAP_RANGE, 14479 },
	{ 0xc580, 0xc580, PDF_CMAP_SINGLE, 2466 },
	{ 0xc581, 0xc583, PDF_CMAP_RANGE, 14481 },
	{ 0xc584, 0xc584, PDF_CMAP_SINGLE, 2467 },
	{ 0xc585, 0xc586, PDF_CMAP_RANGE, 14484 },
	{ 0xc587, 0xc587, PDF_CMAP_SINGLE, 2468 },
	{ 0xc588, 0xc58b, PDF_CMAP_RANGE, 14486 },
	{ 0xc58c, 0xc58d, PDF_CMAP_RANGE, 2469 },
	{ 0xc58e, 0xc591, PDF_CMAP_TABLE, 2901 },
	{ 0xc592, 0xc594, PDF_CMAP_RANGE, 14492 },
	{ 0xc595, 0xc596, PDF_CMAP_TABLE, 2905 },
	{ 0xc597, 0xc598, PDF_CMAP_RANGE, 2474 },
	{ 0xc599, 0xc59b, PDF_CMAP_RANGE, 14496 },
	{ 0xc59c, 0xc59c, PDF_CMAP_SINGLE, 2476 },
	{ 0xc59d, 0xc59f, PDF_CMAP_RANGE, 14499 },
	{ 0xc5a0, 0xc5a0, PDF_CMAP_SINGLE, 2477 },
	{ 0xc5a1, 0xc5a8, PDF_CMAP_RANGE, 14502 },
	{ 0xc5a9, 0xc5a9, PDF_CMAP_SINGLE, 2478 },
	{ 0xc5aa, 0xc5b3, PDF_CMAP_RANGE, 14510 },
	{ 0xc5b4, 0xc5b5, PDF_CMAP_RANGE, 2479 },
	{ 0xc5b6, 0xc5b7, PDF_CMAP_RANGE, 14520 },
	{ 0xc5b8, 0xc5b9, PDF_CMAP_RANGE, 2481 },
	{ 0xc5ba, 0xc5ba, PDF_CMAP_SINGLE, 14522 },
	{ 0xc5bb, 0xc5be, PDF_CMAP_RANGE, 2483 },
	{ 0xc5bf, 0xc5c3, PDF_CMAP_RANGE, 14523 },
	{ 0xc5c4, 0xc5ca, PDF_CMAP_RANGE, 2487 },
	{ 0xc5cb, 0xc5cf, PDF_CMAP_TABLE, 2907 },
	{ 0xc5d0, 0xc5d1, PDF_CMAP_RANGE, 2496 },
	{ 0xc5d2, 0xc5d3, PDF_CMAP_RANGE, 14531 },
	{ 0xc5d4, 0xc5d4, PDF_CMAP_SINGLE, 2498 },
	{ 0xc5d5, 0xc5d7, PDF_CMAP_RANGE, 14533 },
	{ 0xc5d8, 0xc5d8, PDF_CMAP_SINGLE, 2499 },
	{ 0xc5d9, 0xc5df, PDF_CMAP_RANGE, 14536 },
	{ 0xc5e0, 0xc5e1, PDF_CMAP_RANGE, 2500 },
	{ 0xc5e2, 0xc5e5, PDF_CMAP_TABLE, 2912 },
	{ 0xc5e6, 0xc5eb, PDF_CMAP_RANGE, 14545 },
	{ 0xc5ec, 0xc5ee, PDF_CMAP_RANGE, 2504 },
	{ 0xc5ef, 0xc5f0, PDF_CMAP_TABLE, 2916 },
	{ 0xc5f1, 0xc5f3, PDF_CMAP_RANGE, 14552 },
	{ 0xc5f4, 0xc5f5, PDF_CMAP_TABLE, 2918 },
	{ 0xc5f6, 0xc5f7, PDF_CMAP_RANGE, 2509 },
	{ 0xc5f8, 0xc5fb, PDF_CMAP_RANGE, 14556 },
	{ 0xc5fc, 0xc601, PDF_CMAP_RANGE, 2511 },
	{ 0xc602, 0xc604, PDF_CMAP_RANGE, 14560 },
	{ 0xc605, 0xc608, PDF_CMAP_RANGE, 2517 },
	{ 0xc609, 0xc60b, PDF_CMAP_RANGE, 14563 },
	{ 0xc60c, 0xc60c, PDF_CMAP_SINGLE, 2521 },
	{ 0xc60d, 0xc60f, PDF_CMAP_RANGE, 14566 },
	{ 0xc610, 0xc610, PDF_CMAP_SINGLE, 2522 },
	{ 0xc611, 0xc617, PDF_CMAP_RANGE, 14569 },
	{ 0xc618, 0xc619, PDF_CMAP_RANGE, 2523 },
	{ 0xc61a, 0xc61a, PDF_CMAP_SINGLE, 14576 },
	{ 0xc61b, 0xc61c, PDF_CMAP_RANGE, 2525 },
	{ 0xc61d, 0xc623, PDF_CMAP_RANGE, 14577 },
	{ 0xc624, 0xc625, PDF_CMAP_RANGE, 2527 },
	{ 0xc626, 0xc627, PDF_CMAP_RANGE, 14584 },
	{ 0xc628, 0xc628, PDF_CMAP_SINGLE, 2529 },
	{ 0xc629, 0xc62b, PDF_CMAP_RANGE, 14586 },
	{ 0xc62c, 0xc62e, PDF_CMAP_RANGE, 2530 },
	{ 0xc62f, 0xc630, PDF_CMAP_TABLE, 2920 },
	{ 0xc631, 0xc632, PDF_CMAP_RANGE, 14590 },
	{ 0xc633, 0xc635, PDF_CMAP_RANGE, 2534 },
	{ 0xc636, 0xc63b, PDF_CMAP_TABLE, 2922 },
	{ 0xc63c, 0xc63f, PDF_CMAP_RANGE, 14595 },
	{ 0xc640, 0xc641, PDF_CMAP_RANGE, 2540 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 14599 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 2542 },
	{ 0xc645, 0xc647, PDF_CMAP_RANGE, 14601 },
	{ 0xc648, 0xc648, PDF_CMAP_SINGLE, 2543 },
	{ 0xc649, 0xc64f, PDF_CMAP_RANGE, 14604 },
	{ 0xc650, 0xc651, PDF_CMAP_RANGE, 2544 },
	{ 0xc652, 0xc652, PDF_CMAP_SINGLE, 14611 },
	{ 0xc653, 0xc655, PDF_CMAP_RANGE, 2546 },
	{ 0xc656, 0xc65b, PDF_CMAP_RANGE, 14612 },
	{ 0xc65c, 0xc65d, PDF_CMAP_RANGE, 2549 },
	{ 0xc65e, 0xc65f, PDF_CMAP_RANGE, 14618 },
	{ 0xc660, 0xc660, PDF_CMAP_SINGLE, 2551 },
	{ 0xc661, 0xc66b, PDF_CMAP_RANGE, 14620 },
	{ 0xc66c, 0xc66c, PDF_CMAP_SINGLE, 2552 },
	{ 0xc66d, 0xc66e, PDF_CMAP_RANGE, 14631 },
	{ 0xc66f, 0xc671, PDF_CMAP_TABLE, 2928 },
	{ 0xc672, 0xc677, PDF_CMAP_RANGE, 14634 },
	{ 0xc678, 0xc679, PDF_CMAP_RANGE, 2555 },
	{ 0xc67a, 0xc67b, PDF_CMAP_RANGE, 14640 },
	{ 0xc67c, 0xc67c, PDF_CMAP_SINGLE, 2557 },
	{ 0xc67d, 0xc67f, PDF_CMAP_RANGE, 14642 },
	{ 0xc680, 0xc680, PDF_CMAP_SINGLE, 2558 },
	{ 0xc681, 0xc687, PDF_CMAP_RANGE, 14645 },
	{ 0xc688, 0xc689, PDF_CMAP_RANGE, 2559 },
	{ 0xc68a, 0xc68d, PDF_CMAP_TABLE, 2931 },
	{ 0xc68e, 0xc693, PDF_CMAP_RANGE, 14654 },
	{ 0xc694, 0xc695, PDF_CMAP_RANGE, 2563 },
	{ 0xc696, 0xc697, PDF_CMAP_RANGE, 14660 },
	{ 0xc698, 0xc698, PDF_CMAP_SINGLE, 2565 },
	{ 0xc699, 0xc69b, PDF_CMAP_RANGE, 14662 },
	{ 0xc69c, 0xc69c, PDF_CMAP_SINGLE, 2566 },
	{ 0xc69d, 0xc6a3, PDF_CMAP_RANGE, 14665 },
	{ 0xc6a4, 0xc6a5, PDF_CMAP_RANGE, 2567 },
	{ 0xc6a6, 0xc6a9, PDF_CMAP_TABLE, 2935 },
	{ 0xc6aa, 0xc6af, PDF_CMAP_RANGE, 14674 },
	{ 0xc6b0, 0xc6b1, PDF_CMAP_RANGE, 2571 },
	{ 0xc6b2, 0xc6b3, PDF_CMAP_RANGE, 14680 },
	{ 0xc6b4, 0xc6b4, PDF_CMAP_SINGLE, 2573 },
	{ 0xc6b5, 0xc6b7, PDF_CMAP_RANGE, 14682 },
	{ 0xc6b8, 0xc6ba, PDF_CMAP_RANGE, 2574 },
	{ 0xc6bb, 0xc6bf, PDF_CMAP_RANGE, 14685 },
	{ 0xc6c0, 0xc6c1, PDF_CMAP_RANGE, 2577 },
	{ 0xc6c2, 0xc6c5, PDF_CMAP_TABLE, 2939 },
	{ 0xc6c6, 0xc6cb, PDF_CMAP_RANGE, 14692 },
	{ 0xc6cc, 0xc6cd, PDF_CMAP_RANGE, 2581 },
	{ 0xc6ce, 0xc6cf, PDF_CMAP_RANGE, 14698 },
	{ 0xc6d0, 0xc6d0, PDF_CMAP_SINGLE, 2583 },
	{ 0xc6d1, 0xc6d3, PDF_CMAP_RANGE, 14700 },
	{ 0xc6d4, 0xc6d4, PDF_CMAP_SINGLE, 2584 },
	{ 0xc6d5, 0xc6db, PDF_CMAP_RANGE, 14703 },
	{ 0xc6dc, 0xc6dd, PDF_CMAP_RANGE, 2585 },
	{ 0xc6de, 0xc6df, PDF_CMAP_RANGE, 14710 },
	{ 0xc6e0, 0xc6e1, PDF_CMAP_RANGE, 2587 },
	{ 0xc6e2, 0xc6e7, PDF_CMAP_RANGE, 14712 },
	{ 0xc6e8, 0xc6e9, PDF_CMAP_RANGE, 2589 },
	{ 0xc6ea, 0xc6eb, PDF_CMAP_RANGE, 14718 },
	{ 0xc6ec, 0xc6ec, PDF_CMAP_SINGLE, 2591 },
	{ 0xc6ed, 0xc6ef, PDF_CMAP_RANGE, 14720 },
	{ 0xc6f0, 0xc6f0, PDF_CMAP_SINGLE, 2592 },
	{ 0xc6f1, 0xc6f7, PDF_CMAP_RANGE, 14723 },
	{ 0xc6f8, 0xc6f9, PDF_CMAP_RANGE, 2593 },
	{ 0xc6fa, 0xc6fc, PDF_CMAP_RANGE, 14730 },
	{ 0xc6fd, 0xc6fd, PDF_CMAP_SINGLE, 2595 },
	{ 0xc6fe, 0xc703, PDF_CMAP_RANGE, 14733 },
	{ 0xc704, 0xc705, PDF_CMAP_RANGE, 2596 },
	{ 0xc706, 0xc707, PDF_CMAP_RANGE, 14739 },
	{ 0xc708, 0xc708, PDF_CMAP_SINGLE, 2598 },
	{ 0xc709, 0xc70b, PDF_CMAP_RANGE, 14741 },
	{ 0xc70c, 0xc70c, PDF_CMAP_SINGLE, 2599 },
	{ 0xc70d, 0xc713, PDF_CMAP_RANGE, 14744 },
	{ 0xc714, 0xc715, PDF_CMAP_RANGE, 2600 },
	{ 0xc716, 0xc719, PDF_CMAP_TABLE, 2943 },
	{ 0xc71a, 0xc71f, PDF_CMAP_RANGE, 14753 },
	{ 0xc720, 0xc721, PDF_CMAP_RANGE, 2604 },
	{ 0xc722, 0xc723, PDF_CMAP_RANGE, 14759 },
	{ 0xc724, 0xc724, PDF_CMAP_SINGLE, 2606 },
	{ 0xc725, 0xc727, PDF_CMAP_RANGE, 14761 },
	{ 0xc728, 0xc728, PDF_CMAP_SINGLE, 2607 },
	{ 0xc729, 0xc72f, PDF_CMAP_RANGE, 14764 },
	{ 0xc730, 0xc731, PDF_CMAP_RANGE, 2608 },
	{ 0xc732, 0xc737, PDF_CMAP_TABLE, 2947 },
	{ 0xc738, 0xc73b, PDF_CMAP_RANGE, 14774 },
	{ 0xc73c, 0xc73d, PDF_CMAP_RANGE, 2613 },
	{ 0xc73e, 0xc73f, PDF_CMAP_RANGE, 14778 },
	{ 0xc740, 0xc740, PDF_CMAP_SINGLE, 2615 },
	{ 0xc741, 0xc743, PDF_CMAP_RANGE, 14780 },
	{ 0xc744, 0xc744, PDF_CMAP_SINGLE, 2616 },
	{ 0xc745, 0xc749, PDF_CMAP_RANGE, 14783 },
	{ 0xc74a, 0xc74b, PDF_CMAP_TABLE, 2953 },
	{ 0xc74c, 0xc74d, PDF_CMAP_RANGE, 2618 },
	{ 0xc74e, 0xc750, PDF_CMAP_TABLE, 2955 },
	{ 0xc751, 0xc758, PDF_CMAP_RANGE, 2621 },
	{ 0xc759, 0xc75b, PDF_CMAP_RANGE, 14791 },
	{ 0xc75c, 0xc75c, PDF_CMAP_SINGLE, 2629 },
	{ 0xc75d, 0xc75f, PDF_CMAP_RANGE, 14794 },
	{ 0xc760, 0xc760, PDF_CMAP_SINGLE, 2630 },
	{ 0xc761, 0xc767, PDF_CMAP_RANGE, 14797 },
	{ 0xc768, 0xc768, PDF_CMAP_SINGLE, 2631 },
	{ 0xc769, 0xc76a, PDF_CMAP_RANGE, 14804 },
	{ 0xc76b, 0xc76b, PDF_CMAP_SINGLE, 2632 },
	{ 0xc76c, 0xc773, PDF_CMAP_RANGE, 14806 },
	{ 0xc774, 0xc775, PDF_CMAP_RANGE, 2633 },
	{ 0xc776, 0xc777, PDF_CMAP_RANGE, 14814 },
	{ 0xc778, 0xc778, PDF_CMAP_SINGLE, 2635 },
	{ 0xc779, 0xc77b, PDF_CMAP_RANGE, 14816 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 2636 },
	{ 0xc77f, 0xc782, PDF_CMAP_RANGE, 14819 },
	{ 0xc783, 0xc785, PDF_CMAP_RANGE, 2639 },
	{ 0xc786, 0xc786, PDF_CMAP_SINGLE, 14823 },
	{ 0xc787, 0xc78a, PDF_CMAP_RANGE, 2642 },
	{ 0xc78b, 0xc78d, PDF_CMAP_RANGE, 14824 },
	{ 0xc78e, 0xc78f, PDF_CMAP_TABLE, 2958 },
	{ 0xc790, 0xc791, PDF_CMAP_RANGE, 2647 },
	{ 0xc792, 0xc793, PDF_CMAP_RANGE, 14828 },
	{ 0xc794, 0xc795, PDF_CMAP_TABLE, 2960 },
	{ 0xc796, 0xc798, PDF_CMAP_RANGE, 2650 },
	{ 0xc799, 0xc79a, PDF_CMAP_TABLE, 2962 },
	{ 0xc79b, 0xc79f, PDF_CMAP_RANGE, 14832 },
	{ 0xc7a0, 0xc7a1, PDF_CMAP_RANGE, 2654 },
	{ 0xc7a2, 0xc7a2, PDF_CMAP_SINGLE, 14837 },
	{ 0xc7a3, 0xc7a6, PDF_CMAP_RANGE, 2656 },
	{ 0xc7a7, 0xc7ab, PDF_CMAP_RANGE, 14838 },
	{ 0xc7ac, 0xc7ad, PDF_CMAP_RANGE, 2660 },
	{ 0xc7ae, 0xc7af, PDF_CMAP_RANGE, 14843 },
	{ 0xc7b0, 0xc7b0, PDF_CMAP_SINGLE, 2662 },
	{ 0xc7b1, 0xc7b3, PDF_CMAP_RANGE, 14845 },
	{ 0xc7b4, 0xc7b4, PDF_CMAP_SINGLE, 2663 },
	{ 0xc7b5, 0xc7bb, PDF_CMAP_RANGE, 14848 },
	{ 0xc7bc, 0xc7bd, PDF_CMAP_RANGE, 2664 },
	{ 0xc7be, 0xc7be, PDF_CMAP_SINGLE, 14855 },
	{ 0xc7bf, 0xc7c1, PDF_CMAP_RANGE, 2666 },
	{ 0xc7c2, 0xc7c7, PDF_CMAP_RANGE, 14856 },
	{ 0xc7c8, 0xc7c9, PDF_CMAP_RANGE, 2669 },
	{ 0xc7ca, 0xc7cb, PDF_CMAP_RANGE, 14862 },
	{ 0xc7cc, 0xc7d0, PDF_CMAP_TABLE, 2964 },
	{ 0xc7d1, 0xc7d7, PDF_CMAP_RANGE, 14866 },
	{ 0xc7d8, 0xc7d8, PDF_CMAP_SINGLE, 2674 },
	{ 0xc7d9, 0xc7dc, PDF_CMAP_RANGE, 14873 },
	{ 0xc7dd, 0xc7dd, PDF_CMAP_SINGLE, 2675 },
	{ 0xc7de, 0xc7e3, PDF_CMAP_RANGE, 14877 },
	{ 0xc7e4, 0xc7e4, PDF_CMAP_SINGLE, 2676 },
	{ 0xc7e5, 0xc7e7, PDF_CMAP_RANGE, 14883 },
	{ 0xc7e8, 0xc7e8, PDF_CMAP_SINGLE, 2677 },
	{ 0xc7e9, 0xc7eb, PDF_CMAP_RANGE, 14886 },
	{ 0xc7ec, 0xc7ec, PDF_CMAP_SINGLE, 2678 },
	{ 0xc7ed, 0xc7ff, PDF_CMAP_RANGE, 14889 },
	{ 0xc800, 0xc801, PDF_CMAP_RANGE, 2679 },
	{ 0xc802, 0xc803, PDF_CMAP_RANGE, 14908 },
	{ 0xc804, 0xc804, PDF_CMAP_SINGLE, 2681 },
	{ 0xc805, 0xc807, PDF_CMAP_RANGE, 14910 },
	{ 0xc808, 0xc80a, PDF_CMAP_TABLE, 2969 },
	{ 0xc80b, 0xc80f, PDF_CMAP_RANGE, 14914 },
	{ 0xc810, 0xc811, PDF_CMAP_RANGE, 2684 },
	{ 0xc812, 0xc814, PDF_CMAP_TABLE, 2972 },
	{ 0xc815, 0xc816, PDF_CMAP_RANGE, 2687 },
	{ 0xc817, 0xc81b, PDF_CMAP_RANGE, 14921 },
	{ 0xc81c, 0xc81d, PDF_CMAP_RANGE, 2689 },
	{ 0xc81e, 0xc81f, PDF_CMAP_RANGE, 14926 },
	{ 0xc820, 0xc820, PDF_CMAP_SINGLE, 2691 },
	{ 0xc821, 0xc823, PDF_CMAP_RANGE, 14928 },
	{ 0xc824, 0xc824, PDF_CMAP_SINGLE, 2692 },
	{ 0xc825, 0xc82b, PDF_CMAP_RANGE, 14931 },
	{ 0xc82c, 0xc82d, PDF_CMAP_RANGE, 2693 },
	{ 0xc82e, 0xc831, PDF_CMAP_TABLE, 2975 },
	{ 0xc832, 0xc837, PDF_CMAP_RANGE, 14940 },
	{ 0xc838, 0xc838, PDF_CMAP_SINGLE, 2697 },
	{ 0xc839, 0xc83b, PDF_CMAP_RANGE, 14946 },
	{ 0xc83c, 0xc83c, PDF_CMAP_SINGLE, 2698 },
	{ 0xc83d, 0xc83f, PDF_CMAP_RANGE, 14949 },
	{ 0xc840, 0xc840, PDF_CMAP_SINGLE, 2699 },
	{ 0xc841, 0xc847, PDF_CMAP_RANGE, 14952 },
	{ 0xc848, 0xc849, PDF_CMAP_RANGE, 2700 },
	{ 0xc84a, 0xc84b, PDF_CMAP_RANGE, 14959 },
	{ 0xc84c, 0xc84d, PDF_CMAP_RANGE, 2702 },
	{ 0xc84e, 0xc853, PDF_CMAP_RANGE, 14961 },
	{ 0xc854, 0xc854, PDF_CMAP_SINGLE, 2704 },
	{ 0xc855, 0xc86f, PDF_CMAP_RANGE, 14967 },
	{ 0xc870, 0xc871, PDF_CMAP_RANGE, 2705 },
	{ 0xc872, 0xc873, PDF_CMAP_RANGE, 14994 },
	{ 0xc874, 0xc874, PDF_CMAP_SINGLE, 2707 },
	{ 0xc875, 0xc877, PDF_CMAP_RANGE, 14996 },
	{ 0xc878, 0xc87a, PDF_CMAP_TABLE, 2979 },
	{ 0xc87b, 0xc87f, PDF_CMAP_RANGE, 15000 },
	{ 0xc880, 0xc881, PDF_CMAP_RANGE, 2710 },
	{ 0xc882, 0xc884, PDF_CMAP_TABLE, 2982 },
	{ 0xc885, 0xc887, PDF_CMAP_RANGE, 2713 },
	{ 0xc888, 0xc88a, PDF_CMAP_RANGE, 15007 },
	{ 0xc88b, 0xc88d, PDF_CMAP_RANGE, 2716 },
	{ 0xc88e, 0xc893, PDF_CMAP_RANGE, 15010 },
	{ 0xc894, 0xc894, PDF_CMAP_SINGLE, 2719 },
	{ 0xc895, 0xc89c, PDF_CMAP_RANGE, 15016 },
	{ 0xc89d, 0xc8a1, PDF_CMAP_TABLE, 2985 },
	{ 0xc8a2, 0xc8a7, PDF_CMAP_RANGE, 15026 },
	{ 0xc8a8, 0xc8a8, PDF_CMAP_SINGLE, 2723 },
	{ 0xc8a9, 0xc8bb, PDF_CMAP_RANGE, 15032 },
	{ 0xc8bc, 0xc8bd, PDF_CMAP_RANGE, 2724 },
	{ 0xc8be, 0xc8c3, PDF_CMAP_RANGE, 15051 },
	{ 0xc8c4, 0xc8c4, PDF_CMAP_SINGLE, 2726 },
	{ 0xc8c5, 0xc8c7, PDF_CMAP_RANGE, 15057 },
	{ 0xc8c8, 0xc8c8, PDF_CMAP_SINGLE, 2727 },
	{ 0xc8c9, 0xc8cb, PDF_CMAP_RANGE, 15060 },
	{ 0xc8cc, 0xc8cc, PDF_CMAP_SINGLE, 2728 },
	{ 0xc8cd, 0xc8d3, PDF_CMAP_RANGE, 15063 },
	{ 0xc8d4, 0xc8d5, PDF_CMAP_RANGE, 2729 },
	{ 0xc8d6, 0xc8d9, PDF_CMAP_TABLE, 2990 },
	{ 0xc8da, 0xc8df, PDF_CMAP_RANGE, 15072 },
	{ 0xc8e0, 0xc8e1, PDF_CMAP_RANGE, 2733 },
	{ 0xc8e2, 0xc8e3, PDF_CMAP_RANGE, 15078 },
	{ 0xc8e4, 0xc8e4, PDF_CMAP_SINGLE, 2735 },
	{ 0xc8e5, 0xc8f4, PDF_CMAP_RANGE, 15080 },
	{ 0xc8f5, 0xc8f5, PDF_CMAP_SINGLE, 2736 },
	{ 0xc8f6, 0xc8fb, PDF_CMAP_RANGE, 15096 },
	{ 0xc8fc, 0xc8fd, PDF_CMAP_RANGE, 2737 },
	{ 0xc8fe, 0xc8ff, PDF_CMAP_RANGE, 15102 },
	{ 0xc900, 0xc900, PDF_CMAP_SINGLE, 2739 },
	{ 0xc901, 0xc903, PDF_CMAP_RANGE, 15104 },
	{ 0xc904, 0xc906, PDF_CMAP_RANGE, 2740 },
	{ 0xc907, 0xc90b, PDF_CMAP_RANGE, 15107 },
	{ 0xc90c, 0xc90d, PDF_CMAP_RANGE, 2743 },
	{ 0xc90e, 0xc911, PDF_CMAP_TABLE, 2994 },
	{ 0xc912, 0xc917, PDF_CMAP_RANGE, 15114 },
	{ 0xc918, 0xc918, PDF_CMAP_SINGLE, 2747 },
	{ 0xc919, 0xc92b, PDF_CMAP_RANGE, 15120 },
	{ 0xc92c, 0xc92c, PDF_CMAP_SINGLE, 2748 },
	{ 0xc92d, 0xc933, PDF_CMAP_RANGE, 15139 },
	{ 0xc934, 0xc934, PDF_CMAP_SINGLE, 2749 },
	{ 0xc935, 0xc94f, PDF_CMAP_RANGE, 15146 },
	{ 0xc950, 0xc951, PDF_CMAP_RANGE, 2750 },
	{ 0xc952, 0xc953, PDF_CMAP_RANGE, 15173 },
	{ 0xc954, 0xc954, PDF_CMAP_SINGLE, 2752 },
	{ 0xc955, 0xc957, PDF_CMAP_RANGE, 15175 },
	{ 0xc958, 0xc958, PDF_CMAP_SINGLE, 2753 },
	{ 0xc959, 0xc95f, PDF_CMAP_RANGE, 15178 },
	{ 0xc960, 0xc961, PDF_CMAP_RANGE, 2754 },
	{ 0xc962, 0xc963, PDF_CMAP_TABLE, 2998 },
	{ 0xc964, 0xc96b, PDF_CMAP_RANGE, 15186 },
	{ 0xc96c, 0xc96c, PDF_CMAP_SINGLE, 2757 },
	{ 0xc96d, 0xc96f, PDF_CMAP_RANGE, 15194 },
	{ 0xc970, 0xc970, PDF_CMAP_SINGLE, 2758 },
	{ 0xc971, 0xc973, PDF_CMAP_RANGE, 15197 },
	{ 0xc974, 0xc974, PDF_CMAP_SINGLE, 2759 },
	{ 0xc975, 0xc97b, PDF_CMAP_RANGE, 15200 },
	{ 0xc97c, 0xc97c, PDF_CMAP_SINGLE, 2760 },
	{ 0xc97d, 0xc987, PDF_CMAP_RANGE, 15207 },
	{ 0xc988, 0xc989, PDF_CMAP_RANGE, 2761 },
	{ 0xc98a, 0xc98b, PDF_CMAP_RANGE, 15218 },
	{ 0xc98c, 0xc98c, PDF_CMAP_SINGLE, 2763 },
	{ 0xc98d, 0xc98f, PDF_CMAP_RANGE, 15220 },
	{ 0xc990, 0xc990, PDF_CMAP_SINGLE, 2764 },
	{ 0xc991, 0xc997, PDF_CMAP_RANGE, 15223 },
	{ 0xc998, 0xc999, PDF_CMAP_RANGE, 2765 },
	{ 0xc99a, 0xc99d, PDF_CMAP_TABLE, 3000 },
	{ 0xc99e, 0xc9bf, PDF_CMAP_RANGE, 15232 },
	{ 0xc9c0, 0xc9c1, PDF_CMAP_RANGE, 2769 },
	{ 0xc9c2, 0xc9c3, PDF_CMAP_RANGE, 15266 },
	{ 0xc9c4, 0xc9c4, PDF_CMAP_SINGLE, 2771 },
	{ 0xc9c5, 0xc9c6, PDF_CMAP_RANGE, 15268 },
	{ 0xc9c7, 0xc9c8, PDF_CMAP_RANGE, 2772 },
	{ 0xc9c9, 0xc9ca, PDF_CMAP_TABLE, 3004 },
	{ 0xc9cb, 0xc9cf, PDF_CMAP_RANGE, 15271 },
	{ 0xc9d0, 0xc9d1, PDF_CMAP_RANGE, 2775 },
	{ 0xc9d2, 0xc9d4, PDF_CMAP_TABLE, 3006 },
	{ 0xc9d5, 0xc9d6, PDF_CMAP_RANGE, 2778 },
	{ 0xc9d7, 0xc9d8, PDF_CMAP_RANGE, 15278 },
	{ 0xc9d9, 0xc9da, PDF_CMAP_RANGE, 2780 },
	{ 0xc9db, 0xc9db, PDF_CMAP_SINGLE, 15280 },
	{ 0xc9dc, 0xc9dd, PDF_CMAP_RANGE, 2782 },
	{ 0xc9de, 0xc9df, PDF_CMAP_RANGE, 15281 },
	{ 0xc9e0, 0xc9e4, PDF_CMAP_TABLE, 3009 },
	{ 0xc9e5, 0xc9e6, PDF_CMAP_RANGE, 15285 },
	{ 0xc9e7, 0xc9e7, PDF_CMAP_SINGLE, 2787 },
	{ 0xc9e8, 0xc9eb, PDF_CMAP_RANGE, 15287 },
	{ 0xc9ec, 0xc9ed, PDF_CMAP_RANGE, 2788 },
	{ 0xc9ee, 0xc9ee, PDF_CMAP_SINGLE, 15291 },
	{ 0xc9ef, 0xc9f1, PDF_CMAP_RANGE, 2790 },
	{ 0xc9f2, 0xc9f7, PDF_CMAP_RANGE, 15292 },
	{ 0xc9f8, 0xc9f9, PDF_CMAP_RANGE, 2793 },
	{ 0xc9fa, 0xc9fb, PDF_CMAP_RANGE, 15298 },
	{ 0xc9fc, 0xc9fc, PDF_CMAP_SINGLE, 2795 },
	{ 0xc9fd, 0xc9ff, PDF_CMAP_RANGE, 15300 },
	{ 0xca00, 0xca00, PDF_CMAP_SINGLE, 2796 },
	{ 0xca01, 0xca07, PDF_CMAP_RANGE, 15303 },
	{ 0xca08, 0xca09, PDF_CMAP_RANGE, 2797 },
	{ 0xca0a, 0xca0a, PDF_CMAP_SINGLE, 15310 },
	{ 0xca0b, 0xca0d, PDF_CMAP_RANGE, 2799 },
	{ 0xca0e, 0xca13, PDF_CMAP_RANGE, 15311 },
	{ 0xca14, 0xca14, PDF_CMAP_SINGLE, 2802 },
	{ 0xca15, 0xca17, PDF_CMAP_RANGE, 15317 },
	{ 0xca18, 0xca18, PDF_CMAP_SINGLE, 2803 },
	{ 0xca19, 0xca28, PDF_CMAP_RANGE, 15320 },
	{ 0xca29, 0xca29, PDF_CMAP_SINGLE, 2804 },
	{ 0xca2a, 0xca4b, PDF_CMAP_RANGE, 15336 },
	{ 0xca4c, 0xca4d, PDF_CMAP_RANGE, 2805 },
	{ 0xca4e, 0xca4f, PDF_CMAP_RANGE, 15370 },
	{ 0xca50, 0xca50, PDF_CMAP_SINGLE, 2807 },
	{ 0xca51, 0xca53, PDF_CMAP_RANGE, 15372 },
	{ 0xca54, 0xca54, PDF_CMAP_SINGLE, 2808 },
	{ 0xca55, 0xca5b, PDF_CMAP_RANGE, 15375 },
	{ 0xca5c, 0xca5d, PDF_CMAP_RANGE, 2809 },
	{ 0xca5e, 0xca5e, PDF_CMAP_SINGLE, 15382 },
	{ 0xca5f, 0xca61, PDF_CMAP_RANGE, 2811 },
	{ 0xca62, 0xca67, PDF_CMAP_RANGE, 15383 },
	{ 0xca68, 0xca68, PDF_CMAP_SINGLE, 2814 },
	{ 0xca69, 0xca7c, PDF_CMAP_RANGE, 15389 },
	{ 0xca7d, 0xca7d, PDF_CMAP_SINGLE, 2815 },
	{ 0xca7e, 0xca83, PDF_CMAP_RANGE, 15409 },
	{ 0xca84, 0xca84, PDF_CMAP_SINGLE, 2816 },
	{ 0xca85, 0xca97, PDF_CMAP_RANGE, 15415 },
	{ 0xca98, 0xca98, PDF_CMAP_SINGLE, 2817 },
	{ 0xca99, 0xcabb, PDF_CMAP_RANGE, 15434 },
	{ 0xcabc, 0xcabd, PDF_CMAP_RANGE, 2818 },
	{ 0xcabe, 0xcabf, PDF_CMAP_RANGE, 15469 },
	{ 0xcac0, 0xcac0, PDF_CMAP_SINGLE, 2820 },
	{ 0xcac1, 0xcac3, PDF_CMAP_RANGE, 15471 },
	{ 0xcac4, 0xcac4, PDF_CMAP_SINGLE, 2821 },
	{ 0xcac5, 0xcacb, PDF_CMAP_RANGE, 15474 },
	{ 0xcacc, 0xcacd, PDF_CMAP_RANGE, 2822 },
	{ 0xcace, 0xcad3, PDF_CMAP_TABLE, 3014 },
	{ 0xcad4, 0xcad7, PDF_CMAP_RANGE, 15484 },
	{ 0xcad8, 0xcad9, PDF_CMAP_RANGE, 2827 },
	{ 0xcada, 0xcadf, PDF_CMAP_RANGE, 15488 },
	{ 0xcae0, 0xcae0, PDF_CMAP_SINGLE, 2829 },
	{ 0xcae1, 0xcaeb, PDF_CMAP_RANGE, 15494 },
	{ 0xcaec, 0xcaec, PDF_CMAP_SINGLE, 2830 },
	{ 0xcaed, 0xcaf3, PDF_CMAP_RANGE, 15505 },
	{ 0xcaf4, 0xcaf4, PDF_CMAP_SINGLE, 2831 },
	{ 0xcaf5, 0xcb07, PDF_CMAP_RANGE, 15512 },
	{ 0xcb08, 0xcb08, PDF_CMAP_SINGLE, 2832 },
	{ 0xcb09, 0xcb0f, PDF_CMAP_RANGE, 15531 },
	{ 0xcb10, 0xcb10, PDF_CMAP_SINGLE, 2833 },
	{ 0xcb11, 0xcb13, PDF_CMAP_RANGE, 15538 },
	{ 0xcb14, 0xcb14, PDF_CMAP_SINGLE, 2834 },
	{ 0xcb15, 0xcb17, PDF_CMAP_RANGE, 15541 },
	{ 0xcb18, 0xcb18, PDF_CMAP_SINGLE, 2835 },
	{ 0xcb19, 0xcb1f, PDF_CMAP_RANGE, 15544 },
	{ 0xcb20, 0xcb21, PDF_CMAP_RANGE, 2836 },
	{ 0xcb22, 0xcb40, PDF_CMAP_RANGE, 15551 },
	{ 0xcb41, 0xcb41, PDF_CMAP_SINGLE, 2838 },
	{ 0xcb42, 0xcb47, PDF_CMAP_RANGE, 15582 },
	{ 0xcb48, 0xcb49, PDF_CMAP_RANGE, 2839 },
	{ 0xcb4a, 0xcb4b, PDF_CMAP_RANGE, 15588 },
	{ 0xcb4c, 0xcb4c, PDF_CMAP_SINGLE, 2841 },
	{ 0xcb4d, 0xcb4f, PDF_CMAP_RANGE, 15590 },
	{ 0xcb50, 0xcb50, PDF_CMAP_SINGLE, 2842 },
	{ 0xcb51, 0xcb57, PDF_CMAP_RANGE, 15593 },
	{ 0xcb58, 0xcb59, PDF_CMAP_RANGE, 2843 },
	{ 0xcb5a, 0xcb5c, PDF_CMAP_RANGE, 15600 },
	{ 0xcb5d, 0xcb5d, PDF_CMAP_SINGLE, 2845 },
	{ 0xcb5e, 0xcb63, PDF_CMAP_RANGE, 15603 },
	{ 0xcb64, 0xcb64, PDF_CMAP_SINGLE, 2846 },
	{ 0xcb65, 0xcb77, PDF_CMAP_RANGE, 15609 },
	{ 0xcb78, 0xcb79, PDF_CMAP_RANGE, 2847 },
	{ 0xcb7a, 0xcb9b, PDF_CMAP_RANGE, 15628 },
	{ 0xcb9c, 0xcb9c, PDF_CMAP_SINGLE, 2849 },
	{ 0xcb9d, 0xcbb7, PDF_CMAP_RANGE, 15662 },
	{ 0xcbb8, 0xcbb8, PDF_CMAP_SINGLE, 2850 },
	{ 0xcbb9, 0xcbd3, PDF_CMAP_RANGE, 15689 },
	{ 0xcbd4, 0xcbd4, PDF_CMAP_SINGLE, 2851 },
	{ 0xcbd5, 0xcbe3, PDF_CMAP_RANGE, 15716 },
	{ 0xcbe4, 0xcbe4, PDF_CMAP_SINGLE, 2852 },
	{ 0xcbe5, 0xcbe6, PDF_CMAP_RANGE, 15731 },
	{ 0xcbe7, 0xcbe9, PDF_CMAP_TABLE, 3020 },
	{ 0xcbea, 0xcc0b, PDF_CMAP_RANGE, 15734 },
	{ 0xcc0c, 0xcc0d, PDF_CMAP_RANGE, 2855 },
	{ 0xcc0e, 0xcc0f, PDF_CMAP_RANGE, 15768 },
	{ 0xcc10, 0xcc10, PDF_CMAP_SINGLE, 2857 },
	{ 0xcc11, 0xcc13, PDF_CMAP_RANGE, 15770 },
	{ 0xcc14, 0xcc14, PDF_CMAP_SINGLE, 2858 },
	{ 0xcc15, 0xcc1b, PDF_CMAP_RANGE, 15773 },
	{ 0xcc1c, 0xcc1d, PDF_CMAP_RANGE, 2859 },
	{ 0xcc1e, 0xcc20, PDF_CMAP_RANGE, 15780 },
	{ 0xcc21, 0xcc22, PDF_CMAP_RANGE, 2861 },
	{ 0xcc23, 0xcc26, PDF_CMAP_RANGE, 15783 },
	{ 0xcc27, 0xcc29, PDF_CMAP_RANGE, 2863 },
	{ 0xcc2a, 0xcc2b, PDF_CMAP_RANGE, 15787 },
	{ 0xcc2c, 0xcc30, PDF_CMAP_TABLE, 3023 },
	{ 0xcc31, 0xcc37, PDF_CMAP_RANGE, 15791 },
	{ 0xcc38, 0xcc39, PDF_CMAP_RANGE, 2869 },
	{ 0xcc3a, 0xcc3a, PDF_CMAP_SINGLE, 15798 },
	{ 0xcc3b, 0xcc3e, PDF_CMAP_RANGE, 2871 },
	{ 0xcc3f, 0xcc43, PDF_CMAP_RANGE, 15799 },
	{ 0xcc44, 0xcc45, PDF_CMAP_RANGE, 2875 },
	{ 0xcc46, 0xcc47, PDF_CMAP_RANGE, 15804 },
	{ 0xcc48, 0xcc48, PDF_CMAP_SINGLE, 2877 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_RANGE, 15806 },
	{ 0xcc4c, 0xcc4c, PDF_CMAP_SINGLE, 2878 },
	{ 0xcc4d, 0xcc53, PDF_CMAP_RANGE, 15809 },
	{ 0xcc54, 0xcc55, PDF_CMAP_RANGE, 2879 },
	{ 0xcc56, 0xcc56, PDF_CMAP_SINGLE, 15816 },
	{ 0xcc57, 0xcc59, PDF_CMAP_RANGE, 2881 },
	{ 0xcc5a, 0xcc5f, PDF_CMAP_RANGE, 15817 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 2884 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 15823 },
	{ 0xcc64, 0xcc68, PDF_CMAP_TABLE, 3028 },
	{ 0xcc69, 0xcc6f, PDF_CMAP_RANGE, 15828 },
	{ 0xcc70, 0xcc70, PDF_CMAP_SINGLE, 2888 },
	{ 0xcc71, 0xcc74, PDF_CMAP_RANGE, 15835 },
	{ 0xcc75, 0xcc75, PDF_CMAP_SINGLE, 2889 },
	{ 0xcc76, 0xcc97, PDF_CMAP_RANGE, 15839 },
	{ 0xcc98, 0xcc99, PDF_CMAP_RANGE, 2890 },
	{ 0xcc9a, 0xcc9b, PDF_CMAP_RANGE, 15873 },
	{ 0xcc9c, 0xcc9c, PDF_CMAP_SINGLE, 2892 },
	{ 0xcc9d, 0xcc9f, PDF_CMAP_RANGE, 15875 },
	{ 0xcca0, 0xcca0, PDF_CMAP_SINGLE, 2893 },
	{ 0xcca1, 0xcca7, PDF_CMAP_RANGE, 15878 },
	{ 0xcca8, 0xcca9, PDF_CMAP_RANGE, 2894 },
	{ 0xccaa, 0xccaa, PDF_CMAP_SINGLE, 15885 },
	{ 0xccab, 0xccad, PDF_CMAP_RANGE, 2896 },
	{ 0xccae, 0xccb3, PDF_CMAP_RANGE, 15886 },
	{ 0xccb4, 0xccb5, PDF_CMAP_RANGE, 2899 },
	{ 0xccb6, 0xccb7, PDF_CMAP_RANGE, 15892 },
	{ 0xccb8, 0xccb8, PDF_CMAP_SINGLE, 2901 },
	{ 0xccb9, 0xccbb, PDF_CMAP_RANGE, 15894 },
	{ 0xccbc, 0xccbc, PDF_CMAP_SINGLE, 2902 },
	{ 0xccbd, 0xccc3, PDF_CMAP_RANGE, 15897 },
	{ 0xccc4, 0xccc5, PDF_CMAP_RANGE, 2903 },
	{ 0xccc6, 0xccc9, PDF_CMAP_TABLE, 3033 },
	{ 0xccca, 0xcccf, PDF_CMAP_RANGE, 15906 },
	{ 0xccd0, 0xccd0, PDF_CMAP_SINGLE, 2907 },
	{ 0xccd1, 0xccd3, PDF_CMAP_RANGE, 15912 },
	{ 0xccd4, 0xccd4, PDF_CMAP_SINGLE, 2908 },
	{ 0xccd5, 0xcce3, PDF_CMAP_RANGE, 15915 },
	{ 0xcce4, 0xcce4, PDF_CMAP_SINGLE, 2909 },
	{ 0xcce5, 0xcceb, PDF_CMAP_RANGE, 15930 },
	{ 0xccec, 0xccec, PDF_CMAP_SINGLE, 2910 },
	{ 0xcced, 0xccef, PDF_CMAP_RANGE, 15937 },
	{ 0xccf0, 0xccf0, PDF_CMAP_SINGLE, 2911 },
	{ 0xccf1, 0xcd00, PDF_CMAP_RANGE, 15940 },
	{ 0xcd01, 0xcd01, PDF_CMAP_SINGLE, 2912 },
	{ 0xcd02, 0xcd07, PDF_CMAP_RANGE, 15956 },
	{ 0xcd08, 0xcd09, PDF_CMAP_RANGE, 2913 },
	{ 0xcd0a, 0xcd0b, PDF_CMAP_RANGE, 15962 },
	{ 0xcd0c, 0xcd0c, PDF_CMAP_SINGLE, 2915 },
	{ 0xcd0d, 0xcd0f, PDF_CMAP_RANGE, 15964 },
	{ 0xcd10, 0xcd10, PDF_CMAP_SINGLE, 2916 },
	{ 0xcd11, 0xcd17, PDF_CMAP_RANGE, 15967 },
	{ 0xcd18, 0xcd19, PDF_CMAP_RANGE, 2917 },
	{ 0xcd1a, 0xcd1d, PDF_CMAP_TABLE, 3037 },
	{ 0xcd1e, 0xcd23, PDF_CMAP_RANGE, 15976 },
	{ 0xcd24, 0xcd24, PDF_CMAP_SINGLE, 2921 },
	{ 0xcd25, 0xcd27, PDF_CMAP_RANGE, 15982 },
	{ 0xcd28, 0xcd28, PDF_CMAP_SINGLE, 2922 },
	{ 0xcd29, 0xcd2b, PDF_CMAP_RANGE, 15985 },
	{ 0xcd2c, 0xcd2c, PDF_CMAP_SINGLE, 2923 },
	{ 0xcd2d, 0xcd38, PDF_CMAP_RANGE, 15988 },
	{ 0xcd39, 0xcd39, PDF_CMAP_SINGLE, 2924 },
	{ 0xcd3a, 0xcd5b, PDF_CMAP_RANGE, 16000 },
	{ 0xcd5c, 0xcd5c, PDF_CMAP_SINGLE, 2925 },
	{ 0xcd5d, 0xcd5f, PDF_CMAP_RANGE, 16034 },
	{ 0xcd60, 0xcd60, PDF_CMAP_SINGLE, 2926 },
	{ 0xcd61, 0xcd63, PDF_CMAP_RANGE, 16037 },
	{ 0xcd64, 0xcd64, PDF_CMAP_SINGLE, 2927 },
	{ 0xcd65, 0xcd6b, PDF_CMAP_RANGE, 16040 },
	{ 0xcd6c, 0xcd6d, PDF_CMAP_RANGE, 2928 },
	{ 0xcd6e, 0xcd71, PDF_CMAP_TABLE, 3041 },
	{ 0xcd72, 0xcd77, PDF_CMAP_RANGE, 16049 },
	{ 0xcd78, 0xcd78, PDF_CMAP_SINGLE, 2932 },
	{ 0xcd79, 0xcd87, PDF_CMAP_RANGE, 16055 },
	{ 0xcd88, 0xcd88, PDF_CMAP_SINGLE, 2933 },
	{ 0xcd89, 0xcd93, PDF_CMAP_RANGE, 16070 },
	{ 0xcd94, 0xcd95, PDF_CMAP_RANGE, 2934 },
	{ 0xcd96, 0xcd97, PDF_CMAP_RANGE, 16081 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 2936 },
	{ 0xcd99, 0xcd9b, PDF_CMAP_RANGE, 16083 },
	{ 0xcd9c, 0xcd9c, PDF_CMAP_SINGLE, 2937 },
	{ 0xcd9d, 0xcda3, PDF_CMAP_RANGE, 16086 },
	{ 0xcda4, 0xcda5, PDF_CMAP_RANGE, 2938 },
	{ 0xcda6, 0xcda9, PDF_CMAP_TABLE, 3045 },
	{ 0xcdaa, 0xcdaf, PDF_CMAP_RANGE, 16095 },
	{ 0xcdb0, 0xcdb0, PDF_CMAP_SINGLE, 2942 },
	{ 0xcdb1, 0xcdc3, PDF_CMAP_RANGE, 16101 },
	{ 0xcdc4, 0xcdc4, PDF_CMAP_SINGLE, 2943 },
	{ 0xcdc5, 0xcdcb, PDF_CMAP_RANGE, 16120 },
	{ 0xcdcc, 0xcdcc, PDF_CMAP_SINGLE, 2944 },
	{ 0xcdcd, 0xcdcf, PDF_CMAP_RANGE, 16127 },
	{ 0xcdd0, 0xcdd0, PDF_CMAP_SINGLE, 2945 },
	{ 0xcdd1, 0xcde7, PDF_CMAP_RANGE, 16130 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 2946 },
	{ 0xcde9, 0xcdeb, PDF_CMAP_RANGE, 16153 },
	{ 0xcdec, 0xcdec, PDF_CMAP_SINGLE, 2947 },
	{ 0xcded, 0xcdef, PDF_CMAP_RANGE, 16156 },
	{ 0xcdf0, 0xcdf0, PDF_CMAP_SINGLE, 2948 },
	{ 0xcdf1, 0xcdf7, PDF_CMAP_RANGE, 16159 },
	{ 0xcdf8, 0xcdf9, PDF_CMAP_RANGE, 2949 },
	{ 0xcdfa, 0xcdfd, PDF_CMAP_TABLE, 3049 },
	{ 0xcdfe, 0xce03, PDF_CMAP_RANGE, 16168 },
	{ 0xce04, 0xce04, PDF_CMAP_SINGLE, 2953 },
	{ 0xce05, 0xce07, PDF_CMAP_RANGE, 16174 },
	{ 0xce08, 0xce08, PDF_CMAP_SINGLE, 2954 },
	{ 0xce09, 0xce0b, PDF_CMAP_RANGE, 16177 },
	{ 0xce0c, 0xce0c, PDF_CMAP_SINGLE, 2955 },
	{ 0xce0d, 0xce13, PDF_CMAP_RANGE, 16180 },
	{ 0xce14, 0xce14, PDF_CMAP_SINGLE, 2956 },
	{ 0xce15, 0xce18, PDF_CMAP_RANGE, 16187 },
	{ 0xce19, 0xce19, PDF_CMAP_SINGLE, 2957 },
	{ 0xce1a, 0xce1f, PDF_CMAP_RANGE, 16191 },
	{ 0xce20, 0xce21, PDF_CMAP_RANGE, 2958 },
	{ 0xce22, 0xce23, PDF_CMAP_RANGE, 16197 },
	{ 0xce24, 0xce24, PDF_CMAP_SINGLE, 2960 },
	{ 0xce25, 0xce27, PDF_CMAP_RANGE, 16199 },
	{ 0xce28, 0xce28, PDF_CMAP_SINGLE, 2961 },
	{ 0xce29, 0xce2f, PDF_CMAP_RANGE, 16202 },
	{ 0xce30, 0xce31, PDF_CMAP_RANGE, 2962 },
	{ 0xce32, 0xce35, PDF_CMAP_TABLE, 3053 },
	{ 0xce36, 0xce57, PDF_CMAP_RANGE, 16211 },
	{ 0xce58, 0xce59, PDF_CMAP_RANGE, 2966 },
	{ 0xce5a, 0xce5b, PDF_CMAP_RANGE, 16245 },
	{ 0xce5c, 0xce5c, PDF_CMAP_SINGLE, 2968 },
	{ 0xce5d, 0xce5e, PDF_CMAP_RANGE, 16247 },
	{ 0xce5f, 0xce61, PDF_CMAP_RANGE, 2969 },
	{ 0xce62, 0xce67, PDF_CMAP_RANGE, 16249 },
	{ 0xce68, 0xce69, PDF_CMAP_RANGE, 2972 },
	{ 0xce6a, 0xce6d, PDF_CMAP_TABLE, 3057 },
	{ 0xce6e, 0xce73, PDF_CMAP_RANGE, 16257 },
	{ 0xce74, 0xce75, PDF_CMAP_RANGE, 2976 },
	{ 0xce76, 0xce77, PDF_CMAP_RANGE, 16263 },
	{ 0xce78, 0xce78, PDF_CMAP_SINGLE, 2978 },
	{ 0xce79, 0xce7b, PDF_CMAP_RANGE, 16265 },
	{ 0xce7c, 0xce7c, PDF_CMAP_SINGLE, 2979 },
	{ 0xce7d, 0xce83, PDF_CMAP_RANGE, 16268 },
	{ 0xce84, 0xce85, PDF_CMAP_RANGE, 2980 },
	{ 0xce86, 0xce89, PDF_CMAP_TABLE, 3061 },
	{ 0xce8a, 0xce8f, PDF_CMAP_RANGE, 16277 },
	{ 0xce90, 0xce91, PDF_CMAP_RANGE, 2984 },
	{ 0xce92, 0xce93, PDF_CMAP_RANGE, 16283 },
	{ 0xce94, 0xce94, PDF_CMAP_SINGLE, 2986 },
	{ 0xce95, 0xce97, PDF_CMAP_RANGE, 16285 },
	{ 0xce98, 0xce98, PDF_CMAP_SINGLE, 2987 },
	{ 0xce99, 0xce9f, PDF_CMAP_RANGE, 16288 },
	{ 0xcea0, 0xcea1, PDF_CMAP_RANGE, 2988 },
	{ 0xcea2, 0xcea2, PDF_CMAP_SINGLE, 16295 },
	{ 0xcea3, 0xcea5, PDF_CMAP_RANGE, 2990 },
	{ 0xcea6, 0xceab, PDF_CMAP_RANGE, 16296 },
	{ 0xceac, 0xcead, PDF_CMAP_RANGE, 2993 },
	{ 0xceae, 0xcec0, PDF_CMAP_RANGE, 16302 },
	{ 0xcec1, 0xcec1, PDF_CMAP_SINGLE, 2995 },
	{ 0xcec2, 0xcee3, PDF_CMAP_RANGE, 16321 },
	{ 0xcee4, 0xcee5, PDF_CMAP_RANGE, 2996 },
	{ 0xcee6, 0xcee7, PDF_CMAP_RANGE, 16355 },
	{ 0xcee8, 0xcee8, PDF_CMAP_SINGLE, 2998 },
	{ 0xcee9, 0xceea, PDF_CMAP_RANGE, 16357 },
	{ 0xceeb, 0xceec, PDF_CMAP_RANGE, 2999 },
	{ 0xceed, 0xcef3, PDF_CMAP_RANGE, 16359 },
	{ 0xcef4, 0xcef5, PDF_CMAP_RANGE, 3001 },
	{ 0xcef6, 0xcef6, PDF_CMAP_SINGLE, 16366 },
	{ 0xcef7, 0xcef9, PDF_CMAP_RANGE, 3003 },
	{ 0xcefa, 0xceff, PDF_CMAP_RANGE, 16367 },
	{ 0xcf00, 0xcf01, PDF_CMAP_RANGE, 3006 },
	{ 0xcf02, 0xcf03, PDF_CMAP_RANGE, 16373 },
	{ 0xcf04, 0xcf04, PDF_CMAP_SINGLE, 3008 },
	{ 0xcf05, 0xcf07, PDF_CMAP_RANGE, 16375 },
	{ 0xcf08, 0xcf08, PDF_CMAP_SINGLE, 3009 },
	{ 0xcf09, 0xcf0f, PDF_CMAP_RANGE, 16378 },
	{ 0xcf10, 0xcf11, PDF_CMAP_RANGE, 3010 },
	{ 0xcf12, 0xcf15, PDF_CMAP_TABLE, 3065 },
	{ 0xcf16, 0xcf1b, PDF_CMAP_RANGE, 16387 },
	{ 0xcf1c, 0xcf1c, PDF_CMAP_SINGLE, 3014 },
	{ 0xcf1d, 0xcf1f, PDF_CMAP_RANGE, 16393 },
	{ 0xcf20, 0xcf20, PDF_CMAP_SINGLE, 3015 },
	{ 0xcf21, 0xcf23, PDF_CMAP_RANGE, 16396 },
	{ 0xcf24, 0xcf24, PDF_CMAP_SINGLE, 3016 },
	{ 0xcf25, 0xcf2b, PDF_CMAP_RANGE, 16399 },
	{ 0xcf2c, 0xcf2d, PDF_CMAP_RANGE, 3017 },
	{ 0xcf2e, 0xcf2e, PDF_CMAP_SINGLE, 16406 },
	{ 0xcf2f, 0xcf31, PDF_CMAP_RANGE, 3019 },
	{ 0xcf32, 0xcf37, PDF_CMAP_RANGE, 16407 },
	{ 0xcf38, 0xcf38, PDF_CMAP_SINGLE, 3022 },
	{ 0xcf39, 0xcf53, PDF_CMAP_RANGE, 16413 },
	{ 0xcf54, 0xcf55, PDF_CMAP_RANGE, 3023 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 16440 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 3025 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 16442 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 3026 },
	{ 0xcf5d, 0xcf63, PDF_CMAP_RANGE, 16445 },
	{ 0xcf64, 0xcf65, PDF_CMAP_RANGE, 3027 },
	{ 0xcf66, 0xcf69, PDF_CMAP_TABLE, 3069 },
	{ 0xcf6a, 0xcf6f, PDF_CMAP_RANGE, 16454 },
	{ 0xcf70, 0xcf71, PDF_CMAP_RANGE, 3031 },
	{ 0xcf72, 0xcf73, PDF_CMAP_RANGE, 16460 },
	{ 0xcf74, 0xcf74, PDF_CMAP_SINGLE, 3033 },
	{ 0xcf75, 0xcf77, PDF_CMAP_RANGE, 16462 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 3034 },
	{ 0xcf79, 0xcf7f, PDF_CMAP_RANGE, 16465 },
	{ 0xcf80, 0xcf80, PDF_CMAP_SINGLE, 3035 },
	{ 0xcf81, 0xcf84, PDF_CMAP_RANGE, 16472 },
	{ 0xcf85, 0xcf85, PDF_CMAP_SINGLE, 3036 },
	{ 0xcf86, 0xcf8b, PDF_CMAP_RANGE, 16476 },
	{ 0xcf8c, 0xcf8c, PDF_CMAP_SINGLE, 3037 },
	{ 0xcf8d, 0xcfa0, PDF_CMAP_RANGE, 16482 },
	{ 0xcfa1, 0xcfa1, PDF_CMAP_SINGLE, 3038 },
	{ 0xcfa2, 0xcfa7, PDF_CMAP_RANGE, 16502 },
	{ 0xcfa8, 0xcfa8, PDF_CMAP_SINGLE, 3039 },
	{ 0xcfa9, 0xcfaf, PDF_CMAP_RANGE, 16508 },
	{ 0xcfb0, 0xcfb0, PDF_CMAP_SINGLE, 3040 },
	{ 0xcfb1, 0xcfc3, PDF_CMAP_RANGE, 16515 },
	{ 0xcfc4, 0xcfc4, PDF_CMAP_SINGLE, 3041 },
	{ 0xcfc5, 0xcfdf, PDF_CMAP_RANGE, 16534 },
	{ 0xcfe0, 0xcfe1, PDF_CMAP_RANGE, 3042 },
	{ 0xcfe2, 0xcfe3, PDF_CMAP_RANGE, 16561 },
	{ 0xcfe4, 0xcfe4, PDF_CMAP_SINGLE, 3044 },
	{ 0xcfe5, 0xcfe7, PDF_CMAP_RANGE, 16563 },
	{ 0xcfe8, 0xcfe8, PDF_CMAP_SINGLE, 3045 },
	{ 0xcfe9, 0xcfef, PDF_CMAP_RANGE, 16566 },
	{ 0xcff0, 0xcff1, PDF_CMAP_RANGE, 3046 },
	{ 0xcff2, 0xcff5, PDF_CMAP_TABLE, 3073 },
	{ 0xcff6, 0xcffb, PDF_CMAP_RANGE, 16575 },
	{ 0xcffc, 0xcffc, PDF_CMAP_SINGLE, 3050 },
	{ 0xcffd, 0xcfff, PDF_CMAP_RANGE, 16581 },
	{ 0xd000, 0xd000, PDF_CMAP_SINGLE, 3051 },
	{ 0xd001, 0xd003, PDF_CMAP_RANGE, 16584 },
	{ 0xd004, 0xd004, PDF_CMAP_SINGLE, 3052 },
	{ 0xd005, 0xd010, PDF_CMAP_RANGE, 16587 },
	{ 0xd011, 0xd011, PDF_CMAP_SINGLE, 3053 },
	{ 0xd012, 0xd017, PDF_CMAP_RANGE, 16599 },
	{ 0xd018, 0xd018, PDF_CMAP_SINGLE, 3054 },
	{ 0xd019, 0xd02c, PDF_CMAP_RANGE, 16605 },
	{ 0xd02d, 0xd02d, PDF_CMAP_SINGLE, 3055 },
	{ 0xd02e, 0xd033, PDF_CMAP_RANGE, 16625 },
	{ 0xd034, 0xd035, PDF_CMAP_RANGE, 3056 },
	{ 0xd036, 0xd037, PDF_CMAP_RANGE, 16631 },
	{ 0xd038, 0xd038, PDF_CMAP_SINGLE, 3058 },
	{ 0xd039, 0xd03b, PDF_CMAP_RANGE, 16633 },
	{ 0xd03c, 0xd03c, PDF_CMAP_SINGLE, 3059 },
	{ 0xd03d, 0xd043, PDF_CMAP_RANGE, 16636 },
	{ 0xd044, 0xd045, PDF_CMAP_RANGE, 3060 },
	{ 0xd046, 0xd049, PDF_CMAP_TABLE, 3077 },
	{ 0xd04a, 0xd04f, PDF_CMAP_RANGE, 16645 },
	{ 0xd050, 0xd050, PDF_CMAP_SINGLE, 3064 },
	{ 0xd051, 0xd053, PDF_CMAP_RANGE, 16651 },
	{ 0xd054, 0xd054, PDF_CMAP_SINGLE, 3065 },
	{ 0xd055, 0xd057, PDF_CMAP_RANGE, 16654 },
	{ 0xd058, 0xd058, PDF_CMAP_SINGLE, 3066 },
	{ 0xd059, 0xd05f, PDF_CMAP_RANGE, 16657 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 3067 },
	{ 0xd061, 0xd06b, PDF_CMAP_RANGE, 16664 },
	{ 0xd06c, 0xd06d, PDF_CMAP_RANGE, 3068 },
	{ 0xd06e, 0xd06f, PDF_CMAP_RANGE, 16675 },
	{ 0xd070, 0xd070, PDF_CMAP_SINGLE, 3070 },
	{ 0xd071, 0xd073, PDF_CMAP_RANGE, 16677 },
	{ 0xd074, 0xd074, PDF_CMAP_SINGLE, 3071 },
	{ 0xd075, 0xd07b, PDF_CMAP_RANGE, 16680 },
	{ 0xd07c, 0xd07d, PDF_CMAP_RANGE, 3072 },
	{ 0xd07e, 0xd080, PDF_CMAP_RANGE, 16687 },
	{ 0xd081, 0xd081, PDF_CMAP_SINGLE, 3074 },
	{ 0xd082, 0xd0a3, PDF_CMAP_RANGE, 16690 },
	{ 0xd0a4, 0xd0a5, PDF_CMAP_RANGE, 3075 },
	{ 0xd0a6, 0xd0a7, PDF_CMAP_RANGE, 16724 },
	{ 0xd0a8, 0xd0a8, PDF_CMAP_SINGLE, 3077 },
	{ 0xd0a9, 0xd0ab, PDF_CMAP_RANGE, 16726 },
	{ 0xd0ac, 0xd0ac, PDF_CMAP_SINGLE, 3078 },
	{ 0xd0ad, 0xd0b3, PDF_CMAP_RANGE, 16729 },
	{ 0xd0b4, 0xd0b5, PDF_CMAP_RANGE, 3079 },
	{ 0xd0b6, 0xd0b9, PDF_CMAP_TABLE, 3081 },
	{ 0xd0ba, 0xd0bf, PDF_CMAP_RANGE, 16738 },
	{ 0xd0c0, 0xd0c1, PDF_CMAP_RANGE, 3083 },
	{ 0xd0c2, 0xd0c3, PDF_CMAP_RANGE, 16744 },
	{ 0xd0c4, 0xd0c4, PDF_CMAP_SINGLE, 3085 },
	{ 0xd0c5, 0xd0c7, PDF_CMAP_RANGE, 16746 },
	{ 0xd0c8, 0xd0c9, PDF_CMAP_RANGE, 3086 },
	{ 0xd0ca, 0xd0cf, PDF_CMAP_RANGE, 16749 },
	{ 0xd0d0, 0xd0d1, PDF_CMAP_RANGE, 3088 },
	{ 0xd0d2, 0xd0d2, PDF_CMAP_SINGLE, 16755 },
	{ 0xd0d3, 0xd0d5, PDF_CMAP_RANGE, 3090 },
	{ 0xd0d6, 0xd0db, PDF_CMAP_RANGE, 16756 },
	{ 0xd0dc, 0xd0dd, PDF_CMAP_RANGE, 3093 },
	{ 0xd0de, 0xd0df, PDF_CMAP_RANGE, 16762 },
	{ 0xd0e0, 0xd0e0, PDF_CMAP_SINGLE, 3095 },
	{ 0xd0e1, 0xd0e3, PDF_CMAP_RANGE, 16764 },
	{ 0xd0e4, 0xd0e4, PDF_CMAP_SINGLE, 3096 },
	{ 0xd0e5, 0xd0eb, PDF_CMAP_RANGE, 16767 },
	{ 0xd0ec, 0xd0ed, PDF_CMAP_RANGE, 3097 },
	{ 0xd0ee, 0xd0ee, PDF_CMAP_SINGLE, 16774 },
	{ 0xd0ef, 0xd0f1, PDF_CMAP_RANGE, 3099 },
	{ 0xd0f2, 0xd0f7, PDF_CMAP_RANGE, 16775 },
	{ 0xd0f8, 0xd0f8, PDF_CMAP_SINGLE, 3102 },
	{ 0xd0f9, 0xd10c, PDF_CMAP_RANGE, 16781 },
	{ 0xd10d, 0xd10d, PDF_CMAP_SINGLE, 3103 },
	{ 0xd10e, 0xd12f, PDF_CMAP_RANGE, 16801 },
	{ 0xd130, 0xd131, PDF_CMAP_RANGE, 3104 },
	{ 0xd132, 0xd133, PDF_CMAP_RANGE, 16835 },
	{ 0xd134, 0xd134, PDF_CMAP_SINGLE, 3106 },
	{ 0xd135, 0xd137, PDF_CMAP_RANGE, 16837 },
	{ 0xd138, 0xd13a, PDF_CMAP_TABLE, 3085 },
	{ 0xd13b, 0xd13f, PDF_CMAP_RANGE, 16841 },
	{ 0xd140, 0xd141, PDF_CMAP_RANGE, 3109 },
	{ 0xd142, 0xd142, PDF_CMAP_SINGLE, 16846 },
	{ 0xd143, 0xd145, PDF_CMAP_RANGE, 3111 },
	{ 0xd146, 0xd14b, PDF_CMAP_RANGE, 16847 },
	{ 0xd14c, 0xd14d, PDF_CMAP_RANGE, 3114 },
	{ 0xd14e, 0xd14f, PDF_CMAP_RANGE, 16853 },
	{ 0xd150, 0xd150, PDF_CMAP_SINGLE, 3116 },
	{ 0xd151, 0xd153, PDF_CMAP_RANGE, 16855 },
	{ 0xd154, 0xd154, PDF_CMAP_SINGLE, 3117 },
	{ 0xd155, 0xd15b, PDF_CMAP_RANGE, 16858 },
	{ 0xd15c, 0xd15d, PDF_CMAP_RANGE, 3118 },
	{ 0xd15e, 0xd161, PDF_CMAP_TABLE, 3088 },
	{ 0xd162, 0xd167, PDF_CMAP_RANGE, 16867 },
	{ 0xd168, 0xd168, PDF_CMAP_SINGLE, 3122 },
	{ 0xd169, 0xd16b, PDF_CMAP_RANGE, 16873 },
	{ 0xd16c, 0xd16c, PDF_CMAP_SINGLE, 3123 },
	{ 0xd16d, 0xd17b, PDF_CMAP_RANGE, 16876 },
	{ 0xd17c, 0xd17c, PDF_CMAP_SINGLE, 3124 },
	{ 0xd17d, 0xd183, PDF_CMAP_RANGE, 16891 },
	{ 0xd184, 0xd184, PDF_CMAP_SINGLE, 3125 },
	{ 0xd185, 0xd187, PDF_CMAP_RANGE, 16898 },
	{ 0xd188, 0xd188, PDF_CMAP_SINGLE, 3126 },
	{ 0xd189, 0xd19f, PDF_CMAP_RANGE, 16901 },
	{ 0xd1a0, 0xd1a1, PDF_CMAP_RANGE, 3127 },
	{ 0xd1a2, 0xd1a3, PDF_CMAP_RANGE, 16924 },
	{ 0xd1a4, 0xd1a4, PDF_CMAP_SINGLE, 3129 },
	{ 0xd1a5, 0xd1a7, PDF_CMAP_RANGE, 16926 },
	{ 0xd1a8, 0xd1a8, PDF_CMAP_SINGLE, 3130 },
	{ 0xd1a9, 0xd1af, PDF_CMAP_RANGE, 16929 },
	{ 0xd1b0, 0xd1b1, PDF_CMAP_RANGE, 3131 },
	{ 0xd1b2, 0xd1b5, PDF_CMAP_TABLE, 3092 },
	{ 0xd1b6, 0xd1b9, PDF_CMAP_RANGE, 16938 },
	{ 0xd1ba, 0xd1bc, PDF_CMAP_TABLE, 3096 },
	{ 0xd1bd, 0xd1bf, PDF_CMAP_RANGE, 16943 },
	{ 0xd1c0, 0xd1c0, PDF_CMAP_SINGLE, 3137 },
	{ 0xd1c1, 0xd1d7, PDF_CMAP_RANGE, 16946 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 3138 },
	{ 0xd1d9, 0xd1f3, PDF_CMAP_RANGE, 16969 },
	{ 0xd1f4, 0xd1f4, PDF_CMAP_SINGLE, 3139 },
	{ 0xd1f5, 0xd1f7, PDF_CMAP_RANGE, 16996 },
	{ 0xd1f8, 0xd1f8, PDF_CMAP_SINGLE, 3140 },
	{ 0xd1f9, 0xd206, PDF_CMAP_RANGE, 16999 },
	{ 0xd207, 0xd209, PDF_CMAP_TABLE, 3099 },
	{ 0xd20a, 0xd20f, PDF_CMAP_RANGE, 17014 },
	{ 0xd210, 0xd210, PDF_CMAP_SINGLE, 3143 },
	{ 0xd211, 0xd22b, PDF_CMAP_RANGE, 17020 },
	{ 0xd22c, 0xd22d, PDF_CMAP_RANGE, 3144 },
	{ 0xd22e, 0xd22f, PDF_CMAP_RANGE, 17047 },
	{ 0xd230, 0xd230, PDF_CMAP_SINGLE, 3146 },
	{ 0xd231, 0xd233, PDF_CMAP_RANGE, 17049 },
	{ 0xd234, 0xd234, PDF_CMAP_SINGLE, 3147 },
	{ 0xd235, 0xd23b, PDF_CMAP_RANGE, 17052 },
	{ 0xd23c, 0xd23d, PDF_CMAP_RANGE, 3148 },
	{ 0xd23e, 0xd241, PDF_CMAP_TABLE, 3102 },
	{ 0xd242, 0xd247, PDF_CMAP_RANGE, 17061 },
	{ 0xd248, 0xd248, PDF_CMAP_SINGLE, 3152 },
	{ 0xd249, 0xd25b, PDF_CMAP_RANGE, 17067 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 3153 },
	{ 0xd25d, 0xd263, PDF_CMAP_RANGE, 17086 },
	{ 0xd264, 0xd264, PDF_CMAP_SINGLE, 3154 },
	{ 0xd265, 0xd27f, PDF_CMAP_RANGE, 17093 },
	{ 0xd280, 0xd281, PDF_CMAP_RANGE, 3155 },
	{ 0xd282, 0xd283, PDF_CMAP_RANGE, 17120 },
	{ 0xd284, 0xd284, PDF_CMAP_SINGLE, 3157 },
	{ 0xd285, 0xd287, PDF_CMAP_RANGE, 17122 },
	{ 0xd288, 0xd288, PDF_CMAP_SINGLE, 3158 },
	{ 0xd289, 0xd28f, PDF_CMAP_RANGE, 17125 },
	{ 0xd290, 0xd291, PDF_CMAP_RANGE, 3159 },
	{ 0xd292, 0xd294, PDF_CMAP_RANGE, 17132 },
	{ 0xd295, 0xd295, PDF_CMAP_SINGLE, 3161 },
	{ 0xd296, 0xd29b, PDF_CMAP_RANGE, 17135 },
	{ 0xd29c, 0xd29c, PDF_CMAP_SINGLE, 3162 },
	{ 0xd29d, 0xd29f, PDF_CMAP_RANGE, 17141 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 3163 },
	{ 0xd2a1, 0xd2a3, PDF_CMAP_RANGE, 17144 },
	{ 0xd2a4, 0xd2a4, PDF_CMAP_SINGLE, 3164 },
	{ 0xd2a5, 0xd2ab, PDF_CMAP_RANGE, 17147 },
	{ 0xd2ac, 0xd2ac, PDF_CMAP_SINGLE, 3165 },
	{ 0xd2ad, 0xd2b0, PDF_CMAP_RANGE, 17154 },
	{ 0xd2b1, 0xd2b1, PDF_CMAP_SINGLE, 3166 },
	{ 0xd2b2, 0xd2b7, PDF_CMAP_RANGE, 17158 },
	{ 0xd2b8, 0xd2b9, PDF_CMAP_RANGE, 3167 },
	{ 0xd2ba, 0xd2bb, PDF_CMAP_RANGE, 17164 },
	{ 0xd2bc, 0xd2bc, PDF_CMAP_SINGLE, 3169 },
	{ 0xd2bd, 0xd2be, PDF_CMAP_RANGE, 17166 },
	{ 0xd2bf, 0xd2c0, PDF_CMAP_RANGE, 3170 },
	{ 0xd2c1, 0xd2c2, PDF_CMAP_TABLE, 3106 },
	{ 0xd2c3, 0xd2c7, PDF_CMAP_RANGE, 17169 },
	{ 0xd2c8, 0xd2c9, PDF_CMAP_RANGE, 3173 },
	{ 0xd2ca, 0xd2cb, PDF_CMAP_TABLE, 3108 },
	{ 0xd2cc, 0xd2d3, PDF_CMAP_RANGE, 17175 },
	{ 0xd2d4, 0xd2d4, PDF_CMAP_SINGLE, 3176 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 17183 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 3177 },
	{ 0xd2d9, 0xd2db, PDF_CMAP_RANGE, 17186 },
	{ 0xd2dc, 0xd2dc, PDF_CMAP_SINGLE, 3178 },
	{ 0xd2dd, 0xd2e3, PDF_CMAP_RANGE, 17189 },
	{ 0xd2e4, 0xd2e5, PDF_CMAP_RANGE, 3179 },
	{ 0xd2e6, 0xd2ef, PDF_CMAP_RANGE, 17196 },
	{ 0xd2f0, 0xd2f1, PDF_CMAP_RANGE, 3181 },
	{ 0xd2f2, 0xd2f3, PDF_CMAP_RANGE, 17206 },
	{ 0xd2f4, 0xd2f4, PDF_CMAP_SINGLE, 3183 },
	{ 0xd2f5, 0xd2f7, PDF_CMAP_RANGE, 17208 },
	{ 0xd2f8, 0xd2f8, PDF_CMAP_SINGLE, 3184 },
	{ 0xd2f9, 0xd2ff, PDF_CMAP_RANGE, 17211 },
	{ 0xd300, 0xd301, PDF_CMAP_RANGE, 3185 },
	{ 0xd302, 0xd305, PDF_CMAP_TABLE, 3110 },
	{ 0xd306, 0xd30b, PDF_CMAP_RANGE, 17220 },
	{ 0xd30c, 0xd30e, PDF_CMAP_RANGE, 3189 },
	{ 0xd30f, 0xd310, PDF_CMAP_TABLE, 3114 },
	{ 0xd311, 0xd313, PDF_CMAP_RANGE, 17227 },
	{ 0xd314, 0xd316, PDF_CMAP_TABLE, 3116 },
	{ 0xd317, 0xd31b, PDF_CMAP_RANGE, 17231 },
	{ 0xd31c, 0xd31d, PDF_CMAP_RANGE, 3195 },
	{ 0xd31e, 0xd31e, PDF_CMAP_SINGLE, 17236 },
	{ 0xd31f, 0xd321, PDF_CMAP_RANGE, 3197 },
	{ 0xd322, 0xd324, PDF_CMAP_RANGE, 17237 },
	{ 0xd325, 0xd325, PDF_CMAP_SINGLE, 3200 },
	{ 0xd326, 0xd327, PDF_CMAP_RANGE, 17240 },
	{ 0xd328, 0xd329, PDF_CMAP_RANGE, 3201 },
	{ 0xd32a, 0xd32b, PDF_CMAP_RANGE, 17242 },
	{ 0xd32c, 0xd32c, PDF_CMAP_SINGLE, 3203 },
	{ 0xd32d, 0xd32f, PDF_CMAP_RANGE, 17244 },
	{ 0xd330, 0xd330, PDF_CMAP_SINGLE, 3204 },
	{ 0xd331, 0xd337, PDF_CMAP_RANGE, 17247 },
	{ 0xd338, 0xd339, PDF_CMAP_RANGE, 3205 },
	{ 0xd33a, 0xd33a, PDF_CMAP_SINGLE, 17254 },
	{ 0xd33b, 0xd33d, PDF_CMAP_RANGE, 3207 },
	{ 0xd33e, 0xd343, PDF_CMAP_RANGE, 17255 },
	{ 0xd344, 0xd345, PDF_CMAP_RANGE, 3210 },
	{ 0xd346, 0xd37b, PDF_CMAP_RANGE, 17261 },
	{ 0xd37c, 0xd37d, PDF_CMAP_RANGE, 3212 },
	{ 0xd37e, 0xd37f, PDF_CMAP_RANGE, 17315 },
	{ 0xd380, 0xd380, PDF_CMAP_SINGLE, 3214 },
	{ 0xd381, 0xd383, PDF_CMAP_RANGE, 17317 },
	{ 0xd384, 0xd384, PDF_CMAP_SINGLE, 3215 },
	{ 0xd385, 0xd38b, PDF_CMAP_RANGE, 17320 },
	{ 0xd38c, 0xd38d, PDF_CMAP_RANGE, 3216 },
	{ 0xd38e, 0xd38e, PDF_CMAP_SINGLE, 17327 },
	{ 0xd38f, 0xd391, PDF_CMAP_RANGE, 3218 },
	{ 0xd392, 0xd397, PDF_CMAP_RANGE, 17328 },
	{ 0xd398, 0xd399, PDF_CMAP_RANGE, 3221 },
	{ 0xd39a, 0xd39b, PDF_CMAP_RANGE, 17334 },
	{ 0xd39c, 0xd39c, PDF_CMAP_SINGLE, 3223 },
	{ 0xd39d, 0xd39f, PDF_CMAP_RANGE, 17336 },
	{ 0xd3a0, 0xd3a0, PDF_CMAP_SINGLE, 3224 },
	{ 0xd3a1, 0xd3a7, PDF_CMAP_RANGE, 17339 },
	{ 0xd3a8, 0xd3a9, PDF_CMAP_RANGE, 3225 },
	{ 0xd3aa, 0xd3ad, PDF_CMAP_TABLE, 3119 },
	{ 0xd3ae, 0xd3b3, PDF_CMAP_RANGE, 17348 },
	{ 0xd3b4, 0xd3b4, PDF_CMAP_SINGLE, 3229 },
	{ 0xd3b5, 0xd3b7, PDF_CMAP_RANGE, 17354 },
	{ 0xd3b8, 0xd3b8, PDF_CMAP_SINGLE, 3230 },
	{ 0xd3b9, 0xd3bb, PDF_CMAP_RANGE, 17357 },
	{ 0xd3bc, 0xd3bc, PDF_CMAP_SINGLE, 3231 },
	{ 0xd3bd, 0xd3c3, PDF_CMAP_RANGE, 17360 },
	{ 0xd3c4, 0xd3c5, PDF_CMAP_RANGE, 3232 },
	{ 0xd3c6, 0xd3c7, PDF_CMAP_RANGE, 17367 },
	{ 0xd3c8, 0xd3c9, PDF_CMAP_RANGE, 3234 },
	{ 0xd3ca, 0xd3cf, PDF_CMAP_RANGE, 17369 },
	{ 0xd3d0, 0xd3d0, PDF_CMAP_SINGLE, 3236 },
	{ 0xd3d1, 0xd3d7, PDF_CMAP_RANGE, 17375 },
	{ 0xd3d8, 0xd3d8, PDF_CMAP_SINGLE, 3237 },
	{ 0xd3d9, 0xd3e0, PDF_CMAP_RANGE, 17382 },
	{ 0xd3e1, 0xd3e3, PDF_CMAP_TABLE, 3123 },
	{ 0xd3e4, 0xd3eb, PDF_CMAP_RANGE, 17391 },
	{ 0xd3ec, 0xd3ed, PDF_CMAP_RANGE, 3240 },
	{ 0xd3ee, 0xd3ef, PDF_CMAP_RANGE, 17399 },
	{ 0xd3f0, 0xd3f0, PDF_CMAP_SINGLE, 3242 },
	{ 0xd3f1, 0xd3f3, PDF_CMAP_RANGE, 17401 },
	{ 0xd3f4, 0xd3f4, PDF_CMAP_SINGLE, 3243 },
	{ 0xd3f5, 0xd3fb, PDF_CMAP_RANGE, 17404 },
	{ 0xd3fc, 0xd3fd, PDF_CMAP_RANGE, 3244 },
	{ 0xd3fe, 0xd401, PDF_CMAP_TABLE, 3126 },
	{ 0xd402, 0xd407, PDF_CMAP_RANGE, 17413 },
	{ 0xd408, 0xd408, PDF_CMAP_SINGLE, 3248 },
	{ 0xd409, 0xd41c, PDF_CMAP_RANGE, 17419 },
	{ 0xd41d, 0xd41d, PDF_CMAP_SINGLE, 3249 },
	{ 0xd41e, 0xd43f, PDF_CMAP_RANGE, 17439 },
	{ 0xd440, 0xd440, PDF_CMAP_SINGLE, 3250 },
	{ 0xd441, 0xd443, PDF_CMAP_RANGE, 17473 },
	{ 0xd444, 0xd444, PDF_CMAP_SINGLE, 3251 },
	{ 0xd445, 0xd45b, PDF_CMAP_RANGE, 17476 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 3252 },
	{ 0xd45d, 0xd45f, PDF_CMAP_RANGE, 17499 },
	{ 0xd460, 0xd460, PDF_CMAP_SINGLE, 3253 },
	{ 0xd461, 0xd463, PDF_CMAP_RANGE, 17502 },
	{ 0xd464, 0xd464, PDF_CMAP_SINGLE, 3254 },
	{ 0xd465, 0xd46c, PDF_CMAP_RANGE, 17505 },
	{ 0xd46d, 0xd46f, PDF_CMAP_TABLE, 3130 },
	{ 0xd470, 0xd477, PDF_CMAP_RANGE, 17514 },
	{ 0xd478, 0xd479, PDF_CMAP_RANGE, 3257 },
	{ 0xd47a, 0xd47b, PDF_CMAP_RANGE, 17522 },
	{ 0xd47c, 0xd47c, PDF_CMAP_SINGLE, 3259 },
	{ 0xd47d, 0xd47e, PDF_CMAP_RANGE, 17524 },
	{ 0xd47f, 0xd480, PDF_CMAP_RANGE, 3260 },
	{ 0xd481, 0xd482, PDF_CMAP_TABLE, 3133 },
	{ 0xd483, 0xd487, PDF_CMAP_RANGE, 17527 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 3263 },
	{ 0xd48a, 0xd48d, PDF_CMAP_TABLE, 3135 },
	{ 0xd48e, 0xd493, PDF_CMAP_RANGE, 17534 },
	{ 0xd494, 0xd494, PDF_CMAP_SINGLE, 3267 },
	{ 0xd495, 0xd4a8, PDF_CMAP_RANGE, 17540 },
	{ 0xd4a9, 0xd4a9, PDF_CMAP_SINGLE, 3268 },
	{ 0xd4aa, 0xd4cb, PDF_CMAP_RANGE, 17560 },
	{ 0xd4cc, 0xd4cc, PDF_CMAP_SINGLE, 3269 },
	{ 0xd4cd, 0xd4cf, PDF_CMAP_RANGE, 17594 },
	{ 0xd4d0, 0xd4d0, PDF_CMAP_SINGLE, 3270 },
	{ 0xd4d1, 0xd4d3, PDF_CMAP_RANGE, 17597 },
	{ 0xd4d4, 0xd4d4, PDF_CMAP_SINGLE, 3271 },
	{ 0xd4d5, 0xd4db, PDF_CMAP_RANGE, 17600 },
	{ 0xd4dc, 0xd4dc, PDF_CMAP_SINGLE, 3272 },
	{ 0xd4dd, 0xd4de, PDF_CMAP_RANGE, 17607 },
	{ 0xd4df, 0xd4df, PDF_CMAP_SINGLE, 3273 },
	{ 0xd4e0, 0xd4e7, PDF_CMAP_RANGE, 17609 },
	{ 0xd4e8, 0xd4e8, PDF_CMAP_SINGLE, 3274 },
	{ 0xd4e9, 0xd4eb, PDF_CMAP_RANGE, 17617 },
	{ 0xd4ec, 0xd4ec, PDF_CMAP_SINGLE, 3275 },
	{ 0xd4ed, 0xd4ef, PDF_CMAP_RANGE, 17620 },
	{ 0xd4f0, 0xd4f0, PDF_CMAP_SINGLE, 3276 },
	{ 0xd4f1, 0xd4f7, PDF_CMAP_RANGE, 17623 },
	{ 0xd4f8, 0xd4f8, PDF_CMAP_SINGLE, 3277 },
	{ 0xd4f9, 0xd4fa, PDF_CMAP_RANGE, 17630 },
	{ 0xd4fb, 0xd4fd, PDF_CMAP_TABLE, 3139 },
	{ 0xd4fe, 0xd503, PDF_CMAP_RANGE, 17633 },
	{ 0xd504, 0xd504, PDF_CMAP_SINGLE, 3280 },
	{ 0xd505, 0xd507, PDF_CMAP_RANGE, 17639 },
	{ 0xd508, 0xd508, PDF_CMAP_SINGLE, 3281 },
	{ 0xd509, 0xd50b, PDF_CMAP_RANGE, 17642 },
	{ 0xd50c, 0xd50c, PDF_CMAP_SINGLE, 3282 },
	{ 0xd50d, 0xd513, PDF_CMAP_RANGE, 17645 },
	{ 0xd514, 0xd515, PDF_CMAP_RANGE, 3283 },
	{ 0xd516, 0xd517, PDF_CMAP_TABLE, 3142 },
	{ 0xd518, 0xd53b, PDF_CMAP_RANGE, 17653 },
	{ 0xd53c, 0xd53d, PDF_CMAP_RANGE, 3286 },
	{ 0xd53e, 0xd53f, PDF_CMAP_RANGE, 17689 },
	{ 0xd540, 0xd540, PDF_CMAP_SINGLE, 3288 },
	{ 0xd541, 0xd543, PDF_CMAP_RANGE, 17691 },
	{ 0xd544, 0xd544, PDF_CMAP_SINGLE, 3289 },
	{ 0xd545, 0xd54b, PDF_CMAP_RANGE, 17694 },
	{ 0xd54c, 0xd54d, PDF_CMAP_RANGE, 3290 },
	{ 0xd54e, 0xd551, PDF_CMAP_TABLE, 3144 },
	{ 0xd552, 0xd557, PDF_CMAP_RANGE, 17703 },
	{ 0xd558, 0xd559, PDF_CMAP_RANGE, 3294 },
	{ 0xd55a, 0xd55b, PDF_CMAP_RANGE, 17709 },
	{ 0xd55c, 0xd55c, PDF_CMAP_SINGLE, 3296 },
	{ 0xd55d, 0xd55f, PDF_CMAP_RANGE, 17711 },
	{ 0xd560, 0xd560, PDF_CMAP_SINGLE, 3297 },
	{ 0xd561, 0xd564, PDF_CMAP_RANGE, 17714 },
	{ 0xd565, 0xd565, PDF_CMAP_SINGLE, 3298 },
	{ 0xd566, 0xd567, PDF_CMAP_RANGE, 17718 },
	{ 0xd568, 0xd569, PDF_CMAP_RANGE, 3299 },
	{ 0xd56a, 0xd56d, PDF_CMAP_TABLE, 3148 },
	{ 0xd56e, 0xd573, PDF_CMAP_RANGE, 17722 },
	{ 0xd574, 0xd575, PDF_CMAP_RANGE, 3303 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 17728 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 3305 },
	{ 0xd579, 0xd57b, PDF_CMAP_RANGE, 17730 },
	{ 0xd57c, 0xd57c, PDF_CMAP_SINGLE, 3306 },
	{ 0xd57d, 0xd583, PDF_CMAP_RANGE, 17733 },
	{ 0xd584, 0xd585, PDF_CMAP_RANGE, 3307 },
	{ 0xd586, 0xd586, PDF_CMAP_SINGLE, 17740 },
	{ 0xd587, 0xd589, PDF_CMAP_RANGE, 3309 },
	{ 0xd58a, 0xd58f, PDF_CMAP_RANGE, 17741 },
	{ 0xd590, 0xd590, PDF_CMAP_SINGLE, 3312 },
	{ 0xd591, 0xd5a4, PDF_CMAP_RANGE, 17747 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 3313 },
	{ 0xd5a6, 0xd5c7, PDF_CMAP_RANGE, 17767 },
	{ 0xd5c8, 0xd5c9, PDF_CMAP_RANGE, 3314 },
	{ 0xd5ca, 0xd5cb, PDF_CMAP_RANGE, 17801 },
	{ 0xd5cc, 0xd5cc, PDF_CMAP_SINGLE, 3316 },
	{ 0xd5cd, 0xd5cf, PDF_CMAP_RANGE, 17803 },
	{ 0xd5d0, 0xd5d2, PDF_CMAP_TABLE, 3152 },
	{ 0xd5d3, 0xd5d7, PDF_CMAP_RANGE, 17807 },
	{ 0xd5d8, 0xd5d9, PDF_CMAP_RANGE, 3319 },
	{ 0xd5da, 0xd5dd, PDF_CMAP_TABLE, 3155 },
	{ 0xd5de, 0xd5e3, PDF_CMAP_RANGE, 17814 },
	{ 0xd5e4, 0xd5e5, PDF_CMAP_RANGE, 3323 },
	{ 0xd5e6, 0xd5e7, PDF_CMAP_RANGE, 17820 },
	{ 0xd5e8, 0xd5e8, PDF_CMAP_SINGLE, 3325 },
	{ 0xd5e9, 0xd5eb, PDF_CMAP_RANGE, 17822 },
	{ 0xd5ec, 0xd5ec, PDF_CMAP_SINGLE, 3326 },
	{ 0xd5ed, 0xd5f3, PDF_CMAP_RANGE, 17825 },
	{ 0xd5f4, 0xd5f5, PDF_CMAP_RANGE, 3327 },
	{ 0xd5f6, 0xd5f9, PDF_CMAP_TABLE, 3159 },
	{ 0xd5fa, 0xd5ff, PDF_CMAP_RANGE, 17834 },
	{ 0xd600, 0xd601, PDF_CMAP_RANGE, 3331 },
	{ 0xd602, 0xd603, PDF_CMAP_RANGE, 17840 },
	{ 0xd604, 0xd604, PDF_CMAP_SINGLE, 3333 },
	{ 0xd605, 0xd607, PDF_CMAP_RANGE, 17842 },
	{ 0xd608, 0xd608, PDF_CMAP_SINGLE, 3334 },
	{ 0xd609, 0xd60f, PDF_CMAP_RANGE, 17845 },
	{ 0xd610, 0xd611, PDF_CMAP_RANGE, 3335 },
	{ 0xd612, 0xd612, PDF_CMAP_SINGLE, 17852 },
	{ 0xd613, 0xd615, PDF_CMAP_RANGE, 3337 },
	{ 0xd616, 0xd61b, PDF_CMAP_RANGE, 17853 },
	{ 0xd61c, 0xd61c, PDF_CMAP_SINGLE, 3340 },
	{ 0xd61d, 0xd61f, PDF_CMAP_RANGE, 17859 },
	{ 0xd620, 0xd620, PDF_CMAP_SINGLE, 3341 },
	{ 0xd621, 0xd623, PDF_CMAP_RANGE, 17862 },
	{ 0xd624, 0xd624, PDF_CMAP_SINGLE, 3342 },
	{ 0xd625, 0xd62c, PDF_CMAP_RANGE, 17865 },
	{ 0xd62d, 0xd62d, PDF_CMAP_SINGLE, 3343 },
	{ 0xd62e, 0xd637, PDF_CMAP_RANGE, 17873 },
	{ 0xd638, 0xd639, PDF_CMAP_RANGE, 3344 },
	{ 0xd63a, 0xd63b, PDF_CMAP_RANGE, 17883 },
	{ 0xd63c, 0xd63c, PDF_CMAP_SINGLE, 3346 },
	{ 0xd63d, 0xd63f, PDF_CMAP_RANGE, 17885 },
	{ 0xd640, 0xd640, PDF_CMAP_SINGLE, 3347 },
	{ 0xd641, 0xd644, PDF_CMAP_RANGE, 17888 },
	{ 0xd645, 0xd645, PDF_CMAP_SINGLE, 3348 },
	{ 0xd646, 0xd647, PDF_CMAP_RANGE, 17892 },
	{ 0xd648, 0xd649, PDF_CMAP_RANGE, 3349 },
	{ 0xd64a, 0xd64d, PDF_CMAP_TABLE, 3163 },
	{ 0xd64e, 0xd650, PDF_CMAP_RANGE, 17896 },
	{ 0xd651, 0xd651, PDF_CMAP_SINGLE, 3353 },
	{ 0xd652, 0xd653, PDF_CMAP_RANGE, 17899 },
	{ 0xd654, 0xd655, PDF_CMAP_RANGE, 3354 },
	{ 0xd656, 0xd657, PDF_CMAP_RANGE, 17901 },
	{ 0xd658, 0xd658, PDF_CMAP_SINGLE, 3356 },
	{ 0xd659, 0xd65b, PDF_CMAP_RANGE, 17903 },
	{ 0xd65c, 0xd65c, PDF_CMAP_SINGLE, 3357 },
	{ 0xd65d, 0xd666, PDF_CMAP_RANGE, 17906 },
	{ 0xd667, 0xd669, PDF_CMAP_TABLE, 3167 },
	{ 0xd66a, 0xd66f, PDF_CMAP_RANGE, 17917 },
	{ 0xd670, 0xd671, PDF_CMAP_RANGE, 3360 },
	{ 0xd672, 0xd673, PDF_CMAP_RANGE, 17923 },
	{ 0xd674, 0xd674, PDF_CMAP_SINGLE, 3362 },
	{ 0xd675, 0xd682, PDF_CMAP_RANGE, 17925 },
	{ 0xd683, 0xd685, PDF_CMAP_TABLE, 3170 },
	{ 0xd686, 0xd68b, PDF_CMAP_RANGE, 17940 },
	{ 0xd68c, 0xd68d, PDF_CMAP_RANGE, 3365 },
	{ 0xd68e, 0xd68f, PDF_CMAP_RANGE, 17946 },
	{ 0xd690, 0xd690, PDF_CMAP_SINGLE, 3367 },
	{ 0xd691, 0xd693, PDF_CMAP_RANGE, 17948 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 3368 },
	{ 0xd695, 0xd69c, PDF_CMAP_RANGE, 17951 },
	{ 0xd69d, 0xd6a1, PDF_CMAP_TABLE, 3173 },
	{ 0xd6a2, 0xd6a7, PDF_CMAP_RANGE, 17961 },
	{ 0xd6a8, 0xd6a8, PDF_CMAP_SINGLE, 3372 },
	{ 0xd6a9, 0xd6ab, PDF_CMAP_RANGE, 17967 },
	{ 0xd6ac, 0xd6ac, PDF_CMAP_SINGLE, 3373 },
	{ 0xd6ad, 0xd6af, PDF_CMAP_RANGE, 17970 },
	{ 0xd6b0, 0xd6b0, PDF_CMAP_SINGLE, 3374 },
	{ 0xd6b1, 0xd6b8, PDF_CMAP_RANGE, 17973 },
	{ 0xd6b9, 0xd6bb, PDF_CMAP_TABLE, 3178 },
	{ 0xd6bc, 0xd6c3, PDF_CMAP_RANGE, 17982 },
	{ 0xd6c4, 0xd6c5, PDF_CMAP_RANGE, 3377 },
	{ 0xd6c6, 0xd6c7, PDF_CMAP_RANGE, 17990 },
	{ 0xd6c8, 0xd6c8, PDF_CMAP_SINGLE, 3379 },
	{ 0xd6c9, 0xd6cb, PDF_CMAP_RANGE, 17992 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 3380 },
	{ 0xd6cd, 0xd6d0, PDF_CMAP_RANGE, 17995 },
	{ 0xd6d1, 0xd6d1, PDF_CMAP_SINGLE, 3381 },
	{ 0xd6d2, 0xd6d3, PDF_CMAP_RANGE, 17999 },
	{ 0xd6d4, 0xd6d4, PDF_CMAP_SINGLE, 3382 },
	{ 0xd6d5, 0xd6d6, PDF_CMAP_RANGE, 18001 },
	{ 0xd6d7, 0xd6d9, PDF_CMAP_TABLE, 3181 },
	{ 0xd6da, 0xd6df, PDF_CMAP_RANGE, 18004 },
	{ 0xd6e0, 0xd6e0, PDF_CMAP_SINGLE, 3385 },
	{ 0xd6e1, 0xd6e3, PDF_CMAP_RANGE, 18010 },
	{ 0xd6e4, 0xd6e4, PDF_CMAP_SINGLE, 3386 },
	{ 0xd6e5, 0xd6e7, PDF_CMAP_RANGE, 18013 },
	{ 0xd6e8, 0xd6e8, PDF_CMAP_SINGLE, 3387 },
	{ 0xd6e9, 0xd6ef, PDF_CMAP_RANGE, 18016 },
	{ 0xd6f0, 0xd6f0, PDF_CMAP_SINGLE, 3388 },
	{ 0xd6f1, 0xd6f4, PDF_CMAP_RANGE, 18023 },
	{ 0xd6f5, 0xd6f5, PDF_CMAP_SINGLE, 3389 },
	{ 0xd6f6, 0xd6fb, PDF_CMAP_RANGE, 18027 },
	{ 0xd6fc, 0xd6fd, PDF_CMAP_RANGE, 3390 },
	{ 0xd6fe, 0xd6ff, PDF_CMAP_RANGE, 18033 },
	{ 0xd700, 0xd700, PDF_CMAP_SINGLE, 3392 },
	{ 0xd701, 0xd703, PDF_CMAP_RANGE, 18035 },
	{ 0xd704, 0xd704, PDF_CMAP_SINGLE, 3393 },
	{ 0xd705, 0xd710, PDF_CMAP_RANGE, 18038 },
	{ 0xd711, 0xd711, PDF_CMAP_SINGLE, 3394 },
	{ 0xd712, 0xd717, PDF_CMAP_RANGE, 18050 },
	{ 0xd718, 0xd719, PDF_CMAP_RANGE, 3395 },
	{ 0xd71a, 0xd71b, PDF_CMAP_RANGE, 18056 },
	{ 0xd71c, 0xd71c, PDF_CMAP_SINGLE, 3397 },
	{ 0xd71d, 0xd71f, PDF_CMAP_RANGE, 18058 },
	{ 0xd720, 0xd720, PDF_CMAP_SINGLE, 3398 },
	{ 0xd721, 0xd727, PDF_CMAP_RANGE, 18061 },
	{ 0xd728, 0xd729, PDF_CMAP_RANGE, 3399 },
	{ 0xd72a, 0xd72d, PDF_CMAP_TABLE, 3184 },
	{ 0xd72e, 0xd733, PDF_CMAP_RANGE, 18070 },
	{ 0xd734, 0xd735, PDF_CMAP_RANGE, 3403 },
	{ 0xd736, 0xd737, PDF_CMAP_RANGE, 18076 },
	{ 0xd738, 0xd738, PDF_CMAP_SINGLE, 3405 },
	{ 0xd739, 0xd73b, PDF_CMAP_RANGE, 18078 },
	{ 0xd73c, 0xd73c, PDF_CMAP_SINGLE, 3406 },
	{ 0xd73d, 0xd743, PDF_CMAP_RANGE, 18081 },
	{ 0xd744, 0xd744, PDF_CMAP_SINGLE, 3407 },
	{ 0xd745, 0xd746, PDF_CMAP_RANGE, 18088 },
	{ 0xd747, 0xd749, PDF_CMAP_TABLE, 3188 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 18091 },
	{ 0xd750, 0xd751, PDF_CMAP_RANGE, 3410 },
	{ 0xd752, 0xd753, PDF_CMAP_RANGE, 18097 },
	{ 0xd754, 0xd755, PDF_CMAP_TABLE, 3191 },
	{ 0xd756, 0xd759, PDF_CMAP_RANGE, 3413 },
	{ 0xd75a, 0xd75f, PDF_CMAP_RANGE, 18100 },
	{ 0xd760, 0xd761, PDF_CMAP_RANGE, 3417 },
	{ 0xd762, 0xd765, PDF_CMAP_TABLE, 3193 },
	{ 0xd766, 0xd768, PDF_CMAP_RANGE, 18108 },
	{ 0xd769, 0xd769, PDF_CMAP_SINGLE, 3421 },
	{ 0xd76a, 0xd76b, PDF_CMAP_RANGE, 18111 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 3422 },
	{ 0xd76d, 0xd76f, PDF_CMAP_RANGE, 18113 },
	{ 0xd770, 0xd770, PDF_CMAP_SINGLE, 3423 },
	{ 0xd771, 0xd773, PDF_CMAP_RANGE, 18116 },
	{ 0xd774, 0xd774, PDF_CMAP_SINGLE, 3424 },
	{ 0xd775, 0xd77b, PDF_CMAP_RANGE, 18119 },
	{ 0xd77c, 0xd77d, PDF_CMAP_RANGE, 3425 },
	{ 0xd77e, 0xd780, PDF_CMAP_RANGE, 18126 },
	{ 0xd781, 0xd781, PDF_CMAP_SINGLE, 3427 },
	{ 0xd782, 0xd787, PDF_CMAP_RANGE, 18129 },
	{ 0xd788, 0xd789, PDF_CMAP_RANGE, 3428 },
	{ 0xd78a, 0xd78b, PDF_CMAP_RANGE, 18135 },
	{ 0xd78c, 0xd78c, PDF_CMAP_SINGLE, 3430 },
	{ 0xd78d, 0xd78f, PDF_CMAP_RANGE, 18137 },
	{ 0xd790, 0xd790, PDF_CMAP_SINGLE, 3431 },
	{ 0xd791, 0xd797, PDF_CMAP_RANGE, 18140 },
	{ 0xd798, 0xd799, PDF_CMAP_RANGE, 3432 },
	{ 0xd79a, 0xd79d, PDF_CMAP_TABLE, 3197 },
	{ 0xd79e, 0xd7a3, PDF_CMAP_RANGE, 18149 },
	{ 0xf900, 0xf90c, PDF_CMAP_TABLE, 3201 },
	{ 0xf90d, 0xf913, PDF_CMAP_RANGE, 4375 },
	{ 0xf914, 0xf914, PDF_CMAP_SINGLE, 5800 },
	{ 0xf915, 0xf917, PDF_CMAP_RANGE, 4382 },
	{ 0xf918, 0xf91d, PDF_CMAP_RANGE, 4386 },
	{ 0xf91e, 0xf920, PDF_CMAP_RANGE, 4394 },
	{ 0xf921, 0xf922, PDF_CMAP_TABLE, 3214 },
	{ 0xf923, 0xf924, PDF_CMAP_RANGE, 4406 },
	{ 0xf925, 0xf92b, PDF_CMAP_RANGE, 4409 },
	{ 0xf92c, 0xf92d, PDF_CMAP_RANGE, 4419 },
	{ 0xf92e, 0xf92f, PDF_CMAP_TABLE, 3216 },
	{ 0xf930, 0xf931, PDF_CMAP_RANGE, 4513 },
	{ 0xf932, 0xf937, PDF_CMAP_RANGE, 4517 },
	{ 0xf938, 0xf93a, PDF_CMAP_RANGE, 4524 },
	{ 0xf93b, 0xf940, PDF_CMAP_RANGE, 4528 },
	{ 0xf941, 0xf943, PDF_CMAP_RANGE, 4535 },
	{ 0xf944, 0xf945, PDF_CMAP_RANGE, 4541 },
	{ 0xf946, 0xf948, PDF_CMAP_RANGE, 4545 },
	{ 0xf949, 0xf94a, PDF_CMAP_TABLE, 3218 },
	{ 0xf94b, 0xf94e, PDF_CMAP_RANGE, 4566 },
	{ 0xf94f, 0xf950, PDF_CMAP_RANGE, 4571 },
	{ 0xf951, 0xf951, PDF_CMAP_SINGLE, 4576 },
	{ 0xf952, 0xf955, PDF_CMAP_RANGE, 4604 },
	{ 0xf956, 0xf959, PDF_CMAP_RANGE, 4609 },
	{ 0xf95a, 0xf974, PDF_CMAP_TABLE, 3220 },
	{ 0xf975, 0xf977, PDF_CMAP_RANGE, 4425 },
	{ 0xf978, 0xf97a, PDF_CMAP_RANGE, 4429 },
	{ 0xf97b, 0xf97d, PDF_CMAP_RANGE, 4435 },
	{ 0xf97e, 0xf97e, PDF_CMAP_SINGLE, 4439 },
	{ 0xf97f, 0xf980, PDF_CMAP_RANGE, 4442 },
	{ 0xf981, 0xf983, PDF_CMAP_TABLE, 3247 },
	{ 0xf984, 0xf985, PDF_CMAP_RANGE, 4449 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 4453 },
	{ 0xf987, 0xf98c, PDF_CMAP_RANGE, 4455 },
	{ 0xf98d, 0xf98e, PDF_CMAP_TABLE, 3250 },
	{ 0xf98f, 0xf990, PDF_CMAP_RANGE, 4465 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 4161 },
	{ 0xf992, 0xf994, PDF_CMAP_RANGE, 4468 },
	{ 0xf995, 0xf995, PDF_CMAP_SINGLE, 4162 },
	{ 0xf996, 0xf997, PDF_CMAP_RANGE, 4471 },
	{ 0xf998, 0xf999, PDF_CMAP_TABLE, 3252 },
	{ 0xf99a, 0xf99b, PDF_CMAP_RANGE, 4475 },
	{ 0xf99c, 0xf99d, PDF_CMAP_RANGE, 4478 },
	{ 0xf99e, 0xf99e, PDF_CMAP_SINGLE, 6447 },
	{ 0xf99f, 0xf9a0, PDF_CMAP_RANGE, 4481 },
	{ 0xf9a1, 0xf9a5, PDF_CMAP_TABLE, 3254 },
	{ 0xf9a6, 0xf9a8, PDF_CMAP_RANGE, 4487 },
	{ 0xf9a9, 0xf9aa, PDF_CMAP_TABLE, 3259 },
	{ 0xf9ab, 0xf9ad, PDF_CMAP_RANGE, 4493 },
	{ 0xf9ae, 0xf9b0, PDF_CMAP_TABLE, 3261 },
	{ 0xf9b1, 0xf9b4, PDF_CMAP_RANGE, 4501 },
	{ 0xf9b5, 0xf9b5, PDF_CMAP_SINGLE, 4506 },
	{ 0xf9b6, 0xf9b8, PDF_CMAP_RANGE, 4508 },
	{ 0xf9b9, 0xf9b9, PDF_CMAP_SINGLE, 5797 },
	{ 0xf9ba, 0xf9bc, PDF_CMAP_RANGE, 4551 },
	{ 0xf9bd, 0xf9bf, PDF_CMAP_TABLE, 3264 },
	{ 0xf9c0, 0xf9c1, PDF_CMAP_RANGE, 4556 },
	{ 0xf9c2, 0xf9c3, PDF_CMAP_RANGE, 4560 },
	{ 0xf9c4, 0xf9c9, PDF_CMAP_TABLE, 3267 },
	{ 0xf9ca, 0xf9cb, PDF_CMAP_RANGE, 4581 },
	{ 0xf9cc, 0xf9cf, PDF_CMAP_TABLE, 3273 },
	{ 0xf9d0, 0xf9d3, PDF_CMAP_RANGE, 4590 },
	{ 0xf9d4, 0xf9d6, PDF_CMAP_RANGE, 4595 },
	{ 0xf9d7, 0xf9da, PDF_CMAP_RANGE, 4599 },
	{ 0xf9db, 0xf9e0, PDF_CMAP_TABLE, 3277 },
	{ 0xf9e1, 0xf9e2, PDF_CMAP_RANGE, 4620 },
	{ 0xf9e3, 0xf9e6, PDF_CMAP_TABLE, 3283 },
	{ 0xf9e7, 0xf9e9, PDF_CMAP_RANGE, 4632 },
	{ 0xf9ea, 0xf9ea, PDF_CMAP_SINGLE, 4636 },
	{ 0xf9eb, 0xf9ec, PDF_CMAP_RANGE, 4188 },
	{ 0xf9ed, 0xf9ed, PDF_CMAP_SINGLE, 4638 },
	{ 0xf9ee, 0xf9f0, PDF_CMAP_RANGE, 4640 },
	{ 0xf9f1, 0xf9f5, PDF_CMAP_RANGE, 4644 },
	{ 0xf9f6, 0xf9f6, PDF_CMAP_SINGLE, 4650 },
	{ 0xf9f7, 0xf9f9, PDF_CMAP_RANGE, 4653 },
	{ 0xf9fa, 0xfa0b, PDF_CMAP_TABLE, 3287 },
	{ 0xff01, 0xff3b, PDF_CMAP_RANGE, 264 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 112 },
	{ 0xff3d, 0xff5d, PDF_CMAP_RANGE, 324 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 113 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 143 },
	{ 0xffe2, 0xffe3, PDF_CMAP_TABLE, 3305 },
	{ 0xffe5, 0xffe6, PDF_CMAP_TABLE, 3307 },
	{ 0xbca5, 0xbca6, PDF_CMAP_RANGE, 12649 },
	{ 0xbca7, 0xbca8, PDF_CMAP_RANGE, 2031 },
	{ 0xbca9, 0xbcaf, PDF_CMAP_RANGE, 12651 },
	{ 0xbcb0, 0xbcb1, PDF_CMAP_RANGE, 2033 },
	{ 0xbcb2, 0xbcb2, PDF_CMAP_SINGLE, 12658 },
	{ 0xbcb3, 0xbcb5, PDF_CMAP_RANGE, 2035 },
	{ 0xbcb6, 0xbcbb, PDF_CMAP_RANGE, 12659 },
	{ 0xbcbc, 0xbcbd, PDF_CMAP_RANGE, 2038 },
	{ 0xbcbe, 0xbcbf, PDF_CMAP_RANGE, 12665 },
	{ 0xbcc0, 0xbcc0, PDF_CMAP_SINGLE, 2040 },
	{ 0xbcc1, 0xbcc3, PDF_CMAP_RANGE, 12667 },
	{ 0xbcc4, 0xbcc4, PDF_CMAP_SINGLE, 2041 },
	{ 0xbcc5, 0xbccc, PDF_CMAP_RANGE, 12670 },
	{ 0xbccd, 0xbccd, PDF_CMAP_SINGLE, 2042 },
	{ 0xbcce, 0xbcce, PDF_CMAP_SINGLE, 12678 },
	{ 0xbccf, 0xbcd1, PDF_CMAP_RANGE, 2043 },
	{ 0xbcd2, 0xbcd4, PDF_CMAP_RANGE, 12679 },
	{ 0xbcd5, 0xbcd5, PDF_CMAP_SINGLE, 2046 },
	{ 0xbcd6, 0xbcd7, PDF_CMAP_RANGE, 12682 },
	{ 0xbcd8, 0xbcd8, PDF_CMAP_SINGLE, 2047 },
	{ 0xbcd9, 0xbcdb, PDF_CMAP_RANGE, 12684 },
	{ 0xbcdc, 0xbcdc, PDF_CMAP_SINGLE, 2048 },
	{ 0xbcdd, 0xbcf3, PDF_CMAP_RANGE, 12687 },
	{ 0xbcf4, 0xbcf6, PDF_CMAP_RANGE, 2049 },
	{ 0xbcf7, 0xbcf7, PDF_CMAP_SINGLE, 12710 },
	{ 0xbcf8, 0xbcf8, PDF_CMAP_SINGLE, 2052 },
	{ 0xbcf9, 0xbcfb, PDF_CMAP_RANGE, 12711 },
	{ 0xbcfc, 0xbcfc, PDF_CMAP_SINGLE, 2053 },
	{ 0xbcfd, 0xbcff, PDF_CMAP_RANGE, 12714 },
	{ 0xbd00, 0xbd03, PDF_CMAP_RANGE, 12717 },
	{ 0xbd04, 0xbd05, PDF_CMAP_RANGE, 2054 },
	{ 0xbd06, 0xbd06, PDF_CMAP_SINGLE, 12721 },
	{ 0xbd07, 0xbd07, PDF_CMAP_SINGLE, 2056 },
	{ 0xbd08, 0xbd08, PDF_CMAP_SINGLE, 12722 },
	{ 0xbd09, 0xbd09, PDF_CMAP_SINGLE, 2057 },
	{ 0xbd0a, 0xbd0f, PDF_CMAP_RANGE, 12723 },
	{ 0xbd10, 0xbd10, PDF_CMAP_SINGLE, 2058 },
	{ 0xbd11, 0xbd13, PDF_CMAP_RANGE, 12729 },
	{ 0xbd14, 0xbd14, PDF_CMAP_SINGLE, 2059 },
	{ 0xbd15, 0xbd23, PDF_CMAP_RANGE, 12732 },
	{ 0xbd24, 0xbd24, PDF_CMAP_SINGLE, 2060 },
	{ 0xbd25, 0xbd2b, PDF_CMAP_RANGE, 12747 },
	{ 0xbd2c, 0xbd2c, PDF_CMAP_SINGLE, 2061 },
	{ 0xbd2d, 0xbd3f, PDF_CMAP_RANGE, 12754 },
	{ 0xbd40, 0xbd40, PDF_CMAP_SINGLE, 2062 },
	{ 0xbd41, 0xbd47, PDF_CMAP_RANGE, 12773 },
	{ 0xbd48, 0xbd49, PDF_CMAP_RANGE, 2063 },
	{ 0xbd4a, 0xbd4b, PDF_CMAP_RANGE, 12780 },
	{ 0xbd4c, 0xbd4c, PDF_CMAP_SINGLE, 2065 },
	{ 0xbd4d, 0xbd4f, PDF_CMAP_RANGE, 12782 },
	{ 0xbd50, 0xbd50, PDF_CMAP_SINGLE, 2066 },
	{ 0xbd51, 0xbd57, PDF_CMAP_RANGE, 12785 },
	{ 0xbd58, 0xbd59, PDF_CMAP_RANGE, 2067 },
	{ 0xbd5a, 0xbd63, PDF_CMAP_RANGE, 12792 },
	{ 0xbd64, 0xbd64, PDF_CMAP_SINGLE, 2069 },
	{ 0xbd65, 0xbd67, PDF_CMAP_RANGE, 12802 },
	{ 0xbd68, 0xbd68, PDF_CMAP_SINGLE, 2070 },
	{ 0xbd69, 0xbd7f, PDF_CMAP_RANGE, 12805 },
	{ 0xbd80, 0xbd81, PDF_CMAP_RANGE, 2071 },
	{ 0xbd82, 0xbd83, PDF_CMAP_RANGE, 12828 },
	{ 0xbd84, 0xbd84, PDF_CMAP_SINGLE, 2073 },
	{ 0xbd85, 0xbd86, PDF_CMAP_RANGE, 12830 },
	{ 0xbd87, 0xbd8a, PDF_CMAP_RANGE, 2074 },
	{ 0xbd8b, 0xbd8f, PDF_CMAP_RANGE, 12832 },
	{ 0xbd90, 0xbd91, PDF_CMAP_RANGE, 2078 },
	{ 0xbd92, 0xbd92, PDF_CMAP_SINGLE, 12837 },
	{ 0xbd93, 0xbd93, PDF_CMAP_SINGLE, 2080 },
	{ 0xbd94, 0xbd94, PDF_CMAP_SINGLE, 12838 },
	{ 0xbd95, 0xbd95, PDF_CMAP_SINGLE, 2081 },
	{ 0xbd96, 0xbd98, PDF_CMAP_RANGE, 12839 },
	{ 0xbd99, 0xbd9a, PDF_CMAP_RANGE, 2082 },
	{ 0xbd9b, 0xbd9b, PDF_CMAP_SINGLE, 12842 },
	{ 0xbd9c, 0xbd9c, PDF_CMAP_SINGLE, 2084 },
	{ 0xbd9d, 0xbda3, PDF_CMAP_RANGE, 12843 },
	{ 0xbda4, 0xbda4, PDF_CMAP_SINGLE, 2085 },
	{ 0xbda5, 0xbdaf, PDF_CMAP_RANGE, 12850 },
	{ 0xbdb0, 0xbdb0, PDF_CMAP_SINGLE, 2086 },
	{ 0xbdb1, 0xbdb7, PDF_CMAP_RANGE, 12861 },
	{ 0xbdb8, 0xbdb8, PDF_CMAP_SINGLE, 2087 },
	{ 0xbdb9, 0xbdd3, PDF_CMAP_RANGE, 12868 },
	{ 0xbdd4, 0xbdd5, PDF_CMAP_RANGE, 2088 },
	{ 0xbdd6, 0xbdd7, PDF_CMAP_RANGE, 12895 },
	{ 0xbdd8, 0xbdd8, PDF_CMAP_SINGLE, 2090 },
	{ 0xbdd9, 0xbddb, PDF_CMAP_RANGE, 12897 },
	{ 0xbddc, 0xbddc, PDF_CMAP_SINGLE, 2091 },
	{ 0xbddd, 0xbde8, PDF_CMAP_RANGE, 12900 },
	{ 0xbde9, 0xbde9, PDF_CMAP_SINGLE, 2092 },
	{ 0xbdea, 0xbdef, PDF_CMAP_RANGE, 12912 },
	{ 0xbdf0, 0xbdf0, PDF_CMAP_SINGLE, 2093 },
	{ 0xbdf1, 0xbdf3, PDF_CMAP_RANGE, 12918 },
	{ 0xbdf4, 0xbdf4, PDF_CMAP_SINGLE, 2094 },
	{ 0xbdf5, 0xbdf7, PDF_CMAP_RANGE, 12921 },
	{ 0xbdf8, 0xbdf8, PDF_CMAP_SINGLE, 2095 },
	{ 0xbdf9, 0xbdff, PDF_CMAP_RANGE, 12924 },
	{ 0xbe00, 0xbe00, PDF_CMAP_SINGLE, 2096 },
	{ 0xbe01, 0xbe02, PDF_CMAP_RANGE, 12931 },
	{ 0xbe03, 0xbe03, PDF_CMAP_SINGLE, 2097 },
	{ 0xbe04, 0xbe04, PDF_CMAP_SINGLE, 12933 },
	{ 0xbe05, 0xbe05, PDF_CMAP_SINGLE, 2098 },
	{ 0xbe06, 0xbe0b, PDF_CMAP_RANGE, 12934 },
	{ 0xbe0c, 0xbe0d, PDF_CMAP_RANGE, 2099 },
	{ 0xbe0e, 0xbe0f, PDF_CMAP_RANGE, 12940 },
	{ 0xbe10, 0xbe10, PDF_CMAP_SINGLE, 2101 },
	{ 0xbe11, 0xbe13, PDF_CMAP_RANGE, 12942 },
	{ 0xbe14, 0xbe14, PDF_CMAP_SINGLE, 2102 },
	{ 0xbe15, 0xbe1b, PDF_CMAP_RANGE, 12945 },
	{ 0xbe1c, 0xbe1d, PDF_CMAP_RANGE, 2103 },
	{ 0xbe1e, 0xbe1e, PDF_CMAP_SINGLE, 12952 },
	{ 0xbe1f, 0xbe1f, PDF_CMAP_SINGLE, 2105 },
	{ 0xbe20, 0xbe43, PDF_CMAP_RANGE, 12953 },
	{ 0xbe44, 0xbe45, PDF_CMAP_RANGE, 2106 },
	{ 0xbe46, 0xbe47, PDF_CMAP_RANGE, 12989 },
	{ 0xbe48, 0xbe48, PDF_CMAP_SINGLE, 2108 },
	{ 0xbe49, 0xbe4b, PDF_CMAP_RANGE, 12991 },
	{ 0xbe4c, 0xbe4c, PDF_CMAP_SINGLE, 2109 },
	{ 0xbe4d, 0xbe4d, PDF_CMAP_SINGLE, 12994 },
	{ 0xbe4e, 0xbe4e, PDF_CMAP_SINGLE, 2110 },
	{ 0xbe4f, 0xbe53, PDF_CMAP_RANGE, 12995 },
	{ 0xbe54, 0xbe55, PDF_CMAP_RANGE, 2111 },
	{ 0xbe56, 0xbe56, PDF_CMAP_SINGLE, 13000 },
	{ 0xbe57, 0xbe57, PDF_CMAP_SINGLE, 2113 },
	{ 0xbe58, 0xbe58, PDF_CMAP_SINGLE, 13001 },
	{ 0xbe59, 0xbe5b, PDF_CMAP_RANGE, 2114 },
	{ 0xbe5c, 0xbe5f, PDF_CMAP_RANGE, 13002 },
	{ 0xbe60, 0xbe61, PDF_CMAP_RANGE, 2117 },
	{ 0xbe62, 0xbe63, PDF_CMAP_RANGE, 13006 },
	{ 0xbe64, 0xbe64, PDF_CMAP_SINGLE, 2119 },
	{ 0xbe65, 0xbe67, PDF_CMAP_RANGE, 13008 },
	{ 0xbe68, 0xbe68, PDF_CMAP_SINGLE, 2120 },
	{ 0xbe69, 0xbe69, PDF_CMAP_SINGLE, 13011 },
	{ 0xbe6a, 0xbe6a, PDF_CMAP_SINGLE, 2121 },
	{ 0xbe6b, 0xbe6f, PDF_CMAP_RANGE, 13012 },
	{ 0xbe70, 0xbe71, PDF_CMAP_RANGE, 2122 },
	{ 0xbe72, 0xbe72, PDF_CMAP_SINGLE, 13017 },
	{ 0xbe73, 0xbe75, PDF_CMAP_RANGE, 2124 },
	{ 0xbe76, 0xbe7a, PDF_CMAP_RANGE, 13018 },
	{ 0xbe7b, 0xbe7d, PDF_CMAP_RANGE, 2127 },
	{ 0xbe7e, 0xbe7f, PDF_CMAP_RANGE, 13023 },
	{ 0xbe80, 0xbe80, PDF_CMAP_SINGLE, 2130 },
	{ 0xbe81, 0xbe83, PDF_CMAP_RANGE, 13025 },
	{ 0xbe84, 0xbe84, PDF_CMAP_SINGLE, 2131 },
	{ 0xbe85, 0xbe8b, PDF_CMAP_RANGE, 13028 },
	{ 0xbe8c, 0xbe8d, PDF_CMAP_RANGE, 2132 },
	{ 0xbe8e, 0xbe8e, PDF_CMAP_SINGLE, 13035 },
	{ 0xbe8f, 0xbe91, PDF_CMAP_RANGE, 2134 },
	{ 0xbe92, 0xbe97, PDF_CMAP_RANGE, 13036 },
	{ 0xbe98, 0xbe99, PDF_CMAP_RANGE, 2137 },
	{ 0xbe9a, 0xbea7, PDF_CMAP_RANGE, 13042 },
	{ 0xbea8, 0xbea8, PDF_CMAP_SINGLE, 2139 },
	{ 0xbea9, 0xbecf, PDF_CMAP_RANGE, 13056 },
	{ 0xbed0, 0xbed1, PDF_CMAP_RANGE, 2140 },
	{ 0xbed2, 0xbed3, PDF_CMAP_RANGE, 13095 },
	{ 0xbed4, 0xbed4, PDF_CMAP_SINGLE, 2142 },
	{ 0xbed5, 0xbed6, PDF_CMAP_RANGE, 13097 },
	{ 0xbed7, 0xbed8, PDF_CMAP_RANGE, 2143 },
	{ 0xbed9, 0xbedf, PDF_CMAP_RANGE, 13099 },
	{ 0xbee0, 0xbee0, PDF_CMAP_SINGLE, 2145 },
	{ 0xbee1, 0xbee2, PDF_CMAP_RANGE, 13106 },
	{ 0xbee3, 0xbee5, PDF_CMAP_RANGE, 2146 },
	{ 0xbee6, 0xbeeb, PDF_CMAP_RANGE, 13108 },
	{ 0xbeec, 0xbeec, PDF_CMAP_SINGLE, 2149 },
	{ 0xbeed, 0xbeff, PDF_CMAP_RANGE, 13114 },
	{ 0xbf00, 0xbf00, PDF_CMAP_SINGLE, 13133 },
	{ 0xbf01, 0xbf01, PDF_CMAP_SINGLE, 2150 },
	{ 0xbf02, 0xbf07, PDF_CMAP_RANGE, 13134 },
	{ 0xbf08, 0xbf09, PDF_CMAP_RANGE, 2151 },
	{ 0xbf0a, 0xbf17, PDF_CMAP_RANGE, 13140 },
	{ 0xbf18, 0xbf19, PDF_CMAP_RANGE, 2153 },
	{ 0xbf1a, 0xbf1a, PDF_CMAP_SINGLE, 13154 },
	{ 0xbf1b, 0xbf1d, PDF_CMAP_RANGE, 2155 },
	{ 0xbf1e, 0xbf3f, PDF_CMAP_RANGE, 13155 },
	{ 0xbf40, 0xbf41, PDF_CMAP_RANGE, 2158 },
	{ 0xbf42, 0xbf43, PDF_CMAP_RANGE, 13189 },
	{ 0xbf44, 0xbf44, PDF_CMAP_SINGLE, 2160 },
	{ 0xbf45, 0xbf47, PDF_CMAP_RANGE, 13191 },
	{ 0xbf48, 0xbf48, PDF_CMAP_SINGLE, 2161 },
	{ 0xbf49, 0xbf4f, PDF_CMAP_RANGE, 13194 },
	{ 0xbf50, 0xbf51, PDF_CMAP_RANGE, 2162 },
	{ 0xbf52, 0xbf54, PDF_CMAP_RANGE, 13201 },
	{ 0xbf55, 0xbf55, PDF_CMAP_SINGLE, 2164 },
	{ 0xbf56, 0xbf93, PDF_CMAP_RANGE, 13204 },
	{ 0xbf94, 0xbf94, PDF_CMAP_SINGLE, 2165 },
	{ 0xbf95, 0xbfaf, PDF_CMAP_RANGE, 13266 },
	{ 0xbfb0, 0xbfb0, PDF_CMAP_SINGLE, 2166 },
	{ 0xbfb1, 0xbfc4, PDF_CMAP_RANGE, 13293 },
	{ 0xbfc5, 0xbfc5, PDF_CMAP_SINGLE, 2167 },
	{ 0xbfc6, 0xbfcb, PDF_CMAP_RANGE, 13313 },
	{ 0xbfcc, 0xbfcd, PDF_CMAP_RANGE, 2168 },
	{ 0xbfce, 0xbfcf, PDF_CMAP_RANGE, 13319 },
	{ 0xbfd0, 0xbfd0, PDF_CMAP_SINGLE, 2170 },
	{ 0xbfd1, 0xbfd3, PDF_CMAP_RANGE, 13321 },
	{ 0xbfd4, 0xbfd4, PDF_CMAP_SINGLE, 2171 },
	{ 0xbfd5, 0xbfdb, PDF_CMAP_RANGE, 13324 },
	{ 0xbfdc, 0xbfdc, PDF_CMAP_SINGLE, 2172 },
	{ 0xbfdd, 0xbfde, PDF_CMAP_RANGE, 13331 },
	{ 0xbfdf, 0xbfdf, PDF_CMAP_SINGLE, 2173 },
	{ 0xbfe0, 0xbfe0, PDF_CMAP_SINGLE, 13333 },
	{ 0xbfe1, 0xbfe1, PDF_CMAP_SINGLE, 2174 },
	{ 0xbfe2, 0xbfff, PDF_CMAP_RANGE, 13334 },
	{ 0xc000, 0xc03b, PDF_CMAP_RANGE, 13364 },
	{ 0xc03c, 0xc03c, PDF_CMAP_SINGLE, 2175 },
	{ 0xc03d, 0xc050, PDF_CMAP_RANGE, 13424 },
	{ 0xc051, 0xc051, PDF_CMAP_SINGLE, 2176 },
	{ 0xc052, 0xc057, PDF_CMAP_RANGE, 13444 },
	{ 0xc058, 0xc058, PDF_CMAP_SINGLE, 2177 },
	{ 0xc059, 0xc05b, PDF_CMAP_RANGE, 13450 },
	{ 0xc05c, 0xc05c, PDF_CMAP_SINGLE, 2178 },
	{ 0xc05d, 0xc05f, PDF_CMAP_RANGE, 13453 },
	{ 0xc060, 0xc060, PDF_CMAP_SINGLE, 2179 },
	{ 0xc061, 0xc067, PDF_CMAP_RANGE, 13456 },
	{ 0xc068, 0xc069, PDF_CMAP_RANGE, 2180 },
	{ 0xc06a, 0xc08f, PDF_CMAP_RANGE, 13463 },
	{ 0xc090, 0xc091, PDF_CMAP_RANGE, 2182 },
	{ 0xc092, 0xc093, PDF_CMAP_RANGE, 13501 },
	{ 0xc094, 0xc094, PDF_CMAP_SINGLE, 2184 },
	{ 0xc095, 0xc097, PDF_CMAP_RANGE, 13503 },
	{ 0xc098, 0xc098, PDF_CMAP_SINGLE, 2185 },
	{ 0xc099, 0xc09f, PDF_CMAP_RANGE, 13506 },
	{ 0xc0a0, 0xc0a1, PDF_CMAP_RANGE, 2186 },
	{ 0xc0a2, 0xc0a2, PDF_CMAP_SINGLE, 13513 },
	{ 0xc0a3, 0xc0a3, PDF_CMAP_SINGLE, 2188 },
	{ 0xc0a4, 0xc0a4, PDF_CMAP_SINGLE, 13514 },
	{ 0xc0a5, 0xc0a5, PDF_CMAP_SINGLE, 2189 },
	{ 0xc0a6, 0xc0ab, PDF_CMAP_RANGE, 13515 },
	{ 0xc0ac, 0xc0ad, PDF_CMAP_RANGE, 2190 },
	{ 0xc0ae, 0xc0ae, PDF_CMAP_SINGLE, 13521 },
	{ 0xc0af, 0xc0b0, PDF_CMAP_RANGE, 2192 },
	{ 0xc0b1, 0xc0b2, PDF_CMAP_RANGE, 13522 },
	{ 0xc0b3, 0xc0b6, PDF_CMAP_RANGE, 2194 },
	{ 0xc0b7, 0xc0bb, PDF_CMAP_RANGE, 13524 },
	{ 0xc0bc, 0xc0bd, PDF_CMAP_RANGE, 2198 },
	{ 0xc0be, 0xc0be, PDF_CMAP_SINGLE, 13529 },
	{ 0xc0bf, 0xc0c1, PDF_CMAP_RANGE, 2200 },
	{ 0xc0c2, 0xc0c4, PDF_CMAP_RANGE, 13530 },
	{ 0xc0c5, 0xc0c5, PDF_CMAP_SINGLE, 2203 },
	{ 0xc0c6, 0xc0c7, PDF_CMAP_RANGE, 13533 },
	{ 0xc0c8, 0xc0c9, PDF_CMAP_RANGE, 2204 },
	{ 0xc0ca, 0xc0cb, PDF_CMAP_RANGE, 13535 },
	{ 0xc0cc, 0xc0cc, PDF_CMAP_SINGLE, 2206 },
	{ 0xc0cd, 0xc0cf, PDF_CMAP_RANGE, 13537 },
	{ 0xc0d0, 0xc0d0, PDF_CMAP_SINGLE, 2207 },
	{ 0xc0d1, 0xc0d7, PDF_CMAP_RANGE, 13540 },
	{ 0xc0d8, 0xc0d9, PDF_CMAP_RANGE, 2208 },
	{ 0xc0da, 0xc0da, PDF_CMAP_SINGLE, 13547 },
	{ 0xc0db, 0xc0dd, PDF_CMAP_RANGE, 2210 },
	{ 0xc0de, 0xc0e3, PDF_CMAP_RANGE, 13548 },
	{ 0xc0e4, 0xc0e5, PDF_CMAP_RANGE, 2213 },
	{ 0xc0e6, 0xc0e7, PDF_CMAP_RANGE, 13554 },
	{ 0xc0e8, 0xc0e8, PDF_CMAP_SINGLE, 2215 },
	{ 0xc0e9, 0xc0eb, PDF_CMAP_RANGE, 13556 },
	{ 0xc0ec, 0xc0ec, PDF_CMAP_SINGLE, 2216 },
	{ 0xc0ed, 0xc0f3, PDF_CMAP_RANGE, 13559 },
	{ 0xc0f4, 0xc0f5, PDF_CMAP_RANGE, 2217 },
	{ 0xc0f6, 0xc0f6, PDF_CMAP_SINGLE, 13566 },
	{ 0xc0f7, 0xc0f7, PDF_CMAP_SINGLE, 2219 },
	{ 0xc0f8, 0xc0f8, PDF_CMAP_SINGLE, 13567 },
	{ 0xc0f9, 0xc0f9, PDF_CMAP_SINGLE, 2220 },
	{ 0xc0fa, 0xc0ff, PDF_CMAP_RANGE, 13568 },
	{ 0xc100, 0xc100, PDF_CMAP_SINGLE, 2221 },
	{ 0xc101, 0xc103, PDF_CMAP_RANGE, 13574 },
	{ 0xc104, 0xc104, PDF_CMAP_SINGLE, 2222 },
	{ 0xc105, 0xc107, PDF_CMAP_RANGE, 13577 },
	{ 0xc108, 0xc108, PDF_CMAP_SINGLE, 2223 },
	{ 0xc109, 0xc10f, PDF_CMAP_RANGE, 13580 },
	{ 0xc110, 0xc110, PDF_CMAP_SINGLE, 2224 },
	{ 0xc111, 0xc114, PDF_CMAP_RANGE, 13587 },
	{ 0xc115, 0xc115, PDF_CMAP_SINGLE, 2225 },
	{ 0xc116, 0xc11b, PDF_CMAP_RANGE, 13591 },
	{ 0xc11c, 0xc120, PDF_CMAP_RANGE, 2226 },
	{ 0xc121, 0xc122, PDF_CMAP_RANGE, 13597 },
	{ 0xc123, 0xc124, PDF_CMAP_RANGE, 2231 },
	{ 0xc125, 0xc125, PDF_CMAP_SINGLE, 13599 },
	{ 0xc126, 0xc127, PDF_CMAP_RANGE, 2233 },
	{ 0xc128, 0xc12b, PDF_CMAP_RANGE, 13600 },
	{ 0xc12c, 0xc12d, PDF_CMAP_RANGE, 2235 },
	{ 0xc12e, 0xc12e, PDF_CMAP_SINGLE, 13604 },
	{ 0xc12f, 0xc131, PDF_CMAP_RANGE, 2237 },
	{ 0xc132, 0xc135, PDF_CMAP_RANGE, 13605 },
	{ 0xc136, 0xc136, PDF_CMAP_SINGLE, 2240 },
	{ 0xc137, 0xc137, PDF_CMAP_SINGLE, 13609 },
	{ 0xc138, 0xc139, PDF_CMAP_RANGE, 2241 },
	{ 0xc13a, 0xc13b, PDF_CMAP_RANGE, 13610 },
	{ 0xc13c, 0xc13c, PDF_CMAP_SINGLE, 2243 },
	{ 0xc13d, 0xc13f, PDF_CMAP_RANGE, 13612 },
	{ 0xc140, 0xc140, PDF_CMAP_SINGLE, 2244 },
	{ 0xc141, 0xc147, PDF_CMAP_RANGE, 13615 },
	{ 0xc148, 0xc149, PDF_CMAP_RANGE, 2245 },
	{ 0xc14a, 0xc14a, PDF_CMAP_SINGLE, 13622 },
	{ 0xc14b, 0xc14d, PDF_CMAP_RANGE, 2247 },
	{ 0xc14e, 0xc153, PDF_CMAP_RANGE, 13623 },
	{ 0xc154, 0xc155, PDF_CMAP_RANGE, 2250 },
	{ 0xc156, 0xc157, PDF_CMAP_RANGE, 13629 },
	{ 0xc158, 0xc158, PDF_CMAP_SINGLE, 2252 },
	{ 0xc159, 0xc15b, PDF_CMAP_RANGE, 13631 },
	{ 0xc15c, 0xc15c, PDF_CMAP_SINGLE, 2253 },
	{ 0xc15d, 0xc163, PDF_CMAP_RANGE, 13634 },
	{ 0xc164, 0xc165, PDF_CMAP_RANGE, 2254 },
	{ 0xc166, 0xc166, PDF_CMAP_SINGLE, 13641 },
	{ 0xc167, 0xc169, PDF_CMAP_RANGE, 2256 },
	{ 0xc16a, 0xc16f, PDF_CMAP_RANGE, 13642 },
	{ 0xc170, 0xc170, PDF_CMAP_SINGLE, 2259 },
	{ 0xc171, 0xc173, PDF_CMAP_RANGE, 13648 },
	{ 0xc174, 0xc174, PDF_CMAP_SINGLE, 2260 },
	{ 0xc175, 0xc177, PDF_CMAP_RANGE, 13651 },
	{ 0xc178, 0xc178, PDF_CMAP_SINGLE, 2261 },
	{ 0xc179, 0xc184, PDF_CMAP_RANGE, 13654 },
	{ 0xc185, 0xc185, PDF_CMAP_SINGLE, 2262 },
	{ 0xc186, 0xc18b, PDF_CMAP_RANGE, 13666 },
	{ 0xc18c, 0xc18e, PDF_CMAP_RANGE, 2263 },
	{ 0xc18f, 0xc18f, PDF_CMAP_SINGLE, 13672 },
	{ 0xc190, 0xc190, PDF_CMAP_SINGLE, 2266 },
	{ 0xc191, 0xc193, PDF_CMAP_RANGE, 13673 },
	{ 0xc194, 0xc194, PDF_CMAP_SINGLE, 2267 },
	{ 0xc195, 0xc195, PDF_CMAP_SINGLE, 13676 },
	{ 0xc196, 0xc196, PDF_CMAP_SINGLE, 2268 },
	{ 0xc197, 0xc19b, PDF_CMAP_RANGE, 13677 },
	{ 0xc19c, 0xc19d, PDF_CMAP_RANGE, 2269 },
	{ 0xc19e, 0xc19e, PDF_CMAP_SINGLE, 13682 },
	{ 0xc19f, 0xc19f, PDF_CMAP_SINGLE, 2271 },
	{ 0xc1a0, 0xc1a0, PDF_CMAP_SINGLE, 13683 },
	{ 0xc1a1, 0xc1a1, PDF_CMAP_SINGLE, 2272 },
	{ 0xc1a2, 0xc1a4, PDF_CMAP_RANGE, 13684 },
	{ 0xc1a5, 0xc1a5, PDF_CMAP_SINGLE, 2273 },
	{ 0xc1a6, 0xc1a7, PDF_CMAP_RANGE, 13687 },
	{ 0xc1a8, 0xc1a9, PDF_CMAP_RANGE, 2274 },
	{ 0xc1aa, 0xc1ab, PDF_CMAP_RANGE, 13689 },
	{ 0xc1ac, 0xc1ac, PDF_CMAP_SINGLE, 2276 },
	{ 0xc1ad, 0xc1af, PDF_CMAP_RANGE, 13691 },
	{ 0xc1b0, 0xc1b0, PDF_CMAP_SINGLE, 2277 },
	{ 0xc1b1, 0xc1bc, PDF_CMAP_RANGE, 13694 },
	{ 0xc1bd, 0xc1bd, PDF_CMAP_SINGLE, 2278 },
	{ 0xc1be, 0xc1c3, PDF_CMAP_RANGE, 13706 },
	{ 0xc1c4, 0xc1c4, PDF_CMAP_SINGLE, 2279 },
	{ 0xc1c5, 0xc1c7, PDF_CMAP_RANGE, 13712 },
	{ 0xc1c8, 0xc1c8, PDF_CMAP_SINGLE, 2280 },
	{ 0xc1c9, 0xc1cb, PDF_CMAP_RANGE, 13715 },
	{ 0xc1cc, 0xc1cc, PDF_CMAP_SINGLE, 2281 },
	{ 0xc1cd, 0xc1d3, PDF_CMAP_RANGE, 13718 },
	{ 0xc1d4, 0xc1d4, PDF_CMAP_SINGLE, 2282 },
	{ 0xc1d5, 0xc1d6, PDF_CMAP_RANGE, 13725 },
	{ 0xc1d7, 0xc1d8, PDF_CMAP_RANGE, 2283 },
	{ 0xc1d9, 0xc1df, PDF_CMAP_RANGE, 13727 },
	{ 0xc1e0, 0xc1e0, PDF_CMAP_SINGLE, 2285 },
	{ 0xc1e1, 0xc1e3, PDF_CMAP_RANGE, 13734 },
	{ 0xc1e4, 0xc1e4, PDF_CMAP_SINGLE, 2286 },
	{ 0xc1e5, 0xc1e7, PDF_CMAP_RANGE, 13737 },
	{ 0xc1e8, 0xc1e8, PDF_CMAP_SINGLE, 2287 },
	{ 0xc1e9, 0xc1ef, PDF_CMAP_RANGE, 13740 },
	{ 0xc1f0, 0xc1f1, PDF_CMAP_RANGE, 2288 },
	{ 0xc1f2, 0xc1f2, PDF_CMAP_SINGLE, 13747 },
	{ 0xc1f3, 0xc1f3, PDF_CMAP_SINGLE, 2290 },
	{ 0xc1f4, 0xc1fb, PDF_CMAP_RANGE, 13748 },
	{ 0xc1fc, 0xc1fd, PDF_CMAP_RANGE, 2291 },
	{ 0xc1fe, 0xc1ff, PDF_CMAP_RANGE, 13756 },
	{ 0xc200, 0xc200, PDF_CMAP_SINGLE, 2293 },
	{ 0xc201, 0xc203, PDF_CMAP_RANGE, 13758 },
	{ 0xc204, 0xc204, PDF_CMAP_SINGLE, 2294 },
	{ 0xc205, 0xc20b, PDF_CMAP_RANGE, 13761 },
	{ 0xc20c, 0xc20d, PDF_CMAP_RANGE, 2295 },
	{ 0xc20e, 0xc20e, PDF_CMAP_SINGLE, 13768 },
	{ 0xc20f, 0xc20f, PDF_CMAP_SINGLE, 2297 },
	{ 0xc210, 0xc210, PDF_CMAP_SINGLE, 13769 },
	{ 0xc211, 0xc211, PDF_CMAP_SINGLE, 2298 },
	{ 0xc212, 0xc217, PDF_CMAP_RANGE, 13770 },
	{ 0xc218, 0xc219, PDF_CMAP_RANGE, 2299 },
	{ 0xc21a, 0xc21b, PDF_CMAP_RANGE, 13776 },
	{ 0xc21c, 0xc21c, PDF_CMAP_SINGLE, 2301 },
	{ 0xc21d, 0xc21e, PDF_CMAP_RANGE, 13778 },
	{ 0xc21f, 0xc220, PDF_CMAP_RANGE, 2302 },
	{ 0xc221, 0xc227, PDF_CMAP_RANGE, 13780 },
	{ 0xc228, 0xc229, PDF_CMAP_RANGE, 2304 },
	{ 0xc22a, 0xc22a, PDF_CMAP_SINGLE, 13787 },
	{ 0xc22b, 0xc22b, PDF_CMAP_SINGLE, 2306 },
	{ 0xc22c, 0xc22c, PDF_CMAP_SINGLE, 13788 },
	{ 0xc22d, 0xc22d, PDF_CMAP_SINGLE, 2307 },
	{ 0xc22e, 0xc22e, PDF_CMAP_SINGLE, 13789 },
	{ 0xc22f, 0xc22f, PDF_CMAP_SINGLE, 2308 },
	{ 0xc230, 0xc230, PDF_CMAP_SINGLE, 13790 },
	{ 0xc231, 0xc232, PDF_CMAP_RANGE, 2309 },
	{ 0xc233, 0xc233, PDF_CMAP_SINGLE, 13791 },
	{ 0xc234, 0xc234, PDF_CMAP_SINGLE, 2311 },
	{ 0xc235, 0xc247, PDF_CMAP_RANGE, 13792 },
	{ 0xc248, 0xc248, PDF_CMAP_SINGLE, 2312 },
	{ 0xc249, 0xc24f, PDF_CMAP_RANGE, 13811 },
	{ 0xc250, 0xc251, PDF_CMAP_RANGE, 2313 },
	{ 0xc252, 0xc253, PDF_CMAP_RANGE, 13818 },
	{ 0xc254, 0xc254, PDF_CMAP_SINGLE, 2315 },
	{ 0xc255, 0xc257, PDF_CMAP_RANGE, 13820 },
	{ 0xc258, 0xc258, PDF_CMAP_SINGLE, 2316 },
	{ 0xc259, 0xc25f, PDF_CMAP_RANGE, 13823 },
	{ 0xc260, 0xc260, PDF_CMAP_SINGLE, 2317 },
	{ 0xc261, 0xc264, PDF_CMAP_RANGE, 13830 },
	{ 0xc265, 0xc265, PDF_CMAP_SINGLE, 2318 },
	{ 0xc266, 0xc26b, PDF_CMAP_RANGE, 13834 },
	{ 0xc26c, 0xc26d, PDF_CMAP_RANGE, 2319 },
	{ 0xc26e, 0xc26f, PDF_CMAP_RANGE, 13840 },
	{ 0xc270, 0xc270, PDF_CMAP_SINGLE, 2321 },
	{ 0xc271, 0xc273, PDF_CMAP_RANGE, 13842 },
	{ 0xc274, 0xc274, PDF_CMAP_SINGLE, 2322 },
	{ 0xc275, 0xc27b, PDF_CMAP_RANGE, 13845 },
	{ 0xc27c, 0xc27d, PDF_CMAP_RANGE, 2323 },
	{ 0xc27e, 0xc27e, PDF_CMAP_SINGLE, 13852 },
	{ 0xc27f, 0xc27f, PDF_CMAP_SINGLE, 2325 },
	{ 0xc280, 0xc280, PDF_CMAP_SINGLE, 13853 },
	{ 0xc281, 0xc281, PDF_CMAP_SINGLE, 2326 },
	{ 0xc282, 0xc287, PDF_CMAP_RANGE, 13854 },
	{ 0xc288, 0xc289, PDF_CMAP_RANGE, 2327 },
	{ 0xc28a, 0xc28f, PDF_CMAP_RANGE, 13860 },
	{ 0xc290, 0xc290, PDF_CMAP_SINGLE, 2329 },
	{ 0xc291, 0xc297, PDF_CMAP_RANGE, 13866 },
	{ 0xc298, 0xc298, PDF_CMAP_SINGLE, 2330 },
	{ 0xc299, 0xc29a, PDF_CMAP_RANGE, 13873 },
	{ 0xc29b, 0xc29b, PDF_CMAP_SINGLE, 2331 },
	{ 0xc29c, 0xc29c, PDF_CMAP_SINGLE, 13875 },
	{ 0xc29d, 0xc29d, PDF_CMAP_SINGLE, 2332 },
	{ 0xc29e, 0xc2a3, PDF_CMAP_RANGE, 13876 },
	{ 0xc2a4, 0xc2a5, PDF_CMAP_RANGE, 2333 },
	{ 0xc2a6, 0xc2a7, PDF_CMAP_RANGE, 13882 },
	{ 0xc2a8, 0xc2a8, PDF_CMAP_SINGLE, 2335 },
	{ 0xc2a9, 0xc2ab, PDF_CMAP_RANGE, 13884 },
	{ 0xc2ac, 0xc2ad, PDF_CMAP_RANGE, 2336 },
	{ 0xc2ae, 0xc2b3, PDF_CMAP_RANGE, 13887 },
	{ 0xc2b4, 0xc2b5, PDF_CMAP_RANGE, 2338 },
	{ 0xc2b6, 0xc2b6, PDF_CMAP_SINGLE, 13893 },
	{ 0xc2b7, 0xc2b7, PDF_CMAP_SINGLE, 2340 },
	{ 0xc2b8, 0xc2b8, PDF_CMAP_SINGLE, 13894 },
	{ 0xc2b9, 0xc2b9, PDF_CMAP_SINGLE, 2341 },
	{ 0xc2ba, 0xc2db, PDF_CMAP_RANGE, 13895 },
	{ 0xc2dc, 0xc2dd, PDF_CMAP_RANGE, 2342 },
	{ 0xc2de, 0xc2df, PDF_CMAP_RANGE, 13929 },
	{ 0xc2e0, 0xc2e0, PDF_CMAP_SINGLE, 2344 },
	{ 0xc2e1, 0xc2e2, PDF_CMAP_RANGE, 13931 },
	{ 0xc2e3, 0xc2e4, PDF_CMAP_RANGE, 2345 },
	{ 0xc2e5, 0xc2ea, PDF_CMAP_RANGE, 13933 },
	{ 0xc2eb, 0xc2ed, PDF_CMAP_RANGE, 2347 },
	{ 0xc2ee, 0xc2ee, PDF_CMAP_SINGLE, 13939 },
	{ 0xc2ef, 0xc2ef, PDF_CMAP_SINGLE, 2350 },
	{ 0xc2f0, 0xc2f0, PDF_CMAP_SINGLE, 13940 },
	{ 0xc2f1, 0xc2f1, PDF_CMAP_SINGLE, 2351 },
	{ 0xc2f2, 0xc2f5, PDF_CMAP_RANGE, 13941 },
	{ 0xc2f6, 0xc2f6, PDF_CMAP_SINGLE, 2352 },
	{ 0xc2f7, 0xc2f7, PDF_CMAP_SINGLE, 13945 },
	{ 0xc2f8, 0xc2f9, PDF_CMAP_RANGE, 2353 },
	{ 0xc2fa, 0xc2fa, PDF_CMAP_SINGLE, 13946 },
	{ 0xc2fb, 0xc2fc, PDF_CMAP_RANGE, 2355 },
	{ 0xc2fd, 0xc2ff, PDF_CMAP_RANGE, 13947 },
	{ 0xc300, 0xc300, PDF_CMAP_SINGLE, 2357 },
	{ 0xc301, 0xc307, PDF_CMAP_RANGE, 13950 },
	{ 0xc308, 0xc309, PDF_CMAP_RANGE, 2358 },
	{ 0xc30a, 0xc30b, PDF_CMAP_RANGE, 13957 },
	{ 0xc30c, 0xc30d, PDF_CMAP_RANGE, 2360 },
	{ 0xc30e, 0xc312, PDF_CMAP_RANGE, 13959 },
	{ 0xc313, 0xc315, PDF_CMAP_RANGE, 2362 },
	{ 0xc316, 0xc317, PDF_CMAP_RANGE, 13964 },
	{ 0xc318, 0xc318, PDF_CMAP_SINGLE, 2365 },
	{ 0xc319, 0xc31b, PDF_CMAP_RANGE, 13966 },
	{ 0xc31c, 0xc31c, PDF_CMAP_SINGLE, 2366 },
	{ 0xc31d, 0xc323, PDF_CMAP_RANGE, 13969 },
	{ 0xc324, 0xc325, PDF_CMAP_RANGE, 2367 },
	{ 0xc326, 0xc327, PDF_CMAP_RANGE, 13976 },
	{ 0xc328, 0xc329, PDF_CMAP_RANGE, 2369 },
	{ 0xc32a, 0xc344, PDF_CMAP_RANGE, 13978 },
	{ 0xc345, 0xc345, PDF_CMAP_SINGLE, 2371 },
	{ 0xc346, 0xc367, PDF_CMAP_RANGE, 14005 },
	{ 0xc368, 0xc369, PDF_CMAP_RANGE, 2372 },
	{ 0xc36a, 0xc36b, PDF_CMAP_RANGE, 14039 },
	{ 0xc36c, 0xc36c, PDF_CMAP_SINGLE, 2374 },
	{ 0xc36d, 0xc36f, PDF_CMAP_RANGE, 14041 },
	{ 0xc370, 0xc370, PDF_CMAP_SINGLE, 2375 },
	{ 0xc371, 0xc371, PDF_CMAP_SINGLE, 14044 },
	{ 0xc372, 0xc372, PDF_CMAP_SINGLE, 2376 },
	{ 0xc373, 0xc377, PDF_CMAP_RANGE, 14045 },
	{ 0xc378, 0xc379, PDF_CMAP_RANGE, 2377 },
	{ 0xc37a, 0xc37b, PDF_CMAP_RANGE, 14050 },
	{ 0xc37c, 0xc37d, PDF_CMAP_RANGE, 2379 },
	{ 0xc37e, 0xc383, PDF_CMAP_RANGE, 14052 },
	{ 0xc384, 0xc384, PDF_CMAP_SINGLE, 2381 },
	{ 0xc385, 0xc387, PDF_CMAP_RANGE, 14058 },
	{ 0xc388, 0xc388, PDF_CMAP_SINGLE, 2382 },
	{ 0xc389, 0xc38b, PDF_CMAP_RANGE, 14061 },
	{ 0xc38c, 0xc38c, PDF_CMAP_SINGLE, 2383 },
	{ 0xc38d, 0xc3bf, PDF_CMAP_RANGE, 14064 },
	{ 0xc3c0, 0xc3c0, PDF_CMAP_SINGLE, 2384 },
	{ 0xc3c1, 0xc3d7, PDF_CMAP_RANGE, 14115 },
	{ 0xc3d8, 0xc3d9, PDF_CMAP_RANGE, 2385 },
	{ 0xc3da, 0xc3db, PDF_CMAP_RANGE, 14138 },
	{ 0xc3dc, 0xc3dc, PDF_CMAP_SINGLE, 2387 },
	{ 0xc3dd, 0xc3de, PDF_CMAP_RANGE, 14140 },
	{ 0xc3df, 0xc3e0, PDF_CMAP_RANGE, 2388 },
	{ 0xc3e1, 0xc3e1, PDF_CMAP_SINGLE, 14142 },
	{ 0xc3e2, 0xc3e2, PDF_CMAP_SINGLE, 2390 },
	{ 0xc3e3, 0xc3e7, PDF_CMAP_RANGE, 14143 },
	{ 0xc3e8, 0xc3e9, PDF_CMAP_RANGE, 2391 },
	{ 0xc3ea, 0xc3ec, PDF_CMAP_RANGE, 14148 },
	{ 0xc3ed, 0xc3ed, PDF_CMAP_SINGLE, 2393 },
	{ 0xc3ee, 0xc3f3, PDF_CMAP_RANGE, 14151 },
	{ 0xc3f4, 0xc3f5, PDF_CMAP_RANGE, 2394 },
	{ 0xc3f6, 0xc3f7, PDF_CMAP_RANGE, 14157 },
	{ 0xc3f8, 0xc3f8, PDF_CMAP_SINGLE, 2396 },
	{ 0xc3f9, 0xc3ff, PDF_CMAP_RANGE, 14159 },
	{ 0xc400, 0xc407, PDF_CMAP_RANGE, 14166 },
	{ 0xc408, 0xc408, PDF_CMAP_SINGLE, 2397 },
	{ 0xc409, 0xc40f, PDF_CMAP_RANGE, 14174 },
	{ 0xc410, 0xc410, PDF_CMAP_SINGLE, 2398 },
	{ 0xc411, 0xc423, PDF_CMAP_RANGE, 14181 },
	{ 0xc424, 0xc424, PDF_CMAP_SINGLE, 2399 },
	{ 0xc425, 0xc42b, PDF_CMAP_RANGE, 14200 },
	{ 0xc42c, 0xc42c, PDF_CMAP_SINGLE, 2400 },
	{ 0xc42d, 0xc42f, PDF_CMAP_RANGE, 14207 },
	{ 0xc430, 0xc430, PDF_CMAP_SINGLE, 2401 },
	{ 0xc431, 0xc433, PDF_CMAP_RANGE, 14210 },
	{ 0xc434, 0xc434, PDF_CMAP_SINGLE, 2402 },
	{ 0xc435, 0xc43b, PDF_CMAP_RANGE, 14213 },
	{ 0xc43c, 0xc43d, PDF_CMAP_RANGE, 2403 },
	{ 0xc43e, 0xc447, PDF_CMAP_RANGE, 14220 },
	{ 0xc448, 0xc448, PDF_CMAP_SINGLE, 2405 },
	{ 0xc449, 0xc463, PDF_CMAP_RANGE, 14230 },
	{ 0xc464, 0xc465, PDF_CMAP_RANGE, 2406 },
	{ 0xc466, 0xc467, PDF_CMAP_RANGE, 14257 },
	{ 0xc468, 0xc468, PDF_CMAP_SINGLE, 2408 },
	{ 0xc469, 0xc46b, PDF_CMAP_RANGE, 14259 },
	{ 0xc46c, 0xc46c, PDF_CMAP_SINGLE, 2409 },
	{ 0xc46d, 0xc473, PDF_CMAP_RANGE, 14262 },
	{ 0xc474, 0xc475, PDF_CMAP_RANGE, 2410 },
	{ 0xc476, 0xc478, PDF_CMAP_RANGE, 14269 },
	{ 0xc479, 0xc479, PDF_CMAP_SINGLE, 2412 },
	{ 0xc47a, 0xc47f, PDF_CMAP_RANGE, 14272 },
	{ 0xc480, 0xc480, PDF_CMAP_SINGLE, 2413 },
	{ 0xc481, 0xc493, PDF_CMAP_RANGE, 14278 },
	{ 0xc494, 0xc494, PDF_CMAP_SINGLE, 2414 },
	{ 0xc495, 0xc49b, PDF_CMAP_RANGE, 14297 },
	{ 0xc49c, 0xc49c, PDF_CMAP_SINGLE, 2415 },
	{ 0xc49d, 0xc4b7, PDF_CMAP_RANGE, 14304 },
	{ 0xc4b8, 0xc4b8, PDF_CMAP_SINGLE, 2416 },
	{ 0xc4b9, 0xc4bb, PDF_CMAP_RANGE, 14331 },
	{ 0xc4bc, 0xc4bc, PDF_CMAP_SINGLE, 2417 },
	{ 0xc4bd, 0xc4e8, PDF_CMAP_RANGE, 14334 },
	{ 0xc4e9, 0xc4e9, PDF_CMAP_SINGLE, 2418 },
	{ 0xc4ea, 0xc4ef, PDF_CMAP_RANGE, 14378 },
	{ 0xc4f0, 0xc4f1, PDF_CMAP_RANGE, 2419 },
	{ 0xc4f2, 0xc4f3, PDF_CMAP_RANGE, 14384 },
	{ 0xc4f4, 0xc4f4, PDF_CMAP_SINGLE, 2421 },
	{ 0xc4f5, 0xc4f7, PDF_CMAP_RANGE, 14386 },
	{ 0xc4f8, 0xc4f8, PDF_CMAP_SINGLE, 2422 },
	{ 0xc4f9, 0xc4f9, PDF_CMAP_SINGLE, 14389 },
	{ 0xc4fa, 0xc4fa, PDF_CMAP_SINGLE, 2423 },
	{ 0xc4fb, 0xc4fe, PDF_CMAP_RANGE, 14390 },
	{ 0xc4ff, 0xc4ff, PDF_CMAP_SINGLE, 2424 },
	{ 0xc500, 0xc501, PDF_CMAP_RANGE, 2425 },
	{ 0xc502, 0xc50b, PDF_CMAP_RANGE, 14394 },
	{ 0xc50c, 0xc50c, PDF_CMAP_SINGLE, 2427 },
	{ 0xc50d, 0xc50f, PDF_CMAP_RANGE, 14404 },
	{ 0xc510, 0xc510, PDF_CMAP_SINGLE, 2428 },
	{ 0xc511, 0xc513, PDF_CMAP_RANGE, 14407 },
	{ 0xc514, 0xc514, PDF_CMAP_SINGLE, 2429 },
	{ 0xc515, 0xc51b, PDF_CMAP_RANGE, 14410 },
	{ 0xc51c, 0xc51c, PDF_CMAP_SINGLE, 2430 },
	{ 0xc51d, 0xc527, PDF_CMAP_RANGE, 14417 },
	{ 0xc528, 0xc529, PDF_CMAP_RANGE, 2431 },
	{ 0xc52a, 0xc52b, PDF_CMAP_RANGE, 14428 },
	{ 0xc52c, 0xc52c, PDF_CMAP_SINGLE, 2433 },
	{ 0xc52d, 0xc52f, PDF_CMAP_RANGE, 14430 },
	{ 0xc530, 0xc530, PDF_CMAP_SINGLE, 2434 },
	{ 0xc531, 0xc537, PDF_CMAP_RANGE, 14433 },
	{ 0xc538, 0xc539, PDF_CMAP_RANGE, 2435 },
	{ 0xc53a, 0xc53a, PDF_CMAP_SINGLE, 14440 },
	{ 0xc53b, 0xc53b, PDF_CMAP_SINGLE, 2437 },
	{ 0xc53c, 0xc53c, PDF_CMAP_SINGLE, 14441 },
	{ 0xc53d, 0xc53d, PDF_CMAP_SINGLE, 2438 },
	{ 0xc53e, 0xc543, PDF_CMAP_RANGE, 14442 },
	{ 0xc544, 0xc545, PDF_CMAP_RANGE, 2439 },
	{ 0xc546, 0xc547, PDF_CMAP_RANGE, 14448 },
	{ 0xc548, 0xc54a, PDF_CMAP_RANGE, 2441 },
	{ 0xc54b, 0xc54b, PDF_CMAP_SINGLE, 14450 },
	{ 0xc54c, 0xc54e, PDF_CMAP_RANGE, 2444 },
	{ 0xc54f, 0xc552, PDF_CMAP_RANGE, 14451 },
	{ 0xc553, 0xc555, PDF_CMAP_RANGE, 2447 },
	{ 0xc556, 0xc556, PDF_CMAP_SINGLE, 14455 },
	{ 0xc557, 0xc559, PDF_CMAP_RANGE, 2450 },
	{ 0xc55a, 0xc55c, PDF_CMAP_RANGE, 14456 },
	{ 0xc55d, 0xc55e, PDF_CMAP_RANGE, 2453 },
	{ 0xc55f, 0xc55f, PDF_CMAP_SINGLE, 14459 },
	{ 0xc560, 0xc561, PDF_CMAP_RANGE, 2455 },
	{ 0xc562, 0xc563, PDF_CMAP_RANGE, 14460 },
	{ 0xc564, 0xc564, PDF_CMAP_SINGLE, 2457 },
	{ 0xc565, 0xc567, PDF_CMAP_RANGE, 14462 },
	{ 0xc568, 0xc568, PDF_CMAP_SINGLE, 2458 },
	{ 0xc569, 0xc56f, PDF_CMAP_RANGE, 14465 },
	{ 0xc570, 0xc571, PDF_CMAP_RANGE, 2459 },
	{ 0xc572, 0xc572, PDF_CMAP_SINGLE, 14472 },
	{ 0xc573, 0xc575, PDF_CMAP_RANGE, 2461 },
	{ 0xc576, 0xc57b, PDF_CMAP_RANGE, 14473 },
	{ 0xc57c, 0xc57d, PDF_CMAP_RANGE, 2464 },
	{ 0xc57e, 0xc57f, PDF_CMAP_RANGE, 14479 },
	{ 0xc580, 0xc580, PDF_CMAP_SINGLE, 2466 },
	{ 0xc581, 0xc583, PDF_CMAP_RANGE, 14481 },
	{ 0xc584, 0xc584, PDF_CMAP_SINGLE, 2467 },
	{ 0xc585, 0xc586, PDF_CMAP_RANGE, 14484 },
	{ 0xc587, 0xc587, PDF_CMAP_SINGLE, 2468 },
	{ 0xc588, 0xc58b, PDF_CMAP_RANGE, 14486 },
	{ 0xc58c, 0xc58d, PDF_CMAP_RANGE, 2469 },
	{ 0xc58e, 0xc58e, PDF_CMAP_SINGLE, 14490 },
	{ 0xc58f, 0xc58f, PDF_CMAP_SINGLE, 2471 },
	{ 0xc590, 0xc590, PDF_CMAP_SINGLE, 14491 },
	{ 0xc591, 0xc591, PDF_CMAP_SINGLE, 2472 },
	{ 0xc592, 0xc594, PDF_CMAP_RANGE, 14492 },
	{ 0xc595, 0xc595, PDF_CMAP_SINGLE, 2473 },
	{ 0xc596, 0xc596, PDF_CMAP_SINGLE, 14495 },
	{ 0xc597, 0xc598, PDF_CMAP_RANGE, 2474 },
	{ 0xc599, 0xc59b, PDF_CMAP_RANGE, 14496 },
	{ 0xc59c, 0xc59c, PDF_CMAP_SINGLE, 2476 },
	{ 0xc59d, 0xc59f, PDF_CMAP_RANGE, 14499 },
	{ 0xc5a0, 0xc5a0, PDF_CMAP_SINGLE, 2477 },
	{ 0xc5a1, 0xc5a8, PDF_CMAP_RANGE, 14502 },
	{ 0xc5a9, 0xc5a9, PDF_CMAP_SINGLE, 2478 },
	{ 0xc5aa, 0xc5b3, PDF_CMAP_RANGE, 14510 },
	{ 0xc5b4, 0xc5b5, PDF_CMAP_RANGE, 2479 },
	{ 0xc5b6, 0xc5b7, PDF_CMAP_RANGE, 14520 },
	{ 0xc5b8, 0xc5b9, PDF_CMAP_RANGE, 2481 },
	{ 0xc5ba, 0xc5ba, PDF_CMAP_SINGLE, 14522 },
	{ 0xc5bb, 0xc5be, PDF_CMAP_RANGE, 2483 },
	{ 0xc5bf, 0xc5c3, PDF_CMAP_RANGE, 14523 },
	{ 0xc5c4, 0xc5ca, PDF_CMAP_RANGE, 2487 },
	{ 0xc5cb, 0xc5cb, PDF_CMAP_SINGLE, 14528 },
	{ 0xc5cc, 0xc5cc, PDF_CMAP_SINGLE, 2494 },
	{ 0xc5cd, 0xc5cd, PDF_CMAP_SINGLE, 14529 },
	{ 0xc5ce, 0xc5ce, PDF_CMAP_SINGLE, 2495 },
	{ 0xc5cf, 0xc5cf, PDF_CMAP_SINGLE, 14530 },
	{ 0xc5d0, 0xc5d1, PDF_CMAP_RANGE, 2496 },
	{ 0xc5d2, 0xc5d3, PDF_CMAP_RANGE, 14531 },
	{ 0xc5d4, 0xc5d4, PDF_CMAP_SINGLE, 2498 },
	{ 0xc5d5, 0xc5d7, PDF_CMAP_RANGE, 14533 },
	{ 0xc5d8, 0xc5d8, PDF_CMAP_SINGLE, 2499 },
	{ 0xc5d9, 0xc5df, PDF_CMAP_RANGE, 14536 },
	{ 0xc5e0, 0xc5e1, PDF_CMAP_RANGE, 2500 },
	{ 0xc5e2, 0xc5e2, PDF_CMAP_SINGLE, 14543 },
	{ 0xc5e3, 0xc5e3, PDF_CMAP_SINGLE, 2502 },
	{ 0xc5e4, 0xc5e4, PDF_CMAP_SINGLE, 14544 },
	{ 0xc5e5, 0xc5e5, PDF_CMAP_SINGLE, 2503 },
	{ 0xc5e6, 0xc5eb, PDF_CMAP_RANGE, 14545 },
	{ 0xc5ec, 0xc5ee, PDF_CMAP_RANGE, 2504 },
	{ 0xc5ef, 0xc5ef, PDF_CMAP_SINGLE, 14551 },
	{ 0xc5f0, 0xc5f0, PDF_CMAP_SINGLE, 2507 },
	{ 0xc5f1, 0xc5f3, PDF_CMAP_RANGE, 14552 },
	{ 0xc5f4, 0xc5f4, PDF_CMAP_SINGLE, 2508 },
	{ 0xc5f5, 0xc5f5, PDF_CMAP_SINGLE, 14555 },
	{ 0xc5f6, 0xc5f7, PDF_CMAP_RANGE, 2509 },
	{ 0xc5f8, 0xc5fb, PDF_CMAP_RANGE, 14556 },
	{ 0xc5fc, 0xc5ff, PDF_CMAP_RANGE, 2511 },
	{ 0xc600, 0xc601, PDF_CMAP_RANGE, 2515 },
	{ 0xc602, 0xc604, PDF_CMAP_RANGE, 14560 },
	{ 0xc605, 0xc608, PDF_CMAP_RANGE, 2517 },
	{ 0xc609, 0xc60b, PDF_CMAP_RANGE, 14563 },
	{ 0xc60c, 0xc60c, PDF_CMAP_SINGLE, 2521 },
	{ 0xc60d, 0xc60f, PDF_CMAP_RANGE, 14566 },
	{ 0xc610, 0xc610, PDF_CMAP_SINGLE, 2522 },
	{ 0xc611, 0xc617, PDF_CMAP_RANGE, 14569 },
	{ 0xc618, 0xc619, PDF_CMAP_RANGE, 2523 },
	{ 0xc61a, 0xc61a, PDF_CMAP_SINGLE, 14576 },
	{ 0xc61b, 0xc61c, PDF_CMAP_RANGE, 2525 },
	{ 0xc61d, 0xc623, PDF_CMAP_RANGE, 14577 },
	{ 0xc624, 0xc625, PDF_CMAP_RANGE, 2527 },
	{ 0xc626, 0xc627, PDF_CMAP_RANGE, 14584 },
	{ 0xc628, 0xc628, PDF_CMAP_SINGLE, 2529 },
	{ 0xc629, 0xc62b, PDF_CMAP_RANGE, 14586 },
	{ 0xc62c, 0xc62e, PDF_CMAP_RANGE, 2530 },
	{ 0xc62f, 0xc62f, PDF_CMAP_SINGLE, 14589 },
	{ 0xc630, 0xc630, PDF_CMAP_SINGLE, 2533 },
	{ 0xc631, 0xc632, PDF_CMAP_RANGE, 14590 },
	{ 0xc633, 0xc635, PDF_CMAP_RANGE, 2534 },
	{ 0xc636, 0xc636, PDF_CMAP_SINGLE, 14592 },
	{ 0xc637, 0xc637, PDF_CMAP_SINGLE, 2537 },
	{ 0xc638, 0xc638, PDF_CMAP_SINGLE, 14593 },
	{ 0xc639, 0xc639, PDF_CMAP_SINGLE, 2538 },
	{ 0xc63a, 0xc63a, PDF_CMAP_SINGLE, 14594 },
	{ 0xc63b, 0xc63b, PDF_CMAP_SINGLE, 2539 },
	{ 0xc63c, 0xc63f, PDF_CMAP_RANGE, 14595 },
	{ 0xc640, 0xc641, PDF_CMAP_RANGE, 2540 },
	{ 0xc642, 0xc643, PDF_CMAP_RANGE, 14599 },
	{ 0xc644, 0xc644, PDF_CMAP_SINGLE, 2542 },
	{ 0xc645, 0xc647, PDF_CMAP_RANGE, 14601 },
	{ 0xc648, 0xc648, PDF_CMAP_SINGLE, 2543 },
	{ 0xc649, 0xc64f, PDF_CMAP_RANGE, 14604 },
	{ 0xc650, 0xc651, PDF_CMAP_RANGE, 2544 },
	{ 0xc652, 0xc652, PDF_CMAP_SINGLE, 14611 },
	{ 0xc653, 0xc655, PDF_CMAP_RANGE, 2546 },
	{ 0xc656, 0xc65b, PDF_CMAP_RANGE, 14612 },
	{ 0xc65c, 0xc65d, PDF_CMAP_RANGE, 2549 },
	{ 0xc65e, 0xc65f, PDF_CMAP_RANGE, 14618 },
	{ 0xc660, 0xc660, PDF_CMAP_SINGLE, 2551 },
	{ 0xc661, 0xc66b, PDF_CMAP_RANGE, 14620 },
	{ 0xc66c, 0xc66c, PDF_CMAP_SINGLE, 2552 },
	{ 0xc66d, 0xc66e, PDF_CMAP_RANGE, 14631 },
	{ 0xc66f, 0xc66f, PDF_CMAP_SINGLE, 2553 },
	{ 0xc670, 0xc670, PDF_CMAP_SINGLE, 14633 },
	{ 0xc671, 0xc671, PDF_CMAP_SINGLE, 2554 },
	{ 0xc672, 0xc677, PDF_CMAP_RANGE, 14634 },
	{ 0xc678, 0xc679, PDF_CMAP_RANGE, 2555 },
	{ 0xc67a, 0xc67b, PDF_CMAP_RANGE, 14640 },
	{ 0xc67c, 0xc67c, PDF_CMAP_SINGLE, 2557 },
	{ 0xc67d, 0xc67f, PDF_CMAP_RANGE, 14642 },
	{ 0xc680, 0xc680, PDF_CMAP_SINGLE, 2558 },
	{ 0xc681, 0xc687, PDF_CMAP_RANGE, 14645 },
	{ 0xc688, 0xc689, PDF_CMAP_RANGE, 2559 },
	{ 0xc68a, 0xc68a, PDF_CMAP_SINGLE, 14652 },
	{ 0xc68b, 0xc68b, PDF_CMAP_SINGLE, 2561 },
	{ 0xc68c, 0xc68c, PDF_CMAP_SINGLE, 14653 },
	{ 0xc68d, 0xc68d, PDF_CMAP_SINGLE, 2562 },
	{ 0xc68e, 0xc693, PDF_CMAP_RANGE, 14654 },
	{ 0xc694, 0xc695, PDF_CMAP_RANGE, 2563 },
	{ 0xc696, 0xc697, PDF_CMAP_RANGE, 14660 },
	{ 0xc698, 0xc698, PDF_CMAP_SINGLE, 2565 },
	{ 0xc699, 0xc69b, PDF_CMAP_RANGE, 14662 },
	{ 0xc69c, 0xc69c, PDF_CMAP_SINGLE, 2566 },
	{ 0xc69d, 0xc6a3, PDF_CMAP_RANGE, 14665 },
	{ 0xc6a4, 0xc6a5, PDF_CMAP_RANGE, 2567 },
	{ 0xc6a6, 0xc6a6, PDF_CMAP_SINGLE, 14672 },
	{ 0xc6a7, 0xc6a7, PDF_CMAP_SINGLE, 2569 },
	{ 0xc6a8, 0xc6a8, PDF_CMAP_SINGLE, 14673 },
	{ 0xc6a9, 0xc6a9, PDF_CMAP_SINGLE, 2570 },
	{ 0xc6aa, 0xc6af, PDF_CMAP_RANGE, 14674 },
	{ 0xc6b0, 0xc6b1, PDF_CMAP_RANGE, 2571 },
	{ 0xc6b2, 0xc6b3, PDF_CMAP_RANGE, 14680 },
	{ 0xc6b4, 0xc6b4, PDF_CMAP_SINGLE, 2573 },
	{ 0xc6b5, 0xc6b7, PDF_CMAP_RANGE, 14682 },
	{ 0xc6b8, 0xc6ba, PDF_CMAP_RANGE, 2574 },
	{ 0xc6bb, 0xc6bf, PDF_CMAP_RANGE, 14685 },
	{ 0xc6c0, 0xc6c1, PDF_CMAP_RANGE, 2577 },
	{ 0xc6c2, 0xc6c2, PDF_CMAP_SINGLE, 14690 },
	{ 0xc6c3, 0xc6c3, PDF_CMAP_SINGLE, 2579 },
	{ 0xc6c4, 0xc6c4, PDF_CMAP_SINGLE, 14691 },
	{ 0xc6c5, 0xc6c5, PDF_CMAP_SINGLE, 2580 },
	{ 0xc6c6, 0xc6cb, PDF_CMAP_RANGE, 14692 },
	{ 0xc6cc, 0xc6cd, PDF_CMAP_RANGE, 2581 },
	{ 0xc6ce, 0xc6cf, PDF_CMAP_RANGE, 14698 },
	{ 0xc6d0, 0xc6d0, PDF_CMAP_SINGLE, 2583 },
	{ 0xc6d1, 0xc6d3, PDF_CMAP_RANGE, 14700 },
	{ 0xc6d4, 0xc6d4, PDF_CMAP_SINGLE, 2584 },
	{ 0xc6d5, 0xc6db, PDF_CMAP_RANGE, 14703 },
	{ 0xc6dc, 0xc6dd, PDF_CMAP_RANGE, 2585 },
	{ 0xc6de, 0xc6df, PDF_CMAP_RANGE, 14710 },
	{ 0xc6e0, 0xc6e1, PDF_CMAP_RANGE, 2587 },
	{ 0xc6e2, 0xc6e7, PDF_CMAP_RANGE, 14712 },
	{ 0xc6e8, 0xc6e9, PDF_CMAP_RANGE, 2589 },
	{ 0xc6ea, 0xc6eb, PDF_CMAP_RANGE, 14718 },
	{ 0xc6ec, 0xc6ec, PDF_CMAP_SINGLE, 2591 },
	{ 0xc6ed, 0xc6ef, PDF_CMAP_RANGE, 14720 },
	{ 0xc6f0, 0xc6f0, PDF_CMAP_SINGLE, 2592 },
	{ 0xc6f1, 0xc6f7, PDF_CMAP_RANGE, 14723 },
	{ 0xc6f8, 0xc6f9, PDF_CMAP_RANGE, 2593 },
	{ 0xc6fa, 0xc6fc, PDF_CMAP_RANGE, 14730 },
	{ 0xc6fd, 0xc6fd, PDF_CMAP_SINGLE, 2595 },
	{ 0xc6fe, 0xc6ff, PDF_CMAP_RANGE, 14733 },
	{ 0xc700, 0xc703, PDF_CMAP_RANGE, 14735 },
	{ 0xc704, 0xc705, PDF_CMAP_RANGE, 2596 },
	{ 0xc706, 0xc707, PDF_CMAP_RANGE, 14739 },
	{ 0xc708, 0xc708, PDF_CMAP_SINGLE, 2598 },
	{ 0xc709, 0xc70b, PDF_CMAP_RANGE, 14741 },
	{ 0xc70c, 0xc70c, PDF_CMAP_SINGLE, 2599 },
	{ 0xc70d, 0xc713, PDF_CMAP_RANGE, 14744 },
	{ 0xc714, 0xc715, PDF_CMAP_RANGE, 2600 },
	{ 0xc716, 0xc716, PDF_CMAP_SINGLE, 14751 },
	{ 0xc717, 0xc717, PDF_CMAP_SINGLE, 2602 },
	{ 0xc718, 0xc718, PDF_CMAP_SINGLE, 14752 },
	{ 0xc719, 0xc719, PDF_CMAP_SINGLE, 2603 },
	{ 0xc71a, 0xc71f, PDF_CMAP_RANGE, 14753 },
	{ 0xc720, 0xc721, PDF_CMAP_RANGE, 2604 },
	{ 0xc722, 0xc723, PDF_CMAP_RANGE, 14759 },
	{ 0xc724, 0xc724, PDF_CMAP_SINGLE, 2606 },
	{ 0xc725, 0xc727, PDF_CMAP_RANGE, 14761 },
	{ 0xc728, 0xc728, PDF_CMAP_SINGLE, 2607 },
	{ 0xc729, 0xc72f, PDF_CMAP_RANGE, 14764 },
	{ 0xc730, 0xc731, PDF_CMAP_RANGE, 2608 },
	{ 0xc732, 0xc732, PDF_CMAP_SINGLE, 14771 },
	{ 0xc733, 0xc733, PDF_CMAP_SINGLE, 2610 },
	{ 0xc734, 0xc734, PDF_CMAP_SINGLE, 14772 },
	{ 0xc735, 0xc735, PDF_CMAP_SINGLE, 2611 },
	{ 0xc736, 0xc736, PDF_CMAP_SINGLE, 14773 },
	{ 0xc737, 0xc737, PDF_CMAP_SINGLE, 2612 },
	{ 0xc738, 0xc73b, PDF_CMAP_RANGE, 14774 },
	{ 0xc73c, 0xc73d, PDF_CMAP_RANGE, 2613 },
	{ 0xc73e, 0xc73f, PDF_CMAP_RANGE, 14778 },
	{ 0xc740, 0xc740, PDF_CMAP_SINGLE, 2615 },
	{ 0xc741, 0xc743, PDF_CMAP_RANGE, 14780 },
	{ 0xc744, 0xc744, PDF_CMAP_SINGLE, 2616 },
	{ 0xc745, 0xc749, PDF_CMAP_RANGE, 14783 },
	{ 0xc74a, 0xc74a, PDF_CMAP_SINGLE, 2617 },
	{ 0xc74b, 0xc74b, PDF_CMAP_SINGLE, 14788 },
	{ 0xc74c, 0xc74d, PDF_CMAP_RANGE, 2618 },
	{ 0xc74e, 0xc74e, PDF_CMAP_SINGLE, 14789 },
	{ 0xc74f, 0xc74f, PDF_CMAP_SINGLE, 2620 },
	{ 0xc750, 0xc750, PDF_CMAP_SINGLE, 14790 },
	{ 0xc751, 0xc758, PDF_CMAP_RANGE, 2621 },
	{ 0xc759, 0xc75b, PDF_CMAP_RANGE, 14791 },
	{ 0xc75c, 0xc75c, PDF_CMAP_SINGLE, 2629 },
	{ 0xc75d, 0xc75f, PDF_CMAP_RANGE, 14794 },
	{ 0xc760, 0xc760, PDF_CMAP_SINGLE, 2630 },
	{ 0xc761, 0xc767, PDF_CMAP_RANGE, 14797 },
	{ 0xc768, 0xc768, PDF_CMAP_SINGLE, 2631 },
	{ 0xc769, 0xc76a, PDF_CMAP_RANGE, 14804 },
	{ 0xc76b, 0xc76b, PDF_CMAP_SINGLE, 2632 },
	{ 0xc76c, 0xc773, PDF_CMAP_RANGE, 14806 },
	{ 0xc774, 0xc775, PDF_CMAP_RANGE, 2633 },
	{ 0xc776, 0xc777, PDF_CMAP_RANGE, 14814 },
	{ 0xc778, 0xc778, PDF_CMAP_SINGLE, 2635 },
	{ 0xc779, 0xc77b, PDF_CMAP_RANGE, 14816 },
	{ 0xc77c, 0xc77e, PDF_CMAP_RANGE, 2636 },
	{ 0xc77f, 0xc782, PDF_CMAP_RANGE, 14819 },
	{ 0xc783, 0xc785, PDF_CMAP_RANGE, 2639 },
	{ 0xc786, 0xc786, PDF_CMAP_SINGLE, 14823 },
	{ 0xc787, 0xc78a, PDF_CMAP_RANGE, 2642 },
	{ 0xc78b, 0xc78d, PDF_CMAP_RANGE, 14824 },
	{ 0xc78e, 0xc78e, PDF_CMAP_SINGLE, 2646 },
	{ 0xc78f, 0xc78f, PDF_CMAP_SINGLE, 14827 },
	{ 0xc790, 0xc791, PDF_CMAP_RANGE, 2647 },
	{ 0xc792, 0xc793, PDF_CMAP_RANGE, 14828 },
	{ 0xc794, 0xc794, PDF_CMAP_SINGLE, 2649 },
	{ 0xc795, 0xc795, PDF_CMAP_SINGLE, 14830 },
	{ 0xc796, 0xc798, PDF_CMAP_RANGE, 2650 },
	{ 0xc799, 0xc799, PDF_CMAP_SINGLE, 14831 },
	{ 0xc79a, 0xc79a, PDF_CMAP_SINGLE, 2653 },
	{ 0xc79b, 0xc79f, PDF_CMAP_RANGE, 14832 },
	{ 0xc7a0, 0xc7a1, PDF_CMAP_RANGE, 2654 },
	{ 0xc7a2, 0xc7a2, PDF_CMAP_SINGLE, 14837 },
	{ 0xc7a3, 0xc7a6, PDF_CMAP_RANGE, 2656 },
	{ 0xc7a7, 0xc7ab, PDF_CMAP_RANGE, 14838 },
	{ 0xc7ac, 0xc7ad, PDF_CMAP_RANGE, 2660 },
	{ 0xc7ae, 0xc7af, PDF_CMAP_RANGE, 14843 },
	{ 0xc7b0, 0xc7b0, PDF_CMAP_SINGLE, 2662 },
	{ 0xc7b1, 0xc7b3, PDF_CMAP_RANGE, 14845 },
	{ 0xc7b4, 0xc7b4, PDF_CMAP_SINGLE, 2663 },
	{ 0xc7b5, 0xc7bb, PDF_CMAP_RANGE, 14848 },
	{ 0xc7bc, 0xc7bd, PDF_CMAP_RANGE, 2664 },
	{ 0xc7be, 0xc7be, PDF_CMAP_SINGLE, 14855 },
	{ 0xc7bf, 0xc7c1, PDF_CMAP_RANGE, 2666 },
	{ 0xc7c2, 0xc7c7, PDF_CMAP_RANGE, 14856 },
	{ 0xc7c8, 0xc7c9, PDF_CMAP_RANGE, 2669 },
	{ 0xc7ca, 0xc7cb, PDF_CMAP_RANGE, 14862 },
	{ 0xc7cc, 0xc7cc, PDF_CMAP_SINGLE, 2671 },
	{ 0xc7cd, 0xc7cd, PDF_CMAP_SINGLE, 14864 },
	{ 0xc7ce, 0xc7ce, PDF_CMAP_SINGLE, 2672 },
	{ 0xc7cf, 0xc7cf, PDF_CMAP_SINGLE, 14865 },
	{ 0xc7d0, 0xc7d0, PDF_CMAP_SINGLE, 2673 },
	{ 0xc7d1, 0xc7d7, PDF_CMAP_RANGE, 14866 },
	{ 0xc7d8, 0xc7d8, PDF_CMAP_SINGLE, 2674 },
	{ 0xc7d9, 0xc7dc, PDF_CMAP_RANGE, 14873 },
	{ 0xc7dd, 0xc7dd, PDF_CMAP_SINGLE, 2675 },
	{ 0xc7de, 0xc7e3, PDF_CMAP_RANGE, 14877 },
	{ 0xc7e4, 0xc7e4, PDF_CMAP_SINGLE, 2676 },
	{ 0xc7e5, 0xc7e7, PDF_CMAP_RANGE, 14883 },
	{ 0xc7e8, 0xc7e8, PDF_CMAP_SINGLE, 2677 },
	{ 0xc7e9, 0xc7eb, PDF_CMAP_RANGE, 14886 },
	{ 0xc7ec, 0xc7ec, PDF_CMAP_SINGLE, 2678 },
	{ 0xc7ed, 0xc7ff, PDF_CMAP_RANGE, 14889 },
	{ 0xc800, 0xc801, PDF_CMAP_RANGE, 2679 },
	{ 0xc802, 0xc803, PDF_CMAP_RANGE, 14908 },
	{ 0xc804, 0xc804, PDF_CMAP_SINGLE, 2681 },
	{ 0xc805, 0xc807, PDF_CMAP_RANGE, 14910 },
	{ 0xc808, 0xc808, PDF_CMAP_SINGLE, 2682 },
	{ 0xc809, 0xc809, PDF_CMAP_SINGLE, 14913 },
	{ 0xc80a, 0xc80a, PDF_CMAP_SINGLE, 2683 },
	{ 0xc80b, 0xc80f, PDF_CMAP_RANGE, 14914 },
	{ 0xc810, 0xc811, PDF_CMAP_RANGE, 2684 },
	{ 0xc812, 0xc812, PDF_CMAP_SINGLE, 14919 },
	{ 0xc813, 0xc813, PDF_CMAP_SINGLE, 2686 },
	{ 0xc814, 0xc814, PDF_CMAP_SINGLE, 14920 },
	{ 0xc815, 0xc816, PDF_CMAP_RANGE, 2687 },
	{ 0xc817, 0xc81b, PDF_CMAP_RANGE, 14921 },
	{ 0xc81c, 0xc81d, PDF_CMAP_RANGE, 2689 },
	{ 0xc81e, 0xc81f, PDF_CMAP_RANGE, 14926 },
	{ 0xc820, 0xc820, PDF_CMAP_SINGLE, 2691 },
	{ 0xc821, 0xc823, PDF_CMAP_RANGE, 14928 },
	{ 0xc824, 0xc824, PDF_CMAP_SINGLE, 2692 },
	{ 0xc825, 0xc82b, PDF_CMAP_RANGE, 14931 },
	{ 0xc82c, 0xc82d, PDF_CMAP_RANGE, 2693 },
	{ 0xc82e, 0xc82e, PDF_CMAP_SINGLE, 14938 },
	{ 0xc82f, 0xc82f, PDF_CMAP_SINGLE, 2695 },
	{ 0xc830, 0xc830, PDF_CMAP_SINGLE, 14939 },
	{ 0xc831, 0xc831, PDF_CMAP_SINGLE, 2696 },
	{ 0xc832, 0xc837, PDF_CMAP_RANGE, 14940 },
	{ 0xc838, 0xc838, PDF_CMAP_SINGLE, 2697 },
	{ 0xc839, 0xc83b, PDF_CMAP_RANGE, 14946 },
	{ 0xc83c, 0xc83c, PDF_CMAP_SINGLE, 2698 },
	{ 0xc83d, 0xc83f, PDF_CMAP_RANGE, 14949 },
	{ 0xc840, 0xc840, PDF_CMAP_SINGLE, 2699 },
	{ 0xc841, 0xc847, PDF_CMAP_RANGE, 14952 },
	{ 0xc848, 0xc849, PDF_CMAP_RANGE, 2700 },
	{ 0xc84a, 0xc84b, PDF_CMAP_RANGE, 14959 },
	{ 0xc84c, 0xc84d, PDF_CMAP_RANGE, 2702 },
	{ 0xc84e, 0xc853, PDF_CMAP_RANGE, 14961 },
	{ 0xc854, 0xc854, PDF_CMAP_SINGLE, 2704 },
	{ 0xc855, 0xc86f, PDF_CMAP_RANGE, 14967 },
	{ 0xc870, 0xc871, PDF_CMAP_RANGE, 2705 },
	{ 0xc872, 0xc873, PDF_CMAP_RANGE, 14994 },
	{ 0xc874, 0xc874, PDF_CMAP_SINGLE, 2707 },
	{ 0xc875, 0xc877, PDF_CMAP_RANGE, 14996 },
	{ 0xc878, 0xc878, PDF_CMAP_SINGLE, 2708 },
	{ 0xc879, 0xc879, PDF_CMAP_SINGLE, 14999 },
	{ 0xc87a, 0xc87a, PDF_CMAP_SINGLE, 2709 },
	{ 0xc87b, 0xc87f, PDF_CMAP_RANGE, 15000 },
	{ 0xc880, 0xc881, PDF_CMAP_RANGE, 2710 },
	{ 0xc882, 0xc882, PDF_CMAP_SINGLE, 15005 },
	{ 0xc883, 0xc883, PDF_CMAP_SINGLE, 2712 },
	{ 0xc884, 0xc884, PDF_CMAP_SINGLE, 15006 },
	{ 0xc885, 0xc887, PDF_CMAP_RANGE, 2713 },
	{ 0xc888, 0xc88a, PDF_CMAP_RANGE, 15007 },
	{ 0xc88b, 0xc88d, PDF_CMAP_RANGE, 2716 },
	{ 0xc88e, 0xc893, PDF_CMAP_RANGE, 15010 },
	{ 0xc894, 0xc894, PDF_CMAP_SINGLE, 2719 },
	{ 0xc895, 0xc89c, PDF_CMAP_RANGE, 15016 },
	{ 0xc89d, 0xc89d, PDF_CMAP_SINGLE, 2720 },
	{ 0xc89e, 0xc89e, PDF_CMAP_SINGLE, 15024 },
	{ 0xc89f, 0xc89f, PDF_CMAP_SINGLE, 2721 },
	{ 0xc8a0, 0xc8a0, PDF_CMAP_SINGLE, 15025 },
	{ 0xc8a1, 0xc8a1, PDF_CMAP_SINGLE, 2722 },
	{ 0xc8a2, 0xc8a7, PDF_CMAP_RANGE, 15026 },
	{ 0xc8a8, 0xc8a8, PDF_CMAP_SINGLE, 2723 },
	{ 0xc8a9, 0xc8bb, PDF_CMAP_RANGE, 15032 },
	{ 0xc8bc, 0xc8bd, PDF_CMAP_RANGE, 2724 },
	{ 0xc8be, 0xc8c3, PDF_CMAP_RANGE, 15051 },
	{ 0xc8c4, 0xc8c4, PDF_CMAP_SINGLE, 2726 },
	{ 0xc8c5, 0xc8c7, PDF_CMAP_RANGE, 15057 },
	{ 0xc8c8, 0xc8c8, PDF_CMAP_SINGLE, 2727 },
	{ 0xc8c9, 0xc8cb, PDF_CMAP_RANGE, 15060 },
	{ 0xc8cc, 0xc8cc, PDF_CMAP_SINGLE, 2728 },
	{ 0xc8cd, 0xc8d3, PDF_CMAP_RANGE, 15063 },
	{ 0xc8d4, 0xc8d5, PDF_CMAP_RANGE, 2729 },
	{ 0xc8d6, 0xc8d6, PDF_CMAP_SINGLE, 15070 },
	{ 0xc8d7, 0xc8d7, PDF_CMAP_SINGLE, 2731 },
	{ 0xc8d8, 0xc8d8, PDF_CMAP_SINGLE, 15071 },
	{ 0xc8d9, 0xc8d9, PDF_CMAP_SINGLE, 2732 },
	{ 0xc8da, 0xc8df, PDF_CMAP_RANGE, 15072 },
	{ 0xc8e0, 0xc8e1, PDF_CMAP_RANGE, 2733 },
	{ 0xc8e2, 0xc8e3, PDF_CMAP_RANGE, 15078 },
	{ 0xc8e4, 0xc8e4, PDF_CMAP_SINGLE, 2735 },
	{ 0xc8e5, 0xc8f4, PDF_CMAP_RANGE, 15080 },
	{ 0xc8f5, 0xc8f5, PDF_CMAP_SINGLE, 2736 },
	{ 0xc8f6, 0xc8fb, PDF_CMAP_RANGE, 15096 },
	{ 0xc8fc, 0xc8fd, PDF_CMAP_RANGE, 2737 },
	{ 0xc8fe, 0xc8ff, PDF_CMAP_RANGE, 15102 },
	{ 0xc900, 0xc900, PDF_CMAP_SINGLE, 2739 },
	{ 0xc901, 0xc903, PDF_CMAP_RANGE, 15104 },
	{ 0xc904, 0xc906, PDF_CMAP_RANGE, 2740 },
	{ 0xc907, 0xc90b, PDF_CMAP_RANGE, 15107 },
	{ 0xc90c, 0xc90d, PDF_CMAP_RANGE, 2743 },
	{ 0xc90e, 0xc90e, PDF_CMAP_SINGLE, 15112 },
	{ 0xc90f, 0xc90f, PDF_CMAP_SINGLE, 2745 },
	{ 0xc910, 0xc910, PDF_CMAP_SINGLE, 15113 },
	{ 0xc911, 0xc911, PDF_CMAP_SINGLE, 2746 },
	{ 0xc912, 0xc917, PDF_CMAP_RANGE, 15114 },
	{ 0xc918, 0xc918, PDF_CMAP_SINGLE, 2747 },
	{ 0xc919, 0xc92b, PDF_CMAP_RANGE, 15120 },
	{ 0xc92c, 0xc92c, PDF_CMAP_SINGLE, 2748 },
	{ 0xc92d, 0xc933, PDF_CMAP_RANGE, 15139 },
	{ 0xc934, 0xc934, PDF_CMAP_SINGLE, 2749 },
	{ 0xc935, 0xc94f, PDF_CMAP_RANGE, 15146 },
	{ 0xc950, 0xc951, PDF_CMAP_RANGE, 2750 },
	{ 0xc952, 0xc953, PDF_CMAP_RANGE, 15173 },
	{ 0xc954, 0xc954, PDF_CMAP_SINGLE, 2752 },
	{ 0xc955, 0xc957, PDF_CMAP_RANGE, 15175 },
	{ 0xc958, 0xc958, PDF_CMAP_SINGLE, 2753 },
	{ 0xc959, 0xc95f, PDF_CMAP_RANGE, 15178 },
	{ 0xc960, 0xc961, PDF_CMAP_RANGE, 2754 },
	{ 0xc962, 0xc962, PDF_CMAP_SINGLE, 15185 },
	{ 0xc963, 0xc963, PDF_CMAP_SINGLE, 2756 },
	{ 0xc964, 0xc96b, PDF_CMAP_RANGE, 15186 },
	{ 0xc96c, 0xc96c, PDF_CMAP_SINGLE, 2757 },
	{ 0xc96d, 0xc96f, PDF_CMAP_RANGE, 15194 },
	{ 0xc970, 0xc970, PDF_CMAP_SINGLE, 2758 },
	{ 0xc971, 0xc973, PDF_CMAP_RANGE, 15197 },
	{ 0xc974, 0xc974, PDF_CMAP_SINGLE, 2759 },
	{ 0xc975, 0xc97b, PDF_CMAP_RANGE, 15200 },
	{ 0xc97c, 0xc97c, PDF_CMAP_SINGLE, 2760 },
	{ 0xc97d, 0xc987, PDF_CMAP_RANGE, 15207 },
	{ 0xc988, 0xc989, PDF_CMAP_RANGE, 2761 },
	{ 0xc98a, 0xc98b, PDF_CMAP_RANGE, 15218 },
	{ 0xc98c, 0xc98c, PDF_CMAP_SINGLE, 2763 },
	{ 0xc98d, 0xc98f, PDF_CMAP_RANGE, 15220 },
	{ 0xc990, 0xc990, PDF_CMAP_SINGLE, 2764 },
	{ 0xc991, 0xc997, PDF_CMAP_RANGE, 15223 },
	{ 0xc998, 0xc999, PDF_CMAP_RANGE, 2765 },
	{ 0xc99a, 0xc99a, PDF_CMAP_SINGLE, 15230 },
	{ 0xc99b, 0xc99b, PDF_CMAP_SINGLE, 2767 },
	{ 0xc99c, 0xc99c, PDF_CMAP_SINGLE, 15231 },
	{ 0xc99d, 0xc99d, PDF_CMAP_SINGLE, 2768 },
	{ 0xc99e, 0xc9bf, PDF_CMAP_RANGE, 15232 },
	{ 0xc9c0, 0xc9c1, PDF_CMAP_RANGE, 2769 },
	{ 0xc9c2, 0xc9c3, PDF_CMAP_RANGE, 15266 },
	{ 0xc9c4, 0xc9c4, PDF_CMAP_SINGLE, 2771 },
	{ 0xc9c5, 0xc9c6, PDF_CMAP_RANGE, 15268 },
	{ 0xc9c7, 0xc9c8, PDF_CMAP_RANGE, 2772 },
	{ 0xc9c9, 0xc9c9, PDF_CMAP_SINGLE, 15270 },
	{ 0xc9ca, 0xc9ca, PDF_CMAP_SINGLE, 2774 },
	{ 0xc9cb, 0xc9cf, PDF_CMAP_RANGE, 15271 },
	{ 0xc9d0, 0xc9d1, PDF_CMAP_RANGE, 2775 },
	{ 0xc9d2, 0xc9d2, PDF_CMAP_SINGLE, 15276 },
	{ 0xc9d3, 0xc9d3, PDF_CMAP_SINGLE, 2777 },
	{ 0xc9d4, 0xc9d4, PDF_CMAP_SINGLE, 15277 },
	{ 0xc9d5, 0xc9d6, PDF_CMAP_RANGE, 2778 },
	{ 0xc9d7, 0xc9d8, PDF_CMAP_RANGE, 15278 },
	{ 0xc9d9, 0xc9da, PDF_CMAP_RANGE, 2780 },
	{ 0xc9db, 0xc9db, PDF_CMAP_SINGLE, 15280 },
	{ 0xc9dc, 0xc9dd, PDF_CMAP_RANGE, 2782 },
	{ 0xc9de, 0xc9df, PDF_CMAP_RANGE, 15281 },
	{ 0xc9e0, 0xc9e0, PDF_CMAP_SINGLE, 2784 },
	{ 0xc9e1, 0xc9e1, PDF_CMAP_SINGLE, 15283 },
	{ 0xc9e2, 0xc9e2, PDF_CMAP_SINGLE, 2785 },
	{ 0xc9e3, 0xc9e3, PDF_CMAP_SINGLE, 15284 },
	{ 0xc9e4, 0xc9e4, PDF_CMAP_SINGLE, 2786 },
	{ 0xc9e5, 0xc9e6, PDF_CMAP_RANGE, 15285 },
	{ 0xc9e7, 0xc9e7, PDF_CMAP_SINGLE, 2787 },
	{ 0xc9e8, 0xc9eb, PDF_CMAP_RANGE, 15287 },
	{ 0xc9ec, 0xc9ed, PDF_CMAP_RANGE, 2788 },
	{ 0xc9ee, 0xc9ee, PDF_CMAP_SINGLE, 15291 },
	{ 0xc9ef, 0xc9f1, PDF_CMAP_RANGE, 2790 },
	{ 0xc9f2, 0xc9f7, PDF_CMAP_RANGE, 15292 },
	{ 0xc9f8, 0xc9f9, PDF_CMAP_RANGE, 2793 },
	{ 0xc9fa, 0xc9fb, PDF_CMAP_RANGE, 15298 },
	{ 0xc9fc, 0xc9fc, PDF_CMAP_SINGLE, 2795 },
	{ 0xc9fd, 0xc9ff, PDF_CMAP_RANGE, 15300 },
	{ 0xca00, 0xca00, PDF_CMAP_SINGLE, 2796 },
	{ 0xca01, 0xca07, PDF_CMAP_RANGE, 15303 },
	{ 0xca08, 0xca09, PDF_CMAP_RANGE, 2797 },
	{ 0xca0a, 0xca0a, PDF_CMAP_SINGLE, 15310 },
	{ 0xca0b, 0xca0d, PDF_CMAP_RANGE, 2799 },
	{ 0xca0e, 0xca13, PDF_CMAP_RANGE, 15311 },
	{ 0xca14, 0xca14, PDF_CMAP_SINGLE, 2802 },
	{ 0xca15, 0xca17, PDF_CMAP_RANGE, 15317 },
	{ 0xca18, 0xca18, PDF_CMAP_SINGLE, 2803 },
	{ 0xca19, 0xca28, PDF_CMAP_RANGE, 15320 },
	{ 0xca29, 0xca29, PDF_CMAP_SINGLE, 2804 },
	{ 0xca2a, 0xca4b, PDF_CMAP_RANGE, 15336 },
	{ 0xca4c, 0xca4d, PDF_CMAP_RANGE, 2805 },
	{ 0xca4e, 0xca4f, PDF_CMAP_RANGE, 15370 },
	{ 0xca50, 0xca50, PDF_CMAP_SINGLE, 2807 },
	{ 0xca51, 0xca53, PDF_CMAP_RANGE, 15372 },
	{ 0xca54, 0xca54, PDF_CMAP_SINGLE, 2808 },
	{ 0xca55, 0xca5b, PDF_CMAP_RANGE, 15375 },
	{ 0xca5c, 0xca5d, PDF_CMAP_RANGE, 2809 },
	{ 0xca5e, 0xca5e, PDF_CMAP_SINGLE, 15382 },
	{ 0xca5f, 0xca61, PDF_CMAP_RANGE, 2811 },
	{ 0xca62, 0xca67, PDF_CMAP_RANGE, 15383 },
	{ 0xca68, 0xca68, PDF_CMAP_SINGLE, 2814 },
	{ 0xca69, 0xca7c, PDF_CMAP_RANGE, 15389 },
	{ 0xca7d, 0xca7d, PDF_CMAP_SINGLE, 2815 },
	{ 0xca7e, 0xca83, PDF_CMAP_RANGE, 15409 },
	{ 0xca84, 0xca84, PDF_CMAP_SINGLE, 2816 },
	{ 0xca85, 0xca97, PDF_CMAP_RANGE, 15415 },
	{ 0xca98, 0xca98, PDF_CMAP_SINGLE, 2817 },
	{ 0xca99, 0xcabb, PDF_CMAP_RANGE, 15434 },
	{ 0xcabc, 0xcabd, PDF_CMAP_RANGE, 2818 },
	{ 0xcabe, 0xcabf, PDF_CMAP_RANGE, 15469 },
	{ 0xcac0, 0xcac0, PDF_CMAP_SINGLE, 2820 },
	{ 0xcac1, 0xcac3, PDF_CMAP_RANGE, 15471 },
	{ 0xcac4, 0xcac4, PDF_CMAP_SINGLE, 2821 },
	{ 0xcac5, 0xcacb, PDF_CMAP_RANGE, 15474 },
	{ 0xcacc, 0xcacd, PDF_CMAP_RANGE, 2822 },
	{ 0xcace, 0xcace, PDF_CMAP_SINGLE, 15481 },
	{ 0xcacf, 0xcacf, PDF_CMAP_SINGLE, 2824 },
	{ 0xcad0, 0xcad0, PDF_CMAP_SINGLE, 15482 },
	{ 0xcad1, 0xcad1, PDF_CMAP_SINGLE, 2825 },
	{ 0xcad2, 0xcad2, PDF_CMAP_SINGLE, 15483 },
	{ 0xcad3, 0xcad3, PDF_CMAP_SINGLE, 2826 },
	{ 0xcad4, 0xcad7, PDF_CMAP_RANGE, 15484 },
	{ 0xcad8, 0xcad9, PDF_CMAP_RANGE, 2827 },
	{ 0xcada, 0xcadf, PDF_CMAP_RANGE, 15488 },
	{ 0xcae0, 0xcae0, PDF_CMAP_SINGLE, 2829 },
	{ 0xcae1, 0xcaeb, PDF_CMAP_RANGE, 15494 },
	{ 0xcaec, 0xcaec, PDF_CMAP_SINGLE, 2830 },
	{ 0xcaed, 0xcaf3, PDF_CMAP_RANGE, 15505 },
	{ 0xcaf4, 0xcaf4, PDF_CMAP_SINGLE, 2831 },
	{ 0xcaf5, 0xcaff, PDF_CMAP_RANGE, 15512 },
	{ 0xcb00, 0xcb07, PDF_CMAP_RANGE, 15523 },
	{ 0xcb08, 0xcb08, PDF_CMAP_SINGLE, 2832 },
	{ 0xcb09, 0xcb0f, PDF_CMAP_RANGE, 15531 },
	{ 0xcb10, 0xcb10, PDF_CMAP_SINGLE, 2833 },
	{ 0xcb11, 0xcb13, PDF_CMAP_RANGE, 15538 },
	{ 0xcb14, 0xcb14, PDF_CMAP_SINGLE, 2834 },
	{ 0xcb15, 0xcb17, PDF_CMAP_RANGE, 15541 },
	{ 0xcb18, 0xcb18, PDF_CMAP_SINGLE, 2835 },
	{ 0xcb19, 0xcb1f, PDF_CMAP_RANGE, 15544 },
	{ 0xcb20, 0xcb21, PDF_CMAP_RANGE, 2836 },
	{ 0xcb22, 0xcb40, PDF_CMAP_RANGE, 15551 },
	{ 0xcb41, 0xcb41, PDF_CMAP_SINGLE, 2838 },
	{ 0xcb42, 0xcb47, PDF_CMAP_RANGE, 15582 },
	{ 0xcb48, 0xcb49, PDF_CMAP_RANGE, 2839 },
	{ 0xcb4a, 0xcb4b, PDF_CMAP_RANGE, 15588 },
	{ 0xcb4c, 0xcb4c, PDF_CMAP_SINGLE, 2841 },
	{ 0xcb4d, 0xcb4f, PDF_CMAP_RANGE, 15590 },
	{ 0xcb50, 0xcb50, PDF_CMAP_SINGLE, 2842 },
	{ 0xcb51, 0xcb57, PDF_CMAP_RANGE, 15593 },
	{ 0xcb58, 0xcb59, PDF_CMAP_RANGE, 2843 },
	{ 0xcb5a, 0xcb5c, PDF_CMAP_RANGE, 15600 },
	{ 0xcb5d, 0xcb5d, PDF_CMAP_SINGLE, 2845 },
	{ 0xcb5e, 0xcb63, PDF_CMAP_RANGE, 15603 },
	{ 0xcb64, 0xcb64, PDF_CMAP_SINGLE, 2846 },
	{ 0xcb65, 0xcb77, PDF_CMAP_RANGE, 15609 },
	{ 0xcb78, 0xcb79, PDF_CMAP_RANGE, 2847 },
	{ 0xcb7a, 0xcb9b, PDF_CMAP_RANGE, 15628 },
	{ 0xcb9c, 0xcb9c, PDF_CMAP_SINGLE, 2849 },
	{ 0xcb9d, 0xcbb7, PDF_CMAP_RANGE, 15662 },
	{ 0xcbb8, 0xcbb8, PDF_CMAP_SINGLE, 2850 },
	{ 0xcbb9, 0xcbd3, PDF_CMAP_RANGE, 15689 },
	{ 0xcbd4, 0xcbd4, PDF_CMAP_SINGLE, 2851 },
	{ 0xcbd5, 0xcbe3, PDF_CMAP_RANGE, 15716 },
	{ 0xcbe4, 0xcbe4, PDF_CMAP_SINGLE, 2852 },
	{ 0xcbe5, 0xcbe6, PDF_CMAP_RANGE, 15731 },
	{ 0xcbe7, 0xcbe7, PDF_CMAP_SINGLE, 2853 },
	{ 0xcbe8, 0xcbe8, PDF_CMAP_SINGLE, 15733 },
	{ 0xcbe9, 0xcbe9, PDF_CMAP_SINGLE, 2854 },
	{ 0xcbea, 0xcbff, PDF_CMAP_RANGE, 15734 },
	{ 0xcc00, 0xcc0b, PDF_CMAP_RANGE, 15756 },
	{ 0xcc0c, 0xcc0d, PDF_CMAP_RANGE, 2855 },
	{ 0xcc0e, 0xcc0f, PDF_CMAP_RANGE, 15768 },
	{ 0xcc10, 0xcc10, PDF_CMAP_SINGLE, 2857 },
	{ 0xcc11, 0xcc13, PDF_CMAP_RANGE, 15770 },
	{ 0xcc14, 0xcc14, PDF_CMAP_SINGLE, 2858 },
	{ 0xcc15, 0xcc1b, PDF_CMAP_RANGE, 15773 },
	{ 0xcc1c, 0xcc1d, PDF_CMAP_RANGE, 2859 },
	{ 0xcc1e, 0xcc20, PDF_CMAP_RANGE, 15780 },
	{ 0xcc21, 0xcc22, PDF_CMAP_RANGE, 2861 },
	{ 0xcc23, 0xcc26, PDF_CMAP_RANGE, 15783 },
	{ 0xcc27, 0xcc29, PDF_CMAP_RANGE, 2863 },
	{ 0xcc2a, 0xcc2b, PDF_CMAP_RANGE, 15787 },
	{ 0xcc2c, 0xcc2c, PDF_CMAP_SINGLE, 2866 },
	{ 0xcc2d, 0xcc2d, PDF_CMAP_SINGLE, 15789 },
	{ 0xcc2e, 0xcc2e, PDF_CMAP_SINGLE, 2867 },
	{ 0xcc2f, 0xcc2f, PDF_CMAP_SINGLE, 15790 },
	{ 0xcc30, 0xcc30, PDF_CMAP_SINGLE, 2868 },
	{ 0xcc31, 0xcc37, PDF_CMAP_RANGE, 15791 },
	{ 0xcc38, 0xcc39, PDF_CMAP_RANGE, 2869 },
	{ 0xcc3a, 0xcc3a, PDF_CMAP_SINGLE, 15798 },
	{ 0xcc3b, 0xcc3e, PDF_CMAP_RANGE, 2871 },
	{ 0xcc3f, 0xcc43, PDF_CMAP_RANGE, 15799 },
	{ 0xcc44, 0xcc45, PDF_CMAP_RANGE, 2875 },
	{ 0xcc46, 0xcc47, PDF_CMAP_RANGE, 15804 },
	{ 0xcc48, 0xcc48, PDF_CMAP_SINGLE, 2877 },
	{ 0xcc49, 0xcc4b, PDF_CMAP_RANGE, 15806 },
	{ 0xcc4c, 0xcc4c, PDF_CMAP_SINGLE, 2878 },
	{ 0xcc4d, 0xcc53, PDF_CMAP_RANGE, 15809 },
	{ 0xcc54, 0xcc55, PDF_CMAP_RANGE, 2879 },
	{ 0xcc56, 0xcc56, PDF_CMAP_SINGLE, 15816 },
	{ 0xcc57, 0xcc59, PDF_CMAP_RANGE, 2881 },
	{ 0xcc5a, 0xcc5f, PDF_CMAP_RANGE, 15817 },
	{ 0xcc60, 0xcc60, PDF_CMAP_SINGLE, 2884 },
	{ 0xcc61, 0xcc63, PDF_CMAP_RANGE, 15823 },
	{ 0xcc64, 0xcc64, PDF_CMAP_SINGLE, 2885 },
	{ 0xcc65, 0xcc65, PDF_CMAP_SINGLE, 15826 },
	{ 0xcc66, 0xcc66, PDF_CMAP_SINGLE, 2886 },
	{ 0xcc67, 0xcc67, PDF_CMAP_SINGLE, 15827 },
	{ 0xcc68, 0xcc68, PDF_CMAP_SINGLE, 2887 },
	{ 0xcc69, 0xcc6f, PDF_CMAP_RANGE, 15828 },
	{ 0xcc70, 0xcc70, PDF_CMAP_SINGLE, 2888 },
	{ 0xcc71, 0xcc74, PDF_CMAP_RANGE, 15835 },
	{ 0xcc75, 0xcc75, PDF_CMAP_SINGLE, 2889 },
	{ 0xcc76, 0xcc97, PDF_CMAP_RANGE, 15839 },
	{ 0xcc98, 0xcc99, PDF_CMAP_RANGE, 2890 },
	{ 0xcc9a, 0xcc9b, PDF_CMAP_RANGE, 15873 },
	{ 0xcc9c, 0xcc9c, PDF_CMAP_SINGLE, 2892 },
	{ 0xcc9d, 0xcc9f, PDF_CMAP_RANGE, 15875 },
	{ 0xcca0, 0xcca0, PDF_CMAP_SINGLE, 2893 },
	{ 0xcca1, 0xcca7, PDF_CMAP_RANGE, 15878 },
	{ 0xcca8, 0xcca9, PDF_CMAP_RANGE, 2894 },
	{ 0xccaa, 0xccaa, PDF_CMAP_SINGLE, 15885 },
	{ 0xccab, 0xccad, PDF_CMAP_RANGE, 2896 },
	{ 0xccae, 0xccb3, PDF_CMAP_RANGE, 15886 },
	{ 0xccb4, 0xccb5, PDF_CMAP_RANGE, 2899 },
	{ 0xccb6, 0xccb7, PDF_CMAP_RANGE, 15892 },
	{ 0xccb8, 0xccb8, PDF_CMAP_SINGLE, 2901 },
	{ 0xccb9, 0xccbb, PDF_CMAP_RANGE, 15894 },
	{ 0xccbc, 0xccbc, PDF_CMAP_SINGLE, 2902 },
	{ 0xccbd, 0xccc3, PDF_CMAP_RANGE, 15897 },
	{ 0xccc4, 0xccc5, PDF_CMAP_RANGE, 2903 },
	{ 0xccc6, 0xccc6, PDF_CMAP_SINGLE, 15904 },
	{ 0xccc7, 0xccc7, PDF_CMAP_SINGLE, 2905 },
	{ 0xccc8, 0xccc8, PDF_CMAP_SINGLE, 15905 },
	{ 0xccc9, 0xccc9, PDF_CMAP_SINGLE, 2906 },
	{ 0xccca, 0xcccf, PDF_CMAP_RANGE, 15906 },
	{ 0xccd0, 0xccd0, PDF_CMAP_SINGLE, 2907 },
	{ 0xccd1, 0xccd3, PDF_CMAP_RANGE, 15912 },
	{ 0xccd4, 0xccd4, PDF_CMAP_SINGLE, 2908 },
	{ 0xccd5, 0xcce3, PDF_CMAP_RANGE, 15915 },
	{ 0xcce4, 0xcce4, PDF_CMAP_SINGLE, 2909 },
	{ 0xcce5, 0xcceb, PDF_CMAP_RANGE, 15930 },
	{ 0xccec, 0xccec, PDF_CMAP_SINGLE, 2910 },
	{ 0xcced, 0xccef, PDF_CMAP_RANGE, 15937 },
	{ 0xccf0, 0xccf0, PDF_CMAP_SINGLE, 2911 },
	{ 0xccf1, 0xccff, PDF_CMAP_RANGE, 15940 },
	{ 0xcd00, 0xcd00, PDF_CMAP_SINGLE, 15955 },
	{ 0xcd01, 0xcd01, PDF_CMAP_SINGLE, 2912 },
	{ 0xcd02, 0xcd07, PDF_CMAP_RANGE, 15956 },
	{ 0xcd08, 0xcd09, PDF_CMAP_RANGE, 2913 },
	{ 0xcd0a, 0xcd0b, PDF_CMAP_RANGE, 15962 },
	{ 0xcd0c, 0xcd0c, PDF_CMAP_SINGLE, 2915 },
	{ 0xcd0d, 0xcd0f, PDF_CMAP_RANGE, 15964 },
	{ 0xcd10, 0xcd10, PDF_CMAP_SINGLE, 2916 },
	{ 0xcd11, 0xcd17, PDF_CMAP_RANGE, 15967 },
	{ 0xcd18, 0xcd19, PDF_CMAP_RANGE, 2917 },
	{ 0xcd1a, 0xcd1a, PDF_CMAP_SINGLE, 15974 },
	{ 0xcd1b, 0xcd1b, PDF_CMAP_SINGLE, 2919 },
	{ 0xcd1c, 0xcd1c, PDF_CMAP_SINGLE, 15975 },
	{ 0xcd1d, 0xcd1d, PDF_CMAP_SINGLE, 2920 },
	{ 0xcd1e, 0xcd23, PDF_CMAP_RANGE, 15976 },
	{ 0xcd24, 0xcd24, PDF_CMAP_SINGLE, 2921 },
	{ 0xcd25, 0xcd27, PDF_CMAP_RANGE, 15982 },
	{ 0xcd28, 0xcd28, PDF_CMAP_SINGLE, 2922 },
	{ 0xcd29, 0xcd2b, PDF_CMAP_RANGE, 15985 },
	{ 0xcd2c, 0xcd2c, PDF_CMAP_SINGLE, 2923 },
	{ 0xcd2d, 0xcd38, PDF_CMAP_RANGE, 15988 },
	{ 0xcd39, 0xcd39, PDF_CMAP_SINGLE, 2924 },
	{ 0xcd3a, 0xcd5b, PDF_CMAP_RANGE, 16000 },
	{ 0xcd5c, 0xcd5c, PDF_CMAP_SINGLE, 2925 },
	{ 0xcd5d, 0xcd5f, PDF_CMAP_RANGE, 16034 },
	{ 0xcd60, 0xcd60, PDF_CMAP_SINGLE, 2926 },
	{ 0xcd61, 0xcd63, PDF_CMAP_RANGE, 16037 },
	{ 0xcd64, 0xcd64, PDF_CMAP_SINGLE, 2927 },
	{ 0xcd65, 0xcd6b, PDF_CMAP_RANGE, 16040 },
	{ 0xcd6c, 0xcd6d, PDF_CMAP_RANGE, 2928 },
	{ 0xcd6e, 0xcd6e, PDF_CMAP_SINGLE, 16047 },
	{ 0xcd6f, 0xcd6f, PDF_CMAP_SINGLE, 2930 },
	{ 0xcd70, 0xcd70, PDF_CMAP_SINGLE, 16048 },
	{ 0xcd71, 0xcd71, PDF_CMAP_SINGLE, 2931 },
	{ 0xcd72, 0xcd77, PDF_CMAP_RANGE, 16049 },
	{ 0xcd78, 0xcd78, PDF_CMAP_SINGLE, 2932 },
	{ 0xcd79, 0xcd87, PDF_CMAP_RANGE, 16055 },
	{ 0xcd88, 0xcd88, PDF_CMAP_SINGLE, 2933 },
	{ 0xcd89, 0xcd93, PDF_CMAP_RANGE, 16070 },
	{ 0xcd94, 0xcd95, PDF_CMAP_RANGE, 2934 },
	{ 0xcd96, 0xcd97, PDF_CMAP_RANGE, 16081 },
	{ 0xcd98, 0xcd98, PDF_CMAP_SINGLE, 2936 },
	{ 0xcd99, 0xcd9b, PDF_CMAP_RANGE, 16083 },
	{ 0xcd9c, 0xcd9c, PDF_CMAP_SINGLE, 2937 },
	{ 0xcd9d, 0xcda3, PDF_CMAP_RANGE, 16086 },
	{ 0xcda4, 0xcda5, PDF_CMAP_RANGE, 2938 },
	{ 0xcda6, 0xcda6, PDF_CMAP_SINGLE, 16093 },
	{ 0xcda7, 0xcda7, PDF_CMAP_SINGLE, 2940 },
	{ 0xcda8, 0xcda8, PDF_CMAP_SINGLE, 16094 },
	{ 0xcda9, 0xcda9, PDF_CMAP_SINGLE, 2941 },
	{ 0xcdaa, 0xcdaf, PDF_CMAP_RANGE, 16095 },
	{ 0xcdb0, 0xcdb0, PDF_CMAP_SINGLE, 2942 },
	{ 0xcdb1, 0xcdc3, PDF_CMAP_RANGE, 16101 },
	{ 0xcdc4, 0xcdc4, PDF_CMAP_SINGLE, 2943 },
	{ 0xcdc5, 0xcdcb, PDF_CMAP_RANGE, 16120 },
	{ 0xcdcc, 0xcdcc, PDF_CMAP_SINGLE, 2944 },
	{ 0xcdcd, 0xcdcf, PDF_CMAP_RANGE, 16127 },
	{ 0xcdd0, 0xcdd0, PDF_CMAP_SINGLE, 2945 },
	{ 0xcdd1, 0xcde7, PDF_CMAP_RANGE, 16130 },
	{ 0xcde8, 0xcde8, PDF_CMAP_SINGLE, 2946 },
	{ 0xcde9, 0xcdeb, PDF_CMAP_RANGE, 16153 },
	{ 0xcdec, 0xcdec, PDF_CMAP_SINGLE, 2947 },
	{ 0xcded, 0xcdef, PDF_CMAP_RANGE, 16156 },
	{ 0xcdf0, 0xcdf0, PDF_CMAP_SINGLE, 2948 },
	{ 0xcdf1, 0xcdf7, PDF_CMAP_RANGE, 16159 },
	{ 0xcdf8, 0xcdf9, PDF_CMAP_RANGE, 2949 },
	{ 0xcdfa, 0xcdfa, PDF_CMAP_SINGLE, 16166 },
	{ 0xcdfb, 0xcdfb, PDF_CMAP_SINGLE, 2951 },
	{ 0xcdfc, 0xcdfc, PDF_CMAP_SINGLE, 16167 },
	{ 0xcdfd, 0xcdfd, PDF_CMAP_SINGLE, 2952 },
	{ 0xcdfe, 0xcdff, PDF_CMAP_RANGE, 16168 },
	{ 0xce00, 0xce03, PDF_CMAP_RANGE, 16170 },
	{ 0xce04, 0xce04, PDF_CMAP_SINGLE, 2953 },
	{ 0xce05, 0xce07, PDF_CMAP_RANGE, 16174 },
	{ 0xce08, 0xce08, PDF_CMAP_SINGLE, 2954 },
	{ 0xce09, 0xce0b, PDF_CMAP_RANGE, 16177 },
	{ 0xce0c, 0xce0c, PDF_CMAP_SINGLE, 2955 },
	{ 0xce0d, 0xce13, PDF_CMAP_RANGE, 16180 },
	{ 0xce14, 0xce14, PDF_CMAP_SINGLE, 2956 },
	{ 0xce15, 0xce18, PDF_CMAP_RANGE, 16187 },
	{ 0xce19, 0xce19, PDF_CMAP_SINGLE, 2957 },
	{ 0xce1a, 0xce1f, PDF_CMAP_RANGE, 16191 },
	{ 0xce20, 0xce21, PDF_CMAP_RANGE, 2958 },
	{ 0xce22, 0xce23, PDF_CMAP_RANGE, 16197 },
	{ 0xce24, 0xce24, PDF_CMAP_SINGLE, 2960 },
	{ 0xce25, 0xce27, PDF_CMAP_RANGE, 16199 },
	{ 0xce28, 0xce28, PDF_CMAP_SINGLE, 2961 },
	{ 0xce29, 0xce2f, PDF_CMAP_RANGE, 16202 },
	{ 0xce30, 0xce31, PDF_CMAP_RANGE, 2962 },
	{ 0xce32, 0xce32, PDF_CMAP_SINGLE, 16209 },
	{ 0xce33, 0xce33, PDF_CMAP_SINGLE, 2964 },
	{ 0xce34, 0xce34, PDF_CMAP_SINGLE, 16210 },
	{ 0xce35, 0xce35, PDF_CMAP_SINGLE, 2965 },
	{ 0xce36, 0xce57, PDF_CMAP_RANGE, 16211 },
	{ 0xce58, 0xce59, PDF_CMAP_RANGE, 2966 },
	{ 0xce5a, 0xce5b, PDF_CMAP_RANGE, 16245 },
	{ 0xce5c, 0xce5c, PDF_CMAP_SINGLE, 2968 },
	{ 0xce5d, 0xce5e, PDF_CMAP_RANGE, 16247 },
	{ 0xce5f, 0xce61, PDF_CMAP_RANGE, 2969 },
	{ 0xce62, 0xce67, PDF_CMAP_RANGE, 16249 },
	{ 0xce68, 0xce69, PDF_CMAP_RANGE, 2972 },
	{ 0xce6a, 0xce6a, PDF_CMAP_SINGLE, 16255 },
	{ 0xce6b, 0xce6b, PDF_CMAP_SINGLE, 2974 },
	{ 0xce6c, 0xce6c, PDF_CMAP_SINGLE, 16256 },
	{ 0xce6d, 0xce6d, PDF_CMAP_SINGLE, 2975 },
	{ 0xce6e, 0xce73, PDF_CMAP_RANGE, 16257 },
	{ 0xce74, 0xce75, PDF_CMAP_RANGE, 2976 },
	{ 0xce76, 0xce77, PDF_CMAP_RANGE, 16263 },
	{ 0xce78, 0xce78, PDF_CMAP_SINGLE, 2978 },
	{ 0xce79, 0xce7b, PDF_CMAP_RANGE, 16265 },
	{ 0xce7c, 0xce7c, PDF_CMAP_SINGLE, 2979 },
	{ 0xce7d, 0xce83, PDF_CMAP_RANGE, 16268 },
	{ 0xce84, 0xce85, PDF_CMAP_RANGE, 2980 },
	{ 0xce86, 0xce86, PDF_CMAP_SINGLE, 16275 },
	{ 0xce87, 0xce87, PDF_CMAP_SINGLE, 2982 },
	{ 0xce88, 0xce88, PDF_CMAP_SINGLE, 16276 },
	{ 0xce89, 0xce89, PDF_CMAP_SINGLE, 2983 },
	{ 0xce8a, 0xce8f, PDF_CMAP_RANGE, 16277 },
	{ 0xce90, 0xce91, PDF_CMAP_RANGE, 2984 },
	{ 0xce92, 0xce93, PDF_CMAP_RANGE, 16283 },
	{ 0xce94, 0xce94, PDF_CMAP_SINGLE, 2986 },
	{ 0xce95, 0xce97, PDF_CMAP_RANGE, 16285 },
	{ 0xce98, 0xce98, PDF_CMAP_SINGLE, 2987 },
	{ 0xce99, 0xce9f, PDF_CMAP_RANGE, 16288 },
	{ 0xcea0, 0xcea1, PDF_CMAP_RANGE, 2988 },
	{ 0xcea2, 0xcea2, PDF_CMAP_SINGLE, 16295 },
	{ 0xcea3, 0xcea5, PDF_CMAP_RANGE, 2990 },
	{ 0xcea6, 0xceab, PDF_CMAP_RANGE, 16296 },
	{ 0xceac, 0xcead, PDF_CMAP_RANGE, 2993 },
	{ 0xceae, 0xcec0, PDF_CMAP_RANGE, 16302 },
	{ 0xcec1, 0xcec1, PDF_CMAP_SINGLE, 2995 },
	{ 0xcec2, 0xcee3, PDF_CMAP_RANGE, 16321 },
	{ 0xcee4, 0xcee5, PDF_CMAP_RANGE, 2996 },
	{ 0xcee6, 0xcee7, PDF_CMAP_RANGE, 16355 },
	{ 0xcee8, 0xcee8, PDF_CMAP_SINGLE, 2998 },
	{ 0xcee9, 0xceea, PDF_CMAP_RANGE, 16357 },
	{ 0xceeb, 0xceec, PDF_CMAP_RANGE, 2999 },
	{ 0xceed, 0xcef3, PDF_CMAP_RANGE, 16359 },
	{ 0xcef4, 0xcef5, PDF_CMAP_RANGE, 3001 },
	{ 0xcef6, 0xcef6, PDF_CMAP_SINGLE, 16366 },
	{ 0xcef7, 0xcef9, PDF_CMAP_RANGE, 3003 },
	{ 0xcefa, 0xceff, PDF_CMAP_RANGE, 16367 },
	{ 0xcf00, 0xcf01, PDF_CMAP_RANGE, 3006 },
	{ 0xcf02, 0xcf03, PDF_CMAP_RANGE, 16373 },
	{ 0xcf04, 0xcf04, PDF_CMAP_SINGLE, 3008 },
	{ 0xcf05, 0xcf07, PDF_CMAP_RANGE, 16375 },
	{ 0xcf08, 0xcf08, PDF_CMAP_SINGLE, 3009 },
	{ 0xcf09, 0xcf0f, PDF_CMAP_RANGE, 16378 },
	{ 0xcf10, 0xcf11, PDF_CMAP_RANGE, 3010 },
	{ 0xcf12, 0xcf12, PDF_CMAP_SINGLE, 16385 },
	{ 0xcf13, 0xcf13, PDF_CMAP_SINGLE, 3012 },
	{ 0xcf14, 0xcf14, PDF_CMAP_SINGLE, 16386 },
	{ 0xcf15, 0xcf15, PDF_CMAP_SINGLE, 3013 },
	{ 0xcf16, 0xcf1b, PDF_CMAP_RANGE, 16387 },
	{ 0xcf1c, 0xcf1c, PDF_CMAP_SINGLE, 3014 },
	{ 0xcf1d, 0xcf1f, PDF_CMAP_RANGE, 16393 },
	{ 0xcf20, 0xcf20, PDF_CMAP_SINGLE, 3015 },
	{ 0xcf21, 0xcf23, PDF_CMAP_RANGE, 16396 },
	{ 0xcf24, 0xcf24, PDF_CMAP_SINGLE, 3016 },
	{ 0xcf25, 0xcf2b, PDF_CMAP_RANGE, 16399 },
	{ 0xcf2c, 0xcf2d, PDF_CMAP_RANGE, 3017 },
	{ 0xcf2e, 0xcf2e, PDF_CMAP_SINGLE, 16406 },
	{ 0xcf2f, 0xcf31, PDF_CMAP_RANGE, 3019 },
	{ 0xcf32, 0xcf37, PDF_CMAP_RANGE, 16407 },
	{ 0xcf38, 0xcf38, PDF_CMAP_SINGLE, 3022 },
	{ 0xcf39, 0xcf53, PDF_CMAP_RANGE, 16413 },
	{ 0xcf54, 0xcf55, PDF_CMAP_RANGE, 3023 },
	{ 0xcf56, 0xcf57, PDF_CMAP_RANGE, 16440 },
	{ 0xcf58, 0xcf58, PDF_CMAP_SINGLE, 3025 },
	{ 0xcf59, 0xcf5b, PDF_CMAP_RANGE, 16442 },
	{ 0xcf5c, 0xcf5c, PDF_CMAP_SINGLE, 3026 },
	{ 0xcf5d, 0xcf63, PDF_CMAP_RANGE, 16445 },
	{ 0xcf64, 0xcf65, PDF_CMAP_RANGE, 3027 },
	{ 0xcf66, 0xcf66, PDF_CMAP_SINGLE, 16452 },
	{ 0xcf67, 0xcf67, PDF_CMAP_SINGLE, 3029 },
	{ 0xcf68, 0xcf68, PDF_CMAP_SINGLE, 16453 },
	{ 0xcf69, 0xcf69, PDF_CMAP_SINGLE, 3030 },
	{ 0xcf6a, 0xcf6f, PDF_CMAP_RANGE, 16454 },
	{ 0xcf70, 0xcf71, PDF_CMAP_RANGE, 3031 },
	{ 0xcf72, 0xcf73, PDF_CMAP_RANGE, 16460 },
	{ 0xcf74, 0xcf74, PDF_CMAP_SINGLE, 3033 },
	{ 0xcf75, 0xcf77, PDF_CMAP_RANGE, 16462 },
	{ 0xcf78, 0xcf78, PDF_CMAP_SINGLE, 3034 },
	{ 0xcf79, 0xcf7f, PDF_CMAP_RANGE, 16465 },
	{ 0xcf80, 0xcf80, PDF_CMAP_SINGLE, 3035 },
	{ 0xcf81, 0xcf84, PDF_CMAP_RANGE, 16472 },
	{ 0xcf85, 0xcf85, PDF_CMAP_SINGLE, 3036 },
	{ 0xcf86, 0xcf8b, PDF_CMAP_RANGE, 16476 },
	{ 0xcf8c, 0xcf8c, PDF_CMAP_SINGLE, 3037 },
	{ 0xcf8d, 0xcfa0, PDF_CMAP_RANGE, 16482 },
	{ 0xcfa1, 0xcfa1, PDF_CMAP_SINGLE, 3038 },
	{ 0xcfa2, 0xcfa7, PDF_CMAP_RANGE, 16502 },
	{ 0xcfa8, 0xcfa8, PDF_CMAP_SINGLE, 3039 },
	{ 0xcfa9, 0xcfaf, PDF_CMAP_RANGE, 16508 },
	{ 0xcfb0, 0xcfb0, PDF_CMAP_SINGLE, 3040 },
	{ 0xcfb1, 0xcfc3, PDF_CMAP_RANGE, 16515 },
	{ 0xcfc4, 0xcfc4, PDF_CMAP_SINGLE, 3041 },
	{ 0xcfc5, 0xcfdf, PDF_CMAP_RANGE, 16534 },
	{ 0xcfe0, 0xcfe1, PDF_CMAP_RANGE, 3042 },
	{ 0xcfe2, 0xcfe3, PDF_CMAP_RANGE, 16561 },
	{ 0xcfe4, 0xcfe4, PDF_CMAP_SINGLE, 3044 },
	{ 0xcfe5, 0xcfe7, PDF_CMAP_RANGE, 16563 },
	{ 0xcfe8, 0xcfe8, PDF_CMAP_SINGLE, 3045 },
	{ 0xcfe9, 0xcfef, PDF_CMAP_RANGE, 16566 },
	{ 0xcff0, 0xcff1, PDF_CMAP_RANGE, 3046 },
	{ 0xcff2, 0xcff2, PDF_CMAP_SINGLE, 16573 },
	{ 0xcff3, 0xcff3, PDF_CMAP_SINGLE, 3048 },
	{ 0xcff4, 0xcff4, PDF_CMAP_SINGLE, 16574 },
	{ 0xcff5, 0xcff5, PDF_CMAP_SINGLE, 3049 },
	{ 0xcff6, 0xcffb, PDF_CMAP_RANGE, 16575 },
	{ 0xcffc, 0xcffc, PDF_CMAP_SINGLE, 3050 },
	{ 0xcffd, 0xcfff, PDF_CMAP_RANGE, 16581 },
	{ 0xd000, 0xd000, PDF_CMAP_SINGLE, 3051 },
	{ 0xd001, 0xd003, PDF_CMAP_RANGE, 16584 },
	{ 0xd004, 0xd004, PDF_CMAP_SINGLE, 3052 },
	{ 0xd005, 0xd010, PDF_CMAP_RANGE, 16587 },
	{ 0xd011, 0xd011, PDF_CMAP_SINGLE, 3053 },
	{ 0xd012, 0xd017, PDF_CMAP_RANGE, 16599 },
	{ 0xd018, 0xd018, PDF_CMAP_SINGLE, 3054 },
	{ 0xd019, 0xd02c, PDF_CMAP_RANGE, 16605 },
	{ 0xd02d, 0xd02d, PDF_CMAP_SINGLE, 3055 },
	{ 0xd02e, 0xd033, PDF_CMAP_RANGE, 16625 },
	{ 0xd034, 0xd035, PDF_CMAP_RANGE, 3056 },
	{ 0xd036, 0xd037, PDF_CMAP_RANGE, 16631 },
	{ 0xd038, 0xd038, PDF_CMAP_SINGLE, 3058 },
	{ 0xd039, 0xd03b, PDF_CMAP_RANGE, 16633 },
	{ 0xd03c, 0xd03c, PDF_CMAP_SINGLE, 3059 },
	{ 0xd03d, 0xd043, PDF_CMAP_RANGE, 16636 },
	{ 0xd044, 0xd045, PDF_CMAP_RANGE, 3060 },
	{ 0xd046, 0xd046, PDF_CMAP_SINGLE, 16643 },
	{ 0xd047, 0xd047, PDF_CMAP_SINGLE, 3062 },
	{ 0xd048, 0xd048, PDF_CMAP_SINGLE, 16644 },
	{ 0xd049, 0xd049, PDF_CMAP_SINGLE, 3063 },
	{ 0xd04a, 0xd04f, PDF_CMAP_RANGE, 16645 },
	{ 0xd050, 0xd050, PDF_CMAP_SINGLE, 3064 },
	{ 0xd051, 0xd053, PDF_CMAP_RANGE, 16651 },
	{ 0xd054, 0xd054, PDF_CMAP_SINGLE, 3065 },
	{ 0xd055, 0xd057, PDF_CMAP_RANGE, 16654 },
	{ 0xd058, 0xd058, PDF_CMAP_SINGLE, 3066 },
	{ 0xd059, 0xd05f, PDF_CMAP_RANGE, 16657 },
	{ 0xd060, 0xd060, PDF_CMAP_SINGLE, 3067 },
	{ 0xd061, 0xd06b, PDF_CMAP_RANGE, 16664 },
	{ 0xd06c, 0xd06d, PDF_CMAP_RANGE, 3068 },
	{ 0xd06e, 0xd06f, PDF_CMAP_RANGE, 16675 },
	{ 0xd070, 0xd070, PDF_CMAP_SINGLE, 3070 },
	{ 0xd071, 0xd073, PDF_CMAP_RANGE, 16677 },
	{ 0xd074, 0xd074, PDF_CMAP_SINGLE, 3071 },
	{ 0xd075, 0xd07b, PDF_CMAP_RANGE, 16680 },
	{ 0xd07c, 0xd07d, PDF_CMAP_RANGE, 3072 },
	{ 0xd07e, 0xd080, PDF_CMAP_RANGE, 16687 },
	{ 0xd081, 0xd081, PDF_CMAP_SINGLE, 3074 },
	{ 0xd082, 0xd0a3, PDF_CMAP_RANGE, 16690 },
	{ 0xd0a4, 0xd0a5, PDF_CMAP_RANGE, 3075 },
	{ 0xd0a6, 0xd0a7, PDF_CMAP_RANGE, 16724 },
	{ 0xd0a8, 0xd0a8, PDF_CMAP_SINGLE, 3077 },
	{ 0xd0a9, 0xd0ab, PDF_CMAP_RANGE, 16726 },
	{ 0xd0ac, 0xd0ac, PDF_CMAP_SINGLE, 3078 },
	{ 0xd0ad, 0xd0b3, PDF_CMAP_RANGE, 16729 },
	{ 0xd0b4, 0xd0b5, PDF_CMAP_RANGE, 3079 },
	{ 0xd0b6, 0xd0b6, PDF_CMAP_SINGLE, 16736 },
	{ 0xd0b7, 0xd0b7, PDF_CMAP_SINGLE, 3081 },
	{ 0xd0b8, 0xd0b8, PDF_CMAP_SINGLE, 16737 },
	{ 0xd0b9, 0xd0b9, PDF_CMAP_SINGLE, 3082 },
	{ 0xd0ba, 0xd0bf, PDF_CMAP_RANGE, 16738 },
	{ 0xd0c0, 0xd0c1, PDF_CMAP_RANGE, 3083 },
	{ 0xd0c2, 0xd0c3, PDF_CMAP_RANGE, 16744 },
	{ 0xd0c4, 0xd0c4, PDF_CMAP_SINGLE, 3085 },
	{ 0xd0c5, 0xd0c7, PDF_CMAP_RANGE, 16746 },
	{ 0xd0c8, 0xd0c9, PDF_CMAP_RANGE, 3086 },
	{ 0xd0ca, 0xd0cf, PDF_CMAP_RANGE, 16749 },
	{ 0xd0d0, 0xd0d1, PDF_CMAP_RANGE, 3088 },
	{ 0xd0d2, 0xd0d2, PDF_CMAP_SINGLE, 16755 },
	{ 0xd0d3, 0xd0d5, PDF_CMAP_RANGE, 3090 },
	{ 0xd0d6, 0xd0db, PDF_CMAP_RANGE, 16756 },
	{ 0xd0dc, 0xd0dd, PDF_CMAP_RANGE, 3093 },
	{ 0xd0de, 0xd0df, PDF_CMAP_RANGE, 16762 },
	{ 0xd0e0, 0xd0e0, PDF_CMAP_SINGLE, 3095 },
	{ 0xd0e1, 0xd0e3, PDF_CMAP_RANGE, 16764 },
	{ 0xd0e4, 0xd0e4, PDF_CMAP_SINGLE, 3096 },
	{ 0xd0e5, 0xd0eb, PDF_CMAP_RANGE, 16767 },
	{ 0xd0ec, 0xd0ed, PDF_CMAP_RANGE, 3097 },
	{ 0xd0ee, 0xd0ee, PDF_CMAP_SINGLE, 16774 },
	{ 0xd0ef, 0xd0f1, PDF_CMAP_RANGE, 3099 },
	{ 0xd0f2, 0xd0f7, PDF_CMAP_RANGE, 16775 },
	{ 0xd0f8, 0xd0f8, PDF_CMAP_SINGLE, 3102 },
	{ 0xd0f9, 0xd0ff, PDF_CMAP_RANGE, 16781 },
	{ 0xd100, 0xd10c, PDF_CMAP_RANGE, 16788 },
	{ 0xd10d, 0xd10d, PDF_CMAP_SINGLE, 3103 },
	{ 0xd10e, 0xd12f, PDF_CMAP_RANGE, 16801 },
	{ 0xd130, 0xd131, PDF_CMAP_RANGE, 3104 },
	{ 0xd132, 0xd133, PDF_CMAP_RANGE, 16835 },
	{ 0xd134, 0xd134, PDF_CMAP_SINGLE, 3106 },
	{ 0xd135, 0xd137, PDF_CMAP_RANGE, 16837 },
	{ 0xd138, 0xd138, PDF_CMAP_SINGLE, 3107 },
	{ 0xd139, 0xd139, PDF_CMAP_SINGLE, 16840 },
	{ 0xd13a, 0xd13a, PDF_CMAP_SINGLE, 3108 },
	{ 0xd13b, 0xd13f, PDF_CMAP_RANGE, 16841 },
	{ 0xd140, 0xd141, PDF_CMAP_RANGE, 3109 },
	{ 0xd142, 0xd142, PDF_CMAP_SINGLE, 16846 },
	{ 0xd143, 0xd145, PDF_CMAP_RANGE, 3111 },
	{ 0xd146, 0xd14b, PDF_CMAP_RANGE, 16847 },
	{ 0xd14c, 0xd14d, PDF_CMAP_RANGE, 3114 },
	{ 0xd14e, 0xd14f, PDF_CMAP_RANGE, 16853 },
	{ 0xd150, 0xd150, PDF_CMAP_SINGLE, 3116 },
	{ 0xd151, 0xd153, PDF_CMAP_RANGE, 16855 },
	{ 0xd154, 0xd154, PDF_CMAP_SINGLE, 3117 },
	{ 0xd155, 0xd15b, PDF_CMAP_RANGE, 16858 },
	{ 0xd15c, 0xd15d, PDF_CMAP_RANGE, 3118 },
	{ 0xd15e, 0xd15e, PDF_CMAP_SINGLE, 16865 },
	{ 0xd15f, 0xd15f, PDF_CMAP_SINGLE, 3120 },
	{ 0xd160, 0xd160, PDF_CMAP_SINGLE, 16866 },
	{ 0xd161, 0xd161, PDF_CMAP_SINGLE, 3121 },
	{ 0xd162, 0xd167, PDF_CMAP_RANGE, 16867 },
	{ 0xd168, 0xd168, PDF_CMAP_SINGLE, 3122 },
	{ 0xd169, 0xd16b, PDF_CMAP_RANGE, 16873 },
	{ 0xd16c, 0xd16c, PDF_CMAP_SINGLE, 3123 },
	{ 0xd16d, 0xd17b, PDF_CMAP_RANGE, 16876 },
	{ 0xd17c, 0xd17c, PDF_CMAP_SINGLE, 3124 },
	{ 0xd17d, 0xd183, PDF_CMAP_RANGE, 16891 },
	{ 0xd184, 0xd184, PDF_CMAP_SINGLE, 3125 },
	{ 0xd185, 0xd187, PDF_CMAP_RANGE, 16898 },
	{ 0xd188, 0xd188, PDF_CMAP_SINGLE, 3126 },
	{ 0xd189, 0xd19f, PDF_CMAP_RANGE, 16901 },
	{ 0xd1a0, 0xd1a1, PDF_CMAP_RANGE, 3127 },
	{ 0xd1a2, 0xd1a3, PDF_CMAP_RANGE, 16924 },
	{ 0xd1a4, 0xd1a4, PDF_CMAP_SINGLE, 3129 },
	{ 0xd1a5, 0xd1a7, PDF_CMAP_RANGE, 16926 },
	{ 0xd1a8, 0xd1a8, PDF_CMAP_SINGLE, 3130 },
	{ 0xd1a9, 0xd1af, PDF_CMAP_RANGE, 16929 },
	{ 0xd1b0, 0xd1b1, PDF_CMAP_RANGE, 3131 },
	{ 0xd1b2, 0xd1b2, PDF_CMAP_SINGLE, 16936 },
	{ 0xd1b3, 0xd1b3, PDF_CMAP_SINGLE, 3133 },
	{ 0xd1b4, 0xd1b4, PDF_CMAP_SINGLE, 16937 },
	{ 0xd1b5, 0xd1b5, PDF_CMAP_SINGLE, 3134 },
	{ 0xd1b6, 0xd1b9, PDF_CMAP_RANGE, 16938 },
	{ 0xd1ba, 0xd1ba, PDF_CMAP_SINGLE, 3135 },
	{ 0xd1bb, 0xd1bb, PDF_CMAP_SINGLE, 16942 },
	{ 0xd1bc, 0xd1bc, PDF_CMAP_SINGLE, 3136 },
	{ 0xd1bd, 0xd1bf, PDF_CMAP_RANGE, 16943 },
	{ 0xd1c0, 0xd1c0, PDF_CMAP_SINGLE, 3137 },
	{ 0xd1c1, 0xd1d7, PDF_CMAP_RANGE, 16946 },
	{ 0xd1d8, 0xd1d8, PDF_CMAP_SINGLE, 3138 },
	{ 0xd1d9, 0xd1f3, PDF_CMAP_RANGE, 16969 },
	{ 0xd1f4, 0xd1f4, PDF_CMAP_SINGLE, 3139 },
	{ 0xd1f5, 0xd1f7, PDF_CMAP_RANGE, 16996 },
	{ 0xd1f8, 0xd1f8, PDF_CMAP_SINGLE, 3140 },
	{ 0xd1f9, 0xd1ff, PDF_CMAP_RANGE, 16999 },
	{ 0xd200, 0xd206, PDF_CMAP_RANGE, 17006 },
	{ 0xd207, 0xd207, PDF_CMAP_SINGLE, 3141 },
	{ 0xd208, 0xd208, PDF_CMAP_SINGLE, 17013 },
	{ 0xd209, 0xd209, PDF_CMAP_SINGLE, 3142 },
	{ 0xd20a, 0xd20f, PDF_CMAP_RANGE, 17014 },
	{ 0xd210, 0xd210, PDF_CMAP_SINGLE, 3143 },
	{ 0xd211, 0xd22b, PDF_CMAP_RANGE, 17020 },
	{ 0xd22c, 0xd22d, PDF_CMAP_RANGE, 3144 },
	{ 0xd22e, 0xd22f, PDF_CMAP_RANGE, 17047 },
	{ 0xd230, 0xd230, PDF_CMAP_SINGLE, 3146 },
	{ 0xd231, 0xd233, PDF_CMAP_RANGE, 17049 },
	{ 0xd234, 0xd234, PDF_CMAP_SINGLE, 3147 },
	{ 0xd235, 0xd23b, PDF_CMAP_RANGE, 17052 },
	{ 0xd23c, 0xd23d, PDF_CMAP_RANGE, 3148 },
	{ 0xd23e, 0xd23e, PDF_CMAP_SINGLE, 17059 },
	{ 0xd23f, 0xd23f, PDF_CMAP_SINGLE, 3150 },
	{ 0xd240, 0xd240, PDF_CMAP_SINGLE, 17060 },
	{ 0xd241, 0xd241, PDF_CMAP_SINGLE, 3151 },
	{ 0xd242, 0xd247, PDF_CMAP_RANGE, 17061 },
	{ 0xd248, 0xd248, PDF_CMAP_SINGLE, 3152 },
	{ 0xd249, 0xd25b, PDF_CMAP_RANGE, 17067 },
	{ 0xd25c, 0xd25c, PDF_CMAP_SINGLE, 3153 },
	{ 0xd25d, 0xd263, PDF_CMAP_RANGE, 17086 },
	{ 0xd264, 0xd264, PDF_CMAP_SINGLE, 3154 },
	{ 0xd265, 0xd27f, PDF_CMAP_RANGE, 17093 },
	{ 0xd280, 0xd281, PDF_CMAP_RANGE, 3155 },
	{ 0xd282, 0xd283, PDF_CMAP_RANGE, 17120 },
	{ 0xd284, 0xd284, PDF_CMAP_SINGLE, 3157 },
	{ 0xd285, 0xd287, PDF_CMAP_RANGE, 17122 },
	{ 0xd288, 0xd288, PDF_CMAP_SINGLE, 3158 },
	{ 0xd289, 0xd28f, PDF_CMAP_RANGE, 17125 },
	{ 0xd290, 0xd291, PDF_CMAP_RANGE, 3159 },
	{ 0xd292, 0xd294, PDF_CMAP_RANGE, 17132 },
	{ 0xd295, 0xd295, PDF_CMAP_SINGLE, 3161 },
	{ 0xd296, 0xd29b, PDF_CMAP_RANGE, 17135 },
	{ 0xd29c, 0xd29c, PDF_CMAP_SINGLE, 3162 },
	{ 0xd29d, 0xd29f, PDF_CMAP_RANGE, 17141 },
	{ 0xd2a0, 0xd2a0, PDF_CMAP_SINGLE, 3163 },
	{ 0xd2a1, 0xd2a3, PDF_CMAP_RANGE, 17144 },
	{ 0xd2a4, 0xd2a4, PDF_CMAP_SINGLE, 3164 },
	{ 0xd2a5, 0xd2ab, PDF_CMAP_RANGE, 17147 },
	{ 0xd2ac, 0xd2ac, PDF_CMAP_SINGLE, 3165 },
	{ 0xd2ad, 0xd2b0, PDF_CMAP_RANGE, 17154 },
	{ 0xd2b1, 0xd2b1, PDF_CMAP_SINGLE, 3166 },
	{ 0xd2b2, 0xd2b7, PDF_CMAP_RANGE, 17158 },
	{ 0xd2b8, 0xd2b9, PDF_CMAP_RANGE, 3167 },
	{ 0xd2ba, 0xd2bb, PDF_CMAP_RANGE, 17164 },
	{ 0xd2bc, 0xd2bc, PDF_CMAP_SINGLE, 3169 },
	{ 0xd2bd, 0xd2be, PDF_CMAP_RANGE, 17166 },
	{ 0xd2bf, 0xd2c0, PDF_CMAP_RANGE, 3170 },
	{ 0xd2c1, 0xd2c1, PDF_CMAP_SINGLE, 17168 },
	{ 0xd2c2, 0xd2c2, PDF_CMAP_SINGLE, 3172 },
	{ 0xd2c3, 0xd2c7, PDF_CMAP_RANGE, 17169 },
	{ 0xd2c8, 0xd2c9, PDF_CMAP_RANGE, 3173 },
	{ 0xd2ca, 0xd2ca, PDF_CMAP_SINGLE, 17174 },
	{ 0xd2cb, 0xd2cb, PDF_CMAP_SINGLE, 3175 },
	{ 0xd2cc, 0xd2d3, PDF_CMAP_RANGE, 17175 },
	{ 0xd2d4, 0xd2d4, PDF_CMAP_SINGLE, 3176 },
	{ 0xd2d5, 0xd2d7, PDF_CMAP_RANGE, 17183 },
	{ 0xd2d8, 0xd2d8, PDF_CMAP_SINGLE, 3177 },
	{ 0xd2d9, 0xd2db, PDF_CMAP_RANGE, 17186 },
	{ 0xd2dc, 0xd2dc, PDF_CMAP_SINGLE, 3178 },
	{ 0xd2dd, 0xd2e3, PDF_CMAP_RANGE, 17189 },
	{ 0xd2e4, 0xd2e5, PDF_CMAP_RANGE, 3179 },
	{ 0xd2e6, 0xd2ef, PDF_CMAP_RANGE, 17196 },
	{ 0xd2f0, 0xd2f1, PDF_CMAP_RANGE, 3181 },
	{ 0xd2f2, 0xd2f3, PDF_CMAP_RANGE, 17206 },
	{ 0xd2f4, 0xd2f4, PDF_CMAP_SINGLE, 3183 },
	{ 0xd2f5, 0xd2f7, PDF_CMAP_RANGE, 17208 },
	{ 0xd2f8, 0xd2f8, PDF_CMAP_SINGLE, 3184 },
	{ 0xd2f9, 0xd2ff, PDF_CMAP_RANGE, 17211 },
	{ 0xd300, 0xd301, PDF_CMAP_RANGE, 3185 },
	{ 0xd302, 0xd302, PDF_CMAP_SINGLE, 17218 },
	{ 0xd303, 0xd303, PDF_CMAP_SINGLE, 3187 },
	{ 0xd304, 0xd304, PDF_CMAP_SINGLE, 17219 },
	{ 0xd305, 0xd305, PDF_CMAP_SINGLE, 3188 },
	{ 0xd306, 0xd30b, PDF_CMAP_RANGE, 17220 },
	{ 0xd30c, 0xd30e, PDF_CMAP_RANGE, 3189 },
	{ 0xd30f, 0xd30f, PDF_CMAP_SINGLE, 17226 },
	{ 0xd310, 0xd310, PDF_CMAP_SINGLE, 3192 },
	{ 0xd311, 0xd313, PDF_CMAP_RANGE, 17227 },
	{ 0xd314, 0xd314, PDF_CMAP_SINGLE, 3193 },
	{ 0xd315, 0xd315, PDF_CMAP_SINGLE, 17230 },
	{ 0xd316, 0xd316, PDF_CMAP_SINGLE, 3194 },
	{ 0xd317, 0xd31b, PDF_CMAP_RANGE, 17231 },
	{ 0xd31c, 0xd31d, PDF_CMAP_RANGE, 3195 },
	{ 0xd31e, 0xd31e, PDF_CMAP_SINGLE, 17236 },
	{ 0xd31f, 0xd321, PDF_CMAP_RANGE, 3197 },
	{ 0xd322, 0xd324, PDF_CMAP_RANGE, 17237 },
	{ 0xd325, 0xd325, PDF_CMAP_SINGLE, 3200 },
	{ 0xd326, 0xd327, PDF_CMAP_RANGE, 17240 },
	{ 0xd328, 0xd329, PDF_CMAP_RANGE, 3201 },
	{ 0xd32a, 0xd32b, PDF_CMAP_RANGE, 17242 },
	{ 0xd32c, 0xd32c, PDF_CMAP_SINGLE, 3203 },
	{ 0xd32d, 0xd32f, PDF_CMAP_RANGE, 17244 },
	{ 0xd330, 0xd330, PDF_CMAP_SINGLE, 3204 },
	{ 0xd331, 0xd337, PDF_CMAP_RANGE, 17247 },
	{ 0xd338, 0xd339, PDF_CMAP_RANGE, 3205 },
	{ 0xd33a, 0xd33a, PDF_CMAP_SINGLE, 17254 },
	{ 0xd33b, 0xd33d, PDF_CMAP_RANGE, 3207 },
	{ 0xd33e, 0xd343, PDF_CMAP_RANGE, 17255 },
	{ 0xd344, 0xd345, PDF_CMAP_RANGE, 3210 },
	{ 0xd346, 0xd37b, PDF_CMAP_RANGE, 17261 },
	{ 0xd37c, 0xd37d, PDF_CMAP_RANGE, 3212 },
	{ 0xd37e, 0xd37f, PDF_CMAP_RANGE, 17315 },
	{ 0xd380, 0xd380, PDF_CMAP_SINGLE, 3214 },
	{ 0xd381, 0xd383, PDF_CMAP_RANGE, 17317 },
	{ 0xd384, 0xd384, PDF_CMAP_SINGLE, 3215 },
	{ 0xd385, 0xd38b, PDF_CMAP_RANGE, 17320 },
	{ 0xd38c, 0xd38d, PDF_CMAP_RANGE, 3216 },
	{ 0xd38e, 0xd38e, PDF_CMAP_SINGLE, 17327 },
	{ 0xd38f, 0xd391, PDF_CMAP_RANGE, 3218 },
	{ 0xd392, 0xd397, PDF_CMAP_RANGE, 17328 },
	{ 0xd398, 0xd399, PDF_CMAP_RANGE, 3221 },
	{ 0xd39a, 0xd39b, PDF_CMAP_RANGE, 17334 },
	{ 0xd39c, 0xd39c, PDF_CMAP_SINGLE, 3223 },
	{ 0xd39d, 0xd39f, PDF_CMAP_RANGE, 17336 },
	{ 0xd3a0, 0xd3a0, PDF_CMAP_SINGLE, 3224 },
	{ 0xd3a1, 0xd3a7, PDF_CMAP_RANGE, 17339 },
	{ 0xd3a8, 0xd3a9, PDF_CMAP_RANGE, 3225 },
	{ 0xd3aa, 0xd3aa, PDF_CMAP_SINGLE, 17346 },
	{ 0xd3ab, 0xd3ab, PDF_CMAP_SINGLE, 3227 },
	{ 0xd3ac, 0xd3ac, PDF_CMAP_SINGLE, 17347 },
	{ 0xd3ad, 0xd3ad, PDF_CMAP_SINGLE, 3228 },
	{ 0xd3ae, 0xd3b3, PDF_CMAP_RANGE, 17348 },
	{ 0xd3b4, 0xd3b4, PDF_CMAP_SINGLE, 3229 },
	{ 0xd3b5, 0xd3b7, PDF_CMAP_RANGE, 17354 },
	{ 0xd3b8, 0xd3b8, PDF_CMAP_SINGLE, 3230 },
	{ 0xd3b9, 0xd3bb, PDF_CMAP_RANGE, 17357 },
	{ 0xd3bc, 0xd3bc, PDF_CMAP_SINGLE, 3231 },
	{ 0xd3bd, 0xd3c3, PDF_CMAP_RANGE, 17360 },
	{ 0xd3c4, 0xd3c5, PDF_CMAP_RANGE, 3232 },
	{ 0xd3c6, 0xd3c7, PDF_CMAP_RANGE, 17367 },
	{ 0xd3c8, 0xd3c9, PDF_CMAP_RANGE, 3234 },
	{ 0xd3ca, 0xd3cf, PDF_CMAP_RANGE, 17369 },
	{ 0xd3d0, 0xd3d0, PDF_CMAP_SINGLE, 3236 },
	{ 0xd3d1, 0xd3d7, PDF_CMAP_RANGE, 17375 },
	{ 0xd3d8, 0xd3d8, PDF_CMAP_SINGLE, 3237 },
	{ 0xd3d9, 0xd3e0, PDF_CMAP_RANGE, 17382 },
	{ 0xd3e1, 0xd3e1, PDF_CMAP_SINGLE, 3238 },
	{ 0xd3e2, 0xd3e2, PDF_CMAP_SINGLE, 17390 },
	{ 0xd3e3, 0xd3e3, PDF_CMAP_SINGLE, 3239 },
	{ 0xd3e4, 0xd3eb, PDF_CMAP_RANGE, 17391 },
	{ 0xd3ec, 0xd3ed, PDF_CMAP_RANGE, 3240 },
	{ 0xd3ee, 0xd3ef, PDF_CMAP_RANGE, 17399 },
	{ 0xd3f0, 0xd3f0, PDF_CMAP_SINGLE, 3242 },
	{ 0xd3f1, 0xd3f3, PDF_CMAP_RANGE, 17401 },
	{ 0xd3f4, 0xd3f4, PDF_CMAP_SINGLE, 3243 },
	{ 0xd3f5, 0xd3fb, PDF_CMAP_RANGE, 17404 },
	{ 0xd3fc, 0xd3fd, PDF_CMAP_RANGE, 3244 },
	{ 0xd3fe, 0xd3fe, PDF_CMAP_SINGLE, 17411 },
	{ 0xd3ff, 0xd3ff, PDF_CMAP_SINGLE, 3246 },
	{ 0xd400, 0xd400, PDF_CMAP_SINGLE, 17412 },
	{ 0xd401, 0xd401, PDF_CMAP_SINGLE, 3247 },
	{ 0xd402, 0xd407, PDF_CMAP_RANGE, 17413 },
	{ 0xd408, 0xd408, PDF_CMAP_SINGLE, 3248 },
	{ 0xd409, 0xd41c, PDF_CMAP_RANGE, 17419 },
	{ 0xd41d, 0xd41d, PDF_CMAP_SINGLE, 3249 },
	{ 0xd41e, 0xd43f, PDF_CMAP_RANGE, 17439 },
	{ 0xd440, 0xd440, PDF_CMAP_SINGLE, 3250 },
	{ 0xd441, 0xd443, PDF_CMAP_RANGE, 17473 },
	{ 0xd444, 0xd444, PDF_CMAP_SINGLE, 3251 },
	{ 0xd445, 0xd45b, PDF_CMAP_RANGE, 17476 },
	{ 0xd45c, 0xd45c, PDF_CMAP_SINGLE, 3252 },
	{ 0xd45d, 0xd45f, PDF_CMAP_RANGE, 17499 },
	{ 0xd460, 0xd460, PDF_CMAP_SINGLE, 3253 },
	{ 0xd461, 0xd463, PDF_CMAP_RANGE, 17502 },
	{ 0xd464, 0xd464, PDF_CMAP_SINGLE, 3254 },
	{ 0xd465, 0xd46c, PDF_CMAP_RANGE, 17505 },
	{ 0xd46d, 0xd46d, PDF_CMAP_SINGLE, 3255 },
	{ 0xd46e, 0xd46e, PDF_CMAP_SINGLE, 17513 },
	{ 0xd46f, 0xd46f, PDF_CMAP_SINGLE, 3256 },
	{ 0xd470, 0xd477, PDF_CMAP_RANGE, 17514 },
	{ 0xd478, 0xd479, PDF_CMAP_RANGE, 3257 },
	{ 0xd47a, 0xd47b, PDF_CMAP_RANGE, 17522 },
	{ 0xd47c, 0xd47c, PDF_CMAP_SINGLE, 3259 },
	{ 0xd47d, 0xd47e, PDF_CMAP_RANGE, 17524 },
	{ 0xd47f, 0xd480, PDF_CMAP_RANGE, 3260 },
	{ 0xd481, 0xd481, PDF_CMAP_SINGLE, 17526 },
	{ 0xd482, 0xd482, PDF_CMAP_SINGLE, 3262 },
	{ 0xd483, 0xd487, PDF_CMAP_RANGE, 17527 },
	{ 0xd488, 0xd489, PDF_CMAP_RANGE, 3263 },
	{ 0xd48a, 0xd48a, PDF_CMAP_SINGLE, 17532 },
	{ 0xd48b, 0xd48b, PDF_CMAP_SINGLE, 3265 },
	{ 0xd48c, 0xd48c, PDF_CMAP_SINGLE, 17533 },
	{ 0xd48d, 0xd48d, PDF_CMAP_SINGLE, 3266 },
	{ 0xd48e, 0xd493, PDF_CMAP_RANGE, 17534 },
	{ 0xd494, 0xd494, PDF_CMAP_SINGLE, 3267 },
	{ 0xd495, 0xd4a8, PDF_CMAP_RANGE, 17540 },
	{ 0xd4a9, 0xd4a9, PDF_CMAP_SINGLE, 3268 },
	{ 0xd4aa, 0xd4cb, PDF_CMAP_RANGE, 17560 },
	{ 0xd4cc, 0xd4cc, PDF_CMAP_SINGLE, 3269 },
	{ 0xd4cd, 0xd4cf, PDF_CMAP_RANGE, 17594 },
	{ 0xd4d0, 0xd4d0, PDF_CMAP_SINGLE, 3270 },
	{ 0xd4d1, 0xd4d3, PDF_CMAP_RANGE, 17597 },
	{ 0xd4d4, 0xd4d4, PDF_CMAP_SINGLE, 3271 },
	{ 0xd4d5, 0xd4db, PDF_CMAP_RANGE, 17600 },
	{ 0xd4dc, 0xd4dc, PDF_CMAP_SINGLE, 3272 },
	{ 0xd4dd, 0xd4de, PDF_CMAP_RANGE, 17607 },
	{ 0xd4df, 0xd4df, PDF_CMAP_SINGLE, 3273 },
	{ 0xd4e0, 0xd4e7, PDF_CMAP_RANGE, 17609 },
	{ 0xd4e8, 0xd4e8, PDF_CMAP_SINGLE, 3274 },
	{ 0xd4e9, 0xd4eb, PDF_CMAP_RANGE, 17617 },
	{ 0xd4ec, 0xd4ec, PDF_CMAP_SINGLE, 3275 },
	{ 0xd4ed, 0xd4ef, PDF_CMAP_RANGE, 17620 },
	{ 0xd4f0, 0xd4f0, PDF_CMAP_SINGLE, 3276 },
	{ 0xd4f1, 0xd4f7, PDF_CMAP_RANGE, 17623 },
	{ 0xd4f8, 0xd4f8, PDF_CMAP_SINGLE, 3277 },
	{ 0xd4f9, 0xd4fa, PDF_CMAP_RANGE, 17630 },
	{ 0xd4fb, 0xd4fb, PDF_CMAP_SINGLE, 3278 },
	{ 0xd4fc, 0xd4fc, PDF_CMAP_SINGLE, 17632 },
	{ 0xd4fd, 0xd4fd, PDF_CMAP_SINGLE, 3279 },
	{ 0xd4fe, 0xd4ff, PDF_CMAP_RANGE, 17633 },
	{ 0xd500, 0xd503, PDF_CMAP_RANGE, 17635 },
	{ 0xd504, 0xd504, PDF_CMAP_SINGLE, 3280 },
	{ 0xd505, 0xd507, PDF_CMAP_RANGE, 17639 },
	{ 0xd508, 0xd508, PDF_CMAP_SINGLE, 3281 },
	{ 0xd509, 0xd50b, PDF_CMAP_RANGE, 17642 },
	{ 0xd50c, 0xd50c, PDF_CMAP_SINGLE, 3282 },
	{ 0xd50d, 0xd513, PDF_CMAP_RANGE, 17645 },
	{ 0xd514, 0xd515, PDF_CMAP_RANGE, 3283 },
	{ 0xd516, 0xd516, PDF_CMAP_SINGLE, 17652 },
	{ 0xd517, 0xd517, PDF_CMAP_SINGLE, 3285 },
	{ 0xd518, 0xd53b, PDF_CMAP_RANGE, 17653 },
	{ 0xd53c, 0xd53d, PDF_CMAP_RANGE, 3286 },
	{ 0xd53e, 0xd53f, PDF_CMAP_RANGE, 17689 },
	{ 0xd540, 0xd540, PDF_CMAP_SINGLE, 3288 },
	{ 0xd541, 0xd543, PDF_CMAP_RANGE, 17691 },
	{ 0xd544, 0xd544, PDF_CMAP_SINGLE, 3289 },
	{ 0xd545, 0xd54b, PDF_CMAP_RANGE, 17694 },
	{ 0xd54c, 0xd54d, PDF_CMAP_RANGE, 3290 },
	{ 0xd54e, 0xd54e, PDF_CMAP_SINGLE, 17701 },
	{ 0xd54f, 0xd54f, PDF_CMAP_SINGLE, 3292 },
	{ 0xd550, 0xd550, PDF_CMAP_SINGLE, 17702 },
	{ 0xd551, 0xd551, PDF_CMAP_SINGLE, 3293 },
	{ 0xd552, 0xd557, PDF_CMAP_RANGE, 17703 },
	{ 0xd558, 0xd559, PDF_CMAP_RANGE, 3294 },
	{ 0xd55a, 0xd55b, PDF_CMAP_RANGE, 17709 },
	{ 0xd55c, 0xd55c, PDF_CMAP_SINGLE, 3296 },
	{ 0xd55d, 0xd55f, PDF_CMAP_RANGE, 17711 },
	{ 0xd560, 0xd560, PDF_CMAP_SINGLE, 3297 },
	{ 0xd561, 0xd564, PDF_CMAP_RANGE, 17714 },
	{ 0xd565, 0xd565, PDF_CMAP_SINGLE, 3298 },
	{ 0xd566, 0xd567, PDF_CMAP_RANGE, 17718 },
	{ 0xd568, 0xd569, PDF_CMAP_RANGE, 3299 },
	{ 0xd56a, 0xd56a, PDF_CMAP_SINGLE, 17720 },
	{ 0xd56b, 0xd56b, PDF_CMAP_SINGLE, 3301 },
	{ 0xd56c, 0xd56c, PDF_CMAP_SINGLE, 17721 },
	{ 0xd56d, 0xd56d, PDF_CMAP_SINGLE, 3302 },
	{ 0xd56e, 0xd573, PDF_CMAP_RANGE, 17722 },
	{ 0xd574, 0xd575, PDF_CMAP_RANGE, 3303 },
	{ 0xd576, 0xd577, PDF_CMAP_RANGE, 17728 },
	{ 0xd578, 0xd578, PDF_CMAP_SINGLE, 3305 },
	{ 0xd579, 0xd57b, PDF_CMAP_RANGE, 17730 },
	{ 0xd57c, 0xd57c, PDF_CMAP_SINGLE, 3306 },
	{ 0xd57d, 0xd583, PDF_CMAP_RANGE, 17733 },
	{ 0xd584, 0xd585, PDF_CMAP_RANGE, 3307 },
	{ 0xd586, 0xd586, PDF_CMAP_SINGLE, 17740 },
	{ 0xd587, 0xd589, PDF_CMAP_RANGE, 3309 },
	{ 0xd58a, 0xd58f, PDF_CMAP_RANGE, 17741 },
	{ 0xd590, 0xd590, PDF_CMAP_SINGLE, 3312 },
	{ 0xd591, 0xd5a4, PDF_CMAP_RANGE, 17747 },
	{ 0xd5a5, 0xd5a5, PDF_CMAP_SINGLE, 3313 },
	{ 0xd5a6, 0xd5c7, PDF_CMAP_RANGE, 17767 },
	{ 0xd5c8, 0xd5c9, PDF_CMAP_RANGE, 3314 },
	{ 0xd5ca, 0xd5cb, PDF_CMAP_RANGE, 17801 },
	{ 0xd5cc, 0xd5cc, PDF_CMAP_SINGLE, 3316 },
	{ 0xd5cd, 0xd5cf, PDF_CMAP_RANGE, 17803 },
	{ 0xd5d0, 0xd5d0, PDF_CMAP_SINGLE, 3317 },
	{ 0xd5d1, 0xd5d1, PDF_CMAP_SINGLE, 17806 },
	{ 0xd5d2, 0xd5d2, PDF_CMAP_SINGLE, 3318 },
	{ 0xd5d3, 0xd5d7, PDF_CMAP_RANGE, 17807 },
	{ 0xd5d8, 0xd5d9, PDF_CMAP_RANGE, 3319 },
	{ 0xd5da, 0xd5da, PDF_CMAP_SINGLE, 17812 },
	{ 0xd5db, 0xd5db, PDF_CMAP_SINGLE, 3321 },
	{ 0xd5dc, 0xd5dc, PDF_CMAP_SINGLE, 17813 },
	{ 0xd5dd, 0xd5dd, PDF_CMAP_SINGLE, 3322 },
	{ 0xd5de, 0xd5e3, PDF_CMAP_RANGE, 17814 },
	{ 0xd5e4, 0xd5e5, PDF_CMAP_RANGE, 3323 },
	{ 0xd5e6, 0xd5e7, PDF_CMAP_RANGE, 17820 },
	{ 0xd5e8, 0xd5e8, PDF_CMAP_SINGLE, 3325 },
	{ 0xd5e9, 0xd5eb, PDF_CMAP_RANGE, 17822 },
	{ 0xd5ec, 0xd5ec, PDF_CMAP_SINGLE, 3326 },
	{ 0xd5ed, 0xd5f3, PDF_CMAP_RANGE, 17825 },
	{ 0xd5f4, 0xd5f5, PDF_CMAP_RANGE, 3327 },
	{ 0xd5f6, 0xd5f6, PDF_CMAP_SINGLE, 17832 },
	{ 0xd5f7, 0xd5f7, PDF_CMAP_SINGLE, 3329 },
	{ 0xd5f8, 0xd5f8, PDF_CMAP_SINGLE, 17833 },
	{ 0xd5f9, 0xd5f9, PDF_CMAP_SINGLE, 3330 },
	{ 0xd5fa, 0xd5ff, PDF_CMAP_RANGE, 17834 },
	{ 0xd600, 0xd601, PDF_CMAP_RANGE, 3331 },
	{ 0xd602, 0xd603, PDF_CMAP_RANGE, 17840 },
	{ 0xd604, 0xd604, PDF_CMAP_SINGLE, 3333 },
	{ 0xd605, 0xd607, PDF_CMAP_RANGE, 17842 },
	{ 0xd608, 0xd608, PDF_CMAP_SINGLE, 3334 },
	{ 0xd609, 0xd60f, PDF_CMAP_RANGE, 17845 },
	{ 0xd610, 0xd611, PDF_CMAP_RANGE, 3335 },
	{ 0xd612, 0xd612, PDF_CMAP_SINGLE, 17852 },
	{ 0xd613, 0xd615, PDF_CMAP_RANGE, 3337 },
	{ 0xd616, 0xd61b, PDF_CMAP_RANGE, 17853 },
	{ 0xd61c, 0xd61c, PDF_CMAP_SINGLE, 3340 },
	{ 0xd61d, 0xd61f, PDF_CMAP_RANGE, 17859 },
	{ 0xd620, 0xd620, PDF_CMAP_SINGLE, 3341 },
	{ 0xd621, 0xd623, PDF_CMAP_RANGE, 17862 },
	{ 0xd624, 0xd624, PDF_CMAP_SINGLE, 3342 },
	{ 0xd625, 0xd62c, PDF_CMAP_RANGE, 17865 },
	{ 0xd62d, 0xd62d, PDF_CMAP_SINGLE, 3343 },
	{ 0xd62e, 0xd637, PDF_CMAP_RANGE, 17873 },
	{ 0xd638, 0xd639, PDF_CMAP_RANGE, 3344 },
	{ 0xd63a, 0xd63b, PDF_CMAP_RANGE, 17883 },
	{ 0xd63c, 0xd63c, PDF_CMAP_SINGLE, 3346 },
	{ 0xd63d, 0xd63f, PDF_CMAP_RANGE, 17885 },
	{ 0xd640, 0xd640, PDF_CMAP_SINGLE, 3347 },
	{ 0xd641, 0xd644, PDF_CMAP_RANGE, 17888 },
	{ 0xd645, 0xd645, PDF_CMAP_SINGLE, 3348 },
	{ 0xd646, 0xd647, PDF_CMAP_RANGE, 17892 },
	{ 0xd648, 0xd649, PDF_CMAP_RANGE, 3349 },
	{ 0xd64a, 0xd64a, PDF_CMAP_SINGLE, 17894 },
	{ 0xd64b, 0xd64b, PDF_CMAP_SINGLE, 3351 },
	{ 0xd64c, 0xd64c, PDF_CMAP_SINGLE, 17895 },
	{ 0xd64d, 0xd64d, PDF_CMAP_SINGLE, 3352 },
	{ 0xd64e, 0xd650, PDF_CMAP_RANGE, 17896 },
	{ 0xd651, 0xd651, PDF_CMAP_SINGLE, 3353 },
	{ 0xd652, 0xd653, PDF_CMAP_RANGE, 17899 },
	{ 0xd654, 0xd655, PDF_CMAP_RANGE, 3354 },
	{ 0xd656, 0xd657, PDF_CMAP_RANGE, 17901 },
	{ 0xd658, 0xd658, PDF_CMAP_SINGLE, 3356 },
	{ 0xd659, 0xd65b, PDF_CMAP_RANGE, 17903 },
	{ 0xd65c, 0xd65c, PDF_CMAP_SINGLE, 3357 },
	{ 0xd65d, 0xd666, PDF_CMAP_RANGE, 17906 },
	{ 0xd667, 0xd667, PDF_CMAP_SINGLE, 3358 },
	{ 0xd668, 0xd668, PDF_CMAP_SINGLE, 17916 },
	{ 0xd669, 0xd669, PDF_CMAP_SINGLE, 3359 },
	{ 0xd66a, 0xd66f, PDF_CMAP_RANGE, 17917 },
	{ 0xd670, 0xd671, PDF_CMAP_RANGE, 3360 },
	{ 0xd672, 0xd673, PDF_CMAP_RANGE, 17923 },
	{ 0xd674, 0xd674, PDF_CMAP_SINGLE, 3362 },
	{ 0xd675, 0xd682, PDF_CMAP_RANGE, 17925 },
	{ 0xd683, 0xd683, PDF_CMAP_SINGLE, 3363 },
	{ 0xd684, 0xd684, PDF_CMAP_SINGLE, 17939 },
	{ 0xd685, 0xd685, PDF_CMAP_SINGLE, 3364 },
	{ 0xd686, 0xd68b, PDF_CMAP_RANGE, 17940 },
	{ 0xd68c, 0xd68d, PDF_CMAP_RANGE, 3365 },
	{ 0xd68e, 0xd68f, PDF_CMAP_RANGE, 17946 },
	{ 0xd690, 0xd690, PDF_CMAP_SINGLE, 3367 },
	{ 0xd691, 0xd693, PDF_CMAP_RANGE, 17948 },
	{ 0xd694, 0xd694, PDF_CMAP_SINGLE, 3368 },
	{ 0xd695, 0xd69c, PDF_CMAP_RANGE, 17951 },
	{ 0xd69d, 0xd69d, PDF_CMAP_SINGLE, 3369 },
	{ 0xd69e, 0xd69e, PDF_CMAP_SINGLE, 17959 },
	{ 0xd69f, 0xd69f, PDF_CMAP_SINGLE, 3370 },
	{ 0xd6a0, 0xd6a0, PDF_CMAP_SINGLE, 17960 },
	{ 0xd6a1, 0xd6a1, PDF_CMAP_SINGLE, 3371 },
	{ 0xd6a2, 0xd6a7, PDF_CMAP_RANGE, 17961 },
	{ 0xd6a8, 0xd6a8, PDF_CMAP_SINGLE, 3372 },
	{ 0xd6a9, 0xd6ab, PDF_CMAP_RANGE, 17967 },
	{ 0xd6ac, 0xd6ac, PDF_CMAP_SINGLE, 3373 },
	{ 0xd6ad, 0xd6af, PDF_CMAP_RANGE, 17970 },
	{ 0xd6b0, 0xd6b0, PDF_CMAP_SINGLE, 3374 },
	{ 0xd6b1, 0xd6b8, PDF_CMAP_RANGE, 17973 },
	{ 0xd6b9, 0xd6b9, PDF_CMAP_SINGLE, 3375 },
	{ 0xd6ba, 0xd6ba, PDF_CMAP_SINGLE, 17981 },
	{ 0xd6bb, 0xd6bb, PDF_CMAP_SINGLE, 3376 },
	{ 0xd6bc, 0xd6c3, PDF_CMAP_RANGE, 17982 },
	{ 0xd6c4, 0xd6c5, PDF_CMAP_RANGE, 3377 },
	{ 0xd6c6, 0xd6c7, PDF_CMAP_RANGE, 17990 },
	{ 0xd6c8, 0xd6c8, PDF_CMAP_SINGLE, 3379 },
	{ 0xd6c9, 0xd6cb, PDF_CMAP_RANGE, 17992 },
	{ 0xd6cc, 0xd6cc, PDF_CMAP_SINGLE, 3380 },
	{ 0xd6cd, 0xd6d0, PDF_CMAP_RANGE, 17995 },
	{ 0xd6d1, 0xd6d1, PDF_CMAP_SINGLE, 3381 },
	{ 0xd6d2, 0xd6d3, PDF_CMAP_RANGE, 17999 },
	{ 0xd6d4, 0xd6d4, PDF_CMAP_SINGLE, 3382 },
	{ 0xd6d5, 0xd6d6, PDF_CMAP_RANGE, 18001 },
	{ 0xd6d7, 0xd6d7, PDF_CMAP_SINGLE, 3383 },
	{ 0xd6d8, 0xd6d8, PDF_CMAP_SINGLE, 18003 },
	{ 0xd6d9, 0xd6d9, PDF_CMAP_SINGLE, 3384 },
	{ 0xd6da, 0xd6df, PDF_CMAP_RANGE, 18004 },
	{ 0xd6e0, 0xd6e0, PDF_CMAP_SINGLE, 3385 },
	{ 0xd6e1, 0xd6e3, PDF_CMAP_RANGE, 18010 },
	{ 0xd6e4, 0xd6e4, PDF_CMAP_SINGLE, 3386 },
	{ 0xd6e5, 0xd6e7, PDF_CMAP_RANGE, 18013 },
	{ 0xd6e8, 0xd6e8, PDF_CMAP_SINGLE, 3387 },
	{ 0xd6e9, 0xd6ef, PDF_CMAP_RANGE, 18016 },
	{ 0xd6f0, 0xd6f0, PDF_CMAP_SINGLE, 3388 },
	{ 0xd6f1, 0xd6f4, PDF_CMAP_RANGE, 18023 },
	{ 0xd6f5, 0xd6f5, PDF_CMAP_SINGLE, 3389 },
	{ 0xd6f6, 0xd6fb, PDF_CMAP_RANGE, 18027 },
	{ 0xd6fc, 0xd6fd, PDF_CMAP_RANGE, 3390 },
	{ 0xd6fe, 0xd6ff, PDF_CMAP_RANGE, 18033 },
	{ 0xd700, 0xd700, PDF_CMAP_SINGLE, 3392 },
	{ 0xd701, 0xd703, PDF_CMAP_RANGE, 18035 },
	{ 0xd704, 0xd704, PDF_CMAP_SINGLE, 3393 },
	{ 0xd705, 0xd710, PDF_CMAP_RANGE, 18038 },
	{ 0xd711, 0xd711, PDF_CMAP_SINGLE, 3394 },
	{ 0xd712, 0xd717, PDF_CMAP_RANGE, 18050 },
	{ 0xd718, 0xd719, PDF_CMAP_RANGE, 3395 },
	{ 0xd71a, 0xd71b, PDF_CMAP_RANGE, 18056 },
	{ 0xd71c, 0xd71c, PDF_CMAP_SINGLE, 3397 },
	{ 0xd71d, 0xd71f, PDF_CMAP_RANGE, 18058 },
	{ 0xd720, 0xd720, PDF_CMAP_SINGLE, 3398 },
	{ 0xd721, 0xd727, PDF_CMAP_RANGE, 18061 },
	{ 0xd728, 0xd729, PDF_CMAP_RANGE, 3399 },
	{ 0xd72a, 0xd72a, PDF_CMAP_SINGLE, 18068 },
	{ 0xd72b, 0xd72b, PDF_CMAP_SINGLE, 3401 },
	{ 0xd72c, 0xd72c, PDF_CMAP_SINGLE, 18069 },
	{ 0xd72d, 0xd72d, PDF_CMAP_SINGLE, 3402 },
	{ 0xd72e, 0xd733, PDF_CMAP_RANGE, 18070 },
	{ 0xd734, 0xd735, PDF_CMAP_RANGE, 3403 },
	{ 0xd736, 0xd737, PDF_CMAP_RANGE, 18076 },
	{ 0xd738, 0xd738, PDF_CMAP_SINGLE, 3405 },
	{ 0xd739, 0xd73b, PDF_CMAP_RANGE, 18078 },
	{ 0xd73c, 0xd73c, PDF_CMAP_SINGLE, 3406 },
	{ 0xd73d, 0xd743, PDF_CMAP_RANGE, 18081 },
	{ 0xd744, 0xd744, PDF_CMAP_SINGLE, 3407 },
	{ 0xd745, 0xd746, PDF_CMAP_RANGE, 18088 },
	{ 0xd747, 0xd747, PDF_CMAP_SINGLE, 3408 },
	{ 0xd748, 0xd748, PDF_CMAP_SINGLE, 18090 },
	{ 0xd749, 0xd749, PDF_CMAP_SINGLE, 3409 },
	{ 0xd74a, 0xd74f, PDF_CMAP_RANGE, 18091 },
	{ 0xd750, 0xd751, PDF_CMAP_RANGE, 3410 },
	{ 0xd752, 0xd753, PDF_CMAP_RANGE, 18097 },
	{ 0xd754, 0xd754, PDF_CMAP_SINGLE, 3412 },
	{ 0xd755, 0xd755, PDF_CMAP_SINGLE, 18099 },
	{ 0xd756, 0xd759, PDF_CMAP_RANGE, 3413 },
	{ 0xd75a, 0xd75f, PDF_CMAP_RANGE, 18100 },
	{ 0xd760, 0xd761, PDF_CMAP_RANGE, 3417 },
	{ 0xd762, 0xd762, PDF_CMAP_SINGLE, 18106 },
	{ 0xd763, 0xd763, PDF_CMAP_SINGLE, 3419 },
	{ 0xd764, 0xd764, PDF_CMAP_SINGLE, 18107 },
	{ 0xd765, 0xd765, PDF_CMAP_SINGLE, 3420 },
	{ 0xd766, 0xd768, PDF_CMAP_RANGE, 18108 },
	{ 0xd769, 0xd769, PDF_CMAP_SINGLE, 3421 },
	{ 0xd76a, 0xd76b, PDF_CMAP_RANGE, 18111 },
	{ 0xd76c, 0xd76c, PDF_CMAP_SINGLE, 3422 },
	{ 0xd76d, 0xd76f, PDF_CMAP_RANGE, 18113 },
	{ 0xd770, 0xd770, PDF_CMAP_SINGLE, 3423 },
	{ 0xd771, 0xd773, PDF_CMAP_RANGE, 18116 },
	{ 0xd774, 0xd774, PDF_CMAP_SINGLE, 3424 },
	{ 0xd775, 0xd77b, PDF_CMAP_RANGE, 18119 },
	{ 0xd77c, 0xd77d, PDF_CMAP_RANGE, 3425 },
	{ 0xd77e, 0xd780, PDF_CMAP_RANGE, 18126 },
	{ 0xd781, 0xd781, PDF_CMAP_SINGLE, 3427 },
	{ 0xd782, 0xd787, PDF_CMAP_RANGE, 18129 },
	{ 0xd788, 0xd789, PDF_CMAP_RANGE, 3428 },
	{ 0xd78a, 0xd78b, PDF_CMAP_RANGE, 18135 },
	{ 0xd78c, 0xd78c, PDF_CMAP_SINGLE, 3430 },
	{ 0xd78d, 0xd78f, PDF_CMAP_RANGE, 18137 },
	{ 0xd790, 0xd790, PDF_CMAP_SINGLE, 3431 },
	{ 0xd791, 0xd797, PDF_CMAP_RANGE, 18140 },
	{ 0xd798, 0xd799, PDF_CMAP_RANGE, 3432 },
	{ 0xd79a, 0xd79a, PDF_CMAP_SINGLE, 18147 },
	{ 0xd79b, 0xd79b, PDF_CMAP_SINGLE, 3434 },
	{ 0xd79c, 0xd79c, PDF_CMAP_SINGLE, 18148 },
	{ 0xd79d, 0xd79d, PDF_CMAP_SINGLE, 3435 },
	{ 0xd79e, 0xd7a3, PDF_CMAP_RANGE, 18149 },
	{ 0xf900, 0xf900, PDF_CMAP_SINGLE, 4116 },
	{ 0xf901, 0xf901, PDF_CMAP_SINGLE, 3678 },
	{ 0xf902, 0xf902, PDF_CMAP_SINGLE, 7053 },
	{ 0xf903, 0xf903, PDF_CMAP_SINGLE, 3460 },
	{ 0xf904, 0xf904, PDF_CMAP_SINGLE, 7900 },
	{ 0xf905, 0xf905, PDF_CMAP_SINGLE, 3802 },
	{ 0xf906, 0xf906, PDF_CMAP_SINGLE, 3902 },
	{ 0xf907, 0xf907, PDF_CMAP_SINGLE, 3946 },
	{ 0xf908, 0xf908, PDF_CMAP_SINGLE, 3946 },
	{ 0xf909, 0xf909, PDF_CMAP_SINGLE, 3708 },
	{ 0xf90a, 0xf90a, PDF_CMAP_SINGLE, 4131 },
	{ 0xf90b, 0xf90b, PDF_CMAP_SINGLE, 4374 },
	{ 0xf90c, 0xf90c, PDF_CMAP_SINGLE, 4156 },
	{ 0xf90d, 0xf913, PDF_CMAP_RANGE, 4375 },
	{ 0xf914, 0xf914, PDF_CMAP_SINGLE, 5800 },
	{ 0xf915, 0xf917, PDF_CMAP_RANGE, 4382 },
	{ 0xf918, 0xf91d, PDF_CMAP_RANGE, 4386 },
	{ 0xf91e, 0xf920, PDF_CMAP_RANGE, 4394 },
	{ 0xf921, 0xf921, PDF_CMAP_SINGLE, 4399 },
	{ 0xf922, 0xf922, PDF_CMAP_SINGLE, 4403 },
	{ 0xf923, 0xf924, PDF_CMAP_RANGE, 4406 },
	{ 0xf925, 0xf92b, PDF_CMAP_RANGE, 4409 },
	{ 0xf92c, 0xf92d, PDF_CMAP_RANGE, 4419 },
	{ 0xf92e, 0xf92e, PDF_CMAP_SINGLE, 4424 },
	{ 0xf92f, 0xf92f, PDF_CMAP_SINGLE, 4511 },
	{ 0xf930, 0xf931, PDF_CMAP_RANGE, 4513 },
	{ 0xf932, 0xf937, PDF_CMAP_RANGE, 4517 },
	{ 0xf938, 0xf93a, PDF_CMAP_RANGE, 4524 },
	{ 0xf93b, 0xf940, PDF_CMAP_RANGE, 4528 },
	{ 0xf941, 0xf943, PDF_CMAP_RANGE, 4535 },
	{ 0xf944, 0xf945, PDF_CMAP_RANGE, 4541 },
	{ 0xf946, 0xf948, PDF_CMAP_RANGE, 4545 },
	{ 0xf949, 0xf949, PDF_CMAP_SINGLE, 4550 },
	{ 0xf94a, 0xf94a, PDF_CMAP_SINGLE, 4564 },
	{ 0xf94b, 0xf94e, PDF_CMAP_RANGE, 4566 },
	{ 0xf94f, 0xf950, PDF_CMAP_RANGE, 4571 },
	{ 0xf951, 0xf951, PDF_CMAP_SINGLE, 4576 },
	{ 0xf952, 0xf955, PDF_CMAP_RANGE, 4604 },
	{ 0xf956, 0xf959, PDF_CMAP_RANGE, 4609 },
	{ 0xf95a, 0xf95a, PDF_CMAP_SINGLE, 4318 },
	{ 0xf95b, 0xf95b, PDF_CMAP_SINGLE, 4136 },
	{ 0xf95c, 0xf95c, PDF_CMAP_SINGLE, 5800 },
	{ 0xf95d, 0xf95d, PDF_CMAP_SINGLE, 4139 },
	{ 0xf95e, 0xf95e, PDF_CMAP_SINGLE, 4192 },
	{ 0xf95f, 0xf95f, PDF_CMAP_SINGLE, 4167 },
	{ 0xf960, 0xf960, PDF_CMAP_SINGLE, 4172 },
	{ 0xf961, 0xf961, PDF_CMAP_SINGLE, 5552 },
	{ 0xf962, 0xf962, PDF_CMAP_SINGLE, 6424 },
	{ 0xf963, 0xf963, PDF_CMAP_SINGLE, 5151 },
	{ 0xf964, 0xf964, PDF_CMAP_SINGLE, 4922 },
	{ 0xf965, 0xf965, PDF_CMAP_SINGLE, 7518 },
	{ 0xf966, 0xf966, PDF_CMAP_SINGLE, 5079 },
	{ 0xf967, 0xf967, PDF_CMAP_SINGLE, 5109 },
	{ 0xf968, 0xf968, PDF_CMAP_SINGLE, 7607 },
	{ 0xf969, 0xf969, PDF_CMAP_SINGLE, 5584 },
	{ 0xf96a, 0xf96a, PDF_CMAP_SINGLE, 5367 },
	{ 0xf96b, 0xf96b, PDF_CMAP_SINGLE, 7083 },
	{ 0xf96c, 0xf96c, PDF_CMAP_SINGLE, 5362 },
	{ 0xf96d, 0xf96d, PDF_CMAP_SINGLE, 5485 },
	{ 0xf96e, 0xf96e, PDF_CMAP_SINGLE, 6001 },
	{ 0xf96f, 0xf96f, PDF_CMAP_SINGLE, 5460 },
	{ 0xf970, 0xf970, PDF_CMAP_SINGLE, 5317 },
	{ 0xf971, 0xf971, PDF_CMAP_SINGLE, 7009 },
	{ 0xf972, 0xf972, PDF_CMAP_SINGLE, 7370 },
	{ 0xf973, 0xf973, PDF_CMAP_SINGLE, 5678 },
	{ 0xf974, 0xf974, PDF_CMAP_SINGLE, 5874 },
	{ 0xf975, 0xf977, PDF_CMAP_RANGE, 4425 },
	{ 0xf978, 0xf97a, PDF_CMAP_RANGE, 4429 },
	{ 0xf97b, 0xf97d, PDF_CMAP_RANGE, 4435 },
	{ 0xf97e, 0xf97e, PDF_CMAP_SINGLE, 4439 },
	{ 0xf97f, 0xf980, PDF_CMAP_RANGE, 4442 },
	{ 0xf981, 0xf981, PDF_CMAP_SINGLE, 4159 },
	{ 0xf982, 0xf982, PDF_CMAP_SINGLE, 4444 },
	{ 0xf983, 0xf983, PDF_CMAP_SINGLE, 4447 },
	{ 0xf984, 0xf985, PDF_CMAP_RANGE, 4449 },
	{ 0xf986, 0xf986, PDF_CMAP_SINGLE, 4453 },
	{ 0xf987, 0xf98c, PDF_CMAP_RANGE, 4455 },
	{ 0xf98d, 0xf98d, PDF_CMAP_SINGLE, 4463 },
	{ 0xf98e, 0xf98e, PDF_CMAP_SINGLE, 4160 },
	{ 0xf98f, 0xf990, PDF_CMAP_RANGE, 4465 },
	{ 0xf991, 0xf991, PDF_CMAP_SINGLE, 4161 },
	{ 0xf992, 0xf994, PDF_CMAP_RANGE, 4468 },
	{ 0xf995, 0xf995, PDF_CMAP_SINGLE, 4162 },
	{ 0xf996, 0xf997, PDF_CMAP_RANGE, 4471 },
	{ 0xf998, 0xf998, PDF_CMAP_SINGLE, 4474 },
	{ 0xf999, 0xf999, PDF_CMAP_SINGLE, 4473 },
	{ 0xf99a, 0xf99b, PDF_CMAP_RANGE, 4475 },
	{ 0xf99c, 0xf99d, PDF_CMAP_RANGE, 4478 },
	{ 0xf99e, 0xf99e, PDF_CMAP_SINGLE, 6447 },
	{ 0xf99f, 0xf9a0, PDF_CMAP_RANGE, 4481 },
	{ 0xf9a1, 0xf9a1, PDF_CMAP_SINGLE, 5460 },
	{ 0xf9a2, 0xf9a2, PDF_CMAP_SINGLE, 4483 },
	{ 0xf9a3, 0xf9a3, PDF_CMAP_SINGLE, 4163 },
	{ 0xf9a4, 0xf9a4, PDF_CMAP_SINGLE, 4166 },
	{ 0xf9a5, 0xf9a5, PDF_CMAP_SINGLE, 4485 },
	{ 0xf9a6, 0xf9a8, PDF_CMAP_RANGE, 4487 },
	{ 0xf9a9, 0xf9a9, PDF_CMAP_SINGLE, 4491 },
	{ 0xf9aa, 0xf9aa, PDF_CMAP_SINGLE, 4167 },
	{ 0xf9ab, 0xf9ad, PDF_CMAP_RANGE, 4493 },
	{ 0xf9ae, 0xf9ae, PDF_CMAP_SINGLE, 7783 },
	{ 0xf9af, 0xf9af, PDF_CMAP_SINGLE, 4497 },
	{ 0xf9b0, 0xf9b0, PDF_CMAP_SINGLE, 4499 },
	{ 0xf9b1, 0xf9b4, PDF_CMAP_RANGE, 4501 },
	{ 0xf9b5, 0xf9b5, PDF_CMAP_SINGLE, 4506 },
	{ 0xf9b6, 0xf9b8, PDF_CMAP_RANGE, 4508 },
	{ 0xf9b9, 0xf9b9, PDF_CMAP_SINGLE, 5797 },
	{ 0xf9ba, 0xf9bc, PDF_CMAP_RANGE, 4551 },
	{ 0xf9bd, 0xf9bd, PDF_CMAP_SINGLE, 4180 },
	{ 0xf9be, 0xf9be, PDF_CMAP_SINGLE, 4555 },
	{ 0xf9bf, 0xf9bf, PDF_CMAP_SINGLE, 5800 },
	{ 0xf9c0, 0xf9c1, PDF_CMAP_RANGE, 4556 },
	{ 0xf9c2, 0xf9c3, PDF_CMAP_RANGE, 4560 },
	{ 0xf9c4, 0xf9c4, PDF_CMAP_SINGLE, 4563 },
	{ 0xf9c5, 0xf9c5, PDF_CMAP_SINGLE, 7988 },
	{ 0xf9c6, 0xf9c6, PDF_CMAP_SINGLE, 6123 },
	{ 0xf9c7, 0xf9c7, PDF_CMAP_SINGLE, 4577 },
	{ 0xf9c8, 0xf9c8, PDF_CMAP_SINGLE, 4183 },
	{ 0xf9c9, 0xf9c9, PDF_CMAP_SINGLE, 4579 },
	{ 0xf9ca, 0xf9cb, PDF_CMAP_RANGE, 4581 },
	{ 0xf9cc, 0xf9cc, PDF_CMAP_SINGLE, 4584 },
	{ 0xf9cd, 0xf9cd, PDF_CMAP_SINGLE, 4586 },
	{ 0xf9ce, 0xf9ce, PDF_CMAP_SINGLE, 4588 },
	{ 0xf9cf, 0xf9cf, PDF_CMAP_SINGLE, 4184 },
	{ 0xf9d0, 0xf9d3, PDF_CMAP_RANGE, 4590 },
	{ 0xf9d4, 0xf9d6, PDF_CMAP_RANGE, 4595 },
	{ 0xf9d7, 0xf9da, PDF_CMAP_RANGE, 4599 },
	{ 0xf9db, 0xf9db, PDF_CMAP_SINGLE, 5552 },
	{ 0xf9dc, 0xf9dc, PDF_CMAP_SINGLE, 4603 },
	{ 0xf9dd, 0xf9dd, PDF_CMAP_SINGLE, 4614 },
	{ 0xf9de, 0xf9de, PDF_CMAP_SINGLE, 4616 },
	{ 0xf9df, 0xf9df, PDF_CMAP_SINGLE, 4618 },
	{ 0xf9e0, 0xf9e0, PDF_CMAP_SINGLE, 5950 },
	{ 0xf9e1, 0xf9e2, PDF_CMAP_RANGE, 4620 },
	{ 0xf9e3, 0xf9e3, PDF_CMAP_SINGLE, 4187 },
	{ 0xf9e4, 0xf9e4, PDF_CMAP_SINGLE, 4625 },
	{ 0xf9e5, 0xf9e5, PDF_CMAP_SINGLE, 4627 },
	{ 0xf9e6, 0xf9e6, PDF_CMAP_SINGLE, 4629 },
	{ 0xf9e7, 0xf9e9, PDF_CMAP_RANGE, 4632 },
	{ 0xf9ea, 0xf9ea, PDF_CMAP_SINGLE, 4636 },
	{ 0xf9eb, 0xf9ec, PDF_CMAP_RANGE, 4188 },
	{ 0xf9ed, 0xf9ed, PDF_CMAP_SINGLE, 4638 },
	{ 0xf9ee, 0xf9f0, PDF_CMAP_RANGE, 4640 },
	{ 0xf9f1, 0xf9f5, PDF_CMAP_RANGE, 4644 },
	{ 0xf9f6, 0xf9f6, PDF_CMAP_SINGLE, 4650 },
	{ 0xf9f7, 0xf9f9, PDF_CMAP_RANGE, 4653 },
	{ 0xf9fa, 0xf9fa, PDF_CMAP_SINGLE, 5351 },
	{ 0xf9fb, 0xf9fb, PDF_CMAP_SINGLE, 6494 },
	{ 0xf9fc, 0xf9fc, PDF_CMAP_SINGLE, 5731 },
	{ 0xf9fd, 0xf9fd, PDF_CMAP_SINGLE, 5771 },
	{ 0xf9fe, 0xf9fe, PDF_CMAP_SINGLE, 4191 },
	{ 0xf9ff, 0xf9ff, PDF_CMAP_SINGLE, 6484 },
	{ 0xfa00, 0xfa00, PDF_CMAP_SINGLE, 6684 },
	{ 0xfa01, 0xfa01, PDF_CMAP_SINGLE, 4279 },
	{ 0xfa02, 0xfa02, PDF_CMAP_SINGLE, 7139 },
	{ 0xfa03, 0xfa03, PDF_CMAP_SINGLE, 4247 },
	{ 0xfa04, 0xfa04, PDF_CMAP_SINGLE, 4266 },
	{ 0xfa05, 0xfa05, PDF_CMAP_SINGLE, 4340 },
	{ 0xfa06, 0xfa06, PDF_CMAP_SINGLE, 7572 },
	{ 0xfa07, 0xfa07, PDF_CMAP_SINGLE, 5088 },
	{ 0xfa08, 0xfa08, PDF_CMAP_SINGLE, 7709 },
	{ 0xfa09, 0xfa09, PDF_CMAP_SINGLE, 3558 },
	{ 0xfa0a, 0xfa0a, PDF_CMAP_SINGLE, 3644 },
	{ 0xfa0b, 0xfa0b, PDF_CMAP_SINGLE, 3815 },
	{ 0xff01, 0xff3b, PDF_CMAP_RANGE, 264 },
	{ 0xff3c, 0xff3c, PDF_CMAP_SINGLE, 112 },
	{ 0xff3d, 0xff5d, PDF_CMAP_RANGE, 324 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 113 },
	{ 0xffe0, 0xffe1, PDF_CMAP_RANGE, 143 },
	{ 0xffe2, 0xffe2, PDF_CMAP_SINGLE, 194 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 357 },
	{ 0xffe5, 0xffe5, PDF_CMAP_SINGLE, 145 },
	{ 0xffe6, 0xffe6, PDF_CMAP_SINGLE, 323 },
};

static const unsigned short pdf_cmap_UniKS_UTF16_H_table[3309] =
{
	155,107,98,668,176,138,130,244,
	104,206,842,675,177,751,748,752,
	209,131,673,676,768,132,766,669,
	760,761,670,762,671,764,672,765,
	772,678,771,674,767,677,770,202,
	205,204,207,200,203,8582,9326,9324,
	156,8763,263,260,247,250,248,251,
	249,8868,8865,8864,8869,8867,8815,8814,
	8854,195,8855,196,9190,9192,9191,9193,
	9198,9200,9199,9201,151,198,8715,152,
	184,8749,185,8750,180,136,8717,148,
	191,190,137,181,210,8321,8739,8723,
	154,8722,133,153,8734,8748,8747,8486,
	8488,8742,8503,149,8320,106,519,530,
	520,531,521,558,557,532,522,552,
	551,533,524,556,555,535,523,554,
	553,534,525,546,536,527,548,538,
	526,542,547,537,528,544,549,539,
	529,165,164,237,236,235,238,8736,
	167,166,8780,220,219,169,168,8779,
	218,217,163,162,228,227,8787,159,
	8639,161,160,158,157,241,240,242,
	9222,243,9223,226,221,224,225,239,
	255,252,6477,7499,4270,4458,5772,5077,
	6201,3901,7451,5255,5403,4253,4159,6488,
	7247,5504,3791,4077,3604,3481,5764,3804,
	7808,5580,6951,4840,4349,4027,4950,4819,
	6464,6125,6281,4788,8027,6956,4822,5196,
	4776,5774,5587,7867,6803,5129,7962,7521,
	5782,6210,3639,7741,6081,3806,6100,3521,
	5370,6189,6664,7610,4993,7600,4755,4791,
	4781,5709,5413,5710,7870,7755,4653,6900,
	4863,5896,6218,4519,6427,6428,6368,6357,
	5749,6502,6967,3930,5456,7158,6888,3496,
	5368,7232,7756,7709,6409,3644,3474,5921,
	3772,4355,5718,7512,6636,6892,6830,5755,
	7053,5756,7009,6389,6352,4634,4131,6568,
	4846,5147,6227,7203,5220,4744,7733,6307,
	6385,7757,7595,5221,5733,5625,7718,4661,
	3785,3765,3993,5910,6827,4527,4533,4726,
	4663,7927,5401,8018,6758,3766,5402,5222,
	6781,7359,4563,3946,8700,175,104,9330,
	261,662,592,650,665,259,614,663,
	607,8790,611,660,654,664,659,6460,
	6704,4670,6534,5320,5331,7616,7041,5181,
	5492,3893,5041,7882,4192,6029,4154,5241,
	7800,7614,5683,6380,3895,3613,5862,5092,
	5315,5242,6413,6252,6238,7801,6049,6705,
	4696,7670,3868,7685,5947,7710,3660,6706,
	4427,5771,6444,4038,3560,6483,5244,7380,
	6535,5110,5418,4250,4489,6414,3602,3561,
	7671,6415,4051,6050,4063,5074,5005,8005,
	5246,3436,4194,6591,7544,6284,6592,6861,
	6854,6198,5935,6461,5171,6509,5879,4989,
	6462,5247,3476,7015,3839,4506,4764,7966,
	7367,4440,3705,7241,5776,6051,6902,3661,
	7734,5248,7759,5737,5567,5111,4836,4269,
	7967,6395,3662,7043,4941,7337,3969,6837,
	3585,6030,4595,7394,6131,3438,3627,6285,
	7330,6708,6199,5303,5182,7262,6175,6052,
	6645,7177,5419,5075,8035,5249,6396,6903,
	6089,6359,6255,7772,7306,6783,8013,5420,
	3840,4016,7431,4735,7449,4155,6646,4429,
	6312,7499,3787,4591,7791,5043,4064,3898,
	6647,5864,4424,6784,4607,4333,4606,6390,
	8036,7904,3563,7167,6141,7303,5152,6684,
	6031,7490,5037,4614,5304,6759,5561,3970,
	7077,6484,3465,5301,4017,4397,6648,6480,
	6649,4018,5021,4577,6760,3619,3789,3440,
	4479,4024,6785,4169,3624,4334,6229,3510,
	4812,5685,4511,4766,5493,6619,4025,6510,
	4009,3899,4848,7547,5076,5183,7863,5151,
	6536,3841,4188,3900,5772,7149,6478,5686,
	6053,7994,4906,5185,6834,7395,7760,4800,
	6446,6287,3466,4390,6256,7332,3979,7618,
	6201,7044,4052,6202,7316,5568,6032,7251,
	3901,3729,3902,3730,6943,3994,5500,7500,
	3441,7432,7016,5250,6203,8023,3467,7663,
	4128,6620,4335,4749,7969,4616,7450,7711,
	6381,7534,8030,7317,4838,7970,5738,7802,
	4750,6944,3870,5841,7590,7664,6571,3767,
	7953,5252,6980,7169,4374,7971,7655,7150,
	7995,7883,8037,3501,6314,5336,4132,3871,
	5253,7338,5569,6868,5570,5254,4491,3733,
	7550,5902,6259,6258,4218,3511,6855,3581,
	3906,6260,6178,6905,5948,5117,4971,4069,
	3637,7459,3537,6355,5793,6142,5918,5060,
	6537,4273,3857,6002,4274,4241,6650,6057,
	6931,7720,4834,4319,6058,6538,4564,3842,
	3858,4536,5255,6470,6463,7804,5374,4071,
	5403,6135,5630,4190,7151,7433,5118,6143,
	5834,4072,4156,5094,3996,6869,7884,3708,
	5156,7730,7418,6360,6540,5404,5157,4159,
	4170,5936,5186,4697,6144,4039,6595,3734,
	5475,6289,6472,6145,3538,6864,6060,4134,
	5959,5739,4852,6261,3445,5573,5422,3872,
	6488,7754,3790,6481,6489,6832,6490,7955,
	3709,3735,5631,6522,6204,5574,5807,5553,
	6108,7427,7950,6838,3820,6870,6711,6109,
	6397,3580,5423,5759,6316,7885,7691,5960,
	5503,3446,5740,6180,5632,7116,6622,6263,
	4074,6450,4883,3907,7635,6205,5721,4664,
	7078,3803,7368,6061,6146,5760,4167,7117,
	5762,5258,3821,4553,6906,7253,5061,6541,
	6653,6291,6833,5763,4254,4275,5504,5697,
	6361,7136,3736,4186,4853,4180,3948,3587,
	3710,5699,5698,7335,4618,4075,6527,5575,
	3531,4870,5808,4421,4076,5843,3540,4854,
	3513,5375,5795,7152,6871,5640,5506,3791,
	6856,3874,3588,4077,6418,5261,7476,5547,
	3604,5700,7551,7973,6712,5796,7571,4996,
	4242,7536,3481,7530,4160,5045,7706,3482,
	4279,6857,5824,5377,3541,6181,7620,4483,
	4412,6872,6655,4803,7097,7537,3843,5961,
	6714,3605,7930,3964,6787,6451,5172,7854,
	7736,7807,4198,7792,5919,6230,7118,7578,
	6788,5225,7513,7098,6004,5173,6126,6715,
	7931,4600,7974,5378,3671,4280,4364,4972,
	5263,4422,5903,7906,5079,5642,4079,6452,
	7248,6947,4494,5264,7434,6421,5477,6264,
	6473,5643,5702,7932,6491,8010,7341,7636,
	6374,3468,4164,3793,5723,8031,6765,7637,
	5554,6656,6062,6320,7887,7254,5189,4268,
	7312,4281,7132,6321,7793,5797,7385,4178,
	5866,5478,7333,4872,7517,6398,6207,5844,
	3566,6265,6375,6492,3655,7435,7910,4675,
	3822,3469,7794,3567,3672,3544,7137,6168,
	7255,6208,4465,5237,8044,8043,7412,4873,
	3673,3483,7692,6391,6063,4815,4375,7934,
	7737,6399,4466,4243,3804,5667,5579,6369,
	5479,5781,3712,6657,8045,4446,4948,5507,
	7520,5424,7809,5191,5687,4080,3648,7477,
	5914,5380,7676,6686,5830,7266,4817,4165,
	4409,4909,4888,4136,3861,3589,7139,4931,
	7387,6148,3911,6835,7219,4973,3837,5724,
	4129,6934,5678,4137,6717,5963,4676,7763,
	4143,5962,5098,5265,4144,4166,5581,4283,
	4425,7119,7420,3795,7267,5929,6789,4804,
	6766,5883,7888,3629,7999,5867,5327,6149,
	4284,4910,7425,7447,5316,6150,7171,4932,
	7478,7261,4513,7153,6790,3674,4259,7399,
	4400,7079,3592,3875,7878,4401,5583,3739,
	6325,3568,4949,6718,5382,3876,4874,6064,
	7505,3517,5306,3675,5885,6719,6624,5125,
	5584,4484,7539,5193,4912,3768,5266,4027,
	7141,6512,7088,5267,5742,5904,5703,5426,
	6720,6464,4200,6952,6791,5644,6231,3777,
	5688,4952,7810,4751,7845,5161,4041,5950,
	5407,8019,5046,5480,6005,4712,6513,5704,
	7621,6232,5047,5836,6792,5705,6873,8046,
	5481,6066,7890,7935,7811,5743,4677,5062,
	3676,5408,7198,6721,3990,6722,3447,7988,
	8000,6293,5383,4140,5825,5886,6546,7795,
	7572,4459,4221,6152,7573,3769,6033,3678,
	3502,5385,6794,6793,6933,7206,7264,6281,
	6326,5302,7031,4798,4538,4788,4857,4691,
	5091,7080,7304,4620,7707,6575,7249,4085,
	5539,4975,3615,4337,5555,7492,4146,5409,
	3805,6954,3743,7531,4991,4773,3518,7365,
	5990,6327,3450,4157,6875,4579,5707,7127,
	5268,5645,6658,5386,4602,3877,4992,7704,
	4028,3631,6576,3616,7678,3713,4285,4338,
	5346,7891,4130,6577,7342,6110,7220,6067,
	4621,6767,3714,3778,5011,4858,4087,5049,
	6797,4020,4339,4245,6523,3715,4286,3823,
	5725,7268,7592,5452,7389,4820,7222,5932,
	6599,5646,7693,7269,6007,6988,4954,3915,
	7460,7104,7270,6548,4892,5585,7866,6239,
	5647,4015,4366,5915,4042,3620,6549,4287,
	3985,4448,6927,8027,7048,8028,3824,8029,
	7727,7889,6956,6726,7049,5063,4821,4460,
	3991,5837,6989,5648,5586,4485,5228,6402,
	3917,4309,6356,5196,7812,3918,5238,6009,
	6727,6929,3919,5938,7855,3545,6957,4091,
	5162,5765,4092,6079,6267,7679,5965,7370,
	4796,4234,4739,4849,7308,5272,6958,7506,
	5200,6331,7343,5512,3745,7179,5966,7914,
	7776,7154,4893,7556,7481,6388,4914,4878,
	7438,5889,4480,5454,7857,5649,8016,3845,
	6268,8032,7482,6687,6910,7813,4414,7694,
	7371,7764,3680,5513,5472,6184,7960,5967,
	3977,7344,5411,7961,4093,4288,6843,4568,
	4597,6382,7846,7199,5930,7155,7180,7892,
	6602,3520,6730,3593,4289,5274,4933,5801,
	6099,5455,7334,6294,4860,6578,7847,4976,
	4861,4223,5727,6731,7915,6453,7429,6269,
	4148,4582,3920,3716,6083,7105,4746,6493,
	7142,6579,4290,3780,4679,5905,7582,5970,
	6799,7640,4468,4680,6959,5589,7106,7033,
	4915,5634,6529,4515,5412,4224,6800,4341,
	5066,4639,6880,7514,7641,7446,7939,7063,
	3633,7402,4486,4175,6768,7818,6334,4403,
	6912,7778,4310,7642,6014,6370,4461,6732,
	5515,4539,4544,7779,5562,6514,6580,8021,
	6494,3846,7740,3594,7415,6881,6515,6696,
	7858,4383,4977,5163,4469,7992,4716,7893,
	7976,6801,6242,6253,4367,4326,4999,5975,
	6802,7064,5590,7243,5473,7985,5744,4292,
	6154,7731,7483,6587,6516,5129,5869,7962,
	5349,6422,6552,7521,7493,4311,5782,4778,
	4545,5388,4623,4624,7765,4415,7509,4729,
	5708,6836,5482,6604,4808,7901,6272,6080,
	5276,4214,7725,6496,5552,3923,6211,4094,
	4495,4260,4880,5308,7608,6992,7977,4384,
	5651,5067,6423,7713,7782,3999,6735,7743,
	4416,4625,5593,6663,5636,7404,7822,4095,
	4096,5102,6115,6844,3826,5993,4649,4043,
	5201,7484,7848,6212,4779,7302,5430,4173,
	7823,6019,6337,5389,5674,4585,6155,6084,
	6993,7783,4657,4097,4626,4895,3683,5025,
	7065,7583,7494,5768,7181,5371,5518,5068,
	6190,6664,6338,3532,5745,4149,6665,4235,
	4917,6995,7289,4825,4426,8008,6914,6424,
	7868,4246,3547,6883,7192,7610,5520,5519,
	7558,7599,3522,4215,6996,4342,6605,7022,
	7346,8022,4352,3686,6085,7143,7108,4918,
	4368,4935,3572,7920,6213,4978,5484,4293,
	6526,4731,3523,4919,7421,4862,3978,4780,
	3748,6037,5653,4558,7182,3621,4781,4060,
	5709,6404,4204,6134,3881,6961,7120,7373,
	4588,3687,4528,5847,5563,4546,7050,4450,
	4462,4923,5279,4103,6806,6963,6807,3595,
	7826,7290,5746,7869,6739,5081,5907,8052,
	4045,7870,4315,5597,5281,5053,4162,7273,
	3807,7229,7000,5838,6976,7349,7591,6809,
	6235,6977,4296,3454,3720,3749,3771,6628,
	6022,5366,6038,3965,3798,7024,7109,3966,
	6157,7067,4002,6689,4653,3689,6555,6900,
	7291,6770,4496,4031,6668,5654,5006,3849,
	7292,7456,4896,4107,6690,7921,6671,7522,
	6531,5524,7183,4487,5131,6884,6630,6811,
	6697,4434,7070,3582,5166,6812,3548,5873,
	7859,6217,8025,7895,5283,3865,6964,4056,
	5167,5525,4957,5367,6500,6886,7081,4571,
	5498,5526,3525,6846,7746,6813,3691,4924,
	4385,7747,7457,5284,3549,6743,5601,5602,
	6340,3550,4701,7173,7121,4109,7416,6517,
	4610,4742,4126,5208,7034,4207,7209,4881,
	7523,6117,7001,5855,7002,7828,6171,7294,
	5980,7257,6631,4826,6158,6978,5436,5689,
	7942,3722,4317,4405,4702,7643,6859,3854,
	5437,6405,4630,3583,5355,8033,6632,7322,
	5209,6160,4519,6501,4111,4782,6243,4472,
	7258,5488,6191,7200,4542,6368,6429,5638,
	5285,6815,4605,4113,3752,6965,3643,4958,
	3866,6358,6298,6865,4980,3533,6364,5394,
	7766,7324,7767,4722,3472,3695,3552,6120,
	5489,4179,6161,3608,6557,5082,5438,7461,
	4959,4667,5675,4228,7943,4176,4359,6559,
	4650,6967,7354,4261,7003,3930,5941,8034,
	5456,5286,7158,5657,4827,6888,7683,4960,
	4925,7390,4900,7749,3496,4436,3497,5368,
	6518,3967,5137,4360,3575,7830,5168,7487,
	6039,5769,6969,6244,4032,7274,4783,5783,
	6274,5995,7441,7233,7562,3933,5874,3754,
	6608,3755,7612,6921,4828,5017,3456,4784,
	7944,6455,4191,6192,5943,7234,7784,6431,
	7923,4631,6560,5138,5784,4668,4531,3949,
	7872,3756,4611,5213,7314,4297,4732,7532,
	6299,7831,6561,5876,7113,7832,5549,4684,
	5084,6504,5325,5527,6383,3981,7430,4829,
	7989,5319,5751,5396,7986,4406,5752,6564,
	4451,6040,4370,5877,5003,5609,4642,6816,
	4520,5529,5924,5028,3609,7133,7721,4521,
	6221,7834,3799,6817,5105,5753,6916,4452,
	7709,5981,7750,5668,7786,3934,6409,6275,
	4763,5610,7005,6582,4482,6042,6343,5070,
	6565,7566,5071,6304,4983,5086,7511,4298,
	7363,6745,5139,8026,7406,4114,7006,6890,
	6818,7533,5661,6043,3987,6673,8055,7873,
	7698,5358,6971,6457,5490,3725,4830,6075,
	3760,5558,6377,6819,6345,7194,7210,7699,
	6746,3498,6346,6506,5770,7593,6774,4785,
	5818,6305,5543,4965,3657,6442,3556,6634,
	4589,8011,4115,5953,6411,7090,5898,7837,
	6045,7586,5719,7512,6747,5140,6583,3800,
	5234,7875,7495,7423,3832,7129,6613,5499,
	6433,3992,7528,4718,4263,5218,7195,6432,
	4831,7626,5169,4547,6476,7947,6507,3460,
	7751,4719,7160,5142,3936,5364,6972,7315,
	6365,6941,4006,5143,4936,7028,4237,7161,
	3610,4301,7052,7797,6638,7145,4643,5755,
	3968,7053,5819,3988,3957,7196,4686,5073,
	3698,4438,7356,8003,4706,7174,4474,5055,
	4985,4599,5613,5087,6674,5944,7650,6276,
	7175,3888,5756,3761,5035,7009,6173,4177,
	6639,4902,7463,5559,3838,4302,7978,5954,
	7469,7299,3938,4303,3699,6615,7458,5399,
	4500,5545,6822,6917,5106,4475,6895,7010,
	4361,5614,6349,6246,7525,3812,7627,7925,
	4304,4216,6306,5550,4238,7212,6277,5532,
	6823,7054,4362,7162,5445,4561,3598,7602,
	6166,4720,7700,5615,7896,6434,4669,7788,
	4138,3526,7443,3939,6352,6749,7280,6519,
	4986,6896,4760,4387,7843,5313,6412,6567,
	7240,7798,4509,3584,5416,4634,6925,5871,
	4439,4635,4131,5566,6750,5146,7374,3622,
	7497,5329,6676,3534,3940,5670,6751,3941,
	5534,5664,4233,6753,4118,6678,3762,7059,
	4476,3813,7838,7011,3499,4882,3475,7667,
	5008,5996,5820,7668,3983,7473,7164,5029,
	5788,7391,4576,4724,3558,7543,7669,5470,
	5691,7146,6278,7012,6777,4612,4307,7661,
	4593,6226,4603,4264,5619,7926,3727,3636,
	6247,5849,4022,6778,6569,6520,5813,6254,
	5789,7037,3763,6919,6508,6097,6617,6826,
	5622,7285,6533,4636,4142,4847,5170,4502,
	4550,6754,7014,4868,4744,7715,7839,7757,
	6757,3701,7684,5665,6048,6124,4928,4328,
	7489,4504,3702,7464,6780,5857,5804,5221,
	5004,5551,4120,7362,6386,4929,5298,7569,
	5734,7716,5901,3989,7076,5448,4121,6167,
	4661,5909,7596,7358,5666,6899,4174,3942,
	7286,5536,3703,5955,7897,4455,5626,7213,
	3863,7849,4940,4995,4721,5108,4762,3773,
	4833,8012,6393,4526,4125,4534,3952,4745,
	5401,4457,4265,7305,6077,5040,6781,6585,
	1117,9377,1118,9413,1127,9442,1148,1160,
	9472,1161,9485,1167,9486,1168,9489,1172,
	9490,1178,9501,1179,9507,1182,9508,1183,
	9549,1196,9550,1197,1201,9571,1202,9588,
	1213,9589,9655,1230,9699,1242,9700,1243,
	9738,1250,9744,1253,9745,1256,9749,9750,
	1260,1261,9754,1262,9838,1284,1295,9871,
	1296,1306,9929,1307,9930,1308,9938,1311,
	9939,10052,1337,10053,1359,10156,1360,10157,
	1361,10158,1362,10163,1366,10164,1367,10210,
	1375,10211,1376,10218,1380,10227,1392,10228,
	1413,10302,1445,10382,1446,10388,1449,10389,
	1450,10459,1462,1467,10481,1468,10482,1469,
	10500,1477,10501,1478,10616,1501,10617,1504,
	10621,1505,1511,10652,1512,10658,1515,10659,
	1516,1517,10664,10665,1521,10749,1549,10757,
	1556,10758,1557,10759,1558,1559,10762,10835,
	1581,10836,1582,10840,1585,10841,1586,10842,
	1587,10843,1588,10910,1599,10958,1607,10959,
	1608,1616,11032,1617,11033,1618,11067,1629,
	11073,1632,11074,1633,1668,11223,11574,1719,
	11619,1730,11620,1731,1756,11678,1757,11725,
	1764,11743,1774,11744,1775,1787,11785,1788,
	11806,1795,11807,1796,11879,1806,11880,1807,
	1811,11901,1812,11902,1813,11921,1820,11922,
	1821,1830,11996,1831,12015,1838,12016,1839,
	12035,1846,12036,12080,1857,12081,1858,1861,
	12090,12096,1869,12097,1872,12100,1891,12176,
	1892,12182,1895,12183,12222,1915,1921,12258,
	1922,12264,1925,12265,1926,1934,12338,1935,
	12339,1936,1940,12360,1941,12369,1945,12377,
	1952,12378,1953,1954,12382,1959,12397,1960,
	12536,1980,12543,1986,12544,1987,12553,2000,
	12554,2001,12634,2022,12640,2025,12641,2042,
	12678,12710,2052,12721,2056,12722,2057,12837,
	2080,12838,2081,12842,2084,2097,12933,2098,
	12952,2105,2109,12994,2110,13000,2113,13001,
	2120,13011,2121,2173,13333,2174,13513,2188,
	13514,2189,13566,2219,13567,2220,2240,13609,
	13672,2266,2267,13676,2268,13682,2271,13683,
	2272,13747,2290,13768,2297,13769,2298,13787,
	2306,13788,2307,13789,2308,13790,13791,2311,
	13852,2325,13853,2326,2331,13875,2332,13893,
	2340,13894,2341,13939,2350,13940,2351,2352,
	13945,2375,14044,2376,14142,2390,2422,14389,
	2423,14440,2437,14441,2438,14490,2471,14491,
	2472,2473,14495,14528,2494,14529,2495,14530,
	14543,2502,14544,2503,14551,2507,2508,14555,
	14589,2533,14592,2537,14593,2538,14594,2539,
	2553,14633,2554,14652,2561,14653,2562,14672,
	2569,14673,2570,14690,2579,14691,2580,14751,
	2602,14752,2603,14771,2610,14772,2611,14773,
	2612,2617,14788,14789,2620,14790,2646,14827,
	2649,14830,14831,2653,2671,14864,2672,14865,
	2673,2682,14913,2683,14919,2686,14920,14938,
	2695,14939,2696,2708,14999,2709,15005,2712,
	15006,2720,15024,2721,15025,2722,15070,2731,
	15071,2732,15112,2745,15113,2746,15185,2756,
	15230,2767,15231,2768,15270,2774,15276,2777,
	15277,2784,15283,2785,15284,2786,15481,2824,
	15482,2825,15483,2826,2853,15733,2854,2866,
	15789,2867,15790,2868,2885,15826,2886,15827,
	2887,15904,2905,15905,2906,15974,2919,15975,
	2920,16047,2930,16048,2931,16093,2940,16094,
	2941,16166,2951,16167,2952,16209,2964,16210,
	2965,16255,2974,16256,2975,16275,2982,16276,
	2983,16385,3012,16386,3013,16452,3029,16453,
	3030,16573,3048,16574,3049,16643,3062,16644,
	3063,16736,3081,16737,3082,3107,16840,3108,
	16865,3120,16866,3121,16936,3133,16937,3134,
	3135,16942,3136,3141,17013,3142,17059,3150,
	17060,3151,17168,3172,17174,3175,17218,3187,
	17219,3188,17226,3192,3193,17230,3194,17346,
	3227,17347,3228,3238,17390,3239,17411,3246,
	17412,3247,3255,17513,3256,17526,3262,17532,
	3265,17533,3266,3278,17632,3279,17652,3285,
	17701,3292,17702,3293,17720,3301,17721,3302,
	3317,17806,3318,17812,3321,17813,3322,17832,
	3329,17833,3330,17894,3351,17895,3352,3358,
	17916,3359,3363,17939,3364,3369,17959,3370,
	17960,3371,3375,17981,3376,3383,18003,3384,
	18068,3401,18069,3402,3408,18090,3409,3412,
	18099,18106,3419,18107,3420,18147,3434,18148,
	3435,4116,3678,7053,3460,7900,3802,3902,
	3946,3946,3708,4131,4374,4156,4399,4403,
	4424,4511,4550,4564,4318,4136,5800,4139,
	4192,4167,4172,5552,6424,5151,4922,7518,
	5079,5109,7607,5584,5367,7083,5362,5485,
	6001,5460,5317,7009,7370,5678,5874,4159,
	4444,4447,4463,4160,4474,4473,5460,4483,
	4163,4166,4485,4491,4167,7783,4497,4499,
	4180,4555,5800,4563,7988,6123,4577,4183,
	4579,4584,4586,4588,4184,5552,4603,4614,
	4616,4618,5950,4187,4625,4627,4629,5351,
	6494,5731,5771,4191,6484,6684,4279,7139,
	4247,4266,4340,7572,5088,7709,3558,3644,
	3815,194,357,145,323,
};

pdf_cmap pdf_cmap_UniKS_UTF16_H =
{
	-1, "UniKS-UTF16-H", "", nil, 0,
	3, /* codespace table */
	{
		{ 2, 0x0000, 0xd7ff },
		{ 4, 0xdc00, 0xdfff },
		{ 2, 0xe000, 0xffff },
	},
	8553, 8553, (pdf_range*) pdf_cmap_UniKS_UTF16_H_ranges,
	3309, 3309, (unsigned short*) pdf_cmap_UniKS_UTF16_H_table,
};

/* UniKS-UTF16-V */

static const pdf_range pdf_cmap_UniKS_UTF16_V_ranges[] =
{
	{ 0x2013, 0x2014, PDF_CMAP_RANGE, 8059 },
	{ 0x2016, 0x2016, PDF_CMAP_SINGLE, 8061 },
	{ 0x2025, 0x2025, PDF_CMAP_SINGLE, 8058 },
	{ 0x3001, 0x3002, PDF_CMAP_RANGE, 8056 },
	{ 0x3008, 0x3011, PDF_CMAP_RANGE, 8065 },
	{ 0x3013, 0x3013, PDF_CMAP_SINGLE, 8075 },
	{ 0x3014, 0x3015, PDF_CMAP_RANGE, 8063 },
	{ 0xff01, 0xff01, PDF_CMAP_SINGLE, 8076 },
	{ 0xff08, 0xff09, PDF_CMAP_RANGE, 8077 },
	{ 0xff0c, 0xff0c, PDF_CMAP_SINGLE, 8079 },
	{ 0xff0e, 0xff0e, PDF_CMAP_SINGLE, 8080 },
	{ 0xff1a, 0xff1f, PDF_CMAP_RANGE, 8081 },
	{ 0xff3b, 0xff3b, PDF_CMAP_SINGLE, 8087 },
	{ 0xff3d, 0xff3d, PDF_CMAP_SINGLE, 8088 },
	{ 0xff3f, 0xff3f, PDF_CMAP_SINGLE, 8089 },
	{ 0xff5b, 0xff5d, PDF_CMAP_RANGE, 8090 },
	{ 0xff5e, 0xff5e, PDF_CMAP_SINGLE, 8062 },
	{ 0xffe3, 0xffe3, PDF_CMAP_SINGLE, 8093 },
};

static const unsigned short pdf_cmap_UniKS_UTF16_V_table[1] = { 0 };

pdf_cmap pdf_cmap_UniKS_UTF16_V =
{
	-1, "UniKS-UTF16-V", "UniKS-UTF16-H", nil, 1,
	0, /* codespace table */
	{
	/* dummy entry for non-c99 compilers */
	{ 0, 0x0, 0x0 },
	},
	18, 18, (pdf_range*) pdf_cmap_UniKS_UTF16_V_ranges,
	0, 0, (unsigned short*) pdf_cmap_UniKS_UTF16_V_table,
};

